/******************************************************************************
** This file is an amalgamation of many separate C source files from SpatiaLite
** version 3.0.0-beta1.  By combining all the individual C code files into this
** single large file, the entire code can be compiled as a one translation
** unit.  This allows many compilers to do optimizations that would not be
** possible if the files were compiled separately.  Performance improvements
** of 5% are more are commonly seen when SQLite is compiled as a single
** translation unit.
**
** This amalgamation was generated on 2012-01-03 19:16:00 +0100.

Author: Alessandro (Sandro) Furieri <a.furieri@lqt.it>

------------------------------------------------------------------------------

Version: MPL 1.1/GPL 2.0/LGPL 2.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the SpatiaLite library

The Initial Developer of the Original Code is Alessandro Furieri

Portions created by the Initial Developer are Copyright (C) 2008
the Initial Developer. All Rights Reserved.

Contributor(s):
Klaus Foerster klaus.foerster@svg.cc
Luigi Costalli luigi.costalli@gmail.com
The Vanuatu Team - University of Toronto
	Supervisor: Greg Wilson gwilson@cs.toronto.ca

Alternatively, the contents of this file may be used under the terms of
either the GNU General Public License Version 2 or later (the "GPL"), or
the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
in which case the provisions of the GPL or the LGPL are applicable instead
of those above. If you wish to allow use of your version of this file only
under the terms of either the GPL or the LGPL, and not to allow others to
use your version of this file under the terms of the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the GPL or the LGPL. If you do not delete
the provisions above, a recipient may use your version of this file under
the terms of any one of the MPL, the GPL or the LGPL.

*/

#if defined(_WIN32) && !defined(__MINGW32__)
/* MSVC strictly requires this include [off_t] */
#include <sys/types.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <limits.h>
#include <math.h>
#include <float.h>
#include <locale.h>
#include <errno.h>

#include <assert.h>

#if defined(__MINGW32__) || defined(_WIN32)
#define LIBICONV_STATIC
#include <iconv.h>
#define LIBCHARSET_STATIC
#ifdef _MSC_VER
/* <localcharset.h> isn't supported on OSGeo4W */
/* applying a tricky workaround to fix this issue */
extern const char * locale_charset (void);
#else /* sane Windows - not OSGeo4W */
#include <localcharset.h>
#endif /* end localcharset */
#else /* not WINDOWS */
#ifdef __APPLE__
#include <iconv.h>
#include <localcharset.h>
#else /* not Mac OsX */
#include <iconv.h>
#include <langinfo.h>
#endif
#endif

#if defined(_WIN32) || defined(WIN32)
# include <io.h>
#define isatty	_isatty
#define fileno	_fileno
#else
#include <unistd.h>
#endif

#ifndef OMIT_GEOS	/* including GEOS */
#include <geos_c.h>
#endif

#ifndef OMIT_PROJ	/* including PROJ.4 */
#include <proj_api.h>
#endif

#ifndef OMIT_FREEXL	/* including FreeXL */
#include <freexl.h>
#endif

#ifdef _WIN32
#define strcasecmp	_stricmp
#define strncasecmp	_strnicmp
#define atoll	_atoi64
#endif /* not WIN32 */

/*
** alias MACROs to avoid any potential collision
** for linker symbols declared into the sqlite3 code
** internally embedded into SpatiaLite
*/
#define sqlite3_version SPLite3_version
#define sqlite3_libversion SPLite3_libversion
#define sqlite3_sourceid SPLite3_sourceid
#define sqlite3_libversion_number SPLite3_libversion_number
#define sqlite3_compileoption_used SPLite3_compileoption_used
#define sqlite3_compileoption_get SPLite3_compileoption_get
#define sqlite3_threadsafe SPLite3_threadsafe
#define sqlite3_close SPLite3_close
#define sqlite3_exec SPLite3_exec
#define sqlite3_initialize SPLite3_initialize
#define sqlite3_shutdown SPLite3_shutdown
#define sqlite3_os_init SPLite3_os_init
#define sqlite3_os_end SPLite3_os_end
#define sqlite3_config SPLite3_config
#define sqlite3_db_config SPLite3_db_config
#define sqlite3_extended_result_codes SPLite3_extended_result_codes
#define sqlite3_last_insert_rowid SPLite3_last_insert_rowid
#define sqlite3_changes SPLite3_changes
#define sqlite3_total_changes SPLite3_total_changes
#define sqlite3_interrupt SPLite3_interrupt
#define sqlite3_complete SPLite3_complete
#define sqlite3_complete16 SPLite3_complete16
#define sqlite3_busy_handler SPLite3_busy_handler
#define sqlite3_busy_timeout SPLite3_busy_timeout
#define sqlite3_get_table SPLite3_get_table
#define sqlite3_free_table SPLite3_free_table
#define sqlite3_mprintf SPLite3_mprintf
#define sqlite3_vmprintf SPLite3_vmprintf
#define sqlite3_snprintf SPLite3_snprintf
#define sqlite3_vsnprintf SPLite3_vsnprintf
#define sqlite3_malloc SPLite3_malloc
#define sqlite3_realloc SPLite3_realloc
#define sqlite3_free SPLite3_free
#define sqlite3_memory_used SPLite3_memory_used
#define sqlite3_memory_highwater SPLite3_memory_highwater
#define sqlite3_randomness SPLite3_randomness
#define sqlite3_set_authorizer SPLite3_set_authorizer
#define sqlite3_trace SPLite3_trace
#define sqlite3_progress_handler SPLite3_progress_handler
#define sqlite3_open SPLite3_open
#define sqlite3_open16 SPLite3_open16
#define sqlite3_open_v2 SPLite3_open_v2
#define sqlite3_uri_parameter SPLite3_uri_parameter
#define sqlite3_errcode SPLite3_errcode
#define sqlite3_extended_errcode SPLite3_extended_errcode
#define sqlite3_errmsg SPLite3_errmsg
#define sqlite3_errmsg16 SPLite3_errmsg16
#define sqlite3_limit SPLite3_limit
#define sqlite3_prepare SPLite3_prepare
#define sqlite3_prepare_v2 SPLite3_prepare_v2
#define sqlite3_prepare16 SPLite3_prepare16
#define sqlite3_prepare16_v2 SPLite3_prepare16_v2
#define sqlite3_sql SPLite3_sql
#define sqlite3_stmt_readonly SPLite3_stmt_readonly
#define sqlite3_bind_blob SPLite3_bind_blob
#define sqlite3_bind_double SPLite3_bind_double
#define sqlite3_bind_int SPLite3_bind_int
#define sqlite3_bind_int64 SPLite3_bind_int64
#define sqlite3_bind_null SPLite3_bind_null
#define sqlite3_bind_text SPLite3_bind_text
#define sqlite3_bind_text16 SPLite3_bind_text16
#define sqlite3_bind_value SPLite3_bind_value
#define sqlite3_bind_zeroblob SPLite3_bind_zeroblob
#define sqlite3_bind_parameter_count SPLite3_bind_parameter_count
#define sqlite3_bind_parameter_name SPLite3_bind_parameter_name
#define sqlite3_bind_parameter_index SPLite3_bind_parameter_index
#define sqlite3_clear_bindings SPLite3_clear_bindings
#define sqlite3_column_count SPLite3_column_count
#define sqlite3_column_name SPLite3_column_name
#define sqlite3_column_name16 SPLite3_column_name16
#define sqlite3_column_database_name SPLite3_column_database_name
#define sqlite3_column_database_name16 SPLite3_column_database_name16
#define sqlite3_column_table_name SPLite3_column_table_name
#define sqlite3_column_table_name16 SPLite3_column_table_name16
#define sqlite3_column_origin_name SPLite3_column_origin_name
#define sqlite3_column_origin_name16 SPLite3_column_origin_name16
#define sqlite3_column_decltype SPLite3_column_decltype
#define sqlite3_column_decltype16 SPLite3_column_decltype16
#define sqlite3_step SPLite3_step
#define sqlite3_data_count SPLite3_data_count
#define sqlite3_column_blob SPLite3_column_blob
#define sqlite3_column_bytes SPLite3_column_bytes
#define sqlite3_column_bytes16 SPLite3_column_bytes16
#define sqlite3_column_double SPLite3_column_double
#define sqlite3_column_int SPLite3_column_int
#define sqlite3_column_int64 SPLite3_column_int64
#define sqlite3_column_text SPLite3_column_text
#define sqlite3_column_text16 SPLite3_column_text16
#define sqlite3_column_type SPLite3_column_type
#define sqlite3_column_value SPLite3_column_value
#define sqlite3_finalize SPLite3_finalize
#define sqlite3_reset SPLite3_reset
#define sqlite3_create_function SPLite3_create_function
#define sqlite3_create_function16 SPLite3_create_function16
#define sqlite3_create_function_v2 SPLite3_create_function_v2
#define sqlite3_value_blob SPLite3_value_blob
#define sqlite3_value_bytes SPLite3_value_bytes
#define sqlite3_value_bytes16 SPLite3_value_bytes16
#define sqlite3_value_double SPLite3_value_double
#define sqlite3_value_int SPLite3_value_int
#define sqlite3_value_int64 SPLite3_value_int64
#define sqlite3_value_text SPLite3_value_text
#define sqlite3_value_text16 SPLite3_value_text16
#define sqlite3_value_text16le SPLite3_value_text16le
#define sqlite3_value_text16be SPLite3_value_text16be
#define sqlite3_value_type SPLite3_value_type
#define sqlite3_value_numeric_type SPLite3_value_numeric_type
#define sqlite3_aggregate_context SPLite3_aggregate_context
#define sqlite3_user_data SPLite3_user_data
#define sqlite3_context_db_handle SPLite3_context_db_handle
#define sqlite3_get_auxdata SPLite3_get_auxdata
#define sqlite3_set_auxdata SPLite3_set_auxdata
#define sqlite3_result_blob SPLite3_result_blob
#define sqlite3_result_double SPLite3_result_double
#define sqlite3_result_error SPLite3_result_error
#define sqlite3_result_error16 SPLite3_result_error16
#define sqlite3_result_error_toobig SPLite3_result_error_toobig
#define sqlite3_result_error_nomem SPLite3_result_error_nomem
#define sqlite3_result_error_code SPLite3_result_error_code
#define sqlite3_result_int SPLite3_result_int
#define sqlite3_result_int64 SPLite3_result_int64
#define sqlite3_result_null SPLite3_result_null
#define sqlite3_result_text SPLite3_result_text
#define sqlite3_result_text16 SPLite3_result_text16
#define sqlite3_result_text16le SPLite3_result_text16le
#define sqlite3_result_text16be SPLite3_result_text16be
#define sqlite3_result_value SPLite3_result_value
#define sqlite3_result_zeroblob SPLite3_result_zeroblob
#define sqlite3_create_collation SPLite3_create_collation
#define sqlite3_create_collation_v2 SPLite3_create_collation_v2
#define sqlite3_create_collation16 SPLite3_create_collation16
#define sqlite3_collation_needed SPLite3_collation_needed
#define sqlite3_collation_needed16 SPLite3_collation_needed16
#define sqlite3_key SPLite3_key
#define sqlite3_rekey SPLite3_rekey
#define sqlite3_activate_see SPLite3_activate_see
#define sqlite3_activate_cerod SPLite3_activate_cerod
#define sqlite3_sleep SPLite3_sleep
#define sqlite3_temp_directory SPLite3_temp_directory
#define sqlite3_get_autocommit SPLite3_get_autocommit
#define sqlite3_db_handle SPLite3_db_handle
#define sqlite3_next_stmt SPLite3_next_stmt
#define sqlite3_commit_hook SPLite3_commit_hook
#define sqlite3_rollback_hook SPLite3_rollback_hook
#define sqlite3_update_hook SPLite3_update_hook
#define sqlite3_enable_shared_cache SPLite3_enable_shared_cache
#define sqlite3_release_memory SPLite3_release_memory
#define sqlite3_soft_heap_limit64 SPLite3_soft_heap_limit64
#define sqlite3_table_column_metadata SPLite3_table_column_metadata
#define sqlite3_load_extension SPLite3_load_extension
#define sqlite3_enable_load_extension SPLite3_enable_load_extension
#define sqlite3_auto_extension SPLite3_auto_extension
#define sqlite3_reset_auto_extension SPLite3_reset_auto_extension
#define sqlite3_create_module SPLite3_create_module
#define sqlite3_create_module_v2 SPLite3_create_module_v2
#define sqlite3_declare_vtab SPLite3_declare_vtab
#define sqlite3_overload_function SPLite3_overload_function
#define sqlite3_blob_open SPLite3_blob_open
#define sqlite3_blob_close SPLite3_blob_close
#define sqlite3_blob_bytes SPLite3_blob_bytes
#define sqlite3_blob_read SPLite3_blob_read
#define sqlite3_blob_write SPLite3_blob_write
#define sqlite3_vfs_find SPLite3_vfs_find
#define sqlite3_vfs_register SPLite3_vfs_register
#define sqlite3_vfs_unregister SPLite3_vfs_unregister
#define sqlite3_mutex_alloc SPLite3_mutex_alloc
#define sqlite3_mutex_free SPLite3_mutex_free
#define sqlite3_mutex_enter SPLite3_mutex_enter
#define sqlite3_mutex_try SPLite3_mutex_try
#define sqlite3_mutex_leave SPLite3_mutex_leave
#define sqlite3_mutex_held SPLite3_mutex_held
#define sqlite3_mutex_notheld SPLite3_mutex_notheld
#define sqlite3_db_mutex SPLite3_db_mutex
#define sqlite3_file_control SPLite3_file_control
#define sqlite3_test_control SPLite3_test_control
#define sqlite3_status SPLite3_status
#define sqlite3_db_status SPLite3_db_status
#define sqlite3_stmt_status SPLite3_stmt_status
#define sqlite3_backup_init SPLite3_backup_init
#define sqlite3_backup_step SPLite3_backup_step
#define sqlite3_backup_finish SPLite3_backup_finish
#define sqlite3_backup_remaining SPLite3_backup_remaining
#define sqlite3_backup_pagecount SPLite3_backup_pagecount
#define sqlite3_unlock_notify SPLite3_unlock_notify
#define sqlite3_strnicmp SPLite3_strnicmp
#define sqlite3_log SPLite3_log
#define sqlite3_wal_hook SPLite3_wal_hook
#define sqlite3_wal_autocheckpoint SPLite3_wal_autocheckpoint
#define sqlite3_wal_checkpoint SPLite3_wal_checkpoint
#define sqlite3_wal_checkpoint_v2 SPLite3_wal_checkpoint_v2
#define sqlite3_vtab_config SPLite3_vtab_config
#define sqlite3_vtab_on_conflict SPLite3_vtab_on_conflict
#define sqlite3_rtree_geometry_callback SPLite3_rtree_geometry_callback
#define sqlite3_memdebug_vfs_oom_test SPLite3_memdebug_vfs_oom_test
#define sqlite3_memory_alarm SPLite3_memory_alarm
#define sqlite3_soft_heap_limit SPLite3_soft_heap_limit
#define sqlite3_io_error_hit SPLite3_io_error_hit
#define sqlite3_io_error_hardhit SPLite3_io_error_hardhit
#define sqlite3_io_error_pending SPLite3_io_error_pending
#define sqlite3_io_error_persist SPLite3_io_error_persist
#define sqlite3_io_error_benign SPLite3_io_error_benign
#define sqlite3_diskfull_pending SPLite3_diskfull_pending
#define sqlite3_diskfull SPLite3_diskfull
#define sqlite3_open_file_count SPLite3_open_file_count
#define sqlite3_sync_count SPLite3_sync_count
#define sqlite3_fullsync_count SPLite3_fullsync_count
#define sqlite3_current_time SPLite3_current_time
#define sqlite3_hostid_num SPLite3_hostid_num
#define sqlite3_os_type SPLite3_os_type
#define sqlite3_win32_mbcs_to_utf8 SPLite3_win32_mbcs_to_utf8
#define sqlite3_win32_utf8_to_mbcs SPLite3_win32_utf8_to_mbcs
#define sqlite3_pager_readdb_count SPLite3_pager_readdb_count
#define sqlite3_pager_writedb_count SPLite3_pager_writedb_count
#define sqlite3_pager_writej_count SPLite3_pager_writej_count
#define sqlite3_opentemp_count SPLite3_opentemp_count
#define sqlite3_expired SPLite3_expired
#define sqlite3_aggregate_count SPLite3_aggregate_count
#define sqlite3_transfer_bindings SPLite3_transfer_bindings
#define sqlite3_search_count SPLite3_search_count
#define sqlite3_interrupt_count SPLite3_interrupt_count
#define sqlite3_sort_count SPLite3_sort_count
#define sqlite3_max_blobsize SPLite3_max_blobsize
#define sqlite3_found_count SPLite3_found_count
#define sqlite3_blob_reopen SPLite3_blob_reopen
#define sqlite3_like_count SPLite3_like_count
#define sqlite3_xferopt_count SPLite3_xferopt_count
#define sqlite3_profile SPLite3_profile
#define sqlite3_global_recover SPLite3_global_recover
#define sqlite3_thread_cleanup SPLite3_thread_cleanup
#define sqlite3_fts3_enable_parentheses SPLite3_fts3_enable_parentheses
/* end SpatiaLite/sqlite3 alias macros */


/**************** Begin file: sqlite3.h **********/
/*
** 2001 September 15
**
** The author disclaims copyright to this source code.  In place of
** a legal notice, here is a blessing:
**
**    May you do good and not evil.
**    May you find forgiveness for yourself and forgive others.
**    May you share freely, never taking more than you give.
**
*************************************************************************
** This header file defines the interface that the SQLite library
** presents to client programs.  If a C-function, structure, datatype,
** or constant definition does not appear in this file, then it is
** not a published API of SQLite, is subject to change without
** notice, and should not be referenced by programs that use SQLite.
**
** Some of the definitions that are in this file are marked as
** "experimental".  Experimental interfaces are normally new
** features recently added to SQLite.  We do not anticipate changes
** to experimental interfaces but reserve the right to make minor changes
** if experience from use "in the wild" suggest such changes are prudent.
**
** The official C-language API documentation for SQLite is derived
** from comments in this file.  This file is the authoritative source
** on how SQLite interfaces are suppose to operate.
**
** The name of this file under configuration management is "sqlite.h.in".
** The makefile makes some minor changes to this file (such as inserting
** the version number) and changes its name to "sqlite3.h" as
** part of the build process.
*/
#ifndef _SQLITE3_H_
#define _SQLITE3_H_
#include <stdarg.h>     /* Needed for the definition of va_list */

/*
** Make sure we can call this stuff from C++.
*/
#ifdef __cplusplus
extern "C" {
#endif


/*
** Add the ability to override 'extern'
*/
#ifndef SQLITE_EXTERN
# define SQLITE_EXTERN extern
#endif

#ifndef SQLITE_API
# define SQLITE_API
#endif


/*
** These no-op macros are used in front of interfaces to mark those
** interfaces as either deprecated or experimental.  New applications
** should not use deprecated interfaces - they are support for backwards
** compatibility only.  Application writers should be aware that
** experimental interfaces are subject to change in point releases.
**
** These macros used to resolve to various kinds of compiler magic that
** would generate warning messages when they were used.  But that
** compiler magic ended up generating such a flurry of bug reports
** that we have taken it all out and gone back to using simple
** noop macros.
*/
#define SQLITE_DEPRECATED
#define SQLITE_EXPERIMENTAL

/*
** Ensure these symbols were not defined by some previous header file.
*/
#ifdef SQLITE_VERSION
# undef SQLITE_VERSION
#endif
#ifdef SQLITE_VERSION_NUMBER
# undef SQLITE_VERSION_NUMBER
#endif

/*
** CAPI3REF: Compile-Time Library Version Numbers
**
** ^(The [SQLITE_VERSION] C preprocessor macro in the sqlite3.h header
** evaluates to a string literal that is the SQLite version in the
** format "X.Y.Z" where X is the major version number (always 3 for
** SQLite3) and Y is the minor version number and Z is the release number.)^
** ^(The [SQLITE_VERSION_NUMBER] C preprocessor macro resolves to an integer
** with the value (X*1000000 + Y*1000 + Z) where X, Y, and Z are the same
** numbers used in [SQLITE_VERSION].)^
** The SQLITE_VERSION_NUMBER for any given release of SQLite will also
** be larger than the release from which it is derived.  Either Y will
** be held constant and Z will be incremented or else Y will be incremented
** and Z will be reset to zero.
**
** Since version 3.6.18, SQLite source code has been stored in the
** <a href="http://www.fossil-scm.org/">Fossil configuration management
** system</a>.  ^The SQLITE_SOURCE_ID macro evaluates to
** a string which identifies a particular check-in of SQLite
** within its configuration management system.  ^The SQLITE_SOURCE_ID
** string contains the date and time of the check-in (UTC) and an SHA1
** hash of the entire source tree.
**
** See also: [sqlite3_libversion()],
** [sqlite3_libversion_number()], [sqlite3_sourceid()],
** [sqlite_version()] and [sqlite_source_id()].
*/
#define SQLITE_VERSION        "3.7.9"
#define SQLITE_VERSION_NUMBER 3007009
#define SQLITE_SOURCE_ID      "2011-11-01 00:52:41 c7c6050ef060877ebe77b41d959e9df13f8c9b5e"

/*
** CAPI3REF: Run-Time Library Version Numbers
** KEYWORDS: sqlite3_version, sqlite3_sourceid
**
** These interfaces provide the same information as the [SQLITE_VERSION],
** [SQLITE_VERSION_NUMBER], and [SQLITE_SOURCE_ID] C preprocessor macros
** but are associated with the library instead of the header file.  ^(Cautious
** programmers might include assert() statements in their application to
** verify that values returned by these interfaces match the macros in
** the header, and thus insure that the application is
** compiled with matching library and header files.
**
** <blockquote><pre>
** assert( sqlite3_libversion_number()==SQLITE_VERSION_NUMBER );
** assert( strcmp(sqlite3_sourceid(),SQLITE_SOURCE_ID)==0 );
** assert( strcmp(sqlite3_libversion(),SQLITE_VERSION)==0 );
** </pre></blockquote>)^
**
** ^The sqlite3_version[] string constant contains the text of [SQLITE_VERSION]
** macro.  ^The sqlite3_libversion() function returns a pointer to the
** to the sqlite3_version[] string constant.  The sqlite3_libversion()
** function is provided for use in DLLs since DLL users usually do not have
** direct access to string constants within the DLL.  ^The
** sqlite3_libversion_number() function returns an integer equal to
** [SQLITE_VERSION_NUMBER].  ^The sqlite3_sourceid() function returns 
** a pointer to a string constant whose value is the same as the 
** [SQLITE_SOURCE_ID] C preprocessor macro.
**
** See also: [sqlite_version()] and [sqlite_source_id()].
*/
SQLITE_API SQLITE_EXTERN const char sqlite3_version[];
SQLITE_API const char *sqlite3_libversion(void);
SQLITE_API const char *sqlite3_sourceid(void);
SQLITE_API int sqlite3_libversion_number(void);

/*
** CAPI3REF: Run-Time Library Compilation Options Diagnostics
**
** ^The sqlite3_compileoption_used() function returns 0 or 1 
** indicating whether the specified option was defined at 
** compile time.  ^The SQLITE_ prefix may be omitted from the 
** option name passed to sqlite3_compileoption_used().  
**
** ^The sqlite3_compileoption_get() function allows iterating
** over the list of options that were defined at compile time by
** returning the N-th compile time option string.  ^If N is out of range,
** sqlite3_compileoption_get() returns a NULL pointer.  ^The SQLITE_ 
** prefix is omitted from any strings returned by 
** sqlite3_compileoption_get().
**
** ^Support for the diagnostic functions sqlite3_compileoption_used()
** and sqlite3_compileoption_get() may be omitted by specifying the 
** [SQLITE_OMIT_COMPILEOPTION_DIAGS] option at compile time.
**
** See also: SQL functions [sqlite_compileoption_used()] and
** [sqlite_compileoption_get()] and the [compile_options pragma].
*/
#ifndef SQLITE_OMIT_COMPILEOPTION_DIAGS
SQLITE_API int sqlite3_compileoption_used(const char *zOptName);
SQLITE_API const char *sqlite3_compileoption_get(int N);
#endif

/*
** CAPI3REF: Test To See If The Library Is Threadsafe
**
** ^The sqlite3_threadsafe() function returns zero if and only if
** SQLite was compiled mutexing code omitted due to the
** [SQLITE_THREADSAFE] compile-time option being set to 0.
**
** SQLite can be compiled with or without mutexes.  When
** the [SQLITE_THREADSAFE] C preprocessor macro is 1 or 2, mutexes
** are enabled and SQLite is threadsafe.  When the
** [SQLITE_THREADSAFE] macro is 0, 
** the mutexes are omitted.  Without the mutexes, it is not safe
** to use SQLite concurrently from more than one thread.
**
** Enabling mutexes incurs a measurable performance penalty.
** So if speed is of utmost importance, it makes sense to disable
** the mutexes.  But for maximum safety, mutexes should be enabled.
** ^The default behavior is for mutexes to be enabled.
**
** This interface can be used by an application to make sure that the
** version of SQLite that it is linking against was compiled with
** the desired setting of the [SQLITE_THREADSAFE] macro.
**
** This interface only reports on the compile-time mutex setting
** of the [SQLITE_THREADSAFE] flag.  If SQLite is compiled with
** SQLITE_THREADSAFE=1 or =2 then mutexes are enabled by default but
** can be fully or partially disabled using a call to [sqlite3_config()]
** with the verbs [SQLITE_CONFIG_SINGLETHREAD], [SQLITE_CONFIG_MULTITHREAD],
** or [SQLITE_CONFIG_MUTEX].  ^(The return value of the
** sqlite3_threadsafe() function shows only the compile-time setting of
** thread safety, not any run-time changes to that setting made by
** sqlite3_config(). In other words, the return value from sqlite3_threadsafe()
** is unchanged by calls to sqlite3_config().)^
**
** See the [threading mode] documentation for additional information.
*/
SQLITE_API int sqlite3_threadsafe(void);

/*
** CAPI3REF: Database Connection Handle
** KEYWORDS: {database connection} {database connections}
**
** Each open SQLite database is represented by a pointer to an instance of
** the opaque structure named "sqlite3".  It is useful to think of an sqlite3
** pointer as an object.  The [sqlite3_open()], [sqlite3_open16()], and
** [sqlite3_open_v2()] interfaces are its constructors, and [sqlite3_close()]
** is its destructor.  There are many other interfaces (such as
** [sqlite3_prepare_v2()], [sqlite3_create_function()], and
** [sqlite3_busy_timeout()] to name but three) that are methods on an
** sqlite3 object.
*/
typedef struct sqlite3 sqlite3;

/*
** CAPI3REF: 64-Bit Integer Types
** KEYWORDS: sqlite_int64 sqlite_uint64
**
** Because there is no cross-platform way to specify 64-bit integer types
** SQLite includes typedefs for 64-bit signed and unsigned integers.
**
** The sqlite3_int64 and sqlite3_uint64 are the preferred type definitions.
** The sqlite_int64 and sqlite_uint64 types are supported for backwards
** compatibility only.
**
** ^The sqlite3_int64 and sqlite_int64 types can store integer values
** between -9223372036854775808 and +9223372036854775807 inclusive.  ^The
** sqlite3_uint64 and sqlite_uint64 types can store integer values 
** between 0 and +18446744073709551615 inclusive.
*/
#ifdef SQLITE_INT64_TYPE
  typedef SQLITE_INT64_TYPE sqlite_int64;
  typedef unsigned SQLITE_INT64_TYPE sqlite_uint64;
#elif defined(_MSC_VER) || defined(__BORLANDC__)
  typedef __int64 sqlite_int64;
  typedef unsigned __int64 sqlite_uint64;
#else
  typedef long long int sqlite_int64;
  typedef unsigned long long int sqlite_uint64;
#endif
typedef sqlite_int64 sqlite3_int64;
typedef sqlite_uint64 sqlite3_uint64;

/*
** If compiling for a processor that lacks floating point support,
** substitute integer for floating-point.
*/
#ifdef SQLITE_OMIT_FLOATING_POINT
# define double sqlite3_int64
#endif

/*
** CAPI3REF: Closing A Database Connection
**
** ^The sqlite3_close() routine is the destructor for the [sqlite3] object.
** ^Calls to sqlite3_close() return SQLITE_OK if the [sqlite3] object is
** successfully destroyed and all associated resources are deallocated.
**
** Applications must [sqlite3_finalize | finalize] all [prepared statements]
** and [sqlite3_blob_close | close] all [BLOB handles] associated with
** the [sqlite3] object prior to attempting to close the object.  ^If
** sqlite3_close() is called on a [database connection] that still has
** outstanding [prepared statements] or [BLOB handles], then it returns
** SQLITE_BUSY.
**
** ^If [sqlite3_close()] is invoked while a transaction is open,
** the transaction is automatically rolled back.
**
** The C parameter to [sqlite3_close(C)] must be either a NULL
** pointer or an [sqlite3] object pointer obtained
** from [sqlite3_open()], [sqlite3_open16()], or
** [sqlite3_open_v2()], and not previously closed.
** ^Calling sqlite3_close() with a NULL pointer argument is a 
** harmless no-op.
*/
SQLITE_API int sqlite3_close(sqlite3 *);

/*
** The type for a callback function.
** This is legacy and deprecated.  It is included for historical
** compatibility and is not documented.
*/
typedef int (*sqlite3_callback)(void*,int,char**, char**);

/*
** CAPI3REF: One-Step Query Execution Interface
**
** The sqlite3_exec() interface is a convenience wrapper around
** [sqlite3_prepare_v2()], [sqlite3_step()], and [sqlite3_finalize()],
** that allows an application to run multiple statements of SQL
** without having to use a lot of C code. 
**
** ^The sqlite3_exec() interface runs zero or more UTF-8 encoded,
** semicolon-separate SQL statements passed into its 2nd argument,
** in the context of the [database connection] passed in as its 1st
** argument.  ^If the callback function of the 3rd argument to
** sqlite3_exec() is not NULL, then it is invoked for each result row
** coming out of the evaluated SQL statements.  ^The 4th argument to
** sqlite3_exec() is relayed through to the 1st argument of each
** callback invocation.  ^If the callback pointer to sqlite3_exec()
** is NULL, then no callback is ever invoked and result rows are
** ignored.
**
** ^If an error occurs while evaluating the SQL statements passed into
** sqlite3_exec(), then execution of the current statement stops and
** subsequent statements are skipped.  ^If the 5th parameter to sqlite3_exec()
** is not NULL then any error message is written into memory obtained
** from [sqlite3_malloc()] and passed back through the 5th parameter.
** To avoid memory leaks, the application should invoke [sqlite3_free()]
** on error message strings returned through the 5th parameter of
** of sqlite3_exec() after the error message string is no longer needed.
** ^If the 5th parameter to sqlite3_exec() is not NULL and no errors
** occur, then sqlite3_exec() sets the pointer in its 5th parameter to
** NULL before returning.
**
** ^If an sqlite3_exec() callback returns non-zero, the sqlite3_exec()
** routine returns SQLITE_ABORT without invoking the callback again and
** without running any subsequent SQL statements.
**
** ^The 2nd argument to the sqlite3_exec() callback function is the
** number of columns in the result.  ^The 3rd argument to the sqlite3_exec()
** callback is an array of pointers to strings obtained as if from
** [sqlite3_column_text()], one for each column.  ^If an element of a
** result row is NULL then the corresponding string pointer for the
** sqlite3_exec() callback is a NULL pointer.  ^The 4th argument to the
** sqlite3_exec() callback is an array of pointers to strings where each
** entry represents the name of corresponding result column as obtained
** from [sqlite3_column_name()].
**
** ^If the 2nd parameter to sqlite3_exec() is a NULL pointer, a pointer
** to an empty string, or a pointer that contains only whitespace and/or 
** SQL comments, then no SQL statements are evaluated and the database
** is not changed.
**
** Restrictions:
**
** <ul>
** <li> The application must insure that the 1st parameter to sqlite3_exec()
**      is a valid and open [database connection].
** <li> The application must not close [database connection] specified by
**      the 1st parameter to sqlite3_exec() while sqlite3_exec() is running.
** <li> The application must not modify the SQL statement text passed into
**      the 2nd parameter of sqlite3_exec() while sqlite3_exec() is running.
** </ul>
*/
SQLITE_API int sqlite3_exec(
  sqlite3*,                                  /* An open database */
  const char *sql,                           /* SQL to be evaluated */
  int (*callback)(void*,int,char**,char**),  /* Callback function */
  void *,                                    /* 1st argument to callback */
  char **errmsg                              /* Error msg written here */
);

/*
** CAPI3REF: Result Codes
** KEYWORDS: SQLITE_OK {error code} {error codes}
** KEYWORDS: {result code} {result codes}
**
** Many SQLite functions return an integer result code from the set shown
** here in order to indicates success or failure.
**
** New error codes may be added in future versions of SQLite.
**
** See also: [SQLITE_IOERR_READ | extended result codes],
** [sqlite3_vtab_on_conflict()] [SQLITE_ROLLBACK | result codes].
*/
#define SQLITE_OK           0   /* Successful result */
/* beginning-of-error-codes */
#define SQLITE_ERROR        1   /* SQL error or missing database */
#define SQLITE_INTERNAL     2   /* Internal logic error in SQLite */
#define SQLITE_PERM         3   /* Access permission denied */
#define SQLITE_ABORT        4   /* Callback routine requested an abort */
#define SQLITE_BUSY         5   /* The database file is locked */
#define SQLITE_LOCKED       6   /* A table in the database is locked */
#define SQLITE_NOMEM        7   /* A malloc() failed */
#define SQLITE_READONLY     8   /* Attempt to write a readonly database */
#define SQLITE_INTERRUPT    9   /* Operation terminated by sqlite3_interrupt()*/
#define SQLITE_IOERR       10   /* Some kind of disk I/O error occurred */
#define SQLITE_CORRUPT     11   /* The database disk image is malformed */
#define SQLITE_NOTFOUND    12   /* Unknown opcode in sqlite3_file_control() */
#define SQLITE_FULL        13   /* Insertion failed because database is full */
#define SQLITE_CANTOPEN    14   /* Unable to open the database file */
#define SQLITE_PROTOCOL    15   /* Database lock protocol error */
#define SQLITE_EMPTY       16   /* Database is empty */
#define SQLITE_SCHEMA      17   /* The database schema changed */
#define SQLITE_TOOBIG      18   /* String or BLOB exceeds size limit */
#define SQLITE_CONSTRAINT  19   /* Abort due to constraint violation */
#define SQLITE_MISMATCH    20   /* Data type mismatch */
#define SQLITE_MISUSE      21   /* Library used incorrectly */
#define SQLITE_NOLFS       22   /* Uses OS features not supported on host */
#define SQLITE_AUTH        23   /* Authorization denied */
#define SQLITE_FORMAT      24   /* Auxiliary database format error */
#define SQLITE_RANGE       25   /* 2nd parameter to sqlite3_bind out of range */
#define SQLITE_NOTADB      26   /* File opened that is not a database file */
#define SQLITE_ROW         100  /* sqlite3_step() has another row ready */
#define SQLITE_DONE        101  /* sqlite3_step() has finished executing */
/* end-of-error-codes */

/*
** CAPI3REF: Extended Result Codes
** KEYWORDS: {extended error code} {extended error codes}
** KEYWORDS: {extended result code} {extended result codes}
**
** In its default configuration, SQLite API routines return one of 26 integer
** [SQLITE_OK | result codes].  However, experience has shown that many of
** these result codes are too coarse-grained.  They do not provide as
** much information about problems as programmers might like.  In an effort to
** address this, newer versions of SQLite (version 3.3.8 and later) include
** support for additional result codes that provide more detailed information
** about errors. The extended result codes are enabled or disabled
** on a per database connection basis using the
** [sqlite3_extended_result_codes()] API.
**
** Some of the available extended result codes are listed here.
** One may expect the number of extended result codes will be expand
** over time.  Software that uses extended result codes should expect
** to see new result codes in future releases of SQLite.
**
** The SQLITE_OK result code will never be extended.  It will always
** be exactly zero.
*/
#define SQLITE_IOERR_READ              (SQLITE_IOERR | (1<<8))
#define SQLITE_IOERR_SHORT_READ        (SQLITE_IOERR | (2<<8))
#define SQLITE_IOERR_WRITE             (SQLITE_IOERR | (3<<8))
#define SQLITE_IOERR_FSYNC             (SQLITE_IOERR | (4<<8))
#define SQLITE_IOERR_DIR_FSYNC         (SQLITE_IOERR | (5<<8))
#define SQLITE_IOERR_TRUNCATE          (SQLITE_IOERR | (6<<8))
#define SQLITE_IOERR_FSTAT             (SQLITE_IOERR | (7<<8))
#define SQLITE_IOERR_UNLOCK            (SQLITE_IOERR | (8<<8))
#define SQLITE_IOERR_RDLOCK            (SQLITE_IOERR | (9<<8))
#define SQLITE_IOERR_DELETE            (SQLITE_IOERR | (10<<8))
#define SQLITE_IOERR_BLOCKED           (SQLITE_IOERR | (11<<8))
#define SQLITE_IOERR_NOMEM             (SQLITE_IOERR | (12<<8))
#define SQLITE_IOERR_ACCESS            (SQLITE_IOERR | (13<<8))
#define SQLITE_IOERR_CHECKRESERVEDLOCK (SQLITE_IOERR | (14<<8))
#define SQLITE_IOERR_LOCK              (SQLITE_IOERR | (15<<8))
#define SQLITE_IOERR_CLOSE             (SQLITE_IOERR | (16<<8))
#define SQLITE_IOERR_DIR_CLOSE         (SQLITE_IOERR | (17<<8))
#define SQLITE_IOERR_SHMOPEN           (SQLITE_IOERR | (18<<8))
#define SQLITE_IOERR_SHMSIZE           (SQLITE_IOERR | (19<<8))
#define SQLITE_IOERR_SHMLOCK           (SQLITE_IOERR | (20<<8))
#define SQLITE_IOERR_SHMMAP            (SQLITE_IOERR | (21<<8))
#define SQLITE_IOERR_SEEK              (SQLITE_IOERR | (22<<8))
#define SQLITE_LOCKED_SHAREDCACHE      (SQLITE_LOCKED |  (1<<8))
#define SQLITE_BUSY_RECOVERY           (SQLITE_BUSY   |  (1<<8))
#define SQLITE_CANTOPEN_NOTEMPDIR      (SQLITE_CANTOPEN | (1<<8))
#define SQLITE_CORRUPT_VTAB            (SQLITE_CORRUPT | (1<<8))
#define SQLITE_READONLY_RECOVERY       (SQLITE_READONLY | (1<<8))
#define SQLITE_READONLY_CANTLOCK       (SQLITE_READONLY | (2<<8))

/*
** CAPI3REF: Flags For File Open Operations
**
** These bit values are intended for use in the
** 3rd parameter to the [sqlite3_open_v2()] interface and
** in the 4th parameter to the [sqlite3_vfs.xOpen] method.
*/
#define SQLITE_OPEN_READONLY         0x00000001  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_READWRITE        0x00000002  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_CREATE           0x00000004  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_DELETEONCLOSE    0x00000008  /* VFS only */
#define SQLITE_OPEN_EXCLUSIVE        0x00000010  /* VFS only */
#define SQLITE_OPEN_AUTOPROXY        0x00000020  /* VFS only */
#define SQLITE_OPEN_URI              0x00000040  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_MAIN_DB          0x00000100  /* VFS only */
#define SQLITE_OPEN_TEMP_DB          0x00000200  /* VFS only */
#define SQLITE_OPEN_TRANSIENT_DB     0x00000400  /* VFS only */
#define SQLITE_OPEN_MAIN_JOURNAL     0x00000800  /* VFS only */
#define SQLITE_OPEN_TEMP_JOURNAL     0x00001000  /* VFS only */
#define SQLITE_OPEN_SUBJOURNAL       0x00002000  /* VFS only */
#define SQLITE_OPEN_MASTER_JOURNAL   0x00004000  /* VFS only */
#define SQLITE_OPEN_NOMUTEX          0x00008000  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_FULLMUTEX        0x00010000  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_SHAREDCACHE      0x00020000  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_PRIVATECACHE     0x00040000  /* Ok for sqlite3_open_v2() */
#define SQLITE_OPEN_WAL              0x00080000  /* VFS only */

/* Reserved:                         0x00F00000 */

/*
** CAPI3REF: Device Characteristics
**
** The xDeviceCharacteristics method of the [sqlite3_io_methods]
** object returns an integer which is a vector of the these
** bit values expressing I/O characteristics of the mass storage
** device that holds the file that the [sqlite3_io_methods]
** refers to.
**
** The SQLITE_IOCAP_ATOMIC property means that all writes of
** any size are atomic.  The SQLITE_IOCAP_ATOMICnnn values
** mean that writes of blocks that are nnn bytes in size and
** are aligned to an address which is an integer multiple of
** nnn are atomic.  The SQLITE_IOCAP_SAFE_APPEND value means
** that when data is appended to a file, the data is appended
** first then the size of the file is extended, never the other
** way around.  The SQLITE_IOCAP_SEQUENTIAL property means that
** information is written to disk in the same order as calls
** to xWrite().
*/
#define SQLITE_IOCAP_ATOMIC                 0x00000001
#define SQLITE_IOCAP_ATOMIC512              0x00000002
#define SQLITE_IOCAP_ATOMIC1K               0x00000004
#define SQLITE_IOCAP_ATOMIC2K               0x00000008
#define SQLITE_IOCAP_ATOMIC4K               0x00000010
#define SQLITE_IOCAP_ATOMIC8K               0x00000020
#define SQLITE_IOCAP_ATOMIC16K              0x00000040
#define SQLITE_IOCAP_ATOMIC32K              0x00000080
#define SQLITE_IOCAP_ATOMIC64K              0x00000100
#define SQLITE_IOCAP_SAFE_APPEND            0x00000200
#define SQLITE_IOCAP_SEQUENTIAL             0x00000400
#define SQLITE_IOCAP_UNDELETABLE_WHEN_OPEN  0x00000800

/*
** CAPI3REF: File Locking Levels
**
** SQLite uses one of these integer values as the second
** argument to calls it makes to the xLock() and xUnlock() methods
** of an [sqlite3_io_methods] object.
*/
#define SQLITE_LOCK_NONE          0
#define SQLITE_LOCK_SHARED        1
#define SQLITE_LOCK_RESERVED      2
#define SQLITE_LOCK_PENDING       3
#define SQLITE_LOCK_EXCLUSIVE     4

/*
** CAPI3REF: Synchronization Type Flags
**
** When SQLite invokes the xSync() method of an
** [sqlite3_io_methods] object it uses a combination of
** these integer values as the second argument.
**
** When the SQLITE_SYNC_DATAONLY flag is used, it means that the
** sync operation only needs to flush data to mass storage.  Inode
** information need not be flushed. If the lower four bits of the flag
** equal SQLITE_SYNC_NORMAL, that means to use normal fsync() semantics.
** If the lower four bits equal SQLITE_SYNC_FULL, that means
** to use Mac OS X style fullsync instead of fsync().
**
** Do not confuse the SQLITE_SYNC_NORMAL and SQLITE_SYNC_FULL flags
** with the [PRAGMA synchronous]=NORMAL and [PRAGMA synchronous]=FULL
** settings.  The [synchronous pragma] determines when calls to the
** xSync VFS method occur and applies uniformly across all platforms.
** The SQLITE_SYNC_NORMAL and SQLITE_SYNC_FULL flags determine how
** energetic or rigorous or forceful the sync operations are and
** only make a difference on Mac OSX for the default SQLite code.
** (Third-party VFS implementations might also make the distinction
** between SQLITE_SYNC_NORMAL and SQLITE_SYNC_FULL, but among the
** operating systems natively supported by SQLite, only Mac OSX
** cares about the difference.)
*/
#define SQLITE_SYNC_NORMAL        0x00002
#define SQLITE_SYNC_FULL          0x00003
#define SQLITE_SYNC_DATAONLY      0x00010

/*
** CAPI3REF: OS Interface Open File Handle
**
** An [sqlite3_file] object represents an open file in the 
** [sqlite3_vfs | OS interface layer].  Individual OS interface
** implementations will
** want to subclass this object by appending additional fields
** for their own use.  The pMethods entry is a pointer to an
** [sqlite3_io_methods] object that defines methods for performing
** I/O operations on the open file.
*/
typedef struct sqlite3_file sqlite3_file;
struct sqlite3_file {
  const struct sqlite3_io_methods *pMethods;  /* Methods for an open file */
};

/*
** CAPI3REF: OS Interface File Virtual Methods Object
**
** Every file opened by the [sqlite3_vfs.xOpen] method populates an
** [sqlite3_file] object (or, more commonly, a subclass of the
** [sqlite3_file] object) with a pointer to an instance of this object.
** This object defines the methods used to perform various operations
** against the open file represented by the [sqlite3_file] object.
**
** If the [sqlite3_vfs.xOpen] method sets the sqlite3_file.pMethods element 
** to a non-NULL pointer, then the sqlite3_io_methods.xClose method
** may be invoked even if the [sqlite3_vfs.xOpen] reported that it failed.  The
** only way to prevent a call to xClose following a failed [sqlite3_vfs.xOpen]
** is for the [sqlite3_vfs.xOpen] to set the sqlite3_file.pMethods element
** to NULL.
**
** The flags argument to xSync may be one of [SQLITE_SYNC_NORMAL] or
** [SQLITE_SYNC_FULL].  The first choice is the normal fsync().
** The second choice is a Mac OS X style fullsync.  The [SQLITE_SYNC_DATAONLY]
** flag may be ORed in to indicate that only the data of the file
** and not its inode needs to be synced.
**
** The integer values to xLock() and xUnlock() are one of
** <ul>
** <li> [SQLITE_LOCK_NONE],
** <li> [SQLITE_LOCK_SHARED],
** <li> [SQLITE_LOCK_RESERVED],
** <li> [SQLITE_LOCK_PENDING], or
** <li> [SQLITE_LOCK_EXCLUSIVE].
** </ul>
** xLock() increases the lock. xUnlock() decreases the lock.
** The xCheckReservedLock() method checks whether any database connection,
** either in this process or in some other process, is holding a RESERVED,
** PENDING, or EXCLUSIVE lock on the file.  It returns true
** if such a lock exists and false otherwise.
**
** The xFileControl() method is a generic interface that allows custom
** VFS implementations to directly control an open file using the
** [sqlite3_file_control()] interface.  The second "op" argument is an
** integer opcode.  The third argument is a generic pointer intended to
** point to a structure that may contain arguments or space in which to
** write return values.  Potential uses for xFileControl() might be
** functions to enable blocking locks with timeouts, to change the
** locking strategy (for example to use dot-file locks), to inquire
** about the status of a lock, or to break stale locks.  The SQLite
** core reserves all opcodes less than 100 for its own use.
** A [SQLITE_FCNTL_LOCKSTATE | list of opcodes] less than 100 is available.
** Applications that define a custom xFileControl method should use opcodes
** greater than 100 to avoid conflicts.  VFS implementations should
** return [SQLITE_NOTFOUND] for file control opcodes that they do not
** recognize.
**
** The xSectorSize() method returns the sector size of the
** device that underlies the file.  The sector size is the
** minimum write that can be performed without disturbing
** other bytes in the file.  The xDeviceCharacteristics()
** method returns a bit vector describing behaviors of the
** underlying device:
**
** <ul>
** <li> [SQLITE_IOCAP_ATOMIC]
** <li> [SQLITE_IOCAP_ATOMIC512]
** <li> [SQLITE_IOCAP_ATOMIC1K]
** <li> [SQLITE_IOCAP_ATOMIC2K]
** <li> [SQLITE_IOCAP_ATOMIC4K]
** <li> [SQLITE_IOCAP_ATOMIC8K]
** <li> [SQLITE_IOCAP_ATOMIC16K]
** <li> [SQLITE_IOCAP_ATOMIC32K]
** <li> [SQLITE_IOCAP_ATOMIC64K]
** <li> [SQLITE_IOCAP_SAFE_APPEND]
** <li> [SQLITE_IOCAP_SEQUENTIAL]
** </ul>
**
** The SQLITE_IOCAP_ATOMIC property means that all writes of
** any size are atomic.  The SQLITE_IOCAP_ATOMICnnn values
** mean that writes of blocks that are nnn bytes in size and
** are aligned to an address which is an integer multiple of
** nnn are atomic.  The SQLITE_IOCAP_SAFE_APPEND value means
** that when data is appended to a file, the data is appended
** first then the size of the file is extended, never the other
** way around.  The SQLITE_IOCAP_SEQUENTIAL property means that
** information is written to disk in the same order as calls
** to xWrite().
**
** If xRead() returns SQLITE_IOERR_SHORT_READ it must also fill
** in the unread portions of the buffer with zeros.  A VFS that
** fails to zero-fill short reads might seem to work.  However,
** failure to zero-fill short reads will eventually lead to
** database corruption.
*/
typedef struct sqlite3_io_methods sqlite3_io_methods;
struct sqlite3_io_methods {
  int iVersion;
  int (*xClose)(sqlite3_file*);
  int (*xRead)(sqlite3_file*, void*, int iAmt, sqlite3_int64 iOfst);
  int (*xWrite)(sqlite3_file*, const void*, int iAmt, sqlite3_int64 iOfst);
  int (*xTruncate)(sqlite3_file*, sqlite3_int64 size);
  int (*xSync)(sqlite3_file*, int flags);
  int (*xFileSize)(sqlite3_file*, sqlite3_int64 *pSize);
  int (*xLock)(sqlite3_file*, int);
  int (*xUnlock)(sqlite3_file*, int);
  int (*xCheckReservedLock)(sqlite3_file*, int *pResOut);
  int (*xFileControl)(sqlite3_file*, int op, void *pArg);
  int (*xSectorSize)(sqlite3_file*);
  int (*xDeviceCharacteristics)(sqlite3_file*);
  /* Methods above are valid for version 1 */
  int (*xShmMap)(sqlite3_file*, int iPg, int pgsz, int, void volatile**);
  int (*xShmLock)(sqlite3_file*, int offset, int n, int flags);
  void (*xShmBarrier)(sqlite3_file*);
  int (*xShmUnmap)(sqlite3_file*, int deleteFlag);
  /* Methods above are valid for version 2 */
  /* Additional methods may be added in future releases */
};

/*
** CAPI3REF: Standard File Control Opcodes
**
** These integer constants are opcodes for the xFileControl method
** of the [sqlite3_io_methods] object and for the [sqlite3_file_control()]
** interface.
**
** The [SQLITE_FCNTL_LOCKSTATE] opcode is used for debugging.  This
** opcode causes the xFileControl method to write the current state of
** the lock (one of [SQLITE_LOCK_NONE], [SQLITE_LOCK_SHARED],
** [SQLITE_LOCK_RESERVED], [SQLITE_LOCK_PENDING], or [SQLITE_LOCK_EXCLUSIVE])
** into an integer that the pArg argument points to. This capability
** is used during testing and only needs to be supported when SQLITE_TEST
** is defined.
**
** The [SQLITE_FCNTL_SIZE_HINT] opcode is used by SQLite to give the VFS
** layer a hint of how large the database file will grow to be during the
** current transaction.  This hint is not guaranteed to be accurate but it
** is often close.  The underlying VFS might choose to preallocate database
** file space based on this hint in order to help writes to the database
** file run faster.
**
** The [SQLITE_FCNTL_CHUNK_SIZE] opcode is used to request that the VFS
** extends and truncates the database file in chunks of a size specified
** by the user. The fourth argument to [sqlite3_file_control()] should 
** point to an integer (type int) containing the new chunk-size to use
** for the nominated database. Allocating database file space in large
** chunks (say 1MB at a time), may reduce file-system fragmentation and
** improve performance on some systems.
**
** The [SQLITE_FCNTL_FILE_POINTER] opcode is used to obtain a pointer
** to the [sqlite3_file] object associated with a particular database
** connection.  See the [sqlite3_file_control()] documentation for
** additional information.
**
** ^(The [SQLITE_FCNTL_SYNC_OMITTED] opcode is generated internally by
** SQLite and sent to all VFSes in place of a call to the xSync method
** when the database connection has [PRAGMA synchronous] set to OFF.)^
** Some specialized VFSes need this signal in order to operate correctly
** when [PRAGMA synchronous | PRAGMA synchronous=OFF] is set, but most 
** VFSes do not need this signal and should silently ignore this opcode.
** Applications should not call [sqlite3_file_control()] with this
** opcode as doing so may disrupt the operation of the specialized VFSes
** that do require it.  
**
** ^The [SQLITE_FCNTL_WIN32_AV_RETRY] opcode is used to configure automatic
** retry counts and intervals for certain disk I/O operations for the
** windows [VFS] in order to work to provide robustness against
** anti-virus programs.  By default, the windows VFS will retry file read,
** file write, and file delete operations up to 10 times, with a delay
** of 25 milliseconds before the first retry and with the delay increasing
** by an additional 25 milliseconds with each subsequent retry.  This
** opcode allows those to values (10 retries and 25 milliseconds of delay)
** to be adjusted.  The values are changed for all database connections
** within the same process.  The argument is a pointer to an array of two
** integers where the first integer i the new retry count and the second
** integer is the delay.  If either integer is negative, then the setting
** is not changed but instead the prior value of that setting is written
** into the array entry, allowing the current retry settings to be
** interrogated.  The zDbName parameter is ignored.
**
** ^The [SQLITE_FCNTL_PERSIST_WAL] opcode is used to set or query the
** persistent [WAL | Write AHead Log] setting.  By default, the auxiliary
** write ahead log and shared memory files used for transaction control
** are automatically deleted when the latest connection to the database
** closes.  Setting persistent WAL mode causes those files to persist after
** close.  Persisting the files is useful when other processes that do not
** have write permission on the directory containing the database file want
** to read the database file, as the WAL and shared memory files must exist
** in order for the database to be readable.  The fourth parameter to
** [sqlite3_file_control()] for this opcode should be a pointer to an integer.
** That integer is 0 to disable persistent WAL mode or 1 to enable persistent
** WAL mode.  If the integer is -1, then it is overwritten with the current
** WAL persistence setting.
**
** ^The [SQLITE_FCNTL_OVERWRITE] opcode is invoked by SQLite after opening
** a write transaction to indicate that, unless it is rolled back for some
** reason, the entire database file will be overwritten by the current 
** transaction. This is used by VACUUM operations.
*/
#define SQLITE_FCNTL_LOCKSTATE        1
#define SQLITE_GET_LOCKPROXYFILE      2
#define SQLITE_SET_LOCKPROXYFILE      3
#define SQLITE_LAST_ERRNO             4
#define SQLITE_FCNTL_SIZE_HINT        5
#define SQLITE_FCNTL_CHUNK_SIZE       6
#define SQLITE_FCNTL_FILE_POINTER     7
#define SQLITE_FCNTL_SYNC_OMITTED     8
#define SQLITE_FCNTL_WIN32_AV_RETRY   9
#define SQLITE_FCNTL_PERSIST_WAL     10
#define SQLITE_FCNTL_OVERWRITE       11

/*
** CAPI3REF: Mutex Handle
**
** The mutex module within SQLite defines [sqlite3_mutex] to be an
** abstract type for a mutex object.  The SQLite core never looks
** at the internal representation of an [sqlite3_mutex].  It only
** deals with pointers to the [sqlite3_mutex] object.
**
** Mutexes are created using [sqlite3_mutex_alloc()].
*/
typedef struct sqlite3_mutex sqlite3_mutex;

/*
** CAPI3REF: OS Interface Object
**
** An instance of the sqlite3_vfs object defines the interface between
** the SQLite core and the underlying operating system.  The "vfs"
** in the name of the object stands for "virtual file system".  See
** the [VFS | VFS documentation] for further information.
**
** The value of the iVersion field is initially 1 but may be larger in
** future versions of SQLite.  Additional fields may be appended to this
** object when the iVersion value is increased.  Note that the structure
** of the sqlite3_vfs object changes in the transaction between
** SQLite version 3.5.9 and 3.6.0 and yet the iVersion field was not
** modified.
**
** The szOsFile field is the size of the subclassed [sqlite3_file]
** structure used by this VFS.  mxPathname is the maximum length of
** a pathname in this VFS.
**
** Registered sqlite3_vfs objects are kept on a linked list formed by
** the pNext pointer.  The [sqlite3_vfs_register()]
** and [sqlite3_vfs_unregister()] interfaces manage this list
** in a thread-safe way.  The [sqlite3_vfs_find()] interface
** searches the list.  Neither the application code nor the VFS
** implementation should use the pNext pointer.
**
** The pNext field is the only field in the sqlite3_vfs
** structure that SQLite will ever modify.  SQLite will only access
** or modify this field while holding a particular static mutex.
** The application should never modify anything within the sqlite3_vfs
** object once the object has been registered.
**
** The zName field holds the name of the VFS module.  The name must
** be unique across all VFS modules.
**
** [[sqlite3_vfs.xOpen]]
** ^SQLite guarantees that the zFilename parameter to xOpen
** is either a NULL pointer or string obtained
** from xFullPathname() with an optional suffix added.
** ^If a suffix is added to the zFilename parameter, it will
** consist of a single "-" character followed by no more than
** 10 alphanumeric and/or "-" characters.
** ^SQLite further guarantees that
** the string will be valid and unchanged until xClose() is
** called. Because of the previous sentence,
** the [sqlite3_file] can safely store a pointer to the
** filename if it needs to remember the filename for some reason.
** If the zFilename parameter to xOpen is a NULL pointer then xOpen
** must invent its own temporary name for the file.  ^Whenever the 
** xFilename parameter is NULL it will also be the case that the
** flags parameter will include [SQLITE_OPEN_DELETEONCLOSE].
**
** The flags argument to xOpen() includes all bits set in
** the flags argument to [sqlite3_open_v2()].  Or if [sqlite3_open()]
** or [sqlite3_open16()] is used, then flags includes at least
** [SQLITE_OPEN_READWRITE] | [SQLITE_OPEN_CREATE]. 
** If xOpen() opens a file read-only then it sets *pOutFlags to
** include [SQLITE_OPEN_READONLY].  Other bits in *pOutFlags may be set.
**
** ^(SQLite will also add one of the following flags to the xOpen()
** call, depending on the object being opened:
**
** <ul>
** <li>  [SQLITE_OPEN_MAIN_DB]
** <li>  [SQLITE_OPEN_MAIN_JOURNAL]
** <li>  [SQLITE_OPEN_TEMP_DB]
** <li>  [SQLITE_OPEN_TEMP_JOURNAL]
** <li>  [SQLITE_OPEN_TRANSIENT_DB]
** <li>  [SQLITE_OPEN_SUBJOURNAL]
** <li>  [SQLITE_OPEN_MASTER_JOURNAL]
** <li>  [SQLITE_OPEN_WAL]
** </ul>)^
**
** The file I/O implementation can use the object type flags to
** change the way it deals with files.  For example, an application
** that does not care about crash recovery or rollback might make
** the open of a journal file a no-op.  Writes to this journal would
** also be no-ops, and any attempt to read the journal would return
** SQLITE_IOERR.  Or the implementation might recognize that a database
** file will be doing page-aligned sector reads and writes in a random
** order and set up its I/O subsystem accordingly.
**
** SQLite might also add one of the following flags to the xOpen method:
**
** <ul>
** <li> [SQLITE_OPEN_DELETEONCLOSE]
** <li> [SQLITE_OPEN_EXCLUSIVE]
** </ul>
**
** The [SQLITE_OPEN_DELETEONCLOSE] flag means the file should be
** deleted when it is closed.  ^The [SQLITE_OPEN_DELETEONCLOSE]
** will be set for TEMP databases and their journals, transient
** databases, and subjournals.
**
** ^The [SQLITE_OPEN_EXCLUSIVE] flag is always used in conjunction
** with the [SQLITE_OPEN_CREATE] flag, which are both directly
** analogous to the O_EXCL and O_CREAT flags of the POSIX open()
** API.  The SQLITE_OPEN_EXCLUSIVE flag, when paired with the 
** SQLITE_OPEN_CREATE, is used to indicate that file should always
** be created, and that it is an error if it already exists.
** It is <i>not</i> used to indicate the file should be opened 
** for exclusive access.
**
** ^At least szOsFile bytes of memory are allocated by SQLite
** to hold the  [sqlite3_file] structure passed as the third
** argument to xOpen.  The xOpen method does not have to
** allocate the structure; it should just fill it in.  Note that
** the xOpen method must set the sqlite3_file.pMethods to either
** a valid [sqlite3_io_methods] object or to NULL.  xOpen must do
** this even if the open fails.  SQLite expects that the sqlite3_file.pMethods
** element will be valid after xOpen returns regardless of the success
** or failure of the xOpen call.
**
** [[sqlite3_vfs.xAccess]]
** ^The flags argument to xAccess() may be [SQLITE_ACCESS_EXISTS]
** to test for the existence of a file, or [SQLITE_ACCESS_READWRITE] to
** test whether a file is readable and writable, or [SQLITE_ACCESS_READ]
** to test whether a file is at least readable.   The file can be a
** directory.
**
** ^SQLite will always allocate at least mxPathname+1 bytes for the
** output buffer xFullPathname.  The exact size of the output buffer
** is also passed as a parameter to both  methods. If the output buffer
** is not large enough, [SQLITE_CANTOPEN] should be returned. Since this is
** handled as a fatal error by SQLite, vfs implementations should endeavor
** to prevent this by setting mxPathname to a sufficiently large value.
**
** The xRandomness(), xSleep(), xCurrentTime(), and xCurrentTimeInt64()
** interfaces are not strictly a part of the filesystem, but they are
** included in the VFS structure for completeness.
** The xRandomness() function attempts to return nBytes bytes
** of good-quality randomness into zOut.  The return value is
** the actual number of bytes of randomness obtained.
** The xSleep() method causes the calling thread to sleep for at
** least the number of microseconds given.  ^The xCurrentTime()
** method returns a Julian Day Number for the current date and time as
** a floating point value.
** ^The xCurrentTimeInt64() method returns, as an integer, the Julian
** Day Number multiplied by 86400000 (the number of milliseconds in 
** a 24-hour day).  
** ^SQLite will use the xCurrentTimeInt64() method to get the current
** date and time if that method is available (if iVersion is 2 or 
** greater and the function pointer is not NULL) and will fall back
** to xCurrentTime() if xCurrentTimeInt64() is unavailable.
**
** ^The xSetSystemCall(), xGetSystemCall(), and xNestSystemCall() interfaces
** are not used by the SQLite core.  These optional interfaces are provided
** by some VFSes to facilitate testing of the VFS code. By overriding 
** system calls with functions under its control, a test program can
** simulate faults and error conditions that would otherwise be difficult
** or impossible to induce.  The set of system calls that can be overridden
** varies from one VFS to another, and from one version of the same VFS to the
** next.  Applications that use these interfaces must be prepared for any
** or all of these interfaces to be NULL or for their behavior to change
** from one release to the next.  Applications must not attempt to access
** any of these methods if the iVersion of the VFS is less than 3.
*/
typedef struct sqlite3_vfs sqlite3_vfs;
typedef void (*sqlite3_syscall_ptr)(void);
struct sqlite3_vfs {
  int iVersion;            /* Structure version number (currently 3) */
  int szOsFile;            /* Size of subclassed sqlite3_file */
  int mxPathname;          /* Maximum file pathname length */
  sqlite3_vfs *pNext;      /* Next registered VFS */
  const char *zName;       /* Name of this virtual file system */
  void *pAppData;          /* Pointer to application-specific data */
  int (*xOpen)(sqlite3_vfs*, const char *zName, sqlite3_file*,
               int flags, int *pOutFlags);
  int (*xDelete)(sqlite3_vfs*, const char *zName, int syncDir);
  int (*xAccess)(sqlite3_vfs*, const char *zName, int flags, int *pResOut);
  int (*xFullPathname)(sqlite3_vfs*, const char *zName, int nOut, char *zOut);
  void *(*xDlOpen)(sqlite3_vfs*, const char *zFilename);
  void (*xDlError)(sqlite3_vfs*, int nByte, char *zErrMsg);
  void (*(*xDlSym)(sqlite3_vfs*,void*, const char *zSymbol))(void);
  void (*xDlClose)(sqlite3_vfs*, void*);
  int (*xRandomness)(sqlite3_vfs*, int nByte, char *zOut);
  int (*xSleep)(sqlite3_vfs*, int microseconds);
  int (*xCurrentTime)(sqlite3_vfs*, double*);
  int (*xGetLastError)(sqlite3_vfs*, int, char *);
  /*
  ** The methods above are in version 1 of the sqlite_vfs object
  ** definition.  Those that follow are added in version 2 or later
  */
  int (*xCurrentTimeInt64)(sqlite3_vfs*, sqlite3_int64*);
  /*
  ** The methods above are in versions 1 and 2 of the sqlite_vfs object.
  ** Those below are for version 3 and greater.
  */
  int (*xSetSystemCall)(sqlite3_vfs*, const char *zName, sqlite3_syscall_ptr);
  sqlite3_syscall_ptr (*xGetSystemCall)(sqlite3_vfs*, const char *zName);
  const char *(*xNextSystemCall)(sqlite3_vfs*, const char *zName);
  /*
  ** The methods above are in versions 1 through 3 of the sqlite_vfs object.
  ** New fields may be appended in figure versions.  The iVersion
  ** value will increment whenever this happens. 
  */
};

/*
** CAPI3REF: Flags for the xAccess VFS method
**
** These integer constants can be used as the third parameter to
** the xAccess method of an [sqlite3_vfs] object.  They determine
** what kind of permissions the xAccess method is looking for.
** With SQLITE_ACCESS_EXISTS, the xAccess method
** simply checks whether the file exists.
** With SQLITE_ACCESS_READWRITE, the xAccess method
** checks whether the named directory is both readable and writable
** (in other words, if files can be added, removed, and renamed within
** the directory).
** The SQLITE_ACCESS_READWRITE constant is currently used only by the
** [temp_store_directory pragma], though this could change in a future
** release of SQLite.
** With SQLITE_ACCESS_READ, the xAccess method
** checks whether the file is readable.  The SQLITE_ACCESS_READ constant is
** currently unused, though it might be used in a future release of
** SQLite.
*/
#define SQLITE_ACCESS_EXISTS    0
#define SQLITE_ACCESS_READWRITE 1   /* Used by PRAGMA temp_store_directory */
#define SQLITE_ACCESS_READ      2   /* Unused */

/*
** CAPI3REF: Flags for the xShmLock VFS method
**
** These integer constants define the various locking operations
** allowed by the xShmLock method of [sqlite3_io_methods].  The
** following are the only legal combinations of flags to the
** xShmLock method:
**
** <ul>
** <li>  SQLITE_SHM_LOCK | SQLITE_SHM_SHARED
** <li>  SQLITE_SHM_LOCK | SQLITE_SHM_EXCLUSIVE
** <li>  SQLITE_SHM_UNLOCK | SQLITE_SHM_SHARED
** <li>  SQLITE_SHM_UNLOCK | SQLITE_SHM_EXCLUSIVE
** </ul>
**
** When unlocking, the same SHARED or EXCLUSIVE flag must be supplied as
** was given no the corresponding lock.  
**
** The xShmLock method can transition between unlocked and SHARED or
** between unlocked and EXCLUSIVE.  It cannot transition between SHARED
** and EXCLUSIVE.
*/
#define SQLITE_SHM_UNLOCK       1
#define SQLITE_SHM_LOCK         2
#define SQLITE_SHM_SHARED       4
#define SQLITE_SHM_EXCLUSIVE    8

/*
** CAPI3REF: Maximum xShmLock index
**
** The xShmLock method on [sqlite3_io_methods] may use values
** between 0 and this upper bound as its "offset" argument.
** The SQLite core will never attempt to acquire or release a
** lock outside of this range
*/
#define SQLITE_SHM_NLOCK        8


/*
** CAPI3REF: Initialize The SQLite Library
**
** ^The sqlite3_initialize() routine initializes the
** SQLite library.  ^The sqlite3_shutdown() routine
** deallocates any resources that were allocated by sqlite3_initialize().
** These routines are designed to aid in process initialization and
** shutdown on embedded systems.  Workstation applications using
** SQLite normally do not need to invoke either of these routines.
**
** A call to sqlite3_initialize() is an "effective" call if it is
** the first time sqlite3_initialize() is invoked during the lifetime of
** the process, or if it is the first time sqlite3_initialize() is invoked
** following a call to sqlite3_shutdown().  ^(Only an effective call
** of sqlite3_initialize() does any initialization.  All other calls
** are harmless no-ops.)^
**
** A call to sqlite3_shutdown() is an "effective" call if it is the first
** call to sqlite3_shutdown() since the last sqlite3_initialize().  ^(Only
** an effective call to sqlite3_shutdown() does any deinitialization.
** All other valid calls to sqlite3_shutdown() are harmless no-ops.)^
**
** The sqlite3_initialize() interface is threadsafe, but sqlite3_shutdown()
** is not.  The sqlite3_shutdown() interface must only be called from a
** single thread.  All open [database connections] must be closed and all
** other SQLite resources must be deallocated prior to invoking
** sqlite3_shutdown().
**
** Among other things, ^sqlite3_initialize() will invoke
** sqlite3_os_init().  Similarly, ^sqlite3_shutdown()
** will invoke sqlite3_os_end().
**
** ^The sqlite3_initialize() routine returns [SQLITE_OK] on success.
** ^If for some reason, sqlite3_initialize() is unable to initialize
** the library (perhaps it is unable to allocate a needed resource such
** as a mutex) it returns an [error code] other than [SQLITE_OK].
**
** ^The sqlite3_initialize() routine is called internally by many other
** SQLite interfaces so that an application usually does not need to
** invoke sqlite3_initialize() directly.  For example, [sqlite3_open()]
** calls sqlite3_initialize() so the SQLite library will be automatically
** initialized when [sqlite3_open()] is called if it has not be initialized
** already.  ^However, if SQLite is compiled with the [SQLITE_OMIT_AUTOINIT]
** compile-time option, then the automatic calls to sqlite3_initialize()
** are omitted and the application must call sqlite3_initialize() directly
** prior to using any other SQLite interface.  For maximum portability,
** it is recommended that applications always invoke sqlite3_initialize()
** directly prior to using any other SQLite interface.  Future releases
** of SQLite may require this.  In other words, the behavior exhibited
** when SQLite is compiled with [SQLITE_OMIT_AUTOINIT] might become the
** default behavior in some future release of SQLite.
**
** The sqlite3_os_init() routine does operating-system specific
** initialization of the SQLite library.  The sqlite3_os_end()
** routine undoes the effect of sqlite3_os_init().  Typical tasks
** performed by these routines include allocation or deallocation
** of static resources, initialization of global variables,
** setting up a default [sqlite3_vfs] module, or setting up
** a default configuration using [sqlite3_config()].
**
** The application should never invoke either sqlite3_os_init()
** or sqlite3_os_end() directly.  The application should only invoke
** sqlite3_initialize() and sqlite3_shutdown().  The sqlite3_os_init()
** interface is called automatically by sqlite3_initialize() and
** sqlite3_os_end() is called by sqlite3_shutdown().  Appropriate
** implementations for sqlite3_os_init() and sqlite3_os_end()
** are built into SQLite when it is compiled for Unix, Windows, or OS/2.
** When [custom builds | built for other platforms]
** (using the [SQLITE_OS_OTHER=1] compile-time
** option) the application must supply a suitable implementation for
** sqlite3_os_init() and sqlite3_os_end().  An application-supplied
** implementation of sqlite3_os_init() or sqlite3_os_end()
** must return [SQLITE_OK] on success and some other [error code] upon
** failure.
*/
SQLITE_API int sqlite3_initialize(void);
SQLITE_API int sqlite3_shutdown(void);
SQLITE_API int sqlite3_os_init(void);
SQLITE_API int sqlite3_os_end(void);

/*
** CAPI3REF: Configuring The SQLite Library
**
** The sqlite3_config() interface is used to make global configuration
** changes to SQLite in order to tune SQLite to the specific needs of
** the application.  The default configuration is recommended for most
** applications and so this routine is usually not necessary.  It is
** provided to support rare applications with unusual needs.
**
** The sqlite3_config() interface is not threadsafe.  The application
** must insure that no other SQLite interfaces are invoked by other
** threads while sqlite3_config() is running.  Furthermore, sqlite3_config()
** may only be invoked prior to library initialization using
** [sqlite3_initialize()] or after shutdown by [sqlite3_shutdown()].
** ^If sqlite3_config() is called after [sqlite3_initialize()] and before
** [sqlite3_shutdown()] then it will return SQLITE_MISUSE.
** Note, however, that ^sqlite3_config() can be called as part of the
** implementation of an application-defined [sqlite3_os_init()].
**
** The first argument to sqlite3_config() is an integer
** [configuration option] that determines
** what property of SQLite is to be configured.  Subsequent arguments
** vary depending on the [configuration option]
** in the first argument.
**
** ^When a configuration option is set, sqlite3_config() returns [SQLITE_OK].
** ^If the option is unknown or SQLite is unable to set the option
** then this routine returns a non-zero [error code].
*/
SQLITE_API int sqlite3_config(int, ...);

/*
** CAPI3REF: Configure database connections
**
** The sqlite3_db_config() interface is used to make configuration
** changes to a [database connection].  The interface is similar to
** [sqlite3_config()] except that the changes apply to a single
** [database connection] (specified in the first argument).
**
** The second argument to sqlite3_db_config(D,V,...)  is the
** [SQLITE_DBCONFIG_LOOKASIDE | configuration verb] - an integer code 
** that indicates what aspect of the [database connection] is being configured.
** Subsequent arguments vary depending on the configuration verb.
**
** ^Calls to sqlite3_db_config() return SQLITE_OK if and only if
** the call is considered successful.
*/
SQLITE_API int sqlite3_db_config(sqlite3*, int op, ...);

/*
** CAPI3REF: Memory Allocation Routines
**
** An instance of this object defines the interface between SQLite
** and low-level memory allocation routines.
**
** This object is used in only one place in the SQLite interface.
** A pointer to an instance of this object is the argument to
** [sqlite3_config()] when the configuration option is
** [SQLITE_CONFIG_MALLOC] or [SQLITE_CONFIG_GETMALLOC].  
** By creating an instance of this object
** and passing it to [sqlite3_config]([SQLITE_CONFIG_MALLOC])
** during configuration, an application can specify an alternative
** memory allocation subsystem for SQLite to use for all of its
** dynamic memory needs.
**
** Note that SQLite comes with several [built-in memory allocators]
** that are perfectly adequate for the overwhelming majority of applications
** and that this object is only useful to a tiny minority of applications
** with specialized memory allocation requirements.  This object is
** also used during testing of SQLite in order to specify an alternative
** memory allocator that simulates memory out-of-memory conditions in
** order to verify that SQLite recovers gracefully from such
** conditions.
**
** The xMalloc, xRealloc, and xFree methods must work like the
** malloc(), realloc() and free() functions from the standard C library.
** ^SQLite guarantees that the second argument to
** xRealloc is always a value returned by a prior call to xRoundup.
**
** xSize should return the allocated size of a memory allocation
** previously obtained from xMalloc or xRealloc.  The allocated size
** is always at least as big as the requested size but may be larger.
**
** The xRoundup method returns what would be the allocated size of
** a memory allocation given a particular requested size.  Most memory
** allocators round up memory allocations at least to the next multiple
** of 8.  Some allocators round up to a larger multiple or to a power of 2.
** Every memory allocation request coming in through [sqlite3_malloc()]
** or [sqlite3_realloc()] first calls xRoundup.  If xRoundup returns 0, 
** that causes the corresponding memory allocation to fail.
**
** The xInit method initializes the memory allocator.  (For example,
** it might allocate any require mutexes or initialize internal data
** structures.  The xShutdown method is invoked (indirectly) by
** [sqlite3_shutdown()] and should deallocate any resources acquired
** by xInit.  The pAppData pointer is used as the only parameter to
** xInit and xShutdown.
**
** SQLite holds the [SQLITE_MUTEX_STATIC_MASTER] mutex when it invokes
** the xInit method, so the xInit method need not be threadsafe.  The
** xShutdown method is only called from [sqlite3_shutdown()] so it does
** not need to be threadsafe either.  For all other methods, SQLite
** holds the [SQLITE_MUTEX_STATIC_MEM] mutex as long as the
** [SQLITE_CONFIG_MEMSTATUS] configuration option is turned on (which
** it is by default) and so the methods are automatically serialized.
** However, if [SQLITE_CONFIG_MEMSTATUS] is disabled, then the other
** methods must be threadsafe or else make their own arrangements for
** serialization.
**
** SQLite will never invoke xInit() more than once without an intervening
** call to xShutdown().
*/
typedef struct sqlite3_mem_methods sqlite3_mem_methods;
struct sqlite3_mem_methods {
  void *(*xMalloc)(int);         /* Memory allocation function */
  void (*xFree)(void*);          /* Free a prior allocation */
  void *(*xRealloc)(void*,int);  /* Resize an allocation */
  int (*xSize)(void*);           /* Return the size of an allocation */
  int (*xRoundup)(int);          /* Round up request size to allocation size */
  int (*xInit)(void*);           /* Initialize the memory allocator */
  void (*xShutdown)(void*);      /* Deinitialize the memory allocator */
  void *pAppData;                /* Argument to xInit() and xShutdown() */
};

/*
** CAPI3REF: Configuration Options
** KEYWORDS: {configuration option}
**
** These constants are the available integer configuration options that
** can be passed as the first argument to the [sqlite3_config()] interface.
**
** New configuration options may be added in future releases of SQLite.
** Existing configuration options might be discontinued.  Applications
** should check the return code from [sqlite3_config()] to make sure that
** the call worked.  The [sqlite3_config()] interface will return a
** non-zero [error code] if a discontinued or unsupported configuration option
** is invoked.
**
** <dl>
** [[SQLITE_CONFIG_SINGLETHREAD]] <dt>SQLITE_CONFIG_SINGLETHREAD</dt>
** <dd>There are no arguments to this option.  ^This option sets the
** [threading mode] to Single-thread.  In other words, it disables
** all mutexing and puts SQLite into a mode where it can only be used
** by a single thread.   ^If SQLite is compiled with
** the [SQLITE_THREADSAFE | SQLITE_THREADSAFE=0] compile-time option then
** it is not possible to change the [threading mode] from its default
** value of Single-thread and so [sqlite3_config()] will return 
** [SQLITE_ERROR] if called with the SQLITE_CONFIG_SINGLETHREAD
** configuration option.</dd>
**
** [[SQLITE_CONFIG_MULTITHREAD]] <dt>SQLITE_CONFIG_MULTITHREAD</dt>
** <dd>There are no arguments to this option.  ^This option sets the
** [threading mode] to Multi-thread.  In other words, it disables
** mutexing on [database connection] and [prepared statement] objects.
** The application is responsible for serializing access to
** [database connections] and [prepared statements].  But other mutexes
** are enabled so that SQLite will be safe to use in a multi-threaded
** environment as long as no two threads attempt to use the same
** [database connection] at the same time.  ^If SQLite is compiled with
** the [SQLITE_THREADSAFE | SQLITE_THREADSAFE=0] compile-time option then
** it is not possible to set the Multi-thread [threading mode] and
** [sqlite3_config()] will return [SQLITE_ERROR] if called with the
** SQLITE_CONFIG_MULTITHREAD configuration option.</dd>
**
** [[SQLITE_CONFIG_SERIALIZED]] <dt>SQLITE_CONFIG_SERIALIZED</dt>
** <dd>There are no arguments to this option.  ^This option sets the
** [threading mode] to Serialized. In other words, this option enables
** all mutexes including the recursive
** mutexes on [database connection] and [prepared statement] objects.
** In this mode (which is the default when SQLite is compiled with
** [SQLITE_THREADSAFE=1]) the SQLite library will itself serialize access
** to [database connections] and [prepared statements] so that the
** application is free to use the same [database connection] or the
** same [prepared statement] in different threads at the same time.
** ^If SQLite is compiled with
** the [SQLITE_THREADSAFE | SQLITE_THREADSAFE=0] compile-time option then
** it is not possible to set the Serialized [threading mode] and
** [sqlite3_config()] will return [SQLITE_ERROR] if called with the
** SQLITE_CONFIG_SERIALIZED configuration option.</dd>
**
** [[SQLITE_CONFIG_MALLOC]] <dt>SQLITE_CONFIG_MALLOC</dt>
** <dd> ^(This option takes a single argument which is a pointer to an
** instance of the [sqlite3_mem_methods] structure.  The argument specifies
** alternative low-level memory allocation routines to be used in place of
** the memory allocation routines built into SQLite.)^ ^SQLite makes
** its own private copy of the content of the [sqlite3_mem_methods] structure
** before the [sqlite3_config()] call returns.</dd>
**
** [[SQLITE_CONFIG_GETMALLOC]] <dt>SQLITE_CONFIG_GETMALLOC</dt>
** <dd> ^(This option takes a single argument which is a pointer to an
** instance of the [sqlite3_mem_methods] structure.  The [sqlite3_mem_methods]
** structure is filled with the currently defined memory allocation routines.)^
** This option can be used to overload the default memory allocation
** routines with a wrapper that simulations memory allocation failure or
** tracks memory usage, for example. </dd>
**
** [[SQLITE_CONFIG_MEMSTATUS]] <dt>SQLITE_CONFIG_MEMSTATUS</dt>
** <dd> ^This option takes single argument of type int, interpreted as a 
** boolean, which enables or disables the collection of memory allocation 
** statistics. ^(When memory allocation statistics are disabled, the 
** following SQLite interfaces become non-operational:
**   <ul>
**   <li> [sqlite3_memory_used()]
**   <li> [sqlite3_memory_highwater()]
**   <li> [sqlite3_soft_heap_limit64()]
**   <li> [sqlite3_status()]
**   </ul>)^
** ^Memory allocation statistics are enabled by default unless SQLite is
** compiled with [SQLITE_DEFAULT_MEMSTATUS]=0 in which case memory
** allocation statistics are disabled by default.
** </dd>
**
** [[SQLITE_CONFIG_SCRATCH]] <dt>SQLITE_CONFIG_SCRATCH</dt>
** <dd> ^This option specifies a static memory buffer that SQLite can use for
** scratch memory.  There are three arguments:  A pointer an 8-byte
** aligned memory buffer from which the scratch allocations will be
** drawn, the size of each scratch allocation (sz),
** and the maximum number of scratch allocations (N).  The sz
** argument must be a multiple of 16.
** The first argument must be a pointer to an 8-byte aligned buffer
** of at least sz*N bytes of memory.
** ^SQLite will use no more than two scratch buffers per thread.  So
** N should be set to twice the expected maximum number of threads.
** ^SQLite will never require a scratch buffer that is more than 6
** times the database page size. ^If SQLite needs needs additional
** scratch memory beyond what is provided by this configuration option, then 
** [sqlite3_malloc()] will be used to obtain the memory needed.</dd>
**
** [[SQLITE_CONFIG_PAGECACHE]] <dt>SQLITE_CONFIG_PAGECACHE</dt>
** <dd> ^This option specifies a static memory buffer that SQLite can use for
** the database page cache with the default page cache implementation.  
** This configuration should not be used if an application-define page
** cache implementation is loaded using the SQLITE_CONFIG_PCACHE option.
** There are three arguments to this option: A pointer to 8-byte aligned
** memory, the size of each page buffer (sz), and the number of pages (N).
** The sz argument should be the size of the largest database page
** (a power of two between 512 and 32768) plus a little extra for each
** page header.  ^The page header size is 20 to 40 bytes depending on
** the host architecture.  ^It is harmless, apart from the wasted memory,
** to make sz a little too large.  The first
** argument should point to an allocation of at least sz*N bytes of memory.
** ^SQLite will use the memory provided by the first argument to satisfy its
** memory needs for the first N pages that it adds to cache.  ^If additional
** page cache memory is needed beyond what is provided by this option, then
** SQLite goes to [sqlite3_malloc()] for the additional storage space.
** The pointer in the first argument must
** be aligned to an 8-byte boundary or subsequent behavior of SQLite
** will be undefined.</dd>
**
** [[SQLITE_CONFIG_HEAP]] <dt>SQLITE_CONFIG_HEAP</dt>
** <dd> ^This option specifies a static memory buffer that SQLite will use
** for all of its dynamic memory allocation needs beyond those provided
** for by [SQLITE_CONFIG_SCRATCH] and [SQLITE_CONFIG_PAGECACHE].
** There are three arguments: An 8-byte aligned pointer to the memory,
** the number of bytes in the memory buffer, and the minimum allocation size.
** ^If the first pointer (the memory pointer) is NULL, then SQLite reverts
** to using its default memory allocator (the system malloc() implementation),
** undoing any prior invocation of [SQLITE_CONFIG_MALLOC].  ^If the
** memory pointer is not NULL and either [SQLITE_ENABLE_MEMSYS3] or
** [SQLITE_ENABLE_MEMSYS5] are defined, then the alternative memory
** allocator is engaged to handle all of SQLites memory allocation needs.
** The first pointer (the memory pointer) must be aligned to an 8-byte
** boundary or subsequent behavior of SQLite will be undefined.
** The minimum allocation size is capped at 2**12. Reasonable values
** for the minimum allocation size are 2**5 through 2**8.</dd>
**
** [[SQLITE_CONFIG_MUTEX]] <dt>SQLITE_CONFIG_MUTEX</dt>
** <dd> ^(This option takes a single argument which is a pointer to an
** instance of the [sqlite3_mutex_methods] structure.  The argument specifies
** alternative low-level mutex routines to be used in place
** the mutex routines built into SQLite.)^  ^SQLite makes a copy of the
** content of the [sqlite3_mutex_methods] structure before the call to
** [sqlite3_config()] returns. ^If SQLite is compiled with
** the [SQLITE_THREADSAFE | SQLITE_THREADSAFE=0] compile-time option then
** the entire mutexing subsystem is omitted from the build and hence calls to
** [sqlite3_config()] with the SQLITE_CONFIG_MUTEX configuration option will
** return [SQLITE_ERROR].</dd>
**
** [[SQLITE_CONFIG_GETMUTEX]] <dt>SQLITE_CONFIG_GETMUTEX</dt>
** <dd> ^(This option takes a single argument which is a pointer to an
** instance of the [sqlite3_mutex_methods] structure.  The
** [sqlite3_mutex_methods]
** structure is filled with the currently defined mutex routines.)^
** This option can be used to overload the default mutex allocation
** routines with a wrapper used to track mutex usage for performance
** profiling or testing, for example.   ^If SQLite is compiled with
** the [SQLITE_THREADSAFE | SQLITE_THREADSAFE=0] compile-time option then
** the entire mutexing subsystem is omitted from the build and hence calls to
** [sqlite3_config()] with the SQLITE_CONFIG_GETMUTEX configuration option will
** return [SQLITE_ERROR].</dd>
**
** [[SQLITE_CONFIG_LOOKASIDE]] <dt>SQLITE_CONFIG_LOOKASIDE</dt>
** <dd> ^(This option takes two arguments that determine the default
** memory allocation for the lookaside memory allocator on each
** [database connection].  The first argument is the
** size of each lookaside buffer slot and the second is the number of
** slots allocated to each database connection.)^  ^(This option sets the
** <i>default</i> lookaside size. The [SQLITE_DBCONFIG_LOOKASIDE]
** verb to [sqlite3_db_config()] can be used to change the lookaside
** configuration on individual connections.)^ </dd>
**
** [[SQLITE_CONFIG_PCACHE]] <dt>SQLITE_CONFIG_PCACHE</dt>
** <dd> ^(This option takes a single argument which is a pointer to
** an [sqlite3_pcache_methods] object.  This object specifies the interface
** to a custom page cache implementation.)^  ^SQLite makes a copy of the
** object and uses it for page cache memory allocations.</dd>
**
** [[SQLITE_CONFIG_GETPCACHE]] <dt>SQLITE_CONFIG_GETPCACHE</dt>
** <dd> ^(This option takes a single argument which is a pointer to an
** [sqlite3_pcache_methods] object.  SQLite copies of the current
** page cache implementation into that object.)^ </dd>
**
** [[SQLITE_CONFIG_LOG]] <dt>SQLITE_CONFIG_LOG</dt>
** <dd> ^The SQLITE_CONFIG_LOG option takes two arguments: a pointer to a
** function with a call signature of void(*)(void*,int,const char*), 
** and a pointer to void. ^If the function pointer is not NULL, it is
** invoked by [sqlite3_log()] to process each logging event.  ^If the
** function pointer is NULL, the [sqlite3_log()] interface becomes a no-op.
** ^The void pointer that is the second argument to SQLITE_CONFIG_LOG is
** passed through as the first parameter to the application-defined logger
** function whenever that function is invoked.  ^The second parameter to
** the logger function is a copy of the first parameter to the corresponding
** [sqlite3_log()] call and is intended to be a [result code] or an
** [extended result code].  ^The third parameter passed to the logger is
** log message after formatting via [sqlite3_snprintf()].
** The SQLite logging interface is not reentrant; the logger function
** supplied by the application must not invoke any SQLite interface.
** In a multi-threaded application, the application-defined logger
** function must be threadsafe. </dd>
**
** [[SQLITE_CONFIG_URI]] <dt>SQLITE_CONFIG_URI
** <dd> This option takes a single argument of type int. If non-zero, then
** URI handling is globally enabled. If the parameter is zero, then URI handling
** is globally disabled. If URI handling is globally enabled, all filenames
** passed to [sqlite3_open()], [sqlite3_open_v2()], [sqlite3_open16()] or
** specified as part of [ATTACH] commands are interpreted as URIs, regardless
** of whether or not the [SQLITE_OPEN_URI] flag is set when the database
** connection is opened. If it is globally disabled, filenames are
** only interpreted as URIs if the SQLITE_OPEN_URI flag is set when the
** database connection is opened. By default, URI handling is globally
** disabled. The default value may be changed by compiling with the
** [SQLITE_USE_URI] symbol defined.
** </dl>
*/
#define SQLITE_CONFIG_SINGLETHREAD  1  /* nil */
#define SQLITE_CONFIG_MULTITHREAD   2  /* nil */
#define SQLITE_CONFIG_SERIALIZED    3  /* nil */
#define SQLITE_CONFIG_MALLOC        4  /* sqlite3_mem_methods* */
#define SQLITE_CONFIG_GETMALLOC     5  /* sqlite3_mem_methods* */
#define SQLITE_CONFIG_SCRATCH       6  /* void*, int sz, int N */
#define SQLITE_CONFIG_PAGECACHE     7  /* void*, int sz, int N */
#define SQLITE_CONFIG_HEAP          8  /* void*, int nByte, int min */
#define SQLITE_CONFIG_MEMSTATUS     9  /* boolean */
#define SQLITE_CONFIG_MUTEX        10  /* sqlite3_mutex_methods* */
#define SQLITE_CONFIG_GETMUTEX     11  /* sqlite3_mutex_methods* */
/* previously SQLITE_CONFIG_CHUNKALLOC 12 which is now unused. */ 
#define SQLITE_CONFIG_LOOKASIDE    13  /* int int */
#define SQLITE_CONFIG_PCACHE       14  /* sqlite3_pcache_methods* */
#define SQLITE_CONFIG_GETPCACHE    15  /* sqlite3_pcache_methods* */
#define SQLITE_CONFIG_LOG          16  /* xFunc, void* */
#define SQLITE_CONFIG_URI          17  /* int */

/*
** CAPI3REF: Database Connection Configuration Options
**
** These constants are the available integer configuration options that
** can be passed as the second argument to the [sqlite3_db_config()] interface.
**
** New configuration options may be added in future releases of SQLite.
** Existing configuration options might be discontinued.  Applications
** should check the return code from [sqlite3_db_config()] to make sure that
** the call worked.  ^The [sqlite3_db_config()] interface will return a
** non-zero [error code] if a discontinued or unsupported configuration option
** is invoked.
**
** <dl>
** <dt>SQLITE_DBCONFIG_LOOKASIDE</dt>
** <dd> ^This option takes three additional arguments that determine the 
** [lookaside memory allocator] configuration for the [database connection].
** ^The first argument (the third parameter to [sqlite3_db_config()] is a
** pointer to a memory buffer to use for lookaside memory.
** ^The first argument after the SQLITE_DBCONFIG_LOOKASIDE verb
** may be NULL in which case SQLite will allocate the
** lookaside buffer itself using [sqlite3_malloc()]. ^The second argument is the
** size of each lookaside buffer slot.  ^The third argument is the number of
** slots.  The size of the buffer in the first argument must be greater than
** or equal to the product of the second and third arguments.  The buffer
** must be aligned to an 8-byte boundary.  ^If the second argument to
** SQLITE_DBCONFIG_LOOKASIDE is not a multiple of 8, it is internally
** rounded down to the next smaller multiple of 8.  ^(The lookaside memory
** configuration for a database connection can only be changed when that
** connection is not currently using lookaside memory, or in other words
** when the "current value" returned by
** [sqlite3_db_status](D,[SQLITE_CONFIG_LOOKASIDE],...) is zero.
** Any attempt to change the lookaside memory configuration when lookaside
** memory is in use leaves the configuration unchanged and returns 
** [SQLITE_BUSY].)^</dd>
**
** <dt>SQLITE_DBCONFIG_ENABLE_FKEY</dt>
** <dd> ^This option is used to enable or disable the enforcement of
** [foreign key constraints].  There should be two additional arguments.
** The first argument is an integer which is 0 to disable FK enforcement,
** positive to enable FK enforcement or negative to leave FK enforcement
** unchanged.  The second parameter is a pointer to an integer into which
** is written 0 or 1 to indicate whether FK enforcement is off or on
** following this call.  The second parameter may be a NULL pointer, in
** which case the FK enforcement setting is not reported back. </dd>
**
** <dt>SQLITE_DBCONFIG_ENABLE_TRIGGER</dt>
** <dd> ^This option is used to enable or disable [CREATE TRIGGER | triggers].
** There should be two additional arguments.
** The first argument is an integer which is 0 to disable triggers,
** positive to enable triggers or negative to leave the setting unchanged.
** The second parameter is a pointer to an integer into which
** is written 0 or 1 to indicate whether triggers are disabled or enabled
** following this call.  The second parameter may be a NULL pointer, in
** which case the trigger setting is not reported back. </dd>
**
** </dl>
*/
#define SQLITE_DBCONFIG_LOOKASIDE       1001  /* void* int int */
#define SQLITE_DBCONFIG_ENABLE_FKEY     1002  /* int int* */
#define SQLITE_DBCONFIG_ENABLE_TRIGGER  1003  /* int int* */


/*
** CAPI3REF: Enable Or Disable Extended Result Codes
**
** ^The sqlite3_extended_result_codes() routine enables or disables the
** [extended result codes] feature of SQLite. ^The extended result
** codes are disabled by default for historical compatibility.
*/
SQLITE_API int sqlite3_extended_result_codes(sqlite3*, int onoff);

/*
** CAPI3REF: Last Insert Rowid
**
** ^Each entry in an SQLite table has a unique 64-bit signed
** integer key called the [ROWID | "rowid"]. ^The rowid is always available
** as an undeclared column named ROWID, OID, or _ROWID_ as long as those
** names are not also used by explicitly declared columns. ^If
** the table has a column of type [INTEGER PRIMARY KEY] then that column
** is another alias for the rowid.
**
** ^This routine returns the [rowid] of the most recent
** successful [INSERT] into the database from the [database connection]
** in the first argument.  ^As of SQLite version 3.7.7, this routines
** records the last insert rowid of both ordinary tables and [virtual tables].
** ^If no successful [INSERT]s
** have ever occurred on that database connection, zero is returned.
**
** ^(If an [INSERT] occurs within a trigger or within a [virtual table]
** method, then this routine will return the [rowid] of the inserted
** row as long as the trigger or virtual table method is running.
** But once the trigger or virtual table method ends, the value returned 
** by this routine reverts to what it was before the trigger or virtual
** table method began.)^
**
** ^An [INSERT] that fails due to a constraint violation is not a
** successful [INSERT] and does not change the value returned by this
** routine.  ^Thus INSERT OR FAIL, INSERT OR IGNORE, INSERT OR ROLLBACK,
** and INSERT OR ABORT make no changes to the return value of this
** routine when their insertion fails.  ^(When INSERT OR REPLACE
** encounters a constraint violation, it does not fail.  The
** INSERT continues to completion after deleting rows that caused
** the constraint problem so INSERT OR REPLACE will always change
** the return value of this interface.)^
**
** ^For the purposes of this routine, an [INSERT] is considered to
** be successful even if it is subsequently rolled back.
**
** This function is accessible to SQL statements via the
** [last_insert_rowid() SQL function].
**
** If a separate thread performs a new [INSERT] on the same
** database connection while the [sqlite3_last_insert_rowid()]
** function is running and thus changes the last insert [rowid],
** then the value returned by [sqlite3_last_insert_rowid()] is
** unpredictable and might not equal either the old or the new
** last insert [rowid].
*/
SQLITE_API sqlite3_int64 sqlite3_last_insert_rowid(sqlite3*);

/*
** CAPI3REF: Count The Number Of Rows Modified
**
** ^This function returns the number of database rows that were changed
** or inserted or deleted by the most recently completed SQL statement
** on the [database connection] specified by the first parameter.
** ^(Only changes that are directly specified by the [INSERT], [UPDATE],
** or [DELETE] statement are counted.  Auxiliary changes caused by
** triggers or [foreign key actions] are not counted.)^ Use the
** [sqlite3_total_changes()] function to find the total number of changes
** including changes caused by triggers and foreign key actions.
**
** ^Changes to a view that are simulated by an [INSTEAD OF trigger]
** are not counted.  Only real table changes are counted.
**
** ^(A "row change" is a change to a single row of a single table
** caused by an INSERT, DELETE, or UPDATE statement.  Rows that
** are changed as side effects of [REPLACE] constraint resolution,
** rollback, ABORT processing, [DROP TABLE], or by any other
** mechanisms do not count as direct row changes.)^
**
** A "trigger context" is a scope of execution that begins and
** ends with the script of a [CREATE TRIGGER | trigger]. 
** Most SQL statements are
** evaluated outside of any trigger.  This is the "top level"
** trigger context.  If a trigger fires from the top level, a
** new trigger context is entered for the duration of that one
** trigger.  Subtriggers create subcontexts for their duration.
**
** ^Calling [sqlite3_exec()] or [sqlite3_step()] recursively does
** not create a new trigger context.
**
** ^This function returns the number of direct row changes in the
** most recent INSERT, UPDATE, or DELETE statement within the same
** trigger context.
**
** ^Thus, when called from the top level, this function returns the
** number of changes in the most recent INSERT, UPDATE, or DELETE
** that also occurred at the top level.  ^(Within the body of a trigger,
** the sqlite3_changes() interface can be called to find the number of
** changes in the most recently completed INSERT, UPDATE, or DELETE
** statement within the body of the same trigger.
** However, the number returned does not include changes
** caused by subtriggers since those have their own context.)^
**
** See also the [sqlite3_total_changes()] interface, the
** [count_changes pragma], and the [changes() SQL function].
**
** If a separate thread makes changes on the same database connection
** while [sqlite3_changes()] is running then the value returned
** is unpredictable and not meaningful.
*/
SQLITE_API int sqlite3_changes(sqlite3*);

/*
** CAPI3REF: Total Number Of Rows Modified
**
** ^This function returns the number of row changes caused by [INSERT],
** [UPDATE] or [DELETE] statements since the [database connection] was opened.
** ^(The count returned by sqlite3_total_changes() includes all changes
** from all [CREATE TRIGGER | trigger] contexts and changes made by
** [foreign key actions]. However,
** the count does not include changes used to implement [REPLACE] constraints,
** do rollbacks or ABORT processing, or [DROP TABLE] processing.  The
** count does not include rows of views that fire an [INSTEAD OF trigger],
** though if the INSTEAD OF trigger makes changes of its own, those changes 
** are counted.)^
** ^The sqlite3_total_changes() function counts the changes as soon as
** the statement that makes them is completed (when the statement handle
** is passed to [sqlite3_reset()] or [sqlite3_finalize()]).
**
** See also the [sqlite3_changes()] interface, the
** [count_changes pragma], and the [total_changes() SQL function].
**
** If a separate thread makes changes on the same database connection
** while [sqlite3_total_changes()] is running then the value
** returned is unpredictable and not meaningful.
*/
SQLITE_API int sqlite3_total_changes(sqlite3*);

/*
** CAPI3REF: Interrupt A Long-Running Query
**
** ^This function causes any pending database operation to abort and
** return at its earliest opportunity. This routine is typically
** called in response to a user action such as pressing "Cancel"
** or Ctrl-C where the user wants a long query operation to halt
** immediately.
**
** ^It is safe to call this routine from a thread different from the
** thread that is currently running the database operation.  But it
** is not safe to call this routine with a [database connection] that
** is closed or might close before sqlite3_interrupt() returns.
**
** ^If an SQL operation is very nearly finished at the time when
** sqlite3_interrupt() is called, then it might not have an opportunity
** to be interrupted and might continue to completion.
**
** ^An SQL operation that is interrupted will return [SQLITE_INTERRUPT].
** ^If the interrupted SQL operation is an INSERT, UPDATE, or DELETE
** that is inside an explicit transaction, then the entire transaction
** will be rolled back automatically.
**
** ^The sqlite3_interrupt(D) call is in effect until all currently running
** SQL statements on [database connection] D complete.  ^Any new SQL statements
** that are started after the sqlite3_interrupt() call and before the 
** running statements reaches zero are interrupted as if they had been
** running prior to the sqlite3_interrupt() call.  ^New SQL statements
** that are started after the running statement count reaches zero are
** not effected by the sqlite3_interrupt().
** ^A call to sqlite3_interrupt(D) that occurs when there are no running
** SQL statements is a no-op and has no effect on SQL statements
** that are started after the sqlite3_interrupt() call returns.
**
** If the database connection closes while [sqlite3_interrupt()]
** is running then bad things will likely happen.
*/
SQLITE_API void sqlite3_interrupt(sqlite3*);

/*
** CAPI3REF: Determine If An SQL Statement Is Complete
**
** These routines are useful during command-line input to determine if the
** currently entered text seems to form a complete SQL statement or
** if additional input is needed before sending the text into
** SQLite for parsing.  ^These routines return 1 if the input string
** appears to be a complete SQL statement.  ^A statement is judged to be
** complete if it ends with a semicolon token and is not a prefix of a
** well-formed CREATE TRIGGER statement.  ^Semicolons that are embedded within
** string literals or quoted identifier names or comments are not
** independent tokens (they are part of the token in which they are
** embedded) and thus do not count as a statement terminator.  ^Whitespace
** and comments that follow the final semicolon are ignored.
**
** ^These routines return 0 if the statement is incomplete.  ^If a
** memory allocation fails, then SQLITE_NOMEM is returned.
**
** ^These routines do not parse the SQL statements thus
** will not detect syntactically incorrect SQL.
**
** ^(If SQLite has not been initialized using [sqlite3_initialize()] prior 
** to invoking sqlite3_complete16() then sqlite3_initialize() is invoked
** automatically by sqlite3_complete16().  If that initialization fails,
** then the return value from sqlite3_complete16() will be non-zero
** regardless of whether or not the input SQL is complete.)^
**
** The input to [sqlite3_complete()] must be a zero-terminated
** UTF-8 string.
**
** The input to [sqlite3_complete16()] must be a zero-terminated
** UTF-16 string in native byte order.
*/
SQLITE_API int sqlite3_complete(const char *sql);
SQLITE_API int sqlite3_complete16(const void *sql);

/*
** CAPI3REF: Register A Callback To Handle SQLITE_BUSY Errors
**
** ^This routine sets a callback function that might be invoked whenever
** an attempt is made to open a database table that another thread
** or process has locked.
**
** ^If the busy callback is NULL, then [SQLITE_BUSY] or [SQLITE_IOERR_BLOCKED]
** is returned immediately upon encountering the lock.  ^If the busy callback
** is not NULL, then the callback might be invoked with two arguments.
**
** ^The first argument to the busy handler is a copy of the void* pointer which
** is the third argument to sqlite3_busy_handler().  ^The second argument to
** the busy handler callback is the number of times that the busy handler has
** been invoked for this locking event.  ^If the
** busy callback returns 0, then no additional attempts are made to
** access the database and [SQLITE_BUSY] or [SQLITE_IOERR_BLOCKED] is returned.
** ^If the callback returns non-zero, then another attempt
** is made to open the database for reading and the cycle repeats.
**
** The presence of a busy handler does not guarantee that it will be invoked
** when there is lock contention. ^If SQLite determines that invoking the busy
** handler could result in a deadlock, it will go ahead and return [SQLITE_BUSY]
** or [SQLITE_IOERR_BLOCKED] instead of invoking the busy handler.
** Consider a scenario where one process is holding a read lock that
** it is trying to promote to a reserved lock and
** a second process is holding a reserved lock that it is trying
** to promote to an exclusive lock.  The first process cannot proceed
** because it is blocked by the second and the second process cannot
** proceed because it is blocked by the first.  If both processes
** invoke the busy handlers, neither will make any progress.  Therefore,
** SQLite returns [SQLITE_BUSY] for the first process, hoping that this
** will induce the first process to release its read lock and allow
** the second process to proceed.
**
** ^The default busy callback is NULL.
**
** ^The [SQLITE_BUSY] error is converted to [SQLITE_IOERR_BLOCKED]
** when SQLite is in the middle of a large transaction where all the
** changes will not fit into the in-memory cache.  SQLite will
** already hold a RESERVED lock on the database file, but it needs
** to promote this lock to EXCLUSIVE so that it can spill cache
** pages into the database file without harm to concurrent
** readers.  ^If it is unable to promote the lock, then the in-memory
** cache will be left in an inconsistent state and so the error
** code is promoted from the relatively benign [SQLITE_BUSY] to
** the more severe [SQLITE_IOERR_BLOCKED].  ^This error code promotion
** forces an automatic rollback of the changes.  See the
** <a href="/cvstrac/wiki?p=CorruptionFollowingBusyError">
** CorruptionFollowingBusyError</a> wiki page for a discussion of why
** this is important.
**
** ^(There can only be a single busy handler defined for each
** [database connection].  Setting a new busy handler clears any
** previously set handler.)^  ^Note that calling [sqlite3_busy_timeout()]
** will also set or clear the busy handler.
**
** The busy callback should not take any actions which modify the
** database connection that invoked the busy handler.  Any such actions
** result in undefined behavior.
** 
** A busy handler must not close the database connection
** or [prepared statement] that invoked the busy handler.
*/
SQLITE_API int sqlite3_busy_handler(sqlite3*, int(*)(void*,int), void*);

/*
** CAPI3REF: Set A Busy Timeout
**
** ^This routine sets a [sqlite3_busy_handler | busy handler] that sleeps
** for a specified amount of time when a table is locked.  ^The handler
** will sleep multiple times until at least "ms" milliseconds of sleeping
** have accumulated.  ^After at least "ms" milliseconds of sleeping,
** the handler returns 0 which causes [sqlite3_step()] to return
** [SQLITE_BUSY] or [SQLITE_IOERR_BLOCKED].
**
** ^Calling this routine with an argument less than or equal to zero
** turns off all busy handlers.
**
** ^(There can only be a single busy handler for a particular
** [database connection] any any given moment.  If another busy handler
** was defined  (using [sqlite3_busy_handler()]) prior to calling
** this routine, that other busy handler is cleared.)^
*/
SQLITE_API int sqlite3_busy_timeout(sqlite3*, int ms);

/*
** CAPI3REF: Convenience Routines For Running Queries
**
** This is a legacy interface that is preserved for backwards compatibility.
** Use of this interface is not recommended.
**
** Definition: A <b>result table</b> is memory data structure created by the
** [sqlite3_get_table()] interface.  A result table records the
** complete query results from one or more queries.
**
** The table conceptually has a number of rows and columns.  But
** these numbers are not part of the result table itself.  These
** numbers are obtained separately.  Let N be the number of rows
** and M be the number of columns.
**
** A result table is an array of pointers to zero-terminated UTF-8 strings.
** There are (N+1)*M elements in the array.  The first M pointers point
** to zero-terminated strings that  contain the names of the columns.
** The remaining entries all point to query results.  NULL values result
** in NULL pointers.  All other values are in their UTF-8 zero-terminated
** string representation as returned by [sqlite3_column_text()].
**
** A result table might consist of one or more memory allocations.
** It is not safe to pass a result table directly to [sqlite3_free()].
** A result table should be deallocated using [sqlite3_free_table()].
**
** ^(As an example of the result table format, suppose a query result
** is as follows:
**
** <blockquote><pre>
**        Name        | Age
**        -----------------------
**        Alice       | 43
**        Bob         | 28
**        Cindy       | 21
** </pre></blockquote>
**
** There are two column (M==2) and three rows (N==3).  Thus the
** result table has 8 entries.  Suppose the result table is stored
** in an array names azResult.  Then azResult holds this content:
**
** <blockquote><pre>
**        azResult&#91;0] = "Name";
**        azResult&#91;1] = "Age";
**        azResult&#91;2] = "Alice";
**        azResult&#91;3] = "43";
**        azResult&#91;4] = "Bob";
**        azResult&#91;5] = "28";
**        azResult&#91;6] = "Cindy";
**        azResult&#91;7] = "21";
** </pre></blockquote>)^
**
** ^The sqlite3_get_table() function evaluates one or more
** semicolon-separated SQL statements in the zero-terminated UTF-8
** string of its 2nd parameter and returns a result table to the
** pointer given in its 3rd parameter.
**
** After the application has finished with the result from sqlite3_get_table(),
** it must pass the result table pointer to sqlite3_free_table() in order to
** release the memory that was malloced.  Because of the way the
** [sqlite3_malloc()] happens within sqlite3_get_table(), the calling
** function must not try to call [sqlite3_free()] directly.  Only
** [sqlite3_free_table()] is able to release the memory properly and safely.
**
** The sqlite3_get_table() interface is implemented as a wrapper around
** [sqlite3_exec()].  The sqlite3_get_table() routine does not have access
** to any internal data structures of SQLite.  It uses only the public
** interface defined here.  As a consequence, errors that occur in the
** wrapper layer outside of the internal [sqlite3_exec()] call are not
** reflected in subsequent calls to [sqlite3_errcode()] or
** [sqlite3_errmsg()].
*/
SQLITE_API int sqlite3_get_table(
  sqlite3 *db,          /* An open database */
  const char *zSql,     /* SQL to be evaluated */
  char ***pazResult,    /* Results of the query */
  int *pnRow,           /* Number of result rows written here */
  int *pnColumn,        /* Number of result columns written here */
  char **pzErrmsg       /* Error msg written here */
);
SQLITE_API void sqlite3_free_table(char **result);

/*
** CAPI3REF: Formatted String Printing Functions
**
** These routines are work-alikes of the "printf()" family of functions
** from the standard C library.
**
** ^The sqlite3_mprintf() and sqlite3_vmprintf() routines write their
** results into memory obtained from [sqlite3_malloc()].
** The strings returned by these two routines should be
** released by [sqlite3_free()].  ^Both routines return a
** NULL pointer if [sqlite3_malloc()] is unable to allocate enough
** memory to hold the resulting string.
**
** ^(The sqlite3_snprintf() routine is similar to "snprintf()" from
** the standard C library.  The result is written into the
** buffer supplied as the second parameter whose size is given by
** the first parameter. Note that the order of the
** first two parameters is reversed from snprintf().)^  This is an
** historical accident that cannot be fixed without breaking
** backwards compatibility.  ^(Note also that sqlite3_snprintf()
** returns a pointer to its buffer instead of the number of
** characters actually written into the buffer.)^  We admit that
** the number of characters written would be a more useful return
** value but we cannot change the implementation of sqlite3_snprintf()
** now without breaking compatibility.
**
** ^As long as the buffer size is greater than zero, sqlite3_snprintf()
** guarantees that the buffer is always zero-terminated.  ^The first
** parameter "n" is the total size of the buffer, including space for
** the zero terminator.  So the longest string that can be completely
** written will be n-1 characters.
**
** ^The sqlite3_vsnprintf() routine is a varargs version of sqlite3_snprintf().
**
** These routines all implement some additional formatting
** options that are useful for constructing SQL statements.
** All of the usual printf() formatting options apply.  In addition, there
** is are "%q", "%Q", and "%z" options.
**
** ^(The %q option works like %s in that it substitutes a null-terminated
** string from the argument list.  But %q also doubles every '\'' character.
** %q is designed for use inside a string literal.)^  By doubling each '\''
** character it escapes that character and allows it to be inserted into
** the string.
**
** For example, assume the string variable zText contains text as follows:
**
** <blockquote><pre>
**  char *zText = "It's a happy day!";
** </pre></blockquote>
**
** One can use this text in an SQL statement as follows:
**
** <blockquote><pre>
**  char *zSQL = sqlite3_mprintf("INSERT INTO table VALUES('%q')", zText);
**  sqlite3_exec(db, zSQL, 0, 0, 0);
**  sqlite3_free(zSQL);
** </pre></blockquote>
**
** Because the %q format string is used, the '\'' character in zText
** is escaped and the SQL generated is as follows:
**
** <blockquote><pre>
**  INSERT INTO table1 VALUES('It''s a happy day!')
** </pre></blockquote>
**
** This is correct.  Had we used %s instead of %q, the generated SQL
** would have looked like this:
**
** <blockquote><pre>
**  INSERT INTO table1 VALUES('It's a happy day!');
** </pre></blockquote>
**
** This second example is an SQL syntax error.  As a general rule you should
** always use %q instead of %s when inserting text into a string literal.
**
** ^(The %Q option works like %q except it also adds single quotes around
** the outside of the total string.  Additionally, if the parameter in the
** argument list is a NULL pointer, %Q substitutes the text "NULL" (without
** single quotes).)^  So, for example, one could say:
**
** <blockquote><pre>
**  char *zSQL = sqlite3_mprintf("INSERT INTO table VALUES(%Q)", zText);
**  sqlite3_exec(db, zSQL, 0, 0, 0);
**  sqlite3_free(zSQL);
** </pre></blockquote>
**
** The code above will render a correct SQL statement in the zSQL
** variable even if the zText variable is a NULL pointer.
**
** ^(The "%z" formatting option works like "%s" but with the
** addition that after the string has been read and copied into
** the result, [sqlite3_free()] is called on the input string.)^
*/
SQLITE_API char *sqlite3_mprintf(const char*,...);
SQLITE_API char *sqlite3_vmprintf(const char*, va_list);
SQLITE_API char *sqlite3_snprintf(int,char*,const char*, ...);
SQLITE_API char *sqlite3_vsnprintf(int,char*,const char*, va_list);

/*
** CAPI3REF: Memory Allocation Subsystem
**
** The SQLite core uses these three routines for all of its own
** internal memory allocation needs. "Core" in the previous sentence
** does not include operating-system specific VFS implementation.  The
** Windows VFS uses native malloc() and free() for some operations.
**
** ^The sqlite3_malloc() routine returns a pointer to a block
** of memory at least N bytes in length, where N is the parameter.
** ^If sqlite3_malloc() is unable to obtain sufficient free
** memory, it returns a NULL pointer.  ^If the parameter N to
** sqlite3_malloc() is zero or negative then sqlite3_malloc() returns
** a NULL pointer.
**
** ^Calling sqlite3_free() with a pointer previously returned
** by sqlite3_malloc() or sqlite3_realloc() releases that memory so
** that it might be reused.  ^The sqlite3_free() routine is
** a no-op if is called with a NULL pointer.  Passing a NULL pointer
** to sqlite3_free() is harmless.  After being freed, memory
** should neither be read nor written.  Even reading previously freed
** memory might result in a segmentation fault or other severe error.
** Memory corruption, a segmentation fault, or other severe error
** might result if sqlite3_free() is called with a non-NULL pointer that
** was not obtained from sqlite3_malloc() or sqlite3_realloc().
**
** ^(The sqlite3_realloc() interface attempts to resize a
** prior memory allocation to be at least N bytes, where N is the
** second parameter.  The memory allocation to be resized is the first
** parameter.)^ ^ If the first parameter to sqlite3_realloc()
** is a NULL pointer then its behavior is identical to calling
** sqlite3_malloc(N) where N is the second parameter to sqlite3_realloc().
** ^If the second parameter to sqlite3_realloc() is zero or
** negative then the behavior is exactly the same as calling
** sqlite3_free(P) where P is the first parameter to sqlite3_realloc().
** ^sqlite3_realloc() returns a pointer to a memory allocation
** of at least N bytes in size or NULL if sufficient memory is unavailable.
** ^If M is the size of the prior allocation, then min(N,M) bytes
** of the prior allocation are copied into the beginning of buffer returned
** by sqlite3_realloc() and the prior allocation is freed.
** ^If sqlite3_realloc() returns NULL, then the prior allocation
** is not freed.
**
** ^The memory returned by sqlite3_malloc() and sqlite3_realloc()
** is always aligned to at least an 8 byte boundary, or to a
** 4 byte boundary if the [SQLITE_4_BYTE_ALIGNED_MALLOC] compile-time
** option is used.
**
** In SQLite version 3.5.0 and 3.5.1, it was possible to define
** the SQLITE_OMIT_MEMORY_ALLOCATION which would cause the built-in
** implementation of these routines to be omitted.  That capability
** is no longer provided.  Only built-in memory allocators can be used.
**
** The Windows OS interface layer calls
** the system malloc() and free() directly when converting
** filenames between the UTF-8 encoding used by SQLite
** and whatever filename encoding is used by the particular Windows
** installation.  Memory allocation errors are detected, but
** they are reported back as [SQLITE_CANTOPEN] or
** [SQLITE_IOERR] rather than [SQLITE_NOMEM].
**
** The pointer arguments to [sqlite3_free()] and [sqlite3_realloc()]
** must be either NULL or else pointers obtained from a prior
** invocation of [sqlite3_malloc()] or [sqlite3_realloc()] that have
** not yet been released.
**
** The application must not read or write any part of
** a block of memory after it has been released using
** [sqlite3_free()] or [sqlite3_realloc()].
*/
SQLITE_API void *sqlite3_malloc(int);
SQLITE_API void *sqlite3_realloc(void*, int);
SQLITE_API void sqlite3_free(void*);

/*
** CAPI3REF: Memory Allocator Statistics
**
** SQLite provides these two interfaces for reporting on the status
** of the [sqlite3_malloc()], [sqlite3_free()], and [sqlite3_realloc()]
** routines, which form the built-in memory allocation subsystem.
**
** ^The [sqlite3_memory_used()] routine returns the number of bytes
** of memory currently outstanding (malloced but not freed).
** ^The [sqlite3_memory_highwater()] routine returns the maximum
** value of [sqlite3_memory_used()] since the high-water mark
** was last reset.  ^The values returned by [sqlite3_memory_used()] and
** [sqlite3_memory_highwater()] include any overhead
** added by SQLite in its implementation of [sqlite3_malloc()],
** but not overhead added by the any underlying system library
** routines that [sqlite3_malloc()] may call.
**
** ^The memory high-water mark is reset to the current value of
** [sqlite3_memory_used()] if and only if the parameter to
** [sqlite3_memory_highwater()] is true.  ^The value returned
** by [sqlite3_memory_highwater(1)] is the high-water mark
** prior to the reset.
*/
SQLITE_API sqlite3_int64 sqlite3_memory_used(void);
SQLITE_API sqlite3_int64 sqlite3_memory_highwater(int resetFlag);

/*
** CAPI3REF: Pseudo-Random Number Generator
**
** SQLite contains a high-quality pseudo-random number generator (PRNG) used to
** select random [ROWID | ROWIDs] when inserting new records into a table that
** already uses the largest possible [ROWID].  The PRNG is also used for
** the build-in random() and randomblob() SQL functions.  This interface allows
** applications to access the same PRNG for other purposes.
**
** ^A call to this routine stores N bytes of randomness into buffer P.
**
** ^The first time this routine is invoked (either internally or by
** the application) the PRNG is seeded using randomness obtained
** from the xRandomness method of the default [sqlite3_vfs] object.
** ^On all subsequent invocations, the pseudo-randomness is generated
** internally and without recourse to the [sqlite3_vfs] xRandomness
** method.
*/
SQLITE_API void sqlite3_randomness(int N, void *P);

/*
** CAPI3REF: Compile-Time Authorization Callbacks
**
** ^This routine registers an authorizer callback with a particular
** [database connection], supplied in the first argument.
** ^The authorizer callback is invoked as SQL statements are being compiled
** by [sqlite3_prepare()] or its variants [sqlite3_prepare_v2()],
** [sqlite3_prepare16()] and [sqlite3_prepare16_v2()].  ^At various
** points during the compilation process, as logic is being created
** to perform various actions, the authorizer callback is invoked to
** see if those actions are allowed.  ^The authorizer callback should
** return [SQLITE_OK] to allow the action, [SQLITE_IGNORE] to disallow the
** specific action but allow the SQL statement to continue to be
** compiled, or [SQLITE_DENY] to cause the entire SQL statement to be
** rejected with an error.  ^If the authorizer callback returns
** any value other than [SQLITE_IGNORE], [SQLITE_OK], or [SQLITE_DENY]
** then the [sqlite3_prepare_v2()] or equivalent call that triggered
** the authorizer will fail with an error message.
**
** When the callback returns [SQLITE_OK], that means the operation
** requested is ok.  ^When the callback returns [SQLITE_DENY], the
** [sqlite3_prepare_v2()] or equivalent call that triggered the
** authorizer will fail with an error message explaining that
** access is denied. 
**
** ^The first parameter to the authorizer callback is a copy of the third
** parameter to the sqlite3_set_authorizer() interface. ^The second parameter
** to the callback is an integer [SQLITE_COPY | action code] that specifies
** the particular action to be authorized. ^The third through sixth parameters
** to the callback are zero-terminated strings that contain additional
** details about the action to be authorized.
**
** ^If the action code is [SQLITE_READ]
** and the callback returns [SQLITE_IGNORE] then the
** [prepared statement] statement is constructed to substitute
** a NULL value in place of the table column that would have
** been read if [SQLITE_OK] had been returned.  The [SQLITE_IGNORE]
** return can be used to deny an untrusted user access to individual
** columns of a table.
** ^If the action code is [SQLITE_DELETE] and the callback returns
** [SQLITE_IGNORE] then the [DELETE] operation proceeds but the
** [truncate optimization] is disabled and all rows are deleted individually.
**
** An authorizer is used when [sqlite3_prepare | preparing]
** SQL statements from an untrusted source, to ensure that the SQL statements
** do not try to access data they are not allowed to see, or that they do not
** try to execute malicious statements that damage the database.  For
** example, an application may allow a user to enter arbitrary
** SQL queries for evaluation by a database.  But the application does
** not want the user to be able to make arbitrary changes to the
** database.  An authorizer could then be put in place while the
** user-entered SQL is being [sqlite3_prepare | prepared] that
** disallows everything except [SELECT] statements.
**
** Applications that need to process SQL from untrusted sources
** might also consider lowering resource limits using [sqlite3_limit()]
** and limiting database size using the [max_page_count] [PRAGMA]
** in addition to using an authorizer.
**
** ^(Only a single authorizer can be in place on a database connection
** at a time.  Each call to sqlite3_set_authorizer overrides the
** previous call.)^  ^Disable the authorizer by installing a NULL callback.
** The authorizer is disabled by default.
**
** The authorizer callback must not do anything that will modify
** the database connection that invoked the authorizer callback.
** Note that [sqlite3_prepare_v2()] and [sqlite3_step()] both modify their
** database connections for the meaning of "modify" in this paragraph.
**
** ^When [sqlite3_prepare_v2()] is used to prepare a statement, the
** statement might be re-prepared during [sqlite3_step()] due to a 
** schema change.  Hence, the application should ensure that the
** correct authorizer callback remains in place during the [sqlite3_step()].
**
** ^Note that the authorizer callback is invoked only during
** [sqlite3_prepare()] or its variants.  Authorization is not
** performed during statement evaluation in [sqlite3_step()], unless
** as stated in the previous paragraph, sqlite3_step() invokes
** sqlite3_prepare_v2() to reprepare a statement after a schema change.
*/
SQLITE_API int sqlite3_set_authorizer(
  sqlite3*,
  int (*xAuth)(void*,int,const char*,const char*,const char*,const char*),
  void *pUserData
);

/*
** CAPI3REF: Authorizer Return Codes
**
** The [sqlite3_set_authorizer | authorizer callback function] must
** return either [SQLITE_OK] or one of these two constants in order
** to signal SQLite whether or not the action is permitted.  See the
** [sqlite3_set_authorizer | authorizer documentation] for additional
** information.
**
** Note that SQLITE_IGNORE is also used as a [SQLITE_ROLLBACK | return code]
** from the [sqlite3_vtab_on_conflict()] interface.
*/
#define SQLITE_DENY   1   /* Abort the SQL statement with an error */
#define SQLITE_IGNORE 2   /* Don't allow access, but don't generate an error */

/*
** CAPI3REF: Authorizer Action Codes
**
** The [sqlite3_set_authorizer()] interface registers a callback function
** that is invoked to authorize certain SQL statement actions.  The
** second parameter to the callback is an integer code that specifies
** what action is being authorized.  These are the integer action codes that
** the authorizer callback may be passed.
**
** These action code values signify what kind of operation is to be
** authorized.  The 3rd and 4th parameters to the authorization
** callback function will be parameters or NULL depending on which of these
** codes is used as the second parameter.  ^(The 5th parameter to the
** authorizer callback is the name of the database ("main", "temp",
** etc.) if applicable.)^  ^The 6th parameter to the authorizer callback
** is the name of the inner-most trigger or view that is responsible for
** the access attempt or NULL if this access attempt is directly from
** top-level SQL code.
*/
/******************************************* 3rd ************ 4th ***********/
#define SQLITE_CREATE_INDEX          1   /* Index Name      Table Name      */
#define SQLITE_CREATE_TABLE          2   /* Table Name      NULL            */
#define SQLITE_CREATE_TEMP_INDEX     3   /* Index Name      Table Name      */
#define SQLITE_CREATE_TEMP_TABLE     4   /* Table Name      NULL            */
#define SQLITE_CREATE_TEMP_TRIGGER   5   /* Trigger Name    Table Name      */
#define SQLITE_CREATE_TEMP_VIEW      6   /* View Name       NULL            */
#define SQLITE_CREATE_TRIGGER        7   /* Trigger Name    Table Name      */
#define SQLITE_CREATE_VIEW           8   /* View Name       NULL            */
#define SQLITE_DELETE                9   /* Table Name      NULL            */
#define SQLITE_DROP_INDEX           10   /* Index Name      Table Name      */
#define SQLITE_DROP_TABLE           11   /* Table Name      NULL            */
#define SQLITE_DROP_TEMP_INDEX      12   /* Index Name      Table Name      */
#define SQLITE_DROP_TEMP_TABLE      13   /* Table Name      NULL            */
#define SQLITE_DROP_TEMP_TRIGGER    14   /* Trigger Name    Table Name      */
#define SQLITE_DROP_TEMP_VIEW       15   /* View Name       NULL            */
#define SQLITE_DROP_TRIGGER         16   /* Trigger Name    Table Name      */
#define SQLITE_DROP_VIEW            17   /* View Name       NULL            */
#define SQLITE_INSERT               18   /* Table Name      NULL            */
#define SQLITE_PRAGMA               19   /* Pragma Name     1st arg or NULL */
#define SQLITE_READ                 20   /* Table Name      Column Name     */
#define SQLITE_SELECT               21   /* NULL            NULL            */
#define SQLITE_TRANSACTION          22   /* Operation       NULL            */
#define SQLITE_UPDATE               23   /* Table Name      Column Name     */
#define SQLITE_ATTACH               24   /* Filename        NULL            */
#define SQLITE_DETACH               25   /* Database Name   NULL            */
#define SQLITE_ALTER_TABLE          26   /* Database Name   Table Name      */
#define SQLITE_REINDEX              27   /* Index Name      NULL            */
#define SQLITE_ANALYZE              28   /* Table Name      NULL            */
#define SQLITE_CREATE_VTABLE        29   /* Table Name      Module Name     */
#define SQLITE_DROP_VTABLE          30   /* Table Name      Module Name     */
#define SQLITE_FUNCTION             31   /* NULL            Function Name   */
#define SQLITE_SAVEPOINT            32   /* Operation       Savepoint Name  */
#define SQLITE_COPY                  0   /* No longer used */

/*
** CAPI3REF: Tracing And Profiling Functions
**
** These routines register callback functions that can be used for
** tracing and profiling the execution of SQL statements.
**
** ^The callback function registered by sqlite3_trace() is invoked at
** various times when an SQL statement is being run by [sqlite3_step()].
** ^The sqlite3_trace() callback is invoked with a UTF-8 rendering of the
** SQL statement text as the statement first begins executing.
** ^(Additional sqlite3_trace() callbacks might occur
** as each triggered subprogram is entered.  The callbacks for triggers
** contain a UTF-8 SQL comment that identifies the trigger.)^
**
** ^The callback function registered by sqlite3_profile() is invoked
** as each SQL statement finishes.  ^The profile callback contains
** the original statement text and an estimate of wall-clock time
** of how long that statement took to run.  ^The profile callback
** time is in units of nanoseconds, however the current implementation
** is only capable of millisecond resolution so the six least significant
** digits in the time are meaningless.  Future versions of SQLite
** might provide greater resolution on the profiler callback.  The
** sqlite3_profile() function is considered experimental and is
** subject to change in future versions of SQLite.
*/
SQLITE_API void *sqlite3_trace(sqlite3*, void(*xTrace)(void*,const char*), void*);
SQLITE_API SQLITE_EXPERIMENTAL void *sqlite3_profile(sqlite3*,
   void(*xProfile)(void*,const char*,sqlite3_uint64), void*);

/*
** CAPI3REF: Query Progress Callbacks
**
** ^The sqlite3_progress_handler(D,N,X,P) interface causes the callback
** function X to be invoked periodically during long running calls to
** [sqlite3_exec()], [sqlite3_step()] and [sqlite3_get_table()] for
** database connection D.  An example use for this
** interface is to keep a GUI updated during a large query.
**
** ^The parameter P is passed through as the only parameter to the 
** callback function X.  ^The parameter N is the number of 
** [virtual machine instructions] that are evaluated between successive
** invocations of the callback X.
**
** ^Only a single progress handler may be defined at one time per
** [database connection]; setting a new progress handler cancels the
** old one.  ^Setting parameter X to NULL disables the progress handler.
** ^The progress handler is also disabled by setting N to a value less
** than 1.
**
** ^If the progress callback returns non-zero, the operation is
** interrupted.  This feature can be used to implement a
** "Cancel" button on a GUI progress dialog box.
**
** The progress handler callback must not do anything that will modify
** the database connection that invoked the progress handler.
** Note that [sqlite3_prepare_v2()] and [sqlite3_step()] both modify their
** database connections for the meaning of "modify" in this paragraph.
**
*/
SQLITE_API void sqlite3_progress_handler(sqlite3*, int, int(*)(void*), void*);

/*
** CAPI3REF: Opening A New Database Connection
**
** ^These routines open an SQLite database file as specified by the 
** filename argument. ^The filename argument is interpreted as UTF-8 for
** sqlite3_open() and sqlite3_open_v2() and as UTF-16 in the native byte
** order for sqlite3_open16(). ^(A [database connection] handle is usually
** returned in *ppDb, even if an error occurs.  The only exception is that
** if SQLite is unable to allocate memory to hold the [sqlite3] object,
** a NULL will be written into *ppDb instead of a pointer to the [sqlite3]
** object.)^ ^(If the database is opened (and/or created) successfully, then
** [SQLITE_OK] is returned.  Otherwise an [error code] is returned.)^ ^The
** [sqlite3_errmsg()] or [sqlite3_errmsg16()] routines can be used to obtain
** an English language description of the error following a failure of any
** of the sqlite3_open() routines.
**
** ^The default encoding for the database will be UTF-8 if
** sqlite3_open() or sqlite3_open_v2() is called and
** UTF-16 in the native byte order if sqlite3_open16() is used.
**
** Whether or not an error occurs when it is opened, resources
** associated with the [database connection] handle should be released by
** passing it to [sqlite3_close()] when it is no longer required.
**
** The sqlite3_open_v2() interface works like sqlite3_open()
** except that it accepts two additional parameters for additional control
** over the new database connection.  ^(The flags parameter to
** sqlite3_open_v2() can take one of
** the following three values, optionally combined with the 
** [SQLITE_OPEN_NOMUTEX], [SQLITE_OPEN_FULLMUTEX], [SQLITE_OPEN_SHAREDCACHE],
** [SQLITE_OPEN_PRIVATECACHE], and/or [SQLITE_OPEN_URI] flags:)^
**
** <dl>
** ^(<dt>[SQLITE_OPEN_READONLY]</dt>
** <dd>The database is opened in read-only mode.  If the database does not
** already exist, an error is returned.</dd>)^
**
** ^(<dt>[SQLITE_OPEN_READWRITE]</dt>
** <dd>The database is opened for reading and writing if possible, or reading
** only if the file is write protected by the operating system.  In either
** case the database must already exist, otherwise an error is returned.</dd>)^
**
** ^(<dt>[SQLITE_OPEN_READWRITE] | [SQLITE_OPEN_CREATE]</dt>
** <dd>The database is opened for reading and writing, and is created if
** it does not already exist. This is the behavior that is always used for
** sqlite3_open() and sqlite3_open16().</dd>)^
** </dl>
**
** If the 3rd parameter to sqlite3_open_v2() is not one of the
** combinations shown above optionally combined with other
** [SQLITE_OPEN_READONLY | SQLITE_OPEN_* bits]
** then the behavior is undefined.
**
** ^If the [SQLITE_OPEN_NOMUTEX] flag is set, then the database connection
** opens in the multi-thread [threading mode] as long as the single-thread
** mode has not been set at compile-time or start-time.  ^If the
** [SQLITE_OPEN_FULLMUTEX] flag is set then the database connection opens
** in the serialized [threading mode] unless single-thread was
** previously selected at compile-time or start-time.
** ^The [SQLITE_OPEN_SHAREDCACHE] flag causes the database connection to be
** eligible to use [shared cache mode], regardless of whether or not shared
** cache is enabled using [sqlite3_enable_shared_cache()].  ^The
** [SQLITE_OPEN_PRIVATECACHE] flag causes the database connection to not
** participate in [shared cache mode] even if it is enabled.
**
** ^The fourth parameter to sqlite3_open_v2() is the name of the
** [sqlite3_vfs] object that defines the operating system interface that
** the new database connection should use.  ^If the fourth parameter is
** a NULL pointer then the default [sqlite3_vfs] object is used.
**
** ^If the filename is ":memory:", then a private, temporary in-memory database
** is created for the connection.  ^This in-memory database will vanish when
** the database connection is closed.  Future versions of SQLite might
** make use of additional special filenames that begin with the ":" character.
** It is recommended that when a database filename actually does begin with
** a ":" character you should prefix the filename with a pathname such as
** "./" to avoid ambiguity.
**
** ^If the filename is an empty string, then a private, temporary
** on-disk database will be created.  ^This private database will be
** automatically deleted as soon as the database connection is closed.
**
** [[URI filenames in sqlite3_open()]] <h3>URI Filenames</h3>
**
** ^If [URI filename] interpretation is enabled, and the filename argument
** begins with "file:", then the filename is interpreted as a URI. ^URI
** filename interpretation is enabled if the [SQLITE_OPEN_URI] flag is
** set in the fourth argument to sqlite3_open_v2(), or if it has
** been enabled globally using the [SQLITE_CONFIG_URI] option with the
** [sqlite3_config()] method or by the [SQLITE_USE_URI] compile-time option.
** As of SQLite version 3.7.7, URI filename interpretation is turned off
** by default, but future releases of SQLite might enable URI filename
** interpretation by default.  See "[URI filenames]" for additional
** information.
**
** URI filenames are parsed according to RFC 3986. ^If the URI contains an
** authority, then it must be either an empty string or the string 
** "localhost". ^If the authority is not an empty string or "localhost", an 
** error is returned to the caller. ^The fragment component of a URI, if 
** present, is ignored.
**
** ^SQLite uses the path component of the URI as the name of the disk file
** which contains the database. ^If the path begins with a '/' character, 
** then it is interpreted as an absolute path. ^If the path does not begin 
** with a '/' (meaning that the authority section is omitted from the URI)
** then the path is interpreted as a relative path. 
** ^On windows, the first component of an absolute path 
** is a drive specification (e.g. "C:").
**
** [[core URI query parameters]]
** The query component of a URI may contain parameters that are interpreted
** either by SQLite itself, or by a [VFS | custom VFS implementation].
** SQLite interprets the following three query parameters:
**
** <ul>
**   <li> <b>vfs</b>: ^The "vfs" parameter may be used to specify the name of
**     a VFS object that provides the operating system interface that should
**     be used to access the database file on disk. ^If this option is set to
**     an empty string the default VFS object is used. ^Specifying an unknown
**     VFS is an error. ^If sqlite3_open_v2() is used and the vfs option is
**     present, then the VFS specified by the option takes precedence over
**     the value passed as the fourth parameter to sqlite3_open_v2().
**
**   <li> <b>mode</b>: ^(The mode parameter may be set to either "ro", "rw" or
**     "rwc". Attempting to set it to any other value is an error)^. 
**     ^If "ro" is specified, then the database is opened for read-only 
**     access, just as if the [SQLITE_OPEN_READONLY] flag had been set in the 
**     third argument to sqlite3_prepare_v2(). ^If the mode option is set to 
**     "rw", then the database is opened for read-write (but not create) 
**     access, as if SQLITE_OPEN_READWRITE (but not SQLITE_OPEN_CREATE) had 
**     been set. ^Value "rwc" is equivalent to setting both 
**     SQLITE_OPEN_READWRITE and SQLITE_OPEN_CREATE. ^If sqlite3_open_v2() is 
**     used, it is an error to specify a value for the mode parameter that is 
**     less restrictive than that specified by the flags passed as the third 
**     parameter.
**
**   <li> <b>cache</b>: ^The cache parameter may be set to either "shared" or
**     "private". ^Setting it to "shared" is equivalent to setting the
**     SQLITE_OPEN_SHAREDCACHE bit in the flags argument passed to
**     sqlite3_open_v2(). ^Setting the cache parameter to "private" is 
**     equivalent to setting the SQLITE_OPEN_PRIVATECACHE bit.
**     ^If sqlite3_open_v2() is used and the "cache" parameter is present in
**     a URI filename, its value overrides any behaviour requested by setting
**     SQLITE_OPEN_PRIVATECACHE or SQLITE_OPEN_SHAREDCACHE flag.
** </ul>
**
** ^Specifying an unknown parameter in the query component of a URI is not an
** error.  Future versions of SQLite might understand additional query
** parameters.  See "[query parameters with special meaning to SQLite]" for
** additional information.
**
** [[URI filename examples]] <h3>URI filename examples</h3>
**
** <table border="1" align=center cellpadding=5>
** <tr><th> URI filenames <th> Results
** <tr><td> file:data.db <td> 
**          Open the file "data.db" in the current directory.
** <tr><td> file:/home/fred/data.db<br>
**          file:///home/fred/data.db <br> 
**          file://localhost/home/fred/data.db <br> <td> 
**          Open the database file "/home/fred/data.db".
** <tr><td> file://darkstar/home/fred/data.db <td> 
**          An error. "darkstar" is not a recognized authority.
** <tr><td style="white-space:nowrap"> 
**          file:///C:/Documents%20and%20Settings/fred/Desktop/data.db
**     <td> Windows only: Open the file "data.db" on fred's desktop on drive
**          C:. Note that the %20 escaping in this example is not strictly 
**          necessary - space characters can be used literally
**          in URI filenames.
** <tr><td> file:data.db?mode=ro&cache=private <td> 
**          Open file "data.db" in the current directory for read-only access.
**          Regardless of whether or not shared-cache mode is enabled by
**          default, use a private cache.
** <tr><td> file:/home/fred/data.db?vfs=unix-nolock <td>
**          Open file "/home/fred/data.db". Use the special VFS "unix-nolock".
** <tr><td> file:data.db?mode=readonly <td> 
**          An error. "readonly" is not a valid option for the "mode" parameter.
** </table>
**
** ^URI hexadecimal escape sequences (%HH) are supported within the path and
** query components of a URI. A hexadecimal escape sequence consists of a
** percent sign - "%" - followed by exactly two hexadecimal digits 
** specifying an octet value. ^Before the path or query components of a
** URI filename are interpreted, they are encoded using UTF-8 and all 
** hexadecimal escape sequences replaced by a single byte containing the
** corresponding octet. If this process generates an invalid UTF-8 encoding,
** the results are undefined.
**
** <b>Note to Windows users:</b>  The encoding used for the filename argument
** of sqlite3_open() and sqlite3_open_v2() must be UTF-8, not whatever
** codepage is currently defined.  Filenames containing international
** characters must be converted to UTF-8 prior to passing them into
** sqlite3_open() or sqlite3_open_v2().
*/
SQLITE_API int sqlite3_open(
  const char *filename,   /* Database filename (UTF-8) */
  sqlite3 **ppDb          /* OUT: SQLite db handle */
);
SQLITE_API int sqlite3_open16(
  const void *filename,   /* Database filename (UTF-16) */
  sqlite3 **ppDb          /* OUT: SQLite db handle */
);
SQLITE_API int sqlite3_open_v2(
  const char *filename,   /* Database filename (UTF-8) */
  sqlite3 **ppDb,         /* OUT: SQLite db handle */
  int flags,              /* Flags */
  const char *zVfs        /* Name of VFS module to use */
);

/*
** CAPI3REF: Obtain Values For URI Parameters
**
** This is a utility routine, useful to VFS implementations, that checks
** to see if a database file was a URI that contained a specific query 
** parameter, and if so obtains the value of the query parameter.
**
** The zFilename argument is the filename pointer passed into the xOpen()
** method of a VFS implementation.  The zParam argument is the name of the
** query parameter we seek.  This routine returns the value of the zParam
** parameter if it exists.  If the parameter does not exist, this routine
** returns a NULL pointer.
**
** If the zFilename argument to this function is not a pointer that SQLite
** passed into the xOpen VFS method, then the behavior of this routine
** is undefined and probably undesirable.
*/
SQLITE_API const char *sqlite3_uri_parameter(const char *zFilename, const char *zParam);


/*
** CAPI3REF: Error Codes And Messages
**
** ^The sqlite3_errcode() interface returns the numeric [result code] or
** [extended result code] for the most recent failed sqlite3_* API call
** associated with a [database connection]. If a prior API call failed
** but the most recent API call succeeded, the return value from
** sqlite3_errcode() is undefined.  ^The sqlite3_extended_errcode()
** interface is the same except that it always returns the 
** [extended result code] even when extended result codes are
** disabled.
**
** ^The sqlite3_errmsg() and sqlite3_errmsg16() return English-language
** text that describes the error, as either UTF-8 or UTF-16 respectively.
** ^(Memory to hold the error message string is managed internally.
** The application does not need to worry about freeing the result.
** However, the error string might be overwritten or deallocated by
** subsequent calls to other SQLite interface functions.)^
**
** When the serialized [threading mode] is in use, it might be the
** case that a second error occurs on a separate thread in between
** the time of the first error and the call to these interfaces.
** When that happens, the second error will be reported since these
** interfaces always report the most recent result.  To avoid
** this, each thread can obtain exclusive use of the [database connection] D
** by invoking [sqlite3_mutex_enter]([sqlite3_db_mutex](D)) before beginning
** to use D and invoking [sqlite3_mutex_leave]([sqlite3_db_mutex](D)) after
** all calls to the interfaces listed here are completed.
**
** If an interface fails with SQLITE_MISUSE, that means the interface
** was invoked incorrectly by the application.  In that case, the
** error code and message may or may not be set.
*/
SQLITE_API int sqlite3_errcode(sqlite3 *db);
SQLITE_API int sqlite3_extended_errcode(sqlite3 *db);
SQLITE_API const char *sqlite3_errmsg(sqlite3*);
SQLITE_API const void *sqlite3_errmsg16(sqlite3*);

/*
** CAPI3REF: SQL Statement Object
** KEYWORDS: {prepared statement} {prepared statements}
**
** An instance of this object represents a single SQL statement.
** This object is variously known as a "prepared statement" or a
** "compiled SQL statement" or simply as a "statement".
**
** The life of a statement object goes something like this:
**
** <ol>
** <li> Create the object using [sqlite3_prepare_v2()] or a related
**      function.
** <li> Bind values to [host parameters] using the sqlite3_bind_*()
**      interfaces.
** <li> Run the SQL by calling [sqlite3_step()] one or more times.
** <li> Reset the statement using [sqlite3_reset()] then go back
**      to step 2.  Do this zero or more times.
** <li> Destroy the object using [sqlite3_finalize()].
** </ol>
**
** Refer to documentation on individual methods above for additional
** information.
*/
typedef struct sqlite3_stmt sqlite3_stmt;

/*
** CAPI3REF: Run-time Limits
**
** ^(This interface allows the size of various constructs to be limited
** on a connection by connection basis.  The first parameter is the
** [database connection] whose limit is to be set or queried.  The
** second parameter is one of the [limit categories] that define a
** class of constructs to be size limited.  The third parameter is the
** new limit for that construct.)^
**
** ^If the new limit is a negative number, the limit is unchanged.
** ^(For each limit category SQLITE_LIMIT_<i>NAME</i> there is a 
** [limits | hard upper bound]
** set at compile-time by a C preprocessor macro called
** [limits | SQLITE_MAX_<i>NAME</i>].
** (The "_LIMIT_" in the name is changed to "_MAX_".))^
** ^Attempts to increase a limit above its hard upper bound are
** silently truncated to the hard upper bound.
**
** ^Regardless of whether or not the limit was changed, the 
** [sqlite3_limit()] interface returns the prior value of the limit.
** ^Hence, to find the current value of a limit without changing it,
** simply invoke this interface with the third parameter set to -1.
**
** Run-time limits are intended for use in applications that manage
** both their own internal database and also databases that are controlled
** by untrusted external sources.  An example application might be a
** web browser that has its own databases for storing history and
** separate databases controlled by JavaScript applications downloaded
** off the Internet.  The internal databases can be given the
** large, default limits.  Databases managed by external sources can
** be given much smaller limits designed to prevent a denial of service
** attack.  Developers might also want to use the [sqlite3_set_authorizer()]
** interface to further control untrusted SQL.  The size of the database
** created by an untrusted script can be contained using the
** [max_page_count] [PRAGMA].
**
** New run-time limit categories may be added in future releases.
*/
SQLITE_API int sqlite3_limit(sqlite3*, int id, int newVal);

/*
** CAPI3REF: Run-Time Limit Categories
** KEYWORDS: {limit category} {*limit categories}
**
** These constants define various performance limits
** that can be lowered at run-time using [sqlite3_limit()].
** The synopsis of the meanings of the various limits is shown below.
** Additional information is available at [limits | Limits in SQLite].
**
** <dl>
** [[SQLITE_LIMIT_LENGTH]] ^(<dt>SQLITE_LIMIT_LENGTH</dt>
** <dd>The maximum size of any string or BLOB or table row, in bytes.<dd>)^
**
** [[SQLITE_LIMIT_SQL_LENGTH]] ^(<dt>SQLITE_LIMIT_SQL_LENGTH</dt>
** <dd>The maximum length of an SQL statement, in bytes.</dd>)^
**
** [[SQLITE_LIMIT_COLUMN]] ^(<dt>SQLITE_LIMIT_COLUMN</dt>
** <dd>The maximum number of columns in a table definition or in the
** result set of a [SELECT] or the maximum number of columns in an index
** or in an ORDER BY or GROUP BY clause.</dd>)^
**
** [[SQLITE_LIMIT_EXPR_DEPTH]] ^(<dt>SQLITE_LIMIT_EXPR_DEPTH</dt>
** <dd>The maximum depth of the parse tree on any expression.</dd>)^
**
** [[SQLITE_LIMIT_COMPOUND_SELECT]] ^(<dt>SQLITE_LIMIT_COMPOUND_SELECT</dt>
** <dd>The maximum number of terms in a compound SELECT statement.</dd>)^
**
** [[SQLITE_LIMIT_VDBE_OP]] ^(<dt>SQLITE_LIMIT_VDBE_OP</dt>
** <dd>The maximum number of instructions in a virtual machine program
** used to implement an SQL statement.  This limit is not currently
** enforced, though that might be added in some future release of
** SQLite.</dd>)^
**
** [[SQLITE_LIMIT_FUNCTION_ARG]] ^(<dt>SQLITE_LIMIT_FUNCTION_ARG</dt>
** <dd>The maximum number of arguments on a function.</dd>)^
**
** [[SQLITE_LIMIT_ATTACHED]] ^(<dt>SQLITE_LIMIT_ATTACHED</dt>
** <dd>The maximum number of [ATTACH | attached databases].)^</dd>
**
** [[SQLITE_LIMIT_LIKE_PATTERN_LENGTH]]
** ^(<dt>SQLITE_LIMIT_LIKE_PATTERN_LENGTH</dt>
** <dd>The maximum length of the pattern argument to the [LIKE] or
** [GLOB] operators.</dd>)^
**
** [[SQLITE_LIMIT_VARIABLE_NUMBER]]
** ^(<dt>SQLITE_LIMIT_VARIABLE_NUMBER</dt>
** <dd>The maximum index number of any [parameter] in an SQL statement.)^
**
** [[SQLITE_LIMIT_TRIGGER_DEPTH]] ^(<dt>SQLITE_LIMIT_TRIGGER_DEPTH</dt>
** <dd>The maximum depth of recursion for triggers.</dd>)^
** </dl>
*/
#define SQLITE_LIMIT_LENGTH                    0
#define SQLITE_LIMIT_SQL_LENGTH                1
#define SQLITE_LIMIT_COLUMN                    2
#define SQLITE_LIMIT_EXPR_DEPTH                3
#define SQLITE_LIMIT_COMPOUND_SELECT           4
#define SQLITE_LIMIT_VDBE_OP                   5
#define SQLITE_LIMIT_FUNCTION_ARG              6
#define SQLITE_LIMIT_ATTACHED                  7
#define SQLITE_LIMIT_LIKE_PATTERN_LENGTH       8
#define SQLITE_LIMIT_VARIABLE_NUMBER           9
#define SQLITE_LIMIT_TRIGGER_DEPTH            10

/*
** CAPI3REF: Compiling An SQL Statement
** KEYWORDS: {SQL statement compiler}
**
** To execute an SQL query, it must first be compiled into a byte-code
** program using one of these routines.
**
** The first argument, "db", is a [database connection] obtained from a
** prior successful call to [sqlite3_open()], [sqlite3_open_v2()] or
** [sqlite3_open16()].  The database connection must not have been closed.
**
** The second argument, "zSql", is the statement to be compiled, encoded
** as either UTF-8 or UTF-16.  The sqlite3_prepare() and sqlite3_prepare_v2()
** interfaces use UTF-8, and sqlite3_prepare16() and sqlite3_prepare16_v2()
** use UTF-16.
**
** ^If the nByte argument is less than zero, then zSql is read up to the
** first zero terminator. ^If nByte is non-negative, then it is the maximum
** number of  bytes read from zSql.  ^When nByte is non-negative, the
** zSql string ends at either the first '\000' or '\u0000' character or
** the nByte-th byte, whichever comes first. If the caller knows
** that the supplied string is nul-terminated, then there is a small
** performance advantage to be gained by passing an nByte parameter that
** is equal to the number of bytes in the input string <i>including</i>
** the nul-terminator bytes as this saves SQLite from having to
** make a copy of the input string.
**
** ^If pzTail is not NULL then *pzTail is made to point to the first byte
** past the end of the first SQL statement in zSql.  These routines only
** compile the first statement in zSql, so *pzTail is left pointing to
** what remains uncompiled.
**
** ^*ppStmt is left pointing to a compiled [prepared statement] that can be
** executed using [sqlite3_step()].  ^If there is an error, *ppStmt is set
** to NULL.  ^If the input text contains no SQL (if the input is an empty
** string or a comment) then *ppStmt is set to NULL.
** The calling procedure is responsible for deleting the compiled
** SQL statement using [sqlite3_finalize()] after it has finished with it.
** ppStmt may not be NULL.
**
** ^On success, the sqlite3_prepare() family of routines return [SQLITE_OK];
** otherwise an [error code] is returned.
**
** The sqlite3_prepare_v2() and sqlite3_prepare16_v2() interfaces are
** recommended for all new programs. The two older interfaces are retained
** for backwards compatibility, but their use is discouraged.
** ^In the "v2" interfaces, the prepared statement
** that is returned (the [sqlite3_stmt] object) contains a copy of the
** original SQL text. This causes the [sqlite3_step()] interface to
** behave differently in three ways:
**
** <ol>
** <li>
** ^If the database schema changes, instead of returning [SQLITE_SCHEMA] as it
** always used to do, [sqlite3_step()] will automatically recompile the SQL
** statement and try to run it again.
** </li>
**
** <li>
** ^When an error occurs, [sqlite3_step()] will return one of the detailed
** [error codes] or [extended error codes].  ^The legacy behavior was that
** [sqlite3_step()] would only return a generic [SQLITE_ERROR] result code
** and the application would have to make a second call to [sqlite3_reset()]
** in order to find the underlying cause of the problem. With the "v2" prepare
** interfaces, the underlying reason for the error is returned immediately.
** </li>
**
** <li>
** ^If the specific value bound to [parameter | host parameter] in the 
** WHERE clause might influence the choice of query plan for a statement,
** then the statement will be automatically recompiled, as if there had been 
** a schema change, on the first  [sqlite3_step()] call following any change
** to the [sqlite3_bind_text | bindings] of that [parameter]. 
** ^The specific value of WHERE-clause [parameter] might influence the 
** choice of query plan if the parameter is the left-hand side of a [LIKE]
** or [GLOB] operator or if the parameter is compared to an indexed column
** and the [SQLITE_ENABLE_STAT3] compile-time option is enabled.
** the 
** </li>
** </ol>
*/
SQLITE_API int sqlite3_prepare(
  sqlite3 *db,            /* Database handle */
  const char *zSql,       /* SQL statement, UTF-8 encoded */
  int nByte,              /* Maximum length of zSql in bytes. */
  sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
  const char **pzTail     /* OUT: Pointer to unused portion of zSql */
);
SQLITE_API int sqlite3_prepare_v2(
  sqlite3 *db,            /* Database handle */
  const char *zSql,       /* SQL statement, UTF-8 encoded */
  int nByte,              /* Maximum length of zSql in bytes. */
  sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
  const char **pzTail     /* OUT: Pointer to unused portion of zSql */
);
SQLITE_API int sqlite3_prepare16(
  sqlite3 *db,            /* Database handle */
  const void *zSql,       /* SQL statement, UTF-16 encoded */
  int nByte,              /* Maximum length of zSql in bytes. */
  sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
  const void **pzTail     /* OUT: Pointer to unused portion of zSql */
);
SQLITE_API int sqlite3_prepare16_v2(
  sqlite3 *db,            /* Database handle */
  const void *zSql,       /* SQL statement, UTF-16 encoded */
  int nByte,              /* Maximum length of zSql in bytes. */
  sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
  const void **pzTail     /* OUT: Pointer to unused portion of zSql */
);

/*
** CAPI3REF: Retrieving Statement SQL
**
** ^This interface can be used to retrieve a saved copy of the original
** SQL text used to create a [prepared statement] if that statement was
** compiled using either [sqlite3_prepare_v2()] or [sqlite3_prepare16_v2()].
*/
SQLITE_API const char *sqlite3_sql(sqlite3_stmt *pStmt);

/*
** CAPI3REF: Determine If An SQL Statement Writes The Database
**
** ^The sqlite3_stmt_readonly(X) interface returns true (non-zero) if
** and only if the [prepared statement] X makes no direct changes to
** the content of the database file.
**
** Note that [application-defined SQL functions] or
** [virtual tables] might change the database indirectly as a side effect.  
** ^(For example, if an application defines a function "eval()" that 
** calls [sqlite3_exec()], then the following SQL statement would
** change the database file through side-effects:
**
** <blockquote><pre>
**    SELECT eval('DELETE FROM t1') FROM t2;
** </pre></blockquote>
**
** But because the [SELECT] statement does not change the database file
** directly, sqlite3_stmt_readonly() would still return true.)^
**
** ^Transaction control statements such as [BEGIN], [COMMIT], [ROLLBACK],
** [SAVEPOINT], and [RELEASE] cause sqlite3_stmt_readonly() to return true,
** since the statements themselves do not actually modify the database but
** rather they control the timing of when other statements modify the 
** database.  ^The [ATTACH] and [DETACH] statements also cause
** sqlite3_stmt_readonly() to return true since, while those statements
** change the configuration of a database connection, they do not make 
** changes to the content of the database files on disk.
*/
SQLITE_API int sqlite3_stmt_readonly(sqlite3_stmt *pStmt);

/*
** CAPI3REF: Dynamically Typed Value Object
** KEYWORDS: {protected sqlite3_value} {unprotected sqlite3_value}
**
** SQLite uses the sqlite3_value object to represent all values
** that can be stored in a database table. SQLite uses dynamic typing
** for the values it stores.  ^Values stored in sqlite3_value objects
** can be integers, floating point values, strings, BLOBs, or NULL.
**
** An sqlite3_value object may be either "protected" or "unprotected".
** Some interfaces require a protected sqlite3_value.  Other interfaces
** will accept either a protected or an unprotected sqlite3_value.
** Every interface that accepts sqlite3_value arguments specifies
** whether or not it requires a protected sqlite3_value.
**
** The terms "protected" and "unprotected" refer to whether or not
** a mutex is held.  An internal mutex is held for a protected
** sqlite3_value object but no mutex is held for an unprotected
** sqlite3_value object.  If SQLite is compiled to be single-threaded
** (with [SQLITE_THREADSAFE=0] and with [sqlite3_threadsafe()] returning 0)
** or if SQLite is run in one of reduced mutex modes 
** [SQLITE_CONFIG_SINGLETHREAD] or [SQLITE_CONFIG_MULTITHREAD]
** then there is no distinction between protected and unprotected
** sqlite3_value objects and they can be used interchangeably.  However,
** for maximum code portability it is recommended that applications
** still make the distinction between protected and unprotected
** sqlite3_value objects even when not strictly required.
**
** ^The sqlite3_value objects that are passed as parameters into the
** implementation of [application-defined SQL functions] are protected.
** ^The sqlite3_value object returned by
** [sqlite3_column_value()] is unprotected.
** Unprotected sqlite3_value objects may only be used with
** [sqlite3_result_value()] and [sqlite3_bind_value()].
** The [sqlite3_value_blob | sqlite3_value_type()] family of
** interfaces require protected sqlite3_value objects.
*/
typedef struct Mem sqlite3_value;

/*
** CAPI3REF: SQL Function Context Object
**
** The context in which an SQL function executes is stored in an
** sqlite3_context object.  ^A pointer to an sqlite3_context object
** is always first parameter to [application-defined SQL functions].
** The application-defined SQL function implementation will pass this
** pointer through into calls to [sqlite3_result_int | sqlite3_result()],
** [sqlite3_aggregate_context()], [sqlite3_user_data()],
** [sqlite3_context_db_handle()], [sqlite3_get_auxdata()],
** and/or [sqlite3_set_auxdata()].
*/
typedef struct sqlite3_context sqlite3_context;

/*
** CAPI3REF: Binding Values To Prepared Statements
** KEYWORDS: {host parameter} {host parameters} {host parameter name}
** KEYWORDS: {SQL parameter} {SQL parameters} {parameter binding}
**
** ^(In the SQL statement text input to [sqlite3_prepare_v2()] and its variants,
** literals may be replaced by a [parameter] that matches one of following
** templates:
**
** <ul>
** <li>  ?
** <li>  ?NNN
** <li>  :VVV
** <li>  @VVV
** <li>  $VVV
** </ul>
**
** In the templates above, NNN represents an integer literal,
** and VVV represents an alphanumeric identifier.)^  ^The values of these
** parameters (also called "host parameter names" or "SQL parameters")
** can be set using the sqlite3_bind_*() routines defined here.
**
** ^The first argument to the sqlite3_bind_*() routines is always
** a pointer to the [sqlite3_stmt] object returned from
** [sqlite3_prepare_v2()] or its variants.
**
** ^The second argument is the index of the SQL parameter to be set.
** ^The leftmost SQL parameter has an index of 1.  ^When the same named
** SQL parameter is used more than once, second and subsequent
** occurrences have the same index as the first occurrence.
** ^The index for named parameters can be looked up using the
** [sqlite3_bind_parameter_index()] API if desired.  ^The index
** for "?NNN" parameters is the value of NNN.
** ^The NNN value must be between 1 and the [sqlite3_limit()]
** parameter [SQLITE_LIMIT_VARIABLE_NUMBER] (default value: 999).
**
** ^The third argument is the value to bind to the parameter.
**
** ^(In those routines that have a fourth argument, its value is the
** number of bytes in the parameter.  To be clear: the value is the
** number of <u>bytes</u> in the value, not the number of characters.)^
** ^If the fourth parameter is negative, the length of the string is
** the number of bytes up to the first zero terminator.
** If a non-negative fourth parameter is provided to sqlite3_bind_text()
** or sqlite3_bind_text16() then that parameter must be the byte offset
** where the NUL terminator would occur assuming the string were NUL
** terminated.  If any NUL characters occur at byte offsets less than 
** the value of the fourth parameter then the resulting string value will
** contain embedded NULs.  The result of expressions involving strings
** with embedded NULs is undefined.
**
** ^The fifth argument to sqlite3_bind_blob(), sqlite3_bind_text(), and
** sqlite3_bind_text16() is a destructor used to dispose of the BLOB or
** string after SQLite has finished with it.  ^The destructor is called
** to dispose of the BLOB or string even if the call to sqlite3_bind_blob(),
** sqlite3_bind_text(), or sqlite3_bind_text16() fails.  
** ^If the fifth argument is
** the special value [SQLITE_STATIC], then SQLite assumes that the
** information is in static, unmanaged space and does not need to be freed.
** ^If the fifth argument has the value [SQLITE_TRANSIENT], then
** SQLite makes its own private copy of the data immediately, before
** the sqlite3_bind_*() routine returns.
**
** ^The sqlite3_bind_zeroblob() routine binds a BLOB of length N that
** is filled with zeroes.  ^A zeroblob uses a fixed amount of memory
** (just an integer to hold its size) while it is being processed.
** Zeroblobs are intended to serve as placeholders for BLOBs whose
** content is later written using
** [sqlite3_blob_open | incremental BLOB I/O] routines.
** ^A negative value for the zeroblob results in a zero-length BLOB.
**
** ^If any of the sqlite3_bind_*() routines are called with a NULL pointer
** for the [prepared statement] or with a prepared statement for which
** [sqlite3_step()] has been called more recently than [sqlite3_reset()],
** then the call will return [SQLITE_MISUSE].  If any sqlite3_bind_()
** routine is passed a [prepared statement] that has been finalized, the
** result is undefined and probably harmful.
**
** ^Bindings are not cleared by the [sqlite3_reset()] routine.
** ^Unbound parameters are interpreted as NULL.
**
** ^The sqlite3_bind_* routines return [SQLITE_OK] on success or an
** [error code] if anything goes wrong.
** ^[SQLITE_RANGE] is returned if the parameter
** index is out of range.  ^[SQLITE_NOMEM] is returned if malloc() fails.
**
** See also: [sqlite3_bind_parameter_count()],
** [sqlite3_bind_parameter_name()], and [sqlite3_bind_parameter_index()].
*/
SQLITE_API int sqlite3_bind_blob(sqlite3_stmt*, int, const void*, int n, void(*)(void*));
SQLITE_API int sqlite3_bind_double(sqlite3_stmt*, int, double);
SQLITE_API int sqlite3_bind_int(sqlite3_stmt*, int, int);
SQLITE_API int sqlite3_bind_int64(sqlite3_stmt*, int, sqlite3_int64);
SQLITE_API int sqlite3_bind_null(sqlite3_stmt*, int);
SQLITE_API int sqlite3_bind_text(sqlite3_stmt*, int, const char*, int n, void(*)(void*));
SQLITE_API int sqlite3_bind_text16(sqlite3_stmt*, int, const void*, int, void(*)(void*));
SQLITE_API int sqlite3_bind_value(sqlite3_stmt*, int, const sqlite3_value*);
SQLITE_API int sqlite3_bind_zeroblob(sqlite3_stmt*, int, int n);

/*
** CAPI3REF: Number Of SQL Parameters
**
** ^This routine can be used to find the number of [SQL parameters]
** in a [prepared statement].  SQL parameters are tokens of the
** form "?", "?NNN", ":AAA", "$AAA", or "@AAA" that serve as
** placeholders for values that are [sqlite3_bind_blob | bound]
** to the parameters at a later time.
**
** ^(This routine actually returns the index of the largest (rightmost)
** parameter. For all forms except ?NNN, this will correspond to the
** number of unique parameters.  If parameters of the ?NNN form are used,
** there may be gaps in the list.)^
**
** See also: [sqlite3_bind_blob|sqlite3_bind()],
** [sqlite3_bind_parameter_name()], and
** [sqlite3_bind_parameter_index()].
*/
SQLITE_API int sqlite3_bind_parameter_count(sqlite3_stmt*);

/*
** CAPI3REF: Name Of A Host Parameter
**
** ^The sqlite3_bind_parameter_name(P,N) interface returns
** the name of the N-th [SQL parameter] in the [prepared statement] P.
** ^(SQL parameters of the form "?NNN" or ":AAA" or "@AAA" or "$AAA"
** have a name which is the string "?NNN" or ":AAA" or "@AAA" or "$AAA"
** respectively.
** In other words, the initial ":" or "$" or "@" or "?"
** is included as part of the name.)^
** ^Parameters of the form "?" without a following integer have no name
** and are referred to as "nameless" or "anonymous parameters".
**
** ^The first host parameter has an index of 1, not 0.
**
** ^If the value N is out of range or if the N-th parameter is
** nameless, then NULL is returned.  ^The returned string is
** always in UTF-8 encoding even if the named parameter was
** originally specified as UTF-16 in [sqlite3_prepare16()] or
** [sqlite3_prepare16_v2()].
**
** See also: [sqlite3_bind_blob|sqlite3_bind()],
** [sqlite3_bind_parameter_count()], and
** [sqlite3_bind_parameter_index()].
*/
SQLITE_API const char *sqlite3_bind_parameter_name(sqlite3_stmt*, int);

/*
** CAPI3REF: Index Of A Parameter With A Given Name
**
** ^Return the index of an SQL parameter given its name.  ^The
** index value returned is suitable for use as the second
** parameter to [sqlite3_bind_blob|sqlite3_bind()].  ^A zero
** is returned if no matching parameter is found.  ^The parameter
** name must be given in UTF-8 even if the original statement
** was prepared from UTF-16 text using [sqlite3_prepare16_v2()].
**
** See also: [sqlite3_bind_blob|sqlite3_bind()],
** [sqlite3_bind_parameter_count()], and
** [sqlite3_bind_parameter_index()].
*/
SQLITE_API int sqlite3_bind_parameter_index(sqlite3_stmt*, const char *zName);

/*
** CAPI3REF: Reset All Bindings On A Prepared Statement
**
** ^Contrary to the intuition of many, [sqlite3_reset()] does not reset
** the [sqlite3_bind_blob | bindings] on a [prepared statement].
** ^Use this routine to reset all host parameters to NULL.
*/
SQLITE_API int sqlite3_clear_bindings(sqlite3_stmt*);

/*
** CAPI3REF: Number Of Columns In A Result Set
**
** ^Return the number of columns in the result set returned by the
** [prepared statement]. ^This routine returns 0 if pStmt is an SQL
** statement that does not return data (for example an [UPDATE]).
**
** See also: [sqlite3_data_count()]
*/
SQLITE_API int sqlite3_column_count(sqlite3_stmt *pStmt);

/*
** CAPI3REF: Column Names In A Result Set
**
** ^These routines return the name assigned to a particular column
** in the result set of a [SELECT] statement.  ^The sqlite3_column_name()
** interface returns a pointer to a zero-terminated UTF-8 string
** and sqlite3_column_name16() returns a pointer to a zero-terminated
** UTF-16 string.  ^The first parameter is the [prepared statement]
** that implements the [SELECT] statement. ^The second parameter is the
** column number.  ^The leftmost column is number 0.
**
** ^The returned string pointer is valid until either the [prepared statement]
** is destroyed by [sqlite3_finalize()] or until the statement is automatically
** reprepared by the first call to [sqlite3_step()] for a particular run
** or until the next call to
** sqlite3_column_name() or sqlite3_column_name16() on the same column.
**
** ^If sqlite3_malloc() fails during the processing of either routine
** (for example during a conversion from UTF-8 to UTF-16) then a
** NULL pointer is returned.
**
** ^The name of a result column is the value of the "AS" clause for
** that column, if there is an AS clause.  If there is no AS clause
** then the name of the column is unspecified and may change from
** one release of SQLite to the next.
*/
SQLITE_API const char *sqlite3_column_name(sqlite3_stmt*, int N);
SQLITE_API const void *sqlite3_column_name16(sqlite3_stmt*, int N);

/*
** CAPI3REF: Source Of Data In A Query Result
**
** ^These routines provide a means to determine the database, table, and
** table column that is the origin of a particular result column in
** [SELECT] statement.
** ^The name of the database or table or column can be returned as
** either a UTF-8 or UTF-16 string.  ^The _database_ routines return
** the database name, the _table_ routines return the table name, and
** the origin_ routines return the column name.
** ^The returned string is valid until the [prepared statement] is destroyed
** using [sqlite3_finalize()] or until the statement is automatically
** reprepared by the first call to [sqlite3_step()] for a particular run
** or until the same information is requested
** again in a different encoding.
**
** ^The names returned are the original un-aliased names of the
** database, table, and column.
**
** ^The first argument to these interfaces is a [prepared statement].
** ^These functions return information about the Nth result column returned by
** the statement, where N is the second function argument.
** ^The left-most column is column 0 for these routines.
**
** ^If the Nth column returned by the statement is an expression or
** subquery and is not a column value, then all of these functions return
** NULL.  ^These routine might also return NULL if a memory allocation error
** occurs.  ^Otherwise, they return the name of the attached database, table,
** or column that query result column was extracted from.
**
** ^As with all other SQLite APIs, those whose names end with "16" return
** UTF-16 encoded strings and the other functions return UTF-8.
**
** ^These APIs are only available if the library was compiled with the
** [SQLITE_ENABLE_COLUMN_METADATA] C-preprocessor symbol.
**
** If two or more threads call one or more of these routines against the same
** prepared statement and column at the same time then the results are
** undefined.
**
** If two or more threads call one or more
** [sqlite3_column_database_name | column metadata interfaces]
** for the same [prepared statement] and result column
** at the same time then the results are undefined.
*/
SQLITE_API const char *sqlite3_column_database_name(sqlite3_stmt*,int);
SQLITE_API const void *sqlite3_column_database_name16(sqlite3_stmt*,int);
SQLITE_API const char *sqlite3_column_table_name(sqlite3_stmt*,int);
SQLITE_API const void *sqlite3_column_table_name16(sqlite3_stmt*,int);
SQLITE_API const char *sqlite3_column_origin_name(sqlite3_stmt*,int);
SQLITE_API const void *sqlite3_column_origin_name16(sqlite3_stmt*,int);

/*
** CAPI3REF: Declared Datatype Of A Query Result
**
** ^(The first parameter is a [prepared statement].
** If this statement is a [SELECT] statement and the Nth column of the
** returned result set of that [SELECT] is a table column (not an
** expression or subquery) then the declared type of the table
** column is returned.)^  ^If the Nth column of the result set is an
** expression or subquery, then a NULL pointer is returned.
** ^The returned string is always UTF-8 encoded.
**
** ^(For example, given the database schema:
**
** CREATE TABLE t1(c1 VARIANT);
**
** and the following statement to be compiled:
**
** SELECT c1 + 1, c1 FROM t1;
**
** this routine would return the string "VARIANT" for the second result
** column (i==1), and a NULL pointer for the first result column (i==0).)^
**
** ^SQLite uses dynamic run-time typing.  ^So just because a column
** is declared to contain a particular type does not mean that the
** data stored in that column is of the declared type.  SQLite is
** strongly typed, but the typing is dynamic not static.  ^Type
** is associated with individual values, not with the containers
** used to hold those values.
*/
SQLITE_API const char *sqlite3_column_decltype(sqlite3_stmt*,int);
SQLITE_API const void *sqlite3_column_decltype16(sqlite3_stmt*,int);

/*
** CAPI3REF: Evaluate An SQL Statement
**
** After a [prepared statement] has been prepared using either
** [sqlite3_prepare_v2()] or [sqlite3_prepare16_v2()] or one of the legacy
** interfaces [sqlite3_prepare()] or [sqlite3_prepare16()], this function
** must be called one or more times to evaluate the statement.
**
** The details of the behavior of the sqlite3_step() interface depend
** on whether the statement was prepared using the newer "v2" interface
** [sqlite3_prepare_v2()] and [sqlite3_prepare16_v2()] or the older legacy
** interface [sqlite3_prepare()] and [sqlite3_prepare16()].  The use of the
** new "v2" interface is recommended for new applications but the legacy
** interface will continue to be supported.
**
** ^In the legacy interface, the return value will be either [SQLITE_BUSY],
** [SQLITE_DONE], [SQLITE_ROW], [SQLITE_ERROR], or [SQLITE_MISUSE].
** ^With the "v2" interface, any of the other [result codes] or
** [extended result codes] might be returned as well.
**
** ^[SQLITE_BUSY] means that the database engine was unable to acquire the
** database locks it needs to do its job.  ^If the statement is a [COMMIT]
** or occurs outside of an explicit transaction, then you can retry the
** statement.  If the statement is not a [COMMIT] and occurs within an
** explicit transaction then you should rollback the transaction before
** continuing.
**
** ^[SQLITE_DONE] means that the statement has finished executing
** successfully.  sqlite3_step() should not be called again on this virtual
** machine without first calling [sqlite3_reset()] to reset the virtual
** machine back to its initial state.
**
** ^If the SQL statement being executed returns any data, then [SQLITE_ROW]
** is returned each time a new row of data is ready for processing by the
** caller. The values may be accessed using the [column access functions].
** sqlite3_step() is called again to retrieve the next row of data.
**
** ^[SQLITE_ERROR] means that a run-time error (such as a constraint
** violation) has occurred.  sqlite3_step() should not be called again on
** the VM. More information may be found by calling [sqlite3_errmsg()].
** ^With the legacy interface, a more specific error code (for example,
** [SQLITE_INTERRUPT], [SQLITE_SCHEMA], [SQLITE_CORRUPT], and so forth)
** can be obtained by calling [sqlite3_reset()] on the
** [prepared statement].  ^In the "v2" interface,
** the more specific error code is returned directly by sqlite3_step().
**
** [SQLITE_MISUSE] means that the this routine was called inappropriately.
** Perhaps it was called on a [prepared statement] that has
** already been [sqlite3_finalize | finalized] or on one that had
** previously returned [SQLITE_ERROR] or [SQLITE_DONE].  Or it could
** be the case that the same database connection is being used by two or
** more threads at the same moment in time.
**
** For all versions of SQLite up to and including 3.6.23.1, a call to
** [sqlite3_reset()] was required after sqlite3_step() returned anything
** other than [SQLITE_ROW] before any subsequent invocation of
** sqlite3_step().  Failure to reset the prepared statement using 
** [sqlite3_reset()] would result in an [SQLITE_MISUSE] return from
** sqlite3_step().  But after version 3.6.23.1, sqlite3_step() began
** calling [sqlite3_reset()] automatically in this circumstance rather
** than returning [SQLITE_MISUSE].  This is not considered a compatibility
** break because any application that ever receives an SQLITE_MISUSE error
** is broken by definition.  The [SQLITE_OMIT_AUTORESET] compile-time option
** can be used to restore the legacy behavior.
**
** <b>Goofy Interface Alert:</b> In the legacy interface, the sqlite3_step()
** API always returns a generic error code, [SQLITE_ERROR], following any
** error other than [SQLITE_BUSY] and [SQLITE_MISUSE].  You must call
** [sqlite3_reset()] or [sqlite3_finalize()] in order to find one of the
** specific [error codes] that better describes the error.
** We admit that this is a goofy design.  The problem has been fixed
** with the "v2" interface.  If you prepare all of your SQL statements
** using either [sqlite3_prepare_v2()] or [sqlite3_prepare16_v2()] instead
** of the legacy [sqlite3_prepare()] and [sqlite3_prepare16()] interfaces,
** then the more specific [error codes] are returned directly
** by sqlite3_step().  The use of the "v2" interface is recommended.
*/
SQLITE_API int sqlite3_step(sqlite3_stmt*);

/*
** CAPI3REF: Number of columns in a result set
**
** ^The sqlite3_data_count(P) interface returns the number of columns in the
** current row of the result set of [prepared statement] P.
** ^If prepared statement P does not have results ready to return
** (via calls to the [sqlite3_column_int | sqlite3_column_*()] of
** interfaces) then sqlite3_data_count(P) returns 0.
** ^The sqlite3_data_count(P) routine also returns 0 if P is a NULL pointer.
** ^The sqlite3_data_count(P) routine returns 0 if the previous call to
** [sqlite3_step](P) returned [SQLITE_DONE].  ^The sqlite3_data_count(P)
** will return non-zero if previous call to [sqlite3_step](P) returned
** [SQLITE_ROW], except in the case of the [PRAGMA incremental_vacuum]
** where it always returns zero since each step of that multi-step
** pragma returns 0 columns of data.
**
** See also: [sqlite3_column_count()]
*/
SQLITE_API int sqlite3_data_count(sqlite3_stmt *pStmt);

/*
** CAPI3REF: Fundamental Datatypes
** KEYWORDS: SQLITE_TEXT
**
** ^(Every value in SQLite has one of five fundamental datatypes:
**
** <ul>
** <li> 64-bit signed integer
** <li> 64-bit IEEE floating point number
** <li> string
** <li> BLOB
** <li> NULL
** </ul>)^
**
** These constants are codes for each of those types.
**
** Note that the SQLITE_TEXT constant was also used in SQLite version 2
** for a completely different meaning.  Software that links against both
** SQLite version 2 and SQLite version 3 should use SQLITE3_TEXT, not
** SQLITE_TEXT.
*/
#define SQLITE_INTEGER  1
#define SQLITE_FLOAT    2
#define SQLITE_BLOB     4
#define SQLITE_NULL     5
#ifdef SQLITE_TEXT
# undef SQLITE_TEXT
#else
# define SQLITE_TEXT     3
#endif
#define SQLITE3_TEXT     3

/*
** CAPI3REF: Result Values From A Query
** KEYWORDS: {column access functions}
**
** These routines form the "result set" interface.
**
** ^These routines return information about a single column of the current
** result row of a query.  ^In every case the first argument is a pointer
** to the [prepared statement] that is being evaluated (the [sqlite3_stmt*]
** that was returned from [sqlite3_prepare_v2()] or one of its variants)
** and the second argument is the index of the column for which information
** should be returned. ^The leftmost column of the result set has the index 0.
** ^The number of columns in the result can be determined using
** [sqlite3_column_count()].
**
** If the SQL statement does not currently point to a valid row, or if the
** column index is out of range, the result is undefined.
** These routines may only be called when the most recent call to
** [sqlite3_step()] has returned [SQLITE_ROW] and neither
** [sqlite3_reset()] nor [sqlite3_finalize()] have been called subsequently.
** If any of these routines are called after [sqlite3_reset()] or
** [sqlite3_finalize()] or after [sqlite3_step()] has returned
** something other than [SQLITE_ROW], the results are undefined.
** If [sqlite3_step()] or [sqlite3_reset()] or [sqlite3_finalize()]
** are called from a different thread while any of these routines
** are pending, then the results are undefined.
**
** ^The sqlite3_column_type() routine returns the
** [SQLITE_INTEGER | datatype code] for the initial data type
** of the result column.  ^The returned value is one of [SQLITE_INTEGER],
** [SQLITE_FLOAT], [SQLITE_TEXT], [SQLITE_BLOB], or [SQLITE_NULL].  The value
** returned by sqlite3_column_type() is only meaningful if no type
** conversions have occurred as described below.  After a type conversion,
** the value returned by sqlite3_column_type() is undefined.  Future
** versions of SQLite may change the behavior of sqlite3_column_type()
** following a type conversion.
**
** ^If the result is a BLOB or UTF-8 string then the sqlite3_column_bytes()
** routine returns the number of bytes in that BLOB or string.
** ^If the result is a UTF-16 string, then sqlite3_column_bytes() converts
** the string to UTF-8 and then returns the number of bytes.
** ^If the result is a numeric value then sqlite3_column_bytes() uses
** [sqlite3_snprintf()] to convert that value to a UTF-8 string and returns
** the number of bytes in that string.
** ^If the result is NULL, then sqlite3_column_bytes() returns zero.
**
** ^If the result is a BLOB or UTF-16 string then the sqlite3_column_bytes16()
** routine returns the number of bytes in that BLOB or string.
** ^If the result is a UTF-8 string, then sqlite3_column_bytes16() converts
** the string to UTF-16 and then returns the number of bytes.
** ^If the result is a numeric value then sqlite3_column_bytes16() uses
** [sqlite3_snprintf()] to convert that value to a UTF-16 string and returns
** the number of bytes in that string.
** ^If the result is NULL, then sqlite3_column_bytes16() returns zero.
**
** ^The values returned by [sqlite3_column_bytes()] and 
** [sqlite3_column_bytes16()] do not include the zero terminators at the end
** of the string.  ^For clarity: the values returned by
** [sqlite3_column_bytes()] and [sqlite3_column_bytes16()] are the number of
** bytes in the string, not the number of characters.
**
** ^Strings returned by sqlite3_column_text() and sqlite3_column_text16(),
** even empty strings, are always zero terminated.  ^The return
** value from sqlite3_column_blob() for a zero-length BLOB is a NULL pointer.
**
** ^The object returned by [sqlite3_column_value()] is an
** [unprotected sqlite3_value] object.  An unprotected sqlite3_value object
** may only be used with [sqlite3_bind_value()] and [sqlite3_result_value()].
** If the [unprotected sqlite3_value] object returned by
** [sqlite3_column_value()] is used in any other way, including calls
** to routines like [sqlite3_value_int()], [sqlite3_value_text()],
** or [sqlite3_value_bytes()], then the behavior is undefined.
**
** These routines attempt to convert the value where appropriate.  ^For
** example, if the internal representation is FLOAT and a text result
** is requested, [sqlite3_snprintf()] is used internally to perform the
** conversion automatically.  ^(The following table details the conversions
** that are applied:
**
** <blockquote>
** <table border="1">
** <tr><th> Internal<br>Type <th> Requested<br>Type <th>  Conversion
**
** <tr><td>  NULL    <td> INTEGER   <td> Result is 0
** <tr><td>  NULL    <td>  FLOAT    <td> Result is 0.0
** <tr><td>  NULL    <td>   TEXT    <td> Result is NULL pointer
** <tr><td>  NULL    <td>   BLOB    <td> Result is NULL pointer
** <tr><td> INTEGER  <td>  FLOAT    <td> Convert from integer to float
** <tr><td> INTEGER  <td>   TEXT    <td> ASCII rendering of the integer
** <tr><td> INTEGER  <td>   BLOB    <td> Same as INTEGER->TEXT
** <tr><td>  FLOAT   <td> INTEGER   <td> Convert from float to integer
** <tr><td>  FLOAT   <td>   TEXT    <td> ASCII rendering of the float
** <tr><td>  FLOAT   <td>   BLOB    <td> Same as FLOAT->TEXT
** <tr><td>  TEXT    <td> INTEGER   <td> Use atoi()
** <tr><td>  TEXT    <td>  FLOAT    <td> Use atof()
** <tr><td>  TEXT    <td>   BLOB    <td> No change
** <tr><td>  BLOB    <td> INTEGER   <td> Convert to TEXT then use atoi()
** <tr><td>  BLOB    <td>  FLOAT    <td> Convert to TEXT then use atof()
** <tr><td>  BLOB    <td>   TEXT    <td> Add a zero terminator if needed
** </table>
** </blockquote>)^
**
** The table above makes reference to standard C library functions atoi()
** and atof().  SQLite does not really use these functions.  It has its
** own equivalent internal routines.  The atoi() and atof() names are
** used in the table for brevity and because they are familiar to most
** C programmers.
**
** Note that when type conversions occur, pointers returned by prior
** calls to sqlite3_column_blob(), sqlite3_column_text(), and/or
** sqlite3_column_text16() may be invalidated.
** Type conversions and pointer invalidations might occur
** in the following cases:
**
** <ul>
** <li> The initial content is a BLOB and sqlite3_column_text() or
**      sqlite3_column_text16() is called.  A zero-terminator might
**      need to be added to the string.</li>
** <li> The initial content is UTF-8 text and sqlite3_column_bytes16() or
**      sqlite3_column_text16() is called.  The content must be converted
**      to UTF-16.</li>
** <li> The initial content is UTF-16 text and sqlite3_column_bytes() or
**      sqlite3_column_text() is called.  The content must be converted
**      to UTF-8.</li>
** </ul>
**
** ^Conversions between UTF-16be and UTF-16le are always done in place and do
** not invalidate a prior pointer, though of course the content of the buffer
** that the prior pointer references will have been modified.  Other kinds
** of conversion are done in place when it is possible, but sometimes they
** are not possible and in those cases prior pointers are invalidated.
**
** The safest and easiest to remember policy is to invoke these routines
** in one of the following ways:
**
** <ul>
**  <li>sqlite3_column_text() followed by sqlite3_column_bytes()</li>
**  <li>sqlite3_column_blob() followed by sqlite3_column_bytes()</li>
**  <li>sqlite3_column_text16() followed by sqlite3_column_bytes16()</li>
** </ul>
**
** In other words, you should call sqlite3_column_text(),
** sqlite3_column_blob(), or sqlite3_column_text16() first to force the result
** into the desired format, then invoke sqlite3_column_bytes() or
** sqlite3_column_bytes16() to find the size of the result.  Do not mix calls
** to sqlite3_column_text() or sqlite3_column_blob() with calls to
** sqlite3_column_bytes16(), and do not mix calls to sqlite3_column_text16()
** with calls to sqlite3_column_bytes().
**
** ^The pointers returned are valid until a type conversion occurs as
** described above, or until [sqlite3_step()] or [sqlite3_reset()] or
** [sqlite3_finalize()] is called.  ^The memory space used to hold strings
** and BLOBs is freed automatically.  Do <b>not</b> pass the pointers returned
** [sqlite3_column_blob()], [sqlite3_column_text()], etc. into
** [sqlite3_free()].
**
** ^(If a memory allocation error occurs during the evaluation of any
** of these routines, a default value is returned.  The default value
** is either the integer 0, the floating point number 0.0, or a NULL
** pointer.  Subsequent calls to [sqlite3_errcode()] will return
** [SQLITE_NOMEM].)^
*/
SQLITE_API const void *sqlite3_column_blob(sqlite3_stmt*, int iCol);
SQLITE_API int sqlite3_column_bytes(sqlite3_stmt*, int iCol);
SQLITE_API int sqlite3_column_bytes16(sqlite3_stmt*, int iCol);
SQLITE_API double sqlite3_column_double(sqlite3_stmt*, int iCol);
SQLITE_API int sqlite3_column_int(sqlite3_stmt*, int iCol);
SQLITE_API sqlite3_int64 sqlite3_column_int64(sqlite3_stmt*, int iCol);
SQLITE_API const unsigned char *sqlite3_column_text(sqlite3_stmt*, int iCol);
SQLITE_API const void *sqlite3_column_text16(sqlite3_stmt*, int iCol);
SQLITE_API int sqlite3_column_type(sqlite3_stmt*, int iCol);
SQLITE_API sqlite3_value *sqlite3_column_value(sqlite3_stmt*, int iCol);

/*
** CAPI3REF: Destroy A Prepared Statement Object
**
** ^The sqlite3_finalize() function is called to delete a [prepared statement].
** ^If the most recent evaluation of the statement encountered no errors
** or if the statement is never been evaluated, then sqlite3_finalize() returns
** SQLITE_OK.  ^If the most recent evaluation of statement S failed, then
** sqlite3_finalize(S) returns the appropriate [error code] or
** [extended error code].
**
** ^The sqlite3_finalize(S) routine can be called at any point during
** the life cycle of [prepared statement] S:
** before statement S is ever evaluated, after
** one or more calls to [sqlite3_reset()], or after any call
** to [sqlite3_step()] regardless of whether or not the statement has
** completed execution.
**
** ^Invoking sqlite3_finalize() on a NULL pointer is a harmless no-op.
**
** The application must finalize every [prepared statement] in order to avoid
** resource leaks.  It is a grievous error for the application to try to use
** a prepared statement after it has been finalized.  Any use of a prepared
** statement after it has been finalized can result in undefined and
** undesirable behavior such as segfaults and heap corruption.
*/
SQLITE_API int sqlite3_finalize(sqlite3_stmt *pStmt);

/*
** CAPI3REF: Reset A Prepared Statement Object
**
** The sqlite3_reset() function is called to reset a [prepared statement]
** object back to its initial state, ready to be re-executed.
** ^Any SQL statement variables that had values bound to them using
** the [sqlite3_bind_blob | sqlite3_bind_*() API] retain their values.
** Use [sqlite3_clear_bindings()] to reset the bindings.
**
** ^The [sqlite3_reset(S)] interface resets the [prepared statement] S
** back to the beginning of its program.
**
** ^If the most recent call to [sqlite3_step(S)] for the
** [prepared statement] S returned [SQLITE_ROW] or [SQLITE_DONE],
** or if [sqlite3_step(S)] has never before been called on S,
** then [sqlite3_reset(S)] returns [SQLITE_OK].
**
** ^If the most recent call to [sqlite3_step(S)] for the
** [prepared statement] S indicated an error, then
** [sqlite3_reset(S)] returns an appropriate [error code].
**
** ^The [sqlite3_reset(S)] interface does not change the values
** of any [sqlite3_bind_blob|bindings] on the [prepared statement] S.
*/
SQLITE_API int sqlite3_reset(sqlite3_stmt *pStmt);

/*
** CAPI3REF: Create Or Redefine SQL Functions
** KEYWORDS: {function creation routines}
** KEYWORDS: {application-defined SQL function}
** KEYWORDS: {application-defined SQL functions}
**
** ^These functions (collectively known as "function creation routines")
** are used to add SQL functions or aggregates or to redefine the behavior
** of existing SQL functions or aggregates.  The only differences between
** these routines are the text encoding expected for
** the second parameter (the name of the function being created)
** and the presence or absence of a destructor callback for
** the application data pointer.
**
** ^The first parameter is the [database connection] to which the SQL
** function is to be added.  ^If an application uses more than one database
** connection then application-defined SQL functions must be added
** to each database connection separately.
**
** ^The second parameter is the name of the SQL function to be created or
** redefined.  ^The length of the name is limited to 255 bytes in a UTF-8
** representation, exclusive of the zero-terminator.  ^Note that the name
** length limit is in UTF-8 bytes, not characters nor UTF-16 bytes.  
** ^Any attempt to create a function with a longer name
** will result in [SQLITE_MISUSE] being returned.
**
** ^The third parameter (nArg)
** is the number of arguments that the SQL function or
** aggregate takes. ^If this parameter is -1, then the SQL function or
** aggregate may take any number of arguments between 0 and the limit
** set by [sqlite3_limit]([SQLITE_LIMIT_FUNCTION_ARG]).  If the third
** parameter is less than -1 or greater than 127 then the behavior is
** undefined.
**
** ^The fourth parameter, eTextRep, specifies what
** [SQLITE_UTF8 | text encoding] this SQL function prefers for
** its parameters.  Every SQL function implementation must be able to work
** with UTF-8, UTF-16le, or UTF-16be.  But some implementations may be
** more efficient with one encoding than another.  ^An application may
** invoke sqlite3_create_function() or sqlite3_create_function16() multiple
** times with the same function but with different values of eTextRep.
** ^When multiple implementations of the same function are available, SQLite
** will pick the one that involves the least amount of data conversion.
** If there is only a single implementation which does not care what text
** encoding is used, then the fourth argument should be [SQLITE_ANY].
**
** ^(The fifth parameter is an arbitrary pointer.  The implementation of the
** function can gain access to this pointer using [sqlite3_user_data()].)^
**
** ^The sixth, seventh and eighth parameters, xFunc, xStep and xFinal, are
** pointers to C-language functions that implement the SQL function or
** aggregate. ^A scalar SQL function requires an implementation of the xFunc
** callback only; NULL pointers must be passed as the xStep and xFinal
** parameters. ^An aggregate SQL function requires an implementation of xStep
** and xFinal and NULL pointer must be passed for xFunc. ^To delete an existing
** SQL function or aggregate, pass NULL pointers for all three function
** callbacks.
**
** ^(If the ninth parameter to sqlite3_create_function_v2() is not NULL,
** then it is destructor for the application data pointer. 
** The destructor is invoked when the function is deleted, either by being
** overloaded or when the database connection closes.)^
** ^The destructor is also invoked if the call to
** sqlite3_create_function_v2() fails.
** ^When the destructor callback of the tenth parameter is invoked, it
** is passed a single argument which is a copy of the application data 
** pointer which was the fifth parameter to sqlite3_create_function_v2().
**
** ^It is permitted to register multiple implementations of the same
** functions with the same name but with either differing numbers of
** arguments or differing preferred text encodings.  ^SQLite will use
** the implementation that most closely matches the way in which the
** SQL function is used.  ^A function implementation with a non-negative
** nArg parameter is a better match than a function implementation with
** a negative nArg.  ^A function where the preferred text encoding
** matches the database encoding is a better
** match than a function where the encoding is different.  
** ^A function where the encoding difference is between UTF16le and UTF16be
** is a closer match than a function where the encoding difference is
** between UTF8 and UTF16.
**
** ^Built-in functions may be overloaded by new application-defined functions.
**
** ^An application-defined function is permitted to call other
** SQLite interfaces.  However, such calls must not
** close the database connection nor finalize or reset the prepared
** statement in which the function is running.
*/
SQLITE_API int sqlite3_create_function(
  sqlite3 *db,
  const char *zFunctionName,
  int nArg,
  int eTextRep,
  void *pApp,
  void (*xFunc)(sqlite3_context*,int,sqlite3_value**),
  void (*xStep)(sqlite3_context*,int,sqlite3_value**),
  void (*xFinal)(sqlite3_context*)
);
SQLITE_API int sqlite3_create_function16(
  sqlite3 *db,
  const void *zFunctionName,
  int nArg,
  int eTextRep,
  void *pApp,
  void (*xFunc)(sqlite3_context*,int,sqlite3_value**),
  void (*xStep)(sqlite3_context*,int,sqlite3_value**),
  void (*xFinal)(sqlite3_context*)
);
SQLITE_API int sqlite3_create_function_v2(
  sqlite3 *db,
  const char *zFunctionName,
  int nArg,
  int eTextRep,
  void *pApp,
  void (*xFunc)(sqlite3_context*,int,sqlite3_value**),
  void (*xStep)(sqlite3_context*,int,sqlite3_value**),
  void (*xFinal)(sqlite3_context*),
  void(*xDestroy)(void*)
);

/*
** CAPI3REF: Text Encodings
**
** These constant define integer codes that represent the various
** text encodings supported by SQLite.
*/
#define SQLITE_UTF8           1
#define SQLITE_UTF16LE        2
#define SQLITE_UTF16BE        3
#define SQLITE_UTF16          4    /* Use native byte order */
#define SQLITE_ANY            5    /* sqlite3_create_function only */
#define SQLITE_UTF16_ALIGNED  8    /* sqlite3_create_collation only */

/*
** CAPI3REF: Deprecated Functions
** DEPRECATED
**
** These functions are [deprecated].  In order to maintain
** backwards compatibility with older code, these functions continue 
** to be supported.  However, new applications should avoid
** the use of these functions.  To help encourage people to avoid
** using these functions, we are not going to tell you what they do.
*/
#ifndef SQLITE_OMIT_DEPRECATED
SQLITE_API SQLITE_DEPRECATED int sqlite3_aggregate_count(sqlite3_context*);
SQLITE_API SQLITE_DEPRECATED int sqlite3_expired(sqlite3_stmt*);
SQLITE_API SQLITE_DEPRECATED int sqlite3_transfer_bindings(sqlite3_stmt*, sqlite3_stmt*);
SQLITE_API SQLITE_DEPRECATED int sqlite3_global_recover(void);
SQLITE_API SQLITE_DEPRECATED void sqlite3_thread_cleanup(void);
SQLITE_API SQLITE_DEPRECATED int sqlite3_memory_alarm(void(*)(void*,sqlite3_int64,int),void*,sqlite3_int64);
#endif

/*
** CAPI3REF: Obtaining SQL Function Parameter Values
**
** The C-language implementation of SQL functions and aggregates uses
** this set of interface routines to access the parameter values on
** the function or aggregate.
**
** The xFunc (for scalar functions) or xStep (for aggregates) parameters
** to [sqlite3_create_function()] and [sqlite3_create_function16()]
** define callbacks that implement the SQL functions and aggregates.
** The 3rd parameter to these callbacks is an array of pointers to
** [protected sqlite3_value] objects.  There is one [sqlite3_value] object for
** each parameter to the SQL function.  These routines are used to
** extract values from the [sqlite3_value] objects.
**
** These routines work only with [protected sqlite3_value] objects.
** Any attempt to use these routines on an [unprotected sqlite3_value]
** object results in undefined behavior.
**
** ^These routines work just like the corresponding [column access functions]
** except that  these routines take a single [protected sqlite3_value] object
** pointer instead of a [sqlite3_stmt*] pointer and an integer column number.
**
** ^The sqlite3_value_text16() interface extracts a UTF-16 string
** in the native byte-order of the host machine.  ^The
** sqlite3_value_text16be() and sqlite3_value_text16le() interfaces
** extract UTF-16 strings as big-endian and little-endian respectively.
**
** ^(The sqlite3_value_numeric_type() interface attempts to apply
** numeric affinity to the value.  This means that an attempt is
** made to convert the value to an integer or floating point.  If
** such a conversion is possible without loss of information (in other
** words, if the value is a string that looks like a number)
** then the conversion is performed.  Otherwise no conversion occurs.
** The [SQLITE_INTEGER | datatype] after conversion is returned.)^
**
** Please pay particular attention to the fact that the pointer returned
** from [sqlite3_value_blob()], [sqlite3_value_text()], or
** [sqlite3_value_text16()] can be invalidated by a subsequent call to
** [sqlite3_value_bytes()], [sqlite3_value_bytes16()], [sqlite3_value_text()],
** or [sqlite3_value_text16()].
**
** These routines must be called from the same thread as
** the SQL function that supplied the [sqlite3_value*] parameters.
*/
SQLITE_API const void *sqlite3_value_blob(sqlite3_value*);
SQLITE_API int sqlite3_value_bytes(sqlite3_value*);
SQLITE_API int sqlite3_value_bytes16(sqlite3_value*);
SQLITE_API double sqlite3_value_double(sqlite3_value*);
SQLITE_API int sqlite3_value_int(sqlite3_value*);
SQLITE_API sqlite3_int64 sqlite3_value_int64(sqlite3_value*);
SQLITE_API const unsigned char *sqlite3_value_text(sqlite3_value*);
SQLITE_API const void *sqlite3_value_text16(sqlite3_value*);
SQLITE_API const void *sqlite3_value_text16le(sqlite3_value*);
SQLITE_API const void *sqlite3_value_text16be(sqlite3_value*);
SQLITE_API int sqlite3_value_type(sqlite3_value*);
SQLITE_API int sqlite3_value_numeric_type(sqlite3_value*);

/*
** CAPI3REF: Obtain Aggregate Function Context
**
** Implementations of aggregate SQL functions use this
** routine to allocate memory for storing their state.
**
** ^The first time the sqlite3_aggregate_context(C,N) routine is called 
** for a particular aggregate function, SQLite
** allocates N of memory, zeroes out that memory, and returns a pointer
** to the new memory. ^On second and subsequent calls to
** sqlite3_aggregate_context() for the same aggregate function instance,
** the same buffer is returned.  Sqlite3_aggregate_context() is normally
** called once for each invocation of the xStep callback and then one
** last time when the xFinal callback is invoked.  ^(When no rows match
** an aggregate query, the xStep() callback of the aggregate function
** implementation is never called and xFinal() is called exactly once.
** In those cases, sqlite3_aggregate_context() might be called for the
** first time from within xFinal().)^
**
** ^The sqlite3_aggregate_context(C,N) routine returns a NULL pointer if N is
** less than or equal to zero or if a memory allocate error occurs.
**
** ^(The amount of space allocated by sqlite3_aggregate_context(C,N) is
** determined by the N parameter on first successful call.  Changing the
** value of N in subsequent call to sqlite3_aggregate_context() within
** the same aggregate function instance will not resize the memory
** allocation.)^
**
** ^SQLite automatically frees the memory allocated by 
** sqlite3_aggregate_context() when the aggregate query concludes.
**
** The first parameter must be a copy of the
** [sqlite3_context | SQL function context] that is the first parameter
** to the xStep or xFinal callback routine that implements the aggregate
** function.
**
** This routine must be called from the same thread in which
** the aggregate SQL function is running.
*/
SQLITE_API void *sqlite3_aggregate_context(sqlite3_context*, int nBytes);

/*
** CAPI3REF: User Data For Functions
**
** ^The sqlite3_user_data() interface returns a copy of
** the pointer that was the pUserData parameter (the 5th parameter)
** of the [sqlite3_create_function()]
** and [sqlite3_create_function16()] routines that originally
** registered the application defined function.
**
** This routine must be called from the same thread in which
** the application-defined function is running.
*/
SQLITE_API void *sqlite3_user_data(sqlite3_context*);

/*
** CAPI3REF: Database Connection For Functions
**
** ^The sqlite3_context_db_handle() interface returns a copy of
** the pointer to the [database connection] (the 1st parameter)
** of the [sqlite3_create_function()]
** and [sqlite3_create_function16()] routines that originally
** registered the application defined function.
*/
SQLITE_API sqlite3 *sqlite3_context_db_handle(sqlite3_context*);

/*
** CAPI3REF: Function Auxiliary Data
**
** The following two functions may be used by scalar SQL functions to
** associate metadata with argument values. If the same value is passed to
** multiple invocations of the same SQL function during query execution, under
** some circumstances the associated metadata may be preserved. This may
** be used, for example, to add a regular-expression matching scalar
** function. The compiled version of the regular expression is stored as
** metadata associated with the SQL value passed as the regular expression
** pattern.  The compiled regular expression can be reused on multiple
** invocations of the same function so that the original pattern string
** does not need to be recompiled on each invocation.
**
** ^The sqlite3_get_auxdata() interface returns a pointer to the metadata
** associated by the sqlite3_set_auxdata() function with the Nth argument
** value to the application-defined function. ^If no metadata has been ever
** been set for the Nth argument of the function, or if the corresponding
** function parameter has changed since the meta-data was set,
** then sqlite3_get_auxdata() returns a NULL pointer.
**
** ^The sqlite3_set_auxdata() interface saves the metadata
** pointed to by its 3rd parameter as the metadata for the N-th
** argument of the application-defined function.  Subsequent
** calls to sqlite3_get_auxdata() might return this data, if it has
** not been destroyed.
** ^If it is not NULL, SQLite will invoke the destructor
** function given by the 4th parameter to sqlite3_set_auxdata() on
** the metadata when the corresponding function parameter changes
** or when the SQL statement completes, whichever comes first.
**
** SQLite is free to call the destructor and drop metadata on any
** parameter of any function at any time.  ^The only guarantee is that
** the destructor will be called before the metadata is dropped.
**
** ^(In practice, metadata is preserved between function calls for
** expressions that are constant at compile time. This includes literal
** values and [parameters].)^
**
** These routines must be called from the same thread in which
** the SQL function is running.
*/
SQLITE_API void *sqlite3_get_auxdata(sqlite3_context*, int N);
SQLITE_API void sqlite3_set_auxdata(sqlite3_context*, int N, void*, void (*)(void*));


/*
** CAPI3REF: Constants Defining Special Destructor Behavior
**
** These are special values for the destructor that is passed in as the
** final argument to routines like [sqlite3_result_blob()].  ^If the destructor
** argument is SQLITE_STATIC, it means that the content pointer is constant
** and will never change.  It does not need to be destroyed.  ^The
** SQLITE_TRANSIENT value means that the content will likely change in
** the near future and that SQLite should make its own private copy of
** the content before returning.
**
** The typedef is necessary to work around problems in certain
** C++ compilers.  See ticket #2191.
*/
typedef void (*sqlite3_destructor_type)(void*);
#define SQLITE_STATIC      ((sqlite3_destructor_type)0)
#define SQLITE_TRANSIENT   ((sqlite3_destructor_type)-1)

/*
** CAPI3REF: Setting The Result Of An SQL Function
**
** These routines are used by the xFunc or xFinal callbacks that
** implement SQL functions and aggregates.  See
** [sqlite3_create_function()] and [sqlite3_create_function16()]
** for additional information.
**
** These functions work very much like the [parameter binding] family of
** functions used to bind values to host parameters in prepared statements.
** Refer to the [SQL parameter] documentation for additional information.
**
** ^The sqlite3_result_blob() interface sets the result from
** an application-defined function to be the BLOB whose content is pointed
** to by the second parameter and which is N bytes long where N is the
** third parameter.
**
** ^The sqlite3_result_zeroblob() interfaces set the result of
** the application-defined function to be a BLOB containing all zero
** bytes and N bytes in size, where N is the value of the 2nd parameter.
**
** ^The sqlite3_result_double() interface sets the result from
** an application-defined function to be a floating point value specified
** by its 2nd argument.
**
** ^The sqlite3_result_error() and sqlite3_result_error16() functions
** cause the implemented SQL function to throw an exception.
** ^SQLite uses the string pointed to by the
** 2nd parameter of sqlite3_result_error() or sqlite3_result_error16()
** as the text of an error message.  ^SQLite interprets the error
** message string from sqlite3_result_error() as UTF-8. ^SQLite
** interprets the string from sqlite3_result_error16() as UTF-16 in native
** byte order.  ^If the third parameter to sqlite3_result_error()
** or sqlite3_result_error16() is negative then SQLite takes as the error
** message all text up through the first zero character.
** ^If the third parameter to sqlite3_result_error() or
** sqlite3_result_error16() is non-negative then SQLite takes that many
** bytes (not characters) from the 2nd parameter as the error message.
** ^The sqlite3_result_error() and sqlite3_result_error16()
** routines make a private copy of the error message text before
** they return.  Hence, the calling function can deallocate or
** modify the text after they return without harm.
** ^The sqlite3_result_error_code() function changes the error code
** returned by SQLite as a result of an error in a function.  ^By default,
** the error code is SQLITE_ERROR.  ^A subsequent call to sqlite3_result_error()
** or sqlite3_result_error16() resets the error code to SQLITE_ERROR.
**
** ^The sqlite3_result_toobig() interface causes SQLite to throw an error
** indicating that a string or BLOB is too long to represent.
**
** ^The sqlite3_result_nomem() interface causes SQLite to throw an error
** indicating that a memory allocation failed.
**
** ^The sqlite3_result_int() interface sets the return value
** of the application-defined function to be the 32-bit signed integer
** value given in the 2nd argument.
** ^The sqlite3_result_int64() interface sets the return value
** of the application-defined function to be the 64-bit signed integer
** value given in the 2nd argument.
**
** ^The sqlite3_result_null() interface sets the return value
** of the application-defined function to be NULL.
**
** ^The sqlite3_result_text(), sqlite3_result_text16(),
** sqlite3_result_text16le(), and sqlite3_result_text16be() interfaces
** set the return value of the application-defined function to be
** a text string which is represented as UTF-8, UTF-16 native byte order,
** UTF-16 little endian, or UTF-16 big endian, respectively.
** ^SQLite takes the text result from the application from
** the 2nd parameter of the sqlite3_result_text* interfaces.
** ^If the 3rd parameter to the sqlite3_result_text* interfaces
** is negative, then SQLite takes result text from the 2nd parameter
** through the first zero character.
** ^If the 3rd parameter to the sqlite3_result_text* interfaces
** is non-negative, then as many bytes (not characters) of the text
** pointed to by the 2nd parameter are taken as the application-defined
** function result.  If the 3rd parameter is non-negative, then it
** must be the byte offset into the string where the NUL terminator would
** appear if the string where NUL terminated.  If any NUL characters occur
** in the string at a byte offset that is less than the value of the 3rd
** parameter, then the resulting string will contain embedded NULs and the
** result of expressions operating on strings with embedded NULs is undefined.
** ^If the 4th parameter to the sqlite3_result_text* interfaces
** or sqlite3_result_blob is a non-NULL pointer, then SQLite calls that
** function as the destructor on the text or BLOB result when it has
** finished using that result.
** ^If the 4th parameter to the sqlite3_result_text* interfaces or to
** sqlite3_result_blob is the special constant SQLITE_STATIC, then SQLite
** assumes that the text or BLOB result is in constant space and does not
** copy the content of the parameter nor call a destructor on the content
** when it has finished using that result.
** ^If the 4th parameter to the sqlite3_result_text* interfaces
** or sqlite3_result_blob is the special constant SQLITE_TRANSIENT
** then SQLite makes a copy of the result into space obtained from
** from [sqlite3_malloc()] before it returns.
**
** ^The sqlite3_result_value() interface sets the result of
** the application-defined function to be a copy the
** [unprotected sqlite3_value] object specified by the 2nd parameter.  ^The
** sqlite3_result_value() interface makes a copy of the [sqlite3_value]
** so that the [sqlite3_value] specified in the parameter may change or
** be deallocated after sqlite3_result_value() returns without harm.
** ^A [protected sqlite3_value] object may always be used where an
** [unprotected sqlite3_value] object is required, so either
** kind of [sqlite3_value] object can be used with this interface.
**
** If these routines are called from within the different thread
** than the one containing the application-defined function that received
** the [sqlite3_context] pointer, the results are undefined.
*/
SQLITE_API void sqlite3_result_blob(sqlite3_context*, const void*, int, void(*)(void*));
SQLITE_API void sqlite3_result_double(sqlite3_context*, double);
SQLITE_API void sqlite3_result_error(sqlite3_context*, const char*, int);
SQLITE_API void sqlite3_result_error16(sqlite3_context*, const void*, int);
SQLITE_API void sqlite3_result_error_toobig(sqlite3_context*);
SQLITE_API void sqlite3_result_error_nomem(sqlite3_context*);
SQLITE_API void sqlite3_result_error_code(sqlite3_context*, int);
SQLITE_API void sqlite3_result_int(sqlite3_context*, int);
SQLITE_API void sqlite3_result_int64(sqlite3_context*, sqlite3_int64);
SQLITE_API void sqlite3_result_null(sqlite3_context*);
SQLITE_API void sqlite3_result_text(sqlite3_context*, const char*, int, void(*)(void*));
SQLITE_API void sqlite3_result_text16(sqlite3_context*, const void*, int, void(*)(void*));
SQLITE_API void sqlite3_result_text16le(sqlite3_context*, const void*, int,void(*)(void*));
SQLITE_API void sqlite3_result_text16be(sqlite3_context*, const void*, int,void(*)(void*));
SQLITE_API void sqlite3_result_value(sqlite3_context*, sqlite3_value*);
SQLITE_API void sqlite3_result_zeroblob(sqlite3_context*, int n);

/*
** CAPI3REF: Define New Collating Sequences
**
** ^These functions add, remove, or modify a [collation] associated
** with the [database connection] specified as the first argument.
**
** ^The name of the collation is a UTF-8 string
** for sqlite3_create_collation() and sqlite3_create_collation_v2()
** and a UTF-16 string in native byte order for sqlite3_create_collation16().
** ^Collation names that compare equal according to [sqlite3_strnicmp()] are
** considered to be the same name.
**
** ^(The third argument (eTextRep) must be one of the constants:
** <ul>
** <li> [SQLITE_UTF8],
** <li> [SQLITE_UTF16LE],
** <li> [SQLITE_UTF16BE],
** <li> [SQLITE_UTF16], or
** <li> [SQLITE_UTF16_ALIGNED].
** </ul>)^
** ^The eTextRep argument determines the encoding of strings passed
** to the collating function callback, xCallback.
** ^The [SQLITE_UTF16] and [SQLITE_UTF16_ALIGNED] values for eTextRep
** force strings to be UTF16 with native byte order.
** ^The [SQLITE_UTF16_ALIGNED] value for eTextRep forces strings to begin
** on an even byte address.
**
** ^The fourth argument, pArg, is an application data pointer that is passed
** through as the first argument to the collating function callback.
**
** ^The fifth argument, xCallback, is a pointer to the collating function.
** ^Multiple collating functions can be registered using the same name but
** with different eTextRep parameters and SQLite will use whichever
** function requires the least amount of data transformation.
** ^If the xCallback argument is NULL then the collating function is
** deleted.  ^When all collating functions having the same name are deleted,
** that collation is no longer usable.
**
** ^The collating function callback is invoked with a copy of the pArg 
** application data pointer and with two strings in the encoding specified
** by the eTextRep argument.  The collating function must return an
** integer that is negative, zero, or positive
** if the first string is less than, equal to, or greater than the second,
** respectively.  A collating function must always return the same answer
** given the same inputs.  If two or more collating functions are registered
** to the same collation name (using different eTextRep values) then all
** must give an equivalent answer when invoked with equivalent strings.
** The collating function must obey the following properties for all
** strings A, B, and C:
**
** <ol>
** <li> If A==B then B==A.
** <li> If A==B and B==C then A==C.
** <li> If A&lt;B THEN B&gt;A.
** <li> If A&lt;B and B&lt;C then A&lt;C.
** </ol>
**
** If a collating function fails any of the above constraints and that
** collating function is  registered and used, then the behavior of SQLite
** is undefined.
**
** ^The sqlite3_create_collation_v2() works like sqlite3_create_collation()
** with the addition that the xDestroy callback is invoked on pArg when
** the collating function is deleted.
** ^Collating functions are deleted when they are overridden by later
** calls to the collation creation functions or when the
** [database connection] is closed using [sqlite3_close()].
**
** ^The xDestroy callback is <u>not</u> called if the 
** sqlite3_create_collation_v2() function fails.  Applications that invoke
** sqlite3_create_collation_v2() with a non-NULL xDestroy argument should 
** check the return code and dispose of the application data pointer
** themselves rather than expecting SQLite to deal with it for them.
** This is different from every other SQLite interface.  The inconsistency 
** is unfortunate but cannot be changed without breaking backwards 
** compatibility.
**
** See also:  [sqlite3_collation_needed()] and [sqlite3_collation_needed16()].
*/
SQLITE_API int sqlite3_create_collation(
  sqlite3*, 
  const char *zName, 
  int eTextRep, 
  void *pArg,
  int(*xCompare)(void*,int,const void*,int,const void*)
);
SQLITE_API int sqlite3_create_collation_v2(
  sqlite3*, 
  const char *zName, 
  int eTextRep, 
  void *pArg,
  int(*xCompare)(void*,int,const void*,int,const void*),
  void(*xDestroy)(void*)
);
SQLITE_API int sqlite3_create_collation16(
  sqlite3*, 
  const void *zName,
  int eTextRep, 
  void *pArg,
  int(*xCompare)(void*,int,const void*,int,const void*)
);

/*
** CAPI3REF: Collation Needed Callbacks
**
** ^To avoid having to register all collation sequences before a database
** can be used, a single callback function may be registered with the
** [database connection] to be invoked whenever an undefined collation
** sequence is required.
**
** ^If the function is registered using the sqlite3_collation_needed() API,
** then it is passed the names of undefined collation sequences as strings
** encoded in UTF-8. ^If sqlite3_collation_needed16() is used,
** the names are passed as UTF-16 in machine native byte order.
** ^A call to either function replaces the existing collation-needed callback.
**
** ^(When the callback is invoked, the first argument passed is a copy
** of the second argument to sqlite3_collation_needed() or
** sqlite3_collation_needed16().  The second argument is the database
** connection.  The third argument is one of [SQLITE_UTF8], [SQLITE_UTF16BE],
** or [SQLITE_UTF16LE], indicating the most desirable form of the collation
** sequence function required.  The fourth parameter is the name of the
** required collation sequence.)^
**
** The callback function should register the desired collation using
** [sqlite3_create_collation()], [sqlite3_create_collation16()], or
** [sqlite3_create_collation_v2()].
*/
SQLITE_API int sqlite3_collation_needed(
  sqlite3*, 
  void*, 
  void(*)(void*,sqlite3*,int eTextRep,const char*)
);
SQLITE_API int sqlite3_collation_needed16(
  sqlite3*, 
  void*,
  void(*)(void*,sqlite3*,int eTextRep,const void*)
);

#ifdef SQLITE_HAS_CODEC
/*
** Specify the key for an encrypted database.  This routine should be
** called right after sqlite3_open().
**
** The code to implement this API is not available in the public release
** of SQLite.
*/
SQLITE_API int sqlite3_key(
  sqlite3 *db,                   /* Database to be rekeyed */
  const void *pKey, int nKey     /* The key */
);

/*
** Change the key on an open database.  If the current database is not
** encrypted, this routine will encrypt it.  If pNew==0 or nNew==0, the
** database is decrypted.
**
** The code to implement this API is not available in the public release
** of SQLite.
*/
SQLITE_API int sqlite3_rekey(
  sqlite3 *db,                   /* Database to be rekeyed */
  const void *pKey, int nKey     /* The new key */
);

/*
** Specify the activation key for a SEE database.  Unless 
** activated, none of the SEE routines will work.
*/
SQLITE_API void sqlite3_activate_see(
  const char *zPassPhrase        /* Activation phrase */
);
#endif

#ifdef SQLITE_ENABLE_CEROD
/*
** Specify the activation key for a CEROD database.  Unless 
** activated, none of the CEROD routines will work.
*/
SQLITE_API void sqlite3_activate_cerod(
  const char *zPassPhrase        /* Activation phrase */
);
#endif

/*
** CAPI3REF: Suspend Execution For A Short Time
**
** The sqlite3_sleep() function causes the current thread to suspend execution
** for at least a number of milliseconds specified in its parameter.
**
** If the operating system does not support sleep requests with
** millisecond time resolution, then the time will be rounded up to
** the nearest second. The number of milliseconds of sleep actually
** requested from the operating system is returned.
**
** ^SQLite implements this interface by calling the xSleep()
** method of the default [sqlite3_vfs] object.  If the xSleep() method
** of the default VFS is not implemented correctly, or not implemented at
** all, then the behavior of sqlite3_sleep() may deviate from the description
** in the previous paragraphs.
*/
SQLITE_API int sqlite3_sleep(int);

/*
** CAPI3REF: Name Of The Folder Holding Temporary Files
**
** ^(If this global variable is made to point to a string which is
** the name of a folder (a.k.a. directory), then all temporary files
** created by SQLite when using a built-in [sqlite3_vfs | VFS]
** will be placed in that directory.)^  ^If this variable
** is a NULL pointer, then SQLite performs a search for an appropriate
** temporary file directory.
**
** It is not safe to read or modify this variable in more than one
** thread at a time.  It is not safe to read or modify this variable
** if a [database connection] is being used at the same time in a separate
** thread.
** It is intended that this variable be set once
** as part of process initialization and before any SQLite interface
** routines have been called and that this variable remain unchanged
** thereafter.
**
** ^The [temp_store_directory pragma] may modify this variable and cause
** it to point to memory obtained from [sqlite3_malloc].  ^Furthermore,
** the [temp_store_directory pragma] always assumes that any string
** that this variable points to is held in memory obtained from 
** [sqlite3_malloc] and the pragma may attempt to free that memory
** using [sqlite3_free].
** Hence, if this variable is modified directly, either it should be
** made NULL or made to point to memory obtained from [sqlite3_malloc]
** or else the use of the [temp_store_directory pragma] should be avoided.
*/
SQLITE_API SQLITE_EXTERN char *sqlite3_temp_directory;

/*
** CAPI3REF: Test For Auto-Commit Mode
** KEYWORDS: {autocommit mode}
**
** ^The sqlite3_get_autocommit() interface returns non-zero or
** zero if the given database connection is or is not in autocommit mode,
** respectively.  ^Autocommit mode is on by default.
** ^Autocommit mode is disabled by a [BEGIN] statement.
** ^Autocommit mode is re-enabled by a [COMMIT] or [ROLLBACK].
**
** If certain kinds of errors occur on a statement within a multi-statement
** transaction (errors including [SQLITE_FULL], [SQLITE_IOERR],
** [SQLITE_NOMEM], [SQLITE_BUSY], and [SQLITE_INTERRUPT]) then the
** transaction might be rolled back automatically.  The only way to
** find out whether SQLite automatically rolled back the transaction after
** an error is to use this function.
**
** If another thread changes the autocommit status of the database
** connection while this routine is running, then the return value
** is undefined.
*/
SQLITE_API int sqlite3_get_autocommit(sqlite3*);

/*
** CAPI3REF: Find The Database Handle Of A Prepared Statement
**
** ^The sqlite3_db_handle interface returns the [database connection] handle
** to which a [prepared statement] belongs.  ^The [database connection]
** returned by sqlite3_db_handle is the same [database connection]
** that was the first argument
** to the [sqlite3_prepare_v2()] call (or its variants) that was used to
** create the statement in the first place.
*/
SQLITE_API sqlite3 *sqlite3_db_handle(sqlite3_stmt*);

/*
** CAPI3REF: Find the next prepared statement
**
** ^This interface returns a pointer to the next [prepared statement] after
** pStmt associated with the [database connection] pDb.  ^If pStmt is NULL
** then this interface returns a pointer to the first prepared statement
** associated with the database connection pDb.  ^If no prepared statement
** satisfies the conditions of this routine, it returns NULL.
**
** The [database connection] pointer D in a call to
** [sqlite3_next_stmt(D,S)] must refer to an open database
** connection and in particular must not be a NULL pointer.
*/
SQLITE_API sqlite3_stmt *sqlite3_next_stmt(sqlite3 *pDb, sqlite3_stmt *pStmt);

/*
** CAPI3REF: Commit And Rollback Notification Callbacks
**
** ^The sqlite3_commit_hook() interface registers a callback
** function to be invoked whenever a transaction is [COMMIT | committed].
** ^Any callback set by a previous call to sqlite3_commit_hook()
** for the same database connection is overridden.
** ^The sqlite3_rollback_hook() interface registers a callback
** function to be invoked whenever a transaction is [ROLLBACK | rolled back].
** ^Any callback set by a previous call to sqlite3_rollback_hook()
** for the same database connection is overridden.
** ^The pArg argument is passed through to the callback.
** ^If the callback on a commit hook function returns non-zero,
** then the commit is converted into a rollback.
**
** ^The sqlite3_commit_hook(D,C,P) and sqlite3_rollback_hook(D,C,P) functions
** return the P argument from the previous call of the same function
** on the same [database connection] D, or NULL for
** the first call for each function on D.
**
** The callback implementation must not do anything that will modify
** the database connection that invoked the callback.  Any actions
** to modify the database connection must be deferred until after the
** completion of the [sqlite3_step()] call that triggered the commit
** or rollback hook in the first place.
** Note that [sqlite3_prepare_v2()] and [sqlite3_step()] both modify their
** database connections for the meaning of "modify" in this paragraph.
**
** ^Registering a NULL function disables the callback.
**
** ^When the commit hook callback routine returns zero, the [COMMIT]
** operation is allowed to continue normally.  ^If the commit hook
** returns non-zero, then the [COMMIT] is converted into a [ROLLBACK].
** ^The rollback hook is invoked on a rollback that results from a commit
** hook returning non-zero, just as it would be with any other rollback.
**
** ^For the purposes of this API, a transaction is said to have been
** rolled back if an explicit "ROLLBACK" statement is executed, or
** an error or constraint causes an implicit rollback to occur.
** ^The rollback callback is not invoked if a transaction is
** automatically rolled back because the database connection is closed.
**
** See also the [sqlite3_update_hook()] interface.
*/
SQLITE_API void *sqlite3_commit_hook(sqlite3*, int(*)(void*), void*);
SQLITE_API void *sqlite3_rollback_hook(sqlite3*, void(*)(void *), void*);

/*
** CAPI3REF: Data Change Notification Callbacks
**
** ^The sqlite3_update_hook() interface registers a callback function
** with the [database connection] identified by the first argument
** to be invoked whenever a row is updated, inserted or deleted.
** ^Any callback set by a previous call to this function
** for the same database connection is overridden.
**
** ^The second argument is a pointer to the function to invoke when a
** row is updated, inserted or deleted.
** ^The first argument to the callback is a copy of the third argument
** to sqlite3_update_hook().
** ^The second callback argument is one of [SQLITE_INSERT], [SQLITE_DELETE],
** or [SQLITE_UPDATE], depending on the operation that caused the callback
** to be invoked.
** ^The third and fourth arguments to the callback contain pointers to the
** database and table name containing the affected row.
** ^The final callback parameter is the [rowid] of the row.
** ^In the case of an update, this is the [rowid] after the update takes place.
**
** ^(The update hook is not invoked when internal system tables are
** modified (i.e. sqlite_master and sqlite_sequence).)^
**
** ^In the current implementation, the update hook
** is not invoked when duplication rows are deleted because of an
** [ON CONFLICT | ON CONFLICT REPLACE] clause.  ^Nor is the update hook
** invoked when rows are deleted using the [truncate optimization].
** The exceptions defined in this paragraph might change in a future
** release of SQLite.
**
** The update hook implementation must not do anything that will modify
** the database connection that invoked the update hook.  Any actions
** to modify the database connection must be deferred until after the
** completion of the [sqlite3_step()] call that triggered the update hook.
** Note that [sqlite3_prepare_v2()] and [sqlite3_step()] both modify their
** database connections for the meaning of "modify" in this paragraph.
**
** ^The sqlite3_update_hook(D,C,P) function
** returns the P argument from the previous call
** on the same [database connection] D, or NULL for
** the first call on D.
**
** See also the [sqlite3_commit_hook()] and [sqlite3_rollback_hook()]
** interfaces.
*/
SQLITE_API void *sqlite3_update_hook(
  sqlite3*, 
  void(*)(void *,int ,char const *,char const *,sqlite3_int64),
  void*
);

/*
** CAPI3REF: Enable Or Disable Shared Pager Cache
** KEYWORDS: {shared cache}
**
** ^(This routine enables or disables the sharing of the database cache
** and schema data structures between [database connection | connections]
** to the same database. Sharing is enabled if the argument is true
** and disabled if the argument is false.)^
**
** ^Cache sharing is enabled and disabled for an entire process.
** This is a change as of SQLite version 3.5.0. In prior versions of SQLite,
** sharing was enabled or disabled for each thread separately.
**
** ^(The cache sharing mode set by this interface effects all subsequent
** calls to [sqlite3_open()], [sqlite3_open_v2()], and [sqlite3_open16()].
** Existing database connections continue use the sharing mode
** that was in effect at the time they were opened.)^
**
** ^(This routine returns [SQLITE_OK] if shared cache was enabled or disabled
** successfully.  An [error code] is returned otherwise.)^
**
** ^Shared cache is disabled by default. But this might change in
** future releases of SQLite.  Applications that care about shared
** cache setting should set it explicitly.
**
** See Also:  [SQLite Shared-Cache Mode]
*/
SQLITE_API int sqlite3_enable_shared_cache(int);

/*
** CAPI3REF: Attempt To Free Heap Memory
**
** ^The sqlite3_release_memory() interface attempts to free N bytes
** of heap memory by deallocating non-essential memory allocations
** held by the database library.   Memory used to cache database
** pages to improve performance is an example of non-essential memory.
** ^sqlite3_release_memory() returns the number of bytes actually freed,
** which might be more or less than the amount requested.
** ^The sqlite3_release_memory() routine is a no-op returning zero
** if SQLite is not compiled with [SQLITE_ENABLE_MEMORY_MANAGEMENT].
*/
SQLITE_API int sqlite3_release_memory(int);

/*
** CAPI3REF: Impose A Limit On Heap Size
**
** ^The sqlite3_soft_heap_limit64() interface sets and/or queries the
** soft limit on the amount of heap memory that may be allocated by SQLite.
** ^SQLite strives to keep heap memory utilization below the soft heap
** limit by reducing the number of pages held in the page cache
** as heap memory usages approaches the limit.
** ^The soft heap limit is "soft" because even though SQLite strives to stay
** below the limit, it will exceed the limit rather than generate
** an [SQLITE_NOMEM] error.  In other words, the soft heap limit 
** is advisory only.
**
** ^The return value from sqlite3_soft_heap_limit64() is the size of
** the soft heap limit prior to the call.  ^If the argument N is negative
** then no change is made to the soft heap limit.  Hence, the current
** size of the soft heap limit can be determined by invoking
** sqlite3_soft_heap_limit64() with a negative argument.
**
** ^If the argument N is zero then the soft heap limit is disabled.
**
** ^(The soft heap limit is not enforced in the current implementation
** if one or more of following conditions are true:
**
** <ul>
** <li> The soft heap limit is set to zero.
** <li> Memory accounting is disabled using a combination of the
**      [sqlite3_config]([SQLITE_CONFIG_MEMSTATUS],...) start-time option and
**      the [SQLITE_DEFAULT_MEMSTATUS] compile-time option.
** <li> An alternative page cache implementation is specified using
**      [sqlite3_config]([SQLITE_CONFIG_PCACHE],...).
** <li> The page cache allocates from its own memory pool supplied
**      by [sqlite3_config]([SQLITE_CONFIG_PAGECACHE],...) rather than
**      from the heap.
** </ul>)^
**
** Beginning with SQLite version 3.7.3, the soft heap limit is enforced
** regardless of whether or not the [SQLITE_ENABLE_MEMORY_MANAGEMENT]
** compile-time option is invoked.  With [SQLITE_ENABLE_MEMORY_MANAGEMENT],
** the soft heap limit is enforced on every memory allocation.  Without
** [SQLITE_ENABLE_MEMORY_MANAGEMENT], the soft heap limit is only enforced
** when memory is allocated by the page cache.  Testing suggests that because
** the page cache is the predominate memory user in SQLite, most
** applications will achieve adequate soft heap limit enforcement without
** the use of [SQLITE_ENABLE_MEMORY_MANAGEMENT].
**
** The circumstances under which SQLite will enforce the soft heap limit may
** changes in future releases of SQLite.
*/
SQLITE_API sqlite3_int64 sqlite3_soft_heap_limit64(sqlite3_int64 N);

/*
** CAPI3REF: Deprecated Soft Heap Limit Interface
** DEPRECATED
**
** This is a deprecated version of the [sqlite3_soft_heap_limit64()]
** interface.  This routine is provided for historical compatibility
** only.  All new applications should use the
** [sqlite3_soft_heap_limit64()] interface rather than this one.
*/
SQLITE_API SQLITE_DEPRECATED void sqlite3_soft_heap_limit(int N);


/*
** CAPI3REF: Extract Metadata About A Column Of A Table
**
** ^This routine returns metadata about a specific column of a specific
** database table accessible using the [database connection] handle
** passed as the first function argument.
**
** ^The column is identified by the second, third and fourth parameters to
** this function. ^The second parameter is either the name of the database
** (i.e. "main", "temp", or an attached database) containing the specified
** table or NULL. ^If it is NULL, then all attached databases are searched
** for the table using the same algorithm used by the database engine to
** resolve unqualified table references.
**
** ^The third and fourth parameters to this function are the table and column
** name of the desired column, respectively. Neither of these parameters
** may be NULL.
**
** ^Metadata is returned by writing to the memory locations passed as the 5th
** and subsequent parameters to this function. ^Any of these arguments may be
** NULL, in which case the corresponding element of metadata is omitted.
**
** ^(<blockquote>
** <table border="1">
** <tr><th> Parameter <th> Output<br>Type <th>  Description
**
** <tr><td> 5th <td> const char* <td> Data type
** <tr><td> 6th <td> const char* <td> Name of default collation sequence
** <tr><td> 7th <td> int         <td> True if column has a NOT NULL constraint
** <tr><td> 8th <td> int         <td> True if column is part of the PRIMARY KEY
** <tr><td> 9th <td> int         <td> True if column is [AUTOINCREMENT]
** </table>
** </blockquote>)^
**
** ^The memory pointed to by the character pointers returned for the
** declaration type and collation sequence is valid only until the next
** call to any SQLite API function.
**
** ^If the specified table is actually a view, an [error code] is returned.
**
** ^If the specified column is "rowid", "oid" or "_rowid_" and an
** [INTEGER PRIMARY KEY] column has been explicitly declared, then the output
** parameters are set for the explicitly declared column. ^(If there is no
** explicitly declared [INTEGER PRIMARY KEY] column, then the output
** parameters are set as follows:
**
** <pre>
**     data type: "INTEGER"
**     collation sequence: "BINARY"
**     not null: 0
**     primary key: 1
**     auto increment: 0
** </pre>)^
**
** ^(This function may load one or more schemas from database files. If an
** error occurs during this process, or if the requested table or column
** cannot be found, an [error code] is returned and an error message left
** in the [database connection] (to be retrieved using sqlite3_errmsg()).)^
**
** ^This API is only available if the library was compiled with the
** [SQLITE_ENABLE_COLUMN_METADATA] C-preprocessor symbol defined.
*/
SQLITE_API int sqlite3_table_column_metadata(
  sqlite3 *db,                /* Connection handle */
  const char *zDbName,        /* Database name or NULL */
  const char *zTableName,     /* Table name */
  const char *zColumnName,    /* Column name */
  char const **pzDataType,    /* OUTPUT: Declared data type */
  char const **pzCollSeq,     /* OUTPUT: Collation sequence name */
  int *pNotNull,              /* OUTPUT: True if NOT NULL constraint exists */
  int *pPrimaryKey,           /* OUTPUT: True if column part of PK */
  int *pAutoinc               /* OUTPUT: True if column is auto-increment */
);

/*
** CAPI3REF: Load An Extension
**
** ^This interface loads an SQLite extension library from the named file.
**
** ^The sqlite3_load_extension() interface attempts to load an
** SQLite extension library contained in the file zFile.
**
** ^The entry point is zProc.
** ^zProc may be 0, in which case the name of the entry point
** defaults to "sqlite3_extension_init".
** ^The sqlite3_load_extension() interface returns
** [SQLITE_OK] on success and [SQLITE_ERROR] if something goes wrong.
** ^If an error occurs and pzErrMsg is not 0, then the
** [sqlite3_load_extension()] interface shall attempt to
** fill *pzErrMsg with error message text stored in memory
** obtained from [sqlite3_malloc()]. The calling function
** should free this memory by calling [sqlite3_free()].
**
** ^Extension loading must be enabled using
** [sqlite3_enable_load_extension()] prior to calling this API,
** otherwise an error will be returned.
**
** See also the [load_extension() SQL function].
*/
SQLITE_API int sqlite3_load_extension(
  sqlite3 *db,          /* Load the extension into this database connection */
  const char *zFile,    /* Name of the shared library containing extension */
  const char *zProc,    /* Entry point.  Derived from zFile if 0 */
  char **pzErrMsg       /* Put error message here if not 0 */
);

/*
** CAPI3REF: Enable Or Disable Extension Loading
**
** ^So as not to open security holes in older applications that are
** unprepared to deal with extension loading, and as a means of disabling
** extension loading while evaluating user-entered SQL, the following API
** is provided to turn the [sqlite3_load_extension()] mechanism on and off.
**
** ^Extension loading is off by default. See ticket #1863.
** ^Call the sqlite3_enable_load_extension() routine with onoff==1
** to turn extension loading on and call it with onoff==0 to turn
** it back off again.
*/
SQLITE_API int sqlite3_enable_load_extension(sqlite3 *db, int onoff);

/*
** CAPI3REF: Automatically Load Statically Linked Extensions
**
** ^This interface causes the xEntryPoint() function to be invoked for
** each new [database connection] that is created.  The idea here is that
** xEntryPoint() is the entry point for a statically linked SQLite extension
** that is to be automatically loaded into all new database connections.
**
** ^(Even though the function prototype shows that xEntryPoint() takes
** no arguments and returns void, SQLite invokes xEntryPoint() with three
** arguments and expects and integer result as if the signature of the
** entry point where as follows:
**
** <blockquote><pre>
** &nbsp;  int xEntryPoint(
** &nbsp;    sqlite3 *db,
** &nbsp;    const char **pzErrMsg,
** &nbsp;    const struct sqlite3_api_routines *pThunk
** &nbsp;  );
** </pre></blockquote>)^
**
** If the xEntryPoint routine encounters an error, it should make *pzErrMsg
** point to an appropriate error message (obtained from [sqlite3_mprintf()])
** and return an appropriate [error code].  ^SQLite ensures that *pzErrMsg
** is NULL before calling the xEntryPoint().  ^SQLite will invoke
** [sqlite3_free()] on *pzErrMsg after xEntryPoint() returns.  ^If any
** xEntryPoint() returns an error, the [sqlite3_open()], [sqlite3_open16()],
** or [sqlite3_open_v2()] call that provoked the xEntryPoint() will fail.
**
** ^Calling sqlite3_auto_extension(X) with an entry point X that is already
** on the list of automatic extensions is a harmless no-op. ^No entry point
** will be called more than once for each database connection that is opened.
**
** See also: [sqlite3_reset_auto_extension()].
*/
SQLITE_API int sqlite3_auto_extension(void (*xEntryPoint)(void));

/*
** CAPI3REF: Reset Automatic Extension Loading
**
** ^This interface disables all automatic extensions previously
** registered using [sqlite3_auto_extension()].
*/
SQLITE_API void sqlite3_reset_auto_extension(void);

/*
** The interface to the virtual-table mechanism is currently considered
** to be experimental.  The interface might change in incompatible ways.
** If this is a problem for you, do not use the interface at this time.
**
** When the virtual-table mechanism stabilizes, we will declare the
** interface fixed, support it indefinitely, and remove this comment.
*/

/*
** Structures used by the virtual table interface
*/
typedef struct sqlite3_vtab sqlite3_vtab;
typedef struct sqlite3_index_info sqlite3_index_info;
typedef struct sqlite3_vtab_cursor sqlite3_vtab_cursor;
typedef struct sqlite3_module sqlite3_module;

/*
** CAPI3REF: Virtual Table Object
** KEYWORDS: sqlite3_module {virtual table module}
**
** This structure, sometimes called a "virtual table module", 
** defines the implementation of a [virtual tables].  
** This structure consists mostly of methods for the module.
**
** ^A virtual table module is created by filling in a persistent
** instance of this structure and passing a pointer to that instance
** to [sqlite3_create_module()] or [sqlite3_create_module_v2()].
** ^The registration remains valid until it is replaced by a different
** module or until the [database connection] closes.  The content
** of this structure must not change while it is registered with
** any database connection.
*/
struct sqlite3_module {
  int iVersion;
  int (*xCreate)(sqlite3*, void *pAux,
               int argc, const char *const*argv,
               sqlite3_vtab **ppVTab, char**);
  int (*xConnect)(sqlite3*, void *pAux,
               int argc, const char *const*argv,
               sqlite3_vtab **ppVTab, char**);
  int (*xBestIndex)(sqlite3_vtab *pVTab, sqlite3_index_info*);
  int (*xDisconnect)(sqlite3_vtab *pVTab);
  int (*xDestroy)(sqlite3_vtab *pVTab);
  int (*xOpen)(sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor);
  int (*xClose)(sqlite3_vtab_cursor*);
  int (*xFilter)(sqlite3_vtab_cursor*, int idxNum, const char *idxStr,
                int argc, sqlite3_value **argv);
  int (*xNext)(sqlite3_vtab_cursor*);
  int (*xEof)(sqlite3_vtab_cursor*);
  int (*xColumn)(sqlite3_vtab_cursor*, sqlite3_context*, int);
  int (*xRowid)(sqlite3_vtab_cursor*, sqlite3_int64 *pRowid);
  int (*xUpdate)(sqlite3_vtab *, int, sqlite3_value **, sqlite3_int64 *);
  int (*xBegin)(sqlite3_vtab *pVTab);
  int (*xSync)(sqlite3_vtab *pVTab);
  int (*xCommit)(sqlite3_vtab *pVTab);
  int (*xRollback)(sqlite3_vtab *pVTab);
  int (*xFindFunction)(sqlite3_vtab *pVtab, int nArg, const char *zName,
                       void (**pxFunc)(sqlite3_context*,int,sqlite3_value**),
                       void **ppArg);
  int (*xRename)(sqlite3_vtab *pVtab, const char *zNew);
  /* The methods above are in version 1 of the sqlite_module object. Those 
  ** below are for version 2 and greater. */
  int (*xSavepoint)(sqlite3_vtab *pVTab, int);
  int (*xRelease)(sqlite3_vtab *pVTab, int);
  int (*xRollbackTo)(sqlite3_vtab *pVTab, int);
};

/*
** CAPI3REF: Virtual Table Indexing Information
** KEYWORDS: sqlite3_index_info
**
** The sqlite3_index_info structure and its substructures is used as part
** of the [virtual table] interface to
** pass information into and receive the reply from the [xBestIndex]
** method of a [virtual table module].  The fields under **Inputs** are the
** inputs to xBestIndex and are read-only.  xBestIndex inserts its
** results into the **Outputs** fields.
**
** ^(The aConstraint[] array records WHERE clause constraints of the form:
**
** <blockquote>column OP expr</blockquote>
**
** where OP is =, &lt;, &lt;=, &gt;, or &gt;=.)^  ^(The particular operator is
** stored in aConstraint[].op using one of the
** [SQLITE_INDEX_CONSTRAINT_EQ | SQLITE_INDEX_CONSTRAINT_ values].)^
** ^(The index of the column is stored in
** aConstraint[].iColumn.)^  ^(aConstraint[].usable is TRUE if the
** expr on the right-hand side can be evaluated (and thus the constraint
** is usable) and false if it cannot.)^
**
** ^The optimizer automatically inverts terms of the form "expr OP column"
** and makes other simplifications to the WHERE clause in an attempt to
** get as many WHERE clause terms into the form shown above as possible.
** ^The aConstraint[] array only reports WHERE clause terms that are
** relevant to the particular virtual table being queried.
**
** ^Information about the ORDER BY clause is stored in aOrderBy[].
** ^Each term of aOrderBy records a column of the ORDER BY clause.
**
** The [xBestIndex] method must fill aConstraintUsage[] with information
** about what parameters to pass to xFilter.  ^If argvIndex>0 then
** the right-hand side of the corresponding aConstraint[] is evaluated
** and becomes the argvIndex-th entry in argv.  ^(If aConstraintUsage[].omit
** is true, then the constraint is assumed to be fully handled by the
** virtual table and is not checked again by SQLite.)^
**
** ^The idxNum and idxPtr values are recorded and passed into the
** [xFilter] method.
** ^[sqlite3_free()] is used to free idxPtr if and only if
** needToFreeIdxPtr is true.
**
** ^The orderByConsumed means that output from [xFilter]/[xNext] will occur in
** the correct order to satisfy the ORDER BY clause so that no separate
** sorting step is required.
**
** ^The estimatedCost value is an estimate of the cost of doing the
** particular lookup.  A full scan of a table with N entries should have
** a cost of N.  A binary search of a table of N entries should have a
** cost of approximately log(N).
*/
struct sqlite3_index_info {
  /* Inputs */
  int nConstraint;           /* Number of entries in aConstraint */
  struct sqlite3_index_constraint {
     int iColumn;              /* Column on left-hand side of constraint */
     unsigned char op;         /* Constraint operator */
     unsigned char usable;     /* True if this constraint is usable */
     int iTermOffset;          /* Used internally - xBestIndex should ignore */
  } *aConstraint;            /* Table of WHERE clause constraints */
  int nOrderBy;              /* Number of terms in the ORDER BY clause */
  struct sqlite3_index_orderby {
     int iColumn;              /* Column number */
     unsigned char desc;       /* True for DESC.  False for ASC. */
  } *aOrderBy;               /* The ORDER BY clause */
  /* Outputs */
  struct sqlite3_index_constraint_usage {
    int argvIndex;           /* if >0, constraint is part of argv to xFilter */
    unsigned char omit;      /* Do not code a test for this constraint */
  } *aConstraintUsage;
  int idxNum;                /* Number used to identify the index */
  char *idxStr;              /* String, possibly obtained from sqlite3_malloc */
  int needToFreeIdxStr;      /* Free idxStr using sqlite3_free() if true */
  int orderByConsumed;       /* True if output is already ordered */
  double estimatedCost;      /* Estimated cost of using this index */
};

/*
** CAPI3REF: Virtual Table Constraint Operator Codes
**
** These macros defined the allowed values for the
** [sqlite3_index_info].aConstraint[].op field.  Each value represents
** an operator that is part of a constraint term in the wHERE clause of
** a query that uses a [virtual table].
*/
#define SQLITE_INDEX_CONSTRAINT_EQ    2
#define SQLITE_INDEX_CONSTRAINT_GT    4
#define SQLITE_INDEX_CONSTRAINT_LE    8
#define SQLITE_INDEX_CONSTRAINT_LT    16
#define SQLITE_INDEX_CONSTRAINT_GE    32
#define SQLITE_INDEX_CONSTRAINT_MATCH 64

/*
** CAPI3REF: Register A Virtual Table Implementation
**
** ^These routines are used to register a new [virtual table module] name.
** ^Module names must be registered before
** creating a new [virtual table] using the module and before using a
** preexisting [virtual table] for the module.
**
** ^The module name is registered on the [database connection] specified
** by the first parameter.  ^The name of the module is given by the 
** second parameter.  ^The third parameter is a pointer to
** the implementation of the [virtual table module].   ^The fourth
** parameter is an arbitrary client data pointer that is passed through
** into the [xCreate] and [xConnect] methods of the virtual table module
** when a new virtual table is be being created or reinitialized.
**
** ^The sqlite3_create_module_v2() interface has a fifth parameter which
** is a pointer to a destructor for the pClientData.  ^SQLite will
** invoke the destructor function (if it is not NULL) when SQLite
** no longer needs the pClientData pointer.  ^The destructor will also
** be invoked if the call to sqlite3_create_module_v2() fails.
** ^The sqlite3_create_module()
** interface is equivalent to sqlite3_create_module_v2() with a NULL
** destructor.
*/
SQLITE_API int sqlite3_create_module(
  sqlite3 *db,               /* SQLite connection to register module with */
  const char *zName,         /* Name of the module */
  const sqlite3_module *p,   /* Methods for the module */
  void *pClientData          /* Client data for xCreate/xConnect */
);
SQLITE_API int sqlite3_create_module_v2(
  sqlite3 *db,               /* SQLite connection to register module with */
  const char *zName,         /* Name of the module */
  const sqlite3_module *p,   /* Methods for the module */
  void *pClientData,         /* Client data for xCreate/xConnect */
  void(*xDestroy)(void*)     /* Module destructor function */
);

/*
** CAPI3REF: Virtual Table Instance Object
** KEYWORDS: sqlite3_vtab
**
** Every [virtual table module] implementation uses a subclass
** of this object to describe a particular instance
** of the [virtual table].  Each subclass will
** be tailored to the specific needs of the module implementation.
** The purpose of this superclass is to define certain fields that are
** common to all module implementations.
**
** ^Virtual tables methods can set an error message by assigning a
** string obtained from [sqlite3_mprintf()] to zErrMsg.  The method should
** take care that any prior string is freed by a call to [sqlite3_free()]
** prior to assigning a new string to zErrMsg.  ^After the error message
** is delivered up to the client application, the string will be automatically
** freed by sqlite3_free() and the zErrMsg field will be zeroed.
*/
struct sqlite3_vtab {
  const sqlite3_module *pModule;  /* The module for this virtual table */
  int nRef;                       /* NO LONGER USED */
  char *zErrMsg;                  /* Error message from sqlite3_mprintf() */
  /* Virtual table implementations will typically add additional fields */
};

/*
** CAPI3REF: Virtual Table Cursor Object
** KEYWORDS: sqlite3_vtab_cursor {virtual table cursor}
**
** Every [virtual table module] implementation uses a subclass of the
** following structure to describe cursors that point into the
** [virtual table] and are used
** to loop through the virtual table.  Cursors are created using the
** [sqlite3_module.xOpen | xOpen] method of the module and are destroyed
** by the [sqlite3_module.xClose | xClose] method.  Cursors are used
** by the [xFilter], [xNext], [xEof], [xColumn], and [xRowid] methods
** of the module.  Each module implementation will define
** the content of a cursor structure to suit its own needs.
**
** This superclass exists in order to define fields of the cursor that
** are common to all implementations.
*/
struct sqlite3_vtab_cursor {
  sqlite3_vtab *pVtab;      /* Virtual table of this cursor */
  /* Virtual table implementations will typically add additional fields */
};

/*
** CAPI3REF: Declare The Schema Of A Virtual Table
**
** ^The [xCreate] and [xConnect] methods of a
** [virtual table module] call this interface
** to declare the format (the names and datatypes of the columns) of
** the virtual tables they implement.
*/
SQLITE_API int sqlite3_declare_vtab(sqlite3*, const char *zSQL);

/*
** CAPI3REF: Overload A Function For A Virtual Table
**
** ^(Virtual tables can provide alternative implementations of functions
** using the [xFindFunction] method of the [virtual table module].  
** But global versions of those functions
** must exist in order to be overloaded.)^
**
** ^(This API makes sure a global version of a function with a particular
** name and number of parameters exists.  If no such function exists
** before this API is called, a new function is created.)^  ^The implementation
** of the new function always causes an exception to be thrown.  So
** the new function is not good for anything by itself.  Its only
** purpose is to be a placeholder function that can be overloaded
** by a [virtual table].
*/
SQLITE_API int sqlite3_overload_function(sqlite3*, const char *zFuncName, int nArg);

/*
** The interface to the virtual-table mechanism defined above (back up
** to a comment remarkably similar to this one) is currently considered
** to be experimental.  The interface might change in incompatible ways.
** If this is a problem for you, do not use the interface at this time.
**
** When the virtual-table mechanism stabilizes, we will declare the
** interface fixed, support it indefinitely, and remove this comment.
*/

/*
** CAPI3REF: A Handle To An Open BLOB
** KEYWORDS: {BLOB handle} {BLOB handles}
**
** An instance of this object represents an open BLOB on which
** [sqlite3_blob_open | incremental BLOB I/O] can be performed.
** ^Objects of this type are created by [sqlite3_blob_open()]
** and destroyed by [sqlite3_blob_close()].
** ^The [sqlite3_blob_read()] and [sqlite3_blob_write()] interfaces
** can be used to read or write small subsections of the BLOB.
** ^The [sqlite3_blob_bytes()] interface returns the size of the BLOB in bytes.
*/
typedef struct sqlite3_blob sqlite3_blob;

/*
** CAPI3REF: Open A BLOB For Incremental I/O
**
** ^(This interfaces opens a [BLOB handle | handle] to the BLOB located
** in row iRow, column zColumn, table zTable in database zDb;
** in other words, the same BLOB that would be selected by:
**
** <pre>
**     SELECT zColumn FROM zDb.zTable WHERE [rowid] = iRow;
** </pre>)^
**
** ^If the flags parameter is non-zero, then the BLOB is opened for read
** and write access. ^If it is zero, the BLOB is opened for read access.
** ^It is not possible to open a column that is part of an index or primary 
** key for writing. ^If [foreign key constraints] are enabled, it is 
** not possible to open a column that is part of a [child key] for writing.
**
** ^Note that the database name is not the filename that contains
** the database but rather the symbolic name of the database that
** appears after the AS keyword when the database is connected using [ATTACH].
** ^For the main database file, the database name is "main".
** ^For TEMP tables, the database name is "temp".
**
** ^(On success, [SQLITE_OK] is returned and the new [BLOB handle] is written
** to *ppBlob. Otherwise an [error code] is returned and *ppBlob is set
** to be a null pointer.)^
** ^This function sets the [database connection] error code and message
** accessible via [sqlite3_errcode()] and [sqlite3_errmsg()] and related
** functions. ^Note that the *ppBlob variable is always initialized in a
** way that makes it safe to invoke [sqlite3_blob_close()] on *ppBlob
** regardless of the success or failure of this routine.
**
** ^(If the row that a BLOB handle points to is modified by an
** [UPDATE], [DELETE], or by [ON CONFLICT] side-effects
** then the BLOB handle is marked as "expired".
** This is true if any column of the row is changed, even a column
** other than the one the BLOB handle is open on.)^
** ^Calls to [sqlite3_blob_read()] and [sqlite3_blob_write()] for
** an expired BLOB handle fail with a return code of [SQLITE_ABORT].
** ^(Changes written into a BLOB prior to the BLOB expiring are not
** rolled back by the expiration of the BLOB.  Such changes will eventually
** commit if the transaction continues to completion.)^
**
** ^Use the [sqlite3_blob_bytes()] interface to determine the size of
** the opened blob.  ^The size of a blob may not be changed by this
** interface.  Use the [UPDATE] SQL command to change the size of a
** blob.
**
** ^The [sqlite3_bind_zeroblob()] and [sqlite3_result_zeroblob()] interfaces
** and the built-in [zeroblob] SQL function can be used, if desired,
** to create an empty, zero-filled blob in which to read or write using
** this interface.
**
** To avoid a resource leak, every open [BLOB handle] should eventually
** be released by a call to [sqlite3_blob_close()].
*/
SQLITE_API int sqlite3_blob_open(
  sqlite3*,
  const char *zDb,
  const char *zTable,
  const char *zColumn,
  sqlite3_int64 iRow,
  int flags,
  sqlite3_blob **ppBlob
);

/*
** CAPI3REF: Move a BLOB Handle to a New Row
**
** ^This function is used to move an existing blob handle so that it points
** to a different row of the same database table. ^The new row is identified
** by the rowid value passed as the second argument. Only the row can be
** changed. ^The database, table and column on which the blob handle is open
** remain the same. Moving an existing blob handle to a new row can be
** faster than closing the existing handle and opening a new one.
**
** ^(The new row must meet the same criteria as for [sqlite3_blob_open()] -
** it must exist and there must be either a blob or text value stored in
** the nominated column.)^ ^If the new row is not present in the table, or if
** it does not contain a blob or text value, or if another error occurs, an
** SQLite error code is returned and the blob handle is considered aborted.
** ^All subsequent calls to [sqlite3_blob_read()], [sqlite3_blob_write()] or
** [sqlite3_blob_reopen()] on an aborted blob handle immediately return
** SQLITE_ABORT. ^Calling [sqlite3_blob_bytes()] on an aborted blob handle
** always returns zero.
**
** ^This function sets the database handle error code and message.
*/
SQLITE_API SQLITE_EXPERIMENTAL int sqlite3_blob_reopen(sqlite3_blob *, sqlite3_int64);

/*
** CAPI3REF: Close A BLOB Handle
**
** ^Closes an open [BLOB handle].
**
** ^Closing a BLOB shall cause the current transaction to commit
** if there are no other BLOBs, no pending prepared statements, and the
** database connection is in [autocommit mode].
** ^If any writes were made to the BLOB, they might be held in cache
** until the close operation if they will fit.
**
** ^(Closing the BLOB often forces the changes
** out to disk and so if any I/O errors occur, they will likely occur
** at the time when the BLOB is closed.  Any errors that occur during
** closing are reported as a non-zero return value.)^
**
** ^(The BLOB is closed unconditionally.  Even if this routine returns
** an error code, the BLOB is still closed.)^
**
** ^Calling this routine with a null pointer (such as would be returned
** by a failed call to [sqlite3_blob_open()]) is a harmless no-op.
*/
SQLITE_API int sqlite3_blob_close(sqlite3_blob *);

/*
** CAPI3REF: Return The Size Of An Open BLOB
**
** ^Returns the size in bytes of the BLOB accessible via the 
** successfully opened [BLOB handle] in its only argument.  ^The
** incremental blob I/O routines can only read or overwriting existing
** blob content; they cannot change the size of a blob.
**
** This routine only works on a [BLOB handle] which has been created
** by a prior successful call to [sqlite3_blob_open()] and which has not
** been closed by [sqlite3_blob_close()].  Passing any other pointer in
** to this routine results in undefined and probably undesirable behavior.
*/
SQLITE_API int sqlite3_blob_bytes(sqlite3_blob *);

/*
** CAPI3REF: Read Data From A BLOB Incrementally
**
** ^(This function is used to read data from an open [BLOB handle] into a
** caller-supplied buffer. N bytes of data are copied into buffer Z
** from the open BLOB, starting at offset iOffset.)^
**
** ^If offset iOffset is less than N bytes from the end of the BLOB,
** [SQLITE_ERROR] is returned and no data is read.  ^If N or iOffset is
** less than zero, [SQLITE_ERROR] is returned and no data is read.
** ^The size of the blob (and hence the maximum value of N+iOffset)
** can be determined using the [sqlite3_blob_bytes()] interface.
**
** ^An attempt to read from an expired [BLOB handle] fails with an
** error code of [SQLITE_ABORT].
**
** ^(On success, sqlite3_blob_read() returns SQLITE_OK.
** Otherwise, an [error code] or an [extended error code] is returned.)^
**
** This routine only works on a [BLOB handle] which has been created
** by a prior successful call to [sqlite3_blob_open()] and which has not
** been closed by [sqlite3_blob_close()].  Passing any other pointer in
** to this routine results in undefined and probably undesirable behavior.
**
** See also: [sqlite3_blob_write()].
*/
SQLITE_API int sqlite3_blob_read(sqlite3_blob *, void *Z, int N, int iOffset);

/*
** CAPI3REF: Write Data Into A BLOB Incrementally
**
** ^This function is used to write data into an open [BLOB handle] from a
** caller-supplied buffer. ^N bytes of data are copied from the buffer Z
** into the open BLOB, starting at offset iOffset.
**
** ^If the [BLOB handle] passed as the first argument was not opened for
** writing (the flags parameter to [sqlite3_blob_open()] was zero),
** this function returns [SQLITE_READONLY].
**
** ^This function may only modify the contents of the BLOB; it is
** not possible to increase the size of a BLOB using this API.
** ^If offset iOffset is less than N bytes from the end of the BLOB,
** [SQLITE_ERROR] is returned and no data is written.  ^If N is
** less than zero [SQLITE_ERROR] is returned and no data is written.
** The size of the BLOB (and hence the maximum value of N+iOffset)
** can be determined using the [sqlite3_blob_bytes()] interface.
**
** ^An attempt to write to an expired [BLOB handle] fails with an
** error code of [SQLITE_ABORT].  ^Writes to the BLOB that occurred
** before the [BLOB handle] expired are not rolled back by the
** expiration of the handle, though of course those changes might
** have been overwritten by the statement that expired the BLOB handle
** or by other independent statements.
**
** ^(On success, sqlite3_blob_write() returns SQLITE_OK.
** Otherwise, an  [error code] or an [extended error code] is returned.)^
**
** This routine only works on a [BLOB handle] which has been created
** by a prior successful call to [sqlite3_blob_open()] and which has not
** been closed by [sqlite3_blob_close()].  Passing any other pointer in
** to this routine results in undefined and probably undesirable behavior.
**
** See also: [sqlite3_blob_read()].
*/
SQLITE_API int sqlite3_blob_write(sqlite3_blob *, const void *z, int n, int iOffset);

/*
** CAPI3REF: Virtual File System Objects
**
** A virtual filesystem (VFS) is an [sqlite3_vfs] object
** that SQLite uses to interact
** with the underlying operating system.  Most SQLite builds come with a
** single default VFS that is appropriate for the host computer.
** New VFSes can be registered and existing VFSes can be unregistered.
** The following interfaces are provided.
**
** ^The sqlite3_vfs_find() interface returns a pointer to a VFS given its name.
** ^Names are case sensitive.
** ^Names are zero-terminated UTF-8 strings.
** ^If there is no match, a NULL pointer is returned.
** ^If zVfsName is NULL then the default VFS is returned.
**
** ^New VFSes are registered with sqlite3_vfs_register().
** ^Each new VFS becomes the default VFS if the makeDflt flag is set.
** ^The same VFS can be registered multiple times without injury.
** ^To make an existing VFS into the default VFS, register it again
** with the makeDflt flag set.  If two different VFSes with the
** same name are registered, the behavior is undefined.  If a
** VFS is registered with a name that is NULL or an empty string,
** then the behavior is undefined.
**
** ^Unregister a VFS with the sqlite3_vfs_unregister() interface.
** ^(If the default VFS is unregistered, another VFS is chosen as
** the default.  The choice for the new VFS is arbitrary.)^
*/
SQLITE_API sqlite3_vfs *sqlite3_vfs_find(const char *zVfsName);
SQLITE_API int sqlite3_vfs_register(sqlite3_vfs*, int makeDflt);
SQLITE_API int sqlite3_vfs_unregister(sqlite3_vfs*);

/*
** CAPI3REF: Mutexes
**
** The SQLite core uses these routines for thread
** synchronization. Though they are intended for internal
** use by SQLite, code that links against SQLite is
** permitted to use any of these routines.
**
** The SQLite source code contains multiple implementations
** of these mutex routines.  An appropriate implementation
** is selected automatically at compile-time.  ^(The following
** implementations are available in the SQLite core:
**
** <ul>
** <li>   SQLITE_MUTEX_OS2
** <li>   SQLITE_MUTEX_PTHREAD
** <li>   SQLITE_MUTEX_W32
** <li>   SQLITE_MUTEX_NOOP
** </ul>)^
**
** ^The SQLITE_MUTEX_NOOP implementation is a set of routines
** that does no real locking and is appropriate for use in
** a single-threaded application.  ^The SQLITE_MUTEX_OS2,
** SQLITE_MUTEX_PTHREAD, and SQLITE_MUTEX_W32 implementations
** are appropriate for use on OS/2, Unix, and Windows.
**
** ^(If SQLite is compiled with the SQLITE_MUTEX_APPDEF preprocessor
** macro defined (with "-DSQLITE_MUTEX_APPDEF=1"), then no mutex
** implementation is included with the library. In this case the
** application must supply a custom mutex implementation using the
** [SQLITE_CONFIG_MUTEX] option of the sqlite3_config() function
** before calling sqlite3_initialize() or any other public sqlite3_
** function that calls sqlite3_initialize().)^
**
** ^The sqlite3_mutex_alloc() routine allocates a new
** mutex and returns a pointer to it. ^If it returns NULL
** that means that a mutex could not be allocated.  ^SQLite
** will unwind its stack and return an error.  ^(The argument
** to sqlite3_mutex_alloc() is one of these integer constants:
**
** <ul>
** <li>  SQLITE_MUTEX_FAST
** <li>  SQLITE_MUTEX_RECURSIVE
** <li>  SQLITE_MUTEX_STATIC_MASTER
** <li>  SQLITE_MUTEX_STATIC_MEM
** <li>  SQLITE_MUTEX_STATIC_MEM2
** <li>  SQLITE_MUTEX_STATIC_PRNG
** <li>  SQLITE_MUTEX_STATIC_LRU
** <li>  SQLITE_MUTEX_STATIC_LRU2
** </ul>)^
**
** ^The first two constants (SQLITE_MUTEX_FAST and SQLITE_MUTEX_RECURSIVE)
** cause sqlite3_mutex_alloc() to create
** a new mutex.  ^The new mutex is recursive when SQLITE_MUTEX_RECURSIVE
** is used but not necessarily so when SQLITE_MUTEX_FAST is used.
** The mutex implementation does not need to make a distinction
** between SQLITE_MUTEX_RECURSIVE and SQLITE_MUTEX_FAST if it does
** not want to.  ^SQLite will only request a recursive mutex in
** cases where it really needs one.  ^If a faster non-recursive mutex
** implementation is available on the host platform, the mutex subsystem
** might return such a mutex in response to SQLITE_MUTEX_FAST.
**
** ^The other allowed parameters to sqlite3_mutex_alloc() (anything other
** than SQLITE_MUTEX_FAST and SQLITE_MUTEX_RECURSIVE) each return
** a pointer to a static preexisting mutex.  ^Six static mutexes are
** used by the current version of SQLite.  Future versions of SQLite
** may add additional static mutexes.  Static mutexes are for internal
** use by SQLite only.  Applications that use SQLite mutexes should
** use only the dynamic mutexes returned by SQLITE_MUTEX_FAST or
** SQLITE_MUTEX_RECURSIVE.
**
** ^Note that if one of the dynamic mutex parameters (SQLITE_MUTEX_FAST
** or SQLITE_MUTEX_RECURSIVE) is used then sqlite3_mutex_alloc()
** returns a different mutex on every call.  ^But for the static
** mutex types, the same mutex is returned on every call that has
** the same type number.
**
** ^The sqlite3_mutex_free() routine deallocates a previously
** allocated dynamic mutex.  ^SQLite is careful to deallocate every
** dynamic mutex that it allocates.  The dynamic mutexes must not be in
** use when they are deallocated.  Attempting to deallocate a static
** mutex results in undefined behavior.  ^SQLite never deallocates
** a static mutex.
**
** ^The sqlite3_mutex_enter() and sqlite3_mutex_try() routines attempt
** to enter a mutex.  ^If another thread is already within the mutex,
** sqlite3_mutex_enter() will block and sqlite3_mutex_try() will return
** SQLITE_BUSY.  ^The sqlite3_mutex_try() interface returns [SQLITE_OK]
** upon successful entry.  ^(Mutexes created using
** SQLITE_MUTEX_RECURSIVE can be entered multiple times by the same thread.
** In such cases the,
** mutex must be exited an equal number of times before another thread
** can enter.)^  ^(If the same thread tries to enter any other
** kind of mutex more than once, the behavior is undefined.
** SQLite will never exhibit
** such behavior in its own use of mutexes.)^
**
** ^(Some systems (for example, Windows 95) do not support the operation
** implemented by sqlite3_mutex_try().  On those systems, sqlite3_mutex_try()
** will always return SQLITE_BUSY.  The SQLite core only ever uses
** sqlite3_mutex_try() as an optimization so this is acceptable behavior.)^
**
** ^The sqlite3_mutex_leave() routine exits a mutex that was
** previously entered by the same thread.   ^(The behavior
** is undefined if the mutex is not currently entered by the
** calling thread or is not currently allocated.  SQLite will
** never do either.)^
**
** ^If the argument to sqlite3_mutex_enter(), sqlite3_mutex_try(), or
** sqlite3_mutex_leave() is a NULL pointer, then all three routines
** behave as no-ops.
**
** See also: [sqlite3_mutex_held()] and [sqlite3_mutex_notheld()].
*/
SQLITE_API sqlite3_mutex *sqlite3_mutex_alloc(int);
SQLITE_API void sqlite3_mutex_free(sqlite3_mutex*);
SQLITE_API void sqlite3_mutex_enter(sqlite3_mutex*);
SQLITE_API int sqlite3_mutex_try(sqlite3_mutex*);
SQLITE_API void sqlite3_mutex_leave(sqlite3_mutex*);

/*
** CAPI3REF: Mutex Methods Object
**
** An instance of this structure defines the low-level routines
** used to allocate and use mutexes.
**
** Usually, the default mutex implementations provided by SQLite are
** sufficient, however the user has the option of substituting a custom
** implementation for specialized deployments or systems for which SQLite
** does not provide a suitable implementation. In this case, the user
** creates and populates an instance of this structure to pass
** to sqlite3_config() along with the [SQLITE_CONFIG_MUTEX] option.
** Additionally, an instance of this structure can be used as an
** output variable when querying the system for the current mutex
** implementation, using the [SQLITE_CONFIG_GETMUTEX] option.
**
** ^The xMutexInit method defined by this structure is invoked as
** part of system initialization by the sqlite3_initialize() function.
** ^The xMutexInit routine is called by SQLite exactly once for each
** effective call to [sqlite3_initialize()].
**
** ^The xMutexEnd method defined by this structure is invoked as
** part of system shutdown by the sqlite3_shutdown() function. The
** implementation of this method is expected to release all outstanding
** resources obtained by the mutex methods implementation, especially
** those obtained by the xMutexInit method.  ^The xMutexEnd()
** interface is invoked exactly once for each call to [sqlite3_shutdown()].
**
** ^(The remaining seven methods defined by this structure (xMutexAlloc,
** xMutexFree, xMutexEnter, xMutexTry, xMutexLeave, xMutexHeld and
** xMutexNotheld) implement the following interfaces (respectively):
**
** <ul>
**   <li>  [sqlite3_mutex_alloc()] </li>
**   <li>  [sqlite3_mutex_free()] </li>
**   <li>  [sqlite3_mutex_enter()] </li>
**   <li>  [sqlite3_mutex_try()] </li>
**   <li>  [sqlite3_mutex_leave()] </li>
**   <li>  [sqlite3_mutex_held()] </li>
**   <li>  [sqlite3_mutex_notheld()] </li>
** </ul>)^
**
** The only difference is that the public sqlite3_XXX functions enumerated
** above silently ignore any invocations that pass a NULL pointer instead
** of a valid mutex handle. The implementations of the methods defined
** by this structure are not required to handle this case, the results
** of passing a NULL pointer instead of a valid mutex handle are undefined
** (i.e. it is acceptable to provide an implementation that segfaults if
** it is passed a NULL pointer).
**
** The xMutexInit() method must be threadsafe.  ^It must be harmless to
** invoke xMutexInit() multiple times within the same process and without
** intervening calls to xMutexEnd().  Second and subsequent calls to
** xMutexInit() must be no-ops.
**
** ^xMutexInit() must not use SQLite memory allocation ([sqlite3_malloc()]
** and its associates).  ^Similarly, xMutexAlloc() must not use SQLite memory
** allocation for a static mutex.  ^However xMutexAlloc() may use SQLite
** memory allocation for a fast or recursive mutex.
**
** ^SQLite will invoke the xMutexEnd() method when [sqlite3_shutdown()] is
** called, but only if the prior call to xMutexInit returned SQLITE_OK.
** If xMutexInit fails in any way, it is expected to clean up after itself
** prior to returning.
*/
typedef struct sqlite3_mutex_methods sqlite3_mutex_methods;
struct sqlite3_mutex_methods {
  int (*xMutexInit)(void);
  int (*xMutexEnd)(void);
  sqlite3_mutex *(*xMutexAlloc)(int);
  void (*xMutexFree)(sqlite3_mutex *);
  void (*xMutexEnter)(sqlite3_mutex *);
  int (*xMutexTry)(sqlite3_mutex *);
  void (*xMutexLeave)(sqlite3_mutex *);
  int (*xMutexHeld)(sqlite3_mutex *);
  int (*xMutexNotheld)(sqlite3_mutex *);
};

/*
** CAPI3REF: Mutex Verification Routines
**
** The sqlite3_mutex_held() and sqlite3_mutex_notheld() routines
** are intended for use inside assert() statements.  ^The SQLite core
** never uses these routines except inside an assert() and applications
** are advised to follow the lead of the core.  ^The SQLite core only
** provides implementations for these routines when it is compiled
** with the SQLITE_DEBUG flag.  ^External mutex implementations
** are only required to provide these routines if SQLITE_DEBUG is
** defined and if NDEBUG is not defined.
**
** ^These routines should return true if the mutex in their argument
** is held or not held, respectively, by the calling thread.
**
** ^The implementation is not required to provided versions of these
** routines that actually work. If the implementation does not provide working
** versions of these routines, it should at least provide stubs that always
** return true so that one does not get spurious assertion failures.
**
** ^If the argument to sqlite3_mutex_held() is a NULL pointer then
** the routine should return 1.   This seems counter-intuitive since
** clearly the mutex cannot be held if it does not exist.  But
** the reason the mutex does not exist is because the build is not
** using mutexes.  And we do not want the assert() containing the
** call to sqlite3_mutex_held() to fail, so a non-zero return is
** the appropriate thing to do.  ^The sqlite3_mutex_notheld()
** interface should also return 1 when given a NULL pointer.
*/
#ifndef NDEBUG
SQLITE_API int sqlite3_mutex_held(sqlite3_mutex*);
SQLITE_API int sqlite3_mutex_notheld(sqlite3_mutex*);
#endif

/*
** CAPI3REF: Mutex Types
**
** The [sqlite3_mutex_alloc()] interface takes a single argument
** which is one of these integer constants.
**
** The set of static mutexes may change from one SQLite release to the
** next.  Applications that override the built-in mutex logic must be
** prepared to accommodate additional static mutexes.
*/
#define SQLITE_MUTEX_FAST             0
#define SQLITE_MUTEX_RECURSIVE        1
#define SQLITE_MUTEX_STATIC_MASTER    2
#define SQLITE_MUTEX_STATIC_MEM       3  /* sqlite3_malloc() */
#define SQLITE_MUTEX_STATIC_MEM2      4  /* NOT USED */
#define SQLITE_MUTEX_STATIC_OPEN      4  /* sqlite3BtreeOpen() */
#define SQLITE_MUTEX_STATIC_PRNG      5  /* sqlite3_random() */
#define SQLITE_MUTEX_STATIC_LRU       6  /* lru page list */
#define SQLITE_MUTEX_STATIC_LRU2      7  /* NOT USED */
#define SQLITE_MUTEX_STATIC_PMEM      7  /* sqlite3PageMalloc() */

/*
** CAPI3REF: Retrieve the mutex for a database connection
**
** ^This interface returns a pointer the [sqlite3_mutex] object that 
** serializes access to the [database connection] given in the argument
** when the [threading mode] is Serialized.
** ^If the [threading mode] is Single-thread or Multi-thread then this
** routine returns a NULL pointer.
*/
SQLITE_API sqlite3_mutex *sqlite3_db_mutex(sqlite3*);

/*
** CAPI3REF: Low-Level Control Of Database Files
**
** ^The [sqlite3_file_control()] interface makes a direct call to the
** xFileControl method for the [sqlite3_io_methods] object associated
** with a particular database identified by the second argument. ^The
** name of the database is "main" for the main database or "temp" for the
** TEMP database, or the name that appears after the AS keyword for
** databases that are added using the [ATTACH] SQL command.
** ^A NULL pointer can be used in place of "main" to refer to the
** main database file.
** ^The third and fourth parameters to this routine
** are passed directly through to the second and third parameters of
** the xFileControl method.  ^The return value of the xFileControl
** method becomes the return value of this routine.
**
** ^The SQLITE_FCNTL_FILE_POINTER value for the op parameter causes
** a pointer to the underlying [sqlite3_file] object to be written into
** the space pointed to by the 4th parameter.  ^The SQLITE_FCNTL_FILE_POINTER
** case is a short-circuit path which does not actually invoke the
** underlying sqlite3_io_methods.xFileControl method.
**
** ^If the second parameter (zDbName) does not match the name of any
** open database file, then SQLITE_ERROR is returned.  ^This error
** code is not remembered and will not be recalled by [sqlite3_errcode()]
** or [sqlite3_errmsg()].  The underlying xFileControl method might
** also return SQLITE_ERROR.  There is no way to distinguish between
** an incorrect zDbName and an SQLITE_ERROR return from the underlying
** xFileControl method.
**
** See also: [SQLITE_FCNTL_LOCKSTATE]
*/
SQLITE_API int sqlite3_file_control(sqlite3*, const char *zDbName, int op, void*);

/*
** CAPI3REF: Testing Interface
**
** ^The sqlite3_test_control() interface is used to read out internal
** state of SQLite and to inject faults into SQLite for testing
** purposes.  ^The first parameter is an operation code that determines
** the number, meaning, and operation of all subsequent parameters.
**
** This interface is not for use by applications.  It exists solely
** for verifying the correct operation of the SQLite library.  Depending
** on how the SQLite library is compiled, this interface might not exist.
**
** The details of the operation codes, their meanings, the parameters
** they take, and what they do are all subject to change without notice.
** Unlike most of the SQLite API, this function is not guaranteed to
** operate consistently from one release to the next.
*/
SQLITE_API int sqlite3_test_control(int op, ...);

/*
** CAPI3REF: Testing Interface Operation Codes
**
** These constants are the valid operation code parameters used
** as the first argument to [sqlite3_test_control()].
**
** These parameters and their meanings are subject to change
** without notice.  These values are for testing purposes only.
** Applications should not use any of these parameters or the
** [sqlite3_test_control()] interface.
*/
#define SQLITE_TESTCTRL_FIRST                    5
#define SQLITE_TESTCTRL_PRNG_SAVE                5
#define SQLITE_TESTCTRL_PRNG_RESTORE             6
#define SQLITE_TESTCTRL_PRNG_RESET               7
#define SQLITE_TESTCTRL_BITVEC_TEST              8
#define SQLITE_TESTCTRL_FAULT_INSTALL            9
#define SQLITE_TESTCTRL_BENIGN_MALLOC_HOOKS     10
#define SQLITE_TESTCTRL_PENDING_BYTE            11
#define SQLITE_TESTCTRL_ASSERT                  12
#define SQLITE_TESTCTRL_ALWAYS                  13
#define SQLITE_TESTCTRL_RESERVE                 14
#define SQLITE_TESTCTRL_OPTIMIZATIONS           15
#define SQLITE_TESTCTRL_ISKEYWORD               16
#define SQLITE_TESTCTRL_PGHDRSZ                 17
#define SQLITE_TESTCTRL_SCRATCHMALLOC           18
#define SQLITE_TESTCTRL_LOCALTIME_FAULT         19
#define SQLITE_TESTCTRL_LAST                    19

/*
** CAPI3REF: SQLite Runtime Status
**
** ^This interface is used to retrieve runtime status information
** about the performance of SQLite, and optionally to reset various
** highwater marks.  ^The first argument is an integer code for
** the specific parameter to measure.  ^(Recognized integer codes
** are of the form [status parameters | SQLITE_STATUS_...].)^
** ^The current value of the parameter is returned into *pCurrent.
** ^The highest recorded value is returned in *pHighwater.  ^If the
** resetFlag is true, then the highest record value is reset after
** *pHighwater is written.  ^(Some parameters do not record the highest
** value.  For those parameters
** nothing is written into *pHighwater and the resetFlag is ignored.)^
** ^(Other parameters record only the highwater mark and not the current
** value.  For these latter parameters nothing is written into *pCurrent.)^
**
** ^The sqlite3_status() routine returns SQLITE_OK on success and a
** non-zero [error code] on failure.
**
** This routine is threadsafe but is not atomic.  This routine can be
** called while other threads are running the same or different SQLite
** interfaces.  However the values returned in *pCurrent and
** *pHighwater reflect the status of SQLite at different points in time
** and it is possible that another thread might change the parameter
** in between the times when *pCurrent and *pHighwater are written.
**
** See also: [sqlite3_db_status()]
*/
SQLITE_API int sqlite3_status(int op, int *pCurrent, int *pHighwater, int resetFlag);


/*
** CAPI3REF: Status Parameters
** KEYWORDS: {status parameters}
**
** These integer constants designate various run-time status parameters
** that can be returned by [sqlite3_status()].
**
** <dl>
** [[SQLITE_STATUS_MEMORY_USED]] ^(<dt>SQLITE_STATUS_MEMORY_USED</dt>
** <dd>This parameter is the current amount of memory checked out
** using [sqlite3_malloc()], either directly or indirectly.  The
** figure includes calls made to [sqlite3_malloc()] by the application
** and internal memory usage by the SQLite library.  Scratch memory
** controlled by [SQLITE_CONFIG_SCRATCH] and auxiliary page-cache
** memory controlled by [SQLITE_CONFIG_PAGECACHE] is not included in
** this parameter.  The amount returned is the sum of the allocation
** sizes as reported by the xSize method in [sqlite3_mem_methods].</dd>)^
**
** [[SQLITE_STATUS_MALLOC_SIZE]] ^(<dt>SQLITE_STATUS_MALLOC_SIZE</dt>
** <dd>This parameter records the largest memory allocation request
** handed to [sqlite3_malloc()] or [sqlite3_realloc()] (or their
** internal equivalents).  Only the value returned in the
** *pHighwater parameter to [sqlite3_status()] is of interest.  
** The value written into the *pCurrent parameter is undefined.</dd>)^
**
** [[SQLITE_STATUS_MALLOC_COUNT]] ^(<dt>SQLITE_STATUS_MALLOC_COUNT</dt>
** <dd>This parameter records the number of separate memory allocations
** currently checked out.</dd>)^
**
** [[SQLITE_STATUS_PAGECACHE_USED]] ^(<dt>SQLITE_STATUS_PAGECACHE_USED</dt>
** <dd>This parameter returns the number of pages used out of the
** [pagecache memory allocator] that was configured using 
** [SQLITE_CONFIG_PAGECACHE].  The
** value returned is in pages, not in bytes.</dd>)^
**
** [[SQLITE_STATUS_PAGECACHE_OVERFLOW]] 
** ^(<dt>SQLITE_STATUS_PAGECACHE_OVERFLOW</dt>
** <dd>This parameter returns the number of bytes of page cache
** allocation which could not be satisfied by the [SQLITE_CONFIG_PAGECACHE]
** buffer and where forced to overflow to [sqlite3_malloc()].  The
** returned value includes allocations that overflowed because they
** where too large (they were larger than the "sz" parameter to
** [SQLITE_CONFIG_PAGECACHE]) and allocations that overflowed because
** no space was left in the page cache.</dd>)^
**
** [[SQLITE_STATUS_PAGECACHE_SIZE]] ^(<dt>SQLITE_STATUS_PAGECACHE_SIZE</dt>
** <dd>This parameter records the largest memory allocation request
** handed to [pagecache memory allocator].  Only the value returned in the
** *pHighwater parameter to [sqlite3_status()] is of interest.  
** The value written into the *pCurrent parameter is undefined.</dd>)^
**
** [[SQLITE_STATUS_SCRATCH_USED]] ^(<dt>SQLITE_STATUS_SCRATCH_USED</dt>
** <dd>This parameter returns the number of allocations used out of the
** [scratch memory allocator] configured using
** [SQLITE_CONFIG_SCRATCH].  The value returned is in allocations, not
** in bytes.  Since a single thread may only have one scratch allocation
** outstanding at time, this parameter also reports the number of threads
** using scratch memory at the same time.</dd>)^
**
** [[SQLITE_STATUS_SCRATCH_OVERFLOW]] ^(<dt>SQLITE_STATUS_SCRATCH_OVERFLOW</dt>
** <dd>This parameter returns the number of bytes of scratch memory
** allocation which could not be satisfied by the [SQLITE_CONFIG_SCRATCH]
** buffer and where forced to overflow to [sqlite3_malloc()].  The values
** returned include overflows because the requested allocation was too
** larger (that is, because the requested allocation was larger than the
** "sz" parameter to [SQLITE_CONFIG_SCRATCH]) and because no scratch buffer
** slots were available.
** </dd>)^
**
** [[SQLITE_STATUS_SCRATCH_SIZE]] ^(<dt>SQLITE_STATUS_SCRATCH_SIZE</dt>
** <dd>This parameter records the largest memory allocation request
** handed to [scratch memory allocator].  Only the value returned in the
** *pHighwater parameter to [sqlite3_status()] is of interest.  
** The value written into the *pCurrent parameter is undefined.</dd>)^
**
** [[SQLITE_STATUS_PARSER_STACK]] ^(<dt>SQLITE_STATUS_PARSER_STACK</dt>
** <dd>This parameter records the deepest parser stack.  It is only
** meaningful if SQLite is compiled with [YYTRACKMAXSTACKDEPTH].</dd>)^
** </dl>
**
** New status parameters may be added from time to time.
*/
#define SQLITE_STATUS_MEMORY_USED          0
#define SQLITE_STATUS_PAGECACHE_USED       1
#define SQLITE_STATUS_PAGECACHE_OVERFLOW   2
#define SQLITE_STATUS_SCRATCH_USED         3
#define SQLITE_STATUS_SCRATCH_OVERFLOW     4
#define SQLITE_STATUS_MALLOC_SIZE          5
#define SQLITE_STATUS_PARSER_STACK         6
#define SQLITE_STATUS_PAGECACHE_SIZE       7
#define SQLITE_STATUS_SCRATCH_SIZE         8
#define SQLITE_STATUS_MALLOC_COUNT         9

/*
** CAPI3REF: Database Connection Status
**
** ^This interface is used to retrieve runtime status information 
** about a single [database connection].  ^The first argument is the
** database connection object to be interrogated.  ^The second argument
** is an integer constant, taken from the set of
** [SQLITE_DBSTATUS options], that
** determines the parameter to interrogate.  The set of 
** [SQLITE_DBSTATUS options] is likely
** to grow in future releases of SQLite.
**
** ^The current value of the requested parameter is written into *pCur
** and the highest instantaneous value is written into *pHiwtr.  ^If
** the resetFlg is true, then the highest instantaneous value is
** reset back down to the current value.
**
** ^The sqlite3_db_status() routine returns SQLITE_OK on success and a
** non-zero [error code] on failure.
**
** See also: [sqlite3_status()] and [sqlite3_stmt_status()].
*/
SQLITE_API int sqlite3_db_status(sqlite3*, int op, int *pCur, int *pHiwtr, int resetFlg);

/*
** CAPI3REF: Status Parameters for database connections
** KEYWORDS: {SQLITE_DBSTATUS options}
**
** These constants are the available integer "verbs" that can be passed as
** the second argument to the [sqlite3_db_status()] interface.
**
** New verbs may be added in future releases of SQLite. Existing verbs
** might be discontinued. Applications should check the return code from
** [sqlite3_db_status()] to make sure that the call worked.
** The [sqlite3_db_status()] interface will return a non-zero error code
** if a discontinued or unsupported verb is invoked.
**
** <dl>
** [[SQLITE_DBSTATUS_LOOKASIDE_USED]] ^(<dt>SQLITE_DBSTATUS_LOOKASIDE_USED</dt>
** <dd>This parameter returns the number of lookaside memory slots currently
** checked out.</dd>)^
**
** [[SQLITE_DBSTATUS_LOOKASIDE_HIT]] ^(<dt>SQLITE_DBSTATUS_LOOKASIDE_HIT</dt>
** <dd>This parameter returns the number malloc attempts that were 
** satisfied using lookaside memory. Only the high-water value is meaningful;
** the current value is always zero.)^
**
** [[SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE]]
** ^(<dt>SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE</dt>
** <dd>This parameter returns the number malloc attempts that might have
** been satisfied using lookaside memory but failed due to the amount of
** memory requested being larger than the lookaside slot size.
** Only the high-water value is meaningful;
** the current value is always zero.)^
**
** [[SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL]]
** ^(<dt>SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL</dt>
** <dd>This parameter returns the number malloc attempts that might have
** been satisfied using lookaside memory but failed due to all lookaside
** memory already being in use.
** Only the high-water value is meaningful;
** the current value is always zero.)^
**
** [[SQLITE_DBSTATUS_CACHE_USED]] ^(<dt>SQLITE_DBSTATUS_CACHE_USED</dt>
** <dd>This parameter returns the approximate number of of bytes of heap
** memory used by all pager caches associated with the database connection.)^
** ^The highwater mark associated with SQLITE_DBSTATUS_CACHE_USED is always 0.
**
** [[SQLITE_DBSTATUS_SCHEMA_USED]] ^(<dt>SQLITE_DBSTATUS_SCHEMA_USED</dt>
** <dd>This parameter returns the approximate number of of bytes of heap
** memory used to store the schema for all databases associated
** with the connection - main, temp, and any [ATTACH]-ed databases.)^ 
** ^The full amount of memory used by the schemas is reported, even if the
** schema memory is shared with other database connections due to
** [shared cache mode] being enabled.
** ^The highwater mark associated with SQLITE_DBSTATUS_SCHEMA_USED is always 0.
**
** [[SQLITE_DBSTATUS_STMT_USED]] ^(<dt>SQLITE_DBSTATUS_STMT_USED</dt>
** <dd>This parameter returns the approximate number of of bytes of heap
** and lookaside memory used by all prepared statements associated with
** the database connection.)^
** ^The highwater mark associated with SQLITE_DBSTATUS_STMT_USED is always 0.
** </dd>
**
** [[SQLITE_DBSTATUS_CACHE_HIT]] ^(<dt>SQLITE_DBSTATUS_CACHE_HIT</dt>
** <dd>This parameter returns the number of pager cache hits that have
** occurred.)^ ^The highwater mark associated with SQLITE_DBSTATUS_CACHE_HIT 
** is always 0.
** </dd>
**
** [[SQLITE_DBSTATUS_CACHE_MISS]] ^(<dt>SQLITE_DBSTATUS_CACHE_MISS</dt>
** <dd>This parameter returns the number of pager cache misses that have
** occurred.)^ ^The highwater mark associated with SQLITE_DBSTATUS_CACHE_MISS 
** is always 0.
** </dd>
** </dl>
*/
#define SQLITE_DBSTATUS_LOOKASIDE_USED       0
#define SQLITE_DBSTATUS_CACHE_USED           1
#define SQLITE_DBSTATUS_SCHEMA_USED          2
#define SQLITE_DBSTATUS_STMT_USED            3
#define SQLITE_DBSTATUS_LOOKASIDE_HIT        4
#define SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE  5
#define SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL  6
#define SQLITE_DBSTATUS_CACHE_HIT            7
#define SQLITE_DBSTATUS_CACHE_MISS           8
#define SQLITE_DBSTATUS_MAX                  8   /* Largest defined DBSTATUS */


/*
** CAPI3REF: Prepared Statement Status
**
** ^(Each prepared statement maintains various
** [SQLITE_STMTSTATUS counters] that measure the number
** of times it has performed specific operations.)^  These counters can
** be used to monitor the performance characteristics of the prepared
** statements.  For example, if the number of table steps greatly exceeds
** the number of table searches or result rows, that would tend to indicate
** that the prepared statement is using a full table scan rather than
** an index.  
**
** ^(This interface is used to retrieve and reset counter values from
** a [prepared statement].  The first argument is the prepared statement
** object to be interrogated.  The second argument
** is an integer code for a specific [SQLITE_STMTSTATUS counter]
** to be interrogated.)^
** ^The current value of the requested counter is returned.
** ^If the resetFlg is true, then the counter is reset to zero after this
** interface call returns.
**
** See also: [sqlite3_status()] and [sqlite3_db_status()].
*/
SQLITE_API int sqlite3_stmt_status(sqlite3_stmt*, int op,int resetFlg);

/*
** CAPI3REF: Status Parameters for prepared statements
** KEYWORDS: {SQLITE_STMTSTATUS counter} {SQLITE_STMTSTATUS counters}
**
** These preprocessor macros define integer codes that name counter
** values associated with the [sqlite3_stmt_status()] interface.
** The meanings of the various counters are as follows:
**
** <dl>
** [[SQLITE_STMTSTATUS_FULLSCAN_STEP]] <dt>SQLITE_STMTSTATUS_FULLSCAN_STEP</dt>
** <dd>^This is the number of times that SQLite has stepped forward in
** a table as part of a full table scan.  Large numbers for this counter
** may indicate opportunities for performance improvement through 
** careful use of indices.</dd>
**
** [[SQLITE_STMTSTATUS_SORT]] <dt>SQLITE_STMTSTATUS_SORT</dt>
** <dd>^This is the number of sort operations that have occurred.
** A non-zero value in this counter may indicate an opportunity to
** improvement performance through careful use of indices.</dd>
**
** [[SQLITE_STMTSTATUS_AUTOINDEX]] <dt>SQLITE_STMTSTATUS_AUTOINDEX</dt>
** <dd>^This is the number of rows inserted into transient indices that
** were created automatically in order to help joins run faster.
** A non-zero value in this counter may indicate an opportunity to
** improvement performance by adding permanent indices that do not
** need to be reinitialized each time the statement is run.</dd>
** </dl>
*/
#define SQLITE_STMTSTATUS_FULLSCAN_STEP     1
#define SQLITE_STMTSTATUS_SORT              2
#define SQLITE_STMTSTATUS_AUTOINDEX         3

/*
** CAPI3REF: Custom Page Cache Object
**
** The sqlite3_pcache type is opaque.  It is implemented by
** the pluggable module.  The SQLite core has no knowledge of
** its size or internal structure and never deals with the
** sqlite3_pcache object except by holding and passing pointers
** to the object.
**
** See [sqlite3_pcache_methods] for additional information.
*/
typedef struct sqlite3_pcache sqlite3_pcache;

/*
** CAPI3REF: Application Defined Page Cache.
** KEYWORDS: {page cache}
**
** ^(The [sqlite3_config]([SQLITE_CONFIG_PCACHE], ...) interface can
** register an alternative page cache implementation by passing in an 
** instance of the sqlite3_pcache_methods structure.)^
** In many applications, most of the heap memory allocated by 
** SQLite is used for the page cache.
** By implementing a 
** custom page cache using this API, an application can better control
** the amount of memory consumed by SQLite, the way in which 
** that memory is allocated and released, and the policies used to 
** determine exactly which parts of a database file are cached and for 
** how long.
**
** The alternative page cache mechanism is an
** extreme measure that is only needed by the most demanding applications.
** The built-in page cache is recommended for most uses.
**
** ^(The contents of the sqlite3_pcache_methods structure are copied to an
** internal buffer by SQLite within the call to [sqlite3_config].  Hence
** the application may discard the parameter after the call to
** [sqlite3_config()] returns.)^
**
** [[the xInit() page cache method]]
** ^(The xInit() method is called once for each effective 
** call to [sqlite3_initialize()])^
** (usually only once during the lifetime of the process). ^(The xInit()
** method is passed a copy of the sqlite3_pcache_methods.pArg value.)^
** The intent of the xInit() method is to set up global data structures 
** required by the custom page cache implementation. 
** ^(If the xInit() method is NULL, then the 
** built-in default page cache is used instead of the application defined
** page cache.)^
**
** [[the xShutdown() page cache method]]
** ^The xShutdown() method is called by [sqlite3_shutdown()].
** It can be used to clean up 
** any outstanding resources before process shutdown, if required.
** ^The xShutdown() method may be NULL.
**
** ^SQLite automatically serializes calls to the xInit method,
** so the xInit method need not be threadsafe.  ^The
** xShutdown method is only called from [sqlite3_shutdown()] so it does
** not need to be threadsafe either.  All other methods must be threadsafe
** in multithreaded applications.
**
** ^SQLite will never invoke xInit() more than once without an intervening
** call to xShutdown().
**
** [[the xCreate() page cache methods]]
** ^SQLite invokes the xCreate() method to construct a new cache instance.
** SQLite will typically create one cache instance for each open database file,
** though this is not guaranteed. ^The
** first parameter, szPage, is the size in bytes of the pages that must
** be allocated by the cache.  ^szPage will not be a power of two.  ^szPage
** will the page size of the database file that is to be cached plus an
** increment (here called "R") of less than 250.  SQLite will use the
** extra R bytes on each page to store metadata about the underlying
** database page on disk.  The value of R depends
** on the SQLite version, the target platform, and how SQLite was compiled.
** ^(R is constant for a particular build of SQLite. Except, there are two
** distinct values of R when SQLite is compiled with the proprietary
** ZIPVFS extension.)^  ^The second argument to
** xCreate(), bPurgeable, is true if the cache being created will
** be used to cache database pages of a file stored on disk, or
** false if it is used for an in-memory database. The cache implementation
** does not have to do anything special based with the value of bPurgeable;
** it is purely advisory.  ^On a cache where bPurgeable is false, SQLite will
** never invoke xUnpin() except to deliberately delete a page.
** ^In other words, calls to xUnpin() on a cache with bPurgeable set to
** false will always have the "discard" flag set to true.  
** ^Hence, a cache created with bPurgeable false will
** never contain any unpinned pages.
**
** [[the xCachesize() page cache method]]
** ^(The xCachesize() method may be called at any time by SQLite to set the
** suggested maximum cache-size (number of pages stored by) the cache
** instance passed as the first argument. This is the value configured using
** the SQLite "[PRAGMA cache_size]" command.)^  As with the bPurgeable
** parameter, the implementation is not required to do anything with this
** value; it is advisory only.
**
** [[the xPagecount() page cache methods]]
** The xPagecount() method must return the number of pages currently
** stored in the cache, both pinned and unpinned.
** 
** [[the xFetch() page cache methods]]
** The xFetch() method locates a page in the cache and returns a pointer to 
** the page, or a NULL pointer.
** A "page", in this context, means a buffer of szPage bytes aligned at an
** 8-byte boundary. The page to be fetched is determined by the key. ^The
** minimum key value is 1.  After it has been retrieved using xFetch, the page 
** is considered to be "pinned".
**
** If the requested page is already in the page cache, then the page cache
** implementation must return a pointer to the page buffer with its content
** intact.  If the requested page is not already in the cache, then the
** cache implementation should use the value of the createFlag
** parameter to help it determined what action to take:
**
** <table border=1 width=85% align=center>
** <tr><th> createFlag <th> Behaviour when page is not already in cache
** <tr><td> 0 <td> Do not allocate a new page.  Return NULL.
** <tr><td> 1 <td> Allocate a new page if it easy and convenient to do so.
**                 Otherwise return NULL.
** <tr><td> 2 <td> Make every effort to allocate a new page.  Only return
**                 NULL if allocating a new page is effectively impossible.
** </table>
**
** ^(SQLite will normally invoke xFetch() with a createFlag of 0 or 1.  SQLite
** will only use a createFlag of 2 after a prior call with a createFlag of 1
** failed.)^  In between the to xFetch() calls, SQLite may
** attempt to unpin one or more cache pages by spilling the content of
** pinned pages to disk and synching the operating system disk cache.
**
** [[the xUnpin() page cache method]]
** ^xUnpin() is called by SQLite with a pointer to a currently pinned page
** as its second argument.  If the third parameter, discard, is non-zero,
** then the page must be evicted from the cache.
** ^If the discard parameter is
** zero, then the page may be discarded or retained at the discretion of
** page cache implementation. ^The page cache implementation
** may choose to evict unpinned pages at any time.
**
** The cache must not perform any reference counting. A single 
** call to xUnpin() unpins the page regardless of the number of prior calls 
** to xFetch().
**
** [[the xRekey() page cache methods]]
** The xRekey() method is used to change the key value associated with the
** page passed as the second argument. If the cache
** previously contains an entry associated with newKey, it must be
** discarded. ^Any prior cache entry associated with newKey is guaranteed not
** to be pinned.
**
** When SQLite calls the xTruncate() method, the cache must discard all
** existing cache entries with page numbers (keys) greater than or equal
** to the value of the iLimit parameter passed to xTruncate(). If any
** of these pages are pinned, they are implicitly unpinned, meaning that
** they can be safely discarded.
**
** [[the xDestroy() page cache method]]
** ^The xDestroy() method is used to delete a cache allocated by xCreate().
** All resources associated with the specified cache should be freed. ^After
** calling the xDestroy() method, SQLite considers the [sqlite3_pcache*]
** handle invalid, and will not use it with any other sqlite3_pcache_methods
** functions.
*/
typedef struct sqlite3_pcache_methods sqlite3_pcache_methods;
struct sqlite3_pcache_methods {
  void *pArg;
  int (*xInit)(void*);
  void (*xShutdown)(void*);
  sqlite3_pcache *(*xCreate)(int szPage, int bPurgeable);
  void (*xCachesize)(sqlite3_pcache*, int nCachesize);
  int (*xPagecount)(sqlite3_pcache*);
  void *(*xFetch)(sqlite3_pcache*, unsigned key, int createFlag);
  void (*xUnpin)(sqlite3_pcache*, void*, int discard);
  void (*xRekey)(sqlite3_pcache*, void*, unsigned oldKey, unsigned newKey);
  void (*xTruncate)(sqlite3_pcache*, unsigned iLimit);
  void (*xDestroy)(sqlite3_pcache*);
};

/*
** CAPI3REF: Online Backup Object
**
** The sqlite3_backup object records state information about an ongoing
** online backup operation.  ^The sqlite3_backup object is created by
** a call to [sqlite3_backup_init()] and is destroyed by a call to
** [sqlite3_backup_finish()].
**
** See Also: [Using the SQLite Online Backup API]
*/
typedef struct sqlite3_backup sqlite3_backup;

/*
** CAPI3REF: Online Backup API.
**
** The backup API copies the content of one database into another.
** It is useful either for creating backups of databases or
** for copying in-memory databases to or from persistent files. 
**
** See Also: [Using the SQLite Online Backup API]
**
** ^SQLite holds a write transaction open on the destination database file
** for the duration of the backup operation.
** ^The source database is read-locked only while it is being read;
** it is not locked continuously for the entire backup operation.
** ^Thus, the backup may be performed on a live source database without
** preventing other database connections from
** reading or writing to the source database while the backup is underway.
** 
** ^(To perform a backup operation: 
**   <ol>
**     <li><b>sqlite3_backup_init()</b> is called once to initialize the
**         backup, 
**     <li><b>sqlite3_backup_step()</b> is called one or more times to transfer 
**         the data between the two databases, and finally
**     <li><b>sqlite3_backup_finish()</b> is called to release all resources 
**         associated with the backup operation. 
**   </ol>)^
** There should be exactly one call to sqlite3_backup_finish() for each
** successful call to sqlite3_backup_init().
**
** [[sqlite3_backup_init()]] <b>sqlite3_backup_init()</b>
**
** ^The D and N arguments to sqlite3_backup_init(D,N,S,M) are the 
** [database connection] associated with the destination database 
** and the database name, respectively.
** ^The database name is "main" for the main database, "temp" for the
** temporary database, or the name specified after the AS keyword in
** an [ATTACH] statement for an attached database.
** ^The S and M arguments passed to 
** sqlite3_backup_init(D,N,S,M) identify the [database connection]
** and database name of the source database, respectively.
** ^The source and destination [database connections] (parameters S and D)
** must be different or else sqlite3_backup_init(D,N,S,M) will fail with
** an error.
**
** ^If an error occurs within sqlite3_backup_init(D,N,S,M), then NULL is
** returned and an error code and error message are stored in the
** destination [database connection] D.
** ^The error code and message for the failed call to sqlite3_backup_init()
** can be retrieved using the [sqlite3_errcode()], [sqlite3_errmsg()], and/or
** [sqlite3_errmsg16()] functions.
** ^A successful call to sqlite3_backup_init() returns a pointer to an
** [sqlite3_backup] object.
** ^The [sqlite3_backup] object may be used with the sqlite3_backup_step() and
** sqlite3_backup_finish() functions to perform the specified backup 
** operation.
**
** [[sqlite3_backup_step()]] <b>sqlite3_backup_step()</b>
**
** ^Function sqlite3_backup_step(B,N) will copy up to N pages between 
** the source and destination databases specified by [sqlite3_backup] object B.
** ^If N is negative, all remaining source pages are copied. 
** ^If sqlite3_backup_step(B,N) successfully copies N pages and there
** are still more pages to be copied, then the function returns [SQLITE_OK].
** ^If sqlite3_backup_step(B,N) successfully finishes copying all pages
** from source to destination, then it returns [SQLITE_DONE].
** ^If an error occurs while running sqlite3_backup_step(B,N),
** then an [error code] is returned. ^As well as [SQLITE_OK] and
** [SQLITE_DONE], a call to sqlite3_backup_step() may return [SQLITE_READONLY],
** [SQLITE_NOMEM], [SQLITE_BUSY], [SQLITE_LOCKED], or an
** [SQLITE_IOERR_ACCESS | SQLITE_IOERR_XXX] extended error code.
**
** ^(The sqlite3_backup_step() might return [SQLITE_READONLY] if
** <ol>
** <li> the destination database was opened read-only, or
** <li> the destination database is using write-ahead-log journaling
** and the destination and source page sizes differ, or
** <li> the destination database is an in-memory database and the
** destination and source page sizes differ.
** </ol>)^
**
** ^If sqlite3_backup_step() cannot obtain a required file-system lock, then
** the [sqlite3_busy_handler | busy-handler function]
** is invoked (if one is specified). ^If the 
** busy-handler returns non-zero before the lock is available, then 
** [SQLITE_BUSY] is returned to the caller. ^In this case the call to
** sqlite3_backup_step() can be retried later. ^If the source
** [database connection]
** is being used to write to the source database when sqlite3_backup_step()
** is called, then [SQLITE_LOCKED] is returned immediately. ^Again, in this
** case the call to sqlite3_backup_step() can be retried later on. ^(If
** [SQLITE_IOERR_ACCESS | SQLITE_IOERR_XXX], [SQLITE_NOMEM], or
** [SQLITE_READONLY] is returned, then 
** there is no point in retrying the call to sqlite3_backup_step(). These 
** errors are considered fatal.)^  The application must accept 
** that the backup operation has failed and pass the backup operation handle 
** to the sqlite3_backup_finish() to release associated resources.
**
** ^The first call to sqlite3_backup_step() obtains an exclusive lock
** on the destination file. ^The exclusive lock is not released until either 
** sqlite3_backup_finish() is called or the backup operation is complete 
** and sqlite3_backup_step() returns [SQLITE_DONE].  ^Every call to
** sqlite3_backup_step() obtains a [shared lock] on the source database that
** lasts for the duration of the sqlite3_backup_step() call.
** ^Because the source database is not locked between calls to
** sqlite3_backup_step(), the source database may be modified mid-way
** through the backup process.  ^If the source database is modified by an
** external process or via a database connection other than the one being
** used by the backup operation, then the backup will be automatically
** restarted by the next call to sqlite3_backup_step(). ^If the source 
** database is modified by the using the same database connection as is used
** by the backup operation, then the backup database is automatically
** updated at the same time.
**
** [[sqlite3_backup_finish()]] <b>sqlite3_backup_finish()</b>
**
** When sqlite3_backup_step() has returned [SQLITE_DONE], or when the 
** application wishes to abandon the backup operation, the application
** should destroy the [sqlite3_backup] by passing it to sqlite3_backup_finish().
** ^The sqlite3_backup_finish() interfaces releases all
** resources associated with the [sqlite3_backup] object. 
** ^If sqlite3_backup_step() has not yet returned [SQLITE_DONE], then any
** active write-transaction on the destination database is rolled back.
** The [sqlite3_backup] object is invalid
** and may not be used following a call to sqlite3_backup_finish().
**
** ^The value returned by sqlite3_backup_finish is [SQLITE_OK] if no
** sqlite3_backup_step() errors occurred, regardless or whether or not
** sqlite3_backup_step() completed.
** ^If an out-of-memory condition or IO error occurred during any prior
** sqlite3_backup_step() call on the same [sqlite3_backup] object, then
** sqlite3_backup_finish() returns the corresponding [error code].
**
** ^A return of [SQLITE_BUSY] or [SQLITE_LOCKED] from sqlite3_backup_step()
** is not a permanent error and does not affect the return value of
** sqlite3_backup_finish().
**
** [[sqlite3_backup__remaining()]] [[sqlite3_backup_pagecount()]]
** <b>sqlite3_backup_remaining() and sqlite3_backup_pagecount()</b>
**
** ^Each call to sqlite3_backup_step() sets two values inside
** the [sqlite3_backup] object: the number of pages still to be backed
** up and the total number of pages in the source database file.
** The sqlite3_backup_remaining() and sqlite3_backup_pagecount() interfaces
** retrieve these two values, respectively.
**
** ^The values returned by these functions are only updated by
** sqlite3_backup_step(). ^If the source database is modified during a backup
** operation, then the values are not updated to account for any extra
** pages that need to be updated or the size of the source database file
** changing.
**
** <b>Concurrent Usage of Database Handles</b>
**
** ^The source [database connection] may be used by the application for other
** purposes while a backup operation is underway or being initialized.
** ^If SQLite is compiled and configured to support threadsafe database
** connections, then the source database connection may be used concurrently
** from within other threads.
**
** However, the application must guarantee that the destination 
** [database connection] is not passed to any other API (by any thread) after 
** sqlite3_backup_init() is called and before the corresponding call to
** sqlite3_backup_finish().  SQLite does not currently check to see
** if the application incorrectly accesses the destination [database connection]
** and so no error code is reported, but the operations may malfunction
** nevertheless.  Use of the destination database connection while a
** backup is in progress might also also cause a mutex deadlock.
**
** If running in [shared cache mode], the application must
** guarantee that the shared cache used by the destination database
** is not accessed while the backup is running. In practice this means
** that the application must guarantee that the disk file being 
** backed up to is not accessed by any connection within the process,
** not just the specific connection that was passed to sqlite3_backup_init().
**
** The [sqlite3_backup] object itself is partially threadsafe. Multiple 
** threads may safely make multiple concurrent calls to sqlite3_backup_step().
** However, the sqlite3_backup_remaining() and sqlite3_backup_pagecount()
** APIs are not strictly speaking threadsafe. If they are invoked at the
** same time as another thread is invoking sqlite3_backup_step() it is
** possible that they return invalid values.
*/
SQLITE_API sqlite3_backup *sqlite3_backup_init(
  sqlite3 *pDest,                        /* Destination database handle */
  const char *zDestName,                 /* Destination database name */
  sqlite3 *pSource,                      /* Source database handle */
  const char *zSourceName                /* Source database name */
);
SQLITE_API int sqlite3_backup_step(sqlite3_backup *p, int nPage);
SQLITE_API int sqlite3_backup_finish(sqlite3_backup *p);
SQLITE_API int sqlite3_backup_remaining(sqlite3_backup *p);
SQLITE_API int sqlite3_backup_pagecount(sqlite3_backup *p);

/*
** CAPI3REF: Unlock Notification
**
** ^When running in shared-cache mode, a database operation may fail with
** an [SQLITE_LOCKED] error if the required locks on the shared-cache or
** individual tables within the shared-cache cannot be obtained. See
** [SQLite Shared-Cache Mode] for a description of shared-cache locking. 
** ^This API may be used to register a callback that SQLite will invoke 
** when the connection currently holding the required lock relinquishes it.
** ^This API is only available if the library was compiled with the
** [SQLITE_ENABLE_UNLOCK_NOTIFY] C-preprocessor symbol defined.
**
** See Also: [Using the SQLite Unlock Notification Feature].
**
** ^Shared-cache locks are released when a database connection concludes
** its current transaction, either by committing it or rolling it back. 
**
** ^When a connection (known as the blocked connection) fails to obtain a
** shared-cache lock and SQLITE_LOCKED is returned to the caller, the
** identity of the database connection (the blocking connection) that
** has locked the required resource is stored internally. ^After an 
** application receives an SQLITE_LOCKED error, it may call the
** sqlite3_unlock_notify() method with the blocked connection handle as 
** the first argument to register for a callback that will be invoked
** when the blocking connections current transaction is concluded. ^The
** callback is invoked from within the [sqlite3_step] or [sqlite3_close]
** call that concludes the blocking connections transaction.
**
** ^(If sqlite3_unlock_notify() is called in a multi-threaded application,
** there is a chance that the blocking connection will have already
** concluded its transaction by the time sqlite3_unlock_notify() is invoked.
** If this happens, then the specified callback is invoked immediately,
** from within the call to sqlite3_unlock_notify().)^
**
** ^If the blocked connection is attempting to obtain a write-lock on a
** shared-cache table, and more than one other connection currently holds
** a read-lock on the same table, then SQLite arbitrarily selects one of 
** the other connections to use as the blocking connection.
**
** ^(There may be at most one unlock-notify callback registered by a 
** blocked connection. If sqlite3_unlock_notify() is called when the
** blocked connection already has a registered unlock-notify callback,
** then the new callback replaces the old.)^ ^If sqlite3_unlock_notify() is
** called with a NULL pointer as its second argument, then any existing
** unlock-notify callback is canceled. ^The blocked connections 
** unlock-notify callback may also be canceled by closing the blocked
** connection using [sqlite3_close()].
**
** The unlock-notify callback is not reentrant. If an application invokes
** any sqlite3_xxx API functions from within an unlock-notify callback, a
** crash or deadlock may be the result.
**
** ^Unless deadlock is detected (see below), sqlite3_unlock_notify() always
** returns SQLITE_OK.
**
** <b>Callback Invocation Details</b>
**
** When an unlock-notify callback is registered, the application provides a 
** single void* pointer that is passed to the callback when it is invoked.
** However, the signature of the callback function allows SQLite to pass
** it an array of void* context pointers. The first argument passed to
** an unlock-notify callback is a pointer to an array of void* pointers,
** and the second is the number of entries in the array.
**
** When a blocking connections transaction is concluded, there may be
** more than one blocked connection that has registered for an unlock-notify
** callback. ^If two or more such blocked connections have specified the
** same callback function, then instead of invoking the callback function
** multiple times, it is invoked once with the set of void* context pointers
** specified by the blocked connections bundled together into an array.
** This gives the application an opportunity to prioritize any actions 
** related to the set of unblocked database connections.
**
** <b>Deadlock Detection</b>
**
** Assuming that after registering for an unlock-notify callback a 
** database waits for the callback to be issued before taking any further
** action (a reasonable assumption), then using this API may cause the
** application to deadlock. For example, if connection X is waiting for
** connection Y's transaction to be concluded, and similarly connection
** Y is waiting on connection X's transaction, then neither connection
** will proceed and the system may remain deadlocked indefinitely.
**
** To avoid this scenario, the sqlite3_unlock_notify() performs deadlock
** detection. ^If a given call to sqlite3_unlock_notify() would put the
** system in a deadlocked state, then SQLITE_LOCKED is returned and no
** unlock-notify callback is registered. The system is said to be in
** a deadlocked state if connection A has registered for an unlock-notify
** callback on the conclusion of connection B's transaction, and connection
** B has itself registered for an unlock-notify callback when connection
** A's transaction is concluded. ^Indirect deadlock is also detected, so
** the system is also considered to be deadlocked if connection B has
** registered for an unlock-notify callback on the conclusion of connection
** C's transaction, where connection C is waiting on connection A. ^Any
** number of levels of indirection are allowed.
**
** <b>The "DROP TABLE" Exception</b>
**
** When a call to [sqlite3_step()] returns SQLITE_LOCKED, it is almost 
** always appropriate to call sqlite3_unlock_notify(). There is however,
** one exception. When executing a "DROP TABLE" or "DROP INDEX" statement,
** SQLite checks if there are any currently executing SELECT statements
** that belong to the same connection. If there are, SQLITE_LOCKED is
** returned. In this case there is no "blocking connection", so invoking
** sqlite3_unlock_notify() results in the unlock-notify callback being
** invoked immediately. If the application then re-attempts the "DROP TABLE"
** or "DROP INDEX" query, an infinite loop might be the result.
**
** One way around this problem is to check the extended error code returned
** by an sqlite3_step() call. ^(If there is a blocking connection, then the
** extended error code is set to SQLITE_LOCKED_SHAREDCACHE. Otherwise, in
** the special "DROP TABLE/INDEX" case, the extended error code is just 
** SQLITE_LOCKED.)^
*/
SQLITE_API int sqlite3_unlock_notify(
  sqlite3 *pBlocked,                          /* Waiting connection */
  void (*xNotify)(void **apArg, int nArg),    /* Callback function to invoke */
  void *pNotifyArg                            /* Argument to pass to xNotify */
);


/*
** CAPI3REF: String Comparison
**
** ^The [sqlite3_strnicmp()] API allows applications and extensions to
** compare the contents of two buffers containing UTF-8 strings in a
** case-independent fashion, using the same definition of case independence 
** that SQLite uses internally when comparing identifiers.
*/
SQLITE_API int sqlite3_strnicmp(const char *, const char *, int);

/*
** CAPI3REF: Error Logging Interface
**
** ^The [sqlite3_log()] interface writes a message into the error log
** established by the [SQLITE_CONFIG_LOG] option to [sqlite3_config()].
** ^If logging is enabled, the zFormat string and subsequent arguments are
** used with [sqlite3_snprintf()] to generate the final output string.
**
** The sqlite3_log() interface is intended for use by extensions such as
** virtual tables, collating functions, and SQL functions.  While there is
** nothing to prevent an application from calling sqlite3_log(), doing so
** is considered bad form.
**
** The zFormat string must not be NULL.
**
** To avoid deadlocks and other threading problems, the sqlite3_log() routine
** will not use dynamically allocated memory.  The log message is stored in
** a fixed-length buffer on the stack.  If the log message is longer than
** a few hundred characters, it will be truncated to the length of the
** buffer.
*/
SQLITE_API void sqlite3_log(int iErrCode, const char *zFormat, ...);

/*
** CAPI3REF: Write-Ahead Log Commit Hook
**
** ^The [sqlite3_wal_hook()] function is used to register a callback that
** will be invoked each time a database connection commits data to a
** [write-ahead log] (i.e. whenever a transaction is committed in
** [journal_mode | journal_mode=WAL mode]). 
**
** ^The callback is invoked by SQLite after the commit has taken place and 
** the associated write-lock on the database released, so the implementation 
** may read, write or [checkpoint] the database as required.
**
** ^The first parameter passed to the callback function when it is invoked
** is a copy of the third parameter passed to sqlite3_wal_hook() when
** registering the callback. ^The second is a copy of the database handle.
** ^The third parameter is the name of the database that was written to -
** either "main" or the name of an [ATTACH]-ed database. ^The fourth parameter
** is the number of pages currently in the write-ahead log file,
** including those that were just committed.
**
** The callback function should normally return [SQLITE_OK].  ^If an error
** code is returned, that error will propagate back up through the
** SQLite code base to cause the statement that provoked the callback
** to report an error, though the commit will have still occurred. If the
** callback returns [SQLITE_ROW] or [SQLITE_DONE], or if it returns a value
** that does not correspond to any valid SQLite error code, the results
** are undefined.
**
** A single database handle may have at most a single write-ahead log callback 
** registered at one time. ^Calling [sqlite3_wal_hook()] replaces any
** previously registered write-ahead log callback. ^Note that the
** [sqlite3_wal_autocheckpoint()] interface and the
** [wal_autocheckpoint pragma] both invoke [sqlite3_wal_hook()] and will
** those overwrite any prior [sqlite3_wal_hook()] settings.
*/
SQLITE_API void *sqlite3_wal_hook(
  sqlite3*, 
  int(*)(void *,sqlite3*,const char*,int),
  void*
);

/*
** CAPI3REF: Configure an auto-checkpoint
**
** ^The [sqlite3_wal_autocheckpoint(D,N)] is a wrapper around
** [sqlite3_wal_hook()] that causes any database on [database connection] D
** to automatically [checkpoint]
** after committing a transaction if there are N or
** more frames in the [write-ahead log] file.  ^Passing zero or 
** a negative value as the nFrame parameter disables automatic
** checkpoints entirely.
**
** ^The callback registered by this function replaces any existing callback
** registered using [sqlite3_wal_hook()].  ^Likewise, registering a callback
** using [sqlite3_wal_hook()] disables the automatic checkpoint mechanism
** configured by this function.
**
** ^The [wal_autocheckpoint pragma] can be used to invoke this interface
** from SQL.
**
** ^Every new [database connection] defaults to having the auto-checkpoint
** enabled with a threshold of 1000 or [SQLITE_DEFAULT_WAL_AUTOCHECKPOINT]
** pages.  The use of this interface
** is only necessary if the default setting is found to be suboptimal
** for a particular application.
*/
SQLITE_API int sqlite3_wal_autocheckpoint(sqlite3 *db, int N);

/*
** CAPI3REF: Checkpoint a database
**
** ^The [sqlite3_wal_checkpoint(D,X)] interface causes database named X
** on [database connection] D to be [checkpointed].  ^If X is NULL or an
** empty string, then a checkpoint is run on all databases of
** connection D.  ^If the database connection D is not in
** [WAL | write-ahead log mode] then this interface is a harmless no-op.
**
** ^The [wal_checkpoint pragma] can be used to invoke this interface
** from SQL.  ^The [sqlite3_wal_autocheckpoint()] interface and the
** [wal_autocheckpoint pragma] can be used to cause this interface to be
** run whenever the WAL reaches a certain size threshold.
**
** See also: [sqlite3_wal_checkpoint_v2()]
*/
SQLITE_API int sqlite3_wal_checkpoint(sqlite3 *db, const char *zDb);

/*
** CAPI3REF: Checkpoint a database
**
** Run a checkpoint operation on WAL database zDb attached to database 
** handle db. The specific operation is determined by the value of the 
** eMode parameter:
**
** <dl>
** <dt>SQLITE_CHECKPOINT_PASSIVE<dd>
**   Checkpoint as many frames as possible without waiting for any database 
**   readers or writers to finish. Sync the db file if all frames in the log
**   are checkpointed. This mode is the same as calling 
**   sqlite3_wal_checkpoint(). The busy-handler callback is never invoked.
**
** <dt>SQLITE_CHECKPOINT_FULL<dd>
**   This mode blocks (calls the busy-handler callback) until there is no
**   database writer and all readers are reading from the most recent database
**   snapshot. It then checkpoints all frames in the log file and syncs the
**   database file. This call blocks database writers while it is running,
**   but not database readers.
**
** <dt>SQLITE_CHECKPOINT_RESTART<dd>
**   This mode works the same way as SQLITE_CHECKPOINT_FULL, except after 
**   checkpointing the log file it blocks (calls the busy-handler callback)
**   until all readers are reading from the database file only. This ensures 
**   that the next client to write to the database file restarts the log file 
**   from the beginning. This call blocks database writers while it is running,
**   but not database readers.
** </dl>
**
** If pnLog is not NULL, then *pnLog is set to the total number of frames in
** the log file before returning. If pnCkpt is not NULL, then *pnCkpt is set to
** the total number of checkpointed frames (including any that were already
** checkpointed when this function is called). *pnLog and *pnCkpt may be
** populated even if sqlite3_wal_checkpoint_v2() returns other than SQLITE_OK.
** If no values are available because of an error, they are both set to -1
** before returning to communicate this to the caller.
**
** All calls obtain an exclusive "checkpoint" lock on the database file. If
** any other process is running a checkpoint operation at the same time, the 
** lock cannot be obtained and SQLITE_BUSY is returned. Even if there is a 
** busy-handler configured, it will not be invoked in this case.
**
** The SQLITE_CHECKPOINT_FULL and RESTART modes also obtain the exclusive 
** "writer" lock on the database file. If the writer lock cannot be obtained
** immediately, and a busy-handler is configured, it is invoked and the writer
** lock retried until either the busy-handler returns 0 or the lock is
** successfully obtained. The busy-handler is also invoked while waiting for
** database readers as described above. If the busy-handler returns 0 before
** the writer lock is obtained or while waiting for database readers, the
** checkpoint operation proceeds from that point in the same way as 
** SQLITE_CHECKPOINT_PASSIVE - checkpointing as many frames as possible 
** without blocking any further. SQLITE_BUSY is returned in this case.
**
** If parameter zDb is NULL or points to a zero length string, then the
** specified operation is attempted on all WAL databases. In this case the
** values written to output parameters *pnLog and *pnCkpt are undefined. If 
** an SQLITE_BUSY error is encountered when processing one or more of the 
** attached WAL databases, the operation is still attempted on any remaining 
** attached databases and SQLITE_BUSY is returned to the caller. If any other 
** error occurs while processing an attached database, processing is abandoned 
** and the error code returned to the caller immediately. If no error 
** (SQLITE_BUSY or otherwise) is encountered while processing the attached 
** databases, SQLITE_OK is returned.
**
** If database zDb is the name of an attached database that is not in WAL
** mode, SQLITE_OK is returned and both *pnLog and *pnCkpt set to -1. If
** zDb is not NULL (or a zero length string) and is not the name of any
** attached database, SQLITE_ERROR is returned to the caller.
*/
SQLITE_API int sqlite3_wal_checkpoint_v2(
  sqlite3 *db,                    /* Database handle */
  const char *zDb,                /* Name of attached database (or NULL) */
  int eMode,                      /* SQLITE_CHECKPOINT_* value */
  int *pnLog,                     /* OUT: Size of WAL log in frames */
  int *pnCkpt                     /* OUT: Total number of frames checkpointed */
);

/*
** CAPI3REF: Checkpoint operation parameters
**
** These constants can be used as the 3rd parameter to
** [sqlite3_wal_checkpoint_v2()].  See the [sqlite3_wal_checkpoint_v2()]
** documentation for additional information about the meaning and use of
** each of these values.
*/
#define SQLITE_CHECKPOINT_PASSIVE 0
#define SQLITE_CHECKPOINT_FULL    1
#define SQLITE_CHECKPOINT_RESTART 2

/*
** CAPI3REF: Virtual Table Interface Configuration
**
** This function may be called by either the [xConnect] or [xCreate] method
** of a [virtual table] implementation to configure
** various facets of the virtual table interface.
**
** If this interface is invoked outside the context of an xConnect or
** xCreate virtual table method then the behavior is undefined.
**
** At present, there is only one option that may be configured using
** this function. (See [SQLITE_VTAB_CONSTRAINT_SUPPORT].)  Further options
** may be added in the future.
*/
SQLITE_API int sqlite3_vtab_config(sqlite3*, int op, ...);

/*
** CAPI3REF: Virtual Table Configuration Options
**
** These macros define the various options to the
** [sqlite3_vtab_config()] interface that [virtual table] implementations
** can use to customize and optimize their behavior.
**
** <dl>
** <dt>SQLITE_VTAB_CONSTRAINT_SUPPORT
** <dd>Calls of the form
** [sqlite3_vtab_config](db,SQLITE_VTAB_CONSTRAINT_SUPPORT,X) are supported,
** where X is an integer.  If X is zero, then the [virtual table] whose
** [xCreate] or [xConnect] method invoked [sqlite3_vtab_config()] does not
** support constraints.  In this configuration (which is the default) if
** a call to the [xUpdate] method returns [SQLITE_CONSTRAINT], then the entire
** statement is rolled back as if [ON CONFLICT | OR ABORT] had been
** specified as part of the users SQL statement, regardless of the actual
** ON CONFLICT mode specified.
**
** If X is non-zero, then the virtual table implementation guarantees
** that if [xUpdate] returns [SQLITE_CONSTRAINT], it will do so before
** any modifications to internal or persistent data structures have been made.
** If the [ON CONFLICT] mode is ABORT, FAIL, IGNORE or ROLLBACK, SQLite 
** is able to roll back a statement or database transaction, and abandon
** or continue processing the current SQL statement as appropriate. 
** If the ON CONFLICT mode is REPLACE and the [xUpdate] method returns
** [SQLITE_CONSTRAINT], SQLite handles this as if the ON CONFLICT mode
** had been ABORT.
**
** Virtual table implementations that are required to handle OR REPLACE
** must do so within the [xUpdate] method. If a call to the 
** [sqlite3_vtab_on_conflict()] function indicates that the current ON 
** CONFLICT policy is REPLACE, the virtual table implementation should 
** silently replace the appropriate rows within the xUpdate callback and
** return SQLITE_OK. Or, if this is not possible, it may return
** SQLITE_CONSTRAINT, in which case SQLite falls back to OR ABORT 
** constraint handling.
** </dl>
*/
#define SQLITE_VTAB_CONSTRAINT_SUPPORT 1

/*
** CAPI3REF: Determine The Virtual Table Conflict Policy
**
** This function may only be called from within a call to the [xUpdate] method
** of a [virtual table] implementation for an INSERT or UPDATE operation. ^The
** value returned is one of [SQLITE_ROLLBACK], [SQLITE_IGNORE], [SQLITE_FAIL],
** [SQLITE_ABORT], or [SQLITE_REPLACE], according to the [ON CONFLICT] mode
** of the SQL statement that triggered the call to the [xUpdate] method of the
** [virtual table].
*/
SQLITE_API int sqlite3_vtab_on_conflict(sqlite3 *);

/*
** CAPI3REF: Conflict resolution modes
**
** These constants are returned by [sqlite3_vtab_on_conflict()] to
** inform a [virtual table] implementation what the [ON CONFLICT] mode
** is for the SQL statement being evaluated.
**
** Note that the [SQLITE_IGNORE] constant is also used as a potential
** return value from the [sqlite3_set_authorizer()] callback and that
** [SQLITE_ABORT] is also a [result code].
*/
#define SQLITE_ROLLBACK 1
/* #define SQLITE_IGNORE 2 // Also used by sqlite3_authorizer() callback */
#define SQLITE_FAIL     3
/* #define SQLITE_ABORT 4  // Also an error code */
#define SQLITE_REPLACE  5



/*
** Undo the hack that converts floating point types to integer for
** builds on processors without floating point support.
*/
#ifdef SQLITE_OMIT_FLOATING_POINT
# undef double
#endif

#ifdef __cplusplus
}  /* End of the 'extern "C"' block */
#endif
#endif

/*
** 2010 August 30
**
** The author disclaims copyright to this source code.  In place of
** a legal notice, here is a blessing:
**
**    May you do good and not evil.
**    May you find forgiveness for yourself and forgive others.
**    May you share freely, never taking more than you give.
**
*************************************************************************
*/

#ifndef _SQLITE3RTREE_H_
#define _SQLITE3RTREE_H_


#ifdef __cplusplus
extern "C" {
#endif

typedef struct sqlite3_rtree_geometry sqlite3_rtree_geometry;

/*
** Register a geometry callback named zGeom that can be used as part of an
** R-Tree geometry query as follows:
**
**   SELECT ... FROM <rtree> WHERE <rtree col> MATCH $zGeom(... params ...)
*/
SQLITE_API int sqlite3_rtree_geometry_callback(
  sqlite3 *db,
  const char *zGeom,
  int (*xGeom)(sqlite3_rtree_geometry *, int nCoord, double *aCoord, int *pRes),
  void *pContext
);


/*
** A pointer to a structure of the following type is passed as the first
** argument to callbacks registered using rtree_geometry_callback().
*/
struct sqlite3_rtree_geometry {
  void *pContext;                 /* Copy of pContext passed to s_r_g_c() */
  int nParam;                     /* Size of array aParam[] */
  double *aParam;                 /* Parameters passed to SQL geom function */
  void *pUser;                    /* Callback implementation user data */
  void (*xDelUser)(void *);       /* Called by SQLite to clean up pUser */
};


#ifdef __cplusplus
}  /* end of the 'extern "C"' block */
#endif

#endif  /* ifndef _SQLITE3RTREE_H_ */

/**************** End file: sqlite3.h **********/


/**************** Begin file: sqlite3ext.h **********/
/*
** 2006 June 7
**
** The author disclaims copyright to this source code.  In place of
** a legal notice, here is a blessing:
**
**    May you do good and not evil.
**    May you find forgiveness for yourself and forgive others.
**    May you share freely, never taking more than you give.
**
*************************************************************************
** This header file defines the SQLite interface for use by
** shared libraries that want to be imported as extensions into
** an SQLite instance.  Shared libraries that intend to be loaded
** as extensions by SQLite should #include this file instead of 
** sqlite3.h.
*/
#ifndef _SQLITE3EXT_H_
#define _SQLITE3EXT_H_
/* #include "sqlite3.h" */

typedef struct sqlite3_api_routines sqlite3_api_routines;

/*
** The following structure holds pointers to all of the SQLite API
** routines.
**
** WARNING:  In order to maintain backwards compatibility, add new
** interfaces to the end of this structure only.  If you insert new
** interfaces in the middle of this structure, then older different
** versions of SQLite will not be able to load each others' shared
** libraries!
*/
struct sqlite3_api_routines {
  void * (*aggregate_context)(sqlite3_context*,int nBytes);
  int  (*aggregate_count)(sqlite3_context*);
  int  (*bind_blob)(sqlite3_stmt*,int,const void*,int n,void(*)(void*));
  int  (*bind_double)(sqlite3_stmt*,int,double);
  int  (*bind_int)(sqlite3_stmt*,int,int);
  int  (*bind_int64)(sqlite3_stmt*,int,sqlite_int64);
  int  (*bind_null)(sqlite3_stmt*,int);
  int  (*bind_parameter_count)(sqlite3_stmt*);
  int  (*bind_parameter_index)(sqlite3_stmt*,const char*zName);
  const char * (*bind_parameter_name)(sqlite3_stmt*,int);
  int  (*bind_text)(sqlite3_stmt*,int,const char*,int n,void(*)(void*));
  int  (*bind_text16)(sqlite3_stmt*,int,const void*,int,void(*)(void*));
  int  (*bind_value)(sqlite3_stmt*,int,const sqlite3_value*);
  int  (*busy_handler)(sqlite3*,int(*)(void*,int),void*);
  int  (*busy_timeout)(sqlite3*,int ms);
  int  (*changes)(sqlite3*);
  int  (*close)(sqlite3*);
  int  (*collation_needed)(sqlite3*,void*,void(*)(void*,sqlite3*,
                           int eTextRep,const char*));
  int  (*collation_needed16)(sqlite3*,void*,void(*)(void*,sqlite3*,
                             int eTextRep,const void*));
  const void * (*column_blob)(sqlite3_stmt*,int iCol);
  int  (*column_bytes)(sqlite3_stmt*,int iCol);
  int  (*column_bytes16)(sqlite3_stmt*,int iCol);
  int  (*column_count)(sqlite3_stmt*pStmt);
  const char * (*column_database_name)(sqlite3_stmt*,int);
  const void * (*column_database_name16)(sqlite3_stmt*,int);
  const char * (*column_decltype)(sqlite3_stmt*,int i);
  const void * (*column_decltype16)(sqlite3_stmt*,int);
  double  (*column_double)(sqlite3_stmt*,int iCol);
  int  (*column_int)(sqlite3_stmt*,int iCol);
  sqlite_int64  (*column_int64)(sqlite3_stmt*,int iCol);
  const char * (*column_name)(sqlite3_stmt*,int);
  const void * (*column_name16)(sqlite3_stmt*,int);
  const char * (*column_origin_name)(sqlite3_stmt*,int);
  const void * (*column_origin_name16)(sqlite3_stmt*,int);
  const char * (*column_table_name)(sqlite3_stmt*,int);
  const void * (*column_table_name16)(sqlite3_stmt*,int);
  const unsigned char * (*column_text)(sqlite3_stmt*,int iCol);
  const void * (*column_text16)(sqlite3_stmt*,int iCol);
  int  (*column_type)(sqlite3_stmt*,int iCol);
  sqlite3_value* (*column_value)(sqlite3_stmt*,int iCol);
  void * (*commit_hook)(sqlite3*,int(*)(void*),void*);
  int  (*complete)(const char*sql);
  int  (*complete16)(const void*sql);
  int  (*create_collation)(sqlite3*,const char*,int,void*,
                           int(*)(void*,int,const void*,int,const void*));
  int  (*create_collation16)(sqlite3*,const void*,int,void*,
                             int(*)(void*,int,const void*,int,const void*));
  int  (*create_function)(sqlite3*,const char*,int,int,void*,
                          void (*xFunc)(sqlite3_context*,int,sqlite3_value**),
                          void (*xStep)(sqlite3_context*,int,sqlite3_value**),
                          void (*xFinal)(sqlite3_context*));
  int  (*create_function16)(sqlite3*,const void*,int,int,void*,
                            void (*xFunc)(sqlite3_context*,int,sqlite3_value**),
                            void (*xStep)(sqlite3_context*,int,sqlite3_value**),
                            void (*xFinal)(sqlite3_context*));
  int (*create_module)(sqlite3*,const char*,const sqlite3_module*,void*);
  int  (*data_count)(sqlite3_stmt*pStmt);
  sqlite3 * (*db_handle)(sqlite3_stmt*);
  int (*declare_vtab)(sqlite3*,const char*);
  int  (*enable_shared_cache)(int);
  int  (*errcode)(sqlite3*db);
  const char * (*errmsg)(sqlite3*);
  const void * (*errmsg16)(sqlite3*);
  int  (*exec)(sqlite3*,const char*,sqlite3_callback,void*,char**);
  int  (*expired)(sqlite3_stmt*);
  int  (*finalize)(sqlite3_stmt*pStmt);
  void  (*free)(void*);
  void  (*free_table)(char**result);
  int  (*get_autocommit)(sqlite3*);
  void * (*get_auxdata)(sqlite3_context*,int);
  int  (*get_table)(sqlite3*,const char*,char***,int*,int*,char**);
  int  (*global_recover)(void);
  void  (*interruptx)(sqlite3*);
  sqlite_int64  (*last_insert_rowid)(sqlite3*);
  const char * (*libversion)(void);
  int  (*libversion_number)(void);
  void *(*malloc)(int);
  char * (*mprintf)(const char*,...);
  int  (*open)(const char*,sqlite3**);
  int  (*open16)(const void*,sqlite3**);
  int  (*prepare)(sqlite3*,const char*,int,sqlite3_stmt**,const char**);
  int  (*prepare16)(sqlite3*,const void*,int,sqlite3_stmt**,const void**);
  void * (*profile)(sqlite3*,void(*)(void*,const char*,sqlite_uint64),void*);
  void  (*progress_handler)(sqlite3*,int,int(*)(void*),void*);
  void *(*realloc)(void*,int);
  int  (*reset)(sqlite3_stmt*pStmt);
  void  (*result_blob)(sqlite3_context*,const void*,int,void(*)(void*));
  void  (*result_double)(sqlite3_context*,double);
  void  (*result_error)(sqlite3_context*,const char*,int);
  void  (*result_error16)(sqlite3_context*,const void*,int);
  void  (*result_int)(sqlite3_context*,int);
  void  (*result_int64)(sqlite3_context*,sqlite_int64);
  void  (*result_null)(sqlite3_context*);
  void  (*result_text)(sqlite3_context*,const char*,int,void(*)(void*));
  void  (*result_text16)(sqlite3_context*,const void*,int,void(*)(void*));
  void  (*result_text16be)(sqlite3_context*,const void*,int,void(*)(void*));
  void  (*result_text16le)(sqlite3_context*,const void*,int,void(*)(void*));
  void  (*result_value)(sqlite3_context*,sqlite3_value*);
  void * (*rollback_hook)(sqlite3*,void(*)(void*),void*);
  int  (*set_authorizer)(sqlite3*,int(*)(void*,int,const char*,const char*,
                         const char*,const char*),void*);
  void  (*set_auxdata)(sqlite3_context*,int,void*,void (*)(void*));
  char * (*snprintf)(int,char*,const char*,...);
  int  (*step)(sqlite3_stmt*);
  int  (*table_column_metadata)(sqlite3*,const char*,const char*,const char*,
                                char const**,char const**,int*,int*,int*);
  void  (*thread_cleanup)(void);
  int  (*total_changes)(sqlite3*);
  void * (*trace)(sqlite3*,void(*xTrace)(void*,const char*),void*);
  int  (*transfer_bindings)(sqlite3_stmt*,sqlite3_stmt*);
  void * (*update_hook)(sqlite3*,void(*)(void*,int ,char const*,char const*,
                                         sqlite_int64),void*);
  void * (*user_data)(sqlite3_context*);
  const void * (*value_blob)(sqlite3_value*);
  int  (*value_bytes)(sqlite3_value*);
  int  (*value_bytes16)(sqlite3_value*);
  double  (*value_double)(sqlite3_value*);
  int  (*value_int)(sqlite3_value*);
  sqlite_int64  (*value_int64)(sqlite3_value*);
  int  (*value_numeric_type)(sqlite3_value*);
  const unsigned char * (*value_text)(sqlite3_value*);
  const void * (*value_text16)(sqlite3_value*);
  const void * (*value_text16be)(sqlite3_value*);
  const void * (*value_text16le)(sqlite3_value*);
  int  (*value_type)(sqlite3_value*);
  char *(*vmprintf)(const char*,va_list);
  /* Added ??? */
  int (*overload_function)(sqlite3*, const char *zFuncName, int nArg);
  /* Added by 3.3.13 */
  int (*prepare_v2)(sqlite3*,const char*,int,sqlite3_stmt**,const char**);
  int (*prepare16_v2)(sqlite3*,const void*,int,sqlite3_stmt**,const void**);
  int (*clear_bindings)(sqlite3_stmt*);
  /* Added by 3.4.1 */
  int (*create_module_v2)(sqlite3*,const char*,const sqlite3_module*,void*,
                          void (*xDestroy)(void *));
  /* Added by 3.5.0 */
  int (*bind_zeroblob)(sqlite3_stmt*,int,int);
  int (*blob_bytes)(sqlite3_blob*);
  int (*blob_close)(sqlite3_blob*);
  int (*blob_open)(sqlite3*,const char*,const char*,const char*,sqlite3_int64,
                   int,sqlite3_blob**);
  int (*blob_read)(sqlite3_blob*,void*,int,int);
  int (*blob_write)(sqlite3_blob*,const void*,int,int);
  int (*create_collation_v2)(sqlite3*,const char*,int,void*,
                             int(*)(void*,int,const void*,int,const void*),
                             void(*)(void*));
  int (*file_control)(sqlite3*,const char*,int,void*);
  sqlite3_int64 (*memory_highwater)(int);
  sqlite3_int64 (*memory_used)(void);
  sqlite3_mutex *(*mutex_alloc)(int);
  void (*mutex_enter)(sqlite3_mutex*);
  void (*mutex_free)(sqlite3_mutex*);
  void (*mutex_leave)(sqlite3_mutex*);
  int (*mutex_try)(sqlite3_mutex*);
  int (*open_v2)(const char*,sqlite3**,int,const char*);
  int (*release_memory)(int);
  void (*result_error_nomem)(sqlite3_context*);
  void (*result_error_toobig)(sqlite3_context*);
  int (*sleep)(int);
  void (*soft_heap_limit)(int);
  sqlite3_vfs *(*vfs_find)(const char*);
  int (*vfs_register)(sqlite3_vfs*,int);
  int (*vfs_unregister)(sqlite3_vfs*);
  int (*xthreadsafe)(void);
  void (*result_zeroblob)(sqlite3_context*,int);
  void (*result_error_code)(sqlite3_context*,int);
  int (*test_control)(int, ...);
  void (*randomness)(int,void*);
  sqlite3 *(*context_db_handle)(sqlite3_context*);
  int (*extended_result_codes)(sqlite3*,int);
  int (*limit)(sqlite3*,int,int);
  sqlite3_stmt *(*next_stmt)(sqlite3*,sqlite3_stmt*);
  const char *(*sql)(sqlite3_stmt*);
  int (*status)(int,int*,int*,int);
  int (*backup_finish)(sqlite3_backup*);
  sqlite3_backup *(*backup_init)(sqlite3*,const char*,sqlite3*,const char*);
  int (*backup_pagecount)(sqlite3_backup*);
  int (*backup_remaining)(sqlite3_backup*);
  int (*backup_step)(sqlite3_backup*,int);
  const char *(*compileoption_get)(int);
  int (*compileoption_used)(const char*);
  int (*create_function_v2)(sqlite3*,const char*,int,int,void*,
                            void (*xFunc)(sqlite3_context*,int,sqlite3_value**),
                            void (*xStep)(sqlite3_context*,int,sqlite3_value**),
                            void (*xFinal)(sqlite3_context*),
                            void(*xDestroy)(void*));
  int (*db_config)(sqlite3*,int,...);
  sqlite3_mutex *(*db_mutex)(sqlite3*);
  int (*db_status)(sqlite3*,int,int*,int*,int);
  int (*extended_errcode)(sqlite3*);
  void (*log)(int,const char*,...);
  sqlite3_int64 (*soft_heap_limit64)(sqlite3_int64);
  const char *(*sourceid)(void);
  int (*stmt_status)(sqlite3_stmt*,int,int);
  int (*strnicmp)(const char*,const char*,int);
  int (*unlock_notify)(sqlite3*,void(*)(void**,int),void*);
  int (*wal_autocheckpoint)(sqlite3*,int);
  int (*wal_checkpoint)(sqlite3*,const char*);
  void *(*wal_hook)(sqlite3*,int(*)(void*,sqlite3*,const char*,int),void*);
  int (*blob_reopen)(sqlite3_blob*,sqlite3_int64);
  int (*vtab_config)(sqlite3*,int op,...);
  int (*vtab_on_conflict)(sqlite3*);
};

/*
** The following macros redefine the API routines so that they are
** redirected throught the global sqlite3_api structure.
**
** This header file is also used by the loadext.c source file
** (part of the main SQLite library - not an extension) so that
** it can get access to the sqlite3_api_routines structure
** definition.  But the main library does not want to redefine
** the API.  So the redefinition macros are only valid if the
** SQLITE_CORE macros is undefined.
*/
#ifndef SQLITE_CORE
#define SPLite3_aggregate_context      sqlite3_api->aggregate_context
#ifndef SQLITE_OMIT_DEPRECATED
#define SPLite3_aggregate_count        sqlite3_api->aggregate_count
#endif
#define SPLite3_bind_blob              sqlite3_api->bind_blob
#define SPLite3_bind_double            sqlite3_api->bind_double
#define SPLite3_bind_int               sqlite3_api->bind_int
#define SPLite3_bind_int64             sqlite3_api->bind_int64
#define SPLite3_bind_null              sqlite3_api->bind_null
#define SPLite3_bind_parameter_count   sqlite3_api->bind_parameter_count
#define SPLite3_bind_parameter_index   sqlite3_api->bind_parameter_index
#define SPLite3_bind_parameter_name    sqlite3_api->bind_parameter_name
#define SPLite3_bind_text              sqlite3_api->bind_text
#define SPLite3_bind_text16            sqlite3_api->bind_text16
#define SPLite3_bind_value             sqlite3_api->bind_value
#define SPLite3_busy_handler           sqlite3_api->busy_handler
#define SPLite3_busy_timeout           sqlite3_api->busy_timeout
#define SPLite3_changes                sqlite3_api->changes
#define SPLite3_close                  sqlite3_api->close
#define SPLite3_collation_needed       sqlite3_api->collation_needed
#define SPLite3_collation_needed16     sqlite3_api->collation_needed16
#define SPLite3_column_blob            sqlite3_api->column_blob
#define SPLite3_column_bytes           sqlite3_api->column_bytes
#define SPLite3_column_bytes16         sqlite3_api->column_bytes16
#define SPLite3_column_count           sqlite3_api->column_count
#define SPLite3_column_database_name   sqlite3_api->column_database_name
#define SPLite3_column_database_name16 sqlite3_api->column_database_name16
#define SPLite3_column_decltype        sqlite3_api->column_decltype
#define SPLite3_column_decltype16      sqlite3_api->column_decltype16
#define SPLite3_column_double          sqlite3_api->column_double
#define SPLite3_column_int             sqlite3_api->column_int
#define SPLite3_column_int64           sqlite3_api->column_int64
#define SPLite3_column_name            sqlite3_api->column_name
#define SPLite3_column_name16          sqlite3_api->column_name16
#define SPLite3_column_origin_name     sqlite3_api->column_origin_name
#define SPLite3_column_origin_name16   sqlite3_api->column_origin_name16
#define SPLite3_column_table_name      sqlite3_api->column_table_name
#define SPLite3_column_table_name16    sqlite3_api->column_table_name16
#define SPLite3_column_text            sqlite3_api->column_text
#define SPLite3_column_text16          sqlite3_api->column_text16
#define SPLite3_column_type            sqlite3_api->column_type
#define SPLite3_column_value           sqlite3_api->column_value
#define SPLite3_commit_hook            sqlite3_api->commit_hook
#define SPLite3_complete               sqlite3_api->complete
#define SPLite3_complete16             sqlite3_api->complete16
#define SPLite3_create_collation       sqlite3_api->create_collation
#define SPLite3_create_collation16     sqlite3_api->create_collation16
#define SPLite3_create_function        sqlite3_api->create_function
#define SPLite3_create_function16      sqlite3_api->create_function16
#define SPLite3_create_module          sqlite3_api->create_module
#define SPLite3_create_module_v2       sqlite3_api->create_module_v2
#define SPLite3_data_count             sqlite3_api->data_count
#define SPLite3_db_handle              sqlite3_api->db_handle
#define SPLite3_declare_vtab           sqlite3_api->declare_vtab
#define SPLite3_enable_shared_cache    sqlite3_api->enable_shared_cache
#define SPLite3_errcode                sqlite3_api->errcode
#define SPLite3_errmsg                 sqlite3_api->errmsg
#define SPLite3_errmsg16               sqlite3_api->errmsg16
#define SPLite3_exec                   sqlite3_api->exec
#ifndef SQLITE_OMIT_DEPRECATED
#define SPLite3_expired                sqlite3_api->expired
#endif
#define SPLite3_finalize               sqlite3_api->finalize
#define SPLite3_free                   sqlite3_api->free
#define SPLite3_free_table             sqlite3_api->free_table
#define SPLite3_get_autocommit         sqlite3_api->get_autocommit
#define SPLite3_get_auxdata            sqlite3_api->get_auxdata
#define SPLite3_get_table              sqlite3_api->get_table
#ifndef SQLITE_OMIT_DEPRECATED
#define SPLite3_global_recover         sqlite3_api->global_recover
#endif
#define SPLite3_interrupt              sqlite3_api->interruptx
#define SPLite3_last_insert_rowid      sqlite3_api->last_insert_rowid
#define SPLite3_libversion             sqlite3_api->libversion
#define SPLite3_libversion_number      sqlite3_api->libversion_number
#define SPLite3_malloc                 sqlite3_api->malloc
#define SPLite3_mprintf                sqlite3_api->mprintf
#define SPLite3_open                   sqlite3_api->open
#define SPLite3_open16                 sqlite3_api->open16
#define SPLite3_prepare                sqlite3_api->prepare
#define SPLite3_prepare16              sqlite3_api->prepare16
#define SPLite3_prepare_v2             sqlite3_api->prepare_v2
#define SPLite3_prepare16_v2           sqlite3_api->prepare16_v2
#define SPLite3_profile                sqlite3_api->profile
#define SPLite3_progress_handler       sqlite3_api->progress_handler
#define SPLite3_realloc                sqlite3_api->realloc
#define SPLite3_reset                  sqlite3_api->reset
#define SPLite3_result_blob            sqlite3_api->result_blob
#define SPLite3_result_double          sqlite3_api->result_double
#define SPLite3_result_error           sqlite3_api->result_error
#define SPLite3_result_error16         sqlite3_api->result_error16
#define SPLite3_result_int             sqlite3_api->result_int
#define SPLite3_result_int64           sqlite3_api->result_int64
#define SPLite3_result_null            sqlite3_api->result_null
#define SPLite3_result_text            sqlite3_api->result_text
#define SPLite3_result_text16          sqlite3_api->result_text16
#define SPLite3_result_text16be        sqlite3_api->result_text16be
#define SPLite3_result_text16le        sqlite3_api->result_text16le
#define SPLite3_result_value           sqlite3_api->result_value
#define SPLite3_rollback_hook          sqlite3_api->rollback_hook
#define SPLite3_set_authorizer         sqlite3_api->set_authorizer
#define SPLite3_set_auxdata            sqlite3_api->set_auxdata
#define SPLite3_snprintf               sqlite3_api->snprintf
#define SPLite3_step                   sqlite3_api->step
#define SPLite3_table_column_metadata  sqlite3_api->table_column_metadata
#define SPLite3_thread_cleanup         sqlite3_api->thread_cleanup
#define SPLite3_total_changes          sqlite3_api->total_changes
#define SPLite3_trace                  sqlite3_api->trace
#ifndef SQLITE_OMIT_DEPRECATED
#define SPLite3_transfer_bindings      sqlite3_api->transfer_bindings
#endif
#define SPLite3_update_hook            sqlite3_api->update_hook
#define SPLite3_user_data              sqlite3_api->user_data
#define SPLite3_value_blob             sqlite3_api->value_blob
#define SPLite3_value_bytes            sqlite3_api->value_bytes
#define SPLite3_value_bytes16          sqlite3_api->value_bytes16
#define SPLite3_value_double           sqlite3_api->value_double
#define SPLite3_value_int              sqlite3_api->value_int
#define SPLite3_value_int64            sqlite3_api->value_int64
#define SPLite3_value_numeric_type     sqlite3_api->value_numeric_type
#define SPLite3_value_text             sqlite3_api->value_text
#define SPLite3_value_text16           sqlite3_api->value_text16
#define SPLite3_value_text16be         sqlite3_api->value_text16be
#define SPLite3_value_text16le         sqlite3_api->value_text16le
#define SPLite3_value_type             sqlite3_api->value_type
#define SPLite3_vmprintf               sqlite3_api->vmprintf
#define SPLite3_overload_function      sqlite3_api->overload_function
#define SPLite3_prepare_v2             sqlite3_api->prepare_v2
#define SPLite3_prepare16_v2           sqlite3_api->prepare16_v2
#define SPLite3_clear_bindings         sqlite3_api->clear_bindings
#define SPLite3_bind_zeroblob          sqlite3_api->bind_zeroblob
#define SPLite3_blob_bytes             sqlite3_api->blob_bytes
#define SPLite3_blob_close             sqlite3_api->blob_close
#define SPLite3_blob_open              sqlite3_api->blob_open
#define SPLite3_blob_read              sqlite3_api->blob_read
#define SPLite3_blob_write             sqlite3_api->blob_write
#define SPLite3_create_collation_v2    sqlite3_api->create_collation_v2
#define SPLite3_file_control           sqlite3_api->file_control
#define SPLite3_memory_highwater       sqlite3_api->memory_highwater
#define SPLite3_memory_used            sqlite3_api->memory_used
#define SPLite3_mutex_alloc            sqlite3_api->mutex_alloc
#define SPLite3_mutex_enter            sqlite3_api->mutex_enter
#define SPLite3_mutex_free             sqlite3_api->mutex_free
#define SPLite3_mutex_leave            sqlite3_api->mutex_leave
#define SPLite3_mutex_try              sqlite3_api->mutex_try
#define SPLite3_open_v2                sqlite3_api->open_v2
#define SPLite3_release_memory         sqlite3_api->release_memory
#define SPLite3_result_error_nomem     sqlite3_api->result_error_nomem
#define SPLite3_result_error_toobig    sqlite3_api->result_error_toobig
#define SPLite3_sleep                  sqlite3_api->sleep
#define SPLite3_soft_heap_limit        sqlite3_api->soft_heap_limit
#define SPLite3_vfs_find               sqlite3_api->vfs_find
#define SPLite3_vfs_register           sqlite3_api->vfs_register
#define SPLite3_vfs_unregister         sqlite3_api->vfs_unregister
#define SPLite3_threadsafe             sqlite3_api->xthreadsafe
#define SPLite3_result_zeroblob        sqlite3_api->result_zeroblob
#define SPLite3_result_error_code      sqlite3_api->result_error_code
#define SPLite3_test_control           sqlite3_api->test_control
#define SPLite3_randomness             sqlite3_api->randomness
#define SPLite3_context_db_handle      sqlite3_api->context_db_handle
#define SPLite3_extended_result_codes  sqlite3_api->extended_result_codes
#define SPLite3_limit                  sqlite3_api->limit
#define SPLite3_next_stmt              sqlite3_api->next_stmt
#define SPLite3_sql                    sqlite3_api->sql
#define SPLite3_status                 sqlite3_api->status
#define SPLite3_backup_finish          sqlite3_api->backup_finish
#define SPLite3_backup_init            sqlite3_api->backup_init
#define SPLite3_backup_pagecount       sqlite3_api->backup_pagecount
#define SPLite3_backup_remaining       sqlite3_api->backup_remaining
#define SPLite3_backup_step            sqlite3_api->backup_step
#define SPLite3_compileoption_get      sqlite3_api->compileoption_get
#define SPLite3_compileoption_used     sqlite3_api->compileoption_used
#define SPLite3_create_function_v2     sqlite3_api->create_function_v2
#define SPLite3_db_config              sqlite3_api->db_config
#define SPLite3_db_mutex               sqlite3_api->db_mutex
#define SPLite3_db_status              sqlite3_api->db_status
#define SPLite3_extended_errcode       sqlite3_api->extended_errcode
#define SPLite3_log                    sqlite3_api->log
#define SPLite3_soft_heap_limit64      sqlite3_api->soft_heap_limit64
#define SPLite3_sourceid               sqlite3_api->sourceid
#define SPLite3_stmt_status            sqlite3_api->stmt_status
#define SPLite3_strnicmp               sqlite3_api->strnicmp
#define SPLite3_unlock_notify          sqlite3_api->unlock_notify
#define SPLite3_wal_autocheckpoint     sqlite3_api->wal_autocheckpoint
#define SPLite3_wal_checkpoint         sqlite3_api->wal_checkpoint
#define SPLite3_wal_hook               sqlite3_api->wal_hook
#define SPLite3_blob_reopen            sqlite3_api->blob_reopen
#define SPLite3_vtab_config            sqlite3_api->vtab_config
#define SPLite3_vtab_on_conflict       sqlite3_api->vtab_on_conflict
#endif /* SQLITE_CORE */

#define SQLITE_EXTENSION_INIT1     const sqlite3_api_routines *sqlite3_api = 0;
#define SQLITE_EXTENSION_INIT2(v)  sqlite3_api = v;

#endif /* _SQLITE3EXT_H_ */
/**************** End file: sqlite3ext.h **********/


/**************** Begin file: spatialite.h **********/

/**
 \file spatialite.h

 Main SpatiaLite header file
 */
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifdef DLL_EXPORT
#define SPATIALITE_DECLARE __declspec(dllexport)
#else
#define SPATIALITE_DECLARE extern
#endif
#endif

#ifndef _SPATIALITE_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _SPATIALITE_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/**
 Return the current library version.
 */
    SPATIALITE_DECLARE const char *spatialite_version (void);

/**
 Initializes the library. 

 \param verbose if TRUE a short start-up message is shown on stderr

 \note You absolutely must invoke this function before attempting to perform
 any other SpatiaLite's call.

 */
    SPATIALITE_DECLARE void spatialite_init (int verbose);

    /**
     Cleanup spatialite 
     
     This function performs general cleanup, essentially undoing the effect
     of spatialite_init().
     
     \sa spatialite_init
    */
    SPATIALITE_DECLARE void spatialite_cleanup ();

/**
 Dumps a full geometry-table into an external Shapefile

 \param sqlite handle to current DB connection
 \param table the name of the table to be exported
 \param column the name of the geometry column
 \param shp_path pathname of the Shapefile to be exported (no suffix) 
 \param charset a valid GNU ICONV charset to be used for DBF text strings
 \param geom_type "POINT", "LINESTRING", "POLYGON", "MULTIPOLYGON" or NULL
 \param verbose if TRUE a short report is shown on stderr
 \param rows on completion will contain the total number of actually exported rows
 \param err_msg on completion will contain an error message (if any)

 \return 0 on failure, any other value on success
 */
    SPATIALITE_DECLARE int dump_shapefile (sqlite3 * sqlite, char *table,
					   char *column, char *shp_path,
					   char *charset, char *geom_type,
					   int verbose, int *rows,
					   char *err_msg);

/**
 Loads an external Shapefile into a newly created table

 \param sqlite handle to current DB connection
 \param shp_path pathname of the Shapefile to be imported (no suffix) 
 \param table the name of the table to be created
 \param charset a valid GNU ICONV charset to be used for DBF text strings
 \param srid the SRID to be set for Geometries
 \param column the name of the geometry column
 \param coerce2d if TRUE any Geometry will be casted to 2D [XY]
 \param compressed if TRUE compressed Geometries will be created
 \param verbose if TRUE a short report is shown on stderr
 \param spatial_index if TRUE an R*Tree Spatial Index will be created
 \param rows on completion will contain the total number of actually exported rows
 \param err_msg on completion will contain an error message (if any)

 \return 0 on failure, any other value on success
 */
    SPATIALITE_DECLARE int load_shapefile (sqlite3 * sqlite, char *shp_path,
					   char *table, char *charset, int srid,
					   char *column, int coerce2d,
					   int compressed, int verbose,
					   int spatial_index, int *rows,
					   char *err_msg);

/**
 Loads an external DBF file into a newly created table

 \param sqlite handle to current DB connection
 \param dbf_path pathname of the DBF file to be imported
 \param table the name of the table to be created
 \param charset a valid GNU ICONV charset to be used for DBF text strings
 \param verbose if TRUE a short report is shown on stderr
 \param rows on completion will contain the total number of actually exported rows
 \param err_msg on completion will contain an error message (if any)

 \return 0 on failure, any other value on success
 */
    SPATIALITE_DECLARE int load_dbf (sqlite3 * sqlite, char *dbf_path,
				     char *table, char *charset, int verbose,
				     int *rows, char *err_msg);

/**
 Dumps a full table into an external DBF file

 \param sqlite handle to current DB connection
 \param table the name of the table to be exported
 \param dbf_path pathname of the DBF to be exported 
 \param charset a valid GNU ICONV charset to be used for DBF text strings
 \param err_msg on completion will contain an error message (if any)

 \return 0 on failure, any other value on success
 */
    SPATIALITE_DECLARE int dump_dbf (sqlite3 * sqlite, char *table,
				     char *dbf_path, char *charset,
				     char *err_msg);

/**
 Loads an external spreadsheet (.xls) file into a newly created table

 \param sqlite handle to current DB connection
 \param path pathname of the spreadsheet file to be imported
 \param table the name of the table to be created
 \param worksheetIndex the index identifying the worksheet to be imported
 \param first_titles if TRUE the first line is assumed to contain column names
 \param rows on completion will contain the total number of actually exported rows
 \param err_msg on completion will contain an error message (if any)

 \return 0 on failure, any other value on success
 */
    SPATIALITE_DECLARE int load_XL (sqlite3 * sqlite, const char *path,
				    const char *table,
				    unsigned int worksheetIndex,
				    int first_titles, unsigned int *rows,
				    char *err_msg);

/**
 A portable replacement for C99 round()

 \param value a double value

 \return the nearest integeral value
 */
    SPATIALITE_DECLARE double math_round (double value);

/**
 A portable replacement for C99 llabs()

 \param value a 64 bit integer value

 \return the corresponding absolute value
 */
    SPATIALITE_DECLARE sqlite3_int64 math_llabs (sqlite3_int64 value);

/**
 Inserts the inlined EPSG dataset into the "spatial_ref_sys" table

 \param sqlite handle to current DB connection
 \param verbose if TRUE a short report is shown on stderr

 \return 0 on failure, any other value on success

 \note this function is internally invoked by the SQL function 
  InitSpatialMetadata(), and is not usually intended for direct use.
 */
    SPATIALITE_DECLARE int spatial_ref_sys_init (sqlite3 * sqlite, int verbose);

/**
 Checks if a column is actually defined into the given table

 \param sqlite handle to current DB connection
 \param table the table to be checked
 \param column the column to be checked

 \return 0 on success, any other value on success

 \note internally used to detect if some KML attribute defaults to a constant value
 */
    SPATIALITE_DECLARE int
	is_kml_constant (sqlite3 * sqlite, char *table, char *column);

/**
 Dumps a full geometry-table into an external KML file

 \param sqlite handle to current DB connection
 \param table the name of the table to be exported
 \param geom_col the name of the geometry column
 \param kml_path pathname of the KML file to be exported 
 \param name_col column to be used for KML "name" (may be null)
 \param desc_col column to be used for KML "description" (may be null)
 \param precision number of decimal digits for coordinates

 \return 0 on failure, any other value on success
 */
    SPATIALITE_DECLARE int dump_kml (sqlite3 * sqlite, char *table,
				     char *geom_col, char *kml_path,
				     char *name_col, char *desc_col,
				     int precision);

/**
 Checks for duplicated rows into the same table

 \param sqlite handle to current DB connection
 \param table name of the table to be checked
 \param dupl_count on completion will contain the number of duplicated rows found

 \sa remove_duplicated_rows
 \note two (or more) rows are assumed to be duplicated if any column

 value (excluding any Primary Key column) is exacly the same
 */
    SPATIALITE_DECLARE void check_duplicated_rows (sqlite3 * sqlite,
						   char *table,
						   int *dupl_count);

/**
 Remove duplicated rows from a table

 \param sqlite handle to current DB connection
 \param table name of the table to be cleaned

 \sa check_duplicated_rows

 \note when two (or more) duplicated rows exist, only the first occurence
 will be preserved, then deleting any further occurrence.
 */
    SPATIALITE_DECLARE void remove_duplicated_rows (sqlite3 * sqlite,
						    char *table);

/**
 Creates a derived table surely containing elementary Geometries

 \param sqlite handle to current DB connection
 \param inTable name of the input table 
 \param geometry name of the Geometry column
 \param outTable name of the output table to be created
 \param pKey name of the Primary Key column in the output table
 \param multiId name of the column identifying origins in the output table

 \note if the input table contains some kind of complex Geometry
 (MULTIPOINT, MULTILINESTRING, MULTIPOLYGON or GEOMETRYCOLLECTION),
 then many rows are inserted into the output table: each single 
 row will contain the same attributes and an elementaty Geometry.
 All the rows created by expanding the same input row will expose
 the same value in the "multiId" column.
 */
    SPATIALITE_DECLARE void elementary_geometries (sqlite3 * sqlite,
						   char *inTable,
						   char *geometry,
						   char *outTable, char *pKey,
						   char *multiId);

/**
 Dumps a full geometry-table into an external GeoJSON file

 \param sqlite handle to current DB connection
 \param table the name of the table to be exported
 \param geom_col the name of the geometry column
 \param outfile_path pathname for the GeoJSON file to be written to
 \param precision number of decimal digits for coordinates
 \param option the format to use for output

 \note valid values for option are:
   - 0 no option
   - 1 GeoJSON MBR
   - 2 GeoJSON Short CRS (e.g EPSG:4326)
   - 3 MBR + Short CRS
   - 4 GeoJSON Long CRS (e.g urn:ogc:def:crs:EPSG::4326)
   - 5 MBR + Long CRS

 \return 0 on failure, any other value on success
 */
    SPATIALITE_DECLARE int dump_geojson (sqlite3 * sqlite, char *table,
					 char *geom_col, char *outfile_path,
					 int precision, int option);

/**
 Updates the LAYER_STATICS metadata table

 \param sqlite handle to current DB connection
 \param table name of the table to be processed
 \param column name of the geometry to be processed

 \note this function will explore the given table/geometry determining
 the number of rows and the full layer extent; a corresponding table/geometry
 entry is expected to be already declared in the GEOMETRY_COLUMNS table.
 These informations will be permanently stored into the LAYER_STATISTICS
 table; if such table does not yet exists will be implicitly created.
   - if table is NULL, any entry found within GEOMETRY_COLUMNS
     will be processed.
   - if table is not NULL and column is NULL, any geometry
     belonging to the given table will be processed.
   - if both table and column are not NULL, then only the
     given entry will be processed.

 \return 0 on failure, the total count of processed entries on success
 */
    SPATIALITE_DECLARE int update_layer_statistics (sqlite3 * sqlite,
						    const char *table,
						    const char *column);

#ifdef __cplusplus
}
#endif

#endif				/* _SPATIALITE_H */
/**************** End file: spatialite.h **********/


/**************** Begin file: gaiaaux.h **********/

/**
 \file gaiaaux.h

 Auxiliary/helper functions
 */
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifdef DLL_EXPORT
#define GAIAAUX_DECLARE __declspec(dllexport)
#else
#define GAIAAUX_DECLARE extern
#endif
#endif

#ifndef _GAIAAUX_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GAIAAUX_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* constants */
/** SQL single quoted string (text constant) */
#define GAIA_SQL_SINGLE_QUOTE	1001
/** SQL double quoted string (SQL name) */
#define GAIA_SQL_DOUBLE_QUOTE	1002

/* function prototipes */

/**
 Retrieves the Locale Charset

 \return the GNU ICONV name identifying the locale charset
 */
    GAIAAUX_DECLARE const char *gaiaGetLocaleCharset (void);

/**
 Converts a text string from one charset to another

 \param buf the text string to be converted
 \param fromCs the GNU ICONV name identifying the input charset
 \param toCs the GNU ICONV name identifying the output charset

 \return 0 on failure, any other value on success.

 \note this function uses an internal buffer limited to 64KB;
 so it's not safe passing extremely huge-sized text string.
 */
    GAIAAUX_DECLARE int gaiaConvertCharset (char **buf, const char *fromCs,
					    const char *toCs);

/**
 Creates a persistent UTF8 converter object

 \param fromCS the GNU ICONV name identifying the input charset

 \return the handle of the converter object, or NULL on failure

 \sa gaiaFreeUTF8Converter

 \note you must properly destroy the converter object 
 when it isn't any longer used.
 */
    GAIAAUX_DECLARE void *gaiaCreateUTF8Converter (const char *fromCS);

/**
 Destroys an UTF8 converter object

 \param cvtCS the handle identifying the UTF8 convert object  
 (returned by a previous call to gaiaCreateUTF8Converter).

 \sa gaiaCreateUTF8Converter
 */
    GAIAAUX_DECLARE void gaiaFreeUTF8Converter (void *cvtCS);

/**
 Converts a text string to UTF8

 \param cvtCS the handle identifying the UTF8 convert object  
 (returned by a previous call to gaiaCreateUTF8Converter).
 \param buf the input text string
 \param len length (in bytes) of input string
 \param err on completion will contain 0 on success, any other value on failure

 \return the null-terminated UTF8 encoded string: NULL on failure

 \sa gaiaCreateUTF8Converter, gaiaFreeUTF8Converter
 
 \note this function can safely handle strings of arbitrary length,
 and will return the converted string into a dynamically allocated buffer 
 created by malloc(). 
 You are required to explicitly free() any string returned by this function.
 */
    GAIAAUX_DECLARE char *gaiaConvertToUTF8 (void *cvtCS, const char *buf,
					     int len, int *err);

/**
 Checks if a name is a reserved SQLite name

 \param name the name to be checked

 \return 0 if no: any other value if yes

 \sa gaiaIsReservedSqlName, gaiaIllegalSqlName
 */
    GAIAAUX_DECLARE int gaiaIsReservedSqliteName (const char *name);

/**
 Checks if a name is a reserved SQL name

 \param name the name to be checked

 \return 0 if no: any other value if yes

 \sa gaiaIsReservedSqliteName, gaiaIllegalSqlName
 */
    GAIAAUX_DECLARE int gaiaIsReservedSqlName (const char *name);

/**
 Checks if a name is an illegal SQL name

 \param name the name to be checked

 \return 0 if no: any other value if yes

 \sa gaiaIsReservedSqliteName, gaiaIsReservedSqlName
 */
    GAIAAUX_DECLARE int gaiaIllegalSqlName (const char *name);

/**
 Properly formats an SQL text constant

 \param value the text string to be formatted

 \return the formatted string: NULL on failure

 \sa gaiaQuotedSql
 
 \note this function simply is a convenience method corresponding to: 
 gaiaQuotedSQL(value, GAIA_SQL_SINGLE_QUOTE);
 
 \remark passing a string like "Sant'Andrea" will return 'Sant''Andrea'
 */
    GAIAAUX_DECLARE char *gaiaSingleQuotedSql (const char *value);

/**
 Properly formats an SQL name

 \param value the SQL name to be formatted

 \return the formatted string: NULL on failure

 \sa gaiaQuotedSql
 
 \note this function simply is a convenience method corresponding to: 
 gaiaQuotedSQL(value, GAIA_SQL_DOUBLE_QUOTE);

 \remark passing a string like "Sant\"Andrea" will return "Sant""Andrea"
 */
    GAIAAUX_DECLARE char *gaiaDoubleQuotedSql (const char *value);

/**
 Properly formats an SQL generic string

 \param value the string to be formatted
 \param quote GAIA_SQL_SINGLE_QUOTE or GAIA_SQL_DOUBLE_QUOTE

 \return the formatted string: NULL on failure

 \sa gaiaSingleQuotedSql, gaiaDoubleQuotedSql

 \note this function can safely handle strings of arbitrary length,
 and will return the formatted string into a dynamically allocated buffer 
 created by malloc(). 
 You are required to explicitly free() any string returned by this function.
 */
    GAIAAUX_DECLARE char *gaiaQuotedSql (const char *value, int quote);

/*
/ DEPRECATED FUNCTION: gaiaCleanSqlString()
/ this function must not be used for any new project
/ it's still maintained for backward compatibility,
/ but will be probably removed in future versions
*/

/**
 deprecated function

 \param value the string to be formatted

 \sa gaiaQuotedSql

 \note this function is still supported simply for backward compatibility.
 it's intrinsically unsafe (passing huge strings potentially leads to 
 buffer overflows) and you are strongly encouraged to use gaiaQuotedSql()
 as a safest replacement.
 */
    GAIAAUX_DECLARE void gaiaCleanSqlString (char *value);

#ifdef __cplusplus
}
#endif

#endif				/* _GAIAAUX_H */
/**************** End file: gaiaaux.h **********/


/**************** Begin file: gaiaexif.h **********/

/**
 \file gaiaexif.h

 EXIF/image: supporting functions and constants
 */

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifdef DLL_EXPORT
#define GAIAEXIF_DECLARE __declspec(dllexport)
#else
#define GAIAEXIF_DECLARE extern
#endif
#endif

#ifndef _GAIAEXIF_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GAIAEXIF_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* constants used for BLOB value types */
/** generic hexadecimal BLOB */
#define GAIA_HEX_BLOB		0
/** this BLOB does actually contain a GIF image */
#define GAIA_GIF_BLOB		1
/** this BLOB does actually containt a PNG image */
#define GAIA_PNG_BLOB		2
/** this BLOB does actually contain a generic JPEG image */
#define GAIA_JPEG_BLOB		3
/** this BLOB does actually contain a JPEG-EXIF image */
#define GAIA_EXIF_BLOB		4
/** this BLOB does actually contain a JPEG-EXIF image including GPS data */
#define GAIA_EXIF_GPS_BLOB	5
/** this BLOB does actually contain a ZIP compressed file */
#define GAIA_ZIP_BLOB		6
/** this BLOB does actually contain a PDF document */
#define GAIA_PDF_BLOB		7
/** this BLOB does actually contain a SpatiaLite Geometry */
#define GAIA_GEOMETRY_BLOB	8
/** this BLOB does actually contain a TIFF image */
#define GAIA_TIFF_BLOB		9

/* constants used for EXIF value types */
/** unrecognized EXIF value */
#define GAIA_EXIF_NONE		0
/** EXIF value of the BYTE type */
#define GAIA_EXIF_BYTE		1
/** EXIF value of the SHORT type */
#define GAIA_EXIF_SHORT		2
/** EXIF value of the STRING type */
#define GAIA_EXIF_STRING	3
/** EXIF value of the LONG type */
#define GAIA_EXIF_LONG		4
/** EXIF value of the RATIONAL type */
#define GAIA_EXIF_RATIONAL	5
/** EXIF value of the SLONG type */
#define GAIA_EXIF_SLONG		9
/** EXIF value of the SRATIONAL type */
#define GAIA_EXIF_SRATIONAL	10

/**
 Container for an EXIF tag
 */
    typedef struct gaiaExifTagStruct
    {
/* an EXIF TAG */
	/** GPS data included (0/1) */
	char Gps;
	/** EXIF tag ID */
	unsigned short TagId;
	/** EXIF value type */
	unsigned short Type;
	/** number of values */
	unsigned short Count;
	/** tag offset [big- little-endian encoded] */
	unsigned char TagOffset[4];
	/** array of BYTE values */
	unsigned char *ByteValue;
	/** array of STRING values */
	char *StringValue;
	/** array of SHORT values */
	unsigned short *ShortValues;
	/** array of LONG values ] */
	unsigned int *LongValues;
	/** array of RATIONAL values [numerators] */
	unsigned int *LongRationals1;
	/** array of RATIONAL values [denominators] */
	unsigned int *LongRationals2;
	/** array of Signed SHORT values */
	short *SignedShortValues;
	/** array of Signed LONG values */
	int *SignedLongValues;
	/** array of Signed RATIONAL values [numerators] */
	int *SignedLongRationals1;
	/** array of Signed RATIONAL values [denominators] */
	int *SignedLongRationals2;
	/** array of FLOAT values */
	float *FloatValues;
	/** array of DOUBLE values */
	double *DoubleValues;
	/** pointer to next item into the linked list */
	struct gaiaExifTagStruct *Next;
    } gaiaExifTag;
/**
 Typedef for EXIF tag structure.

 \sa gaiaExifTagStruct
 */
    typedef gaiaExifTag *gaiaExifTagPtr;

/**
 Container for a list of EXIF tags
 */
    typedef struct gaiaExifTagListStruct
    {
/* an EXIF TAG LIST */
	/** pointer to first item into the linked list */
	gaiaExifTagPtr First;
	/** pointer to the last item into the linked list */
	gaiaExifTagPtr Last;
	/** number of items */
	int NumTags;
	/** an array of pointers to items */
	gaiaExifTagPtr *TagsArray;
    } gaiaExifTagList;
/**
 Typedef for EXIF tag structure

 \sa gaiaExifTagListStruct
 */
    typedef gaiaExifTagList *gaiaExifTagListPtr;

/* function prototipes */

/**
 Creates a list of EXIF tags by parsing a BLOB of the JPEG-EXIF type

 \param blob the BLOB to be parsed
 \param size the BLOB size (in bytes)

 \return a list of EXIF tags: or NULL if any error is encountered

 \sa gaiaExifTagsFree

 \note you must explicitly destroy the list when it's any longer used.
 */
    GAIAEXIF_DECLARE gaiaExifTagListPtr gaiaGetExifTags (const unsigned char
							 *blob, int size);

/**
 Destroy a list of EXIF tags

 \param tag_list the list to be destroied

 \sa gaiaGetExifTags

 \note the pointer passed to this function must be one returned by a
 previous call to gaiaGetExifTags
 */
    GAIAEXIF_DECLARE void gaiaExifTagsFree (gaiaExifTagListPtr tag_list);

/**
 Return the total number of EXIF tags into the list

 \param tag_list pointer to an EXIF tag list.

 \return the EXIF tag count.

 \sa gaiaGetExifTags, gaiaExifTagsFree
 */
    GAIAEXIF_DECLARE int gaiaGetExifTagsCount (gaiaExifTagListPtr tag_list);

/**
 Retrieves an EXIF tag by its relative position into the list

 \param tag_list pointer to an EXIF tag list.
 \param pos relative item position [first item is 0]

 \return a pointer to the corresponding EXIF tag: NULL if not found

 \sa gaiaGetExifTags, gaiaExifTagsFree, gaiaExifTagsCount
 */
    GAIAEXIF_DECLARE gaiaExifTagPtr gaiaGetExifTagByPos (gaiaExifTagListPtr
							 tag_list,
							 const int pos);

/**
 Return the total number of EXIF tags into the list

 \param tag_list pointer to an EXIF tag list.

 \return the EXIF tag count.

 \sa gaiaGetExifTags, gaiaExifTagsFree
 */
    GAIAEXIF_DECLARE int gaiaGetExifTagsCount (gaiaExifTagListPtr tag_list);

/**
 Retrieves an EXIF tag by its Tag ID

 \param tag_list pointer to an EXIF tag list.
 \param tag_id the Tag ID to be found

 \return a pointer to the corresponding EXIF tag: NULL if not found

 \sa gaiaGetExifTags, gaiaExifTagsFree
 */
    GAIAEXIF_DECLARE gaiaExifTagPtr gaiaGetExifTagById (const gaiaExifTagListPtr
							tag_list,
							const unsigned short
							tag_id);

/**
 Retrieves an EXIF-GPS tag by its Tag ID

 \param tag_list pointer to an EXIF tag list.
 \param tag_id the GPS Tag ID to be found

 \return a pointer to the corresponding EXIF tag: NULL if not found

 \sa gaiaGetExifTags, gaiaExifTagsFree
 */
    GAIAEXIF_DECLARE gaiaExifTagPtr gaiaGetExifGpsTagById (const
							   gaiaExifTagListPtr
							   tag_list,
							   const unsigned short
							   tag_id);

/**
 Retrieves an EXIF tag by its name

 \param tag_list pointer to an EXIF tag list.
 \param tag_name the Tag Name to be found

 \return a pointer to the corresponding EXIF tag: NULL if not found

 \sa gaiaGetExifTags, gaiaExifTagsFree
 */
    GAIAEXIF_DECLARE gaiaExifTagPtr gaiaGetExifTagByName (const
							  gaiaExifTagListPtr
							  tag_list,
							  const char *tag_name);

/**
 Return the Tag ID from an EXIF tag

 \param tag pointer to an EXIF tag
 
 \return the Tag ID

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName
 */
    GAIAEXIF_DECLARE unsigned short gaiaExifTagGetId (const gaiaExifTagPtr tag);

/**
 Return the Tag Name from an EXIF tag

 \param tag pointer to an EXIF tag
 \param tag_name receiving buffer: the Tag Name will be copied here
 \param len length of the receiving buffer

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName
 */
    GAIAEXIF_DECLARE void gaiaExifTagGetName (const gaiaExifTagPtr tag,
					      char *tag_name, int len);

/**
 Checks if an EXIF tag actually is an EXIF-GPS tag

 \param tag pointer to an EXIF tag

 \return 0 if false: any other value if true

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName
 */
    GAIAEXIF_DECLARE int gaiaIsExifGpsTag (const gaiaExifTagPtr tag);

/**
 Return the value type for an EXIF tag

 \param tag pointer to an EXIF tag

 \return the value type: one of GAIA_EXIF_NONE, GAIA_EXIF_BYTE,
 GAIA_EXIF_SHORT, GAIA_EXIF_STRING, GAIA_EXIF_LONG, GAIA_EXIF_RATIONAL,
 GAIA_EXIF_SLONG, GAIA_EXIF_SRATIONAL

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName
 */
    GAIAEXIF_DECLARE unsigned short gaiaExifTagGetValueType (const
							     gaiaExifTagPtr
							     tag);

/**
 Return the total count of values from an EXIF tag
 
 \param tag pointer to an EXIF tag

 \return the number of available values

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName
 */
    GAIAEXIF_DECLARE unsigned short gaiaExifTagGetNumValues (const
							     gaiaExifTagPtr
							     tag);

/**
 Return a BYTE value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the BYTE value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE unsigned char gaiaExifTagGetByteValue (const gaiaExifTagPtr
							    tag, const int ind,
							    int *ok);

/**
 Return a STRING value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param str receiving buffer: the STRING value will be copied here.
 \param len length of the receiving buffer
 \param ok on completion will contain 0 on failure: any other value on success.

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE void gaiaExifTagGetStringValue (const gaiaExifTagPtr tag,
						     char *str, int len,
						     int *ok);

/**
 Return a SHORT value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the SHORT value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE unsigned short gaiaExifTagGetShortValue (const
							      gaiaExifTagPtr
							      tag,
							      const int ind,
							      int *ok);

/**
 Return a LONG value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the LONG value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE unsigned int gaiaExifTagGetLongValue (const gaiaExifTagPtr
							   tag, const int ind,
							   int *ok);

/**
 Return a RATIONAL [numerator] value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the RATIONAL [numerator] value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE unsigned int gaiaExifTagGetRational1Value (const
								gaiaExifTagPtr
								tag,
								const int ind,
								int *ok);

/**
 Return a RATIONAL [denominator] value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the RATIONAL [denominator] value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE unsigned int gaiaExifTagGetRational2Value (const
								gaiaExifTagPtr
								tag,
								const int ind,
								int *ok);

/**
 Return a RATIONAL value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the RATIONAL value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE double gaiaExifTagGetRationalValue (const gaiaExifTagPtr
							 tag, const int ind,
							 int *ok);

/**
 Return a Signed SHORT value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the Signed SHORT value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE short gaiaExifTagGetSignedShortValue (const gaiaExifTagPtr
							   tag, const int ind,
							   int *ok);

/**
 Return a Signed LONG value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the Signed LONG value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE int gaiaExifTagGetSignedLongValue (const gaiaExifTagPtr
							tag, const int ind,
							int *ok);

/**
 Return a SRATIONAL [numerator] value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the SRATIONAL [numerator] value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE int gaiaExifTagGetSignedRational1Value (const
							     gaiaExifTagPtr tag,
							     const int ind,
							     int *ok);

/**
 Return a SRATIONAL [denominator] value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the SRATIONAL [denominator] value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE int gaiaExifTagGetSignedRational2Value (const
							     gaiaExifTagPtr tag,
							     const int ind,
							     int *ok);

/**
 Return a Signed RATIONAL value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the Signed RATIONAL value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE double gaiaExifTagGetSignedRationalValue (const
							       gaiaExifTagPtr
							       tag,
							       const int ind,
							       int *ok);

/**
 Return a FLOAT value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the FLOAT value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE float gaiaExifTagGetFloatValue (const gaiaExifTagPtr tag,
						     const int ind, int *ok);

/**
 Return a DOUBLE value from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param ind value index [first value has index 0].
 \param ok on completion will contain 0 on failure: any other value on success.

 \return the DOUBLE value

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaExifTagGetValueType, gaiaExifTagGetNumValues
 */
    GAIAEXIF_DECLARE double gaiaExifTagGetDoubleValue (const gaiaExifTagPtr tag,
						       const int ind, int *ok);

/**
 Return a human readable description from an EXIF tag

 \param tag pointer to an EXIF tag.
 \param str receiving buffer: the STRING value will be copied here.
 \param len length of the receiving buffer
 \param ok on completion will contain 0 on failure: any other value on success.

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName
 */
    GAIAEXIF_DECLARE void gaiaExifTagGetHumanReadable (const gaiaExifTagPtr tag,
						       char *str, int len,
						       int *ok);

/**
 Attempts to guess the actual content-type of some BLOB

 \param blob the BLOB to be parsed 
 \param size length of the BLOB (in bytes)

 \return the BLOB type: one of GAIA_HEX_BLOB, GAIA_GIF_BLOB, GAIA_PNG_BLOB,
 GAIA_JPEG_BLOB, GAIA_EXIF_BLOB, GAIA_EXIF_GPS_BLOB, GAIA_ZIP_BLOB,
 GAIA_PDF_BLOB, GAIA_GEOMETRY_BLOB, GAIA_TIFF_BLOB
 */
    GAIAEXIF_DECLARE int gaiaGuessBlobType (const unsigned char *blob,
					    int size);
/**
 Return longitude and latitude from an EXIF-GPS tag

 \param blob the BLOB to be parsed 
 \param size length of the BLOB (in bytes)
 \param longitude on success will contain the longitude coordinate
 \param latitude on success will contain the latitude coordinate

 \return 0 on failure: any other value on success

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaIsExifGpsTag
 */
    GAIAEXIF_DECLARE int gaiaGetGpsCoords (const unsigned char *blob, int size,
					   double *longitude, double *latitude);
/**
 Return a text string representing DMS coordinates from an EXIF-GPS tag

 \param blob the BLOB to be parsed 
 \param size length of the BLOB (in bytes)
 \param latlong receiving buffer: the text string will be copied here.
 \param ll_size length of the receiving buffer

 \return 0 on failure: any other value on success

 \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
 gaiaIsExifGpsTag
 */
    GAIAEXIF_DECLARE int gaiaGetGpsLatLong (const unsigned char *blob, int size,
					    char *latlong, int ll_size);

#ifdef __cplusplus
}
#endif

#endif				/* _GAIAEXIF_H */
/**************** End file: gaiaexif.h **********/


/**************** Begin file: gaiageo.h **********/


/**
 \file gaiageo.h

 Geometry handling functions and constants 
 */

#ifndef DOXYGEN_SHOULD_SKIP_THIS
/* stdio.h included for FILE objects. */
/* #include <stdio.h> */
#ifdef DLL_EXPORT
#define GAIAGEO_DECLARE __declspec(dllexport)
#else
#define GAIAGEO_DECLARE extern
#endif
#endif

#ifndef _GAIAGEO_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GAIAGEO_H
#endif

/* #include "gg_const.h" */
/* #include "gg_structs.h" */
/* #include "gg_core.h" */
/* #include "gg_mbr.h" */
/* #include "gg_formats.h" */
/* #include "gg_dynamic.h" */
/* #include "gg_advanced.h" */

#endif /* _GAIAGEO_H */
/**************** End file: gaiageo.h **********/


/**************** Begin file: gg_const.h **********/


/**
 \file gg_const.h

 Geometry constants and macros
 */

#ifndef _GG_CONST_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GG_CONST_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* constant values for generic geometry classes */

/** WKT parser: unknown Geometry type */
#define GAIA_TYPE_NONE		0
/** WKT parser: Point Geometry type */
#define GAIA_TYPE_POINT		1
/** WKT parser: Linestring Geometry type */
#define GAIA_TYPE_LINESTRING	2
/** WKT parser: Polygon Geometry type */
#define GAIA_TYPE_POLYGON	3

/* constants that defines byte storage order  */
/** Big-Endian marker */
#define GAIA_BIG_ENDIAN		0
/** Little-Endian marker */
#define GAIA_LITTLE_ENDIAN	1

/* constants that defines special markers used for encoding of SpatiaLite internal BLOB geometries  */
/** BLOB-Geometry internal marker: START */
#define GAIA_MARK_START		0x00
/** BLOB-Geometry internal marker: END */
#define GAIA_MARK_END		0xFE
/** BLOB-Geometry internal marker: MBR */
#define GAIA_MARK_MBR		0x7C
/** BLOB-Geometry internal marker: ENTITY */
#define GAIA_MARK_ENTITY	0x69

/* constants that defines GEOMETRY CLASSes */
/** BLOB-Geometry CLASS: unknown */
#define GAIA_UNKNOWN			0
/** BLOB-Geometry CLASS: POINT */
#define GAIA_POINT			1
/** BLOB-Geometry CLASS: LINESTRING */
#define GAIA_LINESTRING			2
/** BLOB-Geometry CLASS: POLYGON */
#define GAIA_POLYGON			3
/** BLOB-Geometry CLASS: MULTIPOINT */
#define GAIA_MULTIPOINT			4
/** BLOB-Geometry CLASS: MULTILINESTRING */
#define GAIA_MULTILINESTRING		5
/** BLOB-Geometry CLASS: MULTIPOLYGON */
#define GAIA_MULTIPOLYGON		6
#define GAIA_MULTIPOLYGON		6
/** BLOB-Geometry CLASS: GEOMETRYCOLLECTION */
#define GAIA_GEOMETRYCOLLECTION		7
/** BLOB-Geometry CLASS: POINT Z */
#define GAIA_POINTZ			1001
/** BLOB-Geometry CLASS: LINESTRING Z */
#define GAIA_LINESTRINGZ		1002
/** BLOB-Geometry CLASS: POLYGON Z */
#define GAIA_POLYGONZ			1003
/** BLOB-Geometry CLASS: MULTIPOINT Z */
#define GAIA_MULTIPOINTZ		1004
/** BLOB-Geometry CLASS: MULTILINESTRING Z */
#define GAIA_MULTILINESTRINGZ		1005
/** BLOB-Geometry CLASS: MULTIPOLYGON Z */
#define GAIA_MULTIPOLYGONZ		1006
/** BLOB-Geometry CLASS: GEOMETRYCOLLECTION Z */
#define GAIA_GEOMETRYCOLLECTIONZ	1007
/** BLOB-Geometry CLASS: POINT M */
#define GAIA_POINTM			2001
/** BLOB-Geometry CLASS: LINESTRING M */
#define GAIA_LINESTRINGM		2002
/** BLOB-Geometry CLASS: POLYGON M */
#define GAIA_POLYGONM			2003
/** BLOB-Geometry CLASS: MULTIPOINT M */
#define GAIA_MULTIPOINTM		2004
/** BLOB-Geometry CLASS: MULTILINESTRING M */
#define GAIA_MULTILINESTRINGM		2005
/** BLOB-Geometry CLASS: MULTIPOLYGON M */
#define GAIA_MULTIPOLYGONM		2006
/** BLOB-Geometry CLASS: GEOMETRYCOLLECTION M */
#define GAIA_GEOMETRYCOLLECTIONM	2007
/** BLOB-Geometry CLASS: POINT ZM */
#define GAIA_POINTZM			3001
/** BLOB-Geometry CLASS: LINESTRING ZM */
#define GAIA_LINESTRINGZM		3002
/** BLOB-Geometry CLASS: POLYGON ZM */
#define GAIA_POLYGONZM			3003
/** BLOB-Geometry CLASS: MULTIPOINT ZM */
#define GAIA_MULTIPOINTZM		3004
/** BLOB-Geometry CLASS: MULTILINESTRING ZM */
#define GAIA_MULTILINESTRINGZM		3005
/** BLOB-Geometry CLASS: MULTIPOLYGON ZM */
#define GAIA_MULTIPOLYGONZM		3006
/** BLOB-Geometry CLASS: GEOMETRYCOLLECTION ZM */
#define GAIA_GEOMETRYCOLLECTIONZM	3007

/* constants that defines Compressed GEOMETRY CLASSes */
/** BLOB-Geometry CLASS: compressed LINESTRING */
#define GAIA_COMPRESSED_LINESTRING		1000002
/** BLOB-Geometry CLASS: compressed POLYGON */
#define GAIA_COMPRESSED_POLYGON			1000003
/** BLOB-Geometry CLASS: compressed LINESTRING Z */
#define GAIA_COMPRESSED_LINESTRINGZ		1001002
/** BLOB-Geometry CLASS: compressed POLYGON Z */
#define GAIA_COMPRESSED_POLYGONZ		1001003
/** BLOB-Geometry CLASS: compressed LINESTRING M */
#define GAIA_COMPRESSED_LINESTRINGM		1002002
/** BLOB-Geometry CLASS: compressed POLYGON M */
#define GAIA_COMPRESSED_POLYGONM		1002003
/** BLOB-Geometry CLASS: compressed LINESTRING ZM */
#define GAIA_COMPRESSED_LINESTRINGZM		1003002
/** BLOB-Geometry CLASS: compressed POLYGON ZM */
#define GAIA_COMPRESSED_POLYGONZM		1003003

/* constants that defines GEOS-WKB 3D CLASSes */
/** GEOS-WKB 3D CLASS: POINT Z */
#define GAIA_GEOSWKB_POINTZ			-2147483647
/** GEOS-WKB 3D CLASS: LINESTRING Z */
#define GAIA_GEOSWKB_LINESTRINGZ		-2147483646
/** GEOS-WKB 3D CLASS: POLYGON Z */
#define GAIA_GEOSWKB_POLYGONZ			-2147483645
/** GEOS-WKB 3D CLASS: MULTIPOINT Z */
#define GAIA_GEOSWKB_MULTIPOINTZ		-2147483644
/** GEOS-WKB 3D CLASS: MULTILINESTRING Z */
#define GAIA_GEOSWKB_MULTILINESTRINGZ		-2147483643
/** GEOS-WKB 3D CLASS: MULTIPOLYGON Z */
#define GAIA_GEOSWKB_MULTIPOLYGONZ		-2147483642
/** GEOS-WKB 3D CLASS: POINT Z */
#define GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ	-2147483641

/* constants that defines multitype values */
/** DBF data type: NULL */
#define GAIA_NULL_VALUE		0
/** DBF data type: TEXT */
#define GAIA_TEXT_VALUE		1
/** DBF data type: INT */
#define GAIA_INT_VALUE		2
/** DBF data type: DOUBLE */
#define GAIA_DOUBLE_VALUE	3

/* constants that defines POINT index for LINESTRING */
/** Linestring/Ring functions: START POINT */
#define GAIA_START_POINT	1
/** Linestring/Ring functions: END POINT */
#define GAIA_END_POINT		2
/** Linestring/Ring functions: POINTN */
#define GAIA_POINTN		3

/* constants that defines MBRs spatial relationships */
/** MBR relationships: CONTAINS */
#define GAIA_MBR_CONTAINS	1
/** MBR relationships: DISJOINT */
#define GAIA_MBR_DISJOINT	2
/** MBR relationships: EQUAL */
#define GAIA_MBR_EQUAL		3
/** MBR relationships: INTERSECTS */
#define GAIA_MBR_INTERSECTS	4
/** MBR relationships: OVERLAP */
#define GAIA_MBR_OVERLAPS	5
/** MBR relationships: TOUCHES */
#define GAIA_MBR_TOUCHES	6
/** MBR relationships: WITHIN */
#define GAIA_MBR_WITHIN		7

/* constants used for FilterMBR */
/** FilerMBR relationships: WITHIN */
#define GAIA_FILTER_MBR_WITHIN		74
/** FilerMBR relationships: CONTAINS */
#define GAIA_FILTER_MBR_CONTAINS	77
/** FilerMBR relationships: INTERSECTS */
#define GAIA_FILTER_MBR_INTERSECTS	79
/** FilerMBR relationships: DECLARE */
#define GAIA_FILTER_MBR_DECLARE		89

/* constants defining SVG default values */
/** SVG precision: RELATIVE */
#define GAIA_SVG_DEFAULT_RELATIVE 	0
/** SVG precision: DEFAULT */
#define GAIA_SVG_DEFAULT_PRECISION	6
/** SVG precision: MAX */
#define GAIA_SVG_DEFAULT_MAX_PRECISION 15

/* constants used for VirtualNetwork */
/** VirtualNetwork internal markers: START */
#define GAIA_NET_START		0x67
/** VirtualNetwork internal markers: 64 bit START */
#define GAIA_NET64_START	0x68
/** VirtualNetwork internal markers: A-Stat START */
#define GAIA_NET64_A_STAR_START	0x69
/** VirtualNetwork internal markers: END */
#define GAIA_NET_END		0x87
/** VirtualNetwork internal markers: HEADER */
#define GAIA_NET_HEADER		0xc0
/** VirtualNetwork internal markers: CODE */
#define GAIA_NET_CODE		0xa6
/** VirtualNetwork internal markers: ID */
#define GAIA_NET_ID		0xb5
/** VirtualNetwork internal markers: NODE */
#define GAIA_NET_NODE		0xde
/** VirtualNetwork internal markers: ARC */
#define GAIA_NET_ARC		0x54
/** VirtualNetwork internal markers: TABLE */
#define GAIA_NET_TABLE		0xa0
/** VirtualNetwork internal markers: FROM */
#define GAIA_NET_FROM		0xa1
/** VirtualNetwork internal markers: TO */
#define GAIA_NET_TO		0xa2
/** VirtualNetwork internal markers: GEOM */
#define GAIA_NET_GEOM		0xa3
/** VirtualNetwork internal markers: NAME */
#define GAIA_NET_NAME		0xa4
/** VirtualNetwork internal markers: COEFF */
#define GAIA_NET_A_STAR_COEFF	0xa5
/** VirtualNetwork internal markers: BLOCK */
#define GAIA_NET_BLOCK		0xed

/* constants used for Coordinate Dimensions */
/** Coordinate Dimensions: XY */
#define GAIA_XY		0x00
/** Coordinate Dimensions: XYZ */
#define GAIA_XY_Z	0x01
/** Coordinate Dimensions: XYM */
#define GAIA_XY_M	0x02
/** Coordinate Dimensions: XYZM */
#define GAIA_XY_Z_M	0x03

/* constants used for length unit conversion */
/** Length unit conversion: Kilometer */
#define GAIA_KM		0
/** Length unit conversion: Meter */
#define GAIA_M		1
/** Length unit conversion: Decimeter */
#define GAIA_DM		2
/** Length unit conversion: Centimeter */
#define GAIA_CM		3
/** Length unit conversion: Millimeter */
#define GAIA_MM		4
/** Length unit conversion: International Nautical Mile */
#define GAIA_KMI	5
/** Length unit conversion: Inch */
#define GAIA_IN		6
/** Length unit conversion: Feet */
#define GAIA_FT		7
/** Length unit conversion: Yard */
#define GAIA_YD		8
/** Length unit conversion: Mile */
#define GAIA_MI		9
/** Length unit conversion: Fathom */
#define GAIA_FATH	10
/** Length unit conversion: Chain */
#define GAIA_CH		11
/** Length unit conversion: Link */
#define GAIA_LINK	12
/** Length unit conversion: US Inch */
#define GAIA_US_IN	13
/** Length unit conversion: US Feet */
#define GAIA_US_FT	14
/** Length unit conversion: US Yard */
#define GAIA_US_YD	15
/** Length unit conversion: US Chain */
#define GAIA_US_CH	16
/** Length unit conversion: US Mile */
#define GAIA_US_MI	17
/** Length unit conversion: Indian Yard */
#define GAIA_IND_YD	18
/** Length unit conversion: Indian Feet */
#define GAIA_IND_FT	19
/** Length unit conversion: Indian Chain */
#define GAIA_IND_CH	20
/** Length unit conversion: MIN */
#define GAIA_MIN_UNIT	GAIA_KM
/** Length unit conversion: MAX */
#define GAIA_MAX_UNIT	GAIA_IND_CH

/* constants used for SHAPES */
/** SHP shape: unknown */
#define GAIA_SHP_NULL		0
/** SHP shape: POINT */
#define GAIA_SHP_POINT		1
/** SHP shape: POLYLINE */
#define GAIA_SHP_POLYLINE	3
/** SHP shape: POLYGON */
#define GAIA_SHP_POLYGON	5
/** SHP shape: MULTIPOINT */
#define GAIA_SHP_MULTIPOINT	8
/** SHP shape: POINT Z */
#define GAIA_SHP_POINTZ		11
/** SHP shape: POLYLINE Z */
#define GAIA_SHP_POLYLINEZ	13
/** SHP shape: POLYGON Z */
#define GAIA_SHP_POLYGONZ	15
/** SHP shape: MULTIPOINT Z */
#define GAIA_SHP_MULTIPOINTZ	18
/** SHP shape: POINT M */
#define GAIA_SHP_POINTM		21
/** SHP shape: POLYLINE M */
#define GAIA_SHP_POLYLINEM	23
/** SHP shape: POLYGON M */
#define GAIA_SHP_POLYGONM	25
/** SHP shape: MULTIPOINT M */
#define GAIA_SHP_MULTIPOINTM	28

/* macros */
/**
 macro extracting XY coordinates

 \param xy pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double *] X coordinate
 \param y [double *] Y coordinate 

 \sa gaiaLineGetPoint, gaiaRingGetPoint

 \note using this macro on behalf of COORDs not of [XY] dims may cause serious 
 problems
 */
#define gaiaGetPoint(xy,v,x,y)	\
				{*x = xy[(v) * 2]; \
				 *y = xy[(v) * 2 + 1];}

/**
 macro setting XY coordinates

 \param xy pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double] X coordinate
 \param y [double] Y coordinate

 \sa gaiaLineSetPoint, gaiaRingSetPoint

 \note using this macro on behalf on COORDs not of [XY] dims may cause
 serious problems
 */
#define gaiaSetPoint(xy,v,x,y)	\
				{xy[(v) * 2] = x; \
				 xy[(v) * 2 + 1] = y;}

/**
 macro extracting XYZ coordinates

 \param xyz pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double *] X coordinate
 \param y [double *] Y coordinate 
 \param z [double *] Z coordinate 

 \sa gaiaLineGetPoint, gaiaRingGetPoint

 \note using this macro on behalf of COORDs not of [XYZ] dims may cause serious 
 problems
 */
#define gaiaGetPointXYZ(xyz,v,x,y,z)	\
				{*x = xyz[(v) * 3]; \
				 *y = xyz[(v) * 3 + 1]; \
				 *z = xyz[(v) * 3 + 2];}

/**
 macro setting XYZ coordinates

 \param xyz pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double] X coordinate
 \param y [double] Y coordinate
 \param z [double] Z coordinate

 \sa gaiaLineSetPoint, gaiaRingSetPoint

 \note using this macro on behalf on COORDs not of [XYZ] dims may cause
 serious problems
 */
#define gaiaSetPointXYZ(xyz,v,x,y,z)	\
				{xyz[(v) * 3] = x; \
				 xyz[(v) * 3 + 1] = y; \
				 xyz[(v) * 3 + 2] = z;}

/**
 macro extracting XYM coordinates

 \param xym pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double *] X coordinate
 \param y [double *] Y coordinate 
 \param m [double *] M measure

 \sa gaiaLineGetPoint, gaiaRingGetPoint

 \note using this macro on behalf of COORDs not of [XYM] dims may cause serious 
 problems
 */
#define gaiaGetPointXYM(xym,v,x,y,m)	\
				{*x = xym[(v) * 3]; \
				 *y = xym[(v) * 3 + 1]; \
				 *m = xym[(v) * 3 + 2];}

/**
 macro setting XYM coordinates

 \param xym pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double] X coordinate
 \param y [double] Y coordinate
 \param m [double] M measure

 \sa gaiaLineSetPoint, gaiaRingSetPoint

 \note using this macro on behalf on COORDs not of [XYM] dims may cause
 serious problems
 */
#define gaiaSetPointXYM(xym,v,x,y,m)	\
				{xym[(v) * 3] = x; \
				 xym[(v) * 3 + 1] = y; \
				 xym[(v) * 3 + 2] = m;}

/**
 macro extracting XYZM coordinates

 \param xyzm pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double *] X coordinate
 \param y [double *] Y coordinate 
 \param z [double *] Z coordinate 
 \param m [double *] M measure

 \sa gaiaLineGetPoint, gaiaRingGetPoint

 \note using this macro on behalf of COORDs not of [XYZM] dims may cause serious 
 problems
 */
#define gaiaGetPointXYZM(xyzm,v,x,y,z,m)	\
				{*x = xyzm[(v) * 4]; \
				 *y = xyzm[(v) * 4 + 1]; \
				 *z = xyzm[(v) * 4 + 2]; \
				 *m = xyzm[(v) * 4 + 3];}

/**
 macro setting XYZM coordinates

 \param xyzm pointer [const void *] to COORD mem-array
 \param v [int] point index [first point has index 0]
 \param x [double] X coordinate
 \param y [double] Y coordinate
 \param z [double] Z coordinate
 \param m [double] M measure

 \sa gaiaLineSetPoint, gaiaRingSetPoint

 \note using this macro on behalf on COORDs not of [XYZM] dims may cause
 serious problems
 */
#define gaiaSetPointXYZM(xyzm,v,x,y,z,m)	\
				{xyzm[(v) * 4] = x; \
				 xyzm[(v) * 4 + 1] = y; \
				 xyzm[(v) * 4 + 2] = z; \
				 xyzm[(v) * 4 + 3] = m;}


#ifdef __cplusplus
}
#endif

#endif				/* _GG_CONST_H */
/**************** End file: gg_const.h **********/


/**************** Begin file: gg_structs.h **********/


/**
 \file gg_structs.h

 Geometry structures
 */

#ifndef _GG_STRUCTS_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GG_STRUCTS_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/**
 Container for OGC POINT Geometry
 */
    typedef struct gaiaPointStruct
    {
/* an OpenGis POINT */
/** X coordinate */
	double X;		/* X,Y coordinates */
/** Y coordinate */
	double Y;
/** Z coordinate: only for XYZ and XYZM dims */
	double Z;		/* Z coordinate */
/** M measure: only for XYM and XYZM dims */
	double M;		/* M measure */
/** one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M, GAIA_XY_ZM */
	int DimensionModel;	/* (x,y), (x,y,z), (x,y,m) or (x,y,z,m) */
/** pointer to next item [double linked list] */
	struct gaiaPointStruct *Next;	/* for double-linked list */
/** pointer to previous item [double linked list] */
	struct gaiaPointStruct *Prev;	/* for double-linked list */
    } gaiaPoint;
/**
 Typedef for OGC POINT structure

 \sa gaiaPoint
 */
    typedef gaiaPoint *gaiaPointPtr;

/**
 Container for dynamically growing line/ring
 */
    typedef struct gaiaDynamicLineStruct
    {
/* a generic DYNAMIC LINE object */
/** invalid object */
	int Error;
/** the SRID */
	int Srid;
/** pointer to first POINT [double linked list] */
	gaiaPointPtr First;	/* Points linked list - first */
/** pointer to last POINT [double linked list] */
	gaiaPointPtr Last;	/* Points linked list - last */
    } gaiaDynamicLine;
/**
 Typedef for dynamically growing line/ring structure

 \sa gaiaDynamicLine
 */
    typedef gaiaDynamicLine *gaiaDynamicLinePtr;

/**
 Container for OGC LINESTRING Geometry
 */
    typedef struct gaiaLinestringStruct
    {
/* an OpenGis LINESTRING */
/** number of points [aka vertices] */
	int Points;		/* number of vertices */
/** COORDs mem-array */
	double *Coords;		/* X,Y [vertices] array */
/** MBR: min X */
	double MinX;		/* MBR - BBOX */
/** MBR: min Y */
	double MinY;		/* MBR - BBOX */
/** MBR: max X */
	double MaxX;		/* MBR - BBOX */
/** MBR: max X */
	double MaxY;		/* MBR - BBOX */
/** one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M, GAIA_XY_ZM */
	int DimensionModel;	/* (x,y), (x,y,z), (x,y,m) or (x,y,z,m) */
/** pointer to next item [linked list] */
	struct gaiaLinestringStruct *Next;	/* for linked list */
    } gaiaLinestring;
/**
 Typedef for OGC LINESTRING structure

 \sa gaiaLinestring
 */
    typedef gaiaLinestring *gaiaLinestringPtr;

/**
 Container for OGC RING Geometry
 */
    typedef struct gaiaRingStruct
    {
/* a GIS ring - OpenGis LINESTRING, closed */
/** number of points [aka vertices] */
	int Points;		/* number of vertices */
/** COORDs mem-array */
	double *Coords;		/* X,Y [vertices] array */
/** clockwise / counterclockwise */
	int Clockwise;		/* clockwise / counterclockwise */
/** MBR: min X */
	double MinX;		/* MBR - BBOX */
/** MBR: min Y */
	double MinY;		/* MBR - BBOX */
/** MBR: max X */
	double MaxX;		/* MBR - BBOX */
/** MBR: max Y */
	double MaxY;		/* MBR - BBOX */
/** one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M, GAIA_XY_ZM */
	int DimensionModel;	/* (x,y), (x,y,z), (x,y,m) or (x,y,z,m) */
/** pointer to next item [linked list] */
	struct gaiaRingStruct *Next;	/* for linked list */
/** pointer to belonging Polygon */
	struct gaiaPolygonStruct *Link;	/* polygon reference */
    } gaiaRing;
/**
 Typedef for OGC RING structure

 \sa gaiaRing
 */
    typedef gaiaRing *gaiaRingPtr;

/**
 Container for OGC POLYGON Geometry
 */
    typedef struct gaiaPolygonStruct
    {
/* an OpenGis POLYGON */
/** the exterior ring (mandatory) */
	gaiaRingPtr Exterior;	/* exterior ring */
/** number of interior rings (may be, none) */
	int NumInteriors;	/* number of interior rings */
/** array of interior rings */
	gaiaRingPtr Interiors;	/* interior rings array */
/** index of first unused interior ring */
	int NextInterior;	/* first free interior ring */
/** MBR: min X */
	double MinX;		/* MBR - BBOX */
/** MBR: min Y */
	double MinY;		/* MBR - BBOX */
/** MBR: max X */
	double MaxX;		/* MBR - BBOX */
/** MBR: max Y */
	double MaxY;		/* MBR - BBOX */
/** one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M, GAIA_XY_ZM */
	int DimensionModel;	/* (x,y), (x,y,z), (x,y,m) or (x,y,z,m) */
/** pointer to next item [linked list] */
	struct gaiaPolygonStruct *Next;	/* for linked list */
    } gaiaPolygon;
/**
 Typedef for OGC POLYGON structure
 
 \sa gaiaPolygon
 */
    typedef gaiaPolygon *gaiaPolygonPtr;

/**
 Container for OGC GEOMETRYCOLLECTION Geometry
 */
    typedef struct gaiaGeomCollStruct
    {
/* OpenGis GEOMETRYCOLLECTION */
/** the SRID */
	int Srid;		/* the SRID value for this GEOMETRY */
/** CPU endian arch */
	char endian_arch;	/* littleEndian - bigEndian arch for target CPU */
/** BLOB Geometry endian arch */
	char endian;		/* littleEndian - bigEndian */
/** BLOB-Geometry buffer */
	const unsigned char *blob;	/* WKB encoded buffer */
/** BLOB-Geometry buffer size (in bytes) */
	unsigned long size;	/* buffer size */
/** current offset [BLOB parsing] */
	unsigned long offset;	/* current offset [for parsing] */
/** pointer to first POINT [linked list]; may be NULL */
	gaiaPointPtr FirstPoint;	/* Points linked list - first */
/** pointer to last POINT [linked list]; may be NULL */
	gaiaPointPtr LastPoint;	/* Points linked list - last */
/** pointer to first LINESTRING [linked list]; may be NULL */
	gaiaLinestringPtr FirstLinestring;	/* Linestrings linked list - first */
/** pointer to last LINESTRING [linked list]; may be NULL */
	gaiaLinestringPtr LastLinestring;	/* Linestrings linked list - last */
/** pointer to first POLYGON [linked list]; may be NULL */
	gaiaPolygonPtr FirstPolygon;	/* Polygons linked list - first */
/** pointer to last POLYGON [linked list]; may be NULL */
	gaiaPolygonPtr LastPolygon;	/* Polygons linked list - last */
/** MBR: min X */
	double MinX;		/* MBR - BBOX */
/** MBR: min Y */
	double MinY;		/* MBR - BBOX */
/** MBR: max X */
	double MaxX;		/* MBR - BBOX */
/** MBR: max Y */
	double MaxY;		/* MBR - BBOX */
/** one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M, GAIA_XY_ZM */
	int DimensionModel;	/* (x,y), (x,y,z), (x,y,m) or (x,y,z,m) */
/** any valid Geometry Class type */
	int DeclaredType;	/* the declared TYPE for this Geometry */
/** pointer to next item [linked list] */
	struct gaiaGeomCollStruct *Next;	/* Vanuatu - used for linked list */
    } gaiaGeomColl;
/**
 Typedef for OGC GEOMETRYCOLLECTION structure

 \sa gaiaGeomCool
 */
    typedef gaiaGeomColl *gaiaGeomCollPtr;

/**
 Container similar to LINESTRING [internally used]
 */
    typedef struct gaiaPreRingStruct
    {
/* a LINESTRING used to build rings */
/** pointer to LINESTRING */
	gaiaLinestringPtr Line;	/* a LINESTRING pointer */
/** already used/visited item */
	int AlreadyUsed;	/* a switch to mark an already used line element */
/** pointer to next item [linked list] */
	struct gaiaPreRingStruct *Next;	/* for linked list */
    } gaiaPreRing;
/**
 Typedef for gaiaPreRing structure

 \sa gaiaPreRing
 */
    typedef gaiaPreRing *gaiaPreRingPtr;

/**
 Container for variant (multi-type) value
 */
    typedef struct gaiaValueStruct
    {
/* a DBF field multitype value */
/** data type: one of GAIA_NULL_VALUE, GAIA_INT_VALUE, GAIA_DOUBLE_VALUE, GAIA_TEXT_VALUE */
	short Type;		/* the type */
/** TEXT type value */
	char *TxtValue;		/* the text value */
/** INT type value */
	sqlite3_int64 IntValue;	/* the integer value */
/** DOUBLE type value */
	double DblValue;	/* the double value */
    } gaiaValue;
/**
 Typedef for variant (multi-type) value structure 
 */
    typedef gaiaValue *gaiaValuePtr;

/**
 Container for DBF field
 */
    typedef struct gaiaDbfFieldStruct
    {
/* a DBF field definition - shapefile attribute */
/** field name [max. 10 char] */
	char *Name;		/* field name */
/** DBF data type */
	unsigned char Type;	/* field type */
/** DBF buffer offset [where the field value starts] */
	int Offset;		/* buffer offset [this field begins at *buffer+offset* and extends for *length* bytes */
/** total DBF buffer field length (in bytes) */
	unsigned char Length;	/* field total length [in bytes] */
/** precision (decimal digits) */
	unsigned char Decimals;	/* decimal positions */
/** current variant [multi-type] value */
	gaiaValuePtr Value;	/* the current multitype value for this attribute */
/** pointer to next item [linked list] */
	struct gaiaDbfFieldStruct *Next;	/* pointer to next element in linked list */
    } gaiaDbfField;
/**
 Typedef for DBF field structure 
 */
    typedef gaiaDbfField *gaiaDbfFieldPtr;

/**
 Container for a list of DBF fields
 */
    typedef struct gaiaDbfListStruct
    {
/* a linked list to contain the DBF fields definitions - shapefile attributes */
/** current RowID */
	int RowId;		/* the current RowId */
/** current Geometry */
	gaiaGeomCollPtr Geometry;	/* geometry for current entity */
/** pointer to first DBF field [linked list] */
	gaiaDbfFieldPtr First;	/* pointer to first element in linked list */
/** pointer to last DBF field [linked list] */
	gaiaDbfFieldPtr Last;	/* pointer to last element in linker list */
    } gaiaDbfList;
/**
 Typedef for a list of DBF fields
 
 \sa gaiaDbfList
 */
    typedef gaiaDbfList *gaiaDbfListPtr;

/**
 Container for DBF file handling
 */
    typedef struct gaiaDbfStruct
    {
/* DBF TYPE */
/** DBF endian arch */
	int endian_arch;
/** validity flag: 1 = ready to be processed */
	int Valid;		/* 1 = ready to process */
/** DBF file pathname */
	char *Path;		/* the DBF path */
/** FILE handle */
	FILE *flDbf;		/* the DBF file handle */
/** list of DBF fields */
	gaiaDbfListPtr Dbf;	/* the DBF attributes list */
/** I/O buffer */
	unsigned char *BufDbf;	/* the DBF I/O buffer */
/** header size (in bytes) */
	int DbfHdsz;		/* the DBF header length */
/** record length (in bytes) */
	int DbfReclen;		/* the DBF record length */
/** current file size */
	int DbfSize;		/* current DBF size */
/** current Record Number */
	int DbfRecno;		/* current DBF record number */
/** handle to ICONV converter object */
	void *IconvObj;		/* opaque reference to ICONV converter */
/** last error message (may be NULL) */
	char *LastError;	/* last error message */
    } gaiaDbf;
/** 
 Typedef for DBF file handler structure

 \sa gaiaDbf
 */
    typedef gaiaDbf *gaiaDbfPtr;

/**
 Container for SHP file handling
 */
    typedef struct gaiaShapefileStruct
    {
/* SHAPEFILE TYPE */
/** SHP endian arch */
	int endian_arch;
/** validity flag: 1 = ready to be processed */
	int Valid;		/* 1 = ready to process */
/** read or write mode */
	int ReadOnly;		/* read or write mode */
/** SHP 'abstract' path (no suffixes) */
	char *Path;		/* the shapefile abstract path [no suffixes] */
/** FILE handle to SHX file */
	FILE *flShx;		/* the SHX file handle */
/** FILE handle to SHP file */
	FILE *flShp;		/* the SHP file handle */
/** FILE handle to DBF file */
	FILE *flDbf;		/* the DBF file handle */
/** the SHP shape code */
	int Shape;		/* the SHAPE code for the whole shapefile */
/** list of DBF fields */
	gaiaDbfListPtr Dbf;	/* the DBF attributes list */
/** DBF I/O buffer */
	unsigned char *BufDbf;	/* the DBF I/O buffer */
/** DBF header size (in bytes) */
	int DbfHdsz;		/* the DBF header length */
/** DBF record length (in bytes) */
	int DbfReclen;		/* the DBF record length */
/** DBF current file size (in bytes) */
	int DbfSize;		/* current DBF size */
/** DBF current Record Number */
	int DbfRecno;		/* current DBF record number */
/** SHP I/O buffer */
	unsigned char *BufShp;	/* the SHP I/O buffer */
/** SHP current buffer size (in bytes) */
	int ShpBfsz;		/* the SHP buffer current size */
/** SHP current file size */
	int ShpSize;		/* current SHP size */
/** SHX current file size */
	int ShxSize;		/* current SHX size */
/** Total Extent: min X */
	double MinX;		/* the MBR/BBOX for the whole shapefile */
/** Total Extent: min Y */
	double MinY;
/** Total Extent: max X */
	double MaxX;
/** Total Extent: max Y */
	double MaxY;
/** handle to ICONV converter object */
	void *IconvObj;		/* opaque reference to ICONV converter */
/** last error message (may be NULL) */
	char *LastError;	/* last error message */
/** SHP actual OGC Geometry type */
	int EffectiveType;	/* the effective Geometry-type, as determined by gaiaShpAnalyze() */
/** SHP actual dims: one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M, GAIA_XY_ZM */
	int EffectiveDims;	/* the effective Dimensions [XY, XYZ, XYM, XYZM], as determined by gaiaShpAnalyze() */
    } gaiaShapefile;
/**
 Typedef for SHP file handler structure

 \sa gaiaShapefile
 */
    typedef gaiaShapefile *gaiaShapefilePtr;

/**
 Container for dynamically growing output buffer
 */
    typedef struct gaiaOutBufferStruct
    {
/* a struct handling a dynamically growing output buffer */
/** current buffer */
	char *Buffer;
/** current write offset */
	int WriteOffset;
/** current buffer size (in bytes) */
	int BufferSize;
/** validity flag */
	int Error;
    } gaiaOutBuffer;
/**
 Typedef for dynamically growing output buffer structure

 \sa gaiaOutBuffer
 */
    typedef gaiaOutBuffer *gaiaOutBufferPtr;

#ifndef OMIT_ICONV		/* ICONV enabled: supporting text reader */

/** Virtual Text driver: MAX number of fields */
#define VRTTXT_FIELDS_MAX	65535
/** Virtual Text driver: MAX block size (in bytes) */
#define VRTTXT_BLOCK_MAX 65535

/** Virtual Text driver: TEXT value */
#define VRTTXT_TEXT		1
/** Virtual Text driver: INTEGER value */
#define VRTTXT_INTEGER	2
/** Virtual Text driver: DOUBLE value */
#define VRTTXT_DOUBLE	3
/** Virtual Text driver: NULL value */
#define VRTTXT_NULL	4

/**
 Container for Virtual Text record (line)
 */
    struct vrttxt_line
    {
/* a struct representing a full LINE (aka Record) */
/** current offset (parsing) */
	off_t offset;
/** line length (in bytes) */
	int len;
/** array of field offsets (where each field starts) */
	int field_offsets[VRTTXT_FIELDS_MAX];
/** number of field into the record */
	int num_fields;
/** validity flag */
	int error;
    };

/**
 Container for Virtual Text record (line) offsets 
 */
    struct vrttxt_row
    {
/* a struct storing Row offsets */
/** Line Number */
	int line_no;
/** start offset */
	off_t offset;
/** record (line) length (in bytes) */
	int len;
/** number of fields into this record */
	int num_fields;
    };

/**
 Container for Virtual Text block of records
 */
    struct vrttxt_row_block
    {
/*
/ for efficiency sake, individual Row offsets 
/ are grouped in reasonably sized blocks
*/
/** array of records [lines] */
	struct vrttxt_row rows[VRTTXT_BLOCK_MAX];
/** number of records into the array */
	int num_rows;
/** min Line Number */
	int min_line_no;
/** max Line Number */
	int max_line_no;
/** pointer to next item [linked list] */
	struct vrttxt_row_block *next;
    };

/** 
 Container for Virtual Text column (field) header
 */
    struct vrttxt_column_header
    {
/* a struct representing a Column (aka Field) header */
/** column name */
	char *name;
/** data type: one of GAIA_NULL_VALUE, GAIA_INT_VALUE, GAIA_DOUBLE_VALUE, GAIA_TEXT_VALUE */
	int type;
    };

/**
 Container for Virtual Text file handling
 */
    typedef struct vrttxt_reader
    {
/* the main TXT-Reader struct */
/** array of columns (fields) */
	struct vrttxt_column_header columns[VRTTXT_FIELDS_MAX];
/** FILE handle */
	FILE *text_file;
/** handle to ICONV converter object */
	void *toUtf8;		/* the UTF-8 ICONV converter */
/** field separator character */
	char field_separator;
/** text separator character (quote) */
	char text_separator;
/** decimal separator */
	char decimal_separator;
/** TRUE if the first line contains column names */
	int first_line_titles;
/** validity flag */
	int error;
/** pointer to first block of records [linked list] */
	struct vrttxt_row_block *first;
/** pointer to last block of records [linked list] */
	struct vrttxt_row_block *last;
/** array of pointers to individual records [lines] */
	struct vrttxt_row **rows;
/** number of records */
	int num_rows;
/** current Line Number */
	int line_no;
/** max number of columns (fields) */
	int max_fields;
/** current buffer size */
	int current_buf_sz;
/** current buffer offset [parsing] */
	int current_buf_off;
/** I/O buffer */
	char *line_buffer;
/** current field buffer */
	char *field_buffer;
/** array of field offsets [current record] */
	int field_offsets[VRTTXT_FIELDS_MAX];
/** array of field lengths [current record] */
	int field_lens[VRTTXT_FIELDS_MAX];
/** max field [current record] */
	int max_current_field;
/** current record [line] ready for parsing */
	int current_line_ready;
    } gaiaTextReader;
/**
 Typedef for Virtual Text file handling structure

 \sa gaiaTextReader
 */
    typedef gaiaTextReader *gaiaTextReaderPtr;

#endif				/* end ICONV (text reader) */

#ifdef __cplusplus
}
#endif

#endif				/* _GG_STRUCTS_H */
/**************** End file: gg_structs.h **********/


/**************** Begin file: gg_core.h **********/


/**
 \file gg_core.h

 Geometry handling functions: core
 */

#ifndef _GG_CORE_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GG_CORE_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* function prototypes */

/**
 Safely frees any dynamic memory block allocated by the library itself

 \param ptr pointer to dynamicly allocated memory

 \note on some platforms (most notably, Microsoft Windows) many different
 runtime libraries may actually support the same process.
 \n attempting to free() a memory block allocated by a different runtime 
 module may easily cause fatal memory corruption.
 */
    GAIAGEO_DECLARE void gaiaFree (void *ptr);

/**
 Allocates a 2D POINT [XY]

 \param x the X coordinate.
 \param y the Y coordinate.
 
 \return the pointer to the newly created POINT object: NULL on failure

 \sa gaiaFreePoint

 \note you are responsible to destroy (before or after) any allocated
 POINT, unless you've passed ownership of the POINT object to some 
 further object: in this case destroying the higher order object will
 implicitly destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaAllocPoint (double x, double y);

/**
 Allocates a 3D POINT [XYZ]

 \param x the X coordinate.
 \param y the Y coordinate.
 \param z the Z coordinate.
 
 \return the pointer to the newly created POINT object: NULL on failure

 \sa gaiaFreePoint

 \note you are responsible to destroy (before or after) any allocated
 POINT, unless you've passed ownership of the POINT object to some 
 further object: in this case destroying the higher order object will
 implicitly destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaAllocPointXYZ (double x, double y,
						    double z);

/**
 Allocates a 2D POINT [XYM]

 \param x the X coordinate.
 \param y the Y coordinate.
 \param m the M measure.
 
 \return the pointer to the newly created POINT object: NULL on failure

 \sa gaiaFreePoint

 \note you are responsible to destroy (before or after) any allocated
 POINT, unless you've passed ownership of the POINT object to some 
 further object: in this case destroying the higher order object will
 implicitly destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaAllocPointXYM (double x, double y,
						    double m);

/**
 Allocates a 3D POINT [XYZM]

 \param x the X coordinate.
 \param y the Y coordinate.
 \param z the Z coordinate.
 \param m the M measure.
 
 \return the pointer to the newly created POINT object: NULL on failure

 \sa gaiaFreePoint

 \note you are responsible to destroy (before or after) any allocated
 POINT, unless you've passed ownership of the POINT object to some 
 further object: in this case destroying the higher order object will
 implicitly destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaAllocPointXYZM (double x, double y,
						     double z, double m);

/**
 Destroys a POINT object

 \param ptr pointer to the POINT object to be destroyed

 \sa gaiaAllocPoint, gaiaAllocPointXYZ, gaiaAllocPointXYM, gaiaAllocPointXYZM

 \note attempting to destroy any POINT object whose ownnership has already
 been transferred to some other (higher order) object is a serious
 error, and will easily cause severe memory corruption.
 */
    GAIAGEO_DECLARE void gaiaFreePoint (gaiaPointPtr ptr);

/**
 Allocates a 2D LINESTRING [XY]

 \param vert number of points [aka vertices] into the Linestring

 \return the pointer to newly created LINESTRING object: NULL on failure

 \sa gaiaFreeLinestring, gaiaLineSetPoint, gaiaLineGetPoint, gaiaSetPoint,
 gaiaGetPoint

 \note you are responsible to destroy (before or after) any allocated LINESTRING,
 unless you've passed ownership of the LINESTRING object to some further 
 object: in this case destroying the higher order object will implicitly 
 destroy any contained child object. 
 */
    GAIAGEO_DECLARE gaiaLinestringPtr gaiaAllocLinestring (int vert);

/**
 Allocates a 3D LINESTRING [XYZ]

 \param vert number of points [aka vertices] into the Linestring

 \return the pointer to newly created LINESTRING object: NULL on failure

 \sa gaiaFreeLinestring, gaiaLineSetPoint, gaiaLineGetPoint, gaiaSetPointXYZ,
 gaiaGetPointXYZ

 \note you are responsible to destroy (before or after) any allocated LINESTRING, 
 unless you've passed ownership of the LINESTRING object to some further  
 object: in this case destroying the higher order object will implicitly 
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaLinestringPtr gaiaAllocLinestringXYZ (int vert);

/**
 Allocates a 2D LINESTRING [XYM]

 \param vert number of points [aka vertices] into the Linestring

 \return the pointer to newly created LINESTRING object: NULL on failure

 \sa gaiaFreeLinestring, gaiaLineSetPoint, gaiaLineGetPoint, gaiaSetPointXYM,
 gaiaGetPointXYM

 \note you are responsible to destroy (before or after) any allocated LINESTRING, 
 unless you've passed ownership of the LINESTRING object to some further  
 object: in this case destroying the higher order object will implicitly 
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaLinestringPtr gaiaAllocLinestringXYM (int vert);

/**
 Allocates a 3D LINESTRING [XYZM]

 \param vert number of points [aka vertices] into the Linestring

 \return the pointer to newly created LINESTRING object: NULL on failure

 \sa gaiaFreeLinestring, gaiaLineSetPoint, gaiaLineGetPoint, gaiaSetPointXYZM,
 gaiaGetPointXYZM

 \note you are responsible to destroy (before or after) any allocated LINESTRING, 
 unless you've passed ownership of the LINESTRING object to some further  
 object: in this case destroying the higher order object will implicitly 
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaLinestringPtr gaiaAllocLinestringXYZM (int vert);

/**
 Destroys a LINESTRING object

 \param ptr pointer to the LINESTRING object to be destroyed

 \sa gaiaAllocLinestring, gaiaAllocLinestringXYZ, gaiaAllocLinestringXYM,
  gaiaAllocLinestringXYZM

 \note attempting to destroy any LINESTRING object whose ownnership has already
 been transferred to some other (higher order) object is a serious
 error, and will easily cause severe memory corruption.
 */
    GAIAGEO_DECLARE void gaiaFreeLinestring (gaiaLinestringPtr ptr);

/**
 Copies coordinates between two LINESTRING objects

 \param dst destination LINESTRING [output]
 \param src origin LINESTRING [input]

 \note both LINESTRING objects must have exactly the same number of points:
 if dimensions aren't the same for both objects, then the appropriate 
 conversion will be silently applied.
 */
    GAIAGEO_DECLARE void gaiaCopyLinestringCoords (gaiaLinestringPtr dst,
						   gaiaLinestringPtr src);

/**
 Allocates a 2D RING [XY]

 \param vert number of points [aka vertices] into the Ring

 \return the pointer to newly created RING object: NULL on failure

 \sa gaiaFreeRing, gaiaRingSetPoint, gaiaRingGetPoint, gaiaSetPoint,
 gaiaGetPoint

 \note you are responsible to destroy (before or after) any allocated RING,
 unless you've passed ownership of the RING object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaRingPtr gaiaAllocRing (int vert);

/**
 Allocates a 3D RING [XYZ]

 \param vert number of points [aka vertices] into the Ring

 \return the pointer to newly created RING object: NULL on failure

 \sa gaiaFreeRing, gaiaRingSetPoint, gaiaRingGetPoint, gaiaSetPointXYZ,
 gaiaGetPointXYZ

 \note you are responsible to destroy (before or after) any allocated RING,
 unless you've passed ownership of the RING object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaRingPtr gaiaAllocRingXYZ (int vert);

/**
 Allocates 2D RING [XYM]

 \param vert number of points [aka vertices] into the Ring

 \return the pointer to newly created RING object: NULL on failure

 \sa gaiaFreeRing, gaiaRingSetPoint, gaiaRingGetPoint, gaiaSetPointXYM,
 gaiaGetPointXYM

 \note you are responsible to destroy (before or after) any allocated RING,
 unless you've passed ownership of the RING object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaRingPtr gaiaAllocRingXYM (int vert);

/**
 Allocates a 3D RING [XYZM]

 \param vert number of points [aka vertices] into the Ring

 \return the pointer to newly created RING object: NULL on failure

 \sa gaiaFreeRing, gaiaRingSetPoint, gaiaRingGetPoint, gaiaSetPointXYZM,
 gaiaSetPointXYZM

 \note you are responsible to destroy (before or after) any allocated RING,
 unless you've passed ownership of the RING object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaRingPtr gaiaAllocRingXYZM (int vert);

/**
 Destroys a RING object

 \param ptr pointer to the RING object to be destroyed

 \sa gaiaAllocRing, gaiaAllocRingXYZ, gaiaAllocRingXYM,
  gaiaAllocRingXYZM

 \note attempting to destroy any RING object whose ownnership has already
 been transferred to some other (higher order) object is a serious
 error, and will easily cause severe memory corruption.
 */
    GAIAGEO_DECLARE void gaiaFreeRing (gaiaRingPtr ptr);

/**
 Copies coordinates betwee two RING objects

 \param dst destination RING [output]
 \param src origin RING [input]

 \note both RING objects must have exactly the same number of points:
 if dimensions aren't the same for both objects, then the appropriate
 conversion will be silently applied.
 */
    GAIAGEO_DECLARE void gaiaCopyRingCoords (gaiaRingPtr dst, gaiaRingPtr src);

/**
 Allocates a 2D POLYGON [XY]

 \param vert number of points [aka vertices] into the Exterior Ring.
 \param holes number of Interior Rings [0, if no Interior Ring is required].

 \return the pointer to newly created POLYGON object: NULL on failure

 \sa gaiaFreePolygon

 \note you are responsible to destroy (before or after) any allocated POLYGON,
 unless you've passed ownership of the POLYGON object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaAllocPolygon (int vert, int holes);

/**
 Allocates a 3D POLYGON [XYZ]
    
 \param vert number of points [aka vertices] into the Exterior Ring.
 \param holes number of Interior Rings [0, if no Interior Ring is required].
    
 \return the pointer to newly created POLYGON object: NULL on failure
    
 \sa gaiaFreePolygon
    
 \note you are responsible to destroy (before or after) any allocated POLYGON,
 unless you've passed ownership of the POLYGON object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaAllocPolygonXYZ (int vert, int holes);

/**
 Allocates a 2D POLYGON [XYM]
    
 \param vert number of points [aka vertices] into the Exterior Ring.
 \param holes number of Interior Rings [0, if no Interior Ring is required].
    
 \return the pointer to newly created POLYGON object: NULL on failure
    
 \sa gaiaFreePolygon
    
 \note you are responsible to destroy (before or after) any allocated POLYGON,
 unless you've passed ownership of the POLYGON object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaAllocPolygonXYM (int vert, int holes);

/**
 Allocates a 3D POLYGON [XYZM]
    
 \param vert number of points [aka vertices] into the Exterior Ring.
 \param holes number of Interior Rings [may by 0, if no Interior Ring is required].
    
 \return the pointer to newly created POLYGON object: NULL on failure
    
 \sa gaiaFreePolygon
    
 \note you are responsible to destroy (before or after) any allocated POLYGON,
 unless you've passed ownership of the POLYGON object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaAllocPolygonXYZM (int vert, int holes);

/**
 Allocates a POLYGON

 \param ring pointer to a valid RING object: assumed to be the Polygon's
  Exterior Ring.

 \return the pointer to newly created POLYGON object: NULL on failure

 \sa gaiaAllocRing, gaiaAllocRingXYZ, gaiaAllocRingXYM, gaiaAllocRingXYZM,
 gaiaFreePolygon

 \note you are responsible to destroy (before or after) any allocated POLYGON,
 unless you've passed ownership of the POLYGON object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object. 
 \n Ownership of passed Ring object will be transferred to the 
 Polygon object being created. 
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaCreatePolygon (gaiaRingPtr ring);

/**
 Destroys a POLYGON object

 \param polyg pointer to the POLYGON object to be destroyed

 \sa gaiaAllocPolygon, gaiaAllocPolygonXYZ, gaiaAllocPolygonXYM,
  gaiaAllocPolygonXYZM, gaiaCreatePolygon

 \note attempting to destroy any POLYGON object whose ownnership has already
 been transferred to some other (higher order) object is a serious
 error, and will easily cause severe memory corruption.
 \n Ownerhip of each RING object referenced by a POLYGON object alway belongs
 to the POLYGON itself, so destroying the POLYGON will surely destroy
 any related RING as well.
 */
    GAIAGEO_DECLARE void gaiaFreePolygon (gaiaPolygonPtr polyg);

/**
 Allocates a 2D Geometry [XY]

 \return the pointer to newly created Geometry object: NULL on failure

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaAllocGeomColl (void);

/**
 Allocates a 3D Geometry [XYZ]

 \return the pointer to newly created Geometry object: NULL on failure

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaAllocGeomCollXYZ (void);

/**
 Allocates a 2D Geometry [XYM]

 \return the pointer to newly created Geometry object: NULL on failure

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaAllocGeomCollXYM (void);

/**
 Allocates a 3D Geometry [XYZM]

 \return the pointer to newly created Geometry object: NULL on failure

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further
 object: in this case destroying the higher order object will implicitly
 destroy any contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaAllocGeomCollXYZM (void);

/**
 Destroys a Geometry object

 \param geom pointer to the Geometry object to be destroyed

 \sa gaiaAllocGeomColl, gaiaAllocGeomCollXYZ, gaiaAllocGeomCollXYM,
  gaiaAllocGeomCollXYZM

 \note attempting to destroy any Geometry object whose ownnership has already
 been transferred to some other (higher order) object is a serious
 error, and will easily cause severe memory corruption.
 \n Ownerhip of each POINT, LINESTRING or POLYGON object referenced by a
 Geometry object alway belongs to the Geometry itself, so destroying the 
 Geometry will surely destroy any related elementary geometry item as well.
 */
    GAIAGEO_DECLARE void gaiaFreeGeomColl (gaiaGeomCollPtr geom);

/**
 Creates a new 2D Point [XY] object into a Geometry object

 \param p pointer to the Geometry object
 \param x X coordinate of the Point to be created
 \param y X coordinate of the Point to be created

 \note ownership of the newly created POINT object belongs to the Geometry
 object.
 */
    GAIAGEO_DECLARE void gaiaAddPointToGeomColl (gaiaGeomCollPtr p, double x,
						 double y);

/**
 Creates a new 3D Point [XYZ] object into a Geometry object

 \param p pointer to the Geometry object
 \param x X coordinate of the Point to be created
 \param y X coordinate of the Point to be created
 \param z Z coordinate of the Point to be created

 \note ownership of the newly created POINT object belongs to the Geometry
 object.
 */
    GAIAGEO_DECLARE void gaiaAddPointToGeomCollXYZ (gaiaGeomCollPtr p, double x,
						    double y, double z);

/**
 Creates a new 2D Point [XYM] object into a Geometry object

 \param p pointer to the Geometry object
 \param x X coordinate of the Point to be created
 \param y X coordinate of the Point to be created
 \param m M measure of the Point to be created

 \note ownership of the newly created POINT object belongs to the Geometry
 object.
 */
    GAIAGEO_DECLARE void gaiaAddPointToGeomCollXYM (gaiaGeomCollPtr p, double x,
						    double y, double m);

/**
 Creates a new 3D Point [XYZM] object into a Geometry object

 \param p pointer to the Geometry object
 \param x X coordinate of the Point to be created
 \param y X coordinate of the Point to be created
 \param z Z coordinate of the Point to be created
 \param m M measure of the Point to be created

 \note ownership of the newly created POINT object belongs to the Geometry
 object.
 */
    GAIAGEO_DECLARE void gaiaAddPointToGeomCollXYZM (gaiaGeomCollPtr p,
						     double x, double y,
						     double z, double m);

/**
 Creates a new Linestring object into a Geometry object

 \param p pointer to the Geometry object.
 \param vert number of points [aka vertices] into the Linestring.

 \return the pointer to newly created Linestring: NULL on failure.

 \note ownership of the newly created Linestring object belongs to the Geometry object. 
 \n the newly created Linestring will have the same dimensions as the Geometry has.
 */
    GAIAGEO_DECLARE gaiaLinestringPtr
	gaiaAddLinestringToGeomColl (gaiaGeomCollPtr p, int vert);

/**
 Inserts an already existing Linestring object into a Geometry object

 \param p pointer to the Geometry object.
 \param line pointer to the Linestring object.

 \note ownership of the Linestring object will be transferred to the
 Geometry object.
 */
    GAIAGEO_DECLARE void gaiaInsertLinestringInGeomColl (gaiaGeomCollPtr p,
							 gaiaLinestringPtr
							 line);

/**
 Creates a new Polygon object into a Geometry object

 \param p pointer to the Geometry object.
 \param vert number of points [aka vertices] into the Polygon's Exterior Ring.
 \param interiors number of Interiors Rings [0, if no Interior Ring is required]

 \return the pointer to newly created Polygon: NULL on failure.

 \note ownership of the newly created Polygon object belongs to the Geometry object.
 \n the newly created Polygon will have the same dimensions as the Geometry has.
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaAddPolygonToGeomColl (gaiaGeomCollPtr p,
							     int vert,
							     int interiors);

/**
 Creates a new Polygon object into a Geometry object starting from an
 already existing Ring object

 \param p pointer to the Geometry object.
 \param ring pointer to the Ring object [assumed to represent to Polygon's
 Exterior Ring].

 \return the pointer to the newly created Polygon object: NULL on failure.

 \note ownership of the Ring object will be transferred to the
 Polygon object, and the Polygon object ownerships belongs to the Geometry object.
 \n the Polygon object will have the same dimensions as the Ring object has.
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaInsertPolygonInGeomColl (gaiaGeomCollPtr
								p,
								gaiaRingPtr
								ring);

/**
 Creates a new Interior Ring object into a Polygon object

 \param p pointer to the Polygon object.
 \param pos relative position index [first Interior Ring has index 0].
 \param vert number of points (aka vertices) into the Ring.

 \return the pointer to the newly created Ring object: NULL on failure.

 \sa gaiaAllocPolygon, gaiaAllocPolygonXYZ, gaiaAllocPolygonXYM,
 gaiaAllocPolygonXYZM

 \note ownership of the Ring object belongs to the Polygon object.
 \n the newly created Ring will have the same dimensions the Polygon has.
 */
    GAIAGEO_DECLARE gaiaRingPtr gaiaAddInteriorRing (gaiaPolygonPtr p, int pos,
						     int vert);

/**
 Inserts an already existing Ring object into a Polygon object

 \param p pointer to the Polygon object
 \param ring pointer to the Ring object

 \sa gaiaAddRingToPolygon

 \note ownership of the Ring object still remains to the calling procedure
 (a duplicated copy of the original Ring will be inserted into the Polygon).
 \n the newly created Polygon will have the same dimensions as the Ring has.
 \n if required the Polygon's Interior Rings count could be increased.
 */
    GAIAGEO_DECLARE void gaiaInsertInteriorRing (gaiaPolygonPtr p,
						 gaiaRingPtr ring);

/**
 Inserts an already existing Ring object into a Polygon object

 \param polyg pointer to the Polygon object
 \param ring pointer to the Ring object

 \sa gaiaInsertInteriorRing

 \note ownership of the Ring object will be tranferred to the Polygon object.
 \n the newly created Polygon will have the same dimensions as the Ring has.
 \n if required the Polygon's Interior Rings count could be increased.
 */
    GAIAGEO_DECLARE void gaiaAddRingToPolyg (gaiaPolygonPtr polyg,
					     gaiaRingPtr ring);

/**
 Duplicates a Linestring object

 \param line pointer to Linestring object [origin].

 \return the pointer to newly created Linestring object: NULL on failure.

 \sa gaiaCloneRing, gaiaClonePolygon, gaiaCloneGeomColl,
 gaiaCloneGeomCollPoints, gaiaCloneGeomCollLinestrings, 
 gaiaCloneGeomCollPolygons

 \note the newly created object is an exact copy of the original one. 
 */
    GAIAGEO_DECLARE gaiaLinestringPtr gaiaCloneLinestring (gaiaLinestringPtr
							   line);

/**
 Duplicates a Ring object

 \param ring pointer to Ring object [origin].

 \return the pointer to newly created Ring object: NULL on failure.

 \sa gaiaCloneLinestring, gaiaClonePolygon, gaiaCloneGeomColl,
 gaiaCloneGeomCollPoints, gaiaCloneGeomCollLinestrings, 
 gaiaCloneGeomCollPolygons

 \note the newly created object is an exact copy of the original one. 
 */
    GAIAGEO_DECLARE gaiaRingPtr gaiaCloneRing (gaiaRingPtr ring);

/**
 Duplicates a Polygon object

 \param polyg pointer to Polygon object [origin].

 \return the pointer to newly created Polygon object: NULL on failure.

 \sa gaiaCloneLinestring, gaiaCloneRing, gaiaCloneGeomColl,
 gaiaCloneGeomCollPoints, gaiaCloneGeomCollLinestrings, 
 gaiaCloneGeomCollPolygons

 \note the newly created object is an exact copy of the original one. 
 */
    GAIAGEO_DECLARE gaiaPolygonPtr gaiaClonePolygon (gaiaPolygonPtr polyg);

/**
 Duplicates a Geometry object

 \param geom pointer to Geometry object [origin].

 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneLinestring, gaiaCloneRing, gaiaClonePolygon, 
 gaiaCloneGeomCollPoints, gaiaCloneGeomCollLinestrings,
 gaiaCloneGeomCollPolygons, gaiaCastGeomCollToXY, gaiaCastGeomCollToXYZ,
 gaiaCastGeomCollToXYM, gaiaCastGeomCollToXYZM, gaiaExtractPointsFromGeomColl,
 gaiaExtractLinestringsFromGeomColl, gaiaExtractPolygonsFromGeomColl,
 gaiaMergeGeometries

 \note the newly created object is an exact copy of the original one. 
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaCloneGeomColl (gaiaGeomCollPtr geom);

/**
 Duplicates a Geometry object [Points only]

 \param geom pointer to Geometry object [origin].

 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneLinestring, gaiaCloneRing, gaiaClonePolygon, gaiaCloneGeomColl,
 gaiaCloneGeomCollLinestrings, 
 gaiaCloneGeomCollPolygons

 \note the newly created object is an exact copy of the original one; except
 in that only Point objects will be copied.
 \n Caveat: an empty Geometry could be returned.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaCloneGeomCollPoints (gaiaGeomCollPtr
							     geom);

/**
 Duplicates a Geometry object [Linestrings only]
                                                             
 \param geom pointer to Geometry object [origin].
        
 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneLinestring, gaiaCloneRing, gaiaClonePolygon, gaiaCloneGeomColl,
 gaiaCloneGeomCollPoints, gaiaCloneGeomCollPolygons

 \note the newly created object is an exact copy of the original one; except
 in that only Linestrings objects will be copied.
 \n Caveat: an empty Geometry could be returned.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr
	gaiaCloneGeomCollLinestrings (gaiaGeomCollPtr geom);

/**
 Duplicates a Geometry object [Polygons only]
 
 \param geom pointer to Geometry object [origin].
 
 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneLinestring, gaiaCloneRing, gaiaClonePolygon, gaiaCloneGeomColl,
 gaiaCloneGeomCollPoints, gaiaCloneGeomCollLinestrings

 \note the newly created object is an exact copy of the original one; except
 in that only Polygons objects will be copied.
 \n Caveat: an empty Geometry could be returned.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaCloneGeomCollPolygons (gaiaGeomCollPtr
							       geom);

/**
 Duplicates a Geometry object [casting dimensions to 2D XY]

 \param geom pointer to Geometry object [origin].

 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneGeomColl, gaiaCastGeomCollToXYZ,
 gaiaCastGeomCollToXYM, gaiaCastGeomCollToXYZM

 \note the newly created object is an exact copy of the original one; except
 in that any elementary item  will be casted to 2D [XY] dimensions.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaCastGeomCollToXY (gaiaGeomCollPtr geom);

/**
 Duplicates a Geometry object [casting dimensions to 3D XYZ]

 \param geom pointer to Geometry object [origin].

 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneGeomColl, gaiaCastGeomCollToXY, 
 gaiaCastGeomCollToXYM, gaiaCastGeomCollToXYZM

 \note the newly created object is an exact copy of the original one; except
 in that any elementary item  will be casted to 3D [XYZ] dimensions.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaCastGeomCollToXYZ (gaiaGeomCollPtr
							   geom);

/**
 Duplicates a Geometry object [casting dimensions to 2D XYM]

 \param geom pointer to Geometry object [origin].

 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneGeomColl, gaiaCastGeomCollToXY, gaiaCastGeomCollToXYZ,
 gaiaCastGeomCollToXYZM

 \note the newly created object is an exact copy of the original one; except
 in that any elementary item  will be casted to 2D [XYM] dimensions.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaCastGeomCollToXYM (gaiaGeomCollPtr
							   geom);

/**
 Duplicates a Geometry object [casting dimensions to 3D XYZM]

 \param geom pointer to Geometry object [origin].

 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaCloneGeomColl, gaiaCastGeomCollToXY, gaiaCastGeomCollToXYZ,
 gaiaCastGeomCollToXYM

 \note the newly created object is an exact copy of the original one; except
 in that any elementary item  will be casted to 3D [XYZM] dimensions.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaCastGeomCollToXYZM (gaiaGeomCollPtr
							    geom);

/**
 Gets coodinates from a Linestring's Point

 \param ln pointer to Linestring object.
 \param v relative position of Point: first Point has index 0
 \param x on completion this variable will contain the Point X coordinate.
 \param y on completion this variable will contain the Point Y coordinate.
 \param z on completion this variable will contain the Point Z coordinate.
 \param m on completion this variable will contain the Point M measure.

 \return 0 on failure: any other different value on success.

 \sa gaiaLineSetPoint, gaiaGetPoint, gaiaGetPointXYZ, gaiaGetPointXYM,
 gaiaGetPointXYZM

 \note this function perform the same identical task performed by
 gaiaGetPoint(), gaiaGetPointXYZ(), gaiaGetPointXYM() and gaiaGetPointXYZM()
 macros.
 \n using the gaiaLineGetPoint() function is a little bit slower but is
 intrinsically safest, because misused macros can easily cause severe
 memory corruption.
 \n gaiaLineGetPoint() instead will always ensure that the appropriate
 dimensions (as declared by the Linestring object) will be correctly used.
 */
    GAIAGEO_DECLARE int gaiaLineGetPoint (gaiaLinestringPtr ln, int v,
					  double *x, double *y, double *z,
					  double *m);

/**
 Sets coodinates for a Linestring's Point

 \param ln pointer to Linestring object.
 \param v relative position of Point: first Point has index 0
 \param x the Point's X coordinate.
 \param y the Point's Y coordinate.
 \param z the Point's Z coordinate.
 \param m the Point's M measure.

 \return 0 on failure: any other different value on success.

 \sa gaiaLineGetPoint, gaiaSetPoint, gaiaSetPointXYZ, gaiaSetPointXYM,
 gaiaSetPointXYZM

 \note this function perform the same identical task performed by 
 gaiaSetPoint(), gaiaSetPointXYZ(), gaiaSetPointXYM() and gaiaSetPointXYZM()
 macros.
 \n using the gaiaLineSetPoint() function is a little bit slower but is
 intrinsically safest, because misused macros can easily cause severe
 memory corruption.
 \n gaiaLineSetPoint() instead will always ensure that the appropriate 
 dimensions (as declared by the Linestring object) will be correctly used.
 */
    GAIAGEO_DECLARE int gaiaLineSetPoint (gaiaLinestringPtr ln, int v, double x,
					  double y, double z, double m);

/**
 Gets coodinates from a Ring's Point

 \param rng pointer to Ring object.
 \param v relative position of Point: first Point has index 0
 \param x on completion this variable will contain the Point X coordinate.
 \param y on completion this variable will contain the Point Y coordinate.
 \param z on completion this variable will contain the Point Z coordinate.
 \param m on completion this variable will contain the Point M measure.

 \return 0 on failure: any other different value on success.

 \sa gaiaRingSetPoint, gaiaGetPoint, gaiaGetPointXYZ, gaiaGetPointXYM,
 gaiaGetPointXYZM

 \note this function perform the same identical task performed by
 gaiaGetPoint(), gaiaGetPointXYZ(), gaiaGetPointXYM() and gaiaGetPointXYZM()
 macros.
 \n using the gaiaRingGetPoint() function is a little bit slower but is
 intrinsically safest, because misused macros can easily cause severe
 memory corruption.
 \n gaiaRingGetPoint() instead will always ensure that the appropriate
 dimensions (as declared by the Ring object) will be correctly used.
 */
    GAIAGEO_DECLARE int gaiaRingGetPoint (gaiaRingPtr rng, int v, double *x,
					  double *y, double *z, double *m);

/**
 Sets coodinates for a Ring's Point

 \param rng pointer to Ring object.
 \param v relative position of Point: first Point has index 0
 \param x the Point's X coordinate.
 \param y the Point's Y coordinate.
 \param z the Point's Z coordinate.
 \param m the Point's M measure.

 \return 0 on failure: any other different value on success.

 \sa gaiaRingGetPoint, gaiaGetPoint, gaiaGetPointXYZ, gaiaSetPointXYM,
 gaiaSetPointXYZM

 \note this function perform the same identical task performed by
 gaiaSetPoint(), gaiaSetPointXYZ(), gaiaSetPointXYM() and gaiaSetPointXYZM()
 macros.
 \n using the gaiaRingSetPoint() function is a little bit slower but is
 intrinsically safest, because misused macros can easily cause severe
 memory corruption.
 \n gaiaRingSetPoint() instead will always ensure that the appropriate
 dimensions (as declared by the Ring object) will be correctly used.
 */
    GAIAGEO_DECLARE int gaiaRingSetPoint (gaiaRingPtr rng, int v, double x,
					  double y, double z, double m);

/**
 Determines OGC dimensions for a Geometry object

 \param geom pointer to Geometry object
 
 \return OGC dimensions

 \note OGC dimensions are defined as follows:
 \li if the Geometry doesn't contain any elementary item: \b -1
 \li if the Geometry only contains Point items: \b 0
 \li if the Geometry only contains Point / Linestring items: \b 1
 \li if the Geometry contains some Polygon item: \b 2
 */
    GAIAGEO_DECLARE int gaiaDimension (gaiaGeomCollPtr geom);

/**
 Determines the corresponding Type for a Geometry object
 
 \param geom pointer to Geometry object

 \return the corresponding Geometry Type

 \note Type is one of: GAIA_POINT, GAIA_LINESTRING, GAIA_POLYGON,
 GAIA_MULTIPOINT, GAIA_MULTILINESTRING, GAIA_MULTIPOLYGON,
 GAIA_GEOMETRYCOLLECTION, GAIA_POINTZ, GAIA_LINESTRINGZ, GAIA_POLYGONZ,
 GAIA_MULTIPOINTZ, GAIA_MULTILINESTRINGZ, GAIA_MULTIPOLYGONZ,
 GAIA_GEOMETRYCOLLECTIONZ, GAIA_POINTM, GAIA_LINESTRINGM, GAIA_POLYGONM,
 GAIA_MULTIPOINTM, GAIA_MULTILINESTRINGM, GAIA_MULTIPOLYGONM,
 GAIA_GEOMETRYCOLLECTIONM, GAIA_POINTZM, GAIA_LINESTRINGZM, GAIA_POLYGONZM,
 GAIA_MULTIPOINTZM, GAIA_MULTILINESTRINGZM, GAIA_MULTIPOLYGONZM,
 GAIA_GEOMETRYCOLLECTIONZM
 \n on failure GAIA_NONE will be returned.
 */
    GAIAGEO_DECLARE int gaiaGeometryType (gaiaGeomCollPtr geom);

/**
 Determines the corresponding Type for a Geometry object

 \param geom pointer to Geometry object

 \return the corresponding Geometry Type

 \sa gaiaGeometryType

 \note Type is one of: GAIA_POINT, GAIA_LINESTRING, GAIA_POLYGON,
 GAIA_MULTIPOINT, GAIA_MULTILINESTRING, GAIA_MULTIPOLYGON,
 GAIA_GEOMETRYCOLLECTION
 \n on failure GAIA_NONE will be returned.

 \remark deprecated function (used in earlier SpatiaLite versions).
 */
    GAIAGEO_DECLARE int gaiaGeometryAliasType (gaiaGeomCollPtr geom);

/**
 Checks for empty Geometry object

 \param geom pointer to Geometry object

 \return 0 if the Geometry is empty: otherwise any other different value.

 \note an empty Geometry is a Geometry not containing any elementary
 item: i.e. no Points, no Linestrings and no Polygons at all.
 */
    GAIAGEO_DECLARE int gaiaIsEmpty (gaiaGeomCollPtr geom);

/**
 Checks for toxic Geometry object

 \param geom pointer to Geometry object

 \return 0 if the Geometry is not toxic: otherwise any other different value.

 \sa gaiaSanitize

 \note a \b toxic Geometry is a Geometry containing severely malformed
 Polygons: i.e. containing less than 4 Points, unclosed Rings and so on.
 \n Attempting to pass any toxic Geometry to GEOS supported functions
 will easily cause a crash.
 */
    GAIAGEO_DECLARE int gaiaIsToxic (gaiaGeomCollPtr geom);

/**
 Attempts to sanitize a possibly malformed Geometry object

 \param org pointer to Geometry object.

 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaIsToxic

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaSanitize()
 \n the output Geometry will surely have:
 \li no repeated Points on Linestrings or Rings (i.e. consecutive Points 
 sharing exactly the same coordinates): any repeated Point will be suppressed,
 simply leaving only the first occurrence.
 \li proper Ring closure: for sure any Ring will have exactly coinciding
 first and last Points.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaSanitize (gaiaGeomCollPtr org);


/**
 Attempts to resolve a (Multi)Linestring from a Geometry object

 \param geom pointer to Geometry object.
 \param force_multi: 0 if the returned Geometry could represent a Linestring:
 any other value if casting to MultiLinestring is required unconditionally.

 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaDissolveSegments, gaiaDissolvePoints

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaLinearize()
 \n the input Geometry is expected to contain Polygons only: then any Ring 
 will be transformed into the corresponding Linestring.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaLinearize (gaiaGeomCollPtr geom,
						   int force_multi);

/**
 Attempts to resolve a collection of Segments from a Geometry object

 \param geom pointer to Geometry object.

 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaLinearize, gaiaDissolvePoints

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaDissolveSegments()
 \n the input Geometry can be of any arbitrary type:
 \li any Point will be copied untouched.
 \li any Linestring will be dissolved into Segments.
 \li any Ring will be dissolved into Segments.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaDissolveSegments (gaiaGeomCollPtr geom);

/** 
 Attempts to resolve a collection of Points from a Geometry object

 \param geom pointer to Geometry object.
 
 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaLinearize, gaiaDissolveSegments

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaDissolvePoints()
 \n the input Geometry can be of any arbitrary type:
 \li any Point will be copied untouched.
 \li any Linestring will be dissolved into sparse Points.
 \li any Ring will be dissolved into sparse Points.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaDissolvePoints (gaiaGeomCollPtr geom);

/**
 Extracts any Point from a Geometry object

 \param geom pointer to Geometry object

 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaExtractLinestringsFromGeomColl,
 gaiaExtractPolygonsFromGeomColl, gaiaCloneGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaExtractPointsFromGeomColl()
 \n the newly created Geometry will contain any Point contained into the
 input Geometry.
 \n if the input Geometry doesn't contains any Point, then NULL will be returned.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr
	gaiaExtractPointsFromGeomColl (gaiaGeomCollPtr geom);

/**
 Extracts any Linestring from a Geometry object

 \param geom pointer to Geometry object

 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaExtractPointsFromGeomColl, gaiaExtractPolygonsFromGeomColl, 
 gaiaCloneGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaExtractLinestringsFromGeomColl()
 \n the newly created Geometry will contain any Linestring contained into the
 input Geometry.
 \n if the input Geometry doesn't contains any Linestring, then NULL will be returned.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr
	gaiaExtractLinestringsFromGeomColl (gaiaGeomCollPtr geom);

/**
 Extracts any Polygon from a Geometry object
        
 \param geom pointer to Geometry object

 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaExtractPointsFromGeomColl, gaiaExtractLinestringsFromGeomColl,
 gaiaCloneGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaExtractPolygonsFromGeomColl()
 \n the newly created Geometry will contain any Polygon contained into the
 input Geometry.
 \n if the input Geometry doesn't contains any Polygon, then NULL will be returned.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr
	gaiaExtractPolygonsFromGeomColl (gaiaGeomCollPtr geom);

/**
 Merges two Geometry objects into a single one

 \param geom1 pointer to first Geometry object.
 \param geom2 pointer to second Geometry object.

 \return the pointer to newly created Geometry: NULL on failure.

 \sa gaiaCloneGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry created by gaiaMergeGeometries()
 \n the newly created Geometry will contain any Point, Linestring and/or
 Polygon contained in both input Geometries.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaMergeGeometries (gaiaGeomCollPtr geom1,
							 gaiaGeomCollPtr geom2);

/**
 Measures the geometric length for a Linestring or Ring

 \param dims dimensions: one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M or GAIA_XY_ZM
 \param coords pointed to COORD mem-array
 \param vert number of Points (aka Vertices) within the COORD mem-array

 \return the calculated geometric length

 \sa gaiaGeomCollLength

 \note \b dims, \b coords and \b vert are usually expected to correspond to
 \b DimensionModel, \b Coords and \b Points members from a gaiaLinestringStruct
 or gaiaRingStruct

 \remark internal method: doesn't require any GEOS support.
 */
    GAIAGEO_DECLARE double gaiaMeasureLength (int dims, double *coords,
					      int vert);

/**
 Measures the geometric area for a Ring object

 \param ring pointer to Ring object

 \return the calculated geometric area

 \sa gaiaGeomCollArea

 \remark internal method: doesn't require any GEOS support.
 */
    GAIAGEO_DECLARE double gaiaMeasureArea (gaiaRingPtr ring);

/**
 Determines the Centroid for a Ring object

 \param ring pointer to Ring object.
 \param rx on completion this variable will contain the centroid X coordinate.
 \param ry on completion this variable will contain the centroid Y coordinate.

 \sa gaiaGeomCollCentroid

 \remark internal method: doesn't require any GEOS support.
 */
    GAIAGEO_DECLARE void gaiaRingCentroid (gaiaRingPtr ring, double *rx,
					   double *ry);

/**
 Determines the direction for a Ring object

 \param p pointer to Ring object

 \return 0 if the ring has counter-clockwise direction; any other different
 value for clockwise direction.
 */
    GAIAGEO_DECLARE void gaiaClockwise (gaiaRingPtr p);

/**
 Check if a Point lays on a Ring surface

 \param ring pointer to Ring object
 \param pt_x Point X coordinate
 \param pt_y Point Y coordinate

 \return 0 if false: any other value if true
 */
    GAIAGEO_DECLARE int gaiaIsPointOnRingSurface (gaiaRingPtr ring, double pt_x,
						  double pt_y);

/**
 Checks if a Point lays on a Polygon surface

 \param polyg pointer to Polygon object
 \param x Point X coordinate
 \param y Point Y coordinate

 \return 0 if false: any other value if true
 */
    GAIAGEO_DECLARE int gaiaIsPointOnPolygonSurface (gaiaPolygonPtr polyg,
						     double x, double y);

/**
 Computes the minimum distance intercurring from a Point and a Linestring or Ring

 \param x0 Point X coordinate
 \param y0 Point Y coordinate
 \param dims dimensions: one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M or GAIA_XY_ZM
 \param coords pointed to COORD mem-array
 \param vert number of Points (aka Vertices) within the COORD mem-array

 \return the calculated minumum distance.

 \note \b dims, \b coords and \b vert are usually expected to correspond to
 \b DimensionModel, \b Coords and \b Points members from a gaiaLinestringStruct
 or gaiaRingStruct
 */
    GAIAGEO_DECLARE double gaiaMinDistance (double x0, double y0,
					    int dims, double *coords, int vert);

/**
 Determines the intesection Point between two Segments

 \param x0 on completion this variable will contain the Intersection X coord
 \param y0 on completion this variable will contain the Intersection Y coord
 \param x1 start Point X of first Segment
 \param y1 start Point Y of first Segment
 \param x2 end Point X of first Segment
 \param y2 end Point Y of first Segment
 \param x3 start Point X of second Segment
 \param y3 start Point Y of second Segment
 \param x4 end Point X of second Segment
 \param y4 end Point Y of second Segment

 \return 0 if the Segments doesn't intersect at all: any other value on
 success.
 */
    GAIAGEO_DECLARE int gaiaIntersect (double *x0, double *y0, double x1,
				       double y1, double x2, double y2,
				       double x3, double y3, double x4,
				       double y4);

/**
 Shifts any coordinate within a Geometry object

 \param geom pointer to Geometry object.
 \param shift_x X axis shift factor.
 \param shift_y Y axis shift factor.

 \sa gaiaScaleCoords, gaiaRotateCoords, gaiaReflectCoords, gaiaSwapCoords
 */
    GAIAGEO_DECLARE void gaiaShiftCoords (gaiaGeomCollPtr geom, double shift_x,
					  double shift_y);

/**
 Scales any coordinate within a Geometry object

 \param geom pointer to Geometry object.
 \param scale_x X axis scale factor.
 \param scale_y Y axis scale factor.

 \sa gaiaShiftCoords, gaiaRotateCoords, gaiaReflectCoords, gaiaSwapCoords
 */
    GAIAGEO_DECLARE void gaiaScaleCoords (gaiaGeomCollPtr geom, double scale_x,
					  double scale_y);

/**
 Rotates any coordinate within a Geometry object

 \param geom pointer to Geometry object.
 \param angle rotation angle [expressed in Degrees].

 \sa gaiaShiftCoords, gaiaScaleCoords, gaiaReflectCoords, gaiaSwapCoords
 */
    GAIAGEO_DECLARE void gaiaRotateCoords (gaiaGeomCollPtr geom, double angle);

/**
 Reflects any coordinate within a Geometry object

 \param geom pointer to Geometry object.
 \param x_axis if set to 0, no X axis reflection will be applied:
 otherwise the X axis will be reflected.
 \param y_axis if set to 0, no Y axis reflection will be applied:
 otherwise the Y axis will be reflected.

 \sa gaiaShiftCoords, gaiaScaleCoords, gaiaRotateCoords, gaiaSwapCoords
 */
    GAIAGEO_DECLARE void gaiaReflectCoords (gaiaGeomCollPtr geom, int x_axis,
					    int y_axis);

/**
 Swaps any coordinate within a Geometry object

 \param geom pointer to Geometry object.

 \sa gaiaShiftCoords, gaiaScaleCoords, gaiaRotateCoords, gaiaReflectCoords

 \note the X and Y axes will be swapped.
 */
    GAIAGEO_DECLARE void gaiaSwapCoords (gaiaGeomCollPtr geom);

/**
 Checks if two Linestring objects are equivalent

 \param line1 pointer to first Linestring object.
 \param line2 pointer to second Linestring object.

 \return 0 if false: any other different value if true

 \sa gaiaPolygonEquals

 \note two Linestrings objects are assumed to be equivalent if exactly
 \remark deprecated function (used in earlier SpatiaLite versions).
 the same Points are found in both them.
 */
    GAIAGEO_DECLARE int gaiaLinestringEquals (gaiaLinestringPtr line1,
					      gaiaLinestringPtr line2);

/**
 Checks if two Polygons objects are equivalent

 \param polyg1 pointer to first Polygon object.
 \param polyg2 pointer to second Polygon object.

 \return 0 if false: any other different value if true

 \sa gaiaLinestringEquals

 \note two Polygon objects are assumed to be equivalent if exactly 
 the same Points are found in both them.

 \remark deprecated function (used in earlier SpatiaLite versions).
 */
    GAIAGEO_DECLARE int gaiaPolygonEquals (gaiaPolygonPtr polyg1,
					   gaiaPolygonPtr polyg2);

/**
 Retrieves Geodesic params for an Ellipsoid definition

 \param name text string identifying an Ellipsoid definition.
 \param a on completion this variable will contain the first geodesic param.
 \param b on completion this variable will contain the second geodesic param.
 \param rf on completion this variable will contain the third geodesic param.

 \return 0 on failure: any other value on success.

 \sa gaiaGreatCircleDistance, gaiaGeodesicDistance,
 gaiaGreatCircleTotalLength, gaiaGeodesicTotalLength

 \note supported Ellipsoid definitions are: \b MERIT, \b SGS85, \b GRS80,
 \b IAU76, \b airy, \b APL4.9, \b NWL9D, \b mod_airy, \b andrae, \b aust_SA,
 \b GRS67, \b bessel, \b bess_nam, \b clrk66, \b clrk80, \b CPM, \b delmbr, 
 \b engelis, \b evrst30, \b evrst48, \b evrst56, \b evrst69, \b evrstSS, 
 \b fschr60
 */
    GAIAGEO_DECLARE int gaiaEllipseParams (const char *name, double *a,
					   double *b, double *rf);

/**
 Calculates the Great Circle Distance intercurring between two Points

 \param a first geodesic parameter.
 \param b second geodesic parameter.
 \param lat1 Latitude of first Point.
 \param lon1 Longitude of first Point.
 \param lat2 Latitude of second Point.
 \param lon2 Longitude of second Point.

 \return the calculated Great Circle Distance.
 
 \sa gaiaEllipseParams, gaiaGeodesicDistance,
 gaiaGreatCircleTotalLength, gaiaGeodesicTotalLength

 \note the returned distance is expressed in Kilometers.
 \n the Great Circle method is less accurate but fastest to be calculated.
 */
    GAIAGEO_DECLARE double gaiaGreatCircleDistance (double a, double b,
						    double lat1, double lon1,
						    double lat2, double lon2);

/**
 Calculates the Geodesic Distance intercurring between two Points

 \param a first geodesic parameter.
 \param b second geodesic parameter.
 \param rf third geodesic parameter.
 \param lat1 Latitude of first Point.
 \param lon1 Longitude of first Point.
 \param lat2 Latitude of second Point.
 \param lon2 Longitude of second Point.

 \return the calculated Geodesic Distance.

 \sa gaiaEllipseParams, gaiaGreatCircleDistance, gaiaGreatCircleTotalLength,
 gaiaGeodesicTotalLength

 \note the returned distance is expressed in Kilometers.
 \n the Geodesic method is much more accurate but slowest to be calculated.
 */
    GAIAGEO_DECLARE double gaiaGeodesicDistance (double a, double b, double rf,
						 double lat1, double lon1,
						 double lat2, double lon2);

/**
 Calculates the Great Circle Total Length for a Linestring / Ring

 \param a first geodesic parameter.
 \param b second geodesic parameter.
 \param dims dimensions: one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M or GAIA_XY_ZM
 \param coords pointed to COORD mem-array
 \param vert number of Points (aka Vertices) within the COORD mem-array

 \return the calculated Great Circle Total Length.

 \sa gaiaEllipseParams, gaiaGreatCircleDistance, gaiaGeodesicDistance,
 gaiaGeodesicTotalLength

 \note the returned length is expressed in Kilometers.
 \n the Great Circle method is less accurate but fastest to be calculated.
 \n \b dims, \b coords and \b vert are usually expected to correspond to
 \b DimensionModel, \b Coords and \b Points members from a gaiaLinestringStruct
 or gaiaRingStruct
 */
    GAIAGEO_DECLARE double gaiaGreatCircleTotalLength (double a, double b,
						       int dims, double *coords,
						       int vert);

/**
 Calculates the Geodesic Total Length for a Linestring / Ring

 \param a first geodesic parameter.
 \param b second geodesic parameter.
 \param rf third geodesic parameter.
 \param dims dimensions: one of GAIA_XY, GAIA_XY_Z, GAIA_XY_M or GAIA_XY_ZM
 \param coords pointed to COORD mem-array
 \param vert number of Points (aka Vertices) within the COORD mem-array

 \return the calculated Geodesic Total Length.

 \sa gaiaEllipseParams, gaiaGreatCircleDistance, gaiaGeodesicDistance,
 gaiaGreatCircleTotalLength

 \note the returned length is expressed in Kilometers.
 \n the Geodesic method is much more accurate but slowest to be calculated.
 \n \b dims, \b coords and \b vert are usually expected to correspond to
 \b DimensionModel, \b Coords and \b Points members from a gaiaLinestringStruct
 or gaiaRingStruct
 */
    GAIAGEO_DECLARE double gaiaGeodesicTotalLength (double a, double b,
						    double rf, int dims,
						    double *coords, int vert);

/**
 Convert a Length from a Measure Unit to another

 \param value the length measure to be converted.
 \param unit_from original Measure Unit.
 \param unit_to converted Measure Unit.
 \param cvt on completion this variable will contain the converted length 
 measure. 

 \note supported Measu Units are: GAIA_KM, GAIA_M, GAIA_DM, GAIA_CM, GAIA_MM,
 GAIA_KMI, GAIA_IN, GAIA_FT, GAIA_YD, GAIA_MI, GAIA_FATH, GAIC_CH, GAIA_LINK,
 GAIA_US_IN, GAIA_US_FT, GAIA_US_YD, GAIA_US_CH, GAIA_US_MI, GAIA_IND_YD,
 GAIA_IND_FT, GAIA_IND_CH
 */
    GAIAGEO_DECLARE int gaiaConvertLength (double value, int unit_from,
					   int unit_to, double *cvt);

#ifdef __cplusplus
}
#endif

#endif				/* _GG_CORE_H */
/**************** End file: gg_core.h **********/


/**************** Begin file: gg_mbr.h **********/


/**
 \file gg_mbr.h

 Geometry handling functions: MBR
 */

#ifndef _GG_MBR_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GG_MBR_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* function prototypes */

/**
 Updates the actual MBR for a Linestring object

 \param line pointer to the Linestring object 
 */
    GAIAGEO_DECLARE void gaiaMbrLinestring (gaiaLinestringPtr line);

/**
 Updates the actual MBR for a Ring object

 \param rng pointer to the Ring object
 */
    GAIAGEO_DECLARE void gaiaMbrRing (gaiaRingPtr rng);

/**
 Updates the actual MBR for a Polygon object

 \param polyg pointer to the Polygon object
 */
    GAIAGEO_DECLARE void gaiaMbrPolygon (gaiaPolygonPtr polyg);

/**
 Updates the actual MBR for a Geometry object

 \param geom pointer to the Geometry object
 */
    GAIAGEO_DECLARE void gaiaMbrGeometry (gaiaGeomCollPtr geom);

/**
 Retrieves the MBR (MinX) from a BLOB-Geometry object

 \param blob pointer to BLOB-Geometry.
 \param size the BLOB's size (in bytes).
 \param minx on completion this variable will contain the MBR MinX coordinate.

 \return 0 on failure: any other value on success.

 \sa gaiaGetMbrMaxX, gaiaGetMbrMinY, gaiaGetMbrMaxY
 */
    GAIAGEO_DECLARE int gaiaGetMbrMinX (const unsigned char *blob,
					unsigned int size, double *minx);

/**
 Retrieves the MBR (MaxX) from a BLOB-Geometry object

 \param blob pointer to BLOB-Geometry.
 \param size the BLOB's size (in bytes).
 \param maxx on completion this variable will contain the MBR MaxX coordinate.

 \return 0 on failure: any other value on success.

 \sa gaiaGetMbrMinX, gaiaGetMbrMinY, gaiaGetMbrMaxY
 */
    GAIAGEO_DECLARE int gaiaGetMbrMaxX (const unsigned char *blob,
					unsigned int size, double *maxx);

/**
 Retrieves the MBR (MinY) from a BLOB-Geometry object

 \param blob pointer to BLOB-Geometry.
 \param size the BLOB's size (in bytes).
 \param miny on completion this variable will contain the MBR MinY coordinate.

 \return 0 on failure: any other value on success.

 \sa gaiaGetMbrMinX, gaiaGetMbrMaxX, gaiaGetMbrMaxY
 */
    GAIAGEO_DECLARE int gaiaGetMbrMinY (const unsigned char *blob,
					unsigned int size, double *miny);

/**
 Retrieves the MBR (MaxY) from a BLOB-Geometry object

 \param blob pointer to BLOB-Geometry.
 \param size the BLOB's size (in bytes).
 \param maxy on completion this variable will contain the MBR MaxY coordinate.

 \return 0 on failure: any other value on success.

 \sa gaiaGetMbrMinX, gaiaGetMbrMaxX, gaiaGetMbrMinY
 */
    GAIAGEO_DECLARE int gaiaGetMbrMaxY (const unsigned char *blob,
					unsigned int size, double *maxy);

/**
 Creates a Geometry object corresponding to the Envelope [MBR] for a
 BLOB-Geometry

 \param blob pointer to BLOB-Geometry
 \param size the BLOB's size (in bytes)

 \return the pointer to the newly created Geometry object: NULL on failure

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any 
 contained child object. 
 */

    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromSpatiaLiteBlobMbr (const unsigned
							       char *blob,
							       unsigned int
							       size);

/**
 MBRs comparison: Contains

 \param mbr1 pointer to first Geometry object.
 \param mbr2 pointer to second Geometry object.

 \return 0 if false; any other value if mbr1 spatially \e contains mbr2

 \sa gaiaMbrsDisjoint, gaiaMbrsEqual, gaiaMbrsIntersects,
 gaiaMbrsOverlaps, gaiaMbrsTouches, gaiaMbrsWithin
 */
    GAIAGEO_DECLARE int gaiaMbrsContains (gaiaGeomCollPtr mbr1,
					  gaiaGeomCollPtr mbr2);

/**
 MBRs comparison: Disjoint

 \param mbr1 pointer to first Geometry object.
 \param mbr2 pointer to second Geometry object.

 \return 0 if false; any other value if mbr1 and mbr2 are spatially \e disjoint

 \sa gaiaMbrsContains, gaiaMbrsEqual, gaiaMbrsIntersects,
 gaiaMbrsOverlaps, gaiaMbrsTouches, gaiaMbrsWithin
 */
    GAIAGEO_DECLARE int gaiaMbrsDisjoint (gaiaGeomCollPtr mbr1,
					  gaiaGeomCollPtr mbr2);

/**
 MBRs comparison: Equal

 \param mbr1 pointer to first Geometry object.
 \param mbr2 pointer to second Geometry object.

 \return 0 if false; any other value if mbr1 and mbr2 are spatially \e equal

 \sa gaiaMbrsContains, gaiaMbrsDisjoint, gaiaMbrsIntersects,
 gaiaMbrsOverlaps, gaiaMbrsTouches, gaiaMbrsWithin
 */
    GAIAGEO_DECLARE int gaiaMbrsEqual (gaiaGeomCollPtr mbr1,
				       gaiaGeomCollPtr mbr2);

/**
 MBRs comparison: Intersects

 \param mbr1 pointer to first Geometry object.
 \param mbr2 pointer to second Geometry object.

 \return 0 if false; any other value if mbr1 and mbr2 spatially \e intersect

 \sa gaiaMbrsContains, gaiaMbrsDisjoint, gaiaMbrsEqual, 
 gaiaMbrsOverlaps, gaiaMbrsTouches, gaiaMbrsWithin
 */
    GAIAGEO_DECLARE int gaiaMbrsIntersects (gaiaGeomCollPtr mbr1,
					    gaiaGeomCollPtr mbr2);

/**
 MBRs comparison: Overlaps

 \param mbr1 pointer to first Geometry object.
 \param mbr2 pointer to second Geometry object.

 \return 0 if false; any other value if mbr1 and mbr2 spatially \e overlap

 \sa gaiaMbrsContains, gaiaMbrsDisjoint, gaiaMbrsEqual, gaiaMbrsIntersects,
 gaiaMbrsTouches, gaiaMbrsWithin
 */
    GAIAGEO_DECLARE int gaiaMbrsOverlaps (gaiaGeomCollPtr mbr1,
					  gaiaGeomCollPtr mbr2);

/**
 MBRs comparison: Touches

 \param mbr1 pointer to first Geometry object.
 \param mbr2 pointer to second Geometry object.

 \return 0 if false; any other value if mbr1 and mbr2 spatially \e touche

 \sa gaiaMbrsContains, gaiaMbrsDisjoint, gaiaMbrsEqual, gaiaMbrsIntersects,
 gaiaMbrsOverlaps, gaiaMbrsWithin
 */
    GAIAGEO_DECLARE int gaiaMbrsTouches (gaiaGeomCollPtr mbr1,
					 gaiaGeomCollPtr mbr2);

/**
 MBRs comparison: Within

 \param mbr1 pointer to first Geometry object.
 \param mbr2 pointer to second Geometry object.

 \return 0 if false; any other value if mbr1 is spatially \e within mbr2

 \sa gaiaMbrsContains, gaiaMbrsDisjoint, gaiaMbrsEqual, gaiaMbrsIntersects,
 gaiaMbrsOverlaps, gaiaMbrsTouches
 */
    GAIAGEO_DECLARE int gaiaMbrsWithin (gaiaGeomCollPtr mbr1,
					gaiaGeomCollPtr mbr2);

/**
 Creates a BLOB-Geometry representing an Envelope [MBR]

 \param x1 first X coordinate.
 \param y1 first Y coordinate.
 \param x2 second X coordinate.
 \param y2 second Y coordinate.
 \param srid the SRID associated to the Envelope
 \param result on completion will contain a pointer to newly created
 BLOB-Geometry
 \param size on completion this variabile will contain the BLOB's size (in 
 bytes)

 \sa gaiaBuildCircleMbr

 \note [XY] coords must define two extreme Points identifying a diagonal
 of the MBR [Envelope]
 \n no special order is required for coords: MAX / MIN values will be 
 internally arranged as appropriate.
 */
    GAIAGEO_DECLARE void gaiaBuildMbr (double x1, double y1, double x2,
				       double y2, int srid,
				       unsigned char **result, int *size);

/**
 Creates a BLOB-Geometry representing an Envelope [MBR]

 \param x centre X coordinate.
 \param y centre Y coordinate.
 \param radius the radius of the circle
 \param srid the SRID associated to the Envelope
 \param result on completion will contain a pointer to newly created
 BLOB-Geometry
 \param size on completion this variabile will contain the BLOB's size (in
 bytes)

 \sa gaiaBuildMbr

 \note the \e circle of givern \e radius and \e centre will be used so to
 determine the corresponding \e square Envelope
 */
    GAIAGEO_DECLARE void gaiaBuildCircleMbr (double x, double y, double radius,
					     int srid, unsigned char **result,
					     int *size);

/**
 Creates a BLOB-FilterMBR

 \param x1 first X coordinate.
 \param y1 first Y coordinate.
 \param x2 second X coordinate.
 \param y2 second Y coordinate.
 \param mode one of: GAIA_FILTER_MBR_WITHIN, GAIA_FILTER_MBR_CONTAINS,
 GAIA_FILTER_MBR_INTERSECTS, GAIA_FILTER_MBR_DECLARE
 \param result on completion will contain a pointer to newly created
 BLOB-FilterMBR
 \param size on completion this variabile will contain the BLOB's size (in
 bytes)

 \sa gaiaParseFilterMbr

 \note [XY] coords must define two extreme Points identifying a diagonal
 of the MBR [Envelope]
 \n no special order is required for coords: MAX / MIN values will be
 internally arranged as appropriate.

 \remark internally used to implement Geometry Callback R*Tree filtering.
 */
    GAIAGEO_DECLARE void gaiaBuildFilterMbr (double x1, double y1, double x2,
					     double y2, int mode,
					     unsigned char **result, int *size);

/**
 Creates a BLOB-FilterMBR

 \param result pointer to BLOB-FilterMBR [previously created by 
 gaiaBuildFilterMbr]
 BLOB-Geometry
 \param size BLOB's size (in bytes)
 \param minx on completion this variable will contain the MBR MinX coord.
 \param miny on completion this variable will contain the MBR MinY coord.
 \param maxx on completion this variable will contain the MBR MinY coord.
 \param maxy on completion this variable will contain the MBR MaxY coord.
 \param mode on completion this variable will contain the FilterMBR mode.

 \sa gaiaBuildFilterMbr

 \remark internally used to implement Geometry Callback R*Tree filtering.
 */
    GAIAGEO_DECLARE int gaiaParseFilterMbr (unsigned char *result, int size,
					    double *minx, double *miny,
					    double *maxx, double *maxy,
					    int *mode);

/**
 Computes the Z-Range for a Linestring object

 \param line pointer to the Linestring object
 \param min on completion this variable will contain the min Z value found 
 \param max on completion this variable will contain the max Z value found 

 \note if the Linestring has XY or XYM dims, the Z-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaZRangeLinestring (gaiaLinestringPtr line,
					       double *min, double *max);

/**
 Computes the Z-Range for a Ring object

 \param rng pointer to the Ring object
 \param min on completion this variable will contain the min Z value found
 \param max on completion this variable will contain the max Z value found

 \note if the Ring has XY or XYM dims, the Z-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaZRangeRing (gaiaRingPtr rng, double *min,
					 double *max);

/**
 Computes the Z-Range for a Polygon object

 \param polyg pointer to the Polygon object
 \param min on completion this variable will contain the min Z value found
 \param max on completion this variable will contain the max Z value found

 \note if the Polygon has XY or XYM dims, the Z-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaZRangePolygon (gaiaPolygonPtr polyg, double *min,
					    double *max);

/**
 Computes the Z-Range for a Geometry object

 \param geom pointer to the Geometry object
 \param min on completion this variable will contain the min Z value found
 \param max on completion this variable will contain the max Z value found

 \note if the Geometry has XY or XYM dims, the Z-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaZRangeGeometry (gaiaGeomCollPtr geom, double *min,
					     double *max);

/**
 Computes the M-Range for a Linestring object

 \param line pointer to the Linestring object
 \param min on completion this variable will contain the min M value found
 \param max on completion this variable will contain the max M value found

 \note if the Linestring has XY or XYZ dims, the M-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaMRangeLinestring (gaiaLinestringPtr line,
					       double *min, double *max);

/**
 Computes the M-Range for a Ring object

 \param rng pointer to the Ring object
 \param min on completion this variable will contain the min M value found
 \param max on completion this variable will contain the max M value found

 \note if the Ring has XY or XYZ dims, the M-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaMRangeRing (gaiaRingPtr rng, double *min,
					 double *max);

/**
 Computes the M-Range for a Polygon object

 \param polyg pointer to the Polygon object
 \param min on completion this variable will contain the min M value found
 \param max on completion this variable will contain the max M value found

 \note if the Polygon has XY or XYZ dims, the M-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaMRangePolygon (gaiaPolygonPtr polyg, double *min,
					    double *max);

/**
 Computes the Z-Range for a Geometry object

 \param geom pointer to the Geometry object
 \param min on completion this variable will contain the min M value found
 \param max on completion this variable will contain the max M value found

 \note if the Geometry has XY or XYZ dims, the M-Range is meaningless
 */
    GAIAGEO_DECLARE void gaiaMRangeGeometry (gaiaGeomCollPtr geom, double *min,
					     double *max);


#ifdef __cplusplus
}
#endif

#endif				/* _GG_MBR_H */
/**************** End file: gg_mbr.h **********/


/**************** Begin file: gg_formats.h **********/


/**
 \file gg_formats.h

 Geometry handling functions: formats
 */

#ifndef _GG_FORMATS_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GG_FORMATS_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* function prototypes */

/**
 Test CPU endianness

 \return 0 if big-endian: any other value if little-endian
 */
    GAIAGEO_DECLARE int gaiaEndianArch (void);

/**
 Import an INT-16 value in endian-aware fashion
 
 \param p endian-dependent representation (input buffer).
 \param little_endian 0 if the input buffer is big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \return the internal SHORT value 

 \sa gaiaEndianArch, gaiaExport16

 \note you are expected to pass an input buffer corresponding to an
 allocation size of (at least) 2 bytes.
 */
    GAIAGEO_DECLARE short gaiaImport16 (const unsigned char *p,
					int little_endian,
					int little_endian_arch);

/**
 Import an INT-32 value in endian-aware fashion
 
 \param p endian-dependent representation (input buffer).
 \param little_endian 0 if the input buffer is big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \return the internal INT value 

 \sa gaiaEndianArch, gaiaExport32

 \note you are expected to pass an input buffer corresponding to an
 allocation size of (at least) 4 bytes.
 */
    GAIAGEO_DECLARE int gaiaImport32 (const unsigned char *p, int little_endian,
				      int little_endian_arch);

/**
 Import a FLOAT-32 value in endian-aware fashion
 
 \param p endian-dependent representation (input buffer).
 \param little_endian 0 if the input buffer is big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \return the internal FLOAT value 

 \sa gaiaEndianArch, gaiaExportF32

 \note you are expected to pass an input buffer corresponding to an
 allocation size of (at least) 4 bytes.
 */
    GAIAGEO_DECLARE float gaiaImportF32 (const unsigned char *p,
					 int little_endian,
					 int little_endian_arch);

/**
 Import an DOUBLE-64 in endian-aware fashion
 
 \param p endian-dependent representation (input buffer).
 \param little_endian 0 if the input buffer is big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \return the internal DOUBLE value 

 \sa gaiaEndianArch, gaiaExport64

 \note you are expected to pass an input buffer corresponding to an
 allocation size of (at least) 8 bytes.
 */
    GAIAGEO_DECLARE double gaiaImport64 (const unsigned char *p,
					 int little_endian,
					 int little_endian_arch);

/**
 Import an INT-64 in endian-aware fashion
 
 \param p endian-dependent representation (input buffer).
 \param little_endian 0 if the input buffer is big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \return the internal INT-64 value 

 \sa gaiaEndianArch, gaiaExportI64

 \note you are expected to pass an input buffer corresponding to an
 allocation size of (at least) 8 bytes.
 */
    GAIAGEO_DECLARE sqlite3_int64 gaiaImportI64 (const unsigned char *p,
						 int little_endian,
						 int little_endian_arch);

/**
 Export an INT-16 value in endian-aware fashion
 
 \param p endian-dependent representation (output buffer).
 \param value the internal value to be exported.
 \param little_endian 0 if the output buffer has to be big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \sa gaiaEndianArch, gaiaImport16

 \note you are expected to pass an output buffer corresponding to an
 allocation size of (at least) 2 bytes.
 */
    GAIAGEO_DECLARE void gaiaExport16 (unsigned char *p, short value,
				       int little_endian,
				       int little_endian_arch);

/**
 Export an INT-32 value in endian-aware fashion
 
 \param p endian-dependent representation (output buffer).
 \param value the internal value to be exported.
 \param little_endian 0 if the output buffer has to be big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \sa gaiaEndianArch, gaiaImport32

 \note you are expected to pass an output buffer corresponding to an
 allocation size of (at least) 4 bytes.
 */
    GAIAGEO_DECLARE void gaiaExport32 (unsigned char *p, int value,
				       int little_endian,
				       int little_endian_arch);

/**
 Export a FLOAT-32 value in endian-aware fashion
 
 \param p endian-dependent representation (output buffer).
 \param value the internal value to be exported.
 \param little_endian 0 if the output buffer has to be big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \sa gaiaEndianArch, gaiaImportF32

 \note you are expected to pass an output buffer corresponding to an
 allocation size of (at least) 4 bytes.
 */
    GAIAGEO_DECLARE void gaiaExportF32 (unsigned char *p, float value,
					int little_endian,
					int little_endian_arch);

/**
 Export a DOUBLE value in endian-aware fashion
 
 \param p endian-dependent representation (output buffer).
 \param value the internal value to be exported.
 \param little_endian 0 if the output buffer has to be big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \sa gaiaEndianArch, gaiaImport64

 \note you are expected to pass an output buffer corresponding to an
 allocation size of (at least) 8 bytes.
 */
    GAIAGEO_DECLARE void gaiaExport64 (unsigned char *p, double value,
				       int little_endian,
				       int little_endian_arch);

/**
 Export an INT-64 value in endian-aware fashion
 
 \param p endian-dependent representation (output buffer).
 \param value the internal value to be exported.
 \param little_endian 0 if the output buffer has to be big-endian: any other value
 for little-endian.
 \param little_endian_arch the value returned by gaiaEndianArch()

 \sa gaiaEndianArch, gaiaImportI64

 \note you are expected to pass an output buffer corresponding to an
 allocation size of (at least) 8 bytes.
 */
    GAIAGEO_DECLARE void gaiaExportI64 (unsigned char *p, sqlite3_int64 value,
					int little_endian,
					int little_endian_arch);

/**
 Initializes a dynamically growing Text output buffer

 \param buf pointer to gaiaOutBufferStruct structure
 
 \sa gaiaOutBufferReset, gaiaAppendToOutBuffer

 \note Text notations representing Geometry objects may easily require
 a huge storage amount: the gaiaOutBufferStruct automatically supports
 a dynamically growing output buffer.
 \n You are required to initialize this structure before attempting
 any further operation;
 and you are responsible to cleanup any related memory allocation
 when it's any longer required.
 */
    GAIAGEO_DECLARE void gaiaOutBufferInitialize (gaiaOutBufferPtr buf);

/**
 Resets a dynamically growing Text output buffer to its initial (empty) state

 \param buf pointer to gaiaOutBufferStruct structure
 
 \sa gaiaOutBufferInitialize, gaiaAppendToOutBuffer

 \note You are required to initialize this structure before attempting
 any further operation:
 this function will release any related memory allocation.
 */
    GAIAGEO_DECLARE void gaiaOutBufferReset (gaiaOutBufferPtr buf);

/**
 Appends a text string at the end of Text output buffer

 \param buf pointer to gaiaOutBufferStruct structure.
 \param text the text string to be appended.

 \sa gaiaOutBufferInitialize, gaiaOutBufferReset

 \note You are required to initialize this structure before attempting
 any further operation:
 the dynamically growing Text buffer will be automatically allocated
 and/or extended as required.
 */
    GAIAGEO_DECLARE void gaiaAppendToOutBuffer (gaiaOutBufferPtr buf,
						const char *text);

/**
 Creates a BLOB-Geometry representing a Point

 \param x Point X coordinate.
 \param y Point Y coordinate.
 \param srid the SRID to be set for the Point.
 \param result on completion will containt a pointer to BLOB-Geometry:
 NULL on failure.
 \param size on completion this variable will contain the BLOB's size (in bytes)

 \sa gaiaFromSpatiaLiteBlobWkb

 \note the BLOB buffer corresponds to dynamically allocated memory:
 so you are responsible to free() it [unless SQLite will take care
 of memory cleanup via buffer binding].
 */
    GAIAGEO_DECLARE void gaiaMakePoint (double x, double y, int srid,
					unsigned char **result, int *size);

/**
 Creates a BLOB-Geometry representing a Segment (2-Points Linestring)

 \param geom1 pointer to first Geometry object (expected to represent a Point).
 \param geom2 pointer to second Geometry object (expected to represent a Point).
 \param result on completion will containt a pointer to BLOB-Geometry:
 NULL on failure.
 \param size on completion this variable will contain the BLOB's size (in bytes)

 \sa gaiaFromSpatiaLiteBlobWkb

 \note the BLOB buffer corresponds to dynamically allocated memory:
 so you are responsible to free() it [unless SQLite will take care
 of memory cleanup via buffer binding].
 */
    GAIAGEO_DECLARE void gaiaMakeLine (gaiaGeomCollPtr geom1,
				       gaiaGeomCollPtr geom2,
				       unsigned char **result, int *size);

/**
 Creates a Geometry object from the corresponding BLOB-Geometry 

 \param blob pointer to BLOB-Geometry
 \param size the BLOB's size

 \return the pointer to the newly created Geometry object: NULL on failure

 \sa gaiaFreeGeomColl, gaiaToSpatiaLiteBlobWkb, gaiaToCompressedBlobWkb

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any 
 contained child object. 
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromSpatiaLiteBlobWkb (const unsigned
							       char *blob,
							       unsigned int
							       size);

/**
 Creates a BLOB-Geometry corresponding to a Geometry object

 \param geom pointer to the Geometry object.
 \param result on completion will containt a pointer to BLOB-Geometry:
 NULL on failure.
 \param size on completion this variable will contain the BLOB's size (in bytes)

 \sa gaiaFromSpatiaLiteBlobWkb, gaiaToCompressedBlobWkb

 \note the BLOB buffer corresponds to dynamically allocated memory:
 so you are responsible to free() it [unless SQLite will take care
 of memory cleanup via buffer binding].
 */
    GAIAGEO_DECLARE void gaiaToSpatiaLiteBlobWkb (gaiaGeomCollPtr geom,
						  unsigned char **result,
						  int *size);

/**
 Creates a Compressed BLOB-Geometry corresponding to a Geometry object

 \param geom pointer to the Geometry object.
 \param result on completion will containt a pointer to Compressed BLOB-Geometry:
 NULL on failure.
 \param size on completion this variable will contain the BLOB's size (in bytes)

 \sa gaiaFromSpatiaLiteBlobWkb, gaiaToSpatiaLiteBlobWkb

 \note this function will apply compression to any Linestring / Ring found
 within the Geometry to be encoded.
 \n the returned BLOB buffer corresponds to dynamically allocated memory:
 so you are responsible to free() it [unless SQLite will take care
 of memory cleanup via buffer binding].
 */
    GAIAGEO_DECLARE void gaiaToCompressedBlobWkb (gaiaGeomCollPtr geom,
						  unsigned char **result,
						  int *size);

/**
 Creates a Geometry object from WKB notation

 \param blob pointer to WKB buffer
 \param size the BLOB's size (in bytes)

 \return the pointer to the newly created Geometry object: NULL on failure.

 \sa gaiaToWkb, gaiaToHexWkb, gaiaFromEWKB, gaiaToEWKB

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object. 
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromWkb (const unsigned char *blob,
						 unsigned int size);

/**
 Encodes a Geometry object into WKB notation

 \param geom pointer to Geometry object
 \param result on completion will containt a pointer to the WKB buffer [BLOB]:
 NULL on failure.
 \param size on completion this variable will contain the BLOB's size (in bytes)

 \sa gaiaFromWkb, gaiaToHexWkb, gaiaFromEWKB, gaiaToEWKB

 \note this function will apply 3D WKB encoding as internally intended by
 SpatiaLite: not necessarily intended by other OGC-like implementations.
 \n Anyway, 2D WKB is surely standard and safely interoperable.
 \n the returned BLOB buffer corresponds to dynamically allocated memory:
 so you are responsible to free() it [unless SQLite will take care
 of memory cleanup via buffer binding].
 */
    GAIAGEO_DECLARE void gaiaToWkb (gaiaGeomCollPtr geom,
				    unsigned char **result, int *size);

/**
 Encodes a Geometry object into (hex) WKB notation

 \param geom pointer to Geometry object

 \return the pointer to a text buffer containing WKB translated into plain
 hexadecimal: NULL on failure.

 \sa gaiaFromWkb, gaiaToWkb, gaiaFromEWKB, gaiaToEWKB

 \note the returned buffer corresponds to dynamically allocated memory:
 so you are responsible to free() it [unless SQLite will take care
 of memory cleanup via buffer binding].
 */
    GAIAGEO_DECLARE char *gaiaToHexWkb (gaiaGeomCollPtr geom);

/**
 Encodes a Geometry object into EWKB notation

 \param out_buf pointer to dynamically growing Text buffer
 \param geom pointer to Geometry object

 \sa gaiaFromWkb, gaiaToWkb, gaiaToHexWkb, gaiaFromEWKB, gaiaToEWKB

 \note this function will produce strictly conformat EWKB; you can
 safely use this for PostGIS data exchange.
 */
    GAIAGEO_DECLARE void gaiaToEWKB (gaiaOutBufferPtr out_buf,
				     gaiaGeomCollPtr geom);

/**
 Creates a Geometry object from EWKB notation

 \param in_buffer pointer to EWKB buffer

 \return the pointer to the newly created Geometry object: NULL on failure.

 \sa gaiaToWkb, gaiaToHexWkb, gaiaFromEWKB, gaiaToEWKB

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromEWKB (const unsigned char
						  *in_buffer);

/**
 Creates a Geometry object from FGF notation

 \param blob pointer to FGF buffer
 \param size the BLOB's size (in bytes)

 \return the pointer to the newly created Geometry object: NULL on failure.

 \sa gaiaToFgf

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromFgf (const unsigned char *blob,
						 unsigned int size);

/**
 Encodes a Geometry object into FGF notation

 \param geom pointer to Geometry object
 \param result on completion will containt a pointer to the FGF buffer [BLOB]:
 NULL on failure.
 \param size on completion this variable will contain the BLOB's size (in bytes)
 \param coord_dims one of: GAIA_XY, GAIA_XY_Z, GAIA_XY_M, GAIA_XY_ZM

 \sa gaiaFromFgf

 \note the returned BLOB buffer corresponds to dynamically allocated memory:
 so you are responsible to free() it [unless SQLite will take care
 of memory cleanup via buffer binding].
 */
    GAIAGEO_DECLARE void gaiaToFgf (gaiaGeomCollPtr geom,
				    unsigned char **result, int *size,
				    int coord_dims);

/**
 Creates a Geometry object from WKT notation

 \param in_buffer pointer to WKT buffer
 \param type the expected Geometry Class Type
 \n if actual type defined in WKT doesn't corresponds to this, an error will
 be raised.

 \return the pointer to the newly created Geometry object: NULL on failure

 \sa gaiaOutWkt, gaiaOutWktStrict, gaiaParseEWKT, gaiaToEWKT

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaParseWkt (const unsigned char
						  *in_buffer, short type);

/**
 Encodes a Geometry object into WKT notation

 \param out_buf pointer to dynamically growing Text buffer
 \param geom pointer to Geometry object

 \sa gaiaParseWkt, gaiaOutWktStrict, gaiaParseEWKT, gaiaToEWKT

 \note this function will apply 3D WKT encoding as internally intended by
 SpatiaLite: not necessarily intended by other OGC-like implementations.
 \n Anyway, 2D WKT is surely standard and safely interoperable.
 */
    GAIAGEO_DECLARE void gaiaOutWkt (gaiaOutBufferPtr out_buf,
				     gaiaGeomCollPtr geom);

/**
 Encodes a Geometry object into strict 2D WKT notation

 \param out_buf pointer to dynamically growing Text buffer
 \param geom pointer to Geometry object
 \param precision decimal digits to be used for coordinates

 \sa gaiaParseWkt, gaiaOutWkt, gaiaParseEWKT, gaiaToEWKT

 \note this function will apply strict 2D WKT encoding, so to be surely
 standard and safely interoperable.
 \n Dimensions will be automatically casted to 2D [XY] when required.
 */
    GAIAGEO_DECLARE void gaiaOutWktStrict (gaiaOutBufferPtr out_buf,
					   gaiaGeomCollPtr geom, int precision);

/**
 Creates a Geometry object from EWKT notation

 \param in_buffer pointer to EWKT buffer

 \return the pointer to the newly created Geometry object: NULL on failure

 \sa gaiaParseWkt, gaiaOutWkt, gaiaOutWktStrict, gaiaToEWKT

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaParseEWKT (const unsigned char
						   *in_buffer);

/**
 Encodes a Geometry object into EWKT notation

 \param out_buf pointer to dynamically growing Text buffer
 \param geom pointer to Geometry object

 \sa gaiaParseWkt, gaiaOutWkt, gaiaOutWktStrict, gaiaParseEWKT

 \note this function will apply PostGIS own EWKT encoding.
 */
    GAIAGEO_DECLARE void gaiaToEWKT (gaiaOutBufferPtr out_buf,
				     gaiaGeomCollPtr geom);

/**
 Encodes a WKT 3D Point [XYZ]

 \param out_buf pointer to dynamically growing Text buffer
 \param point pointer to Point object

 \sa gaiaOutLinestringZ, gaiaOutPolygonZ
 
 \remark mainly intended for internal usage.
 */
    GAIAGEO_DECLARE void gaiaOutPointZ (gaiaOutBufferPtr out_buf,
					gaiaPointPtr point);

/**
 Encodes a WKT 3D Linestring [XYZ]

 \param out_buf pointer to dynamically growing Text buffer
 \param linestring pointer to Linestring object

 \sa gaiaOutPointZ, gaiaOutPolygonZ
 
 \remark mainly intended for internal usage.
 */
    GAIAGEO_DECLARE void gaiaOutLinestringZ (gaiaOutBufferPtr out_buf,
					     gaiaLinestringPtr linestring);

/**
 Encodes a WKT 3D Polygon [XYZ]

 \param out_buf pointer to dynamically growing Text buffer
 \param polygon pointer to Point object

 \sa gaiaOutPointZ, gaiaOutLinestringZ
 
 \remark mainly intended for internal usage.
 */
    GAIAGEO_DECLARE void gaiaOutPolygonZ (gaiaOutBufferPtr out_buf,
					  gaiaPolygonPtr polygon);
/**
 Creates a Geometry object from KML notation

 \param in_buffer pointer to KML buffer

 \return the pointer to the newly created Geometry object: NULL on failure

 \sa gaiaOutBareKml, gaiaOutFullKml

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaParseKml (const unsigned char
						  *in_buffer);

/**
 Encodes a Geometry object into KML notation

 \param out_buf pointer to dynamically growing Text buffer
 \param geom pointer to Geometry object 
 \param precision decimal digits to be used for coordinates

 \sa gaiaParseKml, gaiaOutFullKml

 \note this function will export the simplest KML notation (no descriptions).
 */
    GAIAGEO_DECLARE void gaiaOutBareKml (gaiaOutBufferPtr out_buf,
					 gaiaGeomCollPtr geom, int precision);

/**
 Encodes a Geometry object into KML notation

 \param out_buf pointer to dynamically growing Text buffer
 \param name text string to be set as KML \e name 
 \param desc text string to se set as KML \e description 
 \param geom pointer to Geometry object
 \param precision decimal digits to be used for coordinates

 \sa gaiaParseKml, gaiaOutBareKml

 \note this function will export the simplest KML notation (no descriptions).
 */
    GAIAGEO_DECLARE void gaiaOutFullKml (gaiaOutBufferPtr out_buf,
					 const char *name, const char *desc,
					 gaiaGeomCollPtr geom, int precision);

/**
 Creates a Geometry object from GML notation

 \param in_buffer pointer to GML buffer
 \param sqlite_handle handle to current DB connection

 \return the pointer to the newly created Geometry object: NULL on failure

 \sa gaiaOutGml

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaParseGml (const unsigned char
						  *in_buffer,
						  sqlite3 * sqlite_handle);

/**
 Encodes a Geometry object into GML notation

 \param out_buf pointer to dynamically growing Text buffer
 \param version GML version
 \param precision decimal digits to be used for coordinates
 \param geom pointer to Geometry object

 \sa gaiaParseGml

 \note if \e version is set to \b 3, then GMLv3 will be used;
 in any other case GMLv2 will be assumed by default.
 */
    GAIAGEO_DECLARE void gaiaOutGml (gaiaOutBufferPtr out_buf, int version,
				     int precision, gaiaGeomCollPtr geom);

/**
 Creates a Geometry object from GeoJSON notation

 \param in_buffer pointer to GeoJSON buffer

 \return the pointer to the newly created Geometry object: NULL on failure

 \sa gaiaOutGeoJSON

 \note you are responsible to destroy (before or after) any allocated Geometry,
 unless you've passed ownership of the Geometry object to some further object:
 in this case destroying the higher order object will implicitly destroy any
 contained child object.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaParseGeoJSON (const unsigned char
						      *in_buffer);

/**
 Encodes a Geometry object into GeoJSON notation

 \param out_buf pointer to dynamically growing Text buffer
 \param geom pointer to Geometry object
 \param precision decimal digits to be used for coordinates
 \param options GeoJSON specific options

 \sa gaiaParseGeoJSON

 \note \e options can assume the following values:
 \li 1 = BBOX, no CRS
 \li 2 = no BBOX, short form CRS
 \li 3 = BBOX, short form CRS
 \li 4 = no BBOX, long form CRS
 \li 5 = BBOX, long form CRS
 \li any other value: no BBOX and no CRS
 */
    GAIAGEO_DECLARE void gaiaOutGeoJSON (gaiaOutBufferPtr out_buf,
					 gaiaGeomCollPtr geom, int precision,
					 int options);
/**
 Encodes a Geometry object into SVG notation

 \param out_buf pointer to dynamically growing Text buffer
 \param geom pointer to Geometry object
 \param relative flag: relative or absolute coordinates
 \param precision decimal digits to be used for coordinates

 \note if \e relative is set to \b 1, then SVG relative coords will be used:
 in any other case SVG absolute coords will be assumed by default.
 */
    GAIAGEO_DECLARE void gaiaOutSvg (gaiaOutBufferPtr out_buf,
				     gaiaGeomCollPtr geom, int relative,
				     int precision);

/**
 Allocates a new DBF Field Value object [duplicating an existing one]
 
 \param org pointer to input DBF Field Value object.

 \return the pointer to newly created DBF Field object.

 \sa gaiaAllocDbfField, gaiaFreeDbfField, gaiaCloneDbfField, gaiaCloneValue,
 gaiaSetNullValue, gaiaSetIntValue, gaiaSetDoubleValue,
 gaiaSetStrValue

 \note the newly created object is an exact copy of the original one.
 */
    GAIAGEO_DECLARE gaiaValuePtr gaiaCloneValue (gaiaValuePtr org);

/**
 Resets a DBF Field Value object to its initial empty state

 \param p pointer to DBF Field Value object

 \sa gaiaAllocDbfField, gaiaCloneDbfField, gaiaCloneValue,
 gaiaSetNullValue, gaiaSetIntValue, gaiaSetDoubleValue,
 gaiaSetStrValue, gaiaResetDbfEntity
 */
    GAIAGEO_DECLARE void gaiaFreeValue (gaiaValuePtr p);

/**
 Allocates a new DBF Field object

 \param name text string: DBF Field name.
 \param type identifier of the corresponding DBF data type.
 \param offset corresponding offset into the DBF I/O buffer.
 \param length max field length (in bytes).
 \param decimals precision: number of decimal digits.

 \return the pointer to newly created DBF Field object.

 \sa gaiaFreeDbfField, gaiaCloneDbfField, gaiaFreeValue,
 gaiaSetNullValue, gaiaSetIntValue, gaiaSetDoubleValue,
 gaiaSetStrValue

 \note you are responsible to destroy (before or after) any allocated DBF Field,
 unless you've passed ownership to some further object: in this case destroying  the higher order object will implicitly destroy any contained child object.  
 \n supported DBF data types are:
 \li 'C' text string [default]
 \li 'N' numeric
 \li 'D' date
 \li 'L' boolean
 */
    GAIAGEO_DECLARE gaiaDbfFieldPtr gaiaAllocDbfField (char *name,
						       unsigned char type,
						       int offset,
						       unsigned char length,
						       unsigned char decimals);

/**
 Destroys a DBF Field object

 \param p pointer to DBF Field object

 \sa gaiaAllocDbfField, gaiaCloneDbfField, gaiaCloneValue,
 gaiaFreeValue, gaiaSetNullValue, gaiaSetIntValue, gaiaSetDoubleValue,
 gaiaSetStrValue
 */
    GAIAGEO_DECLARE void gaiaFreeDbfField (gaiaDbfFieldPtr p);

/**
 Allocates a new DBF Field object [duplicating an existing one]

 \param org pointer to input DBF Field object.

 \return the pointer to newly created DBF Field object.

 \sa gaiaAllocDbfField, gaiaFreeDbfField, gaiaCloneDbfField, 
 gaiaFreeValue, gaiaSetNullValue, gaiaSetIntValue, gaiaSetDoubleValue,
 gaiaSetStrValue

 \note the newly created object is an exact copy of the original one
 [this including an evantual Field Value].
 */
    GAIAGEO_DECLARE gaiaDbfFieldPtr gaiaCloneDbfField (gaiaDbfFieldPtr org);

/**
 Sets a NULL current value for a DBF Field object

 \param field pointer to DBF Field object
 
 \sa gaiaAllocDbfField, gaiaFreeDbfField, gaiaCloneDbfField,
 gaiaFreeValue, gaiaSetIntValue, gaiaSetDoubleValue,
 gaiaSetStrValue
 */
    GAIAGEO_DECLARE void gaiaSetNullValue (gaiaDbfFieldPtr field);

/**
 Sets an INTEGER current value for a DBF Field object

 \param field pointer to DBF Field object.
 \param value integer value to be set.

 \sa gaiaAllocDbfField, gaiaFreeDbfField, gaiaCloneDbfField,
 gaiaFreeValue, gaiaSetNullValue, gaiaSetDoubleValue,
 gaiaSetStrValue
 */
    GAIAGEO_DECLARE void gaiaSetIntValue (gaiaDbfFieldPtr field,
					  sqlite3_int64 value);

/**
 Sets a DOUBLE current value for a DBF Field object
 
 \param field pointer to DBF Field object.
 \param value double value to be set.
                                          
 \sa gaiaAllocDbfField, gaiaFreeDbfField, gaiaCloneDbfField, 
 gaiaFreeValue, gaiaSetNullValue, gaiaSetIntValue, gaiaSetStrValue
 */
    GAIAGEO_DECLARE void gaiaSetDoubleValue (gaiaDbfFieldPtr field,
					     double value);

/**
 Sets a TEXT current value for a DBF Field object

 \param field pointer to DBF Field object.
 \param str text string value to be set.

 \sa gaiaAllocDbfField, gaiaFreeDbfField, gaiaCloneDbfField,
 gaiaFreeValue, gaiaSetNullValue, gaiaSetIntValue, gaiaSetDoubleValue
 */
    GAIAGEO_DECLARE void gaiaSetStrValue (gaiaDbfFieldPtr field, char *str);

/**
 Creates an initially empty DBF List object

 \return the pointer to newly allocated DBF List object: NULL on failure.

 \sa gaiaFreeDbfList, gaiaIsValidDbfList, 
 gaiaResetDbfEntity, gaiaCloneDbfEntity, gaiaAddDbfField
 
 \note you are responsible to destroy (before or after) any allocated DBF List,
 unless you've passed ownership to some further object: in this case destroying
 the higher order object will implicitly destroy any contained child object. 
 */
    GAIAGEO_DECLARE gaiaDbfListPtr gaiaAllocDbfList (void);

/**
 Destroys a DBF List object
 
 \param list pointer to the DBF List object

 \sa gaiaAllocDbfList, gaiaIsValidDbfList,
 gaiaResetDbfEntity, gaiaCloneDbfEntity, gaiaAddDbfField

 \note attempting to destroy any DBF List object whose ownnership has already 
 been transferred to some other (higher order) object is a serious error,
 and will easily cause severe memory corruption. 
 */
    GAIAGEO_DECLARE void gaiaFreeDbfList (gaiaDbfListPtr list);

/**
 Checks a DBF List object for validity

 \param list pointer to the DBF List object.
 
 \return 0 if not valid: any other value if valid.

 \sa gaiaAllocDbfList, gaiaFreeDbfList, gaiaIsValidDbfList,
 gaiaResetDbfEntity, gaiaCloneDbfEntity, gaiaAddDbfField
 */
    GAIAGEO_DECLARE int gaiaIsValidDbfList (gaiaDbfListPtr list);

/**
 Inserts a further DBF Field object into a DBF List object

 \param list pointer to the DBF List object.
 \param name text string: DBF Field name.
 \param type identifier of the corresponding DBF data type.
 \param offset corresponding offset into the DBF I/O buffer.
 \param length max field length (in bytes).
 \param decimals precision: number of decimal digits.

 \return the pointer to newly created DBF Field object.

 \sa gaiaAllocDbfField

 \note supported DBF data types are:
 \li 'C' text string [default]
 \li 'N' numeric
 \li 'D' date
 \li 'L' boolean
 */
    GAIAGEO_DECLARE gaiaDbfFieldPtr gaiaAddDbfField (gaiaDbfListPtr list,
						     char *name,
						     unsigned char type,
						     int offset,
						     unsigned char length,
						     unsigned char decimals);

/** 
 Resets a DBF List object to its initial empty state

 \param list pointer to the DBF List object.

 \sa gaiaFreeValue

 \note any DBF Field associated to the List object will be reset to its
 initial empty state (i.e. \e no \e value at all).
 */
    GAIAGEO_DECLARE void gaiaResetDbfEntity (gaiaDbfListPtr list);

/**
 Allocates a new DBF List object [duplicating an existing one]

 \param org pointer to input DBF List object.

 \return the pointer to newly created DBF List object.

 \sa gaiaCloneDbfField, gaiaCloneValue,

 \note the newly created object is an exact copy of the original one.
 \n this including any currently set Field Value.
 */
    GAIAGEO_DECLARE gaiaDbfListPtr gaiaCloneDbfEntity (gaiaDbfListPtr org);

/**
 Allocates a new Shapefile object.

 \return the pointer to newly created Shapefile object.

 \sa gaiaFreeShapefile, gaiaOpenShpRead, gaiaOpenShpWrite,
 gaiaReadShpEntity, gaiaShpAnalyze, gaiaWriteShpEntity, gaiaFlushShpHeaders

 \note you are responsible to destroy (before or after) any allocated Shapefile.
 \n you should phisically open the Shapefile in \e read or \e write mode
 before performing any actual I/O operation.
 */
    GAIAGEO_DECLARE gaiaShapefilePtr gaiaAllocShapefile (void);

/**
 Destroys a Shapefile object 

 \param shp pointer to the Shapefile object.

 \sa gaiaAllocShapefile, gaiaOpenShpRead, gaiaOpenShpWrite,
 gaiaReadShpEntity, gaiaShpAnalyze, gaiaWriteShpEntity, gaiaFlushShpHeaders
 
 \note destroying the Shapefile object will close any related file:
 anyway you a responsible to explicitly call gaiaFlushShpHeader
 before destroyng a Shapefile opened in \e write mode.
 */
    GAIAGEO_DECLARE void gaiaFreeShapefile (gaiaShapefilePtr shp);

/** 
 Open a Shapefile in read mode

 \param shp pointer to the Shapefile object.
 \param path \e abstract pathname to the corresponding file-system files.
 \param charFrom GNU ICONV name identifying the input charset encoding.
 \param charTo GNU ICONV name identifying the output charset encoding.

 \sa gaiaAllocShapefile, gaiaFreeShapefile, gaiaOpenShpWrite,
 gaiaReadShpEntity, gaiaShpAnalyze, gaiaWriteShpEntity, gaiaFlushShpHeaders
 
 \note on failure the object member \e Valid will be set to 0; and the
 object member \e LastError will contain the appropriate error message.
 \n the \e abstract pathname should not contain any suffix at all.
 */
    GAIAGEO_DECLARE void gaiaOpenShpRead (gaiaShapefilePtr shp,
					  const char *path,
					  const char *charFrom,
					  const char *charTo);

/**
 Open a Shapefile in read mode

 \param shp pointer to the Shapefile object.
 \param path \e abstract pathname to the corresponding file-system files.
 \param shape the SHAPE code; expected to be one of GAIA_SHP_POINT,
 GAIA_SHP_POLYLINE, GAIA_SHP_POLYGON, GAIA_SHP_MULTIPOINT, GAIA_SHP_POINTZ,
 GAIA_SHP_POLYLINEZ, GAIA_SHP_POLYGONZ, GAIA_SHP_MULTIPOINTZ, 
 GAIA_SHP_POINTM, GAIA_SHP_POLYLINEM, GAIA_SHP_POLYGONM, GAIA_SHP_MULTIPOINTM
 \param list pointer to DBF List object representing the corresponding
 data attributes.
 \param charFrom GNU ICONV name identifying the input charset encoding.
 \param charTo GNU ICONV name identifying the output charset encoding.

 \sa gaiaAllocShapefile, gaiaFreeShapefile, gaiaOpenShpRead, 
 gaiaReadShpEntity, gaiaShpAnalyze, gaiaWriteShpEntity, gaiaFlushShpHeaders
 
 \note on failure the object member \e Valid will be set to 0; and the
 object member \e LastError will contain the appropriate error message.
 \n the \e abstract pathname should not contain any suffix at all.
 */
    GAIAGEO_DECLARE void gaiaOpenShpWrite (gaiaShapefilePtr shp,
					   const char *path, int shape,
					   gaiaDbfListPtr list,
					   const char *charFrom,
					   const char *charTo);

/**
 Reads a feature from a Shapefile object

 \param shp pointer to the Shapefile object.
 \param current_row the row number identifying the feature to be read.
 \param srid feature's SRID 

 \return 0 on failure: any other value on success.

 \sa gaiaAllocShapefile, gaiaFreeShapefile, gaiaOpenShpRead, gaiaOpenShpWrite,
 gaiaShpAnalyze, gaiaWriteShpEntity, gaiaFlushShpHeaders

 \note on completion the Shapefile's \e Dbf member will contain the feature
 read:
 \li the \e Dbf->Geometry member will contain the corresponding Geometry
 \li and the \e Dbf->First member will point to the linked list containing
 the corresponding data attributes [both data formats and values].

 \remark the Shapefile object should be opened in \e read mode.
 */
    GAIAGEO_DECLARE int gaiaReadShpEntity (gaiaShapefilePtr shp,
					   int current_row, int srid);

/**
 Prescans a Shapefile object gathering informations

 \param shp pointer to the Shapefile object.

 \sa gaiaAllocShapefile, gaiaFreeShapefile, gaiaOpenShpRead, gaiaOpenShpWrite,
 gaiaReadShpEntity, gaiaWriteShpEntity, gaiaFlushShpHeaders

 \note on completion the Shapefile's \e EffectiveType will containt the
 Geometry type corresponding to features actually found.

 \remark the Shapefile object should be opened in \e read mode.
 */
    GAIAGEO_DECLARE void gaiaShpAnalyze (gaiaShapefilePtr shp);

/**
 Writes a feature into a Shapefile object
                                            
 \param shp pointer to the Shapefile object.
 \param entity pointer to DBF List object containing both Geometry and Field 
 values.

 \return 0 on failure: any other value on success.

 \sa gaiaAllocShapefile, gaiaFreeShapefile, gaiaOpenShpRead, gaiaOpenShpWrite,
 gaiaReadShpEntity, gaiaShpAnalyze, gaiaFlushShpHeaders

 \remark the Shapefile object should be opened in \e write mode.
 */
    GAIAGEO_DECLARE int gaiaWriteShpEntity (gaiaShapefilePtr shp,
					    gaiaDbfListPtr entity);

/**
 Writes into an output Shapefile any required header / footer

 \param shp pointer to the Shapefile object.

 \sa gaiaAllocShapefile, gaiaFreeShapefile, gaiaOpenShpRead, gaiaOpenShpWrite,
 gaiaReadShpEntity, gaiaShpAnalyze, gaiaWriteShpEntity

 \note forgetting to call gaiaFlushShpHeader for any Shapefile opened in
 \e write mode immediately before destroying the object, will surely 
 cause severe file corruption.
 */
    GAIAGEO_DECLARE void gaiaFlushShpHeaders (gaiaShapefilePtr shp);

/**
 Allocates a new DBF File object.

 \return the pointer to newly created DBF File object.

 \sa gaiaFreeDbf, gaiaOpenDbfRead, gaiaOpenDbfWrite,
 gaiaReadDbfEntity, gaiaWriteDbfEntity, gaiaFlushDbfHeader

 \note you are responsible to destroy (before or after) any allocated DBF File.
 \n you should phisically open the DBF File in \e read or \e write mode
 before performing any actual I/O operation.
 */
    GAIAGEO_DECLARE gaiaDbfPtr gaiaAllocDbf (void);

/**
 Destroys a DBF File object 
    
 \param dbf pointer to the DBF File object.
                                          
 \sa gaiaAllocDbf, gaiaFreeDbf, gaiaOpenDbfWrite,
 gaiaReadDbfEntity, gaiaWriteDbfEntity, gaiaFlushDbfHeader
 
 \note destroying the Shapefile object will close any related file:
 anyway you a responsible to explicitly call gaiaFlushShpHeader
 before destroyng a Shapefile opened in \e write mode.
 */
    GAIAGEO_DECLARE void gaiaFreeDbf (gaiaDbfPtr dbf);

/**
 Open a DBF File in read mode

 \param dbf pointer to the DBF File object.
 \param path pathname to the corresponding file-system file.
 \param charFrom GNU ICONV name identifying the input charset encoding.
 \param charTo GNU ICONV name identifying the output charset encoding.

 \sa gaiaAllocDbf, gaiaFreeDbf, gaiaOpenDbfWrite,
 gaiaReadDbfEntity, gaiaWriteDbfEntity, gaiaFlushDbfHeader

 \note on failure the object member \e Valid will be set to 0; and the
 object member \e LastError will contain the appropriate error message.
 */
    GAIAGEO_DECLARE void gaiaOpenDbfRead (gaiaDbfPtr dbf,
					  const char *path,
					  const char *charFrom,
					  const char *charTo);

/** 
 Open a DBF File in write mode

 \param dbf pointer to the DBF File object.
 \param path pathname to the corresponding file-system file.
 \param charFrom GNU ICONV name identifying the input charset encoding.
 \param charTo GNU ICONV name identifying the output charset encoding.

 \sa gaiaAllocDbf, gaiaFreeDbf, gaiaOpenDbfRead, 
 gaiaReadDbfEntity, gaiaWriteDbfEntity, gaiaFlushDbfHeader
 
 \note on failure the object member \e Valid will be set to 0; and the
 object member \e LastError will contain the appropriate error message.
 */
    GAIAGEO_DECLARE void gaiaOpenDbfWrite (gaiaDbfPtr dbf,
					   const char *path,
					   const char *charFrom,
					   const char *charTo);

/**
 Reads a record from a DBF File object

 \param dbf pointer to the DBF File object.
 \param current_row the row number identifying the record to be read.
 \param deleted on completion this variable will contain 0 if the record
 just read is valid: any other value if the record just read is marked as
 \e logically \e deleted.

 \return 0 on failure: any other value on success.

 \sa gaiaAllocDbf, gaiaFreeDbf, gaiaOpenDbfRead, gaiaOpenDbfWrite,
 gaiaFlushDbfHeader

 \note on completion the DBF File \e First member will point to the 
 linked list containing the corresponding data attributes [both data 
 formats and values].

 \remark the DBF File object should be opened in \e read mode.
 */
    GAIAGEO_DECLARE int gaiaReadDbfEntity (gaiaDbfPtr dbf, int current_row,
					   int *deleted);

/**
 Writes a record into a DBF File object

 \param dbf pointer to the DBF File object.
 \param entity pointer to DBF List object containing Fields and corresponding
 values.

 \return 0 on failure: any other value on success.

 \sa gaiaAllocDbf, gaiaFreeDbf, gaiaOpenDbfRead, gaiaOpenDbfWrite,
 gaiaReadDbfEntity, gaiaFlushDbfHeader

 \remark the DBF File object should be opened in \e write mode.
 */
    GAIAGEO_DECLARE int gaiaWriteDbfEntity (gaiaDbfPtr dbf,
					    gaiaDbfListPtr entity);

/**
 Writes into an output DBF File any required header / footer

 \param dbf pointer to the DBF File object.

 \sa gaiaAllocDbf, gaiaFreeDbf, gaiaOpenDbfRead, gaiaOpenDbfWrite,
 gaiaReadDbfEntity, gaiaWriteDbfEntity

 \note forgetting to call gaiaFlushDbfHeader for any DBF File opened in
 \e write mode immediately before destroying the object, will surely 
 cause severe file corruption.
 */
    GAIAGEO_DECLARE void gaiaFlushDbfHeader (gaiaDbfPtr dbf);



#ifndef OMIT_ICONV		/* ICONV enabled: supporting text reader */

/** 
 Creates a Text Reader object

 \param path to the corresponding file-system file.
 \param field_separator the character acting as a separator between adjacent 
 fields.
 \param text_separator the character used to quote text strings.
 \param decimal_separator the character used as a separator between integer
 and decimal digits for real numeric values.
 \param first_line_titles 0 if the first line contains regular values:
 any other value if the first line contains column names.
 \param encoding GNU ICONV name identifying the input charset encoding.

 \return the pointer to the newly created Text Reader object: NULL on failure

 \sa gaiaTextReaderDestroy, gaiaTextReaderParse,
 gaiaTextReaderGetRow, gaiaTextReaderFetchField

 \note you are responsible to destroy (before or after) any allocated Text
 Reader object.
 */
    GAIAGEO_DECLARE gaiaTextReaderPtr gaiaTextReaderAlloc (const char *path,
							   char field_separator,
							   char text_separator,
							   char
							   decimal_separator,
							   int
							   first_line_titles,
							   const char
							   *encoding);

/**
 Destroys a Text Reader object

 \param reader pointer to Text Reader object.

 \sa gaiaTextReaderAlloc, gaiaTextReaderParse,
 gaiaTextReaderGetRow, gaiaTextReaderFetchField
 */
    GAIAGEO_DECLARE void gaiaTextReaderDestroy (gaiaTextReaderPtr reader);

/**
 Prescans the external file associated to a Text Reade object

 \param reader pointer to Text Reader object.

 \return 0 on failure: any other value on success.

 \sa gaiaTextReaderAlloc, gaiaTextReaderDestroy, 
 gaiaTextReaderGetRow, gaiaTextReaderFetchField

 \note this preliminary step is required so to ensure:
 \li file consistency: checking expected formatting rules.
 \li identifying the number / type / name of fields [aka columns].
 \li identifying the actual number of lines within the file.
 */
    GAIAGEO_DECLARE int gaiaTextReaderParse (gaiaTextReaderPtr reader);

/**
 Reads a line from a Text Reader object
 
 \param reader pointer to Text Reader object.
 \param row_num the Line Number identifying the Line to be read.

 \return 0 on failure: any other value on success.

 \sa gaiaTextReaderAlloc, gaiaTextReaderDestroy, gaiaTextReaderParse,
 gaiaTextReaderFetchField

 \note this function will load the requested Line into the current buffer:
 you can then use gaiaTextReaderFetchField in order to retrieve
 any individual field [aka column] value.
 */
    GAIAGEO_DECLARE int gaiaTextReaderGetRow (gaiaTextReaderPtr reader,
					      int row_num);

/**
 Retrieves an individual field value from the current Line

 \param reader pointer to Text Reader object.
 \param field_num relative field [aka column] index: first field has index 0.
 \param type on completion this variable will contain the value type.
 \param value on completion this variable will contain the current field value.

 \return 0 on failure: any other value on success.

 \sa gaiaTextReaderAlloc, gaiaTextReaderDestroy, gaiaTextReaderParse,
 gaiaTextReaderGetRow
 */
    GAIAGEO_DECLARE int gaiaTextReaderFetchField (gaiaTextReaderPtr reader,
						  int field_num, int *type,
						  const char **value);

#endif				/* end ICONV (text reader) */

#ifdef __cplusplus
}
#endif

#endif				/* _GG_FORMATS_H */
/**************** End file: gg_formats.h **********/


/**************** Begin file: gg_dynamic.h **********/


/**
 \file gg_dynamic.h

 Geometry handling functions: DynamicLine handling
 */

#ifndef _GG_DYNAMIC
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GG_DYNAMIC
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* function prototypes */

/**
 Creates a new dynamicly growing line/ring object

 \return the pointer to newly created object

 \sa gaiaCreateDynamicLine, gaiaFreeDynamicLine

 \note you are responsible to destroy (before or after) any allocated 
 dynamically growing line/ring object.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr gaiaAllocDynamicLine (void);

/**
 Destroys a dynamically growing line/ring object

 \param p pointer to object to be destroyed

 \sa gaiaAllocDynamicLine
 */
    GAIAGEO_DECLARE void gaiaFreeDynamicLine (gaiaDynamicLinePtr p);

/**
 Appends a new 2D Point [XY] at the end of a dynamically growing line/ring
 object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
 
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaAppendPointToDynamicLine (gaiaDynamicLinePtr p, double x, double y);

/**
 Appends a new 3D Point [XYZ] at the end of a dynamically growing line/ring 
 object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
 \param z Z coordinate of the Point
  
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaAppendPointZToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
				       double z);

/**
 Appends a new 2D Point [XYM] at the end of a dynamically growing line/ring 
 object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
 \param m M measure of the Point
  
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaAppendPointMToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
				       double m);

/**
 Appends a new 3D Point [XYZM] at the end of a dynamically growing line/ring 
 object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
 \param z Z coordinate of the Point
 \param m M measure of the Point
  
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaAppendPointZMToDynamicLine (gaiaDynamicLinePtr p, double x,
					double y, double z, double m);

/**
 Appends a new 2D Point [XY] before the first one of a dynamically growing 
 line/ring object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
  
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaPrependPointToDynamicLine (gaiaDynamicLinePtr p, double x,
				       double y);

/**
 Appends a new 3D Point [XYZ] before the first one of a dynamically growing
 line/ring object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
 \param z Z coordinate of the Point
 
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaPrependPointZToDynamicLine (gaiaDynamicLinePtr p, double x,
					double y, double z);

/**
 Appends a new 2D Point [XYM] before the first one of a dynamically growing
 line/ring object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
 \param m M measure of the Point
 
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaPrependPointMToDynamicLine (gaiaDynamicLinePtr p, double x,
					double y, double m);

/**
 Appends a new 3D Point [XYZM] before the first one of a dynamically growing
 line/ring object

 \param p pointer to the dynamically growing line/ring object.
 \param x X coordinate of the Point
 \param y Y coordinate of the Point
 \param z Z coordinate of the Point
 \param m M measure of the Point
 
 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr
	gaiaPrependPointZMToDynamicLine (gaiaDynamicLinePtr p, double x,
					 double y, double z, double m);

/**
 Appends a new 2D Point [XY] immediately after the given Point into a
 dynamically growing line/ring object

 \param p pointer to the dynamically growing line/ring object.
 \param pt pointer to the given Point.
 \param x X coordinate of the Point to be appended
 \param y Y coordinate of the Point to be appended

 \sa gaiaDynamicLiceInsertBefore

 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaDynamicLineInsertAfter (gaiaDynamicLinePtr
							     p, gaiaPointPtr pt,
							     double x,
							     double y);

/**
 Appends a new 2D Point [XY] immediately before the given Point into a
 dynamically growing line/ring object

 \param p pointer to the dynamically growing line/ring object.
 \param pt pointer to the given Point.
 \param x X coordinate of the Point to be appended
 \param y Y coordinate of the Point to be appended

 \sa gaiaDynamicLiceInsertBeforeAfter

 \return the pointer to newly created Point
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaDynamicLineInsertBefore (gaiaDynamicLinePtr
							      p,
							      gaiaPointPtr pt,
							      double x,
							      double y);

/**
 Removes a given Point from a dynamically growing line/ring object

 \param p pointer to dynamically growing line/ring object.
 \param pt pointer to given Point.

 \note the given Point (referenced by its address) will be removed from
 the dynamically growin line/ring object.
 \n the given Point will be then implicitly destroyed.
 */
    GAIAGEO_DECLARE void gaiaDynamicLineDeletePoint (gaiaDynamicLinePtr p,
						     gaiaPointPtr pt);

/**
 Duplicates a dynamically growing line/ring object

 \param org pointer to dynamically growing line/ring object [origin].

 \return the pointer to newly created dynamic growing line/ring object:
 NULL on failure.

 \note the newly created object is an exact copy of the original one.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr gaiaCloneDynamicLine (gaiaDynamicLinePtr
							     org);

/**
 Duplicates and reverts a dynamically growing line/ring object

 \param org pointer to dynamically growing line/ring object [origin].

 \return the pointer to newly created dynamic growing line/ring object:
 NULL on failure.

 \note the newly created object is an exact copy of the origina one, except
 in that direction is reverted.
 \n i.e. first inpunt point becomes last output point, and last input point
 becomes first output point.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr
	gaiaReverseDynamicLine (gaiaDynamicLinePtr org);

/**
 Cuts a dynamically growing line/ring in two halves, using a given
 cut point

 \param org pointer to the input object [the line to be split].
 \param point pointer to given cut point.
 
 \return the pointer to newly created dynamic growing line/ring object:
 NULL on failure.

 \sa gaiaDynamicLineSplitAfter

 \note the newly created object will contain a line going from the orginal
 first point to the cut point [excluded].
 \n on completion the orginal line will be reduced, going from the cut
 point [included] to the original last point.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr
	gaiaDynamicLineSplitBefore (gaiaDynamicLinePtr org, gaiaPointPtr point);

/**
 Cuts a dynamically growing line/ring in two halves, using a given
 cut point 

 \param org pointer to the input object [the line to be split]. 
 \param point pointer to given cut point.

 \return the pointer to newly created dynamic growing line/ring object:
 NULL on failure.

 \sa gaiaDynamicLineSplitBefore

 \note the newly created object will contain a line going from the orginal
 first point to the cut point [included].
 \n on completion the orginal line will be reduced, going from the cut
 point [excluded] to the original last point.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr
	gaiaDynamicLineSplitAfter (gaiaDynamicLinePtr org, gaiaPointPtr point);

/**
 Merges two dynamically growing line/ring object into a single one

 \param org pointer to the first input object [first line].
 \param point pointer to the reference Point object.
 \param toJoin pointer to the second input object [second line].

 \return the pointer to newly created dynamically growing line/ring object 
 [merged line]: NULL on failure.

 \sa gaiaDynamicLineJoinBefore

 \note the reference Point must exists into the first line: the second line
 will then be inserted immediately after the reference Point.
 \n The newly created object will represent the resulting merged line:
 \n both input objects remain untouched.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr
	gaiaDynamicLineJoinAfter (gaiaDynamicLinePtr org, gaiaPointPtr point,
				  gaiaDynamicLinePtr toJoin);

/**
 Merges two dynamically growing line/ring object into a single one

 \param org pointer to the first input object [first line].
 \param point pointer to the reference Point object.
 \param toJoin pointer to the second input object [second line].

 \return the pointer to newly created dynamically growing line/ring object 
 [merged line]: NULL on failure.

 \sa gaiaDynamicLineJoinAfter

 \note the reference Point must exists into the first line: the second line 
 will then be inserted immediately before the reference Point.
 \n The newly created object will represent the resulting merged line: 
 \n both input objects remain untouched.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr
	gaiaDynamicLineJoinBefore (gaiaDynamicLinePtr org, gaiaPointPtr point,
				   gaiaDynamicLinePtr toJoin);

/**
 Finds a Point within a dymically growing line/ring object [by coords]

 \param p pointer to dymamically line/ring object.
 \param x Point X coordinate.
 \param y Point Y coordinate.

 \return the pointer to the corresponding Point object: NULL on failure.

 \sa gaiaDynamicLineFindByPos

 \note if the line object contains more Points sharing the same coordinates,
 a reference to the first one found will be returned.
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaDynamicLineFindByCoords (gaiaDynamicLinePtr
							      p, double x,
							      double y);

/**
 Finds a Point within a dymically growing line/ring object [by position]

 \param p pointer to dymamically line/ring object.
 \param pos relative position [first Point has index 0].

 \return the pointer to the corresponding Point object: NULL on failure.

 \sa gaiaDynamicLineFindByCoords
 */
    GAIAGEO_DECLARE gaiaPointPtr gaiaDynamicLineFindByPos (gaiaDynamicLinePtr p,
							   int pos);

/**
 Creates a new dynamicly growing line/ring object

 \param coords an array of COORDs, any dimension [XY, XYZ, XYM, XYZM]
 \param points number of points [aka vertices] into the array

 \return the pointer to newly created object 

 \sa gaiaAllocDynamicLine, gaiaFreeDynamicLine, gaiaLinestringStruct,
 gaiaRingStruct

 \note you are responsible to destroy (before or after) any allocated 
 dynamically growing line/ring object. 
 \n The COORDs array is usually expected to be one found within a
 gaiaLinestring or gaiaRing object.
 */
    GAIAGEO_DECLARE gaiaDynamicLinePtr gaiaCreateDynamicLine (double *coords,
							      int points);

#ifdef __cplusplus
}
#endif

#endif				/* _GG_DYNAMIC */
/**************** End file: gg_dynamic.h **********/


/**************** Begin file: gg_advanced.h **********/


/**
 \file gg_advanced.h

 Geometry handling functions: advanced
 */

#ifndef _GG_ADVANCED_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define _GG_ADVANCED_H
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#ifndef OMIT_PROJ		/* including PROJ.4 */

/**
 Converts and angle from Radians into Degrees
 \param rads the angle measured in Radians.
 
 \return the angle measured in Degrees.

 \sa gaiaDegsToRads

 \remark \b PROJ.4 support required
 */
    GAIAGEO_DECLARE double gaiaRadsToDegs (double rads);

/**
 Converts and angle from Degrees into Radians
 \param degs the angle measured in Degrees.

 \return the angle measured in Radians.

 \sa gaiaRadsToDegs

 \remark \b PROJ.4 support required
 */
    GAIAGEO_DECLARE double gaiaDegsToRads (double degs);

/**
 Tansforms a Geometry object into a different Reference System
 [aka Reprojection]
 \param org pointer to input Geometry object.
 \param proj_from geodetic parameters string [EPSG format] qualifying the
 input Reference System
 \param proj_to geodetic parameters string [EPSG format] qualifying the
 output Reference System

 \return the pointer to newly created Geometry object: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,  this including any Geometry returned by gaiaGeometryTransform()

 \remark \b PROJ.4 support required
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaTransform (gaiaGeomCollPtr org,
						   char *proj_from,
						   char *proj_to);

#endif				/* end including PROJ.4 */

#ifndef OMIT_GEOS		/* including GEOS */

/**
 Resets the GEOS error and warning messages to an empty state

 \sa gaiaGetGeosErrorMsg, gaiaGetGeosWarningMsg, gaiaSetGeosErrorMsg,
 gaiaSetGeosWarningMsg

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE void gaiaResetGeosMsg (void);

/**
 Return the latest GEOS error message (if any)

 \return the latest GEOS error message: an empty string if no error was
 previoysly found.

 \sa gaiaResetGeosMsg, gaiaGetGeosWarningMsg, gaiaSetGeosErrorMsg,
 gaiaSetGeosWarningMsg

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE const char *gaiaGetGeosErrorMsg (void);

/**
 Return the latest GEOS warning message (if any)

 \return the latest GEOS warning message: an empty string if no warning was 
 previoysly found.

 \sa gaiaResetGeosMsg, gaiaGetGeosErrorMsg, gaiaSetGeosErrorMsg,
 gaiaSetGeosWarningMsg

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE const char *gaiaGetGeosWarningMsg (void);

/**
 Set the current GEOS error message

 \param msg the error message to be set.

 \sa gaiaResetGeosMsg, gaiaGetGeosErrorMsg, gaiaGetGeosWarningMsg,
 gaiaSetGeosWarningMsg

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE void gaiaSetGeosErrorMsg (const char *msg);

/**
 Set the current GEOS warning message

 \param msg the warning message to be set.

 \sa gaiaResetGeosMsg, gaiaGetGeosErrorMsg, gaiaGetGeosWarningMsg,
 gaiaSetGeosErrorMsg

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE void gaiaSetGeosWarningMsg (const char *msg);

/**
 Converts a Geometry object into a GEOS Geometry

 \param gaia pointer to Geometry object

 \return handle to GEOS Geometry
 
 \sa gaiaFromGeos_XY, gaiaFromGeos_XYZ, gaiaFromGeos_XYM, gaiaFromGeos_XYZM

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE void *gaiaToGeos (const gaiaGeomCollPtr gaia);

/**
 Converts a GEOS Geometry into a Geometry object [XY dims]

 \param geos handle to GEOS Geometry

 \return the pointer to the newly created Geometry object

 \sa gaiaToGeos, gaiaFromGeos_XYZ, gaiaFromGeos_XYM, gaiaFromGeos_XYZM

 \note you are responsible to destroy (before or after) any allocated 
 Geometry, this including any Geometry returned by gaiaFromGeos_XY()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromGeos_XY (const void *geos);

/**
 Converts a GEOS Geometry into a Geometry object [XYZ dims]

 \param geos handle to GEOS Geometry
    
 \return the pointer to the newly created Geometry object

 \sa gaiaToGeos, gaiaFromGeos_XY, gaiaFromGeos_XYM, gaiaFromGeos_XYZM
 
 \note you are responsible to destroy (before or after) any allocated 
 Geometry, this including any Geometry returned by gaiaFromGeos_XYZ()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromGeos_XYZ (const void *geos);

/**
 Converts a GEOS Geometry into a Geometry object [XYM dims]

 \param geos handle to GEOS Geometry
    
 \return the pointer to the newly created Geometry object

 \sa gaiaToGeos, gaiaFromGeos_XY, gaiaFromGeos_XYZ, gaiaFromGeos_XYZM
 
 \note you are responsible to destroy (before or after) any allocated 
 Geometry, this including any Geometry returned by gaiaFromGeos_XYM()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromGeos_XYM (const void *geos);

/**
 Converts a GEOS Geometry into a Geometry object [XYZM dims]

 \param geos handle to GEOS Geometry
    
 \return the pointer to the newly created Geometry object

 \sa gaiaToGeos, gaiaFromGeos_XY, gaiaFromGeos_XYZ, gaiaFromGeos_XYM
 
 \note you are responsible to destroy (before or after) any allocated 
 Geometry, this including any Geometry returned by gaiaFromGeos_XYZM()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaFromGeos_XYZM (const void *geos);

/**
 Checks if a Geometry object represents an OGC Simple Geometry

 \param geom pointer to Geometry object.

 \return 0 if false; any other value if true

 \sa gaiaIsClosed, gaiaIsRing, gaiaIsValid

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaIsSimple (gaiaGeomCollPtr geom);

/**
 Checks if a Linestring object represents an OGC Closed Geometry

 \param line pointer to Geometry object.

 \return 0 if false; any other value if true

 \sa gaiaIsSimple, gaiaIsRing, gaiaIsValid

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaIsClosed (gaiaLinestringPtr line);

/**
 Checks if a Linestring object represents an OGC Ring Geometry

 \param line pointer to Geometry object.

 \return 0 if false; any other value if true

 \sa gaiaIsSimple, gaiaIsClosed, gaiaIsValid

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaIsRing (gaiaLinestringPtr line);

/**
 Checks if a Geometry object represents an OGC Valid Geometry

 \param geom pointer to Geometry object.

 \return 0 if false; any other value if true

 \sa gaiaIsSimple, gaiaIsClosed, gaiaIsRing

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaIsValid (gaiaGeomCollPtr geom);

/**
 Measures the total Length for a Geometry object

 \param geom pointer to Geometry object
 \param length on completion this variable will contain the measured length

 \return 0 on failure: any other value on success

 \sa gaiaGeomCollArea, gaiaMeasureLength

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollLength (gaiaGeomCollPtr geom,
					    double *length);
/**
 Measures the total Area for a Geometry object

 \param geom pointer to Geometry object
 \param area on completion this variable will contain the measured area

 \return 0 on failure: any other value on success

 \sa gaiaGeomCollLength, gaiaMeasureArea

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollArea (gaiaGeomCollPtr geom, double *area);


/**
 Attempts to rearrange a generic Geometry object into a Polygon or MultiPolygon

 \param geom the input Geometry object
 \param force_multi if not set to 0, then an eventual Polygon will be 
 returned casted to MultiPolygon

 \return the pointer to newly created Geometry object representing a
 Polygon or MultiPolygon Geometry: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaPolygonize()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaPolygonize (gaiaGeomCollPtr geom,
						    int force_multi);
/**
 Spatial relationship evalution: Equals
 
 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollDisjoint, gaiaGeomCollIntersects, gaiaGeomCollOverlaps,
 gaiaGeomCollCrosses, gaiaGeomCollContains, gaiaGeomCollWithin,
 gaiaGeomCollTouches, gaiaGeomCollRelate

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollEquals (gaiaGeomCollPtr geom1,
					    gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Disjoint

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollIntersects, gaiaGeomCollOverlaps,
 gaiaGeomCollCrosses, gaiaGeomCollContains, gaiaGeomCollWithin,
 gaiaGeomCollTouches, gaiaGeomCollRelate
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollDisjoint (gaiaGeomCollPtr geom1,
					      gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Intesects

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollDisjoint, gaiaGeomCollOverlaps,
 gaiaGeomCollCrosses, gaiaGeomCollContains, gaiaGeomCollWithin,
 gaiaGeomCollTouches, gaiaGeomCollRelate
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollIntersects (gaiaGeomCollPtr geom1,
						gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Overlaps

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollDisjoint, gaiaGeomCollIntersects, 
 gaiaGeomCollCrosses, gaiaGeomCollContains, gaiaGeomCollWithin,
 gaiaGeomCollTouches, gaiaGeomCollRelate
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollOverlaps (gaiaGeomCollPtr geom1,
					      gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Crosses

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollDisjoint, gaiaGeomCollIntersects, 
 gaiaGeomCollOverlaps, gaiaGeomCollContains, gaiaGeomCollWithin,
 gaiaGeomCollTouches, gaiaGeomCollRelate
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollCrosses (gaiaGeomCollPtr geom1,
					     gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Contains

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollDisjoint, gaiaGeomCollIntersects, 
 gaiaGeomCollOverlaps, gaiaGeomCollCrosses, gaiaGeomCollWithin,
 gaiaGeomCollTouches, gaiaGeomCollRelate
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollContains (gaiaGeomCollPtr geom1,
					      gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Within

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollDisjoint, gaiaGeomCollIntersects, 
 gaiaGeomCollOverlaps, gaiaGeomCollCrosses, gaiaGeomCollContains, 
 gaiaGeomCollTouches, gaiaGeomCollRelate
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollWithin (gaiaGeomCollPtr geom1,
					    gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Touches

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollDisjoint, gaiaGeomCollIntersects, 
 gaiaGeomCollOverlaps, gaiaGeomCollCrosses, gaiaGeomCollContains, 
 gaiaGeomCollWithin, gaiaGeomCollRelate
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollTouches (gaiaGeomCollPtr geom1,
					     gaiaGeomCollPtr geom2);

/**
 Spatial relationship evalution: Relate

 \param geom1 the first Geometry object to be evaluated
 \param geom2 the second Geometry object to be evaluated
 \param pattern intersection matrix pattern [DE-9IM]

 \return 0 if false: any other value if true

 \sa gaiaGeomCollEquals, gaiaGeomCollDisjoint, gaiaGeomCollIntersects, 
 gaiaGeomCollOverlaps, gaiaGeomCollCrosses, gaiaGeomCollContains, 
 gaiaGeomCollWithin, gaiaGeomCollTouches
 
 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollRelate (gaiaGeomCollPtr geom1,
					    gaiaGeomCollPtr geom2,
					    const char *pattern);

/**
 Calculates the minimum distance intercurring between two Geometry objects

 \param geom1 the first Geometry object 
 \param geom2 the second Geometry object 
 \param dist on completion this variable will contain the calculated distance

 \return 0 on failuer: any other value on success.

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollDistance (gaiaGeomCollPtr geom1,
					      gaiaGeomCollPtr geom2,
					      double *dist);

/**
 Spatial operator: Intersection
                                              
 \param geom1 the first Geometry object 
 \param geom2 the second Geometry object 

 \return the pointer to newly created Geometry object representing the
 geometry Intersection of both input Geometries: NULL on failure.

 \sa gaiaFreeGeomColl, gaiaGeometryUnion, gaiaGeometryDifference,
 gaiaGeometrySymDifference, gaiaBoundary

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaGeometryIntersection()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaGeometryIntersection (gaiaGeomCollPtr
							      geom1,
							      gaiaGeomCollPtr
							      geom2);

/**
 Spatial operator: Union

 \param geom1 the first Geometry object
 \param geom2 the second Geometry object

 \return the pointer to newly created Geometry object representing the
 geometry Union of both input Geometries: NULL on failure.

 \sa gaiaFreeGeomColl, gaiaUnaryUnion

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaGeometryUnion()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaGeometryUnion (gaiaGeomCollPtr geom1,
						       gaiaGeomCollPtr geom2);

/**
 Spatial operator: Difference

 \param geom1 the first Geometry object
 \param geom2 the second Geometry object

 \return the pointer to newly created Geometry object representing the
 geometry Difference of both input Geometries: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaGeometryDifference()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaGeometryDifference (gaiaGeomCollPtr
							    geom1,
							    gaiaGeomCollPtr
							    geom2);

/**
 Spatial operator: SymDifference

 \param geom1 the first Geometry object
 \param geom2 the second Geometry object

 \return the pointer to newly created Geometry object representing the
 geometry SymDifference of both input Geometries: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaGeometrySymDifference()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaGeometrySymDifference (gaiaGeomCollPtr
							       geom1,
							       gaiaGeomCollPtr
							       geom2);

/**
 Spatial operator: Boundary

 \param geom the Geometry object to be evaluated

 \return the pointer to newly created Geometry object representing the
 geometry Boundary of the input Geometry: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaBoundary()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaBoundary (gaiaGeomCollPtr geom);

/**
 Spatial operator: Centroid

 \param geom pointer to Geometry object.
 \param x on completion this variable will contain the centroid X coordinate 
 \param y on completion this variable will contain the centroid Y coordinate 
 
 \return 0 on failure: any other value on success

 \sa gaiaRingCentroid

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollCentroid (gaiaGeomCollPtr geom, double *x,
					      double *y);

/**
 Spatial operator: PointOnSurface

 \param geom pointer to Geometry object.
 \param x on completion this variable will contain the Point X coordinate  
 \param y on completion this variable will contain the Point Y coordinate
 
 \return 0 on failure: any other value on success

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE int gaiaGetPointOnSurface (gaiaGeomCollPtr geom, double *x,
					       double *y);

/**
 Spatial operator: Simplify

 \param geom the input Geometry object
 \param tolerance approximation threshold

 \return the pointer to newly created Geometry object representing the
 simplified Geometry [applying the Douglas-Peucker algorithm]: NULL on failure.

 \sa gaiaFreeGeomColl, gaiaGeomCollSimplifyPreserveTopology

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaGeomCollSimplify()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaGeomCollSimplify (gaiaGeomCollPtr geom,
							  double tolerance);

/**
 Spatial operator: Simplify [preserving topology]

 \param geom the input Geometry object
 \param tolerance approximation threshold

 \return the pointer to newly created Geometry object representing the
 simplified Geometry [applying the Douglas-Peucker algorithm]: NULL on failure.

 \sa gaiaFreeGeomColl, gaiaGeomCollSimplify

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaGeomCollSimplify()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr
	gaiaGeomCollSimplifyPreserveTopology (gaiaGeomCollPtr geom,
					      double tolerance);

/**
 Spatial operator: ConvexHull

 \param geom the input Geometry object

 \return the pointer to newly created Geometry object representing the
 ConvexHull of input Geometry: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaConvexHull()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaConvexHull (gaiaGeomCollPtr geom);

/** 
 Spatial operator: Buffer

 \param geom the input Geometry object
 \param radius the buffer's radius
 \param points number of points (aka vertices) to be used in order to 
 approximate a circular arc.

 \return the pointer to newly created Geometry object representing the
 Buffer of input Geometry: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaGeomCollBuffer()

 \remark \b GEOS support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaGeomCollBuffer (gaiaGeomCollPtr geom,
							double radius,
							int points);

#ifndef DOXYGEN_SHOULD_IGNORE_THIS
#ifdef GEOS_ADVANCED
#endif

/**
 Calculates the Hausdorff distance intercurring between two Geometry objects

 \param geom1 pointer to first Geometry object
 \param geom2 pointer to second Geometry object
 \param dist on completion this variable will contain the calculated Hausdorff
 distance 

 \return 0 on failure: any other value on success.

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE int gaiaHausdorffDistance (gaiaGeomCollPtr geom1,
					       gaiaGeomCollPtr geom2,
					       double *dist);

/**
 Spatial operator: Offset Curve

 \param geom the input Geometry object
 \param radius the buffer's radius
 \param points number of points (aka vertices) to be used in order to 
 approximate a circular arc.
 \param left_right if set to 1 the left-sided OffsetCurve will be returned;
 otherwise the right-sided one.

 \return the pointer to newly created Geometry object representing the
 OffsetCurve of input Geometry: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaOffsetCurve()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaOffsetCurve (gaiaGeomCollPtr geom,
						     double radius, int points,
						     int left_right);

/**
 Spatial operator: Single Sided Buffer

 \param geom the input Geometry object
 \param radius the buffer's radius
 \param points number of points (aka vertices) to be used in order to
 approximate a circular arc.
 \param left_right if set to 1 the left-sided Buffer will be returned;
 otherwise the right-sided one.

 \return the pointer to newly created Geometry object representing the
 single-sided Buffer of input Geometry: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaSingleSidedBuffer()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaSingleSidedBuffer (gaiaGeomCollPtr geom,
							   double radius,
							   int points,
							   int left_right);

/**
 Spatial operator: Shared Paths

 \param geom1 pointer to first Geometry object
 \param geom2 pointer to second Geometry object

 \return the pointer to newly created Geometry object representing any
 Share Path common to both input geometries: NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaSharedPaths()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaSharedPaths (gaiaGeomCollPtr geom1,
						     gaiaGeomCollPtr geom2);

/**
 Spatial operator: Line Interpolate Point

 \param ln_geom the input Geometry object [expected to be of lineal type]
 \param fraction total length fraction [in the range 0.0 / 1.0]

 \return the pointer to newly created Geometry object representing a Point
 laying on the input Geometry and positioned at the given length fraction:
 NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaLineInterpolatePoint()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaLineInterpolatePoint (gaiaGeomCollPtr
							      ln_geom,
							      double fraction);

/**
 Spatial operator: Line Substring

 \param ln_geom the input Geometry object [expected to be of lineal type]
 \param start_fraction substring start, expressed as total length fraction
 [in the range 0.0 / 1.0]
 \param end_fraction substring end, expressed as total length fraction

 \return the pointer to newly created Geometry object representing a Linestring
 laying on the input Geometry.
 \n this Linestring will begin (and stop) at given total length fractions. 
 NULL on failure.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaLineSubstring()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaLineSubstring (gaiaGeomCollPtr ln_geom,
						       double start_fraction,
						       double end_fraction);

/**
 Spatial operator: Shortest Line

 \param geom1 pointer to the first Geometry object.
 \param geom2 pointer to the second Geometry object.

 \return the pointer to newly created Geometry object representing a Linestring;
 NULL on failure.
 \n the returned Linestring graphically represents the minimum distance 
 intercurrinng between both input geometries.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaShortestLine()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaShortestLine (gaiaGeomCollPtr geom1,
						      gaiaGeomCollPtr geom2);

/**
 Spatial operator: Shortest Line

 \param geom1 pointer to the first Geometry object.
 \param geom2 pointer to the second Geometry object.
 \param tolerance approximation factor

 \return the pointer to newly created Geometry object; NULL on failure.
 \n the returned Geometry represents the first input Geometry (nicely
 \e snapped to the second input Geometry, whenever is possible).

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaShortestLine()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaSnap (gaiaGeomCollPtr geom1,
					      gaiaGeomCollPtr geom2,
					      double tolerance);

/**
 Spatial operator: Line Merge

 \param geom pointer to input Geometry object.

 \return the pointer to newly created Geometry object; NULL on failure.
 \n if possible, this representing a reassembled Linestring or MultiLinestring.

 \sa gaiaFreeGeomColl

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaLineMerge()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaLineMerge (gaiaGeomCollPtr geom);

/**
 Spatial operator: Unary Union

 \param geom the input Geometry object.

 \return the pointer to newly created Geometry object: NULL on failure.
 \n this function is the same as gaiaGeometryUnion, except in that this
 works internally to the input Geometry itself.
 NULL on failure.

 \sa gaiaFreeGeomColl, gaiaGeometryUnion

 \note you are responsible to destroy (before or after) any allocated Geometry,
 this including any Geometry returned by gaiaUnaryUnion()

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE gaiaGeomCollPtr gaiaUnaryUnion (gaiaGeomCollPtr geom);

/**
 Determines the location of the closest Point on Linestring to the given Point

 \param ln_geom pointer to first input Geometry object [expected to be of
 the lineal type].
 \param pt_geom pointer to second input Geometry object [expected to be a
 Point].

 \return the fraction [in the range 0.0 / 1.0] of ln_geom total length
 where the closest Point to pt_geom lays.

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE double gaiaLineLocatePoint (gaiaGeomCollPtr ln_geom,
						gaiaGeomCollPtr pt_geom);

/** 
 Topology check: test if a Geometry covers another one

 \param geom1 pointer to first input Geometry object.
 \param geom2 pointer to second input Geometry object.

 \return 0 if false; any other value if geom1 \e spatially \e covers geom2.

 \sa gaiaGeomCollCoveredBy

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollCovers (gaiaGeomCollPtr geom1,
					    gaiaGeomCollPtr geom2);

/**
 Topology check: test if a Geometry is covered by another one
                                            
 \param geom1 pointer to first input Geometry object.
 \param geom2 pointer to second input Geometry object.
                                               
 \return 0 if false; any other value if geom2 is \e spatially \e covered \e by
 geom1.

 \sa gaiaGeomCollCovers

 \remark \b GEOS-ADVANCED support required.
 */
    GAIAGEO_DECLARE int gaiaGeomCollCoveredBy (gaiaGeomCollPtr geom1,
					       gaiaGeomCollPtr geom2);

#endif				/* end GEOS advanced and experimental features */

#endif				/* end including GEOS */

#ifdef __cplusplus
}
#endif

#endif				/* _GG_ADVANCED_H */
/**************** End file: gg_advanced.h **********/


/**************** Begin file: spatialite.h **********/

int virtualshape_extension_init (sqlite3 * db);
int virtualdbf_extension_init (sqlite3 * db);
int virtualtext_extension_init (sqlite3 * db);
int virtualXL_extension_init (sqlite3 * db);
int virtualnetwork_extension_init (sqlite3 * db);
int virtualfdo_extension_init (sqlite3 * db);
int mbrcache_extension_init (sqlite3 * db);
int virtual_spatialindex_extension_init (sqlite3 * db);
/**************** End file: spatialite.h **********/


/**************** Begin file: gg_sqlaux.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiaaux.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

GAIAAUX_DECLARE int
gaiaIllegalSqlName (const char *name)
{
/* checks if column-name is an SQL illegal name */
    int i;
    int len;
    if (!name)
	return 1;
    len = strlen (name);
    if (len == 0)
	return 1;
    for (i = 0; i < len; i++)
      {
	  if (name[i] >= 'a' && name[i] <= 'z')
	      continue;
	  if (name[i] >= 'A' && name[i] <= 'Z')
	      continue;
	  if (name[i] >= '0' && name[i] <= '9')
	      continue;
	  if (name[i] == '_')
	      continue;
	  /* the name contains an illegal char */
	  return 1;
      }
    if (name[0] >= 'a' && name[0] <= 'z')
	return 0;
    if (name[0] >= 'A' && name[0] <= 'Z')
	return 0;
/* the first char in the name isn't a letter */
    return 1;
}

GAIAAUX_DECLARE int
gaiaIsReservedSqliteName (const char *name)
{
/* checks if column-name is an SQLite reserved keyword */
    char *reserved[] = {
	"ALL",
	"ALTER",
	"AND",
	"AS",
	"AUTOINCREMENT",
	"BETWEEN",
	"BLOB",
	"BY",
	"CASE",
	"CHECK",
	"COLLATE",
	"COMMIT",
	"CONSTRAINT",
	"CREATE",
	"CROSS",
	"DATE",
	"DATETIME",
	"DEFAULT",
	"DEFERRABLE",
	"DELETE",
	"DISTINCT",
	"DOUBLE",
	"DROP",
	"ELSE",
	"ESCAPE",
	"EXCEPT",
	"FOREIGN",
	"FROM",
	"FULL",
	"GLOB",
	"GROUP",
	"HAVING",
	"IN",
	"INDEX",
	"INNER",
	"INSERT",
	"INTEGER",
	"INTERSECT",
	"INTO",
	"IS",
	"ISNULL",
	"JOIN",
	"KEY",
	"LEFT",
	"LIKE",
	"LIMIT",
	"MATCH",
	"NATURAL",
	"NOT",
	"NOTNULL",
	"NULL",
	"ON",
	"OR",
	"ORDER",
	"OUTER",
	"PRAGMA",
	"PRIMARY",
	"REFERENCES",
	"REPLACE",
	"RIGHT",
	"ROLLBACK",
	"SELECT",
	"SET",
	"TABLE",
	"TEMP",
	"TEMPORARY",
	"THEN",
	"TEXT",
	"TIMESTAMP",
	"TO",
	"TRANSACTION",
	"UNION",
	"UNIQUE",
	"UPDATE",
	"USING",
	"VALUES",
	"VIEW",
	"WHEN",
	"WHERE",
	NULL
    };
    char **pw = reserved;
    while (*pw != NULL)
      {
	  if (strcasecmp (name, *pw) == 0)
	      return 1;
	  pw++;
      }
    return 0;
}

GAIAAUX_DECLARE int
gaiaIsReservedSqlName (const char *name)
{
/* checks if column-name is an SQL reserved keyword */
    char *reserved[] = {
	"ABSOLUTE",
	"ACTION",
	"ADD",
	"AFTER",
	"ALL",
	"ALLOCATE",
	"ALTER",
	"AND",
	"ANY",
	"ARE",
	"ARRAY",
	"AS",
	"ASC",
	"ASENSITIVE",
	"ASSERTION",
	"ASYMMETRIC",
	"AT",
	"ATOMIC",
	"AUTHORIZATION",
	"AVG",
	"BEFORE",
	"BEGIN",
	"BETWEEN",
	"BIGINT",
	"BINARY",
	"BIT",
	"BIT_LENGTH",
	"BLOB",
	"BOOLEAN",
	"BOTH",
	"BREADTH",
	"BY",
	"CALL",
	"CALLED",
	"CASCADE",
	"CASCADED",
	"CASE",
	"CAST",
	"CATALOG",
	"CHAR",
	"CHARACTER",
	"CHARACTER_LENGTH",
	"CHAR_LENGTH",
	"CHECK",
	"CLOB",
	"CLOSE",
	"COALESCE",
	"COLLATE",
	"COLLATION",
	"COLUMN",
	"COMMIT",
	"CONDITION",
	"CONNECT",
	"CONNECTION",
	"CONSTRAINT",
	"CONSTRAINTS",
	"CONSTRUCTOR",
	"CONTAINS",
	"CONTINUE",
	"CONVERT",
	"CORRESPONDING",
	"COUNT",
	"CREATE",
	"CROSS",
	"CUBE",
	"CURRENT",
	"CURRENT_DATE",
	"CURRENT_DEFAULT_TRANSFORM_GROUP",
	"CURRENT_PATH",
	"CURRENT_ROLE",
	"CURRENT_TIME",
	"CURRENT_TIMESTAMP",
	"CURRENT_TRANSFORM_GROUP_FOR_TYPE",
	"CURRENT_USER",
	"CURSOR",
	"CYCLE",
	"DATA",
	"DATE",
	"DAY",
	"DEALLOCATE",
	"DEC",
	"DECIMAL",
	"DECLARE",
	"DEFAULT",
	"DEFERRABLE",
	"DEFERRED",
	"DELETE",
	"DEPTH",
	"DEREF",
	"DESC",
	"DESCRIBE",
	"DESCRIPTOR",
	"DETERMINISTIC",
	"DIAGNOSTICS",
	"DISCONNECT",
	"DISTINCT",
	"DO",
	"DOMAIN",
	"DOUBLE",
	"DROP",
	"DYNAMIC",
	"EACH",
	"ELEMENT",
	"ELSE",
	"ELSEIF",
	"END",
	"EQUALS",
	"ESCAPE",
	"EXCEPT",
	"EXCEPTION",
	"EXEC",
	"EXECUTE",
	"EXISTS",
	"EXIT",
	"external",
	"EXTRACT",
	"FALSE",
	"FETCH",
	"FILTER",
	"FIRST",
	"FLOAT",
	"FOR",
	"FOREIGN",
	"FOUND",
	"FREE",
	"FROM",
	"FULL",
	"FUNCTION",
	"GENERAL",
	"GET",
	"GLOBAL",
	"GO",
	"GOTO",
	"GRANT",
	"GROUP",
	"GROUPING",
	"HANDLER",
	"HAVING",
	"HOLD",
	"HOUR",
	"IDENTITY",
	"IF",
	"IMMEDIATE",
	"IN",
	"INDICATOR",
	"INITIALLY",
	"INNER",
	"INOUT",
	"INPUT",
	"INSENSITIVE",
	"INSERT",
	"INT",
	"INTEGER",
	"INTERSECT",
	"INTERVAL",
	"INTO",
	"IS",
	"ISOLATION",
	"ITERATE",
	"JOIN",
	"KEY",
	"LANGUAGE",
	"LARGE",
	"LAST",
	"LATERAL",
	"LEADING",
	"LEAVE",
	"LEFT",
	"LEVEL",
	"LIKE",
	"LOCAL",
	"LOCALTIME",
	"LOCALTIMESTAMP",
	"LOCATOR",
	"LOOP",
	"LOWER",
	"MAP",
	"MATCH",
	"MAX",
	"MEMBER",
	"MERGE",
	"METHOD",
	"MIN",
	"MINUTE",
	"MODIFIES",
	"MODULE",
	"MONTH",
	"MULTISET",
	"NAMES",
	"NATIONAL",
	"NATURAL",
	"NCHAR",
	"NCLOB",
	"NEW",
	"NEXT",
	"NO",
	"NONE",
	"NOT",
	"NULL",
	"NULLIF",
	"NUMERIC",
	"OBJECT",
	"OCTET_LENGTH",
	"OF",
	"OLD",
	"ON",
	"ONLY",
	"OPEN",
	"OPTION",
	"OR",
	"ORDER",
	"ORDINALITY",
	"OUT",
	"OUTER",
	"OUTPUT",
	"OVER",
	"OVERLAPS",
	"PAD",
	"PARAMETER",
	"PARTIAL",
	"PARTITION",
	"PATH",
	"POSITION",
	"PRECISION",
	"PREPARE",
	"PRESERVE",
	"PRIMARY",
	"PRIOR",
	"PRIVILEGES",
	"PROCEDURE",
	"PUBLIC",
	"RANGE",
	"READ",
	"READS",
	"REAL",
	"RECURSIVE",
	"REF",
	"REFERENCES",
	"REFERENCING",
	"RELATIVE",
	"RELEASE",
	"REPEAT",
	"RESIGNAL",
	"RESTRICT",
	"RESULT",
	"RETURN",
	"RETURNS",
	"REVOKE",
	"RIGHT",
	"ROLE",
	"ROLLBACK",
	"ROLLUP",
	"ROUTINE",
	"ROW",
	"ROWS",
	"SAVEPOINT",
	"SCHEMA",
	"SCOPE",
	"SCROLL",
	"SEARCH",
	"SECOND",
	"SECTION",
	"SELECT",
	"SENSITIVE",
	"SESSION",
	"SESSION_USER",
	"SET",
	"SETS",
	"SIGNAL",
	"SIMILAR",
	"SIZE",
	"SMALLINT",
	"SOME",
	"SPACE",
	"SPECIFIC",
	"SPECIFICTYPE",
	"SQL",
	"SQLCODE",
	"SQLERROR",
	"SQLEXCEPTION",
	"SQLSTATE",
	"SQLWARNING",
	"START",
	"STATE",
	"STATIC",
	"SUBMULTISET",
	"SUBSTRING",
	"SUM",
	"SYMMETRIC",
	"SYSTEM",
	"SYSTEM_USER",
	"TABLE",
	"TABLESAMPLE",
	"TEMPORARY",
	"THEN",
	"TIME",
	"TIMESTAMP",
	"TIMEZONE_HOUR",
	"TIMEZONE_MINUTE",
	"TO",
	"TRAILING",
	"TRANSACTION",
	"TRANSLATE",
	"TRANSLATION",
	"TREAT",
	"TRIGGER",
	"TRIM",
	"TRUE",
	"UNDER",
	"UNDO",
	"UNION",
	"UNIQUE",
	"UNKNOWN",
	"UNNEST",
	"UNTIL",
	"UPDATE",
	"UPPER",
	"USAGE",
	"USER",
	"USING",
	"VALUE",
	"VALUES",
	"VARCHAR",
	"VARYING",
	"VIEW",
	"WHEN",
	"WHENEVER",
	"WHERE",
	"WHILE",
	"WINDOW",
	"WITH",
	"WITHIN",
	"WITHOUT",
	"WORK",
	"WRITE",
	"YEAR",
	"ZONE",
	NULL
    };
    char **pw = reserved;
    while (*pw != NULL)
      {
	  if (strcasecmp (name, *pw) == 0)
	      return 1;
	  pw++;
      }
    return 0;
}

GAIAAUX_DECLARE char *
gaiaQuotedSql (const char *value, int quote)
{
/*
/ returns a well formatted TEXT value for SQL
/ 1] strips trailing spaces
/ 2] masks any QUOTE inside the string, appending another QUOTE
/ 3] works for both SINGLE- and DOUBLE-QUOTE
*/
    const char *p_in;
    const char *p_end;
    char qt;
    char *out;
    char *p_out;
    int len = 0;
    int i;

    if (!value)
	return NULL;
    if (quote == GAIA_SQL_SINGLE_QUOTE)
	qt = '\'';
    else if (quote == GAIA_SQL_DOUBLE_QUOTE)
	qt = '"';
    else
	return NULL;

    p_end = value;
    for (i = (strlen (value) - 1); i >= 0; i--)
      {
	  /* stripping trailing spaces */
	  p_end = value + i;
	  if (value[i] != ' ')
	      break;
      }

    p_in = value;
    while (p_in <= p_end)
      {
	  /* computing the output length */
	  len++;
	  if (*p_in == qt)
	      len++;
	  p_in++;
      }
    if (len == 1 && *value == ' ')
      {
	  /* empty string */
	  len = 0;
      }

    out = malloc (len + 1);
    if (!out)
	return NULL;

    if (len == 0)
      {
	  /* empty string */
	  *out = '\0';
	  return out;
      }

    p_out = out;
    p_in = value;
    while (p_in <= p_end)
      {
	  /* creating the output string */
	  if (*p_in == qt)
	      *p_out++ = qt;
	  *p_out++ = *p_in++;
      }
    *p_out = '\0';
    return out;
}

GAIAAUX_DECLARE char *
gaiaSingleQuotedSql (const char *value)
{
/* convenience method supporting SINGLE-QUOTES */
    return gaiaQuotedSql (value, GAIA_SQL_SINGLE_QUOTE);
}

GAIAAUX_DECLARE char *
gaiaDoubleQuotedSql (const char *value)
{
/* convenience method supporting DOUBLE-QUOTES */
    return gaiaQuotedSql (value, GAIA_SQL_DOUBLE_QUOTE);
}

GAIAAUX_DECLARE void
gaiaCleanSqlString (char *value)
{
/*
/ returns a well formatted TEXT value for SQL
/ 1] strips trailing spaces
/ 2] masks any ' inside the string, appending another '
*/
    char new_value[1024];
    char *p;
    int len;
    int i;
    len = strlen (value);
    for (i = (len - 1); i >= 0; i--)
      {
	  /* stripping trailing spaces */
	  if (value[i] == ' ')
	      value[i] = '\0';
	  else
	      break;
      }
    p = new_value;
    for (i = 0; i < len; i++)
      {
	  if (value[i] == '\'')
	      *(p++) = '\'';
	  *(p++) = value[i];
      }
    *p = '\0';
    strcpy (value, new_value);
}
/**************** End file: gg_sqlaux.c **********/


/**************** Begin file: gg_utf8.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <errno.h> */

#if OMIT_ICONV == 0     /* ICONV is absolutely required */

#if defined(__MINGW32__) || defined(_WIN32)
#define LIBICONV_STATIC
/* #include <iconv.h> */
#define LIBCHARSET_STATIC
#ifdef _MSC_VER
/* <localcharset.h> isn't supported on OSGeo4W */
/* applying a tricky workaround to fix this issue */
extern const char * locale_charset (void);
#else /* sane Windows - not OSGeo4W */
/* #include <localcharset.h> */
#endif /* end localcharset */
#else /* not MINGW32 - WIN32 */
#ifdef __APPLE__
/* #include <iconv.h> */
/* #include <localcharset.h> */
#else /* not Mac OsX */
/* #include <iconv.h> */
/* #include <langinfo.h> */
#endif
#endif

/* #include <spatialite/gaiaaux.h> */

GAIAAUX_DECLARE const char *
gaiaGetLocaleCharset ()
{
/* identifies the locale charset */
#if defined(__MINGW32__) || defined(_WIN32)
    return locale_charset ();
#else /* not MINGW32 - WIN32 */
#ifdef __APPLE__
    return locale_charset ();
#else /* not Mac OsX */
    return nl_langinfo (CODESET);
#endif
#endif
}

GAIAAUX_DECLARE int
gaiaConvertCharset (char **buf, const char *fromCs, const char *toCs)
{
/* converting a string from a charset to another "on-the-fly" */
    char utf8buf[65536];
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    iconv_t cvt = iconv_open (toCs, fromCs);
    if (cvt == (iconv_t) (-1))
	goto unsupported;
    len = strlen (*buf);
    utf8len = 65536;
    pBuf = *buf;
    pUtf8buf = utf8buf;
    if (iconv (cvt, &pBuf, &len, &pUtf8buf, &utf8len) == (size_t) (-1))
	goto error;
    utf8buf[65536 - utf8len] = '\0';
    memcpy (*buf, utf8buf, (65536 - utf8len) + 1);
    iconv_close (cvt);
    return 1;
  error:
    iconv_close (cvt);
  unsupported:
    return 0;
}

GAIAAUX_DECLARE void *
gaiaCreateUTF8Converter (const char *fromCS)
{
/* creating a UTF8 converter and returning an opaque reference to it */
    iconv_t cvt = iconv_open ("UTF-8", fromCS);
    if (cvt == (iconv_t) (-1))
	return NULL;
    return cvt;
}

GAIAAUX_DECLARE void
gaiaFreeUTF8Converter (void *cvtCS)
{
/* destroying a UTF8 converter */
    if (cvtCS)
	iconv_close (cvtCS);
}

GAIAAUX_DECLARE char *
gaiaConvertToUTF8 (void *cvtCS, const char *buf, int buflen, int *err)
{
/* converting a string to UTF8 */
    char *utf8buf = 0;
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    int maxlen = buflen * 4;
    char *pUtf8buf;
    *err = 0;
    if (!cvtCS)
      {
	  *err = 1;
	  return NULL;
      }
    utf8buf = malloc (maxlen);
    len = buflen;
    utf8len = maxlen;
    pBuf = (char *) buf;
    pUtf8buf = utf8buf;
    if (iconv (cvtCS, &pBuf, &len, &pUtf8buf, &utf8len) == (size_t) (-1))
      {
	  free (utf8buf);
	  *err = 1;
	  return NULL;
      }
    utf8buf[maxlen - utf8len] = '\0';
    return utf8buf;
}

#endif  /* ICONV enabled/disabled */

/**************** End file: gg_utf8.c **********/


/**************** Begin file: gaia_exif.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <memory.h> */
/* #include <math.h> */
/* #include <float.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION	/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */
/* #include <spatialite/gaiaexif.h> */
/* #include <spatialite.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

static void
exifTagName (char gps, unsigned short tag_id, char *str, int len)
{
/* returns the canonical name corresponding to an EXIF TAG ID */
    int l;
    char *name = "UNKNOWN";
    if (gps)
      {
	  switch (tag_id)
	    {
	    case 0x00:
		name = "GPSVersionID";
		break;
	    case 0x01:
		name = "GPSLatitudeRef";
		break;
	    case 0x02:
		name = "GPSLatitude";
		break;
	    case 0x03:
		name = "GPSLongitudeRef";
		break;
	    case 0x04:
		name = "GPSLongitude";
		break;
	    case 0x05:
		name = "GPSAltitudeRef";
		break;
	    case 0x06:
		name = "GPSAltitude";
		break;
	    case 0x07:
		name = "GPSTimeStamp";
		break;
	    case 0x08:
		name = "GPSSatellites";
		break;
	    case 0x09:
		name = "GPSStatus";
		break;
	    case 0x0A:
		name = "GPSMeasureMode";
		break;
	    case 0x0B:
		name = "GPSDOP";
		break;
	    case 0x0C:
		name = "GPSSpeedRef";
		break;
	    case 0x0D:
		name = "GPSSpeed";
		break;
	    case 0x0E:
		name = "GPSTrackRef";
		break;
	    case 0x0F:
		name = "GPSTrack";
		break;
	    case 0x10:
		name = "GPSImgDirectionRef";
		break;
	    case 0x11:
		name = "GPSImgDirection";
		break;
	    case 0x12:
		name = "GPSMapDatum";
		break;
	    case 0x13:
		name = "GPSDestLatitudeRef";
		break;
	    case 0x14:
		name = "GPSDestLatitude";
		break;
	    case 0x15:
		name = "GPSDestLongitudeRef";
		break;
	    case 0x16:
		name = "GPSDestLongitude";
		break;
	    case 0x17:
		name = "GPSDestBearingRef";
		break;
	    case 0x18:
		name = "GPSDestBearing";
		break;
	    case 0x19:
		name = "GPSDestDistanceRef";
		break;
	    case 0x1A:
		name = "GPSDestDistance";
		break;
	    case 0x1B:
		name = "GPSProcessingMethod";
		break;
	    case 0x1C:
		name = "GPSAreaInformation";
		break;
	    case 0x1D:
		name = "GPSDateStamp";
		break;
	    case 0x1E:
		name = "GPSDifferential";
		break;
	    };
      }
    else
      {
	  switch (tag_id)
	    {
	    case 0x000B:
		name = "ACDComment";
		break;
	    case 0x00FE:
		name = "NewSubFile";
		break;
	    case 0x00FF:
		name = "SubFile";
		break;
	    case 0x0100:
		name = "ImageWidth";
		break;
	    case 0x0101:
		name = "ImageLength";
		break;
	    case 0x0102:
		name = "BitsPerSample";
		break;
	    case 0x0103:
		name = "Compression";
		break;
	    case 0x0106:
		name = "PhotometricInterpretation";
		break;
	    case 0x010A:
		name = "FillOrder";
		break;
	    case 0x010D:
		name = "DocumentName";
		break;
	    case 0x010E:
		name = "ImageDescription";
		break;
	    case 0x010F:
		name = "Make";
		break;
	    case 0x0110:
		name = "Model";
		break;
	    case 0x0111:
		name = "StripOffsets";
		break;
	    case 0x0112:
		name = "Orientation";
		break;
	    case 0x0115:
		name = "SamplesPerPixel";
		break;
	    case 0x0116:
		name = "RowsPerStrip";
		break;
	    case 0x0117:
		name = "StripByteCounts";
		break;
	    case 0x0118:
		name = "MinSampleValue";
		break;
	    case 0x0119:
		name = "MaxSampleValue";
		break;
	    case 0x011A:
		name = "XResolution";
		break;
	    case 0x011B:
		name = "YResolution";
		break;
	    case 0x011C:
		name = "PlanarConfiguration";
		break;
	    case 0x011D:
		name = "PageName";
		break;
	    case 0x011E:
		name = "XPosition";
		break;
	    case 0x011F:
		name = "YPosition";
		break;
	    case 0x0120:
		name = "FreeOffsets";
		break;
	    case 0x0121:
		name = "FreeByteCounts";
		break;
	    case 0x0122:
		name = "GrayResponseUnit";
		break;
	    case 0x0123:
		name = "GrayResponseCurve";
		break;
	    case 0x0124:
		name = "T4Options";
		break;
	    case 0x0125:
		name = "T6Options";
		break;
	    case 0x0128:
		name = "ResolutionUnit";
		break;
	    case 0x0129:
		name = "PageNumber";
		break;
	    case 0x012D:
		name = "TransferFunction";
		break;
	    case 0x0131:
		name = "Software";
		break;
	    case 0x0132:
		name = "DateTime";
		break;
	    case 0x013B:
		name = "Artist";
		break;
	    case 0x013C:
		name = "HostComputer";
		break;
	    case 0x013D:
		name = "Predictor";
		break;
	    case 0x013E:
		name = "WhitePoint";
		break;
	    case 0x013F:
		name = "PrimaryChromaticities";
		break;
	    case 0x0140:
		name = "ColorMap";
		break;
	    case 0x0141:
		name = "HalfToneHints";
		break;
	    case 0x0142:
		name = "TileWidth";
		break;
	    case 0x0143:
		name = "TileLength";
		break;
	    case 0x0144:
		name = "TileOffsets";
		break;
	    case 0x0145:
		name = "TileByteCounts";
		break;
	    case 0x014A:
		name = "SubIFD";
		break;
	    case 0x014C:
		name = "InkSet";
		break;
	    case 0x014D:
		name = "InkNames";
		break;
	    case 0x014E:
		name = "NumberOfInks";
		break;
	    case 0x0150:
		name = "DotRange";
		break;
	    case 0x0151:
		name = "TargetPrinter";
		break;
	    case 0x0152:
		name = "ExtraSample";
		break;
	    case 0x0153:
		name = "SampleFormat";
		break;
	    case 0x0154:
		name = "SMinSampleValue";
		break;
	    case 0x0155:
		name = "SMaxSampleValue";
		break;
	    case 0x0156:
		name = "TransferRange";
		break;
	    case 0x0157:
		name = "ClipPath";
		break;
	    case 0x0158:
		name = "XClipPathUnits";
		break;
	    case 0x0159:
		name = "YClipPathUnits";
		break;
	    case 0x015A:
		name = "Indexed";
		break;
	    case 0x015B:
		name = "JPEGTables";
		break;
	    case 0x015F:
		name = "OPIProxy";
		break;
	    case 0x0200:
		name = "JPEGProc";
		break;
	    case 0x0201:
		name = "JPEGInterchangeFormat";
		break;
	    case 0x0202:
		name = "JPEGInterchangeFormatLength";
		break;
	    case 0x0203:
		name = "JPEGRestartInterval";
		break;
	    case 0x0205:
		name = "JPEGLosslessPredictors";
		break;
	    case 0x0206:
		name = "JPEGPointTransforms";
		break;
	    case 0x0207:
		name = "JPEGQTables";
		break;
	    case 0x0208:
		name = "JPEGDCTables";
		break;
	    case 0x0209:
		name = "JPEGACTables";
		break;
	    case 0x0211:
		name = "YCbCrCoefficients";
		break;
	    case 0x0212:
		name = "YCbCrSubSampling";
		break;
	    case 0x0213:
		name = "YCbCrPositioning";
		break;
	    case 0x0214:
		name = "ReferenceBlackWhite";
		break;
	    case 0x02BC:
		name = "ExtensibleMetadataPlatform";
		break;
	    case 0x0301:
		name = "Gamma";
		break;
	    case 0x0302:
		name = "ICCProfileDescriptor";
		break;
	    case 0x0303:
		name = "SRGBRenderingIntent";
		break;
	    case 0x0320:
		name = "ImageTitle";
		break;
	    case 0x5001:
		name = "ResolutionXUnit";
		break;
	    case 0x5002:
		name = "ResolutionYUnit";
		break;
	    case 0x5003:
		name = "ResolutionXLengthUnit";
		break;
	    case 0x5004:
		name = "ResolutionYLengthUnit";
		break;
	    case 0x5005:
		name = "PrintFlags";
		break;
	    case 0x5006:
		name = "PrintFlagsVersion";
		break;
	    case 0x5007:
		name = "PrintFlagsCrop";
		break;
	    case 0x5008:
		name = "PrintFlagsBleedWidth";
		break;
	    case 0x5009:
		name = "PrintFlagsBleedWidthScale";
		break;
	    case 0x500A:
		name = "HalftoneLPI";
		break;
	    case 0x500B:
		name = "HalftoneLPIUnit";
		break;
	    case 0x500C:
		name = "HalftoneDegree";
		break;
	    case 0x500D:
		name = "HalftoneShape";
		break;
	    case 0x500E:
		name = "HalftoneMisc";
		break;
	    case 0x500F:
		name = "HalftoneScreen";
		break;
	    case 0x5010:
		name = "JPEGQuality";
		break;
	    case 0x5011:
		name = "GridSize";
		break;
	    case 0x5012:
		name = "ThumbnailFormat";
		break;
	    case 0x5013:
		name = "ThumbnailWidth";
		break;
	    case 0x5014:
		name = "ThumbnailHeight";
		break;
	    case 0x5015:
		name = "ThumbnailColorDepth";
		break;
	    case 0x5016:
		name = "ThumbnailPlanes";
		break;
	    case 0x5017:
		name = "ThumbnailRawBytes";
		break;
	    case 0x5018:
		name = "ThumbnailSize";
		break;
	    case 0x5019:
		name = "ThumbnailCompressedSize";
		break;
	    case 0x501A:
		name = "ColorTransferFunction";
		break;
	    case 0x501B:
		name = "ThumbnailData";
		break;
	    case 0x5020:
		name = "ThumbnailImageWidth";
		break;
	    case 0x5021:
		name = "ThumbnailImageHeight";
		break;
	    case 0x5022:
		name = "ThumbnailBitsPerSample";
		break;
	    case 0x5023:
		name = "ThumbnailCompression";
		break;
	    case 0x5024:
		name = "ThumbnailPhotometricInterp";
		break;
	    case 0x5025:
		name = "ThumbnailImageDescription";
		break;
	    case 0x5026:
		name = "ThumbnailEquipMake";
		break;
	    case 0x5027:
		name = "ThumbnailEquipModel";
		break;
	    case 0x5028:
		name = "ThumbnailStripOffsets";
		break;
	    case 0x5029:
		name = "ThumbnailOrientation";
		break;
	    case 0x502A:
		name = "ThumbnailSamplesPerPixel";
		break;
	    case 0x502B:
		name = "ThumbnailRowsPerStrip";
		break;
	    case 0x502C:
		name = "ThumbnailStripBytesCount";
		break;
	    case 0x502D:
		name = "ThumbnailResolutionX";
		break;
	    case 0x502E:
		name = "ThumbnailResolutionY";
		break;
	    case 0x502F:
		name = "ThumbnailPlanarConfig";
		break;
	    case 0x5030:
		name = "ThumbnailResolutionUnit";
		break;
	    case 0x5031:
		name = "ThumbnailTransferFunction";
		break;
	    case 0x5032:
		name = "ThumbnailSoftwareUsed";
		break;
	    case 0x5033:
		name = "ThumbnailDateTime";
		break;
	    case 0x5034:
		name = "ThumbnailArtist";
		break;
	    case 0x5035:
		name = "ThumbnailWhitePoint";
		break;
	    case 0x5036:
		name = "ThumbnailPrimaryChromaticities";
		break;
	    case 0x5037:
		name = "ThumbnailYCbCrCoefficients";
		break;
	    case 0x5038:
		name = "ThumbnailYCbCrSubsampling";
		break;
	    case 0x5039:
		name = "ThumbnailYCbCrPositioning";
		break;
	    case 0x503A:
		name = "ThumbnailRefBlackWhite";
		break;
	    case 0x503B:
		name = "ThumbnailCopyRight";
		break;
	    case 0x5090:
		name = "LuminanceTable";
		break;
	    case 0x5091:
		name = "ChrominanceTable";
		break;
	    case 0x5100:
		name = "FrameDelay";
		break;
	    case 0x5101:
		name = "LoopCount";
		break;
	    case 0x5110:
		name = "PixelUnit";
		break;
	    case 0x5111:
		name = "PixelPerUnitX";
		break;
	    case 0x5112:
		name = "PixelPerUnitY";
		break;
	    case 0x5113:
		name = "PaletteHistogram";
		break;
	    case 0x1000:
		name = "RelatedImageFileFormat";
		break;
	    case 0x800D:
		name = "ImageID";
		break;
	    case 0x80E3:
		name = "Matteing";
		break;
	    case 0x80E4:
		name = "DataType";
		break;
	    case 0x80E5:
		name = "ImageDepth";
		break;
	    case 0x80E6:
		name = "TileDepth";
		break;
	    case 0x828D:
		name = "CFARepeatPatternDim";
		break;
	    case 0x828E:
		name = "CFAPattern";
		break;
	    case 0x828F:
		name = "BatteryLevel";
		break;
	    case 0x8298:
		name = "Copyright";
		break;
	    case 0x829A:
		name = "ExposureTime";
		break;
	    case 0x829D:
		name = "FNumber";
		break;
	    case 0x83BB:
		name = "IPTC/NAA";
		break;
	    case 0x84E3:
		name = "IT8RasterPadding";
		break;
	    case 0x84E5:
		name = "IT8ColorTable";
		break;
	    case 0x8649:
		name = "ImageResourceInformation";
		break;
	    case 0x8769:
		name = "Exif IFD Pointer";
		break;
	    case 0x8773:
		name = "ICC_Profile";
		break;
	    case 0x8822:
		name = "ExposureProgram";
		break;
	    case 0x8824:
		name = "SpectralSensitivity";
		break;
	    case 0x8825:
		name = "GPSInfo IFD Pointer";
		break;
	    case 0x8827:
		name = "ISOSpeedRatings";
		break;
	    case 0x8828:
		name = "OECF";
		break;
	    case 0x9000:
		name = "ExifVersion";
		break;
	    case 0x9003:
		name = "DateTimeOriginal";
		break;
	    case 0x9004:
		name = "DateTimeDigitized";
		break;
	    case 0x9101:
		name = "ComponentsConfiguration";
		break;
	    case 0x9102:
		name = "CompressedBitsPerPixel";
		break;
	    case 0x9201:
		name = "ShutterSpeedValue";
		break;
	    case 0x9202:
		name = "ApertureValue";
		break;
	    case 0x9203:
		name = "BrightnessValue";
		break;
	    case 0x9204:
		name = "ExposureBiasValue";
		break;
	    case 0x9205:
		name = "MaxApertureValue";
		break;
	    case 0x9206:
		name = "SubjectDistance";
		break;
	    case 0x9207:
		name = "MeteringMode";
		break;
	    case 0x9208:
		name = "LightSource";
		break;
	    case 0x9209:
		name = "Flash";
		break;
	    case 0x920A:
		name = "FocalLength";
		break;
	    case 0x920B:
	    case 0xA20B:
		name = "FlashEnergy";
		break;
	    case 0x920C:
	    case 0xA20C:
		name = "SpatialFrequencyResponse";
		break;
	    case 0x920D:
		name = "Noise";
		break;
	    case 0x920E:
	    case 0xA20E:
		name = "FocalPlaneXResolution";
		break;
	    case 0x920F:
	    case 0XA20F:
		name = "FocalPlaneYResolution";
		break;
	    case 0x9210:
	    case 0xA210:
		name = "FocalPlaneResolutionUnit";
		break;
	    case 0x9211:
		name = "ImageNumber";
		break;
	    case 0x9212:
		name = "SecurityClassification";
		break;
	    case 0x9213:
		name = "ImageHistory";
		break;
	    case 0x9214:
	    case 0xA214:
		name = "SubjectLocation";
		break;
	    case 0x9215:
	    case 0xA215:
		name = "ExposureIndex";
		break;
	    case 0x9216:
		name = "TIFF/EPStandardID";
		break;
	    case 0x9217:
	    case 0xA217:
		name = "SensingMethod";
		break;
	    case 0x923F:
		name = "StoNits";
		break;
	    case 0x927C:
		name = "MakerNote";
		break;
	    case 0x9286:
		name = "UserComment";
		break;
	    case 0x9290:
		name = "SubSecTime";
		break;
	    case 0x9291:
		name = "SubSecTimeOriginal";
		break;
	    case 0x9292:
		name = "SubSecTimeDigitized";
		break;
	    case 0xA000:
		name = "FlashpixVersion";
		break;
	    case 0xA001:
		name = "ColorSpace";
		break;
	    case 0xA002:
		name = "ExifImageWidth";
		break;
	    case 0xA003:
		name = "ExifImageLength";
		break;
	    case 0xA004:
		name = "RelatedSoundFile";
		break;
	    case 0xA005:
		name = "Interoperability IFD Pointer";
		break;
	    case 0xA20D:
		name = "Noise";
		break;
	    case 0xA211:
		name = "ImageNumber";
		break;
	    case 0xA212:
		name = "SecurityClassification";
		break;
	    case 0xA213:
		name = "ImageHistory";
		break;
	    case 0xA216:
		name = "TIFF/EPStandardID";
		break;
	    case 0xA300:
		name = "FileSource";
		break;
	    case 0xA301:
		name = "SceneType";
		break;
	    case 0xA302:
		name = "CFAPattern";
		break;
	    case 0xA401:
		name = "CustomRendered";
		break;
	    case 0xA402:
		name = "ExposureMode";
		break;
	    case 0xA403:
		name = "WhiteBalance";
		break;
	    case 0xA404:
		name = "DigitalZoomRatio";
		break;
	    case 0xA405:
		name = "FocalLengthIn35mmFilm";
		break;
	    case 0xA406:
		name = "SceneCaptureType";
		break;
	    case 0xA407:
		name = "GainControl";
		break;
	    case 0xA408:
		name = "Contrast";
		break;
	    case 0xA409:
		name = "Saturation";
		break;
	    case 0xA40A:
		name = "Sharpness";
		break;
	    case 0xA40B:
		name = "DeviceSettingDescription";
		break;
	    case 0xA40C:
		name = "SubjectDistanceRange";
		break;
	    case 0xA420:
		name = "ImageUniqueID";
		break;
	    };
      }
    l = strlen (name);
    if (len > l)
	strcpy (str, name);
    else
      {
	  memset (str, '\0', len);
	  memcpy (str, name, len - 1);
      }
}

static unsigned short
exifImportU16 (const unsigned char *p, int little_endian,
	       int little_endian_arch)
{
/* fetches an unsigned 16bit int from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[2];
	unsigned short short_value;
    } convert;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 1);
		convert.byte[1] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 1);
		convert.byte[1] = *(p + 0);
	    }
      }
    return convert.short_value;
}

static unsigned int
exifImportU32 (const unsigned char *p, int little_endian,
	       int little_endian_arch)
{
/* fetches an unsigned 32bit int from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[4];
	unsigned int int_value;
    } convert;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
      }
    return convert.int_value;
}

static float
exifImportFloat32 (const unsigned char *p, int little_endian,
		   int little_endian_arch)
{
/* fetches a 32bit FLOAT from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[4];
	float float_value;
    } convert;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
      }
    return convert.float_value;
}

static void
exifSetTagValue (gaiaExifTagPtr tag, const unsigned char *blob, int endian_mode,
		 int endian_arch, int app1_offset)
{
/* setting the TAG value */
    int i;
    int sz = 0;
    unsigned int offset;
    const unsigned char *ptr;
    unsigned short short_value;
    unsigned int int_value;
    short sign_short_value;
    int sign_int_value;
    float float_value;
    double double_value;
    if (tag->Type == 1 || tag->Type == 2 || tag->Type == 6 || tag->Type == 7)
	sz = tag->Count;
    if (tag->Type == 3 || tag->Type == 8)
	sz = tag->Count * 2;
    if (tag->Type == 4 || tag->Type == 9 || tag->Type == 11)
	sz = tag->Count * 4;
    if (tag->Type == 5 || tag->Type == 10 || tag->Type == 12)
	sz = tag->Count * 8;
    if (sz <= 4)
      {
	  /* TAG values is stored within the offset */
	  ptr = tag->TagOffset;
      }
    else
      {
	  /* jumping to offset */
	  offset = exifImportU32 (tag->TagOffset, endian_mode, endian_arch);
	  offset += app1_offset + 10;
	  ptr = blob + offset;
      }
    if (tag->Type == 1 || tag->Type == 6 || tag->Type == 7)
      {
	  /* BYTE type */
	  tag->ByteValue = malloc (tag->Count);
	  memcpy (tag->ByteValue, ptr, tag->Count);
      }
    if (tag->Type == 2)
      {
	  /* STRING type */
	  tag->StringValue = malloc (tag->Count);
	  memcpy (tag->StringValue, ptr, tag->Count);
      }
    if (tag->Type == 3)
      {
	  /* SHORT type */
	  tag->ShortValues = malloc (tag->Count * sizeof (unsigned short));
	  for (i = 0; i < tag->Count; i++)
	    {
		short_value =
		    exifImportU16 (ptr + (i * 2), endian_mode, endian_arch);
		*(tag->ShortValues + i) = short_value;
	    }
      }
    if (tag->Type == 4)
      {
	  /* LONG type */
	  tag->LongValues = malloc (tag->Count * sizeof (unsigned int));
	  for (i = 0; i < tag->Count; i++)
	    {
		int_value =
		    exifImportU32 (ptr + (i * 4), endian_mode, endian_arch);
		*(tag->LongValues + i) = int_value;
	    }
      }
    if (tag->Type == 5)
      {
	  /* RATIONAL type */
	  tag->LongRationals1 = malloc (tag->Count * sizeof (unsigned int));
	  tag->LongRationals2 = malloc (tag->Count * sizeof (unsigned int));
	  for (i = 0; i < tag->Count; i++)
	    {
		int_value =
		    exifImportU32 (ptr + (i * 8), endian_mode, endian_arch);
		*(tag->LongRationals1 + i) = int_value;
		int_value =
		    exifImportU32 (ptr + (i * 8) + 4, endian_mode, endian_arch);
		*(tag->LongRationals2 + i) = int_value;
	    }
      }
    if (tag->Type == 8)
      {
	  /* SSHORT type */
	  tag->SignedShortValues = malloc (tag->Count * sizeof (short));
	  for (i = 0; i < tag->Count; i++)
	    {
		sign_short_value =
		    gaiaImport16 (ptr + (i * 2), endian_mode, endian_arch);
		*(tag->SignedShortValues + i) = sign_short_value;
	    }
      }
    if (tag->Type == 9)
      {
	  /* SIGNED LONG type */
	  tag->SignedLongValues = malloc (tag->Count * sizeof (int));
	  for (i = 0; i < tag->Count; i++)
	    {
		sign_int_value =
		    gaiaImport32 (ptr + (i * 4), endian_mode, endian_arch);
		*(tag->SignedLongValues + i) = sign_int_value;
	    }
      }
    if (tag->Type == 10)
      {
	  /* SIGNED RATIONAL type */
	  tag->SignedLongRationals1 = malloc (tag->Count * sizeof (int));
	  tag->SignedLongRationals2 = malloc (tag->Count * sizeof (int));
	  for (i = 0; i < tag->Count; i++)
	    {
		sign_int_value =
		    gaiaImport32 (ptr + (i * 8), endian_mode, endian_arch);
		*(tag->SignedLongRationals1 + i) = sign_int_value;
		sign_int_value =
		    gaiaImport32 (ptr + (i * 8) + 4, endian_mode, endian_arch);
		*(tag->SignedLongRationals2 + i) = sign_int_value;
	    }
      }
    if (tag->Type == 11)
      {
	  /* FLOAT type */
	  tag->FloatValues = malloc (tag->Count * sizeof (float));
	  for (i = 0; i < tag->Count; i++)
	    {
		float_value =
		    exifImportFloat32 (ptr + (i * 4), endian_mode, endian_arch);
		*(tag->FloatValues + i) = float_value;
	    }
      }
    if (tag->Type == 12)
      {
	  /* DOUBLE type */
	  tag->DoubleValues = malloc (tag->Count * sizeof (double));
	  for (i = 0; i < tag->Count; i++)
	    {
		double_value =
		    gaiaImport64 (ptr + (i * 8), endian_mode, endian_arch);
		*(tag->DoubleValues + i) = double_value;
	    }
      }
}

static void
exifParseTag (const unsigned char *blob, unsigned int offset, int endian_mode,
	      int endian_arch, gaiaExifTagListPtr list, int gps,
	      int app1_offset)
{
/* parsing some TAG and inserting into the list */
    unsigned short tag_id;
    unsigned short type;
    unsigned int count;
    gaiaExifTagPtr tag;
    tag_id = exifImportU16 (blob + offset, endian_mode, endian_arch);
    type = exifImportU16 (blob + offset + 2, endian_mode, endian_arch);
    count = exifImportU32 (blob + offset + 4, endian_mode, endian_arch);
    tag = malloc (sizeof (gaiaExifTag));
    tag->Gps = (char) gps;
    tag->TagId = tag_id;
    tag->Type = type;
    tag->Count = (unsigned short) count;
    memcpy (tag->TagOffset, blob + offset + 8, 4);
    tag->ByteValue = NULL;
    tag->StringValue = NULL;
    tag->ShortValues = NULL;
    tag->LongValues = NULL;
    tag->LongRationals1 = NULL;
    tag->LongRationals2 = NULL;
    tag->SignedShortValues = NULL;
    tag->SignedLongValues = NULL;
    tag->SignedLongRationals1 = NULL;
    tag->SignedLongRationals2 = NULL;
    tag->FloatValues = NULL;
    tag->DoubleValues = NULL;
    exifSetTagValue (tag, blob, endian_mode, endian_arch, app1_offset);
    tag->Next = NULL;
    if (!(list->First))
	list->First = tag;
    if (list->Last)
	(list->Last)->Next = tag;
    list->Last = tag;
    (list->NumTags)++;
}

static void
exifExpandIFD (gaiaExifTagListPtr list, const unsigned char *blob,
	       int endian_mode, int endian_arch, int app1_offset)
{
/* trying to expand the EXIF-IFD */
    unsigned int offset;
    unsigned short items;
    unsigned short i;
    gaiaExifTagPtr tag;
    if (!list)
	return;
    tag = list->First;
    while (tag)
      {
	  if (tag->TagId == 34665)
	    {
		/* ok, this one is an IFD pointer */
		offset =
		    exifImportU32 (tag->TagOffset, endian_mode, endian_arch);
		offset += app1_offset + 10;
		items = exifImportU16 (blob + offset, endian_mode, endian_arch);
		offset += 2;
		for (i = 0; i < items; i++)
		  {
		      /* fetching the TAGs */
		      exifParseTag (blob, offset, endian_mode, endian_arch,
				    list, 0, app1_offset);
		      offset += 12;
		  }
	    }
	  tag = tag->Next;
      }
}

static void
exifExpandGPS (gaiaExifTagListPtr list, const unsigned char *blob,
	       int endian_mode, int endian_arch, int app1_offset)
{
/* trying to expand the EXIF-GPS */
    unsigned int offset;
    unsigned short items;
    unsigned short i;
    gaiaExifTagPtr tag;
    if (!list)
	return;
    tag = list->First;
    while (tag)
      {
	  if (tag->TagId == 34853)
	    {
		/* ok, this one is a GPSinfo-IFD pointer */
		offset =
		    exifImportU32 (tag->TagOffset, endian_mode, endian_arch);
		offset += app1_offset + 10;
		items = exifImportU16 (blob + offset, endian_mode, endian_arch);
		offset += 2;
		for (i = 0; i < items; i++)
		  {
		      /* fetching the TAGs */
		      exifParseTag (blob, offset, endian_mode, endian_arch,
				    list, 1, app1_offset);
		      offset += 12;
		  }
	    }
	  tag = tag->Next;
      }
}

GAIAEXIF_DECLARE gaiaExifTagListPtr
gaiaGetExifTags (const unsigned char *blob, int size)
{
/* trying to parse a BLOB as an EXIF photo */
    gaiaExifTagListPtr list;
    int endian_arch = gaiaEndianArch ();
    int endian_mode;
    unsigned short app1_size;
    unsigned int offset;
    unsigned short items;
    unsigned short i;
    int x;
    int app1_offset;
    int app1_marker = 0;
    gaiaExifTagPtr pT;
    if (!blob)
	goto error;
    if (size < 14)
	goto error;
/* cecking for SOI [Start Of Image] */
    if (*(blob + 0) == 0xff && *(blob + 1) == 0xd8)
	;
    else
	goto error;
    app1_offset = 2;
    for (x = 2; x < size; x++)
      {
	  /* retrieving the APP1 Marker */
	  if (*(blob + x) == 0xff)
	      app1_marker = 1;
	  if (*(blob + x) == 0xe1)
	    {
		if (app1_marker)
		  {
		      app1_offset = x - 1;
		      break;
		  }
		else
		    app1_marker = 0;
	    }
      }
/* checking for APP1 Marker */
    if (*(blob + app1_offset) == 0xff && *(blob + app1_offset + 1) == 0xe1)
	;
    else
	goto error;
/* checking for EXIF identifier */
    if (memcmp (blob + app1_offset + 4, "Exif", 4) == 0)
	;
    else
	goto error;
/* checking for Pad */
    if (*(blob + app1_offset + 8) == 0x00 && *(blob + app1_offset + 9) == 0x00)
	;
    else
	goto error;
    if (memcmp (blob + app1_offset + 10, "II", 2) == 0)
	endian_mode = GAIA_LITTLE_ENDIAN;
    else if (memcmp (blob + app1_offset + 10, "MM", 2) == 0)
	endian_mode = GAIA_BIG_ENDIAN;
    else
	goto error;
/* OK: this BLOB seems to contain a valid EXIF */
    app1_size =
	exifImportU16 (blob + app1_offset + 2, endian_mode, endian_arch);
    if ((app1_size + app1_offset + 4) > size)
	goto error;
/* checking for marker */
    if (endian_mode == GAIA_BIG_ENDIAN)
      {
	  if (*(blob + app1_offset + 12) == 0x00
	      && *(blob + app1_offset + 13) == 0x2a)
	      ;
	  else
	      goto error;
      }
    else
      {
	  if (*(blob + app1_offset + 12) == 0x2a
	      && *(blob + app1_offset + 13) == 0x00)
	      ;
	  else
	      goto error;
      }
/* allocating an EXIF TAG LIST */
    list = malloc (sizeof (gaiaExifTagList));
    list->First = NULL;
    list->Last = NULL;
    list->NumTags = 0;
    list->TagsArray = NULL;
    offset = exifImportU32 (blob + app1_offset + 14, endian_mode, endian_arch);
    offset += app1_offset + 10;
/* jump to offset */
    items = exifImportU16 (blob + offset, endian_mode, endian_arch);
    offset += 2;
    for (i = 0; i < items; i++)
      {
/* fetching the EXIF TAGs */
	  exifParseTag (blob, offset, endian_mode, endian_arch, list, 0,
			app1_offset);
	  offset += 12;
      }
/* expanding the IFD and GPS tags */
    exifExpandIFD (list, blob, endian_mode, endian_arch, app1_offset);
    exifExpandGPS (list, blob, endian_mode, endian_arch, app1_offset);
    if (list->NumTags)
      {
	  /* organizing the EXIF TAGS as an Array */
	  list->TagsArray = malloc (sizeof (gaiaExifTagPtr) * list->NumTags);
	  pT = list->First;
	  i = 0;
	  while (pT)
	    {
		*(list->TagsArray + i++) = pT;
		pT = pT->Next;
	    }
      }
    return list;
  error:
    return NULL;
}

GAIAEXIF_DECLARE void
gaiaExifTagsFree (gaiaExifTagListPtr p)
{
/* memory cleanup; freeing the EXIF TAG list */
    gaiaExifTagPtr pT;
    gaiaExifTagPtr pTn;
    if (!p)
	return;
    pT = p->First;
    while (pT)
      {
	  pTn = pT->Next;
	  if (pT->ByteValue)
	      free (pT->ByteValue);
	  if (pT->StringValue)
	      free (pT->StringValue);
	  if (pT->ShortValues)
	      free (pT->ShortValues);
	  if (pT->LongValues)
	      free (pT->LongValues);
	  if (pT->LongRationals1)
	      free (pT->LongRationals1);
	  if (pT->LongRationals2)
	      free (pT->LongRationals2);
	  if (pT->SignedShortValues)
	      free (pT->SignedShortValues);
	  if (pT->SignedLongValues)
	      free (pT->SignedLongValues);
	  if (pT->SignedLongRationals1)
	      free (pT->SignedLongRationals1);
	  if (pT->SignedLongRationals2)
	      free (pT->SignedLongRationals2);
	  if (pT->FloatValues)
	      free (pT->FloatValues);
	  if (pT->DoubleValues)
	      free (pT->DoubleValues);
	  free (pT);
	  pT = pTn;
      }
    if (p->TagsArray)
	free (p->TagsArray);
    free (p);
}

GAIAEXIF_DECLARE int
gaiaGetExifTagsCount (gaiaExifTagListPtr tag_list)
{
/* returns the # TAGSs into this list */
    return tag_list->NumTags;
}

GAIAEXIF_DECLARE gaiaExifTagPtr
gaiaGetExifTagByPos (gaiaExifTagListPtr tag_list, const int pos)
{
/* returns the Nth TAG from this list */
    if (pos >= 0 && pos < tag_list->NumTags)
	return *(tag_list->TagsArray + pos);
    return NULL;
}

GAIAEXIF_DECLARE gaiaExifTagPtr
gaiaGetExifTagById (const gaiaExifTagListPtr tag_list,
		    const unsigned short tag_id)
{
/* returns a not-GPS TAG identified by its ID */
    gaiaExifTagPtr pT = tag_list->First;
    while (pT)
      {
	  if (!(pT->Gps) && pT->TagId == tag_id)
	      return pT;
	  pT = pT->Next;
      }
    return NULL;
}

GAIAEXIF_DECLARE gaiaExifTagPtr
gaiaGetExifGpsTagById (const gaiaExifTagListPtr tag_list,
		       const unsigned short tag_id)
{
/* returns a GPS TAG identified by its ID */
    gaiaExifTagPtr pT = tag_list->First;
    while (pT)
      {
	  if (pT->Gps && pT->TagId == tag_id)
	      return pT;
	  pT = pT->Next;
      }
    return NULL;
}

GAIAEXIF_DECLARE gaiaExifTagPtr
gaiaGetExifTagByName (const gaiaExifTagListPtr tag_list, const char *tag_name)
{
/* returns a TAG identified by its Name */
    char name[128];
    gaiaExifTagPtr pT = tag_list->First;
    while (pT)
      {
	  exifTagName (pT->Gps, pT->TagId, name, 128);
	  if (strcasecmp (name, tag_name) == 0)
	      return pT;
	  pT = pT->Next;
      }
    return NULL;
}

GAIAEXIF_DECLARE unsigned short
gaiaExifTagGetId (const gaiaExifTagPtr tag)
{
/* returns the TAG ID */
    return tag->TagId;
}

GAIAEXIF_DECLARE int
gaiaIsExifGpsTag (const gaiaExifTagPtr tag)
{
/* checks if this one is a GPS tag */
    return tag->Gps;
}

GAIAEXIF_DECLARE void
gaiaExifTagGetName (const gaiaExifTagPtr tag, char *str, int len)
{
/* returns the TAG symbolic Name */
    exifTagName (tag->Gps, tag->TagId, str, len);
}

GAIAEXIF_DECLARE unsigned short
gaiaExifTagGetValueType (const gaiaExifTagPtr tag)
{
/* returns the TAG value Type */
    return tag->Type;
}

GAIAEXIF_DECLARE unsigned short
gaiaExifTagGetNumValues (const gaiaExifTagPtr tag)
{
/* returns the # TAG Values */
    return tag->Count;
}

GAIAEXIF_DECLARE unsigned char
gaiaExifTagGetByteValue (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Byte value */
    if (ind >= 0
	&& ind <
	tag->Count && (tag->Type == 1 || tag->Type == 6 || tag->Type == 7))
      {
	  *ok = 1;
	  return *(tag->ByteValue + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE void
gaiaExifTagGetStringValue (const gaiaExifTagPtr tag, char *str, int len,
			   int *ok)
{
/* returns the String value */
    int l;
    if (tag->Type == 2)
      {
	  *ok = 1;
	  l = strlen (tag->StringValue);
	  if (len > l)
	      strcpy (str, tag->StringValue);
	  else
	    {
		memset (str, '\0', len);
		memcpy (str, tag->StringValue, len - 1);
	    }
	  return;
      }
    *ok = 0;
}

GAIAEXIF_DECLARE unsigned short
gaiaExifTagGetShortValue (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Short value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 3)
      {
	  *ok = 1;
	  return *(tag->ShortValues + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE unsigned int
gaiaExifTagGetLongValue (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Long value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 4)
      {
	  *ok = 1;
	  return *(tag->LongValues + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE unsigned int
gaiaExifTagGetRational1Value (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Rational (1) value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 5)
      {
	  *ok = 1;
	  return *(tag->LongRationals1 + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE unsigned int
gaiaExifTagGetRational2Value (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Rational (2) value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 5)
      {
	  *ok = 1;
	  return *(tag->LongRationals2 + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE double
gaiaExifTagGetRationalValue (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Rational  value as Double */
    double x;
    if (ind >= 0
	&& ind < tag->Count && tag->Type == 5 && *(tag->LongRationals2 + ind))
      {
	  *ok = 1;
	  x = (double) (*(tag->LongRationals1 + ind)) /
	      (double) (*(tag->LongRationals2 + ind));
	  return x;
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE short
gaiaExifTagGetSignedShortValue (const gaiaExifTagPtr tag, const int ind,
				int *ok)
{
/* returns the Nth Signed Short value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 8)
      {
	  *ok = 1;
	  return *(tag->SignedShortValues + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE int
gaiaExifTagGetSignedLongValue (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Signed Long value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 9)
      {
	  *ok = 1;
	  return *(tag->SignedLongValues + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE int
gaiaExifTagGetSignedRational1Value (const gaiaExifTagPtr tag, const int ind,
				    int *ok)
{
/* returns the Nth Signed Rational (1) value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 10)
      {
	  *ok = 1;
	  return *(tag->SignedLongRationals1 + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE int
gaiaExifTagGetSignedRational2Value (const gaiaExifTagPtr tag, const int ind,
				    int *ok)
{
/* returns the Nth Signed Rational (2) value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 10)
      {
	  *ok = 1;
	  return *(tag->SignedLongRationals2 + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE double
gaiaExifTagGetSignedRationalValue (const gaiaExifTagPtr tag, const int ind,
				   int *ok)
{
/* returns the Nth Signed Rational  value as Double */
    double x;
    if (ind >= 0
	&& ind <
	tag->Count && tag->Type == 10 && *(tag->SignedLongRationals2 + ind))
      {
	  *ok = 1;
	  x = (double) (*(tag->SignedLongRationals1 + ind)) /
	      (double) (*(tag->SignedLongRationals2 + ind));
	  return x;
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE float
gaiaExifTagGetFloatValue (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Float value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 11)
      {
	  *ok = 1;
	  return *(tag->FloatValues + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE double
gaiaExifTagGetDoubleValue (const gaiaExifTagPtr tag, const int ind, int *ok)
{
/* returns the Nth Double value */
    if (ind >= 0 && ind < tag->Count && tag->Type == 12)
      {
	  *ok = 1;
	  return *(tag->DoubleValues + ind);
      }
    *ok = 0;
    return 0;
}

GAIAEXIF_DECLARE void
gaiaExifTagGetHumanReadable (const gaiaExifTagPtr tag, char *str, int len,
			     int *ok)
{
/* returns the Human Readable value */
    char *human = "";
    char dummy[1024];
    int l;
    int xok;
    double dblval;
    switch (tag->TagId)
      {
      case 0x0128:		/* ResolutionUnit */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 2:
		      human = "Inches";
		      break;
		  case 3:
		      human = "Centimeters";
		      break;
		  };
	    }
	  break;
      case 0x8822:		/* ExposureProgram */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Not defined";
		      break;
		  case 1:
		      human = "Manual";
		      break;
		  case 2:
		      human = "Normal program";
		      break;
		  case 3:
		      human = "Aperture priority";
		      break;
		  case 4:
		      human = "Shutter priority";
		      break;
		  case 5:
		      human = "Creative program (biased toward depth of field)";
		      break;
		  case 6:
		      human =
			  "Action program (biased toward fast shutter speed)";
		      break;
		  case 7:
		      human =
			  "Portrait mode (for closeup photos with the background out of focus)";
		      break;
		  case 8:
		      human =
			  "Landscape mode (for landscape photos with the background in focus)";
		      break;
		  };
	    }
	  break;
      case 0xA402:		/* ExposureMode */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Auto exposure";
		      break;
		  case 1:
		      human = "Manual exposure";
		      break;
		  case 2:
		      human = "Auto bracket";
		      break;
		  };
	    }
	  break;
      case 0x0112:		/* Orientation */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 1:
		      human = "Normal";
		      break;
		  case 2:
		      human = "Mirrored";
		      break;
		  case 3:
		      human = "Upsidedown";
		      break;
		  case 4:
		      human = "Upsidedown Mirrored";
		      break;
		  case 5:
		      human = "90 deg Clockwise Mirrored";
		      break;
		  case 6:
		      human = "90 deg Counterclocwise";
		      break;
		  case 7:
		      human = "90 deg Counterclocwise Mirrored";
		      break;
		  case 8:
		      human = "90 deg Mirrored";
		      break;
		  };
	    }
	  break;
      case 0x9207:		/* MeteringMode */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 1:
		      human = "Average";
		      break;
		  case 2:
		      human = "Center Weighted Average";
		      break;
		  case 3:
		      human = "Spot";
		      break;
		  case 4:
		      human = "MultiSpot";
		      break;
		  case 5:
		      human = "MultiSegment";
		      break;
		  case 6:
		      human = "Partial";
		      break;
		  case 255:
		      human = "Other";
		      break;
		  };
	    }
	  break;
      case 0xA403:		/* WhiteBalance */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Auto";
		      break;
		  case 1:
		      human = "Sunny";
		      break;
		  case 2:
		      human = "Cloudy";
		      break;
		  case 3:
		      human = "Tungsten";
		      break;
		  case 4:
		      human = "Fluorescent";
		      break;
		  case 5:
		      human = "Flash";
		      break;
		  case 6:
		      human = "Custom";
		      break;
		  case 129:
		      human = "Manual";
		      break;
		  };
	    }
	  break;
      case 0x9209:		/* Flash */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		  case 16:
		  case 24:
		  case 32:
		      human = "No Flash";
		      break;
		  case 1:
		      human = "Flash";
		      break;
		  case 5:
		      human = "Flash, strobe return light not detected";
		      break;
		  case 7:
		      human = "Flash, strobe return light detected";
		      break;
		  case 9:
		      human = "Compulsory Flash";
		      break;
		  case 13:
		      human = "Compulsory Flash, Return light not detected";
		      break;
		  case 15:
		      human = "Compulsory Flash, Return light detected";
		      break;
		  case 25:
		      human = "Flash, Auto-Mode";
		      break;
		  case 29:
		      human = "Flash, Auto-Mode, Return light not detected";
		      break;
		  case 31:
		      human = "Flash, Auto-Mode, Return light detected";
		      break;
		  case 65:
		      human = "Red Eye";
		      break;
		  case 69:
		      human = "Red Eye, Return light not detected";
		      break;
		  case 71:
		      human = "Red Eye, Return light detected";
		      break;
		  case 73:
		      human = "Red Eye, Compulsory Flash";
		      break;
		  case 77:
		      human =
			  "Red Eye, Compulsory Flash, Return light not detected";
		      break;
		  case 79:
		      human =
			  "Red Eye, Compulsory Flash, Return light detected";
		      break;
		  case 89:
		      human = "Red Eye, Auto-Mode";
		      break;
		  case 93:
		      human = "Red Eye, Auto-Mode, Return light not detected";
		      break;
		  case 95:
		      human = "Red Eye, Auto-Mode, Return light detected";
		      break;
		  };
	    }
	  break;
      case 0xA217:		/* SensingMethod */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 1:
		      human = "Not defined";
		      break;
		  case 2:
		      human = "One Chip Color Area Sensor";
		      break;
		  case 3:
		      human = "Two Chip Color Area Sensor";
		      break;
		  case 4:
		      human = "Three Chip Color Area Sensor";
		      break;
		  case 5:
		      human = "Color Sequential Area Sensor";
		      break;
		  case 7:
		      human = "Trilinear Sensor";
		      break;
		  case 8:
		      human = "Color Sequential Linear Sensor";
		      break;
		  };
	    }
	  break;
      case 0xA406:		/* SceneCaptureType */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Standard";
		      break;
		  case 1:
		      human = "Landscape";
		      break;
		  case 2:
		      human = "Portrait";
		      break;
		  case 3:
		      human = "Night scene";
		      break;
		  };
	    }
	  break;
      case 0xA407:		/* GainControl */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "None";
		      break;
		  case 1:
		      human = "Low gain up";
		      break;
		  case 2:
		      human = "High gain up";
		      break;
		  case 3:
		      human = "Low gain down";
		      break;
		  case 4:
		      human = "High gain down";
		      break;
		  };
	    }
	  break;
      case 0xA408:		/* Contrast */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Normal";
		      break;
		  case 1:
		      human = "Soft";
		      break;
		  case 2:
		      human = "Hard";
		      break;
		  };
	    }
	  break;
      case 0xA409:		/* Saturation */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Normal";
		      break;
		  case 1:
		      human = "Low saturation";
		      break;
		  case 2:
		      human = "High saturation";
		      break;
		  };
	    }
	  break;
      case 0xA40A:		/* Sharpness */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Normal";
		      break;
		  case 1:
		      human = "Soft";
		      break;
		  case 2:
		      human = "Hard";
		      break;
		  };
	    }
	  break;
      case 0xA40C:		/* SubjectDistanceRange */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Unknown";
		      break;
		  case 1:
		      human = "Macro";
		      break;
		  case 2:
		      human = "Close view";
		      break;
		  case 3:
		      human = "Distant view";
		      break;
		  };
	    }
	  break;
      case 0x9208:		/* LightSource */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 0:
		      human = "Unknown";
		      break;
		  case 1:
		      human = "Daylight";
		      break;
		  case 2:
		      human = "Fluorescent";
		      break;
		  case 3:
		      human = "Tungsten (incandescent light)";
		      break;
		  case 4:
		      human = "Flash";
		      break;
		  case 9:
		      human = "Fine weather";
		      break;
		  case 10:
		      human = "Cloudy weather";
		      break;
		  case 11:
		      human = "Shade";
		  case 12:
		      human = "Daylight fluorescent (D 5700 – 7100K)";
		      break;
		  case 13:
		      human = "Day white fluorescent (N 4600 – 5400K)";
		      break;
		  case 14:
		      human = "Cool white fluorescent (W 3900 – 4500K)";
		  case 15:
		      human = "White fluorescent (WW 3200 – 3700K)";
		      break;
		  case 17:
		      human = "Standard light A";
		      break;
		  case 18:
		      human = "Standard light B";
		      break;
		  case 19:
		      human = "Standard light C";
		      break;
		  case 20:
		      human = "D55";
		      break;
		  case 21:
		      human = "D65";
		      break;
		  case 22:
		      human = "D75";
		      break;
		  case 23:
		      human = "D50";
		      break;
		  case 24:
		      human = "ISO studio tungsten";
		      break;
		  case 255:
		      human = "other light source";
		      break;
		  };
	    }
	  break;
      case 0xA001:		/* ColorSpace */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		switch (*(tag->ShortValues + 0))
		  {
		  case 1:
		      human = "sRGB";
		      break;
		  case 0xffff:
		      human = "Uncalibrated";
		      break;
		  };
	    }
	  break;
      case 0x8827:		/* ISOSpeedRatings */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		sprintf (dummy, "%u ISO", *(tag->ShortValues + 0));
		human = dummy;
	    }
	  break;
      case 0xA002:		/* ExifImageWidth */
      case 0xA003:		/* ExifImageLength */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		sprintf (dummy, "%u pixels", *(tag->ShortValues + 0));
		human = dummy;
	    }
	  else if (tag->Type == 4 && tag->Count == 1)
	    {
		sprintf (dummy, "%u pixels", *(tag->LongValues + 0));
		human = dummy;
	    }
	  break;
      case 0x829A:		/* ExposureTime */
	  if (tag->Type == 5 && tag->Count == 1)
	    {
		dblval = gaiaExifTagGetRationalValue (tag, 0, &xok);
		if (xok)
		  {
		      if (dblval < 1.0)
			{
			    dblval = 1.0 / dblval;
			    sprintf (dummy, "1/%1.0f sec", dblval);
			    human = dummy;
			}
		      else
			{
			    sprintf (dummy, "%1.0f sec", dblval);
			    human = dummy;
			}
		  }
	    }
	  break;
      case 0x9201:		/* ShutterSpeedValue */
	  if (tag->Type == 10 && tag->Count == 1)
	    {
		dblval = gaiaExifTagGetSignedRationalValue (tag, 0, &xok);
		if (xok)
		  {
		      dblval = exp (dblval * log (2));
		      if (dblval > 1.0)
			  dblval = floor (dblval);
		      if (dblval < 1.0)
			{
			    dblval = math_round (1.0 / dblval);
			    sprintf (dummy, "%1.0f sec", dblval);
			    human = dummy;
			}
		      else
			{
			    sprintf (dummy, "1/%1.0f sec", dblval);
			    human = dummy;
			}
		  }
	    }
	  break;
      case 0x829D:		/* FNumber */
	  if (tag->Type == 5 && tag->Count == 1)
	    {
		dblval = gaiaExifTagGetRationalValue (tag, 0, &xok);
		if (xok)
		  {
		      sprintf (dummy, "F %1.1f", dblval);
		      human = dummy;
		  }
	    }
	  break;
      case 0x9202:		/* ApertureValue */
      case 0x9205:		/* MaxApertureValue */
	  if (tag->Type == 5 && tag->Count == 1)
	    {
		dblval = gaiaExifTagGetRationalValue (tag, 0, &xok);
		if (xok)
		  {
		      dblval = exp ((dblval * log (2)) / 2.0);
		      sprintf (dummy, "F %1.1f", dblval);
		      human = dummy;
		  }
	    }
	  break;
      case 0x920A:		/* FocalLength */
	  if (tag->Type == 5 && tag->Count == 1)
	    {
		dblval = gaiaExifTagGetRationalValue (tag, 0, &xok);
		if (xok)
		  {
		      sprintf (dummy, "%1.1f mm", dblval);
		      human = dummy;
		  }
	    }
	  break;
      case 0xA405:		/* FocalLengthIn35mmFilm */
	  if (tag->Type == 3 && tag->Count == 1)
	    {
		sprintf (dummy, "%u mm", *(tag->ShortValues + 0));
		human = dummy;
	    }
	  break;
      case 0x9204:		/* ExposureBiasValue */
	  if (tag->Type == 10 && tag->Count == 1)
	    {
		dblval = gaiaExifTagGetSignedRationalValue (tag, 0, &xok);
		if (xok)
		  {
		      sprintf (dummy, "%1.2f EV", dblval);
		      human = dummy;
		  }
	    }
	  break;
      };
    l = strlen (human);
    if (l > 0)
      {
	  if (len > l)
	      strcpy (str, human);
	  else
	    {
		memset (str, '\0', len);
		memcpy (str, human, len - 1);
	    }
	  *ok = 1;
	  return;
      }
    *ok = 0;
}

GAIAEXIF_DECLARE int
gaiaGuessBlobType (const unsigned char *blob, int size)
{
/* returns the BLOB content type */
    int jpeg = 0;
    int exif = 0;
    int exif_gps = 0;
    int geom = 1;
    gaiaExifTagListPtr exif_list;
    gaiaExifTagPtr pT;
    unsigned char jpeg1_signature[2];
    unsigned char jpeg2_signature[2];
    unsigned char jpeg3_signature[4];
    unsigned char jfif_signature[4];
    unsigned char exif_signature[4];
    unsigned char png_signature[8];
    unsigned char zip_signature[4];
    unsigned char tiff_signature_little[4];
    unsigned char tiff_signature_big[4];
    jpeg1_signature[0] = 0xff;
    jpeg1_signature[1] = 0xd8;
    jpeg2_signature[0] = 0xff;
    jpeg2_signature[1] = 0xd9;
    jpeg3_signature[0] = 0xff;
    jpeg3_signature[1] = 0xd8;
    jpeg3_signature[2] = 0xff;
    jpeg3_signature[3] = 0xe0;
    jfif_signature[0] = 0x4a;
    jfif_signature[1] = 0x46;
    jfif_signature[2] = 0x49;
    jfif_signature[3] = 0x46;
    exif_signature[0] = 0x45;
    exif_signature[1] = 0x78;
    exif_signature[2] = 0x69;
    exif_signature[3] = 0x66;
    png_signature[0] = 0x89;
    png_signature[1] = 0x50;
    png_signature[2] = 0x4e;
    png_signature[3] = 0x47;
    png_signature[4] = 0x0d;
    png_signature[5] = 0x0a;
    png_signature[6] = 0x1a;
    png_signature[7] = 0x0a;
    zip_signature[0] = 0x50;
    zip_signature[1] = 0x4b;
    zip_signature[2] = 0x03;
    zip_signature[3] = 0x04;
    tiff_signature_little[0] = 'I';
    tiff_signature_little[1] = 'I';
    tiff_signature_little[2] = 0x2a;
    tiff_signature_little[3] = 0x00;
    tiff_signature_big[0] = 'M';
    tiff_signature_big[1] = 'M';
    tiff_signature_big[2] = 0x00;
    tiff_signature_big[3] = 0x2a;
    if (size < 1 || !blob)
	return GAIA_HEX_BLOB;
    if (size > 4)
      {
	  if (memcmp (blob, tiff_signature_big, 4) == 0)
	      return GAIA_TIFF_BLOB;
	  if (memcmp (blob, tiff_signature_little, 4) == 0)
	      return GAIA_TIFF_BLOB;
      }
    if (size > 5)
      {
	  if (strncmp ((char *) blob, "%PDF-", 5) == 0)
	      return GAIA_PDF_BLOB;
      }
    if (size > 4)
      {
	  if (memcmp (blob, zip_signature, 4) == 0)
	      return GAIA_ZIP_BLOB;
      }
    if (size > 6)
      {
	  if (strncmp ((char *) blob, "GIF87a", 6) == 0
	      || strncmp ((char *) blob, "GIF89a", 6) == 0)
	      return GAIA_GIF_BLOB;
      }
    if (size > 8)
      {
	  if (memcmp (blob, png_signature, 8) == 0)
	      return GAIA_PNG_BLOB;
      }
    if (size > 4)
      {
	  if (memcmp (blob, jpeg1_signature, 2) == 0
	      && memcmp (blob + size - 2, jpeg2_signature, 2) == 0)
	      jpeg = 1;		/* this one is the standard JPEG signature */
	  if (memcmp (blob, jpeg3_signature, 4) == 0)
	      jpeg = 1;		/* another common JPEG signature */
      }
    if (size > 10)
      {
	  if (memcmp (blob + 6, jfif_signature, 4) == 0)
	      jpeg = 1;		/* standard JFIF signature */
	  if (memcmp (blob + 6, exif_signature, 4) == 0)
	      jpeg = 1;		/* standard EXIF signature */
      }
    if (jpeg)
      {
	  exif_list = gaiaGetExifTags (blob, size);
	  if (exif_list)
	    {
		exif = 1;
		pT = exif_list->First;
		while (pT)
		  {
		      if (pT->Gps)
			{
			    exif_gps = 1;
			    break;
			}
		      pT = pT->Next;
		  }
		gaiaExifTagsFree (exif_list);
	    }
      }
    if (jpeg && exif && exif_gps)
	return GAIA_EXIF_GPS_BLOB;
    if (jpeg && exif)
	return GAIA_EXIF_BLOB;
    if (jpeg)
	return GAIA_JPEG_BLOB;
/* testing for GEOMETRY */
    if (size < 45)
	geom = 0;
    else
      {
	  if (*(blob + 0) != GAIA_MARK_START)
	      geom = 0;
	  if (*(blob + (size - 1)) != GAIA_MARK_END)
	      geom = 0;
	  if (*(blob + 38) != GAIA_MARK_MBR)
	      geom = 0;
	  if (*(blob + 1) == 0 || *(blob + 1) == 1)
	      ;
	  else
	      geom = 0;
      }
    if (geom)
	return GAIA_GEOMETRY_BLOB;
    return GAIA_HEX_BLOB;
}

GAIAEXIF_DECLARE int
gaiaGetGpsCoords (const unsigned char *blob, int size, double *longitude,
		  double *latitude)
{
/* returns the ExifGps coords, if they exists */
    gaiaExifTagListPtr exif_list;
    gaiaExifTagPtr pT;
    char lat_ref = '\0';
    char long_ref = '\0';
    double lat_degs = -DBL_MAX;
    double lat_mins = -DBL_MAX;
    double lat_secs = -DBL_MAX;
    double long_degs = -DBL_MAX;
    double long_mins = -DBL_MAX;
    double long_secs = -DBL_MAX;
    double dblval;
    double sign;
    int ok;
    if (size < 1 || !blob)
	return 0;
    exif_list = gaiaGetExifTags (blob, size);
    if (exif_list)
      {
	  pT = exif_list->First;
	  while (pT)
	    {
		if (pT->Gps && pT->TagId == 0x01)
		  {
		      /* ok, this one is the GPSLatitudeRef tag */
		      if (pT->Type == 2)
			  lat_ref = *(pT->StringValue);
		  }
		if (pT->Gps && pT->TagId == 0x03)
		  {
		      /* ok, this one is the GPSLongitudeRef tag */
		      if (pT->Type == 2)
			  long_ref = *(pT->StringValue);
		  }
		if (pT->Gps && pT->TagId == 0x02)
		  {
		      /* ok, this one is the GPSLatitude tag */
		      if (pT->Type == 5 && pT->Count == 3)
			{
			    dblval = gaiaExifTagGetRationalValue (pT, 0, &ok);
			    if (ok)
				lat_degs = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 1, &ok);
			    if (ok)
				lat_mins = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 2, &ok);
			    if (ok)
				lat_secs = dblval;
			}
		  }
		if (pT->Gps && pT->TagId == 0x04)
		  {
		      /* ok, this one is the GPSLongitude tag */
		      if (pT->Type == 5 && pT->Count == 3)
			{
			    dblval = gaiaExifTagGetRationalValue (pT, 0, &ok);
			    if (ok)
				long_degs = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 1, &ok);
			    if (ok)
				long_mins = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 2, &ok);
			    if (ok)
				long_secs = dblval;
			}
		  }
		pT = pT->Next;
	    }
	  gaiaExifTagsFree (exif_list);
	  if ((lat_ref == 'N' || lat_ref == 'S' || long_ref == 'E'
	       || long_ref == 'W') && lat_degs != -DBL_MAX
	      && lat_mins != -DBL_MAX && lat_secs != -DBL_MAX
	      && long_degs != -DBL_MAX && long_mins != -DBL_MAX
	      && long_secs != -DBL_MAX)
	    {
		if (lat_ref == 'S')
		    sign = -1.0;
		else
		    sign = 1.0;
		lat_degs = math_round (lat_degs * 1000000.0);
		lat_mins = math_round (lat_mins * 1000000.0);
		lat_secs = math_round (lat_secs * 1000000.0);
		dblval =
		    math_round (lat_degs + (lat_mins / 60.0) +
				(lat_secs / 3600.0)) * (sign / 1000000.0);
		*latitude = dblval;
		if (long_ref == 'W')
		    sign = -1.0;
		else
		    sign = 1.0;
		long_degs = math_round (long_degs * 1000000.0);
		long_mins = math_round (long_mins * 1000000.0);
		long_secs = math_round (long_secs * 1000000.0);
		dblval =
		    math_round (long_degs + (long_mins / 60.0) +
				(long_secs / 3600.0)) * (sign / 1000000.0);
		*longitude = dblval;
		return 1;
	    }
      }
    return 0;
}

GAIAEXIF_DECLARE int
gaiaGetGpsLatLong (const unsigned char *blob, int size, char *latlong,
		   int ll_size)
{
/* returns the ExifGps Latitude and Longitude, if they exists */
    gaiaExifTagListPtr exif_list;
    gaiaExifTagPtr pT;
    char lat_ref = '\0';
    char long_ref = '\0';
    double lat_degs = -DBL_MAX;
    double lat_mins = -DBL_MAX;
    double lat_secs = -DBL_MAX;
    double long_degs = -DBL_MAX;
    double long_mins = -DBL_MAX;
    double long_secs = -DBL_MAX;
    double dblval;
    int ok;
    char ll[1024];
    int len;
    *latlong = '\0';
    if (size < 1 || !blob)
	return 0;
    exif_list = gaiaGetExifTags (blob, size);
    if (exif_list)
      {
	  pT = exif_list->First;
	  while (pT)
	    {
		if (pT->Gps && pT->TagId == 0x01)
		  {
		      /* ok, this one is the GPSLatitudeRef tag */
		      if (pT->Type == 2)
			  lat_ref = *(pT->StringValue);
		  }
		if (pT->Gps && pT->TagId == 0x03)
		  {
		      /* ok, this one is the GPSLongitudeRef tag */
		      if (pT->Type == 2)
			  long_ref = *(pT->StringValue);
		  }
		if (pT->Gps && pT->TagId == 0x02)
		  {
		      /* ok, this one is the GPSLatitude tag */
		      if (pT->Type == 5 && pT->Count == 3)
			{
			    dblval = gaiaExifTagGetRationalValue (pT, 0, &ok);
			    if (ok)
				lat_degs = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 1, &ok);
			    if (ok)
				lat_mins = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 2, &ok);
			    if (ok)
				lat_secs = dblval;
			}
		  }
		if (pT->Gps && pT->TagId == 0x04)
		  {
		      /* ok, this one is the GPSLongitude tag */
		      if (pT->Type == 5 && pT->Count == 3)
			{
			    dblval = gaiaExifTagGetRationalValue (pT, 0, &ok);
			    if (ok)
				long_degs = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 1, &ok);
			    if (ok)
				long_mins = dblval;
			    dblval = gaiaExifTagGetRationalValue (pT, 2, &ok);
			    if (ok)
				long_secs = dblval;
			}
		  }
		pT = pT->Next;
	    }
	  gaiaExifTagsFree (exif_list);
	  if ((lat_ref == 'N' || lat_ref == 'S' || long_ref == 'E'
	       || long_ref == 'W') && lat_degs != -DBL_MAX
	      && lat_mins != -DBL_MAX && lat_secs != -DBL_MAX
	      && long_degs != -DBL_MAX && long_mins != -DBL_MAX
	      && long_secs != -DBL_MAX)
	    {
		sprintf (ll,
			 "%c %1.2f %1.2f %1.2f / %c %1.2f %1.2f %1.2f",
			 lat_ref, lat_degs, lat_mins, lat_secs, long_ref,
			 long_degs, long_mins, long_secs);
		len = strlen (ll);
		if (len < ll_size)
		    strcpy (latlong, ll);
		else
		  {
		      memcpy (latlong, ll, ll_size - 1);
		      latlong[ll_size] = '\0';
		  }
		return 1;
	    }
      }
    return 0;
}
/**************** End file: gaia_exif.c **********/


/**************** Begin file: gg_advanced.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <math.h> */
/* #include <float.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

GAIAGEO_DECLARE double
gaiaMeasureLength (int dims, double *coords, int vert)
{
/* computes the total length */
    double lung = 0.0;
    double xx1;
    double xx2;
    double yy1;
    double yy2;
    double x;
    double y;
    double z;
    double m;
    double dist;
    int ind;
    if (vert <= 0)
	return lung;
    if (dims == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (coords, 0, &xx1, &yy1, &z);
      }
    else if (dims == GAIA_XY_M)
      {
	  gaiaGetPointXYM (coords, 0, &xx1, &yy1, &m);
      }
    else if (dims == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (coords, 0, &xx1, &yy1, &z, &m);
      }
    else
      {
	  gaiaGetPoint (coords, 0, &xx1, &yy1);
      }
    for (ind = 1; ind < vert; ind++)
      {
	  if (dims == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (coords, ind, &xx2, &yy2, &z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaGetPointXYM (coords, ind, &xx2, &yy2, &m);
	    }
	  else if (dims == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (coords, ind, &xx2, &yy2, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (coords, ind, &xx2, &yy2);
	    }
	  x = xx1 - xx2;
	  y = yy1 - yy2;
	  dist = sqrt ((x * x) + (y * y));
	  lung += dist;
	  xx1 = xx2;
	  yy1 = yy2;
      }
    return lung;
}

GAIAGEO_DECLARE double
gaiaMeasureArea (gaiaRingPtr ring)
{
/* computes the area */
    int iv;
    double xx;
    double yy;
    double x;
    double y;
    double z;
    double m;
    double area = 0.0;
    if (!ring)
	return 0.0;
    if (ring->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (ring->Coords, 0, &xx, &yy, &z);
      }
    else if (ring->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (ring->Coords, 0, &xx, &yy, &m);
      }
    else if (ring->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (ring->Coords, 0, &xx, &yy, &z, &m);
      }
    else
      {
	  gaiaGetPoint (ring->Coords, 0, &xx, &yy);
      }
    for (iv = 1; iv < ring->Points; iv++)
      {
	  if (ring->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
	    }
	  else if (ring->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
	    }
	  else if (ring->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (ring->Coords, iv, &x, &y);
	    }
	  area += ((xx * y) - (x * yy));
	  xx = x;
	  yy = y;
      }
    area /= 2.0;
    return fabs (area);
}

GAIAGEO_DECLARE void
gaiaRingCentroid (gaiaRingPtr ring, double *rx, double *ry)
{
/* computes the simple ring centroid */
    double cx = 0.0;
    double cy = 0.0;
    double xx;
    double yy;
    double x;
    double y;
    double z;
    double m;
    double coeff;
    double area;
    double term;
    int iv;
    if (!ring)
      {
	  *rx = -DBL_MAX;
	  *ry = -DBL_MAX;
	  return;
      }
    area = gaiaMeasureArea (ring);
    coeff = 1.0 / (area * 6.0);
    if (ring->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (ring->Coords, 0, &xx, &yy, &z);
      }
    else if (ring->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (ring->Coords, 0, &xx, &yy, &m);
      }
    else if (ring->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (ring->Coords, 0, &xx, &yy, &z, &m);
      }
    else
      {
	  gaiaGetPoint (ring->Coords, 0, &xx, &yy);
      }
    for (iv = 1; iv < ring->Points; iv++)
      {
	  if (ring->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
	    }
	  else if (ring->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
	    }
	  else if (ring->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (ring->Coords, iv, &x, &y);
	    }
	  term = (xx * y) - (x * yy);
	  cx += (xx + x) * term;
	  cy += (yy + y) * term;
	  xx = x;
	  yy = y;
      }
    *rx = fabs (cx * coeff);
    *ry = fabs (cy * coeff);
}

GAIAGEO_DECLARE void
gaiaClockwise (gaiaRingPtr p)
{
/* determines clockwise or anticlockwise direction */
    int ind;
    int ix;
    double xx;
    double yy;
    double x;
    double y;
    double z;
    double m;
    double area = 0.0;
    for (ind = 0; ind < p->Points; ind++)
      {
	  if (p->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (p->Coords, ind, &xx, &yy, &z);
	    }
	  else if (p->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (p->Coords, ind, &xx, &yy, &m);
	    }
	  else if (p->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (p->Coords, ind, &xx, &yy, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (p->Coords, ind, &xx, &yy);
	    }
	  ix = (ind + 1) % p->Points;
	  if (p->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (p->Coords, ix, &x, &y, &z);
	    }
	  else if (p->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (p->Coords, ix, &x, &y, &m);
	    }
	  else if (p->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (p->Coords, ix, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (p->Coords, ix, &x, &y);
	    }
	  area += ((xx * y) - (x * yy));
      }
    area /= 2.0;
    if (area >= 0.0)
	p->Clockwise = 0;
    else
	p->Clockwise = 1;
}

GAIAGEO_DECLARE int
gaiaIsPointOnRingSurface (gaiaRingPtr ring, double pt_x, double pt_y)
{
/* tests if a POINT falls inside a RING */
    int isInternal = 0;
    int cnt;
    int i;
    int j;
    double x;
    double y;
    double z;
    double m;
    double *vert_x;
    double *vert_y;
    double minx = DBL_MAX;
    double miny = DBL_MAX;
    double maxx = -DBL_MAX;
    double maxy = -DBL_MAX;
    cnt = ring->Points;
    cnt--;			/* ignoring last vertex because surely identical to the first one */
    if (cnt < 2)
	return 0;
/* allocating and loading an array of vertices */
    vert_x = malloc (sizeof (double) * (cnt));
    vert_y = malloc (sizeof (double) * (cnt));
    for (i = 0; i < cnt; i++)
      {
	  if (ring->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (ring->Coords, i, &x, &y, &z);
	    }
	  else if (ring->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (ring->Coords, i, &x, &y, &m);
	    }
	  else if (ring->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (ring->Coords, i, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (ring->Coords, i, &x, &y);
	    }
	  vert_x[i] = x;
	  vert_y[i] = y;
	  if (x < minx)
	      minx = x;
	  if (x > maxx)
	      maxx = x;
	  if (y < miny)
	      miny = y;
	  if (y > maxy)
	      maxy = y;
      }
    if (pt_x < minx || pt_x > maxx)
	goto end;		/* outside the bounding box (x axis) */
    if (pt_y < miny || pt_y > maxy)
	goto end;		/* outside the bounding box (y axis) */
    for (i = 0, j = cnt - 1; i < cnt; j = i++)
      {
/* The definitive reference is "Point in Polyon Strategies" by
/  Eric Haines [Gems IV]  pp. 24-46.
/  The code in the Sedgewick book Algorithms (2nd Edition, p.354) is 
/  incorrect.
*/
	  if ((((vert_y[i] <= pt_y) && (pt_y < vert_y[j]))
	       || ((vert_y[j] <= pt_y) && (pt_y < vert_y[i])))
	      && (pt_x <
		  (vert_x[j] - vert_x[i]) * (pt_y - vert_y[i]) / (vert_y[j] -
								  vert_y[i]) +
		  vert_x[i]))
	      isInternal = !isInternal;
      }
  end:
    free (vert_x);
    free (vert_y);
    return isInternal;
}

GAIAGEO_DECLARE double
gaiaMinDistance (double x0, double y0, int dims, double *coords, int n_vert)
{
/* computing minimal distance between a POINT and a linestring/ring */
    double x;
    double y;
    double z;
    double m;
    double ox;
    double oy;
    double lineMag;
    double u;
    double px;
    double py;
    double dist;
    double min_dist = DBL_MAX;
    int iv;
    if (n_vert < 2)
	return min_dist;	/* not a valid linestring */
/* computing distance from first vertex */
    ox = *(coords + 0);
    oy = *(coords + 1);
    min_dist = sqrt (((x0 - ox) * (x0 - ox)) + ((y0 - oy) * (y0 - oy)));
    for (iv = 1; iv < n_vert; iv++)
      {
	  /* segment start-end coordinates */
	  if (dims == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (coords, iv - 1, &ox, &oy, &z);
		gaiaGetPointXYZ (coords, iv, &x, &y, &z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaGetPointXYM (coords, iv - 1, &ox, &oy, &m);
		gaiaGetPointXYM (coords, iv, &x, &y, &m);
	    }
	  else if (dims == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (coords, iv - 1, &ox, &oy, &z, &m);
		gaiaGetPointXYZM (coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (coords, iv - 1, &ox, &oy);
		gaiaGetPoint (coords, iv, &x, &y);
	    }
	  /* computing distance from vertex */
	  dist = sqrt (((x0 - x) * (x0 - x)) + ((y0 - y) * (y0 - y)));
	  if (dist < min_dist)
	      min_dist = dist;
	  /* computing a projection */
	  lineMag = ((x - ox) * (x - ox)) + ((y - oy) * (y - oy));
	  u = (((x0 - ox) * (x - ox)) + ((y0 - oy) * (y - oy))) / lineMag;
	  if (u < 0.0 || u > 1.0)
	      ;			/* closest point does not fall within the line segment */
	  else
	    {
		px = ox + u * (x - ox);
		py = oy + u * (y - oy);
		dist = sqrt (((x0 - px) * (x0 - px)) + ((y0 - py) * (y0 - py)));
		if (dist < min_dist)
		    min_dist = dist;
	    }
      }
    return min_dist;
}

GAIAGEO_DECLARE int
gaiaIsPointOnPolygonSurface (gaiaPolygonPtr polyg, double x, double y)
{
/* tests if a POINT falls inside a POLYGON */
    int ib;
    gaiaRingPtr ring = polyg->Exterior;
    if (gaiaIsPointOnRingSurface (ring, x, y))
      {
	  /* ok, the POINT falls inside the polygon */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		ring = polyg->Interiors + ib;
		if (gaiaIsPointOnRingSurface (ring, x, y))
		  {
		      /* no, the POINT fall inside some hole */
		      return 0;
		  }
	    }
	  return 1;
      }
    return 0;
}

GAIAGEO_DECLARE int
gaiaIntersect (double *x0, double *y0, double x1, double y1, double x2,
	       double y2, double x3, double y3, double x4, double y4)
{
/* computes intersection [if any] between two line segments
/  the intersection POINT has coordinates (x0, y0) 
/  first line is identified by(x1, y1)  and (x2, y2)
/  second line is identified by (x3, y3) and (x4, y4)
*/
    double x;
    double y;
    double a1;
    double b1;
    double c1;
    double a2;
    double b2;
    double c2;
    double m1;
    double m2;
    double p;
    double det_inv;
    double minx1;
    double miny1;
    double maxx1;
    double maxy1;
    double minx2;
    double miny2;
    double maxx2;
    double maxy2;
    int ok1 = 0;
    int ok2 = 0;
/* building line segment's MBRs */
    if (x2 < x1)
      {
	  minx1 = x2;
	  maxx1 = x1;
      }
    else
      {
	  minx1 = x1;
	  maxx1 = x2;
      }
    if (y2 < y1)
      {
	  miny1 = y2;
	  maxy1 = y1;
      }
    else
      {
	  miny1 = y1;
	  maxy1 = y2;
      }
    if (x4 < x3)
      {
	  minx2 = x4;
	  maxx2 = x3;
      }
    else
      {
	  minx2 = x3;
	  maxx2 = x4;
      }
    if (y4 < y3)
      {
	  miny2 = y4;
	  maxy2 = y3;
      }
    else
      {
	  miny2 = y3;
	  maxy2 = y4;
      }
/* checkinkg MBRs first */
    if (minx1 >= maxx2)
	return 0;
    if (miny1 >= maxy2)
	return 0;
    if (maxx1 <= minx2)
	return 0;
    if (maxy1 <= miny2)
	return 0;
    if (minx2 >= maxx1)
	return 0;
    if (miny2 >= maxy1)
	return 0;
    if (maxx2 <= minx1)
	return 0;
    if (maxy2 <= miny1)
	return 0;
/* there is an MBRs intersection - proceeding */
    if ((x2 - x1) != 0.0)
	m1 = (y2 - y1) / (x2 - x1);
    else
	m1 = DBL_MAX;
    if ((x4 - x3) != 0)
	m2 = (y4 - y3) / (x4 - x3);
    else
	m2 = DBL_MAX;
    if (m1 == m2)		/* parallel lines */
	return 0;
    if (m1 == DBL_MAX)
	c1 = y1;
    else
	c1 = (y1 - m1 * x1);
    if (m2 == DBL_MAX)
	c2 = y3;
    else
	c2 = (y3 - m2 * x3);
    if (m1 == DBL_MAX)
      {
	  x = x1;
	  p = m2 * x1;
	  y = p + c2;		/*  first line is vertical */
	  goto check_bbox;
      }
    if (m2 == DBL_MAX)
      {
	  x = x3;
	  p = m1 * x3;
	  y = p + c1;		/* second line is vertical */
	  goto check_bbox;
      }
    a1 = m1;
    a2 = m2;
    b1 = -1;
    b2 = -1;
    det_inv = 1 / (a1 * b2 - a2 * b1);
    x = ((b1 * c2 - b2 * c1) * det_inv);
    y = ((a2 * c1 - a1 * c2) * det_inv);
/* now checking if intersection falls within both segment boundaries */
  check_bbox:
    if (x >= minx1 && x <= maxx1 && y >= miny1 && y <= maxy1)
	ok1 = 1;
    if (x >= minx2 && x <= maxx2 && y >= miny2 && y <= maxy2)
	ok2 = 1;
    if (ok1 && ok2)
      {
	  /* intersection point falls within the segments */
	  *x0 = x;
	  *y0 = y;
	  return 1;
      }
    return 0;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaSanitize (gaiaGeomCollPtr geom)
{
/* 
/ sanitizes a GEOMETRYCOLLECTION:
/ - repeated vertices are omitted
/ - ring closure is enforced anyway  
*/
    int iv;
    int ib;
    double x = 0.0;
    double y = 0.0;
    double z = 0.0;
    double m = 0.0;
    double last_x = 0.0;
    double last_y = 0.0;
    double last_z = 0.0;
    int points;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaLinestringPtr new_line;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr new_polyg;
    gaiaGeomCollPtr new_geom;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!geom)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	new_geom = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	new_geom = gaiaAllocGeomCollXYM ();
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	new_geom = gaiaAllocGeomCollXYZM ();
    else
	new_geom = gaiaAllocGeomColl ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = geom->DeclaredType;
    point = geom->FirstPoint;
    while (point)
      {
	  /* copying POINTs */
	  gaiaAddPointToGeomCollXYZM (new_geom, point->X, point->Y, point->Z,
				      point->M);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* sanitizing LINESTRINGs */
	  points = 0;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		/* PASS I - checking points */
		z = 0.0;
		m = 0.0;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		if (iv > 0)
		  {
		      if (last_x == x && last_y == y && last_z == z)
			  ;
		      else
			  points++;
		  }
		else
		    points++;
		last_x = x;
		last_y = y;
		last_z = z;
	    }
	  if (points < 2)
	    {
		/* illegal LINESTRING - copying the original one */
		new_line = gaiaAddLinestringToGeomColl (new_geom, line->Points);
		gaiaCopyLinestringCoords (new_line, line);
	    }
	  else
	    {
		/* valid LINESTRING - sanitizing */
		new_line = gaiaAddLinestringToGeomColl (new_geom, points);
		points = 0;
		for (iv = 0; iv < line->Points; iv++)
		  {
		      /* PASS II - inserting points */
		      z = 0.0;
		      m = 0.0;
		      if (line->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
			}
		      else if (line->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
			}
		      else if (line->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (line->Coords, iv, &x, &y);
			}
		      if (iv > 0)
			{
			    if (last_x == x && last_y == y && last_z == z)
				;
			    else
			      {
				  if (new_line->DimensionModel == GAIA_XY_Z)
				    {
					gaiaSetPointXYZ (new_line->Coords,
							 points, x, y, z);
				    }
				  else if (new_line->DimensionModel ==
					   GAIA_XY_M)
				    {
					gaiaSetPointXYM (new_line->Coords,
							 points, x, y, m);
				    }
				  else if (new_line->DimensionModel ==
					   GAIA_XY_Z_M)
				    {
					gaiaSetPointXYZM (new_line->Coords,
							  points, x, y, z, m);
				    }
				  else
				    {
					gaiaSetPoint (new_line->Coords, points,
						      x, y);
				    }
				  points++;
			      }
			}
		      else
			{
			    if (new_line->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaSetPointXYZ (new_line->Coords, points, x,
						   y, z);
			      }
			    else if (new_line->DimensionModel == GAIA_XY_M)
			      {
				  gaiaSetPointXYM (new_line->Coords, points, x,
						   y, m);
			      }
			    else if (new_line->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaSetPointXYZM (new_line->Coords, points, x,
						    y, z, m);
			      }
			    else
			      {
				  gaiaSetPoint (new_line->Coords, points, x, y);
			      }
			    points++;
			}
		      last_x = x;
		      last_y = y;
		      last_z = z;
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* copying POLYGONs */
	  i_ring = polyg->Exterior;
	  /* sanitizing EXTERIOR RING */
	  points = 0;
	  for (iv = 0; iv < i_ring->Points; iv++)
	    {
		/* PASS I - checking points */
		z = 0.0;
		m = 0.0;
		if (i_ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (i_ring->Coords, iv, &x, &y, &z);
		  }
		else if (i_ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (i_ring->Coords, iv, &x, &y, &m);
		  }
		else if (i_ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (i_ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (i_ring->Coords, iv, &x, &y);
		  }
		if (iv > 0)
		  {
		      if (last_x == x && last_y == y && last_z == z)
			  ;
		      else
			  points++;
		  }
		else
		      points++;
		last_x = x;
		last_y = y;
		last_z = z;
	    }
	  if (last_x == x && last_y == y && last_z == z)
	      ;
	  else
	    {
		/* forcing RING closure */
		points++;
	    }
	  if (points < 4)
	    {
		/* illegal RING - copying the original one */
		new_polyg =
		    gaiaAddPolygonToGeomColl (new_geom, i_ring->Points,
					      polyg->NumInteriors);
		o_ring = new_polyg->Exterior;
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  else
	    {
		/* valid RING - sanitizing */
		new_polyg =
		    gaiaAddPolygonToGeomColl (new_geom, points,
					      polyg->NumInteriors);
		o_ring = new_polyg->Exterior;
		points = 0;
		for (iv = 0; iv < i_ring->Points; iv++)
		  {
		      /* PASS II - inserting points */
		      z = 0.0;
		      m = 0.0;
		      if (i_ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (i_ring->Coords, iv, &x, &y, &z);
			}
		      else if (i_ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (i_ring->Coords, iv, &x, &y, &m);
			}
		      else if (i_ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (i_ring->Coords, iv, &x, &y, &z,
					      &m);
			}
		      else
			{
			    gaiaGetPoint (i_ring->Coords, iv, &x, &y);
			}
		      if (iv > 0)
			{
			    if (last_x == x && last_y == y && last_z == z)
				;
			    else
			      {
				  if (o_ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaSetPointXYZ (o_ring->Coords, points,
							 x, y, z);
				    }
				  else if (o_ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaSetPointXYM (o_ring->Coords, points,
							 x, y, m);
				    }
				  else if (o_ring->DimensionModel ==
					   GAIA_XY_Z_M)
				    {
					gaiaSetPointXYZM (o_ring->Coords,
							  points, x, y, z, m);
				    }
				  else
				    {
					gaiaSetPoint (o_ring->Coords, points, x,
						      y);
				    }
				  points++;
			      }
			}
		      else
			{
			    if (o_ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaSetPointXYZ (o_ring->Coords, points, x,
						   y, z);
			      }
			    else if (o_ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaSetPointXYM (o_ring->Coords, points, x,
						   y, m);
			      }
			    else if (o_ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaSetPointXYZM (o_ring->Coords, points, x,
						    y, z, m);
			      }
			    else
			      {
				  gaiaSetPoint (o_ring->Coords, points, x, y);
			      }
			    points++;
			}
		      last_x = x;
		      last_y = y;
		      last_z = z;
		  }
	    }
	  /* PASS III - forcing RING closure */
	  z = 0.0;
	  m = 0.0;
	  if (i_ring->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (i_ring->Coords, 0, &x, &y, &z);
	    }
	  else if (i_ring->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (i_ring->Coords, 0, &x, &y, &m);
	    }
	  else if (i_ring->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (i_ring->Coords, 0, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (i_ring->Coords, 0, &x, &y);
	    }
	  points = o_ring->Points - 1;
	  if (o_ring->DimensionModel == GAIA_XY_Z)
	    {
		gaiaSetPointXYZ (o_ring->Coords, points, x, y, z);
	    }
	  else if (o_ring->DimensionModel == GAIA_XY_M)
	    {
		gaiaSetPointXYM (o_ring->Coords, points, x, y, m);
	    }
	  else if (o_ring->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaSetPointXYZM (o_ring->Coords, points, x, y, z, m);
	    }
	  else
	    {
		gaiaSetPoint (o_ring->Coords, points, x, y);
	    }
	  for (ib = 0; ib < new_polyg->NumInteriors; ib++)
	    {
		/* copying each INTERIOR RING [if any] */
		i_ring = polyg->Interiors + ib;
		/* sanitizing an INTERIOR RING */
		points = 0;
		for (iv = 0; iv < i_ring->Points; iv++)
		  {
		      /* PASS I - checking points */
		      z = 0.0;
		      m = 0.0;
		      if (i_ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (i_ring->Coords, iv, &x, &y, &z);
			}
		      else if (i_ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (i_ring->Coords, iv, &x, &y, &m);
			}
		      else if (i_ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (i_ring->Coords, iv, &x, &y, &z,
					      &m);
			}
		      else
			{
			    gaiaGetPoint (i_ring->Coords, iv, &x, &y);
			}
		      if (iv > 0)
			{
			    if (last_x == x && last_y == y && last_z == z)
				;
			    else
				points++;
			}
		      else
			    points++;
		      last_x = x;
		      last_y = y;
		      last_z = z;
		  }
		if (last_x == x && last_y == y && last_z == z)
		    ;
		else
		  {
		      /* forcing RING closure */
		      points++;
		  }
		if (points < 4)
		  {
		      /* illegal RING - copying the original one */
		      o_ring =
			  gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
		      gaiaCopyRingCoords (o_ring, i_ring);
		  }
		else
		  {
		      /* valid RING - sanitizing */
		      o_ring = gaiaAddInteriorRing (new_polyg, ib, points);
		      points = 0;
		      for (iv = 0; iv < i_ring->Points; iv++)
			{
			    /* PASS II - inserting points */
			    z = 0.0;
			    m = 0.0;
			    if (i_ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (i_ring->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (i_ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (i_ring->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (i_ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (i_ring->Coords, iv, &x, &y,
						    &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (i_ring->Coords, iv, &x, &y);
			      }
			    if (iv > 0)
			      {
				  if (last_x == x && last_y == y && last_z == z)
				      ;
				  else
				    {
					if (o_ring->DimensionModel == GAIA_XY_Z)
					  {
					      gaiaSetPointXYZ (o_ring->Coords,
							       points, x, y, z);
					  }
					else if (o_ring->DimensionModel ==
						 GAIA_XY_M)
					  {
					      gaiaSetPointXYM (o_ring->Coords,
							       points, x, y, m);
					  }
					else if (o_ring->DimensionModel ==
						 GAIA_XY_Z_M)
					  {
					      gaiaSetPointXYZM (o_ring->Coords,
								points, x, y, z,
								m);
					  }
					else
					  {
					      gaiaSetPoint (o_ring->Coords,
							    points, x, y);
					  }
					points++;
				    }
			      }
			    else
			      {
				  if (o_ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaSetPointXYZ (o_ring->Coords, points,
							 x, y, z);
				    }
				  else if (o_ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaSetPointXYM (o_ring->Coords, points,
							 x, y, m);
				    }
				  else if (o_ring->DimensionModel ==
					   GAIA_XY_Z_M)
				    {
					gaiaSetPointXYZM (o_ring->Coords,
							  points, x, y, z, m);
				    }
				  else
				    {
					gaiaSetPoint (o_ring->Coords, points, x,
						      y);
				    }
				  points++;
			      }
			    last_x = x;
			    last_y = y;
			    last_z = z;
			}
		      /* PASS III - forcing RING closure */
		      z = 0.0;
		      m = 0.0;
		      if (i_ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (i_ring->Coords, 0, &x, &y, &z);
			}
		      else if (i_ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (i_ring->Coords, 0, &x, &y, &m);
			}
		      else if (i_ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (i_ring->Coords, 0, &x, &y, &z,
					      &m);
			}
		      else
			{
			    gaiaGetPoint (i_ring->Coords, 0, &x, &y);
			}
		      points = o_ring->Points - 1;
		      if (o_ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (o_ring->Coords, points, x, y, z);
			}
		      else if (o_ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (o_ring->Coords, points, x, y, m);
			}
		      else if (o_ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (o_ring->Coords, points, x, y, z,
					      m);
			}
		      else
			{
			    gaiaSetPoint (o_ring->Coords, points, x, y);
			}
		  }
	    }
	  polyg = polyg->Next;
      }
    return new_geom;
}

static int
gaiaIsToxicRing (gaiaRingPtr ring)
{
/* checking a Rings */
    double x0;
    double y0;
    double z0;
    double m0;
    double x1;
    double y1;
    double z1;
    double m1;
    if (ring->Points < 4)
	return 1;
/* checking for unclosed rings */
    gaiaRingGetPoint (ring, 0, &x0, &y0, &z0, &m0);
    gaiaRingGetPoint (ring, ring->Points - 1, &x1, &y1, &z1, &m1);
    if (x0 == x1 && y0 == y1)
	;
    else
	return 1;
    return 0;
}

GAIAGEO_DECLARE int
gaiaIsToxic (gaiaGeomCollPtr geom)
{
/* 
/ identifying toxic geometries 
/ i.e. geoms making GEOS to crash !!!
*/
    int ib;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    if (!geom)
	return 0;
    point = geom->FirstPoint;
    while (point)
      {
	  /* checking POINTs */
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* checking LINESTRINGs */
	  if (line->Points < 2)
	      return 1;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* checking POLYGONs */
	  ring = polyg->Exterior;
	  if (gaiaIsToxicRing (ring))
	      return 1;
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		ring = polyg->Interiors + ib;
		if (gaiaIsToxicRing (ring))
		    return 1;
	    }
	  polyg = polyg->Next;
      }
    return 0;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaLinearize (gaiaGeomCollPtr geom, int force_multi)
{
/* attempts to rearrange a generic Geometry into a (multi)linestring */
    int pts = 0;
    int lns = 0;
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr new_ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int iv;
    int ib;
    double x;
    double y;
    double m;
    double z;

    if (!geom)
	return NULL;
    pt = geom->FirstPoint;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    if (pts || lns)
	return NULL;

    if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    result->Srid = geom->Srid;
    if (force_multi)
	result->DeclaredType = GAIA_MULTILINESTRING;

    pg = geom->FirstPolygon;
    while (pg)
      {
	  /* dissolving any POLYGON as simple LINESTRINGs (rings) */
	  rng = pg->Exterior;
	  new_ln = gaiaAddLinestringToGeomColl (result, rng->Points);
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		/* copying the EXTERIOR RING as LINESTRING */
		if (geom->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
		  }
		else if (geom->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
		  }
		else if (geom->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      gaiaSetPoint (new_ln->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		new_ln = gaiaAddLinestringToGeomColl (result, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      /* copying an INTERIOR RING as LINESTRING */
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			    gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
			}
		      else if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			    gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			    gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			    gaiaSetPoint (new_ln->Coords, iv, x, y);
			}
		  }
	    }
	  pg = pg->Next;
      }
    if (result->FirstLinestring == NULL)
      {
	  gaiaFreeGeomColl (result);
	  return NULL;
      }
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaDissolveSegments (gaiaGeomCollPtr geom)
{
/* attempts to dissolve a Geometry into segments */
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr segment;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    double x0;
    double y0;
    double z0;
    double m0;
    if (!geom)
	return NULL;

    if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    pt = geom->FirstPoint;
    while (pt)
      {
	  if (geom->DimensionModel == GAIA_XY_Z_M)
	      gaiaAddPointToGeomCollXYZM (result, pt->X, pt->Y, pt->Z, pt->M);
	  else if (geom->DimensionModel == GAIA_XY_Z)
	      gaiaAddPointToGeomCollXYZ (result, pt->X, pt->Y, pt->Z);
	  else if (geom->DimensionModel == GAIA_XY_M)
	      gaiaAddPointToGeomCollXYM (result, pt->X, pt->Y, pt->M);
	  else
	      gaiaAddPointToGeomColl (result, pt->X, pt->Y);
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		  }
		if (iv > 0)
		  {
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    if (x != x0 || y != y0 || z != z0 || m != m0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPointXYZM (segment->Coords, 0, x0, y0,
						    z0, m0);
				  gaiaSetPointXYZM (segment->Coords, 1, x, y, z,
						    m);
			      }
			}
		      else if (geom->DimensionModel == GAIA_XY_Z)
			{
			    if (x != x0 || y != y0 || z != z0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPointXYZ (segment->Coords, 0, x0, y0,
						   z0);
				  gaiaSetPointXYZ (segment->Coords, 1, x, y, z);
			      }
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    if (x != x0 || y != y0 || m != m0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPointXYM (segment->Coords, 0, x0, y0,
						   m0);
				  gaiaSetPointXYM (segment->Coords, 1, x, y, m);
			      }
			}
		      else
			{
			    if (x != x0 || y != y0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPoint (segment->Coords, 0, x0, y0);
				  gaiaSetPoint (segment->Coords, 1, x, y);
			      }
			}
		  }
		x0 = x;
		y0 = y;
		z0 = z;
		m0 = m;
	    }
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  rng = pg->Exterior;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		/* exterior Ring */
		if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		  }
		if (iv > 0)
		  {
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    if (x != x0 || y != y0 || z != z0 || m != m0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPointXYZM (segment->Coords, 0, x0, y0,
						    z0, m0);
				  gaiaSetPointXYZM (segment->Coords, 1, x, y, z,
						    m);
			      }
			}
		      else if (geom->DimensionModel == GAIA_XY_Z)
			{
			    if (x != x0 || y != y0 || z != z0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPointXYZ (segment->Coords, 0, x0, y0,
						   z0);
				  gaiaSetPointXYZ (segment->Coords, 1, x, y, z);
			      }
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    if (x != x0 || y != y0 || m != m0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPointXYM (segment->Coords, 0, x0, y0,
						   m0);
				  gaiaSetPointXYM (segment->Coords, 1, x, y, m);
			      }
			}
		      else
			{
			    if (x != x0 || y != y0)
			      {
				  segment =
				      gaiaAddLinestringToGeomColl (result, 2);
				  gaiaSetPoint (segment->Coords, 0, x0, y0);
				  gaiaSetPoint (segment->Coords, 1, x, y);
			      }
			}
		  }
		x0 = x;
		y0 = y;
		z0 = z;
		m0 = m;
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      /* interior Ring */
		      if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      if (iv > 0)
			{
			    if (geom->DimensionModel == GAIA_XY_Z_M)
			      {
				  if (x != x0 || y != y0 || z != z0 || m != m0)
				    {
					segment =
					    gaiaAddLinestringToGeomColl (result,
									 2);
					gaiaSetPointXYZM (segment->Coords, 0,
							  x0, y0, z0, m0);
					gaiaSetPointXYZM (segment->Coords, 1, x,
							  y, z, m);
				    }
			      }
			    else if (geom->DimensionModel == GAIA_XY_Z)
			      {
				  if (x != x0 || y != y0 || z != z0)
				    {
					segment =
					    gaiaAddLinestringToGeomColl (result,
									 2);
					gaiaSetPointXYZ (segment->Coords, 0, x0,
							 y0, z0);
					gaiaSetPointXYZ (segment->Coords, 1, x,
							 y, z);
				    }
			      }
			    else if (geom->DimensionModel == GAIA_XY_M)
			      {
				  if (x != x0 || y != y0 || m != m0)
				    {
					segment =
					    gaiaAddLinestringToGeomColl (result,
									 2);
					gaiaSetPointXYM (segment->Coords, 0, x0,
							 y0, m0);
					gaiaSetPointXYM (segment->Coords, 1, x,
							 y, m);
				    }
			      }
			    else
			      {
				  if (x != x0 || y != y0)
				    {
					segment =
					    gaiaAddLinestringToGeomColl (result,
									 2);
					gaiaSetPoint (segment->Coords, 0, x0,
						      y0);
					gaiaSetPoint (segment->Coords, 1, x, y);
				    }
			      }
			}
		      x0 = x;
		      y0 = y;
		      z0 = z;
		      m0 = m;
		  }
	    }
	  pg = pg->Next;
      }
    result->Srid = geom->Srid;
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaDissolvePoints (gaiaGeomCollPtr geom)
{
/* attempts to dissolve a Geometry into points */
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    if (!geom)
	return NULL;

    if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    pt = geom->FirstPoint;
    while (pt)
      {
	  if (geom->DimensionModel == GAIA_XY_Z_M)
	      gaiaAddPointToGeomCollXYZM (result, pt->X, pt->Y, pt->Z, pt->M);
	  else if (geom->DimensionModel == GAIA_XY_Z)
	      gaiaAddPointToGeomCollXYZ (result, pt->X, pt->Y, pt->Z);
	  else if (geom->DimensionModel == GAIA_XY_M)
	      gaiaAddPointToGeomCollXYM (result, pt->X, pt->Y, pt->M);
	  else
	      gaiaAddPointToGeomColl (result, pt->X, pt->Y);
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		  }
		if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaAddPointToGeomCollXYZM (result, x, y, z, m);
		else if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaAddPointToGeomCollXYZ (result, x, y, z);
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAddPointToGeomCollXYM (result, x, y, m);
		else
		    gaiaAddPointToGeomColl (result, x, y);
	    }
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  rng = pg->Exterior;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		/* exterior Ring */
		if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		  }
		if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaAddPointToGeomCollXYZM (result, x, y, z, m);
		else if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaAddPointToGeomCollXYZ (result, x, y, z);
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAddPointToGeomCollXYM (result, x, y, m);
		else
		    gaiaAddPointToGeomColl (result, x, y);
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      /* interior Ring */
		      if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			  gaiaAddPointToGeomCollXYZM (result, x, y, z, m);
		      else if (geom->DimensionModel == GAIA_XY_Z)
			  gaiaAddPointToGeomCollXYZ (result, x, y, z);
		      else if (geom->DimensionModel == GAIA_XY_M)
			  gaiaAddPointToGeomCollXYM (result, x, y, m);
		      else
			  gaiaAddPointToGeomColl (result, x, y);
		  }
	    }
	  pg = pg->Next;
      }
    result->Srid = geom->Srid;
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaExtractPointsFromGeomColl (gaiaGeomCollPtr geom)
{
/* extracts any POINT from a GeometryCollection */
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    int pts = 0;
    if (!geom)
	return NULL;

    pt = geom->FirstPoint;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    if (!pts)
	return NULL;

    if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    pt = geom->FirstPoint;
    while (pt)
      {
	  if (geom->DimensionModel == GAIA_XY_Z_M)
	      gaiaAddPointToGeomCollXYZM (result, pt->X, pt->Y, pt->Z, pt->M);
	  else if (geom->DimensionModel == GAIA_XY_Z)
	      gaiaAddPointToGeomCollXYZ (result, pt->X, pt->Y, pt->Z);
	  else if (geom->DimensionModel == GAIA_XY_M)
	      gaiaAddPointToGeomCollXYM (result, pt->X, pt->Y, pt->M);
	  else
	      gaiaAddPointToGeomColl (result, pt->X, pt->Y);
	  pt = pt->Next;
      }
    result->Srid = geom->Srid;
    result->DeclaredType = GAIA_MULTIPOINT;
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaExtractLinestringsFromGeomColl (gaiaGeomCollPtr geom)
{
/* extracts any LINESTRING from a GeometryCollection */
    gaiaGeomCollPtr result;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr new_ln;
    int lns = 0;
    int iv;
    double x;
    double y;
    double z;
    double m;
    if (!geom)
	return NULL;

    ln = geom->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    if (!lns)
	return NULL;

    if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    ln = geom->FirstLinestring;
    while (ln)
      {
	  new_ln = gaiaAddLinestringToGeomColl (result, ln->Points);
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      gaiaSetPoint (new_ln->Coords, iv, x, y);
		  }
	    }
	  ln = ln->Next;
      }
    result->Srid = geom->Srid;
    result->DeclaredType = GAIA_MULTILINESTRING;
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaExtractPolygonsFromGeomColl (gaiaGeomCollPtr geom)
{
/* extracts any POLYGON from a GeometryCollection */
    gaiaGeomCollPtr result;
    gaiaPolygonPtr pg;
    gaiaPolygonPtr new_pg;
    gaiaRingPtr rng;
    gaiaRingPtr new_rng;
    int pgs = 0;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    if (!geom)
	return NULL;

    pg = geom->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (!pgs)
	return NULL;

    if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    pg = geom->FirstPolygon;
    while (pg)
      {
	  rng = pg->Exterior;
	  new_pg =
	      gaiaAddPolygonToGeomColl (result, rng->Points, pg->NumInteriors);
	  new_rng = new_pg->Exterior;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      gaiaSetPointXYZM (new_rng->Coords, iv, x, y, z, m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      gaiaSetPointXYZ (new_rng->Coords, iv, x, y, z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      gaiaSetPointXYM (new_rng->Coords, iv, x, y, m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      gaiaSetPoint (new_rng->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		new_rng = gaiaAddInteriorRing (new_pg, ib, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			    gaiaSetPointXYZM (new_rng->Coords, iv, x, y, z, m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			    gaiaSetPointXYZ (new_rng->Coords, iv, x, y, z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			    gaiaSetPointXYM (new_rng->Coords, iv, x, y, m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			    gaiaSetPoint (new_rng->Coords, iv, x, y);
			}
		  }
	    }
	  pg = pg->Next;
      }
    result->Srid = geom->Srid;
    result->DeclaredType = GAIA_MULTIPOLYGON;
    return result;
}
/**************** End file: gg_advanced.c **********/


/**************** Begin file: gg_endian.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */

#ifdef SPL_AMALGAMATION	/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

GAIAGEO_DECLARE int
gaiaEndianArch ()
{
/* checking if target CPU is a little-endian one */
    union cvt
    {
	unsigned char byte[4];
	int int_value;
    } convert;
    convert.int_value = 1;
    if (convert.byte[0] == 0)
	return 0;
    return 1;
}

GAIAGEO_DECLARE short
gaiaImport16 (const unsigned char *p, int little_endian, int little_endian_arch)
{
/* fetches a 16bit int from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[2];
	short short_value;
    } convert;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 1);
		convert.byte[1] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 1);
		convert.byte[1] = *(p + 0);
	    }
      }
    return convert.short_value;
}

GAIAGEO_DECLARE int
gaiaImport32 (const unsigned char *p, int little_endian, int little_endian_arch)
{
/* fetches a 32bit int from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[4];
	int int_value;
    } convert;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
      }
    return convert.int_value;
}

GAIAGEO_DECLARE float
gaiaImportF32 (const unsigned char *p, int little_endian,
	       int little_endian_arch)
{
/* fetches a 32bit float from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[4];
	float flt_value;
    } convert;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 3);
		convert.byte[1] = *(p + 2);
		convert.byte[2] = *(p + 1);
		convert.byte[3] = *(p + 0);
	    }
      }
    return convert.flt_value;
}

GAIAGEO_DECLARE double
gaiaImport64 (const unsigned char *p, int little_endian, int little_endian_arch)
{
/* fetches a 64bit double from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[8];
	double double_value;
    } convert;
    if (little_endian_arch)
      {
/* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 7);
		convert.byte[1] = *(p + 6);
		convert.byte[2] = *(p + 5);
		convert.byte[3] = *(p + 4);
		convert.byte[4] = *(p + 3);
		convert.byte[5] = *(p + 2);
		convert.byte[6] = *(p + 1);
		convert.byte[7] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
		convert.byte[4] = *(p + 4);
		convert.byte[5] = *(p + 5);
		convert.byte[6] = *(p + 6);
		convert.byte[7] = *(p + 7);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
		convert.byte[4] = *(p + 4);
		convert.byte[5] = *(p + 5);
		convert.byte[6] = *(p + 6);
		convert.byte[7] = *(p + 7);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 7);
		convert.byte[1] = *(p + 6);
		convert.byte[2] = *(p + 5);
		convert.byte[3] = *(p + 4);
		convert.byte[4] = *(p + 3);
		convert.byte[5] = *(p + 2);
		convert.byte[6] = *(p + 1);
		convert.byte[7] = *(p + 0);
	    }
      }
    return convert.double_value;
}

GAIAGEO_DECLARE sqlite3_int64
gaiaImportI64 (const unsigned char *p, int little_endian,
	       int little_endian_arch)
{
/* fetches a 64bit INT from BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[8];
	sqlite3_int64 int64_value;
    } convert;
    if (little_endian_arch)
      {
/* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 7);
		convert.byte[1] = *(p + 6);
		convert.byte[2] = *(p + 5);
		convert.byte[3] = *(p + 4);
		convert.byte[4] = *(p + 3);
		convert.byte[5] = *(p + 2);
		convert.byte[6] = *(p + 1);
		convert.byte[7] = *(p + 0);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
		convert.byte[4] = *(p + 4);
		convert.byte[5] = *(p + 5);
		convert.byte[6] = *(p + 6);
		convert.byte[7] = *(p + 7);
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		convert.byte[0] = *(p + 0);
		convert.byte[1] = *(p + 1);
		convert.byte[2] = *(p + 2);
		convert.byte[3] = *(p + 3);
		convert.byte[4] = *(p + 4);
		convert.byte[5] = *(p + 5);
		convert.byte[6] = *(p + 6);
		convert.byte[7] = *(p + 7);
	    }
	  else
	    {
		/* Little Endian data */
		convert.byte[0] = *(p + 7);
		convert.byte[1] = *(p + 6);
		convert.byte[2] = *(p + 5);
		convert.byte[3] = *(p + 4);
		convert.byte[4] = *(p + 3);
		convert.byte[5] = *(p + 2);
		convert.byte[6] = *(p + 1);
		convert.byte[7] = *(p + 0);
	    }
      }
    return convert.int64_value;
}

GAIAGEO_DECLARE void
gaiaExport16 (unsigned char *p, short value, int little_endian,
	      int little_endian_arch)
{
/* stores a 16bit int into a BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[2];
	short short_value;
    } convert;
    convert.short_value = value;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 1) = convert.byte[1];
		*(p + 0) = convert.byte[0];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 1) = convert.byte[0];
		*(p + 0) = convert.byte[1];
	    }
      }
}

GAIAGEO_DECLARE void
gaiaExport32 (unsigned char *p, int value, int little_endian,
	      int little_endian_arch)
{
/* stores a 32bit int into a BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[4];
	int int_value;
    } convert;
    convert.int_value = value;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 3) = convert.byte[0];
		*(p + 2) = convert.byte[1];
		*(p + 1) = convert.byte[2];
		*(p + 0) = convert.byte[3];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 3) = convert.byte[0];
		*(p + 2) = convert.byte[1];
		*(p + 1) = convert.byte[2];
		*(p + 0) = convert.byte[3];
	    }
      }
}

GAIAGEO_DECLARE void
gaiaExportF32 (unsigned char *p, float value, int little_endian,
	       int little_endian_arch)
{
/* stores a 32bit float into a BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[4];
	float flt_value;
    } convert;
    convert.flt_value = value;
    if (little_endian_arch)
      {
	  /* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 3) = convert.byte[0];
		*(p + 2) = convert.byte[1];
		*(p + 1) = convert.byte[2];
		*(p + 0) = convert.byte[3];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 3) = convert.byte[0];
		*(p + 2) = convert.byte[1];
		*(p + 1) = convert.byte[2];
		*(p + 0) = convert.byte[3];
	    }
      }
}

GAIAGEO_DECLARE void
gaiaExport64 (unsigned char *p, double value, int little_endian,
	      int little_endian_arch)
{
/* stores a 64bit double into a BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[8];
	double double_value;
    } convert;
    convert.double_value = value;
    if (little_endian_arch)
      {
/* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 7) = convert.byte[0];
		*(p + 6) = convert.byte[1];
		*(p + 5) = convert.byte[2];
		*(p + 4) = convert.byte[3];
		*(p + 3) = convert.byte[4];
		*(p + 2) = convert.byte[5];
		*(p + 1) = convert.byte[6];
		*(p + 0) = convert.byte[7];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
		*(p + 4) = convert.byte[4];
		*(p + 5) = convert.byte[5];
		*(p + 6) = convert.byte[6];
		*(p + 7) = convert.byte[7];
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
		*(p + 4) = convert.byte[4];
		*(p + 5) = convert.byte[5];
		*(p + 6) = convert.byte[6];
		*(p + 7) = convert.byte[7];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 7) = convert.byte[0];
		*(p + 6) = convert.byte[1];
		*(p + 5) = convert.byte[2];
		*(p + 4) = convert.byte[3];
		*(p + 3) = convert.byte[4];
		*(p + 2) = convert.byte[5];
		*(p + 1) = convert.byte[6];
		*(p + 0) = convert.byte[7];
	    }
      }
}

GAIAGEO_DECLARE void
gaiaExportI64 (unsigned char *p, sqlite3_int64 value, int little_endian,
	       int little_endian_arch)
{
/* stores a 64bit INT into a BLOB respecting declared endiannes */
    union cvt
    {
	unsigned char byte[8];
	sqlite3_int64 int64_value;
    } convert;
    convert.int64_value = value;
    if (little_endian_arch)
      {
/* Litte-Endian architecture [e.g. x86] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 7) = convert.byte[0];
		*(p + 6) = convert.byte[1];
		*(p + 5) = convert.byte[2];
		*(p + 4) = convert.byte[3];
		*(p + 3) = convert.byte[4];
		*(p + 2) = convert.byte[5];
		*(p + 1) = convert.byte[6];
		*(p + 0) = convert.byte[7];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
		*(p + 4) = convert.byte[4];
		*(p + 5) = convert.byte[5];
		*(p + 6) = convert.byte[6];
		*(p + 7) = convert.byte[7];
	    }
      }
    else
      {
	  /* Big Endian architecture [e.g. PPC] */
	  if (!little_endian)
	    {
		/* Big Endian data */
		*(p + 0) = convert.byte[0];
		*(p + 1) = convert.byte[1];
		*(p + 2) = convert.byte[2];
		*(p + 3) = convert.byte[3];
		*(p + 4) = convert.byte[4];
		*(p + 5) = convert.byte[5];
		*(p + 6) = convert.byte[6];
		*(p + 7) = convert.byte[7];
	    }
	  else
	    {
		/* Little Endian data */
		*(p + 7) = convert.byte[0];
		*(p + 6) = convert.byte[1];
		*(p + 5) = convert.byte[2];
		*(p + 4) = convert.byte[3];
		*(p + 3) = convert.byte[4];
		*(p + 2) = convert.byte[5];
		*(p + 1) = convert.byte[6];
		*(p + 0) = convert.byte[7];
	    }
      }
}
/**************** End file: gg_endian.c **********/


/**************** Begin file: gg_geometries.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <memory.h> */
/* #include <math.h> */
/* #include <float.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

GAIAGEO_DECLARE gaiaPointPtr
gaiaAllocPoint (double x, double y)
{
/* POINT object constructor */
    gaiaPointPtr p = malloc (sizeof (gaiaPoint));
    p->X = x;
    p->Y = y;
    p->Z = 0.0;
    p->M = 0.0;
    p->DimensionModel = GAIA_XY;
    p->Next = NULL;
    p->Prev = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaAllocPointXYZ (double x, double y, double z)
{
/* POINT object constructor */
    gaiaPointPtr p = malloc (sizeof (gaiaPoint));
    p->X = x;
    p->Y = y;
    p->Z = z;
    p->M = 0.0;
    p->DimensionModel = GAIA_XY_Z;
    p->Next = NULL;
    p->Prev = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaAllocPointXYM (double x, double y, double m)
{
/* POINT object constructor */
    gaiaPointPtr p = malloc (sizeof (gaiaPoint));
    p->X = x;
    p->Y = y;
    p->Z = 0.0;
    p->M = m;
    p->DimensionModel = GAIA_XY_M;
    p->Next = NULL;
    p->Prev = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaAllocPointXYZM (double x, double y, double z, double m)
{
/* POINT object constructor */
    gaiaPointPtr p = malloc (sizeof (gaiaPoint));
    p->X = x;
    p->Y = y;
    p->Z = z;
    p->M = m;
    p->DimensionModel = GAIA_XY_Z_M;
    p->Next = NULL;
    p->Prev = NULL;
    return p;
}

GAIAGEO_DECLARE void
gaiaFreePoint (gaiaPointPtr ptr)
{
/* POINT object destructor */
    if (ptr != NULL)
	free (ptr);
}

GAIAGEO_DECLARE gaiaLinestringPtr
gaiaAllocLinestring (int vert)
{
/* LINESTRING object constructor */
    gaiaLinestringPtr p = malloc (sizeof (gaiaLinestring));
    p->Coords = malloc (sizeof (double) * (vert * 2));
    p->Points = vert;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaLinestringPtr
gaiaAllocLinestringXYZ (int vert)
{
/* LINESTRING object constructor */
    gaiaLinestringPtr p = malloc (sizeof (gaiaLinestring));
    p->Coords = malloc (sizeof (double) * (vert * 3));
    p->Points = vert;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaLinestringPtr
gaiaAllocLinestringXYM (int vert)
{
/* LINESTRING object constructor */
    gaiaLinestringPtr p = malloc (sizeof (gaiaLinestring));
    p->Coords = malloc (sizeof (double) * (vert * 3));
    p->Points = vert;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_M;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaLinestringPtr
gaiaAllocLinestringXYZM (int vert)
{
/* LINESTRING object constructor */
    gaiaLinestringPtr p = malloc (sizeof (gaiaLinestring));
    p->Coords = malloc (sizeof (double) * (vert * 4));
    p->Points = vert;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z_M;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE void
gaiaFreeLinestring (gaiaLinestringPtr ptr)
{
/* LINESTRING object desctructror */
    if (ptr)
      {
	  if (ptr->Coords)
	      free (ptr->Coords);
	  free (ptr);
      }
}

GAIAGEO_DECLARE int
gaiaLineGetPoint (gaiaLinestringPtr ln, int v, double *x, double *y, double *z,
		  double *m)
{
/* SAFE - getting coords for a vertex in LINESTRING */
    double vx;
    double vy;
    double vz;
    double vm;
    *x = 0.0;
    *y = 0.0;
    *z = 0.0;
    *m = 0.0;
    if (!ln)
	return 0;
    if (v < 0 || v >= ln->Points)
	return 0;
    switch (ln->DimensionModel)
      {
      case GAIA_XY:
	  gaiaGetPoint (ln->Coords, v, &vx, &vy);
	  *x = vx;
	  *y = vy;
	  break;
      case GAIA_XY_Z:
	  gaiaGetPointXYZ (ln->Coords, v, &vx, &vy, &vz);
	  *x = vx;
	  *y = vy;
	  *z = vz;
	  break;
      case GAIA_XY_M:
	  gaiaGetPointXYM (ln->Coords, v, &vx, &vy, &vm);
	  *x = vx;
	  *y = vy;
	  *m = vm;
	  break;
      case GAIA_XY_Z_M:
	  gaiaGetPointXYZM (ln->Coords, v, &vx, &vy, &vz, &vm);
	  *x = vx;
	  *y = vy;
	  *z = vz;
	  *m = vm;
	  break;
      default:
	  return 0;
      };
    return 1;
}

GAIAGEO_DECLARE int
gaiaLineSetPoint (gaiaLinestringPtr ln, int v, double x, double y, double z,
		  double m)
{
/* SAFE - setting coords for a vertex in RING */
    if (!ln)
	return 0;
    if (v < 0 || v >= ln->Points)
	return 0;
    switch (ln->DimensionModel)
      {
      case GAIA_XY:
	  gaiaSetPoint (ln->Coords, v, x, y);
	  break;
      case GAIA_XY_Z:
	  gaiaSetPointXYZ (ln->Coords, v, x, y, z);
	  break;
      case GAIA_XY_M:
	  gaiaSetPointXYM (ln->Coords, v, x, y, m);
	  break;
      case GAIA_XY_Z_M:
	  gaiaSetPointXYZM (ln->Coords, v, x, y, z, m);
	  break;
      default:
	  return 0;
      };
    return 1;
}

GAIAGEO_DECLARE void
gaiaCopyLinestringCoords (gaiaLinestringPtr dst, gaiaLinestringPtr src)
{
/* 
/ copying coords from one Linestring to another
/ maybe, converting from one Dimension Model to a different one
*/
    int iv;
    double x;
    double y;
    double z;
    double m;
    if (!src)
	return;
    if (!dst)
	return;
    if (src->Points != dst->Points)
	return;
    for (iv = 0; iv < dst->Points; iv++)
      {
	  z = 0.0;
	  m = 0.0;
	  if (src->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (src->Coords, iv, &x, &y, &z);
	    }
	  else if (src->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (src->Coords, iv, &x, &y, &m);
	    }
	  else if (src->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (src->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (src->Coords, iv, &x, &y);
	    }
	  if (dst->DimensionModel == GAIA_XY_Z)
	    {
		gaiaSetPointXYZ (dst->Coords, iv, x, y, z);
	    }
	  else if (dst->DimensionModel == GAIA_XY_M)
	    {
		gaiaSetPointXYM (dst->Coords, iv, x, y, m);
	    }
	  else if (dst->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaSetPointXYZM (dst->Coords, iv, x, y, z, m);
	    }
	  else
	    {
		gaiaSetPoint (dst->Coords, iv, x, y);
	    }
      }
}

GAIAGEO_DECLARE gaiaLinestringPtr
gaiaCloneLinestring (gaiaLinestringPtr line)
{
/* clones a LINESTRING */
    gaiaLinestringPtr new_line;
    if (!line)
	return NULL;
    if (line->DimensionModel == GAIA_XY_Z)
	new_line = gaiaAllocLinestringXYZ (line->Points);
    else if (line->DimensionModel == GAIA_XY_M)
	new_line = gaiaAllocLinestringXYM (line->Points);
    else if (line->DimensionModel == GAIA_XY_Z_M)
	new_line = gaiaAllocLinestringXYZM (line->Points);
    else
	new_line = gaiaAllocLinestring (line->Points);
    gaiaCopyLinestringCoords (new_line, line);
    return new_line;
}

GAIAGEO_DECLARE gaiaRingPtr
gaiaAllocRing (int vert)
{
/* ring object constructor */
    gaiaRingPtr p = malloc (sizeof (gaiaRing));
    p->Coords = malloc (sizeof (double) * (vert * 2));
    p->Points = vert;
    p->Link = NULL;
    p->Clockwise = 0;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaRingPtr
gaiaAllocRingXYZ (int vert)
{
/* ring object constructor */
    gaiaRingPtr p = malloc (sizeof (gaiaRing));
    p->Coords = malloc (sizeof (double) * (vert * 3));
    p->Points = vert;
    p->Link = NULL;
    p->Clockwise = 0;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaRingPtr
gaiaAllocRingXYM (int vert)
{
/* ring object constructor */
    gaiaRingPtr p = malloc (sizeof (gaiaRing));
    p->Coords = malloc (sizeof (double) * (vert * 3));
    p->Points = vert;
    p->Link = NULL;
    p->Clockwise = 0;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_M;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaRingPtr
gaiaAllocRingXYZM (int vert)
{
/* ring object constructor */
    gaiaRingPtr p = malloc (sizeof (gaiaRing));
    p->Coords = malloc (sizeof (double) * (vert * 4));
    p->Points = vert;
    p->Link = NULL;
    p->Clockwise = 0;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z_M;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE void
gaiaFreeRing (gaiaRingPtr ptr)
{
/* ring object destructor */
    if (ptr)
      {
	  if (ptr->Coords)
	      free (ptr->Coords);
	  free (ptr);
      }
}

GAIAGEO_DECLARE int
gaiaRingGetPoint (gaiaRingPtr rng, int v, double *x, double *y, double *z,
		  double *m)
{
/* SAFE - getting coords for a vertex in RING */
    double vx;
    double vy;
    double vz;
    double vm;
    *x = 0.0;
    *y = 0.0;
    *z = 0.0;
    *m = 0.0;
    if (!rng)
	return 0;
    if (v < 0 || v >= rng->Points)
	return 0;
    switch (rng->DimensionModel)
      {
      case GAIA_XY:
	  gaiaGetPoint (rng->Coords, v, &vx, &vy);
	  *x = vx;
	  *y = vy;
	  break;
      case GAIA_XY_Z:
	  gaiaGetPointXYZ (rng->Coords, v, &vx, &vy, &vz);
	  *x = vx;
	  *y = vy;
	  *z = vz;
	  break;
      case GAIA_XY_M:
	  gaiaGetPointXYM (rng->Coords, v, &vx, &vy, &vm);
	  *x = vx;
	  *y = vy;
	  *m = vm;
	  break;
      case GAIA_XY_Z_M:
	  gaiaGetPointXYZM (rng->Coords, v, &vx, &vy, &vz, &vm);
	  *x = vx;
	  *y = vy;
	  *z = vz;
	  *m = vm;
	  break;
      default:
	  return 0;
      };
    return 1;
}

GAIAGEO_DECLARE int
gaiaRingSetPoint (gaiaRingPtr rng, int v, double x, double y, double z,
		  double m)
{
/* SAFE - getting coords for a vertex in RING */
    if (!rng)
	return 0;
    if (v < 0 || v >= rng->Points)
	return 0;
    switch (rng->DimensionModel)
      {
      case GAIA_XY:
	  gaiaSetPoint (rng->Coords, v, x, y);
	  break;
      case GAIA_XY_Z:
	  gaiaSetPointXYZ (rng->Coords, v, x, y, z);
	  break;
      case GAIA_XY_M:
	  gaiaSetPointXYM (rng->Coords, v, x, y, m);
	  break;
      case GAIA_XY_Z_M:
	  gaiaSetPointXYZM (rng->Coords, v, x, y, z, m);
	  break;
      default:
	  return 0;
      };
    return 1;
}

GAIAGEO_DECLARE void
gaiaCopyRingCoords (gaiaRingPtr dst, gaiaRingPtr src)
{
/* 
/ copying coords from one Ring to another
/ maybe, converting from one Dimension Model to a different one
*/
    int iv;
    double x;
    double y;
    double z;
    double m;
    if (!src)
	return;
    if (!dst)
	return;
    if (src->Points != dst->Points)
	return;
    for (iv = 0; iv < dst->Points; iv++)
      {
	  z = 0.0;
	  m = 0.0;
	  if (src->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (src->Coords, iv, &x, &y, &z);
	    }
	  else if (src->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (src->Coords, iv, &x, &y, &m);
	    }
	  else if (src->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (src->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (src->Coords, iv, &x, &y);
	    }
	  if (dst->DimensionModel == GAIA_XY_Z)
	    {
		gaiaSetPointXYZ (dst->Coords, iv, x, y, z);
	    }
	  else if (dst->DimensionModel == GAIA_XY_M)
	    {
		gaiaSetPointXYM (dst->Coords, iv, x, y, m);
	    }
	  else if (dst->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaSetPointXYZM (dst->Coords, iv, x, y, z, m);
	    }
	  else
	    {
		gaiaSetPoint (dst->Coords, iv, x, y);
	    }
      }
}

GAIAGEO_DECLARE gaiaRingPtr
gaiaCloneRing (gaiaRingPtr ring)
{
/* clones a RING */
    gaiaRingPtr new_ring;
    if (!ring)
	return NULL;
    if (ring->DimensionModel == GAIA_XY_Z)
	new_ring = gaiaAllocRingXYZ (ring->Points);
    else if (ring->DimensionModel == GAIA_XY_M)
	new_ring = gaiaAllocRingXYM (ring->Points);
    else if (ring->DimensionModel == GAIA_XY_Z_M)
	new_ring = gaiaAllocRingXYZM (ring->Points);
    else
	new_ring = gaiaAllocRing (ring->Points);
    gaiaCopyRingCoords (new_ring, ring);
    return new_ring;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaClonePolygon (gaiaPolygonPtr polyg)
{
/* clones a POLYGON */
    int ib;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!polyg)
	return NULL;
    i_ring = polyg->Exterior;
    if (polyg->DimensionModel == GAIA_XY_Z)
	new_polyg = gaiaAllocPolygonXYZ (i_ring->Points, polyg->NumInteriors);
    else if (polyg->DimensionModel == GAIA_XY_M)
	new_polyg = gaiaAllocPolygonXYM (i_ring->Points, polyg->NumInteriors);
    else if (polyg->DimensionModel == GAIA_XY_Z_M)
	new_polyg = gaiaAllocPolygonXYZM (i_ring->Points, polyg->NumInteriors);
    else
	new_polyg = gaiaAllocPolygon (i_ring->Points, polyg->NumInteriors);
    o_ring = new_polyg->Exterior;
/* copying points for the EXTERIOR RING */
    gaiaCopyRingCoords (o_ring, i_ring);
    for (ib = 0; ib < new_polyg->NumInteriors; ib++)
      {
	  /* copying each INTERIOR RING [if any] */
	  i_ring = polyg->Interiors + ib;
	  o_ring = gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
	  gaiaCopyRingCoords (o_ring, i_ring);
      }
    return new_polyg;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaAllocPolygon (int vert, int excl)
{
/* POLYGON object constructor */
    gaiaPolygonPtr p;
    gaiaRingPtr pP;
    int ind;
    p = malloc (sizeof (gaiaPolygon));
    p->Exterior = gaiaAllocRing (vert);
    p->NumInteriors = excl;
    p->NextInterior = 0;
    p->Next = NULL;
    if (excl == 0)
	p->Interiors = NULL;
    else
	p->Interiors = malloc (sizeof (gaiaRing) * excl);
    for (ind = 0; ind < p->NumInteriors; ind++)
      {
	  pP = p->Interiors + ind;
	  pP->Points = 0;
	  pP->Coords = NULL;
	  pP->Next = NULL;
	  pP->Link = 0;
      }
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY;
    return p;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaAllocPolygonXYZ (int vert, int excl)
{
/* POLYGON object constructor */
    gaiaPolygonPtr p;
    gaiaRingPtr pP;
    int ind;
    p = malloc (sizeof (gaiaPolygon));
    p->Exterior = gaiaAllocRingXYZ (vert);
    p->NumInteriors = excl;
    p->NextInterior = 0;
    p->Next = NULL;
    if (excl == 0)
	p->Interiors = NULL;
    else
	p->Interiors = malloc (sizeof (gaiaRing) * excl);
    for (ind = 0; ind < p->NumInteriors; ind++)
      {
	  pP = p->Interiors + ind;
	  pP->Points = 0;
	  pP->Coords = NULL;
	  pP->Next = NULL;
	  pP->Link = 0;
      }
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z;
    return p;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaAllocPolygonXYM (int vert, int excl)
{
/* POLYGON object constructor */
    gaiaPolygonPtr p;
    gaiaRingPtr pP;
    int ind;
    p = malloc (sizeof (gaiaPolygon));
    p->Exterior = gaiaAllocRingXYM (vert);
    p->NumInteriors = excl;
    p->NextInterior = 0;
    p->Next = NULL;
    if (excl == 0)
	p->Interiors = NULL;
    else
	p->Interiors = malloc (sizeof (gaiaRing) * excl);
    for (ind = 0; ind < p->NumInteriors; ind++)
      {
	  pP = p->Interiors + ind;
	  pP->Points = 0;
	  pP->Coords = NULL;
	  pP->Next = NULL;
	  pP->Link = 0;
      }
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_M;
    return p;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaAllocPolygonXYZM (int vert, int excl)
{
/* POLYGON object constructor */
    gaiaPolygonPtr p;
    gaiaRingPtr pP;
    int ind;
    p = malloc (sizeof (gaiaPolygon));
    p->Exterior = gaiaAllocRingXYZM (vert);
    p->NumInteriors = excl;
    p->NextInterior = 0;
    p->Next = NULL;
    if (excl == 0)
	p->Interiors = NULL;
    else
	p->Interiors = malloc (sizeof (gaiaRing) * excl);
    for (ind = 0; ind < p->NumInteriors; ind++)
      {
	  pP = p->Interiors + ind;
	  pP->Points = 0;
	  pP->Coords = NULL;
	  pP->Next = NULL;
	  pP->Link = 0;
      }
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z_M;
    return p;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaCreatePolygon (gaiaRingPtr ring)
{
/* POLYGON object constructor */
    gaiaPolygonPtr p;
    p = malloc (sizeof (gaiaPolygon));
    p->DimensionModel = ring->DimensionModel;
    if (ring->DimensionModel == GAIA_XY_Z)
	p->Exterior = gaiaAllocRingXYZ (ring->Points);
    else if (ring->DimensionModel == GAIA_XY_M)
	p->Exterior = gaiaAllocRingXYM (ring->Points);
    else if (ring->DimensionModel == GAIA_XY_Z_M)
	p->Exterior = gaiaAllocRingXYZM (ring->Points);
    else
	p->Exterior = gaiaAllocRing (ring->Points);
    p->NumInteriors = 0;
    p->NextInterior = 0;
    p->Next = NULL;
    p->Interiors = NULL;
    gaiaCopyRingCoords (p->Exterior, ring);
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    return p;
}

GAIAGEO_DECLARE void
gaiaFreePolygon (gaiaPolygonPtr p)
{
/* POLYGON object destructor */
    gaiaRingPtr pP;
    int ind;
    if (p->Exterior)
	gaiaFreeRing (p->Exterior);
    for (ind = 0; ind < p->NumInteriors; ind++)
      {
	  pP = p->Interiors + ind;
	  if (pP->Coords)
	      free (pP->Coords);
      }
    if (p->Interiors)
	free (p->Interiors);
    free (p);
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCloneGeomColl (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION */
    int ib;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaLinestringPtr new_line;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr new_polyg;
    gaiaGeomCollPtr new_geom;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!geom)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	new_geom = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	new_geom = gaiaAllocGeomCollXYM ();
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	new_geom = gaiaAllocGeomCollXYZM ();
    else
	new_geom = gaiaAllocGeomColl ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = geom->DeclaredType;
    point = geom->FirstPoint;
    while (point)
      {
	  /* copying POINTs */
	  if (geom->DimensionModel == GAIA_XY_Z)
	      gaiaAddPointToGeomCollXYZ (new_geom, point->X, point->Y,
					 point->Z);
	  else if (geom->DimensionModel == GAIA_XY_M)
	      gaiaAddPointToGeomCollXYM (new_geom, point->X, point->Y,
					 point->M);
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      gaiaAddPointToGeomCollXYZM (new_geom, point->X, point->Y,
					  point->Z, point->M);
	  else
	      gaiaAddPointToGeomColl (new_geom, point->X, point->Y);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* copying LINESTRINGs */
	  new_line = gaiaAddLinestringToGeomColl (new_geom, line->Points);
	  gaiaCopyLinestringCoords (new_line, line);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* copying POLYGONs */
	  i_ring = polyg->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (new_geom, i_ring->Points,
					polyg->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  /* copying points for the EXTERIOR RING */
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < new_polyg->NumInteriors; ib++)
	    {
		/* copying each INTERIOR RING [if any] */
		i_ring = polyg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  polyg = polyg->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCloneGeomCollPoints (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION (Points only) */
    gaiaPointPtr point;
    gaiaGeomCollPtr new_geom;
    if (!geom)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	new_geom = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	new_geom = gaiaAllocGeomCollXYM ();
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	new_geom = gaiaAllocGeomCollXYZM ();
    else
	new_geom = gaiaAllocGeomColl ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = GAIA_MULTIPOINT;
    point = geom->FirstPoint;
    while (point)
      {
	  /* copying POINTs */
	  if (geom->DimensionModel == GAIA_XY_Z)
	      gaiaAddPointToGeomCollXYZ (new_geom, point->X, point->Y,
					 point->Z);
	  else if (geom->DimensionModel == GAIA_XY_M)
	      gaiaAddPointToGeomCollXYM (new_geom, point->X, point->Y,
					 point->M);
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      gaiaAddPointToGeomCollXYZM (new_geom, point->X, point->Y,
					  point->Z, point->M);
	  else
	      gaiaAddPointToGeomColl (new_geom, point->X, point->Y);
	  point = point->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCloneGeomCollLinestrings (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION (Linestrings only) */
    gaiaLinestringPtr line;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr new_geom;
    if (!geom)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	new_geom = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	new_geom = gaiaAllocGeomCollXYM ();
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	new_geom = gaiaAllocGeomCollXYZM ();
    else
	new_geom = gaiaAllocGeomColl ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = GAIA_MULTILINESTRING;
    line = geom->FirstLinestring;
    while (line)
      {
	  /* copying LINESTRINGs */
	  new_line = gaiaAddLinestringToGeomColl (new_geom, line->Points);
	  gaiaCopyLinestringCoords (new_line, line);
	  line = line->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCloneGeomCollPolygons (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION (Polygons only) */
    int ib;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr new_polyg;
    gaiaGeomCollPtr new_geom;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!geom)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	new_geom = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	new_geom = gaiaAllocGeomCollXYM ();
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	new_geom = gaiaAllocGeomCollXYZM ();
    else
	new_geom = gaiaAllocGeomColl ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = GAIA_MULTIPOLYGON;
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* copying POLYGONs */
	  i_ring = polyg->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (new_geom, i_ring->Points,
					polyg->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  /* copying points for the EXTERIOR RING */
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < new_polyg->NumInteriors; ib++)
	    {
		/* copying each INTERIOR RING [if any] */
		i_ring = polyg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  polyg = polyg->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCastGeomCollToXY (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION converting to XY-dimensions */
    int ib;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaLinestringPtr new_line;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr new_polyg;
    gaiaGeomCollPtr new_geom;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!geom)
	return NULL;
    new_geom = gaiaAllocGeomColl ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = geom->DeclaredType;
    point = geom->FirstPoint;
    while (point)
      {
	  /* copying POINTs */
	  gaiaAddPointToGeomColl (new_geom, point->X, point->Y);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* copying LINESTRINGs */
	  new_line = gaiaAddLinestringToGeomColl (new_geom, line->Points);
	  gaiaCopyLinestringCoords (new_line, line);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* copying POLYGONs */
	  i_ring = polyg->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (new_geom, i_ring->Points,
					polyg->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  /* copying points for the EXTERIOR RING */
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < new_polyg->NumInteriors; ib++)
	    {
		/* copying each INTERIOR RING [if any] */
		i_ring = polyg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  polyg = polyg->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCastGeomCollToXYZ (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION converting to XYZ-dimensions */
    int ib;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaLinestringPtr new_line;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr new_polyg;
    gaiaGeomCollPtr new_geom;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!geom)
	return NULL;
    new_geom = gaiaAllocGeomCollXYZ ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = geom->DeclaredType;
    point = geom->FirstPoint;
    while (point)
      {
	  /* copying POINTs */
	  gaiaAddPointToGeomCollXYZ (new_geom, point->X, point->Y, point->Z);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* copying LINESTRINGs */
	  new_line = gaiaAddLinestringToGeomColl (new_geom, line->Points);
	  gaiaCopyLinestringCoords (new_line, line);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* copying POLYGONs */
	  i_ring = polyg->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (new_geom, i_ring->Points,
					polyg->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  /* copying points for the EXTERIOR RING */
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < new_polyg->NumInteriors; ib++)
	    {
		/* copying each INTERIOR RING [if any] */
		i_ring = polyg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  polyg = polyg->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCastGeomCollToXYM (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION converting to XYM-dimensions */
    int ib;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaLinestringPtr new_line;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr new_polyg;
    gaiaGeomCollPtr new_geom;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!geom)
	return NULL;
    new_geom = gaiaAllocGeomCollXYM ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = geom->DeclaredType;
    point = geom->FirstPoint;
    while (point)
      {
	  /* copying POINTs */
	  gaiaAddPointToGeomCollXYM (new_geom, point->X, point->Y, point->M);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* copying LINESTRINGs */
	  new_line = gaiaAddLinestringToGeomColl (new_geom, line->Points);
	  gaiaCopyLinestringCoords (new_line, line);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* copying POLYGONs */
	  i_ring = polyg->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (new_geom, i_ring->Points,
					polyg->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  /* copying points for the EXTERIOR RING */
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < new_polyg->NumInteriors; ib++)
	    {
		/* copying each INTERIOR RING [if any] */
		i_ring = polyg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  polyg = polyg->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaCastGeomCollToXYZM (gaiaGeomCollPtr geom)
{
/* clones a GEOMETRYCOLLECTION converting to XYZM-dimensions */
    int ib;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaLinestringPtr new_line;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr new_polyg;
    gaiaGeomCollPtr new_geom;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    if (!geom)
	return NULL;
    new_geom = gaiaAllocGeomCollXYZM ();
    new_geom->Srid = geom->Srid;
    new_geom->DeclaredType = geom->DeclaredType;
    point = geom->FirstPoint;
    while (point)
      {
	  /* copying POINTs */
	  gaiaAddPointToGeomCollXYZM (new_geom, point->X, point->Y, point->Z,
				      point->M);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* copying LINESTRINGs */
	  new_line = gaiaAddLinestringToGeomColl (new_geom, line->Points);
	  gaiaCopyLinestringCoords (new_line, line);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* copying POLYGONs */
	  i_ring = polyg->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (new_geom, i_ring->Points,
					polyg->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  /* copying points for the EXTERIOR RING */
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < new_polyg->NumInteriors; ib++)
	    {
		/* copying each INTERIOR RING [if any] */
		i_ring = polyg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (new_polyg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  polyg = polyg->Next;
      }
    return new_geom;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaAllocGeomColl ()
{
/* GEOMETRYCOLLECTION object constructor */
    gaiaGeomCollPtr p = malloc (sizeof (gaiaGeomColl));
    p->Srid = -1;
    p->endian = ' ';
    p->offset = 0;
    p->FirstPoint = NULL;
    p->LastPoint = NULL;
    p->FirstLinestring = NULL;
    p->LastLinestring = NULL;
    p->FirstPolygon = NULL;
    p->LastPolygon = NULL;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY;
    p->DeclaredType = GAIA_UNKNOWN;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaAllocGeomCollXYZ ()
{
/* GEOMETRYCOLLECTION object constructor */
    gaiaGeomCollPtr p = malloc (sizeof (gaiaGeomColl));
    p->Srid = -1;
    p->endian = ' ';
    p->offset = 0;
    p->FirstPoint = NULL;
    p->LastPoint = NULL;
    p->FirstLinestring = NULL;
    p->LastLinestring = NULL;
    p->FirstPolygon = NULL;
    p->LastPolygon = NULL;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z;
    p->DeclaredType = GAIA_UNKNOWN;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaAllocGeomCollXYM ()
{
/* GEOMETRYCOLLECTION object constructor */
    gaiaGeomCollPtr p = malloc (sizeof (gaiaGeomColl));
    p->Srid = -1;
    p->endian = ' ';
    p->offset = 0;
    p->FirstPoint = NULL;
    p->LastPoint = NULL;
    p->FirstLinestring = NULL;
    p->LastLinestring = NULL;
    p->FirstPolygon = NULL;
    p->LastPolygon = NULL;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_M;
    p->DeclaredType = GAIA_UNKNOWN;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaAllocGeomCollXYZM ()
{
/* GEOMETRYCOLLECTION object constructor */
    gaiaGeomCollPtr p = malloc (sizeof (gaiaGeomColl));
    p->Srid = -1;
    p->endian = ' ';
    p->offset = 0;
    p->FirstPoint = NULL;
    p->LastPoint = NULL;
    p->FirstLinestring = NULL;
    p->LastLinestring = NULL;
    p->FirstPolygon = NULL;
    p->LastPolygon = NULL;
    p->MinX = DBL_MAX;
    p->MinY = DBL_MAX;
    p->MaxX = -DBL_MAX;
    p->MaxY = -DBL_MAX;
    p->DimensionModel = GAIA_XY_Z_M;
    p->DeclaredType = GAIA_UNKNOWN;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE void
gaiaFreeGeomColl (gaiaGeomCollPtr p)
{
/* GEOMETRYCOLLECTION object destructor */
    gaiaPointPtr pP;
    gaiaPointPtr pPn;
    gaiaLinestringPtr pL;
    gaiaLinestringPtr pLn;
    gaiaPolygonPtr pA;
    gaiaPolygonPtr pAn;
    if (!p)
	return;
    pP = p->FirstPoint;
    while (pP != NULL)
      {
	  pPn = pP->Next;
	  gaiaFreePoint (pP);
	  pP = pPn;
      }
    pL = p->FirstLinestring;
    while (pL != NULL)
      {
	  pLn = pL->Next;
	  gaiaFreeLinestring (pL);
	  pL = pLn;
      }
    pA = p->FirstPolygon;
    while (pA != NULL)
      {
	  pAn = pA->Next;
	  gaiaFreePolygon (pA);
	  pA = pAn;
      }
    free (p);
}

GAIAGEO_DECLARE void
gaiaAddPointToGeomColl (gaiaGeomCollPtr p, double x, double y)
{
/* adding a POINT to this GEOMETRYCOLLECTION */
    gaiaPointPtr point = gaiaAllocPoint (x, y);
    if (p->FirstPoint == NULL)
	p->FirstPoint = point;
    if (p->LastPoint != NULL)
	p->LastPoint->Next = point;
    p->LastPoint = point;
}

GAIAGEO_DECLARE void
gaiaAddPointToGeomCollXYZ (gaiaGeomCollPtr p, double x, double y, double z)
{
/* adding a POINT to this GEOMETRYCOLLECTION */
    gaiaPointPtr point = gaiaAllocPointXYZ (x, y, z);
    if (p->FirstPoint == NULL)
	p->FirstPoint = point;
    if (p->LastPoint != NULL)
	p->LastPoint->Next = point;
    p->LastPoint = point;
}

GAIAGEO_DECLARE void
gaiaAddPointToGeomCollXYM (gaiaGeomCollPtr p, double x, double y, double m)
{
/* adding a POINT to this GEOMETRYCOLLECTION */
    gaiaPointPtr point = gaiaAllocPointXYM (x, y, m);
    if (p->FirstPoint == NULL)
	p->FirstPoint = point;
    if (p->LastPoint != NULL)
	p->LastPoint->Next = point;
    p->LastPoint = point;
}

GAIAGEO_DECLARE void
gaiaAddPointToGeomCollXYZM (gaiaGeomCollPtr p, double x, double y, double z,
			    double m)
{
/* adding a POINT to this GEOMETRYCOLLECTION */
    gaiaPointPtr point = gaiaAllocPointXYZM (x, y, z, m);
    if (p->FirstPoint == NULL)
	p->FirstPoint = point;
    if (p->LastPoint != NULL)
	p->LastPoint->Next = point;
    p->LastPoint = point;
}

GAIAGEO_DECLARE gaiaLinestringPtr
gaiaAddLinestringToGeomColl (gaiaGeomCollPtr p, int vert)
{
/* adding a LINESTRING to this GEOMETRYCOLLECTION */
    gaiaLinestringPtr line;
    if (p->DimensionModel == GAIA_XY_Z)
	line = gaiaAllocLinestringXYZ (vert);
    else if (p->DimensionModel == GAIA_XY_M)
	line = gaiaAllocLinestringXYM (vert);
    else if (p->DimensionModel == GAIA_XY_Z_M)
	line = gaiaAllocLinestringXYZM (vert);
    else
	line = gaiaAllocLinestring (vert);
    if (p->FirstLinestring == NULL)
	p->FirstLinestring = line;
    if (p->LastLinestring != NULL)
	p->LastLinestring->Next = line;
    p->LastLinestring = line;
    return line;
}

GAIAGEO_DECLARE void
gaiaInsertLinestringInGeomColl (gaiaGeomCollPtr p, gaiaLinestringPtr line)
{
/* adding an existing LINESTRING to this GEOMETRYCOLLECTION */
    if (p->FirstLinestring == NULL)
	p->FirstLinestring = line;
    if (p->LastLinestring != NULL)
	p->LastLinestring->Next = line;
    p->LastLinestring = line;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaAddPolygonToGeomColl (gaiaGeomCollPtr p, int vert, int interiors)
{
/* adding a POLYGON to this GEOMETRYCOLLECTION */
    gaiaPolygonPtr polyg;
    if (p->DimensionModel == GAIA_XY_Z)
	polyg = gaiaAllocPolygonXYZ (vert, interiors);
    else if (p->DimensionModel == GAIA_XY_M)
	polyg = gaiaAllocPolygonXYM (vert, interiors);
    else if (p->DimensionModel == GAIA_XY_Z_M)
	polyg = gaiaAllocPolygonXYZM (vert, interiors);
    else
	polyg = gaiaAllocPolygon (vert, interiors);
    if (p->FirstPolygon == NULL)
	p->FirstPolygon = polyg;
    if (p->LastPolygon != NULL)
	p->LastPolygon->Next = polyg;
    p->LastPolygon = polyg;
    return polyg;
}

GAIAGEO_DECLARE gaiaPolygonPtr
gaiaInsertPolygonInGeomColl (gaiaGeomCollPtr p, gaiaRingPtr ring)
{
/* adding a POLYGON to this GEOMETRYCOLLECTION */
    gaiaPolygonPtr polyg;
    polyg = malloc (sizeof (gaiaPolygon));
    polyg->Exterior = ring;
    polyg->NumInteriors = 0;
    polyg->NextInterior = 0;
    polyg->DimensionModel = ring->DimensionModel;
    polyg->Next = NULL;
    polyg->Interiors = NULL;
    polyg->MinX = DBL_MAX;
    polyg->MinY = DBL_MAX;
    polyg->MaxX = -DBL_MAX;
    polyg->MaxY = -DBL_MAX;
    if (p->FirstPolygon == NULL)
	p->FirstPolygon = polyg;
    if (p->LastPolygon != NULL)
	p->LastPolygon->Next = polyg;
    p->LastPolygon = polyg;
    return polyg;
}

GAIAGEO_DECLARE gaiaRingPtr
gaiaAddInteriorRing (gaiaPolygonPtr p, int pos, int vert)
{
/* adding an interior ring to some polygon */
    gaiaRingPtr pP = p->Interiors + pos;
    pP->Points = vert;
    pP->DimensionModel = p->DimensionModel;
    if (pP->DimensionModel == GAIA_XY_Z)
	pP->Coords = malloc (sizeof (double) * (vert * 3));
    else if (pP->DimensionModel == GAIA_XY_M)
	pP->Coords = malloc (sizeof (double) * (vert * 3));
    else if (pP->DimensionModel == GAIA_XY_Z_M)
	pP->Coords = malloc (sizeof (double) * (vert * 4));
    else
	pP->Coords = malloc (sizeof (double) * (vert * 2));
    return pP;
}

GAIAGEO_DECLARE void
gaiaInsertInteriorRing (gaiaPolygonPtr p, gaiaRingPtr ring)
{
/* adding an interior ring to some polygon */
    gaiaRingPtr hole;
    if (p->NumInteriors == 0)
      {
	  /* this one is the first interior ring */
	  p->NumInteriors++;
	  p->Interiors = malloc (sizeof (gaiaRing));
	  hole = p->Interiors;
      }
    else
      {
	  /* some interior ring is already defined */
	  gaiaRingPtr save = p->Interiors;
	  p->Interiors = malloc (sizeof (gaiaRing) * (p->NumInteriors + 1));
	  memcpy (p->Interiors, save, (sizeof (gaiaRing) * p->NumInteriors));
	  free (save);
	  hole = p->Interiors + p->NumInteriors;
	  p->NumInteriors++;
      }
    hole->Points = ring->Points;
    hole->DimensionModel = p->DimensionModel;
    if (hole->DimensionModel == GAIA_XY_Z)
	hole->Coords = malloc (sizeof (double) * (hole->Points * 3));
    else if (hole->DimensionModel == GAIA_XY_M)
	hole->Coords = malloc (sizeof (double) * (hole->Points * 3));
    else if (hole->DimensionModel == GAIA_XY_Z_M)
	hole->Coords = malloc (sizeof (double) * (hole->Points * 4));
    else
	hole->Coords = malloc (sizeof (double) * (hole->Points * 2));
    gaiaCopyRingCoords (hole, ring);
}

GAIAGEO_DECLARE void
gaiaAddRingToPolyg (gaiaPolygonPtr polyg, gaiaRingPtr ring)
{
/* adds an interior ring to this POLYGON  */
    gaiaRingPtr old_interiors = NULL;
    if (!(polyg->Interiors))
      {
	  /* this one is the first interior ring */
	  polyg->Interiors = ring;
	  polyg->NumInteriors = 1;
      }
    else
      {
	  /* adding another interior ring */
	  old_interiors = polyg->Interiors;
	  polyg->Interiors =
	      malloc (sizeof (gaiaRing) * (polyg->NumInteriors + 1));
	  memcpy (polyg->Interiors, old_interiors,
		  (sizeof (gaiaRing) * polyg->NumInteriors));
	  memcpy (polyg->Interiors + polyg->NumInteriors, ring,
		  sizeof (gaiaRing));
	  (polyg->NumInteriors)++;
	  free (old_interiors);
	  free (ring);
      }
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaAllocDynamicLine ()
{
/* DYNAMIC LINE object constructor */
    gaiaDynamicLinePtr p = malloc (sizeof (gaiaDynamicLine));
    p->Error = 0;
    p->Srid = -1;
    p->First = NULL;
    p->Last = NULL;
    return p;
}

GAIAGEO_DECLARE void
gaiaFreeDynamicLine (gaiaDynamicLinePtr p)
{
/* DYNAMIC LINE object destructor */
    gaiaPointPtr pP;
    gaiaPointPtr pPn;
    pP = p->First;
    while (pP != NULL)
      {
	  pPn = pP->Next;
	  gaiaFreePoint (pP);
	  pP = pPn;
      }
    free (p);
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaAppendPointToDynamicLine (gaiaDynamicLinePtr p, double x, double y)
{
/* inserts a new POINT to this DYNAMIC LINE after the last one */
    gaiaPointPtr point = gaiaAllocPoint (x, y);
    point->Prev = p->Last;
    if (p->First == NULL)
	p->First = point;
    if (p->Last != NULL)
	p->Last->Next = point;
    p->Last = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaAppendPointZToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
			       double z)
{
/* inserts a new POINT to this DYNAMIC LINE after the last one */
    gaiaPointPtr point = gaiaAllocPointXYZ (x, y, z);
    point->Prev = p->Last;
    if (p->First == NULL)
	p->First = point;
    if (p->Last != NULL)
	p->Last->Next = point;
    p->Last = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaAppendPointMToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
			       double m)
{
/* inserts a new POINT to this DYNAMIC LINE after the last one */
    gaiaPointPtr point = gaiaAllocPointXYM (x, y, m);
    point->Prev = p->Last;
    if (p->First == NULL)
	p->First = point;
    if (p->Last != NULL)
	p->Last->Next = point;
    p->Last = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaAppendPointZMToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
				double z, double m)
{
/* inserts a new POINT to this DYNAMIC LINE after the last one */
    gaiaPointPtr point = gaiaAllocPointXYZM (x, y, z, m);
    point->Prev = p->Last;
    if (p->First == NULL)
	p->First = point;
    if (p->Last != NULL)
	p->Last->Next = point;
    p->Last = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaPrependPointToDynamicLine (gaiaDynamicLinePtr p, double x, double y)
{
/* inserts a new POINT to this DYNAMIC LINE before the first one */
    gaiaPointPtr point = gaiaAllocPoint (x, y);
    point->Next = p->First;
    if (p->Last == NULL)
	p->Last = point;
    if (p->First != NULL)
	p->First->Prev = point;
    p->First = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaPrependPointZToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
				double z)
{
/* inserts a new POINT to this DYNAMIC LINE before the first one */
    gaiaPointPtr point = gaiaAllocPointXYZ (x, y, z);
    point->Next = p->First;
    if (p->Last == NULL)
	p->Last = point;
    if (p->First != NULL)
	p->First->Prev = point;
    p->First = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaPrependPointMToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
				double m)
{
/* inserts a new POINT to this DYNAMIC LINE before the first one */
    gaiaPointPtr point = gaiaAllocPointXYM (x, y, m);
    point->Next = p->First;
    if (p->Last == NULL)
	p->Last = point;
    if (p->First != NULL)
	p->First->Prev = point;
    p->First = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaPrependPointZMToDynamicLine (gaiaDynamicLinePtr p, double x, double y,
				 double z, double m)
{
/* inserts a new POINT to this DYNAMIC LINE before the first one */
    gaiaPointPtr point = gaiaAllocPointXYZM (x, y, z, m);
    point->Next = p->First;
    if (p->Last == NULL)
	p->Last = point;
    if (p->First != NULL)
	p->First->Prev = point;
    p->First = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaDynamicLineInsertAfter (gaiaDynamicLinePtr p, gaiaPointPtr pt, double x,
			    double y)
{
/* inserts a new POINT to this DYNAMIC LINE after the referenced POINT */
    gaiaPointPtr point = gaiaAllocPoint (x, y);
    point->Prev = pt;
    point->Next = pt->Next;
    if (pt->Next)
	pt->Next->Prev = point;
    pt->Next = point;
    if (pt == p->Last)
	p->Last = point;
    return point;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaDynamicLineInsertBefore (gaiaDynamicLinePtr p, gaiaPointPtr pt, double x,
			     double y)
{
/* inserts a new POINT to this DYNAMIC LINE before the referenced POINT */
    gaiaPointPtr point = gaiaAllocPoint (x, y);
    point->Next = pt;
    point->Prev = pt->Prev;
    if (pt->Prev)
	pt->Prev->Next = point;
    pt->Prev = point;
    if (pt == p->First)
	p->First = point;
    return point;
}

GAIAGEO_DECLARE void
gaiaDynamicLineDeletePoint (gaiaDynamicLinePtr p, gaiaPointPtr pt)
{
/* deletes a POINT from this DYNAMIC LINE */
    if (pt->Prev)
	pt->Prev->Next = pt->Next;
    if (pt->Next)
	pt->Next->Prev = pt->Prev;
    if (pt == p->First)
	p->First = pt->Next;
    if (pt == p->Last)
	p->Last = pt->Prev;
    gaiaFreePoint (pt);
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaCloneDynamicLine (gaiaDynamicLinePtr org)
{
/* creates a new line obtained by simply copying the current one */
    gaiaPointPtr pt;
    gaiaDynamicLinePtr dst = gaiaAllocDynamicLine ();
    pt = org->First;
    while (pt)
      {
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  pt = pt->Next;
      }
    return dst;
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaReverseDynamicLine (gaiaDynamicLinePtr org)
{
/* creates a new line obtained by inverting the current one */
    gaiaPointPtr pt;
    gaiaDynamicLinePtr dst = gaiaAllocDynamicLine ();
    pt = org->Last;
    while (pt)
      {
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  pt = pt->Prev;
      }
    return dst;
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaDynamicLineSplitBefore (gaiaDynamicLinePtr org, gaiaPointPtr point)
{
/* creates a new line obtained by cutting the current one in two */
    gaiaDynamicLinePtr dst = gaiaAllocDynamicLine ();
    dst->First = org->First;
    dst->Last = point->Prev;
    point->Prev->Next = NULL;
    org->First = point;
    point->Prev = NULL;
    return dst;
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaDynamicLineSplitAfter (gaiaDynamicLinePtr org, gaiaPointPtr point)
{
/* creates a new line obtained by cutting the current one in two */
    gaiaDynamicLinePtr dst = gaiaAllocDynamicLine ();
    dst->First = point->Next;
    dst->Last = org->Last;
    point->Next->Prev = NULL;
    org->Last = point;
    point->Next = NULL;
    return dst;
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaDynamicLineJoinAfter (gaiaDynamicLinePtr org, gaiaPointPtr point,
			  gaiaDynamicLinePtr toJoin)
{
/* creates a new line obtained by joining the current one with another one */
    gaiaPointPtr pt;
    gaiaDynamicLinePtr dst = gaiaAllocDynamicLine ();
    pt = org->First;
    while (pt)
      {
	  /* inserting the first slice since the delimiting POINT included */
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  if (pt == point)
	      break;
	  pt = pt->Next;
      }
    pt = toJoin->First;
    while (pt)
      {
	  /* inserting the other line */
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  pt = pt->Next;
      }
    pt = point->Next;
    while (pt)
      {
	  /* inserting the second slice after the delimiting POINT */
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  pt = pt->Next;
      }
    return dst;
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaDynamicLineJoinBefore (gaiaDynamicLinePtr org, gaiaPointPtr point,
			   gaiaDynamicLinePtr toJoin)
{
/* creates a new line obtained by joining the current one with another one */
    gaiaPointPtr pt;
    gaiaDynamicLinePtr dst = gaiaAllocDynamicLine ();
    pt = org->First;
    while (pt)
      {
	  /* inserting the first slice since the delimiting POINT excluded */
	  if (pt == point)
	      break;
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  pt = pt->Next;
      }
    pt = toJoin->First;
    while (pt)
      {
	  /* inserting the other line */
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  pt = pt->Next;
      }
    pt = point;
    while (pt)
      {
	  /* inserting the second slice beginning from the delimiting POINT */
	  gaiaAppendPointToDynamicLine (dst, pt->X, pt->Y);
	  pt = pt->Next;
      }
    return dst;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaDynamicLineFindByCoords (gaiaDynamicLinePtr p, double x, double y)
{
/* finds a POINT inside this DYNAMIC LINE */
    gaiaPointPtr pP;
    pP = p->First;
    while (pP != NULL)
      {
	  if (pP->X == x && pP->Y == y)
	      return pP;
	  pP = pP->Next;
      }
    return NULL;
}

GAIAGEO_DECLARE gaiaPointPtr
gaiaDynamicLineFindByPos (gaiaDynamicLinePtr p, int pos)
{
/* finds a POINT inside this DYNAMIC LINE */
    int n = 0;
    gaiaPointPtr pP;
    pP = p->First;
    while (pP != NULL)
      {
	  if (pos == n)
	      return pP;
	  n++;
	  pP = pP->Next;
      }
    return NULL;
}

GAIAGEO_DECLARE gaiaDynamicLinePtr
gaiaCreateDynamicLine (double *coords, int points)
{
/* creates a DynamicLine from an array of coordinates */
    int iv;
    double x;
    double y;
    gaiaDynamicLinePtr line = gaiaAllocDynamicLine ();
    for (iv = 0; iv < points; iv++)
      {
	  gaiaGetPoint (coords, iv, &x, &y);
	  gaiaAppendPointToDynamicLine (line, x, y);
      }
    return line;
}

GAIAGEO_DECLARE void
gaiaMbrLinestring (gaiaLinestringPtr line)
{
/* computes the MBR for this linestring */
    int iv;
    double x;
    double y;
    double z;
    double m;
    line->MinX = DBL_MAX;
    line->MinY = DBL_MAX;
    line->MaxX = -DBL_MAX;
    line->MaxY = -DBL_MAX;
    for (iv = 0; iv < line->Points; iv++)
      {
	  if (line->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	    }
	  else if (line->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	    }
	  else if (line->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (line->Coords, iv, &x, &y);
	    }
	  if (x < line->MinX)
	      line->MinX = x;
	  if (y < line->MinY)
	      line->MinY = y;
	  if (x > line->MaxX)
	      line->MaxX = x;
	  if (y > line->MaxY)
	      line->MaxY = y;
      }
}

GAIAGEO_DECLARE void
gaiaMbrRing (gaiaRingPtr rng)
{
/* computes the MBR for this ring */
    int iv;
    double x;
    double y;
    double z;
    double m;
    rng->MinX = DBL_MAX;
    rng->MinY = DBL_MAX;
    rng->MaxX = -DBL_MAX;
    rng->MaxY = -DBL_MAX;
    for (iv = 0; iv < rng->Points; iv++)
      {
	  if (rng->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
	    }
	  else if (rng->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
	    }
	  else if (rng->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (rng->Coords, iv, &x, &y);
	    }
	  if (x < rng->MinX)
	      rng->MinX = x;
	  if (y < rng->MinY)
	      rng->MinY = y;
	  if (x > rng->MaxX)
	      rng->MaxX = x;
	  if (y > rng->MaxY)
	      rng->MaxY = y;
      }
}

GAIAGEO_DECLARE void
gaiaMbrPolygon (gaiaPolygonPtr polyg)
{
/* computes the MBR for this polygon */
    gaiaRingPtr rng;
    polyg->MinX = DBL_MAX;
    polyg->MinY = DBL_MAX;
    polyg->MaxX = -DBL_MAX;
    polyg->MaxY = -DBL_MAX;
    rng = polyg->Exterior;
    gaiaMbrRing (rng);
    if (rng->MinX < polyg->MinX)
	polyg->MinX = rng->MinX;
    if (rng->MinY < polyg->MinY)
	polyg->MinY = rng->MinY;
    if (rng->MaxX > polyg->MaxX)
	polyg->MaxX = rng->MaxX;
    if (rng->MaxY > polyg->MaxY)
	polyg->MaxY = rng->MaxY;
}

GAIAGEO_DECLARE void
gaiaMbrGeometry (gaiaGeomCollPtr geom)
{
/* computes the MBR for this geometry */
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    geom->MinX = DBL_MAX;
    geom->MinY = DBL_MAX;
    geom->MaxX = -DBL_MAX;
    geom->MaxY = -DBL_MAX;
    point = geom->FirstPoint;
    while (point)
      {
	  if (point->X < geom->MinX)
	      geom->MinX = point->X;
	  if (point->Y < geom->MinY)
	      geom->MinY = point->Y;
	  if (point->X > geom->MaxX)
	      geom->MaxX = point->X;
	  if (point->Y > geom->MaxY)
	      geom->MaxY = point->Y;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  gaiaMbrLinestring (line);
	  if (line->MinX < geom->MinX)
	      geom->MinX = line->MinX;
	  if (line->MinY < geom->MinY)
	      geom->MinY = line->MinY;
	  if (line->MaxX > geom->MaxX)
	      geom->MaxX = line->MaxX;
	  if (line->MaxY > geom->MaxY)
	      geom->MaxY = line->MaxY;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  gaiaMbrPolygon (polyg);
	  if (polyg->MinX < geom->MinX)
	      geom->MinX = polyg->MinX;
	  if (polyg->MinY < geom->MinY)
	      geom->MinY = polyg->MinY;
	  if (polyg->MaxX > geom->MaxX)
	      geom->MaxX = polyg->MaxX;
	  if (polyg->MaxY > geom->MaxY)
	      geom->MaxY = polyg->MaxY;
	  polyg = polyg->Next;
      }
}

GAIAGEO_DECLARE void
gaiaMRangeLinestring (gaiaLinestringPtr line, double *min, double *max)
{
/* computes the M-range [min/max] for this linestring */
    int iv;
    double x;
    double y;
    double z;
    double m;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    for (iv = 0; iv < line->Points; iv++)
      {
	  m = 0.0;
	  if (line->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	    }
	  else if (line->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	    }
	  else if (line->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (line->Coords, iv, &x, &y);
	    }
	  if (m < *min)
	      *min = m;
	  if (m > *max)
	      *max = m;
      }
}

GAIAGEO_DECLARE void
gaiaMRangeRing (gaiaRingPtr rng, double *min, double *max)
{
/* computes the M-range [min/max] for this ring */
    int iv;
    double x;
    double y;
    double z;
    double m;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    for (iv = 0; iv < rng->Points; iv++)
      {
	  m = 0.0;
	  if (rng->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
	    }
	  else if (rng->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
	    }
	  else if (rng->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (rng->Coords, iv, &x, &y);
	    }
	  if (m < *min)
	      *min = m;
	  if (m > *max)
	      *max = m;
      }
}

GAIAGEO_DECLARE void
gaiaMRangePolygon (gaiaPolygonPtr polyg, double *min, double *max)
{
/* computes the M-range [min/max] for this polygon */
    gaiaRingPtr rng;
    int ib;
    double r_min;
    double r_max;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    rng = polyg->Exterior;
    gaiaMRangeRing (rng, &r_min, &r_max);
    if (r_min < *min)
	*min = r_min;
    if (r_max > *max)
	*max = r_max;
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  rng = polyg->Interiors + ib;
	  gaiaMRangeRing (rng, &r_min, &r_max);
	  if (r_min < *min)
	      *min = r_min;
	  if (r_max > *max)
	      *max = r_max;
      }
}

GAIAGEO_DECLARE void
gaiaMRangeGeometry (gaiaGeomCollPtr geom, double *min, double *max)
{
/* computes the M-range [min/max] for this geometry */
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    double m;
    double r_min;
    double r_max;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    point = geom->FirstPoint;
    while (point)
      {
	  m = 0.0;
	  if (point->DimensionModel == GAIA_XY_M
	      || point->DimensionModel == GAIA_XY_Z_M)
	      m = point->M;
	  if (m < *min)
	      *min = m;
	  if (m > *max)
	      *max = m;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  gaiaMRangeLinestring (line, &r_min, &r_max);
	  if (r_min < *min)
	      *min = r_min;
	  if (r_max > *max)
	      *max = r_max;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  gaiaMRangePolygon (polyg, &r_min, &r_max);
	  if (r_min < *min)
	      *min = r_min;
	  if (r_max > *max)
	      *max = r_max;
	  polyg = polyg->Next;
      }
}

GAIAGEO_DECLARE void
gaiaZRangeLinestring (gaiaLinestringPtr line, double *min, double *max)
{
/* computes the Z-range [min/max] for this linestring */
    int iv;
    double x;
    double y;
    double z;
    double m;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    for (iv = 0; iv < line->Points; iv++)
      {
	  z = 0.0;
	  m = 0.0;
	  if (line->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	    }
	  else if (line->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	    }
	  else if (line->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (line->Coords, iv, &x, &y);
	    }
	  if (z < *min)
	      *min = z;
	  if (z > *max)
	      *max = z;
      }
}

GAIAGEO_DECLARE void
gaiaZRangeRing (gaiaRingPtr rng, double *min, double *max)
{
/* computes the Z-range [min/max] for this ring */
    int iv;
    double x;
    double y;
    double z;
    double m;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    for (iv = 0; iv < rng->Points; iv++)
      {
	  z = 0.0;
	  m = 0.0;
	  if (rng->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
	    }
	  else if (rng->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
	    }
	  else if (rng->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (rng->Coords, iv, &x, &y);
	    }
	  if (m < *min)
	      *min = m;
	  if (m > *max)
	      *max = m;
      }
}

GAIAGEO_DECLARE void
gaiaZRangePolygon (gaiaPolygonPtr polyg, double *min, double *max)
{
/* computes the Z-range [min/max] for this polygon */
    gaiaRingPtr rng;
    int ib;
    double r_min;
    double r_max;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    rng = polyg->Exterior;
    gaiaZRangeRing (rng, &r_min, &r_max);
    if (r_min < *min)
	*min = r_min;
    if (r_max > *max)
	*max = r_max;
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  rng = polyg->Interiors + ib;
	  gaiaZRangeRing (rng, &r_min, &r_max);
	  if (r_min < *min)
	      *min = r_min;
	  if (r_max > *max)
	      *max = r_max;
      }
}

GAIAGEO_DECLARE void
gaiaZRangeGeometry (gaiaGeomCollPtr geom, double *min, double *max)
{
/* computes the Z-range [min/max] for this geometry */
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    double z;
    double r_min;
    double r_max;
    *min = DBL_MAX;
    *max = -DBL_MAX;
    point = geom->FirstPoint;
    while (point)
      {
	  z = 0.0;
	  if (point->DimensionModel == GAIA_XY_Z
	      || point->DimensionModel == GAIA_XY_Z_M)
	      z = point->Z;
	  if (z < *min)
	      *min = z;
	  if (z > *max)
	      *max = z;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  gaiaZRangeLinestring (line, &r_min, &r_max);
	  if (r_min < *min)
	      *min = r_min;
	  if (r_max > *max)
	      *max = r_max;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  gaiaZRangePolygon (polyg, &r_min, &r_max);
	  if (r_min < *min)
	      *min = r_min;
	  if (r_max > *max)
	      *max = r_max;
	  polyg = polyg->Next;
      }
}

GAIAGEO_DECLARE int
gaiaDimension (gaiaGeomCollPtr geom)
{
/* determines the Dimension for this geometry */
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    if (!geom)
	return -1;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counts how many points are there */
	  n_points++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counts how many linestrings are there */
	  n_linestrings++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counts how many polygons are there */
	  n_polygons++;
	  polyg = polyg->Next;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons == 0)
	return -1;
    if (n_points > 0 && n_linestrings == 0 && n_polygons == 0)
	return 0;
    if (n_linestrings > 0 && n_polygons == 0)
	return 1;
    return 2;
}

GAIAGEO_DECLARE int
gaiaGeometryType (gaiaGeomCollPtr geom)
{
/* determines the Class for this geometry */
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    int ib;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    int dm = GAIA_XY;
    if (!geom)
	return GAIA_UNKNOWN;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counts how many points are there */
	  n_points++;
	  if (point->DimensionModel == GAIA_XY_Z)
	    {
		if (dm == GAIA_XY)
		    dm = GAIA_XY_Z;
		else if (dm == GAIA_XY_M)
		    dm = GAIA_XY_Z_M;
	    }
	  else if (point->DimensionModel == GAIA_XY_M)
	    {
		if (dm == GAIA_XY)
		    dm = GAIA_XY_M;
		else if (dm == GAIA_XY_Z)
		    dm = GAIA_XY_Z_M;
	    }
	  else if (point->DimensionModel == GAIA_XY_Z_M)
	      dm = GAIA_XY_Z_M;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counts how many linestrings are there */
	  n_linestrings++;
	  if (line->DimensionModel == GAIA_XY_Z)
	    {
		if (dm == GAIA_XY)
		    dm = GAIA_XY_Z;
		else if (dm == GAIA_XY_M)
		    dm = GAIA_XY_Z_M;
	    }
	  else if (line->DimensionModel == GAIA_XY_M)
	    {
		if (dm == GAIA_XY)
		    dm = GAIA_XY_M;
		else if (dm == GAIA_XY_Z)
		    dm = GAIA_XY_Z_M;
	    }
	  else if (line->DimensionModel == GAIA_XY_Z_M)
	      dm = GAIA_XY_Z_M;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counts how many polygons are there */
	  n_polygons++;
	  ring = polyg->Exterior;
	  if (ring->DimensionModel == GAIA_XY_Z)
	    {
		if (dm == GAIA_XY)
		    dm = GAIA_XY_Z;
		else if (dm == GAIA_XY_M)
		    dm = GAIA_XY_Z_M;
	    }
	  else if (ring->DimensionModel == GAIA_XY_M)
	    {
		if (dm == GAIA_XY)
		    dm = GAIA_XY_M;
		else if (dm == GAIA_XY_Z)
		    dm = GAIA_XY_Z_M;
	    }
	  else if (ring->DimensionModel == GAIA_XY_Z_M)
	      dm = GAIA_XY_Z_M;
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		ring = polyg->Interiors + ib;
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      if (dm == GAIA_XY)
			  dm = GAIA_XY_Z;
		      else if (dm == GAIA_XY_M)
			  dm = GAIA_XY_Z_M;
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      if (dm == GAIA_XY)
			  dm = GAIA_XY_M;
		      else if (dm == GAIA_XY_Z)
			  dm = GAIA_XY_Z_M;
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		    dm = GAIA_XY_Z_M;
	    }
	  polyg = polyg->Next;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons == 0)
	return GAIA_UNKNOWN;
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_MULTIPOINTZ;
		if (dm == GAIA_XY_M)
		    return GAIA_MULTIPOINTM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_MULTIPOINTZM;
		else
		    return GAIA_MULTIPOINT;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_GEOMETRYCOLLECTIONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_GEOMETRYCOLLECTIONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_GEOMETRYCOLLECTIONZM;
		else
		    return GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_POINTZ;
		if (dm == GAIA_XY_M)
		    return GAIA_POINTM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_POINTZM;
		else
		    return GAIA_POINT;
	    }
      }
    if (n_points > 0 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_GEOMETRYCOLLECTIONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_GEOMETRYCOLLECTIONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_GEOMETRYCOLLECTIONZM;
		else
		    return GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_MULTIPOINTZ;
		if (dm == GAIA_XY_M)
		    return GAIA_MULTIPOINTM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_MULTIPOINTZM;
		else
		    return GAIA_MULTIPOINT;
	    }
      }
    if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_MULTILINESTRINGZ;
		if (dm == GAIA_XY_M)
		    return GAIA_MULTILINESTRINGM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_MULTILINESTRINGZM;
		else
		    return GAIA_MULTILINESTRING;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_GEOMETRYCOLLECTIONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_GEOMETRYCOLLECTIONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_GEOMETRYCOLLECTIONZM;
		else
		    return GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_LINESTRINGZ;
		if (dm == GAIA_XY_M)
		    return GAIA_LINESTRINGM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_LINESTRINGZM;
		else
		    return GAIA_LINESTRING;
	    }
      }
    if (n_points == 0 && n_linestrings > 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_GEOMETRYCOLLECTIONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_GEOMETRYCOLLECTIONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_GEOMETRYCOLLECTIONZM;
		else
		    return GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_MULTILINESTRINGZ;
		if (dm == GAIA_XY_M)
		    return GAIA_MULTILINESTRINGM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_MULTILINESTRINGZM;
		else
		    return GAIA_MULTILINESTRING;
	    }
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_MULTIPOLYGONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_MULTIPOLYGONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_MULTIPOLYGONZM;
		else
		    return GAIA_MULTIPOLYGON;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_GEOMETRYCOLLECTIONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_GEOMETRYCOLLECTIONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_GEOMETRYCOLLECTIONZM;
		else
		    return GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_POLYGONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_POLYGONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_POLYGONZM;
		else
		    return GAIA_POLYGON;
	    }
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons > 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_GEOMETRYCOLLECTIONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_GEOMETRYCOLLECTIONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_GEOMETRYCOLLECTIONZM;
		else
		    return GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (dm == GAIA_XY_Z)
		    return GAIA_MULTIPOLYGONZ;
		if (dm == GAIA_XY_M)
		    return GAIA_MULTIPOLYGONM;
		if (dm == GAIA_XY_Z_M)
		    return GAIA_MULTIPOLYGONZM;
		else
		    return GAIA_MULTIPOLYGON;
	    }
      }
    if (dm == GAIA_XY_Z)
	return GAIA_GEOMETRYCOLLECTIONZ;
    if (dm == GAIA_XY_M)
	return GAIA_GEOMETRYCOLLECTIONM;
    if (dm == GAIA_XY_Z_M)
	return GAIA_GEOMETRYCOLLECTIONZM;
    else
	return GAIA_GEOMETRYCOLLECTION;
}

GAIAGEO_DECLARE int
gaiaGeometryAliasType (gaiaGeomCollPtr geom)
{
/* determines the AliasClass for this geometry */
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    if (!geom)
	return GAIA_UNKNOWN;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counts how many points are there */
	  n_points++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counts how many linestrings are there */
	  n_linestrings++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counts how many polygons are there */
	  n_polygons++;
	  polyg = polyg->Next;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons == 0)
	return GAIA_UNKNOWN;
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	      return GAIA_MULTIPOINT;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_POINT;
      }
    if (n_points >= 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_MULTIPOINT;
      }
    if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	      return GAIA_MULTILINESTRING;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_LINESTRING;
      }
    if (n_points == 0 && n_linestrings >= 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_MULTILINESTRING;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	      return GAIA_MULTIPOLYGON;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_POLYGON;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons >= 1)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_MULTIPOLYGON;
      }
    return GAIA_GEOMETRYCOLLECTION;
}

GAIAGEO_DECLARE int
gaiaIsEmpty (gaiaGeomCollPtr geom)
{
/* checks if this GEOMETRYCOLLECTION is an empty one */
    if (!geom)
	return 1;
    if (geom->FirstPoint != NULL)
      {
	  /* there is at least one point */
	  return 0;
      }
    if (geom->FirstLinestring != NULL)
      {
	  /* there is at least one linestring */
	  return 0;
      }
    if (geom->FirstPolygon != NULL)
      {
	  /* there is at least one polygon */
	  return 0;
      }
    return 1;
}

GAIAGEO_DECLARE int
gaiaIsClosed (gaiaLinestringPtr line)
{
/* checks if this linestring is a closed one */
    double x0;
    double y0;
    double x1;
    double y1;
    double z;
    double m;
    if (!line)
	return 0;
    if (line->Points < 3)
	return 0;
    if (line->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (line->Coords, 0, &x0, &y0, &z);
      }
    else if (line->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (line->Coords, 0, &x0, &y0, &m);
      }
    else if (line->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (line->Coords, 0, &x0, &y0, &z, &m);
      }
    else
      {
	  gaiaGetPoint (line->Coords, 0, &x0, &y0);
      }
    if (line->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (line->Coords, (line->Points - 1), &x1, &y1, &z);
      }
    else if (line->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (line->Coords, (line->Points - 1), &x1, &y1, &m);
      }
    else if (line->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (line->Coords, (line->Points - 1), &x1, &y1, &z, &m);
      }
    else
      {
	  gaiaGetPoint (line->Coords, (line->Points - 1), &x1, &y1);
      }
    if (x0 == x1 && y0 == y1)
	return 1;
    return 0;
}

GAIAGEO_DECLARE int
gaiaMbrsEqual (gaiaGeomCollPtr mbr1, gaiaGeomCollPtr mbr2)
{
/* 
/ checks if two MBRs are identical
/
/ returns 1 if TRUE
/ 0 if FALSE
*/
    if (mbr1->MinX != mbr2->MinX)
	return 0;
    if (mbr1->MinY != mbr2->MinY)
	return 0;
    if (mbr1->MaxX != mbr2->MaxX)
	return 0;
    if (mbr1->MaxY != mbr2->MaxY)
	return 0;
    return 1;
}

GAIAGEO_DECLARE int
gaiaMbrsDisjoint (gaiaGeomCollPtr mbr1, gaiaGeomCollPtr mbr2)
{
/* 
/ checks if two MBRs are disjoint
/
/ returns 1 if TRUE
/ 0 if FALSE
*/
    if (mbr1->MinX > mbr2->MaxX)
	return 1;
    if (mbr1->MinY > mbr2->MaxY)
	return 1;
    if (mbr1->MaxX < mbr2->MinX)
	return 1;
    if (mbr1->MaxY < mbr2->MinY)
	return 1;
    if (mbr2->MinX > mbr1->MaxX)
	return 1;
    if (mbr2->MinY > mbr1->MaxY)
	return 1;
    if (mbr2->MaxX < mbr1->MinX)
	return 1;
    if (mbr2->MaxY < mbr1->MinY)
	return 1;
    return 0;
}

GAIAGEO_DECLARE int
gaiaMbrsTouches (gaiaGeomCollPtr mbr1, gaiaGeomCollPtr mbr2)
{
/* 
/ checks if two MBRs touches
/
/ returns 1 if TRUE
/ 0 if FALSE
*/
    if (mbr1->MinX == mbr2->MinX)
	return 1;
    if (mbr1->MinY == mbr2->MinY)
	return 1;
    if (mbr1->MaxX == mbr2->MaxX)
	return 1;
    if (mbr1->MaxY == mbr2->MaxY)
	return 1;
    return 0;
}

GAIAGEO_DECLARE int
gaiaMbrsIntersects (gaiaGeomCollPtr mbr1, gaiaGeomCollPtr mbr2)
{
/* 
/ checks if two MBRs intersect
/
/ returns 1 if TRUE
/ 0 if FALSE
*/
    if (gaiaMbrsDisjoint (mbr1, mbr2))
	return 0;
    return 1;
}

GAIAGEO_DECLARE int
gaiaMbrsOverlaps (gaiaGeomCollPtr mbr1, gaiaGeomCollPtr mbr2)
{
/* 
/ checks if two MBRs overlap
/
/ returns 1 if TRUE
/ 0 if FALSE
*/
    if (gaiaMbrsDisjoint (mbr1, mbr2))
	return 0;
    if (mbr1->MinX >= mbr2->MinX && mbr1->MinX <= mbr2->MaxX)
	return 1;
    if (mbr1->MaxX >= mbr2->MinX && mbr1->MaxX <= mbr2->MaxX)
	return 1;
    if (mbr1->MinY >= mbr2->MinY && mbr1->MinY <= mbr2->MaxY)
	return 1;
    if (mbr1->MaxY >= mbr2->MinY && mbr1->MaxY <= mbr2->MaxY)
	return 1;
    return 0;
}

GAIAGEO_DECLARE int
gaiaMbrsContains (gaiaGeomCollPtr mbr1, gaiaGeomCollPtr mbr2)
{
/* 
/ checks if MBR-1 completely contains MBR-2
/
/ returns 1 if TRUE
/ 0 if FALSE
*/
    int ok_1 = 0;
    int ok_2 = 0;
    int ok_3 = 0;
    int ok_4 = 0;
    if (mbr2->MinX >= mbr1->MinX && mbr2->MinX <= mbr1->MaxX)
	ok_1 = 1;
    if (mbr2->MaxX >= mbr1->MinX && mbr2->MaxX <= mbr1->MaxX)
	ok_2 = 1;
    if (mbr2->MinY >= mbr1->MinY && mbr2->MinY <= mbr1->MaxY)
	ok_3 = 1;
    if (mbr2->MaxY >= mbr1->MinY && mbr2->MaxY <= mbr1->MaxY)
	ok_4 = 1;
    if (ok_1 && ok_2 && ok_3 && ok_4)
	return 1;
    return 0;
}

GAIAGEO_DECLARE int
gaiaMbrsWithin (gaiaGeomCollPtr mbr1, gaiaGeomCollPtr mbr2)
{
/* 
/ checks if MBR-2 completely contains MBR-1
/
/ returns 1 if TRUE
/ 0 if FALSE
*/
    int ok_1 = 0;
    int ok_2 = 0;
    int ok_3 = 0;
    int ok_4 = 0;
    if (mbr1->MinX >= mbr2->MinX && mbr1->MinX <= mbr2->MaxX)
	ok_1 = 1;
    if (mbr1->MaxX >= mbr2->MinX && mbr1->MaxX <= mbr2->MaxX)
	ok_2 = 1;
    if (mbr1->MinY >= mbr2->MinY && mbr1->MinY <= mbr2->MaxY)
	ok_3 = 1;
    if (mbr1->MaxY >= mbr2->MinY && mbr1->MaxY <= mbr2->MaxY)
	ok_4 = 1;
    if (ok_1 && ok_2 && ok_3 && ok_4)
	return 1;
    return 0;
}

GAIAGEO_DECLARE void
gaiaMakePoint (double x, double y, int srid, unsigned char **result, int *size)
{
/* build a Blob encoded Geometry representing a POINT */
    unsigned char *ptr;
    int endian_arch = gaiaEndianArch ();
/* computing the Blob size and then allocating it */
    *size = 44;			/* header size */
    *size += (sizeof (double) * 2);	/* [x,y] coords */
    *result = malloc (*size);
    ptr = *result;
/* setting the Blob value */
    *ptr = GAIA_MARK_START;	/* START signature */
    *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
    gaiaExport32 (ptr + 2, srid, 1, endian_arch);	/* the SRID */
    gaiaExport64 (ptr + 6, x, 1, endian_arch);	/* MBR - minimum X */
    gaiaExport64 (ptr + 14, y, 1, endian_arch);	/* MBR - minimum Y */
    gaiaExport64 (ptr + 22, x, 1, endian_arch);	/* MBR - maximum X */
    gaiaExport64 (ptr + 30, y, 1, endian_arch);	/* MBR - maximum Y */
    *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
    gaiaExport32 (ptr + 39, GAIA_POINT, 1, endian_arch);	/* class POINT */
    gaiaExport64 (ptr + 43, x, 1, endian_arch);	/* X */
    gaiaExport64 (ptr + 51, y, 1, endian_arch);	/* Y */
    *(ptr + 59) = GAIA_MARK_END;	/* END signature */
}

GAIAGEO_DECLARE void
gaiaMakeLine (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2,
	      unsigned char **result, int *size)
{
/* build a Blob encoded Geometry representing a LINESTRING (segment) */
    int pts;
    int lns;
    int pgs;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaGeomCollPtr g;
    int dims0;
    int dims1;
    int dims;
    double x0;
    double y0;
    double z0;
    double m0;
    double x1;
    double y1;
    double z1;
    double m1;

/* checking if GEOM-1 simply is a POINT */
    if (geom1 == NULL)
      {
	  *result = NULL;
	  *size = 0;
	  return;
      }
    pts = 0;
    lns = 0;
    pgs = 0;
    pt = geom1->FirstPoint;
    while (pt)
      {
	  pts++;
	  x0 = pt->X;
	  y0 = pt->Y;
	  z0 = pt->Z;
	  m0 = pt->M;
	  dims0 = pt->DimensionModel;
	  pt = pt->Next;
      }
    ln = geom1->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    pg = geom1->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (pts == 1 && lns == 0 && pgs == 0)
	;
    else
      {
	  /* failure: not a simple POINT */
	  *result = NULL;
	  *size = 0;
	  return;
      }

/* checking if GEOM-2 simply is a POINT */
    if (geom2 == NULL)
      {
	  *result = NULL;
	  *size = 0;
	  return;
      }
    pts = 0;
    lns = 0;
    pgs = 0;
    pt = geom2->FirstPoint;
    while (pt)
      {
	  pts++;
	  x1 = pt->X;
	  y1 = pt->Y;
	  z1 = pt->Z;
	  m1 = pt->M;
	  dims1 = pt->DimensionModel;
	  pt = pt->Next;
      }
    ln = geom2->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    pg = geom2->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (pts == 1 && lns == 0 && pgs == 0)
	;
    else
      {
	  /* failure: not a simple POINT */
	  *result = NULL;
	  *size = 0;
	  return;
      }

/* building a new Geometry */
    if (dims0 == dims1)
	dims = dims0;
    else
      {
	  if (dims0 == GAIA_XY_Z_M || dims1 == GAIA_XY_Z_M)
	      dims = GAIA_XY_Z_M;
	  else if (dims0 == GAIA_XY_Z && dims1 == GAIA_XY_M)
	      dims = GAIA_XY_Z_M;
	  else if (dims0 == GAIA_XY_M && dims1 == GAIA_XY_Z)
	      dims = GAIA_XY_Z_M;
	  else if (dims0 == GAIA_XY_Z)
	      dims = GAIA_XY_Z;
	  else if (dims1 == GAIA_XY_Z)
	      dims = GAIA_XY_Z;
	  else if (dims0 == GAIA_XY_M)
	      dims = GAIA_XY_M;
	  else if (dims1 == GAIA_XY_M)
	      dims = GAIA_XY_M;
	  else
	      dims = GAIA_XY;
      }
    if (dims == GAIA_XY_Z_M)
	g = gaiaAllocGeomCollXYZM ();
    else if (dims == GAIA_XY_Z)
	g = gaiaAllocGeomCollXYZ ();
    else if (dims == GAIA_XY_M)
	g = gaiaAllocGeomCollXYM ();
    else
	g = gaiaAllocGeomColl ();
    g->Srid = geom1->Srid;
    g->DeclaredType = GAIA_LINESTRING;
    ln = gaiaAddLinestringToGeomColl (g, 2);
    if (dims == GAIA_XY_Z_M)
      {
	  gaiaSetPointXYZM (ln->Coords, 0, x0, y0, z0, m0);
	  gaiaSetPointXYZM (ln->Coords, 1, x1, y1, z1, m1);
      }
    else if (dims == GAIA_XY_Z)
      {
	  gaiaSetPointXYZ (ln->Coords, 0, x0, y0, z0);
	  gaiaSetPointXYZ (ln->Coords, 1, x1, y1, z1);
      }
    else if (dims == GAIA_XY_M)
      {
	  gaiaSetPointXYM (ln->Coords, 0, x0, y0, m0);
	  gaiaSetPointXYM (ln->Coords, 1, x1, y1, m1);
      }
    else
      {
	  gaiaSetPoint (ln->Coords, 0, x0, y0);
	  gaiaSetPoint (ln->Coords, 1, x1, y1);
      }
/* converting to Binary Blob */
    gaiaToSpatiaLiteBlobWkb (g, result, size);
    gaiaFreeGeomColl (g);
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaMergeGeometries (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* mergine two generic Geometries into a single one */
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr new_ln;
    gaiaPolygonPtr pg;
    gaiaPolygonPtr new_pg;
    gaiaRingPtr rng;
    gaiaRingPtr new_rng;
    int dims1;
    int dims2;
    int dims;
    double x;
    double y;
    double z;
    double m;
    int iv;
    int ib;

    if (geom1 == NULL || geom2 == NULL)
	return NULL;
    dims1 = geom1->DimensionModel;
    dims2 = geom2->DimensionModel;
/* building a new Geometry */
    if (dims1 == dims2)
	dims = dims1;
    else
      {
	  if (dims1 == GAIA_XY_Z_M || dims2 == GAIA_XY_Z_M)
	      dims = GAIA_XY_Z_M;
	  else if (dims1 == GAIA_XY_Z && dims2 == GAIA_XY_M)
	      dims = GAIA_XY_Z_M;
	  else if (dims1 == GAIA_XY_M && dims2 == GAIA_XY_Z)
	      dims = GAIA_XY_Z_M;
	  else if (dims1 == GAIA_XY_Z)
	      dims = GAIA_XY_Z;
	  else if (dims2 == GAIA_XY_Z)
	      dims = GAIA_XY_Z;
	  else if (dims1 == GAIA_XY_M)
	      dims = GAIA_XY_M;
	  else if (dims2 == GAIA_XY_M)
	      dims = GAIA_XY_M;
	  else
	      dims = GAIA_XY;
      }
    if (dims == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (dims == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (dims == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    result->Srid = geom1->Srid;

    pt = geom1->FirstPoint;
    while (pt)
      {
	  /* copying POINTs from GEOM-1 */
	  z = 0.0;
	  m = 0.0;
	  if (pt->DimensionModel == GAIA_XY_Z_M)
	    {
		x = pt->X;
		y = pt->Y;
		z = pt->Z;
		m = pt->M;
	    }
	  else if (pt->DimensionModel == GAIA_XY_Z)
	    {
		x = pt->X;
		y = pt->Y;
		z = pt->Z;
	    }
	  else if (pt->DimensionModel == GAIA_XY_M)
	    {
		x = pt->X;
		y = pt->Y;
		m = pt->M;
	    }
	  else
	    {
		x = pt->X;
		y = pt->Y;
	    }
	  if (result->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaAddPointToGeomCollXYZM (result, x, y, z, m);
	    }
	  else if (result->DimensionModel == GAIA_XY_Z)
	    {
		gaiaAddPointToGeomCollXYZ (result, x, y, z);
	    }
	  else if (result->DimensionModel == GAIA_XY_M)
	    {
		gaiaAddPointToGeomCollXYM (result, x, y, m);
	    }
	  else
	    {
		gaiaAddPointToGeomColl (result, x, y);
	    }
	  pt = pt->Next;
      }

    pt = geom2->FirstPoint;
    while (pt)
      {
	  /* copying POINTs from GEOM-2 */
	  z = 0.0;
	  m = 0.0;
	  if (pt->DimensionModel == GAIA_XY_Z_M)
	    {
		x = pt->X;
		y = pt->Y;
		z = pt->Z;
		m = pt->M;
	    }
	  else if (pt->DimensionModel == GAIA_XY_Z)
	    {
		x = pt->X;
		y = pt->Y;
		z = pt->Z;
	    }
	  else if (pt->DimensionModel == GAIA_XY_M)
	    {
		x = pt->X;
		y = pt->Y;
		m = pt->M;
	    }
	  else
	    {
		x = pt->X;
		y = pt->Y;
	    }
	  if (result->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaAddPointToGeomCollXYZM (result, x, y, z, m);
	    }
	  else if (result->DimensionModel == GAIA_XY_Z)
	    {
		gaiaAddPointToGeomCollXYZ (result, x, y, z);
	    }
	  else if (result->DimensionModel == GAIA_XY_M)
	    {
		gaiaAddPointToGeomCollXYM (result, x, y, m);
	    }
	  else
	    {
		gaiaAddPointToGeomColl (result, x, y);
	    }
	  pt = pt->Next;
      }

    ln = geom1->FirstLinestring;
    while (ln)
      {
	  /* copying LINESTRINGs from GEOM-1 */
	  new_ln = gaiaAddLinestringToGeomColl (result, ln->Points);
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		  }
		if (new_ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
		  }
		else
		  {
		      gaiaSetPoint (new_ln->Coords, iv, x, y);
		  }
	    }
	  ln = ln->Next;
      }

    ln = geom2->FirstLinestring;
    while (ln)
      {
	  /* copying LINESTRINGs from GEOM-2 */
	  new_ln = gaiaAddLinestringToGeomColl (result, ln->Points);
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		  }
		if (new_ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
		  }
		else
		  {
		      gaiaSetPoint (new_ln->Coords, iv, x, y);
		  }
	    }
	  ln = ln->Next;
      }

    pg = geom1->FirstPolygon;
    while (pg)
      {
	  /* copying POLYGONs from GEOM-1 */
	  rng = pg->Exterior;
	  new_pg =
	      gaiaAddPolygonToGeomColl (result, rng->Points, pg->NumInteriors);
	  new_rng = new_pg->Exterior;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		/* Exterior Ring */
		z = 0.0;
		m = 0.0;
		if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		  }
		if (new_rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (new_rng->Coords, iv, x, y, z, m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (new_rng->Coords, iv, x, y, z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (new_rng->Coords, iv, x, y, m);
		  }
		else
		  {
		      gaiaSetPoint (new_rng->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		/* Interior Rings */
		rng = pg->Interiors + ib;
		new_rng = gaiaAddInteriorRing (new_pg, ib, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      z = 0.0;
		      m = 0.0;
		      if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      if (new_rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (new_rng->Coords, iv, x, y, z, m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (new_rng->Coords, iv, x, y, z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (new_rng->Coords, iv, x, y, m);
			}
		      else
			{
			    gaiaSetPoint (new_rng->Coords, iv, x, y);
			}
		  }
	    }
	  pg = pg->Next;
      }

    pg = geom2->FirstPolygon;
    while (pg)
      {
	  /* copying POLYGONs from GEOM-2 */
	  rng = pg->Exterior;
	  new_pg =
	      gaiaAddPolygonToGeomColl (result, rng->Points, pg->NumInteriors);
	  new_rng = new_pg->Exterior;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		/* Exterior Ring */
		z = 0.0;
		m = 0.0;
		if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		  }
		if (new_rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (new_rng->Coords, iv, x, y, z, m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (new_rng->Coords, iv, x, y, z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (new_rng->Coords, iv, x, y, m);
		  }
		else
		  {
		      gaiaSetPoint (new_rng->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		/* Interior Rings */
		rng = pg->Interiors + ib;
		new_rng = gaiaAddInteriorRing (new_pg, ib, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      z = 0.0;
		      m = 0.0;
		      if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      if (new_rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (new_rng->Coords, iv, x, y, z, m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (new_rng->Coords, iv, x, y, z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (new_rng->Coords, iv, x, y, m);
			}
		      else
			{
			    gaiaSetPoint (new_rng->Coords, iv, x, y);
			}
		  }
	    }
	  pg = pg->Next;
      }

    return result;
}

GAIAGEO_DECLARE void
gaiaBuildMbr (double x1, double y1, double x2, double y2, int srid,
	      unsigned char **result, int *size)
{
/* build a Blob encoded Geometry representing an MBR */
    unsigned char *ptr;
    double minx;
    double maxx;
    double miny;
    double maxy;
    int endian_arch = gaiaEndianArch ();
/* computing MinMax coords */
    if (x1 > x2)
      {
	  maxx = x1;
	  minx = x2;
      }
    else
      {
	  minx = x1;
	  maxx = x2;
      }
    if (y1 > y2)
      {
	  maxy = y1;
	  miny = y2;
      }
    else
      {
	  miny = y1;
	  maxy = y2;
      }
/* computing the Blob size and then allocating it */
    *size = 44;			/* header size */
    *size += (8 + ((sizeof (double) * 2) * 5));	/* # rings + # points + [x.y] array - exterior ring */
    *result = malloc (*size);
    ptr = *result;
/* setting the Blob value */
    *ptr = GAIA_MARK_START;	/* START signature */
    *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
    gaiaExport32 (ptr + 2, srid, 1, endian_arch);	/* the SRID */
    gaiaExport64 (ptr + 6, minx, 1, endian_arch);	/* MBR - minimum X */
    gaiaExport64 (ptr + 14, miny, 1, endian_arch);	/* MBR - minimum Y */
    gaiaExport64 (ptr + 22, maxx, 1, endian_arch);	/* MBR - maximum X */
    gaiaExport64 (ptr + 30, maxy, 1, endian_arch);	/* MBR - maximum Y */
    *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
    gaiaExport32 (ptr + 39, GAIA_POLYGON, 1, endian_arch);	/* class POLYGON */
    gaiaExport32 (ptr + 43, 1, 1, endian_arch);	/* # rings */
    gaiaExport32 (ptr + 47, 5, 1, endian_arch);	/* # points - exterior ring */
    ptr += 51;
/* setting Envelope points */
    gaiaExport64 (ptr, minx, 1, endian_arch);
    gaiaExport64 (ptr + 8, miny, 1, endian_arch);
    ptr += 16;
    gaiaExport64 (ptr, maxx, 1, endian_arch);
    gaiaExport64 (ptr + 8, miny, 1, endian_arch);
    ptr += 16;
    gaiaExport64 (ptr, maxx, 1, endian_arch);
    gaiaExport64 (ptr + 8, maxy, 1, endian_arch);
    ptr += 16;
    gaiaExport64 (ptr, minx, 1, endian_arch);
    gaiaExport64 (ptr + 8, maxy, 1, endian_arch);
    ptr += 16;
    gaiaExport64 (ptr, minx, 1, endian_arch);
    gaiaExport64 (ptr + 8, miny, 1, endian_arch);
    ptr += 16;
    *ptr = GAIA_MARK_END;	/* END signature */
}

GAIAGEO_DECLARE void
gaiaBuildCircleMbr (double x, double y, double radius, int srid,
		    unsigned char **result, int *size)
{
/* build a Blob encoded Geometry representing an MBR */
    int sz;
    unsigned char *res = NULL;
    double minx = x - radius;
    double maxx = x + radius;
    double miny = y - radius;
    double maxy = y + radius;
    gaiaBuildMbr (minx, miny, maxx, maxy, srid, &res, &sz);
    if (!res)
      {
	  *result = NULL;
	  *size = 0;
      }
    else
      {
	  *result = res;
	  *size = sz;
      }
}

GAIAGEO_DECLARE void
gaiaBuildFilterMbr (double x1, double y1, double x2, double y2, int mode,
		    unsigned char **result, int *size)
{
/* build a filter for an MBR */
    unsigned char *ptr;
    double minx;
    double maxx;
    double miny;
    double maxy;
    int endian_arch = gaiaEndianArch ();
    char filter = GAIA_FILTER_MBR_WITHIN;
    if (mode == GAIA_FILTER_MBR_CONTAINS)
	filter = GAIA_FILTER_MBR_CONTAINS;
    if (mode == GAIA_FILTER_MBR_INTERSECTS)
	filter = GAIA_FILTER_MBR_INTERSECTS;
    if (mode == GAIA_FILTER_MBR_DECLARE)
	filter = GAIA_FILTER_MBR_DECLARE;
/* computing MinMax coords */
    if (x1 > x2)
      {
	  maxx = x1;
	  minx = x2;
      }
    else
      {
	  minx = x1;
	  maxx = x2;
      }
    if (y1 > y2)
      {
	  maxy = y1;
	  miny = y2;
      }
    else
      {
	  miny = y1;
	  maxy = y2;
      }
/* computing the Blob size and then allocating it */
    *size = 37;			/* MBR filter size */
    *result = malloc (*size);
    ptr = *result;
/* setting the Blob value */
    *ptr = filter;		/* signature */
    ptr++;
    gaiaExport64 (ptr, minx, 1, endian_arch);	/* MBR - minimum X */
    ptr += 8;
    *ptr = filter;		/* signature */
    ptr++;
    gaiaExport64 (ptr, miny, 1, endian_arch);	/* MBR - minimum Y */
    ptr += 8;
    *ptr = filter;		/* signature */
    ptr++;
    gaiaExport64 (ptr, maxx, 1, endian_arch);	/* MBR - maximum X */
    ptr += 8;
    *ptr = filter;		/* signature */
    ptr++;
    gaiaExport64 (ptr, maxy, 1, endian_arch);	/* MBR - maximum Y */
    ptr += 8;
    *ptr = filter;		/* signature */
}


GAIAGEO_DECLARE int
gaiaParseFilterMbr (unsigned char *ptr, int size, double *minx, double *miny,
		    double *maxx, double *maxy, int *mode)
{
/* parsing a filter for an MBR */
    char decl_mode;
    int endian_arch = gaiaEndianArch ();
    if (size != 37)
	return 0;		/* cannot be an MBR Filter */
    if (!ptr)
	return 0;		/* cannot be an MBR Filter */
    decl_mode = *(ptr + 0);
    if (decl_mode == GAIA_FILTER_MBR_WITHIN)
	;
    else if (decl_mode == GAIA_FILTER_MBR_CONTAINS)
	;
    else if (decl_mode == GAIA_FILTER_MBR_INTERSECTS)
	;
    else if (decl_mode == GAIA_FILTER_MBR_DECLARE)
	;
    else
	return 0;		/* cannot be an MBR Filter */
    if (*(ptr + 9)
	== decl_mode
	&& *(ptr +
	     18) ==
	decl_mode && *(ptr + 27) == decl_mode && *(ptr + 36) == decl_mode)
	;
    else
	return 0;		/* cannot be an MBR Filter */
    *mode = decl_mode;
    *minx = gaiaImport64 (ptr + 1, 1, endian_arch);
    *miny = gaiaImport64 (ptr + 10, 1, endian_arch);
    *maxx = gaiaImport64 (ptr + 19, 1, endian_arch);
    *maxy = gaiaImport64 (ptr + 28, 1, endian_arch);
    return 1;
}

GAIAGEO_DECLARE int
gaiaGetMbrMinX (const unsigned char *blob, unsigned int size, double *minx)
{
/* returns the MinX coordinate value for a Blob encoded Geometry */
    int little_endian;
    int endian_arch = gaiaEndianArch ();
    if (size < 45)
	return 0;		/* cannot be an internal BLOB WKB geometry */
    if (*(blob + 0) != GAIA_MARK_START)
	return 0;		/* failed to recognize START signature */
    if (*(blob + (size - 1)) != GAIA_MARK_END)
	return 0;		/* failed to recognize END signature */
    if (*(blob + 38) != GAIA_MARK_MBR)
	return 0;		/* failed to recognize MBR signature */
    if (*(blob + 1) == GAIA_LITTLE_ENDIAN)
	little_endian = 1;
    else if (*(blob + 1) == GAIA_BIG_ENDIAN)
	little_endian = 0;
    else
	return 0;		/* unknown encoding; neither little-endian nor big-endian */
    *minx = gaiaImport64 (blob + 6, little_endian, endian_arch);
    return 1;
}

GAIAGEO_DECLARE int
gaiaGetMbrMaxX (const unsigned char *blob, unsigned int size, double *maxx)
{
/* returns the MaxX coordinate value for a Blob encoded Geometry */
    int little_endian;
    int endian_arch = gaiaEndianArch ();
    if (size < 45)
	return 0;		/* cannot be an internal BLOB WKB geometry */
    if (*(blob + 0) != GAIA_MARK_START)
	return 0;		/* failed to recognize START signature */
    if (*(blob + (size - 1)) != GAIA_MARK_END)
	return 0;		/* failed to recognize END signature */
    if (*(blob + 38) != GAIA_MARK_MBR)
	return 0;		/* failed to recognize MBR signature */
    if (*(blob + 1) == GAIA_LITTLE_ENDIAN)
	little_endian = 1;
    else if (*(blob + 1) == GAIA_BIG_ENDIAN)
	little_endian = 0;
    else
	return 0;		/* unknown encoding; neither little-endian nor big-endian */
    *maxx = gaiaImport64 (blob + 22, little_endian, endian_arch);
    return 1;
}

GAIAGEO_DECLARE int
gaiaGetMbrMinY (const unsigned char *blob, unsigned int size, double *miny)
{
/* returns the MinY coordinate value for a Blob encoded Geometry */
    int little_endian;
    int endian_arch = gaiaEndianArch ();
    if (size < 45)
	return 0;		/* cannot be an internal BLOB WKB geometry */
    if (*(blob + 0) != GAIA_MARK_START)
	return 0;		/* failed to recognize START signature */
    if (*(blob + (size - 1)) != GAIA_MARK_END)
	return 0;		/* failed to recognize END signature */
    if (*(blob + 38) != GAIA_MARK_MBR)
	return 0;		/* failed to recognize MBR signature */
    if (*(blob + 1) == GAIA_LITTLE_ENDIAN)
	little_endian = 1;
    else if (*(blob + 1) == GAIA_BIG_ENDIAN)
	little_endian = 0;
    else
	return 0;		/* unknown encoding; neither little-endian nor big-endian */
    *miny = gaiaImport64 (blob + 14, little_endian, endian_arch);
    return 1;
}

GAIAGEO_DECLARE int
gaiaGetMbrMaxY (const unsigned char *blob, unsigned int size, double *maxy)
{
/* returns the MaxY coordinate value for a Blob encoded Geometry */
    int little_endian;
    int endian_arch = gaiaEndianArch ();
    if (size < 45)
	return 0;		/* cannot be an internal BLOB WKB geometry */
    if (*(blob + 0) != GAIA_MARK_START)
	return 0;		/* failed to recognize START signature */
    if (*(blob + (size - 1)) != GAIA_MARK_END)
	return 0;		/* failed to recognize END signature */
    if (*(blob + 38) != GAIA_MARK_MBR)
	return 0;		/* failed to recognize MBR signature */
    if (*(blob + 1) == GAIA_LITTLE_ENDIAN)
	little_endian = 1;
    else if (*(blob + 1) == GAIA_BIG_ENDIAN)
	little_endian = 0;
    else
	return 0;		/* unknown encoding; neither little-endian nor big-endian */
    *maxy = gaiaImport64 (blob + 30, little_endian, endian_arch);
    return 1;
}
/**************** End file: gg_geometries.c **********/


/**************** Begin file: gg_relations.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <float.h> */

#ifndef OMIT_GEOS		/* including GEOS */
/* #include <geos_c.h> */
#endif

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

/* GLOBAL variables */
char gaia_geos_error_msg[2048];
char gaia_geos_warning_msg[2048];

GAIAGEO_DECLARE void
gaiaResetGeosMsg ()
{
/* resets the GEOS error and warning messages */
    *gaia_geos_error_msg = '\0';
    *gaia_geos_warning_msg = '\0';
}

GAIAGEO_DECLARE const char *
gaiaGetGeosErrorMsg ()
{
/* return the latest GEOS error message */
    return gaia_geos_error_msg;
}

GAIAGEO_DECLARE const char *
gaiaGetGeosWarningMsg ()
{
/* return the latest GEOS error message */
    return gaia_geos_warning_msg;
}

GAIAGEO_DECLARE void
gaiaSetGeosErrorMsg (const char *msg)
{
/* return the latest GEOS error message */
    strcpy (gaia_geos_error_msg, msg);
}

GAIAGEO_DECLARE void
gaiaSetGeosWarningMsg (const char *msg)
{
/* return the latest GEOS error message */
    strcpy (gaia_geos_warning_msg, msg);
}

static int
check_point (double *coords, int points, double x, double y)
{
/* checks if [X,Y] point is defined into this coordinate array [Linestring or Ring] */
    int iv;
    double xx;
    double yy;
    for (iv = 0; iv < points; iv++)
      {
	  gaiaGetPoint (coords, iv, &xx, &yy);
	  if (xx == x && yy == y)
	      return 1;
      }
    return 0;
}

GAIAGEO_DECLARE int
gaiaLinestringEquals (gaiaLinestringPtr line1, gaiaLinestringPtr line2)
{
/* checks if two Linestrings are "spatially equal" */
    int iv;
    double x;
    double y;
    if (line1->Points != line2->Points)
	return 0;
    for (iv = 0; iv < line1->Points; iv++)
      {
	  gaiaGetPoint (line1->Coords, iv, &x, &y);
	  if (!check_point (line2->Coords, line2->Points, x, y))
	      return 0;
      }
    return 1;
}

GAIAGEO_DECLARE int
gaiaPolygonEquals (gaiaPolygonPtr polyg1, gaiaPolygonPtr polyg2)
{
/* checks if two Polygons are "spatially equal" */
    int ib;
    int ib2;
    int iv;
    int ok2;
    double x;
    double y;
    gaiaRingPtr ring1;
    gaiaRingPtr ring2;
    if (polyg1->NumInteriors != polyg2->NumInteriors)
	return 0;
/* checking the EXTERIOR RINGs */
    ring1 = polyg1->Exterior;
    ring2 = polyg2->Exterior;
    if (ring1->Points != ring2->Points)
	return 0;
    for (iv = 0; iv < ring1->Points; iv++)
      {
	  gaiaGetPoint (ring1->Coords, iv, &x, &y);
	  if (!check_point (ring2->Coords, ring2->Points, x, y))
	      return 0;
      }
    for (ib = 0; ib < polyg1->NumInteriors; ib++)
      {
	  /* checking the INTERIOR RINGS */
	  int ok = 0;
	  ring1 = polyg1->Interiors + ib;
	  for (ib2 = 0; ib2 < polyg2->NumInteriors; ib2++)
	    {
		ok2 = 1;
		ring2 = polyg2->Interiors + ib2;
		for (iv = 0; iv < ring1->Points; iv++)
		  {
		      gaiaGetPoint (ring1->Coords, iv, &x, &y);
		      if (!check_point (ring2->Coords, ring2->Points, x, y))
			{
			    ok2 = 0;
			    break;
			}
		  }
		if (ok2)
		  {
		      ok = 1;
		      break;
		  }
	    }
	  if (!ok)
	      return 0;
      }
    return 1;
}

#ifndef OMIT_GEOS		/* including GEOS */

GAIAGEO_DECLARE int
gaiaGeomCollEquals (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if two Geometries are "spatially equal" */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSEquals (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollIntersects (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if two Geometries do "spatially intersects" */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSIntersects (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollDisjoint (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if two Geometries are "spatially disjoint" */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSDisjoint (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollOverlaps (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if two Geometries do "spatially overlaps" */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSOverlaps (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollCrosses (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if two Geometries do "spatially crosses" */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSCrosses (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollTouches (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if two Geometries do "spatially touches" */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSTouches (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollWithin (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if GEOM-1 is completely contained within GEOM-2 */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSWithin (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollContains (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if GEOM-1 completely contains GEOM-2 */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSContains (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollRelate (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2,
		    const char *pattern)
{
/* checks if if GEOM-1 and GEOM-2 have a spatial relationship as specified by the pattern Matrix */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSRelatePattern (g1, g2, pattern);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollLength (gaiaGeomCollPtr geom, double *xlength)
{
/* computes the total length for this Geometry */
    double length;
    int ret;
    GEOSGeometry *g;
    if (!geom)
	return 0;
    g = gaiaToGeos (geom);
    ret = GEOSLength (g, &length);
    GEOSGeom_destroy (g);
    if (ret)
	*xlength = length;
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollArea (gaiaGeomCollPtr geom, double *xarea)
{
/* computes the total area for this Geometry */
    double area;
    int ret;
    GEOSGeometry *g;
    if (!geom)
	return 0;
    g = gaiaToGeos (geom);
    ret = GEOSArea (g, &area);
    GEOSGeom_destroy (g);
    if (ret)
	*xarea = area;
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollDistance (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2,
		      double *xdist)
{
/* computes the minimum distance intercurring between GEOM-1 and GEOM-2 */
    double dist;
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return 0;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSDistance (g1, g2, &dist);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (ret)
	*xdist = dist;
    return ret;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaGeometryIntersection (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* builds a new geometry representing the "spatial intersection" of GEOM-1 and GEOM-2 */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    GEOSGeometry *g3;
    if (!geom1 || !geom2)
	return NULL;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    g3 = GEOSIntersection (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (!g3)
	return NULL;
    if (geom1->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g3);
    else if (geom1->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g3);
    else if (geom1->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g3);
    else
	geo = gaiaFromGeos_XY (g3);
    GEOSGeom_destroy (g3);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom1->Srid;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaGeometryUnion (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* builds a new geometry representing the "spatial union" of GEOM-1 and GEOM-2 */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    GEOSGeometry *g3;
    if (!geom1 || !geom2)
	return NULL;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    g3 = GEOSUnion (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (geom1->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g3);
    else if (geom1->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g3);
    else if (geom1->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g3);
    else
	geo = gaiaFromGeos_XY (g3);
    GEOSGeom_destroy (g3);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom1->Srid;
    if (geo->DeclaredType == GAIA_POINT &&
	geom1->DeclaredType == GAIA_MULTIPOINT)
	geo->DeclaredType = GAIA_MULTIPOINT;
    if (geo->DeclaredType == GAIA_LINESTRING &&
	geom1->DeclaredType == GAIA_MULTILINESTRING)
	geo->DeclaredType = GAIA_MULTILINESTRING;
    if (geo->DeclaredType == GAIA_POLYGON &&
	geom1->DeclaredType == GAIA_MULTIPOLYGON)
	geo->DeclaredType = GAIA_MULTIPOLYGON;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaGeometryDifference (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* builds a new geometry representing the "spatial difference" of GEOM-1 and GEOM-2 */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    GEOSGeometry *g3;
    if (!geom1 || !geom2)
	return NULL;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    g3 = GEOSDifference (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (!g3)
	return NULL;
    if (geom1->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g3);
    else if (geom1->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g3);
    else if (geom1->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g3);
    else
	geo = gaiaFromGeos_XY (g3);
    GEOSGeom_destroy (g3);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom1->Srid;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaGeometrySymDifference (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* builds a new geometry representing the "spatial symmetric difference" of GEOM-1 and GEOM-2 */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    GEOSGeometry *g3;
    if (!geom1 || !geom2)
	return NULL;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    g3 = GEOSSymDifference (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (!g3)
	return NULL;
    if (geom1->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g3);
    else if (geom1->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g3);
    else if (geom1->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g3);
    else
	geo = gaiaFromGeos_XY (g3);
    GEOSGeom_destroy (g3);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom1->Srid;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaBoundary (gaiaGeomCollPtr geom)
{
/* builds a new geometry representing the conbinatorial boundary of GEOM */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom)
	return NULL;
    g1 = gaiaToGeos (geom);
    g2 = GEOSBoundary (g1);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom->Srid;
    return geo;
}

GAIAGEO_DECLARE int
gaiaGeomCollCentroid (gaiaGeomCollPtr geom, double *x, double *y)
{
/* returns a Point representing the centroid for this Geometry */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom)
	return 0;
    g1 = gaiaToGeos (geom);
    g2 = GEOSGetCentroid (g1);
    GEOSGeom_destroy (g1);
    if (!g2)
	return 0;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return 0;
    if (geo->FirstPoint)
      {
	  *x = geo->FirstPoint->X;
	  *y = geo->FirstPoint->Y;
	  gaiaFreeGeomColl (geo);
	  return 1;
      }
    gaiaFreeGeomColl (geo);
    return 0;
}

GAIAGEO_DECLARE int
gaiaGetPointOnSurface (gaiaGeomCollPtr geom, double *x, double *y)
{
/* returns a Point guaranteed to lie on the Surface */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom)
	return 0;
    g1 = gaiaToGeos (geom);
    g2 = GEOSPointOnSurface (g1);
    GEOSGeom_destroy (g1);
    if (!g2)
	return 0;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return 0;
    if (geo->FirstPoint)
      {
	  *x = geo->FirstPoint->X;
	  *y = geo->FirstPoint->Y;
	  gaiaFreeGeomColl (geo);
	  return 1;
      }
    gaiaFreeGeomColl (geo);
    return 0;
}

GAIAGEO_DECLARE int
gaiaIsSimple (gaiaGeomCollPtr geom)
{
/* checks if this GEOMETRYCOLLECTION is a simple one */
    int ret;
    GEOSGeometry *g;
    if (!geom)
	return -1;
    if (gaiaIsToxic (geom))
	return 0;
    g = gaiaToGeos (geom);
    ret = GEOSisSimple (g);
    GEOSGeom_destroy (g);
    if (ret == 2)
	return -1;
    return ret;
}

GAIAGEO_DECLARE int
gaiaIsRing (gaiaLinestringPtr line)
{
/* checks if this LINESTRING can be a valid RING */
    gaiaGeomCollPtr geo;
    gaiaLinestringPtr line2;
    int ret;
    int iv;
    double x;
    double y;
    double z;
    double m;
    GEOSGeometry *g;
    if (!line)
	return -1;
    if (line->DimensionModel == GAIA_XY_Z)
	geo = gaiaAllocGeomCollXYZ ();
    else if (line->DimensionModel == GAIA_XY_M)
	geo = gaiaAllocGeomCollXYM ();
    else if (line->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaAllocGeomCollXYZM ();
    else
	geo = gaiaAllocGeomColl ();
    line2 = gaiaAddLinestringToGeomColl (geo, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  z = 0.0;
	  m = 0.0;
	  if (line->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	    }
	  else if (line->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	    }
	  else if (line->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (line->Coords, iv, &x, &y);
	    }
	  if (line2->DimensionModel == GAIA_XY_Z)
	    {
		gaiaSetPointXYZ (line2->Coords, iv, x, y, z);
	    }
	  else if (line2->DimensionModel == GAIA_XY_M)
	    {
		gaiaSetPointXYM (line2->Coords, iv, x, y, m);
	    }
	  else if (line2->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaSetPointXYZM (line2->Coords, iv, x, y, z, m);
	    }
	  else
	    {
		gaiaSetPoint (line2->Coords, iv, x, y);
	    }
      }
    g = gaiaToGeos (geo);
    gaiaFreeGeomColl (geo);
    ret = GEOSisRing (g);
    GEOSGeom_destroy (g);
    if (ret == 2)
	return -1;
    return ret;
}

GAIAGEO_DECLARE int
gaiaIsValid (gaiaGeomCollPtr geom)
{
/* checks if this GEOMETRYCOLLECTION is a valid one */
    int ret;
    GEOSGeometry *g;
    gaiaResetGeosMsg ();
    if (!geom)
	return -1;
    if (gaiaIsToxic (geom))
	return 0;
    g = gaiaToGeos (geom);
    ret = GEOSisValid (g);
    GEOSGeom_destroy (g);
    if (ret == 2)
	return -1;
    return ret;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaGeomCollSimplify (gaiaGeomCollPtr geom, double tolerance)
{
/* builds a simplified geometry using the Douglas-Peuker algorihtm */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom)
	return NULL;
    g1 = gaiaToGeos (geom);
    g2 = GEOSSimplify (g1, tolerance);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom->Srid;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaGeomCollSimplifyPreserveTopology (gaiaGeomCollPtr geom, double tolerance)
{
/* builds a simplified geometry using the Douglas-Peuker algorihtm [preserving topology] */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom)
	return NULL;
    g1 = gaiaToGeos (geom);
    g2 = GEOSTopologyPreserveSimplify (g1, tolerance);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom->Srid;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaConvexHull (gaiaGeomCollPtr geom)
{
/* builds a geometry that is the convex hull of GEOM */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom)
	return NULL;
    g1 = gaiaToGeos (geom);
    g2 = GEOSConvexHull (g1);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom->Srid;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaGeomCollBuffer (gaiaGeomCollPtr geom, double radius, int points)
{
/* builds a geometry that is the GIS buffer of GEOM */
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom)
	return NULL;
    if (gaiaIsToxic (geom))
	return NULL;
    g1 = gaiaToGeos (geom);
    g2 = GEOSBuffer (g1, radius, points);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom->Srid;
    return geo;
}

static void
test_interior_ring (gaiaDynamicLinePtr dyn1, gaiaDynamicLinePtr dyn2,
		    int *contains, int *within, int *crosses)
{
/* testing if Ring-1 contains Ring-2 */
    gaiaGeomCollPtr geom1;
    gaiaGeomCollPtr geom2;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int iv;
    int pts;
    gaiaPointPtr pt;

/* creating the Polygon-1 geometry */
    pts = 0;
    pt = dyn1->First;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    geom1 = gaiaAllocGeomColl ();
    pg = gaiaAddPolygonToGeomColl (geom1, pts, 0);
    rng = pg->Exterior;
    iv = 0;
    pt = dyn1->First;
    while (pt)
      {
	  /* EXTERIOR RING */
	  gaiaSetPoint (rng->Coords, iv, pt->X, pt->Y);
	  iv++;
	  pt = pt->Next;
      }

/* creating the Polygon-2 geometry */
    pts = 0;
    pt = dyn2->First;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    geom2 = gaiaAllocGeomColl ();
    pg = gaiaAddPolygonToGeomColl (geom2, pts, 0);
    rng = pg->Exterior;
    iv = 0;
    pt = dyn2->First;
    while (pt)
      {
	  /* EXTERIOR RING */
	  gaiaSetPoint (rng->Coords, iv, pt->X, pt->Y);
	  iv++;
	  pt = pt->Next;
      }
    *contains = gaiaGeomCollContains (geom1, geom2);
    *within = gaiaGeomCollWithin (geom1, geom2);
    *crosses = gaiaGeomCollCrosses (geom1, geom2);
    gaiaFreeGeomColl (geom1);
    gaiaFreeGeomColl (geom2);
}

static gaiaDynamicLinePtr
build_dyn_ring (gaiaLinestringPtr ln)
{
/* creating a DynamicLine from a Linestring */
    int iv;
    double x;
    double y;
    double m;
    double z;
    gaiaDynamicLinePtr dyn = gaiaAllocDynamicLine ();
    for (iv = 0; iv < ln->Points; iv++)
      {
	  if (ln->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		gaiaAppendPointZMToDynamicLine (dyn, x, y, z, m);
	    }
	  else if (ln->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		gaiaAppendPointZToDynamicLine (dyn, x, y, z);
	    }
	  else if (ln->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		gaiaAppendPointMToDynamicLine (dyn, x, y, m);
	    }
	  else
	    {
		gaiaGetPoint (ln->Coords, iv, &x, &y);
		gaiaAppendPointToDynamicLine (dyn, x, y);
	    }
      }
    return dyn;
}

static int
is_closed_dyn_ring (gaiaDynamicLinePtr dyn)
{
/* checking if a candidate Ring is already closed */
    gaiaPointPtr pt1;
    gaiaPointPtr pt2;
    if (!dyn)
	return 0;
    pt1 = dyn->First;
    pt2 = dyn->Last;
    if (pt1 == NULL || pt2 == NULL)
	return 0;
    if (pt1 == pt2)
	return 0;
    if (pt1->X == pt2->X && pt1->Y == pt2->Y && pt1->Z == pt2->Z)
	return 1;
    return 0;
}

static int
to_be_appended (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln)
{
/* checks is the Linestring has to be appended to the DynamicLine */
    gaiaPointPtr pt = dyn->Last;
    int iv = 0;
    double x;
    double y;
    double z;
    double m;
    if (ln->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
      }
    else if (ln->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
      }
    else if (ln->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
      }
    else
      {
	  gaiaGetPoint (ln->Coords, iv, &x, &y);
      }
    if (ln->DimensionModel == GAIA_XY_Z_M || ln->DimensionModel == GAIA_XY_Z)
      {
	  if (pt->X == x && pt->Y == y && pt->Z == z)
	      return 1;
      }
    else
      {
	  if (pt->X == x && pt->Y == y)
	      return 1;
      }
    return 0;
}

static int
to_be_prepended (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln)
{
/* checks is the Linestring has to be prepended to the DynamicLine */
    gaiaPointPtr pt = dyn->First;
    int iv = ln->Points - 1;
    double x;
    double y;
    double z;
    double m;
    if (ln->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
      }
    else if (ln->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
      }
    else if (ln->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
      }
    else
      {
	  gaiaGetPoint (ln->Coords, iv, &x, &y);
      }
    if (ln->DimensionModel == GAIA_XY_Z_M || ln->DimensionModel == GAIA_XY_Z)
      {
	  if (pt->X == x && pt->Y == y && pt->Z == z)
	      return 1;
      }
    else
      {
	  if (pt->X == x && pt->Y == y)
	      return 1;
      }
    return 0;
}

static int
to_be_appended_reverse (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln)
{
/* checks is the Linestring (reversed) has to be appended to the DynamicLine */
    gaiaPointPtr pt = dyn->Last;
    int iv = ln->Points - 1;
    double x;
    double y;
    double z;
    double m;
    if (ln->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
      }
    else if (ln->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
      }
    else if (ln->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
      }
    else
      {
	  gaiaGetPoint (ln->Coords, iv, &x, &y);
      }
    if (ln->DimensionModel == GAIA_XY_Z_M || ln->DimensionModel == GAIA_XY_Z)
      {
	  if (pt->X == x && pt->Y == y && pt->Z == z)
	      return 1;
      }
    else
      {
	  if (pt->X == x && pt->Y == y)
	      return 1;
      }
    return 0;
}

static int
to_be_prepended_reverse (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln)
{
/* checks is the Linestring (reversed) has to be prepended to the DynamicLine */
    gaiaPointPtr pt = dyn->First;
    int iv = 0;
    double x;
    double y;
    double z;
    double m;
    if (ln->DimensionModel == GAIA_XY_Z_M)
      {
	  gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
      }
    else if (ln->DimensionModel == GAIA_XY_Z)
      {
	  gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
      }
    else if (ln->DimensionModel == GAIA_XY_M)
      {
	  gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
      }
    else
      {
	  gaiaGetPoint (ln->Coords, iv, &x, &y);
      }
    if (ln->DimensionModel == GAIA_XY_Z_M || ln->DimensionModel == GAIA_XY_Z)
      {
	  if (pt->X == x && pt->Y == y && pt->Z == z)
	      return 1;
      }
    else
      {
	  if (pt->X == x && pt->Y == y)
	      return 1;
      }
    return 0;
}

static void
append_to_ring (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln, int reversed)
{
/* appending a Linestring to a DynamicRing */
    int iv;
    double x;
    double y;
    double z;
    double m;
    if (reversed)
      {
	  for (iv = ln->Points - 2; iv >= 0; iv--)
	    {
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      gaiaAppendPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      gaiaAppendPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      gaiaAppendPointMToDynamicLine (dyn, x, y, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      gaiaAppendPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
    else
      {
	  for (iv = 1; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      gaiaAppendPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      gaiaAppendPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      gaiaAppendPointMToDynamicLine (dyn, x, y, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      gaiaAppendPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
}

static void
prepend_to_ring (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln, int reversed)
{
/* appending a Linestring to a DynamicRing */
    int iv;
    double x;
    double y;
    double z;
    double m;
    if (reversed)
      {
	  for (iv = 1; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      gaiaPrependPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      gaiaPrependPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      gaiaPrependPointMToDynamicLine (dyn, x, y, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      gaiaPrependPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
    else
      {
	  for (iv = ln->Points - 2; iv >= 0; iv--)
	    {
		if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      gaiaPrependPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      gaiaPrependPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      gaiaPrependPointMToDynamicLine (dyn, x, y, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      gaiaPrependPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaPolygonize (gaiaGeomCollPtr geom, int force_multi)
{
/* attempts to rearrange a generic Geometry into a (multi)polygon */
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int dummy;
    int ok;
    int ok2;
    int i;
    int i2;
    int iv;
    int ib;
    double x;
    double y;
    double m;
    double z;
    double x0;
    double y0;
    double z0;
    int contains;
    int within;
    int crosses;
    int num_interiors;
    gaiaLinestringPtr *ln_array = NULL;
    gaiaDynamicLinePtr *dyn_array = NULL;
    gaiaDynamicLinePtr *ext_array = NULL;
    gaiaDynamicLinePtr dyn;
    gaiaDynamicLinePtr dyn2;

    if (!geom)
	return NULL;
    pt = geom->FirstPoint;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (pts || pgs)
	return NULL;
    ln = geom->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    if (!lns)
	return NULL;
/* allocating and initializing aux-arrays */
    ln_array = malloc (sizeof (gaiaLinestringPtr) * lns);
    dyn_array = malloc (sizeof (gaiaDynamicLinePtr) * lns);
    ext_array = malloc (sizeof (gaiaDynamicLinePtr) * lns);
    i = 0;
    ln = geom->FirstLinestring;
    while (ln)
      {
	  ln_array[i] = ln;
	  dyn_array[i] = NULL;
	  ext_array[i] = NULL;
	  i++;
	  ln = ln->Next;
      }

    for (i = 0; i < lns; i++)
      {
	  /* processing closed rings */
	  ln = ln_array[i];
	  iv = ln->Points - 1;
	  if (ln->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (ln->Coords, 0, &x0, &y0, &z0, &m);
		gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		if (x0 == x && y0 == y && z0 == z)
		  {
		      dyn_array[i] = build_dyn_ring (ln);
		      ln_array[i] = NULL;
		  }
	    }
	  else if (ln->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (ln->Coords, 0, &x0, &y0, &z0);
		gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		if (x0 == x && y0 == y && z0 == z)
		  {
		      dyn_array[i] = build_dyn_ring (ln);
		      ln_array[i] = NULL;
		  }
	    }
	  else if (ln->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (ln->Coords, 0, &x0, &y0, &m);
		gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		if (x0 == x && y0 == y)
		  {
		      dyn_array[i] = build_dyn_ring (ln);
		      ln_array[i] = NULL;
		  }
	    }
	  else
	    {
		gaiaGetPoint (ln->Coords, 0, &x0, &y0);
		gaiaGetPoint (ln->Coords, iv, &x, &y);
		if (x0 == x && y0 == y)
		  {
		      dyn_array[i] = build_dyn_ring (ln);
		      ln_array[i] = NULL;
		  }
	    }
      }

    ok = 1;
    while (ok)
      {
	  if (dummy == 0)
	      ok = dummy;	/* simply suppressing stupid compiler warnings */
	  ok = 0;
	  for (i = 0; i < lns; i++)
	    {
		/* attempting to create rings */
		ln = ln_array[i];
		if (ln == NULL)
		    continue;
		ok = 1;
		dyn_array[i] = build_dyn_ring (ln);
		ln_array[i] = NULL;
		dyn = dyn_array[i];
		ok2 = 1;
		while (ok2)
		  {
		      ok2 = 0;
		      for (i2 = 0; i2 < lns; i2++)
			{
			    if (is_closed_dyn_ring (dyn) == 1)
				goto ring_done;
			    ln = ln_array[i2];
			    if (ln == NULL)
				continue;
			    if (to_be_appended (dyn, ln) == 1)
			      {
				  append_to_ring (dyn, ln, 0);
				  ln_array[i2] = NULL;
				  ok2 = 1;
				  break;
			      }
			    if (to_be_prepended (dyn, ln) == 1)
			      {
				  prepend_to_ring (dyn, ln, 0);
				  ln_array[i2] = NULL;
				  ok2 = 1;
				  break;
			      }
			    if (to_be_appended_reverse (dyn, ln) == 1)
			      {
				  append_to_ring (dyn, ln, 1);
				  ln_array[i2] = NULL;
				  ok2 = 1;
				  break;
			      }
			    if (to_be_prepended_reverse (dyn, ln) == 1)
			      {
				  prepend_to_ring (dyn, ln, 1);
				  ln_array[i2] = NULL;
				  ok2 = 1;
				  break;
			      }
			}
		  }
	    }
	ring_done:
	  dummy = 0;
      }

    ok = 1;
    for (i = 0; i < lns; i++)
      {
	  /* checking if any ring is closed */
	  dyn = dyn_array[i];
	  if (dyn == NULL)
	      continue;
	  if (is_closed_dyn_ring (dyn) == 0)
	      ok = 0;
      }
    if (ok == 0)
      {
	  /* invalid: quitting */
	  for (i = 0; i < lns; i++)
	    {
		dyn = dyn_array[i];
		if (dyn == NULL)
		    continue;
		gaiaFreeDynamicLine (dyn);
	    }
	  free (dyn_array);
	  free (ext_array);
	  free (ln_array);
	  return NULL;
      }

    ok = 1;
    for (i = 0; i < lns; i++)
      {
	  /* testing interior/exterior relationships */
	  dyn = dyn_array[i];
	  if (dyn == NULL)
	      continue;
	  for (i2 = i + 1; i2 < lns; i2++)
	    {
		/* testing interior/exterior relationships */
		dyn2 = dyn_array[i2];
		if (dyn2 == NULL)
		    continue;
		test_interior_ring (dyn, dyn2, &contains, &within, &crosses);
		if (contains)
		    ext_array[i2] = dyn;
		if (within)
		    ext_array[i] = dyn2;
		if (crosses)
		    ok = 0;
	    }
      }
    if (ok == 0)
      {
	  /* invalid: quitting */
	  for (i = 0; i < lns; i++)
	    {
		dyn = dyn_array[i];
		if (dyn == NULL)
		    continue;
		gaiaFreeDynamicLine (dyn);
	    }
	  free (dyn_array);
	  free (ext_array);
	  free (ln_array);
	  return NULL;
      }

    if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaAllocGeomCollXYZM ();
    else if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaAllocGeomCollXYZ ();
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaAllocGeomCollXYM ();
    else
	result = gaiaAllocGeomColl ();
    result->Srid = geom->Srid;
    if (force_multi)
	result->DeclaredType = GAIA_MULTIPOLYGON;

    for (i = 0; i < lns; i++)
      {
	  /* creating Polygons */
	  dyn = dyn_array[i];
	  if (dyn == NULL)
	      continue;
	  if (ext_array[i] != NULL)
	    {
		/* skipping any INTERIOR RING */
		continue;
	    }
	  pts = 0;
	  pt = dyn->First;
	  while (pt)
	    {
		/* counting how many points are there */
		pts++;
		pt = pt->Next;
	    }
	  num_interiors = 0;
	  for (i2 = 0; i2 < lns; i2++)
	    {
		if (ext_array[i2] == dyn)
		    num_interiors++;
	    }
	  pg = gaiaAddPolygonToGeomColl (result, pts, num_interiors);
	  rng = pg->Exterior;
	  iv = 0;
	  pt = dyn->First;
	  while (pt)
	    {
		/* EXTERIOR RING */
		if (result->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (rng->Coords, iv, pt->X, pt->Y, pt->Z,
					pt->M);
		  }
		else if (result->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (rng->Coords, iv, pt->X, pt->Y, pt->Z);
		  }
		else if (result->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (rng->Coords, iv, pt->X, pt->Y, pt->M);
		  }
		else
		  {
		      gaiaSetPoint (rng->Coords, iv, pt->X, pt->Y);
		  }
		iv++;
		pt = pt->Next;
	    }
	  ib = 0;
	  for (i2 = 0; i2 < lns; i2++)
	    {
		/* inserting any INTERIOR RING */
		if (ext_array[i2] == dyn)
		  {
		      dyn2 = dyn_array[i2];
		      ok = 1;
		      pts = 0;
		      pt = dyn2->First;
		      while (pt)
			{
			    /* counting how many points are there */
			    pts++;
			    pt = pt->Next;
			}
		      rng = gaiaAddInteriorRing (pg, ib, pts);
		      ib++;
		      iv = 0;
		      pt = dyn2->First;
		      while (pt)
			{
			    /* INTERIOR RING */
			    if (result->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaSetPointXYZM (rng->Coords, iv, pt->X,
						    pt->Y, pt->Z, pt->M);
			      }
			    else if (result->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaSetPointXYZ (rng->Coords, iv, pt->X,
						   pt->Y, pt->Z);
			      }
			    else if (result->DimensionModel == GAIA_XY_M)
			      {
				  gaiaSetPointXYM (rng->Coords, iv, pt->X,
						   pt->Y, pt->M);
			      }
			    else
			      {
				  gaiaSetPoint (rng->Coords, iv, pt->X, pt->Y);
			      }
			    iv++;
			    pt = pt->Next;
			}
		  }
	    }
      }

/* memory cleanup */
    for (i = 0; i < lns; i++)
      {
	  dyn = dyn_array[i];
	  if (dyn == NULL)
	      continue;
	  gaiaFreeDynamicLine (dyn);
      }
    free (dyn_array);
    free (ext_array);
    free (ln_array);
    if (result->FirstPolygon == NULL)
      {
	  gaiaFreeGeomColl (result);
	  return NULL;
      }
    return result;
}

#ifdef GEOS_ADVANCED		/* GEOS advanced and experimental features */

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaOffsetCurve (gaiaGeomCollPtr geom, double radius, int points,
		 int left_right)
{
/*
// builds a geometry that is the OffsetCurve of GEOM 
// (which is expected to be of the LINESTRING type)
//
*/
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    int closed = 0;
    if (!geom)
	return NULL;

/* checking the input geometry for validity */
    pt = geom->FirstPoint;
    while (pt)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  if (gaiaIsClosed (ln))
	      closed++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  /* counting how many POLYGON are there */
	  pgs++;
	  pg = pg->Next;
      }
    if (pts > 0 || pgs > 0 || lns > 1 || closed > 0)
	return NULL;

/* all right: this one simply is a LINESTRING */
    geom->DeclaredType = GAIA_LINESTRING;

    g1 = gaiaToGeos (geom);
    g2 = GEOSSingleSidedBuffer (g1, radius, points, GEOSBUF_JOIN_ROUND, 5.0,
				left_right);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom->Srid;
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaSingleSidedBuffer (gaiaGeomCollPtr geom, double radius, int points,
		       int left_right)
{
/*
// builds a geometry that is the SingleSided BUFFER of GEOM 
// (which is expected to be of the LINESTRING type)
//
*/
    gaiaGeomCollPtr geo;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    GEOSBufferParams *params = NULL;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    int closed = 0;
    if (!geom)
	return NULL;

/* checking the input geometry for validity */
    pt = geom->FirstPoint;
    while (pt)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  if (gaiaIsClosed (ln))
	      closed++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  /* counting how many POLYGON are there */
	  pgs++;
	  pg = pg->Next;
      }
    if (pts > 0 || pgs > 0 || lns > 1 || closed > 0)
	return NULL;

/* all right: this one simply is a LINESTRING */
    geom->DeclaredType = GAIA_LINESTRING;

    g1 = gaiaToGeos (geom);
/* setting up Buffer params */
    params = GEOSBufferParams_create ();
    GEOSBufferParams_setJoinStyle (params, GEOSBUF_JOIN_ROUND);
    GEOSBufferParams_setMitreLimit (params, 5.0);
    GEOSBufferParams_setQuadrantSegments (params, points);
    GEOSBufferParams_setSingleSided (params, 1);

/* creating the SingleSided Buffer */
    if (left_right == 0)
      {
	  /* right-sided requires NEGATIVE radius */
	  radius *= -1.0;
      }
    g2 = GEOSBufferWithParams (g1, params, radius);
    GEOSGeom_destroy (g1);
    GEOSBufferParams_destroy (params);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g2);
    else
	geo = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom->Srid;
    return geo;
}

GAIAGEO_DECLARE int
gaiaHausdorffDistance (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2,
		       double *xdist)
{
/* 
/ computes the (discrete) Hausdorff distance intercurring 
/ between GEOM-1 and GEOM-2 
*/
    double dist;
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return 0;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSHausdorffDistance (g1, g2, &dist);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (ret)
	*xdist = dist;
    return ret;
}

static gaiaGeomCollPtr
geom_as_lines (gaiaGeomCollPtr geom)
{
/* transforms a Geometry into a LINESTRING/MULTILINESTRING (if possible) */
    gaiaGeomCollPtr result;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr new_ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;

    if (!geom)
	return NULL;
    if (geom->FirstPoint != NULL)
      {
	  /* invalid: GEOM contains at least one POINT */
	  return NULL;
      }

    switch (geom->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  result = gaiaAllocGeomCollXYZM ();
	  break;
      case GAIA_XY_Z:
	  result = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  result = gaiaAllocGeomCollXYM ();
	  break;
      default:
	  result = gaiaAllocGeomColl ();
	  break;
      };
    result->Srid = geom->Srid;
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* copying any Linestring */
	  new_ln = gaiaAddLinestringToGeomColl (result, ln->Points);
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      gaiaSetPoint (new_ln->Coords, iv, x, y);
		  }
	    }
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  /* copying any Polygon Ring (as Linestring) */
	  rng = pg->Exterior;
	  new_ln = gaiaAddLinestringToGeomColl (result, rng->Points);
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		/* exterior Ring */
		if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      gaiaSetPoint (new_ln->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		new_ln = gaiaAddLinestringToGeomColl (result, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      /* any interior Ring */
		      if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			    gaiaSetPointXYZ (new_ln->Coords, iv, x, y, z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			    gaiaSetPointXYM (new_ln->Coords, iv, x, y, m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			    gaiaSetPointXYZM (new_ln->Coords, iv, x, y, z, m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			    gaiaSetPoint (new_ln->Coords, iv, x, y);
			}
		  }
	    }
	  pg = pg->Next;
      }
    return result;
}

static void
add_shared_linestring (gaiaGeomCollPtr geom, gaiaDynamicLinePtr dyn)
{
/* adding a LINESTRING from Dynamic Line */
    int count = 0;
    gaiaLinestringPtr ln;
    gaiaPointPtr pt;
    int iv;

    if (!geom)
	return;
    if (!dyn)
	return;
    pt = dyn->First;
    while (pt)
      {
	  /* counting how many Points are there */
	  count++;
	  pt = pt->Next;
      }
    if (count == 0)
	return;
    ln = gaiaAddLinestringToGeomColl (geom, count);
    iv = 0;
    pt = dyn->First;
    while (pt)
      {
	  /* copying points into the LINESTRING */
	  if (ln->DimensionModel == GAIA_XY_Z)
	    {
		gaiaSetPointXYZ (ln->Coords, iv, pt->X, pt->Y, pt->Z);
	    }
	  else if (ln->DimensionModel == GAIA_XY_M)
	    {
		gaiaSetPointXYM (ln->Coords, iv, pt->X, pt->Y, pt->M);
	    }
	  else if (ln->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaSetPointXYZM (ln->Coords, iv, pt->X, pt->Y, pt->Z, pt->M);
	    }
	  else
	    {
		gaiaSetPoint (ln->Coords, iv, pt->X, pt->Y);
	    }
	  iv++;
	  pt = pt->Next;
      }
}

static void
append_shared_path (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln, int order)
{
/* appends a Shared Path item to Dynamic Line */
    int iv;
    double x;
    double y;
    double z;
    double m;
    if (order)
      {
	  /* reversed order */
	  for (iv = ln->Points - 1; iv >= 0; iv--)
	    {
		if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      if (x == dyn->Last->X && y == dyn->Last->Y
			  && z == dyn->Last->Z)
			  ;
		      else
			  gaiaAppendPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      if (x == dyn->Last->X && y == dyn->Last->Y
			  && m == dyn->Last->M)
			  ;
		      else
			  gaiaAppendPointMToDynamicLine (dyn, x, y, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      if (x == dyn->Last->X && y == dyn->Last->Y
			  && z == dyn->Last->Z && m == dyn->Last->M)
			  ;
		      else
			  gaiaAppendPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      if (x == dyn->Last->X && y == dyn->Last->Y)
			  ;
		      else
			  gaiaAppendPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
    else
      {
	  /* conformant order */
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      if (x == dyn->Last->X && y == dyn->Last->Y
			  && z == dyn->Last->Z)
			  ;
		      else
			  gaiaAppendPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      if (x == dyn->Last->X && y == dyn->Last->Y
			  && m == dyn->Last->M)
			  ;
		      else
			  gaiaAppendPointMToDynamicLine (dyn, x, y, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      if (x == dyn->Last->X && y == dyn->Last->Y
			  && z == dyn->Last->Z && m == dyn->Last->M)
			  ;
		      else
			  gaiaAppendPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      if (x == dyn->Last->X && y == dyn->Last->Y)
			  ;
		      else
			  gaiaAppendPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
}

static void
prepend_shared_path (gaiaDynamicLinePtr dyn, gaiaLinestringPtr ln, int order)
{
/* prepends a Shared Path item to Dynamic Line */
    int iv;
    double x;
    double y;
    double z;
    double m;
    if (order)
      {
	  /* reversed order */
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      if (x == dyn->First->X && y == dyn->First->Y
			  && z == dyn->First->Z)
			  ;
		      else
			  gaiaPrependPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      if (x == dyn->First->X && y == dyn->First->Y
			  && m == dyn->First->M)
			  ;
		      else
			  gaiaPrependPointMToDynamicLine (dyn, x, y, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      if (x == dyn->First->X && y == dyn->First->Y
			  && z == dyn->First->Z && m == dyn->First->M)
			  ;
		      else
			  gaiaPrependPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      if (x == dyn->First->X && y == dyn->First->Y)
			  ;
		      else
			  gaiaPrependPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
    else
      {
	  /* conformant order */
	  for (iv = ln->Points - 1; iv >= 0; iv--)
	    {
		if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      if (x == dyn->First->X && y == dyn->First->Y
			  && z == dyn->First->Z)
			  ;
		      else
			  gaiaPrependPointZToDynamicLine (dyn, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      if (x == dyn->First->X && y == dyn->First->Y
			  && m == dyn->First->M)
			  ;
		      else
			  gaiaPrependPointMToDynamicLine (dyn, x, y, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      if (x == dyn->First->X && y == dyn->First->Y
			  && z == dyn->First->Z && m == dyn->First->M)
			  ;
		      else
			  gaiaPrependPointZMToDynamicLine (dyn, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      if (x == dyn->First->X && y == dyn->First->Y)
			  ;
		      else
			  gaiaPrependPointToDynamicLine (dyn, x, y);
		  }
	    }
      }
}

static gaiaGeomCollPtr
arrange_shared_paths (gaiaGeomCollPtr geom)
{
/* final aggregation step for shared paths */
    gaiaLinestringPtr ln;
    gaiaLinestringPtr *ln_array;
    gaiaGeomCollPtr result;
    gaiaDynamicLinePtr dyn;
    int count;
    int i;
    int i2;
    int iv;
    double x;
    double y;
    double z;
    double m;
    int ok;
    int ok2;

    if (!geom)
	return NULL;
    count = 0;
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* counting how many Linestrings are there */
	  count++;
	  ln = ln->Next;
      }
    if (count == 0)
	return NULL;

    ln_array = malloc (sizeof (gaiaLinestringPtr) * count);
    i = 0;
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* populating the Linestring references array */
	  ln_array[i++] = ln;
	  ln = ln->Next;
      }

/* allocating a new Geometry [MULTILINESTRING] */
    switch (geom->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  result = gaiaAllocGeomCollXYZM ();
	  break;
      case GAIA_XY_Z:
	  result = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  result = gaiaAllocGeomCollXYM ();
	  break;
      default:
	  result = gaiaAllocGeomColl ();
	  break;
      };
    result->Srid = geom->Srid;
    result->DeclaredType = GAIA_MULTILINESTRING;

    ok = 1;
    while (ok)
      {
	  /* looping until we have processed any input item */
	  ok = 0;
	  for (i = 0; i < count; i++)
	    {
		if (ln_array[i] != NULL)
		  {
		      /* starting a new LINESTRING */
		      dyn = gaiaAllocDynamicLine ();
		      ln = ln_array[i];
		      ln_array[i] = NULL;
		      for (iv = 0; iv < ln->Points; iv++)
			{
			    /* inserting the 'seed' path */
			    if (ln->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
				  gaiaAppendPointZToDynamicLine (dyn, x, y, z);
			      }
			    else if (ln->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
				  gaiaAppendPointMToDynamicLine (dyn, x, y, m);
			      }
			    else if (ln->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z,
						    &m);
				  gaiaAppendPointZMToDynamicLine (dyn, x, y, z,
								  m);
			      }
			    else
			      {
				  gaiaGetPoint (ln->Coords, iv, &x, &y);
				  gaiaAppendPointToDynamicLine (dyn, x, y);
			      }
			}
		      ok2 = 1;
		      while (ok2)
			{
			    /* looping until we have checked any other item */
			    ok2 = 0;
			    for (i2 = 0; i2 < count; i2++)
			      {
				  /* expanding the 'seed' path */
				  if (ln_array[i2] == NULL)
				      continue;
				  ln = ln_array[i2];
				  /* checking the first vertex */
				  iv = 0;
				  if (ln->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ln->Coords, iv, &x, &y,
							 &z);
				    }
				  else if (ln->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ln->Coords, iv, &x, &y,
							 &m);
				    }
				  else if (ln->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ln->Coords, iv, &x,
							  &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ln->Coords, iv, &x, &y);
				    }
				  if (x == dyn->Last->X && y == dyn->Last->Y)
				    {
					/* appending this item to the 'seed' (conformant order) */
					append_shared_path (dyn, ln, 0);
					ln_array[i2] = NULL;
					ok2 = 1;
					continue;
				    }
				  if (x == dyn->First->X && y == dyn->First->Y)
				    {
					/* prepending this item to the 'seed' (reversed order) */
					prepend_shared_path (dyn, ln, 1);
					ln_array[i2] = NULL;
					ok2 = 1;
					continue;
				    }
				  /* checking the last vertex */
				  iv = ln->Points - 1;
				  if (ln->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ln->Coords, iv, &x, &y,
							 &z);
				    }
				  else if (ln->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ln->Coords, iv, &x, &y,
							 &m);
				    }
				  else if (ln->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ln->Coords, iv, &x,
							  &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ln->Coords, iv, &x, &y);
				    }
				  if (x == dyn->Last->X && y == dyn->Last->Y)
				    {
					/* appending this item to the 'seed' (reversed order) */
					append_shared_path (dyn, ln, 1);
					ln_array[i2] = NULL;
					ok2 = 1;
					continue;
				    }
				  if (x == dyn->First->X && y == dyn->First->Y)
				    {
					/* prepending this item to the 'seed' (conformant order) */
					prepend_shared_path (dyn, ln, 0);
					ln_array[i2] = NULL;
					ok2 = 1;
					continue;
				    }
			      }
			}
		      add_shared_linestring (result, dyn);
		      gaiaFreeDynamicLine (dyn);
		      ok = 1;
		      break;
		  }
	    }
      }
    free (ln_array);
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaSharedPaths (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/*
// builds a geometry containing Shared Paths commons to GEOM1 & GEOM2 
// (which are expected to be of the LINESTRING/MULTILINESTRING type)
//
*/
    gaiaGeomCollPtr geo;
    gaiaGeomCollPtr result;
    gaiaGeomCollPtr line1;
    gaiaGeomCollPtr line2;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    GEOSGeometry *g3;
    if (!geom1)
	return NULL;
    if (!geom2)
	return NULL;
/* transforming input geoms as Lines */
    line1 = geom_as_lines (geom1);
    line2 = geom_as_lines (geom2);
    if (line1 == NULL || line2 == NULL)
      {
	  if (line1)
	      gaiaFreeGeomColl (line1);
	  if (line2)
	      gaiaFreeGeomColl (line2);
	  return NULL;
      }

    g1 = gaiaToGeos (line1);
    g2 = gaiaToGeos (line2);
    gaiaFreeGeomColl (line1);
    gaiaFreeGeomColl (line2);
    g3 = GEOSSharedPaths (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (!g3)
	return NULL;
    if (geom1->DimensionModel == GAIA_XY_Z)
	geo = gaiaFromGeos_XYZ (g3);
    else if (geom1->DimensionModel == GAIA_XY_M)
	geo = gaiaFromGeos_XYM (g3);
    else if (geom1->DimensionModel == GAIA_XY_Z_M)
	geo = gaiaFromGeos_XYZM (g3);
    else
	geo = gaiaFromGeos_XY (g3);
    GEOSGeom_destroy (g3);
    if (geo == NULL)
	return NULL;
    geo->Srid = geom1->Srid;
    result = arrange_shared_paths (geo);
    gaiaFreeGeomColl (geo);
    return result;
}

GAIAGEO_DECLARE int
gaiaGeomCollCovers (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if geom1 "spatially covers" geom2 */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSCovers (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE int
gaiaGeomCollCoveredBy (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* checks if geom1 is "spatially covered by" geom2 */
    int ret;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1;
    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    ret = GEOSCoveredBy (g1, g2);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return ret;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaLineInterpolatePoint (gaiaGeomCollPtr geom, double fraction)
{
/*
 * attempts to intepolate a point on line at dist "fraction" 
 *
 * the fraction is expressed into the range from 0.0 to 1.0
 */
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    GEOSGeometry *g;
    GEOSGeometry *g_pt;
    double length;
    double projection;
    if (!geom)
	return NULL;

/* checking if a single Linestring has been passed */
    pt = geom->FirstPoint;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (pts == 0 && lns == 1 && pgs == 0)
	;
    else
	return NULL;

    g = gaiaToGeos (geom);
    if (GEOSLength (g, &length))
      {
	  /* transforming fraction to length */
	  if (fraction < 0.0)
	      fraction = 0.0;
	  if (fraction > 1.0)
	      fraction = 1.0;
	  projection = length * fraction;
      }
    else
      {
	  GEOSGeom_destroy (g);
	  return NULL;
      }
    g_pt = GEOSInterpolate (g, projection);
    GEOSGeom_destroy (g);
    if (!g_pt)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaFromGeos_XYZ (g_pt);
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaFromGeos_XYM (g_pt);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaFromGeos_XYZM (g_pt);
    else
	result = gaiaFromGeos_XY (g_pt);
    GEOSGeom_destroy (g_pt);
    if (result == NULL)
	return NULL;
    result->Srid = geom->Srid;
    return result;
}

GAIAGEO_DECLARE double
gaiaLineLocatePoint (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* 
 * attempts to compute the location of the closest point on LineString 
 * to the given Point, as a fraction of total 2d line length 
 *
 * the fraction is expressed into the range from 0.0 to 1.0
 */
    int pts1 = 0;
    int lns1 = 0;
    int pgs1 = 0;
    int pts2 = 0;
    int lns2 = 0;
    int pgs2 = 0;
    double length;
    double projection;
    double result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    if (!geom1 || !geom2)
	return -1.0;

/* checking if a single Linestring has been passed */
    pt = geom1->FirstPoint;
    while (pt)
      {
	  pts1++;
	  pt = pt->Next;
      }
    ln = geom1->FirstLinestring;
    while (ln)
      {
	  lns1++;
	  ln = ln->Next;
      }
    pg = geom1->FirstPolygon;
    while (pg)
      {
	  pgs1++;
	  pg = pg->Next;
      }
    if (pts1 == 0 && lns1 >= 1 && pgs1 == 0)
	;
    else
	return -1.0;

/* checking if a single Point has been passed */
    pt = geom2->FirstPoint;
    while (pt)
      {
	  pts2++;
	  pt = pt->Next;
      }
    ln = geom2->FirstLinestring;
    while (ln)
      {
	  lns2++;
	  ln = ln->Next;
      }
    pg = geom2->FirstPolygon;
    while (pg)
      {
	  pgs2++;
	  pg = pg->Next;
      }
    if (pts2 == 1 && lns2 == 0 && pgs2 == 0)
	;
    else
	return -1.0;

    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    projection = GEOSProject (g1, g2);
    if (GEOSLength (g1, &length))
      {
	  /* normalizing as a fraction between 0.0 and 1.0 */
	  result = projection / length;
      }
    else
	result = -1.0;
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaLineSubstring (gaiaGeomCollPtr geom, double start_fraction,
		   double end_fraction)
{
/* 
 * attempts to build a new Linestring being a substring of the input one starting 
 * and ending at the given fractions of total 2d length 
 */
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaGeomCollPtr result;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr out;
    gaiaPolygonPtr pg;
    GEOSGeometry *g;
    GEOSGeometry *g_start;
    GEOSGeometry *g_end;
    GEOSCoordSequence *cs;
    const GEOSCoordSequence *in_cs;
    GEOSGeometry *segm;
    double length;
    double total = 0.0;
    double start;
    double end;
    int iv;
    int i_start = -1;
    int i_end = -1;
    int points;
    double x;
    double y;
    double z;
    double m;
    unsigned int dims;
    if (!geom)
	return NULL;

/* checking if a single Linestring has been passed */
    pt = geom->FirstPoint;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (pts == 0 && lns == 1 && pgs == 0)
	;
    else
	return NULL;

    if (start_fraction < 0.0)
	start_fraction = 0.0;
    if (start_fraction > 1.0)
	start_fraction = 1.0;
    if (end_fraction < 0.0)
	end_fraction = 0.0;
    if (end_fraction > 1.0)
	end_fraction = 1.0;
    if (start_fraction >= end_fraction)
	return NULL;
    g = gaiaToGeos (geom);
    if (GEOSLength (g, &length))
      {
	  start = length * start_fraction;
	  end = length * end_fraction;
      }
    else
      {
	  GEOSGeom_destroy (g);
	  return NULL;
      }
    g_start = GEOSInterpolate (g, start);
    g_end = GEOSInterpolate (g, end);
    GEOSGeom_destroy (g);
    if (!g_start || !g_end)
	return NULL;

/* identifying first and last valid vertex */
    ln = geom->FirstLinestring;
    for (iv = 0; iv < ln->Points; iv++)
      {

	  double x0;
	  double y0;
	  switch (ln->DimensionModel)
	    {
	    case GAIA_XY_Z:
		gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		break;
	    case GAIA_XY_M:
		gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		break;
	    case GAIA_XY_Z_M:
		gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		break;
	    default:
		gaiaGetPoint (ln->Coords, iv, &x, &y);
		break;
	    };

	  if (iv > 0)
	    {
		cs = GEOSCoordSeq_create (2, 2);
		GEOSCoordSeq_setX (cs, 0, x0);
		GEOSCoordSeq_setY (cs, 0, y0);
		GEOSCoordSeq_setX (cs, 1, x);
		GEOSCoordSeq_setY (cs, 1, y);
		segm = GEOSGeom_createLineString (cs);
		GEOSLength (segm, &length);
		total += length;
		GEOSGeom_destroy (segm);
		if (total > start && i_start < 0)
		    i_start = iv;
		if (total < end)
		    i_end = iv;
	    }
	  x0 = x;
	  y0 = y;
      }
    if (i_start < 0 || i_end < 0)
      {
	  i_start = -1;
	  i_end = -1;
	  points = 2;
      }
    else
	points = i_end - i_start + 3;

/* creating the output geometry */
    switch (ln->DimensionModel)
      {
      case GAIA_XY_Z:
	  result = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  result = gaiaAllocGeomCollXYM ();
	  break;
      case GAIA_XY_Z_M:
	  result = gaiaAllocGeomCollXYZM ();
	  break;
      default:
	  result = gaiaAllocGeomColl ();
	  break;
      };
    result->Srid = geom->Srid;
    out = gaiaAddLinestringToGeomColl (result, points);

/* start vertex */
    points = 0;
    in_cs = GEOSGeom_getCoordSeq (g_start);
    GEOSCoordSeq_getDimensions (in_cs, &dims);
    if (dims == 3)
      {
	  GEOSCoordSeq_getX (in_cs, 0, &x);
	  GEOSCoordSeq_getY (in_cs, 0, &y);
	  GEOSCoordSeq_getZ (in_cs, 0, &z);
	  m = 0.0;
      }
    else
      {
	  GEOSCoordSeq_getX (in_cs, 0, &x);
	  GEOSCoordSeq_getY (in_cs, 0, &y);
	  z = 0.0;
	  m = 0.0;
      }
    GEOSGeom_destroy (g_start);
    switch (out->DimensionModel)
      {
      case GAIA_XY_Z:
	  gaiaSetPointXYZ (out->Coords, points, x, y, z);
	  break;
      case GAIA_XY_M:
	  gaiaSetPointXYM (out->Coords, points, x, y, 0.0);
	  break;
      case GAIA_XY_Z_M:
	  gaiaSetPointXYZM (out->Coords, points, x, y, z, 0.0);
	  break;
      default:
	  gaiaSetPoint (out->Coords, points, x, y);
	  break;
      };
    points++;

    if (i_start < 0 || i_end < 0)
	;
    else
      {
	  for (iv = i_start; iv <= i_end; iv++)
	    {
		z = 0.0;
		m = 0.0;
		switch (ln->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      break;
		  case GAIA_XY_M:
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      break;
		  default:
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      break;
		  };
		switch (out->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaSetPointXYZ (out->Coords, points, x, y, z);
		      break;
		  case GAIA_XY_M:
		      gaiaSetPointXYM (out->Coords, points, x, y, 0.0);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaSetPointXYZM (out->Coords, points, x, y, z, 0.0);
		      break;
		  default:
		      gaiaSetPoint (out->Coords, points, x, y);
		      break;
		  };
		points++;
	    }
      }

/* end vertex */
    in_cs = GEOSGeom_getCoordSeq (g_end);
    GEOSCoordSeq_getDimensions (in_cs, &dims);
    if (dims == 3)
      {
	  GEOSCoordSeq_getX (in_cs, 0, &x);
	  GEOSCoordSeq_getY (in_cs, 0, &y);
	  GEOSCoordSeq_getZ (in_cs, 0, &z);
	  m = 0.0;
      }
    else
      {
	  GEOSCoordSeq_getX (in_cs, 0, &x);
	  GEOSCoordSeq_getY (in_cs, 0, &y);
	  z = 0.0;
	  m = 0.0;
      }
    GEOSGeom_destroy (g_end);
    switch (out->DimensionModel)
      {
      case GAIA_XY_Z:
	  gaiaSetPointXYZ (out->Coords, points, x, y, z);
	  break;
      case GAIA_XY_M:
	  gaiaSetPointXYM (out->Coords, points, x, y, 0.0);
	  break;
      case GAIA_XY_Z_M:
	  gaiaSetPointXYZM (out->Coords, points, x, y, z, 0.0);
	  break;
      default:
	  gaiaSetPoint (out->Coords, points, x, y);
	  break;
      };
    return result;
}

static GEOSGeometry *
buildGeosPoints (const gaiaGeomCollPtr gaia)
{
/* converting a GAIA Geometry into a GEOS Geometry of POINTS */
    int pts = 0;
    unsigned int dims;
    int iv;
    int ib;
    int nItem;
    double x;
    double y;
    double z;
    double m;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    GEOSGeometry *geos;
    GEOSGeometry *geos_item;
    GEOSGeometry **geos_coll;
    GEOSCoordSequence *cs;
    if (!gaia)
	return NULL;
    pt = gaia->FirstPoint;
    while (pt)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  pt = pt->Next;
      }
    ln = gaia->FirstLinestring;
    while (ln)
      {
	  /* counting how many POINTs are there */
	  pts += ln->Points;
	  ln = ln->Next;
      }
    pg = gaia->FirstPolygon;
    while (pg)
      {
	  /* counting how many POINTs are there */
	  rng = pg->Exterior;
	  pts += rng->Points - 1;	/* exterior ring */
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		/* interior ring */
		rng = pg->Interiors + ib;
		pts += rng->Points - 1;
	    }
	  pg = pg->Next;
      }
    if (pts == 0)
	return NULL;
    switch (gaia->DimensionModel)
      {
      case GAIA_XY_Z:
      case GAIA_XY_Z_M:
	  dims = 3;
	  break;
      default:
	  dims = 2;
	  break;
      };
    nItem = 0;
    geos_coll = malloc (sizeof (GEOSGeometry *) * (pts));
    pt = gaia->FirstPoint;
    while (pt)
      {
	  cs = GEOSCoordSeq_create (1, dims);
	  switch (pt->DimensionModel)
	    {
	    case GAIA_XY_Z:
	    case GAIA_XY_Z_M:
		GEOSCoordSeq_setX (cs, 0, pt->X);
		GEOSCoordSeq_setY (cs, 0, pt->Y);
		GEOSCoordSeq_setZ (cs, 0, pt->Z);
		break;
	    default:
		GEOSCoordSeq_setX (cs, 0, pt->X);
		GEOSCoordSeq_setY (cs, 0, pt->Y);
		break;
	    };
	  geos_item = GEOSGeom_createPoint (cs);
	  *(geos_coll + nItem++) = geos_item;
	  pt = pt->Next;
      }
    ln = gaia->FirstLinestring;
    while (ln)
      {
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		switch (ln->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      break;
		  case GAIA_XY_M:
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      break;
		  default:
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      break;
		  };
		cs = GEOSCoordSeq_create (1, dims);
		if (dims == 3)
		  {
		      GEOSCoordSeq_setX (cs, 0, x);
		      GEOSCoordSeq_setY (cs, 0, y);
		      GEOSCoordSeq_setZ (cs, 0, z);
		  }
		else
		  {
		      GEOSCoordSeq_setX (cs, 0, x);
		      GEOSCoordSeq_setY (cs, 0, y);
		  }
		geos_item = GEOSGeom_createPoint (cs);
		*(geos_coll + nItem++) = geos_item;
	    }
	  ln = ln->Next;
      }
    pg = gaia->FirstPolygon;
    while (pg)
      {
	  rng = pg->Exterior;
	  for (iv = 1; iv < rng->Points; iv++)
	    {
		/* exterior ring */
		z = 0.0;
		m = 0.0;
		switch (rng->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      break;
		  case GAIA_XY_M:
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      break;
		  default:
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      break;
		  };
		cs = GEOSCoordSeq_create (1, dims);
		if (dims == 3)
		  {
		      GEOSCoordSeq_setX (cs, 0, x);
		      GEOSCoordSeq_setY (cs, 0, y);
		      GEOSCoordSeq_setZ (cs, 0, z);
		  }
		else
		  {
		      GEOSCoordSeq_setX (cs, 0, x);
		      GEOSCoordSeq_setY (cs, 0, y);
		  }
		geos_item = GEOSGeom_createPoint (cs);
		*(geos_coll + nItem++) = geos_item;
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		/* interior ring */
		rng = pg->Interiors + ib;
		for (iv = 1; iv < rng->Points; iv++)
		  {
		      /* exterior ring */
		      z = 0.0;
		      m = 0.0;
		      switch (rng->DimensionModel)
			{
			case GAIA_XY_Z:
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			    break;
			case GAIA_XY_M:
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			    break;
			case GAIA_XY_Z_M:
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			    break;
			default:
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			    break;
			};
		      cs = GEOSCoordSeq_create (1, dims);
		      if (dims == 3)
			{
			    GEOSCoordSeq_setX (cs, 0, x);
			    GEOSCoordSeq_setY (cs, 0, y);
			    GEOSCoordSeq_setZ (cs, 0, z);
			}
		      else
			{
			    GEOSCoordSeq_setX (cs, 0, x);
			    GEOSCoordSeq_setY (cs, 0, y);
			}
		      geos_item = GEOSGeom_createPoint (cs);
		      *(geos_coll + nItem++) = geos_item;
		  }
	    }
	  pg = pg->Next;
      }
    geos = GEOSGeom_createCollection (GEOS_MULTIPOINT, geos_coll, pts);
    free (geos_coll);
    GEOSSetSRID (geos, gaia->Srid);
    return geos;
}

static GEOSGeometry *
buildGeosSegments (const gaiaGeomCollPtr gaia)
{
/* converting a GAIA Geometry into a GEOS Geometry of SEGMENTS */
    int segms = 0;
    unsigned int dims;
    int iv;
    int ib;
    int nItem;
    double x;
    double y;
    double z;
    double m;
    double x0;
    double y0;
    double z0;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    GEOSGeometry *geos;
    GEOSGeometry *geos_item;
    GEOSGeometry **geos_coll;
    GEOSCoordSequence *cs;
    if (!gaia)
	return NULL;
    ln = gaia->FirstLinestring;
    while (ln)
      {
	  /* counting how many SEGMENTs are there */
	  segms += ln->Points - 1;
	  ln = ln->Next;
      }
    pg = gaia->FirstPolygon;
    while (pg)
      {
	  /* counting how many SEGMENTs are there */
	  rng = pg->Exterior;
	  segms += rng->Points - 1;	/* exterior ring */
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		/* interior ring */
		rng = pg->Interiors + ib;
		segms += rng->Points - 1;
	    }
	  pg = pg->Next;
      }
    if (segms == 0)
	return NULL;
    switch (gaia->DimensionModel)
      {
      case GAIA_XY_Z:
      case GAIA_XY_Z_M:
	  dims = 3;
	  break;
      default:
	  dims = 2;
	  break;
      };
    nItem = 0;
    geos_coll = malloc (sizeof (GEOSGeometry *) * (segms));
    ln = gaia->FirstLinestring;
    while (ln)
      {
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		switch (ln->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      break;
		  case GAIA_XY_M:
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      break;
		  default:
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      break;
		  };
		if (iv > 0)
		  {
		      cs = GEOSCoordSeq_create (2, dims);
		      if (dims == 3)
			{
			    GEOSCoordSeq_setX (cs, 0, x0);
			    GEOSCoordSeq_setY (cs, 0, y0);
			    GEOSCoordSeq_setZ (cs, 0, z0);
			    GEOSCoordSeq_setX (cs, 1, x);
			    GEOSCoordSeq_setY (cs, 1, y);
			    GEOSCoordSeq_setZ (cs, 1, z);
			}
		      else
			{
			    GEOSCoordSeq_setX (cs, 0, x0);
			    GEOSCoordSeq_setY (cs, 0, y0);
			    GEOSCoordSeq_setX (cs, 1, x);
			    GEOSCoordSeq_setY (cs, 1, y);
			}
		      geos_item = GEOSGeom_createLineString (cs);
		      *(geos_coll + nItem++) = geos_item;
		  }
		x0 = x;
		y0 = y;
		z0 = z;
	    }
	  ln = ln->Next;
      }
    pg = gaia->FirstPolygon;
    while (pg)
      {
	  rng = pg->Exterior;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		/* exterior ring */
		z = 0.0;
		m = 0.0;
		switch (rng->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      break;
		  case GAIA_XY_M:
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      break;
		  default:
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      break;
		  };
		if (iv > 0)
		  {
		      cs = GEOSCoordSeq_create (2, dims);
		      if (dims == 3)
			{
			    GEOSCoordSeq_setX (cs, 0, x0);
			    GEOSCoordSeq_setY (cs, 0, y0);
			    GEOSCoordSeq_setZ (cs, 0, z0);
			    GEOSCoordSeq_setX (cs, 1, x);
			    GEOSCoordSeq_setY (cs, 1, y);
			    GEOSCoordSeq_setZ (cs, 1, z);
			}
		      else
			{
			    GEOSCoordSeq_setX (cs, 0, x0);
			    GEOSCoordSeq_setY (cs, 0, y0);
			    GEOSCoordSeq_setX (cs, 1, x);
			    GEOSCoordSeq_setY (cs, 1, y);
			}
		      geos_item = GEOSGeom_createLineString (cs);
		      *(geos_coll + nItem++) = geos_item;
		  }
		x0 = x;
		y0 = y;
		z0 = z;
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		/* interior ring */
		rng = pg->Interiors + ib;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      /* exterior ring */
		      z = 0.0;
		      m = 0.0;
		      switch (rng->DimensionModel)
			{
			case GAIA_XY_Z:
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			    break;
			case GAIA_XY_M:
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			    break;
			case GAIA_XY_Z_M:
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			    break;
			default:
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			    break;
			};
		      if (iv > 0)
			{
			    cs = GEOSCoordSeq_create (2, dims);
			    if (dims == 3)
			      {
				  GEOSCoordSeq_setX (cs, 0, x0);
				  GEOSCoordSeq_setY (cs, 0, y0);
				  GEOSCoordSeq_setZ (cs, 0, z0);
				  GEOSCoordSeq_setX (cs, 1, x);
				  GEOSCoordSeq_setY (cs, 1, y);
				  GEOSCoordSeq_setZ (cs, 1, z);
			      }
			    else
			      {
				  GEOSCoordSeq_setX (cs, 0, x0);
				  GEOSCoordSeq_setY (cs, 0, y0);
				  GEOSCoordSeq_setX (cs, 1, x);
				  GEOSCoordSeq_setY (cs, 1, y);
			      }
			    geos_item = GEOSGeom_createLineString (cs);
			    *(geos_coll + nItem++) = geos_item;
			}
		      x0 = x;
		      y0 = y;
		      z0 = z;
		  }
	    }
	  pg = pg->Next;
      }
    geos = GEOSGeom_createCollection (GEOS_MULTILINESTRING, geos_coll, segms);
    free (geos_coll);
    GEOSSetSRID (geos, gaia->Srid);
    return geos;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaShortestLine (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2)
{
/* attempts to compute the the shortest line between two geometries */
    GEOSGeometry *g1_points;
    GEOSGeometry *g1_segments;
    const GEOSGeometry *g1_item;
    GEOSGeometry *g2_points;
    GEOSGeometry *g2_segments;
    const GEOSGeometry *g2_item;
    const GEOSCoordSequence *cs;
    GEOSGeometry *g_pt;
    gaiaGeomCollPtr result;
    gaiaLinestringPtr ln;
    int nItems1;
    int nItems2;
    int it1;
    int it2;
    unsigned int dims;
    double x_ini;
    double y_ini;
    double z_ini;
    double x_fin;
    double y_fin;
    double z_fin;
    double dist;
    double min_dist = DBL_MAX;
    double projection;
    if (!geom1 || !geom2)
	return NULL;

    g1_points = buildGeosPoints (geom1);
    g1_segments = buildGeosSegments (geom1);
    g2_points = buildGeosPoints (geom2);
    g2_segments = buildGeosSegments (geom2);

    if (g1_points && g2_points)
      {
	  /* computing distances between POINTs */
	  nItems1 = GEOSGetNumGeometries (g1_points);
	  nItems2 = GEOSGetNumGeometries (g2_points);
	  for (it1 = 0; it1 < nItems1; it1++)
	    {
		g1_item = GEOSGetGeometryN (g1_points, it1);
		for (it2 = 0; it2 < nItems2; it2++)
		  {
		      g2_item = GEOSGetGeometryN (g2_points, it2);
		      if (GEOSDistance (g1_item, g2_item, &dist))
			{
			    if (dist < min_dist)
			      {
				  /* saving min-dist points */
				  min_dist = dist;
				  cs = GEOSGeom_getCoordSeq (g1_item);
				  GEOSCoordSeq_getDimensions (cs, &dims);
				  if (dims == 3)
				    {
					GEOSCoordSeq_getX (cs, 0, &x_ini);
					GEOSCoordSeq_getY (cs, 0, &y_ini);
					GEOSCoordSeq_getZ (cs, 0, &z_ini);
				    }
				  else
				    {
					GEOSCoordSeq_getX (cs, 0, &x_ini);
					GEOSCoordSeq_getY (cs, 0, &y_ini);
					z_ini = 0.0;
				    }
				  cs = GEOSGeom_getCoordSeq (g2_item);
				  GEOSCoordSeq_getDimensions (cs, &dims);
				  if (dims == 3)
				    {
					GEOSCoordSeq_getX (cs, 0, &x_fin);
					GEOSCoordSeq_getY (cs, 0, &y_fin);
					GEOSCoordSeq_getZ (cs, 0, &z_fin);
				    }
				  else
				    {
					GEOSCoordSeq_getX (cs, 0, &x_fin);
					GEOSCoordSeq_getY (cs, 0, &y_fin);
					z_fin = 0.0;
				    }
			      }
			}
		  }
	    }
      }

    if (g1_points && g2_segments)
      {
	  /* computing distances between POINTs (g1) and SEGMENTs (g2) */
	  nItems1 = GEOSGetNumGeometries (g1_points);
	  nItems2 = GEOSGetNumGeometries (g2_segments);
	  for (it1 = 0; it1 < nItems1; it1++)
	    {
		g1_item = GEOSGetGeometryN (g1_points, it1);
		for (it2 = 0; it2 < nItems2; it2++)
		  {
		      g2_item = GEOSGetGeometryN (g2_segments, it2);
		      if (GEOSDistance (g1_item, g2_item, &dist))
			{
			    if (dist < min_dist)
			      {
				  /* saving min-dist points */
				  projection = GEOSProject (g2_item, g1_item);
				  g_pt = GEOSInterpolate (g2_item, projection);
				  if (g_pt)
				    {
					min_dist = dist;
					cs = GEOSGeom_getCoordSeq (g1_item);
					GEOSCoordSeq_getDimensions (cs, &dims);
					if (dims == 3)
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_ini);
					      GEOSCoordSeq_getY (cs, 0, &y_ini);
					      GEOSCoordSeq_getZ (cs, 0, &z_ini);
					  }
					else
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_ini);
					      GEOSCoordSeq_getY (cs, 0, &y_ini);
					      z_ini = 0.0;
					  }
					cs = GEOSGeom_getCoordSeq (g_pt);
					GEOSCoordSeq_getDimensions (cs, &dims);
					if (dims == 3)
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_fin);
					      GEOSCoordSeq_getY (cs, 0, &y_fin);
					      GEOSCoordSeq_getZ (cs, 0, &z_fin);
					  }
					else
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_fin);
					      GEOSCoordSeq_getY (cs, 0, &y_fin);
					      z_fin = 0.0;
					  }
					GEOSGeom_destroy (g_pt);
				    }
			      }
			}
		  }
	    }
      }

    if (g1_segments && g2_points)
      {
	  /* computing distances between SEGMENTs (g1) and POINTs (g2) */
	  nItems1 = GEOSGetNumGeometries (g1_segments);
	  nItems2 = GEOSGetNumGeometries (g2_points);
	  for (it1 = 0; it1 < nItems1; it1++)
	    {
		g1_item = GEOSGetGeometryN (g1_segments, it1);
		for (it2 = 0; it2 < nItems2; it2++)
		  {
		      g2_item = GEOSGetGeometryN (g2_points, it2);
		      if (GEOSDistance (g1_item, g2_item, &dist))
			{
			    if (dist < min_dist)
			      {
				  /* saving min-dist points */
				  projection = GEOSProject (g1_item, g2_item);
				  g_pt = GEOSInterpolate (g1_item, projection);
				  if (g_pt)
				    {
					min_dist = dist;
					cs = GEOSGeom_getCoordSeq (g_pt);
					GEOSCoordSeq_getDimensions (cs, &dims);
					if (dims == 3)
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_ini);
					      GEOSCoordSeq_getY (cs, 0, &y_ini);
					      GEOSCoordSeq_getZ (cs, 0, &z_ini);
					  }
					else
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_ini);
					      GEOSCoordSeq_getY (cs, 0, &y_ini);
					      z_ini = 0.0;
					  }
					cs = GEOSGeom_getCoordSeq (g2_item);
					GEOSCoordSeq_getDimensions (cs, &dims);
					if (dims == 3)
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_fin);
					      GEOSCoordSeq_getY (cs, 0, &y_fin);
					      GEOSCoordSeq_getZ (cs, 0, &z_fin);
					  }
					else
					  {
					      GEOSCoordSeq_getX (cs, 0, &x_fin);
					      GEOSCoordSeq_getY (cs, 0, &y_fin);
					      z_fin = 0.0;
					  }
					GEOSGeom_destroy (g_pt);
				    }
			      }
			}
		  }
	    }
      }
    if (g1_points)
	GEOSGeom_destroy (g1_points);
    if (g1_segments)
	GEOSGeom_destroy (g1_segments);
    if (g2_points)
	GEOSGeom_destroy (g2_points);
    if (g2_segments)
	GEOSGeom_destroy (g2_segments);
    if (min_dist == DBL_MAX || min_dist <= 0.0)
	return NULL;

/* building the shortest line */
    switch (geom1->DimensionModel)
      {
      case GAIA_XY_Z:
	  result = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  result = gaiaAllocGeomCollXYM ();
	  break;
      case GAIA_XY_Z_M:
	  result = gaiaAllocGeomCollXYZM ();
	  break;
      default:
	  result = gaiaAllocGeomColl ();
	  break;
      };
    result->Srid = geom1->Srid;
    ln = gaiaAddLinestringToGeomColl (result, 2);
    switch (ln->DimensionModel)
      {
      case GAIA_XY_Z:
	  gaiaSetPointXYZ (ln->Coords, 0, x_ini, y_ini, z_ini);
	  gaiaSetPointXYZ (ln->Coords, 1, x_fin, y_fin, z_fin);
	  break;
      case GAIA_XY_M:
	  gaiaSetPointXYM (ln->Coords, 0, x_ini, y_ini, 0.0);
	  gaiaSetPointXYM (ln->Coords, 1, x_fin, y_fin, 0.0);
	  break;
      case GAIA_XY_Z_M:
	  gaiaSetPointXYZM (ln->Coords, 0, x_ini, y_ini, z_ini, 0.0);
	  gaiaSetPointXYZM (ln->Coords, 1, x_fin, y_fin, z_fin, 0.0);
	  break;
      default:
	  gaiaSetPoint (ln->Coords, 0, x_ini, y_ini);
	  gaiaSetPoint (ln->Coords, 1, x_fin, y_fin);
	  break;
      };
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaSnap (gaiaGeomCollPtr geom1, gaiaGeomCollPtr geom2, double tolerance)
{
/* attempts to "snap" geom1 on geom2 using the given tolerance */
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    GEOSGeometry *g3;
    gaiaGeomCollPtr result;
    if (!geom1 || !geom2)
	return NULL;

    g1 = gaiaToGeos (geom1);
    g2 = gaiaToGeos (geom2);
    g3 = GEOSSnap (g1, g2, tolerance);
    GEOSGeom_destroy (g1);
    GEOSGeom_destroy (g2);
    if (!g3)
	return NULL;
    if (geom1->DimensionModel == GAIA_XY_Z)
	result = gaiaFromGeos_XYZ (g3);
    else if (geom1->DimensionModel == GAIA_XY_M)
	result = gaiaFromGeos_XYM (g3);
    else if (geom1->DimensionModel == GAIA_XY_Z_M)
	result = gaiaFromGeos_XYZM (g3);
    else
	result = gaiaFromGeos_XY (g3);
    GEOSGeom_destroy (g3);
    if (result == NULL)
	return NULL;
    result->Srid = geom1->Srid;
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaLineMerge (gaiaGeomCollPtr geom)
{
/* attempts to reassemble lines from a collection of sparse fragments */
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    gaiaGeomCollPtr result;
    if (!geom)
	return NULL;
    if (gaiaIsToxic (geom))
	return NULL;

    g1 = gaiaToGeos (geom);
    g2 = GEOSLineMerge (g1);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaFromGeos_XYZM (g2);
    else
	result = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (result == NULL)
	return NULL;
    result->Srid = geom->Srid;
    return result;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaUnaryUnion (gaiaGeomCollPtr geom)
{
/* Unary Union (single Collection) */
    GEOSGeometry *g1;
    GEOSGeometry *g2;
    gaiaGeomCollPtr result;
    if (!geom)
	return NULL;
    if (gaiaIsToxic (geom))
	return NULL;
    g1 = gaiaToGeos (geom);
    g2 = GEOSUnaryUnion (g1);
    GEOSGeom_destroy (g1);
    if (!g2)
	return NULL;
    if (geom->DimensionModel == GAIA_XY_Z)
	result = gaiaFromGeos_XYZ (g2);
    else if (geom->DimensionModel == GAIA_XY_M)
	result = gaiaFromGeos_XYM (g2);
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	result = gaiaFromGeos_XYZM (g2);
    else
	result = gaiaFromGeos_XY (g2);
    GEOSGeom_destroy (g2);
    if (result == NULL)
	return NULL;
    result->Srid = geom->Srid;
    return result;
}

#endif /* end GEOS advanced and experimental features */

#endif /* end including GEOS */
/**************** End file: gg_relations.c **********/


/**************** Begin file: gg_geoscvt.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */

#ifndef OMIT_GEOS		/* including GEOS */
/* #include <geos_c.h> */
#endif

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

#ifndef OMIT_GEOS		/* including GEOS */

static GEOSGeometry *
toGeosGeometry (const gaiaGeomCollPtr gaia)
{
/* converting a GAIA Geometry into a GEOS Geometry */
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    int type;
    int geos_type;
    unsigned int dims;
    int iv;
    int ib;
    int nItem;
    double x;
    double y;
    double z;
    double m;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    GEOSGeometry *geos;
    GEOSGeometry *geos_ext;
    GEOSGeometry *geos_int;
    GEOSGeometry *geos_item;
    GEOSGeometry **geos_holes;
    GEOSGeometry **geos_coll;
    GEOSCoordSequence *cs;
    if (!gaia)
	return NULL;
    pt = gaia->FirstPoint;
    while (pt)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  pt = pt->Next;
      }
    ln = gaia->FirstLinestring;
    while (ln)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  ln = ln->Next;
      }
    pg = gaia->FirstPolygon;
    while (pg)
      {
	  /* counting how many POLYGONs are there */
	  pgs++;
	  pg = pg->Next;
      }
    if (pts == 0 && lns == 0 && pgs == 0)
	type = GAIA_UNKNOWN;
    else if (pts == 1 && lns == 0 && pgs == 0)
      {
	  if (gaia->DeclaredType == GAIA_MULTIPOINT)
	      type = GAIA_MULTIPOINT;
	  else if (gaia->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_POINT;
      }
    else if (pts == 0 && lns == 1 && pgs == 0)
      {
	  if (gaia->DeclaredType == GAIA_MULTILINESTRING)
	      type = GAIA_MULTILINESTRING;
	  else if (gaia->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_LINESTRING;
      }
    else if (pts == 0 && lns == 0 && pgs == 1)
      {
	  if (gaia->DeclaredType == GAIA_MULTIPOLYGON)
	      type = GAIA_MULTIPOLYGON;
	  else if (gaia->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_POLYGON;
      }
    else if (pts > 1 && lns == 0 && pgs == 0)
      {
	  if (gaia->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTIPOINT;
      }
    else if (pts == 0 && lns > 1 && pgs == 0)
      {
	  if (gaia->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTILINESTRING;
      }
    else if (pts == 0 && lns == 0 && pgs > 1)
      {
	  if (gaia->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTIPOLYGON;
      }
    else
	type = GAIA_GEOMETRYCOLLECTION;
    switch (gaia->DimensionModel)
      {
      case GAIA_XY_Z:
      case GAIA_XY_Z_M:
	  dims = 3;
	  break;
      default:
	  dims = 2;
	  break;
      };
    switch (type)
      {
      case GAIA_POINT:
	  pt = gaia->FirstPoint;
	  cs = GEOSCoordSeq_create (1, dims);
	  switch (gaia->DimensionModel)
	    {
	    case GAIA_XY_Z:
	    case GAIA_XY_Z_M:
		GEOSCoordSeq_setX (cs, 0, pt->X);
		GEOSCoordSeq_setY (cs, 0, pt->Y);
		GEOSCoordSeq_setZ (cs, 0, pt->Z);
		break;
	    default:
		GEOSCoordSeq_setX (cs, 0, pt->X);
		GEOSCoordSeq_setY (cs, 0, pt->Y);
		break;
	    };
	  geos = GEOSGeom_createPoint (cs);
	  break;
      case GAIA_LINESTRING:
	  ln = gaia->FirstLinestring;
	  cs = GEOSCoordSeq_create (ln->Points, dims);
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		switch (ln->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      GEOSCoordSeq_setZ (cs, iv, z);
		      break;
		  case GAIA_XY_M:
		      gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      GEOSCoordSeq_setZ (cs, iv, z);
		      break;
		  default:
		      gaiaGetPoint (ln->Coords, iv, &x, &y);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      break;
		  };
	    }
	  geos = GEOSGeom_createLineString (cs);
	  break;
      case GAIA_POLYGON:
	  pg = gaia->FirstPolygon;
	  rng = pg->Exterior;
	  /* exterior ring */
	  cs = GEOSCoordSeq_create (rng->Points, dims);
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		switch (rng->DimensionModel)
		  {
		  case GAIA_XY_Z:
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      GEOSCoordSeq_setZ (cs, iv, z);
		      break;
		  case GAIA_XY_M:
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      break;
		  case GAIA_XY_Z_M:
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      GEOSCoordSeq_setZ (cs, iv, z);
		      break;
		  default:
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      GEOSCoordSeq_setX (cs, iv, x);
		      GEOSCoordSeq_setY (cs, iv, y);
		      break;
		  };
	    }
	  geos_ext = GEOSGeom_createLinearRing (cs);
	  geos_holes = NULL;
	  if (pg->NumInteriors > 0)
	    {
		geos_holes =
		    malloc (sizeof (GEOSGeometry *) * pg->NumInteriors);
		for (ib = 0; ib < pg->NumInteriors; ib++)
		  {
		      /* interior ring */
		      rng = pg->Interiors + ib;
		      cs = GEOSCoordSeq_create (rng->Points, dims);
		      for (iv = 0; iv < rng->Points; iv++)
			{
			    switch (rng->DimensionModel)
			      {
			      case GAIA_XY_Z:
				  gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
				  GEOSCoordSeq_setX (cs, iv, x);
				  GEOSCoordSeq_setY (cs, iv, y);
				  GEOSCoordSeq_setZ (cs, iv, z);
				  break;
			      case GAIA_XY_M:
				  gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
				  GEOSCoordSeq_setX (cs, iv, x);
				  GEOSCoordSeq_setY (cs, iv, y);
				  break;
			      case GAIA_XY_Z_M:
				  gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z,
						    &m);
				  GEOSCoordSeq_setX (cs, iv, x);
				  GEOSCoordSeq_setY (cs, iv, y);
				  GEOSCoordSeq_setZ (cs, iv, z);
				  break;
			      default:
				  gaiaGetPoint (rng->Coords, iv, &x, &y);
				  GEOSCoordSeq_setX (cs, iv, x);
				  GEOSCoordSeq_setY (cs, iv, y);
				  break;
			      };
			}
		      geos_int = GEOSGeom_createLinearRing (cs);
		      *(geos_holes + ib) = geos_int;
		  }
	    }
	  geos =
	      GEOSGeom_createPolygon (geos_ext, geos_holes, pg->NumInteriors);
	  if (geos_holes)
	      free (geos_holes);
	  break;
      case GAIA_MULTIPOINT:
      case GAIA_MULTILINESTRING:
      case GAIA_MULTIPOLYGON:
      case GAIA_GEOMETRYCOLLECTION:
	  nItem = 0;
	  geos_coll = malloc (sizeof (GEOSGeometry *) * (pts + lns + pgs));
	  pt = gaia->FirstPoint;
	  while (pt)
	    {
		cs = GEOSCoordSeq_create (1, dims);
		switch (pt->DimensionModel)
		  {
		  case GAIA_XY_Z:
		  case GAIA_XY_Z_M:
		      GEOSCoordSeq_setX (cs, 0, pt->X);
		      GEOSCoordSeq_setY (cs, 0, pt->Y);
		      GEOSCoordSeq_setZ (cs, 0, pt->Z);
		      break;
		  default:
		      GEOSCoordSeq_setX (cs, 0, pt->X);
		      GEOSCoordSeq_setY (cs, 0, pt->Y);
		      break;
		  };
		geos_item = GEOSGeom_createPoint (cs);
		*(geos_coll + nItem++) = geos_item;
		pt = pt->Next;
	    }
	  ln = gaia->FirstLinestring;
	  while (ln)
	    {
		cs = GEOSCoordSeq_create (ln->Points, dims);
		for (iv = 0; iv < ln->Points; iv++)
		  {
		      switch (ln->DimensionModel)
			{
			case GAIA_XY_Z:
			    gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    GEOSCoordSeq_setZ (cs, iv, z);
			    break;
			case GAIA_XY_M:
			    gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    break;
			case GAIA_XY_Z_M:
			    gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    GEOSCoordSeq_setZ (cs, iv, z);
			    break;
			default:
			    gaiaGetPoint (ln->Coords, iv, &x, &y);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    break;
			};
		  }
		geos_item = GEOSGeom_createLineString (cs);
		*(geos_coll + nItem++) = geos_item;
		ln = ln->Next;
	    }
	  pg = gaia->FirstPolygon;
	  while (pg)
	    {
		rng = pg->Exterior;
		/* exterior ring */
		cs = GEOSCoordSeq_create (rng->Points, dims);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      switch (rng->DimensionModel)
			{
			case GAIA_XY_Z:
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    GEOSCoordSeq_setZ (cs, iv, z);
			    break;
			case GAIA_XY_M:
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    break;
			case GAIA_XY_Z_M:
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    GEOSCoordSeq_setZ (cs, iv, z);
			    break;
			default:
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			    GEOSCoordSeq_setX (cs, iv, x);
			    GEOSCoordSeq_setY (cs, iv, y);
			    break;
			};
		  }
		geos_ext = GEOSGeom_createLinearRing (cs);
		geos_holes = NULL;
		if (pg->NumInteriors > 0)
		  {
		      geos_holes =
			  malloc (sizeof (GEOSGeometry *) * pg->NumInteriors);
		      for (ib = 0; ib < pg->NumInteriors; ib++)
			{
			    /* interior ring */
			    rng = pg->Interiors + ib;
			    cs = GEOSCoordSeq_create (rng->Points, dims);
			    for (iv = 0; iv < rng->Points; iv++)
			      {
				  switch (rng->DimensionModel)
				    {
				    case GAIA_XY_Z:
					gaiaGetPointXYZ (rng->Coords, iv, &x,
							 &y, &z);
					GEOSCoordSeq_setX (cs, iv, x);
					GEOSCoordSeq_setY (cs, iv, y);
					GEOSCoordSeq_setZ (cs, iv, z);
					break;
				    case GAIA_XY_M:
					gaiaGetPointXYM (rng->Coords, iv, &x,
							 &y, &m);
					GEOSCoordSeq_setX (cs, iv, x);
					GEOSCoordSeq_setY (cs, iv, y);
					break;
				    case GAIA_XY_Z_M:
					gaiaGetPointXYZM (rng->Coords, iv, &x,
							  &y, &z, &m);
					GEOSCoordSeq_setX (cs, iv, x);
					GEOSCoordSeq_setY (cs, iv, y);
					GEOSCoordSeq_setZ (cs, iv, z);
					break;
				    default:
					gaiaGetPoint (rng->Coords, iv, &x, &y);
					GEOSCoordSeq_setX (cs, iv, x);
					GEOSCoordSeq_setY (cs, iv, y);
					break;
				    };
			      }
			    geos_int = GEOSGeom_createLinearRing (cs);
			    *(geos_holes + ib) = geos_int;
			}
		  }
		geos_item =
		    GEOSGeom_createPolygon (geos_ext, geos_holes,
					    pg->NumInteriors);
		if (geos_holes)
		    free (geos_holes);
		*(geos_coll + nItem++) = geos_item;
		pg = pg->Next;
	    }
	  geos_type = GEOS_GEOMETRYCOLLECTION;
	  if (type == GAIA_MULTIPOINT)
	      geos_type = GEOS_MULTIPOINT;
	  if (type == GAIA_MULTILINESTRING)
	      geos_type = GEOS_MULTILINESTRING;
	  if (type == GAIA_MULTIPOLYGON)
	      geos_type = GEOS_MULTIPOLYGON;
	  geos =
	      GEOSGeom_createCollection (geos_type, geos_coll, pts + lns + pgs);
	  if (geos_coll)
	      free (geos_coll);
	  break;
      default:
	  geos = NULL;
      };
    if (geos)
	GEOSSetSRID (geos, gaia->Srid);
    return geos;
}

static gaiaGeomCollPtr
fromGeosGeometry (const GEOSGeometry * geos, const int dimension_model)
{
/* converting a GEOS Geometry into a GAIA Geometry */
    int type;
    int itemType;
    unsigned int dims;
    int iv;
    int ib;
    int it;
    int sub_it;
    int nItems;
    int nSubItems;
    int holes;
    unsigned int points;
    double x;
    double y;
    double z;
    const GEOSCoordSequence *cs;
    const GEOSGeometry *geos_ring;
    const GEOSGeometry *geos_item;
    const GEOSGeometry *geos_sub_item;
    gaiaGeomCollPtr gaia = NULL;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    if (!geos)
	return NULL;
    type = GEOSGeomTypeId (geos);
    switch (type)
      {
      case GEOS_POINT:
	  if (dimension_model == GAIA_XY_Z)
	      gaia = gaiaAllocGeomCollXYZ ();
	  else if (dimension_model == GAIA_XY_M)
	      gaia = gaiaAllocGeomCollXYM ();
	  else if (dimension_model == GAIA_XY_Z_M)
	      gaia = gaiaAllocGeomCollXYZM ();
	  else
	      gaia = gaiaAllocGeomColl ();
	  gaia->DeclaredType = GAIA_POINT;
	  gaia->Srid = GEOSGetSRID (geos);
	  cs = GEOSGeom_getCoordSeq (geos);
	  GEOSCoordSeq_getDimensions (cs, &dims);
	  if (dims == 3)
	    {
		GEOSCoordSeq_getX (cs, 0, &x);
		GEOSCoordSeq_getY (cs, 0, &y);
		GEOSCoordSeq_getZ (cs, 0, &z);
	    }
	  else
	    {
		GEOSCoordSeq_getX (cs, 0, &x);
		GEOSCoordSeq_getY (cs, 0, &y);
		z = 0.0;
	    }
	  if (dimension_model == GAIA_XY_Z)
	      gaiaAddPointToGeomCollXYZ (gaia, x, y, z);
	  else if (dimension_model == GAIA_XY_M)
	      gaiaAddPointToGeomCollXYM (gaia, x, y, 0.0);
	  else if (dimension_model == GAIA_XY_Z_M)
	      gaiaAddPointToGeomCollXYZM (gaia, x, y, z, 0.0);
	  else
	      gaiaAddPointToGeomColl (gaia, x, y);
	  break;
      case GEOS_LINESTRING:
	  if (dimension_model == GAIA_XY_Z)
	      gaia = gaiaAllocGeomCollXYZ ();
	  else if (dimension_model == GAIA_XY_M)
	      gaia = gaiaAllocGeomCollXYM ();
	  else if (dimension_model == GAIA_XY_Z_M)
	      gaia = gaiaAllocGeomCollXYZM ();
	  else
	      gaia = gaiaAllocGeomColl ();
	  gaia->DeclaredType = GAIA_LINESTRING;
	  gaia->Srid = GEOSGetSRID (geos);
	  cs = GEOSGeom_getCoordSeq (geos);
	  GEOSCoordSeq_getDimensions (cs, &dims);
	  GEOSCoordSeq_getSize (cs, &points);
	  ln = gaiaAddLinestringToGeomColl (gaia, points);
	  for (iv = 0; iv < (int) points; iv++)
	    {
		if (dims == 3)
		  {
		      GEOSCoordSeq_getX (cs, iv, &x);
		      GEOSCoordSeq_getY (cs, iv, &y);
		      GEOSCoordSeq_getZ (cs, iv, &z);
		  }
		else
		  {
		      GEOSCoordSeq_getX (cs, iv, &x);
		      GEOSCoordSeq_getY (cs, iv, &y);
		      z = 0.0;
		  }
		if (dimension_model == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (ln->Coords, iv, x, y, z);
		  }
		else if (dimension_model == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (ln->Coords, iv, x, y, 0.0);
		  }
		else if (dimension_model == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (ln->Coords, iv, x, y, z, 0.0);
		  }
		else
		  {
		      gaiaSetPoint (ln->Coords, iv, x, y);
		  }
	    }
	  break;
      case GEOS_POLYGON:
	  if (dimension_model == GAIA_XY_Z)
	      gaia = gaiaAllocGeomCollXYZ ();
	  else if (dimension_model == GAIA_XY_M)
	      gaia = gaiaAllocGeomCollXYM ();
	  else if (dimension_model == GAIA_XY_Z_M)
	      gaia = gaiaAllocGeomCollXYZM ();
	  else
	      gaia = gaiaAllocGeomColl ();
	  gaia->DeclaredType = GAIA_POLYGON;
	  gaia->Srid = GEOSGetSRID (geos);
	  /* exterior ring */
	  holes = GEOSGetNumInteriorRings (geos);
	  geos_ring = GEOSGetExteriorRing (geos);
	  cs = GEOSGeom_getCoordSeq (geos_ring);
	  GEOSCoordSeq_getDimensions (cs, &dims);
	  GEOSCoordSeq_getSize (cs, &points);
	  pg = gaiaAddPolygonToGeomColl (gaia, points, holes);
	  rng = pg->Exterior;
	  for (iv = 0; iv < (int) points; iv++)
	    {
		if (dims == 3)
		  {
		      GEOSCoordSeq_getX (cs, iv, &x);
		      GEOSCoordSeq_getY (cs, iv, &y);
		      GEOSCoordSeq_getZ (cs, iv, &z);
		  }
		else
		  {
		      GEOSCoordSeq_getX (cs, iv, &x);
		      GEOSCoordSeq_getY (cs, iv, &y);
		      z = 0.0;
		  }
		if (dimension_model == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (rng->Coords, iv, x, y, z);
		  }
		else if (dimension_model == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (rng->Coords, iv, x, y, 0.0);
		  }
		else if (dimension_model == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (rng->Coords, iv, x, y, z, 0.0);
		  }
		else
		  {
		      gaiaSetPoint (rng->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < holes; ib++)
	    {
		/* interior rings */
		geos_ring = GEOSGetInteriorRingN (geos, ib);
		cs = GEOSGeom_getCoordSeq (geos_ring);
		GEOSCoordSeq_getDimensions (cs, &dims);
		GEOSCoordSeq_getSize (cs, &points);
		rng = gaiaAddInteriorRing (pg, ib, points);
		for (iv = 0; iv < (int) points; iv++)
		  {
		      if (dims == 3)
			{
			    GEOSCoordSeq_getX (cs, iv, &x);
			    GEOSCoordSeq_getY (cs, iv, &y);
			    GEOSCoordSeq_getZ (cs, iv, &z);
			}
		      else
			{
			    GEOSCoordSeq_getX (cs, iv, &x);
			    GEOSCoordSeq_getY (cs, iv, &y);
			    z = 0.0;
			}
		      if (dimension_model == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (rng->Coords, iv, x, y, z);
			}
		      else if (dimension_model == GAIA_XY_M)
			{
			    gaiaSetPointXYM (rng->Coords, iv, x, y, 0.0);
			}
		      else if (dimension_model == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (rng->Coords, iv, x, y, z, 0.0);
			}
		      else
			{
			    gaiaSetPoint (rng->Coords, iv, x, y);
			}
		  }
	    }
	  break;
      case GEOS_MULTIPOINT:
      case GEOS_MULTILINESTRING:
      case GEOS_MULTIPOLYGON:
      case GEOS_GEOMETRYCOLLECTION:
	  if (dimension_model == GAIA_XY_Z)
	      gaia = gaiaAllocGeomCollXYZ ();
	  else if (dimension_model == GAIA_XY_M)
	      gaia = gaiaAllocGeomCollXYM ();
	  else if (dimension_model == GAIA_XY_Z_M)
	      gaia = gaiaAllocGeomCollXYZM ();
	  else
	      gaia = gaiaAllocGeomColl ();
	  if (type == GEOS_MULTIPOINT)
	      gaia->DeclaredType = GAIA_MULTIPOINT;
	  else if (type == GEOS_MULTILINESTRING)
	      gaia->DeclaredType = GAIA_MULTILINESTRING;
	  else if (type == GEOS_MULTIPOLYGON)
	      gaia->DeclaredType = GAIA_MULTIPOLYGON;
	  else
	      gaia->DeclaredType = GAIA_GEOMETRYCOLLECTION;
	  gaia->Srid = GEOSGetSRID (geos);
	  nItems = GEOSGetNumGeometries (geos);
	  for (it = 0; it < nItems; it++)
	    {
		/* looping on elementaty geometries */
		geos_item = GEOSGetGeometryN (geos, it);
		itemType = GEOSGeomTypeId (geos_item);
		switch (itemType)
		  {
		  case GEOS_POINT:
		      cs = GEOSGeom_getCoordSeq (geos_item);
		      GEOSCoordSeq_getDimensions (cs, &dims);
		      if (dims == 3)
			{
			    GEOSCoordSeq_getX (cs, 0, &x);
			    GEOSCoordSeq_getY (cs, 0, &y);
			    GEOSCoordSeq_getZ (cs, 0, &z);
			}
		      else
			{
			    GEOSCoordSeq_getX (cs, 0, &x);
			    GEOSCoordSeq_getY (cs, 0, &y);
			    z = 0.0;
			}
		      if (dimension_model == GAIA_XY_Z)
			  gaiaAddPointToGeomCollXYZ (gaia, x, y, z);
		      else if (dimension_model == GAIA_XY_M)
			  gaiaAddPointToGeomCollXYM (gaia, x, y, 0.0);
		      else if (dimension_model == GAIA_XY_Z_M)
			  gaiaAddPointToGeomCollXYZM (gaia, x, y, z, 0.0);
		      else
			  gaiaAddPointToGeomColl (gaia, x, y);
		      break;
		  case GEOS_LINESTRING:
		      cs = GEOSGeom_getCoordSeq (geos_item);
		      GEOSCoordSeq_getDimensions (cs, &dims);
		      GEOSCoordSeq_getSize (cs, &points);
		      ln = gaiaAddLinestringToGeomColl (gaia, points);
		      for (iv = 0; iv < (int) points; iv++)
			{
			    if (dims == 3)
			      {
				  GEOSCoordSeq_getX (cs, iv, &x);
				  GEOSCoordSeq_getY (cs, iv, &y);
				  GEOSCoordSeq_getZ (cs, iv, &z);
			      }
			    else
			      {
				  GEOSCoordSeq_getX (cs, iv, &x);
				  GEOSCoordSeq_getY (cs, iv, &y);
				  z = 0.0;
			      }
			    if (dimension_model == GAIA_XY_Z)
			      {
				  gaiaSetPointXYZ (ln->Coords, iv, x, y, z);
			      }
			    else if (dimension_model == GAIA_XY_M)
			      {
				  gaiaSetPointXYM (ln->Coords, iv, x, y, 0.0);
			      }
			    else if (dimension_model == GAIA_XY_Z_M)
			      {
				  gaiaSetPointXYZM (ln->Coords, iv, x, y, z,
						    0.0);
			      }
			    else
			      {
				  gaiaSetPoint (ln->Coords, iv, x, y);
			      }
			}
		      break;
		  case GEOS_MULTILINESTRING:
		      nSubItems = GEOSGetNumGeometries (geos_item);
		      for (sub_it = 0; sub_it < nSubItems; sub_it++)
			{
			    /* looping on elementaty geometries */
			    geos_sub_item =
				GEOSGetGeometryN (geos_item, sub_it);
			    cs = GEOSGeom_getCoordSeq (geos_sub_item);
			    GEOSCoordSeq_getDimensions (cs, &dims);
			    GEOSCoordSeq_getSize (cs, &points);
			    ln = gaiaAddLinestringToGeomColl (gaia, points);
			    for (iv = 0; iv < (int) points; iv++)
			      {
				  if (dims == 3)
				    {
					GEOSCoordSeq_getX (cs, iv, &x);
					GEOSCoordSeq_getY (cs, iv, &y);
					GEOSCoordSeq_getZ (cs, iv, &z);
				    }
				  else
				    {
					GEOSCoordSeq_getX (cs, iv, &x);
					GEOSCoordSeq_getY (cs, iv, &y);
					z = 0.0;
				    }
				  if (dimension_model == GAIA_XY_Z)
				    {
					gaiaSetPointXYZ (ln->Coords, iv, x, y,
							 z);
				    }
				  else if (dimension_model == GAIA_XY_M)
				    {
					gaiaSetPointXYM (ln->Coords, iv, x, y,
							 0.0);
				    }
				  else if (dimension_model == GAIA_XY_Z_M)
				    {
					gaiaSetPointXYZM (ln->Coords, iv, x, y,
							  z, 0.0);
				    }
				  else
				    {
					gaiaSetPoint (ln->Coords, iv, x, y);
				    }
			      }
			}
		      break;
		  case GEOS_POLYGON:
		      /* exterior ring */
		      holes = GEOSGetNumInteriorRings (geos_item);
		      geos_ring = GEOSGetExteriorRing (geos_item);
		      cs = GEOSGeom_getCoordSeq (geos_ring);
		      GEOSCoordSeq_getDimensions (cs, &dims);
		      GEOSCoordSeq_getSize (cs, &points);
		      pg = gaiaAddPolygonToGeomColl (gaia, points, holes);
		      rng = pg->Exterior;
		      for (iv = 0; iv < (int) points; iv++)
			{
			    if (dims == 3)
			      {
				  GEOSCoordSeq_getX (cs, iv, &x);
				  GEOSCoordSeq_getY (cs, iv, &y);
				  GEOSCoordSeq_getZ (cs, iv, &z);
			      }
			    else
			      {
				  GEOSCoordSeq_getX (cs, iv, &x);
				  GEOSCoordSeq_getY (cs, iv, &y);
				  z = 0.0;
			      }
			    if (dimension_model == GAIA_XY_Z)
			      {
				  gaiaSetPointXYZ (rng->Coords, iv, x, y, z);
			      }
			    else if (dimension_model == GAIA_XY_M)
			      {
				  gaiaSetPointXYM (rng->Coords, iv, x, y, 0.0);
			      }
			    else if (dimension_model == GAIA_XY_Z_M)
			      {
				  gaiaSetPointXYZM (rng->Coords, iv, x, y, z,
						    0.0);
			      }
			    else
			      {
				  gaiaSetPoint (rng->Coords, iv, x, y);
			      }
			}
		      for (ib = 0; ib < holes; ib++)
			{
			    /* interior rings */
			    geos_ring = GEOSGetInteriorRingN (geos_item, ib);
			    cs = GEOSGeom_getCoordSeq (geos_ring);
			    GEOSCoordSeq_getDimensions (cs, &dims);
			    GEOSCoordSeq_getSize (cs, &points);
			    rng = gaiaAddInteriorRing (pg, ib, points);
			    for (iv = 0; iv < (int) points; iv++)
			      {
				  if (dims == 3)
				    {
					GEOSCoordSeq_getX (cs, iv, &x);
					GEOSCoordSeq_getY (cs, iv, &y);
					GEOSCoordSeq_getZ (cs, iv, &z);
				    }
				  else
				    {
					GEOSCoordSeq_getX (cs, iv, &x);
					GEOSCoordSeq_getY (cs, iv, &y);
					z = 0.0;
				    }
				  if (dimension_model == GAIA_XY_Z)
				    {
					gaiaSetPointXYZ (rng->Coords, iv, x, y,
							 z);
				    }
				  else if (dimension_model == GAIA_XY_M)
				    {
					gaiaSetPointXYM (rng->Coords, iv, x, y,
							 0.0);
				    }
				  else if (dimension_model == GAIA_XY_Z_M)
				    {
					gaiaSetPointXYZM (rng->Coords, iv, x, y,
							  z, 0.0);
				    }
				  else
				    {
					gaiaSetPoint (rng->Coords, iv, x, y);
				    }
			      }
			}
		      break;
		  };
	    }
	  break;
      };
    return gaia;
}

GAIAGEO_DECLARE void *
gaiaToGeos (const gaiaGeomCollPtr gaia)
{
/* converting a GAIA Geometry into a GEOS Geometry */
    return toGeosGeometry (gaia);
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromGeos_XY (const void *xgeos)
{
/* converting a GEOS Geometry into a GAIA Geometry [XY] */
    const GEOSGeometry *geos = xgeos;
    return fromGeosGeometry (geos, GAIA_XY);
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromGeos_XYZ (const void *xgeos)
{
/* converting a GEOS Geometry into a GAIA Geometry [XYZ] */
    const GEOSGeometry *geos = xgeos;
    return fromGeosGeometry (geos, GAIA_XY_Z);
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromGeos_XYM (const void *xgeos)
{
/* converting a GEOS Geometry into a GAIA Geometry [XYM] */
    const GEOSGeometry *geos = xgeos;
    return fromGeosGeometry (geos, GAIA_XY_M);
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromGeos_XYZM (const void *xgeos)
{
/* converting a GEOS Geometry into a GAIA Geometry [XYZM] */
    const GEOSGeometry *geos = xgeos;
    return fromGeosGeometry (geos, GAIA_XY_Z_M);
}

#endif /* end including GEOS */
/**************** End file: gg_geoscvt.c **********/


/**************** Begin file: gg_shape.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <math.h> */
/* #include <float.h> */
/* #include <errno.h> */

#if OMIT_ICONV == 0		/* if ICONV is disabled no SHP support is available */

#if defined(__MINGW32__) || defined(_WIN32)
#define LIBICONV_STATIC
/* #include <iconv.h> */
#define LIBCHARSET_STATIC
#ifdef _MSC_VER
/* <localcharset.h> isn't supported on OSGeo4W */
/* applying a tricky workaround to fix this issue */
extern const char *locale_charset (void);
#else /* sane Windows - not OSGeo4W */
/* #include <localcharset.h> */
#endif /* end localcharset */
#else /* not MINGW32 */
#ifdef __APPLE__
/* #include <iconv.h> */
/* #include <localcharset.h> */
#else /* not Mac OsX */
/* #include <iconv.h> */
/* #include <langinfo.h> */
#endif
#endif

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

#ifdef _WIN32
#define atoll	_atoi64
#endif /* not WIN32 */

#define SHAPEFILE_NO_DATA 1e-38

GAIAGEO_DECLARE void
gaiaFreeValue (gaiaValuePtr p)
{
/* frees all memory allocations for this DBF Field value */
    if (!p)
	return;
    if (p->TxtValue)
	free (p->TxtValue);
    free (p);
}

GAIAGEO_DECLARE void
gaiaSetNullValue (gaiaDbfFieldPtr field)
{
/* assignes a NULL value to some DBF field */
    if (field->Value)
	gaiaFreeValue (field->Value);
    field->Value = malloc (sizeof (gaiaValue));
    field->Value->Type = GAIA_NULL_VALUE;
    field->Value->TxtValue = NULL;
}

GAIAGEO_DECLARE void
gaiaSetIntValue (gaiaDbfFieldPtr field, sqlite3_int64 value)
{
/* assignes an INTEGER value to some DBF field */
    if (field->Value)
	gaiaFreeValue (field->Value);
    field->Value = malloc (sizeof (gaiaValue));
    field->Value->Type = GAIA_INT_VALUE;
    field->Value->TxtValue = NULL;
    field->Value->IntValue = value;
}

GAIAGEO_DECLARE void
gaiaSetDoubleValue (gaiaDbfFieldPtr field, double value)
{
/* assignes a DOUBLE value to some DBF field */
    if (field->Value)
	gaiaFreeValue (field->Value);
    field->Value = malloc (sizeof (gaiaValue));
    field->Value->Type = GAIA_DOUBLE_VALUE;
    field->Value->TxtValue = NULL;
    field->Value->DblValue = value;
}

GAIAGEO_DECLARE void
gaiaSetStrValue (gaiaDbfFieldPtr field, char *str)
{
/* assignes a STRING value to some DBF field */
    int len = strlen (str);
    if (field->Value)
	gaiaFreeValue (field->Value);
    field->Value = malloc (sizeof (gaiaValue));
    field->Value->Type = GAIA_TEXT_VALUE;
    field->Value->TxtValue = malloc (len + 1);
    strcpy (field->Value->TxtValue, str);
}

GAIAGEO_DECLARE gaiaDbfFieldPtr
gaiaAllocDbfField (char *name, unsigned char type,
		   int offset, unsigned char length, unsigned char decimals)
{
/* allocates and initializes a DBF Field definition */
    gaiaDbfFieldPtr p = malloc (sizeof (gaiaDbfField));
    int len = strlen (name);
    p->Name = malloc (len + 1);
    strcpy (p->Name, name);
    p->Type = type;
    p->Offset = offset;
    p->Length = length;
    p->Decimals = decimals;
    p->Value = NULL;
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE void
gaiaFreeDbfField (gaiaDbfFieldPtr p)
{
/* frees all memory allocations for this DBF Field definition */
    if (!p)
	return;
    if (p->Name)
	free (p->Name);
    if (p->Value)
	gaiaFreeValue (p->Value);
    free (p);
}

GAIAGEO_DECLARE gaiaDbfFieldPtr
gaiaCloneDbfField (gaiaDbfFieldPtr org)
{
/* creating a new DBF LIST copied from the original one */
    gaiaDbfFieldPtr p = malloc (sizeof (gaiaDbfField));
    int len = strlen (org->Name);
    p->Name = malloc (len + 1);
    strcpy (p->Name, org->Name);
    p->Type = org->Type;
    p->Offset = org->Offset;
    p->Length = org->Length;
    p->Decimals = org->Decimals;
    p->Value = gaiaCloneValue (org->Value);
    p->Next = NULL;
    return p;
}

GAIAGEO_DECLARE gaiaDbfListPtr
gaiaAllocDbfList ()
{
/* allocates and initializes the DBF Fields list */
    gaiaDbfListPtr list = malloc (sizeof (gaiaDbfList));
    list->RowId = 0;
    list->Geometry = NULL;
    list->First = NULL;
    list->Last = NULL;
    return list;
}

GAIAGEO_DECLARE void
gaiaFreeDbfList (gaiaDbfListPtr list)
{
/* frees all memory allocations related to DBF Fields list */
    gaiaDbfFieldPtr p;
    gaiaDbfFieldPtr pn;
    if (!list)
	return;
    p = list->First;
    while (p)
      {
	  pn = p->Next;
	  gaiaFreeDbfField (p);
	  p = pn;
      }
    if (list->Geometry)
	gaiaFreeGeomColl (list->Geometry);
    free (list);
}

GAIAGEO_DECLARE int
gaiaIsValidDbfList (gaiaDbfListPtr list)
{
/* checks if the DBF fields list contains any invalid data type */
    gaiaDbfFieldPtr p;
    if (!list)
	return 0;
    p = list->First;
    while (p)
      {
	  if (p->Type == 'N' || p->Type == 'C' || p->Type == 'L'
	      || p->Type == 'D' || p->Type == 'F')
	      ;
	  else
	      return 0;
	  p = p->Next;
      }
    return 1;
}

GAIAGEO_DECLARE gaiaDbfFieldPtr
gaiaAddDbfField (gaiaDbfListPtr list, char *name, unsigned char type,
		 int offset, unsigned char length, unsigned char decimals)
{
/* inserts a Field in the DBF Fields list */
    gaiaDbfFieldPtr p;
    if (!list)
	return NULL;
    p = gaiaAllocDbfField (name, type, offset, length, decimals);
    if (!(list->First))
	list->First = p;
    if (list->Last)
	list->Last->Next = p;
    list->Last = p;
    return p;
}

GAIAGEO_DECLARE void
gaiaResetDbfEntity (gaiaDbfListPtr list)
{
/* resets data values */
    gaiaDbfFieldPtr p;
    if (!list)
	return;
    p = list->First;
    while (p)
      {
	  if (p->Value)
	      gaiaFreeValue (p->Value);
	  p->Value = NULL;
	  p = p->Next;
      }
    if (list->Geometry)
	gaiaFreeGeomColl (list->Geometry);
    list->Geometry = NULL;
}

GAIAGEO_DECLARE gaiaValuePtr
gaiaCloneValue (gaiaValuePtr org)
{
/* creating a new VARIANT value copied from the original one */
    gaiaValuePtr value;
    int len;
    value = malloc (sizeof (gaiaValue));
    value->Type = GAIA_NULL_VALUE;
    value->TxtValue = NULL;
    switch (org->Type)
      {
      case GAIA_INT_VALUE:
	  value->Type = GAIA_INT_VALUE;
	  value->IntValue = org->IntValue;
	  break;
      case GAIA_DOUBLE_VALUE:
	  value->Type = GAIA_DOUBLE_VALUE;
	  value->DblValue = org->DblValue;
	  break;
      case GAIA_TEXT_VALUE:
	  value->Type = GAIA_TEXT_VALUE;
	  len = strlen (org->TxtValue);
	  value->TxtValue = malloc (len + 1);
	  strcpy (value->TxtValue, org->TxtValue);
      };
    return value;
}

GAIAGEO_DECLARE gaiaDbfListPtr
gaiaCloneDbfEntity (gaiaDbfListPtr org)
{
/* creating a new DBF LIST copied from the original one */
    gaiaDbfFieldPtr p;
    gaiaDbfFieldPtr newFld;
    gaiaDbfListPtr entity = gaiaAllocDbfList ();
    entity->RowId = org->RowId;
    if (org->Geometry)
	entity->Geometry = gaiaCloneGeomColl (org->Geometry);
    p = org->First;
    while (p)
      {
	  newFld =
	      gaiaAddDbfField (entity, p->Name, p->Type, p->Offset, p->Length,
			       p->Decimals);
	  if (p->Value)
	      newFld->Value = gaiaCloneValue (p->Value);
	  p = p->Next;
      }
    return entity;
}

GAIAGEO_DECLARE gaiaShapefilePtr
gaiaAllocShapefile ()
{
/* allocates and initializes the Shapefile object */
    gaiaShapefilePtr shp = malloc (sizeof (gaiaShapefile));
    shp->endian_arch = 1;
    shp->Path = NULL;
    shp->Shape = -1;
    shp->EffectiveType = GAIA_UNKNOWN;
    shp->EffectiveDims = GAIA_XY;
    shp->flShp = NULL;
    shp->flShx = NULL;
    shp->flDbf = NULL;
    shp->Dbf = NULL;
    shp->BufShp = NULL;
    shp->ShpBfsz = 0;
    shp->BufDbf = NULL;
    shp->DbfHdsz = 0;
    shp->DbfReclen = 0;
    shp->DbfSize = 0;
    shp->DbfRecno = 0;
    shp->ShpSize = 0;
    shp->ShxSize = 0;
    shp->MinX = DBL_MAX;
    shp->MinY = DBL_MAX;
    shp->MaxX = -DBL_MAX;
    shp->MaxY = -DBL_MAX;
    shp->Valid = 0;
    shp->IconvObj = NULL;
    shp->LastError = NULL;
    return shp;
}

GAIAGEO_DECLARE void
gaiaFreeShapefile (gaiaShapefilePtr shp)
{
/* frees all memory allocations related to the Shapefile object */
    if (shp->Path)
	free (shp->Path);
    if (shp->flShp)
	fclose (shp->flShp);
    if (shp->flShx)
	fclose (shp->flShx);
    if (shp->flDbf)
	fclose (shp->flDbf);
    if (shp->Dbf)
	gaiaFreeDbfList (shp->Dbf);
    if (shp->BufShp)
	free (shp->BufShp);
    if (shp->BufDbf)
	free (shp->BufDbf);
    if (shp->IconvObj)
	iconv_close ((iconv_t) shp->IconvObj);
    if (shp->LastError)
	free (shp->LastError);
    free (shp);
}

GAIAGEO_DECLARE void
gaiaOpenShpRead (gaiaShapefilePtr shp, const char *path, const char *charFrom,
		 const char *charTo)
{
/* trying to open the shapefile and initial checkings */
    FILE *fl_shx = NULL;
    FILE *fl_shp = NULL;
    FILE *fl_dbf = NULL;
    char xpath[1024];
    int rd;
    unsigned char buf_shx[256];
    unsigned char *buf_shp = NULL;
    int buf_size = 1024;
    int shape;
    unsigned char bf[1024];
    int dbf_size;
    int dbf_reclen = 0;
    int off_dbf;
    int ind;
    char field_name[2048];
    char *sys_err;
    char errMsg[1024];
    iconv_t iconv_ret;
    char utf8buf[2048];
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    int endian_arch = gaiaEndianArch ();
    gaiaDbfListPtr dbf_list = NULL;
    if (charFrom && charTo)
      {
	  iconv_ret = iconv_open (charTo, charFrom);
	  if (iconv_ret == (iconv_t) (-1))
	    {
		sprintf (errMsg, "conversion from '%s' to '%s' not available\n",
			 charFrom, charTo);
		goto unsupported_conversion;
	    }
	  shp->IconvObj = iconv_ret;
      }
    else
      {
	  sprintf (errMsg, "a NULL charset-name was passed\n");
	  goto unsupported_conversion;
      }
    if (shp->flShp != NULL || shp->flShx != NULL || shp->flDbf != NULL)
      {
	  sprintf (errMsg,
		   "attempting to reopen an already opened Shapefile\n");
	  goto unsupported_conversion;
      }
    sprintf (xpath, "%s.shx", path);
    fl_shx = fopen (xpath, "rb");
    if (!fl_shx)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for reading: %s", xpath,
		   sys_err);
	  goto no_file;
      }
    sprintf (xpath, "%s.shp", path);
    fl_shp = fopen (xpath, "rb");
    if (!fl_shp)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for reading: %s", xpath,
		   sys_err);
	  goto no_file;
      }
    sprintf (xpath, "%s.dbf", path);
    fl_dbf = fopen (xpath, "rb");
    if (!fl_dbf)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for reading: %s", xpath,
		   sys_err);
	  goto no_file;
      }
/* reading SHX file header */
    rd = fread (buf_shx, sizeof (unsigned char), 100, fl_shx);
    if (rd != 100)
	goto error;
    if (gaiaImport32 (buf_shx + 0, GAIA_BIG_ENDIAN, endian_arch) != 9994)	/* checks the SHX magic number */
	goto error;
/* reading SHP file header */
    buf_shp = malloc (sizeof (unsigned char) * buf_size);
    rd = fread (buf_shp, sizeof (unsigned char), 100, fl_shp);
    if (rd != 100)
	goto error;
    if (gaiaImport32 (buf_shp + 0, GAIA_BIG_ENDIAN, endian_arch) != 9994)	/* checks the SHP magic number */
	goto error;
    shape = gaiaImport32 (buf_shp + 32, GAIA_LITTLE_ENDIAN, endian_arch);
    if (shape == GAIA_SHP_POINT || shape == GAIA_SHP_POINTZ
	|| shape == GAIA_SHP_POINTM || shape == GAIA_SHP_POLYLINE
	|| shape == GAIA_SHP_POLYLINEZ || shape == GAIA_SHP_POLYLINEM
	|| shape == GAIA_SHP_POLYGON || shape == GAIA_SHP_POLYGONZ
	|| shape == GAIA_SHP_POLYGONM || shape == GAIA_SHP_MULTIPOINT
	|| shape == GAIA_SHP_MULTIPOINTZ || shape == GAIA_SHP_MULTIPOINTM)
	;
    else
	goto unsupported;
/* reading DBF file header */
    rd = fread (bf, sizeof (unsigned char), 32, fl_dbf);
    if (rd != 32)
	goto error;
    if (*bf != 0x03)		/* checks the DBF magic number */
	goto error;
    dbf_size = gaiaImport16 (bf + 8, GAIA_LITTLE_ENDIAN, endian_arch);
    dbf_reclen = gaiaImport16 (bf + 10, GAIA_LITTLE_ENDIAN, endian_arch);
    dbf_size--;
    off_dbf = 0;
    dbf_list = gaiaAllocDbfList ();
    for (ind = 32; ind < dbf_size; ind += 32)
      {
	  /* fetches DBF fields definitions */
	  rd = fread (bf, sizeof (unsigned char), 32, fl_dbf);
	  if (rd != 32)
	      goto error;
	  memcpy (field_name, bf, 11);
	  field_name[11] = '\0';
	  len = strlen ((char *) field_name);
	  utf8len = 2048;
	  pBuf = (char *) field_name;
	  pUtf8buf = utf8buf;
	  if (iconv
	      ((iconv_t) (shp->IconvObj), &pBuf, &len, &pUtf8buf,
	       &utf8len) == (size_t) (-1))
	      goto conversion_error;
	  memcpy (field_name, utf8buf, 2048 - utf8len);
	  field_name[2048 - utf8len] = '\0';
	  gaiaAddDbfField (dbf_list, field_name, *(bf + 11), off_dbf,
			   *(bf + 16), *(bf + 17));
	  off_dbf += *(bf + 16);
      }
    if (!gaiaIsValidDbfList (dbf_list))
      {
	  /* invalid DBF */
	  goto illegal_dbf;
      }
    len = strlen (path);
    shp->Path = malloc (len + 1);
    strcpy (shp->Path, path);
    shp->ReadOnly = 1;
    shp->Shape = shape;
    switch (shape)
      {
	  /* setting up a prudential geometry type */
      case GAIA_SHP_POINT:
      case GAIA_SHP_POINTZ:
      case GAIA_SHP_POINTM:
	  shp->EffectiveType = GAIA_POINT;
	  break;
      case GAIA_SHP_POLYLINE:
      case GAIA_SHP_POLYLINEZ:
      case GAIA_SHP_POLYLINEM:
	  shp->EffectiveType = GAIA_MULTILINESTRING;
	  break;
      case GAIA_SHP_POLYGON:
      case GAIA_SHP_POLYGONZ:
      case GAIA_SHP_POLYGONM:
	  shp->EffectiveType = GAIA_MULTIPOLYGON;
	  break;
      case GAIA_SHP_MULTIPOINT:
      case GAIA_SHP_MULTIPOINTZ:
      case GAIA_SHP_MULTIPOINTM:
	  shp->EffectiveType = GAIA_MULTIPOINT;
	  break;
      }
    switch (shape)
      {
	  /* setting up a prudential dimension model */
      case GAIA_SHP_POINTZ:
      case GAIA_SHP_POLYLINEZ:
      case GAIA_SHP_POLYGONZ:
      case GAIA_SHP_MULTIPOINTZ:
	  shp->EffectiveDims = GAIA_XY_Z_M;
	  break;
      case GAIA_SHP_POINTM:
      case GAIA_SHP_POLYLINEM:
      case GAIA_SHP_POLYGONM:
      case GAIA_SHP_MULTIPOINTM:
	  shp->EffectiveDims = GAIA_XY_M;
	  break;
      default:
	  shp->EffectiveDims = GAIA_XY;
	  break;
      }
    shp->flShp = fl_shp;
    shp->flShx = fl_shx;
    shp->flDbf = fl_dbf;
    shp->Dbf = dbf_list;
/* saving the SHP buffer */
    shp->BufShp = buf_shp;
    shp->ShpBfsz = buf_size;
/* allocating DBF buffer */
    shp->BufDbf = malloc (sizeof (unsigned char) * dbf_reclen);
    shp->DbfHdsz = dbf_size + 1;
    shp->DbfReclen = dbf_reclen;
    shp->Valid = 1;
    shp->endian_arch = endian_arch;
    return;
  unsupported_conversion:
/* illegal charset */
    if (shp->LastError)
	free (shp->LastError);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    return;
  no_file:
/* one of shapefile's files can't be accessed */
    if (shp->LastError)
	free (shp->LastError);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    if (fl_shx)
	fclose (fl_shx);
    if (fl_shp)
	fclose (fl_shp);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
  error:
/* the shapefile is invalid or corrupted */
    if (shp->LastError)
	free (shp->LastError);
    sprintf (errMsg, "'%s' is corrupted / has invalid format", path);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    gaiaFreeDbfList (dbf_list);
    if (buf_shp)
	free (buf_shp);
    fclose (fl_shx);
    fclose (fl_shp);
    fclose (fl_dbf);
    return;
  unsupported:
/* the shapefile has an unrecognized shape type */
    if (shp->LastError)
	free (shp->LastError);
    sprintf (errMsg, "'%s' shape=%d is not supported", path, shape);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    gaiaFreeDbfList (dbf_list);
    if (buf_shp)
	free (buf_shp);
    fclose (fl_shx);
    fclose (fl_shp);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
  illegal_dbf:
/* the DBF-file contains unsupported data types */
    if (shp->LastError)
	free (shp->LastError);
    sprintf (errMsg, "'%s.dbf' contains unsupported data types", path);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    gaiaFreeDbfList (dbf_list);
    if (buf_shp)
	free (buf_shp);
    fclose (fl_shx);
    fclose (fl_shp);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
  conversion_error:
/* libiconv error */
    if (shp->LastError)
	free (shp->LastError);
    sprintf (errMsg, "'%s.dbf' field name: invalid character sequence", path);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    gaiaFreeDbfList (dbf_list);
    if (buf_shp)
	free (buf_shp);
    fclose (fl_shx);
    fclose (fl_shp);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
}

GAIAGEO_DECLARE void
gaiaOpenShpWrite (gaiaShapefilePtr shp, const char *path, int shape,
		  gaiaDbfListPtr dbf_list, const char *charFrom,
		  const char *charTo)
{
/* trying to create the shapefile */
    FILE *fl_shx = NULL;
    FILE *fl_shp = NULL;
    FILE *fl_dbf = NULL;
    char xpath[1024];
    unsigned char *buf_shp = NULL;
    int buf_size = 1024;
    unsigned char *dbf_buf = NULL;
    gaiaDbfFieldPtr fld;
    char *sys_err;
    char errMsg[1024];
    short dbf_reclen = 0;
    int shp_size = 0;
    int shx_size = 0;
    unsigned short dbf_size = 0;
    iconv_t iconv_ret;
    int endian_arch = gaiaEndianArch ();
    char buf[2048];
    char utf8buf[2048];
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    int defaultId = 1;
    if (charFrom && charTo)
      {
	  iconv_ret = iconv_open (charTo, charFrom);
	  if (iconv_ret == (iconv_t) (-1))
	    {
		sprintf (errMsg, "conversion from '%s' to '%s' not available\n",
			 charFrom, charTo);
		goto unsupported_conversion;
	    }
	  shp->IconvObj = iconv_ret;
      }
    else
      {
	  sprintf (errMsg, "a NULL charset-name was passed\n");
	  goto unsupported_conversion;
      }
    if (shp->flShp != NULL || shp->flShx != NULL || shp->flDbf != NULL)
      {
	  sprintf (errMsg,
		   "attempting to reopen an already opened Shapefile\n");
	  goto unsupported_conversion;
      }
    buf_shp = malloc (buf_size);
/* trying to open shapefile files */
    sprintf (xpath, "%s.shx", path);
    fl_shx = fopen (xpath, "wb");
    if (!fl_shx)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for writing: %s", xpath,
		   sys_err);
	  goto no_file;
      }
    sprintf (xpath, "%s.shp", path);
    fl_shp = fopen (xpath, "wb");
    if (!fl_shp)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for writing: %s", xpath,
		   sys_err);
	  goto no_file;
      }
    sprintf (xpath, "%s.dbf", path);
    fl_dbf = fopen (xpath, "wb");
    if (!fl_dbf)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for writing: %s", xpath,
		   sys_err);
	  goto no_file;
      }
/* allocating DBF buffer */
    dbf_reclen = 1;		/* an extra byte is needed because in DBF rows first byte is a marker for deletion */
    fld = dbf_list->First;
    while (fld)
      {
	  /* computing the DBF record length */
	  dbf_reclen += fld->Length;
	  fld = fld->Next;
      }
    dbf_buf = malloc (dbf_reclen);
/* writing an empty SHP file header */
    memset (buf_shp, 0, 100);
    fwrite (buf_shp, 1, 100, fl_shp);
    shp_size = 50;		/* note: shapefile [SHP and SHX] counts sizes in WORDS of 16 bits, not in bytes of 8 bits !!!! */
/* writing an empty SHX file header */
    memset (buf_shp, 0, 100);
    fwrite (buf_shp, 1, 100, fl_shx);
    shx_size = 50;
/* writing the DBF file header */
    memset (buf_shp, '\0', 32);
    fwrite (buf_shp, 1, 32, fl_dbf);
    dbf_size = 32;		/* note: DBF counts sizes in bytes */
    fld = dbf_list->First;
    while (fld)
      {
	  /* exporting DBF Fields specifications */
	  memset (buf_shp, 0, 32);
	  strcpy (buf, fld->Name);
	  len = strlen (buf);
	  utf8len = 2048;
	  pBuf = buf;
	  pUtf8buf = utf8buf;
	  if (iconv
	      ((iconv_t) (shp->IconvObj), &pBuf, &len, &pUtf8buf,
	       &utf8len) == (size_t) (-1))
	      sprintf (buf, "FLD#%d", defaultId++);
	  else
	    {
		memcpy (buf, utf8buf, 2048 - utf8len);
		buf[2048 - utf8len] = '\0';
		if (strlen (buf) > 10)
		    sprintf (buf, "FLD#%d", defaultId++);
	    }
	  memcpy (buf_shp, buf, strlen (buf));
	  *(buf_shp + 11) = fld->Type;
	  *(buf_shp + 16) = fld->Length;
	  *(buf_shp + 17) = fld->Decimals;
	  fwrite (buf_shp, 1, 32, fl_dbf);
	  dbf_size += 32;
	  fld = fld->Next;
      }
    fwrite ("\r", 1, 1, fl_dbf);	/* this one is a special DBF delimiter that closes file header */
    dbf_size++;
/* setting up the SHP struct */
    len = strlen (path);
    shp->Path = malloc (len + 1);
    strcpy (shp->Path, path);
    shp->ReadOnly = 0;
    switch (shape)
      {
	  /* setting up SHAPE and dimensions */
      case GAIA_POINT:
	  shp->Shape = GAIA_SHP_POINT;
	  shp->EffectiveType = GAIA_POINT;
	  shp->EffectiveDims = GAIA_XY;
	  break;
      case GAIA_POINTZ:
	  shp->Shape = GAIA_SHP_POINTZ;
	  shp->EffectiveType = GAIA_POINT;
	  shp->EffectiveDims = GAIA_XY_Z;
	  break;
      case GAIA_POINTM:
	  shp->Shape = GAIA_SHP_POINTM;
	  shp->EffectiveType = GAIA_POINT;
	  shp->EffectiveDims = GAIA_XY_M;
	  break;
      case GAIA_POINTZM:
	  shp->Shape = GAIA_SHP_POINTZ;
	  shp->EffectiveType = GAIA_POINT;
	  shp->EffectiveDims = GAIA_XY_Z_M;
	  break;
      case GAIA_MULTIPOINT:
	  shp->Shape = GAIA_SHP_MULTIPOINT;
	  shp->EffectiveType = GAIA_MULTIPOINT;
	  shp->EffectiveDims = GAIA_XY;
	  break;
      case GAIA_MULTIPOINTZ:
	  shp->Shape = GAIA_SHP_MULTIPOINTZ;
	  shp->EffectiveType = GAIA_MULTIPOINT;
	  shp->EffectiveDims = GAIA_XY_Z;
	  break;
      case GAIA_MULTIPOINTM:
	  shp->Shape = GAIA_SHP_MULTIPOINTM;
	  shp->EffectiveType = GAIA_MULTIPOINT;
	  shp->EffectiveDims = GAIA_XY_M;
	  break;
      case GAIA_MULTIPOINTZM:
	  shp->Shape = GAIA_SHP_MULTIPOINTZ;
	  shp->EffectiveType = GAIA_MULTIPOINT;
	  shp->EffectiveDims = GAIA_XY_Z_M;
	  break;
      case GAIA_LINESTRING:
	  shp->Shape = GAIA_SHP_POLYLINE;
	  shp->EffectiveType = GAIA_LINESTRING;
	  shp->EffectiveDims = GAIA_XY;
	  break;
      case GAIA_LINESTRINGZ:
	  shp->Shape = GAIA_SHP_POLYLINEZ;
	  shp->EffectiveType = GAIA_LINESTRING;
	  shp->EffectiveDims = GAIA_XY_Z;
	  break;
      case GAIA_LINESTRINGM:
	  shp->Shape = GAIA_SHP_POLYLINEM;
	  shp->EffectiveType = GAIA_LINESTRING;
	  shp->EffectiveDims = GAIA_XY_M;
	  break;
      case GAIA_LINESTRINGZM:
	  shp->Shape = GAIA_SHP_POLYLINEZ;
	  shp->EffectiveType = GAIA_LINESTRING;
	  shp->EffectiveDims = GAIA_XY_Z_M;
	  break;
      case GAIA_MULTILINESTRING:
	  shp->Shape = GAIA_SHP_POLYLINE;
	  shp->EffectiveType = GAIA_MULTILINESTRING;
	  shp->EffectiveDims = GAIA_XY;
	  break;
      case GAIA_MULTILINESTRINGZ:
	  shp->Shape = GAIA_SHP_POLYLINEZ;
	  shp->EffectiveType = GAIA_MULTILINESTRING;
	  shp->EffectiveDims = GAIA_XY_Z;
	  break;
      case GAIA_MULTILINESTRINGM:
	  shp->Shape = GAIA_SHP_POLYLINEM;
	  shp->EffectiveType = GAIA_MULTILINESTRING;
	  shp->EffectiveDims = GAIA_XY_M;
	  break;
      case GAIA_MULTILINESTRINGZM:
	  shp->Shape = GAIA_SHP_POLYLINEZ;
	  shp->EffectiveType = GAIA_MULTILINESTRING;
	  shp->EffectiveDims = GAIA_XY_Z_M;
	  break;
      case GAIA_POLYGON:
	  shp->Shape = GAIA_SHP_POLYGON;
	  shp->EffectiveType = GAIA_POLYGON;
	  shp->EffectiveDims = GAIA_XY;
	  break;
      case GAIA_POLYGONZ:
	  shp->Shape = GAIA_SHP_POLYGONZ;
	  shp->EffectiveType = GAIA_POLYGON;
	  shp->EffectiveDims = GAIA_XY_Z;
	  break;
      case GAIA_POLYGONM:
	  shp->Shape = GAIA_SHP_POLYGONM;
	  shp->EffectiveType = GAIA_POLYGON;
	  shp->EffectiveDims = GAIA_XY_M;
	  break;
      case GAIA_POLYGONZM:
	  shp->Shape = GAIA_SHP_POLYGONZ;
	  shp->EffectiveType = GAIA_POLYGON;
	  shp->EffectiveDims = GAIA_XY_Z_M;
	  break;
      case GAIA_MULTIPOLYGON:
	  shp->Shape = GAIA_SHP_POLYGON;
	  shp->EffectiveType = GAIA_MULTIPOLYGON;
	  shp->EffectiveDims = GAIA_XY;
	  break;
      case GAIA_MULTIPOLYGONZ:
	  shp->Shape = GAIA_SHP_POLYGONZ;
	  shp->EffectiveType = GAIA_MULTIPOLYGON;
	  shp->EffectiveDims = GAIA_XY_Z;
	  break;
      case GAIA_MULTIPOLYGONM:
	  shp->Shape = GAIA_SHP_POLYGONM;
	  shp->EffectiveType = GAIA_MULTIPOLYGON;
	  shp->EffectiveDims = GAIA_XY_M;
	  break;
      case GAIA_MULTIPOLYGONZM:
	  shp->Shape = GAIA_SHP_POLYGONZ;
	  shp->EffectiveType = GAIA_MULTIPOLYGON;
	  shp->EffectiveDims = GAIA_XY_Z_M;
	  break;
      };
    shp->flShp = fl_shp;
    shp->flShx = fl_shx;
    shp->flDbf = fl_dbf;
    shp->Dbf = dbf_list;
    shp->BufShp = buf_shp;
    shp->ShpBfsz = buf_size;
    shp->BufDbf = dbf_buf;
    shp->DbfHdsz = dbf_size + 1;
    shp->DbfReclen = dbf_reclen;
    shp->DbfSize = dbf_size;
    shp->DbfRecno = 0;
    shp->ShpSize = shp_size;
    shp->ShxSize = shx_size;
    shp->MinX = DBL_MAX;
    shp->MinY = DBL_MAX;
    shp->MaxX = -DBL_MAX;
    shp->MaxY = -DBL_MAX;
    shp->Valid = 1;
    shp->endian_arch = endian_arch;
    return;
  unsupported_conversion:
/* illegal charset */
    if (shp->LastError)
	free (shp->LastError);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    return;
  no_file:
/* one of shapefile's files can't be created/opened */
    if (shp->LastError)
	free (shp->LastError);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    if (buf_shp)
	free (buf_shp);
    if (fl_shx)
	fclose (fl_shx);
    if (fl_shp)
	fclose (fl_shp);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
}

static double
to_sqlite_julian_date (int year, int month, int day, double *julian)
{
/* trying to convert an 'YYYY-MM-DD' date into a JulianDate [double] */
    int Y;
    int M;
    int D;
    int A;
    int B;
    int X1;
    int X2;
    if (year < 1900 || year > 2400)
	return 0;
    if (month < 1 || month > 12)
	return 0;
    if (day < 1)
	return 0;
    switch (month)
      {
      case 2:
	  if ((year / 4) == 0)
	    {
		if (day > 29)
		    return 0;
	    }
	  else
	    {
		if (day > 28)
		    return 0;
	    }
	  break;
      case 4:
      case 6:
      case 9:
      case 11:
	  if (day > 30)
	      return 0;
	  break;
      default:
	  if (day > 31)
	      return 0;
      };
/* computing the Julian date */
    Y = year;
    M = month;
    D = day;
    if (M <= 2)
      {
	  Y--;
	  M += 12;
      }
    A = Y / 100;
    B = 2 - A + (A / 4);
    X1 = 36525 * (Y + 4716) / 100;
    X2 = 306001 * (M + 1) / 10000;
    *julian = (double) (X1 + X2 + D + B - 1524.5);
    return 1;
}

static int
parseDbfField (unsigned char *buf_dbf, void *iconv_obj, gaiaDbfFieldPtr pFld)
{
/* parsing a generic DBF field */
    unsigned char buf[512];
    char utf8buf[2048];
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    int i;
    memcpy (buf, buf_dbf + pFld->Offset + 1, pFld->Length);
    buf[pFld->Length] = '\0';
    if (*buf == '\0')
	gaiaSetNullValue (pFld);
    else
      {
	  if (pFld->Type == 'N')
	    {
		/* NUMERIC value */
		if (pFld->Decimals > 0 || pFld->Length > 18)
		    gaiaSetDoubleValue (pFld, atof ((char *) buf));
		else
		    gaiaSetIntValue (pFld, atoll ((char *) buf));
	    }
	  else if (pFld->Type == 'F')
	    {
		/* FLOAT value */
		gaiaSetDoubleValue (pFld, atof ((char *) buf));
	    }
	  else if (pFld->Type == 'D')
	    {
		/* DATE value */
		if (strlen ((char *) buf) != 8)
		    gaiaSetNullValue (pFld);
		else
		  {
		      /* converting into a Julian Date */
		      double julian;
		      char date[5];
		      int year = 0;
		      int month = 0;
		      int day = 0;
		      date[0] = buf[0];
		      date[1] = buf[1];
		      date[2] = buf[2];
		      date[3] = buf[3];
		      date[4] = '\0';
		      year = atoi (date);
		      date[0] = buf[4];
		      date[1] = buf[5];
		      date[2] = '\0';
		      month = atoi (date);
		      date[0] = buf[6];
		      date[1] = buf[7];
		      date[2] = '\0';
		      day = atoi (date);
		      if (to_sqlite_julian_date (year, month, day, &julian))
			  gaiaSetDoubleValue (pFld, julian);
		      else
			  gaiaSetNullValue (pFld);
		  }
	    }
	  else if (pFld->Type == 'L')
	    {
		/* LOGICAL [aka Boolean] value */
		if (*buf == '1' || *buf == 't' || *buf == 'T'
		    || *buf == 'Y' || *buf == 'y')
		    gaiaSetIntValue (pFld, 1);
		else
		    gaiaSetIntValue (pFld, 0);
	    }
	  else
	    {
		/* CHARACTER [aka String, Text] value */
		for (i = strlen ((char *) buf) - 1; i > 1; i--)
		  {
		      /* cleaning up trailing spaces */
		      if (buf[i] == ' ')
			  buf[i] = '\0';
		      else
			  break;
		  }
		len = strlen ((char *) buf);
		utf8len = 2048;
		pBuf = (char *) buf;
		pUtf8buf = utf8buf;
		if (iconv
		    ((iconv_t) (iconv_obj), &pBuf, &len, &pUtf8buf,
		     &utf8len) == (size_t) (-1))
		    return 0;
		memcpy (buf, utf8buf, 2048 - utf8len);
		buf[2048 - utf8len] = '\0';
		gaiaSetStrValue (pFld, (char *) buf);
	    }
      }
    return 1;
}

GAIAGEO_DECLARE int
gaiaReadShpEntity (gaiaShapefilePtr shp, int current_row, int srid)
{
/* trying to read an entity from shapefile */
    unsigned char buf[512];
    int len;
    int rd;
    int skpos;
    int offset;
    int off_shp;
    int sz;
    int shape;
    double x;
    double y;
    double z;
    double m;
    int points;
    int n;
    int n1;
    int base;
    int baseZ;
    int baseM;
    int start;
    int end;
    int iv;
    int ind;
    int max_size;
    int min_size;
    int hasM;
    char errMsg[1024];
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring = NULL;
    gaiaDbfFieldPtr pFld;
/* positioning and reading the SHX file */
    offset = 100 + (current_row * 8);	/* 100 bytes for the header + current row displacement; each SHX row = 8 bytes */
    skpos = fseek (shp->flShx, offset, SEEK_SET);
    if (skpos != 0)
	goto eof;
    rd = fread (buf, sizeof (unsigned char), 8, shp->flShx);
    if (rd != 8)
	goto eof;
    off_shp = gaiaImport32 (buf, GAIA_BIG_ENDIAN, shp->endian_arch);
/* positioning and reading the DBF file */
    offset = shp->DbfHdsz + (current_row * shp->DbfReclen);
    skpos = fseek (shp->flDbf, offset, SEEK_SET);
    if (skpos != 0)
	goto error;
    rd = fread (shp->BufDbf, sizeof (unsigned char), shp->DbfReclen,
		shp->flDbf);
    if (rd != shp->DbfReclen)
	goto error;
/* positioning and reading corresponding SHP entity - geometry */
    offset = off_shp * 2;
    skpos = fseek (shp->flShp, offset, SEEK_SET);
    if (skpos != 0)
	goto error;
    rd = fread (buf, sizeof (unsigned char), 12, shp->flShp);
    if (rd != 12)
	goto error;
    sz = gaiaImport32 (buf + 4, GAIA_BIG_ENDIAN, shp->endian_arch);
    shape = gaiaImport32 (buf + 8, GAIA_LITTLE_ENDIAN, shp->endian_arch);
    if (shape == GAIA_SHP_NULL)
      {
	  /* handling a NULL shape */
	  goto null_shape;
      }
    else if (shape != shp->Shape)
	goto error;
    if ((sz * 2) > shp->ShpBfsz)
      {
	  /* current buffer is too small; we need to allocate a bigger buffer */
	  free (shp->BufShp);
	  shp->ShpBfsz = sz * 2;
	  shp->BufShp = malloc (sizeof (unsigned char) * shp->ShpBfsz);
      }
    if (shape == GAIA_SHP_POINT)
      {
	  /* shape point */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 16, shp->flShp);
	  if (rd != 16)
	      goto error;
	  x = gaiaImport64 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  y = gaiaImport64 (shp->BufShp + 8, GAIA_LITTLE_ENDIAN,
			    shp->endian_arch);
	  if (shp->EffectiveDims == GAIA_XY_Z)
	    {
		geom = gaiaAllocGeomCollXYZ ();
		gaiaAddPointToGeomCollXYZ (geom, x, y, 0.0);
	    }
	  else if (shp->EffectiveDims == GAIA_XY_M)
	    {
		geom = gaiaAllocGeomCollXYM ();
		gaiaAddPointToGeomCollXYM (geom, x, y, 0.0);
	    }
	  else if (shp->EffectiveDims == GAIA_XY_Z_M)
	    {
		geom = gaiaAllocGeomCollXYZM ();
		gaiaAddPointToGeomCollXYZM (geom, x, y, 0.0, 0.0);
	    }
	  else
	    {
		geom = gaiaAllocGeomColl ();
		gaiaAddPointToGeomColl (geom, x, y);
	    }
	  geom->DeclaredType = GAIA_POINT;
	  geom->Srid = srid;
      }
    if (shape == GAIA_SHP_POINTZ)
      {
	  /* shape point Z */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	    {
		/* required by some buggish SHP (e.g. the GDAL/OGR ones) */
		if (rd != 24)
		    goto error;
	    }
	  x = gaiaImport64 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  y = gaiaImport64 (shp->BufShp + 8, GAIA_LITTLE_ENDIAN,
			    shp->endian_arch);
	  z = gaiaImport64 (shp->BufShp + 16, GAIA_LITTLE_ENDIAN,
			    shp->endian_arch);
	  if (rd == 24)
	      m = 0.0;
	  else
	      m = gaiaImport64 (shp->BufShp + 24, GAIA_LITTLE_ENDIAN,
				shp->endian_arch);
	  if (shp->EffectiveDims == GAIA_XY_Z)
	    {
		geom = gaiaAllocGeomCollXYZ ();
		gaiaAddPointToGeomCollXYZ (geom, x, y, z);
	    }
	  else if (shp->EffectiveDims == GAIA_XY_M)
	    {
		geom = gaiaAllocGeomCollXYM ();
		gaiaAddPointToGeomCollXYM (geom, x, y, m);
	    }
	  else if (shp->EffectiveDims == GAIA_XY_Z_M)
	    {
		geom = gaiaAllocGeomCollXYZM ();
		gaiaAddPointToGeomCollXYZM (geom, x, y, z, m);
	    }
	  else
	    {
		geom = gaiaAllocGeomColl ();
		gaiaAddPointToGeomColl (geom, x, y);
	    }
	  geom->DeclaredType = GAIA_POINT;
	  geom->Srid = srid;
      }
    if (shape == GAIA_SHP_POINTM)
      {
	  /* shape point M */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 24, shp->flShp);
	  if (rd != 24)
	      goto error;
	  x = gaiaImport64 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  y = gaiaImport64 (shp->BufShp + 8, GAIA_LITTLE_ENDIAN,
			    shp->endian_arch);
	  m = gaiaImport64 (shp->BufShp + 16, GAIA_LITTLE_ENDIAN,
			    shp->endian_arch);
	  if (shp->EffectiveDims == GAIA_XY_Z)
	    {
		geom = gaiaAllocGeomCollXYZ ();
		gaiaAddPointToGeomCollXYZ (geom, x, y, 0.0);
	    }
	  else if (shp->EffectiveDims == GAIA_XY_M)
	    {
		geom = gaiaAllocGeomCollXYM ();
		gaiaAddPointToGeomCollXYM (geom, x, y, m);
	    }
	  else if (shp->EffectiveDims == GAIA_XY_Z_M)
	    {
		geom = gaiaAllocGeomCollXYZM ();
		gaiaAddPointToGeomCollXYZM (geom, x, y, 0.0, m);
	    }
	  else
	    {
		geom = gaiaAllocGeomColl ();
		gaiaAddPointToGeomColl (geom, x, y);
	    }
	  geom->DeclaredType = GAIA_POINT;
	  geom->Srid = srid;
      }
    if (shape == GAIA_SHP_POLYLINE)
      {
	  /* shape polyline */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
			     shp->endian_arch);
	  base = 8 + (n * 4);
	  start = 0;
	  for (ind = 0; ind < n; ind++)
	    {
		if (ind < (n - 1))
		    end =
			gaiaImport32 (shp->BufShp + 8 + ((ind + 1) * 4),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    end = n1;
		points = end - start;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    line = gaiaAllocLinestringXYZ (points);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    line = gaiaAllocLinestringXYM (points);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    line = gaiaAllocLinestringXYZM (points);
		else
		    line = gaiaAllocLinestring (points);
		points = 0;
		for (iv = start; iv < end; iv++)
		  {
		      x = gaiaImport64 (shp->BufShp + base + (iv * 16),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      y = gaiaImport64 (shp->BufShp + base + (iv * 16) +
					8, GAIA_LITTLE_ENDIAN,
					shp->endian_arch);
		      if (shp->EffectiveDims == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (line->Coords, points, x, y, 0.0);
			}
		      else if (shp->EffectiveDims == GAIA_XY_M)
			{
			    gaiaSetPointXYM (line->Coords, points, x, y, 0.0);
			}
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (line->Coords, points, x, y,
					      0.0, 0.0);
			}
		      else
			{
			    gaiaSetPoint (line->Coords, points, x, y);
			}
		      start++;
		      points++;
		  }
		if (!geom)
		  {
		      if (shp->EffectiveDims == GAIA_XY_Z)
			  geom = gaiaAllocGeomCollXYZ ();
		      else if (shp->EffectiveDims == GAIA_XY_M)
			  geom = gaiaAllocGeomCollXYM ();
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			  geom = gaiaAllocGeomCollXYZM ();
		      else
			  geom = gaiaAllocGeomColl ();
		      if (shp->EffectiveType == GAIA_LINESTRING)
			  geom->DeclaredType = GAIA_LINESTRING;
		      else
			  geom->DeclaredType = GAIA_MULTILINESTRING;
		      geom->Srid = srid;
		  }
		gaiaInsertLinestringInGeomColl (geom, line);
	    }
      }
    if (shape == GAIA_SHP_POLYLINEZ)
      {
	  /* shape polyline Z */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
			     shp->endian_arch);
	  hasM = 0;
	  max_size = 38 + (2 * n) + (n1 * 16);	/* size [in 16 bits words !!!] ZM */
	  min_size = 30 + (2 * n) + (n1 * 12);	/* size [in 16 bits words !!!] Z-only */
	  if (sz < min_size)
	      goto error;
	  if (sz == max_size)
	      hasM = 1;
	  base = 8 + (n * 4);
	  baseZ = base + (n1 * 16) + 16;
	  baseM = baseZ + (n1 * 8) + 16;
	  start = 0;
	  for (ind = 0; ind < n; ind++)
	    {
		if (ind < (n - 1))
		    end =
			gaiaImport32 (shp->BufShp + 8 + ((ind + 1) * 4),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    end = n1;
		points = end - start;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    line = gaiaAllocLinestringXYZ (points);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    line = gaiaAllocLinestringXYM (points);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    line = gaiaAllocLinestringXYZM (points);
		else
		    line = gaiaAllocLinestring (points);
		points = 0;
		for (iv = start; iv < end; iv++)
		  {
		      x = gaiaImport64 (shp->BufShp + base + (iv * 16),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      y = gaiaImport64 (shp->BufShp + base + (iv * 16) +
					8, GAIA_LITTLE_ENDIAN,
					shp->endian_arch);
		      z = gaiaImport64 (shp->BufShp + baseZ + (iv * 8),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      if (hasM)
			  m = gaiaImport64 (shp->BufShp + baseM +
					    (iv * 8), GAIA_LITTLE_ENDIAN,
					    shp->endian_arch);
		      else
			  m = 0.0;
		      if (m < SHAPEFILE_NO_DATA)
			  m = 0.0;
		      if (shp->EffectiveDims == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (line->Coords, points, x, y, z);
			}
		      else if (shp->EffectiveDims == GAIA_XY_M)
			{
			    gaiaSetPointXYM (line->Coords, points, x, y, m);
			}
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (line->Coords, points, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (line->Coords, points, x, y);
			}
		      start++;
		      points++;
		  }
		if (!geom)
		  {
		      if (shp->EffectiveDims == GAIA_XY_Z)
			  geom = gaiaAllocGeomCollXYZ ();
		      else if (shp->EffectiveDims == GAIA_XY_M)
			  geom = gaiaAllocGeomCollXYM ();
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			  geom = gaiaAllocGeomCollXYZM ();
		      else
			  geom = gaiaAllocGeomColl ();
		      if (shp->EffectiveType == GAIA_LINESTRING)
			  geom->DeclaredType = GAIA_LINESTRING;
		      else
			  geom->DeclaredType = GAIA_MULTILINESTRING;
		      geom->Srid = srid;
		  }
		gaiaInsertLinestringInGeomColl (geom, line);
	    }
      }
    if (shape == GAIA_SHP_POLYLINEM)
      {
	  /* shape polyline M */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
			     shp->endian_arch);
	  hasM = 0;
	  max_size = 30 + (2 * n) + (n1 * 12);	/* size [in 16 bits words !!!] M */
	  min_size = 22 + (2 * n) + (n1 * 8);	/* size [in 16 bits words !!!] no-M */
	  if (sz < min_size)
	      goto error;
	  if (sz == max_size)
	      hasM = 1;
	  base = 8 + (n * 4);
	  baseM = base + (n1 * 16) + 16;
	  start = 0;
	  for (ind = 0; ind < n; ind++)
	    {
		if (ind < (n - 1))
		    end =
			gaiaImport32 (shp->BufShp + 8 + ((ind + 1) * 4),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    end = n1;
		points = end - start;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    line = gaiaAllocLinestringXYZ (points);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    line = gaiaAllocLinestringXYM (points);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    line = gaiaAllocLinestringXYZM (points);
		else
		    line = gaiaAllocLinestring (points);
		points = 0;
		for (iv = start; iv < end; iv++)
		  {
		      x = gaiaImport64 (shp->BufShp + base + (iv * 16),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      y = gaiaImport64 (shp->BufShp + base + (iv * 16) +
					8, GAIA_LITTLE_ENDIAN,
					shp->endian_arch);
		      if (hasM)
			  m = gaiaImport64 (shp->BufShp + baseM +
					    (iv * 8), GAIA_LITTLE_ENDIAN,
					    shp->endian_arch);
		      else
			  m = 0.0;
		      if (m < SHAPEFILE_NO_DATA)
			  m = 0.0;
		      if (shp->EffectiveDims == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (line->Coords, points, x, y, 0.0);
			}
		      else if (shp->EffectiveDims == GAIA_XY_M)
			{
			    gaiaSetPointXYM (line->Coords, points, x, y, m);
			}
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (line->Coords, points, x, y,
					      0.0, m);
			}
		      else
			{
			    gaiaSetPoint (line->Coords, points, x, y);
			}
		      start++;
		      points++;
		  }
		if (!geom)
		  {
		      if (shp->EffectiveDims == GAIA_XY_Z)
			  geom = gaiaAllocGeomCollXYZ ();
		      else if (shp->EffectiveDims == GAIA_XY_M)
			  geom = gaiaAllocGeomCollXYM ();
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			  geom = gaiaAllocGeomCollXYZM ();
		      else
			  geom = gaiaAllocGeomColl ();
		      if (shp->EffectiveType == GAIA_LINESTRING)
			  geom->DeclaredType = GAIA_LINESTRING;
		      else
			  geom->DeclaredType = GAIA_MULTILINESTRING;
		      geom->Srid = srid;
		  }
		gaiaInsertLinestringInGeomColl (geom, line);
	    }
      }
    if (shape == GAIA_SHP_POLYGON)
      {
	  /* shape polygon */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
			     shp->endian_arch);
	  base = 8 + (n * 4);
	  start = 0;
	  for (ind = 0; ind < n; ind++)
	    {
		if (ind < (n - 1))
		    end =
			gaiaImport32 (shp->BufShp + 8 + ((ind + 1) * 4),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    end = n1;
		points = end - start;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    ring = gaiaAllocRingXYZ (points);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    ring = gaiaAllocRingXYM (points);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    ring = gaiaAllocRingXYZM (points);
		else
		    ring = gaiaAllocRing (points);
		points = 0;
		for (iv = start; iv < end; iv++)
		  {
		      x = gaiaImport64 (shp->BufShp + base + (iv * 16),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      y = gaiaImport64 (shp->BufShp + base + (iv * 16) +
					8, GAIA_LITTLE_ENDIAN,
					shp->endian_arch);
		      if (shp->EffectiveDims == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, points, x, y, 0.0);
			}
		      else if (shp->EffectiveDims == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, points, x, y, 0.0);
			}
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, points, x, y,
					      0.0, 0.0);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, points, x, y);
			}
		      start++;
		      points++;
		  }
		if (!geom)
		  {
		      /* new geometry - new need to allocate a new POLYGON */
		      if (shp->EffectiveDims == GAIA_XY_Z)
			  geom = gaiaAllocGeomCollXYZ ();
		      else if (shp->EffectiveDims == GAIA_XY_M)
			  geom = gaiaAllocGeomCollXYM ();
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			  geom = gaiaAllocGeomCollXYZM ();
		      else
			  geom = gaiaAllocGeomColl ();
		      if (shp->EffectiveType == GAIA_POLYGON)
			  geom->DeclaredType = GAIA_POLYGON;
		      else
			  geom->DeclaredType = GAIA_MULTIPOLYGON;
		      geom->Srid = srid;
		      polyg = gaiaInsertPolygonInGeomColl (geom, ring);
		  }
		else
		  {
		      gaiaClockwise (ring);
		      if (ring->Clockwise)
			{
			    /* this one is a POLYGON exterior ring - we need to allocate e new POLYGON */
			    polyg = gaiaInsertPolygonInGeomColl (geom, ring);
			}
		      else
			{
			    /* adding an interior ring to current POLYGON */
			    gaiaAddRingToPolyg (polyg, ring);
			}
		  }
	    }
      }
    if (shape == GAIA_SHP_POLYGONZ)
      {
	  /* shape polygon Z */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
			     shp->endian_arch);
	  hasM = 0;
	  max_size = 38 + (2 * n) + (n1 * 16);	/* size [in 16 bits words !!!] ZM */
	  min_size = 30 + (2 * n) + (n1 * 12);	/* size [in 16 bits words !!!] Z-only */
	  if (sz < min_size)
	      goto error;
	  if (sz == max_size)
	      hasM = 1;
	  base = 8 + (n * 4);
	  baseZ = base + (n1 * 16) + 16;
	  baseM = baseZ + (n1 * 8) + 16;
	  start = 0;
	  for (ind = 0; ind < n; ind++)
	    {
		if (ind < (n - 1))
		    end =
			gaiaImport32 (shp->BufShp + 8 + ((ind + 1) * 4),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    end = n1;
		points = end - start;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    ring = gaiaAllocRingXYZ (points);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    ring = gaiaAllocRingXYM (points);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    ring = gaiaAllocRingXYZM (points);
		else
		    ring = gaiaAllocRing (points);
		points = 0;
		for (iv = start; iv < end; iv++)
		  {
		      x = gaiaImport64 (shp->BufShp + base + (iv * 16),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      y = gaiaImport64 (shp->BufShp + base + (iv * 16) +
					8, GAIA_LITTLE_ENDIAN,
					shp->endian_arch);
		      z = gaiaImport64 (shp->BufShp + baseZ + (iv * 8),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      if (hasM)
			  m = gaiaImport64 (shp->BufShp + baseM +
					    (iv * 8), GAIA_LITTLE_ENDIAN,
					    shp->endian_arch);
		      else
			  m = 0.0;
		      if (m < SHAPEFILE_NO_DATA)
			  m = 0.0;
		      if (shp->EffectiveDims == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, points, x, y, z);
			}
		      else if (shp->EffectiveDims == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, points, x, y, m);
			}
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, points, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, points, x, y);
			}
		      start++;
		      points++;
		  }
		if (!geom)
		  {
		      /* new geometry - new need to allocate a new POLYGON */
		      if (shp->EffectiveDims == GAIA_XY_Z)
			  geom = gaiaAllocGeomCollXYZ ();
		      else if (shp->EffectiveDims == GAIA_XY_M)
			  geom = gaiaAllocGeomCollXYM ();
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			  geom = gaiaAllocGeomCollXYZM ();
		      else
			  geom = gaiaAllocGeomColl ();
		      if (shp->EffectiveType == GAIA_POLYGON)
			  geom->DeclaredType = GAIA_POLYGON;
		      else
			  geom->DeclaredType = GAIA_MULTIPOLYGON;
		      geom->Srid = srid;
		      polyg = gaiaInsertPolygonInGeomColl (geom, ring);
		  }
		else
		  {
		      gaiaClockwise (ring);
		      if (ring->Clockwise)
			{
			    /* this one is a POLYGON exterior ring - we need to allocate e new POLYGON */
			    polyg = gaiaInsertPolygonInGeomColl (geom, ring);
			}
		      else
			{
			    /* adding an interior ring to current POLYGON */
			    gaiaAddRingToPolyg (polyg, ring);
			}
		  }
	    }
      }
    if (shape == GAIA_SHP_POLYGONM)
      {
	  /* shape polygon M */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
			     shp->endian_arch);
	  hasM = 0;
	  max_size = 30 + (2 * n) + (n1 * 12);	/* size [in 16 bits words !!!] M */
	  min_size = 22 + (2 * n) + (n1 * 8);	/* size [in 16 bits words !!!] no-M */
	  if (sz < min_size)
	      goto error;
	  if (sz == max_size)
	      hasM = 1;
	  base = 8 + (n * 4);
	  baseM = base + (n1 * 16) + 16;
	  start = 0;
	  for (ind = 0; ind < n; ind++)
	    {
		if (ind < (n - 1))
		    end =
			gaiaImport32 (shp->BufShp + 8 + ((ind + 1) * 4),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    end = n1;
		points = end - start;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    ring = gaiaAllocRingXYZ (points);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    ring = gaiaAllocRingXYM (points);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    ring = gaiaAllocRingXYZM (points);
		else
		    ring = gaiaAllocRing (points);
		points = 0;
		for (iv = start; iv < end; iv++)
		  {
		      x = gaiaImport64 (shp->BufShp + base + (iv * 16),
					GAIA_LITTLE_ENDIAN, shp->endian_arch);
		      y = gaiaImport64 (shp->BufShp + base + (iv * 16) +
					8, GAIA_LITTLE_ENDIAN,
					shp->endian_arch);
		      if (hasM)
			  m = gaiaImport64 (shp->BufShp + baseM +
					    (iv * 8), GAIA_LITTLE_ENDIAN,
					    shp->endian_arch);
		      m = 0.0;
		      if (m < SHAPEFILE_NO_DATA)
			  m = 0.0;
		      if (shp->EffectiveDims == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, points, x, y, 0.0);
			}
		      else if (shp->EffectiveDims == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, points, x, y, m);
			}
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, points, x, y,
					      0.0, m);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, points, x, y);
			}
		      start++;
		      points++;
		  }
		if (!geom)
		  {
		      /* new geometry - new need to allocate a new POLYGON */
		      if (shp->EffectiveDims == GAIA_XY_Z)
			  geom = gaiaAllocGeomCollXYZ ();
		      else if (shp->EffectiveDims == GAIA_XY_M)
			  geom = gaiaAllocGeomCollXYM ();
		      else if (shp->EffectiveDims == GAIA_XY_Z_M)
			  geom = gaiaAllocGeomCollXYZM ();
		      else
			  geom = gaiaAllocGeomColl ();
		      if (shp->EffectiveType == GAIA_POLYGON)
			  geom->DeclaredType = GAIA_POLYGON;
		      else
			  geom->DeclaredType = GAIA_MULTIPOLYGON;
		      geom->Srid = srid;
		      polyg = gaiaInsertPolygonInGeomColl (geom, ring);
		  }
		else
		  {
		      gaiaClockwise (ring);
		      if (ring->Clockwise)
			{
			    /* this one is a POLYGON exterior ring - we need to allocate e new POLYGON */
			    polyg = gaiaInsertPolygonInGeomColl (geom, ring);
			}
		      else
			{
			    /* adding an interior ring to current POLYGON */
			    gaiaAddRingToPolyg (polyg, ring);
			}
		  }
	    }
      }
    if (shape == GAIA_SHP_MULTIPOINT)
      {
	  /* shape multipoint */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  if (shp->EffectiveDims == GAIA_XY_Z)
	      geom = gaiaAllocGeomCollXYZ ();
	  else if (shp->EffectiveDims == GAIA_XY_M)
	      geom = gaiaAllocGeomCollXYM ();
	  else if (shp->EffectiveDims == GAIA_XY_Z_M)
	      geom = gaiaAllocGeomCollXYZM ();
	  else
	      geom = gaiaAllocGeomColl ();
	  geom->DeclaredType = GAIA_MULTIPOINT;
	  geom->Srid = srid;
	  for (iv = 0; iv < n; iv++)
	    {
		x = gaiaImport64 (shp->BufShp + 4 + (iv * 16),
				  GAIA_LITTLE_ENDIAN, shp->endian_arch);
		y = gaiaImport64 (shp->BufShp + 4 + (iv * 16) + 8,
				  GAIA_LITTLE_ENDIAN, shp->endian_arch);
		if (shp->EffectiveDims == GAIA_XY_Z)
		    gaiaAddPointToGeomCollXYZ (geom, x, y, 0.0);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    gaiaAddPointToGeomCollXYM (geom, x, y, 0.0);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    gaiaAddPointToGeomCollXYZM (geom, x, y, 0.0, 0.0);
		else
		    gaiaAddPointToGeomColl (geom, x, y);
	    }
      }
    if (shape == GAIA_SHP_MULTIPOINTZ)
      {
	  /* shape multipoint Z */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  hasM = 0;
	  max_size = 38 + (n * 16);	/* size [in 16 bits words !!!] ZM */
	  min_size = 30 + (n * 12);	/* size [in 16 bits words !!!] Z-only */
	  if (sz < min_size)
	      goto error;
	  if (sz == max_size)
	      hasM = 1;
	  baseZ = 4 + (n * 16) + 16;
	  baseM = baseZ + (n * 8) + 16;
	  if (shp->EffectiveDims == GAIA_XY_Z)
	      geom = gaiaAllocGeomCollXYZ ();
	  else if (shp->EffectiveDims == GAIA_XY_M)
	      geom = gaiaAllocGeomCollXYM ();
	  else if (shp->EffectiveDims == GAIA_XY_Z_M)
	      geom = gaiaAllocGeomCollXYZM ();
	  else
	      geom = gaiaAllocGeomColl ();
	  geom->DeclaredType = GAIA_MULTIPOINT;
	  geom->Srid = srid;
	  for (iv = 0; iv < n; iv++)
	    {
		x = gaiaImport64 (shp->BufShp + 4 + (iv * 16),
				  GAIA_LITTLE_ENDIAN, shp->endian_arch);
		y = gaiaImport64 (shp->BufShp + 4 + (iv * 16) + 8,
				  GAIA_LITTLE_ENDIAN, shp->endian_arch);
		z = gaiaImport64 (shp->BufShp + baseZ + (iv * 8),
				  GAIA_LITTLE_ENDIAN, shp->endian_arch);
		if (hasM)
		    m = gaiaImport64 (shp->BufShp + baseM + (iv * 8),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    m = 0.0;
		if (m < SHAPEFILE_NO_DATA)
		    m = 0.0;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    gaiaAddPointToGeomCollXYZ (geom, x, y, z);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    gaiaAddPointToGeomCollXYM (geom, x, y, m);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    gaiaAddPointToGeomCollXYZM (geom, x, y, z, m);
		else
		    gaiaAddPointToGeomColl (geom, x, y);
	    }
      }
    if (shape == GAIA_SHP_MULTIPOINTM)
      {
	  /* shape multipoint M */
	  rd = fread (shp->BufShp, sizeof (unsigned char), 32, shp->flShp);
	  if (rd != 32)
	      goto error;
	  rd = fread (shp->BufShp, sizeof (unsigned char), (sz * 2) - 36,
		      shp->flShp);
	  if (rd != (sz * 2) - 36)
	      goto error;
	  n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  hasM = 0;
	  max_size = 30 + (n * 12);	/* size [in 16 bits words !!!] M */
	  min_size = 22 + (n * 8);	/* size [in 16 bits words !!!] no-M */
	  if (sz < min_size)
	      goto error;
	  if (sz == max_size)
	      hasM = 1;
	  baseM = 4 + (n * 16) + 16;
	  if (shp->EffectiveDims == GAIA_XY_Z)
	      geom = gaiaAllocGeomCollXYZ ();
	  else if (shp->EffectiveDims == GAIA_XY_M)
	      geom = gaiaAllocGeomCollXYM ();
	  else if (shp->EffectiveDims == GAIA_XY_Z_M)
	      geom = gaiaAllocGeomCollXYZM ();
	  else
	      geom = gaiaAllocGeomColl ();
	  geom->DeclaredType = GAIA_MULTIPOINT;
	  geom->Srid = srid;
	  for (iv = 0; iv < n; iv++)
	    {
		x = gaiaImport64 (shp->BufShp + 4 + (iv * 16),
				  GAIA_LITTLE_ENDIAN, shp->endian_arch);
		y = gaiaImport64 (shp->BufShp + 4 + (iv * 16) + 8,
				  GAIA_LITTLE_ENDIAN, shp->endian_arch);
		if (hasM)
		    m = gaiaImport64 (shp->BufShp + baseM + (iv * 8),
				      GAIA_LITTLE_ENDIAN, shp->endian_arch);
		else
		    m = 0.0;
		if (m < SHAPEFILE_NO_DATA)
		    m = 0.0;
		if (shp->EffectiveDims == GAIA_XY_Z)
		    gaiaAddPointToGeomCollXYZ (geom, x, y, 0.0);
		else if (shp->EffectiveDims == GAIA_XY_M)
		    gaiaAddPointToGeomCollXYM (geom, x, y, m);
		else if (shp->EffectiveDims == GAIA_XY_Z_M)
		    gaiaAddPointToGeomCollXYZM (geom, x, y, 0.0, m);
		else
		    gaiaAddPointToGeomColl (geom, x, y);
	    }
      }
/* setting up the current SHP ENTITY */
  null_shape:
    gaiaResetDbfEntity (shp->Dbf);
    shp->Dbf->RowId = current_row;
    shp->Dbf->Geometry = geom;
/* fetching the DBF values */
    pFld = shp->Dbf->First;
    while (pFld)
      {
	  if (!parseDbfField (shp->BufDbf, shp->IconvObj, pFld))
	      goto conversion_error;
	  pFld = pFld->Next;
      }
    if (shp->LastError)
	free (shp->LastError);
    shp->LastError = NULL;
    return 1;
  eof:
    if (shp->LastError)
	free (shp->LastError);
    shp->LastError = NULL;
    return 0;
  error:
    if (shp->LastError)
	free (shp->LastError);
    sprintf (errMsg, "'%s' is corrupted / has invalid format", shp->Path);
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    return 0;
  conversion_error:
    if (shp->LastError)
	free (shp->LastError);
    sprintf (errMsg, "Invalid character sequence");
    len = strlen (errMsg);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, errMsg);
    return 0;
}

static void
gaiaSaneClockwise (gaiaPolygonPtr polyg)
{
/*
/ when exporting POLYGONs to SHAPEFILE, we must guarantee that:
/ - all EXTERIOR RING must be clockwise
/ - all INTERIOR RING must be anti-clockwise
/
/ this function checks for the above conditions,
/ and if needed inverts the rings
*/
    int ib;
    int iv;
    int iv2;
    double x;
    double y;
    double z;
    double m;
    gaiaRingPtr new_ring;
    gaiaRingPtr ring = polyg->Exterior;
    gaiaClockwise (ring);
    if (!(ring->Clockwise))
      {
	  /* exterior ring needs inversion */
	  if (ring->DimensionModel == GAIA_XY_Z)
	      new_ring = gaiaAllocRingXYZ (ring->Points);
	  else if (ring->DimensionModel == GAIA_XY_M)
	      new_ring = gaiaAllocRingXYM (ring->Points);
	  else if (ring->DimensionModel == GAIA_XY_Z_M)
	      new_ring = gaiaAllocRingXYZM (ring->Points);
	  else
	      new_ring = gaiaAllocRing (ring->Points);
	  iv2 = 0;
	  for (iv = ring->Points - 1; iv >= 0; iv--)
	    {
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		      gaiaSetPointXYZ (new_ring->Coords, iv2, x, y, z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		      gaiaSetPointXYM (new_ring->Coords, iv2, x, y, m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		      gaiaSetPointXYZM (new_ring->Coords, iv2, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		      gaiaSetPoint (new_ring->Coords, iv2, x, y);
		  }
		iv2++;
	    }
	  polyg->Exterior = new_ring;
	  gaiaFreeRing (ring);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  gaiaClockwise (ring);
	  if (ring->Clockwise)
	    {
		/* interior ring needs inversion */
		if (ring->DimensionModel == GAIA_XY_Z)
		    new_ring = gaiaAllocRingXYZ (ring->Points);
		else if (ring->DimensionModel == GAIA_XY_M)
		    new_ring = gaiaAllocRingXYM (ring->Points);
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		    new_ring = gaiaAllocRingXYZM (ring->Points);
		else
		    new_ring = gaiaAllocRing (ring->Points);
		iv2 = 0;
		for (iv = ring->Points - 1; iv >= 0; iv--)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			    gaiaSetPointXYZ (new_ring->Coords, iv2, x, y, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			    gaiaSetPointXYM (new_ring->Coords, iv2, x, y, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			    gaiaSetPointXYZM (new_ring->Coords, iv2, x, y,
					      z, m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			    gaiaSetPoint (new_ring->Coords, iv2, x, y);
			}
		      iv2++;
		  }
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (new_ring->Coords, iv, &x, &y, &z);
			    gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (new_ring->Coords, iv, &x, &y, &m);
			    gaiaSetPointXYM (ring->Coords, iv, x, y, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (new_ring->Coords, iv, &x, &y,
					      &z, &m);
			    gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
			}
		      else
			{
			    gaiaGetPoint (new_ring->Coords, iv, &x, &y);
			    gaiaSetPoint (ring->Coords, iv, x, y);
			}
		  }
		gaiaFreeRing (new_ring);
	    }
      }
}

GAIAGEO_DECLARE int
gaiaWriteShpEntity (gaiaShapefilePtr shp, gaiaDbfListPtr entity)
{
/* trying to write an entity into shapefile */
    char dummy[128];
    char fmt[16];
    int endian_arch = shp->endian_arch;
    gaiaDbfFieldPtr fld;
    int iv;
    int tot_ln;
    int tot_v;
    int tot_pts;
    int this_size;
    int ix;
    double x;
    double y;
    double z;
    double m;
    int hasM;
    double minZ;
    double maxZ;
    double minM;
    double maxM;
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    char buf[512];
    char utf8buf[2048];
/* writing the DBF record */
    memset (shp->BufDbf, '\0', shp->DbfReclen);
    *(shp->BufDbf) = ' ';	/* in DBF first byte of each row marks for validity or deletion */
    fld = entity->First;
    while (fld)
      {
	  /* transferring field values */
	  switch (fld->Type)
	    {
	    case 'L':
		if (!(fld->Value))
		    *(shp->BufDbf + fld->Offset) = '?';
		else if (fld->Value->Type != GAIA_INT_VALUE)
		    *(shp->BufDbf + fld->Offset + 1) = '?';
		else
		  {
		      if (fld->Value->IntValue == 0)
			  *(shp->BufDbf + fld->Offset + 1) = 'N';
		      else
			  *(shp->BufDbf + fld->Offset + 1) = 'Y';
		  }
		break;
	    case 'D':
		memset (shp->BufDbf + fld->Offset + 1, '0', 8);
		if (fld->Value)
		  {
		      if (fld->Value->Type == GAIA_TEXT_VALUE)
			{
			    if (strlen (fld->Value->TxtValue) == 8)
				memcpy (shp->BufDbf + fld->Offset + 1,
					fld->Value->TxtValue, 8);
			}
		  }
		break;
	    case 'C':
		memset (shp->BufDbf + fld->Offset + 1, ' ', fld->Length);
		if (fld->Value)
		  {
		      if (fld->Value->Type == GAIA_TEXT_VALUE)
			{
			    strcpy (buf, fld->Value->TxtValue);
			    len = strlen (buf);
			    utf8len = 2048;
			    pBuf = buf;
			    pUtf8buf = utf8buf;
			    if (iconv
				((iconv_t) (shp->IconvObj), &pBuf, &len,
				 &pUtf8buf, &utf8len) == (size_t) (-1))
				goto conversion_error;
			    memcpy (buf, utf8buf, 2048 - utf8len);
			    buf[2048 - utf8len] = '\0';
			    if (strlen (buf) < fld->Length)
				memcpy (shp->BufDbf + fld->Offset + 1, buf,
					strlen (buf));
			    else
				memcpy (shp->BufDbf + fld->Offset + 1, buf,
					fld->Length);
			}
		  }
		break;
	    case 'N':
		memset (shp->BufDbf + fld->Offset + 1, '\0', fld->Length);
		if (fld->Value)
		  {
		      if (fld->Value->Type == GAIA_INT_VALUE)
			{
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT - M$ runtime doesn't supports %lld for 64 bits */
			    sprintf (dummy, "%I64d", fld->Value->IntValue);
#else
			    sprintf (dummy, "%lld", fld->Value->IntValue);
#endif
			    if (strlen (dummy) <= fld->Length)
				memcpy (shp->BufDbf + fld->Offset + 1,
					dummy, strlen (dummy));
			}
		      if (fld->Value->Type == GAIA_DOUBLE_VALUE)
			{
			    sprintf (fmt, "%%1.%df", fld->Decimals);
			    sprintf (dummy, fmt, fld->Value->DblValue);
			    if (strlen (dummy) <= fld->Length)
				memcpy (shp->BufDbf + fld->Offset + 1,
					dummy, strlen (dummy));
			}
		  }
		break;
	    };
	  fld = fld->Next;
      }
    if (!(entity->Geometry))
      {
	  /* exporting a NULL Shape */
	  gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
	  gaiaExport32 (shp->BufShp + 4, 2, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
	  fwrite (shp->BufShp, 1, 8, shp->flShx);
	  (shp->ShxSize) += 4;	/* updating current SHX file position [in 16 bits words !!!] */
	  gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
	  gaiaExport32 (shp->BufShp + 4, 2, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
	  gaiaExport32 (shp->BufShp + 8, GAIA_SHP_NULL, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = NULL */
	  fwrite (shp->BufShp, 1, 12, shp->flShp);
	  (shp->ShpSize) += 6;	/* updating current SHP file position [in 16 bits words !!!] */
      }
    else
      {
	  /* updates the shapefile main MBR-BBOX */
	  gaiaMbrGeometry (entity->Geometry);
	  if (entity->Geometry->MinX < shp->MinX)
	      shp->MinX = entity->Geometry->MinX;
	  if (entity->Geometry->MaxX > shp->MaxX)
	      shp->MaxX = entity->Geometry->MaxX;
	  if (entity->Geometry->MinY < shp->MinY)
	      shp->MinY = entity->Geometry->MinY;
	  if (entity->Geometry->MaxY > shp->MaxY)
	      shp->MaxY = entity->Geometry->MaxY;
	  if (shp->Shape == GAIA_SHP_POINT)
	    {
		/* this one is expected to be a POINT */
		gaiaPointPtr pt = entity->Geometry->FirstPoint;
		if (!pt)
		  {
		      strcpy (dummy,
			      "a POINT is expected, but there is no POINT in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		/* inserting POINT entity into SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, 10, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;	/* updating current SHX file position [in 16 bits words !!!] */
		/* inserting POINT into SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, 10, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POINT, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POINT */
		gaiaExport64 (shp->BufShp + 12, pt->X, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports X coordinate */
		gaiaExport64 (shp->BufShp + 20, pt->Y, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports Y coordinate */
		fwrite (shp->BufShp, 1, 28, shp->flShp);
		(shp->ShpSize) += 14;	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_POINTZ)
	    {
		/* this one is expected to be a POINT Z */
		gaiaPointPtr pt = entity->Geometry->FirstPoint;
		if (!pt)
		  {
		      strcpy (dummy,
			      "a POINT is expected, but there is no POINT in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		/* inserting POINT Z entity into SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, 18, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;	/* updating current SHX file position [in 16 bits words !!!] */
		/* inserting POINT into SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, 18, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POINTZ, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POINT Z */
		gaiaExport64 (shp->BufShp + 12, pt->X, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports X coordinate */
		gaiaExport64 (shp->BufShp + 20, pt->Y, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports Y coordinate */
		gaiaExport64 (shp->BufShp + 28, pt->Z, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports Z coordinate */
		gaiaExport64 (shp->BufShp + 36, pt->M, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports M coordinate */
		fwrite (shp->BufShp, 1, 44, shp->flShp);
		(shp->ShpSize) += 22;	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_POINTM)
	    {
		/* this one is expected to be a POINT M */
		gaiaPointPtr pt = entity->Geometry->FirstPoint;
		if (!pt)
		  {
		      strcpy (dummy,
			      "a POINT is expected, but there is no POINT in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		/* inserting POINT entity into SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, 14, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;	/* updating current SHX file position [in 16 bits words !!!] */
		/* inserting POINT into SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, 14, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POINTM, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POINT M */
		gaiaExport64 (shp->BufShp + 12, pt->X, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports X coordinate */
		gaiaExport64 (shp->BufShp + 20, pt->Y, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports Y coordinate */
		gaiaExport64 (shp->BufShp + 28, pt->Y, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports M coordinate */
		fwrite (shp->BufShp, 1, 36, shp->flShp);
		(shp->ShpSize) += 18;	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_POLYLINE)
	    {
		/* this one is expected to be a LINESTRING / MULTILINESTRING */
		gaiaLinestringPtr line;
		tot_ln = 0;
		tot_v = 0;
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* computes # lines and total # points */
		      tot_v += line->Points;
		      tot_ln++;
		      line = line->Next;
		  }
		if (!tot_ln)
		  {
		      strcpy (dummy,
			      "a LINESTRING is expected, but there is no LINESTRING in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		this_size = 22 + (2 * tot_ln) + (tot_v * 8);	/* size [in 16 bits words !!!] for this SHP entity */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting LINESTRING or MULTILINESTRING in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting LINESTRING or MULTILINESTRING in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POLYLINE, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POLYLINE */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_ln, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # lines in this polyline */
		gaiaExport32 (shp->BufShp + 48, tot_v, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		tot_v = 0;	/* resets points counter */
		ix = 52;	/* sets current buffer offset */
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports start point index for each line */
		      gaiaExport32 (shp->BufShp + ix, tot_v,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      tot_v += line->Points;
		      ix += 4;
		      line = line->Next;
		  }
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports points for each line */
		      for (iv = 0; iv < line->Points; iv++)
			{
			    /* exports a POINT [x,y] */
			    if (line->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (line->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (line->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (line->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (line->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (line->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (line->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, x,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			    gaiaExport64 (shp->BufShp + ix, y,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      line = line->Next;
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_POLYLINEZ)
	    {
		/* this one is expected to be a LINESTRING / MULTILINESTRING Z */
		gaiaLinestringPtr line;
		gaiaZRangeGeometry (entity->Geometry, &minZ, &maxZ);
		gaiaMRangeGeometry (entity->Geometry, &minM, &maxM);
		tot_ln = 0;
		tot_v = 0;
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* computes # lines and total # points */
		      tot_v += line->Points;
		      tot_ln++;
		      line = line->Next;
		  }
		if (!tot_ln)
		  {
		      strcpy (dummy,
			      "a LINESTRING is expected, but there is no LINESTRING in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		hasM = 0;
		if (shp->EffectiveDims == GAIA_XY_M
		    || shp->EffectiveDims == GAIA_XY_Z_M)
		    hasM = 1;
		if (hasM)
		    this_size = 38 + (2 * tot_ln) + (tot_v * 16);	/* size [in 16 bits words !!!] ZM */
		else
		    this_size = 30 + (2 * tot_ln) + (tot_v * 12);	/* size [in 16 bits words !!!] Z-only */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting LINESTRING or MULTILINESTRING in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting LINESTRING or MULTILINESTRING in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POLYLINEZ, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POLYLINE Z */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_ln, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # lines in this polyline */
		gaiaExport32 (shp->BufShp + 48, tot_v, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		tot_v = 0;	/* resets points counter */
		ix = 52;	/* sets current buffer offset */
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports start point index for each line */
		      gaiaExport32 (shp->BufShp + ix, tot_v,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      tot_v += line->Points;
		      ix += 4;
		      line = line->Next;
		  }
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports points for each line */
		      for (iv = 0; iv < line->Points; iv++)
			{
			    /* exports a POINT [x,y] */
			    if (line->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (line->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (line->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (line->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (line->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (line->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (line->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, x,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			    gaiaExport64 (shp->BufShp + ix, y,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      line = line->Next;
		  }
		/* exporting the Z-range [min/max] */
		gaiaExport64 (shp->BufShp + ix, minZ, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		gaiaExport64 (shp->BufShp + ix, maxZ, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports Z-values for each line */
		      for (iv = 0; iv < line->Points; iv++)
			{
			    /* exports Z-value */
			    z = 0.0;
			    if (line->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (line->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (line->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (line->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (line->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (line->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (line->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, z,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      line = line->Next;
		  }
		if (hasM)
		  {
		      /* exporting the M-range [min/max] */
		      gaiaExport64 (shp->BufShp + ix, minM,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      gaiaExport64 (shp->BufShp + ix, maxM,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      line = entity->Geometry->FirstLinestring;
		      while (line)
			{
			    /* exports M-values for each line */
			    for (iv = 0; iv < line->Points; iv++)
			      {
				  /* exports M-value */
				  m = 0.0;
				  if (line->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (line->Coords, iv,
							 &x, &y, &z);
				    }
				  else if (line->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (line->Coords, iv,
							 &x, &y, &m);
				    }
				  else if (line->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (line->Coords, iv,
							  &x, &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (line->Coords, iv, &x, &y);
				    }
				  gaiaExport64 (shp->BufShp + ix, m,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
			      }
			    line = line->Next;
			}
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_POLYLINEM)
	    {
		/* this one is expected to be a LINESTRING / MULTILINESTRING M */
		gaiaLinestringPtr line;
		gaiaMRangeGeometry (entity->Geometry, &minM, &maxM);
		tot_ln = 0;
		tot_v = 0;
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* computes # lines and total # points */
		      tot_v += line->Points;
		      tot_ln++;
		      line = line->Next;
		  }
		if (!tot_ln)
		  {
		      strcpy (dummy,
			      "a LINESTRING is expected, but there is no LINESTRING in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		this_size = 30 + (2 * tot_ln) + (tot_v * 12);	/* size [in 16 bits words !!!] for this SHP entity */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting LINESTRING or MULTILINESTRING in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting LINESTRING or MULTILINESTRING in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POLYLINEM, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POLYLINE M */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_ln, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # lines in this polyline */
		gaiaExport32 (shp->BufShp + 48, tot_v, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		tot_v = 0;	/* resets points counter */
		ix = 52;	/* sets current buffer offset */
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports start point index for each line */
		      gaiaExport32 (shp->BufShp + ix, tot_v,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      tot_v += line->Points;
		      ix += 4;
		      line = line->Next;
		  }
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports points for each line */
		      for (iv = 0; iv < line->Points; iv++)
			{
			    /* exports a POINT [x,y] */
			    if (line->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (line->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (line->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (line->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (line->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (line->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (line->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, x,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			    gaiaExport64 (shp->BufShp + ix, y,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      line = line->Next;
		  }
		/* exporting the M-range [min/max] */
		gaiaExport64 (shp->BufShp + ix, minM, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		gaiaExport64 (shp->BufShp + ix, maxM, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		line = entity->Geometry->FirstLinestring;
		while (line)
		  {
		      /* exports M-values for each line */
		      for (iv = 0; iv < line->Points; iv++)
			{
			    /* exports M-value */
			    m = 0.0;
			    if (line->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (line->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (line->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (line->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (line->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (line->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (line->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, m,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      line = line->Next;
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_POLYGON)
	    {
		/* this one is expected to be a POLYGON or a MULTIPOLYGON */
		gaiaPolygonPtr polyg;
		gaiaRingPtr ring;
		int ib;
		tot_ln = 0;
		tot_v = 0;
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* computes # rings and total # points */
		      gaiaSaneClockwise (polyg);	/* we must assure that exterior ring is clockwise, and interior rings are anti-clockwise */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      tot_v += ring->Points;
		      tot_ln++;
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    tot_v += ring->Points;
			    tot_ln++;
			}
		      polyg = polyg->Next;
		  }
		if (!tot_ln)
		  {
		      strcpy (dummy,
			      "a POLYGON is expected, but there is no POLYGON in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		this_size = 22 + (2 * tot_ln) + (tot_v * 8);	/* size [in 16 bits words !!!] for this SHP entity */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting POLYGON or MULTIPOLYGON in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting POLYGON or MULTIPOLYGON in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POLYGON, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POLYGON */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_ln, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # rings in this polygon */
		gaiaExport32 (shp->BufShp + 48, tot_v, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		tot_v = 0;	/* resets points counter */
		ix = 52;	/* sets current buffer offset */
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports start point index for each line */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      gaiaExport32 (shp->BufShp + ix, tot_v,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      tot_v += ring->Points;
		      ix += 4;
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    gaiaExport32 (shp->BufShp + ix, tot_v,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    tot_v += ring->Points;
			    ix += 4;
			}
		      polyg = polyg->Next;
		  }
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports points for each ring */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      for (iv = 0; iv < ring->Points; iv++)
			{
			    /* exports a POINT [x,y] - exterior ring */
			    if (ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (ring->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (ring->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (ring->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (ring->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, x,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			    gaiaExport64 (shp->BufShp + ix, y,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    for (iv = 0; iv < ring->Points; iv++)
			      {
				  /* exports a POINT [x,y] - interior ring */
				  if (ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ring->Coords, iv,
							 &x, &y, &z);
				    }
				  else if (ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ring->Coords, iv,
							 &x, &y, &m);
				    }
				  else if (ring->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ring->Coords, iv,
							  &x, &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ring->Coords, iv, &x, &y);
				    }
				  gaiaExport64 (shp->BufShp + ix, x,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
				  gaiaExport64 (shp->BufShp + ix, y,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
			      }
			}
		      polyg = polyg->Next;
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);
	    }
	  if (shp->Shape == GAIA_SHP_POLYGONZ)
	    {
		/* this one is expected to be a POLYGON or a MULTIPOLYGON Z */
		gaiaPolygonPtr polyg;
		gaiaRingPtr ring;
		int ib;
		gaiaZRangeGeometry (entity->Geometry, &minZ, &maxZ);
		gaiaMRangeGeometry (entity->Geometry, &minM, &maxM);
		tot_ln = 0;
		tot_v = 0;
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* computes # rings and total # points */
		      gaiaSaneClockwise (polyg);	/* we must assure that exterior ring is clockwise, and interior rings are anti-clockwise */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      tot_v += ring->Points;
		      tot_ln++;
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    tot_v += ring->Points;
			    tot_ln++;
			}
		      polyg = polyg->Next;
		  }
		if (!tot_ln)
		  {
		      strcpy (dummy,
			      "a POLYGON is expected, but there is no POLYGON in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		hasM = 0;
		if (shp->EffectiveDims == GAIA_XY_M
		    || shp->EffectiveDims == GAIA_XY_Z_M)
		    hasM = 1;
		if (hasM)
		    this_size = 38 + (2 * tot_ln) + (tot_v * 16);	/* size [in 16 bits words !!!] ZM */
		else
		    this_size = 30 + (2 * tot_ln) + (tot_v * 12);	/* size [in 16 bits words !!!] Z-only */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting POLYGON or MULTIPOLYGON in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting POLYGON or MULTIPOLYGON in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POLYGONZ, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POLYGON Z */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_ln, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # rings in this polygon */
		gaiaExport32 (shp->BufShp + 48, tot_v, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		tot_v = 0;	/* resets points counter */
		ix = 52;	/* sets current buffer offset */
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports start point index for each line */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      gaiaExport32 (shp->BufShp + ix, tot_v,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      tot_v += ring->Points;
		      ix += 4;
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    gaiaExport32 (shp->BufShp + ix, tot_v,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    tot_v += ring->Points;
			    ix += 4;
			}
		      polyg = polyg->Next;
		  }
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports points for each ring */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      for (iv = 0; iv < ring->Points; iv++)
			{
			    /* exports a POINT [x,y] - exterior ring */
			    if (ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (ring->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (ring->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (ring->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (ring->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, x,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			    gaiaExport64 (shp->BufShp + ix, y,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    for (iv = 0; iv < ring->Points; iv++)
			      {
				  /* exports a POINT [x,y] - interior ring */
				  if (ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ring->Coords, iv,
							 &x, &y, &z);
				    }
				  else if (ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ring->Coords, iv,
							 &x, &y, &m);
				    }
				  else if (ring->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ring->Coords, iv,
							  &x, &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ring->Coords, iv, &x, &y);
				    }
				  gaiaExport64 (shp->BufShp + ix, x,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
				  gaiaExport64 (shp->BufShp + ix, y,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
			      }
			}
		      polyg = polyg->Next;
		  }
		/* exporting the Z-range [min/max] */
		gaiaExport64 (shp->BufShp + ix, minZ, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		gaiaExport64 (shp->BufShp + ix, maxZ, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports Z-values for each ring */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      for (iv = 0; iv < ring->Points; iv++)
			{
			    /* exports Z-values - exterior ring */
			    z = 0.0;
			    if (ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (ring->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (ring->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (ring->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (ring->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, z,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    for (iv = 0; iv < ring->Points; iv++)
			      {
				  /* exports Z-values - interior ring */
				  z = 0.0;
				  if (ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ring->Coords, iv,
							 &x, &y, &z);
				    }
				  else if (ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ring->Coords, iv,
							 &x, &y, &m);
				    }
				  else if (ring->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ring->Coords, iv,
							  &x, &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ring->Coords, iv, &x, &y);
				    }
				  gaiaExport64 (shp->BufShp + ix, z,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
			      }
			}
		      polyg = polyg->Next;
		  }
		if (hasM)
		  {
		      /* exporting the M-range [min/max] */
		      gaiaExport64 (shp->BufShp + ix, minM,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      gaiaExport64 (shp->BufShp + ix, maxM,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      polyg = entity->Geometry->FirstPolygon;
		      while (polyg)
			{
			    /* exports M-values for each ring */
			    ring = polyg->Exterior;	/* this one is the exterior ring */
			    for (iv = 0; iv < ring->Points; iv++)
			      {
				  /* exports M-values - exterior ring */
				  m = 0.0;
				  if (ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ring->Coords, iv,
							 &x, &y, &z);
				    }
				  else if (ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ring->Coords, iv,
							 &x, &y, &m);
				    }
				  else if (ring->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ring->Coords, iv,
							  &x, &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ring->Coords, iv, &x, &y);
				    }
				  gaiaExport64 (shp->BufShp + ix, m,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
			      }
			    for (ib = 0; ib < polyg->NumInteriors; ib++)
			      {
				  /* that ones are the interior rings */
				  ring = polyg->Interiors + ib;
				  for (iv = 0; iv < ring->Points; iv++)
				    {
					/* exports M-values - interior ring */
					m = 0.0;
					if (ring->DimensionModel == GAIA_XY_Z)
					  {
					      gaiaGetPointXYZ (ring->Coords,
							       iv, &x, &y, &z);
					  }
					else if (ring->DimensionModel ==
						 GAIA_XY_M)
					  {
					      gaiaGetPointXYM (ring->Coords,
							       iv, &x, &y, &m);
					  }
					else if (ring->DimensionModel ==
						 GAIA_XY_Z_M)
					  {
					      gaiaGetPointXYZM (ring->Coords,
								iv, &x, &y, &z,
								&m);
					  }
					else
					  {
					      gaiaGetPoint (ring->Coords,
							    iv, &x, &y);
					  }
					gaiaExport64 (shp->BufShp + ix, m,
						      GAIA_LITTLE_ENDIAN,
						      endian_arch);
					ix += 8;
				    }
			      }
			    polyg = polyg->Next;
			}
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);
	    }
	  if (shp->Shape == GAIA_SHP_POLYGONM)
	    {
		/* this one is expected to be a POLYGON or a MULTIPOLYGON M */
		gaiaPolygonPtr polyg;
		gaiaRingPtr ring;
		int ib;
		gaiaMRangeGeometry (entity->Geometry, &minM, &maxM);
		tot_ln = 0;
		tot_v = 0;
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* computes # rings and total # points */
		      gaiaSaneClockwise (polyg);	/* we must assure that exterior ring is clockwise, and interior rings are anti-clockwise */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      tot_v += ring->Points;
		      tot_ln++;
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    tot_v += ring->Points;
			    tot_ln++;
			}
		      polyg = polyg->Next;
		  }
		if (!tot_ln)
		  {
		      strcpy (dummy,
			      "a POLYGON is expected, but there is no POLYGON in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		this_size = 30 + (2 * tot_ln) + (tot_v * 12);	/* size [in 16 bits words !!!] for this SHP entity */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting POLYGON or MULTIPOLYGON in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting POLYGON or MULTIPOLYGON in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_POLYGONM, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = POLYGON M */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_ln, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # rings in this polygon */
		gaiaExport32 (shp->BufShp + 48, tot_v, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		tot_v = 0;	/* resets points counter */
		ix = 52;	/* sets current buffer offset */
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports start point index for each line */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      gaiaExport32 (shp->BufShp + ix, tot_v,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      tot_v += ring->Points;
		      ix += 4;
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    gaiaExport32 (shp->BufShp + ix, tot_v,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    tot_v += ring->Points;
			    ix += 4;
			}
		      polyg = polyg->Next;
		  }
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports points for each ring */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      for (iv = 0; iv < ring->Points; iv++)
			{
			    /* exports a POINT [x,y] - exterior ring */
			    if (ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (ring->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (ring->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (ring->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (ring->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, x,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			    gaiaExport64 (shp->BufShp + ix, y,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    for (iv = 0; iv < ring->Points; iv++)
			      {
				  /* exports a POINT [x,y] - interior ring */
				  if (ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ring->Coords, iv,
							 &x, &y, &z);
				    }
				  else if (ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ring->Coords, iv,
							 &x, &y, &m);
				    }
				  else if (ring->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ring->Coords, iv,
							  &x, &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ring->Coords, iv, &x, &y);
				    }
				  gaiaExport64 (shp->BufShp + ix, x,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
				  gaiaExport64 (shp->BufShp + ix, y,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
			      }
			}
		      polyg = polyg->Next;
		  }
		/* exporting the M-range [min/max] */
		gaiaExport64 (shp->BufShp + ix, minM, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		gaiaExport64 (shp->BufShp + ix, maxM, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		polyg = entity->Geometry->FirstPolygon;
		while (polyg)
		  {
		      /* exports M-values for each ring */
		      ring = polyg->Exterior;	/* this one is the exterior ring */
		      for (iv = 0; iv < ring->Points; iv++)
			{
			    /* exports M-values - exterior ring */
			    m = 0.0;
			    if (ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (ring->Coords, iv, &x, &y,
						   &z);
			      }
			    else if (ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (ring->Coords, iv, &x, &y,
						   &m);
			      }
			    else if (ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (ring->Coords, iv, &x,
						    &y, &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (ring->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (shp->BufShp + ix, m,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			}
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    /* that ones are the interior rings */
			    ring = polyg->Interiors + ib;
			    for (iv = 0; iv < ring->Points; iv++)
			      {
				  /* exports M-values - interior ring */
				  m = 0.0;
				  if (ring->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ring->Coords, iv,
							 &x, &y, &z);
				    }
				  else if (ring->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ring->Coords, iv,
							 &x, &y, &m);
				    }
				  else if (ring->DimensionModel == GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ring->Coords, iv,
							  &x, &y, &z, &m);
				    }
				  else
				    {
					gaiaGetPoint (ring->Coords, iv, &x, &y);
				    }
				  gaiaExport64 (shp->BufShp + ix, m,
						GAIA_LITTLE_ENDIAN,
						endian_arch);
				  ix += 8;
			      }
			}
		      polyg = polyg->Next;
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);
	    }
	  if (shp->Shape == GAIA_SHP_MULTIPOINT)
	    {
		/* this one is expected to be a MULTIPOINT */
		gaiaPointPtr pt;
		tot_pts = 0;
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* computes # points */
		      tot_pts++;
		      pt = pt->Next;
		  }
		if (!tot_pts)
		  {
		      strcpy (dummy,
			      "a MULTIPOINT is expected, but there is no POINT/MULTIPOINT in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		this_size = 20 + (tot_pts * 8);	/* size [in 16 bits words !!!] for this SHP entity */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting MULTIPOINT in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting MULTIPOINT in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_MULTIPOINT, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = MULTIPOINT */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_pts, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		ix = 48;	/* sets current buffer offset */
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* exports each point */
		      gaiaExport64 (shp->BufShp + ix, pt->X,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      gaiaExport64 (shp->BufShp + ix, pt->Y,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      pt = pt->Next;
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_MULTIPOINTZ)
	    {
		/* this one is expected to be a MULTIPOINT Z */
		gaiaPointPtr pt;
		gaiaZRangeGeometry (entity->Geometry, &minZ, &maxZ);
		gaiaMRangeGeometry (entity->Geometry, &minM, &maxM);
		tot_pts = 0;
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* computes # points */
		      tot_pts++;
		      pt = pt->Next;
		  }
		if (!tot_pts)
		  {
		      strcpy (dummy,
			      "a MULTIPOINT is expected, but there is no POINT/MULTIPOINT in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		hasM = 0;
		if (shp->EffectiveDims == GAIA_XY_M
		    || shp->EffectiveDims == GAIA_XY_Z_M)
		    hasM = 1;
		if (hasM)
		    this_size = 36 + (tot_pts * 16);	/* size [in 16 bits words !!!] ZM */
		else
		    this_size = 28 + (tot_pts * 12);	/* size [in 16 bits words !!!] Z-only */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting MULTIPOINT in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting MULTIPOINT in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_MULTIPOINTZ, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = MULTIPOINT Z */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_pts, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		ix = 48;	/* sets current buffer offset */
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* exports each point */
		      gaiaExport64 (shp->BufShp + ix, pt->X,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      gaiaExport64 (shp->BufShp + ix, pt->Y,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      pt = pt->Next;
		  }
		/* exporting the Z-range [min/max] */
		gaiaExport64 (shp->BufShp + ix, minZ, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		gaiaExport64 (shp->BufShp + ix, maxZ, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* exports Z-values */
		      gaiaExport64 (shp->BufShp + ix, pt->Z,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      pt = pt->Next;
		  }
		if (hasM)
		  {
		      /* exporting the M-range [min/max] */
		      gaiaExport64 (shp->BufShp + ix, minM,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      gaiaExport64 (shp->BufShp + ix, maxM,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      pt = entity->Geometry->FirstPoint;
		      while (pt)
			{
			    /* exports M-values */
			    gaiaExport64 (shp->BufShp + ix, pt->M,
					  GAIA_LITTLE_ENDIAN, endian_arch);
			    ix += 8;
			    pt = pt->Next;
			}
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);	/* updating current SHP file position [in 16 bits words !!!] */
	    }
	  if (shp->Shape == GAIA_SHP_MULTIPOINTM)
	    {
		/* this one is expected to be a MULTIPOINT M */
		gaiaPointPtr pt;
		gaiaMRangeGeometry (entity->Geometry, &minM, &maxM);
		tot_pts = 0;
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* computes # points */
		      tot_pts++;
		      pt = pt->Next;
		  }
		if (!tot_pts)
		  {
		      strcpy (dummy,
			      "a MULTIPOINT is expected, but there is no POINT/MULTIPOINT in geometry");
		      if (shp->LastError)
			  free (shp->LastError);
		      len = strlen (dummy);
		      shp->LastError = malloc (len + 1);
		      strcpy (shp->LastError, dummy);
		      return 0;
		  }
		this_size = 28 + (tot_pts * 12);	/* size [in 16 bits words !!!] for this SHP entity */
		if ((this_size * 2) + 1024 > shp->ShpBfsz)
		  {
		      /* current buffer is too small; we need to allocate a bigger one */
		      free (shp->BufShp);
		      shp->ShpBfsz = (this_size * 2) + 1024;
		      shp->BufShp = malloc (shp->ShpBfsz);
		  }
		/* inserting MULTIPOINT in SHX file */
		gaiaExport32 (shp->BufShp, shp->ShpSize, GAIA_BIG_ENDIAN, endian_arch);	/* exports current SHP file position */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entitiy size [in 16 bits words !!!] */
		fwrite (shp->BufShp, 1, 8, shp->flShx);
		(shp->ShxSize) += 4;
		/* inserting MULTIPOINT in SHP file */
		gaiaExport32 (shp->BufShp, shp->DbfRecno + 1, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity ID */
		gaiaExport32 (shp->BufShp + 4, this_size, GAIA_BIG_ENDIAN, endian_arch);	/* exports entity size [in 16 bits words !!!] */
		gaiaExport32 (shp->BufShp + 8, GAIA_SHP_MULTIPOINTM, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports geometry type = MULTIPOINT M */
		gaiaExport64 (shp->BufShp + 12, entity->Geometry->MinX, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the MBR for this geometry */
		gaiaExport64 (shp->BufShp + 20, entity->Geometry->MinY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 28, entity->Geometry->MaxX,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport64 (shp->BufShp + 36, entity->Geometry->MaxY,
			      GAIA_LITTLE_ENDIAN, endian_arch);
		gaiaExport32 (shp->BufShp + 44, tot_pts, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports total # points */
		ix = 48;	/* sets current buffer offset */
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* exports each point */
		      gaiaExport64 (shp->BufShp + ix, pt->X,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      gaiaExport64 (shp->BufShp + ix, pt->Y,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      pt = pt->Next;
		  }
		/* exporting the M-range [min/max] */
		gaiaExport64 (shp->BufShp + ix, minM, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		gaiaExport64 (shp->BufShp + ix, maxM, GAIA_LITTLE_ENDIAN,
			      endian_arch);
		ix += 8;
		pt = entity->Geometry->FirstPoint;
		while (pt)
		  {
		      /* exports M-values */
		      gaiaExport64 (shp->BufShp + ix, pt->M,
				    GAIA_LITTLE_ENDIAN, endian_arch);
		      ix += 8;
		      pt = pt->Next;
		  }
		fwrite (shp->BufShp, 1, ix, shp->flShp);
		(shp->ShpSize) += (ix / 2);	/* updating current SHP file position [in 16 bits words !!!] */
	    }
      }
/* inserting entity in DBF file */
    fwrite (shp->BufDbf, 1, shp->DbfReclen, shp->flDbf);
    (shp->DbfRecno)++;
    return 1;
  conversion_error:
    if (shp->LastError)
	free (shp->LastError);
    sprintf (dummy, "Invalid character sequence");
    len = strlen (dummy);
    shp->LastError = malloc (len + 1);
    strcpy (shp->LastError, dummy);
    return 0;
}

GAIAGEO_DECLARE void
gaiaFlushShpHeaders (gaiaShapefilePtr shp)
{
/* updates the various file headers */
    FILE *fl_shp = shp->flShp;
    FILE *fl_shx = shp->flShx;
    FILE *fl_dbf = shp->flDbf;
    int shp_size = shp->ShpSize;
    int shx_size = shp->ShxSize;
    int dbf_size = shp->DbfSize;
    int dbf_reclen = shp->DbfReclen;
    int dbf_recno = shp->DbfRecno;
    int endian_arch = shp->endian_arch;
    double minx = shp->MinX;
    double miny = shp->MinY;
    double maxx = shp->MaxX;
    double maxy = shp->MaxY;
    unsigned char *buf_shp = shp->BufShp;
/* writing the SHP file header */
    fseek (fl_shp, 0, SEEK_SET);	/* repositioning at SHP file start */
    gaiaExport32 (buf_shp, 9994, GAIA_BIG_ENDIAN, endian_arch);	/* SHP magic number */
    gaiaExport32 (buf_shp + 4, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 8, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 12, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 16, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 20, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 24, shp_size, GAIA_BIG_ENDIAN, endian_arch);	/* SHP file size - measured in 16 bits words !!! */
    gaiaExport32 (buf_shp + 28, 1000, GAIA_LITTLE_ENDIAN, endian_arch);	/* version */
    gaiaExport32 (buf_shp + 32, shp->Shape, GAIA_LITTLE_ENDIAN, endian_arch);	/* ESRI shape */
    gaiaExport64 (buf_shp + 36, minx, GAIA_LITTLE_ENDIAN, endian_arch);	/* the MBR/BBOX for the whole shapefile */
    gaiaExport64 (buf_shp + 44, miny, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 52, maxx, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 60, maxy, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 68, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 76, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 84, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 92, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    fwrite (buf_shp, 1, 100, fl_shp);
/* writing the SHX file header */
    fseek (fl_shx, 0, SEEK_SET);	/* repositioning at SHX file start */
    gaiaExport32 (buf_shp, 9994, GAIA_BIG_ENDIAN, endian_arch);	/* SHP magic number */
    gaiaExport32 (buf_shp + 4, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 8, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 12, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 16, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 20, 0, GAIA_BIG_ENDIAN, endian_arch);
    gaiaExport32 (buf_shp + 24, shx_size, GAIA_BIG_ENDIAN, endian_arch);	/* SHXfile size - measured in 16 bits words !!! */
    gaiaExport32 (buf_shp + 28, 1000, GAIA_LITTLE_ENDIAN, endian_arch);	/* version */
    gaiaExport32 (buf_shp + 32, shp->Shape, GAIA_LITTLE_ENDIAN, endian_arch);	/* ESRI shape */
    gaiaExport64 (buf_shp + 36, minx, GAIA_LITTLE_ENDIAN, endian_arch);	/* the MBR for the whole shapefile */
    gaiaExport64 (buf_shp + 44, miny, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 52, maxx, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 60, maxy, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 68, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 76, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 84, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    gaiaExport64 (buf_shp + 92, 0.0, GAIA_LITTLE_ENDIAN, endian_arch);
    fwrite (buf_shp, 1, 100, fl_shx);
/* writing the DBF file header */
    *buf_shp = 0x1a;		/* DBF - this is theEOF marker */
    fwrite (buf_shp, 1, 1, fl_dbf);
    fseek (fl_dbf, 0, SEEK_SET);	/* repositioning at DBF file start */
    memset (buf_shp, '\0', 32);
    *buf_shp = 0x03;		/* DBF magic number */
    *(buf_shp + 1) = 1;		/* this is supposed to be the last update date [Year, Month, Day], but we ignore it at all */
    *(buf_shp + 2) = 1;
    *(buf_shp + 3) = 1;
    gaiaExport32 (buf_shp + 4, dbf_recno, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # records in this DBF */
    gaiaExport16 (buf_shp + 8, (short) dbf_size, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the file header size */
    gaiaExport16 (buf_shp + 10, (short) dbf_reclen, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the record length */
    fwrite (buf_shp, 1, 32, fl_dbf);
}

GAIAGEO_DECLARE void
gaiaShpAnalyze (gaiaShapefilePtr shp)
{
/* analyzing the SHP content, in order to detect if there are LINESTRINGS or MULTILINESTRINGS 
/ the same check is needed in order to detect if there are POLYGONS or MULTIPOLYGONS 
 */
    unsigned char buf[512];
    int rd;
    int skpos;
    int offset;
    int off_shp;
    int sz;
    int shape;
    int points;
    int n;
    int n1;
    int base;
    int start;
    int end;
    int iv;
    int ind;
    double x;
    double y;
    int polygons;
    int ZM_size;
    int multi = 0;
    int hasM = 0;
    int current_row = 0;
    gaiaRingPtr ring = NULL;
    while (1)
      {
	  /* positioning and reading the SHX file */
	  offset = 100 + (current_row * 8);	/* 100 bytes for the header + current row displacement; each SHX row = 8 bytes */
	  skpos = fseek (shp->flShx, offset, SEEK_SET);
	  if (skpos != 0)
	      goto exit;
	  rd = fread (buf, sizeof (unsigned char), 8, shp->flShx);
	  if (rd != 8)
	      goto exit;
	  off_shp = gaiaImport32 (buf, GAIA_BIG_ENDIAN, shp->endian_arch);
	  /* positioning and reading corresponding SHP entity - geometry */
	  offset = off_shp * 2;
	  skpos = fseek (shp->flShp, offset, SEEK_SET);
	  if (skpos != 0)
	      goto exit;
	  rd = fread (buf, sizeof (unsigned char), 12, shp->flShp);
	  if (rd != 12)
	      goto exit;
	  sz = gaiaImport32 (buf + 4, GAIA_BIG_ENDIAN, shp->endian_arch);
	  shape = gaiaImport32 (buf + 8, GAIA_LITTLE_ENDIAN, shp->endian_arch);
	  if ((sz * 2) > shp->ShpBfsz)
	    {
		/* current buffer is too small; we need to allocate a bigger buffer */
		free (shp->BufShp);
		shp->ShpBfsz = sz * 2;
		shp->BufShp = malloc (sizeof (unsigned char) * shp->ShpBfsz);
	    }
	  if (shape == GAIA_SHP_POLYLINE || shape == GAIA_SHP_POLYLINEZ
	      || shape == GAIA_SHP_POLYLINEM)
	    {
		/* shape polyline */
		rd = fread (shp->BufShp, sizeof (unsigned char), 32,
			    shp->flShp);
		if (rd != 32)
		    goto exit;
		rd = fread (shp->BufShp, sizeof (unsigned char),
			    (sz * 2) - 36, shp->flShp);
		if (rd != (sz * 2) - 36)
		    goto exit;
		n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN,
				  shp->endian_arch);
		n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
				   shp->endian_arch);
		if (n > 1)
		    multi++;
		if (shape == GAIA_SHP_POLYLINEZ)
		  {
		      ZM_size = 38 + (2 * n) + (n1 * 16);	/* size [in 16 bits words !!!] ZM */
		      if (sz == ZM_size)
			  hasM = 1;
		  }
	    }
	  if (shape == GAIA_SHP_POLYGON || shape == GAIA_SHP_POLYGONZ
	      || shape == GAIA_SHP_POLYGONM)
	    {
		/* shape polygon */
		polygons = 0;
		rd = fread (shp->BufShp, sizeof (unsigned char), 32,
			    shp->flShp);
		if (rd != 32)
		    goto exit;
		rd = fread (shp->BufShp, sizeof (unsigned char),
			    (sz * 2) - 36, shp->flShp);
		if (rd != (sz * 2) - 36)
		    goto exit;
		n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN,
				  shp->endian_arch);
		n1 = gaiaImport32 (shp->BufShp + 4, GAIA_LITTLE_ENDIAN,
				   shp->endian_arch);
		base = 8 + (n * 4);
		start = 0;
		for (ind = 0; ind < n; ind++)
		  {
		      if (ind < (n - 1))
			  end =
			      gaiaImport32 (shp->BufShp + 8 +
					    ((ind + 1) * 4),
					    GAIA_LITTLE_ENDIAN,
					    shp->endian_arch);
		      else
			  end = n1;
		      points = end - start;
		      ring = gaiaAllocRing (points);
		      points = 0;
		      for (iv = start; iv < end; iv++)
			{
			    x = gaiaImport64 (shp->BufShp + base +
					      (iv * 16), GAIA_LITTLE_ENDIAN,
					      shp->endian_arch);
			    y = gaiaImport64 (shp->BufShp + base +
					      (iv * 16) + 8,
					      GAIA_LITTLE_ENDIAN,
					      shp->endian_arch);
			    gaiaSetPoint (ring->Coords, points, x, y);
			    start++;
			    points++;
			}
		      if (!polygons)
			{
			    /* this one is the first POLYGON */
			    polygons = 1;
			}
		      else
			{
			    gaiaClockwise (ring);
			    if (ring->Clockwise)
			      {
				  /* this one is a different POLYGON exterior ring - we need to allocate e new POLYGON */
				  polygons++;
			      }
			}
		      gaiaFreeRing (ring);
		      ring = NULL;
		  }
		if (polygons > 1)
		    multi++;
		if (shape == GAIA_SHP_POLYGONZ)
		  {
		      ZM_size = 38 + (2 * n) + (n1 * 16);	/* size [in 16 bits words !!!] ZM */
		      if (sz == ZM_size)
			  hasM = 1;
		  }
	    }
	  if (shape == GAIA_SHP_MULTIPOINTZ)
	    {
		/* shape multipoint Z */
		rd = fread (shp->BufShp, sizeof (unsigned char), 32,
			    shp->flShp);
		if (rd != 32)
		    goto exit;
		rd = fread (shp->BufShp, sizeof (unsigned char),
			    (sz * 2) - 36, shp->flShp);
		if (rd != (sz * 2) - 36)
		    goto exit;
		n = gaiaImport32 (shp->BufShp, GAIA_LITTLE_ENDIAN,
				  shp->endian_arch);
		ZM_size = 38 + (n * 16);	/* size [in 16 bits words !!!] ZM */
		if (sz == ZM_size)
		    hasM = 1;
	    }
	  current_row++;
      }
  exit:
    if (ring)
	gaiaFreeRing (ring);
    if (shp->LastError)
	free (shp->LastError);
    shp->LastError = NULL;
/* setting the EffectiveType, as determined by this analysis */
    if (shp->Shape == GAIA_SHP_POLYLINE || shp->Shape == GAIA_SHP_POLYLINEZ
	|| shp->Shape == GAIA_SHP_POLYLINEM)
      {
	  /* SHAPE polyline */
	  if (multi)
	      shp->EffectiveType = GAIA_MULTILINESTRING;
	  else
	      shp->EffectiveType = GAIA_LINESTRING;
      }
    if (shp->Shape == GAIA_SHP_POLYGON || shp->Shape == GAIA_SHP_POLYGONZ
	|| shp->Shape == GAIA_SHP_POLYGONM)
      {
	  /* SHAPE polygon */
	  if (multi)
	      shp->EffectiveType = GAIA_MULTIPOLYGON;
	  else
	      shp->EffectiveType = GAIA_POLYGON;
      }
    if (shp->Shape == GAIA_SHP_POLYLINEZ || shp->Shape == GAIA_SHP_POLYGONZ
	|| shp->Shape == GAIA_SHP_MULTIPOINTZ)
      {
	  if (hasM)
	      shp->EffectiveDims = GAIA_XY_Z_M;
	  else
	      shp->EffectiveDims = GAIA_XY_Z;
      }
}

GAIAGEO_DECLARE gaiaDbfPtr
gaiaAllocDbf ()
{
/* allocates and initializes the DBF object */
    gaiaDbfPtr dbf = malloc (sizeof (gaiaDbf));
    dbf->endian_arch = 1;
    dbf->Path = NULL;
    dbf->flDbf = NULL;
    dbf->Dbf = NULL;
    dbf->BufDbf = NULL;
    dbf->DbfHdsz = 0;
    dbf->DbfReclen = 0;
    dbf->DbfSize = 0;
    dbf->DbfRecno = 0;
    dbf->Valid = 0;
    dbf->IconvObj = NULL;
    dbf->LastError = NULL;
    return dbf;
}

GAIAGEO_DECLARE void
gaiaFreeDbf (gaiaDbfPtr dbf)
{
/* frees all memory allocations related to the DBF object */
    if (dbf->Path)
	free (dbf->Path);
    if (dbf->flDbf)
	fclose (dbf->flDbf);
    if (dbf->Dbf)
	gaiaFreeDbfList (dbf->Dbf);
    if (dbf->BufDbf)
	free (dbf->BufDbf);
    if (dbf->IconvObj)
	iconv_close ((iconv_t) dbf->IconvObj);
    if (dbf->LastError)
	free (dbf->LastError);
    free (dbf);
}

GAIAGEO_DECLARE void
gaiaOpenDbfRead (gaiaDbfPtr dbf, const char *path, const char *charFrom,
		 const char *charTo)
{
/* trying to open the DBF and initial checkings */
    FILE *fl_dbf = NULL;
    int rd;
    unsigned char bf[1024];
    int dbf_size;
    int dbf_reclen = 0;
    int off_dbf;
    int ind;
    char field_name[2048];
    char *sys_err;
    char errMsg[1024];
    iconv_t iconv_ret;
    char utf8buf[2048];
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    int endian_arch = gaiaEndianArch ();
    gaiaDbfListPtr dbf_list = NULL;
    if (charFrom && charTo)
      {
	  iconv_ret = iconv_open (charTo, charFrom);
	  if (iconv_ret == (iconv_t) (-1))
	    {
		sprintf (errMsg,
			 "conversion from '%s' to '%s' not available\n",
			 charFrom, charTo);
		goto unsupported_conversion;
	    }
	  dbf->IconvObj = iconv_ret;
      }
    else
      {
	  sprintf (errMsg, "a NULL charset-name was passed\n");
	  goto unsupported_conversion;
      }
    if (dbf->flDbf != NULL)
      {
	  sprintf (errMsg, "attempting to reopen an already opened DBF\n");
	  goto unsupported_conversion;
      }
    fl_dbf = fopen (path, "rb");
    if (!fl_dbf)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for reading: %s", path,
		   sys_err);
	  goto no_file;
      }
/* reading DBF file header */
    rd = fread (bf, sizeof (unsigned char), 32, fl_dbf);
    if (rd != 32)
	goto error;
    if (*bf != 0x03)		/* checks the DBF magic number */
	goto error;
    dbf_size = gaiaImport16 (bf + 8, GAIA_LITTLE_ENDIAN, endian_arch);
    dbf_reclen = gaiaImport16 (bf + 10, GAIA_LITTLE_ENDIAN, endian_arch);
    dbf_size--;
    off_dbf = 0;
    dbf_list = gaiaAllocDbfList ();
    for (ind = 32; ind < dbf_size; ind += 32)
      {
	  /* fetches DBF fields definitions */
	  rd = fread (bf, sizeof (unsigned char), 32, fl_dbf);
	  if (rd != 32)
	      goto error;
	  memcpy (field_name, bf, 11);
	  field_name[11] = '\0';
	  len = strlen ((char *) field_name);
	  utf8len = 2048;
	  pBuf = (char *) field_name;
	  pUtf8buf = utf8buf;
	  if (iconv
	      ((iconv_t) (dbf->IconvObj), &pBuf, &len, &pUtf8buf,
	       &utf8len) == (size_t) (-1))
	      goto conversion_error;
	  memcpy (field_name, utf8buf, 2048 - utf8len);
	  field_name[2048 - utf8len] = '\0';
	  gaiaAddDbfField (dbf_list, field_name, *(bf + 11), off_dbf,
			   *(bf + 16), *(bf + 17));
	  off_dbf += *(bf + 16);
      }
    if (!gaiaIsValidDbfList (dbf_list))
      {
	  /* invalid DBF */
	  goto illegal_dbf;
      }
    len = strlen (path);
    dbf->Path = malloc (len + 1);
    strcpy (dbf->Path, path);
    dbf->flDbf = fl_dbf;
    dbf->Dbf = dbf_list;
/* allocating DBF buffer */
    dbf->BufDbf = malloc (sizeof (unsigned char) * dbf_reclen);
    dbf->DbfHdsz = dbf_size + 1;
    dbf->DbfReclen = dbf_reclen;
    dbf->Valid = 1;
    dbf->endian_arch = endian_arch;
    return;
  unsupported_conversion:
/* illegal charset */
    if (dbf->LastError)
	free (dbf->LastError);
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    return;
  no_file:
/* the DBF file can't be accessed */
    if (dbf->LastError)
	free (dbf->LastError);
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
  error:
/* the DBF is invalid or corrupted */
    if (dbf->LastError)
	free (dbf->LastError);
    sprintf (errMsg, "'%s' is corrupted / has invalid format", path);
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    gaiaFreeDbfList (dbf_list);
    fclose (fl_dbf);
    return;
  illegal_dbf:
/* the DBF-file contains unsupported data types */
    if (dbf->LastError)
	free (dbf->LastError);
    sprintf (errMsg, "'%s' contains unsupported data types", path);
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    gaiaFreeDbfList (dbf_list);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
  conversion_error:
/* libiconv error */
    if (dbf->LastError)
	free (dbf->LastError);
    sprintf (errMsg, "'%s' field name: invalid character sequence", path);
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    gaiaFreeDbfList (dbf_list);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
}

GAIAGEO_DECLARE void
gaiaOpenDbfWrite (gaiaDbfPtr dbf, const char *path, const char *charFrom,
		  const char *charTo)
{
/* trying to create the DBF file */
    FILE *fl_dbf = NULL;
    unsigned char bf[1024];
    unsigned char *dbf_buf = NULL;
    gaiaDbfFieldPtr fld;
    char *sys_err;
    char errMsg[1024];
    short dbf_reclen = 0;
    unsigned short dbf_size = 0;
    iconv_t iconv_ret;
    char buf[2048];
    char utf8buf[2048];
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    int defaultId = 1;
    if (charFrom && charTo)
      {
	  iconv_ret = iconv_open (charTo, charFrom);
	  if (iconv_ret == (iconv_t) (-1))
	    {
		sprintf (errMsg, "conversion from '%s' to '%s' not available\n",
			 charFrom, charTo);
		goto unsupported_conversion;
	    }
	  dbf->IconvObj = iconv_ret;
      }
    else
      {
	  sprintf (errMsg, "a NULL charset-name was passed\n");
	  goto unsupported_conversion;
      }
    if (dbf->flDbf != NULL)
      {
	  sprintf (errMsg, "attempting to reopen an already opened DBF file\n");
	  goto unsupported_conversion;
      }
/* trying to open the DBF file */
    fl_dbf = fopen (path, "wb");
    if (!fl_dbf)
      {
	  sys_err = strerror (errno);
	  sprintf (errMsg, "unable to open '%s' for writing: %s", path,
		   sys_err);
	  goto no_file;
      }
/* allocating DBF buffer */
    dbf_reclen = 1;		/* an extra byte is needed because in DBF rows first byte is a marker for deletion */
    fld = dbf->Dbf->First;
    while (fld)
      {
	  /* computing the DBF record length */
	  dbf_reclen += fld->Length;
	  fld = fld->Next;
      }
    dbf_buf = malloc (dbf_reclen);
/* writing the DBF file header */
    memset (bf, '\0', 32);
    fwrite (bf, 1, 32, fl_dbf);
    dbf_size = 32;		/* note: DBF counts sizes in bytes */
    fld = dbf->Dbf->First;
    while (fld)
      {
	  /* exporting DBF Fields specifications */
	  memset (bf, 0, 32);
	  strcpy (buf, fld->Name);
	  len = strlen (buf);
	  utf8len = 2048;
	  pBuf = buf;
	  pUtf8buf = utf8buf;
	  if (iconv
	      ((iconv_t) (dbf->IconvObj), &pBuf, &len, &pUtf8buf,
	       &utf8len) == (size_t) (-1))
	      sprintf (buf, "FLD#%d", defaultId++);
	  else
	    {
		memcpy (buf, utf8buf, 2048 - utf8len);
		buf[2048 - utf8len] = '\0';
		if (strlen (buf) > 10)
		    sprintf (buf, "FLD#%d", defaultId++);
	    }
	  memcpy (bf, buf, strlen (buf));
	  *(bf + 11) = fld->Type;
	  *(bf + 16) = fld->Length;
	  *(bf + 17) = fld->Decimals;
	  fwrite (bf, 1, 32, fl_dbf);
	  dbf_size += 32;
	  fld = fld->Next;
      }
    fwrite ("\r", 1, 1, fl_dbf);	/* this one is a special DBF delimiter that closes file header */
    dbf_size++;
    dbf->Valid = 1;
    dbf->flDbf = fl_dbf;
    dbf->BufDbf = dbf_buf;
    dbf->DbfHdsz = dbf_size + 1;
    dbf->DbfReclen = dbf_reclen;
    dbf->DbfSize = dbf_size;
    dbf->DbfRecno = 0;
    return;
  unsupported_conversion:
/* illegal charset */
    if (dbf->LastError)
	free (dbf->LastError);
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    return;
  no_file:
/* the DBF file can't be created/opened */
    if (dbf->LastError)
	free (dbf->LastError);
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    if (dbf_buf)
	free (dbf_buf);
    if (fl_dbf)
	fclose (fl_dbf);
    return;
}

GAIAGEO_DECLARE int
gaiaWriteDbfEntity (gaiaDbfPtr dbf, gaiaDbfListPtr entity)
{
/* trying to write an entity into some DBF file */
    char dummy[128];
    char fmt[16];
    gaiaDbfFieldPtr fld;
#if !defined(__MINGW32__) && defined(_WIN32)
    const char *pBuf;
#else /* not WIN32 */
    char *pBuf;
#endif
    size_t len;
    size_t utf8len;
    char *pUtf8buf;
    char buf[512];
    char utf8buf[2048];
/* writing the DBF record */
    memset (dbf->BufDbf, '\0', dbf->DbfReclen);
    *(dbf->BufDbf) = ' ';	/* in DBF first byte of each row marks for validity or deletion */
    fld = entity->First;
    while (fld)
      {
	  /* transferring field values */
	  switch (fld->Type)
	    {
	    case 'L':
		if (!(fld->Value))
		    *(dbf->BufDbf + fld->Offset) = '?';
		else if (fld->Value->Type != GAIA_INT_VALUE)
		    *(dbf->BufDbf + fld->Offset + 1) = '?';
		else
		  {
		      if (fld->Value->IntValue == 0)
			  *(dbf->BufDbf + fld->Offset + 1) = 'N';
		      else
			  *(dbf->BufDbf + fld->Offset + 1) = 'Y';
		  }
		break;
	    case 'D':
		memset (dbf->BufDbf + fld->Offset + 1, '0', 8);
		if (fld->Value)
		  {
		      if (fld->Value->Type == GAIA_TEXT_VALUE)
			{
			    if (strlen (fld->Value->TxtValue) == 8)
				memcpy (dbf->BufDbf + fld->Offset + 1,
					fld->Value->TxtValue, 8);
			}
		  }
		break;
	    case 'C':
		memset (dbf->BufDbf + fld->Offset + 1, ' ', fld->Length);
		if (fld->Value)
		  {
		      if (fld->Value->Type == GAIA_TEXT_VALUE)
			{
			    strcpy (buf, fld->Value->TxtValue);
			    len = strlen (buf);
			    utf8len = 2048;
			    pBuf = buf;
			    pUtf8buf = utf8buf;
			    if (iconv
				((iconv_t) (dbf->IconvObj), &pBuf, &len,
				 &pUtf8buf, &utf8len) == (size_t) (-1))
				goto conversion_error;
			    memcpy (buf, utf8buf, 2048 - utf8len);
			    buf[2048 - utf8len] = '\0';
			    if (strlen (buf) < fld->Length)
				memcpy (dbf->BufDbf + fld->Offset + 1, buf,
					strlen (buf));
			    else
				memcpy (dbf->BufDbf + fld->Offset + 1, buf,
					fld->Length);
			}
		  }
		break;
	    case 'N':
		memset (dbf->BufDbf + fld->Offset + 1, '\0', fld->Length);
		if (fld->Value)
		  {
		      if (fld->Value->Type == GAIA_INT_VALUE)
			{
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT - M$ runtime doesn't supports %lld for 64 bits */
			    sprintf (dummy, "%I64d", fld->Value->IntValue);
#else
			    sprintf (dummy, "%lld", fld->Value->IntValue);
#endif
			    if (strlen (dummy) <= fld->Length)
				memcpy (dbf->BufDbf + fld->Offset + 1,
					dummy, strlen (dummy));
			}
		      if (fld->Value->Type == GAIA_DOUBLE_VALUE)
			{
			    sprintf (fmt, "%%1.%df", fld->Decimals);
			    sprintf (dummy, fmt, fld->Value->DblValue);
			    if (strlen (dummy) <= fld->Length)
				memcpy (dbf->BufDbf + fld->Offset + 1,
					dummy, strlen (dummy));
			}
		  }
		break;
	    };
	  fld = fld->Next;
      }
/* inserting entity in DBF file */
    fwrite (dbf->BufDbf, 1, dbf->DbfReclen, dbf->flDbf);
    (dbf->DbfRecno)++;
    return 1;
  conversion_error:
    if (dbf->LastError)
	free (dbf->LastError);
    sprintf (dummy, "Invalid character sequence");
    len = strlen (dummy);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, dummy);
    return 0;
}

GAIAGEO_DECLARE void
gaiaFlushDbfHeader (gaiaDbfPtr dbf)
{
/* updates the DBF file header */
    FILE *fl_dbf = dbf->flDbf;
    int dbf_size = dbf->DbfSize;
    int dbf_reclen = dbf->DbfReclen;
    int dbf_recno = dbf->DbfRecno;
    int endian_arch = dbf->endian_arch;
    unsigned char bf[64];
/* writing the DBF file header */
    *bf = 0x1a;			/* DBF - this is theEOF marker */
    fwrite (bf, 1, 1, fl_dbf);
    fseek (fl_dbf, 0, SEEK_SET);	/* repositioning at DBF file start */
    memset (bf, '\0', 32);
    *bf = 0x03;			/* DBF magic number */
    *(bf + 1) = 1;		/* this is supposed to be the last update date [Year, Month, Day], but we ignore it at all */
    *(bf + 2) = 1;
    *(bf + 3) = 1;
    gaiaExport32 (bf + 4, dbf_recno, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports # records in this DBF */
    gaiaExport16 (bf + 8, (short) dbf_size, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the file header size */
    gaiaExport16 (bf + 10, (short) dbf_reclen, GAIA_LITTLE_ENDIAN, endian_arch);	/* exports the record length */
    fwrite (bf, 1, 32, fl_dbf);
}

GAIAGEO_DECLARE int
gaiaReadDbfEntity (gaiaDbfPtr dbf, int current_row, int *deleted)
{
/* trying to read an entity from DBF */
    int rd;
    int skpos;
    int offset;
    int len;
    char errMsg[1024];
    gaiaDbfFieldPtr pFld;
/* positioning and reading the DBF file */
    offset = dbf->DbfHdsz + (current_row * dbf->DbfReclen);
    skpos = fseek (dbf->flDbf, offset, SEEK_SET);
    if (skpos != 0)
	goto eof;
    rd = fread (dbf->BufDbf, sizeof (unsigned char), dbf->DbfReclen,
		dbf->flDbf);
    if (rd != dbf->DbfReclen)
	goto eof;
/* setting up the current DBF ENTITY */
    gaiaResetDbfEntity (dbf->Dbf);
    dbf->Dbf->RowId = current_row;
    if (*(dbf->BufDbf) == '*')
      {
	  /* deleted row */
	  *deleted = 1;
	  if (dbf->LastError)
	      free (dbf->LastError);
	  dbf->LastError = NULL;
	  return 1;
      }
/* fetching the DBF values */
    pFld = dbf->Dbf->First;
    while (pFld)
      {
	  if (!parseDbfField (dbf->BufDbf, dbf->IconvObj, pFld))
	      goto conversion_error;
	  pFld = pFld->Next;
      }
    if (dbf->LastError)
	free (dbf->LastError);
    dbf->LastError = NULL;
    *deleted = 0;
    return 1;
  eof:
    if (dbf->LastError)
	free (dbf->LastError);
    dbf->LastError = NULL;
    return 0;
  conversion_error:
    if (dbf->LastError)
	free (dbf->LastError);
    sprintf (errMsg, "Invalid character sequence");
    len = strlen (errMsg);
    dbf->LastError = malloc (len + 1);
    strcpy (dbf->LastError, errMsg);
    return 0;
}

#endif /* ICONV enabled/disabled */
/**************** End file: gg_shape.c **********/


/**************** Begin file: gg_transform.c **********/

/* #include <sys/types.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifndef OMIT_PROJ		/* including PROJ.4 */
/* #include <proj_api.h> */
#endif

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

GAIAGEO_DECLARE void
gaiaShiftCoords (gaiaGeomCollPtr geom, double shift_x, double shift_y)
{
/* returns a geometry that is the old old geometry with required shifting applied to coordinates */
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaPointPtr point;
    gaiaPolygonPtr polyg;
    gaiaLinestringPtr line;
    gaiaRingPtr ring;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* shifting POINTs */
	  point->X += shift_x;
	  point->Y += shift_y;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* shifting LINESTRINGs */
	  for (iv = 0; iv < line->Points; iv++)
	    {
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		x += shift_x;
		y += shift_y;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (line->Coords, iv, x, y, z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (line->Coords, iv, x, y, m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (line->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (line->Coords, iv, x, y);
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* shifting POLYGONs */
	  ring = polyg->Exterior;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* shifting the EXTERIOR RING */
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		x += shift_x;
		y += shift_y;
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (ring->Coords, iv, x, y, m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (ring->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		/* shifting the INTERIOR RINGs */
		ring = polyg->Interiors + ib;
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			}
		      x += shift_x;
		      y += shift_y;
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, iv, x, y, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, iv, x, y);
			}
		  }
	    }
	  polyg = polyg->Next;
      }
    gaiaMbrGeometry (geom);
}

GAIAGEO_DECLARE void
gaiaScaleCoords (gaiaGeomCollPtr geom, double scale_x, double scale_y)
{
/* returns a geometry that is the old old geometry with required scaling applied to coordinates */
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaPointPtr point;
    gaiaPolygonPtr polyg;
    gaiaLinestringPtr line;
    gaiaRingPtr ring;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* scaling POINTs */
	  point->X *= scale_x;
	  point->Y *= scale_y;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* scaling LINESTRINGs */
	  for (iv = 0; iv < line->Points; iv++)
	    {
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		x *= scale_x;
		y *= scale_y;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (line->Coords, iv, x, y, z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (line->Coords, iv, x, y, m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (line->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (line->Coords, iv, x, y);
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* scaling POLYGONs */
	  ring = polyg->Exterior;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* scaling the EXTERIOR RING */
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		x *= scale_x;
		y *= scale_y;
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (ring->Coords, iv, x, y, m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (ring->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		/* scaling the INTERIOR RINGs */
		ring = polyg->Interiors + ib;
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			}
		      x *= scale_x;
		      y *= scale_y;
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, iv, x, y, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, iv, x, y);
			}
		  }
	    }
	  polyg = polyg->Next;
      }
    gaiaMbrGeometry (geom);
}

GAIAGEO_DECLARE void
gaiaRotateCoords (gaiaGeomCollPtr geom, double angle)
{
/* returns a geometry that is the old old geometry with required rotation applied to coordinates */
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    double nx;
    double ny;
    double rad = angle * 0.0174532925199432958;
    double cosine = cos (rad);
    double sine = sin (rad);
    gaiaPointPtr point;
    gaiaPolygonPtr polyg;
    gaiaLinestringPtr line;
    gaiaRingPtr ring;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* shifting POINTs */
	  x = point->X;
	  y = point->Y;
	  point->X = (x * cosine) + (y * sine);
	  point->Y = (y * cosine) - (x * sine);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* rotating LINESTRINGs */
	  for (iv = 0; iv < line->Points; iv++)
	    {
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		nx = (x * cosine) + (y * sine);
		ny = (y * cosine) - (x * sine);
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (line->Coords, iv, nx, ny, z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (line->Coords, iv, nx, ny, m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (line->Coords, iv, nx, ny, z, m);
		  }
		else
		  {
		      gaiaSetPoint (line->Coords, iv, nx, ny);
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* rotating POLYGONs */
	  ring = polyg->Exterior;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* rotating the EXTERIOR RING */
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		nx = (x * cosine) + (y * sine);
		ny = (y * cosine) - (x * sine);
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (ring->Coords, iv, nx, ny, z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (ring->Coords, iv, nx, ny, m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (ring->Coords, iv, nx, ny, z, m);
		  }
		else
		  {
		      gaiaSetPoint (ring->Coords, iv, nx, ny);
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		/* rotating the INTERIOR RINGs */
		ring = polyg->Interiors + ib;
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			}
		      nx = (x * cosine) + (y * sine);
		      ny = (y * cosine) - (x * sine);
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, iv, nx, ny, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, iv, nx, ny, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, iv, nx, ny, z, m);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, iv, nx, ny);
			}
		  }
	    }
	  polyg = polyg->Next;
      }
    gaiaMbrGeometry (geom);
}

GAIAGEO_DECLARE void
gaiaReflectCoords (gaiaGeomCollPtr geom, int x_axis, int y_axis)
{
/* returns a geometry that is the old old geometry with required reflection applied to coordinates */
    int ib;
    int iv;
    double x;
    double y;
    double z = 0.0;
    double m = 0.0;
    gaiaPointPtr point;
    gaiaPolygonPtr polyg;
    gaiaLinestringPtr line;
    gaiaRingPtr ring;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* reflecting POINTs */
	  if (x_axis)
	      point->X *= -1.0;
	  if (y_axis)
	      point->Y *= -1.0;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* reflecting LINESTRINGs */
	  for (iv = 0; iv < line->Points; iv++)
	    {
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		if (x_axis)
		    x *= -1.0;
		if (y_axis)
		    y *= -1.0;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (line->Coords, iv, x, y, z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (line->Coords, iv, x, y, m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (line->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (line->Coords, iv, x, y);
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* reflecting POLYGONs */
	  ring = polyg->Exterior;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* reflecting the EXTERIOR RING */
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		if (x_axis)
		    x *= -1.0;
		if (y_axis)
		    y *= -1.0;
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (ring->Coords, iv, x, y, m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (ring->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		/* reflecting the INTERIOR RINGs */
		ring = polyg->Interiors + ib;
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			}
		      if (x_axis)
			  x *= -1.0;
		      if (y_axis)
			  y *= -1.0;
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, iv, x, y, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, iv, x, y);
			}
		  }
	    }
	  polyg = polyg->Next;
      }
    gaiaMbrGeometry (geom);
}

GAIAGEO_DECLARE void
gaiaSwapCoords (gaiaGeomCollPtr geom)
{
/* returns a geometry that is the old old geometry with swapped x- and y-coordinates */
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    double sv;
    gaiaPointPtr point;
    gaiaPolygonPtr polyg;
    gaiaLinestringPtr line;
    gaiaRingPtr ring;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* swapping POINTs */
	  sv = point->X;
	  point->X = point->Y;
	  point->Y = sv;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* swapping LINESTRINGs */
	  for (iv = 0; iv < line->Points; iv++)
	    {
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		sv = x;
		x = y;
		y = sv;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (line->Coords, iv, x, y, z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (line->Coords, iv, x, y, m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (line->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (line->Coords, iv, x, y);
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* swapping POLYGONs */
	  ring = polyg->Exterior;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* shifting the EXTERIOR RING */
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		sv = x;
		x = y;
		y = sv;
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaSetPointXYM (ring->Coords, iv, x, y, m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
		  }
		else
		  {
		      gaiaSetPoint (ring->Coords, iv, x, y);
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		/* swapping the INTERIOR RINGs */
		ring = polyg->Interiors + ib;
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			}
		      sv = x;
		      x = y;
		      y = sv;
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (ring->Coords, iv, x, y, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (ring->Coords, iv, x, y);
			}
		  }
	    }
	  polyg = polyg->Next;
      }
    gaiaMbrGeometry (geom);
}

#ifndef OMIT_PROJ		/* including PROJ.4 */

static int
gaiaIsLongLat (char *str)
{
/* checks if we have to do with ANGLES if +proj=longlat is defined */
    if (strstr (str, "+proj=longlat") != NULL)
	return 1;
    return 0;
}

GAIAGEO_DECLARE double
gaiaRadsToDegs (double rads)
{
/* converts an ANGLE from radians to degrees */
    return rads * RAD_TO_DEG;
}

GAIAGEO_DECLARE double
gaiaDegsToRads (double degs)
{
/* converts an ANGLE from degrees to radians */
    return degs * DEG_TO_RAD;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaTransform (gaiaGeomCollPtr org, char *proj_from, char *proj_to)
{
/* creates a new GEOMETRY reprojecting coordinates from the original one */
    int ib;
    int cnt;
    int i;
    double *xx;
    double *yy;
    double *zz;
    double *mm = NULL;
    double x;
    double y;
    double z = 0.0;
    double m = 0.0;
    int error = 0;
    int from_angle;
    int to_angle;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr dst_ln;
    gaiaPolygonPtr pg;
    gaiaPolygonPtr dst_pg;
    gaiaRingPtr rng;
    gaiaRingPtr dst_rng;
    projPJ from_cs = pj_init_plus (proj_from);
    projPJ to_cs = pj_init_plus (proj_to);
    gaiaGeomCollPtr dst;
    if (org->DimensionModel == GAIA_XY_Z)
	dst = gaiaAllocGeomCollXYZ ();
    else if (org->DimensionModel == GAIA_XY_M)
	dst = gaiaAllocGeomCollXYM ();
    else if (org->DimensionModel == GAIA_XY_Z_M)
	dst = gaiaAllocGeomCollXYZM ();
    else
	dst = gaiaAllocGeomColl ();
/* setting up projection parameters */
    from_angle = gaiaIsLongLat (proj_from);
    to_angle = gaiaIsLongLat (proj_to);
    if (!from_cs)
	return dst;
    if (!to_cs)
	return dst;
    cnt = 0;
    pt = org->FirstPoint;
    while (pt)
      {
	  /* counting POINTs */
	  cnt++;
	  pt = pt->Next;
      }
    if (cnt)
      {
	  /* reprojecting POINTs */
	  xx = malloc (sizeof (double) * cnt);
	  yy = malloc (sizeof (double) * cnt);
	  zz = malloc (sizeof (double) * cnt);
	  if (org->DimensionModel == GAIA_XY_M
	      || org->DimensionModel == GAIA_XY_Z_M)
	      mm = malloc (sizeof (double) * cnt);
	  i = 0;
	  pt = org->FirstPoint;
	  while (pt)
	    {
		/* inserting points to be converted in temporary arrays */
		if (from_angle)
		  {
		      xx[i] = gaiaDegsToRads (pt->X);
		      yy[i] = gaiaDegsToRads (pt->Y);
		  }
		else
		  {
		      xx[i] = pt->X;
		      yy[i] = pt->Y;
		  }
		if (org->DimensionModel == GAIA_XY_Z
		    || org->DimensionModel == GAIA_XY_Z_M)
		    zz[i] = pt->Z;
		else
		    zz[i] = 0.0;
		if (org->DimensionModel == GAIA_XY_M
		    || org->DimensionModel == GAIA_XY_Z_M)
		    mm[i] = pt->M;
		i++;
		pt = pt->Next;
	    }
	  /* applying reprojection        */
	  if (pj_transform (from_cs, to_cs, cnt, 0, xx, yy, zz) == 0)
	    {
		/* inserting the reprojected POINTs in the new GEOMETRY */
		for (i = 0; i < cnt; i++)
		  {
		      if (to_angle)
			{
			    x = gaiaRadsToDegs (xx[i]);
			    y = gaiaRadsToDegs (yy[i]);
			}
		      else
			{
			    x = xx[i];
			    y = yy[i];
			}
		      if (org->DimensionModel == GAIA_XY_Z
			  || org->DimensionModel == GAIA_XY_Z_M)
			  z = zz[i];
		      else
			  z = 0.0;
		      if (org->DimensionModel == GAIA_XY_M
			  || org->DimensionModel == GAIA_XY_Z_M)
			  m = mm[i];
		      else
			  m = 0.0;
		      if (dst->DimensionModel == GAIA_XY_Z)
			  gaiaAddPointToGeomCollXYZ (dst, x, y, z);
		      else if (dst->DimensionModel == GAIA_XY_M)
			  gaiaAddPointToGeomCollXYM (dst, x, y, m);
		      else if (dst->DimensionModel == GAIA_XY_Z_M)
			  gaiaAddPointToGeomCollXYZM (dst, x, y, z, m);
		      else
			  gaiaAddPointToGeomColl (dst, x, y);
		  }
	    }
	  else
	      error = 1;
	  free (xx);
	  free (yy);
	  free (zz);
	  if (org->DimensionModel == GAIA_XY_M
	      || org->DimensionModel == GAIA_XY_Z_M)
	      free (mm);
      }
    if (error)
	goto stop;
    ln = org->FirstLinestring;
    while (ln)
      {
	  /* reprojecting LINESTRINGs */
	  cnt = ln->Points;
	  xx = malloc (sizeof (double) * cnt);
	  yy = malloc (sizeof (double) * cnt);
	  zz = malloc (sizeof (double) * cnt);
	  if (ln->DimensionModel == GAIA_XY_M
	      || ln->DimensionModel == GAIA_XY_Z_M)
	      mm = malloc (sizeof (double) * cnt);
	  for (i = 0; i < cnt; i++)
	    {
		/* inserting points to be converted in temporary arrays */
		if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, i, &x, &y, &z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, i, &x, &y, &m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, i, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, i, &x, &y);
		  }
		if (from_angle)
		  {
		      xx[i] = gaiaDegsToRads (x);
		      yy[i] = gaiaDegsToRads (y);
		  }
		else
		  {
		      xx[i] = x;
		      yy[i] = y;
		  }
		if (ln->DimensionModel == GAIA_XY_Z
		    || ln->DimensionModel == GAIA_XY_Z_M)
		    zz[i] = z;
		else
		    zz[i] = 0.0;
		if (ln->DimensionModel == GAIA_XY_M
		    || ln->DimensionModel == GAIA_XY_Z_M)
		    mm[i] = m;
	    }
	  /* applying reprojection        */
	  if (pj_transform (from_cs, to_cs, cnt, 0, xx, yy, zz) == 0)
	    {
		/* inserting the reprojected LINESTRING in the new GEOMETRY */
		dst_ln = gaiaAddLinestringToGeomColl (dst, cnt);
		for (i = 0; i < cnt; i++)
		  {
		      /* setting LINESTRING points */
		      if (to_angle)
			{
			    x = gaiaRadsToDegs (xx[i]);
			    y = gaiaRadsToDegs (yy[i]);
			}
		      else
			{
			    x = xx[i];
			    y = yy[i];
			}
		      if (ln->DimensionModel == GAIA_XY_Z
			  || ln->DimensionModel == GAIA_XY_Z_M)
			  z = zz[i];
		      else
			  z = 0.0;
		      if (ln->DimensionModel == GAIA_XY_M
			  || ln->DimensionModel == GAIA_XY_Z_M)
			  m = mm[i];
		      else
			  m = 0.0;
		      if (dst_ln->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (dst_ln->Coords, i, x, y, z);
			}
		      else if (dst_ln->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (dst_ln->Coords, i, x, y, m);
			}
		      else if (dst_ln->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (dst_ln->Coords, i, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (dst_ln->Coords, i, x, y);
			}
		  }
	    }
	  else
	      error = 1;
	  free (xx);
	  free (yy);
	  free (zz);
	  if (ln->DimensionModel == GAIA_XY_M
	      || ln->DimensionModel == GAIA_XY_Z_M)
	      free (mm);
	  if (error)
	      goto stop;
	  ln = ln->Next;
      }
    pg = org->FirstPolygon;
    while (pg)
      {
	  /* reprojecting POLYGONs */
	  rng = pg->Exterior;
	  cnt = rng->Points;
	  dst_pg = gaiaAddPolygonToGeomColl (dst, cnt, pg->NumInteriors);
	  xx = malloc (sizeof (double) * cnt);
	  yy = malloc (sizeof (double) * cnt);
	  zz = malloc (sizeof (double) * cnt);
	  if (rng->DimensionModel == GAIA_XY_M
	      || rng->DimensionModel == GAIA_XY_Z_M)
	      mm = malloc (sizeof (double) * cnt);
	  for (i = 0; i < cnt; i++)
	    {
		/* inserting points to be converted in temporary arrays [EXTERIOR RING] */
		if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, i, &x, &y, &z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, i, &x, &y, &m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, i, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, i, &x, &y);
		  }
		if (from_angle)
		  {
		      xx[i] = gaiaDegsToRads (x);
		      yy[i] = gaiaDegsToRads (y);
		  }
		else
		  {
		      xx[i] = x;
		      yy[i] = y;
		  }
		if (rng->DimensionModel == GAIA_XY_Z
		    || rng->DimensionModel == GAIA_XY_Z_M)
		    zz[i] = z;
		else
		    zz[i] = 0.0;
		if (rng->DimensionModel == GAIA_XY_M
		    || rng->DimensionModel == GAIA_XY_Z_M)
		    mm[i] = m;
	    }
	  /* applying reprojection        */
	  if (pj_transform (from_cs, to_cs, cnt, 0, xx, yy, zz) == 0)
	    {
		/* inserting the reprojected POLYGON in the new GEOMETRY */
		dst_rng = dst_pg->Exterior;
		for (i = 0; i < cnt; i++)
		  {
		      /* setting EXTERIOR RING points */
		      if (to_angle)
			{
			    x = gaiaRadsToDegs (xx[i]);
			    y = gaiaRadsToDegs (yy[i]);
			}
		      else
			{
			    x = xx[i];
			    y = yy[i];
			}
		      if (rng->DimensionModel == GAIA_XY_Z
			  || rng->DimensionModel == GAIA_XY_Z_M)
			  z = zz[i];
		      else
			  z = 0.0;
		      if (rng->DimensionModel == GAIA_XY_M
			  || rng->DimensionModel == GAIA_XY_Z_M)
			  m = mm[i];
		      else
			  m = 0.0;
		      if (dst_rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaSetPointXYZ (dst_rng->Coords, i, x, y, z);
			}
		      else if (dst_rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaSetPointXYM (dst_rng->Coords, i, x, y, m);
			}
		      else if (dst_rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaSetPointXYZM (dst_rng->Coords, i, x, y, z, m);
			}
		      else
			{
			    gaiaSetPoint (dst_rng->Coords, i, x, y);
			}
		  }
	    }
	  else
	      error = 1;
	  free (xx);
	  free (yy);
	  free (zz);
	  if (rng->DimensionModel == GAIA_XY_M
	      || rng->DimensionModel == GAIA_XY_Z_M)
	      free (mm);
	  if (error)
	      goto stop;
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		/* processing INTERIOR RINGS */
		rng = pg->Interiors + ib;
		cnt = rng->Points;
		xx = malloc (sizeof (double) * cnt);
		yy = malloc (sizeof (double) * cnt);
		zz = malloc (sizeof (double) * cnt);
		if (rng->DimensionModel == GAIA_XY_M
		    || rng->DimensionModel == GAIA_XY_Z_M)
		    mm = malloc (sizeof (double) * cnt);
		for (i = 0; i < cnt; i++)
		  {
		      /* inserting points to be converted in temporary arrays [INTERIOR RING] */
		      if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, i, &x, &y, &z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, i, &x, &y, &m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, i, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, i, &x, &y);
			}
		      if (from_angle)
			{
			    xx[i] = gaiaDegsToRads (x);
			    yy[i] = gaiaDegsToRads (y);
			}
		      else
			{
			    xx[i] = x;
			    yy[i] = y;
			}
		      if (rng->DimensionModel == GAIA_XY_Z
			  || rng->DimensionModel == GAIA_XY_Z_M)
			  zz[i] = z;
		      else
			  zz[i] = 0.0;
		      if (rng->DimensionModel == GAIA_XY_M
			  || rng->DimensionModel == GAIA_XY_Z_M)
			  mm[i] = m;
		  }
		/* applying reprojection        */
		if (pj_transform (from_cs, to_cs, cnt, 0, xx, yy, zz) == 0)
		  {
		      /* inserting the reprojected POLYGON in the new GEOMETRY */
		      dst_rng = gaiaAddInteriorRing (dst_pg, ib, cnt);
		      for (i = 0; i < cnt; i++)
			{
			    /* setting INTERIOR RING points */
			    if (to_angle)
			      {
				  x = gaiaRadsToDegs (xx[i]);
				  y = gaiaRadsToDegs (yy[i]);
			      }
			    else
			      {
				  x = xx[i];
				  y = yy[i];
			      }
			    if (rng->DimensionModel == GAIA_XY_Z
				|| rng->DimensionModel == GAIA_XY_Z_M)
				z = zz[i];
			    else
				z = 0.0;
			    if (rng->DimensionModel == GAIA_XY_M
				|| rng->DimensionModel == GAIA_XY_Z_M)
				m = mm[i];
			    else
				m = 0.0;
			    if (dst_rng->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaSetPointXYZ (dst_rng->Coords, i, x, y, z);
			      }
			    else if (dst_rng->DimensionModel == GAIA_XY_M)
			      {
				  gaiaSetPointXYM (dst_rng->Coords, i, x, y, m);
			      }
			    else if (dst_rng->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaSetPointXYZM (dst_rng->Coords, i, x, y, z,
						    m);
			      }
			    else
			      {
				  gaiaSetPoint (dst_rng->Coords, i, x, y);
			      }
			}
		  }
		else
		    error = 1;
		free (xx);
		free (yy);
		free (zz);
		if (rng->DimensionModel == GAIA_XY_M
		    || rng->DimensionModel == GAIA_XY_Z_M)
		    free (mm);
		if (error)
		    goto stop;
	    }
	  pg = pg->Next;
      }
/* destroying the PROJ4 params */
  stop:
    pj_free (from_cs);
    pj_free (to_cs);
    if (error)
      {
	  /* some error occurred */
	  gaiaPointPtr pP;
	  gaiaPointPtr pPn;
	  gaiaLinestringPtr pL;
	  gaiaLinestringPtr pLn;
	  gaiaPolygonPtr pA;
	  gaiaPolygonPtr pAn;
	  pP = dst->FirstPoint;
	  while (pP != NULL)
	    {
		pPn = pP->Next;
		gaiaFreePoint (pP);
		pP = pPn;
	    }
	  pL = dst->FirstLinestring;
	  while (pL != NULL)
	    {
		pLn = pL->Next;
		gaiaFreeLinestring (pL);
		pL = pLn;
	    }
	  pA = dst->FirstPolygon;
	  while (pA != NULL)
	    {
		pAn = pA->Next;
		gaiaFreePolygon (pA);
		pA = pAn;
	    }
	  dst->FirstPoint = NULL;
	  dst->LastPoint = NULL;
	  dst->FirstLinestring = NULL;
	  dst->LastLinestring = NULL;
	  dst->FirstPolygon = NULL;
	  dst->LastPolygon = NULL;
      }
    if (dst)
      {
	  gaiaMbrGeometry (dst);
	  dst->DeclaredType = org->DeclaredType;
      }
    return dst;
}

#endif /* end including PROJ.4 */
/**************** End file: gg_transform.c **********/


/**************** Begin file: gg_wkb.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <float.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

static void
ParseWkbPoint (gaiaGeomCollPtr geo)
{
/* decodes a POINT from WKB */
    double x;
    double y;
    if (geo->size < geo->offset + 16)
	return;
    x = gaiaImport64 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
		      geo->endian_arch);
    geo->offset += 16;
    gaiaAddPointToGeomColl (geo, x, y);
}

static void
ParseWkbPointZ (gaiaGeomCollPtr geo)
{
/* decodes a POINTZ from WKB */
    double x;
    double y;
    double z;
    if (geo->size < geo->offset + 24)
	return;
    x = gaiaImport64 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
		      geo->endian_arch);
    z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
		      geo->endian_arch);
    geo->offset += 24;
    gaiaAddPointToGeomCollXYZ (geo, x, y, z);
}

static void
ParseWkbPointM (gaiaGeomCollPtr geo)
{
/* decodes a POINTM from WKB */
    double x;
    double y;
    double m;
    if (geo->size < geo->offset + 24)
	return;
    x = gaiaImport64 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
		      geo->endian_arch);
    m = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
		      geo->endian_arch);
    geo->offset += 24;
    gaiaAddPointToGeomCollXYM (geo, x, y, m);
}

static void
ParseWkbPointZM (gaiaGeomCollPtr geo)
{
/* decodes a POINTZM from WKB */
    double x;
    double y;
    double z;
    double m;
    if (geo->size < geo->offset + 32)
	return;
    x = gaiaImport64 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
		      geo->endian_arch);
    z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
		      geo->endian_arch);
    m = gaiaImport64 (geo->blob + (geo->offset + 24), geo->endian,
		      geo->endian_arch);
    geo->offset += 32;
    gaiaAddPointToGeomCollXYZM (geo, x, y, z, m);
}

static void
ParseWkbLine (gaiaGeomCollPtr geo)
{
/* decodes a LINESTRING from WKB */
    int points;
    int iv;
    double x;
    double y;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (16 * points))
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
			    geo->endian_arch);
	  gaiaSetPoint (line->Coords, iv, x, y);
	  geo->offset += 16;
      }
}

static void
ParseWkbLineZ (gaiaGeomCollPtr geo)
{
/* decodes a LINESTRINGZ from WKB */
    int points;
    int iv;
    double x;
    double y;
    double z;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (24 * points))
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
			    geo->endian_arch);
	  z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
			    geo->endian_arch);
	  gaiaSetPointXYZ (line->Coords, iv, x, y, z);
	  geo->offset += 24;
      }
}

static void
ParseWkbLineM (gaiaGeomCollPtr geo)
{
/* decodes a LINESTRINGM from WKB */
    int points;
    int iv;
    double x;
    double y;
    double m;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (24 * points))
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
			    geo->endian_arch);
	  m = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
			    geo->endian_arch);
	  gaiaSetPointXYM (line->Coords, iv, x, y, m);
	  geo->offset += 24;
      }
}

static void
ParseWkbLineZM (gaiaGeomCollPtr geo)
{
/* decodes a LINESTRINGZM from WKB */
    int points;
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (32 * points))
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
			    geo->endian_arch);
	  z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
			    geo->endian_arch);
	  m = gaiaImport64 (geo->blob + (geo->offset + 24), geo->endian,
			    geo->endian_arch);
	  gaiaSetPointXYZM (line->Coords, iv, x, y, z, m);
	  geo->offset += 32;
      }
}

static void
ParseWkbPolygon (gaiaGeomCollPtr geo)
{
/* decodes a POLYGON from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (16 * nverts))
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		geo->offset += 16;
		gaiaSetPoint (ring->Coords, iv, x, y);
	    }
      }
}

static void
ParseWkbPolygonZ (gaiaGeomCollPtr geo)
{
/* decodes a POLYGONZ from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (24 * nverts))
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
				  geo->endian_arch);
		geo->offset += 24;
		gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
	    }
      }
}

static void
ParseWkbPolygonM (gaiaGeomCollPtr geo)
{
/* decodes a POLYGONM from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    double m;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (24 * nverts))
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		m = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
				  geo->endian_arch);
		geo->offset += 24;
		gaiaSetPointXYM (ring->Coords, iv, x, y, m);
	    }
      }
}

static void
ParseWkbPolygonZM (gaiaGeomCollPtr geo)
{
/* decodes a POLYGONZM from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (32 * nverts))
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
				  geo->endian_arch);
		m = gaiaImport64 (geo->blob + (geo->offset + 24), geo->endian,
				  geo->endian_arch);
		geo->offset += 32;
		gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
	    }
      }
}

static void
ParseCompressedWkbLine (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED LINESTRING from WKB */
    int points;
    int iv;
    double x;
    double y;
    double last_x = 0.0;
    double last_y = 0.0;
    float fx;
    float fy;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (8 * points) + 16)
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  if (iv == 0 || iv == (points - 1))
	    {
		/* first and last vertices are uncompressed */
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		geo->offset += 16;
	    }
	  else
	    {
		/* any other intermediate vertex is compressed */
		fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
				    geo->endian_arch);
		fy = gaiaImportF32 (geo->blob + (geo->offset + 4), geo->endian,
				    geo->endian_arch);
		x = last_x + fx;
		y = last_y + fy;
		geo->offset += 8;
	    }
	  gaiaSetPoint (line->Coords, iv, x, y);
	  last_x = x;
	  last_y = y;
      }
}

static void
ParseCompressedWkbLineZ (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED LINESTRINGZ from WKB */
    int points;
    int iv;
    double x;
    double y;
    double z;
    double last_x = 0.0;
    double last_y = 0.0;
    double last_z = 0.0;
    float fx;
    float fy;
    float fz;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (12 * points) + 24)
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  if (iv == 0 || iv == (points - 1))
	    {
		/* first and last vertices are uncompressed */
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
				  geo->endian_arch);
		geo->offset += 24;
	    }
	  else
	    {
		/* any other intermediate vertex is compressed */
		fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
				    geo->endian_arch);
		fy = gaiaImportF32 (geo->blob + (geo->offset + 4), geo->endian,
				    geo->endian_arch);
		fz = gaiaImportF32 (geo->blob + (geo->offset + 8), geo->endian,
				    geo->endian_arch);
		x = last_x + fx;
		y = last_y + fy;
		z = last_z + fz;
		geo->offset += 12;
	    }
	  gaiaSetPointXYZ (line->Coords, iv, x, y, z);
	  last_x = x;
	  last_y = y;
	  last_z = z;
      }
}

static void
ParseCompressedWkbLineM (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED LINESTRINGM from WKB */
    int points;
    int iv;
    double x;
    double y;
    double m;
    double last_x = 0.0;
    double last_y = 0.0;
    float fx;
    float fy;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (16 * points) + 16)
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  if (iv == 0 || iv == (points - 1))
	    {
		/* first and last vertices are uncompressed */
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		m = gaiaImport64 (geo->blob + (geo->offset + 24), geo->endian,
				  geo->endian_arch);
		geo->offset += 24;
	    }
	  else
	    {
		/* any other intermediate vertex is compressed */
		fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
				    geo->endian_arch);
		fy = gaiaImportF32 (geo->blob + (geo->offset + 4), geo->endian,
				    geo->endian_arch);
		m = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		x = last_x + fx;
		y = last_y + fy;
		geo->offset += 16;
	    }
	  gaiaSetPointXYM (line->Coords, iv, x, y, m);
	  last_x = x;
	  last_y = y;
      }
}

static void
ParseCompressedWkbLineZM (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED LINESTRINGZM from WKB */
    int points;
    int iv;
    double x;
    double y;
    double z;
    double m;
    double last_x = 0.0;
    double last_y = 0.0;
    double last_z = 0.0;
    float fx;
    float fy;
    float fz;
    gaiaLinestringPtr line;
    if (geo->size < geo->offset + 4)
	return;
    points =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    if (geo->size < geo->offset + (20 * points) + 24)
	return;
    line = gaiaAddLinestringToGeomColl (geo, points);
    for (iv = 0; iv < points; iv++)
      {
	  if (iv == 0 || iv == (points - 1))
	    {
		/* first and last vertices are uncompressed */
		x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
				  geo->endian_arch);
		y = gaiaImport64 (geo->blob + (geo->offset + 8), geo->endian,
				  geo->endian_arch);
		z = gaiaImport64 (geo->blob + (geo->offset + 16), geo->endian,
				  geo->endian_arch);
		m = gaiaImport64 (geo->blob + (geo->offset + 24), geo->endian,
				  geo->endian_arch);
		geo->offset += 32;
	    }
	  else
	    {
		/* any other intermediate vertex is compressed */
		fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
				    geo->endian_arch);
		fy = gaiaImportF32 (geo->blob + (geo->offset + 4), geo->endian,
				    geo->endian_arch);
		fz = gaiaImportF32 (geo->blob + (geo->offset + 8), geo->endian,
				    geo->endian_arch);
		m = gaiaImport64 (geo->blob + (geo->offset + 12), geo->endian,
				  geo->endian_arch);
		x = last_x + fx;
		y = last_y + fy;
		z = last_z + fz;
		geo->offset += 20;
	    }
	  gaiaSetPointXYZM (line->Coords, iv, x, y, z, m);
	  last_x = x;
	  last_y = y;
	  last_z = z;
      }
}

static void
ParseCompressedWkbPolygon (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED POLYGON from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    double last_x = 0.0;
    double last_y = 0.0;
    float fx;
    float fy;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (8 * nverts) + 16)
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		if (iv == 0 || iv == (nverts - 1))
		  {
		      /* first and last vertices are uncompressed */
		      x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
					geo->endian_arch);
		      y = gaiaImport64 (geo->blob + (geo->offset + 8),
					geo->endian, geo->endian_arch);
		      geo->offset += 16;
		  }
		else
		  {
		      /* any other intermediate vertex is compressed */
		      fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
					  geo->endian_arch);
		      fy = gaiaImportF32 (geo->blob + (geo->offset + 4),
					  geo->endian, geo->endian_arch);
		      x = last_x + fx;
		      y = last_y + fy;
		      geo->offset += 8;
		  }
		gaiaSetPoint (ring->Coords, iv, x, y);
		last_x = x;
		last_y = y;
	    }
      }
}

static void
ParseCompressedWkbPolygonZ (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED POLYGONZ from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double last_x = 0.0;
    double last_y = 0.0;
    double last_z = 0.0;
    float fx;
    float fy;
    float fz;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (12 * nverts) + 24)
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		if (iv == 0 || iv == (nverts - 1))
		  {
		      /* first and last vertices are uncompressed */
		      x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
					geo->endian_arch);
		      y = gaiaImport64 (geo->blob + (geo->offset + 8),
					geo->endian, geo->endian_arch);
		      z = gaiaImport64 (geo->blob + (geo->offset + 16),
					geo->endian, geo->endian_arch);
		      geo->offset += 24;
		  }
		else
		  {
		      /* any other intermediate vertex is compressed */
		      fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
					  geo->endian_arch);
		      fy = gaiaImportF32 (geo->blob + (geo->offset + 4),
					  geo->endian, geo->endian_arch);
		      fz = gaiaImportF32 (geo->blob + (geo->offset + 8),
					  geo->endian, geo->endian_arch);
		      x = last_x + fx;
		      y = last_y + fy;
		      z = last_z + fz;
		      geo->offset += 12;
		  }
		gaiaSetPointXYZ (ring->Coords, iv, x, y, z);
		last_x = x;
		last_y = y;
		last_z = z;
	    }
      }
}

static void
ParseCompressedWkbPolygonM (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED POLYGONM from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    double m;
    double last_x = 0.0;
    double last_y = 0.0;
    float fx;
    float fy;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (16 * nverts) + 16)
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		if (iv == 0 || iv == (nverts - 1))
		  {
		      /* first and last vertices are uncompressed */
		      x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
					geo->endian_arch);
		      y = gaiaImport64 (geo->blob + (geo->offset + 8),
					geo->endian, geo->endian_arch);
		      m = gaiaImport64 (geo->blob + (geo->offset + 24),
					geo->endian, geo->endian_arch);
		      geo->offset += 24;
		  }
		else
		  {
		      /* any other intermediate vertex is compressed */
		      fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
					  geo->endian_arch);
		      fy = gaiaImportF32 (geo->blob + (geo->offset + 4),
					  geo->endian, geo->endian_arch);
		      m = gaiaImport64 (geo->blob + (geo->offset + 8),
					geo->endian, geo->endian_arch);
		      x = last_x + fx;
		      y = last_y + fy;
		      geo->offset += 16;
		  }
		gaiaSetPointXYM (ring->Coords, iv, x, y, m);
		last_x = x;
		last_y = y;
	    }
      }
}

static void
ParseCompressedWkbPolygonZM (gaiaGeomCollPtr geo)
{
/* decodes a COMPRESSED POLYGONZM from WKB */
    int rings;
    int nverts;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    double last_x = 0.0;
    double last_y = 0.0;
    double last_z = 0.0;
    float fx;
    float fy;
    float fz;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr ring;
    if (geo->size < geo->offset + 4)
	return;
    rings =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (geo->size < geo->offset + 4)
	      return;
	  nverts =
	      gaiaImport32 (geo->blob + geo->offset, geo->endian,
			    geo->endian_arch);
	  geo->offset += 4;
	  if (geo->size < geo->offset + (20 * nverts) + 24)
	      return;
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geo, nverts, rings - 1);
		ring = polyg->Exterior;
	    }
	  else
	      ring = gaiaAddInteriorRing (polyg, ib - 1, nverts);
	  for (iv = 0; iv < nverts; iv++)
	    {
		if (iv == 0 || iv == (nverts - 1))
		  {
		      /* first and last vertices are uncompressed */
		      x = gaiaImport64 (geo->blob + geo->offset, geo->endian,
					geo->endian_arch);
		      y = gaiaImport64 (geo->blob + (geo->offset + 8),
					geo->endian, geo->endian_arch);
		      z = gaiaImport64 (geo->blob + (geo->offset + 16),
					geo->endian, geo->endian_arch);
		      m = gaiaImport64 (geo->blob + (geo->offset + 24),
					geo->endian, geo->endian_arch);
		      geo->offset += 32;
		  }
		else
		  {
		      /* any other intermediate vertex is compressed */
		      fx = gaiaImportF32 (geo->blob + geo->offset, geo->endian,
					  geo->endian_arch);
		      fy = gaiaImportF32 (geo->blob + (geo->offset + 4),
					  geo->endian, geo->endian_arch);
		      fz = gaiaImportF32 (geo->blob + (geo->offset + 8),
					  geo->endian, geo->endian_arch);
		      m = gaiaImport64 (geo->blob + (geo->offset + 12),
					geo->endian, geo->endian_arch);
		      x = last_x + fx;
		      y = last_y + fy;
		      z = last_z + fz;
		      geo->offset += 20;
		  }
		gaiaSetPointXYZM (ring->Coords, iv, x, y, z, m);
		last_x = x;
		last_y = y;
		last_z = z;
	    }
      }
}

static void
ParseWkbGeometry (gaiaGeomCollPtr geo)
{
/* decodes a MULTIxx or GEOMETRYCOLLECTION from SpatiaLite BLOB */
    int entities;
    int type;
    int ie;
    if (geo->size < geo->offset + 4)
	return;
    entities =
	gaiaImport32 (geo->blob + geo->offset, geo->endian, geo->endian_arch);
    geo->offset += 4;
    for (ie = 0; ie < entities; ie++)
      {
	  if (geo->size < geo->offset + 5)
	      return;
	  type =
	      gaiaImport32 (geo->blob + geo->offset + 1, geo->endian,
			    geo->endian_arch);
	  geo->offset += 5;
	  switch (type)
	    {
	    case GAIA_POINT:
		ParseWkbPoint (geo);
		break;
	    case GAIA_POINTZ:
	    case GAIA_GEOSWKB_POINTZ:
		ParseWkbPointZ (geo);
		break;
	    case GAIA_POINTM:
		ParseWkbPointM (geo);
		break;
	    case GAIA_POINTZM:
		ParseWkbPointZM (geo);
		break;
	    case GAIA_LINESTRING:
		ParseWkbLine (geo);
		break;
	    case GAIA_LINESTRINGZ:
	    case GAIA_GEOSWKB_LINESTRINGZ:
		ParseWkbLineZ (geo);
		break;
	    case GAIA_LINESTRINGM:
		ParseWkbLineM (geo);
		break;
	    case GAIA_LINESTRINGZM:
		ParseWkbLineZM (geo);
		break;
	    case GAIA_POLYGON:
		ParseWkbPolygon (geo);
		break;
	    case GAIA_POLYGONZ:
	    case GAIA_GEOSWKB_POLYGONZ:
		ParseWkbPolygonZ (geo);
		break;
	    case GAIA_POLYGONM:
		ParseWkbPolygonM (geo);
		break;
	    case GAIA_POLYGONZM:
		ParseWkbPolygonZM (geo);
		break;
	    case GAIA_COMPRESSED_LINESTRING:
		ParseCompressedWkbLine (geo);
		break;
	    case GAIA_COMPRESSED_LINESTRINGZ:
		ParseCompressedWkbLineZ (geo);
		break;
	    case GAIA_COMPRESSED_LINESTRINGM:
		ParseCompressedWkbLineM (geo);
		break;
	    case GAIA_COMPRESSED_LINESTRINGZM:
		ParseCompressedWkbLineZM (geo);
		break;
	    case GAIA_COMPRESSED_POLYGON:
		ParseCompressedWkbPolygon (geo);
		break;
	    case GAIA_COMPRESSED_POLYGONZ:
		ParseCompressedWkbPolygonZ (geo);
		break;
	    case GAIA_COMPRESSED_POLYGONM:
		ParseCompressedWkbPolygonM (geo);
		break;
	    case GAIA_COMPRESSED_POLYGONZM:
		ParseCompressedWkbPolygonZM (geo);
		break;
	    default:
		break;
	    };
      }
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromSpatiaLiteBlobWkb (const unsigned char *blob, unsigned int size)
{
/* decoding from SpatiaLite BLOB to GEOMETRY */
    int type;
    int little_endian;
    int endian_arch = gaiaEndianArch ();
    gaiaGeomCollPtr geo = NULL;
    if (size < 45)
	return NULL;		/* cannot be an internal BLOB WKB geometry */
    if (*(blob + 0) != GAIA_MARK_START)
	return NULL;		/* failed to recognize START signature */
    if (*(blob + (size - 1)) != GAIA_MARK_END)
	return NULL;		/* failed to recognize END signature */
    if (*(blob + 38) != GAIA_MARK_MBR)
	return NULL;		/* failed to recognize MBR signature */
    if (*(blob + 1) == GAIA_LITTLE_ENDIAN)
	little_endian = 1;
    else if (*(blob + 1) == GAIA_BIG_ENDIAN)
	little_endian = 0;
    else
	return NULL;		/* unknown encoding; nor litte-endian neither big-endian */
    type = gaiaImport32 (blob + 39, little_endian, endian_arch);
    geo = gaiaAllocGeomColl ();
    geo->Srid = gaiaImport32 (blob + 2, little_endian, endian_arch);
    geo->endian_arch = (char) endian_arch;
    geo->endian = (char) little_endian;
    geo->blob = blob;
    geo->size = size;
    geo->offset = 43;
    switch (type)
      {
	  /* setting up DimensionModel */
      case GAIA_POINTZ:
      case GAIA_LINESTRINGZ:
      case GAIA_POLYGONZ:
      case GAIA_MULTIPOINTZ:
      case GAIA_MULTILINESTRINGZ:
      case GAIA_MULTIPOLYGONZ:
      case GAIA_GEOMETRYCOLLECTIONZ:
      case GAIA_COMPRESSED_LINESTRINGZ:
      case GAIA_COMPRESSED_POLYGONZ:
	  geo->DimensionModel = GAIA_XY_Z;
	  break;
      case GAIA_POINTM:
      case GAIA_LINESTRINGM:
      case GAIA_POLYGONM:
      case GAIA_MULTIPOINTM:
      case GAIA_MULTILINESTRINGM:
      case GAIA_MULTIPOLYGONM:
      case GAIA_GEOMETRYCOLLECTIONM:
      case GAIA_COMPRESSED_LINESTRINGM:
      case GAIA_COMPRESSED_POLYGONM:
	  geo->DimensionModel = GAIA_XY_M;
	  break;
      case GAIA_POINTZM:
      case GAIA_LINESTRINGZM:
      case GAIA_POLYGONZM:
      case GAIA_MULTIPOINTZM:
      case GAIA_MULTILINESTRINGZM:
      case GAIA_MULTIPOLYGONZM:
      case GAIA_GEOMETRYCOLLECTIONZM:
      case GAIA_COMPRESSED_LINESTRINGZM:
      case GAIA_COMPRESSED_POLYGONZM:
	  geo->DimensionModel = GAIA_XY_Z_M;
	  break;
      default:
	  geo->DimensionModel = GAIA_XY;
	  break;
      };
    switch (type)
      {
	  /* parsing elementary geometries */
      case GAIA_POINT:
	  ParseWkbPoint (geo);
	  break;
      case GAIA_POINTZ:
	  ParseWkbPointZ (geo);
	  break;
      case GAIA_POINTM:
	  ParseWkbPointM (geo);
	  break;
      case GAIA_POINTZM:
	  ParseWkbPointZM (geo);
	  break;
      case GAIA_LINESTRING:
	  ParseWkbLine (geo);
	  break;
      case GAIA_LINESTRINGZ:
	  ParseWkbLineZ (geo);
	  break;
      case GAIA_LINESTRINGM:
	  ParseWkbLineM (geo);
	  break;
      case GAIA_LINESTRINGZM:
	  ParseWkbLineZM (geo);
	  break;
      case GAIA_POLYGON:
	  ParseWkbPolygon (geo);
	  break;
      case GAIA_POLYGONZ:
	  ParseWkbPolygonZ (geo);
	  break;
      case GAIA_POLYGONM:
	  ParseWkbPolygonM (geo);
	  break;
      case GAIA_POLYGONZM:
	  ParseWkbPolygonZM (geo);
	  break;
      case GAIA_COMPRESSED_LINESTRING:
	  ParseCompressedWkbLine (geo);
	  break;
      case GAIA_COMPRESSED_LINESTRINGZ:
	  ParseCompressedWkbLineZ (geo);
	  break;
      case GAIA_COMPRESSED_LINESTRINGM:
	  ParseCompressedWkbLineM (geo);
	  break;
      case GAIA_COMPRESSED_LINESTRINGZM:
	  ParseCompressedWkbLineZM (geo);
	  break;
      case GAIA_COMPRESSED_POLYGON:
	  ParseCompressedWkbPolygon (geo);
	  break;
      case GAIA_COMPRESSED_POLYGONZ:
	  ParseCompressedWkbPolygonZ (geo);
	  break;
      case GAIA_COMPRESSED_POLYGONM:
	  ParseCompressedWkbPolygonM (geo);
	  break;
      case GAIA_COMPRESSED_POLYGONZM:
	  ParseCompressedWkbPolygonZM (geo);
	  break;
      case GAIA_MULTIPOINT:
      case GAIA_MULTIPOINTZ:
      case GAIA_MULTIPOINTM:
      case GAIA_MULTIPOINTZM:
      case GAIA_MULTILINESTRING:
      case GAIA_MULTILINESTRINGZ:
      case GAIA_MULTILINESTRINGM:
      case GAIA_MULTILINESTRINGZM:
      case GAIA_MULTIPOLYGON:
      case GAIA_MULTIPOLYGONZ:
      case GAIA_MULTIPOLYGONM:
      case GAIA_MULTIPOLYGONZM:
      case GAIA_GEOMETRYCOLLECTION:
      case GAIA_GEOMETRYCOLLECTIONZ:
      case GAIA_GEOMETRYCOLLECTIONM:
      case GAIA_GEOMETRYCOLLECTIONZM:
	  ParseWkbGeometry (geo);
	  break;
      default:
	  break;
      };
    geo->MinX = gaiaImport64 (blob + 6, little_endian, endian_arch);
    geo->MinY = gaiaImport64 (blob + 14, little_endian, endian_arch);
    geo->MaxX = gaiaImport64 (blob + 22, little_endian, endian_arch);
    geo->MaxY = gaiaImport64 (blob + 30, little_endian, endian_arch);
    switch (type)
      {
	  /* setting up DeclaredType */
      case GAIA_POINT:
      case GAIA_POINTZ:
      case GAIA_POINTM:
      case GAIA_POINTZM:
	  geo->DeclaredType = GAIA_POINT;
	  break;
      case GAIA_LINESTRING:
      case GAIA_LINESTRINGZ:
      case GAIA_LINESTRINGM:
      case GAIA_LINESTRINGZM:
      case GAIA_COMPRESSED_LINESTRING:
      case GAIA_COMPRESSED_LINESTRINGZ:
      case GAIA_COMPRESSED_LINESTRINGM:
      case GAIA_COMPRESSED_LINESTRINGZM:
	  geo->DeclaredType = GAIA_LINESTRING;
	  break;
      case GAIA_POLYGON:
      case GAIA_POLYGONZ:
      case GAIA_POLYGONM:
      case GAIA_POLYGONZM:
      case GAIA_COMPRESSED_POLYGON:
      case GAIA_COMPRESSED_POLYGONZ:
      case GAIA_COMPRESSED_POLYGONM:
      case GAIA_COMPRESSED_POLYGONZM:
	  geo->DeclaredType = GAIA_POLYGON;
	  break;
      case GAIA_MULTIPOINT:
      case GAIA_MULTIPOINTZ:
      case GAIA_MULTIPOINTM:
      case GAIA_MULTIPOINTZM:
	  geo->DeclaredType = GAIA_MULTIPOINT;
	  break;
      case GAIA_MULTILINESTRING:
      case GAIA_MULTILINESTRINGZ:
      case GAIA_MULTILINESTRINGM:
      case GAIA_MULTILINESTRINGZM:
	  geo->DeclaredType = GAIA_MULTILINESTRING;
	  break;
      case GAIA_MULTIPOLYGON:
      case GAIA_MULTIPOLYGONZ:
      case GAIA_MULTIPOLYGONM:
      case GAIA_MULTIPOLYGONZM:
	  geo->DeclaredType = GAIA_MULTIPOLYGON;
	  break;
      case GAIA_GEOMETRYCOLLECTION:
      case GAIA_GEOMETRYCOLLECTIONZ:
      case GAIA_GEOMETRYCOLLECTIONM:
      case GAIA_GEOMETRYCOLLECTIONZM:
	  geo->DeclaredType = GAIA_GEOMETRYCOLLECTION;
	  break;
      default:
	  geo->DeclaredType = GAIA_UNKNOWN;
	  break;
      };
    return geo;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromSpatiaLiteBlobMbr (const unsigned char *blob, unsigned int size)
{
/* decoding from SpatiaLite BLOB to GEOMETRY [MBR only] */
    int little_endian;
    int endian_arch = gaiaEndianArch ();
    double minx;
    double miny;
    double maxx;
    double maxy;
    gaiaGeomCollPtr geo = NULL;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    if (size < 45)
	return NULL;		/* cannot be an internal BLOB WKB geometry */
    if (*(blob + 0) != GAIA_MARK_START)
	return NULL;		/* failed to recognize START signature */
    if (*(blob + (size - 1)) != GAIA_MARK_END)
	return NULL;		/* failed to recognize END signature */
    if (*(blob + 38) != GAIA_MARK_MBR)
	return NULL;		/* failed to recognize MBR signature */
    if (*(blob + 1) == GAIA_LITTLE_ENDIAN)
	little_endian = 1;
    else if (*(blob + 1) == GAIA_BIG_ENDIAN)
	little_endian = 0;
    else
	return NULL;		/* unknown encoding; nor litte-endian neither big-endian */
    geo = gaiaAllocGeomColl ();
    polyg = gaiaAddPolygonToGeomColl (geo, 5, 0);
    ring = polyg->Exterior;
    minx = gaiaImport64 (blob + 6, little_endian, endian_arch);
    miny = gaiaImport64 (blob + 14, little_endian, endian_arch);
    maxx = gaiaImport64 (blob + 22, little_endian, endian_arch);
    maxy = gaiaImport64 (blob + 30, little_endian, endian_arch);
    gaiaSetPoint (ring->Coords, 0, minx, miny);	/* vertex # 1 */
    gaiaSetPoint (ring->Coords, 1, maxx, miny);	/* vertex # 2 */
    gaiaSetPoint (ring->Coords, 2, maxx, maxy);	/* vertex # 3 */
    gaiaSetPoint (ring->Coords, 3, minx, maxy);	/* vertex # 4 */
    gaiaSetPoint (ring->Coords, 4, minx, miny);	/* vertex # 5 [same as vertex # 1 to close the polygon] */
    return geo;
}

GAIAGEO_DECLARE void
gaiaToSpatiaLiteBlobWkb (gaiaGeomCollPtr geom, unsigned char **result,
			 int *size)
{
/* builds the SpatiaLite BLOB representation for this GEOMETRY */
    int ib;
    int iv;
    double x;
    double y;
    double z = 0.0;
    double m = 0.0;
    int entities = 0;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    int type;
    unsigned char *ptr;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    int endian_arch = gaiaEndianArch ();
    gaiaMbrGeometry (geom);
/* how many entities, and of what kind, do we have ? */
    pt = geom->FirstPoint;
    while (pt)
      {
	  point = pt;
	  entities++;
	  n_points++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  line = ln;
	  entities++;
	  n_linestrings++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  polyg = pg;
	  entities++;
	  n_polygons++;
	  pg = pg->Next;
      }
    *size = 0;
    *result = NULL;
    if (n_points == 0 && n_polygons == 0 && n_linestrings == 0)
	return;
/* ok, we can determine the geometry class */
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOINTZM;
		else
		    type = GAIA_MULTIPOINT;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_POINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_POINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_POINTZM;
		else
		    type = GAIA_POINT;
	    }
      }
    else if (n_points > 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOINTZM;
		else
		    type = GAIA_MULTIPOINT;
	    }
      }
    else if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTILINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTILINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTILINESTRINGZM;
		else
		    type = GAIA_MULTILINESTRING;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_LINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_LINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_LINESTRINGZM;
		else
		    type = GAIA_LINESTRING;
	    }
      }
    else if (n_points == 0 && n_linestrings > 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTILINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTILINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTILINESTRINGZM;
		else
		    type = GAIA_MULTILINESTRING;
	    }
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOLYGONZM;
		else
		    type = GAIA_MULTIPOLYGON;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_POLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_POLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_POLYGONZM;
		else
		    type = GAIA_POLYGON;
	    }
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons > 1)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOLYGONZM;
		else
		    type = GAIA_MULTIPOLYGON;
	    }
      }
    else
      {
	  if (geom->DimensionModel == GAIA_XY_Z)
	      type = GAIA_GEOMETRYCOLLECTIONZ;
	  else if (geom->DimensionModel == GAIA_XY_M)
	      type = GAIA_GEOMETRYCOLLECTIONM;
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      type = GAIA_GEOMETRYCOLLECTIONZM;
	  else
	      type = GAIA_GEOMETRYCOLLECTION;
      }
/* and now we compute the size of BLOB */
    *size = 44;			/* header size */
    switch (type)
      {
      case GAIA_POINT:
	  *size += (sizeof (double) * 2);	/* [x,y] coords */
	  break;
      case GAIA_POINTZ:
	  *size += (sizeof (double) * 3);	/* [x,y,z] coords */
	  break;
      case GAIA_POINTM:
	  *size += (sizeof (double) * 3);	/* [x,y,m] coords */
	  break;
      case GAIA_POINTZM:
	  *size += (sizeof (double) * 4);	/* [x,y,z,m] coords */
	  break;
      case GAIA_LINESTRING:
	  *size += (4 + ((sizeof (double) * 2) * line->Points));	/* # points + [x,y] for each vertex */
	  break;
      case GAIA_LINESTRINGZ:
	  *size += (4 + ((sizeof (double) * 3) * line->Points));	/* # points + [x,y,z] for each vertex */
	  break;
      case GAIA_LINESTRINGM:
	  *size += (4 + ((sizeof (double) * 3) * line->Points));	/* # points + [x,y,m] for each vertex */
	  break;
      case GAIA_LINESTRINGZM:
	  *size += (4 + ((sizeof (double) * 4) * line->Points));	/* # points + [x,y,z,m] for each vertex */
	  break;
      case GAIA_POLYGON:
	  rng = polyg->Exterior;
	  *size += (8 + ((sizeof (double) * 2) * rng->Points));	/* # rings + # points + [x.y] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + ((sizeof (double) * 2) * rng->Points));	/* # points + [x,y] array - interior ring */
	    }
	  break;
      case GAIA_POLYGONZ:
	  rng = polyg->Exterior;
	  *size += (8 + ((sizeof (double) * 3) * rng->Points));	/* # rings + # points + [x,y,z] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + ((sizeof (double) * 3) * rng->Points));	/* # points + [x,y,z] array - interior ring */
	    }
	  break;
      case GAIA_POLYGONM:
	  rng = polyg->Exterior;
	  *size += (8 + ((sizeof (double) * 3) * rng->Points));	/* # rings + # points + [x,y,m] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + ((sizeof (double) * 3) * rng->Points));	/* # points + [x,y,m] array - interior ring */
	    }
	  break;
      case GAIA_POLYGONZM:
	  rng = polyg->Exterior;
	  *size += (8 + ((sizeof (double) * 4) * rng->Points));	/* # rings + # points + [x,y,z,m] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + ((sizeof (double) * 4) * rng->Points));	/* # points + [x,y,z,m] array - interior ring */
	    }
	  break;
      default:
	  /* this one is not a simple geometry; should be a MULTIxxxx or a GEOMETRYCOLLECTION */
	  *size += 4;		/* # entities */
	  point = geom->FirstPoint;
	  while (point)
	    {
		*size += 5;	/* entity header */
		if (geom->DimensionModel == GAIA_XY_Z
		    || geom->DimensionModel == GAIA_XY_M)
		    *size += (sizeof (double) * 3);	/* three doubles for each POINT */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *size += (sizeof (double) * 4);	/* four doubles for each POINT */
		else
		    *size += (sizeof (double) * 2);	/* two doubles for each POINT */
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		*size += 5;	/* entity header */
		if (geom->DimensionModel == GAIA_XY_Z
		    || geom->DimensionModel == GAIA_XY_M)
		    *size += (4 + ((sizeof (double) * 3) * line->Points));	/* # points + [x,y,z] for each vertex */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *size += (4 + ((sizeof (double) * 4) * line->Points));	/* # points + [x,y,z,m] for each vertex */
		else
		    *size += (4 + ((sizeof (double) * 2) * line->Points));	/* # points + [x,y] for each vertex */
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		*size += 5;	/* entity header */
		rng = polyg->Exterior;
		if (geom->DimensionModel == GAIA_XY_Z
		    || geom->DimensionModel == GAIA_XY_M)
		    *size += (8 + ((sizeof (double) * 3) * rng->Points));	/* # rings + # points + [x,y,z] array - exterior ring */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *size += (8 + ((sizeof (double) * 4) * rng->Points));	/* # rings + # points + [x,y,z,m] array - exterior ring */
		else
		    *size += (8 + ((sizeof (double) * 2) * rng->Points));	/* # rings + # points + [x,y] array - exterior ring */
		for (ib = 0; ib < polyg->NumInteriors; ib++)
		  {
		      rng = polyg->Interiors + ib;
		      if (geom->DimensionModel == GAIA_XY_Z
			  || geom->DimensionModel == GAIA_XY_M)
			  *size += (4 + ((sizeof (double) * 3) * rng->Points));	/* # points + [x,y,z] array - interior ring */
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			  *size += (4 + ((sizeof (double) * 4) * rng->Points));	/* # points + [x,y,z,m] array - interior ring */
		      else
			  *size += (4 + ((sizeof (double) * 2) * rng->Points));	/* # points + [x,y] array - interior ring */
		  }
		polyg = polyg->Next;
	    }
      };
    *result = malloc (*size);
    ptr = *result;
/* and finally we build the BLOB */
    switch (type)
      {
      case GAIA_POINT:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINT, 1, endian_arch);	/* class POINT */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  *(ptr + 59) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POINTZ:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINTZ, 1, endian_arch);	/* class POINT XYZ */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  gaiaExport64 (ptr + 59, point->Z, 1, endian_arch);	/* Z */
	  *(ptr + 67) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POINTM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINTM, 1, endian_arch);	/* class POINT XYM */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  gaiaExport64 (ptr + 59, point->M, 1, endian_arch);	/* M */
	  *(ptr + 67) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POINTZM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINTZM, 1, endian_arch);	/* class POINT XYZM */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  gaiaExport64 (ptr + 59, point->Z, 1, endian_arch);	/* M */
	  gaiaExport64 (ptr + 67, point->M, 1, endian_arch);	/* Z */
	  *(ptr + 75) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRING:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_LINESTRING, 1, endian_arch);	/* class LINESTRING */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPoint (line->Coords, iv, &x, &y);
		gaiaExport64 (ptr, x, 1, endian_arch);
		gaiaExport64 (ptr + 8, y, 1, endian_arch);
		ptr += 16;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRINGZ:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_LINESTRINGZ, 1, endian_arch);	/* class LINESTRING XYZ */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		gaiaExport64 (ptr, x, 1, endian_arch);
		gaiaExport64 (ptr + 8, y, 1, endian_arch);
		gaiaExport64 (ptr + 16, z, 1, endian_arch);
		ptr += 24;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRINGM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_LINESTRINGM, 1, endian_arch);	/* class LINESTRING XYM */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		gaiaExport64 (ptr, x, 1, endian_arch);
		gaiaExport64 (ptr + 8, y, 1, endian_arch);
		gaiaExport64 (ptr + 16, m, 1, endian_arch);
		ptr += 24;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRINGZM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_LINESTRINGZM, 1, endian_arch);	/* class LINESTRING XYZM */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		gaiaExport64 (ptr, x, 1, endian_arch);
		gaiaExport64 (ptr + 8, y, 1, endian_arch);
		gaiaExport64 (ptr + 16, z, 1, endian_arch);
		gaiaExport64 (ptr + 24, m, 1, endian_arch);
		ptr += 32;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGON:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POLYGON, 1, endian_arch);	/* class POLYGON */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPoint (rng->Coords, iv, &x, &y);
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
		ptr += 16;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
		      ptr += 16;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGONZ:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POLYGONZ, 1, endian_arch);	/* class POLYGON XYZ */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
		gaiaExport64 (ptr + 16, z, 1, endian_arch);	/* Z - exterior ring */
		ptr += 24;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
		      gaiaExport64 (ptr + 16, z, 1, endian_arch);	/* Z - interior ring */
		      ptr += 24;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGONM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POLYGONM, 1, endian_arch);	/* class POLYGON XYM */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
		gaiaExport64 (ptr + 16, m, 1, endian_arch);	/* M - exterior ring */
		ptr += 24;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
		      gaiaExport64 (ptr + 16, m, 1, endian_arch);	/* M - interior ring */
		      ptr += 24;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGONZM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POLYGONZM, 1, endian_arch);	/* class POLYGON */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
		gaiaExport64 (ptr + 16, z, 1, endian_arch);	/* Z - exterior ring */
		gaiaExport64 (ptr + 24, m, 1, endian_arch);	/* M - exterior ring */
		ptr += 32;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
		      gaiaExport64 (ptr + 16, z, 1, endian_arch);	/* Z - exterior ring */
		      gaiaExport64 (ptr + 24, m, 1, endian_arch);	/* M - exterior ring */
		      ptr += 32;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      default:
	  /* this one is a MULTIxxxx or a GEOMETRYCOLLECTION - building the main header */
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, type, 1, endian_arch);	/* geometric class */
	  gaiaExport32 (ptr + 43, entities, 1, endian_arch);	/* # entities */
	  ptr += 47;
	  point = geom->FirstPoint;
	  while (point)
	    {
		*ptr = GAIA_MARK_ENTITY;	/* ENTITY signature */
		if (geom->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINTZ, 1, endian_arch);	/* class POINT XYZ */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      gaiaExport64 (ptr + 21, point->Z, 1, endian_arch);	/* Z */
		      ptr += 29;
		  }
		else if (geom->DimensionModel == GAIA_XY_M)
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINTM, 1, endian_arch);	/* class POINT XYM */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      gaiaExport64 (ptr + 21, point->M, 1, endian_arch);	/* M */
		      ptr += 29;
		  }
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINTZM, 1, endian_arch);	/* class POINT XYZM */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      gaiaExport64 (ptr + 21, point->Z, 1, endian_arch);	/* Z */
		      gaiaExport64 (ptr + 29, point->M, 1, endian_arch);	/* M */
		      ptr += 37;
		  }
		else
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINT, 1, endian_arch);	/* class POINT */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      ptr += 21;
		  }
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		*ptr = GAIA_MARK_ENTITY;	/* ENTITY signature */
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaExport32 (ptr + 1, GAIA_LINESTRINGZ, 1, endian_arch);	/* class LINESTRING XYZ */
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaExport32 (ptr + 1, GAIA_LINESTRINGM, 1, endian_arch);	/* class LINESTRING XYM */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaExport32 (ptr + 1, GAIA_LINESTRINGZM, 1, endian_arch);	/* class LINESTRING XYZM */
		else
		    gaiaExport32 (ptr + 1, GAIA_LINESTRING, 1, endian_arch);	/* class LINESTRING */
		gaiaExport32 (ptr + 5, line->Points, 1, endian_arch);	/* # points */
		ptr += 9;
		for (iv = 0; iv < line->Points; iv++)
		  {
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
			}
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (line->Coords, iv, &x, &y);
			}
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y */
		      ptr += 16;
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
			    gaiaExport64 (ptr + 8, m, 1, endian_arch);	/* M */
			    ptr += 16;
			}
		  }
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		*ptr = GAIA_MARK_ENTITY;	/* ENTITY signature */
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaExport32 (ptr + 1, GAIA_POLYGONZ, 1, endian_arch);	/* class POLYGON XYZ */
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaExport32 (ptr + 1, GAIA_POLYGONM, 1, endian_arch);	/* class POLYGON XYM */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaExport32 (ptr + 1, GAIA_POLYGONZM, 1, endian_arch);	/* class POLYGON XYZM */
		else
		    gaiaExport32 (ptr + 1, GAIA_POLYGON, 1, endian_arch);	/* class POLYGON */
		gaiaExport32 (ptr + 5, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
		rng = polyg->Exterior;
		gaiaExport32 (ptr + 9, rng->Points, 1, endian_arch);	/* # points - exterior ring */
		ptr += 13;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
		      ptr += 16;
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
			    gaiaExport64 (ptr + 8, m, 1, endian_arch);	/* M */
			    ptr += 16;
			}
		  }
		for (ib = 0; ib < polyg->NumInteriors; ib++)
		  {
		      rng = polyg->Interiors + ib;
		      gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		      ptr += 4;
		      for (iv = 0; iv < rng->Points; iv++)
			{
			    if (geom->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			      }
			    else if (geom->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			      }
			    else if (geom->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (rng->Coords, iv, &x, &y,
						    &z, &m);
			      }
			    else
			      {
				  gaiaGetPoint (rng->Coords, iv, &x, &y);
			      }
			    gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
			    gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
			    ptr += 16;
			    if (geom->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
				  ptr += 8;
			      }
			    if (geom->DimensionModel == GAIA_XY_M)
			      {
				  gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
				  ptr += 8;
			      }
			    if (geom->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
				  gaiaExport64 (ptr + 8, m, 1, endian_arch);	/* M */
				  ptr += 16;
			      }
			}
		  }
		polyg = polyg->Next;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
      };
}

GAIAGEO_DECLARE void
gaiaToCompressedBlobWkb (gaiaGeomCollPtr geom, unsigned char **result,
			 int *size)
{
/* 
/ builds the SpatiaLite BLOB representation for this GEOMETRY 
/ geometry-compression will be applied to LINESTRINGs and RINGs
*/
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    double last_x = 0.0;
    double last_y = 0.0;
    double last_z = 0.0;
    float fx;
    float fy;
    float fz;
    int entities = 0;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    int type;
    unsigned char *ptr;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    int endian_arch = gaiaEndianArch ();
    gaiaMbrGeometry (geom);
/* how many entities, and of what kind, do we have ? */
    pt = geom->FirstPoint;
    while (pt)
      {
	  point = pt;
	  entities++;
	  n_points++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  line = ln;
	  entities++;
	  n_linestrings++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  polyg = pg;
	  entities++;
	  n_polygons++;
	  pg = pg->Next;
      }
    *size = 0;
    *result = NULL;
    if (n_points == 0 && n_polygons == 0 && n_linestrings == 0)
	return;
/* ok, we can determine the geometry class */
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOINTZM;
		else
		    type = GAIA_MULTIPOINT;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_POINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_POINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_POINTZM;
		else
		    type = GAIA_POINT;
	    }
      }
    else if (n_points > 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOINTZM;
		else
		    type = GAIA_MULTIPOINT;
	    }
      }
    else if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTILINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTILINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTILINESTRINGZM;
		else
		    type = GAIA_MULTILINESTRING;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_LINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_LINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_LINESTRINGZM;
		else
		    type = GAIA_LINESTRING;
	    }
      }
    else if (n_points == 0 && n_linestrings > 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTILINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTILINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTILINESTRINGZM;
		else
		    type = GAIA_MULTILINESTRING;
	    }
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOLYGONZM;
		else
		    type = GAIA_MULTIPOLYGON;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_POLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_POLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_POLYGONZM;
		else
		    type = GAIA_POLYGON;
	    }
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons > 1)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOLYGONZM;
		else
		    type = GAIA_MULTIPOLYGON;
	    }
      }
    else
      {
	  if (geom->DimensionModel == GAIA_XY_Z)
	      type = GAIA_GEOMETRYCOLLECTIONZ;
	  else if (geom->DimensionModel == GAIA_XY_M)
	      type = GAIA_GEOMETRYCOLLECTIONM;
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      type = GAIA_GEOMETRYCOLLECTIONZM;
	  else
	      type = GAIA_GEOMETRYCOLLECTION;
      }
/* and now we compute the size of BLOB */
    *size = 44;			/* header size */
    switch (type)
      {
      case GAIA_POINT:
	  *size += (sizeof (double) * 2);	/* [x,y] coords */
	  break;
      case GAIA_POINTZ:
	  *size += (sizeof (double) * 3);	/* [x,y,z] coords */
	  break;
      case GAIA_POINTM:
	  *size += (sizeof (double) * 3);	/* [x,y,m] coords */
	  break;
      case GAIA_POINTZM:
	  *size += (sizeof (double) * 4);	/* [x,y,z,m] coords */
	  break;
      case GAIA_LINESTRING:
	  *size += (4 + (8 * line->Points) + 16);	/* # points + [x,y] for each vertex */
	  break;
      case GAIA_LINESTRINGZ:
	  *size += (4 + (12 * line->Points) + 24);	/* # points + [x,y,z] for each vertex */
	  break;
      case GAIA_LINESTRINGM:
	  *size += (4 + (16 * line->Points) + 16);	/* # points + [x,y,m] for each vertex */
	  break;
      case GAIA_LINESTRINGZM:
	  *size += (4 + (20 * line->Points) + 24);	/* # points + [x,y,z,m] for each vertex */
	  break;
      case GAIA_POLYGON:
	  rng = polyg->Exterior;
	  *size += (8 + (8 * rng->Points) + 16);	/* # rings + # points + [x.y] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + (8 * rng->Points) + 16);	/* # points + [x,y] array - interior ring */
	    }
	  break;
      case GAIA_POLYGONZ:
	  rng = polyg->Exterior;
	  *size += (8 + (12 * rng->Points) + 24);	/* # rings + # points + [x,y,z] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + (12 * rng->Points) + 24);	/* # points + [x,y,z] array - interior ring */
	    }
	  break;
      case GAIA_POLYGONM:
	  rng = polyg->Exterior;
	  *size += (8 + (16 * rng->Points) + 16);	/* # rings + # points + [x,y,m] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + (16 * rng->Points) + 16);	/* # points + [x,y,m] array - interior ring */
	    }
	  break;
      case GAIA_POLYGONZM:
	  rng = polyg->Exterior;
	  *size += (8 + (20 * rng->Points) + 24);	/* # rings + # points + [x,y,z,m] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + (20 * rng->Points) + 24);	/* # points + [x,y,z,m] array - interior ring */
	    }
	  break;
      default:
	  /* this one is not a simple geometry; should be a MULTIxxxx or a GEOMETRYCOLLECTION */
	  *size += 4;		/* # entities */
	  point = geom->FirstPoint;
	  while (point)
	    {
		*size += 5;	/* entity header */
		if (geom->DimensionModel == GAIA_XY_Z
		    || geom->DimensionModel == GAIA_XY_M)
		    *size += (sizeof (double) * 3);	/* three doubles for each POINT */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *size += (sizeof (double) * 4);	/* four doubles for each POINT */
		else
		    *size += (sizeof (double) * 2);	/* two doubles for each POINT */
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		*size += 5;	/* entity header */
		if (geom->DimensionModel == GAIA_XY_Z)
		    *size += (4 + (12 * line->Points) + 24);	/* # points + [x,y,z] for each vertex */
		else if (geom->DimensionModel == GAIA_XY_M)
		    *size += (4 + (16 * line->Points) + 16);	/* # points + [x,y,m] for each vertex */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *size += (4 + (20 * line->Points) + 24);	/* # points + [x,y,z,m] for each vertex */
		else
		    *size += (4 + (8 * line->Points) + 16);	/* # points + [x,y] for each vertex */
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		*size += 5;	/* entity header */
		rng = polyg->Exterior;
		if (geom->DimensionModel == GAIA_XY_Z)
		    *size += (8 + (12 * rng->Points) + 24);	/* # rings + # points + [x,y,z] array - exterior ring */
		else if (geom->DimensionModel == GAIA_XY_M)
		    *size += (8 + (16 * rng->Points) + 16);	/* # rings + # points + [x,y,m] array - exterior ring */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *size += (8 + (20 * rng->Points) + 24);	/* # rings + # points + [x,y,z,m] array - exterior ring */
		else
		    *size += (8 + (8 * rng->Points) + 16);	/* # rings + # points + [x,y] array - exterior ring */
		for (ib = 0; ib < polyg->NumInteriors; ib++)
		  {
		      rng = polyg->Interiors + ib;
		      if (geom->DimensionModel == GAIA_XY_Z)
			  *size += (4 + (12 * rng->Points) + 24);	/* # points + [x,y,z] array - interior ring */
		      else if (geom->DimensionModel == GAIA_XY_M)
			  *size += (4 + (16 * rng->Points) + 16);	/* # points + [x,y,m] array - interior ring */
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			  *size += (4 + (20 * rng->Points) + 24);	/* # points + [x,y,z,m] array - interior ring */
		      else
			  *size += (4 + (8 * rng->Points) + 16);	/* # points + [x,y] array - interior ring */
		  }
		polyg = polyg->Next;
	    }
      };
    *result = malloc (*size);
    ptr = *result;
/* and finally we build the BLOB */
    switch (type)
      {
      case GAIA_POINT:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINT, 1, endian_arch);	/* class POINT */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  *(ptr + 59) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POINTZ:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINTZ, 1, endian_arch);	/* class POINT XYZ */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  gaiaExport64 (ptr + 59, point->Z, 1, endian_arch);	/* Z */
	  *(ptr + 67) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POINTM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINTM, 1, endian_arch);	/* class POINT XYM */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  gaiaExport64 (ptr + 59, point->M, 1, endian_arch);	/* M */
	  *(ptr + 67) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POINTZM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_POINTZM, 1, endian_arch);	/* class POINT XYZM */
	  gaiaExport64 (ptr + 43, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 51, point->Y, 1, endian_arch);	/* Y */
	  gaiaExport64 (ptr + 59, point->Z, 1, endian_arch);	/* M */
	  gaiaExport64 (ptr + 67, point->M, 1, endian_arch);	/* Z */
	  *(ptr + 75) = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRING:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_LINESTRING, 1, endian_arch);	/* class LINESTRING */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPoint (line->Coords, iv, &x, &y);
		if (iv == 0 || iv == (line->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      ptr += 16;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      ptr += 8;
		  }
		last_x = x;
		last_y = y;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRINGZ:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_LINESTRINGZ, 1, endian_arch);	/* class LINESTRING XYZ */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		if (iv == 0 || iv == (line->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      gaiaExport64 (ptr + 16, z, 1, endian_arch);
		      ptr += 24;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      fz = (float) (z - last_z);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      gaiaExportF32 (ptr + 8, fz, 1, endian_arch);
		      ptr += 12;
		  }
		last_x = x;
		last_y = y;
		last_z = z;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRINGM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_LINESTRINGM, 1, endian_arch);	/* class LINESTRING XYM */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		if (iv == 0 || iv == (line->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      gaiaExport64 (ptr + 16, m, 1, endian_arch);
		      ptr += 24;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      gaiaExport64 (ptr + 8, m, 1, endian_arch);
		      ptr += 16;
		  }
		last_x = x;
		last_y = y;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_LINESTRINGZM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_LINESTRINGZM, 1, endian_arch);	/* class LINESTRING XYZM */
	  gaiaExport32 (ptr + 43, line->Points, 1, endian_arch);	/* # points */
	  ptr += 47;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		if (iv == 0 || iv == (line->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      gaiaExport64 (ptr + 16, z, 1, endian_arch);
		      gaiaExport64 (ptr + 24, m, 1, endian_arch);
		      ptr += 32;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      fz = (float) (z - last_z);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      gaiaExportF32 (ptr + 8, fz, 1, endian_arch);
		      gaiaExport64 (ptr + 12, m, 1, endian_arch);
		      ptr += 20;
		  }
		last_x = x;
		last_y = y;
		last_z = z;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGON:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_POLYGON, 1, endian_arch);	/* class POLYGON */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPoint (rng->Coords, iv, &x, &y);
		if (iv == 0 || iv == (rng->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      ptr += 16;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      ptr += 8;
		  }
		last_x = x;
		last_y = y;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      if (iv == 0 || iv == (rng->Points - 1))
			{
			    /* first and last vertices are uncompressed */
			    gaiaExport64 (ptr, x, 1, endian_arch);
			    gaiaExport64 (ptr + 8, y, 1, endian_arch);
			    ptr += 16;
			}
		      else
			{
			    /* compressing any other intermediate vertex */
			    fx = (float) (x - last_x);
			    fy = (float) (y - last_y);
			    gaiaExportF32 (ptr, fx, 1, endian_arch);
			    gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
			    ptr += 8;
			}
		      last_x = x;
		      last_y = y;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGONZ:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_POLYGONZ, 1, endian_arch);	/* class POLYGON XYZ */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		if (iv == 0 || iv == (rng->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      gaiaExport64 (ptr + 16, z, 1, endian_arch);
		      ptr += 24;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      fz = (float) (z - last_z);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      gaiaExportF32 (ptr + 8, fz, 1, endian_arch);
		      ptr += 12;
		  }
		last_x = x;
		last_y = y;
		last_z = z;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      if (iv == 0 || iv == (rng->Points - 1))
			{
			    /* first and last vertices are uncompressed */
			    gaiaExport64 (ptr, x, 1, endian_arch);
			    gaiaExport64 (ptr + 8, y, 1, endian_arch);
			    gaiaExport64 (ptr + 16, z, 1, endian_arch);
			    ptr += 24;
			}
		      else
			{
			    /* compressing any other intermediate vertex */
			    fx = (float) (x - last_x);
			    fy = (float) (y - last_y);
			    fz = (float) (z - last_z);
			    gaiaExportF32 (ptr, fx, 1, endian_arch);
			    gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
			    gaiaExportF32 (ptr + 8, fz, 1, endian_arch);
			    ptr += 12;
			}
		      last_x = x;
		      last_y = y;
		      last_z = z;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGONM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_POLYGONM, 1, endian_arch);	/* class POLYGON XYM */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		if (iv == 0 || iv == (rng->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      gaiaExport64 (ptr + 16, m, 1, endian_arch);
		      ptr += 24;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      gaiaExport64 (ptr + 8, m, 1, endian_arch);
		      ptr += 16;
		  }
		last_x = x;
		last_y = y;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      if (iv == 0 || iv == (rng->Points - 1))
			{
			    /* first and last vertices are uncompressed */
			    gaiaExport64 (ptr, x, 1, endian_arch);
			    gaiaExport64 (ptr + 8, y, 1, endian_arch);
			    gaiaExport64 (ptr + 16, m, 1, endian_arch);
			    ptr += 24;
			}
		      else
			{
			    /* compressing any other intermediate vertex */
			    fx = (float) (x - last_x);
			    fy = (float) (y - last_y);
			    gaiaExportF32 (ptr, fx, 1, endian_arch);
			    gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
			    gaiaExport64 (ptr + 8, m, 1, endian_arch);
			    ptr += 16;
			}
		      last_x = x;
		      last_y = y;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      case GAIA_POLYGONZM:
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, GAIA_COMPRESSED_POLYGONZM, 1, endian_arch);	/* class POLYGON */
	  gaiaExport32 (ptr + 43, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 47, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 51;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		if (iv == 0 || iv == (rng->Points - 1))
		  {
		      /* first and last vertices are uncompressed */
		      gaiaExport64 (ptr, x, 1, endian_arch);
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);
		      gaiaExport64 (ptr + 16, z, 1, endian_arch);
		      gaiaExport64 (ptr + 24, m, 1, endian_arch);
		      ptr += 32;
		  }
		else
		  {
		      /* compressing any other intermediate vertex */
		      fx = (float) (x - last_x);
		      fy = (float) (y - last_y);
		      fz = (float) (z - last_z);
		      gaiaExportF32 (ptr, fx, 1, endian_arch);
		      gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
		      gaiaExportF32 (ptr + 8, fz, 1, endian_arch);
		      gaiaExport64 (ptr + 12, m, 1, endian_arch);
		      ptr += 20;
		  }
		last_x = x;
		last_y = y;
		last_z = z;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      if (iv == 0 || iv == (rng->Points - 1))
			{
			    /* first and last vertices are uncompressed */
			    gaiaExport64 (ptr, x, 1, endian_arch);
			    gaiaExport64 (ptr + 8, y, 1, endian_arch);
			    gaiaExport64 (ptr + 16, z, 1, endian_arch);
			    gaiaExport64 (ptr + 24, m, 1, endian_arch);
			    ptr += 32;
			}
		      else
			{
			    /* compressing any other intermediate vertex */
			    fx = (float) (x - last_x);
			    fy = (float) (y - last_y);
			    fz = (float) (z - last_z);
			    gaiaExportF32 (ptr, fx, 1, endian_arch);
			    gaiaExportF32 (ptr + 4, fy, 1, endian_arch);
			    gaiaExportF32 (ptr + 8, fz, 1, endian_arch);
			    gaiaExport64 (ptr + 12, m, 1, endian_arch);
			    ptr += 20;
			}
		      last_x = x;
		      last_y = y;
		      last_z = z;
		  }
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
	  break;
      default:
	  /* this one is a MULTIxxxx or a GEOMETRYCOLLECTION - building the main header */
	  *ptr = GAIA_MARK_START;	/* START signature */
	  *(ptr + 1) = GAIA_LITTLE_ENDIAN;	/* byte ordering */
	  gaiaExport32 (ptr + 2, geom->Srid, 1, endian_arch);	/* the SRID */
	  gaiaExport64 (ptr + 6, geom->MinX, 1, endian_arch);	/* MBR - minimum X */
	  gaiaExport64 (ptr + 14, geom->MinY, 1, endian_arch);	/* MBR - minimum Y */
	  gaiaExport64 (ptr + 22, geom->MaxX, 1, endian_arch);	/* MBR - maximum X */
	  gaiaExport64 (ptr + 30, geom->MaxY, 1, endian_arch);	/* MBR - maximum Y */
	  *(ptr + 38) = GAIA_MARK_MBR;	/* MBR signature */
	  gaiaExport32 (ptr + 39, type, 1, endian_arch);	/* geometric class */
	  gaiaExport32 (ptr + 43, entities, 1, endian_arch);	/* # entities */
	  ptr += 47;
	  point = geom->FirstPoint;
	  while (point)
	    {
		*ptr = GAIA_MARK_ENTITY;	/* ENTITY signature */
		if (geom->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINTZ, 1, endian_arch);	/* class POINT XYZ */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      gaiaExport64 (ptr + 21, point->Z, 1, endian_arch);	/* Z */
		      ptr += 29;
		  }
		else if (geom->DimensionModel == GAIA_XY_M)
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINTM, 1, endian_arch);	/* class POINT XYM */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      gaiaExport64 (ptr + 21, point->M, 1, endian_arch);	/* M */
		      ptr += 29;
		  }
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINTZM, 1, endian_arch);	/* class POINT XYZM */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      gaiaExport64 (ptr + 21, point->Z, 1, endian_arch);	/* Z */
		      gaiaExport64 (ptr + 29, point->M, 1, endian_arch);	/* M */
		      ptr += 37;
		  }
		else
		  {
		      gaiaExport32 (ptr + 1, GAIA_POINT, 1, endian_arch);	/* class POINT */
		      gaiaExport64 (ptr + 5, point->X, 1, endian_arch);	/* X */
		      gaiaExport64 (ptr + 13, point->Y, 1, endian_arch);	/* Y */
		      ptr += 21;
		  }
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		*ptr = GAIA_MARK_ENTITY;	/* ENTITY signature */
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_LINESTRINGZ, 1, endian_arch);	/* class LINESTRING XYZ */
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_LINESTRINGM, 1, endian_arch);	/* class LINESTRING XYM */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_LINESTRINGZM, 1, endian_arch);	/* class LINESTRING XYZM */
		else
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_LINESTRING, 1, endian_arch);	/* class LINESTRING */
		gaiaExport32 (ptr + 5, line->Points, 1, endian_arch);	/* # points */
		ptr += 9;
		for (iv = 0; iv < line->Points; iv++)
		  {
		      z = 0.0;
		      m = 0.0;
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
			}
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (line->Coords, iv, &x, &y);
			}
		      if (iv == 0 || iv == (line->Points - 1))
			{
			    /* first and last vertices are uncompressed */
			    gaiaExport64 (ptr, x, 1, endian_arch);	/* X */
			    gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y */
			    ptr += 16;
			}
		      else
			{
			    /* compressing any other intermediate vertex */
			    fx = (float) (x - last_x);
			    fy = (float) (y - last_y);
			    gaiaExportF32 (ptr, fx, 1, endian_arch);	/* X */
			    gaiaExportF32 (ptr + 4, fy, 1, endian_arch);	/* Y */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    if (iv == 0 || iv == (line->Points - 1))
			      {
				  /* first and last vertices are uncompressed */
				  gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
				  ptr += 8;
			      }
			    else
			      {
				  /* compressing any other intermediate vertex */
				  fz = (float) (z - last_z);
				  gaiaExportF32 (ptr, fz, 1, endian_arch);	/* Z */
				  ptr += 4;
			      }
			}
		      if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    if (iv == 0 || iv == (line->Points - 1))
			      {
				  /* first and last vertices are uncompressed */
				  gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
				  ptr += 8;
			      }
			    else
			      {
				  /* compressing any other intermediate vertex */
				  fz = (float) (z - last_z);
				  gaiaExportF32 (ptr, fz, 1, endian_arch);	/* Z */
				  ptr += 4;
			      }
			    gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
			    ptr += 8;
			}
		      last_x = x;
		      last_y = y;
		      last_z = z;
		  }
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		*ptr = GAIA_MARK_ENTITY;	/* ENTITY signature */
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_POLYGONZ, 1, endian_arch);	/* class POLYGON XYZ */
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_POLYGONM, 1, endian_arch);	/* class POLYGON XYM */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_POLYGONZM, 1, endian_arch);	/* class POLYGON XYZM */
		else
		    gaiaExport32 (ptr + 1, GAIA_COMPRESSED_POLYGON, 1, endian_arch);	/* class POLYGON */
		gaiaExport32 (ptr + 5, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
		rng = polyg->Exterior;
		gaiaExport32 (ptr + 9, rng->Points, 1, endian_arch);	/* # points - exterior ring */
		ptr += 13;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      z = 0.0;
		      m = 0.0;
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      if (iv == 0 || iv == (rng->Points - 1))
			{
			    /* first and last vertices are uncompressed */
			    gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
			    gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
			    ptr += 16;
			}
		      else
			{
			    /* compressing any other intermediate vertex */
			    fx = (float) (x - last_x);
			    fy = (float) (y - last_y);
			    gaiaExportF32 (ptr, fx, 1, endian_arch);	/* X */
			    gaiaExportF32 (ptr + 4, fy, 1, endian_arch);	/* Y */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    if (iv == 0 || iv == (rng->Points - 1))
			      {
				  /* first and last vertices are uncompressed */
				  gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
				  ptr += 8;
			      }
			    else
			      {
				  /* compressing any other intermediate vertex */
				  fz = (float) (z - last_z);
				  gaiaExportF32 (ptr, fz, 1, endian_arch);	/* Z */
				  ptr += 4;
			      }
			}
		      if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
			    ptr += 8;
			}
		      if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    if (iv == 0 || iv == (rng->Points - 1))
			      {
				  /* first and last vertices are uncompressed */
				  gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
				  ptr += 8;
			      }
			    else
			      {
				  /* compressing any other intermediate vertex */
				  fz = (float) (z - last_z);
				  gaiaExportF32 (ptr, fz, 1, endian_arch);	/* Z */
				  ptr += 4;
			      }
			    gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
			    ptr += 8;
			}
		      last_x = x;
		      last_y = y;
		      last_z = z;
		  }
		for (ib = 0; ib < polyg->NumInteriors; ib++)
		  {
		      rng = polyg->Interiors + ib;
		      gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		      ptr += 4;
		      for (iv = 0; iv < rng->Points; iv++)
			{
			    z = 0.0;
			    m = 0.0;
			    if (geom->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			      }
			    else if (geom->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			      }
			    else if (geom->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z,
						    &m);
			      }
			    else
			      {
				  gaiaGetPoint (rng->Coords, iv, &x, &y);
			      }
			    if (iv == 0 || iv == (rng->Points - 1))
			      {
				  /* first and last vertices are uncompressed */
				  gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
				  gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
				  ptr += 16;
			      }
			    else
			      {
				  /* compressing any other intermediate vertex */
				  fx = (float) (x - last_x);
				  fy = (float) (y - last_y);
				  gaiaExportF32 (ptr, fx, 1, endian_arch);	/* X */
				  gaiaExportF32 (ptr + 4, fy, 1, endian_arch);	/* Y */
				  ptr += 8;
			      }
			    if (geom->DimensionModel == GAIA_XY_Z)
			      {
				  if (iv == 0 || iv == (rng->Points - 1))
				    {
					/* first and last vertices are uncompressed */
					gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
					ptr += 8;
				    }
				  else
				    {
					/* compressing any other intermediate vertex */
					fz = (float) (z - last_z);
					gaiaExportF32 (ptr, fz, 1, endian_arch);	/* Z */
					ptr += 4;
				    }
			      }
			    if (geom->DimensionModel == GAIA_XY_M)
			      {
				  gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
				  ptr += 8;
			      }
			    if (geom->DimensionModel == GAIA_XY_Z_M)
			      {
				  if (iv == 0 || iv == (rng->Points - 1))
				    {
					/* first and last vertices are uncompressed */
					gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
					ptr += 8;
				    }
				  else
				    {
					/* compressing any other intermediate vertex */
					fz = (float) (z - last_z);
					gaiaExportF32 (ptr, fz, 1, endian_arch);	/* Z */
					ptr += 4;
				    }
				  gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
				  ptr += 8;
			      }
			    last_x = x;
			    last_y = y;
			    last_z = z;
			}
		  }
		polyg = polyg->Next;
	    }
	  *ptr = GAIA_MARK_END;	/* END signature */
      };
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromWkb (const unsigned char *blob, unsigned int size)
{
/* decoding from WKB to GEOMETRY  */
    int type;
    int little_endian;
    gaiaGeomCollPtr geo = NULL;
    int endian_arch = gaiaEndianArch ();
    if (size < 5)
	return NULL;
    if (*(blob + 0) == 0x01)
	little_endian = GAIA_LITTLE_ENDIAN;
    else
	little_endian = GAIA_BIG_ENDIAN;
    type = gaiaImport32 (blob + 1, little_endian, endian_arch);
    if (type == GAIA_POINTZ || type == GAIA_LINESTRINGZ || type == GAIA_POLYGONZ
	|| type == GAIA_MULTIPOINTZ || type == GAIA_MULTILINESTRINGZ
	|| type == GAIA_MULTIPOLYGONZ || type == GAIA_GEOMETRYCOLLECTIONZ
	|| type == GAIA_GEOSWKB_POINTZ || type == GAIA_GEOSWKB_LINESTRINGZ
	|| type == GAIA_GEOSWKB_POLYGONZ || type == GAIA_GEOSWKB_MULTIPOINTZ
	|| type == GAIA_GEOSWKB_MULTILINESTRINGZ
	|| type == GAIA_GEOSWKB_MULTIPOLYGONZ
	|| type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
	geo = gaiaAllocGeomCollXYZ ();
    else if (type == GAIA_POINTM || type == GAIA_LINESTRINGM
	     || type == GAIA_POLYGONM || type == GAIA_MULTIPOINTM
	     || type == GAIA_MULTILINESTRINGM || type == GAIA_MULTIPOLYGONM
	     || type == GAIA_GEOMETRYCOLLECTIONM)
	geo = gaiaAllocGeomCollXYM ();
    else if (type == GAIA_POINTZM || type == GAIA_LINESTRINGZM
	     || type == GAIA_POLYGONZM || type == GAIA_MULTIPOINTZM
	     || type == GAIA_MULTILINESTRINGZM || type == GAIA_MULTIPOLYGONZM
	     || type == GAIA_GEOMETRYCOLLECTIONZM)
	geo = gaiaAllocGeomCollXYZM ();
    else
	geo = gaiaAllocGeomColl ();
    geo->Srid = -1;
    geo->endian_arch = (char) endian_arch;
    geo->endian = (char) little_endian;
    geo->blob = blob;
    geo->size = size;
    geo->offset = 5;
    switch (type)
      {
      case GAIA_POINT:
	  ParseWkbPoint (geo);
	  break;
      case GAIA_POINTZ:
      case GAIA_GEOSWKB_POINTZ:
	  ParseWkbPointZ (geo);
	  break;
      case GAIA_POINTM:
	  ParseWkbPointM (geo);
	  break;
      case GAIA_POINTZM:
	  ParseWkbPointZM (geo);
	  break;
      case GAIA_LINESTRING:
	  ParseWkbLine (geo);
	  break;
      case GAIA_LINESTRINGZ:
      case GAIA_GEOSWKB_LINESTRINGZ:
	  ParseWkbLineZ (geo);
	  break;
      case GAIA_LINESTRINGM:
	  ParseWkbLineM (geo);
	  break;
      case GAIA_LINESTRINGZM:
	  ParseWkbLineZM (geo);
	  break;
      case GAIA_POLYGON:
	  ParseWkbPolygon (geo);
	  break;
      case GAIA_POLYGONZ:
      case GAIA_GEOSWKB_POLYGONZ:
	  ParseWkbPolygonZ (geo);
	  break;
      case GAIA_POLYGONM:
	  ParseWkbPolygonM (geo);
	  break;
      case GAIA_POLYGONZM:
	  ParseWkbPolygonZM (geo);
	  break;
      case GAIA_MULTIPOINT:
      case GAIA_MULTILINESTRING:
      case GAIA_MULTIPOLYGON:
      case GAIA_GEOMETRYCOLLECTION:
      case GAIA_MULTIPOINTZ:
      case GAIA_MULTILINESTRINGZ:
      case GAIA_MULTIPOLYGONZ:
      case GAIA_GEOMETRYCOLLECTIONZ:
      case GAIA_GEOSWKB_MULTIPOINTZ:
      case GAIA_GEOSWKB_MULTILINESTRINGZ:
      case GAIA_GEOSWKB_MULTIPOLYGONZ:
      case GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ:
      case GAIA_MULTIPOINTM:
      case GAIA_MULTILINESTRINGM:
      case GAIA_MULTIPOLYGONM:
      case GAIA_GEOMETRYCOLLECTIONM:
      case GAIA_MULTIPOINTZM:
      case GAIA_MULTILINESTRINGZM:
      case GAIA_MULTIPOLYGONZM:
      case GAIA_GEOMETRYCOLLECTIONZM:
	  ParseWkbGeometry (geo);
	  break;
      default:
	  break;
      };
    gaiaMbrGeometry (geo);
    switch (type)
      {
      case GAIA_POINT:
      case GAIA_POINTZ:
      case GAIA_GEOSWKB_POINTZ:
      case GAIA_POINTM:
      case GAIA_POINTZM:
	  geo->DeclaredType = GAIA_POINT;
	  break;
      case GAIA_LINESTRING:
      case GAIA_LINESTRINGZ:
      case GAIA_GEOSWKB_LINESTRINGZ:
      case GAIA_LINESTRINGM:
      case GAIA_LINESTRINGZM:
	  geo->DeclaredType = GAIA_LINESTRING;
	  break;
      case GAIA_POLYGON:
      case GAIA_POLYGONZ:
      case GAIA_GEOSWKB_POLYGONZ:
      case GAIA_POLYGONM:
      case GAIA_POLYGONZM:
	  geo->DeclaredType = GAIA_POLYGON;
	  break;
      case GAIA_MULTIPOINT:
      case GAIA_MULTIPOINTZ:
      case GAIA_GEOSWKB_MULTIPOINTZ:
      case GAIA_MULTIPOINTM:
      case GAIA_MULTIPOINTZM:
	  geo->DeclaredType = GAIA_MULTIPOINT;
	  break;
      case GAIA_MULTILINESTRING:
      case GAIA_MULTILINESTRINGZ:
      case GAIA_GEOSWKB_MULTILINESTRINGZ:
      case GAIA_MULTILINESTRINGM:
      case GAIA_MULTILINESTRINGZM:
	  geo->DeclaredType = GAIA_MULTILINESTRING;
	  break;
      case GAIA_MULTIPOLYGON:
      case GAIA_MULTIPOLYGONZ:
      case GAIA_GEOSWKB_MULTIPOLYGONZ:
      case GAIA_MULTIPOLYGONM:
      case GAIA_MULTIPOLYGONZM:
	  geo->DeclaredType = GAIA_MULTIPOLYGON;
	  break;
      case GAIA_GEOMETRYCOLLECTION:
      case GAIA_GEOMETRYCOLLECTIONZ:
      case GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ:
      case GAIA_GEOMETRYCOLLECTIONM:
      case GAIA_GEOMETRYCOLLECTIONZM:
	  geo->DeclaredType = GAIA_GEOMETRYCOLLECTION;
	  break;
      }
    return geo;
}

GAIAGEO_DECLARE char *
gaiaToHexWkb (gaiaGeomCollPtr geom)
{
/* builds the hexadecimal WKB representation for this GEOMETRY */
    unsigned char *wkb = NULL;
    int size = 0;
    char *hexbuf = NULL;
    int i;
    char hex[16];
    char *p;
    gaiaToWkb (geom, &wkb, &size);
    if (!wkb)
	return NULL;
    hexbuf = malloc ((size * 2) + 1);
    p = hexbuf;
    for (i = 0; i < size; i++)
      {
	  sprintf (hex, "%02X", *(wkb + i));
	  *p++ = hex[0];
	  *p++ = hex[1];
      }
    *p = '\0';
    return hexbuf;
}

GAIAGEO_DECLARE void
gaiaToWkb (gaiaGeomCollPtr geom, unsigned char **result, int *size)
{
/* builds the WKB representation for this GEOMETRY */
    int ib;
    int iv;
    double x;
    double y;
    double z = 0.0;
    double m = 0.0;
    int entities = 0;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    int type;
    unsigned char *ptr;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    int endian_arch = gaiaEndianArch ();
    gaiaMbrGeometry (geom);
/* how many entities, and of what kind, do we have ? */
    pt = geom->FirstPoint;
    while (pt)
      {
	  point = pt;
	  entities++;
	  n_points++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  line = ln;
	  entities++;
	  n_linestrings++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  polyg = pg;
	  entities++;
	  n_polygons++;
	  pg = pg->Next;
      }
    *size = 0;
    *result = NULL;
    if (n_points == 0 && n_polygons == 0 && n_linestrings == 0)
	return;
/* ok, we can determine the geometry class */
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOINTZM;
		else
		    type = GAIA_MULTIPOINT;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_POINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_POINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_POINTZM;
		else
		    type = GAIA_POINT;
	    }
      }
    else if (n_points > 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOINTZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOINTM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOINTZM;
		else
		    type = GAIA_MULTIPOINT;
	    }
      }
    else if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTILINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTILINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTILINESTRINGZM;
		else
		    type = GAIA_MULTILINESTRING;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_LINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_LINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_LINESTRINGZM;
		else
		    type = GAIA_LINESTRING;
	    }
      }
    else if (n_points == 0 && n_linestrings > 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTILINESTRINGZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTILINESTRINGM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTILINESTRINGZM;
		else
		    type = GAIA_MULTILINESTRING;
	    }
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOLYGONZM;
		else
		    type = GAIA_MULTIPOLYGON;
	    }
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_POLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_POLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_POLYGONZM;
		else
		    type = GAIA_POLYGON;
	    }
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons > 1)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_GEOMETRYCOLLECTIONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_GEOMETRYCOLLECTIONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_GEOMETRYCOLLECTIONZM;
		else
		    type = GAIA_GEOMETRYCOLLECTION;
	    }
	  else
	    {
		if (geom->DimensionModel == GAIA_XY_Z)
		    type = GAIA_MULTIPOLYGONZ;
		else if (geom->DimensionModel == GAIA_XY_M)
		    type = GAIA_MULTIPOLYGONM;
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    type = GAIA_MULTIPOLYGONZM;
		else
		    type = GAIA_MULTIPOLYGON;
	    }
      }
    else
      {
	  if (geom->DimensionModel == GAIA_XY_Z)
	      type = GAIA_GEOMETRYCOLLECTIONZ;
	  else if (geom->DimensionModel == GAIA_XY_M)
	      type = GAIA_GEOMETRYCOLLECTIONM;
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      type = GAIA_GEOMETRYCOLLECTIONZM;
	  else
	      type = GAIA_GEOMETRYCOLLECTION;
      }
/* and now we compute the size of WKB */
    *size = 5;			/* header size */
    if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION
	|| type == GAIA_MULTIPOINTZ || type == GAIA_MULTILINESTRINGZ
	|| type == GAIA_MULTIPOLYGONZ || type == GAIA_GEOMETRYCOLLECTIONZ
	|| type == GAIA_MULTIPOINTM || type == GAIA_MULTILINESTRINGM
	|| type == GAIA_MULTIPOLYGONM || type == GAIA_GEOMETRYCOLLECTIONM
	|| type == GAIA_MULTIPOINTZM || type == GAIA_MULTILINESTRINGZM
	|| type == GAIA_MULTIPOLYGONZM || type == GAIA_GEOMETRYCOLLECTIONZM)
	*size += 4;
    point = geom->FirstPoint;
    while (point)
      {
	  if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	      || type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION
	      || type == GAIA_MULTIPOINTZ || type == GAIA_MULTILINESTRINGZ
	      || type == GAIA_MULTIPOLYGONZ || type == GAIA_GEOMETRYCOLLECTIONZ
	      || type == GAIA_MULTIPOINTM || type == GAIA_MULTILINESTRINGM
	      || type == GAIA_MULTIPOLYGONM || type == GAIA_GEOMETRYCOLLECTIONM
	      || type == GAIA_MULTIPOINTZM || type == GAIA_MULTILINESTRINGZM
	      || type == GAIA_MULTIPOLYGONZM
	      || type == GAIA_GEOMETRYCOLLECTIONZM)
	      *size += 5;
	  if (geom->DimensionModel == GAIA_XY_Z
	      || geom->DimensionModel == GAIA_XY_M)
	      *size += (sizeof (double) * 3);	/* three doubles for each POINT */
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      *size += (sizeof (double) * 4);	/* four doubles for each POINT */
	  else
	      *size += (sizeof (double) * 2);	/* two doubles for each POINT */
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	      || type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION
	      || type == GAIA_MULTIPOINTZ || type == GAIA_MULTILINESTRINGZ
	      || type == GAIA_MULTIPOLYGONZ || type == GAIA_GEOMETRYCOLLECTIONZ
	      || type == GAIA_MULTIPOINTM || type == GAIA_MULTILINESTRINGM
	      || type == GAIA_MULTIPOLYGONM || type == GAIA_GEOMETRYCOLLECTIONM
	      || type == GAIA_MULTIPOINTZM || type == GAIA_MULTILINESTRINGZM
	      || type == GAIA_MULTIPOLYGONZM
	      || type == GAIA_GEOMETRYCOLLECTIONZM)
	      *size += 5;
	  if (geom->DimensionModel == GAIA_XY_Z
	      || geom->DimensionModel == GAIA_XY_M)
	      *size += (4 + ((sizeof (double) * 3) * line->Points));	/* # points + [x,y,z] for each vertex */
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      *size += (4 + ((sizeof (double) * 4) * line->Points));	/* # points + [x,y,z,m] for each vertex */
	  else
	      *size += (4 + ((sizeof (double) * 2) * line->Points));	/* # points + [x,y] for each vertex */
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	      || type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION
	      || type == GAIA_MULTIPOINTZ || type == GAIA_MULTILINESTRINGZ
	      || type == GAIA_MULTIPOLYGONZ || type == GAIA_GEOMETRYCOLLECTIONZ
	      || type == GAIA_MULTIPOINTM || type == GAIA_MULTILINESTRINGM
	      || type == GAIA_MULTIPOLYGONM || type == GAIA_GEOMETRYCOLLECTIONM
	      || type == GAIA_MULTIPOINTZM || type == GAIA_MULTILINESTRINGZM
	      || type == GAIA_MULTIPOLYGONZM
	      || type == GAIA_GEOMETRYCOLLECTIONZM)
	      *size += 5;
	  rng = polyg->Exterior;
	  if (geom->DimensionModel == GAIA_XY_Z
	      || geom->DimensionModel == GAIA_XY_M)
	      *size += (8 + ((sizeof (double) * 3) * rng->Points));	/* # rings + # points + [x,y,z] array - exterior ring */
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      *size += (8 + ((sizeof (double) * 4) * rng->Points));	/* # rings + # points + [x,y,z,m] array - exterior ring */
	  else
	      *size += (8 + ((sizeof (double) * 2) * rng->Points));	/* # rings + # points + [x,y] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		if (geom->DimensionModel == GAIA_XY_Z
		    || geom->DimensionModel == GAIA_XY_M)
		    *size += (4 + ((sizeof (double) * 3) * rng->Points));	/* # points + [x,y,z] array - interior ring */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *size += (4 + ((sizeof (double) * 4) * rng->Points));	/* # points + [x,y,z,m] array - interior ring */
		else
		    *size += (4 + ((sizeof (double) * 2) * rng->Points));	/* # points + [x,y] array - interior ring */
	    }
	  polyg = polyg->Next;
      }
    *result = malloc (*size);
    ptr = *result;
/* and finally we build the WKB */
    *ptr = 0x01;		/* little endian byte order */
    gaiaExport32 (ptr + 1, type, 1, endian_arch);	/* the main CLASS TYPE */
    ptr += 5;
    if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION
	|| type == GAIA_MULTIPOINTZ || type == GAIA_MULTILINESTRINGZ
	|| type == GAIA_MULTIPOLYGONZ || type == GAIA_GEOMETRYCOLLECTIONZ
	|| type == GAIA_MULTIPOINTM || type == GAIA_MULTILINESTRINGM
	|| type == GAIA_MULTIPOLYGONM || type == GAIA_GEOMETRYCOLLECTIONM
	|| type == GAIA_MULTIPOINTZM || type == GAIA_MULTILINESTRINGZM
	|| type == GAIA_MULTIPOLYGONZM || type == GAIA_GEOMETRYCOLLECTIONZM)
      {
	  gaiaExport32 (ptr, entities, 1, endian_arch);	/* it's a collection; # entities */
	  ptr += 4;
      }
    point = geom->FirstPoint;
    while (point)
      {
	  if (type == GAIA_MULTIPOINT || type == GAIA_GEOMETRYCOLLECTION
	      || type == GAIA_MULTIPOINTZ || type == GAIA_GEOMETRYCOLLECTIONZ
	      || type == GAIA_MULTIPOINTM || type == GAIA_GEOMETRYCOLLECTIONM
	      || type == GAIA_MULTIPOINTZM || type == GAIA_GEOMETRYCOLLECTIONZM)
	    {
		*ptr = 0x01;
		/* it's a collection: the CLASS TYPE for this element */
		if (type == GAIA_MULTIPOINTZ
		    || type == GAIA_GEOMETRYCOLLECTIONZ)
		    gaiaExport32 (ptr + 1, GAIA_POINTZ, 1, endian_arch);
		else if (type == GAIA_MULTIPOINTM
			 || type == GAIA_GEOMETRYCOLLECTIONM)
		    gaiaExport32 (ptr + 1, GAIA_POINTM, 1, endian_arch);
		else if (type == GAIA_MULTIPOINTZM
			 || type == GAIA_GEOMETRYCOLLECTIONZM)
		    gaiaExport32 (ptr + 1, GAIA_POINTZM, 1, endian_arch);
		else
		    gaiaExport32 (ptr + 1, GAIA_POINT, 1, endian_arch);
		ptr += 5;
	    }
	  gaiaExport64 (ptr, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 8, point->Y, 1, endian_arch);	/* Y */
	  ptr += 16;
	  if (type == GAIA_POINTZ || type == GAIA_MULTIPOINTZ
	      || type == GAIA_GEOMETRYCOLLECTIONZ)
	    {
		gaiaExport64 (ptr, point->Z, 1, endian_arch);	/* Z */
		ptr += 8;
	    }
	  if (type == GAIA_POINTM || type == GAIA_MULTIPOINTM
	      || type == GAIA_GEOMETRYCOLLECTIONM)
	    {
		gaiaExport64 (ptr, point->M, 1, endian_arch);	/* M */
		ptr += 8;
	    }
	  if (type == GAIA_POINTZM || type == GAIA_MULTIPOINTZM
	      || type == GAIA_GEOMETRYCOLLECTIONZM)
	    {
		gaiaExport64 (ptr, point->Z, 1, endian_arch);	/* Z */
		gaiaExport64 (ptr + 8, point->M, 1, endian_arch);	/* M */
		ptr += 16;
	    }
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  if (type == GAIA_MULTILINESTRING || type == GAIA_GEOMETRYCOLLECTION
	      || type == GAIA_MULTILINESTRINGZ
	      || type == GAIA_GEOMETRYCOLLECTIONZ
	      || type == GAIA_MULTILINESTRINGM
	      || type == GAIA_GEOMETRYCOLLECTIONM
	      || type == GAIA_MULTILINESTRINGZM
	      || type == GAIA_GEOMETRYCOLLECTIONZM)
	    {
		*ptr = 0x01;
		/* it's a collection: the CLASS TYPE for this element */
		if (type == GAIA_MULTILINESTRINGZ
		    || type == GAIA_GEOMETRYCOLLECTIONZ)
		    gaiaExport32 (ptr + 1, GAIA_LINESTRINGZ, 1, endian_arch);
		else if (type == GAIA_MULTILINESTRINGM
			 || type == GAIA_GEOMETRYCOLLECTIONM)
		    gaiaExport32 (ptr + 1, GAIA_LINESTRINGM, 1, endian_arch);
		else if (type == GAIA_MULTILINESTRINGZM
			 || type == GAIA_GEOMETRYCOLLECTIONZM)
		    gaiaExport32 (ptr + 1, GAIA_LINESTRINGZM, 1, endian_arch);
		else
		    gaiaExport32 (ptr + 1, GAIA_LINESTRING, 1, endian_arch);
		ptr += 5;
	    }
	  gaiaExport32 (ptr, line->Points, 1, endian_arch);	/* # points */
	  ptr += 4;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		if (type == GAIA_LINESTRINGZ || type == GAIA_MULTILINESTRINGZ
		    || type == GAIA_GEOMETRYCOLLECTIONZ)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (type == GAIA_LINESTRINGM
			 || type == GAIA_MULTILINESTRINGM
			 || type == GAIA_GEOMETRYCOLLECTIONM)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (type == GAIA_LINESTRINGZM
			 || type == GAIA_MULTILINESTRINGZM
			 || type == GAIA_GEOMETRYCOLLECTIONZM)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y */
		ptr += 16;
		if (type == GAIA_LINESTRINGZ || type == GAIA_MULTILINESTRINGZ
		    || type == GAIA_GEOMETRYCOLLECTIONZ)
		  {
		      gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
		      ptr += 8;
		  }
		if (type == GAIA_LINESTRINGM || type == GAIA_MULTILINESTRINGM
		    || type == GAIA_GEOMETRYCOLLECTIONM)
		  {
		      gaiaExport64 (ptr, m, 1, endian_arch);	/* M */
		      ptr += 8;
		  }
		if (type == GAIA_LINESTRINGZM || type == GAIA_MULTILINESTRINGZM
		    || type == GAIA_GEOMETRYCOLLECTIONZM)
		  {
		      gaiaExport64 (ptr, z, 1, endian_arch);	/* Z */
		      gaiaExport64 (ptr + 8, m, 1, endian_arch);	/* M */
		      ptr += 16;
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  if (type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION
	      || type == GAIA_MULTIPOLYGONZ || type == GAIA_GEOMETRYCOLLECTIONZ
	      || type == GAIA_MULTIPOLYGONM || type == GAIA_GEOMETRYCOLLECTIONM
	      || type == GAIA_MULTIPOLYGONZM
	      || type == GAIA_GEOMETRYCOLLECTIONZM)
	    {
		*ptr = 0x01;
		/* it's a collection: the CLASS TYPE for this element */
		if (type == GAIA_MULTIPOLYGONZ
		    || type == GAIA_GEOMETRYCOLLECTIONZ)
		    gaiaExport32 (ptr + 1, GAIA_POLYGONZ, 1, endian_arch);
		else if (type == GAIA_MULTIPOLYGONM
			 || type == GAIA_GEOMETRYCOLLECTIONM)
		    gaiaExport32 (ptr + 1, GAIA_POLYGONM, 1, endian_arch);
		else if (type == GAIA_MULTIPOLYGONZM
			 || type == GAIA_GEOMETRYCOLLECTIONZM)
		    gaiaExport32 (ptr + 1, GAIA_POLYGONZM, 1, endian_arch);
		else
		    gaiaExport32 (ptr + 1, GAIA_POLYGON, 1, endian_arch);
		ptr += 5;
	    }
	  gaiaExport32 (ptr, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 4, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 8;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		if (type == GAIA_POLYGONZ || type == GAIA_MULTIPOLYGONZ
		    || type == GAIA_GEOMETRYCOLLECTIONZ)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		  }
		else if (type == GAIA_POLYGONM || type == GAIA_MULTIPOLYGONM
			 || type == GAIA_GEOMETRYCOLLECTIONM)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		  }
		else if (type == GAIA_POLYGONZM || type == GAIA_MULTIPOLYGONZM
			 || type == GAIA_GEOMETRYCOLLECTIONZM)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		  }
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
		ptr += 16;
		if (type == GAIA_POLYGONZ || type == GAIA_MULTIPOLYGONZ
		    || type == GAIA_GEOMETRYCOLLECTIONZ)
		  {
		      gaiaExport64 (ptr, z, 1, endian_arch);	/* Z - exterior ring */
		      ptr += 8;
		  }
		if (type == GAIA_POLYGONM || type == GAIA_MULTIPOLYGONM
		    || type == GAIA_GEOMETRYCOLLECTIONM)
		  {
		      gaiaExport64 (ptr, m, 1, endian_arch);	/* M - exterior ring */
		      ptr += 8;
		  }
		if (type == GAIA_POLYGONZM || type == GAIA_MULTIPOLYGONZM
		    || type == GAIA_GEOMETRYCOLLECTIONZM)
		  {
		      gaiaExport64 (ptr, z, 1, endian_arch);	/* Z - exterior ring */
		      gaiaExport64 (ptr + 8, m, 1, endian_arch);	/* M - exterior ring */
		      ptr += 16;
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      if (type == GAIA_POLYGONZ || type == GAIA_MULTIPOLYGONZ
			  || type == GAIA_GEOMETRYCOLLECTIONZ)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (type == GAIA_POLYGONM
			       || type == GAIA_MULTIPOLYGONM
			       || type == GAIA_GEOMETRYCOLLECTIONM)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else if (type == GAIA_POLYGONZM
			       || type == GAIA_MULTIPOLYGONZM
			       || type == GAIA_GEOMETRYCOLLECTIONZM)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
		      ptr += 16;
		      if (type == GAIA_POLYGONZ || type == GAIA_MULTIPOLYGONZ
			  || type == GAIA_GEOMETRYCOLLECTIONZ)
			{
			    gaiaExport64 (ptr, z, 1, endian_arch);	/* Z - exterior ring */
			    ptr += 8;
			}
		      if (type == GAIA_POLYGONM || type == GAIA_MULTIPOLYGONM
			  || type == GAIA_GEOMETRYCOLLECTIONM)
			{
			    gaiaExport64 (ptr, m, 1, endian_arch);	/* M - exterior ring */
			    ptr += 8;
			}
		      if (type == GAIA_POLYGONZM || type == GAIA_MULTIPOLYGONZM
			  || type == GAIA_GEOMETRYCOLLECTIONZM)
			{
			    gaiaExport64 (ptr, z, 1, endian_arch);	/* Z - exterior ring */
			    gaiaExport64 (ptr + 8, m, 1, endian_arch);	/* M - exterior ring */
			    ptr += 16;
			}
		  }
	    }
	  polyg = polyg->Next;
      }
}

static int
ewkbGetPoint (gaiaGeomCollPtr geom, unsigned char *blob,
	      int offset, int blob_size, int endian, int endian_arch, int dims)
{
/* decodes a POINT from PostGIS EWKB binary GEOMETRY */
    double x;
    double y;
    double z;
    double m;
    switch (dims)
      {
      case GAIA_XY_Z_M:
	  if (blob_size < offset + 32)
	      return -1;
	  break;
      case GAIA_XY_Z:
      case GAIA_XY_M:
	  if (blob_size < offset + 24)
	      return -1;
	  break;
      default:
	  if (blob_size < offset + 16)
	      return -1;
	  break;
      }
    x = gaiaImport64 (blob + offset, endian, endian_arch);
    offset += 8;
    y = gaiaImport64 (blob + offset, endian, endian_arch);
    offset += 8;
    if (dims == GAIA_XY_Z_M)
      {
	  z = gaiaImport64 (blob + offset, endian, endian_arch);
	  offset += 8;
	  m = gaiaImport64 (blob + offset, endian, endian_arch);
	  offset += 8;
	  gaiaAddPointToGeomCollXYZM (geom, x, y, z, m);
      }
    else if (dims == GAIA_XY_Z)
      {
	  z = gaiaImport64 (blob + offset, endian, endian_arch);
	  offset += 8;
	  gaiaAddPointToGeomCollXYZ (geom, x, y, z);
      }
    else if (dims == GAIA_XY_M)
      {
	  m = gaiaImport64 (blob + offset, endian, endian_arch);
	  offset += 8;
	  gaiaAddPointToGeomCollXYM (geom, x, y, m);
      }
    else
	gaiaAddPointToGeomColl (geom, x, y);
    return offset;
}

static int
ewkbGetLinestring (gaiaGeomCollPtr geom, unsigned char *blob,
		   int offset, int blob_size, int endian,
		   int endian_arch, int dims)
{
/* decodes a LINESTRING from PostGIS binary GEOMETRY */
    int npoints;
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaLinestringPtr ln;
    if (blob_size < offset + 4)
	return -1;
    npoints = gaiaImport32 (blob + offset, endian, endian_arch);
    offset += 4;
    switch (dims)
      {
      case GAIA_XY_Z_M:
	  if (blob_size < offset + (32 * npoints))
	      return -1;
	  break;
      case GAIA_XY_Z:
      case GAIA_XY_M:
	  if (blob_size < offset + (24 * npoints))
	      return -1;
	  break;
      default:
	  if (blob_size < offset + (16 * npoints))
	      return -1;
	  break;
      }
    ln = gaiaAddLinestringToGeomColl (geom, npoints);
    for (iv = 0; iv < npoints; iv++)
      {
	  x = gaiaImport64 (blob + offset, endian, endian_arch);
	  offset += 8;
	  y = gaiaImport64 (blob + offset, endian, endian_arch);
	  offset += 8;
	  if (dims == GAIA_XY_Z_M)
	    {
		z = gaiaImport64 (blob + offset, endian, endian_arch);
		offset += 8;
		m = gaiaImport64 (blob + offset, endian, endian_arch);
		offset += 8;
		gaiaSetPointXYZM (ln->Coords, iv, x, y, z, m);
	    }
	  else if (dims == GAIA_XY_Z)
	    {
		z = gaiaImport64 (blob + offset, endian, endian_arch);
		offset += 8;
		gaiaSetPointXYZ (ln->Coords, iv, x, y, z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		m = gaiaImport64 (blob + offset, endian, endian_arch);
		offset += 8;
		gaiaSetPointXYM (ln->Coords, iv, x, y, m);
	    }
	  else
	      gaiaSetPoint (ln->Coords, iv, x, y);
      }
    return offset;
}

static int
ewkbGetPolygon (gaiaGeomCollPtr geom, unsigned char *blob,
		int offset, int blob_size, int endian,
		int endian_arch, int dims)
{
/* decodes a POLYGON from PostGIS binary GEOMETRY */
    int rings;
    int npoints;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    gaiaPolygonPtr polyg = NULL;
    gaiaRingPtr rng;
    if (blob_size < offset + 4)
	return -1;
    rings = gaiaImport32 (blob + offset, endian, endian_arch);
    offset += 4;
    for (ib = 0; ib < rings; ib++)
      {
	  if (blob_size < offset + 4)
	      return -1;
	  npoints = gaiaImport32 (blob + offset, endian, endian_arch);
	  offset += 4;
	  switch (dims)
	    {
	    case GAIA_XY_Z_M:
		if (blob_size < offset + (32 * npoints))
		    return -1;
		break;
	    case GAIA_XY_Z:
	    case GAIA_XY_M:
		if (blob_size < offset + (24 * npoints))
		    return -1;
		break;
	    default:
		if (blob_size < offset + (16 * npoints))
		    return -1;
		break;
	    }
	  if (ib == 0)
	    {
		polyg = gaiaAddPolygonToGeomColl (geom, npoints, rings - 1);
		rng = polyg->Exterior;
	    }
	  else
	      rng = gaiaAddInteriorRing (polyg, ib - 1, npoints);
	  for (iv = 0; iv < npoints; iv++)
	    {
		x = gaiaImport64 (blob + offset, endian, endian_arch);
		offset += 8;
		y = gaiaImport64 (blob + offset, endian, endian_arch);
		offset += 8;
		if (dims == GAIA_XY_Z_M)
		  {
		      z = gaiaImport64 (blob + offset, endian, endian_arch);
		      offset += 8;
		      m = gaiaImport64 (blob + offset, endian, endian_arch);
		      offset += 8;
		      gaiaSetPointXYZM (rng->Coords, iv, x, y, z, m);
		  }
		else if (dims == GAIA_XY_Z)
		  {
		      z = gaiaImport64 (blob + offset, endian, endian_arch);
		      offset += 8;
		      gaiaSetPointXYZ (rng->Coords, iv, x, y, z);
		  }
		else if (dims == GAIA_XY_M)
		  {
		      m = gaiaImport64 (blob + offset, endian, endian_arch);
		      offset += 8;
		      gaiaSetPointXYM (rng->Coords, iv, x, y, m);
		  }
		else
		    gaiaSetPoint (rng->Coords, iv, x, y);
	    }
      }
    return offset;
}

static int
ewkbGetMultiGeometry (gaiaGeomCollPtr geom, unsigned char *blob,
		      int offset, int blob_size, int endian,
		      int endian_arch, int dims)
{
/* decodes a MultiGeometry from PostGIS EWKB binary GEOMETRY */
    int entities;
    int type;
    unsigned char xtype[4];
    int ie;
    int off;
    if (blob_size < offset + 4)
	return -1;
    entities = gaiaImport32 (blob + offset, endian, endian_arch);
    offset += 4;
    for (ie = 0; ie < entities; ie++)
      {
	  if (blob_size < offset + 5)
	      return -1;
	  memcpy (xtype, blob + offset + 1, 4);
	  if (endian)
	      xtype[3] = 0x00;
	  else
	      xtype[0] = 0x00;
	  type = gaiaImport32 (xtype, endian, endian_arch);
	  offset += 5;
	  switch (type)
	    {
	    case GAIA_POINT:
		off =
		    ewkbGetPoint (geom, blob, offset, blob_size, endian,
				  endian_arch, dims);
		if (off < 0)
		    return -1;
		offset += off;
		break;
	    case GAIA_LINESTRING:
		off =
		    ewkbGetLinestring (geom, blob, offset, blob_size, endian,
				       endian_arch, dims);
		if (off < 0)
		    return -1;
		offset += off;
		break;
	    case GAIA_POLYGON:
		off =
		    ewkbGetPolygon (geom, blob, offset, blob_size, endian,
				    endian_arch, dims);
		if (off < 0)
		    return -1;
		offset += off;
	    };
      }
    return offset;
}

static int
parseHexEwkbByte (const unsigned char high, const unsigned char low,
		  unsigned char *byte)
{
/* parsing an Hexadecimal byte */
    unsigned char hex;
    switch (high)
      {
      case '0':
	  hex = 16 * 0;
	  break;
      case '1':
	  hex = 16 * 1;
	  break;
      case '2':
	  hex = 16 * 2;
	  break;
      case '3':
	  hex = 16 * 3;
	  break;
      case '4':
	  hex = 16 * 4;
	  break;
      case '5':
	  hex = 16 * 5;
	  break;
      case '6':
	  hex = 16 * 6;
	  break;
      case '7':
	  hex = 16 * 7;
	  break;
      case '8':
	  hex = 16 * 8;
	  break;
      case '9':
	  hex = 16 * 9;
	  break;
      case 'A':
      case 'a':
	  hex = 16 * 10;
	  break;
      case 'B':
      case 'b':
	  hex = 16 * 11;
	  break;
      case 'C':
      case 'c':
	  hex = 16 * 12;
	  break;
      case 'D':
      case 'd':
	  hex = 16 * 13;
	  break;
      case 'E':
      case 'e':
	  hex = 16 * 14;
	  break;
      case 'F':
      case 'f':
	  hex = 16 * 15;
	  break;
      default:
	  return 0;
      };
    switch (low)
      {
      case '0':
	  hex += 0;
	  break;
      case '1':
	  hex += 1;
	  break;
      case '2':
	  hex += 2;
	  break;
      case '3':
	  hex += 3;
	  break;
      case '4':
	  hex += 4;
	  break;
      case '5':
	  hex += 5;
	  break;
      case '6':
	  hex += 6;
	  break;
      case '7':
	  hex += 7;
	  break;
      case '8':
	  hex += 8;
	  break;
      case '9':
	  hex += 9;
	  break;
      case 'A':
      case 'a':
	  hex += 10;
	  break;
      case 'B':
      case 'b':
	  hex += 11;
	  break;
      case 'C':
      case 'c':
	  hex += 12;
	  break;
      case 'D':
      case 'd':
	  hex += 13;
	  break;
      case 'E':
      case 'e':
	  hex += 14;
	  break;
      case 'F':
      case 'f':
	  hex += 15;
	  break;
      default:
	  return 0;
      };
    *byte = hex;
    return 1;
}

static unsigned char *
parseHexEWKB (const unsigned char *blob_hex, int *blob_size)
{
/* parsing an Hexadecimal EWKB Geometry */
    unsigned char *blob;
    unsigned char *p_out;
    const unsigned char *p_in;
    char high;
    char low;
    unsigned char hex;
    int size;
    int len = strlen ((const char *) blob_hex);
    size = len / 2;
    if (size * 2 != len)
	return NULL;
    blob = malloc (size);
    if (!blob)
	return NULL;
    *blob_size = size;
    p_in = blob_hex;
    p_out = blob;
    while (*p_in != '\0')
      {
	  high = *p_in++;
	  low = *p_in++;
	  if (!parseHexEwkbByte (high, low, &hex))
	    {
		free (blob);
		return NULL;
	    }
	  *p_out++ = hex;
      }
    *blob_size = size;
    return blob;
}

gaiaGeomCollPtr
gaiaFromEWKB (const unsigned char *in_buffer)
{
/* creates a Gaia own Geometry from GEOS/PostGIS EWKB */
    unsigned char *blob;
    int blob_size;
    unsigned char xtype[4];
    unsigned char xdims;
    int type;
    int has_z = 0;
    int has_m = 0;
    int dims = GAIA_XY;
    int srid;
    int endian;
    int endian_arch = gaiaEndianArch ();
    gaiaGeomCollPtr geom = NULL;
    blob = parseHexEWKB (in_buffer, &blob_size);
    if (!blob)
	return NULL;
    if (blob_size < 9)
	return NULL;
    if (*(blob + 0) == 0x01)
	endian = 1;
    else
	endian = 0;
    memcpy (xtype, blob + 1, 4);
    if (endian)
      {
	  xdims = xtype[3];
	  xtype[3] = 0x00;
      }
    else
      {
	  xdims = xtype[0];
	  xtype[0] = 0x00;
      }
    type = gaiaImport32 (xtype, endian, endian_arch);
    if (xdims & 0x40)
	has_m = 1;
    if (xdims & 0x80)
	has_z = 1;
    if (has_m && has_z)
      {
	  dims = GAIA_XY_Z_M;
	  geom = gaiaAllocGeomCollXYZM ();
      }
    else if (has_m)
      {
	  dims = GAIA_XY_M;
	  geom = gaiaAllocGeomCollXYM ();
      }
    else if (has_z)
      {
	  dims = GAIA_XY_Z;
	  geom = gaiaAllocGeomCollXYZ ();
      }
    else
      {
	  dims = GAIA_XY;
	  geom = gaiaAllocGeomColl ();
      }
    srid = gaiaImport32 (blob + 5, endian, endian_arch);
    geom->Srid = srid;
    if (geom->Srid <= 0)
	geom->Srid = -1;
    switch (type)
      {
      case GAIA_POINT:
	  ewkbGetPoint (geom, blob, 9, blob_size, endian, endian_arch, dims);
	  break;
      case GAIA_LINESTRING:
	  ewkbGetLinestring (geom, blob, 9, blob_size, endian, endian_arch,
			     dims);
	  break;
      case GAIA_POLYGON:
	  ewkbGetPolygon (geom, blob, 9, blob_size, endian, endian_arch, dims);
	  break;
      default:
	  ewkbGetMultiGeometry (geom, blob, 9, blob_size, endian, endian_arch,
				dims);
	  break;
      };

    return geom;
}

GAIAGEO_DECLARE void
gaiaToEWKB (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom)
{
/* prints the GEOS/PostGIS EWKB text representation of current geometry */
    char buf[2048];
    unsigned char endian_buf[16];
    char byte[3];
    char *ptr;
    int size;
    int type;
    int entities = 0;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    int i;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    int endian_arch = gaiaEndianArch ();
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;

/* precomputing the required size */
    sprintf (buf, "SRID=%d;", geom->Srid);
    size = strlen (buf);	/* the header size */
    size++;			/* terminating '\0' */
    pt = geom->FirstPoint;
    while (pt)
      {
	  point = pt;
	  entities++;
	  n_points++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  line = ln;
	  entities++;
	  n_linestrings++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  polyg = pg;
	  entities++;
	  n_polygons++;
	  pg = pg->Next;
      }
    if (n_points == 0 && n_polygons == 0 && n_linestrings == 0)
	return;
/* ok, we can determine the geometry class */
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	      type = GAIA_MULTIPOINT;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_POINT;
      }
    else if (n_points > 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTIPOINT;
      }
    else if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	      type = GAIA_MULTILINESTRING;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_LINESTRING;
      }
    else if (n_points == 0 && n_linestrings > 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTILINESTRING;
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	      type = GAIA_MULTIPOLYGON;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_POLYGON;
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons > 1)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTIPOLYGON;
      }
    else
	type = GAIA_GEOMETRYCOLLECTION;
/* and now we compute the size of EWKB */
    size += 10;			/* header size */
    if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	size += 8;
    point = geom->FirstPoint;
    while (point)
      {
	  if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	      || type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	      size += 10;
	  if (geom->DimensionModel == GAIA_XY_Z
	      || geom->DimensionModel == GAIA_XY_M)
	      size += 48;	/* three doubles for each POINT */
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      size += 64;	/* four doubles for each POINT */
	  else
	      size += 32;	/* two doubles for each POINT */
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	      || type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	      size += 10;
	  if (geom->DimensionModel == GAIA_XY_Z
	      || geom->DimensionModel == GAIA_XY_M)
	      size += 8 + (line->Points * 48);	/* three doubles for each VERTEX */
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      size += 8 + (line->Points * 64);	/* four doubles for each VERTEX */
	  else
	      size += 8 + (line->Points * 32);	/* two doubles for each VERTEX */
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	      || type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	      size += 10;
	  rng = polyg->Exterior;
	  if (geom->DimensionModel == GAIA_XY_Z
	      || geom->DimensionModel == GAIA_XY_M)
	      size += 16 + (rng->Points * 48);	/* three doubles for each VERTEX */
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	      size += 16 + (rng->Points * 64);	/* four doubles for each VERTEX */
	  else
	      size += 16 + (rng->Points * 32);	/* two doubles for each VERTEX */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		if (geom->DimensionModel == GAIA_XY_Z
		    || geom->DimensionModel == GAIA_XY_M)
		    size += 8 + (rng->Points * 48);	/* three doubles for each VERTEX */
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    size += 8 + (rng->Points * 64);	/* four doubles for each VERTEX */
		else
		    size += 8 + (rng->Points * 32);	/* two doubles for each VERTEX */
	    }
	  polyg = polyg->Next;
      }
/* and finally we build the EWKB expression */
    sprintf (buf, "SRID=%d;", geom->Srid);
    gaiaAppendToOutBuffer (out_buf, buf);
    ptr = buf;

    *ptr++ = '0';		/* little endian byte order */
    *ptr++ = '1';
    gaiaExport32 (endian_buf, type, 1, endian_arch);	/* the main CLASS TYPE */
    for (i = 0; i < 3; i++)
      {
	  /* CAVEAT: the 4th byte in PostGIS encodes M/Z presence !!!! */
	  sprintf (byte, "%02X", endian_buf[i]);
	  *ptr++ = byte[0];
	  *ptr++ = byte[1];
      }
/* marking M/Z presence */
    if (geom->DimensionModel == GAIA_XY_Z)
	*ptr++ = '8';
    else if (geom->DimensionModel == GAIA_XY_M)
	*ptr++ = '4';
    else if (geom->DimensionModel == GAIA_XY_Z_M)
	*ptr++ = 'C';
    else
	*ptr++ = '0';
    *ptr++ = '0';
    *ptr++ = '\0';
    gaiaAppendToOutBuffer (out_buf, buf);
    ptr = buf;
    if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
      {
	  gaiaExport32 (endian_buf, entities, 1, endian_arch);	/* it's a collection; # entities */
	  for (i = 0; i < 4; i++)
	    {
		sprintf (byte, "%02X", endian_buf[i]);
		*ptr++ = byte[0];
		*ptr++ = byte[1];
	    }
      }
    point = geom->FirstPoint;
    while (point)
      {
	  if ((ptr - buf) > 1024)
	    {
		/* flushing the internal buffer */
		*ptr++ = '\0';
		gaiaAppendToOutBuffer (out_buf, buf);
		ptr = buf;
	    }
	  if (type == GAIA_MULTIPOINT || type == GAIA_GEOMETRYCOLLECTION)
	    {
		*ptr++ = '0';
		*ptr++ = '1';
		/* it's a collection: the CLASS TYPE for this element */
		gaiaExport32 (endian_buf, GAIA_POINT, 1, endian_arch);
		for (i = 0; i < 3; i++)
		  {
		      /* CAVEAT: the 4th byte in PostGIS encodes M/Z presence !!!! */
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		/* marking M/Z presence */
		if (geom->DimensionModel == GAIA_XY_Z)
		    *ptr++ = '8';
		else if (geom->DimensionModel == GAIA_XY_M)
		    *ptr++ = '4';
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *ptr++ = 'C';
		else
		    *ptr++ = '0';
		*ptr++ = '0';
	    }
	  gaiaExport64 (endian_buf, point->X, 1, endian_arch);	/* X */
	  for (i = 0; i < 8; i++)
	    {
		sprintf (byte, "%02X", endian_buf[i]);
		*ptr++ = byte[0];
		*ptr++ = byte[1];
	    }
	  gaiaExport64 (endian_buf, point->Y, 1, endian_arch);	/* Y */
	  for (i = 0; i < 8; i++)
	    {
		sprintf (byte, "%02X", endian_buf[i]);
		*ptr++ = byte[0];
		*ptr++ = byte[1];
	    }
	  if (geom->DimensionModel == GAIA_XY_Z)
	    {
		gaiaExport64 (endian_buf, point->Z, 1, endian_arch);	/* Z */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
	    }
	  else if (geom->DimensionModel == GAIA_XY_M)
	    {
		gaiaExport64 (endian_buf, point->M, 1, endian_arch);	/* M */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
	    }
	  else if (geom->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaExport64 (endian_buf, point->Z, 1, endian_arch);	/* Z */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		gaiaExport64 (endian_buf, point->M, 1, endian_arch);	/* M */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
	    }
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  if ((ptr - buf) > 1024)
	    {
		/* flushing the internal buffer */
		*ptr++ = '\0';
		gaiaAppendToOutBuffer (out_buf, buf);
		ptr = buf;
	    }
	  if (type == GAIA_MULTILINESTRING || type == GAIA_GEOMETRYCOLLECTION)
	    {
		*ptr++ = '0';
		*ptr++ = '1';
		/* it's a collection: the CLASS TYPE for this element */
		gaiaExport32 (endian_buf, GAIA_LINESTRING, 1, endian_arch);
		for (i = 0; i < 3; i++)
		  {
		      /* CAVEAT: the 4th byte in PostGIS encodes M/Z presence !!!! */
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		/* marking M/Z presence */
		if (geom->DimensionModel == GAIA_XY_Z)
		    *ptr++ = '8';
		else if (geom->DimensionModel == GAIA_XY_M)
		    *ptr++ = '4';
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *ptr++ = 'C';
		else
		    *ptr++ = '0';
		*ptr++ = '0';
	    }
	  gaiaExport32 (endian_buf, line->Points, 1, endian_arch);	/* # points */
	  for (i = 0; i < 4; i++)
	    {
		sprintf (byte, "%02X", endian_buf[i]);
		*ptr++ = byte[0];
		*ptr++ = byte[1];
	    }
	  for (iv = 0; iv < line->Points; iv++)
	    {
		if ((ptr - buf) > 1024)
		  {
		      /* flushing the internal buffer */
		      *ptr++ = '\0';
		      gaiaAppendToOutBuffer (out_buf, buf);
		      ptr = buf;
		  }
		gaiaLineGetPoint (line, iv, &x, &y, &z, &m);
		gaiaExport64 (endian_buf, x, 1, endian_arch);	/* X */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		gaiaExport64 (endian_buf, y, 1, endian_arch);	/* Y */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		if (geom->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaExport64 (endian_buf, z, 1, endian_arch);	/* Z */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		  }
		else if (geom->DimensionModel == GAIA_XY_M)
		  {
		      gaiaExport64 (endian_buf, m, 1, endian_arch);	/* M */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		  }
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaExport64 (endian_buf, z, 1, endian_arch);	/* Z */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		      gaiaExport64 (endian_buf, m, 1, endian_arch);	/* M */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  if ((ptr - buf) > 1024)
	    {
		/* flushing the internal buffer */
		*ptr++ = '\0';
		gaiaAppendToOutBuffer (out_buf, buf);
		ptr = buf;
	    }
	  if (type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	    {
		*ptr++ = '0';
		*ptr++ = '1';
		/* it's a collection: the CLASS TYPE for this element */
		gaiaExport32 (endian_buf, GAIA_POLYGON, 1, endian_arch);
		for (i = 0; i < 3; i++)
		  {
		      /* CAVEAT: the 4th byte in PostGIS encodes M/Z presence !!!! */
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		/* marking M/Z presence */
		if (geom->DimensionModel == GAIA_XY_Z)
		    *ptr++ = '8';
		else if (geom->DimensionModel == GAIA_XY_M)
		    *ptr++ = '4';
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    *ptr++ = 'C';
		else
		    *ptr++ = '0';
		*ptr++ = '0';
	    }
	  gaiaExport32 (endian_buf, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  for (i = 0; i < 4; i++)
	    {
		sprintf (byte, "%02X", endian_buf[i]);
		*ptr++ = byte[0];
		*ptr++ = byte[1];
	    }
	  rng = polyg->Exterior;
	  gaiaExport32 (endian_buf, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  for (i = 0; i < 4; i++)
	    {
		sprintf (byte, "%02X", endian_buf[i]);
		*ptr++ = byte[0];
		*ptr++ = byte[1];
	    }
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		if ((ptr - buf) > 1024)
		  {
		      /* flushing the internal buffer */
		      *ptr++ = '\0';
		      gaiaAppendToOutBuffer (out_buf, buf);
		      ptr = buf;
		  }
		gaiaRingGetPoint (rng, iv, &x, &y, &z, &m);
		gaiaExport64 (endian_buf, x, 1, endian_arch);	/* X - exterior ring */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		gaiaExport64 (endian_buf, y, 1, endian_arch);	/* Y - exterior ring */
		for (i = 0; i < 8; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		if (geom->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaExport64 (endian_buf, z, 1, endian_arch);	/* Z */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		  }
		else if (geom->DimensionModel == GAIA_XY_M)
		  {
		      gaiaExport64 (endian_buf, m, 1, endian_arch);	/* M */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		  }
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaExport64 (endian_buf, z, 1, endian_arch);	/* Z */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		      gaiaExport64 (endian_buf, m, 1, endian_arch);	/* M */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (endian_buf, rng->Points, 1, endian_arch);	/* # points - interior ring */
		for (i = 0; i < 4; i++)
		  {
		      sprintf (byte, "%02X", endian_buf[i]);
		      *ptr++ = byte[0];
		      *ptr++ = byte[1];
		  }
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      if ((ptr - buf) > 1024)
			{
			    /* flushing the internal buffer */
			    *ptr++ = '\0';
			    gaiaAppendToOutBuffer (out_buf, buf);
			    ptr = buf;
			}
		      gaiaRingGetPoint (rng, iv, &x, &y, &z, &m);
		      gaiaExport64 (endian_buf, x, 1, endian_arch);	/* X - interior ring */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		      gaiaExport64 (endian_buf, y, 1, endian_arch);	/* Y - interior ring */
		      for (i = 0; i < 8; i++)
			{
			    sprintf (byte, "%02X", endian_buf[i]);
			    *ptr++ = byte[0];
			    *ptr++ = byte[1];
			}
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaExport64 (endian_buf, z, 1, endian_arch);	/* Z */
			    for (i = 0; i < 8; i++)
			      {
				  sprintf (byte, "%02X", endian_buf[i]);
				  *ptr++ = byte[0];
				  *ptr++ = byte[1];
			      }
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaExport64 (endian_buf, m, 1, endian_arch);	/* M */
			    for (i = 0; i < 8; i++)
			      {
				  sprintf (byte, "%02X", endian_buf[i]);
				  *ptr++ = byte[0];
				  *ptr++ = byte[1];
			      }
			}
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaExport64 (endian_buf, z, 1, endian_arch);	/* Z */
			    for (i = 0; i < 8; i++)
			      {
				  sprintf (byte, "%02X", endian_buf[i]);
				  *ptr++ = byte[0];
				  *ptr++ = byte[1];
			      }
			    gaiaExport64 (endian_buf, m, 1, endian_arch);	/* M */
			    for (i = 0; i < 8; i++)
			      {
				  sprintf (byte, "%02X", endian_buf[i]);
				  *ptr++ = byte[0];
				  *ptr++ = byte[1];
			      }
			}
		  }
	    }
	  polyg = polyg->Next;
      }

/* terminating the EWKB string */
    *ptr = '\0';
    gaiaAppendToOutBuffer (out_buf, buf);
}

static int
coordDimsFromFgf (int endian_arch, const unsigned char *blob, unsigned int size,
		  int *type)
{
/* decoding the coordinate Dimensions for an FGF Geometry */
    int coord_dims;
    if (size < 4)
	return 0;
    coord_dims = gaiaImport32 (blob, GAIA_LITTLE_ENDIAN, endian_arch);
    *type = coord_dims;
    switch (coord_dims)
      {
      case GAIA_XY:
	  return 2;
      case GAIA_XY_M:
      case GAIA_XY_Z:
	  return 3;
      case GAIA_XY_Z_M:
	  return 4;
      default:
	  return 0;
      }
}

static int
pointFromFgf (gaiaGeomCollPtr geom, int endian_arch, const unsigned char *blob,
	      unsigned int size, unsigned int *consumed)
{
/* decoding a POINT Geometry from FGF  */
    double x;
    double y;
    double z;
    double m;
    unsigned int sz = size;
    const unsigned char *ptr = blob;
    int coord_dims;
    int type;
/* checking Geometry Type */
    if (sz < 4)
	return 0;
    if (gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch) != GAIA_POINT)
	return 0;
    ptr += 4;
    sz -= 4;
/* checking size */
    if (sz < 4)
	return 0;
    coord_dims = coordDimsFromFgf (endian_arch, ptr, size, &type);
    if (!coord_dims)
	return 0;
    ptr += 4;
    sz -= 4;
    if (sz < (coord_dims * sizeof (double)))
	return 0;
    if (consumed)
	*consumed = coord_dims * sizeof (double);
    if (type == GAIA_XY_Z)
      {
	  /* building the POINTZ */
	  x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
	  y = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  z = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  gaiaAddPointToGeomCollXYZ (geom, x, y, z);
      }
    else if (type == GAIA_XY_M)
      {
	  /* building the POINTM */
	  x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
	  y = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  m = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  gaiaAddPointToGeomCollXYM (geom, x, y, m);
      }
    else if (type == GAIA_XY_Z_M)
      {
	  /* building the POINTZM */
	  x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
	  y = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  z = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  m = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  gaiaAddPointToGeomCollXYZM (geom, x, y, z, m);
      }
    else
      {
	  /* building the POINT */
	  x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
	  y = gaiaImport64 (ptr + 8, GAIA_LITTLE_ENDIAN, endian_arch);
	  gaiaAddPointToGeomColl (geom, x, y);
      }
    return 1;
}

static int
linestringFromFgf (gaiaGeomCollPtr geom, int endian_arch,
		   const unsigned char *blob, unsigned int size,
		   unsigned int *consumed)
{
/* decoding a LINESTRING Geometry from FGF  */
    gaiaLinestringPtr ln;
    int pts;
    int iv;
    double x;
    double y;
    unsigned int ln_sz;
    unsigned int sz = size;
    const unsigned char *ptr = blob;
    int coord_dims;
    int type;
/* checking Geometry Type */
    if (sz < 4)
	return 0;
    if (gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch) != GAIA_LINESTRING)
	return 0;
    ptr += 4;
    sz -= 4;
/* checking size */
    coord_dims = coordDimsFromFgf (endian_arch, ptr, size, &type);
    if (!coord_dims)
	return 0;
    ptr += 4;
    sz -= 4;
/* how many points are there ? */
    if (sz < 4)
	return 0;
    pts = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
    ptr += 4;
    sz -= 4;
    if (pts < 2)
	return 0;
    ln_sz = pts * coord_dims * sizeof (double);
    if (sz < ln_sz)
	return 0;
    if (consumed)
	*consumed = (12 + ln_sz);
    if (type == GAIA_XY_Z)
      {
	  /* building the LINESTRINGZ */
	  geom->DimensionModel = GAIA_XY_Z;
	  ln = gaiaAddLinestringToGeomColl (geom, pts);
	  for (iv = 0; iv < pts; iv++)
	    {
		/* inserting vertices into the linestring */
		x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
		y = gaiaImport64 (ptr + sizeof (double), GAIA_LITTLE_ENDIAN,
				  endian_arch);
		ptr += (coord_dims * sizeof (double));
		gaiaSetPoint (ln->Coords, iv, x, y);
	    }
      }
    else if (type == GAIA_XY_M)
      {
	  /* building the LINESTRINGM */
	  geom->DimensionModel = GAIA_XY_M;
	  ln = gaiaAddLinestringToGeomColl (geom, pts);
	  for (iv = 0; iv < pts; iv++)
	    {
		/* inserting vertices into the linestring */
		x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
		y = gaiaImport64 (ptr + sizeof (double), GAIA_LITTLE_ENDIAN,
				  endian_arch);
		ptr += (coord_dims * sizeof (double));
		gaiaSetPoint (ln->Coords, iv, x, y);
	    }
      }
    else if (type == GAIA_XY_Z_M)
      {
	  /* building the LINESTRINGZM */
	  geom->DimensionModel = GAIA_XY_Z_M;
	  ln = gaiaAddLinestringToGeomColl (geom, pts);
	  for (iv = 0; iv < pts; iv++)
	    {
		/* inserting vertices into the linestring */
		x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
		y = gaiaImport64 (ptr + sizeof (double), GAIA_LITTLE_ENDIAN,
				  endian_arch);
		ptr += (coord_dims * sizeof (double));
		gaiaSetPoint (ln->Coords, iv, x, y);
	    }
      }
    else
      {
	  /* building the LINESTRING */
	  geom->DimensionModel = GAIA_XY;
	  ln = gaiaAddLinestringToGeomColl (geom, pts);
	  for (iv = 0; iv < pts; iv++)
	    {
		/* inserting vertices into the linestring */
		x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
		y = gaiaImport64 (ptr + sizeof (double), GAIA_LITTLE_ENDIAN,
				  endian_arch);
		ptr += (coord_dims * sizeof (double));
		gaiaSetPoint (ln->Coords, iv, x, y);
	    }
      }
    return 1;
}

static int
polygonFromFgf (gaiaGeomCollPtr geom, int endian_arch,
		const unsigned char *blob, unsigned int size,
		unsigned int *consumed)
{
/* decoding a POLYGON Geometry from FGF  */
    gaiaPolygonPtr pg = NULL;
    gaiaRingPtr rng;
    int rings;
    int ir;
    int pts;
    int iv;
    double x;
    double y;
    double z;
    double m;
    unsigned int rng_sz;
    unsigned int sz = size;
    const unsigned char *ptr = blob;
    int coord_dims;
    int type;
    unsigned int bytes = 0;
/* checking Geometry Type */
    if (sz < 4)
	return 0;
    if (gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch) != GAIA_POLYGON)
	return 0;
    ptr += 4;
    sz -= 4;
    bytes += 4;
/* checking size */
    coord_dims = coordDimsFromFgf (endian_arch, ptr, size, &type);
    if (!coord_dims)
	return 0;
    ptr += 4;
    sz -= 4;
    bytes += 4;
/* how many rings are there ? */
    if (sz < 4)
	return 0;
    rings = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
    ptr += 4;
    sz -= 4;
    bytes += 4;
    if (rings < 1)
	return 0;
    for (ir = 0; ir < rings; ir++)
      {
	  /* fetching Polygon's rings */
	  if (sz < 4)
	      return 0;
	  pts = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
	  ptr += 4;
	  sz -= 4;
	  bytes += 4;
	  if (pts < 4)
	      return 0;
	  rng_sz = pts * coord_dims * sizeof (double);
	  if (sz < rng_sz)
	      return 0;
	  bytes += rng_sz;
	  if (type == GAIA_XY_Z)
	    {
		/* POLYGONZ */
		geom->DimensionModel = GAIA_XY_Z;
		if (ir == 0)
		  {
		      /* building the EXTERIOR RING */
		      pg = gaiaAddPolygonToGeomColl (geom, pts, rings - 1);
		      rng = pg->Exterior;
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into the EXTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    z = gaiaImport64 (ptr + (sizeof (double) * 2),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPointXYZ (rng->Coords, iv, x, y, z);
			}
		  }
		else
		  {
		      /* building an INTERIOR RING */
		      rng = gaiaAddInteriorRing (pg, ir - 1, pts);
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into some INTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    z = gaiaImport64 (ptr + (sizeof (double) * 2),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPointXYZ (rng->Coords, iv, x, y, z);
			}
		  }
	    }
	  if (type == GAIA_XY_M)
	    {
		/* POLYGONM */
		geom->DimensionModel = GAIA_XY_M;
		if (ir == 0)
		  {
		      /* building the EXTERIOR RING */
		      pg = gaiaAddPolygonToGeomColl (geom, pts, rings - 1);
		      rng = pg->Exterior;
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into the EXTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    m = gaiaImport64 (ptr + (sizeof (double) * 2),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPointXYM (rng->Coords, iv, x, y, m);
			}
		  }
		else
		  {
		      /* building an INTERIOR RING */
		      rng = gaiaAddInteriorRing (pg, ir - 1, pts);
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into some INTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    m = gaiaImport64 (ptr + (sizeof (double) * 2),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPointXYM (rng->Coords, iv, x, y, m);
			}
		  }
	    }
	  else if (type == GAIA_XY_Z_M)
	    {
		/* POLYGONZM */
		geom->DimensionModel = GAIA_XY_Z_M;
		if (ir == 0)
		  {
		      /* building the EXTERIOR RING */
		      pg = gaiaAddPolygonToGeomColl (geom, pts, rings - 1);
		      rng = pg->Exterior;
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into the EXTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    z = gaiaImport64 (ptr + (sizeof (double) * 2),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    m = gaiaImport64 (ptr + (sizeof (double) * 3),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPointXYZM (rng->Coords, iv, x, y, z, m);
			}
		  }
		else
		  {
		      /* building an INTERIOR RING */
		      rng = gaiaAddInteriorRing (pg, ir - 1, pts);
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into some INTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    z = gaiaImport64 (ptr + (sizeof (double) * 2),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    m = gaiaImport64 (ptr + (sizeof (double) * 3),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPointXYZM (rng->Coords, iv, x, y, z, m);
			}
		  }
	    }
	  else
	    {
		/* POLYGON */
		geom->DimensionModel = GAIA_XY;
		if (ir == 0)
		  {
		      /* building the EXTERIOR RING */
		      pg = gaiaAddPolygonToGeomColl (geom, pts, rings - 1);
		      rng = pg->Exterior;
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into the EXTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPoint (rng->Coords, iv, x, y);
			}
		  }
		else
		  {
		      /* building an INTERIOR RING */
		      rng = gaiaAddInteriorRing (pg, ir - 1, pts);
		      for (iv = 0; iv < pts; iv++)
			{
			    /* inserting vertices into some INTERIOR Ring */
			    x = gaiaImport64 (ptr, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    y = gaiaImport64 (ptr + sizeof (double),
					      GAIA_LITTLE_ENDIAN, endian_arch);
			    ptr += (coord_dims * sizeof (double));
			    gaiaSetPoint (rng->Coords, iv, x, y);
			}
		  }
	    }
	  sz -= rng_sz;
      }
    if (consumed)
	*consumed = bytes;
    return 1;
}

static int
multiPointFromFgf (gaiaGeomCollPtr geom, int endian_arch,
		   const unsigned char *blob, unsigned int size)
{
/* decoding a MULTIPOINT Geometry from FGF  */
    int pts;
    int ipt;
    unsigned int sz = size;
    const unsigned char *ptr = blob;
    unsigned int consumed;
/* checking Geometry Type */
    if (sz < 4)
	return 0;
    if (gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch) != GAIA_MULTIPOINT)
	return 0;
    ptr += 4;
    sz -= 4;
/* how many points are there ? */
    if (sz < 4)
	return 0;
    pts = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
    ptr += 4;
    sz -= 4;
    if (pts < 1)
	return 0;
    for (ipt = 0; ipt < pts; ipt++)
      {
	  /* fetching individual Points from FGF */
	  if (!pointFromFgf (geom, endian_arch, ptr, sz, &consumed))
	      return 0;
	  ptr += consumed;
	  sz -= consumed;
      }
    return 1;
}

static int
multiLinestringFromFgf (gaiaGeomCollPtr geom, int endian_arch,
			const unsigned char *blob, unsigned int size)
{
/* decoding a MULTILINESTRING Geometry from FGF  */
    int lns;
    int iln;
    unsigned int sz = size;
    const unsigned char *ptr = blob;
    unsigned int consumed;
/* checking Geometry Type */
    if (sz < 4)
	return 0;
    if (gaiaImport32
	(ptr, GAIA_LITTLE_ENDIAN, endian_arch) != GAIA_MULTILINESTRING)
	return 0;
    ptr += 4;
    sz -= 4;
/* how many linestrings are there ? */
    if (sz < 4)
	return 0;
    lns = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
    ptr += 4;
    sz -= 4;
    if (lns < 1)
	return 0;
    for (iln = 0; iln < lns; iln++)
      {
	  /* fetching individual Linestrings from FGF */
	  if (!linestringFromFgf (geom, endian_arch, ptr, sz, &consumed))
	      return 0;
	  ptr += consumed;
	  sz -= consumed;
      }
    return 1;
}

static int
multiPolygonFromFgf (gaiaGeomCollPtr geom, int endian_arch,
		     const unsigned char *blob, unsigned int size)
{
/* decoding a MULTIPOLYGON Geometry from FGF  */
    int pgs;
    int ipg;
    unsigned int sz = size;
    const unsigned char *ptr = blob;
    unsigned int consumed;
/* checking Geometry Type */
    if (sz < 4)
	return 0;
    if (gaiaImport32
	(ptr, GAIA_LITTLE_ENDIAN, endian_arch) != GAIA_MULTIPOLYGON)
	return 0;
    ptr += 4;
    sz -= 4;
/* how many polygons are there ? */
    if (sz < 4)
	return 0;
    pgs = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
    ptr += 4;
    sz -= 4;
    if (pgs < 1)
	return 0;
    for (ipg = 0; ipg < pgs; ipg++)
      {
	  /* fetching individual Polygons from FGF */
	  if (!polygonFromFgf (geom, endian_arch, ptr, sz, &consumed))
	      return 0;
	  ptr += consumed;
	  sz -= consumed;
      }
    return 1;
}

static int
geomCollectionFromFgf (gaiaGeomCollPtr geom, int endian_arch,
		       const unsigned char *blob, unsigned int size)
{
/* decoding a  GEOMETRYCOLLECTION Geometry from FGF  */
    int geoms;
    int ig;
    int geom_type;
    unsigned int sz = size;
    const unsigned char *ptr = blob;
    unsigned int consumed;
/* checking Geometry Type */
    if (sz < 4)
	return 0;
    if (gaiaImport32
	(ptr, GAIA_LITTLE_ENDIAN, endian_arch) != GAIA_GEOMETRYCOLLECTION)
	return 0;
    ptr += 4;
    sz -= 4;
/* how many individual Geometries are there ? */
    if (sz < 4)
	return 0;
    geoms = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
    ptr += 4;
    sz -= 4;
    if (geoms < 1)
	return 0;
    for (ig = 0; ig < geoms; ig++)
      {
	  /* fetching individual Geometries from FGF */
	  if (sz < 4)
	      return 0;
	  geom_type = gaiaImport32 (ptr, GAIA_LITTLE_ENDIAN, endian_arch);
	  switch (geom_type)
	    {
	    case GAIA_POINT:
		if (!pointFromFgf (geom, endian_arch, ptr, sz, &consumed))
		    return 0;
		break;
	    case GAIA_LINESTRING:
		if (!linestringFromFgf (geom, endian_arch, ptr, sz, &consumed))
		    return 0;
		break;
	    case GAIA_POLYGON:
		if (!polygonFromFgf (geom, endian_arch, ptr, sz, &consumed))
		    return 0;
		break;
	    default:		/* unsupported geometry type */
		return 0;
		break;
	    };
	  ptr += consumed;
	  sz -= consumed;
      }
    return 1;
}

GAIAGEO_DECLARE gaiaGeomCollPtr
gaiaFromFgf (const unsigned char *blob, unsigned int size)
{
/* decoding from FGF to GEOMETRY  */
    gaiaGeomCollPtr geom = NULL;
    int geom_type;
    int endian_arch = gaiaEndianArch ();
    if (size < 4)
	return NULL;
/* checking FGF type */
    geom_type = gaiaImport32 (blob, GAIA_LITTLE_ENDIAN, endian_arch);
    geom = gaiaAllocGeomColl ();
    geom->DeclaredType = geom_type;
    switch (geom_type)
      {
      case GAIA_POINT:
	  if (pointFromFgf (geom, endian_arch, blob, size, NULL))
	      return geom;
	  break;
      case GAIA_LINESTRING:
	  if (linestringFromFgf (geom, endian_arch, blob, size, NULL))
	      return geom;
	  break;
      case GAIA_POLYGON:
	  if (polygonFromFgf (geom, endian_arch, blob, size, NULL))
	      return geom;
	  break;
      case GAIA_MULTIPOINT:
	  if (multiPointFromFgf (geom, endian_arch, blob, size))
	      return geom;
	  break;
      case GAIA_MULTILINESTRING:
	  if (multiLinestringFromFgf (geom, endian_arch, blob, size))
	      return geom;
	  break;
      case GAIA_MULTIPOLYGON:
	  if (multiPolygonFromFgf (geom, endian_arch, blob, size))
	      return geom;
	  break;
      case GAIA_GEOMETRYCOLLECTION:
	  if (geomCollectionFromFgf (geom, endian_arch, blob, size))
	      return geom;
	  break;
      default:			/* unsupported geometry type */
	  break;
      };
    gaiaFreeGeomColl (geom);
    return NULL;
}

GAIAGEO_DECLARE void
gaiaToFgf (gaiaGeomCollPtr geom, unsigned char **result, int *size,
	   int coord_dims)
{
/* builds the FGF representation for this GEOMETRY */
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    int entities = 0;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    int type;
    int n_coords;
    unsigned char *ptr;
    int sz = 0;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    int endian_arch = gaiaEndianArch ();
    gaiaMbrGeometry (geom);
    switch (coord_dims)
      {
      case GAIA_XY:
	  n_coords = 2;
	  break;
      case GAIA_XY_M:
      case GAIA_XY_Z:
	  n_coords = 3;
	  break;
      case GAIA_XY_Z_M:
	  n_coords = 4;
	  break;
      default:
	  n_coords = 0;
	  break;
      }
/* how many entities, and of what kind, do we have ? */
    pt = geom->FirstPoint;
    while (pt)
      {
	  point = pt;
	  entities++;
	  n_points++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  line = ln;
	  entities++;
	  n_linestrings++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  polyg = pg;
	  entities++;
	  n_polygons++;
	  pg = pg->Next;
      }
    *size = 0;
    sz = 0;
    *result = NULL;
    if (n_points == 0 && n_polygons == 0 && n_linestrings == 0)
	return;
/* ok, we can determine the geometry class */
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	      type = GAIA_MULTIPOINT;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_POINT;
      }
    else if (n_points > 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTIPOINT;
      }
    else if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	      type = GAIA_MULTILINESTRING;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_LINESTRING;
      }
    else if (n_points == 0 && n_linestrings > 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTILINESTRING;
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	      type = GAIA_MULTIPOLYGON;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_POLYGON;
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons > 1)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOMETRYCOLLECTION;
	  else
	      type = GAIA_MULTIPOLYGON;
      }
    else
	type = GAIA_GEOMETRYCOLLECTION;
/* and now we compute the size of FGF */
    if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	sz += 8;
    point = geom->FirstPoint;
    while (point)
      {
	  sz += (8 + (n_coords * sizeof (double)));	/* the size of each POINT */
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  sz += (12 + ((n_coords * sizeof (double)) * line->Points));	/* # points + [x,y] for each vertex */
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  rng = polyg->Exterior;
	  sz += (16 + ((n_coords * sizeof (double)) * rng->Points));	/* # rings + # points + [x.y] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		sz += (4 + ((n_coords * sizeof (double)) * rng->Points));	/* # points + [x,y] array - interior ring */
	    }
	  polyg = polyg->Next;
      }
    *size = sz;
    ptr = malloc (sz);
    *result = ptr;
/* and finally we build the FGF */
    if (type == GAIA_MULTIPOINT || type == GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
      {
	  gaiaExport32 (ptr, type, GAIA_LITTLE_ENDIAN, endian_arch);	/* Geometry Type */
	  ptr += 4;
	  gaiaExport32 (ptr, entities, GAIA_LITTLE_ENDIAN, endian_arch);	/* it's a collection; # entities */
	  ptr += 4;
      }
    point = geom->FirstPoint;
    while (point)
      {
	  gaiaExport32 (ptr, GAIA_POINT, GAIA_LITTLE_ENDIAN, endian_arch);	/* the CLASS TYPE for this element */
	  ptr += 4;
	  gaiaExport32 (ptr, coord_dims, GAIA_LITTLE_ENDIAN, endian_arch);	/* the CoordDimension */
	  ptr += 4;
	  gaiaExport64 (ptr, point->X, GAIA_LITTLE_ENDIAN, endian_arch);	/* X */
	  ptr += 8;
	  gaiaExport64 (ptr, point->Y, GAIA_LITTLE_ENDIAN, endian_arch);	/* Y */
	  ptr += 8;
	  if (n_coords > 2)
	    {
		/* the third coordinate [Z or M] */
		if (coord_dims == GAIA_XY_Z || coord_dims == GAIA_XY_Z_M)
		    gaiaExport64 (ptr, point->Z, GAIA_LITTLE_ENDIAN,
				  endian_arch);
		else
		    gaiaExport64 (ptr, point->M, GAIA_LITTLE_ENDIAN,
				  endian_arch);
		ptr += 8;
	    }
	  if (n_coords > 3)
	    {
		/* the fourth coordinate [M] */
		gaiaExport64 (ptr, point->M, GAIA_LITTLE_ENDIAN, endian_arch);
		ptr += 8;
	    }
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  gaiaExport32 (ptr, GAIA_LINESTRING, GAIA_LITTLE_ENDIAN, endian_arch);	/* the CLASS TYPE for this element */
	  ptr += 4;
	  gaiaExport32 (ptr, coord_dims, GAIA_LITTLE_ENDIAN, endian_arch);	/* the CoordDimension */
	  ptr += 4;
	  gaiaExport32 (ptr, line->Points, GAIA_LITTLE_ENDIAN, endian_arch);	/* # points */
	  ptr += 4;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		if (geom->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (geom->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		gaiaExport64 (ptr, x, GAIA_LITTLE_ENDIAN, endian_arch);	/* X */
		ptr += 8;
		gaiaExport64 (ptr, y, GAIA_LITTLE_ENDIAN, endian_arch);	/* Y */
		ptr += 8;
		if (n_coords > 2)
		  {
		      /* the third coordinate [Z or M] */
		      if (coord_dims == GAIA_XY_Z || coord_dims == GAIA_XY_Z_M)
			  gaiaExport64 (ptr, z, GAIA_LITTLE_ENDIAN,
					endian_arch);
		      else
			  gaiaExport64 (ptr, m, GAIA_LITTLE_ENDIAN,
					endian_arch);
		      ptr += 8;
		  }
		if (n_coords > 3)
		  {
		      /* the fourth coordinate [M]; */
		      gaiaExport64 (ptr, m, GAIA_LITTLE_ENDIAN, endian_arch);
		      ptr += 8;
		  }
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  gaiaExport32 (ptr, GAIA_POLYGON, GAIA_LITTLE_ENDIAN, endian_arch);	/* the CLASS TYPE for this element */
	  ptr += 4;
	  gaiaExport32 (ptr, coord_dims, GAIA_LITTLE_ENDIAN, endian_arch);	/* the CoordDimension */
	  ptr += 4;
	  gaiaExport32 (ptr, polyg->NumInteriors + 1, GAIA_LITTLE_ENDIAN, endian_arch);	/* # rings */
	  ptr += 4;
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr, rng->Points, GAIA_LITTLE_ENDIAN, endian_arch);	/* # points - exterior ring */
	  ptr += 4;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		if (geom->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		  }
		else if (geom->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		  }
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		  }
		gaiaExport64 (ptr, x, GAIA_LITTLE_ENDIAN, endian_arch);	/* X - exterior ring */
		ptr += 8;
		gaiaExport64 (ptr, y, GAIA_LITTLE_ENDIAN, endian_arch);	/* Y - exterior ring */
		ptr += 8;
		if (n_coords > 2)
		  {
		      /* the third coordinate [Z or M] */
		      if (coord_dims == GAIA_XY_Z || coord_dims == GAIA_XY_Z_M)
			  gaiaExport64 (ptr, z, GAIA_LITTLE_ENDIAN,
					endian_arch);
		      else
			  gaiaExport64 (ptr, m, GAIA_LITTLE_ENDIAN,
					endian_arch);
		      ptr += 8;
		  }
		if (n_coords > 3)
		  {
		      /* the fourth coordinate [M] */
		      gaiaExport64 (ptr, m, GAIA_LITTLE_ENDIAN, endian_arch);
		      ptr += 8;
		  }
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      z = 0.0;
		      m = 0.0;
		      if (geom->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (geom->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else if (geom->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      gaiaExport64 (ptr, x, GAIA_LITTLE_ENDIAN, endian_arch);	/* X - interior ring */
		      ptr += 8;
		      gaiaExport64 (ptr, y, GAIA_LITTLE_ENDIAN, endian_arch);	/* Y - interior ring */
		      ptr += 8;
		      if (n_coords > 2)
			{
			    /* the third coordinate [Z or M]; defaulting to ZERO */
			    if (coord_dims == GAIA_XY_Z
				|| coord_dims == GAIA_XY_Z_M)
				gaiaExport64 (ptr, z, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    else
				gaiaExport64 (ptr, m, GAIA_LITTLE_ENDIAN,
					      endian_arch);
			    ptr += 8;
			}
		      if (n_coords > 3)
			{
			    /* the fourth coordinate [M] */
			    gaiaExport64 (ptr, m, GAIA_LITTLE_ENDIAN,
					  endian_arch);
			    ptr += 8;
			}
		  }
	    }
	  polyg = polyg->Next;
      }
}
/**************** End file: gg_wkb.c **********/


/**************** Begin file: gg_geodesic.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <math.h> */
/* #include <float.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION	/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

#define DEG2RAD	0.0174532925199432958
#define PI	3.14159265358979323846

struct ellipses
{
    char *name;
    double a;			/* equatorial radius - meters */
    double rf;			/* reverse flattening */
    double b;			/* polar radius - meters */
};

#if defined(_WIN32) && !defined(__MINGW32__)
#define isnan	_isnan
#endif /* not WIN32-MSVC */

GAIAGEO_DECLARE int
gaiaEllipseParams (const char *name, double *a, double *b, double *rf)
{
/* trying to find ellipse params */
    struct ellipses ellps_list[] = {
	{"MERIT", 6378137.0, 298.257, -1.0},
	{"SGS85", 6378136.0, 298.257, -1.0},
	{"GRS80", 6378137.0, 298.257222101, -1.0},
	{"IAU76", 6378140.0, 298.257, -1.0},
	{"airy", 6377563.396, -1.0, 6356256.910},
	{"APL4.9", 6378137.0, 298.25, -1.0},
	{"NWL9D", 6378145.0, 298.25, -1.0},
	{"mod_airy", 6377340.189, -1.0, 6356034.446},
	{"andrae", 6377104.43, 300.0, -1.0},
	{"aust_SA", 378160.0, 298.25, -1.0},
	{"GRS67", 6378160.0, 298.2471674270, -1.0},
	{"bessel", 6377397.155, 299.1528128, -1.0},
	{"bess_nam", 6377483.865, 299.1528128, -1.0},
	{"clrk66", 6378206.4, -1.0, 6356583.8},
	{"clrk80", 6378249.145, 293.4663, -1.0},
	{"CPM", 6375738.7, 334.29, -1.0},
	{"delmbr", 6376428.0, 311.5, -1.0},
	{"engelis", 6378136.05, 298.2566, -1.0},
	{"evrst30", 6377276.345, 300.8017, -1.0},
	{"evrst48", 6377304.063, 300.8017, -1.0},
	{"evrst56", 6377301.243, 300.8017, -1.0},
	{"evrst69", 6377295.664, 300.8017, -1.0},
	{"evrstSS", 6377298.556, 300.8017, -1.0},
	{"fschr60", 6378166.0, 298.3, -1.0},
	{"fschr60m", 6378155.0, 298.3, -1.0},
	{"fschr68", 6378150.0, 298.3, -1.0},
	{"helmert", 6378200.0, 298.3, -1.0},
	{"hough", 6378270.0, 297.0, -1.0},
	{"intl", 6378388.0, 297.0, -1.0},
	{"krass", 6378245.0, 298.3, -1.0},
	{"kaula", 6378163.0, 298.24, -1.0},
	{"lerch", 6378139.0, 298.257, -1.0},
	{"mprts", 6397300.0, 191.0, -1.0},
	{"new_intl", 6378157.5, -1.0, 6356772.2},
	{"plessis", 6376523.0, -1.0, 6355863.0},
	{"SEasia", 6378155.0, -1.0, 6356773.3205},
	{"walbeck", 6376896.0, -1.0, 6355834.8467},
	{"WGS60", 6378165.0, 298.3, -1.0},
	{"WGS66", 6378145.0, 298.25, -1.0},
	{"WGS72", 6378135.0, 298.26, -1.0},
	{"WGS84", 6378137.0, 298.257223563, -1.0},
	{"sphere", 6370997.0, -1.0, 6370997.0},
	{NULL, -1.0, -1.0, -1.0}
    };
    struct ellipses *pe = ellps_list;
    while (1)
      {
	  if (pe->name == NULL)
	      break;
	  if (strcmp (pe->name, name) == 0)
	    {
		*a = pe->a;
		if (pe->rf < 0.0)
		  {
		      *b = pe->b;
		      *rf = 1.0 / ((pe->a - pe->b) / pe->a);
		  }
		else
		  {
		      *b = (pe->a * (1.0 - (1.0 / pe->rf)));
		      *rf = pe->rf;
		  }
		return 1;
	    }
	  pe++;
      }
    return 0;
}

GAIAGEO_DECLARE double
gaiaGreatCircleDistance (double a, double b, double lat1, double lon1,
			 double lat2, double lon2)
{
/*
/ Calculate great-circle distance (in m) between two points specified by 
/ latitude/longitude (in decimal degrees) using Aviation Formulary
/
/ http://williams.best.vwh.net/avform.htm#Dist
/
*/
    double latrad1 = lat1 * DEG2RAD;
    double lonrad1 = lon1 * DEG2RAD;
    double latrad2 = lat2 * DEG2RAD;
    double lonrad2 = lon2 * DEG2RAD;
    double avg_radius;
    double k1 = (sin ((latrad1 - latrad2) / 2.0));
    double k2 = (sin ((lonrad1 - lonrad2) / 2.0));
    double dist;
    dist =
	2.0 * asin (sqrt (k1 * k1 + cos (latrad1) * cos (latrad2) * k2 * k2));
    if (dist < 0.0)
	dist = dist + PI;
    if (a == b)
	avg_radius = a;
    else
	avg_radius = (2.0 * a + b) / 3.0;
    dist = dist * avg_radius;
    return dist;
}

GAIAGEO_DECLARE double
gaiaGeodesicDistance (double a, double b, double rf, double lat1, double lon1,
		      double lat2, double lon2)
{
/*
/ Calculate geodesic distance (in m) 
/ between two points specified by latitude/longitude 
/ (in decimal degrees) using Vincenty inverse formula for ellipsoids
/
/ based on original JavaScript by (c) Chris Veness 2002-2008 
/ http://www.movable-type.co.uk/scripts/latlong-vincenty.html
/
*/
    double f = 1.0 / rf;
    double L = (lon2 - lon1) * DEG2RAD;
    double U1 = atan ((1.0 - f) * tan (lat1 * DEG2RAD));
    double U2 = atan ((1.0 - f) * tan (lat2 * DEG2RAD));
    double sinU1 = sin (U1);
    double cosU1 = cos (U1);
    double sinU2 = sin (U2);
    double cosU2 = cos (U2);
    double lambda = L;
    double lambdaP;
    double sinLambda;
    double cosLambda;
    double sinSigma;
    double cosSigma;
    double sigma;
    double sinAlpha;
    double cosSqAlpha;
    double cos2SigmaM;
    double C;
    double uSq;
    double A;
    double B;
    double deltaSigma;
    double s;
    int iterLimit = 100;
    do
      {
	  sinLambda = sin (lambda);
	  cosLambda = cos (lambda);
	  sinSigma =
	      sqrt ((cosU2 * sinLambda) * (cosU2 * sinLambda) +
		    (cosU1 * sinU2 -
		     sinU1 * cosU2 * cosLambda) * (cosU1 * sinU2 -
						   sinU1 * cosU2 * cosLambda));
	  if (sinSigma == 0.0)
	      return 0.0;	/* co-incident points */
	  cosSigma = sinU1 * sinU2 + cosU1 * cosU2 * cosLambda;
	  sigma = atan2 (sinSigma, cosSigma);
	  sinAlpha = cosU1 * cosU2 * sinLambda / sinSigma;
	  cosSqAlpha = 1.0 - sinAlpha * sinAlpha;
	  cos2SigmaM = cosSigma - 2.0 * sinU1 * sinU2 / cosSqAlpha;
	  if (isnan (cos2SigmaM))
	      cos2SigmaM = 0;	/* equatorial line */
	  C = f / 16.0 * cosSqAlpha * (4.0 + f * (4.0 - 3.0 * cosSqAlpha));
	  lambdaP = lambda;
	  lambda =
	      L + (1.0 - C) * f * sinAlpha * (sigma +
					      C * sinSigma * (cos2SigmaM +
							      C * cosSigma *
							      (-1.0 +
							       2.0 *
							       cos2SigmaM *
							       cos2SigmaM)));
      }
    while (fabs (lambda - lambdaP) > 1e-12 && --iterLimit > 0);
    if (iterLimit == 0)
	return -1.0;		/* formula failed to converge */
    uSq = cosSqAlpha * (a * a - b * b) / (b * b);
    A = 1.0 + uSq / 16384.0 * (4096.0 +
			       uSq * (-768.0 + uSq * (320.0 - 175.0 * uSq)));
    B = uSq / 1024.0 * (256.0 + uSq * (-128.0 + uSq * (74.0 - 47.0 * uSq)));
    deltaSigma =
	B * sinSigma * (cos2SigmaM +
			B / 4.0 * (cosSigma *
				   (-1.0 + 2.0 * cos2SigmaM * cos2SigmaM) -
				   B / 6.0 * cos2SigmaM * (-3.0 +
							   4.0 * sinSigma *
							   sinSigma) * (-3.0 +
									4.0 *
									cos2SigmaM
									*
									cos2SigmaM)));
    s = b * A * (sigma - deltaSigma);
    return s;
}

GAIAGEO_DECLARE void
gaiaFree (void *ptr)
{
/* freeing a generic memory allocation */
    if (!ptr)
	return;
    free (ptr);
}

GAIAGEO_DECLARE double
gaiaGreatCircleTotalLength (double a, double b, int dims, double *coords,
			    int vert)
{
/* computing the GreatCircle total length for some Linestring/Ring */
    int iv;
    double x1 = 0.0;
    double y1 = 0.0;
    double x2;
    double y2;
    double z;
    double m;
    double len = 0.0;
    for (iv = 0; iv < vert; iv++)
      {
	  if (dims == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (coords, iv, &x2, &y2, &z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaGetPointXYM (coords, iv, &x2, &y2, &m);
	    }
	  else if (dims == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (coords, iv, &x2, &y2, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (coords, iv, &x2, &y2);
	    }
	  if (iv > 0)
	      len += gaiaGreatCircleDistance (a, b, y1, x1, y2, x2);
	  x1 = x2;
	  y1 = y2;
      }
    return len;
}

GAIAGEO_DECLARE double
gaiaGeodesicTotalLength (double a, double b, double rf, int dims,
			 double *coords, int vert)
{
/* computing the Geodesic total length for some Linestring/Ring */
    int iv;
    double x1 = 0.0;
    double y1 = 0.0;
    double x2;
    double y2;
    double z;
    double m;
    double l;
    double len = 0.0;
    for (iv = 0; iv < vert; iv++)
      {
	  if (dims == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (coords, iv, &x2, &y2, &z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaGetPointXYM (coords, iv, &x2, &y2, &m);
	    }
	  else if (dims == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (coords, iv, &x2, &y2, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (coords, iv, &x2, &y2);
	    }
	  if (iv > 0)
	    {
		l = gaiaGeodesicDistance (a, b, rf, y1, x1, y2, x2);
		if (l < 0.0)
		    return -1.0;
		len += l;
	    }
	  x1 = x2;
	  y1 = y2;
      }
    return len;
}

GAIAGEO_DECLARE int
gaiaConvertLength (double value, int unit_from, int unit_to, double *cvt)
{
/* converting length from one unit to another */
    double m;
    double factors[] = {
	1000.0, 1.0, 0.1, 0.01, 0.001, 1852.0, 0.0254, 0.3048, 0.9144,
	1609.344, 1.8288, 20.1168, 0.201168, 1.0, 0.304800609601219,
	0.914401828803658, 20.11684023368047, 1609.347218694437, 0.91439523,
	0.30479841, 20.11669506
    };
    factors[GAIA_US_IN] /= 39.37;
    if (unit_from < GAIA_MIN_UNIT || unit_from > GAIA_MAX_UNIT)
	return 0;
    if (unit_to < GAIA_MIN_UNIT || unit_to > GAIA_MAX_UNIT)
	return 0;
    if (unit_from == unit_to)
      {
	  /* same unit */
	  *cvt = value;
      }
    else if (unit_from == GAIA_M)
      {
	  /* from Meters to .. */
	  *cvt = value / factors[unit_to];
      }
    else if (unit_to == GAIA_M)
      {
	  /* from .. to Meters */
	  *cvt = value * factors[unit_from];
      }
    else
      {
	  m = value * factors[unit_from];
	  *cvt = m / factors[unit_to];
      }
    return 1;
}

#undef DEG2RAD
#undef PI
/**************** End file: gg_geodesic.c **********/


/**************** Begin file: spatialite.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <math.h> */
/* #include <float.h> */
/* #include <locale.h> */
/* #include <errno.h> */

#if defined(_WIN32) || defined(WIN32)
/* #include <io.h> */
#define isatty	_isatty
#else
/* #include <unistd.h> */
#endif

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */
/* #include <spatialite/gaiaexif.h> */
/* #include <spatialite/spatialite.h> */
/* #include <spatialite.h> */

#ifndef OMIT_GEOS		/* including GEOS */
/* #include <geos_c.h> */
#endif

#ifndef OMIT_PROJ		/* including PROJ.4 */
/* #include <proj_api.h> */
#endif

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

#define GAIA_UNUSED() if (argc || argv) argc = argc;

#ifndef OMIT_GEOCALLBACKS	/* supporting RTree geometry callbacks */
struct gaia_rtree_mbr
{
/* a struct used by R*Tree GeometryCallback functions [MBR] */
    double minx;
    double miny;
    double maxx;
    double maxy;
};
#endif /* end RTree geometry callbacks */

static SQLITE_EXTENSION_INIT1 struct spatial_index_str
{
/* a struct to implement a linked list of spatial-indexes */
    char ValidRtree;
    char ValidCache;
    char *TableName;
    char *ColumnName;
    struct spatial_index_str *Next;
};

struct stddev_str
{
/* a struct to implement StandardVariation and Variance aggregate functions */
    int cleaned;
    double mean;
    double quot;
    double count;
};

struct fdo_table
{
/* a struct to implement a linked-list for FDO-ORG table names */
    char *table;
    struct fdo_table *next;
};

static void
fnct_spatialite_version (sqlite3_context * context, int argc,
			 sqlite3_value ** argv)
{
/* SQL function:
/ spatialite_version()
/
/ return a text string representing the current SpatiaLite version
*/
    int len;
    const char *p_result = spatialite_version ();
    GAIA_UNUSED ();
    len = strlen (p_result);
    sqlite3_result_text (context, p_result, len, SQLITE_TRANSIENT);
}

static void
fnct_geos_version (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ geos_version()
/
/ return a text string representing the current GEOS version
/ or NULL if GEOS is currently unsupported
*/

#ifndef OMIT_GEOS		/* GEOS version */
    int len;
    const char *p_result = GEOSversion ();
    GAIA_UNUSED ();
    len = strlen (p_result);
    sqlite3_result_text (context, p_result, len, SQLITE_TRANSIENT);
#else
    sqlite3_result_null (context);
#endif
}


static void
fnct_proj4_version (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ proj4_version()
/
/ return a text string representing the current PROJ.4 version
/ or NULL if PROJ.4 is currently unsupported
*/

#ifndef OMIT_PROJ		/* PROJ.4 version */
    int len;
    const char *p_result = pj_get_release ();
    GAIA_UNUSED ();
    len = strlen (p_result);
    sqlite3_result_text (context, p_result, len, SQLITE_TRANSIENT);
#else
    sqlite3_result_null (context);
#endif
}

static void
clean_sql_string (char *buf)
{
/* well-formatting a string to be used as an SQL string-value */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    while (*in != '\0')
      {
	  if (*in == '\'')
	      *out++ = '\'';
	  *out++ = *in++;
      }
    *out = '\0';
}

static void
double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

static void
fnct_GeometryConstraints (sqlite3_context * context, int argc,
			  sqlite3_value ** argv)
{
/* SQL function:
/ GeometryConstraints(BLOBencoded geometry, geometry-type, srid)
/ GeometryConstraints(BLOBencoded geometry, geometry-type, srid, dimensions)
/
/ checks geometry constraints, returning:
/
/ -1 - if some error occurred
/ 1 - if geometry constraints validation passes
/ 0 - if geometry constraints validation fails
/
*/
    int little_endian;
    int endian_arch = gaiaEndianArch ();
    unsigned char *p_blob = NULL;
    int n_bytes = 0;
    int srid;
    int geom_srid = -1;
    const unsigned char *type;
    int xtype;
    int geom_type = -1;
    int geom_normalized_type;
    const unsigned char *dimensions;
    int dims = GAIA_XY;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_BLOB
	|| sqlite3_value_type (argv[0]) == SQLITE_NULL)
	;
    else
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_TEXT)
	type = sqlite3_value_text (argv[1]);
    else
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	srid = sqlite3_value_int (argv[2]);
    else
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (argc == 4)
      {
	  /* explicit dimensions - supporting XYZM */
	  dimensions = sqlite3_value_text (argv[3]);
	  if (strcasecmp ((char *) dimensions, "XYZ") == 0)
	      dims = GAIA_XY_Z;
	  else if (strcasecmp ((char *) dimensions, "XYM") == 0)
	      dims = GAIA_XY_M;
	  else if (strcasecmp ((char *) dimensions, "XYZM") == 0)
	      dims = GAIA_XY_Z_M;
	  else
	      dims = GAIA_XY;
      }
    if (sqlite3_value_type (argv[0]) == SQLITE_BLOB)
      {
	  p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
	  n_bytes = sqlite3_value_bytes (argv[0]);
      }
    if (p_blob)
      {
	  /* quick Geometry validation */
	  if (n_bytes < 45)
	      goto illegal_geometry;	/* cannot be an internal BLOB WKB geometry */
	  if (*(p_blob + 0) != GAIA_MARK_START)
	      goto illegal_geometry;	/* failed to recognize START signature */
	  if (*(p_blob + (n_bytes - 1)) != GAIA_MARK_END)
	      goto illegal_geometry;	/* failed to recognize END signature */
	  if (*(p_blob + 38) != GAIA_MARK_MBR)
	      goto illegal_geometry;	/* failed to recognize MBR signature */
	  if (*(p_blob + 1) == GAIA_LITTLE_ENDIAN)
	      little_endian = 1;
	  else if (*(p_blob + 1) == GAIA_BIG_ENDIAN)
	      little_endian = 0;
	  else
	      goto illegal_geometry;	/* unknown encoding; neither little-endian nor big-endian */
	  geom_type = gaiaImport32 (p_blob + 39, little_endian, endian_arch);
	  geom_srid = gaiaImport32 (p_blob + 2, little_endian, endian_arch);
	  goto valid_geometry;
	illegal_geometry:
	  sqlite3_result_int (context, -1);
	  return;
      }
  valid_geometry:
    xtype = GAIA_UNKNOWN;
    if (strcasecmp ((char *) type, "POINT") == 0)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_POINTZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_POINTM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_POINTZM;
		break;
	    default:
		xtype = GAIA_POINT;
		break;
	    };
      }
    if (strcasecmp ((char *) type, "LINESTRING") == 0)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_LINESTRINGZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_LINESTRINGM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_LINESTRINGZM;
		break;
	    default:
		xtype = GAIA_LINESTRING;
		break;
	    };
      }
    if (strcasecmp ((char *) type, "POLYGON") == 0)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_POLYGONZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_POLYGONM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_POLYGONZM;
		break;
	    default:
		xtype = GAIA_POLYGON;
		break;
	    };
      }
    if (strcasecmp ((char *) type, "MULTIPOINT") == 0)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_MULTIPOINTZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_MULTIPOINTM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_MULTIPOINTZM;
		break;
	    default:
		xtype = GAIA_MULTIPOINT;
		break;
	    };
      }
    if (strcasecmp ((char *) type, "MULTILINESTRING") == 0)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_MULTILINESTRINGZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_MULTILINESTRINGM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_MULTILINESTRINGZM;
		break;
	    default:
		xtype = GAIA_MULTILINESTRING;
		break;
	    };
      }
    if (strcasecmp ((char *) type, "MULTIPOLYGON") == 0)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_MULTIPOLYGONZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_MULTIPOLYGONM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_MULTIPOLYGONZM;
		break;
	    default:
		xtype = GAIA_MULTIPOLYGON;
		break;
	    };
      }
    if (strcasecmp ((char *) type, "GEOMETRYCOLLECTION") == 0)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_GEOMETRYCOLLECTIONZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_GEOMETRYCOLLECTIONM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_GEOMETRYCOLLECTIONZM;
		break;
	    default:
		xtype = GAIA_GEOMETRYCOLLECTION;
		break;
	    };
      }
    switch (geom_type)
      {
	  /* adjusting COMPRESSED Geometries */
      case GAIA_COMPRESSED_LINESTRING:
	  geom_normalized_type = GAIA_LINESTRING;
	  break;
      case GAIA_COMPRESSED_LINESTRINGZ:
	  geom_normalized_type = GAIA_LINESTRINGZ;
	  break;
      case GAIA_COMPRESSED_LINESTRINGM:
	  geom_normalized_type = GAIA_LINESTRINGM;
	  break;
      case GAIA_COMPRESSED_LINESTRINGZM:
	  geom_normalized_type = GAIA_LINESTRINGZM;
	  break;
      case GAIA_COMPRESSED_POLYGON:
	  geom_normalized_type = GAIA_POLYGON;
	  break;
      case GAIA_COMPRESSED_POLYGONZ:
	  geom_normalized_type = GAIA_POLYGONZ;
	  break;
      case GAIA_COMPRESSED_POLYGONM:
	  geom_normalized_type = GAIA_POLYGONM;
	  break;
      case GAIA_COMPRESSED_POLYGONZM:
	  geom_normalized_type = GAIA_POLYGONZM;
	  break;
      default:
	  geom_normalized_type = geom_type;
	  break;
      };
    if (strcasecmp ((char *) type, "GEOMETRY") == 0)
	xtype = -1;
    if (xtype == GAIA_UNKNOWN)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = 1;
	  if (p_blob)
	    {
		/* skipping NULL Geometry; this is assumed to be always good */
		if (geom_srid != srid)
		    ret = 0;
		if (xtype == -1)
		    ;
		else if (xtype != geom_normalized_type)
		    ret = 0;
	    }
	  sqlite3_result_int (context, ret);
      }
}

static void
fnct_RTreeAlign (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ RTreeAlign(RTree-table-name, PKID-value, BLOBencoded geometry)
/
/ attempts to update the associated R*Tree, returning:
/
/ -1 - if some invalid arg was passed
/ 1 - succesfull update
/ 0 - update failure
/
*/
    unsigned char *p_blob = NULL;
    int n_bytes = 0;
    sqlite3_int64 pkid;
    const unsigned char *rtree_table;
    gaiaGeomCollPtr geom = NULL;
    int ret;
    char table_name[1024];
    char sql[4192];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_TEXT)
	rtree_table = sqlite3_value_text (argv[0]);
    else
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	pkid = sqlite3_value_int64 (argv[1]);
    else
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_BLOB
	|| sqlite3_value_type (argv[2]) == SQLITE_NULL)
	;
    else
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_BLOB)
      {
	  p_blob = (unsigned char *) sqlite3_value_blob (argv[2]);
	  n_bytes = sqlite3_value_bytes (argv[2]);
	  geom = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
      }

    if (geom == NULL)
      {
	  /* NULL geometry: nothing to do */
	  sqlite3_result_int (context, 1);
      }
    else
      {
	  /* INSERTing into the R*Tree */
	  strcpy (table_name, rtree_table);
	  if (*(table_name + 0) == '"'
	      && *(table_name + strlen (table_name) - 1) == '"')
	      ;			/* earlier versions may pass an already quoted name */
	  else
	      double_quoted_sql (table_name);
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT: M$ runtime doesn't supports %lld for 64 bits */
	  sprintf (sql, "INSERT INTO %s (pkid, xmin, ymin, xmax, ymax) "
		   "VALUES (%I64d, %1.12f, %1.12f, %1.12f, %1.12f)",
		   table_name, pkid, geom->MinX, geom->MinY, geom->MaxX,
		   geom->MaxY);
#else
	  sprintf (sql, "INSERT INTO %s (pkid, xmin, ymin, xmax, ymax) "
		   "VALUES (%lld, %1.12f, %1.12f, %1.12f, %1.12f)",
		   table_name, pkid, geom->MinX, geom->MinY, geom->MaxX,
		   geom->MaxY);
#endif
	  gaiaFreeGeomColl (geom);
	  ret = sqlite3_exec (sqlite, sql, NULL, NULL, NULL);
	  if (ret != SQLITE_OK)
	      sqlite3_result_int (context, 0);
	  else
	      sqlite3_result_int (context, 1);
      }
}

static int
checkSpatialMetaData (sqlite3 * sqlite)
{
/* internal utility function:
/
/ for FDO-OGR interoperability:
/ tests the SpatialMetadata type, returning:
/
/ 0 - if no valid SpatialMetaData where found
/ 1 - if SpatiaLite-like SpatialMetadata where found
/ 2- if FDO-OGR-like SpatialMetadata where found
/
*/
    int spatialite_rs = 0;
    int fdo_rs = 0;
    int spatialite_gc = 0;
    int fdo_gc = 0;
    int rs_srid = 0;
    int auth_name = 0;
    int auth_srid = 0;
    int srtext = 0;
    int ref_sys_name = 0;
    int proj4text = 0;
    int f_table_name = 0;
    int f_geometry_column = 0;
    int geometry_type = 0;
    int coord_dimension = 0;
    int gc_srid = 0;
    int geometry_format = 0;
    int type = 0;
    int spatial_index_enabled = 0;
    char sql[1024];
    int ret;
    const char *name;
    int i;
    char **results;
    int rows;
    int columns;
/* checking the GEOMETRY_COLUMNS table */
    strcpy (sql, "PRAGMA table_info(geometry_columns)");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	goto unknown;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, "f_table_name") == 0)
		    f_table_name = 1;
		if (strcasecmp (name, "f_geometry_column") == 0)
		    f_geometry_column = 1;
		if (strcasecmp (name, "geometry_type") == 0)
		    geometry_type = 1;
		if (strcasecmp (name, "coord_dimension") == 0)
		    coord_dimension = 1;
		if (strcasecmp (name, "srid") == 0)
		    gc_srid = 1;
		if (strcasecmp (name, "geometry_format") == 0)
		    geometry_format = 1;
		if (strcasecmp (name, "type") == 0)
		    type = 1;
		if (strcasecmp (name, "spatial_index_enabled") == 0)
		    spatial_index_enabled = 1;
	    }
      }
    sqlite3_free_table (results);
    if (f_table_name
	&&
	f_geometry_column
	&& type && coord_dimension && gc_srid && spatial_index_enabled)
	spatialite_gc = 1;
    if (f_table_name
	&&
	f_geometry_column
	&& geometry_type && coord_dimension && gc_srid && geometry_format)
	fdo_gc = 1;
/* checking the SPATIAL_REF_SYS table */
    strcpy (sql, "PRAGMA table_info(spatial_ref_sys)");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	goto unknown;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, "srid") == 0)
		    rs_srid = 1;
		if (strcasecmp (name, "auth_name") == 0)
		    auth_name = 1;
		if (strcasecmp (name, "auth_srid") == 0)
		    auth_srid = 1;
		if (strcasecmp (name, "srtext") == 0)
		    srtext = 1;
		if (strcasecmp (name, "ref_sys_name") == 0)
		    ref_sys_name = 1;
		if (strcasecmp (name, "proj4text") == 0)
		    proj4text = 1;
	    }
      }
    sqlite3_free_table (results);
    if (rs_srid && auth_name && auth_srid && ref_sys_name && proj4text)
	spatialite_rs = 1;
    if (rs_srid && auth_name && auth_srid && srtext)
	fdo_rs = 1;
/* verifying the MetaData format */
    if (spatialite_gc && spatialite_rs)
	return 1;
    if (fdo_gc && fdo_rs)
	return 2;
  unknown:
    return 0;
}

static void
add_fdo_table (struct fdo_table **first, struct fdo_table **last,
	       const char *table, int len)
{
/* adds an FDO-OGR styled Geometry Table to corresponding linked list */
    struct fdo_table *p = malloc (sizeof (struct fdo_table));
    p->table = malloc (len + 1);
    strcpy (p->table, table);
    p->next = NULL;
    if (!(*first))
	(*first) = p;
    if ((*last))
	(*last)->next = p;
    (*last) = p;
}

static void
free_fdo_tables (struct fdo_table *first)
{
/* memory cleanup; destroying the FDO-OGR tables linked list */
    struct fdo_table *p;
    struct fdo_table *pn;
    p = first;
    while (p)
      {
	  pn = p->next;
	  if (p->table)
	      free (p->table);
	  free (p);
	  p = pn;
      }
}

static void
fnct_AutoFDOStart (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AutoFDOStart(void)
/
/ for FDO-OGR interoperability:
/ tests the SpatialMetadata type, then automatically
/ creating a VirtualFDO table for each FDO-OGR main table 
/ declared within FDO-styled SpatialMetadata
/
*/
    int ret;
    const char *name;
    int i;
    char **results;
    int rows;
    int columns;
    char sql[1024];
    int count = 0;
    struct fdo_table *first = NULL;
    struct fdo_table *last = NULL;
    struct fdo_table *p;
    int len;
    char xname[1024];
    char xtable[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (checkSpatialMetaData (sqlite) == 2)
      {
	  /* ok, creating VirtualFDO tables */
	  strcpy (sql, "SELECT DISTINCT f_table_name FROM geometry_columns");
	  ret =
	      sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (rows < 1)
	      ;
	  else
	    {
		for (i = 1; i <= rows; i++)
		  {
		      name = results[(i * columns) + 0];
		      if (name)
			{
			    len = strlen (name);
			    add_fdo_table (&first, &last, name, len);
			}
		  }
	    }
	  sqlite3_free_table (results);
	  p = first;
	  while (p)
	    {
		/* destroying the VirtualFDO table [if existing] */
		sprintf (xname, "fdo_%s", p->table);
		double_quoted_sql (xname);
		sprintf (sql, "DROP TABLE IF EXISTS %s", xname);
		ret = sqlite3_exec (sqlite, sql, NULL, 0, NULL);
		if (ret != SQLITE_OK)
		    goto error;
		/* creating the VirtualFDO table */
		strcpy (xtable, p->table);
		double_quoted_sql (xtable);
		sprintf (sql, "CREATE VIRTUAL TABLE %s USING VirtualFDO(%s)",
			 xname, xtable);
		ret = sqlite3_exec (sqlite, sql, NULL, 0, NULL);
		if (ret != SQLITE_OK)
		    goto error;
		count++;
		p = p->next;
	    }
	error:
	  free_fdo_tables (first);
	  sqlite3_result_int (context, count);
	  return;
      }
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_AutoFDOStop (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AutoFDOStop(void)
/
/ for FDO-OGR interoperability:
/ tests the SpatialMetadata type, then automatically
/ removes any VirtualFDO table 
/
*/
    int ret;
    const char *name;
    int i;
    char **results;
    int rows;
    int columns;
    char sql[1024];
    int count = 0;
    struct fdo_table *first = NULL;
    struct fdo_table *last = NULL;
    struct fdo_table *p;
    int len;
    char xname[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (checkSpatialMetaData (sqlite) == 2)
      {
	  /* ok, creating VirtualFDO tables */
	  strcpy (sql, "SELECT DISTINCT f_table_name FROM geometry_columns");
	  ret =
	      sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (rows < 1)
	      ;
	  else
	    {
		for (i = 1; i <= rows; i++)
		  {
		      name = results[(i * columns) + 0];
		      if (name)
			{
			    len = strlen (name);
			    add_fdo_table (&first, &last, name, len);
			}
		  }
	    }
	  sqlite3_free_table (results);
	  p = first;
	  while (p)
	    {
		/* destroying the VirtualFDO table [if existing] */
		sprintf (xname, "fdo_%s", p->table);
		double_quoted_sql (xname);
		sprintf (sql, "DROP TABLE IF EXISTS %s", xname);
		ret = sqlite3_exec (sqlite, sql, NULL, 0, NULL);
		if (ret != SQLITE_OK)
		    goto error;
		count++;
		p = p->next;
	    }
	error:
	  free_fdo_tables (first);
	  sqlite3_result_int (context, count);
	  return;
      }
    sqlite3_result_int (context, 0);
    return;
}

static int
testSpatiaLiteHistory (sqlite3 * sqlite)
{
/* internal utility function:
/
/ checks if the SPATIALITE_HISTORY table already exists
/
*/
    int event_id = 0;
    int table_name = 0;
    int geometry_column = 0;
    int event = 0;
    int timestamp = 0;
    int ver_sqlite = 0;
    int ver_splite = 0;
    char sql[1024];
    int ret;
    const char *name;
    int i;
    char **results;
    int rows;
    int columns;
/* checking the SPATIALITE_HISTORY table */
    strcpy (sql, "PRAGMA table_info(spatialite_history)");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	return 0;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, "event_id") == 0)
		    event_id = 1;
		if (strcasecmp (name, "table_name") == 0)
		    table_name = 1;
		if (strcasecmp (name, "geometry_column") == 0)
		    geometry_column = 1;
		if (strcasecmp (name, "event") == 0)
		    event = 1;
		if (strcasecmp (name, "timestamp") == 0)
		    timestamp = 1;
		if (strcasecmp (name, "ver_sqlite") == 0)
		    ver_sqlite = 1;
		if (strcasecmp (name, "ver_splite") == 0)
		    ver_splite = 1;
	    }
      }
    sqlite3_free_table (results);
    if (event_id && table_name && geometry_column && event && timestamp
	&& ver_sqlite && ver_splite)
	return 1;
    return 0;
}

static int
checkSpatiaLiteHistory (sqlite3 * sqlite)
{
/* internal utility function:
/
/ checks if the SPATIALITE_HISTORY table already exists
/ if not, such table will then be created
/
*/
    char sql[1024];
    char *errMsg = NULL;
    int ret;

    if (testSpatiaLiteHistory (sqlite))
	return 1;

/* creating the SPATIALITE_HISTORY table */
    strcpy (sql, "CREATE TABLE IF NOT EXISTS ");
    strcat (sql, "spatialite_history (\n");
    strcat (sql, "event_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,\n");
    strcat (sql, "table_name TEXT NOT NULL,\n");
    strcat (sql, "geometry_column TEXT,\n");
    strcat (sql, "event TEXT NOT NULL,\n");
    strcat (sql, "timestamp TEXT NOT NULL,\n");
    strcat (sql, "ver_sqlite TEXT NOT NULL,\n");
    strcat (sql, "ver_splite TEXT NOT NULL)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	return 0;

    if (testSpatiaLiteHistory (sqlite))
	return 1;
    return 0;
}

static void
updateSpatiaLiteHistory (sqlite3 * sqlite, const char *table,
			 const char *geom, const char *operation)
{
/* inserting a row in SPATIALITE_HISTORY */
    char sql[2048];
    sqlite3_stmt *stmt = NULL;
    int ret;

    if (checkSpatiaLiteHistory (sqlite) == 0)
	return;

    strcpy (sql, "INSERT INTO spatialite_history ");
    strcat (sql, "(event_id, table_name, geometry_column, event, timestamp, ");
    strcat (sql, "ver_sqlite, ver_splite) ");
    strcat (sql,
	    "VALUES (NULL, ?, ?, ?, DateTime('now'), sqlite_version(), spatialite_version())");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n%s\n", sql, sqlite3_errmsg (sqlite));
	  goto stop;
      }
    sqlite3_reset (stmt);
    sqlite3_clear_bindings (stmt);
    sqlite3_bind_text (stmt, 1, table, strlen (table), SQLITE_STATIC);
    if (!geom)
	sqlite3_bind_null (stmt, 2);
    else
	sqlite3_bind_text (stmt, 2, geom, strlen (geom), SQLITE_STATIC);
    sqlite3_bind_text (stmt, 3, operation, strlen (operation), SQLITE_STATIC);
    ret = sqlite3_step (stmt);
    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	goto stop;
    fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));

  stop:
    if (stmt)
	sqlite3_finalize (stmt);
}

static int
createAdvancedMetaData (sqlite3 * sqlite)
{
/* creating the advanced MetaData tables */
    char sql[1024];
    char *errMsg = NULL;
    int ret;
/* creating the VIEWS_GEOMETRY_COLUMNS table */
    strcpy (sql, "CREATE TABLE IF NOT EXISTS ");
    strcat (sql, "views_geometry_columns (\n");
    strcat (sql, "view_name TEXT NOT NULL,\n");
    strcat (sql, "view_geometry TEXT NOT NULL,\n");
    strcat (sql, "view_rowid TEXT NOT NULL,\n");
    strcat (sql, "f_table_name VARCHAR(256) NOT NULL,\n");
    strcat (sql, "f_geometry_column VARCHAR(256) NOT NULL,\n");
    strcat (sql, "CONSTRAINT pk_geom_cols_views PRIMARY KEY ");
    strcat (sql, "(view_name, view_geometry),\n");
    strcat (sql, "CONSTRAINT fk_views_geom_cols FOREIGN KEY ");
    strcat (sql, "(f_table_name, f_geometry_column) ");
    strcat (sql, "REFERENCES geometry_columns ");
    strcat (sql, "(f_table_name, f_geometry_column) ");
    strcat (sql, "ON DELETE CASCADE)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	return 0;
/* creating an INDEX supporting the GEOMETRY_COLUMNS FK */
    strcpy (sql, "CREATE INDEX IF NOT EXISTS ");
    strcat (sql, "idx_viewsjoin ON views_geometry_columns\n");
    strcat (sql, "(f_table_name, f_geometry_column)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	return 0;
/* creating the VIRTS_GEOMETRY_COLUMNS table */
    strcpy (sql, "CREATE TABLE IF NOT EXISTS ");
    strcat (sql, "virts_geometry_columns (\n");
    strcat (sql, "virt_name TEXT NOT NULL,\n");
    strcat (sql, "virt_geometry TEXT NOT NULL,\n");
    strcat (sql, "type VARCHAR(30) NOT NULL,\n");
    strcat (sql, "srid INTEGER NOT NULL,\n");
    strcat (sql, "CONSTRAINT pk_geom_cols_virts PRIMARY KEY ");
    strcat (sql, "(virt_name, virt_geometry),\n");
    strcat (sql, "CONSTRAINT fk_vgc_srid FOREIGN KEY ");
    strcat (sql, "(srid) REFERENCES spatial_ref_sys (srid))");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	return 0;
/* creating an INDEX supporting the SPATIAL_REF_SYS FK */
    strcpy (sql, "CREATE INDEX IF NOT EXISTS ");
    strcat (sql, "idx_virtssrid ON virts_geometry_columns\n");
    strcat (sql, "(srid)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	return 0;
/* creating the GEOMETRY_COLUMNS_AUTH table */
    strcpy (sql, "CREATE TABLE IF NOT EXISTS ");
    strcat (sql, "geometry_columns_auth (\n");
    strcat (sql, "f_table_name VARCHAR(256) NOT NULL,\n");
    strcat (sql, "f_geometry_column VARCHAR(256) NOT NULL,\n");
    strcat (sql, "read_only INTEGER NOT NULL,\n");
    strcat (sql, "hidden INTEGER NOT NULL,\n");
    strcat (sql, "CONSTRAINT pk_gc_auth PRIMARY KEY ");
    strcat (sql, "(f_table_name, f_geometry_column),\n");
    strcat (sql, "CONSTRAINT fk_gc_auth FOREIGN KEY ");
    strcat (sql, "(f_table_name, f_geometry_column) ");
    strcat (sql, "REFERENCES geometry_columns ");
    strcat (sql, "(f_table_name, f_geometry_column) ");
    strcat (sql, "ON DELETE CASCADE)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	return 0;
    return 1;
}

static void
fnct_CheckSpatialMetaData (sqlite3_context * context, int argc,
			   sqlite3_value ** argv)
{
/* SQL function:
/ CheckSpatialMetaData(void)
/
/ for FDO-OGR interoperability:
/ tests the SpatialMetadata type, returning:
/
/ 0 - if no valid SpatialMetaData where found
/ 1 - if SpatiaLite-like SpatialMetadata where found
/ 2- if FDO-OGR-like SpatialMetadata where found
/
*/
    sqlite3 *sqlite;
    int ret;
    GAIA_UNUSED ();
    sqlite = sqlite3_context_db_handle (context);
    ret = checkSpatialMetaData (sqlite);
    if (ret == 1)
      {
	  /* trying to create the advanced metadata tables */
	  createAdvancedMetaData (sqlite);
      }
    sqlite3_result_int (context, ret);
    return;
}

static void
fnct_InitSpatialMetaData (sqlite3_context * context, int argc,
			  sqlite3_value ** argv)
{
/* SQL function:
/ InitSpatialMetaData(void)
/
/ creates the SPATIAL_REF_SYS and GEOMETRY_COLUMNS tables
/ returns 1 on success
/ 0 on failure
*/
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
/* creating the SPATIAL_REF_SYS table */
    strcpy (sql, "CREATE TABLE spatial_ref_sys (\n");
    strcat (sql, "srid INTEGER NOT NULL PRIMARY KEY,\n");
    strcat (sql, "auth_name TEXT NOT NULL,\n");
    strcat (sql, "auth_srid INTEGER NOT NULL,\n");
    strcat (sql, "ref_sys_name TEXT,\n");
    strcat (sql, "proj4text TEXT NOT NULL,\n");
    strcat (sql, "srs_wkt TEXT) ");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    strcpy (sql, "CREATE UNIQUE INDEX idx_spatial_ref_sys \n");
    strcat (sql, "ON spatial_ref_sys (auth_srid, auth_name)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    updateSpatiaLiteHistory (sqlite, "spatial_ref_sys", NULL,
			     "table successfully created");
/* creating the GEOMETRY_COLUMN table */
    strcpy (sql, "CREATE TABLE geometry_columns (\n");
    strcat (sql, "f_table_name TEXT NOT NULL,\n");
    strcat (sql, "f_geometry_column TEXT NOT NULL,\n");
    strcat (sql, "type TEXT NOT NULL,\n");
    strcat (sql, "coord_dimension TEXT NOT NULL,\n");
    strcat (sql, "srid INTEGER NOT NULL,\n");
    strcat (sql, "spatial_index_enabled INTEGER NOT NULL,\n");
    strcat (sql, "CONSTRAINT pk_geom_cols PRIMARY KEY ");
    strcat (sql, "(f_table_name, f_geometry_column),\n");
    strcat (sql, "CONSTRAINT fk_gc_srs FOREIGN KEY ");
    strcat (sql, "(srid) REFERENCES spatial_ref_sys (srid))");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    updateSpatiaLiteHistory (sqlite, "geometry_columns", NULL,
			     "table successfully created");
/* creating an INDEX corresponding to the SRID FK */
    strcpy (sql, "CREATE INDEX idx_srid_geocols ON geometry_columns\n");
    strcat (sql, "(srid) ");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
/* creating the GEOM_COLS_REF_SYS view */
    strcpy (sql, "CREATE VIEW geom_cols_ref_sys AS\n");
    strcat (sql, "SELECT f_table_name, f_geometry_column, type,\n");
    strcat (sql, "coord_dimension, spatial_ref_sys.srid AS srid,\n");
    strcat (sql, "auth_name, auth_srid, ref_sys_name, proj4text\n");
    strcat (sql, "FROM geometry_columns, spatial_ref_sys\n");
    strcat (sql, "WHERE geometry_columns.srid = spatial_ref_sys.srid");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    if (!createAdvancedMetaData (sqlite))
	goto error;
/* creating the SpatialIndex VIRTUAL TABLE */
    strcpy (sql, "CREATE VIRTUAL TABLE SpatialIndex ");
    strcat (sql, "USING VirtualSpatialIndex()");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    if (spatial_ref_sys_init (sqlite, 0))
	updateSpatiaLiteHistory (sqlite, "spatial_ref_sys", NULL,
				 "table successfully populated");
    sqlite3_result_int (context, 1);
    return;
  error:
    fprintf (stderr, " InitSpatiaMetaData ()error:\"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static int
recoverGeomColumn (sqlite3 * sqlite, const unsigned char *table,
		   const unsigned char *column, int xtype, int dims, int srid)
{
/* checks if TABLE.COLUMN exists and has the required features */
    int ok = 1;
    char sql[1024];
    int type;
    sqlite3_stmt *stmt;
    gaiaGeomCollPtr geom;
    const void *blob_value;
    int len;
    int ret;
    int i_col;
    char xcolumn[1024];
    char xtable[1024];
    strcpy (xcolumn, (char *) column);
    double_quoted_sql (xcolumn);
    strcpy (xtable, (char *) table);
    double_quoted_sql (xtable);
    sprintf (sql, "SELECT %s FROM %s", xcolumn, xtable);
/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "recoverGeomColumn: error %d \"%s\"\n",
		   sqlite3_errcode (sqlite), sqlite3_errmsg (sqlite));
	  return 0;
      }
    while (1)
      {
	  /* scrolling the result set rows */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* checking Geometry features */
		geom = NULL;
		for (i_col = 0; i_col < sqlite3_column_count (stmt); i_col++)
		  {
		      if (sqlite3_column_type (stmt, i_col) != SQLITE_BLOB)
			  ok = 0;
		      else
			{
			    blob_value = sqlite3_column_blob (stmt, i_col);
			    len = sqlite3_column_bytes (stmt, i_col);
			    geom = gaiaFromSpatiaLiteBlobWkb (blob_value, len);
			    if (!geom)
				ok = 0;
			    else
			      {
				  if (geom->DimensionModel != dims)
				      ok = 0;
				  if (geom->Srid != srid)
				      ok = 0;
				  type = gaiaGeometryType (geom);
				  if (xtype == -1)
				      ;	/* GEOMETRY */
				  else
				    {
					if (xtype == type)
					    ;
					else
					    ok = 0;
				    }
				  gaiaFreeGeomColl (geom);
			      }
			}
		  }
	    }
	  if (!ok)
	      break;
      }
    ret = sqlite3_finalize (stmt);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "recoverGeomColumn: error %d \"%s\"\n",
		   sqlite3_errcode (sqlite), sqlite3_errmsg (sqlite));
	  return 0;
      }
    return ok;
}

static void
buildSpatialIndex (sqlite3 * sqlite, const unsigned char *table,
		   const char *col_name)
{
/* loading a SpatialIndex [RTree] */
    char sql[2048];
    char sql2[1024];
    char *errMsg = NULL;
    int ret;
    char xname[1024];
    char xtable[1024];
    sprintf (xname, "idx_%s_%s", table, col_name);
    double_quoted_sql (xname);
    sprintf (sql, "INSERT INTO %s (pkid, xmin, xmax, ymin, ymax) ", xname);
    strcpy (xname, col_name);
    double_quoted_sql (xname);
    strcpy (xtable, (char *) table);
    double_quoted_sql (xtable);
    sprintf (sql2,
	     "SELECT ROWID, MbrMinX(%s), MbrMaxX(%s), MbrMinY(%s), MbrMaxY(%s) FROM %s",
	     xname, xname, xname, xname, xtable);
    strcat (sql, sql2);
    sprintf (sql2, " WHERE MbrMinX(%s) IS NOT NULL", xname);
    strcat (sql, sql2);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "buildSpatialIndex error: \"%s\"\n", errMsg);
	  sqlite3_free (errMsg);
      }
}

static void
updateGeometryTriggers (sqlite3 * sqlite, const unsigned char *table,
			const unsigned char *column)
{
/* updates triggers for some Spatial Column */
    char sql[256];
    char trigger[4096];
    char **results;
    int ret;
    int rows;
    int columns;
    int i;
    char tblname[256];
    char colname[256];
    char col_type[32];
    char col_srid[32];
    char col_index[32];
    char col_dims[64];
    int index;
    int cached;
    int dims;
    char *txt_dims;
    int len;
    char *errMsg = NULL;
    char dummy[512];
    char sqltable[1024];
    char sqlcolumn[1024];
    char xname[1024];
    char xcolname[1024];
    char xtable[1024];
    char xindex[1024];
    struct spatial_index_str *first_idx = NULL;
    struct spatial_index_str *last_idx = NULL;
    struct spatial_index_str *curr_idx;
    struct spatial_index_str *next_idx;
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    sprintf (sql,
	     "SELECT f_table_name, f_geometry_column, type, srid, spatial_index_enabled, coord_dimension "
	     "FROM geometry_columns WHERE f_table_name LIKE '%s' AND f_geometry_column LIKE '%s'",
	     sqltable, sqlcolumn);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "updateTableTriggers: \"%s\"\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    for (i = 1; i <= rows; i++)
      {
	  /* preparing the triggers */
	  strcpy (tblname, results[(i * columns)]);
	  strcpy (colname, results[(i * columns) + 1]);
	  strcpy (col_type, results[(i * columns) + 2]);
	  /* 
	     / Even Rouault - 3 Mar 2010 
	     / the OGR driver wrongly inserts a NULL SRID
	     / into GEOMETRY_COLUMNS, so we must check such
	     / an odd condition to avoid a crash
	   */
	  if (results[(i * columns) + 3] == NULL)
	      strcpy (col_srid, "-1");
	  else
	      strcpy (col_srid, results[(i * columns) + 3]);
	  strcpy (col_index, results[(i * columns) + 4]);
	  strcpy (col_dims, results[(i * columns) + 5]);
	  if (atoi (col_index) == 1)
	      index = 1;
	  else
	      index = 0;
	  if (atoi (col_index) == 2)
	      cached = 1;
	  else
	      cached = 0;
	  dims = GAIA_XY;
	  if (strcasecmp (col_dims, "XYZ") == 0)
	      dims = GAIA_XY_Z;
	  if (strcasecmp (col_dims, "XYM") == 0)
	      dims = GAIA_XY_M;
	  if (strcasecmp (col_dims, "XYZM") == 0)
	      dims = GAIA_XY_Z_M;
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		txt_dims = "XYZ";
		break;
	    case GAIA_XY_M:
		txt_dims = "XYM";
		break;
	    case GAIA_XY_Z_M:
		txt_dims = "XYZM";
		break;
	    default:
		txt_dims = "XY";
		break;
	    };

	  /* trying to delete old versions [v2.0, v2.2] triggers[if any] */
	  strcpy (sqltable, (char *) tblname);
	  clean_sql_string (sqltable);
	  strcpy (sqlcolumn, (char *) colname);
	  clean_sql_string (sqlcolumn);
	  sprintf (xname, "gti_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  sprintf (xname, "gtu_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  sprintf (xname, "gsi_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  sprintf (xname, "gsu_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  /* end deletion old versions [v2.0, v2.2] triggers[if any] */

	  /* deleting the old INSERT trigger TYPE [if any] */
	  sprintf (xname, "ggi_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  /* inserting the new INSERT trigger TYPE */
	  strcpy (xtable, tblname);
	  double_quoted_sql (xtable);
	  strcpy (xcolname, colname);
	  double_quoted_sql (xcolname);
	  sprintf (trigger, "CREATE TRIGGER %s BEFORE INSERT ON %s\n", xname,
		   xtable);
	  strcat (trigger, "FOR EACH ROW BEGIN\n");
	  sprintf (dummy,
		   "SELECT RAISE(ROLLBACK, '%s.%s violates Geometry constraint [geom-type or SRID not allowed]')\n",
		   sqltable, sqlcolumn);
	  strcat (trigger, dummy);
	  strcat (trigger, "WHERE (SELECT type FROM geometry_columns\n");
	  sprintf (dummy,
		   "WHERE f_table_name = '%s' AND f_geometry_column = '%s'\n",
		   sqltable, sqlcolumn);
	  strcat (trigger, dummy);
	  sprintf (dummy,
		   "AND GeometryConstraints(NEW.%s, type, srid, '%s') = 1) IS NULL;\n",
		   xcolname, txt_dims);
	  strcat (trigger, dummy);
	  strcat (trigger, "END;");
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  /* deleting the old UPDATE trigger TYPE [if any] */
	  sprintf (xname, "ggu_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  /* inserting the new UPDATE trigger TYPE */
	  sprintf (trigger, "CREATE TRIGGER %s BEFORE UPDATE ON %s\n", xname,
		   xtable);
	  strcat (trigger, "FOR EACH ROW BEGIN\n");
	  sprintf (dummy,
		   "SELECT RAISE(ROLLBACK, '%s.%s violates Geometry constraint [geom-type or SRID not allowed]')\n",
		   sqltable, sqlcolumn);
	  strcat (trigger, dummy);
	  strcat (trigger, "WHERE (SELECT type FROM geometry_columns\n");
	  sprintf (dummy,
		   "WHERE f_table_name = '%s' AND f_geometry_column = '%s'\n",
		   sqltable, sqlcolumn);
	  strcat (trigger, dummy);
	  sprintf (dummy,
		   "AND GeometryConstraints(NEW.%s, type, srid, '%s') = 1) IS NULL;\n",
		   xcolname, txt_dims);
	  strcat (trigger, dummy);
	  strcat (trigger, "END;");
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  /* inserting SpatialIndex information into the linked list */
	  curr_idx = malloc (sizeof (struct spatial_index_str));
	  len = strlen (tblname);
	  curr_idx->TableName = malloc (len + 1);
	  strcpy (curr_idx->TableName, tblname);
	  len = strlen ((char *) colname);
	  curr_idx->ColumnName = malloc (len + 1);
	  strcpy (curr_idx->ColumnName, (char *) colname);
	  curr_idx->ValidRtree = (char) index;
	  curr_idx->ValidCache = (char) cached;
	  curr_idx->Next = NULL;
	  if (!first_idx)
	      first_idx = curr_idx;
	  if (last_idx)
	      last_idx->Next = curr_idx;
	  last_idx = curr_idx;
	  /* deleting the old INSERT trigger SPATIAL_INDEX [if any] */
	  sprintf (xname, "gii_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (index)
	    {
		/* inserting the new INSERT trigger SRID */
		sprintf (xindex, "idx_%s_%s", tblname, colname);
		double_quoted_sql (xindex);
		sprintf (trigger, "CREATE TRIGGER %s AFTER INSERT ON %s\n",
			 xname, xtable);
		strcat (trigger, "FOR EACH ROW BEGIN\n");
		sprintf (dummy, "DELETE FROM %s WHERE pkid=NEW.ROWID;\n",
			 xindex);
		strcat (trigger, dummy);
		sprintf (xindex, "idx_%s_%s", tblname, colname);
		clean_sql_string (xindex);
		sprintf (dummy, "SELECT RTreeAlign('%s', NEW.ROWID, NEW.%s);",
			 xindex, xcolname);
		strcat (trigger, dummy);
		strcat (trigger, "END;");
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
	    }
	  /* deleting the old UPDATE trigger SPATIAL_INDEX [if any] */
	  sprintf (xname, "giu_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (index)
	    {
		/* inserting the new UPDATE trigger SRID */
		sprintf (xindex, "idx_%s_%s", tblname, colname);
		double_quoted_sql (xindex);
		sprintf (trigger, "CREATE TRIGGER %s AFTER UPDATE ON %s\n",
			 xname, xtable);
		strcat (trigger, "FOR EACH ROW BEGIN\n");
		sprintf (dummy, "DELETE FROM %s WHERE pkid=NEW.ROWID;\n",
			 xindex);
		strcat (trigger, dummy);
		sprintf (xindex, "idx_%s_%s", tblname, colname);
		clean_sql_string (xindex);
		sprintf (dummy, "SELECT RTreeAlign('%s', NEW.ROWID, NEW.%s);",
			 xindex, xcolname);
		strcat (trigger, dummy);
		strcat (trigger, "END;");
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
	    }
	  /* deleting the old UPDATE trigger SPATIAL_INDEX [if any] */
	  sprintf (xname, "gid_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (index)
	    {
		/* inserting the new DELETE trigger SRID */
		sprintf (xindex, "idx_%s_%s", tblname, colname);
		double_quoted_sql (xindex);
		sprintf (trigger, "CREATE TRIGGER %s AFTER DELETE ON %s\n",
			 xname, xtable);
		strcat (trigger, "FOR EACH ROW BEGIN\n");
		sprintf (dummy, "DELETE FROM %s WHERE pkid = OLD.ROWID;\n",
			 xindex);
		strcat (trigger, dummy);
		strcat (trigger, "END;");
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
	    }
	  /* deleting the old INSERT trigger MBR_CACHE [if any] */
	  sprintf (xname, "gci_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (cached)
	    {
		/* inserting the new INSERT trigger SRID */
		sprintf (xindex, "cache_%s_%s", tblname, colname);
		double_quoted_sql (xindex);
		sprintf (trigger, "CREATE TRIGGER %s AFTER INSERT ON %s\n",
			 xname, xtable);
		strcat (trigger, "FOR EACH ROW BEGIN\n");
		sprintf (dummy,
			 "INSERT INTO %s (rowid, mbr) VALUES (NEW.ROWID,\nBuildMbrFilter(",
			 xindex);
		strcat (trigger, dummy);
		sprintf (dummy, "MbrMinX(NEW.%s), ", xcolname);
		strcat (trigger, dummy);
		sprintf (dummy, "MbrMinY(NEW.%s), ", xcolname);
		strcat (trigger, dummy);
		sprintf (dummy, "MbrMaxX(NEW.%s), ", xcolname);
		strcat (trigger, dummy);
		sprintf (dummy, "MbrMaxY(NEW.%s)));\n", xcolname);
		strcat (trigger, dummy);
		strcat (trigger, "END;");
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
	    }
	  /* deleting the old UPDATE trigger MBR_CACHE [if any] */
	  sprintf (xname, "gcu_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (cached)
	    {
		/* inserting the new UPDATE trigger SRID */
		sprintf (xindex, "cache_%s_%s", tblname, colname);
		double_quoted_sql (xindex);
		sprintf (trigger, "CREATE TRIGGER %s AFTER UPDATE ON %s\n",
			 xname, xtable);
		strcat (trigger, "FOR EACH ROW BEGIN\n");
		sprintf (dummy, "UPDATE %s SET ", xindex);
		strcat (trigger, dummy);
		sprintf (dummy, "mbr = BuildMbrFilter(MbrMinX(NEW.%s), ",
			 xcolname);
		strcat (trigger, dummy);
		sprintf (dummy, "MbrMinY(NEW.%s), ", xcolname);
		strcat (trigger, dummy);
		sprintf (dummy, "MbrMaxX(NEW.%s), ", xcolname);
		strcat (trigger, dummy);
		sprintf (dummy, "MbrMaxY(NEW.%s))\n", xcolname);
		strcat (trigger, dummy);
		strcat (trigger, "WHERE rowid = NEW.ROWID;\n");
		strcat (trigger, "END;");
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
	    }
	  /* deleting the old UPDATE trigger MBR_CACHE [if any] */
	  sprintf (xname, "gcd_%s_%s", tblname, colname);
	  double_quoted_sql (xname);
	  sprintf (trigger, "DROP TRIGGER IF EXISTS %s", xname);
	  ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
	  if (ret != SQLITE_OK)
	      goto error;
	  if (cached)
	    {
		/* inserting the new DELETE trigger SRID */
		sprintf (xindex, "cache_%s_%s", tblname, colname);
		double_quoted_sql (xindex);
		sprintf (trigger, "CREATE TRIGGER %s AFTER DELETE ON %s\n",
			 xname, xtable);
		strcat (trigger, "FOR EACH ROW BEGIN\n");
		sprintf (dummy, "DELETE FROM %s WHERE rowid = OLD.ROWID;\n",
			 xindex);
		strcat (trigger, dummy);
		strcat (trigger, "END;");
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
	    }
      }
    sqlite3_free_table (results);
/* now we'll adjust any related SpatialIndex as required */
    curr_idx = first_idx;
    while (curr_idx)
      {
	  if (curr_idx->ValidRtree)
	    {
		/* building RTree SpatialIndex */
		sprintf (xindex, "idx_%s_%s", curr_idx->TableName,
			 curr_idx->ColumnName);
		double_quoted_sql (xindex);
		sprintf (trigger, "CREATE VIRTUAL TABLE %s USING rtree(\n",
			 xindex);
		strcat (trigger, "pkid, xmin, xmax, ymin, ymax)");
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
		buildSpatialIndex (sqlite,
				   (unsigned char *) (curr_idx->TableName),
				   curr_idx->ColumnName);
	    }
	  if (curr_idx->ValidCache)
	    {
		/* building MbrCache SpatialIndex */
		sprintf (xindex, "cache_%s_%s", curr_idx->TableName,
			 curr_idx->ColumnName);
		double_quoted_sql (xindex);
		strcpy (xtable, curr_idx->TableName);
		double_quoted_sql (xtable);
		strcpy (xcolname, curr_idx->ColumnName);
		double_quoted_sql (xcolname);
		sprintf (trigger,
			 "CREATE VIRTUAL TABLE %s USING MbrCache(%s, %s)\n",
			 xindex, xtable, xcolname);
		ret = sqlite3_exec (sqlite, trigger, NULL, NULL, &errMsg);
		if (ret != SQLITE_OK)
		    goto error;
	    }
	  curr_idx = curr_idx->Next;
      }
    goto index_cleanup;
  error:
    fprintf (stderr, "updateTableTriggers: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
  index_cleanup:
    curr_idx = first_idx;
    while (curr_idx)
      {
	  next_idx = curr_idx->Next;
	  if (curr_idx->TableName)
	      free (curr_idx->TableName);
	  if (curr_idx->ColumnName)
	      free (curr_idx->ColumnName);
	  free (curr_idx);
	  curr_idx = next_idx;
      }
}

static void
fnct_AddGeometryColumn (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ AddGeometryColumn(table, column, srid, type , dimension  [  , not-null ]  )
/
/ creates a new COLUMN of given TYPE into TABLE
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    const unsigned char *type;
    const unsigned char *txt_dims;
    int xtype;
    int srid = -1;
    int dimension = 2;
    int dims = -1;
    char dummy[32];
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    char tblname[256];
    char xtable[1024];
    char xcolumn[1024];
    char sqltable[1024];
    char sqlcolumn[1024];
    int notNull = 0;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    if (sqlite3_value_type (argv[2]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: argument 3 [SRID] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    srid = sqlite3_value_int (argv[2]);
    if (sqlite3_value_type (argv[3]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: argument 4 [geometry_type] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    type = sqlite3_value_text (argv[3]);
    if (sqlite3_value_type (argv[4]) == SQLITE_INTEGER)
      {
	  dimension = sqlite3_value_int (argv[4]);
	  if (dimension == 2)
	      dims = GAIA_XY;
	  if (dimension == 3)
	      dims = GAIA_XY_Z;
      }
    else if (sqlite3_value_type (argv[4]) == SQLITE_TEXT)
      {
	  txt_dims = sqlite3_value_text (argv[4]);
	  if (strcasecmp ((char *) txt_dims, "XY") == 0)
	      dims = GAIA_XY;
	  if (strcasecmp ((char *) txt_dims, "XYZ") == 0)
	      dims = GAIA_XY_Z;
	  if (strcasecmp ((char *) txt_dims, "XYM") == 0)
	      dims = GAIA_XY_M;
	  if (strcasecmp ((char *) txt_dims, "XYZM") == 0)
	      dims = GAIA_XY_Z_M;
      }
    else
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: argument 5 [dimension] is not of the Integer or Text type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (argc > 5)
      {
	  /* optional NOT NULL arg */
	  if (sqlite3_value_type (argv[5]) != SQLITE_INTEGER)
	    {
		fprintf (stderr,
			 "AddGeometryColumn() error: argument 6 [not null] is not of the Integer type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
	  notNull = sqlite3_value_int (argv[5]);
      }
    xtype = GAIA_UNKNOWN;
    if (strcasecmp ((char *) type, "POINT") == 0)
	xtype = GAIA_POINT;
    if (strcasecmp ((char *) type, "LINESTRING") == 0)
	xtype = GAIA_LINESTRING;
    if (strcasecmp ((char *) type, "POLYGON") == 0)
	xtype = GAIA_POLYGON;
    if (strcasecmp ((char *) type, "MULTIPOINT") == 0)
	xtype = GAIA_MULTIPOINT;
    if (strcasecmp ((char *) type, "MULTILINESTRING") == 0)
	xtype = GAIA_MULTILINESTRING;
    if (strcasecmp ((char *) type, "MULTIPOLYGON") == 0)
	xtype = GAIA_MULTIPOLYGON;
    if (strcasecmp ((char *) type, "GEOMETRYCOLLECTION") == 0)
	xtype = GAIA_GEOMETRYCOLLECTION;
    if (strcasecmp ((char *) type, "GEOMETRY") == 0)
	xtype = -1;
    if (xtype == GAIA_UNKNOWN)
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: argument 4 [geometry_type] has an illegal value\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (dims == GAIA_XY || dims == GAIA_XY_Z || dims == GAIA_XY_M
	|| dims == GAIA_XY_Z_M)
	;
    else
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: argument 5 [dimension] ILLEGAL VALUE\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
/* checking if the table exists */
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE '%s'",
	     sqltable);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "AddGeometryColumn: \"%s\"\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    *tblname = '\0';
    for (i = 1; i <= rows; i++)
	strcpy (tblname, results[(i * columns)]);
    sqlite3_free_table (results);
    if (*tblname == '\0')
      {
	  fprintf (stderr,
		   "AddGeometryColumn() error: table '%s' does not exist\n",
		   table);
	  sqlite3_result_int (context, 0);
	  return;
      }
/* trying to add the column */
    strcpy (xtable, (char *) table);
    double_quoted_sql (xtable);
    strcpy (xcolumn, (char *) column);
    double_quoted_sql (xcolumn);
    strcpy (sql, "ALTER TABLE ");
    strcat (sql, xtable);
    strcat (sql, " ADD COLUMN ");
    strcat (sql, xcolumn);
    strcat (sql, " ");
    switch (xtype)
      {
      case GAIA_POINT:
	  strcat (sql, "POINT");
	  break;
      case GAIA_LINESTRING:
	  strcat (sql, "LINESTRING");
	  break;
      case GAIA_POLYGON:
	  strcat (sql, "POLYGON");
	  break;
      case GAIA_MULTIPOINT:
	  strcat (sql, "MULTIPOINT");
	  break;
      case GAIA_MULTILINESTRING:
	  strcat (sql, "MULTILINESTRING");
	  break;
      case GAIA_MULTIPOLYGON:
	  strcat (sql, "MULTIPOLYGON");
	  break;
      case GAIA_GEOMETRYCOLLECTION:
	  strcat (sql, "GEOMETRYCOLLECTION");
	  break;
      case -1:
	  strcat (sql, "GEOMETRY");
	  break;
      };
    if (notNull)
      {
	  /* adding a NOT NULL clause */
	  strcat (sql, " NOT NULL DEFAULT ''");
      }
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
/*ok, inserting into geometry_columns [Spatial Metadata] */
    strcpy (sql,
	    "INSERT INTO geometry_columns (f_table_name, f_geometry_column, type, ");
    strcat (sql, "coord_dimension, srid, spatial_index_enabled) VALUES (");
    strcat (sql, "'");
    strcat (sql, sqltable);
    strcat (sql, "', '");
    strcat (sql, sqlcolumn);
    strcat (sql, "', '");
    switch (xtype)
      {
      case GAIA_POINT:
	  strcat (sql, "POINT");
	  break;
      case GAIA_LINESTRING:
	  strcat (sql, "LINESTRING");
	  break;
      case GAIA_POLYGON:
	  strcat (sql, "POLYGON");
	  break;
      case GAIA_MULTIPOINT:
	  strcat (sql, "MULTIPOINT");
	  break;
      case GAIA_MULTILINESTRING:
	  strcat (sql, "MULTILINESTRING");
	  break;
      case GAIA_MULTIPOLYGON:
	  strcat (sql, "MULTIPOLYGON");
	  break;
      case GAIA_GEOMETRYCOLLECTION:
	  strcat (sql, "GEOMETRYCOLLECTION");
	  break;
      case -1:
	  strcat (sql, "GEOMETRY");
	  break;
      };
    strcat (sql, "', '");
    switch (dims)
      {
      case GAIA_XY:
	  strcat (sql, "XY");
	  break;
      case GAIA_XY_Z:
	  strcat (sql, "XYZ");
	  break;
      case GAIA_XY_M:
	  strcat (sql, "XYM");
	  break;
      case GAIA_XY_Z_M:
	  strcat (sql, "XYZM");
	  break;
      };
    strcat (sql, "', ");
    if (srid <= 0)
	strcat (sql, "-1");
    else
      {
	  sprintf (dummy, "%d", srid);
	  strcat (sql, dummy);
      }
    strcat (sql, ", 0)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    updateGeometryTriggers (sqlite, table, column);
    sqlite3_result_int (context, 1);
    strcpy (sql, "Geometry [");
    switch (xtype)
      {
      case GAIA_POINT:
	  strcat (sql, "POINT");
	  break;
      case GAIA_LINESTRING:
	  strcat (sql, "LINESTRING");
	  break;
      case GAIA_POLYGON:
	  strcat (sql, "POLYGON");
	  break;
      case GAIA_MULTIPOINT:
	  strcat (sql, "MULTIPOINT");
	  break;
      case GAIA_MULTILINESTRING:
	  strcat (sql, "MULTILINESTRING");
	  break;
      case GAIA_MULTIPOLYGON:
	  strcat (sql, "MULTIPOLYGON");
	  break;
      case GAIA_GEOMETRYCOLLECTION:
	  strcat (sql, "GEOMETRYCOLLECTION");
	  break;
      case -1:
	  strcat (sql, "GEOMETRY");
	  break;
      };
    strcat (sql, ",");
    switch (dims)
      {
      case GAIA_XY:
	  strcat (sql, "XY");
	  break;
      case GAIA_XY_Z:
	  strcat (sql, "XYZ");
	  break;
      case GAIA_XY_M:
	  strcat (sql, "XYM");
	  break;
      case GAIA_XY_Z_M:
	  strcat (sql, "XYZM");
	  break;
      };
    sprintf (sqlcolumn, ",SRID=%d", (srid <= 0) ? -1 : srid);
    strcat (sql, sqlcolumn);
    strcat (sql, "] successfully created");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    fprintf (stderr, "AddGeometryColumn() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_RecoverGeometryColumn (sqlite3_context * context, int argc,
			    sqlite3_value ** argv)
{
/* SQL function:
/ RecoverGeometryColumn(table, column, srid, type , dimension )
/
/ checks if an existing TABLE.COLUMN satisfies the required geometric features
/ if yes adds it to SpatialMetaData and enabling triggers
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    const unsigned char *type;
    int xtype;
    int xxtype;
    int srid = -1;
    const unsigned char *txt_dims;
    int dimension = 2;
    int dims = -1;
    char dummy[32];
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    char tblname[256];
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    if (sqlite3_value_type (argv[2]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: argument 3 [SRID] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    srid = sqlite3_value_int (argv[2]);
    if (sqlite3_value_type (argv[3]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: argument 4 [geometry_type] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    type = sqlite3_value_text (argv[3]);
    if (sqlite3_value_type (argv[4]) == SQLITE_INTEGER)
      {
	  dimension = sqlite3_value_int (argv[4]);
	  if (dimension == 2)
	      dims = GAIA_XY;
	  if (dimension == 3)
	      dims = GAIA_XY_Z;
      }
    else if (sqlite3_value_type (argv[4]) == SQLITE_TEXT)
      {
	  txt_dims = sqlite3_value_text (argv[4]);
	  if (strcasecmp ((char *) txt_dims, "XY") == 0)
	      dims = GAIA_XY;
	  if (strcasecmp ((char *) txt_dims, "XYZ") == 0)
	      dims = GAIA_XY_Z;
	  if (strcasecmp ((char *) txt_dims, "XYM") == 0)
	      dims = GAIA_XY_M;
	  if (strcasecmp ((char *) txt_dims, "XYZM") == 0)
	      dims = GAIA_XY_Z_M;
      }
    else
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: argument 5 [dimension] is not of the Integer or Text type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    xtype = GAIA_UNKNOWN;
    if (strcasecmp ((char *) type, "POINT") == 0)
	xtype = GAIA_POINT;
    if (strcasecmp ((char *) type, "LINESTRING") == 0)
	xtype = GAIA_LINESTRING;
    if (strcasecmp ((char *) type, "POLYGON") == 0)
	xtype = GAIA_POLYGON;
    if (strcasecmp ((char *) type, "MULTIPOINT") == 0)
	xtype = GAIA_MULTIPOINT;
    if (strcasecmp ((char *) type, "MULTILINESTRING") == 0)
	xtype = GAIA_MULTILINESTRING;
    if (strcasecmp ((char *) type, "MULTIPOLYGON") == 0)
	xtype = GAIA_MULTIPOLYGON;
    if (strcasecmp ((char *) type, "GEOMETRYCOLLECTION") == 0)
	xtype = GAIA_GEOMETRYCOLLECTION;
    if (strcasecmp ((char *) type, "GEOMETRY") == 0)
	xtype = -1;
    if (xtype == GAIA_UNKNOWN)
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: argument 4 [geometry_type] has an illegal value\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (dims == GAIA_XY || dims == GAIA_XY_Z || dims == GAIA_XY_M
	|| dims == GAIA_XY_Z_M)
	;
    else
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: argument 5 [dimension] ILLEGAL VALUE\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
/* checking if the table exists */
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE '%s'",
	     sqltable);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "RecoverGeometryColumn: \"%s\"\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    *tblname = '\0';
    for (i = 1; i <= rows; i++)
      {
	  /* preparing the triggers */
	  strcpy (tblname, results[(i * columns)]);
      }
    sqlite3_free_table (results);
    if (*tblname == '\0')
      {
	  fprintf (stderr,
		   "RecoverGeometryColumn() error: table '%s' does not exist\n",
		   table);
	  sqlite3_result_int (context, 0);
	  return;
      }
/* adjusting the actual GeometryType */
    xxtype = xtype;
    xtype = GAIA_UNKNOWN;
    if (xxtype == GAIA_POINT)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_POINTZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_POINTM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_POINTZM;
		break;
	    default:
		xtype = GAIA_POINT;
		break;
	    };
      }
    if (xxtype == GAIA_LINESTRING)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_LINESTRINGZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_LINESTRINGM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_LINESTRINGZM;
		break;
	    default:
		xtype = GAIA_LINESTRING;
		break;
	    };
      }
    if (xxtype == GAIA_POLYGON)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_POLYGONZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_POLYGONM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_POLYGONZM;
		break;
	    default:
		xtype = GAIA_POLYGON;
		break;
	    };
      }
    if (xxtype == GAIA_MULTIPOINT)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_MULTIPOINTZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_MULTIPOINTM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_MULTIPOINTZM;
		break;
	    default:
		xtype = GAIA_MULTIPOINT;
		break;
	    };
      }
    if (xxtype == GAIA_MULTILINESTRING)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_MULTILINESTRINGZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_MULTILINESTRINGM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_MULTILINESTRINGZM;
		break;
	    default:
		xtype = GAIA_MULTILINESTRING;
		break;
	    };
      }
    if (xxtype == GAIA_MULTIPOLYGON)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_MULTIPOLYGONZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_MULTIPOLYGONM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_MULTIPOLYGONZM;
		break;
	    default:
		xtype = GAIA_MULTIPOLYGON;
		break;
	    };
      }
    if (xxtype == GAIA_GEOMETRYCOLLECTION)
      {
	  switch (dims)
	    {
	    case GAIA_XY_Z:
		xtype = GAIA_GEOMETRYCOLLECTIONZ;
		break;
	    case GAIA_XY_M:
		xtype = GAIA_GEOMETRYCOLLECTIONM;
		break;
	    case GAIA_XY_Z_M:
		xtype = GAIA_GEOMETRYCOLLECTIONZM;
		break;
	    default:
		xtype = GAIA_GEOMETRYCOLLECTION;
		break;
	    };
      }
    if (xxtype == -1)
	xtype = -1;		/* GEOMETRY */
    if (!recoverGeomColumn (sqlite, table, column, xtype, dims, srid))
      {
	  fprintf (stderr, "RecoverGeometryColumn(): validation failed\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    strcpy (sql,
	    "INSERT INTO geometry_columns (f_table_name, f_geometry_column, type, ");
    strcat (sql, "coord_dimension, srid, spatial_index_enabled) VALUES (");
    strcat (sql, "'");
    strcat (sql, sqltable);
    strcat (sql, "', '");
    strcat (sql, sqlcolumn);
    strcat (sql, "', '");
    switch (xtype)
      {
      case GAIA_POINT:
      case GAIA_POINTZ:
      case GAIA_POINTM:
      case GAIA_POINTZM:
	  strcat (sql, "POINT");
	  break;
      case GAIA_LINESTRING:
      case GAIA_LINESTRINGZ:
      case GAIA_LINESTRINGM:
      case GAIA_LINESTRINGZM:
	  strcat (sql, "LINESTRING");
	  break;
      case GAIA_POLYGON:
      case GAIA_POLYGONZ:
      case GAIA_POLYGONM:
      case GAIA_POLYGONZM:
	  strcat (sql, "POLYGON");
	  break;
      case GAIA_MULTIPOINT:
      case GAIA_MULTIPOINTZ:
      case GAIA_MULTIPOINTM:
      case GAIA_MULTIPOINTZM:
	  strcat (sql, "MULTIPOINT");
	  break;
      case GAIA_MULTILINESTRING:
      case GAIA_MULTILINESTRINGZ:
      case GAIA_MULTILINESTRINGM:
      case GAIA_MULTILINESTRINGZM:
	  strcat (sql, "MULTILINESTRING");
	  break;
      case GAIA_MULTIPOLYGON:
      case GAIA_MULTIPOLYGONZ:
      case GAIA_MULTIPOLYGONM:
      case GAIA_MULTIPOLYGONZM:
	  strcat (sql, "MULTIPOLYGON");
	  break;
      case GAIA_GEOMETRYCOLLECTION:
      case GAIA_GEOMETRYCOLLECTIONZ:
      case GAIA_GEOMETRYCOLLECTIONM:
      case GAIA_GEOMETRYCOLLECTIONZM:
	  strcat (sql, "GEOMETRYCOLLECTION");
	  break;
      case -1:
	  strcat (sql, "GEOMETRY");
	  break;
      };
    strcat (sql, "', '");
    switch (dims)
      {
      case GAIA_XY:
	  strcat (sql, "XY");
	  break;
      case GAIA_XY_Z:
	  strcat (sql, "XYZ");
	  break;
      case GAIA_XY_M:
	  strcat (sql, "XYM");
	  break;
      case GAIA_XY_Z_M:
	  strcat (sql, "XYZM");
	  break;
      };
    strcat (sql, "', ");
    if (srid <= 0)
	strcat (sql, "-1");
    else
      {
	  sprintf (dummy, "%d", srid);
	  strcat (sql, dummy);
      }
    strcat (sql, ", 0)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    updateGeometryTriggers (sqlite, table, column);
    sqlite3_result_int (context, 1);
    strcpy (sql, "Geometry [");
    switch (xtype)
      {
      case GAIA_POINT:
	  strcat (sql, "POINT");
	  break;
      case GAIA_LINESTRING:
	  strcat (sql, "LINESTRING");
	  break;
      case GAIA_POLYGON:
	  strcat (sql, "POLYGON");
	  break;
      case GAIA_MULTIPOINT:
	  strcat (sql, "MULTIPOINT");
	  break;
      case GAIA_MULTILINESTRING:
	  strcat (sql, "MULTILINESTRING");
	  break;
      case GAIA_MULTIPOLYGON:
	  strcat (sql, "MULTIPOLYGON");
	  break;
      case GAIA_GEOMETRYCOLLECTION:
	  strcat (sql, "GEOMETRYCOLLECTION");
	  break;
      case -1:
	  strcat (sql, "GEOMETRY");
	  break;
      };
    strcat (sql, ",");
    switch (dims)
      {
      case GAIA_XY:
	  strcat (sql, "XY");
	  break;
      case GAIA_XY_Z:
	  strcat (sql, "XYZ");
	  break;
      case GAIA_XY_M:
	  strcat (sql, "XYM");
	  break;
      case GAIA_XY_Z_M:
	  strcat (sql, "XYZM");
	  break;
      };
    sprintf (sqlcolumn, ",SRID=%d", (srid <= 0) ? -1 : srid);
    strcat (sql, sqlcolumn);
    strcat (sql, "] successfully recovered");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    fprintf (stderr, "RecoverGeometryColumn() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_DiscardGeometryColumn (sqlite3_context * context, int argc,
			    sqlite3_value ** argv)
{
/* SQL function:
/ DiscardGeometryColumn(table, column)
/
/ removes TABLE.COLUMN from the Spatial MetaData [thus disabling triggers too]
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char xname[1024];
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "DiscardGeometryColumn() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "DiscardGeometryColumn() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    sprintf (sql,
	     "DELETE FROM geometry_columns WHERE f_table_name LIKE '%s' AND f_geometry_column LIKE '%s'",
	     sqltable, sqlcolumn);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
/* removing triggers too */
    sprintf (xname, "ggi_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "ggu_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gii_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "giu_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gid_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gci_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gcu_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gcd_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;

    /* trying to delete old versions [v2.0, v2.2] triggers[if any] */
    sprintf (xname, "gti_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gtu_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gsi_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sprintf (xname, "gsu_%s_%s", (char *) table, (char *) column);
    double_quoted_sql (xname);
    sprintf (sql, "DROP TRIGGER IF EXISTS %s", xname);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    /* end deletion old versions [v2.0, v2.2] triggers[if any] */

    sqlite3_result_int (context, 1);
    strcpy (sql, "Geometry successfully discarded");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    fprintf (stderr, "DiscardGeometryColumn() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_InitFDOSpatialMetaData (sqlite3_context * context, int argc,
			     sqlite3_value ** argv)
{
/* SQL function:
/ InitFDOSpatialMetaData(void)
/
/ creates the FDO-styled SPATIAL_REF_SYS and GEOMETRY_COLUMNS tables
/ returns 1 on success
/ 0 on failure
*/
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
/* creating the SPATIAL_REF_SYS tables */
    strcpy (sql, "CREATE TABLE spatial_ref_sys (\n");
    strcat (sql, "srid INTEGER PRIMARY KEY,\n");
    strcat (sql, "auth_name TEXT,\n");
    strcat (sql, "auth_srid INTEGER,\n");
    strcat (sql, "srtext TEXT)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
/* creating the GEOMETRY_COLUMN tables */
    strcpy (sql, "CREATE TABLE geometry_columns (\n");
    strcat (sql, "f_table_name TEXT,\n");
    strcat (sql, "f_geometry_column TEXT,\n");
    strcat (sql, "geometry_type INTEGER,\n");
    strcat (sql, "coord_dimension INTEGER,\n");
    strcat (sql, "srid INTEGER,\n");
    strcat (sql, "geometry_format TEXT)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sqlite3_result_int (context, 1);
    return;
  error:fprintf (stderr, "InitFDOSpatiaMetaData() error: \"%s\"\n",
	     errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static int
recoverFDOGeomColumn (sqlite3 * sqlite, const unsigned char *table,
		      const unsigned char *column, int xtype, int srid)
{
/* checks if TABLE.COLUMN exists and has the required features */
    int ok = 1;
    char sql[1024];
    int type;
    sqlite3_stmt *stmt;
    gaiaGeomCollPtr geom;
    const void *blob_value;
    int len;
    int ret;
    int i_col;
    char xcolumn[1024];
    char xtable[1024];
    strcpy (xcolumn, (char *) column);
    double_quoted_sql (xcolumn);
    strcpy (xtable, (char *) table);
    double_quoted_sql (xtable);
    sprintf (sql, "SELECT %s FROM %s", xcolumn, xtable);
/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "recoverFDOGeomColumn: error %d \"%s\"\n",
		   sqlite3_errcode (sqlite), sqlite3_errmsg (sqlite));
	  return 0;
      }
    while (1)
      {
	  /* scrolling the result set rows */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* checking Geometry features */
		geom = NULL;
		for (i_col = 0; i_col < sqlite3_column_count (stmt); i_col++)
		  {
		      if (sqlite3_column_type (stmt, i_col) != SQLITE_BLOB)
			  ok = 0;
		      else
			{
			    blob_value = sqlite3_column_blob (stmt, i_col);
			    len = sqlite3_column_bytes (stmt, i_col);
			    geom = gaiaFromSpatiaLiteBlobWkb (blob_value, len);
			    if (!geom)
				ok = 0;
			    else
			      {
				  if (geom->Srid != srid)
				      ok = 0;
				  type = gaiaGeometryType (geom);
				  if (xtype == type)
				      ;
				  else
				      ok = 0;
				  gaiaFreeGeomColl (geom);
			      }
			}
		  }
	    }
	  if (!ok)
	      break;
      }
    ret = sqlite3_finalize (stmt);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "recoverFDOGeomColumn: error %d \"%s\"\n",
		   sqlite3_errcode (sqlite), sqlite3_errmsg (sqlite));
	  return 0;
      }
    return ok;
}

static void
fnct_AddFDOGeometryColumn (sqlite3_context * context, int argc,
			   sqlite3_value ** argv)
{
/* SQL function:
/ AddFDOGeometryColumn(table, column, srid, geometry_type , dimension, geometry_format )
/
/ creates a new COLUMN of given TYPE into TABLE
/ returns 1 on success
/ 0 on failure
*/
    const char *table;
    const char *column;
    const char *format;
    char xformat[64];
    int type;
    int srid = -1;
    int dimension = 2;
    char dummy[32];
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    char tblname[256];
    char xtable[1024];
    char xcolumn[1024];
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = (const char *) sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = (const char *) sqlite3_value_text (argv[1]);
    if (sqlite3_value_type (argv[2]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 3 [SRID] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    srid = sqlite3_value_int (argv[2]);
    if (sqlite3_value_type (argv[3]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 4 [geometry_type] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    type = sqlite3_value_int (argv[3]);
    if (sqlite3_value_type (argv[4]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 5 [dimension] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    dimension = sqlite3_value_int (argv[4]);
    if (sqlite3_value_type (argv[5]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 6 [geometry_format] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    format = (const char *) sqlite3_value_text (argv[5]);
    if (type ==
	GAIA_POINT
	|| type ==
	GAIA_LINESTRING
	|| type ==
	GAIA_POLYGON
	|| type ==
	GAIA_MULTIPOINT
	|| type ==
	GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	;
    else
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 4 [geometry_type] has an illegal value\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (dimension < 2 || dimension > 4)
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 5 [dimension] current version only accepts dimension=2,3,4\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (strcasecmp (format, "WKT") == 0)
	strcpy (xformat, "WKT");
    else if (strcasecmp (format, "WKB") == 0)
	strcpy (xformat, "WKB");
    else if (strcasecmp (format, "FGF") == 0)
	strcpy (xformat, "FGF");
    else
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: argument 6 [geometry_format] has to be one of: WKT,WKB,FGF\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
/* checking if the table exists */
    strcpy (xtable, (char *) table);
    double_quoted_sql (xtable);
    strcpy (xcolumn, (char *) column);
    double_quoted_sql (xcolumn);
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE '%s'",
	     sqltable);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "AddFDOGeometryColumn: \"%s\"\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    *tblname = '\0';
    for (i = 1; i <= rows; i++)
      {
	  strcpy (tblname, results[(i * columns)]);
      }
    sqlite3_free_table (results);
    if (*tblname == '\0')
      {
	  fprintf (stderr,
		   "AddFDOGeometryColumn() error: table '%s' does not exist\n",
		   table);
	  sqlite3_result_int (context, 0);
	  return;
      }
/* trying to add the column */
    strcpy (sql, "ALTER TABLE ");
    strcat (sql, xtable);
    strcat (sql, " ADD COLUMN ");
    strcat (sql, xcolumn);
    strcat (sql, " BLOB");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
/*ok, inserting into geometry_columns [FDO Spatial Metadata] */
    strcpy (sql,
	    "INSERT INTO geometry_columns (f_table_name, f_geometry_column, geometry_type, ");
    strcat (sql, "coord_dimension, srid, geometry_format) VALUES (");
    strcat (sql, "'");
    strcat (sql, sqltable);
    strcat (sql, "', '");
    strcat (sql, sqlcolumn);
    strcat (sql, "', ");
    sprintf (dummy, "%d, %d, ", type, dimension);
    strcat (sql, dummy);
    if (srid <= 0)
	strcat (sql, "-1");
    else
      {
	  sprintf (dummy, "%d", srid);
	  strcat (sql, dummy);
      }
    strcat (sql, ", '");
    strcat (sql, xformat);
    strcat (sql, "')");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sqlite3_result_int (context, 1);
    return;
  error:
    fprintf (stderr, "AddFDOGeometryColumn() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_RecoverFDOGeometryColumn (sqlite3_context * context, int argc,
			       sqlite3_value ** argv)
{
/* SQL function:
/ RecoverFDOGeometryColumn(table, column, srid, geometry_type , dimension, geometry_format )
/
/ checks if an existing TABLE.COLUMN satisfies the required geometric features
/ if yes adds it to FDO-styled SpatialMetaData 
/ returns 1 on success
/ 0 on failure
*/
    const char *table;
    const char *column;
    const char *format;
    char xformat[64];
    int type;
    int srid = -1;
    int dimension = 2;
    char dummy[32];
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    char tblname[256];
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = (const char *) sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = (const char *) sqlite3_value_text (argv[1]);
    if (sqlite3_value_type (argv[2]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 3 [SRID] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    srid = sqlite3_value_int (argv[2]);
    if (sqlite3_value_type (argv[3]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 4 [geometry_type] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    type = sqlite3_value_int (argv[3]);
    if (sqlite3_value_type (argv[4]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 5 [dimension] is not of the Integer type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    dimension = sqlite3_value_int (argv[4]);
    if (sqlite3_value_type (argv[5]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 6 [geometry_format] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    format = (const char *) sqlite3_value_text (argv[5]);
    if (type ==
	GAIA_POINT
	|| type ==
	GAIA_LINESTRING
	|| type ==
	GAIA_POLYGON
	|| type ==
	GAIA_MULTIPOINT
	|| type ==
	GAIA_MULTILINESTRING
	|| type == GAIA_MULTIPOLYGON || type == GAIA_GEOMETRYCOLLECTION)
	;
    else
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 4 [geometry_type] has an illegal value\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (dimension < 2 || dimension > 4)
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 5 [dimension] current version only accepts dimension=2,3,4\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (strcasecmp (format, "WKT") == 0)
	strcpy (xformat, "WKT");
    else if (strcasecmp (format, "WKB") == 0)
	strcpy (xformat, "WKB");
    else if (strcasecmp (format, "FGF") == 0)
	strcpy (xformat, "FGF");
    else
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: argument 6 [geometry_format] has to be one of: WKT,WKB,FGF\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
/* checking if the table exists */
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE '%s'",
	     sqltable);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "RecoverFDOGeometryColumn: \"%s\"\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    *tblname = '\0';
    for (i = 1; i <= rows; i++)
      {
	  strcpy (tblname, results[(i * columns)]);
      }
    sqlite3_free_table (results);
    if (*tblname == '\0')
      {
	  fprintf (stderr,
		   "RecoverFDOGeometryColumn() error: table '%s' does not exist\n",
		   table);
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (!recoverFDOGeomColumn
	(sqlite, (const unsigned char *) table,
	 (const unsigned char *) column, type, srid))
      {
	  fprintf (stderr, "RecoverFDOGeometryColumn(): validation failed\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    strcpy (sqltable, (char *) tblname);
    clean_sql_string (sqltable);
    strcpy (sql,
	    "INSERT INTO geometry_columns (f_table_name, f_geometry_column, geometry_type, ");
    strcat (sql, "coord_dimension, srid, geometry_format) VALUES (");
    strcat (sql, "'");
    strcat (sql, sqltable);
    strcat (sql, "', '");
    strcat (sql, sqlcolumn);
    strcat (sql, "', ");
    sprintf (dummy, "%d, %d, ", type, dimension);
    strcat (sql, dummy);
    if (srid <= 0)
	strcat (sql, "-1");
    else
      {
	  sprintf (dummy, "%d", srid);
	  strcat (sql, dummy);
      }
    strcat (sql, ", '");
    strcat (sql, xformat);
    strcat (sql, "')");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sqlite3_result_int (context, 1);
    return;
  error:
    fprintf (stderr, "RecoverFDOGeometryColumn() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_DiscardFDOGeometryColumn (sqlite3_context * context, int argc,
			       sqlite3_value ** argv)
{
/* SQL function:
/ DiscardFDOGeometryColumn(table, column)
/
/ removes TABLE.COLUMN from the Spatial MetaData
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "DiscardFDOGeometryColumn() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "DiscardFDOGeometryColumn() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    sprintf (sql,
	     "DELETE FROM geometry_columns WHERE f_table_name LIKE '%s' AND f_geometry_column LIKE '%s'",
	     sqltable, sqlcolumn);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    sqlite3_result_int (context, 1);
    return;
  error:
    fprintf (stderr, "DiscardFDOGeometryColumn() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static int
eval_rtree_entry (int ok_geom, double geom_value, int ok_rtree,
		  double rtree_value)
{
/* evaluating geom-coord and rtree-coord */
    if (!ok_geom && !ok_rtree)
	return 1;
    if (ok_geom && ok_rtree)
      {
	  float g = (float) geom_value;
	  float r = (float) rtree_value;
	  if (g != r)
	      return 0;
	  return 1;
      }
    return 0;
}

static int
check_spatial_index (sqlite3 * sqlite, const unsigned char *table,
		     const unsigned char *geom)
{
/* attempting to check an R*Tree for consistency */
    char xtable[1024];
    char xgeom[1024];
    char idx_name[2048];
    char sql[8192];
    char sql2[2048];
    int ret;
    int is_defined = 0;
    sqlite3_stmt *stmt;
    sqlite3_int64 count_geom;
    sqlite3_int64 count_rtree;
    double g_xmin;
    double g_ymin;
    double g_xmax;
    double g_ymax;
    int ok_g_xmin;
    int ok_g_ymin;
    int ok_g_xmax;
    int ok_g_ymax;
    double i_xmin;
    double i_ymin;
    double i_xmax;
    double i_ymax;
    int ok_i_xmin;
    int ok_i_ymin;
    int ok_i_xmax;
    int ok_i_ymax;

/* checking if the R*Tree Spatial Index is defined */
    strcpy (xtable, (const char *) table);
    clean_sql_string (xtable);
    strcpy (xgeom, (const char *) geom);
    clean_sql_string (xgeom);
    strcpy (sql, "SELECT Count(*) FROM geometry_columns ");
    sprintf (sql2, "WHERE f_table_name LIKE '%s' ", xtable);
    strcat (sql, sql2);
    sprintf (sql2, "AND f_geometry_column LIKE '%s' ", xgeom);
    strcat (sql, sql2);
    strcat (sql, "AND spatial_index_enabled = 1");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CheckSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	      is_defined = sqlite3_column_int (stmt, 0);
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
    sqlite3_finalize (stmt);
    if (!is_defined)
	return -1;

    sprintf (xgeom, "%s", geom);
    double_quoted_sql (xgeom);
    strcpy (xtable, (const char *) table);
    double_quoted_sql (xtable);
    sprintf (idx_name, "idx_%s_%s", table, geom);
    double_quoted_sql (idx_name);

/* counting how many Geometries are set into the main-table */
    sprintf (sql, "SELECT Count(*) FROM %s ", xtable);
    sprintf (sql2, "WHERE ST_GeometryType(%s) IS NOT NULL", xgeom);
    strcat (sql, sql2);
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CheckSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	      count_geom = sqlite3_column_int (stmt, 0);
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
    sqlite3_finalize (stmt);

/* counting how many R*Tree entries are defined */
    sprintf (sql, "SELECT Count(*) FROM %s", idx_name);
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CheckSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	      count_rtree = sqlite3_column_int (stmt, 0);
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
    sqlite3_finalize (stmt);
    if (count_geom != count_rtree)
      {
	  /* unexpected count difference */
	  return 0;
      }

/* checking the geometry-table against the corresponding R*Tree */
    sprintf (sql, "SELECT ");
    sprintf (sql2, "MbrMinX(g.%s), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "MbrMinY(g.%s), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "MbrMaxX(g.%s), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "MbrMaxY(g.%s), ", xgeom);
    strcat (sql, sql2);
    strcat (sql, "i.xmin, i.ymin, i.xmax, i.ymax\n");
    sprintf (sql2, "FROM %s AS g\n", xtable);
    strcat (sql, sql2);
    sprintf (sql2, "LEFT JOIN %s AS i ", idx_name);
    strcat (sql, sql2);
    strcat (sql, "ON (g.ROWID = i.pkid)");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CheckSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* checking a row */
		ok_g_xmin = 1;
		ok_g_ymin = 1;
		ok_g_xmax = 1;
		ok_g_ymax = 1;
		ok_i_xmin = 1;
		ok_i_ymin = 1;
		ok_i_xmax = 1;
		ok_i_ymax = 1;
		if (sqlite3_column_type (stmt, 0) == SQLITE_NULL)
		    ok_g_xmin = 0;
		else
		    g_xmin = sqlite3_column_double (stmt, 0);
		if (sqlite3_column_type (stmt, 1) == SQLITE_NULL)
		    ok_g_ymin = 0;
		else
		    g_ymin = sqlite3_column_double (stmt, 1);
		if (sqlite3_column_type (stmt, 2) == SQLITE_NULL)
		    ok_g_xmax = 0;
		else
		    g_xmax = sqlite3_column_double (stmt, 2);
		if (sqlite3_column_type (stmt, 3) == SQLITE_NULL)
		    ok_g_ymax = 0;
		else
		    g_ymax = sqlite3_column_double (stmt, 3);
		if (sqlite3_column_type (stmt, 4) == SQLITE_NULL)
		    ok_i_xmin = 0;
		else
		    i_xmin = sqlite3_column_double (stmt, 4);
		if (sqlite3_column_type (stmt, 5) == SQLITE_NULL)
		    ok_i_ymin = 0;
		else
		    i_ymin = sqlite3_column_double (stmt, 5);
		if (sqlite3_column_type (stmt, 6) == SQLITE_NULL)
		    ok_i_xmax = 0;
		else
		    i_xmax = sqlite3_column_double (stmt, 6);
		if (sqlite3_column_type (stmt, 7) == SQLITE_NULL)
		    ok_i_ymax = 0;
		else
		    i_ymax = sqlite3_column_double (stmt, 7);
		if (eval_rtree_entry (ok_g_xmin, g_xmin, ok_i_xmin, i_xmin) ==
		    0)
		    goto mismatching;
		if (eval_rtree_entry (ok_g_ymin, g_ymin, ok_i_ymin, i_ymin) ==
		    0)
		    goto mismatching;
		if (eval_rtree_entry (ok_g_xmax, g_xmax, ok_i_xmax, i_xmax) ==
		    0)
		    goto mismatching;
		if (eval_rtree_entry (ok_g_ymax, g_ymax, ok_i_ymax, i_ymax) ==
		    0)
		    goto mismatching;
	    }
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
/* we have now to finalize the query [memory cleanup] */
    sqlite3_finalize (stmt);


/* now we'll check the R*Tree against the corresponding geometry-table */
    sprintf (sql, "SELECT ");
    sprintf (sql2, "MbrMinX(g.%s), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "MbrMinY(g.%s), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "MbrMaxX(g.%s), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "MbrMaxY(g.%s), ", xgeom);
    strcat (sql, sql2);
    strcat (sql, "i.xmin, i.ymin, i.xmax, i.ymax\n");
    sprintf (sql2, "FROM %s AS i\n", idx_name);
    strcat (sql, sql2);
    sprintf (sql2, "LEFT JOIN %s AS g ", xtable);
    strcat (sql, sql2);
    strcat (sql, "ON (g.ROWID = i.pkid)");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CheckSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* checking a row */
		ok_g_xmin = 1;
		ok_g_ymin = 1;
		ok_g_xmax = 1;
		ok_g_ymax = 1;
		ok_i_xmin = 1;
		ok_i_ymin = 1;
		ok_i_xmax = 1;
		ok_i_ymax = 1;
		if (sqlite3_column_type (stmt, 0) == SQLITE_NULL)
		    ok_g_xmin = 0;
		else
		    g_xmin = sqlite3_column_double (stmt, 0);
		if (sqlite3_column_type (stmt, 1) == SQLITE_NULL)
		    ok_g_ymin = 0;
		else
		    g_ymin = sqlite3_column_double (stmt, 1);
		if (sqlite3_column_type (stmt, 2) == SQLITE_NULL)
		    ok_g_xmax = 0;
		else
		    g_xmax = sqlite3_column_double (stmt, 2);
		if (sqlite3_column_type (stmt, 3) == SQLITE_NULL)
		    ok_g_ymax = 0;
		else
		    g_ymax = sqlite3_column_double (stmt, 3);
		if (sqlite3_column_type (stmt, 4) == SQLITE_NULL)
		    ok_i_xmin = 0;
		else
		    i_xmin = sqlite3_column_double (stmt, 4);
		if (sqlite3_column_type (stmt, 5) == SQLITE_NULL)
		    ok_i_ymin = 0;
		else
		    i_ymin = sqlite3_column_double (stmt, 5);
		if (sqlite3_column_type (stmt, 6) == SQLITE_NULL)
		    ok_i_xmax = 0;
		else
		    i_xmax = sqlite3_column_double (stmt, 6);
		if (sqlite3_column_type (stmt, 7) == SQLITE_NULL)
		    ok_i_ymax = 0;
		else
		    i_ymax = sqlite3_column_double (stmt, 7);
		if (eval_rtree_entry (ok_g_xmin, g_xmin, ok_i_xmin, i_xmin) ==
		    0)
		    goto mismatching;
		if (eval_rtree_entry (ok_g_ymin, g_ymin, ok_i_ymin, i_ymin) ==
		    0)
		    goto mismatching;
		if (eval_rtree_entry (ok_g_xmax, g_xmax, ok_i_xmax, i_xmax) ==
		    0)
		    goto mismatching;
		if (eval_rtree_entry (ok_g_ymax, g_ymax, ok_i_ymax, i_ymax) ==
		    0)
		    goto mismatching;
	    }
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
    sqlite3_finalize (stmt);
    strcpy (sql, "Check SpatialIndex: is valid");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) geom, sql);
    return 1;
  mismatching:
    sqlite3_finalize (stmt);
    strcpy (sql, "Check SpatialIndex: INCONSISTENCIES detected");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) geom, sql);
    return 0;
}

static int
check_any_spatial_index (sqlite3 * sqlite)
{
/* attempting to check any defined R*Tree for consistency */
    const unsigned char *table;
    const unsigned char *column;
    int status;
    char sql[1024];
    int ret;
    int invalid_rtree = 0;
    sqlite3_stmt *stmt;

/* retrieving any defined R*Tree */
    strcpy (sql,
	    "SELECT f_table_name, f_geometry_column FROM geometry_columns ");
    strcat (sql, "WHERE spatial_index_enabled = 1");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CheckSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* checking a single R*Tree */
		table = sqlite3_column_text (stmt, 0);
		column = sqlite3_column_text (stmt, 1);
		status = check_spatial_index (sqlite, table, column);
		if (status < 0)
		  {
		      sqlite3_finalize (stmt);
		      return -1;
		  }
		if (status == 0)
		    invalid_rtree = 1;
	    }
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
    sqlite3_finalize (stmt);
    if (invalid_rtree)
	return 0;
    return 1;
}

static void
fnct_CheckSpatialIndex (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ CheckSpatialIndex()
/ CheckSpatialIndex(table, column)
/
/ checks a SpatialIndex for consistency, returning:
/ 1 - the R*Tree is fully consistent
/ 0 - the R*Tree is inconsistent
/ NULL on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    int status;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (argc == 0)
      {
	  /* no arguments: we must check any defined R*Tree */
	  status = check_any_spatial_index (sqlite);
	  if (status < 0)
	      sqlite3_result_null (context);
	  else if (status > 0)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }

    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "CheckSpatialIndex() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_null (context);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "CheckSpatialIndex() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_null (context);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    status = check_spatial_index (sqlite, table, column);
    if (status < 0)
	sqlite3_result_null (context);
    else if (status > 0)
	sqlite3_result_int (context, 1);
    else
	sqlite3_result_int (context, 0);
}

static int
recover_spatial_index (sqlite3 * sqlite, const unsigned char *table,
		       const unsigned char *geom)
{
/* attempting to rebuild an R*Tree */
    char sql[8192];
    char sql2[2048];
    char *errMsg = NULL;
    int ret;
    char xtable[1024];
    char xgeom[1024];
    char idx_name[2048];
    int is_defined = 0;
    sqlite3_stmt *stmt;

/* checking if the R*Tree Spatial Index is defined */
    strcpy (xtable, (const char *) table);
    clean_sql_string (xtable);
    strcpy (xgeom, (const char *) geom);
    clean_sql_string (xgeom);
    strcpy (sql, "SELECT Count(*) FROM geometry_columns ");
    sprintf (sql2, "WHERE f_table_name LIKE '%s' ", xtable);
    strcat (sql, sql2);
    sprintf (sql2, "AND f_geometry_column LIKE '%s' ", xgeom);
    strcat (sql, sql2);
    strcat (sql, "AND spatial_index_enabled = 1");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "RecoverSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	      is_defined = sqlite3_column_int (stmt, 0);
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
    sqlite3_finalize (stmt);
    if (!is_defined)
	return -1;

/* erasing the R*Tree table */
    sprintf (idx_name, "idx_%s_%s", table, geom);
    double_quoted_sql (idx_name);
    sprintf (sql, "DELETE FROM %s", idx_name);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
/* populating the R*Tree table from scratch */
    buildSpatialIndex (sqlite, table, (const char *) geom);
    strcpy (sql, "SpatialIndex: successfully recovered");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) geom, sql);
    return 1;
  error:
    fprintf (stderr, "RecoverSpatialIndex() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    return 0;
}

static int
recover_any_spatial_index (sqlite3 * sqlite, int no_check)
{
/* attempting to rebuild any defined R*Tree */
    const unsigned char *table;
    const unsigned char *column;
    int status;
    char sql[1024];
    int ret;
    int to_be_fixed;
    sqlite3_stmt *stmt;

/* retrieving any defined R*Tree */
    strcpy (sql,
	    "SELECT f_table_name, f_geometry_column FROM geometry_columns ");
    strcat (sql, "WHERE spatial_index_enabled = 1");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "RecoverSpatialIndex SQL error: %s\n",
		   sqlite3_errmsg (sqlite));
	  return -1;
      }
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* checking a single R*Tree */
		table = sqlite3_column_text (stmt, 0);
		column = sqlite3_column_text (stmt, 1);
		to_be_fixed = 1;
		if (!no_check)
		  {
		      status = check_spatial_index (sqlite, table, column);
		      if (status < 0)
			{
			    /* some unexpected error occurred */
			    goto fatal_error;
			}
		      else if (status > 0)
			{
			    /* the Spatial Index is already valid */
			    to_be_fixed = 0;
			}
		  }
		if (to_be_fixed)
		  {
		      /* rebuilding the Spatial Index */
		      status = recover_spatial_index (sqlite, table, column);
		      if (status < 0)
			{
			    /* some unexpected error occurred */
			    goto fatal_error;
			}
		      else if (status == 0)
			  goto error;
		  }
	    }
	  else
	    {
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return -1;
	    }
      }
    sqlite3_finalize (stmt);
    return 1;
  error:
    sqlite3_finalize (stmt);
    return 0;
  fatal_error:
    sqlite3_finalize (stmt);
    return -1;
}

static void
fnct_RecoverSpatialIndex (sqlite3_context * context, int argc,
			  sqlite3_value ** argv)
{
/* SQL function:
/ RecoverSpatialIndex()
/ RecoverSpatialIndex(no_check)
/ RecoverSpatialIndex(table, column)
/ RecoverSpatialIndex(table, column, no_check)
/
/ attempts to rebuild a SpatialIndex, returning:
/ 1 - on success
/ 0 - on failure
/ NULL if any syntax error is detected
*/
    const unsigned char *table;
    const unsigned char *column;
    int no_check = 0;
    int status;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (argc <= 1)
      {
	  /* no arguments: we must rebuild any defined R*Tree */
	  if (argc == 1)
	    {
		if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
		    no_check = sqlite3_value_int (argv[0]);
		else
		  {
		      fprintf (stderr,
			       "RecoverSpatialIndex() error: argument 1 [no_check] is not of the Integer type\n");
		      sqlite3_result_null (context);
		      return;
		  }
	    }
	  status = recover_any_spatial_index (sqlite, no_check);
	  if (status < 0)
	      sqlite3_result_null (context);
	  else if (status > 0)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }

    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverSpatialIndex() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_null (context);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RecoverSpatialIndex() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_null (context);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    if (argc == 3)
      {
	  if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	      no_check = sqlite3_value_int (argv[2]);
	  else
	    {
		fprintf (stderr,
			 "RecoverSpatialIndex() error: argument 2 [no_check] is not of the Integer type\n");
		sqlite3_result_null (context);
		return;
	    }
      }
    if (!no_check)
      {
	  /* checking the current SpatialIndex validity */
	  status = check_spatial_index (sqlite, table, column);
	  if (status < 0)
	    {
		/* some unexpected error occurred */
		sqlite3_result_null (context);
		return;
	    }
	  else if (status > 0)
	    {
		/* the Spatial Index is already valid */
		sqlite3_result_int (context, 1);
		return;
	    }
      }
/* rebuilding the Spatial Index */
    status = recover_spatial_index (sqlite, table, column);
    if (status < 0)
	sqlite3_result_null (context);
    else if (status > 0)
	sqlite3_result_int (context, 1);
    else
	sqlite3_result_int (context, 0);
}

static void
fnct_CreateSpatialIndex (sqlite3_context * context, int argc,
			 sqlite3_value ** argv)
{
/* SQL function:
/ CreateSpatialIndex(table, column )
/
/ creates a SpatialIndex based on Column and Table
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "CreateSpatialIndex() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "CreateSpatialIndex() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    strcpy (sql,
	    "UPDATE geometry_columns SET spatial_index_enabled = 1 WHERE f_table_name LIKE '");
    strcat (sql, sqltable);
    strcat (sql, "' AND f_geometry_column LIKE '");
    strcat (sql, sqlcolumn);
    strcat (sql, "' AND spatial_index_enabled = 0");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    if (sqlite3_changes (sqlite) == 0)
      {
	  fprintf (stderr,
		   "CreateSpatialIndex() error: either \"%s\".\"%s\" isn't a Geometry column or a SpatialIndex is already defined\n",
		   table, column);
	  sqlite3_result_int (context, 0);
	  return;
      }
    updateGeometryTriggers (sqlite, table, column);
    sqlite3_result_int (context, 1);
    strcpy (sql, "R*Tree Spatial Index successfully created");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    fprintf (stderr, "CreateSpatialIndex() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_CreateMbrCache (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CreateMbrCache(table, column )
/
/ creates an MBR Cache based on Column and Table
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "CreateMbrCache() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "CreateMbrCache() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    strcpy (sql,
	    "UPDATE geometry_columns SET spatial_index_enabled = 2 WHERE f_table_name LIKE '");
    strcat (sql, sqltable);
    strcat (sql, "' AND f_geometry_column LIKE '");
    strcat (sql, sqlcolumn);
    strcat (sql, "' AND spatial_index_enabled = 0");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    if (sqlite3_changes (sqlite) == 0)
      {
	  fprintf (stderr,
		   "CreateMbrCache() error: either \"%s\".\"%s\" isn't a Geometry column or a SpatialIndex is already defined\n",
		   table, column);
	  sqlite3_result_int (context, 0);
	  return;
      }
    updateGeometryTriggers (sqlite, table, column);
    sqlite3_result_int (context, 1);
    strcpy (sql, "MbrCache successfully created");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    fprintf (stderr, "CreateMbrCache() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_DisableSpatialIndex (sqlite3_context * context, int argc,
			  sqlite3_value ** argv)
{
/* SQL function:
/ DisableSpatialIndex(table, column )
/
/ disables a SpatialIndex based on Column and Table
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "DisableSpatialIndex() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "DisableSpatialIndex() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    strcpy (sql,
	    "UPDATE geometry_columns SET spatial_index_enabled = 0 WHERE f_table_name LIKE '");
    strcat (sql, sqltable);
    strcat (sql, "' AND f_geometry_column LIKE '");
    strcat (sql, sqlcolumn);
    strcat (sql, "' AND spatial_index_enabled <> 0");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
	goto error;
    if (sqlite3_changes (sqlite) == 0)
      {
	  fprintf (stderr,
		   "DisableSpatialIndex() error: either \"%s\".\"%s\" isn't a Geometry column or no SpatialIndex is defined\n",
		   table, column);
	  sqlite3_result_int (context, 0);
	  return;
      }
    updateGeometryTriggers (sqlite, table, column);
    sqlite3_result_int (context, 1);
    strcpy (sql, "SpatialIndex successfully disabled");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    fprintf (stderr, "DisableSpatialIndex() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_RebuildGeometryTriggers (sqlite3_context * context, int argc,
			      sqlite3_value ** argv)
{
/* SQL function:
/ RebuildGeometryTriggers(table, column )
/
/ rebuilds Geometry Triggers (constraints)  based on Column and Table
/ returns 1 on success
/ 0 on failure
*/
    const unsigned char *table;
    const unsigned char *column;
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    char **results;
    int rows;
    int columns;
    char sqltable[1024];
    char sqlcolumn[1024];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RebuildGeometryTriggers() error: argument 1 [table_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    table = sqlite3_value_text (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
      {
	  fprintf (stderr,
		   "RebuildGeometryTriggers() error: argument 2 [column_name] is not of the String type\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    column = sqlite3_value_text (argv[1]);
    strcpy (sqltable, (char *) table);
    clean_sql_string (sqltable);
    strcpy (sqlcolumn, (char *) column);
    clean_sql_string (sqlcolumn);
    strcpy (sql,
	    "SELECT f_table_name FROM geometry_columns WHERE f_table_name LIKE '");
    strcat (sql, sqltable);
    strcat (sql, "' AND f_geometry_column LIKE '");
    strcat (sql, sqlcolumn);
    strcat (sql, "'");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	goto error;
    sqlite3_free_table (results);
    if (rows <= 0)
      {
	  fprintf (stderr,
		   "RebuildGeometryTriggers() error: \"%s\".\"%s\" isn't a Geometry column\n",
		   table, column);
	  sqlite3_result_int (context, 0);
	  return;
      }
    updateGeometryTriggers (sqlite, table, column);
    sqlite3_result_int (context, 1);
    strcpy (sql, "Geometry Triggers successfully rebuilt");
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    fprintf (stderr, "RebuildGeometryTriggers() error: \"%s\"\n", errMsg);
    sqlite3_free (errMsg);
    sqlite3_result_int (context, 0);
    return;
}


static int
check_topo_table (sqlite3 * sqlite, const char *table, int is_view)
{
/* checking if some Topology-related table/view already exists */
    int exists = 0;
    char sql[2048];
    char sqltable[1024];
    char *errMsg = NULL;
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    strcpy (sqltable, table);
    clean_sql_string (sqltable);
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = '%s' AND name LIKE '%s'",
	     (!is_view) ? "table" : "view", table);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
	return 0;
    for (i = 1; i <= rows; i++)
	exists = 1;
    sqlite3_free_table (results);
    return exists;
}

static int
create_topo_nodes (sqlite3 * sqlite, const char *table, int srid, int dims)
{
/* creating the topo_nodes table */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE TABLE %s (\n", sqltable);
    strcat (sql, "node_id INTEGER PRIMARY KEY AUTOINCREMENT,\n");
    strcat (sql, "gml_id TEXT)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE '%s' error: %s\n", table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    clean_sql_string (sqltable);
    sprintf (sql,
	     "SELECT AddGeometryColumn('%s', 'Geometry', %d, 'POINT', '%s')",
	     sqltable, srid, (dims == GAIA_XY_Z) ? "XYZ" : "XY");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "AddGeometryColumn '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    sprintf (sql, "SELECT CreateSpatialIndex('%s', 'Geometry')", sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CreateSpatialIndex '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_gml", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (gml_id)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('gml_id') error: %s\n", sqltable,
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_topo_edges (sqlite3 * sqlite, const char *table, int srid, int dims)
{
/* creating the topo_edges table */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE TABLE %s (\n", sqltable);
    strcat (sql, "edge_id INTEGER PRIMARY KEY AUTOINCREMENT,\n");
    strcat (sql, "node_from_href TEXT,\n");
    strcat (sql, "node_to_href TEXT,\n");
    strcat (sql, "gml_id TEXT)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE '%s' error: %s\n", table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    clean_sql_string (sqltable);
    sprintf (sql,
	     "SELECT AddGeometryColumn('%s', 'Geometry', %d, 'LINESTRING', '%s')",
	     sqltable, srid, (dims == GAIA_XY_Z) ? "XYZ" : "XY");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "AddGeometryColumn '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    sprintf (sql, "SELECT CreateSpatialIndex('%s', 'Geometry')", sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CreateSpatialIndex '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_gml_id", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (gml_id)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('gml_id') error: %s\n", sqltable,
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_from", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (node_from_href)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('node_from_href') error: %s\n",
		   sqltable, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_to", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (node_to_href)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('node_to_href') error: %s\n",
		   sqltable, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_topo_faces (sqlite3 * sqlite, const char *table, int srid, int dims)
{
/* creating the topo_faces table */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE TABLE %s (\n", sqltable);
    strcat (sql, "face_id INTEGER PRIMARY KEY AUTOINCREMENT,\n");
    strcat (sql, "gml_id TEXT)");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE '%s' error: %s\n", table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    clean_sql_string (sqltable);
    sprintf (sql,
	     "SELECT AddGeometryColumn('%s', 'Geometry', %d, 'POLYGON', '%s')",
	     sqltable, srid, (dims == GAIA_XY_Z) ? "XYZ" : "XY");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "AddGeometryColumn '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    sprintf (sql, "SELECT CreateSpatialIndex('%s', 'Geometry')", sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CreateSpatialIndex '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_gml", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (gml_id)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('gml_id') error: %s\n", sqltable,
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_topo_faces_edges (sqlite3 * sqlite, const char *table,
			 const char *table2)
{
/* creating the topo_faces_edges table */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE TABLE %s (\n", sqltable);
    strcat (sql, "face_id INTEGER NOT NULL,\n");
    strcat (sql, "sub INTEGER NOT NULL,\n");
    strcat (sql, "gml_id TEXT,\n");
    strcat (sql, "orientation TEXT,\n");
    strcat (sql, "CONSTRAINT pk_faces_edges PRIMARY KEY ");
    strcat (sql, "(face_id, sub),\n");
    strcat (sql, "CONSTRAINT fk_faces_edges FOREIGN KEY ");
    strcat (sql, "(face_id) REFERENCES ");
    strcpy (sql2, table2);
    double_quoted_sql (sql2);
    strcat (sql, sql2);
    strcat (sql, " (face_id))\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE '%s' error: %s\n", table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_edge", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (gml_id)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('gml_id') error: %s\n", sqltable,
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_topo_curves (sqlite3 * sqlite, const char *table, const char *table2,
		    int srid, int dims)
{
/* creating the topo_curves table */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE TABLE %s (\n", sqltable);
    strcat (sql, "curve_id INTEGER NOT NULL,\n");
    strcat (sql, "edge_id INTEGER NOT NULL,\n");
    strcat (sql, "CONSTRAINT pk_curves PRIMARY KEY ");
    strcat (sql, "(curve_id, edge_id),\n");
    strcat (sql, "CONSTRAINT fk_curves FOREIGN KEY ");
    strcat (sql, "(edge_id) REFERENCES ");
    strcpy (sql2, table2);
    double_quoted_sql (sql2);
    strcat (sql, sql2);
    strcat (sql, " (edge_id))\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE '%s' error: %s\n", table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    clean_sql_string (sqltable);
    sprintf (sql,
	     "SELECT AddGeometryColumn('%s', 'Geometry', %d, 'MULTILINESTRING', '%s')",
	     sqltable, srid, (dims == GAIA_XY_Z) ? "XYZ" : "XY");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "AddGeometryColumn '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    sprintf (sql, "SELECT CreateSpatialIndex('%s', 'Geometry')", sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CreateSpatialIndex '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_edge", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (edge_id)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('edge_id') error: %s\n", sqltable,
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_topo_surfaces (sqlite3 * sqlite, const char *table, const char *table2,
		      int srid, int dims)
{
/* creating the topo_surfaces table */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE TABLE %s (\n", sqltable);
    strcat (sql, "surface_id INTEGER NOT NULL,\n");
    strcat (sql, "face_id INTEGER NOT NULL,\n");
    strcat (sql, "CONSTRAINT pk_surfaces PRIMARY KEY ");
    strcat (sql, "(surface_id, face_id),\n");
    strcat (sql, "CONSTRAINT fk_surfaces FOREIGN KEY ");
    strcat (sql, "(face_id) REFERENCES ");
    strcpy (sql2, table2);
    double_quoted_sql (sql2);
    strcat (sql, sql2);
    strcat (sql, " (face_id))\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE '%s' error: %s\n", table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    clean_sql_string (sqltable);
    sprintf (sql,
	     "SELECT AddGeometryColumn('%s', 'Geometry', %d, 'MULTIPOLYGON', '%s')",
	     sqltable, srid, (dims == GAIA_XY_Z) ? "XYZ" : "XY");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "AddGeometryColumn '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    sprintf (sql, "SELECT CreateSpatialIndex('%s', 'Geometry')", sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CreateSpatialIndex '%s'.'Geometry' error: %s\n",
		   table, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    strcpy (sqltable, table);
    double_quoted_sql (sqltable);
    sprintf (sql2, "idx_%s_face", sqltable);
    double_quoted_sql (sql2);
    sprintf (sql, "CREATE INDEX %s ON %s (face_id)", sql2, sqltable);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "Create Index '%s'('face_id') error: %s\n", sqltable,
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_check_node_ids (sqlite3 * sqlite, const char *view,
		       const char *table_nodes)
{
/* creating the check node ids VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT gml_id AS gml_id, Count(node_id) AS count\n");
    strcpy (sqltable, table_nodes);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "GROUP BY gml_id\n");
    strcat (sql, "HAVING count > 1\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_check_node_geoms (sqlite3 * sqlite, const char *view,
			 const char *table_nodes)
{
/* creating the check node geoms VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT n1.node_id AS node1_id, n1.gml_id AS node1_gml_id, ");
    strcat (sql, "n2.node_id AS node2_id, n2.gml_id AS node2_gml_id\n");
    strcpy (sqltable, table_nodes);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s AS n1\n", sqltable);
    strcat (sql, sql2);
    sprintf (sql2, "JOIN %s AS n2 ON (\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "  n1.node_id <> n2.node_id AND\n");
    strcat (sql, "  ST_Equals(n1.Geometry, n2.Geometry) = 1 AND\n");
    strcat (sql, "  n2.node_id IN (\n");
    strcat (sql, "	SELECT ROWID FROM SpatialIndex\n");
    strcpy (sqltable, table_nodes);
    clean_sql_string (sqltable);
    sprintf (sql2, "	WHERE f_table_name = '%s' AND\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "	  search_frame = n1.Geometry))\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_check_edge_ids (sqlite3 * sqlite, const char *view,
		       const char *table_edges)
{
/* creating the check edge ids VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT gml_id AS gml_id, Count(edge_id) AS count\n");
    strcpy (sqltable, table_edges);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "GROUP BY gml_id\n");
    strcat (sql, "HAVING count > 1\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_check_edge_geoms (sqlite3 * sqlite, const char *view,
			 const char *table_edges)
{
/* creating the check edge geoms VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT e1.edge_id AS edge1_id, e1.gml_id AS edge1_gml_id, ");
    strcat (sql, "e2.edge_id AS edge2_id, e2.gml_id AS edge2_gml_id\n");
    strcpy (sqltable, table_edges);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s AS e1\n", sqltable);
    strcat (sql, sql2);
    sprintf (sql2, "JOIN %s AS e2 ON (\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "  e1.edge_id <> e2.edge_id AND\n");
    strcat (sql, "NOT (e1.node_from_href = e2.node_from_href ");
    strcat (sql, "AND e1.node_to_href = e2.node_to_href) AND\n");
    strcat (sql, "  ST_Crosses(e1.Geometry, e2.Geometry) = 1 AND\n");
    strcat (sql, "  e2.edge_id IN (\n");
    strcat (sql, "	SELECT ROWID FROM SpatialIndex\n");
    strcpy (sqltable, table_edges);
    clean_sql_string (sqltable);
    sprintf (sql2, "	WHERE f_table_name = '%s' AND\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "	  search_frame = e1.Geometry))\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_check_face_ids (sqlite3 * sqlite, const char *view,
		       const char *table_faces)
{
/* creating the check face ids VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT gml_id AS gml_id, Count(face_id) AS count\n");
    strcpy (sqltable, table_faces);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "GROUP BY gml_id\n");
    strcat (sql, "HAVING count > 1\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_faces_resolved (sqlite3 * sqlite, const char *view, const char *faces,
		       const char *faces_edges, const char *edges)
{
/* creating the Faces Resolved VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT f.face_id AS face_id, ");
    strcat (sql, "ST_Polygonize(e.Geometry) AS Geometry\n");
    strcpy (sqltable, faces);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s AS f\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "LEFT JOIN ");
    strcpy (sqltable, faces_edges);
    double_quoted_sql (sqltable);
    strcat (sql, sqltable);
    double_quoted_sql (sqltable);
    strcat (sql, " AS fe ON (fe.face_id = f.face_id)\n");
    strcat (sql, "LEFT JOIN ");
    strcpy (sqltable, edges);
    double_quoted_sql (sqltable);
    strcat (sql, sqltable);
    double_quoted_sql (sqltable);
    strcat (sql, " AS e ON (e.gml_id = fe.gml_id)\n");
    strcat (sql, "GROUP BY f.face_id\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_curves_resolved (sqlite3 * sqlite, const char *view, const char *curves,
			char *edges)
{
/* creating the Curves Resolved VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT c.curve_id AS curve_id, ");
    strcat (sql, "CastToMultiLinestring(Collect(e.Geometry)) AS Geometry\n");
    strcpy (sqltable, curves);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s AS c\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "LEFT JOIN ");
    strcpy (sqltable, edges);
    double_quoted_sql (sqltable);
    strcat (sql, sqltable);
    double_quoted_sql (sqltable);
    strcat (sql, " AS e ON (e.edge_id = c.edge_id)\n");
    strcat (sql, "GROUP BY c.curve_id\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_surfaces_resolved (sqlite3 * sqlite, const char *view,
			  const char *surfaces, const char *faces)
{
/* creating the Surfaces Resolved VIEW */
    char sql[2048];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;
    strcpy (sqltable, view);
    double_quoted_sql (sqltable);
    sprintf (sql, "CREATE VIEW %s AS\n", sqltable);
    strcat (sql, "SELECT s.surface_id AS surface_id, ");
    strcat (sql, "CastToMultipolygon(Collect(f.Geometry)) AS Geometry\n");
    strcpy (sqltable, surfaces);
    double_quoted_sql (sqltable);
    sprintf (sql2, "FROM %s AS s\n", sqltable);
    strcat (sql, sql2);
    strcat (sql, "LEFT JOIN ");
    strcpy (sqltable, faces);
    double_quoted_sql (sqltable);
    strcat (sql, sqltable);
    double_quoted_sql (sqltable);
    strcat (sql, " AS f ON (f.face_id = s.face_id)\n");
    strcat (sql, "GROUP BY s.surface_id\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE VIEW '%s' error: %s\n", view, err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
create_topo_master (sqlite3 * sqlite)
{
/* creating the topo_master table */
    char sql[4196];
    int ret;
    char *err_msg = NULL;

/* creating the table */
    strcpy (sql, "CREATE TABLE topology_master (\n");
    strcat (sql, "nodes TEXT NOT NULL,\n");
    strcat (sql, "edges TEXT NOT NULL,\n");
    strcat (sql, "faces TEXT NOT NULL,\n");
    strcat (sql, "faces_edges TEXT NOT NULL,\n");
    strcat (sql, "curves TEXT NOT NULL,\n");
    strcat (sql, "surfaces TEXT NOT NULL,\n");
    strcat (sql, "check_node_ids TEXT NOT NULL,\n");
    strcat (sql, "check_node_geoms TEXT NOT NULL,\n");
    strcat (sql, "check_edge_ids TEXT NOT NULL,\n");
    strcat (sql, "check_edge_geoms TEXT NOT NULL,\n");
    strcat (sql, "check_face_ids TEXT NOT NULL,\n");
    strcat (sql, "faces_resolved TEXT NOT NULL,\n");
    strcat (sql, "curves_resolved TEXT NOT NULL,\n");
    strcat (sql, "surfaces_resolved TEXT NOT NULL,\n");
    strcat (sql, "coord_dimension TEXT NOT NULL,\n");
    strcat (sql, "srid INTEGER NOT NULL,\n");
    strcat (sql, "CONSTRAINT fk_topo_master FOREIGN KEY \n");
    strcat (sql, "(srid) REFERENCES spatial_ref_sys (srid))\n");
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE 'topology_master' error: %s\n",
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static int
update_topo_master (sqlite3 * sqlite, const char *nodes, const char *edges,
		    const char *faces, const char *faces_edges,
		    const char *curves, const char *surfaces,
		    const char *check_nodes, const char *check_node_geoms,
		    const char *check_edges, const char *check_edge_geoms,
		    const char *check_faces, const char *faces_res,
		    const char *curves_res, const char *surfaces_res, int srid,
		    int dims)
{
/* updating the topo_master table */
    char sql[4196];
    char sql2[2048];
    char sqltable[1024];
    int ret;
    char *err_msg = NULL;

/* inserting Topology data into MASTER */
    strcpy (sql, "INSERT INTO topology_master ");
    strcat (sql, "(nodes, edges, faces, faces_edges, ");
    strcat (sql, "curves, surfaces, check_node_ids, ");
    strcat (sql, "check_node_geoms, check_edge_ids, ");
    strcat (sql, "check_edge_geoms, check_face_ids, ");
    strcat (sql, "faces_resolved, curves_resolved, ");
    strcat (sql, "surfaces_resolved, ");
    strcat (sql, "coord_dimension, srid) ");
    strcat (sql, "VALUES (");
    strcpy (sqltable, nodes);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, edges);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, faces);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, faces_edges);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, curves);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, surfaces);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, check_nodes);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, check_node_geoms);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, check_edges);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, check_edge_geoms);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, check_faces);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, faces_res);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, curves_res);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    strcpy (sqltable, surfaces_res);
    clean_sql_string (sqltable);
    sprintf (sql2, "'%s', ", sqltable);
    strcat (sql, sql2);
    sprintf (sql2, "'%s', %d)", (dims == GAIA_XY_Z) ? "XYZ" : "XY", srid);
    strcat (sql, sql2);
    ret = sqlite3_exec (sqlite, sql, NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "INSERT INTO 'topology_master' error: %s\n",
		   err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    return 1;
}

static void
fnct_CreateTopologyTables (sqlite3_context * context, int argc,
			   sqlite3_value ** argv)
{
/* SQL function:
/ CreateTopologyTables(srid, coord_dims)
/  or
/ CreateTopologyTables(prefix, srid, coord_dims)
/
/ creates any Topology related table 
/ returns 1 on success
/ 0 on failure
*/
    const char *prefix = "topo_";
    const unsigned char *txt_dims;
    int srid = -1;
    int dimension;
    int dims = -1;
    char table_curves[1024];
    char table_surfaces[1024];
    char table_nodes[1024];
    char table_edges[1024];
    char table_faces[1024];
    char table_faces_edges[1024];
    char view_check_node_ids[1024];
    char view_check_node_geoms[1024];
    char view_check_edge_ids[1024];
    char view_check_edge_geoms[1024];
    char view_check_face_ids[1024];
    char view_faces_resolved[1024];
    char view_curves_resolved[1024];
    char view_surfaces_resolved[1024];
    const char *tables[16];
    int views[16];
    int *p_view;
    const char **p_tbl;
    int ok_table;
    int create_master = 1;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (argc == 3)
      {
	  if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
	    {
		fprintf (stderr,
			 "CreateTopologyTables() error: argument 1 [table_prefix] is not of the String type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
	  prefix = (char *) sqlite3_value_text (argv[0]);
	  if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
	    {
		fprintf (stderr,
			 "CreateTopologyTables() error: argument 2 [SRID] is not of the Integer type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
	  srid = sqlite3_value_int (argv[1]);
	  if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	    {
		dimension = sqlite3_value_int (argv[2]);
		if (dimension == 2)
		    dims = GAIA_XY;
		if (dimension == 3)
		    dims = GAIA_XY_Z;
	    }
	  else if (sqlite3_value_type (argv[2]) == SQLITE_TEXT)
	    {
		txt_dims = sqlite3_value_text (argv[2]);
		if (strcasecmp ((char *) txt_dims, "XY") == 0)
		    dims = GAIA_XY;
		if (strcasecmp ((char *) txt_dims, "XYZ") == 0)
		    dims = GAIA_XY_Z;
	    }
	  else
	    {
		fprintf (stderr,
			 "CreateTopologyTables() error: argument 3 [dimension] is not of the Integer or Text type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
      }
    else
      {
	  if (sqlite3_value_type (argv[0]) != SQLITE_INTEGER)
	    {
		fprintf (stderr,
			 "CreateTopologyTables() error: argument 1 [SRID] is not of the Integer type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
	  srid = sqlite3_value_int (argv[0]);
	  if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	    {
		dimension = sqlite3_value_int (argv[1]);
		if (dimension == 2)
		    dims = GAIA_XY;
		if (dimension == 3)
		    dims = GAIA_XY_Z;
	    }
	  else if (sqlite3_value_type (argv[1]) == SQLITE_TEXT)
	    {
		txt_dims = sqlite3_value_text (argv[1]);
		if (strcasecmp ((char *) txt_dims, "XY") == 0)
		    dims = GAIA_XY;
		if (strcasecmp ((char *) txt_dims, "XYZ") == 0)
		    dims = GAIA_XY_Z;
	    }
	  else
	    {
		fprintf (stderr,
			 "CreateTopologyTables() error: argument 2 [dimension] is not of the Integer or Text type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
      }
    if (dims == GAIA_XY || dims == GAIA_XY_Z)
	;
    else
      {
	  fprintf (stderr,
		   "CreateTopologyTables() error: [dimension] ILLEGAL VALUE\n");
	  sqlite3_result_int (context, 0);
	  return;
      }
    if (srid <= 0)
      {
	  fprintf (stderr,
		   "CreateTopologyTables() error: [SRID] ILLEGAL VALUE\n");
	  sqlite3_result_int (context, 0);
	  return;
      }

/* checking Topology tables */
    tables[0] = "topology_master";
    views[0] = 0;
    sprintf (table_curves, "%scurves", prefix);
    tables[1] = table_curves;
    views[1] = 0;
    sprintf (table_surfaces, "%ssurfaces", prefix);
    tables[2] = table_surfaces;
    views[2] = 0;
    sprintf (table_nodes, "%snodes", prefix);
    tables[3] = table_nodes;
    views[3] = 0;
    sprintf (table_edges, "%sedges", prefix);
    tables[4] = table_edges;
    views[4] = 0;
    sprintf (table_faces, "%sfaces", prefix);
    tables[5] = table_faces;
    views[5] = 0;
    sprintf (table_faces_edges, "%sfaces_edges", prefix);
    tables[6] = table_faces_edges;
    views[6] = 0;
    sprintf (view_check_node_ids, "%snodes_check_dupl_ids", prefix);
    tables[7] = view_check_node_ids;
    views[7] = 1;
    sprintf (view_check_node_geoms, "%snodes_check_dupl_geoms", prefix);
    tables[8] = view_check_node_geoms;
    views[8] = 1;
    sprintf (view_check_edge_ids, "%sedges_check_dupl_ids", prefix);
    tables[9] = view_check_edge_ids;
    views[9] = 1;
    sprintf (view_check_edge_geoms, "%sedges_check_dupl_geoms", prefix);
    tables[10] = view_check_edge_geoms;
    views[10] = 1;
    sprintf (view_check_face_ids, "%sfaces_check_dupl_ids", prefix);
    tables[11] = view_check_face_ids;
    views[11] = 1;
    sprintf (view_faces_resolved, "%sfaces_resolved", prefix);
    tables[12] = view_faces_resolved;
    views[12] = 1;
    sprintf (view_curves_resolved, "%scurves_resolved", prefix);
    tables[13] = view_curves_resolved;
    views[13] = 1;
    sprintf (view_surfaces_resolved, "%ssurfaces_resolved", prefix);
    tables[14] = view_surfaces_resolved;
    views[14] = 1;
    tables[15] = NULL;
    p_view = views;
    p_tbl = tables;
    while (*p_tbl != NULL)
      {
	  ok_table = check_topo_table (sqlite, *p_tbl, *p_view);
	  if (ok_table)
	    {
		if (strcmp (*p_tbl, "topology_master") == 0)
		    create_master = 0;
		else
		  {
		      fprintf (stderr,
			       "CreateTopologyTables() error: table '%s' already exists\n",
			       *p_tbl);
		      sqlite3_result_int (context, 0);
		      return;
		  }
	    }
	  p_tbl++;
	  p_view++;
      }

/* creating Topology tables */
    if (create_master)
      {
	  if (!create_topo_master (sqlite))
	      goto error;
      }
    if (!create_topo_nodes (sqlite, table_nodes, srid, dims))
	goto error;
    if (!create_topo_edges (sqlite, table_edges, srid, dims))
	goto error;
    if (!create_topo_faces (sqlite, table_faces, srid, dims))
	goto error;
    if (!create_topo_faces_edges (sqlite, table_faces_edges, table_faces))
	goto error;
    if (!create_topo_curves (sqlite, table_curves, table_edges, srid, dims))
	goto error;
    if (!create_topo_surfaces (sqlite, table_surfaces, table_faces, srid, dims))
	goto error;
    if (!create_check_node_ids (sqlite, view_check_node_ids, table_nodes))
	goto error;
    if (!create_check_node_geoms (sqlite, view_check_node_geoms, table_nodes))
	goto error;
    if (!create_check_edge_ids (sqlite, view_check_edge_ids, table_edges))
	goto error;
    if (!create_check_edge_geoms (sqlite, view_check_edge_geoms, table_edges))
	goto error;
    if (!create_check_face_ids (sqlite, view_check_face_ids, table_faces))
	goto error;
    if (!create_faces_resolved
	(sqlite, view_faces_resolved, table_faces, table_faces_edges,
	 table_edges))
	goto error;
    if (!create_curves_resolved
	(sqlite, view_curves_resolved, table_curves, table_edges))
	goto error;
    if (!create_surfaces_resolved
	(sqlite, view_surfaces_resolved, table_surfaces, table_faces))
	goto error;
    if (!update_topo_master
	(sqlite, table_nodes, table_edges, table_faces, table_faces_edges,
	 table_curves, table_surfaces, view_check_node_ids,
	 view_check_node_geoms, view_check_edge_ids, view_check_edge_geoms,
	 view_check_face_ids, view_faces_resolved, view_curves_resolved,
	 view_surfaces_resolved, srid, dims))
	goto error;
    updateSpatiaLiteHistory (sqlite, "*** TOPOLOGY ***", NULL,
			     "Topology tables successfully created");
    sqlite3_result_int (context, 1);
    return;

  error:
    sqlite3_result_int (context, 0);
    return;
}

static void
fnct_UpdateLayerStatistics (sqlite3_context * context, int argc,
			    sqlite3_value ** argv)
{
/* SQL function:
/ UpdateLayerStatistics(table, column )
/
/ Updates LAYER_STATISTICS [based on Column and Table]
/ returns 1 on success
/ 0 on failure
*/
    const char *sql;
    const unsigned char *table = NULL;
    const unsigned char *column = NULL;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (argc >= 1)
      {
	  if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
	    {
		fprintf (stderr,
			 "UpdateLayerStatistics() error: argument 1 [table_name] is not of the String type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
	  table = sqlite3_value_text (argv[0]);
      }
    if (argc >= 2)
      {
	  if (sqlite3_value_type (argv[1]) != SQLITE_TEXT)
	    {
		fprintf (stderr,
			 "UpdateLayerStatistics() error: argument 2 [column_name] is not of the String type\n");
		sqlite3_result_int (context, 0);
		return;
	    }
	  column = sqlite3_value_text (argv[1]);
      }
    if (!update_layer_statistics (sqlite, table, column))
	goto error;
    sqlite3_result_int (context, 1);
    sql = "UpdateLayerStatistics";
    if (table == NULL)
	table = "ALL-TABLES";
    if (column == NULL)
	column = "ALL-GEOMETRY-COLUMNS";
    updateSpatiaLiteHistory (sqlite, (const char *) table,
			     (const char *) column, sql);
    return;
  error:
    sqlite3_result_int (context, 0);
    return;
}

static gaiaPointPtr
simplePoint (gaiaGeomCollPtr geo)
{
/* helper function
/ if this GEOMETRY contains only one POINT, and no other elementary geometry
/ the POINT address will be returned
/ otherwise NULL will be returned
*/
    int cnt = 0;
    gaiaPointPtr point;
    gaiaPointPtr this_point = NULL;
    if (!geo)
	return NULL;
    if (geo->FirstLinestring || geo->FirstPolygon)
	return NULL;
    point = geo->FirstPoint;
    while (point)
      {
	  /* counting how many POINTs are there */
	  cnt++;
	  this_point = point;
	  point = point->Next;
      }
    if (cnt == 1 && this_point)
	return this_point;
    return NULL;
}

static gaiaLinestringPtr
simpleLinestring (gaiaGeomCollPtr geo)
{
/* helper function
/ if this GEOMETRY contains only one LINESTRING, and no other elementary geometry
/ the LINESTRING address will be returned
/ otherwise NULL will be returned
*/
    int cnt = 0;
    gaiaLinestringPtr line;
    gaiaLinestringPtr this_line = NULL;
    if (!geo)
	return NULL;
    if (geo->FirstPoint || geo->FirstPolygon)
	return NULL;
    line = geo->FirstLinestring;
    while (line)
      {
	  /* counting how many LINESTRINGs are there */
	  cnt++;
	  this_line = line;
	  line = line->Next;
      }
    if (cnt == 1 && this_line)
	return this_line;
    return NULL;
}

static gaiaPolygonPtr
simplePolygon (gaiaGeomCollPtr geo)
{
/* helper function
/ if this GEOMETRY contains only one POLYGON, and no other elementary geometry
/ the POLYGON address will be returned
/ otherwise NULL will be returned
*/
    int cnt = 0;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr this_polyg = NULL;
    if (!geo)
	return NULL;
    if (geo->FirstPoint || geo->FirstLinestring)
	return NULL;
    polyg = geo->FirstPolygon;
    while (polyg)
      {
	  /* counting how many POLYGONs are there */
	  cnt++;
	  this_polyg = polyg;
	  polyg = polyg->Next;
      }
    if (cnt == 1 && this_polyg)
	return this_polyg;
    return NULL;
}

static void
fnct_AsText (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsText(BLOB encoded geometry)
/
/ returns the corresponding WKT encoded value
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    gaiaOutBufferInitialize (&out_buf);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaOutWkt (&out_buf, geo);
	  if (out_buf.Error || out_buf.Buffer == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		len = out_buf.WriteOffset;
		sqlite3_result_text (context, out_buf.Buffer, len, free);
		out_buf.Buffer = NULL;
	    }
      }
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_AsWkt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsWkt(BLOB encoded geometry [, Integer precision])
/
/ returns the corresponding WKT encoded value
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    int precision = 15;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (argc == 2)
      {
	  if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	      precision = sqlite3_value_int (argv[1]);
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    gaiaOutBufferInitialize (&out_buf);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaOutWktStrict (&out_buf, geo, precision);
	  if (out_buf.Error || out_buf.Buffer == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		len = out_buf.WriteOffset;
		sqlite3_result_text (context, out_buf.Buffer, len, free);
		out_buf.Buffer = NULL;
	    }
      }
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

/*
/
/ AsSvg(geometry,[relative], [precision]) implementation
/
////////////////////////////////////////////////////////////
/
/ Author: Klaus Foerster klaus.foerster@svg.cc
/ version 0.9. 2008 September 21
 /
 */

static void
fnct_AsSvg (sqlite3_context * context, int argc, sqlite3_value ** argv,
	    int relative, int precision)
{
/* SQL function:
   AsSvg(BLOB encoded geometry, [int relative], [int precision])
   returns the corresponding SVG encoded value or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  /* make sure relative is 0 or 1 */
	  if (relative > 0)
	      relative = 1;
	  else
	      relative = 0;
	  /* make sure precision is between 0 and 15 - default to 6 if absent */
	  if (precision > GAIA_SVG_DEFAULT_MAX_PRECISION)
	      precision = GAIA_SVG_DEFAULT_MAX_PRECISION;
	  if (precision < 0)
	      precision = 0;
	  /* produce SVG-notation - actual work is done in gaiageo/gg_wkt.c */
	  gaiaOutBufferInitialize (&out_buf);
	  gaiaOutSvg (&out_buf, geo, relative, precision);
	  if (out_buf.Error || out_buf.Buffer == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		len = out_buf.WriteOffset;
		sqlite3_result_text (context, out_buf.Buffer, len, free);
		out_buf.Buffer = NULL;
	    }
      }
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_AsSvg1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* called without additional arguments */
    fnct_AsSvg (context, argc, argv, GAIA_SVG_DEFAULT_RELATIVE,
		GAIA_SVG_DEFAULT_PRECISION);
}

static void
fnct_AsSvg2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* called with relative-switch */
    if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	fnct_AsSvg (context, argc, argv, sqlite3_value_int (argv[1]),
		    GAIA_SVG_DEFAULT_PRECISION);
    else
	sqlite3_result_null (context);
}

static void
fnct_AsSvg3 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* called with relative-switch and precision-argument */
    if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER
	&& sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	fnct_AsSvg (context, argc, argv, sqlite3_value_int (argv[1]),
		    sqlite3_value_int (argv[2]));
    else
	sqlite3_result_null (context);
}

/* END of Klaus Foerster AsSvg() implementation */

static void
proj_params (sqlite3 * sqlite, int srid, char *proj_params)
{
/* retrives the PROJ params from SPATIAL_SYS_REF table, if possible */
    char sql[256];
    char **results;
    int rows;
    int columns;
    int i;
    int ret;
    char *errMsg = NULL;
    *proj_params = '\0';
    sprintf (sql,
	     "SELECT proj4text FROM spatial_ref_sys WHERE srid = %d", srid);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "unknown SRID: %d\t<%s>\n", srid, errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    for (i = 1; i <= rows; i++)
	strcpy (proj_params, results[(i * columns)]);
    if (*proj_params == '\0')
	fprintf (stderr, "unknown SRID: %d\n", srid);
    sqlite3_free_table (results);
}

#ifndef OMIT_PROJ		/* PROJ.4 is strictly required to support KML */
static void
fnct_AsKml1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsKml(BLOB encoded geometry [, Integer precision])
/
/ returns the corresponding 'bare geom' KML representation 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geo_wgs84;
    char proj_from[2048];
    char proj_to[2048];
    int precision = 15;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (argc == 2)
      {
	  if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	      precision = sqlite3_value_int (argv[1]);
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    gaiaOutBufferInitialize (&out_buf);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (geo->Srid == 4326)
	      ;			/* already WGS84 */
	  else if (geo->Srid == -1)
	    {
		/* unknown SRID: giving up */
		sqlite3_result_null (context);
		goto stop;
	    }
	  else
	    {
		/* attempting to reproject into WGS84 */
		proj_params (sqlite, geo->Srid, proj_from);
		proj_params (sqlite, 4326, proj_to);
		if (*proj_to == '\0' || *proj_from == '\0')
		  {
		      sqlite3_result_null (context);
		      goto stop;
		  }
		geo_wgs84 = gaiaTransform (geo, proj_from, proj_to);
		if (!geo_wgs84)
		  {
		      sqlite3_result_null (context);
		      goto stop;
		  }
		/* ok, reprojection was successful */
		gaiaFreeGeomColl (geo);
		geo = geo_wgs84;
	    }
	  /* produce KML-notation - actual work is done in gaiageo/gg_wkt.c */
	  gaiaOutBareKml (&out_buf, geo, precision);
	  if (out_buf.Error || out_buf.Buffer == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		len = out_buf.WriteOffset;
		sqlite3_result_text (context, out_buf.Buffer, len, free);
		out_buf.Buffer = NULL;
	    }
      }
  stop:
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_AsKml3 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsKml(Anything name, Anything description, BLOB encoded geometry [, Integer precision])
/
/ returns the corresponding 'full' KML representation 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geo_wgs84;
    sqlite3_int64 int_value;
    double dbl_value;
    const char *name;
    const char *desc;
    char *name_malloc = NULL;
    char *desc_malloc = NULL;
    char dummy[128];
    char proj_from[2048];
    char proj_to[2048];
    int precision = 15;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    switch (sqlite3_value_type (argv[0]))
      {
      case SQLITE_TEXT:
	  name = (const char *) sqlite3_value_text (argv[0]);
	  len = strlen (name);
	  name_malloc = malloc (len + 1);
	  strcpy (name_malloc, name);
	  name = name_malloc;
	  break;
      case SQLITE_INTEGER:
	  int_value = sqlite3_value_int64 (argv[0]);
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT: M$ runtime doesn't supports %lld for 64 bits */
	  sprintf (dummy, "%I64d", int_value);
#else
	  sprintf (dummy, "%lld", int_value);
#endif
	  len = strlen (dummy);
	  name_malloc = malloc (len + 1);
	  strcpy (name_malloc, dummy);
	  name = name_malloc;
	  break;
      case SQLITE_FLOAT:
	  dbl_value = sqlite3_value_double (argv[0]);
	  sprintf (dummy, "%1.6f", dbl_value);
	  len = strlen (dummy);
	  name_malloc = malloc (len + 1);
	  strcpy (name_malloc, dummy);
	  name = name_malloc;
	  break;
      case SQLITE_BLOB:
	  name = "BLOB";
	  break;
      default:
	  name = "NULL";
	  break;
      };
    switch (sqlite3_value_type (argv[1]))
      {
      case SQLITE_TEXT:
	  desc = (const char *) sqlite3_value_text (argv[1]);
	  len = strlen (desc);
	  desc_malloc = malloc (len + 1);
	  strcpy (desc_malloc, desc);
	  desc = desc_malloc;
	  break;
      case SQLITE_INTEGER:
	  int_value = sqlite3_value_int64 (argv[1]);
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT: M$ runtime doesn't supports %lld for 64 bits */
	  sprintf (dummy, "%I64d", int_value);
#else
	  sprintf (dummy, "%lld", int_value);
#endif
	  len = strlen (dummy);
	  desc_malloc = malloc (len + 1);
	  strcpy (desc_malloc, dummy);
	  desc = desc_malloc;
	  break;
      case SQLITE_FLOAT:
	  dbl_value = sqlite3_value_double (argv[1]);
	  sprintf (dummy, "%1.6f", dbl_value);
	  len = strlen (dummy);
	  desc_malloc = malloc (len + 1);
	  strcpy (desc_malloc, dummy);
	  desc = desc_malloc;
	  break;
      case SQLITE_BLOB:
	  desc = "BLOB";
	  break;
      default:
	  desc = "NULL";
	  break;
      };
    gaiaOutBufferInitialize (&out_buf);
    if (sqlite3_value_type (argv[2]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  goto stop;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[2]);
    n_bytes = sqlite3_value_bytes (argv[2]);
    if (argc == 4)
      {
	  if (sqlite3_value_type (argv[3]) == SQLITE_INTEGER)
	      precision = sqlite3_value_int (argv[3]);
	  else
	    {
		sqlite3_result_null (context);
		goto stop;
	    }
      }
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (geo->Srid == 4326)
	      ;			/* already WGS84 */
	  else if (geo->Srid == -1)
	    {
		/* unknown SRID: giving up */
		sqlite3_result_null (context);
		goto stop;
	    }
	  else
	    {
		/* attempting to reproject into WGS84 */
		proj_params (sqlite, geo->Srid, proj_from);
		proj_params (sqlite, 4326, proj_to);
		if (*proj_to == '\0' || *proj_from == '\0')
		  {
		      sqlite3_result_null (context);
		      goto stop;
		  }
		geo_wgs84 = gaiaTransform (geo, proj_from, proj_to);
		if (!geo_wgs84)
		  {
		      sqlite3_result_null (context);
		      goto stop;
		  }
		/* ok, reprojection was successful */
		gaiaFreeGeomColl (geo);
		geo = geo_wgs84;
	    }
	  /* produce KML-notation - actual work is done in gaiageo/gg_wkt.c */
	  gaiaOutFullKml (&out_buf, name, desc, geo, precision);
	  if (out_buf.Error || out_buf.Buffer == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		len = out_buf.WriteOffset;
		sqlite3_result_text (context, out_buf.Buffer, len, free);
		out_buf.Buffer = NULL;
	    }
      }
  stop:
    gaiaFreeGeomColl (geo);
    if (name_malloc)
	free (name_malloc);
    if (desc_malloc)
	free (desc_malloc);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_AsKml (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsKml(Anything name, Anything description, BLOB encoded geometry)
/     or
/ AsKml(BLOB encoded geometry)
/
/ returns the corresponding KML representation 
/ or NULL if any error is encountered
*/
    if (argc == 3 || argc == 4)
	fnct_AsKml3 (context, argc, argv);
    else
	fnct_AsKml1 (context, argc, argv);
}
#endif /* end including PROJ.4 */

static void
fnct_AsGml (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsGml(BLOB encoded geometry)
/    or
/ AsGml(integer version, BLOB encoded geometry)
/    or
/ AsGml(integer version, BLOB encoded geometry, integer precision)
/
/ *version* may be 2 (GML 2.1.2) or 3 (GML 3.1.1)
/ default *version*: 2
/
/ *precision* is the number of output decimal digits
/ default *precision*: 15
/
/ returns the corresponding GML representation 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    int version = 2;
    int precision = 15;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (argc == 3)
      {
	  if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
	      version = sqlite3_value_int (argv[0]);
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
	  if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
	    {
		sqlite3_result_null (context);
		return;
	    }
	  p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
	  n_bytes = sqlite3_value_bytes (argv[1]);
	  if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	      precision = sqlite3_value_int (argv[2]);
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    else if (argc == 2)
      {
	  if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER
	      && sqlite3_value_type (argv[1]) == SQLITE_BLOB)
	    {
		version = sqlite3_value_int (argv[0]);
		p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
		n_bytes = sqlite3_value_bytes (argv[1]);
	    }
	  else if (sqlite3_value_type (argv[0]) == SQLITE_BLOB
		   && sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	    {
		p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
		n_bytes = sqlite3_value_bytes (argv[0]);
		precision = sqlite3_value_int (argv[1]);
	    }
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    else
      {
	  if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
	    {
		sqlite3_result_null (context);
		return;
	    }
	  p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
	  n_bytes = sqlite3_value_bytes (argv[0]);
      }
    gaiaOutBufferInitialize (&out_buf);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (geo->Srid == -1)
	      sqlite3_result_null (context);	/* unknown SRID: giving up */
	  else
	    {
		/* produce GML-notation - actual work is done in gaiageo/gg_wkt.c */
		gaiaOutGml (&out_buf, version, precision, geo);
		if (out_buf.Error || out_buf.Buffer == NULL)
		    sqlite3_result_null (context);
		else
		  {
		      len = out_buf.WriteOffset;
		      sqlite3_result_text (context, out_buf.Buffer, len, free);
		      out_buf.Buffer = NULL;
		  }
	    }
      }
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_AsGeoJSON (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsGeoJSON(BLOB encoded geometry)
/    or
/ AsGeoJSON(BLOB encoded geometry, integer precision)
/    or
/ AsGeoJSON(BLOB encoded geometry, integer precision, integer options)
/
/ *precision* is the number of output decimal digits
/ default *precision*: 15
/
/ *options* may be one of the followings:
/   0 = no options [default]
/   1 = GeoJSON MBR
/   2 = GeoJSON Short CRS (e.g EPSG:4326) 
/   3 = 1 + 2 (Mbr + shortCrs)
/   4 = GeoJSON Long CRS (e.g urn:ogc:def:crs:EPSG::4326)
/   5 = 1 + 4 (Mbr + longCrs)
/
/ returns the corresponding GML representation 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    int precision = 15;
    int options = 0;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (argc == 3)
      {
	  if (sqlite3_value_type (argv[0]) == SQLITE_BLOB
	      && sqlite3_value_type (argv[1]) == SQLITE_INTEGER
	      && sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	    {
		p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
		n_bytes = sqlite3_value_bytes (argv[0]);
		precision = sqlite3_value_int (argv[1]);
		options = sqlite3_value_int (argv[2]);
		if (options >= 1 && options <= 5)
		    ;
		else
		    options = 0;
	    }
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    else if (argc == 2)
      {
	  if (sqlite3_value_type (argv[0]) == SQLITE_BLOB
	      && sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	    {
		p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
		n_bytes = sqlite3_value_bytes (argv[0]);
		precision = sqlite3_value_int (argv[1]);
	    }
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    else
      {
	  if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
	    {
		sqlite3_result_null (context);
		return;
	    }
	  p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
	  n_bytes = sqlite3_value_bytes (argv[0]);
      }
    gaiaOutBufferInitialize (&out_buf);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (geo->Srid == -1)
	      sqlite3_result_null (context);	/* unknown SRID: giving up */
	  else
	    {
		/* produce GeoJSON-notation - actual work is done in gaiageo/gg_wkt.c */
		gaiaOutGeoJSON (&out_buf, geo, precision, options);
		if (out_buf.Error || out_buf.Buffer == NULL)
		    sqlite3_result_null (context);
		else
		  {
		      len = out_buf.WriteOffset;
		      sqlite3_result_text (context, out_buf.Buffer, len, free);
		      out_buf.Buffer = NULL;
		  }
	    }
      }
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_AsBinary (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsBinary(BLOB encoded geometry)
/
/ returns the corresponding WKB encoded value
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaToWkb (geo, &p_result, &len);
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_AsFGF (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsFGF(BLOB encoded geometry)
/
/ returns the corresponding FGF encoded value
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    int coord_dims;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  fprintf (stderr,
		   "AsFGF() error: argument 2 [geom_coords] is not of the Integer type\n");
	  sqlite3_result_null (context);
	  return;
      }
    coord_dims = sqlite3_value_int (argv[1]);
    if (coord_dims
	== 0 || coord_dims == 1 || coord_dims == 2 || coord_dims == 3)
	;
    else
      {
	  fprintf (stderr,
		   "AsFGF() error: argument 2 [geom_coords] out of range [0,1,2,3]\n");
	  sqlite3_result_null (context);
	  return;
      }
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaToFgf (geo, &p_result, &len, coord_dims);
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_MakePoint1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MakePoint(double X, double Y)
/
/ builds a POINT 
/ or NULL if any error is encountered
*/
    int len;
    int int_value;
    unsigned char *p_result = NULL;
    double x;
    double y;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaMakePoint (x, y, -1, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_MakePoint2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MakePoint(double X, double Y, int SRID)
/
/ builds a POINT 
/ or NULL if any error is encountered
*/
    int len;
    int int_value;
    unsigned char *p_result = NULL;
    double x;
    double y;
    int srid;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	srid = sqlite3_value_int (argv[2]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaMakePoint (x, y, srid, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
}

static void
addGeomPointToDynamicLine (gaiaDynamicLinePtr dyn, gaiaGeomCollPtr geom)
{
/* appending a simple-Point Geometry to a Dynamic Line */
    int pts;
    int lns;
    int pgs;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;

    if (dyn == NULL)
	return;
    if (dyn->Error)
	return;
/* checking if GEOM simply is a POINT */
    if (geom == NULL)
      {
	  dyn->Error = 1;
	  return;
      }
    pts = 0;
    lns = 0;
    pgs = 0;
    pt = geom->FirstPoint;
    while (pt)
      {
	  pts++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  lns++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (pts == 1 && lns == 0 && pgs == 0)
	;
    else
      {
	  /* failure: not a simple POINT */
	  dyn->Error = 1;
	  return;
      }

    if (dyn->Srid != geom->Srid)
      {
	  /* failure: SRID mismatch */
	  dyn->Error = 1;
	  return;
      }

    switch (geom->FirstPoint->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  gaiaAppendPointZMToDynamicLine (dyn, geom->FirstPoint->X,
					  geom->FirstPoint->Y,
					  geom->FirstPoint->Z,
					  geom->FirstPoint->M);
	  break;
      case GAIA_XY_Z:
	  gaiaAppendPointZToDynamicLine (dyn, geom->FirstPoint->X,
					 geom->FirstPoint->Y,
					 geom->FirstPoint->Z);
	  break;
      case GAIA_XY_M:
	  gaiaAppendPointMToDynamicLine (dyn, geom->FirstPoint->X,
					 geom->FirstPoint->Y,
					 geom->FirstPoint->M);
	  break;
      default:
	  gaiaAppendPointToDynamicLine (dyn, geom->FirstPoint->X,
					geom->FirstPoint->Y);
	  break;
      }
}

static void
fnct_MakeLine_step (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MakeLine(BLOBencoded geom)
/
/ aggregate function - STEP
/
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geom;
    gaiaDynamicLinePtr *p;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geom = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geom)
	return;
    p = sqlite3_aggregate_context (context, sizeof (gaiaDynamicLinePtr));
    if (!(*p))
      {
	  /* this is the first row */
	  *p = gaiaAllocDynamicLine ();
	  (*p)->Srid = geom->Srid;
	  addGeomPointToDynamicLine (*p, geom);
	  gaiaFreeGeomColl (geom);
      }
    else
      {
	  /* subsequent rows */
	  addGeomPointToDynamicLine (*p, geom);
	  gaiaFreeGeomColl (geom);
      }
}

static gaiaGeomCollPtr
geomFromDynamicLine (gaiaDynamicLinePtr dyn)
{
/* attempting to build a Geometry from a Dynamic Line */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr ln = NULL;
    gaiaPointPtr pt;
    int iv;
    int count = 0;
    int dims = GAIA_XY;

    if (dyn == NULL)
	return NULL;
    if (dyn->Error)
	return NULL;

    pt = dyn->First;
    while (pt)
      {
	  /* counting points and checking dims */
	  count++;
	  if (dims == GAIA_XY && pt->DimensionModel != GAIA_XY)
	      dims = pt->DimensionModel;
	  if (dims == GAIA_XY_Z
	      && (pt->DimensionModel == GAIA_XY_M
		  || pt->DimensionModel == GAIA_XY_Z_M))
	      dims = GAIA_XY_Z_M;
	  if (dims == GAIA_XY_M
	      && (pt->DimensionModel == GAIA_XY_Z
		  || pt->DimensionModel == GAIA_XY_Z_M))
	      dims = GAIA_XY_Z_M;
	  pt = pt->Next;
      }
    if (count == 0)
	return NULL;

    switch (dims)
      {
      case GAIA_XY_Z_M:
	  geom = gaiaAllocGeomCollXYZM ();
	  ln = gaiaAllocLinestringXYZM (count);
	  break;
      case GAIA_XY_Z:
	  geom = gaiaAllocGeomCollXYZ ();
	  ln = gaiaAllocLinestringXYZ (count);
	  break;
      case GAIA_XY_M:
	  geom = gaiaAllocGeomCollXYM ();
	  ln = gaiaAllocLinestringXYM (count);
	  break;
      default:
	  geom = gaiaAllocGeomColl ();
	  ln = gaiaAllocLinestring (count);
	  break;
      };

    if (geom != NULL && ln != NULL)
      {
	  gaiaInsertLinestringInGeomColl (geom, ln);
	  geom->Srid = dyn->Srid;
      }
    else
      {
	  if (geom)
	      gaiaFreeGeomColl (geom);
	  if (ln)
	      gaiaFreeLinestring (ln);
	  return NULL;
      }

    iv = 0;
    pt = dyn->First;
    while (pt)
      {
	  /* setting linestring points */
	  if (dims == GAIA_XY_Z_M)
	    {
		gaiaSetPointXYZM (ln->Coords, iv, pt->X, pt->Y, pt->Z, pt->M);
	    }
	  else if (dims == GAIA_XY_Z)
	    {
		gaiaSetPointXYZ (ln->Coords, iv, pt->X, pt->Y, pt->Z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaSetPointXYM (ln->Coords, iv, pt->X, pt->Y, pt->M);
	    }
	  else
	    {
		gaiaSetPoint (ln->Coords, iv, pt->X, pt->Y);
	    }
	  iv++;
	  pt = pt->Next;
      }
    return geom;
}

static void
fnct_MakeLine_final (sqlite3_context * context)
{
/* SQL function:
/ MakeLine(BLOBencoded geom)
/
/ aggregate function - FINAL
/
*/
    gaiaGeomCollPtr result;
    gaiaDynamicLinePtr *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    result = geomFromDynamicLine (*p);
    gaiaFreeDynamicLine (*p);
    if (!result)
	sqlite3_result_null (context);
    else
      {
	  /* builds the BLOB geometry to be returned */
	  int len;
	  unsigned char *p_result = NULL;
	  gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
	  sqlite3_result_blob (context, p_result, len, free);
	  gaiaFreeGeomColl (result);
      }
}

static void
fnct_MakeLine (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MakeLine(point-geometry geom1, point-geometry geom2)
/
/ builds a SEGMENT joining two POINTs 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_blob;
    int n_bytes;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  goto stop;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1)
      {
	  sqlite3_result_null (context);
	  goto stop;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  goto stop;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo2)
      {
	  sqlite3_result_null (context);
	  goto stop;
      }
    gaiaMakeLine (geo1, geo2, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
  stop:
    if (geo1)
	gaiaFreeGeomColl (geo1);
    if (geo2)
	gaiaFreeGeomColl (geo2);
}

static void
fnct_Collect_step (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Collect(BLOBencoded geom)
/
/ aggregate function - STEP
/
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geom;
    gaiaGeomCollPtr result;
    gaiaGeomCollPtr *p;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geom = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geom)
	return;
    p = sqlite3_aggregate_context (context, sizeof (gaiaGeomCollPtr));
    if (!(*p))
      {
	  /* this is the first row */
	  *p = geom;
      }
    else
      {
	  /* subsequent rows */
	  result = gaiaMergeGeometries (*p, geom);
	  gaiaFreeGeomColl (*p);
	  *p = result;
	  gaiaFreeGeomColl (geom);
      }
}

static void
fnct_Collect_final (sqlite3_context * context)
{
/* SQL function:
/ Collect(BLOBencoded geom)
/
/ aggregate function - FINAL
/
*/
    gaiaGeomCollPtr result;
    gaiaGeomCollPtr *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    result = *p;
    if (!result)
	sqlite3_result_null (context);
    else if (gaiaIsEmpty (result))
      {
	  gaiaFreeGeomColl (result);
	  sqlite3_result_null (context);
      }
    else
      {
	  /* builds the BLOB geometry to be returned */
	  int len;
	  unsigned char *p_result = NULL;
	  gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
	  sqlite3_result_blob (context, p_result, len, free);
	  gaiaFreeGeomColl (result);
      }
}

static void
fnct_Collect (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Collect(geometry geom1, geometry geom2)
/
/ merges two generic GEOMETRIES into a single one 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaMergeGeometries (geo1, geo2);
	  if (!result)
	      sqlite3_result_null (context);
	  else if (gaiaIsEmpty (result))
	    {
		gaiaFreeGeomColl (result);
		sqlite3_result_null (context);
	    }
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
geom_from_text1 (sqlite3_context * context, int argc, sqlite3_value ** argv,
		 short type)
{
/* SQL function:
/ GeomFromText(WKT encoded geometry)
/
/ returns the current geometry by parsing WKT encoded string 
/ or NULL if any error is encountered
/
/ if *type* is a negative value can accept any GEOMETRY CLASS
/ otherwise only requests conforming with required CLASS are valid
*/
    int len;
    unsigned char *p_result = NULL;
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseWkt (text, type);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
geom_from_text2 (sqlite3_context * context, int argc, sqlite3_value ** argv,
		 short type)
{
/* SQL function:
/ GeomFromText(WKT encoded geometry, SRID)
/
/ returns the current geometry by parsing WKT encoded string 
/ or NULL if any error is encountered
/
/ if *type* is a negative value can accept any GEOMETRY CLASS
/ otherwise only requests conforming with required CLASS are valid
*/
    int len;
    unsigned char *p_result = NULL;
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseWkt (text, type);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = sqlite3_value_int (argv[1]);
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static int
check_wkb (const unsigned char *wkb, int size, short type)
{
/* checking type coherency for WKB encoded GEOMETRY */
    int little_endian;
    int wkb_type;
    int endian_arch = gaiaEndianArch ();
    if (size < 5)
	return 0;		/* too short to be a WKB */
    if (*(wkb + 0) == 0x01)
	little_endian = GAIA_LITTLE_ENDIAN;
    else if (*(wkb + 0) == 0x00)
	little_endian = GAIA_BIG_ENDIAN;
    else
	return 0;		/* illegal byte ordering; neither BIG-ENDIAN nor LITTLE-ENDIAN */
    wkb_type = gaiaImport32 (wkb + 1, little_endian, endian_arch);
    if (wkb_type == GAIA_POINT || wkb_type == GAIA_LINESTRING
	|| wkb_type == GAIA_POLYGON || wkb_type == GAIA_MULTIPOINT
	|| wkb_type == GAIA_MULTILINESTRING || wkb_type == GAIA_MULTIPOLYGON
	|| wkb_type == GAIA_GEOMETRYCOLLECTION || wkb_type == GAIA_POINTZ
	|| wkb_type == GAIA_LINESTRINGZ || wkb_type == GAIA_POLYGONZ
	|| wkb_type == GAIA_MULTIPOINTZ || wkb_type == GAIA_MULTILINESTRINGZ
	|| wkb_type == GAIA_MULTIPOLYGONZ
	|| wkb_type == GAIA_GEOMETRYCOLLECTIONZ || wkb_type == GAIA_POINTM
	|| wkb_type == GAIA_LINESTRINGM || wkb_type == GAIA_POLYGONM
	|| wkb_type == GAIA_MULTIPOINTM || wkb_type == GAIA_MULTILINESTRINGM
	|| wkb_type == GAIA_MULTIPOLYGONM
	|| wkb_type == GAIA_GEOMETRYCOLLECTIONM || wkb_type == GAIA_POINTZM
	|| wkb_type == GAIA_LINESTRINGZM || wkb_type == GAIA_POLYGONZM
	|| wkb_type == GAIA_MULTIPOINTZM || wkb_type == GAIA_MULTILINESTRINGZM
	|| wkb_type == GAIA_MULTIPOLYGONZM
	|| wkb_type == GAIA_GEOMETRYCOLLECTIONZM)
	;
    else
	return 0;		/* illegal GEOMETRY CLASS */
    if (type < 0)
	;			/* no restrinction about GEOMETRY CLASS TYPE */
    else
      {
	  if (wkb_type != type)
	      return 0;		/* invalid CLASS TYPE for request */
      }
    return 1;
}

static void
geom_from_wkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv,
		short type)
{
/* SQL function:
/ GeomFromWKB(WKB encoded geometry)
/
/ returns the current geometry by parsing a WKB encoded blob 
/ or NULL if any error is encountered
/
/ if *type* is a negative value can accept any GEOMETRY CLASS
/ otherwise only requests conforming with required CLASS are valid
*/
    int len;
    int n_bytes;
    unsigned char *p_result = NULL;
    const unsigned char *wkb;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    wkb = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!check_wkb (wkb, n_bytes, type))
	return;
    geo = gaiaFromWkb (wkb, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
geom_from_wkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv,
		short type)
{
/* SQL function:
/ GeomFromWKB(WKB encoded geometry, SRID)
/
/ returns the current geometry by parsing a WKB encoded blob
/ or NULL if any error is encountered
/
/ if *type* is a negative value can accept any GEOMETRY CLASS
/ otherwise only requests conforming with required CLASS are valid
*/
    int len;
    int n_bytes;
    unsigned char *p_result = NULL;
    const unsigned char *wkb;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    wkb = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!check_wkb (wkb, n_bytes, type))
	return;
    geo = gaiaFromWkb (wkb, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = sqlite3_value_int (argv[1]);
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_GeometryFromFGF1 (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromFGF(FGF encoded geometry)
/
/ returns the current geometry by parsing an FGF encoded blob 
/ or NULL if any error is encountered
/
/ if *type* is a negative value can accept any GEOMETRY CLASS
/ otherwise only requests conforming with required CLASS are valid
*/
    int len;
    int n_bytes;
    unsigned char *p_result = NULL;
    const unsigned char *fgf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    fgf = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromFgf (fgf, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_GeometryFromFGF2 (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromFGF(FGF encoded geometry, SRID)
/
/ returns the current geometry by parsing an FGF encoded string 
/ or NULL if any error is encountered
/
/ if *type* is a negative value can accept any GEOMETRY CLASS
/ otherwise only requests conforming with required CLASS are valid
*/
    int len;
    int n_bytes;
    unsigned char *p_result = NULL;
    const unsigned char *fgf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    fgf = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromFgf (fgf, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = sqlite3_value_int (argv[1]);
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

/*
/ the following functions simply readdress the request to geom_from_text?()
/ setting the appropriate GEOMETRY CLASS TYPE
*/

static void
fnct_GeomFromText1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) -1);
}

static void
fnct_GeomFromText2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) -1);
}

static void
fnct_GeomCollFromText1 (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) GAIA_GEOMETRYCOLLECTION);
}

static void
fnct_GeomCollFromText2 (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) GAIA_GEOMETRYCOLLECTION);
}

static void
fnct_LineFromText1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) GAIA_LINESTRING);
}

static void
fnct_LineFromText2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) GAIA_LINESTRING);
}

static void
fnct_PointFromText1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) GAIA_POINT);
}

static void
fnct_PointFromText2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) GAIA_POINT);
}

static void
fnct_PolyFromText1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) GAIA_POLYGON);
}

static void
fnct_PolyFromText2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) GAIA_POLYGON);
}

static void
fnct_MLineFromText1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) GAIA_MULTILINESTRING);
}

static void
fnct_MLineFromText2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) GAIA_MULTILINESTRING);
}

static void
fnct_MPointFromText1 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) GAIA_MULTIPOINT);
}

static void
fnct_MPointFromText2 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) GAIA_MULTIPOINT);
}

static void
fnct_MPolyFromText1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text1 (context, argc, argv, (short) GAIA_MULTIPOLYGON);
}

static void
fnct_MPolyFromText2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_text2 (context, argc, argv, (short) GAIA_MULTIPOLYGON);
}

/*
/ the following functions simply readdress the request to geom_from_wkb?()
/ setting the appropriate GEOMETRY CLASS TYPE
*/

static void
fnct_GeomFromWkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) -1);
}

static void
fnct_GeomFromWkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) -1);
}

static void
fnct_GeomCollFromWkb1 (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) GAIA_GEOMETRYCOLLECTION);
}

static void
fnct_GeomCollFromWkb2 (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) GAIA_GEOMETRYCOLLECTION);
}

static void
fnct_LineFromWkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) GAIA_LINESTRING);
}

static void
fnct_LineFromWkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) GAIA_LINESTRING);
}

static void
fnct_PointFromWkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) GAIA_POINT);
}

static void
fnct_PointFromWkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) GAIA_POINT);
}

static void
fnct_PolyFromWkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) GAIA_POLYGON);
}

static void
fnct_PolyFromWkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) GAIA_POLYGON);
}

static void
fnct_MLineFromWkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) GAIA_MULTILINESTRING);
}

static void
fnct_MLineFromWkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) GAIA_MULTILINESTRING);
}

static void
fnct_MPointFromWkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) GAIA_MULTIPOINT);
}

static void
fnct_MPointFromWkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) GAIA_MULTIPOINT);
}

static void
fnct_MPolyFromWkb1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb1 (context, argc, argv, (short) GAIA_MULTIPOLYGON);
}

static void
fnct_MPolyFromWkb2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    geom_from_wkb2 (context, argc, argv, (short) GAIA_MULTIPOLYGON);
}

static void
fnct_CompressGeometry (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ CompressGeometry(BLOB encoded geometry)
/
/ returns a COMPRESSED geometry [if a valid Geometry was supplied]
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaToCompressedBlobWkb (geo, &p_result, &len);
	  sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_UncompressGeometry (sqlite3_context * context, int argc,
			 sqlite3_value ** argv)
{
/* SQL function:
/ UncompressGeometry(BLOB encoded geometry)
/
/ returns an UNCOMPRESSED geometry [if a valid Geometry was supplied] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
	  sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_SanitizeGeometry (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ SanitizeGeometry(BLOB encoded geometry)
/
/ returns a SANITIZED geometry [if a valid Geometry was supplied]
/ or NULL in any other case
/
/ Sanitizing includes:
/ - repeated vertices suppression
/ - enforcing ring closure
/
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr sanitized = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  sanitized = gaiaSanitize (geo);
	  gaiaToSpatiaLiteBlobWkb (sanitized, &p_result, &len);
	  sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
    gaiaFreeGeomColl (sanitized);
}

static void
cast_count (gaiaGeomCollPtr geom, int *pts, int *lns, int *pgs)
{
/* counting elementary geometries */
    int n_pts = 0;
    int n_lns = 0;
    int n_pgs = 0;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    if (geom)
      {
	  pt = geom->FirstPoint;
	  while (pt)
	    {
		n_pts++;
		pt = pt->Next;
	    }
	  ln = geom->FirstLinestring;
	  while (ln)
	    {
		n_lns++;
		ln = ln->Next;
	    }
	  pg = geom->FirstPolygon;
	  while (pg)
	    {
		n_pgs++;
		pg = pg->Next;
	    }
      }
    *pts = n_pts;
    *lns = n_lns;
    *pgs = n_pgs;
}

static void
fnct_CastToPoint (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToPoint(BLOB encoded geometry)
/
/ returns a POINT-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts == 1 && lns == 0 && pgs == 0)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_POINT;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToLinestring (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ CastToLinestring(BLOB encoded geometry)
/
/ returns a LINESTRING-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts == 0 && lns == 1 && pgs == 0)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_LINESTRING;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToPolygon (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToPolygon(BLOB encoded geometry)
/
/ returns a POLYGON-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts == 0 && lns == 0 && pgs == 1)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_POLYGON;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToMultiPoint (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ CastToMultiPoint(BLOB encoded geometry)
/
/ returns a MULTIPOINT-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts >= 1 && lns == 0 && pgs == 0)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_MULTIPOINT;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToMultiLinestring (sqlite3_context * context, int argc,
			    sqlite3_value ** argv)
{
/* SQL function:
/ CastToMultiLinestring(BLOB encoded geometry)
/
/ returns a MULTILINESTRING-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts == 0 && lns >= 1 && pgs == 0)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_MULTILINESTRING;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToMultiPolygon (sqlite3_context * context, int argc,
			 sqlite3_value ** argv)
{
/* SQL function:
/ CastToMultiPolygon(BLOB encoded geometry)
/
/ returns a MULTIPOLYGON-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts == 0 && lns == 0 && pgs >= 1)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_MULTIPOLYGON;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToGeometryCollection (sqlite3_context * context, int argc,
			       sqlite3_value ** argv)
{
/* SQL function:
/ CastToGeometryCollection(BLOB encoded geometry)
/
/ returns a GEOMETRYCOLLECTION-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts >= 1 || lns >= 1 || pgs >= 1)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToMulti (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToMulti(BLOB encoded geometry)
/
/ returns a MULTIPOINT, MULTILINESTRING, MULTIPOLYGON or
/ GEOMETRYCOLLECTION-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts >= 1 || lns >= 1 || pgs >= 1)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		if (pts >= 1 && lns == 0 && pgs == 0)
		    geom2->DeclaredType = GAIA_MULTIPOINT;
		else if (pts == 0 && lns >= 1 && pgs == 0)
		    geom2->DeclaredType = GAIA_MULTILINESTRING;
		else if (pts == 0 && lns == 0 && pgs >= 1)
		    geom2->DeclaredType = GAIA_MULTIPOLYGON;
		else
		    geom2->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToSingle (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToSingle(BLOB encoded geometry)
/
/ returns a POINT, LINESTRING or POLYGON-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    int ok;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  ok = 0;
	  if (pts == 1 && lns == 0 && pgs == 0)
	      ok = 1;
	  if (pts == 0 && lns == 1 && pgs == 0)
	      ok = 1;
	  if (pts == 0 && lns == 0 && pgs == 1)
	      ok = 1;
	  if (ok)
	    {
		geom2 = gaiaCloneGeomColl (geo);
		geom2->Srid = geo->Srid;
		if (pts == 1)
		    geom2->DeclaredType = GAIA_POINT;
		else if (lns == 1)
		    geom2->DeclaredType = GAIA_LINESTRING;
		else
		    geom2->DeclaredType = GAIA_POLYGON;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToXY (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToXY(BLOB encoded geometry)
/
/ returns an XY-dimension Geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  geom2 = gaiaCastGeomCollToXY (geo);
	  if (geom2)
	    {
		geom2->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToXYZ (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToXY(BLOB encoded geometry)
/
/ returns an XY-dimension Geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  geom2 = gaiaCastGeomCollToXYZ (geo);
	  if (geom2)
	    {
		geom2->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToXYM (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToXY(BLOB encoded geometry)
/
/ returns an XYM-dimension Geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  geom2 = gaiaCastGeomCollToXYM (geo);
	  if (geom2)
	    {
		geom2->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CastToXYZM (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CastToXY(BLOB encoded geometry)
/
/ returns an XYZM-dimension Geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  geom2 = gaiaCastGeomCollToXYZM (geo);
	  if (geom2)
	    {
		geom2->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_ExtractMultiPoint (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ ExtractMultiPoint(BLOB encoded geometry)
/
/ returns a MULTIPOINT-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pts >= 1)
	    {
		geom2 = gaiaCloneGeomCollPoints (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_MULTIPOINT;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_ExtractMultiLinestring (sqlite3_context * context, int argc,
			     sqlite3_value ** argv)
{
/* SQL function:
/ ExtractMultiLinestring(BLOB encoded geometry)
/
/ returns a MULTILINESTRING-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (lns >= 1)
	    {
		geom2 = gaiaCloneGeomCollLinestrings (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_MULTILINESTRING;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_ExtractMultiPolygon (sqlite3_context * context, int argc,
			  sqlite3_value ** argv)
{
/* SQL function:
/ ExtractMultiPolygon(BLOB encoded geometry)
/
/ returns a MULTIPOLYGON-type geometry [if conversion is possible] 
/ or NULL in any other case
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    int pts;
    int lns;
    int pgs;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  cast_count (geo, &pts, &lns, &pgs);
	  if (pgs >= 1)
	    {
		geom2 = gaiaCloneGeomCollPolygons (geo);
		geom2->Srid = geo->Srid;
		geom2->DeclaredType = GAIA_MULTIPOLYGON;
		gaiaToSpatiaLiteBlobWkb (geom2, &p_result, &len);
		gaiaFreeGeomColl (geom2);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Dimension (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Dimension(BLOB encoded geometry)
/
/ returns:
/ 0 if geometry is a POINT or MULTIPOINT
/ 1 if geometry is a LINESTRING or MULTILINESTRING
/ 2 if geometry is a POLYGON or MULTIPOLYGON
/ 0, 1, 2, for GEOMETRYCOLLECTIONS according to geometries contained inside
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int dim;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  dim = gaiaDimension (geo);
	  sqlite3_result_int (context, dim);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CoordDimension (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CoordDimension(BLOB encoded geometry)
/
/ returns:
/ 'XY', 'XYM', 'XYZ', 'XYZM'
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    char *p_dim = NULL;
    char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (geo->DimensionModel == GAIA_XY)
	      p_dim = "XY";
	  else if (geo->DimensionModel == GAIA_XY_Z)
	      p_dim = "XYZ";
	  else if (geo->DimensionModel == GAIA_XY_M)
	      p_dim = "XYM";
	  else if (geo->DimensionModel == GAIA_XY_Z_M)
	      p_dim = "XYZM";
	  if (p_dim)
	    {
		len = strlen (p_dim);
		p_result = malloc (len + 1);
		strcpy (p_result, p_dim);
	    }
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	    {
		len = strlen (p_result);
		sqlite3_result_text (context, p_result, len, free);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_GeometryType (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeometryType(BLOB encoded geometry)
/
/ returns the class for current geometry:
/ 'POINT' or 'MULTIPOINT' [Z, M, ZM]
/ 'LINESTRING' or 'MULTILINESTRING' [Z, M, ZM]
/ 'POLYGON' or 'MULTIPOLYGON' [Z, M, ZM]
/ 'GEOMETRYCOLLECTION'  [Z, M, ZM]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    int type;
    char *p_type = NULL;
    char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  type = gaiaGeometryType (geo);
	  switch (type)
	    {
	    case GAIA_POINT:
		p_type = "POINT";
		break;
	    case GAIA_POINTZ:
		p_type = "POINT Z";
		break;
	    case GAIA_POINTM:
		p_type = "POINT M";
		break;
	    case GAIA_POINTZM:
		p_type = "POINT ZM";
		break;
	    case GAIA_MULTIPOINT:
		p_type = "MULTIPOINT";
		break;
	    case GAIA_MULTIPOINTZ:
		p_type = "MULTIPOINT Z";
		break;
	    case GAIA_MULTIPOINTM:
		p_type = "MULTIPOINT M";
		break;
	    case GAIA_MULTIPOINTZM:
		p_type = "MULTIPOINT ZM";
		break;
	    case GAIA_LINESTRING:
	    case GAIA_COMPRESSED_LINESTRING:
		p_type = "LINESTRING";
		break;
	    case GAIA_LINESTRINGZ:
	    case GAIA_COMPRESSED_LINESTRINGZ:
		p_type = "LINESTRING Z";
		break;
	    case GAIA_LINESTRINGM:
	    case GAIA_COMPRESSED_LINESTRINGM:
		p_type = "LINESTRING M";
		break;
	    case GAIA_LINESTRINGZM:
	    case GAIA_COMPRESSED_LINESTRINGZM:
		p_type = "LINESTRING ZM";
		break;
	    case GAIA_MULTILINESTRING:
		p_type = "MULTILINESTRING";
		break;
	    case GAIA_MULTILINESTRINGZ:
		p_type = "MULTILINESTRING Z";
		break;
	    case GAIA_MULTILINESTRINGM:
		p_type = "MULTILINESTRING M";
		break;
	    case GAIA_MULTILINESTRINGZM:
		p_type = "MULTILINESTRING ZM";
		break;
	    case GAIA_POLYGON:
	    case GAIA_COMPRESSED_POLYGON:
		p_type = "POLYGON";
		break;
	    case GAIA_POLYGONZ:
	    case GAIA_COMPRESSED_POLYGONZ:
		p_type = "POLYGON Z";
		break;
	    case GAIA_POLYGONM:
	    case GAIA_COMPRESSED_POLYGONM:
		p_type = "POLYGON M";
		break;
	    case GAIA_POLYGONZM:
	    case GAIA_COMPRESSED_POLYGONZM:
		p_type = "POLYGON ZM";
		break;
	    case GAIA_MULTIPOLYGON:
		p_type = "MULTIPOLYGON";
		break;
	    case GAIA_MULTIPOLYGONZ:
		p_type = "MULTIPOLYGON Z";
		break;
	    case GAIA_MULTIPOLYGONM:
		p_type = "MULTIPOLYGON M";
		break;
	    case GAIA_MULTIPOLYGONZM:
		p_type = "MULTIPOLYGON ZM";
		break;
	    case GAIA_GEOMETRYCOLLECTION:
		p_type = "GEOMETRYCOLLECTION";
		break;
	    case GAIA_GEOMETRYCOLLECTIONZ:
		p_type = "GEOMETRYCOLLECTION Z";
		break;
	    case GAIA_GEOMETRYCOLLECTIONM:
		p_type = "GEOMETRYCOLLECTION M";
		break;
	    case GAIA_GEOMETRYCOLLECTIONZM:
		p_type = "GEOMETRYCOLLECTION ZM";
		break;
	    };
	  if (p_type)
	    {
		len = strlen (p_type);
		p_result = malloc (len + 1);
		strcpy (p_result, p_type);
	    }
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	    {
		len = strlen (p_result);
		sqlite3_result_text (context, p_result, len, free);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_GeometryAliasType (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ GeometryAliasType(BLOB encoded geometry)
/
/ returns the alias-class for current geometry:
/ 'POINT'
/ 'LINESTRING'
/ 'POLYGON'
/ 'MULTIPOINT'
/ 'MULTILINESTRING'
/ 'MULTIPOLYGON'
/ 'GEOMETRYCOLLECTION' 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    int type;
    char *p_type = NULL;
    char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  type = gaiaGeometryAliasType (geo);
	  switch (type)
	    {
	    case GAIA_POINT:
		p_type = "POINT";
		break;
	    case GAIA_MULTIPOINT:
		p_type = "MULTIPOINT";
		break;
	    case GAIA_LINESTRING:
		p_type = "LINESTRING";
		break;
	    case GAIA_MULTILINESTRING:
		p_type = "MULTILINESTRING";
		break;
	    case GAIA_POLYGON:
		p_type = "POLYGON";
		break;
	    case GAIA_MULTIPOLYGON:
		p_type = "MULTIPOLYGON";
		break;
	    case GAIA_GEOMETRYCOLLECTION:
		p_type = "GEOMETRYCOLLECTION";
		break;
	    };
	  if (p_type)
	    {
		len = strlen (p_type);
		p_result = malloc (len + 1);
		strcpy (p_result, p_type);
	    }
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	    {
		len = strlen (p_result);
		sqlite3_result_text (context, p_result, len, free);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_SridFromAuthCRS (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ SridFromAuthCRS(auth_name, auth_srid)
/
/ returns the SRID
/ or NULL if any error is encountered
*/
    const unsigned char *auth_name;
    int auth_srid;
    int srid = -1;
    char sql[1024];
    char sql2[1024];
    char **results;
    int n_rows;
    int n_columns;
    char *err_msg = NULL;
    int ret;
    int i;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    auth_name = sqlite3_value_text (argv[0]);
    auth_srid = sqlite3_value_int (argv[1]);

    sprintf (sql, "SELECT srid FROM spatial_ref_sys ");
    sprintf (sql2, "WHERE auth_name LIKE '%s' AND auth_srid = %d", auth_name,
	     auth_srid);
    strcat (sql, sql2);
    ret =
	sqlite3_get_table (sqlite, sql, &results, &n_rows, &n_columns,
			   &err_msg);
    if (ret != SQLITE_OK)
	goto done;
    if (n_rows >= 1)
      {
	  for (i = 1; i <= n_rows; i++)
	      srid = atoi (results[(i * n_columns) + 0]);
      }
    sqlite3_free_table (results);
  done:
    sqlite3_result_int (context, srid);
}

static void
fnct_SRID (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Srid(BLOB encoded geometry)
/
/ returns the SRID
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
	sqlite3_result_int (context, geo->Srid);
    gaiaFreeGeomColl (geo);
}

static void
fnct_SetSRID (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ SetSrid(BLOBencoded geometry, srid)
/
/ returns a new geometry that is the original one received, but with the new SRID [no coordinates translation is applied]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    int srid;
    unsigned char *p_result = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	srid = sqlite3_value_int (argv[1]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  geo->Srid = srid;
	  gaiaToSpatiaLiteBlobWkb (geo, &p_result, &n_bytes);
	  sqlite3_result_blob (context, p_result, n_bytes, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_IsEmpty (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ IsEmpty(BLOB encoded geometry)
/
/ returns:
/ 1 if this geometry contains no elementary geometries
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_int (context, 1);
    else
	sqlite3_result_int (context, gaiaIsEmpty (geo));
    gaiaFreeGeomColl (geo);
}

static void
fnct_Envelope (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Envelope(BLOB encoded geometry)
/
/ returns the MBR for current geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr bbox;
    gaiaPolygonPtr polyg;
    gaiaRingPtr rect;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (gaiaIsEmpty (geo))
	      sqlite3_result_null (context);
	  else
	    {
		gaiaMbrGeometry (geo);
		bbox = gaiaAllocGeomColl ();
		bbox->Srid = geo->Srid;
		polyg = gaiaAddPolygonToGeomColl (bbox, 5, 0);
		rect = polyg->Exterior;
		gaiaSetPoint (rect->Coords, 0, geo->MinX, geo->MinY);	/* vertex # 1 */
		gaiaSetPoint (rect->Coords, 1, geo->MaxX, geo->MinY);	/* vertex # 2 */
		gaiaSetPoint (rect->Coords, 2, geo->MaxX, geo->MaxY);	/* vertex # 3 */
		gaiaSetPoint (rect->Coords, 3, geo->MinX, geo->MaxY);	/* vertex # 4 */
		gaiaSetPoint (rect->Coords, 4, geo->MinX, geo->MinY);	/* vertex # 5 [same as vertex # 1 to close the polygon] */
		gaiaToSpatiaLiteBlobWkb (bbox, &p_result, &len);
		gaiaFreeGeomColl (bbox);
		sqlite3_result_blob (context, p_result, len, free);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
build_filter_mbr (sqlite3_context * context, int argc,
		  sqlite3_value ** argv, int mode)
{
/* SQL functions:
/ BuildMbrFilter(double X1, double Y1, double X2, double Y2)
/ FilterMBRWithin(double X1, double Y1, double X2, double Y2)
/ FilterMBRContain(double X1, double Y1, double X2, double Y2)
/ FilterMBRIntersects(double X1, double Y1, double X2, double Y2)
/
/ builds a generic filter for MBR from two points (identifying a rectangle's diagonal) 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    double x1;
    double y1;
    double x2;
    double y2;
    int int_value;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x1 = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x1 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y1 = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y1 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	x2 = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  x2 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[3]) == SQLITE_FLOAT)
	y2 = sqlite3_value_double (argv[3]);
    else if (sqlite3_value_type (argv[3]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[3]);
	  y2 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaBuildFilterMbr (x1, y1, x2, y2, mode, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
}

/*
/ the following functions simply readdress the request to build_filter_mbr()
/ setting the appropriate MODe
*/

static void
fnct_BuildMbrFilter (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    build_filter_mbr (context, argc, argv, GAIA_FILTER_MBR_DECLARE);
}

static void
fnct_FilterMbrWithin (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
    build_filter_mbr (context, argc, argv, GAIA_FILTER_MBR_WITHIN);
}

static void
fnct_FilterMbrContains (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
    build_filter_mbr (context, argc, argv, GAIA_FILTER_MBR_CONTAINS);
}

static void
fnct_FilterMbrIntersects (sqlite3_context * context, int argc,
			  sqlite3_value ** argv)
{
    build_filter_mbr (context, argc, argv, GAIA_FILTER_MBR_INTERSECTS);
}

static void
fnct_BuildMbr1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ BuildMBR(double X1, double Y1, double X2, double Y2)
/
/ builds an MBR from two points (identifying a rectangle's diagonal) 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    double x1;
    double y1;
    double x2;
    double y2;
    int int_value;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x1 = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x1 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y1 = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y1 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	x2 = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  x2 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[3]) == SQLITE_FLOAT)
	y2 = sqlite3_value_double (argv[3]);
    else if (sqlite3_value_type (argv[3]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[3]);
	  y2 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaBuildMbr (x1, y1, x2, y2, -1, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_BuildMbr2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ BuildMBR(double X1, double Y1, double X2, double Y2, int SRID)
/
/ builds an MBR from two points (identifying a rectangle's diagonal) 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    double x1;
    double y1;
    double x2;
    double y2;
    int int_value;
    int srid;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x1 = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x1 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y1 = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y1 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	x2 = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  x2 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[3]) == SQLITE_FLOAT)
	y2 = sqlite3_value_double (argv[3]);
    else if (sqlite3_value_type (argv[3]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[3]);
	  y2 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[4]) == SQLITE_INTEGER)
	srid = sqlite3_value_int (argv[4]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaBuildMbr (x1, y1, x2, y2, srid, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_BuildCircleMbr1 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ BuildCircleMBR(double X, double Y, double radius)
/
/ builds an MBR from two points (identifying a rectangle's diagonal) 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    double x;
    double y;
    double radius;
    int int_value;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	radius = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  radius = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaBuildCircleMbr (x, y, radius, -1, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_BuildCircleMbr2 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ BuildCircleMBR(double X, double Y, double radius, int SRID)
/
/ builds an MBR from two points (identifying a rectangle's diagonal) 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    double x;
    double y;
    double radius;
    int int_value;
    int srid;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	radius = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  radius = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[3]) == SQLITE_INTEGER)
	srid = sqlite3_value_int (argv[3]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaBuildCircleMbr (x, y, radius, srid, &p_result, &len);
    if (!p_result)
	sqlite3_result_null (context);
    else
	sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_Extent_step (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Extent(BLOBencoded geom)
/
/ aggregate function - STEP
/
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geom;
    double **p;
    double *max_min;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geom = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geom)
	return;
    gaiaMbrGeometry (geom);
    p = sqlite3_aggregate_context (context, sizeof (double **));
    if (!(*p))
      {
	  /* this is the first row */
	  max_min = malloc (sizeof (double) * 4);
	  *(max_min + 0) = geom->MinX;
	  *(max_min + 1) = geom->MinY;
	  *(max_min + 2) = geom->MaxX;
	  *(max_min + 3) = geom->MaxY;
	  *p = max_min;
      }
    else
      {
	  /* subsequent rows */
	  max_min = *p;
	  if (geom->MinX < *(max_min + 0))
	      *(max_min + 0) = geom->MinX;
	  if (geom->MinY < *(max_min + 1))
	      *(max_min + 1) = geom->MinY;
	  if (geom->MaxX > *(max_min + 2))
	      *(max_min + 2) = geom->MaxX;
	  if (geom->MaxY > *(max_min + 3))
	      *(max_min + 3) = geom->MaxY;
      }
    gaiaFreeGeomColl (geom);
}

static void
fnct_Extent_final (sqlite3_context * context)
{
/* SQL function:
/ Extent(BLOBencoded geom)
/
/ aggregate function - FINAL
/
*/
    gaiaGeomCollPtr result;
    gaiaPolygonPtr polyg;
    gaiaRingPtr rect;
    double *max_min;
    double minx;
    double miny;
    double maxx;
    double maxy;
    double **p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    max_min = *p;
    if (!max_min)
      {
	  sqlite3_result_null (context);
	  return;
      }
    result = gaiaAllocGeomColl ();
    if (!result)
	sqlite3_result_null (context);
    else
      {
	  /* builds the BLOB geometry to be returned */
	  int len;
	  unsigned char *p_result = NULL;
	  polyg = gaiaAddPolygonToGeomColl (result, 5, 0);
	  rect = polyg->Exterior;
	  minx = *(max_min + 0);
	  miny = *(max_min + 1);
	  maxx = *(max_min + 2);
	  maxy = *(max_min + 3);
	  gaiaSetPoint (rect->Coords, 0, minx, miny);	/* vertex # 1 */
	  gaiaSetPoint (rect->Coords, 1, maxx, miny);	/* vertex # 2 */
	  gaiaSetPoint (rect->Coords, 2, maxx, maxy);	/* vertex # 3 */
	  gaiaSetPoint (rect->Coords, 3, minx, maxy);	/* vertex # 4 */
	  gaiaSetPoint (rect->Coords, 4, minx, miny);	/* vertex # 5 [same as vertex # 1 to close the polygon] */
	  gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
	  sqlite3_result_blob (context, p_result, len, free);
	  gaiaFreeGeomColl (result);
      }
    free (max_min);
}

static void
fnct_MbrMinX (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MbrMinX(BLOB encoded GEMETRY)
/
/ returns the MinX coordinate for current geometry's MBR 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double coord;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!gaiaGetMbrMinX (p_blob, n_bytes, &coord))
	sqlite3_result_null (context);
    else
	sqlite3_result_double (context, coord);
}

static void
fnct_MbrMaxX (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MbrMaxX(BLOB encoded GEMETRY)
/
/ returns the MaxX coordinate for current geometry's MBR 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double coord;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!gaiaGetMbrMaxX (p_blob, n_bytes, &coord))
	sqlite3_result_null (context);
    else
	sqlite3_result_double (context, coord);
}

static void
fnct_MbrMinY (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MbrMinY(BLOB encoded GEMETRY)
/
/ returns the MinY coordinate for current geometry's MBR 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double coord;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!gaiaGetMbrMinY (p_blob, n_bytes, &coord))
	sqlite3_result_null (context);
    else
	sqlite3_result_double (context, coord);
}

static void
fnct_MbrMaxY (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ MbrMaxY(BLOB encoded GEMETRY)
/
/ returns the MaxY coordinate for current geometry's MBR 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double coord;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!gaiaGetMbrMaxY (p_blob, n_bytes, &coord))
	sqlite3_result_null (context);
    else
	sqlite3_result_double (context, coord);
}

#ifndef OMIT_GEOCALLBACKS	/* supporting RTree geometry callbacks */
static void
gaia_mbr_del (void *p)
{
/* freeing data used by R*Tree Geometry Callback */
    sqlite3_free (p);
}

static int
fnct_RTreeIntersects (sqlite3_rtree_geometry * p, int nCoord, double *aCoord,
		      int *pRes)
{
/* R*Tree Geometry callback function:
/ ... MATCH RTreeIntersects(double x1, double y1, double x2, double y2)
*/
    struct gaia_rtree_mbr *mbr;
    double xmin;
    double xmax;
    double ymin;
    double ymax;
    float fminx;
    float fminy;
    float fmaxx;
    float fmaxy;
    double tic;
    double tic2;

    if (p->pUser == 0)
      {
	  /* first call: we must check args and then initialize the MBR struct */
	  if (nCoord != 4)
	      return SQLITE_ERROR;
	  if (p->nParam != 4)
	      return SQLITE_ERROR;
	  mbr = (struct gaia_rtree_mbr *) (p->pUser =
					   sqlite3_malloc (sizeof
							   (struct
							    gaia_rtree_mbr)));
	  if (!mbr)
	      return SQLITE_NOMEM;
	  p->xDelUser = gaia_mbr_del;
	  xmin = p->aParam[0];
	  ymin = p->aParam[1];
	  xmax = p->aParam[2];
	  ymax = p->aParam[3];
	  if (xmin > xmax)
	    {
		xmin = p->aParam[2];
		xmax = p->aParam[0];
	    }
	  if (ymin > ymax)
	    {
		ymin = p->aParam[3];
		ymax = p->aParam[1];
	    }

	  /* adjusting the MBR so to compensate for DOUBLE/FLOAT truncations */
	  fminx = (float) xmin;
	  fminy = (float) ymin;
	  fmaxx = (float) xmax;
	  fmaxy = (float) ymax;
	  tic = fabs (xmin - fminx);
	  tic2 = fabs (ymin - fminy);
	  if (tic2 > tic)
	      tic = tic2;
	  tic2 = fabs (xmax - fmaxx);
	  if (tic2 > tic)
	      tic = tic2;
	  tic2 = fabs (ymax - fmaxy);
	  if (tic2 > tic)
	      tic = tic2;
	  tic *= 2.0;

	  mbr->minx = xmin - tic;
	  mbr->miny = ymin - tic;
	  mbr->maxx = xmax + tic;
	  mbr->maxy = ymax + tic;
      }

    mbr = (struct gaia_rtree_mbr *) (p->pUser);
    xmin = aCoord[0];
    xmax = aCoord[1];
    ymin = aCoord[2];
    ymax = aCoord[3];
    *pRes = 1;
/* evaluating Intersects relationship */
    if (xmin > mbr->maxx)
	*pRes = 0;
    if (xmax < mbr->minx)
	*pRes = 0;
    if (ymin > mbr->maxy)
	*pRes = 0;
    if (ymax < mbr->miny)
	*pRes = 0;
    return SQLITE_OK;
}

static int
fnct_RTreeDistWithin (sqlite3_rtree_geometry * p, int nCoord, double *aCoord,
		      int *pRes)
{
/* R*Tree Geometry callback function:
/ ... MATCH RTreeDistWithin(double x, double y, double radius)
*/
    struct gaia_rtree_mbr *mbr;
    double xmin;
    double xmax;
    double ymin;
    double ymax;

    if (p->pUser == 0)
      {
	  /* first call: we must check args and then initialize the MBR struct */
	  if (nCoord != 4)
	      return SQLITE_ERROR;
	  if (p->nParam != 3)
	      return SQLITE_ERROR;
	  mbr = (struct gaia_rtree_mbr *) (p->pUser =
					   sqlite3_malloc (sizeof
							   (struct
							    gaia_rtree_mbr)));
	  if (!mbr)
	      return SQLITE_NOMEM;
	  p->xDelUser = gaia_mbr_del;
	  mbr->minx = p->aParam[0] - p->aParam[2];
	  mbr->miny = p->aParam[1] - p->aParam[2];
	  mbr->maxx = p->aParam[0] + p->aParam[2];
	  mbr->maxy = p->aParam[1] + p->aParam[2];
      }

    mbr = (struct gaia_rtree_mbr *) (p->pUser);
    xmin = aCoord[0];
    xmax = aCoord[1];
    ymin = aCoord[2];
    ymax = aCoord[3];
    *pRes = 1;
/* evaluating Intersects relationship */
    if (xmin > mbr->maxx)
	*pRes = 0;
    if (xmax < mbr->minx)
	*pRes = 0;
    if (ymin > mbr->maxy)
	*pRes = 0;
    if (ymax < mbr->miny)
	*pRes = 0;
    return SQLITE_OK;
}
#endif /* end RTree geometry callbacks */

static void
fnct_X (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ X(BLOB encoded POINT)
/
/ returns the X coordinate for current POINT geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaPointPtr point;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  point = simplePoint (geo);
	  if (!point)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, point->X);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Y (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Y(BLOB encoded POINT)
/
/ returns the Y coordinate for current POINT geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaPointPtr point;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  point = simplePoint (geo);
	  if (!point)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, point->Y);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Z (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Z(BLOB encoded POINT)
/
/ returns the Z coordinate for current POINT geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaPointPtr point;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  point = simplePoint (geo);
	  if (!point)
	      sqlite3_result_null (context);
	  else
	    {
		if (point->DimensionModel == GAIA_XY_Z
		    || point->DimensionModel == GAIA_XY_Z_M)
		    sqlite3_result_double (context, point->Z);
		else
		    sqlite3_result_null (context);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_M (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ M(BLOB encoded POINT)
/
/ returns the M coordinate for current POINT geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaPointPtr point;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  point = simplePoint (geo);
	  if (!point)
	      sqlite3_result_null (context);
	  else
	    {
		if (point->DimensionModel == GAIA_XY_M
		    || point->DimensionModel == GAIA_XY_Z_M)
		    sqlite3_result_double (context, point->M);
		else
		    sqlite3_result_null (context);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_NumPoints (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ NumPoints(BLOB encoded LINESTRING)
/
/ returns the number of vertices for current LINESTRING geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaLinestringPtr line;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  line = simpleLinestring (geo);
	  if (!line)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_int (context, line->Points);
      }
    gaiaFreeGeomColl (geo);
}

static void
point_n (sqlite3_context * context, int argc, sqlite3_value ** argv,
	 int request)
{
/* SQL functions:
/ StartPoint(BLOB encoded LINESTRING geometry)
/ EndPoint(BLOB encoded LINESTRING geometry)
/ PointN(BLOB encoded LINESTRING geometry, integer point_no)
/
/ returns the Nth POINT for current LINESTRING geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int vertex;
    int len;
    double x;
    double y;
    double z;
    double m;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    gaiaLinestringPtr line;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (request == GAIA_POINTN)
      {
	  /* PointN() requires point index to be defined as an SQL function argument */
	  if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
	    {
		sqlite3_result_null (context);
		return;
	    }
	  vertex = sqlite3_value_int (argv[1]);
      }
    else if (request == GAIA_END_POINT)
	vertex = -1;		/* EndPoint() specifies a negative point index */
    else
	vertex = 1;		/* StartPoint() */
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  line = simpleLinestring (geo);
	  if (!line)
	      sqlite3_result_null (context);
	  else
	    {
		if (vertex < 0)
		    vertex = line->Points - 1;
		else
		    vertex -= 1;	/* decreasing the point index by 1, because PointN counts starting at index 1 */
		if (vertex >= 0 && vertex < line->Points)
		  {
		      if (line->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (line->Coords, vertex, &x, &y, &z);
			    result = gaiaAllocGeomCollXYZ ();
			    result->Srid = geo->Srid;
			    gaiaAddPointToGeomCollXYZ (result, x, y, z);
			}
		      else if (line->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (line->Coords, vertex, &x, &y, &m);
			    result = gaiaAllocGeomCollXYM ();
			    result->Srid = geo->Srid;
			    gaiaAddPointToGeomCollXYM (result, x, y, m);
			}
		      else if (line->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (line->Coords, vertex, &x, &y, &z,
					      &m);
			    result = gaiaAllocGeomCollXYZM ();
			    result->Srid = geo->Srid;
			    gaiaAddPointToGeomCollXYZM (result, x, y, z, m);
			}
		      else
			{
			    gaiaGetPoint (line->Coords, vertex, &x, &y);
			    result = gaiaAllocGeomColl ();
			    result->Srid = geo->Srid;
			    gaiaAddPointToGeomColl (result, x, y);
			}
		  }
		else
		    result = NULL;
		if (!result)
		    sqlite3_result_null (context);
		else
		  {
		      gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		      gaiaFreeGeomColl (result);
		      sqlite3_result_blob (context, p_result, len, free);
		  }
	    }
      }
    gaiaFreeGeomColl (geo);
}

/*
/ the following functions simply readdress the request to point_n()
/ setting the appropriate request mode
*/

static void
fnct_StartPoint (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    point_n (context, argc, argv, GAIA_START_POINT);
}

static void
fnct_EndPoint (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    point_n (context, argc, argv, GAIA_END_POINT);
}

static void
fnct_PointN (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    point_n (context, argc, argv, GAIA_POINTN);
}

static void
fnct_ExteriorRing (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL functions:
/ ExteriorRing(BLOB encoded POLYGON geometry)
/
/ returns the EXTERIOR RING for current POLYGON geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int iv;
    double x;
    double y;
    double z;
    double m;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    gaiaLinestringPtr line;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  polyg = simplePolygon (geo);
	  if (!polyg)
	      sqlite3_result_null (context);
	  else
	    {
		ring = polyg->Exterior;
		if (ring->DimensionModel == GAIA_XY_Z)
		    result = gaiaAllocGeomCollXYZ ();
		else if (ring->DimensionModel == GAIA_XY_M)
		    result = gaiaAllocGeomCollXYM ();
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		    result = gaiaAllocGeomCollXYZM ();
		else
		    result = gaiaAllocGeomColl ();
		result->Srid = geo->Srid;
		line = gaiaAddLinestringToGeomColl (result, ring->Points);
		for (iv = 0; iv < line->Points; iv++)
		  {
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			    gaiaSetPointXYZ (line->Coords, iv, x, y, z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			    gaiaSetPointXYM (line->Coords, iv, x, y, m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			    gaiaSetPointXYZM (line->Coords, iv, x, y, z, m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			    gaiaSetPoint (line->Coords, iv, x, y);
			}
		  }
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		gaiaFreeGeomColl (result);
		sqlite3_result_blob (context, p_result, len, free);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_NumInteriorRings (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ NumInteriorRings(BLOB encoded POLYGON)
/
/ returns the number of INTERIOR RINGS for current POLYGON geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaPolygonPtr polyg;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  polyg = simplePolygon (geo);
	  if (!polyg)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_int (context, polyg->NumInteriors);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_InteriorRingN (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL functions:
/ InteriorRingN(BLOB encoded POLYGON geometry)
/
/ returns the Nth INTERIOR RING for current POLYGON geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int border;
    int iv;
    double x;
    double y;
    double z;
    double m;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    gaiaLinestringPtr line;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    border = sqlite3_value_int (argv[1]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  polyg = simplePolygon (geo);
	  if (!polyg)
	      sqlite3_result_null (context);
	  else
	    {
		if (border >= 1 && border <= polyg->NumInteriors)
		  {
		      ring = polyg->Interiors + (border - 1);
		      if (ring->DimensionModel == GAIA_XY_Z)
			  result = gaiaAllocGeomCollXYZ ();
		      else if (ring->DimensionModel == GAIA_XY_M)
			  result = gaiaAllocGeomCollXYM ();
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			  result = gaiaAllocGeomCollXYZM ();
		      else
			  result = gaiaAllocGeomColl ();
		      result->Srid = geo->Srid;
		      line = gaiaAddLinestringToGeomColl (result, ring->Points);
		      for (iv = 0; iv < line->Points; iv++)
			{
			    if (ring->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (ring->Coords, iv, &x, &y,
						   &z);
				  gaiaSetPointXYZ (line->Coords, iv, x, y, z);
			      }
			    else if (ring->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (ring->Coords, iv, &x, &y,
						   &m);
				  gaiaSetPointXYM (line->Coords, iv, x, y, m);
			      }
			    else if (ring->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (ring->Coords, iv, &x, &y,
						    &z, &m);
				  gaiaSetPointXYZM (line->Coords, iv, x, y, z,
						    m);
			      }
			    else
			      {
				  gaiaGetPoint (ring->Coords, iv, &x, &y);
				  gaiaSetPoint (line->Coords, iv, x, y);
			      }
			}
		      gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		      gaiaFreeGeomColl (result);
		      sqlite3_result_blob (context, p_result, len, free);
		  }
		else
		    sqlite3_result_null (context);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_NumGeometries (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ NumGeometries(BLOB encoded GEOMETRYCOLLECTION)
/
/ returns the number of elementary geometries for current geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int cnt = 0;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  point = geo->FirstPoint;
	  while (point)
	    {
		/* counts how many points are there */
		cnt++;
		point = point->Next;
	    }
	  line = geo->FirstLinestring;
	  while (line)
	    {
		/* counts how many linestrings are there */
		cnt++;
		line = line->Next;
	    }
	  polyg = geo->FirstPolygon;
	  while (polyg)
	    {
		/* counts how many polygons are there */
		cnt++;
		polyg = polyg->Next;
	    }
	  sqlite3_result_int (context, cnt);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_GeometryN (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeometryN(BLOB encoded GEOMETRYCOLLECTION geometry)
/
/ returns the Nth geometry for current GEOMETRYCOLLECTION or MULTIxxxx geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int entity;
    int len;
    int cnt = 0;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaLinestringPtr line2;
    gaiaPolygonPtr polyg;
    gaiaPolygonPtr polyg2;
    gaiaRingPtr ring_in;
    gaiaRingPtr ring_out;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    entity = sqlite3_value_int (argv[1]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  point = geo->FirstPoint;
	  while (point)
	    {
		/* counts how many points are there */
		cnt++;
		if (cnt == entity)
		  {
		      /* ok, required elementary geometry is this POINT */
		      if (point->DimensionModel == GAIA_XY_Z)
			  result = gaiaAllocGeomCollXYZ ();
		      else if (point->DimensionModel == GAIA_XY_M)
			  result = gaiaAllocGeomCollXYM ();
		      else if (point->DimensionModel == GAIA_XY_Z_M)
			  result = gaiaAllocGeomCollXYZM ();
		      else
			  result = gaiaAllocGeomColl ();
		      result->Srid = geo->Srid;
		      if (point->DimensionModel == GAIA_XY_Z)
			  gaiaAddPointToGeomCollXYZ (result, point->X,
						     point->Y, point->Z);
		      else if (point->DimensionModel == GAIA_XY_M)
			  gaiaAddPointToGeomCollXYM (result, point->X,
						     point->Y, point->M);
		      else if (point->DimensionModel == GAIA_XY_Z_M)
			  gaiaAddPointToGeomCollXYZM (result, point->X,
						      point->Y, point->Z,
						      point->M);
		      else
			  gaiaAddPointToGeomColl (result, point->X, point->Y);
		      goto skip;
		  }
		point = point->Next;
	    }
	  line = geo->FirstLinestring;
	  while (line)
	    {
		/* counts how many linestrings are there */
		cnt++;
		if (cnt == entity)
		  {
		      /* ok, required elementary geometry is this LINESTRING */
		      if (line->DimensionModel == GAIA_XY_Z)
			  result = gaiaAllocGeomCollXYZ ();
		      else if (line->DimensionModel == GAIA_XY_M)
			  result = gaiaAllocGeomCollXYM ();
		      else if (line->DimensionModel == GAIA_XY_Z_M)
			  result = gaiaAllocGeomCollXYZM ();
		      else
			  result = gaiaAllocGeomColl ();
		      result->Srid = geo->Srid;
		      line2 =
			  gaiaAddLinestringToGeomColl (result, line->Points);
		      for (iv = 0; iv < line2->Points; iv++)
			{
			    if (line->DimensionModel == GAIA_XY_Z)
			      {
				  gaiaGetPointXYZ (line->Coords, iv, &x, &y,
						   &z);
				  gaiaSetPointXYZ (line2->Coords, iv, x, y, z);
			      }
			    else if (line->DimensionModel == GAIA_XY_M)
			      {
				  gaiaGetPointXYM (line->Coords, iv, &x, &y,
						   &m);
				  gaiaSetPointXYM (line2->Coords, iv, x, y, m);
			      }
			    else if (line->DimensionModel == GAIA_XY_Z_M)
			      {
				  gaiaGetPointXYZM (line->Coords, iv, &x, &y,
						    &z, &m);
				  gaiaSetPointXYZM (line2->Coords, iv, x, y,
						    z, m);
			      }
			    else
			      {
				  gaiaGetPoint (line->Coords, iv, &x, &y);
				  gaiaSetPoint (line2->Coords, iv, x, y);
			      }
			}
		      goto skip;
		  }
		line = line->Next;
	    }
	  polyg = geo->FirstPolygon;
	  while (polyg)
	    {
		/* counts how many polygons are there */
		cnt++;
		if (cnt == entity)
		  {
		      /* ok, required elementary geometry is this POLYGON */
		      if (polyg->DimensionModel == GAIA_XY_Z)
			  result = gaiaAllocGeomCollXYZ ();
		      else if (polyg->DimensionModel == GAIA_XY_M)
			  result = gaiaAllocGeomCollXYM ();
		      else if (polyg->DimensionModel == GAIA_XY_Z_M)
			  result = gaiaAllocGeomCollXYZM ();
		      else
			  result = gaiaAllocGeomColl ();
		      result->Srid = geo->Srid;
		      ring_in = polyg->Exterior;
		      polyg2 =
			  gaiaAddPolygonToGeomColl (result, ring_in->Points,
						    polyg->NumInteriors);
		      ring_out = polyg2->Exterior;
		      for (iv = 0; iv < ring_out->Points; iv++)
			{
			    /* copying the exterior ring POINTs */
			    gaiaGetPoint (ring_in->Coords, iv, &x, &y);
			    gaiaSetPoint (ring_out->Coords, iv, x, y);
			}
		      for (ib = 0; ib < polyg2->NumInteriors; ib++)
			{
			    /* processing the interior rings */
			    ring_in = polyg->Interiors + ib;
			    ring_out =
				gaiaAddInteriorRing (polyg2, ib,
						     ring_in->Points);
			    for (iv = 0; iv < ring_out->Points; iv++)
			      {
				  if (ring_in->DimensionModel == GAIA_XY_Z)
				    {
					gaiaGetPointXYZ (ring_in->Coords, iv,
							 &x, &y, &z);
					gaiaSetPointXYZ (ring_out->Coords, iv,
							 x, y, z);
				    }
				  else if (ring_in->DimensionModel == GAIA_XY_M)
				    {
					gaiaGetPointXYM (ring_in->Coords, iv,
							 &x, &y, &m);
					gaiaSetPointXYM (ring_out->Coords, iv,
							 x, y, m);
				    }
				  else if (ring_in->DimensionModel ==
					   GAIA_XY_Z_M)
				    {
					gaiaGetPointXYZM (ring_in->Coords, iv,
							  &x, &y, &z, &m);
					gaiaSetPointXYZM (ring_out->Coords,
							  iv, x, y, z, m);
				    }
				  else
				    {
					gaiaGetPoint (ring_in->Coords, iv, &x,
						      &y);
					gaiaSetPoint (ring_out->Coords, iv, x,
						      y);
				    }
			      }
			}
		      goto skip;
		  }
		polyg = polyg->Next;
	    }
	skip:
	  if (result)
	    {
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		gaiaFreeGeomColl (result);
		sqlite3_result_blob (context, p_result, len, free);
	    }
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo);
}

static void
mbrs_eval (sqlite3_context * context, int argc, sqlite3_value ** argv,
	   int request)
{
/* SQL function:
/ MBRsomething(BLOB encoded GEOMETRY-1, BLOB encoded GEOMETRY-2)
/
/ returns:
/ 1 if the required spatial relationship between the two MBRs is TRUE
/ 0 otherwise
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int ret;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobMbr (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobMbr (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  ret = 0;
	  gaiaMbrGeometry (geo1);
	  gaiaMbrGeometry (geo2);
	  switch (request)
	    {
	    case GAIA_MBR_CONTAINS:
		ret = gaiaMbrsContains (geo1, geo2);
		break;
	    case GAIA_MBR_DISJOINT:
		ret = gaiaMbrsDisjoint (geo1, geo2);
		break;
	    case GAIA_MBR_EQUAL:
		ret = gaiaMbrsEqual (geo1, geo2);
		break;
	    case GAIA_MBR_INTERSECTS:
		ret = gaiaMbrsIntersects (geo1, geo2);
		break;
	    case GAIA_MBR_OVERLAPS:
		ret = gaiaMbrsOverlaps (geo1, geo2);
		break;
	    case GAIA_MBR_TOUCHES:
		ret = gaiaMbrsTouches (geo1, geo2);
		break;
	    case GAIA_MBR_WITHIN:
		ret = gaiaMbrsWithin (geo1, geo2);
		break;
	    }
	  if (ret < 0)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

/*
/ the following functions simply readdress the mbr_eval()
/ setting the appropriate request mode
*/

static void
fnct_MbrContains (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    mbrs_eval (context, argc, argv, GAIA_MBR_CONTAINS);
}

static void
fnct_MbrDisjoint (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    mbrs_eval (context, argc, argv, GAIA_MBR_DISJOINT);
}

static void
fnct_MbrEqual (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    mbrs_eval (context, argc, argv, GAIA_MBR_EQUAL);
}

static void
fnct_MbrIntersects (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    mbrs_eval (context, argc, argv, GAIA_MBR_INTERSECTS);
}

static void
fnct_MbrOverlaps (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    mbrs_eval (context, argc, argv, GAIA_MBR_OVERLAPS);
}

static void
fnct_MbrTouches (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    mbrs_eval (context, argc, argv, GAIA_MBR_TOUCHES);
}

static void
fnct_MbrWithin (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    mbrs_eval (context, argc, argv, GAIA_MBR_WITHIN);
}

static void
fnct_ShiftCoords (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ ShiftCoords(BLOBencoded geometry, shiftX, shiftY)
/
/ returns a new geometry that is the original one received, but with shifted coordinates
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    double shift_x;
    double shift_y;
    int int_value;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	shift_x = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  shift_x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	shift_y = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  shift_y = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaShiftCoords (geo, shift_x, shift_y);
	  gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_ScaleCoords (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ ScaleCoords(BLOBencoded geometry, scale_factor_x [, scale_factor_y])
/
/ returns a new geometry that is the original one received, but with scaled coordinates
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    double scale_x;
    double scale_y;
    int int_value;
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	scale_x = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  scale_x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (argc == 2)
	scale_y = scale_x;	/* this one is an isotropic scaling request */
    else
      {
	  /* an anisotropic scaling is requested */
	  if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	      scale_y = sqlite3_value_double (argv[2]);
	  else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	    {
		int_value = sqlite3_value_int (argv[2]);
		scale_y = int_value;
	    }
	  else
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaScaleCoords (geo, scale_x, scale_y);
	  gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_RotateCoords (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ RotateCoords(BLOBencoded geometry, angle)
/
/ returns a new geometry that is the original one received, but with rotated coordinates
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    double angle;
    int int_value;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	angle = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  angle = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaRotateCoords (geo, angle);
	  gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_ReflectCoords (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ ReflectCoords(BLOBencoded geometry, x_axis,  y_axis)
/
/ returns a new geometry that is the original one received, but with mirrored coordinates
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    int x_axis;
    int y_axis;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	x_axis = sqlite3_value_int (argv[1]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	y_axis = sqlite3_value_int (argv[2]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaReflectCoords (geo, x_axis, y_axis);
	  gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_SwapCoords (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ SwapCoords(BLOBencoded geometry)
/
/ returns a new geometry that is the original one received, but with swapped x- and y-coordinate
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaSwapCoords (geo);
	  gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
	  if (!p_result)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_blob (context, p_result, len, free);
      }
    gaiaFreeGeomColl (geo);
}

static int
get_ellipse_params (sqlite3 * sqlite, int srid, double *a, double *b,
		    double *rf)
{
/* 
/ retrieves the PROJ +ellps=xx [+a=xx +b=xx] params 
/from SPATIAL_SYS_REF table, if possible 
*/
    char proj4text[2048];
    char *p_proj;
    char *p_ellps;
    char *p_a;
    char *p_b;
    char *p_end;
    proj_params (sqlite, srid, proj4text);
    if (*proj4text == '\0')
	return 0;
/* parsing the proj4text geodesic string */
    p_proj = strstr (proj4text, "+proj=");
    p_ellps = strstr (proj4text, "+ellps=");
    p_a = strstr (proj4text, "+a=");
    p_b = strstr (proj4text, "+b=");
/* checking if +proj=longlat is true */
    if (!p_proj)
	return 0;
    p_end = strchr (p_proj, ' ');
    if (p_end)
	*p_end = '\0';
    if (strcmp (p_proj + 6, "longlat") != 0)
	return 0;
    if (p_ellps)
      {
	  /* trying to retrieve the ellipsoid params by name */
	  p_end = strchr (p_ellps, ' ');
	  if (p_end)
	      *p_end = '\0';
	  if (gaiaEllipseParams (p_ellps + 7, a, b, rf))
	      return 1;
      }
    if (p_a && p_b)
      {
	  /* trying to retrieve the +a=xx and +b=xx args */
	  p_end = strchr (p_a, ' ');
	  if (p_end)
	      *p_end = '\0';
	  p_end = strchr (p_b, ' ');
	  if (p_end)
	      *p_end = '\0';
	  *a = atof (p_a + 3);
	  *b = atof (p_b + 3);
	  *rf = 1.0 / ((*a - *b) / *a);
	  return 1;
      }
    return 0;
}

static void
fnct_FromEWKB (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromEWKB(EWKB encoded geometry)
/
/ returns the current geometry by parsing Geos/PostGis EWKB encoded string 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaFromEWKB (text);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_ToEWKB (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsEWKB(BLOB encoded geometry)
/
/ returns a text string corresponding to Geos/PostGIS EWKB notation 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
      {
	  sqlite3_result_null (context);
	  return;
      }
    else
      {
	  gaiaOutBufferInitialize (&out_buf);
	  gaiaToEWKB (&out_buf, geo);
	  if (out_buf.Error || out_buf.Buffer == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		len = out_buf.WriteOffset;
		sqlite3_result_text (context, out_buf.Buffer, len, free);
		out_buf.Buffer = NULL;
	    }
      }
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_ToEWKT (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ AsEWKT(BLOB encoded geometry)
/
/ returns the corresponding PostGIS EWKT encoded value
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    gaiaOutBuffer out_buf;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    gaiaOutBufferInitialize (&out_buf);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  gaiaToEWKT (&out_buf, geo);
	  if (out_buf.Error || out_buf.Buffer == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		len = out_buf.WriteOffset;
		sqlite3_result_text (context, out_buf.Buffer, len, free);
		out_buf.Buffer = NULL;
	    }
      }
    gaiaFreeGeomColl (geo);
    gaiaOutBufferReset (&out_buf);
}

static void
fnct_FromEWKT (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromEWKT(EWKT encoded geometry)
/
/ returns the current geometry by parsing EWKT  (PostGIS) encoded string 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseEWKT (text);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_FromGeoJSON (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromGeoJSON(GeoJSON encoded geometry)
/
/ returns the current geometry by parsing GeoJSON encoded string 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseGeoJSON (text);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_FromKml (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromKml(KML encoded geometry)
/
/ returns the current geometry by parsing KML encoded string 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseKml (text);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_FromGml (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromGml(GML encoded geometry)
/
/ returns the current geometry by parsing GML encoded string 
/ or NULL if any error is encountered
*/
    int len;
    unsigned char *p_result = NULL;
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseGml (text, sqlite);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geo, &p_result, &len);
    gaiaFreeGeomColl (geo);
    sqlite3_result_blob (context, p_result, len, free);
}

static void
fnct_LinesFromRings (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ LinesFromRings(BLOBencoded geometry, BOOL multi_linestring)
/
/ returns a new geometry [LINESTRING or MULTILINESTRING] representing 
/ the linearization for current (MULTI)POLYGON geometry
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr geom_new = NULL;
    int len;
    int multi_linestring;
    unsigned char *p_result = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (argc == 2)
      {
	  if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	      multi_linestring = sqlite3_value_int (argv[1]);
      }
    geom_new = gaiaLinearize (geo, multi_linestring);
    if (!geom_new)
	goto invalid;
    gaiaFreeGeomColl (geo);
    gaiaToSpatiaLiteBlobWkb (geom_new, &p_result, &len);
    gaiaFreeGeomColl (geom_new);
    sqlite3_result_blob (context, p_result, len, free);
    return;
  invalid:
    if (geo)
	gaiaFreeGeomColl (geo);
    sqlite3_result_null (context);
}

#ifndef OMIT_GEOS		/* including GEOS */

static void
fnct_BuildArea (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ BuildArea(BLOBencoded geometry)
/
/ Assuming that Geometry represents a set of sparse Linestrings,
/ this function will attempt to reassemble a single Polygon
/ (or a set of Polygons)
/ NULL is returned for invalid arguments
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaPolygonize (geo, 0);
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}


static void
fnct_Polygonize_step (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ Polygonize(BLOBencoded geom)
/
/ aggregate function - STEP
/
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geom;
    gaiaGeomCollPtr result;
    gaiaGeomCollPtr *p;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geom = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geom)
	return;
    p = sqlite3_aggregate_context (context, sizeof (gaiaGeomCollPtr));
    if (!(*p))
      {
	  /* this is the first row */
	  *p = geom;
      }
    else
      {
	  /* subsequent rows */
	  result = gaiaMergeGeometries (*p, geom);
	  gaiaFreeGeomColl (*p);
	  *p = result;
	  gaiaFreeGeomColl (geom);
      }
}

static void
fnct_Polygonize_final (sqlite3_context * context)
{
/* SQL function:
/ Polygonize(BLOBencoded geom)
/
/ aggregate function - FINAL
/
*/
    gaiaGeomCollPtr result;
    gaiaGeomCollPtr geom;
    gaiaGeomCollPtr *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    result = *p;
    if (!result)
	sqlite3_result_null (context);
    else
      {
	  geom = gaiaPolygonize (result, 0);
	  if (geom == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		geom->Srid = result->Srid;
		gaiaToSpatiaLiteBlobWkb (geom, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (geom);
	    }
	  gaiaFreeGeomColl (result);
      }
}

#endif /* end including GEOS */

static void
fnct_DissolveSegments (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ DissolveSegments(BLOBencoded geometry)
/
/ Dissolves any LINESTRING or RING into elementary segments
/ NULL is returned for invalid arguments
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaDissolveSegments (geo);
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_DissolvePoints (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ DissolvePoints(BLOBencoded geometry)
/
/ Dissolves any LINESTRING or RING into elementary Vertices
/ NULL is returned for invalid arguments
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaDissolvePoints (geo);
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_CollectionExtract (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ CollectionExtract(BLOBencoded geometry, Integer type)
/
/ Extracts from a GEOMETRYCOLLECTION any item of the required TYPE
/ 1=Point - 2=Linestring - 3=Polygon
/ NULL is returned for invalid arguments
*/
    unsigned char *p_blob;
    int n_bytes;
    int type;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	type = sqlite3_value_int (argv[1]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (type == 1 || type == 2 || type == 3)
	;
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  switch (type)
	    {
	    case 1:
		result = gaiaExtractPointsFromGeomColl (geo);
		break;
	    case 2:
		result = gaiaExtractLinestringsFromGeomColl (geo);
		break;
	    case 3:
		result = gaiaExtractPolygonsFromGeomColl (geo);
		break;
	    };
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

#ifndef OMIT_PROJ		/* including PROJ.4 */

static void
fnct_Transform (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Transform(BLOBencoded geometry, srid)
/
/ returns a new geometry that is the original one received, but with the new SRID [no coordinates translation is applied]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    int srid_from;
    int srid_to;
    char proj_from[2048];
    char proj_to[2048];
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
	srid_to = sqlite3_value_int (argv[1]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  *proj_from = '\0';
	  *proj_to = '\0';
	  srid_from = geo->Srid;
	  proj_params (sqlite, srid_from, proj_from);
	  proj_params (sqlite, srid_to, proj_to);
	  if (*proj_to == '\0' || *proj_from == '\0')
	    {
		gaiaFreeGeomColl (geo);
		sqlite3_result_null (context);
		return;
	    }
	  result = gaiaTransform (geo, proj_from, proj_to);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = srid_to;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

#endif /* end including PROJ.4 */

#ifndef OMIT_GEOS		/* including GEOS */

static void
fnct_Boundary (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Boundary(BLOB encoded geometry)
/
/ returns the combinatorial boundary for current geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr boundary;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (gaiaIsEmpty (geo))
	      sqlite3_result_null (context);
	  else
	    {
		boundary = gaiaBoundary (geo);
		if (!boundary)
		    sqlite3_result_null (context);
		else
		  {
		      gaiaToSpatiaLiteBlobWkb (boundary, &p_result, &len);
		      gaiaFreeGeomColl (boundary);
		      sqlite3_result_blob (context, p_result, len, free);
		  }
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_IsClosed (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ IsClosed(BLOB encoded LINESTRING or MULTILINESTRING geometry)
/
/ returns:
/ 1 if this LINESTRING is closed [or if this is a MULTILINESTRING and every LINESTRINGs are closed] 
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaLinestringPtr line;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_int (context, -1);
    else
      {
	  line = simpleLinestring (geo);
	  if (!line < 0)
	      sqlite3_result_int (context, -1);
	  else
	      sqlite3_result_int (context, gaiaIsClosed (line));
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_IsSimple (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ IsSimple(BLOB encoded GEOMETRY)
/
/ returns:
/ 1 if this GEOMETRY is simple
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int ret;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaIsSimple (geo);
	  if (ret < 0)
	      sqlite3_result_int (context, -1);
	  else
	      sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_IsRing (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ IsRing(BLOB encoded LINESTRING geometry)
/
/ returns:
/ 1 if this LINESTRING is a valid RING
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int ret;
    gaiaGeomCollPtr geo = NULL;
    gaiaLinestringPtr line;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_int (context, -1);
    else
      {
	  line = simpleLinestring (geo);
	  if (!line < 0)
	      sqlite3_result_int (context, -1);
	  else
	    {
		ret = gaiaIsRing (line);
		sqlite3_result_int (context, ret);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_IsValid (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ IsValid(BLOB encoded GEOMETRY)
/
/ returns:
/ 1 if this GEOMETRY is a valid one
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int ret;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaIsValid (geo);
	  if (ret < 0)
	      sqlite3_result_int (context, -1);
	  else
	      sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Length (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GLength(BLOB encoded GEOMETRYCOLLECTION)
/
/ returns  the total length for current geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double length = 0.0;
    int ret;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  ret = gaiaGeomCollLength (geo, &length);
	  if (!ret)
	      sqlite3_result_null (context);
	  sqlite3_result_double (context, length);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Area (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Area(BLOB encoded GEOMETRYCOLLECTION)
/
/ returns the total area for current geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double area = 0.0;
    int ret;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  ret = gaiaGeomCollArea (geo, &area);
	  if (!ret)
	      sqlite3_result_null (context);
	  sqlite3_result_double (context, area);
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Centroid (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Centroid(BLOBencoded POLYGON or MULTIPOLYGON geometry)
/
/ returns a POINT representing the centroid for current POLYGON / MULTIPOLYGON geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    int ret;
    double x;
    double y;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (gaiaIsEmpty (geo))
	      sqlite3_result_null (context);
	  else
	    {
		ret = gaiaGeomCollCentroid (geo, &x, &y);
		if (!ret)
		    sqlite3_result_null (context);
		else
		  {
		      result = gaiaAllocGeomColl ();
		      result->Srid = geo->Srid;
		      gaiaAddPointToGeomColl (result, x, y);
		      gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		      gaiaFreeGeomColl (result);
		      sqlite3_result_blob (context, p_result, len, free);
		  }
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_PointOnSurface (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ PointOnSurface(BLOBencoded POLYGON or MULTIPOLYGON geometry)
/
/ returns a POINT guaranteed to lie on the Surface
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    double x;
    double y;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (!gaiaGetPointOnSurface (geo, &x, &y))
	      sqlite3_result_null (context);
	  else
	    {
		result = gaiaAllocGeomColl ();
		gaiaAddPointToGeomColl (result, x, y);
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		gaiaFreeGeomColl (result);
		sqlite3_result_blob (context, p_result, len, free);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Simplify (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Simplify(BLOBencoded geometry, tolerance)
/
/ returns a new geometry that is a caricature of the original one received, but simplified using the Douglas-Peuker algorihtm
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    int int_value;
    double tolerance;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	tolerance = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  tolerance = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaGeomCollSimplify (geo, tolerance);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_SimplifyPreserveTopology (sqlite3_context * context, int argc,
			       sqlite3_value ** argv)
{
/* SQL function:
/ SimplifyPreserveTopology(BLOBencoded geometry, tolerance)
/
/ returns a new geometry that is a caricature of the original one received, but simplified using the Douglas-Peuker algorihtm [preserving topology]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    int int_value;
    double tolerance;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	tolerance = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  tolerance = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaGeomCollSimplifyPreserveTopology (geo, tolerance);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_ConvexHull (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ ConvexHull(BLOBencoded geometry)
/
/ returns a new geometry representing the CONVEX HULL for current geometry
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int len;
    unsigned char *p_result = NULL;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaConvexHull (geo);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Buffer (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Buffer(BLOBencoded geometry, radius)
/
/ returns a new geometry representing the BUFFER for current geometry
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    double radius;
    int int_value;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	radius = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  radius = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaGeomCollBuffer (geo, radius, 30);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_Intersection (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Intersection(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns a new geometry representing the INTERSECTION of both geometries
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaGeometryIntersection (geo1, geo2);
	  if (!result)
	      sqlite3_result_null (context);
	  else if (gaiaIsEmpty (result))
	    {
		gaiaFreeGeomColl (result);
		sqlite3_result_null (context);
	    }
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Union_step (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Union(BLOBencoded geom)
/
/ aggregate function - STEP
/
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geom;
    gaiaGeomCollPtr result;
    gaiaGeomCollPtr *p;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geom = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geom)
	return;
    p = sqlite3_aggregate_context (context, sizeof (gaiaGeomCollPtr));
    if (!(*p))
      {
	  /* this is the first row */
	  *p = geom;
      }
    else
      {
	  /* subsequent rows */
	  result = gaiaGeometryUnion (*p, geom);
	  gaiaFreeGeomColl (*p);
	  *p = result;
	  gaiaFreeGeomColl (geom);
      }
}

static void
fnct_Union_final (sqlite3_context * context)
{
/* SQL function:
/ Union(BLOBencoded geom)
/
/ aggregate function - FINAL
/
*/
    gaiaGeomCollPtr result;
    gaiaGeomCollPtr *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    result = *p;
    if (!result)
	sqlite3_result_null (context);
    else if (gaiaIsEmpty (result))
      {
	  gaiaFreeGeomColl (result);
	  sqlite3_result_null (context);
      }
    else
      {
	  /* builds the BLOB geometry to be returned */
	  int len;
	  unsigned char *p_result = NULL;
	  gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
	  sqlite3_result_blob (context, p_result, len, free);
	  gaiaFreeGeomColl (result);
      }
}

static void
fnct_Union (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Union(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns a new geometry representing the UNION of both geometries
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaGeometryUnion (geo1, geo2);
	  if (!result)
	      sqlite3_result_null (context);
	  else if (gaiaIsEmpty (result))
	    {
		gaiaFreeGeomColl (result);
		sqlite3_result_null (context);
	    }
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Difference (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Difference(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns a new geometry representing the DIFFERENCE of both geometries
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaGeometryDifference (geo1, geo2);
	  if (!result)
	      sqlite3_result_null (context);
	  else if (gaiaIsEmpty (result))
	    {
		gaiaFreeGeomColl (result);
		sqlite3_result_null (context);
	    }
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_SymDifference (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ SymDifference(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns a new geometry representing the SYMMETRIC DIFFERENCE of both geometries
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaGeometrySymDifference (geo1, geo2);
	  if (!result)
	      sqlite3_result_null (context);
	  else if (gaiaIsEmpty (result))
	    {
		gaiaFreeGeomColl (result);
		sqlite3_result_null (context);
	    }
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Equals (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Equals(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if the two geometries are "spatially equal"
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollEquals (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Intersects (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Intersects(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if the two geometries do "spatially intersects"
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollIntersects (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Disjoint (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Disjoint(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if the two geometries are "spatially disjoint"
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollDisjoint (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Overlaps (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Overlaps(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if the two geometries do "spatially overlaps"
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollOverlaps (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Crosses (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Crosses(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if the two geometries do "spatially crosses"
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollCrosses (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Touches (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Touches(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if the two geometries do "spatially touches"
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollTouches (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Within (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Within(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if GEOM-1 is completely contained within GEOM-2
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollWithin (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Contains (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Contains(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if GEOM-1 completely contains GEOM-2
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollContains (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Relate (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Relate(BLOBencoded geom1, BLOBencoded geom2, string pattern)
/
/ returns:
/ 1 if GEOM-1 and GEOM-2 have a spatial relationship as specified by the patternMatrix 
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    const unsigned char *pattern;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[2]) != SQLITE_TEXT)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    pattern = sqlite3_value_text (argv[2]);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollRelate (geo1, geo2, (char *) pattern);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Distance (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Distance(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns the distance between GEOM-1 and GEOM-2
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    double dist;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  ret = gaiaGeomCollDistance (geo1, geo2, &dist);
	  if (!ret)
	      sqlite3_result_null (context);
	  sqlite3_result_double (context, dist);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_PtDistWithin (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ PtDistWithin(BLOBencoded geom1, BLOBencoded geom2, double dist 
/ [, boolen use_spheroid])
/
/ returns TRUE if the distance between GEOM-1 and GEOM-2
/ is less or equal to dist
/
/ - if both geom1 and geom2 are in the 4326 (WGS84) SRID,
/   (and does actually contains a single POINT each one)
/   dist is assumed to be measured in Meters
/ - in this case the optional arg use_spheroid is
/   checked to determine if geodesic distance has to be
/   computed on the sphere (quickest) or on the spheroid 
/   default: use_spheroid = FALSE
/ 
/ in any other case the "plain" distance is evaluated
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    double ref_dist;
    int use_spheroid = 0;
    double x0;
    double y0;
    double x1;
    double y1;
    int pt0 = 0;
    int ln0 = 0;
    int pg0 = 0;
    int pt1 = 0;
    int ln1 = 0;
    int pg1 = 0;
    double dist;
    double a;
    double b;
    double rf;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER
	|| sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	;
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (argc == 4)
      {
	  /* optional use_spheroid arg */
	  if (sqlite3_value_type (argv[3]) != SQLITE_INTEGER)
	    {
		sqlite3_result_null (context);
		return;
	    }
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int dst = sqlite3_value_int (argv[2]);
	  ref_dist = dst;
      }
    else
	ref_dist = sqlite3_value_double (argv[2]);
    if (argc == 4)
	use_spheroid = sqlite3_value_int (argv[3]);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  if (geo1->Srid == 4326 && geo2->Srid == 4326)
	    {
		/* checking for single points */
		pt = geo1->FirstPoint;
		while (pt)
		  {
		      x0 = pt->X;
		      y0 = pt->Y;
		      pt0++;
		      pt = pt->Next;
		  }
		ln = geo1->FirstLinestring;
		while (ln)
		  {
		      ln0++;
		      ln = ln->Next;
		  }
		pg = geo1->FirstPolygon;
		while (pg)
		  {
		      pg0++;
		      pg = pg->Next;
		  }
		pt = geo2->FirstPoint;
		while (pt)
		  {
		      x1 = pt->X;
		      y1 = pt->Y;
		      pt1++;
		      pt = pt->Next;
		  }
		ln = geo2->FirstLinestring;
		while (ln)
		  {
		      ln1++;
		      ln = ln->Next;
		  }
		pg = geo2->FirstPolygon;
		while (pg)
		  {
		      pg1++;
		      pg = pg->Next;
		  }
		if (pt0 == 1 && pt1 == 1 && ln0 == 0 && ln1 == 0 && pg0 == 0
		    && pg1 == 0)
		  {
		      /* using geodesic distance */
		      a = 6378137.0;
		      rf = 298.257223563;
		      b = (a * (1.0 - (1.0 / rf)));
		      if (use_spheroid)
			{
			    dist =
				gaiaGeodesicDistance (a, b, rf, y0, x0, y1, x1);
			    if (dist <= ref_dist)
				sqlite3_result_int (context, 1);
			    else
				sqlite3_result_int (context, 0);
			}
		      else
			{
			    dist =
				gaiaGreatCircleDistance (a, b, y0, x0, y1, x1);
			    if (dist <= ref_dist)
				sqlite3_result_int (context, 1);
			    else
				sqlite3_result_int (context, 0);
			}
		  }
		goto stop;
	    }
/* defaulting to flat distance */
	  ret = gaiaGeomCollDistance (geo1, geo2, &dist);
	  if (!ret)
	      sqlite3_result_null (context);
	  if (dist <= ref_dist)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
      }
  stop:
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
geos_error (const char *fmt, ...)
{
/* reporting some GEOS error */
    va_list ap;
    char msg[2048];
    va_start (ap, fmt);
    vsprintf (msg, fmt, ap);
    va_end (ap);
    fprintf (stderr, "GEOS error: %s\n", msg);
    gaiaSetGeosErrorMsg (msg);
}


static void
geos_warning (const char *fmt, ...)
{
/* reporting some GEOS warning */
    va_list ap;
    char msg[2048];
    va_start (ap, fmt);
    vsprintf (msg, fmt, ap);
    va_end (ap);
    fprintf (stderr, "GEOS warning: %s\n", msg);
    gaiaSetGeosWarningMsg (msg);
}

static void
fnct_aux_polygonize (sqlite3_context * context, gaiaGeomCollPtr geom_org,
		     int force_multipolygon, int allow_multipolygon)
{
/* a  common function performing any kind of polygonization op */
    gaiaGeomCollPtr geom_new = NULL;
    int len;
    unsigned char *p_result = NULL;
    gaiaPolygonPtr pg;
    int pgs = 0;
    if (!geom_org)
	goto invalid;
    geom_new = gaiaPolygonize (geom_org, force_multipolygon);
    if (!geom_new)
	goto invalid;
    gaiaFreeGeomColl (geom_org);
    pg = geom_new->FirstPolygon;
    while (pg)
      {
	  pgs++;
	  pg = pg->Next;
      }
    if (pgs > 1 && allow_multipolygon == 0)
      {
	  /* invalid: a POLYGON is expected !!! */
	  gaiaFreeGeomColl (geom_new);
	  sqlite3_result_null (context);
	  return;
      }
    gaiaToSpatiaLiteBlobWkb (geom_new, &p_result, &len);
    gaiaFreeGeomColl (geom_new);
    sqlite3_result_blob (context, p_result, len, free);
    return;
  invalid:
    if (geom_org)
	gaiaFreeGeomColl (geom_org);
    sqlite3_result_null (context);
}

/*
/ the following functions performs initial argument checking, 
/ and then readdressing the request to fnct_aux_polygonize()
/ for actual processing
*/

static void
fnct_BdPolyFromText1 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ BdPolyFromText(WKT encoded MULTILINESTRING)
/
/ returns the current geometry [POLYGON] by parsing a WKT encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseWkt (text, -1);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = -1;
    fnct_aux_polygonize (context, geo, 0, 0);
    return;
}

static void
fnct_BdPolyFromText2 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ BdPolyFromText(WKT encoded MULTILINESTRING, SRID)
/
/ returns the current geometry [POLYGON] by parsing a WKT encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseWkt (text, -1);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = sqlite3_value_int (argv[1]);
    fnct_aux_polygonize (context, geo, 0, 0);
    return;
}

static void
fnct_BdMPolyFromText1 (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ BdMPolyFromText(WKT encoded MULTILINESTRING)
/
/ returns the current geometry [MULTIPOLYGON] by parsing a WKT encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseWkt (text, -1);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = -1;
    fnct_aux_polygonize (context, geo, 1, 1);
    return;
}

static void
fnct_BdMPolyFromText2 (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ BdMPolyFromText(WKT encoded MULTILINESTRING, SRID)
/
/ returns the current geometry [MULTIPOLYGON] by parsing a WKT encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    const unsigned char *text;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_TEXT)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    text = sqlite3_value_text (argv[0]);
    geo = gaiaParseWkt (text, -1);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = sqlite3_value_int (argv[1]);
    fnct_aux_polygonize (context, geo, 1, 1);
    return;
}

static void
fnct_BdPolyFromWKB1 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ BdPolyFromWKB(WKB encoded MULTILINESTRING)
/
/ returns the current geometry [POLYGON] by parsing a WKB encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    int n_bytes;
    const unsigned char *wkb;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    wkb = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!check_wkb (wkb, n_bytes, -1))
	return;
    geo = gaiaFromWkb (wkb, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = -1;
    fnct_aux_polygonize (context, geo, 0, 0);
    return;
}

static void
fnct_BdPolyFromWKB2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ BdPolyFromWKB(WKB encoded MULTILINESTRING)
/
/ returns the current geometry [POLYGON] by parsing a WKB encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    int n_bytes;
    const unsigned char *wkb;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    wkb = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!check_wkb (wkb, n_bytes, -1))
	return;
    geo = gaiaFromWkb (wkb, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = sqlite3_value_int (argv[1]);
    fnct_aux_polygonize (context, geo, 0, 0);
    return;
}

static void
fnct_BdMPolyFromWKB1 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ BdMPolyFromWKB(WKB encoded MULTILINESTRING)
/
/ returns the current geometry [MULTIPOLYGON] by parsing a WKB encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    int n_bytes;
    const unsigned char *wkb;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    wkb = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!check_wkb (wkb, n_bytes, -1))
	return;
    geo = gaiaFromWkb (wkb, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = -1;
    fnct_aux_polygonize (context, geo, 1, 1);
    return;
}

static void
fnct_BdMPolyFromWKB2 (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ BdMPolyFromWKB(WKB encoded MULTILINESTRING)
/
/ returns the current geometry [MULTIPOLYGON] by parsing a WKB encoded MULTILINESTRING 
/ or NULL if any error is encountered
/
*/
    int n_bytes;
    const unsigned char *wkb;
    gaiaGeomCollPtr geo = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_INTEGER)
      {
	  sqlite3_result_null (context);
	  return;
      }
    wkb = sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (!check_wkb (wkb, n_bytes, -1))
	return;
    geo = gaiaFromWkb (wkb, n_bytes);
    if (geo == NULL)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (geo->DeclaredType != GAIA_MULTILINESTRING)
      {
	  sqlite3_result_null (context);
	  return;
      }
    geo->Srid = sqlite3_value_int (argv[1]);
    fnct_aux_polygonize (context, geo, 1, 1);
    return;
}

#ifdef GEOS_ADVANCED		/* GEOS advanced and experimental features */

static void
fnct_OffsetCurve (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ OffsetCurve(BLOBencoded geometry, radius, left-or-right-side)
/
/ returns a new geometry representing the OFFSET-CURVE for current geometry
/ [a LINESTRING is expected]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    double radius;
    int int_value;
    int left_right;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	radius = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  radius = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	left_right = sqlite3_value_int (argv[2]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaOffsetCurve (geo, radius, 16, left_right);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_SingleSidedBuffer (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ SingleSidedBuffer(BLOBencoded geometry, radius, left-or-right-side)
/
/ returns a new geometry representing the SingleSided BUFFER 
/ for current geometry [a LINESTRING is expected]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    double radius;
    int int_value;
    int left_right;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	radius = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  radius = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
	left_right = sqlite3_value_int (argv[2]);
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaSingleSidedBuffer (geo, radius, 16, left_right);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_HausdorffDistance (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ HausdorffDistance(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns the discrete Hausdorff distance between GEOM-1 and GEOM-2
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    double dist;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_null (context);
    else
      {
	  ret = gaiaHausdorffDistance (geo1, geo2, &dist);
	  if (!ret)
	      sqlite3_result_null (context);
	  sqlite3_result_double (context, dist);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_SharedPaths (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ SharedPaths(BLOBencoded geometry1, BLOBencoded geometry2)
/
/ returns a new geometry representing common (shared) Edges
/ [two LINESTRINGs/MULTILINESTRINGs are expected]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo1 == NULL || geo2 == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaSharedPaths (geo1, geo2);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo1->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Covers (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Covers(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if GEOM-1 "spatially covers" GEOM-2
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollCovers (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_CoveredBy (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ CoveredBy(BLOBencoded geom1, BLOBencoded geom2)
/
/ returns:
/ 1 if GEOM-1 is "spatially covered by" GEOM-2
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    int ret;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo1 || !geo2)
	sqlite3_result_int (context, -1);
    else
      {
	  ret = gaiaGeomCollCoveredBy (geo1, geo2);
	  sqlite3_result_int (context, ret);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_LineInterpolatePoint (sqlite3_context * context, int argc,
			   sqlite3_value ** argv)
{
/* SQL function:
/ LineInterpolatePoint(BLOBencoded geometry1, double fraction)
/
/ returns a new geometry representing a point interpolated along a line
/ [a LINESTRING is expected / fraction ranging from 0.0 to 1.0]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int int_value;
    double fraction;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	fraction = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  fraction = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaLineInterpolatePoint (geo, fraction);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_LineLocatePoint (sqlite3_context * context, int argc,
		      sqlite3_value ** argv)
{
/* SQL function:
/ LineLocatePoint(BLOBencoded geometry1, BLOBencoded geometry2)
/
/ return a number (between 0.0 and 1.0) representing the location 
/ of the closest point on LineString to the given Point, as a fraction 
/ of total 2d line length
/
/ - geom1 is expected to represent some LINESTRING
/ - geom2 is expected to represent some POINT
*/
    unsigned char *p_blob;
    int n_bytes;
    double fraction;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo1 == NULL || geo2 == NULL)
	sqlite3_result_null (context);
    else
      {
	  fraction = gaiaLineLocatePoint (geo1, geo2);
	  if (fraction >= 0.0 && fraction <= 1.0)
	      sqlite3_result_double (context, fraction);
	  else
	      sqlite3_result_null (context);
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_LineSubstring (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ LineSubstring(BLOBencoded geometry1, double start_fraction, double end_fraction)
/
/ Return a Linestring being a substring of the input one starting and ending at 
/ the given fractions of total 2d length [fractions ranging from 0.0 to 1.0]
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int int_value;
    double fraction1;
    double fraction2;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	fraction1 = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  fraction1 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	fraction2 = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  fraction2 = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaLineSubstring (geo, fraction1, fraction2);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

static void
fnct_ClosestPoint (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ ClosestPoint(BLOBencoded geometry1, BLOBencoded geometry2)
/
/ Returns the Point on geom1 that is closest to geom2
/ NULL is returned for invalid arguments (or if distance is ZERO)
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo1 == NULL || geo2 == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaShortestLine (geo1, geo2);
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else if (result->FirstLinestring == NULL)
	    {
		gaiaFreeGeomColl (result);
		sqlite3_result_null (context);
	    }
	  else
	    {
		/* builds the BLOB geometry to be returned */
		double x;
		double y;
		double z;
		double m;
		int len;
		unsigned char *p_result = NULL;
		gaiaGeomCollPtr pt = NULL;
		gaiaLinestringPtr ln = result->FirstLinestring;
		if (ln->DimensionModel == GAIA_XY_Z)
		    pt = gaiaAllocGeomCollXYZ ();
		else if (ln->DimensionModel == GAIA_XY_M)
		    pt = gaiaAllocGeomCollXYM ();
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		    pt = gaiaAllocGeomCollXYZM ();
		else
		    pt = gaiaAllocGeomColl ();
		if (ln->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ln->Coords, 0, &x, &y, &z);
		      gaiaAddPointToGeomCollXYZ (pt, x, y, z);
		  }
		else if (ln->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ln->Coords, 0, &x, &y, &m);
		      gaiaAddPointToGeomCollXYM (pt, x, y, m);
		  }
		else if (ln->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ln->Coords, 0, &x, &y, &z, &m);
		      gaiaAddPointToGeomCollXYZM (pt, x, y, z, m);
		  }
		else
		  {
		      gaiaGetPoint (ln->Coords, 0, &x, &y);
		      gaiaAddPointToGeomColl (pt, x, y);
		  }
		pt->Srid = geo1->Srid;
		gaiaToSpatiaLiteBlobWkb (pt, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
		gaiaFreeGeomColl (pt);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_ShortestLine (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ ShortestLine(BLOBencoded geometry1, BLOBencoded geometry2)
/
/ Returns the shortest line between two geometries
/ NULL is returned for invalid arguments (or if distance is ZERO)
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo1 == NULL || geo2 == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaShortestLine (geo1, geo2);
	  sqlite3_result_null (context);
	  if (!result)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo1->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_Snap (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ Snap(BLOBencoded geometry1, BLOBencoded geometry2, double tolerance)
/
/ Returns a new Geometry corresponding to geom1 snapped to geom2
/ and using the given tolerance
/ NULL is returned for invalid arguments (or if distance is ZERO)
*/
    unsigned char *p_blob;
    int n_bytes;
    int int_value;
    double tolerance;
    gaiaGeomCollPtr geo1 = NULL;
    gaiaGeomCollPtr geo2 = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[2]) == SQLITE_FLOAT)
	tolerance = sqlite3_value_double (argv[2]);
    else if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[2]);
	  tolerance = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo1 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    p_blob = (unsigned char *) sqlite3_value_blob (argv[1]);
    n_bytes = sqlite3_value_bytes (argv[1]);
    geo2 = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo1 == NULL || geo2 == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaSnap (geo1, geo2, tolerance);
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo1->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo1);
    gaiaFreeGeomColl (geo2);
}

static void
fnct_LineMerge (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ LineMerge(BLOBencoded geometry)
/
/ Assuming that Geometry represents a set of sparse Linestrings,
/ this function will attempt to reassemble a single line
/ (or a set of lines)
/ NULL is returned for invalid arguments
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaLineMerge (geo);
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}


static void
fnct_UnaryUnion (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ UnaryUnion(BLOBencoded geometry)
/
/ exactly like Union, but using a single Collection
/ NULL is returned for invalid arguments
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geo = NULL;
    gaiaGeomCollPtr result;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (geo == NULL)
	sqlite3_result_null (context);
    else
      {
	  result = gaiaUnaryUnion (geo);
	  if (result == NULL)
	      sqlite3_result_null (context);
	  else
	    {
		/* builds the BLOB geometry to be returned */
		int len;
		unsigned char *p_result = NULL;
		result->Srid = geo->Srid;
		gaiaToSpatiaLiteBlobWkb (result, &p_result, &len);
		sqlite3_result_blob (context, p_result, len, free);
		gaiaFreeGeomColl (result);
	    }
      }
    gaiaFreeGeomColl (geo);
}

#endif /* end GEOS advanced and experimental features */

#endif /* end including GEOS */

#ifndef OMIT_MATHSQL		/* supporting SQL math functions */

static void
fnct_math_acos (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ acos(double X)
/
/ Returns the arc cosine of X, that is, the value whose cosine is X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    errno = 0;
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = acos (sqlite3_value_double (argv[0]));
	  if (errno == EDOM)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = acos (x);
	  if (errno == EDOM)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_asin (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ asin(double X)
/
/ Returns the arc sine of X, that is, the value whose sine is X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    errno = 0;
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = asin (sqlite3_value_double (argv[0]));
	  if (errno == EDOM)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = asin (x);
	  if (errno == EDOM)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_atan (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ atan(double X)
/
/ Returns the arc tangent of X, that is, the value whose tangent is X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = atan (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = atan (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_ceil (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ ceil(double X)
/
/ Returns the smallest integer value not less than X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = ceil (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = ceil (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_cos (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ cos(double X)
/
/ Returns the cosine of X, where X is given in radians
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = cos (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = cos (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_cot (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ cot(double X)
/
/ Returns the cotangent of X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    double tang;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    tang = tan (x);
    if (tang == 0.0)
      {
	  sqlite3_result_null (context);
	  return;
      }
    x = 1.0 / tang;
    sqlite3_result_double (context, x);
}

static void
fnct_math_degrees (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ degrees(double X)
/
/ Returns the argument X, converted from radians to degrees
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    x = x * 57.29577951308232;
    sqlite3_result_double (context, x);
}

static void
fnct_math_exp (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ exp(double X)
/
/ Returns the value of e (the base of natural logarithms) raised to the power of X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = exp (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = exp (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_floor (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ floor(double X)
/
/ Returns the largest integer value not greater than X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = floor (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = floor (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_logn (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ log(double X)
/
/ Returns the natural logarithm of X; that is, the base-e logarithm of X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    errno = 0;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = log (sqlite3_value_double (argv[0]));
	  if (errno == EDOM || errno == ERANGE)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = log (x);
	  if (errno == EDOM || errno == ERANGE)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_logn2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ log(double B, double X)
/
/ Returns the logarithm of X to the base B
/ or NULL if any error is encountered
*/
    int int_value;
    double x = 0.0;
    double b = 1.0;
    double log1;
    double log2;
    errno = 0;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	b = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  b = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (x <= 0.0 || b <= 1.0)
      {
	  sqlite3_result_null (context);
	  return;
      }
    log1 = log (x);
    if (errno == EDOM || errno == ERANGE)
      {
	  sqlite3_result_null (context);
	  return;
      }
    log2 = log (b);
    if (errno == EDOM || errno == ERANGE)
      {
	  sqlite3_result_null (context);
	  return;
      }
    sqlite3_result_double (context, log1 / log2);
}

static void
fnct_math_log_2 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ log2(double X)
/
/ Returns the base-2 logarithm of X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    double log1;
    double log2;
    errno = 0;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    log1 = log (x);
    if (errno == EDOM || errno == ERANGE)
      {
	  sqlite3_result_null (context);
	  return;
      }
    log2 = log (2.0);
    sqlite3_result_double (context, log1 / log2);
}

static void
fnct_math_log_10 (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ log10(double X)
/
/ Returns the base-10 logarithm of X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    double log1;
    double log2;
    GAIA_UNUSED ();
    errno = 0;
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    log1 = log (x);
    if (errno == EDOM || errno == ERANGE)
      {
	  sqlite3_result_null (context);
	  return;
      }
    log2 = log (10.0);
    sqlite3_result_double (context, log1 / log2);
}

static void
fnct_math_pi (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ pi(void)
/
/ Returns the value of (pi)
*/
    GAIA_UNUSED ();
    sqlite3_result_double (context, 3.14159265358979323846);
}

static void
fnct_math_pow (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ pow(double X, double Y)
/
/ Returns the value of X raised to the power of Y.
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    double y;
    double p;
    GAIA_UNUSED ();
    errno = 0;
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (sqlite3_value_type (argv[1]) == SQLITE_FLOAT)
	y = sqlite3_value_double (argv[1]);
    else if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[1]);
	  y = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    p = pow (x, y);
    if (errno == EDOM)
	sqlite3_result_null (context);
    else
	sqlite3_result_double (context, p);
}

static void
fnct_math_stddev_step (sqlite3_context * context, int argc,
		       sqlite3_value ** argv)
{
/* SQL function:
/ stddev_pop(double X)
/ stddev_samp(double X)
/ var_pop(double X)
/ var_samp(double X)
/
/ aggregate function - STEP
/
*/
    struct stddev_str *p;
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
	return;
    p = sqlite3_aggregate_context (context, sizeof (struct stddev_str));
    if (!(p->cleaned))
      {
	  p->cleaned = 1;
	  p->mean = x;
	  p->quot = 0.0;
	  p->count = 0.0;
      }
    p->count += 1.0;
    p->quot =
	p->quot +
	(((p->count - 1.0) * ((x - p->mean) * (x - p->mean))) / p->count);
    p->mean = p->mean + ((x - p->mean) / p->count);
}

static void
fnct_math_stddev_pop_final (sqlite3_context * context)
{
/* SQL function:
/ stddev_pop(double X)
/ aggregate function -  FINAL
/
*/
    double x;
    struct stddev_str *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    x = sqrt (p->quot / p->count);
    sqlite3_result_double (context, x);
}

static void
fnct_math_stddev_samp_final (sqlite3_context * context)
{
/* SQL function:
/ stddev_samp(double X)
/ aggregate function -  FINAL
/
*/
    double x;
    struct stddev_str *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    x = sqrt (p->quot / (p->count - 1.0));
    sqlite3_result_double (context, x);
}

static void
fnct_math_var_pop_final (sqlite3_context * context)
{
/* SQL function:
/ var_pop(double X)
/ aggregate function -  FINAL
/
*/
    double x;
    struct stddev_str *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    x = p->quot / p->count;
    sqlite3_result_double (context, x);
}

static void
fnct_math_var_samp_final (sqlite3_context * context)
{
/* SQL function:
/ var_samp(double X)
/ aggregate function -  FINAL
/
*/
    double x;
    struct stddev_str *p = sqlite3_aggregate_context (context, 0);
    if (!p)
      {
	  sqlite3_result_null (context);
	  return;
      }
    x = p->quot / (p->count - 1.0);
    sqlite3_result_double (context, x);
}

static void
fnct_math_radians (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ radians(double X)
/
/ Returns the argument X, converted from degrees to radians
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    x = x * .0174532925199432958;
    sqlite3_result_double (context, x);
}


static void
fnct_math_round (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ round(double X)
/
/ Returns the the nearest integer, but round halfway cases away from zero
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = math_round (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = math_round (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_sign (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ sign(double X)
/
/ Returns the sign of the argument as -1, 0, or 1, depending on whether X is negative, zero, or positive
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	x = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (x > 0.0)
	sqlite3_result_double (context, 1.0);
    else if (x < 0.0)
	sqlite3_result_double (context, -1.0);
    else
	sqlite3_result_double (context, 0.0);
}

static void
fnct_math_sin (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ sin(double X)
/
/ Returns the sine of X, where X is given in radians
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = sin (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = sin (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_sqrt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ sqrt(double X)
/
/ Returns the square root of a non-negative number X
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    errno = 0;
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = sqrt (sqlite3_value_double (argv[0]));
	  if (errno)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = sqrt (x);
	  if (errno == EDOM)
	      sqlite3_result_null (context);
	  else
	      sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

static void
fnct_math_tan (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ tan(double X)
/
/ Returns the tangent of X, where X is given in radians
/ or NULL if any error is encountered
*/
    int int_value;
    double x;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
      {
	  x = tan (sqlite3_value_double (argv[0]));
	  sqlite3_result_double (context, x);
      }
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  x = int_value;
	  x = tan (x);
	  sqlite3_result_double (context, x);
      }
    else
	sqlite3_result_null (context);
}

#endif /* end supporting SQL math functions */

static void
fnct_GeomFromExifGpsBlob (sqlite3_context * context, int argc,
			  sqlite3_value ** argv)
{
/* SQL function:
/ GeomFromExifGpsBlob(BLOB encoded image)
/
/ returns:
/ a POINT geometry
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    gaiaGeomCollPtr geom;
    unsigned char *geoblob;
    int geosize;
    double longitude;
    double latitude;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    if (gaiaGetGpsCoords (p_blob, n_bytes, &longitude, &latitude))
      {
	  geom = gaiaAllocGeomColl ();
	  geom->Srid = 4326;
	  gaiaAddPointToGeomColl (geom, longitude, latitude);
	  gaiaToSpatiaLiteBlobWkb (geom, &geoblob, &geosize);
	  gaiaFreeGeomColl (geom);
	  sqlite3_result_blob (context, geoblob, geosize, free);
      }
    else
	sqlite3_result_null (context);
}

static void
blob_guess (sqlite3_context * context, int argc, sqlite3_value ** argv,
	    int request)
{
/* SQL function:
/ IsGifBlob(BLOB encoded image)
/ IsPngBlob, IsJpegBlob, IsExifBlob, IsExifGpsBlob, IsTiffBlob,
/ IsZipBlob, IsPdfBlob,IsGeometryBlob
/
/ returns:
/ 1 if the required BLOB_TYPE is TRUE
/ 0 otherwise
/ or -1 if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    int blob_type;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_int (context, -1);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    blob_type = gaiaGuessBlobType (p_blob, n_bytes);
    if (request == GAIA_GEOMETRY_BLOB)
      {
	  if (blob_type == GAIA_GEOMETRY_BLOB)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_ZIP_BLOB)
      {
	  if (blob_type == GAIA_ZIP_BLOB)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_PDF_BLOB)
      {
	  if (blob_type == GAIA_PDF_BLOB)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_TIFF_BLOB)
      {
	  if (blob_type == GAIA_TIFF_BLOB)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_GIF_BLOB)
      {
	  if (blob_type == GAIA_GIF_BLOB)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_PNG_BLOB)
      {
	  if (blob_type == GAIA_PNG_BLOB)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_JPEG_BLOB)
      {
	  if (blob_type == GAIA_JPEG_BLOB || blob_type == GAIA_EXIF_BLOB
	      || blob_type == GAIA_EXIF_GPS_BLOB)
	      sqlite3_result_int (context, 1);
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_EXIF_BLOB)
      {
	  if (blob_type == GAIA_EXIF_BLOB || blob_type == GAIA_EXIF_GPS_BLOB)
	    {
		sqlite3_result_int (context, 1);
	    }
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    if (request == GAIA_EXIF_GPS_BLOB)
      {
	  if (blob_type == GAIA_EXIF_GPS_BLOB)
	    {
		sqlite3_result_int (context, 1);
	    }
	  else
	      sqlite3_result_int (context, 0);
	  return;
      }
    sqlite3_result_int (context, -1);
}

/*
/ the following functions simply readdress the blob_guess()
/ setting the appropriate request mode
*/

static void
fnct_IsGeometryBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_GEOMETRY_BLOB);
}

static void
fnct_IsZipBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_ZIP_BLOB);
}

static void
fnct_IsPdfBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_PDF_BLOB);
}

static void
fnct_IsTiffBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_TIFF_BLOB);
}

static void
fnct_IsGifBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_GIF_BLOB);
}

static void
fnct_IsPngBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_PNG_BLOB);
}

static void
fnct_IsJpegBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_JPEG_BLOB);
}

static void
fnct_IsExifBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_EXIF_BLOB);
}

static void
fnct_IsExifGpsBlob (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    blob_guess (context, argc, argv, GAIA_EXIF_GPS_BLOB);
}

static void
fnct_GeodesicLength (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
/* SQL function:
/ GeodesicLength(BLOB encoded GEOMETRYCOLLECTION)
/
/ returns  the total Geodesic length for current geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double l;
    double length = 0.0;
    double a;
    double b;
    double rf;
    gaiaGeomCollPtr geo = NULL;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    int ib;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (get_ellipse_params (sqlite, geo->Srid, &a, &b, &rf))
	    {
		line = geo->FirstLinestring;
		while (line)
		  {
		      /* Linestrings */
		      l = gaiaGeodesicTotalLength (a, b, rf,
						   line->DimensionModel,
						   line->Coords, line->Points);
		      if (l < 0.0)
			{
			    length = -1.0;
			    break;
			}
		      length += l;
		      line = line->Next;
		  }
		if (length >= 0)
		  {
		      /* Polygons */
		      polyg = geo->FirstPolygon;
		      while (polyg)
			{
			    /* exterior Ring */
			    ring = polyg->Exterior;
			    l = gaiaGeodesicTotalLength (a, b, rf,
							 ring->DimensionModel,
							 ring->Coords,
							 ring->Points);
			    if (l < 0.0)
			      {
				  length = -1.0;
				  break;
			      }
			    length += l;
			    for (ib = 0; ib < polyg->NumInteriors; ib++)
			      {
				  /* interior Rings */
				  ring = polyg->Interiors + ib;
				  l = gaiaGeodesicTotalLength (a, b, rf,
							       ring->
							       DimensionModel,
							       ring->Coords,
							       ring->Points);
				  if (l < 0.0)
				    {
					length = -1.0;
					break;
				    }
				  length += l;
			      }
			    if (length < 0.0)
				break;
			    polyg = polyg->Next;
			}
		  }
		if (length < 0.0)
		    sqlite3_result_null (context);
		else
		    sqlite3_result_double (context, length);
	    }
	  else
	      sqlite3_result_null (context);
	  gaiaFreeGeomColl (geo);
      }
}

static void
fnct_GreatCircleLength (sqlite3_context * context, int argc,
			sqlite3_value ** argv)
{
/* SQL function:
/ GreatCircleLength(BLOB encoded GEOMETRYCOLLECTION)
/
/ returns  the total Great Circle length for current geometry 
/ or NULL if any error is encountered
*/
    unsigned char *p_blob;
    int n_bytes;
    double length = 0.0;
    double a;
    double b;
    double rf;
    gaiaGeomCollPtr geo = NULL;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    int ib;
    sqlite3 *sqlite = sqlite3_context_db_handle (context);
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
      {
	  sqlite3_result_null (context);
	  return;
      }
    p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
    n_bytes = sqlite3_value_bytes (argv[0]);
    geo = gaiaFromSpatiaLiteBlobWkb (p_blob, n_bytes);
    if (!geo)
	sqlite3_result_null (context);
    else
      {
	  if (get_ellipse_params (sqlite, geo->Srid, &a, &b, &rf))
	    {
		line = geo->FirstLinestring;
		while (line)
		  {
		      /* Linestrings */
		      length +=
			  gaiaGreatCircleTotalLength (a, b,
						      line->DimensionModel,
						      line->Coords,
						      line->Points);
		      line = line->Next;
		  }
		if (length >= 0)
		  {
		      /* Polygons */
		      polyg = geo->FirstPolygon;
		      while (polyg)
			{
			    /* exterior Ring */
			    ring = polyg->Exterior;
			    length +=
				gaiaGreatCircleTotalLength (a, b,
							    ring->
							    DimensionModel,
							    ring->Coords,
							    ring->Points);
			    for (ib = 0; ib < polyg->NumInteriors; ib++)
			      {
				  /* interior Rings */
				  ring = polyg->Interiors + ib;
				  length +=
				      gaiaGreatCircleTotalLength (a, b,
								  ring->
								  DimensionModel,
								  ring->Coords,
								  ring->Points);
			      }
			    polyg = polyg->Next;
			}
		  }
		sqlite3_result_double (context, length);
	    }
	  else
	      sqlite3_result_null (context);
	  gaiaFreeGeomColl (geo);
      }
}

static void
convertUnit (sqlite3_context * context, int argc, sqlite3_value ** argv,
	     int unit_from, int unit_to)
{
/* SQL functions:
/ CvtToKm(), CvtToDm(), CvtToCm(), CvtToMm(), CvtToKmi(), CvtToIn(), CvtToFt(),
/ CvtToYd(), CvtToMi(), CvtToFath(), CvtToCh(), CvtToLink(), CvtToUsIn(), 
/ CvtToUsFt(), CvtToUsYd(), CvtToUsCh(), CvtToUsMi(), CvtToIndFt(), 
/ CvtToIndYd(), CvtToIndCh(), 
/ CvtFromKm(), CvtFromDm(), CvtFromCm(), CvtFromMm(), CvtFromKmi(), 
/ CvtFromIn(), CvtFromFt(), CvtFromYd(), CvtFromMi(), CvtFromFath(), 
/ CvtFromCh(), CvtFromLink(), CvtFromUsIn(), CvtFromUsFt(), CvtFromUsYd(), 
/ CvtFromUsCh(), CvtFromUsMi(), CvtFromIndFt(), CvtFromIndYd(), 
/ CvtFromIndCh()
/
/ converts a Length from one unit to a different one
/ or NULL if any error is encountered
*/
    double cvt;
    double value;
    int int_value;
    GAIA_UNUSED ();
    if (sqlite3_value_type (argv[0]) == SQLITE_FLOAT)
	value = sqlite3_value_double (argv[0]);
    else if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
      {
	  int_value = sqlite3_value_int (argv[0]);
	  value = int_value;
      }
    else
      {
	  sqlite3_result_null (context);
	  return;
      }
    if (!gaiaConvertLength (value, unit_from, unit_to, &cvt))
	sqlite3_result_null (context);
    else
	sqlite3_result_double (context, cvt);
}

static void
fnct_cvtToKm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_KM);
}

static void
fnct_cvtToDm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_DM);
}

static void
fnct_cvtToCm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_CM);
}

static void
fnct_cvtToMm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_MM);
}

static void
fnct_cvtToKmi (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_KMI);
}

static void
fnct_cvtToIn (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_IN);
}

static void
fnct_cvtToFt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_FT);
}

static void
fnct_cvtToYd (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_YD);
}

static void
fnct_cvtToMi (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_MI);
}

static void
fnct_cvtToFath (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_FATH);
}

static void
fnct_cvtToCh (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_CH);
}

static void
fnct_cvtToLink (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_LINK);
}

static void
fnct_cvtToUsIn (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_US_IN);
}

static void
fnct_cvtToUsFt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_US_FT);
}

static void
fnct_cvtToUsYd (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_US_YD);
}

static void
fnct_cvtToUsCh (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_US_CH);
}

static void
fnct_cvtToUsMi (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_US_MI);
}

static void
fnct_cvtToIndFt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_IND_FT);
}

static void
fnct_cvtToIndYd (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_IND_YD);
}

static void
fnct_cvtToIndCh (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_M, GAIA_IND_CH);
}

static void
fnct_cvtFromKm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_KM, GAIA_M);
}

static void
fnct_cvtFromDm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_DM, GAIA_M);
}

static void
fnct_cvtFromCm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_CM, GAIA_M);
}

static void
fnct_cvtFromMm (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_MM, GAIA_M);
}

static void
fnct_cvtFromKmi (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_KMI, GAIA_M);
}

static void
fnct_cvtFromIn (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_IN, GAIA_M);
}

static void
fnct_cvtFromFt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_FT, GAIA_M);
}

static void
fnct_cvtFromYd (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_YD, GAIA_M);
}

static void
fnct_cvtFromMi (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_MI, GAIA_M);
}

static void
fnct_cvtFromFath (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_FATH, GAIA_M);
}

static void
fnct_cvtFromCh (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_CH, GAIA_M);
}

static void
fnct_cvtFromLink (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_LINK, GAIA_M);
}

static void
fnct_cvtFromUsIn (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_US_IN, GAIA_M);
}

static void
fnct_cvtFromUsFt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_US_FT, GAIA_M);
}

static void
fnct_cvtFromUsYd (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_US_YD, GAIA_M);
}

static void
fnct_cvtFromUsCh (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_US_CH, GAIA_M);
}

static void
fnct_cvtFromUsMi (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_US_MI, GAIA_M);
}

static void
fnct_cvtFromIndFt (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_IND_FT, GAIA_M);
}

static void
fnct_cvtFromIndYd (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_IND_YD, GAIA_M);
}

static void
fnct_cvtFromIndCh (sqlite3_context * context, int argc, sqlite3_value ** argv)
{
    convertUnit (context, argc, argv, GAIA_IND_CH, GAIA_M);
}

static void
register_spatialite_sql_functions (sqlite3 * db)
{
    sqlite3_create_function (db, "spatialite_version", 0, SQLITE_ANY, 0,
			     fnct_spatialite_version, 0, 0);
    sqlite3_create_function (db, "proj4_version", 0, SQLITE_ANY, 0,
			     fnct_proj4_version, 0, 0);
    sqlite3_create_function (db, "geos_version", 0, SQLITE_ANY, 0,
			     fnct_geos_version, 0, 0);
    sqlite3_create_function (db, "GeometryConstraints", 3, SQLITE_ANY, 0,
			     fnct_GeometryConstraints, 0, 0);
    sqlite3_create_function (db, "GeometryConstraints", 4, SQLITE_ANY, 0,
			     fnct_GeometryConstraints, 0, 0);
    sqlite3_create_function (db, "RTreeAlign", 3, SQLITE_ANY, 0,
			     fnct_RTreeAlign, 0, 0);
    sqlite3_create_function (db, "CheckSpatialMetaData", 0, SQLITE_ANY, 0,
			     fnct_CheckSpatialMetaData, 0, 0);
    sqlite3_create_function (db, "AutoFDOStart", 0, SQLITE_ANY, 0,
			     fnct_AutoFDOStart, 0, 0);
    sqlite3_create_function (db, "AutoFDOStop", 0, SQLITE_ANY, 0,
			     fnct_AutoFDOStop, 0, 0);
    sqlite3_create_function (db, "InitFDOSpatialMetaData", 0, SQLITE_ANY, 0,
			     fnct_InitFDOSpatialMetaData, 0, 0);
    sqlite3_create_function (db, "AddFDOGeometryColumn", 6, SQLITE_ANY, 0,
			     fnct_AddFDOGeometryColumn, 0, 0);
    sqlite3_create_function (db, "RecoverFDOGeometryColumn", 6, SQLITE_ANY, 0,
			     fnct_RecoverFDOGeometryColumn, 0, 0);
    sqlite3_create_function (db, "DiscardFDOGeometryColumn", 2, SQLITE_ANY, 0,
			     fnct_DiscardFDOGeometryColumn, 0, 0);
    sqlite3_create_function (db, "InitSpatialMetaData", 0, SQLITE_ANY, 0,
			     fnct_InitSpatialMetaData, 0, 0);
    sqlite3_create_function (db, "AddGeometryColumn", 5, SQLITE_ANY, 0,
			     fnct_AddGeometryColumn, 0, 0);
    sqlite3_create_function (db, "AddGeometryColumn", 6, SQLITE_ANY, 0,
			     fnct_AddGeometryColumn, 0, 0);
    sqlite3_create_function (db, "RecoverGeometryColumn", 5, SQLITE_ANY, 0,
			     fnct_RecoverGeometryColumn, 0, 0);
    sqlite3_create_function (db, "DiscardGeometryColumn", 2, SQLITE_ANY, 0,
			     fnct_DiscardGeometryColumn, 0, 0);
    sqlite3_create_function (db, "RecoverSpatialIndex", 0, SQLITE_ANY, 0,
			     fnct_RecoverSpatialIndex, 0, 0);
    sqlite3_create_function (db, "RecoverSpatialIndex", 1, SQLITE_ANY, 0,
			     fnct_RecoverSpatialIndex, 0, 0);
    sqlite3_create_function (db, "RecoverSpatialIndex", 2, SQLITE_ANY, 0,
			     fnct_RecoverSpatialIndex, 0, 0);
    sqlite3_create_function (db, "RecoverSpatialIndex", 3, SQLITE_ANY, 0,
			     fnct_RecoverSpatialIndex, 0, 0);
    sqlite3_create_function (db, "CheckSpatialIndex", 0, SQLITE_ANY, 0,
			     fnct_CheckSpatialIndex, 0, 0);
    sqlite3_create_function (db, "CheckSpatialIndex", 2, SQLITE_ANY, 0,
			     fnct_CheckSpatialIndex, 0, 0);
    sqlite3_create_function (db, "CreateSpatialIndex", 2, SQLITE_ANY, 0,
			     fnct_CreateSpatialIndex, 0, 0);
    sqlite3_create_function (db, "CreateMbrCache", 2, SQLITE_ANY, 0,
			     fnct_CreateMbrCache, 0, 0);
    sqlite3_create_function (db, "DisableSpatialIndex", 2, SQLITE_ANY, 0,
			     fnct_DisableSpatialIndex, 0, 0);
    sqlite3_create_function (db, "RebuildGeometryTriggers", 2, SQLITE_ANY, 0,
			     fnct_RebuildGeometryTriggers, 0, 0);
    sqlite3_create_function (db, "CreateTopologyTables", 2, SQLITE_ANY, 0,
			     fnct_CreateTopologyTables, 0, 0);
    sqlite3_create_function (db, "CreateTopologyTables", 3, SQLITE_ANY, 0,
			     fnct_CreateTopologyTables, 0, 0);
    sqlite3_create_function (db, "UpdateLayerStatistics", 0, SQLITE_ANY, 0,
			     fnct_UpdateLayerStatistics, 0, 0);
    sqlite3_create_function (db, "UpdateLayerStatistics", 1, SQLITE_ANY, 0,
			     fnct_UpdateLayerStatistics, 0, 0);
    sqlite3_create_function (db, "UpdateLayerStatistics", 2, SQLITE_ANY, 0,
			     fnct_UpdateLayerStatistics, 0, 0);
    sqlite3_create_function (db, "AsText", 1, SQLITE_ANY, 0, fnct_AsText, 0, 0);
    sqlite3_create_function (db, "ST_AsText", 1, SQLITE_ANY, 0, fnct_AsText,
			     0, 0);
    sqlite3_create_function (db, "AsWkt", 1, SQLITE_ANY, 0, fnct_AsWkt, 0, 0);
    sqlite3_create_function (db, "AsWkt", 2, SQLITE_ANY, 0, fnct_AsWkt, 0, 0);
    sqlite3_create_function (db, "AsSvg", 1, SQLITE_ANY, 0, fnct_AsSvg1, 0, 0);
    sqlite3_create_function (db, "AsSvg", 2, SQLITE_ANY, 0, fnct_AsSvg2, 0, 0);
    sqlite3_create_function (db, "AsSvg", 3, SQLITE_ANY, 0, fnct_AsSvg3, 0, 0);

#ifndef OMIT_PROJ		/* PROJ.4 is strictly required to support KML */
    sqlite3_create_function (db, "AsKml", 1, SQLITE_ANY, 0, fnct_AsKml, 0, 0);
    sqlite3_create_function (db, "AsKml", 2, SQLITE_ANY, 0, fnct_AsKml, 0, 0);
    sqlite3_create_function (db, "AsKml", 3, SQLITE_ANY, 0, fnct_AsKml, 0, 0);
    sqlite3_create_function (db, "AsKml", 4, SQLITE_ANY, 0, fnct_AsKml, 0, 0);
#endif /* end including PROJ.4 */

    sqlite3_create_function (db, "AsGml", 1, SQLITE_ANY, 0, fnct_AsGml, 0, 0);
    sqlite3_create_function (db, "AsGml", 2, SQLITE_ANY, 0, fnct_AsGml, 0, 0);
    sqlite3_create_function (db, "AsGml", 3, SQLITE_ANY, 0, fnct_AsGml, 0, 0);
    sqlite3_create_function (db, "GeomFromGml", 1, SQLITE_ANY, 0,
			     fnct_FromGml, 0, 0);
    sqlite3_create_function (db, "AsGeoJSON", 1, SQLITE_ANY, 0, fnct_AsGeoJSON,
			     0, 0);
    sqlite3_create_function (db, "AsGeoJSON", 2, SQLITE_ANY, 0, fnct_AsGeoJSON,
			     0, 0);
    sqlite3_create_function (db, "AsGeoJSON", 3, SQLITE_ANY, 0, fnct_AsGeoJSON,
			     0, 0);
    sqlite3_create_function (db, "GeomFromGeoJSON", 1, SQLITE_ANY, 0,
			     fnct_FromGeoJSON, 0, 0);
    sqlite3_create_function (db, "GeomFromKml", 1, SQLITE_ANY, 0,
			     fnct_FromKml, 0, 0);
    sqlite3_create_function (db, "AsFGF", 2, SQLITE_ANY, 0, fnct_AsFGF, 0, 0);
    sqlite3_create_function (db, "GeomFromEWKB", 1, SQLITE_ANY, 0,
			     fnct_FromEWKB, 0, 0);
    sqlite3_create_function (db, "AsEWKB", 1, SQLITE_ANY, 0, fnct_ToEWKB, 0, 0);
    sqlite3_create_function (db, "AsEWKT", 1, SQLITE_ANY, 0, fnct_ToEWKT, 0, 0);
    sqlite3_create_function (db, "GeomFromEWKT", 1, SQLITE_ANY, 0,
			     fnct_FromEWKT, 0, 0);
    sqlite3_create_function (db, "AsBinary", 1, SQLITE_ANY, 0, fnct_AsBinary, 0,
			     0);
    sqlite3_create_function (db, "ST_AsBinary", 1, SQLITE_ANY, 0, fnct_AsBinary,
			     0, 0);
    sqlite3_create_function (db, "GeomFromText", 1, SQLITE_ANY, 0,
			     fnct_GeomFromText1, 0, 0);
    sqlite3_create_function (db, "GeomFromText", 2, SQLITE_ANY, 0,
			     fnct_GeomFromText2, 0, 0);
    sqlite3_create_function (db, "GeometryFromText", 1, SQLITE_ANY, 0,
			     fnct_GeomFromText1, 0, 0);
    sqlite3_create_function (db, "GeometryFromText", 2, SQLITE_ANY, 0,
			     fnct_GeomFromText2, 0, 0);
    sqlite3_create_function (db, "GeomCollFromText", 1, SQLITE_ANY, 0,
			     fnct_GeomCollFromText1, 0, 0);
    sqlite3_create_function (db, "GeomCollFromText", 2, SQLITE_ANY, 0,
			     fnct_GeomCollFromText2, 0, 0);
    sqlite3_create_function (db, "GeometryCollectionFromText", 1, SQLITE_ANY, 0,
			     fnct_GeomCollFromText1, 0, 0);
    sqlite3_create_function (db, "GeometryCollectionFromText", 2, SQLITE_ANY, 0,
			     fnct_GeomCollFromText2, 0, 0);
    sqlite3_create_function (db, "PointFromText", 1, SQLITE_ANY, 0,
			     fnct_PointFromText1, 0, 0);
    sqlite3_create_function (db, "PointFromText", 2, SQLITE_ANY, 0,
			     fnct_PointFromText2, 0, 0);
    sqlite3_create_function (db, "LineFromText", 1, SQLITE_ANY, 0,
			     fnct_LineFromText1, 0, 0);
    sqlite3_create_function (db, "LineFromText", 2, SQLITE_ANY, 0,
			     fnct_LineFromText2, 0, 0);
    sqlite3_create_function (db, "LineStringFromText", 1, SQLITE_ANY, 0,
			     fnct_LineFromText1, 0, 0);
    sqlite3_create_function (db, "LineStringFromText", 2, SQLITE_ANY, 0,
			     fnct_LineFromText2, 0, 0);
    sqlite3_create_function (db, "PolyFromText", 1, SQLITE_ANY, 0,
			     fnct_PolyFromText1, 0, 0);
    sqlite3_create_function (db, "PolyFromText", 2, SQLITE_ANY, 0,
			     fnct_PolyFromText2, 0, 0);
    sqlite3_create_function (db, "PolygonFromText", 1, SQLITE_ANY, 0,
			     fnct_PolyFromText1, 0, 0);
    sqlite3_create_function (db, "PolygonFromText", 2, SQLITE_ANY, 0,
			     fnct_PolyFromText2, 0, 0);
    sqlite3_create_function (db, "MPointFromText", 1, SQLITE_ANY, 0,
			     fnct_MPointFromText1, 0, 0);
    sqlite3_create_function (db, "MPointFromText", 2, SQLITE_ANY, 0,
			     fnct_MPointFromText2, 0, 0);
    sqlite3_create_function (db, "MultiPointFromText", 1, SQLITE_ANY, 0,
			     fnct_MPointFromText1, 0, 0);
    sqlite3_create_function (db, "MultiPointFromText", 2, SQLITE_ANY, 0,
			     fnct_MPointFromText2, 0, 0);
    sqlite3_create_function (db, "MLineFromText", 1, SQLITE_ANY, 0,
			     fnct_MLineFromText1, 0, 0);
    sqlite3_create_function (db, "MLineFromText", 2, SQLITE_ANY, 0,
			     fnct_MLineFromText2, 0, 0);
    sqlite3_create_function (db, "MultiLineStringFromText", 1, SQLITE_ANY, 0,
			     fnct_MLineFromText1, 0, 0);
    sqlite3_create_function (db, "MultiLineStringFromText", 2, SQLITE_ANY, 0,
			     fnct_MLineFromText2, 0, 0);
    sqlite3_create_function (db, "MPolyFromText", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromText1, 0, 0);
    sqlite3_create_function (db, "MPolyFromText", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromText2, 0, 0);
    sqlite3_create_function (db, "MultiPolygonFromText", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromText1, 0, 0);
    sqlite3_create_function (db, "MultiPolygonFromText", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromText2, 0, 0);
    sqlite3_create_function (db, "GeomFromWKB", 1, SQLITE_ANY, 0,
			     fnct_GeomFromWkb1, 0, 0);
    sqlite3_create_function (db, "GeomFromWKB", 2, SQLITE_ANY, 0,
			     fnct_GeomFromWkb2, 0, 0);
    sqlite3_create_function (db, "GeometryFromWKB", 1, SQLITE_ANY, 0,
			     fnct_GeomFromWkb1, 0, 0);
    sqlite3_create_function (db, "GeometryFromWKB", 2, SQLITE_ANY, 0,
			     fnct_GeomFromWkb2, 0, 0);
    sqlite3_create_function (db, "GeomCollFromWKB", 1, SQLITE_ANY, 0,
			     fnct_GeomCollFromWkb1, 0, 0);
    sqlite3_create_function (db, "GeomCollFromWKB", 2, SQLITE_ANY, 0,
			     fnct_GeomCollFromWkb2, 0, 0);
    sqlite3_create_function (db, "GeometryCollectionFromWKB", 1, SQLITE_ANY, 0,
			     fnct_GeomCollFromWkb1, 0, 0);
    sqlite3_create_function (db, "GeometryCollectionFromWKB", 2, SQLITE_ANY, 0,
			     fnct_GeomCollFromWkb2, 0, 0);
    sqlite3_create_function (db, "PointFromWKB", 1, SQLITE_ANY, 0,
			     fnct_PointFromWkb1, 0, 0);
    sqlite3_create_function (db, "PointFromWKB", 2, SQLITE_ANY, 0,
			     fnct_PointFromWkb2, 0, 0);
    sqlite3_create_function (db, "LineFromWKB", 1, SQLITE_ANY, 0,
			     fnct_LineFromWkb1, 0, 0);
    sqlite3_create_function (db, "LineFromWKB", 2, SQLITE_ANY, 0,
			     fnct_LineFromWkb2, 0, 0);
    sqlite3_create_function (db, "LineStringFromWKB", 1, SQLITE_ANY, 0,
			     fnct_LineFromWkb1, 0, 0);
    sqlite3_create_function (db, "LineStringFromWKB", 2, SQLITE_ANY, 0,
			     fnct_LineFromWkb2, 0, 0);
    sqlite3_create_function (db, "PolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_PolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "PolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_PolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "PolygonFromWKB", 1, SQLITE_ANY, 0,
			     fnct_PolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "PolygonFromWKB", 2, SQLITE_ANY, 0,
			     fnct_PolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "MPointFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPointFromWkb1, 0, 0);
    sqlite3_create_function (db, "MPointFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPointFromWkb2, 0, 0);
    sqlite3_create_function (db, "MultiPointFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPointFromWkb1, 0, 0);
    sqlite3_create_function (db, "MultiPointFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPointFromWkb2, 0, 0);
    sqlite3_create_function (db, "MLineFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MLineFromWkb1, 0, 0);
    sqlite3_create_function (db, "MLineFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MLineFromWkb2, 0, 0);
    sqlite3_create_function (db, "MultiLineStringFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MLineFromWkb1, 0, 0);
    sqlite3_create_function (db, "MultiLineStringFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MLineFromWkb2, 0, 0);
    sqlite3_create_function (db, "MPolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "MPolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "MultiPolygonFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "MultiPolygonFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_GeomFromText", 1, SQLITE_ANY, 0,
			     fnct_GeomFromText1, 0, 0);
    sqlite3_create_function (db, "ST_GeomFromText", 2, SQLITE_ANY, 0,
			     fnct_GeomFromText2, 0, 0);
    sqlite3_create_function (db, "ST_GeometryFromText", 1, SQLITE_ANY, 0,
			     fnct_GeomFromText1, 0, 0);
    sqlite3_create_function (db, "ST_GeometryFromText", 2, SQLITE_ANY, 0,
			     fnct_GeomFromText2, 0, 0);
    sqlite3_create_function (db, "ST_GeomCollFromText", 1, SQLITE_ANY, 0,
			     fnct_GeomCollFromText1, 0, 0);
    sqlite3_create_function (db, "ST_GeomCollFromText", 2, SQLITE_ANY, 0,
			     fnct_GeomCollFromText2, 0, 0);
    sqlite3_create_function (db, "ST_GeometryCollectionFromText", 1, SQLITE_ANY,
			     0, fnct_GeomCollFromText1, 0, 0);
    sqlite3_create_function (db, "ST_GeometryCollectionFromText", 2, SQLITE_ANY,
			     0, fnct_GeomCollFromText2, 0, 0);
    sqlite3_create_function (db, "ST_PointFromText", 1, SQLITE_ANY, 0,
			     fnct_PointFromText1, 0, 0);
    sqlite3_create_function (db, "ST_PointFromText", 2, SQLITE_ANY, 0,
			     fnct_PointFromText2, 0, 0);
    sqlite3_create_function (db, "ST_LineFromText", 1, SQLITE_ANY, 0,
			     fnct_LineFromText1, 0, 0);
    sqlite3_create_function (db, "ST_LineFromText", 2, SQLITE_ANY, 0,
			     fnct_LineFromText2, 0, 0);
    sqlite3_create_function (db, "ST_LineStringFromText", 1, SQLITE_ANY, 0,
			     fnct_LineFromText1, 0, 0);
    sqlite3_create_function (db, "ST_LineStringFromText", 2, SQLITE_ANY, 0,
			     fnct_LineFromText2, 0, 0);
    sqlite3_create_function (db, "ST_PolyFromText", 1, SQLITE_ANY, 0,
			     fnct_PolyFromText1, 0, 0);
    sqlite3_create_function (db, "ST_PolyFromText", 2, SQLITE_ANY, 0,
			     fnct_PolyFromText2, 0, 0);
    sqlite3_create_function (db, "ST_PolygonFromText", 1, SQLITE_ANY, 0,
			     fnct_PolyFromText1, 0, 0);
    sqlite3_create_function (db, "ST_PolygonFromText", 2, SQLITE_ANY, 0,
			     fnct_PolyFromText2, 0, 0);
    sqlite3_create_function (db, "ST_MPointFromText", 1, SQLITE_ANY, 0,
			     fnct_MPointFromText1, 0, 0);
    sqlite3_create_function (db, "ST_MPointFromText", 2, SQLITE_ANY, 0,
			     fnct_MPointFromText2, 0, 0);
    sqlite3_create_function (db, "ST_MultiPointFromText", 1, SQLITE_ANY, 0,
			     fnct_MPointFromText1, 0, 0);
    sqlite3_create_function (db, "ST_MultiPointFromText", 2, SQLITE_ANY, 0,
			     fnct_MPointFromText2, 0, 0);
    sqlite3_create_function (db, "ST_MLineFromText", 1, SQLITE_ANY, 0,
			     fnct_MLineFromText1, 0, 0);
    sqlite3_create_function (db, "ST_MLineFromText", 2, SQLITE_ANY, 0,
			     fnct_MLineFromText2, 0, 0);
    sqlite3_create_function (db, "ST_MultiLineStringFromText", 1, SQLITE_ANY, 0,
			     fnct_MLineFromText1, 0, 0);
    sqlite3_create_function (db, "ST_MultiLineStringFromText", 2, SQLITE_ANY, 0,
			     fnct_MLineFromText2, 0, 0);
    sqlite3_create_function (db, "ST_MPolyFromText", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromText1, 0, 0);
    sqlite3_create_function (db, "ST_MPolyFromText", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromText2, 0, 0);
    sqlite3_create_function (db, "ST_MultiPolygonFromText", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromText1, 0, 0);
    sqlite3_create_function (db, "ST_MultiPolygonFromText", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromText2, 0, 0);
    sqlite3_create_function (db, "ST_GeomFromWKB", 1, SQLITE_ANY, 0,
			     fnct_GeomFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_GeomFromWKB", 2, SQLITE_ANY, 0,
			     fnct_GeomFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_GeometryFromWKB", 1, SQLITE_ANY, 0,
			     fnct_GeomFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_GeometryFromWKB", 2, SQLITE_ANY, 0,
			     fnct_GeomFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_GeomCollFromWKB", 1, SQLITE_ANY, 0,
			     fnct_GeomCollFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_GeomCollFromWKB", 2, SQLITE_ANY, 0,
			     fnct_GeomCollFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_GeometryCollectionFromWKB", 1, SQLITE_ANY,
			     0, fnct_GeomCollFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_GeometryCollectionFromWKB", 2, SQLITE_ANY,
			     0, fnct_GeomCollFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_PointFromWKB", 1, SQLITE_ANY, 0,
			     fnct_PointFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_PointFromWKB", 2, SQLITE_ANY, 0,
			     fnct_PointFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_LineFromWKB", 1, SQLITE_ANY, 0,
			     fnct_LineFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_LineFromWKB", 2, SQLITE_ANY, 0,
			     fnct_LineFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_LineStringFromWKB", 1, SQLITE_ANY, 0,
			     fnct_LineFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_LineStringFromWKB", 2, SQLITE_ANY, 0,
			     fnct_LineFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_PolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_PolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_PolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_PolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_PolygonFromWKB", 1, SQLITE_ANY, 0,
			     fnct_PolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_PolygonFromWKB", 2, SQLITE_ANY, 0,
			     fnct_PolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_MPointFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPointFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_MPointFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPointFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_MultiPointFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPointFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_MultiPointFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPointFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_MLineFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MLineFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_MLineFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MLineFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_MultiLineStringFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MLineFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_MultiLineStringFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MLineFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_MPolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_MPolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "ST_MultiPolygonFromWKB", 1, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb1, 0, 0);
    sqlite3_create_function (db, "ST_MultiPolygonFromWKB", 2, SQLITE_ANY, 0,
			     fnct_MPolyFromWkb2, 0, 0);
    sqlite3_create_function (db, "GeomFromFGF", 1, SQLITE_ANY, 0,
			     fnct_GeometryFromFGF1, 0, 0);
    sqlite3_create_function (db, "GeomFromFGF", 2, SQLITE_ANY, 0,
			     fnct_GeometryFromFGF2, 0, 0);
    sqlite3_create_function (db, "CompressGeometry", 1, SQLITE_ANY, 0,
			     fnct_CompressGeometry, 0, 0);
    sqlite3_create_function (db, "UncompressGeometry", 1, SQLITE_ANY, 0,
			     fnct_UncompressGeometry, 0, 0);
    sqlite3_create_function (db, "SanitizeGeometry", 1, SQLITE_ANY, 0,
			     fnct_SanitizeGeometry, 0, 0);
    sqlite3_create_function (db, "CastToPoint", 1, SQLITE_ANY, 0,
			     fnct_CastToPoint, 0, 0);
    sqlite3_create_function (db, "CastToLinestring", 1, SQLITE_ANY, 0,
			     fnct_CastToLinestring, 0, 0);
    sqlite3_create_function (db, "CastToPolygon", 1, SQLITE_ANY, 0,
			     fnct_CastToPolygon, 0, 0);
    sqlite3_create_function (db, "CastToMultiPoint", 1, SQLITE_ANY, 0,
			     fnct_CastToMultiPoint, 0, 0);
    sqlite3_create_function (db, "CastToMultiLinestring", 1, SQLITE_ANY, 0,
			     fnct_CastToMultiLinestring, 0, 0);
    sqlite3_create_function (db, "CastToMultiPolygon", 1, SQLITE_ANY, 0,
			     fnct_CastToMultiPolygon, 0, 0);
    sqlite3_create_function (db, "CastToGeometryCollection", 1, SQLITE_ANY, 0,
			     fnct_CastToGeometryCollection, 0, 0);
    sqlite3_create_function (db, "CastToMulti", 1, SQLITE_ANY, 0,
			     fnct_CastToMulti, 0, 0);
    sqlite3_create_function (db, "ST_Multi", 1, SQLITE_ANY, 0, fnct_CastToMulti,
			     0, 0);
    sqlite3_create_function (db, "CastToSingle", 1, SQLITE_ANY, 0,
			     fnct_CastToSingle, 0, 0);
    sqlite3_create_function (db, "CastToXY", 1, SQLITE_ANY, 0, fnct_CastToXY, 0,
			     0);
    sqlite3_create_function (db, "CastToXYZ", 1, SQLITE_ANY, 0, fnct_CastToXYZ,
			     0, 0);
    sqlite3_create_function (db, "CastToXYM", 1, SQLITE_ANY, 0, fnct_CastToXYM,
			     0, 0);
    sqlite3_create_function (db, "CastToXYZM", 1, SQLITE_ANY, 0,
			     fnct_CastToXYZM, 0, 0);
    sqlite3_create_function (db, "ExtractMultiPoint", 1, SQLITE_ANY, 0,
			     fnct_ExtractMultiPoint, 0, 0);
    sqlite3_create_function (db, "ExtractMultiLinestring", 1, SQLITE_ANY, 0,
			     fnct_ExtractMultiLinestring, 0, 0);
    sqlite3_create_function (db, "ExtractMultiPolygon", 1, SQLITE_ANY, 0,
			     fnct_ExtractMultiPolygon, 0, 0);
    sqlite3_create_function (db, "Dimension", 1, SQLITE_ANY, 0, fnct_Dimension,
			     0, 0);
    sqlite3_create_function (db, "ST_Dimension", 1, SQLITE_ANY, 0,
			     fnct_Dimension, 0, 0);
    sqlite3_create_function (db, "CoordDimension", 1, SQLITE_ANY, 0,
			     fnct_CoordDimension, 0, 0);
    sqlite3_create_function (db, "GeometryType", 1, SQLITE_ANY, 0,
			     fnct_GeometryType, 0, 0);
    sqlite3_create_function (db, "ST_GeometryType", 1, SQLITE_ANY, 0,
			     fnct_GeometryType, 0, 0);
    sqlite3_create_function (db, "GeometryAliasType", 1, SQLITE_ANY, 0,
			     fnct_GeometryAliasType, 0, 0);
    sqlite3_create_function (db, "SridFromAuthCRS", 2, SQLITE_ANY, 0,
			     fnct_SridFromAuthCRS, 0, 0);
    sqlite3_create_function (db, "SRID", 1, SQLITE_ANY, 0, fnct_SRID, 0, 0);
    sqlite3_create_function (db, "ST_SRID", 1, SQLITE_ANY, 0, fnct_SRID, 0, 0);
    sqlite3_create_function (db, "SetSRID", 2, SQLITE_ANY, 0, fnct_SetSRID, 0,
			     0);
    sqlite3_create_function (db, "IsEmpty", 1, SQLITE_ANY, 0, fnct_IsEmpty, 0,
			     0);
    sqlite3_create_function (db, "ST_IsEmpty", 1, SQLITE_ANY, 0, fnct_IsEmpty,
			     0, 0);
    sqlite3_create_function (db, "Envelope", 1, SQLITE_ANY, 0, fnct_Envelope,
			     0, 0);
    sqlite3_create_function (db, "ST_Envelope", 1, SQLITE_ANY, 0,
			     fnct_Envelope, 0, 0);
    sqlite3_create_function (db, "X", 1, SQLITE_ANY, 0, fnct_X, 0, 0);
    sqlite3_create_function (db, "Y", 1, SQLITE_ANY, 0, fnct_Y, 0, 0);
    sqlite3_create_function (db, "Z", 1, SQLITE_ANY, 0, fnct_Z, 0, 0);
    sqlite3_create_function (db, "M", 1, SQLITE_ANY, 0, fnct_M, 0, 0);
    sqlite3_create_function (db, "ST_X", 1, SQLITE_ANY, 0, fnct_X, 0, 0);
    sqlite3_create_function (db, "ST_Y", 1, SQLITE_ANY, 0, fnct_Y, 0, 0);
    sqlite3_create_function (db, "ST_Z", 1, SQLITE_ANY, 0, fnct_Z, 0, 0);
    sqlite3_create_function (db, "ST_M", 1, SQLITE_ANY, 0, fnct_M, 0, 0);
    sqlite3_create_function (db, "NumPoints", 1, SQLITE_ANY, 0,
			     fnct_NumPoints, 0, 0);
    sqlite3_create_function (db, "ST_NumPoints", 1, SQLITE_ANY, 0,
			     fnct_NumPoints, 0, 0);
    sqlite3_create_function (db, "StartPoint", 1, SQLITE_ANY, 0,
			     fnct_StartPoint, 0, 0);
    sqlite3_create_function (db, "EndPoint", 1, SQLITE_ANY, 0, fnct_EndPoint,
			     0, 0);
    sqlite3_create_function (db, "ST_StartPoint", 1, SQLITE_ANY, 0,
			     fnct_StartPoint, 0, 0);
    sqlite3_create_function (db, "ST_EndPoint", 1, SQLITE_ANY, 0,
			     fnct_EndPoint, 0, 0);
    sqlite3_create_function (db, "PointN", 2, SQLITE_ANY, 0, fnct_PointN, 0, 0);
    sqlite3_create_function (db, "ST_PointN", 2, SQLITE_ANY, 0, fnct_PointN,
			     0, 0);
    sqlite3_create_function (db, "ExteriorRing", 1, SQLITE_ANY, 0,
			     fnct_ExteriorRing, 0, 0);
    sqlite3_create_function (db, "ST_ExteriorRing", 1, SQLITE_ANY, 0,
			     fnct_ExteriorRing, 0, 0);
    sqlite3_create_function (db, "NumInteriorRing", 1, SQLITE_ANY, 0,
			     fnct_NumInteriorRings, 0, 0);
    sqlite3_create_function (db, "NumInteriorRings", 1, SQLITE_ANY, 0,
			     fnct_NumInteriorRings, 0, 0);
    sqlite3_create_function (db, "ST_NumInteriorRing", 1, SQLITE_ANY, 0,
			     fnct_NumInteriorRings, 0, 0);
    sqlite3_create_function (db, "InteriorRingN", 2, SQLITE_ANY, 0,
			     fnct_InteriorRingN, 0, 0);
    sqlite3_create_function (db, "ST_InteriorRingN", 2, SQLITE_ANY, 0,
			     fnct_InteriorRingN, 0, 0);
    sqlite3_create_function (db, "NumGeometries", 1, SQLITE_ANY, 0,
			     fnct_NumGeometries, 0, 0);
    sqlite3_create_function (db, "ST_NumGeometries", 1, SQLITE_ANY, 0,
			     fnct_NumGeometries, 0, 0);
    sqlite3_create_function (db, "GeometryN", 2, SQLITE_ANY, 0,
			     fnct_GeometryN, 0, 0);
    sqlite3_create_function (db, "ST_GeometryN", 2, SQLITE_ANY, 0,
			     fnct_GeometryN, 0, 0);
    sqlite3_create_function (db, "MBRContains", 2, SQLITE_ANY, 0,
			     fnct_MbrContains, 0, 0);
    sqlite3_create_function (db, "MbrDisjoint", 2, SQLITE_ANY, 0,
			     fnct_MbrDisjoint, 0, 0);
    sqlite3_create_function (db, "MBREqual", 2, SQLITE_ANY, 0, fnct_MbrEqual,
			     0, 0);
    sqlite3_create_function (db, "MbrIntersects", 2, SQLITE_ANY, 0,
			     fnct_MbrIntersects, 0, 0);
    sqlite3_create_function (db, "MBROverlaps", 2, SQLITE_ANY, 0,
			     fnct_MbrOverlaps, 0, 0);
    sqlite3_create_function (db, "MbrTouches", 2, SQLITE_ANY, 0,
			     fnct_MbrTouches, 0, 0);
    sqlite3_create_function (db, "MbrWithin", 2, SQLITE_ANY, 0,
			     fnct_MbrWithin, 0, 0);
    sqlite3_create_function (db, "ShiftCoords", 3, SQLITE_ANY, 0,
			     fnct_ShiftCoords, 0, 0);
    sqlite3_create_function (db, "ShiftCoordinates", 3, SQLITE_ANY, 0,
			     fnct_ShiftCoords, 0, 0);
    sqlite3_create_function (db, "ScaleCoords", 2, SQLITE_ANY, 0,
			     fnct_ScaleCoords, 0, 0);
    sqlite3_create_function (db, "ScaleCoordinates", 2, SQLITE_ANY, 0,
			     fnct_ScaleCoords, 0, 0);
    sqlite3_create_function (db, "ScaleCoords", 3, SQLITE_ANY, 0,
			     fnct_ScaleCoords, 0, 0);
    sqlite3_create_function (db, "ScaleCoordinates", 3, SQLITE_ANY, 0,
			     fnct_ScaleCoords, 0, 0);
    sqlite3_create_function (db, "RotateCoords", 2, SQLITE_ANY, 0,
			     fnct_RotateCoords, 0, 0);
    sqlite3_create_function (db, "RotateCoordinates", 2, SQLITE_ANY, 0,
			     fnct_RotateCoords, 0, 0);
    sqlite3_create_function (db, "ReflectCoords", 3, SQLITE_ANY, 0,
			     fnct_ReflectCoords, 0, 0);
    sqlite3_create_function (db, "ReflectCoordinates", 3, SQLITE_ANY, 0,
			     fnct_ReflectCoords, 0, 0);
    sqlite3_create_function (db, "SwapCoords", 1, SQLITE_ANY, 0,
			     fnct_SwapCoords, 0, 0);
    sqlite3_create_function (db, "SwapCoordinates", 1, SQLITE_ANY, 0,
			     fnct_SwapCoords, 0, 0);
    sqlite3_create_function (db, "BuildMbr", 4, SQLITE_ANY, 0, fnct_BuildMbr1,
			     0, 0);
    sqlite3_create_function (db, "BuildMbr", 5, SQLITE_ANY, 0, fnct_BuildMbr2,
			     0, 0);
    sqlite3_create_function (db, "BuildCircleMbr", 3, SQLITE_ANY, 0,
			     fnct_BuildCircleMbr1, 0, 0);
    sqlite3_create_function (db, "BuildCircleMbr", 4, SQLITE_ANY, 0,
			     fnct_BuildCircleMbr2, 0, 0);
    sqlite3_create_function (db, "Extent", 1, SQLITE_ANY, 0, 0,
			     fnct_Extent_step, fnct_Extent_final);
    sqlite3_create_function (db, "MbrMinX", 1, SQLITE_ANY, 0, fnct_MbrMinX, 0,
			     0);
    sqlite3_create_function (db, "MbrMaxX", 1, SQLITE_ANY, 0, fnct_MbrMaxX, 0,
			     0);
    sqlite3_create_function (db, "MbrMinY", 1, SQLITE_ANY, 0, fnct_MbrMinY, 0,
			     0);
    sqlite3_create_function (db, "MbrMaxY", 1, SQLITE_ANY, 0, fnct_MbrMaxY, 0,
			     0);
    sqlite3_create_function (db, "MakePoint", 2, SQLITE_ANY, 0, fnct_MakePoint1,
			     0, 0);
    sqlite3_create_function (db, "MakePoint", 3, SQLITE_ANY, 0, fnct_MakePoint2,
			     0, 0);
    sqlite3_create_function (db, "MakeLine", 1, SQLITE_ANY, 0, 0,
			     fnct_MakeLine_step, fnct_MakeLine_final);
    sqlite3_create_function (db, "MakeLine", 2, SQLITE_ANY, 0, fnct_MakeLine, 0,
			     0);
    sqlite3_create_function (db, "Collect", 1, SQLITE_ANY, 0, 0,
			     fnct_Collect_step, fnct_Collect_final);
    sqlite3_create_function (db, "Collect", 2, SQLITE_ANY, 0, fnct_Collect, 0,
			     0);
    sqlite3_create_function (db, "ST_Collect", 1, SQLITE_ANY, 0, 0,
			     fnct_Collect_step, fnct_Collect_final);
    sqlite3_create_function (db, "ST_Collect", 2, SQLITE_ANY, 0, fnct_Collect,
			     0, 0);
    sqlite3_create_function (db, "BuildMbrFilter", 4, SQLITE_ANY, 0,
			     fnct_BuildMbrFilter, 0, 0);
    sqlite3_create_function (db, "FilterMbrWithin", 4, SQLITE_ANY, 0,
			     fnct_FilterMbrWithin, 0, 0);
    sqlite3_create_function (db, "FilterMbrContains", 4, SQLITE_ANY, 0,
			     fnct_FilterMbrContains, 0, 0);
    sqlite3_create_function (db, "FilterMbrIntersects", 4, SQLITE_ANY, 0,
			     fnct_FilterMbrIntersects, 0, 0);
    sqlite3_create_function (db, "LinesFromRings", 1, SQLITE_ANY, 0,
			     fnct_LinesFromRings, 0, 0);
    sqlite3_create_function (db, "LinesFromRings", 2, SQLITE_ANY, 0,
			     fnct_LinesFromRings, 0, 0);

#ifndef OMIT_GEOS		/* including GEOS */
    sqlite3_create_function (db, "BuildArea", 1, SQLITE_ANY, 0, fnct_BuildArea,
			     0, 0);
    sqlite3_create_function (db, "ST_BuildArea", 1, SQLITE_ANY, 0,
			     fnct_BuildArea, 0, 0);
    sqlite3_create_function (db, "Polygonize", 1, SQLITE_ANY, 0, 0,
			     fnct_Polygonize_step, fnct_Polygonize_final);
    sqlite3_create_function (db, "ST_Polygonize", 1, SQLITE_ANY, 0, 0,
			     fnct_Polygonize_step, fnct_Polygonize_final);
#endif /* end including GEOS */

    sqlite3_create_function (db, "DissolveSegments", 1, SQLITE_ANY, 0,
			     fnct_DissolveSegments, 0, 0);
    sqlite3_create_function (db, "ST_DissolveSegments", 1, SQLITE_ANY, 0,
			     fnct_DissolveSegments, 0, 0);
    sqlite3_create_function (db, "DissolvePoints", 1, SQLITE_ANY, 0,
			     fnct_DissolvePoints, 0, 0);
    sqlite3_create_function (db, "ST_DissolvePoints", 1, SQLITE_ANY, 0,
			     fnct_DissolvePoints, 0, 0);
    sqlite3_create_function (db, "CollectionExtract", 2, SQLITE_ANY, 0,
			     fnct_CollectionExtract, 0, 0);
    sqlite3_create_function (db, "ST_CollectionExtract", 2, SQLITE_ANY, 0,
			     fnct_CollectionExtract, 0, 0);
#ifndef OMIT_GEOCALLBACKS	/* supporting RTree geometry callbacks */
    sqlite3_rtree_geometry_callback (db, "RTreeWithin", fnct_RTreeIntersects,
				     0);
    sqlite3_rtree_geometry_callback (db, "RTreeContains", fnct_RTreeIntersects,
				     0);
    sqlite3_rtree_geometry_callback (db, "RTreeIntersects",
				     fnct_RTreeIntersects, 0);
    sqlite3_rtree_geometry_callback (db, "RTreeDistWithin",
				     fnct_RTreeDistWithin, 0);
#endif /* end RTree geometry callbacks */

/* some BLOB/JPEG/EXIF functions */
    sqlite3_create_function (db, "IsGeometryBlob", 1, SQLITE_ANY, 0,
			     fnct_IsGeometryBlob, 0, 0);
    sqlite3_create_function (db, "IsZipBlob", 1, SQLITE_ANY, 0,
			     fnct_IsZipBlob, 0, 0);
    sqlite3_create_function (db, "IsPdfBlob", 1, SQLITE_ANY, 0,
			     fnct_IsPdfBlob, 0, 0);
    sqlite3_create_function (db, "IsTiffBlob", 1, SQLITE_ANY, 0,
			     fnct_IsTiffBlob, 0, 0);
    sqlite3_create_function (db, "IsGifBlob", 1, SQLITE_ANY, 0,
			     fnct_IsGifBlob, 0, 0);
    sqlite3_create_function (db, "IsPngBlob", 1, SQLITE_ANY, 0,
			     fnct_IsPngBlob, 0, 0);
    sqlite3_create_function (db, "IsJpegBlob", 1, SQLITE_ANY, 0,
			     fnct_IsJpegBlob, 0, 0);
    sqlite3_create_function (db, "IsExifBlob", 1, SQLITE_ANY, 0,
			     fnct_IsExifBlob, 0, 0);
    sqlite3_create_function (db, "IsExifGpsBlob", 1, SQLITE_ANY, 0,
			     fnct_IsExifGpsBlob, 0, 0);
    sqlite3_create_function (db, "GeomFromExifGpsBlob", 1, SQLITE_ANY, 0,
			     fnct_GeomFromExifGpsBlob, 0, 0);

/* some Geodesic functions */
    sqlite3_create_function (db, "GreatCircleLength", 1, SQLITE_ANY, 0,
			     fnct_GreatCircleLength, 0, 0);
    sqlite3_create_function (db, "GeodesicLength", 1, SQLITE_ANY, 0,
			     fnct_GeodesicLength, 0, 0);

/* some Length Unit conversion functions */
    sqlite3_create_function (db, "CvtToKm", 1, SQLITE_ANY, 0, fnct_cvtToKm, 0,
			     0);
    sqlite3_create_function (db, "CvtToDm", 1, SQLITE_ANY, 0, fnct_cvtToDm, 0,
			     0);
    sqlite3_create_function (db, "CvtToCm", 1, SQLITE_ANY, 0, fnct_cvtToCm, 0,
			     0);
    sqlite3_create_function (db, "CvtToMm", 1, SQLITE_ANY, 0, fnct_cvtToMm, 0,
			     0);
    sqlite3_create_function (db, "CvtToKmi", 1, SQLITE_ANY, 0, fnct_cvtToKmi,
			     0, 0);
    sqlite3_create_function (db, "CvtToIn", 1, SQLITE_ANY, 0, fnct_cvtToIn, 0,
			     0);
    sqlite3_create_function (db, "CvtToFt", 1, SQLITE_ANY, 0, fnct_cvtToFt, 0,
			     0);
    sqlite3_create_function (db, "CvtToYd", 1, SQLITE_ANY, 0, fnct_cvtToYd, 0,
			     0);
    sqlite3_create_function (db, "CvtToMi", 1, SQLITE_ANY, 0, fnct_cvtToMi, 0,
			     0);
    sqlite3_create_function (db, "CvtToFath", 1, SQLITE_ANY, 0,
			     fnct_cvtToFath, 0, 0);
    sqlite3_create_function (db, "CvtToCh", 1, SQLITE_ANY, 0, fnct_cvtToCh, 0,
			     0);
    sqlite3_create_function (db, "CvtToLink", 1, SQLITE_ANY, 0,
			     fnct_cvtToLink, 0, 0);
    sqlite3_create_function (db, "CvtToUsIn", 1, SQLITE_ANY, 0,
			     fnct_cvtToUsIn, 0, 0);
    sqlite3_create_function (db, "CvtToUsFt", 1, SQLITE_ANY, 0,
			     fnct_cvtToUsFt, 0, 0);
    sqlite3_create_function (db, "CvtToUsYd", 1, SQLITE_ANY, 0,
			     fnct_cvtToUsYd, 0, 0);
    sqlite3_create_function (db, "CvtToUsCh", 1, SQLITE_ANY, 0,
			     fnct_cvtToUsCh, 0, 0);
    sqlite3_create_function (db, "CvtToUsMi", 1, SQLITE_ANY, 0,
			     fnct_cvtToUsMi, 0, 0);
    sqlite3_create_function (db, "CvtToIndFt", 1, SQLITE_ANY, 0,
			     fnct_cvtToIndFt, 0, 0);
    sqlite3_create_function (db, "CvtToIndYd", 1, SQLITE_ANY, 0,
			     fnct_cvtToIndYd, 0, 0);
    sqlite3_create_function (db, "CvtToIndCh", 1, SQLITE_ANY, 0,
			     fnct_cvtToIndCh, 0, 0);
    sqlite3_create_function (db, "CvtFromKm", 1, SQLITE_ANY, 0,
			     fnct_cvtFromKm, 0, 0);
    sqlite3_create_function (db, "CvtFromDm", 1, SQLITE_ANY, 0,
			     fnct_cvtFromDm, 0, 0);
    sqlite3_create_function (db, "CvtFromCm", 1, SQLITE_ANY, 0,
			     fnct_cvtFromCm, 0, 0);
    sqlite3_create_function (db, "CvtFromMm", 1, SQLITE_ANY, 0,
			     fnct_cvtFromMm, 0, 0);
    sqlite3_create_function (db, "CvtFromKmi", 1, SQLITE_ANY, 0,
			     fnct_cvtFromKmi, 0, 0);
    sqlite3_create_function (db, "CvtFromIn", 1, SQLITE_ANY, 0,
			     fnct_cvtFromIn, 0, 0);
    sqlite3_create_function (db, "CvtFromFt", 1, SQLITE_ANY, 0,
			     fnct_cvtFromFt, 0, 0);
    sqlite3_create_function (db, "CvtFromYd", 1, SQLITE_ANY, 0,
			     fnct_cvtFromYd, 0, 0);
    sqlite3_create_function (db, "CvtFromMi", 1, SQLITE_ANY, 0,
			     fnct_cvtFromMi, 0, 0);
    sqlite3_create_function (db, "CvtFromFath", 1, SQLITE_ANY, 0,
			     fnct_cvtFromFath, 0, 0);
    sqlite3_create_function (db, "CvtFromCh", 1, SQLITE_ANY, 0,
			     fnct_cvtFromCh, 0, 0);
    sqlite3_create_function (db, "CvtFromLink", 1, SQLITE_ANY, 0,
			     fnct_cvtFromLink, 0, 0);
    sqlite3_create_function (db, "CvtFromUsIn", 1, SQLITE_ANY, 0,
			     fnct_cvtFromUsIn, 0, 0);
    sqlite3_create_function (db, "CvtFromUsFt", 1, SQLITE_ANY, 0,
			     fnct_cvtFromUsFt, 0, 0);
    sqlite3_create_function (db, "CvtFromUsYd", 1, SQLITE_ANY, 0,
			     fnct_cvtFromUsYd, 0, 0);
    sqlite3_create_function (db, "CvtFromUsCh", 1, SQLITE_ANY, 0,
			     fnct_cvtFromUsCh, 0, 0);
    sqlite3_create_function (db, "CvtFromUsMi", 1, SQLITE_ANY, 0,
			     fnct_cvtFromUsMi, 0, 0);
    sqlite3_create_function (db, "CvtFromIndFt", 1, SQLITE_ANY, 0,
			     fnct_cvtFromIndFt, 0, 0);
    sqlite3_create_function (db, "CvtFromIndYd", 1, SQLITE_ANY, 0,
			     fnct_cvtFromIndYd, 0, 0);
    sqlite3_create_function (db, "CvtFromIndCh", 1, SQLITE_ANY, 0,
			     fnct_cvtFromIndCh, 0, 0);

#ifndef OMIT_MATHSQL		/* supporting SQL math functions */

/* some extra math functions */
    sqlite3_create_function (db, "acos", 1, SQLITE_ANY, 0, fnct_math_acos, 0,
			     0);
    sqlite3_create_function (db, "asin", 1, SQLITE_ANY, 0, fnct_math_asin, 0,
			     0);
    sqlite3_create_function (db, "atan", 1, SQLITE_ANY, 0, fnct_math_atan, 0,
			     0);
    sqlite3_create_function (db, "ceil", 1, SQLITE_ANY, 0, fnct_math_ceil, 0,
			     0);
    sqlite3_create_function (db, "ceiling", 1, SQLITE_ANY, 0, fnct_math_ceil,
			     0, 0);
    sqlite3_create_function (db, "cos", 1, SQLITE_ANY, 0, fnct_math_cos, 0, 0);
    sqlite3_create_function (db, "cot", 1, SQLITE_ANY, 0, fnct_math_cot, 0, 0);
    sqlite3_create_function (db, "degrees", 1, SQLITE_ANY, 0,
			     fnct_math_degrees, 0, 0);
    sqlite3_create_function (db, "exp", 1, SQLITE_ANY, 0, fnct_math_exp, 0, 0);
    sqlite3_create_function (db, "floor", 1, SQLITE_ANY, 0, fnct_math_floor,
			     0, 0);
    sqlite3_create_function (db, "ln", 1, SQLITE_ANY, 0, fnct_math_logn, 0, 0);
    sqlite3_create_function (db, "log", 1, SQLITE_ANY, 0, fnct_math_logn, 0, 0);
    sqlite3_create_function (db, "log", 2, SQLITE_ANY, 0, fnct_math_logn2, 0,
			     0);
    sqlite3_create_function (db, "log2", 1, SQLITE_ANY, 0, fnct_math_log_2, 0,
			     0);
    sqlite3_create_function (db, "log10", 1, SQLITE_ANY, 0, fnct_math_log_10,
			     0, 0);
    sqlite3_create_function (db, "pi", 0, SQLITE_ANY, 0, fnct_math_pi, 0, 0);
    sqlite3_create_function (db, "pow", 2, SQLITE_ANY, 0, fnct_math_pow, 0, 0);
    sqlite3_create_function (db, "power", 2, SQLITE_ANY, 0, fnct_math_pow, 0,
			     0);
    sqlite3_create_function (db, "radians", 1, SQLITE_ANY, 0,
			     fnct_math_radians, 0, 0);
    sqlite3_create_function (db, "round", 1, SQLITE_ANY, 0, fnct_math_round,
			     0, 0);
    sqlite3_create_function (db, "sign", 1, SQLITE_ANY, 0, fnct_math_sign, 0,
			     0);
    sqlite3_create_function (db, "sin", 1, SQLITE_ANY, 0, fnct_math_sin, 0, 0);
    sqlite3_create_function (db, "stddev_pop", 1, SQLITE_ANY, 0, 0,
			     fnct_math_stddev_step, fnct_math_stddev_pop_final);
    sqlite3_create_function (db, "stddev_samp", 1, SQLITE_ANY, 0, 0,
			     fnct_math_stddev_step,
			     fnct_math_stddev_samp_final);
    sqlite3_create_function (db, "sqrt", 1, SQLITE_ANY, 0, fnct_math_sqrt, 0,
			     0);
    sqlite3_create_function (db, "tan", 1, SQLITE_ANY, 0, fnct_math_tan, 0, 0);
    sqlite3_create_function (db, "var_pop", 1, SQLITE_ANY, 0, 0,
			     fnct_math_stddev_step, fnct_math_var_pop_final);
    sqlite3_create_function (db, "var_samp", 1, SQLITE_ANY, 0, 0,
			     fnct_math_stddev_step, fnct_math_var_samp_final);

#endif /* end supporting SQL math functions */

#ifndef OMIT_PROJ		/* including PROJ.4 */

    sqlite3_create_function (db, "Transform", 2, SQLITE_ANY, 0,
			     fnct_Transform, 0, 0);
    sqlite3_create_function (db, "ST_Transform", 2, SQLITE_ANY, 0,
			     fnct_Transform, 0, 0);

#endif /* end including PROJ.4 */

#ifndef OMIT_GEOS		/* including GEOS */

    initGEOS (geos_warning, geos_error);
    sqlite3_create_function (db, "Boundary", 1, SQLITE_ANY, 0, fnct_Boundary,
			     0, 0);
    sqlite3_create_function (db, "ST_Boundary", 1, SQLITE_ANY, 0,
			     fnct_Boundary, 0, 0);
    sqlite3_create_function (db, "IsClosed", 1, SQLITE_ANY, 0, fnct_IsClosed,
			     0, 0);
    sqlite3_create_function (db, "ST_IsClosed", 1, SQLITE_ANY, 0,
			     fnct_IsClosed, 0, 0);
    sqlite3_create_function (db, "IsSimple", 1, SQLITE_ANY, 0, fnct_IsSimple,
			     0, 0);
    sqlite3_create_function (db, "ST_IsSimple", 1, SQLITE_ANY, 0,
			     fnct_IsSimple, 0, 0);
    sqlite3_create_function (db, "IsRing", 1, SQLITE_ANY, 0, fnct_IsRing, 0, 0);
    sqlite3_create_function (db, "ST_IsRing", 1, SQLITE_ANY, 0, fnct_IsRing,
			     0, 0);
    sqlite3_create_function (db, "IsValid", 1, SQLITE_ANY, 0, fnct_IsValid, 0,
			     0);
    sqlite3_create_function (db, "ST_IsValid", 1, SQLITE_ANY, 0, fnct_IsValid,
			     0, 0);
    sqlite3_create_function (db, "GLength", 1, SQLITE_ANY, 0, fnct_Length, 0,
			     0);
    sqlite3_create_function (db, "ST_Length", 1, SQLITE_ANY, 0, fnct_Length,
			     0, 0);
    sqlite3_create_function (db, "Area", 1, SQLITE_ANY, 0, fnct_Area, 0, 0);
    sqlite3_create_function (db, "ST_Area", 1, SQLITE_ANY, 0, fnct_Area, 0, 0);
    sqlite3_create_function (db, "Centroid", 1, SQLITE_ANY, 0, fnct_Centroid,
			     0, 0);
    sqlite3_create_function (db, "ST_Centroid", 1, SQLITE_ANY, 0,
			     fnct_Centroid, 0, 0);
    sqlite3_create_function (db, "PointOnSurface", 1, SQLITE_ANY, 0,
			     fnct_PointOnSurface, 0, 0);
    sqlite3_create_function (db, "ST_PointOnSurface", 1, SQLITE_ANY, 0,
			     fnct_PointOnSurface, 0, 0);
    sqlite3_create_function (db, "Simplify", 2, SQLITE_ANY, 0, fnct_Simplify,
			     0, 0);
    sqlite3_create_function (db, "ST_Generalize", 2, SQLITE_ANY, 0,
			     fnct_Simplify, 0, 0);
    sqlite3_create_function (db, "SimplifyPreserveTopology", 2, SQLITE_ANY, 0,
			     fnct_SimplifyPreserveTopology, 0, 0);
    sqlite3_create_function (db, "ConvexHull", 1, SQLITE_ANY, 0,
			     fnct_ConvexHull, 0, 0);
    sqlite3_create_function (db, "ST_ConvexHull", 1, SQLITE_ANY, 0,
			     fnct_ConvexHull, 0, 0);
    sqlite3_create_function (db, "Buffer", 2, SQLITE_ANY, 0, fnct_Buffer, 0, 0);
    sqlite3_create_function (db, "ST_Buffer", 2, SQLITE_ANY, 0, fnct_Buffer,
			     0, 0);
    sqlite3_create_function (db, "Intersection", 2, SQLITE_ANY, 0,
			     fnct_Intersection, 0, 0);
    sqlite3_create_function (db, "ST_Intersection", 2, SQLITE_ANY, 0,
			     fnct_Intersection, 0, 0);
    sqlite3_create_function (db, "GUnion", 1, SQLITE_ANY, 0, 0,
			     fnct_Union_step, fnct_Union_final);
    sqlite3_create_function (db, "GUnion", 2, SQLITE_ANY, 0, fnct_Union, 0, 0);
    sqlite3_create_function (db, "ST_Union", 1, SQLITE_ANY, 0, 0,
			     fnct_Union_step, fnct_Union_final);
    sqlite3_create_function (db, "ST_Union", 2, SQLITE_ANY, 0, fnct_Union, 0,
			     0);
    sqlite3_create_function (db, "Difference", 2, SQLITE_ANY, 0,
			     fnct_Difference, 0, 0);
    sqlite3_create_function (db, "ST_Difference", 2, SQLITE_ANY, 0,
			     fnct_Difference, 0, 0);
    sqlite3_create_function (db, "SymDifference", 2, SQLITE_ANY, 0,
			     fnct_SymDifference, 0, 0);
    sqlite3_create_function (db, "ST_SymDifference", 2, SQLITE_ANY, 0,
			     fnct_SymDifference, 0, 0);
    sqlite3_create_function (db, "Equals", 2, SQLITE_ANY, 0, fnct_Equals, 0, 0);
    sqlite3_create_function (db, "ST_Equals", 2, SQLITE_ANY, 0, fnct_Equals,
			     0, 0);
    sqlite3_create_function (db, "Intersects", 2, SQLITE_ANY, 0,
			     fnct_Intersects, 0, 0);
    sqlite3_create_function (db, "ST_Intersects", 2, SQLITE_ANY, 0,
			     fnct_Intersects, 0, 0);
    sqlite3_create_function (db, "Disjoint", 2, SQLITE_ANY, 0, fnct_Disjoint,
			     0, 0);
    sqlite3_create_function (db, "ST_Disjoint", 2, SQLITE_ANY, 0,
			     fnct_Disjoint, 0, 0);
    sqlite3_create_function (db, "Overlaps", 2, SQLITE_ANY, 0, fnct_Overlaps,
			     0, 0);
    sqlite3_create_function (db, "ST_Overlaps", 2, SQLITE_ANY, 0,
			     fnct_Overlaps, 0, 0);
    sqlite3_create_function (db, "Crosses", 2, SQLITE_ANY, 0, fnct_Crosses, 0,
			     0);
    sqlite3_create_function (db, "ST_Crosses", 2, SQLITE_ANY, 0, fnct_Crosses,
			     0, 0);
    sqlite3_create_function (db, "Touches", 2, SQLITE_ANY, 0, fnct_Touches, 0,
			     0);
    sqlite3_create_function (db, "ST_Touches", 2, SQLITE_ANY, 0, fnct_Touches,
			     0, 0);
    sqlite3_create_function (db, "Within", 2, SQLITE_ANY, 0, fnct_Within, 0, 0);
    sqlite3_create_function (db, "ST_Within", 2, SQLITE_ANY, 0, fnct_Within,
			     0, 0);
    sqlite3_create_function (db, "Contains", 2, SQLITE_ANY, 0, fnct_Contains,
			     0, 0);
    sqlite3_create_function (db, "ST_Contains", 2, SQLITE_ANY, 0,
			     fnct_Contains, 0, 0);
    sqlite3_create_function (db, "Relate", 3, SQLITE_ANY, 0, fnct_Relate, 0, 0);
    sqlite3_create_function (db, "ST_Relate", 3, SQLITE_ANY, 0, fnct_Relate,
			     0, 0);
    sqlite3_create_function (db, "Distance", 2, SQLITE_ANY, 0, fnct_Distance,
			     0, 0);
    sqlite3_create_function (db, "ST_Distance", 2, SQLITE_ANY, 0,
			     fnct_Distance, 0, 0);
    sqlite3_create_function (db, "PtDistWithin", 3, SQLITE_ANY, 0,
			     fnct_PtDistWithin, 0, 0);
    sqlite3_create_function (db, "PtDistWithin", 4, SQLITE_ANY, 0,
			     fnct_PtDistWithin, 0, 0);
    sqlite3_create_function (db, "BdPolyFromText", 1, SQLITE_ANY, 0,
			     fnct_BdPolyFromText1, 0, 0);
    sqlite3_create_function (db, "BdPolyFromText", 2, SQLITE_ANY, 0,
			     fnct_BdPolyFromText2, 0, 0);
    sqlite3_create_function (db, "BdMPolyFromText", 1, SQLITE_ANY, 0,
			     fnct_BdMPolyFromText1, 0, 0);
    sqlite3_create_function (db, "BdMPolyFromText", 2, SQLITE_ANY, 0,
			     fnct_BdMPolyFromText2, 0, 0);
    sqlite3_create_function (db, "BdPolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_BdPolyFromWKB1, 0, 0);
    sqlite3_create_function (db, "BdPolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_BdPolyFromWKB2, 0, 0);
    sqlite3_create_function (db, "BdMPolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_BdMPolyFromWKB1, 0, 0);
    sqlite3_create_function (db, "BdMPolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_BdMPolyFromWKB2, 0, 0);
    sqlite3_create_function (db, "ST_BdPolyFromText", 1, SQLITE_ANY, 0,
			     fnct_BdPolyFromText1, 0, 0);
    sqlite3_create_function (db, "ST_BdPolyFromText", 2, SQLITE_ANY, 0,
			     fnct_BdPolyFromText2, 0, 0);
    sqlite3_create_function (db, "ST_BdMPolyFromText", 1, SQLITE_ANY, 0,
			     fnct_BdMPolyFromText1, 0, 0);
    sqlite3_create_function (db, "ST_BdMPolyFromText", 2, SQLITE_ANY, 0,
			     fnct_BdMPolyFromText2, 0, 0);
    sqlite3_create_function (db, "ST_BdPolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_BdPolyFromWKB1, 0, 0);
    sqlite3_create_function (db, "ST_BdPolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_BdPolyFromWKB2, 0, 0);
    sqlite3_create_function (db, "ST_BdMPolyFromWKB", 1, SQLITE_ANY, 0,
			     fnct_BdMPolyFromWKB1, 0, 0);
    sqlite3_create_function (db, "ST_BdMPolyFromWKB", 2, SQLITE_ANY, 0,
			     fnct_BdMPolyFromWKB2, 0, 0);

#ifdef GEOS_ADVANCED		/* GEOS advanced and experimental features */

    sqlite3_create_function (db, "OffsetCurve", 3, SQLITE_ANY, 0,
			     fnct_OffsetCurve, 0, 0);
    sqlite3_create_function (db, "ST_OffsetCurve", 3, SQLITE_ANY, 0,
			     fnct_OffsetCurve, 0, 0);
    sqlite3_create_function (db, "SingleSidedBuffer", 3, SQLITE_ANY, 0,
			     fnct_SingleSidedBuffer, 0, 0);
    sqlite3_create_function (db, "ST_SingleSidedBuffer", 3, SQLITE_ANY, 0,
			     fnct_SingleSidedBuffer, 0, 0);
    sqlite3_create_function (db, "HausdorffDistance", 2, SQLITE_ANY, 0,
			     fnct_HausdorffDistance, 0, 0);
    sqlite3_create_function (db, "ST_HausdorffDistance", 2, SQLITE_ANY, 0,
			     fnct_HausdorffDistance, 0, 0);
    sqlite3_create_function (db, "SharedPaths", 2, SQLITE_ANY, 0,
			     fnct_SharedPaths, 0, 0);
    sqlite3_create_function (db, "ST_SharedPaths", 2, SQLITE_ANY, 0,
			     fnct_SharedPaths, 0, 0);
    sqlite3_create_function (db, "Covers", 2, SQLITE_ANY, 0, fnct_Covers, 0, 0);
    sqlite3_create_function (db, "ST_Covers", 2, SQLITE_ANY, 0, fnct_Covers, 0,
			     0);
    sqlite3_create_function (db, "CoveredBy", 2, SQLITE_ANY, 0, fnct_CoveredBy,
			     0, 0);
    sqlite3_create_function (db, "ST_CoveredBy", 2, SQLITE_ANY, 0,
			     fnct_CoveredBy, 0, 0);
    sqlite3_create_function (db, "Line_Interpolate_Point", 2, SQLITE_ANY, 0,
			     fnct_LineInterpolatePoint, 0, 0);
    sqlite3_create_function (db, "ST_Line_Interpolate_Point", 2, SQLITE_ANY, 0,
			     fnct_LineInterpolatePoint, 0, 0);
    sqlite3_create_function (db, "Line_Locate_Point", 2, SQLITE_ANY, 0,
			     fnct_LineLocatePoint, 0, 0);
    sqlite3_create_function (db, "ST_Line_Locate_Point", 2, SQLITE_ANY, 0,
			     fnct_LineLocatePoint, 0, 0);
    sqlite3_create_function (db, "Line_Substring", 3, SQLITE_ANY, 0,
			     fnct_LineSubstring, 0, 0);
    sqlite3_create_function (db, "ST_Line_Substring", 3, SQLITE_ANY, 0,
			     fnct_LineSubstring, 0, 0);
    sqlite3_create_function (db, "ClosestPoint", 2, SQLITE_ANY, 0,
			     fnct_ClosestPoint, 0, 0);
    sqlite3_create_function (db, "ST_ClosestPoint", 2, SQLITE_ANY, 0,
			     fnct_ClosestPoint, 0, 0);
    sqlite3_create_function (db, "ShortestLine", 2, SQLITE_ANY, 0,
			     fnct_ShortestLine, 0, 0);
    sqlite3_create_function (db, "ST_ShortestLine", 2, SQLITE_ANY, 0,
			     fnct_ShortestLine, 0, 0);
    sqlite3_create_function (db, "Snap", 3, SQLITE_ANY, 0, fnct_Snap, 0, 0);
    sqlite3_create_function (db, "ST_Snap", 3, SQLITE_ANY, 0, fnct_Snap, 0, 0);
    sqlite3_create_function (db, "LineMerge", 1, SQLITE_ANY, 0,
			     fnct_LineMerge, 0, 0);
    sqlite3_create_function (db, "ST_LineMerge", 1, SQLITE_ANY, 0,
			     fnct_LineMerge, 0, 0);
    sqlite3_create_function (db, "UnaryUnion", 1, SQLITE_ANY, 0,
			     fnct_UnaryUnion, 0, 0);
    sqlite3_create_function (db, "ST_UnaryUnion", 1, SQLITE_ANY, 0,
			     fnct_UnaryUnion, 0, 0);

#endif /* end GEOS advanced and experimental features */

#endif /* end including GEOS */
}

static void
init_spatialite_virtualtables (sqlite3 * db)
{
#ifndef OMIT_ICONV		/* when ICONV is disabled SHP/DBF/TXT cannot be supported */
/* initializing the VirtualShape  extension */
    virtualshape_extension_init (db);
/* initializing the VirtualDbf  extension */
    virtualdbf_extension_init (db);
/* initializing the VirtualText extension */
    virtualtext_extension_init (db);
#ifndef OMIT_FREEXL
/* initializing the VirtualXL  extension */
    virtualXL_extension_init (db);
#endif /* FreeXL enabled/disable */
#endif /* ICONV enabled/disabled */

/* initializing the VirtualNetwork  extension */
    virtualnetwork_extension_init (db);
/* initializing the MbrCache  extension */
    mbrcache_extension_init (db);
/* initializing the VirtualFDO  extension */
    virtualfdo_extension_init (db);
/* initializing the VirtualSpatialIndex  extension */
    virtual_spatialindex_extension_init (db);
}

static void
init_static_spatialite (sqlite3 * db, char **pzErrMsg,
			const sqlite3_api_routines * pApi)
{
    SQLITE_EXTENSION_INIT2 (pApi);
/* setting the POSIX locale for numeric */
    setlocale (LC_NUMERIC, "POSIX");
    *pzErrMsg = NULL;

    register_spatialite_sql_functions (db);

    init_spatialite_virtualtables (db);

/* setting a timeout handler */
    sqlite3_busy_timeout (db, 5000);
}

void
spatialite_init (int verbose)
{
/* used when SQLite initializes SpatiaLite via statically linked lib */
    sqlite3_auto_extension ((void (*)(void)) init_static_spatialite);
    if (isatty (1))
      {
	  /* printing "hello" message only when stdout is on console */
	  if (verbose)
	    {
		printf ("SpatiaLite version ..: %s", spatialite_version ());
		printf ("\tSupported Extensions:\n");
#ifndef OMIT_ICONV		/* ICONV is required by SHP/DBF/TXT */
		printf ("\t- 'VirtualShape'\t[direct Shapefile access]\n");
		printf ("\t- 'VirtualDbf'\t\t[direct DBF access]\n");
#ifndef OMIT_FREEXL
		printf ("\t- 'VirtualXL'\t\t[direct XLS access]\n");
#endif /* end FreeXL conditional */
		printf ("\t- 'VirtualText'\t\t[direct CSV/TXT access]\n");
#endif /* end ICONV conditional */
		printf ("\t- 'VirtualNetwork'\t[Dijkstra shortest path]\n");
		printf ("\t- 'RTree'\t\t[Spatial Index - R*Tree]\n");
		printf ("\t- 'MbrCache'\t\t[Spatial Index - MBR cache]\n");
		printf ("\t- 'VirtualSpatialIndex'\t[R*Tree metahandler]\n");
		printf ("\t- 'VirtualFDO'\t\t[FDO-OGR interoperability]\n");
		printf ("\t- 'SpatiaLite'\t\t[Spatial SQL - OGC]\n");
	    }
#ifndef OMIT_PROJ		/* PROJ.4 version */
	  if (verbose)
	      printf ("PROJ.4 version ......: %s\n", pj_get_release ());
#endif /* end including PROJ.4 */
#ifndef OMIT_GEOS		/* GEOS version */
	  if (verbose)
	      printf ("GEOS version ........: %s\n", GEOSversion ());
#endif /* end GEOS version */
      }
}

void
spatialite_cleanup ()
{
#ifndef OMIT_GEOS
    finishGEOS ();
#endif
}

SPATIALITE_DECLARE int
sqlite3_extension_init (sqlite3 * db, char **pzErrMsg,
			const sqlite3_api_routines * pApi)
{
/* SQLite invokes this routine once when it dynamically loads the extension. */
    SQLITE_EXTENSION_INIT2 (pApi);
    setlocale (LC_NUMERIC, "POSIX");
    *pzErrMsg = NULL;

    register_spatialite_sql_functions (db);

    init_spatialite_virtualtables (db);

    /* setting a timeout handler */
    sqlite3_busy_timeout (db, 5000);
    return 0;
}

SPATIALITE_DECLARE sqlite3_int64
math_llabs (sqlite3_int64 value)
{
/* replacing the C99 llabs() function */
    return value < 0 ? -value : value;
}

SPATIALITE_DECLARE double
math_round (double value)
{
/* replacing the C99 round() function */
    double min = floor (value);
    if (fabs (value - min) < 0.5)
	return min;
    return min + 1.0;
}

static int
check_layer_statistics (sqlite3 * sqlite)
{
/*
/ checks the LAYER_STATISTICS table for validity;
/ if the table doesn't exist, attempts to create
*/
    char sql[8192];
    char **results;
    int rows;
    int columns;
    int ret;
    int raster_layer = 0;
    int table_name = 0;
    int geometry_column = 0;
    int row_count = 0;
    int extent_min_x = 0;
    int extent_min_y = 0;
    int extent_max_x = 0;
    int extent_max_y = 0;
    int i;
    const char *name;

/* checking the LAYER_STATISTICS table */
    strcpy (sql, "PRAGMA table_info(layer_statistics)");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	return 0;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, "raster_layer") == 0)
		    raster_layer = 1;
		if (strcasecmp (name, "table_name") == 0)
		    table_name = 1;
		if (strcasecmp (name, "geometry_column") == 0)
		    geometry_column = 1;
		if (strcasecmp (name, "row_count") == 0)
		    row_count = 1;
		if (strcasecmp (name, "extent_min_x") == 0)
		    extent_min_x = 1;
		if (strcasecmp (name, "extent_min_y") == 0)
		    extent_min_x = 1;
		if (strcasecmp (name, "extent_max_x") == 0)
		    extent_max_x = 1;
		if (strcasecmp (name, "extent_max_y") == 0)
		    extent_max_y = 1;
	    }
      }
    sqlite3_free_table (results);

/* LAYER_STATISTICS already exists and has a valid layout */
    if (raster_layer && table_name && geometry_column && row_count
	&& extent_min_x && extent_max_x && extent_max_y)
	return 1;
/* LAYER_STATISTICS already exists, but has an invalid layout */
    if (raster_layer || table_name || geometry_column || row_count
	|| extent_min_x || extent_max_x || extent_max_y)
	return 0;

/* attempting to create LAYER_STATISTICS */
    strcpy (sql, "CREATE TABLE layer_statistics (\n");
    strcat (sql, "raster_layer INTEGER NOT NULL,\n");
    strcat (sql, "table_name TEXT NOT NULL,\n");
    strcat (sql, "geometry_column TEXT NOT NULL,\n");
    strcat (sql, "row_count INTEGER,\n");
    strcat (sql, "extent_min_x DOUBLE,\n");
    strcat (sql, "extent_min_y DOUBLE,\n");
    strcat (sql, "extent_max_x DOUBLE,\n");
    strcat (sql, "extent_max_y DOUBLE,\n");
    strcat (sql, "CONSTRAINT pk_layer_statistics PRIMARY KEY ");
    strcat (sql, "(raster_layer, table_name, geometry_column),\n");
    strcat (sql, "CONSTRAINT fk_layer_statistics FOREIGN KEY ");
    strcat (sql, "(table_name, geometry_column) REFERENCES ");
    strcat (sql, "geometry_columns (f_table_name, f_geometry_column) ");
    strcat (sql, "ON DELETE CASCADE)");
    ret = sqlite3_exec (sqlite, sql, NULL, 0, NULL);
    if (ret != SQLITE_OK)
	return 0;
    return 1;
}

static int
do_update_layer_statistics (sqlite3 * sqlite, const char *table,
			    const char *column, int count, int has_coords,
			    double min_x, double min_y, double max_x,
			    double max_y)
{
/* update LAYER_STATISTICS [single table/geometry] */
    char sql[8192];
    int ret;
    int error = 0;
    sqlite3_stmt *stmt;

    if (!check_layer_statistics (sqlite))
	return 0;
    strcpy (sql, "INSERT OR REPLACE INTO layer_statistics ");
    strcat (sql, "(raster_layer, table_name, geometry_column, ");
    strcat (sql, "row_count, extent_min_x, extent_min_y, ");
    strcat (sql, "extent_max_x, extent_max_y) ");
    strcat (sql, "VALUES (0, ?, ?, ?, ?, ?, ?, ?)");

/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	return 0;

/* binding INSERT params */
    sqlite3_reset (stmt);
    sqlite3_clear_bindings (stmt);
    sqlite3_bind_text (stmt, 1, table, strlen (table), SQLITE_STATIC);
    sqlite3_bind_text (stmt, 2, column, strlen (column), SQLITE_STATIC);
    sqlite3_bind_int (stmt, 3, count);
    if (has_coords)
      {
	  sqlite3_bind_double (stmt, 4, min_x);
	  sqlite3_bind_double (stmt, 5, min_y);
	  sqlite3_bind_double (stmt, 6, max_x);
	  sqlite3_bind_double (stmt, 7, max_y);
      }
    else
      {
	  sqlite3_bind_null (stmt, 4);
	  sqlite3_bind_null (stmt, 5);
	  sqlite3_bind_null (stmt, 6);
	  sqlite3_bind_null (stmt, 7);
      }
    ret = sqlite3_step (stmt);
    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	;
    else
	error = 1;
    ret = sqlite3_finalize (stmt);
    if (ret != SQLITE_OK)
	return 0;
    if (error)
	return 0;
    return 1;
}

static int
check_views_layer_statistics (sqlite3 * sqlite)
{
/*
/ checks the VIEWS_LAYER_STATISTICS table for validity;
/ if the table doesn't exist, attempts to create
*/
    char sql[8192];
    char **results;
    int rows;
    int columns;
    int ret;
    int view_name = 0;
    int view_geometry = 0;
    int row_count = 0;
    int extent_min_x = 0;
    int extent_min_y = 0;
    int extent_max_x = 0;
    int extent_max_y = 0;
    int i;
    const char *name;

/* checking the VIEWS_LAYER_STATISTICS table */
    strcpy (sql, "PRAGMA table_info(views_layer_statistics)");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	return 0;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, "view_name") == 0)
		    view_name = 1;
		if (strcasecmp (name, "view_geometry") == 0)
		    view_geometry = 1;
		if (strcasecmp (name, "row_count") == 0)
		    row_count = 1;
		if (strcasecmp (name, "extent_min_x") == 0)
		    extent_min_x = 1;
		if (strcasecmp (name, "extent_min_y") == 0)
		    extent_min_x = 1;
		if (strcasecmp (name, "extent_max_x") == 0)
		    extent_max_x = 1;
		if (strcasecmp (name, "extent_max_y") == 0)
		    extent_max_y = 1;
	    }
      }
    sqlite3_free_table (results);

/* VIEWS_LAYER_STATISTICS already exists and has a valid layout */
    if (view_name && view_geometry && row_count && extent_min_x && extent_max_x
	&& extent_max_y)
	return 1;
/* VIEWS_LAYER_STATISTICS already exists, but has an invalid layout */
    if (view_name || view_geometry || row_count || extent_min_x || extent_max_x
	|| extent_max_y)
	return 0;

/* attempting to create VIEWS_LAYER_STATISTICS */
    strcpy (sql, "CREATE TABLE views_layer_statistics (\n");
    strcat (sql, "view_name TEXT NOT NULL,\n");
    strcat (sql, "view_geometry TEXT NOT NULL,\n");
    strcat (sql, "row_count INTEGER,\n");
    strcat (sql, "extent_min_x DOUBLE,\n");
    strcat (sql, "extent_min_y DOUBLE,\n");
    strcat (sql, "extent_max_x DOUBLE,\n");
    strcat (sql, "extent_max_y DOUBLE,\n");
    strcat (sql, "CONSTRAINT pk_views_layer_statistics PRIMARY KEY ");
    strcat (sql, "(view_name, view_geometry),\n");
    strcat (sql, "CONSTRAINT fk_views_layer_statistics FOREIGN KEY ");
    strcat (sql, "(view_name, view_geometry) REFERENCES ");
    strcat (sql, "views_geometry_columns (view_name, view_geometry) ");
    strcat (sql, "ON DELETE CASCADE)");
    ret = sqlite3_exec (sqlite, sql, NULL, 0, NULL);
    if (ret != SQLITE_OK)
	return 0;
    return 1;
}

static int
do_update_views_layer_statistics (sqlite3 * sqlite, const char *table,
				  const char *column, int count, int has_coords,
				  double min_x, double min_y, double max_x,
				  double max_y)
{
/* update VIEWS_LAYER_STATISTICS [single table/geometry] */
    char sql[8192];
    int ret;
    int error = 0;
    sqlite3_stmt *stmt;

    if (!check_views_layer_statistics (sqlite))
	return 0;
    strcpy (sql, "INSERT OR REPLACE INTO views_layer_statistics ");
    strcat (sql, "(view_name, view_geometry, ");
    strcat (sql, "row_count, extent_min_x, extent_min_y, ");
    strcat (sql, "extent_max_x, extent_max_y) ");
    strcat (sql, "VALUES (?, ?, ?, ?, ?, ?, ?)");

/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	return 0;

/* binding INSERT params */
    sqlite3_reset (stmt);
    sqlite3_clear_bindings (stmt);
    sqlite3_bind_text (stmt, 1, table, strlen (table), SQLITE_STATIC);
    sqlite3_bind_text (stmt, 2, column, strlen (column), SQLITE_STATIC);
    sqlite3_bind_int (stmt, 3, count);
    if (has_coords)
      {
	  sqlite3_bind_double (stmt, 4, min_x);
	  sqlite3_bind_double (stmt, 5, min_y);
	  sqlite3_bind_double (stmt, 6, max_x);
	  sqlite3_bind_double (stmt, 7, max_y);
      }
    else
      {
	  sqlite3_bind_null (stmt, 4);
	  sqlite3_bind_null (stmt, 5);
	  sqlite3_bind_null (stmt, 6);
	  sqlite3_bind_null (stmt, 7);
      }
    ret = sqlite3_step (stmt);
    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	;
    else
	error = 1;
    ret = sqlite3_finalize (stmt);
    if (ret != SQLITE_OK)
	return 0;
    if (error)
	return 0;
    return 1;
}

static int
check_virts_layer_statistics (sqlite3 * sqlite)
{
/*
/ checks the VIRTS_LAYER_STATISTICS table for validity;
/ if the table doesn't exist, attempts to create
*/
    char sql[8192];
    char **results;
    int rows;
    int columns;
    int ret;
    int virt_name = 0;
    int virt_geometry = 0;
    int row_count = 0;
    int extent_min_x = 0;
    int extent_min_y = 0;
    int extent_max_x = 0;
    int extent_max_y = 0;
    int i;
    const char *name;

/* checking the VIRTS_LAYER_STATISTICS table */
    strcpy (sql, "PRAGMA table_info(virts_layer_statistics)");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	return 0;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, "virt_name") == 0)
		    virt_name = 1;
		if (strcasecmp (name, "virt_geometry") == 0)
		    virt_geometry = 1;
		if (strcasecmp (name, "row_count") == 0)
		    row_count = 1;
		if (strcasecmp (name, "extent_min_x") == 0)
		    extent_min_x = 1;
		if (strcasecmp (name, "extent_min_y") == 0)
		    extent_min_x = 1;
		if (strcasecmp (name, "extent_max_x") == 0)
		    extent_max_x = 1;
		if (strcasecmp (name, "extent_max_y") == 0)
		    extent_max_y = 1;
	    }
      }
    sqlite3_free_table (results);

/* VIRTS_LAYER_STATISTICS already exists and has a valid layout */
    if (virt_name && virt_geometry && row_count && extent_min_x && extent_max_x
	&& extent_max_y)
	return 1;
/* VIRTS_LAYER_STATISTICS already exists, but has an invalid layout */
    if (virt_name || virt_geometry || row_count || extent_min_x || extent_max_x
	|| extent_max_y)
	return 0;

/* attempting to create VIRTS_LAYER_STATISTICS */
    strcpy (sql, "CREATE TABLE virts_layer_statistics (\n");
    strcat (sql, "virt_name TEXT NOT NULL,\n");
    strcat (sql, "virt_geometry TEXT NOT NULL,\n");
    strcat (sql, "row_count INTEGER,\n");
    strcat (sql, "extent_min_x DOUBLE,\n");
    strcat (sql, "extent_min_y DOUBLE,\n");
    strcat (sql, "extent_max_x DOUBLE,\n");
    strcat (sql, "extent_max_y DOUBLE,\n");
    strcat (sql, "CONSTRAINT pk_virts_layer_statistics PRIMARY KEY ");
    strcat (sql, "(virt_name, virt_geometry),\n");
    strcat (sql, "CONSTRAINT fk_virts_layer_statistics FOREIGN KEY ");
    strcat (sql, "(virt_name, virt_geometry) REFERENCES ");
    strcat (sql, "virts_geometry_columns (virt_name, virt_geometry) ");
    strcat (sql, "ON DELETE CASCADE)");
    ret = sqlite3_exec (sqlite, sql, NULL, 0, NULL);
    if (ret != SQLITE_OK)
	return 0;
    return 1;
}

static int
do_update_virts_layer_statistics (sqlite3 * sqlite, const char *table,
				  const char *column, int count, int has_coords,
				  double min_x, double min_y, double max_x,
				  double max_y)
{
/* update VIRTS_LAYER_STATISTICS [single table/geometry] */
    char sql[8192];
    int ret;
    int error = 0;
    sqlite3_stmt *stmt;

    if (!check_virts_layer_statistics (sqlite))
	return 0;
    strcpy (sql, "INSERT OR REPLACE INTO virts_layer_statistics ");
    strcat (sql, "(virt_name, virt_geometry, ");
    strcat (sql, "row_count, extent_min_x, extent_min_y, ");
    strcat (sql, "extent_max_x, extent_max_y) ");
    strcat (sql, "VALUES (?, ?, ?, ?, ?, ?, ?)");

/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	return 0;

/* binding INSERT params */
    sqlite3_reset (stmt);
    sqlite3_clear_bindings (stmt);
    sqlite3_bind_text (stmt, 1, table, strlen (table), SQLITE_STATIC);
    sqlite3_bind_text (stmt, 2, column, strlen (column), SQLITE_STATIC);
    sqlite3_bind_int (stmt, 3, count);
    if (has_coords)
      {
	  sqlite3_bind_double (stmt, 4, min_x);
	  sqlite3_bind_double (stmt, 5, min_y);
	  sqlite3_bind_double (stmt, 6, max_x);
	  sqlite3_bind_double (stmt, 7, max_y);
      }
    else
      {
	  sqlite3_bind_null (stmt, 4);
	  sqlite3_bind_null (stmt, 5);
	  sqlite3_bind_null (stmt, 6);
	  sqlite3_bind_null (stmt, 7);
      }
    ret = sqlite3_step (stmt);
    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	;
    else
	error = 1;
    ret = sqlite3_finalize (stmt);
    if (ret != SQLITE_OK)
	return 0;
    if (error)
	return 0;
    return 1;
}

#define SPATIALITE_STATISTICS_GENUINE	1
#define SPATIALITE_STATISTICS_VIEWS	2
#define SPATIALITE_STATISTICS_VIRTS	3

static int
do_compute_layer_statistics (sqlite3 * sqlite, const char *table,
			     const char *column, int stat_type)
{
/* computes LAYER_STATISTICS [single table/geometry] */
    char xtable[1024];
    char xgeom[1024];
    char sql[8192];
    char sql2[2048];
    int ret;
    int error = 0;
    int count;
    double min_x;
    double min_y;
    double max_x;
    double max_y;
    int has_coords = 1;
    sqlite3_stmt *stmt;

    strcpy (xtable, table);
    double_quoted_sql (xtable);
    strcpy (xgeom, column);
    double_quoted_sql (xgeom);
    strcpy (sql, "SELECT Count(*), ");
    sprintf (sql2, "Min(MbrMinX(%s)), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "Min(MbrMinY(%s)), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "Max(MbrMaxX(%s)), ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "Max(MbrMaxY(%s)) ", xgeom);
    strcat (sql, sql2);
    sprintf (sql2, "FROM %s", xtable);
    strcat (sql, sql2);

/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	return 0;
    while (1)
      {
	  /* scrolling the result set rows */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		count = sqlite3_column_int (stmt, 0);
		if (sqlite3_column_type (stmt, 1) == SQLITE_NULL)
		    has_coords = 0;
		else
		    min_x = sqlite3_column_double (stmt, 1);
		if (sqlite3_column_type (stmt, 2) == SQLITE_NULL)
		    has_coords = 0;
		else
		    min_y = sqlite3_column_double (stmt, 2);
		if (sqlite3_column_type (stmt, 3) == SQLITE_NULL)
		    has_coords = 0;
		else
		    max_x = sqlite3_column_double (stmt, 3);
		if (sqlite3_column_type (stmt, 4) == SQLITE_NULL)
		    has_coords = 0;
		else
		    max_y = sqlite3_column_double (stmt, 4);
		switch (stat_type)
		  {
		  case SPATIALITE_STATISTICS_GENUINE:
		      if (!do_update_layer_statistics
			  (sqlite, table, column, count, has_coords, min_x,
			   min_y, max_x, max_y))
			  error = 1;
		      break;
		  case SPATIALITE_STATISTICS_VIEWS:
		      if (!do_update_views_layer_statistics
			  (sqlite, table, column, count, has_coords, min_x,
			   min_y, max_x, max_y))
			  error = 1;
		      break;
		  case SPATIALITE_STATISTICS_VIRTS:
		      if (!do_update_virts_layer_statistics
			  (sqlite, table, column, count, has_coords, min_x,
			   min_y, max_x, max_y))
			  error = 1;
		      break;
		  };
	    }
	  else
	      error = 1;
      }
    ret = sqlite3_finalize (stmt);
    if (ret != SQLITE_OK)
	return 0;
    if (error)
	return 0;
    return 1;
}

static int
genuine_layer_statistics (sqlite3 * sqlite, const char *table,
			  const char *column)
{
/* updating genuine LAYER_STATISTICS metadata */
    int count = 0;
    char xtable[1024];
    char xgeom[1024];
    char sql[8192];
    char sql2[2048];
    int ret;
    const char *f_table_name;
    const char *f_geometry_column;
    int i;
    char **results;
    int rows;
    int columns;
    int error = 0;

    if (table == NULL && column == NULL)
      {
	  /* processing any table/geometry found in GEOMETRY_COLUMNS */
	  strcpy (sql, "SELECT f_table_name, f_geometry_column ");
	  strcat (sql, "FROM geometry_columns");
      }
    else if (column == NULL)
      {
	  /* processing any geometry belonging to this table */
	  strcpy (xtable, table);
	  clean_sql_string (xtable);
	  strcpy (sql, "SELECT f_table_name, f_geometry_column ");
	  strcat (sql, "FROM geometry_columns ");
	  sprintf (sql2, "WHERE f_table_name LIKE '%s'", xtable);
	  strcat (sql, sql2);
      }
    else
      {
	  /* processing a single table/geometry entry */
	  strcpy (xtable, table);
	  clean_sql_string (xtable);
	  strcpy (xgeom, column);
	  clean_sql_string (xgeom);
	  strcpy (sql, "SELECT f_table_name, f_geometry_column ");
	  strcat (sql, "FROM geometry_columns ");
	  sprintf (sql2, "WHERE f_table_name LIKE '%s' ", xtable);
	  strcat (sql, sql2);
	  sprintf (sql2, "AND f_geometry_column LIKE '%s'", xgeom);
	  strcat (sql2, sql);
      }
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	return -1;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		f_table_name = results[(i * columns) + 0];
		f_geometry_column = results[(i * columns) + 1];
		if (!do_compute_layer_statistics
		    (sqlite, f_table_name, f_geometry_column,
		     SPATIALITE_STATISTICS_GENUINE))
		  {
		      error = 1;
		      break;
		  }
		else
		    count++;
	    }
      }
    sqlite3_free_table (results);
    if (error)
	return -1;
    return count;
}

static int
views_layer_statistics (sqlite3 * sqlite, const char *table, const char *column)
{
/* updating VIEWS_LAYER_STATISTICS metadata */
    int count = 0;
    char xtable[1024];
    char xgeom[1024];
    char sql[8192];
    char sql2[2048];
    int ret;
    const char *view_name;
    const char *view_geometry;
    int i;
    char **results;
    int rows;
    int columns;
    int error = 0;

    if (table == NULL && column == NULL)
      {
	  /* processing any table/geometry found in VIEWS_GEOMETRY_COLUMNS */
	  strcpy (sql, "SELECT view_name, view_geometry ");
	  strcat (sql, "FROM views_geometry_columns");
      }
    else if (column == NULL)
      {
	  /* processing any geometry belonging to this table */
	  strcpy (xtable, table);
	  clean_sql_string (xtable);
	  strcpy (sql, "SELECT view_name, view_geometry ");
	  strcat (sql, "FROM views_geometry_columns ");
	  sprintf (sql2, "WHERE view_name LIKE '%s'", xtable);
	  strcat (sql, sql2);
      }
    else
      {
	  /* processing a single table/geometry entry */
	  strcpy (xtable, table);
	  clean_sql_string (xtable);
	  strcpy (xgeom, column);
	  clean_sql_string (xgeom);
	  strcpy (sql, "SELECT view_name, view_geometry ");
	  strcat (sql, "FROM views_geometry_columns ");
	  sprintf (sql2, "WHERE view_name LIKE '%s' ", xtable);
	  strcat (sql, sql2);
	  sprintf (sql2, "AND view_geometry LIKE '%s'", xgeom);
	  strcat (sql2, sql);
      }
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	return -1;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		view_name = results[(i * columns) + 0];
		view_geometry = results[(i * columns) + 1];
		if (!do_compute_layer_statistics
		    (sqlite, view_name, view_geometry,
		     SPATIALITE_STATISTICS_VIEWS))
		  {
		      error = 1;
		      break;
		  }
		else
		    count++;
	    }
      }
    sqlite3_free_table (results);
    if (error)
	return -1;
    return count;
}

static int
virts_layer_statistics (sqlite3 * sqlite, const char *table, const char *column)
{
/* updating VIRTS_LAYER_STATISTICS metadata */
    int count = 0;
    char xtable[1024];
    char xgeom[1024];
    char sql[8192];
    char sql2[2048];
    int ret;
    const char *f_table_name;
    const char *f_geometry_column;
    int i;
    char **results;
    int rows;
    int columns;
    int error = 0;

    if (table == NULL && column == NULL)
      {
	  /* processing any table/geometry found in GEOMETRY_COLUMNS */
	  strcpy (sql, "SELECT virt_name, virt_geometry ");
	  strcat (sql, "FROM virts_geometry_columns");
      }
    else if (column == NULL)
      {
	  /* processing any geometry belonging to this table */
	  strcpy (xtable, table);
	  clean_sql_string (xtable);
	  strcpy (sql, "SELECT virt_name, virt_geometry ");
	  strcat (sql, "FROM virts_geometry_columns ");
	  sprintf (sql2, "WHERE virt_name LIKE '%s'", xtable);
	  strcat (sql, sql2);
      }
    else
      {
	  /* processing a single table/geometry entry */
	  strcpy (xtable, table);
	  clean_sql_string (xtable);
	  strcpy (xgeom, column);
	  clean_sql_string (xgeom);
	  strcpy (sql, "SELECT virt_name, virt_geometry ");
	  strcat (sql, "FROM virts_geometry_columns ");
	  sprintf (sql2, "WHERE virt_name LIKE '%s' ", xtable);
	  strcat (sql, sql2);
	  sprintf (sql2, "AND virt_geometry LIKE '%s'", xgeom);
	  strcat (sql2, sql);
      }
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	return -1;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		f_table_name = results[(i * columns) + 0];
		f_geometry_column = results[(i * columns) + 1];
		if (!do_compute_layer_statistics
		    (sqlite, f_table_name, f_geometry_column,
		     SPATIALITE_STATISTICS_VIRTS))
		  {
		      error = 1;
		      break;
		  }
		else
		    count++;
	    }
      }
    sqlite3_free_table (results);
    if (error)
	return -1;
    return count;
}

SPATIALITE_DECLARE int
update_layer_statistics (sqlite3 * sqlite, const char *table,
			 const char *column)
{
/* updating LAYER_STATISTICS metadata [main] */
    int ret;
    int count = 0;
    ret = genuine_layer_statistics (sqlite, table, column);
    if (ret < 0)
	return 0;
    count += ret;
    ret = views_layer_statistics (sqlite, table, column);
    if (ret < 0)
	return 0;
    count += ret;
    ret = virts_layer_statistics (sqlite, table, column);
    if (ret < 0)
	return 0;
    count += ret;
    return count;
}
/**************** End file: spatialite.c **********/


/**************** Begin file: mbrcache.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <float.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiageo.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

#if defined(_WIN32) && !defined(__MINGW32__)
#define LONG64_MAX	_I64_MAX
#define LONG64_MIN	_I64_MIN
#else
#define LONG64_MAX	9223372036854775807LL
#define LONG64_MIN	(-LONG64_MAX + 1)
#endif

static struct sqlite3_module my_mbr_module;

/*

memory structs used to store the MBR's cache

the basic idea is to implement a hierarchy in order to avoid
excessive memory fragmentation and achieve better performance

- the cache is a linked-list of cache page elements
  - each cache page contains an array of 32 cache blocks
    - each cache block contains an array of 32 cache cells
so a single cache page con store up to 1024 cache cells

*/

struct mbr_cache_cell
{
/* 
a  cached entity 
*/

/* the entity's ROWID */
    sqlite3_int64 rowid;
/* the MBR */
    double minx;
    double miny;
    double maxx;
    double maxy;
};

struct mbr_cache_block
{
/*
a block of 32 cached entities
*/

/* 
allocation bitmap: the meaning of each bit is:
1 - corresponding cache cell is in use
0 - corresponding cache cell is unused
*/
    unsigned int bitmap;
/* 
the MBR corresponding to this cache block 
i.e. the combined MBR for any contained cell
*/
    double minx;
    double miny;
    double maxx;
    double maxy;
/* the cache cells array */
    struct mbr_cache_cell cells[32];
};

struct mbr_cache_page
{
/*
a page containing 32 cached blocks
*/

/* 
allocation bitmap: the meaning of each bit is:
1 - corresponding cache block is in full
0 - corresponding cache block is not full
*/
    unsigned int bitmap;
/* 
the MBR corresponding to this cache page
i.e. the combined MBR for any contained block
*/
    double minx;
    double miny;
    double maxx;
    double maxy;
/* the cache blocks array */
    struct mbr_cache_block blocks[32];
/* the min-max rowid for this page */
    sqlite3_int64 min_rowid;
    sqlite3_int64 max_rowid;
/* pointer to next element into the cached pages linked list */
    struct mbr_cache_page *next;
};

struct mbr_cache
{
/*
the MBR's cache
implemented as a cache pages linked list
*/

/* pointers used to handle the cache pages linked list */
    struct mbr_cache_page *first;
    struct mbr_cache_page *last;
/*
 pointer used to identify the current cache page when inserting a new cache cell
 */
    struct mbr_cache_page *current;
};

typedef struct MbrCacheStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
    struct mbr_cache *cache;	/* the  MBR's cache */
    char *table_name;		/* the main table to be cached */
    char *column_name;		/* the column to be cached */
    int error;			/* some previous error disables any operation */
} MbrCache;
typedef MbrCache *MbrCachePtr;

typedef struct MbrCacheCursortStruct
{
/* extends the sqlite3_vtab_cursor struct */
    MbrCachePtr pVtab;		/* Virtual table of this cursor */
    int eof;			/* the EOF marker */
/* 
positioning parameters while performing a cache search 
*/
    struct mbr_cache_page *current_page;
    int current_block_index;
    int current_cell_index;
    struct mbr_cache_cell *current_cell;
/* 
the strategy to use:
    0 = sequential scan
    1 = find rowid
    2 = spatial search
*/
    int strategy;
/* the MBR to search for */
    double minx;
    double miny;
    double maxx;
    double maxy;
/*
the MBR search mode:
    0 = WITHIN
    1 = CONTAIN
*/
    int mbr_mode;
} MbrCacheCursor;
typedef MbrCacheCursor *MbrCacheCursorPtr;

static unsigned int
cache_bitmask (int x)
{
/* return the bitmask corresponding to index X */
    switch (x)
      {
      case 0:
	  return 0x80000000;
      case 1:
	  return 0x40000000;
      case 2:
	  return 0x20000000;
      case 3:
	  return 0x10000000;
      case 4:
	  return 0x08000000;
      case 5:
	  return 0x04000000;
      case 6:
	  return 0x02000000;
      case 7:
	  return 0x01000000;
      case 8:
	  return 0x00800000;
      case 9:
	  return 0x00400000;
      case 10:
	  return 0x00200000;
      case 11:
	  return 0x00100000;
      case 12:
	  return 0x00080000;
      case 13:
	  return 0x00040000;
      case 14:
	  return 0x00020000;
      case 15:
	  return 0x00010000;
      case 16:
	  return 0x00008000;
      case 17:
	  return 0x00004000;
      case 18:
	  return 0x00002000;
      case 19:
	  return 0x00001000;
      case 20:
	  return 0x00000800;
      case 21:
	  return 0x00000400;
      case 22:
	  return 0x00000200;
      case 23:
	  return 0x00000100;
      case 24:
	  return 0x00000080;
      case 25:
	  return 0x00000040;
      case 26:
	  return 0x00000020;
      case 27:
	  return 0x00000010;
      case 28:
	  return 0x00000008;
      case 29:
	  return 0x00000004;
      case 30:
	  return 0x00000002;
      case 31:
	  return 0x00000001;
      };
    return 0x00000000;
}

static void
mbrc_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

static struct mbr_cache *
cache_alloc (void)
{
/* allocates and initializes an empty cache struct */
    struct mbr_cache *p = malloc (sizeof (struct mbr_cache));
    p->first = NULL;
    p->last = NULL;
    p->current = NULL;
    return p;
}

static struct mbr_cache_page *
cache_page_alloc (void)
{
/* allocates and initializes a cache page */
    int i;
    struct mbr_cache_block *pb;
    struct mbr_cache_page *p = malloc (sizeof (struct mbr_cache_page));
    p->bitmap = 0x00000000;
    p->next = NULL;
    p->minx = DBL_MAX;
    p->miny = DBL_MAX;
    p->maxx = -DBL_MAX;
    p->maxy = -DBL_MAX;
    for (i = 0; i < 32; i++)
      {
	  pb = p->blocks + i;
	  pb->bitmap = 0x00000000;
	  pb->minx = DBL_MAX;
	  pb->miny = DBL_MAX;
	  pb->maxx = -DBL_MAX;
	  pb->maxy = DBL_MAX;
      }
    p->max_rowid = LONG64_MIN;
    p->min_rowid = LONG64_MAX;
    return p;
}

static void
cache_destroy (struct mbr_cache *p)
{
/* memory cleanup; destroying a cache and any page into the cache */
    struct mbr_cache_page *pp;
    struct mbr_cache_page *ppn;
    if (!p)
	return;
    pp = p->first;
    while (pp)
      {
	  ppn = pp->next;
	  free (pp);
	  pp = ppn;
      }
    free (p);
}

static int
cache_get_free_block (struct mbr_cache_page *pp)
{
/* scans a cache page, returning the index of the first available block containing a free cell */
    int ib;
    for (ib = 0; ib < 32; ib++)
      {
	  if ((pp->bitmap & cache_bitmask (ib)) == 0x00000000)
	      return ib;
      }
    return -1;
}

static void
cache_fix_page_bitmap (struct mbr_cache_page *pp)
{
/* updating the cache page bitmap */
    int ib;
    for (ib = 0; ib < 32; ib++)
      {
	  if (pp->blocks[ib].bitmap == 0xffffffff)
	    {
		/* all the cells into this block are used; marking the page bitmap */
		pp->bitmap |= cache_bitmask (ib);
	    }
      }
}

static int
cache_get_free_cell (struct mbr_cache_block *pb)
{
/* scans a cache block, returning the index of the first free cell */
    int ic;
    for (ic = 0; ic < 32; ic++)
      {
	  if ((pb->bitmap & cache_bitmask (ic)) == 0x00000000)
	      return ic;
      }
    return -1;
}

static struct mbr_cache_page *
cache_get_free_page (struct mbr_cache *p)
{
/* return a pointer to the first cache page containing a free cell */
    struct mbr_cache_page *pp;
    if (!(p->first))
      {
	  /* the cache is empty; so we surely need to allocate the first page */
	  pp = cache_page_alloc ();
	  p->first = pp;
	  p->last = pp;
	  p->current = pp;
	  return pp;
      }
    if (p->current)
      {
	  /* checking if there is at least a free block into the current page */
	  if (p->current->bitmap != 0xffffffff)
	      return p->current;
      }
    pp = p->first;
    while (pp)
      {
	  /* scanning the page list in order to discover if there is an existing page not yet completely filled */
	  if (pp->bitmap != 0xffffffff)
	    {
		p->current = pp;
		return pp;
	    }
	  pp = pp->next;
      }
/* we have to allocate a new page */
    pp = cache_page_alloc ();
    p->last->next = pp;
    p->last = pp;
    p->current = pp;
    return pp;
}

static void
cache_insert_cell (struct mbr_cache *p, sqlite3_int64 rowid, double minx,
		   double miny, double maxx, double maxy)
{
/* inserting a new cell */
    struct mbr_cache_page *pp = cache_get_free_page (p);
    int ib = cache_get_free_block (pp);
    struct mbr_cache_block *pb = pp->blocks + ib;
    int ic = cache_get_free_cell (pb);
    struct mbr_cache_cell *pc = pb->cells + ic;
    pc->rowid = rowid;
    pc->minx = minx;
    pc->miny = miny;
    pc->maxx = maxx;
    pc->maxy = maxy;
/* marking the cache cell as used into the block bitmap */
    pb->bitmap |= cache_bitmask (ic);
/* updating the cache block MBR */
    if (pb->minx > minx)
	pb->minx = minx;
    if (pb->maxx < maxx)
	pb->maxx = maxx;
    if (pb->miny > miny)
	pb->miny = miny;
    if (pb->maxy < maxy)
	pb->maxy = maxy;
/* updating the cache page MBR */
    if (pp->minx > minx)
	pp->minx = minx;
    if (pp->maxx < maxx)
	pp->maxx = maxx;
    if (pp->miny > miny)
	pp->miny = miny;
    if (pp->maxy < maxy)
	pp->maxy = maxy;
/* fixing the cache page bitmap */
    cache_fix_page_bitmap (pp);
/* updating min-max rowid into the cache page */
    if (pp->min_rowid > rowid)
	pp->min_rowid = rowid;
    if (pp->max_rowid < rowid)
	pp->max_rowid = rowid;
}

static struct mbr_cache *
cache_load (sqlite3 * handle, const char *table, const char *column)
{
/* 
initial loading the MBR cache
retrieving any existing entity from the main table 
*/
    sqlite3_stmt *stmt;
    int ret;
    char sql[256];
    sqlite3_int64 rowid;
    double minx;
    double maxx;
    double miny;
    double maxy;
    int v1;
    int v2;
    int v3;
    int v4;
    int v5;
    struct mbr_cache *p_cache;
    char xcolumn[1024];
    char xtable[1024];
    strcpy (xcolumn, column);
    mbrc_double_quoted_sql (xcolumn);
    strcpy (xtable, table);
    mbrc_double_quoted_sql (xtable);
    sprintf (sql,
	     "SELECT ROWID, MbrMinX(%s), MbrMinY(%s), MbrMaxX(%s), MbrMaxY(%s) FROM %s",
	     xcolumn, xcolumn, xcolumn, xcolumn, xtable);
    ret = sqlite3_prepare_v2 (handle, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
/* some error occurred */
	  fprintf (stderr, "cache SQL error: %s\n", sqlite3_errmsg (handle));
	  return NULL;
      }
    p_cache = cache_alloc ();
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		v1 = 0;
		v2 = 0;
		v3 = 0;
		v4 = 0;
		v5 = 0;
		if (sqlite3_column_type (stmt, 0) == SQLITE_INTEGER)
		    v1 = 1;
		if (sqlite3_column_type (stmt, 1) == SQLITE_FLOAT)
		    v2 = 1;
		if (sqlite3_column_type (stmt, 1) == SQLITE_FLOAT)
		    v3 = 1;
		if (sqlite3_column_type (stmt, 1) == SQLITE_FLOAT)
		    v4 = 1;
		if (sqlite3_column_type (stmt, 1) == SQLITE_FLOAT)
		    v5 = 1;
		if (v1 && v2 && v3 && v4 && v5)
		  {
		      /* ok, this entity is a valid one; inserting them into the MBR's cache */
		      rowid = sqlite3_column_int (stmt, 0);
		      minx = sqlite3_column_double (stmt, 1);
		      miny = sqlite3_column_double (stmt, 2);
		      maxx = sqlite3_column_double (stmt, 3);
		      maxy = sqlite3_column_double (stmt, 4);
		      cache_insert_cell (p_cache, rowid, minx, miny, maxx,
					 maxy);
		  }
	    }
	  else
	    {
/* some unexpected error occurred */
		printf ("sqlite3_step() error: %s\n", sqlite3_errmsg (handle));
		sqlite3_finalize (stmt);
		cache_destroy (p_cache);
		return NULL;
	    }
      }
/* we have now to finalize the query [memory cleanup] */
    sqlite3_finalize (stmt);
    return p_cache;
}

static int
cache_find_next_cell (struct mbr_cache_page **page, int *i_block, int *i_cell,
		      struct mbr_cache_cell **cell)
{
/* finding next cached cell */
    struct mbr_cache_page *pp = *page;
    struct mbr_cache_block *pb;
    struct mbr_cache_cell *pc;
    int ib;
    int ic;
    int sib = *i_block;
    int sic = *i_cell;
    while (pp)
      {
	  for (ib = sib; ib < 32; ib++)
	    {
		pb = pp->blocks + ib;
		for (ic = sic; ic < 32; ic++)
		  {
		      if ((pb->bitmap & cache_bitmask (ic)) == 0x00000000)
			  continue;
		      pc = pb->cells + ic;
		      if (pc == *cell)
			{
			    /* this one is the current cell */
			    continue;
			}
		      /* next cell found */
		      *page = pp;
		      *i_block = ib;
		      *i_cell = ic;
		      *cell = pc;
		      return 1;
		  }
		sic = 0;
	    }
	  sib = 0;
	  pp = pp->next;
      }
    return 0;
}

static int
cache_find_next_mbr (struct mbr_cache_page **page, int *i_block, int *i_cell,
		     struct mbr_cache_cell **cell, double minx, double miny,
		     double maxx, double maxy, int mode)
{
/* finding next cached cell */
    struct mbr_cache_page *pp = *page;
    struct mbr_cache_block *pb;
    struct mbr_cache_cell *pc;
    int ib;
    int ic;
    int sib = *i_block;
    int sic = *i_cell;
    int ok_mbr;
    while (pp)
      {
	  ok_mbr = 0;
	  if (pp->maxx >= minx && pp->minx <= maxx && pp->maxy >= miny
	      && pp->miny <= maxy)
	      ok_mbr = 1;
	  if (ok_mbr)
	    {
		for (ib = sib; ib < 32; ib++)
		  {
		      pb = pp->blocks + ib;
		      ok_mbr = 0;
		      if (pb->maxx >= minx && pb->minx <= maxx
			  && pb->maxy >= miny && pb->miny <= maxy)
			  ok_mbr = 1;
		      if (ok_mbr)
			{
			    for (ic = sic; ic < 32; ic++)
			      {
				  if ((pb->bitmap & cache_bitmask (ic)) ==
				      0x00000000)
				      continue;
				  pc = pb->cells + ic;
				  ok_mbr = 0;
				  if (mode == GAIA_FILTER_MBR_INTERSECTS)
				    {
					/* MBR INTERSECTS */
					if (pc->maxx >= minx && pc->minx <= maxx
					    && pc->maxy >= miny
					    && pc->miny <= maxy)
					    ok_mbr = 1;
				    }
				  else if (mode == GAIA_FILTER_MBR_CONTAINS)
				    {
					/* MBR CONTAINS */
					if (minx >= pc->minx && maxx <= pc->maxx
					    && miny >= pc->miny
					    && maxy <= pc->maxy)
					    ok_mbr = 1;
				    }
				  else
				    {
					/* MBR WITHIN */
					if (pc->minx >= minx && pc->maxx <= maxx
					    && pc->miny >= miny
					    && pc->maxy <= maxy)
					    ok_mbr = 1;
				    }
				  if (ok_mbr)
				    {
					if (pc == *cell)
					  {
					      /* this one is the current cell */
					      continue;
					  }
					/* next cell found */
					*page = pp;
					*i_block = ib;
					*i_cell = ic;
					*cell = pc;
					return 1;
				    }
			      }
			}
		      sic = 0;
		  }
	    }
	  sib = 0;
	  pp = pp->next;
      }
    return 0;
}

static struct mbr_cache_cell *
cache_find_by_rowid (struct mbr_cache_page *pp, sqlite3_int64 rowid)
{
/* trying to find a row by rowid from the Mbr cache */
    struct mbr_cache_block *pb;
    struct mbr_cache_cell *pc;
    int ib;
    int ic;
    while (pp)
      {
	  if (rowid >= pp->min_rowid && rowid <= pp->max_rowid)
	    {
		for (ib = 0; ib < 32; ib++)
		  {
		      pb = pp->blocks + ib;
		      for (ic = 0; ic < 32; ic++)
			{
			    if ((pb->bitmap & cache_bitmask (ic)) == 0x00000000)
				continue;
			    pc = pb->cells + ic;
			    if (pc->rowid == rowid)
				return pc;
			}
		  }
	    }
	  pp = pp->next;
      }
    return 0;
}

static void
cache_update_page (struct mbr_cache_page *pp, int i_block)
{
/* updating the cache block and cache page MBR after a DELETE or UPDATE occurred */
    struct mbr_cache_block *pb;
    struct mbr_cache_cell *pc;
    int ib;
    int ic;
/* updating the cache block MBR */
    pb = pp->blocks + i_block;
    pb->minx = DBL_MAX;
    pb->miny = DBL_MAX;
    pb->maxx = -DBL_MAX;
    pb->maxy = -DBL_MAX;
    for (ic = 0; ic < 32; ic++)
      {
	  if ((pb->bitmap & cache_bitmask (ic)) == 0x00000000)
	      continue;
	  pc = pb->cells + ic;
	  if (pb->minx > pc->minx)
	      pb->minx = pc->minx;
	  if (pb->miny > pc->miny)
	      pb->miny = pc->miny;
	  if (pb->maxx < pc->maxx)
	      pb->maxx = pc->maxx;
	  if (pb->maxy < pc->maxy)
	      pb->maxy = pc->maxy;
      }
/* updating the cache page MBR */
    pp->minx = DBL_MAX;
    pp->miny = DBL_MAX;
    pp->maxx = -DBL_MAX;
    pp->maxy = -DBL_MAX;
    pp->min_rowid = LONG64_MAX;
    pp->max_rowid = LONG64_MIN;
    for (ib = 0; ib < 32; ib++)
      {
	  pb = pp->blocks + ib;
	  for (ic = 0; ic < 32; ic++)
	    {
		if ((pb->bitmap & cache_bitmask (ic)) == 0x00000000)
		    continue;
		pc = pb->cells + ic;
		if (pp->minx > pc->minx)
		    pp->minx = pc->minx;
		if (pp->miny > pc->miny)
		    pp->miny = pc->miny;
		if (pp->maxx < pc->maxx)
		    pp->maxx = pc->maxx;
		if (pp->maxy < pc->maxy)
		    pp->maxy = pc->maxy;
		if (pp->min_rowid > pc->rowid)
		    pp->min_rowid = pc->rowid;
		if (pp->max_rowid < pc->rowid)
		    pp->max_rowid = pc->rowid;
	    }
      }
}

static int
cache_delete_cell (struct mbr_cache_page *pp, sqlite3_int64 rowid)
{
/* trying to delete a row identified by rowid from the Mbr cache */
    struct mbr_cache_block *pb;
    struct mbr_cache_cell *pc;
    int ib;
    int ic;
    while (pp)
      {
	  if (rowid >= pp->min_rowid && rowid <= pp->max_rowid)
	    {
		for (ib = 0; ib < 32; ib++)
		  {
		      pb = pp->blocks + ib;
		      for (ic = 0; ic < 32; ic++)
			{
			    if ((pb->bitmap & cache_bitmask (ic)) == 0x00000000)
				continue;
			    pc = pb->cells + ic;
			    if (pc->rowid == rowid)
			      {
				  /* marking the cell as free */
				  pb->bitmap &= ~(cache_bitmask (ic));
				  /* marking the block as not full */
				  pp->bitmap &= ~(cache_bitmask (ib));
				  /* updating the cache block and cache page MBR */
				  cache_update_page (pp, ib);
				  return 1;
			      }
			}
		  }
	    }
	  pp = pp->next;
      }
    return 0;
}

static int
cache_update_cell (struct mbr_cache_page *pp, sqlite3_int64 rowid, double minx,
		   double miny, double maxx, double maxy)
{
/* trying to update a row identified by rowid from the Mbr cache */
    struct mbr_cache_block *pb;
    struct mbr_cache_cell *pc;
    int ib;
    int ic;
    while (pp)
      {
	  if (rowid >= pp->min_rowid && rowid <= pp->max_rowid)
	    {
		for (ib = 0; ib < 32; ib++)
		  {
		      pb = pp->blocks + ib;
		      for (ic = 0; ic < 32; ic++)
			{
			    if ((pb->bitmap & cache_bitmask (ic)) == 0x00000000)
				continue;
			    pc = pb->cells + ic;
			    if (pc->rowid == rowid)
			      {
				  /* updating the cell MBR */
				  pc->minx = minx;
				  pc->miny = miny;
				  pc->maxx = maxx;
				  pc->maxy = maxy;
				  /* updating the cache block and cache page MBR */
				  cache_update_page (pp, ib);
				  return 1;
			      }
			}
		  }
	    }
	  pp = pp->next;
      }
    return 0;
}

static int
mbrc_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	     sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table and caches related Geometry column */
    int err;
    int ret;
    int i;
    int len;
    int n_rows;
    int n_columns;
    const char *vtable;
    const char *table;
    const char *column;
    const char *col_name;
    char **results;
    char *err_msg = NULL;
    char sql[4096];
    int ok_col;
    MbrCachePtr p_vt;
    char xname[1024];
    char x_vtable[1024];
    char x_table[1024];
    char x_column[1024];
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
    p_vt = (MbrCachePtr) sqlite3_malloc (sizeof (MbrCache));
    if (!p_vt)
	return SQLITE_NOMEM;
    *ppVTab = (sqlite3_vtab *) p_vt;
    p_vt->pModule = &my_mbr_module;
    p_vt->nRef = 0;
    p_vt->zErrMsg = NULL;
    p_vt->db = db;
    p_vt->table_name = NULL;
    p_vt->column_name = NULL;
    p_vt->cache = NULL;
/* checking for table_name and geo_column_name */
    if (argc == 5)
      {
	  vtable = argv[2];
	  len = strlen (vtable);
	  if ((*(vtable + 0) == '\'' || *(vtable + 0) == '"')
	      && (*(vtable + len - 1) == '\'' || *(vtable + len - 1) == '"'))
	    {
/* the VirtualTableName is enclosed between quotes - we need to dequote it */
		strcpy (x_vtable, vtable + 1);
		len = strlen (x_vtable);
		*(x_vtable + len - 1) = '\0';
		vtable = x_vtable;
	    }
	  table = argv[3];
	  len = strlen (table);
	  if ((*(table + 0) == '\'' || *(table + 0) == '"')
	      && (*(table + len - 1) == '\'' || *(table + len - 1) == '"'))
	    {
/* the MainTableName is enclosed between quotes - we need to dequote it */
		strcpy (x_table, table + 1);
		len = strlen (x_table);
		*(x_table + len - 1) = '\0';
		table = x_table;
	    }
	  column = argv[4];
	  len = strlen (column);
	  if ((*(column + 0) == '\'' || *(column + 0) == '"')
	      && (*(column + len - 1) == '\'' || *(column + len - 1) == '"'))
	    {
/* the GeometryColumnName is enclosed between quotes - we need to dequote it */
		strcpy (x_column, column + 1);
		len = strlen (x_column);
		*(x_column + len - 1) = '\0';
		column = x_column;
	    }
	  len = strlen (table);
	  p_vt->table_name = sqlite3_malloc (len + 1);
	  strcpy (p_vt->table_name, table);
	  len = strlen (column);
	  p_vt->column_name = sqlite3_malloc (len + 1);
	  strcpy (p_vt->column_name, column);
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[MbrCache module] CREATE VIRTUAL: illegal arg list {table_name, geo_column_name}");
	  return SQLITE_ERROR;
      }
/* retrieving the base table columns */
    err = 0;
    ok_col = 0;
    strcpy (xname, table);
    mbrc_double_quoted_sql (xname);
    sprintf (sql, "PRAGMA table_info(%s)", xname);
    ret = sqlite3_get_table (db, sql, &results, &n_rows, &n_columns, &err_msg);
    if (ret != SQLITE_OK)
      {
	  err = 1;
	  goto illegal;
      }
    if (n_rows > 1)
      {
	  for (i = 1; i <= n_rows; i++)
	    {
		col_name = results[(i * n_columns) + 1];
		if (strcasecmp (col_name, column) == 0)
		    ok_col = 1;
	    }
	  sqlite3_free_table (results);
	  if (!ok_col)
	      err = 1;
      }
    else
	err = 1;
  illegal:
    if (err)
      {
	  /* something is going the wrong way; creating a stupid default table */
	  strcpy (xname, vtable);
	  mbrc_double_quoted_sql (xname);
	  sprintf (sql, "CREATE TABLE %s (rowid INTEGER, mbr BLOB)", xname);
	  if (sqlite3_declare_vtab (db, sql) != SQLITE_OK)
	    {
		*pzErr =
		    sqlite3_mprintf
		    ("[MbrCache module] cannot build the VirtualTable\n");
		return SQLITE_ERROR;
	    }
	  p_vt->error = 1;
	  *ppVTab = (sqlite3_vtab *) p_vt;
	  return SQLITE_OK;
      }
    p_vt->error = 0;
    strcpy (xname, vtable);
    mbrc_double_quoted_sql (xname);
    sprintf (sql, "CREATE TABLE %s (", xname);
    strcat (sql, "rowid INTEGER, mbr BLOB)");
    if (sqlite3_declare_vtab (db, sql) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[MbrCache module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       sql);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    return SQLITE_OK;
}

static int
mbrc_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	      sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table - simply aliases mbrc_create() */
    return mbrc_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
mbrc_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIdxInfo)
{
/* best index selection */
    int i;
    int err = 1;
    int errors = 0;
    int mbr = 0;
    int rowid = 0;
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    for (i = 0; i < pIdxInfo->nConstraint; i++)
      {
	  /* verifying the constraints */
	  struct sqlite3_index_constraint *p = &(pIdxInfo->aConstraint[i]);
	  if (p->usable)
	    {
		if (p->iColumn == 0 && p->op == SQLITE_INDEX_CONSTRAINT_EQ)
		    rowid++;
		else if (p->iColumn == 1 && p->op == SQLITE_INDEX_CONSTRAINT_EQ)
		    mbr++;
		else
		    errors++;
	    }
      }
    if (mbr == 1 && rowid == 0 && errors == 0)
      {
	  /* this one is a valid spatially-filtered query */
	  pIdxInfo->idxNum = 2;
	  for (i = 0; i < pIdxInfo->nConstraint; i++)
	    {
		pIdxInfo->aConstraintUsage[i].argvIndex = 1;
		pIdxInfo->aConstraintUsage[i].omit = 1;
	    }
	  err = 0;
      }
    if (mbr == 0 && rowid == 1 && errors == 0)
      {
	  /* this one is a valid rowid-filtered query */
	  pIdxInfo->idxNum = 1;
	  pIdxInfo->estimatedCost = 1.0;
	  for (i = 0; i < pIdxInfo->nConstraint; i++)
	    {
		if (pIdxInfo->aConstraint[i].usable)
		  {
		      pIdxInfo->aConstraintUsage[i].argvIndex = 1;
		      pIdxInfo->aConstraintUsage[i].omit = 1;
		  }
	    }
	  err = 0;
      }
    if (mbr == 0 && rowid == 0 && errors == 0)
      {
	  /* this one is a valid unfiltered query */
	  pIdxInfo->idxNum = 0;
	  err = 0;
      }
    if (err)
      {
	  /* illegal query */
	  pIdxInfo->idxNum = -1;
      }
    return SQLITE_OK;
}

static int
mbrc_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    MbrCachePtr p_vt = (MbrCachePtr) pVTab;
    if (p_vt->cache)
	cache_destroy (p_vt->cache);
    if (p_vt->table_name)
	sqlite3_free (p_vt->table_name);
    if (p_vt->column_name)
	sqlite3_free (p_vt->column_name);
    sqlite3_free (p_vt);
    return SQLITE_OK;
}

static int
mbrc_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases mbrc_disconnect() */
    return mbrc_disconnect (pVTab);
}

static void
mbrc_read_row_unfiltered (MbrCacheCursorPtr cursor)
{
/* trying to read the next row from the Mbr cache - unfiltered mode */
    struct mbr_cache_page *page = cursor->current_page;
    struct mbr_cache_cell *cell = cursor->current_cell;
    int i_block = cursor->current_block_index;
    int i_cell = cursor->current_cell_index;
    if (cache_find_next_cell (&page, &i_block, &i_cell, &cell))
      {
	  cursor->current_page = page;
	  cursor->current_block_index = i_block;
	  cursor->current_cell_index = i_cell;
	  cursor->current_cell = cell;
      }
    else
	cursor->eof = 1;
}

static void
mbrc_read_row_filtered (MbrCacheCursorPtr cursor)
{
/* trying to read the next row from the Mbr cache - spatially filter mode */
    struct mbr_cache_page *page = cursor->current_page;
    struct mbr_cache_cell *cell = cursor->current_cell;
    int i_block = cursor->current_block_index;
    int i_cell = cursor->current_cell_index;
    if (cache_find_next_mbr
	(&page, &i_block, &i_cell, &cell, cursor->minx, cursor->miny,
	 cursor->maxx, cursor->maxy, cursor->mbr_mode))
      {
	  cursor->current_page = page;
	  cursor->current_block_index = i_block;
	  cursor->current_cell_index = i_cell;
	  cursor->current_cell = cell;
      }
    else
	cursor->eof = 1;
}

static void
mbrc_read_row_by_rowid (MbrCacheCursorPtr cursor, sqlite3_int64 rowid)
{
/* trying to find a row by rowid from the Mbr cache */
    struct mbr_cache_cell *cell =
	cache_find_by_rowid (cursor->pVtab->cache->first, rowid);
    if (cell)
	cursor->current_cell = cell;
    else
      {
	  cursor->current_cell = NULL;
	  cursor->eof = 1;
      }
}

static int
mbrc_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    MbrCachePtr p_vt = (MbrCachePtr) pVTab;
    MbrCacheCursorPtr cursor =
	(MbrCacheCursorPtr) sqlite3_malloc (sizeof (MbrCacheCursor));
    if (cursor == NULL)
	return SQLITE_ERROR;
    cursor->pVtab = p_vt;
    if (p_vt->error)
      {
	  cursor->eof = 1;
	  *ppCursor = (sqlite3_vtab_cursor *) cursor;
	  return SQLITE_OK;
      }
    if (!(p_vt->cache))
	p_vt->cache =
	    cache_load (p_vt->db, p_vt->table_name, p_vt->column_name);
    cursor->current_page = cursor->pVtab->cache->first;
    cursor->current_block_index = 0;
    cursor->current_cell_index = 0;
    cursor->current_cell = NULL;
    cursor->eof = 0;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    return SQLITE_OK;
}

static int
mbrc_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    sqlite3_free (pCursor);
    return SQLITE_OK;
}

static int
mbrc_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	     int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    MbrCacheCursorPtr cursor = (MbrCacheCursorPtr) pCursor;
    if (idxStr || argc)
	idxStr = idxStr;	/* unused arg warning suppression */
    if (cursor->pVtab->error)
      {
	  cursor->eof = 1;
	  return SQLITE_OK;
      }
    cursor->current_page = cursor->pVtab->cache->first;
    cursor->current_block_index = 0;
    cursor->current_cell_index = 0;
    cursor->current_cell = NULL;
    cursor->eof = 0;
    cursor->strategy = idxNum;
    if (idxNum == 0)
      {
	  /* unfiltered mode */
	  mbrc_read_row_unfiltered (cursor);
	  return SQLITE_OK;
      }
    if (idxNum == 1)
      {
	  /* filtering by ROWID */
	  sqlite3_int64 rowid = sqlite3_value_int64 (argv[0]);
	  mbrc_read_row_by_rowid (cursor, rowid);
	  return SQLITE_OK;
      }
    if (idxNum == 2)
      {
	  /* filtering by MBR spatial relation */
	  unsigned char *p_blob;
	  int n_bytes;
	  double minx;
	  double miny;
	  double maxx;
	  double maxy;
	  int mode;
	  if (sqlite3_value_type (argv[0]) != SQLITE_BLOB)
	      cursor->eof = 1;
	  else
	    {
		p_blob = (unsigned char *) sqlite3_value_blob (argv[0]);
		n_bytes = sqlite3_value_bytes (argv[0]);
		if (gaiaParseFilterMbr
		    (p_blob, n_bytes, &minx, &miny, &maxx, &maxy, &mode))
		  {
		      if (mode == GAIA_FILTER_MBR_WITHIN
			  || mode == GAIA_FILTER_MBR_CONTAINS
			  || mode == GAIA_FILTER_MBR_INTERSECTS)
			{
			    cursor->minx = minx;
			    cursor->miny = miny;
			    cursor->maxx = maxx;
			    cursor->maxy = maxy;
			    cursor->mbr_mode = mode;
			    mbrc_read_row_filtered (cursor);
			}
		      else
			  cursor->eof = 1;
		  }
	    }
	  return SQLITE_OK;
      }
/* illegal query mode */
    cursor->eof = 1;
    return SQLITE_OK;
}

static int
mbrc_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching a next row from cursor */
    MbrCacheCursorPtr cursor = (MbrCacheCursorPtr) pCursor;
    if (cursor->pVtab->error)
      {
	  cursor->eof = 1;
	  return SQLITE_OK;
      }
    if (cursor->strategy == 0)
	mbrc_read_row_unfiltered (cursor);
    else if (cursor->strategy == 2)
	mbrc_read_row_filtered (cursor);
    else
	cursor->eof = 1;
    return SQLITE_OK;
}

static int
mbrc_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    MbrCacheCursorPtr cursor = (MbrCacheCursorPtr) pCursor;
    return cursor->eof;
}

static int
mbrc_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	     int column)
{
/* fetching value for the Nth column */
    MbrCacheCursorPtr cursor = (MbrCacheCursorPtr) pCursor;
    if (!(cursor->current_cell))
	sqlite3_result_null (pContext);
    else
      {
	  if (column == 0)
	    {
		/* the PRIMARY KEY column */
		sqlite3_result_int64 (pContext, cursor->current_cell->rowid);
	    }
	  if (column == 1)
	    {
		/* the MBR column */
		char envelope[1024];
		sprintf (envelope,
			 "POLYGON((%1.2f %1.2f, %1.2f %1.2f, %1.2f %1.2f, %1.2f %1.2f, %1.2f %1.2f))",
			 cursor->current_cell->minx, cursor->current_cell->miny,
			 cursor->current_cell->maxx, cursor->current_cell->miny,
			 cursor->current_cell->maxx, cursor->current_cell->maxy,
			 cursor->current_cell->minx, cursor->current_cell->maxy,
			 cursor->current_cell->minx,
			 cursor->current_cell->miny);
		sqlite3_result_text (pContext, envelope, strlen (envelope),
				     SQLITE_TRANSIENT);
	    }
      }
    return SQLITE_OK;
}

static int
mbrc_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    MbrCacheCursorPtr cursor = (MbrCacheCursorPtr) pCursor;
    *pRowid = cursor->current_cell->rowid;
    return SQLITE_OK;
}

static int
mbrc_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	     sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    sqlite3_int64 rowid;
    unsigned char *p_blob;
    int n_bytes;
    double minx;
    double miny;
    double maxx;
    double maxy;
    int mode;
    int illegal = 0;
    MbrCachePtr p_vtab = (MbrCachePtr) pVTab;
    if (pRowid)
	pRowid = pRowid;	/* unused arg warning suppression */
    if (p_vtab->error)
	return SQLITE_OK;
    if (!(p_vtab->cache))
	p_vtab->cache =
	    cache_load (p_vtab->db, p_vtab->table_name, p_vtab->column_name);
    if (argc == 1)
      {
	  /* performing a DELETE */
	  if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
	    {
		rowid = sqlite3_value_int64 (argv[0]);
		cache_delete_cell (p_vtab->cache->first, rowid);
	    }
	  else
	      illegal = 1;
      }
    else
      {
	  if (sqlite3_value_type (argv[0]) == SQLITE_NULL)
	    {
		/* performing an INSERT */
		if (argc == 4)
		  {
		      if (sqlite3_value_type (argv[2]) == SQLITE_INTEGER
			  && sqlite3_value_type (argv[3]) == SQLITE_BLOB)
			{
			    rowid = sqlite3_value_int64 (argv[2]);
			    p_blob =
				(unsigned char *) sqlite3_value_blob (argv[3]);
			    n_bytes = sqlite3_value_bytes (argv[3]);
			    if (gaiaParseFilterMbr
				(p_blob, n_bytes, &minx, &miny, &maxx, &maxy,
				 &mode))
			      {
				  if (mode == GAIA_FILTER_MBR_DECLARE)
				    {
					if (!cache_find_by_rowid
					    (p_vtab->cache->first, rowid))
					    cache_insert_cell (p_vtab->cache,
							       rowid, minx,
							       miny, maxx,
							       maxy);
				    }
				  else
				      illegal = 1;
			      }
			    else
				illegal = 1;
			}
		      else
			  illegal = 1;
		  }
		else
		    illegal = 1;
	    }
	  else
	    {
		/* performing an UPDATE */
		if (argc == 4)
		  {
		      if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER
			  && sqlite3_value_type (argv[3]) == SQLITE_BLOB)
			{
			    rowid = sqlite3_value_int64 (argv[0]);
			    p_blob =
				(unsigned char *) sqlite3_value_blob (argv[3]);
			    n_bytes = sqlite3_value_bytes (argv[3]);
			    if (gaiaParseFilterMbr
				(p_blob, n_bytes, &minx, &miny, &maxx, &maxy,
				 &mode))
			      {
				  if (mode == GAIA_FILTER_MBR_DECLARE)
				      cache_update_cell (p_vtab->cache->first,
							 rowid, minx, miny,
							 maxx, maxy);
				  else
				      illegal = 1;
			      }
			    else
				illegal = 1;
			}
		      else
			  illegal = 1;
		  }
		else
		    illegal = 1;
	    }
      }
    if (illegal)
	return SQLITE_MISMATCH;
    return SQLITE_OK;
}

static int
mbrc_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
mbrc_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
mbrc_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
mbrc_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3MbrCacheInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    my_mbr_module.iVersion = 1;
    my_mbr_module.xCreate = &mbrc_create;
    my_mbr_module.xConnect = &mbrc_connect;
    my_mbr_module.xBestIndex = &mbrc_best_index;
    my_mbr_module.xDisconnect = &mbrc_disconnect;
    my_mbr_module.xDestroy = &mbrc_destroy;
    my_mbr_module.xOpen = &mbrc_open;
    my_mbr_module.xClose = &mbrc_close;
    my_mbr_module.xFilter = &mbrc_filter;
    my_mbr_module.xNext = &mbrc_next;
    my_mbr_module.xEof = &mbrc_eof;
    my_mbr_module.xColumn = &mbrc_column;
    my_mbr_module.xRowid = &mbrc_rowid;
    my_mbr_module.xUpdate = &mbrc_update;
    my_mbr_module.xBegin = &mbrc_begin;
    my_mbr_module.xSync = &mbrc_sync;
    my_mbr_module.xCommit = &mbrc_commit;
    my_mbr_module.xRollback = &mbrc_rollback;
    my_mbr_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "MbrCache", &my_mbr_module, NULL, 0);
    return rc;
}

int
mbrcache_extension_init (sqlite3 * db)
{
    return sqlite3MbrCacheInit (db);
}
/**************** End file: mbrcache.c **********/


/**************** Begin file: virtualshape.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

#ifndef OMIT_ICONV		/* if ICONV is disabled no SHP support is available */

static struct sqlite3_module my_shape_module;

typedef struct VirtualShapeStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
    gaiaShapefilePtr Shp;	/* the Shapefile struct */
    int Srid;			/* the Shapefile SRID */
} VirtualShape;
typedef VirtualShape *VirtualShapePtr;

typedef struct VirtualShapeConstraintStruct
{
/* a constraint to be verified for xFilter */
    int iColumn;		/* Column on left-hand side of constraint */
    int op;			/* Constraint operator */
    char valueType;		/* value Type ('I'=int,'D'=double,'T'=text) */
    sqlite3_int64 intValue;	/* Int64 comparison value */
    double dblValue;		/* Double comparison value */
    char *txtValue;		/* Text comparison value */
    struct VirtualShapeConstraintStruct *next;
} VirtualShapeConstraint;
typedef VirtualShapeConstraint *VirtualShapeConstraintPtr;


typedef struct VirtualShapeCursorStruct
{
/* extends the sqlite3_vtab_cursor struct */
    VirtualShapePtr pVtab;	/* Virtual table of this cursor */
    long current_row;		/* the current row ID */
    int blobSize;
    unsigned char *blobGeometry;
    int eof;			/* the EOF marker */
    VirtualShapeConstraintPtr firstConstraint;
    VirtualShapeConstraintPtr lastConstraint;
} VirtualShapeCursor;
typedef VirtualShapeCursor *VirtualShapeCursorPtr;

static void
vshp_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

static int
vshp_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	     sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table connected to some shapefile */
    char buf[4096];
    char field[128];
    VirtualShapePtr p_vt;
    char path[2048];
    char encoding[128];
    const char *pEncoding = NULL;
    int len;
    const char *pPath = NULL;
    int srid;
    gaiaDbfFieldPtr pFld;
    int cnt;
    int col_cnt;
    int seed;
    int dup;
    int idup;
    char dummyName[4096];
    char **col_name = NULL;
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
/* checking for shapefile PATH */
    if (argc == 6)
      {
	  pPath = argv[3];
	  len = strlen (pPath);
	  if ((*(pPath + 0) == '\'' || *(pPath + 0) == '"')
	      && (*(pPath + len - 1) == '\'' || *(pPath + len - 1) == '"'))
	    {
		/* the path is enclosed between quotes - we need to dequote it */
		strcpy (path, pPath + 1);
		len = strlen (path);
		*(path + len - 1) = '\0';
	    }
	  else
	      strcpy (path, pPath);
	  pEncoding = argv[4];
	  len = strlen (pEncoding);
	  if ((*(pEncoding + 0) == '\'' || *(pEncoding + 0) == '"')
	      && (*(pEncoding + len - 1) == '\''
		  || *(pEncoding + len - 1) == '"'))
	    {
		/* the charset-name is enclosed between quotes - we need to dequote it */
		strcpy (encoding, pEncoding + 1);
		len = strlen (encoding);
		*(encoding + len - 1) = '\0';
	    }
	  else
	      strcpy (encoding, pEncoding);
	  srid = atoi (argv[5]);
	  if (srid <= 0)
	      srid = -1;
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualShape module] CREATE VIRTUAL: illegal arg list {shp_path, encoding, srid}");
	  return SQLITE_ERROR;
      }
    p_vt = (VirtualShapePtr) sqlite3_malloc (sizeof (VirtualShape));
    if (!p_vt)
	return SQLITE_NOMEM;
    p_vt->pModule = &my_shape_module;
    p_vt->nRef = 0;
    p_vt->zErrMsg = NULL;
    p_vt->db = db;
    p_vt->Shp = gaiaAllocShapefile ();
    p_vt->Srid = srid;
/* trying to open files etc in order to ensure we actually have a genuine shapefile */
    gaiaOpenShpRead (p_vt->Shp, path, encoding, "UTF-8");
    if (!(p_vt->Shp->Valid))
      {
	  /* something is going the wrong way; creating a stupid default table */
	  strcpy (dummyName, argv[2]);
	  vshp_double_quoted_sql (dummyName);
	  sprintf (buf, "CREATE TABLE %s (PKUID INTEGER, Geometry BLOB)",
		   dummyName);
	  if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
	    {
		*pzErr =
		    sqlite3_mprintf
		    ("[VirtualShape module] cannot build a table from Shapefile\n");
		return SQLITE_ERROR;
	    }
	  *ppVTab = (sqlite3_vtab *) p_vt;
	  return SQLITE_OK;
      }
    if (p_vt->Shp->Shape == 3 || p_vt->Shp->Shape == 13 ||
	p_vt->Shp->Shape == 23 || p_vt->Shp->Shape == 5 ||
	p_vt->Shp->Shape == 15 || p_vt->Shp->Shape == 25)
      {
	  /* fixing anyway the Geometry type for LINESTRING/MULTILINESTRING or POLYGON/MULTIPOLYGON */
	  gaiaShpAnalyze (p_vt->Shp);
      }
/* preparing the COLUMNs for this VIRTUAL TABLE */
    strcpy (buf, "CREATE TABLE ");
    strcpy (dummyName, argv[2]);
    vshp_double_quoted_sql (dummyName);
    strcat (buf, dummyName);
    strcat (buf, " (PKUID INTEGER, Geometry BLOB");
/* checking for duplicate / illegal column names and antialising them */
    col_cnt = 0;
    pFld = p_vt->Shp->Dbf->First;
    while (pFld)
      {
	  /* counting DBF fields */
	  col_cnt++;
	  pFld = pFld->Next;
      }
    col_name = malloc (sizeof (char *) * col_cnt);
    cnt = 0;
    seed = 0;
    pFld = p_vt->Shp->Dbf->First;
    while (pFld)
      {
	  sprintf (dummyName, "%s", pFld->Name);
	  vshp_double_quoted_sql (dummyName);
	  dup = 0;
	  for (idup = 0; idup < cnt; idup++)
	    {
		if (strcasecmp (dummyName, *(col_name + idup)) == 0)
		    dup = 1;
	    }
	  if (strcasecmp (dummyName, "PKUID") == 0)
	      dup = 1;
	  if (strcasecmp (dummyName, "Geometry") == 0)
	      dup = 1;
	  if (dup)
	    {
		sprintf (dummyName, "COL_%d", seed++);
		vshp_double_quoted_sql (dummyName);
	    }
	  if (pFld->Type == 'N')
	    {
		if (pFld->Decimals > 0 || pFld->Length > 18)
		    sprintf (field, "%s DOUBLE", dummyName);
		else
		    sprintf (field, "%s INTEGER", dummyName);
	    }
	  else if (pFld->Type == 'F')
	      sprintf (field, "%s DOUBLE", dummyName);
	  else
	      sprintf (field, "%s VARCHAR(%d)", dummyName, pFld->Length);
	  strcat (buf, ", ");
	  strcat (buf, field);
	  len = strlen (dummyName);
	  *(col_name + cnt) = malloc (len + 1);
	  strcpy (*(col_name + cnt), dummyName);
	  cnt++;
	  pFld = pFld->Next;
      }
    strcat (buf, ")");
    if (col_name)
      {
	  /* releasing memory allocation for column names */
	  for (cnt = 0; cnt < col_cnt; cnt++)
	      free (*(col_name + cnt));
	  free (col_name);
      }
    if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualShape module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       buf);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    return SQLITE_OK;
}

static int
vshp_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	      sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table to some shapefile - simply aliases vshp_create() */
    return vshp_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
vshp_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIndex)
{
/* best index selection */
    int i;
    int iArg = 0;
    char str[2048];
    char buf[64];

    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */

    *str = '\0';
    for (i = 0; i < pIndex->nConstraint; i++)
      {
	  if (pIndex->aConstraint[i].usable)
	    {
		iArg++;
		pIndex->aConstraintUsage[i].argvIndex = iArg;
		pIndex->aConstraintUsage[i].omit = 1;
		sprintf (buf, "%d:%d,", pIndex->aConstraint[i].iColumn,
			 pIndex->aConstraint[i].op);
		strcat (str, buf);
	    }
      }
    if (*str != '\0')
      {
	  pIndex->idxStr = sqlite3_mprintf ("%s", str);
	  pIndex->needToFreeIdxStr = 1;
      }

    return SQLITE_OK;
}

static int
vshp_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    VirtualShapePtr p_vt = (VirtualShapePtr) pVTab;
    if (p_vt->Shp)
	gaiaFreeShapefile (p_vt->Shp);
    sqlite3_free (p_vt);
    return SQLITE_OK;
}

static int
vshp_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases vshp_disconnect() */
    return vshp_disconnect (pVTab);
}

static void
vshp_read_row (VirtualShapeCursorPtr cursor)
{
/* trying to read a "row" from shapefile */
    int ret;
    gaiaGeomCollPtr geom;
    if (!(cursor->pVtab->Shp->Valid))
      {
	  cursor->eof = 1;
	  return;
      }
    if (cursor->blobGeometry)
      {
	  free (cursor->blobGeometry);
	  cursor->blobGeometry = NULL;
      }
    ret =
	gaiaReadShpEntity (cursor->pVtab->Shp, cursor->current_row,
			   cursor->pVtab->Srid);
    if (!ret)
      {
	  if (!(cursor->pVtab->Shp->LastError))	/* normal SHP EOF */
	    {
		cursor->eof = 1;
		return;
	    }
	  /* an error occurred */
	  fprintf (stderr, "%s\n", cursor->pVtab->Shp->LastError);
	  cursor->eof = 1;
	  return;
      }
    cursor->current_row++;
    geom = cursor->pVtab->Shp->Dbf->Geometry;
    if (geom)
      {
	  /* preparing the BLOB representing Geometry */
	  gaiaToSpatiaLiteBlobWkb (geom, &(cursor->blobGeometry),
				   &(cursor->blobSize));
      }
}

static int
vshp_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    VirtualShapeCursorPtr cursor =
	(VirtualShapeCursorPtr) sqlite3_malloc (sizeof (VirtualShapeCursor));
    if (cursor == NULL)
	return SQLITE_ERROR;
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
    cursor->pVtab = (VirtualShapePtr) pVTab;
    cursor->current_row = 0;
    cursor->blobGeometry = NULL;
    cursor->blobSize = 0;
    cursor->eof = 0;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    vshp_read_row (cursor);
    return SQLITE_OK;
}

static void
vshp_free_constraints (VirtualShapeCursorPtr cursor)
{
/* memory cleanup - cursor constraints */
    VirtualShapeConstraintPtr pC;
    VirtualShapeConstraintPtr pCn;
    pC = cursor->firstConstraint;
    while (pC)
      {
	  pCn = pC->next;
	  if (pC->txtValue)
	      sqlite3_free (pC->txtValue);
	  sqlite3_free (pC);
	  pC = pCn;
      }
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
}

static int
vshp_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    VirtualShapeCursorPtr cursor = (VirtualShapeCursorPtr) pCursor;
    if (cursor->blobGeometry)
	free (cursor->blobGeometry);
    vshp_free_constraints (cursor);
    sqlite3_free (pCursor);
    return SQLITE_OK;
}

static int
vshp_parse_constraint (const char *str, int index, int *iColumn, int *op)
{
/* parsing a constraint string */
    char buf[64];
    const char *in = str;
    char *out = buf;
    int i = 0;
    int found = 0;

    *out = '\0';
    while (*in != '\0')
      {
	  if (*in == ',')
	    {
		if (index == i)
		  {
		      *out = '\0';
		      found = 1;
		      break;
		  }
		i++;
		in++;
		continue;
	    }
	  if (index == i)
	      *out++ = *in;
	  in++;
      }
    if (!found)
	return 0;
    in = buf;
    for (i = 0; i < (int) strlen (buf); i++)
      {
	  if (buf[i] == ':')
	    {
		buf[i] = '\0';
		*iColumn = atoi (buf);
		*op = atoi (buf + i + 1);
		return 1;
	    }
	  in++;
      }
    return 0;
}

static int
vshp_eval_constraints (VirtualShapeCursorPtr cursor)
{
/* evaluating Filter constraints */
    int nCol;
    gaiaDbfFieldPtr pFld;
    VirtualShapeConstraintPtr pC = cursor->firstConstraint;
    if (pC == NULL)
	return 1;
    while (pC)
      {
	  int ok = 0;
	  if (pC->iColumn == 0)
	    {
		/* the PRIMARY KEY column */
		if (pC->valueType == 'I')
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (cursor->current_row == pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (cursor->current_row > pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (cursor->current_row <= pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (cursor->current_row < pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (cursor->current_row >= pC->intValue)
				ok = 1;
			    break;
			};
		  }
		goto done;
	    }
	  nCol = 2;
	  pFld = cursor->pVtab->Shp->Dbf->First;
	  while (pFld)
	    {
		if (nCol == pC->iColumn)
		  {
		      if ((pFld->Value))
			{
			    switch (pFld->Value->Type)
			      {
			      case GAIA_INT_VALUE:
				  if (pC->valueType == 'I')
				    {
					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (pFld->Value->IntValue ==
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (pFld->Value->IntValue >
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (pFld->Value->IntValue <=
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (pFld->Value->IntValue <
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (pFld->Value->IntValue >=
						  pC->intValue)
						  ok = 1;
					      break;
					  };
				    }
				  break;
			      case GAIA_DOUBLE_VALUE:
				  if (pC->valueType == 'I')
				    {
					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (pFld->Value->DblValue ==
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (pFld->Value->DblValue >
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (pFld->Value->DblValue <=
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (pFld->Value->DblValue <
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (pFld->Value->DblValue >=
						  pC->intValue)
						  ok = 1;
					      break;
					  };
				    }
				  if (pC->valueType == 'D')
				    {
					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (pFld->Value->DblValue ==
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (pFld->Value->DblValue >
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (pFld->Value->DblValue <=
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (pFld->Value->DblValue <
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (pFld->Value->DblValue >=
						  pC->dblValue)
						  ok = 1;
					      break;
					  }
				    }
				  break;
			      case GAIA_TEXT_VALUE:
				  if (pC->valueType == 'T' && pC->txtValue)
				    {
					int ret;
					ret =
					    strcmp (pFld->Value->TxtValue,
						    pC->txtValue);
					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (ret == 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (ret > 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (ret <= 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (ret < 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (ret >= 0)
						  ok = 1;
					      break;
					  };
				    }
				  break;
			      };
			}
		      goto done;
		  }
		nCol++;
		pFld = pFld->Next;
	    }
	done:
	  if (!ok)
	      return 0;
	  pC = pC->next;
      }
    return 1;
}

static int
vshp_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	     int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    int i;
    int iColumn;
    int op;
    int len;
    VirtualShapeConstraintPtr pC;
    VirtualShapeCursorPtr cursor = (VirtualShapeCursorPtr) pCursor;
    if (idxNum)
	idxNum = idxNum;	/* unused arg warning suppression */

/* resetting any previously set filter constraint */
    vshp_free_constraints (cursor);

    for (i = 0; i < argc; i++)
      {
	  if (!vshp_parse_constraint (idxStr, i, &iColumn, &op))
	      continue;
	  pC = sqlite3_malloc (sizeof (VirtualShapeConstraint));
	  if (!pC)
	      continue;
	  pC->iColumn = iColumn;
	  pC->op = op;
	  pC->valueType = '\0';
	  pC->txtValue = NULL;
	  pC->next = NULL;

	  if (sqlite3_value_type (argv[i]) == SQLITE_INTEGER)
	    {
		pC->valueType = 'I';
		pC->intValue = sqlite3_value_int64 (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_FLOAT)
	    {
		pC->valueType = 'D';
		pC->dblValue = sqlite3_value_double (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_TEXT)
	    {
		pC->valueType = 'T';
		len = sqlite3_value_bytes (argv[i]) + 1;
		pC->txtValue = (char *) sqlite3_malloc (len);
		if (pC->txtValue)
		    strcpy (pC->txtValue,
			    (char *) sqlite3_value_text (argv[i]));
	    }
	  if (cursor->firstConstraint == NULL)
	      cursor->firstConstraint = pC;
	  if (cursor->lastConstraint != NULL)
	      cursor->lastConstraint->next = pC;
	  cursor->lastConstraint = pC;
      }

    cursor->current_row = 0;
    if (cursor->blobGeometry)
	free (cursor->blobGeometry);
    cursor->blobGeometry = NULL;
    cursor->blobSize = 0;
    cursor->eof = 0;
    while (1)
      {
	  vshp_read_row (cursor);
	  if (cursor->eof)
	      break;
	  if (vshp_eval_constraints (cursor))
	      break;
      }
    return SQLITE_OK;
}

static int
vshp_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching a next row from cursor */
    VirtualShapeCursorPtr cursor = (VirtualShapeCursorPtr) pCursor;
    while (1)
      {
	  vshp_read_row (cursor);
	  if (cursor->eof)
	      break;
	  if (vshp_eval_constraints (cursor))
	      break;
      }
    return SQLITE_OK;
}

static int
vshp_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    VirtualShapeCursorPtr cursor = (VirtualShapeCursorPtr) pCursor;
    return cursor->eof;
}

static int
vshp_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	     int column)
{
/* fetching value for the Nth column */
    int nCol = 2;
    gaiaGeomCollPtr geom;
    gaiaDbfFieldPtr pFld;
    VirtualShapeCursorPtr cursor = (VirtualShapeCursorPtr) pCursor;
    if (column == 0)
      {
	  /* the PRIMARY KEY column */
	  sqlite3_result_int (pContext, cursor->current_row);
	  return SQLITE_OK;
      }
    if (column == 1)
      {
	  /* the GEOMETRY column */
	  geom = cursor->pVtab->Shp->Dbf->Geometry;
	  if (geom)
	      sqlite3_result_blob (pContext, cursor->blobGeometry,
				   cursor->blobSize, SQLITE_STATIC);
	  else
	      sqlite3_result_null (pContext);
	  return SQLITE_OK;
      }
    pFld = cursor->pVtab->Shp->Dbf->First;
    while (pFld)
      {
	  /* column values */
	  if (nCol == column)
	    {
		if (!(pFld->Value))
		    sqlite3_result_null (pContext);
		else
		  {
		      switch (pFld->Value->Type)
			{
			case GAIA_INT_VALUE:
			    sqlite3_result_int64 (pContext,
						  pFld->Value->IntValue);
			    break;
			case GAIA_DOUBLE_VALUE:
			    sqlite3_result_double (pContext,
						   pFld->Value->DblValue);
			    break;
			case GAIA_TEXT_VALUE:
			    sqlite3_result_text (pContext,
						 pFld->Value->TxtValue,
						 strlen (pFld->Value->TxtValue),
						 SQLITE_STATIC);
			    break;
			default:
			    sqlite3_result_null (pContext);
			    break;
			}
		  }
		break;
	    }
	  nCol++;
	  pFld = pFld->Next;
      }
    return SQLITE_OK;
}

static int
vshp_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    VirtualShapeCursorPtr cursor = (VirtualShapeCursorPtr) pCursor;
    *pRowid = cursor->current_row;
    return SQLITE_OK;
}

static int
vshp_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	     sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    if (pVTab || argc || argv || pRowid)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_READONLY;
}

static int
vshp_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vshp_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vshp_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vshp_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3VirtualShapeInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    my_shape_module.iVersion = 1;
    my_shape_module.xCreate = &vshp_create;
    my_shape_module.xConnect = &vshp_connect;
    my_shape_module.xBestIndex = &vshp_best_index;
    my_shape_module.xDisconnect = &vshp_disconnect;
    my_shape_module.xDestroy = &vshp_destroy;
    my_shape_module.xOpen = &vshp_open;
    my_shape_module.xClose = &vshp_close;
    my_shape_module.xFilter = &vshp_filter;
    my_shape_module.xNext = &vshp_next;
    my_shape_module.xEof = &vshp_eof;
    my_shape_module.xColumn = &vshp_column;
    my_shape_module.xRowid = &vshp_rowid;
    my_shape_module.xUpdate = &vshp_update;
    my_shape_module.xBegin = &vshp_begin;
    my_shape_module.xSync = &vshp_sync;
    my_shape_module.xCommit = &vshp_commit;
    my_shape_module.xRollback = &vshp_rollback;
    my_shape_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "VirtualShape", &my_shape_module, NULL, 0);
    return rc;
}

int
virtualshape_extension_init (sqlite3 * db)
{
    return sqlite3VirtualShapeInit (db);
}

#endif /* ICONV enabled/disabled */
/**************** End file: virtualshape.c **********/


/**************** Begin file: virtualdbf.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

#ifndef OMIT_ICONV		/* if ICONV is disabled no DBF support is available */

static struct sqlite3_module my_dbf_module;

typedef struct VirtualDbfStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
    gaiaDbfPtr dbf;		/* the DBF struct */
} VirtualDbf;
typedef VirtualDbf *VirtualDbfPtr;

typedef struct VirtualDbfConstraintStruct
{
/* a constraint to be verified for xFilter */
    int iColumn;		/* Column on left-hand side of constraint */
    int op;			/* Constraint operator */
    char valueType;		/* value Type ('I'=int,'D'=double,'T'=text) */
    sqlite3_int64 intValue;	/* Int64 comparison value */
    double dblValue;		/* Double comparison value */
    char *txtValue;		/* Text comparison value */
    struct VirtualDbfConstraintStruct *next;
} VirtualDbfConstraint;
typedef VirtualDbfConstraint *VirtualDbfConstraintPtr;

typedef struct VirtualDbfCursorStruct
{
/* extends the sqlite3_vtab_cursor struct */
    VirtualDbfPtr pVtab;	/* Virtual table of this cursor */
    long current_row;		/* the current row ID */
    int eof;			/* the EOF marker */
    VirtualDbfConstraintPtr firstConstraint;
    VirtualDbfConstraintPtr lastConstraint;
} VirtualDbfCursor;

typedef VirtualDbfCursor *VirtualDbfCursorPtr;

static void
vdbf_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

static int
vdbf_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	     sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table connected to some DBF */
    char buf[4096];
    char field[128];
    VirtualDbfPtr p_vt;
    char path[2048];
    char encoding[128];
    const char *pEncoding = NULL;
    int len;
    const char *pPath = NULL;
    gaiaDbfFieldPtr pFld;
    int cnt;
    int col_cnt;
    int seed;
    int dup;
    int idup;
    char dummyName[4096];
    char **col_name = NULL;
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
/* checking for DBF PATH */
    if (argc == 5)
      {
	  pPath = argv[3];
	  len = strlen (pPath);
	  if ((*(pPath + 0) == '\'' || *(pPath + 0) == '"')
	      && (*(pPath + len - 1) == '\'' || *(pPath + len - 1) == '"'))
	    {
		/* the path is enclosed between quotes - we need to dequote it */
		strcpy (path, pPath + 1);
		len = strlen (path);
		*(path + len - 1) = '\0';
	    }
	  else
	      strcpy (path, pPath);
	  pEncoding = argv[4];
	  len = strlen (pEncoding);
	  if ((*(pEncoding + 0) == '\'' || *(pEncoding + 0) == '"')
	      && (*(pEncoding + len - 1) == '\''
		  || *(pEncoding + len - 1) == '"'))
	    {
		/* the charset-name is enclosed between quotes - we need to dequote it */
		strcpy (encoding, pEncoding + 1);
		len = strlen (encoding);
		*(encoding + len - 1) = '\0';
	    }
	  else
	      strcpy (encoding, pEncoding);
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualDbf module] CREATE VIRTUAL: illegal arg list {dbf_path, encoding}");
	  return SQLITE_ERROR;
      }
    p_vt = (VirtualDbfPtr) sqlite3_malloc (sizeof (VirtualDbf));
    if (!p_vt)
	return SQLITE_NOMEM;
    p_vt->pModule = &my_dbf_module;
    p_vt->nRef = 0;
    p_vt->zErrMsg = NULL;
    p_vt->db = db;
    p_vt->dbf = gaiaAllocDbf ();
/* trying to open file */
    gaiaOpenDbfRead (p_vt->dbf, path, encoding, "UTF-8");
    if (!(p_vt->dbf->Valid))
      {
	  /* something is going the wrong way; creating a stupid default table */
	  strcpy (dummyName, argv[2]);
	  vdbf_double_quoted_sql (dummyName);
	  sprintf (buf, "CREATE TABLE %s (PKUID INTEGER)", dummyName);
	  if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
	    {
		*pzErr =
		    sqlite3_mprintf
		    ("[VirtualDbf module] cannot build a table from DBF\n");
		return SQLITE_ERROR;
	    }
	  *ppVTab = (sqlite3_vtab *) p_vt;
	  return SQLITE_OK;
      }
/* preparing the COLUMNs for this VIRTUAL TABLE */
    strcpy (buf, "CREATE TABLE ");
    strcpy (dummyName, argv[2]);
    vdbf_double_quoted_sql (dummyName);
    strcat (buf, dummyName);
    strcat (buf, " (PKUID INTEGER");
/* checking for duplicate / illegal column names and antialising them */
    col_cnt = 0;
    pFld = p_vt->dbf->Dbf->First;
    while (pFld)
      {
	  /* counting DBF fields */
	  col_cnt++;
	  pFld = pFld->Next;
      }
    col_name = malloc (sizeof (char *) * col_cnt);
    cnt = 0;
    seed = 0;
    pFld = p_vt->dbf->Dbf->First;
    while (pFld)
      {
	  sprintf (dummyName, "%s", pFld->Name);
	  vdbf_double_quoted_sql (dummyName);
	  dup = 0;
	  for (idup = 0; idup < cnt; idup++)
	    {
		if (strcasecmp (dummyName, *(col_name + idup)) == 0)
		    dup = 1;
	    }
	  if (strcasecmp (dummyName, "PKUID") == 0)
	      dup = 1;
	  if (dup)
	    {
		sprintf (dummyName, "COL_%d", seed++);
		vdbf_double_quoted_sql (dummyName);
	    }
	  if (pFld->Type == 'N')
	    {
		if (pFld->Decimals > 0 || pFld->Length > 18)
		    sprintf (field, "%s DOUBLE", dummyName);
		else
		    sprintf (field, "%s INTEGER", dummyName);
	    }
	  else if (pFld->Type == 'F')
	      sprintf (field, "%s DOUBLE", dummyName);
	  else
	      sprintf (field, "%s VARCHAR(%d)", dummyName, pFld->Length);
	  strcat (buf, ", ");
	  strcat (buf, field);
	  len = strlen (dummyName);
	  *(col_name + cnt) = malloc (len + 1);
	  strcpy (*(col_name + cnt), dummyName);
	  cnt++;
	  pFld = pFld->Next;
      }
    strcat (buf, ")");
    if (col_name)
      {
	  /* releasing memory allocation for column names */
	  for (cnt = 0; cnt < col_cnt; cnt++)
	      free (*(col_name + cnt));
	  free (col_name);
      }
    if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualDbf module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       buf);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    return SQLITE_OK;
}

static int
vdbf_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	      sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table to some DBF - simply aliases vdbf_create() */
    return vdbf_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
vdbf_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIndex)
{
/* best index selection */
    int i;
    int iArg = 0;
    char str[2048];
    char buf[64];

    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */

    *str = '\0';
    for (i = 0; i < pIndex->nConstraint; i++)
      {
	  if (pIndex->aConstraint[i].usable)
	    {
		iArg++;
		pIndex->aConstraintUsage[i].argvIndex = iArg;
		pIndex->aConstraintUsage[i].omit = 1;
		sprintf (buf, "%d:%d,", pIndex->aConstraint[i].iColumn,
			 pIndex->aConstraint[i].op);
		strcat (str, buf);
	    }
      }
    if (*str != '\0')
      {
	  pIndex->idxStr = sqlite3_mprintf ("%s", str);
	  pIndex->needToFreeIdxStr = 1;
      }

    return SQLITE_OK;
}

static int
vdbf_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    VirtualDbfPtr p_vt = (VirtualDbfPtr) pVTab;
    if (p_vt->dbf)
	gaiaFreeDbf (p_vt->dbf);
    sqlite3_free (p_vt);
    return SQLITE_OK;
}

static int
vdbf_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases vdbf_disconnect() */
    return vdbf_disconnect (pVTab);
}

static void
vdbf_read_row (VirtualDbfCursorPtr cursor, int *deleted_row)
{
/* trying to read a "row" from DBF */
    int ret;
    int deleted;
    if (!(cursor->pVtab->dbf->Valid))
      {
	  cursor->eof = 1;
	  return;
      }
    ret = gaiaReadDbfEntity (cursor->pVtab->dbf, cursor->current_row, &deleted);
    if (!ret)
      {
	  if (!(cursor->pVtab->dbf->LastError))	/* normal DBF EOF */
	    {
		cursor->eof = 1;
		return;
	    }
	  /* an error occurred */
	  fprintf (stderr, "%s\n", cursor->pVtab->dbf->LastError);
	  cursor->eof = 1;
	  return;
      }
    cursor->current_row++;
    *deleted_row = deleted;
}

static int
vdbf_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    int deleted;
    VirtualDbfCursorPtr cursor =
	(VirtualDbfCursorPtr) sqlite3_malloc (sizeof (VirtualDbfCursor));
    if (cursor == NULL)
	return SQLITE_ERROR;
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
    cursor->pVtab = (VirtualDbfPtr) pVTab;
    cursor->current_row = 0;
    cursor->eof = 0;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    while (1)
      {
	  vdbf_read_row (cursor, &deleted);
	  if (cursor->eof)
	      break;
	  if (!deleted)
	      break;
      }
    return SQLITE_OK;
}

static void
vdbf_free_constraints (VirtualDbfCursorPtr cursor)
{
/* memory cleanup - cursor constraints */
    VirtualDbfConstraintPtr pC;
    VirtualDbfConstraintPtr pCn;
    pC = cursor->firstConstraint;
    while (pC)
      {
	  pCn = pC->next;
	  if (pC->txtValue)
	      sqlite3_free (pC->txtValue);
	  sqlite3_free (pC);
	  pC = pCn;
      }
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
}

static int
vdbf_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    VirtualDbfCursorPtr cursor = (VirtualDbfCursorPtr) pCursor;
    vdbf_free_constraints (cursor);
    sqlite3_free (pCursor);
    return SQLITE_OK;
}

static int
vdbf_parse_constraint (const char *str, int index, int *iColumn, int *op)
{
/* parsing a constraint string */
    char buf[64];
    const char *in = str;
    char *out = buf;
    int i = 0;
    int found = 0;

    *out = '\0';
    while (*in != '\0')
      {
	  if (*in == ',')
	    {
		if (index == i)
		  {
		      *out = '\0';
		      found = 1;
		      break;
		  }
		i++;
		in++;
		continue;
	    }
	  if (index == i)
	      *out++ = *in;
	  in++;
      }
    if (!found)
	return 0;
    in = buf;
    for (i = 0; i < (int) strlen (buf); i++)
      {
	  if (buf[i] == ':')
	    {
		buf[i] = '\0';
		*iColumn = atoi (buf);
		*op = atoi (buf + i + 1);
		return 1;
	    }
	  in++;
      }
    return 0;
}

static int
vdbf_eval_constraints (VirtualDbfCursorPtr cursor)
{
/* evaluating Filter constraints */
    int nCol;
    gaiaDbfFieldPtr pFld;
    VirtualDbfConstraintPtr pC = cursor->firstConstraint;
    if (pC == NULL)
	return 1;
    while (pC)
      {
	  int ok = 0;
	  if (pC->iColumn == 0)
	    {
		/* the PRIMARY KEY column */
		if (pC->valueType == 'I')
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (cursor->current_row == pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (cursor->current_row > pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (cursor->current_row <= pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (cursor->current_row < pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (cursor->current_row >= pC->intValue)
				ok = 1;
			    break;
			};
		  }
		goto done;
	    }
	  nCol = 1;
	  pFld = cursor->pVtab->dbf->Dbf->First;
	  while (pFld)
	    {
		if (nCol == pC->iColumn)
		  {
		      if ((pFld->Value))
			{
			    switch (pFld->Value->Type)
			      {
			      case GAIA_INT_VALUE:
				  if (pC->valueType == 'I')
				    {

					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (pFld->Value->IntValue ==
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (pFld->Value->IntValue >
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (pFld->Value->IntValue <=
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (pFld->Value->IntValue <
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (pFld->Value->IntValue >=
						  pC->intValue)
						  ok = 1;
					      break;
					  };
				    }
				  break;
			      case GAIA_DOUBLE_VALUE:
				  if (pC->valueType == 'I')
				    {

					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (pFld->Value->DblValue ==
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (pFld->Value->DblValue >
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (pFld->Value->DblValue <=
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (pFld->Value->DblValue <
						  pC->intValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (pFld->Value->DblValue >=
						  pC->intValue)
						  ok = 1;
					      break;
					  };
				    }
				  if (pC->valueType == 'D')
				    {

					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (pFld->Value->DblValue ==
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (pFld->Value->DblValue >
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (pFld->Value->DblValue <=
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (pFld->Value->DblValue <
						  pC->dblValue)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (pFld->Value->DblValue >=
						  pC->dblValue)
						  ok = 1;
					      break;
					  }
				    }
				  break;
			      case GAIA_TEXT_VALUE:
				  if (pC->valueType == 'T' && pC->txtValue)
				    {

					int ret;
					ret =
					    strcmp (pFld->Value->TxtValue,
						    pC->txtValue);
					switch (pC->op)
					  {
					  case SQLITE_INDEX_CONSTRAINT_EQ:
					      if (ret == 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GT:
					      if (ret > 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LE:
					      if (ret <= 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_LT:
					      if (ret < 0)
						  ok = 1;
					      break;
					  case SQLITE_INDEX_CONSTRAINT_GE:
					      if (ret >= 0)
						  ok = 1;
					      break;
					  };
				    }
				  break;
			      };
			}
		      goto done;
		  }
		nCol++;
		pFld = pFld->Next;
	    }
	done:
	  if (!ok)
	      return 0;
	  pC = pC->next;
      }
    return 1;
}

static int
vdbf_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	     int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    int i;
    int iColumn;
    int op;
    int len;
    int deleted;
    VirtualDbfConstraintPtr pC;
    VirtualDbfCursorPtr cursor = (VirtualDbfCursorPtr) pCursor;
    if (idxNum)
	idxNum = idxNum;	/* unused arg warning suppression */

/* resetting any previously set filter constraint */
    vdbf_free_constraints (cursor);

    for (i = 0; i < argc; i++)
      {
	  if (!vdbf_parse_constraint (idxStr, i, &iColumn, &op))
	      continue;
	  pC = sqlite3_malloc (sizeof (VirtualDbfConstraint));
	  if (!pC)
	      continue;
	  pC->iColumn = iColumn;
	  pC->op = op;
	  pC->valueType = '\0';
	  pC->txtValue = NULL;
	  pC->next = NULL;

	  if (sqlite3_value_type (argv[i]) == SQLITE_INTEGER)
	    {
		pC->valueType = 'I';
		pC->intValue = sqlite3_value_int64 (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_FLOAT)
	    {
		pC->valueType = 'D';
		pC->dblValue = sqlite3_value_double (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_TEXT)
	    {
		pC->valueType = 'T';
		len = sqlite3_value_bytes (argv[i]) + 1;
		pC->txtValue = (char *) sqlite3_malloc (len);
		if (pC->txtValue)
		    strcpy (pC->txtValue,
			    (char *) sqlite3_value_text (argv[i]));
	    }
	  if (cursor->firstConstraint == NULL)
	      cursor->firstConstraint = pC;
	  if (cursor->lastConstraint != NULL)
	      cursor->lastConstraint->next = pC;
	  cursor->lastConstraint = pC;
      }

    cursor->current_row = 0;
    cursor->eof = 0;
    while (1)
      {
	  vdbf_read_row (cursor, &deleted);
	  if (cursor->eof)
	      break;
	  if (deleted)
	      continue;
	  if (vdbf_eval_constraints (cursor))
	      break;
      }
    return SQLITE_OK;
}

static int
vdbf_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching a next row from cursor */
    int deleted;
    VirtualDbfCursorPtr cursor = (VirtualDbfCursorPtr) pCursor;
    while (1)
      {
	  vdbf_read_row (cursor, &deleted);
	  if (cursor->eof)
	      break;
	  if (deleted)
	      continue;
	  if (vdbf_eval_constraints (cursor))
	      break;
      }
    return SQLITE_OK;
}

static int
vdbf_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    VirtualDbfCursorPtr cursor = (VirtualDbfCursorPtr) pCursor;
    return cursor->eof;
}

static int
vdbf_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	     int column)
{
/* fetching value for the Nth column */
    int nCol = 1;
    gaiaDbfFieldPtr pFld;
    VirtualDbfCursorPtr cursor = (VirtualDbfCursorPtr) pCursor;
    if (column == 0)
      {
	  /* the PRIMARY KEY column */
	  sqlite3_result_int (pContext, cursor->current_row);
	  return SQLITE_OK;
      }
    pFld = cursor->pVtab->dbf->Dbf->First;
    while (pFld)
      {
	  /* column values */
	  if (nCol == column)
	    {
		if (!(pFld->Value))
		    sqlite3_result_null (pContext);
		else
		  {
		      switch (pFld->Value->Type)
			{
			case GAIA_INT_VALUE:
			    sqlite3_result_int64 (pContext,
						  pFld->Value->IntValue);
			    break;
			case GAIA_DOUBLE_VALUE:
			    sqlite3_result_double (pContext,
						   pFld->Value->DblValue);
			    break;
			case GAIA_TEXT_VALUE:
			    sqlite3_result_text (pContext,
						 pFld->Value->TxtValue,
						 strlen (pFld->Value->TxtValue),
						 SQLITE_STATIC);
			    break;
			default:
			    sqlite3_result_null (pContext);
			    break;
			}
		  }
		break;
	    }
	  nCol++;
	  pFld = pFld->Next;
      }
    return SQLITE_OK;
}

static int
vdbf_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    VirtualDbfCursorPtr cursor = (VirtualDbfCursorPtr) pCursor;
    *pRowid = cursor->current_row;
    return SQLITE_OK;
}

static int
vdbf_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	     sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    if (pVTab || argc || argv || pRowid)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_READONLY;
}

static int
vdbf_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vdbf_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vdbf_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vdbf_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3VirtualDbfInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    my_dbf_module.iVersion = 1;
    my_dbf_module.xCreate = &vdbf_create;
    my_dbf_module.xConnect = &vdbf_connect;
    my_dbf_module.xBestIndex = &vdbf_best_index;
    my_dbf_module.xDisconnect = &vdbf_disconnect;
    my_dbf_module.xDestroy = &vdbf_destroy;
    my_dbf_module.xOpen = &vdbf_open;
    my_dbf_module.xClose = &vdbf_close;
    my_dbf_module.xFilter = &vdbf_filter;
    my_dbf_module.xNext = &vdbf_next;
    my_dbf_module.xEof = &vdbf_eof;
    my_dbf_module.xColumn = &vdbf_column;
    my_dbf_module.xRowid = &vdbf_rowid;
    my_dbf_module.xUpdate = &vdbf_update;
    my_dbf_module.xBegin = &vdbf_begin;
    my_dbf_module.xSync = &vdbf_sync;
    my_dbf_module.xCommit = &vdbf_commit;
    my_dbf_module.xRollback = &vdbf_rollback;
    my_dbf_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "VirtualDbf", &my_dbf_module, NULL, 0);
    return rc;
}

int
virtualdbf_extension_init (sqlite3 * db)
{
    return sqlite3VirtualDbfInit (db);
}

#endif /* ICONV enabled/disabled */
/**************** End file: virtualdbf.c **********/


/**************** Begin file: virtualXL.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */

#ifndef OMIT_FREEXL
/* #include <freexl.h> */
#endif

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

#ifndef OMIT_ICONV		/* if ICONV is disabled no XL support is available */
#ifndef OMIT_FREEXL		/* FreeXL is disabled */

static struct sqlite3_module my_XL_module;

typedef struct VirtualXLStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
    const void *XL_handle;	/* the XL handle */
    unsigned int rows;		/* Worksheet #rows */
    unsigned short columns;	/* Worksheet #columns */
    char firstLineTitles;	/* 'Y' or 'N' */
} VirtualXL;
typedef VirtualXL *VirtualXLPtr;

typedef struct VirtualXLConstraintStruct
{
/* a constraint to be verified for xFilter */
    int iColumn;		/* Column on left-hand side of constraint */
    int op;			/* Constraint operator */
    char valueType;		/* value Type ('I'=int,'D'=double,'T'=text) */
    sqlite3_int64 intValue;	/* Int64 comparison value */
    double dblValue;		/* Double comparison value */
    char *txtValue;		/* Text comparison value */
    struct VirtualXLConstraintStruct *next;
} VirtualXLConstraint;
typedef VirtualXLConstraint *VirtualXLConstraintPtr;

typedef struct VirtualXLCursorStruct
{
/* extends the sqlite3_vtab_cursor struct */
    VirtualXLPtr pVtab;		/* Virtual table of this cursor */
    unsigned int current_row;	/* the current row ID */
    int eof;			/* the EOF marker */
    VirtualXLConstraintPtr firstConstraint;
    VirtualXLConstraintPtr lastConstraint;
} VirtualXLCursor;
typedef VirtualXLCursor *VirtualXLCursorPtr;

static void
vXL_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

static int
vXL_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	    sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table connected to some XLS file */
    char buf[4096];
    VirtualXLPtr p_vt;
    char path[2048];
    char firstLineTitles = 'N';
    unsigned int worksheet = 0;
    unsigned int max_worksheet;
    unsigned int info;
    unsigned int rows;
    unsigned short columns;
    unsigned short col;
    int len;
    int ret;
    const void *handle;
    const char *pPath = NULL;
    char dummyName[128];
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
/* checking for XLS PATH */
    if (argc == 4 || argc == 5 || argc == 6)
      {
	  pPath = argv[3];
	  len = strlen (pPath);
	  if ((*(pPath + 0) == '\'' || *(pPath + 0) == '"')
	      && (*(pPath + len - 1) == '\'' || *(pPath + len - 1) == '"'))
	    {
		/* the path is enclosed between quotes - we need to dequote it */
		strcpy (path, pPath + 1);
		len = strlen (path);
		*(path + len - 1) = '\0';
	    }
	  else
	      strcpy (path, pPath);
	  if (argc == 5 || argc == 6)
	      worksheet = atoi (argv[4]);
	  if (argc == 6)
	    {
		if (atoi (argv[5]) == 1)
		    firstLineTitles = 'Y';
	    }
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualXL module] CREATE VIRTUAL: illegal arg list {xls_path [, worksheet_index [, first_line_titles(1/0)]]}");
	  return SQLITE_ERROR;
      }
/* allocating the main XL module */
    p_vt = (VirtualXLPtr) sqlite3_malloc (sizeof (VirtualXL));
    if (!p_vt)
	return SQLITE_NOMEM;
    p_vt->pModule = &my_XL_module;
    p_vt->nRef = 0;
    p_vt->zErrMsg = NULL;
    p_vt->db = db;
    p_vt->XL_handle = NULL;
    p_vt->rows = 0;
    p_vt->columns = 0;
    p_vt->firstLineTitles = firstLineTitles;
/* opening the .XLS file [Workbook] */
    ret = freexl_open (path, &handle);
    if (ret != FREEXL_OK)
      {
	  /* free memory */
	  freexl_close (handle);
	  /* something is going the wrong way; creating a stupid default table */
	  strcpy (dummyName, argv[2]);
	  vXL_double_quoted_sql (dummyName);
	  sprintf (buf, "CREATE TABLE %s (PKUID INTEGER)", dummyName);
	  if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
	    {
		*pzErr =
		    sqlite3_mprintf
		    ("[VirtualXL module] cannot build a table from XL\n");
		return SQLITE_ERROR;
	    }
	  *ppVTab = (sqlite3_vtab *) p_vt;
	  return SQLITE_OK;
      }
/* checking if Password Protected [obfuscated] */
    freexl_get_info (handle, FREEXL_BIFF_PASSWORD, &info);
    if (info != FREEXL_BIFF_PLAIN)
      {
	  /* free memory */
	  freexl_close (handle);
	  /* Obfuscated: creating a stupid default table */
	  strcpy (dummyName, argv[2]);
	  vXL_double_quoted_sql (dummyName);
	  sprintf (buf, "CREATE TABLE %s (PKUID INTEGER)", dummyName);
	  if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
	    {
		*pzErr =
		    sqlite3_mprintf
		    ("[VirtualXL module] Password protected [obfuscated] .xls\n");
		return SQLITE_ERROR;
	    }
	  *ppVTab = (sqlite3_vtab *) p_vt;
	  return SQLITE_OK;
      }
/* querying how many Worksheets are there */
    freexl_get_info (handle, FREEXL_BIFF_SHEET_COUNT, &max_worksheet);
    if (worksheet >= max_worksheet)
      {
	  /* free memory */
	  freexl_close (handle);
	  /* no such Worksheet: creating a stupid default table */
	  strcpy (dummyName, argv[2]);
	  vXL_double_quoted_sql (dummyName);
	  sprintf (buf, "CREATE TABLE %s (PKUID INTEGER)", dummyName);
	  if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
	    {
		*pzErr =
		    sqlite3_mprintf
		    ("[VirtualXL module] no such Worksheet [index=%u]\n",
		     worksheet);
		return SQLITE_ERROR;
	    }
	  *ppVTab = (sqlite3_vtab *) p_vt;
	  return SQLITE_OK;
      }
/* selecting the currently active XL Worksheet */
    freexl_select_active_worksheet (handle, worksheet);
    freexl_worksheet_dimensions (handle, &rows, &columns);
    p_vt->XL_handle = handle;
    p_vt->rows = rows;
    p_vt->columns = columns;
/* preparing the COLUMNs for this VIRTUAL TABLE */
    strcpy (buf, "CREATE TABLE ");
    strcpy (dummyName, argv[2]);
    vXL_double_quoted_sql (dummyName);
    strcat (buf, dummyName);
    strcat (buf, " (row_no INTEGER");
    if (firstLineTitles == 'Y')
      {
	  /* fetching column names */
	  for (col = 0; col < columns; col++)
	    {
		FreeXL_CellValue cell;
		int ret = freexl_get_cell_value (handle, 0, col, &cell);
		if (ret != FREEXL_OK)
		    sprintf (dummyName, "col_%d", col);
		else
		  {
		      if (cell.type == FREEXL_CELL_INT)
			  sprintf (dummyName, "%d", cell.value.int_value);
		      else if (cell.type == FREEXL_CELL_DOUBLE)
			  sprintf (dummyName, "%1.2f", cell.value.double_value);
		      else if (cell.type == FREEXL_CELL_TEXT
			       || cell.type == FREEXL_CELL_SST_TEXT
			       || cell.type == FREEXL_CELL_DATE
			       || cell.type == FREEXL_CELL_DATETIME
			       || cell.type == FREEXL_CELL_TIME)
			{
			    int len = strlen (cell.value.text_value);
			    if (len < 256)
				strcpy (dummyName, cell.value.text_value);
			    else
				sprintf (dummyName, "col_%d", col);
			}
		      else
			  sprintf (dummyName, "col_%d", col);
		  }
		vXL_double_quoted_sql (dummyName);
		strcat (buf, ", ");
		strcat (buf, dummyName);
	    }
      }
    else
      {
	  /* setting default column names */
	  for (col = 0; col < columns; col++)
	    {
		sprintf (dummyName, "col_%d", col);
		vXL_double_quoted_sql (dummyName);
		strcat (buf, ", ");
		strcat (buf, dummyName);
	    }
      }
    strcat (buf, ")");
    if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualXL module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       buf);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    return SQLITE_OK;
}

static int
vXL_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	     sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table to a .xls file - simply aliases vXL_create() */
    return vXL_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
vXL_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIndex)
{
/* best index selection */
    int i;
    int iArg = 0;
    char str[2048];
    char buf[64];

    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */

    *str = '\0';
    for (i = 0; i < pIndex->nConstraint; i++)
      {
	  if (pIndex->aConstraint[i].usable)
	    {
		iArg++;
		pIndex->aConstraintUsage[i].argvIndex = iArg;
		pIndex->aConstraintUsage[i].omit = 1;
		sprintf (buf, "%d:%d,", pIndex->aConstraint[i].iColumn,
			 pIndex->aConstraint[i].op);
		strcat (str, buf);
	    }
      }
    if (*str != '\0')
      {
	  pIndex->idxStr = sqlite3_mprintf ("%s", str);
	  pIndex->needToFreeIdxStr = 1;
      }

    return SQLITE_OK;
}

static int
vXL_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    VirtualXLPtr p_vt = (VirtualXLPtr) pVTab;
    if (p_vt->XL_handle)
	freexl_close (p_vt->XL_handle);
    sqlite3_free (p_vt);
    return SQLITE_OK;
}

static int
vXL_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases vXL_disconnect() */
    return vXL_disconnect (pVTab);
}

static void
vXL_read_row (VirtualXLCursorPtr cursor)
{
/* trying to read a "row" from XL */
    cursor->current_row++;
    if (cursor->current_row > cursor->pVtab->rows)
      {
	  cursor->eof = 1;
	  return;
      }
}

static int
vXL_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    VirtualXLCursorPtr cursor =
	(VirtualXLCursorPtr) sqlite3_malloc (sizeof (VirtualXLCursor));
    if (cursor == NULL)
	return SQLITE_ERROR;
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
    cursor->pVtab = (VirtualXLPtr) pVTab;
    if (cursor->pVtab->firstLineTitles == 'Y')
	cursor->current_row = 1;
    else
	cursor->current_row = 0;
    cursor->eof = 0;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    vXL_read_row (cursor);
    return SQLITE_OK;
}

static void
vXL_free_constraints (VirtualXLCursorPtr cursor)
{
/* memory cleanup - cursor constraints */
    VirtualXLConstraintPtr pC;
    VirtualXLConstraintPtr pCn;
    pC = cursor->firstConstraint;
    while (pC)
      {
	  pCn = pC->next;
	  if (pC->txtValue)
	      sqlite3_free (pC->txtValue);
	  sqlite3_free (pC);
	  pC = pCn;
      }
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
}

static int
vXL_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    VirtualXLCursorPtr cursor = (VirtualXLCursorPtr) pCursor;
    vXL_free_constraints (cursor);
    sqlite3_free (pCursor);
    return SQLITE_OK;
}

static int
vXL_parse_constraint (const char *str, int index, int *iColumn, int *op)
{
/* parsing a constraint string */
    char buf[64];
    const char *in = str;
    char *out = buf;
    int i = 0;
    int found = 0;

    *out = '\0';
    while (*in != '\0')
      {
	  if (*in == ',')
	    {
		if (index == i)
		  {
		      *out = '\0';
		      found = 1;
		      break;
		  }
		i++;
		in++;
		continue;
	    }
	  if (index == i)
	      *out++ = *in;
	  in++;
      }
    if (!found)
	return 0;
    in = buf;
    for (i = 0; i < (int) strlen (buf); i++)
      {
	  if (buf[i] == ':')
	    {
		buf[i] = '\0';
		*iColumn = atoi (buf);
		*op = atoi (buf + i + 1);
		return 1;
	    }
	  in++;
      }
    return 0;
}

static int
vXL_eval_constraints (VirtualXLCursorPtr cursor)
{
/* evaluating Filter constraints */
    FreeXL_CellValue cell;
    VirtualXLConstraintPtr pC = cursor->firstConstraint;
    if (pC == NULL)
	return 1;
    while (pC)
      {
	  int ok = 0;
	  if (pC->iColumn == 0)
	    {
		/* the PRIMARY KEY column */
		if (pC->valueType == 'I')
		  {
		      int cur_row = cursor->current_row;
		      if (cursor->pVtab->firstLineTitles == 'Y')
			  cur_row--;
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (cur_row == pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (cur_row > pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (cur_row <= pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (cur_row < pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (cur_row >= pC->intValue)
				ok = 1;
			    break;
			};
		  }
		goto done;
	    }
	  if (cursor->pVtab->XL_handle != NULL
	      && cursor->current_row <= cursor->pVtab->rows
	      && pC->iColumn <= cursor->pVtab->columns)
	      freexl_get_cell_value (cursor->pVtab->XL_handle,
				     cursor->current_row - 1, pC->iColumn - 1,
				     &cell);
	  else
	      cell.type = FREEXL_CELL_NULL;
	  if (cell.type == FREEXL_CELL_INT)
	    {
		if (pC->valueType == 'I')
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (cell.value.int_value == pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (cell.value.int_value > pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (cell.value.int_value <= pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (cell.value.int_value < pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (cell.value.int_value >= pC->intValue)
				ok = 1;
			    break;
			};
		  }
		if (pC->valueType == 'D')
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (cell.value.int_value == pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (cell.value.int_value > pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (cell.value.int_value <= pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (cell.value.int_value < pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (cell.value.int_value >= pC->dblValue)
				ok = 1;
			    break;
			};
		  }
	    }
	  if (cell.type == FREEXL_CELL_DOUBLE)
	    {
		if (pC->valueType == 'I')
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (cell.value.double_value == pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (cell.value.double_value > pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (cell.value.double_value <= pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (cell.value.double_value < pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (cell.value.double_value >= pC->intValue)
				ok = 1;
			    break;
			};
		  }
		if (pC->valueType == 'D')
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (cell.value.double_value == pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (cell.value.double_value > pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (cell.value.double_value <= pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (cell.value.double_value < pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (cell.value.double_value >= pC->dblValue)
				ok = 1;
			    break;
			};
		  }
	    }
	  if ((cell.type == FREEXL_CELL_TEXT
	       || cell.type == FREEXL_CELL_SST_TEXT
	       || cell.type == FREEXL_CELL_DATE
	       || cell.type == FREEXL_CELL_DATETIME
	       || cell.type == FREEXL_CELL_TIME) && pC->valueType == 'T')
	    {
		int ret = strcmp (cell.value.text_value, pC->txtValue);
		switch (pC->op)
		  {
		  case SQLITE_INDEX_CONSTRAINT_EQ:
		      if (ret == 0)
			  ok = 1;
		      break;
		  case SQLITE_INDEX_CONSTRAINT_GT:
		      if (ret > 0)
			  ok = 1;
		      break;
		  case SQLITE_INDEX_CONSTRAINT_LE:
		      if (ret <= 0)
			  ok = 1;
		      break;
		  case SQLITE_INDEX_CONSTRAINT_LT:
		      if (ret < 0)
			  ok = 1;
		      break;
		  case SQLITE_INDEX_CONSTRAINT_GE:
		      if (ret >= 0)
			  ok = 1;
		      break;
		  };
	    }
	done:
	  if (!ok)
	      return 0;
	  pC = pC->next;
      }
    return 1;
}

static int
vXL_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	    int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    int i;
    int iColumn;
    int op;
    int len;
    VirtualXLConstraintPtr pC;
    VirtualXLCursorPtr cursor = (VirtualXLCursorPtr) pCursor;
    if (idxNum)
	idxNum = idxNum;	/* unused arg warning suppression */

/* resetting any previously set filter constraint */
    vXL_free_constraints (cursor);

    for (i = 0; i < argc; i++)
      {
	  if (!vXL_parse_constraint (idxStr, i, &iColumn, &op))
	      continue;
	  pC = sqlite3_malloc (sizeof (VirtualXLConstraint));
	  if (!pC)
	      continue;
	  pC->iColumn = iColumn;
	  pC->op = op;
	  pC->valueType = '\0';
	  pC->txtValue = NULL;
	  pC->next = NULL;

	  if (sqlite3_value_type (argv[i]) == SQLITE_INTEGER)
	    {
		pC->valueType = 'I';
		pC->intValue = sqlite3_value_int64 (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_FLOAT)
	    {
		pC->valueType = 'D';
		pC->dblValue = sqlite3_value_double (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_TEXT)
	    {
		pC->valueType = 'T';
		len = sqlite3_value_bytes (argv[i]) + 1;
		pC->txtValue = (char *) sqlite3_malloc (len);
		if (pC->txtValue)
		    strcpy (pC->txtValue,
			    (char *) sqlite3_value_text (argv[i]));
	    }
	  if (cursor->firstConstraint == NULL)
	      cursor->firstConstraint = pC;
	  if (cursor->lastConstraint != NULL)
	      cursor->lastConstraint->next = pC;
	  cursor->lastConstraint = pC;
      }

    if (cursor->pVtab->firstLineTitles == 'Y')
	cursor->current_row = 1;
    else
	cursor->current_row = 0;
    cursor->eof = 0;
    while (1)
      {
	  vXL_read_row (cursor);
	  if (cursor->eof)
	      break;
	  if (vXL_eval_constraints (cursor))
	      break;
      }
    return SQLITE_OK;
}

static int
vXL_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching a next row from cursor */
    VirtualXLCursorPtr cursor = (VirtualXLCursorPtr) pCursor;
    while (1)
      {
	  vXL_read_row (cursor);
	  if (cursor->eof)
	      break;
	  if (vXL_eval_constraints (cursor))
	      break;
      }
    return SQLITE_OK;
}

static int
vXL_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    VirtualXLCursorPtr cursor = (VirtualXLCursorPtr) pCursor;
    return cursor->eof;
}

static int
vXL_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	    int column)
{
/* fetching value for the Nth column */
    FreeXL_CellValue cell;
    VirtualXLCursorPtr cursor = (VirtualXLCursorPtr) pCursor;
    if (column == 0)
      {
	  /* the PRIMARY KEY column */
	  if (cursor->pVtab->firstLineTitles == 'Y')
	      sqlite3_result_int (pContext, cursor->current_row - 1);
	  else
	      sqlite3_result_int (pContext, cursor->current_row);
	  return SQLITE_OK;
      }
    if (cursor->pVtab->XL_handle != NULL
	&& cursor->current_row <= cursor->pVtab->rows
	&& column <= cursor->pVtab->columns)
	freexl_get_cell_value (cursor->pVtab->XL_handle,
			       cursor->current_row - 1, column - 1, &cell);
    else
	cell.type = FREEXL_CELL_NULL;
    switch (cell.type)
      {
      case FREEXL_CELL_INT:
	  sqlite3_result_int (pContext, cell.value.int_value);
	  break;
      case FREEXL_CELL_DOUBLE:
	  sqlite3_result_double (pContext, cell.value.double_value);
	  break;
      case FREEXL_CELL_TEXT:
      case FREEXL_CELL_SST_TEXT:
      case FREEXL_CELL_DATE:
      case FREEXL_CELL_DATETIME:
      case FREEXL_CELL_TIME:
	  sqlite3_result_text (pContext, cell.value.text_value,
			       strlen (cell.value.text_value), SQLITE_STATIC);
	  break;
      default:
	  sqlite3_result_null (pContext);
	  break;
      };
    return SQLITE_OK;
}

static int
vXL_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    VirtualXLCursorPtr cursor = (VirtualXLCursorPtr) pCursor;
    if (cursor->pVtab->firstLineTitles == 'Y')
	*pRowid = cursor->current_row - 1;
    else
	*pRowid = cursor->current_row;
    return SQLITE_OK;
}

static int
vXL_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	    sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    if (pVTab || argc || argv || pRowid)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_READONLY;
}

static int
vXL_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vXL_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vXL_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vXL_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3VirtualXLInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    my_XL_module.iVersion = 1;
    my_XL_module.xCreate = &vXL_create;
    my_XL_module.xConnect = &vXL_connect;
    my_XL_module.xBestIndex = &vXL_best_index;
    my_XL_module.xDisconnect = &vXL_disconnect;
    my_XL_module.xDestroy = &vXL_destroy;
    my_XL_module.xOpen = &vXL_open;
    my_XL_module.xClose = &vXL_close;
    my_XL_module.xFilter = &vXL_filter;
    my_XL_module.xNext = &vXL_next;
    my_XL_module.xEof = &vXL_eof;
    my_XL_module.xColumn = &vXL_column;
    my_XL_module.xRowid = &vXL_rowid;
    my_XL_module.xUpdate = &vXL_update;
    my_XL_module.xBegin = &vXL_begin;
    my_XL_module.xSync = &vXL_sync;
    my_XL_module.xCommit = &vXL_commit;
    my_XL_module.xRollback = &vXL_rollback;
    my_XL_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "VirtualXL", &my_XL_module, NULL, 0);
    return rc;
}

int
virtualXL_extension_init (sqlite3 * db)
{
    return sqlite3VirtualXLInit (db);
}

#endif /* FreeXL enabled/disabled */
#endif /* ICONV enabled/disabled */
/**************** End file: virtualXL.c **********/


/**************** Begin file: virtualnetwork.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <math.h> */
/* #include <float.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */

static struct sqlite3_module my_net_module;

#define VNET_DIJKSTRA_ALGORITHM	1
#define VNET_A_STAR_ALGORITHM	2

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

/******************************************************************************
/
/ VirtualNetwork structs
/
******************************************************************************/

typedef struct NetworkArcStruct
{
/* an ARC */
    const struct NetworkNodeStruct *NodeFrom;
    const struct NetworkNodeStruct *NodeTo;
    sqlite3_int64 ArcRowid;
    double Cost;
} NetworkArc;
typedef NetworkArc *NetworkArcPtr;

typedef struct NetworkNodeStruct
{
/* a NODE */
    int InternalIndex;
    sqlite3_int64 Id;
    char *Code;
    double CoordX;
    double CoordY;
    int NumArcs;
    NetworkArcPtr Arcs;
} NetworkNode;
typedef NetworkNode *NetworkNodePtr;

typedef struct NetworkStruct
{
/* the main NETWORK structure */
    int Net64;
    int AStar;
    int EndianArch;
    int MaxCodeLength;
    int CurrentIndex;
    int NodeCode;
    int NumNodes;
    char *TableName;
    char *FromColumn;
    char *ToColumn;
    char *GeometryColumn;
    char *NameColumn;
    double AStarHeuristicCoeff;
    NetworkNodePtr Nodes;
} Network;
typedef Network *NetworkPtr;

typedef struct ArcSolutionStruct
{
/* Geometry corresponding to an Arc used by Dijkstra shortest path solution */
    sqlite3_int64 ArcRowid;
    char *FromCode;
    char *ToCode;
    sqlite3_int64 FromId;
    sqlite3_int64 ToId;
    int Points;
    double *Coords;
    int Srid;
    char *Name;
    struct ArcSolutionStruct *Next;

} ArcSolution;
typedef ArcSolution *ArcSolutionPtr;

typedef struct RowSolutionStruct
{
/* a row into the shortest path solution */
    NetworkArcPtr Arc;
    char *Name;
    struct RowSolutionStruct *Next;

} RowSolution;
typedef RowSolution *RowSolutionPtr;

typedef struct SolutionStruct
{
/* the shortest path solution */
    ArcSolutionPtr FirstArc;
    ArcSolutionPtr LastArc;
    NetworkNodePtr From;
    NetworkNodePtr To;
    RowSolutionPtr First;
    RowSolutionPtr Last;
    RowSolutionPtr CurrentRow;
    sqlite3_int64 CurrentRowId;
    double TotalCost;
    gaiaGeomCollPtr Geometry;
} Solution;
typedef Solution *SolutionPtr;

/******************************************************************************
/
/ Dijkstra and A* common structs
/
******************************************************************************/

typedef struct RoutingNode
{
    int Id;
    struct RoutingNode **To;
    NetworkArcPtr *Link;
    int DimTo;
    struct RoutingNode *PreviousNode;
    NetworkArcPtr Arc;
    double Distance;
    double HeuristicDistance;
    int Inspected;
} RoutingNode;
typedef RoutingNode *RoutingNodePtr;

typedef struct RoutingNodes
{
    RoutingNodePtr Nodes;
    NetworkArcPtr *ArcsBuffer;
    RoutingNodePtr *NodesBuffer;
    int Dim;
    int DimLink;
} RoutingNodes;
typedef RoutingNodes *RoutingNodesPtr;

typedef struct RoutingHeapStruct
{
    RoutingNodePtr *Values;
    int Head;
    int Tail;
} RoutingHeap;
typedef RoutingHeap *RoutingHeapPtr;

/******************************************************************************
/
/ VirtualTable structs
/
******************************************************************************/

typedef struct VirtualNetworkStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
    NetworkPtr graph;		/* the NETWORK structure */
    RoutingNodesPtr routing;	/* the ROUTING structure */
    int currentAlgorithm;	/* the currently selected Shortest Path Algorithm */
} VirtualNetwork;
typedef VirtualNetwork *VirtualNetworkPtr;

typedef struct VirtualNetworkCursortStruct
{
/* extends the sqlite3_vtab_cursor struct */
    VirtualNetworkPtr pVtab;	/* Virtual table of this cursor */
    SolutionPtr solution;	/* the current solution */
    int eof;			/* the EOF marker */
} VirtualNetworkCursor;
typedef VirtualNetworkCursor *VirtualNetworkCursorPtr;

/*
/
/  implementation of the Dijkstra Shortest Path algorithm
/
////////////////////////////////////////////////////////////
/
/ Author: Luigi Costalli luigi.costalli@gmail.com
/ version 1.0. 2008 October 21
/
*/

static RoutingNodesPtr
routing_init (NetworkPtr graph)
{
/* allocating and initializing the ROUTING struct */
    int i;
    int j;
    int cnt = 0;
    RoutingNodesPtr nd;
    RoutingNodePtr ndn;
    NetworkNodePtr nn;
/* allocating the main Nodes struct */
    nd = malloc (sizeof (RoutingNodes));
/* allocating and initializing  Nodes array */
    nd->Nodes = malloc (sizeof (RoutingNode) * graph->NumNodes);
    nd->Dim = graph->NumNodes;
    nd->DimLink = 0;

/* pre-alloc buffer strategy - GENSCHER 2010-01-05 */
    for (i = 0; i < graph->NumNodes; cnt += graph->Nodes[i].NumArcs, i++);
    nd->NodesBuffer = malloc (sizeof (RoutingNodePtr) * cnt);
    nd->ArcsBuffer = malloc (sizeof (NetworkArcPtr) * cnt);

    cnt = 0;
    for (i = 0; i < graph->NumNodes; i++)
      {
	  /* initializing the Nodes array */
	  nn = graph->Nodes + i;
	  ndn = nd->Nodes + i;
	  ndn->Id = nn->InternalIndex;
	  ndn->DimTo = nn->NumArcs;
	  ndn->To = &(nd->NodesBuffer[cnt]);
	  ndn->Link = &(nd->ArcsBuffer[cnt]);
	  cnt += nn->NumArcs;

	  for (j = 0; j < nn->NumArcs; j++)
	    {
		/*  setting the outcoming Arcs for the current Node */
		nd->DimLink++;
		ndn->To[j] = nd->Nodes + nn->Arcs[j].NodeTo->InternalIndex;
		ndn->Link[j] = nn->Arcs + j;
	    }
      }
    return (nd);
}

static void
routing_free (RoutingNodes * e)
{
/* memory cleanup; freeing the ROUTING struct */
    free (e->ArcsBuffer);
    free (e->NodesBuffer);
    free (e->Nodes);
    free (e);
}

static RoutingHeapPtr
routing_heap_init (int dim)
{
/* allocating the Nodes ordered list */
    RoutingHeapPtr h;
    h = malloc (sizeof (RoutingHeap));
    h->Values = malloc (sizeof (RoutingNodePtr) * dim);
    h->Head = 0;
    h->Tail = 0;
    return (h);
}

static void
routing_heap_free (RoutingHeapPtr h)
{
/* freeing the Nodes ordered list */
    free (h->Values);
    free (h);
}

static void
routing_push (RoutingHeapPtr h, RoutingNodePtr n)
{
/* inserting a Node into the list */
    h->Values[h->Tail] = n;
    h->Tail++;
}

static RoutingNodePtr
dijkstra_pop (RoutingHeapPtr h)
{
/* fetching the minimum value */
    int i;
    RoutingNodePtr n;
    double min = DBL_MAX;
    int i_min = h->Head;
    for (i = h->Head; i < h->Tail; i++)
      {
	  n = h->Values[i];
	  if (n->Distance < min)
	    {
		min = n->Distance;
		i_min = i;
	    }
      }
    if (i_min > h->Head)
      {
	  n = h->Values[i_min];
	  h->Values[i_min] = h->Values[h->Head];
	  h->Values[h->Head] = n;
      }
    n = h->Values[h->Head];
    h->Head++;
    return (n);
}

static NetworkArcPtr *
dijkstra_shortest_path (RoutingNodesPtr e, NetworkNodePtr pfrom,
			NetworkNodePtr pto, int *ll)
{
/* identifying the Shortest Path - Dijkstra's algorithm */
    int from;
    int to;
    int i;
    int k;
    RoutingNodePtr n;
    RoutingNodePtr p_to;
    NetworkArcPtr p_link;
    int cnt;
    NetworkArcPtr *result;
    RoutingHeapPtr h;
/* setting From/To */
    from = pfrom->InternalIndex;
    to = pto->InternalIndex;
/* initializing the heap */
    h = routing_heap_init (e->DimLink);
/* initializing the graph */
    for (i = 0; i < e->Dim; i++)
      {
	  n = e->Nodes + i;
	  n->PreviousNode = NULL;
	  n->Arc = NULL;
	  n->Inspected = 0;
	  n->Distance = DBL_MAX;
      }
/* pushes the From node into the Nodes list */
    e->Nodes[from].Distance = 0.0;
    routing_push (h, e->Nodes + from);
    while (h->Tail != h->Head)
      {
	  /* Dijsktra loop */
	  n = dijkstra_pop (h);
	  if (n->Id == to)
	    {
		/* destination reached */
		break;
	    }
	  n->Inspected = 1;
	  for (i = 0; i < n->DimTo; i++)
	    {
		p_to = *(n->To + i);
		p_link = *(n->Link + i);
		if (p_to->Inspected == 0)
		  {
		      if (p_to->Distance == DBL_MAX)
			{
			    /* inserting a new node into the list */
			    p_to->Distance = n->Distance + p_link->Cost;
			    p_to->PreviousNode = n;
			    p_to->Arc = p_link;
			    routing_push (h, p_to);
			}
		      else if (p_to->Distance > n->Distance + p_link->Cost)
			{
			    /* updating an already inserted node */
			    p_to->Distance = n->Distance + p_link->Cost;
			    p_to->PreviousNode = n;
			    p_to->Arc = p_link;
			}
		  }
	    }
      }
    routing_heap_free (h);
    cnt = 0;
    n = e->Nodes + to;
    while (n->PreviousNode != NULL)
      {
	  /* counting how many Arcs are into the Shortest Path solution */
	  cnt++;
	  n = n->PreviousNode;
      }
/* allocating the solution */
    result = malloc (sizeof (NetworkArcPtr) * cnt);
    k = cnt - 1;
    n = e->Nodes + to;
    while (n->PreviousNode != NULL)
      {
	  /* inserting an Arc  into the solution */
	  result[k] = n->Arc;
	  n = n->PreviousNode;
	  k--;
      }
    *ll = cnt;
    return (result);
}

/* END of Luigi Costalli Dijkstra Shortest Path implementation */

/*
/
/  implementation of the A* Shortest Path algorithm
/
*/

static RoutingNodePtr
a_star_pop (RoutingHeapPtr h)
{
/* fetching the minimum value */
    int i;
    RoutingNodePtr n;
    double min = DBL_MAX;
    int i_min = h->Head;
    for (i = h->Head; i < h->Tail; i++)
      {
	  n = h->Values[i];
	  if (n->HeuristicDistance < min)
	    {
		min = n->HeuristicDistance;
		i_min = i;
	    }
      }
    if (i_min > h->Head)
      {
	  n = h->Values[i_min];
	  h->Values[i_min] = h->Values[h->Head];
	  h->Values[h->Head] = n;
      }
    n = h->Values[h->Head];
    h->Head++;
    return (n);
}

static double
a_star_heuristic_distance (NetworkNodePtr n1, NetworkNodePtr n2, double coeff)
{
/* computing the euclidean distance intercurring between two nodes */
    double dx = n1->CoordX - n2->CoordX;
    double dy = n1->CoordY - n2->CoordY;
    double dist = sqrt ((dx * dx) + (dy * dy)) * coeff;
    return dist;
}

static NetworkArcPtr *
a_star_shortest_path (RoutingNodesPtr e, NetworkNodePtr nodes,
		      NetworkNodePtr pfrom, NetworkNodePtr pto,
		      double heuristic_coeff, int *ll)
{
/* identifying the Shortest Path - A* algorithm */
    int from;
    int to;
    int i;
    int k;
    RoutingNodePtr pAux;
    RoutingNodePtr n;
    RoutingNodePtr p_to;
    NetworkNodePtr pOrg;
    NetworkNodePtr pDest;
    NetworkArcPtr p_link;
    int cnt;
    NetworkArcPtr *result;
    RoutingHeapPtr h;
/* setting From/To */
    from = pfrom->InternalIndex;
    to = pto->InternalIndex;
    pAux = e->Nodes + from;
    pOrg = nodes + pAux->Id;
    pAux = e->Nodes + to;
    pDest = nodes + pAux->Id;
/* initializing the heap */
    h = routing_heap_init (e->DimLink);
/* initializing the graph */
    for (i = 0; i < e->Dim; i++)
      {
	  n = e->Nodes + i;
	  n->PreviousNode = NULL;
	  n->Arc = NULL;
	  n->Inspected = 0;
	  n->Distance = DBL_MAX;
	  n->HeuristicDistance = DBL_MAX;
      }
/* pushes the From node into the Nodes list */
    e->Nodes[from].Distance = 0.0;
    e->Nodes[from].HeuristicDistance =
	a_star_heuristic_distance (pOrg, pDest, heuristic_coeff);
    routing_push (h, e->Nodes + from);
    while (h->Tail != h->Head)
      {
	  /* A* loop */
	  n = a_star_pop (h);
	  if (n->Id == to)
	    {
		/* destination reached */
		break;
	    }
	  n->Inspected = 1;
	  for (i = 0; i < n->DimTo; i++)
	    {
		p_to = *(n->To + i);
		p_link = *(n->Link + i);
		if (p_to->Inspected == 0)
		  {
		      if (p_to->Distance == DBL_MAX)
			{
			    /* inserting a new node into the list */
			    p_to->Distance = n->Distance + p_link->Cost;
			    pOrg = nodes + p_to->Id;
			    p_to->HeuristicDistance =
				p_to->Distance +
				a_star_heuristic_distance (pOrg, pDest,
							   heuristic_coeff);
			    p_to->PreviousNode = n;
			    p_to->Arc = p_link;
			    routing_push (h, p_to);
			}
		      else if (p_to->Distance > n->Distance + p_link->Cost)
			{
			    /* updating an already inserted node */
			    p_to->Distance = n->Distance + p_link->Cost;
			    pOrg = nodes + p_to->Id;
			    p_to->HeuristicDistance =
				p_to->Distance +
				a_star_heuristic_distance (pOrg, pDest,
							   heuristic_coeff);
			    p_to->PreviousNode = n;
			    p_to->Arc = p_link;
			}
		  }
	    }
      }
    routing_heap_free (h);
    cnt = 0;
    n = e->Nodes + to;
    while (n->PreviousNode != NULL)
      {
	  /* counting how many Arcs are into the Shortest Path solution */
	  cnt++;
	  n = n->PreviousNode;
      }
/* allocating the solution */
    result = malloc (sizeof (NetworkArcPtr) * cnt);
    k = cnt - 1;
    n = e->Nodes + to;
    while (n->PreviousNode != NULL)
      {
	  /* inserting an Arc  into the solution */
	  result[k] = n->Arc;
	  n = n->PreviousNode;
	  k--;
      }
    *ll = cnt;
    return (result);
}

/* END of A* Shortest Path implementation */

static void
vnet_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

static void
vnet_dequote (char *buf)
{
/* dequoting an SQL string */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    char strip = '\0';
    int first = 0;
    int len = strlen (buf);
    if (buf[0] == '\'' && buf[len - 1] == '\'')
	strip = '\'';
    if (buf[0] == '"' && buf[len - 1] == '"')
	strip = '"';
    if (strip == '\0')
	return;
    strcpy (tmp, buf + 1);
    len = strlen (tmp);
    tmp[len - 1] = '\0';
    while (*in != '\0')
      {
	  if (*in == strip)
	    {
		if (first)
		  {
		      first = 0;
		      in++;
		      continue;
		  }
		else
		  {
		      first = 1;
		      *out++ = *in++;
		      continue;
		  }
	    }
	  first = 0;
	  *out++ = *in++;
      }
    *out = '\0';
}

static int
cmp_nodes_code (const void *p1, const void *p2)
{
/* compares two nodes  by CODE [for BSEARCH] */
    NetworkNodePtr pN1 = (NetworkNodePtr) p1;
    NetworkNodePtr pN2 = (NetworkNodePtr) p2;
    return strcmp (pN1->Code, pN2->Code);
}

static int
cmp_nodes_id (const void *p1, const void *p2)
{
/* compares two nodes  by ID [for BSEARCH] */
    NetworkNodePtr pN1 = (NetworkNodePtr) p1;
    NetworkNodePtr pN2 = (NetworkNodePtr) p2;
    if (pN1->Id == pN2->Id)
	return 0;
    if (pN1->Id > pN2->Id)
	return 1;
    return -1;
}

static NetworkNodePtr
find_node_by_code (NetworkPtr graph, const char *code)
{
/* searching a Node (by Code) into the sorted list */
    NetworkNodePtr ret;
    NetworkNode pN;
    pN.Code = (char *) code;
    ret =
	bsearch (&pN, graph->Nodes, graph->NumNodes, sizeof (NetworkNode),
		 cmp_nodes_code);
    return ret;
}

static NetworkNodePtr
find_node_by_id (NetworkPtr graph, const sqlite3_int64 id)
{
/* searching a Node (by Id) into the sorted list */
    NetworkNodePtr ret;
    NetworkNode pN;
    pN.Id = id;
    ret =
	bsearch (&pN, graph->Nodes, graph->NumNodes, sizeof (NetworkNode),
		 cmp_nodes_id);
    return ret;
}

static void
delete_solution (SolutionPtr solution)
{
/* deleting the current solution */
    ArcSolutionPtr pA;
    ArcSolutionPtr pAn;
    RowSolutionPtr pR;
    RowSolutionPtr pRn;
    if (!solution)
	return;
    pA = solution->FirstArc;
    while (pA)
      {
	  pAn = pA->Next;
	  if (pA->FromCode)
	      free (pA->FromCode);
	  if (pA->ToCode)
	      free (pA->ToCode);
	  if (pA->Coords)
	      free (pA->Coords);
	  if (pA->Name)
	      free (pA->Name);
	  free (pA);
	  pA = pAn;
      }
    pR = solution->First;
    while (pR)
      {
	  pRn = pR->Next;
	  if (pR->Name)
	      free (pR->Name);
	  free (pR);
	  pR = pRn;
      }
    if (solution->Geometry)
	gaiaFreeGeomColl (solution->Geometry);
    free (solution);
}

static void
reset_solution (SolutionPtr solution)
{
/* resetting the current solution */
    ArcSolutionPtr pA;
    ArcSolutionPtr pAn;
    RowSolutionPtr pR;
    RowSolutionPtr pRn;
    if (!solution)
	return;
    pA = solution->FirstArc;
    while (pA)
      {
	  pAn = pA->Next;
	  if (pA->FromCode)
	      free (pA->FromCode);
	  if (pA->ToCode)
	      free (pA->ToCode);
	  if (pA->Coords)
	      free (pA->Coords);
	  free (pA);
	  pA = pAn;
      }
    pR = solution->First;
    while (pR)
      {
	  pRn = pR->Next;
	  if (pR->Name)
	      free (pR->Name);
	  free (pR);
	  pR = pRn;
      }
    if (solution->Geometry)
	gaiaFreeGeomColl (solution->Geometry);
    solution->FirstArc = NULL;
    solution->LastArc = NULL;
    solution->From = NULL;
    solution->To = NULL;
    solution->First = NULL;
    solution->Last = NULL;
    solution->CurrentRow = NULL;
    solution->CurrentRowId = 0;
    solution->TotalCost = 0.0;
    solution->Geometry = NULL;
}

static SolutionPtr
alloc_solution (void)
{
/* allocates and initializes the current solution */
    SolutionPtr p = malloc (sizeof (Solution));
    p->FirstArc = NULL;
    p->LastArc = NULL;
    p->From = NULL;
    p->To = NULL;
    p->First = NULL;
    p->Last = NULL;
    p->CurrentRow = NULL;
    p->CurrentRowId = 0;
    p->TotalCost = 0.0;
    p->Geometry = NULL;
    return p;
}

static void
add_arc_to_solution (SolutionPtr solution, NetworkArcPtr arc)
{
/* inserts an Arc into the Shortest Path solution */
    RowSolutionPtr p = malloc (sizeof (RowSolution));
    p->Arc = arc;
    p->Name = NULL;
    p->Next = NULL;
    solution->TotalCost += arc->Cost;
    if (!(solution->First))
	solution->First = p;
    if (solution->Last)
	solution->Last->Next = p;
    solution->Last = p;
}

static void
add_arc_geometry_to_solution (SolutionPtr solution, sqlite3_int64 arc_id,
			      const char *from_code, const char *to_code,
			      sqlite3_int64 from_id, sqlite3_int64 to_id,
			      int points, double *coords, int srid,
			      const char *name)
{
/* inserts an Arc Geometry into the Shortest Path solution */
    int len;
    ArcSolutionPtr p = malloc (sizeof (ArcSolution));
    p->ArcRowid = arc_id;
    p->FromCode = NULL;
    len = strlen (from_code);
    if (len > 0)
      {
	  p->FromCode = malloc (len + 1);
	  strcpy (p->FromCode, from_code);
      }
    p->ToCode = NULL;
    len = strlen (to_code);
    if (len > 0)
      {
	  p->ToCode = malloc (len + 1);
	  strcpy (p->ToCode, to_code);
      }
    p->FromId = from_id;
    p->ToId = to_id;
    p->Points = points;
    p->Coords = coords;
    p->Srid = srid;
    if (!name)
	p->Name = NULL;
    else
      {
	  len = strlen (name);
	  p->Name = malloc (len + 1);
	  strcpy (p->Name, name);
      }
    p->Next = NULL;
    if (!(solution->FirstArc))
	solution->FirstArc = p;
    if (solution->LastArc)
	solution->LastArc->Next = p;
    solution->LastArc = p;
}

static void
build_solution (sqlite3 * handle, NetworkPtr graph, SolutionPtr solution,
		NetworkArcPtr * shortest_path, int cnt)
{
/* formatting the Shortest Path solution */
    int i;
    char sql[8192];
    int err;
    int error = 0;
    int ret;
    sqlite3_int64 arc_id;
    const unsigned char *blob;
    int size;
    sqlite3_int64 from_id;
    sqlite3_int64 to_id;
    char from_code[128];
    char to_code[128];
    char name[2048];
    int tbd;
    int ind;
    int base = 0;
    int block = 128;
    int how_many;
    sqlite3_stmt *stmt;
    char xfrom[1024];
    char xto[1024];
    char xgeom[1024];
    char xname[1024];
    char xtable[1024];
    if (cnt > 0)
      {
	  /* building the solution */
	  for (i = 0; i < cnt; i++)
	    {
		add_arc_to_solution (solution, shortest_path[i]);
	    }
      }
    tbd = cnt;
    while (tbd > 0)
      {
	  /* requesting max 128 arcs at each time */
	  if (tbd < block)
	      how_many = tbd;
	  else
	      how_many = block;
/* preparing the Geometry representing this solution [reading arcs] */
	  if (graph->NameColumn)
	    {
		/* a Name column is defined */
		strcpy (xfrom, graph->FromColumn);
		vnet_double_quoted_sql (xfrom);
		strcpy (xto, graph->ToColumn);
		vnet_double_quoted_sql (xto);
		strcpy (xgeom, graph->GeometryColumn);
		vnet_double_quoted_sql (xgeom);
		strcpy (xname, graph->NameColumn);
		vnet_double_quoted_sql (xname);
		strcpy (xtable, graph->TableName);
		vnet_double_quoted_sql (xtable);
		sprintf (sql,
			 "SELECT ROWID, %s, %s, %s, %s FROM %s WHERE ROWID IN (",
			 xfrom, xto, xgeom, xname, xtable);
	    }
	  else
	    {
		/* no Name column is defined */
		strcpy (xfrom, graph->FromColumn);
		vnet_double_quoted_sql (xfrom);
		strcpy (xto, graph->ToColumn);
		vnet_double_quoted_sql (xto);
		strcpy (xgeom, graph->GeometryColumn);
		vnet_double_quoted_sql (xgeom);
		strcpy (xtable, graph->TableName);
		vnet_double_quoted_sql (xtable);
		sprintf (sql,
			 "SELECT ROWID, %s, %s, %s FROM %s WHERE ROWID IN (",
			 xfrom, xto, xgeom, xtable);
	    }
	  for (i = 0; i < how_many; i++)
	    {
		if (i == 0)
		    strcat (sql, "?");
		else
		    strcat (sql, ",?");
	    }
	  strcat (sql, ")");
	  ret = sqlite3_prepare_v2 (handle, sql, strlen (sql), &stmt, NULL);
	  if (ret != SQLITE_OK)
	    {
		error = 1;
		goto abort;
	    }
	  sqlite3_reset (stmt);
	  sqlite3_clear_bindings (stmt);
	  ind = 1;
	  for (i = 0; i < cnt; i++)
	    {
		if (i < base)
		    continue;
		if (i >= (base + how_many))
		    break;
		sqlite3_bind_int64 (stmt, ind, shortest_path[i]->ArcRowid);
		ind++;
	    }
	  while (1)
	    {
		ret = sqlite3_step (stmt);
		if (ret == SQLITE_DONE)
		    break;
		if (ret == SQLITE_ROW)
		  {
		      arc_id = -1;
		      from_id = -1;
		      to_id = -1;
		      *from_code = '\0';
		      *to_code = '\0';
		      blob = NULL;
		      size = 0;
		      *name = '\0';
		      err = 0;
		      if (sqlite3_column_type (stmt, 0) == SQLITE_INTEGER)
			  arc_id = sqlite3_column_int64 (stmt, 0);
		      else
			  err = 1;
		      if (graph->NodeCode)
			{
			    /* nodes are identified by TEXT codes */
			    if (sqlite3_column_type (stmt, 1) == SQLITE_TEXT)
				strcpy (from_code,
					(char *) sqlite3_column_text (stmt, 1));
			    else
				err = 1;
			    if (sqlite3_column_type (stmt, 2) == SQLITE_TEXT)
				strcpy (to_code,
					(char *) sqlite3_column_text (stmt, 2));
			    else
				err = 1;
			}
		      else
			{
			    /* nodes are identified by INTEGER ids */
			    if (sqlite3_column_type (stmt, 1) == SQLITE_INTEGER)
				from_id = sqlite3_column_int64 (stmt, 1);
			    else
				err = 1;
			    if (sqlite3_column_type (stmt, 2) == SQLITE_INTEGER)
				to_id = sqlite3_column_int64 (stmt, 2);
			    else
				err = 1;
			}
		      if (sqlite3_column_type (stmt, 3) == SQLITE_BLOB)
			{
			    blob =
				(const unsigned char *)
				sqlite3_column_blob (stmt, 3);
			    size = sqlite3_column_bytes (stmt, 3);
			}
		      else
			  err = 1;
		      if (graph->NameColumn)
			{
			    if (sqlite3_column_type (stmt, 4) == SQLITE_TEXT)
				strcpy (name,
					(char *) sqlite3_column_text (stmt, 4));
			}
		      if (err)
			  error = 1;
		      else
			{
			    /* saving the Arc geometry into the temporary struct */
			    gaiaGeomCollPtr geom =
				gaiaFromSpatiaLiteBlobWkb (blob, size);
			    if (geom)
			      {
				  /* OK, we have fetched a valid Geometry */
				  if (geom->FirstPoint == NULL
				      && geom->FirstPolygon == NULL
				      && geom->FirstLinestring != NULL
				      && geom->FirstLinestring ==
				      geom->LastLinestring)
				    {
					/* Geometry is LINESTRING as expected */
					int iv;
					int points =
					    geom->FirstLinestring->Points;
					double *coords =
					    malloc (sizeof (double) *
						    (points * 2));
					for (iv = 0; iv < points; iv++)
					  {
					      double x;
					      double y;
					      gaiaGetPoint
						  (geom->FirstLinestring->Coords,
						   iv, &x, &y);
					      *(coords + ((iv * 2) + 0)) = x;
					      *(coords + ((iv * 2) + 1)) = y;
					  }
					add_arc_geometry_to_solution (solution,
								      arc_id,
								      from_code,
								      to_code,
								      from_id,
								      to_id,
								      points,
								      coords,
								      geom->Srid,
								      name);
				    }
				  else
				      error = 1;
				  gaiaFreeGeomColl (geom);
			      }
			    else
				error = 1;
			}
		  }
	    }
	  sqlite3_finalize (stmt);
	  tbd -= how_many;
	  base += how_many;
      }
  abort:
    if (shortest_path)
	free (shortest_path);
    if (!error)
      {
	  /* building the Geometry representing the Shortest Path Solution */
	  gaiaLinestringPtr ln;
	  int tot_pts = 0;
	  RowSolutionPtr pR;
	  ArcSolutionPtr pA;
	  int srid = -1;
	  if (solution->FirstArc)
	      srid = (solution->FirstArc)->Srid;
	  pR = solution->First;
	  while (pR)
	    {
		pA = solution->FirstArc;
		while (pA)
		  {
		      /* computing how many vertices do we need to build the LINESTRING */
		      if (pR->Arc->ArcRowid == pA->ArcRowid)
			{
			    if (pR == solution->First)
				tot_pts += pA->Points;
			    else
				tot_pts += (pA->Points - 1);
			    if (pA->Srid != srid)
				srid = -1;
			}
		      pA = pA->Next;
		  }
		pR = pR->Next;
	    }
	  /* creating the Shortest Path Geometry - LINESTRING */
	  ln = gaiaAllocLinestring (tot_pts);
	  solution->Geometry = gaiaAllocGeomColl ();
	  solution->Geometry->Srid = srid;
	  gaiaInsertLinestringInGeomColl (solution->Geometry, ln);
	  tot_pts = 0;
	  pR = solution->First;
	  while (pR)
	    {
		/* building the LINESTRING */
		int skip;
		if (pR == solution->First)
		    skip = 0;	/* for first arc we must copy any vertex */
		else
		    skip = 1;	/* for subsequent arcs we must skip first vertex [already inserted from previous arc] */
		pA = solution->FirstArc;
		while (pA)
		  {
		      if (pR->Arc->ArcRowid == pA->ArcRowid)
			{
			    /* copying vertices from correspoinding Arc Geometry */
			    int ini;
			    int iv;
			    int rev;
			    double x;
			    double y;
			    if (graph->NodeCode)
			      {
				  /* nodes are identified by TEXT codes */
				  if (strcmp
				      (pR->Arc->NodeFrom->Code,
				       pA->ToCode) == 0)
				      rev = 1;
				  else
				      rev = 0;
			      }
			    else
			      {
				  /* nodes are identified by INTEGER ids */
				  if (pR->Arc->NodeFrom->Id == pA->ToId)
				      rev = 1;
				  else
				      rev = 0;
			      }
			    if (rev)
			      {
				  /* copying Arc vertices in reverse order */
				  if (skip)
				      ini = pA->Points - 2;
				  else
				      ini = pA->Points - 1;
				  for (iv = ini; iv >= 0; iv--)
				    {
					x = *(pA->Coords + ((iv * 2) + 0));
					y = *(pA->Coords + ((iv * 2) + 1));
					gaiaSetPoint (ln->Coords, tot_pts, x,
						      y);
					tot_pts++;
				    }
			      }
			    else
			      {
				  /* copying Arc vertices in normal order */
				  if (skip)
				      ini = 1;
				  else
				      ini = 0;
				  for (iv = ini; iv < pA->Points; iv++)
				    {
					x = *(pA->Coords + ((iv * 2) + 0));
					y = *(pA->Coords + ((iv * 2) + 1));
					gaiaSetPoint (ln->Coords, tot_pts, x,
						      y);
					tot_pts++;
				    }
			      }
			    if (pA->Name)
			      {
				  int len = strlen (pA->Name);
				  pR->Name = malloc (len + 1);
				  strcpy (pR->Name, pA->Name);
			      }
			    break;
			}
		      pA = pA->Next;
		  }
		pR = pR->Next;
	    }
      }
}

static void
dijkstra_solve (sqlite3 * handle, NetworkPtr graph, RoutingNodesPtr routing,
		SolutionPtr solution)
{
/* computing a Dijkstra Shortest Path solution */
    int cnt;
    NetworkArcPtr *shortest_path =
	dijkstra_shortest_path (routing, solution->From, solution->To, &cnt);
    build_solution (handle, graph, solution, shortest_path, cnt);
}

static void
a_star_solve (sqlite3 * handle, NetworkPtr graph, RoutingNodesPtr routing,
	      SolutionPtr solution)
{
/* computing an A* Shortest Path solution */
    int cnt;
    NetworkArcPtr *shortest_path =
	a_star_shortest_path (routing, graph->Nodes, solution->From,
			      solution->To, graph->AStarHeuristicCoeff, &cnt);
    build_solution (handle, graph, solution, shortest_path, cnt);
}

static void
network_free (NetworkPtr p)
{
/* memory cleanup; freeing any allocation for the network struct */
    NetworkNodePtr pN;
    int i;
    if (!p)
	return;
    for (i = 0; i < p->NumNodes; i++)
      {
	  pN = p->Nodes + i;
	  if (pN->Code)
	      free (pN->Code);
	  if (pN->Arcs)
	      free (pN->Arcs);
      }
    if (p->TableName)
	free (p->TableName);
    if (p->FromColumn)
	free (p->FromColumn);
    if (p->ToColumn)
	free (p->ToColumn);
    if (p->GeometryColumn)
	free (p->GeometryColumn);
    if (p->NameColumn)
	free (p->NameColumn);
    free (p);
}

static NetworkPtr
network_init (const unsigned char *blob, int size)
{
/* parsing the HEADER block */
    NetworkPtr graph;
    int net64;
    int aStar = 0;
    int nodes;
    int node_code;
    int max_code_length;
    int endian_arch = gaiaEndianArch ();
    const char *table;
    const char *from;
    const char *to;
    const char *geom;
    const char *name = NULL;
    double a_star_coeff = 1.0;
    int len;
    const unsigned char *ptr;
    if (size < 9)
	return NULL;
    if (*(blob + 0) == GAIA_NET_START)	/* signature - legacy format using 32bit ints */
	net64 = 0;
    else if (*(blob + 0) == GAIA_NET64_START)	/* signature - format using 64bit ints */
	net64 = 1;
    else if (*(blob + 0) == GAIA_NET64_A_STAR_START)	/* signature - format using 64bit ints AND supporting A* */
      {
	  net64 = 1;
	  aStar = 1;
      }
    else
	return NULL;
    if (*(blob + 1) != GAIA_NET_HEADER)	/* signature */
	return NULL;
    nodes = gaiaImport32 (blob + 2, 1, endian_arch);	/* # nodes */
    if (nodes <= 0)
	return NULL;
    if (*(blob + 6) == GAIA_NET_CODE)	/* Nodes identified by a TEXT code */
	node_code = 1;
    else if (*(blob + 6) == GAIA_NET_ID)	/* Nodes indentified by an INTEGER id */
	node_code = 0;
    else
	return NULL;
    max_code_length = *(blob + 7);	/* Max TEXT Code length */
    if (*(blob + 8) != GAIA_NET_TABLE)	/* signature for TABLE NAME */
	return NULL;
    ptr = blob + 9;
    len = gaiaImport16 (ptr, 1, endian_arch);	/* TABLE NAME is varlen */
    ptr += 2;
    table = (char *) ptr;
    ptr += len;
    if (*ptr != GAIA_NET_FROM)	/* signature for FromNode COLUMN */
	return NULL;
    ptr++;
    len = gaiaImport16 (ptr, 1, endian_arch);	/* FromNode COLUMN is varlen */
    ptr += 2;
    from = (char *) ptr;
    ptr += len;
    if (*ptr != GAIA_NET_TO)	/* signature for ToNode COLUMN */
	return NULL;
    ptr++;
    len = gaiaImport16 (ptr, 1, endian_arch);	/* ToNode COLUMN is varlen */
    ptr += 2;
    to = (char *) ptr;
    ptr += len;
    if (*ptr != GAIA_NET_GEOM)	/* signature for Geometry COLUMN */
	return NULL;
    ptr++;
    len = gaiaImport16 (ptr, 1, endian_arch);	/* Geometry COLUMN is varlen */
    ptr += 2;
    geom = (char *) ptr;
    ptr += len;
    if (net64)
      {
	  if (*ptr != GAIA_NET_NAME)	/* signature for Name COLUMN - may be empty */
	      return NULL;
	  ptr++;
	  len = gaiaImport16 (ptr, 1, endian_arch);	/* Name COLUMN is varlen */
	  ptr += 2;
	  name = (char *) ptr;
	  ptr += len;
      }
    if (net64 && aStar)
      {
	  if (*ptr != GAIA_NET_A_STAR_COEFF)	/* signature for A* Heuristic Coeff */
	      return NULL;
	  ptr++;
	  a_star_coeff = gaiaImport64 (ptr, 1, endian_arch);
	  ptr += 8;
      }
    if (*ptr != GAIA_NET_END)	/* signature */
	return NULL;
    graph = malloc (sizeof (Network));
    graph->Net64 = net64;
    graph->AStar = aStar;
    graph->EndianArch = endian_arch;
    graph->CurrentIndex = 0;
    graph->NodeCode = node_code;
    graph->MaxCodeLength = max_code_length;
    graph->NumNodes = nodes;
    graph->Nodes = malloc (sizeof (NetworkNode) * nodes);
    len = strlen (table);
    graph->TableName = malloc (len + 1);
    strcpy (graph->TableName, table);
    len = strlen (from);
    graph->FromColumn = malloc (len + 1);
    strcpy (graph->FromColumn, from);
    len = strlen (to);
    graph->ToColumn = malloc (len + 1);
    strcpy (graph->ToColumn, to);
    len = strlen (geom);
    graph->GeometryColumn = malloc (len + 1);
    strcpy (graph->GeometryColumn, geom);
    if (!net64)
      {
	  /* Name column is not supported */
	  graph->NameColumn = NULL;
      }
    else
      {
	  len = strlen (name);
	  if (len <= 1)
	      graph->NameColumn = NULL;
	  else
	    {
		graph->NameColumn = malloc (len + 1);
		strcpy (graph->NameColumn, name);
	    }
      }
    graph->AStarHeuristicCoeff = a_star_coeff;
    return graph;
}

static int
network_block (NetworkPtr graph, const unsigned char *blob, int size)
{
/* parsing a NETWORK Block */
    const unsigned char *in = blob;
    int nodes;
    int i;
    int ia;
    int index;
    char code[256];
    double x;
    double y;
    sqlite3_int64 nodeId = -1;
    int arcs;
    NetworkNodePtr pN;
    NetworkArcPtr pA;
    int len;
    sqlite3_int64 arcId;
    int nodeToIdx;
    double cost;
    if (size < 3)
	goto error;
    if (*in++ != GAIA_NET_BLOCK)	/* signature */
	goto error;
    nodes = gaiaImport16 (in, 1, graph->EndianArch);	/* # Nodes */
    in += 2;
    for (i = 0; i < nodes; i++)
      {
	  /* parsing each node */
	  if ((size - (in - blob)) < 5)
	      goto error;
	  if (*in++ != GAIA_NET_NODE)	/* signature */
	      goto error;
	  index = gaiaImport32 (in, 1, graph->EndianArch);	/* node internal index */
	  in += 4;
	  if (index < 0 || index >= graph->NumNodes)
	      goto error;
	  if (graph->NodeCode)
	    {
		/* Nodes are identified by a TEXT Code */
		if ((size - (in - blob)) < graph->MaxCodeLength)
		    goto error;
		memcpy (code, in, graph->MaxCodeLength);
		in += graph->MaxCodeLength;
	    }
	  else
	    {
		/* Nodes are identified by an INTEGER Id */
		if (graph->Net64)
		  {
		      if ((size - (in - blob)) < 8)
			  goto error;
		      nodeId = gaiaImportI64 (in, 1, graph->EndianArch);	/* the Node ID: 64bit */
		      in += 8;
		  }
		else
		  {
		      if ((size - (in - blob)) < 4)
			  goto error;
		      nodeId = gaiaImport32 (in, 1, graph->EndianArch);	/* the Node ID: 32bit */
		      in += 4;
		  }
	    }
	  if (graph->AStar)
	    {
		/* fetching node's X,Y coords */
		if ((size - (in - blob)) < 8)
		    goto error;
		x = gaiaImport64 (in, 1, graph->EndianArch);	/* X coord */
		in += 8;
		if ((size - (in - blob)) < 8)
		    goto error;
		y = gaiaImport64 (in, 1, graph->EndianArch);	/* Y coord */
		in += 8;
	    }
	  else
	    {
		x = DBL_MAX;
		y = DBL_MAX;
	    }
	  if ((size - (in - blob)) < 2)
	      goto error;
	  arcs = gaiaImport16 (in, 1, graph->EndianArch);	/* # Arcs */
	  in += 2;
	  if (arcs < 0)
	      goto error;
	  /* initializing the Node */
	  pN = graph->Nodes + index;
	  pN->InternalIndex = index;
	  if (graph->NodeCode)
	    {
		/* Nodes are identified by a TEXT Code */
		pN->Id = -1;
		len = strlen (code);
		pN->Code = malloc (len + 1);
		strcpy (pN->Code, code);
	    }
	  else
	    {
		/* Nodes are identified by an INTEGER Id */
		pN->Id = nodeId;
		pN->Code = NULL;
	    }
	  pN->CoordX = x;
	  pN->CoordY = y;
	  pN->NumArcs = arcs;
	  if (arcs)
	    {
		/* parsing the Arcs */
		pN->Arcs = malloc (sizeof (NetworkArc) * arcs);
		for (ia = 0; ia < arcs; ia++)
		  {
		      /* parsing each Arc */
		      if (graph->Net64)
			{
			    if ((size - (in - blob)) < 22)
				goto error;
			}
		      else
			{
			    if ((size - (in - blob)) < 18)
				goto error;
			}
		      if (*in++ != GAIA_NET_ARC)	/* signature */
			  goto error;
		      if (graph->Net64)
			{
			    arcId = gaiaImportI64 (in, 1, graph->EndianArch);	/* # Arc ROWID: 64bit */
			    in += 8;
			}
		      else
			{
			    arcId = gaiaImport32 (in, 1, graph->EndianArch);	/* # Arc ROWID: 32bit */
			    in += 4;
			}
		      nodeToIdx = gaiaImport32 (in, 1, graph->EndianArch);	/* # NodeTo internal index */
		      in += 4;
		      cost = gaiaImport64 (in, 1, graph->EndianArch);	/* # Cost */
		      in += 8;
		      if (*in++ != GAIA_NET_END)	/* signature */
			  goto error;
		      pA = pN->Arcs + ia;
		      /* initializing the Arc */
		      if (nodeToIdx < 0 || nodeToIdx >= graph->NumNodes)
			  goto error;
		      pA->NodeFrom = pN;
		      pA->NodeTo = graph->Nodes + nodeToIdx;
		      pA->ArcRowid = arcId;
		      pA->Cost = cost;
		  }
	    }
	  else
	      pN->Arcs = NULL;
	  if ((size - (in - blob)) < 1)
	      goto error;
	  if (*in++ != GAIA_NET_END)	/* signature */
	      goto error;
      }
    return 1;
  error:
    return 0;
}

static NetworkPtr
load_network (sqlite3 * handle, const char *table)
{
/* loads the NETWORK struct */
    NetworkPtr graph = NULL;
    sqlite3_stmt *stmt;
    char sql[1024];
    int ret;
    int header = 1;
    const unsigned char *blob;
    int size;
    char xname[1024];
    strcpy (xname, table);
    vnet_double_quoted_sql (xname);
    sprintf (sql, "SELECT NetworkData FROM %s ORDER BY Id", xname);
    ret = sqlite3_prepare_v2 (handle, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	goto abort;
    while (1)
      {
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		if (sqlite3_column_type (stmt, 0) == SQLITE_BLOB)
		  {
		      blob =
			  (const unsigned char *) sqlite3_column_blob (stmt, 0);
		      size = sqlite3_column_bytes (stmt, 0);
		      if (header)
			{
			    /* parsing the HEADER block */
			    graph = network_init (blob, size);
			    header = 0;
			}
		      else
			{
			    /* parsing ordinary Blocks */
			    if (!graph)
			      {
				  sqlite3_finalize (stmt);
				  goto abort;
			      }
			    if (!network_block (graph, blob, size))
			      {
				  sqlite3_finalize (stmt);
				  goto abort;
			      }
			}
		  }
		else
		  {
		      sqlite3_finalize (stmt);
		      goto abort;
		  }
	    }
	  else
	    {
		sqlite3_finalize (stmt);
		goto abort;
	    }
      }
    sqlite3_finalize (stmt);
    return graph;
  abort:
    network_free (graph);
    return NULL;
}

static int
vnet_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	     sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table connected to some shapefile */
    VirtualNetworkPtr p_vt;
    char buf[1024];
    int err;
    int ret;
    int i;
    int n_rows;
    int n_columns;
    char vtable[1024];
    char table[1024];
    const char *col_name = NULL;
    char **results;
    char *err_msg = NULL;
    char sql[4096];
    int ok_id;
    int ok_data;
    char xname[1024];
    NetworkPtr graph = NULL;
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
/* checking for table_name and geo_column_name */
    if (argc == 4)
      {
	  strcpy (vtable, argv[2]);
	  vnet_dequote (vtable);
	  strcpy (table, argv[3]);
	  vnet_dequote (table);
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualNetwork module] CREATE VIRTUAL: illegal arg list {NETWORK-DATAtable}\n");
	  return SQLITE_ERROR;
      }
/* retrieving the base table columns */
    err = 0;
    ok_id = 0;
    ok_data = 0;
    strcpy (xname, table);
    vnet_double_quoted_sql (xname);
    sprintf (sql, "PRAGMA table_info(%s)", xname);
    ret = sqlite3_get_table (db, sql, &results, &n_rows, &n_columns, &err_msg);
    if (ret != SQLITE_OK)
      {
	  err = 1;
	  goto illegal;
      }
    if (n_rows > 1)
      {
	  for (i = 1; i <= n_rows; i++)
	    {
		col_name = results[(i * n_columns) + 1];
		if (strcasecmp (col_name, "id") == 0)
		    ok_id = 1;
		if (strcasecmp (col_name, "networkdata") == 0)
		    ok_data = 1;
	    }
	  sqlite3_free_table (results);
	  if (!ok_id)
	      err = 1;
	  if (!ok_data)
	      err = 1;
      }
    else
	err = 1;
  illegal:
    if (err)
      {
	  /* something is going the wrong way */
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualNetwork module] cannot build a valid NETWORK\n");
	  return SQLITE_ERROR;
      }
    p_vt = (VirtualNetworkPtr) sqlite3_malloc (sizeof (VirtualNetwork));
    if (!p_vt)
	return SQLITE_NOMEM;
    graph = load_network (db, table);
    if (!graph)
      {
	  /* something is going the wrong way */
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualNetwork module] cannot build a valid NETWORK\n");
	  return SQLITE_ERROR;
      }
    p_vt->db = db;
    p_vt->graph = graph;
    p_vt->currentAlgorithm = VNET_DIJKSTRA_ALGORITHM;
    p_vt->routing = NULL;
    p_vt->pModule = &my_net_module;
    p_vt->nRef = 0;
    p_vt->zErrMsg = NULL;
/* preparing the COLUMNs for this VIRTUAL TABLE */
    strcpy (buf, "CREATE TABLE ");
    strcpy (xname, vtable);
    vnet_double_quoted_sql (xname);
    strcat (buf, xname);
    strcat (buf, " (Algorithm TEXT, ArcRowid INTEGER, ");
    if (p_vt->graph->NodeCode)
	strcat (buf, "NodeFrom TEXT, NodeTo TEXT,");
    else
	strcat (buf, "NodeFrom INTEGER, NodeTo INTEGER,");
    strcat (buf, " Cost DOUBLE, Geometry BLOB");
    if (p_vt->graph->NameColumn)
	strcat (buf, ", Name TEXT)");
    else
	strcat (buf, ")");
    if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualNetwork module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       buf);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    p_vt->routing = routing_init (p_vt->graph);
    return SQLITE_OK;
}

static int
vnet_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	      sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table to some shapefile - simply aliases vshp_create() */
    return vnet_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
vnet_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIdxInfo)
{
/* best index selection */
    int i;
    int errors = 0;
    int err = 1;
    int from = 0;
    int to = 0;
    int i_from = -1;
    int i_to = -1;
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    for (i = 0; i < pIdxInfo->nConstraint; i++)
      {
	  /* verifying the constraints */
	  struct sqlite3_index_constraint *p = &(pIdxInfo->aConstraint[i]);
	  if (p->usable)
	    {
		if (p->iColumn == 2 && p->op == SQLITE_INDEX_CONSTRAINT_EQ)
		  {
		      from++;
		      i_from = i;
		  }
		else if (p->iColumn == 3 && p->op == SQLITE_INDEX_CONSTRAINT_EQ)
		  {
		      to++;
		      i_to = i;
		  }
		else
		    errors++;
	    }
      }
    if (from == 1 && to == 1 && errors == 0)
      {
	  /* this one is a valid Shortest Path query */
	  if (i_from < i_to)
	      pIdxInfo->idxNum = 1;	/* first arg is FROM */
	  else
	      pIdxInfo->idxNum = 2;	/* first arg is TO */
	  pIdxInfo->estimatedCost = 1.0;
	  for (i = 0; i < pIdxInfo->nConstraint; i++)
	    {
		if (pIdxInfo->aConstraint[i].usable)
		  {
		      pIdxInfo->aConstraintUsage[i].argvIndex = i + 1;
		      pIdxInfo->aConstraintUsage[i].omit = 1;
		  }
	    }
	  err = 0;
      }
    if (err)
      {
	  /* illegal query */
	  pIdxInfo->idxNum = 0;
      }
    return SQLITE_OK;
}

static int
vnet_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    VirtualNetworkPtr p_vt = (VirtualNetworkPtr) pVTab;
    if (p_vt->routing)
	routing_free (p_vt->routing);
    if (p_vt->graph)
	network_free (p_vt->graph);
    sqlite3_free (p_vt);
    return SQLITE_OK;
}

static int
vnet_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases vshp_disconnect() */
    return vnet_disconnect (pVTab);
}

static void
vnet_read_row (VirtualNetworkCursorPtr cursor)
{
/* trying to read a "row" from Shortest Path solution */
    if (cursor->solution->CurrentRow == NULL)
	cursor->eof = 1;
    else
	cursor->eof = 0;
    return;
}

static int
vnet_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    VirtualNetworkCursorPtr cursor =
	(VirtualNetworkCursorPtr)
	sqlite3_malloc (sizeof (VirtualNetworkCursor));
    if (cursor == NULL)
	return SQLITE_ERROR;
    cursor->pVtab = (VirtualNetworkPtr) pVTab;
    cursor->solution = alloc_solution ();
    cursor->eof = 0;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    return SQLITE_OK;
}

static int
vnet_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    VirtualNetworkCursorPtr cursor = (VirtualNetworkCursorPtr) pCursor;
    delete_solution (cursor->solution);
    sqlite3_free (pCursor);
    return SQLITE_OK;
}

static int
vnet_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	     int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    int node_code = 0;
    VirtualNetworkCursorPtr cursor = (VirtualNetworkCursorPtr) pCursor;
    VirtualNetworkPtr net = (VirtualNetworkPtr) cursor->pVtab;
    if (idxStr)
	idxStr = idxStr;	/* unused arg warning suppression */
    node_code = net->graph->NodeCode;
    reset_solution (cursor->solution);
    cursor->eof = 1;
    if (idxNum == 1 && argc == 2)
      {
	  /* retrieving the Shortest Path From/To params */
	  if (node_code)
	    {
		/* Nodes are identified by TEXT Codes */
		if (sqlite3_value_type (argv[0]) == SQLITE_TEXT)
		    cursor->solution->From =
			find_node_by_code (net->graph,
					   (char *)
					   sqlite3_value_text (argv[0]));
		if (sqlite3_value_type (argv[1]) == SQLITE_TEXT)
		    cursor->solution->To =
			find_node_by_code (net->graph,
					   (char *)
					   sqlite3_value_text (argv[1]));
	    }
	  else
	    {
		/* Nodes are identified by INT Ids */
		if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
		    cursor->solution->From =
			find_node_by_id (net->graph,
					 sqlite3_value_int (argv[0]));
		if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
		    cursor->solution->To =
			find_node_by_id (net->graph,
					 sqlite3_value_int (argv[1]));
	    }
      }
    if (idxNum == 2 && argc == 2)
      {
	  /* retrieving the Shortest Path To/From params */
	  if (node_code)
	    {
		/* Nodes are identified by TEXT Codes */
		if (sqlite3_value_type (argv[0]) == SQLITE_TEXT)
		    cursor->solution->To =
			find_node_by_code (net->graph,
					   (char *)
					   sqlite3_value_text (argv[0]));
		if (sqlite3_value_type (argv[1]) == SQLITE_TEXT)
		    cursor->solution->From =
			find_node_by_code (net->graph,
					   (char *)
					   sqlite3_value_text (argv[1]));
	    }
	  else
	    {
		/* Nodes are identified by INT Ids */
		if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
		    cursor->solution->To =
			find_node_by_id (net->graph,
					 sqlite3_value_int (argv[0]));
		if (sqlite3_value_type (argv[1]) == SQLITE_INTEGER)
		    cursor->solution->From =
			find_node_by_id (net->graph,
					 sqlite3_value_int (argv[1]));
	    }
      }
    if (cursor->solution->From && cursor->solution->To)
      {
	  cursor->eof = 0;
	  if (net->currentAlgorithm == VNET_A_STAR_ALGORITHM)
	      a_star_solve (net->db, net->graph, net->routing,
			    cursor->solution);
	  else
	      dijkstra_solve (net->db, net->graph, net->routing,
			      cursor->solution);
	  return SQLITE_OK;
      }
    cursor->eof = 0;
    return SQLITE_OK;
}

static int
vnet_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching a next row from cursor */
    VirtualNetworkCursorPtr cursor = (VirtualNetworkCursorPtr) pCursor;
    if (cursor->solution->CurrentRowId == 0)
	cursor->solution->CurrentRow = cursor->solution->First;
    else
	cursor->solution->CurrentRow = cursor->solution->CurrentRow->Next;
    if (!(cursor->solution->CurrentRow))
      {
	  cursor->eof = 1;
	  return SQLITE_OK;
      }
    (cursor->solution->CurrentRowId)++;
    vnet_read_row (cursor);
    return SQLITE_OK;
}

static int
vnet_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    VirtualNetworkCursorPtr cursor = (VirtualNetworkCursorPtr) pCursor;
    return cursor->eof;
}

static int
vnet_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	     int column)
{
/* fetching value for the Nth column */
    RowSolutionPtr row;
    int node_code = 0;
    const char *algorithm;
    VirtualNetworkCursorPtr cursor = (VirtualNetworkCursorPtr) pCursor;
    VirtualNetworkPtr net = (VirtualNetworkPtr) cursor->pVtab;
    node_code = net->graph->NodeCode;
    if (cursor->solution->CurrentRow == 0)
      {
	  /* special case: this one is the solution summary */
	  if (column == 0)
	    {
		/* the currently used Algorithm */
		if (net->currentAlgorithm == VNET_A_STAR_ALGORITHM)
		    algorithm = "A*";
		else
		    algorithm = "Dijkstra";
		sqlite3_result_text (pContext, algorithm, strlen (algorithm),
				     SQLITE_STATIC);
	    }
	  if (cursor->solution->From == NULL || cursor->solution->To == NULL)
	    {
		/* empty [uninitialized] solution */
		if (column > 0)
		    sqlite3_result_null (pContext);
		return SQLITE_OK;
	    }
	  if (column == 1)
	    {
		/* the ArcRowId column */
		sqlite3_result_null (pContext);
	    }
	  if (column == 2)
	    {
		/* the NodeFrom column */
		if (node_code)
		    sqlite3_result_text (pContext, cursor->solution->From->Code,
					 strlen (cursor->solution->From->Code),
					 SQLITE_STATIC);
		else
		    sqlite3_result_int64 (pContext, cursor->solution->From->Id);
	    }
	  if (column == 3)
	    {
		/* the NodeTo column */
		if (node_code)
		    sqlite3_result_text (pContext, cursor->solution->To->Code,
					 strlen (cursor->solution->To->Code),
					 SQLITE_STATIC);
		else
		    sqlite3_result_int64 (pContext, cursor->solution->To->Id);
	    }
	  if (column == 4)
	    {
		/* the Cost column */
		sqlite3_result_double (pContext, cursor->solution->TotalCost);
	    }
	  if (column == 5)
	    {
		/* the Geometry column */
		if (!(cursor->solution->Geometry))
		    sqlite3_result_null (pContext);
		else
		  {
		      /* builds the BLOB geometry to be returned */
		      int len;
		      unsigned char *p_result = NULL;
		      gaiaToSpatiaLiteBlobWkb (cursor->solution->Geometry,
					       &p_result, &len);
		      sqlite3_result_blob (pContext, p_result, len, free);
		  }
	    }
	  if (column == 6)
	    {
		/* the [optional] Name column */
		sqlite3_result_null (pContext);
	    }
      }
    else
      {
	  /* ordinary case: this one is an Arc used by the solution */
	  row = cursor->solution->CurrentRow;
	  if (column == 0)
	    {
		/* the currently used Algorithm */
		if (net->currentAlgorithm == VNET_A_STAR_ALGORITHM)
		    algorithm = "A*";
		else
		    algorithm = "Dijkstra";
		sqlite3_result_text (pContext, algorithm, strlen (algorithm),
				     SQLITE_STATIC);
	    }
	  if (column == 1)
	    {
		/* the ArcRowId column */
		sqlite3_result_int64 (pContext, row->Arc->ArcRowid);
	    }
	  if (column == 2)
	    {
		/* the NodeFrom column */
		if (node_code)
		    sqlite3_result_text (pContext, row->Arc->NodeFrom->Code,
					 strlen (row->Arc->NodeFrom->Code),
					 SQLITE_STATIC);
		else
		    sqlite3_result_int64 (pContext, row->Arc->NodeFrom->Id);
	    }
	  if (column == 3)
	    {
		/* the NodeTo column */
		if (node_code)
		    sqlite3_result_text (pContext, row->Arc->NodeTo->Code,
					 strlen (row->Arc->NodeTo->Code),
					 SQLITE_STATIC);
		else
		    sqlite3_result_int64 (pContext, row->Arc->NodeTo->Id);
	    }
	  if (column == 4)
	    {
		/* the Cost column */
		sqlite3_result_double (pContext, row->Arc->Cost);
	    }
	  if (column == 5)
	    {
		/* the Geometry column */
		sqlite3_result_null (pContext);
	    }
	  if (column == 6)
	    {
		/* the [optional] Name column */
		if (row->Name)
		    sqlite3_result_text (pContext, row->Name,
					 strlen (row->Name), SQLITE_STATIC);
		else
		    sqlite3_result_null (pContext);
	    }
      }
    return SQLITE_OK;
}

static int
vnet_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    VirtualNetworkCursorPtr cursor = (VirtualNetworkCursorPtr) pCursor;
    *pRowid = cursor->solution->CurrentRowId;
    return SQLITE_OK;
}

static int
vnet_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	     sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    VirtualNetworkPtr p_vtab = (VirtualNetworkPtr) pVTab;
    if (pRowid)
	pRowid = pRowid;	/* unused arg warning suppression */
    if (argc == 1)
      {
	  /* performing a DELETE is forbidden */
	  return SQLITE_READONLY;
      }
    else
      {
	  if (sqlite3_value_type (argv[0]) == SQLITE_NULL)
	    {
		/* performing an INSERT is forbidden */
		return SQLITE_READONLY;
	    }
	  else
	    {
		/* performing an UPDATE */
		if (argc == 9)
		  {
		      p_vtab->currentAlgorithm = VNET_DIJKSTRA_ALGORITHM;
		      if (sqlite3_value_type (argv[2]) == SQLITE_TEXT)
			{
			    const unsigned char *algorithm =
				sqlite3_value_text (argv[2]);
			    if (strcmp ((char *) algorithm, "A*") == 0)
				p_vtab->currentAlgorithm =
				    VNET_A_STAR_ALGORITHM;
			    if (strcmp ((char *) algorithm, "a*") == 0)
				p_vtab->currentAlgorithm =
				    VNET_A_STAR_ALGORITHM;
			}
		      if (p_vtab->graph->AStar == 0)
			  p_vtab->currentAlgorithm = VNET_DIJKSTRA_ALGORITHM;
		  }
		return SQLITE_OK;
	    }
      }
    return SQLITE_READONLY;
}

static int
vnet_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vnet_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vnet_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vnet_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3VirtualNetworkInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    my_net_module.iVersion = 1;
    my_net_module.xCreate = &vnet_create;
    my_net_module.xConnect = &vnet_connect;
    my_net_module.xBestIndex = &vnet_best_index;
    my_net_module.xDisconnect = &vnet_disconnect;
    my_net_module.xDestroy = &vnet_destroy;
    my_net_module.xOpen = &vnet_open;
    my_net_module.xClose = &vnet_close;
    my_net_module.xFilter = &vnet_filter;
    my_net_module.xNext = &vnet_next;
    my_net_module.xEof = &vnet_eof;
    my_net_module.xColumn = &vnet_column;
    my_net_module.xRowid = &vnet_rowid;
    my_net_module.xUpdate = &vnet_update;
    my_net_module.xBegin = &vnet_begin;
    my_net_module.xSync = &vnet_sync;
    my_net_module.xCommit = &vnet_commit;
    my_net_module.xRollback = &vnet_rollback;
    my_net_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "VirtualNetwork", &my_net_module, NULL, 0);
    return rc;
}

int
virtualnetwork_extension_init (sqlite3 * db)
{
    return sqlite3VirtualNetworkInit (db);
}
/**************** End file: virtualnetwork.c **********/


/**************** Begin file: virtualspatialindex.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <math.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

static struct sqlite3_module my_spidx_module;


/******************************************************************************
/
/ VirtualTable structs
/
******************************************************************************/

typedef struct VirtualSpatialIndexStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
} VirtualSpatialIndex;
typedef VirtualSpatialIndex *VirtualSpatialIndexPtr;

typedef struct VirtualSpatialIndexCursorStruct
{
/* extends the sqlite3_vtab_cursor struct */
    VirtualSpatialIndexPtr pVtab;	/* Virtual table of this cursor */
    int eof;			/* the EOF marker */
    sqlite3_stmt *stmt;
    sqlite3_int64 CurrentRowId;
} VirtualSpatialIndexCursor;
typedef VirtualSpatialIndexCursor *VirtualSpatialIndexCursorPtr;

static void
vspidx_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

static void
vspidx_clean_sql_string (char *buf)
{
/* returns a well formatted TEXT value for SQL */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    while (*in != '\0')
      {
	  if (*in == '\'')
	      *out++ = '\'';
	  *out++ = *in++;
      }
    *out = '\0';
}

static void
vspidx_dequote (char *buf)
{
/* dequoting an SQL string */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    char strip = '\0';
    int first = 0;
    int len = strlen (buf);
    if (buf[0] == '\'' && buf[len - 1] == '\'')
	strip = '\'';
    if (buf[0] == '"' && buf[len - 1] == '"')
	strip = '"';
    if (strip == '\0')
	return;
    strcpy (tmp, buf + 1);
    len = strlen (tmp);
    tmp[len - 1] = '\0';
    while (*in != '\0')
      {
	  if (*in == strip)
	    {
		if (first)
		  {
		      first = 0;
		      in++;
		      continue;
		  }
		else
		  {
		      first = 1;
		      *out++ = *in++;
		      continue;
		  }
	    }
	  first = 0;
	  *out++ = *in++;
      }
    *out = '\0';
}

static int
vspidx_check_rtree (sqlite3 * sqlite, const char *table_name,
		    const char *geom_column)
{
/* checks if the required RTree is actually defined */
    char sql[4096];
    char xtable[1024];
    char xcolumn[1024];
    int ret;
    int i;
    int n_rows;
    int n_columns;
    char **results;
    int count = 0;

    strcpy (xtable, table_name);
    strcpy (xcolumn, geom_column);
    strcpy (sql,
	    "SELECT Count(*) FROM geometry_columns WHERE f_table_name LIKE '");
    vspidx_clean_sql_string (xtable);
    strcat (sql, xtable);
    strcat (sql, "' AND f_geometry_column LIKE '");
    vspidx_clean_sql_string (xcolumn);
    strcat (sql, xcolumn);
    strcat (sql, "' AND spatial_index_enabled = 1");
    ret = sqlite3_get_table (sqlite, sql, &results, &n_rows, &n_columns, NULL);
    if (ret != SQLITE_OK)
	return 0;
    if (n_rows >= 1)
      {
	  for (i = 1; i <= n_rows; i++)
	      count = atoi (results[(i * n_columns) + 0]);
      }
    sqlite3_free_table (results);
    if (count != 1)
	return 0;
    return 1;
}

static int
vspidx_find_rtree (sqlite3 * sqlite, const char *table_name, char *geom_column)
{
/* attempts to find the corresponding RTree Geometry Column */
    char sql[4096];
    char xtable[1024];
    char xcolumn[1024];
    int ret;
    int i;
    int n_rows;
    int n_columns;
    char **results;
    int count = 0;

    strcpy (xtable, table_name);
    strcpy (sql,
	    "SELECT f_geometry_column FROM geometry_columns WHERE f_table_name LIKE '");
    vspidx_clean_sql_string (xtable);
    strcat (sql, xtable);
    strcat (sql, "' AND spatial_index_enabled = 1");
    ret = sqlite3_get_table (sqlite, sql, &results, &n_rows, &n_columns, NULL);
    if (ret != SQLITE_OK)
	return 0;
    if (n_rows >= 1)
      {
	  for (i = 1; i <= n_rows; i++)
	    {
		count++;
		strcpy (xcolumn, results[(i * n_columns) + 0]);
	    }
      }
    sqlite3_free_table (results);
    if (count != 1)
	return 0;
    strcpy (geom_column, xcolumn);
    return 1;
}

static int
vspidx_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	       sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table for R*Tree SpatialIndex metahandling */
    VirtualSpatialIndexPtr p_vt;
    char buf[1024];
    char vtable[1024];
    char xname[1024];
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
    if (argc == 3)
      {
	  strcpy (vtable, argv[2]);
	  vspidx_dequote (vtable);
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualSpatialIndex module] CREATE VIRTUAL: illegal arg list {void}\n");
	  return SQLITE_ERROR;
      }
    p_vt =
	(VirtualSpatialIndexPtr) sqlite3_malloc (sizeof (VirtualSpatialIndex));
    if (!p_vt)
	return SQLITE_NOMEM;
    p_vt->db = db;
    p_vt->pModule = &my_spidx_module;
    p_vt->nRef = 0;
    p_vt->zErrMsg = NULL;
/* preparing the COLUMNs for this VIRTUAL TABLE */
    strcpy (buf, "CREATE TABLE ");
    strcpy (xname, vtable);
    vspidx_double_quoted_sql (xname);
    strcat (buf, xname);
    strcat (buf,
	    " (f_table_name TEXT, f_geometry_column TEXT, search_frame BLOB)");
    if (sqlite3_declare_vtab (db, buf) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualSpatialIndex module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       buf);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    return SQLITE_OK;
}

static int
vspidx_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
		sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table - simply aliases vspidx_create() */
    return vspidx_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
vspidx_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIdxInfo)
{
/* best index selection */
    int i;
    int errors = 0;
    int err = 1;
    int table = 0;
    int geom = 0;
    int mbr = 0;
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    for (i = 0; i < pIdxInfo->nConstraint; i++)
      {
	  /* verifying the constraints */
	  struct sqlite3_index_constraint *p = &(pIdxInfo->aConstraint[i]);
	  if (p->usable)
	    {
		if (p->iColumn == 0 && p->op == SQLITE_INDEX_CONSTRAINT_EQ)
		      table++;
		else if (p->iColumn == 1 && p->op == SQLITE_INDEX_CONSTRAINT_EQ)
		      geom++;
		else if (p->iColumn == 2 && p->op == SQLITE_INDEX_CONSTRAINT_EQ)
		      mbr++;
		else
		    errors++;
	    }
      }
    if (table == 1 && (geom == 0 || geom == 1) && mbr == 1 && errors == 0)
      {
	  /* this one is a valid SpatialIndex query */
	  if (geom == 1)
	      pIdxInfo->idxNum = 1;
	  else
	      pIdxInfo->idxNum = 2;
	  pIdxInfo->estimatedCost = 1.0;
	  for (i = 0; i < pIdxInfo->nConstraint; i++)
	    {
		if (pIdxInfo->aConstraint[i].usable)
		  {
		      pIdxInfo->aConstraintUsage[i].argvIndex = i + 1;
		      pIdxInfo->aConstraintUsage[i].omit = 1;
		  }
	    }
	  err = 0;
      }
    if (err)
      {
	  /* illegal query */
	  pIdxInfo->idxNum = 0;
      }
    return SQLITE_OK;
}

static int
vspidx_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    VirtualSpatialIndexPtr p_vt = (VirtualSpatialIndexPtr) pVTab;
    sqlite3_free (p_vt);
    return SQLITE_OK;
}

static int
vspidx_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases vspidx_disconnect() */
    return vspidx_disconnect (pVTab);
}

static int
vspidx_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    VirtualSpatialIndexCursorPtr cursor =
	(VirtualSpatialIndexCursorPtr)
	sqlite3_malloc (sizeof (VirtualSpatialIndexCursor));
    if (cursor == NULL)
	return SQLITE_ERROR;
    cursor->pVtab = (VirtualSpatialIndexPtr) pVTab;
    cursor->stmt = NULL;
    cursor->eof = 1;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    return SQLITE_OK;
}

static int
vspidx_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    VirtualSpatialIndexCursorPtr cursor =
	(VirtualSpatialIndexCursorPtr) pCursor;
    if (cursor->stmt)
	sqlite3_finalize (cursor->stmt);
    sqlite3_free (pCursor);
    return SQLITE_OK;
}

static int
vspidx_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	       int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    char table_name[1024];
    char geom_column[1024];
    char idx_name[1024];
    char sql[4096];
    gaiaGeomCollPtr geom = NULL;
    int ok_table = 0;
    int ok_geom = 0;
    const unsigned char *blob;
    int size;
    int exists;
    int ret;
    sqlite3_stmt *stmt;
    float minx;
    float miny;
    float maxx;
    float maxy;
    double tic;
    double tic2;
    VirtualSpatialIndexCursorPtr cursor =
	(VirtualSpatialIndexCursorPtr) pCursor;
    VirtualSpatialIndexPtr spidx = (VirtualSpatialIndexPtr) cursor->pVtab;
    if (idxStr)
	idxStr = idxStr;	/* unused arg warning suppression */
    cursor->eof = 1;
    if (idxNum == 1 && argc == 3)
      {
	  /* retrieving the Table/Column/MBR params */
	  if (sqlite3_value_type (argv[0]) == SQLITE_TEXT)
	    {
		strcpy (table_name,
			(const char *) sqlite3_value_text (argv[0]));
		ok_table = 1;
	    }
	  if (sqlite3_value_type (argv[1]) == SQLITE_TEXT)
	    {
		strcpy (geom_column,
			(const char *) sqlite3_value_text (argv[1]));
		ok_geom = 1;
	    }
	  if (sqlite3_value_type (argv[2]) == SQLITE_BLOB)
	    {
		blob = sqlite3_value_blob (argv[2]);
		size = sqlite3_value_bytes (argv[2]);
		geom = gaiaFromSpatiaLiteBlobWkb (blob, size);
	    }
	  if (ok_table && ok_geom && geom)
	      ;
	  else
	    {
		/* invalid args */
		goto stop;
	    }
      }
    if (idxNum == 2 && argc == 2)
      {
	  /* retrieving the Table/MBR params */
	  if (sqlite3_value_type (argv[0]) == SQLITE_TEXT)
	    {
		strcpy (table_name,
			(const char *) sqlite3_value_text (argv[0]));
		ok_table = 1;
	    }
	  if (sqlite3_value_type (argv[1]) == SQLITE_BLOB)
	    {
		blob = sqlite3_value_blob (argv[1]);
		size = sqlite3_value_bytes (argv[1]);
		geom = gaiaFromSpatiaLiteBlobWkb (blob, size);
	    }
	  if (ok_table && geom)
	      ;
	  else
	    {
		/* invalid args */
		goto stop;
	    }
      }

/* checking if the corresponding R*Tree exists */
    if (ok_geom)
	exists = vspidx_check_rtree (spidx->db, table_name, geom_column);
    else
	exists = vspidx_find_rtree (spidx->db, table_name, geom_column);
    if (!exists)
	goto stop;

/* building the RTree query */
    sprintf (idx_name, "idx_%s_%s", table_name, geom_column);
    vspidx_double_quoted_sql (idx_name);
    sprintf (sql, "SELECT pkid FROM %s WHERE ", idx_name);
    strcat (sql, "xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?");
    ret = sqlite3_prepare_v2 (spidx->db, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	goto stop;
/* binding stmt params [MBR] */
    gaiaMbrGeometry (geom);

/* adjusting the MBR so to compensate for DOUBLE/FLOAT truncations */
    minx = (float)(geom->MinX);
    miny = (float)(geom->MinY);
    maxx = (float)(geom->MaxX);
    maxy = (float)(geom->MaxY);
    tic = fabs (geom->MinX - minx);
    tic2 = fabs (geom->MinY - miny);
    if (tic2 > tic)
	tic = tic2;
    tic2 = fabs (geom->MaxX - maxx);
    if (tic2 > tic)
	tic = tic2;
    tic2 = fabs (geom->MaxY - maxy);
    if (tic2 > tic)
	tic = tic2;
    tic *= 2.0;
    sqlite3_bind_double (stmt, 1, geom->MaxX + tic);
    sqlite3_bind_double (stmt, 2, geom->MinX - tic);
    sqlite3_bind_double (stmt, 3, geom->MaxY + tic);
    sqlite3_bind_double (stmt, 4, geom->MinY - tic);
    cursor->stmt = stmt;
    cursor->eof = 0;
/* fetching the first ResultSet's row */
    ret = sqlite3_step (cursor->stmt);
    if (ret == SQLITE_ROW)
	cursor->CurrentRowId = sqlite3_column_int64 (cursor->stmt, 0);
    else
	cursor->eof = 1;
  stop:
    return SQLITE_OK;
}

static int
vspidx_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching a next row from cursor */
    int ret;
    VirtualSpatialIndexCursorPtr cursor =
	(VirtualSpatialIndexCursorPtr) pCursor;
    ret = sqlite3_step (cursor->stmt);
    if (ret == SQLITE_ROW)
	cursor->CurrentRowId = sqlite3_column_int64 (cursor->stmt, 0);
    else
	cursor->eof = 1;
    return SQLITE_OK;
}

static int
vspidx_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    VirtualSpatialIndexCursorPtr cursor =
	(VirtualSpatialIndexCursorPtr) pCursor;
    return cursor->eof;
}

static int
vspidx_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	       int column)
{
/* fetching value for the Nth column */
    VirtualSpatialIndexCursorPtr cursor =
	(VirtualSpatialIndexCursorPtr) pCursor;
    if (cursor || column)
	cursor = cursor;	/* unused arg warning suppression */
    if (column)
	column = column;	/* unused arg warning suppression */
    sqlite3_result_null (pContext);
    return SQLITE_OK;
}

static int
vspidx_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    VirtualSpatialIndexCursorPtr cursor =
	(VirtualSpatialIndexCursorPtr) pCursor;
    *pRowid = cursor->CurrentRowId;
    return SQLITE_OK;
}

static int
vspidx_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	       sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    if (pRowid || argc || argv || pVTab)
	pRowid = pRowid;	/* unused arg warning suppression */
/* read only datasource */
    return SQLITE_READONLY;
}

static int
vspidx_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vspidx_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vspidx_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vspidx_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3VirtualSpatialIndexInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    my_spidx_module.iVersion = 1;
    my_spidx_module.xCreate = &vspidx_create;
    my_spidx_module.xConnect = &vspidx_connect;
    my_spidx_module.xBestIndex = &vspidx_best_index;
    my_spidx_module.xDisconnect = &vspidx_disconnect;
    my_spidx_module.xDestroy = &vspidx_destroy;
    my_spidx_module.xOpen = &vspidx_open;
    my_spidx_module.xClose = &vspidx_close;
    my_spidx_module.xFilter = &vspidx_filter;
    my_spidx_module.xNext = &vspidx_next;
    my_spidx_module.xEof = &vspidx_eof;
    my_spidx_module.xColumn = &vspidx_column;
    my_spidx_module.xRowid = &vspidx_rowid;
    my_spidx_module.xUpdate = &vspidx_update;
    my_spidx_module.xBegin = &vspidx_begin;
    my_spidx_module.xSync = &vspidx_sync;
    my_spidx_module.xCommit = &vspidx_commit;
    my_spidx_module.xRollback = &vspidx_rollback;
    my_spidx_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "VirtualSpatialIndex", &my_spidx_module, NULL,
			      0);
    return rc;
}

int
virtual_spatialindex_extension_init (sqlite3 * db)
{
    return sqlite3VirtualSpatialIndexInit (db);
}
/**************** End file: virtualspatialindex.c **********/


/**************** Begin file: virtualfdo.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <limits.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */

/* constants definining FDO-OGR Geometry formats */
#define FDO_OGR_NONE	0
#define FDO_OGR_WKT	1
#define FDO_OGR_WKB	2
#define FDO_OGR_FGF	3
#define FDO_OGR_SPLITE	4

#ifdef _WIN32
#define strcasecmp	_stricmp
#define strncasecmp	_strnicmp
#endif /* not WIN32 */

#if defined(_WIN32) && !defined(__MINGW32__)
#define LONG64_MAX	_I64_MAX
#define LONG64_MIN	_I64_MIN
#else
#define LONG64_MAX	9223372036854775807LL
#define LONG64_MIN	(-LONG64_MAX + 1)
#endif

struct sqlite3_module my_fdo_module;

typedef struct SqliteValue
{
/* a multitype storing a column value */
    int Type;
    sqlite3_int64 IntValue;
    double DoubleValue;
    char *Text;
    unsigned char *Blob;
    int Size;
} SqliteValue;
typedef SqliteValue *SqliteValuePtr;

typedef struct VirtualFDOStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
    char *table;		/* the real-table name */
    int nColumns;		/* the # columns into the table */
    char **Column;		/* the name for each column */
    char **Type;		/* the type for each column */
    int *NotNull;		/* NotNull clause for each column */
    SqliteValuePtr *Value;	/* the current-row value for each column */
    int nGeometries;		/* # Geometry columns into the table */
    char **GeoColumn;		/* the name for each Geometry column */
    int *Srid;			/* the SRID for each Geometry column */
    int *GeoType;		/* the Type for each Geometry column */
    int *Format;		/* the Format for each Geometry column */
    int *CoordDimensions;	/* # Dimensions for each Geometry column */
} VirtualFDO;
typedef VirtualFDO *VirtualFDOPtr;

typedef struct VirtualFDOCursorStruct
{
/* extends the sqlite3_vtab_cursor struct */
    VirtualFDOPtr pVtab;	/* Virtual table of this cursor */
    sqlite3_int64 current_row;	/* the current row ID */
    int eof;			/* the EOF marker */
} VirtualFDOCursor;
typedef VirtualFDOCursor *VirtualFDOCursorPtr;

static void
vfdo_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}


static void
vfdo_clean_sql_string (char *buf)
{
/* returns a well formatted TEXT value for SQL */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    while (*in != '\0')
      {
	  if (*in == '\'')
	      *out++ = '\'';
	  *out++ = *in++;
      }
    *out = '\0';
}

static void
vfdo_dequote (char *buf)
{
/* dequoting an SQL string */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    char strip = '\0';
    int first = 0;
    int len = strlen (buf);
    if (buf[0] == '\'' && buf[len - 1] == '\'')
	strip = '\'';
    if (buf[0] == '"' && buf[len - 1] == '"')
	strip = '"';
    if (strip == '\0')
	return;
    strcpy (tmp, buf + 1);
    len = strlen (tmp);
    tmp[len - 1] = '\0';
    while (*in != '\0')
      {
	  if (*in == strip)
	    {
		if (first)
		  {
		      first = 0;
		      in++;
		      continue;
		  }
		else
		  {
		      first = 1;
		      *out++ = *in++;
		      continue;
		  }
	    }
	  first = 0;
	  *out++ = *in++;
      }
    *out = '\0';
}

static SqliteValuePtr
value_alloc (void)
{
/* allocates and initialites a Value multitype */
    SqliteValuePtr p = malloc (sizeof (SqliteValue));
    p->Type = SQLITE_NULL;
    p->Text = NULL;
    p->Blob = NULL;
    return p;
}

static void
value_free (SqliteValuePtr p)
{
/* freeing a Value multitype */
    if (!p)
	return;
    if (p->Text)
	free (p->Text);
    if (p->Blob)
	free (p->Blob);
    free (p);
}

static void
value_set_null (SqliteValuePtr p)
{
/* setting a NULL value to the multitype */
    if (!p)
	return;
    p->Type = SQLITE_NULL;
    if (p->Text)
	free (p->Text);
    if (p->Blob)
	free (p->Blob);
    p->Text = NULL;
    p->Blob = NULL;
}

static void
value_set_int (SqliteValuePtr p, sqlite3_int64 value)
{
/* setting an INT value to the multitype */
    if (!p)
	return;
    p->Type = SQLITE_INTEGER;
    if (p->Text)
	free (p->Text);
    if (p->Blob)
	free (p->Blob);
    p->Text = NULL;
    p->Blob = NULL;
    p->IntValue = value;
}

static void
value_set_double (SqliteValuePtr p, double value)
{
/* setting a DOUBLE value to the multitype */
    if (!p)
	return;
    p->Type = SQLITE_FLOAT;
    if (p->Text)
	free (p->Text);
    if (p->Blob)
	free (p->Blob);
    p->Text = NULL;
    p->Blob = NULL;
    p->DoubleValue = value;
}

static void
value_set_text (SqliteValuePtr p, const char *value, int size)
{
/* setting a TEXT value to the multitype */
    if (!p)
	return;
    p->Type = SQLITE_TEXT;
    if (p->Text)
	free (p->Text);
    if (p->Blob)
	free (p->Blob);
    p->Blob = NULL;
    p->Text = malloc (size);
    memcpy (p->Text, value, size);
    p->Size = size;
}

static void
value_set_blob (SqliteValuePtr p, const unsigned char *value, int size)
{
/* setting a BLOB value to the multitype */
    if (!p)
	return;
    p->Type = SQLITE_BLOB;
    if (p->Text)
	free (p->Text);
    if (p->Blob)
	free (p->Blob);
    p->Text = NULL;
    p->Blob = malloc (size);
    memcpy (p->Blob, value, size);
    p->Size = size;
}

static void
free_table (VirtualFDOPtr p_vt)
{
/* memory cleanup; freeing the virtual table struct */
    int i;
    if (!p_vt)
	return;
    if (p_vt->Column)
      {
	  for (i = 0; i < p_vt->nColumns; i++)
	    {
		if (*(p_vt->Column + i))
		    sqlite3_free (*(p_vt->Column + i));
	    }
	  sqlite3_free (p_vt->Column);
      }
    if (p_vt->Type)
      {
	  for (i = 0; i < p_vt->nColumns; i++)
	    {
		if (*(p_vt->Type + i))
		    sqlite3_free (*(p_vt->Type + i));
	    }
	  sqlite3_free (p_vt->Type);
      }
    if (p_vt->NotNull)
	sqlite3_free (p_vt->NotNull);
    if (p_vt->Value)
      {
	  for (i = 0; i < p_vt->nColumns; i++)
	    {
		if (*(p_vt->Value + i))
		    value_free (*(p_vt->Value + i));
	    }
	  sqlite3_free (p_vt->Value);
      }
    if (p_vt->GeoColumn)
      {
	  for (i = 0; i < p_vt->nGeometries; i++)
	    {
		if (*(p_vt->GeoColumn + i))
		    sqlite3_free (*(p_vt->GeoColumn + i));
	    }
	  sqlite3_free (p_vt->GeoColumn);
      }
    if (p_vt->Srid)
	sqlite3_free (p_vt->Srid);
    if (p_vt->GeoType)
	sqlite3_free (p_vt->GeoType);
    if (p_vt->Format)
	sqlite3_free (p_vt->Format);
    if (p_vt->CoordDimensions)
	sqlite3_free (p_vt->CoordDimensions);
    sqlite3_free (p_vt);
}


static void
vfdoOutWkt3D (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom)
{
/* prints the GEOS-WKT-3D representation of current geometry */
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counting how many POLYGONs are there */
	  pgs++;
	  polyg = polyg->Next;
      }
    if ((pts + lns + pgs) == 1
	&& (geom->DeclaredType == GAIA_POINT
	    || geom->DeclaredType == GAIA_LINESTRING
	    || geom->DeclaredType == GAIA_POLYGON))
      {
	  /* we have only one elementary geometry */
	  point = geom->FirstPoint;
	  while (point)
	    {
		/* processing POINT */
		gaiaAppendToOutBuffer (out_buf, "POINT (");
		gaiaOutPointZ (out_buf, point);
		gaiaAppendToOutBuffer (out_buf, ")");
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		/* processing LINESTRING */
		gaiaAppendToOutBuffer (out_buf, "LINESTRING (");
		gaiaOutLinestringZ (out_buf, line);
		gaiaAppendToOutBuffer (out_buf, ")");
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		/* processing POLYGONZ */
		gaiaAppendToOutBuffer (out_buf, "POLYGON (");
		gaiaOutPolygonZ (out_buf, polyg);
		gaiaAppendToOutBuffer (out_buf, ")");
		polyg = polyg->Next;
	    }
      }
    else
      {
	  /* we have some kind of complex geometry */
	  if (pts > 0 && lns == 0 && pgs == 0
	      && geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		/* some kind of MULTIPOINT */
		gaiaAppendToOutBuffer (out_buf, "MULTIPOINT (");
		point = geom->FirstPoint;
		while (point)
		  {
		      if (point != geom->FirstPoint)
			  gaiaAppendToOutBuffer (out_buf, ", ");
		      gaiaOutPointZ (out_buf, point);
		      point = point->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns > 0 && pgs == 0
		   && geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		/* some kind of MULTILINESTRING */
		gaiaAppendToOutBuffer (out_buf, "MULTILINESTRING (");
		line = geom->FirstLinestring;
		while (line)
		  {
		      if (line != geom->FirstLinestring)
			  gaiaAppendToOutBuffer (out_buf, ", (");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      gaiaOutLinestringZ (out_buf, line);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      line = line->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns == 0 && pgs > 0
		   && geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		/* some kind of MULTIPOLYGON */
		gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGON (");
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      if (polyg != geom->FirstPolygon)
			  gaiaAppendToOutBuffer (out_buf, ", (");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      gaiaOutPolygonZ (out_buf, polyg);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else
	    {
		/* some kind of GEOMETRYCOLLECTION */
		int ie = 0;
		gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTION (");
		point = geom->FirstPoint;
		while (point)
		  {
		      /* processing POINTs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ", ");
		      ie++;
		      gaiaAppendToOutBuffer (out_buf, "POINT (");
		      gaiaOutPointZ (out_buf, point);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      point = point->Next;
		  }
		line = geom->FirstLinestring;
		while (line)
		  {
		      /* processing LINESTRINGs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ", ");
		      ie++;
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING (");
		      gaiaOutLinestringZ (out_buf, line);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      line = line->Next;
		  }
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      /* processing POLYGONs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ", ");
		      ie++;
		      gaiaAppendToOutBuffer (out_buf, "POLYGON (");
		      gaiaOutPolygonZ (out_buf, polyg);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
      }
}

static void
vfdoToWkb3D (gaiaGeomCollPtr geom, unsigned char **result, int *size)
{
/* builds the GEOS-WKB-3D representation for this GEOMETRY */
    int ib;
    int iv;
    double x;
    double y;
    double z = 0.0;
    double m = 0.0;
    int entities = 0;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    int type;
    unsigned char *ptr;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    gaiaPointPtr point = NULL;
    gaiaLinestringPtr line = NULL;
    gaiaPolygonPtr polyg = NULL;
    int endian_arch = gaiaEndianArch ();
    gaiaMbrGeometry (geom);
/* how many entities, and of what kind, do we have ? */
    pt = geom->FirstPoint;
    while (pt)
      {
	  point = pt;
	  entities++;
	  n_points++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  line = ln;
	  entities++;
	  n_linestrings++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  polyg = pg;
	  entities++;
	  n_polygons++;
	  pg = pg->Next;
      }
    *size = 0;
    *result = NULL;
    if (n_points == 0 && n_polygons == 0 && n_linestrings == 0)
	return;
/* ok, we can determine the geometry class */
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	      type = GAIA_GEOSWKB_MULTIPOINTZ;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ;
	  else
	      type = GAIA_GEOSWKB_POINTZ;
      }
    else if (n_points > 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ;
	  else
	      type = GAIA_GEOSWKB_MULTIPOINTZ;
      }
    else if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	      type = GAIA_GEOSWKB_MULTILINESTRINGZ;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ;
	  else
	      type = GAIA_GEOSWKB_LINESTRINGZ;
      }
    else if (n_points == 0 && n_linestrings > 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ;
	  else
	      type = GAIA_GEOSWKB_MULTILINESTRINGZ;
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	      type = GAIA_GEOSWKB_MULTIPOLYGONZ;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ;
	  else
	      type = GAIA_GEOSWKB_POLYGONZ;
      }
    else if (n_points == 0 && n_linestrings == 0 && n_polygons > 1)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      type = GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ;
	  else
	      type = GAIA_GEOSWKB_MULTIPOLYGONZ;
      }
    else
	type = GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ;
/* and now we compute the size of WKB */
    *size = 5;			/* header size */
    if (type == GAIA_GEOSWKB_MULTIPOINTZ
	|| type == GAIA_GEOSWKB_MULTILINESTRINGZ
	|| type == GAIA_GEOSWKB_MULTIPOLYGONZ
	|| type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
	*size += 4;
    point = geom->FirstPoint;
    while (point)
      {
	  if (type == GAIA_GEOSWKB_MULTIPOINTZ
	      || type == GAIA_GEOSWKB_MULTILINESTRINGZ
	      || type == GAIA_GEOSWKB_MULTIPOLYGONZ
	      || type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
	      *size += 5;
	  *size += (sizeof (double) * 3);	/* three doubles for each POINT */
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  if (type == GAIA_GEOSWKB_MULTIPOINTZ
	      || type == GAIA_GEOSWKB_MULTILINESTRINGZ
	      || type == GAIA_GEOSWKB_MULTIPOLYGONZ
	      || type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
	      *size += 5;
	  *size += (4 + ((sizeof (double) * 3) * line->Points));	/* # points + [x,y,z] for each vertex */
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  if (type == GAIA_GEOSWKB_MULTIPOINTZ
	      || type == GAIA_GEOSWKB_MULTILINESTRINGZ
	      || type == GAIA_GEOSWKB_MULTIPOLYGONZ
	      || type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
	      *size += 5;
	  rng = polyg->Exterior;
	  *size += (8 + ((sizeof (double) * 3) * rng->Points));	/* # rings + # points + [x,y,z] array - exterior ring */
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		*size += (4 + ((sizeof (double) * 3) * rng->Points));	/* # points + [x,y,z] array - interior ring */
	    }
	  polyg = polyg->Next;
      }
    *result = malloc (*size);
    ptr = *result;
/* and finally we build the WKB */
    *ptr = 0x01;		/* little endian byte order */
    gaiaExport32 (ptr + 1, type, 1, endian_arch);	/* the main CLASS TYPE */
    ptr += 5;
    if (type == GAIA_GEOSWKB_MULTIPOINTZ
	|| type == GAIA_GEOSWKB_MULTILINESTRINGZ
	|| type == GAIA_GEOSWKB_MULTIPOLYGONZ
	|| type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
      {
	  gaiaExport32 (ptr, entities, 1, endian_arch);	/* it's a collection; # entities */
	  ptr += 4;
      }
    point = geom->FirstPoint;
    while (point)
      {
	  if (type == GAIA_GEOSWKB_MULTIPOINTZ
	      || type == GAIA_GEOMETRYCOLLECTIONZ)
	    {
		*ptr = 0x01;
		/* it's a collection: the CLASS TYPE for this element */
		gaiaExport32 (ptr + 1, GAIA_GEOSWKB_POINTZ, 1, endian_arch);
		ptr += 5;
	    }
	  gaiaExport64 (ptr, point->X, 1, endian_arch);	/* X */
	  gaiaExport64 (ptr + 8, point->Y, 1, endian_arch);	/* Y */
	  gaiaExport64 (ptr + 16, point->Z, 1, endian_arch);	/* Z */
	  ptr += 24;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  if (type == GAIA_GEOSWKB_MULTILINESTRINGZ
	      || type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
	    {
		*ptr = 0x01;
		/* it's a collection: the CLASS TYPE for this element */
		gaiaExport32 (ptr + 1, GAIA_GEOSWKB_LINESTRINGZ, 1,
			      endian_arch);
		ptr += 5;
	    }
	  gaiaExport32 (ptr, line->Points, 1, endian_arch);	/* # points */
	  ptr += 4;
	  for (iv = 0; iv < line->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y */
		gaiaExport64 (ptr + 16, z, 1, endian_arch);	/* Z */
		ptr += 24;
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  if (type == GAIA_GEOSWKB_MULTIPOLYGONZ
	      || type == GAIA_GEOSWKB_GEOMETRYCOLLECTIONZ)
	    {
		*ptr = 0x01;
		/* it's a collection: the CLASS TYPE for this element */
		gaiaExport32 (ptr + 1, GAIA_GEOSWKB_POLYGONZ, 1, endian_arch);
		ptr += 5;
	    }
	  gaiaExport32 (ptr, polyg->NumInteriors + 1, 1, endian_arch);	/* # rings */
	  rng = polyg->Exterior;
	  gaiaExport32 (ptr + 4, rng->Points, 1, endian_arch);	/* # points - exterior ring */
	  ptr += 8;
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		z = 0.0;
		m = 0.0;
		if (rng->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		  }
		else if (rng->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		  }
		else if (rng->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		  }
		gaiaExport64 (ptr, x, 1, endian_arch);	/* X - exterior ring */
		gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - exterior ring */
		gaiaExport64 (ptr + 16, z, 1, endian_arch);	/* Z - exterior ring */
		ptr += 24;
	    }
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		rng = polyg->Interiors + ib;
		gaiaExport32 (ptr, rng->Points, 1, endian_arch);	/* # points - interior ring */
		ptr += 4;
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      z = 0.0;
		      m = 0.0;
		      if (rng->DimensionModel == GAIA_XY_Z)
			{
			    gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
			}
		      else if (rng->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
			}
		      else if (rng->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (rng->Coords, iv, &x, &y);
			}
		      gaiaExport64 (ptr, x, 1, endian_arch);	/* X - interior ring */
		      gaiaExport64 (ptr + 8, y, 1, endian_arch);	/* Y - interior ring */
		      gaiaExport64 (ptr + 16, z, 1, endian_arch);	/* Z - exterior ring */
		      ptr += 24;
		  }
	    }
	  polyg = polyg->Next;
      }
}

static int
vfdoGeometryType (gaiaGeomCollPtr geom)
{
/* determinates the Class for this geometry */
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    int n_points = 0;
    int n_linestrings = 0;
    int n_polygons = 0;
    if (!geom)
	return GAIA_UNKNOWN;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counts how many points are there */
	  n_points++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counts how many linestrings are there */
	  n_linestrings++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counts how many polygons are there */
	  n_polygons++;
	  polyg = polyg->Next;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons == 0)
	return GAIA_UNKNOWN;
    if (n_points == 1 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT)
	      return GAIA_MULTIPOINT;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_POINT;
      }
    if (n_points > 0 && n_linestrings == 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_MULTIPOINT;
      }
    if (n_points == 0 && n_linestrings == 1 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_MULTILINESTRING)
	      return GAIA_MULTILINESTRING;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_LINESTRING;
      }
    if (n_points == 0 && n_linestrings > 0 && n_polygons == 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_MULTILINESTRING;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOLYGON)
	      return GAIA_MULTIPOLYGON;
	  else if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_POLYGON;
      }
    if (n_points == 0 && n_linestrings == 0 && n_polygons > 0)
      {
	  if (geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      return GAIA_GEOMETRYCOLLECTION;
	  else
	      return GAIA_MULTIPOLYGON;
      }
    return GAIA_GEOMETRYCOLLECTION;
}

static int
vfdo_insert_row (VirtualFDOPtr p_vt, sqlite3_int64 * rowid, int argc,
		 sqlite3_value ** argv)
{
/* trying to insert a row into FDO-OGR real-table */
    sqlite3_stmt *stmt;
    int ret;
    int i;
    int ic;
    int ig;
    int geom_done;
    int err_geom = 0;
    int geom_constraint_err = 0;
    char prefix[16];
    const char *text;
    const unsigned char *blob;
    gaiaOutBuffer out_buf;
    unsigned char *blob_wkb;
    int size;
    char sql[4096];
    char buf[256];
    char xname[1024];
    gaiaGeomCollPtr geom = NULL;
    strcpy (xname, p_vt->table);
    vfdo_double_quoted_sql (xname);
    sprintf (sql, "INSERT INTO %s ", xname);
    for (ic = 0; ic < p_vt->nColumns; ic++)
      {
	  if (ic == 0)
	      strcpy (prefix, "(");
	  else
	      strcpy (prefix, ", ");
	  strcpy (xname, *(p_vt->Column + ic));
	  vfdo_double_quoted_sql (xname);
	  sprintf (buf, "%s%s", prefix, xname);
	  strcat (sql, buf);
      }
    strcat (sql, ") VALUES ");
    for (ic = 0; ic < p_vt->nColumns; ic++)
      {
	  if (ic == 0)
	      strcpy (prefix, "(");
	  else
	      strcpy (prefix, ", ");
	  sprintf (buf, "%s?", prefix);
	  strcat (sql, buf);
      }
    strcat (sql, ")");
    ret = sqlite3_prepare_v2 (p_vt->db, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	return SQLITE_ERROR;
    for (i = 2; i < argc; i++)
      {
	  geom_done = 0;
	  for (ig = 0; ig < p_vt->nGeometries; ig++)
	    {
		if (strcasecmp
		    (*(p_vt->Column + i - 2), *(p_vt->GeoColumn + ig)) == 0)
		  {
		      /* this one is a Geometry column */
		      if (sqlite3_value_type (argv[i]) == SQLITE_BLOB)
			{
			    blob = sqlite3_value_blob (argv[i]);
			    size = sqlite3_value_bytes (argv[i]);
			    geom = gaiaFromSpatiaLiteBlobWkb (blob, size);
			    if (geom)
			      {
				  if (geom->Srid != *(p_vt->Srid + ig))
				    {
					/* SRID constraint violation */
					geom_constraint_err = 1;
					goto error;
				    }
				  /* checking for TYPE constraint violation */
				  if (vfdoGeometryType (geom) !=
				      *(p_vt->GeoType + ig))
				    {
					/* Geometry TYPE constraint violation */
					geom_constraint_err = 1;
					goto error;
				    }
				  switch (*(p_vt->Format + ig))
				    {
				    case FDO_OGR_WKT:
					gaiaOutBufferInitialize (&out_buf);
					if (*(p_vt->CoordDimensions + ig) ==
					    GAIA_XY_Z)
					    vfdoOutWkt3D (&out_buf, geom);
					else
					    gaiaOutWkt (&out_buf, geom);
					if (out_buf.Error == 0
					    && out_buf.Buffer != NULL)
					  {
					      sqlite3_bind_text (stmt, i - 1,
								 out_buf.Buffer,
								 out_buf.WriteOffset,
								 free);
					      out_buf.Buffer = NULL;
					      gaiaOutBufferReset (&out_buf);
					  }
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    case FDO_OGR_WKB:
					if (*(p_vt->CoordDimensions + ig) ==
					    GAIA_XY_Z)
					    vfdoToWkb3D (geom, &blob_wkb,
							 &size);
					else
					    gaiaToWkb (geom, &blob_wkb, &size);
					if (blob_wkb)
					    sqlite3_bind_blob (stmt, i - 1,
							       blob_wkb, size,
							       free);
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    case FDO_OGR_FGF:
					gaiaToFgf (geom, &blob_wkb, &size,
						   *(p_vt->CoordDimensions +
						     ig));
					if (blob_wkb)
					    sqlite3_bind_blob (stmt, i - 1,
							       blob_wkb, size,
							       free);
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    case FDO_OGR_SPLITE:
					gaiaToSpatiaLiteBlobWkb (geom,
								 &blob_wkb,
								 &size);
					if (blob_wkb)
					    sqlite3_bind_blob (stmt, i - 1,
							       blob_wkb, size,
							       free);
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    default:
					err_geom = 1;
					goto error;
					break;
				    };
			      }
			    else
			      {
				  err_geom = 1;
				  goto error;
			      }
			}
		      else if (sqlite3_value_type (argv[i]) == SQLITE_NULL)
			  sqlite3_bind_null (stmt, i - 1);
		      else
			{
			    err_geom = 1;
			    goto error;
			}
		      geom_done = 1;
		  }
	    }
	  if (geom_done)
	    {
		if (geom)
		  {
		      /* memory cleanup: Kashif Rasul 14 Jan 2010 */
		      gaiaFreeGeomColl (geom);
		      geom = NULL;
		  }
		continue;
	    }
	  switch (sqlite3_value_type (argv[i]))
	    {
	    case SQLITE_INTEGER:
		sqlite3_bind_int64 (stmt, i - 1, sqlite3_value_int64 (argv[i]));
		break;
	    case SQLITE_FLOAT:
		sqlite3_bind_double (stmt, i - 1,
				     sqlite3_value_double (argv[i]));
		break;
	    case SQLITE_TEXT:
		text = (char *) sqlite3_value_text (argv[i]);
		size = sqlite3_value_bytes (argv[i]);
		sqlite3_bind_text (stmt, i - 1, text, size, SQLITE_STATIC);
		break;
	    case SQLITE_BLOB:
		blob = sqlite3_value_blob (argv[i]);
		size = sqlite3_value_bytes (argv[i]);
		sqlite3_bind_blob (stmt, i - 1, blob, size, SQLITE_STATIC);
		break;
	    case SQLITE_NULL:
	    default:
		sqlite3_bind_null (stmt, i - 1);
		break;
	    };
      }
  error:
    if (geom)
      {
	  /* memory cleanup: Kashif Rasul 14 Jan 2010 */
	  gaiaFreeGeomColl (geom);
      }
    if (err_geom || geom_constraint_err)
      {
	  sqlite3_finalize (stmt);
	  return SQLITE_CONSTRAINT;
      }
    ret = sqlite3_step (stmt);
    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	;
    else
      {
	  sqlite3_finalize (stmt);
	  return ret;
      }
    sqlite3_finalize (stmt);
    *rowid = sqlite3_last_insert_rowid (p_vt->db);
    return SQLITE_OK;
}

static int
vfdo_update_row (VirtualFDOPtr p_vt, sqlite3_int64 rowid, int argc,
		 sqlite3_value ** argv)
{
/* trying to update a row in FDO-OGR real-table */
    sqlite3_stmt *stmt;
    int ret;
    int i;
    int ic;
    int ig;
    int geom_done;
    int err_geom = 0;
    int geom_constraint_err = 0;
    char prefix[16];
    const char *text;
    const unsigned char *blob;
    gaiaOutBuffer out_buf;
    unsigned char *blob_wkb;
    int size;
    char sql[4096];
    char buf[256];
    char xname[1024];
    gaiaGeomCollPtr geom;
    strcpy (xname, p_vt->table);
    vfdo_double_quoted_sql (xname);
    sprintf (sql, "UPDATE %s SET", xname);
    for (ic = 0; ic < p_vt->nColumns; ic++)
      {
	  if (ic == 0)
	      strcpy (prefix, " ");
	  else
	      strcpy (prefix, ", ");
	  strcpy (xname, *(p_vt->Column + ic));
	  vfdo_double_quoted_sql (xname);
	  sprintf (buf, "%s%s = ?", prefix, xname);
	  strcat (sql, buf);
      }
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT: M$ rutime doesn't supports %lld for 64 bits */
    sprintf (buf, " WHERE ROWID = %I64d", rowid);
#else
    sprintf (buf, " WHERE ROWID = %lld", rowid);
#endif
    strcat (sql, buf);
    ret = sqlite3_prepare_v2 (p_vt->db, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	return SQLITE_ERROR;
    for (i = 2; i < argc; i++)
      {
	  geom_done = 0;
	  for (ig = 0; ig < p_vt->nGeometries; ig++)
	    {
		if (strcasecmp
		    (*(p_vt->Column + i - 2), *(p_vt->GeoColumn + ig)) == 0)
		  {
		      /* this one is a Geometry column */
		      if (sqlite3_value_type (argv[i]) == SQLITE_BLOB)
			{
			    blob = sqlite3_value_blob (argv[i]);
			    size = sqlite3_value_bytes (argv[i]);
			    geom = gaiaFromSpatiaLiteBlobWkb (blob, size);
			    if (geom)
			      {
				  if (geom->Srid != *(p_vt->Srid + ig))
				    {
					/* SRID constraint violation */
					geom_constraint_err = 1;
					goto error;
				    }
				  /* checking for TYPE constraint violation */
				  if (vfdoGeometryType (geom) !=
				      *(p_vt->GeoType + ig))
				    {
					/* Geometry TYPE constraint violation */
					geom_constraint_err = 1;
					goto error;
				    }
				  switch (*(p_vt->Format + ig))
				    {
				    case FDO_OGR_WKT:
					gaiaOutBufferInitialize (&out_buf);
					if (*(p_vt->CoordDimensions + ig) ==
					    GAIA_XY_Z)
					    vfdoOutWkt3D (&out_buf, geom);
					else
					    gaiaOutWkt (&out_buf, geom);
					if (out_buf.Error == 0
					    && out_buf.Buffer != NULL)
					  {
					      sqlite3_bind_text (stmt, i - 1,
								 out_buf.Buffer,
								 out_buf.WriteOffset,
								 free);
					      out_buf.Buffer = NULL;
					      gaiaOutBufferReset (&out_buf);
					  }
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    case FDO_OGR_WKB:
					if (*(p_vt->CoordDimensions + ig) ==
					    GAIA_XY_Z)
					    vfdoToWkb3D (geom, &blob_wkb,
							 &size);
					else
					    gaiaToWkb (geom, &blob_wkb, &size);
					if (blob_wkb)
					    sqlite3_bind_blob (stmt, i - 1,
							       blob_wkb, size,
							       free);
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    case FDO_OGR_FGF:
					gaiaToFgf (geom, &blob_wkb, &size,
						   *(p_vt->CoordDimensions +
						     ig));
					if (blob_wkb)
					    sqlite3_bind_blob (stmt, i - 1,
							       blob_wkb, size,
							       free);
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    case FDO_OGR_SPLITE:
					gaiaToSpatiaLiteBlobWkb (geom,
								 &blob_wkb,
								 &size);
					if (blob_wkb)
					    sqlite3_bind_blob (stmt, i - 1,
							       blob_wkb, size,
							       free);
					else
					  {
					      err_geom = 1;
					      goto error;
					  }
					break;
				    default:
					err_geom = 1;
					goto error;
					break;
				    };
			      }
			    else
			      {
				  err_geom = 1;
				  goto error;
			      }
			}
		      else if (sqlite3_value_type (argv[i]) == SQLITE_NULL)
			  sqlite3_bind_null (stmt, i - 1);
		      else
			{
			    err_geom = 1;
			    goto error;
			}
		      geom_done = 1;
		  }
	    }
	  if (geom_done)
	      continue;
	  switch (sqlite3_value_type (argv[i]))
	    {
	    case SQLITE_INTEGER:
		sqlite3_bind_int64 (stmt, i - 1, sqlite3_value_int64 (argv[i]));
		break;
	    case SQLITE_FLOAT:
		sqlite3_bind_double (stmt, i - 1,
				     sqlite3_value_double (argv[i]));
		break;
	    case SQLITE_TEXT:
		text = (char *) sqlite3_value_text (argv[i]);
		size = sqlite3_value_bytes (argv[i]);
		sqlite3_bind_text (stmt, i - 1, text, size, SQLITE_STATIC);
		break;
	    case SQLITE_BLOB:
		blob = sqlite3_value_blob (argv[i]);
		size = sqlite3_value_bytes (argv[i]);
		sqlite3_bind_blob (stmt, i - 1, blob, size, SQLITE_STATIC);
		break;
	    case SQLITE_NULL:
	    default:
		sqlite3_bind_null (stmt, i - 1);
		break;
	    };
      }
  error:
    if (err_geom || geom_constraint_err)
      {
	  sqlite3_finalize (stmt);
	  return SQLITE_CONSTRAINT;
      }
    ret = sqlite3_step (stmt);
    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	;
    else
      {
	  sqlite3_finalize (stmt);
	  return ret;
      }
    sqlite3_finalize (stmt);
    return SQLITE_OK;
}

static int
vfdo_delete_row (VirtualFDOPtr p_vt, sqlite3_int64 rowid)
{
/* trying to delete a row from FDO-OGR real-table */
    char sql[1024];
    int ret;
    char xname[1024];
    strcpy (xname, p_vt->table);
    vfdo_double_quoted_sql (xname);
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT: M$ runtime doesn't supports %lld for 64 bits */
    sprintf (sql, "DELETE FROM %s WHERE ROWID = %I64d", xname, rowid);
#else
    sprintf (sql, "DELETE FROM %s WHERE ROWID = %lld", xname, rowid);
#endif
    ret = sqlite3_exec (p_vt->db, sql, NULL, NULL, NULL);
    return ret;
}

char *
vfdo_convertWKT3D (const char *wkt)
{
/* converting WKT 3D */
    char *out;
    char *o;
    const char *p = wkt;
    int len = strlen (wkt);
    int extra = 0;
    while (*p != '\0')
      {
	  if (strncasecmp (p, "POINT", 5) == 0)
	    {
		p += 5;
		extra++;
		continue;
	    }
	  if (strncasecmp (p, "LINESTRING", 10) == 0)
	    {
		p += 10;
		extra++;
		continue;
	    }
	  if (strncasecmp (p, "POLYGON", 7) == 0)
	    {
		p += 7;
		extra++;
		continue;
	    }
	  if (strncasecmp (p, "MULTIPOINT", 10) == 0)
	    {
		p += 10;
		extra++;
		continue;
	    }
	  if (strncasecmp (p, "MULTILINESTRING", 15) == 0)
	    {
		p += 15;
		extra++;
		continue;
	    }
	  if (strncasecmp (p, "MULTIPOLYGON", 12) == 0)
	    {
		p += 12;
		extra++;
		continue;
	    }
	  if (strncasecmp (p, "GEOMETRYCOLLECTION", 18) == 0)
	    {
		p += 18;
		extra++;
		continue;
	    }
	  p++;
      }
    out = malloc (len + extra + 1);
    o = out;
    p = wkt;
    while (*p != '\0')
      {
	  if (strncasecmp (p, "POINT", 5) == 0)
	    {
		strcpy (o, "POINTZ");
		o += 6;
		p += 5;
		continue;
	    }
	  if (strncasecmp (p, "LINESTRING", 10) == 0)
	    {
		strcpy (o, "LINESTRINGZ");
		o += 11;
		p += 10;
		continue;
	    }
	  if (strncasecmp (p, "POLYGON", 7) == 0)
	    {
		strcpy (o, "POLYGONZ");
		o += 8;
		p += 7;
		continue;
	    }
	  if (strncasecmp (p, "MULTIPOINT", 10) == 0)
	    {
		strcpy (o, "MULTIPOINTZ");
		o += 11;
		p += 10;
		continue;
	    }
	  if (strncasecmp (p, "MULTILINESTRING", 15) == 0)
	    {
		strcpy (o, "MULTILINESTRINGZ");
		o += 16;
		p += 15;
		continue;
	    }
	  if (strncasecmp (p, "MULTIPOLYGON", 12) == 0)
	    {
		strcpy (o, "MULTIPOLYGONZ");
		o += 13;
		p += 12;
		continue;
	    }
	  if (strncasecmp (p, "GEOMETRYCOLLECTION", 18) == 0)
	    {
		strcpy (o, "GEOMETRYCOLLECTIONZ");
		o += 19;
		p += 18;
		continue;
	    }
	  *o++ = *p++;
      }
    *o = '\0';
    return out;
}

static void
vfdo_read_row (VirtualFDOCursorPtr cursor)
{
/* trying to read a row from FDO-OGR real-table */
    sqlite3_stmt *stmt;
    int ret;
    char sql[4096];
    char buf[256];
    int ic;
    int ig;
    unsigned char *wkt;
    const char *text;
    const unsigned char *blob;
    unsigned char *xblob;
    int size;
    sqlite3_int64 pk;
    int geom_done;
    gaiaGeomCollPtr geom;
    char xname[1024];
    strcpy (sql, "SELECT ROWID");
    for (ic = 0; ic < cursor->pVtab->nColumns; ic++)
      {
	  strcpy (xname, *(cursor->pVtab->Column + ic));
	  vfdo_double_quoted_sql (xname);
	  sprintf (buf, ",%s", xname);
	  strcat (sql, buf);
      }
    strcpy (xname, cursor->pVtab->table);
    vfdo_double_quoted_sql (xname);
    sprintf (buf,
#if defined(_WIN32) || defined (__MINGW32__)
/* CAVEAT: M$ runtime doesn't supports %lld for 64 bits */
	     " FROM %s WHERE ROWID >= %I64d",
#else
	     " FROM %s WHERE ROWID >= %lld",
#endif
	     xname, cursor->current_row);
    strcat (sql, buf);
    ret =
	sqlite3_prepare_v2 (cursor->pVtab->db, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  /* an error occurred */
	  cursor->eof = 1;
	  return;
      }
    ret = sqlite3_step (stmt);
    if (ret == SQLITE_ROW)
      {
	  pk = sqlite3_column_int64 (stmt, 0);
	  for (ic = 0; ic < cursor->pVtab->nColumns; ic++)
	    {
		/* fetching column values */
		geom_done = 0;
		for (ig = 0; ig < cursor->pVtab->nGeometries; ig++)
		  {
		      if (strcasecmp
			  (*(cursor->pVtab->Column + ic),
			   *(cursor->pVtab->GeoColumn + ig)) == 0)
			{
			    /* this one is a Geometry column */
			    switch (*(cursor->pVtab->Format + ig))
			      {
			      case FDO_OGR_WKT:
				  if (sqlite3_column_type (stmt, ic + 1) ==
				      SQLITE_TEXT)
				    {
					/* trying to parse a WKT Geometry */
					char delete_wkt = 0;
					wkt = (unsigned char *)
					    sqlite3_column_text (stmt, ic + 1);
					if (*
					    (cursor->pVtab->CoordDimensions +
					     ig) == GAIA_XY_Z)
					  {
					      wkt = (unsigned char *)
						  vfdo_convertWKT3D ((char *)
								     wkt);
					      if (wkt == NULL)
						{
						    value_set_null (*
								    (cursor->
								     pVtab->
								     Value +
								     ic));
						    continue;
						}
					      delete_wkt = 1;
					  }
					geom = gaiaParseWkt (wkt, -1);
					if (delete_wkt)
					    free ((void *) wkt);
					if (!geom)
					    value_set_null (*
							    (cursor->pVtab->
							     Value + ic));
					else
					  {
					      geom->Srid =
						  *(cursor->pVtab->Srid + ig);
					      gaiaToSpatiaLiteBlobWkb (geom,
								       &xblob,
								       &size);
					      if (xblob)
						  value_set_blob (*
								  (cursor->
								   pVtab->
								   Value + ic),
								  xblob, size);
					      else
						  value_set_null (*
								  (cursor->
								   pVtab->
								   Value + ic));
					      gaiaFreeGeomColl (geom);
					  }
				    }
				  else
				      value_set_null (*
						      (cursor->pVtab->Value +
						       ic));
				  break;
			      case FDO_OGR_WKB:
				  if (sqlite3_column_type (stmt, ic + 1) ==
				      SQLITE_BLOB)
				    {
					/* trying to parse a WKB Geometry */
					blob =
					    sqlite3_column_blob (stmt, ic + 1);
					size =
					    sqlite3_column_bytes (stmt, ic + 1);
					geom = gaiaFromWkb (blob, size);
					if (!geom)
					    value_set_null (*
							    (cursor->pVtab->
							     Value + ic));
					else
					  {
					      geom->Srid =
						  *(cursor->pVtab->Srid + ig);
					      gaiaToSpatiaLiteBlobWkb (geom,
								       &xblob,
								       &size);
					      if (xblob)
						  value_set_blob (*
								  (cursor->
								   pVtab->
								   Value + ic),
								  xblob, size);
					      else
						  value_set_null (*
								  (cursor->
								   pVtab->
								   Value + ic));
					      gaiaFreeGeomColl (geom);
					  }
				    }
				  else
				      value_set_null (*
						      (cursor->pVtab->Value +
						       ic));
				  break;
			      case FDO_OGR_FGF:
				  if (sqlite3_column_type (stmt, ic + 1) ==
				      SQLITE_BLOB)
				    {
					/* trying to parse an FGF Geometry */
					blob =
					    sqlite3_column_blob (stmt, ic + 1);
					size =
					    sqlite3_column_bytes (stmt, ic + 1);
					geom = gaiaFromFgf (blob, size);
					if (!geom)
					    value_set_null (*
							    (cursor->pVtab->
							     Value + ic));
					else
					  {
					      geom->Srid =
						  *(cursor->pVtab->Srid + ig);
					      gaiaToSpatiaLiteBlobWkb (geom,
								       &xblob,
								       &size);
					      if (xblob)
						  value_set_blob (*
								  (cursor->
								   pVtab->
								   Value + ic),
								  xblob, size);
					      else
						  value_set_null (*
								  (cursor->
								   pVtab->
								   Value + ic));
					      gaiaFreeGeomColl (geom);
					  }
				    }
				  else
				      value_set_null (*
						      (cursor->pVtab->Value +
						       ic));
				  break;
			      case FDO_OGR_SPLITE:
				  if (sqlite3_column_type (stmt, ic + 1) ==
				      SQLITE_BLOB)
				    {
					/* trying to parse a SPATIALITE Geometry */
					blob =
					    sqlite3_column_blob (stmt, ic + 1);
					size =
					    sqlite3_column_bytes (stmt, ic + 1);
					value_set_blob (*
							(cursor->pVtab->Value +
							 ic), blob, size);
				    }
				  else
				      value_set_null (*
						      (cursor->pVtab->Value +
						       ic));
				  break;
			      default:
				  value_set_null (*(cursor->pVtab->Value + ic));
				  break;
			      };
			    geom_done = 1;
			}
		  }
		if (geom_done)
		    continue;
		switch (sqlite3_column_type (stmt, ic + 1))
		  {
		  case SQLITE_INTEGER:
		      value_set_int (*(cursor->pVtab->Value + ic),
				     sqlite3_column_int64 (stmt, ic + 1));
		      break;
		  case SQLITE_FLOAT:
		      value_set_double (*(cursor->pVtab->Value + ic),
					sqlite3_column_double (stmt, ic + 1));
		      break;
		  case SQLITE_TEXT:
		      text = (char *) sqlite3_column_text (stmt, ic + 1);
		      size = sqlite3_column_bytes (stmt, ic + 1);
		      value_set_text (*(cursor->pVtab->Value + ic), text, size);
		      break;
		  case SQLITE_BLOB:
		      blob = sqlite3_column_blob (stmt, ic + 1);
		      size = sqlite3_column_bytes (stmt, ic + 1);
		      value_set_blob (*(cursor->pVtab->Value + ic), blob, size);
		      break;
		  case SQLITE_NULL:
		  default:
		      value_set_null (*(cursor->pVtab->Value + ic));
		      break;
		  };
	    }
      }
    else
      {
	  /* an error occurred */
	  sqlite3_finalize (stmt);
	  cursor->eof = 1;
	  return;
      }
    sqlite3_finalize (stmt);
    cursor->eof = 0;
    cursor->current_row = pk;
}


static int
vfdo_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	     sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table connected to some FDO-OGR table */
    char vtable[1024];
    char table[1024];
    int ret;
    int i;
    int len;
    int n_rows;
    int n_columns;
    const char *col_name;
    const char *col_type;
    const char *format;
    int coord_dimension;
    int not_null;
    int srid;
    int type;
    char **results;
    char sql[4096];
    char buf[256];
    char prefix[16];
    char xname[1024];
    VirtualFDOPtr p_vt = NULL;
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
/* checking for table_name */
    if (argc == 4)
      {
	  strcpy (vtable, argv[2]);
	  vfdo_dequote (vtable);
	  strcpy (table, argv[3]);
	  vfdo_dequote (table);
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualFDO module] CREATE VIRTUAL: illegal arg list {table_name}\n");
	  return SQLITE_ERROR;
      }
/* retrieving the base table columns */
    strcpy (xname, table);
    vfdo_double_quoted_sql (xname);
    sprintf (sql, "PRAGMA table_info(%s)", xname);
    ret = sqlite3_get_table (db, sql, &results, &n_rows, &n_columns, NULL);
    if (ret != SQLITE_OK)
	goto illegal;
    if (n_rows >= 1)
      {
	  p_vt = (VirtualFDOPtr) sqlite3_malloc (sizeof (VirtualFDO));
	  if (!p_vt)
	      return SQLITE_NOMEM;
	  p_vt->db = db;
	  p_vt->nRef = 0;
	  p_vt->zErrMsg = NULL;
	  len = strlen (table);
	  p_vt->table = sqlite3_malloc (len + 1);
	  strcpy (p_vt->table, table);
	  p_vt->nColumns = n_rows;
	  p_vt->Column = sqlite3_malloc (sizeof (char *) * n_rows);
	  p_vt->Type = sqlite3_malloc (sizeof (char *) * n_rows);
	  p_vt->NotNull = sqlite3_malloc (sizeof (int) * n_rows);
	  p_vt->Value = sqlite3_malloc (sizeof (SqliteValuePtr) * n_rows);
	  for (i = 0; i < n_rows; i++)
	    {
		*(p_vt->Column + i) = NULL;
		*(p_vt->Type + i) = NULL;
		*(p_vt->NotNull + i) = -1;
		*(p_vt->Value + i) = value_alloc ();
	    }
	  p_vt->nGeometries = 0;
	  p_vt->GeoColumn = NULL;
	  p_vt->Srid = NULL;
	  p_vt->GeoType = NULL;
	  p_vt->Format = NULL;
	  p_vt->CoordDimensions = NULL;
	  for (i = 1; i <= n_rows; i++)
	    {
		col_name = results[(i * n_columns) + 1];
		col_type = results[(i * n_columns) + 2];
		if (atoi (results[(i * n_columns) + 3]) == 0)
		    not_null = 0;
		else
		    not_null = 1;
		len = strlen (col_name);
		*(p_vt->Column + (i - 1)) = sqlite3_malloc (len + 1);
		strcpy (*(p_vt->Column + (i - 1)), col_name);
		len = strlen (col_type);
		*(p_vt->Type + (i - 1)) = sqlite3_malloc (len + 1);
		strcpy (*(p_vt->Type + (i - 1)), col_type);
		*(p_vt->NotNull + (i - 1)) = not_null;
	    }
	  sqlite3_free_table (results);
      }
    else
	goto illegal;
/* retrieving the base table columns */
    strcpy (sql,
	    "SELECT f_geometry_column, geometry_type, srid, geometry_format, coord_dimension\n");
    strcat (sql, "FROM geometry_columns WHERE f_table_name LIKE '");
    strcpy (xname, table);
    vfdo_clean_sql_string (xname);
    strcat (sql, xname);
    strcat (sql, "'");
    ret = sqlite3_get_table (db, sql, &results, &n_rows, &n_columns, NULL);
    if (ret != SQLITE_OK)
	goto illegal;
    if (n_rows >= 1)
      {
	  p_vt->nGeometries = n_rows;
	  p_vt->GeoColumn = sqlite3_malloc (sizeof (char *) * n_rows);
	  p_vt->Srid = sqlite3_malloc (sizeof (char *) * n_rows);
	  p_vt->GeoType = sqlite3_malloc (sizeof (int) * n_rows);
	  p_vt->Format = sqlite3_malloc (sizeof (int) * n_rows);
	  p_vt->CoordDimensions = sqlite3_malloc (sizeof (int) * n_rows);
	  for (i = 0; i < n_rows; i++)
	    {
		*(p_vt->GeoColumn + i) = NULL;
		*(p_vt->Srid + i) = -1;
		*(p_vt->GeoType + i) = -1;
		*(p_vt->Format + i) = FDO_OGR_NONE;
		*(p_vt->CoordDimensions + i) = GAIA_XY;
	    }
	  for (i = 1; i <= n_rows; i++)
	    {
		col_name = results[(i * n_columns) + 0];
		type = atoi (results[(i * n_columns) + 1]);
		srid = atoi (results[(i * n_columns) + 2]);
		format = results[(i * n_columns) + 3];
		coord_dimension = atoi (results[(i * n_columns) + 4]);
		len = strlen (col_name);
		*(p_vt->GeoColumn + (i - 1)) = sqlite3_malloc (len + 1);
		strcpy (*(p_vt->GeoColumn + (i - 1)), col_name);
		*(p_vt->GeoType + (i - 1)) = type;
		*(p_vt->Srid + (i - 1)) = srid;
		if (strcasecmp (format, "WKT") == 0)
		    *(p_vt->Format + (i - 1)) = FDO_OGR_WKT;
		if (strcasecmp (format, "WKB") == 0)
		    *(p_vt->Format + (i - 1)) = FDO_OGR_WKB;
		if (strcasecmp (format, "FGF") == 0)
		    *(p_vt->Format + (i - 1)) = FDO_OGR_FGF;
		if (strcasecmp (format, "SPATIALITE") == 0)
		    *(p_vt->Format + (i - 1)) = FDO_OGR_SPLITE;
		if (coord_dimension == 3)
		    *(p_vt->CoordDimensions + (i - 1)) = GAIA_XY_Z;
		else if (coord_dimension == 4)
		    *(p_vt->CoordDimensions + (i - 1)) = GAIA_XY_Z_M;
		else
		    *(p_vt->CoordDimensions + (i - 1)) = GAIA_XY;
	    }
	  sqlite3_free_table (results);
      }
    else
	goto illegal;
/* preparing the COLUMNs for this VIRTUAL TABLE */
    strcpy (sql, "CREATE TABLE ");
    strcpy (xname, vtable);
    vfdo_double_quoted_sql (xname);
    strcat (sql, xname);
    strcat (sql, " ");
    for (i = 0; i < p_vt->nColumns; i++)
      {
	  if (i == 0)
	      strcpy (prefix, "(");
	  else
	      strcpy (prefix, ", ");
	  strcpy (xname, *(p_vt->Column + i));
	  vfdo_double_quoted_sql (xname);
	  sprintf (buf, "%s%s %s", prefix, xname, *(p_vt->Type + i));
	  if (*(p_vt->NotNull + i))
	      strcat (buf, " NOT NULL");
	  strcat (sql, buf);
      }
    strcat (sql, ")");
    if (sqlite3_declare_vtab (db, sql) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualFDO module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       sql);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    return SQLITE_OK;
  illegal:
/* something is going the wrong way */
    if (p_vt)
	free_table (p_vt);
    *pzErr =
	sqlite3_mprintf
	("[VirtualFDO module] '%s' isn't a valid FDO-OGR Geometry table\n",
	 table);
    return SQLITE_ERROR;
}

static int
vfdo_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	      sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table to some shapefile - simply aliases vfdo_create() */
    return vfdo_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
vfdo_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIndex)
{
/* best index selection */
    if (pVTab || pIndex)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vfdo_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    VirtualFDOPtr p_vt = (VirtualFDOPtr) pVTab;
    free_table (p_vt);
    return SQLITE_OK;
}

static int
vfdo_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases vfdo_disconnect() */
    return vfdo_disconnect (pVTab);
}

static int
vfdo_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    VirtualFDOCursorPtr cursor =
	(VirtualFDOCursorPtr) sqlite3_malloc (sizeof (VirtualFDOCursor));
    if (cursor == NULL)
	return SQLITE_ERROR;
    cursor->pVtab = (VirtualFDOPtr) pVTab;
    cursor->current_row = LONG64_MIN;
    cursor->eof = 0;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    vfdo_read_row (cursor);
    return SQLITE_OK;
}

static int
vfdo_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    sqlite3_free (pCursor);
    return SQLITE_OK;
}

static int
vfdo_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	     int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    if (pCursor || idxNum || idxStr || argc || argv)
	pCursor = pCursor;	/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vfdo_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching next row from cursor */
    VirtualFDOCursorPtr cursor = (VirtualFDOCursorPtr) pCursor;
    (cursor->current_row)++;
    vfdo_read_row (cursor);
    return SQLITE_OK;
}

static int
vfdo_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    VirtualFDOCursorPtr cursor = (VirtualFDOCursorPtr) pCursor;
    return cursor->eof;
}

static int
vfdo_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	     int column)
{
/* fetching value for the Nth column */
    VirtualFDOCursorPtr cursor = (VirtualFDOCursorPtr) pCursor;
    SqliteValuePtr value;
    if (column >= 0 && column < cursor->pVtab->nColumns)
      {
	  value = *(cursor->pVtab->Value + column);
	  switch (value->Type)
	    {
	    case SQLITE_INTEGER:
		sqlite3_result_int64 (pContext, value->IntValue);
		break;
	    case SQLITE_FLOAT:
		sqlite3_result_double (pContext, value->DoubleValue);
		break;
	    case SQLITE_TEXT:
		sqlite3_result_text (pContext, value->Text, value->Size,
				     SQLITE_STATIC);
		break;
	    case SQLITE_BLOB:
		sqlite3_result_blob (pContext, value->Blob, value->Size,
				     SQLITE_STATIC);
		break;
	    default:
		sqlite3_result_null (pContext);
		break;
	    };
      }
    else
	sqlite3_result_null (pContext);
    return SQLITE_OK;
}

static int
vfdo_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    VirtualFDOCursorPtr cursor = (VirtualFDOCursorPtr) pCursor;
    *pRowid = cursor->current_row;
    return SQLITE_OK;
}

static int
vfdo_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	     sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    sqlite3_int64 rowid = 0;
    int ret;
    VirtualFDOPtr p_vt = (VirtualFDOPtr) pVTab;
    if (argc == 1)
      {
	  /* performing a DELETE */
	  if (sqlite3_value_type (argv[0]) == SQLITE_INTEGER)
	    {
		rowid = sqlite3_value_int64 (argv[0]);
		ret = vfdo_delete_row (p_vt, rowid);
	    }
	  else
	      ret = SQLITE_MISMATCH;
      }
    else
      {
	  if (sqlite3_value_type (argv[0]) == SQLITE_NULL)
	    {
		/* performing an INSERT */
		ret = vfdo_insert_row (p_vt, &rowid, argc, argv);
		if (ret == SQLITE_OK)
		    *pRowid = rowid;
	    }
	  else
	    {
		/* performing an UPDATE */
		rowid = sqlite3_value_int64 (argv[0]);
		ret = vfdo_update_row (p_vt, rowid, argc, argv);
	    }
      }
    return ret;
}

static int
vfdo_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vfdo_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vfdo_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vfdo_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3VirtualFDOInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    my_fdo_module.iVersion = 1;
    my_fdo_module.xCreate = &vfdo_create;
    my_fdo_module.xConnect = &vfdo_connect;
    my_fdo_module.xBestIndex = &vfdo_best_index;
    my_fdo_module.xDisconnect = &vfdo_disconnect;
    my_fdo_module.xDestroy = &vfdo_destroy;
    my_fdo_module.xOpen = &vfdo_open;
    my_fdo_module.xClose = &vfdo_close;
    my_fdo_module.xFilter = &vfdo_filter;
    my_fdo_module.xNext = &vfdo_next;
    my_fdo_module.xEof = &vfdo_eof;
    my_fdo_module.xColumn = &vfdo_column;
    my_fdo_module.xRowid = &vfdo_rowid;
    my_fdo_module.xUpdate = &vfdo_update;
    my_fdo_module.xBegin = &vfdo_begin;
    my_fdo_module.xSync = &vfdo_sync;
    my_fdo_module.xCommit = &vfdo_commit;
    my_fdo_module.xRollback = &vfdo_rollback;
    my_fdo_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "VirtualFDO", &my_fdo_module, NULL, 0);
    return rc;
}

int
virtualfdo_extension_init (sqlite3 * db)
{
    return sqlite3VirtualFDOInit (db);
}
/**************** End file: virtualfdo.c **********/


/**************** Begin file: virtualtext.c **********/

#if defined(_WIN32) && !defined(__MINGW32__)
/* MSVC strictly requires this include [off_t] */
/* #include <sys/types.h> */
#endif

/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/spatialite.h> */
/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

#if OMIT_ICONV == 0		/* if ICONV is disabled no TXT support is available */

struct sqlite3_module virtualtext_module;

typedef struct VirtualTextStruct
{
/* extends the sqlite3_vtab struct */
    const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
    int nRef;			/* # references: USED INTERNALLY BY SQLITE */
    char *zErrMsg;		/* error message: USED INTERNALLY BY SQLITE */
    sqlite3 *db;		/* the sqlite db holding the virtual table */
    gaiaTextReaderPtr reader;	/* the TextReader object */
} VirtualText;
typedef VirtualText *VirtualTextPtr;

typedef struct VirtualTextConstraintStruct
{
/* a constraint to be verified for xFilter */
    int iColumn;		/* Column on left-hand side of constraint */
    int op;			/* Constraint operator */
    char valueType;		/* value Type ('I'=int,'D'=double,'T'=text) */
    sqlite3_int64 intValue;	/* Int64 comparison value */
    double dblValue;		/* Double comparison value */
    char *txtValue;		/* Text comparison value */
    struct VirtualTextConstraintStruct *next;
} VirtualTextConstraint;
typedef VirtualTextConstraint *VirtualTextConstraintPtr;

typedef struct VirtualTextCursortStruct
{
/* extends the sqlite3_vtab_cursor struct */
    VirtualTextPtr pVtab;	/* Virtual table of this cursor */
    long current_row;		/* the current row ID */
    int eof;			/* the EOF marker */
    VirtualTextConstraintPtr firstConstraint;
    VirtualTextConstraintPtr lastConstraint;
} VirtualTextCursor;
typedef VirtualTextCursor *VirtualTextCursorPtr;

static void
text_clean_integer (char *value)
{
/* cleaning an integer value */
    char last;
    char buffer[35536];
    int len = strlen (value);
    last = value[len - 1];
    if (last == '-' || last == '+')
      {
	  /* trailing sign; transforming into a leading sign */
	  *buffer = last;
	  strcpy (buffer + 1, value);
	  buffer[len - 1] = '\0';
	  strcpy (value, buffer);
      }
}

static void
text_clean_double (char *value)
{
/* cleaning an integer value */
    char *p;
    char last;
    char buffer[35536];
    int len = strlen (value);
    last = value[len - 1];
    if (last == '-' || last == '+')
      {
	  /* trailing sign; transforming into a leading sign */
	  *buffer = last;
	  strcpy (buffer + 1, value);
	  buffer[len - 1] = '\0';
	  strcpy (value, buffer);
      }
    p = value;
    while (*p != '\0')
      {
	  /* transforming COMMAs into POINTs */
	  if (*p == ',')
	      *p = '.';
	  p++;
      }
}

static int
vtxt_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	     sqlite3_vtab ** ppVTab, char **pzErr)
{
/* creates the virtual table connected to some TEXT file */
    char path[2048];
    char encoding[128];
    const char *vtable;
    const char *pEncoding = NULL;
    int len;
    gaiaTextReaderPtr text = NULL;
    const char *pPath = NULL;
    char field_separator = '\t';
    char text_separator = '"';
    char decimal_separator = '.';
    char first_line_titles = 1;
    int i;
    char sql[65535];
    int seed;
    int dup;
    int idup;
    char dummyName[4096];
    char **col_name = NULL;
    VirtualTextPtr p_vt;
    if (pAux)
	pAux = pAux;		/* unused arg warning suppression */
/* checking for TEXTfile PATH */
    if (argc >= 5 && argc <= 9)
      {
	  vtable = argv[1];
	  pPath = argv[3];
	  len = strlen (pPath);
	  if ((*(pPath + 0) == '\'' || *(pPath + 0) == '"')
	      && (*(pPath + len - 1) == '\'' || *(pPath + len - 1) == '"'))
	    {
		/* the path is enclosed between quotes - we need to dequote it */
		strcpy (path, pPath + 1);
		len = strlen (path);
		*(path + len - 1) = '\0';
	    }
	  else
	      strcpy (path, pPath);
	  pEncoding = argv[4];
	  len = strlen (pEncoding);
	  if ((*(pEncoding + 0) == '\'' || *(pEncoding + 0) == '"')
	      && (*(pEncoding + len - 1) == '\''
		  || *(pEncoding + len - 1) == '"'))
	    {
		/* the charset-name is enclosed between quotes - we need to dequote it */
		strcpy (encoding, pEncoding + 1);
		len = strlen (encoding);
		*(encoding + len - 1) = '\0';
	    }
	  else
	      strcpy (encoding, pEncoding);
	  if (argc >= 6)
	    {
		if (*(argv[5]) == '0' || *(argv[5]) == 'n' || *(argv[5]) == 'N')
		    first_line_titles = 0;
	    }
	  if (argc >= 7)
	    {
		if (strcasecmp (argv[6], "COMMA") == 0)
		    decimal_separator = ',';
		if (strcasecmp (argv[6], "POINT") == 0)
		    decimal_separator = '.';
	    }
	  if (argc >= 8)
	    {
		if (strcasecmp (argv[7], "SINGLEQUOTE") == 0)
		    text_separator = '\'';
		if (strcasecmp (argv[7], "DOUBLEQUOTE") == 0)
		    text_separator = '"';
		if (strcasecmp (argv[7], "NONE") == 0)
		    text_separator = '\0';
	    }
	  if (argc == 9)
	    {
		if (strlen (argv[8]) == 3)
		  {
		      if (strcasecmp (argv[8], "TAB") == 0)
			  field_separator = '\t';
		      if (*(argv[8] + 0) == '\'' && *(argv[8] + 2) == '\'')
			  field_separator = *(argv[8] + 1);
		  }
	    }
      }
    else
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualText module] CREATE VIRTUAL: illegal arg list\n"
	       "\t\t{ text_path, encoding [, first_row_as_titles [, [decimal_separator [, text_separator, [field_separator] ] ] ] }\n");
	  return SQLITE_ERROR;
      }
    p_vt = (VirtualTextPtr) sqlite3_malloc (sizeof (VirtualText));
    if (!p_vt)
	return SQLITE_NOMEM;
    p_vt->pModule = &virtualtext_module;
    p_vt->nRef = 0;
    p_vt->zErrMsg = NULL;
    p_vt->db = db;
    text = gaiaTextReaderAlloc (path, field_separator,
				text_separator, decimal_separator,
				first_line_titles, encoding);
    if (text)
      {
	  if (gaiaTextReaderParse (text) == 0)
	    {
		gaiaTextReaderDestroy (text);
		text = NULL;
	    }
      }
    if (!text)
      {
	  /* something is going the wrong way; creating a stupid default table */
	  fprintf (stderr, "VirtualText: invalid data source\n");
	  fflush (stderr);
	  sprintf (sql, "CREATE TABLE %s (ROWNO INTEGER)", vtable);
	  if (sqlite3_declare_vtab (db, sql) != SQLITE_OK)
	    {
		*pzErr =
		    sqlite3_mprintf
		    ("[VirtualText module] cannot build a table from TEXT file\n");
		return SQLITE_ERROR;
	    }
	  p_vt->reader = NULL;
	  *ppVTab = (sqlite3_vtab *) p_vt;
	  return SQLITE_OK;
      }
    p_vt->reader = text;
/* preparing the COLUMNs for this VIRTUAL TABLE */
    sprintf (sql, "CREATE TABLE %s (ROWNO INTEGER", vtable);
    col_name = malloc (sizeof (char *) * text->max_fields);
    seed = 0;
    for (i = 0; i < text->max_fields; i++)
      {
	  strcat (sql, ", ");
	  sprintf (dummyName, "\"%s\"", text->columns[i].name);
	  dup = 0;
	  for (idup = 0; idup < i; idup++)
	    {
		if (strcasecmp (dummyName, *(col_name + idup)) == 0)
		    dup = 1;
	    }
	  if (strcasecmp (dummyName, "ROWNO") == 0)
	      dup = 1;
	  if (dup)
	      sprintf (dummyName, "DUPCOL_%d", seed++);
	  len = strlen (dummyName);
	  *(col_name + i) = malloc (len + 1);
	  strcpy (*(col_name + i), dummyName);
	  strcat (sql, dummyName);
	  if (text->columns[i].type == VRTTXT_INTEGER)
	      strcat (sql, " INTEGER");
	  else if (text->columns[i].type == VRTTXT_DOUBLE)
	      strcat (sql, " DOUBLE");
	  else
	      strcat (sql, " TEXT");
      }
    strcat (sql, ")");
    if (col_name)
      {
	  /* releasing memory allocation for column names */
	  for (i = 0; i < text->max_fields; i++)
	      free (*(col_name + i));
	  free (col_name);
      }
    if (sqlite3_declare_vtab (db, sql) != SQLITE_OK)
      {
	  *pzErr =
	      sqlite3_mprintf
	      ("[VirtualText module] CREATE VIRTUAL: invalid SQL statement \"%s\"",
	       sql);
	  return SQLITE_ERROR;
      }
    *ppVTab = (sqlite3_vtab *) p_vt;
    return SQLITE_OK;
}

static int
vtxt_connect (sqlite3 * db, void *pAux, int argc, const char *const *argv,
	      sqlite3_vtab ** ppVTab, char **pzErr)
{
/* connects the virtual table to some shapefile - simply aliases vshp_create() */
    return vtxt_create (db, pAux, argc, argv, ppVTab, pzErr);
}

static int
vtxt_best_index (sqlite3_vtab * pVTab, sqlite3_index_info * pIndex)
{
/* best index selection */
    if (pVTab || pIndex)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vtxt_disconnect (sqlite3_vtab * pVTab)
{
/* disconnects the virtual table */
    VirtualTextPtr p_vt = (VirtualTextPtr) pVTab;
    if (p_vt->reader)
	gaiaTextReaderDestroy (p_vt->reader);
    sqlite3_free (p_vt);
    return SQLITE_OK;
}

static int
vtxt_destroy (sqlite3_vtab * pVTab)
{
/* destroys the virtual table - simply aliases vtxt_disconnect() */
    return vtxt_disconnect (pVTab);
}

static int
vtxt_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
{
/* opening a new cursor */
    gaiaTextReaderPtr text;
    VirtualTextCursorPtr cursor =
	(VirtualTextCursorPtr) sqlite3_malloc (sizeof (VirtualTextCursor));
    if (cursor == NULL)
	return SQLITE_NOMEM;
    cursor->pVtab = (VirtualTextPtr) pVTab;
    cursor->current_row = 0;
    cursor->eof = 0;
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
    *ppCursor = (sqlite3_vtab_cursor *) cursor;
    text = cursor->pVtab->reader;
    if (!text)
	cursor->eof = 1;
    else
      {
	  if (!gaiaTextReaderGetRow (text, cursor->current_row))
	      cursor->eof = 1;
      }
    return SQLITE_OK;
}


static void
vtxt_free_constraints (VirtualTextCursorPtr cursor)
{
/* memory cleanup - cursor constraints */
    VirtualTextConstraintPtr pC;
    VirtualTextConstraintPtr pCn;
    pC = cursor->firstConstraint;
    while (pC)
      {
	  pCn = pC->next;
	  if (pC->txtValue)
	      sqlite3_free (pC->txtValue);
	  sqlite3_free (pC);
	  pC = pCn;
      }
    cursor->firstConstraint = NULL;
    cursor->lastConstraint = NULL;
}

static int
vtxt_close (sqlite3_vtab_cursor * pCursor)
{
/* closing the cursor */
    VirtualTextCursorPtr cursor = (VirtualTextCursorPtr) pCursor;
    vtxt_free_constraints (cursor);
    sqlite3_free (cursor);
    return SQLITE_OK;
}


static int
vtxt_parse_constraint (const char *str, int index, int *iColumn, int *op)
{
/* parsing a constraint string */
    char buf[64];
    const char *in = str;
    char *out = buf;
    int i = 0;
    int found = 0;

    *out = '\0';
    while (*in != '\0')
      {
	  if (*in == ',')
	    {
		if (index == i)
		  {
		      *out = '\0';
		      found = 1;
		      break;
		  }
		i++;
		in++;
		continue;
	    }
	  if (index == i)
	      *out++ = *in;
	  in++;
      }
    if (!found)
	return 0;
    in = buf;
    for (i = 0; i < (int) strlen (buf); i++)
      {
	  if (buf[i] == ':')
	    {
		buf[i] = '\0';
		*iColumn = atoi (buf);
		*op = atoi (buf + i + 1);
		return 1;
	    }
	  in++;
      }
    return 0;
}

static int
vtxt_eval_constraints (VirtualTextCursorPtr cursor)
{
/* evaluating Filter constraints */
    int nCol;
    int i;
    char buf[4096];
    int type;
    const char *value;
    sqlite3_int64 int_value;
    double dbl_value;
    const char *txt_value;
    int is_int;
    int is_dbl;
    int is_txt;
    gaiaTextReaderPtr text = cursor->pVtab->reader;
    VirtualTextConstraintPtr pC;
    if (text->current_line_ready == 0)
	return SQLITE_ERROR;
    pC = cursor->firstConstraint;
    while (pC)
      {
	  int ok = 0;
	  if (pC->iColumn == 0)
	    {
		/* the ROWNO column */
		int_value = cursor->current_row;
		goto eval;
	    }
	  nCol = 1;
	  for (i = 0; i < text->max_fields; i++)
	    {
		is_int = 0;
		is_dbl = 0;
		is_txt = 0;
		if (nCol == pC->iColumn)
		  {
		      if (!gaiaTextReaderFetchField (text, i, &type, &value))
			  ;
		      else
			{
			    if (type == VRTTXT_INTEGER)
			      {
				  strcpy (buf, value);
				  text_clean_integer (buf);
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT - M$ runtime has non-standard functions for 64 bits */
				  int_value = _atoi64 (buf);
#else
				  int_value = atoll (buf);
#endif
				  is_int = 1;
			      }
			    else if (type == VRTTXT_DOUBLE)
			      {
				  strcpy (buf, value);
				  text_clean_double (buf);
				  dbl_value = atof (buf);
				  is_dbl = 1;
			      }
			    else if (type == VRTTXT_TEXT)
			      {
				  txt_value = value;
				  is_txt = 1;
			      }
			}
		      goto eval;
		  }
		nCol++;
	    }
	  return 0;
	eval:
	  ok = 0;
	  if (pC->valueType == 'I')
	    {
		if (is_int)
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (int_value == pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (int_value > pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (int_value <= pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (int_value < pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (int_value >= pC->intValue)
				ok = 1;
			    break;
			};
		  }
		if (is_dbl)
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (dbl_value == pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (dbl_value > pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (dbl_value <= pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (dbl_value < pC->intValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (dbl_value >= pC->intValue)
				ok = 1;
			    break;
			};
		  }
	    }
	  if (pC->valueType == 'D')
	    {
		if (is_int)
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (int_value == pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (int_value > pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (int_value <= pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (int_value < pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (int_value >= pC->dblValue)
				ok = 1;
			    break;
			};
		  }
		if (is_dbl)
		  {
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (dbl_value == pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (dbl_value > pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (dbl_value <= pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (dbl_value < pC->dblValue)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (dbl_value >= pC->dblValue)
				ok = 1;
			    break;
			};
		  }
	    }
	  if (pC->valueType == 'T')
	    {
		if (is_txt)
		  {
		      int ret = strcmp (txt_value, pC->txtValue);
		      switch (pC->op)
			{
			case SQLITE_INDEX_CONSTRAINT_EQ:
			    if (ret == 0)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GT:
			    if (ret > 0)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LE:
			    if (ret <= 0)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_LT:
			    if (ret < 0)
				ok = 1;
			    break;
			case SQLITE_INDEX_CONSTRAINT_GE:
			    if (ret >= 0)
				ok = 1;
			    break;
			};
		  }
	    }
	  if (!ok)
	      return 0;
	  pC = pC->next;
      }
    return 1;
}

static int
vtxt_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
	     int argc, sqlite3_value ** argv)
{
/* setting up a cursor filter */
    int i;
    int iColumn;
    int op;
    int len;
    VirtualTextConstraintPtr pC;
    VirtualTextCursorPtr cursor = (VirtualTextCursorPtr) pCursor;
    gaiaTextReaderPtr text = cursor->pVtab->reader;
    if (idxNum)
	idxNum = idxNum;	/* unused arg warning suppression */

/* resetting any previously set filter constraint */
    vtxt_free_constraints (cursor);

    for (i = 0; i < argc; i++)
      {
	  if (!vtxt_parse_constraint (idxStr, i, &iColumn, &op))
	      continue;
	  pC = sqlite3_malloc (sizeof (VirtualTextConstraint));
	  if (!pC)
	      continue;
	  pC->iColumn = iColumn;
	  pC->op = op;
	  pC->valueType = '\0';
	  pC->txtValue = NULL;
	  pC->next = NULL;

	  if (sqlite3_value_type (argv[i]) == SQLITE_INTEGER)
	    {
		pC->valueType = 'I';
		pC->intValue = sqlite3_value_int64 (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_FLOAT)
	    {
		pC->valueType = 'D';
		pC->dblValue = sqlite3_value_double (argv[i]);
	    }
	  if (sqlite3_value_type (argv[i]) == SQLITE_TEXT)
	    {
		pC->valueType = 'T';
		len = sqlite3_value_bytes (argv[i]) + 1;
		pC->txtValue = (char *) sqlite3_malloc (len);
		if (pC->txtValue)
		    strcpy (pC->txtValue,
			    (char *) sqlite3_value_text (argv[i]));
	    }
	  if (cursor->firstConstraint == NULL)
	      cursor->firstConstraint = pC;
	  if (cursor->lastConstraint != NULL)
	      cursor->lastConstraint->next = pC;
	  cursor->lastConstraint = pC;
      }

    cursor->current_row = 0;
    cursor->eof = 0;
    while (1)
      {
	  if (!gaiaTextReaderGetRow (text, cursor->current_row))
	    {
		cursor->eof = 1;
		break;
	    }
	  if (vtxt_eval_constraints (cursor))
	      break;
	  cursor->current_row++;
      }
    return SQLITE_OK;
}

static int
vtxt_next (sqlite3_vtab_cursor * pCursor)
{
/* fetching next row from cursor */
    VirtualTextCursorPtr cursor = (VirtualTextCursorPtr) pCursor;
    gaiaTextReaderPtr text = cursor->pVtab->reader;
    if (!text)
	cursor->eof = 1;
    else
      {
	  while (1)
	    {
		cursor->current_row++;
		if (!gaiaTextReaderGetRow (text, cursor->current_row))
		  {
		      cursor->eof = 1;
		      break;
		  }
		if (vtxt_eval_constraints (cursor))
		    break;
	    }
      }
    return SQLITE_OK;
}

static int
vtxt_eof (sqlite3_vtab_cursor * pCursor)
{
/* cursor EOF */
    VirtualTextCursorPtr cursor = (VirtualTextCursorPtr) pCursor;
    return cursor->eof;
}

static int
vtxt_column (sqlite3_vtab_cursor * pCursor, sqlite3_context * pContext,
	     int column)
{
/* fetching value for the Nth column */
    int nCol = 1;
    int i;
    char buf[4096];
    int type;
    const char *value;
    VirtualTextCursorPtr cursor = (VirtualTextCursorPtr) pCursor;
    gaiaTextReaderPtr text = cursor->pVtab->reader;
    if (column == 0)
      {
	  /* the ROWNO column */
	  sqlite3_result_int (pContext, cursor->current_row);
	  return SQLITE_OK;
      }
    if (text->current_line_ready == 0)
	return SQLITE_ERROR;
    for (i = 0; i < text->max_fields; i++)
      {
	  if (nCol == column)
	    {
		if (!gaiaTextReaderFetchField (text, i, &type, &value))
		    sqlite3_result_null (pContext);
		else
		  {
		      if (type == VRTTXT_INTEGER)
			{
			    strcpy (buf, value);
			    text_clean_integer (buf);
#if defined(_WIN32) || defined(__MINGW32__)
/* CAVEAT - M$ runtime has non-standard functions for 64 bits */
			    sqlite3_result_int64 (pContext, _atoi64 (buf));
#else
			    sqlite3_result_int64 (pContext, atoll (buf));
#endif
			}
		      else if (type == VRTTXT_DOUBLE)
			{
			    strcpy (buf, value);
			    text_clean_double (buf);
			    sqlite3_result_double (pContext, atof (buf));
			}
		      else if (type == VRTTXT_TEXT)
			  sqlite3_result_text (pContext, value, strlen (value),
					       free);
		      else
			  sqlite3_result_null (pContext);
		  }
	    }
	  nCol++;
      }
    return SQLITE_OK;
}

static int
vtxt_rowid (sqlite3_vtab_cursor * pCursor, sqlite_int64 * pRowid)
{
/* fetching the ROWID */
    VirtualTextCursorPtr cursor = (VirtualTextCursorPtr) pCursor;
    *pRowid = cursor->current_row;
    return SQLITE_OK;
}

static int
vtxt_update (sqlite3_vtab * pVTab, int argc, sqlite3_value ** argv,
	     sqlite_int64 * pRowid)
{
/* generic update [INSERT / UPDATE / DELETE */
    if (pVTab || argc || argv || pRowid)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_READONLY;
}

static int
vtxt_begin (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vtxt_sync (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vtxt_commit (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

static int
vtxt_rollback (sqlite3_vtab * pVTab)
{
/* BEGIN TRANSACTION */
    if (pVTab)
	pVTab = pVTab;		/* unused arg warning suppression */
    return SQLITE_OK;
}

int
sqlite3VirtualTextInit (sqlite3 * db)
{
    int rc = SQLITE_OK;
    virtualtext_module.iVersion = 1;
    virtualtext_module.xCreate = &vtxt_create;
    virtualtext_module.xConnect = &vtxt_connect;
    virtualtext_module.xBestIndex = &vtxt_best_index;
    virtualtext_module.xDisconnect = &vtxt_disconnect;
    virtualtext_module.xDestroy = &vtxt_destroy;
    virtualtext_module.xOpen = &vtxt_open;
    virtualtext_module.xClose = &vtxt_close;
    virtualtext_module.xFilter = &vtxt_filter;
    virtualtext_module.xNext = &vtxt_next;
    virtualtext_module.xEof = &vtxt_eof;
    virtualtext_module.xColumn = &vtxt_column;
    virtualtext_module.xRowid = &vtxt_rowid;
    virtualtext_module.xUpdate = &vtxt_update;
    virtualtext_module.xBegin = &vtxt_begin;
    virtualtext_module.xSync = &vtxt_sync;
    virtualtext_module.xCommit = &vtxt_commit;
    virtualtext_module.xRollback = &vtxt_rollback;
    virtualtext_module.xFindFunction = NULL;
    sqlite3_create_module_v2 (db, "VirtualText", &virtualtext_module, NULL, 0);
    return rc;
}

int
virtualtext_extension_init (sqlite3 * db)
{
    return sqlite3VirtualTextInit (db);
}

/*
**
** TextReader implementation
**
*/

static struct vrttxt_row_block *
vrttxt_block_alloc ()
{
/* allocating a rows Block */
    struct vrttxt_row_block *p = malloc (sizeof (struct vrttxt_row_block));
    if (!p)
	return NULL;
    p->num_rows = 0;
    p->min_line_no = -1;
    p->max_line_no = -1;
    p->next = NULL;
    return p;
}

static void
vrttxt_block_destroy (struct vrttxt_row_block *p)
{
/* destroying a rows Block */
    if (p)
	free (p);
}

GAIAGEO_DECLARE void
gaiaTextReaderDestroy (gaiaTextReaderPtr reader)
{
/* destroying the main TXT-Reader */
    int col;
    struct vrttxt_row_block *blk;
    struct vrttxt_row_block *blkN;
    if (reader)
      {
	  blk = reader->first;
	  while (blk)
	    {
		/* destroying the row offset Blocks */
		blkN = blk->next;
		vrttxt_block_destroy (blk);
		blk = blkN;
	    }
	  /* freeing the input buffers */
	  if (reader->line_buffer)
	      free (reader->line_buffer);
	  if (reader->field_buffer)
	      free (reader->field_buffer);
	  /* freeing the row offsets array */
	  if (reader->rows)
	      free (reader->rows);
	  /* closing the input file */
	  fclose (reader->text_file);
	  for (col = 0; col < VRTTXT_FIELDS_MAX; col++)
	    {
		/* destroying column headers */
		if (reader->columns[col].name != NULL)
		    free (reader->columns[col].name);
	    }
	  gaiaFreeUTF8Converter (reader->toUtf8);
	  free (reader);
      }
}

GAIAGEO_DECLARE gaiaTextReaderPtr
gaiaTextReaderAlloc (const char *path, char field_separator,
		     char text_separator, char decimal_separator,
		     int first_line_titles, const char *encoding)
{
/* allocating the main TXT-Reader */
    int col;
    gaiaTextReaderPtr reader;
    FILE *in = fopen (path, "rb");	/* opening the input file */
    if (in == NULL)
	return NULL;

/* allocating and initializing the struct */
    reader = malloc (sizeof (gaiaTextReader));
    if (!reader)
      {
	  fclose (in);
	  return NULL;
      }
    reader->text_file = in;
    reader->field_separator = field_separator;
    reader->text_separator = text_separator;
    reader->decimal_separator = decimal_separator;
    reader->first_line_titles = first_line_titles;
    reader->toUtf8 = gaiaCreateUTF8Converter (encoding);
    if (reader->toUtf8 == (void *) 0)
      {
	  fclose (in);
	  return NULL;
      }
    reader->error = 0;
    reader->first = NULL;
    reader->last = NULL;
    reader->rows = NULL;
    reader->num_rows = 0;
    reader->line_no = 0;
    reader->max_fields = 0;
    reader->max_current_field = 0;
    reader->current_line_ready = 0;
    reader->current_buf_sz = 1024;
    reader->line_buffer = malloc (1024);
    reader->field_buffer = malloc (1024);
    if (reader->line_buffer == NULL || reader->field_buffer == NULL)
      {
	  /* insufficient memory: no input buffers */
	  gaiaTextReaderDestroy (reader);
	  return NULL;
      }
    for (col = 0; col < VRTTXT_FIELDS_MAX; col++)
      {
	  /* initializing column headers */
	  reader->columns[col].name = NULL;
	  reader->columns[col].type = VRTTXT_NULL;
      }
    return reader;
}

static void
vrttxt_line_init (struct vrttxt_line *line, off_t offset)
{
/* initializing a LINE struct */
    line->offset = offset;
    line->len = 0;
    line->num_fields = 0;
    line->error = 0;
}

static void
vrttxt_line_end (struct vrttxt_line *line, off_t offset)
{
/* completing a Line struct (EndOfLine encountered) */
    line->len = offset - line->offset;
}

static void
vrttxt_add_field (struct vrttxt_line *line, off_t offset)
{
/* adding a Field offset to the current Line */
    if (line->num_fields >= VRTTXT_FIELDS_MAX)
      {
	  line->error = 1;
	  return;
      }
    line->field_offsets[line->num_fields] = offset - line->offset;
    line->num_fields++;
}

static int
vrttxt_is_integer (const char *value)
{
/* checking if this value can be an INTEGER */
    int invalids = 0;
    int digits = 0;
    int signs = 0;
    char last = '\0';
    const char *p = value;
    while (*p != '\0')
      {
	  last = *p;
	  if (*p >= '0' && *p <= '9')
	      digits++;
	  else if (*p == '+' || *p == '-')
	      signs++;
	  else
	      invalids++;
	  p++;
      }
    if (invalids)
	return 0;		/* invalid chars where found */
    if (signs > 1)
	return 0;		/* more than a single sign */
    if (signs)
      {
	  if (*value == '+' || *value == '-' || last == '+' || last == '-')
	      ;
	  else
	      return 0;		/* sign is not the first/last string char */
      }
    return 1;			/* ok, can be a valid INTEGER value */
}

static int
vrttxt_is_double (const char *value, char decimal_separator)
{
/* checking if this value can be a DOUBLE */
    int invalids = 0;
    int digits = 0;
    int signs = 0;
    int points = 0;
    char last = '\0';
    const char *p = value;
    while (*p != '\0')
      {
	  last = *p;
	  if (*p >= '0' && *p <= '9')
	      digits++;
	  else if (*p == '+' || *p == '-')
	      signs++;
	  else
	    {
		if (decimal_separator == ',')
		  {
		      if (*p == ',')
			  points++;
		      else
			  invalids++;
		  }
		else
		  {
		      if (*p == '.')
			  points++;
		      else
			  invalids++;
		  }
	    }
	  p++;
      }
    if (invalids)
	return 0;		/* invalid chars where found */
    if (points > 1)
	return 0;		/* more than a single decimal separator */
    if (signs > 1)
	return 0;		/* more than a single sign */
    if (signs)
      {
	  if (*value == '+' || *value == '-' || last == '+' || last == '-')
	      ;
	  else
	      return 0;		/* sign is not the first/last string char */
      }
    return 1;			/* ok, can be a valid DOUBLE value */
}

static int
vrttxt_check_type (const char *value, char decimal_separator)
{
/* checking the Field type */
    if (*value == '\0')
	return VRTTXT_NULL;
    if (vrttxt_is_integer (value))
	return VRTTXT_INTEGER;
    if (vrttxt_is_double (value, decimal_separator))
	return VRTTXT_DOUBLE;
    return VRTTXT_TEXT;
}

static int
vrttxt_set_column_title (gaiaTextReaderPtr txt, int col_no, const char *name)
{
/* setting a Column header name */
    int err;
    int ind;
    char *utf8text;
    char *str = (char *) name;
    int len = strlen (str);
    if (str[0] == txt->text_separator && str[len - 1] == txt->text_separator)
      {
	  /* cleaning the enclosing quotes */
	  str[len - 1] = '\0';
	  str = (char *) (name + 1);
	  len -= 2;
	  if (len <= 0)
	      return 0;
      }
    utf8text = gaiaConvertToUTF8 (txt->toUtf8, str, len, &err);
    if (err)
      {
	  if (utf8text)
	      free (utf8text);
	  return 0;
      }
    else
	str = utf8text;
    len = strlen (str);
    for (ind = 0; ind < len; ind++)
      {
	  /* masking spaces and so on within the column name */
	  switch (str[ind])
	    {
	    case ' ':
	    case '\t':
	    case '-':
	    case '+':
	    case '*':
	    case '/':
	    case '(':
	    case ')':
	    case '[':
	    case ']':
	    case '{':
	    case '}':
		str[ind] = '_';
		break;
	    }
      }
    if (txt->columns[col_no].name)
	free (txt->columns[col_no].name);
    txt->columns[col_no].name = malloc (len + 1);
    if (txt->columns[col_no].name == NULL)
	return 0;
    strcpy (txt->columns[col_no].name, utf8text);
    free (utf8text);
    return 1;
}

static void
vrttxt_add_line (gaiaTextReaderPtr txt, struct vrttxt_line *line)
{
/* appending a Line offset to the main TXT-Reader */
    struct vrttxt_row_block *p_block;
    struct vrttxt_row *p_row;
    int ind;
    int off;
    int len;
    int value_type;
    int column_type;
    int first_line = 0;
    if (txt->line_no == 0)
	first_line = 1;
    if (line->error)
      {
	  txt->error = 1;
	  txt->line_no++;
	  return;
      }
    if (line->num_fields == 0)
      {
	  txt->line_no++;
	  return;
      }
    p_block = txt->last;
    if (p_block == NULL)
      {
	  /* the offset Blocks list is empty: allocating the first Block */
	  p_block = vrttxt_block_alloc ();
	  if (!p_block)
	    {
		txt->error = 1;
		txt->line_no++;
		return;
	    }
	  if (txt->first == NULL)
	      txt->first = p_block;
	  if (txt->last != NULL)
	      txt->last->next = p_block;
	  txt->last = p_block;
      }
    else if (p_block->num_rows >= VRTTXT_BLOCK_MAX)
      {
	  /* the current offset Block is full: expanding the list */
	  p_block = vrttxt_block_alloc ();
	  if (!p_block)
	    {
		txt->error = 1;
		txt->line_no++;
		return;
	    }
	  if (txt->first == NULL)
	      txt->first = p_block;
	  if (txt->last != NULL)
	      txt->last->next = p_block;
	  txt->last = p_block;
      }
/* inserting the Row offset into the offset Block */
    p_row = p_block->rows + p_block->num_rows;
    p_block->num_rows++;
    p_row->line_no = txt->line_no;
    if (p_block->min_line_no < 0)
	p_block->min_line_no = p_row->line_no;
    if (p_block->max_line_no < p_row->line_no)
	p_block->max_line_no = p_row->line_no;
    txt->line_no++;
    p_row->offset = line->offset;
    p_row->len = line->len;
    p_row->num_fields = line->num_fields;
    if (line->num_fields > txt->max_fields)
	txt->max_fields = line->num_fields;
    off = 0;
    for (ind = 0; ind < p_row->num_fields; ind++)
      {
	  /* setting the corresponding Column (aka Field) header */
	  len = line->field_offsets[ind] - off;
	  if (len == 0)
	      *(txt->field_buffer) = '\0';
	  else
	    {
		/* retrieving the current Field Value */
		memcpy (txt->field_buffer, txt->line_buffer + off, len);
		*(txt->field_buffer + len) = '\0';
	    }
	  if (txt->first_line_titles && first_line)
	    {
		/* first line: the current value is the Column Name */
		if (!vrttxt_set_column_title (txt, ind, txt->field_buffer))
		    txt->error = 1;
	    }
	  else
	    {
		/* plain Field Value */
		value_type =
		    vrttxt_check_type (txt->field_buffer,
				       txt->decimal_separator);
		column_type = txt->columns[ind].type;
		switch (value_type)
		  {
		      /* checking the Column type */
		  case VRTTXT_INTEGER:
		      if (column_type == VRTTXT_NULL)
			  txt->columns[ind].type = VRTTXT_INTEGER;
		      break;
		  case VRTTXT_DOUBLE:
		      if (column_type == VRTTXT_NULL
			  || column_type == VRTTXT_INTEGER)
			  txt->columns[ind].type = VRTTXT_DOUBLE;
		      break;
		  case VRTTXT_TEXT:
		      txt->columns[ind].type = VRTTXT_TEXT;
		      break;
		  default:
		      break;
		  };
	    }
	  off = line->field_offsets[ind] + 1;
      }
}

static void
vrttxt_line_push (gaiaTextReaderPtr txt, char c)
{
/* inserting a single char into the dynamically growing buffer */
    if (txt->error)
	return;
    if ((txt->current_buf_off + 1) >= txt->current_buf_sz)
      {
	  /* expanding the input buffer */
	  int new_sz;
	  char *new_buf;
	  /*
	     / allocation strategy:
	     / - the input buffer has an initial size of 1024 bytes
	     /   (good for short lines)
	     / - the second step allocates 4196 bytes
	     / - the third step allocates 65536 bytes
	     /   (good for medium sized lines)
	     / - after this the buffer allocation will be increased
	     /   be 1MB at each step (good for huge sized lines)
	   */
	  if (txt->current_buf_sz < 4196)
	      new_sz = 4196;
	  else if (txt->current_buf_sz < 65536)
	      new_sz = 65536;
	  else
	      new_sz = txt->current_buf_sz + (1024 * 1024);
	  new_buf = malloc (new_sz);
	  if (!new_buf)
	    {
		txt->error = 1;
		return;
	    }
	  txt->current_buf_sz = new_sz;
	  memcpy (new_buf, txt->line_buffer, txt->current_buf_off);
	  free (txt->line_buffer);
	  txt->line_buffer = new_buf;
	  free (txt->field_buffer);
	  txt->field_buffer = malloc (new_sz);
	  if (txt->field_buffer == NULL)
	    {
		txt->error = 1;
		return;
	    }
      }
    *(txt->line_buffer + txt->current_buf_off) = c;
    txt->current_buf_off++;
/* ensuring that input buffer will be null terminated anyway */
    *(txt->line_buffer + txt->current_buf_off) = '\0';
}

static void
vrttxt_build_line_array (gaiaTextReaderPtr txt)
{
/* creating the final Line offsets array */
    struct vrttxt_row_block *p_block;
    int i;
    int cnt = 0;
    int first_line = 1;
    if (txt->rows)
	free (txt->rows);
    txt->rows = NULL;
    txt->num_rows = 0;
    p_block = txt->first;
    while (p_block)
      {
	  /* counting how many lines are there */
	  if (p_block == txt->first && txt->first_line_titles)
	      txt->num_rows += p_block->num_rows - 1;
	  else
	      txt->num_rows += p_block->num_rows;
	  p_block = p_block->next;
      }
    txt->rows = malloc (sizeof (struct vrttxt_row *) * txt->num_rows);
    if (txt->rows == NULL)
      {
	  /* insufficient memory */
	  txt->error = 1;
	  return;
      }
    p_block = txt->first;
    while (p_block)
      {
	  for (i = 0; i < p_block->num_rows; i++)
	    {
		/* setting Line references into the array */
		if (first_line && txt->first_line_titles)
		  {
		      first_line = 0;
		      continue;	/* skipping the first line (column names) */
		  }
		*(txt->rows + cnt++) = p_block->rows + i;
	    }
	  p_block = p_block->next;
      }
}

GAIAGEO_DECLARE int
gaiaTextReaderParse (gaiaTextReaderPtr txt)
{
/* 
/ preliminary parsing
/ - reading the input file until EOF
/ - then feeding the Row offsets structs
/   to be used for any subsequent access
*/
    char name[64];
    int ind;
    int i2;
    int c;
    int masked = 0;
    int token_start = 1;
    int row_offset = 0;
    off_t offset = 0;
    struct vrttxt_line line;
    vrttxt_line_init (&line, 0);
    txt->current_buf_off = 0;

    while ((c = getc (txt->text_file)) != EOF)
      {
	  if (c == txt->text_separator)
	    {
		if (masked)
		    masked = 0;
		else
		  {
		      if (token_start)
			  masked = 1;
		  }
		vrttxt_line_push (txt, c);
		if (txt->error)
		    return 0;
		row_offset++;
		offset++;
		continue;
	    }
	  token_start = 0;
	  if (c == '\r')
	    {
		if (masked)
		  {
		      vrttxt_line_push (txt, c);
		      if (txt->error)
			  return 0;
		      row_offset++;
		  }
		offset++;
		continue;
	    }
	  if (c == '\n')
	    {
		if (masked)
		  {
		      vrttxt_line_push (txt, c);
		      if (txt->error)
			  return 0;
		      row_offset++;
		      offset++;
		      continue;
		  }
		vrttxt_add_field (&line, offset);
		vrttxt_line_end (&line, offset);
		vrttxt_add_line (txt, &line);
		if (txt->error)
		    return 0;
		vrttxt_line_init (&line, offset + 1);
		txt->current_buf_off = 0;
		token_start = 1;
		row_offset = 0;
		offset++;
		continue;
	    }
	  if (c == txt->field_separator)
	    {
		if (masked)
		  {
		      vrttxt_line_push (txt, c);
		      if (txt->error)
			  return 0;
		      row_offset++;
		      offset++;
		      continue;
		  }
		vrttxt_line_push (txt, c);
		if (txt->error)
		    return 0;
		row_offset++;
		vrttxt_add_field (&line, offset);
		token_start = 1;
		offset++;
		continue;
	    }
	  vrttxt_line_push (txt, c);
	  if (txt->error)
	      return 0;
	  row_offset++;
	  offset++;
      }
    if (txt->error)
	return 0;
    if (txt->first_line_titles)
      {
	  /* checking for duplicate column names */
	  for (ind = 0; ind < txt->max_fields; ind++)
	    {
		for (i2 = 0; i2 < ind; i2++)
		  {
		      if (strcasecmp
			  (txt->columns[i2].name, txt->columns[ind].name) == 0)
			{
			    sprintf (name, "COL%03d", ind + 1);
			    if (!vrttxt_set_column_title (txt, ind, name))
			      {
				  txt->error = 1;
				  return 0;
			      }
			}
		  }
	    }
      }
    else
      {
	  /* setting convenience column names */
	  for (ind = 0; ind < txt->max_fields; ind++)
	    {
		sprintf (name, "COL%03d", ind + 1);
		if (!vrttxt_set_column_title (txt, ind, name))
		  {
		      txt->error = 1;
		      return 0;
		  }
	    }
      }
    if (txt->error)
	return 0;
    vrttxt_build_line_array (txt);
    if (txt->error)
	return 0;
    return 1;
}

GAIAGEO_DECLARE int
gaiaTextReaderGetRow (gaiaTextReaderPtr txt, int line_no)
{
/* reading a Line (identified by relative number */
    int i;
    char c;
    int masked = 0;
    int token_start = 1;
    int fld = 0;
    int offset = 0;
    struct vrttxt_row *p_row;
    txt->current_line_ready = 0;
    txt->max_current_field = 0;
    if (line_no < 0 || line_no >= txt->num_rows || txt->rows == NULL)
	return 0;
    p_row = *(txt->rows + line_no);
    if (fseek (txt->text_file, p_row->offset, SEEK_SET) != 0)
	return 0;
    if (fread (txt->line_buffer, 1, p_row->len, txt->text_file) !=
	(unsigned int) (p_row->len))
	return 0;
    txt->field_offsets[0] = 0;
    for (i = 0; i < p_row->len; i++)
      {
	  /* parsing Fields */
	  c = *(txt->line_buffer + i);
	  if (c == txt->text_separator)
	    {
		if (masked)
		    masked = 0;
		else
		  {
		      if (token_start)
			  masked = 1;
		  }
		offset++;
		continue;
	    }
	  token_start = 0;
	  if (c == '\r')
	    {
		offset++;
		continue;
	    }
	  if (c == txt->field_separator)
	    {
		if (masked)
		  {
		      offset++;
		      continue;
		  }
		txt->field_offsets[fld + 1] = offset + 1;
		txt->field_lens[fld] = -1;
		txt->field_lens[fld] = offset - txt->field_offsets[fld];
		fld++;
		txt->max_current_field = fld;
		token_start = 1;
		offset++;
		continue;
	    }
	  offset++;
      }
    if (offset > 0)
      {
	  txt->field_lens[fld] = offset - txt->field_offsets[fld];
	  fld++;
	  txt->max_current_field = fld;
      }
    txt->current_line_ready = 1;
    return 1;
}

GAIAGEO_DECLARE int
gaiaTextReaderFetchField (gaiaTextReaderPtr txt, int field_idx, int *type,
			  const char **value)
{
/* fetching a field value */
    char *utf8text = NULL;
    int err;
    int len;
    char *str;
    if (txt->current_line_ready == 0)
      {
	  *type = VRTTXT_NULL;
	  *value = NULL;
	  return 0;
      }
    if (field_idx < 0 || field_idx >= txt->max_fields)
      {
	  *type = VRTTXT_NULL;
	  *value = NULL;
	  return 0;
      }
    if (field_idx < 0 || field_idx >= txt->max_current_field)
      {
	  *type = VRTTXT_NULL;
	  *value = NULL;
	  return 0;
      }
    *type = txt->columns[field_idx].type;
    if (txt->field_lens[field_idx] == 0)
	*(txt->field_buffer) = '\0';
    memcpy (txt->field_buffer, txt->line_buffer + txt->field_offsets[field_idx],
	    txt->field_lens[field_idx]);
    *(txt->field_buffer + txt->field_lens[field_idx]) = '\0';
    *value = txt->field_buffer;
    if (*value == '\0')
	*type = VRTTXT_NULL;
    else if (*type == VRTTXT_TEXT)
      {
	  /* converting to UTF-8 */
	  str = (char *) *value;
	  len = strlen (str);
	  if (str[len - 1] == '\r')
	    {
		/* skipping trailing CR, if any */
		str[len - 1] = '\0';
		len--;
	    }
	  if (str[0] == txt->text_separator
	      && str[len - 1] == txt->text_separator)
	    {
		/* cleaning the enclosing quotes */
		str[len - 1] = '\0';
		str = (char *) (*value + 1);
		len -= 2;
		if (len <= 0)
		  {
		      *type = VRTTXT_NULL;
		      *value = NULL;
		      return 1;
		  }
	    }
	  utf8text = gaiaConvertToUTF8 (txt->toUtf8, str, len, &err);
	  if (err)
	    {
		/* memory cleanup: Kashif Rasul 14 Jan 2010 */
		if (utf8text)
		    free (utf8text);
		*type = VRTTXT_NULL;
		*value = NULL;
		return 0;
	    }
	  *value = utf8text;
      }
    return 1;
}

#endif /* ICONV enabled/disabled */
/**************** End file: virtualtext.c **********/


/**************** Begin file: version.c **********/

#ifdef SPL_AMALGAMATION	/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite.h> */

const char spatialiteversion[] = VERSION;

SPATIALITE_DECLARE const char *
spatialite_version (void)
{
    return spatialiteversion;
}
/**************** End file: version.c **********/


/**************** Begin file: gg_wkt.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

static void
gaiaOutClean (char *buffer)
{
/* cleans unneeded trailing zeros */
    int i;
    for (i = strlen (buffer) - 1; i > 0; i--)
      {
	  if (buffer[i] == '0')
	      buffer[i] = '\0';
	  else
	      break;
      }
    if (buffer[i] == '.')
	buffer[i] = '\0';
}

GAIAGEO_DECLARE void
gaiaOutBufferInitialize (gaiaOutBufferPtr buf)
{
/* initializing a dynamically growing output buffer */
    buf->Buffer = NULL;
    buf->WriteOffset = 0;
    buf->BufferSize = 0;
    buf->Error = 0;
}

GAIAGEO_DECLARE void
gaiaOutBufferReset (gaiaOutBufferPtr buf)
{
/* cleaning a dynamically growing output buffer */
    if (buf->Buffer)
	free (buf->Buffer);
    buf->Buffer = NULL;
    buf->WriteOffset = 0;
    buf->BufferSize = 0;
    buf->Error = 0;
}

GAIAGEO_DECLARE void
gaiaAppendToOutBuffer (gaiaOutBufferPtr buf, const char *text)
{
/* appending a text string */
    int len = strlen (text);
    int free_size = buf->BufferSize - buf->WriteOffset;
    if ((len + 1) > free_size)
      {
	  /* we must allocate a bigger buffer */
	  int new_size;
	  char *new_buf;
	  if (buf->BufferSize == 0)
	      new_size = 1024;
	  else if (buf->BufferSize <= 4196)
	      new_size = buf->BufferSize + (len + 1) + 4196;
	  else if (buf->BufferSize <= 65536)
	      new_size = buf->BufferSize + (len + 1) + 65536;
	  else
	      new_size = buf->BufferSize + (len + 1) + (1024 * 1024);
	  new_buf = malloc (new_size);
	  if (!new_buf)
	    {
		buf->Error = 1;
		return;
	    }
	  memcpy (new_buf, buf->Buffer, buf->WriteOffset);
	  if (buf->Buffer)
	      free (buf->Buffer);
	  buf->Buffer = new_buf;
	  buf->BufferSize = new_size;
      }
    strcpy (buf->Buffer + buf->WriteOffset, text);
    buf->WriteOffset += len;
}

static void
gaiaOutPointStrict (gaiaOutBufferPtr out_buf, gaiaPointPtr point, int precision)
{
/* formats a WKT POINT [Strict 2D] */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    sprintf (buf_x, "%.*f", precision, point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%.*f", precision, point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf, "%s %s", buf_x, buf_y);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
gaiaOutPoint (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats a WKT POINT */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    sprintf (buf_x, "%1.6f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.6f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf, "%s %s", buf_x, buf_y);
    gaiaAppendToOutBuffer (out_buf, buf);
}

GAIAGEO_DECLARE void
gaiaOutPointZ (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats a WKT POINTZ */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    sprintf (buf_x, "%1.6f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.6f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf_z, "%1.6f", point->Z);
    gaiaOutClean (buf_z);
    sprintf (buf, "%s %s %s", buf_x, buf_y, buf_z);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
gaiaOutPointM (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats a WKT POINTM */
    char buf_x[128];
    char buf_y[128];
    char buf_m[128];
    char buf[512];
    sprintf (buf_x, "%1.6f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.6f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf_m, "%1.6f", point->M);
    gaiaOutClean (buf_m);
    sprintf (buf, "%s %s %s", buf_x, buf_y, buf_m);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
gaiaOutPointZM (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats a WKT POINTZM */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf_m[128];
    char buf[1024];
    sprintf (buf_x, "%1.6f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.6f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf_z, "%1.6f", point->Z);
    gaiaOutClean (buf_z);
    sprintf (buf_m, "%1.6f", point->M);
    gaiaOutClean (buf_m);
    sprintf (buf, "%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
gaiaOutEwktPoint (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats an EWKT POINT */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    sprintf (buf_x, "%1.15f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.15f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf, "%s %s", buf_x, buf_y);
    gaiaAppendToOutBuffer (out_buf, buf);
}

GAIAGEO_DECLARE void
gaiaOutEwktPointZ (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats an EWKT POINTZ */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    sprintf (buf_x, "%1.15f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.15f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf_z, "%1.15f", point->Z);
    gaiaOutClean (buf_z);
    sprintf (buf, "%s %s %s", buf_x, buf_y, buf_z);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
gaiaOutEwktPointM (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats an EWKT POINTM */
    char buf_x[128];
    char buf_y[128];
    char buf_m[128];
    char buf[512];
    sprintf (buf_x, "%1.15f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.15f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf_m, "%1.15f", point->M);
    gaiaOutClean (buf_m);
    sprintf (buf, "%s %s %s", buf_x, buf_y, buf_m);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
gaiaOutEwktPointZM (gaiaOutBufferPtr out_buf, gaiaPointPtr point)
{
/* formats an EWKT POINTZM */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf_m[128];
    char buf[1024];
    sprintf (buf_x, "%1.15f", point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%1.15f", point->Y);
    gaiaOutClean (buf_y);
    sprintf (buf_z, "%1.15f", point->Z);
    gaiaOutClean (buf_z);
    sprintf (buf_m, "%1.15f", point->M);
    gaiaOutClean (buf_m);
    sprintf (buf, "%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
gaiaOutLinestringStrict (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line,
			 int precision)
{
/* formats a WKT LINESTRING [Strict 2D] */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    double x;
    double y;
    double z;
    double m;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  if (line->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	    }
	  else if (line->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	    }
	  else if (line->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (line->Coords, iv, &x, &y);
	    }
	  sprintf (buf_x, "%.*f", precision, x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, y);
	  gaiaOutClean (buf_y);
	  if (iv > 0)
	      sprintf (buf, ",%s %s", buf_x, buf_y);
	  else
	      sprintf (buf, "%s %s", buf_x, buf_y);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
gaiaOutLinestring (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats a WKT LINESTRING */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    double x;
    double y;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPoint (line->Coords, iv, &x, &y);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  if (iv > 0)
	      sprintf (buf, ", %s %s", buf_x, buf_y);
	  else
	      sprintf (buf, "%s %s", buf_x, buf_y);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

GAIAGEO_DECLARE void
gaiaOutLinestringZ (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats a WKT LINESTRINGZ */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    double x;
    double y;
    double z;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.6f", z);
	  gaiaOutClean (buf_z);
	  if (iv > 0)
	      sprintf (buf, ", %s %s %s", buf_x, buf_y, buf_z);
	  else
	      sprintf (buf, "%s %s %s", buf_x, buf_y, buf_z);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
gaiaOutLinestringM (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats a WKT LINESTRINGM */
    char buf_x[128];
    char buf_y[128];
    char buf_m[128];
    char buf[512];
    double x;
    double y;
    double m;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_m, "%1.6f", m);
	  gaiaOutClean (buf_m);
	  if (iv > 0)
	      sprintf (buf, ", %s %s %s", buf_x, buf_y, buf_m);
	  else
	      sprintf (buf, "%s %s %s", buf_x, buf_y, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
gaiaOutLinestringZM (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats a WKT LINESTRINGZM */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf_m[128];
    char buf[1024];
    double x;
    double y;
    double z;
    double m;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.6f", z);
	  gaiaOutClean (buf_z);
	  sprintf (buf_m, "%1.6f", m);
	  gaiaOutClean (buf_m);
	  if (iv > 0)
	      sprintf (buf, ", %s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  else
	      sprintf (buf, "%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
gaiaOutEwktLinestring (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats an EWKT LINESTRING */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    double x;
    double y;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPoint (line->Coords, iv, &x, &y);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  if (iv > 0)
	      sprintf (buf, ",%s %s", buf_x, buf_y);
	  else
	      sprintf (buf, "%s %s", buf_x, buf_y);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

GAIAGEO_DECLARE void
gaiaOutEwktLinestringZ (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats an EWKT LINESTRINGZ */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    double x;
    double y;
    double z;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.15f", z);
	  gaiaOutClean (buf_z);
	  if (iv > 0)
	      sprintf (buf, ",%s %s %s", buf_x, buf_y, buf_z);
	  else
	      sprintf (buf, "%s %s %s", buf_x, buf_y, buf_z);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
gaiaOutEwktLinestringM (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats an EWKT LINESTRINGM */
    char buf_x[128];
    char buf_y[128];
    char buf_m[128];
    char buf[512];
    double x;
    double y;
    double m;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_m, "%1.15f", m);
	  gaiaOutClean (buf_m);
	  if (iv > 0)
	      sprintf (buf, ",%s %s %s", buf_x, buf_y, buf_m);
	  else
	      sprintf (buf, "%s %s %s", buf_x, buf_y, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
gaiaOutEwktLinestringZM (gaiaOutBufferPtr out_buf, gaiaLinestringPtr line)
{
/* formats an EWKT LINESTRINGZM */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf_m[128];
    char buf[1024];
    double x;
    double y;
    double z;
    double m;
    int iv;
    for (iv = 0; iv < line->Points; iv++)
      {
	  gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.15f", z);
	  gaiaOutClean (buf_z);
	  sprintf (buf_m, "%1.15f", m);
	  gaiaOutClean (buf_m);
	  if (iv > 0)
	      sprintf (buf, ",%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  else
	      sprintf (buf, "%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
gaiaOutPolygonStrict (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg,
		      int precision)
{
/* formats a WKT POLYGON [Strict 2D] */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  if (ring->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
	    }
	  else if (ring->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
	    }
	  else if (ring->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (ring->Coords, iv, &x, &y);
	    }
	  sprintf (buf_x, "%.*f", precision, x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, y);
	  gaiaOutClean (buf_y);
	  if (iv == 0)
	      sprintf (buf, "(%s %s", buf_x, buf_y);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ",%s %s)", buf_x, buf_y);
	  else
	      sprintf (buf, ",%s %s", buf_x, buf_y);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		sprintf (buf_x, "%.*f", precision, x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%.*f", precision, y);
		gaiaOutClean (buf_y);
		if (iv == 0)
		    sprintf (buf, ",(%s %s", buf_x, buf_y);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ",%s %s)", buf_x, buf_y);
		else
		    sprintf (buf, ",%s %s", buf_x, buf_y);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

static void
gaiaOutPolygon (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats a WKT POLYGON */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    int ib;
    int iv;
    double x;
    double y;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPoint (ring->Coords, iv, &x, &y);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  if (iv == 0)
	      sprintf (buf, "(%s %s", buf_x, buf_y);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ", %s %s)", buf_x, buf_y);
	  else
	      sprintf (buf, ", %s %s", buf_x, buf_y);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPoint (ring->Coords, iv, &x, &y);
		sprintf (buf_x, "%1.6f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.6f", y);
		gaiaOutClean (buf_y);
		if (iv == 0)
		    sprintf (buf, ", (%s %s", buf_x, buf_y);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ", %s %s)", buf_x, buf_y);
		else
		    sprintf (buf, ", %s %s", buf_x, buf_y);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

GAIAGEO_DECLARE void
gaiaOutPolygonZ (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats a WKT POLYGONZ */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    int ib;
    int iv;
    double x;
    double y;
    double z;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.6f", z);
	  gaiaOutClean (buf_z);
	  if (iv == 0)
	      sprintf (buf, "(%s %s %s", buf_x, buf_y, buf_z);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ", %s %s %s)", buf_x, buf_y, buf_z);
	  else
	      sprintf (buf, ", %s %s %s", buf_x, buf_y, buf_z);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		sprintf (buf_x, "%1.6f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.6f", y);
		gaiaOutClean (buf_y);
		sprintf (buf_z, "%1.6f", z);
		gaiaOutClean (buf_z);
		if (iv == 0)
		    sprintf (buf, ", (%s %s %s", buf_x, buf_y, buf_z);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ", %s %s %s)", buf_x, buf_y, buf_z);
		else
		    sprintf (buf, ", %s %s %s", buf_x, buf_y, buf_z);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

static void
gaiaOutPolygonM (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats a WKT POLYGONM */
    char buf_x[128];
    char buf_y[128];
    char buf_m[128];
    char buf[512];
    int ib;
    int iv;
    double x;
    double y;
    double m;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_m, "%1.6f", m);
	  gaiaOutClean (buf_m);
	  if (iv == 0)
	      sprintf (buf, "(%s %s %s", buf_x, buf_y, buf_m);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ", %s %s %s)", buf_x, buf_y, buf_m);
	  else
	      sprintf (buf, ", %s %s %s", buf_x, buf_y, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		sprintf (buf_x, "%1.6f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.6f", y);
		gaiaOutClean (buf_y);
		sprintf (buf_m, "%1.6f", m);
		gaiaOutClean (buf_m);
		if (iv == 0)
		    sprintf (buf, ", (%s %s %s", buf_x, buf_y, buf_m);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ", %s %s %s)", buf_x, buf_y, buf_m);
		else
		    sprintf (buf, ", %s %s %s", buf_x, buf_y, buf_m);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

static void
gaiaOutPolygonZM (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats a WKT POLYGONZM */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf_m[128];
    char buf[1024];
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
	  sprintf (buf_x, "%1.6f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.6f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.6f", z);
	  gaiaOutClean (buf_z);
	  sprintf (buf_m, "%1.6f", m);
	  gaiaOutClean (buf_m);
	  if (iv == 0)
	      sprintf (buf, "(%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ", %s %s %s %s)", buf_x, buf_y, buf_z, buf_m);
	  else
	      sprintf (buf, ", %s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		sprintf (buf_x, "%1.6f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.6f", y);
		gaiaOutClean (buf_y);
		sprintf (buf_z, "%1.6f", z);
		gaiaOutClean (buf_z);
		sprintf (buf_m, "%1.6f", m);
		gaiaOutClean (buf_m);
		if (iv == 0)
		    sprintf (buf, ", (%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ", %s %s %s %s)", buf_x, buf_y, buf_z, buf_m);
		else
		    sprintf (buf, ", %s %s %s %s", buf_x, buf_y, buf_z, buf_m);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

static void
gaiaOutEwktPolygon (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats an EWKT POLYGON */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    int ib;
    int iv;
    double x;
    double y;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPoint (ring->Coords, iv, &x, &y);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  if (iv == 0)
	      sprintf (buf, "(%s %s", buf_x, buf_y);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ",%s %s)", buf_x, buf_y);
	  else
	      sprintf (buf, ",%s %s", buf_x, buf_y);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPoint (ring->Coords, iv, &x, &y);
		sprintf (buf_x, "%1.15f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.15f", y);
		gaiaOutClean (buf_y);
		if (iv == 0)
		    sprintf (buf, ",(%s %s", buf_x, buf_y);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ",%s %s)", buf_x, buf_y);
		else
		    sprintf (buf, ",%s %s", buf_x, buf_y);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

GAIAGEO_DECLARE void
gaiaOutEwktPolygonZ (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats an EWKT POLYGONZ */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    int ib;
    int iv;
    double x;
    double y;
    double z;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.15f", z);
	  gaiaOutClean (buf_z);
	  if (iv == 0)
	      sprintf (buf, "(%s %s %s", buf_x, buf_y, buf_z);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ",%s %s %s)", buf_x, buf_y, buf_z);
	  else
	      sprintf (buf, ",%s %s %s", buf_x, buf_y, buf_z);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		sprintf (buf_x, "%1.15f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.15f", y);
		gaiaOutClean (buf_y);
		sprintf (buf_z, "%1.15f", z);
		gaiaOutClean (buf_z);
		if (iv == 0)
		    sprintf (buf, ",(%s %s %s", buf_x, buf_y, buf_z);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ",%s %s %s)", buf_x, buf_y, buf_z);
		else
		    sprintf (buf, ",%s %s %s", buf_x, buf_y, buf_z);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

static void
gaiaOutEwktPolygonM (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats an EWKT POLYGONM */
    char buf_x[128];
    char buf_y[128];
    char buf_m[128];
    char buf[512];
    int ib;
    int iv;
    double x;
    double y;
    double m;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_m, "%1.15f", m);
	  gaiaOutClean (buf_m);
	  if (iv == 0)
	      sprintf (buf, "(%s %s %s", buf_x, buf_y, buf_m);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ",%s %s %s)", buf_x, buf_y, buf_m);
	  else
	      sprintf (buf, ",%s %s %s", buf_x, buf_y, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		sprintf (buf_x, "%1.15f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.15f", y);
		gaiaOutClean (buf_y);
		sprintf (buf_m, "%1.15f", m);
		gaiaOutClean (buf_m);
		if (iv == 0)
		    sprintf (buf, ",(%s %s %s", buf_x, buf_y, buf_m);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ",%s %s %s)", buf_x, buf_y, buf_m);
		else
		    sprintf (buf, ",%s %s %s", buf_x, buf_y, buf_m);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

static void
gaiaOutEwktPolygonZM (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polyg)
{
/* formats an EWKT POLYGONZM */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf_m[128];
    char buf[1024];
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaRingPtr ring = polyg->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
	  sprintf (buf_x, "%1.15f", x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%1.15f", y);
	  gaiaOutClean (buf_y);
	  sprintf (buf_z, "%1.15f", z);
	  gaiaOutClean (buf_z);
	  sprintf (buf_m, "%1.15f", m);
	  gaiaOutClean (buf_m);
	  if (iv == 0)
	      sprintf (buf, "(%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  else if (iv == (ring->Points - 1))
	      sprintf (buf, ",%s %s %s %s)", buf_x, buf_y, buf_z, buf_m);
	  else
	      sprintf (buf, ",%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    for (ib = 0; ib < polyg->NumInteriors; ib++)
      {
	  ring = polyg->Interiors + ib;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		sprintf (buf_x, "%1.15f", x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%1.15f", y);
		gaiaOutClean (buf_y);
		sprintf (buf_z, "%1.15f", z);
		gaiaOutClean (buf_z);
		sprintf (buf_m, "%1.15f", m);
		gaiaOutClean (buf_m);
		if (iv == 0)
		    sprintf (buf, ",(%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
		else if (iv == (ring->Points - 1))
		    sprintf (buf, ",%s %s %s %s)", buf_x, buf_y, buf_z, buf_m);
		else
		    sprintf (buf, ",%s %s %s %s", buf_x, buf_y, buf_z, buf_m);
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
      }
}

GAIAGEO_DECLARE void
gaiaOutWkt (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom)
{
/* prints the WKT representation of current geometry */
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counting how many POLYGONs are there */
	  pgs++;
	  polyg = polyg->Next;
      }
    if ((pts + lns + pgs) == 1
	&& (geom->DeclaredType == GAIA_POINT
	    || geom->DeclaredType == GAIA_LINESTRING
	    || geom->DeclaredType == GAIA_POLYGON))
      {
	  /* we have only one elementary geometry */
	  point = geom->FirstPoint;
	  while (point)
	    {
		if (point->DimensionModel == GAIA_XY_Z)
		  {
		      /* processing POINTZ */
		      gaiaAppendToOutBuffer (out_buf, "POINT Z(");
		      gaiaOutPointZ (out_buf, point);
		  }
		else if (point->DimensionModel == GAIA_XY_M)
		  {
		      /* processing POINTM */
		      gaiaAppendToOutBuffer (out_buf, "POINT M(");
		      gaiaOutPointM (out_buf, point);
		  }
		else if (point->DimensionModel == GAIA_XY_Z_M)
		  {
		      /* processing POINTZM */
		      gaiaAppendToOutBuffer (out_buf, "POINT ZM(");
		      gaiaOutPointZM (out_buf, point);
		  }
		else
		  {
		      /* processing POINT */
		      gaiaAppendToOutBuffer (out_buf, "POINT(");
		      gaiaOutPoint (out_buf, point);
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      /* processing LINESTRINGZ */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING Z(");
		      gaiaOutLinestringZ (out_buf, line);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      /* processing LINESTRINGM */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING M(");
		      gaiaOutLinestringM (out_buf, line);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      /* processing LINESTRINGZM */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING ZM(");
		      gaiaOutLinestringZM (out_buf, line);
		  }
		else
		  {
		      /* processing LINESTRING */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
		      gaiaOutLinestring (out_buf, line);
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		if (polyg->DimensionModel == GAIA_XY_Z)
		  {
		      /* processing POLYGONZ */
		      gaiaAppendToOutBuffer (out_buf, "POLYGON Z(");
		      gaiaOutPolygonZ (out_buf, polyg);
		  }
		else if (polyg->DimensionModel == GAIA_XY_M)
		  {
		      /* processing POLYGONM */
		      gaiaAppendToOutBuffer (out_buf, "POLYGON M(");
		      gaiaOutPolygonM (out_buf, polyg);
		  }
		else if (polyg->DimensionModel == GAIA_XY_Z_M)
		  {
		      /* processing POLYGONZM */
		      gaiaAppendToOutBuffer (out_buf, "POLYGON ZM(");
		      gaiaOutPolygonZM (out_buf, polyg);
		  }
		else
		  {
		      /* processing POLYGON */
		      gaiaAppendToOutBuffer (out_buf, "POLYGON(");
		      gaiaOutPolygon (out_buf, polyg);
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
		polyg = polyg->Next;
	    }
      }
    else
      {
	  /* we have some kind of complex geometry */
	  if (pts > 0 && lns == 0 && pgs == 0
	      && geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		/* some kind of MULTIPOINT */
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOINT Z(");
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOINT M(");
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOINT ZM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOINT(");
		point = geom->FirstPoint;
		while (point)
		  {
		      if (point->DimensionModel == GAIA_XY_Z)
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ", ");
			    gaiaOutPointZ (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_M)
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ", ");
			    gaiaOutPointM (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_Z_M)
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ", ");
			    gaiaOutPointZM (out_buf, point);
			}
		      else
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ", ");
			    gaiaOutPoint (out_buf, point);
			}
		      point = point->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns > 0 && pgs == 0
		   && geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		/* some kind of MULTILINESTRING */
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaAppendToOutBuffer (out_buf, "MULTILINESTRING Z(");
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTILINESTRING M(");
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTILINESTRING ZM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "MULTILINESTRING(");
		line = geom->FirstLinestring;
		while (line)
		  {
		      if (line != geom->FirstLinestring)
			  gaiaAppendToOutBuffer (out_buf, ", (");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      if (line->DimensionModel == GAIA_XY_Z)
			{
			    gaiaOutLinestringZ (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (line->DimensionModel == GAIA_XY_M)
			{
			    gaiaOutLinestringM (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (line->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaOutLinestringZM (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else
			{
			    gaiaOutLinestring (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      line = line->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns == 0 && pgs > 0
		   && geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		/* some kind of MULTIPOLYGON */
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGON Z(");
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGON M(");
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGON ZM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGON(");
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      if (polyg != geom->FirstPolygon)
			  gaiaAppendToOutBuffer (out_buf, ", (");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      if (polyg->DimensionModel == GAIA_XY_Z)
			{
			    gaiaOutPolygonZ (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (polyg->DimensionModel == GAIA_XY_M)
			{
			    gaiaOutPolygonM (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (polyg->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaOutPolygonZM (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else
			{
			    gaiaOutPolygon (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else
	    {
		/* some kind of GEOMETRYCOLLECTION */
		int ie = 0;
		if (geom->DimensionModel == GAIA_XY_Z)
		    gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTION Z(");
		else if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTION M(");
		else if (geom->DimensionModel == GAIA_XY_Z_M)
		    gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTION ZM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTION(");
		point = geom->FirstPoint;
		while (point)
		  {
		      /* processing POINTs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ", ");
		      ie++;
		      if (point->DimensionModel == GAIA_XY_Z)
			{
			    gaiaAppendToOutBuffer (out_buf, "POINT Z(");
			    gaiaOutPointZ (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POINT M(");
			    gaiaOutPointM (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POINT ZM(");
			    gaiaOutPointZM (out_buf, point);
			}
		      else
			{
			    gaiaAppendToOutBuffer (out_buf, "POINT(");
			    gaiaOutPoint (out_buf, point);
			}
		      gaiaAppendToOutBuffer (out_buf, ")");
		      point = point->Next;
		  }
		line = geom->FirstLinestring;
		while (line)
		  {
		      /* processing LINESTRINGs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ", ");
		      ie++;
		      if (line->DimensionModel == GAIA_XY_Z)
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRING Z(");
			    gaiaOutLinestringZ (out_buf, line);
			}
		      else if (line->DimensionModel == GAIA_XY_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRING M(");
			    gaiaOutLinestringM (out_buf, line);
			}
		      else if (line->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRING ZM(");
			    gaiaOutLinestringZM (out_buf, line);
			}
		      else
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
			    gaiaOutLinestring (out_buf, line);
			}
		      gaiaAppendToOutBuffer (out_buf, ")");
		      line = line->Next;
		  }
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      /* processing POLYGONs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ", ");
		      ie++;
		      if (polyg->DimensionModel == GAIA_XY_Z)
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGON Z(");
			    gaiaOutPolygonZ (out_buf, polyg);
			}
		      else if (polyg->DimensionModel == GAIA_XY_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGON M(");
			    gaiaOutPolygonM (out_buf, polyg);
			}
		      else if (polyg->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGON ZM(");
			    gaiaOutPolygonZM (out_buf, polyg);
			}
		      else
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGON(");
			    gaiaOutPolygon (out_buf, polyg);
			}
		      gaiaAppendToOutBuffer (out_buf, ")");
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
      }
}

GAIAGEO_DECLARE void
gaiaOutWktStrict (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom, int precision)
{
/* 
 * prints the WKT representation of current geometry 
 * strictly conformant 2D WKT implementation
*/
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counting how many POLYGONs are there */
	  pgs++;
	  polyg = polyg->Next;
      }
    if ((pts + lns + pgs) == 1
	&& (geom->DeclaredType == GAIA_POINT
	    || geom->DeclaredType == GAIA_LINESTRING
	    || geom->DeclaredType == GAIA_POLYGON))
      {
	  /* we have only one elementary geometry */
	  point = geom->FirstPoint;
	  while (point)
	    {
		/* processing POINT */
		gaiaAppendToOutBuffer (out_buf, "POINT(");
		gaiaOutPointStrict (out_buf, point, precision);
		gaiaAppendToOutBuffer (out_buf, ")");
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		/* processing LINESTRING */
		gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
		gaiaOutLinestringStrict (out_buf, line, precision);
		gaiaAppendToOutBuffer (out_buf, ")");
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		/* processing POLYGON */
		gaiaAppendToOutBuffer (out_buf, "POLYGON(");
		gaiaOutPolygonStrict (out_buf, polyg, precision);
		gaiaAppendToOutBuffer (out_buf, ")");
		polyg = polyg->Next;
	    }
      }
    else
      {
	  /* we have some kind of complex geometry */
	  if (pts > 0 && lns == 0 && pgs == 0
	      && geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		/* some kind of MULTIPOINT */
		gaiaAppendToOutBuffer (out_buf, "MULTIPOINT(");
		point = geom->FirstPoint;
		while (point)
		  {
		      if (point != geom->FirstPoint)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      gaiaOutPointStrict (out_buf, point, precision);
		      point = point->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns > 0 && pgs == 0
		   && geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		/* some kind of MULTILINESTRING */
		gaiaAppendToOutBuffer (out_buf, "MULTILINESTRING(");
		line = geom->FirstLinestring;
		while (line)
		  {
		      if (line != geom->FirstLinestring)
			  gaiaAppendToOutBuffer (out_buf, ",(");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      gaiaOutLinestringStrict (out_buf, line, precision);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      line = line->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns == 0 && pgs > 0
		   && geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		/* some kind of MULTIPOLYGON */
		gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGON(");
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      if (polyg != geom->FirstPolygon)
			  gaiaAppendToOutBuffer (out_buf, ",(");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      gaiaOutPolygonStrict (out_buf, polyg, precision);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else
	    {
		/* some kind of GEOMETRYCOLLECTION */
		int ie = 0;
		gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTION(");
		point = geom->FirstPoint;
		while (point)
		  {
		      /* processing POINTs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      ie++;
		      gaiaAppendToOutBuffer (out_buf, "POINT(");
		      gaiaOutPointStrict (out_buf, point, precision);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      point = point->Next;
		  }
		line = geom->FirstLinestring;
		while (line)
		  {
		      /* processing LINESTRINGs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      ie++;
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
		      gaiaOutLinestringStrict (out_buf, line, precision);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      line = line->Next;
		  }
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      /* processing POLYGONs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      ie++;
		      gaiaAppendToOutBuffer (out_buf, "POLYGON(");
		      gaiaOutPolygonStrict (out_buf, polyg, precision);
		      gaiaAppendToOutBuffer (out_buf, ")");
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
      }
}

GAIAGEO_DECLARE void
gaiaToEWKT (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom)
{
/* prints the EWKT representation of current geometry */
    char buf[128];
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    if (!geom)
	return;
    sprintf (buf, "SRID=%d;", geom->Srid);
    gaiaAppendToOutBuffer (out_buf, buf);
    point = geom->FirstPoint;
    while (point)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counting how many POLYGONs are there */
	  pgs++;
	  polyg = polyg->Next;
      }
    if ((pts + lns + pgs) == 1
	&& (geom->DeclaredType == GAIA_POINT
	    || geom->DeclaredType == GAIA_LINESTRING
	    || geom->DeclaredType == GAIA_POLYGON))
      {
	  /* we have only one elementary geometry */
	  point = geom->FirstPoint;
	  while (point)
	    {
		if (point->DimensionModel == GAIA_XY_Z)
		  {
		      /* processing POINTZ */
		      gaiaAppendToOutBuffer (out_buf, "POINT(");
		      gaiaOutEwktPointZ (out_buf, point);
		  }
		else if (point->DimensionModel == GAIA_XY_M)
		  {
		      /* processing POINTM */
		      gaiaAppendToOutBuffer (out_buf, "POINTM(");
		      gaiaOutEwktPointM (out_buf, point);
		  }
		else if (point->DimensionModel == GAIA_XY_Z_M)
		  {
		      /* processing POINTZM */
		      gaiaAppendToOutBuffer (out_buf, "POINT(");
		      gaiaOutEwktPointZM (out_buf, point);
		  }
		else
		  {
		      /* processing POINT */
		      gaiaAppendToOutBuffer (out_buf, "POINT(");
		      gaiaOutEwktPoint (out_buf, point);
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      /* processing LINESTRINGZ */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
		      gaiaOutEwktLinestringZ (out_buf, line);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      /* processing LINESTRINGM */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRINGM(");
		      gaiaOutEwktLinestringM (out_buf, line);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      /* processing LINESTRINGZM */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
		      gaiaOutEwktLinestringZM (out_buf, line);
		  }
		else
		  {
		      /* processing LINESTRING */
		      gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
		      gaiaOutEwktLinestring (out_buf, line);
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		if (polyg->DimensionModel == GAIA_XY_Z)
		  {
		      /* processing POLYGONZ */
		      gaiaAppendToOutBuffer (out_buf, "POLYGON(");
		      gaiaOutEwktPolygonZ (out_buf, polyg);
		  }
		else if (polyg->DimensionModel == GAIA_XY_M)
		  {
		      /* processing POLYGONM */
		      gaiaAppendToOutBuffer (out_buf, "POLYGONM(");
		      gaiaOutEwktPolygonM (out_buf, polyg);
		  }
		else if (polyg->DimensionModel == GAIA_XY_Z_M)
		  {
		      /* processing POLYGONZM */
		      gaiaAppendToOutBuffer (out_buf, "POLYGON(");
		      gaiaOutEwktPolygonZM (out_buf, polyg);
		  }
		else
		  {
		      /* processing POLYGON */
		      gaiaAppendToOutBuffer (out_buf, "POLYGON(");
		      gaiaOutEwktPolygon (out_buf, polyg);
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
		polyg = polyg->Next;
	    }
      }
    else
      {
	  /* we have some kind of complex geometry */
	  if (pts > 0 && lns == 0 && pgs == 0
	      && geom->DeclaredType == GAIA_MULTIPOINT)
	    {
		/* some kind of MULTIPOINT */
		if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOINTM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOINT(");
		point = geom->FirstPoint;
		while (point)
		  {
		      if (point->DimensionModel == GAIA_XY_Z)
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ",");
			    gaiaOutEwktPointZ (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_M)
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ",");
			    gaiaOutEwktPointM (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_Z_M)
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ",");
			    gaiaOutEwktPointZM (out_buf, point);
			}
		      else
			{
			    if (point != geom->FirstPoint)
				gaiaAppendToOutBuffer (out_buf, ",");
			    gaiaOutEwktPoint (out_buf, point);
			}
		      point = point->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns > 0 && pgs == 0
		   && geom->DeclaredType == GAIA_MULTILINESTRING)
	    {
		/* some kind of MULTILINESTRING */
		if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTILINESTRINGM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "MULTILINESTRING(");
		line = geom->FirstLinestring;
		while (line)
		  {
		      if (line != geom->FirstLinestring)
			  gaiaAppendToOutBuffer (out_buf, ",(");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      if (line->DimensionModel == GAIA_XY_Z)
			{
			    gaiaOutEwktLinestringZ (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (line->DimensionModel == GAIA_XY_M)
			{
			    gaiaOutEwktLinestringM (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (line->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaOutEwktLinestringZM (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else
			{
			    gaiaOutEwktLinestring (out_buf, line);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      line = line->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else if (pts == 0 && lns == 0 && pgs > 0
		   && geom->DeclaredType == GAIA_MULTIPOLYGON)
	    {
		/* some kind of MULTIPOLYGON */
		if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGONM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "MULTIPOLYGON(");
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      if (polyg != geom->FirstPolygon)
			  gaiaAppendToOutBuffer (out_buf, ",(");
		      else
			  gaiaAppendToOutBuffer (out_buf, "(");
		      if (polyg->DimensionModel == GAIA_XY_Z)
			{
			    gaiaOutEwktPolygonZ (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (polyg->DimensionModel == GAIA_XY_M)
			{
			    gaiaOutEwktPolygonM (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else if (polyg->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaOutEwktPolygonZM (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      else
			{
			    gaiaOutEwktPolygon (out_buf, polyg);
			    gaiaAppendToOutBuffer (out_buf, ")");
			}
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
	  else
	    {
		/* some kind of GEOMETRYCOLLECTION */
		int ie = 0;
		if (geom->DimensionModel == GAIA_XY_M)
		    gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTIONM(");
		else
		    gaiaAppendToOutBuffer (out_buf, "GEOMETRYCOLLECTION(");
		point = geom->FirstPoint;
		while (point)
		  {
		      /* processing POINTs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      ie++;
		      if (point->DimensionModel == GAIA_XY_Z)
			{
			    gaiaAppendToOutBuffer (out_buf, "POINT(");
			    gaiaOutEwktPointZ (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POINTM(");
			    gaiaOutEwktPointM (out_buf, point);
			}
		      else if (point->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POINT(");
			    gaiaOutEwktPointZM (out_buf, point);
			}
		      else
			{
			    gaiaAppendToOutBuffer (out_buf, "POINT(");
			    gaiaOutEwktPoint (out_buf, point);
			}
		      gaiaAppendToOutBuffer (out_buf, ")");
		      point = point->Next;
		  }
		line = geom->FirstLinestring;
		while (line)
		  {
		      /* processing LINESTRINGs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      ie++;
		      if (line->DimensionModel == GAIA_XY_Z)
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
			    gaiaOutEwktLinestringZ (out_buf, line);
			}
		      else if (line->DimensionModel == GAIA_XY_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRINGM(");
			    gaiaOutEwktLinestringM (out_buf, line);
			}
		      else if (line->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
			    gaiaOutEwktLinestringZM (out_buf, line);
			}
		      else
			{
			    gaiaAppendToOutBuffer (out_buf, "LINESTRING(");
			    gaiaOutEwktLinestring (out_buf, line);
			}
		      gaiaAppendToOutBuffer (out_buf, ")");
		      line = line->Next;
		  }
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      /* processing POLYGONs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      ie++;
		      if (polyg->DimensionModel == GAIA_XY_Z)
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGON(");
			    gaiaOutEwktPolygonZ (out_buf, polyg);
			}
		      else if (polyg->DimensionModel == GAIA_XY_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGONM(");
			    gaiaOutEwktPolygonM (out_buf, polyg);
			}
		      else if (polyg->DimensionModel == GAIA_XY_Z_M)
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGON(");
			    gaiaOutEwktPolygonZM (out_buf, polyg);
			}
		      else
			{
			    gaiaAppendToOutBuffer (out_buf, "POLYGON(");
			    gaiaOutEwktPolygon (out_buf, polyg);
			}
		      gaiaAppendToOutBuffer (out_buf, ")");
		      polyg = polyg->Next;
		  }
		gaiaAppendToOutBuffer (out_buf, ")");
	    }
      }
}

/*
/
/  Gaia common support for SVG encoded geometries
/
////////////////////////////////////////////////////////////
/
/ Author: Klaus Foerster klaus.foerster@svg.cc
/ version 0.9. 2008 September 21
 /
 */

static void
SvgCoords (gaiaOutBufferPtr out_buf, gaiaPointPtr point, int precision)
{
/* formats POINT as SVG-attributes x,y */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    sprintf (buf_x, "%.*f", precision, point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%.*f", precision, point->Y * -1);
    gaiaOutClean (buf_y);
    sprintf (buf, "x=\"%s\" y=\"%s\"", buf_x, buf_y);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
SvgCircle (gaiaOutBufferPtr out_buf, gaiaPointPtr point, int precision)
{
/* formats POINT as SVG-attributes cx,cy */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    sprintf (buf_x, "%.*f", precision, point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%.*f", precision, point->Y * -1);
    gaiaOutClean (buf_y);
    sprintf (buf, "cx=\"%s\" cy=\"%s\"", buf_x, buf_y);
    gaiaAppendToOutBuffer (out_buf, buf);
}

static void
SvgPathRelative (gaiaOutBufferPtr out_buf, int dims, int points, double *coords,
		 int precision, int closePath)
{
/* formats LINESTRING as SVG-path d-attribute with relative coordinate moves */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    double x;
    double y;
    double z;
    double m;
    double lastX = 0.0;
    double lastY = 0.0;
    int iv;
    for (iv = 0; iv < points; iv++)
      {
	  if (dims == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (coords, iv, &x, &y, &z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaGetPointXYM (coords, iv, &x, &y, &m);
	    }
	  else if (dims == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (coords, iv, &x, &y);
	    }
	  sprintf (buf_x, "%.*f", precision, x - lastX);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, (y - lastY) * -1);
	  gaiaOutClean (buf_y);
	  if (iv == 0)
	      sprintf (buf, "M %s %s l ", buf_x, buf_y);
	  else
	      sprintf (buf, "%s %s ", buf_x, buf_y);
	  lastX = x;
	  lastY = y;
	  if (iv == points - 1 && closePath == 1)
	      sprintf (buf, "z ");
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

static void
SvgPathAbsolute (gaiaOutBufferPtr out_buf, int dims, int points, double *coords,
		 int precision, int closePath)
{
/* formats LINESTRING as SVG-path d-attribute with relative coordinate moves */
    char buf_x[128];
    char buf_y[128];
    char buf[256];
    double x;
    double y;
    double z;
    double m;
    int iv;
    for (iv = 0; iv < points; iv++)
      {
	  if (dims == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (coords, iv, &x, &y, &z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaGetPointXYM (coords, iv, &x, &y, &m);
	    }
	  else if (dims == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (coords, iv, &x, &y);
	    }
	  sprintf (buf_x, "%.*f", precision, x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, y * -1);
	  gaiaOutClean (buf_y);
	  if (iv == 0)
	      sprintf (buf, "M %s %s L ", buf_x, buf_y);
	  else
	      sprintf (buf, "%s %s ", buf_x, buf_y);
	  if (iv == points - 1 && closePath == 1)
	      sprintf (buf, "z ");
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
}

GAIAGEO_DECLARE void
gaiaOutSvg (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom, int relative,
	    int precision)
{
/* prints the SVG representation of current geometry */
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    int ib;
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    if (!geom)
	return;
    point = geom->FirstPoint;
    while (point)
      {
	  /* counting how many POINTs are there */
	  pts++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* counting how many LINESTRINGs are there */
	  lns++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* counting how many POLYGONs are there */
	  pgs++;
	  polyg = polyg->Next;
      }

    if ((pts + lns + pgs) == 1)
      {
	  /* we have only one elementary geometry */
	  point = geom->FirstPoint;
	  while (point)
	    {
		/* processing POINT */
		if (relative == 1)
		    SvgCoords (out_buf, point, precision);
		else
		    SvgCircle (out_buf, point, precision);
		point = point->Next;
	    }
	  line = geom->FirstLinestring;
	  while (line)
	    {
		/* processing LINESTRING */
		if (relative == 1)
		    SvgPathRelative (out_buf, line->DimensionModel,
				     line->Points, line->Coords, precision, 0);
		else
		    SvgPathAbsolute (out_buf, line->DimensionModel,
				     line->Points, line->Coords, precision, 0);
		line = line->Next;
	    }
	  polyg = geom->FirstPolygon;
	  while (polyg)
	    {
		/* process exterior and interior rings */
		ring = polyg->Exterior;
		if (relative == 1)
		  {
		      SvgPathRelative (out_buf, ring->DimensionModel,
				       ring->Points, ring->Coords, precision,
				       1);
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    ring = polyg->Interiors + ib;
			    SvgPathRelative (out_buf, ring->DimensionModel,
					     ring->Points, ring->Coords,
					     precision, 1);
			}
		  }
		else
		  {
		      SvgPathAbsolute (out_buf, ring->DimensionModel,
				       ring->Points, ring->Coords, precision,
				       1);
		      for (ib = 0; ib < polyg->NumInteriors; ib++)
			{
			    ring = polyg->Interiors + ib;
			    SvgPathAbsolute (out_buf, ring->DimensionModel,
					     ring->Points, ring->Coords,
					     precision, 1);
			}
		  }
		polyg = polyg->Next;
	    }
      }
    else
      {
	  /* we have some kind of complex geometry */
	  if (pts > 0 && lns == 0 && pgs == 0)
	    {
		/* this one is a MULTIPOINT */
		point = geom->FirstPoint;
		while (point)
		  {
		      /* processing POINTs */
		      if (point != geom->FirstPoint)
			  gaiaAppendToOutBuffer (out_buf, ",");
		      if (relative == 1)
			  SvgCoords (out_buf, point, precision);
		      else
			  SvgCircle (out_buf, point, precision);
		      point = point->Next;
		  }
	    }
	  else if (pts == 0 && lns > 0 && pgs == 0)
	    {
		/* this one is a MULTILINESTRING */
		line = geom->FirstLinestring;
		while (line)
		  {
		      /* processing LINESTRINGs */
		      if (relative == 1)
			  SvgPathRelative (out_buf, line->DimensionModel,
					   line->Points, line->Coords,
					   precision, 0);
		      else
			  SvgPathAbsolute (out_buf, line->DimensionModel,
					   line->Points, line->Coords,
					   precision, 0);
		      line = line->Next;
		  }
	    }
	  else if (pts == 0 && lns == 0 && pgs > 0)
	    {
		/* this one is a MULTIPOLYGON */
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      /* processing POLYGONs */
		      ring = polyg->Exterior;
		      if (relative == 1)
			{
			    SvgPathRelative (out_buf, ring->DimensionModel,
					     ring->Points, ring->Coords,
					     precision, 1);
			    for (ib = 0; ib < polyg->NumInteriors; ib++)
			      {
				  ring = polyg->Interiors + ib;
				  SvgPathRelative (out_buf,
						   ring->DimensionModel,
						   ring->Points, ring->Coords,
						   precision, 1);
			      }
			}
		      else
			{
			    SvgPathAbsolute (out_buf, ring->DimensionModel,
					     ring->Points, ring->Coords,
					     precision, 1);
			    for (ib = 0; ib < polyg->NumInteriors; ib++)
			      {
				  ring = polyg->Interiors + ib;
				  SvgPathAbsolute (out_buf,
						   ring->DimensionModel,
						   ring->Points, ring->Coords,
						   precision, 1);
			      }
			}
		      polyg = polyg->Next;
		  }
	    }
	  else
	    {
		/* this one is a GEOMETRYCOLLECTION */
		int ie = 0;
		point = geom->FirstPoint;
		while (point)
		  {
		      /* processing POINTs */
		      if (ie > 0)
			{
			    gaiaAppendToOutBuffer (out_buf, ";");
			}
		      ie++;
		      if (relative == 1)
			  SvgCoords (out_buf, point, precision);
		      else
			  SvgCircle (out_buf, point, precision);
		      point = point->Next;
		  }
		line = geom->FirstLinestring;
		while (line)
		  {
		      /* processing LINESTRINGs */
		      if (ie > 0)
			  gaiaAppendToOutBuffer (out_buf, ";");
		      ie++;
		      if (relative == 1)
			  SvgPathRelative (out_buf, line->DimensionModel,
					   line->Points, line->Coords,
					   precision, 0);
		      else
			  SvgPathAbsolute (out_buf, line->DimensionModel,
					   line->Points, line->Coords,
					   precision, 0);
		      line = line->Next;
		  }
		polyg = geom->FirstPolygon;
		while (polyg)
		  {
		      /* processing POLYGONs */
		      ie++;
		      /* process exterior and interior rings */
		      ring = polyg->Exterior;
		      if (relative == 1)
			{
			    SvgPathRelative (out_buf, ring->DimensionModel,
					     ring->Points, ring->Coords,
					     precision, 1);
			    for (ib = 0; ib < polyg->NumInteriors; ib++)
			      {
				  ring = polyg->Interiors + ib;
				  SvgPathRelative (out_buf,
						   ring->DimensionModel,
						   ring->Points, ring->Coords,
						   precision, 1);
			      }
			}
		      else
			{
			    SvgPathAbsolute (out_buf, ring->DimensionModel,
					     ring->Points, ring->Coords,
					     precision, 1);
			    for (ib = 0; ib < polyg->NumInteriors; ib++)
			      {
				  ring = polyg->Interiors + ib;
				  SvgPathAbsolute (out_buf,
						   ring->DimensionModel,
						   ring->Points, ring->Coords,
						   precision, 1);
			      }
			}
		      polyg = polyg->Next;
		  }
	    }
      }
}

/* END of Klaus Foerster SVG implementation */


static char *
XmlClean (const char *string)
{
/* well formatting a text string for XML */
    int ind;
    char *clean;
    char *p_out;
    int len = strlen (string);
    clean = malloc (len * 3);
    if (!clean)
	return NULL;
    p_out = clean;
    for (ind = 0; ind < len; ind++)
      {
	  /* masking XML special chars */
	  switch (string[ind])
	    {
	    case '&':
		*p_out++ = '&';
		*p_out++ = 'a';
		*p_out++ = 'm';
		*p_out++ = 'p';
		*p_out++ = ';';
		break;
	    case '<':
		*p_out++ = '&';
		*p_out++ = 'l';
		*p_out++ = 't';
		*p_out++ = ';';
		break;
	    case '>':
		*p_out++ = '&';
		*p_out++ = 'g';
		*p_out++ = 't';
		*p_out++ = ';';
		break;
	    case '"':
		*p_out++ = '&';
		*p_out++ = 'q';
		*p_out++ = 'u';
		*p_out++ = 'o';
		*p_out++ = 't';
		*p_out++ = ';';
		break;
	    default:
		*p_out++ = string[ind];
		break;
	    };
      }
    *p_out = '\0';
    return clean;
}

static void
out_kml_point (gaiaOutBufferPtr out_buf, gaiaPointPtr point, int precision)
{
/* formats POINT as KML [x,y] */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    sprintf (buf_x, "%.*f", precision, point->X);
    gaiaOutClean (buf_x);
    sprintf (buf_y, "%.*f", precision, point->Y);
    gaiaOutClean (buf_y);
    if (point->DimensionModel == GAIA_XY_Z
	|| point->DimensionModel == GAIA_XY_Z_M)
      {
	  sprintf (buf_z, "%.*f", precision, point->Z);
	  gaiaOutClean (buf_z);
      }
    gaiaAppendToOutBuffer (out_buf, "<Point><coordinates>");
    if (point->DimensionModel == GAIA_XY_Z
	|| point->DimensionModel == GAIA_XY_Z_M)
	sprintf (buf, "%s,%s,%s", buf_x, buf_y, buf_z);
    else
	sprintf (buf, "%s,%s", buf_x, buf_y);
    gaiaAppendToOutBuffer (out_buf, buf);
    gaiaAppendToOutBuffer (out_buf, "</coordinates></Point>");
}

static void
out_kml_linestring (gaiaOutBuffer * out_buf, int dims, int points,
		    double *coords, int precision)
{
/* formats LINESTRING as KML [x,y] */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    int iv;
    double x;
    double y;
    double z;
    double m;
    gaiaAppendToOutBuffer (out_buf, "<LineString><coordinates>");
    for (iv = 0; iv < points; iv++)
      {
	  /* exporting vertices */
	  if (dims == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (coords, iv, &x, &y, &z);
	    }
	  else if (dims == GAIA_XY_M)
	    {
		gaiaGetPointXYM (coords, iv, &x, &y, &m);
	    }
	  else if (dims == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (coords, iv, &x, &y);
	    }
	  sprintf (buf_x, "%.*f", precision, x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, y);
	  gaiaOutClean (buf_y);
	  if (dims == GAIA_XY_Z || dims == GAIA_XY_Z_M)
	    {
		sprintf (buf_z, "%.*f", precision, z);
		gaiaOutClean (buf_z);
		if (iv == 0)
		    sprintf (buf, "%s,%s,%s", buf_x, buf_y, buf_z);
		else
		    sprintf (buf, " %s,%s,%s", buf_x, buf_y, buf_z);
	    }
	  else
	    {
		if (iv == 0)
		    sprintf (buf, "%s,%s", buf_x, buf_y);
		else
		    sprintf (buf, " %s,%s", buf_x, buf_y);
	    }
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    gaiaAppendToOutBuffer (out_buf, "</coordinates></LineString>");
}

static void
out_kml_polygon (gaiaOutBufferPtr out_buf, gaiaPolygonPtr polygon,
		 int precision)
{
/* formats POLYGON as KML [x,y] */
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    char buf[512];
    gaiaRingPtr ring;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    gaiaAppendToOutBuffer (out_buf, "<Polygon>");
    gaiaAppendToOutBuffer (out_buf,
			   "<outerBoundaryIs><LinearRing><coordinates>");
    ring = polygon->Exterior;
    for (iv = 0; iv < ring->Points; iv++)
      {
	  /* exporting vertices [Exterior Ring] */
	  if (ring->DimensionModel == GAIA_XY_Z)
	    {
		gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
	    }
	  else if (ring->DimensionModel == GAIA_XY_M)
	    {
		gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
	    }
	  else if (ring->DimensionModel == GAIA_XY_Z_M)
	    {
		gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
	    }
	  else
	    {
		gaiaGetPoint (ring->Coords, iv, &x, &y);
	    }
	  sprintf (buf_x, "%.*f", precision, x);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, y);
	  gaiaOutClean (buf_y);
	  if (ring->DimensionModel == GAIA_XY_Z
	      || ring->DimensionModel == GAIA_XY_Z_M)
	    {
		sprintf (buf_z, "%.*f", precision, z);
		gaiaOutClean (buf_z);
		if (iv == 0)
		    sprintf (buf, "%s,%s,%s", buf_x, buf_y, buf_z);
		else
		    sprintf (buf, " %s,%s,%s", buf_x, buf_y, buf_z);
	    }
	  else
	    {
		if (iv == 0)
		    sprintf (buf, "%s,%s", buf_x, buf_y);
		else
		    sprintf (buf, " %s,%s", buf_x, buf_y);
	    }
	  gaiaAppendToOutBuffer (out_buf, buf);
      }
    gaiaAppendToOutBuffer (out_buf,
			   "</coordinates></LinearRing></outerBoundaryIs>");
    for (ib = 0; ib < polygon->NumInteriors; ib++)
      {
	  /* interior rings */
	  ring = polygon->Interiors + ib;
	  gaiaAppendToOutBuffer (out_buf,
				 "<innerBoundaryIs><LinearRing><coordinates>");
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* exporting vertices [Interior Ring] */
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		sprintf (buf_x, "%.*f", precision, x);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%.*f", precision, y);
		gaiaOutClean (buf_y);
		if (ring->DimensionModel == GAIA_XY_Z
		    || ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      sprintf (buf_z, "%.*f", precision, z);
		      gaiaOutClean (buf_z);
		      if (iv == 0)
			  sprintf (buf, "%s,%s,%s", buf_x, buf_y, buf_z);
		      else
			  sprintf (buf, " %s,%s,%s", buf_x, buf_y, buf_z);
		  }
		else
		  {
		      if (iv == 0)
			  sprintf (buf, "%s,%s", buf_x, buf_y);
		      else
			  sprintf (buf, " %s,%s", buf_x, buf_y);
		  }
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  gaiaAppendToOutBuffer (out_buf,
				 "</coordinates></LinearRing></innerBoundaryIs>");
      }
    strcpy (buf, "</Polygon>");
    gaiaAppendToOutBuffer (out_buf, buf);
}

GAIAGEO_DECLARE void
gaiaOutFullKml (gaiaOutBufferPtr out_buf, const char *name, const char *desc,
		gaiaGeomCollPtr geom, int precision)
{
/* prints the 'full' KML representation of current geometry */
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    int count = 0;
    char *xml_clean;
    if (!geom)
	return;
    if (precision > 18)
	precision = 18;

/* counting how many elementary geometries are there */
    point = geom->FirstPoint;
    while (point)
      {
	  count++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  count++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  count++;
	  polyg = polyg->Next;
      }
    if (count == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT ||
	      geom->DeclaredType == GAIA_MULTILINESTRING ||
	      geom->DeclaredType == GAIA_MULTIPOLYGON ||
	      geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      count = 2;
      }

    gaiaAppendToOutBuffer (out_buf, "<Placemark><name>");
    xml_clean = XmlClean (name);
    if (xml_clean)
      {
	  gaiaAppendToOutBuffer (out_buf, xml_clean);
	  free (xml_clean);
      }
    else
	gaiaAppendToOutBuffer (out_buf, " ");
    gaiaAppendToOutBuffer (out_buf, "</name><description>");
    xml_clean = XmlClean (desc);
    if (xml_clean)
      {
	  gaiaAppendToOutBuffer (out_buf, xml_clean);
	  free (xml_clean);
      }
    else
	gaiaAppendToOutBuffer (out_buf, " ");
    gaiaAppendToOutBuffer (out_buf, "</description>");

    if (count > 1)
      {
	  /* MultiGeometry start */
	  gaiaAppendToOutBuffer (out_buf, "<MultiGeometry>");
      }

    point = geom->FirstPoint;
    while (point)
      {
	  /* processing POINT */
	  out_kml_point (out_buf, point, precision);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* processing LINESTRING */
	  out_kml_linestring (out_buf, line->DimensionModel,
			      line->Points, line->Coords, precision);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* processing POLYGON */
	  out_kml_polygon (out_buf, polyg, precision);
	  polyg = polyg->Next;
      }

    if (count > 1)
      {
	  /* MultiGeometry end */
	  gaiaAppendToOutBuffer (out_buf, "</MultiGeometry>");
      }
    gaiaAppendToOutBuffer (out_buf, "</Placemark>");
}

GAIAGEO_DECLARE void
gaiaOutBareKml (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom, int precision)
{
/* prints the 'bare' KML representation of current geometry */
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    int count = 0;
    if (!geom)
	return;
    if (precision > 18)
	precision = 18;

/* counting how many elementary geometries are there */
    point = geom->FirstPoint;
    while (point)
      {
	  count++;
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  count++;
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  count++;
	  polyg = polyg->Next;
      }
    if (count == 1)
      {
	  if (geom->DeclaredType == GAIA_MULTIPOINT ||
	      geom->DeclaredType == GAIA_MULTILINESTRING ||
	      geom->DeclaredType == GAIA_MULTIPOLYGON ||
	      geom->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      count = 2;
      }

    if (count > 1)
      {
	  /* MultiGeometry start */
	  gaiaAppendToOutBuffer (out_buf, "<MultiGeometry>");
      }

    point = geom->FirstPoint;
    while (point)
      {
	  /* processing POINT */
	  out_kml_point (out_buf, point, precision);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* processing LINESTRING */
	  out_kml_linestring (out_buf, line->DimensionModel, line->Points,
			      line->Coords, precision);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* processing POLYGON */
	  out_kml_polygon (out_buf, polyg, precision);
	  polyg = polyg->Next;
      }

    if (count > 1)
      {
	  /* MultiGeometry end */
	  gaiaAppendToOutBuffer (out_buf, "</MultiGeometry>");
      }
}

GAIAGEO_DECLARE void
gaiaOutGml (gaiaOutBufferPtr out_buf, int version, int precision,
	    gaiaGeomCollPtr geom)
{
/*
/ prints the GML representation of current geometry
/ *result* returns the encoded GML or NULL if any error is encountered
*/
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    int has_z;
    int is_multi = 1;
    int is_coll = 0;
    char buf[1024];
    char buf_x[128];
    char buf_y[128];
    char buf_z[128];
    if (!geom)
	return;
    if (precision > 18)
	precision = 18;

    switch (geom->DeclaredType)
      {
      case GAIA_POINT:
      case GAIA_LINESTRING:
      case GAIA_POLYGON:
	  *buf = '\0';
	  is_multi = 0;
	  break;
      case GAIA_MULTIPOINT:
	  if (geom->Srid == -1)
	      strcpy (buf, "<gml:MultiPoint>");
	  else
	      sprintf (buf, "<gml:MultiPoint srsName=\"EPSG:%d\">", geom->Srid);
	  break;
      case GAIA_MULTILINESTRING:
	  if (version == 3)
	    {
		if (geom->Srid == -1)
		    strcpy (buf, "<gml:MultiCurve>");
		else
		    sprintf (buf, "<gml:MultiCurve srsName=\"EPSG:%d\">",
			     geom->Srid);
	    }
	  else
	    {
		if (geom->Srid == -1)
		    strcpy (buf, "<gml:MultiLineString>");
		else
		    sprintf (buf,
			     "<gml:MultiLineString srsName=\"EPSG:%d\">",
			     geom->Srid);
	    }
	  break;
      case GAIA_MULTIPOLYGON:
	  if (version == 3)
	    {
		if (geom->Srid == -1)
		    strcpy (buf, "<gml:MultiSurface>");
		else
		    sprintf (buf, "<gml:MultiSurface srsName=\"EPSG:%d\">",
			     geom->Srid);
	    }
	  else
	    {
		if (geom->Srid == -1)
		    strcpy (buf, "<gml:MultiPolygon>");
		else
		    sprintf (buf, "<gml:MultiPolygon srsName=\"EPSG:%d\">",
			     geom->Srid);
	    }
	  break;
      default:
	  if (geom->Srid == -1)
	      strcpy (buf, "<gml:MultiGeometry>");
	  else
	      sprintf (buf, "<gml:MultiGeometry srsName=\"EPSG:%d\">",
		       geom->Srid);
	  is_coll = 1;
	  break;
      };
    gaiaAppendToOutBuffer (out_buf, buf);
    point = geom->FirstPoint;
    while (point)
      {
	  /* processing POINT */
	  if (is_multi)
	    {
		if (is_coll)
		    strcpy (buf, "<gml:geometryMember>");
		else
		    strcpy (buf, "<gml:pointMember>");
		strcat (buf, "<gml:Point>");
	    }
	  else
	    {
		if (geom->Srid == -1)
		    strcpy (buf, "<gml:Point>");
		else
		    sprintf (buf, "<gml:Point srsName=\"EPSG:%d\">",
			     geom->Srid);
	    }
	  if (version == 3)
	    {
		if (point->DimensionModel == GAIA_XY_Z
		    || point->DimensionModel == GAIA_XY_Z_M)
		    strcat (buf, "<gml:pos srsDimension=\"3\">");
		else
		    strcat (buf, "<gml:pos srsDimension=\"2\">");
	    }
	  else
	      strcat (buf, "<gml:coordinates>");
	  sprintf (buf_x, "%.*f", precision, point->X);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, point->Y);
	  gaiaOutClean (buf_y);
	  if (point->DimensionModel == GAIA_XY_Z
	      || point->DimensionModel == GAIA_XY_Z_M)
	    {
		sprintf (buf_z, "%.*f", precision, point->Z);
		gaiaOutClean (buf_z);
		if (version == 3)
		  {
		      strcat (buf, buf_x);
		      strcat (buf, " ");
		      strcat (buf, buf_y);
		      strcat (buf, " ");
		      strcat (buf, buf_z);
		  }
		else
		  {
		      strcat (buf, buf_x);
		      strcat (buf, ",");
		      strcat (buf, buf_y);
		      strcat (buf, ",");
		      strcat (buf, buf_z);
		  }
	    }
	  else
	    {
		if (version == 3)
		  {
		      strcat (buf, buf_x);
		      strcat (buf, " ");
		      strcat (buf, buf_y);
		  }
		else
		  {
		      strcat (buf, buf_x);
		      strcat (buf, ",");
		      strcat (buf, buf_y);
		  }
	    }
	  if (version == 3)
	      strcat (buf, "</gml:pos>");
	  else
	      strcat (buf, "</gml:coordinates>");
	  if (is_multi)
	    {
		strcat (buf, "</gml:Point>");
		if (is_coll)
		    strcat (buf, "</gml:geometryMember>");
		else
		    strcat (buf, "</gml:pointMember>");
	    }
	  else
	      strcat (buf, "</gml:Point>");
	  gaiaAppendToOutBuffer (out_buf, buf);
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* processing LINESTRING */
	  if (is_multi)
	    {
		if (version == 3)
		  {
		      if (is_coll)
			  strcpy (buf, "<gml:geometryMember>");
		      else
			  strcpy (buf, "<gml:curveMember>");
		      strcat (buf, "<gml:Curve>");
		      strcat (buf, "<gml:segments>");
		      strcat (buf, "<gml:LineStringSegment>");
		      if (line->DimensionModel == GAIA_XY_Z
			  || line->DimensionModel == GAIA_XY_Z_M)
			  strcat (buf, "<gml:posList srsDimension=\"3\">");
		      else
			  strcat (buf, "<gml:posList srsDimension=\"2\">");
		  }
		else
		  {
		      if (is_coll)
			  strcpy (buf, "<gml:geometryMember>");
		      else
			  strcpy (buf, "<gml:lineStringMember>");
		      strcat (buf, "<gml:LineString>");
		      strcat (buf, "<gml:coordinates>");
		  }
	    }
	  else
	    {
		if (version == 3)
		  {
		      if (geom->Srid == -1)
			  strcpy (buf, "<gml:Curve>");
		      else
			  sprintf (buf, "<gml:Curve srsName=\"EPSG:%d\">",
				   geom->Srid);
		      strcat (buf, "<gml:segments>");
		      strcat (buf, "<gml:LineStringSegment>");
		      if (line->DimensionModel == GAIA_XY_Z
			  || line->DimensionModel == GAIA_XY_Z_M)
			  strcat (buf, "<gml:posList srsDimension=\"3\">");
		      else
			  strcat (buf, "<gml:posList srsDimension=\"2\">");
		  }
		else
		  {
		      if (geom->Srid == -1)
			  strcpy (buf, "<gml:LineString>");
		      else
			  sprintf (buf, "<gml:LineString srsName=\"EPSG:%d\">",
				   geom->Srid);
		      strcat (buf, "<gml:coordinates>");
		  }
	    }
	  gaiaAppendToOutBuffer (out_buf, buf);
	  for (iv = 0; iv < line->Points; iv++)
	    {
		/* exporting vertices */
		has_z = 0;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      has_z = 1;
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      has_z = 1;
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		if (iv == 0)
		    *buf = '\0';
		else
		    strcpy (buf, " ");
		if (has_z)
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      sprintf (buf_z, "%.*f", precision, z);
		      gaiaOutClean (buf_z);
		      if (version == 3)
			{
			    strcat (buf, buf_x);
			    strcat (buf, " ");
			    strcat (buf, buf_y);
			    strcat (buf, " ");
			    strcat (buf, buf_z);
			}
		      else
			{
			    strcat (buf, buf_x);
			    strcat (buf, ",");
			    strcat (buf, buf_y);
			    strcat (buf, ",");
			    strcat (buf, buf_z);
			}
		  }
		else
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      if (version == 3)
			{
			    strcat (buf, buf_x);
			    strcat (buf, " ");
			    strcat (buf, buf_y);
			}
		      else
			{
			    strcat (buf, buf_x);
			    strcat (buf, ",");
			    strcat (buf, buf_y);
			}
		  }
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  if (is_multi)
	    {
		if (version == 3)
		  {
		      strcpy (buf, "</gml:posList>");
		      strcat (buf, "</gml:LineStringSegment>");
		      strcat (buf, "</gml:segments>");
		      strcat (buf, "</gml:Curve>");
		      if (is_coll)
			  strcat (buf, "</gml:geometryMember>");
		      else
			  strcat (buf, "</gml:curveMember>");
		  }
		else
		  {
		      strcpy (buf, "</gml:coordinates>");
		      strcat (buf, "</gml:LineString>");
		      if (is_coll)
			  strcat (buf, "</gml:geometryMember>");
		      else
			  strcat (buf, "</gml:lineStringMember>");
		  }
	    }
	  else
	    {
		if (version == 3)
		  {
		      strcpy (buf, "</gml:posList>");
		      strcat (buf, "</gml:LineStringSegment>");
		      strcat (buf, "</gml:segments>");
		      strcat (buf, "</gml:Curve>");
		  }
		else
		  {
		      strcpy (buf, "</gml:coordinates>");
		      strcat (buf, "</gml:LineString>");
		  }
	    }
	  gaiaAppendToOutBuffer (out_buf, buf);
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* processing POLYGON */
	  ring = polyg->Exterior;
	  if (is_multi)
	    {
		if (version == 3)
		  {
		      if (is_coll)
			  strcpy (buf, "<gml:geometryMember>");
		      else
			  strcpy (buf, "<gml:surfaceMember>");
		      strcat (buf, "<gml:Polygon>");
		      strcat (buf, "<gml:exterior>");
		      strcat (buf, "<gml:LinearRing>");
		      if (ring->DimensionModel == GAIA_XY_Z
			  || ring->DimensionModel == GAIA_XY_Z_M)
			  strcat (buf, "<gml:posList srsDimension=\"3\">");
		      else
			  strcat (buf, "<gml:posList srsDimension=\"2\">");
		  }
		else
		  {
		      if (is_coll)
			  strcpy (buf, "<gml:geometryMember>");
		      else
			  strcpy (buf, "<gml:polygonMember>");
		      strcat (buf, "<gml:Polygon>");
		      strcat (buf, "<gml:outerBoundaryIs>");
		      strcat (buf, "<gml:LinearRing>");
		      strcat (buf, "<gml:coordinates>");
		  }
	    }
	  else
	    {
		if (geom->Srid == -1)
		    strcpy (buf, "<gml:Polygon>");
		else
		    sprintf (buf, "<gml:Polygon srsName=\"EPSG:%d\">",
			     geom->Srid);
		if (version == 3)
		  {
		      strcat (buf, "<gml:exterior>");
		      strcat (buf, "<gml:LinearRing>");
		      if (ring->DimensionModel == GAIA_XY_Z
			  || ring->DimensionModel == GAIA_XY_Z_M)
			  strcat (buf, "<gml:posList srsDimension=\"3\">");
		      else
			  strcat (buf, "<gml:posList srsDimension=\"2\">");
		  }
		else
		  {
		      strcat (buf, "<gml:outerBoundaryIs>");
		      strcat (buf, "<gml:LinearRing>");
		      strcat (buf, "<gml:coordinates>");
		  }
	    }
	  gaiaAppendToOutBuffer (out_buf, buf);
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* exporting vertices [Interior Ring] */
		has_z = 0;
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      has_z = 1;
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      has_z = 1;
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		if (iv == 0)
		    *buf = '\0';
		else
		    strcpy (buf, " ");
		if (has_z)
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      sprintf (buf_z, "%.*f", precision, z);
		      gaiaOutClean (buf_z);
		      if (version == 3)
			{
			    strcat (buf, buf_x);
			    strcat (buf, " ");
			    strcat (buf, buf_y);
			    strcat (buf, " ");
			    strcat (buf, buf_z);
			}
		      else
			{
			    strcat (buf, buf_x);
			    strcat (buf, ",");
			    strcat (buf, buf_y);
			    strcat (buf, ",");
			    strcat (buf, buf_z);
			}
		  }
		else
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      if (version == 3)
			{
			    strcat (buf, buf_x);
			    strcat (buf, " ");
			    strcat (buf, buf_y);
			}
		      else
			{
			    strcat (buf, buf_x);
			    strcat (buf, ",");
			    strcat (buf, buf_y);
			}
		  }
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  /* closing the Exterior Ring */
	  if (version == 3)
	    {
		strcpy (buf, "</gml:posList>");
		strcat (buf, "</gml:LinearRing>");
		strcat (buf, "</gml:exterior>");
	    }
	  else
	    {
		strcpy (buf, "</gml:coordinates>");
		strcat (buf, "</gml:LinearRing>");
		strcat (buf, "</gml:outerBoundaryIs>");
	    }
	  gaiaAppendToOutBuffer (out_buf, buf);
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		/* interior rings */
		ring = polyg->Interiors + ib;
		if (version == 3)
		  {
		      strcpy (buf, "<gml:interior>");
		      strcat (buf, "<gml:LinearRing>");
		      if (ring->DimensionModel == GAIA_XY_Z
			  || ring->DimensionModel == GAIA_XY_Z_M)
			  strcat (buf, "<gml:posList srsDimension=\"3\">");
		      else
			  strcat (buf, "<gml:posList srsDimension=\"2\">");
		  }
		else
		  {
		      strcpy (buf, "<gml:innerBoundaryIs>");
		      strcat (buf, "<gml:LinearRing>");
		      strcat (buf, "<gml:coordinates>");
		  }
		gaiaAppendToOutBuffer (out_buf, buf);
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      /* exporting vertices [Interior Ring] */
		      has_z = 0;
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    has_z = 1;
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    has_z = 1;
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			}
		      if (iv == 0)
			  *buf = '\0';
		      else
			  strcpy (buf, " ");
		      if (has_z)
			{
			    sprintf (buf_x, "%.*f", precision, x);
			    gaiaOutClean (buf_x);
			    sprintf (buf_y, "%.*f", precision, y);
			    gaiaOutClean (buf_y);
			    sprintf (buf_z, "%.*f", precision, z);
			    gaiaOutClean (buf_z);
			    if (version == 3)
			      {
				  strcat (buf, buf_x);
				  strcat (buf, " ");
				  strcat (buf, buf_y);
				  strcat (buf, " ");
				  strcat (buf, buf_z);
			      }
			    else
			      {
				  strcat (buf, buf_x);
				  strcat (buf, ",");
				  strcat (buf, buf_y);
				  strcat (buf, ",");
				  strcat (buf, buf_z);
			      }
			}
		      else
			{
			    sprintf (buf_x, "%.*f", precision, x);
			    gaiaOutClean (buf_x);
			    sprintf (buf_y, "%.*f", precision, y);
			    gaiaOutClean (buf_y);
			    if (version == 3)
			      {
				  strcat (buf, buf_x);
				  strcat (buf, " ");
				  strcat (buf, buf_y);
			      }
			    else
			      {
				  strcat (buf, buf_x);
				  strcat (buf, ",");
				  strcat (buf, buf_y);
			      }
			}
		      gaiaAppendToOutBuffer (out_buf, buf);
		  }
		/* closing the Interior Ring */
		if (version == 3)
		  {
		      strcpy (buf, "</gml:posList>");
		      strcat (buf, "</gml:LinearRing>");
		      strcat (buf, "</gml:interior>");
		  }
		else
		  {
		      strcpy (buf, "</gml:coordinates>");
		      strcat (buf, "</gml:LinearRing>");
		      strcat (buf, "</gml:innerBoundaryIs>");
		  }
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  /* closing the Polygon */
	  if (is_multi)
	    {
		if (version == 3)
		  {
		      strcpy (buf, "</gml:Polygon>");
		      if (is_coll)
			  strcat (buf, "</gml:geometryMember>");
		      else
			  strcat (buf, "</gml:surfaceMember>");
		  }
		else
		  {
		      strcpy (buf, "</gml:Polygon>");
		      if (is_coll)
			  strcat (buf, "</gml:geometryMember>");
		      else
			  strcat (buf, "</gml:polygonMember>");
		  }
	    }
	  else
	      strcpy (buf, "</gml:Polygon>");
	  gaiaAppendToOutBuffer (out_buf, buf);
	  polyg = polyg->Next;
      }
    switch (geom->DeclaredType)
      {
      case GAIA_POINT:
      case GAIA_LINESTRING:
      case GAIA_POLYGON:
	  *buf = '\0';
	  break;
      case GAIA_MULTIPOINT:
	  sprintf (buf, "</gml:MultiPoint>");
	  break;
      case GAIA_MULTILINESTRING:
	  if (version == 3)
	      sprintf (buf, "</gml:MultiCurve>");
	  else
	      sprintf (buf, "</gml:MultiLineString>");
	  break;
      case GAIA_MULTIPOLYGON:
	  if (version == 3)
	      sprintf (buf, "</gml:MultiSurface>");
	  else
	      sprintf (buf, "</gml:MultiPolygon>");
	  break;
      default:
	  sprintf (buf, "</gml:MultiGeometry>");
	  break;
      };
    gaiaAppendToOutBuffer (out_buf, buf);
}

GAIAGEO_DECLARE void
gaiaOutGeoJSON (gaiaOutBufferPtr out_buf, gaiaGeomCollPtr geom, int precision,
		int options)
{
/*
/ prints the GeoJSON representation of current geometry
/ *result* returns the encoded GeoJSON or NULL if any error is encountered
*/
    gaiaPointPtr point;
    gaiaLinestringPtr line;
    gaiaPolygonPtr polyg;
    gaiaRingPtr ring;
    int iv;
    int ib;
    double x;
    double y;
    double z;
    double m;
    int has_z;
    int is_multi = 0;
    int multi_count = 0;
    char bbox[1024];
    char crs[1024];
    char buf[2048];
    char buf_x[128];
    char buf_y[128];
    char buf_m[128];
    char buf_z[128];
    char endJson[16];
    if (!geom)
	return;
    if (precision > 18)
	precision = 18;

    if (options != 0)
      {
	  *bbox = '\0';
	  *crs = '\0';
	  if (geom->Srid != -1)
	    {
		if (options == 2 || options == 3)
		  {
		      /* including short CRS */
		      sprintf (crs,
			       ",\"crs\":{\"type\":\"name\",\"properties\":{\"name\":\"EPSG:%d\"}}",
			       geom->Srid);
		  }
		if (options == 4 || options == 5)
		  {
		      /* including long CRS */
		      sprintf (crs,
			       ",\"crs\":{\"type\":\"name\",\"properties\":{\"name\":\"urn:ogc:def:crs:EPSG:%d\"}}",
			       geom->Srid);
		  }
	    }
	  if (options == 1 || options == 3 || options == 5)
	    {
		/* including BBOX */
		gaiaMbrGeometry (geom);
		sprintf (buf_x, "%.*f", precision, geom->MinX);
		gaiaOutClean (buf_x);
		sprintf (buf_y, "%.*f", precision, geom->MinY);
		gaiaOutClean (buf_y);
		sprintf (buf_z, "%.*f", precision, geom->MaxX);
		gaiaOutClean (buf_z);
		sprintf (buf_m, "%.*f", precision, geom->MaxY);
		gaiaOutClean (buf_m);
		sprintf (bbox, ",\"bbox\":[%s,%s,%s,%s]", buf_x, buf_y, buf_z,
			 buf_m);
	    }
	  switch (geom->DeclaredType)
	    {
	    case GAIA_POINT:
		strcpy (buf, "{\"type\":\"Point\"");
		strcat (buf, crs);
		strcat (buf, bbox);
		strcat (buf, ",\"coordinates\":");
		strcpy (endJson, "}");
		break;
	    case GAIA_LINESTRING:
		strcpy (buf, "{\"type\":\"LineString\"");
		strcat (buf, crs);
		strcat (buf, bbox);
		strcat (buf, ",\"coordinates\":[");
		strcpy (endJson, "}");
		break;
	    case GAIA_POLYGON:
		strcpy (buf, "{\"type\":\"Polygon\"");
		strcat (buf, crs);
		strcat (buf, bbox);
		strcat (buf, ",\"coordinates\":[");
		strcpy (endJson, "}");
		break;
	    case GAIA_MULTIPOINT:
		strcpy (buf, "{\"type\":\"MultiPoint\"");
		strcat (buf, crs);
		strcat (buf, bbox);
		strcat (buf, ",\"coordinates\":[");
		strcpy (endJson, "]}");
		break;
	    case GAIA_MULTILINESTRING:
		strcpy (buf, "{\"type\":\"MultiLineString\"");
		strcat (buf, crs);
		strcat (buf, bbox);
		strcat (buf, ",\"coordinates\":[[");
		strcpy (endJson, "]}");
		break;
	    case GAIA_MULTIPOLYGON:
		strcpy (buf, "{\"type\":\"MultiPolygon\"");
		strcat (buf, crs);
		strcat (buf, bbox);
		strcat (buf, ",\"coordinates\":[[");
		strcpy (endJson, "]}");
		break;
	    default:
		strcpy (buf, "{\"type\":\"GeometryCollection\"");
		strcat (buf, crs);
		strcat (buf, bbox);
		strcat (buf, ",\"geometries\":[");
		strcpy (endJson, "]}");
		is_multi = 1;
		break;
	    };
      }
    else
      {
	  /* omitting BBOX */
	  switch (geom->DeclaredType)
	    {
	    case GAIA_POINT:
		strcpy (buf, "{\"type\":\"Point\",\"coordinates\":");
		strcpy (endJson, "}");
		break;
	    case GAIA_LINESTRING:
		strcpy (buf, "{\"type\":\"LineString\",\"coordinates\":[");
		strcpy (endJson, "}");
		break;
	    case GAIA_POLYGON:
		strcpy (buf, "{\"type\":\"Polygon\",\"coordinates\":[");
		strcpy (endJson, "}");
		break;
	    case GAIA_MULTIPOINT:
		strcpy (buf, "{\"type\":\"MultiPoint\",\"coordinates\":[");
		strcpy (endJson, "]}");
		break;
	    case GAIA_MULTILINESTRING:
		strcpy (buf,
			"{\"type\":\"MultiLineString\",\"coordinates\":[[");
		strcpy (endJson, "]}");
		break;
	    case GAIA_MULTIPOLYGON:
		strcpy (buf, "{\"type\":\"MultiPolygon\",\"coordinates\":[[");
		strcpy (endJson, "]}");
		break;
	    default:
		strcpy (buf,
			"{\"type\":\"GeometryCollection\",\"geometries\":[");
		strcpy (endJson, "]}");
		is_multi = 1;
		break;
	    };
      }
    gaiaAppendToOutBuffer (out_buf, buf);
    point = geom->FirstPoint;
    while (point)
      {
	  /* processing POINT */
	  if (is_multi)
	    {
		if (multi_count > 0)
		    strcpy (buf, ",{\"type\":\"Point\",\"coordinates\":");
		else
		    strcpy (buf, "{\"type\":\"Point\",\"coordinates\":");
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  else if (point != geom->FirstPoint)
	    {
		/* adding a further Point */
		strcpy (buf, ",");
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  sprintf (buf_x, "%.*f", precision, point->X);
	  gaiaOutClean (buf_x);
	  sprintf (buf_y, "%.*f", precision, point->Y);
	  gaiaOutClean (buf_y);
	  has_z = 0;
	  if (point->DimensionModel == GAIA_XY_Z
	      || point->DimensionModel == GAIA_XY_Z_M)
	    {
		sprintf (buf_z, "%.*f", precision, point->Z);
		gaiaOutClean (buf_z);
		has_z = 1;
	    }
	  if (has_z)
	      sprintf (buf, "[%s,%s,%s]", buf_x, buf_y, buf_z);
	  else
	      sprintf (buf, "[%s,%s]", buf_x, buf_y);
	  gaiaAppendToOutBuffer (out_buf, buf);
	  if (is_multi)
	    {
		strcpy (buf, "}");
		gaiaAppendToOutBuffer (out_buf, buf);
		multi_count++;
	    }
	  point = point->Next;
      }
    line = geom->FirstLinestring;
    while (line)
      {
	  /* processing LINESTRING */
	  if (is_multi)
	    {
		if (multi_count > 0)
		    strcpy (buf, ",{\"type\":\"LineString\",\"coordinates\":[");
		else
		    strcpy (buf, "{\"type\":\"LineString\",\"coordinates\":[");
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  else if (line != geom->FirstLinestring)
	    {
		/* opening a further LineString */
		strcpy (buf, ",[");
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  for (iv = 0; iv < line->Points; iv++)
	    {
		/* exporting vertices */
		has_z = 0;
		if (line->DimensionModel == GAIA_XY_Z)
		  {
		      has_z = 1;
		      gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
		  }
		else if (line->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
		  }
		else if (line->DimensionModel == GAIA_XY_Z_M)
		  {
		      has_z = 1;
		      gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (line->Coords, iv, &x, &y);
		  }
		if (has_z)
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      sprintf (buf_z, "%.*f", precision, z);
		      gaiaOutClean (buf_z);
		      if (iv == 0)
			  sprintf (buf, "[%s,%s,%s]", buf_x, buf_y, buf_z);
		      else
			  sprintf (buf, ",[%s,%s,%s]", buf_x, buf_y, buf_z);
		  }
		else
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      if (iv == 0)
			  sprintf (buf, "[%s,%s]", buf_x, buf_y);
		      else
			  sprintf (buf, ",[%s,%s]", buf_x, buf_y);
		  }
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  /* closing the LineString */
	  strcpy (buf, "]");
	  gaiaAppendToOutBuffer (out_buf, buf);
	  if (is_multi)
	    {
		strcpy (buf, "}");
		gaiaAppendToOutBuffer (out_buf, buf);
		multi_count++;
	    }
	  line = line->Next;
      }
    polyg = geom->FirstPolygon;
    while (polyg)
      {
	  /* processing POLYGON */
	  if (is_multi)
	    {
		if (multi_count > 0)
		    strcpy (buf, ",{\"type\":\"Polygon\",\"coordinates\":[");
		else
		    strcpy (buf, "{\"type\":\"Polygon\",\"coordinates\":[");
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  else if (polyg != geom->FirstPolygon)
	    {
		/* opening a further Polygon */
		strcpy (buf, ",[");
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  ring = polyg->Exterior;
	  for (iv = 0; iv < ring->Points; iv++)
	    {
		/* exporting vertices [Interior Ring] */
		has_z = 0;
		if (ring->DimensionModel == GAIA_XY_Z)
		  {
		      has_z = 1;
		      gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
		  }
		else if (ring->DimensionModel == GAIA_XY_M)
		  {
		      gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
		  }
		else if (ring->DimensionModel == GAIA_XY_Z_M)
		  {
		      has_z = 1;
		      gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
		  }
		else
		  {
		      gaiaGetPoint (ring->Coords, iv, &x, &y);
		  }
		if (has_z)
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      sprintf (buf_z, "%.*f", precision, z);
		      gaiaOutClean (buf_z);
		      if (iv == 0)
			  sprintf (buf, "[[%s,%s,%s]", buf_x, buf_y, buf_z);
		      else
			  sprintf (buf, ",[%s,%s,%s]", buf_x, buf_y, buf_z);
		  }
		else
		  {
		      sprintf (buf_x, "%.*f", precision, x);
		      gaiaOutClean (buf_x);
		      sprintf (buf_y, "%.*f", precision, y);
		      gaiaOutClean (buf_y);
		      if (iv == 0)
			  sprintf (buf, "[[%s,%s]", buf_x, buf_y);
		      else
			  sprintf (buf, ",[%s,%s]", buf_x, buf_y);
		  }
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  /* closing the Exterior Ring */
	  strcpy (buf, "]");
	  gaiaAppendToOutBuffer (out_buf, buf);
	  for (ib = 0; ib < polyg->NumInteriors; ib++)
	    {
		/* interior rings */
		ring = polyg->Interiors + ib;
		for (iv = 0; iv < ring->Points; iv++)
		  {
		      /* exporting vertices [Interior Ring] */
		      has_z = 0;
		      if (ring->DimensionModel == GAIA_XY_Z)
			{
			    has_z = 1;
			    gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
			}
		      else if (ring->DimensionModel == GAIA_XY_M)
			{
			    gaiaGetPointXYM (ring->Coords, iv, &x, &y, &m);
			}
		      else if (ring->DimensionModel == GAIA_XY_Z_M)
			{
			    has_z = 1;
			    gaiaGetPointXYZM (ring->Coords, iv, &x, &y, &z, &m);
			}
		      else
			{
			    gaiaGetPoint (ring->Coords, iv, &x, &y);
			}
		      if (has_z)
			{
			    sprintf (buf_x, "%.*f", precision, x);
			    gaiaOutClean (buf_x);
			    sprintf (buf_y, "%.*f", precision, y);
			    gaiaOutClean (buf_y);
			    sprintf (buf_z, "%.*f", precision, z);
			    gaiaOutClean (buf_z);
			    if (iv == 0)
				sprintf (buf, ",[[%s,%s,%s]", buf_x, buf_y,
					 buf_z);
			    else
				sprintf (buf, ",[%s,%s,%s]", buf_x, buf_y,
					 buf_z);
			}
		      else
			{
			    sprintf (buf_x, "%.*f", precision, x);
			    gaiaOutClean (buf_x);
			    sprintf (buf_y, "%.*f", precision, y);
			    gaiaOutClean (buf_y);
			    if (iv == 0)
				sprintf (buf, ",[[%s,%s]", buf_x, buf_y);
			    else
				sprintf (buf, ",[%s,%s]", buf_x, buf_y);
			}
		      gaiaAppendToOutBuffer (out_buf, buf);
		  }
		/* closing the Interior Ring */
		strcpy (buf, "]");
		gaiaAppendToOutBuffer (out_buf, buf);
	    }
	  /* closing the Polygon */
	  strcpy (buf, "]");
	  gaiaAppendToOutBuffer (out_buf, buf);
	  if (is_multi)
	    {
		strcpy (buf, "}");
		gaiaAppendToOutBuffer (out_buf, buf);
		multi_count++;
	    }
	  polyg = polyg->Next;
      }
    strcpy (buf, endJson);
    gaiaAppendToOutBuffer (out_buf, buf);
}
/**************** End file: gg_wkt.c **********/


/**************** Begin file: srs_init.c **********/

/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

#ifdef _WIN32
#define strcasecmp	_stricmp
#endif /* not WIN32 */

/* #include <spatialite.h> */

struct epsg_defs
{
    int srid;
    char *auth_name;
    int auth_srid;
    char *ref_sys_name;
    char *proj4text;
    char *srs_wkt;
    struct epsg_defs *next;
};

static void
free_epsg_def (struct epsg_defs *ptr)
{
/* memory cleanup - destroying an EPSG def item */
    if (ptr->auth_name)
	free (ptr->auth_name);
    if (ptr->ref_sys_name)
	free (ptr->ref_sys_name);
    if (ptr->proj4text)
	free (ptr->proj4text);
    if (ptr->srs_wkt)
	free (ptr->srs_wkt);
    free (ptr);
}

static struct epsg_defs *
add_epsg_def (struct epsg_defs **first, struct epsg_defs **last, int srid,
	      const char *auth_name, int auth_srid, const char *ref_sys_name)
{
/* appending an EPSG def to the list */
    int len;
    struct epsg_defs *p = malloc (sizeof (struct epsg_defs));
    if (!p)
	return NULL;
    p->srid = srid;
    p->auth_name = NULL;
    p->auth_srid = auth_srid;
    p->ref_sys_name = NULL;
    p->proj4text = NULL;
    p->srs_wkt = NULL;
    p->next = NULL;
    if (auth_name)
      {
	  len = strlen (auth_name);
	  if (len > 0)
	    {
		p->auth_name = malloc (len + 1);
		if (p->auth_name == NULL)
		    goto error;
		strcpy (p->auth_name, auth_name);
	    }
      }
    if (ref_sys_name)
      {
	  len = strlen (ref_sys_name);
	  if (len > 0)
	    {
		p->ref_sys_name = malloc (len + 1);
		if (p->ref_sys_name == NULL)
		    goto error;
		strcpy (p->ref_sys_name, ref_sys_name);
	    }
      }
    if (*first == NULL)
	*first = p;
    if (*last != NULL)
	(*last)->next = p;
    *last = p;
    return p;
  error:
    free_epsg_def (p);
    return NULL;
}

static void
add_proj4text (struct epsg_defs *p, int count, const char *text)
{
/* creating the PROJ4TEXT string */
    int len;
    int olen;
    char *string;
    if (text == NULL)
	return;
    len = strlen (text);
    if (!count)
      {
	  p->proj4text = malloc (len + 1);
	  if (p->proj4text == NULL)
	      return;
	  strcpy (p->proj4text, text);
	  return;
      }
    if (p->proj4text == NULL)
	return;
    olen = strlen (p->proj4text);
    string = malloc (len + olen + 1);
    if (string == NULL)
	return;
    strcpy (string, p->proj4text);
    free (p->proj4text);
    p->proj4text = string;
    strcat (p->proj4text, text);
}

static void
add_srs_wkt (struct epsg_defs *p, int count, const char *text)
{
/* creating the SRS_WKT string */
    int len;
    int olen;
    char *string;
    if (text == NULL)
	return;
    len = strlen (text);
    if (!count)
      {
	  p->srs_wkt = malloc (len + 1);
	  if (p->srs_wkt == NULL)
	      return;
	  strcpy (p->srs_wkt, text);
	  return;
      }
    if (p->srs_wkt == NULL)
	return;
    olen = strlen (p->srs_wkt);
    string = malloc (len + olen + 1);
    if (string == NULL)
	return;
    strcpy (string, p->srs_wkt);
    free (p->srs_wkt);
    p->srs_wkt = string;
    strcat (p->srs_wkt, text);
}

static void
initialize_epsg_00 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2000, "epsg", 2000,
		      "Anguilla 1957 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1, "_0=400000 +y_0=0 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Anguilla 1957 / British West Indies Grid\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Anguilla 1957\",DATUM[\"Anguilla_1957\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7012\"]],AUTHORITY[\"EPSG\",\"6600\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4600\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-62],PARAMETER[\"scale_factor\",0.9995],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",400000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"2000\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2001, "epsg", 2001,
		      "Antigua 1943 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1, "_0=400000 +y_0=0 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Antigua 1943 / British West Indies Grid\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Antigua 1943\",DATUM[\"Antigua_1943\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7012\"]],AUTHORITY[\"EPSG\",\"6601\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4601\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-62],PARAMETER[\"scale_factor\",0.9995],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",400000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"2001\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2002, "epsg", 2002,
		      "Dominica 1945 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1,
		   "_0=400000 +y_0=0 +ellps=clrk80 +towgs84=725,685,536,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Dominica 1945 / British West Indies Grid\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Dominica 1945\",DATUM[\"Dominica_1945\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7012\"]],TOWGS84[725,685,536,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"6602\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4602\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",0],PARAMETER[\"central_meridian\",-62],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.9995],PARAMETER[\"false_easting\",400000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2002");
    add_srs_wkt (p, 12, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2003, "epsg", 2003,
		      "Grenada 1953 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1,
		   "_0=400000 +y_0=0 +ellps=clrk80 +towgs84=72,213.7,93,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Grenada 1953 / British West Indies Grid\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Grenada 1953\",DATUM[\"Grenada_1953\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7012\"]],TOWGS84[72,213.7,93,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6603\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4603\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-62],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9995],PARAMETER[\"false_easting\",400000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2003\"");
    add_srs_wkt (p, 12, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2004, "epsg", 2004,
		      "Montserrat 1958 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1,
		   "_0=400000 +y_0=0 +ellps=clrk80 +towgs84=174,359,365,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Montserrat 1958 / British West Indies Grid\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"Montserrat 1958\",DATUM[\"Montserrat_1958\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7012\"]],TOWGS84[174,359,365,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6604\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4604\"");
    add_srs_wkt (p, 7,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",0],PARAMETER[\"central_meridian\",-62],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"scale_factor\",0.9995],PARAMETER[\"false_easting\",400");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12,
		 "\"2004\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2005, "epsg", 2005,
		      "St. Kitts 1955 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1, "_0=400000 +y_0=0 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"St. Kitts 1955 / British West Indies Grid\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"St. Kitts 1955\",DATUM[\"St_Kitts_1955\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6605\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4605\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-62],PARAMETER[\"scale_factor\",0.9995],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",400000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2005\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2006, "epsg", 2006,
		      "St. Lucia 1955 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1,
		   "_0=400000 +y_0=0 +ellps=clrk80 +towgs84=-149,128,296,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"St. Lucia 1955 / British West Indies Grid\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"St. Lucia 1955\",DATUM[\"St_Lucia_1955\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7012\"]],TOWGS84[-149,128,296,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6606\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4606\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-62],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.9995],PARAMETER[\"false_easting\",40000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "2006\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 2007, "epsg", 2007,
		      "St. Vincent 45 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1,
		   "_0=400000 +y_0=0 +ellps=clrk80 +towgs84=195.671,332.517,");
    add_proj4text (p, 2, "274.607,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"St. Vincent 45 / British West Indies Grid\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"St. Vincent 1945\",DATUM[\"St_Vincent_1945\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7012\"]],TOWGS84[195.671,332.517,274.607,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6607\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4607\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-6");
    add_srs_wkt (p, 10,
		 "2],PARAMETER[\"scale_factor\",0.9995],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",400000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"2007\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 2008, "epsg", 2008,
		      "NAD27(CGQ77) / SCoPQ zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-55.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 2\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-55.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2008\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2009, "epsg", 2009,
		      "NAD27(CGQ77) / SCoPQ zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-58.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 3\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-58.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2009\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2010, "epsg", 2010,
		      "NAD27(CGQ77) / SCoPQ zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-61.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 4\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-61.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2010\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2011, "epsg", 2011,
		      "NAD27(CGQ77) / SCoPQ zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-64.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 5\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-64.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2011\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2012, "epsg", 2012,
		      "NAD27(CGQ77) / SCoPQ zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-67.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 6\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-67.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2012\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2013, "epsg", 2013,
		      "NAD27(CGQ77) / SCoPQ zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-70.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 7\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-70.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2013\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2014, "epsg", 2014,
		      "NAD27(CGQ77) / SCoPQ zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-73.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 8\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-73.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2014\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2015, "epsg", 2015,
		      "NAD27(CGQ77) / SCoPQ zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-76.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 9\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-76.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2015\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2016, "epsg", 2016,
		      "NAD27(CGQ77) / SCoPQ zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-79.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / SCoPQ zone 10\",GEOGCS[\"NAD27(C");
    add_srs_wkt (p, 1,
		 "GQ77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-79.5],PARAMETER[\"scale_factor\",0.9999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",304800],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2016\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2017, "epsg", 2017,
		      "NAD27(76) / MTM zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-73.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 8\",GEOGCS[\"NAD27(76)\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clark");
    add_srs_wkt (p, 2,
		 "e 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-73.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2017\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 2018, "epsg", 2018,
		      "NAD27(76) / MTM zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-76.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 9\",GEOGCS[\"NAD27(76)\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clark");
    add_srs_wkt (p, 2,
		 "e 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-76.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2018\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 2019, "epsg", 2019,
		      "NAD27(76) / MTM zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-79.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 10\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-79.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",304800],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"2019\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2020, "epsg", 2020,
		      "NAD27(76) / MTM zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-82.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 11\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-82.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",304800],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"2020\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2021, "epsg", 2021,
		      "NAD27(76) / MTM zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-81 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 12\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-81],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2021\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2022, "epsg", 2022,
		      "NAD27(76) / MTM zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-84 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 13\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-84],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2022\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2023, "epsg", 2023,
		      "NAD27(76) / MTM zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-87 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 14\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-87],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2023\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2024, "epsg", 2024,
		      "NAD27(76) / MTM zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-90 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 15\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-90],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2024\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2025, "epsg", 2025,
		      "NAD27(76) / MTM zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-93 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 16\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-93],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2025\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2026, "epsg", 2026,
		      "NAD27(76) / MTM zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-96 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / MTM zone 17\",GEOGCS[\"NAD27(76)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-96],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",304800],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2026\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2027, "epsg", 2027,
		      "NAD27(76) / UTM zone 15N");
    add_proj4text (p, 0, "+proj=utm +zone=15 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / UTM zone 15N\",GEOGCS[\"NAD27(76)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-93],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2027\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2028, "epsg", 2028,
		      "NAD27(76) / UTM zone 16N");
    add_proj4text (p, 0, "+proj=utm +zone=16 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / UTM zone 16N\",GEOGCS[\"NAD27(76)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-87],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2028\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2029, "epsg", 2029,
		      "NAD27(76) / UTM zone 17N");
    add_proj4text (p, 0, "+proj=utm +zone=17 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / UTM zone 17N\",GEOGCS[\"NAD27(76)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-81],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2029\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2030, "epsg", 2030,
		      "NAD27(76) / UTM zone 18N");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(76) / UTM zone 18N\",GEOGCS[\"NAD27(76)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927_1976\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6608\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4608\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-75],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2030\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2031, "epsg", 2031,
		      "NAD27(CGQ77) / UTM zone 17N");
    add_proj4text (p, 0, "+proj=utm +zone=17 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / UTM zone 17N\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-81],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"2031\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2032, "epsg", 2032,
		      "NAD27(CGQ77) / UTM zone 18N");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / UTM zone 18N\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-75],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"2032\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2033, "epsg", 2033,
		      "NAD27(CGQ77) / UTM zone 19N");
    add_proj4text (p, 0, "+proj=utm +zone=19 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / UTM zone 19N\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-69],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"2033\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2034, "epsg", 2034,
		      "NAD27(CGQ77) / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / UTM zone 20N\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-63],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"2034\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2035, "epsg", 2035,
		      "NAD27(CGQ77) / UTM zone 21N");
    add_proj4text (p, 0, "+proj=utm +zone=21 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / UTM zone 21N\",GEOGCS[\"NAD27(CG");
    add_srs_wkt (p, 1,
		 "Q77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-57],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"2035\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2036, "epsg", 2036,
		      "NAD83(CSRS98) / New Brunswick Stereo (deprecated)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=46.5 +lon_0=-66.5 +k=0.999912 +x_0=2");
    add_proj4text (p, 1,
		   "500000 +y_0=7500000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / New Brunswick Stereo (deprecate");
    add_srs_wkt (p, 1,
		 "d)\",GEOGCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spa");
    add_srs_wkt (p, 2,
		 "tial_Reference_System\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "74532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Oblique_Stereographic\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"latitude_of_origin\",46.5],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",-66.5],PARAMETER[\"scale_factor\",0.999912],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",2500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",7500000],AUTHORITY[\"EPSG\",\"2036\"],AXIS[\"Northing");
    add_srs_wkt (p, 13, "\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2037, "epsg", 2037,
		      "NAD83(CSRS98) / UTM zone 19N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 19N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-69],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2037\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2038, "epsg", 2038,
		      "NAD83(CSRS98) / UTM zone 20N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 20N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-63],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2038\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2039, "epsg", 2039,
		      "Israel / Israeli TM Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31.73439361111111 +lon_0=35.204516944");
    add_proj4text (p, 1,
		   "44445 +k=1.0000067 +x_0=219529.584 +y_0=626907.39 +ellps");
    add_proj4text (p, 2, "=GRS80 +towgs84=-48,55,52,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Israel / Israeli TM Grid\",GEOGCS[\"Israel\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Israel\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 2,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[-48,55,52,0,0,0");
    add_srs_wkt (p, 3,
		 ",0],AUTHORITY[\"EPSG\",\"6141\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 5,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"4141\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 7,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 8,
		 "titude_of_origin\",31.73439361111111],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",35.20451694444445],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",1.0000067],PARAMETER[\"false_easting\",219529.584],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",626907.39],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"2039\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 2040, "epsg", 2040,
		      "Locodjo 1965 / UTM zone 30N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +ellps=clrk80 +towgs84=-125,53,467,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Locodjo 1965 / UTM zone 30N\",GEOGCS[\"Locodjo ");
    add_srs_wkt (p, 1,
		 "1965\",DATUM[\"Locodjo_1965\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-125,53,467,0,0,0,0],AUTHORITY[\"EPSG\",\"6142\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4142\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-3],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2040\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2041, "epsg", 2041,
		      "Abidjan 1987 / UTM zone 30N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +ellps=clrk80 +towgs84=-124.76,53,466");
    add_proj4text (p, 1, ".79,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Abidjan 1987 / UTM zone 30N\",GEOGCS[\"Abidjan ");
    add_srs_wkt (p, 1,
		 "1987\",DATUM[\"Abidjan_1987\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-124.76,53,466.79,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "143\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4143\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 8,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",-3],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2041\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2042, "epsg", 2042,
		      "Locodjo 1965 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=clrk80 +towgs84=-125,53,467,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Locodjo 1965 / UTM zone 29N\",GEOGCS[\"Locodjo ");
    add_srs_wkt (p, 1,
		 "1965\",DATUM[\"Locodjo_1965\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-125,53,467,0,0,0,0],AUTHORITY[\"EPSG\",\"6142\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4142\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-9],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2042\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2043, "epsg", 2043,
		      "Abidjan 1987 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=clrk80 +towgs84=-124.76,53,466");
    add_proj4text (p, 1, ".79,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Abidjan 1987 / UTM zone 29N\",GEOGCS[\"Abidjan ");
    add_srs_wkt (p, 1,
		 "1987\",DATUM[\"Abidjan_1987\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-124.76,53,466.79,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "143\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4143\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 8,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",-9],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2043\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2044, "epsg", 2044,
		      "Hanoi 1972 / Gauss-Kruger zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=18500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=-17.51,-108.32,-62.39,0,0,0,0 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hanoi 1972 / Gauss-Kruger zone 18\",GEOGCS[\"Ha");
    add_srs_wkt (p, 1,
		 "noi 1972\",DATUM[\"Hanoi_1972\",SPHEROID[\"Krassowsky 19");
    add_srs_wkt (p, 2,
		 "40\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[-17.51,-108.32,-62.39,0,0,0,0],AUTHORITY[\"EPSG\",\"614");
    add_srs_wkt (p, 4,
		 "7\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4147\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",105],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "1],PARAMETER[\"false_easting\",18500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"2044\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2045, "epsg", 2045,
		      "Hanoi 1972 / Gauss-Kruger zone 19");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=19500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=-17.51,-108.32,-62.39,0,0,0,0 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hanoi 1972 / Gauss-Kruger zone 19\",GEOGCS[\"Ha");
    add_srs_wkt (p, 1,
		 "noi 1972\",DATUM[\"Hanoi_1972\",SPHEROID[\"Krassowsky 19");
    add_srs_wkt (p, 2,
		 "40\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[-17.51,-108.32,-62.39,0,0,0,0],AUTHORITY[\"EPSG\",\"614");
    add_srs_wkt (p, 4,
		 "7\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4147\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",111],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "1],PARAMETER[\"false_easting\",19500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"2045\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2056, "epsg", 2056, "CH1903+ / LV95");
    add_proj4text (p, 0,
		   "+proj=somerc +lat_0=46.95240555555556 +lon_0=7.439583333");
    add_proj4text (p, 1,
		   "333333 +k_0=1 +x_0=2600000 +y_0=1200000 +ellps=bessel +t");
    add_proj4text (p, 2,
		   "owgs84=674.374,15.056,405.346,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"CH1903+ / LV95\",GEOGCS[\"CH1903+\",DATUM[\"CH1");
    add_srs_wkt (p, 1,
		 "903\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7004\"]],TOWGS84[674.374,15.056,405.");
    add_srs_wkt (p, 3,
		 "346,0,0,0,0],AUTHORITY[\"EPSG\",\"6150\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4150\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"latitude_of_center\",46.95240555555556],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"longitude_of_center\",7.439583333333333],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"azimuth\",90],PARAMETER[\"rectified_grid_angle\",90]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",2600000],PARAMETER[\"false_northing\",1200000],AUTHOR");
    add_srs_wkt (p, 13,
		 "ITY[\"EPSG\",\"2056\"],AXIS[\"Y\",EAST],AXIS[\"X\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 2057, "epsg", 2057,
		      "Rassadiran / Nakhl e Taqi");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=27.51882880555555 +lonc=52.6035391666");
    add_proj4text (p, 1,
		   "6667 +alpha=0.5716611944444444 +k=0.999895934 +x_0=65837");
    add_proj4text (p, 2,
		   "7.437 +y_0=3044969.194 +ellps=intl +towgs84=-133.63,-157");
    add_proj4text (p, 3, ".5,-158.62,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Rassadiran / Nakhl e Taqi\",GEOGCS[\"Rassadiran");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Rassadiran\",SPHEROID[\"International 1924\",");
    add_srs_wkt (p, 2,
		 "6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-133.6");
    add_srs_wkt (p, 3,
		 "3,-157.5,-158.62,0,0,0,0],AUTHORITY[\"EPSG\",\"6153\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4153\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Hotine_Oblique_");
    add_srs_wkt (p, 8,
		 "Mercator\"],PARAMETER[\"latitude_of_center\",27.51882880");
    add_srs_wkt (p, 9,
		 "555555],PARAMETER[\"longitude_of_center\",52.60353916666");
    add_srs_wkt (p, 10,
		 "667],PARAMETER[\"azimuth\",0.5716611944444444],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"rectified_grid_angle\",0.5716611944444444],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"scale_factor\",0.999895934],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 13,
		 ",658377.437],PARAMETER[\"false_northing\",3044969.194],A");
    add_srs_wkt (p, 14,
		 "UTHORITY[\"EPSG\",\"2057\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 15, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2058, "epsg", 2058,
		      "ED50(ED77) / UTM zone 38N");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50(ED77) / UTM zone 38N\",GEOGCS[\"ED50(ED77)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Datum_1950_1977\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6154\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4154\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",45],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"2058\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 2059, "epsg", 2059,
		      "ED50(ED77) / UTM zone 39N");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50(ED77) / UTM zone 39N\",GEOGCS[\"ED50(ED77)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Datum_1950_1977\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6154\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4154\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",51],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"2059\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 2060, "epsg", 2060,
		      "ED50(ED77) / UTM zone 40N");
    add_proj4text (p, 0, "+proj=utm +zone=40 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50(ED77) / UTM zone 40N\",GEOGCS[\"ED50(ED77)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Datum_1950_1977\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6154\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4154\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",57],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"2060\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 2061, "epsg", 2061,
		      "ED50(ED77) / UTM zone 41N");
    add_proj4text (p, 0, "+proj=utm +zone=41 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50(ED77) / UTM zone 41N\",GEOGCS[\"ED50(ED77)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Datum_1950_1977\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6154\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4154\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",63],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"2061\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 2062, "epsg", 2062,
		      "Madrid 1870 (Madrid) / Spain");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40 +lat_0=40 +lon_0=0 +k_0=0.9988085293");
    add_proj4text (p, 1,
		   " +x_0=600000 +y_0=600000 +a=6378298.3 +b=6356657.1426695");
    add_proj4text (p, 2, "61 +pm=madrid +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Madrid 1870 (Madrid) / Spain\",GEOGCS[\"Madrid ");
    add_srs_wkt (p, 1,
		 "1870 (Madrid)\",DATUM[\"Madrid_1870_Madrid\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Struve 1860\",6378298.3,294.73,AUTHORITY[\"EPSG\",\"7028");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6903\"]],PRIMEM[\"Madrid\",-3.");
    add_srs_wkt (p, 4,
		 "687938888888889,AUTHORITY[\"EPSG\",\"8905\"]],UNIT[\"deg");
    add_srs_wkt (p, 5,
		 "ree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"4903\"]],UNIT[\"metre\",1,AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Coni");
    add_srs_wkt (p, 8,
		 "c_1SP\"],PARAMETER[\"latitude_of_origin\",40],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",0],PARAMETER[\"scale_factor\",0.998");
    add_srs_wkt (p, 10,
		 "8085293],PARAMETER[\"false_easting\",600000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",600000],AUTHORITY[\"EPSG\",\"2062\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2063, "epsg", 2063,
		      "Dabola 1981 / UTM zone 28N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +a=6378249.2 +b=6356515 +towgs84=-23,");
    add_proj4text (p, 1, "259,-9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Dabola 1981 / UTM zone 28N (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Conakry 1905\",DATUM[\"Conakry_1905\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7011\"]],TOWGS84[-23,259,-9,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6315\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4315\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-15],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "2063\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 2064, "epsg", 2064,
		      "Dabola 1981 / UTM zone 29N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +a=6378249.2 +b=6356515 +towgs84=-23,");
    add_proj4text (p, 1, "259,-9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Dabola 1981 / UTM zone 29N (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Conakry 1905\",DATUM[\"Conakry_1905\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7011\"]],TOWGS84[-23,259,-9,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6315\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4315\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-9],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.9996],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12,
		 "064\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2065, "epsg", 2065,
		      "S-JTSK (Ferro) / Krovak");
    add_proj4text (p, 0,
		   "+proj=krovak +lat_0=49.5 +lon_0=42.5 +alpha=30.288139722");
    add_proj4text (p, 1,
		   "22222 +k=0.9999 +x_0=0 +y_0=0 +ellps=bessel +pm=ferro +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"S-JTSK (Ferro) / Krovak\",GEOGCS[\"S-JTSK (Ferr");
    add_srs_wkt (p, 1,
		 "o)\",DATUM[\"S_JTSK_Ferro\",SPHEROID[\"Bessel 1841\",637");
    add_srs_wkt (p, 2,
		 "7397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6818\"]],PRIMEM[\"Ferro\",-17.6666666666");
    add_srs_wkt (p, 4,
		 "6667,AUTHORITY[\"EPSG\",\"8909\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4818\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Krovak\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "center\",49.5],PARAMETER[\"longitude_of_center\",42.5],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"azimuth\",30.28813972222222],PARAMETER[\"pseu");
    add_srs_wkt (p, 10,
		 "do_standard_parallel_1\",78.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2065\"],AXIS[\"X\",");
    add_srs_wkt (p, 13, "SOUTH],AXIS[\"Y\",WEST]]");
    p = add_epsg_def (first, last, 2066, "epsg", 2066,
		      "Mount Dillon / Tobago Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=11.25217861111111 +lon_0=-60.686008888");
    add_proj4text (p, 1,
		   "88889 +x_0=37718.66159325 +y_0=36209.91512952 +a=6378293");
    add_proj4text (p, 2,
		   ".645208759 +b=6356617.987679838 +to_meter=0.201166195164");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mount Dillon / Tobago Grid\",GEOGCS[\"Mount Dil");
    add_srs_wkt (p, 1,
		 "lon\",DATUM[\"Mount_Dillon\",SPHEROID[\"Clarke 1858\",63");
    add_srs_wkt (p, 2,
		 "78293.645208759,294.2606763692569,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "007\"]],AUTHORITY[\"EPSG\",\"6157\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4157\"]],UNIT[\"Clarke's link\",0.201166195164,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9039\"]],PROJECTION[\"Cassini_Soldn");
    add_srs_wkt (p, 8,
		 "er\"],PARAMETER[\"latitude_of_origin\",11.25217861111111");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-60.68600888888889],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",187500],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",180000],AUTHORITY[\"EPSG\",\"2066\"],AXIS[\"Eastin");
    add_srs_wkt (p, 12, "g\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2067, "epsg", 2067,
		      "Naparima 1955 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Naparima 1955 / UTM zone 20N\",GEOGCS[\"Naparim");
    add_srs_wkt (p, 1,
		 "a 1955\",DATUM[\"Naparima_1955\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 2,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6158\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"415");
    add_srs_wkt (p, 6,
		 "8\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",-63],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11,
		 "\",\"2067\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NO");
    add_srs_wkt (p, 12, "RTH]]");
    p = add_epsg_def (first, last, 2068, "epsg", 2068, "ELD79 / Libya zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=0.9999 +x_0=200000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 5\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",9],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "cale_factor\",0.9999],PARAMETER[\"false_easting\",200000");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 11, "068\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2069, "epsg", 2069, "ELD79 / Libya zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=11 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 6\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",11],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2069\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2070, "epsg", 2070, "ELD79 / Libya zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 7\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",13],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2070\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2071, "epsg", 2071, "ELD79 / Libya zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 8\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",15],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2071\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2072, "epsg", 2072, "ELD79 / Libya zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=17 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 9\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",17],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2072\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2073, "epsg", 2073, "ELD79 / Libya zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=19 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 10\",GEOGCS[\"ELD79\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"European_Libyan_Datum_1979\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",0],PARAMETER[\"central_meridian\",19],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2073\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2074, "epsg", 2074, "ELD79 / Libya zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 11\",GEOGCS[\"ELD79\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"European_Libyan_Datum_1979\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2074\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2075, "epsg", 2075, "ELD79 / Libya zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=23 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 12\",GEOGCS[\"ELD79\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"European_Libyan_Datum_1979\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",0],PARAMETER[\"central_meridian\",23],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2075\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2076, "epsg", 2076, "ELD79 / Libya zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=25 +k=0.9999 +x_0=200000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / Libya zone 13\",GEOGCS[\"ELD79\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"European_Libyan_Datum_1979\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",0],PARAMETER[\"central_meridian\",25],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2076\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2077, "epsg", 2077, "ELD79 / UTM zone 32N");
    add_proj4text (p, 0, "+proj=utm +zone=32 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / UTM zone 32N\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",9],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "cale_factor\",0.9996],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 11,
		 "077\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2078, "epsg", 2078, "ELD79 / UTM zone 33N");
    add_proj4text (p, 0, "+proj=utm +zone=33 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / UTM zone 33N\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",15],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "2078\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2079, "epsg", 2079, "ELD79 / UTM zone 34N");
    add_proj4text (p, 0, "+proj=utm +zone=34 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / UTM zone 34N\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "2079\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2080, "epsg", 2080, "ELD79 / UTM zone 35N");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / UTM zone 35N\",GEOGCS[\"ELD79\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Libyan_Datum_1979\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "2080\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2081, "epsg", 2081,
		      "Chos Malal 1914 / Argentina zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-69 +k=1 +x_0=2500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Chos Malal 1914 / Argentina zone 2\",GEOGCS[\"C");
    add_srs_wkt (p, 1,
		 "hos Malal 1914\",DATUM[\"Chos_Malal_1914\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6160\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4160\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",-90],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-69],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",2500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2081\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2082, "epsg", 2082,
		      "Pampa del Castillo / Argentina zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-69 +k=1 +x_0=2500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=intl +towgs84=27.5,14,186.4,0,0,0,0 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pampa del Castillo / Argentina zone 2\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Pampa del Castillo\",DATUM[\"Pampa_del_Castillo\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7022\"]],TOWGS84[27.5,14,186.4,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6161\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4161\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",-90],PARAMETER[\"central_meridian\",-69],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",2500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2082");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2083, "epsg", 2083,
		      "Hito XVIII 1963 / Argentina zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-69 +k=1 +x_0=2500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hito XVIII 1963 / Argentina zone 2\",GEOGCS[\"H");
    add_srs_wkt (p, 1,
		 "ito XVIII 1963\",DATUM[\"Hito_XVIII_1963\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6254\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4254\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",-90],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-69],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",2500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2083\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2084, "epsg", 2084,
		      "Hito XVIII 1963 / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hito XVIII 1963 / UTM zone 19S\",GEOGCS[\"Hito ");
    add_srs_wkt (p, 1,
		 "XVIII 1963\",DATUM[\"Hito_XVIII_1963\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6254\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4254\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-69],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"2084\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2085, "epsg", 2085,
		      "NAD27 / Cuba Norte (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=22.35 +lat_0=22.35 +lon_0=-81 +k_0=0.99");
    add_proj4text (p, 1,
		   "993602 +x_0=500000 +y_0=280296.016 +ellps=clrk66 +datum=");
    add_proj4text (p, 2, "NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Cuba Norte (deprecated)\",GEOGCS[\"NAD2");
    add_srs_wkt (p, 1,
		 "7\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clark");
    add_srs_wkt (p, 2,
		 "e 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_origin\",22.35],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",-81],PARAMETER[\"scale_factor\",0.9999360");
    add_srs_wkt (p, 10,
		 "2],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",280296.016],AUTHORITY[\"EPSG\",\"2085\"],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",NORTH],AXIS[\"X\",EAST]]");
    p = add_epsg_def (first, last, 2086, "epsg", 2086,
		      "NAD27 / Cuba Sur (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=20.71666666666667 +lat_0=20.71666666666");
    add_proj4text (p, 1,
		   "667 +lon_0=-76.83333333333333 +k_0=0.99994848 +x_0=50000");
    add_proj4text (p, 2,
		   "0 +y_0=229126.939 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Cuba Sur (deprecated)\",GEOGCS[\"NAD27\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 2,
		 "866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"latitude_of_origin\",20.71666666666667],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-76.83333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.99994848],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",229126.939],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2086\"],AXIS[\"Y\",NORTH],AXIS[\"X\",EAST]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 2087, "epsg", 2087, "ELD79 / TM 12 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ELD79 / TM 12 NE\",GEOGCS[\"ELD79\",DATUM[\"Eur");
    add_srs_wkt (p, 1,
		 "opean_Libyan_Datum_1979\",SPHEROID[\"International 1924\"");
    add_srs_wkt (p, 2,
		 ",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6159\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4159\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",12],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2087\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2088, "epsg", 2088, "Carthage / TM 11 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=11 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +a=6378249.2 +b=6356515 +datum=carthage +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Carthage / TM 11 NE\",GEOGCS[\"Carthage\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Carthage\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,2");
    add_srs_wkt (p, 2,
		 "93.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6223\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4223\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",11],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "2088\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2089, "epsg", 2089,
		      "Yemen NGN96 / UTM zone 38N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Yemen NGN96 / UTM zone 38N\",GEOGCS[\"Yemen NGN");
    add_srs_wkt (p, 1,
		 "96\",DATUM[\"Yemen_National_Geodetic_Network_1996\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6163\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4163\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",45],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2089\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2090, "epsg", 2090,
		      "Yemen NGN96 / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Yemen NGN96 / UTM zone 39N\",GEOGCS[\"Yemen NGN");
    add_srs_wkt (p, 1,
		 "96\",DATUM[\"Yemen_National_Geodetic_Network_1996\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6163\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4163\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",51],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2090\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2091, "epsg", 2091,
		      "South Yemen / Gauss Kruger zone 8 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=-76,-138,67,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"South Yemen / Gauss Kruger zone 8 (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"South Yemen\",DATUM[\"South_Yemen\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[-76,-138,67,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6164\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4164\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",45],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",1],PARAMETER[\"false_easting\",8500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2091\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2092, "epsg", 2092,
		      "South Yemen / Gauss Kruger zone 9 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=-76,-138,67,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"South Yemen / Gauss Kruger zone 9 (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"South Yemen\",DATUM[\"South_Yemen\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[-76,-138,67,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6164\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4164\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",51],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",1],PARAMETER[\"false_easting\",9500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2092\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2093, "epsg", 2093,
		      "Hanoi 1972 / GK 106 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=106 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=-17.51,-108.32,-62.39,0,0,0,0 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hanoi 1972 / GK 106 NE\",GEOGCS[\"Hanoi 1972\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Hanoi_1972\",SPHEROID[\"Krassowsky 1940\",637824");
    add_srs_wkt (p, 2,
		 "5,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[-17.51,-10");
    add_srs_wkt (p, 3,
		 "8.32,-62.39,0,0,0,0],AUTHORITY[\"EPSG\",\"6147\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4147\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",106],PARAMETER[\"scale_factor\",1],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2093\"],AXIS[\"X\",NORTH],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2094, "epsg", 2094, "WGS 72BE / TM 106 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=106 +k=0.9996 +x_0=500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.814,-0.38 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / TM 106 NE\",GEOGCS[\"WGS 72BE\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TOWG");
    add_srs_wkt (p, 3,
		 "S84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",106],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"2094\"],AXIS[\"Eas");
    add_srs_wkt (p, 12, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2095, "epsg", 2095, "Bissau / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=intl +towgs84=-173,253,27,0,0,");
    add_proj4text (p, 1, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bissau / UTM zone 28N\",GEOGCS[\"Bissau\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Bissau\",SPHEROID[\"International 1924\",6378388,297,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-173,253,27,0,0,0,");
    add_srs_wkt (p, 3,
		 "0],AUTHORITY[\"EPSG\",\"6165\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4165\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",-15]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_ea");
    add_srs_wkt (p, 10,
		 "sting\",500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2095\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 12, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 2096, "epsg", 2096,
		      "Korean 1985 / Korea East Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=129 +k=1 +x_0=200000 +y_0=5");
    add_proj4text (p, 1, "00000 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Korean 1985 / Korea East Belt\",GEOGCS[\"Korean");
    add_srs_wkt (p, 1,
		 " 1985\",DATUM[\"Korean_Datum_1985\",SPHEROID[\"Bessel 18");
    add_srs_wkt (p, 2,
		 "41\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6162\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4162\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",38],PARAMETER[\"central_meridian\",129");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",200000],PARAMETER[\"false_northing\",500000],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"2096\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 2097, "epsg", 2097,
		      "Korean 1985 / Korea Central Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=127 +k=1 +x_0=200000 +y_0=5");
    add_proj4text (p, 1, "00000 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Korean 1985 / Korea Central Belt\",GEOGCS[\"Kor");
    add_srs_wkt (p, 1,
		 "ean 1985\",DATUM[\"Korean_Datum_1985\",SPHEROID[\"Bessel");
    add_srs_wkt (p, 2,
		 " 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6162\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4162\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",38],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "127],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",200000],PARAMETER[\"false_northing\",500000],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"2097\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 12, "ST]]");
    p = add_epsg_def (first, last, 2098, "epsg", 2098,
		      "Korean 1985 / Korea West Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=125 +k=1 +x_0=200000 +y_0=5");
    add_proj4text (p, 1, "00000 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Korean 1985 / Korea West Belt\",GEOGCS[\"Korean");
    add_srs_wkt (p, 1,
		 " 1985\",DATUM[\"Korean_Datum_1985\",SPHEROID[\"Bessel 18");
    add_srs_wkt (p, 2,
		 "41\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6162\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4162\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",38],PARAMETER[\"central_meridian\",125");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",200000],PARAMETER[\"false_northing\",500000],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"2098\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 2099, "epsg", 2099,
		      "Qatar 1948 / Qatar Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=25.38236111111111 +lon_0=50.7613888888");
    add_proj4text (p, 1,
		   "8889 +x_0=100000 +y_0=100000 +ellps=helmert +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Qatar 1948 / Qatar Grid\",GEOGCS[\"Qatar 1948\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Qatar_1948\",SPHEROID[\"Helmert 1906\",6378200,");
    add_srs_wkt (p, 2,
		 "298.3,AUTHORITY[\"EPSG\",\"7020\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6286\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4286\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Cassin");
    add_srs_wkt (p, 7,
		 "i_Soldner\"],PARAMETER[\"latitude_of_origin\",25.3823611");
    add_srs_wkt (p, 8,
		 "1111111],PARAMETER[\"central_meridian\",50.7613888888888");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"false_easting\",100000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",100000],AUTHORITY[\"EPSG\",\"2099\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2100, "epsg", 2100, "GGRS87 / Greek Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=GRS80 +datum=GGRS87 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GGRS87 / Greek Grid\",GEOGCS[\"GGRS87\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Greek_Geodetic_Reference_System_1987\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[-199.87,74.79,246.62,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6121\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4121\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",24],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2100\"],AX");
    add_srs_wkt (p, 12, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2101, "epsg", 2101,
		      "Lake / Maracaibo Grid M1");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=10.16666666666667 +lat_0=10.16666666666");
    add_proj4text (p, 1,
		   "667 +lon_0=-71.60561777777777 +k_0=1 +x_0=0 +y_0=-52684.");
    add_proj4text (p, 2, "972 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lake / Maracaibo Grid M1\",GEOGCS[\"Lake\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"Lake\",SPHEROID[\"International 1924\",6378388,297,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6249\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4249\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Confo");
    add_srs_wkt (p, 7,
		 "rmal_Conic_1SP\"],PARAMETER[\"latitude_of_origin\",10.16");
    add_srs_wkt (p, 8,
		 "666666666667],PARAMETER[\"central_meridian\",-71.6056177");
    add_srs_wkt (p, 9,
		 "7777777],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",0],PARAMETER[\"false_northing\",-52684.972],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"2101\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 2102, "epsg", 2102, "Lake / Maracaibo Grid");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=10.16666666666667 +lat_0=10.16666666666");
    add_proj4text (p, 1,
		   "667 +lon_0=-71.60561777777777 +k_0=1 +x_0=200000 +y_0=14");
    add_proj4text (p, 2, "7315.028 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lake / Maracaibo Grid\",GEOGCS[\"Lake\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Lake\",SPHEROID[\"International 1924\",6378388,297,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6249\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5,
		 "2\"]],AUTHORITY[\"EPSG\",\"4249\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conforma");
    add_srs_wkt (p, 7,
		 "l_Conic_1SP\"],PARAMETER[\"latitude_of_origin\",10.16666");
    add_srs_wkt (p, 8,
		 "666666667],PARAMETER[\"central_meridian\",-71.6056177777");
    add_srs_wkt (p, 9,
		 "7777],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_ea");
    add_srs_wkt (p, 10,
		 "sting\",200000],PARAMETER[\"false_northing\",147315.028]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"2102\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 2103, "epsg", 2103,
		      "Lake / Maracaibo Grid M3");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=10.16666666666667 +lat_0=10.16666666666");
    add_proj4text (p, 1,
		   "667 +lon_0=-71.60561777777777 +k_0=1 +x_0=500000 +y_0=44");
    add_proj4text (p, 2, "7315.028 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lake / Maracaibo Grid M3\",GEOGCS[\"Lake\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"Lake\",SPHEROID[\"International 1924\",6378388,297,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6249\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4249\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Confo");
    add_srs_wkt (p, 7,
		 "rmal_Conic_1SP\"],PARAMETER[\"latitude_of_origin\",10.16");
    add_srs_wkt (p, 8,
		 "666666666667],PARAMETER[\"central_meridian\",-71.6056177");
    add_srs_wkt (p, 9,
		 "7777777],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",447315.0");
    add_srs_wkt (p, 11,
		 "28],AUTHORITY[\"EPSG\",\"2103\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2104, "epsg", 2104,
		      "Lake / Maracaibo La Rosa Grid");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=10.16666666666667 +lat_0=10.16666666666");
    add_proj4text (p, 1,
		   "667 +lon_0=-71.60561777777777 +k_0=1 +x_0=-17044 +y_0=-2");
    add_proj4text (p, 2, "3139.97 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lake / Maracaibo La Rosa Grid\",GEOGCS[\"Lake\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Lake\",SPHEROID[\"International 1924\",6378388,");
    add_srs_wkt (p, 2,
		 "297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "249\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4249\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_1SP\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",10.16666666666667],PARAMETER[\"central_meridian\",-71.6");
    add_srs_wkt (p, 9,
		 "0561777777777],PARAMETER[\"scale_factor\",1],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",-17044],PARAMETER[\"false_northing\",-23");
    add_srs_wkt (p, 11,
		 "139.97],AUTHORITY[\"EPSG\",\"2104\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2105, "epsg", 2105,
		      "NZGD2000 / Mount Eden 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-36.87972222222222 +lon_0=174.7641666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999 +x_0=400000 +y_0=800000 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Mount Eden 2000\",GEOGCS[\"NZGD2000\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-36.879722222");
    add_srs_wkt (p, 9,
		 "22222],PARAMETER[\"central_meridian\",174.7641666666667]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",400000],PARAMETER[\"false_northing\",800000],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2105\"],AXIS[\"Northing\",NORTH],AXIS[");
    add_srs_wkt (p, 13, "\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2106, "epsg", 2106,
		      "NZGD2000 / Bay of Plenty 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-37.76111111111111 +lon_0=176.4661111");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Bay of Plenty 2000\",GEOGCS[\"NZGD20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"616");
    add_srs_wkt (p, 4,
		 "7\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-37.761111");
    add_srs_wkt (p, 9,
		 "11111111],PARAMETER[\"central_meridian\",176.46611111111");
    add_srs_wkt (p, 10,
		 "11],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",400000],PARAMETER[\"false_northing\",800000],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"2106\"],AXIS[\"Northing\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Easting\",EAST]]");
    p = add_epsg_def (first, last, 2107, "epsg", 2107,
		      "NZGD2000 / Poverty Bay 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-38.62444444444444 +lon_0=177.8855555");
    add_proj4text (p, 1,
		   "555556 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Poverty Bay 2000\",GEOGCS[\"NZGD2000");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-38.624444444");
    add_srs_wkt (p, 9,
		 "44444],PARAMETER[\"central_meridian\",177.8855555555556]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",400000],PARAMETER[\"false_northing\",800000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2107\"],AXIS[\"Northing\",NORTH],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST]]");
    p = add_epsg_def (first, last, 2108, "epsg", 2108,
		      "NZGD2000 / Hawkes Bay 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-39.65083333333333 +lon_0=176.6736111");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Hawkes Bay 2000\",GEOGCS[\"NZGD2000\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-39.650833333");
    add_srs_wkt (p, 9,
		 "33333],PARAMETER[\"central_meridian\",176.6736111111111]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",400000],PARAMETER[\"false_northing\",800000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2108\"],AXIS[\"Northing\",NORTH],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST]]");
    p = add_epsg_def (first, last, 2109, "epsg", 2109,
		      "NZGD2000 / Taranaki 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-39.13555555555556 +lon_0=174.2277777");
    add_proj4text (p, 1,
		   "777778 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Taranaki 2000\",GEOGCS[\"NZGD2000\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",-39.13555555555");
    add_srs_wkt (p, 9,
		 "556],PARAMETER[\"central_meridian\",174.2277777777778],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",400000],PARAMETER[\"false_northing\",800000],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2109\"],AXIS[\"Northing\",NORTH],AXIS[\"Easti");
    add_srs_wkt (p, 13, "ng\",EAST]]");
    p = add_epsg_def (first, last, 2110, "epsg", 2110,
		      "NZGD2000 / Tuhirangi 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-39.51222222222222 +lon_0=175.64 +k=1");
    add_proj4text (p, 1,
		   " +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Tuhirangi 2000\",GEOGCS[\"NZGD2000\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-39.512222222");
    add_srs_wkt (p, 9,
		 "22222],PARAMETER[\"central_meridian\",175.64],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"scale_factor\",1],PARAMETER[\"false_easting\",400000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",800000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12,
		 "\"2110\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2111, "epsg", 2111,
		      "NZGD2000 / Wanganui 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-40.24194444444444 +lon_0=175.4880555");
    add_proj4text (p, 1,
		   "555555 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Wanganui 2000\",GEOGCS[\"NZGD2000\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",-40.24194444444");
    add_srs_wkt (p, 9,
		 "444],PARAMETER[\"central_meridian\",175.4880555555555],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",400000],PARAMETER[\"false_northing\",800000],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2111\"],AXIS[\"Northing\",NORTH],AXIS[\"Easti");
    add_srs_wkt (p, 13, "ng\",EAST]]");
    p = add_epsg_def (first, last, 2112, "epsg", 2112,
		      "NZGD2000 / Wairarapa 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-40.92527777777777 +lon_0=175.6472222");
    add_proj4text (p, 1,
		   "222222 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Wairarapa 2000\",GEOGCS[\"NZGD2000\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-40.925277777");
    add_srs_wkt (p, 9,
		 "77777],PARAMETER[\"central_meridian\",175.6472222222222]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",400000],PARAMETER[\"false_northing\",800000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2112\"],AXIS[\"Northing\",NORTH],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST]]");
    p = add_epsg_def (first, last, 2113, "epsg", 2113,
		      "NZGD2000 / Wellington 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.3011111111111 +lon_0=174.77638888");
    add_proj4text (p, 1,
		   "88889 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs84");
    add_proj4text (p, 2, "=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Wellington 2000\",GEOGCS[\"NZGD2000\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-41.301111111");
    add_srs_wkt (p, 9,
		 "1111],PARAMETER[\"central_meridian\",174.7763888888889],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",400000],PARAMETER[\"false_northing\",800000],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2113\"],AXIS[\"Northing\",NORTH],AXIS[\"Easti");
    add_srs_wkt (p, 13, "ng\",EAST]]");
    p = add_epsg_def (first, last, 2114, "epsg", 2114,
		      "NZGD2000 / Collingwood 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-40.71472222222223 +lon_0=172.6719444");
    add_proj4text (p, 1,
		   "444444 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Collingwood 2000\",GEOGCS[\"NZGD2000");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-40.714722222");
    add_srs_wkt (p, 9,
		 "22223],PARAMETER[\"central_meridian\",172.6719444444444]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",400000],PARAMETER[\"false_northing\",800000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2114\"],AXIS[\"Northing\",NORTH],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST]]");
    p = add_epsg_def (first, last, 2115, "epsg", 2115,
		      "NZGD2000 / Nelson 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.27444444444444 +lon_0=173.2991666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Nelson 2000\",GEOGCS[\"NZGD2000\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",-41.274444444444");
    add_srs_wkt (p, 9,
		 "44],PARAMETER[\"central_meridian\",173.2991666666667],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "400000],PARAMETER[\"false_northing\",800000],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2115\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST]]");
    p = add_epsg_def (first, last, 2116, "epsg", 2116,
		      "NZGD2000 / Karamea 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.28972222222222 +lon_0=172.1088888");
    add_proj4text (p, 1,
		   "888889 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Karamea 2000\",GEOGCS[\"NZGD2000\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",-41.28972222222");
    add_srs_wkt (p, 9,
		 "222],PARAMETER[\"central_meridian\",172.1088888888889],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",400000],PARAMETER[\"false_northing\",800000],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2116\"],AXIS[\"Northing\",NORTH],AXIS[\"Easti");
    add_srs_wkt (p, 13, "ng\",EAST]]");
    p = add_epsg_def (first, last, 2117, "epsg", 2117,
		      "NZGD2000 / Buller 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.81055555555555 +lon_0=171.5811111");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Buller 2000\",GEOGCS[\"NZGD2000\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",-41.810555555555");
    add_srs_wkt (p, 9,
		 "55],PARAMETER[\"central_meridian\",171.5811111111111],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "400000],PARAMETER[\"false_northing\",800000],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2117\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST]]");
    p = add_epsg_def (first, last, 2118, "epsg", 2118, "NZGD2000 / Grey 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-42.33361111111111 +lon_0=171.5497222");
    add_proj4text (p, 1,
		   "222222 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Grey 2000\",GEOGCS[\"NZGD2000\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 198");
    add_srs_wkt (p, 2,
		 "0\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],");
    add_srs_wkt (p, 3,
		 "TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",-42.33361111111111]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",171.5497222222222],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",400");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",800000],AUTHORITY[\"EP");
    add_srs_wkt (p, 12,
		 "SG\",\"2118\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\"");
    add_srs_wkt (p, 13, ",EAST]]");
    p = add_epsg_def (first, last, 2119, "epsg", 2119, "NZGD2000 / Amuri 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-42.68888888888888 +lon_0=173.01 +k=1");
    add_proj4text (p, 1,
		   " +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Amuri 2000\",GEOGCS[\"NZGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",-42.6888888888888");
    add_srs_wkt (p, 9,
		 "8],PARAMETER[\"central_meridian\",173.01],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",400000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",800000],AUTHORITY[\"EPSG\",\"21");
    add_srs_wkt (p, 12,
		 "19\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2120, "epsg", 2120,
		      "NZGD2000 / Marlborough 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.54444444444444 +lon_0=173.8019444");
    add_proj4text (p, 1,
		   "444444 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Marlborough 2000\",GEOGCS[\"NZGD2000");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-41.544444444");
    add_srs_wkt (p, 9,
		 "44444],PARAMETER[\"central_meridian\",173.8019444444444]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",400000],PARAMETER[\"false_northing\",800000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2120\"],AXIS[\"Northing\",NORTH],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST]]");
    p = add_epsg_def (first, last, 2121, "epsg", 2121,
		      "NZGD2000 / Hokitika 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-42.88611111111111 +lon_0=170.9797222");
    add_proj4text (p, 1,
		   "222222 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Hokitika 2000\",GEOGCS[\"NZGD2000\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",-42.88611111111");
    add_srs_wkt (p, 9,
		 "111],PARAMETER[\"central_meridian\",170.9797222222222],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",400000],PARAMETER[\"false_northing\",800000],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2121\"],AXIS[\"Northing\",NORTH],AXIS[\"Easti");
    add_srs_wkt (p, 13, "ng\",EAST]]");
    p = add_epsg_def (first, last, 2122, "epsg", 2122,
		      "NZGD2000 / Okarito 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.11 +lon_0=170.2608333333333 +k=1 ");
    add_proj4text (p, 1,
		   "+x_0=400000 +y_0=800000 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Okarito 2000\",GEOGCS[\"NZGD2000\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",-43.11],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",170.2608333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",400000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",800000],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12,
		 "122\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2123, "epsg", 2123,
		      "NZGD2000 / Jacksons Bay 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.97777777777778 +lon_0=168.6061111");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Jacksons Bay 2000\",GEOGCS[\"NZGD200");
    add_srs_wkt (p, 1,
		 "0\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-43.977777777");
    add_srs_wkt (p, 9,
		 "77778],PARAMETER[\"central_meridian\",168.6061111111111]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",400000],PARAMETER[\"false_northing\",800000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2123\"],AXIS[\"Northing\",NORTH],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST]]");
    p = add_epsg_def (first, last, 2124, "epsg", 2124,
		      "NZGD2000 / Mount Pleasant 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.59055555555556 +lon_0=172.7269444");
    add_proj4text (p, 1,
		   "444445 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Mount Pleasant 2000\",GEOGCS[\"NZGD2");
    add_srs_wkt (p, 1,
		 "000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"616");
    add_srs_wkt (p, 4,
		 "7\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-43.590555");
    add_srs_wkt (p, 9,
		 "55555556],PARAMETER[\"central_meridian\",172.72694444444");
    add_srs_wkt (p, 10,
		 "45],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",400000],PARAMETER[\"false_northing\",800000],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"2124\"],AXIS[\"Northing\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Easting\",EAST]]");
    p = add_epsg_def (first, last, 2125, "epsg", 2125,
		      "NZGD2000 / Gawler 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.74861111111111 +lon_0=171.3605555");
    add_proj4text (p, 1,
		   "555555 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Gawler 2000\",GEOGCS[\"NZGD2000\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",-43.748611111111");
    add_srs_wkt (p, 9,
		 "11],PARAMETER[\"central_meridian\",171.3605555555555],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "400000],PARAMETER[\"false_northing\",800000],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2125\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST]]");
    p = add_epsg_def (first, last, 2126, "epsg", 2126,
		      "NZGD2000 / Timaru 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-44.40194444444445 +lon_0=171.0572222");
    add_proj4text (p, 1,
		   "222222 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Timaru 2000\",GEOGCS[\"NZGD2000\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",-44.401944444444");
    add_srs_wkt (p, 9,
		 "45],PARAMETER[\"central_meridian\",171.0572222222222],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "400000],PARAMETER[\"false_northing\",800000],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2126\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST]]");
    p = add_epsg_def (first, last, 2127, "epsg", 2127,
		      "NZGD2000 / Lindis Peak 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-44.735 +lon_0=169.4675 +k=1 +x_0=400");
    add_proj4text (p, 1,
		   "000 +y_0=800000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Lindis Peak 2000\",GEOGCS[\"NZGD2000");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-44.735],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",169.4675],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",1],PARAMETER[\"false_easting\",400000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",800000],AUTHORITY[\"EPSG\",\"2127\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2128, "epsg", 2128,
		      "NZGD2000 / Mount Nicholas 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.13277777777778 +lon_0=168.3986111");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Mount Nicholas 2000\",GEOGCS[\"NZGD2");
    add_srs_wkt (p, 1,
		 "000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"616");
    add_srs_wkt (p, 4,
		 "7\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-45.132777");
    add_srs_wkt (p, 9,
		 "77777778],PARAMETER[\"central_meridian\",168.39861111111");
    add_srs_wkt (p, 10,
		 "11],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",400000],PARAMETER[\"false_northing\",800000],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"2128\"],AXIS[\"Northing\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Easting\",EAST]]");
    p = add_epsg_def (first, last, 2129, "epsg", 2129,
		      "NZGD2000 / Mount York 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.56361111111111 +lon_0=167.7386111");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Mount York 2000\",GEOGCS[\"NZGD2000\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-45.563611111");
    add_srs_wkt (p, 9,
		 "11111],PARAMETER[\"central_meridian\",167.7386111111111]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",400000],PARAMETER[\"false_northing\",800000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2129\"],AXIS[\"Northing\",NORTH],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST]]");
    p = add_epsg_def (first, last, 2130, "epsg", 2130,
		      "NZGD2000 / Observation Point 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.81611111111111 +lon_0=170.6283333");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Observation Point 2000\",GEOGCS[\"NZ");
    add_srs_wkt (p, 1,
		 "GD2000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",-45.81");
    add_srs_wkt (p, 9,
		 "611111111111],PARAMETER[\"central_meridian\",170.6283333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",400000],PARAMETER[\"false_northing\",800000],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"2130\"],AXIS[\"Northing\",NORTH],AXI");
    add_srs_wkt (p, 13, "S[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2131, "epsg", 2131,
		      "NZGD2000 / North Taieri 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.86138888888889 +lon_0=170.2825 +k");
    add_proj4text (p, 1,
		   "=0.99996 +x_0=400000 +y_0=800000 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 2, ",0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / North Taieri 2000\",GEOGCS[\"NZGD200");
    add_srs_wkt (p, 1,
		 "0\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",-45.861388888");
    add_srs_wkt (p, 9,
		 "88889],PARAMETER[\"central_meridian\",170.2825],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.99996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "400000],PARAMETER[\"false_northing\",800000],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2131\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST]]");
    p = add_epsg_def (first, last, 2132, "epsg", 2132, "NZGD2000 / Bluff 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-46.6 +lon_0=168.3427777777778 +k=1 +");
    add_proj4text (p, 1,
		   "x_0=400000 +y_0=800000 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Bluff 2000\",GEOGCS[\"NZGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",-46.6],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",168.3427777777778],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",1],PARAMETER[\"false_easting\",400000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",800000],AUTHORITY[\"EPSG\",\"2132");
    add_srs_wkt (p, 12, "\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2133, "epsg", 2133,
		      "NZGD2000 / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / UTM zone 58S\",GEOGCS[\"NZGD2000\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",165],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"2133\"],AXIS[");
    add_srs_wkt (p, 12, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2134, "epsg", 2134,
		      "NZGD2000 / UTM zone 59S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / UTM zone 59S\",GEOGCS[\"NZGD2000\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",171],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"2134\"],AXIS[");
    add_srs_wkt (p, 12, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2135, "epsg", 2135,
		      "NZGD2000 / UTM zone 60S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / UTM zone 60S\",GEOGCS[\"NZGD2000\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",177],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"2135\"],AXIS[");
    add_srs_wkt (p, 12, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2136, "epsg", 2136,
		      "Accra / Ghana National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.666666666666667 +lon_0=-1 +k=0.9997");
    add_proj4text (p, 1,
		   "5 +x_0=274319.7391633579 +y_0=0 +a=6378300 +b=6356751.68");
    add_proj4text (p, 2, "9189189 +to_meter=0.3047997101815088 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Accra / Ghana National Grid\",GEOGCS[\"Accra\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Accra\",SPHEROID[\"War Office\",6378300,296,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7029\"]],AUTHORITY[\"EPSG\",\"6168\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5,
		 "22\"]],AUTHORITY[\"EPSG\",\"4168\"]],UNIT[\"Gold Coast f");
    add_srs_wkt (p, 6,
		 "oot\",0.3047997101815088,AUTHORITY[\"EPSG\",\"9094\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_origin\",4.666666666666667],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",-1],PARAMETER[\"scale_factor\",0.99975],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",900000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"2136\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_01 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2137, "epsg", 2137, "Accra / TM 1 NW");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-1 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +a=6378300 +b=6356751.689189189 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Accra / TM 1 NW\",GEOGCS[\"Accra\",DATUM[\"Accr");
    add_srs_wkt (p, 1,
		 "a\",SPHEROID[\"War Office\",6378300,296,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 2,
		 "\",\"7029\"]],AUTHORITY[\"EPSG\",\"6168\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 3,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"4168\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 7,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 8,
		 "dian\",-1],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 10,
		 "],AUTHORITY[\"EPSG\",\"2137\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 11, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2138, "epsg", 2138,
		      "NAD27(CGQ77) / Quebec Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=60 +lat_2=46 +lat_0=44 +lon_0=-68.5 +x_");
    add_proj4text (p, 1, "0=0 +y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27(CGQ77) / Quebec Lambert\",GEOGCS[\"NAD27(");
    add_srs_wkt (p, 1,
		 "CGQ77)\",DATUM[\"North_American_Datum_1927_CGQ77\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6609\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4609\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",60],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_2\",46],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",44],PARAMETER[\"central_meridian\",-68.5],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",0],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"2138\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 2139, "epsg", 2139,
		      "NAD83(CSRS98) / SCoPQ zone 2 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-55.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / SCoPQ zone 2 (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-55.5],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",304800],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"2139\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2140, "epsg", 2140,
		      "NAD83(CSRS98) / MTM zone 3 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-58.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 3 (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-58.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2140\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2141, "epsg", 2141,
		      "NAD83(CSRS98) / MTM zone 4 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-61.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 4 (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-61.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2141\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2142, "epsg", 2142,
		      "NAD83(CSRS98) / MTM zone 5 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-64.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 5 (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-64.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2142\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2143, "epsg", 2143,
		      "NAD83(CSRS98) / MTM zone 6 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-67.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 6 (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-67.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2143\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2144, "epsg", 2144,
		      "NAD83(CSRS98) / MTM zone 7 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-70.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 7 (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-70.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2144\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2145, "epsg", 2145,
		      "NAD83(CSRS98) / MTM zone 8 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-73.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 8 (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-73.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2145\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2146, "epsg", 2146,
		      "NAD83(CSRS98) / MTM zone 9 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-76.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 9 (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-76.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2146\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2147, "epsg", 2147,
		      "NAD83(CSRS98) / MTM zone 10 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-79.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / MTM zone 10 (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "99433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-79.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2147\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 2148, "epsg", 2148,
		      "NAD83(CSRS98) / UTM zone 21N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 21N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-57],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2148\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2149, "epsg", 2149,
		      "NAD83(CSRS98) / UTM zone 18N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 18N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-75],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2149\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2150, "epsg", 2150,
		      "NAD83(CSRS98) / UTM zone 17N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 17N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-81],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2150\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2151, "epsg", 2151,
		      "NAD83(CSRS98) / UTM zone 13N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 13N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-105],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2151\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 13, "g\",NORTH]]");
    p = add_epsg_def (first, last, 2152, "epsg", 2152,
		      "NAD83(CSRS98) / UTM zone 12N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 12N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-111],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2152\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 13, "g\",NORTH]]");
    p = add_epsg_def (first, last, 2153, "epsg", 2153,
		      "NAD83(CSRS98) / UTM zone 11N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / UTM zone 11N (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4140\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-117],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2153\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 13, "g\",NORTH]]");
    p = add_epsg_def (first, last, 2154, "epsg", 2154, "RGF93 / Lambert-93");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=44 +lat_0=46.5 +lon_0=3 +x_0=");
    add_proj4text (p, 1,
		   "700000 +y_0=6600000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / Lambert-93\",GEOGCS[\"RGF93\",DATUM[\"R");
    add_srs_wkt (p, 1,
		 "eseau_Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS");
    add_srs_wkt (p, 3,
		 "84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",49],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",44],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 10,
		 "\",46.5],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",700000],PARAMETER[\"false_northing\",6600");
    add_srs_wkt (p, 12,
		 "000],AUTHORITY[\"EPSG\",\"2154\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2155, "epsg", 2155,
		      "American Samoa 1962 / American Samoa Lambert (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-14.26666666666667 +lat_0=-14.266666666");
    add_proj4text (p, 1,
		   "66667 +lon_0=170 +k_0=1 +x_0=152400.3048006096 +y_0=0 +e");
    add_proj4text (p, 2,
		   "llps=clrk66 +towgs84=-115,118,426,0,0,0,0 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"American Samoa 1962 / American Samoa Lambert (d");
    add_srs_wkt (p, 1,
		 "eprecated)\",GEOGCS[\"American Samoa 1962\",DATUM[\"Amer");
    add_srs_wkt (p, 2,
		 "ican_Samoa_1962\",SPHEROID[\"Clarke 1866\",6378206.4,294");
    add_srs_wkt (p, 3,
		 ".9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],TOWGS84[-11");
    add_srs_wkt (p, 4,
		 "5,118,426,0,0,0,0],AUTHORITY[\"EPSG\",\"6169\"]],PRIMEM[");
    add_srs_wkt (p, 5,
		 "\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"deg");
    add_srs_wkt (p, 6,
		 "ree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"4169\"]],UNIT[\"US survey foot\",0.");
    add_srs_wkt (p, 8,
		 "3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTIO");
    add_srs_wkt (p, 9,
		 "N[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 10,
		 "of_origin\",-14.26666666666667],PARAMETER[\"central_meri");
    add_srs_wkt (p, 11,
		 "dian\",170],PARAMETER[\"scale_factor\",1],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",500000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 13,
		 "THORITY[\"EPSG\",\"2155\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 14, "ORTH]]");
    p = add_epsg_def (first, last, 2156, "epsg", 2156,
		      "NAD83(HARN) / UTM zone 59S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 59S (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_N");
    add_srs_wkt (p, 2,
		 "etwork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",171],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"2156\"],AXIS[");
    add_srs_wkt (p, 12, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2157, "epsg", 2157,
		      "IRENET95 / Irish Transverse Mercator");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=53.5 +lon_0=-8 +k=0.99982 +x_0=600000");
    add_proj4text (p, 1,
		   " +y_0=750000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IRENET95 / Irish Transverse Mercator\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "IRENET95\",DATUM[\"IRENET95\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6173\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4173\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",53.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-8],PARAMETER[\"scale_factor\",0.99982],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",600000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",750000],AUTHORITY[\"EPSG\",\"2157\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2158, "epsg", 2158,
		      "IRENET95 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IRENET95 / UTM zone 29N\",GEOGCS[\"IRENET95\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"IRENET95\",SPHEROID[\"GRS 1980\",6378137,298.2572");
    add_srs_wkt (p, 2,
		 "22101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,");
    add_srs_wkt (p, 3,
		 "0],AUTHORITY[\"EPSG\",\"6173\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4173\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",-9],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2158\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 2159, "epsg", 2159,
		      "Sierra Leone 1924 / New Colony Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=6.666666666666667 +lon_0=-12 +k=1 +x_");
    add_proj4text (p, 1,
		   "0=152399.8550907544 +y_0=0 +a=6378300 +b=6356751.6891891");
    add_proj4text (p, 2, "89 +to_meter=0.3047997101815088 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sierra Leone 1924 / New Colony Grid\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Sierra Leone 1924\",DATUM[\"Sierra_Leone_Colony_1924\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"War Office\",6378300,296,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7029\"]],AUTHORITY[\"EPSG\",\"6174\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4174\"]],UNIT[\"Gold Coast foot\",0.304799710");
    add_srs_wkt (p, 7,
		 "1815088,AUTHORITY[\"EPSG\",\"9094\"]],PROJECTION[\"Trans");
    add_srs_wkt (p, 8,
		 "verse_Mercator\"],PARAMETER[\"latitude_of_origin\",6.666");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"central_meridian\",-12],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "2159\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 2160, "epsg", 2160,
		      "Sierra Leone 1924 / New War Office Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=6.666666666666667 +lon_0=-12 +k=1 +x_");
    add_proj4text (p, 1,
		   "0=243839.7681452071 +y_0=182879.8261089053 +a=6378300 +b");
    add_proj4text (p, 2,
		   "=6356751.689189189 +to_meter=0.3047997101815088 +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sierra Leone 1924 / New War Office Grid\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Sierra Leone 1924\",DATUM[\"Sierra_Leone_Colony_1924");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"War Office\",6378300,296,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7029\"]],AUTHORITY[\"EPSG\",\"6174\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4174\"]],UNIT[\"Gold Coast foot\",0.304799");
    add_srs_wkt (p, 7,
		 "7101815088,AUTHORITY[\"EPSG\",\"9094\"]],PROJECTION[\"Tr");
    add_srs_wkt (p, 8,
		 "ansverse_Mercator\"],PARAMETER[\"latitude_of_origin\",6.");
    add_srs_wkt (p, 9,
		 "666666666666667],PARAMETER[\"central_meridian\",-12],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",8");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",600000],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"2160\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2161, "epsg", 2161,
		      "Sierra Leone 1968 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=clrk80 +towgs84=-88,4,101,0,0,");
    add_proj4text (p, 1, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sierra Leone 1968 / UTM zone 28N\",GEOGCS[\"Sie");
    add_srs_wkt (p, 1,
		 "rra Leone 1968\",DATUM[\"Sierra_Leone_1968\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7012\"]],TOWGS84[-88,4,101,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6175\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4175\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",-15],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2161\"");
    add_srs_wkt (p, 12, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2162, "epsg", 2162,
		      "Sierra Leone 1968 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=clrk80 +towgs84=-88,4,101,0,0,");
    add_proj4text (p, 1, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sierra Leone 1968 / UTM zone 29N\",GEOGCS[\"Sie");
    add_srs_wkt (p, 1,
		 "rra Leone 1968\",DATUM[\"Sierra_Leone_1968\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7012\"]],TOWGS84[-88,4,101,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6175\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4175\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",-9],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9996],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2162\"");
    add_srs_wkt (p, 12, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2163, "epsg", 2163, "unnamed");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=45 +lon_0=-100 +x_0=0 +y_0=0 +a=637099");
    add_proj4text (p, 1, "7 +b=6370997 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"unnamed\",GEOGCS[\"unnamed ellipse\",DATUM[\"un");
    add_srs_wkt (p, 1,
		 "known\",SPHEROID[\"unnamed\",6370997,0]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 2,
		 "ich\",0],UNIT[\"degree\",0.0174532925199433]],PROJECTION");
    add_srs_wkt (p, 3,
		 "[\"Lambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 4,
		 "of_center\",45],PARAMETER[\"longitude_of_center\",-100],");
    add_srs_wkt (p, 5,
		 "PARAMETER[\"false_easting\",0],PARAMETER[\"false_northin");
    add_srs_wkt (p, 6,
		 "g\",0],UNIT[\"Meter\",1],AUTHORITY[\"EPSG\",\"2163\"]]");
    p = add_epsg_def (first, last, 2164, "epsg", 2164,
		      "Locodjo 1965 / TM 5 NW");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-5 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=clrk80 +towgs84=-125,53,467,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Locodjo 1965 / TM 5 NW\",GEOGCS[\"Locodjo 1965\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Locodjo_1965\",SPHEROID[\"Clarke 1880 (RGS)\",6");
    add_srs_wkt (p, 2,
		 "378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[-125,53,467,0,0,0,0],AUTHORITY[\"EPSG\",\"6142\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4142\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",-5],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",0],AUTHORITY[\"EPSG\",\"2164\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2165, "epsg", 2165,
		      "Abidjan 1987 / TM 5 NW");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-5 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=clrk80 +towgs84=-124.76,53,466.79,0,0,0,0 +un");
    add_proj4text (p, 2, "its=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Abidjan 1987 / TM 5 NW\",GEOGCS[\"Abidjan 1987\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Abidjan_1987\",SPHEROID[\"Clarke 1880 (RGS)\",6");
    add_srs_wkt (p, 2,
		 "378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[-124.76,53,466.79,0,0,0,0],AUTHORITY[\"EPSG\",\"6143\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4143\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-5],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2165\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2166, "epsg", 2166,
		      "Pulkovo 1942(83) / Gauss Kruger zone 3 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=3500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / Gauss Kruger zone 3 (depreca");
    add_srs_wkt (p, 1,
		 "ted)\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_");
    add_srs_wkt (p, 2,
		 "83\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",9],PARAMETER[\"scale_factor\",1],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",3500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2166\"],AXIS[\"X\",NORTH],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2167, "epsg", 2167,
		      "Pulkovo 1942(83) / Gauss Kruger zone 4 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / Gauss Kruger zone 4 (depreca");
    add_srs_wkt (p, 1,
		 "ted)\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_");
    add_srs_wkt (p, 2,
		 "83\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",12],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",4500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"2167\"],AXIS[\"X\",NORTH],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2168, "epsg", 2168,
		      "Pulkovo 1942(83) / Gauss Kruger zone 5 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / Gauss Kruger zone 5 (depreca");
    add_srs_wkt (p, 1,
		 "ted)\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_");
    add_srs_wkt (p, 2,
		 "83\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",15],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",5500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"2168\"],AXIS[\"X\",NORTH],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2169, "epsg", 2169,
		      "Luxembourg 1930 / Gauss");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=49.83333333333334 +lon_0=6.1666666666");
    add_proj4text (p, 1,
		   "66667 +k=1 +x_0=80000 +y_0=100000 +ellps=intl +towgs84=-");
    add_proj4text (p, 2,
		   "193,13.7,-39.3,-0.41,-2.933,2.688,0.43 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Luxembourg 1930 / Gauss\",GEOGCS[\"Luxembourg 1");
    add_srs_wkt (p, 1,
		 "930\",DATUM[\"Luxembourg_1930\",SPHEROID[\"International");
    add_srs_wkt (p, 2,
		 " 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS8");
    add_srs_wkt (p, 3,
		 "4[-193,13.7,-39.3,-0.41,-2.933,2.688,0.43],AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"6181\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4181\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",49.83333333333334],PARAMETER[\"central_meridian\",6.166");
    add_srs_wkt (p, 10,
		 "666666666667],PARAMETER[\"scale_factor\",1],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",80000],PARAMETER[\"false_northing\",1000");
    add_srs_wkt (p, 12,
		 "00],AUTHORITY[\"EPSG\",\"2169\"],AXIS[\"X\",NORTH],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2170, "epsg", 2170,
		      "MGI / Slovenia Grid (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=bessel +datum=hermannskogel +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Slovenia Grid (deprecated)\",GEOGCS[\"MGI");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Militar_Geographische_Institute\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137,1.474,5");
    add_srs_wkt (p, 4,
		 ".297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 5,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 6,
		 "0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",15],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"2170\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",EAST]]");
    p = add_epsg_def (first, last, 2171, "epsg", 2171,
		      "Pulkovo 1942(58) / Poland zone I (deprecated)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=50.625 +lon_0=21.08333333333333 +k=0");
    add_proj4text (p, 1,
		   ".9998 +x_0=4637000 +y_0=5647000 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Poland zone I (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Oblique_Stereographic\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_origin\",50.625],PARAMETER[\"cen");
    add_srs_wkt (p, 9,
		 "tral_meridian\",21.08333333333333],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 10,
		 "tor\",0.9998],PARAMETER[\"false_easting\",4637000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",5647000],AUTHORITY[\"EPSG\",\"21");
    add_srs_wkt (p, 12, "71\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2172, "epsg", 2172,
		      "Pulkovo 1942(58) / Poland zone II");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=53.00194444444445 +lon_0=21.50277777");
    add_proj4text (p, 1,
		   "777778 +k=0.9998 +x_0=4603000 +y_0=5806000 +ellps=krass ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Poland zone II\",GEOGCS[\"Pu");
    add_srs_wkt (p, 1,
		 "lkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID[\"Kr");
    add_srs_wkt (p, 2,
		 "assowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Oblique_Stereographic\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",53.00194444444445],PARAMETER[\"centr");
    add_srs_wkt (p, 9,
		 "al_meridian\",21.50277777777778],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 10,
		 "r\",0.9998],PARAMETER[\"false_easting\",4603000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",5806000],AUTHORITY[\"EPSG\",\"2172");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2173, "epsg", 2173,
		      "Pulkovo 1942(58) / Poland zone III");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=53.58333333333334 +lon_0=17.00833333");
    add_proj4text (p, 1,
		   "333333 +k=0.9998 +x_0=3501000 +y_0=5999000 +ellps=krass ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Poland zone III\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID[\"K");
    add_srs_wkt (p, 2,
		 "rassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Oblique_Stereographic\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",53.58333333333334],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",17.00833333333333],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9998],PARAMETER[\"false_easting\",3501000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",5999000],AUTHORITY[\"EPSG\",\"217");
    add_srs_wkt (p, 12, "3\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2174, "epsg", 2174,
		      "Pulkovo 1942(58) / Poland zone IV");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=51.67083333333333 +lon_0=16.67222222");
    add_proj4text (p, 1,
		   "222222 +k=0.9998 +x_0=3703000 +y_0=5627000 +ellps=krass ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Poland zone IV\",GEOGCS[\"Pu");
    add_srs_wkt (p, 1,
		 "lkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID[\"Kr");
    add_srs_wkt (p, 2,
		 "assowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Oblique_Stereographic\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",51.67083333333333],PARAMETER[\"centr");
    add_srs_wkt (p, 9,
		 "al_meridian\",16.67222222222222],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 10,
		 "r\",0.9998],PARAMETER[\"false_easting\",3703000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",5627000],AUTHORITY[\"EPSG\",\"2174");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2175, "epsg", 2175,
		      "Pulkovo 1942(58) / Poland zone V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18.95833333333333 +k=0.99998");
    add_proj4text (p, 1,
		   "3 +x_0=237000 +y_0=-4700000 +ellps=krass +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Poland zone V\",GEOGCS[\"Pul");
    add_srs_wkt (p, 1,
		 "kovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID[\"Kra");
    add_srs_wkt (p, 2,
		 "ssowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",18.9");
    add_srs_wkt (p, 9,
		 "5833333333333],PARAMETER[\"scale_factor\",0.999983],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",237000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",-4700000],AUTHORITY[\"EPSG\",\"2175\"],AXIS[\"X\",N");
    add_srs_wkt (p, 12, "ORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2176, "epsg", 2176,
		      "ETRS89 / Poland CS2000 zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.999923 +x_0=5500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Poland CS2000 zone 5\",GEOGCS[\"ETRS89");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Terrestrial_Reference_System_1989\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",15],PARAMETER[\"scale_factor\",0.999923],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",5500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"2176\"],AXIS[\"x\",NORTH],AX");
    add_srs_wkt (p, 12, "IS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 2177, "epsg", 2177,
		      "ETRS89 / Poland CS2000 zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=0.999923 +x_0=6500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Poland CS2000 zone 6\",GEOGCS[\"ETRS89");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Terrestrial_Reference_System_1989\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",18],PARAMETER[\"scale_factor\",0.999923],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",6500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"2177\"],AXIS[\"x\",NORTH],AX");
    add_srs_wkt (p, 12, "IS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 2178, "epsg", 2178,
		      "ETRS89 / Poland CS2000 zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=0.999923 +x_0=7500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Poland CS2000 zone 7\",GEOGCS[\"ETRS89");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Terrestrial_Reference_System_1989\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",21],PARAMETER[\"scale_factor\",0.999923],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",7500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"2178\"],AXIS[\"x\",NORTH],AX");
    add_srs_wkt (p, 12, "IS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 2179, "epsg", 2179,
		      "ETRS89 / Poland CS2000 zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.999923 +x_0=8500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Poland CS2000 zone 8\",GEOGCS[\"ETRS89");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Terrestrial_Reference_System_1989\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",24],PARAMETER[\"scale_factor\",0.999923],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",8500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"2179\"],AXIS[\"x\",NORTH],AX");
    add_srs_wkt (p, 12, "IS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 2180, "epsg", 2180, "ETRS89 / Poland CS92");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=19 +k=0.9993 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=-5300000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Poland CS92\",GEOGCS[\"ETRS89\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"European_Terrestrial_Reference_System_1989\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",19],PARAMETER[\"scale_factor\",0.9993],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",-530000");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"2180\"],AXIS[\"x\",NORTH],AXIS[\"");
    add_srs_wkt (p, 12, "y\",EAST]]");
    p = add_epsg_def (first, last, 2188, "epsg", 2188,
		      "Azores Occidental 1939 / UTM zone 25N");
    add_proj4text (p, 0, "+proj=utm +zone=25 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Azores Occidental 1939 / UTM zone 25N\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Azores Occidental 1939\",DATUM[\"Azores_Occidental_Isl");
    add_srs_wkt (p, 2,
		 "ands_1939\",SPHEROID[\"International 1924\",6378388,297,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6182\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4182\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-33],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"2188\"],AXIS[\"Eas");
    add_srs_wkt (p, 12, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2189, "epsg", 2189,
		      "Azores Central 1948 / UTM zone 26N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +ellps=intl +towgs84=-104,167,-38,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Azores Central 1948 / UTM zone 26N\",GEOGCS[\"A");
    add_srs_wkt (p, 1,
		 "zores Central 1948\",DATUM[\"Azores_Central_Islands_1948");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7022\"]],TOWGS84[-104,167,-38,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6183\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"418");
    add_srs_wkt (p, 7,
		 "3\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",-27],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12,
		 "\",\"2189\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NO");
    add_srs_wkt (p, 13, "RTH]]");
    p = add_epsg_def (first, last, 2190, "epsg", 2190,
		      "Azores Oriental 1940 / UTM zone 26N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +ellps=intl +towgs84=-203,141,53,0,0,");
    add_proj4text (p, 1, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Azores Oriental 1940 / UTM zone 26N\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Azores Oriental 1940\",DATUM[\"Azores_Oriental_Islands_1");
    add_srs_wkt (p, 2,
		 "940\",SPHEROID[\"International 1924\",6378388,297,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7022\"]],TOWGS84[-203,141,53,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6184\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "184\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-27],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"2190\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 2191, "epsg", 2191,
		      "Madeira 1936 / UTM zone 28N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Madeira 1936 / UTM zone 28N (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Madeira 1936\",DATUM[\"Madeira_1936\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6185\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "2925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"4185\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 7,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 8,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-15");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"2191\"],AXIS[\"Easting\",EAST],AXIS[\"Nort");
    add_srs_wkt (p, 12, "hing\",NORTH]]");
    p = add_epsg_def (first, last, 2192, "epsg", 2192,
		      "ED50 / France EuroLambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.8 +lat_0=46.8 +lon_0=2.3372291666666");
    add_proj4text (p, 1,
		   "67 +k_0=0.99987742 +x_0=600000 +y_0=2200000 +ellps=intl ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / France EuroLambert\",GEOGCS[\"ED50\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"European_Datum_1950\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 8,
		 "f_origin\",46.8],PARAMETER[\"central_meridian\",2.337229");
    add_srs_wkt (p, 9,
		 "166666667],PARAMETER[\"scale_factor\",0.99987742],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",600000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",2200000],AUTHORITY[\"EPSG\",\"2192\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2193, "epsg", 2193,
		      "NZGD2000 / New Zealand Transverse Mercator 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=173 +k=0.9996 +x_0=1600000 +");
    add_proj4text (p, 1,
		   "y_0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / New Zealand Transverse Mercator 2000");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"NZGD2000\",DATUM[\"New_Zealand_Geodetic_Datu");
    add_srs_wkt (p, 2,
		 "m_2000\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",0],PARAMETER[\"central_meridian\",173],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",1");
    add_srs_wkt (p, 11,
		 "600000],PARAMETER[\"false_northing\",10000000],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"2193\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 13, "ing\",EAST]]");
    p = add_epsg_def (first, last, 2194, "epsg", 2194,
		      "American Samoa 1962 / American Samoa Lambert (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-14.26666666666667 +lat_0=-14.266666666");
    add_proj4text (p, 1,
		   "66667 +lon_0=-170 +k_0=1 +x_0=152400.3048006096 +y_0=0 +");
    add_proj4text (p, 2,
		   "ellps=clrk66 +towgs84=-115,118,426,0,0,0,0 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"American Samoa 1962 / American Samoa Lambert (d");
    add_srs_wkt (p, 1,
		 "eprecated)\",GEOGCS[\"American Samoa 1962\",DATUM[\"Amer");
    add_srs_wkt (p, 2,
		 "ican_Samoa_1962\",SPHEROID[\"Clarke 1866\",6378206.4,294");
    add_srs_wkt (p, 3,
		 ".9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],TOWGS84[-11");
    add_srs_wkt (p, 4,
		 "5,118,426,0,0,0,0],AUTHORITY[\"EPSG\",\"6169\"]],PRIMEM[");
    add_srs_wkt (p, 5,
		 "\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"deg");
    add_srs_wkt (p, 6,
		 "ree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"4169\"]],UNIT[\"US survey foot\",0.");
    add_srs_wkt (p, 8,
		 "3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTIO");
    add_srs_wkt (p, 9,
		 "N[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 10,
		 "of_origin\",-14.26666666666667],PARAMETER[\"central_meri");
    add_srs_wkt (p, 11,
		 "dian\",-170],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",500000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"2194\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 14, "NORTH]]");
    p = add_epsg_def (first, last, 2195, "epsg", 2195,
		      "NAD83(HARN) / UTM zone 2S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +south +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 2S\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-171],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"2195\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2196, "epsg", 2196,
		      "ETRS89 / Kp2000 Jutland");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9.5 +k=0.99995 +x_0=200000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Kp2000 Jutland\",GEOGCS[\"ETRS89\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"European_Terrestrial_Reference_System_1989\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",9.5],PARAMETER[\"scale_factor\",0.99995],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",200000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"2196\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2197, "epsg", 2197,
		      "ETRS89 / Kp2000 Zealand");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=0.99995 +x_0=500000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Kp2000 Zealand\",GEOGCS[\"ETRS89\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"European_Terrestrial_Reference_System_1989\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",12],PARAMETER[\"scale_factor\",0.99995],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2197\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2198, "epsg", 2198,
		      "ETRS89 / Kp2000 Bornholm");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=900000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Kp2000 Bornholm\",GEOGCS[\"ETRS89\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"European_Terrestrial_Reference_System_1989\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",15],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",900000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"2198\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 2199, "epsg", 2199,
		      "Albanian 1987 / Gauss Kruger zone 4 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Albanian 1987 / Gauss Kruger zone 4 (deprecated");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"Albanian 1987\",DATUM[\"Albanian_1987\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7024\"]],AUTHORITY[\"EPSG\",\"6191\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4191\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",21],PARAMETER[\"scale_factor\",1],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",4500000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 11,
		 "THORITY[\"EPSG\",\"2199\"],AXIS[\"X\",NORTH],AXIS[\"Y\",");
    add_srs_wkt (p, 12, "EAST]]");
    p = add_epsg_def (first, last, 2200, "epsg", 2200,
		      "ATS77 / New Brunswick Stereographic (ATS77)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=46.5 +lon_0=-66.5 +k=0.999912 +x_0=3");
    add_proj4text (p, 1,
		   "00000 +y_0=800000 +a=6378135 +b=6356750.304921594 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ATS77 / New Brunswick Stereographic (ATS77)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"ATS77\",DATUM[\"Average_Terrestrial_System_1977\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Average Terrestrial System 1977\",6378135,29");
    add_srs_wkt (p, 3,
		 "8.257,AUTHORITY[\"EPSG\",\"7041\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6122\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4122\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Obliqu");
    add_srs_wkt (p, 8,
		 "e_Stereographic\"],PARAMETER[\"latitude_of_origin\",46.5");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-66.5],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.999912],PARAMETER[\"false_easting\",300000]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",800000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"2200\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAS");
    add_srs_wkt (p, 13, "T]]");
    p = add_epsg_def (first, last, 2201, "epsg", 2201, "REGVEN / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"REGVEN / UTM zone 18N\",GEOGCS[\"REGVEN\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Red_Geodesica_Venezolana\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6189\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4189\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-75],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"2201\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2202, "epsg", 2202, "REGVEN / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"REGVEN / UTM zone 19N\",GEOGCS[\"REGVEN\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Red_Geodesica_Venezolana\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6189\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4189\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-69],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"2202\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2203, "epsg", 2203, "REGVEN / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"REGVEN / UTM zone 20N\",GEOGCS[\"REGVEN\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Red_Geodesica_Venezolana\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6189\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4189\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-63],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"2203\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2204, "epsg", 2204, "NAD27 / Tennessee");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.25 +lat_2=36.41666666666666 +lat_0=3");
    add_proj4text (p, 1,
		   "4.66666666666666 +lon_0=-86 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2,
		   "30480.06096012192 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Tennessee\",GEOGCS[\"NAD27\",DATUM[\"No");
    add_srs_wkt (p, 1,
		 "rth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",63782");
    add_srs_wkt (p, 2,
		 "06.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_");
    add_srs_wkt (p, 8,
		 "Conic_2SP\"],PARAMETER[\"standard_parallel_1\",35.25],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"standard_parallel_2\",36.41666666666666],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",34.66666666666666],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-86],PARAMETER[\"false_easting\",2");
    add_srs_wkt (p, 12,
		 "000000],PARAMETER[\"false_northing\",100000],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2204\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2205, "epsg", 2205,
		      "NAD83 / Kentucky North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=38.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=500000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kentucky North\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",37.96666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",38.96666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",37.5],PARAMETER[\"central_meridian\",-84.25],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"2205\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2206, "epsg", 2206,
		      "ED50 / 3-degree Gauss-Kruger zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / 3-degree Gauss-Kruger zone 9\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4230\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",27],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",950");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"2206\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2207, "epsg", 2207,
		      "ED50 / 3-degree Gauss-Kruger zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=10500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / 3-degree Gauss-Kruger zone 10\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4230\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",30],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",105");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"2207\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2208, "epsg", 2208,
		      "ED50 / 3-degree Gauss-Kruger zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=11500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / 3-degree Gauss-Kruger zone 11\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4230\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",33],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",115");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"2208\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2209, "epsg", 2209,
		      "ED50 / 3-degree Gauss-Kruger zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=36 +k=1 +x_0=12500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / 3-degree Gauss-Kruger zone 12\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4230\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",36],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",125");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"2209\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2210, "epsg", 2210,
		      "ED50 / 3-degree Gauss-Kruger zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=13500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / 3-degree Gauss-Kruger zone 13\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4230\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",39],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",135");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"2210\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2211, "epsg", 2211,
		      "ED50 / 3-degree Gauss-Kruger zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=42 +k=1 +x_0=14500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / 3-degree Gauss-Kruger zone 14\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4230\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",42],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",145");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"2211\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2212, "epsg", 2212,
		      "ED50 / 3-degree Gauss-Kruger zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=15500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / 3-degree Gauss-Kruger zone 15\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4230\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",45],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",155");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"2212\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2213, "epsg", 2213, "ETRS89 / TM 30 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / TM 30 NE\",GEOGCS[\"ETRS89\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Terrestrial_Reference_System_1989\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",3");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"2213\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 12, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 2214, "epsg", 2214,
		      "Douala 1948 / AOF west (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=10.5 +k=0.999 +x_0=1000000 +");
    add_proj4text (p, 1,
		   "y_0=1000000 +ellps=intl +towgs84=-206.1,-174.7,-87.7,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Douala 1948 / AOF west (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Douala 1948\",DATUM[\"Douala_1948\",SPHEROID[\"Internati");
    add_srs_wkt (p, 2,
		 "onal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[-206.1,-174.7,-87.7,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6192\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4192\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",10.5],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.999],PARAMETER[\"false_easting\",1000000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"2214");
    add_srs_wkt (p, 12, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2215, "epsg", 2215,
		      "Manoca 1962 / UTM zone 32N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +a=6378249.2 +b=6356515 +towgs84=-70.");
    add_proj4text (p, 1, "9,-151.8,-41.4,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Manoca 1962 / UTM zone 32N\",GEOGCS[\"Manoca 19");
    add_srs_wkt (p, 1,
		 "62\",DATUM[\"Manoca_1962\",SPHEROID[\"Clarke 1880 (IGN)\"");
    add_srs_wkt (p, 2,
		 ",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-70.9,-151.8,-41.4,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6193\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4193\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",9],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 10,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2215\"],AXI");
    add_srs_wkt (p, 12, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2216, "epsg", 2216,
		      "Qornoq 1927 / UTM zone 22N");
    add_proj4text (p, 0, "+proj=utm +zone=22 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Qornoq 1927 / UTM zone 22N\",GEOGCS[\"Qornoq 19");
    add_srs_wkt (p, 1,
		 "27\",DATUM[\"Qornoq_1927\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6194\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4194\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-51],PARAMETER[\"sc");
    add_srs_wkt (p, 9,
		 "ale_factor\",0.9996],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"22");
    add_srs_wkt (p, 11,
		 "16\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2217, "epsg", 2217,
		      "Qornoq 1927 / UTM zone 23N");
    add_proj4text (p, 0, "+proj=utm +zone=23 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Qornoq 1927 / UTM zone 23N\",GEOGCS[\"Qornoq 19");
    add_srs_wkt (p, 1,
		 "27\",DATUM[\"Qornoq_1927\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6194\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4194\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-45],PARAMETER[\"sc");
    add_srs_wkt (p, 9,
		 "ale_factor\",0.9996],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"22");
    add_srs_wkt (p, 11,
		 "17\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2219, "epsg", 2219, "ATS77 / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +a=6378135 +b=6356750.304921594 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ATS77 / UTM zone 19N\",GEOGCS[\"ATS77\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Average_Terrestrial_System_1977\",SPHEROID[\"Average Ter");
    add_srs_wkt (p, 2,
		 "restrial System 1977\",6378135,298.257,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7041\"]],AUTHORITY[\"EPSG\",\"6122\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4122\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-69],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2219\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2220, "epsg", 2220, "ATS77 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +a=6378135 +b=6356750.304921594 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ATS77 / UTM zone 20N\",GEOGCS[\"ATS77\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Average_Terrestrial_System_1977\",SPHEROID[\"Average Ter");
    add_srs_wkt (p, 2,
		 "restrial System 1977\",6378135,298.257,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7041\"]],AUTHORITY[\"EPSG\",\"6122\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4122\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-63],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2220\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2222, "epsg", 2222,
		      "NAD83 / Arizona East (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-110.1666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=ft");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arizona East (ft)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",31],PARAMETER[\"central_meridian\",-110.1");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",700000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2222\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2223, "epsg", 2223,
		      "NAD83 / Arizona Central (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-111.9166666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=ft");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arizona Central (ft)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",31],PARAMETER[\"central_meridian\",-111.");
    add_srs_wkt (p, 9,
		 "9166666666667],PARAMETER[\"scale_factor\",0.9999],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",700000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"2223\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2224, "epsg", 2224,
		      "NAD83 / Arizona West (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-113.75 +k=0.999933333 +x_0");
    add_proj4text (p, 1,
		   "=213360 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=ft +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arizona West (ft)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",31],PARAMETER[\"central_meridian\",-113.7");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.999933333],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",700000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"2224\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 2225, "epsg", 2225,
		      "NAD83 / California zone 1 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.66666666666666 +lat_2=40 +lat_0=39.3");
    add_proj4text (p, 1,
		   "3333333333334 +lon_0=-122 +x_0=2000000.0001016 +y_0=5000");
    add_proj4text (p, 2,
		   "00.0001016001 +ellps=GRS80 +datum=NAD83 +units=us-ft +no");
    add_proj4text (p, 3, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 1 (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.");
    add_srs_wkt (p, 9,
		 "66666666666666],PARAMETER[\"standard_parallel_2\",40],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",39.33333333333334],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-122],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",6561666.667],PARAMETER[\"false_northing\",1640416.667");
    add_srs_wkt (p, 13,
		 "],AUTHORITY[\"EPSG\",\"2225\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 2226, "epsg", 2226,
		      "NAD83 / California zone 2 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.83333333333334 +lat_2=38.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=37.66666666666666 +lon_0=-122 +x_0=2000000.00");
    add_proj4text (p, 2,
		   "01016 +y_0=500000.0001016001 +ellps=GRS80 +datum=NAD83 +");
    add_proj4text (p, 3, "units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 2 (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.");
    add_srs_wkt (p, 9,
		 "83333333333334],PARAMETER[\"standard_parallel_2\",38.333");
    add_srs_wkt (p, 10,
		 "33333333334],PARAMETER[\"latitude_of_origin\",37.6666666");
    add_srs_wkt (p, 11,
		 "6666666],PARAMETER[\"central_meridian\",-122],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",6561666.667],PARAMETER[\"false_northin");
    add_srs_wkt (p, 13,
		 "g\",1640416.667],AUTHORITY[\"EPSG\",\"2226\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2227, "epsg", 2227,
		      "NAD83 / California zone 3 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.5 +lon_0=-120.5 +x_0=2000000.0001016 +y_0=");
    add_proj4text (p, 2,
		   "500000.0001016001 +ellps=GRS80 +datum=NAD83 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 3 (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.");
    add_srs_wkt (p, 9,
		 "43333333333333],PARAMETER[\"standard_parallel_2\",37.066");
    add_srs_wkt (p, 10,
		 "66666666667],PARAMETER[\"latitude_of_origin\",36.5],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-120.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",6561666.667],PARAMETER[\"false_northing\",1640416");
    add_srs_wkt (p, 13,
		 ".667],AUTHORITY[\"EPSG\",\"2227\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2228, "epsg", 2228,
		      "NAD83 / California zone 4 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.25 +lat_2=36 +lat_0=35.3333333333333");
    add_proj4text (p, 1,
		   "4 +lon_0=-119 +x_0=2000000.0001016 +y_0=500000.000101600");
    add_proj4text (p, 2, "1 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 4 (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",37.");
    add_srs_wkt (p, 9,
		 "25],PARAMETER[\"standard_parallel_2\",36],PARAMETER[\"la");
    add_srs_wkt (p, 10,
		 "titude_of_origin\",35.33333333333334],PARAMETER[\"centra");
    add_srs_wkt (p, 11,
		 "l_meridian\",-119],PARAMETER[\"false_easting\",6561666.6");
    add_srs_wkt (p, 12,
		 "67],PARAMETER[\"false_northing\",1640416.667],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"2228\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2229, "epsg", 2229,
		      "NAD83 / California zone 5 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.46666666666667 +lat_2=34.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.5 +lon_0=-118 +x_0=2000000.0001016 +y_0=50");
    add_proj4text (p, 2,
		   "0000.0001016001 +ellps=GRS80 +datum=NAD83 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 5 (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",35.");
    add_srs_wkt (p, 9,
		 "46666666666667],PARAMETER[\"standard_parallel_2\",34.033");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"latitude_of_origin\",33.5],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-118],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",6561666.667],PARAMETER[\"false_northing\",1640416.6");
    add_srs_wkt (p, 13,
		 "67],AUTHORITY[\"EPSG\",\"2229\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2230, "epsg", 2230,
		      "NAD83 / California zone 6 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.88333333333333 +lat_2=32.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=32.16666666666666 +lon_0=-116.25 +x_0=2000000");
    add_proj4text (p, 2,
		   ".0001016 +y_0=500000.0001016001 +ellps=GRS80 +datum=NAD8");
    add_proj4text (p, 3, "3 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 6 (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",33.");
    add_srs_wkt (p, 9,
		 "88333333333333],PARAMETER[\"standard_parallel_2\",32.783");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"latitude_of_origin\",32.1666666");
    add_srs_wkt (p, 11,
		 "6666666],PARAMETER[\"central_meridian\",-116.25],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",6561666.667],PARAMETER[\"false_nort");
    add_srs_wkt (p, 13,
		 "hing\",1640416.667],AUTHORITY[\"EPSG\",\"2230\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2231, "epsg", 2231,
		      "NAD83 / Colorado North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.78333333333333 +lat_2=39.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=39.33333333333334 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "288036576 +y_0=304800.6096012192 +ellps=GRS80 +datum=NAD");
    add_proj4text (p, 3, "83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Colorado North (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.78333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",39.71666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",39.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-105.5],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",3000000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 13,
		 "0000],AUTHORITY[\"EPSG\",\"2231\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2232, "epsg", 2232,
		      "NAD83 / Colorado Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.75 +lat_2=38.45 +lat_0=37.8333333333");
    add_proj4text (p, 1,
		   "3334 +lon_0=-105.5 +x_0=914401.8288036576 +y_0=304800.60");
    add_proj4text (p, 2,
		   "96012192 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Colorado Central (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.75");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"standard_parallel_2\",38.45],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",37.83333333333334],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-105.5],PARAMETER[\"false_easting\",300000");
    add_srs_wkt (p, 12,
		 "0],PARAMETER[\"false_northing\",1000000],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"2232\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2233, "epsg", 2233,
		      "NAD83 / Colorado South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.66666666666666 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "288036576 +y_0=304800.6096012192 +ellps=GRS80 +datum=NAD");
    add_proj4text (p, 3, "83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Colorado South (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.43333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",37.23333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"latitude_of_origin\",36.666666666666");
    add_srs_wkt (p, 11,
		 "66],PARAMETER[\"central_meridian\",-105.5],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",3000000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 13,
		 "0000],AUTHORITY[\"EPSG\",\"2233\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2234, "epsg", 2234,
		      "NAD83 / Connecticut (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.86666666666667 +lat_2=41.2 +lat_0=40");
    add_proj4text (p, 1,
		   ".83333333333334 +lon_0=-72.75 +x_0=304800.6096012192 +y_");
    add_proj4text (p, 2,
		   "0=152400.3048006096 +ellps=GRS80 +datum=NAD83 +units=us-");
    add_proj4text (p, 3, "ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Connecticut (ftUS)\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_");
    add_srs_wkt (p, 8,
		 "Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.866666");
    add_srs_wkt (p, 9,
		 "66666667],PARAMETER[\"standard_parallel_2\",41.2],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"latitude_of_origin\",40.83333333333334],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"central_meridian\",-72.75],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",1000000],PARAMETER[\"false_northing\",500000],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"2234\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2235, "epsg", 2235,
		      "NAD83 / Delaware (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=-75.41666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "995 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Delaware (ftUS)\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",38],PARAMETER[\"centr");
    add_srs_wkt (p, 9,
		 "al_meridian\",-75.41666666666667],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.999995],PARAMETER[\"false_easting\",656166.667],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2235");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2236, "epsg", 2236,
		      "NAD83 / Florida East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-81 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +datum");
    add_proj4text (p, 2, "=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Florida East (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",24.33333333333333]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",-81],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.999941177],PARAMETER[\"false_easting\",656166.");
    add_srs_wkt (p, 11,
		 "667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2236\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2237, "epsg", 2237,
		      "NAD83 / Florida West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-82 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +datum");
    add_proj4text (p, 2, "=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Florida West (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",24.33333333333333]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",-82],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.999941177],PARAMETER[\"false_easting\",656166.");
    add_srs_wkt (p, 11,
		 "667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2237\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2238, "epsg", 2238,
		      "NAD83 / Florida North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.75 +lat_2=29.58333333333333 +lat_0=2");
    add_proj4text (p, 1,
		   "9 +lon_0=-84.5 +x_0=600000 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Florida North (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",30.75],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",29.58333333333333],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",29],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-84.5],PARAMETER[\"false_easting\",1968500],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2238\"]");
    add_srs_wkt (p, 13, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2239, "epsg", 2239,
		      "NAD83 / Georgia East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-82.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +datum=NAD8");
    add_proj4text (p, 2, "3 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Georgia East (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",30],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-82.16666666666667],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",656166.667],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"223");
    add_srs_wkt (p, 12, "9\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2240, "epsg", 2240,
		      "NAD83 / Georgia West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-84.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=699999.9998983998 +y_0=0 +ellps=GRS80 +datum=NAD8");
    add_proj4text (p, 2, "3 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Georgia West (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",30],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-84.16666666666667],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",2296583.333]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"22");
    add_srs_wkt (p, 12, "40\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2241, "epsg", 2241,
		      "NAD83 / Idaho East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-112.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999473679999999 +x_0=200000.0001016002 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Idaho East (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",41.66666666666666],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",-112.1666666666667],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.999947368],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",656166.667],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"2241\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 2242, "epsg", 2242,
		      "NAD83 / Idaho Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-114 +k=0.99");
    add_proj4text (p, 1,
		   "99473679999999 +x_0=500000.0001016001 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Idaho Central (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",41.66666666666666");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-114],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.999947368],PARAMETER[\"false_easting\",16404");
    add_srs_wkt (p, 11,
		 "16.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2242\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2243, "epsg", 2243,
		      "NAD83 / Idaho West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-115.75 +k=0");
    add_proj4text (p, 1,
		   ".999933333 +x_0=800000.0001016001 +y_0=0 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Idaho West (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",41.66666666666666],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",-115.75],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.999933333],PARAMETER[\"false_easting\",2624");
    add_srs_wkt (p, 11,
		 "666.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2243\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2244, "epsg", 2244,
		      "NAD83 / Indiana East (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=99999.99989839978 +y_0=249364.9987299975 +");
    add_proj4text (p, 2, "ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Indiana East (ftUS) (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4269\"]],UNIT[\"US survey foot\",0.30480060");
    add_srs_wkt (p, 7,
		 "96012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",37.5");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-85.66666666666667],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999966667],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",328083.333],PARAMETER[\"false_northing\",818125");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"2244\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 2245, "epsg", 2245,
		      "NAD83 / Indiana West (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=249364.9987299975 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Indiana West (ftUS) (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4269\"]],UNIT[\"US survey foot\",0.30480060");
    add_srs_wkt (p, 7,
		 "96012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",37.5");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-87.08333333333333],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999966667],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",2952750],PARAMETER[\"false_northing\",818125],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"2245\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 2246, "epsg", 2246,
		      "NAD83 / Kentucky North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=38.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=500000.0001016001 +y_");
    add_proj4text (p, 2, "0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kentucky North (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",37.96666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",38.96666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",37.5],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-84.25],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",1640416.667],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"2246\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2247, "epsg", 2247,
		      "NAD83 / Kentucky South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.93333333333333 +lat_2=36.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001016001 +y_0=500000.0001016001 +ellps=GRS80 +datum=NAD");
    add_proj4text (p, 3, "83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kentucky South (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",37.93333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",36.73333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"latitude_of_origin\",36.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-85.75],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",1640416.667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",1640416.667],AUTHORITY[\"EPSG\",\"2247\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 14, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2248, "epsg", 2248,
		      "NAD83 / Maryland (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.45 +lat_2=38.3 +lat_0=37.66666666666");
    add_proj4text (p, 1,
		   "666 +lon_0=-77 +x_0=399999.9998983998 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maryland (ftUS)\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 8,
		 "ic_2SP\"],PARAMETER[\"standard_parallel_1\",39.45],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_2\",38.3],PARAMETER[\"latitude_");
    add_srs_wkt (p, 10,
		 "of_origin\",37.66666666666666],PARAMETER[\"central_merid");
    add_srs_wkt (p, 11,
		 "ian\",-77],PARAMETER[\"false_easting\",1312333.333],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2248\"]");
    add_srs_wkt (p, 13, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2249, "epsg", 2249,
		      "NAD83 / Massachusetts Mainland (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.68333333333333 +lat_2=41.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41 +lon_0=-71.5 +x_0=200000.0001016002 +y_0=7");
    add_proj4text (p, 2,
		   "50000 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Massachusetts Mainland (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "42.68333333333333],PARAMETER[\"standard_parallel_2\",41.");
    add_srs_wkt (p, 10,
		 "71666666666667],PARAMETER[\"latitude_of_origin\",41],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"central_meridian\",-71.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",656166.667],PARAMETER[\"false_northing\",2460625]");
    add_srs_wkt (p, 13,
		 ",AUTHORITY[\"EPSG\",\"2249\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 2250, "epsg", 2250,
		      "NAD83 / Massachusetts Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.48333333333333 +lat_2=41.28333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-70.5 +x_0=500000.0001016001 +y_0=0");
    add_proj4text (p, 2, " +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Massachusetts Island (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "41.48333333333333],PARAMETER[\"standard_parallel_2\",41.");
    add_srs_wkt (p, 10,
		 "28333333333333],PARAMETER[\"latitude_of_origin\",41],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"central_meridian\",-70.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",1640416.667],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 13,
		 "ORITY[\"EPSG\",\"2250\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 2251, "epsg", 2251,
		      "NAD83 / Michigan North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.08333333333334 +lat_2=45.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=44.78333333333333 +lon_0=-87 +x_0=7999999.999");
    add_proj4text (p, 2,
		   "968001 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=ft +no_de");
    add_proj4text (p, 3, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Michigan North (ft)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",47.08333333333334],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",45.48333333333333],PARAMETER[\"lat");
    add_srs_wkt (p, 10,
		 "itude_of_origin\",44.78333333333333],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-87],PARAMETER[\"false_easting\",26246719.16");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "251\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2252, "epsg", 2252,
		      "NAD83 / Michigan Central (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.7 +lat_2=44.18333333333333 +lat_0=43");
    add_proj4text (p, 1,
		   ".31666666666667 +lon_0=-84.36666666666666 +x_0=5999999.9");
    add_proj4text (p, 2,
		   "99976001 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=ft +no_");
    add_proj4text (p, 3, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Michigan Central (ft)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",45.7],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",44.18333333333333],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",43.31666666666667],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 11,
		 "84.36666666666666],PARAMETER[\"false_easting\",19685039.");
    add_srs_wkt (p, 12,
		 "37],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "2252\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2253, "epsg", 2253,
		      "NAD83 / Michigan South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.66666666666666 +lat_2=42.1 +lat_0=41");
    add_proj4text (p, 1,
		   ".5 +lon_0=-84.36666666666666 +x_0=3999999.999984 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=GRS80 +datum=NAD83 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Michigan South (ft)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",43.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",42.1],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",41.5],PARAMETER[\"central_meridian\",-84.3666666666");
    add_srs_wkt (p, 11,
		 "6666],PARAMETER[\"false_easting\",13123359.58],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2253\"],AXIS");
    add_srs_wkt (p, 13, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2254, "epsg", 2254,
		      "NAD83 / Mississippi East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-88.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +datum=N");
    add_proj4text (p, 2, "AD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Mississippi East (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",29.5],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-88.83333333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.99995],PARAMETER[\"false_easting\",98425");
    add_srs_wkt (p, 11,
		 "0.0000000002],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2254\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2255, "epsg", 2255,
		      "NAD83 / Mississippi West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-90.33333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=699999.9998983998 +y_0=0 +ellps=GRS80 +datum=N");
    add_proj4text (p, 2, "AD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Mississippi West (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",29.5],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-90.33333333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.99995],PARAMETER[\"false_easting\",22965");
    add_srs_wkt (p, 11,
		 "83.333],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2255\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2256, "epsg", 2256, "NAD83 / Montana (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=45 +lat_0=44.25 +lon_0=-109.5");
    add_proj4text (p, 1,
		   " +x_0=599999.9999976 +y_0=0 +ellps=GRS80 +datum=NAD83 +u");
    add_proj4text (p, 2, "nits=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Montana (ft)\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",49],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 9,
		 "45],PARAMETER[\"latitude_of_origin\",44.25],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",-109.5],PARAMETER[\"false_easting\",1");
    add_srs_wkt (p, 11,
		 "968503.937],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"2256\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2257, "epsg", 2257,
		      "NAD83 / New Mexico East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-104.3333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "909091 +x_0=165000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Mexico East (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",31],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-104.3333333333333],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.999909091],PARAMETER[\"false_easting\",541");
    add_srs_wkt (p, 11,
		 "337.5],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2257\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2258, "epsg", 2258,
		      "NAD83 / New Mexico Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-106.25 +k=0.9999 +x_0=5000");
    add_proj4text (p, 1,
		   "00.0001016001 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us");
    add_proj4text (p, 2, "-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Mexico Central (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",31],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-106.25],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",1640416.667],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2258\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2259, "epsg", 2259,
		      "NAD83 / New Mexico West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-107.8333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "916667 +x_0=830000.0001016001 +y_0=0 +ellps=GRS80 +datum");
    add_proj4text (p, 2, "=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Mexico West (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",31],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-107.8333333333333],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.999916667],PARAMETER[\"false_easting\",272");
    add_srs_wkt (p, 11,
		 "3091.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 12, "SG\",\"2259\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2260, "epsg", 2260,
		      "NAD83 / New York East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York East (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",38.83333333333334");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-74.5],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.9999],PARAMETER[\"false_easting\",492125],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2260");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2261, "epsg", 2261,
		      "NAD83 / New York Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-76.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=249999.9998983998 +y_0=0 +ellps=GRS80 +datum=N");
    add_proj4text (p, 2, "AD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York Central (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",40],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-76.58333333333333],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.9999375],PARAMETER[\"false_easting\",82020");
    add_srs_wkt (p, 11,
		 "8.3330000002],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2261\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2262, "epsg", 2262,
		      "NAD83 / New York West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-78.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=350000.0001016001 +y_0=0 +ellps=GRS80 +datum=N");
    add_proj4text (p, 2, "AD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York West (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",40],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-78.58333333333333],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999375],PARAMETER[\"false_easting\",1148291.");
    add_srs_wkt (p, 11,
		 "667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2262\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2263, "epsg", 2263,
		      "NAD83 / New York Long Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.03333333333333 +lat_2=40.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=40.16666666666666 +lon_0=-74 +x_0=300000.0000");
    add_proj4text (p, 2,
		   "000001 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no");
    add_proj4text (p, 3, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York Long Island (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "41.03333333333333],PARAMETER[\"standard_parallel_2\",40.");
    add_srs_wkt (p, 10,
		 "66666666666666],PARAMETER[\"latitude_of_origin\",40.1666");
    add_srs_wkt (p, 11,
		 "6666666666],PARAMETER[\"central_meridian\",-74],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",984250.0000000002],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"2263\"],AXIS[\"X\",E");
    add_srs_wkt (p, 14, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2264, "epsg", 2264,
		      "NAD83 / North Carolina (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=33.75 +lon_0=-79 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2, "0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / North Carolina (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.16666");
    add_srs_wkt (p, 9,
		 "666666666],PARAMETER[\"standard_parallel_2\",34.33333333");
    add_srs_wkt (p, 10,
		 "333334],PARAMETER[\"latitude_of_origin\",33.75],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-79],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"2264\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2265, "epsg", 2265,
		      "NAD83 / North Dakota North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.43333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=47 +lon_0=-100.5 +x_0=599999.9999976 +y_0=0 +");
    add_proj4text (p, 2, "ellps=GRS80 +datum=NAD83 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / North Dakota North (ft)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "02\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"standard_parallel_1\",48.73333333333333],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"standard_parallel_2\",47.43333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",47],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 11,
		 "-100.5],PARAMETER[\"false_easting\",1968503.937],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2265\"],AX");
    add_srs_wkt (p, 13, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2266, "epsg", 2266,
		      "NAD83 / North Dakota South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.48333333333333 +lat_2=46.18333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=45.66666666666666 +lon_0=-100.5 +x_0=599999.9");
    add_proj4text (p, 2,
		   "999976 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=ft +no_de");
    add_proj4text (p, 3, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / North Dakota South (ft)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "02\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"standard_parallel_1\",47.48333333333333],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"standard_parallel_2\",46.18333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",45.66666666666666],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-100.5],PARAMETER[\"false_easting\",19685");
    add_srs_wkt (p, 12,
		 "03.937],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13, "\",\"2266\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2267, "epsg", 2267,
		      "NAD83 / Oklahoma North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.76666666666667 +lat_2=35.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=35 +lon_0=-98 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oklahoma North (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.76666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",35.56666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",35],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-98],PARAMETER[\"false_easting\",1968");
    add_srs_wkt (p, 12,
		 "500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 13, "\"2267\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2268, "epsg", 2268,
		      "NAD83 / Oklahoma South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.23333333333333 +lat_2=33.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.33333333333334 +lon_0=-98 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oklahoma South (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",35.23333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",33.93333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"latitude_of_origin\",33.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-98],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",1968500],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 13,
		 "HORITY[\"EPSG\",\"2268\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 2269, "epsg", 2269,
		      "NAD83 / Oregon North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46 +lat_2=44.33333333333334 +lat_0=43.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=2500000.0001424 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=GRS80 +datum=NAD83 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oregon North (ft)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"standard_parallel_1\",46],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 9,
		 "l_2\",44.33333333333334],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 10,
		 ",43.66666666666666],PARAMETER[\"central_meridian\",-120.");
    add_srs_wkt (p, 11,
		 "5],PARAMETER[\"false_easting\",8202099.738],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2269\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2270, "epsg", 2270,
		      "NAD83 / Oregon South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44 +lat_2=42.33333333333334 +lat_0=41.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=1500000.0001464 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=GRS80 +datum=NAD83 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oregon South (ft)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"standard_parallel_1\",44],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 9,
		 "l_2\",42.33333333333334],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 10,
		 ",41.66666666666666],PARAMETER[\"central_meridian\",-120.");
    add_srs_wkt (p, 11,
		 "5],PARAMETER[\"false_easting\",4921259.843],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2270\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2271, "epsg", 2271,
		      "NAD83 / Pennsylvania North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.95 +lat_2=40.88333333333333 +lat_0=4");
    add_proj4text (p, 1,
		   "0.16666666666666 +lon_0=-77.75 +x_0=600000 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Pennsylvania North (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.");
    add_srs_wkt (p, 9,
		 "95],PARAMETER[\"standard_parallel_2\",40.88333333333333]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"latitude_of_origin\",40.16666666666666],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"central_meridian\",-77.75],PARAMETER[\"false_ea");
    add_srs_wkt (p, 12,
		 "sting\",1968500],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"2271\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 14, "]");
    p = add_epsg_def (first, last, 2272, "epsg", 2272,
		      "NAD83 / Pennsylvania South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.96666666666667 +lat_2=39.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=39.33333333333334 +lon_0=-77.75 +x_0=600000 +");
    add_proj4text (p, 2,
		   "y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Pennsylvania South (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.");
    add_srs_wkt (p, 9,
		 "96666666666667],PARAMETER[\"standard_parallel_2\",39.933");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"latitude_of_origin\",39.3333333");
    add_srs_wkt (p, 11,
		 "3333334],PARAMETER[\"central_meridian\",-77.75],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"2272\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2273, "epsg", 2273,
		      "NAD83 / South Carolina (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.83333333333334 +lat_2=32.5 +lat_0=31");
    add_proj4text (p, 1,
		   ".83333333333333 +lon_0=-81 +x_0=609600 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / South Carolina (ft)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",34.83333333333334],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",32.5],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",31.83333333333333],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 11,
		 "81],PARAMETER[\"false_easting\",2000000],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"2273\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_02 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2274, "epsg", 2274,
		      "NAD83 / Tennessee (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.41666666666666 +lat_2=35.25 +lat_0=3");
    add_proj4text (p, 1,
		   "4.33333333333334 +lon_0=-86 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Tennessee (ftUS)\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",36.41666666");
    add_srs_wkt (p, 9,
		 "666666],PARAMETER[\"standard_parallel_2\",35.25],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"latitude_of_origin\",34.33333333333334],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-86],PARAMETER[\"false_easting\",1968");
    add_srs_wkt (p, 12,
		 "500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 13, "\"2274\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2275, "epsg", 2275,
		      "NAD83 / Texas North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.18333333333333 +lat_2=34.65 +lat_0=3");
    add_proj4text (p, 1,
		   "4 +lon_0=-101.5 +x_0=200000.0001016002 +y_0=999999.99989");
    add_proj4text (p, 2,
		   "83998 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas North (ftUS)\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_");
    add_srs_wkt (p, 8,
		 "Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.183333");
    add_srs_wkt (p, 9,
		 "33333333],PARAMETER[\"standard_parallel_2\",34.65],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",34],PARAMETER[\"central_meri");
    add_srs_wkt (p, 11,
		 "dian\",-101.5],PARAMETER[\"false_easting\",656166.667],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",3280833.333],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"2275\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2276, "epsg", 2276,
		      "NAD83 / Texas North Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.96666666666667 +lat_2=32.13333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=31.66666666666667 +lon_0=-98.5 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=2000000.0001016 +ellps=GRS80 +datum=NAD83 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas North Central (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4269\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 9,
		 "3.96666666666667],PARAMETER[\"standard_parallel_2\",32.1");
    add_srs_wkt (p, 10,
		 "3333333333333],PARAMETER[\"latitude_of_origin\",31.66666");
    add_srs_wkt (p, 11,
		 "666666667],PARAMETER[\"central_meridian\",-98.5],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",1968500],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",6561666.667],AUTHORITY[\"EPSG\",\"2276\"],AXIS[\"X\",");
    add_srs_wkt (p, 14, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2277, "epsg", 2277,
		      "NAD83 / Texas Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=31.88333333333333 +lat_2=30.11666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=29.66666666666667 +lon_0=-100.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=699999.9998983998 +y_0=3000000 +ellps=GRS80 +datum=N");
    add_proj4text (p, 3, "AD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas Central (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",31.88333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",30.11666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",29.666666666666");
    add_srs_wkt (p, 11,
		 "67],PARAMETER[\"central_meridian\",-100.3333333333333],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_easting\",2296583.333],PARAMETER[\"fals");
    add_srs_wkt (p, 13,
		 "e_northing\",9842500.000000002],AUTHORITY[\"EPSG\",\"227");
    add_srs_wkt (p, 14, "7\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2278, "epsg", 2278,
		      "NAD83 / Texas South Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.28333333333333 +lat_2=28.38333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=27.83333333333333 +lon_0=-99 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=3999999.9998984 +ellps=GRS80 +datum=NAD83 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas South Central (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4269\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 9,
		 "0.28333333333333],PARAMETER[\"standard_parallel_2\",28.3");
    add_srs_wkt (p, 10,
		 "8333333333333],PARAMETER[\"latitude_of_origin\",27.83333");
    add_srs_wkt (p, 11,
		 "333333333],PARAMETER[\"central_meridian\",-99],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",13123333.333],AUTHORITY[\"EPSG\",\"2278\"],AXIS[\"X\",E");
    add_srs_wkt (p, 14, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2279, "epsg", 2279,
		      "NAD83 / Texas South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.66666666666667 +lon_0=-98.5 +x_0=300000.00");
    add_proj4text (p, 2,
		   "00000001 +y_0=5000000.0001016 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 3, "+units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas South (ftUS)\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_");
    add_srs_wkt (p, 8,
		 "Conic_2SP\"],PARAMETER[\"standard_parallel_1\",27.833333");
    add_srs_wkt (p, 9,
		 "33333333],PARAMETER[\"standard_parallel_2\",26.166666666");
    add_srs_wkt (p, 10,
		 "66667],PARAMETER[\"latitude_of_origin\",25.6666666666666");
    add_srs_wkt (p, 11,
		 "7],PARAMETER[\"central_meridian\",-98.5],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_easting\",984250.0000000002],PARAMETER[\"false_northi");
    add_srs_wkt (p, 13,
		 "ng\",16404166.667],AUTHORITY[\"EPSG\",\"2279\"],AXIS[\"X");
    add_srs_wkt (p, 14, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2280, "epsg", 2280,
		      "NAD83 / Utah North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001504 +y_0=999999.9999960001 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 3, "+units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah North (ft)\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",41.78333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_2\",40.71666666666667],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",40.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 11,
		 "ridian\",-111.5],PARAMETER[\"false_easting\",1640419.948");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",3280839.895],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2280\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2281, "epsg", 2281,
		      "NAD83 / Utah Central (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000.0001504 +y_0=");
    add_proj4text (p, 2,
		   "1999999.999992 +ellps=GRS80 +datum=NAD83 +units=ft +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah Central (ft)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"standard_parallel_1\",40.65],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",39.01666666666667],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",38.33333333333334],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 11,
		 "111.5],PARAMETER[\"false_easting\",1640419.948],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_northing\",6561679.79],AUTHORITY[\"EPSG\",\"22");
    add_srs_wkt (p, 13, "81\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2282, "epsg", 2282,
		      "NAD83 / Utah South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000.0001504 +y_0=");
    add_proj4text (p, 2,
		   "2999999.999988 +ellps=GRS80 +datum=NAD83 +units=ft +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah South (ft)\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",38.35],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 9,
		 "el_2\",37.21666666666667],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 10,
		 "\",36.66666666666666],PARAMETER[\"central_meridian\",-11");
    add_srs_wkt (p, 11,
		 "1.5],PARAMETER[\"false_easting\",1640419.948],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_northing\",9842519.685],AUTHORITY[\"EPSG\",\"228");
    add_srs_wkt (p, 13, "2\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2283, "epsg", 2283,
		      "NAD83 / Virginia North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.2 +lat_2=38.03333333333333 +lat_0=37");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-78.5 +x_0=3500000.0001016 +y_0=2");
    add_proj4text (p, 2,
		   "000000.0001016 +ellps=GRS80 +datum=NAD83 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Virginia North (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.2],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"standard_parallel_2\",38.03333333333333],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",37.66666666666666],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-78.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",11482916.667],PARAMETER[\"false_northing\",6561666.667]");
    add_srs_wkt (p, 13,
		 ",AUTHORITY[\"EPSG\",\"2283\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 2284, "epsg", 2284,
		      "NAD83 / Virginia South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=36.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.33333333333334 +lon_0=-78.5 +x_0=3500000.0");
    add_proj4text (p, 2,
		   "001016 +y_0=999999.9998983998 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 3, "+units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Virginia South (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",37.96666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",36.76666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",36.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-78.5],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",11482916.667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",3280833.333],AUTHORITY[\"EPSG\",\"2284\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 14, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2285, "epsg", 2285,
		      "NAD83 / Washington North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.5 +lat_0=47");
    add_proj4text (p, 1,
		   " +lon_0=-120.8333333333333 +x_0=500000.0001016001 +y_0=0");
    add_proj4text (p, 2, " +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Washington North (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",48.73");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",47.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",47],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-120.8333333333333],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",1640416.667],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"2285\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2286, "epsg", 2286,
		      "NAD83 / Washington South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.33333333333334 +lat_2=45.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=45.33333333333334 +lon_0=-120.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001016001 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Washington South (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",47.33");
    add_srs_wkt (p, 9,
		 "333333333334],PARAMETER[\"standard_parallel_2\",45.83333");
    add_srs_wkt (p, 10,
		 "333333334],PARAMETER[\"latitude_of_origin\",45.333333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-120.5],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",1640416.667],PARAMETER[\"false_northin");
    add_srs_wkt (p, 13,
		 "g\",0],AUTHORITY[\"EPSG\",\"2286\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2287, "epsg", 2287,
		      "NAD83 / Wisconsin North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.76666666666667 +lat_2=45.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=45.16666666666666 +lon_0=-90 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wisconsin North (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",46.76");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",45.56666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"latitude_of_origin\",45.166666666");
    add_srs_wkt (p, 11,
		 "66666],PARAMETER[\"central_meridian\",-90],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",1968500],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"2287\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 2288, "epsg", 2288,
		      "NAD83 / Wisconsin Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.5 +lat_2=44.25 +lat_0=43.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wisconsin Central (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",45.");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"standard_parallel_2\",44.25],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",43.83333333333334],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-90],PARAMETER[\"false_easting\",1968500]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"22");
    add_srs_wkt (p, 13, "88\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2289, "epsg", 2289,
		      "NAD83 / Wisconsin South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.06666666666667 +lat_2=42.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=42 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wisconsin South (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",44.06");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",42.73333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",42],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-90],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "1968500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"2289\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2290, "epsg", 2290,
		      "ATS77 / Prince Edward Isl. Stereographic (ATS77)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=47.25 +lon_0=-63 +k=0.999912 +x_0=70");
    add_proj4text (p, 1,
		   "0000 +y_0=400000 +a=6378135 +b=6356750.304921594 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ATS77 / Prince Edward Isl. Stereographic (ATS77");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"ATS77\",DATUM[\"Average_Terrestrial_System_");
    add_srs_wkt (p, 2,
		 "1977\",SPHEROID[\"Average Terrestrial System 1977\",6378");
    add_srs_wkt (p, 3,
		 "135,298.257,AUTHORITY[\"EPSG\",\"7041\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6122\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4122\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Oblique_Stereographic\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",47.25],PARAMETER[\"central_meridian\",-63],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.999912],PARAMETER[\"false_easting\",700");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",400000],AUTHORITY[\"EP");
    add_srs_wkt (p, 12,
		 "SG\",\"2290\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 2291, "epsg", 2291,
		      "NAD83(CSRS98) / Prince Edward Isl. Stereographic (NAD83) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=47.25 +lon_0=-63 +k=0.999912 +x_0=40");
    add_proj4text (p, 1,
		   "0000 +y_0=800000 +a=6378135 +b=6356750.304921594 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / Prince Edward Isl. Stereographi");
    add_srs_wkt (p, 1,
		 "c (NAD83) (deprecated)\",GEOGCS[\"ATS77\",DATUM[\"Averag");
    add_srs_wkt (p, 2,
		 "e_Terrestrial_System_1977\",SPHEROID[\"Average Terrestri");
    add_srs_wkt (p, 3,
		 "al System 1977\",6378135,298.257,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 4,
		 "41\"]],AUTHORITY[\"EPSG\",\"6122\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4122\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 8,
		 "001\"]],PROJECTION[\"Oblique_Stereographic\"],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"latitude_of_origin\",47.25],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",-63],PARAMETER[\"scale_factor\",0.999912],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",400000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 12,
		 "800000],AUTHORITY[\"EPSG\",\"2291\"],AXIS[\"E(X)\",EAST]");
    add_srs_wkt (p, 13, ",AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2292, "epsg", 2292,
		      "NAD83(CSRS98) / Prince Edward Isl. Stereographic (NAD83) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=47.25 +lon_0=-63 +k=0.999912 +x_0=40");
    add_proj4text (p, 1,
		   "0000 +y_0=800000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +un");
    add_proj4text (p, 2, "its=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS98) / Prince Edward Isl. Stereographi");
    add_srs_wkt (p, 1,
		 "c (NAD83) (deprecated)\",GEOGCS[\"NAD83(CSRS98)\",DATUM[");
    add_srs_wkt (p, 2,
		 "\"NAD83_Canadian_Spatial_Reference_System\",SPHEROID[\"G");
    add_srs_wkt (p, 3,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 4,
		 "9\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6140\"");
    add_srs_wkt (p, 5,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 6,
		 "UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9108\"]],AUTHORITY[\"EPSG\",\"4140\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 8,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Oblique_Stere");
    add_srs_wkt (p, 9,
		 "ographic\"],PARAMETER[\"latitude_of_origin\",47.25],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"central_meridian\",-63],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 11,
		 "\",0.999912],PARAMETER[\"false_easting\",400000],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_northing\",800000],AUTHORITY[\"EPSG\",\"2292\"");
    add_srs_wkt (p, 13, "],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2294, "epsg", 2294,
		      "ATS77 / MTM Nova Scotia zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-61.5 +k=0.9999 +x_0=4500000");
    add_proj4text (p, 1,
		   " +y_0=0 +a=6378135 +b=6356750.304921594 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ATS77 / MTM Nova Scotia zone 4\",GEOGCS[\"ATS77");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Average_Terrestrial_System_1977\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Average Terrestrial System 1977\",6378135,298.257,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7041\"]],AUTHORITY[\"EPSG\",\"6122\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4122\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-61.5],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "9],PARAMETER[\"false_easting\",4500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2294\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2295, "epsg", 2295,
		      "ATS77 / MTM Nova Scotia zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-64.5 +k=0.9999 +x_0=5500000");
    add_proj4text (p, 1,
		   " +y_0=0 +a=6378135 +b=6356750.304921594 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ATS77 / MTM Nova Scotia zone 5\",GEOGCS[\"ATS77");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Average_Terrestrial_System_1977\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Average Terrestrial System 1977\",6378135,298.257,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7041\"]],AUTHORITY[\"EPSG\",\"6122\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4122\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-64.5],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "9],PARAMETER[\"false_easting\",5500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2295\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2308, "epsg", 2308, "Batavia / TM 109 SE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=109 +k=0.9996 +x_0=500000 +y");
    add_proj4text (p, 1, "_0=10000000 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Batavia / TM 109 SE\",GEOGCS[\"Batavia\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"Batavia\",SPHEROID[\"Bessel 1841\",6377397.155,299.152");
    add_srs_wkt (p, 2,
		 "8128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6211\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4211\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",109],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"2308");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2309, "epsg", 2309, "WGS 84 / TM 116 SE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=116 +k=0.9996 +x_0=500000 +y");
    add_proj4text (p, 1,
		   "_0=10000000 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / TM 116 SE\",GEOGCS[\"WGS 84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 4,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 5,
		 "122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 6,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 7,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "central_meridian\",116],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"2309\"],AXIS[");
    add_srs_wkt (p, 11, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2310, "epsg", 2310, "WGS 84 / TM 132 SE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=0.9996 +x_0=500000 +y");
    add_proj4text (p, 1,
		   "_0=10000000 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / TM 132 SE\",GEOGCS[\"WGS 84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 4,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 5,
		 "122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 6,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 7,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "central_meridian\",132],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"2310\"],AXIS[");
    add_srs_wkt (p, 11, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2311, "epsg", 2311, "WGS 84 / TM 6 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=6 +k=0.9996 +x_0=500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / TM 6 NE\",GEOGCS[\"WGS 84\",DATUM[\"WG");
    add_srs_wkt (p, 1,
		 "S_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5,
		 "2\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 7,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 8,
		 "entral_meridian\",6],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"2311\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2312, "epsg", 2312, "Garoua / UTM zone 33N");
    add_proj4text (p, 0, "+proj=utm +zone=33 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Garoua / UTM zone 33N\",GEOGCS[\"Garoua\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Garoua\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,2");
    add_srs_wkt (p, 2,
		 "93.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6197\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4197\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",15],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2312\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2313, "epsg", 2313,
		      "Kousseri / UTM zone 33N");
    add_proj4text (p, 0, "+proj=utm +zone=33 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kousseri / UTM zone 33N\",GEOGCS[\"Kousseri\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Kousseri\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6198\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4198\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",15],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2313\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2314, "epsg", 2314,
		      "Trinidad 1903 / Trinidad Grid (ftCla)");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=10.44166666666667 +lon_0=-61.333333333");
    add_proj4text (p, 1,
		   "33334 +x_0=86501.46392052001 +y_0=65379.0134283 +a=63782");
    add_proj4text (p, 2,
		   "93.645208759 +b=6356617.987679838 +to_meter=0.3047972654");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Trinidad 1903 / Trinidad Grid (ftCla)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Trinidad 1903\",DATUM[\"Trinidad_1903\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1858\",6378293.645208759,294.2606763692569,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7007\"]],AUTHORITY[\"EPSG\",\"6302\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4302\"]],UNIT[\"Clarke's foot\",0");
    add_srs_wkt (p, 7,
		 ".3047972654,AUTHORITY[\"EPSG\",\"9005\"]],PROJECTION[\"C");
    add_srs_wkt (p, 8,
		 "assini_Soldner\"],PARAMETER[\"latitude_of_origin\",10.44");
    add_srs_wkt (p, 9,
		 "166666666667],PARAMETER[\"central_meridian\",-61.3333333");
    add_srs_wkt (p, 10,
		 "3333334],PARAMETER[\"false_easting\",283800],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",214500],AUTHORITY[\"EPSG\",\"2314\"],AX");
    add_srs_wkt (p, 12, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2315, "epsg", 2315,
		      "Campo Inchauspe / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / UTM zone 19S\",GEOGCS[\"Campo");
    add_srs_wkt (p, 1,
		 " Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-69],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"2315\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2316, "epsg", 2316,
		      "Campo Inchauspe / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / UTM zone 20S\",GEOGCS[\"Campo");
    add_srs_wkt (p, 1,
		 " Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-63],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"2316\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2317, "epsg", 2317, "PSAD56 / ICN Regional");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=9 +lat_2=3 +lat_0=6 +lon_0=-66 +x_0=100");
    add_proj4text (p, 1, "0000 +y_0=1000000 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / ICN Regional\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"standard_parallel_1\",9],PARAMETER[\"standard_par");
    add_srs_wkt (p, 9,
		 "allel_2\",3],PARAMETER[\"latitude_of_origin\",6],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"central_meridian\",-66],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",1000000],PARAMETER[\"false_northing\",1000000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"2317\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 2318, "epsg", 2318,
		      "Ain el Abd / Aramco Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=17 +lat_2=33 +lat_0=25.08951 +lon_0=48 ");
    add_proj4text (p, 1, "+x_0=0 +y_0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Ain el Abd / Aramco Lambert\",GEOGCS[\"Ain el A");
    add_srs_wkt (p, 1,
		 "bd\",DATUM[\"Ain_el_Abd_1970\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4204\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",17],PARAMETER[\"standard_parallel_2\",33");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",25.08951],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",48],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2318\"");
    add_srs_wkt (p, 12, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2319, "epsg", 2319, "ED50 / TM27");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM27\",GEOGCS[\"ED50\",DATUM[\"European_");
    add_srs_wkt (p, 1,
		 "Datum_1950\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6230");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",27],PARAMETER[\"scale_factor\",1");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2319\"],AXIS[\"X\",NO");
    add_srs_wkt (p, 11, "RTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2320, "epsg", 2320, "ED50 / TM30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM30\",GEOGCS[\"ED50\",DATUM[\"European_");
    add_srs_wkt (p, 1,
		 "Datum_1950\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6230");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",30],PARAMETER[\"scale_factor\",1");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2320\"],AXIS[\"X\",NO");
    add_srs_wkt (p, 11, "RTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2321, "epsg", 2321, "ED50 / TM33");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM33\",GEOGCS[\"ED50\",DATUM[\"European_");
    add_srs_wkt (p, 1,
		 "Datum_1950\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6230");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",33],PARAMETER[\"scale_factor\",1");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2321\"],AXIS[\"X\",NO");
    add_srs_wkt (p, 11, "RTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2322, "epsg", 2322, "ED50 / TM36");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=36 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM36\",GEOGCS[\"ED50\",DATUM[\"European_");
    add_srs_wkt (p, 1,
		 "Datum_1950\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6230");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",36],PARAMETER[\"scale_factor\",1");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2322\"],AXIS[\"X\",NO");
    add_srs_wkt (p, 11, "RTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2323, "epsg", 2323, "ED50 / TM39");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM39\",GEOGCS[\"ED50\",DATUM[\"European_");
    add_srs_wkt (p, 1,
		 "Datum_1950\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6230");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",39],PARAMETER[\"scale_factor\",1");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2323\"],AXIS[\"X\",NO");
    add_srs_wkt (p, 11, "RTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2324, "epsg", 2324, "ED50 / TM42");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=42 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM42\",GEOGCS[\"ED50\",DATUM[\"European_");
    add_srs_wkt (p, 1,
		 "Datum_1950\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6230");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",42],PARAMETER[\"scale_factor\",1");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2324\"],AXIS[\"X\",NO");
    add_srs_wkt (p, 11, "RTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2325, "epsg", 2325, "ED50 / TM45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM45\",GEOGCS[\"ED50\",DATUM[\"European_");
    add_srs_wkt (p, 1,
		 "Datum_1950\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6230");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",45],PARAMETER[\"scale_factor\",1");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2325\"],AXIS[\"X\",NO");
    add_srs_wkt (p, 11, "RTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2326, "epsg", 2326,
		      "Hong Kong 1980 Grid System");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=22.31213333333334 +lon_0=114.17855555");
    add_proj4text (p, 1,
		   "55556 +k=1 +x_0=836694.05 +y_0=819069.8 +ellps=intl +tow");
    add_proj4text (p, 2,
		   "gs84=-162.619,-276.959,-161.764,0.067753,-2.24365,-1.158");
    add_proj4text (p, 3, "83,-1.09425 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hong Kong 1980 Grid System\",GEOGCS[\"Hong Kong");
    add_srs_wkt (p, 1,
		 " 1980\",DATUM[\"Hong_Kong_1980\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 2,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS");
    add_srs_wkt (p, 3,
		 "84[-162.619,-276.959,-161.764,0.067753,-2.24365,-1.15883");
    add_srs_wkt (p, 4,
		 ",-1.09425],AUTHORITY[\"EPSG\",\"6611\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4611\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"latitude_of_origin\",22.31213333333334],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",114.1785555555556],PARAMETER[\"scale_");
    add_srs_wkt (p, 11,
		 "factor\",1],PARAMETER[\"false_easting\",836694.05],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_northing\",819069.8],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13,
		 "326\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 2327, "epsg", 2327,
		      "Xian 1980 / Gauss-Kruger zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=13500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 13\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",75],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",13500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2327\"],A");
    add_srs_wkt (p, 11, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2328, "epsg", 2328,
		      "Xian 1980 / Gauss-Kruger zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=14500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 14\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",81],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",14500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2328\"],A");
    add_srs_wkt (p, 11, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2329, "epsg", 2329,
		      "Xian 1980 / Gauss-Kruger zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=15500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 15\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",87],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",15500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2329\"],A");
    add_srs_wkt (p, 11, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2330, "epsg", 2330,
		      "Xian 1980 / Gauss-Kruger zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=16500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 16\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",93],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",16500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2330\"],A");
    add_srs_wkt (p, 11, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2331, "epsg", 2331,
		      "Xian 1980 / Gauss-Kruger zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=17500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 17\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",99],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",17500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2331\"],A");
    add_srs_wkt (p, 11, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2332, "epsg", 2332,
		      "Xian 1980 / Gauss-Kruger zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=18500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 18\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",105],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",18500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2332\"],");
    add_srs_wkt (p, 11, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2333, "epsg", 2333,
		      "Xian 1980 / Gauss-Kruger zone 19");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=19500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 19\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",111],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",19500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2333\"],");
    add_srs_wkt (p, 11, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2334, "epsg", 2334,
		      "Xian 1980 / Gauss-Kruger zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=20500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 20\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",117],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",20500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2334\"],");
    add_srs_wkt (p, 11, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2335, "epsg", 2335,
		      "Xian 1980 / Gauss-Kruger zone 21");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=21500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 21\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",123],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",21500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2335\"],");
    add_srs_wkt (p, 11, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2336, "epsg", 2336,
		      "Xian 1980 / Gauss-Kruger zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=22500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 22\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",129],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",22500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2336\"],");
    add_srs_wkt (p, 11, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2337, "epsg", 2337,
		      "Xian 1980 / Gauss-Kruger zone 23");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=23500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger zone 23\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",135],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",23500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2337\"],");
    add_srs_wkt (p, 11, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2338, "epsg", 2338,
		      "Xian 1980 / Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 75E\",GEOGCS[\"Xian");
    add_srs_wkt (p, 1,
		 " 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",63781");
    add_srs_wkt (p, 2,
		 "40,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",75],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2338\"],AXI");
    add_srs_wkt (p, 11, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2339, "epsg", 2339,
		      "Xian 1980 / Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 81E\",GEOGCS[\"Xian");
    add_srs_wkt (p, 1,
		 " 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",63781");
    add_srs_wkt (p, 2,
		 "40,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",81],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2339\"],AXI");
    add_srs_wkt (p, 11, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2340, "epsg", 2340,
		      "Xian 1980 / Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 87E\",GEOGCS[\"Xian");
    add_srs_wkt (p, 1,
		 " 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",63781");
    add_srs_wkt (p, 2,
		 "40,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",87],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2340\"],AXI");
    add_srs_wkt (p, 11, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2341, "epsg", 2341,
		      "Xian 1980 / Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 93E\",GEOGCS[\"Xian");
    add_srs_wkt (p, 1,
		 " 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",63781");
    add_srs_wkt (p, 2,
		 "40,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",93],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2341\"],AXI");
    add_srs_wkt (p, 11, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2342, "epsg", 2342,
		      "Xian 1980 / Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 99E\",GEOGCS[\"Xian");
    add_srs_wkt (p, 1,
		 " 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",63781");
    add_srs_wkt (p, 2,
		 "40,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",99],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",1],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2342\"],AXI");
    add_srs_wkt (p, 11, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2343, "epsg", 2343,
		      "Xian 1980 / Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 105E\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",105],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2343\"],AX");
    add_srs_wkt (p, 11, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2344, "epsg", 2344,
		      "Xian 1980 / Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 111E\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",111],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2344\"],AX");
    add_srs_wkt (p, 11, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2345, "epsg", 2345,
		      "Xian 1980 / Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 117E\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",117],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2345\"],AX");
    add_srs_wkt (p, 11, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2346, "epsg", 2346,
		      "Xian 1980 / Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 123E\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",123],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2346\"],AX");
    add_srs_wkt (p, 11, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2347, "epsg", 2347,
		      "Xian 1980 / Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 129E\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",129],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2347\"],AX");
    add_srs_wkt (p, 11, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2348, "epsg", 2348,
		      "Xian 1980 / Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / Gauss-Kruger CM 135E\",GEOGCS[\"Xia");
    add_srs_wkt (p, 1,
		 "n 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 1980\",6378");
    add_srs_wkt (p, 2,
		 "140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",135],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2348\"],AX");
    add_srs_wkt (p, 11, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2349, "epsg", 2349,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 25");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=25500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 25\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",75],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",255000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2349\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2350, "epsg", 2350,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=26500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 26\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",78],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",265000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2350\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2351, "epsg", 2351,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 27");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=27500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 27\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",81],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",275000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2351\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2352, "epsg", 2352,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=28500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 28\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",84],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",285000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2352\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2353, "epsg", 2353,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 29");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=29500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 29\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",87],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",295000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2353\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2354, "epsg", 2354,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=30500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 30\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",90],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",305000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2354\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2355, "epsg", 2355,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=31500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 31\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",93],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",315000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2355\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2356, "epsg", 2356,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 32");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=32500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 32\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",96],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",325000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2356\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2357, "epsg", 2357,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 33");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=33500000 +y_0=0");
    add_proj4text (p, 1, " +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 33\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",99],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",335000");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2357\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2358, "epsg", 2358,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 34");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=34500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 34\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",102],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",34500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2358\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2359, "epsg", 2359,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 35");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=35500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 35\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",105],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",35500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2359\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2360, "epsg", 2360,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 36");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=36500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 36\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",108],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",36500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2360\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2361, "epsg", 2361,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 37");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=37500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 37\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",111],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",37500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2361\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2362, "epsg", 2362,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 38");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=38500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 38\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",114],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",38500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2362\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2363, "epsg", 2363,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 39");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=39500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 39\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",117],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",39500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2363\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2364, "epsg", 2364,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 40");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=40500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 40\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",120],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",40500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2364\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2365, "epsg", 2365,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 41");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=41500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 41\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",123],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",41500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2365\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2366, "epsg", 2366,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 42");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=42500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 42\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",126],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",42500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2366\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2367, "epsg", 2367,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 43");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=43500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 43\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",129],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",43500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2367\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2368, "epsg", 2368,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 44");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=44500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 44\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",132],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",44500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2368\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2369, "epsg", 2369,
		      "Xian 1980 / 3-degree Gauss-Kruger zone 45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=45500000 +y_0=");
    add_proj4text (p, 1, "0 +a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger zone 45\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",135],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",45500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"2369\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2370, "epsg", 2370,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 75E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",75],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "70\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2371, "epsg", 2371,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 78E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 78E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",78],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "71\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2372, "epsg", 2372,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 81E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",81],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "72\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2373, "epsg", 2373,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 84E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 84E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",84],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "73\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2374, "epsg", 2374,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 87E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",87],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "74\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2375, "epsg", 2375,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 90E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 90E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",90],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "75\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2376, "epsg", 2376,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 93E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",93],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "76\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2377, "epsg", 2377,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 96E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 96E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",96],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "77\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2378, "epsg", 2378,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 99E\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 198");
    add_srs_wkt (p, 2,
		 "0\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4610");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",99],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23");
    add_srs_wkt (p, 11, "78\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2379, "epsg", 2379,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 102E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 102E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",102],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2379\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2380, "epsg", 2380,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 105E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",105],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2380\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2381, "epsg", 2381,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 108E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 108E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",108],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2381\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2382, "epsg", 2382,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 111E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",111],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2382\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2383, "epsg", 2383,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 114E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 114E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",114],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2383\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2384, "epsg", 2384,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 117E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",117],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2384\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2385, "epsg", 2385,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 120E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 120E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",120],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2385\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2386, "epsg", 2386,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 123E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",123],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2386\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2387, "epsg", 2387,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 126E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 126E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",126],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2387\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2388, "epsg", 2388,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 129E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",129],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2388\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2389, "epsg", 2389,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 132E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 132E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",132],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2389\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2390, "epsg", 2390,
		      "Xian 1980 / 3-degree Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+a=6378140 +b=6356755.288157528 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Xian 1980 / 3-degree Gauss-Kruger CM 135E\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian 19");
    add_srs_wkt (p, 2,
		 "80\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"461");
    add_srs_wkt (p, 6,
		 "0\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",135],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "2390\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2391, "epsg", 2391, "KKJ / Finland zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=1500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KKJ / Finland zone 1\",GEOGCS[\"KKJ\",DATUM[\"K");
    add_srs_wkt (p, 1,
		 "artastokoordinaattijarjestelma_1966\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6123\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4123\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",21],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",1");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"2391\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2392, "epsg", 2392, "KKJ / Finland zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=2500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KKJ / Finland zone 2\",GEOGCS[\"KKJ\",DATUM[\"K");
    add_srs_wkt (p, 1,
		 "artastokoordinaattijarjestelma_1966\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6123\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4123\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",24],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",2");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"2392\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2393, "epsg", 2393,
		      "KKJ / Finland Uniform Coordinate System");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=3500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KKJ / Finland Uniform Coordinate System\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"KKJ\",DATUM[\"Kartastokoordinaattijarjestelma_1966\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6123\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4123\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",27],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",3500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2393\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 2394, "epsg", 2394, "KKJ / Finland zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KKJ / Finland zone 4\",GEOGCS[\"KKJ\",DATUM[\"K");
    add_srs_wkt (p, 1,
		 "artastokoordinaattijarjestelma_1966\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6123\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4123\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",30],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",4");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"2394\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2395, "epsg", 2395,
		      "South Yemen / Gauss-Kruger zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=-76,-138,67,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"South Yemen / Gauss-Kruger zone 8\",GEOGCS[\"So");
    add_srs_wkt (p, 1,
		 "uth Yemen\",DATUM[\"South_Yemen\",SPHEROID[\"Krassowsky ");
    add_srs_wkt (p, 2,
		 "1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS");
    add_srs_wkt (p, 3,
		 "84[-76,-138,67,0,0,0,0],AUTHORITY[\"EPSG\",\"6164\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4164\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",45],PARAMETER[\"scale_factor\",1],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",8500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"2395\"],AXIS[\"X\",NORTH],A");
    add_srs_wkt (p, 12, "XIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2396, "epsg", 2396,
		      "South Yemen / Gauss-Kruger zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=-76,-138,67,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"South Yemen / Gauss-Kruger zone 9\",GEOGCS[\"So");
    add_srs_wkt (p, 1,
		 "uth Yemen\",DATUM[\"South_Yemen\",SPHEROID[\"Krassowsky ");
    add_srs_wkt (p, 2,
		 "1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS");
    add_srs_wkt (p, 3,
		 "84[-76,-138,67,0,0,0,0],AUTHORITY[\"EPSG\",\"6164\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4164\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",51],PARAMETER[\"scale_factor\",1],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",9500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"2396\"],AXIS[\"X\",NORTH],A");
    add_srs_wkt (p, 12, "XIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2397, "epsg", 2397,
		      "Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=3500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 3");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",9],PARAMETER[\"scale_factor\",1],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",3500000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"2397\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 2398, "epsg", 2398,
		      "Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 4");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",12],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",4500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2398\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 2399, "epsg", 2399,
		      "Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 5");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",15],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",5500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"2399\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 2400, "epsg", 2400,
		      "RT90 2.5 gon W (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15.80827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT90 2.5 gon W (deprecated)\",GEOGCS[\"RT90\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Rikets_koordinatsystem_1990\",SPHEROID[\"Bessel 1");
    add_srs_wkt (p, 2,
		 "841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6124\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4124\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",15.8");
    add_srs_wkt (p, 9,
		 "0827777777778],PARAMETER[\"scale_factor\",1],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",1500000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"2400\"],AXIS[\"X\",NORTH],AXIS[\"Y");
    add_srs_wkt (p, 12, "\",EAST]]");
    p = add_epsg_def (first, last, 2401, "epsg", 2401,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 25");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=25500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 25\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",7");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",25500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2401\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2402, "epsg", 2402,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=26500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 26\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",7");
    add_srs_wkt (p, 9,
		 "8],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",26500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2402\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2403, "epsg", 2403,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 27");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=27500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 27\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",27500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2403\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2404, "epsg", 2404,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=28500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 28\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",28500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2404\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2405, "epsg", 2405,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 29");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=29500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 29\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",29500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2405\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2406, "epsg", 2406,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=30500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 30\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",30500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2406\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2407, "epsg", 2407,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=31500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 31\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",31500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2407\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2408, "epsg", 2408,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 32");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=32500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 32\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",32500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2408\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2409, "epsg", 2409,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 33");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=33500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 33\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",33500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"2409\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2410, "epsg", 2410,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 34");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=34500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 34\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "02],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",34500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2410\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2411, "epsg", 2411,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 35");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=35500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 35\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "05],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",35500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2411\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2412, "epsg", 2412,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 36");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=36500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 36\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "08],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",36500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2412\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2413, "epsg", 2413,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 37");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=37500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 37\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "11],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",37500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2413\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2414, "epsg", 2414,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 38");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=38500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 38\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "14],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",38500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2414\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_03 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2415, "epsg", 2415,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 39");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=39500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 39\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "17],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",39500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2415\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2416, "epsg", 2416,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 40");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=40500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 40\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "20],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",40500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2416\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2417, "epsg", 2417,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 41");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=41500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 41\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "23],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",41500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2417\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2418, "epsg", 2418,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 42");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=42500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 42\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "26],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",42500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2418\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2419, "epsg", 2419,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 43");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=43500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 43\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "29],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",43500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2419\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2420, "epsg", 2420,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 44");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=44500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 44\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "32],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",44500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2420\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2421, "epsg", 2421,
		      "Beijing 1954 / 3-degree Gauss-Kruger zone 45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=45500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger zone 45\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "35],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",45500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"2421\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 2422, "epsg", 2422,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 75E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",7");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2422\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2423, "epsg", 2423,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 78E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 78E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",7");
    add_srs_wkt (p, 9,
		 "8],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2423\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2424, "epsg", 2424,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 81E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2424\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2425, "epsg", 2425,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 84E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 84E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2425\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2426, "epsg", 2426,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 87E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2426\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2427, "epsg", 2427,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 90E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 90E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2427\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2428, "epsg", 2428,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 93E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2428\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2429, "epsg", 2429,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 96E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 96E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2429\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2430, "epsg", 2430,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 99E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"2430\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2431, "epsg", 2431,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 102E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 102E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "02],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2431\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2432, "epsg", 2432,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 105E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "05],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2432\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2433, "epsg", 2433,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 108E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 108E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "08],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2433\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2434, "epsg", 2434,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 111E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "11],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2434\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2435, "epsg", 2435,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 114E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 114E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "14],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2435\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2436, "epsg", 2436,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 117E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "17],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2436\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2437, "epsg", 2437,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 120E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 120E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "20],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2437\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2438, "epsg", 2438,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 123E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "23],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2438\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2439, "epsg", 2439,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 126E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 126E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "26],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2439\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2440, "epsg", 2440,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 129E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "29],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2440\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2441, "epsg", 2441,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 132E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 132E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "32],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2441\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2442, "epsg", 2442,
		      "Beijing 1954 / 3-degree Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / 3-degree Gauss-Kruger CM 135E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "35],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2442\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2443, "epsg", 2443,
		      "JGD2000 / Japan Plane Rectangular CS I");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=33 +lon_0=129.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS I\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",33],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",129.5],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2443\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2444, "epsg", 2444,
		      "JGD2000 / Japan Plane Rectangular CS II");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=33 +lon_0=131 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS II\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",33],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",131],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 10,
		 "ctor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2444\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2445, "epsg", 2445,
		      "JGD2000 / Japan Plane Rectangular CS III");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=132.1666666666667 +k=0.9999");
    add_proj4text (p, 1,
		   " +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS III\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",36],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",132.1666666666667],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2445\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2446, "epsg", 2446,
		      "JGD2000 / Japan Plane Rectangular CS IV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=33 +lon_0=133.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS IV\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",33],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",133.5],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2446\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2447, "epsg", 2447,
		      "JGD2000 / Japan Plane Rectangular CS V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=134.3333333333333 +k=0.9999");
    add_proj4text (p, 1,
		   " +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS V\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",36],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",134.3333333333333],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2447\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2448, "epsg", 2448,
		      "JGD2000 / Japan Plane Rectangular CS VI");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=136 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS VI\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",36],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",136],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 10,
		 "ctor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2448\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2449, "epsg", 2449,
		      "JGD2000 / Japan Plane Rectangular CS VII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=137.1666666666667 +k=0.9999");
    add_proj4text (p, 1,
		   " +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS VII\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",36],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",137.1666666666667],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2449\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2450, "epsg", 2450,
		      "JGD2000 / Japan Plane Rectangular CS VIII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=138.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS VIII\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",3");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"central_meridian\",138.5],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.9999],PARAMETER[\"false_easting\",0],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2450\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2451, "epsg", 2451,
		      "JGD2000 / Japan Plane Rectangular CS IX");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=139.8333333333333 +k=0.9999");
    add_proj4text (p, 1,
		   " +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS IX\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",36],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",139.8333333333333],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2451\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2452, "epsg", 2452,
		      "JGD2000 / Japan Plane Rectangular CS X");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=140.8333333333333 +k=0.9999");
    add_proj4text (p, 1,
		   " +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS X\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",40],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",140.8333333333333],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2452\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2453, "epsg", 2453,
		      "JGD2000 / Japan Plane Rectangular CS XI");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=44 +lon_0=140.25 +k=0.9999 +x_0=0 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XI\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",44],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",140.25],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",0],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2453\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2454, "epsg", 2454,
		      "JGD2000 / Japan Plane Rectangular CS XII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=44 +lon_0=142.25 +k=0.9999 +x_0=0 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XII\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",44],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",142.25],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",0],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2454\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2455, "epsg", 2455,
		      "JGD2000 / Japan Plane Rectangular CS XIII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=44 +lon_0=144.25 +k=0.9999 +x_0=0 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XIII\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"central_meridian\",144.25],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",0.9999],PARAMETER[\"false_easting\",0],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2455\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2456, "epsg", 2456,
		      "JGD2000 / Japan Plane Rectangular CS XIV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=142 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XIV\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",26],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",142],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 10,
		 "ctor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2456\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2457, "epsg", 2457,
		      "JGD2000 / Japan Plane Rectangular CS XV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=127.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XV\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",26],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",127.5],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2457\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2458, "epsg", 2458,
		      "JGD2000 / Japan Plane Rectangular CS XVI");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=124 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XVI\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",26],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",124],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 10,
		 "ctor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2458\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2459, "epsg", 2459,
		      "JGD2000 / Japan Plane Rectangular CS XVII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=131 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XVII\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",2");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"central_meridian\",131],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",0],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2459\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2460, "epsg", 2460,
		      "JGD2000 / Japan Plane Rectangular CS XVIII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=20 +lon_0=136 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XVIII\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "20],PARAMETER[\"central_meridian\",136],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.9999],PARAMETER[\"false_easting\",0],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2460\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2461, "epsg", 2461,
		      "JGD2000 / Japan Plane Rectangular CS XIX");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=154 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / Japan Plane Rectangular CS XIX\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",26],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",154],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 10,
		 "ctor\",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2461\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2462, "epsg", 2462,
		      "Albanian 1987 / Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Albanian 1987 / Gauss-Kruger zone 4\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Albanian 1987\",DATUM[\"Albanian_1987\",SPHEROID[\"Krass");
    add_srs_wkt (p, 2,
		 "owsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6191\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4191\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",21],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "4500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11, "G\",\"2462\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2463, "epsg", 2463,
		      "Pulkovo 1995 / Gauss-Kruger CM 21E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 21E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2463\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2464, "epsg", 2464,
		      "Pulkovo 1995 / Gauss-Kruger CM 27E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 27E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2464\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2465, "epsg", 2465,
		      "Pulkovo 1995 / Gauss-Kruger CM 33E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 33E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2465\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2466, "epsg", 2466,
		      "Pulkovo 1995 / Gauss-Kruger CM 39E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 39E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",39],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2466\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2467, "epsg", 2467,
		      "Pulkovo 1995 / Gauss-Kruger CM 45E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 45E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",45],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2467\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2468, "epsg", 2468,
		      "Pulkovo 1995 / Gauss-Kruger CM 51E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 51E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",51],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2468\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2469, "epsg", 2469,
		      "Pulkovo 1995 / Gauss-Kruger CM 57E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 57E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",57],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2469\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2470, "epsg", 2470,
		      "Pulkovo 1995 / Gauss-Kruger CM 63E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 63E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",63],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2470\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2471, "epsg", 2471,
		      "Pulkovo 1995 / Gauss-Kruger CM 69E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 69E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",69],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2471\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2472, "epsg", 2472,
		      "Pulkovo 1995 / Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 75E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",75],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2472\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2473, "epsg", 2473,
		      "Pulkovo 1995 / Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 81E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",81],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2473\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2474, "epsg", 2474,
		      "Pulkovo 1995 / Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 87E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",87],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2474\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2475, "epsg", 2475,
		      "Pulkovo 1995 / Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 93E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",93],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2475\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2476, "epsg", 2476,
		      "Pulkovo 1995 / Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 99E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",99],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2476\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2477, "epsg", 2477,
		      "Pulkovo 1995 / Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 105E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",105],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2477\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2478, "epsg", 2478,
		      "Pulkovo 1995 / Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 111E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",111],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2478\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2479, "epsg", 2479,
		      "Pulkovo 1995 / Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 117E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",117],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2479\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2480, "epsg", 2480,
		      "Pulkovo 1995 / Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 123E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",123],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2480\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2481, "epsg", 2481,
		      "Pulkovo 1995 / Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 129E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",129],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2481\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2482, "epsg", 2482,
		      "Pulkovo 1995 / Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 135E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",135],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2482\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2483, "epsg", 2483,
		      "Pulkovo 1995 / Gauss-Kruger CM 141E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 141E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2483\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2484, "epsg", 2484,
		      "Pulkovo 1995 / Gauss-Kruger CM 147E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 147E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",147],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2484\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2485, "epsg", 2485,
		      "Pulkovo 1995 / Gauss-Kruger CM 153E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 153E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",153],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2485\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2486, "epsg", 2486,
		      "Pulkovo 1995 / Gauss-Kruger CM 159E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 159E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",159],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2486\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2487, "epsg", 2487,
		      "Pulkovo 1995 / Gauss-Kruger CM 165E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 165E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",165],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2487\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2488, "epsg", 2488,
		      "Pulkovo 1995 / Gauss-Kruger CM 171E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 171E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",171],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2488\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2489, "epsg", 2489,
		      "Pulkovo 1995 / Gauss-Kruger CM 177E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 177E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",177],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2489\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2490, "epsg", 2490,
		      "Pulkovo 1995 / Gauss-Kruger CM 177W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 177W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-177],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2490\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2491, "epsg", 2491,
		      "Pulkovo 1995 / Gauss-Kruger CM 171W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger CM 171W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-171],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2491\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2492, "epsg", 2492,
		      "Pulkovo 1942 / Gauss-Kruger CM 9E (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=500000 +y_0=0 +e");
    add_proj4text (p, 1, "llps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 9E (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",9],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2492\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2493, "epsg", 2493,
		      "Pulkovo 1942 / Gauss-Kruger CM 15E (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 15E (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",15],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2493\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2494, "epsg", 2494,
		      "Pulkovo 1942 / Gauss-Kruger CM 21E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 21E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2494\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2495, "epsg", 2495,
		      "Pulkovo 1942 / Gauss-Kruger CM 27E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 27E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2495\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2496, "epsg", 2496,
		      "Pulkovo 1942 / Gauss-Kruger CM 33E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 33E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2496\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2497, "epsg", 2497,
		      "Pulkovo 1942 / Gauss-Kruger CM 39E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 39E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",39],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2497\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2498, "epsg", 2498,
		      "Pulkovo 1942 / Gauss-Kruger CM 45E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 45E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",45],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2498\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2499, "epsg", 2499,
		      "Pulkovo 1942 / Gauss-Kruger CM 51E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 51E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",51],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2499\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2500, "epsg", 2500,
		      "Pulkovo 1942 / Gauss-Kruger CM 57E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 57E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",57],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2500\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2501, "epsg", 2501,
		      "Pulkovo 1942 / Gauss-Kruger CM 63E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 63E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",63],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2501\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2502, "epsg", 2502,
		      "Pulkovo 1942 / Gauss-Kruger CM 69E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 69E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",69],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2502\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2503, "epsg", 2503,
		      "Pulkovo 1942 / Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 75E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",75],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2503\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2504, "epsg", 2504,
		      "Pulkovo 1942 / Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 81E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",81],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2504\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2505, "epsg", 2505,
		      "Pulkovo 1942 / Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 87E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",87],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2505\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2506, "epsg", 2506,
		      "Pulkovo 1942 / Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 93E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",93],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2506\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2507, "epsg", 2507,
		      "Pulkovo 1942 / Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 99E\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",99],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2507\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2508, "epsg", 2508,
		      "Pulkovo 1942 / Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 105E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",105],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2508\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2509, "epsg", 2509,
		      "Pulkovo 1942 / Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 111E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",111],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2509\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2510, "epsg", 2510,
		      "Pulkovo 1942 / Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 117E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",117],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2510\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2511, "epsg", 2511,
		      "Pulkovo 1942 / Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 123E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",123],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2511\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2512, "epsg", 2512,
		      "Pulkovo 1942 / Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 129E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",129],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2512\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2513, "epsg", 2513,
		      "Pulkovo 1942 / Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 135E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",135],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2513\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2514, "epsg", 2514,
		      "Pulkovo 1942 / Gauss-Kruger CM 141E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 141E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2514\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2515, "epsg", 2515,
		      "Pulkovo 1942 / Gauss-Kruger CM 147E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 147E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",147],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2515\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2516, "epsg", 2516,
		      "Pulkovo 1942 / Gauss-Kruger CM 153E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 153E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",153],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2516\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2517, "epsg", 2517,
		      "Pulkovo 1942 / Gauss-Kruger CM 159E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 159E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",159],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2517\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2518, "epsg", 2518,
		      "Pulkovo 1942 / Gauss-Kruger CM 165E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 165E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",165],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2518\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2519, "epsg", 2519,
		      "Pulkovo 1942 / Gauss-Kruger CM 171E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 171E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",171],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2519\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2520, "epsg", 2520,
		      "Pulkovo 1942 / Gauss-Kruger CM 177E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 177E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",177],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2520\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2521, "epsg", 2521,
		      "Pulkovo 1942 / Gauss-Kruger CM 177W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 177W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",-177],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2521\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2522, "epsg", 2522,
		      "Pulkovo 1942 / Gauss-Kruger CM 171W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger CM 171W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",-171],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2522\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2523, "epsg", 2523,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=7500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 7\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",21],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",75000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2523\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2524, "epsg", 2524,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 8\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",24],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",85000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2524\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2525, "epsg", 2525,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 9\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",27],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",95000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2525\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2526, "epsg", 2526,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=10500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 10\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",30],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",105");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2526\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2527, "epsg", 2527,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=11500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 11\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",33],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",115");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2527\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2528, "epsg", 2528,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=36 +k=1 +x_0=12500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 12\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",36],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",125");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2528\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2529, "epsg", 2529,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=13500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 13\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",39],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",135");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2529\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2530, "epsg", 2530,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=42 +k=1 +x_0=14500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 14\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",42],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",145");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2530\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2531, "epsg", 2531,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=15500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 15\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",45],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",155");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2531\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2532, "epsg", 2532,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=48 +k=1 +x_0=16500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 16\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",48],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",165");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2532\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2533, "epsg", 2533,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=17500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 17\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",51],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",175");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2533\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2534, "epsg", 2534,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=54 +k=1 +x_0=18500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 18\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",54],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",185");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2534\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2535, "epsg", 2535,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 19");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=19500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 19\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",57],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",195");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2535\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2536, "epsg", 2536,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=60 +k=1 +x_0=20500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 20\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",60],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",205");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2536\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2537, "epsg", 2537,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 21");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=21500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 21\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",63],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",215");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2537\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2538, "epsg", 2538,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=66 +k=1 +x_0=22500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 22\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",66],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",225");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2538\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2539, "epsg", 2539,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 23");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=23500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 23\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",69],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",235");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2539\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2540, "epsg", 2540,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 24");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=72 +k=1 +x_0=24500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 24\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",72],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",245");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2540\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2541, "epsg", 2541,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 25");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=25500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 25\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",75],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",255");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2541\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2542, "epsg", 2542,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=26500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 26\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",78],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",265");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2542\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_04 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2543, "epsg", 2543,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 27");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=27500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 27\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",81],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",275");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2543\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2544, "epsg", 2544,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=28500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 28\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",84],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",285");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2544\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2545, "epsg", 2545,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 29");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=29500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 29\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",87],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",295");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2545\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2546, "epsg", 2546,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=30500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 30\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",90],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",305");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2546\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2547, "epsg", 2547,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=31500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 31\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",93],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",315");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2547\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2548, "epsg", 2548,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 32");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=32500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 32\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",96],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",325");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2548\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2549, "epsg", 2549,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 33");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=33500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 33\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",335");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2549\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2550, "epsg", 2550,
		      "Samboja / UTM zone 50S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=bessel +towgs84=-404.78");
    add_proj4text (p, 1, ",685.68,45.47,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Samboja / UTM zone 50S (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Samboja\",DATUM[\"Samboja\",SPHEROID[\"Bessel 1841\",637");
    add_srs_wkt (p, 2,
		 "7397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],TOWGS");
    add_srs_wkt (p, 3,
		 "84[-404.78,685.68,45.47,0,0,0,0],AUTHORITY[\"EPSG\",\"61");
    add_srs_wkt (p, 4,
		 "25\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9108\"]],AUTHORITY[\"EPSG\",\"4125\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",117],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"2550\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2551, "epsg", 2551,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 34");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=34500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 34\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",102],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",34");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2551\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2552, "epsg", 2552,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 35");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=35500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 35\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",35");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2552\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2553, "epsg", 2553,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 36");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=36500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 36\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",108],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",36");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2553\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2554, "epsg", 2554,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 37");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=37500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 37\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",37");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2554\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2555, "epsg", 2555,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 38");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=38500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 38\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",114],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",38");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2555\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2556, "epsg", 2556,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 39");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=39500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 39\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",39");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2556\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2557, "epsg", 2557,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 40");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=40500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 40\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",120],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",40");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2557\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2558, "epsg", 2558,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 41");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=41500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 41\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",41");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2558\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2559, "epsg", 2559,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 42");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=42500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 42\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",126],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",42");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2559\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2560, "epsg", 2560,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 43");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=43500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 43\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",43");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2560\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2561, "epsg", 2561,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 44");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=44500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 44\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",132],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",44");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2561\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2562, "epsg", 2562,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=45500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 45\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",45");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2562\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2563, "epsg", 2563,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 46");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=138 +k=1 +x_0=46500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 46\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",138],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",46");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2563\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2564, "epsg", 2564,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 47");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=47500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 47\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",141],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",47");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2564\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2565, "epsg", 2565,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 48");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=144 +k=1 +x_0=48500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 48\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",144],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",48");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2565\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2566, "epsg", 2566,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 49");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=49500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 49\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",147],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",49");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2566\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2567, "epsg", 2567,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 50");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=150 +k=1 +x_0=50500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 50\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",150],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2567\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2568, "epsg", 2568,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 51");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=51500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 51\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",153],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",51");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2568\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2569, "epsg", 2569,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 52");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=156 +k=1 +x_0=52500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 52\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",156],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",52");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2569\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2570, "epsg", 2570,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 53");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=53500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 53\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",159],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",53");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2570\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2571, "epsg", 2571,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 54");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=162 +k=1 +x_0=54500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 54\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",162],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",54");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2571\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2572, "epsg", 2572,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 55");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=55500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 55\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",165],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",55");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2572\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2573, "epsg", 2573,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 56");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=168 +k=1 +x_0=56500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 56\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",168],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",56");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2573\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2574, "epsg", 2574,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 57");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=57500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 57\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",171],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",57");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2574\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2575, "epsg", 2575,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 58");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=174 +k=1 +x_0=58500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 58\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",174],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",58");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2575\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2576, "epsg", 2576,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 59");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=59500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 59\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",177],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",59");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2576\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2577, "epsg", 2577,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 60 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=180 +k=1 +x_0=60000000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 60 (d");
    add_srs_wkt (p, 1,
		 "eprecated)\",GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_194");
    add_srs_wkt (p, 2,
		 "2\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0");
    add_srs_wkt (p, 4,
		 ".85,-0.12],AUTHORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 10,
		 "\",180],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",60000000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"2577\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 13, "ST]]");
    p = add_epsg_def (first, last, 2578, "epsg", 2578,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 61");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=61500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 61\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-177],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "1500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2578\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2579, "epsg", 2579,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 62");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-174 +k=1 +x_0=62500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 62\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-174],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "2500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2579\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2580, "epsg", 2580,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 63");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=63500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 63\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-171],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "3500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2580\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2581, "epsg", 2581,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 64");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-168 +k=1 +x_0=64500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 64\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-168],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "4500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2581\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2582, "epsg", 2582,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 21E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 21E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",21],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2582\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2583, "epsg", 2583,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 24E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 24E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",24],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2583\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2584, "epsg", 2584,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 27E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 27E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",27],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2584\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2585, "epsg", 2585,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 30E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 30E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",30],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2585\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2586, "epsg", 2586,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 33E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 33E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",33],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2586\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2587, "epsg", 2587,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 36E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=36 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 36E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",36],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2587\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2588, "epsg", 2588,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 39E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 39E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",39],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2588\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2589, "epsg", 2589,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 42E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=42 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 42E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",42],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2589\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2590, "epsg", 2590,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 45E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 45E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",45],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2590\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2591, "epsg", 2591,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 48E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=48 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 48E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",48],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2591\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2592, "epsg", 2592,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 51E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 51E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",51],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2592\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2593, "epsg", 2593,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 54E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=54 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 54E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",54],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2593\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2594, "epsg", 2594,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 57E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 57E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",57],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2594\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2595, "epsg", 2595,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 60E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=60 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 60E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",60],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2595\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2596, "epsg", 2596,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 63E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 63E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",63],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2596\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2597, "epsg", 2597,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 66E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=66 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 66E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",66],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2597\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2598, "epsg", 2598,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 69E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 69E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",69],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2598\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2599, "epsg", 2599,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 72E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=72 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 72E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",72],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2599\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2600, "epsg", 2600,
		      "Lietuvos Koordinoei Sistema 1994 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9998 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lietuvos Koordinoei Sistema 1994 (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"LKS94\",DATUM[\"Lithuania_1994_ETRS89\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6126\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4669\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",24],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9998],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2600\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2601, "epsg", 2601,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 75E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",75],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2601\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2602, "epsg", 2602,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 78E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 78E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",78],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2602\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2603, "epsg", 2603,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 81E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",81],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2603\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2604, "epsg", 2604,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 84E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 84E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",84],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2604\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2605, "epsg", 2605,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 87E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",87],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2605\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2606, "epsg", 2606,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 90E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 90E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",90],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2606\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2607, "epsg", 2607,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 93E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",93],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2607\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2608, "epsg", 2608,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 96E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 96E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",96],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2608\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2609, "epsg", 2609,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 99E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",99],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2609\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2610, "epsg", 2610,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 102E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 102E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",102],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2610\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2611, "epsg", 2611,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 105E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2611\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2612, "epsg", 2612,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 108E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 108E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",108],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2612\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2613, "epsg", 2613,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 111E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2613\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2614, "epsg", 2614,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 114E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 114E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",114],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2614\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2615, "epsg", 2615,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 117E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2615\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2616, "epsg", 2616,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 120E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 120E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",120],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2616\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2617, "epsg", 2617,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 123E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2617\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2618, "epsg", 2618,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 126E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 126E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",126],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2618\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2619, "epsg", 2619,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 129E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2619\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2620, "epsg", 2620,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 132E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 132E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",132],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2620\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2621, "epsg", 2621,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 135E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2621\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2622, "epsg", 2622,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 138E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=138 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 138E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",138],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2622\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2623, "epsg", 2623,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 141E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 141E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",141],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2623\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2624, "epsg", 2624,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 144E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=144 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 144E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",144],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2624\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2625, "epsg", 2625,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 147E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 147E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",147],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2625\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2626, "epsg", 2626,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 150E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=150 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 150E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",150],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2626\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2627, "epsg", 2627,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 153E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 153E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",153],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2627\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2628, "epsg", 2628,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 156E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=156 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 156E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",156],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2628\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2629, "epsg", 2629,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 159E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 159E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",159],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2629\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2630, "epsg", 2630,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 162E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=162 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 162E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",162],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2630\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2631, "epsg", 2631,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 165E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 165E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",165],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2631\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2632, "epsg", 2632,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 168E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=168 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 168E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",168],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2632\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2633, "epsg", 2633,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 171E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 171E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",171],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2633\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2634, "epsg", 2634,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 174E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=174 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 174E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",174],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2634\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2635, "epsg", 2635,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 177E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 177E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",177],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2635\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2636, "epsg", 2636,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 180E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=180 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 180E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",180],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2636\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2637, "epsg", 2637,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 177W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 177W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-177],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2637\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2638, "epsg", 2638,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 174W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-174 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 174W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-174],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2638\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2639, "epsg", 2639,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 171W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 171W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-171],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2639\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2640, "epsg", 2640,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 168W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-168 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 168W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-168],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2640\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2641, "epsg", 2641,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=7500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 7\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",21],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",7500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2641\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2642, "epsg", 2642,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 8\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",24],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",8500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2642\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2643, "epsg", 2643,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 9\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",27],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",9500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2643\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2644, "epsg", 2644,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=10500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 10\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",30],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",105");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2644\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2645, "epsg", 2645,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=11500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 11\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",33],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",115");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2645\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2646, "epsg", 2646,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=36 +k=1 +x_0=12500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 12\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",36],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",125");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2646\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2647, "epsg", 2647,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=13500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 13\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",39],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",135");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2647\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2648, "epsg", 2648,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=42 +k=1 +x_0=14500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 14\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",42],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",145");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2648\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2649, "epsg", 2649,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=15500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 15\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",45],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",155");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2649\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2650, "epsg", 2650,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=48 +k=1 +x_0=16500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 16\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",48],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",165");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2650\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2651, "epsg", 2651,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=17500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 17\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",51],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",175");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2651\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2652, "epsg", 2652,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=54 +k=1 +x_0=18500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 18\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",54],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",185");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2652\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2653, "epsg", 2653,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 19");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=19500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 19\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",57],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",195");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2653\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2654, "epsg", 2654,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=60 +k=1 +x_0=20500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 20\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",60],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",205");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2654\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2655, "epsg", 2655,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 21");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=21500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 21\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",63],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",215");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2655\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2656, "epsg", 2656,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=66 +k=1 +x_0=22500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 22\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",66],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",225");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2656\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2657, "epsg", 2657,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 23");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=23500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 23\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",69],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",235");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2657\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2658, "epsg", 2658,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 24");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=72 +k=1 +x_0=24500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 24\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",72],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",245");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2658\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2659, "epsg", 2659,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 25");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=25500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 25\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",75],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",255");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2659\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2660, "epsg", 2660,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=26500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 26\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",78],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",265");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2660\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2661, "epsg", 2661,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 27");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=27500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 27\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",81],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",275");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2661\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2662, "epsg", 2662,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=28500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 28\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",84],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",285");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2662\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2663, "epsg", 2663,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 29");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=29500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 29\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",87],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",295");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2663\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2664, "epsg", 2664,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=30500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 30\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",90],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",305");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2664\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2665, "epsg", 2665,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=31500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 31\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",93],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",315");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2665\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2666, "epsg", 2666,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 32");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=32500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 32\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",96],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",325");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2666\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2667, "epsg", 2667,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 33");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=33500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 33\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",335");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2667\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2668, "epsg", 2668,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 34");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=34500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 34\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",102],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",34");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2668\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2669, "epsg", 2669,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 35");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=35500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 35\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",35");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2669\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2670, "epsg", 2670,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 36");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=36500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 36\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",108],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",36");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2670\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_05 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2671, "epsg", 2671,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 37");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=37500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 37\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",37");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2671\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2672, "epsg", 2672,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 38");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=38500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 38\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",114],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",38");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2672\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2673, "epsg", 2673,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 39");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=39500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 39\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",39");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2673\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2674, "epsg", 2674,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 40");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=40500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 40\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",120],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",40");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2674\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2675, "epsg", 2675,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 41");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=41500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 41\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",41");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2675\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2676, "epsg", 2676,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 42");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=42500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 42\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",126],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",42");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2676\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2677, "epsg", 2677,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 43");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=43500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 43\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",43");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2677\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2678, "epsg", 2678,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 44");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=44500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 44\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",132],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",44");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2678\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2679, "epsg", 2679,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=45500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 45\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",45");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2679\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2680, "epsg", 2680,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 46");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=138 +k=1 +x_0=46500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 46\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",138],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",46");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2680\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2681, "epsg", 2681,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 47");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=47500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 47\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",141],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",47");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2681\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2682, "epsg", 2682,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 48");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=144 +k=1 +x_0=48500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 48\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",144],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",48");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2682\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2683, "epsg", 2683,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 49");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=49500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 49\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",147],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",49");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2683\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2684, "epsg", 2684,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 50");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=150 +k=1 +x_0=50500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 50\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",150],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2684\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2685, "epsg", 2685,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 51");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=51500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 51\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",153],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",51");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2685\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2686, "epsg", 2686,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 52");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=156 +k=1 +x_0=52500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 52\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",156],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",52");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2686\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2687, "epsg", 2687,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 53");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=53500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 53\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",159],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",53");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2687\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2688, "epsg", 2688,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 54");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=162 +k=1 +x_0=54500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 54\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",162],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",54");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2688\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2689, "epsg", 2689,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 55");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=55500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 55\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",165],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",55");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2689\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2690, "epsg", 2690,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 56");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=168 +k=1 +x_0=56500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 56\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",168],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",56");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2690\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2691, "epsg", 2691,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 57");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=57500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 57\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",171],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",57");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2691\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2692, "epsg", 2692,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 58");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=174 +k=1 +x_0=58500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 58\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",174],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",58");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2692\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2693, "epsg", 2693,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 59");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=59500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 59\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",177],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",59");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"2693\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2694, "epsg", 2694,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 60 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=180 +k=1 +x_0=60000000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 60 (d");
    add_srs_wkt (p, 1,
		 "eprecated)\",GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_199");
    add_srs_wkt (p, 2,
		 "5\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,");
    add_srs_wkt (p, 4,
		 "0.16,-0.12],AUTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",180],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",60000000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 12,
		 "THORITY[\"EPSG\",\"2694\"],AXIS[\"X\",NORTH],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "EAST]]");
    p = add_epsg_def (first, last, 2695, "epsg", 2695,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 61");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=61500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16");
    add_proj4text (p, 2, ",-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 61\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-177],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "1500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2695\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2696, "epsg", 2696,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 62");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-174 +k=1 +x_0=62500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16");
    add_proj4text (p, 2, ",-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 62\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-174],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "2500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2696\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2697, "epsg", 2697,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 63");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=63500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16");
    add_proj4text (p, 2, ",-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 63\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-171],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "3500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2697\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2698, "epsg", 2698,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 64");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-168 +k=1 +x_0=64500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16");
    add_proj4text (p, 2, ",-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 64\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-168],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 11,
		 "4500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"2698\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2699, "epsg", 2699,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 21E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 21E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",21],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2699\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2700, "epsg", 2700,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 24E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 24E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",24],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2700\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2701, "epsg", 2701,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 27E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 27E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",27],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2701\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2702, "epsg", 2702,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 30E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 30E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",30],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2702\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2703, "epsg", 2703,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 33E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 33E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",33],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2703\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2704, "epsg", 2704,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 36E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=36 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 36E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",36],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2704\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2705, "epsg", 2705,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 39E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 39E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",39],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2705\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2706, "epsg", 2706,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 42E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=42 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 42E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",42],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2706\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2707, "epsg", 2707,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 45E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 45E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",45],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2707\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2708, "epsg", 2708,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 48E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=48 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 48E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",48],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2708\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2709, "epsg", 2709,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 51E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 51E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",51],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2709\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2710, "epsg", 2710,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 54E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=54 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 54E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",54],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2710\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2711, "epsg", 2711,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 57E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 57E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",57],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2711\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2712, "epsg", 2712,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 60E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=60 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 60E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",60],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2712\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2713, "epsg", 2713,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 63E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 63E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",63],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2713\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2714, "epsg", 2714,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 66E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=66 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 66E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",66],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2714\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2715, "epsg", 2715,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 69E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 69E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",69],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2715\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2716, "epsg", 2716,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 72E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=72 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 72E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",72],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2716\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2717, "epsg", 2717,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 75E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",75],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2717\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2718, "epsg", 2718,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 78E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=78 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 78E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",78],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2718\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2719, "epsg", 2719,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 81E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",81],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2719\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2720, "epsg", 2720,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 84E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=84 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 84E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",84],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2720\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2721, "epsg", 2721,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 87E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",87],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2721\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2722, "epsg", 2722,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 90E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 90E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",90],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2722\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2723, "epsg", 2723,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 93E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",93],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2723\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2724, "epsg", 2724,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 96E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=96 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 96E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",96],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2724\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2725, "epsg", 2725,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 99E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",99],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2725\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2726, "epsg", 2726,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 102E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=102 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 102E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",102],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2726\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2727, "epsg", 2727,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 105E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2727\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2728, "epsg", 2728,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 108E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=108 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 108E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",108],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2728\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2729, "epsg", 2729,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 111E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2729\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2730, "epsg", 2730,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 114E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=114 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 114E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",114],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2730\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2731, "epsg", 2731,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 117E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2731\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2732, "epsg", 2732,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 120E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=120 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 120E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",120],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2732\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2733, "epsg", 2733,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 123E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2733\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2734, "epsg", 2734,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 126E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=126 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 126E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",126],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2734\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2735, "epsg", 2735,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 129E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2735\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2736, "epsg", 2736, "Tete / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +ellps=clrk66 +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tete / UTM zone 36S\",GEOGCS[\"Tete\",DATUM[\"T");
    add_srs_wkt (p, 1,
		 "ete\",SPHEROID[\"Clarke 1866\",6378206.4,294.97869821390");
    add_srs_wkt (p, 2,
		 "06,AUTHORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"61");
    add_srs_wkt (p, 3,
		 "27\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4127\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",33],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 9,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"2736\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2737, "epsg", 2737, "Tete / UTM zone 37S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +south +ellps=clrk66 +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tete / UTM zone 37S\",GEOGCS[\"Tete\",DATUM[\"T");
    add_srs_wkt (p, 1,
		 "ete\",SPHEROID[\"Clarke 1866\",6378206.4,294.97869821390");
    add_srs_wkt (p, 2,
		 "06,AUTHORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"61");
    add_srs_wkt (p, 3,
		 "27\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4127\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",39],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 9,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"2737\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2738, "epsg", 2738,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 132E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=132 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 132E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",132],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2738\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2739, "epsg", 2739,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 135E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2739\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2740, "epsg", 2740,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 138E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=138 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 138E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",138],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2740\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2741, "epsg", 2741,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 141E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 141E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",141],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2741\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2742, "epsg", 2742,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 144E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=144 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 144E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",144],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2742\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2743, "epsg", 2743,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 147E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 147E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",147],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2743\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2744, "epsg", 2744,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 150E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=150 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 150E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",150],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2744\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2745, "epsg", 2745,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 153E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 153E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",153],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2745\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2746, "epsg", 2746,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 156E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=156 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 156E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",156],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2746\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2747, "epsg", 2747,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 159E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 159E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",159],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2747\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2748, "epsg", 2748,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 162E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=162 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 162E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",162],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2748\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2749, "epsg", 2749,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 165E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 165E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",165],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2749\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2750, "epsg", 2750,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 168E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=168 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 168E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",168],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2750\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2751, "epsg", 2751,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 171E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 171E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",171],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2751\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2752, "epsg", 2752,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 174E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=174 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 174E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",174],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2752\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2753, "epsg", 2753,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 177E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 177E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",177],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2753\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2754, "epsg", 2754,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 180E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=180 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 180E\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",180],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2754\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2755, "epsg", 2755,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 177W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 177W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-177],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2755\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2756, "epsg", 2756,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 174W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-174 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 174W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-174],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2756\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2757, "epsg", 2757,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 171W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 171W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-171],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2757\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2758, "epsg", 2758,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 168W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-168 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 168W\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-168],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2758\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 2759, "epsg", 2759,
		      "NAD83(HARN) / Alabama East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30.5 +lon_0=-85.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9996 +x_0=200000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Alabama East\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",30.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-85.83333333333333],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.99996],PARAMETER[\"false_easting\",200000],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2759\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2760, "epsg", 2760,
		      "NAD83(HARN) / Alabama West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-87.5 +k=0.999933333 +x_0=6");
    add_proj4text (p, 1, "00000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Alabama West\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",30],PARAMETER[\"central_mer");
    add_srs_wkt (p, 9,
		 "idian\",-87.5],PARAMETER[\"scale_factor\",0.999933333],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",600000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2760\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2761, "epsg", 2761,
		      "NAD83(HARN) / Arizona East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-110.1666666666667 +k=0.999");
    add_proj4text (p, 1, "9 +x_0=213360 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arizona East\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",31],PARAMETER[\"central_mer");
    add_srs_wkt (p, 9,
		 "idian\",-110.1666666666667],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9999],PARAMETER[\"false_easting\",213360],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2761\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2762, "epsg", 2762,
		      "NAD83(HARN) / Arizona Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-111.9166666666667 +k=0.999");
    add_proj4text (p, 1, "9 +x_0=213360 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arizona Central\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",31],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-111.9166666666667],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999],PARAMETER[\"false_easting\",213360],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2762\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2763, "epsg", 2763,
		      "NAD83(HARN) / Arizona West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-113.75 +k=0.999933333 +x_0");
    add_proj4text (p, 1, "=213360 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arizona West\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",31],PARAMETER[\"central_mer");
    add_srs_wkt (p, 9,
		 "idian\",-113.75],PARAMETER[\"scale_factor\",0.999933333]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",213360],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"2763\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 12, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2764, "epsg", 2764,
		      "NAD83(HARN) / Arkansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.23333333333333 +lat_2=34.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=34.33333333333334 +lon_0=-92 +x_0=400000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arkansas North\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",36.2333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",34.93333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",34.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-92],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",400000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2764\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2765, "epsg", 2765,
		      "NAD83(HARN) / Arkansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.76666666666667 +lat_2=33.3 +lat_0=32");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-92 +x_0=400000 +y_0=400000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arkansas South\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",34.7666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",33.3],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",32.66666666666666],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-92],PARAMETER[\"false_easting\",400000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",400000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "2765\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2766, "epsg", 2766,
		      "NAD83(HARN) / California zone 1");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.66666666666666 +lat_2=40 +lat_0=39.3");
    add_proj4text (p, 1,
		   "3333333333334 +lon_0=-122 +x_0=2000000 +y_0=500000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 1\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",41.6666666666666");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"standard_parallel_2\",40],PARAMETER[\"lat");
    add_srs_wkt (p, 10,
		 "itude_of_origin\",39.33333333333334],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-122],PARAMETER[\"false_easting\",2000000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "2766\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2767, "epsg", 2767,
		      "NAD83(HARN) / California zone 2");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.83333333333334 +lat_2=38.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=37.66666666666666 +lon_0=-122 +x_0=2000000 +y");
    add_proj4text (p, 2, "_0=500000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 2\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",39.8333333333333");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",38.33333333333334],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",37.66666666666666],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-122],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",2000000],PARAMETER[\"false_northing\",500000],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"2767\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 14, "H]]");
    p = add_epsg_def (first, last, 2768, "epsg", 2768,
		      "NAD83(HARN) / California zone 3");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.5 +lon_0=-120.5 +x_0=2000000 +y_0=500000 +");
    add_proj4text (p, 2, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 3\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",38.4333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",37.06666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",36.5],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-120.5],PARAMETER[\"false_easting\",200000");
    add_srs_wkt (p, 12,
		 "0],PARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13, "\",\"2768\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2769, "epsg", 2769,
		      "NAD83(HARN) / California zone 4");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.25 +lat_2=36 +lat_0=35.3333333333333");
    add_proj4text (p, 1,
		   "4 +lon_0=-119 +x_0=2000000 +y_0=500000 +ellps=GRS80 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 4\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",37.25],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_2\",36],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 10,
		 "gin\",35.33333333333334],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 11,
		 "-119],PARAMETER[\"false_easting\",2000000],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_northing\",500000],AUTHORITY[\"EPSG\",\"2769\"],AXI");
    add_srs_wkt (p, 13, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2770, "epsg", 2770,
		      "NAD83(HARN) / California zone 5");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.46666666666667 +lat_2=34.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.5 +lon_0=-118 +x_0=2000000 +y_0=500000 +el");
    add_proj4text (p, 2, "lps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 5\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",35.4666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",34.03333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",33.5],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-118],PARAMETER[\"false_easting\",2000000]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"2770\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2771, "epsg", 2771,
		      "NAD83(HARN) / California zone 6");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.88333333333333 +lat_2=32.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=32.16666666666666 +lon_0=-116.25 +x_0=2000000");
    add_proj4text (p, 2, " +y_0=500000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 6\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",33.8833333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",32.78333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",32.16666666666666],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-116.25],PARAMETER[\"false_ea");
    add_srs_wkt (p, 12,
		 "sting\",2000000],PARAMETER[\"false_northing\",500000],AU");
    add_srs_wkt (p, 13,
		 "THORITY[\"EPSG\",\"2771\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 14, "ORTH]]");
    p = add_epsg_def (first, last, 2772, "epsg", 2772,
		      "NAD83(HARN) / Colorado North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.78333333333333 +lat_2=39.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=39.33333333333334 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2, "289 +y_0=304800.6096 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Colorado North\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",40.7833333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",39.71666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",39.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-105.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",914401.8289],PARAMETER[\"false_northing\",304800.");
    add_srs_wkt (p, 13,
		 "6096],AUTHORITY[\"EPSG\",\"2772\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2773, "epsg", 2773,
		      "NAD83(HARN) / Colorado Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.75 +lat_2=38.45 +lat_0=37.8333333333");
    add_proj4text (p, 1,
		   "3334 +lon_0=-105.5 +x_0=914401.8289 +y_0=304800.6096 +el");
    add_proj4text (p, 2, "lps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Colorado Central\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",39.75],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_2\",38.45],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",37.83333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-105.5],PARAMETER[\"false_easting\",914401.8289],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",304800.6096],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"2773\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2774, "epsg", 2774,
		      "NAD83(HARN) / Colorado South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.66666666666666 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2, "289 +y_0=304800.6096 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Colorado South\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",38.4333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",37.23333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",36.66666666666666],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-105.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",914401.8289],PARAMETER[\"false_northing\",304800.");
    add_srs_wkt (p, 13,
		 "6096],AUTHORITY[\"EPSG\",\"2774\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2775, "epsg", 2775,
		      "NAD83(HARN) / Connecticut");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.86666666666667 +lat_2=41.2 +lat_0=40");
    add_proj4text (p, 1,
		   ".83333333333334 +lon_0=-72.75 +x_0=304800.6096 +y_0=1524");
    add_proj4text (p, 2, "00.3048 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Connecticut\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",41.86666666666667],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",41.2],PARAMETER[\"latit");
    add_srs_wkt (p, 10,
		 "ude_of_origin\",40.83333333333334],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-72.75],PARAMETER[\"false_easting\",304800.609");
    add_srs_wkt (p, 12,
		 "6],PARAMETER[\"false_northing\",152400.3048],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2775\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2776, "epsg", 2776,
		      "NAD83(HARN) / Delaware");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=-75.41666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "995 +x_0=200000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Delaware\",GEOGCS[\"NAD83(HARN)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",38],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-75.41666666666667],PARAMETER[\"scale_factor\",0.9999");
    add_srs_wkt (p, 10,
		 "95],PARAMETER[\"false_easting\",200000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2776\"],AXIS[\"X\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2777, "epsg", 2777,
		      "NAD83(HARN) / Florida East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-81 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Florida East\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",24.33333333333333],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-81],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".999941177],PARAMETER[\"false_easting\",200000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2777\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2778, "epsg", 2778,
		      "NAD83(HARN) / Florida West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-82 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Florida West\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",24.33333333333333],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-82],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".999941177],PARAMETER[\"false_easting\",200000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2778\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2779, "epsg", 2779,
		      "NAD83(HARN) / Florida North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.75 +lat_2=29.58333333333333 +lat_0=2");
    add_proj4text (p, 1,
		   "9 +lon_0=-84.5 +x_0=600000 +y_0=0 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Florida North\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",30.75],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",29.58333333333333],PARAMETER[\"lat");
    add_srs_wkt (p, 10,
		 "itude_of_origin\",29],PARAMETER[\"central_meridian\",-84");
    add_srs_wkt (p, 11,
		 ".5],PARAMETER[\"false_easting\",600000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"2779\"],AXIS[\"X\",");
    add_srs_wkt (p, 13, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2780, "epsg", 2780,
		      "NAD83(HARN) / Georgia East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-82.16666666666667 +k=0.999");
    add_proj4text (p, 1, "9 +x_0=200000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Georgia East\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",30],PARAMETER[\"central_mer");
    add_srs_wkt (p, 9,
		 "idian\",-82.16666666666667],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9999],PARAMETER[\"false_easting\",200000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2780\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2781, "epsg", 2781,
		      "NAD83(HARN) / Georgia West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-84.16666666666667 +k=0.999");
    add_proj4text (p, 1, "9 +x_0=700000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Georgia West\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",30],PARAMETER[\"central_mer");
    add_srs_wkt (p, 9,
		 "idian\",-84.16666666666667],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9999],PARAMETER[\"false_easting\",700000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2781\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2782, "epsg", 2782,
		      "NAD83(HARN) / Hawaii zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=18.83333333333333 +lon_0=-155.5 +k=0.");
    add_proj4text (p, 1,
		   "999966667 +x_0=500000 +y_0=0 +ellps=GRS80 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Hawaii zone 1\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",18.83333333333333],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-155.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999966667],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2782\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2783, "epsg", 2783,
		      "NAD83(HARN) / Hawaii zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=20.33333333333333 +lon_0=-156.6666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=500000 +y_0=0 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Hawaii zone 2\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",20.33333333333333],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-156.6666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999966667],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2783\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2784, "epsg", 2784,
		      "NAD83(HARN) / Hawaii zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.16666666666667 +lon_0=-158 +k=0.99");
    add_proj4text (p, 1,
		   "999 +x_0=500000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Hawaii zone 3\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",21.16666666666667],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-158],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.99999],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2784\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2785, "epsg", 2785,
		      "NAD83(HARN) / Hawaii zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.83333333333333 +lon_0=-159.5 +k=0.");
    add_proj4text (p, 1,
		   "99999 +x_0=500000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Hawaii zone 4\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",21.83333333333333],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-159.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.99999],PARAMETER[\"false_easting\",500000],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2785\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2786, "epsg", 2786,
		      "NAD83(HARN) / Hawaii zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.66666666666667 +lon_0=-160.1666666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=500000 +y_0=0 +ellps=GRS80 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Hawaii zone 5\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",21.66666666666667],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-160.1666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2786\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2787, "epsg", 2787,
		      "NAD83(HARN) / Idaho East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-112.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999473679999999 +x_0=200000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Idaho East\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",41.66666666666666],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-112.1666666666667],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.999947368],PARAMETER[\"false_easting\",200");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2787\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2788, "epsg", 2788,
		      "NAD83(HARN) / Idaho Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-114 +k=0.99");
    add_proj4text (p, 1,
		   "99473679999999 +x_0=500000 +y_0=0 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Idaho Central\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",41.66666666666666],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-114],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.999947368],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2788\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2789, "epsg", 2789,
		      "NAD83(HARN) / Idaho West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-115.75 +k=0");
    add_proj4text (p, 1,
		   ".999933333 +x_0=800000 +y_0=0 +ellps=GRS80 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Idaho West\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",41.66666666666666],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-115.75],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999933333],PARAMETER[\"false_easting\",800000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2789\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2790, "epsg", 2790,
		      "NAD83(HARN) / Illinois East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-88.33333333");
    add_proj4text (p, 1,
		   "333333 +k=0.9999749999999999 +x_0=300000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Illinois East\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",36.66666666666666],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-88.33333333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999975],PARAMETER[\"false_easting\",3000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "2790\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2791, "epsg", 2791,
		      "NAD83(HARN) / Illinois West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-90.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=700000 +y_0=0 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Illinois West\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",36.66666666666666],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-90.16666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999941177],PARAMETER[\"false_easting\",7");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2791\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2792, "epsg", 2792,
		      "NAD83(HARN) / Indiana East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=100000 +y_0=250000 +ellps=GRS80 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Indiana East\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",37.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-85.66666666666667],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999966667],PARAMETER[\"false_easting\",100000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",250000],AUTHORITY[\"EPSG\",\"2792");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2793, "epsg", 2793,
		      "NAD83(HARN) / Indiana West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=250000 +ellps=GRS80 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Indiana West\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",37.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-87.08333333333333],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999966667],PARAMETER[\"false_easting\",900000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",250000],AUTHORITY[\"EPSG\",\"2793");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2794, "epsg", 2794,
		      "NAD83(HARN) / Iowa North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.26666666666667 +lat_2=42.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41.5 +lon_0=-93.5 +x_0=1500000 +y_0=1000000 +");
    add_proj4text (p, 2, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Iowa North\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",43.26666666666667],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",42.06666666666667],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",41.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-93.5],PARAMETER[\"false_easting\",1500000],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "2794\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2795, "epsg", 2795,
		      "NAD83(HARN) / Iowa South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.61666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40 +lon_0=-93.5 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Iowa South\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",41.78333333333333],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",40.61666666666667],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",40],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-93.5],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2795\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2796, "epsg", 2796,
		      "NAD83(HARN) / Kansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.78333333333333 +lat_2=38.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=38.33333333333334 +lon_0=-98 +x_0=400000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kansas North\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",39.78333333333333]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",38.71666666666667],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",38.33333333333334],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-98],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",400000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"2796\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2797, "epsg", 2797,
		      "NAD83(HARN) / Kansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.56666666666667 +lat_2=37.26666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.66666666666666 +lon_0=-98.5 +x_0=400000 +y");
    add_proj4text (p, 2, "_0=400000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kansas South\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",38.56666666666667]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",37.26666666666667],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",36.66666666666666],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-98.5],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",400000],PARAMETER[\"false_northing\",400000],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"2797\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 14, "]");
    p = add_epsg_def (first, last, 2798, "epsg", 2798,
		      "NAD83(HARN) / Kentucky North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=38.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=500000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kentucky North\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",37.9666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",38.96666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",37.5],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-84.25],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "798\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_06 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2799, "epsg", 2799,
		      "NAD83(HARN) / Kentucky South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.93333333333333 +lat_2=36.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=500000 +");
    add_proj4text (p, 2, "y_0=500000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kentucky South\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",37.9333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",36.73333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",36.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-85.75],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",500000],PARAMETER[\"false_northing\",500000],AUTH");
    add_srs_wkt (p, 13,
		 "ORITY[\"EPSG\",\"2799\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 2800, "epsg", 2800,
		      "NAD83(HARN) / Louisiana North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.66666666666666 +lat_2=31.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=30.5 +lon_0=-92.5 +x_0=1000000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Louisiana North\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",32.6666666666666");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"standard_parallel_2\",31.16666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",30.5],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-92.5],PARAMETER[\"false_easting\",1000000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "800\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2801, "epsg", 2801,
		      "NAD83(HARN) / Louisiana South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.7 +lat_2=29.3 +lat_0=28.5 +lon_0=-91");
    add_proj4text (p, 1,
		   ".33333333333333 +x_0=1000000 +y_0=0 +ellps=GRS80 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Louisiana South\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",30.7],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_2\",29.3],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 10,
		 "igin\",28.5],PARAMETER[\"central_meridian\",-91.33333333");
    add_srs_wkt (p, 11,
		 "333333],PARAMETER[\"false_easting\",1000000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2801\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2802, "epsg", 2802,
		      "NAD83(HARN) / Maine East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine East\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",43.66666666666666],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-68.5],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9999],PARAMETER[\"false_easting\",300000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2802\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2803, "epsg", 2803,
		      "NAD83(HARN) / Maine West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine West\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",42.83333333333334],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-70.16666666666667],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.999966667],PARAMETER[\"false_easting\",900");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2803\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2804, "epsg", 2804,
		      "NAD83(HARN) / Maryland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.45 +lat_2=38.3 +lat_0=37.66666666666");
    add_proj4text (p, 1,
		   "666 +lon_0=-77 +x_0=400000 +y_0=0 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maryland\",GEOGCS[\"NAD83(HARN)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"standard_parallel_1\",39.45],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",38.3],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 10,
		 ",37.66666666666666],PARAMETER[\"central_meridian\",-77],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",400000],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"2804\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2805, "epsg", 2805,
		      "NAD83(HARN) / Massachusetts Mainland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.68333333333333 +lat_2=41.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41 +lon_0=-71.5 +x_0=200000 +y_0=750000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Massachusetts Mainland\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",42.6833333");
    add_srs_wkt (p, 9,
		 "3333333],PARAMETER[\"standard_parallel_2\",41.7166666666");
    add_srs_wkt (p, 10,
		 "6667],PARAMETER[\"latitude_of_origin\",41],PARAMETER[\"c");
    add_srs_wkt (p, 11,
		 "entral_meridian\",-71.5],PARAMETER[\"false_easting\",200");
    add_srs_wkt (p, 12,
		 "000],PARAMETER[\"false_northing\",750000],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"2805\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2806, "epsg", 2806,
		      "NAD83(HARN) / Massachusetts Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.48333333333333 +lat_2=41.28333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-70.5 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Massachusetts Island\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",41.4833333");
    add_srs_wkt (p, 9,
		 "3333333],PARAMETER[\"standard_parallel_2\",41.2833333333");
    add_srs_wkt (p, 10,
		 "3333],PARAMETER[\"latitude_of_origin\",41],PARAMETER[\"c");
    add_srs_wkt (p, 11,
		 "entral_meridian\",-70.5],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 12,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 13, "\"2806\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2807, "epsg", 2807,
		      "NAD83(HARN) / Michigan North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.08333333333334 +lat_2=45.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=44.78333333333333 +lon_0=-87 +x_0=8000000 +y_");
    add_proj4text (p, 2, "0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Michigan North\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",47.0833333333333");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",45.48333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",44.78333333333333],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-87],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",8000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2807\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2808, "epsg", 2808,
		      "NAD83(HARN) / Michigan Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.7 +lat_2=44.18333333333333 +lat_0=43");
    add_proj4text (p, 1,
		   ".31666666666667 +lon_0=-84.36666666666666 +x_0=6000000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Michigan Central\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",45.7],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_2\",44.18333333333333],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",43.31666666666667],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-84.36666666666666],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",6000000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"2808\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2809, "epsg", 2809,
		      "NAD83(HARN) / Michigan South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.66666666666666 +lat_2=42.1 +lat_0=41");
    add_proj4text (p, 1,
		   ".5 +lon_0=-84.36666666666666 +x_0=4000000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Michigan South\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",43.6666666666666");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"standard_parallel_2\",42.1],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",41.5],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-84.36666666666666],PARAMETER[\"false_easting\",4000000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "809\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2810, "epsg", 2810,
		      "NAD83(HARN) / Minnesota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000 +y");
    add_proj4text (p, 2, "_0=100000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota North\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",48.6333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",47.03333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",46.5],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-93.1],PARAMETER[\"false_easting\",800000]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_northing\",100000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"2810\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2811, "epsg", 2811,
		      "NAD83(HARN) / Minnesota Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000 +y_0=100000 +ellps=GRS80 +un");
    add_proj4text (p, 2, "its=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota Central\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",47.05],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_2\",45.61666666666667],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",45],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 11,
		 "-94.25],PARAMETER[\"false_easting\",800000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",100000],AUTHORITY[\"EPSG\",\"2811\"],AX");
    add_srs_wkt (p, 13, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2812, "epsg", 2812,
		      "NAD83(HARN) / Minnesota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000 +y_0=100000 +ellps=");
    add_proj4text (p, 2, "GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota South\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",45.2166666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",43.78333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",43],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-94],PARAMETER[\"false_easting\",800000],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_northing\",100000],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "812\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2813, "epsg", 2813,
		      "NAD83(HARN) / Mississippi East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-88.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=300000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Mississippi East\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",29.5],PARAMETER[\"central");
    add_srs_wkt (p, 9,
		 "_meridian\",-88.83333333333333],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.99995],PARAMETER[\"false_easting\",300000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2813\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2814, "epsg", 2814,
		      "NAD83(HARN) / Mississippi West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-90.33333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=700000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Mississippi West\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",29.5],PARAMETER[\"central");
    add_srs_wkt (p, 9,
		 "_meridian\",-90.33333333333333],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.99995],PARAMETER[\"false_easting\",700000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2814\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2815, "epsg", 2815,
		      "NAD83(HARN) / Missouri East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-90.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=250000 +y_0=0 +ellps=GRS80 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Missouri East\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",35.83333333333334],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-90.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999933333],PARAMETER[\"false_easting\",250000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2815\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2816, "epsg", 2816,
		      "NAD83(HARN) / Missouri Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-92.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=500000 +y_0=0 +ellps=GRS80 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Missouri Central\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",35.83333333333334],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"central_meridian\",-92.5],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.999933333],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2816\"]");
    add_srs_wkt (p, 12, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2817, "epsg", 2817,
		      "NAD83(HARN) / Missouri West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.16666666666666 +lon_0=-94.5 +k=0.9");
    add_proj4text (p, 1,
		   "99941177 +x_0=850000 +y_0=0 +ellps=GRS80 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Missouri West\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",36.16666666666666],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-94.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999941177],PARAMETER[\"false_easting\",850000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2817\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2818, "epsg", 2818, "NAD83(HARN) / Montana");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=45 +lat_0=44.25 +lon_0=-109.5");
    add_proj4text (p, 1, " +x_0=600000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Montana\",GEOGCS[\"NAD83(HARN)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"standard_parallel_1\",49],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_2\",45],PARAMETER[\"latitude_of_origin\",44.2");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"central_meridian\",-109.5],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",600000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 12,
		 "THORITY[\"EPSG\",\"2818\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 2819, "epsg", 2819,
		      "NAD83(HARN) / Nebraska");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000 +y_0=0 +ellps=GRS80 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nebraska\",GEOGCS[\"NAD83(HARN)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"standard_parallel_1\",43],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_2\",40],PARAMETER[\"latitude_of_origin\",39.8");
    add_srs_wkt (p, 10,
		 "3333333333334],PARAMETER[\"central_meridian\",-100],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",0],AUTHORITY[\"EPSG\",\"2819\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2820, "epsg", 2820,
		      "NAD83(HARN) / Nevada East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-115.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=200000 +y_0=8000000 +ellps=GRS80 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nevada East\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",34.75],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-115.5833333333333],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9999],PARAMETER[\"false_easting\",200000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",8000000],AUTHORITY[\"EPSG\",\"2820\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2821, "epsg", 2821,
		      "NAD83(HARN) / Nevada Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-116.6666666666667 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=500000 +y_0=6000000 +ellps=GRS80 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nevada Central\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",34.75],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-116.6666666666667],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 10,
		 "r\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",6000000],AUTHORITY[\"EPSG\",\"2821\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2822, "epsg", 2822,
		      "NAD83(HARN) / Nevada West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-118.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=800000 +y_0=4000000 +ellps=GRS80 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nevada West\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",34.75],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-118.5833333333333],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9999],PARAMETER[\"false_easting\",800000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",4000000],AUTHORITY[\"EPSG\",\"2822\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2823, "epsg", 2823,
		      "NAD83(HARN) / New Hampshire");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-71.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=300000 +y_0=0 +ellps=GRS80 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Hampshire\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",42.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-71.66666666666667],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999966667],PARAMETER[\"false_easting\",300000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2823\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2824, "epsg", 2824,
		      "NAD83(HARN) / New Jersey");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Jersey\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",38.83333333333334],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-74.5],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9999],PARAMETER[\"false_easting\",150000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2824\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2825, "epsg", 2825,
		      "NAD83(HARN) / New Mexico East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-104.3333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "909091 +x_0=165000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Mexico East\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",31],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-104.3333333333333],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999909091],PARAMETER[\"false_easting\",165000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2825\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2826, "epsg", 2826,
		      "NAD83(HARN) / New Mexico Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-106.25 +k=0.9999 +x_0=5000");
    add_proj4text (p, 1, "00 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Mexico Central\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",31],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-106.25],PARAMETER[\"scale_factor\",0.9999],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",0],AUTHORITY[\"EPSG\",\"2826\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2827, "epsg", 2827,
		      "NAD83(HARN) / New Mexico West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-107.8333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "916667 +x_0=830000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Mexico West\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",31],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-107.8333333333333],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999916667],PARAMETER[\"false_easting\",830000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2827\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2828, "epsg", 2828,
		      "NAD83(HARN) / New York East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York East\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",38.83333333333334],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-74.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999],PARAMETER[\"false_easting\",150000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2828\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2829, "epsg", 2829,
		      "NAD83(HARN) / New York Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-76.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=250000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York Central\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",40],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-76.58333333333333],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999375],PARAMETER[\"false_easting\",250000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2829\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2830, "epsg", 2830,
		      "NAD83(HARN) / New York West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-78.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=350000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York West\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",40],PARAMETER[\"central_mer");
    add_srs_wkt (p, 9,
		 "idian\",-78.58333333333333],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9999375],PARAMETER[\"false_easting\",350000],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"2830\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2831, "epsg", 2831,
		      "NAD83(HARN) / New York Long Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.03333333333333 +lat_2=40.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=40.16666666666666 +lon_0=-74 +x_0=300000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York Long Island\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",41.0333333");
    add_srs_wkt (p, 9,
		 "3333333],PARAMETER[\"standard_parallel_2\",40.6666666666");
    add_srs_wkt (p, 10,
		 "6666],PARAMETER[\"latitude_of_origin\",40.16666666666666");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"central_meridian\",-74],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "easting\",300000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 13,
		 "ITY[\"EPSG\",\"2831\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 2832, "epsg", 2832,
		      "NAD83(HARN) / North Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.43333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=47 +lon_0=-100.5 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / North Dakota North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",48.7333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",47.43333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",47],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-100.5],PARAMETER[\"false_easting\",600000],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"283");
    add_srs_wkt (p, 13, "2\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2833, "epsg", 2833,
		      "NAD83(HARN) / North Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.48333333333333 +lat_2=46.18333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=45.66666666666666 +lon_0=-100.5 +x_0=600000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / North Dakota South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",47.4833333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",46.18333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",45.66666666666666],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-100.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",600000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"2833\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2834, "epsg", 2834,
		      "NAD83(HARN) / Ohio North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.7 +lat_2=40.43333333333333 +lat_0=39");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Ohio North\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",41.7],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_2\",40.43333333333333],PARAMETER[\"latit");
    add_srs_wkt (p, 10,
		 "ude_of_origin\",39.66666666666666],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-82.5],PARAMETER[\"false_easting\",600000],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2834\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2835, "epsg", 2835,
		      "NAD83(HARN) / Ohio South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.03333333333333 +lat_2=38.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=38 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Ohio South\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",40.03333333333333],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",38.73333333333333],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",38],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-82.5],PARAMETER[\"false_easting\",600000],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2835\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2836, "epsg", 2836,
		      "NAD83(HARN) / Oklahoma North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.76666666666667 +lat_2=35.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=35 +lon_0=-98 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oklahoma North\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",36.7666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",35.56666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",35],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-98],PARAMETER[\"false_easting\",600000],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2836\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2837, "epsg", 2837,
		      "NAD83(HARN) / Oklahoma South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.23333333333333 +lat_2=33.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.33333333333334 +lon_0=-98 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oklahoma South\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",35.2333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",33.93333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",33.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-98],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",600000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2837\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2838, "epsg", 2838,
		      "NAD83(HARN) / Oregon North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46 +lat_2=44.33333333333334 +lat_0=43.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=2500000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oregon North\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",46],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_2\",44.33333333333334],PARAMETER[\"latit");
    add_srs_wkt (p, 10,
		 "ude_of_origin\",43.66666666666666],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-120.5],PARAMETER[\"false_easting\",2500000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2838");
    add_srs_wkt (p, 13, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2839, "epsg", 2839,
		      "NAD83(HARN) / Oregon South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44 +lat_2=42.33333333333334 +lat_0=41.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=1500000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oregon South\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",44],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_2\",42.33333333333334],PARAMETER[\"latit");
    add_srs_wkt (p, 10,
		 "ude_of_origin\",41.66666666666666],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-120.5],PARAMETER[\"false_easting\",1500000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2839");
    add_srs_wkt (p, 13, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2840, "epsg", 2840,
		      "NAD83(HARN) / Rhode Island");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.08333333333334 +lon_0=-71.5 +k=0.9");
    add_proj4text (p, 1,
		   "9999375 +x_0=100000 +y_0=0 +ellps=GRS80 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Rhode Island\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",41.08333333333334],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-71.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.99999375],PARAMETER[\"false_easting\",100000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2840\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2841, "epsg", 2841,
		      "NAD83(HARN) / South Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.68333333333333 +lat_2=44.41666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=43.83333333333334 +lon_0=-100 +x_0=600000 +y_");
    add_proj4text (p, 2, "0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / South Dakota North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",45.6833333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",44.41666666666666],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",43.83333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-100],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",600000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2841\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2842, "epsg", 2842,
		      "NAD83(HARN) / South Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.4 +lat_2=42.83333333333334 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=600000 +y");
    add_proj4text (p, 2, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / South Dakota South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",44.4],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_2\",42.83333333333334],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",42.33333333333334],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-100.3333333333333],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",600000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"2842\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2843, "epsg", 2843,
		      "NAD83(HARN) / Tennessee");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.41666666666666 +lat_2=35.25 +lat_0=3");
    add_proj4text (p, 1,
		   "4.33333333333334 +lon_0=-86 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Tennessee\",GEOGCS[\"NAD83(HARN)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"standard_parallel_1\",36.41666666666666],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"standard_parallel_2\",35.25],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",34.33333333333334],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-86],PARAMETER[\"false_easting\",600000],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2843\"],AX");
    add_srs_wkt (p, 13, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2844, "epsg", 2844,
		      "NAD83(HARN) / Texas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.18333333333333 +lat_2=34.65 +lat_0=3");
    add_proj4text (p, 1,
		   "4 +lon_0=-101.5 +x_0=200000 +y_0=1000000 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas North\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",36.18333333333333],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",34.65],PARAMETER[\"lati");
    add_srs_wkt (p, 10,
		 "tude_of_origin\",34],PARAMETER[\"central_meridian\",-101");
    add_srs_wkt (p, 11,
		 ".5],PARAMETER[\"false_easting\",200000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",1000000],AUTHORITY[\"EPSG\",\"2844\"],AXIS[");
    add_srs_wkt (p, 13, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2845, "epsg", 2845,
		      "NAD83(HARN) / Texas North Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.96666666666667 +lat_2=32.13333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=31.66666666666667 +lon_0=-98.5 +x_0=600000 +y");
    add_proj4text (p, 2, "_0=2000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas North Central\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",33.96666666");
    add_srs_wkt (p, 9,
		 "666667],PARAMETER[\"standard_parallel_2\",32.13333333333");
    add_srs_wkt (p, 10,
		 "333],PARAMETER[\"latitude_of_origin\",31.66666666666667]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"central_meridian\",-98.5],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_easting\",600000],PARAMETER[\"false_northing\",2000000]");
    add_srs_wkt (p, 13,
		 ",AUTHORITY[\"EPSG\",\"2845\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 2846, "epsg", 2846,
		      "NAD83(HARN) / Texas Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=31.88333333333333 +lat_2=30.11666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=29.66666666666667 +lon_0=-100.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=700000 +y_0=3000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas Central\",GEOGCS[\"NAD83(HA");
    add_srs_wkt (p, 1,
		 "RN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",31.88333333333333]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",30.11666666666667],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",29.66666666666667],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-100.3333333333333],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",700000],PARAMETER[\"false_northing\",300");
    add_srs_wkt (p, 13,
		 "0000],AUTHORITY[\"EPSG\",\"2846\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2847, "epsg", 2847,
		      "NAD83(HARN) / Texas South Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.28333333333333 +lat_2=28.38333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=27.83333333333333 +lon_0=-99 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=4000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas South Central\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",30.28333333");
    add_srs_wkt (p, 9,
		 "333333],PARAMETER[\"standard_parallel_2\",28.38333333333");
    add_srs_wkt (p, 10,
		 "333],PARAMETER[\"latitude_of_origin\",27.83333333333333]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"central_meridian\",-99],PARAMETER[\"false_e");
    add_srs_wkt (p, 12,
		 "asting\",600000],PARAMETER[\"false_northing\",4000000],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"2847\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 14, "NORTH]]");
    p = add_epsg_def (first, last, 2848, "epsg", 2848,
		      "NAD83(HARN) / Texas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.66666666666667 +lon_0=-98.5 +x_0=300000 +y");
    add_proj4text (p, 2, "_0=5000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas South\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",27.83333333333333],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",26.16666666666667],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",25.66666666666667],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-98.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",300000],PARAMETER[\"false_northing\",5000000],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"2848\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2849, "epsg", 2849,
		      "NAD83(HARN) / Utah North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000 +");
    add_proj4text (p, 2, "y_0=1000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah North\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",41.78333333333333],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",40.71666666666667],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",40.33333333333334],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-111.5],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",500000],PARAMETER[\"false_northing\",1000000],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"2849\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 14, "]");
    p = add_epsg_def (first, last, 2850, "epsg", 2850,
		      "NAD83(HARN) / Utah Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000 +y_0=2000000 ");
    add_proj4text (p, 2, "+ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah Central\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",40.65],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",39.01666666666667],PARAMETER[\"lat");
    add_srs_wkt (p, 10,
		 "itude_of_origin\",38.33333333333334],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-111.5],PARAMETER[\"false_easting\",500000],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_northing\",2000000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"2850\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2851, "epsg", 2851,
		      "NAD83(HARN) / Utah South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000 +y_0=3000000 ");
    add_proj4text (p, 2, "+ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah South\",GEOGCS[\"NAD83(HARN)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",38.35],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",37.21666666666667],PARAMETER[\"lati");
    add_srs_wkt (p, 10,
		 "tude_of_origin\",36.66666666666666],PARAMETER[\"central_");
    add_srs_wkt (p, 11,
		 "meridian\",-111.5],PARAMETER[\"false_easting\",500000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",3000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 13, "\"2851\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2852, "epsg", 2852, "NAD83(HARN) / Vermont");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-72.5 +k=0.999964286 +x_0");
    add_proj4text (p, 1, "=500000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Vermont\",GEOGCS[\"NAD83(HARN)\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",42.5],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-72.5],PARAMETER[\"scale_factor\",0.999964286],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"2852\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2853, "epsg", 2853,
		      "NAD83(HARN) / Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.2 +lat_2=38.03333333333333 +lat_0=37");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-78.5 +x_0=3500000 +y_0=2000000 +");
    add_proj4text (p, 2, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Virginia North\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",39.2],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_2\",38.03333333333333],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",37.66666666666666],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-78.5],PARAMETER[\"false_easting\",3500000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",2000000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13, "\",\"2853\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2854, "epsg", 2854,
		      "NAD83(HARN) / Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=36.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.33333333333334 +lon_0=-78.5 +x_0=3500000 +");
    add_proj4text (p, 2, "y_0=1000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Virginia South\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",37.9666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",36.76666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",36.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-78.5],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",3500000],PARAMETER[\"false_northing\",1000000],AUT");
    add_srs_wkt (p, 13,
		 "HORITY[\"EPSG\",\"2854\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 2855, "epsg", 2855,
		      "NAD83(HARN) / Washington North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.5 +lat_0=47");
    add_proj4text (p, 1,
		   " +lon_0=-120.8333333333333 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Washington North\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",48.7333333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",47.5],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",47],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 11,
		 "120.8333333333333],PARAMETER[\"false_easting\",500000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2855");
    add_srs_wkt (p, 13, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2856, "epsg", 2856,
		      "NAD83(HARN) / Washington South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.33333333333334 +lat_2=45.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=45.33333333333334 +lon_0=-120.5 +x_0=500000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Washington South\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",47.3333333333333");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",45.83333333333334],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",45.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-120.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"2856\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2857, "epsg", 2857,
		      "NAD83(HARN) / West Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1, "5 +x_0=600000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / West Virginia North\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",40.25],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"standard_parallel_2\",39],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 10,
		 "f_origin\",38.5],PARAMETER[\"central_meridian\",-79.5],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",600000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2857\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 13, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2858, "epsg", 2858,
		      "NAD83(HARN) / West Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / West Virginia South\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",38.88333333");
    add_srs_wkt (p, 9,
		 "333333],PARAMETER[\"standard_parallel_2\",37.48333333333");
    add_srs_wkt (p, 10,
		 "333],PARAMETER[\"latitude_of_origin\",37],PARAMETER[\"ce");
    add_srs_wkt (p, 11,
		 "ntral_meridian\",-81],PARAMETER[\"false_easting\",600000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "858\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2859, "epsg", 2859,
		      "NAD83(HARN) / Wisconsin North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.76666666666667 +lat_2=45.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=45.16666666666666 +lon_0=-90 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wisconsin North\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",46.7666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",45.56666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",45.16666666666666],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-90],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",600000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"2859\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2860, "epsg", 2860,
		      "NAD83(HARN) / Wisconsin Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.5 +lat_2=44.25 +lat_0=43.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wisconsin Central\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",45.5],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_2\",44.25],PARAMETER[\"latitude_of_o");
    add_srs_wkt (p, 10,
		 "rigin\",43.83333333333334],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-90],PARAMETER[\"false_easting\",600000],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"2860\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2861, "epsg", 2861,
		      "NAD83(HARN) / Wisconsin South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.06666666666667 +lat_2=42.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=42 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wisconsin South\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",44.0666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",42.73333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",42],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-90],PARAMETER[\"false_easting\",600000],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2861\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2862, "epsg", 2862,
		      "NAD83(HARN) / Wyoming East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-105.1666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=200000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming East\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",40.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-105.1666666666667],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999375],PARAMETER[\"false_easting\",200000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2862\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2863, "epsg", 2863,
		      "NAD83(HARN) / Wyoming East Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-107.3333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=400000 +y_0=100000 +ellps=GRS80 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming East Central\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",40.5],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-107.3333333333333],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999375],PARAMETER[\"false_easting\",400000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",100000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"2863\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2864, "epsg", 2864,
		      "NAD83(HARN) / Wyoming West Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-108.75 +k=0.9999375 +x_0");
    add_proj4text (p, 1, "=600000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming West Central\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",40.5],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-108.75],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999375],PARAMETER[\"false_easting\",600000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2864\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2865, "epsg", 2865,
		      "NAD83(HARN) / Wyoming West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-110.0833333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=800000 +y_0=100000 +ellps=GRS80 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming West\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",40.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-110.0833333333333],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999375],PARAMETER[\"false_easting\",800000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",100000],AUTHORITY[\"EPSG\",\"2865\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2866, "epsg", 2866,
		      "NAD83(HARN) / Puerto Rico and Virgin Is.");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18.43333333333333 +lat_2=18.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=17.83333333333333 +lon_0=-66.43333333333334 +");
    add_proj4text (p, 2,
		   "x_0=200000 +y_0=200000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Puerto Rico and Virgin Is.\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_");
    add_srs_wkt (p, 2,
		 "Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Confo");
    add_srs_wkt (p, 8,
		 "rmal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",18.4");
    add_srs_wkt (p, 9,
		 "3333333333333],PARAMETER[\"standard_parallel_2\",18.0333");
    add_srs_wkt (p, 10,
		 "3333333333],PARAMETER[\"latitude_of_origin\",17.83333333");
    add_srs_wkt (p, 11,
		 "333333],PARAMETER[\"central_meridian\",-66.4333333333333");
    add_srs_wkt (p, 12,
		 "4],PARAMETER[\"false_easting\",200000],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",200000],AUTHORITY[\"EPSG\",\"2866\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2867, "epsg", 2867,
		      "NAD83(HARN) / Arizona East (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-110.1666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arizona East (ft)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9002\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",31],PARAMETER[\"centr");
    add_srs_wkt (p, 9,
		 "al_meridian\",-110.1666666666667],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9999],PARAMETER[\"false_easting\",700000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2867\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2868, "epsg", 2868,
		      "NAD83(HARN) / Arizona Central (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-111.9166666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arizona Central (ft)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",31],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-111.9166666666667],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",700000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2868\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2869, "epsg", 2869,
		      "NAD83(HARN) / Arizona West (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-113.75 +k=0.999933333 +x_0");
    add_proj4text (p, 1, "=213360 +y_0=0 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arizona West (ft)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9002\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",31],PARAMETER[\"centr");
    add_srs_wkt (p, 9,
		 "al_meridian\",-113.75],PARAMETER[\"scale_factor\",0.9999");
    add_srs_wkt (p, 10,
		 "33333],PARAMETER[\"false_easting\",700000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"2869\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2870, "epsg", 2870,
		      "NAD83(HARN) / California zone 1 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.66666666666666 +lat_2=40 +lat_0=39.3");
    add_proj4text (p, 1,
		   "3333333333334 +lon_0=-122 +x_0=2000000.0001016 +y_0=5000");
    add_proj4text (p, 2, "00.0001016001 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 1 (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",41.66666666666666],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",40],PARAMETER[\"latitude_of_origin\",39");
    add_srs_wkt (p, 11,
		 ".33333333333334],PARAMETER[\"central_meridian\",-122],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_easting\",6561666.667],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",1640416.667],AUTHORITY[\"EPSG\",\"2870\"],AX");
    add_srs_wkt (p, 14, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2871, "epsg", 2871,
		      "NAD83(HARN) / California zone 2 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.83333333333334 +lat_2=38.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=37.66666666666666 +lon_0=-122 +x_0=2000000.00");
    add_proj4text (p, 2,
		   "01016 +y_0=500000.0001016001 +ellps=GRS80 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 2 (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",39.83333333333334],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",38.33333333333334],PARAMETER[\"latitude");
    add_srs_wkt (p, 11,
		 "_of_origin\",37.66666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 12,
		 "dian\",-122],PARAMETER[\"false_easting\",6561666.667],PA");
    add_srs_wkt (p, 13,
		 "RAMETER[\"false_northing\",1640416.667],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 14, "\",\"2871\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2872, "epsg", 2872,
		      "NAD83(HARN) / California zone 3 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.5 +lon_0=-120.5 +x_0=2000000.0001016 +y_0=");
    add_proj4text (p, 2,
		   "500000.0001016001 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 3 (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",38.43333333333333],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",37.06666666666667],PARAMETER[\"latitude");
    add_srs_wkt (p, 11,
		 "_of_origin\",36.5],PARAMETER[\"central_meridian\",-120.5");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",6561666.667],PARAMETER[\"f");
    add_srs_wkt (p, 13,
		 "alse_northing\",1640416.667],AUTHORITY[\"EPSG\",\"2872\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2873, "epsg", 2873,
		      "NAD83(HARN) / California zone 4 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.25 +lat_2=36 +lat_0=35.3333333333333");
    add_proj4text (p, 1,
		   "4 +lon_0=-119 +x_0=2000000.0001016 +y_0=500000.000101600");
    add_proj4text (p, 2, "1 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 4 (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",37.25],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 10,
		 "_2\",36],PARAMETER[\"latitude_of_origin\",35.33333333333");
    add_srs_wkt (p, 11,
		 "334],PARAMETER[\"central_meridian\",-119],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",6561666.667],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "1640416.667],AUTHORITY[\"EPSG\",\"2873\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 14, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2874, "epsg", 2874,
		      "NAD83(HARN) / California zone 5 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.46666666666667 +lat_2=34.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.5 +lon_0=-118 +x_0=2000000.0001016 +y_0=50");
    add_proj4text (p, 2, "0000.0001016001 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 5 (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",35.46666666666667],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",34.03333333333333],PARAMETER[\"latitude");
    add_srs_wkt (p, 11,
		 "_of_origin\",33.5],PARAMETER[\"central_meridian\",-118],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_easting\",6561666.667],PARAMETER[\"fal");
    add_srs_wkt (p, 13,
		 "se_northing\",1640416.667],AUTHORITY[\"EPSG\",\"2874\"],");
    add_srs_wkt (p, 14, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2875, "epsg", 2875,
		      "NAD83(HARN) / California zone 6 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.88333333333333 +lat_2=32.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=32.16666666666666 +lon_0=-116.25 +x_0=2000000");
    add_proj4text (p, 2,
		   ".0001016 +y_0=500000.0001016001 +ellps=GRS80 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California zone 6 (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",33.88333333333333],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",32.78333333333333],PARAMETER[\"latitude");
    add_srs_wkt (p, 11,
		 "_of_origin\",32.16666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 12,
		 "dian\",-116.25],PARAMETER[\"false_easting\",6561666.667]");
    add_srs_wkt (p, 13,
		 ",PARAMETER[\"false_northing\",1640416.667],AUTHORITY[\"E");
    add_srs_wkt (p, 14, "PSG\",\"2875\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2876, "epsg", 2876,
		      "NAD83(HARN) / Colorado North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.78333333333333 +lat_2=39.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=39.33333333333334 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "288036576 +y_0=304800.6096012192 +ellps=GRS80 +units=us-");
    add_proj4text (p, 3, "ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Colorado North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",40.78333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",39.71666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",39.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-105.5],PARAMETER[\"false_easting\",3000000],PARAMETE");
    add_srs_wkt (p, 13,
		 "R[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"2876\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2877, "epsg", 2877,
		      "NAD83(HARN) / Colorado Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.75 +lat_2=38.45 +lat_0=37.8333333333");
    add_proj4text (p, 1,
		   "3334 +lon_0=-105.5 +x_0=914401.8288036576 +y_0=304800.60");
    add_proj4text (p, 2, "96012192 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Colorado Central (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foo");
    add_srs_wkt (p, 7,
		 "t\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",39.75],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "2\",38.45],PARAMETER[\"latitude_of_origin\",37.833333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-105.5],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",3000000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "1000000],AUTHORITY[\"EPSG\",\"2877\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 14, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2878, "epsg", 2878,
		      "NAD83(HARN) / Colorado South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.66666666666666 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "288036576 +y_0=304800.6096012192 +ellps=GRS80 +units=us-");
    add_proj4text (p, 3, "ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Colorado South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",38.43333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",37.23333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",36.66666666666666],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-105.5],PARAMETER[\"false_easting\",3000000],PARAMETE");
    add_srs_wkt (p, 13,
		 "R[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"2878\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2879, "epsg", 2879,
		      "NAD83(HARN) / Connecticut (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.86666666666667 +lat_2=41.2 +lat_0=40");
    add_proj4text (p, 1,
		   ".83333333333334 +lon_0=-72.75 +x_0=304800.6096012192 +y_");
    add_proj4text (p, 2,
		   "0=152400.3048006096 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Connecticut (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",41.86666666666667],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",41.2],PARAMETER[\"latitude_of_origin\",40.833333");
    add_srs_wkt (p, 11,
		 "33333334],PARAMETER[\"central_meridian\",-72.75],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",1000000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",500000],AUTHORITY[\"EPSG\",\"2879\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 14, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2880, "epsg", 2880,
		      "NAD83(HARN) / Delaware (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=-75.41666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "995 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +units=us");
    add_proj4text (p, 2, "-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Delaware (ftUS)\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",38],PARAMETER[\"central_meridian\",-75.41666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",0.999995],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",656166.667],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2880\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 13, "RTH]]");
    p = add_epsg_def (first, last, 2881, "epsg", 2881,
		      "NAD83(HARN) / Florida East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-81 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +units");
    add_proj4text (p, 2, "=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Florida East (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",24.33333333333333],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "-81],PARAMETER[\"scale_factor\",0.999941177],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",656166.667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"2881\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2882, "epsg", 2882,
		      "NAD83(HARN) / Florida West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-82 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +units");
    add_proj4text (p, 2, "=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Florida West (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",24.33333333333333],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "-82],PARAMETER[\"scale_factor\",0.999941177],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",656166.667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"2882\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2883, "epsg", 2883,
		      "NAD83(HARN) / Florida North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.75 +lat_2=29.58333333333333 +lat_0=2");
    add_proj4text (p, 1,
		   "9 +lon_0=-84.5 +x_0=600000 +y_0=0 +ellps=GRS80 +units=us");
    add_proj4text (p, 2, "-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Florida North (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",30.75],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 10,
		 ",29.58333333333333],PARAMETER[\"latitude_of_origin\",29]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"central_meridian\",-84.5],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_easting\",1968500],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 13,
		 "ORITY[\"EPSG\",\"2883\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 2884, "epsg", 2884,
		      "NAD83(HARN) / Georgia East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-82.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Georgia East (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",30],PARAMETER[\"central_meridian\",-82.16666666666");
    add_srs_wkt (p, 10,
		 "667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",656166.667],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"2884\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2885, "epsg", 2885,
		      "NAD83(HARN) / Georgia West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-84.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=699999.9998983998 +y_0=0 +ellps=GRS80 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Georgia West (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",30],PARAMETER[\"central_meridian\",-84.16666666666");
    add_srs_wkt (p, 10,
		 "667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",2296583.333],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"2885\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 2886, "epsg", 2886,
		      "NAD83(HARN) / Idaho East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-112.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999473679999999 +x_0=200000.0001016002 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Idaho East (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",41.66666666666666],PARAMETER[\"central_meridian\",-112.");
    add_srs_wkt (p, 10,
		 "1666666666667],PARAMETER[\"scale_factor\",0.999947368],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",656166.667],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"2886\"],AXIS[\"X\",E");
    add_srs_wkt (p, 13, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2887, "epsg", 2887,
		      "NAD83(HARN) / Idaho Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-114 +k=0.99");
    add_proj4text (p, 1,
		   "99473679999999 +x_0=500000.0001016001 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Idaho Central (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",41.66666666666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-114],PARAMETER[\"scale_factor\",0.999947368],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",1640416.667],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",0],AUTHORITY[\"EPSG\",\"2887\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2888, "epsg", 2888,
		      "NAD83(HARN) / Idaho West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-115.75 +k=0");
    add_proj4text (p, 1,
		   ".999933333 +x_0=800000.0001016001 +y_0=0 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Idaho West (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",41.66666666666666],PARAMETER[\"central_meridian\",-115.");
    add_srs_wkt (p, 10,
		 "75],PARAMETER[\"scale_factor\",0.999933333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",2624666.667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"2888\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2889, "epsg", 2889,
		      "NAD83(HARN) / Indiana East (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=99999.99989839978 +y_0=249364.9987299975 +");
    add_proj4text (p, 2, "ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Indiana East (ftUS) (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regi");
    add_srs_wkt (p, 2,
		 "onal_Network\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"61");
    add_srs_wkt (p, 4,
		 "52\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US surv");
    add_srs_wkt (p, 7,
		 "ey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",37.5],PARAMETER[\"central_meridian\",-85");
    add_srs_wkt (p, 10,
		 ".66666666666667],PARAMETER[\"scale_factor\",0.999966667]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",328083.333],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_northing\",818125],AUTHORITY[\"EPSG\",\"2889\"],AXIS[");
    add_srs_wkt (p, 13, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2890, "epsg", 2890,
		      "NAD83(HARN) / Indiana West (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=249364.9987299975 +ellps=GRS80");
    add_proj4text (p, 2, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Indiana West (ftUS) (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regi");
    add_srs_wkt (p, 2,
		 "onal_Network\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"61");
    add_srs_wkt (p, 4,
		 "52\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US surv");
    add_srs_wkt (p, 7,
		 "ey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",37.5],PARAMETER[\"central_meridian\",-87");
    add_srs_wkt (p, 10,
		 ".08333333333333],PARAMETER[\"scale_factor\",0.999966667]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",2952750],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",818125],AUTHORITY[\"EPSG\",\"2890\"],AXIS[\"X");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2891, "epsg", 2891,
		      "NAD83(HARN) / Kentucky North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=38.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=500000.0001016001 +y_");
    add_proj4text (p, 2, "0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kentucky North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",37.96666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",38.96666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",37.5],PARAMETER[\"central_meridian\",-84.25],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_easting\",1640416.667],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"2891\"],AXIS[\"X\",E");
    add_srs_wkt (p, 14, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2892, "epsg", 2892,
		      "NAD83(HARN) / Kentucky South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.93333333333333 +lat_2=36.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001016001 +y_0=500000.0001016001 +ellps=GRS80 +units=us-");
    add_proj4text (p, 3, "ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kentucky South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",37.93333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",36.73333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",36.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-85.75],PARAMETER[\"false_easting\",1640416.667],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",1640416.667],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"2892\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2893, "epsg", 2893,
		      "NAD83(HARN) / Maryland (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.45 +lat_2=38.3 +lat_0=37.66666666666");
    add_proj4text (p, 1,
		   "666 +lon_0=-77 +x_0=399999.9998983998 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maryland (ftUS)\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",39.45],PARAMETER[\"standard_parallel_2\",38.3");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"latitude_of_origin\",37.66666666666666],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"central_meridian\",-77],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",1312333.333],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"2893\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 14, "H]]");
    p = add_epsg_def (first, last, 2894, "epsg", 2894,
		      "NAD83(HARN) / Massachusetts Mainland (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.68333333333333 +lat_2=41.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41 +lon_0=-71.5 +x_0=200000.0001016002 +y_0=7");
    add_proj4text (p, 2, "50000 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Massachusetts Mainland (ftUS)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Region");
    add_srs_wkt (p, 2,
		 "al_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US surv");
    add_srs_wkt (p, 7,
		 "ey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",42.68333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",41.71666666666667],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",41],PARAMETER[\"central_meridian\",-71");
    add_srs_wkt (p, 12,
		 ".5],PARAMETER[\"false_easting\",656166.667],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",2460625],AUTHORITY[\"EPSG\",\"2894\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2895, "epsg", 2895,
		      "NAD83(HARN) / Massachusetts Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.48333333333333 +lat_2=41.28333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-70.5 +x_0=500000.0001016001 +y_0=0");
    add_proj4text (p, 2, " +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Massachusetts Island (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional");
    add_srs_wkt (p, 2,
		 "_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",41.48333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",41.28333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",41],PARAMETER[\"central_meridian\",-70.5]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_easting\",1640416.667],PARAMETER[\"fa");
    add_srs_wkt (p, 13,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"2895\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2896, "epsg", 2896,
		      "NAD83(HARN) / Michigan North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.08333333333334 +lat_2=45.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=44.78333333333333 +lon_0=-87 +x_0=7999999.999");
    add_proj4text (p, 2, "968001 +y_0=0 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Michigan North (ft)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conforma");
    add_srs_wkt (p, 8,
		 "l_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",47.0833");
    add_srs_wkt (p, 9,
		 "3333333334],PARAMETER[\"standard_parallel_2\",45.4833333");
    add_srs_wkt (p, 10,
		 "3333333],PARAMETER[\"latitude_of_origin\",44.78333333333");
    add_srs_wkt (p, 11,
		 "333],PARAMETER[\"central_meridian\",-87],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_easting\",26246719.16],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 13,
		 "],AUTHORITY[\"EPSG\",\"2896\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 2897, "epsg", 2897,
		      "NAD83(HARN) / Michigan Central (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.7 +lat_2=44.18333333333333 +lat_0=43");
    add_proj4text (p, 1,
		   ".31666666666667 +lon_0=-84.36666666666666 +x_0=5999999.9");
    add_proj4text (p, 2, "99976001 +y_0=0 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Michigan Central (ft)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conform");
    add_srs_wkt (p, 8,
		 "al_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",45.7],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"standard_parallel_2\",44.18333333333333],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"latitude_of_origin\",43.31666666666667],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"central_meridian\",-84.36666666666666],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",19685039.37],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"2897\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2898, "epsg", 2898,
		      "NAD83(HARN) / Michigan South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.66666666666666 +lat_2=42.1 +lat_0=41");
    add_proj4text (p, 1,
		   ".5 +lon_0=-84.36666666666666 +x_0=3999999.999984 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Michigan South (ft)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conforma");
    add_srs_wkt (p, 8,
		 "l_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43.6666");
    add_srs_wkt (p, 9,
		 "6666666666],PARAMETER[\"standard_parallel_2\",42.1],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",41.5],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-84.36666666666666],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",13123359.58],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"2898\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2899, "epsg", 2899,
		      "NAD83(HARN) / Mississippi East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-88.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +units=u");
    add_proj4text (p, 2, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Mississippi East (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foo");
    add_srs_wkt (p, 7,
		 "t\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",29.5],PARAMETER[\"central_meridian\",-88.83333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"scale_factor\",0.99995],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",984250.0000000002],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2899\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2900, "epsg", 2900,
		      "NAD83(HARN) / Mississippi West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-90.33333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=699999.9998983998 +y_0=0 +ellps=GRS80 +units=u");
    add_proj4text (p, 2, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Mississippi West (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foo");
    add_srs_wkt (p, 7,
		 "t\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",29.5],PARAMETER[\"central_meridian\",-90.33333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"scale_factor\",0.99995],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",2296583.333],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",0],AUTHORITY[\"EPSG\",\"2900\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2901, "epsg", 2901,
		      "NAD83(HARN) / Montana (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=45 +lat_0=44.25 +lon_0=-109.5");
    add_proj4text (p, 1,
		   " +x_0=599999.9999976 +y_0=0 +ellps=GRS80 +units=ft +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Montana (ft)\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conformal_Conic");
    add_srs_wkt (p, 8,
		 "_2SP\"],PARAMETER[\"standard_parallel_1\",49],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_2\",45],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",44.25],PARAMETER[\"central_meridian\",-109.5],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",1968503.937],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"2901\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2902, "epsg", 2902,
		      "NAD83(HARN) / New Mexico East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-104.3333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "909091 +x_0=165000 +y_0=0 +ellps=GRS80 +units=us-ft +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Mexico East (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",31],PARAMETER[\"central_meridian\",-104.333333333");
    add_srs_wkt (p, 10,
		 "3333],PARAMETER[\"scale_factor\",0.999909091],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",541337.5],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"2902\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2903, "epsg", 2903,
		      "NAD83(HARN) / New Mexico Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-106.25 +k=0.9999 +x_0=5000");
    add_proj4text (p, 1,
		   "00.0001016001 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Mexico Central (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_N");
    add_srs_wkt (p, 2,
		 "etwork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey f");
    add_srs_wkt (p, 7,
		 "oot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",31],PARAMETER[\"central_meridian\",-106.25],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",1640416.667],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"2903\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 2904, "epsg", 2904,
		      "NAD83(HARN) / New Mexico West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-107.8333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "916667 +x_0=830000.0001016001 +y_0=0 +ellps=GRS80 +units");
    add_proj4text (p, 2, "=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Mexico West (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",31],PARAMETER[\"central_meridian\",-107.833333333");
    add_srs_wkt (p, 10,
		 "3333],PARAMETER[\"scale_factor\",0.999916667],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",2723091.667],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"2904\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2905, "epsg", 2905,
		      "NAD83(HARN) / New York East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +units=us-ft +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York East (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",38.83333333333334],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-74.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",492125],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"2905\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 2906, "epsg", 2906,
		      "NAD83(HARN) / New York Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-76.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=249999.9998983998 +y_0=0 +ellps=GRS80 +units=u");
    add_proj4text (p, 2, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York Central (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foo");
    add_srs_wkt (p, 7,
		 "t\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",40],PARAMETER[\"central_meridian\",-76.5833333");
    add_srs_wkt (p, 10,
		 "3333333],PARAMETER[\"scale_factor\",0.9999375],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",820208.3330000002],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2906\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2907, "epsg", 2907,
		      "NAD83(HARN) / New York West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-78.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=350000.0001016001 +y_0=0 +ellps=GRS80 +units=u");
    add_proj4text (p, 2, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York West (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",40],PARAMETER[\"central_meridian\",-78.5833333333");
    add_srs_wkt (p, 10,
		 "3333],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",1148291.667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"2907\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2908, "epsg", 2908,
		      "NAD83(HARN) / New York Long Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.03333333333333 +lat_2=40.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=40.16666666666666 +lon_0=-74 +x_0=300000.0000");
    add_proj4text (p, 2, "000001 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New York Long Island (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional");
    add_srs_wkt (p, 2,
		 "_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",41.03333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",40.66666666666666],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",40.16666666666666],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-74],PARAMETER[\"false_easting\",984250.0000000");
    add_srs_wkt (p, 13,
		 "002],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 14, "\"2908\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2909, "epsg", 2909,
		      "NAD83(HARN) / North Dakota North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.43333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=47 +lon_0=-100.5 +x_0=599999.9999976 +y_0=0 +");
    add_proj4text (p, 2, "ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / North Dakota North (ft)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",48.");
    add_srs_wkt (p, 9,
		 "73333333333333],PARAMETER[\"standard_parallel_2\",47.433");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"latitude_of_origin\",47],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-100.5],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",1968503.937],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 13,
		 "ITY[\"EPSG\",\"2909\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 2910, "epsg", 2910,
		      "NAD83(HARN) / North Dakota South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.48333333333333 +lat_2=46.18333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=45.66666666666666 +lon_0=-100.5 +x_0=599999.9");
    add_proj4text (p, 2, "999976 +y_0=0 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / North Dakota South (ft)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",47.");
    add_srs_wkt (p, 9,
		 "48333333333333],PARAMETER[\"standard_parallel_2\",46.183");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"latitude_of_origin\",45.6666666");
    add_srs_wkt (p, 11,
		 "6666666],PARAMETER[\"central_meridian\",-100.5],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",1968503.937],PARAMETER[\"false_north");
    add_srs_wkt (p, 13,
		 "ing\",0],AUTHORITY[\"EPSG\",\"2910\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 14, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2911, "epsg", 2911,
		      "NAD83(HARN) / Oklahoma North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.76666666666667 +lat_2=35.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=35 +lon_0=-98 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oklahoma North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",36.76666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",35.56666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",35],PARAMETER[\"central_meridian\",-98],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",1968500],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",0],AUTHORITY[\"EPSG\",\"2911\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2912, "epsg", 2912,
		      "NAD83(HARN) / Oklahoma South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.23333333333333 +lat_2=33.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.33333333333334 +lon_0=-98 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oklahoma South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",35.23333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",33.93333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",33.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-98],PARAMETER[\"false_easting\",1968500],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2912\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2913, "epsg", 2913,
		      "NAD83(HARN) / Oregon North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46 +lat_2=44.33333333333334 +lat_0=43.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=2500000.0001424 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oregon North (ft)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",46],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"standard_parallel_2\",44.33333333333334],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"latitude_of_origin\",43.66666666666666],PARAMETER[\"ce");
    add_srs_wkt (p, 11,
		 "ntral_meridian\",-120.5],PARAMETER[\"false_easting\",820");
    add_srs_wkt (p, 12,
		 "2099.738],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"2913\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2914, "epsg", 2914,
		      "NAD83(HARN) / Oregon South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44 +lat_2=42.33333333333334 +lat_0=41.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=1500000.0001464 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oregon South (ft)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",44],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"standard_parallel_2\",42.33333333333334],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"latitude_of_origin\",41.66666666666666],PARAMETER[\"ce");
    add_srs_wkt (p, 11,
		 "ntral_meridian\",-120.5],PARAMETER[\"false_easting\",492");
    add_srs_wkt (p, 12,
		 "1259.843],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"2914\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2915, "epsg", 2915,
		      "NAD83(HARN) / Tennessee (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.41666666666666 +lat_2=35.25 +lat_0=3");
    add_proj4text (p, 1,
		   "4.33333333333334 +lon_0=-86 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Tennessee (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",36.41666666666666],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",35.25],PARAMETER[\"latitude_of_origin\",34.33333");
    add_srs_wkt (p, 11,
		 "333333334],PARAMETER[\"central_meridian\",-86],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"2915\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2916, "epsg", 2916,
		      "NAD83(HARN) / Texas North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.18333333333333 +lat_2=34.65 +lat_0=3");
    add_proj4text (p, 1,
		   "4 +lon_0=-101.5 +x_0=200000.0001016002 +y_0=999999.99989");
    add_proj4text (p, 2, "83998 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas North (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",36.18333333333333],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",34.65],PARAMETER[\"latitude_of_origin\",34],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-101.5],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",656166.667],PARAMETER[\"false_northing\",3280833.");
    add_srs_wkt (p, 13,
		 "333],AUTHORITY[\"EPSG\",\"2916\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 14, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2917, "epsg", 2917,
		      "NAD83(HARN) / Texas North Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.96666666666667 +lat_2=32.13333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=31.66666666666667 +lon_0=-98.5 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=2000000.0001016 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas North Central (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_");
    add_srs_wkt (p, 2,
		 "Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",33.96666666666667],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",32.13333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",31.66666666666667],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-98.5],PARAMETER[\"false_easting\",1968500],PAR");
    add_srs_wkt (p, 13,
		 "AMETER[\"false_northing\",6561666.667],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"2917\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2918, "epsg", 2918,
		      "NAD83(HARN) / Texas Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=31.88333333333333 +lat_2=30.11666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=29.66666666666667 +lon_0=-100.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=699999.9998983998 +y_0=3000000 +ellps=GRS80 +units=u");
    add_proj4text (p, 3, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas Central (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",31.88333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",30.11666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",29.66666666666667],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-100.3333333333333],PARAMETER[\"false_easting\",22965");
    add_srs_wkt (p, 13,
		 "83.333],PARAMETER[\"false_northing\",9842500.000000002],");
    add_srs_wkt (p, 14,
		 "AUTHORITY[\"EPSG\",\"2918\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 15, ",NORTH]]");
    p = add_epsg_def (first, last, 2919, "epsg", 2919,
		      "NAD83(HARN) / Texas South Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.28333333333333 +lat_2=28.38333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=27.83333333333333 +lon_0=-99 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=3999999.9998984 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas South Central (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_");
    add_srs_wkt (p, 2,
		 "Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",30.28333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",28.38333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",27.83333333333333],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-99],PARAMETER[\"false_easting\",1968500],PARAM");
    add_srs_wkt (p, 13,
		 "ETER[\"false_northing\",13123333.333],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"2919\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2920, "epsg", 2920,
		      "NAD83(HARN) / Texas South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.66666666666667 +lon_0=-98.5 +x_0=300000.00");
    add_proj4text (p, 2,
		   "00000001 +y_0=5000000.0001016 +ellps=GRS80 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas South (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",27.83333333333333],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",26.16666666666667],PARAMETER[\"latitude_of_origi");
    add_srs_wkt (p, 11,
		 "n\",25.66666666666667],PARAMETER[\"central_meridian\",-9");
    add_srs_wkt (p, 12,
		 "8.5],PARAMETER[\"false_easting\",984250.0000000002],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",16404166.667],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"2920\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2921, "epsg", 2921,
		      "NAD83(HARN) / Utah North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001504 +y_0=999999.9999960001 +ellps=GRS80 +units=ft +no");
    add_proj4text (p, 3, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah North (ft)\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",41.78333333");
    add_srs_wkt (p, 9,
		 "333333],PARAMETER[\"standard_parallel_2\",40.71666666666");
    add_srs_wkt (p, 10,
		 "667],PARAMETER[\"latitude_of_origin\",40.33333333333334]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"central_meridian\",-111.5],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",1640419.948],PARAMETER[\"false_northing\",32");
    add_srs_wkt (p, 13,
		 "80839.895],AUTHORITY[\"EPSG\",\"2921\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 14, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2922, "epsg", 2922,
		      "NAD83(HARN) / Utah Central (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000.0001504 +y_0=");
    add_proj4text (p, 2, "1999999.999992 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah Central (ft)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",40.65],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"standard_parallel_2\",39.01666666666667],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"latitude_of_origin\",38.33333333333334],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-111.5],PARAMETER[\"false_easting\",1");
    add_srs_wkt (p, 12,
		 "640419.948],PARAMETER[\"false_northing\",6561679.79],AUT");
    add_srs_wkt (p, 13,
		 "HORITY[\"EPSG\",\"2922\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 2923, "epsg", 2923,
		      "NAD83(HARN) / Utah South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000.0001504 +y_0=");
    add_proj4text (p, 2, "2999999.999988 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah South (ft)\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_2SP\"],PARAMETER[\"standard_parallel_1\",38.35],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"standard_parallel_2\",37.21666666666667],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"latitude_of_origin\",36.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-111.5],PARAMETER[\"false_easting\",1");
    add_srs_wkt (p, 12,
		 "640419.948],PARAMETER[\"false_northing\",9842519.685],AU");
    add_srs_wkt (p, 13,
		 "THORITY[\"EPSG\",\"2923\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 14, "ORTH]]");
    p = add_epsg_def (first, last, 2924, "epsg", 2924,
		      "NAD83(HARN) / Virginia North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.2 +lat_2=38.03333333333333 +lat_0=37");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-78.5 +x_0=3500000.0001016 +y_0=2");
    add_proj4text (p, 2, "000000.0001016 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Virginia North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",39.2],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 10,
		 "38.03333333333333],PARAMETER[\"latitude_of_origin\",37.6");
    add_srs_wkt (p, 11,
		 "6666666666666],PARAMETER[\"central_meridian\",-78.5],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_easting\",11482916.667],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",6561666.667],AUTHORITY[\"EPSG\",\"2924\"],AX");
    add_srs_wkt (p, 14, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2925, "epsg", 2925,
		      "NAD83(HARN) / Virginia South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=36.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.33333333333334 +lon_0=-78.5 +x_0=3500000.0");
    add_proj4text (p, 2,
		   "001016 +y_0=999999.9998983998 +ellps=GRS80 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Virginia South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",37.96666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",36.76666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",36.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-78.5],PARAMETER[\"false_easting\",11482916.667],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",3280833.333],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"2925\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2926, "epsg", 2926,
		      "NAD83(HARN) / Washington North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.5 +lat_0=47");
    add_proj4text (p, 1,
		   " +lon_0=-120.8333333333333 +x_0=500000.0001016001 +y_0=0");
    add_proj4text (p, 2, " +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Washington North (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foo");
    add_srs_wkt (p, 7,
		 "t\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",48.73333333333333],PARAMETER[\"standa");
    add_srs_wkt (p, 10,
		 "rd_parallel_2\",47.5],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 11,
		 "7],PARAMETER[\"central_meridian\",-120.8333333333333],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_easting\",1640416.667],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"2926\"],AXIS[\"X\",E");
    add_srs_wkt (p, 14, "AST],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_07 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 2927, "epsg", 2927,
		      "NAD83(HARN) / Washington South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.33333333333334 +lat_2=45.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=45.33333333333334 +lon_0=-120.5 +x_0=500000.0");
    add_proj4text (p, 2, "001016001 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Washington South (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Net");
    add_srs_wkt (p, 2,
		 "work\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foo");
    add_srs_wkt (p, 7,
		 "t\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",47.33333333333334],PARAMETER[\"standa");
    add_srs_wkt (p, 10,
		 "rd_parallel_2\",45.83333333333334],PARAMETER[\"latitude_");
    add_srs_wkt (p, 11,
		 "of_origin\",45.33333333333334],PARAMETER[\"central_merid");
    add_srs_wkt (p, 12,
		 "ian\",-120.5],PARAMETER[\"false_easting\",1640416.667],P");
    add_srs_wkt (p, 13,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2927");
    add_srs_wkt (p, 14, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2928, "epsg", 2928,
		      "NAD83(HARN) / Wisconsin North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.76666666666667 +lat_2=45.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=45.16666666666666 +lon_0=-90 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wisconsin North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",46.76666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",45.56666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",45.16666666666666],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-90],PARAMETER[\"false_easting\",1968500],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2928\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2929, "epsg", 2929,
		      "NAD83(HARN) / Wisconsin Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.5 +lat_2=44.25 +lat_0=43.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80 +units=us");
    add_proj4text (p, 2, "-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wisconsin Central (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",45.5],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "2\",44.25],PARAMETER[\"latitude_of_origin\",43.833333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-90],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",1968500],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"2929\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 2930, "epsg", 2930,
		      "NAD83(HARN) / Wisconsin South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.06666666666667 +lat_2=42.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=42 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wisconsin South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",44.06666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",42.73333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",42],PARAMETER[\"central_meridian\",-90],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",1968500],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",0],AUTHORITY[\"EPSG\",\"2930\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2931, "epsg", 2931, "Beduaram / TM 13 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +a=6378249.2 +b=6356515 +towgs84=-106,-87,188,0,0,0,");
    add_proj4text (p, 2, "0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beduaram / TM 13 NE\",GEOGCS[\"Beduaram\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Beduaram\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,2");
    add_srs_wkt (p, 2,
		 "93.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-");
    add_srs_wkt (p, 3,
		 "106,-87,188,0,0,0,0],AUTHORITY[\"EPSG\",\"6213\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4213\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",13],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"2931\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2932, "epsg", 2932,
		      "QND95 / Qatar National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.45 +lon_0=51.21666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "9999 +x_0=200000 +y_0=300000 +ellps=intl +towgs84=-119.4");
    add_proj4text (p, 2,
		   "25,-303.659,-11.0006,1.1643,0.174458,1.09626,3.65706 +un");
    add_proj4text (p, 3, "its=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"QND95 / Qatar National Grid\",GEOGCS[\"QND95\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Qatar_National_Datum_1995\",SPHEROID[\"Internati");
    add_srs_wkt (p, 2,
		 "onal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[-119.425,-303.659,-11.0006,1.1643,0.174458,1.09626");
    add_srs_wkt (p, 4,
		 ",3.65706],AUTHORITY[\"EPSG\",\"6614\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4614\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"latitude_of_origin\",24.45],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",51.21666666666667],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 11,
		 "9999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_northing\",300000],AUTHORITY[\"EPSG\",\"2932\"],AXIS");
    add_srs_wkt (p, 13, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2933, "epsg", 2933, "Segara / UTM zone 50S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=bessel +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Segara / UTM zone 50S\",GEOGCS[\"Segara\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Gunung_Segara\",SPHEROID[\"Bessel 1841\",6377397.155,");
    add_srs_wkt (p, 2,
		 "299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6613\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4613\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",117],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"2933\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 2934, "epsg", 2934,
		      "Segara (Jakarta) / NEIEZ (deprecated)");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=110 +k=0.997 +x_0=3900000 +y_0=900000 ");
    add_proj4text (p, 1, "+ellps=bessel +pm=jakarta +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Segara (Jakarta) / NEIEZ (deprecated)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Segara (Jakarta)\",DATUM[\"Gunung_Segara_Jakarta\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6820\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Jakarta\",106.8077194444444,AUTHORITY[\"EPSG\",\"8908");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4820\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercato");
    add_srs_wkt (p, 8,
		 "r_1SP\"],PARAMETER[\"central_meridian\",110],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.997],PARAMETER[\"false_easting\",390000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",900000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"2934\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2935, "epsg", 2935,
		      "Pulkovo 1942 / CS63 zone A1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1166666666666667 +lon_0=41.53333333");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=1300000 +y_0=0 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone A1\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.11");
    add_srs_wkt (p, 9,
		 "66666666666667],PARAMETER[\"central_meridian\",41.533333");
    add_srs_wkt (p, 10,
		 "33333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",1300000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2935\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST]]");
    p = add_epsg_def (first, last, 2936, "epsg", 2936,
		      "Pulkovo 1942 / CS63 zone A2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1166666666666667 +lon_0=44.53333333");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=2300000 +y_0=0 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone A2\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.11");
    add_srs_wkt (p, 9,
		 "66666666666667],PARAMETER[\"central_meridian\",44.533333");
    add_srs_wkt (p, 10,
		 "33333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",2300000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2936\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST]]");
    p = add_epsg_def (first, last, 2937, "epsg", 2937,
		      "Pulkovo 1942 / CS63 zone A3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1166666666666667 +lon_0=47.53333333");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=3300000 +y_0=0 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone A3\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.11");
    add_srs_wkt (p, 9,
		 "66666666666667],PARAMETER[\"central_meridian\",47.533333");
    add_srs_wkt (p, 10,
		 "33333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",3300000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2937\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST]]");
    p = add_epsg_def (first, last, 2938, "epsg", 2938,
		      "Pulkovo 1942 / CS63 zone A4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1166666666666667 +lon_0=50.53333333");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=4300000 +y_0=0 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone A4\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.11");
    add_srs_wkt (p, 9,
		 "66666666666667],PARAMETER[\"central_meridian\",50.533333");
    add_srs_wkt (p, 10,
		 "33333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",4300000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2938\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST]]");
    p = add_epsg_def (first, last, 2939, "epsg", 2939,
		      "Pulkovo 1942 / CS63 zone K2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1333333333333333 +lon_0=50.76666666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=2300000 +y_0=0 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone K2\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.13");
    add_srs_wkt (p, 9,
		 "33333333333333],PARAMETER[\"central_meridian\",50.766666");
    add_srs_wkt (p, 10,
		 "66666667],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",2300000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2939\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST]]");
    p = add_epsg_def (first, last, 2940, "epsg", 2940,
		      "Pulkovo 1942 / CS63 zone K3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1333333333333333 +lon_0=53.76666666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=3300000 +y_0=0 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone K3\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.13");
    add_srs_wkt (p, 9,
		 "33333333333333],PARAMETER[\"central_meridian\",53.766666");
    add_srs_wkt (p, 10,
		 "66666667],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",3300000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2940\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST]]");
    p = add_epsg_def (first, last, 2941, "epsg", 2941,
		      "Pulkovo 1942 / CS63 zone K4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1333333333333333 +lon_0=56.76666666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=4300000 +y_0=0 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone K4\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.13");
    add_srs_wkt (p, 9,
		 "33333333333333],PARAMETER[\"central_meridian\",56.766666");
    add_srs_wkt (p, 10,
		 "66666667],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",4300000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2941\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST]]");
    p = add_epsg_def (first, last, 2942, "epsg", 2942,
		      "Porto Santo / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=intl +towgs84=-499,-249,314,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Porto Santo / UTM zone 28N\",GEOGCS[\"Porto San");
    add_srs_wkt (p, 1,
		 "to\",DATUM[\"Porto_Santo_1936\",SPHEROID[\"International");
    add_srs_wkt (p, 2,
		 " 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS8");
    add_srs_wkt (p, 3,
		 "4[-499,-249,314,0,0,0,0],AUTHORITY[\"EPSG\",\"6615\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4615\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-15],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",0],AUTHORITY[\"EPSG\",\"2942\"],AXIS[\"Eastin");
    add_srs_wkt (p, 12, "g\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2943, "epsg", 2943,
		      "Selvagem Grande / UTM zone 28N");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Selvagem Grande / UTM zone 28N\",GEOGCS[\"Selva");
    add_srs_wkt (p, 1,
		 "gem Grande\",DATUM[\"Selvagem_Grande\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6616\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4616\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-15],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2943\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 2944, "epsg", 2944,
		      "NAD83(CSRS) / SCoPQ zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-55.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / SCoPQ zone 2\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-55.5],PARAMETER[\"scale_factor\",0.9999],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",304800],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"2944\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2945, "epsg", 2945,
		      "NAD83(CSRS) / MTM zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-58.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 3\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-58.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2945\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2946, "epsg", 2946,
		      "NAD83(CSRS) / MTM zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-61.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 4\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-61.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2946\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2947, "epsg", 2947,
		      "NAD83(CSRS) / MTM zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-64.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 5\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-64.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2947\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2948, "epsg", 2948,
		      "NAD83(CSRS) / MTM zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-67.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 6\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-67.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2948\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2949, "epsg", 2949,
		      "NAD83(CSRS) / MTM zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-70.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 7\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-70.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2949\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2950, "epsg", 2950,
		      "NAD83(CSRS) / MTM zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-73.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 8\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-73.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2950\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2951, "epsg", 2951,
		      "NAD83(CSRS) / MTM zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-76.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 9\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-76.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2951\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2952, "epsg", 2952,
		      "NAD83(CSRS) / MTM zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-79.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 10\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-79.5],PARAMETER[\"scale_factor\",0.9999],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",304800],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"2952\"],AXIS[\"E(X)\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 2953, "epsg", 2953,
		      "NAD83(CSRS) / New Brunswick Stereographic");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=46.5 +lon_0=-66.5 +k=0.999912 +x_0=2");
    add_proj4text (p, 1, "500000 +y_0=7500000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / New Brunswick Stereographic\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"614");
    add_srs_wkt (p, 4,
		 "0\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Oblique_St");
    add_srs_wkt (p, 8,
		 "ereographic\"],PARAMETER[\"latitude_of_origin\",46.5],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"central_meridian\",-66.5],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 10,
		 "ctor\",0.999912],PARAMETER[\"false_easting\",2500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",7500000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "2953\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 2954, "epsg", 2954,
		      "NAD83(CSRS) / Prince Edward Isl. Stereographic (NAD83)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=47.25 +lon_0=-63 +k=0.999912 +x_0=40");
    add_proj4text (p, 1, "0000 +y_0=800000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Prince Edward Isl. Stereographic ");
    add_srs_wkt (p, 1,
		 "(NAD83)\",GEOGCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_");
    add_srs_wkt (p, 2,
		 "Spatial_Reference_System\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Oblique_Stereographic\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",47.25],PARAMETER[\"central_meridian\",-63],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.999912],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",400000],PARAMETER[\"false_northing\",800000],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"2954\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 2955, "epsg", 2955,
		      "NAD83(CSRS) / UTM zone 11N");
    add_proj4text (p, 0, "+proj=utm +zone=11 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 11N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-117],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"2955\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2956, "epsg", 2956,
		      "NAD83(CSRS) / UTM zone 12N");
    add_proj4text (p, 0, "+proj=utm +zone=12 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 12N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-111],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"2956\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2957, "epsg", 2957,
		      "NAD83(CSRS) / UTM zone 13N");
    add_proj4text (p, 0, "+proj=utm +zone=13 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 13N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-105],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"2957\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2958, "epsg", 2958,
		      "NAD83(CSRS) / UTM zone 17N");
    add_proj4text (p, 0, "+proj=utm +zone=17 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 17N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-81],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2958\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2959, "epsg", 2959,
		      "NAD83(CSRS) / UTM zone 18N");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 18N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-75],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2959\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2960, "epsg", 2960,
		      "NAD83(CSRS) / UTM zone 19N");
    add_proj4text (p, 0, "+proj=utm +zone=19 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 19N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-69],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2960\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2961, "epsg", 2961,
		      "NAD83(CSRS) / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 20N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-63],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2961\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2962, "epsg", 2962,
		      "NAD83(CSRS) / UTM zone 21N");
    add_proj4text (p, 0, "+proj=utm +zone=21 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 21N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-57],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"2962\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2964, "epsg", 2964, "NAD27 / Alaska Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=55 +lat_2=65 +lat_0=50 +lon_0=-154 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska Albers\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Albers_Conic_E");
    add_srs_wkt (p, 8,
		 "qual_Area\"],PARAMETER[\"standard_parallel_1\",55],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_2\",65],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 10,
		 "_center\",50],PARAMETER[\"longitude_of_center\",-154],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",0],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"2964\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 2965, "epsg", 2965,
		      "NAD83 / Indiana East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=99999.99989839978 +y_0=249999.9998983998 +");
    add_proj4text (p, 2, "ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Indiana East (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",37.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-85.66666666666667],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.999966667],PARAMETER[\"false_easting\",32808");
    add_srs_wkt (p, 11,
		 "3.333],PARAMETER[\"false_northing\",820208.3330000002],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"2965\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 2966, "epsg", 2966,
		      "NAD83 / Indiana West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=249999.9998983998 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Indiana West (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",37.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-87.08333333333333],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.999966667],PARAMETER[\"false_easting\",29527");
    add_srs_wkt (p, 11,
		 "50],PARAMETER[\"false_northing\",820208.3330000002],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"2966\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 2967, "epsg", 2967,
		      "NAD83(HARN) / Indiana East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=99999.99989839978 +y_0=249999.9998983998 +");
    add_proj4text (p, 2, "ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Indiana East (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",37.5],PARAMETER[\"central_meridian\",-85.666666666");
    add_srs_wkt (p, 10,
		 "66667],PARAMETER[\"scale_factor\",0.999966667],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",328083.333],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",820208.3330000002],AUTHORITY[\"EPSG\",\"2967\"],AXIS");
    add_srs_wkt (p, 13, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2968, "epsg", 2968,
		      "NAD83(HARN) / Indiana West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=249999.9998983998 +ellps=GRS80");
    add_proj4text (p, 2, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Indiana West (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",37.5],PARAMETER[\"central_meridian\",-87.083333333");
    add_srs_wkt (p, 10,
		 "33333],PARAMETER[\"scale_factor\",0.999966667],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",2952750],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",820208.3330000002],AUTHORITY[\"EPSG\",\"2968\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2969, "epsg", 2969,
		      "Fort Marigot / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=intl +towgs84=137,248,-430,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fort Marigot / UTM zone 20N\",GEOGCS[\"Fort Mar");
    add_srs_wkt (p, 1,
		 "igot\",DATUM[\"Fort_Marigot\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "137,248,-430,0,0,0,0],AUTHORITY[\"EPSG\",\"6621\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4621\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",-63],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2969\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2970, "epsg", 2970,
		      "Guadeloupe 1948 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Guadeloupe 1948 / UTM zone 20N\",GEOGCS[\"Guade");
    add_srs_wkt (p, 1,
		 "loupe 1948\",DATUM[\"Guadeloupe_1948\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6622\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4622\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-63],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2970\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 2971, "epsg", 2971, "CSG67 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=intl +towgs84=-186,230,110,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"CSG67 / UTM zone 22N\",GEOGCS[\"CSG67\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Centre_Spatial_Guyanais_1967\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[-186,230,110,0,0,0,0],AUTHORITY[\"EPSG\",\"6623\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4623\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",-51],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"2971\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2972, "epsg", 2972, "RGFG95 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=GRS80 +towgs84=2,2,-2,0,0,0,0 ");
    add_proj4text (p, 1, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGFG95 / UTM zone 22N\",GEOGCS[\"RGFG95\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Reseau_Geodesique_Francais_Guyane_1995\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],TOWGS84[2,2,-2,0,0,0,0],AUTHORITY[\"EPSG\",\"6624\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4624\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-51],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"2972\"],AXIS[\"Eas");
    add_srs_wkt (p, 12, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2973, "epsg", 2973,
		      "Martinique 1938 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Martinique 1938 / UTM zone 20N\",GEOGCS[\"Marti");
    add_srs_wkt (p, 1,
		 "nique 1938\",DATUM[\"Martinique_1938\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6625\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4625\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-63],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2973\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 2975, "epsg", 2975, "RGR92 / UTM zone 40S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=40 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGR92 / UTM zone 40S\",GEOGCS[\"RGR92\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Reseau_Geodesique_de_la_Reunion_1992\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6627\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4627\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",57],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 11,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"2975\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2976, "epsg", 2976,
		      "Tahiti 52 / UTM zone 6S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +south +ellps=intl +towgs84=162,117,15");
    add_proj4text (p, 1, "4,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tahiti 52 / UTM zone 6S\",GEOGCS[\"Tahiti 52\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Tahiti_52\",SPHEROID[\"International 1924\",6378");
    add_srs_wkt (p, 2,
		 "388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[162,117,15");
    add_srs_wkt (p, 3,
		 "4,0,0,0,0],AUTHORITY[\"EPSG\",\"6628\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4628\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-147],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 11,
		 "00000],AUTHORITY[\"EPSG\",\"2976\"],AXIS[\"Easting\",EAS");
    add_srs_wkt (p, 12, "T],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2977, "epsg", 2977,
		      "Tahaa 54 / UTM zone 5S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tahaa 54 / UTM zone 5S\",GEOGCS[\"Tahaa 54\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Tahaa_54\",SPHEROID[\"International 1924\",6378388");
    add_srs_wkt (p, 2,
		 ",297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6629\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4629\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",-153],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"297");
    add_srs_wkt (p, 11,
		 "7\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2978, "epsg", 2978,
		      "IGN72 Nuku Hiva / UTM zone 7S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN72 Nuku Hiva / UTM zone 7S\",GEOGCS[\"IGN72 ");
    add_srs_wkt (p, 1,
		 "Nuku Hiva\",DATUM[\"IGN72_Nuku_Hiva\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6630\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4630\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-141],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"2978\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2979, "epsg", 2979,
		      "K0 1949 / UTM zone 42S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +south +ellps=intl +towgs84=145,-187,");
    add_proj4text (p, 1, "103,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"K0 1949 / UTM zone 42S (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "K0 1949\",DATUM[\"K0_1949\",SPHEROID[\"International 192");
    add_srs_wkt (p, 2,
		 "4\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[14");
    add_srs_wkt (p, 3,
		 "5,-187,103,0,0,0,0],AUTHORITY[\"EPSG\",\"6631\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4631\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",69],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",10000000],AUTHORITY[\"EPSG\",\"2979\"],AXIS[\"Easti");
    add_srs_wkt (p, 12, "ng\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2980, "epsg", 2980,
		      "Combani 1950 / UTM zone 38S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +south +ellps=intl +towgs84=-382,-59,");
    add_proj4text (p, 1, "-262,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Combani 1950 / UTM zone 38S\",GEOGCS[\"Combani ");
    add_srs_wkt (p, 1,
		 "1950\",DATUM[\"Combani_1950\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "-382,-59,-262,0,0,0,0],AUTHORITY[\"EPSG\",\"6632\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4632\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",45],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"2980\"],AXIS[\"Eas");
    add_srs_wkt (p, 12, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2981, "epsg", 2981,
		      "IGN56 Lifou / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN56 Lifou / UTM zone 58S\",GEOGCS[\"IGN56 Lif");
    add_srs_wkt (p, 1,
		 "ou\",DATUM[\"IGN56_Lifou\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6633\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4633\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",165],PARAMETER[\"sc");
    add_srs_wkt (p, 9,
		 "ale_factor\",0.9996],PARAMETER[\"false_easting\",500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11,
		 "\",\"2981\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NO");
    add_srs_wkt (p, 12, "RTH]]");
    p = add_epsg_def (first, last, 2982, "epsg", 2982,
		      "IGN72 Grand Terre / UTM zone 58S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN72 Grand Terre / UTM zone 58S (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"IGN72 Grand Terre\",DATUM[\"IGN72_Grande_Terre");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6634\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4634\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",165],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",10000000],AUTHORITY[\"EPSG\",\"2982\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2983, "epsg", 2983,
		      "ST87 Ouvea / UTM zone 58S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +towgs84=-122.383,");
    add_proj4text (p, 1,
		   "-188.696,103.344,3.5107,-4.9668,-5.7047,4.4798 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ST87 Ouvea / UTM zone 58S (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"ST87 Ouvea\",DATUM[\"ST87_Ouvea\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-122.383,-188.696,103.344,3.5107,-4.9668,-5.7047,");
    add_srs_wkt (p, 4,
		 "4.4798],AUTHORITY[\"EPSG\",\"6635\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4635\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 10,
		 "65],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"2983\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2984, "epsg", 2984,
		      "RGNC 1991 / Lambert New Caledonia (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-20.66666666666667 +lat_2=-22.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-21.5 +lon_0=166 +x_0=400000 +y_0=300000 +e");
    add_proj4text (p, 2, "llps=intl +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGNC 1991 / Lambert New Caledonia (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"RGNC 1991\",DATUM[\"Reseau_Geodesique_Nouvelle");
    add_srs_wkt (p, 2,
		 "_Caledonie_1991\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 3,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[0,0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0],AUTHORITY[\"EPSG\",\"6645\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 6,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"4645\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 8,
		 "\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"standard_parallel_1\",-20.66666666666667],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"standard_parallel_2\",-22.33333333333333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",-21.5],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",166],PARAMETER[\"false_easting\",400000],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",300000],AUTHORITY[\"EPSG\",\"2984\"],AX");
    add_srs_wkt (p, 14, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2987, "epsg", 2987,
		      "Saint Pierre et Miquelon 1950 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=clrk66 +towgs84=30,430,368,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Saint Pierre et Miquelon 1950 / UTM zone 21N\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Saint Pierre et Miquelon 1950\",DATUM[\"Saint_P");
    add_srs_wkt (p, 2,
		 "ierre_et_Miquelon_1950\",SPHEROID[\"Clarke 1866\",637820");
    add_srs_wkt (p, 3,
		 "6.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],TOWG");
    add_srs_wkt (p, 4,
		 "S84[30,430,368,0,0,0,0],AUTHORITY[\"EPSG\",\"6638\"]],PR");
    add_srs_wkt (p, 5,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 7,
		 "\"]],AUTHORITY[\"EPSG\",\"4638\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 8,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 9,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 10,
		 "ntral_meridian\",-57],PARAMETER[\"scale_factor\",0.9996]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"2987\"],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2988, "epsg", 2988, "MOP78 / UTM zone 1S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MOP78 / UTM zone 1S\",GEOGCS[\"MOP78\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "MOP78\",SPHEROID[\"International 1924\",6378388,297,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6639\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5,
		 "22\"]],AUTHORITY[\"EPSG\",\"4639\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 7,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "central_meridian\",-177],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"2988\"],AXIS");
    add_srs_wkt (p, 11, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2989, "epsg", 2989,
		      "RRAF 1991 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RRAF 1991 / UTM zone 20N\",GEOGCS[\"RRAF 1991\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Reseau_de_Reference_des_Antilles_Francaises_199");
    add_srs_wkt (p, 2,
		 "1\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"6640\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4640\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",0],PARAMETER[\"central_meridian\",-63],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.9996],PARAMETER[\"false_easting\",500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2989");
    add_srs_wkt (p, 12, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2990, "epsg", 2990,
		      "Reunion 1947 / TM Reunion (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-21.11666666666667 +lon_0=55.53333333");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=50000 +y_0=160000 +ellps=intl +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Reunion 1947 / TM Reunion (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Reunion 1947\",DATUM[\"Reunion_1947\",SPHEROID[\"Inte");
    add_srs_wkt (p, 2,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6626\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4626\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",-21.11666666666667],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",55.53333333333333],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",1],PARAMETER[\"false_easting\",50000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",160000],AUTHORITY[\"EPSG\",\"2990\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2991, "epsg", 2991,
		      "NAD83 / Oregon Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=45.5 +lat_0=41.75 +lon_0=-120");
    add_proj4text (p, 1,
		   ".5 +x_0=400000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oregon Lambert\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",43],PARAMETER[\"standard_parallel_2\",45");
    add_srs_wkt (p, 9,
		 ".5],PARAMETER[\"latitude_of_origin\",41.75],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",-120.5],PARAMETER[\"false_easting\",4");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"2991\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2992, "epsg", 2992,
		      "NAD83 / Oregon Lambert (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=45.5 +lat_0=41.75 +lon_0=-120");
    add_proj4text (p, 1,
		   ".5 +x_0=399999.9999984 +y_0=0 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 2, "+units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oregon Lambert (ft)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",43],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 9,
		 "el_2\",45.5],PARAMETER[\"latitude_of_origin\",41.75],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"central_meridian\",-120.5],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",1312335.958],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"2992\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 13, "RTH]]");
    p = add_epsg_def (first, last, 2993, "epsg", 2993,
		      "NAD83(HARN) / Oregon Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=45.5 +lat_0=41.75 +lon_0=-120");
    add_proj4text (p, 1,
		   ".5 +x_0=400000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oregon Lambert\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",43],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",45.5],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",41.75],PARAMETER[\"central_meridian\",-120.5],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",400000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"2993\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2994, "epsg", 2994,
		      "NAD83(HARN) / Oregon Lambert (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=45.5 +lat_0=41.75 +lon_0=-120");
    add_proj4text (p, 1,
		   ".5 +x_0=399999.9999984 +y_0=0 +ellps=GRS80 +units=ft +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Oregon Lambert (ft)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conforma");
    add_srs_wkt (p, 8,
		 "l_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_2\",45.5],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",41.75],PARAMETER[\"central_meridian\",-120");
    add_srs_wkt (p, 11,
		 ".5],PARAMETER[\"false_easting\",1312335.958],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"2994\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 2995, "epsg", 2995,
		      "IGN53 Mare / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN53 Mare / UTM zone 58S\",GEOGCS[\"IGN53 Mare");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"IGN53_Mare\",SPHEROID[\"International 1924\",");
    add_srs_wkt (p, 2,
		 "6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6641\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4641\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",165],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"2995\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 2996, "epsg", 2996,
		      "ST84 Ile des Pins / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ST84 Ile des Pins / UTM zone 58S\",GEOGCS[\"ST8");
    add_srs_wkt (p, 1,
		 "4 Ile des Pins\",DATUM[\"ST84_Ile_des_Pins\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],AUTHORITY[\"EPSG\",\"6642\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4642\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "65],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"2996\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 2997, "epsg", 2997,
		      "ST71 Belep / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +towgs84=-480.26,-");
    add_proj4text (p, 1,
		   "438.32,-643.429,16.3119,20.1721,-4.0349,-111.7 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ST71 Belep / UTM zone 58S\",GEOGCS[\"ST71 Belep");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"ST71_Belep\",SPHEROID[\"International 1924\",");
    add_srs_wkt (p, 2,
		 "6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-480.2");
    add_srs_wkt (p, 3,
		 "6,-438.32,-643.429,16.3119,20.1721,-4.0349,-111.7],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6643\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"464");
    add_srs_wkt (p, 7,
		 "3\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",165],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",10000000],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"2997\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 13, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 2998, "epsg", 2998,
		      "NEA74 Noumea / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NEA74 Noumea / UTM zone 58S\",GEOGCS[\"NEA74 No");
    add_srs_wkt (p, 1,
		 "umea\",DATUM[\"NEA74_Noumea\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6644\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4644\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",165],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"2998\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 2999, "epsg", 2999,
		      "Grand Comoros / UTM zone 38S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Grand Comoros / UTM zone 38S\",GEOGCS[\"Grand C");
    add_srs_wkt (p, 1,
		 "omoros\",DATUM[\"Grand_Comoros\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 2,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6646\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"464");
    add_srs_wkt (p, 6,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",45],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",10000000],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"2999\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3000, "epsg", 3000, "Segara / NEIEZ");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=110 +k=0.997 +x_0=3900000 +y_0=900000 ");
    add_proj4text (p, 1, "+ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Segara / NEIEZ\",GEOGCS[\"Segara\",DATUM[\"Gunu");
    add_srs_wkt (p, 1,
		 "ng_Segara\",SPHEROID[\"Bessel 1841\",6377397.155,299.152");
    add_srs_wkt (p, 2,
		 "8128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6613\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4613\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercat");
    add_srs_wkt (p, 7,
		 "or_1SP\"],PARAMETER[\"central_meridian\",110],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"scale_factor\",0.997],PARAMETER[\"false_easting\",3900");
    add_srs_wkt (p, 9,
		 "000],PARAMETER[\"false_northing\",900000],AUTHORITY[\"EP");
    add_srs_wkt (p, 10, "SG\",\"3000\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3001, "epsg", 3001, "Batavia / NEIEZ");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=110 +k=0.997 +x_0=3900000 +y_0=900000 ");
    add_proj4text (p, 1, "+ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Batavia / NEIEZ\",GEOGCS[\"Batavia\",DATUM[\"Ba");
    add_srs_wkt (p, 1,
		 "tavia\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6211");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4211\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercator_1");
    add_srs_wkt (p, 7,
		 "SP\"],PARAMETER[\"central_meridian\",110],PARAMETER[\"sc");
    add_srs_wkt (p, 8,
		 "ale_factor\",0.997],PARAMETER[\"false_easting\",3900000]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"false_northing\",900000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 10, ",\"3001\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3002, "epsg", 3002, "Makassar / NEIEZ");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=110 +k=0.997 +x_0=3900000 +y_0=900000 ");
    add_proj4text (p, 1,
		   "+ellps=bessel +towgs84=-587.8,519.75,145.76,0,0,0,0 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Makassar / NEIEZ\",GEOGCS[\"Makassar\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Makassar\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528");
    add_srs_wkt (p, 2,
		 "128,AUTHORITY[\"EPSG\",\"7004\"]],TOWGS84[-587.8,519.75,");
    add_srs_wkt (p, 3,
		 "145.76,0,0,0,0],AUTHORITY[\"EPSG\",\"6257\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4257\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Mercator_1SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"central_meridian\",110],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "997],PARAMETER[\"false_easting\",3900000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",900000],AUTHORITY[\"EPSG\",\"3002\"],AXIS");
    add_srs_wkt (p, 11, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3003, "epsg", 3003,
		      "Monte Mario / Italy zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=0.9996 +x_0=1500000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Monte Mario / Italy zone 1\",GEOGCS[\"Monte Mar");
    add_srs_wkt (p, 1,
		 "io\",DATUM[\"Monte_Mario\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6265\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4265\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",9],PARAMETER[\"scal");
    add_srs_wkt (p, 9,
		 "e_factor\",0.9996],PARAMETER[\"false_easting\",1500000],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"300");
    add_srs_wkt (p, 11, "3\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3004, "epsg", 3004,
		      "Monte Mario / Italy zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9996 +x_0=2520000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Monte Mario / Italy zone 2\",GEOGCS[\"Monte Mar");
    add_srs_wkt (p, 1,
		 "io\",DATUM[\"Monte_Mario\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6265\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4265\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",15],PARAMETER[\"sca");
    add_srs_wkt (p, 9,
		 "le_factor\",0.9996],PARAMETER[\"false_easting\",2520000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "04\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3005, "epsg", 3005, "NAD83 / BC Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=50 +lat_2=58.5 +lat_0=45 +lon_0=-126 +x");
    add_proj4text (p, 1,
		   "_0=1000000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / BC Albers\",GEOGCS[\"NAD83\",DATUM[\"No");
    add_srs_wkt (p, 1,
		 "rth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 2,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Albers_Conic_Equal_Area\"],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 8,
		 "lel_1\",50],PARAMETER[\"standard_parallel_2\",58.5],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"latitude_of_center\",45],PARAMETER[\"longitude_o");
    add_srs_wkt (p, 10,
		 "f_center\",-126],PARAMETER[\"false_easting\",1000000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3005\"");
    add_srs_wkt (p, 12, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3006, "epsg", 3006, "SWEREF99 TM");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 TM\",GEOGCS[\"SWEREF99\",DATUM[\"SWERE");
    add_srs_wkt (p, 1,
		 "F99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",15],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3006\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3007, "epsg", 3007, "SWEREF99 12 00");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=150000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 12 00\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",12],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",150000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "07\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3008, "epsg", 3008, "SWEREF99 13 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.5 +k=1 +x_0=150000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 13 30\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",13.5],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",15000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3008\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3009, "epsg", 3009, "SWEREF99 15 00");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=150000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 15 00\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",15],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",150000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "09\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3010, "epsg", 3010, "SWEREF99 16 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16.5 +k=1 +x_0=150000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 16 30\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",16.5],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",15000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3010\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3011, "epsg", 3011, "SWEREF99 18 00");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=150000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 18 00\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",18],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",150000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "11\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3012, "epsg", 3012, "SWEREF99 14 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=14.25 +k=1 +x_0=150000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 14 15\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",14.25],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1500");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3012\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3013, "epsg", 3013, "SWEREF99 15 45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15.75 +k=1 +x_0=150000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 15 45\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",15.75],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1500");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3013\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3014, "epsg", 3014, "SWEREF99 17 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=17.25 +k=1 +x_0=150000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 17 15\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",17.25],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1500");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3014\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3015, "epsg", 3015, "SWEREF99 18 45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18.75 +k=1 +x_0=150000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 18 45\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",18.75],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1500");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3015\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3016, "epsg", 3016, "SWEREF99 20 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=20.25 +k=1 +x_0=150000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 20 15\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",20.25],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1500");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3016\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3017, "epsg", 3017, "SWEREF99 21 45");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21.75 +k=1 +x_0=150000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 21 45\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",21.75],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1500");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3017\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3018, "epsg", 3018, "SWEREF99 23 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=23.25 +k=1 +x_0=150000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 23 15\",GEOGCS[\"SWEREF99\",DATUM[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",23.25],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1500");
    add_srs_wkt (p, 10,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3018\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3019, "epsg", 3019, "RT90 7.5 gon V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=11.30827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT90 7.5 gon V\",GEOGCS[\"RT90\",DATUM[\"Rikets");
    add_srs_wkt (p, 1,
		 "_koordinatsystem_1990\",SPHEROID[\"Bessel 1841\",6377397");
    add_srs_wkt (p, 2,
		 ".155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6124\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4124\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",11.3082777777777");
    add_srs_wkt (p, 9,
		 "8],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",1500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3019\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3020, "epsg", 3020, "RT90 5 gon V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.55827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT90 5 gon V\",GEOGCS[\"RT90\",DATUM[\"Rikets_k");
    add_srs_wkt (p, 1,
		 "oordinatsystem_1990\",SPHEROID[\"Bessel 1841\",6377397.1");
    add_srs_wkt (p, 2,
		 "55,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6124\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4124\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",13.55827777777778],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",1500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"3020\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3021, "epsg", 3021, "RT90 2.5 gon V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15.80827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT90 2.5 gon V\",GEOGCS[\"RT90\",DATUM[\"Rikets");
    add_srs_wkt (p, 1,
		 "_koordinatsystem_1990\",SPHEROID[\"Bessel 1841\",6377397");
    add_srs_wkt (p, 2,
		 ".155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6124\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4124\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",15.8082777777777");
    add_srs_wkt (p, 9,
		 "8],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",1500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3021\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3022, "epsg", 3022, "RT90 0 gon");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18.05827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT90 0 gon\",GEOGCS[\"RT90\",DATUM[\"Rikets_koo");
    add_srs_wkt (p, 1,
		 "rdinatsystem_1990\",SPHEROID[\"Bessel 1841\",6377397.155");
    add_srs_wkt (p, 2,
		 ",299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6124\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4124\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",18.05827777777778],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "1500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11, "G\",\"3022\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3023, "epsg", 3023, "RT90 2.5 gon O");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=20.30827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT90 2.5 gon O\",GEOGCS[\"RT90\",DATUM[\"Rikets");
    add_srs_wkt (p, 1,
		 "_koordinatsystem_1990\",SPHEROID[\"Bessel 1841\",6377397");
    add_srs_wkt (p, 2,
		 ".155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6124\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4124\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",20.3082777777777");
    add_srs_wkt (p, 9,
		 "8],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",1500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3023\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3024, "epsg", 3024, "RT90 5 gon O");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=22.55827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT90 5 gon O\",GEOGCS[\"RT90\",DATUM[\"Rikets_k");
    add_srs_wkt (p, 1,
		 "oordinatsystem_1990\",SPHEROID[\"Bessel 1841\",6377397.1");
    add_srs_wkt (p, 2,
		 "55,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6124\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4124\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",22.55827777777778],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",1500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"3024\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3025, "epsg", 3025, "RT38 7.5 gon V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=11.30827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT38 7.5 gon V\",GEOGCS[\"RT38\",DATUM[\"Stockh");
    add_srs_wkt (p, 1,
		 "olm_1938\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528");
    add_srs_wkt (p, 2,
		 "128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "308\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4308\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 7,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"central_meridian\",11.30827777777778],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"scale_factor\",1],PARAMETER[\"false_easting\",1500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "25\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3026, "epsg", 3026, "RT38 5 gon V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.55827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT38 5 gon V\",GEOGCS[\"RT38\",DATUM[\"Stockhol");
    add_srs_wkt (p, 1,
		 "m_1938\",SPHEROID[\"Bessel 1841\",6377397.155,299.152812");
    add_srs_wkt (p, 2,
		 "8,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"630");
    add_srs_wkt (p, 3,
		 "8\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4308\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",13.55827777777778],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "cale_factor\",1],PARAMETER[\"false_easting\",1500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3026\"");
    add_srs_wkt (p, 11, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3027, "epsg", 3027, "RT38 2.5 gon V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15.80827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT38 2.5 gon V\",GEOGCS[\"RT38\",DATUM[\"Stockh");
    add_srs_wkt (p, 1,
		 "olm_1938\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528");
    add_srs_wkt (p, 2,
		 "128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "308\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4308\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 7,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"central_meridian\",15.80827777777778],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"scale_factor\",1],PARAMETER[\"false_easting\",1500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "27\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3028, "epsg", 3028, "RT38 0 gon");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18.05827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT38 0 gon\",GEOGCS[\"RT38\",DATUM[\"Stockholm_");
    add_srs_wkt (p, 1,
		 "1938\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6308\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4308\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",18.05827777777778],PARAMETER[\"scal");
    add_srs_wkt (p, 9,
		 "e_factor\",1],PARAMETER[\"false_easting\",1500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3028\"],");
    add_srs_wkt (p, 11, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3029, "epsg", 3029, "RT38 2.5 gon O");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=20.30827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT38 2.5 gon O\",GEOGCS[\"RT38\",DATUM[\"Stockh");
    add_srs_wkt (p, 1,
		 "olm_1938\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528");
    add_srs_wkt (p, 2,
		 "128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "308\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4308\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 7,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"central_meridian\",20.30827777777778],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"scale_factor\",1],PARAMETER[\"false_easting\",1500000]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "29\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3030, "epsg", 3030, "RT38 5 gon O");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=22.55827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT38 5 gon O\",GEOGCS[\"RT38\",DATUM[\"Stockhol");
    add_srs_wkt (p, 1,
		 "m_1938\",SPHEROID[\"Bessel 1841\",6377397.155,299.152812");
    add_srs_wkt (p, 2,
		 "8,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"630");
    add_srs_wkt (p, 3,
		 "8\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4308\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",22.55827777777778],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "cale_factor\",1],PARAMETER[\"false_easting\",1500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3030\"");
    add_srs_wkt (p, 11, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3031, "epsg", 3031,
		      "WGS 84 / Antarctic Polar Stereographic");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-71 +lon_0=0 +k=1 +x_0=0 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Antarctic Polar Stereographic\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",63781");
    add_srs_wkt (p, 2,
		 "37,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Polar_Stereographic\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",-71],PARAMETER[\"central_meridian\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",1],PARAMETER[\"false_easting\",0],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3031\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3032, "epsg", 3032,
		      "WGS 84 / Australian Antarctic Polar Stereographic");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-71 +lon_0=70 +k=1 +x_0=6");
    add_proj4text (p, 1,
		   "000000 +y_0=6000000 +ellps=WGS84 +datum=WGS84 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Australian Antarctic Polar Stereograph");
    add_srs_wkt (p, 1,
		 "ic\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS");
    add_srs_wkt (p, 2,
		 " 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4326\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Polar_Stereographic\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",-71],PARAMETER[\"central_meridian\",70],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",6000000],PARAMETER[\"false_northing\",6000000],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3032\"],AXIS[\"Easting\",UNKNOWN],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3033, "epsg", 3033,
		      "WGS 84 / Australian Antarctic Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.5 +lat_2=-74.5 +lat_0=-50 +lon_0=70");
    add_proj4text (p, 1,
		   " +x_0=6000000 +y_0=6000000 +ellps=WGS84 +datum=WGS84 +un");
    add_proj4text (p, 2, "its=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Australian Antarctic Lambert\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",-68.5],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "74.5],PARAMETER[\"latitude_of_origin\",-50],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",70],PARAMETER[\"false_easting\",60000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",6000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 12,
		 "SG\",\"3033\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3034, "epsg", 3034, "ETRS89 / ETRS-LCC");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35 +lat_2=65 +lat_0=52 +lon_0=10 +x_0=4");
    add_proj4text (p, 1, "000000 +y_0=2800000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-LCC\",GEOGCS[\"ETRS89\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Terrestrial_Reference_System_1989\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"standard_parallel_1\",35],PARAMETER[\"standard_p");
    add_srs_wkt (p, 9,
		 "arallel_2\",65],PARAMETER[\"latitude_of_origin\",52],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"central_meridian\",10],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 11,
		 "g\",4000000],PARAMETER[\"false_northing\",2800000],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"3034\"],AXIS[\"Northing\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Easting\",EAST]]");
    p = add_epsg_def (first, last, 3035, "epsg", 3035, "ETRS89 / ETRS-LAEA");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=52 +lon_0=10 +x_0=4321000 +y_0=3210000");
    add_proj4text (p, 1, " +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-LAEA\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Lambert_Azimuthal_Equal_Area\"],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"latitude_of_center\",52],PARAMETER[\"longitude_");
    add_srs_wkt (p, 9,
		 "of_center\",10],PARAMETER[\"false_easting\",4321000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",3210000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3035\"],AXIS[\"Y\",NORTH],AXIS[\"X\",EAST]]");
    p = add_epsg_def (first, last, 3036, "epsg", 3036, "Moznet / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +ellps=WGS84 +towgs84=0,0,0,-0");
    add_proj4text (p, 1, ",-0,-0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Moznet / UTM zone 36S\",GEOGCS[\"Moznet\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Moznet_ITRF94\",SPHEROID[\"WGS 84\",6378137,298.25722");
    add_srs_wkt (p, 2,
		 "3563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,-0,-0,-");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"6130\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4130\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",33");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3036\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3037, "epsg", 3037, "Moznet / UTM zone 37S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +south +ellps=WGS84 +towgs84=0,0,0,-0");
    add_proj4text (p, 1, ",-0,-0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Moznet / UTM zone 37S\",GEOGCS[\"Moznet\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Moznet_ITRF94\",SPHEROID[\"WGS 84\",6378137,298.25722");
    add_srs_wkt (p, 2,
		 "3563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,-0,-0,-");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"6130\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4130\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",39");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3037\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3038, "epsg", 3038, "ETRS89 / ETRS-TM26");
    add_proj4text (p, 0, "+proj=utm +zone=26 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM26\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "27],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"3038\"],AXIS[\"Northing\",NORTH],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST]]");
    p = add_epsg_def (first, last, 3039, "epsg", 3039, "ETRS89 / ETRS-TM27");
    add_proj4text (p, 0, "+proj=utm +zone=27 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM27\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "21],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"3039\"],AXIS[\"Northing\",NORTH],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST]]");
    p = add_epsg_def (first, last, 3040, "epsg", 3040, "ETRS89 / ETRS-TM28");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM28\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "15],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"3040\"],AXIS[\"Northing\",NORTH],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST]]");
    p = add_epsg_def (first, last, 3041, "epsg", 3041, "ETRS89 / ETRS-TM29");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM29\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3041\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3042, "epsg", 3042, "ETRS89 / ETRS-TM30");
    add_proj4text (p, 0, "+proj=utm +zone=30 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM30\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3042\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3043, "epsg", 3043, "ETRS89 / ETRS-TM31");
    add_proj4text (p, 0, "+proj=utm +zone=31 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM31\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",3");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"3043\"],AXIS[\"Northing\",NORTH],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST]]");
    p = add_epsg_def (first, last, 3044, "epsg", 3044, "ETRS89 / ETRS-TM32");
    add_proj4text (p, 0, "+proj=utm +zone=32 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM32\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"3044\"],AXIS[\"Northing\",NORTH],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST]]");
    p = add_epsg_def (first, last, 3045, "epsg", 3045, "ETRS89 / ETRS-TM33");
    add_proj4text (p, 0, "+proj=utm +zone=33 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM33\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3045\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3046, "epsg", 3046, "ETRS89 / ETRS-TM34");
    add_proj4text (p, 0, "+proj=utm +zone=34 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM34\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",2");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3046\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3047, "epsg", 3047, "ETRS89 / ETRS-TM35");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM35\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",2");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3047\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3048, "epsg", 3048, "ETRS89 / ETRS-TM36");
    add_proj4text (p, 0, "+proj=utm +zone=36 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM36\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",3");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3048\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3049, "epsg", 3049, "ETRS89 / ETRS-TM37");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM37\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",3");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3049\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3050, "epsg", 3050, "ETRS89 / ETRS-TM38");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM38\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",4");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3050\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3051, "epsg", 3051, "ETRS89 / ETRS-TM39");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM39\",GEOGCS[\"ETRS89\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "European_Terrestrial_Reference_System_1989\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",5");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3051\"],AXIS[\"Northing\",NORTH],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST]]");
    p = add_epsg_def (first, last, 3054, "epsg", 3054,
		      "Hjorsey 1955 / UTM zone 26N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +ellps=intl +towgs84=-73,46,-86,0,0,0");
    add_proj4text (p, 1, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hjorsey 1955 / UTM zone 26N\",GEOGCS[\"Hjorsey ");
    add_srs_wkt (p, 1,
		 "1955\",DATUM[\"Hjorsey_1955\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "-73,46,-86,0,0,0,0],AUTHORITY[\"EPSG\",\"6658\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4658\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-27],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3054\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3055, "epsg", 3055,
		      "Hjorsey 1955 / UTM zone 27N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +ellps=intl +towgs84=-73,46,-86,0,0,0");
    add_proj4text (p, 1, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hjorsey 1955 / UTM zone 27N\",GEOGCS[\"Hjorsey ");
    add_srs_wkt (p, 1,
		 "1955\",DATUM[\"Hjorsey_1955\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "-73,46,-86,0,0,0,0],AUTHORITY[\"EPSG\",\"6658\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4658\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-21],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3055\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3056, "epsg", 3056,
		      "Hjorsey 1955 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=intl +towgs84=-73,46,-86,0,0,0");
    add_proj4text (p, 1, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hjorsey 1955 / UTM zone 28N\",GEOGCS[\"Hjorsey ");
    add_srs_wkt (p, 1,
		 "1955\",DATUM[\"Hjorsey_1955\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "-73,46,-86,0,0,0,0],AUTHORITY[\"EPSG\",\"6658\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4658\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-15],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3056\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3057, "epsg", 3057, "ISN93 / Lambert 1993");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=64.25 +lat_2=65.75 +lat_0=65 +lon_0=-19");
    add_proj4text (p, 1,
		   " +x_0=500000 +y_0=500000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ISN93 / Lambert 1993\",GEOGCS[\"ISN93\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Islands_Network_1993\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 2,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6659\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4659\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"standard_parallel_1\",64.25],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_2\",65.75],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 10,
		 "65],PARAMETER[\"central_meridian\",-19],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",500000],PARAMETER[\"false_northing\",500000]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"3057\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3058, "epsg", 3058,
		      "Helle 1954 / Jan Mayen Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-8.5 +k=1 +x_0=50000 +y_0=-7");
    add_proj4text (p, 1,
		   "800000 +ellps=intl +towgs84=982.609,552.753,-540.873,32.");
    add_proj4text (p, 2, "3934,-153.257,-96.2266,16.805 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Helle 1954 / Jan Mayen Grid\",GEOGCS[\"Helle 19");
    add_srs_wkt (p, 1,
		 "54\",DATUM[\"Helle_1954\",SPHEROID[\"International 1924\"");
    add_srs_wkt (p, 2,
		 ",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[982.6");
    add_srs_wkt (p, 3,
		 "09,552.753,-540.873,32.3934,-153.257,-96.2266,16.805],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6660\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4660\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-8.5],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",-7800000],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3058\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3059, "epsg", 3059, "LKS92 / Latvia TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=-6000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LKS92 / Latvia TM\",GEOGCS[\"LKS92\",DATUM[\"La");
    add_srs_wkt (p, 1,
		 "tvia_1992\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6661\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "661\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_origin\",0],PARAMETER[\"central_meridian\",24],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",-6000000],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3059\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3060, "epsg", 3060,
		      "IGN72 Grande Terre / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN72 Grande Terre / UTM zone 58S\",GEOGCS[\"IG");
    add_srs_wkt (p, 1,
		 "N72 Grande Terre\",DATUM[\"IGN72_Grande_Terre\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6634\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4662\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",165],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",500000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 11,
		 "00],AUTHORITY[\"EPSG\",\"3060\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_08 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 3061, "epsg", 3061,
		      "Porto Santo 1995 / UTM zone 28N");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Porto Santo 1995 / UTM zone 28N\",GEOGCS[\"Port");
    add_srs_wkt (p, 1,
		 "o Santo 1995\",DATUM[\"Porto_Santo_1995\",SPHEROID[\"Int");
    add_srs_wkt (p, 2,
		 "ernational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6663\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4663\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",-15]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_ea");
    add_srs_wkt (p, 10,
		 "sting\",500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3061\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 12, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 3062, "epsg", 3062,
		      "Azores Oriental 1995 / UTM zone 26N");
    add_proj4text (p, 0, "+proj=utm +zone=26 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Azores Oriental 1995 / UTM zone 26N\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Azores Oriental 1995\",DATUM[\"Azores_Oriental_Islands_1");
    add_srs_wkt (p, 2,
		 "995\",SPHEROID[\"International 1924\",6378388,297,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6664\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4664\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-27],PARAMETER[\"scale_factor\",0.9996]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3062\"],AXIS[\"Easting");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3063, "epsg", 3063,
		      "Azores Central 1995 / UTM zone 26N");
    add_proj4text (p, 0, "+proj=utm +zone=26 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Azores Central 1995 / UTM zone 26N\",GEOGCS[\"A");
    add_srs_wkt (p, 1,
		 "zores Central 1995\",DATUM[\"Azores_Central_Islands_1995");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6665\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4665\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-27],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3063\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3064, "epsg", 3064, "IGM95 / UTM zone 32N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGM95 / UTM zone 32N\",GEOGCS[\"IGM95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Istituto_Geografico_Militaire_1995\",SPHEROID[\"WGS 84\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6670\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4670\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",9],PARAMETER[\"scale_factor\",0.9996],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"3064\"],AXIS[\"Easting\",EAS");
    add_srs_wkt (p, 12, "T],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3065, "epsg", 3065, "IGM95 / UTM zone 33N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGM95 / UTM zone 33N\",GEOGCS[\"IGM95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Istituto_Geografico_Militaire_1995\",SPHEROID[\"WGS 84\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6670\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4670\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",15],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3065\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3066, "epsg", 3066, "ED50 / Jordan TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=37 +k=0.9998 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=-3000000 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / Jordan TM\",GEOGCS[\"ED50\",DATUM[\"Euro");
    add_srs_wkt (p, 1,
		 "pean_Datum_1950\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 2,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",37],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 9,
		 "\",0.9998],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_northing\",-3000000],AUTHORITY[\"EPSG\",\"3066\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3067, "epsg", 3067, "ETRS89 / ETRS-TM35FIN");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-TM35FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",27],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"3067\"],AXIS[\"Easting\",EAST],AXIS[\"N");
    add_srs_wkt (p, 12, "orthing\",NORTH]]");
    p = add_epsg_def (first, last, 3068, "epsg", 3068, "DHDN / Soldner Berlin");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=52.41864827777778 +lon_0=13.6272036666");
    add_proj4text (p, 1,
		   "6667 +x_0=40000 +y_0=10000 +ellps=bessel +datum=potsdam ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / Soldner Berlin\",GEOGCS[\"DHDN\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Deutsches_Hauptdreiecksnetz\",SPHEROID[\"Bessel 1841\",6");
    add_srs_wkt (p, 2,
		 "377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6314\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "314\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitude_of_o");
    add_srs_wkt (p, 8,
		 "rigin\",52.41864827777778],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",13.62720366666667],PARAMETER[\"false_easting\",40000],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",10000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3068\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3069, "epsg", 3069,
		      "NAD27 / Wisconsin Transverse Mercator");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-90 +k=0.9996 +x_0=500000 +y");
    add_proj4text (p, 1,
		   "_0=-4500000 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wisconsin Transverse Mercator\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-90],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",-4500000],AUTHORITY[\"EPSG\",\"3069\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3070, "epsg", 3070,
		      "NAD83 / Wisconsin Transverse Mercator");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-90 +k=0.9996 +x_0=520000 +y");
    add_proj4text (p, 1,
		   "_0=-4480000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wisconsin Transverse Mercator\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "90],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",520000],PARAMETER[\"false_northing\",-4480000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"3070\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 3071, "epsg", 3071,
		      "NAD83(HARN) / Wisconsin Transverse Mercator");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-90 +k=0.9996 +x_0=520000 +y");
    add_proj4text (p, 1, "_0=-4480000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wisconsin Transverse Mercator\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Region");
    add_srs_wkt (p, 2,
		 "al_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-90],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",520000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",-4480000],AUTHORITY[\"EPSG\",\"3071\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3072, "epsg", 3072,
		      "NAD83 / Maine CS2000 East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.83333333333334 +lon_0=-67.875 +k=0");
    add_proj4text (p, 1,
		   ".99998 +x_0=700000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine CS2000 East\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 8,
		 "f_origin\",43.83333333333334],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-67.875],PARAMETER[\"scale_factor\",0.99998],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",700000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"3072\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3073, "epsg", 3073,
		      "NAD83 / Maine CS2000 Central (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43 +lon_0=-69.125 +k=0.99998 +x_0=500");
    add_proj4text (p, 1,
		   "000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine CS2000 Central (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",43],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",-69.125],PARAMETER[\"scale_factor\",0.99998],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"3073\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3074, "epsg", 3074,
		      "NAD83 / Maine CS2000 West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.375 +k=0");
    add_proj4text (p, 1,
		   ".99998 +x_0=300000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine CS2000 West\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 8,
		 "f_origin\",42.83333333333334],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-70.375],PARAMETER[\"scale_factor\",0.99998],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",300000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"3074\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3075, "epsg", 3075,
		      "NAD83(HARN) / Maine CS2000 East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.83333333333334 +lon_0=-67.875 +k=0");
    add_proj4text (p, 1,
		   ".99998 +x_0=700000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine CS2000 East\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",43.83333333333334],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"central_meridian\",-67.875],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.99998],PARAMETER[\"false_easting\",700000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3075\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3076, "epsg", 3076,
		      "NAD83(HARN) / Maine CS2000 Central (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43 +lon_0=-69.125 +k=0.99998 +x_0=500");
    add_proj4text (p, 1, "000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine CS2000 Central (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Re");
    add_srs_wkt (p, 2,
		 "gional_Network\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6152\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",43],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"central_meridian\",-69.125],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.99998],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3076\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3077, "epsg", 3077,
		      "NAD83(HARN) / Maine CS2000 West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.375 +k=0");
    add_proj4text (p, 1,
		   ".99998 +x_0=300000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine CS2000 West\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",42.83333333333334],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"central_meridian\",-70.375],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.99998],PARAMETER[\"false_easting\",300000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3077\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3078, "epsg", 3078,
		      "NAD83 / Michigan Oblique Mercator");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=45.30916666666666 +lonc=-86 +alpha=33");
    add_proj4text (p, 1,
		   "7.25556 +k=0.9996 +x_0=2546731.496 +y_0=-4354009.816 +el");
    add_proj4text (p, 2, "lps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Michigan Oblique Mercator\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_center\",45.30916666666666],PARAMETER[\"long");
    add_srs_wkt (p, 9,
		 "itude_of_center\",-86],PARAMETER[\"azimuth\",337.25556],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"rectified_grid_angle\",337.25556],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.9996],PARAMETER[\"false_easting\",254");
    add_srs_wkt (p, 12,
		 "6731.496],PARAMETER[\"false_northing\",-4354009.816],AUT");
    add_srs_wkt (p, 13,
		 "HORITY[\"EPSG\",\"3078\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 3079, "epsg", 3079,
		      "NAD83(HARN) / Michigan Oblique Mercator");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=45.30916666666666 +lonc=-86 +alpha=33");
    add_proj4text (p, 1,
		   "7.25556 +k=0.9996 +x_0=2546731.496 +y_0=-4354009.816 +el");
    add_proj4text (p, 2, "lps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Michigan Oblique Mercator\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_N");
    add_srs_wkt (p, 2,
		 "etwork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Hotine_Oblique");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_center\",45.3091666");
    add_srs_wkt (p, 9,
		 "6666666],PARAMETER[\"longitude_of_center\",-86],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"azimuth\",337.25556],PARAMETER[\"rectified_grid_angl");
    add_srs_wkt (p, 11,
		 "e\",337.25556],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_easting\",2546731.496],PARAMETER[\"false_nor");
    add_srs_wkt (p, 13,
		 "thing\",-4354009.816],AUTHORITY[\"EPSG\",\"3079\"],AXIS[");
    add_srs_wkt (p, 14, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3080, "epsg", 3080, "NAD27 / Shackleford");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.41666666666667 +lat_2=34.91666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=31.16666666666667 +lon_0=-100 +x_0=914400 +y_");
    add_proj4text (p, 2,
		   "0=914400 +ellps=clrk66 +datum=NAD27 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Shackleford\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",27.41666666666667],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",34.91666666666666],PARAMETER[\"lat");
    add_srs_wkt (p, 10,
		 "itude_of_origin\",31.16666666666667],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-100],PARAMETER[\"false_easting\",3000000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",3000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 13, "\"3080\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3081, "epsg", 3081,
		      "NAD83 / Texas State Mapping System");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.41666666666667 +lat_2=34.91666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=31.16666666666667 +lon_0=-100 +x_0=1000000 +y");
    add_proj4text (p, 2,
		   "_0=1000000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas State Mapping System\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"standard_parallel_1\",27.41666666666667],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"standard_parallel_2\",34.91666666666666],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",31.16666666666667],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-100],PARAMETER[\"false_easting\",1000000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",1000000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13, "\",\"3081\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3082, "epsg", 3082,
		      "NAD83 / Texas Centric Lambert Conformal");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.5 +lat_2=35 +lat_0=18 +lon_0=-100 +x");
    add_proj4text (p, 1,
		   "_0=1500000 +y_0=5000000 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas Centric Lambert Conformal\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"standard_parallel_1\",27.5],PARAMETER[\"stand");
    add_srs_wkt (p, 9,
		 "ard_parallel_2\",35],PARAMETER[\"latitude_of_origin\",18");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"central_meridian\",-100],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_easting\",1500000],PARAMETER[\"false_northing\",5000000");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3082\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3083, "epsg", 3083,
		      "NAD83 / Texas Centric Albers Equal Area");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=27.5 +lat_2=35 +lat_0=18 +lon_0=-100 +x");
    add_proj4text (p, 1,
		   "_0=1500000 +y_0=6000000 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas Centric Albers Equal Area\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Albers_Conic_Equal_Area\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"standard_parallel_1\",27.5],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",35],PARAMETER[\"latitude_of_center\",18],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"longitude_of_center\",-100],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",1500000],PARAMETER[\"false_northing\",6000000]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"3083\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3084, "epsg", 3084,
		      "NAD83(HARN) / Texas Centric Lambert Conformal");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.5 +lat_2=35 +lat_0=18 +lon_0=-100 +x");
    add_proj4text (p, 1,
		   "_0=1500000 +y_0=5000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas Centric Lambert Conformal\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regi");
    add_srs_wkt (p, 2,
		 "onal_Network\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"61");
    add_srs_wkt (p, 4,
		 "52\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",2");
    add_srs_wkt (p, 9,
		 "7.5],PARAMETER[\"standard_parallel_2\",35],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",18],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 11,
		 "100],PARAMETER[\"false_easting\",1500000],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_northing\",5000000],AUTHORITY[\"EPSG\",\"3084\"],AXI");
    add_srs_wkt (p, 13, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3085, "epsg", 3085,
		      "NAD83(HARN) / Texas Centric Albers Equal Area");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=27.5 +lat_2=35 +lat_0=18 +lon_0=-100 +x");
    add_proj4text (p, 1,
		   "_0=1500000 +y_0=6000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Texas Centric Albers Equal Area\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regi");
    add_srs_wkt (p, 2,
		 "onal_Network\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"61");
    add_srs_wkt (p, 4,
		 "52\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Albers_Con");
    add_srs_wkt (p, 8,
		 "ic_Equal_Area\"],PARAMETER[\"standard_parallel_1\",27.5]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",35],PARAMETER[\"latit");
    add_srs_wkt (p, 10,
		 "ude_of_center\",18],PARAMETER[\"longitude_of_center\",-1");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_easting\",1500000],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_northing\",6000000],AUTHORITY[\"EPSG\",\"3085\"],AXIS");
    add_srs_wkt (p, 13, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3086, "epsg", 3086,
		      "NAD83 / Florida GDL Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=24 +lat_2=31.5 +lat_0=24 +lon_0=-84 +x_");
    add_proj4text (p, 1,
		   "0=400000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Florida GDL Albers\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Albers_Conic_Equal_Area\"],PARAMETER[\"stand");
    add_srs_wkt (p, 8,
		 "ard_parallel_1\",24],PARAMETER[\"standard_parallel_2\",3");
    add_srs_wkt (p, 9,
		 "1.5],PARAMETER[\"latitude_of_center\",24],PARAMETER[\"lo");
    add_srs_wkt (p, 10,
		 "ngitude_of_center\",-84],PARAMETER[\"false_easting\",400");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"3086\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3087, "epsg", 3087,
		      "NAD83(HARN) / Florida GDL Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=24 +lat_2=31.5 +lat_0=24 +lon_0=-84 +x_");
    add_proj4text (p, 1, "0=400000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Florida GDL Albers\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Albers_Conic_Equal_Area\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",24],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",31.5],PARAMETER[\"latitude_of_center\"");
    add_srs_wkt (p, 10,
		 ",24],PARAMETER[\"longitude_of_center\",-84],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",400000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"3087\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3088, "epsg", 3088,
		      "NAD83 / Kentucky Single Zone");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.08333333333334 +lat_2=38.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=1500000 ");
    add_proj4text (p, 2,
		   "+y_0=1000000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kentucky Single Zone\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",37.08333333333334],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_2\",38.66666666666666],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",36.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 11,
		 "ridian\",-85.75],PARAMETER[\"false_easting\",1500000],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3088\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3089, "epsg", 3089,
		      "NAD83 / Kentucky Single Zone (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.08333333333334 +lat_2=38.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=1500000 ");
    add_proj4text (p, 2,
		   "+y_0=999999.9998983998 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 3, "us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kentucky Single Zone (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "37.08333333333334],PARAMETER[\"standard_parallel_2\",38.");
    add_srs_wkt (p, 10,
		 "66666666666666],PARAMETER[\"latitude_of_origin\",36.3333");
    add_srs_wkt (p, 11,
		 "3333333334],PARAMETER[\"central_meridian\",-85.75],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_easting\",4921250],PARAMETER[\"false_northi");
    add_srs_wkt (p, 13,
		 "ng\",3280833.333],AUTHORITY[\"EPSG\",\"3089\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3090, "epsg", 3090,
		      "NAD83(HARN) / Kentucky Single Zone");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.08333333333334 +lat_2=38.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=1500000 ");
    add_proj4text (p, 2, "+y_0=1000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kentucky Single Zone\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",37.0833333");
    add_srs_wkt (p, 9,
		 "3333334],PARAMETER[\"standard_parallel_2\",38.6666666666");
    add_srs_wkt (p, 10,
		 "6666],PARAMETER[\"latitude_of_origin\",36.33333333333334");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"central_meridian\",-85.75],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_easting\",1500000],PARAMETER[\"false_northing\",10000");
    add_srs_wkt (p, 13,
		 "00],AUTHORITY[\"EPSG\",\"3090\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3091, "epsg", 3091,
		      "NAD83(HARN) / Kentucky Single Zone (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.08333333333334 +lat_2=38.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=1500000 ");
    add_proj4text (p, 2,
		   "+y_0=999999.9998983998 +ellps=GRS80 +units=us-ft +no_def");
    add_proj4text (p, 3, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kentucky Single Zone (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional");
    add_srs_wkt (p, 2,
		 "_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",37.08333333333334],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",38.66666666666666],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",36.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-85.75],PARAMETER[\"false_easting\",4921250],PA");
    add_srs_wkt (p, 13,
		 "RAMETER[\"false_northing\",3280833.333],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 14, "\",\"3091\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3092, "epsg", 3092, "Tokyo / UTM zone 51N");
    add_proj4text (p, 0, "+proj=utm +zone=51 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / UTM zone 51N\",GEOGCS[\"Tokyo\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6301");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",123],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 9,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3092\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3093, "epsg", 3093, "Tokyo / UTM zone 52N");
    add_proj4text (p, 0, "+proj=utm +zone=52 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / UTM zone 52N\",GEOGCS[\"Tokyo\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6301");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",129],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 9,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3093\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3094, "epsg", 3094, "Tokyo / UTM zone 53N");
    add_proj4text (p, 0, "+proj=utm +zone=53 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / UTM zone 53N\",GEOGCS[\"Tokyo\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6301");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",135],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 9,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3094\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3095, "epsg", 3095, "Tokyo / UTM zone 54N");
    add_proj4text (p, 0, "+proj=utm +zone=54 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / UTM zone 54N\",GEOGCS[\"Tokyo\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6301");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",141],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 9,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3095\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3096, "epsg", 3096, "Tokyo / UTM zone 55N");
    add_proj4text (p, 0, "+proj=utm +zone=55 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / UTM zone 55N\",GEOGCS[\"Tokyo\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6301");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",147],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 9,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3096\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3097, "epsg", 3097,
		      "JGD2000 / UTM zone 51N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / UTM zone 51N\",GEOGCS[\"JGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Japanese_Geodetic_Datum_2000\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6612\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",123],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3097\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3098, "epsg", 3098,
		      "JGD2000 / UTM zone 52N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / UTM zone 52N\",GEOGCS[\"JGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Japanese_Geodetic_Datum_2000\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6612\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",129],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3098\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3099, "epsg", 3099,
		      "JGD2000 / UTM zone 53N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / UTM zone 53N\",GEOGCS[\"JGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Japanese_Geodetic_Datum_2000\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6612\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",135],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3099\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3100, "epsg", 3100,
		      "JGD2000 / UTM zone 54N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / UTM zone 54N\",GEOGCS[\"JGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Japanese_Geodetic_Datum_2000\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6612\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",141],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3100\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3101, "epsg", 3101,
		      "JGD2000 / UTM zone 55N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JGD2000 / UTM zone 55N\",GEOGCS[\"JGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Japanese_Geodetic_Datum_2000\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6612\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4612\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",147],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3101\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3102, "epsg", 3102,
		      "American Samoa 1962 / American Samoa Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-14.26666666666667 +lat_0=-14.266666666");
    add_proj4text (p, 1,
		   "66667 +lon_0=-170 +k_0=1 +x_0=152400.3048006096 +y_0=951");
    add_proj4text (p, 2,
		   "69.31165862332 +ellps=clrk66 +towgs84=-115,118,426,0,0,0");
    add_proj4text (p, 3, ",0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"American Samoa 1962 / American Samoa Lambert\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"American Samoa 1962\",DATUM[\"American_Samoa_19");
    add_srs_wkt (p, 2,
		 "62\",SPHEROID[\"Clarke 1866\",6378206.4,294.978698213900");
    add_srs_wkt (p, 3,
		 "6,AUTHORITY[\"EPSG\",\"7008\"]],TOWGS84[-115,118,426,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6169\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4169\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_1SP\"],PARAMETER[\"latitude_of_origin\",-");
    add_srs_wkt (p, 10,
		 "14.26666666666667],PARAMETER[\"central_meridian\",-170],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",500000],PARAMETER[\"false_northing\",312234.65],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"3102\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 14, "]");
    p = add_epsg_def (first, last, 3103, "epsg", 3103,
		      "Mauritania 1999 / UTM zone 28N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mauritania 1999 / UTM zone 28N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Mauritania 1999\",DATUM[\"Mauritania_1999\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6681\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4681\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-15],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3103\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3104, "epsg", 3104,
		      "Mauritania 1999 / UTM zone 29N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mauritania 1999 / UTM zone 29N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Mauritania 1999\",DATUM[\"Mauritania_1999\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6681\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4681\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-9],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3104\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3105, "epsg", 3105,
		      "Mauritania 1999 / UTM zone 30N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=30 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mauritania 1999 / UTM zone 30N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Mauritania 1999\",DATUM[\"Mauritania_1999\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6681\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4681\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",-3],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3105\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3106, "epsg", 3106,
		      "Gulshan 303 / Bangladesh Transverse Mercator");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=90 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +a=6377276.345 +b=6356075.41314024 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Gulshan 303 / Bangladesh Transverse Mercator\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Gulshan 303\",DATUM[\"Gulshan_303\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Everest 1830 (1937 Adjustment)\",6377276.345,300.8017,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6682\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4682\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",90],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3106\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3107, "epsg", 3107, "GDA94 / SA Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-28 +lat_2=-36 +lat_0=-32 +lon_0=135 +x");
    add_proj4text (p, 1,
		   "_0=1000000 +y_0=2000000 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / SA Lambert\",GEOGCS[\"GDA94\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "eocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 8,
		 "ic_2SP\"],PARAMETER[\"standard_parallel_1\",-28],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"standard_parallel_2\",-36],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",-32],PARAMETER[\"central_meridian\",135],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",1000000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",2000000],AUTHORITY[\"EPSG\",\"3107\"],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3108, "epsg", 3108,
		      "ETRS89 / Guernsey Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=49.5 +lon_0=-2.416666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99997 +x_0=47000 +y_0=50000 +ellps=GRS80 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Guernsey Grid\",GEOGCS[\"ETRS89\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"European_Terrestrial_Reference_System_1989\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",49.5],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",-2.416666666666667],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "99997],PARAMETER[\"false_easting\",47000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",50000],AUTHORITY[\"EPSG\",\"3108\"],AXIS[");
    add_srs_wkt (p, 12, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3109, "epsg", 3109,
		      "ETRS89 / Jersey Transverse Mercator");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=49.225 +lon_0=-2.135 +k=0.99999990000");
    add_proj4text (p, 1,
		   "00001 +x_0=40000 +y_0=70000 +ellps=GRS80 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Jersey Transverse Mercator\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "ETRS89\",DATUM[\"European_Terrestrial_Reference_System_1");
    add_srs_wkt (p, 2,
		 "989\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",49.225],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"central_meridian\",-2.135],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999999],PARAMETER[\"false_easting\",40000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",70000],AUTHORITY[\"EPSG\",\"3109\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3110, "epsg", 3110, "AGD66 / Vicgrid66");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-36 +lat_2=-38 +lat_0=-37 +lon_0=145 +x");
    add_proj4text (p, 1,
		   "_0=2500000 +y_0=4500000 +ellps=aust_SA +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / Vicgrid66\",GEOGCS[\"AGD66\",DATUM[\"Au");
    add_srs_wkt (p, 1,
		 "stralian_Geodetic_Datum_1966\",SPHEROID[\"Australian Nat");
    add_srs_wkt (p, 2,
		 "ional Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "3\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"standard_parallel_1\",-36],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",-38],PARAMETER[\"latitude_of_origin\",-37],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"central_meridian\",145],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",2500000],PARAMETER[\"false_northing\",4500000],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"3110\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3111, "epsg", 3111, "GDA94 / Vicgrid94");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-36 +lat_2=-38 +lat_0=-37 +lon_0=145 +x");
    add_proj4text (p, 1,
		   "_0=2500000 +y_0=2500000 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / Vicgrid94\",GEOGCS[\"GDA94\",DATUM[\"Ge");
    add_srs_wkt (p, 1,
		 "ocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 8,
		 "ic_2SP\"],PARAMETER[\"standard_parallel_1\",-36],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"standard_parallel_2\",-38],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",-37],PARAMETER[\"central_meridian\",145],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",2500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",2500000],AUTHORITY[\"EPSG\",\"3111\"],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3112, "epsg", 3112,
		      "GDA94 / Geoscience Australia Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-18 +lat_2=-36 +lat_0=0 +lon_0=134 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / Geoscience Australia Lambert\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "GDA94\",DATUM[\"Geocentric_Datum_of_Australia_1994\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6283\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 8,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_1\",-18],PARAMETER[\"standard_parallel_2\",-36],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"latitude_of_origin\",0],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 11,
		 "n\",134],PARAMETER[\"false_easting\",0],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3112\"],AXIS[\"East");
    add_srs_wkt (p, 13, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3113, "epsg", 3113, "GDA94 / BCSG02");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-28 +lon_0=153 +k=0.99999 +x_0=50000 ");
    add_proj4text (p, 1,
		   "+y_0=100000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / BCSG02\",GEOGCS[\"GDA94\",DATUM[\"Geoce");
    add_srs_wkt (p, 1,
		 "ntric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS8");
    add_srs_wkt (p, 3,
		 "4[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",-28],PARAMETER[\"central_");
    add_srs_wkt (p, 9,
		 "meridian\",153],PARAMETER[\"scale_factor\",0.99999],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",50000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",100000],AUTHORITY[\"EPSG\",\"3113\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3114, "epsg", 3114,
		      "MAGNA-SIRGAS / Colombia Far West zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.596200416666666 +lon_0=-80.07750791");
    add_proj4text (p, 1,
		   "666666 +k=1 +x_0=1000000 +y_0=1000000 +ellps=GRS80 +towg");
    add_proj4text (p, 2, "s84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MAGNA-SIRGAS / Colombia Far West zone\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"MAGNA-SIRGAS\",DATUM[\"Marco_Geocentrico_Nacional_de_R");
    add_srs_wkt (p, 2,
		 "eferencia\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6686\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "686\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",4.596200416666666],PARAMETER[\"central_merid");
    add_srs_wkt (p, 10,
		 "ian\",-80.07750791666666],PARAMETER[\"scale_factor\",1],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",1000000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",1000000],AUTHORITY[\"EPSG\",\"3114\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3115, "epsg", 3115,
		      "MAGNA-SIRGAS / Colombia West zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.596200416666666 +lon_0=-77.07750791");
    add_proj4text (p, 1,
		   "666666 +k=1 +x_0=1000000 +y_0=1000000 +ellps=GRS80 +towg");
    add_proj4text (p, 2, "s84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MAGNA-SIRGAS / Colombia West zone\",GEOGCS[\"MA");
    add_srs_wkt (p, 1,
		 "GNA-SIRGAS\",DATUM[\"Marco_Geocentrico_Nacional_de_Refer");
    add_srs_wkt (p, 2,
		 "encia\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6686\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4686\"");
    add_srs_wkt (p, 7,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",4.596200416666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-77.07750791666666],PARAMETER[\"scale_factor\",1],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",1000000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",1000000],AUTHORITY[\"EPSG\",\"3115\"],AXIS[\"Northi");
    add_srs_wkt (p, 13, "ng\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3116, "epsg", 3116,
		      "MAGNA-SIRGAS / Colombia Bogota zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.596200416666666 +lon_0=-74.07750791");
    add_proj4text (p, 1,
		   "666666 +k=1 +x_0=1000000 +y_0=1000000 +ellps=GRS80 +towg");
    add_proj4text (p, 2, "s84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MAGNA-SIRGAS / Colombia Bogota zone\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "MAGNA-SIRGAS\",DATUM[\"Marco_Geocentrico_Nacional_de_Ref");
    add_srs_wkt (p, 2,
		 "erencia\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6686\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"468");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",4.596200416666666],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",-74.07750791666666],PARAMETER[\"scale_factor\",1],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",1000000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",1000000],AUTHORITY[\"EPSG\",\"3116\"],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3117, "epsg", 3117,
		      "MAGNA-SIRGAS / Colombia East Central zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.596200416666666 +lon_0=-71.07750791");
    add_proj4text (p, 1,
		   "666666 +k=1 +x_0=1000000 +y_0=1000000 +ellps=GRS80 +towg");
    add_proj4text (p, 2, "s84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MAGNA-SIRGAS / Colombia East Central zone\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"MAGNA-SIRGAS\",DATUM[\"Marco_Geocentrico_Nacional_");
    add_srs_wkt (p, 2,
		 "de_Referencia\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6686\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4686\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",4.596200416666666],PARAMETER[\"central_m");
    add_srs_wkt (p, 10,
		 "eridian\",-71.07750791666666],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",1],PARAMETER[\"false_easting\",1000000],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_northing\",1000000],AUTHORITY[\"EPSG\",\"3117\"],AXIS");
    add_srs_wkt (p, 13, "[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3118, "epsg", 3118,
		      "MAGNA-SIRGAS / Colombia East zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.596200416666666 +lon_0=-68.07750791");
    add_proj4text (p, 1,
		   "666666 +k=1 +x_0=1000000 +y_0=1000000 +ellps=GRS80 +towg");
    add_proj4text (p, 2, "s84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MAGNA-SIRGAS / Colombia East zone\",GEOGCS[\"MA");
    add_srs_wkt (p, 1,
		 "GNA-SIRGAS\",DATUM[\"Marco_Geocentrico_Nacional_de_Refer");
    add_srs_wkt (p, 2,
		 "encia\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6686\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4686\"");
    add_srs_wkt (p, 7,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",4.596200416666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-68.07750791666666],PARAMETER[\"scale_factor\",1],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",1000000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",1000000],AUTHORITY[\"EPSG\",\"3118\"],AXIS[\"Northi");
    add_srs_wkt (p, 13, "ng\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3119, "epsg", 3119,
		      "Douala 1948 / AEF west");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=10.5 +k=0.999 +x_0=1000000 +");
    add_proj4text (p, 1,
		   "y_0=1000000 +ellps=intl +towgs84=-206.1,-174.7,-87.7,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Douala 1948 / AEF west\",GEOGCS[\"Douala 1948\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Douala_1948\",SPHEROID[\"International 1924\",6");
    add_srs_wkt (p, 2,
		 "378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-206.1,");
    add_srs_wkt (p, 3,
		 "-174.7,-87.7,0,0,0,0],AUTHORITY[\"EPSG\",\"6192\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4192\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",10.5],PARAMETER[\"scale_factor\",0.999],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",1000000],PARAMETER[\"false_no");
    add_srs_wkt (p, 11,
		 "rthing\",1000000],AUTHORITY[\"EPSG\",\"3119\"],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3120, "epsg", 3120,
		      "Pulkovo 1942(58) / Poland zone I");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=50.625 +lon_0=21.08333333333333 +k=0");
    add_proj4text (p, 1,
		   ".9998 +x_0=4637000 +y_0=5467000 +ellps=krass +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Poland zone I\",GEOGCS[\"Pul");
    add_srs_wkt (p, 1,
		 "kovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID[\"Kra");
    add_srs_wkt (p, 2,
		 "ssowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Oblique_Stereographic\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",50.625],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",21.08333333333333],PARAMETER[\"scale_factor\",0.9998]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",4637000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",5467000],AUTHORITY[\"EPSG\",\"3120\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3121, "epsg", 3121,
		      "PRS92 / Philippines zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1,
		   "y_0=0 +ellps=clrk66 +towgs84=-127.62,-67.24,-47.04,-3.06");
    add_proj4text (p, 2, "8,4.903,1.578,-1.06 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PRS92 / Philippines zone 1\",GEOGCS[\"PRS92\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Philippine_Reference_System_1992\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],TOWGS84[-127.62,-67.24,-47.04,-3.068,4.903,1");
    add_srs_wkt (p, 4,
		 ".578,-1.06],AUTHORITY[\"EPSG\",\"6683\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4683\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",117],PARAMETER[\"scale_factor\",0.99995],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3121\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3122, "epsg", 3122,
		      "PRS92 / Philippines zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=119 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1,
		   "y_0=0 +ellps=clrk66 +towgs84=-127.62,-67.24,-47.04,-3.06");
    add_proj4text (p, 2, "8,4.903,1.578,-1.06 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PRS92 / Philippines zone 2\",GEOGCS[\"PRS92\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Philippine_Reference_System_1992\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],TOWGS84[-127.62,-67.24,-47.04,-3.068,4.903,1");
    add_srs_wkt (p, 4,
		 ".578,-1.06],AUTHORITY[\"EPSG\",\"6683\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4683\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",119],PARAMETER[\"scale_factor\",0.99995],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3122\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3123, "epsg", 3123,
		      "PRS92 / Philippines zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=121 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1,
		   "y_0=0 +ellps=clrk66 +towgs84=-127.62,-67.24,-47.04,-3.06");
    add_proj4text (p, 2, "8,4.903,1.578,-1.06 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PRS92 / Philippines zone 3\",GEOGCS[\"PRS92\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Philippine_Reference_System_1992\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],TOWGS84[-127.62,-67.24,-47.04,-3.068,4.903,1");
    add_srs_wkt (p, 4,
		 ".578,-1.06],AUTHORITY[\"EPSG\",\"6683\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4683\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",121],PARAMETER[\"scale_factor\",0.99995],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3123\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3124, "epsg", 3124,
		      "PRS92 / Philippines zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1,
		   "y_0=0 +ellps=clrk66 +towgs84=-127.62,-67.24,-47.04,-3.06");
    add_proj4text (p, 2, "8,4.903,1.578,-1.06 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PRS92 / Philippines zone 4\",GEOGCS[\"PRS92\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Philippine_Reference_System_1992\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],TOWGS84[-127.62,-67.24,-47.04,-3.068,4.903,1");
    add_srs_wkt (p, 4,
		 ".578,-1.06],AUTHORITY[\"EPSG\",\"6683\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4683\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",123],PARAMETER[\"scale_factor\",0.99995],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3124\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3125, "epsg", 3125,
		      "PRS92 / Philippines zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=125 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1,
		   "y_0=0 +ellps=clrk66 +towgs84=-127.62,-67.24,-47.04,-3.06");
    add_proj4text (p, 2, "8,4.903,1.578,-1.06 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PRS92 / Philippines zone 5\",GEOGCS[\"PRS92\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Philippine_Reference_System_1992\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],TOWGS84[-127.62,-67.24,-47.04,-3.068,4.903,1");
    add_srs_wkt (p, 4,
		 ".578,-1.06],AUTHORITY[\"EPSG\",\"6683\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4683\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",125],PARAMETER[\"scale_factor\",0.99995],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3125\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3126, "epsg", 3126, "ETRS89 / ETRS-GK19FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=19 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK19FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",19],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3126\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3127, "epsg", 3127, "ETRS89 / ETRS-GK20FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=20 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK20FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",20],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3127\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3128, "epsg", 3128, "ETRS89 / ETRS-GK21FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK21FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",21],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3128\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3129, "epsg", 3129, "ETRS89 / ETRS-GK22FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=22 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK22FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",22],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3129\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3130, "epsg", 3130, "ETRS89 / ETRS-GK23FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=23 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK23FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",23],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3130\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3131, "epsg", 3131, "ETRS89 / ETRS-GK24FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK24FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",24],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3131\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3132, "epsg", 3132, "ETRS89 / ETRS-GK25FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=25 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK25FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",25],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3132\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3133, "epsg", 3133, "ETRS89 / ETRS-GK26FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=26 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK26FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",26],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3133\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3134, "epsg", 3134, "ETRS89 / ETRS-GK27FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK27FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",27],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3134\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3135, "epsg", 3135, "ETRS89 / ETRS-GK28FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK28FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",28],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3135\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3136, "epsg", 3136, "ETRS89 / ETRS-GK29FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=29 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK29FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",29],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3136\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3137, "epsg", 3137, "ETRS89 / ETRS-GK30FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK30FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",30],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3137\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3138, "epsg", 3138, "ETRS89 / ETRS-GK31FIN");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=31 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / ETRS-GK31FIN\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",31],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3138\"],AXIS[\"Northing\",NORTH],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST]]");
    p = add_epsg_def (first, last, 3140, "epsg", 3140,
		      "Viti Levu 1912 / Viti Levu Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=-18 +lon_0=178 +x_0=109435.392 +y_0=14");
    add_proj4text (p, 1,
		   "1622.272 +a=6378306.3696 +b=6356571.996 +towgs84=51,391,");
    add_proj4text (p, 2, "-36,0,0,0,0 +to_meter=0.201168 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Viti Levu 1912 / Viti Levu Grid\",GEOGCS[\"Viti");
    add_srs_wkt (p, 1,
		 " Levu 1912\",DATUM[\"Viti_Levu_1912\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1880 (international foot)\",6378306.3696,293.46630765563");
    add_srs_wkt (p, 3,
		 "49,AUTHORITY[\"EPSG\",\"7055\"]],TOWGS84[51,391,-36,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6752\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4752\"]],UNIT[\"link\",0.201168,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9098\"]],PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",-18],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",178],PARAMETER[\"false_easting\",544000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",704000],AUTHORITY[\"EPSG\",\"3140\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3141, "epsg", 3141,
		      "Fiji 1956 / UTM zone 60S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +south +ellps=intl +towgs84=265.025,3");
    add_proj4text (p, 1, "84.929,-194.046,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fiji 1956 / UTM zone 60S\",GEOGCS[\"Fiji 1956\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Fiji_1956\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[265.025,3");
    add_srs_wkt (p, 3,
		 "84.929,-194.046,0,0,0,0],AUTHORITY[\"EPSG\",\"6721\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4721\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",177],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",10000000],AUTHORITY[\"EPSG\",\"3141\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3142, "epsg", 3142,
		      "Fiji 1956 / UTM zone 1S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +south +ellps=intl +towgs84=265.025,38");
    add_proj4text (p, 1, "4.929,-194.046,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fiji 1956 / UTM zone 1S\",GEOGCS[\"Fiji 1956\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Fiji_1956\",SPHEROID[\"International 1924\",6378");
    add_srs_wkt (p, 2,
		 "388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[265.025,38");
    add_srs_wkt (p, 3,
		 "4.929,-194.046,0,0,0,0],AUTHORITY[\"EPSG\",\"6721\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4721\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-177],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",10000000],AUTHORITY[\"EPSG\",\"3142\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3143, "epsg", 3143,
		      "Fiji 1986 / Fiji Map Grid (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-17 +lon_0=178.75 +k=0.99985 +x_0=200");
    add_proj4text (p, 1, "0000 +y_0=4000000 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fiji 1986 / Fiji Map Grid (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Fiji 1986\",DATUM[\"Fiji_Geodetic_Datum_1986\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6720\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4720\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",-17],PARAMETER[\"central_meridian\",17");
    add_srs_wkt (p, 9,
		 "8.75],PARAMETER[\"scale_factor\",0.99985],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",2000000],PARAMETER[\"false_northing\",4000");
    add_srs_wkt (p, 11,
		 "000],AUTHORITY[\"EPSG\",\"3143\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3146, "epsg", 3146,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 6\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",18],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",65000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "3146\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3147, "epsg", 3147,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger CM 18E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger CM 18E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",18],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "3147\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3148, "epsg", 3148,
		      "Indian 1960 / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +a=6377276.345 +b=6356075.41314024 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1960 / UTM zone 48N\",GEOGCS[\"Indian 19");
    add_srs_wkt (p, 1,
		 "60\",DATUM[\"Indian_1960\",SPHEROID[\"Everest 1830 (1937");
    add_srs_wkt (p, 2,
		 " Adjustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7015\"]],AUTHORITY[\"EPSG\",\"6131\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4131\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",105],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",500000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"3148\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3149, "epsg", 3149,
		      "Indian 1960 / UTM zone 49N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +a=6377276.345 +b=6356075.41314024 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1960 / UTM zone 49N\",GEOGCS[\"Indian 19");
    add_srs_wkt (p, 1,
		 "60\",DATUM[\"Indian_1960\",SPHEROID[\"Everest 1830 (1937");
    add_srs_wkt (p, 2,
		 " Adjustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7015\"]],AUTHORITY[\"EPSG\",\"6131\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4131\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",111],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",500000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"3149\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3150, "epsg", 3150,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 6\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",18],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",6500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"3150\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3151, "epsg", 3151,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger CM 18E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger CM 18E\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",18],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "3151\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3152, "epsg", 3152, "ST74");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18.05779 +k=0.99999425 +x_0=");
    add_proj4text (p, 1,
		   "100178.1808 +y_0=-6500614.7836 +ellps=GRS80 +towgs84=0,0");
    add_proj4text (p, 2, ",0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ST74\",GEOGCS[\"SWEREF99\",DATUM[\"SWEREF99\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 7,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",18.05779],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "cale_factor\",0.99999425],PARAMETER[\"false_easting\",10");
    add_srs_wkt (p, 10,
		 "0178.1808],PARAMETER[\"false_northing\",-6500614.7836],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"3152\"],AXIS[\"x\",NORTH],AXIS[\"y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3153, "epsg", 3153,
		      "NAD83(CSRS) / BC Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=50 +lat_2=58.5 +lat_0=45 +lon_0=-126 +x");
    add_proj4text (p, 1, "_0=1000000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / BC Albers\",GEOGCS[\"NAD83(CSRS)\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Albers_Conic_Equal_Area\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"standard_parallel_1\",50],PARAMETER[\"standard");
    add_srs_wkt (p, 9,
		 "_parallel_2\",58.5],PARAMETER[\"latitude_of_center\",45]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"longitude_of_center\",-126],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_easting\",1000000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 12,
		 "THORITY[\"EPSG\",\"3153\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3154, "epsg", 3154,
		      "NAD83(CSRS) / UTM zone 7N");
    add_proj4text (p, 0, "+proj=utm +zone=7 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 7N\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-141],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"3154\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3155, "epsg", 3155,
		      "NAD83(CSRS) / UTM zone 8N");
    add_proj4text (p, 0, "+proj=utm +zone=8 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 8N\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-135],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"3155\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3156, "epsg", 3156,
		      "NAD83(CSRS) / UTM zone 9N");
    add_proj4text (p, 0, "+proj=utm +zone=9 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 9N\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-129],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"3156\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3157, "epsg", 3157,
		      "NAD83(CSRS) / UTM zone 10N");
    add_proj4text (p, 0, "+proj=utm +zone=10 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 10N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-123],PARAMETER[\"scale_factor\",0.9996],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"3157\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3158, "epsg", 3158,
		      "NAD83(CSRS) / UTM zone 14N");
    add_proj4text (p, 0, "+proj=utm +zone=14 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 14N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-99],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3158\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3159, "epsg", 3159,
		      "NAD83(CSRS) / UTM zone 15N");
    add_proj4text (p, 0, "+proj=utm +zone=15 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 15N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-93],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3159\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3160, "epsg", 3160,
		      "NAD83(CSRS) / UTM zone 16N");
    add_proj4text (p, 0, "+proj=utm +zone=16 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 16N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-87],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3160\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3161, "epsg", 3161,
		      "NAD83 / Ontario MNR Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.5 +lat_2=53.5 +lat_0=0 +lon_0=-85 +x");
    add_proj4text (p, 1,
		   "_0=930000 +y_0=6430000 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Ontario MNR Lambert\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",44.5],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_2\",53.5],PARAMETER[\"latitude_of_origin\",0],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"central_meridian\",-85],PARAMETER[\"false_easting\",9");
    add_srs_wkt (p, 11,
		 "30000],PARAMETER[\"false_northing\",6430000],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3161\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3162, "epsg", 3162,
		      "NAD83(CSRS) / Ontario MNR Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.5 +lat_2=53.5 +lat_0=0 +lon_0=-85 +x");
    add_proj4text (p, 1,
		   "_0=930000 +y_0=6430000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Ontario MNR Lambert\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",44.5],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"standard_parallel_2\",53.5],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",-85],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",930000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",6430000],AUTHORITY[\"EPSG\",\"3162\"],AXIS[\"East");
    add_srs_wkt (p, 13, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3163, "epsg", 3163,
		      "RGNC91-93 / Lambert New Caledonia");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-20.66666666666667 +lat_2=-22.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-21.5 +lon_0=166 +x_0=400000 +y_0=300000 +e");
    add_proj4text (p, 2, "llps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGNC91-93 / Lambert New Caledonia\",GEOGCS[\"RG");
    add_srs_wkt (p, 1,
		 "NC91-93\",DATUM[\"Reseau_Geodesique_de_Nouvelle_Caledoni");
    add_srs_wkt (p, 2,
		 "e_91_93\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6749\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"474");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",-20.66666666666667],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",-22.33333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 11,
		 "e_of_origin\",-21.5],PARAMETER[\"central_meridian\",166]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_easting\",400000],PARAMETER[\"false_n");
    add_srs_wkt (p, 13,
		 "orthing\",300000],AUTHORITY[\"EPSG\",\"3163\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3164, "epsg", 3164,
		      "ST87 Ouvea / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=WGS84 +towgs84=-56.263,");
    add_proj4text (p, 1, "16.136,-22.856,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ST87 Ouvea / UTM zone 58S\",GEOGCS[\"ST87 Ouvea");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"ST87_Ouvea\",SPHEROID[\"WGS 84\",6378137,298.");
    add_srs_wkt (p, 2,
		 "257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[-56.263,");
    add_srs_wkt (p, 3,
		 "16.136,-22.856,0,0,0,0],AUTHORITY[\"EPSG\",\"6750\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4750\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",165],PARAMETER[\"scale_factor\",0.9996]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",10000000],AUTHORITY[\"EPSG\",\"3164\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3165, "epsg", 3165,
		      "NEA74 Noumea / Noumea Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-22.24469175 +lat_2=-22.29469175 +lat_0");
    add_proj4text (p, 1,
		   "=-22.26969175 +lon_0=166.44242575 +x_0=0.66 +y_0=1.02 +e");
    add_proj4text (p, 2, "llps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NEA74 Noumea / Noumea Lambert\",GEOGCS[\"NEA74 ");
    add_srs_wkt (p, 1,
		 "Noumea\",DATUM[\"NEA74_Noumea\",SPHEROID[\"International");
    add_srs_wkt (p, 2,
		 " 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6644\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4644");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",-22.24469175],PARAMETER[\"standard_par");
    add_srs_wkt (p, 9,
		 "allel_2\",-22.29469175],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 10,
		 ",-22.26969175],PARAMETER[\"central_meridian\",166.442425");
    add_srs_wkt (p, 11,
		 "75],PARAMETER[\"false_easting\",0.66],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",1.02],AUTHORITY[\"EPSG\",\"3165\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3166, "epsg", 3166,
		      "NEA74 Noumea / Noumea Lambert 2");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-22.24472222222222 +lat_2=-22.294722222");
    add_proj4text (p, 1,
		   "22222 +lat_0=-22.26972222222222 +lon_0=166.4425 +x_0=8.3");
    add_proj4text (p, 2,
		   "13000000000001 +y_0=-2.354 +ellps=intl +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NEA74 Noumea / Noumea Lambert 2\",GEOGCS[\"NEA7");
    add_srs_wkt (p, 1,
		 "4 Noumea\",DATUM[\"NEA74_Noumea\",SPHEROID[\"Internation");
    add_srs_wkt (p, 2,
		 "al 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6644\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"46");
    add_srs_wkt (p, 6,
		 "44\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",-22.24472222222222],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",-22.29472222222222],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",-22.26972222222222],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",166.4425],PARAMETER[\"false_easting\",8.313],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",-2.354],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3166\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3167, "epsg", 3167,
		      "Kertau (RSO) / RSO Malaya (ch)");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=102.25 +alpha=323.0257905 +k=");
    add_proj4text (p, 1,
		   "0.99984 +x_0=40000 +y_0=0 +a=6377295.664 +b=6356094.6679");
    add_proj4text (p, 2, "15204 +to_meter=20.116756 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kertau (RSO) / RSO Malaya (ch)\",GEOGCS[\"Kerta");
    add_srs_wkt (p, 1,
		 "u (RSO)\",DATUM[\"Kertau_RSO\",SPHEROID[\"Everest 1830 (");
    add_srs_wkt (p, 2,
		 "RSO 1969)\",6377295.664,300.8017,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "56\"]],AUTHORITY[\"EPSG\",\"6751\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4751\"]],UNIT[\"British chain (Sears 1922 trunca");
    add_srs_wkt (p, 7,
		 "ted)\",20.116756,AUTHORITY[\"EPSG\",\"9301\"]],PROJECTIO");
    add_srs_wkt (p, 8,
		 "N[\"Hotine_Oblique_Mercator\"],PARAMETER[\"latitude_of_c");
    add_srs_wkt (p, 9,
		 "enter\",4],PARAMETER[\"longitude_of_center\",102.25],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"azimuth\",323.0257905],PARAMETER[\"rectified_gr");
    add_srs_wkt (p, 11,
		 "id_angle\",323.1301023611111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 12,
		 ",0.99984],PARAMETER[\"false_easting\",1988.392164223695]");
    add_srs_wkt (p, 13,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"31");
    add_srs_wkt (p, 14,
		 "67\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3168, "epsg", 3168,
		      "Kertau (RSO) / RSO Malaya (m)");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=102.25 +alpha=323.0257905 +k=");
    add_proj4text (p, 1,
		   "0.99984 +x_0=804670.24 +y_0=0 +a=6377295.664 +b=6356094.");
    add_proj4text (p, 2, "667915204 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kertau (RSO) / RSO Malaya (m)\",GEOGCS[\"Kertau");
    add_srs_wkt (p, 1,
		 " (RSO)\",DATUM[\"Kertau_RSO\",SPHEROID[\"Everest 1830 (R");
    add_srs_wkt (p, 2,
		 "SO 1969)\",6377295.664,300.8017,AUTHORITY[\"EPSG\",\"705");
    add_srs_wkt (p, 3,
		 "6\"]],AUTHORITY[\"EPSG\",\"6751\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4751\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_center\",4],PARAMETER[\"longitude_of_cen");
    add_srs_wkt (p, 9,
		 "ter\",102.25],PARAMETER[\"azimuth\",323.0257905],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"rectified_grid_angle\",323.1301023611111],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"scale_factor\",0.99984],PARAMETER[\"false_easting\",8");
    add_srs_wkt (p, 12,
		 "04670.24],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13,
		 "SG\",\"3168\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 14, "NORTH]]");
    p = add_epsg_def (first, last, 3169, "epsg", 3169,
		      "RGNC91-93 / UTM zone 57S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGNC91-93 / UTM zone 57S\",GEOGCS[\"RGNC91-93\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Reseau_Geodesique_de_Nouvelle_Caledonie_91_93\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6749\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4749\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",159],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12,
		 "\"3169\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 3170, "epsg", 3170,
		      "RGNC91-93 / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGNC91-93 / UTM zone 58S\",GEOGCS[\"RGNC91-93\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Reseau_Geodesique_de_Nouvelle_Caledonie_91_93\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6749\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4749\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",165],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12,
		 "\"3170\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 3171, "epsg", 3171,
		      "RGNC91-93 / UTM zone 59S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGNC91-93 / UTM zone 59S\",GEOGCS[\"RGNC91-93\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Reseau_Geodesique_de_Nouvelle_Caledonie_91_93\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6749\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4749\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",171],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12,
		 "\"3171\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 3172, "epsg", 3172,
		      "IGN53 Mare / UTM zone 59S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN53 Mare / UTM zone 59S\",GEOGCS[\"IGN53 Mare");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"IGN53_Mare\",SPHEROID[\"International 1924\",");
    add_srs_wkt (p, 2,
		 "6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6641\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4641\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",171],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"3172\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 3174, "epsg", 3174,
		      "NAD83 / Great Lakes Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=42.122774 +lat_2=49.01518 +lat_0=45.568");
    add_proj4text (p, 1,
		   "977 +lon_0=-84.455955 +x_0=1000000 +y_0=1000000 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Great Lakes Albers\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Albers_Conic_Equal_Area\"],PARAMETER[\"stand");
    add_srs_wkt (p, 8,
		 "ard_parallel_1\",42.122774],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 9,
		 "l_2\",49.01518],PARAMETER[\"latitude_of_center\",45.5689");
    add_srs_wkt (p, 10,
		 "77],PARAMETER[\"longitude_of_center\",-84.455955],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",1000000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",1000000],AUTHORITY[\"EPSG\",\"3174\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3175, "epsg", 3175,
		      "NAD83 / Great Lakes and St Lawrence Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=42.122774 +lat_2=49.01518 +lat_0=45.568");
    add_proj4text (p, 1,
		   "977 +lon_0=-83.248627 +x_0=1000000 +y_0=1000000 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Great Lakes and St Lawrence Albers\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Albers_Conic_Equal_Area\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"standard_parallel_1\",42.122774],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",49.01518],PARAMETER[\"latitude_of_c");
    add_srs_wkt (p, 10,
		 "enter\",45.568977],PARAMETER[\"longitude_of_center\",-83");
    add_srs_wkt (p, 11,
		 ".248627],PARAMETER[\"false_easting\",1000000],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"3175\"]");
    add_srs_wkt (p, 13, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3176, "epsg", 3176,
		      "Indian 1960 / TM 106 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=106 +k=0.9996 +x_0=500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +a=6377276.345 +b=6356075.41314024 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1960 / TM 106 NE\",GEOGCS[\"Indian 1960\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Indian_1960\",SPHEROID[\"Everest 1830 (1937 Adj");
    add_srs_wkt (p, 2,
		 "ustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "5\"]],AUTHORITY[\"EPSG\",\"6131\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4131\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "06],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"3176\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 3177, "epsg", 3177, "LGD2006 / Libya TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=17 +k=0.9965000000000001 +x_");
    add_proj4text (p, 1,
		   "0=1000000 +y_0=0 +ellps=intl +towgs84=-208.406,-109.878,");
    add_proj4text (p, 2, "-2.5764,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM\",GEOGCS[\"LGD2006\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Libyan_Geodetic_Datum_2006\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-");
    add_srs_wkt (p, 3,
		 "208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 8,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",17],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9965],PARAMETER[\"false_easting\",1000000],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"3177\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3178, "epsg", 3178, "GR96 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 18N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-75],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3178\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3179, "epsg", 3179, "GR96 / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 19N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-69],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3179\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3180, "epsg", 3180, "GR96 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 20N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-63],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3180\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3181, "epsg", 3181, "GR96 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 21N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-57],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3181\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3182, "epsg", 3182, "GR96 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 22N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-51],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3182\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3183, "epsg", 3183, "GR96 / UTM zone 23N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 23N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-45],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3183\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3184, "epsg", 3184, "GR96 / UTM zone 24N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 24N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-39],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3184\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3185, "epsg", 3185, "GR96 / UTM zone 25N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 25N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-33],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3185\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3186, "epsg", 3186, "GR96 / UTM zone 26N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 26N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-27],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3186\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3187, "epsg", 3187, "GR96 / UTM zone 27N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 27N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-21],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3187\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3188, "epsg", 3188, "GR96 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 28N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-15],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3188\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3189, "epsg", 3189, "GR96 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GR96 / UTM zone 29N\",GEOGCS[\"GR96\",DATUM[\"G");
    add_srs_wkt (p, 1,
		 "reenland_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 2,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6747\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4747\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-9],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"3189\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 3190, "epsg", 3190,
		      "LGD2006 / Libya TM zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=0.99995 +x_0=200000 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,0");
    add_proj4text (p, 2, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 5\",GEOGCS[\"LGD2006\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",9],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.99995],PARAMETER[\"false_easting\",200000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3190\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3191, "epsg", 3191,
		      "LGD2006 / Libya TM zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=11 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 6\",GEOGCS[\"LGD2006\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",11],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.99995],PARAMETER[\"false_easting\",200000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3191\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3192, "epsg", 3192,
		      "LGD2006 / Libya TM zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 7\",GEOGCS[\"LGD2006\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",13],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.99995],PARAMETER[\"false_easting\",200000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3192\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_09 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 3193, "epsg", 3193,
		      "LGD2006 / Libya TM zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 8\",GEOGCS[\"LGD2006\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",15],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.99995],PARAMETER[\"false_easting\",200000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3193\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3194, "epsg", 3194,
		      "LGD2006 / Libya TM zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=17 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 9\",GEOGCS[\"LGD2006\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",17],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.99995],PARAMETER[\"false_easting\",200000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3194\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3195, "epsg", 3195,
		      "LGD2006 / Libya TM zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=19 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 10\",GEOGCS[\"LGD2006\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "TOWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",0],PARAMETER[\"central_meridian\",19],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99995],PARAMETER[\"false_easting\",200000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3195");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3196, "epsg", 3196,
		      "LGD2006 / Libya TM zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 11\",GEOGCS[\"LGD2006\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "TOWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99995],PARAMETER[\"false_easting\",200000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3196");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3197, "epsg", 3197,
		      "LGD2006 / Libya TM zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=23 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 12\",GEOGCS[\"LGD2006\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "TOWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",0],PARAMETER[\"central_meridian\",23],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99995],PARAMETER[\"false_easting\",200000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3197");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3198, "epsg", 3198,
		      "LGD2006 / Libya TM zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=25 +k=0.99995 +x_0=200000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-208.406,-109.878,-2.5764,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / Libya TM zone 13\",GEOGCS[\"LGD2006\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "TOWGS84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",0],PARAMETER[\"central_meridian\",25],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99995],PARAMETER[\"false_easting\",200000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3198");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3199, "epsg", 3199,
		      "LGD2006 / UTM zone 32N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +ellps=intl +towgs84=-208.406,-109.87");
    add_proj4text (p, 1, "8,-2.5764,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / UTM zone 32N\",GEOGCS[\"LGD2006\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internation");
    add_srs_wkt (p, 2,
		 "al 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWG");
    add_srs_wkt (p, 3,
		 "S84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",9],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 10,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3199\"],AXI");
    add_srs_wkt (p, 12, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3200, "epsg", 3200, "FD58 / Iraq zone");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.5 +lat_0=32.5 +lon_0=45 +k_0=0.99878");
    add_proj4text (p, 1,
		   "64078000001 +x_0=1500000 +y_0=1166200 +ellps=clrk80 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"FD58 / Iraq zone\",GEOGCS[\"FD58\",DATUM[\"Fina");
    add_srs_wkt (p, 1,
		 "l_Datum_1958\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 2,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6132\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4132\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"L");
    add_srs_wkt (p, 7,
		 "ambert_Conformal_Conic_1SP\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",32.5],PARAMETER[\"central_meridian\",45],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",0.9987864078],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",1500000],PARAMETER[\"false_northing\",1166200],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"3200\"],AXIS[\"Easting\",EAST],AXIS[\"N");
    add_srs_wkt (p, 12, "orthing\",NORTH]]");
    p = add_epsg_def (first, last, 3201, "epsg", 3201,
		      "LGD2006 / UTM zone 33N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +ellps=intl +towgs84=-208.406,-109.87");
    add_proj4text (p, 1, "8,-2.5764,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / UTM zone 33N\",GEOGCS[\"LGD2006\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internation");
    add_srs_wkt (p, 2,
		 "al 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWG");
    add_srs_wkt (p, 3,
		 "S84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",15],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3201\"],AX");
    add_srs_wkt (p, 12, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3202, "epsg", 3202,
		      "LGD2006 / UTM zone 34N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +ellps=intl +towgs84=-208.406,-109.87");
    add_proj4text (p, 1, "8,-2.5764,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / UTM zone 34N\",GEOGCS[\"LGD2006\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internation");
    add_srs_wkt (p, 2,
		 "al 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWG");
    add_srs_wkt (p, 3,
		 "S84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",21],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3202\"],AX");
    add_srs_wkt (p, 12, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3203, "epsg", 3203,
		      "LGD2006 / UTM zone 35N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +ellps=intl +towgs84=-208.406,-109.87");
    add_proj4text (p, 1, "8,-2.5764,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LGD2006 / UTM zone 35N\",GEOGCS[\"LGD2006\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Libyan_Geodetic_Datum_2006\",SPHEROID[\"Internation");
    add_srs_wkt (p, 2,
		 "al 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWG");
    add_srs_wkt (p, 3,
		 "S84[-208.406,-109.878,-2.5764,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6754\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4754\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",27],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3203\"],AX");
    add_srs_wkt (p, 12, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3204, "epsg", 3204,
		      "WGS 84 / SCAR IMW SP19-20");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-60.66666666666666 +lat_2=-63.333333333");
    add_proj4text (p, 1,
		   "33334 +lat_0=-90 +lon_0=-66 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SP19-20\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-60.66666666666666],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "63.33333333333334],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-66],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3204\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3205, "epsg", 3205,
		      "WGS 84 / SCAR IMW SP21-22");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-60.66666666666666 +lat_2=-63.333333333");
    add_proj4text (p, 1,
		   "33334 +lat_0=-90 +lon_0=-54 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SP21-22\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-60.66666666666666],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "63.33333333333334],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-54],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3205\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3206, "epsg", 3206,
		      "WGS 84 / SCAR IMW SP23-24");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-60.66666666666666 +lat_2=-63.333333333");
    add_proj4text (p, 1,
		   "33334 +lat_0=-90 +lon_0=-42 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SP23-24\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-60.66666666666666],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "63.33333333333334],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-42],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3206\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3207, "epsg", 3207,
		      "WGS 84 / SCAR IMW SQ01-02");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-174 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ01-02\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-174],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3207\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3208, "epsg", 3208,
		      "WGS 84 / SCAR IMW SQ19-20");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-66 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ19-20\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-66],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3208\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3209, "epsg", 3209,
		      "WGS 84 / SCAR IMW SQ21-22");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-54 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ21-22\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-54],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3209\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3210, "epsg", 3210,
		      "WGS 84 / SCAR IMW SQ37-38");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=42 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ37-38\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",42],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3210\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3211, "epsg", 3211,
		      "WGS 84 / SCAR IMW SQ39-40");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=54 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ39-40\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",54],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3211\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3212, "epsg", 3212,
		      "WGS 84 / SCAR IMW SQ41-42");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=66 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ41-42\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",66],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3212\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3213, "epsg", 3213,
		      "WGS 84 / SCAR IMW SQ43-44");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=78 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ43-44\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",78],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3213\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3214, "epsg", 3214,
		      "WGS 84 / SCAR IMW SQ45-46");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=90 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ45-46\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",90],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3214\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3215, "epsg", 3215,
		      "WGS 84 / SCAR IMW SQ47-48");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=102 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ47-48\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",102],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3215\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3216, "epsg", 3216,
		      "WGS 84 / SCAR IMW SQ49-50");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=114 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ49-50\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",114],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3216\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3217, "epsg", 3217,
		      "WGS 84 / SCAR IMW SQ51-52");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=126 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ51-52\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",126],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3217\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3218, "epsg", 3218,
		      "WGS 84 / SCAR IMW SQ53-54");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=138 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ53-54\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",138],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3218\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3219, "epsg", 3219,
		      "WGS 84 / SCAR IMW SQ55-56");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=150 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ55-56\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",150],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3219\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3220, "epsg", 3220,
		      "WGS 84 / SCAR IMW SQ57-58");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-64.66666666666667 +lat_2=-67.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=162 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SQ57-58\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-64.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "67.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",162],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3220\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3221, "epsg", 3221,
		      "WGS 84 / SCAR IMW SR13-14");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-102 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR13-14\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-102],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3221\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3222, "epsg", 3222,
		      "WGS 84 / SCAR IMW SR15-16");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-90 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR15-16\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-90],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3222\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3223, "epsg", 3223,
		      "WGS 84 / SCAR IMW SR17-18");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-78 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR17-18\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-78],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3223\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3224, "epsg", 3224,
		      "WGS 84 / SCAR IMW SR19-20");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-66 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR19-20\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-66],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3224\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3225, "epsg", 3225,
		      "WGS 84 / SCAR IMW SR27-28");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-18 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR27-28\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-18],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3225\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3226, "epsg", 3226,
		      "WGS 84 / SCAR IMW SR29-30");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-6 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR29-30\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-6],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3226\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3227, "epsg", 3227,
		      "WGS 84 / SCAR IMW SR31-32");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=6 +x_0=0 +y_0=0 +ellps=WGS84 +da");
    add_proj4text (p, 2, "tum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR31-32\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",6],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 12,
		 "SG\",\"3227\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3228, "epsg", 3228,
		      "WGS 84 / SCAR IMW SR33-34");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=18 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR33-34\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",18],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3228\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3229, "epsg", 3229,
		      "WGS 84 / SCAR IMW SR35-36");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=30 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR35-36\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",30],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3229\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3230, "epsg", 3230,
		      "WGS 84 / SCAR IMW SR37-38");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=42 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR37-38\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",42],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3230\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3231, "epsg", 3231,
		      "WGS 84 / SCAR IMW SR39-40");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=54 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR39-40\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",54],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3231\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3232, "epsg", 3232,
		      "WGS 84 / SCAR IMW SR41-42");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=66 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR41-42\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",66],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3232\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3233, "epsg", 3233,
		      "WGS 84 / SCAR IMW SR43-44");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=78 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR43-44\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",78],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3233\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3234, "epsg", 3234,
		      "WGS 84 / SCAR IMW SR45-46");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=90 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR45-46\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",90],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3234\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3235, "epsg", 3235,
		      "WGS 84 / SCAR IMW SR47-48");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=102 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR47-48\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",102],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3235\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3236, "epsg", 3236,
		      "WGS 84 / SCAR IMW SR49-50");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=114 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR49-50\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",114],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3236\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3237, "epsg", 3237,
		      "WGS 84 / SCAR IMW SR51-52");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=126 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR51-52\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",126],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3237\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3238, "epsg", 3238,
		      "WGS 84 / SCAR IMW SR53-54");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=138 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR53-54\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",138],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3238\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3239, "epsg", 3239,
		      "WGS 84 / SCAR IMW SR55-56");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=150 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR55-56\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",150],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3239\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3240, "epsg", 3240,
		      "WGS 84 / SCAR IMW SR57-58");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=162 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR57-58\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",162],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3240\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3241, "epsg", 3241,
		      "WGS 84 / SCAR IMW SR59-60");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-68.66666666666667 +lat_2=-71.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=174 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SR59-60\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-68.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "71.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",174],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3241\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3242, "epsg", 3242,
		      "WGS 84 / SCAR IMW SS04-06");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-153 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS04-06\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-153],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3242\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3243, "epsg", 3243,
		      "WGS 84 / SCAR IMW SS07-09");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-135 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS07-09\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-135],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3243\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3244, "epsg", 3244,
		      "WGS 84 / SCAR IMW SS10-12");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-117 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS10-12\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-117],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3244\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3245, "epsg", 3245,
		      "WGS 84 / SCAR IMW SS13-15");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-99 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS13-15\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-99],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3245\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3246, "epsg", 3246,
		      "WGS 84 / SCAR IMW SS16-18");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-81 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS16-18\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-81],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3246\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3247, "epsg", 3247,
		      "WGS 84 / SCAR IMW SS19-21");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-63 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS19-21\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-63],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3247\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3248, "epsg", 3248,
		      "WGS 84 / SCAR IMW SS25-27");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-27 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS25-27\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-27],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3248\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3249, "epsg", 3249,
		      "WGS 84 / SCAR IMW SS28-30");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-9 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS28-30\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-9],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3249\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3250, "epsg", 3250,
		      "WGS 84 / SCAR IMW SS31-33");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=9 +x_0=0 +y_0=0 +ellps=WGS84 +da");
    add_proj4text (p, 2, "tum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS31-33\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",9],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 12,
		 "SG\",\"3250\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3251, "epsg", 3251,
		      "WGS 84 / SCAR IMW SS34-36");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=27 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS34-36\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",27],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3251\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3252, "epsg", 3252,
		      "WGS 84 / SCAR IMW SS37-39");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=45 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS37-39\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",45],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3252\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3253, "epsg", 3253,
		      "WGS 84 / SCAR IMW SS40-42");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=63 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS40-42\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",63],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3253\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3254, "epsg", 3254,
		      "WGS 84 / SCAR IMW SS43-45");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=81 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS43-45\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",81],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3254\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3255, "epsg", 3255,
		      "WGS 84 / SCAR IMW SS46-48");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=99 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS46-48\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",99],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3255\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3256, "epsg", 3256,
		      "WGS 84 / SCAR IMW SS49-51");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=117 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS49-51\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",117],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3256\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3257, "epsg", 3257,
		      "WGS 84 / SCAR IMW SS52-54");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=135 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS52-54\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",135],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3257\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3258, "epsg", 3258,
		      "WGS 84 / SCAR IMW SS55-57");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=153 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS55-57\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",153],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3258\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3259, "epsg", 3259,
		      "WGS 84 / SCAR IMW SS58-60");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-72.66666666666667 +lat_2=-75.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=171 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SS58-60\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-72.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "75.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",171],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3259\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3260, "epsg", 3260,
		      "WGS 84 / SCAR IMW ST01-04");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-168 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST01-04\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-168],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3260\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3261, "epsg", 3261,
		      "WGS 84 / SCAR IMW ST05-08");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-144 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST05-08\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-144],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3261\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3262, "epsg", 3262,
		      "WGS 84 / SCAR IMW ST09-12");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-120 +x_0=0 +y_0=0 +ellps=WGS84 ");
    add_proj4text (p, 2, "+datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST09-12\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-120],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3262\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3263, "epsg", 3263,
		      "WGS 84 / SCAR IMW ST13-16");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-96 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST13-16\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-96],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3263\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3264, "epsg", 3264,
		      "WGS 84 / SCAR IMW ST17-20");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-72 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST17-20\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-72],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3264\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3265, "epsg", 3265,
		      "WGS 84 / SCAR IMW ST21-24");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-48 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST21-24\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-48],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3265\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3266, "epsg", 3266,
		      "WGS 84 / SCAR IMW ST25-28");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=-24 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST25-28\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-24],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3266\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3267, "epsg", 3267,
		      "WGS 84 / SCAR IMW ST29-32");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=0 +x_0=0 +y_0=0 +ellps=WGS84 +da");
    add_proj4text (p, 2, "tum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST29-32\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",0],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 12,
		 "SG\",\"3267\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3268, "epsg", 3268,
		      "WGS 84 / SCAR IMW ST33-36");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=24 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST33-36\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",24],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3268\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3269, "epsg", 3269,
		      "WGS 84 / SCAR IMW ST37-40");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=48 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST37-40\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",48],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3269\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3270, "epsg", 3270,
		      "WGS 84 / SCAR IMW ST41-44");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=72 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST41-44\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",72],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3270\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3271, "epsg", 3271,
		      "WGS 84 / SCAR IMW ST45-48");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=96 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 2, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST45-48\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",96],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3271\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3272, "epsg", 3272,
		      "WGS 84 / SCAR IMW ST49-52");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=120 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST49-52\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",120],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3272\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3273, "epsg", 3273,
		      "WGS 84 / SCAR IMW ST53-56");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=144 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST53-56\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",144],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3273\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3274, "epsg", 3274,
		      "WGS 84 / SCAR IMW ST57-60");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=168 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW ST57-60\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert");
    add_srs_wkt (p, 7,
		 "_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 8,
		 ",-76.66666666666667],PARAMETER[\"standard_parallel_2\",-");
    add_srs_wkt (p, 9,
		 "79.33333333333333],PARAMETER[\"latitude_of_origin\",-90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",168],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12,
		 "EPSG\",\"3274\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3275, "epsg", 3275,
		      "WGS 84 / SCAR IMW SU01-05");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-165 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU01-05\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-165],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3275");
    add_srs_wkt (p, 11,
		 "\"],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 3276, "epsg", 3276,
		      "WGS 84 / SCAR IMW SU06-10");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-135 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU06-10\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-135],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3276");
    add_srs_wkt (p, 11,
		 "\"],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 3277, "epsg", 3277,
		      "WGS 84 / SCAR IMW SU11-15");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-105 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU11-15\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-105],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3277");
    add_srs_wkt (p, 11,
		 "\"],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 3278, "epsg", 3278,
		      "WGS 84 / SCAR IMW SU16-20");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-75 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU16-20\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-75],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3278\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3279, "epsg", 3279,
		      "WGS 84 / SCAR IMW SU21-25");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-45 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU21-25\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-45],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3279\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3280, "epsg", 3280,
		      "WGS 84 / SCAR IMW SU26-30");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-15 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU26-30\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-15],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3280\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3281, "epsg", 3281,
		      "WGS 84 / SCAR IMW SU31-35");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=15 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU31-35\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",15],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3281\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3282, "epsg", 3282,
		      "WGS 84 / SCAR IMW SU36-40");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=45 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU36-40\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",45],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3282\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3283, "epsg", 3283,
		      "WGS 84 / SCAR IMW SU41-45");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=75 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU41-45\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",75],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3283\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3284, "epsg", 3284,
		      "WGS 84 / SCAR IMW SU46-50");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=105 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU46-50\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",105],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3284\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3285, "epsg", 3285,
		      "WGS 84 / SCAR IMW SU51-55");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=135 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU51-55\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",135],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3285\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3286, "epsg", 3286,
		      "WGS 84 / SCAR IMW SU56-60");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=165 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SU56-60\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",165],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3286\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3287, "epsg", 3287,
		      "WGS 84 / SCAR IMW SV01-10");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-150 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SV01-10\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-150],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3287");
    add_srs_wkt (p, 11,
		 "\"],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 3288, "epsg", 3288,
		      "WGS 84 / SCAR IMW SV11-20");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-90 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SV11-20\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-90],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3288\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3289, "epsg", 3289,
		      "WGS 84 / SCAR IMW SV21-30");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=-30 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SV21-30\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",-30],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3289\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3290, "epsg", 3290,
		      "WGS 84 / SCAR IMW SV31-40");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=30 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SV31-40\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",30],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3290\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3291, "epsg", 3291,
		      "WGS 84 / SCAR IMW SV41-50");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=90 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SV41-50\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",90],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3291\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3292, "epsg", 3292,
		      "WGS 84 / SCAR IMW SV51-60");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=150 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SV51-60\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",150],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3292\"");
    add_srs_wkt (p, 11,
		 "],AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3293, "epsg", 3293,
		      "WGS 84 / SCAR IMW SW01-60");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-80.23861111111111 +lon_0");
    add_proj4text (p, 1,
		   "=0 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / SCAR IMW SW01-60\",GEOGCS[\"WGS 84\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223");
    add_srs_wkt (p, 2,
		 "563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_S");
    add_srs_wkt (p, 7,
		 "tereographic\"],PARAMETER[\"latitude_of_origin\",-80.238");
    add_srs_wkt (p, 8,
		 "61111111111],PARAMETER[\"central_meridian\",0],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3293\"]");
    add_srs_wkt (p, 11,
		 ",AXIS[\"Easting\",UNKNOWN],AXIS[\"Northing\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3294, "epsg", 3294,
		      "WGS 84 / USGS Transantarctic Mountains");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-78 +lon_0=162 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 2, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / USGS Transantarctic Mountains\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",63781");
    add_srs_wkt (p, 2,
		 "37,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standar");
    add_srs_wkt (p, 8,
		 "d_parallel_1\",-76.66666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",-79.33333333333333],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 10,
		 "_origin\",-78],PARAMETER[\"central_meridian\",162],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",0],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3294\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3296, "epsg", 3296, "RGPF / UTM zone 5S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +south +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGPF / UTM zone 5S\",GEOGCS[\"RGPF\",DATUM[\"Re");
    add_srs_wkt (p, 1,
		 "seau_Geodesique_de_la_Polynesie_Francaise\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6687\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4687\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "153],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",1000000");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3296\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3297, "epsg", 3297, "RGPF / UTM zone 6S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +south +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGPF / UTM zone 6S\",GEOGCS[\"RGPF\",DATUM[\"Re");
    add_srs_wkt (p, 1,
		 "seau_Geodesique_de_la_Polynesie_Francaise\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6687\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4687\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "147],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",1000000");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3297\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3298, "epsg", 3298, "RGPF / UTM zone 7S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +south +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGPF / UTM zone 7S\",GEOGCS[\"RGPF\",DATUM[\"Re");
    add_srs_wkt (p, 1,
		 "seau_Geodesique_de_la_Polynesie_Francaise\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6687\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4687\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",1000000");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3298\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3299, "epsg", 3299, "RGPF / UTM zone 8S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +south +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGPF / UTM zone 8S\",GEOGCS[\"RGPF\",DATUM[\"Re");
    add_srs_wkt (p, 1,
		 "seau_Geodesique_de_la_Polynesie_Francaise\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6687\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4687\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "135],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",1000000");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3299\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3300, "epsg", 3300,
		      "Estonian Coordinate System of 1992");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=59.33333333333334 +lat_2=58 +lat_0=57.5");
    add_proj4text (p, 1,
		   "1755393055556 +lon_0=24 +x_0=500000 +y_0=6375000 +ellps=");
    add_proj4text (p, 2,
		   "GRS80 +towgs84=0.055,-0.541,-0.185,0.0183,-0.0003,-0.007");
    add_proj4text (p, 3, ",-0.014 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Estonian Coordinate System of 1992\",GEOGCS[\"E");
    add_srs_wkt (p, 1,
		 "ST92\",DATUM[\"Estonia_1992\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0.055,-0.541,-0.185,0.0183,-0.0003,-0.007,-0.014],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6133\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4133");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_1\",59.33333333333334],PARAMETER[\"standar");
    add_srs_wkt (p, 10,
		 "d_parallel_2\",58],PARAMETER[\"latitude_of_origin\",57.5");
    add_srs_wkt (p, 11,
		 "1755393055556],PARAMETER[\"central_meridian\",24],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",6375000],AUTHORITY[\"EPSG\",\"3300\"],AXIS[\"X\",NORT");
    add_srs_wkt (p, 14, "H],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3301, "epsg", 3301,
		      "Estonian Coordinate System of 1997");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=59.33333333333334 +lat_2=58 +lat_0=57.5");
    add_proj4text (p, 1,
		   "1755393055556 +lon_0=24 +x_0=500000 +y_0=6375000 +ellps=");
    add_proj4text (p, 2, "GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Estonian Coordinate System of 1997\",GEOGCS[\"E");
    add_srs_wkt (p, 1,
		 "ST97\",DATUM[\"Estonia_1997\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6180\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4180\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",59.33333333333334]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",58],PARAMETER[\"latit");
    add_srs_wkt (p, 10,
		 "ude_of_origin\",57.51755393055556],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",24],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_northing\",6375000],AUTHORITY[\"EPSG\",\"330");
    add_srs_wkt (p, 13, "1\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3302, "epsg", 3302,
		      "IGN63 Hiva Oa / UTM zone 7S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN63 Hiva Oa / UTM zone 7S\",GEOGCS[\"IGN63 Hi");
    add_srs_wkt (p, 1,
		 "va Oa\",DATUM[\"IGN63_Hiva_Oa\",SPHEROID[\"International");
    add_srs_wkt (p, 2,
		 " 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6689\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4689");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",-141],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",10000000],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3302\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 3303, "epsg", 3303,
		      "Fatu Iva 72 / UTM zone 7S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +south +ellps=intl +towgs84=347.103,10");
    add_proj4text (p, 1,
		   "78.12,2623.92,-33.8875,70.6773,-9.3943,186.074 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fatu Iva 72 / UTM zone 7S\",GEOGCS[\"Fatu Iva 7");
    add_srs_wkt (p, 1,
		 "2\",DATUM[\"Fatu_Iva_72\",SPHEROID[\"International 1924\"");
    add_srs_wkt (p, 2,
		 ",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[347.1");
    add_srs_wkt (p, 3,
		 "03,1078.12,2623.92,-33.8875,70.6773,-9.3943,186.074],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6688\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "688\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-141],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"3303\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 3304, "epsg", 3304,
		      "Tahiti 79 / UTM zone 6S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tahiti 79 / UTM zone 6S\",GEOGCS[\"Tahiti 79\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Tahiti_79\",SPHEROID[\"International 1924\",6378");
    add_srs_wkt (p, 2,
		 "388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6690\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4690\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",-147],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 9,
		 "ctor\",0.9996],PARAMETER[\"false_easting\",500000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 11,
		 "304\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3305, "epsg", 3305,
		      "Moorea 87 / UTM zone 6S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +south +ellps=intl +towgs84=215.525,14");
    add_proj4text (p, 1,
		   "9.593,176.229,-3.2624,-1.692,-1.1571,10.4773 +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Moorea 87 / UTM zone 6S\",GEOGCS[\"Moorea 87\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Moorea_87\",SPHEROID[\"International 1924\",6378");
    add_srs_wkt (p, 2,
		 "388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[215.525,14");
    add_srs_wkt (p, 3,
		 "9.593,176.229,-3.2624,-1.692,-1.1571,10.4773],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6691\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4691\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-147],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 12,
		 "SG\",\"3305\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3306, "epsg", 3306,
		      "Maupiti 83 / UTM zone 5S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +south +ellps=intl +towgs84=217.037,86");
    add_proj4text (p, 1, ".959,23.956,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Maupiti 83 / UTM zone 5S\",GEOGCS[\"Maupiti 83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Maupiti_83\",SPHEROID[\"International 1924\",63");
    add_srs_wkt (p, 2,
		 "78388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[217.037,");
    add_srs_wkt (p, 3,
		 "86.959,23.956,0,0,0,0],AUTHORITY[\"EPSG\",\"6692\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4692\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",-153],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 11,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"3306\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3307, "epsg", 3307,
		      "Nakhl-e Ghanem / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=WGS84 +towgs84=0,-0.15,0.68,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nakhl-e Ghanem / UTM zone 39N\",GEOGCS[\"Nakhl-");
    add_srs_wkt (p, 1,
		 "e Ghanem\",DATUM[\"Nakhl_e_Ghanem\",SPHEROID[\"WGS 84\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWG");
    add_srs_wkt (p, 3,
		 "S84[0,-0.15,0.68,0,0,0,0],AUTHORITY[\"EPSG\",\"6693\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4693\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",51],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",0],AUTHORITY[\"EPSG\",\"3307\"],AXIS[\"Eastin");
    add_srs_wkt (p, 12, "g\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3308, "epsg", 3308, "GDA94 / NSW Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-30.75 +lat_2=-35.75 +lat_0=-33.25 +lon");
    add_proj4text (p, 1,
		   "_0=147 +x_0=9300000 +y_0=4500000 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 2, ",0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / NSW Lambert\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",-30.75],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"standard_parallel_2\",-35.75],PARAMETER[\"lati");
    add_srs_wkt (p, 10,
		 "tude_of_origin\",-33.25],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 11,
		 "147],PARAMETER[\"false_easting\",9300000],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_northing\",4500000],AUTHORITY[\"EPSG\",\"3308\"],AXI");
    add_srs_wkt (p, 13, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3309, "epsg", 3309,
		      "NAD27 / California Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=34 +lat_2=40.5 +lat_0=0 +lon_0=-120 +x_");
    add_proj4text (p, 1,
		   "0=0 +y_0=-4000000 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California Albers\",GEOGCS[\"NAD27\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866");
    add_srs_wkt (p, 2,
		 "\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Albers_Conic_Equal_Area\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",34],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 9,
		 "el_2\",40.5],PARAMETER[\"latitude_of_center\",0],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"longitude_of_center\",-120],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",0],PARAMETER[\"false_northing\",-4000000],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"3309\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3310, "epsg", 3310,
		      "NAD83 / California Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=34 +lat_2=40.5 +lat_0=0 +lon_0=-120 +x_");
    add_proj4text (p, 1,
		   "0=0 +y_0=-4000000 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California Albers\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Albers_Conic_Equal_Area\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",34],PARAMETER[\"standard_parallel_2\",40");
    add_srs_wkt (p, 9,
		 ".5],PARAMETER[\"latitude_of_center\",0],PARAMETER[\"long");
    add_srs_wkt (p, 10,
		 "itude_of_center\",-120],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",-4000000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"3310\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3311, "epsg", 3311,
		      "NAD83(HARN) / California Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=34 +lat_2=40.5 +lat_0=0 +lon_0=-120 +x_");
    add_proj4text (p, 1, "0=0 +y_0=-4000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / California Albers\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Albers_Conic_Equal_Area\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",34],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",40.5],PARAMETER[\"latitude_of_center\"");
    add_srs_wkt (p, 10,
		 ",0],PARAMETER[\"longitude_of_center\",-120],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",0],PARAMETER[\"false_northing\",-4000000");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3311\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3312, "epsg", 3312, "CSG67 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=intl +towgs84=-186,230,110,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"CSG67 / UTM zone 21N\",GEOGCS[\"CSG67\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Centre_Spatial_Guyanais_1967\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[-186,230,110,0,0,0,0],AUTHORITY[\"EPSG\",\"6623\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4623\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",-57],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"3312\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3313, "epsg", 3313, "RGFG95 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=GRS80 +towgs84=2,2,-2,0,0,0,0 ");
    add_proj4text (p, 1, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGFG95 / UTM zone 21N\",GEOGCS[\"RGFG95\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Reseau_Geodesique_Francais_Guyane_1995\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],TOWGS84[2,2,-2,0,0,0,0],AUTHORITY[\"EPSG\",\"6624\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4624\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-57],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"3313\"],AXIS[\"Eas");
    add_srs_wkt (p, 12, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3314, "epsg", 3314,
		      "Katanga 1955 / Katanga Lambert (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-6.5 +lat_2=-11.5 +lat_0=0 +lon_0=26 +x");
    add_proj4text (p, 1,
		   "_0=0 +y_0=0 +ellps=clrk66 +towgs84=-103.746,-9.614,-255.");
    add_proj4text (p, 2, "95,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Katanga 1955 / Katanga Lambert (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Katanga 1955\",DATUM[\"Katanga_1955\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7008\"]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4695\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",-6.5],PARAMETER[\"standard_para");
    add_srs_wkt (p, 10,
		 "llel_2\",-11.5],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",26],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13,
		 ",\"3314\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 14, "H]]");
    p = add_epsg_def (first, last, 3315, "epsg", 3315,
		      "Katanga 1955 / Katanga TM (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-9 +lon_0=26 +k=0.9998 +x_0=0 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=clrk66 +towgs84=-103.746,-9.614,-255.95,0,0,0,0 +");
    add_proj4text (p, 2, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Katanga 1955 / Katanga TM (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Katanga 1955\",DATUM[\"Katanga_1955\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7008\"]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "695\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",-9],PARAMETER[\"central_meridian\",26],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9998],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3315\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3316, "epsg", 3316,
		      "Kasai 1953 / Congo TM zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=22 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kasai 1953 / Congo TM zone 22\",GEOGCS[\"Kasai ");
    add_srs_wkt (p, 1,
		 "1953\",DATUM[\"Kasai_1953\",SPHEROID[\"Clarke 1880 (RGS)");
    add_srs_wkt (p, 2,
		 "\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6696\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "696\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_origin\",0],PARAMETER[\"central_meridian\",22],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",10000000],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3316\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 12, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 3317, "epsg", 3317,
		      "Kasai 1953 / Congo TM zone 24");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kasai 1953 / Congo TM zone 24\",GEOGCS[\"Kasai ");
    add_srs_wkt (p, 1,
		 "1953\",DATUM[\"Kasai_1953\",SPHEROID[\"Clarke 1880 (RGS)");
    add_srs_wkt (p, 2,
		 "\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6696\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "696\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_origin\",0],PARAMETER[\"central_meridian\",24],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",10000000],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3317\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 12, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 3318, "epsg", 3318,
		      "IGC 1962 / Congo TM zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 12\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",12],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3318\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3319, "epsg", 3319,
		      "IGC 1962 / Congo TM zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=14 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 14\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",14],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3319\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3320, "epsg", 3320,
		      "IGC 1962 / Congo TM zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 16\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",16],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3320\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3321, "epsg", 3321,
		      "IGC 1962 / Congo TM zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 18\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",18],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3321\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_10 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 3322, "epsg", 3322,
		      "IGC 1962 / Congo TM zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=20 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 20\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",20],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3322\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3323, "epsg", 3323,
		      "IGC 1962 / Congo TM zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=22 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 22\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",22],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3323\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3324, "epsg", 3324,
		      "IGC 1962 / Congo TM zone 24");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 24\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",24],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3324\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3325, "epsg", 3325,
		      "IGC 1962 / Congo TM zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=26 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 26\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",26],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3325\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3326, "epsg", 3326,
		      "IGC 1962 / Congo TM zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 28\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",28],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3326\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3327, "epsg", 3327,
		      "IGC 1962 / Congo TM zone 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGC 1962 / Congo TM zone 30\",GEOGCS[\"IGC 1962");
    add_srs_wkt (p, 1,
		 " 6th Parallel South\",DATUM[\"IGC_1962_Arc_of_the_6th_Pa");
    add_srs_wkt (p, 2,
		 "rallel_South\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6697\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4697\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",30],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3327\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3328, "epsg", 3328,
		      "Pulkovo 1942(58) / GUGiK-80");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=52.16666666666666 +lon_0=19.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999714 +x_0=500000 +y_0=500000 +ellps=krass ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / GUGiK-80\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Oblique_Stereographic\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",52.16666666666666],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",19.16666666666667],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".999714],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",500000],AUTHORITY[\"EPSG\",\"3328\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3329, "epsg", 3329,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 5");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",15],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",5500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3329\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3330, "epsg", 3330,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 6");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",18],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",6500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3330\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3331, "epsg", 3331,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=7500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 7");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",21],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",7500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3331\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3332, "epsg", 3332,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 8");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",24],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",8500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3332\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3333, "epsg", 3333,
		      "Pulkovo 1942(58) / Gauss-Kruger zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=3500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Gauss-Kruger zone 3\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",15],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",3500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3333\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3334, "epsg", 3334,
		      "Pulkovo 1942(58) / Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Gauss-Kruger zone 4\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",21],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",4500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3334\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3335, "epsg", 3335,
		      "Pulkovo 1942(58) / Gauss-Kruger zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Gauss-Kruger zone 5\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",27],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",5500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3335\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3336, "epsg", 3336,
		      "IGN 1962 Kerguelen / UTM zone 42S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +south +ellps=intl +towgs84=145,-187,");
    add_proj4text (p, 1, "103,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN 1962 Kerguelen / UTM zone 42S\",GEOGCS[\"IG");
    add_srs_wkt (p, 1,
		 "N 1962 Kerguelen\",DATUM[\"IGN_1962_Kerguelen\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7022\"]],TOWGS84[145,-187,103,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6698\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4698\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",69],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"333");
    add_srs_wkt (p, 12,
		 "6\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3337, "epsg", 3337,
		      "Le Pouce 1934 / Mauritius Grid");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-20.19506944444445 +lat_0=-20.195069444");
    add_proj4text (p, 1,
		   "44445 +lon_0=57.52182777777778 +k_0=1 +x_0=1000000 +y_0=");
    add_proj4text (p, 2,
		   "1000000 +ellps=clrk80 +towgs84=-770.1,158.4,-498.2,0,0,0");
    add_proj4text (p, 3, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Le Pouce 1934 / Mauritius Grid\",GEOGCS[\"Le Po");
    add_srs_wkt (p, 1,
		 "uce 1934\",DATUM[\"Le_Pouce_1934\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 2,
		 "0 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-770.1,158.4,-498.2,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6699\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4699\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"L");
    add_srs_wkt (p, 8,
		 "ambert_Conformal_Conic_1SP\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",-20.19506944444445],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",57.52182777777778],PARAMETER[\"scale_factor\",1],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",1000000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",1000000],AUTHORITY[\"EPSG\",\"3337\"],AXIS[\"Easting");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3338, "epsg", 3338, "NAD83 / Alaska Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=55 +lat_2=65 +lat_0=50 +lon_0=-154 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska Albers\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Albers_Conic_Equal_Area\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 8,
		 "rallel_1\",55],PARAMETER[\"standard_parallel_2\",65],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"latitude_of_center\",50],PARAMETER[\"longitude_");
    add_srs_wkt (p, 10,
		 "of_center\",-154],PARAMETER[\"false_easting\",0],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3338\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3339, "epsg", 3339,
		      "IGCB 1955 / Congo TM zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=clrk80 +towgs84=-79.9,-158,-168.9,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGCB 1955 / Congo TM zone 12\",GEOGCS[\"IGCB 19");
    add_srs_wkt (p, 1,
		 "55\",DATUM[\"Institut_Geographique_du_Congo_Belge_1955\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7012\"]],TOWGS84[-79.9,-158,-168.9,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6701\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4701\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",12");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"3339\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3340, "epsg", 3340,
		      "IGCB 1955 / Congo TM zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=14 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=clrk80 +towgs84=-79.9,-158,-168.9,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGCB 1955 / Congo TM zone 14\",GEOGCS[\"IGCB 19");
    add_srs_wkt (p, 1,
		 "55\",DATUM[\"Institut_Geographique_du_Congo_Belge_1955\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7012\"]],TOWGS84[-79.9,-158,-168.9,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6701\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4701\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",14");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"3340\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3341, "epsg", 3341,
		      "IGCB 1955 / Congo TM zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=clrk80 +towgs84=-79.9,-158,-168.9,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGCB 1955 / Congo TM zone 16\",GEOGCS[\"IGCB 19");
    add_srs_wkt (p, 1,
		 "55\",DATUM[\"Institut_Geographique_du_Congo_Belge_1955\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7012\"]],TOWGS84[-79.9,-158,-168.9,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6701\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4701\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",16");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"3341\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3342, "epsg", 3342,
		      "IGCB 1955 / UTM zone 33S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +south +ellps=clrk80 +towgs84=-79.9,-");
    add_proj4text (p, 1, "158,-168.9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGCB 1955 / UTM zone 33S\",GEOGCS[\"IGCB 1955\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Institut_Geographique_du_Congo_Belge_1955\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7012\"]],TOWGS84[-79.9,-158,-168.9,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6701\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4701\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",15],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",10000000],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"3342\"],AXIS[\"Easting\",EAST],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH]]");
    p = add_epsg_def (first, last, 3343, "epsg", 3343,
		      "Mauritania 1999 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mauritania 1999 / UTM zone 28N\",GEOGCS[\"Mauri");
    add_srs_wkt (p, 1,
		 "tania 1999\",DATUM[\"Mauritania_1999\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6702\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4702\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-15],PARAMETER[\"scale_factor\",0.9996]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3343\"],AXIS[\"Easting");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3344, "epsg", 3344,
		      "Mauritania 1999 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mauritania 1999 / UTM zone 29N\",GEOGCS[\"Mauri");
    add_srs_wkt (p, 1,
		 "tania 1999\",DATUM[\"Mauritania_1999\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6702\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4702\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-9],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 11,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"3344\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3345, "epsg", 3345,
		      "Mauritania 1999 / UTM zone 30N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mauritania 1999 / UTM zone 30N\",GEOGCS[\"Mauri");
    add_srs_wkt (p, 1,
		 "tania 1999\",DATUM[\"Mauritania_1999\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6702\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4702\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",-3],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 11,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"3345\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3346, "epsg", 3346, "LKS94 / Lithuania TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9998 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"LKS94 / Lithuania TM\",GEOGCS[\"LKS94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Lithuania_1994_ETRS89\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 2,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6126\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4669\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",24],PARAMETER[\"scale_factor\",0.9998],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"3346\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 12, "ST]]");
    p = add_epsg_def (first, last, 3347, "epsg", 3347,
		      "NAD83 / Statistics Canada Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=77 +lat_0=63.390675 +lon_0=-9");
    add_proj4text (p, 1,
		   "1.86666666666666 +x_0=6200000 +y_0=3000000 +ellps=GRS80 ");
    add_proj4text (p, 2, "+datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Statistics Canada Lambert\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"standard_parallel_1\",49],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",77],PARAMETER[\"latitude_of_origin\",63.390675]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"central_meridian\",-91.86666666666666],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",6200000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",3000000],AUTHORITY[\"EPSG\",\"3347\"],AXIS[\"Easti");
    add_srs_wkt (p, 13, "ng\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3348, "epsg", 3348,
		      "NAD83(CSRS) / Statistics Canada Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=77 +lat_0=63.390675 +lon_0=-9");
    add_proj4text (p, 1,
		   "1.86666666666666 +x_0=6200000 +y_0=3000000 +ellps=GRS80 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Statistics Canada Lambert\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Confo");
    add_srs_wkt (p, 8,
		 "rmal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",49],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"standard_parallel_2\",77],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",63.390675],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-91.86666666666666],PARAMETER[\"false_easting\",6200000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",3000000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13,
		 "\",\"3348\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 3349, "epsg", 3349,
		      "WGS 84 / PDC Mercator (deprecated)");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=-150 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +");
    add_proj4text (p, 1, "datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / PDC Mercator (deprecated)\",GEOGCS[\"W");
    add_srs_wkt (p, 1,
		 "GS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,2");
    add_srs_wkt (p, 2,
		 "98.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Mercator_1SP\"],PARAMETER[\"central_meridian\",-150],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 9,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 10,
		 "3349\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 11, "");
    p = add_epsg_def (first, last, 3350, "epsg", 3350,
		      "Pulkovo 1942 / CS63 zone C0");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1 +lon_0=21.95 +k=1 +x_0=250000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone C0\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.1]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",21.95],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",1],PARAMETER[\"false_easting\",250000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3350\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3351, "epsg", 3351,
		      "Pulkovo 1942 / CS63 zone C1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1 +lon_0=24.95 +k=1 +x_0=1250000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone C1\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.1]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",24.95],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",1],PARAMETER[\"false_easting\",1250000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3351\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3352, "epsg", 3352,
		      "Pulkovo 1942 / CS63 zone C2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0.1 +lon_0=27.95 +k=1 +x_0=2250000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / CS63 zone C2\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassowsky 1940");
    add_srs_wkt (p, 2,
		 "\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[2");
    add_srs_wkt (p, 3,
		 "3.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0.1]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",27.95],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",1],PARAMETER[\"false_easting\",2250000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3352\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3353, "epsg", 3353,
		      "Mhast (onshore) / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mhast (onshore) / UTM zone 32S\",GEOGCS[\"Mhast");
    add_srs_wkt (p, 1,
		 " (onshore)\",DATUM[\"Mhast_onshore\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6704\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4704\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",9],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",10000000],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3353\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 12, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 3354, "epsg", 3354,
		      "Mhast (offshore) / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mhast (offshore) / UTM zone 32S\",GEOGCS[\"Mhas");
    add_srs_wkt (p, 1,
		 "t (offshore)\",DATUM[\"Mhast_offshore\",SPHEROID[\"Inter");
    add_srs_wkt (p, 2,
		 "national 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6705\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4705\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",9],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",10000000],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"3354\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 3355, "epsg", 3355,
		      "Egypt Gulf of Suez S-650 TL / Red Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=31 +k=1 +x_0=615000 +y_0=81");
    add_proj4text (p, 1,
		   "0000 +ellps=helmert +towgs84=-146.21,112.63,4.05,0,0,0,0");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Egypt Gulf of Suez S-650 TL / Red Belt\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Egypt Gulf of Suez S-650 TL\",DATUM[\"Egypt_Gulf_of_S");
    add_srs_wkt (p, 2,
		 "uez_S_650_TL\",SPHEROID[\"Helmert 1906\",6378200,298.3,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7020\"]],TOWGS84[-146.21,112.63,4.05");
    add_srs_wkt (p, 4,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6706\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4706\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"latitude_of_origin\",30],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",31],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",615000],PARAMETER[\"false_northing\",810000],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"3355\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3356, "epsg", 3356,
		      "Grand Cayman 1959 / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=clrk66 +towgs84=67.8,106.1,138");
    add_proj4text (p, 1, ".8,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Grand Cayman 1959 / UTM zone 17N\",GEOGCS[\"Gra");
    add_srs_wkt (p, 1,
		 "nd Cayman 1959\",DATUM[\"Grand_Cayman_1959\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7008\"]],TOWGS84[67.8,106.1,138.8,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6723\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4723");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",0],PARAMETER[\"central_meridian\",-81],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"3356\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 3357, "epsg", 3357,
		      "Little Cayman 1961 / UTM zone 17N");
    add_proj4text (p, 0, "+proj=utm +zone=17 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Little Cayman 1961 / UTM zone 17N\",GEOGCS[\"Li");
    add_srs_wkt (p, 1,
		 "ttle Cayman 1961\",DATUM[\"Little_Cayman_1961\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6726\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4726\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-81],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3357\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3358, "epsg", 3358,
		      "NAD83(HARN) / North Carolina");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=33.75 +lon_0=-79 +x_0=609601.22 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / North Carolina\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",36.1666666666666");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"standard_parallel_2\",34.33333333333334],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",33.75],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-79],PARAMETER[\"false_easting\",609601.2");
    add_srs_wkt (p, 12,
		 "2],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3358\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3359, "epsg", 3359,
		      "NAD83(HARN) / North Carolina (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=33.75 +lon_0=-79 +x_0=609601.2192024385 +y_0=");
    add_proj4text (p, 2, "0 +ellps=GRS80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / North Carolina (ftUS) (deprecated");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_R");
    add_srs_wkt (p, 2,
		 "egional_Network\",SPHEROID[\"GRS 1980\",6378137,298.2572");
    add_srs_wkt (p, 3,
		 "22101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6152\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foo");
    add_srs_wkt (p, 7,
		 "t\",0.3048,AUTHORITY[\"EPSG\",\"9002\"]],PROJECTION[\"La");
    add_srs_wkt (p, 8,
		 "mbert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 9,
		 "el_1\",36.16666666666666],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "2\",34.33333333333334],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 11,
		 "33.75],PARAMETER[\"central_meridian\",-79],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",2000004.000008],PARAMETER[\"false_northin");
    add_srs_wkt (p, 13,
		 "g\",0],AUTHORITY[\"EPSG\",\"3359\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3360, "epsg", 3360,
		      "NAD83(HARN) / South Carolina");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.83333333333334 +lat_2=32.5 +lat_0=31");
    add_proj4text (p, 1,
		   ".83333333333333 +lon_0=-81 +x_0=609600 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / South Carolina\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",34.8333333333333");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",32.5],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",31.83333333333333],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-81],PARAMETER[\"false_easting\",609600],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3360");
    add_srs_wkt (p, 13, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3361, "epsg", 3361,
		      "NAD83(HARN) / South Carolina (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.83333333333334 +lat_2=32.5 +lat_0=31");
    add_proj4text (p, 1,
		   ".83333333333333 +lon_0=-81 +x_0=609600 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / South Carolina (ft)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"foot\",0.3048,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9002\"]],PROJECTION[\"Lambert_Conforma");
    add_srs_wkt (p, 8,
		 "l_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",34.8333");
    add_srs_wkt (p, 9,
		 "3333333334],PARAMETER[\"standard_parallel_2\",32.5],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",31.83333333333333],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-81],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"3361\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3362, "epsg", 3362,
		      "NAD83(HARN) / Pennsylvania North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.95 +lat_2=40.88333333333333 +lat_0=4");
    add_proj4text (p, 1,
		   "0.16666666666666 +lon_0=-77.75 +x_0=600000 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Pennsylvania North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",41.95],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_2\",40.88333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",40.16666666666666],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-77.75],PARAMETER[\"false_easting\",60000");
    add_srs_wkt (p, 12,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3362\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3363, "epsg", 3363,
		      "NAD83(HARN) / Pennsylvania North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.95 +lat_2=40.88333333333333 +lat_0=4");
    add_proj4text (p, 1,
		   "0.16666666666666 +lon_0=-77.75 +x_0=600000 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Pennsylvania North (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_N");
    add_srs_wkt (p, 2,
		 "etwork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey f");
    add_srs_wkt (p, 7,
		 "oot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",41.95],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 10,
		 "l_2\",40.88333333333333],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",40.16666666666666],PARAMETER[\"central_meridian\",-77.7");
    add_srs_wkt (p, 12,
		 "5],PARAMETER[\"false_easting\",1968500],PARAMETER[\"fals");
    add_srs_wkt (p, 13,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3363\"],AXIS[\"X\",");
    add_srs_wkt (p, 14, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3364, "epsg", 3364,
		      "NAD83(HARN) / Pennsylvania South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.96666666666667 +lat_2=39.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=39.33333333333334 +lon_0=-77.75 +x_0=600000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Pennsylvania South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",40.9666666666666");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"standard_parallel_2\",39.93333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",39.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-77.75],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",600000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"3364\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3365, "epsg", 3365,
		      "NAD83(HARN) / Pennsylvania South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.96666666666667 +lat_2=39.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=39.33333333333334 +lon_0=-77.75 +x_0=600000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Pennsylvania South (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_N");
    add_srs_wkt (p, 2,
		 "etwork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey f");
    add_srs_wkt (p, 7,
		 "oot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",40.96666666666667],PARAMETER[\"stan");
    add_srs_wkt (p, 10,
		 "dard_parallel_2\",39.93333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 11,
		 "e_of_origin\",39.33333333333334],PARAMETER[\"central_mer");
    add_srs_wkt (p, 12,
		 "idian\",-77.75],PARAMETER[\"false_easting\",1968500],PAR");
    add_srs_wkt (p, 13,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3365\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3366, "epsg", 3366,
		      "Hong Kong 1963 Grid System (deprecated)");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=22.31213333333334 +lon_0=114.178555555");
    add_proj4text (p, 1,
		   "5556 +x_0=40243.57775604237 +y_0=19069.93351512578 +a=63");
    add_proj4text (p, 2,
		   "78293.645208759 +b=6356617.987679838 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hong Kong 1963 Grid System (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Hong Kong 1963\",DATUM[\"Hong_Kong_1963\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1858\",6378293.645208759,294.2606763692569,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7007\"]],AUTHORITY[\"EPSG\",\"6738\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4738\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Cassini_Soldner\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",22.31213333333334],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"central_meridian\",114.1785555555556],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",40243.57775604237],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",19069.93351512578],AUTHORITY[\"EPSG\",\"3366");
    add_srs_wkt (p, 12, "\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3367, "epsg", 3367,
		      "IGN Astro 1960 / UTM zone 28N");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN Astro 1960 / UTM zone 28N\",GEOGCS[\"IGN As");
    add_srs_wkt (p, 1,
		 "tro 1960\",DATUM[\"IGN_Astro_1960\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "80 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6700\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4700\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-1");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3367\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 12, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 3368, "epsg", 3368,
		      "IGN Astro 1960 / UTM zone 29N");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN Astro 1960 / UTM zone 29N\",GEOGCS[\"IGN As");
    add_srs_wkt (p, 1,
		 "tro 1960\",DATUM[\"IGN_Astro_1960\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "80 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6700\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4700\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-9");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"3368\"],AXIS[\"Easting\",EAST],AXIS[\"Nort");
    add_srs_wkt (p, 12, "hing\",NORTH]]");
    p = add_epsg_def (first, last, 3369, "epsg", 3369,
		      "IGN Astro 1960 / UTM zone 30N");
    add_proj4text (p, 0, "+proj=utm +zone=30 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGN Astro 1960 / UTM zone 30N\",GEOGCS[\"IGN As");
    add_srs_wkt (p, 1,
		 "tro 1960\",DATUM[\"IGN_Astro_1960\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "80 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6700\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4700\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-3");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"3369\"],AXIS[\"Easting\",EAST],AXIS[\"Nort");
    add_srs_wkt (p, 12, "hing\",NORTH]]");
    p = add_epsg_def (first, last, 3370, "epsg", 3370, "NAD27 / UTM zone 59N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 59N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",171],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"3370\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 3371, "epsg", 3371, "NAD27 / UTM zone 60N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 60N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",177],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"3371\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 3372, "epsg", 3372, "NAD83 / UTM zone 59N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 59N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",171],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "3372\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3373, "epsg", 3373, "NAD83 / UTM zone 60N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 60N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",177],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "3373\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3374, "epsg", 3374, "FD54 / UTM zone 29N");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"FD54 / UTM zone 29N\",GEOGCS[\"FD54\",DATUM[\"F");
    add_srs_wkt (p, 1,
		 "aroe_Datum_1954\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 2,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6741\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4741\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",-9],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 9,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3374\"],AXIS");
    add_srs_wkt (p, 11, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3375, "epsg", 3375,
		      "GDM2000 / Peninsula RSO");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=102.25 +alpha=323.02579646666");
    add_proj4text (p, 1,
		   "66 +k=0.99984 +x_0=804671 +y_0=0 +ellps=GRS80 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Peninsula RSO\",GEOGCS[\"GDM2000\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_center\",4],PARAMETER[\"longitude_of_center\"");
    add_srs_wkt (p, 9,
		 ",102.25],PARAMETER[\"azimuth\",323.0257964666666],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"rectified_grid_angle\",323.1301023611111],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"scale_factor\",0.99984],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "804671],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13,
		 "\",\"3375\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 3376, "epsg", 3376,
		      "GDM2000 / East Malaysia BRSO");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=115 +alpha=53.31580995 +k=0.9");
    add_proj4text (p, 1, "9984 +x_0=0 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / East Malaysia BRSO\",GEOGCS[\"GDM2000");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_center\",4],PARAMETER[\"longitude_of_cen");
    add_srs_wkt (p, 9,
		 "ter\",115],PARAMETER[\"azimuth\",53.31580995],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"rectified_grid_angle\",53.13010236111111],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "scale_factor\",0.99984],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3376");
    add_srs_wkt (p, 13, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3377, "epsg", 3377, "GDM2000 / Johor Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=2.121679744444445 +lon_0=103.427936236");
    add_proj4text (p, 1,
		   "1111 +x_0=-14810.562 +y_0=8758.32 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Johor Grid\",GEOGCS[\"GDM2000\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",2.121679744444445],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",103.4279362361111],PARAMETER[\"false_easting\",-14810");
    add_srs_wkt (p, 10,
		 ".562],PARAMETER[\"false_northing\",8758.32],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"3377\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 3378, "epsg", 3378,
		      "GDM2000 / Sembilan and Melaka Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=2.682347636111111 +lon_0=101.974905041");
    add_proj4text (p, 1,
		   "6667 +x_0=3673.785 +y_0=-4240.573 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Sembilan and Melaka Grid\",GEOGCS[\"G");
    add_srs_wkt (p, 1,
		 "DM2000\",DATUM[\"Geodetic_Datum_of_Malaysia_2000\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Cassini_Soldner\"],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"latitude_of_origin\",2.682347636111111],PARAMETER[\"ce");
    add_srs_wkt (p, 9,
		 "ntral_meridian\",101.9749050416667],PARAMETER[\"false_ea");
    add_srs_wkt (p, 10,
		 "sting\",3673.785],PARAMETER[\"false_northing\",-4240.573");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"3378\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3379, "epsg", 3379, "GDM2000 / PahangGrid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=3.769388088888889 +lon_0=102.368298983");
    add_proj4text (p, 1,
		   "3333 +x_0=-7368.228 +y_0=6485.858 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / PahangGrid\",GEOGCS[\"GDM2000\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",3.769388088888889],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",102.3682989833333],PARAMETER[\"false_easting\",-7368.");
    add_srs_wkt (p, 10,
		 "228],PARAMETER[\"false_northing\",6485.858],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"3379\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 3380, "epsg", 3380,
		      "GDM2000 / Selangor Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=3.68464905 +lon_0=101.3891079138889 +x");
    add_proj4text (p, 1,
		   "_0=-34836.161 +y_0=56464.049 +ellps=GRS80 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Selangor Grid\",GEOGCS[\"GDM2000\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_origin\",3.68464905],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "01.3891079138889],PARAMETER[\"false_easting\",-34836.161");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_northing\",56464.049],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"3380\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 3381, "epsg", 3381,
		      "GDM2000 / Terengganu Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=4.9762852 +lon_0=103.070275625 +x_0=19");
    add_proj4text (p, 1,
		   "594.245 +y_0=3371.895 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Terengganu Grid\",GEOGCS[\"GDM2000\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitud");
    add_srs_wkt (p, 8,
		 "e_of_origin\",4.9762852],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "103.070275625],PARAMETER[\"false_easting\",19594.245],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",3371.895],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"3381\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 3382, "epsg", 3382, "GDM2000 / Pinang Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=5.421517541666667 +lon_0=100.344376963");
    add_proj4text (p, 1,
		   "8889 +x_0=-23.414 +y_0=62.283 +ellps=GRS80 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Pinang Grid\",GEOGCS[\"GDM2000\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS 198");
    add_srs_wkt (p, 2,
		 "0\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 8,
		 "f_origin\",5.421517541666667],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",100.3443769638889],PARAMETER[\"false_easting\",-23.");
    add_srs_wkt (p, 10,
		 "414],PARAMETER[\"false_northing\",62.283],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"3382\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 3383, "epsg", 3383,
		      "GDM2000 / Kedah and Perlis Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=5.964672713888889 +lon_0=100.636371111");
    add_proj4text (p, 1, "1111 +x_0=0 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Kedah and Perlis Grid\",GEOGCS[\"GDM2");
    add_srs_wkt (p, 1,
		 "000\",DATUM[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",5.964672713888889],PARAMETER[\"centr");
    add_srs_wkt (p, 9,
		 "al_meridian\",100.6363711111111],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11,
		 "\",\"3383\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NO");
    add_srs_wkt (p, 12, "RTH]]");
    p = add_epsg_def (first, last, 3384, "epsg", 3384, "GDM2000 / Perak Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=4.859063022222222 +lon_0=100.815410586");
    add_proj4text (p, 1,
		   "1111 +x_0=-1.769 +y_0=133454.779 +ellps=GRS80 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Perak Grid\",GEOGCS[\"GDM2000\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",4.859063022222222],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",100.8154105861111],PARAMETER[\"false_easting\",-1.769");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_northing\",133454.779],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"3384\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 3385, "epsg", 3385,
		      "GDM2000 / Kelantan Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=5.972543658333334 +lon_0=102.295241669");
    add_proj4text (p, 1,
		   "4444 +x_0=13227.851 +y_0=8739.894 +ellps=GRS80 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDM2000 / Kelantan Grid\",GEOGCS[\"GDM2000\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Geodetic_Datum_of_Malaysia_2000\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6742\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4742\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Cassini_Soldner\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_origin\",5.972543658333334],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",102.2952416694444],PARAMETER[\"false_easting\",132");
    add_srs_wkt (p, 10,
		 "27.851],PARAMETER[\"false_northing\",8739.894],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3385\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 3386, "epsg", 3386, "KKJ / Finland zone 0");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KKJ / Finland zone 0\",GEOGCS[\"KKJ\",DATUM[\"K");
    add_srs_wkt (p, 1,
		 "artastokoordinaattijarjestelma_1966\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6123\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4123\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",18],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"3386\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3387, "epsg", 3387, "KKJ / Finland zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KKJ / Finland zone 5\",GEOGCS[\"KKJ\",DATUM[\"K");
    add_srs_wkt (p, 1,
		 "artastokoordinaattijarjestelma_1966\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6123\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4123\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",0],PARAMETER[\"central_meridian\",33],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"3387\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3388, "epsg", 3388,
		      "Pulkovo 1942 / Caspian Sea Mercator");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=51 +k=1 +x_0=0 +y_0=0 +ellps=krass +un");
    add_proj4text (p, 1, "its=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Caspian Sea Mercator\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Mercator_1SP\"],PARAMETER[\"central_meridian\",51],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3388\"],AXIS[\"none\",NORTH],AXIS[\"none\",EAST]]");
    p = add_epsg_def (first, last, 3389, "epsg", 3389,
		      "Pulkovo 1942 / 3-degree Gauss-Kruger zone 60");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=180 +k=1 +x_0=60500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / 3-degree Gauss-Kruger zone 60\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",180],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",60");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"3389\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3390, "epsg", 3390,
		      "Pulkovo 1995 / 3-degree Gauss-Kruger zone 60");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=180 +k=1 +x_0=60500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / 3-degree Gauss-Kruger zone 60\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",180],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",60");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"3390\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3391, "epsg", 3391,
		      "Karbala 1979 / UTM zone 37N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +ellps=clrk80 +towgs84=84.1,-320.1,21");
    add_proj4text (p, 1, "8.7,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Karbala 1979 / UTM zone 37N\",GEOGCS[\"Karbala ");
    add_srs_wkt (p, 1,
		 "1979\",DATUM[\"Karbala_1979\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[84.1,-320.1,218.7,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "743\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4743\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 8,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",39],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3391\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3392, "epsg", 3392,
		      "Karbala 1979 / UTM zone 38N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +ellps=clrk80 +towgs84=84.1,-320.1,21");
    add_proj4text (p, 1, "8.7,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Karbala 1979 / UTM zone 38N\",GEOGCS[\"Karbala ");
    add_srs_wkt (p, 1,
		 "1979\",DATUM[\"Karbala_1979\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[84.1,-320.1,218.7,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "743\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4743\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 8,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",45],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3392\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3393, "epsg", 3393,
		      "Karbala 1979 / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=clrk80 +towgs84=84.1,-320.1,21");
    add_proj4text (p, 1, "8.7,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Karbala 1979 / UTM zone 39N\",GEOGCS[\"Karbala ");
    add_srs_wkt (p, 1,
		 "1979\",DATUM[\"Karbala_1979\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[84.1,-320.1,218.7,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "743\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4743\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 8,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",51],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3393\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3394, "epsg", 3394,
		      "Nahrwan 1934 / Iraq zone");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.5 +lat_0=32.5 +lon_0=45 +k_0=0.99878");
    add_proj4text (p, 1,
		   "64078000001 +x_0=1500000 +y_0=1166200 +ellps=clrk80 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nahrwan 1934 / Iraq zone\",GEOGCS[\"Nahrwan 193");
    add_srs_wkt (p, 1,
		 "4\",DATUM[\"Nahrwan_1934\",SPHEROID[\"Clarke 1880 (RGS)\"");
    add_srs_wkt (p, 2,
		 ",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6744\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"474");
    add_srs_wkt (p, 6,
		 "4\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",32.5],PARAMETER[\"central_meridian\",4");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9987864078],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",1500000],PARAMETER[\"false_northing\",11");
    add_srs_wkt (p, 11,
		 "66200],AUTHORITY[\"EPSG\",\"3394\"],AXIS[\"Easting\",EAS");
    add_srs_wkt (p, 12, "T],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3395, "epsg", 3395,
		      "WGS 84 / World Mercator");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=0 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +dat");
    add_proj4text (p, 1, "um=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / World Mercator\",GEOGCS[\"WGS 84\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.25722356");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"632");
    add_srs_wkt (p, 3,
		 "6\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercator_1");
    add_srs_wkt (p, 7,
		 "SP\"],PARAMETER[\"central_meridian\",0],PARAMETER[\"scal");
    add_srs_wkt (p, 8,
		 "e_factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3395\"],AXIS[\"");
    add_srs_wkt (p, 10, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3396, "epsg", 3396,
		      "PD/83 / 3-degree Gauss-Kruger zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=3500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PD/83 / 3-degree Gauss-Kruger zone 3\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "PD/83\",DATUM[\"Potsdam_Datum_83\",SPHEROID[\"Bessel 184");
    add_srs_wkt (p, 2,
		 "1\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6746\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4746\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",9],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",3");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"3396\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3397, "epsg", 3397,
		      "PD/83 / 3-degree Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PD/83 / 3-degree Gauss-Kruger zone 4\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "PD/83\",DATUM[\"Potsdam_Datum_83\",SPHEROID[\"Bessel 184");
    add_srs_wkt (p, 2,
		 "1\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6746\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4746\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",12],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "4500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11, "G\",\"3397\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3398, "epsg", 3398,
		      "RD/83 / 3-degree Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RD/83 / 3-degree Gauss-Kruger zone 4\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "RD/83\",DATUM[\"Rauenberg_Datum_83\",SPHEROID[\"Bessel 1");
    add_srs_wkt (p, 2,
		 "841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6745\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4745\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",12],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",4500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"3398\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3399, "epsg", 3399,
		      "RD/83 / 3-degree Gauss-Kruger zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RD/83 / 3-degree Gauss-Kruger zone 5\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "RD/83\",DATUM[\"Rauenberg_Datum_83\",SPHEROID[\"Bessel 1");
    add_srs_wkt (p, 2,
		 "841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6745\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4745\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",15],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",5500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"3399\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3400, "epsg", 3400,
		      "NAD83 / Alberta 10-TM (Forest)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-115 +k=0.9992 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alberta 10-TM (Forest)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-115],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",0.9992],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3400\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 3401, "epsg", 3401,
		      "NAD83 / Alberta 10-TM (Resource)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-115 +k=0.9992 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alberta 10-TM (Resource)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",-115");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9992],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"3401\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 3402, "epsg", 3402,
		      "NAD83(CSRS) / Alberta 10-TM (Forest)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-115 +k=0.9992 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Alberta 10-TM (Forest)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-115],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "92],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3402\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3403, "epsg", 3403,
		      "NAD83(CSRS) / Alberta 10-TM (Resource)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-115 +k=0.9992 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Alberta 10-TM (Resource)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-115],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9992],PARAMETER[\"false_easting\",0],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3403\"],AXIS[\"Easting");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3404, "epsg", 3404,
		      "NAD83(HARN) / North Carolina (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=33.75 +lon_0=-79 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2, "0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / North Carolina (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",36.16666666666666],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",34.33333333333334],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",33.75],PARAMETER[\"central_meridian\",-79],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_easting\",2000000],PARAMETER[\"false_north");
    add_srs_wkt (p, 13,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3404\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 14, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3405, "epsg", 3405,
		      "VN-2000 / UTM zone 48N");
    add_proj4text (p, 0, "+proj=utm +zone=48 +ellps=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"VN-2000 / UTM zone 48N\",GEOGCS[\"VN-2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Vietnam_2000\",SPHEROID[\"WGS 84\",6378137,298.2572");
    add_srs_wkt (p, 2,
		 "23563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6756\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4756\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",105],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3405\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3406, "epsg", 3406,
		      "VN-2000 / UTM zone 49N");
    add_proj4text (p, 0, "+proj=utm +zone=49 +ellps=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"VN-2000 / UTM zone 49N\",GEOGCS[\"VN-2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Vietnam_2000\",SPHEROID[\"WGS 84\",6378137,298.2572");
    add_srs_wkt (p, 2,
		 "23563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6756\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4756\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",111],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3406\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3407, "epsg", 3407,
		      "Hong Kong 1963 Grid System");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=22.31213333333334 +lon_0=114.178555555");
    add_proj4text (p, 1,
		   "5556 +x_0=40243.57775604237 +y_0=19069.93351512578 +a=63");
    add_proj4text (p, 2,
		   "78293.645208759 +b=6356617.987679838 +to_meter=0.3047972");
    add_proj4text (p, 3, "654 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hong Kong 1963 Grid System\",GEOGCS[\"Hong Kong");
    add_srs_wkt (p, 1,
		 " 1963\",DATUM[\"Hong_Kong_1963\",SPHEROID[\"Clarke 1858\"");
    add_srs_wkt (p, 2,
		 ",6378293.645208759,294.2606763692569,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7007\"]],AUTHORITY[\"EPSG\",\"6738\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4738\"]],UNIT[\"Clarke's foot\",0.304797265");
    add_srs_wkt (p, 7,
		 "4,AUTHORITY[\"EPSG\",\"9005\"]],PROJECTION[\"Cassini_Sol");
    add_srs_wkt (p, 8,
		 "dner\"],PARAMETER[\"latitude_of_origin\",22.312133333333");
    add_srs_wkt (p, 9,
		 "34],PARAMETER[\"central_meridian\",114.1785555555556],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",132033.92],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",62565.96],AUTHORITY[\"EPSG\",\"3407\"],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3408, "epsg", 3408, "unnamed");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=90 +lon_0=0 +x_0=0 +y_0=0 +a=6371228 +");
    add_proj4text (p, 1, "b=6371228 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"unnamed\",GEOGCS[\"unnamed ellipse\",DATUM[\"un");
    add_srs_wkt (p, 1,
		 "known\",SPHEROID[\"unnamed\",6371228,0]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 2,
		 "ich\",0],UNIT[\"degree\",0.0174532925199433]],PROJECTION");
    add_srs_wkt (p, 3,
		 "[\"Lambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 4,
		 "of_center\",90],PARAMETER[\"longitude_of_center\",0],PAR");
    add_srs_wkt (p, 5,
		 "AMETER[\"false_easting\",0],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 6, ",0],UNIT[\"Meter\",1],AUTHORITY[\"EPSG\",\"3408\"]]");
    p = add_epsg_def (first, last, 3409, "epsg", 3409, "unnamed");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=-90 +lon_0=0 +x_0=0 +y_0=0 +a=6371228 ");
    add_proj4text (p, 1, "+b=6371228 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"unnamed\",GEOGCS[\"unnamed ellipse\",DATUM[\"un");
    add_srs_wkt (p, 1,
		 "known\",SPHEROID[\"unnamed\",6371228,0]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 2,
		 "ich\",0],UNIT[\"degree\",0.0174532925199433]],PROJECTION");
    add_srs_wkt (p, 3,
		 "[\"Lambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 4,
		 "of_center\",-90],PARAMETER[\"longitude_of_center\",0],PA");
    add_srs_wkt (p, 5,
		 "RAMETER[\"false_easting\",0],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 6, ",0],UNIT[\"Meter\",1],AUTHORITY[\"EPSG\",\"3409\"]]");
    p = add_epsg_def (first, last, 3410, "epsg", 3410,
		      "NSIDC EASE-Grid Global");
    add_proj4text (p, 0,
		   "+proj=cea +lon_0=0 +lat_ts=30 +x_0=0 +y_0=0 +a=6371228 +");
    add_proj4text (p, 1, "b=6371228 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NSIDC EASE-Grid Global\",GEOGCS[\"Unspecified d");
    add_srs_wkt (p, 1,
		 "atum based upon the International 1924 Authalic Sphere\"");
    add_srs_wkt (p, 2,
		 ",DATUM[\"Not_specified_based_on_International_1924_Autha");
    add_srs_wkt (p, 3,
		 "lic_Sphere\",SPHEROID[\"International 1924 Authalic Sphe");
    add_srs_wkt (p, 4,
		 "re\",6371228,0,AUTHORITY[\"EPSG\",\"7057\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"6053\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4053\"]],U");
    add_srs_wkt (p, 8,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 9,
		 "[\"Cylindrical_Equal_Area\"],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 10,
		 "el_1\",30],PARAMETER[\"central_meridian\",0],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",0],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"3410\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 3411, "epsg", 3411,
		      "NSIDC Sea Ice Polar Stereographic North");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=90 +lat_ts=70 +lon_0=-45 +k=1 +x_0=0 ");
    add_proj4text (p, 1, "+y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NSIDC Sea Ice Polar Stereographic North\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Unspecified datum based upon the Hughes 1980 ellipso");
    add_srs_wkt (p, 2,
		 "id\",DATUM[\"Not_specified_based_on_Hughes_1980_ellipsoi");
    add_srs_wkt (p, 3,
		 "d\",SPHEROID[\"Hughes 1980\",6378273,298.279411123061,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"7058\"]],AUTHORITY[\"EPSG\",\"6054\"]");
    add_srs_wkt (p, 5,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4054\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 8,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_Stereog");
    add_srs_wkt (p, 9,
		 "raphic\"],PARAMETER[\"latitude_of_origin\",70],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"central_meridian\",-45],PARAMETER[\"scale_factor\",1]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",0],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3411\"],AXIS[\"X\",UNKNOWN]");
    add_srs_wkt (p, 13, ",AXIS[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3412, "epsg", 3412,
		      "NSIDC Sea Ice Polar Stereographic South");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 ");
    add_proj4text (p, 1, "+y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NSIDC Sea Ice Polar Stereographic South\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Unspecified datum based upon the Hughes 1980 ellipso");
    add_srs_wkt (p, 2,
		 "id\",DATUM[\"Not_specified_based_on_Hughes_1980_ellipsoi");
    add_srs_wkt (p, 3,
		 "d\",SPHEROID[\"Hughes 1980\",6378273,298.279411123061,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"7058\"]],AUTHORITY[\"EPSG\",\"6054\"]");
    add_srs_wkt (p, 5,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4054\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 8,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_Stereog");
    add_srs_wkt (p, 9,
		 "raphic\"],PARAMETER[\"latitude_of_origin\",-70],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"central_meridian\",0],PARAMETER[\"scale_factor\",1],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",0],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"3412\"],AXIS[\"X\",UNKNOWN],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3413, "epsg", 3413,
		      "WGS 84 / NSIDC Sea Ice Polar Stereographic North");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=90 +lat_ts=70 +lon_0=-45 +k=1 +x_0=0 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / NSIDC Sea Ice Polar Stereographic Nort");
    add_srs_wkt (p, 1,
		 "h\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS ");
    add_srs_wkt (p, 2,
		 "84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4326\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Polar_Stereographic\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",70],PARAMETER[\"central_meridian\",-45],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3413\"],AXIS[\"X\",UNKNOWN],AXIS[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3414, "epsg", 3414, "SVY21 / Singapore TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=1.366666666666667 +lon_0=103.83333333");
    add_proj4text (p, 1,
		   "33333 +k=1 +x_0=28001.642 +y_0=38744.572 +ellps=WGS84 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SVY21 / Singapore TM\",GEOGCS[\"SVY21\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "SVY21\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6757\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5,
		 "\"]],AUTHORITY[\"EPSG\",\"4757\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 6,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 7,
		 "or\"],PARAMETER[\"latitude_of_origin\",1.366666666666667");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",103.8333333333333],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",28");
    add_srs_wkt (p, 10,
		 "001.642],PARAMETER[\"false_northing\",38744.572],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"3414\"],AXIS[\"Northing\",NORTH],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST]]");
    p = add_epsg_def (first, last, 3415, "epsg", 3415,
		      "WGS 72BE / South China Sea Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18 +lat_2=24 +lat_0=21 +lon_0=114 +x_0=");
    add_proj4text (p, 1,
		   "500000 +y_0=500000 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 2, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / South China Sea Lambert\",GEOGCS[\"W");
    add_srs_wkt (p, 1,
		 "GS 72BE\",DATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7043\"]],TOWGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6324\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_1\",18],PARAMETER[\"standard_parallel_2\",24],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",21],PARAMETER[\"central_meri");
    add_srs_wkt (p, 11,
		 "dian\",114],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"3415\"");
    add_srs_wkt (p, 13, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3416, "epsg", 3416,
		      "ETRS89 / Austria Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=46 +lat_0=47.5 +lon_0=13.3333");
    add_proj4text (p, 1,
		   "3333333333 +x_0=400000 +y_0=400000 +ellps=GRS80 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Austria Lambert\",GEOGCS[\"ETRS89\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"European_Terrestrial_Reference_System_1989\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",49],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",46],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 10,
		 "7.5],PARAMETER[\"central_meridian\",13.33333333333333],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",400000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",400000],AUTHORITY[\"EPSG\",\"3416\"],AXIS[\"X\",");
    add_srs_wkt (p, 13, "NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3417, "epsg", 3417,
		      "NAD83 / Iowa North (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.26666666666667 +lat_2=42.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41.5 +lon_0=-93.5 +x_0=1500000 +y_0=999999.99");
    add_proj4text (p, 2,
		   "99898402 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Iowa North (ft US)\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_");
    add_srs_wkt (p, 8,
		 "Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43.266666");
    add_srs_wkt (p, 9,
		 "66666667],PARAMETER[\"standard_parallel_2\",42.066666666");
    add_srs_wkt (p, 10,
		 "66667],PARAMETER[\"latitude_of_origin\",41.5],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"central_meridian\",-93.5],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "4921250],PARAMETER[\"false_northing\",3280833.333300001]");
    add_srs_wkt (p, 13,
		 ",AUTHORITY[\"EPSG\",\"3417\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 3418, "epsg", 3418,
		      "NAD83 / Iowa South (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.61666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40 +lon_0=-93.5 +x_0=500000.00001016 +y_0=0 +");
    add_proj4text (p, 2, "ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Iowa South (ft US)\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_");
    add_srs_wkt (p, 8,
		 "Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.783333");
    add_srs_wkt (p, 9,
		 "33333333],PARAMETER[\"standard_parallel_2\",40.616666666");
    add_srs_wkt (p, 10,
		 "66667],PARAMETER[\"latitude_of_origin\",40],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-93.5],PARAMETER[\"false_easting\",16");
    add_srs_wkt (p, 12,
		 "40416.6667],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"3418\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3419, "epsg", 3419,
		      "NAD83 / Kansas North (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.78333333333333 +lat_2=38.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=38.33333333333334 +lon_0=-98 +x_0=399999.9999");
    add_proj4text (p, 2,
		   "8984 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kansas North (ft US)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.78333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",38.71666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",38.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-98],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",1312333.3333],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 13,
		 "],AUTHORITY[\"EPSG\",\"3419\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 3420, "epsg", 3420,
		      "NAD83 / Kansas South (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.56666666666667 +lat_2=37.26666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.66666666666666 +lon_0=-98.5 +x_0=399999.99");
    add_proj4text (p, 2,
		   "998984 +y_0=399999.99998984 +ellps=GRS80 +datum=NAD83 +u");
    add_proj4text (p, 3, "nits=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kansas South (ft US)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.56666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",37.26666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",36.666666666666");
    add_srs_wkt (p, 11,
		 "66],PARAMETER[\"central_meridian\",-98.5],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",1312333.3333],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",1312333.3333],AUTHORITY[\"EPSG\",\"3420\"],AXIS[\"X\",E");
    add_srs_wkt (p, 14, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3421, "epsg", 3421,
		      "NAD83 / Nevada East (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-115.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=200000.00001016 +y_0=8000000.000010163 +ellps=");
    add_proj4text (p, 2, "GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nevada East (ft US)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",34.75],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-115.5833333333333],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",656166.666");
    add_srs_wkt (p, 11,
		 "7],PARAMETER[\"false_northing\",26246666.66670001],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"3421\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 3422, "epsg", 3422,
		      "NAD83 / Nevada Central (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-116.6666666666667 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=500000.00001016 +y_0=6000000 +ellps=GRS80 +dat");
    add_proj4text (p, 2, "um=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nevada Central (ft US)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",34.75],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-116.6666666666667],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",16404");
    add_srs_wkt (p, 11,
		 "16.6667],PARAMETER[\"false_northing\",19685000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"3422\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3423, "epsg", 3423,
		      "NAD83 / Nevada West (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-118.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=800000.0000101599 +y_0=3999999.99998984 +ellps");
    add_proj4text (p, 2, "=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nevada West (ft US)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",34.75],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-118.5833333333333],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",2624666.66");
    add_srs_wkt (p, 11,
		 "67],PARAMETER[\"false_northing\",13123333.3333],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"3423\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3424, "epsg", 3424,
		      "NAD83 / New Jersey (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Jersey (ft US)\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",38.83333333333334]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",-74.5],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",492125],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3424\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3425, "epsg", 3425,
		      "NAD83(HARN) / Iowa North (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.26666666666667 +lat_2=42.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41.5 +lon_0=-93.5 +x_0=1500000 +y_0=999999.99");
    add_proj4text (p, 2, "99898402 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Iowa North (ft US)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",43.26666666666667],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",42.06666666666667],PARAMETER[\"latitude_of_origi");
    add_srs_wkt (p, 11,
		 "n\",41.5],PARAMETER[\"central_meridian\",-93.5],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",4921250],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",3280833.333300001],AUTHORITY[\"EPSG\",\"3425\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3426, "epsg", 3426,
		      "NAD83(HARN) / Iowa South (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.61666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40 +lon_0=-93.5 +x_0=500000.00001016 +y_0=0 +");
    add_proj4text (p, 2, "ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Iowa South (ft US)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",41.78333333333333],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",40.61666666666667],PARAMETER[\"latitude_of_origi");
    add_srs_wkt (p, 11,
		 "n\",40],PARAMETER[\"central_meridian\",-93.5],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",1640416.6667],PARAMETER[\"false_northi");
    add_srs_wkt (p, 13,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3426\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 14, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3427, "epsg", 3427,
		      "NAD83(HARN) / Kansas North (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.78333333333333 +lat_2=38.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=38.33333333333334 +lon_0=-98 +x_0=399999.9999");
    add_proj4text (p, 2, "8984 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kansas North (ft US)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",39.78333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",38.71666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",38.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-98],PARAMETER[\"false_easting\",1312333.3333],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3427\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3428, "epsg", 3428,
		      "NAD83(HARN) / Kansas South (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.56666666666667 +lat_2=37.26666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.66666666666666 +lon_0=-98.5 +x_0=399999.99");
    add_proj4text (p, 2,
		   "998984 +y_0=399999.99998984 +ellps=GRS80 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Kansas South (ft US)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",38.56666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",37.26666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",36.66666666666666],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-98.5],PARAMETER[\"false_easting\",1312333.3333],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",1312333.3333],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"3428\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3429, "epsg", 3429,
		      "NAD83(HARN) / Nevada East (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-115.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=200000.00001016 +y_0=8000000.000010163 +ellps=");
    add_proj4text (p, 2, "GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nevada East (ft US)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",34.75],PARAMETER[\"central_meridian\",-115.5833333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",656166.6667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",26246666.66670001],AUTHORITY[\"EPSG\",\"3429\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3430, "epsg", 3430,
		      "NAD83(HARN) / Nevada Central (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-116.6666666666667 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=500000.00001016 +y_0=6000000 +ellps=GRS80 +uni");
    add_proj4text (p, 2, "ts=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nevada Central (ft US)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",34.75],PARAMETER[\"central_meridian\",-116.666666");
    add_srs_wkt (p, 10,
		 "6666667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",1640416.6667],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",19685000],AUTHORITY[\"EPSG\",\"3430\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3431, "epsg", 3431,
		      "NAD83(HARN) / Nevada West (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-118.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=800000.0000101599 +y_0=3999999.99998984 +ellps");
    add_proj4text (p, 2, "=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nevada West (ft US)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",34.75],PARAMETER[\"central_meridian\",-118.5833333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",2624666.6667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",13123333.3333],AUTHORITY[\"EPSG\",\"3431\"],AXIS[\"X\",");
    add_srs_wkt (p, 13, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3432, "epsg", 3432,
		      "NAD83(HARN) / New Jersey (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +units=us-ft +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Jersey (ft US)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",38.83333333333334],PARAMETER[\"central_meridian\",-74.5");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",492125],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"3432\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 13, "]");
    p = add_epsg_def (first, last, 3433, "epsg", 3433,
		      "NAD83 / Arkansas North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.23333333333333 +lat_2=34.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=34.33333333333334 +lon_0=-92 +x_0=399999.9999");
    add_proj4text (p, 2,
		   "8984 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arkansas North (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.23333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",34.93333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"latitude_of_origin\",34.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-92],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",1312333.3333],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 13,
		 "],AUTHORITY[\"EPSG\",\"3433\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 3434, "epsg", 3434,
		      "NAD83 / Arkansas South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.76666666666667 +lat_2=33.3 +lat_0=32");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-92 +x_0=399999.99998984 +y_0=399");
    add_proj4text (p, 2,
		   "999.99998984 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_");
    add_proj4text (p, 3, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arkansas South (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",34.76666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",33.3],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",32.66666666666666],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-92],PARAMETER[\"false_easting\",1");
    add_srs_wkt (p, 12,
		 "312333.3333],PARAMETER[\"false_northing\",1312333.3333],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"3434\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 3435, "epsg", 3435,
		      "NAD83 / Illinois East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-88.33333333");
    add_proj4text (p, 1,
		   "333333 +k=0.9999749999999999 +x_0=300000.0000000001 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Illinois East (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",36.66666666666666");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-88.33333333333333],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999975],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",984250.0000000002],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"3435\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3436, "epsg", 3436,
		      "NAD83 / Illinois West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-90.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=699999.9999898402 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Illinois West (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",36.66666666666666");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-90.16666666666667],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999941177],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",2296583.333300001],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3436\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3437, "epsg", 3437,
		      "NAD83 / New Hampshire (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-71.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +dat");
    add_proj4text (p, 2, "um=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Hampshire (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",42.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-71.66666666666667],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.999966667],PARAMETER[\"false_easting\",98425");
    add_srs_wkt (p, 11,
		 "0.0000000002],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"3437\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3438, "epsg", 3438,
		      "NAD83 / Rhode Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.08333333333334 +lon_0=-71.5 +k=0.9");
    add_proj4text (p, 1,
		   "9999375 +x_0=99999.99998983997 +y_0=0 +ellps=GRS80 +datu");
    add_proj4text (p, 2, "m=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Rhode Island (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",41.08333333333334]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",-71.5],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99999375],PARAMETER[\"false_easting\",328083");
    add_srs_wkt (p, 11,
		 ".3333],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"3438\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3439, "epsg", 3439, "PSD93 / UTM zone 39N");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSD93 / UTM zone 39N\",GEOGCS[\"PSD93\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "PDO_Survey_Datum_1993\",SPHEROID[\"Clarke 1880 (RGS)\",6");
    add_srs_wkt (p, 2,
		 "378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6134\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4134\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",0],PARAMETER[\"central_meridian\",51],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "3439\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3440, "epsg", 3440, "PSD93 / UTM zone 40N");
    add_proj4text (p, 0, "+proj=utm +zone=40 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSD93 / UTM zone 40N\",GEOGCS[\"PSD93\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "PDO_Survey_Datum_1993\",SPHEROID[\"Clarke 1880 (RGS)\",6");
    add_srs_wkt (p, 2,
		 "378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6134\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4134\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",0],PARAMETER[\"central_meridian\",57],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "3440\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3441, "epsg", 3441,
		      "NAD83(HARN) / Arkansas North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.23333333333333 +lat_2=34.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=34.33333333333334 +lon_0=-92 +x_0=399999.9999");
    add_proj4text (p, 2, "8984 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arkansas North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",36.23333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",34.93333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",34.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-92],PARAMETER[\"false_easting\",1312333.3333],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3441\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3442, "epsg", 3442,
		      "NAD83(HARN) / Arkansas South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.76666666666667 +lat_2=33.3 +lat_0=32");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-92 +x_0=399999.99998984 +y_0=399");
    add_proj4text (p, 2, "999.99998984 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Arkansas South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",34.76666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",33.3],PARAMETER[\"latitude_of_origin\",32.6");
    add_srs_wkt (p, 11,
		 "6666666666666],PARAMETER[\"central_meridian\",-92],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_easting\",1312333.3333],PARAMETER[\"false_n");
    add_srs_wkt (p, 13,
		 "orthing\",1312333.3333],AUTHORITY[\"EPSG\",\"3442\"],AXI");
    add_srs_wkt (p, 14, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3443, "epsg", 3443,
		      "NAD83(HARN) / Illinois East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-88.33333333");
    add_proj4text (p, 1,
		   "333333 +k=0.9999749999999999 +x_0=300000.0000000001 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Illinois East (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",36.66666666666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-88.33333333333333],PARAMETER[\"scale_factor\",0.999975");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_easting\",984250.0000000002],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3443\"],AX");
    add_srs_wkt (p, 13, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3444, "epsg", 3444,
		      "NAD83(HARN) / Illinois West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-90.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=699999.9999898402 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Illinois West (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",36.66666666666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-90.16666666666667],PARAMETER[\"scale_factor\",0.999941");
    add_srs_wkt (p, 11,
		 "177],PARAMETER[\"false_easting\",2296583.333300001],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3444\"]");
    add_srs_wkt (p, 13, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3445, "epsg", 3445,
		      "NAD83(HARN) / New Hampshire (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-71.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +uni");
    add_proj4text (p, 2, "ts=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / New Hampshire (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",42.5],PARAMETER[\"central_meridian\",-71.66666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"scale_factor\",0.999966667],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_easting\",984250.0000000002],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"3445\"],AXIS[\"X\",E");
    add_srs_wkt (p, 13, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3446, "epsg", 3446,
		      "NAD83(HARN) / Rhode Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.08333333333334 +lon_0=-71.5 +k=0.9");
    add_proj4text (p, 1,
		   "9999375 +x_0=99999.99998983997 +y_0=0 +ellps=GRS80 +unit");
    add_proj4text (p, 2, "s=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Rhode Island (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",41.08333333333334],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "-71.5],PARAMETER[\"scale_factor\",0.99999375],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",328083.3333],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"3446\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3447, "epsg", 3447,
		      "ETRS89 / Belgian Lambert 2005");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.83333333333334 +lat_2=51.16666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=50.797815 +lon_0=4.359215833333333 +x_0=15032");
    add_proj4text (p, 2, "8 +y_0=166262 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Belgian Lambert 2005\",GEOGCS[\"ETRS89");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Terrestrial_Reference_System_1989\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",49.8333333333333");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",51.16666666666666],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",50.797815],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",4.359215833333333],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "easting\",150328],PARAMETER[\"false_northing\",166262],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"3447\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 14, "NORTH]]");
    p = add_epsg_def (first, last, 3448, "epsg", 3448,
		      "JAD2001 / Jamaica Metric Grid");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18 +lat_0=18 +lon_0=-77 +k_0=1 +x_0=750");
    add_proj4text (p, 1,
		   "000 +y_0=650000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JAD2001 / Jamaica Metric Grid\",GEOGCS[\"JAD200");
    add_srs_wkt (p, 1,
		 "1\",DATUM[\"Jamaica_2001\",SPHEROID[\"WGS 84\",6378137,2");
    add_srs_wkt (p, 2,
		 "98.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6758\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 4,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 5,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"4758\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_origin\",18],PARAMETER[\"central");
    add_srs_wkt (p, 9,
		 "_meridian\",-77],PARAMETER[\"scale_factor\",1],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",750000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "650000],AUTHORITY[\"EPSG\",\"3448\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3449, "epsg", 3449,
		      "JAD2001 / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JAD2001 / UTM zone 17N\",GEOGCS[\"JAD2001\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Jamaica_2001\",SPHEROID[\"WGS 84\",6378137,298.2572");
    add_srs_wkt (p, 2,
		 "23563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,");
    add_srs_wkt (p, 3,
		 "0],AUTHORITY[\"EPSG\",\"6758\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4758\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",-81]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_ea");
    add_srs_wkt (p, 10,
		 "sting\",500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3449\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 12, "ing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_11 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 3450, "epsg", 3450,
		      "JAD2001 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JAD2001 / UTM zone 18N\",GEOGCS[\"JAD2001\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Jamaica_2001\",SPHEROID[\"WGS 84\",6378137,298.2572");
    add_srs_wkt (p, 2,
		 "23563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,");
    add_srs_wkt (p, 3,
		 "0],AUTHORITY[\"EPSG\",\"6758\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4758\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",-75]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_ea");
    add_srs_wkt (p, 10,
		 "sting\",500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3450\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 12, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 3451, "epsg", 3451,
		      "NAD83 / Louisiana North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.66666666666666 +lat_2=31.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=30.5 +lon_0=-92.5 +x_0=999999.9999898402 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Louisiana North (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",32.66");
    add_srs_wkt (p, 9,
		 "666666666666],PARAMETER[\"standard_parallel_2\",31.16666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"latitude_of_origin\",30.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-92.5],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",3280833.333300001],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"3451\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 14, "NORTH]]");
    p = add_epsg_def (first, last, 3452, "epsg", 3452,
		      "NAD83 / Louisiana South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.7 +lat_2=29.3 +lat_0=28.5 +lon_0=-91");
    add_proj4text (p, 1,
		   ".33333333333333 +x_0=999999.9999898402 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Louisiana South (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",30.7]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",29.3],PARAMETER[\"lat");
    add_srs_wkt (p, 10,
		 "itude_of_origin\",28.5],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 11,
		 "91.33333333333333],PARAMETER[\"false_easting\",3280833.3");
    add_srs_wkt (p, 12,
		 "33300001],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"3452\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3453, "epsg", 3453,
		      "NAD83 / Louisiana Offshore (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.5 +lon_0=-91.33333333333333 +x_0=999999.99");
    add_proj4text (p, 2,
		   "99898402 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Louisiana Offshore (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",27.");
    add_srs_wkt (p, 9,
		 "83333333333333],PARAMETER[\"standard_parallel_2\",26.166");
    add_srs_wkt (p, 10,
		 "66666666667],PARAMETER[\"latitude_of_origin\",25.5],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-91.33333333333333],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_easting\",3280833.333300001],PARAMETER[\"false_");
    add_srs_wkt (p, 13,
		 "northing\",0],AUTHORITY[\"EPSG\",\"3453\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 14, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3454, "epsg", 3454,
		      "NAD83 / South Dakota North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.4 +lat_2=42.83333333333334 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / South Dakota North (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",44.");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",42.83333333333334],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",42.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-100.3333333333333],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"3454\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3455, "epsg", 3455,
		      "NAD83 / South Dakota South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.4 +lat_2=42.83333333333334 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / South Dakota South (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",44.");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",42.83333333333334],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",42.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-100.3333333333333],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"3455\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3456, "epsg", 3456,
		      "NAD83(HARN) / Louisiana North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.66666666666666 +lat_2=31.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=30.5 +lon_0=-92.5 +x_0=999999.9999898402 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Louisiana North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",32.66666666666666],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",31.16666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",30.5],PARAMETER[\"central_meridian\",-92.5],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_easting\",3280833.333300001],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3456\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3457, "epsg", 3457,
		      "NAD83(HARN) / Louisiana South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.7 +lat_2=29.3 +lat_0=28.5 +lon_0=-91");
    add_proj4text (p, 1,
		   ".33333333333333 +x_0=999999.9999898402 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Louisiana South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",30.7],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 10,
		 "29.3],PARAMETER[\"latitude_of_origin\",28.5],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-91.33333333333333],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_easting\",3280833.333300001],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",0],AUTHORITY[\"EPSG\",\"3457\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3458, "epsg", 3458,
		      "NAD83(HARN) / South Dakota North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.68333333333333 +lat_2=44.41666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=43.83333333333334 +lon_0=-100 +x_0=600000 +y_");
    add_proj4text (p, 2, "0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / South Dakota North (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_N");
    add_srs_wkt (p, 2,
		 "etwork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey f");
    add_srs_wkt (p, 7,
		 "oot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",45.68333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 10,
		 "dard_parallel_2\",44.41666666666666],PARAMETER[\"latitud");
    add_srs_wkt (p, 11,
		 "e_of_origin\",43.83333333333334],PARAMETER[\"central_mer");
    add_srs_wkt (p, 12,
		 "idian\",-100],PARAMETER[\"false_easting\",1968500],PARAM");
    add_srs_wkt (p, 13,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3458\"],");
    add_srs_wkt (p, 14, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3459, "epsg", 3459,
		      "NAD83(HARN) / South Dakota South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.4 +lat_2=42.83333333333334 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=600000 +y");
    add_proj4text (p, 2, "_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / South Dakota South (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_N");
    add_srs_wkt (p, 2,
		 "etwork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey f");
    add_srs_wkt (p, 7,
		 "oot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",44.4],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 10,
		 "_2\",42.83333333333334],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",42.33333333333334],PARAMETER[\"central_meridian\",-100.");
    add_srs_wkt (p, 12,
		 "3333333333333],PARAMETER[\"false_easting\",1968500],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3459\"]");
    add_srs_wkt (p, 14, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3460, "epsg", 3460,
		      "Fiji 1986 / Fiji Map Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-17 +lon_0=178.75 +k=0.99985 +x_0=200");
    add_proj4text (p, 1, "0000 +y_0=4000000 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fiji 1986 / Fiji Map Grid\",GEOGCS[\"Fiji 1986\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Fiji_Geodetic_Datum_1986\",SPHEROID[\"WGS 72\",");
    add_srs_wkt (p, 2,
		 "6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6720\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4720\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",-17],PARAMETER[\"central_meridian\",178.75],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.99985],PARAMETER[\"false_easting\",2");
    add_srs_wkt (p, 10,
		 "000000],PARAMETER[\"false_northing\",4000000],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3460\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3461, "epsg", 3461,
		      "Dabola 1981 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +a=6378249.2 +b=6356515 +towgs84=-83,");
    add_proj4text (p, 1, "37,124,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Dabola 1981 / UTM zone 28N\",GEOGCS[\"Dabola 19");
    add_srs_wkt (p, 1,
		 "81\",DATUM[\"Dabola_1981\",SPHEROID[\"Clarke 1880 (IGN)\"");
    add_srs_wkt (p, 2,
		 ",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-83,37,124,0,0,0,0],AUTHORITY[\"EPSG\",\"6155");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4155\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-15],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3461\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3462, "epsg", 3462,
		      "Dabola 1981 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +a=6378249.2 +b=6356515 +towgs84=-83,");
    add_proj4text (p, 1, "37,124,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Dabola 1981 / UTM zone 29N\",GEOGCS[\"Dabola 19");
    add_srs_wkt (p, 1,
		 "81\",DATUM[\"Dabola_1981\",SPHEROID[\"Clarke 1880 (IGN)\"");
    add_srs_wkt (p, 2,
		 ",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-83,37,124,0,0,0,0],AUTHORITY[\"EPSG\",\"6155");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4155\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-9],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"3462\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3463, "epsg", 3463,
		      "NAD83 / Maine CS2000 Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.5 +lon_0=-69.125 +k=0.99998 +x_0=5");
    add_proj4text (p, 1,
		   "00000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine CS2000 Central\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",43.5],PARAMETER[\"central_meridian\",-69.12");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.99998],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"3463\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 3464, "epsg", 3464,
		      "NAD83(HARN) / Maine CS2000 Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.5 +lon_0=-69.125 +k=0.99998 +x_0=5");
    add_proj4text (p, 1, "00000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine CS2000 Central\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",43.5],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-69.125],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "99998],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"3464\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3465, "epsg", 3465,
		      "NAD83(NSRS2007) / Alabama East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30.5 +lon_0=-85.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9996 +x_0=200000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alabama East\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",30.5],PARAMETER[\"central_meridian\",-85.833");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",0.99996],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",200000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3465\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3466, "epsg", 3466,
		      "NAD83(NSRS2007) / Alabama West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-87.5 +k=0.999933333 +x_0=6");
    add_proj4text (p, 1,
		   "00000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alabama West\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",30],PARAMETER[\"central_meridian\",-87.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.999933333],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",600000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"3466\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 3467, "epsg", 3467,
		      "NAD83(NSRS2007) / Alaska Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=55 +lat_2=65 +lat_0=50 +lon_0=-154 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska Albers\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Albers_Conic_Equal_Area\"],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_1\",55],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 10,
		 "65],PARAMETER[\"latitude_of_center\",50],PARAMETER[\"lon");
    add_srs_wkt (p, 11,
		 "gitude_of_center\",-154],PARAMETER[\"false_easting\",0],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"346");
    add_srs_wkt (p, 13, "7\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3468, "epsg", 3468,
		      "NAD83(NSRS2007) / Alaska zone 1");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=57 +lonc=-133.6666666666667 +alpha=32");
    add_proj4text (p, 1,
		   "3.1301023611111 +k=0.9999 +x_0=5000000 +y_0=-5000000 +el");
    add_proj4text (p, 2, "lps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 1\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETER[\"lati");
    add_srs_wkt (p, 9,
		 "tude_of_center\",57],PARAMETER[\"longitude_of_center\",-");
    add_srs_wkt (p, 10,
		 "133.6666666666667],PARAMETER[\"azimuth\",323.13010236111");
    add_srs_wkt (p, 11,
		 "11],PARAMETER[\"rectified_grid_angle\",323.1301023611111");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_e");
    add_srs_wkt (p, 13,
		 "asting\",5000000],PARAMETER[\"false_northing\",-5000000]");
    add_srs_wkt (p, 14,
		 ",AUTHORITY[\"EPSG\",\"3468\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 15, ",NORTH]]");
    p = add_epsg_def (first, last, 3469, "epsg", 3469,
		      "NAD83(NSRS2007) / Alaska zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-142 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 2\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-142],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3469\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3470, "epsg", 3470,
		      "NAD83(NSRS2007) / Alaska zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-146 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 3\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-146],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3470\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3471, "epsg", 3471,
		      "NAD83(NSRS2007) / Alaska zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-150 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 4\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-150],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3471\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3472, "epsg", 3472,
		      "NAD83(NSRS2007) / Alaska zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-154 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 5\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-154],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3472\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3473, "epsg", 3473,
		      "NAD83(NSRS2007) / Alaska zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-158 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 6\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-158],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3473\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3474, "epsg", 3474,
		      "NAD83(NSRS2007) / Alaska zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-162 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 7\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-162],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3474\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3475, "epsg", 3475,
		      "NAD83(NSRS2007) / Alaska zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-166 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 8\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-166],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3475\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3476, "epsg", 3476,
		      "NAD83(NSRS2007) / Alaska zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-170 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1,
		   "+y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 9\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",54],PARAMETER[\"central_meridian\",-170],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 11,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 12, "EPSG\",\"3476\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3477, "epsg", 3477,
		      "NAD83(NSRS2007) / Alaska zone 10");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=53.83333333333334 +lat_2=51.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=51 +lon_0=-176 +x_0=1000000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Alaska zone 10\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",53.83333333333334],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",51.83333333333334],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",51],PARAMETER[\"central_meridian\",-176],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_easting\",1000000],PARAMETER[\"false_n");
    add_srs_wkt (p, 13,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3477\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 14, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3478, "epsg", 3478,
		      "NAD83(NSRS2007) / Arizona Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-111.9166666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arizona Central\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",31],PARAMETER[\"central_meridian\",-111.9");
    add_srs_wkt (p, 10,
		 "166666666667],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",213360],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3478\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3479, "epsg", 3479,
		      "NAD83(NSRS2007) / Arizona Central (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-111.9166666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arizona Central (ft)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 8,
		 "\"9002\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"latitude_of_origin\",31],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 10,
		 "\",-111.9166666666667],PARAMETER[\"scale_factor\",0.9999");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_easting\",700000],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",0],AUTHORITY[\"EPSG\",\"3479\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3480, "epsg", 3480,
		      "NAD83(NSRS2007) / Arizona East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-110.1666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arizona East\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",31],PARAMETER[\"central_meridian\",-110.1666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",213360],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3480\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3481, "epsg", 3481,
		      "NAD83(NSRS2007) / Arizona East (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-110.1666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arizona East (ft)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "02\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",31],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 10,
		 "110.1666666666667],PARAMETER[\"scale_factor\",0.9999],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",700000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3481\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3482, "epsg", 3482,
		      "NAD83(NSRS2007) / Arizona West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-113.75 +k=0.999933333 +x_0");
    add_proj4text (p, 1,
		   "=213360 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arizona West\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",31],PARAMETER[\"central_meridian\",-113.75],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",0.999933333],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",213360],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"3482\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 3483, "epsg", 3483,
		      "NAD83(NSRS2007) / Arizona West (ft)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-113.75 +k=0.999933333 +x_0");
    add_proj4text (p, 1,
		   "=213360 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arizona West (ft)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "02\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",31],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 10,
		 "113.75],PARAMETER[\"scale_factor\",0.999933333],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_easting\",700000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3483\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3484, "epsg", 3484,
		      "NAD83(NSRS2007) / Arkansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.23333333333333 +lat_2=34.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=34.33333333333334 +lon_0=-92 +x_0=400000 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arkansas North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",36.23333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",34.93333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",34.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-92],PARAMETER[\"false_easting\",400000],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3484\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3485, "epsg", 3485,
		      "NAD83(NSRS2007) / Arkansas North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.23333333333333 +lat_2=34.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=34.33333333333334 +lon_0=-92 +x_0=399999.9999");
    add_proj4text (p, 2,
		   "8984 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=u");
    add_proj4text (p, 3, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arkansas North (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "36.23333333333333],PARAMETER[\"standard_parallel_2\",34.");
    add_srs_wkt (p, 11,
		 "93333333333333],PARAMETER[\"latitude_of_origin\",34.3333");
    add_srs_wkt (p, 12,
		 "3333333334],PARAMETER[\"central_meridian\",-92],PARAMETE");
    add_srs_wkt (p, 13,
		 "R[\"false_easting\",1312333.3333],PARAMETER[\"false_nort");
    add_srs_wkt (p, 14,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3485\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 15, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3486, "epsg", 3486,
		      "NAD83(NSRS2007) / Arkansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.76666666666667 +lat_2=33.3 +lat_0=32");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-92 +x_0=400000 +y_0=400000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arkansas South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",34.76666666666667],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",33.3],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",32.66666666666666],PARAMETER[\"central_meridian\",-92],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_easting\",400000],PARAMETER[\"false_no");
    add_srs_wkt (p, 13,
		 "rthing\",400000],AUTHORITY[\"EPSG\",\"3486\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3487, "epsg", 3487,
		      "NAD83(NSRS2007) / Arkansas South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.76666666666667 +lat_2=33.3 +lat_0=32");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-92 +x_0=399999.99998984 +y_0=399");
    add_proj4text (p, 2,
		   "999.99998984 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 3, "us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Arkansas South (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "34.76666666666667],PARAMETER[\"standard_parallel_2\",33.");
    add_srs_wkt (p, 11,
		 "3],PARAMETER[\"latitude_of_origin\",32.66666666666666],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"central_meridian\",-92],PARAMETER[\"false_eas");
    add_srs_wkt (p, 13,
		 "ting\",1312333.3333],PARAMETER[\"false_northing\",131233");
    add_srs_wkt (p, 14,
		 "3.3333],AUTHORITY[\"EPSG\",\"3487\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 15, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3488, "epsg", 3488,
		      "NAD83(NSRS2007) / California Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=34 +lat_2=40.5 +lat_0=0 +lon_0=-120 +x_");
    add_proj4text (p, 1,
		   "0=0 +y_0=-4000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California Albers\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Albers_Conic_Equal_Area\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",34],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 10,
		 "\",40.5],PARAMETER[\"latitude_of_center\",0],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "longitude_of_center\",-120],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "0],PARAMETER[\"false_northing\",-4000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"3488\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3489, "epsg", 3489,
		      "NAD83(NSRS2007) / California zone 1");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.66666666666666 +lat_2=40 +lat_0=39.3");
    add_proj4text (p, 1,
		   "3333333333334 +lon_0=-122 +x_0=2000000 +y_0=500000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 1\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",41.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",40],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 11,
		 "\",39.33333333333334],PARAMETER[\"central_meridian\",-12");
    add_srs_wkt (p, 12,
		 "2],PARAMETER[\"false_easting\",2000000],PARAMETER[\"fals");
    add_srs_wkt (p, 13,
		 "e_northing\",500000],AUTHORITY[\"EPSG\",\"3489\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3490, "epsg", 3490,
		      "NAD83(NSRS2007) / California zone 1 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.66666666666666 +lat_2=40 +lat_0=39.3");
    add_proj4text (p, 1,
		   "3333333333334 +lon_0=-122 +x_0=2000000.0001016 +y_0=5000");
    add_proj4text (p, 2,
		   "00.0001016001 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units");
    add_proj4text (p, 3, "=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 1 (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",41.66666666666666],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 11,
		 "40],PARAMETER[\"latitude_of_origin\",39.33333333333334],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"central_meridian\",-122],PARAMETER[\"false_e");
    add_srs_wkt (p, 13,
		 "asting\",6561666.667],PARAMETER[\"false_northing\",16404");
    add_srs_wkt (p, 14,
		 "16.667],AUTHORITY[\"EPSG\",\"3490\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 15, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3491, "epsg", 3491,
		      "NAD83(NSRS2007) / California zone 2");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.83333333333334 +lat_2=38.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=37.66666666666666 +lon_0=-122 +x_0=2000000 +y");
    add_proj4text (p, 2,
		   "_0=500000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 2\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",39.83333333333334],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",38.33333333333334],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",37.66666666666666],PARAMETER[\"central");
    add_srs_wkt (p, 12,
		 "_meridian\",-122],PARAMETER[\"false_easting\",2000000],P");
    add_srs_wkt (p, 13,
		 "ARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 14, "3491\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3492, "epsg", 3492,
		      "NAD83(NSRS2007) / California zone 2 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.83333333333334 +lat_2=38.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=37.66666666666666 +lon_0=-122 +x_0=2000000.00");
    add_proj4text (p, 2,
		   "01016 +y_0=500000.0001016001 +ellps=GRS80 +towgs84=0,0,0");
    add_proj4text (p, 3, ",0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 2 (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",39.83333333333334],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 11,
		 "38.33333333333334],PARAMETER[\"latitude_of_origin\",37.6");
    add_srs_wkt (p, 12,
		 "6666666666666],PARAMETER[\"central_meridian\",-122],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_easting\",6561666.667],PARAMETER[\"false_n");
    add_srs_wkt (p, 14,
		 "orthing\",1640416.667],AUTHORITY[\"EPSG\",\"3492\"],AXIS");
    add_srs_wkt (p, 15, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3493, "epsg", 3493,
		      "NAD83(NSRS2007) / California zone 3");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.5 +lon_0=-120.5 +x_0=2000000 +y_0=500000 +");
    add_proj4text (p, 2,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 3\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",38.43333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",37.06666666666667],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",36.5],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 12,
		 "120.5],PARAMETER[\"false_easting\",2000000],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",500000],AUTHORITY[\"EPSG\",\"3493\"],AX");
    add_srs_wkt (p, 14, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3494, "epsg", 3494,
		      "NAD83(NSRS2007) / California zone 3 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.5 +lon_0=-120.5 +x_0=2000000.0001016 +y_0=");
    add_proj4text (p, 2,
		   "500000.0001016001 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 3, "nits=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 3 (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",38.43333333333333],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 11,
		 "37.06666666666667],PARAMETER[\"latitude_of_origin\",36.5");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"central_meridian\",-120.5],PARAMETER[\"fal");
    add_srs_wkt (p, 13,
		 "se_easting\",6561666.667],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 14,
		 "640416.667],AUTHORITY[\"EPSG\",\"3494\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 15, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3495, "epsg", 3495,
		      "NAD83(NSRS2007) / California zone 4");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.25 +lat_2=36 +lat_0=35.3333333333333");
    add_proj4text (p, 1,
		   "4 +lon_0=-119 +x_0=2000000 +y_0=500000 +ellps=GRS80 +tow");
    add_proj4text (p, 2, "gs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 4\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",37.25],PARAMETER[\"standard_par");
    add_srs_wkt (p, 10,
		 "allel_2\",36],PARAMETER[\"latitude_of_origin\",35.333333");
    add_srs_wkt (p, 11,
		 "33333334],PARAMETER[\"central_meridian\",-119],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_easting\",2000000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",500000],AUTHORITY[\"EPSG\",\"3495\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 14, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3496, "epsg", 3496,
		      "NAD83(NSRS2007) / California zone 4 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.25 +lat_2=36 +lat_0=35.3333333333333");
    add_proj4text (p, 1,
		   "4 +lon_0=-119 +x_0=2000000.0001016 +y_0=500000.000101600");
    add_proj4text (p, 2,
		   "1 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 4 (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",37.25],PARAMETER[\"standard_parallel_2\",36],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"latitude_of_origin\",35.33333333333334],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "central_meridian\",-119],PARAMETER[\"false_easting\",656");
    add_srs_wkt (p, 13,
		 "1666.667],PARAMETER[\"false_northing\",1640416.667],AUTH");
    add_srs_wkt (p, 14,
		 "ORITY[\"EPSG\",\"3496\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 15, "TH]]");
    p = add_epsg_def (first, last, 3497, "epsg", 3497,
		      "NAD83(NSRS2007) / California zone 5");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.46666666666667 +lat_2=34.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.5 +lon_0=-118 +x_0=2000000 +y_0=500000 +el");
    add_proj4text (p, 2, "lps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 5\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",35.46666666666667],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",34.03333333333333],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",33.5],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 12,
		 "118],PARAMETER[\"false_easting\",2000000],PARAMETER[\"fa");
    add_srs_wkt (p, 13,
		 "lse_northing\",500000],AUTHORITY[\"EPSG\",\"3497\"],AXIS");
    add_srs_wkt (p, 14, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3498, "epsg", 3498,
		      "NAD83(NSRS2007) / California zone 5 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.46666666666667 +lat_2=34.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.5 +lon_0=-118 +x_0=2000000.0001016 +y_0=50");
    add_proj4text (p, 2,
		   "0000.0001016001 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +uni");
    add_proj4text (p, 3, "ts=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 5 (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",35.46666666666667],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 11,
		 "34.03333333333333],PARAMETER[\"latitude_of_origin\",33.5");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"central_meridian\",-118],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_easting\",6561666.667],PARAMETER[\"false_northing\",164");
    add_srs_wkt (p, 14,
		 "0416.667],AUTHORITY[\"EPSG\",\"3498\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 15, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3499, "epsg", 3499,
		      "NAD83(NSRS2007) / California zone 6");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.88333333333333 +lat_2=32.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=32.16666666666666 +lon_0=-116.25 +x_0=2000000");
    add_proj4text (p, 2,
		   " +y_0=500000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 3, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 6\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",33.88333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",32.78333333333333],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",32.16666666666666],PARAMETER[\"central");
    add_srs_wkt (p, 12,
		 "_meridian\",-116.25],PARAMETER[\"false_easting\",2000000");
    add_srs_wkt (p, 13,
		 "],PARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"3499\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3500, "epsg", 3500,
		      "NAD83(NSRS2007) / California zone 6 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.88333333333333 +lat_2=32.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=32.16666666666666 +lon_0=-116.25 +x_0=2000000");
    add_proj4text (p, 2,
		   ".0001016 +y_0=500000.0001016001 +ellps=GRS80 +towgs84=0,");
    add_proj4text (p, 3, "0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / California zone 6 (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",33.88333333333333],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 11,
		 "32.78333333333333],PARAMETER[\"latitude_of_origin\",32.1");
    add_srs_wkt (p, 12,
		 "6666666666666],PARAMETER[\"central_meridian\",-116.25],P");
    add_srs_wkt (p, 13,
		 "ARAMETER[\"false_easting\",6561666.667],PARAMETER[\"fals");
    add_srs_wkt (p, 14,
		 "e_northing\",1640416.667],AUTHORITY[\"EPSG\",\"3500\"],A");
    add_srs_wkt (p, 15, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3501, "epsg", 3501,
		      "NAD83(NSRS2007) / Colorado Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.75 +lat_2=38.45 +lat_0=37.8333333333");
    add_proj4text (p, 1,
		   "3334 +lon_0=-105.5 +x_0=914401.8289 +y_0=304800.6096 +el");
    add_proj4text (p, 2, "lps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Colorado Central\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",39.75],PARAMETER[\"standard_par");
    add_srs_wkt (p, 10,
		 "allel_2\",38.45],PARAMETER[\"latitude_of_origin\",37.833");
    add_srs_wkt (p, 11,
		 "33333333334],PARAMETER[\"central_meridian\",-105.5],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_easting\",914401.8289],PARAMETER[\"false_n");
    add_srs_wkt (p, 13,
		 "orthing\",304800.6096],AUTHORITY[\"EPSG\",\"3501\"],AXIS");
    add_srs_wkt (p, 14, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3502, "epsg", 3502,
		      "NAD83(NSRS2007) / Colorado Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.75 +lat_2=38.45 +lat_0=37.8333333333");
    add_proj4text (p, 1,
		   "3334 +lon_0=-105.5 +x_0=914401.8288036576 +y_0=304800.60");
    add_proj4text (p, 2,
		   "96012192 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Colorado Central (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 8,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 9,
		 "Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 10,
		 ",39.75],PARAMETER[\"standard_parallel_2\",38.45],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"latitude_of_origin\",37.83333333333334],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "central_meridian\",-105.5],PARAMETER[\"false_easting\",3");
    add_srs_wkt (p, 13,
		 "000000],PARAMETER[\"false_northing\",1000000],AUTHORITY[");
    add_srs_wkt (p, 14,
		 "\"EPSG\",\"3502\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3503, "epsg", 3503,
		      "NAD83(NSRS2007) / Colorado North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.78333333333333 +lat_2=39.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=39.33333333333334 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "289 +y_0=304800.6096 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 3, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Colorado North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",40.78333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",39.71666666666667],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",39.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-105.5],PARAMETER[\"false_easting\",914401.8289");
    add_srs_wkt (p, 13,
		 "],PARAMETER[\"false_northing\",304800.6096],AUTHORITY[\"");
    add_srs_wkt (p, 14, "EPSG\",\"3503\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3504, "epsg", 3504,
		      "NAD83(NSRS2007) / Colorado North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.78333333333333 +lat_2=39.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=39.33333333333334 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "288036576 +y_0=304800.6096012192 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 3, ",0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Colorado North (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "40.78333333333333],PARAMETER[\"standard_parallel_2\",39.");
    add_srs_wkt (p, 11,
		 "71666666666667],PARAMETER[\"latitude_of_origin\",39.3333");
    add_srs_wkt (p, 12,
		 "3333333334],PARAMETER[\"central_meridian\",-105.5],PARAM");
    add_srs_wkt (p, 13,
		 "ETER[\"false_easting\",3000000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 14,
		 "ng\",1000000],AUTHORITY[\"EPSG\",\"3504\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 15, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3505, "epsg", 3505,
		      "NAD83(NSRS2007) / Colorado South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.66666666666666 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "289 +y_0=304800.6096 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 3, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Colorado South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",38.43333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",37.23333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",36.66666666666666],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-105.5],PARAMETER[\"false_easting\",914401.8289");
    add_srs_wkt (p, 13,
		 "],PARAMETER[\"false_northing\",304800.6096],AUTHORITY[\"");
    add_srs_wkt (p, 14, "EPSG\",\"3505\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3506, "epsg", 3506,
		      "NAD83(NSRS2007) / Colorado South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.66666666666666 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "288036576 +y_0=304800.6096012192 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 3, ",0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Colorado South (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "38.43333333333333],PARAMETER[\"standard_parallel_2\",37.");
    add_srs_wkt (p, 11,
		 "23333333333333],PARAMETER[\"latitude_of_origin\",36.6666");
    add_srs_wkt (p, 12,
		 "6666666666],PARAMETER[\"central_meridian\",-105.5],PARAM");
    add_srs_wkt (p, 13,
		 "ETER[\"false_easting\",3000000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 14,
		 "ng\",1000000],AUTHORITY[\"EPSG\",\"3506\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 15, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3507, "epsg", 3507,
		      "NAD83(NSRS2007) / Connecticut");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.86666666666667 +lat_2=41.2 +lat_0=40");
    add_proj4text (p, 1,
		   ".83333333333334 +lon_0=-72.75 +x_0=304800.6096 +y_0=1524");
    add_proj4text (p, 2,
		   "00.3048 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no");
    add_proj4text (p, 3, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Connecticut\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",41.86666666666667],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",41.2],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 11,
		 "40.83333333333334],PARAMETER[\"central_meridian\",-72.75");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",304800.6096],PARAMETER[\"f");
    add_srs_wkt (p, 13,
		 "alse_northing\",152400.3048],AUTHORITY[\"EPSG\",\"3507\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3508, "epsg", 3508,
		      "NAD83(NSRS2007) / Connecticut (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.86666666666667 +lat_2=41.2 +lat_0=40");
    add_proj4text (p, 1,
		   ".83333333333334 +lon_0=-72.75 +x_0=304800.6096012192 +y_");
    add_proj4text (p, 2,
		   "0=152400.3048006096 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 ");
    add_proj4text (p, 3, "+units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Connecticut (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.86");
    add_srs_wkt (p, 10,
		 "666666666667],PARAMETER[\"standard_parallel_2\",41.2],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"latitude_of_origin\",40.83333333333334],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"central_meridian\",-72.75],PARAMETER[\"false_easti");
    add_srs_wkt (p, 13,
		 "ng\",1000000],PARAMETER[\"false_northing\",500000],AUTHO");
    add_srs_wkt (p, 14,
		 "RITY[\"EPSG\",\"3508\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 15, "H]]");
    p = add_epsg_def (first, last, 3509, "epsg", 3509,
		      "NAD83(NSRS2007) / Delaware");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=-75.41666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "995 +x_0=200000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Delaware\",GEOGCS[\"NAD83(NSR");
    add_srs_wkt (p, 1,
		 "S2007)\",DATUM[\"NAD83_National_Spatial_Reference_System");
    add_srs_wkt (p, 2,
		 "_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4759\"");
    add_srs_wkt (p, 7,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",38],PARAMETER[\"central_meridian\",-75.4166666666");
    add_srs_wkt (p, 10,
		 "6667],PARAMETER[\"scale_factor\",0.999995],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",200000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"3509\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3510, "epsg", 3510,
		      "NAD83(NSRS2007) / Delaware (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=-75.41666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "995 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Delaware (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,AU");
    add_srs_wkt (p, 8,
		 "THORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 9,
		 "ator\"],PARAMETER[\"latitude_of_origin\",38],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",-75.41666666666667],PARAMETER[\"scale");
    add_srs_wkt (p, 11,
		 "_factor\",0.999995],PARAMETER[\"false_easting\",656166.6");
    add_srs_wkt (p, 12,
		 "67],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3510\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3511, "epsg", 3511,
		      "NAD83(NSRS2007) / Florida East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-81 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Florida East\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",24.33333333333333],PARAMETER[\"central_merid");
    add_srs_wkt (p, 10,
		 "ian\",-81],PARAMETER[\"scale_factor\",0.999941177],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",200000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"3511\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3512, "epsg", 3512,
		      "NAD83(NSRS2007) / Florida East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-81 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +towgs");
    add_proj4text (p, 2, "84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Florida East (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",24.33333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"central_meridian\",-81],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "scale_factor\",0.999941177],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "656166.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"3512\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3513, "epsg", 3513,
		      "NAD83(NSRS2007) / Florida GDL Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=24 +lat_2=31.5 +lat_0=24 +lon_0=-84 +x_");
    add_proj4text (p, 1,
		   "0=400000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Florida GDL Albers\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Albers_Conic_Equal_Area\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",24],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 10,
		 "\",31.5],PARAMETER[\"latitude_of_center\",24],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"longitude_of_center\",-84],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",400000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"3513\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3514, "epsg", 3514,
		      "NAD83(NSRS2007) / Florida North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.75 +lat_2=29.58333333333333 +lat_0=2");
    add_proj4text (p, 1,
		   "9 +lon_0=-84.5 +x_0=600000 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Florida North\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",30.75],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 10,
		 "el_2\",29.58333333333333],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 11,
		 "\",29],PARAMETER[\"central_meridian\",-84.5],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",600000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"3514\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 3515, "epsg", 3515,
		      "NAD83(NSRS2007) / Florida North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.75 +lat_2=29.58333333333333 +lat_0=2");
    add_proj4text (p, 1,
		   "9 +lon_0=-84.5 +x_0=600000 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Florida North (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 9,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 10,
		 "0.75],PARAMETER[\"standard_parallel_2\",29.5833333333333");
    add_srs_wkt (p, 11,
		 "3],PARAMETER[\"latitude_of_origin\",29],PARAMETER[\"cent");
    add_srs_wkt (p, 12,
		 "ral_meridian\",-84.5],PARAMETER[\"false_easting\",196850");
    add_srs_wkt (p, 13,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 14, "3515\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3516, "epsg", 3516,
		      "NAD83(NSRS2007) / Florida West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-82 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Florida West\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",24.33333333333333],PARAMETER[\"central_merid");
    add_srs_wkt (p, 10,
		 "ian\",-82],PARAMETER[\"scale_factor\",0.999941177],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",200000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"3516\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3517, "epsg", 3517,
		      "NAD83(NSRS2007) / Florida West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-82 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +towgs");
    add_proj4text (p, 2, "84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Florida West (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",24.33333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"central_meridian\",-82],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "scale_factor\",0.999941177],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "656166.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"3517\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3518, "epsg", 3518,
		      "NAD83(NSRS2007) / Georgia East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-82.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=200000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Georgia East\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",30],PARAMETER[\"central_meridian\",-82.16666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",200000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3518\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3519, "epsg", 3519,
		      "NAD83(NSRS2007) / Georgia East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-82.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=200000.0001016002 +y_0=0 +ellps=GRS80 +towgs84=0,");
    add_proj4text (p, 2, "0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Georgia East (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",30],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"central_meridian\",-82.16666666666667],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",65616");
    add_srs_wkt (p, 12,
		 "6.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"3519\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3520, "epsg", 3520,
		      "NAD83(NSRS2007) / Georgia West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-84.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=700000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Georgia West\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",30],PARAMETER[\"central_meridian\",-84.16666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",700000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3520\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3521, "epsg", 3521,
		      "NAD83(NSRS2007) / Georgia West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-84.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=699999.9998983998 +y_0=0 +ellps=GRS80 +towgs84=0,");
    add_proj4text (p, 2, "0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Georgia West (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",30],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"central_meridian\",-84.16666666666667],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",22965");
    add_srs_wkt (p, 12,
		 "83.333],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13, "\",\"3521\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3522, "epsg", 3522,
		      "NAD83(NSRS2007) / Idaho Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-114 +k=0.99");
    add_proj4text (p, 1,
		   "99473679999999 +x_0=500000 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Idaho Central\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",41.66666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",-114],PARAMETER[\"scale_factor\",0.999947368],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3522\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 13, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3523, "epsg", 3523,
		      "NAD83(NSRS2007) / Idaho Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-114 +k=0.99");
    add_proj4text (p, 1,
		   "99473679999999 +x_0=500000.0001016001 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Idaho Central (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 9,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",41.6666666");
    add_srs_wkt (p, 10,
		 "6666666],PARAMETER[\"central_meridian\",-114],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.999947368],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",1640416.667],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"3523\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3524, "epsg", 3524,
		      "NAD83(NSRS2007) / Idaho East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-112.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999473679999999 +x_0=200000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Idaho East\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",41.66666666666666],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",-112.1666666666667],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 11,
		 "947368],PARAMETER[\"false_easting\",200000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3524\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3525, "epsg", 3525,
		      "NAD83(NSRS2007) / Idaho East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-112.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999473679999999 +x_0=200000.0001016002 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_");
    add_proj4text (p, 3, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Idaho East (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 9,
		 "cator\"],PARAMETER[\"latitude_of_origin\",41.66666666666");
    add_srs_wkt (p, 10,
		 "666],PARAMETER[\"central_meridian\",-112.1666666666667],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"scale_factor\",0.999947368],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",656166.667],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"3525\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 3526, "epsg", 3526,
		      "NAD83(NSRS2007) / Idaho West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-115.75 +k=0");
    add_proj4text (p, 1,
		   ".999933333 +x_0=800000 +y_0=0 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 2, "0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Idaho West\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",41.66666666666666],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",-115.75],PARAMETER[\"scale_factor\",0.999933333],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",800000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3526\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 13, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3527, "epsg", 3527,
		      "NAD83(NSRS2007) / Idaho West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-115.75 +k=0");
    add_proj4text (p, 1,
		   ".999933333 +x_0=800000.0001016001 +y_0=0 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Idaho West (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 9,
		 "cator\"],PARAMETER[\"latitude_of_origin\",41.66666666666");
    add_srs_wkt (p, 10,
		 "666],PARAMETER[\"central_meridian\",-115.75],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "scale_factor\",0.999933333],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "2624666.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"3527\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3528, "epsg", 3528,
		      "NAD83(NSRS2007) / Illinois East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-88.33333333");
    add_proj4text (p, 1,
		   "333333 +k=0.9999749999999999 +x_0=300000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Illinois East\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",36.66666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",-88.33333333333333],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 11,
		 "999975],PARAMETER[\"false_easting\",300000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3528\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3529, "epsg", 3529,
		      "NAD83(NSRS2007) / Illinois East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-88.33333333");
    add_proj4text (p, 1,
		   "333333 +k=0.9999749999999999 +x_0=300000.0000000001 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_");
    add_proj4text (p, 3, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Illinois East (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 9,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",36.6666666");
    add_srs_wkt (p, 10,
		 "6666666],PARAMETER[\"central_meridian\",-88.333333333333");
    add_srs_wkt (p, 11,
		 "33],PARAMETER[\"scale_factor\",0.999975],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_easting\",984250.0000000002],PARAMETER[\"false_northi");
    add_srs_wkt (p, 13,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3529\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 14, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3530, "epsg", 3530,
		      "NAD83(NSRS2007) / Illinois West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-90.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=700000 +y_0=0 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Illinois West\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",36.66666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",-90.16666666666667],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 11,
		 "999941177],PARAMETER[\"false_easting\",700000],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3530\"],AXIS");
    add_srs_wkt (p, 13, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3531, "epsg", 3531,
		      "NAD83(NSRS2007) / Illinois West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-90.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=699999.9999898402 +y_0=0 +ell");
    add_proj4text (p, 2,
		   "ps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Illinois West (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 9,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",36.6666666");
    add_srs_wkt (p, 10,
		 "6666666],PARAMETER[\"central_meridian\",-90.166666666666");
    add_srs_wkt (p, 11,
		 "67],PARAMETER[\"scale_factor\",0.999941177],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",2296583.333300001],PARAMETER[\"false_nor");
    add_srs_wkt (p, 13,
		 "thing\",0],AUTHORITY[\"EPSG\",\"3531\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 14, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3532, "epsg", 3532,
		      "NAD83(NSRS2007) / Indiana East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=100000 +y_0=250000 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 2, ",0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Indiana East\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",37.5],PARAMETER[\"central_meridian\",-85.666");
    add_srs_wkt (p, 10,
		 "66666666667],PARAMETER[\"scale_factor\",0.999966667],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",100000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",250000],AUTHORITY[\"EPSG\",\"3532\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3533, "epsg", 3533,
		      "NAD83(NSRS2007) / Indiana East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=99999.99989839978 +y_0=249999.9998983998 +");
    add_proj4text (p, 2,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Indiana East (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",37.5],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"central_meridian\",-85.66666666666667],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.999966667],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",328083.333],PARAMETER[\"false_northing\",820208.3330000");
    add_srs_wkt (p, 13,
		 "002],AUTHORITY[\"EPSG\",\"3533\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 14, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3534, "epsg", 3534,
		      "NAD83(NSRS2007) / Indiana West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=250000 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 2, ",0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Indiana West\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",37.5],PARAMETER[\"central_meridian\",-87.083");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",0.999966667],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",900000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",250000],AUTHORITY[\"EPSG\",\"3534\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3535, "epsg", 3535,
		      "NAD83(NSRS2007) / Indiana West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=249999.9998983998 +ellps=GRS80");
    add_proj4text (p, 2, " +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Indiana West (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",37.5],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"central_meridian\",-87.08333333333333],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.999966667],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2952750],PARAMETER[\"false_northing\",820208.3330000002");
    add_srs_wkt (p, 13,
		 "],AUTHORITY[\"EPSG\",\"3535\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 3536, "epsg", 3536,
		      "NAD83(NSRS2007) / Iowa North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.26666666666667 +lat_2=42.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41.5 +lon_0=-93.5 +x_0=1500000 +y_0=1000000 +");
    add_proj4text (p, 2,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Iowa North\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",43.26666666666667],PARAMETER[\"standa");
    add_srs_wkt (p, 10,
		 "rd_parallel_2\",42.06666666666667],PARAMETER[\"latitude_");
    add_srs_wkt (p, 11,
		 "of_origin\",41.5],PARAMETER[\"central_meridian\",-93.5],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_easting\",1500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 13,
		 "orthing\",1000000],AUTHORITY[\"EPSG\",\"3536\"],AXIS[\"X");
    add_srs_wkt (p, 14, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3537, "epsg", 3537,
		      "NAD83(NSRS2007) / Iowa North (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.26666666666667 +lat_2=42.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41.5 +lon_0=-93.5 +x_0=1500000 +y_0=999999.99");
    add_proj4text (p, 2,
		   "99898402 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Iowa North (ft US)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43.26");
    add_srs_wkt (p, 10,
		 "666666666667],PARAMETER[\"standard_parallel_2\",42.06666");
    add_srs_wkt (p, 11,
		 "666666667],PARAMETER[\"latitude_of_origin\",41.5],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"central_meridian\",-93.5],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 13,
		 "g\",4921250],PARAMETER[\"false_northing\",3280833.333300");
    add_srs_wkt (p, 14,
		 "001],AUTHORITY[\"EPSG\",\"3537\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 15, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3538, "epsg", 3538,
		      "NAD83(NSRS2007) / Iowa South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.61666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40 +lon_0=-93.5 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Iowa South\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",41.78333333333333],PARAMETER[\"standa");
    add_srs_wkt (p, 10,
		 "rd_parallel_2\",40.61666666666667],PARAMETER[\"latitude_");
    add_srs_wkt (p, 11,
		 "of_origin\",40],PARAMETER[\"central_meridian\",-93.5],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 13,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3538\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 14, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3539, "epsg", 3539,
		      "NAD83(NSRS2007) / Iowa South (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.61666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40 +lon_0=-93.5 +x_0=500000.00001016 +y_0=0 +");
    add_proj4text (p, 2,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Iowa South (ft US)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.78");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"standard_parallel_2\",40.61666");
    add_srs_wkt (p, 11,
		 "666666667],PARAMETER[\"latitude_of_origin\",40],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"central_meridian\",-93.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 13,
		 ",1640416.6667],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 14,
		 "[\"EPSG\",\"3539\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3540, "epsg", 3540,
		      "NAD83(NSRS2007) / Kansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.78333333333333 +lat_2=38.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=38.33333333333334 +lon_0=-98 +x_0=400000 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kansas North\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",39.78333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 10,
		 "dard_parallel_2\",38.71666666666667],PARAMETER[\"latitud");
    add_srs_wkt (p, 11,
		 "e_of_origin\",38.33333333333334],PARAMETER[\"central_mer");
    add_srs_wkt (p, 12,
		 "idian\",-98],PARAMETER[\"false_easting\",400000],PARAMET");
    add_srs_wkt (p, 13,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3540\"],AX");
    add_srs_wkt (p, 14, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3541, "epsg", 3541,
		      "NAD83(NSRS2007) / Kansas North (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.78333333333333 +lat_2=38.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=38.33333333333334 +lon_0=-98 +x_0=399999.9999");
    add_proj4text (p, 2,
		   "8984 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=u");
    add_proj4text (p, 3, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kansas North (ft US)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 9,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 10,
		 "9.78333333333333],PARAMETER[\"standard_parallel_2\",38.7");
    add_srs_wkt (p, 11,
		 "1666666666667],PARAMETER[\"latitude_of_origin\",38.33333");
    add_srs_wkt (p, 12,
		 "333333334],PARAMETER[\"central_meridian\",-98],PARAMETER");
    add_srs_wkt (p, 13,
		 "[\"false_easting\",1312333.3333],PARAMETER[\"false_north");
    add_srs_wkt (p, 14,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3541\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 15, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3542, "epsg", 3542,
		      "NAD83(NSRS2007) / Kansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.56666666666667 +lat_2=37.26666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.66666666666666 +lon_0=-98.5 +x_0=400000 +y");
    add_proj4text (p, 2,
		   "_0=400000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kansas South\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",38.56666666666667],PARAMETER[\"stan");
    add_srs_wkt (p, 10,
		 "dard_parallel_2\",37.26666666666667],PARAMETER[\"latitud");
    add_srs_wkt (p, 11,
		 "e_of_origin\",36.66666666666666],PARAMETER[\"central_mer");
    add_srs_wkt (p, 12,
		 "idian\",-98.5],PARAMETER[\"false_easting\",400000],PARAM");
    add_srs_wkt (p, 13,
		 "ETER[\"false_northing\",400000],AUTHORITY[\"EPSG\",\"354");
    add_srs_wkt (p, 14, "2\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3543, "epsg", 3543,
		      "NAD83(NSRS2007) / Kansas South (ft US)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.56666666666667 +lat_2=37.26666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.66666666666666 +lon_0=-98.5 +x_0=399999.99");
    add_proj4text (p, 2,
		   "998984 +y_0=399999.99998984 +ellps=GRS80 +towgs84=0,0,0,");
    add_proj4text (p, 3, "0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kansas South (ft US)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 9,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 10,
		 "8.56666666666667],PARAMETER[\"standard_parallel_2\",37.2");
    add_srs_wkt (p, 11,
		 "6666666666667],PARAMETER[\"latitude_of_origin\",36.66666");
    add_srs_wkt (p, 12,
		 "666666666],PARAMETER[\"central_meridian\",-98.5],PARAMET");
    add_srs_wkt (p, 13,
		 "ER[\"false_easting\",1312333.3333],PARAMETER[\"false_nor");
    add_srs_wkt (p, 14,
		 "thing\",1312333.3333],AUTHORITY[\"EPSG\",\"3543\"],AXIS[");
    add_srs_wkt (p, 15, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3544, "epsg", 3544,
		      "NAD83(NSRS2007) / Kentucky North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=38.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=500000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kentucky North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",37.96666666666667],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",38.96666666666667],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",37.5],PARAMETER[\"central_meridian\",-84.");
    add_srs_wkt (p, 12,
		 "25],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 13,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3544\"],AXIS[\"X\",");
    add_srs_wkt (p, 14, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3545, "epsg", 3545,
		      "NAD83(NSRS2007) / Kentucky North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=38.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=500000.0001016001 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no");
    add_proj4text (p, 3, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kentucky North (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "37.96666666666667],PARAMETER[\"standard_parallel_2\",38.");
    add_srs_wkt (p, 11,
		 "96666666666667],PARAMETER[\"latitude_of_origin\",37.5],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"central_meridian\",-84.25],PARAMETER[\"false_");
    add_srs_wkt (p, 13,
		 "easting\",1640416.667],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 14,
		 "UTHORITY[\"EPSG\",\"3545\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 15, "NORTH]]");
    p = add_epsg_def (first, last, 3546, "epsg", 3546,
		      "NAD83(NSRS2007) / Kentucky Single Zone");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.08333333333334 +lat_2=38.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=1500000 ");
    add_proj4text (p, 2,
		   "+y_0=1000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 3, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kentucky Single Zone\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_1\",37.08333333333334],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"standard_parallel_2\",38.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",36.33333333333334],PARAMETER[\"cent");
    add_srs_wkt (p, 12,
		 "ral_meridian\",-85.75],PARAMETER[\"false_easting\",15000");
    add_srs_wkt (p, 13,
		 "00],PARAMETER[\"false_northing\",1000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 14, "SG\",\"3546\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3547, "epsg", 3547,
		      "NAD83(NSRS2007) / Kentucky Single Zone (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.08333333333334 +lat_2=38.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=1500000 ");
    add_proj4text (p, 2,
		   "+y_0=999999.9998983998 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 3, ",0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kentucky Single Zone (ftUS)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spati");
    add_srs_wkt (p, 2,
		 "al_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006");
    add_srs_wkt (p, 8,
		 "096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lam");
    add_srs_wkt (p, 9,
		 "bert_Conformal_Conic_2SP\"],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 10,
		 "l_1\",37.08333333333334],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 11,
		 "\",38.66666666666666],PARAMETER[\"latitude_of_origin\",3");
    add_srs_wkt (p, 12,
		 "6.33333333333334],PARAMETER[\"central_meridian\",-85.75]");
    add_srs_wkt (p, 13,
		 ",PARAMETER[\"false_easting\",4921250],PARAMETER[\"false_");
    add_srs_wkt (p, 14,
		 "northing\",3280833.333],AUTHORITY[\"EPSG\",\"3547\"],AXI");
    add_srs_wkt (p, 15, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3548, "epsg", 3548,
		      "NAD83(NSRS2007) / Kentucky South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.93333333333333 +lat_2=36.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=500000 +");
    add_proj4text (p, 2,
		   "y_0=500000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kentucky South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",37.93333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",36.73333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",36.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-85.75],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 13,
		 "AMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 14, "548\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3549, "epsg", 3549,
		      "NAD83(NSRS2007) / Kentucky South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.93333333333333 +lat_2=36.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001016001 +y_0=500000.0001016001 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 3, ",0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Kentucky South (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "37.93333333333333],PARAMETER[\"standard_parallel_2\",36.");
    add_srs_wkt (p, 11,
		 "73333333333333],PARAMETER[\"latitude_of_origin\",36.3333");
    add_srs_wkt (p, 12,
		 "3333333334],PARAMETER[\"central_meridian\",-85.75],PARAM");
    add_srs_wkt (p, 13,
		 "ETER[\"false_easting\",1640416.667],PARAMETER[\"false_no");
    add_srs_wkt (p, 14,
		 "rthing\",1640416.667],AUTHORITY[\"EPSG\",\"3549\"],AXIS[");
    add_srs_wkt (p, 15, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3550, "epsg", 3550,
		      "NAD83(NSRS2007) / Louisiana North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.66666666666666 +lat_2=31.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=30.5 +lon_0=-92.5 +x_0=1000000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Louisiana North\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_1\",32.66666666666666],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "tandard_parallel_2\",31.16666666666667],PARAMETER[\"lati");
    add_srs_wkt (p, 11,
		 "tude_of_origin\",30.5],PARAMETER[\"central_meridian\",-9");
    add_srs_wkt (p, 12,
		 "2.5],PARAMETER[\"false_easting\",1000000],PARAMETER[\"fa");
    add_srs_wkt (p, 13,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"3550\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3551, "epsg", 3551,
		      "NAD83(NSRS2007) / Louisiana North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.66666666666666 +lat_2=31.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=30.5 +lon_0=-92.5 +x_0=999999.9999898402 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_");
    add_proj4text (p, 3, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Louisiana North (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "32.66666666666666],PARAMETER[\"standard_parallel_2\",31.");
    add_srs_wkt (p, 11,
		 "16666666666667],PARAMETER[\"latitude_of_origin\",30.5],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"central_meridian\",-92.5],PARAMETER[\"false_e");
    add_srs_wkt (p, 13,
		 "asting\",3280833.333300001],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 14,
		 ",0],AUTHORITY[\"EPSG\",\"3551\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 15, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3552, "epsg", 3552,
		      "NAD83(NSRS2007) / Louisiana South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.7 +lat_2=29.3 +lat_0=28.5 +lon_0=-91");
    add_proj4text (p, 1,
		   ".33333333333333 +x_0=1000000 +y_0=0 +ellps=GRS80 +towgs8");
    add_proj4text (p, 2, "4=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Louisiana South\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_1\",30.7],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",29.3],PARAMETER[\"latitude_of_origin\",28.5],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"central_meridian\",-91.33333333333333],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",1000000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"3552\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3553, "epsg", 3553,
		      "NAD83(NSRS2007) / Louisiana South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.7 +lat_2=29.3 +lat_0=28.5 +lon_0=-91");
    add_proj4text (p, 1,
		   ".33333333333333 +x_0=999999.9999898402 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Louisiana South (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "30.7],PARAMETER[\"standard_parallel_2\",29.3],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"latitude_of_origin\",28.5],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 12,
		 "n\",-91.33333333333333],PARAMETER[\"false_easting\",3280");
    add_srs_wkt (p, 13,
		 "833.333300001],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 14,
		 "[\"EPSG\",\"3553\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3554, "epsg", 3554,
		      "NAD83(NSRS2007) / Maine CS2000 Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.5 +lon_0=-69.125 +k=0.99998 +x_0=5");
    add_proj4text (p, 1,
		   "00000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine CS2000 Central\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",43.5],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-69.125],PARAMETER[\"scale_factor\",0.99998],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3554\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3555, "epsg", 3555,
		      "NAD83(NSRS2007) / Maine CS2000 East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.83333333333334 +lon_0=-67.875 +k=0");
    add_proj4text (p, 1,
		   ".99998 +x_0=700000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine CS2000 East\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",43.83333333333334],PARAMETER[\"central_m");
    add_srs_wkt (p, 10,
		 "eridian\",-67.875],PARAMETER[\"scale_factor\",0.99998],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",700000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",0],AUTHORITY[\"EPSG\",\"3555\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 13, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3556, "epsg", 3556,
		      "NAD83(NSRS2007) / Maine CS2000 West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.375 +k=0");
    add_proj4text (p, 1,
		   ".99998 +x_0=300000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine CS2000 West\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",42.83333333333334],PARAMETER[\"central_m");
    add_srs_wkt (p, 10,
		 "eridian\",-70.375],PARAMETER[\"scale_factor\",0.99998],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",300000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",0],AUTHORITY[\"EPSG\",\"3556\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 13, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3557, "epsg", 3557,
		      "NAD83(NSRS2007) / Maine East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine East\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",43.66666666666666],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",-68.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",300000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3557\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3558, "epsg", 3558,
		      "NAD83(NSRS2007) / Maine West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine West\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",42.83333333333334],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",-70.16666666666667],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 11,
		 "966667],PARAMETER[\"false_easting\",900000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3558\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3559, "epsg", 3559,
		      "NAD83(NSRS2007) / Maryland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.45 +lat_2=38.3 +lat_0=37.66666666666");
    add_proj4text (p, 1,
		   "666 +lon_0=-77 +x_0=400000 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maryland\",GEOGCS[\"NAD83(NSR");
    add_srs_wkt (p, 1,
		 "S2007)\",DATUM[\"NAD83_National_Spatial_Reference_System");
    add_srs_wkt (p, 2,
		 "_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4759\"");
    add_srs_wkt (p, 7,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",39.45],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 10,
		 ",38.3],PARAMETER[\"latitude_of_origin\",37.6666666666666");
    add_srs_wkt (p, 11,
		 "6],PARAMETER[\"central_meridian\",-77],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_easting\",400000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"3559\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 14, "H]]");
    p = add_epsg_def (first, last, 3560, "epsg", 3560,
		      "NAD83 / Utah North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "0001016 +y_0=999999.9999898402 +ellps=GRS80 +datum=NAD83");
    add_proj4text (p, 3, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah North (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",41.7833333");
    add_srs_wkt (p, 9,
		 "3333333],PARAMETER[\"standard_parallel_2\",40.7166666666");
    add_srs_wkt (p, 10,
		 "6667],PARAMETER[\"latitude_of_origin\",40.33333333333334");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"central_meridian\",-111.5],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_easting\",1640416.6667],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "3280833.333300001],AUTHORITY[\"EPSG\",\"3560\"],AXIS[\"X");
    add_srs_wkt (p, 14, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3561, "epsg", 3561,
		      "Old Hawaiian / Hawaii zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=18.83333333333333 +lon_0=-155.5 +k=0.");
    add_proj4text (p, 1,
		   "999966667 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +u");
    add_proj4text (p, 2, "nits=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Old Hawaiian / Hawaii zone 1\",GEOGCS[\"Old Haw");
    add_srs_wkt (p, 1,
		 "aiian\",DATUM[\"Old_Hawaiian\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6135\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4135\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",18.83333333333");
    add_srs_wkt (p, 9,
		 "333],PARAMETER[\"central_meridian\",-155.5],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.999966667],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"3561\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3562, "epsg", 3562,
		      "Old Hawaiian / Hawaii zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=20.33333333333333 +lon_0=-156.6666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=152400.3048006096 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=clrk66 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Old Hawaiian / Hawaii zone 2\",GEOGCS[\"Old Haw");
    add_srs_wkt (p, 1,
		 "aiian\",DATUM[\"Old_Hawaiian\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6135\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4135\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",20.33333333333");
    add_srs_wkt (p, 9,
		 "333],PARAMETER[\"central_meridian\",-156.6666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",0.999966667],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"3562\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 3563, "epsg", 3563,
		      "Old Hawaiian / Hawaii zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.16666666666667 +lon_0=-158 +k=0.99");
    add_proj4text (p, 1,
		   "999 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +units=u");
    add_proj4text (p, 2, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Old Hawaiian / Hawaii zone 3\",GEOGCS[\"Old Haw");
    add_srs_wkt (p, 1,
		 "aiian\",DATUM[\"Old_Hawaiian\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6135\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4135\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",21.16666666666");
    add_srs_wkt (p, 9,
		 "667],PARAMETER[\"central_meridian\",-158],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",0.99999],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 12, "563\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3564, "epsg", 3564,
		      "Old Hawaiian / Hawaii zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.83333333333333 +lon_0=-159.5 +k=0.");
    add_proj4text (p, 1,
		   "99999 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +units");
    add_proj4text (p, 2, "=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Old Hawaiian / Hawaii zone 4\",GEOGCS[\"Old Haw");
    add_srs_wkt (p, 1,
		 "aiian\",DATUM[\"Old_Hawaiian\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6135\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4135\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",21.83333333333");
    add_srs_wkt (p, 9,
		 "333],PARAMETER[\"central_meridian\",-159.5],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.99999],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "3564\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3565, "epsg", 3565,
		      "Old Hawaiian / Hawaii zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.66666666666667 +lon_0=-160.1666666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 ");
    add_proj4text (p, 2, "+units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Old Hawaiian / Hawaii zone 5\",GEOGCS[\"Old Haw");
    add_srs_wkt (p, 1,
		 "aiian\",DATUM[\"Old_Hawaiian\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6135\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4135\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",21.66666666666");
    add_srs_wkt (p, 9,
		 "667],PARAMETER[\"central_meridian\",-160.1666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"3565\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3566, "epsg", 3566,
		      "NAD83 / Utah Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000.00001016 +y_0");
    add_proj4text (p, 2,
		   "=2000000.00001016 +ellps=GRS80 +datum=NAD83 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah Central (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_");
    add_srs_wkt (p, 8,
		 "Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.65],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"standard_parallel_2\",39.01666666666667],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",38.33333333333334],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-111.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",1640416.6667],PARAMETER[\"false_northing\",6561666.6667");
    add_srs_wkt (p, 13,
		 "00001],AUTHORITY[\"EPSG\",\"3566\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3567, "epsg", 3567,
		      "NAD83 / Utah South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000.00001016 +y_0");
    add_proj4text (p, 2,
		   "=3000000 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah South (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",38.35],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_2\",37.21666666666667],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"latitude_of_origin\",36.66666666666666],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"central_meridian\",-111.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",1640416.6667],PARAMETER[\"false_northing\",9842500.0000");
    add_srs_wkt (p, 13,
		 "00002],AUTHORITY[\"EPSG\",\"3567\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3568, "epsg", 3568,
		      "NAD83(HARN) / Utah North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "0001016 +y_0=999999.9999898402 +ellps=GRS80 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah North (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",41.78333333333333],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",40.71666666666667],PARAMETER[\"latitude_of_origi");
    add_srs_wkt (p, 11,
		 "n\",40.33333333333334],PARAMETER[\"central_meridian\",-1");
    add_srs_wkt (p, 12,
		 "11.5],PARAMETER[\"false_easting\",1640416.6667],PARAMETE");
    add_srs_wkt (p, 13,
		 "R[\"false_northing\",3280833.333300001],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 14, "\",\"3568\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3569, "epsg", 3569,
		      "NAD83(HARN) / Utah Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000.00001016 +y_0");
    add_proj4text (p, 2,
		   "=2000000.00001016 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah Central (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_1\",40.65],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 10,
		 "39.01666666666667],PARAMETER[\"latitude_of_origin\",38.3");
    add_srs_wkt (p, 11,
		 "3333333333334],PARAMETER[\"central_meridian\",-111.5],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_easting\",1640416.6667],PARAMETER[\"fals");
    add_srs_wkt (p, 13,
		 "e_northing\",6561666.666700001],AUTHORITY[\"EPSG\",\"356");
    add_srs_wkt (p, 14, "9\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3570, "epsg", 3570,
		      "NAD83(HARN) / Utah South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000.00001016 +y_0");
    add_proj4text (p, 2, "=3000000 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Utah South (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",38.35],PARAMETER[\"standard_parallel_2\",37.2");
    add_srs_wkt (p, 10,
		 "1666666666667],PARAMETER[\"latitude_of_origin\",36.66666");
    add_srs_wkt (p, 11,
		 "666666666],PARAMETER[\"central_meridian\",-111.5],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_easting\",1640416.6667],PARAMETER[\"false_no");
    add_srs_wkt (p, 13,
		 "rthing\",9842500.000000002],AUTHORITY[\"EPSG\",\"3570\"]");
    add_srs_wkt (p, 14, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3571, "epsg", 3571,
		      "WGS 84 / North Pole LAEA Bering Sea");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=90 +lon_0=180 +x_0=0 +y_0=0 +ellps=WGS");
    add_proj4text (p, 1, "84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / North Pole LAEA Bering Sea\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,");
    add_srs_wkt (p, 2,
		 "298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_center\",90],PARAMETER[\"longitude_of_center\",180],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",0],PARAMETER[\"false_northing");
    add_srs_wkt (p, 10,
		 "\",0],AUTHORITY[\"EPSG\",\"3571\"],AXIS[\"X\",UNKNOWN],A");
    add_srs_wkt (p, 11, "XIS[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3572, "epsg", 3572,
		      "WGS 84 / North Pole LAEA Alaska");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=90 +lon_0=-150 +x_0=0 +y_0=0 +ellps=WG");
    add_proj4text (p, 1, "S84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / North Pole LAEA Alaska\",GEOGCS[\"WGS ");
    add_srs_wkt (p, 1,
		 "84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.");
    add_srs_wkt (p, 2,
		 "257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"L");
    add_srs_wkt (p, 7,
		 "ambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_of_c");
    add_srs_wkt (p, 8,
		 "enter\",90],PARAMETER[\"longitude_of_center\",-150],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"false_easting\",0],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 10,
		 "0],AUTHORITY[\"EPSG\",\"3572\"],AXIS[\"X\",UNKNOWN],AXIS");
    add_srs_wkt (p, 11, "[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3573, "epsg", 3573,
		      "WGS 84 / North Pole LAEA Canada");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=90 +lon_0=-100 +x_0=0 +y_0=0 +ellps=WG");
    add_proj4text (p, 1, "S84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / North Pole LAEA Canada\",GEOGCS[\"WGS ");
    add_srs_wkt (p, 1,
		 "84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.");
    add_srs_wkt (p, 2,
		 "257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"L");
    add_srs_wkt (p, 7,
		 "ambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_of_c");
    add_srs_wkt (p, 8,
		 "enter\",90],PARAMETER[\"longitude_of_center\",-100],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"false_easting\",0],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 10,
		 "0],AUTHORITY[\"EPSG\",\"3573\"],AXIS[\"X\",UNKNOWN],AXIS");
    add_srs_wkt (p, 11, "[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3574, "epsg", 3574,
		      "WGS 84 / North Pole LAEA Atlantic");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=90 +lon_0=-40 +x_0=0 +y_0=0 +ellps=WGS");
    add_proj4text (p, 1, "84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / North Pole LAEA Atlantic\",GEOGCS[\"WG");
    add_srs_wkt (p, 1,
		 "S 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,29");
    add_srs_wkt (p, 2,
		 "8.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Lambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "center\",90],PARAMETER[\"longitude_of_center\",-40],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"false_easting\",0],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 10,
		 "0],AUTHORITY[\"EPSG\",\"3574\"],AXIS[\"X\",UNKNOWN],AXIS");
    add_srs_wkt (p, 11, "[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3575, "epsg", 3575,
		      "WGS 84 / North Pole LAEA Europe");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=90 +lon_0=10 +x_0=0 +y_0=0 +ellps=WGS8");
    add_proj4text (p, 1, "4 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / North Pole LAEA Europe\",GEOGCS[\"WGS ");
    add_srs_wkt (p, 1,
		 "84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.");
    add_srs_wkt (p, 2,
		 "257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"L");
    add_srs_wkt (p, 7,
		 "ambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_of_c");
    add_srs_wkt (p, 8,
		 "enter\",90],PARAMETER[\"longitude_of_center\",10],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"false_easting\",0],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 10,
		 ",AUTHORITY[\"EPSG\",\"3575\"],AXIS[\"X\",UNKNOWN],AXIS[\"");
    add_srs_wkt (p, 11, "Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3576, "epsg", 3576,
		      "WGS 84 / North Pole LAEA Russia");
    add_proj4text (p, 0,
		   "+proj=laea +lat_0=90 +lon_0=90 +x_0=0 +y_0=0 +ellps=WGS8");
    add_proj4text (p, 1, "4 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / North Pole LAEA Russia\",GEOGCS[\"WGS ");
    add_srs_wkt (p, 1,
		 "84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.");
    add_srs_wkt (p, 2,
		 "257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"L");
    add_srs_wkt (p, 7,
		 "ambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_of_c");
    add_srs_wkt (p, 8,
		 "enter\",90],PARAMETER[\"longitude_of_center\",90],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"false_easting\",0],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 10,
		 ",AUTHORITY[\"EPSG\",\"3576\"],AXIS[\"X\",UNKNOWN],AXIS[\"");
    add_srs_wkt (p, 11, "Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3577, "epsg", 3577,
		      "GDA94 / Australian Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=-18 +lat_2=-36 +lat_0=0 +lon_0=132 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / Australian Albers\",GEOGCS[\"GDA94\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Albers_Conic_");
    add_srs_wkt (p, 8,
		 "Equal_Area\"],PARAMETER[\"standard_parallel_1\",-18],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_2\",-36],PARAMETER[\"latitude");
    add_srs_wkt (p, 10,
		 "_of_center\",0],PARAMETER[\"longitude_of_center\",132],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",0],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",0],AUTHORITY[\"EPSG\",\"3577\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Northing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_12 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 3578, "epsg", 3578, "NAD83 / Yukon Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=61.66666666666666 +lat_2=68 +lat_0=59 +");
    add_proj4text (p, 1,
		   "lon_0=-132.5 +x_0=500000 +y_0=500000 +ellps=GRS80 +datum");
    add_proj4text (p, 2, "=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Yukon Albers\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Albers_Conic_Equal_Area\"],PARAMETER[\"standard_par");
    add_srs_wkt (p, 8,
		 "allel_1\",61.66666666666666],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 9,
		 "el_2\",68],PARAMETER[\"latitude_of_center\",59],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"longitude_of_center\",-132.5],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",500000],PARAMETER[\"false_northing\",500000],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"3578\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 3579, "epsg", 3579,
		      "NAD83(CSRS) / Yukon Albers");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=61.66666666666666 +lat_2=68 +lat_0=59 +");
    add_proj4text (p, 1,
		   "lon_0=-132.5 +x_0=500000 +y_0=500000 +ellps=GRS80 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Yukon Albers\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Albers_Conic_Equal_Area\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",61.66666666666666],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",68],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_center\",59],PARAMETER[\"longitude_of_center\",-132");
    add_srs_wkt (p, 11,
		 ".5],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",500000],AUTHORITY[\"EPSG\",\"3579\"],AXIS[\"");
    add_srs_wkt (p, 13, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3580, "epsg", 3580, "NAD83 / NWT Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=62 +lat_2=70 +lat_0=0 +lon_0=-112 +x_0=");
    add_proj4text (p, 1,
		   "0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / NWT Lambert\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",62],PARAMETER[\"standard_parallel_2\",70],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",-112],PARAMETER[\"false_easting\",0],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3580\"],AXI");
    add_srs_wkt (p, 12, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3581, "epsg", 3581,
		      "NAD83(CSRS) / NWT Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=62 +lat_2=70 +lat_0=0 +lon_0=-112 +x_0=");
    add_proj4text (p, 1, "0 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / NWT Lambert\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",62],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",70],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 10,
		 "\",0],PARAMETER[\"central_meridian\",-112],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",0],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"3581\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 3582, "epsg", 3582,
		      "NAD83(NSRS2007) / Maryland (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.45 +lat_2=38.3 +lat_0=37.66666666666");
    add_proj4text (p, 1,
		   "666 +lon_0=-77 +x_0=399999.9998983998 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maryland (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,AU");
    add_srs_wkt (p, 8,
		 "THORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conform");
    add_srs_wkt (p, 9,
		 "al_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.45]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"standard_parallel_2\",38.3],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",37.66666666666666],PARAMETER[\"central");
    add_srs_wkt (p, 12,
		 "_meridian\",-77],PARAMETER[\"false_easting\",1312333.333");
    add_srs_wkt (p, 13,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 14, "582\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3583, "epsg", 3583,
		      "NAD83(NSRS2007) / Massachusetts Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.48333333333333 +lat_2=41.28333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-70.5 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Massachusetts Island\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_1\",41.48333333333333],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"standard_parallel_2\",41.28333333333333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",41],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 12,
		 "-70.5],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 13,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"3583\"],AXIS[\"X");
    add_srs_wkt (p, 14, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3584, "epsg", 3584,
		      "NAD83(NSRS2007) / Massachusetts Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.48333333333333 +lat_2=41.28333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-70.5 +x_0=500000.0001016001 +y_0=0");
    add_proj4text (p, 2,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_de");
    add_proj4text (p, 3, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Massachusetts Island (ftUS)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spati");
    add_srs_wkt (p, 2,
		 "al_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006");
    add_srs_wkt (p, 8,
		 "096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lam");
    add_srs_wkt (p, 9,
		 "bert_Conformal_Conic_2SP\"],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 10,
		 "l_1\",41.48333333333333],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 11,
		 "\",41.28333333333333],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 12,
		 "1],PARAMETER[\"central_meridian\",-70.5],PARAMETER[\"fal");
    add_srs_wkt (p, 13,
		 "se_easting\",1640416.667],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 14,
		 "],AUTHORITY[\"EPSG\",\"3584\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 15, "\",NORTH]]");
    p = add_epsg_def (first, last, 3585, "epsg", 3585,
		      "NAD83(NSRS2007) / Massachusetts Mainland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.68333333333333 +lat_2=41.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41 +lon_0=-71.5 +x_0=200000 +y_0=750000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Massachusetts Mainland\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 8,
		 "001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"standard_parallel_1\",42.68333333333333],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"standard_parallel_2\",41.71666666666667],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"latitude_of_origin\",41],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 12,
		 ",-71.5],PARAMETER[\"false_easting\",200000],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",750000],AUTHORITY[\"EPSG\",\"3585\"],AX");
    add_srs_wkt (p, 14, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3586, "epsg", 3586,
		      "NAD83(NSRS2007) / Massachusetts Mainland (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.68333333333333 +lat_2=41.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41 +lon_0=-71.5 +x_0=200000.0001016002 +y_0=7");
    add_proj4text (p, 2,
		   "50000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Massachusetts Mainland (ftUS)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spa");
    add_srs_wkt (p, 2,
		 "tial_Reference_System_2007\",SPHEROID[\"GRS 1980\",63781");
    add_srs_wkt (p, 3,
		 "37,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 5,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 6,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.30480");
    add_srs_wkt (p, 8,
		 "06096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"L");
    add_srs_wkt (p, 9,
		 "ambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_1\",42.68333333333333],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 11,
		 "_2\",41.71666666666667],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 12,
		 ",41],PARAMETER[\"central_meridian\",-71.5],PARAMETER[\"f");
    add_srs_wkt (p, 13,
		 "alse_easting\",656166.667],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 14,
		 "2460625],AUTHORITY[\"EPSG\",\"3586\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 15, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3587, "epsg", 3587,
		      "NAD83(NSRS2007) / Michigan Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.7 +lat_2=44.18333333333333 +lat_0=43");
    add_proj4text (p, 1,
		   ".31666666666667 +lon_0=-84.36666666666666 +x_0=6000000 +");
    add_proj4text (p, 2,
		   "y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Michigan Central\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",45.7],PARAMETER[\"standard_para");
    add_srs_wkt (p, 10,
		 "llel_2\",44.18333333333333],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 11,
		 "in\",43.31666666666667],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 12,
		 "84.36666666666666],PARAMETER[\"false_easting\",6000000],");
    add_srs_wkt (p, 13,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"358");
    add_srs_wkt (p, 14, "7\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3588, "epsg", 3588,
		      "NAD83(NSRS2007) / Michigan Central (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.7 +lat_2=44.18333333333333 +lat_0=43");
    add_proj4text (p, 1,
		   ".31666666666667 +lon_0=-84.36666666666666 +x_0=5999999.9");
    add_proj4text (p, 2,
		   "99976001 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +uni");
    add_proj4text (p, 3, "ts=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Michigan Central (ft)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9002\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"standard_parallel_1\",45.7],PARAMETER[\"stan");
    add_srs_wkt (p, 10,
		 "dard_parallel_2\",44.18333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 11,
		 "e_of_origin\",43.31666666666667],PARAMETER[\"central_mer");
    add_srs_wkt (p, 12,
		 "idian\",-84.36666666666666],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 13,
		 "19685039.37],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 14, "EPSG\",\"3588\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3589, "epsg", 3589,
		      "NAD83(NSRS2007) / Michigan North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.08333333333334 +lat_2=45.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=44.78333333333333 +lon_0=-87 +x_0=8000000 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_def");
    add_proj4text (p, 3, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Michigan North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",47.08333333333334],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",45.48333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",44.78333333333333],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-87],PARAMETER[\"false_easting\",8000000],PARAM");
    add_srs_wkt (p, 13,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3589\"],");
    add_srs_wkt (p, 14, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3590, "epsg", 3590,
		      "NAD83(NSRS2007) / Michigan North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.08333333333334 +lat_2=45.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=44.78333333333333 +lon_0=-87 +x_0=7999999.999");
    add_proj4text (p, 2,
		   "968001 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units");
    add_proj4text (p, 3, "=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Michigan North (ft)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9002\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_1\",47.08333333333334],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"standard_parallel_2\",45.48333333333333],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"latitude_of_origin\",44.78333333333333],PARAMETER[\"c");
    add_srs_wkt (p, 12,
		 "entral_meridian\",-87],PARAMETER[\"false_easting\",26246");
    add_srs_wkt (p, 13,
		 "719.16],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 14, "\",\"3590\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3591, "epsg", 3591,
		      "NAD83(NSRS2007) / Michigan Oblique Mercator");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=45.30916666666666 +lonc=-86 +alpha=33");
    add_proj4text (p, 1,
		   "7.25556 +k=0.9996 +x_0=2546731.496 +y_0=-4354009.816 +el");
    add_proj4text (p, 2, "lps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Michigan Oblique Mercator\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial");
    add_srs_wkt (p, 2,
		 "_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 3,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"latitude_of_center\",45.30916666666666],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"longitude_of_center\",-86],PARAMETER[\"azimuth\",337");
    add_srs_wkt (p, 11,
		 ".25556],PARAMETER[\"rectified_grid_angle\",337.25556],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 13,
		 "ng\",2546731.496],PARAMETER[\"false_northing\",-4354009.");
    add_srs_wkt (p, 14,
		 "816],AUTHORITY[\"EPSG\",\"3591\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 15, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3592, "epsg", 3592,
		      "NAD83(NSRS2007) / Michigan South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.66666666666666 +lat_2=42.1 +lat_0=41");
    add_proj4text (p, 1,
		   ".5 +lon_0=-84.36666666666666 +x_0=4000000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Michigan South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",43.66666666666666],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",42.1],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",41.5],PARAMETER[\"central_meridian\",-84.36666666666666");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",4000000],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"3592\"],AXIS[\"X\",E");
    add_srs_wkt (p, 14, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3593, "epsg", 3593,
		      "NAD83(NSRS2007) / Michigan South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.66666666666666 +lat_2=42.1 +lat_0=41");
    add_proj4text (p, 1,
		   ".5 +lon_0=-84.36666666666666 +x_0=3999999.999984 +y_0=0 ");
    add_proj4text (p, 2,
		   "+ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Michigan South (ft)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9002\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_1\",43.66666666666666],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"standard_parallel_2\",42.1],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 11,
		 "f_origin\",41.5],PARAMETER[\"central_meridian\",-84.3666");
    add_srs_wkt (p, 12,
		 "6666666666],PARAMETER[\"false_easting\",13123359.58],PAR");
    add_srs_wkt (p, 13,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3593\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3594, "epsg", 3594,
		      "NAD83(NSRS2007) / Minnesota Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000 +y_0=100000 +ellps=GRS80 +to");
    add_proj4text (p, 2, "wgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota Central\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",47.05],PARAMETER[\"standard_par");
    add_srs_wkt (p, 10,
		 "allel_2\",45.61666666666667],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 11,
		 "gin\",45],PARAMETER[\"central_meridian\",-94.25],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",800000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",100000],AUTHORITY[\"EPSG\",\"3594\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 14, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3595, "epsg", 3595,
		      "NAD83(NSRS2007) / Minnesota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000 +y");
    add_proj4text (p, 2,
		   "_0=100000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota North\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_1\",48.63333333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "tandard_parallel_2\",47.03333333333333],PARAMETER[\"lati");
    add_srs_wkt (p, 11,
		 "tude_of_origin\",46.5],PARAMETER[\"central_meridian\",-9");
    add_srs_wkt (p, 12,
		 "3.1],PARAMETER[\"false_easting\",800000],PARAMETER[\"fal");
    add_srs_wkt (p, 13,
		 "se_northing\",100000],AUTHORITY[\"EPSG\",\"3595\"],AXIS[");
    add_srs_wkt (p, 14, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3596, "epsg", 3596,
		      "NAD83(NSRS2007) / Minnesota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000 +y_0=100000 +ellps=");
    add_proj4text (p, 2, "GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota South\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_1\",45.21666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "tandard_parallel_2\",43.78333333333333],PARAMETER[\"lati");
    add_srs_wkt (p, 11,
		 "tude_of_origin\",43],PARAMETER[\"central_meridian\",-94]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_easting\",800000],PARAMETER[\"false_n");
    add_srs_wkt (p, 13,
		 "orthing\",100000],AUTHORITY[\"EPSG\",\"3596\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3597, "epsg", 3597,
		      "NAD83(NSRS2007) / Mississippi East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-88.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=300000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Mississippi East\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",29.5],PARAMETER[\"central_meridian\",-88");
    add_srs_wkt (p, 10,
		 ".83333333333333],PARAMETER[\"scale_factor\",0.99995],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",300000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3597\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 13, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3598, "epsg", 3598,
		      "NAD83(NSRS2007) / Mississippi East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-88.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +towgs84");
    add_proj4text (p, 2, "=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Mississippi East (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 8,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transver");
    add_srs_wkt (p, 9,
		 "se_Mercator\"],PARAMETER[\"latitude_of_origin\",29.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"central_meridian\",-88.83333333333333],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"scale_factor\",0.99995],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",984250.0000000002],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 13,
		 "ORITY[\"EPSG\",\"3598\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 3599, "epsg", 3599,
		      "NAD83(NSRS2007) / Mississippi West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-90.33333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=700000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Mississippi West\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",29.5],PARAMETER[\"central_meridian\",-90");
    add_srs_wkt (p, 10,
		 ".33333333333333],PARAMETER[\"scale_factor\",0.99995],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",700000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3599\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 13, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3600, "epsg", 3600,
		      "NAD83(NSRS2007) / Mississippi West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-90.33333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=699999.9998983998 +y_0=0 +ellps=GRS80 +towgs84");
    add_proj4text (p, 2, "=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Mississippi West (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 8,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transver");
    add_srs_wkt (p, 9,
		 "se_Mercator\"],PARAMETER[\"latitude_of_origin\",29.5],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"central_meridian\",-90.33333333333333],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"scale_factor\",0.99995],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2296583.333],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"3600\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3601, "epsg", 3601,
		      "NAD83(NSRS2007) / Missouri Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-92.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=500000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,");
    add_proj4text (p, 2, "0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Missouri Central\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",35.83333333333334],PARAMETER[\"central_m");
    add_srs_wkt (p, 10,
		 "eridian\",-92.5],PARAMETER[\"scale_factor\",0.999933333]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3601\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3602, "epsg", 3602,
		      "NAD83(NSRS2007) / Missouri East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-90.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=250000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,");
    add_proj4text (p, 2, "0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Missouri East\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",35.83333333333334],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",-90.5],PARAMETER[\"scale_factor\",0.999933333],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",250000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3602\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3603, "epsg", 3603,
		      "NAD83(NSRS2007) / Missouri West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.16666666666666 +lon_0=-94.5 +k=0.9");
    add_proj4text (p, 1,
		   "99941177 +x_0=850000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,");
    add_proj4text (p, 2, "0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Missouri West\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",36.16666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",-94.5],PARAMETER[\"scale_factor\",0.999941177],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",850000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3603\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3604, "epsg", 3604,
		      "NAD83(NSRS2007) / Montana");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=45 +lat_0=44.25 +lon_0=-109.5");
    add_proj4text (p, 1,
		   " +x_0=600000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Montana\",GEOGCS[\"NAD83(NSRS");
    add_srs_wkt (p, 1,
		 "2007)\",DATUM[\"NAD83_National_Spatial_Reference_System_");
    add_srs_wkt (p, 2,
		 "2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4759\"]");
    add_srs_wkt (p, 7,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_1\",49],PARAMETER[\"standard_parallel_2\",45]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"latitude_of_origin\",44.25],PARAMETER[\"cen");
    add_srs_wkt (p, 11,
		 "tral_meridian\",-109.5],PARAMETER[\"false_easting\",6000");
    add_srs_wkt (p, 12,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3604\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3605, "epsg", 3605,
		      "NAD83(NSRS2007) / Montana (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=45 +lat_0=44.25 +lon_0=-109.5");
    add_proj4text (p, 1,
		   " +x_0=599999.9999976 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,");
    add_proj4text (p, 2, "0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Montana (ft)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"9002\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",49],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 10,
		 "el_2\",45],PARAMETER[\"latitude_of_origin\",44.25],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"central_meridian\",-109.5],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",1968503.937],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"3605\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 14, "H]]");
    p = add_epsg_def (first, last, 3606, "epsg", 3606,
		      "NAD83(NSRS2007) / Nebraska");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000 +y_0=0 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 2, "0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nebraska\",GEOGCS[\"NAD83(NSR");
    add_srs_wkt (p, 1,
		 "S2007)\",DATUM[\"NAD83_National_Spatial_Reference_System");
    add_srs_wkt (p, 2,
		 "_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4759\"");
    add_srs_wkt (p, 7,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",43],PARAMETER[\"standard_parallel_2\",40");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"latitude_of_origin\",39.83333333333334],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"central_meridian\",-100],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"3606\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3607, "epsg", 3607,
		      "NAD83(NSRS2007) / Nevada Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-116.6666666666667 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=500000 +y_0=6000000 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 2, "0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nevada Central\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",34.75],PARAMETER[\"central_meridian\",-116.");
    add_srs_wkt (p, 10,
		 "6666666666667],PARAMETER[\"scale_factor\",0.9999],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",6000000],AUTHORITY[\"EPSG\",\"3607\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3608, "epsg", 3608,
		      "NAD83(NSRS2007) / Nevada Central (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-116.6666666666667 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=500000.00001016 +y_0=6000000 +ellps=GRS80 +tow");
    add_proj4text (p, 2, "gs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nevada Central (ft US)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transvers");
    add_srs_wkt (p, 9,
		 "e_Mercator\"],PARAMETER[\"latitude_of_origin\",34.75],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"central_meridian\",-116.6666666666667],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "1640416.6667],PARAMETER[\"false_northing\",19685000],AUT");
    add_srs_wkt (p, 13,
		 "HORITY[\"EPSG\",\"3608\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 3609, "epsg", 3609,
		      "NAD83(NSRS2007) / Nevada East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-115.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=200000 +y_0=8000000 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 2, "0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nevada East\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",34.75],PARAMETER[\"central_meridian\",-115.58");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",0.9999],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_easting\",200000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",8000000],AUTHORITY[\"EPSG\",\"3609\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3610, "epsg", 3610,
		      "NAD83(NSRS2007) / Nevada East (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-115.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=200000.00001016 +y_0=8000000.000010163 +ellps=");
    add_proj4text (p, 2, "GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nevada East (ft US)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",34.75],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"central_meridian\",-115.5833333333333],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",65");
    add_srs_wkt (p, 12,
		 "6166.6667],PARAMETER[\"false_northing\",26246666.6667000");
    add_srs_wkt (p, 13,
		 "1],AUTHORITY[\"EPSG\",\"3610\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3611, "epsg", 3611,
		      "NAD83(NSRS2007) / Nevada West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-118.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=800000 +y_0=4000000 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 2, "0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nevada West\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",34.75],PARAMETER[\"central_meridian\",-118.58");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",0.9999],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_easting\",800000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",4000000],AUTHORITY[\"EPSG\",\"3611\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3612, "epsg", 3612,
		      "NAD83(NSRS2007) / Nevada West (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-118.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=800000.0000101599 +y_0=3999999.99998984 +ellps");
    add_proj4text (p, 2, "=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nevada West (ft US)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",34.75],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"central_meridian\",-118.5833333333333],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",26");
    add_srs_wkt (p, 12,
		 "24666.6667],PARAMETER[\"false_northing\",13123333.3333],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"3612\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 3613, "epsg", 3613,
		      "NAD83(NSRS2007) / New Hampshire");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-71.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=300000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,");
    add_proj4text (p, 2, "0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Hampshire\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",42.5],PARAMETER[\"central_meridian\",-71.66");
    add_srs_wkt (p, 10,
		 "666666666667],PARAMETER[\"scale_factor\",0.999966667],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",300000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3613\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3614, "epsg", 3614,
		      "NAD83(NSRS2007) / New Hampshire (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-71.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +tow");
    add_proj4text (p, 2, "gs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Hampshire (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 9,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",42.5],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"central_meridian\",-71.66666666666667],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"scale_factor\",0.999966667],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",984250.0000000002],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 13,
		 "THORITY[\"EPSG\",\"3614\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 14, "ORTH]]");
    p = add_epsg_def (first, last, 3615, "epsg", 3615,
		      "NAD83(NSRS2007) / New Jersey");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Jersey\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",38.83333333333334],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",-74.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",150000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3615\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3616, "epsg", 3616,
		      "NAD83(NSRS2007) / New Jersey (ft US)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Jersey (ft US)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 9,
		 "cator\"],PARAMETER[\"latitude_of_origin\",38.83333333333");
    add_srs_wkt (p, 10,
		 "334],PARAMETER[\"central_meridian\",-74.5],PARAMETER[\"s");
    add_srs_wkt (p, 11,
		 "cale_factor\",0.9999],PARAMETER[\"false_easting\",492125");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 13, "616\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3617, "epsg", 3617,
		      "NAD83(NSRS2007) / New Mexico Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-106.25 +k=0.9999 +x_0=5000");
    add_proj4text (p, 1,
		   "00 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Mexico Central\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",31],PARAMETER[\"central_meridian\",-106.");
    add_srs_wkt (p, 10,
		 "25],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"3617\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 3618, "epsg", 3618,
		      "NAD83(NSRS2007) / New Mexico Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-106.25 +k=0.9999 +x_0=5000");
    add_proj4text (p, 1,
		   "00.0001016001 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Mexico Central (ftUS)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial");
    add_srs_wkt (p, 2,
		 "_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 3,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609");
    add_srs_wkt (p, 8,
		 "6012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Trans");
    add_srs_wkt (p, 9,
		 "verse_Mercator\"],PARAMETER[\"latitude_of_origin\",31],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"central_meridian\",-106.25],PARAMETER[\"scale");
    add_srs_wkt (p, 11,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",1640416.66");
    add_srs_wkt (p, 12,
		 "7],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3618\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3619, "epsg", 3619,
		      "NAD83(NSRS2007) / New Mexico East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-104.3333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "909091 +x_0=165000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Mexico East\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",31],PARAMETER[\"central_meridian\",-104.3");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"scale_factor\",0.999909091],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",165000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3619\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3620, "epsg", 3620,
		      "NAD83(NSRS2007) / New Mexico East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-104.3333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "909091 +x_0=165000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Mexico East (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transvers");
    add_srs_wkt (p, 9,
		 "e_Mercator\"],PARAMETER[\"latitude_of_origin\",31],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"central_meridian\",-104.3333333333333],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.999909091],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",541337.5],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 13, "PSG\",\"3620\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3621, "epsg", 3621,
		      "NAD83(NSRS2007) / New Mexico West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-107.8333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "916667 +x_0=830000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Mexico West\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 9,
		 "de_of_origin\",31],PARAMETER[\"central_meridian\",-107.8");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"scale_factor\",0.999916667],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",830000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3621\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3622, "epsg", 3622,
		      "NAD83(NSRS2007) / New Mexico West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-107.8333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "916667 +x_0=830000.0001016001 +y_0=0 +ellps=GRS80 +towgs");
    add_proj4text (p, 2, "84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New Mexico West (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transvers");
    add_srs_wkt (p, 9,
		 "e_Mercator\"],PARAMETER[\"latitude_of_origin\",31],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"central_meridian\",-107.8333333333333],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.999916667],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2723091.667],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"3622\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3623, "epsg", 3623,
		      "NAD83(NSRS2007) / New York Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-76.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=250000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York Central\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",40],PARAMETER[\"central_meridian\",-76.5");
    add_srs_wkt (p, 10,
		 "8333333333333],PARAMETER[\"scale_factor\",0.9999375],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",250000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",0],AUTHORITY[\"EPSG\",\"3623\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 13, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3624, "epsg", 3624,
		      "NAD83(NSRS2007) / New York Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-76.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=249999.9998983998 +y_0=0 +ellps=GRS80 +towgs84");
    add_proj4text (p, 2, "=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York Central (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 8,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transver");
    add_srs_wkt (p, 9,
		 "se_Mercator\"],PARAMETER[\"latitude_of_origin\",40],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"central_meridian\",-76.58333333333333],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"scale_factor\",0.9999375],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",820208.3330000002],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 13,
		 "ORITY[\"EPSG\",\"3624\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 3625, "epsg", 3625,
		      "NAD83(NSRS2007) / New York East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York East\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",38.83333333333334],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",-74.5],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",150000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3625\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3626, "epsg", 3626,
		      "NAD83(NSRS2007) / New York East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York East (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 9,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",38.8333333");
    add_srs_wkt (p, 10,
		 "3333334],PARAMETER[\"central_meridian\",-74.5],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",49");
    add_srs_wkt (p, 12,
		 "2125],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"3626\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3627, "epsg", 3627,
		      "NAD83(NSRS2007) / New York Long Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.03333333333333 +lat_2=40.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=40.16666666666666 +lon_0=-74 +x_0=300000 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York Long Island\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_1\",41.03333333333333],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"standard_parallel_2\",40.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",40.16666666666666],PARAMETER[\"cent");
    add_srs_wkt (p, 12,
		 "ral_meridian\",-74],PARAMETER[\"false_easting\",300000],");
    add_srs_wkt (p, 13,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"362");
    add_srs_wkt (p, 14, "7\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3628, "epsg", 3628,
		      "NAD83(NSRS2007) / New York Long Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.03333333333333 +lat_2=40.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=40.16666666666666 +lon_0=-74 +x_0=300000.0000");
    add_proj4text (p, 2,
		   "000001 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units");
    add_proj4text (p, 3, "=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York Long Island (ftUS)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spati");
    add_srs_wkt (p, 2,
		 "al_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006");
    add_srs_wkt (p, 8,
		 "096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lam");
    add_srs_wkt (p, 9,
		 "bert_Conformal_Conic_2SP\"],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 10,
		 "l_1\",41.03333333333333],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 11,
		 "\",40.66666666666666],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 12,
		 "0.16666666666666],PARAMETER[\"central_meridian\",-74],PA");
    add_srs_wkt (p, 13,
		 "RAMETER[\"false_easting\",984250.0000000002],PARAMETER[\"");
    add_srs_wkt (p, 14,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3628\"],AXIS[\"");
    add_srs_wkt (p, 15, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3629, "epsg", 3629,
		      "NAD83(NSRS2007) / New York West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-78.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=350000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York West\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",40],PARAMETER[\"central_meridian\",-78.5833");
    add_srs_wkt (p, 10,
		 "3333333333],PARAMETER[\"scale_factor\",0.9999375],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",350000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",0],AUTHORITY[\"EPSG\",\"3629\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3630, "epsg", 3630,
		      "NAD83(NSRS2007) / New York West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-78.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=350000.0001016001 +y_0=0 +ellps=GRS80 +towgs84");
    add_proj4text (p, 2, "=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / New York West (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 9,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",40],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"central_meridian\",-78.58333333333333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "scale_factor\",0.9999375],PARAMETER[\"false_easting\",11");
    add_srs_wkt (p, 12,
		 "48291.667],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 13, "PSG\",\"3630\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3631, "epsg", 3631,
		      "NAD83(NSRS2007) / North Carolina");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=33.75 +lon_0=-79 +x_0=609601.22 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / North Carolina\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",36.16666666666666],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",34.33333333333334],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",33.75],PARAMETER[\"central_meridian\",-79");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",609601.22],PARAMETER[\"fal");
    add_srs_wkt (p, 13,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"3631\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3632, "epsg", 3632,
		      "NAD83(NSRS2007) / North Carolina (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=33.75 +lon_0=-79 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2,
		   "0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / North Carolina (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "36.16666666666666],PARAMETER[\"standard_parallel_2\",34.");
    add_srs_wkt (p, 11,
		 "33333333333334],PARAMETER[\"latitude_of_origin\",33.75],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"central_meridian\",-79],PARAMETER[\"false_ea");
    add_srs_wkt (p, 13,
		 "sting\",2000000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 14,
		 "TY[\"EPSG\",\"3632\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 15, "]");
    p = add_epsg_def (first, last, 3633, "epsg", 3633,
		      "NAD83(NSRS2007) / North Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.43333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=47 +lon_0=-100.5 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / North Dakota North\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",48.73333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",47.43333333333333],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",47],PARAMETER[\"central_meridian\",-10");
    add_srs_wkt (p, 12,
		 "0.5],PARAMETER[\"false_easting\",600000],PARAMETER[\"fal");
    add_srs_wkt (p, 13,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"3633\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3634, "epsg", 3634,
		      "NAD83(NSRS2007) / North Dakota North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.43333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=47 +lon_0=-100.5 +x_0=599999.9999976 +y_0=0 +");
    add_proj4text (p, 2,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / North Dakota North (ft)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9002\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"standard_parallel_1\",48.73333333333333],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"standard_parallel_2\",47.43333333333333],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"latitude_of_origin\",47],PARAMETER[\"central_merid");
    add_srs_wkt (p, 12,
		 "ian\",-100.5],PARAMETER[\"false_easting\",1968503.937],P");
    add_srs_wkt (p, 13,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3634");
    add_srs_wkt (p, 14, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3635, "epsg", 3635,
		      "NAD83(NSRS2007) / North Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.48333333333333 +lat_2=46.18333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=45.66666666666666 +lon_0=-100.5 +x_0=600000 +");
    add_proj4text (p, 2,
		   "y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / North Dakota South\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",47.48333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",46.18333333333333],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",45.66666666666666],PARAMETER[\"central");
    add_srs_wkt (p, 12,
		 "_meridian\",-100.5],PARAMETER[\"false_easting\",600000],");
    add_srs_wkt (p, 13,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"363");
    add_srs_wkt (p, 14, "5\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3636, "epsg", 3636,
		      "NAD83(NSRS2007) / North Dakota South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.48333333333333 +lat_2=46.18333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=45.66666666666666 +lon_0=-100.5 +x_0=599999.9");
    add_proj4text (p, 2,
		   "999976 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units");
    add_proj4text (p, 3, "=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / North Dakota South (ft)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9002\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"standard_parallel_1\",47.48333333333333],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"standard_parallel_2\",46.18333333333333],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"latitude_of_origin\",45.66666666666666],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"central_meridian\",-100.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 13,
		 ",1968503.937],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 14,
		 "\"EPSG\",\"3636\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3637, "epsg", 3637,
		      "NAD83(NSRS2007) / Ohio North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.7 +lat_2=40.43333333333333 +lat_0=39");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Ohio North\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",41.7],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 10,
		 "\",40.43333333333333],PARAMETER[\"latitude_of_origin\",3");
    add_srs_wkt (p, 11,
		 "9.66666666666666],PARAMETER[\"central_meridian\",-82.5],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_easting\",600000],PARAMETER[\"false_no");
    add_srs_wkt (p, 13,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"3637\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 14, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3638, "epsg", 3638,
		      "NAD83(NSRS2007) / Ohio South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.03333333333333 +lat_2=38.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=38 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Ohio South\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",40.03333333333333],PARAMETER[\"standa");
    add_srs_wkt (p, 10,
		 "rd_parallel_2\",38.73333333333333],PARAMETER[\"latitude_");
    add_srs_wkt (p, 11,
		 "of_origin\",38],PARAMETER[\"central_meridian\",-82.5],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_easting\",600000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 13,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3638\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 14, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3639, "epsg", 3639,
		      "NAD83(NSRS2007) / Oklahoma North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.76666666666667 +lat_2=35.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=35 +lon_0=-98 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oklahoma North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",36.76666666666667],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",35.56666666666667],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",35],PARAMETER[\"central_meridian\",-98],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_easting\",600000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 13,
		 "thing\",0],AUTHORITY[\"EPSG\",\"3639\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 14, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3640, "epsg", 3640,
		      "NAD83(NSRS2007) / Oklahoma North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.76666666666667 +lat_2=35.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=35 +lon_0=-98 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oklahoma North (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "36.76666666666667],PARAMETER[\"standard_parallel_2\",35.");
    add_srs_wkt (p, 11,
		 "56666666666667],PARAMETER[\"latitude_of_origin\",35],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"central_meridian\",-98],PARAMETER[\"false_easti");
    add_srs_wkt (p, 13,
		 "ng\",1968500],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 14,
		 "\"EPSG\",\"3640\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3641, "epsg", 3641,
		      "NAD83(NSRS2007) / Oklahoma South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.23333333333333 +lat_2=33.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.33333333333334 +lon_0=-98 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oklahoma South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",35.23333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",33.93333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",33.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-98],PARAMETER[\"false_easting\",600000],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3641\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3642, "epsg", 3642,
		      "NAD83(NSRS2007) / Oklahoma South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.23333333333333 +lat_2=33.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.33333333333334 +lon_0=-98 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_");
    add_proj4text (p, 3, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oklahoma South (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "35.23333333333333],PARAMETER[\"standard_parallel_2\",33.");
    add_srs_wkt (p, 11,
		 "93333333333333],PARAMETER[\"latitude_of_origin\",33.3333");
    add_srs_wkt (p, 12,
		 "3333333334],PARAMETER[\"central_meridian\",-98],PARAMETE");
    add_srs_wkt (p, 13,
		 "R[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 14,
		 ",0],AUTHORITY[\"EPSG\",\"3642\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 15, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3643, "epsg", 3643,
		      "NAD83(NSRS2007) / Oregon Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=45.5 +lat_0=41.75 +lon_0=-120");
    add_proj4text (p, 1,
		   ".5 +x_0=400000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,");
    add_proj4text (p, 2, "0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oregon Lambert\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",43],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "2\",45.5],PARAMETER[\"latitude_of_origin\",41.75],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-120.5],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",400000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"3643\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3644, "epsg", 3644,
		      "NAD83(NSRS2007) / Oregon Lambert (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=45.5 +lat_0=41.75 +lon_0=-120");
    add_proj4text (p, 1,
		   ".5 +x_0=399999.9999984 +y_0=0 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 2, "0,0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oregon Lambert (ft)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9002\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_1\",43],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",45.5],PARAMETER[\"latitude_of_origin\",41.7");
    add_srs_wkt (p, 11,
		 "5],PARAMETER[\"central_meridian\",-120.5],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",1312335.958],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "0],AUTHORITY[\"EPSG\",\"3644\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3645, "epsg", 3645,
		      "NAD83(NSRS2007) / Oregon North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46 +lat_2=44.33333333333334 +lat_0=43.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=2500000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oregon North\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",46],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 10,
		 "\",44.33333333333334],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 11,
		 "3.66666666666666],PARAMETER[\"central_meridian\",-120.5]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_easting\",2500000],PARAMETER[\"false_");
    add_srs_wkt (p, 13,
		 "northing\",0],AUTHORITY[\"EPSG\",\"3645\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 14, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3646, "epsg", 3646,
		      "NAD83(NSRS2007) / Oregon North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46 +lat_2=44.33333333333334 +lat_0=43.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=2500000.0001424 +y_0=0 ");
    add_proj4text (p, 2,
		   "+ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oregon North (ft)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "02\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_1\",46],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 10,
		 "rallel_2\",44.33333333333334],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 11,
		 "igin\",43.66666666666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 12,
		 ",-120.5],PARAMETER[\"false_easting\",8202099.738],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3646\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3647, "epsg", 3647,
		      "NAD83(NSRS2007) / Oregon South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44 +lat_2=42.33333333333334 +lat_0=41.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=1500000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oregon South\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",44],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 10,
		 "\",42.33333333333334],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 11,
		 "1.66666666666666],PARAMETER[\"central_meridian\",-120.5]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_easting\",1500000],PARAMETER[\"false_");
    add_srs_wkt (p, 13,
		 "northing\",0],AUTHORITY[\"EPSG\",\"3647\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 14, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3648, "epsg", 3648,
		      "NAD83(NSRS2007) / Oregon South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44 +lat_2=42.33333333333334 +lat_0=41.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=1500000.0001464 +y_0=0 ");
    add_proj4text (p, 2,
		   "+ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Oregon South (ft)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "02\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_1\",44],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 10,
		 "rallel_2\",42.33333333333334],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 11,
		 "igin\",41.66666666666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 12,
		 ",-120.5],PARAMETER[\"false_easting\",4921259.843],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3648\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3649, "epsg", 3649,
		      "NAD83(NSRS2007) / Pennsylvania North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.95 +lat_2=40.88333333333333 +lat_0=4");
    add_proj4text (p, 1,
		   "0.16666666666666 +lon_0=-77.75 +x_0=600000 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Pennsylvania North\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",41.95],PARAMETER[\"standard_par");
    add_srs_wkt (p, 10,
		 "allel_2\",40.88333333333333],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 11,
		 "gin\",40.16666666666666],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 12,
		 "-77.75],PARAMETER[\"false_easting\",600000],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3649\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3650, "epsg", 3650,
		      "NAD83(NSRS2007) / Pennsylvania North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.95 +lat_2=40.88333333333333 +lat_0=4");
    add_proj4text (p, 1,
		   "0.16666666666666 +lon_0=-77.75 +x_0=600000 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Pennsylvania North (ftUS)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial");
    add_srs_wkt (p, 2,
		 "_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 3,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609");
    add_srs_wkt (p, 8,
		 "6012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambe");
    add_srs_wkt (p, 9,
		 "rt_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "1\",41.95],PARAMETER[\"standard_parallel_2\",40.88333333");
    add_srs_wkt (p, 11,
		 "333333],PARAMETER[\"latitude_of_origin\",40.166666666666");
    add_srs_wkt (p, 12,
		 "66],PARAMETER[\"central_meridian\",-77.75],PARAMETER[\"f");
    add_srs_wkt (p, 13,
		 "alse_easting\",1968500],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 14,
		 "AUTHORITY[\"EPSG\",\"3650\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 15, ",NORTH]]");
    p = add_epsg_def (first, last, 3651, "epsg", 3651,
		      "NAD83(NSRS2007) / Pennsylvania South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.96666666666667 +lat_2=39.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=39.33333333333334 +lon_0=-77.75 +x_0=600000 +");
    add_proj4text (p, 2,
		   "y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Pennsylvania South\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",40.96666666666667],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",39.93333333333333],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",39.33333333333334],PARAMETER[\"central");
    add_srs_wkt (p, 12,
		 "_meridian\",-77.75],PARAMETER[\"false_easting\",600000],");
    add_srs_wkt (p, 13,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"365");
    add_srs_wkt (p, 14, "1\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3652, "epsg", 3652,
		      "NAD83(NSRS2007) / Pennsylvania South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.96666666666667 +lat_2=39.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=39.33333333333334 +lon_0=-77.75 +x_0=600000 +");
    add_proj4text (p, 2,
		   "y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Pennsylvania South (ftUS)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial");
    add_srs_wkt (p, 2,
		 "_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 3,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609");
    add_srs_wkt (p, 8,
		 "6012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambe");
    add_srs_wkt (p, 9,
		 "rt_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "1\",40.96666666666667],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 11,
		 ",39.93333333333333],PARAMETER[\"latitude_of_origin\",39.");
    add_srs_wkt (p, 12,
		 "33333333333334],PARAMETER[\"central_meridian\",-77.75],P");
    add_srs_wkt (p, 13,
		 "ARAMETER[\"false_easting\",1968500],PARAMETER[\"false_no");
    add_srs_wkt (p, 14,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"3652\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 15, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3653, "epsg", 3653,
		      "NAD83(NSRS2007) / Rhode Island");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.08333333333334 +lon_0=-71.5 +k=0.9");
    add_proj4text (p, 1,
		   "9999375 +x_0=100000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0");
    add_proj4text (p, 2, ",0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Rhode Island\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",41.08333333333334],PARAMETER[\"central_merid");
    add_srs_wkt (p, 10,
		 "ian\",-71.5],PARAMETER[\"scale_factor\",0.99999375],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",100000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3653\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3654, "epsg", 3654,
		      "NAD83(NSRS2007) / Rhode Island (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.08333333333334 +lon_0=-71.5 +k=0.9");
    add_proj4text (p, 1,
		   "9999375 +x_0=99999.99998983997 +y_0=0 +ellps=GRS80 +towg");
    add_proj4text (p, 2, "s84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Rhode Island (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",41.08333333");
    add_srs_wkt (p, 10,
		 "333334],PARAMETER[\"central_meridian\",-71.5],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.99999375],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",328083.3333],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"3654\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3655, "epsg", 3655,
		      "NAD83(NSRS2007) / South Carolina");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.83333333333334 +lat_2=32.5 +lat_0=31");
    add_proj4text (p, 1,
		   ".83333333333333 +lon_0=-81 +x_0=609600 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / South Carolina\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",34.83333333333334],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",32.5],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",31.83333333333333],PARAMETER[\"central_meridian\",-81],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_easting\",609600],PARAMETER[\"false_no");
    add_srs_wkt (p, 13,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"3655\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 14, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3656, "epsg", 3656,
		      "NAD83(NSRS2007) / South Carolina (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.83333333333334 +lat_2=32.5 +lat_0=31");
    add_proj4text (p, 1,
		   ".83333333333333 +lon_0=-81 +x_0=609600 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / South Carolina (ft)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9002\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_1\",34.83333333333334],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"standard_parallel_2\",32.5],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 11,
		 "f_origin\",31.83333333333333],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 12,
		 "an\",-81],PARAMETER[\"false_easting\",2000000],PARAMETER");
    add_srs_wkt (p, 13,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3656\"],AXIS");
    add_srs_wkt (p, 14, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3657, "epsg", 3657,
		      "NAD83(NSRS2007) / South Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.68333333333333 +lat_2=44.41666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=43.83333333333334 +lon_0=-100 +x_0=600000 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_def");
    add_proj4text (p, 3, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / South Dakota North\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",45.68333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",44.41666666666666],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",43.83333333333334],PARAMETER[\"central");
    add_srs_wkt (p, 12,
		 "_meridian\",-100],PARAMETER[\"false_easting\",600000],PA");
    add_srs_wkt (p, 13,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3657\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3658, "epsg", 3658,
		      "NAD83(NSRS2007) / South Dakota North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.68333333333333 +lat_2=44.41666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=43.83333333333334 +lon_0=-100 +x_0=600000 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no");
    add_proj4text (p, 3, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / South Dakota North (ftUS)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial");
    add_srs_wkt (p, 2,
		 "_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 3,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609");
    add_srs_wkt (p, 8,
		 "6012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambe");
    add_srs_wkt (p, 9,
		 "rt_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "1\",45.68333333333333],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 11,
		 ",44.41666666666666],PARAMETER[\"latitude_of_origin\",43.");
    add_srs_wkt (p, 12,
		 "83333333333334],PARAMETER[\"central_meridian\",-100],PAR");
    add_srs_wkt (p, 13,
		 "AMETER[\"false_easting\",1968500],PARAMETER[\"false_nort");
    add_srs_wkt (p, 14,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3658\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 15, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3659, "epsg", 3659,
		      "NAD83(NSRS2007) / South Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.4 +lat_2=42.83333333333334 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_de");
    add_proj4text (p, 3, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / South Dakota South\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",44.4],PARAMETER[\"standard_para");
    add_srs_wkt (p, 10,
		 "llel_2\",42.83333333333334],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 11,
		 "in\",42.33333333333334],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 12,
		 "100.3333333333333],PARAMETER[\"false_easting\",600000],P");
    add_srs_wkt (p, 13,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3659");
    add_srs_wkt (p, 14, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3660, "epsg", 3660,
		      "NAD83(NSRS2007) / South Dakota South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.4 +lat_2=42.83333333333334 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / South Dakota South (ftUS)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial");
    add_srs_wkt (p, 2,
		 "_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 3,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 5,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609");
    add_srs_wkt (p, 8,
		 "6012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambe");
    add_srs_wkt (p, 9,
		 "rt_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "1\",44.4],PARAMETER[\"standard_parallel_2\",42.833333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"latitude_of_origin\",42.3333333333333");
    add_srs_wkt (p, 12,
		 "4],PARAMETER[\"central_meridian\",-100.3333333333333],PA");
    add_srs_wkt (p, 13,
		 "RAMETER[\"false_easting\",1968500],PARAMETER[\"false_nor");
    add_srs_wkt (p, 14,
		 "thing\",0],AUTHORITY[\"EPSG\",\"3660\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 15, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3661, "epsg", 3661,
		      "NAD83(NSRS2007) / Tennessee");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.41666666666666 +lat_2=35.25 +lat_0=3");
    add_proj4text (p, 1,
		   "4.33333333333334 +lon_0=-86 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Tennessee\",GEOGCS[\"NAD83(NS");
    add_srs_wkt (p, 1,
		 "RS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syste");
    add_srs_wkt (p, 2,
		 "m_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4759");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_1\",36.41666666666666],PARAMETER[\"standar");
    add_srs_wkt (p, 10,
		 "d_parallel_2\",35.25],PARAMETER[\"latitude_of_origin\",3");
    add_srs_wkt (p, 11,
		 "4.33333333333334],PARAMETER[\"central_meridian\",-86],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_easting\",600000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 13,
		 "hing\",0],AUTHORITY[\"EPSG\",\"3661\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 14, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3662, "epsg", 3662,
		      "NAD83(NSRS2007) / Tennessee (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.41666666666666 +lat_2=35.25 +lat_0=3");
    add_proj4text (p, 1,
		   "4.33333333333334 +lon_0=-86 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Tennessee (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.41");
    add_srs_wkt (p, 10,
		 "666666666666],PARAMETER[\"standard_parallel_2\",35.25],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"latitude_of_origin\",34.33333333333334],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"central_meridian\",-86],PARAMETER[\"false_easting");
    add_srs_wkt (p, 13,
		 "\",1968500],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 14, "EPSG\",\"3662\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3663, "epsg", 3663,
		      "NAD83(NSRS2007) / Texas Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=31.88333333333333 +lat_2=30.11666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=29.66666666666667 +lon_0=-100.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=700000 +y_0=3000000 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 3, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas Central\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_S");
    add_srs_wkt (p, 2,
		 "ystem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",31.88333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",30.11666666666667],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",29.66666666666667],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-100.3333333333333],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 13,
		 ",700000],PARAMETER[\"false_northing\",3000000],AUTHORITY");
    add_srs_wkt (p, 14,
		 "[\"EPSG\",\"3663\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3664, "epsg", 3664,
		      "NAD83(NSRS2007) / Texas Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=31.88333333333333 +lat_2=30.11666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=29.66666666666667 +lon_0=-100.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=699999.9998983998 +y_0=3000000 +ellps=GRS80 +towgs84");
    add_proj4text (p, 3, "=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas Central (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 8,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 9,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 10,
		 "1.88333333333333],PARAMETER[\"standard_parallel_2\",30.1");
    add_srs_wkt (p, 11,
		 "1666666666667],PARAMETER[\"latitude_of_origin\",29.66666");
    add_srs_wkt (p, 12,
		 "666666667],PARAMETER[\"central_meridian\",-100.333333333");
    add_srs_wkt (p, 13,
		 "3333],PARAMETER[\"false_easting\",2296583.333],PARAMETER");
    add_srs_wkt (p, 14,
		 "[\"false_northing\",9842500.000000002],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 15, ",\"3664\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3665, "epsg", 3665,
		      "NAD83(NSRS2007) / Texas Centric Albers Equal Area");
    add_proj4text (p, 0,
		   "+proj=aea +lat_1=27.5 +lat_2=35 +lat_0=18 +lon_0=-100 +x");
    add_proj4text (p, 1,
		   "_0=1500000 +y_0=6000000 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas Centric Albers Equal Ar");
    add_srs_wkt (p, 1,
		 "ea\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_S");
    add_srs_wkt (p, 2,
		 "patial_Reference_System_2007\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 3,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84");
    add_srs_wkt (p, 4,
		 "[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"G");
    add_srs_wkt (p, 5,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 6,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 8,
		 "EPSG\",\"9001\"]],PROJECTION[\"Albers_Conic_Equal_Area\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"standard_parallel_1\",27.5],PARAMETER[\"st");
    add_srs_wkt (p, 10,
		 "andard_parallel_2\",35],PARAMETER[\"latitude_of_center\"");
    add_srs_wkt (p, 11,
		 ",18],PARAMETER[\"longitude_of_center\",-100],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",1500000],PARAMETER[\"false_northing\",60");
    add_srs_wkt (p, 13,
		 "00000],AUTHORITY[\"EPSG\",\"3665\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3666, "epsg", 3666,
		      "NAD83(NSRS2007) / Texas Centric Lambert Conformal");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.5 +lat_2=35 +lat_0=18 +lon_0=-100 +x");
    add_proj4text (p, 1,
		   "_0=1500000 +y_0=5000000 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas Centric Lambert Conform");
    add_srs_wkt (p, 1,
		 "al\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_S");
    add_srs_wkt (p, 2,
		 "patial_Reference_System_2007\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 3,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84");
    add_srs_wkt (p, 4,
		 "[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"G");
    add_srs_wkt (p, 5,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 6,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 8,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 9,
		 "SP\"],PARAMETER[\"standard_parallel_1\",27.5],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"standard_parallel_2\",35],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 11,
		 "in\",18],PARAMETER[\"central_meridian\",-100],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",1500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "5000000],AUTHORITY[\"EPSG\",\"3666\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 14, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3667, "epsg", 3667,
		      "NAD83(NSRS2007) / Texas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.18333333333333 +lat_2=34.65 +lat_0=3");
    add_proj4text (p, 1,
		   "4 +lon_0=-101.5 +x_0=200000 +y_0=1000000 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas North\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",36.18333333333333],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",34.65],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",34],PARAMETER[\"central_meridian\",-101.5],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",200000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 13,
		 "0000],AUTHORITY[\"EPSG\",\"3667\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 14, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3668, "epsg", 3668,
		      "NAD83(NSRS2007) / Texas North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.18333333333333 +lat_2=34.65 +lat_0=3");
    add_proj4text (p, 1,
		   "4 +lon_0=-101.5 +x_0=200000.0001016002 +y_0=999999.99989");
    add_proj4text (p, 2,
		   "83998 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.18");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"standard_parallel_2\",34.65],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"latitude_of_origin\",34],PARAMETER[\"central_");
    add_srs_wkt (p, 12,
		 "meridian\",-101.5],PARAMETER[\"false_easting\",656166.66");
    add_srs_wkt (p, 13,
		 "7],PARAMETER[\"false_northing\",3280833.333],AUTHORITY[\"");
    add_srs_wkt (p, 14, "EPSG\",\"3668\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3669, "epsg", 3669,
		      "NAD83(NSRS2007) / Texas North Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.96666666666667 +lat_2=32.13333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=31.66666666666667 +lon_0=-98.5 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=2000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas North Central\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_1\",33.96666666666667],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"standard_parallel_2\",32.13333333333333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",31.66666666666667],PARAMETER[\"cent");
    add_srs_wkt (p, 12,
		 "ral_meridian\",-98.5],PARAMETER[\"false_easting\",600000");
    add_srs_wkt (p, 13,
		 "],PARAMETER[\"false_northing\",2000000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 14, "\",\"3669\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3670, "epsg", 3670,
		      "NAD83(NSRS2007) / Texas North Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.96666666666667 +lat_2=32.13333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=31.66666666666667 +lon_0=-98.5 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=2000000.0001016 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 3, "units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas North Central (ftUS)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatia");
    add_srs_wkt (p, 2,
		 "l_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 3,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060");
    add_srs_wkt (p, 8,
		 "96012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 9,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 10,
		 "_1\",33.96666666666667],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 11,
		 ",32.13333333333333],PARAMETER[\"latitude_of_origin\",31.");
    add_srs_wkt (p, 12,
		 "66666666666667],PARAMETER[\"central_meridian\",-98.5],PA");
    add_srs_wkt (p, 13,
		 "RAMETER[\"false_easting\",1968500],PARAMETER[\"false_nor");
    add_srs_wkt (p, 14,
		 "thing\",6561666.667],AUTHORITY[\"EPSG\",\"3670\"],AXIS[\"");
    add_srs_wkt (p, 15, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3671, "epsg", 3671,
		      "NAD83(NSRS2007) / Texas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.66666666666667 +lon_0=-98.5 +x_0=300000 +y");
    add_proj4text (p, 2,
		   "_0=5000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas South\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",27.83333333333333],PARAMETER[\"stand");
    add_srs_wkt (p, 10,
		 "ard_parallel_2\",26.16666666666667],PARAMETER[\"latitude");
    add_srs_wkt (p, 11,
		 "_of_origin\",25.66666666666667],PARAMETER[\"central_meri");
    add_srs_wkt (p, 12,
		 "dian\",-98.5],PARAMETER[\"false_easting\",300000],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",5000000],AUTHORITY[\"EPSG\",\"367");
    add_srs_wkt (p, 14, "1\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3672, "epsg", 3672,
		      "NAD83(NSRS2007) / Texas South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.66666666666667 +lon_0=-98.5 +x_0=300000.00");
    add_proj4text (p, 2,
		   "00000001 +y_0=5000000.0001016 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 3, "0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",27.83");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"standard_parallel_2\",26.16666");
    add_srs_wkt (p, 11,
		 "666666667],PARAMETER[\"latitude_of_origin\",25.666666666");
    add_srs_wkt (p, 12,
		 "66667],PARAMETER[\"central_meridian\",-98.5],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_easting\",984250.0000000002],PARAMETER[\"false_nor");
    add_srs_wkt (p, 14,
		 "thing\",16404166.667],AUTHORITY[\"EPSG\",\"3672\"],AXIS[");
    add_srs_wkt (p, 15, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3673, "epsg", 3673,
		      "NAD83(NSRS2007) / Texas South Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.28333333333333 +lat_2=28.38333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=27.83333333333333 +lon_0=-99 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=4000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas South Central\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_1\",30.28333333333333],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"standard_parallel_2\",28.38333333333333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",27.83333333333333],PARAMETER[\"cent");
    add_srs_wkt (p, 12,
		 "ral_meridian\",-99],PARAMETER[\"false_easting\",600000],");
    add_srs_wkt (p, 13,
		 "PARAMETER[\"false_northing\",4000000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"3673\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3674, "epsg", 3674,
		      "NAD83(NSRS2007) / Texas South Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.28333333333333 +lat_2=28.38333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=27.83333333333333 +lon_0=-99 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=3999999.9998984 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +un");
    add_proj4text (p, 3, "its=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Texas South Central (ftUS)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatia");
    add_srs_wkt (p, 2,
		 "l_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 3,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060");
    add_srs_wkt (p, 8,
		 "96012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 9,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 10,
		 "_1\",30.28333333333333],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 11,
		 ",28.38333333333333],PARAMETER[\"latitude_of_origin\",27.");
    add_srs_wkt (p, 12,
		 "83333333333333],PARAMETER[\"central_meridian\",-99],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_easting\",1968500],PARAMETER[\"false_north");
    add_srs_wkt (p, 14,
		 "ing\",13123333.333],AUTHORITY[\"EPSG\",\"3674\"],AXIS[\"");
    add_srs_wkt (p, 15, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3675, "epsg", 3675,
		      "NAD83(NSRS2007) / Utah Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000 +y_0=2000000 ");
    add_proj4text (p, 2,
		   "+ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah Central\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_1\",40.65],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 10,
		 "l_2\",39.01666666666667],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",38.33333333333334],PARAMETER[\"central_meridian\",-111.");
    add_srs_wkt (p, 12,
		 "5],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",2000000],AUTHORITY[\"EPSG\",\"3675\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3676, "epsg", 3676,
		      "NAD83(NSRS2007) / Utah Central (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000.0001504 +y_0=");
    add_proj4text (p, 2,
		   "1999999.999992 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 3, "s=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah Central (ft)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "02\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"standard_parallel_1\",40.65],PARAMETER[\"standard");
    add_srs_wkt (p, 10,
		 "_parallel_2\",39.01666666666667],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 11,
		 "_origin\",38.33333333333334],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 12,
		 "n\",-111.5],PARAMETER[\"false_easting\",1640419.948],PAR");
    add_srs_wkt (p, 13,
		 "AMETER[\"false_northing\",6561679.79],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"3676\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3677, "epsg", 3677,
		      "NAD83(NSRS2007) / Utah Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000.00001016 +y_0");
    add_proj4text (p, 2,
		   "=2000000.00001016 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 3, "nits=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah Central (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Con");
    add_srs_wkt (p, 9,
		 "formal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40");
    add_srs_wkt (p, 10,
		 ".65],PARAMETER[\"standard_parallel_2\",39.01666666666667");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"latitude_of_origin\",38.33333333333334],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"central_meridian\",-111.5],PARAMETER[\"false_e");
    add_srs_wkt (p, 13,
		 "asting\",1640416.6667],PARAMETER[\"false_northing\",6561");
    add_srs_wkt (p, 14,
		 "666.666700001],AUTHORITY[\"EPSG\",\"3677\"],AXIS[\"X\",E");
    add_srs_wkt (p, 15, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3678, "epsg", 3678,
		      "NAD83(NSRS2007) / Utah North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000 +");
    add_proj4text (p, 2,
		   "y_0=1000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah North\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",41.78333333333333],PARAMETER[\"standa");
    add_srs_wkt (p, 10,
		 "rd_parallel_2\",40.71666666666667],PARAMETER[\"latitude_");
    add_srs_wkt (p, 11,
		 "of_origin\",40.33333333333334],PARAMETER[\"central_merid");
    add_srs_wkt (p, 12,
		 "ian\",-111.5],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"367");
    add_srs_wkt (p, 14, "8\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3679, "epsg", 3679,
		      "NAD83(NSRS2007) / Utah North (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001504 +y_0=999999.9999960001 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 3, "0,0,0,0,0 +units=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah North (ft)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "2\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_1\",41.78333333333333],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"standard_parallel_2\",40.71666666666667],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",40.33333333333334],PARAMETER[\"cent");
    add_srs_wkt (p, 12,
		 "ral_meridian\",-111.5],PARAMETER[\"false_easting\",16404");
    add_srs_wkt (p, 13,
		 "19.948],PARAMETER[\"false_northing\",3280839.895],AUTHOR");
    add_srs_wkt (p, 14,
		 "ITY[\"EPSG\",\"3679\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 15, "]]");
    p = add_epsg_def (first, last, 3680, "epsg", 3680,
		      "NAD83(NSRS2007) / Utah North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "0001016 +y_0=999999.9999898402 +ellps=GRS80 +towgs84=0,0");
    add_proj4text (p, 3, ",0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.78");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"standard_parallel_2\",40.71666");
    add_srs_wkt (p, 11,
		 "666666667],PARAMETER[\"latitude_of_origin\",40.333333333");
    add_srs_wkt (p, 12,
		 "33334],PARAMETER[\"central_meridian\",-111.5],PARAMETER[");
    add_srs_wkt (p, 13,
		 "\"false_easting\",1640416.6667],PARAMETER[\"false_northi");
    add_srs_wkt (p, 14,
		 "ng\",3280833.333300001],AUTHORITY[\"EPSG\",\"3680\"],AXI");
    add_srs_wkt (p, 15, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3681, "epsg", 3681,
		      "NAD83(NSRS2007) / Utah South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000 +y_0=3000000 ");
    add_proj4text (p, 2,
		   "+ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah South\",GEOGCS[\"NAD83(N");
    add_srs_wkt (p, 1,
		 "SRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Syst");
    add_srs_wkt (p, 2,
		 "em_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"475");
    add_srs_wkt (p, 7,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_1\",38.35],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 10,
		 "2\",37.21666666666667],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 11,
		 "36.66666666666666],PARAMETER[\"central_meridian\",-111.5");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 13,
		 "northing\",3000000],AUTHORITY[\"EPSG\",\"3681\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3682, "epsg", 3682,
		      "NAD83(NSRS2007) / Utah South (ft)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000.0001504 +y_0=");
    add_proj4text (p, 2,
		   "2999999.999988 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 3, "s=ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah South (ft)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"foot\",0.3048,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "2\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_1\",38.35],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",37.21666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",36.66666666666666],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-111.5],PARAMETER[\"false_easting\",1640419.948],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",9842519.685],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"3682\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3683, "epsg", 3683,
		      "NAD83(NSRS2007) / Utah South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000.00001016 +y_0");
    add_proj4text (p, 2,
		   "=3000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Utah South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.35");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"standard_parallel_2\",37.21666666666667],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"latitude_of_origin\",36.66666666666666],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"central_meridian\",-111.5],PARAMETER[\"false_east");
    add_srs_wkt (p, 13,
		 "ing\",1640416.6667],PARAMETER[\"false_northing\",9842500");
    add_srs_wkt (p, 14,
		 ".000000002],AUTHORITY[\"EPSG\",\"3683\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 15, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3684, "epsg", 3684,
		      "NAD83(NSRS2007) / Vermont");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-72.5 +k=0.999964286 +x_0");
    add_proj4text (p, 1,
		   "=500000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Vermont\",GEOGCS[\"NAD83(NSRS");
    add_srs_wkt (p, 1,
		 "2007)\",DATUM[\"NAD83_National_Spatial_Reference_System_");
    add_srs_wkt (p, 2,
		 "2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4759\"]");
    add_srs_wkt (p, 7,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",42.5],PARAMETER[\"central_meridian\",-72.5],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",0.999964286],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"3684\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3685, "epsg", 3685,
		      "NAD83(NSRS2007) / Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.2 +lat_2=38.03333333333333 +lat_0=37");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-78.5 +x_0=3500000 +y_0=2000000 +");
    add_proj4text (p, 2,
		   "ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Virginia North\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",39.2],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 10,
		 "l_2\",38.03333333333333],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",37.66666666666666],PARAMETER[\"central_meridian\",-78.5");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",3500000],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",2000000],AUTHORITY[\"EPSG\",\"3685\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3686, "epsg", 3686,
		      "NAD83(NSRS2007) / Virginia North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.2 +lat_2=38.03333333333333 +lat_0=37");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-78.5 +x_0=3500000.0001016 +y_0=2");
    add_proj4text (p, 2,
		   "000000.0001016 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 3, "s=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Virginia North (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "39.2],PARAMETER[\"standard_parallel_2\",38.0333333333333");
    add_srs_wkt (p, 11,
		 "3],PARAMETER[\"latitude_of_origin\",37.66666666666666],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"central_meridian\",-78.5],PARAMETER[\"false_e");
    add_srs_wkt (p, 13,
		 "asting\",11482916.667],PARAMETER[\"false_northing\",6561");
    add_srs_wkt (p, 14,
		 "666.667],AUTHORITY[\"EPSG\",\"3686\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 15, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3687, "epsg", 3687,
		      "NAD83(NSRS2007) / Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=36.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.33333333333334 +lon_0=-78.5 +x_0=3500000 +");
    add_proj4text (p, 2,
		   "y_0=1000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Virginia South\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",37.96666666666667],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",36.76666666666667],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",36.33333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 12,
		 "ridian\",-78.5],PARAMETER[\"false_easting\",3500000],PAR");
    add_srs_wkt (p, 13,
		 "AMETER[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 14, "3687\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3688, "epsg", 3688,
		      "NAD83(NSRS2007) / Virginia South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=36.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.33333333333334 +lon_0=-78.5 +x_0=3500000.0");
    add_proj4text (p, 2,
		   "001016 +y_0=999999.9998983998 +ellps=GRS80 +towgs84=0,0,");
    add_proj4text (p, 3, "0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Virginia South (ftUS)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Ref");
    add_srs_wkt (p, 2,
		 "erence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "37.96666666666667],PARAMETER[\"standard_parallel_2\",36.");
    add_srs_wkt (p, 11,
		 "76666666666667],PARAMETER[\"latitude_of_origin\",36.3333");
    add_srs_wkt (p, 12,
		 "3333333334],PARAMETER[\"central_meridian\",-78.5],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_easting\",11482916.667],PARAMETER[\"false_no");
    add_srs_wkt (p, 14,
		 "rthing\",3280833.333],AUTHORITY[\"EPSG\",\"3688\"],AXIS[");
    add_srs_wkt (p, 15, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3689, "epsg", 3689,
		      "NAD83(NSRS2007) / Washington North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.5 +lat_0=47");
    add_proj4text (p, 1,
		   " +lon_0=-120.8333333333333 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Washington North\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",48.73333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",47.5],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 11,
		 "in\",47],PARAMETER[\"central_meridian\",-120.83333333333");
    add_srs_wkt (p, 12,
		 "33],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 13,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3689\"],AXIS[\"X\",");
    add_srs_wkt (p, 14, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3690, "epsg", 3690,
		      "NAD83(NSRS2007) / Washington North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.5 +lat_0=47");
    add_proj4text (p, 1,
		   " +lon_0=-120.8333333333333 +x_0=500000.0001016001 +y_0=0");
    add_proj4text (p, 2,
		   " +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_de");
    add_proj4text (p, 3, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Washington North (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 8,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 9,
		 "Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 10,
		 ",48.73333333333333],PARAMETER[\"standard_parallel_2\",47");
    add_srs_wkt (p, 11,
		 ".5],PARAMETER[\"latitude_of_origin\",47],PARAMETER[\"cen");
    add_srs_wkt (p, 12,
		 "tral_meridian\",-120.8333333333333],PARAMETER[\"false_ea");
    add_srs_wkt (p, 13,
		 "sting\",1640416.667],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 14,
		 "HORITY[\"EPSG\",\"3690\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 15, "RTH]]");
    p = add_epsg_def (first, last, 3691, "epsg", 3691,
		      "NAD83(NSRS2007) / Washington South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.33333333333334 +lat_2=45.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=45.33333333333334 +lon_0=-120.5 +x_0=500000 +");
    add_proj4text (p, 2,
		   "y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Washington South\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",47.33333333333334],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "standard_parallel_2\",45.83333333333334],PARAMETER[\"lat");
    add_srs_wkt (p, 11,
		 "itude_of_origin\",45.33333333333334],PARAMETER[\"central");
    add_srs_wkt (p, 12,
		 "_meridian\",-120.5],PARAMETER[\"false_easting\",500000],");
    add_srs_wkt (p, 13,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"369");
    add_srs_wkt (p, 14, "1\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3692, "epsg", 3692,
		      "NAD83(NSRS2007) / Washington South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.33333333333334 +lat_2=45.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=45.33333333333334 +lon_0=-120.5 +x_0=500000.0");
    add_proj4text (p, 2,
		   "001016001 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +un");
    add_proj4text (p, 3, "its=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Washington South (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_R");
    add_srs_wkt (p, 2,
		 "eference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 8,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 9,
		 "Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 10,
		 ",47.33333333333334],PARAMETER[\"standard_parallel_2\",45");
    add_srs_wkt (p, 11,
		 ".83333333333334],PARAMETER[\"latitude_of_origin\",45.333");
    add_srs_wkt (p, 12,
		 "33333333334],PARAMETER[\"central_meridian\",-120.5],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_easting\",1640416.667],PARAMETER[\"false_n");
    add_srs_wkt (p, 14,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3692\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 15, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3693, "epsg", 3693,
		      "NAD83(NSRS2007) / West Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=600000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / West Virginia North\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_1\",40.25],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",39],PARAMETER[\"latitude_of_origin\",38.5],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"central_meridian\",-79.5],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "easting\",600000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 13,
		 "ITY[\"EPSG\",\"3693\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 3694, "epsg", 3694,
		      "NAD83(NSRS2007) / West Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / West Virginia South\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_1\",38.88333333333333],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"standard_parallel_2\",37.48333333333333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",37],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 12,
		 "-81],PARAMETER[\"false_easting\",600000],PARAMETER[\"fal");
    add_srs_wkt (p, 13,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"3694\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3695, "epsg", 3695,
		      "NAD83(NSRS2007) / Wisconsin Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.5 +lat_2=44.25 +lat_0=43.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wisconsin Central\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",45.5],PARAMETER[\"standard_para");
    add_srs_wkt (p, 10,
		 "llel_2\",44.25],PARAMETER[\"latitude_of_origin\",43.8333");
    add_srs_wkt (p, 11,
		 "3333333334],PARAMETER[\"central_meridian\",-90],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",600000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"3695\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3696, "epsg", 3696,
		      "NAD83(NSRS2007) / Wisconsin Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.5 +lat_2=44.25 +lat_0=43.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wisconsin Central (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",45.5],PARAMETER[\"standard_parallel_2\",44.25],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"latitude_of_origin\",43.83333333333334],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"central_meridian\",-90],PARAMETER[\"false_easting\",19");
    add_srs_wkt (p, 13,
		 "68500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 14, ",\"3696\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3697, "epsg", 3697,
		      "NAD83(NSRS2007) / Wisconsin North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.76666666666667 +lat_2=45.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=45.16666666666666 +lon_0=-90 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wisconsin North\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_1\",46.76666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "tandard_parallel_2\",45.56666666666667],PARAMETER[\"lati");
    add_srs_wkt (p, 11,
		 "tude_of_origin\",45.16666666666666],PARAMETER[\"central_");
    add_srs_wkt (p, 12,
		 "meridian\",-90],PARAMETER[\"false_easting\",600000],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3697\"]");
    add_srs_wkt (p, 14, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3698, "epsg", 3698,
		      "NAD83(NSRS2007) / Wisconsin North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.76666666666667 +lat_2=45.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=45.16666666666666 +lon_0=-90 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_");
    add_proj4text (p, 3, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wisconsin North (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "46.76666666666667],PARAMETER[\"standard_parallel_2\",45.");
    add_srs_wkt (p, 11,
		 "56666666666667],PARAMETER[\"latitude_of_origin\",45.1666");
    add_srs_wkt (p, 12,
		 "6666666666],PARAMETER[\"central_meridian\",-90],PARAMETE");
    add_srs_wkt (p, 13,
		 "R[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 14,
		 ",0],AUTHORITY[\"EPSG\",\"3698\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 15, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3699, "epsg", 3699,
		      "NAD83(NSRS2007) / Wisconsin South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.06666666666667 +lat_2=42.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=42 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wisconsin South\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 8,
		 "],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_1\",44.06666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "tandard_parallel_2\",42.73333333333333],PARAMETER[\"lati");
    add_srs_wkt (p, 11,
		 "tude_of_origin\",42],PARAMETER[\"central_meridian\",-90]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_easting\",600000],PARAMETER[\"false_n");
    add_srs_wkt (p, 13,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3699\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 14, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3700, "epsg", 3700,
		      "NAD83(NSRS2007) / Wisconsin South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.06666666666667 +lat_2=42.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=42 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wisconsin South (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "44.06666666666667],PARAMETER[\"standard_parallel_2\",42.");
    add_srs_wkt (p, 11,
		 "73333333333333],PARAMETER[\"latitude_of_origin\",42],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"central_meridian\",-90],PARAMETER[\"false_easti");
    add_srs_wkt (p, 13,
		 "ng\",1968500],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 14,
		 "\"EPSG\",\"3700\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3701, "epsg", 3701,
		      "NAD83(NSRS2007) / Wisconsin Transverse Mercator");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-90 +k=0.9996 +x_0=520000 +y");
    add_proj4text (p, 1,
		   "_0=-4480000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wisconsin Transverse Mercator");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spa");
    add_srs_wkt (p, 2,
		 "tial_Reference_System_2007\",SPHEROID[\"GRS 1980\",63781");
    add_srs_wkt (p, 3,
		 "37,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 5,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 6,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 8,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",-90],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",520000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 12,
		 "-4480000],AUTHORITY[\"EPSG\",\"3701\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3702, "epsg", 3702,
		      "NAD83(NSRS2007) / Wyoming East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-105.1666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=200000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 2, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming East\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",40.5],PARAMETER[\"central_meridian\",-105.16");
    add_srs_wkt (p, 10,
		 "66666666667],PARAMETER[\"scale_factor\",0.9999375],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",200000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"3702\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3703, "epsg", 3703,
		      "NAD83(NSRS2007) / Wyoming East Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-107.3333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=400000 +y_0=100000 +ellps=GRS80 +towgs84=0,0");
    add_proj4text (p, 2, ",0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming East Central\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",40.5],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-107.3333333333333],PARAMETER[\"scale_factor\",0.999937");
    add_srs_wkt (p, 11,
		 "5],PARAMETER[\"false_easting\",400000],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_northing\",100000],AUTHORITY[\"EPSG\",\"3703\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3704, "epsg", 3704,
		      "NAD83(NSRS2007) / Wyoming West Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-108.75 +k=0.9999375 +x_0");
    add_proj4text (p, 1,
		   "=600000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming West Central\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refe");
    add_srs_wkt (p, 2,
		 "rence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 3,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",40.5],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-108.75],PARAMETER[\"scale_factor\",0.9999375],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_easting\",600000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3704\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3705, "epsg", 3705,
		      "NAD83(NSRS2007) / Wyoming West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-110.0833333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=800000 +y_0=100000 +ellps=GRS80 +towgs84=0,0");
    add_proj4text (p, 2, ",0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming West\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",40.5],PARAMETER[\"central_meridian\",-110.08");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",0.9999375],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",800000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",100000],AUTHORITY[\"EPSG\",\"3705\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_13 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 3706, "epsg", 3706,
		      "NAD83(NSRS2007) / UTM zone 59N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 59N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",171],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3706\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3707, "epsg", 3707,
		      "NAD83(NSRS2007) / UTM zone 60N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 60N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",177],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3707\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3708, "epsg", 3708,
		      "NAD83(NSRS2007) / UTM zone 1N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 1N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-177],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3708\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3709, "epsg", 3709,
		      "NAD83(NSRS2007) / UTM zone 2N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 2N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-171],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3709\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3710, "epsg", 3710,
		      "NAD83(NSRS2007) / UTM zone 3N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 3N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-165],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3710\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3711, "epsg", 3711,
		      "NAD83(NSRS2007) / UTM zone 4N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 4N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-159],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3711\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3712, "epsg", 3712,
		      "NAD83(NSRS2007) / UTM zone 5N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 5N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-153],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3712\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3713, "epsg", 3713,
		      "NAD83(NSRS2007) / UTM zone 6N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 6N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-147],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3713\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3714, "epsg", 3714,
		      "NAD83(NSRS2007) / UTM zone 7N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 7N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-141],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3714\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3715, "epsg", 3715,
		      "NAD83(NSRS2007) / UTM zone 8N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 8N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-135],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3715\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3716, "epsg", 3716,
		      "NAD83(NSRS2007) / UTM zone 9N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 9N\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sys");
    add_srs_wkt (p, 2,
		 "tem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 7,
		 "59\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",-129],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3716\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3717, "epsg", 3717,
		      "NAD83(NSRS2007) / UTM zone 10N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 10N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-123],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3717\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3718, "epsg", 3718,
		      "NAD83(NSRS2007) / UTM zone 11N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 11N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-117],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3718\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3719, "epsg", 3719,
		      "NAD83(NSRS2007) / UTM zone 12N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 12N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-111],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3719\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3720, "epsg", 3720,
		      "NAD83(NSRS2007) / UTM zone 13N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 13N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-105],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12,
		 "PSG\",\"3720\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3721, "epsg", 3721,
		      "NAD83(NSRS2007) / UTM zone 14N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 14N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-99],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3721\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3722, "epsg", 3722,
		      "NAD83(NSRS2007) / UTM zone 15N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 15N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-93],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3722\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3723, "epsg", 3723,
		      "NAD83(NSRS2007) / UTM zone 16N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 16N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-87],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3723\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3724, "epsg", 3724,
		      "NAD83(NSRS2007) / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 17N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-81],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3724\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3725, "epsg", 3725,
		      "NAD83(NSRS2007) / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 18N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-75],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3725\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3726, "epsg", 3726,
		      "NAD83(NSRS2007) / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / UTM zone 19N\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",-69],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12,
		 "G\",\"3726\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3727, "epsg", 3727,
		      "Reunion 1947 / TM Reunion");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-21.11666666666667 +lon_0=55.53333333");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=160000 +y_0=50000 +ellps=intl +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Reunion 1947 / TM Reunion\",GEOGCS[\"Reunion 19");
    add_srs_wkt (p, 1,
		 "47\",DATUM[\"Reunion_1947\",SPHEROID[\"International 192");
    add_srs_wkt (p, 2,
		 "4\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6626\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4626\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",-21.11666666666667],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "55.53333333333333],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",160000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",50000],AUTHORITY[\"EPSG\",\"3727\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3728, "epsg", 3728,
		      "NAD83(NSRS2007) / Ohio North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.7 +lat_2=40.43333333333333 +lat_0=39");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Ohio North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.7]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"standard_parallel_2\",40.43333333333333],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"latitude_of_origin\",39.66666666666666],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"central_meridian\",-82.5],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 13,
		 "g\",1968500],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 14, "EPSG\",\"3728\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3729, "epsg", 3729,
		      "NAD83(NSRS2007) / Ohio South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.03333333333333 +lat_2=38.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=38 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Ohio South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 9,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.03");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"standard_parallel_2\",38.73333");
    add_srs_wkt (p, 11,
		 "333333333],PARAMETER[\"latitude_of_origin\",38],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"central_meridian\",-82.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 13,
		 ",1968500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 14, "SG\",\"3729\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3730, "epsg", 3730,
		      "NAD83(NSRS2007) / Wyoming East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-105.1666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=200000.00001016 +y_0=0 +ellps=GRS80 +towgs84");
    add_proj4text (p, 2, "=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming East (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",40.5],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"central_meridian\",-105.1666666666667],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.9999375],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "656166.6667],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13, "EPSG\",\"3730\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3731, "epsg", 3731,
		      "NAD83(NSRS2007) / Wyoming East Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-107.3333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=399999.99998984 +y_0=99999.99998983997 +ellp");
    add_proj4text (p, 2,
		   "s=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming East Central (ftUS)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spati");
    add_srs_wkt (p, 2,
		 "al_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006");
    add_srs_wkt (p, 8,
		 "096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Tra");
    add_srs_wkt (p, 9,
		 "nsverse_Mercator\"],PARAMETER[\"latitude_of_origin\",40.");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"central_meridian\",-107.3333333333333],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"scale_factor\",0.9999375],PARAMETER[\"false_ea");
    add_srs_wkt (p, 12,
		 "sting\",1312333.3333],PARAMETER[\"false_northing\",32808");
    add_srs_wkt (p, 13,
		 "3.3333],AUTHORITY[\"EPSG\",\"3731\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 14, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3732, "epsg", 3732,
		      "NAD83(NSRS2007) / Wyoming West Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-108.75 +k=0.9999375 +x_0");
    add_proj4text (p, 1,
		   "=600000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 2, "s=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming West Central (ftUS)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spati");
    add_srs_wkt (p, 2,
		 "al_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006");
    add_srs_wkt (p, 8,
		 "096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Tra");
    add_srs_wkt (p, 9,
		 "nsverse_Mercator\"],PARAMETER[\"latitude_of_origin\",40.");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"central_meridian\",-108.75],PARAMETER[\"s");
    add_srs_wkt (p, 11,
		 "cale_factor\",0.9999375],PARAMETER[\"false_easting\",196");
    add_srs_wkt (p, 12,
		 "8500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"3732\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3733, "epsg", 3733,
		      "NAD83(NSRS2007) / Wyoming West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-110.0833333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=800000.0000101599 +y_0=99999.99998983997 +el");
    add_proj4text (p, 2,
		   "lps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Wyoming West (ftUS)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4759\"]],UNIT[\"US survey foot\",0.304800609601219");
    add_srs_wkt (p, 8,
		 "2,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 9,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",40.5],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"central_meridian\",-110.0833333333333],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"scale_factor\",0.9999375],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "2624666.6667],PARAMETER[\"false_northing\",328083.3333],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"3733\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 3734, "epsg", 3734,
		      "NAD83 / Ohio North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.7 +lat_2=40.43333333333333 +lat_0=39");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Ohio North (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",41.7],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"standard_parallel_2\",40.43333333333333],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"latitude_of_origin\",39.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-82.5],PARAMETER[\"false_easting\",19");
    add_srs_wkt (p, 12,
		 "68500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"3734\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3735, "epsg", 3735,
		      "NAD83 / Ohio South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.03333333333333 +lat_2=38.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=38 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Ohio South (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",40.0333333");
    add_srs_wkt (p, 9,
		 "3333333],PARAMETER[\"standard_parallel_2\",38.7333333333");
    add_srs_wkt (p, 10,
		 "3333],PARAMETER[\"latitude_of_origin\",38],PARAMETER[\"c");
    add_srs_wkt (p, 11,
		 "entral_meridian\",-82.5],PARAMETER[\"false_easting\",196");
    add_srs_wkt (p, 12,
		 "8500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"3735\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3736, "epsg", 3736,
		      "NAD83 / Wyoming East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-105.1666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=200000.00001016 +y_0=0 +ellps=GRS80 +datum=N");
    add_proj4text (p, 2, "AD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming East (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",40.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-105.1666666666667],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999375],PARAMETER[\"false_easting\",656166.");
    add_srs_wkt (p, 11,
		 "6667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"3736\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3737, "epsg", 3737,
		      "NAD83 / Wyoming East Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-107.3333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=399999.99998984 +y_0=99999.99998983997 +ellp");
    add_proj4text (p, 2, "s=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming East Central (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transvers");
    add_srs_wkt (p, 8,
		 "e_Mercator\"],PARAMETER[\"latitude_of_origin\",40.5],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",-107.3333333333333],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9999375],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",1312333.3333],PARAMETER[\"false_northing\",328083.3333]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"3737\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 3738, "epsg", 3738,
		      "NAD83 / Wyoming West Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-108.75 +k=0.9999375 +x_0");
    add_proj4text (p, 1,
		   "=600000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming West Central (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transvers");
    add_srs_wkt (p, 8,
		 "e_Mercator\"],PARAMETER[\"latitude_of_origin\",40.5],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",-108.75],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",0.9999375],PARAMETER[\"false_easting\",1968500],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"373");
    add_srs_wkt (p, 12, "8\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3739, "epsg", 3739,
		      "NAD83 / Wyoming West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-110.0833333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=800000.0000101599 +y_0=99999.99998983997 +el");
    add_proj4text (p, 2, "lps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming West (ftUS)\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercato");
    add_srs_wkt (p, 8,
		 "r\"],PARAMETER[\"latitude_of_origin\",40.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-110.0833333333333],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9999375],PARAMETER[\"false_easting\",2624666");
    add_srs_wkt (p, 11,
		 ".6667],PARAMETER[\"false_northing\",328083.3333],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"3739\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 13, "]");
    p = add_epsg_def (first, last, 3740, "epsg", 3740,
		      "NAD83(HARN) / UTM zone 10N");
    add_proj4text (p, 0, "+proj=utm +zone=10 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 10N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-123],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3740\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3741, "epsg", 3741,
		      "NAD83(HARN) / UTM zone 11N");
    add_proj4text (p, 0, "+proj=utm +zone=11 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 11N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-117],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3741\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3742, "epsg", 3742,
		      "NAD83(HARN) / UTM zone 12N");
    add_proj4text (p, 0, "+proj=utm +zone=12 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 12N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-111],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3742\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3743, "epsg", 3743,
		      "NAD83(HARN) / UTM zone 13N");
    add_proj4text (p, 0, "+proj=utm +zone=13 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 13N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-105],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3743\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3744, "epsg", 3744,
		      "NAD83(HARN) / UTM zone 14N");
    add_proj4text (p, 0, "+proj=utm +zone=14 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 14N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-99],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3744\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3745, "epsg", 3745,
		      "NAD83(HARN) / UTM zone 15N");
    add_proj4text (p, 0, "+proj=utm +zone=15 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 15N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-93],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3745\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3746, "epsg", 3746,
		      "NAD83(HARN) / UTM zone 16N");
    add_proj4text (p, 0, "+proj=utm +zone=16 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 16N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-87],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3746\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3747, "epsg", 3747,
		      "NAD83(HARN) / UTM zone 17N");
    add_proj4text (p, 0, "+proj=utm +zone=17 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 17N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-81],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3747\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3748, "epsg", 3748,
		      "NAD83(HARN) / UTM zone 18N");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 18N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-75],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3748\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3749, "epsg", 3749,
		      "NAD83(HARN) / UTM zone 19N");
    add_proj4text (p, 0, "+proj=utm +zone=19 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 19N\",GEOGCS[\"NAD83(HAR");
    add_srs_wkt (p, 1,
		 "N)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-69],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"3749\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3750, "epsg", 3750,
		      "NAD83(HARN) / UTM zone 4N");
    add_proj4text (p, 0, "+proj=utm +zone=4 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 4N\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-159],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"3750\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3751, "epsg", 3751,
		      "NAD83(HARN) / UTM zone 5N");
    add_proj4text (p, 0, "+proj=utm +zone=5 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / UTM zone 5N\",GEOGCS[\"NAD83(HARN");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-153],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"3751\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3752, "epsg", 3752,
		      "WGS 84 / Mercator 41 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=100 +lat_ts=-41 +x_0=0 +y_0=0 +ellps=W");
    add_proj4text (p, 1, "GS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Mercator 41 (deprecated)\",GEOGCS[\"WG");
    add_srs_wkt (p, 1,
		 "S 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,29");
    add_srs_wkt (p, 2,
		 "8.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Mercator_1SP\"],PARAMETER[\"latitude_of_origin\",-41],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"central_meridian\",100],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"3752\"],AXIS[\"X\",E");
    add_srs_wkt (p, 11, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3753, "epsg", 3753,
		      "NAD83(HARN) / Ohio North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.7 +lat_2=40.43333333333333 +lat_0=39");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Ohio North (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",41.7],PARAMETER[\"standard_parallel_2\",40.43");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"latitude_of_origin\",39.666666");
    add_srs_wkt (p, 11,
		 "66666666],PARAMETER[\"central_meridian\",-82.5],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",1968500],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"3753\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3754, "epsg", 3754,
		      "NAD83(HARN) / Ohio South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.03333333333333 +lat_2=38.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=38 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Ohio South (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",40.03333333333333],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",38.73333333333333],PARAMETER[\"latitude_of_origi");
    add_srs_wkt (p, 11,
		 "n\",38],PARAMETER[\"central_meridian\",-82.5],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",1968500],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "0],AUTHORITY[\"EPSG\",\"3754\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3755, "epsg", 3755,
		      "NAD83(HARN) / Wyoming East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-105.1666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=200000.00001016 +y_0=0 +ellps=GRS80 +units=u");
    add_proj4text (p, 2, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming East (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",40.5],PARAMETER[\"central_meridian\",-105.16666666");
    add_srs_wkt (p, 10,
		 "66667],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",656166.6667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3755\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3756, "epsg", 3756,
		      "NAD83(HARN) / Wyoming East Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-107.3333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=399999.99998984 +y_0=99999.99998983997 +ellp");
    add_proj4text (p, 2, "s=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming East Central (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional");
    add_srs_wkt (p, 2,
		 "_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",40.5],PARAMETER[\"central_meridian\",-107.3");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"scale_factor\",0.9999375],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",1312333.3333],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",328083.3333],AUTHORITY[\"EPSG\",\"3756\"],AXI");
    add_srs_wkt (p, 13, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3757, "epsg", 3757,
		      "NAD83(HARN) / Wyoming West Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-108.75 +k=0.9999375 +x_0");
    add_proj4text (p, 1, "=600000 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming West Central (ftUS)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional");
    add_srs_wkt (p, 2,
		 "_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",40.5],PARAMETER[\"central_meridian\",-108.7");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_easting\",1968500],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 12,
		 "THORITY[\"EPSG\",\"3757\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3758, "epsg", 3758,
		      "NAD83(HARN) / Wyoming West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-110.0833333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=800000.0000101599 +y_0=99999.99998983997 +el");
    add_proj4text (p, 2, "lps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Wyoming West (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",40.5],PARAMETER[\"central_meridian\",-110.08333333");
    add_srs_wkt (p, 10,
		 "33333],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",2624666.6667],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",328083.3333],AUTHORITY[\"EPSG\",\"3758\"],AXIS[\"X\",");
    add_srs_wkt (p, 13, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3759, "epsg", 3759,
		      "NAD83 / Hawaii zone 3 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.16666666666667 +lon_0=-158 +k=0.99");
    add_proj4text (p, 1,
		   "999 +x_0=500000.00001016 +y_0=0 +ellps=GRS80 +datum=NAD8");
    add_proj4text (p, 2, "3 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Hawaii zone 3 (ftUS)\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",21.16666666666667");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-158],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99999],PARAMETER[\"false_easting\",1640416.6");
    add_srs_wkt (p, 11,
		 "667],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"3759\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3760, "epsg", 3760,
		      "NAD83(HARN) / Hawaii zone 3 (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.16666666666667 +lon_0=-158 +k=0.99");
    add_proj4text (p, 1,
		   "999 +x_0=500000.00001016 +y_0=0 +ellps=GRS80 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Hawaii zone 3 (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Networ");
    add_srs_wkt (p, 2,
		 "k\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",21.16666666666667],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-158],PARAMETER[\"scale_factor\",0.99999],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1640416.6667],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"3760\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 3761, "epsg", 3761,
		      "NAD83(CSRS) / UTM zone 22N");
    add_proj4text (p, 0, "+proj=utm +zone=22 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / UTM zone 22N\",GEOGCS[\"NAD83(CSR");
    add_srs_wkt (p, 1,
		 "S)\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-51],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"3761\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3762, "epsg", 3762,
		      "WGS 84 / South Georgia Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-54 +lat_2=-54.75 +lat_0=-55 +lon_0=-37");
    add_proj4text (p, 1,
		   " +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / South Georgia Lambert\",GEOGCS[\"WGS 8");
    add_srs_wkt (p, 1,
		 "4\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.2");
    add_srs_wkt (p, 2,
		 "57223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 7,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 8,
		 "_1\",-54],PARAMETER[\"standard_parallel_2\",-54.75],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"latitude_of_origin\",-55],PARAMETER[\"central_me");
    add_srs_wkt (p, 10,
		 "ridian\",-37],PARAMETER[\"false_easting\",0],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3762\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3763, "epsg", 3763,
		      "ETRS89 / Portugal TM06");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=39.66825833333333 +lon_0=-8.133108333");
    add_proj4text (p, 1,
		   "333334 +k=1 +x_0=0 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Portugal TM06\",GEOGCS[\"ETRS89\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"European_Terrestrial_Reference_System_1989\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",39.66825833333333],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-8.133108333333334],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3763\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3764, "epsg", 3764,
		      "NZGD2000 / Chatham Island Circuit 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-44 +lon_0=-176.5 +k=1 +x_0=400000 +y");
    add_proj4text (p, 1,
		   "_0=800000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Chatham Island Circuit 2000\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NZGD2000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "-44],PARAMETER[\"central_meridian\",-176.5],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",400000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",800000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3764\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3765, "epsg", 3765, "HTRS96 / Croatia TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16.5 +k=0.9999 +x_0=500000 +");
    add_proj4text (p, 1,
		   "y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"HTRS96 / Croatia TM\",GEOGCS[\"HTRS96\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Croatian_Terrestrial_Reference_System\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6761\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4761\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",16.5],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "9],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"3765\"],AXIS[\"Easti");
    add_srs_wkt (p, 12, "ng\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3766, "epsg", 3766, "HTRS96 / Croatia LCC");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.91666666666666 +lat_2=43.08333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=0 +lon_0=16.5 +x_0=0 +y_0=0 +ellps=GRS80 +tow");
    add_proj4text (p, 2, "gs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"HTRS96 / Croatia LCC\",GEOGCS[\"HTRS96\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"Croatian_Terrestrial_Reference_System\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6761\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4761\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conform");
    add_srs_wkt (p, 8,
		 "al_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",45.916");
    add_srs_wkt (p, 9,
		 "66666666666],PARAMETER[\"standard_parallel_2\",43.083333");
    add_srs_wkt (p, 10,
		 "33333334],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"central_meridian\",16.5],PARAMETER[\"false_easting\",0");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 13,
		 "766\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3767, "epsg", 3767, "HTRS96 / UTM zone 33N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"HTRS96 / UTM zone 33N\",GEOGCS[\"HTRS96\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Croatian_Terrestrial_Reference_System\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6761\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4761\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",15],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3767\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3768, "epsg", 3768, "HTRS96 / UTM zone 34N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"HTRS96 / UTM zone 34N\",GEOGCS[\"HTRS96\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Croatian_Terrestrial_Reference_System\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6761\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4761\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",21],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3768\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3769, "epsg", 3769,
		      "Bermuda 1957 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bermuda 1957 / UTM zone 20N\",GEOGCS[\"Bermuda ");
    add_srs_wkt (p, 1,
		 "1957\",DATUM[\"Bermuda_1957\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6216\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4216\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-63],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"3769\"],AXIS[\"Easting\",EAST],AXIS[\"Northin");
    add_srs_wkt (p, 12, "g\",NORTH]]");
    p = add_epsg_def (first, last, 3770, "epsg", 3770,
		      "BDA2000 / Bermuda 2000 National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=32 +lon_0=-64.75 +k=1 +x_0=550000 +y_");
    add_proj4text (p, 1,
		   "0=100000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"BDA2000 / Bermuda 2000 National Grid\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "BDA2000\",DATUM[\"Bermuda_2000\",SPHEROID[\"WGS 84\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6762\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4762\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",32],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",-64.75],PARAMETER[\"scale_factor\",1],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",550000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",100000],AUTHORITY[\"EPSG\",\"3770\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3771, "epsg", 3771,
		      "NAD27 / Alberta 3TM ref merid 111 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-111 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alberta 3TM ref merid 111 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-111],PARAMETER[\"scale_factor\",0.9999],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",0],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"3771\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3772, "epsg", 3772,
		      "NAD27 / Alberta 3TM ref merid 114 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-114 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alberta 3TM ref merid 114 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-114],PARAMETER[\"scale_factor\",0.9999],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",0],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"3772\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3773, "epsg", 3773,
		      "NAD27 / Alberta 3TM ref merid 117 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-117 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alberta 3TM ref merid 117 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-117],PARAMETER[\"scale_factor\",0.9999],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",0],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"3773\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3774, "epsg", 3774,
		      "NAD27 / Alberta 3TM ref merid 120 W (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-120 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alberta 3TM ref merid 120 W (deprecated");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"NAD27\",DATUM[\"North_American_Datum_1927\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Clarke 1866\",6378206.4,294.9786982139006,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4267\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-120],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",0],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"3774\"],AXIS[\"Easting");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3775, "epsg", 3775,
		      "NAD83 / Alberta 3TM ref merid 111 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-111 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alberta 3TM ref merid 111 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "111],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3775\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 3776, "epsg", 3776,
		      "NAD83 / Alberta 3TM ref merid 114 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-114 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alberta 3TM ref merid 114 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "114],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3776\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 3777, "epsg", 3777,
		      "NAD83 / Alberta 3TM ref merid 117 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-117 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alberta 3TM ref merid 117 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "117],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3777\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 3778, "epsg", 3778,
		      "NAD83 / Alberta 3TM ref merid 120 W (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-120 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alberta 3TM ref merid 120 W (deprecated");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-120],PARAMETER[\"scale_factor\",0.9999],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",0],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"3778\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3779, "epsg", 3779,
		      "NAD83(CSRS) / Alberta 3TM ref merid 111 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-111 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Alberta 3TM ref merid 111 W\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"614");
    add_srs_wkt (p, 4,
		 "0\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3779\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3780, "epsg", 3780,
		      "NAD83(CSRS) / Alberta 3TM ref merid 114 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-114 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Alberta 3TM ref merid 114 W\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"614");
    add_srs_wkt (p, 4,
		 "0\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-114],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3780\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3781, "epsg", 3781,
		      "NAD83(CSRS) / Alberta 3TM ref merid 117 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-117 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Alberta 3TM ref merid 117 W\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"614");
    add_srs_wkt (p, 4,
		 "0\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-117],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3781\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3782, "epsg", 3782,
		      "NAD83(CSRS) / Alberta 3TM ref merid 120 W (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-120 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Alberta 3TM ref merid 120 W (depr");
    add_srs_wkt (p, 1,
		 "ecated)\",GEOGCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_");
    add_srs_wkt (p, 2,
		 "Spatial_Reference_System\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",-120],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.9999],PARAMETER[\"false_easting\",0],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3782\"");
    add_srs_wkt (p, 12, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3783, "epsg", 3783,
		      "Pitcairn 2006 / Pitcairn TM 2006");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-25.06855261111111 +lon_0=-130.112967");
    add_proj4text (p, 1,
		   "1111111 +k=1 +x_0=14200 +y_0=15500 +ellps=WGS84 +towgs84");
    add_proj4text (p, 2, "=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pitcairn 2006 / Pitcairn TM 2006\",GEOGCS[\"Pit");
    add_srs_wkt (p, 1,
		 "cairn 2006\",DATUM[\"Pitcairn_2006\",SPHEROID[\"WGS 84\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6763\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4763\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",-25.06855261111111],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",-130.1129671111111],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",1420");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",15500],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"3783\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 3784, "epsg", 3784,
		      "Pitcairn 1967 / UTM zone 9S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +south +ellps=intl +towgs84=185,165,42");
    add_proj4text (p, 1, ",0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pitcairn 1967 / UTM zone 9S\",GEOGCS[\"Pitcairn");
    add_srs_wkt (p, 1,
		 " 1967\",DATUM[\"Pitcairn_1967\",SPHEROID[\"International");
    add_srs_wkt (p, 2,
		 " 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS8");
    add_srs_wkt (p, 3,
		 "4[185,165,42,0,0,0,0],AUTHORITY[\"EPSG\",\"6729\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4729\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",-129],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 11,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"3784\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3785, "epsg", 3785,
		      "Popular Visualisation CRS / Mercator (deprecated)");
    add_proj4text (p, 0,
		   "+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 ");
    add_proj4text (p, 1,
		   "+x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext ");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Popular Visualisation CRS / Mercator (deprecate");
    add_srs_wkt (p, 1,
		 "d)\",GEOGCS[\"Popular Visualisation CRS\",DATUM[\"Popula");
    add_srs_wkt (p, 2,
		 "r_Visualisation_Datum\",SPHEROID[\"Popular Visualisation");
    add_srs_wkt (p, 3,
		 " Sphere\",6378137,0,AUTHORITY[\"EPSG\",\"7059\"]],TOWGS8");
    add_srs_wkt (p, 4,
		 "4[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6055\"]],PRIMEM[\"");
    add_srs_wkt (p, 5,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 6,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"4055\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 8,
		 "EPSG\",\"9001\"]],PROJECTION[\"Mercator_1SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"central_meridian\",0],PARAMETER[\"scale_factor\",1],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",0],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],EXTENSION[\"PROJ4\",\"+proj=merc +a=6378137 +b=637");
    add_srs_wkt (p, 12,
		 "8137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +unit");
    add_srs_wkt (p, 13,
		 "s=m +nadgrids=@null +wktext  +no_defs\"],AUTHORITY[\"EPS");
    add_srs_wkt (p, 14, "G\",\"3785\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3786, "epsg", 3786,
		      "World Equidistant Cylindrical (Sphere) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=eqc +lat_ts=0 +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +a=6");
    add_proj4text (p, 1, "371007 +b=6371007 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"World Equidistant Cylindrical (Sphere) (depreca");
    add_srs_wkt (p, 1,
		 "ted)\",GEOGCS[\"Unspecified datum based upon the GRS 198");
    add_srs_wkt (p, 2,
		 "0 Authalic Sphere\",DATUM[\"Not_specified_based_on_GRS_1");
    add_srs_wkt (p, 3,
		 "980_Authalic_Sphere\",SPHEROID[\"GRS 1980 Authalic Spher");
    add_srs_wkt (p, 4,
		 "e\",6371007,0,AUTHORITY[\"EPSG\",\"7048\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"6047\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4047\"]],U");
    add_srs_wkt (p, 8,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 9,
		 "[\"Equirectangular\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"central_meridian\",0],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12, "PSG\",\"3786\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3787, "epsg", 3787,
		      "MGI / Slovene National Grid (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=-5000000 +ellps=bessel +datum=hermannskogel +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Slovene National Grid (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"MGI\",DATUM[\"Militar_Geographische_Institute\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137");
    add_srs_wkt (p, 4,
		 ",1.474,5.297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIME");
    add_srs_wkt (p, 5,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 6,
		 "egree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",15],PARAMETER[\"scale_factor\",0.9999],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",-5000000],AUTHORITY[\"EPSG\",\"3787\"],AXIS[\"Y\",E");
    add_srs_wkt (p, 13, "AST],AXIS[\"X\",NORTH]]");
    p = add_epsg_def (first, last, 3788, "epsg", 3788,
		      "NZGD2000 / Auckland Islands TM 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=166 +k=1 +x_0=3500000 +y_0=1");
    add_proj4text (p, 1,
		   "0000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Auckland Islands TM 2000\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NZGD2000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",166],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 10,
		 "tor\",1],PARAMETER[\"false_easting\",3500000],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"3788\"");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3789, "epsg", 3789,
		      "NZGD2000 / Campbell Island TM 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=169 +k=1 +x_0=3500000 +y_0=1");
    add_proj4text (p, 1,
		   "0000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Campbell Island TM 2000\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "ZGD2000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",169],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 10,
		 "tor\",1],PARAMETER[\"false_easting\",3500000],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"3789\"");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3790, "epsg", 3790,
		      "NZGD2000 / Antipodes Islands TM 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=179 +k=1 +x_0=3500000 +y_0=1");
    add_proj4text (p, 1,
		   "0000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Antipodes Islands TM 2000\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NZGD2000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",179],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 10,
		 "tor\",1],PARAMETER[\"false_easting\",3500000],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"3790\"");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3791, "epsg", 3791,
		      "NZGD2000 / Raoul Island TM 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-178 +k=1 +x_0=3500000 +y_0=");
    add_proj4text (p, 1,
		   "10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Raoul Island TM 2000\",GEOGCS[\"NZGD");
    add_srs_wkt (p, 1,
		 "2000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metr");
    add_srs_wkt (p, 7,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 8,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"central_meridian\",-178],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 10,
		 "r\",1],PARAMETER[\"false_easting\",3500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"3791\"],");
    add_srs_wkt (p, 12, "AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3793, "epsg", 3793,
		      "NZGD2000 / Chatham Islands TM 2000");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-176.5 +k=1 +x_0=3500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / Chatham Islands TM 2000\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "ZGD2000\",DATUM[\"New_Zealand_Geodetic_Datum_2000\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",-176.5],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",1],PARAMETER[\"false_easting\",3500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"379");
    add_srs_wkt (p, 12,
		 "3\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 3794, "epsg", 3794,
		      "Slovenia 1996 / Slovene National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=-5000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Slovenia 1996 / Slovene National Grid\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Slovenia 1996\",DATUM[\"Slovenia_Geodetic_Datum_1996\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6765\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4765\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",15],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",-5000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "3794\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 3795, "epsg", 3795, "NAD27 / Cuba Norte");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=23 +lat_2=21.7 +lat_0=22.35 +lon_0=-81 ");
    add_proj4text (p, 1,
		   "+x_0=500000 +y_0=280296.016 +ellps=clrk66 +datum=NAD27 +");
    add_proj4text (p, 2, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Cuba Norte\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",23],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 9,
		 "2\",21.7],PARAMETER[\"latitude_of_origin\",22.35],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"central_meridian\",-81],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",280296.016],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"3795\"],AXIS[\"Y\",NORTH],AXIS[\"X\",EAST");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 3796, "epsg", 3796, "NAD27 / Cuba Sur");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=21.3 +lat_2=20.13333333333333 +lat_0=20");
    add_proj4text (p, 1,
		   ".71666666666667 +lon_0=-76.83333333333333 +x_0=500000 +y");
    add_proj4text (p, 2,
		   "_0=229126.939 +ellps=clrk66 +datum=NAD27 +units=m +no_de");
    add_proj4text (p, 3, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Cuba Sur\",GEOGCS[\"NAD27\",DATUM[\"Nor");
    add_srs_wkt (p, 1,
		 "th_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637820");
    add_srs_wkt (p, 2,
		 "6.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "67\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",21.3],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 9,
		 "2\",20.13333333333333],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 10,
		 "20.71666666666667],PARAMETER[\"central_meridian\",-76.83");
    add_srs_wkt (p, 11,
		 "333333333333],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_northing\",229126.939],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3796\"],AXIS[\"Y\",NORTH],AXIS[\"X\",EAST]]");
    p = add_epsg_def (first, last, 3797, "epsg", 3797, "NAD27 / MTQ Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=50 +lat_2=46 +lat_0=44 +lon_0=-70 +x_0=");
    add_proj4text (p, 1,
		   "800000 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / MTQ Lambert\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",50],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 9,
		 "2\",46],PARAMETER[\"latitude_of_origin\",44],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",-70],PARAMETER[\"false_easting\",8000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "3797\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3798, "epsg", 3798, "NAD83 / MTQ Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=50 +lat_2=46 +lat_0=44 +lon_0=-70 +x_0=");
    add_proj4text (p, 1,
		   "800000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTQ Lambert\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",50],PARAMETER[\"standard_parallel_2\",46],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"latitude_of_origin\",44],PARAMETER[\"central");
    add_srs_wkt (p, 10,
		 "_meridian\",-70],PARAMETER[\"false_easting\",800000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3798\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3799, "epsg", 3799,
		      "NAD83(CSRS) / MTQ Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=50 +lat_2=46 +lat_0=44 +lon_0=-70 +x_0=");
    add_proj4text (p, 1, "800000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTQ Lambert\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",50],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",46],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 10,
		 "\",44],PARAMETER[\"central_meridian\",-70],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",800000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"3799\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3800, "epsg", 3800,
		      "NAD27 / Alberta 3TM ref merid 120 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-120 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alberta 3TM ref merid 120 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-120],PARAMETER[\"scale_factor\",0.9999],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",0],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"3800\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3801, "epsg", 3801,
		      "NAD83 / Alberta 3TM ref merid 120 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-120 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alberta 3TM ref merid 120 W\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "120],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3801\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 3802, "epsg", 3802,
		      "NAD83(CSRS) / Alberta 3TM ref merid 120 W");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-120 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Alberta 3TM ref merid 120 W\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Refer");
    add_srs_wkt (p, 2,
		 "ence_System\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"614");
    add_srs_wkt (p, 4,
		 "0\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-120],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9999],PARAMETER[\"false_easting\",0],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"3802\"],AXIS[\"East");
    add_srs_wkt (p, 12, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3812, "epsg", 3812,
		      "ETRS89 / Belgian Lambert 2008");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.83333333333334 +lat_2=51.16666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=50.797815 +lon_0=4.359215833333333 +x_0=64932");
    add_proj4text (p, 2, "8 +y_0=665262 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / Belgian Lambert 2008\",GEOGCS[\"ETRS89");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"European_Terrestrial_Reference_System_1989\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",49.8333333333333");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"standard_parallel_2\",51.16666666666666],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",50.797815],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",4.359215833333333],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "easting\",649328],PARAMETER[\"false_northing\",665262],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"3812\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 14, "NORTH]]");
    p = add_epsg_def (first, last, 3814, "epsg", 3814,
		      "NAD83 / Mississippi TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=32.5 +lon_0=-89.75 +k=0.9998335 +x_0=");
    add_proj4text (p, 1,
		   "500000 +y_0=1300000 +ellps=GRS80 +datum=NAD83 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Mississippi TM\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",32.5],PARAMETER[\"central_meridian\",-89.75],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9998335],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",1300000],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"3814\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 3815, "epsg", 3815,
		      "NAD83(HARN) / Mississippi TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=32.5 +lon_0=-89.75 +k=0.9998335 +x_0=");
    add_proj4text (p, 1, "500000 +y_0=1300000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Mississippi TM\",GEOGCS[\"NAD83(H");
    add_srs_wkt (p, 1,
		 "ARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",32.5],PARAMETER[\"central");
    add_srs_wkt (p, 9,
		 "_meridian\",-89.75],PARAMETER[\"scale_factor\",0.9998335");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",1300000],AUTHORITY[\"EPSG\",\"3815\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3816, "epsg", 3816,
		      "NAD83(NSRS2007) / Mississippi TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=32.5 +lon_0=-89.75 +k=0.9998335 +x_0=");
    add_proj4text (p, 1,
		   "500000 +y_0=1300000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Mississippi TM\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference_");
    add_srs_wkt (p, 2,
		 "System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",32.5],PARAMETER[\"central_meridian\",-89.75");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9998335],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",500000],PARAMETER[\"false_northing\",1300000");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"3816\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 13, "\",NORTH]]");
    p = add_epsg_def (first, last, 3819, "epsg", 3819, "HD1909");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=595.48,121.69,515.3");
    add_proj4text (p, 1, "5,4.115,-2.9383,0.853,-3.408 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"HD1909\",DATUM[\"Hungarian_Datum_1909\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7004\"]],TOWGS84[595.48,121.69,515.35,4.115,-2.93");
    add_srs_wkt (p, 3,
		 "83,0.853,-3.408],AUTHORITY[\"EPSG\",\"1024\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6, "THORITY[\"EPSG\",\"3819\"]]");
    p = add_epsg_def (first, last, 3821, "epsg", 3821, "TWD67");
    add_proj4text (p, 0, "+proj=longlat +ellps=aust_SA +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"TWD67\",DATUM[\"Taiwan_Datum_1967\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "GRS 1967 Modified\",6378160,298.25,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7050\"]],AUTHORITY[\"EPSG\",\"1025\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"3821\"]]");
    p = add_epsg_def (first, last, 3824, "epsg", 3824, "TWD97");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"TWD97\",DATUM[\"Taiwan_Datum_1997\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1026\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"3824\"]]");
    p = add_epsg_def (first, last, 3825, "epsg", 3825, "TWD97 / TM2 zone 119");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=119 +k=0.9999 +x_0=250000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TWD97 / TM2 zone 119\",GEOGCS[\"TWD97\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Taiwan_Datum_1997\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 2,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"1026\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"3824\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",11");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",250000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3825\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 3826, "epsg", 3826, "TWD97 / TM2 zone 121");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=121 +k=0.9999 +x_0=250000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TWD97 / TM2 zone 121\",GEOGCS[\"TWD97\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Taiwan_Datum_1997\",SPHEROID[\"GRS 1980\",6378137,298.25");
    add_srs_wkt (p, 2,
		 "7222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"1026\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"3824\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",12");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",250000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3826\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 3827, "epsg", 3827, "TWD67 / TM2 zone 119");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=119 +k=0.9999 +x_0=250000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TWD67 / TM2 zone 119\",GEOGCS[\"TWD67\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Taiwan_Datum_1967\",SPHEROID[\"GRS 1967 Modified\",63781");
    add_srs_wkt (p, 2,
		 "60,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"1025\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3821\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 7,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",119],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",0.9999],PARAMETER[\"false_easting\",250000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3827\"");
    add_srs_wkt (p, 11, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3828, "epsg", 3828, "TWD67 / TM2 zone 121");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=121 +k=0.9999 +x_0=250000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TWD67 / TM2 zone 121\",GEOGCS[\"TWD67\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Taiwan_Datum_1967\",SPHEROID[\"GRS 1967 Modified\",63781");
    add_srs_wkt (p, 2,
		 "60,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"1025\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3821\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 7,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",121],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",0.9999],PARAMETER[\"false_easting\",250000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3828\"");
    add_srs_wkt (p, 11, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3829, "epsg", 3829,
		      "Hu Tzu Shan / UTM zone 51N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +ellps=intl +towgs84=-637,-549,-203,0");
    add_proj4text (p, 1, ",0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Hu Tzu Shan / UTM zone 51N\",GEOGCS[\"Hu Tzu Sh");
    add_srs_wkt (p, 1,
		 "an 1950\",DATUM[\"Hu_Tzu_Shan_1950\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[-637,-549,-203,0,0,0,0],AUTHORITY[\"EPSG\",\"6236");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4236\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",123],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3829\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3832, "epsg", 3832, "WGS 84 / PDC Mercator");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=150 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 1, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / PDC Mercator\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercator_1SP\"");
    add_srs_wkt (p, 7,
		 "],PARAMETER[\"central_meridian\",150],PARAMETER[\"scale_");
    add_srs_wkt (p, 8,
		 "factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"f");
    add_srs_wkt (p, 9,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"3832\"],AXIS[\"E");
    add_srs_wkt (p, 10, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3833, "epsg", 3833,
		      "Pulkovo 1942(58) / Gauss-Kruger zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=2500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Gauss-Kruger zone 2\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",9],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",2500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3833\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3834, "epsg", 3834,
		      "Pulkovo 1942(83) / Gauss-Kruger zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=2500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / Gauss-Kruger zone 2\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",9],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",2500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"3834\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3835, "epsg", 3835,
		      "Pulkovo 1942(83) / Gauss-Kruger zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=3500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / Gauss-Kruger zone 3\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",15],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",3500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3835\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3836, "epsg", 3836,
		      "Pulkovo 1942(83) / Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / Gauss-Kruger zone 4\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",21],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",4500000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"3836\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 3837, "epsg", 3837,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=3500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 3");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",9],PARAMETER[\"scale_factor\",1],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",3500000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"3837\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3838, "epsg", 3838,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 4");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",12],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",4500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3838\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3839, "epsg", 3839,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 9");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",27],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",9500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3839\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3840, "epsg", 3840,
		      "Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=30 +k=1 +x_0=10500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 1");
    add_srs_wkt (p, 1,
		 "0\",GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4179\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",30],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",10500000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"3840\"],AXIS[\"X\",NORTH],AXIS[\"Y");
    add_srs_wkt (p, 12, "\",EAST]]");
    p = add_epsg_def (first, last, 3841, "epsg", 3841,
		      "Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 6");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",18],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",6500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3841\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3842, "epsg", 3842,
		      "Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 7");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",18],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",6500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3842\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3843, "epsg", 3843,
		      "Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 8");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4178\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",18],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",6500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"3843\"],AXIS[\"X\",NORTH],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",EAST]]");
    p = add_epsg_def (first, last, 3844, "epsg", 3844,
		      "Pulkovo 1942(58) / Stereo70");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=46 +lon_0=25 +k=0.99975 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=500000 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942(58) / Stereo70\",GEOGCS[\"Pulkovo ");
    add_srs_wkt (p, 1,
		 "1942(58)\",DATUM[\"Pulkovo_1942_58\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4179\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Oblique_Stereographic\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",46],PARAMETER[\"central_meridian\",25],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",0.99975],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",500000],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"3844\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAS");
    add_srs_wkt (p, 12, "T]]");
    p = add_epsg_def (first, last, 3845, "epsg", 3845,
		      "SWEREF99 / RT90 7.5 gon V emulation");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=11.30625 +k=1.000006 +x_0=15");
    add_proj4text (p, 1,
		   "00025.141 +y_0=-667.282 +ellps=GRS80 +towgs84=0,0,0,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 / RT90 7.5 gon V emulation\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "SWEREF99\",DATUM[\"SWEREF99\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6619\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4619\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",11.30625],PARAMETER[\"scale_factor\",1.000006],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",1500025.141],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",-667.282],AUTHORITY[\"EPSG\",\"3845\"],AXIS[");
    add_srs_wkt (p, 12, "\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3846, "epsg", 3846,
		      "SWEREF99 / RT90 5 gon V emulation");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.55626666666667 +k=1.00000");
    add_proj4text (p, 1,
		   "58 +x_0=1500044.695 +y_0=-667.13 +ellps=GRS80 +towgs84=0");
    add_proj4text (p, 2, ",0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 / RT90 5 gon V emulation\",GEOGCS[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",DATUM[\"SWEREF99\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6619\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4619\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",13.55626666666667],PARAMETER[\"scale_factor\",1.00");
    add_srs_wkt (p, 10,
		 "00058],PARAMETER[\"false_easting\",1500044.695],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",-667.13],AUTHORITY[\"EPSG\",\"3846\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3847, "epsg", 3847,
		      "SWEREF99 / RT90 2.5 gon V emulation");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15.80628452944445 +k=1.00000");
    add_proj4text (p, 1,
		   "561024 +x_0=1500064.274 +y_0=-667.711 +ellps=GRS80 +towg");
    add_proj4text (p, 2, "s84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 / RT90 2.5 gon V emulation\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "SWEREF99\",DATUM[\"SWEREF99\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6619\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4619\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",15.80628452944445],PARAMETER[\"scale_factor\",1.0");
    add_srs_wkt (p, 10,
		 "0000561024],PARAMETER[\"false_easting\",1500064.274],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",-667.711],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "3847\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3848, "epsg", 3848,
		      "SWEREF99 / RT90 0 gon emulation");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18.0563 +k=1.0000054 +x_0=15");
    add_proj4text (p, 1,
		   "00083.521 +y_0=-668.8440000000001 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 / RT90 0 gon emulation\",GEOGCS[\"SWER");
    add_srs_wkt (p, 1,
		 "EF99\",DATUM[\"SWEREF99\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 2,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6619\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4619\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",18.0563],PARAMETER[\"scale_factor\",1.0000054],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",1500083.521],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",-668.844],AUTHORITY[\"EPSG\",\"3848\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3849, "epsg", 3849,
		      "SWEREF99 / RT90 2.5 gon O emulation");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=20.30631666666667 +k=1.00000");
    add_proj4text (p, 1,
		   "52 +x_0=1500102.765 +y_0=-670.706 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 / RT90 2.5 gon O emulation\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "SWEREF99\",DATUM[\"SWEREF99\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6619\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4619\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",20.30631666666667],PARAMETER[\"scale_factor\",1.0");
    add_srs_wkt (p, 10,
		 "000052],PARAMETER[\"false_easting\",1500102.765],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",-670.706],AUTHORITY[\"EPSG\",\"384");
    add_srs_wkt (p, 12, "9\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3850, "epsg", 3850,
		      "SWEREF99 / RT90 5 gon O emulation");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=22.55633333333333 +k=1.00000");
    add_proj4text (p, 1,
		   "49 +x_0=1500121.846 +y_0=-672.557 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SWEREF99 / RT90 5 gon O emulation\",GEOGCS[\"SW");
    add_srs_wkt (p, 1,
		 "EREF99\",DATUM[\"SWEREF99\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6619\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4619\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",22.55633333333333],PARAMETER[\"scale_factor\",1.00");
    add_srs_wkt (p, 10,
		 "00049],PARAMETER[\"false_easting\",1500121.846],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",-672.557],AUTHORITY[\"EPSG\",\"3850");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3851, "epsg", 3851, "NZGD2000 / NZCS2000");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-37.5 +lat_2=-44.5 +lat_0=-41 +lon_0=17");
    add_proj4text (p, 1,
		   "3 +x_0=3000000 +y_0=7000000 +ellps=GRS80 +towgs84=0,0,0,");
    add_proj4text (p, 2, "0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD2000 / NZCS2000\",GEOGCS[\"NZGD2000\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"New_Zealand_Geodetic_Datum_2000\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6167\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4167\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",-37.5],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_2\",-44.5],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",-41],PARAMETER[\"central_meridian\",173],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",3000000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",7000000],AUTHORITY[\"EPSG\",\"3851\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_14 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 3852, "epsg", 3852, "RSRGD2000 / DGLC2000");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-76.66666666666667 +lat_2=-79.333333333");
    add_proj4text (p, 1,
		   "33333 +lat_0=-90 +lon_0=157 +x_0=500000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RSRGD2000 / DGLC2000\",GEOGCS[\"RSRGD2000\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Ross_Sea_Region_Geodetic_Datum_2000\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6764\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4764\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Confo");
    add_srs_wkt (p, 8,
		 "rmal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",-76.");
    add_srs_wkt (p, 9,
		 "66666666666667],PARAMETER[\"standard_parallel_2\",-79.33");
    add_srs_wkt (p, 10,
		 "333333333333],PARAMETER[\"latitude_of_origin\",-90],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",157],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"3852\"],AXIS[\"Northing\",NORTH],AXIS[\"Easting");
    add_srs_wkt (p, 14, "\",EAST]]");
    p = add_epsg_def (first, last, 3854, "epsg", 3854, "County ST74");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18.05787 +k=0.99999506 +x_0=");
    add_proj4text (p, 1,
		   "100182.7406 +y_0=-6500620.1207 +ellps=GRS80 +towgs84=0,0");
    add_proj4text (p, 2, ",0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"County ST74\",GEOGCS[\"SWEREF99\",DATUM[\"SWERE");
    add_srs_wkt (p, 1,
		 "F99\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6619\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4619\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",18.05787],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",0.99999506],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",100182.7406],PARAMETER[\"false_northing\",-6500620.");
    add_srs_wkt (p, 11,
		 "1207],AUTHORITY[\"EPSG\",\"3854\"],AXIS[\"x\",NORTH],AXI");
    add_srs_wkt (p, 12, "S[\"y\",EAST]]");
    p = add_epsg_def (first, last, 3857, "epsg", 3857,
		      "WGS 84 / Pseudo-Mercator");
    add_proj4text (p, 0,
		   "+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 ");
    add_proj4text (p, 1,
		   "+x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext ");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Pseudo-Mercator\",GEOGCS[\"WGS 84\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.2572235");
    add_srs_wkt (p, 2,
		 "63,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"63");
    add_srs_wkt (p, 3,
		 "26\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercator_1");
    add_srs_wkt (p, 7,
		 "SP\"],PARAMETER[\"central_meridian\",0],PARAMETER[\"scal");
    add_srs_wkt (p, 8,
		 "e_factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "false_northing\",0],EXTENSION[\"PROJ4\",\"+proj=merc +a=");
    add_srs_wkt (p, 10,
		 "6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=");
    add_srs_wkt (p, 11,
		 "0 +k=1.0 +units=m +nadgrids=@null +wktext  +no_defs\"],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"3857\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 3889, "epsg", 3889, "IGRS");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGRS\",DATUM[\"Iraqi_Geospatial_Reference_Syste");
    add_srs_wkt (p, 1,
		 "m\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"1029\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3889\"]]");
    p = add_epsg_def (first, last, 3890, "epsg", 3890, "IGRS / UTM zone 37N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGRS / UTM zone 37N\",GEOGCS[\"IGRS\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "raqi_Geospatial_Reference_System\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1029\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"3889\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",39],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3890\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3891, "epsg", 3891, "IGRS / UTM zone 38N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGRS / UTM zone 38N\",GEOGCS[\"IGRS\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "raqi_Geospatial_Reference_System\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1029\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"3889\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",45],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3891\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3892, "epsg", 3892, "IGRS / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"IGRS / UTM zone 39N\",GEOGCS[\"IGRS\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "raqi_Geospatial_Reference_System\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1029\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"3889\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 9,
		 "l_meridian\",51],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"3892\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3893, "epsg", 3893,
		      "ED50 / Iraq National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.02626833333333 +lon_0=46.5 +k=0.99");
    add_proj4text (p, 1, "94 +x_0=800000 +y_0=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / Iraq National Grid\",GEOGCS[\"ED50\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"European_Datum_1950\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",29.02626833333333],PARAMETER[\"central_meridian\",46.");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9994],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",800000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"3893\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 12, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 3906, "epsg", 3906, "MGI 1901");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"MGI 1901\",DATUM[\"MGI_1901\",SPHEROID[\"Bessel");
    add_srs_wkt (p, 1,
		 " 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 2,
		 "4\"]],AUTHORITY[\"EPSG\",\"1031\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"3906\"]]");
    p = add_epsg_def (first, last, 3907, "epsg", 3907,
		      "MGI 1901 / Balkans zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=5500000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI 1901 / Balkans zone 5\",GEOGCS[\"MGI 1901\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"MGI_1901\",SPHEROID[\"Bessel 1841\",6377397.155");
    add_srs_wkt (p, 2,
		 ",299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"1031\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3906\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",15],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",5500000],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3907");
    add_srs_wkt (p, 11, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3908, "epsg", 3908,
		      "MGI 1901 / Balkans zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=0.9999 +x_0=6500000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI 1901 / Balkans zone 6\",GEOGCS[\"MGI 1901\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"MGI_1901\",SPHEROID[\"Bessel 1841\",6377397.155");
    add_srs_wkt (p, 2,
		 ",299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"1031\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3906\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",18],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",6500000],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3908");
    add_srs_wkt (p, 11, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3909, "epsg", 3909,
		      "MGI 1901 / Balkans zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=0.9999 +x_0=7500000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI 1901 / Balkans zone 7\",GEOGCS[\"MGI 1901\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"MGI_1901\",SPHEROID[\"Bessel 1841\",6377397.155");
    add_srs_wkt (p, 2,
		 ",299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"1031\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3906\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",7500000],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3909");
    add_srs_wkt (p, 11, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3910, "epsg", 3910,
		      "MGI 1901 / Balkans zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9999 +x_0=8500000 +y");
    add_proj4text (p, 1, "_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI 1901 / Balkans zone 8\",GEOGCS[\"MGI 1901\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"MGI_1901\",SPHEROID[\"Bessel 1841\",6377397.155");
    add_srs_wkt (p, 2,
		 ",299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"1031\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3906\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",24],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9999],PARAMETER[\"false_easting\",8500000],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3910");
    add_srs_wkt (p, 11, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3911, "epsg", 3911,
		      "MGI 1901 / Slovenia Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI 1901 / Slovenia Grid\",GEOGCS[\"MGI 1901\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"MGI_1901\",SPHEROID[\"Bessel 1841\",6377397.155,");
    add_srs_wkt (p, 2,
		 "299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"1031\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3906\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",15],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",0.9999],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3911\"");
    add_srs_wkt (p, 11, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 3912, "epsg", 3912,
		      "MGI 1901 / Slovene National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=-5000000 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI 1901 / Slovene National Grid\",GEOGCS[\"MGI");
    add_srs_wkt (p, 1,
		 " 1901\",DATUM[\"MGI_1901\",SPHEROID[\"Bessel 1841\",6377");
    add_srs_wkt (p, 2,
		 "397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"1031\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"3906");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",0],PARAMETER[\"central_meridian\",15],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",-5000000],AUTHORITY[\"");
    add_srs_wkt (p, 11, "EPSG\",\"3912\"],AXIS[\"Y\",EAST],AXIS[\"X\",NORTH]]");
    p = add_epsg_def (first, last, 3920, "epsg", 3920,
		      "Puerto Rico / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=clrk66 +towgs84=11,72,-101,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Puerto Rico / UTM zone 20N\",GEOGCS[\"Puerto Ri");
    add_srs_wkt (p, 1,
		 "co\",DATUM[\"Puerto_Rico\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[11,72,-101,0,0,0,0],AUTHORITY[\"EPSG\",\"6139\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4139\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-63],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"3920\"],AXIS[\"Easti");
    add_srs_wkt (p, 12, "ng\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3942, "epsg", 3942, "RGF93 / CC42");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.25 +lat_2=42.75 +lat_0=42 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=1200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC42\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",41.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",42.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",42],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",120");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3942\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3943, "epsg", 3943, "RGF93 / CC43");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.25 +lat_2=43.75 +lat_0=43 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=2200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC43\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",42.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",43.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",43],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",220");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3943\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3944, "epsg", 3944, "RGF93 / CC44");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.25 +lat_2=44.75 +lat_0=44 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=3200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC44\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",43.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",44.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",44],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",320");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3944\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3945, "epsg", 3945, "RGF93 / CC45");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.25 +lat_2=45.75 +lat_0=45 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=4200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC45\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",44.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",45.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",45],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",420");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3945\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3946, "epsg", 3946, "RGF93 / CC46");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.25 +lat_2=46.75 +lat_0=46 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=5200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC46\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",45.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",46.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",46],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",520");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3946\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3947, "epsg", 3947, "RGF93 / CC47");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.25 +lat_2=47.75 +lat_0=47 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=6200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC47\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",46.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",47.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",47],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",620");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3947\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3948, "epsg", 3948, "RGF93 / CC48");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.25 +lat_2=48.75 +lat_0=48 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=7200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC48\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",47.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",48.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",48],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",720");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3948\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3949, "epsg", 3949, "RGF93 / CC49");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.25 +lat_2=49.75 +lat_0=49 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=8200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC49\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",48.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",49.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",49],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",820");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3949\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3950, "epsg", 3950, "RGF93 / CC50");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.25 +lat_2=50.75 +lat_0=50 +lon_0=3 +");
    add_proj4text (p, 1,
		   "x_0=1700000 +y_0=9200000 +ellps=GRS80 +towgs84=0,0,0,0,0");
    add_proj4text (p, 2, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGF93 / CC50\",GEOGCS[\"RGF93\",DATUM[\"Reseau_");
    add_srs_wkt (p, 1,
		 "Geodesique_Francais_1993\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6171\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4171\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"standard_parallel_1\",49.25],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "tandard_parallel_2\",50.75],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",50],PARAMETER[\"central_meridian\",3],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",1700000],PARAMETER[\"false_northing\",920");
    add_srs_wkt (p, 12,
		 "0000],AUTHORITY[\"EPSG\",\"3950\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3968, "epsg", 3968,
		      "NAD83 / Virginia Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37 +lat_2=39.5 +lat_0=36 +lon_0=-79.5 +");
    add_proj4text (p, 1,
		   "x_0=0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Virginia Lambert\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 8,
		 "ndard_parallel_1\",37],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 9,
		 ",39.5],PARAMETER[\"latitude_of_origin\",36],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",-79.5],PARAMETER[\"false_easting\",0]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"39");
    add_srs_wkt (p, 12, "68\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3969, "epsg", 3969,
		      "NAD83(HARN) / Virginia Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37 +lat_2=39.5 +lat_0=36 +lon_0=-79.5 +");
    add_proj4text (p, 1, "x_0=0 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Virginia Lambert\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",37],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",39.5],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",36],PARAMETER[\"central_meridian\",-79.5],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",0],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 12,
		 "THORITY[\"EPSG\",\"3969\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 3970, "epsg", 3970,
		      "NAD83(NSRS2007) / Virginia Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37 +lat_2=39.5 +lat_0=36 +lon_0=-79.5 +");
    add_proj4text (p, 1,
		   "x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Virginia Lambert\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referenc");
    add_srs_wkt (p, 2,
		 "e_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.257222");
    add_srs_wkt (p, 3,
		 "101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",37],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 10,
		 "el_2\",39.5],PARAMETER[\"latitude_of_origin\",36],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-79.5],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"3970\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3975, "epsg", 3975,
		      "WGS 84 / NSIDC EASE-Grid Global");
    add_proj4text (p, 0,
		   "+proj=cea +lon_0=0 +lat_ts=30 +x_0=0 +y_0=0 +ellps=WGS84");
    add_proj4text (p, 1, " +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / NSIDC EASE-Grid Global\",GEOGCS[\"WGS ");
    add_srs_wkt (p, 1,
		 "84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.");
    add_srs_wkt (p, 2,
		 "257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"C");
    add_srs_wkt (p, 7,
		 "ylindrical_Equal_Area\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 8,
		 "\",30],PARAMETER[\"central_meridian\",0],PARAMETER[\"fal");
    add_srs_wkt (p, 9,
		 "se_easting\",0],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 10,
		 "Y[\"EPSG\",\"3975\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 11, "");
    p = add_epsg_def (first, last, 3976, "epsg", 3976,
		      "WGS 84 / NSIDC Sea Ice Polar Stereographic South");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / NSIDC Sea Ice Polar Stereographic Sout");
    add_srs_wkt (p, 1,
		 "h\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS ");
    add_srs_wkt (p, 2,
		 "84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4326\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Polar_Stereographic\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",-70],PARAMETER[\"central_meridian\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "3976\"],AXIS[\"X\",UNKNOWN],AXIS[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3978, "epsg", 3978,
		      "NAD83 / Canada Atlas Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=77 +lat_0=49 +lon_0=-95 +x_0=");
    add_proj4text (p, 1,
		   "0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Canada Atlas Lambert\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",49],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 9,
		 "2\",77],PARAMETER[\"latitude_of_origin\",49],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "central_meridian\",-95],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3978");
    add_srs_wkt (p, 12, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3979, "epsg", 3979,
		      "NAD83(CSRS) / Canada Atlas Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=77 +lat_0=49 +lon_0=-95 +x_0=");
    add_proj4text (p, 1, "0 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / Canada Atlas Lambert\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Reference_Sy");
    add_srs_wkt (p, 2,
		 "stem\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conforma");
    add_srs_wkt (p, 8,
		 "l_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",49],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"standard_parallel_2\",77],PARAMETER[\"latitude_");
    add_srs_wkt (p, 10,
		 "of_origin\",49],PARAMETER[\"central_meridian\",-95],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",0],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"3979\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 13, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 3985, "epsg", 3985,
		      "Katanga 1955 / Katanga Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=-6.5 +lat_2=-11.5 +lat_0=9 +lon_0=26 +x");
    add_proj4text (p, 1,
		   "_0=500000 +y_0=500000 +ellps=clrk66 +towgs84=-103.746,-9");
    add_proj4text (p, 2, ".614,-255.95,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Katanga 1955 / Katanga Lambert\",GEOGCS[\"Katan");
    add_srs_wkt (p, 1,
		 "ga 1955\",DATUM[\"Katanga_1955\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4695\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 9,
		 "arallel_1\",-6.5],PARAMETER[\"standard_parallel_2\",-11.");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"latitude_of_origin\",9],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",26],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "3985\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3986, "epsg", 3986,
		      "Katanga 1955 / Katanga Gauss zone A");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-9 +lon_0=30 +k=1 +x_0=200000 +y_0=50");
    add_proj4text (p, 1,
		   "0000 +ellps=clrk66 +towgs84=-103.746,-9.614,-255.95,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Katanga 1955 / Katanga Gauss zone A\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Katanga 1955\",DATUM[\"Katanga_1955\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7008\"]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4695");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",-9],PARAMETER[\"central_meridian\",30],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",200000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"3986\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3987, "epsg", 3987,
		      "Katanga 1955 / Katanga Gauss zone B");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-9 +lon_0=28 +k=1 +x_0=200000 +y_0=50");
    add_proj4text (p, 1,
		   "0000 +ellps=clrk66 +towgs84=-103.746,-9.614,-255.95,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Katanga 1955 / Katanga Gauss zone B\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Katanga 1955\",DATUM[\"Katanga_1955\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7008\"]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4695");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",-9],PARAMETER[\"central_meridian\",28],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",200000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"3987\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3988, "epsg", 3988,
		      "Katanga 1955 / Katanga Gauss zone C");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-9 +lon_0=26 +k=1 +x_0=200000 +y_0=50");
    add_proj4text (p, 1,
		   "0000 +ellps=clrk66 +towgs84=-103.746,-9.614,-255.95,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Katanga 1955 / Katanga Gauss zone C\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Katanga 1955\",DATUM[\"Katanga_1955\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7008\"]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4695");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",-9],PARAMETER[\"central_meridian\",26],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",200000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"3988\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3989, "epsg", 3989,
		      "Katanga 1955 / Katanga Gauss zone D");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-9 +lon_0=24 +k=1 +x_0=200000 +y_0=50");
    add_proj4text (p, 1,
		   "0000 +ellps=clrk66 +towgs84=-103.746,-9.614,-255.95,0,0,");
    add_proj4text (p, 2, "0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Katanga 1955 / Katanga Gauss zone D\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Katanga 1955\",DATUM[\"Katanga_1955\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7008\"]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4695");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",-9],PARAMETER[\"central_meridian\",24],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",200000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"3989\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3991, "epsg", 3991,
		      "Puerto Rico State Plane CS of 1927");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18.43333333333333 +lat_2=18.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=17.83333333333333 +lon_0=-66.43333333333334 +");
    add_proj4text (p, 2,
		   "x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +towgs84=11,7");
    add_proj4text (p, 3, "2,-101,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Puerto Rico State Plane CS of 1927\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "uerto Rico\",DATUM[\"Puerto_Rico\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],TOWGS84[11,72,-101,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "139\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4139\"]],UNIT[\"US s");
    add_srs_wkt (p, 7,
		 "urvey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "3\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"standard_parallel_1\",18.43333333333333],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"standard_parallel_2\",18.03333333333333],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "latitude_of_origin\",17.83333333333333],PARAMETER[\"cent");
    add_srs_wkt (p, 12,
		 "ral_meridian\",-66.43333333333334],PARAMETER[\"false_eas");
    add_srs_wkt (p, 13,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 14,
		 "[\"EPSG\",\"3991\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3992, "epsg", 3992,
		      "Puerto Rico / St. Croix");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18.43333333333333 +lat_2=18.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=17.83333333333333 +lon_0=-66.43333333333334 +");
    add_proj4text (p, 2,
		   "x_0=152400.3048006096 +y_0=30480.06096012192 +ellps=clrk");
    add_proj4text (p, 3,
		   "66 +towgs84=11,72,-101,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Puerto Rico / St. Croix\",GEOGCS[\"Puerto Rico\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Puerto_Rico\",SPHEROID[\"Clarke 1866\",6378206.");
    add_srs_wkt (p, 2,
		 "4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],TOWGS8");
    add_srs_wkt (p, 3,
		 "4[11,72,-101,0,0,0,0],AUTHORITY[\"EPSG\",\"6139\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4139\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",18.43333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",18.03333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",17.83333333333333],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-66.43333333333334],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 13,
		 "0],PARAMETER[\"false_northing\",100000],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 14, "\",\"3992\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3994, "epsg", 3994, "WGS 84 / Mercator 41");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=100 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +d");
    add_proj4text (p, 1, "atum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Mercator 41\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercator_1SP\"");
    add_srs_wkt (p, 7,
		 "],PARAMETER[\"central_meridian\",100],PARAMETER[\"scale_");
    add_srs_wkt (p, 8,
		 "factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"f");
    add_srs_wkt (p, 9,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"3994\"],AXIS[\"X");
    add_srs_wkt (p, 10, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 3995, "epsg", 3995,
		      "WGS 84 / Arctic Polar Stereographic");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=90 +lat_ts=71 +lon_0=0 +k=1 +x_0=0 +y");
    add_proj4text (p, 1, "_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Arctic Polar Stereographic\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,");
    add_srs_wkt (p, 2,
		 "298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Polar_Stereographic\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",71],PARAMETER[\"central_meridian\",0],PARAMETER[\"sca");
    add_srs_wkt (p, 9,
		 "le_factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"3995\"],AXIS[");
    add_srs_wkt (p, 11, "\"X\",UNKNOWN],AXIS[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3996, "epsg", 3996,
		      "WGS 84 / IBCAO Polar Stereographic");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=90 +lat_ts=75 +lon_0=0 +k=1 +x_0=0 +y");
    add_proj4text (p, 1, "_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / IBCAO Polar Stereographic\",GEOGCS[\"W");
    add_srs_wkt (p, 1,
		 "GS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,2");
    add_srs_wkt (p, 2,
		 "98.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Polar_Stereographic\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",75],PARAMETER[\"central_meridian\",0],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"3996\"],AXIS[\"");
    add_srs_wkt (p, 11, "X\",UNKNOWN],AXIS[\"Y\",UNKNOWN]]");
    p = add_epsg_def (first, last, 3997, "epsg", 3997,
		      "WGS 84 / Dubai Local TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=55.33333333333334 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "500000 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Dubai Local TM\",GEOGCS[\"WGS 84\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.25722356");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"632");
    add_srs_wkt (p, 3,
		 "6\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",55.33333333333334],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "cale_factor\",1],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3997\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 4001, "epsg", 4001,
		      "Unknown datum based upon the Airy 1830 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=airy +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Airy 1830 ellipsoi");
    add_srs_wkt (p, 1,
		 "d\",DATUM[\"Not_specified_based_on_Airy_1830_ellipsoid\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Airy 1830\",6377563.396,299.3249646,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7001\"]],AUTHORITY[\"EPSG\",\"6001\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6, "]],AUTHORITY[\"EPSG\",\"4001\"]]");
    p = add_epsg_def (first, last, 4002, "epsg", 4002,
		      "Unknown datum based upon the Airy Modified 1849 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=mod_airy +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Airy Modified 1849");
    add_srs_wkt (p, 1,
		 " ellipsoid\",DATUM[\"Not_specified_based_on_Airy_Modifie");
    add_srs_wkt (p, 2,
		 "d_1849_ellipsoid\",SPHEROID[\"Airy Modified 1849\",63773");
    add_srs_wkt (p, 3,
		 "40.189,299.3249646,AUTHORITY[\"EPSG\",\"7002\"]],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6002\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4002\"");
    add_srs_wkt (p, 7, "]]");
    p = add_epsg_def (first, last, 4003, "epsg", 4003,
		      "Unknown datum based upon the Australian National Spheroid");
    add_proj4text (p, 0, "+proj=longlat +ellps=aust_SA +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Australian Nationa");
    add_srs_wkt (p, 1,
		 "l Spheroid\",DATUM[\"Not_specified_based_on_Australian_N");
    add_srs_wkt (p, 2,
		 "ational_Spheroid\",SPHEROID[\"Australian National Sphero");
    add_srs_wkt (p, 3,
		 "id\",6378160,298.25,AUTHORITY[\"EPSG\",\"7003\"]],AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"6003\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4003");
    add_srs_wkt (p, 7, "\"]]");
    p = add_epsg_def (first, last, 4004, "epsg", 4004,
		      "Unknown datum based upon the Bessel 1841 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Bessel 1841 ellips");
    add_srs_wkt (p, 1,
		 "oid\",DATUM[\"Not_specified_based_on_Bessel_1841_ellipso");
    add_srs_wkt (p, 2,
		 "id\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6004\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "9122\"]],AUTHORITY[\"EPSG\",\"4004\"]]");
    p = add_epsg_def (first, last, 4005, "epsg", 4005,
		      "Unknown datum based upon the Bessel Modified ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377492.018 +b=6356173.508712696 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Bessel Modified el");
    add_srs_wkt (p, 1,
		 "lipsoid\",DATUM[\"Not_specified_based_on_Bessel_Modified");
    add_srs_wkt (p, 2,
		 "_ellipsoid\",SPHEROID[\"Bessel Modified\",6377492.018,29");
    add_srs_wkt (p, 3,
		 "9.1528128,AUTHORITY[\"EPSG\",\"7005\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6005\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4005\"]]");
    p = add_epsg_def (first, last, 4006, "epsg", 4006,
		      "Unknown datum based upon the Bessel Namibia ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=bess_nam +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Bessel Namibia ell");
    add_srs_wkt (p, 1,
		 "ipsoid\",DATUM[\"Not_specified_based_on_Bessel_Namibia_e");
    add_srs_wkt (p, 2,
		 "llipsoid\",SPHEROID[\"Bessel Namibia (GLM)\",6377483.865");
    add_srs_wkt (p, 3,
		 "280419,299.1528128,AUTHORITY[\"EPSG\",\"7046\"]],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6006\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4006\"");
    add_srs_wkt (p, 7, "]]");
    p = add_epsg_def (first, last, 4007, "epsg", 4007,
		      "Unknown datum based upon the Clarke 1858 ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378293.645208759 +b=6356617.987679838 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1858 ellips");
    add_srs_wkt (p, 1,
		 "oid\",DATUM[\"Not_specified_based_on_Clarke_1858_ellipso");
    add_srs_wkt (p, 2,
		 "id\",SPHEROID[\"Clarke 1858\",6378293.645208759,294.2606");
    add_srs_wkt (p, 3,
		 "763692569,AUTHORITY[\"EPSG\",\"7007\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6007\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4007\"]]");
    p = add_epsg_def (first, last, 4008, "epsg", 4008,
		      "Unknown datum based upon the Clarke 1866 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1866 ellips");
    add_srs_wkt (p, 1,
		 "oid\",DATUM[\"Not_specified_based_on_Clarke_1866_ellipso");
    add_srs_wkt (p, 2,
		 "id\",SPHEROID[\"Clarke 1866\",6378206.4,294.978698213900");
    add_srs_wkt (p, 3,
		 "6,AUTHORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"600");
    add_srs_wkt (p, 4,
		 "8\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4008\"]]");
    p = add_epsg_def (first, last, 4009, "epsg", 4009,
		      "Unknown datum based upon the Clarke 1866 Michigan ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378450.047548896 +b=6356826.621488444 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1866 Michig");
    add_srs_wkt (p, 1,
		 "an ellipsoid\",DATUM[\"Not_specified_based_on_Clarke_186");
    add_srs_wkt (p, 2,
		 "6_Michigan_ellipsoid\",SPHEROID[\"Clarke 1866 Michigan\"");
    add_srs_wkt (p, 3,
		 ",6378450.047548896,294.9786971646739,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"7009\"]],AUTHORITY[\"EPSG\",\"6009\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7, "Y[\"EPSG\",\"4009\"]]");
    p = add_epsg_def (first, last, 4010, "epsg", 4010,
		      "Unknown datum based upon the Clarke 1880 (Benoit) ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378300.789 +b=6356566.435 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1880 (Benoi");
    add_srs_wkt (p, 1,
		 "t) ellipsoid\",DATUM[\"Not_specified_based_on_Clarke_188");
    add_srs_wkt (p, 2,
		 "0_Benoit_ellipsoid\",SPHEROID[\"Clarke 1880 (Benoit)\",6");
    add_srs_wkt (p, 3,
		 "378300.789,293.4663155389802,AUTHORITY[\"EPSG\",\"7010\"");
    add_srs_wkt (p, 4,
		 "]],AUTHORITY[\"EPSG\",\"6010\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 6,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7, "\",\"4010\"]]");
    p = add_epsg_def (first, last, 4011, "epsg", 4011,
		      "Unknown datum based upon the Clarke 1880 (IGN) ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1880 (IGN) ");
    add_srs_wkt (p, 1,
		 "ellipsoid\",DATUM[\"Not_specified_based_on_Clarke_1880_I");
    add_srs_wkt (p, 2,
		 "GN_ellipsoid\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,");
    add_srs_wkt (p, 3,
		 "293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"6011\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4011\"]");
    add_srs_wkt (p, 7, "]");
    p = add_epsg_def (first, last, 4012, "epsg", 4012,
		      "Unknown datum based upon the Clarke 1880 (RGS) ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1880 (RGS) ");
    add_srs_wkt (p, 1,
		 "ellipsoid\",DATUM[\"Not_specified_based_on_Clarke_1880_R");
    add_srs_wkt (p, 2,
		 "GS_ellipsoid\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6012\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4012\"]]");
    p = add_epsg_def (first, last, 4013, "epsg", 4013,
		      "Unknown datum based upon the Clarke 1880 (Arc) ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.145 +b=6356514.966398753 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1880 (Arc) ");
    add_srs_wkt (p, 1,
		 "ellipsoid\",DATUM[\"Not_specified_based_on_Clarke_1880_A");
    add_srs_wkt (p, 2,
		 "rc_ellipsoid\",SPHEROID[\"Clarke 1880 (Arc)\",6378249.14");
    add_srs_wkt (p, 3,
		 "5,293.4663077,AUTHORITY[\"EPSG\",\"7013\"]],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6013\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4013\"]]");
    p = add_epsg_def (first, last, 4014, "epsg", 4014,
		      "Unknown datum based upon the Clarke 1880 (SGA 1922) ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356514.996941779 +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1880 (SGA 1");
    add_srs_wkt (p, 1,
		 "922) ellipsoid\",DATUM[\"Not_specified_based_on_Clarke_1");
    add_srs_wkt (p, 2,
		 "880_SGA_1922_ellipsoid\",SPHEROID[\"Clarke 1880 (SGA 192");
    add_srs_wkt (p, 3,
		 "2)\",6378249.2,293.46598,AUTHORITY[\"EPSG\",\"7014\"]],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6014\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7, "4014\"]]");
    p = add_epsg_def (first, last, 4015, "epsg", 4015,
		      "Unknown datum based upon the Everest 1830 (1937 Adjustment) ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377276.345 +b=6356075.41314024 +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Everest 1830 (1937");
    add_srs_wkt (p, 1,
		 " Adjustment) ellipsoid\",DATUM[\"Not_specified_based_on_");
    add_srs_wkt (p, 2,
		 "Everest_1830_1937_Adjustment_ellipsoid\",SPHEROID[\"Ever");
    add_srs_wkt (p, 3,
		 "est 1830 (1937 Adjustment)\",6377276.345,300.8017,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6015\"]],PR");
    add_srs_wkt (p, 5,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 7, "\"]],AUTHORITY[\"EPSG\",\"4015\"]]");
    p = add_epsg_def (first, last, 4016, "epsg", 4016,
		      "Unknown datum based upon the Everest 1830 (1967 Definition) ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=evrstSS +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Everest 1830 (1967");
    add_srs_wkt (p, 1,
		 " Definition) ellipsoid\",DATUM[\"Not_specified_based_on_");
    add_srs_wkt (p, 2,
		 "Everest_1830_1967_Definition_ellipsoid\",SPHEROID[\"Ever");
    add_srs_wkt (p, 3,
		 "est 1830 (1967 Definition)\",6377298.556,300.8017,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"7016\"]],AUTHORITY[\"EPSG\",\"6016\"]],PR");
    add_srs_wkt (p, 5,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 7, "\"]],AUTHORITY[\"EPSG\",\"4016\"]]");
    p = add_epsg_def (first, last, 4018, "epsg", 4018,
		      "Unknown datum based upon the Everest 1830 Modified ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377304.063 +b=6356103.038993155 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Everest 1830 Modif");
    add_srs_wkt (p, 1,
		 "ied ellipsoid\",DATUM[\"Not_specified_based_on_Everest_1");
    add_srs_wkt (p, 2,
		 "830_Modified_ellipsoid\",SPHEROID[\"Everest 1830 Modifie");
    add_srs_wkt (p, 3,
		 "d\",6377304.063,300.8017,AUTHORITY[\"EPSG\",\"7018\"]],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6018\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7, "4018\"]]");
    p = add_epsg_def (first, last, 4019, "epsg", 4019,
		      "Unknown datum based upon the GRS 1980 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the GRS 1980 ellipsoid");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Not_specified_based_on_GRS_1980_ellipsoid\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6019\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6, "THORITY[\"EPSG\",\"4019\"]]");
    p = add_epsg_def (first, last, 4020, "epsg", 4020,
		      "Unknown datum based upon the Helmert 1906 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=helmert +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Helmert 1906 ellip");
    add_srs_wkt (p, 1,
		 "soid\",DATUM[\"Not_specified_based_on_Helmert_1906_ellip");
    add_srs_wkt (p, 2,
		 "soid\",SPHEROID[\"Helmert 1906\",6378200,298.3,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7020\"]],AUTHORITY[\"EPSG\",\"6020\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6, "],AUTHORITY[\"EPSG\",\"4020\"]]");
    p = add_epsg_def (first, last, 4021, "epsg", 4021,
		      "Unknown datum based upon the Indonesian National Spheroid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378160 +b=6356774.50408554 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Indonesian Nationa");
    add_srs_wkt (p, 1,
		 "l Spheroid\",DATUM[\"Not_specified_based_on_Indonesian_N");
    add_srs_wkt (p, 2,
		 "ational_Spheroid\",SPHEROID[\"Indonesian National Sphero");
    add_srs_wkt (p, 3,
		 "id\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6021\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"402");
    add_srs_wkt (p, 7, "1\"]]");
    p = add_epsg_def (first, last, 4022, "epsg", 4022,
		      "Unknown datum based upon the International 1924 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the International 1924");
    add_srs_wkt (p, 1,
		 " ellipsoid\",DATUM[\"Not_specified_based_on_Internationa");
    add_srs_wkt (p, 2,
		 "l_1924_ellipsoid\",SPHEROID[\"International 1924\",63783");
    add_srs_wkt (p, 3,
		 "88,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6022\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4022\"]]");
    p = add_epsg_def (first, last, 4023, "epsg", 4023, "MOLDREF99");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"MOLDREF99\",DATUM[\"MOLDREF99\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 1,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 2,
		 "]],AUTHORITY[\"EPSG\",\"1032\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 4,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5, "\",\"4023\"]]");
    p = add_epsg_def (first, last, 4024, "epsg", 4024,
		      "Unknown datum based upon the Krassowsky 1940 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=krass +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Krassowsky 1940 el");
    add_srs_wkt (p, 1,
		 "lipsoid\",DATUM[\"Not_specified_based_on_Krassowsky_1940");
    add_srs_wkt (p, 2,
		 "_ellipsoid\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6024\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "9122\"]],AUTHORITY[\"EPSG\",\"4024\"]]");
    p = add_epsg_def (first, last, 4025, "epsg", 4025,
		      "Unknown datum based upon the NWL 9D ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the NWL 9D ellipsoid\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Not_specified_based_on_NWL_9D_ellipsoid\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"NWL 9D\",6378145,298.25,AUTHORITY[\"EPSG\",\"7025\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6025\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6, "\",\"4025\"]]");
    p = add_epsg_def (first, last, 4026, "epsg", 4026,
		      "MOLDREF99 / Moldova TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28.4 +k=0.9999400000000001 +");
    add_proj4text (p, 1,
		   "x_0=200000 +y_0=-5000000 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MOLDREF99 / Moldova TM\",GEOGCS[\"MOLDREF99\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"MOLDREF99\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 2,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"1032\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4023\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",28.4],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 9,
		 "ctor\",0.99994],PARAMETER[\"false_easting\",200000],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_northing\",-5000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "4026\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 4027, "epsg", 4027,
		      "Unknown datum based upon the Plessis 1817 ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6376523 +b=6355862.933255573 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Plessis 1817 ellip");
    add_srs_wkt (p, 1,
		 "soid\",DATUM[\"Not_specified_based_on_Plessis_1817_ellip");
    add_srs_wkt (p, 2,
		 "soid\",SPHEROID[\"Plessis 1817\",6376523,308.64,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7027\"]],AUTHORITY[\"EPSG\",\"6027\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6, "]],AUTHORITY[\"EPSG\",\"4027\"]]");
    p = add_epsg_def (first, last, 4028, "epsg", 4028,
		      "Unknown datum based upon the Struve 1860 ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378298.3 +b=6356657.142669561 +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Struve 1860 ellips");
    add_srs_wkt (p, 1,
		 "oid\",DATUM[\"Not_specified_based_on_Struve_1860_ellipso");
    add_srs_wkt (p, 2,
		 "id\",SPHEROID[\"Struve 1860\",6378298.3,294.73,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7028\"]],AUTHORITY[\"EPSG\",\"6028\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6, "],AUTHORITY[\"EPSG\",\"4028\"]]");
    p = add_epsg_def (first, last, 4029, "epsg", 4029,
		      "Unknown datum based upon the War Office ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378300 +b=6356751.689189189 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the War Office ellipso");
    add_srs_wkt (p, 1,
		 "id\",DATUM[\"Not_specified_based_on_War_Office_ellipsoid");
    add_srs_wkt (p, 2,
		 "\",SPHEROID[\"War Office\",6378300,296,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7029\"]],AUTHORITY[\"EPSG\",\"6029\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6, "TY[\"EPSG\",\"4029\"]]");
    p = add_epsg_def (first, last, 4030, "epsg", 4030,
		      "Unknown datum based upon the WGS 84 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS84 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the WGS 84 ellipsoid\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Not_specified_based_on_WGS_84_ellipsoid\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7030\"]],AUTHORITY[\"EPSG\",\"6030\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6, "Y[\"EPSG\",\"4030\"]]");
    p = add_epsg_def (first, last, 4031, "epsg", 4031,
		      "Unknown datum based upon the GEM 10C ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS84 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the GEM 10C ellipsoid\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Not_specified_based_on_GEM_10C_ellipsoid\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GEM 10C\",6378137,298.257223563,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7031\"]],AUTHORITY[\"EPSG\",\"6031\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6, "TY[\"EPSG\",\"4031\"]]");
    p = add_epsg_def (first, last, 4032, "epsg", 4032,
		      "Unknown datum based upon the OSU86F ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378136.2 +b=6356751.516927429 +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the OSU86F ellipsoid\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Not_specified_based_on_OSU86F_ellipsoid\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"OSU86F\",6378136.2,298.257223563,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7032\"]],AUTHORITY[\"EPSG\",\"6032\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6, "TY[\"EPSG\",\"4032\"]]");
    p = add_epsg_def (first, last, 4033, "epsg", 4033,
		      "Unknown datum based upon the OSU91A ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378136.3 +b=6356751.616592146 +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the OSU91A ellipsoid\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Not_specified_based_on_OSU91A_ellipsoid\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"OSU91A\",6378136.3,298.257223563,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7033\"]],AUTHORITY[\"EPSG\",\"6033\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6, "TY[\"EPSG\",\"4033\"]]");
    p = add_epsg_def (first, last, 4034, "epsg", 4034,
		      "Unknown datum based upon the Clarke 1880 ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.144808011 +b=6356514.966204134 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Clarke 1880 ellips");
    add_srs_wkt (p, 1,
		 "oid\",DATUM[\"Not_specified_based_on_Clarke_1880_ellipso");
    add_srs_wkt (p, 2,
		 "id\",SPHEROID[\"Clarke 1880\",6378249.144808011,293.4663");
    add_srs_wkt (p, 3,
		 "076556349,AUTHORITY[\"EPSG\",\"7034\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6034\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4034\"]]");
    p = add_epsg_def (first, last, 4035, "epsg", 4035,
		      "Unknown datum based upon the Authalic Sphere");
    add_proj4text (p, 0, "+proj=longlat +a=6371000 +b=6371000 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Authalic Sphere\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Not_specified_based_on_Authalic_Sphere\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Sphere\",6371000,0,AUTHORITY[\"EPSG\",\"7035\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6035\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "33,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"40");
    add_srs_wkt (p, 6, "35\"]]");
    p = add_epsg_def (first, last, 4036, "epsg", 4036,
		      "Unknown datum based upon the GRS 1967 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS67 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the GRS 1967 ellipsoid");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Not_specified_based_on_GRS_1967_ellipsoid\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1967\",6378160,298.247167427,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7036\"]],AUTHORITY[\"EPSG\",\"6036\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6, "THORITY[\"EPSG\",\"4036\"]]");
    p = add_epsg_def (first, last, 4037, "epsg", 4037, "WGS 84 / TMzn35N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / TMzn35N\",GEOGCS[\"WGS 84\",DATUM[\"WG");
    add_srs_wkt (p, 1,
		 "S_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5,
		 "2\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 7,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 8,
		 "entral_meridian\",27],PARAMETER[\"scale_factor\",0.9996]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 10,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"4037\"],AXIS[\"Northin");
    add_srs_wkt (p, 11, "g\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 4038, "epsg", 4038, "WGS 84 / TMzn36N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / TMzn36N\",GEOGCS[\"WGS 84\",DATUM[\"WG");
    add_srs_wkt (p, 1,
		 "S_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5,
		 "2\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 7,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 8,
		 "entral_meridian\",33],PARAMETER[\"scale_factor\",0.9996]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 10,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"4038\"],AXIS[\"Northin");
    add_srs_wkt (p, 11, "g\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 4041, "epsg", 4041,
		      "Unknown datum based upon the Average Terrestrial System 1977 ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378135 +b=6356750.304921594 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Average Terrestria");
    add_srs_wkt (p, 1,
		 "l System 1977 ellipsoid\",DATUM[\"Not_specified_based_on");
    add_srs_wkt (p, 2,
		 "_Average_Terrestrial_System_1977_ellipsoid\",SPHEROID[\"");
    add_srs_wkt (p, 3,
		 "Average Terrestrial System 1977\",6378135,298.257,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"7041\"]],AUTHORITY[\"EPSG\",\"6041\"]],PR");
    add_srs_wkt (p, 5,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 7, "\"]],AUTHORITY[\"EPSG\",\"4041\"]]");
    p = add_epsg_def (first, last, 4042, "epsg", 4042,
		      "Unknown datum based upon the Everest (1830 Definition) ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377299.36559538 +b=6356098.359005156 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Everest (1830 Defi");
    add_srs_wkt (p, 1,
		 "nition) ellipsoid\",DATUM[\"Not_specified_based_on_Evere");
    add_srs_wkt (p, 2,
		 "st_1830_Definition_ellipsoid\",SPHEROID[\"Everest (1830 ");
    add_srs_wkt (p, 3,
		 "Definition)\",6377299.36559538,300.8017255433552,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"7042\"]],AUTHORITY[\"EPSG\",\"6042\"]],PRI");
    add_srs_wkt (p, 5,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 6,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 7, "]],AUTHORITY[\"EPSG\",\"4042\"]]");
    p = add_epsg_def (first, last, 4043, "epsg", 4043,
		      "Unknown datum based upon the WGS 72 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS72 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the WGS 72 ellipsoid\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Not_specified_based_on_WGS_72_ellipsoid\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6043\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6, "\",\"4043\"]]");
    p = add_epsg_def (first, last, 4044, "epsg", 4044,
		      "Unknown datum based upon the Everest 1830 (1962 Definition) ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377301.243 +b=6356100.230165384 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Everest 1830 (1962");
    add_srs_wkt (p, 1,
		 " Definition) ellipsoid\",DATUM[\"Not_specified_based_on_");
    add_srs_wkt (p, 2,
		 "Everest_1830_1962_Definition_ellipsoid\",SPHEROID[\"Ever");
    add_srs_wkt (p, 3,
		 "est 1830 (1962 Definition)\",6377301.243,300.8017255,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"7044\"]],AUTHORITY[\"EPSG\",\"6044\"]]");
    add_srs_wkt (p, 5,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7, "122\"]],AUTHORITY[\"EPSG\",\"4044\"]]");
    p = add_epsg_def (first, last, 4045, "epsg", 4045,
		      "Unknown datum based upon the Everest 1830 (1975 Definition) ellipsoid");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377299.151 +b=6356098.145120132 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unknown datum based upon the Everest 1830 (1975");
    add_srs_wkt (p, 1,
		 " Definition) ellipsoid\",DATUM[\"Not_specified_based_on_");
    add_srs_wkt (p, 2,
		 "Everest_1830_1975_Definition_ellipsoid\",SPHEROID[\"Ever");
    add_srs_wkt (p, 3,
		 "est 1830 (1975 Definition)\",6377299.151,300.8017255,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"7045\"]],AUTHORITY[\"EPSG\",\"6045\"]]");
    add_srs_wkt (p, 5,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7, "122\"]],AUTHORITY[\"EPSG\",\"4045\"]]");
    p = add_epsg_def (first, last, 4046, "epsg", 4046, "RGRDC 2005");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RGRDC 2005\",DATUM[\"Reseau_Geodesique_de_la_RD");
    add_srs_wkt (p, 1,
		 "C_2005\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046");
    add_srs_wkt (p, 6, "\"]]");
    p = add_epsg_def (first, last, 4047, "epsg", 4047,
		      "Unspecified datum based upon the GRS 1980 Authalic Sphere");
    add_proj4text (p, 0, "+proj=longlat +a=6371007 +b=6371007 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unspecified datum based upon the GRS 1980 Autha");
    add_srs_wkt (p, 1,
		 "lic Sphere\",DATUM[\"Not_specified_based_on_GRS_1980_Aut");
    add_srs_wkt (p, 2,
		 "halic_Sphere\",SPHEROID[\"GRS 1980 Authalic Sphere\",637");
    add_srs_wkt (p, 3,
		 "1007,0,AUTHORITY[\"EPSG\",\"7048\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6047\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4047\"]]");
    p = add_epsg_def (first, last, 4048, "epsg", 4048,
		      "RGRDC 2005 / Congo TM zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 12\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",12],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4048\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4049, "epsg", 4049,
		      "RGRDC 2005 / Congo TM zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=14 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 14\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",14],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4049\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4050, "epsg", 4050,
		      "RGRDC 2005 / Congo TM zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 16\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",16],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4050\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4051, "epsg", 4051,
		      "RGRDC 2005 / Congo TM zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 18\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",18],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4051\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4052, "epsg", 4052,
		      "Unspecified datum based upon the Clarke 1866 Authalic Sphere");
    add_proj4text (p, 0, "+proj=longlat +a=6370997 +b=6370997 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unspecified datum based upon the Clarke 1866 Au");
    add_srs_wkt (p, 1,
		 "thalic Sphere\",DATUM[\"Not_specified_based_on_Clarke_18");
    add_srs_wkt (p, 2,
		 "66_Authalic_Sphere\",SPHEROID[\"Clarke 1866 Authalic Sph");
    add_srs_wkt (p, 3,
		 "ere\",6370997,0,AUTHORITY[\"EPSG\",\"7052\"]],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6052\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4052\"]]");
    add_srs_wkt (p, 7, "");
    p = add_epsg_def (first, last, 4053, "epsg", 4053,
		      "Unspecified datum based upon the International 1924 Authalic Sphere");
    add_proj4text (p, 0, "+proj=longlat +a=6371228 +b=6371228 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unspecified datum based upon the International ");
    add_srs_wkt (p, 1,
		 "1924 Authalic Sphere\",DATUM[\"Not_specified_based_on_In");
    add_srs_wkt (p, 2,
		 "ternational_1924_Authalic_Sphere\",SPHEROID[\"Internatio");
    add_srs_wkt (p, 3,
		 "nal 1924 Authalic Sphere\",6371228,0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"7057\"]],AUTHORITY[\"EPSG\",\"6053\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7, "Y[\"EPSG\",\"4053\"]]");
    p = add_epsg_def (first, last, 4054, "epsg", 4054,
		      "Unspecified datum based upon the Hughes 1980 ellipsoid");
    add_proj4text (p, 0, "+proj=longlat +a=6378273 +b=6356889.449 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Unspecified datum based upon the Hughes 1980 el");
    add_srs_wkt (p, 1,
		 "lipsoid\",DATUM[\"Not_specified_based_on_Hughes_1980_ell");
    add_srs_wkt (p, 2,
		 "ipsoid\",SPHEROID[\"Hughes 1980\",6378273,298.2794111230");
    add_srs_wkt (p, 3,
		 "61,AUTHORITY[\"EPSG\",\"7058\"]],AUTHORITY[\"EPSG\",\"60");
    add_srs_wkt (p, 4,
		 "54\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4054\"]]");
    p = add_epsg_def (first, last, 4055, "epsg", 4055,
		      "Popular Visualisation CRS");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378137 +b=6378137 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 1, ",0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Popular Visualisation CRS\",DATUM[\"Popular_Vis");
    add_srs_wkt (p, 1,
		 "ualisation_Datum\",SPHEROID[\"Popular Visualisation Sphe");
    add_srs_wkt (p, 2,
		 "re\",6378137,0,AUTHORITY[\"EPSG\",\"7059\"]],TOWGS84[0,0");
    add_srs_wkt (p, 3,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6055\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6, "TY[\"EPSG\",\"4055\"]]");
    p = add_epsg_def (first, last, 4056, "epsg", 4056,
		      "RGRDC 2005 / Congo TM zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=20 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 20\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",20],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4056\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4057, "epsg", 4057,
		      "RGRDC 2005 / Congo TM zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=22 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 22\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",22],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4057\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4058, "epsg", 4058,
		      "RGRDC 2005 / Congo TM zone 24");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 24\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",24],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4058\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4059, "epsg", 4059,
		      "RGRDC 2005 / Congo TM zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=26 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 26\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",26],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4059\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4060, "epsg", 4060,
		      "RGRDC 2005 / Congo TM zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28 +k=0.9999 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / Congo TM zone 28\",GEOGCS[\"RGRDC ");
    add_srs_wkt (p, 1,
		 "2005\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "1033\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",28],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9999],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"4060\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4061, "epsg", 4061,
		      "RGRDC 2005 / UTM zone 33S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / UTM zone 33S\",GEOGCS[\"RGRDC 2005");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1033\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",15],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"4061\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4062, "epsg", 4062,
		      "RGRDC 2005 / UTM zone 34S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / UTM zone 34S\",GEOGCS[\"RGRDC 2005");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1033\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",21],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"4062\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4063, "epsg", 4063,
		      "RGRDC 2005 / UTM zone 35S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RGRDC 2005 / UTM zone 35S\",GEOGCS[\"RGRDC 2005");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Reseau_Geodesique_de_la_RDC_2005\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "19\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1033\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4046\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",27],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"4063\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 4071, "epsg", 4071, "Chua / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Chua / UTM zone 23S\",GEOGCS[\"Chua\",DATUM[\"C");
    add_srs_wkt (p, 1,
		 "hua\",SPHEROID[\"International 1924\",6378388,297,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6224\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5,
		 "\"]],AUTHORITY[\"EPSG\",\"4224\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 6,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 7,
		 "or\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"ce");
    add_srs_wkt (p, 8,
		 "ntral_meridian\",-45],PARAMETER[\"scale_factor\",0.9996]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 10,
		 "orthing\",10000000],AUTHORITY[\"EPSG\",\"4071\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 4075, "epsg", 4075, "SREF98");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"SREF98\",DATUM[\"Serbian_Reference_Network_1998");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"1034\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4075\"]]");
    p = add_epsg_def (first, last, 4081, "epsg", 4081, "REGCAN95");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"REGCAN95\",DATUM[\"Red_Geodesica_de_Canarias_19");
    add_srs_wkt (p, 1,
		 "95\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"1035\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4081\"]]");
    add_srs_wkt (p, 6, "");
    p = add_epsg_def (first, last, 4082, "epsg", 4082,
		      "REGCAN95 / UTM zone 27N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"REGCAN95 / UTM zone 27N\",GEOGCS[\"REGCAN95\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Red_Geodesica_de_Canarias_1995\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1035\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4081\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-21],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",0],AUTHORITY[\"EPSG\",\"4082\"],AXIS[\"Eastin");
    add_srs_wkt (p, 12, "g\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 4083, "epsg", 4083,
		      "REGCAN95 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"REGCAN95 / UTM zone 28N\",GEOGCS[\"REGCAN95\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Red_Geodesica_de_Canarias_1995\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 2,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"1035\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4081\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-15],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",0],AUTHORITY[\"EPSG\",\"4083\"],AXIS[\"Eastin");
    add_srs_wkt (p, 12, "g\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 4120, "epsg", 4120, "Greek");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Greek\",DATUM[\"Greek\",SPHEROID[\"Bessel 1841\"");
    add_srs_wkt (p, 1,
		 ",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"6120\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 4,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "4120\"]]");
    p = add_epsg_def (first, last, 4121, "epsg", 4121, "GGRS87");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +datum=GGRS87 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"GGRS87\",DATUM[\"Greek_Geodetic_Reference_Syste");
    add_srs_wkt (p, 1,
		 "m_1987\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7019\"]],TOWGS84[-199.87,74.79,246.62,");
    add_srs_wkt (p, 3,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6121\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6, "\"EPSG\",\"4121\"]]");
    p = add_epsg_def (first, last, 4122, "epsg", 4122, "ATS77");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378135 +b=6356750.304921594 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ATS77\",DATUM[\"Average_Terrestrial_System_1977");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"Average Terrestrial System 1977\",6378135,");
    add_srs_wkt (p, 2,
		 "298.257,AUTHORITY[\"EPSG\",\"7041\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6122\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4122\"]]");
    p = add_epsg_def (first, last, 4123, "epsg", 4123, "KKJ");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"KKJ\",DATUM[\"Kartastokoordinaattijarjestelma_1");
    add_srs_wkt (p, 1,
		 "966\",SPHEROID[\"International 1924\",6378388,297,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6123\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4123\"]]");
    p = add_epsg_def (first, last, 4124, "epsg", 4124, "RT90");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RT90\",DATUM[\"Rikets_koordinatsystem_1990\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6124\"]],PRIME");
    add_srs_wkt (p, 3,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 4,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 5, "],AUTHORITY[\"EPSG\",\"4124\"]]");
    p = add_epsg_def (first, last, 4125, "epsg", 4125, "Samboja");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=-404.78,685.68,45.4");
    add_proj4text (p, 1, "7,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Samboja\",DATUM[\"Samboja\",SPHEROID[\"Bessel 1");
    add_srs_wkt (p, 1,
		 "841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[-404.78,685.68,45.47,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6125\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY");
    add_srs_wkt (p, 5, "[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"4125\"]]");
    p = add_epsg_def (first, last, 4126, "epsg", 4126, "LKS94 (ETRS89)");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"LKS94 (ETRS89)\",DATUM[\"Lithuania_1994_ETRS89\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6126\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4126\"]]");
    p = add_epsg_def (first, last, 4127, "epsg", 4127, "Tete");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tete\",DATUM[\"Tete\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 1,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 2,
		 "],AUTHORITY[\"EPSG\",\"6127\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 4,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4127\"]]");
    p = add_epsg_def (first, last, 4128, "epsg", 4128, "Madzansua");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Madzansua\",DATUM[\"Madzansua\",SPHEROID[\"Clar");
    add_srs_wkt (p, 1,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7008\"]],AUTHORITY[\"EPSG\",\"6128\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4128\"]]");
    p = add_epsg_def (first, last, 4129, "epsg", 4129, "Observatario");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Observatario\",DATUM[\"Observatario\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6129\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4129\"]]");
    p = add_epsg_def (first, last, 4130, "epsg", 4130, "Moznet");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,-0,-0,-0,0 +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Moznet\",DATUM[\"Moznet_ITRF94\",SPHEROID[\"WGS");
    add_srs_wkt (p, 1,
		 " 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]");
    add_srs_wkt (p, 2,
		 "],TOWGS84[0,0,0,-0,-0,-0,0],AUTHORITY[\"EPSG\",\"6130\"]");
    add_srs_wkt (p, 3,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4130\"]]");
    p = add_epsg_def (first, last, 4131, "epsg", 4131, "Indian 1960");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377276.345 +b=6356075.41314024 +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Indian 1960\",DATUM[\"Indian_1960\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Everest 1830 (1937 Adjustment)\",6377276.345,300.8017,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6131\"]");
    add_srs_wkt (p, 3,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4131\"]]");
    p = add_epsg_def (first, last, 4132, "epsg", 4132, "FD58");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"FD58\",DATUM[\"Final_Datum_1958\",SPHEROID[\"Cl");
    add_srs_wkt (p, 1,
		 "arke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7012\"]],AUTHORITY[\"EPSG\",\"6132\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4132\"]]");
    p = add_epsg_def (first, last, 4133, "epsg", 4133, "EST92");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0.055,-0.541,-0.185,");
    add_proj4text (p, 1, "0.0183,-0.0003,-0.007,-0.014 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"EST92\",DATUM[\"Estonia_1992\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 1,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 2,
		 "],TOWGS84[0.055,-0.541,-0.185,0.0183,-0.0003,-0.007,-0.0");
    add_srs_wkt (p, 3,
		 "14],AUTHORITY[\"EPSG\",\"6133\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 5,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 6, "G\",\"4133\"]]");
    p = add_epsg_def (first, last, 4134, "epsg", 4134, "PSD93");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"PSD93\",DATUM[\"PDO_Survey_Datum_1993\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6134\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4134\"]]");
    p = add_epsg_def (first, last, 4135, "epsg", 4135, "Old Hawaiian");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Old Hawaiian\",DATUM[\"Old_Hawaiian\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6135\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4135\"]]");
    p = add_epsg_def (first, last, 4136, "epsg", 4136, "St. Lawrence Island");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"St. Lawrence Island\",DATUM[\"St_Lawrence_Islan");
    add_srs_wkt (p, 1,
		 "d\",SPHEROID[\"Clarke 1866\",6378206.4,294.9786982139006");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6136");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4136\"]]");
    p = add_epsg_def (first, last, 4137, "epsg", 4137, "St. Paul Island");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"St. Paul Island\",DATUM[\"St_Paul_Island\",SPHE");
    add_srs_wkt (p, 1,
		 "ROID[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6137\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5, "]],AUTHORITY[\"EPSG\",\"4137\"]]");
    p = add_epsg_def (first, last, 4138, "epsg", 4138, "St. George Island");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"St. George Island\",DATUM[\"St_George_Island\",");
    add_srs_wkt (p, 1,
		 "SPHEROID[\"Clarke 1866\",6378206.4,294.9786982139006,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6138\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 4,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 5, "122\"]],AUTHORITY[\"EPSG\",\"4138\"]]");
    p = add_epsg_def (first, last, 4139, "epsg", 4139, "Puerto Rico");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=11,72,-101,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Puerto Rico\",DATUM[\"Puerto_Rico\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7008\"]],TOWGS84[11,72,-101,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6139\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4139\"]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_15 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 4140, "epsg", 4140, "NAD83(CSRS98)");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD83(CSRS98)\",DATUM[\"NAD83_Canadian_Spatial_");
    add_srs_wkt (p, 1,
		 "Reference_System\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 2,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 3,
		 ",0],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 5,
		 "925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6, "\",\"4140\"]]");
    p = add_epsg_def (first, last, 4141, "epsg", 4141, "Israel");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=-48,55,52,0,0,0,0 +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Israel\",DATUM[\"Israel\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 1,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 2,
		 "GS84[-48,55,52,0,0,0,0],AUTHORITY[\"EPSG\",\"6141\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4141\"]]");
    p = add_epsg_def (first, last, 4142, "epsg", 4142, "Locodjo 1965");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-125,53,467,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Locodjo 1965\",DATUM[\"Locodjo_1965\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],TOWGS84[-125,53,467,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6142\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4142\"]]");
    p = add_epsg_def (first, last, 4143, "epsg", 4143, "Abidjan 1987");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-124.76,53,466.79,0");
    add_proj4text (p, 1, ",0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Abidjan 1987\",DATUM[\"Abidjan_1987\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],TOWGS84[-124.76,53,466.79,0,0,0,0],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6143\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 6, "3\"]]");
    p = add_epsg_def (first, last, 4144, "epsg", 4144, "Kalianpur 1937");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377276.345 +b=6356075.41314024 +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kalianpur 1937\",DATUM[\"Kalianpur_1937\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Everest 1830 (1937 Adjustment)\",6377276.345,300.8");
    add_srs_wkt (p, 2,
		 "017,AUTHORITY[\"EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "144\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5, "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4144\"]]");
    p = add_epsg_def (first, last, 4145, "epsg", 4145, "Kalianpur 1962");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377301.243 +b=6356100.230165384 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kalianpur 1962\",DATUM[\"Kalianpur_1962\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Everest 1830 (1962 Definition)\",6377301.243,300.8");
    add_srs_wkt (p, 2,
		 "017255,AUTHORITY[\"EPSG\",\"7044\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6145\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4145\"]]");
    p = add_epsg_def (first, last, 4146, "epsg", 4146, "Kalianpur 1975");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377299.151 +b=6356098.145120132 +towgs");
    add_proj4text (p, 1, "84=295,736,257,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kalianpur 1975\",DATUM[\"Kalianpur_1975\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Everest 1830 (1975 Definition)\",6377299.151,300.8");
    add_srs_wkt (p, 2,
		 "017255,AUTHORITY[\"EPSG\",\"7045\"]],TOWGS84[295,736,257");
    add_srs_wkt (p, 3,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 4,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 5,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 6, "[\"EPSG\",\"4146\"]]");
    p = add_epsg_def (first, last, 4147, "epsg", 4147, "Hanoi 1972");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=krass +towgs84=-17.51,-108.32,-62.3");
    add_proj4text (p, 1, "9,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hanoi 1972\",DATUM[\"Hanoi_1972\",SPHEROID[\"Kr");
    add_srs_wkt (p, 1,
		 "assowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[-17.51,-108.32,-62.39,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6147\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4147\"]]");
    p = add_epsg_def (first, last, 4148, "epsg", 4148, "Hartebeesthoek94");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hartebeesthoek94\",DATUM[\"Hartebeesthoek94\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6148\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4148\"]]");
    p = add_epsg_def (first, last, 4149, "epsg", 4149, "CH1903");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=674.374,15.056,405.");
    add_proj4text (p, 1, "346,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"CH1903\",DATUM[\"CH1903\",SPHEROID[\"Bessel 184");
    add_srs_wkt (p, 1,
		 "1\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]");
    add_srs_wkt (p, 2,
		 "],TOWGS84[674.374,15.056,405.346,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6149\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4149\"]]");
    p = add_epsg_def (first, last, 4150, "epsg", 4150, "CH1903+");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=674.374,15.056,405.");
    add_proj4text (p, 1, "346,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"CH1903+\",DATUM[\"CH1903\",SPHEROID[\"Bessel 18");
    add_srs_wkt (p, 1,
		 "41\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[674.374,15.056,405.346,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6150\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4150\"]]");
    p = add_epsg_def (first, last, 4151, "epsg", 4151, "CHTRF95");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"CHTRF95\",DATUM[\"Swiss_Terrestrial_Reference_F");
    add_srs_wkt (p, 1,
		 "rame_1995\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6151\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6, "151\"]]");
    p = add_epsg_def (first, last, 4152, "epsg", 4152, "NAD83(HARN)");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regio");
    add_srs_wkt (p, 1,
		 "nal_Network\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 2,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"615");
    add_srs_wkt (p, 3,
		 "2\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]]");
    p = add_epsg_def (first, last, 4153, "epsg", 4153, "Rassadiran");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-133.63,-157.5,-158.6");
    add_proj4text (p, 1, "2,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Rassadiran\",DATUM[\"Rassadiran\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],TOWGS84[-133.63,-157.5,-158.62,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6153\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4153\"]]");
    p = add_epsg_def (first, last, 4154, "epsg", 4154, "ED50(ED77)");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ED50(ED77)\",DATUM[\"European_Datum_1950_1977\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6154\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4154\"]]");
    p = add_epsg_def (first, last, 4155, "epsg", 4155, "Dabola 1981");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-83,37,12");
    add_proj4text (p, 1, "4,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Dabola 1981\",DATUM[\"Dabola_1981\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Clarke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7011\"]],TOWGS84[-83,37,124,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6155\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4155");
    add_srs_wkt (p, 6, "\"]]");
    p = add_epsg_def (first, last, 4156, "epsg", 4156, "S-JTSK");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"S-JTSK\",DATUM[\"Jednotne_Trigonometricke_Site_");
    add_srs_wkt (p, 1,
		 "Katastralni\",SPHEROID[\"Bessel 1841\",6377397.155,299.1");
    add_srs_wkt (p, 2,
		 "528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6156\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4156\"]]");
    p = add_epsg_def (first, last, 4157, "epsg", 4157, "Mount Dillon");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378293.645208759 +b=6356617.987679838 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mount Dillon\",DATUM[\"Mount_Dillon\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1858\",6378293.645208759,294.2606763692569,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7007\"]],AUTHORITY[\"EPSG\",\"6157\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5, "22\"]],AUTHORITY[\"EPSG\",\"4157\"]]");
    p = add_epsg_def (first, last, 4158, "epsg", 4158, "Naparima 1955");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Naparima 1955\",DATUM[\"Naparima_1955\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7022\"]],AUTHORITY[\"EPSG\",\"6158\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 3,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 4,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4158\"]]");
    p = add_epsg_def (first, last, 4159, "epsg", 4159, "ELD79");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ELD79\",DATUM[\"European_Libyan_Datum_1979\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"International 1924\",6378388,297,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6159\"]],PRIMEM[\"G");
    add_srs_wkt (p, 3,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 4,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4159\"]]");
    p = add_epsg_def (first, last, 4160, "epsg", 4160, "Chos Malal 1914");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Chos Malal 1914\",DATUM[\"Chos_Malal_1914\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6160\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4160\"]]");
    p = add_epsg_def (first, last, 4161, "epsg", 4161, "Pampa del Castillo");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=27.5,14,186.4,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pampa del Castillo\",DATUM[\"Pampa_del_Castillo");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],TOWGS84[27.5,14,186.4,0,0,0,0],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6161\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"41");
    add_srs_wkt (p, 6, "61\"]]");
    p = add_epsg_def (first, last, 4162, "epsg", 4162, "Korean 1985");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Korean 1985\",DATUM[\"Korean_Datum_1985\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6162\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4162\"]]");
    p = add_epsg_def (first, last, 4163, "epsg", 4163, "Yemen NGN96");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Yemen NGN96\",DATUM[\"Yemen_National_Geodetic_N");
    add_srs_wkt (p, 1,
		 "etwork_1996\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6163\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6, "163\"]]");
    p = add_epsg_def (first, last, 4164, "epsg", 4164, "South Yemen");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=krass +towgs84=-76,-138,67,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"South Yemen\",DATUM[\"South_Yemen\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 2,
		 "4\"]],TOWGS84[-76,-138,67,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6164\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4164\"]]");
    p = add_epsg_def (first, last, 4165, "epsg", 4165, "Bissau");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-173,253,27,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bissau\",DATUM[\"Bissau\",SPHEROID[\"Internatio");
    add_srs_wkt (p, 1,
		 "nal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOW");
    add_srs_wkt (p, 2,
		 "GS84[-173,253,27,0,0,0,0],AUTHORITY[\"EPSG\",\"6165\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5, "22\"]],AUTHORITY[\"EPSG\",\"4165\"]]");
    p = add_epsg_def (first, last, 4166, "epsg", 4166, "Korean 1995");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Korean 1995\",DATUM[\"Korean_Datum_1995\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "166\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5, "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4166\"]]");
    p = add_epsg_def (first, last, 4167, "epsg", 4167, "NZGD2000");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NZGD2000\",DATUM[\"New_Zealand_Geodetic_Datum_2");
    add_srs_wkt (p, 1,
		 "000\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6167\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4167\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4168, "epsg", 4168, "Accra");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378300 +b=6356751.689189189 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Accra\",DATUM[\"Accra\",SPHEROID[\"War Office\"");
    add_srs_wkt (p, 1,
		 ",6378300,296,AUTHORITY[\"EPSG\",\"7029\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"6168\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 4, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4168\"]]");
    p = add_epsg_def (first, last, 4169, "epsg", 4169, "American Samoa 1962");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=-115,118,426,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"American Samoa 1962\",DATUM[\"American_Samoa_19");
    add_srs_wkt (p, 1,
		 "62\",SPHEROID[\"Clarke 1866\",6378206.4,294.978698213900");
    add_srs_wkt (p, 2,
		 "6,AUTHORITY[\"EPSG\",\"7008\"]],TOWGS84[-115,118,426,0,0");
    add_srs_wkt (p, 3,
		 ",0,0],AUTHORITY[\"EPSG\",\"6169\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6, "PSG\",\"4169\"]]");
    p = add_epsg_def (first, last, 4170, "epsg", 4170, "SIRGAS 1995");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"SIRGAS 1995\",DATUM[\"Sistema_de_Referencia_Geo");
    add_srs_wkt (p, 1,
		 "centrico_para_America_del_Sur_1995\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6170\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6, "]],AUTHORITY[\"EPSG\",\"4170\"]]");
    p = add_epsg_def (first, last, 4171, "epsg", 4171, "RGF93");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RGF93\",DATUM[\"Reseau_Geodesique_Francais_1993");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6171\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4171\"]]");
    p = add_epsg_def (first, last, 4172, "epsg", 4172, "POSGAR");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"POSGAR\",DATUM[\"Posiciones_Geodesicas_Argentin");
    add_srs_wkt (p, 1,
		 "as\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6172\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"4172\"]]");
    p = add_epsg_def (first, last, 4173, "epsg", 4173, "IRENET95");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IRENET95\",DATUM[\"IRENET95\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 1,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 2,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6173\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4173\"]]");
    p = add_epsg_def (first, last, 4174, "epsg", 4174, "Sierra Leone 1924");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378300 +b=6356751.689189189 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Sierra Leone 1924\",DATUM[\"Sierra_Leone_Colony");
    add_srs_wkt (p, 1,
		 "_1924\",SPHEROID[\"War Office\",6378300,296,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7029\"]],AUTHORITY[\"EPSG\",\"6174\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4174\"]]");
    p = add_epsg_def (first, last, 4175, "epsg", 4175, "Sierra Leone 1968");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-88,4,101,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Sierra Leone 1968\",DATUM[\"Sierra_Leone_1968\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7012\"]],TOWGS84[-88,4,101,0,0,0,0],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6175\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6, "175\"]]");
    p = add_epsg_def (first, last, 4176, "epsg", 4176, "Australian Antarctic");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Australian Antarctic\",DATUM[\"Australian_Antar");
    add_srs_wkt (p, 1,
		 "ctic_Datum_1998\",SPHEROID[\"GRS 1980\",6378137,298.2572");
    add_srs_wkt (p, 2,
		 "22101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,");
    add_srs_wkt (p, 3,
		 "0],AUTHORITY[\"EPSG\",\"6176\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6, "\",\"4176\"]]");
    p = add_epsg_def (first, last, 4178, "epsg", 4178, "Pulkovo 1942(83)");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=krass +towgs84=24,-123,-94,0.02,-0.");
    add_proj4text (p, 1, "25,-0.13,1.1 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pulkovo 1942(83)\",DATUM[\"Pulkovo_1942_83\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7024\"]],AUTHORITY[\"EPSG\",\"6178\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4178\"]]");
    p = add_epsg_def (first, last, 4179, "epsg", 4179, "Pulkovo 1942(58)");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=krass +towgs84=33.4,-146.6,-76.3,-0");
    add_proj4text (p, 1, ".359,-0.053,0.844,-0.84 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pulkovo 1942(58)\",DATUM[\"Pulkovo_1942_58\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7024\"]],AUTHORITY[\"EPSG\",\"6179\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4179\"]]");
    p = add_epsg_def (first, last, 4180, "epsg", 4180, "EST97");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"EST97\",DATUM[\"Estonia_1997\",SPHEROID[\"GRS 1");
    add_srs_wkt (p, 1,
		 "980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]");
    add_srs_wkt (p, 2,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6180\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5, "2\"]],AUTHORITY[\"EPSG\",\"4180\"]]");
    p = add_epsg_def (first, last, 4181, "epsg", 4181, "Luxembourg 1930");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-193,13.7,-39.3,-0.41");
    add_proj4text (p, 1, ",-2.933,2.688,0.43 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Luxembourg 1930\",DATUM[\"Luxembourg_1930\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],TOWGS84[-193,13.7,-39.3,-0.41,-2.933,2.68");
    add_srs_wkt (p, 3,
		 "8,0.43],AUTHORITY[\"EPSG\",\"6181\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6, "EPSG\",\"4181\"]]");
    p = add_epsg_def (first, last, 4182, "epsg", 4182,
		      "Azores Occidental 1939");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Azores Occidental 1939\",DATUM[\"Azores_Occiden");
    add_srs_wkt (p, 1,
		 "tal_Islands_1939\",SPHEROID[\"International 1924\",63783");
    add_srs_wkt (p, 2,
		 "88,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6182\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4182\"]]");
    p = add_epsg_def (first, last, 4183, "epsg", 4183, "Azores Central 1948");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-104,167,-38,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Azores Central 1948\",DATUM[\"Azores_Central_Is");
    add_srs_wkt (p, 1,
		 "lands_1948\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-104,167,-38,0,0,");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"6183\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6, "SG\",\"4183\"]]");
    p = add_epsg_def (first, last, 4184, "epsg", 4184, "Azores Oriental 1940");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-203,141,53,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Azores Oriental 1940\",DATUM[\"Azores_Oriental_");
    add_srs_wkt (p, 1,
		 "Islands_1940\",SPHEROID[\"International 1924\",6378388,2");
    add_srs_wkt (p, 2,
		 "97,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-203,141,53,0,0");
    add_srs_wkt (p, 3,
		 ",0,0],AUTHORITY[\"EPSG\",\"6184\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6, "PSG\",\"4184\"]]");
    p = add_epsg_def (first, last, 4185, "epsg", 4185, "Madeira 1936");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Madeira 1936\",DATUM[\"Madeira_1936\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6185\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "74532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4185\"]]");
    p = add_epsg_def (first, last, 4188, "epsg", 4188, "OSNI 1952");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=airy +towgs84=482.5,-130.6,564.6,-1");
    add_proj4text (p, 1, ".042,-0.214,-0.631,8.15 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"OSNI 1952\",DATUM[\"OSNI_1952\",SPHEROID[\"Airy");
    add_srs_wkt (p, 1,
		 " 1830\",6377563.396,299.3249646,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 2,
		 "1\"]],TOWGS84[482.5,-130.6,564.6,-1.042,-0.214,-0.631,8.");
    add_srs_wkt (p, 3,
		 "15],AUTHORITY[\"EPSG\",\"6188\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 5,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 6, "G\",\"4188\"]]");
    p = add_epsg_def (first, last, 4189, "epsg", 4189, "REGVEN");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"REGVEN\",DATUM[\"Red_Geodesica_Venezolana\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6189\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4189\"]]");
    p = add_epsg_def (first, last, 4190, "epsg", 4190, "POSGAR 98");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"POSGAR 98\",DATUM[\"Posiciones_Geodesicas_Argen");
    add_srs_wkt (p, 1,
		 "tinas_1998\",SPHEROID[\"GRS 1980\",6378137,298.257222101");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6190");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4190\"]]");
    p = add_epsg_def (first, last, 4191, "epsg", 4191, "Albanian 1987");
    add_proj4text (p, 0, "+proj=longlat +ellps=krass +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Albanian 1987\",DATUM[\"Albanian_1987\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7024\"]],AUTHORITY[\"EPSG\",\"6191\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4191\"]]");
    p = add_epsg_def (first, last, 4192, "epsg", 4192, "Douala 1948");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-206.1,-174.7,-87.7,0");
    add_proj4text (p, 1, ",0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Douala 1948\",DATUM[\"Douala_1948\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],TOWGS84[-206.1,-174.7,-87.7,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6192\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4192\"]]");
    p = add_epsg_def (first, last, 4193, "epsg", 4193, "Manoca 1962");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-70.9,-15");
    add_proj4text (p, 1, "1.8,-41.4,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Manoca 1962\",DATUM[\"Manoca_1962\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Clarke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7011\"]],TOWGS84[-70.9,-151.8,-41.4,0,0,0,0");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6193\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"4193\"]]");
    p = add_epsg_def (first, last, 4194, "epsg", 4194, "Qornoq 1927");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Qornoq 1927\",DATUM[\"Qornoq_1927\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],AUTHORITY[\"EPSG\",\"6194\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4194\"]]");
    p = add_epsg_def (first, last, 4195, "epsg", 4195, "Scoresbysund 1952");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=105,326,-102.5,0,0,0.");
    add_proj4text (p, 1, "814,-0.6 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Scoresbysund 1952\",DATUM[\"Scoresbysund_1952\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],TOWGS84[105,326,-102.5,0,0,0.814,-0.6]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6195\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"4195\"]]");
    p = add_epsg_def (first, last, 4196, "epsg", 4196, "Ammassalik 1958");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-45,417,-3.5,0,0,0.81");
    add_proj4text (p, 1, "4,-0.6 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Ammassalik 1958\",DATUM[\"Ammassalik_1958\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],TOWGS84[-45,417,-3.5,0,0,0.814,-0.6],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6196\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"41");
    add_srs_wkt (p, 6, "96\"]]");
    p = add_epsg_def (first, last, 4197, "epsg", 4197, "Garoua");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Garoua\",DATUM[\"Garoua\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 1,
		 "0 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"");
    add_srs_wkt (p, 2,
		 "]],AUTHORITY[\"EPSG\",\"6197\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 4,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5, "\",\"4197\"]]");
    p = add_epsg_def (first, last, 4198, "epsg", 4198, "Kousseri");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kousseri\",DATUM[\"Kousseri\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "012\"]],AUTHORITY[\"EPSG\",\"6198\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4198\"]]");
    p = add_epsg_def (first, last, 4199, "epsg", 4199, "Egypt 1930");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Egypt 1930\",DATUM[\"Egypt_1930\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],AUTHORITY[\"EPSG\",\"6199\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 3,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 4,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"4199\"]]");
    p = add_epsg_def (first, last, 4200, "epsg", 4200, "Pulkovo 1995");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=krass +towgs84=24.82,-131.21,-82.66");
    add_proj4text (p, 1, ",-0,-0,0.16,-0.12 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "4200\"]]");
    p = add_epsg_def (first, last, 4201, "epsg", 4201, "Adindan");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Adindan\",DATUM[\"Adindan\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 1,
		 "880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 2,
		 "2\"]],AUTHORITY[\"EPSG\",\"6201\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4201\"]]");
    p = add_epsg_def (first, last, 4202, "epsg", 4202, "AGD66");
    add_proj4text (p, 0, "+proj=longlat +ellps=aust_SA +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"AGD66\",DATUM[\"Australian_Geodetic_Datum_1966\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Australian National Spheroid\",6378160,298.2");
    add_srs_wkt (p, 2,
		 "5,AUTHORITY[\"EPSG\",\"7003\"]],AUTHORITY[\"EPSG\",\"620");
    add_srs_wkt (p, 3,
		 "2\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4202\"]]");
    p = add_epsg_def (first, last, 4203, "epsg", 4203, "AGD84");
    add_proj4text (p, 0, "+proj=longlat +ellps=aust_SA +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"AGD84\",DATUM[\"Australian_Geodetic_Datum_1984\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Australian National Spheroid\",6378160,298.2");
    add_srs_wkt (p, 2,
		 "5,AUTHORITY[\"EPSG\",\"7003\"]],AUTHORITY[\"EPSG\",\"620");
    add_srs_wkt (p, 3,
		 "3\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4203\"]]");
    p = add_epsg_def (first, last, 4204, "epsg", 4204, "Ain el Abd");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Ain el Abd\",DATUM[\"Ain_el_Abd_1970\",SPHEROID");
    add_srs_wkt (p, 1,
		 "[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4204\"]]");
    p = add_epsg_def (first, last, 4205, "epsg", 4205, "Afgooye");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=krass +towgs84=-43,-163,45,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Afgooye\",DATUM[\"Afgooye\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 1,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 2,
		 "WGS84[-43,-163,45,0,0,0,0],AUTHORITY[\"EPSG\",\"6205\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 4,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 5, "122\"]],AUTHORITY[\"EPSG\",\"4205\"]]");
    p = add_epsg_def (first, last, 4206, "epsg", 4206, "Agadez");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Agadez\",DATUM[\"Agadez\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 1,
		 "0 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7011\"]],AUTHORITY[\"EPSG\",\"6206\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4206\"]]");
    p = add_epsg_def (first, last, 4207, "epsg", 4207, "Lisbon");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lisbon\",DATUM[\"Lisbon_1937\",SPHEROID[\"Inter");
    add_srs_wkt (p, 1,
		 "national 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]");
    add_srs_wkt (p, 2,
		 "],AUTHORITY[\"EPSG\",\"6207\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 4,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4207\"]]");
    p = add_epsg_def (first, last, 4208, "epsg", 4208, "Aratu");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Aratu\",DATUM[\"Aratu\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 1,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"6208\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 4,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"420");
    add_srs_wkt (p, 5, "8\"]]");
    p = add_epsg_def (first, last, 4209, "epsg", 4209, "Arc 1950");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.145 +b=6356514.966398753 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Arc 1950\",DATUM[\"Arc_1950\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (Arc)\",6378249.145,293.4663077,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7013\"]],AUTHORITY[\"EPSG\",\"6209\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4209\"]]");
    p = add_epsg_def (first, last, 4210, "epsg", 4210, "Arc 1960");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Arc 1960\",DATUM[\"Arc_1960\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "012\"]],AUTHORITY[\"EPSG\",\"6210\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4210\"]]");
    p = add_epsg_def (first, last, 4211, "epsg", 4211, "Batavia");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Batavia\",DATUM[\"Batavia\",SPHEROID[\"Bessel 1");
    add_srs_wkt (p, 1,
		 "841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 2,
		 "]],AUTHORITY[\"EPSG\",\"6211\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 4,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5, "\",\"4211\"]]");
    p = add_epsg_def (first, last, 4212, "epsg", 4212, "Barbados 1938");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=31.95,300.99,419.19");
    add_proj4text (p, 1, ",0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Barbados 1938\",DATUM[\"Barbados_1938\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7012\"]],TOWGS84[31.95,300.99,419.19,0,0,0,0],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6212\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "4212\"]]");
    p = add_epsg_def (first, last, 4213, "epsg", 4213, "Beduaram");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-106,-87,");
    add_proj4text (p, 1, "188,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Beduaram\",DATUM[\"Beduaram\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7011\"]],TOWGS84[-106,-87,188,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6213\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4213\"]]");
    add_srs_wkt (p, 6, "");
    p = add_epsg_def (first, last, 4214, "epsg", 4214, "Beijing 1954");
    add_proj4text (p, 0, "+proj=longlat +ellps=krass +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4214\"]]");
    p = add_epsg_def (first, last, 4215, "epsg", 4215, "Belge 1950");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Belge 1950\",DATUM[\"Reseau_National_Belge_1950");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6215\"]],PRIME");
    add_srs_wkt (p, 3,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 4,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 5, "],AUTHORITY[\"EPSG\",\"4215\"]]");
    p = add_epsg_def (first, last, 4216, "epsg", 4216, "Bermuda 1957");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bermuda 1957\",DATUM[\"Bermuda_1957\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6216\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4216\"]]");
    p = add_epsg_def (first, last, 4218, "epsg", 4218, "Bogota 1975");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=307,304,-318,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bogota 1975\",DATUM[\"Bogota_1975\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],TOWGS84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6218\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4218\"]]");
    p = add_epsg_def (first, last, 4219, "epsg", 4219, "Bukit Rimpah");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=-384,664,-48,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bukit Rimpah\",DATUM[\"Bukit_Rimpah\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 2,
		 "\",\"7004\"]],TOWGS84[-384,664,-48,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6219\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4219\"]]");
    p = add_epsg_def (first, last, 4220, "epsg", 4220, "Camacupa");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Camacupa\",DATUM[\"Camacupa\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "012\"]],AUTHORITY[\"EPSG\",\"6220\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4220\"]]");
    p = add_epsg_def (first, last, 4221, "epsg", 4221, "Campo Inchauspe");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Campo Inchauspe\",DATUM[\"Campo_Inchauspe\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4221\"]]");
    p = add_epsg_def (first, last, 4222, "epsg", 4222, "Cape");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.145 +b=6356514.966398753 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Cape\",DATUM[\"Cape\",SPHEROID[\"Clarke 1880 (A");
    add_srs_wkt (p, 1,
		 "rc)\",6378249.145,293.4663077,AUTHORITY[\"EPSG\",\"7013\"");
    add_srs_wkt (p, 2,
		 "]],AUTHORITY[\"EPSG\",\"6222\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 4,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5, "\",\"4222\"]]");
    p = add_epsg_def (first, last, 4223, "epsg", 4223, "Carthage");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +datum=carthage +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Carthage\",DATUM[\"Carthage\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6223\"]],PRIMEM[\"G");
    add_srs_wkt (p, 3,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 4,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4223\"]]");
    p = add_epsg_def (first, last, 4224, "epsg", 4224, "Chua");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Chua\",DATUM[\"Chua\",SPHEROID[\"International ");
    add_srs_wkt (p, 1,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"6224\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4224\"");
    add_srs_wkt (p, 5, "]]");
    p = add_epsg_def (first, last, 4225, "epsg", 4225, "Corrego Alegre");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-206,172,-6,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Corrego Alegre\",DATUM[\"Corrego_Alegre\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7022\"]],TOWGS84[-206,172,-6,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6225\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4225\"]]");
    p = add_epsg_def (first, last, 4226, "epsg", 4226, "Cote d'Ivoire");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Cote d'Ivoire\",DATUM[\"Cote_d_Ivoire\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1880 (IGN)\",6378249.2,293.4660212936265,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6226\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"910");
    add_srs_wkt (p, 5, "8\"]],AUTHORITY[\"EPSG\",\"4226\"]]");
    p = add_epsg_def (first, last, 4227, "epsg", 4227, "Deir ez Zor");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Deir ez Zor\",DATUM[\"Deir_ez_Zor\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Clarke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6227\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5, "]],AUTHORITY[\"EPSG\",\"4227\"]]");
    p = add_epsg_def (first, last, 4228, "epsg", 4228, "Douala");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Douala\",DATUM[\"Douala\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 1,
		 "0 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7011\"]],AUTHORITY[\"EPSG\",\"6228\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4228\"]]");
    p = add_epsg_def (first, last, 4229, "epsg", 4229, "Egypt 1907");
    add_proj4text (p, 0, "+proj=longlat +ellps=helmert +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Egypt 1907\",DATUM[\"Egypt_1907\",SPHEROID[\"He");
    add_srs_wkt (p, 1,
		 "lmert 1906\",6378200,298.3,AUTHORITY[\"EPSG\",\"7020\"]]");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"6229\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 4,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4229\"]]");
    p = add_epsg_def (first, last, 4230, "epsg", 4230, "ED50");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ED50\",DATUM[\"European_Datum_1950\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],AUTHORITY[\"EPSG\",\"6230\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4230\"]]");
    p = add_epsg_def (first, last, 4231, "epsg", 4231, "ED87");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ED87\",DATUM[\"European_Datum_1987\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],AUTHORITY[\"EPSG\",\"6231\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4231\"]]");
    p = add_epsg_def (first, last, 4232, "epsg", 4232, "Fahud");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Fahud\",DATUM[\"Fahud\",SPHEROID[\"Clarke 1880 ");
    add_srs_wkt (p, 1,
		 "(RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]");
    add_srs_wkt (p, 2,
		 "],AUTHORITY[\"EPSG\",\"6232\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 4,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4232\"]]");
    p = add_epsg_def (first, last, 4233, "epsg", 4233, "Gandajika 1970");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-133,-321,50,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Gandajika 1970\",DATUM[\"Gandajika_1970\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7022\"]],TOWGS84[-133,-321,50,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6233\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4233\"]]");
    p = add_epsg_def (first, last, 4234, "epsg", 4234, "Garoua");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Garoua\",DATUM[\"Garoua\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 1,
		 "0 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7011\"]],AUTHORITY[\"EPSG\",\"6234\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4234\"]]");
    p = add_epsg_def (first, last, 4235, "epsg", 4235, "Guyane Francaise");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Guyane Francaise\",DATUM[\"Guyane_Francaise\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"International 1924\",6378388,297,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6235\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4235\"]]");
    p = add_epsg_def (first, last, 4236, "epsg", 4236, "Hu Tzu Shan 1950");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-637,-549,-203,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hu Tzu Shan 1950\",DATUM[\"Hu_Tzu_Shan_1950\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"International 1924\",6378388,297,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7022\"]],TOWGS84[-637,-549,-203,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6236\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4236\"");
    add_srs_wkt (p, 6, "]]");
    p = add_epsg_def (first, last, 4237, "epsg", 4237, "HD72");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS67 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"HD72\",DATUM[\"Hungarian_Datum_1972\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"GRS 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7036\"]],AUTHORITY[\"EPSG\",\"6237\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4237\"]]");
    p = add_epsg_def (first, last, 4238, "epsg", 4238, "ID74");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378160 +b=6356774.50408554 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ID74\",DATUM[\"Indonesian_Datum_1974\",SPHEROID");
    add_srs_wkt (p, 1,
		 "[\"Indonesian National Spheroid\",6378160,298.247,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7021\"]],AUTHORITY[\"EPSG\",\"6238\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4238\"]]");
    p = add_epsg_def (first, last, 4239, "epsg", 4239, "Indian 1954");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377276.345 +b=6356075.41314024 +towgs8");
    add_proj4text (p, 1, "4=217,823,299,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Indian 1954\",DATUM[\"Indian_1954\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Everest 1830 (1937 Adjustment)\",6377276.345,300.8017,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7015\"]],TOWGS84[217,823,299,0,0,0,0]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6239\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"4239\"]]");
    p = add_epsg_def (first, last, 4240, "epsg", 4240, "Indian 1975");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377276.345 +b=6356075.41314024 +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Indian 1975\",DATUM[\"Indian_1975\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Everest 1830 (1937 Adjustment)\",6377276.345,300.8017,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6240\"]");
    add_srs_wkt (p, 3,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4240\"]]");
    p = add_epsg_def (first, last, 4241, "epsg", 4241, "Jamaica 1875");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.144808011 +b=6356514.966204134 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Jamaica 1875\",DATUM[\"Jamaica_1875\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880\",6378249.144808011,293.4663076556349,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7034\"]],AUTHORITY[\"EPSG\",\"6241\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5, "22\"]],AUTHORITY[\"EPSG\",\"4241\"]]");
    p = add_epsg_def (first, last, 4242, "epsg", 4242, "JAD69");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"JAD69\",DATUM[\"Jamaica_1969\",SPHEROID[\"Clark");
    add_srs_wkt (p, 1,
		 "e 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7008\"]],AUTHORITY[\"EPSG\",\"6242\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 3,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 4,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4242\"]]");
    p = add_epsg_def (first, last, 4243, "epsg", 4243, "Kalianpur 1880");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377299.36559538 +b=6356098.359005156 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kalianpur 1880\",DATUM[\"Kalianpur_1880\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Everest (1830 Definition)\",6377299.36559538,300.8");
    add_srs_wkt (p, 2,
		 "017255433552,AUTHORITY[\"EPSG\",\"7042\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6243\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4243\"]]");
    p = add_epsg_def (first, last, 4244, "epsg", 4244, "Kandawala");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377276.345 +b=6356075.41314024 +towgs8");
    add_proj4text (p, 1, "4=-97,787,86,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kandawala\",DATUM[\"Kandawala\",SPHEROID[\"Ever");
    add_srs_wkt (p, 1,
		 "est 1830 (1937 Adjustment)\",6377276.345,300.8017,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7015\"]],TOWGS84[-97,787,86,0,0,0,0],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6244\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6, "44\"]]");
    p = add_epsg_def (first, last, 4245, "epsg", 4245, "Kertau 1968");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377304.063 +b=6356103.038993155 +towgs");
    add_proj4text (p, 1, "84=-11,851,5,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kertau 1968\",DATUM[\"Kertau_1968\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Everest 1830 Modified\",6377304.063,300.8017,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7018\"]],TOWGS84[-11,851,5,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6245\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4245\"]]");
    p = add_epsg_def (first, last, 4246, "epsg", 4246, "KOC");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-294.7,-200.1,525.5");
    add_proj4text (p, 1, ",0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"KOC\",DATUM[\"Kuwait_Oil_Company\",SPHEROID[\"C");
    add_srs_wkt (p, 1,
		 "larke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7012\"]],TOWGS84[-294.7,-200.1,525.5,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6246\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4246\"");
    add_srs_wkt (p, 6, "]]");
    p = add_epsg_def (first, last, 4247, "epsg", 4247, "La Canoa");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-273.5,110.6,-357.9,0");
    add_proj4text (p, 1, ",0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"La Canoa\",DATUM[\"La_Canoa\",SPHEROID[\"Intern");
    add_srs_wkt (p, 1,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 2,
		 ",TOWGS84[-273.5,110.6,-357.9,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6247\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4247\"]]");
    p = add_epsg_def (first, last, 4248, "epsg", 4248, "PSAD56");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"PSAD56\",DATUM[\"Provisional_South_American_Dat");
    add_srs_wkt (p, 1,
		 "um_1956\",SPHEROID[\"International 1924\",6378388,297,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6248\"]");
    add_srs_wkt (p, 3,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4248\"]]");
    p = add_epsg_def (first, last, 4249, "epsg", 4249, "Lake");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lake\",DATUM[\"Lake\",SPHEROID[\"International ");
    add_srs_wkt (p, 1,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"6249\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4249\"");
    add_srs_wkt (p, 5, "]]");
    p = add_epsg_def (first, last, 4250, "epsg", 4250, "Leigon");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-130,29,364,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Leigon\",DATUM[\"Leigon\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 1,
		 "0 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[-130,29,364,0,0,0,0],AUTHORITY[\"EPSG\",\"625");
    add_srs_wkt (p, 3,
		 "0\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4250\"]]");
    p = add_epsg_def (first, last, 4251, "epsg", 4251, "Liberia 1964");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-90,40,88,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Liberia 1964\",DATUM[\"Liberia_1964\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],TOWGS84[-90,40,88,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6251\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4251\"]]");
    p = add_epsg_def (first, last, 4252, "epsg", 4252, "Lome");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lome\",DATUM[\"Lome\",SPHEROID[\"Clarke 1880 (I");
    add_srs_wkt (p, 1,
		 "GN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "011\"]],AUTHORITY[\"EPSG\",\"6252\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4252\"]]");
    p = add_epsg_def (first, last, 4253, "epsg", 4253, "Luzon 1911");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Luzon 1911\",DATUM[\"Luzon_1911\",SPHEROID[\"Cl");
    add_srs_wkt (p, 1,
		 "arke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 2,
		 "\",\"7008\"]],AUTHORITY[\"EPSG\",\"6253\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 3,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4253\"]]");
    p = add_epsg_def (first, last, 4254, "epsg", 4254, "Hito XVIII 1963");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hito XVIII 1963\",DATUM[\"Hito_XVIII_1963\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6254\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4254\"]]");
    p = add_epsg_def (first, last, 4255, "epsg", 4255, "Herat North");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-333,-222,114,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Herat North\",DATUM[\"Herat_North\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],TOWGS84[-333,-222,114,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6255\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4255\"]]");
    p = add_epsg_def (first, last, 4256, "epsg", 4256, "Mahe 1971");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=41,-220,-134,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mahe 1971\",DATUM[\"Mahe_1971\",SPHEROID[\"Clar");
    add_srs_wkt (p, 1,
		 "ke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7012\"]],TOWGS84[41,-220,-134,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6256\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4256\"]]");
    p = add_epsg_def (first, last, 4257, "epsg", 4257, "Makassar");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=-587.8,519.75,145.7");
    add_proj4text (p, 1, "6,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Makassar\",DATUM[\"Makassar\",SPHEROID[\"Bessel");
    add_srs_wkt (p, 1,
		 " 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 2,
		 "4\"]],TOWGS84[-587.8,519.75,145.76,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6257\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4257\"]]");
    p = add_epsg_def (first, last, 4258, "epsg", 4258, "ETRS89");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ETRS89\",DATUM[\"European_Terrestrial_Reference");
    add_srs_wkt (p, 1,
		 "_System_1989\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 2,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 3,
		 "58\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4258\"]]");
    p = add_epsg_def (first, last, 4259, "epsg", 4259, "Malongo 1987");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Malongo 1987\",DATUM[\"Malongo_1987\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6259\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4259\"]]");
    p = add_epsg_def (first, last, 4260, "epsg", 4260, "Manoca");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-70.9,-151.8,-41.4,");
    add_proj4text (p, 1, "0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Manoca\",DATUM[\"Manoca\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 1,
		 "0 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[-70.9,-151.8,-41.4,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6260\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"4260\"]]");
    p = add_epsg_def (first, last, 4261, "epsg", 4261, "Merchich");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=31,146,47");
    add_proj4text (p, 1, ",0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Merchich\",DATUM[\"Merchich\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7011\"]],TOWGS84[31,146,47,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6261\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4261\"]]");
    p = add_epsg_def (first, last, 4262, "epsg", 4262, "Massawa");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=639,405,60,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Massawa\",DATUM[\"Massawa\",SPHEROID[\"Bessel 1");
    add_srs_wkt (p, 1,
		 "841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[639,405,60,0,0,0,0],AUTHORITY[\"EPSG\",\"6262");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4262\"]]");
    p = add_epsg_def (first, last, 4263, "epsg", 4263, "Minna");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Minna\",DATUM[\"Minna\",SPHEROID[\"Clarke 1880 ");
    add_srs_wkt (p, 1,
		 "(RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]");
    add_srs_wkt (p, 2,
		 "],AUTHORITY[\"EPSG\",\"6263\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 4,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4263\"]]");
    p = add_epsg_def (first, last, 4264, "epsg", 4264, "Mhast");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-252.95,-4.11,-96.38,");
    add_proj4text (p, 1, "0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mhast\",DATUM[\"Mhast\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 1,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS");
    add_srs_wkt (p, 2,
		 "84[-252.95,-4.11,-96.38,0,0,0,0],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 3,
		 "64\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4264\"]]");
    p = add_epsg_def (first, last, 4265, "epsg", 4265, "Monte Mario");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Monte Mario\",DATUM[\"Monte_Mario\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],AUTHORITY[\"EPSG\",\"6265\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4265\"]]");
    p = add_epsg_def (first, last, 4266, "epsg", 4266, "M'poraloko");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"M'poraloko\",DATUM[\"M_poraloko\",SPHEROID[\"Cl");
    add_srs_wkt (p, 1,
		 "arke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[");
    add_srs_wkt (p, 2,
		 "\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6266\"]],PRIMEM");
    add_srs_wkt (p, 3,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 4,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 5, ",AUTHORITY[\"EPSG\",\"4266\"]]");
    p = add_epsg_def (first, last, 4267, "epsg", 4267, "NAD27");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +datum=NAD27 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD27\",DATUM[\"North_American_Datum_1927\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4267\"]]");
    p = add_epsg_def (first, last, 4268, "epsg", 4268, "NAD27 Michigan");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378450.047548896 +b=6356826.621488444 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD27 Michigan\",DATUM[\"NAD_Michigan\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1866 Michigan\",6378450.047548896,294.9786971");
    add_srs_wkt (p, 2,
		 "646739,AUTHORITY[\"EPSG\",\"7009\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6268\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4268\"]]");
    p = add_epsg_def (first, last, 4269, "epsg", 4269, "NAD83");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +datum=NAD83 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4269\"]]");
    p = add_epsg_def (first, last, 4270, "epsg", 4270, "Nahrwan 1967");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Nahrwan 1967\",DATUM[\"Nahrwan_1967\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6270\"]],PRIMEM[\"G");
    add_srs_wkt (p, 3,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 4,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4270\"]]");
    p = add_epsg_def (first, last, 4271, "epsg", 4271, "Naparima 1972");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Naparima 1972\",DATUM[\"Naparima_1972\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7022\"]],AUTHORITY[\"EPSG\",\"6271\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 3,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 4,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4271\"]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_16 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
    struct epsg_defs *p;
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    p = add_epsg_def (first, last, 4272, "epsg", 4272, "NZGD49");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +datum=nzgd49 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NZGD49\",DATUM[\"New_Zealand_Geodetic_Datum_194");
    add_srs_wkt (p, 1,
		 "9\",SPHEROID[\"International 1924\",6378388,297,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7022\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0");
    add_srs_wkt (p, 3,
		 ".1,1.024,-4.5993],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6, "THORITY[\"EPSG\",\"4272\"]]");
    p = add_epsg_def (first, last, 4273, "epsg", 4273, "NGO 1948");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377492.018 +b=6356173.508712696 +towgs");
    add_proj4text (p, 1, "84=278.3,93,474.5,7.889,0.05,-6.61,6.21 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NGO 1948\",DATUM[\"NGO_1948\",SPHEROID[\"Bessel");
    add_srs_wkt (p, 1,
		 " Modified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7005\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6273\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "4273\"]]");
    p = add_epsg_def (first, last, 4274, "epsg", 4274, "Datum 73");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Datum 73\",DATUM[\"Datum_73\",SPHEROID[\"Intern");
    add_srs_wkt (p, 1,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"6274\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 4,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4274\"]]");
    p = add_epsg_def (first, last, 4275, "epsg", 4275, "NTF");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-168,-60,");
    add_proj4text (p, 1, "320,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NTF\",DATUM[\"Nouvelle_Triangulation_Francaise\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212936");
    add_srs_wkt (p, 2,
		 "265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,320,0");
    add_srs_wkt (p, 3,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6275\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6, "EPSG\",\"4275\"]]");
    p = add_epsg_def (first, last, 4276, "epsg", 4276, "NSWC 9Z-2");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NSWC 9Z-2\",DATUM[\"NSWC_9Z_2\",SPHEROID[\"NWL ");
    add_srs_wkt (p, 1,
		 "9D\",6378145,298.25,AUTHORITY[\"EPSG\",\"7025\"]],AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"6276\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4276");
    add_srs_wkt (p, 5, "\"]]");
    p = add_epsg_def (first, last, 4277, "epsg", 4277, "OSGB 1936");
    add_proj4text (p, 0, "+proj=longlat +ellps=airy +datum=OSGB36 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"OSGB 1936\",DATUM[\"OSGB_1936\",SPHEROID[\"Airy");
    add_srs_wkt (p, 1,
		 " 1830\",6377563.396,299.3249646,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 2,
		 "1\"]],AUTHORITY[\"EPSG\",\"6277\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4277\"]]");
    p = add_epsg_def (first, last, 4278, "epsg", 4278, "OSGB70");
    add_proj4text (p, 0, "+proj=longlat +ellps=airy +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"OSGB70\",DATUM[\"OSGB_1970_SN\",SPHEROID[\"Airy");
    add_srs_wkt (p, 1,
		 " 1830\",6377563.396,299.3249646,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 2,
		 "1\"]],AUTHORITY[\"EPSG\",\"6278\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4278\"]]");
    p = add_epsg_def (first, last, 4279, "epsg", 4279, "OS(SN)80");
    add_proj4text (p, 0, "+proj=longlat +ellps=airy +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"OS(SN)80\",DATUM[\"OS_SN_1980\",SPHEROID[\"Airy");
    add_srs_wkt (p, 1,
		 " 1830\",6377563.396,299.3249646,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 2,
		 "1\"]],AUTHORITY[\"EPSG\",\"6279\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4279\"]]");
    p = add_epsg_def (first, last, 4280, "epsg", 4280, "Padang");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Padang\",DATUM[\"Padang_1884\",SPHEROID[\"Besse");
    add_srs_wkt (p, 1,
		 "l 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "04\"]],AUTHORITY[\"EPSG\",\"6280\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4280\"]]");
    p = add_epsg_def (first, last, 4281, "epsg", 4281, "Palestine 1923");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378300.789 +b=6356566.435 +towgs84=-27");
    add_proj4text (p, 1,
		   "5.722,94.7824,340.894,-8.001,-4.42,-11.821,1 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Palestine 1923\",DATUM[\"Palestine_1923\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Clarke 1880 (Benoit)\",6378300.789,293.46631553898");
    add_srs_wkt (p, 2,
		 "02,AUTHORITY[\"EPSG\",\"7010\"]],TOWGS84[-275.722,94.782");
    add_srs_wkt (p, 3,
		 "4,340.894,-8.001,-4.42,-11.821,1],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 4,
		 "281\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 5,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 6, "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4281\"]]");
    p = add_epsg_def (first, last, 4282, "epsg", 4282, "Pointe Noire");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pointe Noire\",DATUM[\"Congo_1960_Pointe_Noire\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212936");
    add_srs_wkt (p, 2,
		 "265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "282\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5, "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4282\"]]");
    p = add_epsg_def (first, last, 4283, "epsg", 4283, "GDA94");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"GDA94\",DATUM[\"Geocentric_Datum_of_Australia_1");
    add_srs_wkt (p, 1,
		 "994\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6283\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4283\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4284, "epsg", 4284, "Pulkovo 1942");
    add_proj4text (p, 0, "+proj=longlat +ellps=krass +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "4284\"]]");
    p = add_epsg_def (first, last, 4285, "epsg", 4285, "Qatar 1974");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Qatar 1974\",DATUM[\"Qatar_1974\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],AUTHORITY[\"EPSG\",\"6285\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 3,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 4,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"4285\"]]");
    p = add_epsg_def (first, last, 4286, "epsg", 4286, "Qatar 1948");
    add_proj4text (p, 0, "+proj=longlat +ellps=helmert +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Qatar 1948\",DATUM[\"Qatar_1948\",SPHEROID[\"He");
    add_srs_wkt (p, 1,
		 "lmert 1906\",6378200,298.3,AUTHORITY[\"EPSG\",\"7020\"]]");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"6286\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 4,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4286\"]]");
    p = add_epsg_def (first, last, 4287, "epsg", 4287, "Qornoq");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=164,138,-189,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Qornoq\",DATUM[\"Qornoq\",SPHEROID[\"Internatio");
    add_srs_wkt (p, 1,
		 "nal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOW");
    add_srs_wkt (p, 2,
		 "GS84[164,138,-189,0,0,0,0],AUTHORITY[\"EPSG\",\"6287\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 4,
		 "IT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5, "08\"]],AUTHORITY[\"EPSG\",\"4287\"]]");
    p = add_epsg_def (first, last, 4288, "epsg", 4288, "Loma Quintana");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Loma Quintana\",DATUM[\"Loma_Quintana\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7022\"]],AUTHORITY[\"EPSG\",\"6288\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 3,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 4,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4288\"]]");
    p = add_epsg_def (first, last, 4289, "epsg", 4289, "Amersfoort");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Amersfoort\",DATUM[\"Amersfoort\",SPHEROID[\"Be");
    add_srs_wkt (p, 1,
		 "ssel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7004\"]],AUTHORITY[\"EPSG\",\"6289\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4289\"]]");
    p = add_epsg_def (first, last, 4291, "epsg", 4291, "SAD69");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS67 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"SAD69\",DATUM[\"South_American_Datum_1969\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"GRS 1967\",6378160,298.247167427,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7036\"]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"4291\"]]");
    p = add_epsg_def (first, last, 4292, "epsg", 4292, "Sapper Hill 1943");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-355,21,72,0,0,0,0 +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Sapper Hill 1943\",DATUM[\"Sapper_Hill_1943\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"International 1924\",6378388,297,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7022\"]],TOWGS84[-355,21,72,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6292\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4292\"]]");
    p = add_epsg_def (first, last, 4293, "epsg", 4293, "Schwarzeck");
    add_proj4text (p, 0, "+proj=longlat +ellps=bess_nam +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Schwarzeck\",DATUM[\"Schwarzeck\",SPHEROID[\"Be");
    add_srs_wkt (p, 1,
		 "ssel Namibia (GLM)\",6377483.865280419,299.1528128,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7046\"]],AUTHORITY[\"EPSG\",\"6293\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5, "2\"]],AUTHORITY[\"EPSG\",\"4293\"]]");
    p = add_epsg_def (first, last, 4294, "epsg", 4294, "Segora");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Segora\",DATUM[\"Segora\",SPHEROID[\"Bessel 184");
    add_srs_wkt (p, 1,
		 "1\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]");
    add_srs_wkt (p, 2,
		 "],AUTHORITY[\"EPSG\",\"6294\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 4,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4294\"]]");
    p = add_epsg_def (first, last, 4295, "epsg", 4295, "Serindung");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Serindung\",DATUM[\"Serindung\",SPHEROID[\"Bess");
    add_srs_wkt (p, 1,
		 "el 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "004\"]],AUTHORITY[\"EPSG\",\"6295\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4295\"]]");
    p = add_epsg_def (first, last, 4296, "epsg", 4296, "Sudan");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Sudan\",DATUM[\"Sudan\",SPHEROID[\"Clarke 1880 ");
    add_srs_wkt (p, 1,
		 "(IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7011\"]],AUTHORITY[\"EPSG\",\"6296\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "74532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4296\"]]");
    p = add_epsg_def (first, last, 4297, "epsg", 4297, "Tananarive");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-189,-242,-91,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tananarive\",DATUM[\"Tananarive_1925\",SPHEROID");
    add_srs_wkt (p, 1,
		 "[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],TOWGS84[-189,-242,-91,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6297\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4297\"]]");
    p = add_epsg_def (first, last, 4298, "epsg", 4298, "Timbalai 1948");
    add_proj4text (p, 0, "+proj=longlat +ellps=evrstSS +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Timbalai 1948\",DATUM[\"Timbalai_1948\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Everest 1830 (1967 Definition)\",6377298.556,300.801");
    add_srs_wkt (p, 2,
		 "7,AUTHORITY[\"EPSG\",\"7016\"]],AUTHORITY[\"EPSG\",\"629");
    add_srs_wkt (p, 3,
		 "8\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4298\"]]");
    p = add_epsg_def (first, last, 4299, "epsg", 4299, "TM65");
    add_proj4text (p, 0, "+proj=longlat +ellps=mod_airy +datum=ire65 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"TM65\",DATUM[\"TM65\",SPHEROID[\"Airy Modified ");
    add_srs_wkt (p, 1,
		 "1849\",6377340.189,299.3249646,AUTHORITY[\"EPSG\",\"7002");
    add_srs_wkt (p, 2,
		 "\"]],AUTHORITY[\"EPSG\",\"6299\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 3,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 4,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"4299\"]]");
    p = add_epsg_def (first, last, 4300, "epsg", 4300, "TM75");
    add_proj4text (p, 0, "+proj=longlat +ellps=mod_airy +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"TM75\",DATUM[\"Geodetic_Datum_of_1965\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Airy Modified 1849\",6377340.189,299.3249646,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7002\"]],AUTHORITY[\"EPSG\",\"6300\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5, "]],AUTHORITY[\"EPSG\",\"4300\"]]");
    p = add_epsg_def (first, last, 4301, "epsg", 4301, "Tokyo");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\"");
    add_srs_wkt (p, 1,
		 ",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 4,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "4301\"]]");
    p = add_epsg_def (first, last, 4302, "epsg", 4302, "Trinidad 1903");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378293.645208759 +b=6356617.987679838 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Trinidad 1903\",DATUM[\"Trinidad_1903\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1858\",6378293.645208759,294.2606763692569,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7007\"]],AUTHORITY[\"EPSG\",\"6302\"]");
    add_srs_wkt (p, 3,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4302\"]]");
    p = add_epsg_def (first, last, 4303, "epsg", 4303, "TC(1948)");
    add_proj4text (p, 0, "+proj=longlat +ellps=helmert +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"TC(1948)\",DATUM[\"Trucial_Coast_1948\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Helmert 1906\",6378200,298.3,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "20\"]],AUTHORITY[\"EPSG\",\"6303\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4303\"]]");
    p = add_epsg_def (first, last, 4304, "epsg", 4304, "Voirol 1875");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-73,-247,");
    add_proj4text (p, 1, "227,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Voirol 1875\",DATUM[\"Voirol_1875\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Clarke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7011\"]],TOWGS84[-73,-247,227,0,0,0,0],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6304\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"43");
    add_srs_wkt (p, 6, "04\"]]");
    p = add_epsg_def (first, last, 4306, "epsg", 4306, "Bern 1938");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bern 1938\",DATUM[\"Bern_1938\",SPHEROID[\"Bess");
    add_srs_wkt (p, 1,
		 "el 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "004\"]],AUTHORITY[\"EPSG\",\"6306\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4306\"]]");
    p = add_epsg_def (first, last, 4307, "epsg", 4307, "Nord Sahara 1959");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Nord Sahara 1959\",DATUM[\"Nord_Sahara_1959\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6307\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4307\"]]");
    p = add_epsg_def (first, last, 4308, "epsg", 4308, "RT38");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RT38\",DATUM[\"Stockholm_1938\",SPHEROID[\"Bess");
    add_srs_wkt (p, 1,
		 "el 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "004\"]],AUTHORITY[\"EPSG\",\"6308\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4308\"]]");
    p = add_epsg_def (first, last, 4309, "epsg", 4309, "Yacare");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-155,171,37,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Yacare\",DATUM[\"Yacare\",SPHEROID[\"Internatio");
    add_srs_wkt (p, 1,
		 "nal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOW");
    add_srs_wkt (p, 2,
		 "GS84[-155,171,37,0,0,0,0],AUTHORITY[\"EPSG\",\"6309\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5, "22\"]],AUTHORITY[\"EPSG\",\"4309\"]]");
    p = add_epsg_def (first, last, 4310, "epsg", 4310, "Yoff");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Yoff\",DATUM[\"Yoff\",SPHEROID[\"Clarke 1880 (I");
    add_srs_wkt (p, 1,
		 "GN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "011\"]],AUTHORITY[\"EPSG\",\"6310\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4310\"]]");
    p = add_epsg_def (first, last, 4311, "epsg", 4311, "Zanderij");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-265,120,-358,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Zanderij\",DATUM[\"Zanderij\",SPHEROID[\"Intern");
    add_srs_wkt (p, 1,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 2,
		 ",TOWGS84[-265,120,-358,0,0,0,0],AUTHORITY[\"EPSG\",\"631");
    add_srs_wkt (p, 3,
		 "1\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4311\"]]");
    p = add_epsg_def (first, last, 4312, "epsg", 4312, "MGI");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +datum=hermannskogel +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"MGI\",DATUM[\"Militar_Geographische_Institute\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7004\"]],TOWGS84[577.326,90.129,463.919,5");
    add_srs_wkt (p, 3,
		 ".137,1.474,5.297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108");
    add_srs_wkt (p, 6, "\"]],AUTHORITY[\"EPSG\",\"4312\"]]");
    p = add_epsg_def (first, last, 4313, "epsg", 4313, "Belge 1972");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=106.869,-52.2978,103.");
    add_proj4text (p, 1, "724,-0.33657,0.456955,-1.84218,1 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Belge 1972\",DATUM[\"Reseau_National_Belge_1972");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],TOWGS84[106.869,-52.2978,103.724,-0");
    add_srs_wkt (p, 3,
		 ".33657,0.456955,-1.84218,1],AUTHORITY[\"EPSG\",\"6313\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "9122\"]],AUTHORITY[\"EPSG\",\"4313\"]]");
    p = add_epsg_def (first, last, 4314, "epsg", 4314, "DHDN");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +datum=potsdam +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIME");
    add_srs_wkt (p, 3,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 4,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 5, "],AUTHORITY[\"EPSG\",\"4314\"]]");
    p = add_epsg_def (first, last, 4315, "epsg", 4315, "Conakry 1905");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-23,259,-");
    add_proj4text (p, 1, "9,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Conakry 1905\",DATUM[\"Conakry_1905\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7011\"]],TOWGS84[-23,259,-9,0,0,0,0],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6315\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"43");
    add_srs_wkt (p, 6, "15\"]]");
    p = add_epsg_def (first, last, 4316, "epsg", 4316, "Dealul Piscului 1930");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Dealul Piscului 1930\",DATUM[\"Dealul_Piscului_");
    add_srs_wkt (p, 1,
		 "1930\",SPHEROID[\"International 1924\",6378388,297,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6316\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5, "2\"]],AUTHORITY[\"EPSG\",\"4316\"]]");
    p = add_epsg_def (first, last, 4317, "epsg", 4317, "Dealul Piscului 1970");
    add_proj4text (p, 0, "+proj=longlat +ellps=krass +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Dealul Piscului 1970\",DATUM[\"Dealul_Piscului_");
    add_srs_wkt (p, 1,
		 "1970\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6317\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4317\"]]");
    p = add_epsg_def (first, last, 4318, "epsg", 4318, "NGN");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=-3.2,-5.7,2.8,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NGN\",DATUM[\"National_Geodetic_Network\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7030\"]],TOWGS84[-3.2,-5.7,2.8,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6318\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4318\"]]");
    p = add_epsg_def (first, last, 4319, "epsg", 4319, "KUDAMS");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"KUDAMS\",DATUM[\"Kuwait_Utility\",SPHEROID[\"GR");
    add_srs_wkt (p, 1,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 2,
		 "\"]],AUTHORITY[\"EPSG\",\"6319\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 3,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 4,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"4319\"]]");
    p = add_epsg_def (first, last, 4322, "epsg", 4322, "WGS 72");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS72 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"WGS 72\",DATUM[\"WGS_1972\",SPHEROID[\"WGS 72\"");
    add_srs_wkt (p, 1,
		 ",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],AUTHORITY[");
    add_srs_wkt (p, 2,
		 "\"EPSG\",\"6322\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4322\"]]");
    add_srs_wkt (p, 5, "");
    p = add_epsg_def (first, last, 4324, "epsg", 4324, "WGS 72BE");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.814,-0");
    add_proj4text (p, 1, ".38 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"WGS 72BE\",DATUM[\"WGS_1972_Transit_Broadcast_E");
    add_srs_wkt (p, 1,
		 "phemeris\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORITY[");
    add_srs_wkt (p, 2,
		 "\"EPSG\",\"7043\"]],TOWGS84[0,0,1.9,0,0,0.814,-0.38],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6324\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6, "324\"]]");

#endif /* loading unconditionally the 4326 EPSG def */
    p = add_epsg_def (first, last, 4326, "epsg", 4326, "WGS 84");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\"");
    add_srs_wkt (p, 1,
		 ",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 4,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 5, "326\"]]");
#ifndef OMIT_EPSG		/* resuming conditional EPSG initialization */

    p = add_epsg_def (first, last, 4600, "epsg", 4600, "Anguilla 1957");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Anguilla 1957\",DATUM[\"Anguilla_1957\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6600\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4600\"]]");
    p = add_epsg_def (first, last, 4601, "epsg", 4601, "Antigua 1943");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Antigua 1943\",DATUM[\"Antigua_1943\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6601\"]],PRIMEM[\"G");
    add_srs_wkt (p, 3,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 4,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4601\"]]");
    p = add_epsg_def (first, last, 4602, "epsg", 4602, "Dominica 1945");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=725,685,536,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Dominica 1945\",DATUM[\"Dominica_1945\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7012\"]],TOWGS84[725,685,536,0,0,0,0],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6602\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4602\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4603, "epsg", 4603, "Grenada 1953");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=72,213.7,93,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Grenada 1953\",DATUM[\"Grenada_1953\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],TOWGS84[72,213.7,93,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6603\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4603\"]]");
    p = add_epsg_def (first, last, 4604, "epsg", 4604, "Montserrat 1958");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=174,359,365,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Montserrat 1958\",DATUM[\"Montserrat_1958\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7012\"]],TOWGS84[174,359,365,0,0,0,0],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6604\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"460");
    add_srs_wkt (p, 6, "4\"]]");
    p = add_epsg_def (first, last, 4605, "epsg", 4605, "St. Kitts 1955");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"St. Kitts 1955\",DATUM[\"St_Kitts_1955\",SPHERO");
    add_srs_wkt (p, 1,
		 "ID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6605\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4605\"]]");
    p = add_epsg_def (first, last, 4606, "epsg", 4606, "St. Lucia 1955");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-149,128,296,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"St. Lucia 1955\",DATUM[\"St_Lucia_1955\",SPHERO");
    add_srs_wkt (p, 1,
		 "ID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7012\"]],TOWGS84[-149,128,296,0,0,0,0],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6606\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4606\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4607, "epsg", 4607, "St. Vincent 1945");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=195.671,332.517,274");
    add_proj4text (p, 1, ".607,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"St. Vincent 1945\",DATUM[\"St_Vincent_1945\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7012\"]],TOWGS84[195.671,332.517,274.607,0");
    add_srs_wkt (p, 3,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6607\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6, "EPSG\",\"4607\"]]");
    p = add_epsg_def (first, last, 4608, "epsg", 4608, "NAD27(76)");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD27(76)\",DATUM[\"North_American_Datum_1927_1");
    add_srs_wkt (p, 1,
		 "976\",SPHEROID[\"Clarke 1866\",6378206.4,294.97869821390");
    add_srs_wkt (p, 2,
		 "06,AUTHORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"66");
    add_srs_wkt (p, 3,
		 "08\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4608\"]]");
    p = add_epsg_def (first, last, 4609, "epsg", 4609, "NAD27(CGQ77)");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD27(CGQ77)\",DATUM[\"North_American_Datum_192");
    add_srs_wkt (p, 1,
		 "7_CGQ77\",SPHEROID[\"Clarke 1866\",6378206.4,294.9786982");
    add_srs_wkt (p, 2,
		 "139006,AUTHORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6609\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4609\"]]");
    p = add_epsg_def (first, last, 4610, "epsg", 4610, "Xian 1980");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378140 +b=6356755.288157528 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Xian 1980\",DATUM[\"Xian_1980\",SPHEROID[\"Xian");
    add_srs_wkt (p, 1,
		 " 1980\",6378140,298.257,AUTHORITY[\"EPSG\",\"7049\"]],AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"6610\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 4,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "4610\"]]");
    p = add_epsg_def (first, last, 4611, "epsg", 4611, "Hong Kong 1980");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-162.619,-276.959,-16");
    add_proj4text (p, 1, "1.764,0.067753,-2.24365,-1.15883,-1.09425 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hong Kong 1980\",DATUM[\"Hong_Kong_1980\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7022\"]],TOWGS84[-162.619,-276.959,-161.764,0.067753,");
    add_srs_wkt (p, 3,
		 "-2.24365,-1.15883,-1.09425],AUTHORITY[\"EPSG\",\"6611\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "9122\"]],AUTHORITY[\"EPSG\",\"4611\"]]");
    p = add_epsg_def (first, last, 4612, "epsg", 4612, "JGD2000");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"JGD2000\",DATUM[\"Japanese_Geodetic_Datum_2000\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6612\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4612\"]]");
    p = add_epsg_def (first, last, 4613, "epsg", 4613, "Segara");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Segara\",DATUM[\"Gunung_Segara\",SPHEROID[\"Bes");
    add_srs_wkt (p, 1,
		 "sel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7004\"]],AUTHORITY[\"EPSG\",\"6613\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4613\"]]");
    p = add_epsg_def (first, last, 4614, "epsg", 4614, "QND95");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-119.425,-303.659,-11");
    add_proj4text (p, 1, ".0006,1.1643,0.174458,1.09626,3.65706 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"QND95\",DATUM[\"Qatar_National_Datum_1995\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],TOWGS84[-119.425,-303.659,-11.0006,1.1643");
    add_srs_wkt (p, 3,
		 ",0.174458,1.09626,3.65706],AUTHORITY[\"EPSG\",\"6614\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6, "122\"]],AUTHORITY[\"EPSG\",\"4614\"]]");
    p = add_epsg_def (first, last, 4615, "epsg", 4615, "Porto Santo");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-499,-249,314,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Porto Santo\",DATUM[\"Porto_Santo_1936\",SPHERO");
    add_srs_wkt (p, 1,
		 "ID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7022\"]],TOWGS84[-499,-249,314,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6615\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4615\"]]");
    p = add_epsg_def (first, last, 4616, "epsg", 4616, "Selvagem Grande");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Selvagem Grande\",DATUM[\"Selvagem_Grande\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6616\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4616\"]]");
    p = add_epsg_def (first, last, 4617, "epsg", 4617, "NAD83(CSRS)");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD83(CSRS)\",DATUM[\"NAD83_Canadian_Spatial_Re");
    add_srs_wkt (p, 1,
		 "ference_System\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 2,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6140\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4617\"]]");
    p = add_epsg_def (first, last, 4618, "epsg", 4618, "SAD69");
    add_proj4text (p, 0, "+proj=longlat +ellps=aust_SA +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"SAD69\",DATUM[\"South_American_Datum_1969\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"GRS 1967 Modified\",6378160,298.25,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7050\"]],AUTHORITY[\"EPSG\",\"6618\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4618\"]]");
    p = add_epsg_def (first, last, 4619, "epsg", 4619, "SWEREF99");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"SWEREF99\",DATUM[\"SWEREF99\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 1,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 2,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6619\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4619\"]]");
    p = add_epsg_def (first, last, 4620, "epsg", 4620, "Point 58");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-106,-129,165,0,0,0");
    add_proj4text (p, 1, ",0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Point 58\",DATUM[\"Point_58\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 1,
		 " 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "012\"]],TOWGS84[-106,-129,165,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6620\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4620\"]]");
    p = add_epsg_def (first, last, 4621, "epsg", 4621, "Fort Marigot");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=137,248,-430,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Fort Marigot\",DATUM[\"Fort_Marigot\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],TOWGS84[137,248,-430,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6621\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4621\"]]");
    p = add_epsg_def (first, last, 4622, "epsg", 4622, "Guadeloupe 1948");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Guadeloupe 1948\",DATUM[\"Guadeloupe_1948\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6622\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4622\"]]");
    p = add_epsg_def (first, last, 4623, "epsg", 4623, "CSG67");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-186,230,110,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"CSG67\",DATUM[\"Centre_Spatial_Guyanais_1967\",");
    add_srs_wkt (p, 1,
		 "SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],TOWGS84[-186,230,110,0,0,0,0],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6623\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4623\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4624, "epsg", 4624, "RGFG95");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=2,2,-2,0,0,0,0 +no_d");
    add_proj4text (p, 1, "efs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RGFG95\",DATUM[\"Reseau_Geodesique_Francais_Guy");
    add_srs_wkt (p, 1,
		 "ane_1995\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[2,2,-2,0,0,0,0],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6624\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6, "624\"]]");
    p = add_epsg_def (first, last, 4625, "epsg", 4625, "Martinique 1938");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Martinique 1938\",DATUM[\"Martinique_1938\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6625\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4625\"]]");
    p = add_epsg_def (first, last, 4626, "epsg", 4626, "Reunion 1947");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Reunion 1947\",DATUM[\"Reunion_1947\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6626\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4626\"]]");
    p = add_epsg_def (first, last, 4627, "epsg", 4627, "RGR92");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RGR92\",DATUM[\"Reseau_Geodesique_de_la_Reunion");
    add_srs_wkt (p, 1,
		 "_1992\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6627\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4627\"");
    add_srs_wkt (p, 6, "]]");
    p = add_epsg_def (first, last, 4628, "epsg", 4628, "Tahiti 52");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=162,117,154,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tahiti 52\",DATUM[\"Tahiti_52\",SPHEROID[\"Inte");
    add_srs_wkt (p, 1,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[162,117,154,0,0,0,0],AUTHORITY[\"EPSG\",\"662");
    add_srs_wkt (p, 3,
		 "8\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4628\"]]");
    p = add_epsg_def (first, last, 4629, "epsg", 4629, "Tahaa 54");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tahaa 54\",DATUM[\"Tahaa_54\",SPHEROID[\"Intern");
    add_srs_wkt (p, 1,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"6629\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 4,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4629\"]]");
    p = add_epsg_def (first, last, 4630, "epsg", 4630, "IGN72 Nuku Hiva");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN72 Nuku Hiva\",DATUM[\"IGN72_Nuku_Hiva\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6630\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4630\"]]");
    p = add_epsg_def (first, last, 4631, "epsg", 4631, "K0 1949");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=145,-187,103,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"K0 1949\",DATUM[\"K0_1949\",SPHEROID[\"Internat");
    add_srs_wkt (p, 1,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 2,
		 "OWGS84[145,-187,103,0,0,0,0],AUTHORITY[\"EPSG\",\"6631\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4631\"]]");
    p = add_epsg_def (first, last, 4632, "epsg", 4632, "Combani 1950");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-382,-59,-262,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Combani 1950\",DATUM[\"Combani_1950\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],TOWGS84[-382,-59,-262,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6632\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4632\"]]");
    p = add_epsg_def (first, last, 4633, "epsg", 4633, "IGN56 Lifou");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN56 Lifou\",DATUM[\"IGN56_Lifou\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],AUTHORITY[\"EPSG\",\"6633\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4633\"]]");
    p = add_epsg_def (first, last, 4634, "epsg", 4634, "IGN72 Grand Terre");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN72 Grand Terre\",DATUM[\"IGN72_Grande_Terre\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6634\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4634\"]]");
    p = add_epsg_def (first, last, 4635, "epsg", 4635, "ST87 Ouvea");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-122.383,-188.696,103");
    add_proj4text (p, 1, ".344,3.5107,-4.9668,-5.7047,4.4798 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ST87 Ouvea\",DATUM[\"ST87_Ouvea\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],TOWGS84[-122.383,-188.696,103.344,3.5107,-4.9668,-5");
    add_srs_wkt (p, 3,
		 ".7047,4.4798],AUTHORITY[\"EPSG\",\"6635\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6, "RITY[\"EPSG\",\"4635\"]]");
    p = add_epsg_def (first, last, 4636, "epsg", 4636, "Petrels 1972");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=365,194,166,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Petrels 1972\",DATUM[\"Petrels_1972\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],TOWGS84[365,194,166,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6636\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4636\"]]");
    p = add_epsg_def (first, last, 4637, "epsg", 4637, "Perroud 1950");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=325,154,172,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Perroud 1950\",DATUM[\"Pointe_Geologie_Perroud_");
    add_srs_wkt (p, 1,
		 "1950\",SPHEROID[\"International 1924\",6378388,297,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7022\"]],TOWGS84[325,154,172,0,0,0,0],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6637\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "4637\"]]");
    p = add_epsg_def (first, last, 4638, "epsg", 4638,
		      "Saint Pierre et Miquelon 1950");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=30,430,368,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Saint Pierre et Miquelon 1950\",DATUM[\"Saint_P");
    add_srs_wkt (p, 1,
		 "ierre_et_Miquelon_1950\",SPHEROID[\"Clarke 1866\",637820");
    add_srs_wkt (p, 2,
		 "6.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],TOWG");
    add_srs_wkt (p, 3,
		 "S84[30,430,368,0,0,0,0],AUTHORITY[\"EPSG\",\"6638\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6, "\"]],AUTHORITY[\"EPSG\",\"4638\"]]");
    p = add_epsg_def (first, last, 4639, "epsg", 4639, "MOP78");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"MOP78\",DATUM[\"MOP78\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 1,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"6639\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 4,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"463");
    add_srs_wkt (p, 5, "9\"]]");
    p = add_epsg_def (first, last, 4640, "epsg", 4640, "RRAF 1991");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RRAF 1991\",DATUM[\"Reseau_de_Reference_des_Ant");
    add_srs_wkt (p, 1,
		 "illes_Francaises_1991\",SPHEROID[\"WGS 84\",6378137,298.");
    add_srs_wkt (p, 2,
		 "257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 3,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6640\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6, "PSG\",\"4640\"]]");
    p = add_epsg_def (first, last, 4641, "epsg", 4641, "IGN53 Mare");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN53 Mare\",DATUM[\"IGN53_Mare\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],AUTHORITY[\"EPSG\",\"6641\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 3,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 4,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"4641\"]]");
    p = add_epsg_def (first, last, 4642, "epsg", 4642, "ST84 Ile des Pins");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ST84 Ile des Pins\",DATUM[\"ST84_Ile_des_Pins\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6642\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4642\"]]");
    p = add_epsg_def (first, last, 4643, "epsg", 4643, "ST71 Belep");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-480.26,-438.32,-643.");
    add_proj4text (p, 1, "429,16.3119,20.1721,-4.0349,-111.7 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ST71 Belep\",DATUM[\"ST71_Belep\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],TOWGS84[-480.26,-438.32,-643.429,16.3119,20.1721,-4");
    add_srs_wkt (p, 3,
		 ".0349,-111.7],AUTHORITY[\"EPSG\",\"6643\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6, "RITY[\"EPSG\",\"4643\"]]");
    p = add_epsg_def (first, last, 4644, "epsg", 4644, "NEA74 Noumea");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NEA74 Noumea\",DATUM[\"NEA74_Noumea\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6644\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4644\"]]");
    p = add_epsg_def (first, last, 4645, "epsg", 4645, "RGNC 1991");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=0,0,0,0,0,0,0 +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RGNC 1991\",DATUM[\"Reseau_Geodesique_Nouvelle_");
    add_srs_wkt (p, 1,
		 "Caledonie_1991\",SPHEROID[\"International 1924\",6378388");
    add_srs_wkt (p, 2,
		 ",297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6645\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"4645\"]]");
    p = add_epsg_def (first, last, 4646, "epsg", 4646, "Grand Comoros");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Grand Comoros\",DATUM[\"Grand_Comoros\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7022\"]],AUTHORITY[\"EPSG\",\"6646\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 3,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 4,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4646\"]]");
    p = add_epsg_def (first, last, 4657, "epsg", 4657, "Reykjavik 1900");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377019.27 +b=6355762.5391 +towgs84=-28");
    add_proj4text (p, 1, ",199,5,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Reykjavik 1900\",DATUM[\"Reykjavik_1900\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Danish 1876\",6377019.27,300,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7051\"]],TOWGS84[-28,199,5,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6657\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4657\"]]");
    p = add_epsg_def (first, last, 4658, "epsg", 4658, "Hjorsey 1955");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-73,46,-86,0,0,0,0 +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hjorsey 1955\",DATUM[\"Hjorsey_1955\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],TOWGS84[-73,46,-86,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6658\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4658\"]]");
    p = add_epsg_def (first, last, 4659, "epsg", 4659, "ISN93");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ISN93\",DATUM[\"Islands_Network_1993\",SPHEROID");
    add_srs_wkt (p, 1,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"665");
    add_srs_wkt (p, 3,
		 "9\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4659\"]]");
    p = add_epsg_def (first, last, 4660, "epsg", 4660, "Helle 1954");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=982.609,552.753,-540.");
    add_proj4text (p, 1, "873,32.3934,-153.257,-96.2266,16.805 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Helle 1954\",DATUM[\"Helle_1954\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],TOWGS84[982.609,552.753,-540.873,32.3934,-153.257,-");
    add_srs_wkt (p, 3,
		 "96.2266,16.805],AUTHORITY[\"EPSG\",\"6660\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6, "HORITY[\"EPSG\",\"4660\"]]");
    p = add_epsg_def (first, last, 4661, "epsg", 4661, "LKS92");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"LKS92\",DATUM[\"Latvia_1992\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 1,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 2,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6661\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 4,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 5, "\"]],AUTHORITY[\"EPSG\",\"4661\"]]");
    p = add_epsg_def (first, last, 4662, "epsg", 4662, "IGN72 Grande Terre");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN72 Grande Terre\",DATUM[\"IGN72_Grande_Terre");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6634\"]],PRIME");
    add_srs_wkt (p, 3,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 4,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 5, "],AUTHORITY[\"EPSG\",\"4662\"]]");
    p = add_epsg_def (first, last, 4663, "epsg", 4663, "Porto Santo 1995");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Porto Santo 1995\",DATUM[\"Porto_Santo_1995\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"International 1924\",6378388,297,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6663\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 4,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 5, "THORITY[\"EPSG\",\"4663\"]]");
    p = add_epsg_def (first, last, 4664, "epsg", 4664, "Azores Oriental 1995");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Azores Oriental 1995\",DATUM[\"Azores_Oriental_");
    add_srs_wkt (p, 1,
		 "Islands_1995\",SPHEROID[\"International 1924\",6378388,2");
    add_srs_wkt (p, 2,
		 "97,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"66");
    add_srs_wkt (p, 3,
		 "64\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4664\"]]");
    p = add_epsg_def (first, last, 4665, "epsg", 4665, "Azores Central 1995");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Azores Central 1995\",DATUM[\"Azores_Central_Is");
    add_srs_wkt (p, 1,
		 "lands_1995\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6665");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4665\"]]");
    p = add_epsg_def (first, last, 4666, "epsg", 4666, "Lisbon 1890");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lisbon 1890\",DATUM[\"Lisbon_1890\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7004\"]],AUTHORITY[\"EPSG\",\"6666\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4666\"]]");
    p = add_epsg_def (first, last, 4667, "epsg", 4667, "IKBD-92");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IKBD-92\",DATUM[\"Iraq_Kuwait_Boundary_Datum_19");
    add_srs_wkt (p, 1,
		 "92\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6667\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4667\"]]");
    p = add_epsg_def (first, last, 4668, "epsg", 4668, "ED79");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-86,-98,-119,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ED79\",DATUM[\"European_Datum_1979\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],TOWGS84[-86,-98,-119,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6668\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4668\"]]");
    p = add_epsg_def (first, last, 4669, "epsg", 4669, "LKS94");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"LKS94\",DATUM[\"Lithuania_1994_ETRS89\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "126\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5, "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4669\"]]");
    p = add_epsg_def (first, last, 4670, "epsg", 4670, "IGM95");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGM95\",DATUM[\"Istituto_Geografico_Militaire_1");
    add_srs_wkt (p, 1,
		 "995\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6670\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4670\"]]");
    p = add_epsg_def (first, last, 4671, "epsg", 4671, "Voirol 1879");
    add_proj4text (p, 0, "+proj=longlat +a=6378249.2 +b=6356515 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Voirol 1879\",DATUM[\"Voirol_1879\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Clarke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6671\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5, "]],AUTHORITY[\"EPSG\",\"4671\"]]");
    p = add_epsg_def (first, last, 4672, "epsg", 4672, "Chatham Islands 1971");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=175,-38,113,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Chatham Islands 1971\",DATUM[\"Chatham_Islands_");
    add_srs_wkt (p, 1,
		 "Datum_1971\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[175,-38,113,0,0,0");
    add_srs_wkt (p, 3,
		 ",0],AUTHORITY[\"EPSG\",\"6672\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 5,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 6, "G\",\"4672\"]]");
    p = add_epsg_def (first, last, 4673, "epsg", 4673, "Chatham Islands 1979");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=174.05,-25.49,112.57,");
    add_proj4text (p, 1, "-0,-0,0.554,0.2263 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Chatham Islands 1979\",DATUM[\"Chatham_Islands_");
    add_srs_wkt (p, 1,
		 "Datum_1979\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[174.05,-25.49,112");
    add_srs_wkt (p, 3,
		 ".57,-0,-0,0.554,0.2263],AUTHORITY[\"EPSG\",\"6673\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6, "\"]],AUTHORITY[\"EPSG\",\"4673\"]]");
    p = add_epsg_def (first, last, 4674, "epsg", 4674, "SIRGAS 2000");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"SIRGAS 2000\",DATUM[\"Sistema_de_Referencia_Geo");
    add_srs_wkt (p, 1,
		 "centrico_para_America_del_Sur_2000\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6674\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6, "]],AUTHORITY[\"EPSG\",\"4674\"]]");
    p = add_epsg_def (first, last, 4675, "epsg", 4675, "Guam 1963");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=-100,-248,259,0,0,0");
    add_proj4text (p, 1, ",0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Guam 1963\",DATUM[\"Guam_1963\",SPHEROID[\"Clar");
    add_srs_wkt (p, 1,
		 "ke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7008\"]],TOWGS84[-100,-248,259,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6675\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4675\"]]");
    p = add_epsg_def (first, last, 4676, "epsg", 4676, "Vientiane 1982");
    add_proj4text (p, 0, "+proj=longlat +ellps=krass +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Vientiane 1982\",DATUM[\"Vientiane_1982\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7024\"]],AUTHORITY[\"EPSG\",\"6676\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4676\"]]");
    p = add_epsg_def (first, last, 4677, "epsg", 4677, "Lao 1993");
    add_proj4text (p, 0, "+proj=longlat +ellps=krass +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lao 1993\",DATUM[\"Lao_1993\",SPHEROID[\"Krasso");
    add_srs_wkt (p, 1,
		 "wsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"6677\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 4,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5, "\"4677\"]]");
    p = add_epsg_def (first, last, 4678, "epsg", 4678, "Lao 1997");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=krass +towgs84=44.585,-131.212,-39.");
    add_proj4text (p, 1, "544,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lao 1997\",DATUM[\"Lao_National_Datum_1997\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7024\"]],TOWGS84[44.585,-131.212,-39.544,0,0,0,0],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6678\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6, "\"4678\"]]");
    p = add_epsg_def (first, last, 4679, "epsg", 4679, "Jouik 1961");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-80.01,253.26,291.1");
    add_proj4text (p, 1, "9,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Jouik 1961\",DATUM[\"Jouik_1961\",SPHEROID[\"Cl");
    add_srs_wkt (p, 1,
		 "arke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7012\"]],TOWGS84[-80.01,253.26,291.19,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6679\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4679");
    add_srs_wkt (p, 6, "\"]]");
    p = add_epsg_def (first, last, 4680, "epsg", 4680, "Nouakchott 1965");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=124.5,-63.5,-281,0,");
    add_proj4text (p, 1, "0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Nouakchott 1965\",DATUM[\"Nouakchott_1965\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7012\"]],TOWGS84[124.5,-63.5,-281,0,0,0,0],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6680\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6, "\"4680\"]]");
    p = add_epsg_def (first, last, 4681, "epsg", 4681, "Mauritania 1999");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mauritania 1999\",DATUM[\"Mauritania_1999\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6681\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5, "]],AUTHORITY[\"EPSG\",\"4681\"]]");
    p = add_epsg_def (first, last, 4682, "epsg", 4682, "Gulshan 303");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377276.345 +b=6356075.41314024 +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Gulshan 303\",DATUM[\"Gulshan_303\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Everest 1830 (1937 Adjustment)\",6377276.345,300.8017,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6682\"]");
    add_srs_wkt (p, 3,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4682\"]]");
    p = add_epsg_def (first, last, 4683, "epsg", 4683, "PRS92");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=-127.62,-67.24,-47.");
    add_proj4text (p, 1, "04,-3.068,4.903,1.578,-1.06 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"PRS92\",DATUM[\"Philippine_Reference_System_199");
    add_srs_wkt (p, 1,
		 "2\",SPHEROID[\"Clarke 1866\",6378206.4,294.9786982139006");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7008\"]],TOWGS84[-127.62,-67.24,-4");
    add_srs_wkt (p, 3,
		 "7.04,-3.068,4.903,1.578,-1.06],AUTHORITY[\"EPSG\",\"6683");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4683\"]]");
    p = add_epsg_def (first, last, 4684, "epsg", 4684, "Gan 1970");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-133,-321,50,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Gan 1970\",DATUM[\"Gan_1970\",SPHEROID[\"Intern");
    add_srs_wkt (p, 1,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 2,
		 ",TOWGS84[-133,-321,50,0,0,0,0],AUTHORITY[\"EPSG\",\"6684");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4684\"]]");
    p = add_epsg_def (first, last, 4685, "epsg", 4685, "Gandajika");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Gandajika\",DATUM[\"Gandajika\",SPHEROID[\"Inte");
    add_srs_wkt (p, 1,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 2,
		 "]],AUTHORITY[\"EPSG\",\"6685\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 4,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5, "\",\"4685\"]]");
    p = add_epsg_def (first, last, 4686, "epsg", 4686, "MAGNA-SIRGAS");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"MAGNA-SIRGAS\",DATUM[\"Marco_Geocentrico_Nacion");
    add_srs_wkt (p, 1,
		 "al_de_Referencia\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 2,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 3,
		 ",0],AUTHORITY[\"EPSG\",\"6686\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 5,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 6, "G\",\"4686\"]]");
    p = add_epsg_def (first, last, 4687, "epsg", 4687, "RGPF");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RGPF\",DATUM[\"Reseau_Geodesique_de_la_Polynesi");
    add_srs_wkt (p, 1,
		 "e_Francaise\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 2,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"668");
    add_srs_wkt (p, 3,
		 "7\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4687\"]]");
    p = add_epsg_def (first, last, 4688, "epsg", 4688, "Fatu Iva 72");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=347.103,1078.12,2623.");
    add_proj4text (p, 1, "92,-33.8875,70.6773,-9.3943,186.074 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Fatu Iva 72\",DATUM[\"Fatu_Iva_72\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],TOWGS84[347.103,1078.12,2623.92,-33.8875,70.6773,");
    add_srs_wkt (p, 3,
		 "-9.3943,186.074],AUTHORITY[\"EPSG\",\"6688\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6, "THORITY[\"EPSG\",\"4688\"]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_17 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 4689, "epsg", 4689, "IGN63 Hiva Oa");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN63 Hiva Oa\",DATUM[\"IGN63_Hiva_Oa\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7022\"]],AUTHORITY[\"EPSG\",\"6689\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 3,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 4,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4689\"]]");
    p = add_epsg_def (first, last, 4690, "epsg", 4690, "Tahiti 79");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tahiti 79\",DATUM[\"Tahiti_79\",SPHEROID[\"Inte");
    add_srs_wkt (p, 1,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 2,
		 "]],AUTHORITY[\"EPSG\",\"6690\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 4,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5, "\",\"4690\"]]");
    p = add_epsg_def (first, last, 4691, "epsg", 4691, "Moorea 87");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=215.525,149.593,176.2");
    add_proj4text (p, 1, "29,-3.2624,-1.692,-1.1571,10.4773 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Moorea 87\",DATUM[\"Moorea_87\",SPHEROID[\"Inte");
    add_srs_wkt (p, 1,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[215.525,149.593,176.229,-3.2624,-1.692,-1.157");
    add_srs_wkt (p, 3,
		 "1,10.4773],AUTHORITY[\"EPSG\",\"6691\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6, "Y[\"EPSG\",\"4691\"]]");
    p = add_epsg_def (first, last, 4692, "epsg", 4692, "Maupiti 83");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=217.037,86.959,23.956");
    add_proj4text (p, 1, ",0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Maupiti 83\",DATUM[\"Maupiti_83\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],TOWGS84[217.037,86.959,23.956,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6692\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4692\"]]");
    p = add_epsg_def (first, last, 4693, "epsg", 4693, "Nakhl-e Ghanem");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,-0.15,0.68,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Nakhl-e Ghanem\",DATUM[\"Nakhl_e_Ghanem\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7030\"]],TOWGS84[0,-0.15,0.68,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6693\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4693\"]]");
    p = add_epsg_def (first, last, 4694, "epsg", 4694, "POSGAR 94");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"POSGAR 94\",DATUM[\"Posiciones_Geodesicas_Argen");
    add_srs_wkt (p, 1,
		 "tinas_1994\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6694\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"46");
    add_srs_wkt (p, 6, "94\"]]");
    p = add_epsg_def (first, last, 4695, "epsg", 4695, "Katanga 1955");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=-103.746,-9.614,-25");
    add_proj4text (p, 1, "5.95,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Katanga 1955\",DATUM[\"Katanga_1955\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7008\"]],TOWGS84[-103.746,-9.614,-255.95,0,0,0,");
    add_srs_wkt (p, 3,
		 "0],AUTHORITY[\"EPSG\",\"6695\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6, "\",\"4695\"]]");
    p = add_epsg_def (first, last, 4696, "epsg", 4696, "Kasai 1953");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kasai 1953\",DATUM[\"Kasai_1953\",SPHEROID[\"Cl");
    add_srs_wkt (p, 1,
		 "arke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7012\"]],AUTHORITY[\"EPSG\",\"6696\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4696\"]]");
    p = add_epsg_def (first, last, 4697, "epsg", 4697,
		      "IGC 1962 6th Parallel South");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGC 1962 6th Parallel South\",DATUM[\"IGC_1962_");
    add_srs_wkt (p, 1,
		 "Arc_of_the_6th_Parallel_South\",SPHEROID[\"Clarke 1880 (");
    add_srs_wkt (p, 2,
		 "RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6697\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"4697\"]]");
    p = add_epsg_def (first, last, 4698, "epsg", 4698, "IGN 1962 Kerguelen");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=145,-187,103,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN 1962 Kerguelen\",DATUM[\"IGN_1962_Kerguelen");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],TOWGS84[145,-187,103,0,0,0,0],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6698\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"469");
    add_srs_wkt (p, 6, "8\"]]");
    p = add_epsg_def (first, last, 4699, "epsg", 4699, "Le Pouce 1934");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-770.1,158.4,-498.2");
    add_proj4text (p, 1, ",0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Le Pouce 1934\",DATUM[\"Le_Pouce_1934\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7012\"]],TOWGS84[-770.1,158.4,-498.2,0,0,0,0],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6699\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "4699\"]]");
    p = add_epsg_def (first, last, 4700, "epsg", 4700, "IGN Astro 1960");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGN Astro 1960\",DATUM[\"IGN_Astro_1960\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[");
    add_srs_wkt (p, 2,
		 "\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6700\"]],PRIMEM");
    add_srs_wkt (p, 3,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 4,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 5, ",AUTHORITY[\"EPSG\",\"4700\"]]");
    p = add_epsg_def (first, last, 4701, "epsg", 4701, "IGCB 1955");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-79.9,-158,-168.9,0");
    add_proj4text (p, 1, ",0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"IGCB 1955\",DATUM[\"Institut_Geographique_du_Co");
    add_srs_wkt (p, 1,
		 "ngo_Belge_1955\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.");
    add_srs_wkt (p, 2,
		 "145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],TOWGS84[-79.9,");
    add_srs_wkt (p, 3,
		 "-158,-168.9,0,0,0,0],AUTHORITY[\"EPSG\",\"6701\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6, "],AUTHORITY[\"EPSG\",\"4701\"]]");
    p = add_epsg_def (first, last, 4702, "epsg", 4702, "Mauritania 1999");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mauritania 1999\",DATUM[\"Mauritania_1999\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6702\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4702\"]]");
    p = add_epsg_def (first, last, 4703, "epsg", 4703, "Mhast 1951");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mhast 1951\",DATUM[\"Missao_Hidrografico_Angola");
    add_srs_wkt (p, 1,
		 "_y_Sao_Tome_1951\",SPHEROID[\"Clarke 1880 (RGS)\",637824");
    add_srs_wkt (p, 2,
		 "9.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6703\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4703\"]]");
    p = add_epsg_def (first, last, 4704, "epsg", 4704, "Mhast (onshore)");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mhast (onshore)\",DATUM[\"Mhast_onshore\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7022\"]],AUTHORITY[\"EPSG\",\"6704\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4704\"]]");
    p = add_epsg_def (first, last, 4705, "epsg", 4705, "Mhast (offshore)");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Mhast (offshore)\",DATUM[\"Mhast_offshore\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],AUTHORITY[\"EPSG\",\"6705\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 3,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 4,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4705\"]]");
    p = add_epsg_def (first, last, 4706, "epsg", 4706,
		      "Egypt Gulf of Suez S-650 TL");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=helmert +towgs84=-146.21,112.63,4.0");
    add_proj4text (p, 1, "5,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Egypt Gulf of Suez S-650 TL\",DATUM[\"Egypt_Gul");
    add_srs_wkt (p, 1,
		 "f_of_Suez_S_650_TL\",SPHEROID[\"Helmert 1906\",6378200,2");
    add_srs_wkt (p, 2,
		 "98.3,AUTHORITY[\"EPSG\",\"7020\"]],TOWGS84[-146.21,112.6");
    add_srs_wkt (p, 3,
		 "3,4.05,0,0,0,0],AUTHORITY[\"EPSG\",\"6706\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6, "HORITY[\"EPSG\",\"4706\"]]");
    p = add_epsg_def (first, last, 4707, "epsg", 4707, "Tern Island 1961");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=114,-116,-333,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tern Island 1961\",DATUM[\"Tern_Island_1961\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"International 1924\",6378388,297,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7022\"]],TOWGS84[114,-116,-333,0,0,0,0],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6707\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4707\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4708, "epsg", 4708, "Cocos Islands 1965");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=aust_SA +towgs84=-491,-22,435,0,0,0");
    add_proj4text (p, 1, ",0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Cocos Islands 1965\",DATUM[\"Cocos_Islands_1965");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"Australian National Spheroid\",6378160,298");
    add_srs_wkt (p, 2,
		 ".25,AUTHORITY[\"EPSG\",\"7003\"]],TOWGS84[-491,-22,435,0");
    add_srs_wkt (p, 3,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6708\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6, "EPSG\",\"4708\"]]");
    p = add_epsg_def (first, last, 4709, "epsg", 4709, "Iwo Jima 1945");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=145,75,-272,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Iwo Jima 1945\",DATUM[\"Iwo_Jima_1945\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7022\"]],TOWGS84[145,75,-272,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6709\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4709\"]]");
    p = add_epsg_def (first, last, 4710, "epsg", 4710, "St. Helena 1971");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-320,550,-494,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"St. Helena 1971\",DATUM[\"St_Helena_1971\",SPHE");
    add_srs_wkt (p, 1,
		 "ROID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 2,
		 "\",\"7022\"]],TOWGS84[-320,550,-494,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6710\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4710\"]]");
    p = add_epsg_def (first, last, 4711, "epsg", 4711, "Marcus Island 1952");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=124,-234,-25,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Marcus Island 1952\",DATUM[\"Marcus_Island_1952");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],TOWGS84[124,-234,-25,0,0,0,0],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6711\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"471");
    add_srs_wkt (p, 6, "1\"]]");
    p = add_epsg_def (first, last, 4712, "epsg", 4712, "Ascension Island 1958");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-205,107,53,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Ascension Island 1958\",DATUM[\"Ascension_Islan");
    add_srs_wkt (p, 1,
		 "d_1958\",SPHEROID[\"International 1924\",6378388,297,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7022\"]],TOWGS84[-205,107,53,0,0,0,0],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6712\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6, "\"4712\"]]");
    p = add_epsg_def (first, last, 4713, "epsg", 4713, "Ayabelle Lighthouse");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=-79,-129,145,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Ayabelle Lighthouse\",DATUM[\"Ayabelle_Lighthou");
    add_srs_wkt (p, 1,
		 "se\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7012\"]],TOWGS84[-79,-129,145,0,0,0");
    add_srs_wkt (p, 3,
		 ",0],AUTHORITY[\"EPSG\",\"6713\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 5,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 6, "G\",\"4713\"]]");
    p = add_epsg_def (first, last, 4714, "epsg", 4714, "Bellevue");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-127,-769,472,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bellevue\",DATUM[\"Bellevue\",SPHEROID[\"Intern");
    add_srs_wkt (p, 1,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 2,
		 ",TOWGS84[-127,-769,472,0,0,0,0],AUTHORITY[\"EPSG\",\"671");
    add_srs_wkt (p, 3,
		 "4\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4714\"]]");
    p = add_epsg_def (first, last, 4715, "epsg", 4715, "Camp Area Astro");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-104,-129,239,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Camp Area Astro\",DATUM[\"Camp_Area_Astro\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"International 1924\",6378388,297,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7022\"]],TOWGS84[-104,-129,239,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6715\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4715\"]]");
    add_srs_wkt (p, 6, "");
    p = add_epsg_def (first, last, 4716, "epsg", 4716, "Phoenix Islands 1966");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=298,-304,-375,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Phoenix Islands 1966\",DATUM[\"Phoenix_Islands_");
    add_srs_wkt (p, 1,
		 "1966\",SPHEROID[\"International 1924\",6378388,297,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7022\"]],TOWGS84[298,-304,-375,0,0,0,0],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6716\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6, "\"4716\"]]");
    p = add_epsg_def (first, last, 4717, "epsg", 4717, "Cape Canaveral");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=-2,151,181,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Cape Canaveral\",DATUM[\"Cape_Canaveral\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7008\"]],TOWGS84[-2,151,181,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6717\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4717");
    add_srs_wkt (p, 6, "\"]]");
    p = add_epsg_def (first, last, 4718, "epsg", 4718, "Solomon 1968");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Solomon 1968\",DATUM[\"Solomon_1968\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6718\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4718\"]]");
    p = add_epsg_def (first, last, 4719, "epsg", 4719, "Easter Island 1967");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=211,147,111,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Easter Island 1967\",DATUM[\"Easter_Island_1967");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],TOWGS84[211,147,111,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6719\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4719");
    add_srs_wkt (p, 6, "\"]]");
    p = add_epsg_def (first, last, 4720, "epsg", 4720, "Fiji 1986");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS72 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Fiji 1986\",DATUM[\"Fiji_Geodetic_Datum_1986\",");
    add_srs_wkt (p, 1,
		 "SPHEROID[\"WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7043\"]],AUTHORITY[\"EPSG\",\"6720\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 3,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 4,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 5, "\"EPSG\",\"4720\"]]");
    p = add_epsg_def (first, last, 4721, "epsg", 4721, "Fiji 1956");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=265.025,384.929,-194.");
    add_proj4text (p, 1, "046,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Fiji 1956\",DATUM[\"Fiji_1956\",SPHEROID[\"Inte");
    add_srs_wkt (p, 1,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[265.025,384.929,-194.046,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6721\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4721\"]]");
    p = add_epsg_def (first, last, 4722, "epsg", 4722, "South Georgia 1968");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-794,119,-298,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"South Georgia 1968\",DATUM[\"South_Georgia_1968");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"International 1924\",6378388,297,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7022\"]],TOWGS84[-794,119,-298,0,0,0,0],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6722\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 6, "22\"]]");
    p = add_epsg_def (first, last, 4723, "epsg", 4723, "Grand Cayman 1959");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk66 +towgs84=67.8,106.1,138.8,0,");
    add_proj4text (p, 1, "0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Grand Cayman 1959\",DATUM[\"Grand_Cayman_1959\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Clarke 1866\",6378206.4,294.9786982139006,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7008\"]],TOWGS84[67.8,106.1,138.8,0,0");
    add_srs_wkt (p, 3,
		 ",0,0],AUTHORITY[\"EPSG\",\"6723\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6, "PSG\",\"4723\"]]");
    p = add_epsg_def (first, last, 4724, "epsg", 4724, "Diego Garcia 1969");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=208,-435,-229,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Diego Garcia 1969\",DATUM[\"Diego_Garcia_1969\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],TOWGS84[208,-435,-229,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6724\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4724\"");
    add_srs_wkt (p, 6, "]]");
    p = add_epsg_def (first, last, 4725, "epsg", 4725, "Johnston Island 1961");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=189,-79,-202,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Johnston Island 1961\",DATUM[\"Johnston_Island_");
    add_srs_wkt (p, 1,
		 "1961\",SPHEROID[\"International 1924\",6378388,297,AUTHO");
    add_srs_wkt (p, 2,
		 "RITY[\"EPSG\",\"7022\"]],TOWGS84[189,-79,-202,0,0,0,0],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6725\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6, "4725\"]]");
    p = add_epsg_def (first, last, 4726, "epsg", 4726, "Little Cayman 1961");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Little Cayman 1961\",DATUM[\"Little_Cayman_1961");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"Clarke 1866\",6378206.4,294.9786982139006,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6726\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4726\"]]");
    p = add_epsg_def (first, last, 4727, "epsg", 4727, "Midway 1961");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Midway 1961\",DATUM[\"Midway_1961\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],AUTHORITY[\"EPSG\",\"6727\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4727\"]]");
    p = add_epsg_def (first, last, 4728, "epsg", 4728,
		      "Pico de las Nieves 1984");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-307,-92,127,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pico de las Nieves 1984\",DATUM[\"Pico_de_las_N");
    add_srs_wkt (p, 1,
		 "ieves_1984\",SPHEROID[\"International 1924\",6378388,297");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-307,-92,127,0,0,");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"6728\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6, "SG\",\"4728\"]]");
    p = add_epsg_def (first, last, 4729, "epsg", 4729, "Pitcairn 1967");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=185,165,42,0,0,0,0 +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pitcairn 1967\",DATUM[\"Pitcairn_1967\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7022\"]],TOWGS84[185,165,42,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6729\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4729\"]]");
    p = add_epsg_def (first, last, 4730, "epsg", 4730, "Santo 1965");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=170,42,84,0,0,0,0 +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Santo 1965\",DATUM[\"Santo_1965\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],TOWGS84[170,42,84,0,0,0,0],AUTHORITY[\"EPSG\",\"673");
    add_srs_wkt (p, 3,
		 "0\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4730\"]]");
    p = add_epsg_def (first, last, 4731, "epsg", 4731, "Viti Levu 1916");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=51,391,-36,0,0,0,0 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Viti Levu 1916\",DATUM[\"Viti_Levu_1916\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[");
    add_srs_wkt (p, 2,
		 "\"EPSG\",\"7012\"]],TOWGS84[51,391,-36,0,0,0,0],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6731\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4731\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4732, "epsg", 4732, "Marshall Islands 1960");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378270 +b=6356794.343434343 +towgs84=1");
    add_proj4text (p, 1, "02,52,-38,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Marshall Islands 1960\",DATUM[\"Marshall_Island");
    add_srs_wkt (p, 1,
		 "s_1960\",SPHEROID[\"Hough 1960\",6378270,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7053\"]],TOWGS84[102,52,-38,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6732\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4732\"]]");
    add_srs_wkt (p, 6, "");
    p = add_epsg_def (first, last, 4733, "epsg", 4733, "Wake Island 1952");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=276,-57,149,0,0,0,0 +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Wake Island 1952\",DATUM[\"Wake_Island_1952\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"International 1924\",6378388,297,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7022\"]],TOWGS84[276,-57,149,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6733\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4733\"]]");
    add_srs_wkt (p, 6, "");
    p = add_epsg_def (first, last, 4734, "epsg", 4734, "Tristan 1968");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-632,438,-609,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tristan 1968\",DATUM[\"Tristan_1968\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7022\"]],TOWGS84[-632,438,-609,0,0,0,0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6734\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5, "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4734\"]]");
    p = add_epsg_def (first, last, 4735, "epsg", 4735, "Kusaie 1951");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=647,1777,-1124,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kusaie 1951\",DATUM[\"Kusaie_1951\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "22\"]],TOWGS84[647,1777,-1124,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6735\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4735\"]]");
    p = add_epsg_def (first, last, 4736, "epsg", 4736, "Deception Island");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=260,12,-147,0,0,0,0");
    add_proj4text (p, 1, " +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Deception Island\",DATUM[\"Deception_Island\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7012\"]],TOWGS84[260,12,-147,0,0,0,0],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6736\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6, "736\"]]");
    p = add_epsg_def (first, last, 4737, "epsg", 4737, "Korea 2000");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Korea 2000\",DATUM[\"Geocentric_datum_of_Korea\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6737\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4737\"]]");
    p = add_epsg_def (first, last, 4738, "epsg", 4738, "Hong Kong 1963");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378293.645208759 +b=6356617.987679838 ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hong Kong 1963\",DATUM[\"Hong_Kong_1963\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Clarke 1858\",6378293.645208759,294.2606763692569,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7007\"]],AUTHORITY[\"EPSG\",\"6738\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4738\"]]");
    p = add_epsg_def (first, last, 4739, "epsg", 4739, "Hong Kong 1963(67)");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-156,-271,-189,0,0,0,");
    add_proj4text (p, 1, "0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Hong Kong 1963(67)\",DATUM[\"Hong_Kong_1963_67\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],TOWGS84[-156,-271,-189,0,0,0,0],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6739\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4739");
    add_srs_wkt (p, 6, "\"]]");
    p = add_epsg_def (first, last, 4740, "epsg", 4740, "PZ-90");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378136 +b=6356751.361745712 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"PZ-90\",DATUM[\"Parametrop_Zemp_1990\",SPHEROID");
    add_srs_wkt (p, 1,
		 "[\"PZ-90\",6378136,298.257839303,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 2,
		 "54\"]],AUTHORITY[\"EPSG\",\"6740\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 3,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 4,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"4740\"]]");
    p = add_epsg_def (first, last, 4741, "epsg", 4741, "FD54");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"FD54\",DATUM[\"Faroe_Datum_1954\",SPHEROID[\"In");
    add_srs_wkt (p, 1,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 2,
		 "\"]],AUTHORITY[\"EPSG\",\"6741\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 3,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 4,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"4741\"]]");
    p = add_epsg_def (first, last, 4742, "epsg", 4742, "GDM2000");
    add_proj4text (p, 0, "+proj=longlat +ellps=GRS80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"GDM2000\",DATUM[\"Geodetic_Datum_of_Malaysia_20");
    add_srs_wkt (p, 1,
		 "00\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6742\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5, "]],AUTHORITY[\"EPSG\",\"4742\"]]");
    p = add_epsg_def (first, last, 4743, "epsg", 4743, "Karbala 1979");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=clrk80 +towgs84=84.1,-320.1,218.7,0");
    add_proj4text (p, 1, ",0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Karbala 1979\",DATUM[\"Karbala_1979\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],TOWGS84[84.1,-320.1,218.7,0,0,0,0],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6743\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"474");
    add_srs_wkt (p, 6, "3\"]]");
    p = add_epsg_def (first, last, 4744, "epsg", 4744, "Nahrwan 1934");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Nahrwan 1934\",DATUM[\"Nahrwan_1934\",SPHEROID[");
    add_srs_wkt (p, 1,
		 "\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6744\"]],PRIMEM[\"G");
    add_srs_wkt (p, 3,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 4,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 5, "HORITY[\"EPSG\",\"4744\"]]");
    p = add_epsg_def (first, last, 4745, "epsg", 4745, "RD/83");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RD/83\",DATUM[\"Rauenberg_Datum_83\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7004\"]],AUTHORITY[\"EPSG\",\"6745\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 3,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 4,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 5, "TY[\"EPSG\",\"4745\"]]");
    p = add_epsg_def (first, last, 4746, "epsg", 4746, "PD/83");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"PD/83\",DATUM[\"Potsdam_Datum_83\",SPHEROID[\"B");
    add_srs_wkt (p, 1,
		 "essel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 2,
		 "\"7004\"]],AUTHORITY[\"EPSG\",\"6746\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 3,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 4,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"4746\"]]");
    p = add_epsg_def (first, last, 4747, "epsg", 4747, "GR96");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"GR96\",DATUM[\"Greenland_1996\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 1,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6747\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5, "22\"]],AUTHORITY[\"EPSG\",\"4747\"]]");
    p = add_epsg_def (first, last, 4748, "epsg", 4748, "Vanua Levu 1915");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378306.3696 +b=6356571.996 +towgs84=51");
    add_proj4text (p, 1, ",391,-36,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Vanua Levu 1915\",DATUM[\"Vanua_Levu_1915\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"Clarke 1880 (international foot)\",6378306.3696,");
    add_srs_wkt (p, 2,
		 "293.4663076556349,AUTHORITY[\"EPSG\",\"7055\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "51,391,-36,0,0,0,0],AUTHORITY[\"EPSG\",\"6748\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6, ",AUTHORITY[\"EPSG\",\"4748\"]]");
    p = add_epsg_def (first, last, 4749, "epsg", 4749, "RGNC91-93");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RGNC91-93\",DATUM[\"Reseau_Geodesique_de_Nouvel");
    add_srs_wkt (p, 1,
		 "le_Caledonie_91_93\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 2,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0");
    add_srs_wkt (p, 3,
		 ",0,0],AUTHORITY[\"EPSG\",\"6749\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6, "PSG\",\"4749\"]]");
    p = add_epsg_def (first, last, 4750, "epsg", 4750, "ST87 Ouvea");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=-56.263,16.136,-22.8");
    add_proj4text (p, 1, "56,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"ST87 Ouvea\",DATUM[\"ST87_Ouvea\",SPHEROID[\"WG");
    add_srs_wkt (p, 1,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 2,
		 "]],TOWGS84[-56.263,16.136,-22.856,0,0,0,0],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6750\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5, "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4750\"]]");
    p = add_epsg_def (first, last, 4751, "epsg", 4751, "Kertau (RSO)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377295.664 +b=6356094.667915204 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Kertau (RSO)\",DATUM[\"Kertau_RSO\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "Everest 1830 (RSO 1969)\",6377295.664,300.8017,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7056\"]],AUTHORITY[\"EPSG\",\"6751\"]],PRIME");
    add_srs_wkt (p, 3,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 4,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 5, "],AUTHORITY[\"EPSG\",\"4751\"]]");
    p = add_epsg_def (first, last, 4752, "epsg", 4752, "Viti Levu 1912");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378306.3696 +b=6356571.996 +towgs84=51");
    add_proj4text (p, 1, ",391,-36,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Viti Levu 1912\",DATUM[\"Viti_Levu_1912\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Clarke 1880 (international foot)\",6378306.3696,29");
    add_srs_wkt (p, 2,
		 "3.4663076556349,AUTHORITY[\"EPSG\",\"7055\"]],TOWGS84[51");
    add_srs_wkt (p, 3,
		 ",391,-36,0,0,0,0],AUTHORITY[\"EPSG\",\"6752\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6, "THORITY[\"EPSG\",\"4752\"]]");
    p = add_epsg_def (first, last, 4753, "epsg", 4753, "fk89");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"fk89\",DATUM[\"fk89\",SPHEROID[\"International ");
    add_srs_wkt (p, 1,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"6753\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4753\"");
    add_srs_wkt (p, 5, "]]");
    p = add_epsg_def (first, last, 4754, "epsg", 4754, "LGD2006");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-208.406,-109.878,-2.");
    add_proj4text (p, 1, "5764,0,0,0,0 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"LGD2006\",DATUM[\"Libyan_Geodetic_Datum_2006\",");
    add_srs_wkt (p, 1,
		 "SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],TOWGS84[-208.406,-109.878,-2.5764,0,0,");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"6754\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6, "SG\",\"4754\"]]");
    p = add_epsg_def (first, last, 4755, "epsg", 4755, "DGN95");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"DGN95\",DATUM[\"Datum_Geodesi_Nasional_1995\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPS");
    add_srs_wkt (p, 2,
		 "G\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6755\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4755\"]]");
    p = add_epsg_def (first, last, 4756, "epsg", 4756, "VN-2000");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS84 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"VN-2000\",DATUM[\"Vietnam_2000\",SPHEROID[\"WGS");
    add_srs_wkt (p, 1,
		 " 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]");
    add_srs_wkt (p, 2,
		 "],AUTHORITY[\"EPSG\",\"6756\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 4,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"4756\"]]");
    p = add_epsg_def (first, last, 4757, "epsg", 4757, "SVY21");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS84 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"SVY21\",DATUM[\"SVY21\",SPHEROID[\"WGS 84\",637");
    add_srs_wkt (p, 1,
		 "8137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"6757\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4757\"");
    add_srs_wkt (p, 5, "]]");
    p = add_epsg_def (first, last, 4758, "epsg", 4758, "JAD2001");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"JAD2001\",DATUM[\"Jamaica_2001\",SPHEROID[\"WGS");
    add_srs_wkt (p, 1,
		 " 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]");
    add_srs_wkt (p, 2,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6758\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5, "2\"]],AUTHORITY[\"EPSG\",\"4758\"]]");
    p = add_epsg_def (first, last, 4759, "epsg", 4759, "NAD83(NSRS2007)");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatia");
    add_srs_wkt (p, 1,
		 "l_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 2,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6, "Y[\"EPSG\",\"4759\"]]");
    p = add_epsg_def (first, last, 4760, "epsg", 4760, "WGS 66");
    add_proj4text (p, 0, "+proj=longlat +ellps=WGS66 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"WGS 66\",DATUM[\"World_Geodetic_System_1966\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"NWL 9D\",6378145,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 2,
		 "025\"]],AUTHORITY[\"EPSG\",\"6760\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 3,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 4,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"4760\"]]");
    p = add_epsg_def (first, last, 4761, "epsg", 4761, "HTRS96");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"HTRS96\",DATUM[\"Croatian_Terrestrial_Reference");
    add_srs_wkt (p, 1,
		 "_System\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6761\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"476");
    add_srs_wkt (p, 6, "1\"]]");
    p = add_epsg_def (first, last, 4762, "epsg", 4762, "BDA2000");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"BDA2000\",DATUM[\"Bermuda_2000\",SPHEROID[\"WGS");
    add_srs_wkt (p, 1,
		 " 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]");
    add_srs_wkt (p, 2,
		 "],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6762\"]],P");
    add_srs_wkt (p, 3,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 4,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 5, "2\"]],AUTHORITY[\"EPSG\",\"4762\"]]");
    p = add_epsg_def (first, last, 4763, "epsg", 4763, "Pitcairn 2006");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Pitcairn 2006\",DATUM[\"Pitcairn_2006\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 2,
		 "7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"676");
    add_srs_wkt (p, 3,
		 "3\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4763\"]]");
    p = add_epsg_def (first, last, 4764, "epsg", 4764, "RSRGD2000");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RSRGD2000\",DATUM[\"Ross_Sea_Region_Geodetic_Da");
    add_srs_wkt (p, 1,
		 "tum_2000\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6764\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"47");
    add_srs_wkt (p, 6, "64\"]]");
    p = add_epsg_def (first, last, 4765, "epsg", 4765, "Slovenia 1996");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Slovenia 1996\",DATUM[\"Slovenia_Geodetic_Datum");
    add_srs_wkt (p, 1,
		 "_1996\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6765\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4765\"");
    add_srs_wkt (p, 6, "]]");
    p = add_epsg_def (first, last, 4801, "epsg", 4801, "Bern 1898 (Bern)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=bern +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bern 1898 (Bern)\",DATUM[\"CH1903_Bern\",SPHERO");
    add_srs_wkt (p, 1,
		 "ID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"E");
    add_srs_wkt (p, 2,
		 "PSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6801\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Bern\",7.439583333333333,AUTHORITY[\"EPSG\",\"8907\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4801\"]]");
    p = add_epsg_def (first, last, 4802, "epsg", 4802, "Bogota 1975 (Bogota)");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +pm=bogota +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Bogota 1975 (Bogota)\",DATUM[\"Bogota_1975_Bogo");
    add_srs_wkt (p, 1,
		 "ta\",SPHEROID[\"International 1924\",6378388,297,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6802\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Bogota\",-74.08091666666667,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 4,
		 "904\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4802\"]]");
    p = add_epsg_def (first, last, 4803, "epsg", 4803, "Lisbon (Lisbon)");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +pm=lisbon +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lisbon (Lisbon)\",DATUM[\"Lisbon_1937_Lisbon\",");
    add_srs_wkt (p, 1,
		 "SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6803\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Lisbon\",-9.131906111111112,AUTHORITY[\"EPSG\",\"8902\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9122\"]],AUTHORITY[\"EPSG\",\"4803\"]]");
    p = add_epsg_def (first, last, 4804, "epsg", 4804, "Makassar (Jakarta)");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=bessel +towgs84=-587.8,519.75,145.7");
    add_proj4text (p, 1, "6,0,0,0,0 +pm=jakarta +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Makassar (Jakarta)\",DATUM[\"Makassar_Jakarta\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7004\"]],TOWGS84[-587.8,519.75,145.76,0,0");
    add_srs_wkt (p, 3,
		 ",0,0],AUTHORITY[\"EPSG\",\"6804\"]],PRIMEM[\"Jakarta\",1");
    add_srs_wkt (p, 4,
		 "06.8077194444444,AUTHORITY[\"EPSG\",\"8908\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6, ",AUTHORITY[\"EPSG\",\"4804\"]]");
    p = add_epsg_def (first, last, 4805, "epsg", 4805, "MGI (Ferro)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=ferro +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"MGI (Ferro)\",DATUM[\"Militar_Geographische_Ins");
    add_srs_wkt (p, 1,
		 "titut_Ferro\",SPHEROID[\"Bessel 1841\",6377397.155,299.1");
    add_srs_wkt (p, 2,
		 "528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"6805\"]],PRIMEM[\"Ferro\",-17.66666666666667,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8909\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805");
    add_srs_wkt (p, 6, "\"]]");
    p = add_epsg_def (first, last, 4806, "epsg", 4806, "Monte Mario (Rome)");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +pm=rome +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Monte Mario (Rome)\",DATUM[\"Monte_Mario_Rome\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 2,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6806\"]],PRIMEM[\"");
    add_srs_wkt (p, 3,
		 "Rome\",12.45233333333333,AUTHORITY[\"EPSG\",\"8906\"]],U");
    add_srs_wkt (p, 4,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4806\"]]");
    p = add_epsg_def (first, last, 4807, "epsg", 4807, "NTF (Paris)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-168,-60,");
    add_proj4text (p, 1, "320,0,0,0,0 +pm=paris +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Fr");
    add_srs_wkt (p, 1,
		 "ancaise_Paris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2");
    add_srs_wkt (p, 2,
		 ",293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[-168,-60,320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Paris\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"grad\",0.01570796326794897,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6, "05\"]],AUTHORITY[\"EPSG\",\"4807\"]]");
    p = add_epsg_def (first, last, 4808, "epsg", 4808, "Padang (Jakarta)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=jakarta +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Padang (Jakarta)\",DATUM[\"Padang_1884_Jakarta\"");
    add_srs_wkt (p, 1,
		 ",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6808\"]],PR");
    add_srs_wkt (p, 3,
		 "IMEM[\"Jakarta\",106.8077194444444,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8908\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4808\"]]");
    p = add_epsg_def (first, last, 4809, "epsg", 4809, "Belge 1950 (Brussels)");
    add_proj4text (p, 0, "+proj=longlat +ellps=intl +pm=brussels +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Belge 1950 (Brussels)\",DATUM[\"Reseau_National");
    add_srs_wkt (p, 1,
		 "_Belge_1950_Brussels\",SPHEROID[\"International 1924\",6");
    add_srs_wkt (p, 2,
		 "378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6809\"]],PRIMEM[\"Brussels\",4.367975,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8910\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4809\"]");
    add_srs_wkt (p, 6, "]");
    p = add_epsg_def (first, last, 4810, "epsg", 4810, "Tananarive (Paris)");
    add_proj4text (p, 0,
		   "+proj=longlat +ellps=intl +towgs84=-189,-242,-91,0,0,0,0");
    add_proj4text (p, 1, " +pm=paris +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Tananarive (Paris)\",DATUM[\"Tananarive_1925_Pa");
    add_srs_wkt (p, 1,
		 "ris\",SPHEROID[\"International 1924\",6378388,297,AUTHOR");
    add_srs_wkt (p, 2,
		 "ITY[\"EPSG\",\"7022\"]],TOWGS84[-189,-242,-91,0,0,0,0],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6810\"]],PRIMEM[\"Paris\",2.33722917");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.015707963");
    add_srs_wkt (p, 5,
		 "26794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6, ",\"4810\"]]");
    p = add_epsg_def (first, last, 4811, "epsg", 4811, "Voirol 1875 (Paris)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +towgs84=-73,-247,");
    add_proj4text (p, 1, "227,0,0,0,0 +pm=paris +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Voirol 1875 (Paris)\",DATUM[\"Voirol_1875_Paris");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.46602129");
    add_srs_wkt (p, 2,
		 "36265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-73,-247,227");
    add_srs_wkt (p, 3,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6811\"]],PRIMEM[\"Paris\"");
    add_srs_wkt (p, 4,
		 ",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",");
    add_srs_wkt (p, 5,
		 "0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHOR");
    add_srs_wkt (p, 6, "ITY[\"EPSG\",\"4811\"]]");
    p = add_epsg_def (first, last, 4813, "epsg", 4813, "Batavia (Jakarta)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=jakarta +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Batavia (Jakarta)\",DATUM[\"Batavia_Jakarta\",S");
    add_srs_wkt (p, 1,
		 "PHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6813\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Jakarta\",106.8077194444444,AUTHORITY[\"EPSG\",\"89");
    add_srs_wkt (p, 4,
		 "08\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"E");
    add_srs_wkt (p, 5, "PSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4813\"]]");
    p = add_epsg_def (first, last, 4814, "epsg", 4814, "RT38 (Stockholm)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=stockholm +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"RT38 (Stockholm)\",DATUM[\"Stockholm_1938_Stock");
    add_srs_wkt (p, 1,
		 "holm\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6814\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Stockholm\",18.05827777777778,AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"8911\"]],UNIT[\"degree\",0.01745329251994328,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4814\"]]");
    p = add_epsg_def (first, last, 4815, "epsg", 4815, "Greek (Athens)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=athens +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Greek (Athens)\",DATUM[\"Greek_Athens\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6815\"]],PRIMEM[\"A");
    add_srs_wkt (p, 3,
		 "thens\",23.7163375,AUTHORITY[\"EPSG\",\"8912\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5, "]],AUTHORITY[\"EPSG\",\"4815\"]]");
    p = add_epsg_def (first, last, 4816, "epsg", 4816, "Carthage (Paris)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +pm=paris +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Carthage (Paris)\",DATUM[\"Carthage_Paris\",SPH");
    add_srs_wkt (p, 1,
		 "EROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212936265,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6816\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Paris\",2.33722917,AUTHORITY[\"EPSG\",\"8903");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"grad\",0.01570796326794897,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5, ",\"9105\"]],AUTHORITY[\"EPSG\",\"4816\"]]");
    p = add_epsg_def (first, last, 4817, "epsg", 4817, "NGO 1948 (Oslo)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6377492.018 +b=6356173.508712696 +towgs");
    add_proj4text (p, 1,
		   "84=278.3,93,474.5,7.889,0.05,-6.61,6.21 +pm=oslo +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NGO 1948 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHER");
    add_srs_wkt (p, 1,
		 "OID[\"Bessel Modified\",6377492.018,299.1528128,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7005\"]],TOWGS84[278.3,93,474.5,7.889,0.05,");
    add_srs_wkt (p, 3,
		 "-6.61,6.21],AUTHORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\"");
    add_srs_wkt (p, 4,
		 ",10.72291666666667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6, "]],AUTHORITY[\"EPSG\",\"4817\"]]");
    p = add_epsg_def (first, last, 4818, "epsg", 4818, "S-JTSK (Ferro)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=ferro +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"S-JTSK (Ferro)\",DATUM[\"S_JTSK_Ferro\",SPHEROI");
    add_srs_wkt (p, 1,
		 "D[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EP");
    add_srs_wkt (p, 2,
		 "SG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6818\"]],PRIMEM[\"F");
    add_srs_wkt (p, 3,
		 "erro\",-17.66666666666667,AUTHORITY[\"EPSG\",\"8909\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5, "9122\"]],AUTHORITY[\"EPSG\",\"4818\"]]");
    p = add_epsg_def (first, last, 4819, "epsg", 4819,
		      "Nord Sahara 1959 (Paris)");
    add_proj4text (p, 0, "+proj=longlat +ellps=clrk80 +pm=paris +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Nord Sahara 1959 (Paris)\",DATUM[\"Nord_Sahara_");
    add_srs_wkt (p, 1,
		 "1959_Paris\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,");
    add_srs_wkt (p, 2,
		 "293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6819\"]],PRIMEM[\"Paris\",2.33722917,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8903\"]],UNIT[\"grad\",0.01570796326794897,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9105\"]],AUTHORITY[\"EPSG\",\"4819\"]]");
    p = add_epsg_def (first, last, 4820, "epsg", 4820, "Segara (Jakarta)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=jakarta +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Segara (Jakarta)\",DATUM[\"Gunung_Segara_Jakart");
    add_srs_wkt (p, 1,
		 "a\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6820\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Jakarta\",106.8077194444444,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8908\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4820\"]]");
    p = add_epsg_def (first, last, 4821, "epsg", 4821, "Voirol 1879 (Paris)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378249.2 +b=6356515 +pm=paris +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Voirol 1879 (Paris)\",DATUM[\"Voirol_1879_Paris");
    add_srs_wkt (p, 1,
		 "\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.46602129");
    add_srs_wkt (p, 2,
		 "36265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6821\"]],PRIMEM[\"Paris\",2.33722917,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8903\"]],UNIT[\"grad\",0.01570796326794897,AUTHORITY[\"");
    add_srs_wkt (p, 5, "EPSG\",\"9105\"]],AUTHORITY[\"EPSG\",\"4821\"]]");
    p = add_epsg_def (first, last, 4901, "epsg", 4901, "unnamed ellipse");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6376523 +b=6355862.933255573 +pm=2.3372");
    add_proj4text (p, 1, "291666985 +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"unnamed ellipse\",DATUM[\"unknown\",SPHEROID[\"");
    add_srs_wkt (p, 1,
		 "unnamed\",6376523,308.6399999999991]],PRIMEM[\"unnamed\"");
    add_srs_wkt (p, 2,
		 ",2.3372291666985],UNIT[\"degree\",0.0174532925199433],AU");
    add_srs_wkt (p, 3, "THORITY[\"EPSG\",\"4901\"]]");
    p = add_epsg_def (first, last, 4902, "epsg", 4902, "NDG (Paris)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6376523 +b=6355862.933255573 +pm=paris ");
    add_proj4text (p, 1, "+no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"NDG (Paris)\",DATUM[\"Nord_de_Guerre_Paris\",SP");
    add_srs_wkt (p, 1,
		 "HEROID[\"Plessis 1817\",6376523,308.64,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 2,
		 ",\"7027\"]],AUTHORITY[\"EPSG\",\"6902\"]],PRIMEM[\"Paris");
    add_srs_wkt (p, 3,
		 "\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\"");
    add_srs_wkt (p, 4,
		 ",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHO");
    add_srs_wkt (p, 5, "RITY[\"EPSG\",\"4902\"]]");
    p = add_epsg_def (first, last, 4903, "epsg", 4903, "Madrid 1870 (Madrid)");
    add_proj4text (p, 0,
		   "+proj=longlat +a=6378298.3 +b=6356657.142669561 +pm=madr");
    add_proj4text (p, 1, "id +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Madrid 1870 (Madrid)\",DATUM[\"Madrid_1870_Madr");
    add_srs_wkt (p, 1,
		 "id\",SPHEROID[\"Struve 1860\",6378298.3,294.73,AUTHORITY");
    add_srs_wkt (p, 2,
		 "[\"EPSG\",\"7028\"]],AUTHORITY[\"EPSG\",\"6903\"]],PRIME");
    add_srs_wkt (p, 3,
		 "M[\"Madrid\",-3.687938888888889,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "5\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5, "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4903\"]]");
    p = add_epsg_def (first, last, 4904, "epsg", 4904, "Lisbon 1890 (Lisbon)");
    add_proj4text (p, 0, "+proj=longlat +ellps=bessel +pm=lisbon +no_defs");
    add_srs_wkt (p, 0,
		 "GEOGCS[\"Lisbon 1890 (Lisbon)\",DATUM[\"Lisbon_1890_Lisb");
    add_srs_wkt (p, 1,
		 "on\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AU");
    add_srs_wkt (p, 2,
		 "THORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6904\"]");
    add_srs_wkt (p, 3,
		 "],PRIMEM[\"Lisbon\",-9.131906111111112,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8902\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5, "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4904\"]]");
    p = add_epsg_def (first, last, 20004, "epsg", 20004,
		      "Pulkovo 1995 / Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 4\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",4500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20004\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20005, "epsg", 20005,
		      "Pulkovo 1995 / Gauss-Kruger zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 5\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",5500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20005\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20006, "epsg", 20006,
		      "Pulkovo 1995 / Gauss-Kruger zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 6\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",6500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20006\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20007, "epsg", 20007,
		      "Pulkovo 1995 / Gauss-Kruger zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=7500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 7\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",39],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",7500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20007\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20008, "epsg", 20008,
		      "Pulkovo 1995 / Gauss-Kruger zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 8\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",45],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",8500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20008\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20009, "epsg", 20009,
		      "Pulkovo 1995 / Gauss-Kruger zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 9\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",51],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",9500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20009\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20010, "epsg", 20010,
		      "Pulkovo 1995 / Gauss-Kruger zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=10500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 10\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",57],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",10500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "0\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20011, "epsg", 20011,
		      "Pulkovo 1995 / Gauss-Kruger zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=11500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 11\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",63],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",11500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "1\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20012, "epsg", 20012,
		      "Pulkovo 1995 / Gauss-Kruger zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=12500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 12\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",69],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",12500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "2\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20013, "epsg", 20013,
		      "Pulkovo 1995 / Gauss-Kruger zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=13500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 13\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",75],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",13500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "3\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20014, "epsg", 20014,
		      "Pulkovo 1995 / Gauss-Kruger zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=14500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 14\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",81],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",14500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "4\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20015, "epsg", 20015,
		      "Pulkovo 1995 / Gauss-Kruger zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=15500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 15\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",87],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",15500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "5\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20016, "epsg", 20016,
		      "Pulkovo 1995 / Gauss-Kruger zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=16500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 16\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",93],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",16500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "6\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20017, "epsg", 20017,
		      "Pulkovo 1995 / Gauss-Kruger zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=17500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 17\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",99],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",17500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2001");
    add_srs_wkt (p, 12, "7\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20018, "epsg", 20018,
		      "Pulkovo 1995 / Gauss-Kruger zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=18500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 18\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",105],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",18500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "18\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20019, "epsg", 20019,
		      "Pulkovo 1995 / Gauss-Kruger zone 19");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=19500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 19\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",111],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",19500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "19\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20020, "epsg", 20020,
		      "Pulkovo 1995 / Gauss-Kruger zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=20500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 20\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",117],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",20500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "20\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20021, "epsg", 20021,
		      "Pulkovo 1995 / Gauss-Kruger zone 21");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=21500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 21\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",123],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",21500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "21\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20022, "epsg", 20022,
		      "Pulkovo 1995 / Gauss-Kruger zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=22500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 22\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",129],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",22500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "22\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20023, "epsg", 20023,
		      "Pulkovo 1995 / Gauss-Kruger zone 23");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=23500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 23\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",135],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",23500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "23\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20024, "epsg", 20024,
		      "Pulkovo 1995 / Gauss-Kruger zone 24");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=24500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 24\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",24500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "24\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20025, "epsg", 20025,
		      "Pulkovo 1995 / Gauss-Kruger zone 25");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=25500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 25\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",147],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",25500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "25\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20026, "epsg", 20026,
		      "Pulkovo 1995 / Gauss-Kruger zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=26500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 26\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",153],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",26500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "26\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20027, "epsg", 20027,
		      "Pulkovo 1995 / Gauss-Kruger zone 27");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=27500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 27\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",159],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",27500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "27\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20028, "epsg", 20028,
		      "Pulkovo 1995 / Gauss-Kruger zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=28500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 28\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",165],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",28500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "28\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20029, "epsg", 20029,
		      "Pulkovo 1995 / Gauss-Kruger zone 29");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=29500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 29\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",171],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",29500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "29\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20030, "epsg", 20030,
		      "Pulkovo 1995 / Gauss-Kruger zone 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=30500000 +y_0=");
    add_proj4text (p, 1,
		   "0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,");
    add_proj4text (p, 2, "-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 30\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",177],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",30500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "30\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_18 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 20031, "epsg", 20031,
		      "Pulkovo 1995 / Gauss-Kruger zone 31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=31500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16");
    add_proj4text (p, 2, ",-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 31\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-177],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",31500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "31\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20032, "epsg", 20032,
		      "Pulkovo 1995 / Gauss-Kruger zone 32");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=32500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16");
    add_proj4text (p, 2, ",-0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger zone 32\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4200\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",0],PARAMETER[\"central_meridian\",-171],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",1],PARAMETER[\"false_easting\",32500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"200");
    add_srs_wkt (p, 12, "32\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20064, "epsg", 20064,
		      "Pulkovo 1995 / Gauss-Kruger 4N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 4N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",21],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "20064\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20065, "epsg", 20065,
		      "Pulkovo 1995 / Gauss-Kruger 5N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 5N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",27],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "20065\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20066, "epsg", 20066,
		      "Pulkovo 1995 / Gauss-Kruger 6N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 6N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",33],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "20066\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20067, "epsg", 20067,
		      "Pulkovo 1995 / Gauss-Kruger 7N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 7N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",39],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "20067\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20068, "epsg", 20068,
		      "Pulkovo 1995 / Gauss-Kruger 8N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 8N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",45],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "20068\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20069, "epsg", 20069,
		      "Pulkovo 1995 / Gauss-Kruger 9N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 9N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",51],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",5000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "20069\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20070, "epsg", 20070,
		      "Pulkovo 1995 / Gauss-Kruger 10N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 10N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",57],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20070\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20071, "epsg", 20071,
		      "Pulkovo 1995 / Gauss-Kruger 11N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 11N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",63],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20071\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20072, "epsg", 20072,
		      "Pulkovo 1995 / Gauss-Kruger 12N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 12N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",69],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20072\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20073, "epsg", 20073,
		      "Pulkovo 1995 / Gauss-Kruger 13N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 13N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",75],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20073\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20074, "epsg", 20074,
		      "Pulkovo 1995 / Gauss-Kruger 14N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 14N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",81],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20074\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20075, "epsg", 20075,
		      "Pulkovo 1995 / Gauss-Kruger 15N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 15N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",87],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20075\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20076, "epsg", 20076,
		      "Pulkovo 1995 / Gauss-Kruger 16N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 16N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",93],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20076\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20077, "epsg", 20077,
		      "Pulkovo 1995 / Gauss-Kruger 17N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1,
		   "ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0.");
    add_proj4text (p, 2, "12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 17N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"20077\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20078, "epsg", 20078,
		      "Pulkovo 1995 / Gauss-Kruger 18N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 18N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20078\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20079, "epsg", 20079,
		      "Pulkovo 1995 / Gauss-Kruger 19N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 19N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20079\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20080, "epsg", 20080,
		      "Pulkovo 1995 / Gauss-Kruger 20N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 20N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20080\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20081, "epsg", 20081,
		      "Pulkovo 1995 / Gauss-Kruger 21N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 21N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20081\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20082, "epsg", 20082,
		      "Pulkovo 1995 / Gauss-Kruger 22N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 22N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20082\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20083, "epsg", 20083,
		      "Pulkovo 1995 / Gauss-Kruger 23N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 23N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20083\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20084, "epsg", 20084,
		      "Pulkovo 1995 / Gauss-Kruger 24N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 24N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",141],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20084\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20085, "epsg", 20085,
		      "Pulkovo 1995 / Gauss-Kruger 25N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 25N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",147],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20085\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20086, "epsg", 20086,
		      "Pulkovo 1995 / Gauss-Kruger 26N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 26N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",153],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20086\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20087, "epsg", 20087,
		      "Pulkovo 1995 / Gauss-Kruger 27N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 27N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",159],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20087\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20088, "epsg", 20088,
		      "Pulkovo 1995 / Gauss-Kruger 28N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 28N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",165],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20088\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20089, "epsg", 20089,
		      "Pulkovo 1995 / Gauss-Kruger 29N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 29N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",171],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20089\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20090, "epsg", 20090,
		      "Pulkovo 1995 / Gauss-Kruger 30N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-0");
    add_proj4text (p, 2, ".12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 30N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",177],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20090\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20091, "epsg", 20091,
		      "Pulkovo 1995 / Gauss-Kruger 31N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 31N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-177],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20091\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20092, "epsg", 20092,
		      "Pulkovo 1995 / Gauss-Kruger 32N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1,
		   " +ellps=krass +towgs84=24.82,-131.21,-82.66,-0,-0,0.16,-");
    add_proj4text (p, 2, "0.12 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1995 / Gauss-Kruger 32N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1995\",DATUM[\"Pulkovo_1995\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[24.82,-131.21,-82.66,-0,-0,0.16,-0.12],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6200\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4200\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-171],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"20092\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 20135, "epsg", 20135,
		      "Adindan / UTM zone 35N");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Adindan / UTM zone 35N\",GEOGCS[\"Adindan\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Adindan\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 2,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6201\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4201\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 7,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",27],PARAMETER[\"scale_f");
    add_srs_wkt (p, 9,
		 "actor\",0.9996],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20135\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20136, "epsg", 20136,
		      "Adindan / UTM zone 36N");
    add_proj4text (p, 0, "+proj=utm +zone=36 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Adindan / UTM zone 36N\",GEOGCS[\"Adindan\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Adindan\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 2,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6201\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4201\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 7,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",33],PARAMETER[\"scale_f");
    add_srs_wkt (p, 9,
		 "actor\",0.9996],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20136\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20137, "epsg", 20137,
		      "Adindan / UTM zone 37N");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Adindan / UTM zone 37N\",GEOGCS[\"Adindan\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Adindan\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 2,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6201\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4201\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 7,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",39],PARAMETER[\"scale_f");
    add_srs_wkt (p, 9,
		 "actor\",0.9996],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20137\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20138, "epsg", 20138,
		      "Adindan / UTM zone 38N");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Adindan / UTM zone 38N\",GEOGCS[\"Adindan\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Adindan\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.14");
    add_srs_wkt (p, 2,
		 "5,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"6201\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4201\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 7,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"central_meridian\",45],PARAMETER[\"scale_f");
    add_srs_wkt (p, 9,
		 "actor\",0.9996],PARAMETER[\"false_easting\",500000],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"20138\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20248, "epsg", 20248, "AGD66 / AMG zone 48");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 48\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "05],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20248\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20249, "epsg", 20249, "AGD66 / AMG zone 49");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 49\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "11],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20249\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20250, "epsg", 20250, "AGD66 / AMG zone 50");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 50\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "17],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20250\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20251, "epsg", 20251, "AGD66 / AMG zone 51");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 51\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "23],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20251\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20252, "epsg", 20252, "AGD66 / AMG zone 52");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 52\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "29],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20252\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20253, "epsg", 20253, "AGD66 / AMG zone 53");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 53\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "35],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20253\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20254, "epsg", 20254, "AGD66 / AMG zone 54");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 54\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "41],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20254\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20255, "epsg", 20255, "AGD66 / AMG zone 55");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 55\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "47],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20255\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20256, "epsg", 20256, "AGD66 / AMG zone 56");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 56\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "53],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20256\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20257, "epsg", 20257, "AGD66 / AMG zone 57");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 57\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "59],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20257\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20258, "epsg", 20258, "AGD66 / AMG zone 58");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD66 / AMG zone 58\",GEOGCS[\"AGD66\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1966\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6202\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4202\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "65],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20258\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20348, "epsg", 20348, "AGD84 / AMG zone 48");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 48\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "05],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20348\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20349, "epsg", 20349, "AGD84 / AMG zone 49");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 49\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "11],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20349\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20350, "epsg", 20350, "AGD84 / AMG zone 50");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 50\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "17],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20350\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20351, "epsg", 20351, "AGD84 / AMG zone 51");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 51\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "23],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20351\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20352, "epsg", 20352, "AGD84 / AMG zone 52");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 52\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "29],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20352\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20353, "epsg", 20353, "AGD84 / AMG zone 53");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 53\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "35],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20353\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20354, "epsg", 20354, "AGD84 / AMG zone 54");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 54\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "41],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20354\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20355, "epsg", 20355, "AGD84 / AMG zone 55");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 55\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "47],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20355\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20356, "epsg", 20356, "AGD84 / AMG zone 56");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 56\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "53],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20356\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20357, "epsg", 20357, "AGD84 / AMG zone 57");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 57\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "59],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20357\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20358, "epsg", 20358, "AGD84 / AMG zone 58");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"AGD84 / AMG zone 58\",GEOGCS[\"AGD84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Australian_Geodetic_Datum_1984\",SPHEROID[\"Australian N");
    add_srs_wkt (p, 2,
		 "ational Spheroid\",6378160,298.25,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "003\"]],AUTHORITY[\"EPSG\",\"6203\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4203\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "65],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",10000000");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"20358\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20436, "epsg", 20436,
		      "Ain el Abd / UTM zone 36N");
    add_proj4text (p, 0, "+proj=utm +zone=36 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Ain el Abd / UTM zone 36N\",GEOGCS[\"Ain el Abd");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Ain_el_Abd_1970\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4204\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "20436\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 20437, "epsg", 20437,
		      "Ain el Abd / UTM zone 37N");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Ain el Abd / UTM zone 37N\",GEOGCS[\"Ain el Abd");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Ain_el_Abd_1970\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4204\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",39],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "20437\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 20438, "epsg", 20438,
		      "Ain el Abd / UTM zone 38N");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Ain el Abd / UTM zone 38N\",GEOGCS[\"Ain el Abd");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Ain_el_Abd_1970\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4204\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",45],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "20438\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 20439, "epsg", 20439,
		      "Ain el Abd / UTM zone 39N");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Ain el Abd / UTM zone 39N\",GEOGCS[\"Ain el Abd");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Ain_el_Abd_1970\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4204\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",51],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "20439\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 20440, "epsg", 20440,
		      "Ain el Abd / UTM zone 40N");
    add_proj4text (p, 0, "+proj=utm +zone=40 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Ain el Abd / UTM zone 40N\",GEOGCS[\"Ain el Abd");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Ain_el_Abd_1970\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4204\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",57],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "20440\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 20499, "epsg", 20499,
		      "Ain el Abd / Bahrain Grid");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Ain el Abd / Bahrain Grid\",GEOGCS[\"Ain el Abd");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Ain_el_Abd_1970\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6204\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4204\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",51],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "20499\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 20538, "epsg", 20538,
		      "Afgooye / UTM zone 38N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +ellps=krass +towgs84=-43,-163,45,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Afgooye / UTM zone 38N\",GEOGCS[\"Afgooye\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Afgooye\",SPHEROID[\"Krassowsky 1940\",6378245,298.");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[-43,-163,45,0,0,");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"6205\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4205\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",45");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"20538\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 12, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 20539, "epsg", 20539,
		      "Afgooye / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=krass +towgs84=-43,-163,45,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Afgooye / UTM zone 39N\",GEOGCS[\"Afgooye\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Afgooye\",SPHEROID[\"Krassowsky 1940\",6378245,298.");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7024\"]],TOWGS84[-43,-163,45,0,0,");
    add_srs_wkt (p, 3,
		 "0,0],AUTHORITY[\"EPSG\",\"6205\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4205\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",51");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"20539\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 12, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 20790, "epsg", 20790,
		      "Lisbon (Lisbon) / Portuguese National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=39.66666666666666 +lon_0=1 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "200000 +y_0=300000 +ellps=intl +pm=lisbon +units=m +no_d");
    add_proj4text (p, 2, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lisbon (Lisbon) / Portuguese National Grid\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"Lisbon (Lisbon)\",DATUM[\"Lisbon_1937_Lisbon\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"International 1924\",6378388,297,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6803\"]],PRIMEM[\"L");
    add_srs_wkt (p, 4,
		 "isbon\",-9.131906111111112,AUTHORITY[\"EPSG\",\"8902\"]]");
    add_srs_wkt (p, 5,
		 ",UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"9122\"]],AUTHORITY[\"EPSG\",\"4803\"]],UNIT[\"metre\",");
    add_srs_wkt (p, 7,
		 "1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_");
    add_srs_wkt (p, 8,
		 "Mercator\"],PARAMETER[\"latitude_of_origin\",39.66666666");
    add_srs_wkt (p, 9,
		 "666666],PARAMETER[\"central_meridian\",1],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",200000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",300000],AUTHORITY[\"EPSG\",\"20");
    add_srs_wkt (p, 12, "790\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 20791, "epsg", 20791,
		      "Lisbon (Lisbon) / Portuguese Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=39.66666666666666 +lon_0=1 +k=1 +x_0=");
    add_proj4text (p, 1, "0 +y_0=0 +ellps=intl +pm=lisbon +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lisbon (Lisbon) / Portuguese Grid\",GEOGCS[\"Li");
    add_srs_wkt (p, 1,
		 "sbon (Lisbon)\",DATUM[\"Lisbon_1937_Lisbon\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],AUTHORITY[\"EPSG\",\"6803\"]],PRIMEM[\"Lisbon\",-");
    add_srs_wkt (p, 4,
		 "9.131906111111112,AUTHORITY[\"EPSG\",\"8902\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4803\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",39.66666666666666],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"central_meridian\",1],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"20791\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 20822, "epsg", 20822,
		      "Aratu / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Aratu / UTM zone 22S\",GEOGCS[\"Aratu\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Aratu\",SPHEROID[\"International 1924\",6378388,297,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6208\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5,
		 "22\"]],AUTHORITY[\"EPSG\",\"4208\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 7,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "central_meridian\",-51],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"20822\"],AXIS");
    add_srs_wkt (p, 11, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20823, "epsg", 20823,
		      "Aratu / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Aratu / UTM zone 23S\",GEOGCS[\"Aratu\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Aratu\",SPHEROID[\"International 1924\",6378388,297,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6208\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5,
		 "22\"]],AUTHORITY[\"EPSG\",\"4208\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 7,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "central_meridian\",-45],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"20823\"],AXIS");
    add_srs_wkt (p, 11, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20824, "epsg", 20824,
		      "Aratu / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Aratu / UTM zone 24S\",GEOGCS[\"Aratu\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Aratu\",SPHEROID[\"International 1924\",6378388,297,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6208\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5,
		 "22\"]],AUTHORITY[\"EPSG\",\"4208\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 7,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "central_meridian\",-39],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"20824\"],AXIS");
    add_srs_wkt (p, 11, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 20934, "epsg", 20934,
		      "Arc 1950 / UTM zone 34S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +south +a=6378249.145 +b=6356514.9663");
    add_proj4text (p, 1, "98753 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1950 / UTM zone 34S\",GEOGCS[\"Arc 1950\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1950\",SPHEROID[\"Clarke 1880 (Arc)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.4663077,AUTHORITY[\"EPSG\",\"7013\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6209\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4209\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"20934\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 20935, "epsg", 20935,
		      "Arc 1950 / UTM zone 35S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +south +a=6378249.145 +b=6356514.9663");
    add_proj4text (p, 1, "98753 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1950 / UTM zone 35S\",GEOGCS[\"Arc 1950\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1950\",SPHEROID[\"Clarke 1880 (Arc)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.4663077,AUTHORITY[\"EPSG\",\"7013\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6209\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4209\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"20935\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 20936, "epsg", 20936,
		      "Arc 1950 / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +a=6378249.145 +b=6356514.9663");
    add_proj4text (p, 1, "98753 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1950 / UTM zone 36S\",GEOGCS[\"Arc 1950\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1950\",SPHEROID[\"Clarke 1880 (Arc)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.4663077,AUTHORITY[\"EPSG\",\"7013\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6209\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4209\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"20936\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 21035, "epsg", 21035,
		      "Arc 1960 / UTM zone 35S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +south +ellps=clrk80 +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1960 / UTM zone 35S\",GEOGCS[\"Arc 1960\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1960\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6210\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4210\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"21035\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 21036, "epsg", 21036,
		      "Arc 1960 / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +ellps=clrk80 +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1960 / UTM zone 36S\",GEOGCS[\"Arc 1960\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1960\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6210\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4210\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"21036\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 21037, "epsg", 21037,
		      "Arc 1960 / UTM zone 37S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +south +ellps=clrk80 +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1960 / UTM zone 37S\",GEOGCS[\"Arc 1960\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1960\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6210\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4210\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",39],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"21037\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 21095, "epsg", 21095,
		      "Arc 1960 / UTM zone 35N");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1960 / UTM zone 35N\",GEOGCS[\"Arc 1960\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1960\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6210\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4210\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"21095");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21096, "epsg", 21096,
		      "Arc 1960 / UTM zone 36N");
    add_proj4text (p, 0, "+proj=utm +zone=36 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1960 / UTM zone 36N\",GEOGCS[\"Arc 1960\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1960\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6210\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4210\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"21096");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21097, "epsg", 21097,
		      "Arc 1960 / UTM zone 37N");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Arc 1960 / UTM zone 37N\",GEOGCS[\"Arc 1960\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Arc_1960\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6210\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4210\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",39],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"21097");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21100, "epsg", 21100,
		      "Batavia (Jakarta) / NEIEZ (deprecated)");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=110 +k=0.997 +x_0=3900000 +y_0=900000 ");
    add_proj4text (p, 1, "+ellps=bessel +pm=jakarta +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Batavia (Jakarta) / NEIEZ (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Batavia (Jakarta)\",DATUM[\"Batavia_Jakarta\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6813\"]],PRIMEM[\"J");
    add_srs_wkt (p, 4,
		 "akarta\",106.8077194444444,AUTHORITY[\"EPSG\",\"8908\"]]");
    add_srs_wkt (p, 5,
		 ",UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"9122\"]],AUTHORITY[\"EPSG\",\"4813\"]],UNIT[\"metre\",");
    add_srs_wkt (p, 7,
		 "1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Mercator_1S");
    add_srs_wkt (p, 8,
		 "P\"],PARAMETER[\"central_meridian\",110],PARAMETER[\"sca");
    add_srs_wkt (p, 9,
		 "le_factor\",0.997],PARAMETER[\"false_easting\",3900000],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"false_northing\",900000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"21100\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 21148, "epsg", 21148,
		      "Batavia / UTM zone 48S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=bessel +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Batavia / UTM zone 48S\",GEOGCS[\"Batavia\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Batavia\",SPHEROID[\"Bessel 1841\",6377397.155,299.");
    add_srs_wkt (p, 2,
		 "1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6211\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4211\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",105],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 9,
		 "tor\",0.9996],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"21");
    add_srs_wkt (p, 11,
		 "148\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21149, "epsg", 21149,
		      "Batavia / UTM zone 49S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=bessel +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Batavia / UTM zone 49S\",GEOGCS[\"Batavia\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Batavia\",SPHEROID[\"Bessel 1841\",6377397.155,299.");
    add_srs_wkt (p, 2,
		 "1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6211\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4211\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",111],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 9,
		 "tor\",0.9996],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"21");
    add_srs_wkt (p, 11,
		 "149\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21150, "epsg", 21150,
		      "Batavia / UTM zone 50S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=bessel +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Batavia / UTM zone 50S\",GEOGCS[\"Batavia\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Batavia\",SPHEROID[\"Bessel 1841\",6377397.155,299.");
    add_srs_wkt (p, 2,
		 "1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6211\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4211\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",117],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 9,
		 "tor\",0.9996],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"21");
    add_srs_wkt (p, 11,
		 "150\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21291, "epsg", 21291,
		      "Barbados 1938 / British West Indies Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-62 +k=0.9995000000000001 +x");
    add_proj4text (p, 1,
		   "_0=400000 +y_0=0 +ellps=clrk80 +towgs84=31.95,300.99,419");
    add_proj4text (p, 2, ".19,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Barbados 1938 / British West Indies Grid\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Barbados 1938\",DATUM[\"Barbados_1938\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7012\"]],TOWGS84[31.95,300.99,419.19,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6212\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"421");
    add_srs_wkt (p, 7,
		 "2\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",-62],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",0.9995],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "400000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12,
		 "\",\"21291\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 13, "ORTH]]");
    p = add_epsg_def (first, last, 21292, "epsg", 21292,
		      "Barbados 1938 / Barbados National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=13.17638888888889 +lon_0=-59.55972222");
    add_proj4text (p, 1,
		   "222222 +k=0.9999986 +x_0=30000 +y_0=75000 +ellps=clrk80 ");
    add_proj4text (p, 2,
		   "+towgs84=31.95,300.99,419.19,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Barbados 1938 / Barbados National Grid\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Barbados 1938\",DATUM[\"Barbados_1938\",SPHEROID[\"Cl");
    add_srs_wkt (p, 2,
		 "arke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7012\"]],TOWGS84[31.95,300.99,419.19,0,0,0,0],AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"6212\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 5,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4212\"");
    add_srs_wkt (p, 7,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 9,
		 "igin\",13.17638888888889],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 10,
		 ",-59.55972222222222],PARAMETER[\"scale_factor\",0.999998");
    add_srs_wkt (p, 11,
		 "6],PARAMETER[\"false_easting\",30000],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",75000],AUTHORITY[\"EPSG\",\"21292\"],AXIS[\"E");
    add_srs_wkt (p, 13, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21413, "epsg", 21413,
		      "Beijing 1954 / Gauss-Kruger zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=13500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 13\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",75],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",135");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21413\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21414, "epsg", 21414,
		      "Beijing 1954 / Gauss-Kruger zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=14500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 14\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",81],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",145");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21414\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21415, "epsg", 21415,
		      "Beijing 1954 / Gauss-Kruger zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=15500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 15\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",87],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",155");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21415\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21416, "epsg", 21416,
		      "Beijing 1954 / Gauss-Kruger zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=16500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 16\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",93],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",165");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21416\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21417, "epsg", 21417,
		      "Beijing 1954 / Gauss-Kruger zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=17500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 17\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",175");
    add_srs_wkt (p, 10,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21417\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21418, "epsg", 21418,
		      "Beijing 1954 / Gauss-Kruger zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=18500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 18\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",18");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"21418\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21419, "epsg", 21419,
		      "Beijing 1954 / Gauss-Kruger zone 19");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=19500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 19\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",19");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"21419\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21420, "epsg", 21420,
		      "Beijing 1954 / Gauss-Kruger zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=20500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 20\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",20");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"21420\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21421, "epsg", 21421,
		      "Beijing 1954 / Gauss-Kruger zone 21");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=21500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 21\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",21");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"21421\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21422, "epsg", 21422,
		      "Beijing 1954 / Gauss-Kruger zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=22500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 22\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",22");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"21422\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21423, "epsg", 21423,
		      "Beijing 1954 / Gauss-Kruger zone 23");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=23500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger zone 23\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",23");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"21423\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21453, "epsg", 21453,
		      "Beijing 1954 / Gauss-Kruger CM 75E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 75E\",GEOGCS[\"B");
    add_srs_wkt (p, 1,
		 "eijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",75],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"21453\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21454, "epsg", 21454,
		      "Beijing 1954 / Gauss-Kruger CM 81E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 81E\",GEOGCS[\"B");
    add_srs_wkt (p, 1,
		 "eijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",81],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"21454\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21455, "epsg", 21455,
		      "Beijing 1954 / Gauss-Kruger CM 87E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 87E\",GEOGCS[\"B");
    add_srs_wkt (p, 1,
		 "eijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",87],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"21455\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21456, "epsg", 21456,
		      "Beijing 1954 / Gauss-Kruger CM 93E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 93E\",GEOGCS[\"B");
    add_srs_wkt (p, 1,
		 "eijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",93],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"21456\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21457, "epsg", 21457,
		      "Beijing 1954 / Gauss-Kruger CM 99E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 99E\",GEOGCS[\"B");
    add_srs_wkt (p, 1,
		 "eijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 10,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"21457\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21458, "epsg", 21458,
		      "Beijing 1954 / Gauss-Kruger CM 105E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 105E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21458\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21459, "epsg", 21459,
		      "Beijing 1954 / Gauss-Kruger CM 111E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 111E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21459\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21460, "epsg", 21460,
		      "Beijing 1954 / Gauss-Kruger CM 117E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 117E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21460\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21461, "epsg", 21461,
		      "Beijing 1954 / Gauss-Kruger CM 123E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 123E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21461\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21462, "epsg", 21462,
		      "Beijing 1954 / Gauss-Kruger CM 129E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 129E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21462\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21463, "epsg", 21463,
		      "Beijing 1954 / Gauss-Kruger CM 135E");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger CM 135E\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"21463\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21473, "epsg", 21473,
		      "Beijing 1954 / Gauss-Kruger 13N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 13N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",7");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"21473\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21474, "epsg", 21474,
		      "Beijing 1954 / Gauss-Kruger 14N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 14N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"21474\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21475, "epsg", 21475,
		      "Beijing 1954 / Gauss-Kruger 15N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 15N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"21475\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21476, "epsg", 21476,
		      "Beijing 1954 / Gauss-Kruger 16N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 16N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"21476\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21477, "epsg", 21477,
		      "Beijing 1954 / Gauss-Kruger 17N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 17N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"21477\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21478, "epsg", 21478,
		      "Beijing 1954 / Gauss-Kruger 18N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 18N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "05],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"21478\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21479, "epsg", 21479,
		      "Beijing 1954 / Gauss-Kruger 19N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 19N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "11],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"21479\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21480, "epsg", 21480,
		      "Beijing 1954 / Gauss-Kruger 20N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 20N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "17],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"21480\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21481, "epsg", 21481,
		      "Beijing 1954 / Gauss-Kruger 21N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 21N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "23],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"21481\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21482, "epsg", 21482,
		      "Beijing 1954 / Gauss-Kruger 22N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 22N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "29],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"21482\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21483, "epsg", 21483,
		      "Beijing 1954 / Gauss-Kruger 23N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Beijing 1954 / Gauss-Kruger 23N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Beijing 1954\",DATUM[\"Beijing_1954\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],AUTHORITY[\"EPSG\",\"6214\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4214\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 9,
		 "35],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"21483\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 21500, "epsg", 21500,
		      "Belge 1950 (Brussels) / Belge Lambert 50");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.83333333333334 +lat_2=51.16666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=90 +lon_0=0 +x_0=150000 +y_0=5400000 +ellps=i");
    add_proj4text (p, 2, "ntl +pm=brussels +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Belge 1950 (Brussels) / Belge Lambert 50\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Belge 1950 (Brussels)\",DATUM[\"Reseau_National_Bel");
    add_srs_wkt (p, 2,
		 "ge_1950_Brussels\",SPHEROID[\"International 1924\",63783");
    add_srs_wkt (p, 3,
		 "88,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6809\"]],PRIMEM[\"Brussels\",4.367975,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 5,
		 "\",\"8910\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4809\"]],UN");
    add_srs_wkt (p, 7,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",49.83333333333334],PARAMETER[\"standard_paral");
    add_srs_wkt (p, 10,
		 "lel_2\",51.16666666666666],PARAMETER[\"latitude_of_origi");
    add_srs_wkt (p, 11,
		 "n\",90],PARAMETER[\"central_meridian\",0],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",150000],PARAMETER[\"false_northing\",54000");
    add_srs_wkt (p, 13,
		 "00],AUTHORITY[\"EPSG\",\"21500\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 14, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 21780, "epsg", 21780,
		      "Bern 1898 (Bern) / LV03C");
    add_proj4text (p, 0,
		   "+proj=somerc +lat_0=46.95240555555556 +lon_0=0 +k_0=1 +x");
    add_proj4text (p, 1,
		   "_0=0 +y_0=0 +ellps=bessel +pm=bern +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bern 1898 (Bern) / LV03C\",GEOGCS[\"Bern 1898 (");
    add_srs_wkt (p, 1,
		 "Bern)\",DATUM[\"CH1903_Bern\",SPHEROID[\"Bessel 1841\",6");
    add_srs_wkt (p, 2,
		 "377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6801\"]],PRIMEM[\"Bern\",7.43958333333");
    add_srs_wkt (p, 4,
		 "3333,AUTHORITY[\"EPSG\",\"8907\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4801\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "9001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"latitude_of_center\",46.95240555555556],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "longitude_of_center\",0],PARAMETER[\"azimuth\",90],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"rectified_grid_angle\",90],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 11,
		 "or\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"21780\"],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "EAST],AXIS[\"X\",NORTH]]");
    p = add_epsg_def (first, last, 21781, "epsg", 21781, "CH1903 / LV03");
    add_proj4text (p, 0,
		   "+proj=somerc +lat_0=46.95240555555556 +lon_0=7.439583333");
    add_proj4text (p, 1,
		   "333333 +k_0=1 +x_0=600000 +y_0=200000 +ellps=bessel +tow");
    add_proj4text (p, 2,
		   "gs84=674.374,15.056,405.346,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"CH1903 / LV03\",GEOGCS[\"CH1903\",DATUM[\"CH190");
    add_srs_wkt (p, 1,
		 "3\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7004\"]],TOWGS84[674.374,15.056,405.34");
    add_srs_wkt (p, 3,
		 "6,0,0,0,0],AUTHORITY[\"EPSG\",\"6149\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4149\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_center\",46.95240555555556],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"longitude_of_center\",7.439583333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "azimuth\",90],PARAMETER[\"rectified_grid_angle\",90],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",6");
    add_srs_wkt (p, 12,
		 "00000],PARAMETER[\"false_northing\",200000],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"21781\"],AXIS[\"Y\",EAST],AXIS[\"X\",NORTH]]");
    p = add_epsg_def (first, last, 21782, "epsg", 21782, "CH1903 / LV03C-G");
    add_proj4text (p, 0,
		   "+proj=somerc +lat_0=46.95240555555556 +lon_0=7.439583333");
    add_proj4text (p, 1,
		   "333333 +k_0=1 +x_0=0 +y_0=0 +ellps=bessel +towgs84=674.3");
    add_proj4text (p, 2, "74,15.056,405.346,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"CH1903 / LV03C-G\",GEOGCS[\"CH1903\",DATUM[\"CH");
    add_srs_wkt (p, 1,
		 "1903\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7004\"]],TOWGS84[674.374,15.056,405");
    add_srs_wkt (p, 3,
		 ".346,0,0,0,0],AUTHORITY[\"EPSG\",\"6149\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4149\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_center\",46.95240555555556],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"longitude_of_center\",7.439583333333333],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"azimuth\",90],PARAMETER[\"rectified_grid_angle\",90");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"21782\"],AXIS[\"Y\",EAST],AXIS[\"X\",NORTH]]");
    p = add_epsg_def (first, last, 21817, "epsg", 21817,
		      "Bogota 1975 / UTM zone 17N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=intl +towgs84=307,304,-318,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / UTM zone 17N (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Bogota 1975\",DATUM[\"Bogota_1975\",SPHEROID[\"Inter");
    add_srs_wkt (p, 2,
		 "national 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",\"621");
    add_srs_wkt (p, 4,
		 "8\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-81],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"21817\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21818, "epsg", 21818,
		      "Bogota 1975 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=intl +towgs84=307,304,-318,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / UTM zone 18N\",GEOGCS[\"Bogota 19");
    add_srs_wkt (p, 1,
		 "75\",DATUM[\"Bogota_1975\",SPHEROID[\"International 1924");
    add_srs_wkt (p, 2,
		 "\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[307");
    add_srs_wkt (p, 3,
		 ",304,-318,0,0,0,0],AUTHORITY[\"EPSG\",\"6218\"]],PRIMEM[");
    add_srs_wkt (p, 4,
		 "\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"deg");
    add_srs_wkt (p, 5,
		 "ree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"metre\",1,AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"]");
    add_srs_wkt (p, 8,
		 ",PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central");
    add_srs_wkt (p, 9,
		 "_meridian\",-75],PARAMETER[\"scale_factor\",0.9996],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",500000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",0],AUTHORITY[\"EPSG\",\"21818\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 21891, "epsg", 21891,
		      "Bogota 1975 / Colombia West zone (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-77.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia West zone (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"Bogota 1975\",DATUM[\"Bogota_1975\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],TOWGS84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6218\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",4.59");
    add_srs_wkt (p, 9,
		 "9047222222222],PARAMETER[\"central_meridian\",-77.080916");
    add_srs_wkt (p, 10,
		 "66666667],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",1000000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"21891\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 21892, "epsg", 21892,
		      "Bogota 1975 / Colombia Bogota zone (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-74.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia Bogota zone (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Bogota 1975\",DATUM[\"Bogota_1975\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7022\"]],TOWGS84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6218\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",4.59");
    add_srs_wkt (p, 9,
		 "9047222222222],PARAMETER[\"central_meridian\",-74.080916");
    add_srs_wkt (p, 10,
		 "66666667],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",1000000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"21892\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 21893, "epsg", 21893,
		      "Bogota 1975 / Colombia East Central zone (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-71.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia East Central zone (depre");
    add_srs_wkt (p, 1,
		 "cated)\",GEOGCS[\"Bogota 1975\",DATUM[\"Bogota_1975\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"International 1924\",6378388,297,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7022\"]],TOWGS84[307,304,-318,0,0,0,0],AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"6218\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 5,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4218\"]]");
    add_srs_wkt (p, 7,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 9,
		 "in\",4.599047222222222],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 10,
		 "71.08091666666667],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",1000000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",1000000],AUTHORITY[\"EPSG\",\"21893\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 21894, "epsg", 21894,
		      "Bogota 1975 / Colombia East (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-68.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia East (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Bogota 1975\",DATUM[\"Bogota_1975\",SPHEROID[\"Inte");
    add_srs_wkt (p, 2,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 4,
		 "18\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",4.59904722");
    add_srs_wkt (p, 9,
		 "2222222],PARAMETER[\"central_meridian\",-68.080916666666");
    add_srs_wkt (p, 10,
		 "67],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",1000000],PARAMETER[\"false_northing\",1000000],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"21894\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 13, "ORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_19 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 21896, "epsg", 21896,
		      "Bogota 1975 / Colombia West zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-77.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia West zone\",GEOGCS[\"Bog");
    add_srs_wkt (p, 1,
		 "ota 1975\",DATUM[\"Bogota_1975\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 2,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS");
    add_srs_wkt (p, 3,
		 "84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",\"6218\"]],P");
    add_srs_wkt (p, 4,
		 "RIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT");
    add_srs_wkt (p, 5,
		 "[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"912");
    add_srs_wkt (p, 6,
		 "2\"]],AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"metre\",1,AUT");
    add_srs_wkt (p, 7,
		 "HORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merca");
    add_srs_wkt (p, 8,
		 "tor\"],PARAMETER[\"latitude_of_origin\",4.59904722222222");
    add_srs_wkt (p, 9,
		 "2],PARAMETER[\"central_meridian\",-77.08091666666667],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "1000000],PARAMETER[\"false_northing\",1000000],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"21896\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 21897, "epsg", 21897,
		      "Bogota 1975 / Colombia Bogota zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-74.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia Bogota zone\",GEOGCS[\"B");
    add_srs_wkt (p, 1,
		 "ogota 1975\",DATUM[\"Bogota_1975\",SPHEROID[\"Internatio");
    add_srs_wkt (p, 2,
		 "nal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",\"6218\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",4.599047222222");
    add_srs_wkt (p, 9,
		 "222],PARAMETER[\"central_meridian\",-74.08091666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",1000000],PARAMETER[\"false_northing\",1000000],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"21897\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]");
    add_srs_wkt (p, 13, "]");
    p = add_epsg_def (first, last, 21898, "epsg", 21898,
		      "Bogota 1975 / Colombia East Central zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-71.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia East Central zone\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Bogota 1975\",DATUM[\"Bogota_1975\",SPHEROID[\"Inte");
    add_srs_wkt (p, 2,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[307,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 4,
		 "18\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",4.59904722");
    add_srs_wkt (p, 9,
		 "2222222],PARAMETER[\"central_meridian\",-71.080916666666");
    add_srs_wkt (p, 10,
		 "67],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",1000000],PARAMETER[\"false_northing\",1000000],AUT");
    add_srs_wkt (p, 12,
		 "HORITY[\"EPSG\",\"21898\"],AXIS[\"X\",NORTH],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "EAST]]");
    p = add_epsg_def (first, last, 21899, "epsg", 21899,
		      "Bogota 1975 / Colombia East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.599047222222222 +lon_0=-68.08091666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=1000000 +y_0=1000000 +ellps=intl +towgs");
    add_proj4text (p, 2, "84=307,304,-318,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Bogota 1975 / Colombia East\",GEOGCS[\"Bogota 1");
    add_srs_wkt (p, 1,
		 "975\",DATUM[\"Bogota_1975\",SPHEROID[\"International 192");
    add_srs_wkt (p, 2,
		 "4\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[30");
    add_srs_wkt (p, 3,
		 "7,304,-318,0,0,0,0],AUTHORITY[\"EPSG\",\"6218\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4218\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",4.599047222222222],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"central_meridian\",-68.08091666666667],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",10000");
    add_srs_wkt (p, 11,
		 "00],PARAMETER[\"false_northing\",1000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 12, "SG\",\"21899\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22032, "epsg", 22032,
		      "Camacupa / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=clrk80 +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Camacupa / UTM zone 32S\",GEOGCS[\"Camacupa\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Camacupa\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6220\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4220\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",9],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",0.9996],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "22032\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 22033, "epsg", 22033,
		      "Camacupa / UTM zone 33S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +south +ellps=clrk80 +units=m +no_def");
    add_proj4text (p, 1, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Camacupa / UTM zone 33S\",GEOGCS[\"Camacupa\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Camacupa\",SPHEROID[\"Clarke 1880 (RGS)\",6378249");
    add_srs_wkt (p, 2,
		 ".145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6220\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4220\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",15],PARAMETER[\"scale");
    add_srs_wkt (p, 9,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",500000],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"22033\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 22091, "epsg", 22091,
		      "Camacupa / TM 11.30 SE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=11.5 +k=0.9996 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Camacupa / TM 11.30 SE\",GEOGCS[\"Camacupa\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Camacupa\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.");
    add_srs_wkt (p, 2,
		 "145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6220\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4220\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",11.5],PARAMETER[\"scal");
    add_srs_wkt (p, 9,
		 "e_factor\",0.9996],PARAMETER[\"false_easting\",500000],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",10000000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11,
		 ",\"22091\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 22092, "epsg", 22092, "Camacupa / TM 12 SE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=10000000 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Camacupa / TM 12 SE\",GEOGCS[\"Camacupa\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Camacupa\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145");
    add_srs_wkt (p, 2,
		 ",293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6220\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4220\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",12],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"220");
    add_srs_wkt (p, 11,
		 "92\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22171, "epsg", 22171,
		      "POSGAR 98 / Argentina 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-72 +k=1 +x_0=1500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 98 / Argentina 1\",GEOGCS[\"POSGAR 98\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1998\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6190\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4190\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-90],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-72],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",1500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"22171\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 22172, "epsg", 22172,
		      "POSGAR 98 / Argentina 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-69 +k=1 +x_0=2500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 98 / Argentina 2\",GEOGCS[\"POSGAR 98\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1998\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6190\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4190\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-90],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-69],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",2500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"22172\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 22173, "epsg", 22173,
		      "POSGAR 98 / Argentina 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-66 +k=1 +x_0=3500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 98 / Argentina 3\",GEOGCS[\"POSGAR 98\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1998\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6190\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4190\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-90],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-66],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",3500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"22173\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 22174, "epsg", 22174,
		      "POSGAR 98 / Argentina 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-63 +k=1 +x_0=4500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 98 / Argentina 4\",GEOGCS[\"POSGAR 98\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1998\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6190\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4190\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-90],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-63],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",4500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"22174\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 22175, "epsg", 22175,
		      "POSGAR 98 / Argentina 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-60 +k=1 +x_0=5500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 98 / Argentina 5\",GEOGCS[\"POSGAR 98\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1998\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6190\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4190\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-90],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-60],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",5500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"22175\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 22176, "epsg", 22176,
		      "POSGAR 98 / Argentina 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-57 +k=1 +x_0=6500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 98 / Argentina 6\",GEOGCS[\"POSGAR 98\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1998\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6190\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4190\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-90],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-57],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",6500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"22176\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 22177, "epsg", 22177,
		      "POSGAR 98 / Argentina 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-54 +k=1 +x_0=7500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 98 / Argentina 7\",GEOGCS[\"POSGAR 98\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1998\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6190\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4190\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-90],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-54],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",7500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"22177\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 22181, "epsg", 22181,
		      "POSGAR 94 / Argentina 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-72 +k=1 +x_0=1500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 94 / Argentina 1\",GEOGCS[\"POSGAR 94\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1994\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6694");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4694\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-90],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-72],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",1],PARAMETER[\"false_easting\",1500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"22181\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22182, "epsg", 22182,
		      "POSGAR 94 / Argentina 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-69 +k=1 +x_0=2500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 94 / Argentina 2\",GEOGCS[\"POSGAR 94\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1994\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6694");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4694\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-90],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-69],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",1],PARAMETER[\"false_easting\",2500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"22182\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22183, "epsg", 22183,
		      "POSGAR 94 / Argentina 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-66 +k=1 +x_0=3500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 94 / Argentina 3\",GEOGCS[\"POSGAR 94\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1994\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6694");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4694\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-90],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-66],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",1],PARAMETER[\"false_easting\",3500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"22183\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22184, "epsg", 22184,
		      "POSGAR 94 / Argentina 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-63 +k=1 +x_0=4500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 94 / Argentina 4\",GEOGCS[\"POSGAR 94\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1994\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6694");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4694\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-90],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-63],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",1],PARAMETER[\"false_easting\",4500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"22184\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22185, "epsg", 22185,
		      "POSGAR 94 / Argentina 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-60 +k=1 +x_0=5500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 94 / Argentina 5\",GEOGCS[\"POSGAR 94\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1994\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6694");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4694\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-90],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-60],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",1],PARAMETER[\"false_easting\",5500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"22185\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22186, "epsg", 22186,
		      "POSGAR 94 / Argentina 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-57 +k=1 +x_0=6500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 94 / Argentina 6\",GEOGCS[\"POSGAR 94\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1994\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6694");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4694\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-90],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-57],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",1],PARAMETER[\"false_easting\",6500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"22186\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22187, "epsg", 22187,
		      "POSGAR 94 / Argentina 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-54 +k=1 +x_0=7500000 +y_0");
    add_proj4text (p, 1,
		   "=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"POSGAR 94 / Argentina 7\",GEOGCS[\"POSGAR 94\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Posiciones_Geodesicas_Argentinas_1994\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6694");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4694\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",-90],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-54],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",1],PARAMETER[\"false_easting\",7500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"22187\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22191, "epsg", 22191,
		      "Campo Inchauspe / Argentina 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-72 +k=1 +x_0=1500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / Argentina 1\",GEOGCS[\"Campo ");
    add_srs_wkt (p, 1,
		 "Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",-90],PARAMETER[\"central_meridian\",-72],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",1500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"22191\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22192, "epsg", 22192,
		      "Campo Inchauspe / Argentina 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-69 +k=1 +x_0=2500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / Argentina 2\",GEOGCS[\"Campo ");
    add_srs_wkt (p, 1,
		 "Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",-90],PARAMETER[\"central_meridian\",-69],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",2500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"22192\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22193, "epsg", 22193,
		      "Campo Inchauspe / Argentina 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-66 +k=1 +x_0=3500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / Argentina 3\",GEOGCS[\"Campo ");
    add_srs_wkt (p, 1,
		 "Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",-90],PARAMETER[\"central_meridian\",-66],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",3500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"22193\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22194, "epsg", 22194,
		      "Campo Inchauspe / Argentina 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-63 +k=1 +x_0=4500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / Argentina 4\",GEOGCS[\"Campo ");
    add_srs_wkt (p, 1,
		 "Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",-90],PARAMETER[\"central_meridian\",-63],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",4500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"22194\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22195, "epsg", 22195,
		      "Campo Inchauspe / Argentina 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-60 +k=1 +x_0=5500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / Argentina 5\",GEOGCS[\"Campo ");
    add_srs_wkt (p, 1,
		 "Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",-90],PARAMETER[\"central_meridian\",-60],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",5500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"22195\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22196, "epsg", 22196,
		      "Campo Inchauspe / Argentina 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-57 +k=1 +x_0=6500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / Argentina 6\",GEOGCS[\"Campo ");
    add_srs_wkt (p, 1,
		 "Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",-90],PARAMETER[\"central_meridian\",-57],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",6500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"22196\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22197, "epsg", 22197,
		      "Campo Inchauspe / Argentina 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-90 +lon_0=-54 +k=1 +x_0=7500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Campo Inchauspe / Argentina 7\",GEOGCS[\"Campo ");
    add_srs_wkt (p, 1,
		 "Inchauspe\",DATUM[\"Campo_Inchauspe\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"6221\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 5,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 6,
		 "\"4221\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]");
    add_srs_wkt (p, 7,
		 "],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitu");
    add_srs_wkt (p, 8,
		 "de_of_origin\",-90],PARAMETER[\"central_meridian\",-54],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",7500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 11, "SG\",\"22197\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 22234, "epsg", 22234, "Cape / UTM zone 34S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +south +a=6378249.145 +b=6356514.9663");
    add_proj4text (p, 1, "98753 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Cape / UTM zone 34S\",GEOGCS[\"Cape\",DATUM[\"C");
    add_srs_wkt (p, 1,
		 "ape\",SPHEROID[\"Clarke 1880 (Arc)\",6378249.145,293.466");
    add_srs_wkt (p, 2,
		 "3077,AUTHORITY[\"EPSG\",\"7013\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6222\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4222\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",21],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 9,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"22234");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22235, "epsg", 22235, "Cape / UTM zone 35S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +south +a=6378249.145 +b=6356514.9663");
    add_proj4text (p, 1, "98753 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Cape / UTM zone 35S\",GEOGCS[\"Cape\",DATUM[\"C");
    add_srs_wkt (p, 1,
		 "ape\",SPHEROID[\"Clarke 1880 (Arc)\",6378249.145,293.466");
    add_srs_wkt (p, 2,
		 "3077,AUTHORITY[\"EPSG\",\"7013\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6222\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4222\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",27],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 9,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"22235");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22236, "epsg", 22236, "Cape / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +a=6378249.145 +b=6356514.9663");
    add_proj4text (p, 1, "98753 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Cape / UTM zone 36S\",GEOGCS[\"Cape\",DATUM[\"C");
    add_srs_wkt (p, 1,
		 "ape\",SPHEROID[\"Clarke 1880 (Arc)\",6378249.145,293.466");
    add_srs_wkt (p, 2,
		 "3077,AUTHORITY[\"EPSG\",\"7013\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6222\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4222\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",33],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 9,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_northing\",10000000],AUTHORITY[\"EPSG\",\"22236");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22332, "epsg", 22332,
		      "Carthage / UTM zone 32N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +a=6378249.2 +b=6356515 +datum=cartha");
    add_proj4text (p, 1, "ge +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Carthage / UTM zone 32N\",GEOGCS[\"Carthage\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Carthage\",SPHEROID[\"Clarke 1880 (IGN)\",6378249");
    add_srs_wkt (p, 2,
		 ".2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6223\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"422");
    add_srs_wkt (p, 6,
		 "3\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",9],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11,
		 ",\"22332\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 22391, "epsg", 22391,
		      "Carthage / Nord Tunisie");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36 +lat_0=36 +lon_0=9.9 +k_0=0.99962554");
    add_proj4text (p, 1,
		   "4 +x_0=500000 +y_0=300000 +a=6378249.2 +b=6356515 +datum");
    add_proj4text (p, 2, "=carthage +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Carthage / Nord Tunisie\",GEOGCS[\"Carthage\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Carthage\",SPHEROID[\"Clarke 1880 (IGN)\",6378249");
    add_srs_wkt (p, 2,
		 ".2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6223\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"422");
    add_srs_wkt (p, 6,
		 "3\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",36],PARAMETER[\"central_meridian\",9.9");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.999625544],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",500000],PARAMETER[\"false_northing\",30000");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"22391\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 22392, "epsg", 22392,
		      "Carthage / Sud Tunisie");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.3 +lat_0=33.3 +lon_0=9.9 +k_0=0.9996");
    add_proj4text (p, 1,
		   "25769 +x_0=500000 +y_0=300000 +a=6378249.2 +b=6356515 +d");
    add_proj4text (p, 2, "atum=carthage +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Carthage / Sud Tunisie\",GEOGCS[\"Carthage\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Carthage\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.");
    add_srs_wkt (p, 2,
		 "2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6223\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4223");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lati");
    add_srs_wkt (p, 8,
		 "tude_of_origin\",33.3],PARAMETER[\"central_meridian\",9.");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",0.999625769],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",500000],PARAMETER[\"false_northing\",3000");
    add_srs_wkt (p, 11,
		 "00],AUTHORITY[\"EPSG\",\"22392\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 22521, "epsg", 22521,
		      "Corrego Alegre / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=intl +towgs84=-206,172,");
    add_proj4text (p, 1, "-6,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Corrego Alegre / UTM zone 21S\",GEOGCS[\"Correg");
    add_srs_wkt (p, 1,
		 "o Alegre\",DATUM[\"Corrego_Alegre\",SPHEROID[\"Internati");
    add_srs_wkt (p, 2,
		 "onal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[-206,172,-6,0,0,0,0],AUTHORITY[\"EPSG\",\"6225\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4225\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-57],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"22521\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22522, "epsg", 22522,
		      "Corrego Alegre / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=intl +towgs84=-206,172,");
    add_proj4text (p, 1, "-6,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Corrego Alegre / UTM zone 22S\",GEOGCS[\"Correg");
    add_srs_wkt (p, 1,
		 "o Alegre\",DATUM[\"Corrego_Alegre\",SPHEROID[\"Internati");
    add_srs_wkt (p, 2,
		 "onal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[-206,172,-6,0,0,0,0],AUTHORITY[\"EPSG\",\"6225\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4225\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-51],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"22522\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22523, "epsg", 22523,
		      "Corrego Alegre / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=intl +towgs84=-206,172,");
    add_proj4text (p, 1, "-6,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Corrego Alegre / UTM zone 23S\",GEOGCS[\"Correg");
    add_srs_wkt (p, 1,
		 "o Alegre\",DATUM[\"Corrego_Alegre\",SPHEROID[\"Internati");
    add_srs_wkt (p, 2,
		 "onal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[-206,172,-6,0,0,0,0],AUTHORITY[\"EPSG\",\"6225\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4225\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-45],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"22523\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22524, "epsg", 22524,
		      "Corrego Alegre / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=intl +towgs84=-206,172,");
    add_proj4text (p, 1, "-6,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Corrego Alegre / UTM zone 24S\",GEOGCS[\"Correg");
    add_srs_wkt (p, 1,
		 "o Alegre\",DATUM[\"Corrego_Alegre\",SPHEROID[\"Internati");
    add_srs_wkt (p, 2,
		 "onal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[-206,172,-6,0,0,0,0],AUTHORITY[\"EPSG\",\"6225\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4225\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-39],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"22524\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22525, "epsg", 22525,
		      "Corrego Alegre / UTM zone 25S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=intl +towgs84=-206,172,");
    add_proj4text (p, 1, "-6,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Corrego Alegre / UTM zone 25S\",GEOGCS[\"Correg");
    add_srs_wkt (p, 1,
		 "o Alegre\",DATUM[\"Corrego_Alegre\",SPHEROID[\"Internati");
    add_srs_wkt (p, 2,
		 "onal 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[-206,172,-6,0,0,0,0],AUTHORITY[\"EPSG\",\"6225\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 5,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 6,
		 "122\"]],AUTHORITY[\"EPSG\",\"4225\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-33],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"22525\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22700, "epsg", 22700,
		      "Deir ez Zor / Levant Zone");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.65 +lat_0=34.65 +lon_0=37.35 +k_0=0.");
    add_proj4text (p, 1,
		   "9996256 +x_0=300000 +y_0=300000 +a=6378249.2 +b=6356515 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Deir ez Zor / Levant Zone\",GEOGCS[\"Deir ez Zo");
    add_srs_wkt (p, 1,
		 "r\",DATUM[\"Deir_ez_Zor\",SPHEROID[\"Clarke 1880 (IGN)\"");
    add_srs_wkt (p, 2,
		 ",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6227\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4227\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"latitude_of_origin\",34.65],PARAMETER[\"central_mer");
    add_srs_wkt (p, 9,
		 "idian\",37.35],PARAMETER[\"scale_factor\",0.9996256],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",300000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",300000],AUTHORITY[\"EPSG\",\"22700\"],AXIS[\"X\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 22770, "epsg", 22770,
		      "Deir ez Zor / Syria Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.65 +lat_0=34.65 +lon_0=37.35 +k_0=0.");
    add_proj4text (p, 1,
		   "9996256 +x_0=300000 +y_0=300000 +a=6378249.2 +b=6356515 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Deir ez Zor / Syria Lambert\",GEOGCS[\"Deir ez ");
    add_srs_wkt (p, 1,
		 "Zor\",DATUM[\"Deir_ez_Zor\",SPHEROID[\"Clarke 1880 (IGN)");
    add_srs_wkt (p, 2,
		 "\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6227\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4227\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",34.65],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",37.35],PARAMETER[\"scale_factor\",0.9996256],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",300000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",300000],AUTHORITY[\"EPSG\",\"22770\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 22780, "epsg", 22780,
		      "Deir ez Zor / Levant Stereographic");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=34.2 +lon_0=39.15 +k=0.9995341 +x_0=");
    add_proj4text (p, 1, "0 +y_0=0 +a=6378249.2 +b=6356515 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Deir ez Zor / Levant Stereographic\",GEOGCS[\"D");
    add_srs_wkt (p, 1,
		 "eir ez Zor\",DATUM[\"Deir_ez_Zor\",SPHEROID[\"Clarke 188");
    add_srs_wkt (p, 2,
		 "0 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7011\"]],AUTHORITY[\"EPSG\",\"6227\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4227\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Oblique_Stereographic\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",34.2],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",39.15],PARAMETER[\"scale_factor\",0.9995341],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",0],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"22780\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 22832, "epsg", 22832,
		      "Douala / UTM zone 32N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +a=6378249.2 +b=6356515 +units=m +no_");
    add_proj4text (p, 1, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Douala / UTM zone 32N (deprecated)\",GEOGCS[\"D");
    add_srs_wkt (p, 1,
		 "ouala\",DATUM[\"Douala\",SPHEROID[\"Clarke 1880 (IGN)\",");
    add_srs_wkt (p, 2,
		 "6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6228\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4228\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",9],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"22832\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 22991, "epsg", 22991,
		      "Egypt 1907 / Blue Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=35 +k=1 +x_0=300000 +y_0=11");
    add_proj4text (p, 1, "00000 +ellps=helmert +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Egypt 1907 / Blue Belt\",GEOGCS[\"Egypt 1907\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Egypt_1907\",SPHEROID[\"Helmert 1906\",6378200,2");
    add_srs_wkt (p, 2,
		 "98.3,AUTHORITY[\"EPSG\",\"7020\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6229\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4229\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",30],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"central_meridian\",35],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_northing\",1100000],AUTHORITY[\"EPSG\",\"22991\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22992, "epsg", 22992,
		      "Egypt 1907 / Red Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=31 +k=1 +x_0=615000 +y_0=81");
    add_proj4text (p, 1, "0000 +ellps=helmert +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Egypt 1907 / Red Belt\",GEOGCS[\"Egypt 1907\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Egypt_1907\",SPHEROID[\"Helmert 1906\",6378200,29");
    add_srs_wkt (p, 2,
		 "8.3,AUTHORITY[\"EPSG\",\"7020\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "229\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4229\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 7,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",30],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",31],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 9,
		 "\",1],PARAMETER[\"false_easting\",615000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",810000],AUTHORITY[\"EPSG\",\"22992\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22993, "epsg", 22993,
		      "Egypt 1907 / Purple Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=27 +k=1 +x_0=700000 +y_0=20");
    add_proj4text (p, 1, "0000 +ellps=helmert +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Egypt 1907 / Purple Belt\",GEOGCS[\"Egypt 1907\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Egypt_1907\",SPHEROID[\"Helmert 1906\",6378200,");
    add_srs_wkt (p, 2,
		 "298.3,AUTHORITY[\"EPSG\",\"7020\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6229\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4229\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",30],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"central_meridian\",27],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",1],PARAMETER[\"false_easting\",700000],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_northing\",200000],AUTHORITY[\"EPSG\",\"22993\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 22994, "epsg", 22994,
		      "Egypt 1907 / Extended Purple Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=27 +k=1 +x_0=700000 +y_0=12");
    add_proj4text (p, 1, "00000 +ellps=helmert +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Egypt 1907 / Extended Purple Belt\",GEOGCS[\"Eg");
    add_srs_wkt (p, 1,
		 "ypt 1907\",DATUM[\"Egypt_1907\",SPHEROID[\"Helmert 1906\"");
    add_srs_wkt (p, 2,
		 ",6378200,298.3,AUTHORITY[\"EPSG\",\"7020\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6229\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4229\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",30],PARAMETER[\"central_meridian\",27],PARAMETER[\"sc");
    add_srs_wkt (p, 9,
		 "ale_factor\",1],PARAMETER[\"false_easting\",700000],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_northing\",1200000],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 11,
		 "2994\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 23028, "epsg", 23028, "ED50 / UTM zone 28N");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 28N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",-15],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 9,
		 "tor\",0.9996],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23028\"],");
    add_srs_wkt (p, 11, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23029, "epsg", 23029, "ED50 / UTM zone 29N");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 29N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",-9],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23029\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23030, "epsg", 23030, "ED50 / UTM zone 30N");
    add_proj4text (p, 0, "+proj=utm +zone=30 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 30N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",-3],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23030\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23031, "epsg", 23031, "ED50 / UTM zone 31N");
    add_proj4text (p, 0, "+proj=utm +zone=31 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 31N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",3],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23031\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23032, "epsg", 23032, "ED50 / UTM zone 32N");
    add_proj4text (p, 0, "+proj=utm +zone=32 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 32N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",9],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23032\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23033, "epsg", 23033, "ED50 / UTM zone 33N");
    add_proj4text (p, 0, "+proj=utm +zone=33 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 33N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",15],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23033\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23034, "epsg", 23034, "ED50 / UTM zone 34N");
    add_proj4text (p, 0, "+proj=utm +zone=34 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 34N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",21],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23034\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23035, "epsg", 23035, "ED50 / UTM zone 35N");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 35N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",27],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23035\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23036, "epsg", 23036, "ED50 / UTM zone 36N");
    add_proj4text (p, 0, "+proj=utm +zone=36 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 36N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",33],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23036\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23037, "epsg", 23037, "ED50 / UTM zone 37N");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 37N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",39],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23037\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23038, "epsg", 23038, "ED50 / UTM zone 38N");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / UTM zone 38N\",GEOGCS[\"ED50\",DATUM[\"E");
    add_srs_wkt (p, 1,
		 "uropean_Datum_1950\",SPHEROID[\"International 1924\",637");
    add_srs_wkt (p, 2,
		 "8388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",45],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23038\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23090, "epsg", 23090, "ED50 / TM 0 N");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=0 +k=0.9996 +x_0=500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM 0 N\",GEOGCS[\"ED50\",DATUM[\"Europea");
    add_srs_wkt (p, 1,
		 "n_Datum_1950\",SPHEROID[\"International 1924\",6378388,2");
    add_srs_wkt (p, 2,
		 "97,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 3,
		 "30\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",0],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"23090\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23095, "epsg", 23095, "ED50 / TM 5 NE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=5 +k=0.9996 +x_0=500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ED50 / TM 5 NE\",GEOGCS[\"ED50\",DATUM[\"Europe");
    add_srs_wkt (p, 1,
		 "an_Datum_1950\",SPHEROID[\"International 1924\",6378388,");
    add_srs_wkt (p, 2,
		 "297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "230\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4230\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 7,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"central_meridian\",5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 9,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"23095\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23239, "epsg", 23239,
		      "Fahud / UTM zone 39N");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fahud / UTM zone 39N\",GEOGCS[\"Fahud\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Fahud\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.4");
    add_srs_wkt (p, 2,
		 "65,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 3,
		 "32\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4232\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",51],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 9,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"23239\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23240, "epsg", 23240,
		      "Fahud / UTM zone 40N");
    add_proj4text (p, 0, "+proj=utm +zone=40 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Fahud / UTM zone 40N\",GEOGCS[\"Fahud\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Fahud\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.4");
    add_srs_wkt (p, 2,
		 "65,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 3,
		 "32\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4232\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",57],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 9,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"23240\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23433, "epsg", 23433,
		      "Garoua / UTM zone 33N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +a=6378249.2 +b=6356515 +units=m +no_");
    add_proj4text (p, 1, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Garoua / UTM zone 33N (deprecated)\",GEOGCS[\"G");
    add_srs_wkt (p, 1,
		 "aroua\",DATUM[\"Garoua\",SPHEROID[\"Clarke 1880 (IGN)\",");
    add_srs_wkt (p, 2,
		 "6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6234\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4234\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",15],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"23433\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 23700, "epsg", 23700, "HD72 / EOV");
    add_proj4text (p, 0,
		   "+proj=somerc +lat_0=47.14439372222222 +lon_0=19.04857177");
    add_proj4text (p, 1,
		   "777778 +k_0=0.99993 +x_0=650000 +y_0=200000 +ellps=GRS67");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"HD72 / EOV\",GEOGCS[\"HD72\",DATUM[\"Hungarian_");
    add_srs_wkt (p, 1,
		 "Datum_1972\",SPHEROID[\"GRS 1967\",6378160,298.247167427");
    add_srs_wkt (p, 2,
		 ",AUTHORITY[\"EPSG\",\"7036\"]],AUTHORITY[\"EPSG\",\"6237");
    add_srs_wkt (p, 3,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 4,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4237\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Hotine_Obl");
    add_srs_wkt (p, 7,
		 "ique_Mercator\"],PARAMETER[\"latitude_of_center\",47.144");
    add_srs_wkt (p, 8,
		 "39372222222],PARAMETER[\"longitude_of_center\",19.048571");
    add_srs_wkt (p, 9,
		 "77777778],PARAMETER[\"azimuth\",90],PARAMETER[\"rectifie");
    add_srs_wkt (p, 10,
		 "d_grid_angle\",90],PARAMETER[\"scale_factor\",0.99993],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",650000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",200000],AUTHORITY[\"EPSG\",\"23700\"],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"X\",NORTH]]");
    p = add_epsg_def (first, last, 23830, "epsg", 23830,
		      "DGN95 / Indonesia TM-3 zone 46.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=94.5 +k=0.9999 +x_0=200000 +");
    add_proj4text (p, 1,
		   "y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 46.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",94.5],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "99],PARAMETER[\"false_easting\",200000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",1500000],AUTHORITY[\"EPSG\",\"23830\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23831, "epsg", 23831,
		      "DGN95 / Indonesia TM-3 zone 47.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=97.5 +k=0.9999 +x_0=200000 +");
    add_proj4text (p, 1,
		   "y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 47.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",97.5],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "99],PARAMETER[\"false_easting\",200000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",1500000],AUTHORITY[\"EPSG\",\"23831\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23832, "epsg", 23832,
		      "DGN95 / Indonesia TM-3 zone 47.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=100.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 47.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",100.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23832\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23833, "epsg", 23833,
		      "DGN95 / Indonesia TM-3 zone 48.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=103.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 48.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",103.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23833\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23834, "epsg", 23834,
		      "DGN95 / Indonesia TM-3 zone 48.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=106.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 48.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",106.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23834\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23835, "epsg", 23835,
		      "DGN95 / Indonesia TM-3 zone 49.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=109.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 49.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",109.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23835\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23836, "epsg", 23836,
		      "DGN95 / Indonesia TM-3 zone 49.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=112.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 49.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",112.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23836\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23837, "epsg", 23837,
		      "DGN95 / Indonesia TM-3 zone 50.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=115.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 50.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",115.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23837\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23838, "epsg", 23838,
		      "DGN95 / Indonesia TM-3 zone 50.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=118.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 50.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",118.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23838\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23839, "epsg", 23839,
		      "DGN95 / Indonesia TM-3 zone 51.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=121.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 51.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",121.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23839\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23840, "epsg", 23840,
		      "DGN95 / Indonesia TM-3 zone 51.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=124.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 51.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",124.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23840\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23841, "epsg", 23841,
		      "DGN95 / Indonesia TM-3 zone 52.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=127.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 52.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",127.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23841\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23842, "epsg", 23842,
		      "DGN95 / Indonesia TM-3 zone 52.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=130.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 52.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",130.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23842\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23843, "epsg", 23843,
		      "DGN95 / Indonesia TM-3 zone 53.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=133.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 53.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",133.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23843\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23844, "epsg", 23844,
		      "DGN95 / Indonesia TM-3 zone 53.2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=136.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 53.2\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",136.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23844\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23845, "epsg", 23845,
		      "DGN95 / Indonesia TM-3 zone 54.1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=139.5 +k=0.9999 +x_0=200000 ");
    add_proj4text (p, 1,
		   "+y_0=1500000 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / Indonesia TM-3 zone 54.1\",GEOGCS[\"DGN");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Datum_Geodesi_Nasional_1995\",SPHEROID[\"WG");
    add_srs_wkt (p, 2,
		 "S 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 8,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",139.5],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "999],PARAMETER[\"false_easting\",200000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",1500000],AUTHORITY[\"EPSG\",\"23845\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 23846, "epsg", 23846, "ID74 / UTM zone 46N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 46N\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",93],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"23846\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 23847, "epsg", 23847, "ID74 / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 47N\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"23847\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 23848, "epsg", 23848, "ID74 / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 48N\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"23848\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 23849, "epsg", 23849, "ID74 / UTM zone 49N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 49N\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"23849\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 23850, "epsg", 23850, "ID74 / UTM zone 50N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 50N\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"23850\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 23851, "epsg", 23851, "ID74 / UTM zone 51N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 51N\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"23851\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 23852, "epsg", 23852, "ID74 / UTM zone 52N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 52N\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"23852\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 23853, "epsg", 23853,
		      "ID74 / UTM zone 53N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +a=6378160 +b=6356774.50408554 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 53N (deprecated)\",GEOGCS[\"ID7");
    add_srs_wkt (p, 1,
		 "4\",DATUM[\"Indonesian_Datum_1974\",SPHEROID[\"Indonesia");
    add_srs_wkt (p, 2,
		 "n National Spheroid\",6378160,298.247,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7021\"]],AUTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",135],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"23853\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23866, "epsg", 23866,
		      "DGN95 / UTM zone 46N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 46N\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",93],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"23866\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23867, "epsg", 23867,
		      "DGN95 / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 47N\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",99],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"23867\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23868, "epsg", 23868,
		      "DGN95 / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 48N\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",105],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"23868\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23869, "epsg", 23869,
		      "DGN95 / UTM zone 49N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 49N\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",111],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"23869\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23870, "epsg", 23870,
		      "DGN95 / UTM zone 50N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 50N\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",117],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"23870\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23871, "epsg", 23871,
		      "DGN95 / UTM zone 51N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 51N\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",123],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"23871\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23872, "epsg", 23872,
		      "DGN95 / UTM zone 52N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 52N\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",129],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"23872\"],AXIS[\"Easting\",EAST],A");
    add_srs_wkt (p, 12, "XIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23877, "epsg", 23877,
		      "DGN95 / UTM zone 47S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 47S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",99],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 11,
		 "00000],AUTHORITY[\"EPSG\",\"23877\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23878, "epsg", 23878,
		      "DGN95 / UTM zone 48S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 48S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",105],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"23878\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23879, "epsg", 23879,
		      "DGN95 / UTM zone 49S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 49S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",111],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"23879\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23880, "epsg", 23880,
		      "DGN95 / UTM zone 50S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 50S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",117],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"23880\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23881, "epsg", 23881,
		      "DGN95 / UTM zone 51S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 51S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",123],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"23881\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23882, "epsg", 23882,
		      "DGN95 / UTM zone 52S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 52S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",129],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"23882\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23883, "epsg", 23883,
		      "DGN95 / UTM zone 53S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 53S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",135],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"23883\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23884, "epsg", 23884,
		      "DGN95 / UTM zone 54S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +ellps=WGS84 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DGN95 / UTM zone 54S\",GEOGCS[\"DGN95\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Datum_Geodesi_Nasional_1995\",SPHEROID[\"WGS 84\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,");
    add_srs_wkt (p, 3,
		 "0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6755\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4755\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "0000000],AUTHORITY[\"EPSG\",\"23884\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 12, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23886, "epsg", 23886,
		      "ID74 / UTM zone 46S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 46S (deprecated)\",GEOGCS[\"ID7");
    add_srs_wkt (p, 1,
		 "4\",DATUM[\"Indonesian_Datum_1974\",SPHEROID[\"Indonesia");
    add_srs_wkt (p, 2,
		 "n National Spheroid\",6378160,298.247,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7021\"]],AUTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",93],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 11,
		 "00000],AUTHORITY[\"EPSG\",\"23886\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23887, "epsg", 23887, "ID74 / UTM zone 47S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 47S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",10000000],AUTHORIT");
    add_srs_wkt (p, 11,
		 "Y[\"EPSG\",\"23887\"],AXIS[\"Easting\",EAST],AXIS[\"Nort");
    add_srs_wkt (p, 12, "hing\",NORTH]]");
    p = add_epsg_def (first, last, 23888, "epsg", 23888, "ID74 / UTM zone 48S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 48S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"23888\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 23889, "epsg", 23889, "ID74 / UTM zone 49S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 49S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"23889\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 23890, "epsg", 23890, "ID74 / UTM zone 50S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 50S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"23890\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 23891, "epsg", 23891, "ID74 / UTM zone 51S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 51S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"23891\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 23892, "epsg", 23892, "ID74 / UTM zone 52S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 52S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"23892\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 23893, "epsg", 23893, "ID74 / UTM zone 53S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 53S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"23893\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 23894, "epsg", 23894, "ID74 / UTM zone 54S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +a=6378160 +b=6356774.50408554");
    add_proj4text (p, 1, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ID74 / UTM zone 54S\",GEOGCS[\"ID74\",DATUM[\"I");
    add_srs_wkt (p, 1,
		 "ndonesian_Datum_1974\",SPHEROID[\"Indonesian National Sp");
    add_srs_wkt (p, 2,
		 "heroid\",6378160,298.247,AUTHORITY[\"EPSG\",\"7021\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6238\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4238\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",141],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",10000000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"23894\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 23946, "epsg", 23946,
		      "Indian 1954 / UTM zone 46N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +a=6377276.345 +b=6356075.41314024 +t");
    add_proj4text (p, 1, "owgs84=217,823,299,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1954 / UTM zone 46N\",GEOGCS[\"Indian 19");
    add_srs_wkt (p, 1,
		 "54\",DATUM[\"Indian_1954\",SPHEROID[\"Everest 1830 (1937");
    add_srs_wkt (p, 2,
		 " Adjustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7015\"]],TOWGS84[217,823,299,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6239\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4239\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",93],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23946\"],A");
    add_srs_wkt (p, 12, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23947, "epsg", 23947,
		      "Indian 1954 / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +a=6377276.345 +b=6356075.41314024 +t");
    add_proj4text (p, 1, "owgs84=217,823,299,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1954 / UTM zone 47N\",GEOGCS[\"Indian 19");
    add_srs_wkt (p, 1,
		 "54\",DATUM[\"Indian_1954\",SPHEROID[\"Everest 1830 (1937");
    add_srs_wkt (p, 2,
		 " Adjustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7015\"]],TOWGS84[217,823,299,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6239\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4239\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",99],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",0.9996],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23947\"],A");
    add_srs_wkt (p, 12, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 23948, "epsg", 23948,
		      "Indian 1954 / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +a=6377276.345 +b=6356075.41314024 +t");
    add_proj4text (p, 1, "owgs84=217,823,299,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1954 / UTM zone 48N\",GEOGCS[\"Indian 19");
    add_srs_wkt (p, 1,
		 "54\",DATUM[\"Indian_1954\",SPHEROID[\"Everest 1830 (1937");
    add_srs_wkt (p, 2,
		 " Adjustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7015\"]],TOWGS84[217,823,299,0,0,0,0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6239\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4239\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",105],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 10,
		 "tor\",0.9996],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"23948\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24047, "epsg", 24047,
		      "Indian 1975 / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +a=6377276.345 +b=6356075.41314024 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1975 / UTM zone 47N\",GEOGCS[\"Indian 19");
    add_srs_wkt (p, 1,
		 "75\",DATUM[\"Indian_1975\",SPHEROID[\"Everest 1830 (1937");
    add_srs_wkt (p, 2,
		 " Adjustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7015\"]],AUTHORITY[\"EPSG\",\"6240\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4240\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",99],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"24047\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24048, "epsg", 24048,
		      "Indian 1975 / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +a=6377276.345 +b=6356075.41314024 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Indian 1975 / UTM zone 48N\",GEOGCS[\"Indian 19");
    add_srs_wkt (p, 1,
		 "75\",DATUM[\"Indian_1975\",SPHEROID[\"Everest 1830 (1937");
    add_srs_wkt (p, 2,
		 " Adjustment)\",6377276.345,300.8017,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7015\"]],AUTHORITY[\"EPSG\",\"6240\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4240\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",105],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",500000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"24048\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24100, "epsg", 24100,
		      "Jamaica 1875 / Jamaica (Old Grid)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18 +lat_0=18 +lon_0=-77 +k_0=1 +x_0=167");
    add_proj4text (p, 1,
		   "638.49597 +y_0=121918.90616 +a=6378249.144808011 +b=6356");
    add_proj4text (p, 2, "514.966204134 +to_meter=0.3047972654 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Jamaica 1875 / Jamaica (Old Grid)\",GEOGCS[\"Ja");
    add_srs_wkt (p, 1,
		 "maica 1875\",DATUM[\"Jamaica_1875\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "80\",6378249.144808011,293.4663076556349,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7034\"]],AUTHORITY[\"EPSG\",\"6241\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4241\"]],UNIT[\"Clarke's foot\",0.304797");
    add_srs_wkt (p, 7,
		 "2654,AUTHORITY[\"EPSG\",\"9005\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 8,
		 "Conformal_Conic_1SP\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "18],PARAMETER[\"central_meridian\",-77],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",1],PARAMETER[\"false_easting\",550000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",400000],AUTHORITY[\"EPSG\",\"2410");
    add_srs_wkt (p, 12,
		 "0\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24200, "epsg", 24200,
		      "JAD69 / Jamaica National Grid");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18 +lat_0=18 +lon_0=-77 +k_0=1 +x_0=250");
    add_proj4text (p, 1, "000 +y_0=150000 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"JAD69 / Jamaica National Grid\",GEOGCS[\"JAD69\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Jamaica_1969\",SPHEROID[\"Clarke 1866\",6378206");
    add_srs_wkt (p, 2,
		 ".4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6242\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"424");
    add_srs_wkt (p, 6,
		 "2\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",18],PARAMETER[\"central_meridian\",-77");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",250000],PARAMETER[\"false_northing\",150000],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"24200\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 12, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 24305, "epsg", 24305,
		      "Kalianpur 1937 / UTM zone 45N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=45 +a=6377276.345 +b=6356075.41314024 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1937 / UTM zone 45N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1937\",DATUM[\"Kalianpur_1937\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1937 Adjustment)\",6377276.345,300.8017,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6144\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4144\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",87],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"24305\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24306, "epsg", 24306,
		      "Kalianpur 1937 / UTM zone 46N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +a=6377276.345 +b=6356075.41314024 +u");
    add_proj4text (p, 1, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1937 / UTM zone 46N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1937\",DATUM[\"Kalianpur_1937\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1937 Adjustment)\",6377276.345,300.8017,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6144\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4144\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",93],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"24306\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24311, "epsg", 24311,
		      "Kalianpur 1962 / UTM zone 41N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=41 +a=6377301.243 +b=6356100.230165384 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1962 / UTM zone 41N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1962\",DATUM[\"Kalianpur_1962\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1962 Definition)\",6377301.243,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7044\"]],AUTHORITY[\"EPSG\",\"6145\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4145\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",63],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",0],AUTHORITY[\"EPSG\",\"24311\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24312, "epsg", 24312,
		      "Kalianpur 1962 / UTM zone 42N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +a=6377301.243 +b=6356100.230165384 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1962 / UTM zone 42N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1962\",DATUM[\"Kalianpur_1962\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1962 Definition)\",6377301.243,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7044\"]],AUTHORITY[\"EPSG\",\"6145\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4145\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",69],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",0],AUTHORITY[\"EPSG\",\"24312\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24313, "epsg", 24313,
		      "Kalianpur 1962 / UTM zone 43N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=43 +a=6377301.243 +b=6356100.230165384 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1962 / UTM zone 43N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1962\",DATUM[\"Kalianpur_1962\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1962 Definition)\",6377301.243,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7044\"]],AUTHORITY[\"EPSG\",\"6145\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4145\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",75],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",0],AUTHORITY[\"EPSG\",\"24313\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24342, "epsg", 24342,
		      "Kalianpur 1975 / UTM zone 42N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +a=6377299.151 +b=6356098.145120132 +");
    add_proj4text (p, 1, "towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / UTM zone 42N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1975 Definition)\",6377299.151,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",69],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"24342\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 24343, "epsg", 24343,
		      "Kalianpur 1975 / UTM zone 43N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=43 +a=6377299.151 +b=6356098.145120132 +");
    add_proj4text (p, 1, "towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / UTM zone 43N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1975 Definition)\",6377299.151,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",75],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"24343\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 24344, "epsg", 24344,
		      "Kalianpur 1975 / UTM zone 44N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=44 +a=6377299.151 +b=6356098.145120132 +");
    add_proj4text (p, 1, "towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / UTM zone 44N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1975 Definition)\",6377299.151,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",81],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"24344\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 13, "TH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_20 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 24345, "epsg", 24345,
		      "Kalianpur 1975 / UTM zone 45N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=45 +a=6377299.151 +b=6356098.145120132 +");
    add_proj4text (p, 1, "towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / UTM zone 45N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1975 Definition)\",6377299.151,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",87],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"24345\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 24346, "epsg", 24346,
		      "Kalianpur 1975 / UTM zone 46N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +a=6377299.151 +b=6356098.145120132 +");
    add_proj4text (p, 1, "towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / UTM zone 46N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1975 Definition)\",6377299.151,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",93],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"24346\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 24347, "epsg", 24347,
		      "Kalianpur 1975 / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +a=6377299.151 +b=6356098.145120132 +");
    add_proj4text (p, 1, "towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / UTM zone 47N\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1975 Definition)\",6377299.151,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",99],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"24347\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 24370, "epsg", 24370,
		      "Kalianpur 1880 / India zone 0");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.5 +lat_0=39.5 +lon_0=68 +k_0=0.99846");
    add_proj4text (p, 1,
		   "154 +x_0=2153865.73916853 +y_0=2368292.194628102 +a=6377");
    add_proj4text (p, 2,
		   "299.36559538 +b=6356098.359005156 +to_meter=0.9143985307");
    add_proj4text (p, 3, "444408 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1880 / India zone 0\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1880\",DATUM[\"Kalianpur_1880\",SPHEROID[\"Everest (");
    add_srs_wkt (p, 2,
		 "1830 Definition)\",6377299.36559538,300.8017255433552,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7042\"]],AUTHORITY[\"EPSG\",\"6243\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4243\"]],UNIT[\"Indian yar");
    add_srs_wkt (p, 7,
		 "d\",0.9143985307444408,AUTHORITY[\"EPSG\",\"9084\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",39.5],PARAMETER[\"central_meridian\",6");
    add_srs_wkt (p, 10,
		 "8],PARAMETER[\"scale_factor\",0.99846154],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",2355500],PARAMETER[\"false_northing\",2590");
    add_srs_wkt (p, 12,
		 "000],AUTHORITY[\"EPSG\",\"24370\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24371, "epsg", 24371,
		      "Kalianpur 1880 / India zone I");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.5 +lat_0=32.5 +lon_0=68 +k_0=0.99878");
    add_proj4text (p, 1,
		   "641 +x_0=2743195.592233322 +y_0=914398.5307444407 +a=637");
    add_proj4text (p, 2,
		   "7299.36559538 +b=6356098.359005156 +to_meter=0.914398530");
    add_proj4text (p, 3, "7444408 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1880 / India zone I\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1880\",DATUM[\"Kalianpur_1880\",SPHEROID[\"Everest (");
    add_srs_wkt (p, 2,
		 "1830 Definition)\",6377299.36559538,300.8017255433552,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7042\"]],AUTHORITY[\"EPSG\",\"6243\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4243\"]],UNIT[\"Indian yar");
    add_srs_wkt (p, 7,
		 "d\",0.9143985307444408,AUTHORITY[\"EPSG\",\"9084\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",32.5],PARAMETER[\"central_meridian\",6");
    add_srs_wkt (p, 10,
		 "8],PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",3000000],PARAMETER[\"false_northing\",1000");
    add_srs_wkt (p, 12,
		 "000],AUTHORITY[\"EPSG\",\"24371\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24372, "epsg", 24372,
		      "Kalianpur 1880 / India zone IIa");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26 +lat_0=26 +lon_0=74 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.592233322 +y_0=914398.5307444407 +a=6377299");
    add_proj4text (p, 2,
		   ".36559538 +b=6356098.359005156 +to_meter=0.9143985307444");
    add_proj4text (p, 3, "408 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1880 / India zone IIa\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1880\",DATUM[\"Kalianpur_1880\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " (1830 Definition)\",6377299.36559538,300.8017255433552,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7042\"]],AUTHORITY[\"EPSG\",\"6243\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4243\"]],UNIT[\"Indian yar");
    add_srs_wkt (p, 7,
		 "d\",0.9143985307444408,AUTHORITY[\"EPSG\",\"9084\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",26],PARAMETER[\"central_meridian\",74]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",3000000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"24372\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24373, "epsg", 24373,
		      "Kalianpur 1880 / India zone III");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=19 +lat_0=19 +lon_0=80 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.592233322 +y_0=914398.5307444407 +a=6377299");
    add_proj4text (p, 2,
		   ".36559538 +b=6356098.359005156 +to_meter=0.9143985307444");
    add_proj4text (p, 3, "408 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1880 / India zone III\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1880\",DATUM[\"Kalianpur_1880\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " (1830 Definition)\",6377299.36559538,300.8017255433552,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7042\"]],AUTHORITY[\"EPSG\",\"6243\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4243\"]],UNIT[\"Indian yar");
    add_srs_wkt (p, 7,
		 "d\",0.9143985307444408,AUTHORITY[\"EPSG\",\"9084\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",19],PARAMETER[\"central_meridian\",80]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",3000000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"24373\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24374, "epsg", 24374,
		      "Kalianpur 1880 / India zone IV");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=12 +lat_0=12 +lon_0=80 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.592233322 +y_0=914398.5307444407 +a=6377299");
    add_proj4text (p, 2,
		   ".36559538 +b=6356098.359005156 +to_meter=0.9143985307444");
    add_proj4text (p, 3, "408 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1880 / India zone IV\",GEOGCS[\"Kalia");
    add_srs_wkt (p, 1,
		 "npur 1880\",DATUM[\"Kalianpur_1880\",SPHEROID[\"Everest ");
    add_srs_wkt (p, 2,
		 "(1830 Definition)\",6377299.36559538,300.8017255433552,A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"7042\"]],AUTHORITY[\"EPSG\",\"6243\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4243\"]],UNIT[\"Indian yar");
    add_srs_wkt (p, 7,
		 "d\",0.9143985307444408,AUTHORITY[\"EPSG\",\"9084\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",12],PARAMETER[\"central_meridian\",80]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",3000000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"24374\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24375, "epsg", 24375,
		      "Kalianpur 1937 / India zone IIb");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26 +lat_0=26 +lon_0=90 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743185.69 +y_0=914395.23 +a=6377276.345 +b=6356075");
    add_proj4text (p, 2, ".41314024 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1937 / India zone IIb\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1937\",DATUM[\"Kalianpur_1937\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " 1830 (1937 Adjustment)\",6377276.345,300.8017,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7015\"]],AUTHORITY[\"EPSG\",\"6144\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4144\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_1SP\"],PARAMETER[\"latitude_of_origin\",26],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",90],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "99878641],PARAMETER[\"false_easting\",2743185.69],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",914395.23],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12,
		 "4375\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 24376, "epsg", 24376,
		      "Kalianpur 1962 / India zone I");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.5 +lat_0=32.5 +lon_0=68 +k_0=0.99878");
    add_proj4text (p, 1,
		   "641 +x_0=2743196.4 +y_0=914398.8 +a=6377301.243 +b=63561");
    add_proj4text (p, 2, "00.230165384 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1962 / India zone I\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1962\",DATUM[\"Kalianpur_1962\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1962 Definition)\",6377301.243,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7044\"]],AUTHORITY[\"EPSG\",\"6145\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4145\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_1SP\"],PARAMETER[\"latitude_of_origin\",32.5],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",68],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.99878641],PARAMETER[\"false_easting\",2743196.4],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",914398.8],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "24376\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 13, "]");
    p = add_epsg_def (first, last, 24377, "epsg", 24377,
		      "Kalianpur 1962 / India zone IIa");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26 +lat_0=26 +lon_0=74 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743196.4 +y_0=914398.8 +a=6377301.243 +b=6356100.2");
    add_proj4text (p, 2, "30165384 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1962 / India zone IIa\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1962\",DATUM[\"Kalianpur_1962\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " 1830 (1962 Definition)\",6377301.243,300.8017255,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7044\"]],AUTHORITY[\"EPSG\",\"6145\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4145\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_1SP\"],PARAMETER[\"latitude_of_origin\",26],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",74],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.99878641],PARAMETER[\"false_easting\",2743196.4],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",914398.8],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12,
		 "24377\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 13, "]");
    p = add_epsg_def (first, last, 24378, "epsg", 24378,
		      "Kalianpur 1975 / India zone I");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.5 +lat_0=32.5 +lon_0=68 +k_0=0.99878");
    add_proj4text (p, 1,
		   "641 +x_0=2743195.5 +y_0=914398.5 +a=6377299.151 +b=63560");
    add_proj4text (p, 2,
		   "98.145120132 +towgs84=295,736,257,0,0,0,0 +units=m +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / India zone I\",GEOGCS[\"Kalian");
    add_srs_wkt (p, 1,
		 "pur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1975 Definition)\",6377299.151,300.8017255,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"414");
    add_srs_wkt (p, 7,
		 "6\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",32.5],PARAMETER[\"central_meridian\",6");
    add_srs_wkt (p, 10,
		 "8],PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",2743195.5],PARAMETER[\"false_northing\",91");
    add_srs_wkt (p, 12,
		 "4398.5],AUTHORITY[\"EPSG\",\"24378\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 13, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24379, "epsg", 24379,
		      "Kalianpur 1975 / India zone IIa");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26 +lat_0=26 +lon_0=74 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.5 +y_0=914398.5 +a=6377299.151 +b=6356098.1");
    add_proj4text (p, 2,
		   "45120132 +towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / India zone IIa\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " 1830 (1975 Definition)\",6377299.151,300.8017255,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "146\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",26],PARAMETER[\"central_meridian\",7");
    add_srs_wkt (p, 10,
		 "4],PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",2743195.5],PARAMETER[\"false_northing\",91");
    add_srs_wkt (p, 12,
		 "4398.5],AUTHORITY[\"EPSG\",\"24379\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 13, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24380, "epsg", 24380,
		      "Kalianpur 1975 / India zone IIb");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26 +lat_0=26 +lon_0=90 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.5 +y_0=914398.5 +a=6377299.151 +b=6356098.1");
    add_proj4text (p, 2,
		   "45120132 +towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / India zone IIb\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " 1830 (1975 Definition)\",6377299.151,300.8017255,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "146\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",26],PARAMETER[\"central_meridian\",9");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",2743195.5],PARAMETER[\"false_northing\",91");
    add_srs_wkt (p, 12,
		 "4398.5],AUTHORITY[\"EPSG\",\"24380\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 13, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24381, "epsg", 24381,
		      "Kalianpur 1975 / India zone III");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=19 +lat_0=19 +lon_0=80 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.5 +y_0=914398.5 +a=6377299.151 +b=6356098.1");
    add_proj4text (p, 2,
		   "45120132 +towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / India zone III\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " 1830 (1975 Definition)\",6377299.151,300.8017255,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "146\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",19],PARAMETER[\"central_meridian\",8");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",2743195.5],PARAMETER[\"false_northing\",91");
    add_srs_wkt (p, 12,
		 "4398.5],AUTHORITY[\"EPSG\",\"24381\"],AXIS[\"Easting\",E");
    add_srs_wkt (p, 13, "AST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24382, "epsg", 24382,
		      "Kalianpur 1880 / India zone IIb");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26 +lat_0=26 +lon_0=90 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.592233322 +y_0=914398.5307444407 +a=6377299");
    add_proj4text (p, 2,
		   ".36559538 +b=6356098.359005156 +to_meter=0.9143985307444");
    add_proj4text (p, 3, "408 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1880 / India zone IIb\",GEOGCS[\"Kali");
    add_srs_wkt (p, 1,
		 "anpur 1880\",DATUM[\"Kalianpur_1880\",SPHEROID[\"Everest");
    add_srs_wkt (p, 2,
		 " (1830 Definition)\",6377299.36559538,300.8017255433552,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7042\"]],AUTHORITY[\"EPSG\",\"6243\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4243\"]],UNIT[\"Indian yar");
    add_srs_wkt (p, 7,
		 "d\",0.9143985307444408,AUTHORITY[\"EPSG\",\"9084\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",26],PARAMETER[\"central_meridian\",90]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",3000000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"24382\"],AXIS[\"Easting\",EAST],");
    add_srs_wkt (p, 13, "AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24383, "epsg", 24383,
		      "Kalianpur 1975 / India zone IV");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=12 +lat_0=12 +lon_0=80 +k_0=0.99878641 ");
    add_proj4text (p, 1,
		   "+x_0=2743195.5 +y_0=914398.5 +a=6377299.151 +b=6356098.1");
    add_proj4text (p, 2,
		   "45120132 +towgs84=295,736,257,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kalianpur 1975 / India zone IV\",GEOGCS[\"Kalia");
    add_srs_wkt (p, 1,
		 "npur 1975\",DATUM[\"Kalianpur_1975\",SPHEROID[\"Everest ");
    add_srs_wkt (p, 2,
		 "1830 (1975 Definition)\",6377299.151,300.8017255,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7045\"]],TOWGS84[295,736,257,0,0,0,0],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6146\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"41");
    add_srs_wkt (p, 7,
		 "46\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",12],PARAMETER[\"central_meridian\",80");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.99878641],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_easting\",2743195.5],PARAMETER[\"false_northing\",914");
    add_srs_wkt (p, 12,
		 "398.5],AUTHORITY[\"EPSG\",\"24383\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24500, "epsg", 24500,
		      "Kertau 1968 / Singapore Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=1.287646666666667 +lon_0=103.853002222");
    add_proj4text (p, 1,
		   "2222 +x_0=30000 +y_0=30000 +a=6377304.063 +b=6356103.038");
    add_proj4text (p, 2, "993155 +towgs84=-11,851,5,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kertau 1968 / Singapore Grid\",GEOGCS[\"Kertau ");
    add_srs_wkt (p, 1,
		 "1968\",DATUM[\"Kertau_1968\",SPHEROID[\"Everest 1830 Mod");
    add_srs_wkt (p, 2,
		 "ified\",6377304.063,300.8017,AUTHORITY[\"EPSG\",\"7018\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-11,851,5,0,0,0,0],AUTHORITY[\"EPSG\",\"6245\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4245\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Cassini_Soldn");
    add_srs_wkt (p, 8,
		 "er\"],PARAMETER[\"latitude_of_origin\",1.287646666666667");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",103.8530022222222],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",30000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",30000],AUTHORITY[\"EPSG\",\"24500\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24547, "epsg", 24547,
		      "Kertau 1968 / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +a=6377304.063 +b=6356103.038993155 +");
    add_proj4text (p, 1, "towgs84=-11,851,5,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kertau 1968 / UTM zone 47N\",GEOGCS[\"Kertau 19");
    add_srs_wkt (p, 1,
		 "68\",DATUM[\"Kertau_1968\",SPHEROID[\"Everest 1830 Modif");
    add_srs_wkt (p, 2,
		 "ied\",6377304.063,300.8017,AUTHORITY[\"EPSG\",\"7018\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[-11,851,5,0,0,0,0],AUTHORITY[\"EPSG\",\"6245\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4245\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",99],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"24547\"],AXIS[\"Eas");
    add_srs_wkt (p, 12, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24548, "epsg", 24548,
		      "Kertau 1968 / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +a=6377304.063 +b=6356103.038993155 +");
    add_proj4text (p, 1, "towgs84=-11,851,5,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kertau 1968 / UTM zone 48N\",GEOGCS[\"Kertau 19");
    add_srs_wkt (p, 1,
		 "68\",DATUM[\"Kertau_1968\",SPHEROID[\"Everest 1830 Modif");
    add_srs_wkt (p, 2,
		 "ied\",6377304.063,300.8017,AUTHORITY[\"EPSG\",\"7018\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[-11,851,5,0,0,0,0],AUTHORITY[\"EPSG\",\"6245\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4245\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",105],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"24548\"],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24571, "epsg", 24571,
		      "Kertau / R.S.O. Malaya (ch) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=102.25 +alpha=323.0257905 +k=");
    add_proj4text (p, 1,
		   "0.99984 +x_0=804671.2997750348 +y_0=0 +a=6377304.063 +b=");
    add_proj4text (p, 2,
		   "6356103.038993155 +towgs84=-11,851,5,0,0,0,0 +to_meter=2");
    add_proj4text (p, 3, "0.11678249437587 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Kertau / R.S.O. Malaya (ch) (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Kertau 1968\",DATUM[\"Kertau_1968\",SPHEROID[\"Ever");
    add_srs_wkt (p, 2,
		 "est 1830 Modified\",6377304.063,300.8017,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7018\"]],TOWGS84[-11,851,5,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6245\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4245\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"British chain (Benoit 1895 B)\",20.11678249437587,AUT");
    add_srs_wkt (p, 8,
		 "HORITY[\"EPSG\",\"9062\"]],PROJECTION[\"Hotine_Oblique_M");
    add_srs_wkt (p, 9,
		 "ercator\"],PARAMETER[\"latitude_of_center\",4],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"longitude_of_center\",102.25],PARAMETER[\"azimuth\",3");
    add_srs_wkt (p, 11,
		 "23.0257905],PARAMETER[\"rectified_grid_angle\",323.13010");
    add_srs_wkt (p, 12,
		 "23611111],PARAMETER[\"scale_factor\",0.99984],PARAMETER[");
    add_srs_wkt (p, 13,
		 "\"false_easting\",40000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 14,
		 ",AUTHORITY[\"EPSG\",\"24571\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 15, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24600, "epsg", 24600, "KOC Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.5 +lat_0=32.5 +lon_0=45 +k_0=0.99878");
    add_proj4text (p, 1,
		   "64078000001 +x_0=1500000 +y_0=1166200 +ellps=clrk80 +tow");
    add_proj4text (p, 2, "gs84=-294.7,-200.1,525.5,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KOC Lambert\",GEOGCS[\"KOC\",DATUM[\"Kuwait_Oil");
    add_srs_wkt (p, 1,
		 "_Company\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,29");
    add_srs_wkt (p, 2,
		 "3.465,AUTHORITY[\"EPSG\",\"7012\"]],TOWGS84[-294.7,-200.");
    add_srs_wkt (p, 3,
		 "1,525.5,0,0,0,0],AUTHORITY[\"EPSG\",\"6246\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4246\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"latitude_of_origin\",32.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",45],PARAMETER[\"scale_factor\",0.9987");
    add_srs_wkt (p, 10,
		 "864078],PARAMETER[\"false_easting\",1500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",1166200],AUTHORITY[\"EPSG\",\"24600\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24718, "epsg", 24718,
		      "La Canoa / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=intl +towgs84=-273.5,110.6,-35");
    add_proj4text (p, 1, "7.9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"La Canoa / UTM zone 18N\",GEOGCS[\"La Canoa\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"La_Canoa\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 2,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-273.5,110.6");
    add_srs_wkt (p, 3,
		 ",-357.9,0,0,0,0],AUTHORITY[\"EPSG\",\"6247\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4247\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-75],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"24718\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24719, "epsg", 24719,
		      "La Canoa / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=intl +towgs84=-273.5,110.6,-35");
    add_proj4text (p, 1, "7.9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"La Canoa / UTM zone 19N\",GEOGCS[\"La Canoa\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"La_Canoa\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 2,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-273.5,110.6");
    add_srs_wkt (p, 3,
		 ",-357.9,0,0,0,0],AUTHORITY[\"EPSG\",\"6247\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4247\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-69],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"24719\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24720, "epsg", 24720,
		      "La Canoa / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=intl +towgs84=-273.5,110.6,-35");
    add_proj4text (p, 1, "7.9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"La Canoa / UTM zone 20N\",GEOGCS[\"La Canoa\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"La_Canoa\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 2,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-273.5,110.6");
    add_srs_wkt (p, 3,
		 ",-357.9,0,0,0,0],AUTHORITY[\"EPSG\",\"6247\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4247\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-63],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"24720\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24817, "epsg", 24817,
		      "PSAD56 / UTM zone 17N");
    add_proj4text (p, 0, "+proj=utm +zone=17 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 17N\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-8");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"24817\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 24818, "epsg", 24818,
		      "PSAD56 / UTM zone 18N");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 18N\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-7");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"24818\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 24819, "epsg", 24819,
		      "PSAD56 / UTM zone 19N");
    add_proj4text (p, 0, "+proj=utm +zone=19 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 19N\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-6");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"24819\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 24820, "epsg", 24820,
		      "PSAD56 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 20N\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-6");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"24820\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 24821, "epsg", 24821,
		      "PSAD56 / UTM zone 21N");
    add_proj4text (p, 0, "+proj=utm +zone=21 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 21N\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-5");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"24821\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 24877, "epsg", 24877,
		      "PSAD56 / UTM zone 17S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 17S\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-8");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",10000000]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"24877\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24878, "epsg", 24878,
		      "PSAD56 / UTM zone 18S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 18S\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-7");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",10000000]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"24878\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24879, "epsg", 24879,
		      "PSAD56 / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 19S\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-6");
    add_srs_wkt (p, 9,
		 "9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",10000000]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"24879\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24880, "epsg", 24880,
		      "PSAD56 / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 20S\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-6");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",10000000]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"24880\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24881, "epsg", 24881,
		      "PSAD56 / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 21S\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-5");
    add_srs_wkt (p, 9,
		 "7],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",10000000]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"24881\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24882, "epsg", 24882,
		      "PSAD56 / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / UTM zone 22S\",GEOGCS[\"PSAD56\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Provisional_South_American_Datum_1956\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",-5");
    add_srs_wkt (p, 9,
		 "1],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",500000],PARAMETER[\"false_northing\",10000000]");
    add_srs_wkt (p, 11,
		 ",AUTHORITY[\"EPSG\",\"24882\"],AXIS[\"Easting\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 24891, "epsg", 24891,
		      "PSAD56 / Peru west zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-6 +lon_0=-80.5 +k=0.99983008 +x_0=22");
    add_proj4text (p, 1, "2000 +y_0=1426834.743 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / Peru west zone\",GEOGCS[\"PSAD56\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Provisional_South_American_Datum_1956\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",-6],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-80.5],PARAMETER[\"scale_factor\",0.99983008],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",222000],PARAMETER[\"false_northing\",1");
    add_srs_wkt (p, 11,
		 "426834.743],AUTHORITY[\"EPSG\",\"24891\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 12, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 24892, "epsg", 24892,
		      "PSAD56 / Peru central zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-9.5 +lon_0=-76 +k=0.99932994 +x_0=72");
    add_proj4text (p, 1, "0000 +y_0=1039979.159 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / Peru central zone\",GEOGCS[\"PSAD56\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"Provisional_South_American_Datum_1956\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7022\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",-9.5],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-76],PARAMETER[\"scale_factor\",0.99932994],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",720000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",1039979.159],AUTHORITY[\"EPSG\",\"24892\"],AXIS[\"X\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 24893, "epsg", 24893,
		      "PSAD56 / Peru east zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-9.5 +lon_0=-70.5 +k=0.99952992 +x_0=");
    add_proj4text (p, 1,
		   "1324000 +y_0=1040084.558 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"PSAD56 / Peru east zone\",GEOGCS[\"PSAD56\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Provisional_South_American_Datum_1956\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],AUTHORITY[\"EPSG\",\"6248\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4248\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",-9.5],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",-70.5],PARAMETER[\"scale_factor\",0.99952992],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",1324000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",1040084.558],AUTHORITY[\"EPSG\",\"24893\"],AXIS[\"X\",E");
    add_srs_wkt (p, 12, "AST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 25000, "epsg", 25000,
		      "Leigon / Ghana Metre Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4.666666666666667 +lon_0=-1 +k=0.9997");
    add_proj4text (p, 1,
		   "5 +x_0=274319.51 +y_0=0 +ellps=clrk80 +towgs84=-130,29,3");
    add_proj4text (p, 2, "64,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Leigon / Ghana Metre Grid\",GEOGCS[\"Leigon\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Leigon\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.1");
    add_srs_wkt (p, 2,
		 "45,293.465,AUTHORITY[\"EPSG\",\"7012\"]],TOWGS84[-130,29");
    add_srs_wkt (p, 3,
		 ",364,0,0,0,0],AUTHORITY[\"EPSG\",\"6250\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4250\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",4.666666666666667],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-1],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "99975],PARAMETER[\"false_easting\",274319.51],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"25000\"],AXIS");
    add_srs_wkt (p, 12, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25231, "epsg", 25231, "Lome / UTM zone 31N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=31 +a=6378249.2 +b=6356515 +units=m +no_");
    add_proj4text (p, 1, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Lome / UTM zone 31N\",GEOGCS[\"Lome\",DATUM[\"L");
    add_srs_wkt (p, 1,
		 "ome\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.46602");
    add_srs_wkt (p, 2,
		 "12936265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6252\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4252\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",3],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 9,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"25231\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25391, "epsg", 25391,
		      "Luzon 1911 / Philippines zone I");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Luzon 1911 / Philippines zone I\",GEOGCS[\"Luzo");
    add_srs_wkt (p, 1,
		 "n 1911\",DATUM[\"Luzon_1911\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6253\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4253\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",117],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.99995],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"25391\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 25392, "epsg", 25392,
		      "Luzon 1911 / Philippines zone II");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=119 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Luzon 1911 / Philippines zone II\",GEOGCS[\"Luz");
    add_srs_wkt (p, 1,
		 "on 1911\",DATUM[\"Luzon_1911\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6253\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4253\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",119],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.99995],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"25392\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 25393, "epsg", 25393,
		      "Luzon 1911 / Philippines zone III");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=121 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Luzon 1911 / Philippines zone III\",GEOGCS[\"Lu");
    add_srs_wkt (p, 1,
		 "zon 1911\",DATUM[\"Luzon_1911\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6253\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4253\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 8,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",121]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"scale_factor\",0.99995],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"25393\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 25394, "epsg", 25394,
		      "Luzon 1911 / Philippines zone IV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Luzon 1911 / Philippines zone IV\",GEOGCS[\"Luz");
    add_srs_wkt (p, 1,
		 "on 1911\",DATUM[\"Luzon_1911\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6253\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4253\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",123],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.99995],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"25394\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 25395, "epsg", 25395,
		      "Luzon 1911 / Philippines zone V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=125 +k=0.99995 +x_0=500000 +");
    add_proj4text (p, 1, "y_0=0 +ellps=clrk66 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Luzon 1911 / Philippines zone V\",GEOGCS[\"Luzo");
    add_srs_wkt (p, 1,
		 "n 1911\",DATUM[\"Luzon_1911\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6253\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4253\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",125],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.99995],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"25395\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 25700, "epsg", 25700,
		      "Makassar (Jakarta) / NEIEZ (deprecated)");
    add_proj4text (p, 0,
		   "+proj=merc +lon_0=110 +k=0.997 +x_0=3900000 +y_0=900000 ");
    add_proj4text (p, 1,
		   "+ellps=bessel +towgs84=-587.8,519.75,145.76,0,0,0,0 +pm=");
    add_proj4text (p, 2, "jakarta +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Makassar (Jakarta) / NEIEZ (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Makassar (Jakarta)\",DATUM[\"Makassar_Jakarta\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7004\"]],TOWGS84[-587.8,519.75,145.76,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6804\"]],PRIMEM[\"Jakarta\",106.807");
    add_srs_wkt (p, 5,
		 "7194444444,AUTHORITY[\"EPSG\",\"8908\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 6,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"4804\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 8,
		 "SG\",\"9001\"]],PROJECTION[\"Mercator_1SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",110],PARAMETER[\"scale_factor\",0.997");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",3900000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",900000],AUTHORITY[\"EPSG\",\"25700\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 25828, "epsg", 25828,
		      "ETRS89 / UTM zone 28N");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 28N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",-15],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",500000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"25828\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25829, "epsg", 25829,
		      "ETRS89 / UTM zone 29N");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 29N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",-9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25829\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25830, "epsg", 25830,
		      "ETRS89 / UTM zone 30N");
    add_proj4text (p, 0, "+proj=utm +zone=30 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 30N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",-3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25830\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25831, "epsg", 25831,
		      "ETRS89 / UTM zone 31N");
    add_proj4text (p, 0, "+proj=utm +zone=31 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 31N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"25831\"],AXIS[\"Easting\",EAST],AXIS[\"N");
    add_srs_wkt (p, 12, "orthing\",NORTH]]");
    p = add_epsg_def (first, last, 25832, "epsg", 25832,
		      "ETRS89 / UTM zone 32N");
    add_proj4text (p, 0, "+proj=utm +zone=32 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 32N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"25832\"],AXIS[\"Easting\",EAST],AXIS[\"N");
    add_srs_wkt (p, 12, "orthing\",NORTH]]");
    p = add_epsg_def (first, last, 25833, "epsg", 25833,
		      "ETRS89 / UTM zone 33N");
    add_proj4text (p, 0, "+proj=utm +zone=33 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 33N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",15],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25833\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25834, "epsg", 25834,
		      "ETRS89 / UTM zone 34N");
    add_proj4text (p, 0, "+proj=utm +zone=34 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 34N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",21],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25834\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25835, "epsg", 25835,
		      "ETRS89 / UTM zone 35N");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 35N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",27],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25835\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25836, "epsg", 25836,
		      "ETRS89 / UTM zone 36N");
    add_proj4text (p, 0, "+proj=utm +zone=36 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 36N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",33],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25836\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25837, "epsg", 25837,
		      "ETRS89 / UTM zone 37N");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 37N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",39],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25837\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25838, "epsg", 25838,
		      "ETRS89 / UTM zone 38N");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / UTM zone 38N\",GEOGCS[\"ETRS89\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"European_Terrestrial_Reference_System_1989\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",45],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25838\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 25884, "epsg", 25884,
		      "ETRS89 / TM Baltic93");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"ETRS89 / TM Baltic93\",GEOGCS[\"ETRS89\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"European_Terrestrial_Reference_System_1989\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4258\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 9,
		 ",24],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"25884\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 25932, "epsg", 25932,
		      "Malongo 1987 / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Malongo 1987 / UTM zone 32S\",GEOGCS[\"Malongo ");
    add_srs_wkt (p, 1,
		 "1987\",DATUM[\"Malongo_1987\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6259\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4259\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",9],PARAMETER[\"s");
    add_srs_wkt (p, 9,
		 "cale_factor\",0.9996],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"25932\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 26191, "epsg", 26191,
		      "Merchich / Nord Maroc");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.3 +lat_0=33.3 +lon_0=-5.4 +k_0=0.999");
    add_proj4text (p, 1,
		   "625769 +x_0=500000 +y_0=300000 +a=6378249.2 +b=6356515 +");
    add_proj4text (p, 2, "towgs84=31,146,47,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Merchich / Nord Maroc\",GEOGCS[\"Merchich\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Merchich\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2");
    add_srs_wkt (p, 2,
		 ",293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[31,146,47,0,0,0,0],AUTHORITY[\"EPSG\",\"6261\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4261\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 8,
		 "ic_1SP\"],PARAMETER[\"latitude_of_origin\",33.3],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-5.4],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999625769],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",300000],AUTHORITY[\"EPSG\",\"2619");
    add_srs_wkt (p, 12, "1\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26192, "epsg", 26192,
		      "Merchich / Sud Maroc");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=29.7 +lat_0=29.7 +lon_0=-5.4 +k_0=0.999");
    add_proj4text (p, 1,
		   "6155960000001 +x_0=500000 +y_0=300000 +a=6378249.2 +b=63");
    add_proj4text (p, 2, "56515 +towgs84=31,146,47,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Merchich / Sud Maroc\",GEOGCS[\"Merchich\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"Merchich\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,");
    add_srs_wkt (p, 2,
		 "293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[");
    add_srs_wkt (p, 3,
		 "31,146,47,0,0,0,0],AUTHORITY[\"EPSG\",\"6261\"]],PRIMEM[");
    add_srs_wkt (p, 4,
		 "\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"deg");
    add_srs_wkt (p, 5,
		 "ree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"4261\"]],UNIT[\"metre\",1,AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Coni");
    add_srs_wkt (p, 8,
		 "c_1SP\"],PARAMETER[\"latitude_of_origin\",29.7],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-5.4],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.999615596],PARAMETER[\"false_easting\",500000],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_northing\",300000],AUTHORITY[\"EPSG\",\"26192");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26193, "epsg", 26193,
		      "Merchich / Sahara (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26.1 +lat_0=26.1 +lon_0=-5.4 +k_0=0.999");
    add_proj4text (p, 1,
		   "6 +x_0=1200000 +y_0=400000 +a=6378249.2 +b=6356515 +towg");
    add_proj4text (p, 2, "s84=31,146,47,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Merchich / Sahara (deprecated)\",GEOGCS[\"Merch");
    add_srs_wkt (p, 1,
		 "ich\",DATUM[\"Merchich\",SPHEROID[\"Clarke 1880 (IGN)\",");
    add_srs_wkt (p, 2,
		 "6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[31,146,47,0,0,0,0],AUTHORITY[\"EPSG\",\"6261\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4261\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Confo");
    add_srs_wkt (p, 8,
		 "rmal_Conic_1SP\"],PARAMETER[\"latitude_of_origin\",26.1]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"central_meridian\",-5.4],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9996],PARAMETER[\"false_easting\",1200000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",400000],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "26193\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26194, "epsg", 26194,
		      "Merchich / Sahara Nord");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26.1 +lat_0=26.1 +lon_0=-5.4 +k_0=0.999");
    add_proj4text (p, 1,
		   "616304 +x_0=1200000 +y_0=400000 +a=6378249.2 +b=6356515 ");
    add_proj4text (p, 2, "+towgs84=31,146,47,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Merchich / Sahara Nord\",GEOGCS[\"Merchich\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Merchich\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.");
    add_srs_wkt (p, 2,
		 "2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS8");
    add_srs_wkt (p, 3,
		 "4[31,146,47,0,0,0,0],AUTHORITY[\"EPSG\",\"6261\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4261\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Co");
    add_srs_wkt (p, 8,
		 "nic_1SP\"],PARAMETER[\"latitude_of_origin\",26.1],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"central_meridian\",-5.4],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999616304],PARAMETER[\"false_easting\",1200000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",400000],AUTHORITY[\"EPSG\",\"261");
    add_srs_wkt (p, 12, "94\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26195, "epsg", 26195,
		      "Merchich / Sahara Sud");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=22.5 +lat_0=22.5 +lon_0=-5.4 +k_0=0.999");
    add_proj4text (p, 1,
		   "616437 +x_0=1500000 +y_0=400000 +a=6378249.2 +b=6356515 ");
    add_proj4text (p, 2, "+towgs84=31,146,47,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Merchich / Sahara Sud\",GEOGCS[\"Merchich\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Merchich\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2");
    add_srs_wkt (p, 2,
		 ",293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[31,146,47,0,0,0,0],AUTHORITY[\"EPSG\",\"6261\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4261\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 8,
		 "ic_1SP\"],PARAMETER[\"latitude_of_origin\",22.5],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-5.4],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.999616437],PARAMETER[\"false_easting\",1500000],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_northing\",400000],AUTHORITY[\"EPSG\",\"261");
    add_srs_wkt (p, 12, "95\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26237, "epsg", 26237,
		      "Massawa / UTM zone 37N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +ellps=bessel +towgs84=639,405,60,0,0");
    add_proj4text (p, 1, ",0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Massawa / UTM zone 37N\",GEOGCS[\"Massawa\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Massawa\",SPHEROID[\"Bessel 1841\",6377397.155,299.");
    add_srs_wkt (p, 2,
		 "1528128,AUTHORITY[\"EPSG\",\"7004\"]],TOWGS84[639,405,60");
    add_srs_wkt (p, 3,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6262\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 4,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 5,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"4262\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",39],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",500000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 11,
		 "THORITY[\"EPSG\",\"26237\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26331, "epsg", 26331,
		      "Minna / UTM zone 31N");
    add_proj4text (p, 0, "+proj=utm +zone=31 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Minna / UTM zone 31N\",GEOGCS[\"Minna\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Minna\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.4");
    add_srs_wkt (p, 2,
		 "65,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 3,
		 "63\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4263\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",3],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26331\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26332, "epsg", 26332,
		      "Minna / UTM zone 32N");
    add_proj4text (p, 0, "+proj=utm +zone=32 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Minna / UTM zone 32N\",GEOGCS[\"Minna\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Minna\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,293.4");
    add_srs_wkt (p, 2,
		 "65,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"62");
    add_srs_wkt (p, 3,
		 "63\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4263\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 6,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 7,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"central_meridian\",9],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26332\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26391, "epsg", 26391,
		      "Minna / Nigeria West Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4 +lon_0=4.5 +k=0.99975 +x_0=230738.2");
    add_proj4text (p, 1, "6 +y_0=0 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Minna / Nigeria West Belt\",GEOGCS[\"Minna\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Minna\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145");
    add_srs_wkt (p, 2,
		 ",293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6263\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4263\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",4],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",4.5],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 9,
		 "tor\",0.99975],PARAMETER[\"false_easting\",230738.26],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26391");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26392, "epsg", 26392,
		      "Minna / Nigeria Mid Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4 +lon_0=8.5 +k=0.99975 +x_0=670553.9");
    add_proj4text (p, 1, "8 +y_0=0 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Minna / Nigeria Mid Belt\",GEOGCS[\"Minna\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Minna\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145,");
    add_srs_wkt (p, 2,
		 "293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6263\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4263\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",4],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",8.5],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 9,
		 "tor\",0.99975],PARAMETER[\"false_easting\",670553.98],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26392");
    add_srs_wkt (p, 11, "\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26393, "epsg", 26393,
		      "Minna / Nigeria East Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=4 +lon_0=12.5 +k=0.99975 +x_0=1110369");
    add_proj4text (p, 1, ".7 +y_0=0 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Minna / Nigeria East Belt\",GEOGCS[\"Minna\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Minna\",SPHEROID[\"Clarke 1880 (RGS)\",6378249.145");
    add_srs_wkt (p, 2,
		 ",293.465,AUTHORITY[\"EPSG\",\"7012\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6263\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4263\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",4],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",12.5],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 9,
		 "ctor\",0.99975],PARAMETER[\"false_easting\",1110369.7],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2639");
    add_srs_wkt (p, 11,
		 "3\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26432, "epsg", 26432,
		      "Mhast / UTM zone 32S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=intl +towgs84=-252.95,-");
    add_proj4text (p, 1, "4.11,-96.38,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Mhast / UTM zone 32S (deprecated)\",GEOGCS[\"Mh");
    add_srs_wkt (p, 1,
		 "ast\",DATUM[\"Mhast\",SPHEROID[\"International 1924\",63");
    add_srs_wkt (p, 2,
		 "78388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-252.95,");
    add_srs_wkt (p, 3,
		 "-4.11,-96.38,0,0,0,0],AUTHORITY[\"EPSG\",\"6264\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4264\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",9],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 11,
		 "ing\",10000000],AUTHORITY[\"EPSG\",\"26432\"],AXIS[\"Eas");
    add_srs_wkt (p, 12, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26591, "epsg", 26591,
		      "Monte Mario (Rome) / Italy zone 1 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-3.45233333333333 +k=0.9996 ");
    add_proj4text (p, 1,
		   "+x_0=1500000 +y_0=0 +ellps=intl +pm=rome +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Monte Mario (Rome) / Italy zone 1 (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"Monte Mario (Rome)\",DATUM[\"Monte_Mario_Rome\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6806\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Rome\",12.45233333333333,AUTHORITY[\"EPSG\",\"8906\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4806\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-3.45233333333333],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.9996],PARAMETER[\"false_easting\",1500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"265");
    add_srs_wkt (p, 12, "91\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26592, "epsg", 26592,
		      "Monte Mario (Rome) / Italy zone 2 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=2.54766666666666 +k=0.9996 +");
    add_proj4text (p, 1,
		   "x_0=2520000 +y_0=0 +ellps=intl +pm=rome +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Monte Mario (Rome) / Italy zone 2 (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"Monte Mario (Rome)\",DATUM[\"Monte_Mario_Rome\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"International 1924\",6378388,297,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"6806\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Rome\",12.45233333333333,AUTHORITY[\"EPSG\",\"8906\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4806\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 8,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",2.54766666666666],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.9996],PARAMETER[\"false_easting\",2520000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2659");
    add_srs_wkt (p, 12, "2\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26632, "epsg", 26632,
		      "M'poraloko / UTM zone 32N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +a=6378249.2 +b=6356515 +units=m +no_");
    add_proj4text (p, 1, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"M'poraloko / UTM zone 32N\",GEOGCS[\"M'poraloko");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"M_poraloko\",SPHEROID[\"Clarke 1880 (IGN)\",6");
    add_srs_wkt (p, 2,
		 "378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6266\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4266\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",9],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26632\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26692, "epsg", 26692,
		      "M'poraloko / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +a=6378249.2 +b=6356515 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"M'poraloko / UTM zone 32S\",GEOGCS[\"M'poraloko");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"M_poraloko\",SPHEROID[\"Clarke 1880 (IGN)\",6");
    add_srs_wkt (p, 2,
		 "378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6266\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4266\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",9],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",10000000],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"26692\"],AXIS[\"Easting\",EAST],AXIS[\"N");
    add_srs_wkt (p, 12, "orthing\",NORTH]]");
    p = add_epsg_def (first, last, 26701, "epsg", 26701, "NAD27 / UTM zone 1N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 1N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-177],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26701\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26702, "epsg", 26702, "NAD27 / UTM zone 2N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 2N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-171],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26702\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26703, "epsg", 26703, "NAD27 / UTM zone 3N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 3N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-165],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26703\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26704, "epsg", 26704, "NAD27 / UTM zone 4N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 4N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-159],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26704\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26705, "epsg", 26705, "NAD27 / UTM zone 5N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 5N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-153],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26705\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26706, "epsg", 26706, "NAD27 / UTM zone 6N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 6N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-147],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26706\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26707, "epsg", 26707, "NAD27 / UTM zone 7N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 7N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-141],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26707\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26708, "epsg", 26708, "NAD27 / UTM zone 8N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 8N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-135],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26708\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26709, "epsg", 26709, "NAD27 / UTM zone 9N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 9N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-129],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26709\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26710, "epsg", 26710,
		      "NAD27 / UTM zone 10N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 10N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-123],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26710\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26711, "epsg", 26711,
		      "NAD27 / UTM zone 11N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 11N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-117],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26711\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26712, "epsg", 26712,
		      "NAD27 / UTM zone 12N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 12N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-111],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26712\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26713, "epsg", 26713,
		      "NAD27 / UTM zone 13N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 13N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-105],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9996],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26713\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 26714, "epsg", 26714,
		      "NAD27 / UTM zone 14N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 14N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-99],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26714\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26715, "epsg", 26715,
		      "NAD27 / UTM zone 15N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 15N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-93],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26715\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26716, "epsg", 26716,
		      "NAD27 / UTM zone 16N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 16N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-87],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26716\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26717, "epsg", 26717,
		      "NAD27 / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 17N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-81],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26717\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26718, "epsg", 26718,
		      "NAD27 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 18N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-75],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26718\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26719, "epsg", 26719,
		      "NAD27 / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 19N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-69],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26719\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26720, "epsg", 26720,
		      "NAD27 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 20N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-63],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26720\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26721, "epsg", 26721,
		      "NAD27 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 21N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-57],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26721\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26722, "epsg", 26722,
		      "NAD27 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=clrk66 +datum=NAD27 +units=m +");
    add_proj4text (p, 1, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / UTM zone 22N\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-51],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9996],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"26722\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26729, "epsg", 26729,
		      "NAD27 / Alabama East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30.5 +lon_0=-85.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9996 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=");
    add_proj4text (p, 2, "NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alabama East\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",30.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-85.83333333333333],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99996],PARAMETER[\"false_easting\",500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2672");
    add_srs_wkt (p, 12, "9\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26730, "epsg", 26730,
		      "NAD27 / Alabama West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-87.5 +k=0.999933333 +x_0=1");
    add_proj4text (p, 1,
		   "52400.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +unit");
    add_proj4text (p, 2, "s=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alabama West\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",30],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-87.5],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "9933333],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"26730\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26731, "epsg", 26731,
		      "NAD27 / Alaska zone 1");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=57 +lonc=-133.6666666666667 +alpha=32");
    add_proj4text (p, 1,
		   "3.1301023611111 +k=0.9999 +x_0=5000000.001016002 +y_0=-5");
    add_proj4text (p, 2,
		   "000000.001016002 +ellps=clrk66 +datum=NAD27 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 1\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Hotine_Oblique");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_center\",57],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"longitude_of_center\",-133.6666666666667],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"azimuth\",323.1301023611111],PARAMETER[\"rectified_g");
    add_srs_wkt (p, 11,
		 "rid_angle\",323.1301023611111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 12,
		 ",0.9999],PARAMETER[\"false_easting\",16404166.67],PARAME");
    add_srs_wkt (p, 13,
		 "TER[\"false_northing\",-16404166.67],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 14, "\"26731\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26732, "epsg", 26732,
		      "NAD27 / Alaska zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-142 +k=0.9999 +x_0=152400.");
    add_proj4text (p, 1,
		   "3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 2\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-142],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26732\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26733, "epsg", 26733,
		      "NAD27 / Alaska zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-146 +k=0.9999 +x_0=152400.");
    add_proj4text (p, 1,
		   "3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 3\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-146],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26733\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26734, "epsg", 26734,
		      "NAD27 / Alaska zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-150 +k=0.9999 +x_0=152400.");
    add_proj4text (p, 1,
		   "3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 4\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-150],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26734\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26735, "epsg", 26735,
		      "NAD27 / Alaska zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-154 +k=0.9999 +x_0=152400.");
    add_proj4text (p, 1,
		   "3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 5\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-154],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26735\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26736, "epsg", 26736,
		      "NAD27 / Alaska zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-158 +k=0.9999 +x_0=152400.");
    add_proj4text (p, 1,
		   "3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 6\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-158],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26736\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26737, "epsg", 26737,
		      "NAD27 / Alaska zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-162 +k=0.9999 +x_0=213360.");
    add_proj4text (p, 1,
		   "4267208534 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 7\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-162],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",700000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26737\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26738, "epsg", 26738,
		      "NAD27 / Alaska zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-166 +k=0.9999 +x_0=152400.");
    add_proj4text (p, 1,
		   "3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 8\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-166],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26738\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26739, "epsg", 26739,
		      "NAD27 / Alaska zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-170 +k=0.9999 +x_0=182880.");
    add_proj4text (p, 1,
		   "3657607315 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 2, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 9\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",54],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-170],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9999],PARAMETER[\"false_easting\",600000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26739\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26740, "epsg", 26740,
		      "NAD27 / Alaska zone 10");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=53.83333333333334 +lat_2=51.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=51 +lon_0=-176 +x_0=914401.8288036576 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Alaska zone 10\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",53.83");
    add_srs_wkt (p, 9,
		 "333333333334],PARAMETER[\"standard_parallel_2\",51.83333");
    add_srs_wkt (p, 10,
		 "333333334],PARAMETER[\"latitude_of_origin\",51],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-176],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",3000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"26740\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26741, "epsg", 26741,
		      "NAD27 / California zone I");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.66666666666666 +lat_2=40 +lat_0=39.3");
    add_proj4text (p, 1,
		   "3333333333334 +lon_0=-122 +x_0=609601.2192024384 +y_0=0 ");
    add_proj4text (p, 2, "+ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone I\",GEOGCS[\"NAD27\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866");
    add_srs_wkt (p, 2,
		 "\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4267\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",4");
    add_srs_wkt (p, 9,
		 "1.66666666666666],PARAMETER[\"standard_parallel_2\",40],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",39.33333333333334],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-122],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"26741\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26742, "epsg", 26742,
		      "NAD27 / California zone II");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.83333333333334 +lat_2=38.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=37.66666666666666 +lon_0=-122 +x_0=609601.219");
    add_proj4text (p, 2,
		   "2024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone II\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "39.83333333333334],PARAMETER[\"standard_parallel_2\",38.");
    add_srs_wkt (p, 10,
		 "33333333333334],PARAMETER[\"latitude_of_origin\",37.6666");
    add_srs_wkt (p, 11,
		 "6666666666],PARAMETER[\"central_meridian\",-122],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",2000000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",0],AUTHORITY[\"EPSG\",\"26742\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26743, "epsg", 26743,
		      "NAD27 / California zone III");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.5 +lon_0=-120.5 +x_0=609601.2192024384 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone III\",GEOGCS[\"NAD27\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "66\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "08\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "38.43333333333333],PARAMETER[\"standard_parallel_2\",37.");
    add_srs_wkt (p, 10,
		 "06666666666667],PARAMETER[\"latitude_of_origin\",36.5],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"central_meridian\",-120.5],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "easting\",2000000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"26743\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 26744, "epsg", 26744,
		      "NAD27 / California zone IV");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.25 +lat_2=36 +lat_0=35.3333333333333");
    add_proj4text (p, 1,
		   "4 +lon_0=-119 +x_0=609601.2192024384 +y_0=0 +ellps=clrk6");
    add_proj4text (p, 2, "6 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone IV\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "37.25],PARAMETER[\"standard_parallel_2\",36],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",35.33333333333334],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-119],PARAMETER[\"false_easting\",2000000");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "6744\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26745, "epsg", 26745,
		      "NAD27 / California zone V");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.46666666666667 +lat_2=34.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.5 +lon_0=-118 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2, "0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone V\",GEOGCS[\"NAD27\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866");
    add_srs_wkt (p, 2,
		 "\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4267\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 9,
		 "5.46666666666667],PARAMETER[\"standard_parallel_2\",34.0");
    add_srs_wkt (p, 10,
		 "3333333333333],PARAMETER[\"latitude_of_origin\",33.5],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"central_meridian\",-118],PARAMETER[\"false_eas");
    add_srs_wkt (p, 12,
		 "ting\",2000000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 13,
		 "Y[\"EPSG\",\"26745\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 14, "]");
    p = add_epsg_def (first, last, 26746, "epsg", 26746,
		      "NAD27 / California zone VI");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.88333333333333 +lat_2=32.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=32.16666666666666 +lon_0=-116.25 +x_0=609601.");
    add_proj4text (p, 2,
		   "2192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone VI\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "33.88333333333333],PARAMETER[\"standard_parallel_2\",32.");
    add_srs_wkt (p, 10,
		 "78333333333333],PARAMETER[\"latitude_of_origin\",32.1666");
    add_srs_wkt (p, 11,
		 "6666666666],PARAMETER[\"central_meridian\",-116.25],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_easting\",2000000],PARAMETER[\"false_north");
    add_srs_wkt (p, 13,
		 "ing\",0],AUTHORITY[\"EPSG\",\"26746\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 14, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26747, "epsg", 26747,
		      "NAD27 / California zone VII (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.41666666666666 +lat_2=33.86666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=34.13333333333333 +lon_0=-118.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=1276106.450596901 +y_0=127079.524511049 +ellps=clrk6");
    add_proj4text (p, 3, "6 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone VII (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROI");
    add_srs_wkt (p, 2,
		 "D[\"Clarke 1866\",6378206.4,294.9786982139006,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM");
    add_srs_wkt (p, 4,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 5,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"4267\"]],UNIT[\"US survey foot\",0");
    add_srs_wkt (p, 7,
		 ".3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTI");
    add_srs_wkt (p, 8,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 9,
		 "_parallel_1\",34.41666666666666],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 10,
		 "rallel_2\",33.86666666666667],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 11,
		 "igin\",34.13333333333333],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 12,
		 ",-118.3333333333333],PARAMETER[\"false_easting\",4186692");
    add_srs_wkt (p, 13,
		 ".58],PARAMETER[\"false_northing\",416926.74],AUTHORITY[\"");
    add_srs_wkt (p, 14,
		 "EPSG\",\"26747\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26748, "epsg", 26748,
		      "NAD27 / Arizona East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-110.1666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD");
    add_proj4text (p, 2, "27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Arizona East\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",31],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-110.1666666666667],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26748\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26749, "epsg", 26749,
		      "NAD27 / Arizona Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-111.9166666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD");
    add_proj4text (p, 2, "27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Arizona Central\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",31],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-111.9166666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.9999],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12, "6749\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26750, "epsg", 26750,
		      "NAD27 / Arizona West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-113.75 +k=0.999933333 +x_0");
    add_proj4text (p, 1,
		   "=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +un");
    add_proj4text (p, 2, "its=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Arizona West\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",31],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-113.75],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "999933333],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26750\"],AXI");
    add_srs_wkt (p, 12, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26751, "epsg", 26751,
		      "NAD27 / Arkansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.23333333333333 +lat_2=34.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=34.33333333333334 +lon_0=-92 +x_0=609601.2192");
    add_proj4text (p, 2,
		   "024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Arkansas North\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.23");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",34.93333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",34.333333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-92],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",2000000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"26751\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 26752, "epsg", 26752,
		      "NAD27 / Arkansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.76666666666667 +lat_2=33.3 +lat_0=32");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-92 +x_0=609601.2192024384 +y_0=0");
    add_proj4text (p, 2, " +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Arkansas South\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",34.76");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",33.3],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",32.66666666666666],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-92],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"26752\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26753, "epsg", 26753,
		      "NAD27 / Colorado North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.71666666666667 +lat_2=40.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=39.33333333333334 +lon_0=-105.5 +x_0=609601.2");
    add_proj4text (p, 2,
		   "192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Colorado North\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.71");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",40.78333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",39.333333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-105.5],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",2000000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "0],AUTHORITY[\"EPSG\",\"26753\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26754, "epsg", 26754,
		      "NAD27 / Colorado Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.75 +lat_2=38.45 +lat_0=37.8333333333");
    add_proj4text (p, 1,
		   "3334 +lon_0=-105.5 +x_0=609601.2192024384 +y_0=0 +ellps=");
    add_proj4text (p, 2, "clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Colorado Central\",GEOGCS[\"NAD27\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.");
    add_srs_wkt (p, 9,
		 "75],PARAMETER[\"standard_parallel_2\",38.45],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",37.83333333333334],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-105.5],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 12,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "26754\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26755, "epsg", 26755,
		      "NAD27 / Colorado South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.66666666666666 +lon_0=-105.5 +x_0=609601.2");
    add_proj4text (p, 2,
		   "192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Colorado South\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.43");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",37.23333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",36.666666666");
    add_srs_wkt (p, 11,
		 "66666],PARAMETER[\"central_meridian\",-105.5],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",2000000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "0],AUTHORITY[\"EPSG\",\"26755\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26756, "epsg", 26756, "NAD27 / Connecticut");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.86666666666667 +lat_2=41.2 +lat_0=40");
    add_proj4text (p, 1,
		   ".83333333333334 +lon_0=-72.75 +x_0=182880.3657607315 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Connecticut\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.86666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",41.2],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",40.83333333333334],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-72.75],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",600000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"26756\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26757, "epsg", 26757, "NAD27 / Delaware");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=-75.41666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "995 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=N");
    add_proj4text (p, 2, "AD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Delaware\",GEOGCS[\"NAD27\",DATUM[\"Nor");
    add_srs_wkt (p, 1,
		 "th_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637820");
    add_srs_wkt (p, 2,
		 "6.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "67\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",38],PARAMETER[\"cen");
    add_srs_wkt (p, 9,
		 "tral_meridian\",-75.41666666666667],PARAMETER[\"scale_fa");
    add_srs_wkt (p, 10,
		 "ctor\",0.999995],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26757\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26758, "epsg", 26758,
		      "NAD27 / Florida East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-81 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datu");
    add_proj4text (p, 2, "m=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Florida East\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",24.33333333333333");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-81],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.999941177],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12, "6758\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_21 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 26759, "epsg", 26759,
		      "NAD27 / Florida West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-82 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datu");
    add_proj4text (p, 2, "m=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Florida West\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",24.33333333333333");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-82],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.999941177],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12, "6759\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26760, "epsg", 26760,
		      "NAD27 / Florida North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.75 +lat_2=29.58333333333333 +lat_0=2");
    add_proj4text (p, 1,
		   "9 +lon_0=-84.5 +x_0=609601.2192024384 +y_0=0 +ellps=clrk");
    add_proj4text (p, 2, "66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Florida North\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",30.75");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"standard_parallel_2\",29.58333333333333],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"latitude_of_origin\",29],PARAMETER[\"central_");
    add_srs_wkt (p, 11,
		 "meridian\",-84.5],PARAMETER[\"false_easting\",2000000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2676");
    add_srs_wkt (p, 13, "0\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26766, "epsg", 26766,
		      "NAD27 / Georgia East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-82.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD");
    add_proj4text (p, 2, "27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Georgia East\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",30],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-82.16666666666667],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26766\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26767, "epsg", 26767,
		      "NAD27 / Georgia West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-84.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD");
    add_proj4text (p, 2, "27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Georgia West\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",30],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-84.16666666666667],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26767\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26768, "epsg", 26768, "NAD27 / Idaho East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-112.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999473679999999 +x_0=152400.3048006096 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Idaho East\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",41.66666666666666");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-112.1666666666667],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999947368],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"26768\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 26769, "epsg", 26769,
		      "NAD27 / Idaho Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-114 +k=0.99");
    add_proj4text (p, 1,
		   "99473679999999 +x_0=152400.3048006096 +y_0=0 +ellps=clrk");
    add_proj4text (p, 2, "66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Idaho Central\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",41.66666666666");
    add_srs_wkt (p, 9,
		 "666],PARAMETER[\"central_meridian\",-114],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",0.999947368],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"26769\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26770, "epsg", 26770, "NAD27 / Idaho West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-115.75 +k=0");
    add_proj4text (p, 1,
		   ".999933333 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +");
    add_proj4text (p, 2, "datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Idaho West\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",41.66666666666666");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-115.75],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",0.999933333],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"26770\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26771, "epsg", 26771,
		      "NAD27 / Illinois East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-88.33333333");
    add_proj4text (p, 1,
		   "333333 +k=0.9999749999999999 +x_0=152400.3048006096 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Illinois East\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",36.66666666666");
    add_srs_wkt (p, 9,
		 "666],PARAMETER[\"central_meridian\",-88.33333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",0.999975],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"26771\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 26772, "epsg", 26772,
		      "NAD27 / Illinois West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-90.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=152400.3048006096 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Illinois West\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",36.66666666666");
    add_srs_wkt (p, 9,
		 "666],PARAMETER[\"central_meridian\",-90.16666666666667],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"scale_factor\",0.999941177],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"26772\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 13, "RTH]]");
    p = add_epsg_def (first, last, 26773, "epsg", 26773,
		      "NAD27 / Indiana East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +da");
    add_proj4text (p, 2, "tum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Indiana East\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",37.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-85.66666666666667],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.999966667],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "26773\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26774, "epsg", 26774,
		      "NAD27 / Indiana West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +da");
    add_proj4text (p, 2, "tum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Indiana West\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",37.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-87.08333333333333],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.999966667],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "26774\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26775, "epsg", 26775, "NAD27 / Iowa North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.26666666666667 +lat_2=42.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41.5 +lon_0=-93.5 +x_0=609601.2192024384 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Iowa North\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43.26666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",42.06666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",41.5],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-93.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"26775\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26776, "epsg", 26776, "NAD27 / Iowa South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.61666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40 +lon_0=-93.5 +x_0=609601.2192024384 +y_0=0");
    add_proj4text (p, 2, " +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Iowa South\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.78333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",40.61666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",40],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-93.5],PARAMETER[\"false_easting\",20");
    add_srs_wkt (p, 12,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"26776\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26777, "epsg", 26777,
		      "NAD27 / Kansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.78333333333333 +lat_2=38.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=38.33333333333334 +lon_0=-98 +x_0=609601.2192");
    add_proj4text (p, 2,
		   "024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Kansas North\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.78333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",38.71666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",38.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-98],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",2000000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 13,
		 "HORITY[\"EPSG\",\"26777\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 14, "ORTH]]");
    p = add_epsg_def (first, last, 26778, "epsg", 26778,
		      "NAD27 / Kansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.56666666666667 +lat_2=37.26666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.66666666666666 +lon_0=-98.5 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Kansas South\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.56666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",37.26666666");
    add_srs_wkt (p, 10,
		 "666667],PARAMETER[\"latitude_of_origin\",36.666666666666");
    add_srs_wkt (p, 11,
		 "66],PARAMETER[\"central_meridian\",-98.5],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",2000000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"26778\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 26779, "epsg", 26779,
		      "NAD27 / Kentucky North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=38.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=609601.2192024384 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Kentucky North\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",37.96");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",38.96666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"latitude_of_origin\",37.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-84.25],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"26779\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26780, "epsg", 26780,
		      "NAD27 / Kentucky South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.73333333333333 +lat_2=37.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=609601.2");
    add_proj4text (p, 2,
		   "192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Kentucky South\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.73");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",37.93333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",36.333333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-85.75],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_easting\",2000000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 13,
		 "0],AUTHORITY[\"EPSG\",\"26780\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 14, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26781, "epsg", 26781,
		      "NAD27 / Louisiana North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=31.16666666666667 +lat_2=32.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=30.66666666666667 +lon_0=-92.5 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Louisiana North\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",31.");
    add_srs_wkt (p, 9,
		 "16666666666667],PARAMETER[\"standard_parallel_2\",32.666");
    add_srs_wkt (p, 10,
		 "66666666666],PARAMETER[\"latitude_of_origin\",30.6666666");
    add_srs_wkt (p, 11,
		 "6666667],PARAMETER[\"central_meridian\",-92.5],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_easting\",2000000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"26781\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 14, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26782, "epsg", 26782,
		      "NAD27 / Louisiana South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=29.3 +lat_2=30.7 +lat_0=28.666666666666");
    add_proj4text (p, 1,
		   "67 +lon_0=-91.33333333333333 +x_0=609601.2192024384 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Louisiana South\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",29.");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",30.7],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",28.66666666666667],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-91.33333333333333],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"26782\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 14, "");
    p = add_epsg_def (first, last, 26783, "epsg", 26783, "NAD27 / Maine East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.83333333333334 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=N");
    add_proj4text (p, 2, "AD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Maine East\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",43.83333333333334");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-68.5],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.9999],PARAMETER[\"false_easting\",500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2678");
    add_srs_wkt (p, 12, "3\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26784, "epsg", 26784, "NAD27 / Maine West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=152400.3048006096 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Maine West\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",42.83333333333334");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-70.16666666666667],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999966667],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"26784\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 26785, "epsg", 26785, "NAD27 / Maryland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.3 +lat_2=39.45 +lat_0=37.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-77 +x_0=243840.4876809754 +y_0=0 +ellps=clrk");
    add_proj4text (p, 2, "66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Maryland\",GEOGCS[\"NAD27\",DATUM[\"Nor");
    add_srs_wkt (p, 1,
		 "th_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637820");
    add_srs_wkt (p, 2,
		 "6.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "67\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 8,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",38.3],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"standard_parallel_2\",39.45],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",37.83333333333334],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-77],PARAMETER[\"false_easting\",800000.00000000");
    add_srs_wkt (p, 12,
		 "02],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "26785\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26786, "epsg", 26786,
		      "NAD27 / Massachusetts Mainland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.71666666666667 +lat_2=42.68333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-71.5 +x_0=182880.3657607315 +y_0=0");
    add_proj4text (p, 2, " +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Massachusetts Mainland\",GEOGCS[\"NAD27");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 2,
		 " 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 7,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 8,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 9,
		 "\",41.71666666666667],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 10,
		 "42.68333333333333],PARAMETER[\"latitude_of_origin\",41],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"central_meridian\",-71.5],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "easting\",600000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 13,
		 "ITY[\"EPSG\",\"26786\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 14, "H]]");
    p = add_epsg_def (first, last, 26787, "epsg", 26787,
		      "NAD27 / Massachusetts Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.28333333333333 +lat_2=41.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-70.5 +x_0=60960.12192024384 +y_0=0");
    add_proj4text (p, 2, " +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Massachusetts Island\",GEOGCS[\"NAD27\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 2,
		 "866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 7,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 8,
		 "Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 9,
		 ",41.28333333333333],PARAMETER[\"standard_parallel_2\",41");
    add_srs_wkt (p, 10,
		 ".48333333333333],PARAMETER[\"latitude_of_origin\",41],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"central_meridian\",-70.5],PARAMETER[\"false_ea");
    add_srs_wkt (p, 12,
		 "sting\",200000],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 13,
		 "Y[\"EPSG\",\"26787\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 14, "]");
    p = add_epsg_def (first, last, 26791, "epsg", 26791,
		      "NAD27 / Minnesota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.03333333333333 +lat_2=48.63333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Minnesota North\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",47.");
    add_srs_wkt (p, 9,
		 "03333333333333],PARAMETER[\"standard_parallel_2\",48.633");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"latitude_of_origin\",46.5],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-93.1],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"26791\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 14, "");
    p = add_epsg_def (first, last, 26792, "epsg", 26792,
		      "NAD27 / Minnesota Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.61666666666667 +lat_2=47.05 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=609601.2192024384 +y_0=0 +ellps=clr");
    add_proj4text (p, 2, "k66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Minnesota Central\",GEOGCS[\"NAD27\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866");
    add_srs_wkt (p, 2,
		 "\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4267\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",4");
    add_srs_wkt (p, 9,
		 "5.61666666666667],PARAMETER[\"standard_parallel_2\",47.0");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"latitude_of_origin\",45],PARAMETER[\"cent");
    add_srs_wkt (p, 11,
		 "ral_meridian\",-94.25],PARAMETER[\"false_easting\",20000");
    add_srs_wkt (p, 12,
		 "00],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "26792\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26793, "epsg", 26793,
		      "NAD27 / Minnesota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.78333333333333 +lat_2=45.21666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=43 +lon_0=-94 +x_0=609601.2192024384 +y_0=0 +");
    add_proj4text (p, 2, "ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Minnesota South\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43.");
    add_srs_wkt (p, 9,
		 "78333333333333],PARAMETER[\"standard_parallel_2\",45.216");
    add_srs_wkt (p, 10,
		 "66666666667],PARAMETER[\"latitude_of_origin\",43],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-94],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"26793\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26794, "epsg", 26794,
		      "NAD27 / Mississippi East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.66666666666667 +lon_0=-88.83333333");
    add_proj4text (p, 1,
		   "333333 +k=0.99996 +x_0=152400.3048006096 +y_0=0 +ellps=c");
    add_proj4text (p, 2, "lrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Mississippi East\",GEOGCS[\"NAD27\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",29.666666666");
    add_srs_wkt (p, 9,
		 "66667],PARAMETER[\"central_meridian\",-88.83333333333333");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.99996],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"26794\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 26795, "epsg", 26795,
		      "NAD27 / Mississippi West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30.5 +lon_0=-90.33333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99941177 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +da");
    add_proj4text (p, 2, "tum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Mississippi West\",GEOGCS[\"NAD27\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",30.5],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"central_meridian\",-90.33333333333333],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.999941177],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"26795\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26796, "epsg", 26796,
		      "NAD27 / Missouri East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-90.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +da");
    add_proj4text (p, 2, "tum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Missouri East\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",35.83333333333");
    add_srs_wkt (p, 9,
		 "334],PARAMETER[\"central_meridian\",-90.5],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999933333],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"26796\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26797, "epsg", 26797,
		      "NAD27 / Missouri Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-92.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +da");
    add_proj4text (p, 2, "tum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Missouri Central\",GEOGCS[\"NAD27\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",35.833333333");
    add_srs_wkt (p, 9,
		 "33334],PARAMETER[\"central_meridian\",-92.5],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.999933333],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"26797\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26798, "epsg", 26798,
		      "NAD27 / Missouri West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.16666666666666 +lon_0=-94.5 +k=0.9");
    add_proj4text (p, 1,
		   "99941177 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +da");
    add_proj4text (p, 2, "tum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Missouri West\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",36.16666666666");
    add_srs_wkt (p, 9,
		 "666],PARAMETER[\"central_meridian\",-94.5],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999941177],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"26798\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26799, "epsg", 26799,
		      "NAD27 / California zone VII");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.41666666666666 +lat_2=33.86666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=34.13333333333333 +lon_0=-118.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=1276106.450596901 +y_0=1268253.006858014 +ellps=clrk");
    add_proj4text (p, 3, "66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / California zone VII\",GEOGCS[\"NAD27\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "66\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "08\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "34.41666666666666],PARAMETER[\"standard_parallel_2\",33.");
    add_srs_wkt (p, 10,
		 "86666666666667],PARAMETER[\"latitude_of_origin\",34.1333");
    add_srs_wkt (p, 11,
		 "3333333333],PARAMETER[\"central_meridian\",-118.33333333");
    add_srs_wkt (p, 12,
		 "33333],PARAMETER[\"false_easting\",4186692.58],PARAMETER");
    add_srs_wkt (p, 13,
		 "[\"false_northing\",4160926.74],AUTHORITY[\"EPSG\",\"267");
    add_srs_wkt (p, 14, "99\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26801, "epsg", 26801,
		      "NAD Michigan / Michigan East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.5 +lon_0=-83.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99942857 +x_0=152400.3048006096 +y_0=0 +a=6378450.047548");
    add_proj4text (p, 2, "896 +b=6356826.621488444 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD Michigan / Michigan East\",GEOGCS[\"NAD27 M");
    add_srs_wkt (p, 1,
		 "ichigan\",DATUM[\"NAD_Michigan\",SPHEROID[\"Clarke 1866 ");
    add_srs_wkt (p, 2,
		 "Michigan\",6378450.047548896,294.9786971646739,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7009\"]],AUTHORITY[\"EPSG\",\"6268\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4268\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",41.5],PARAMETER[\"central_meridian\",-83.666666666");
    add_srs_wkt (p, 10,
		 "66667],PARAMETER[\"scale_factor\",0.999942857],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"26801\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26802, "epsg", 26802,
		      "NAD Michigan / Michigan Old Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.5 +lon_0=-85.75 +k=0.999909091 +x_");
    add_proj4text (p, 1,
		   "0=152400.3048006096 +y_0=0 +a=6378450.047548896 +b=63568");
    add_proj4text (p, 2, "26.621488444 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD Michigan / Michigan Old Central\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27 Michigan\",DATUM[\"NAD_Michigan\",SPHEROID[\"Clark");
    add_srs_wkt (p, 2,
		 "e 1866 Michigan\",6378450.047548896,294.9786971646739,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7009\"]],AUTHORITY[\"EPSG\",\"6268\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4268\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",41.5],PARAMETER[\"central_meridian\",-85.75");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.999909091],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",500000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 12,
		 "THORITY[\"EPSG\",\"26802\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 26803, "epsg", 26803,
		      "NAD Michigan / Michigan West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.5 +lon_0=-88.75 +k=0.999909091 +x_");
    add_proj4text (p, 1,
		   "0=152400.3048006096 +y_0=0 +a=6378450.047548896 +b=63568");
    add_proj4text (p, 2, "26.621488444 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD Michigan / Michigan West\",GEOGCS[\"NAD27 M");
    add_srs_wkt (p, 1,
		 "ichigan\",DATUM[\"NAD_Michigan\",SPHEROID[\"Clarke 1866 ");
    add_srs_wkt (p, 2,
		 "Michigan\",6378450.047548896,294.9786971646739,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7009\"]],AUTHORITY[\"EPSG\",\"6268\"]],PRIME");
    add_srs_wkt (p, 4,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 5,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 6,
		 "],AUTHORITY[\"EPSG\",\"4268\"]],UNIT[\"US survey foot\",");
    add_srs_wkt (p, 7,
		 "0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",41.5],PARAMETER[\"central_meridian\",-88.75],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.999909091],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"26803\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 26811, "epsg", 26811,
		      "NAD Michigan / Michigan North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.48333333333333 +lat_2=47.08333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=44.78333333333333 +lon_0=-87 +x_0=609601.2192");
    add_proj4text (p, 2,
		   "024384 +y_0=0 +a=6378450.047548896 +b=6356826.621488444 ");
    add_proj4text (p, 3, "+units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD Michigan / Michigan North\",GEOGCS[\"NAD27 ");
    add_srs_wkt (p, 1,
		 "Michigan\",DATUM[\"NAD_Michigan\",SPHEROID[\"Clarke 1866");
    add_srs_wkt (p, 2,
		 " Michigan\",6378450.047548896,294.9786971646739,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7009\"]],AUTHORITY[\"EPSG\",\"6268\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4268\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",45.48333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",47.08333333333334],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",44.78333333333333],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",-87],PARAMETER[\"false_easting\",2000000],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"26811\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26812, "epsg", 26812,
		      "NAD Michigan / Michigan Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.18333333333333 +lat_2=45.7 +lat_0=43");
    add_proj4text (p, 1,
		   ".31666666666667 +lon_0=-84.33333333333333 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +a=6378450.047548896 +b=6356826.62148844");
    add_proj4text (p, 3, "4 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD Michigan / Michigan Central\",GEOGCS[\"NAD2");
    add_srs_wkt (p, 1,
		 "7 Michigan\",DATUM[\"NAD_Michigan\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "66 Michigan\",6378450.047548896,294.9786971646739,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7009\"]],AUTHORITY[\"EPSG\",\"6268\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4268\"]],UNIT[\"US survey foot");
    add_srs_wkt (p, 7,
		 "\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_1\",44.18333333333333],PARAMETER[\"standar");
    add_srs_wkt (p, 10,
		 "d_parallel_2\",45.7],PARAMETER[\"latitude_of_origin\",43");
    add_srs_wkt (p, 11,
		 ".31666666666667],PARAMETER[\"central_meridian\",-84.3333");
    add_srs_wkt (p, 12,
		 "3333333333],PARAMETER[\"false_easting\",2000000],PARAMET");
    add_srs_wkt (p, 13,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26812\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26813, "epsg", 26813,
		      "NAD Michigan / Michigan South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.1 +lat_2=43.66666666666666 +lat_0=41");
    add_proj4text (p, 1,
		   ".5 +lon_0=-84.33333333333333 +x_0=609601.2192024384 +y_0");
    add_proj4text (p, 2,
		   "=0 +a=6378450.047548896 +b=6356826.621488444 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD Michigan / Michigan South\",GEOGCS[\"NAD27 ");
    add_srs_wkt (p, 1,
		 "Michigan\",DATUM[\"NAD_Michigan\",SPHEROID[\"Clarke 1866");
    add_srs_wkt (p, 2,
		 " Michigan\",6378450.047548896,294.9786971646739,AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"7009\"]],AUTHORITY[\"EPSG\",\"6268\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4268\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",42.1],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 10,
		 "43.66666666666666],PARAMETER[\"latitude_of_origin\",41.5");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"central_meridian\",-84.33333333333333],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_easting\",2000000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 13,
		 "hing\",0],AUTHORITY[\"EPSG\",\"26813\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 14, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26814, "epsg", 26814,
		      "NAD83 / Maine East (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine East (ftUS) (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",43.66666666666666],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-68.5],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "99],PARAMETER[\"false_easting\",300000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"26814\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26815, "epsg", 26815,
		      "NAD83 / Maine West (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine West (ftUS) (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",42.83333333333334],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",-70.16666666666667],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.999966667],PARAMETER[\"false_easting\",900000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12, "6815\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26819, "epsg", 26819,
		      "NAD83 / Minnesota North (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000.00");
    add_proj4text (p, 2,
		   "00101601 +y_0=99999.99998984 +ellps=GRS80 +datum=NAD83 +");
    add_proj4text (p, 3, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota North (ftUS) (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",48.63333333333333]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",47.03333333333333],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",46.5],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-93.1],PARAMETER[\"false_easting\",800000.00");
    add_srs_wkt (p, 12,
		 "00101601],PARAMETER[\"false_northing\",99999.99998984],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"26819\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 26820, "epsg", 26820,
		      "NAD83 / Minnesota Central (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000.0000101601 +y_0=99999.999989");
    add_proj4text (p, 2, "84 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota Central (ftUS) (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",47.05],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_2\",45.61666666666667],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",45],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 11,
		 "-94.25],PARAMETER[\"false_easting\",800000.0000101601],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",99999.99998984],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"26820\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26821, "epsg", 26821,
		      "NAD83 / Minnesota South (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000.0000101601 +y_0=999");
    add_proj4text (p, 2,
		   "99.99998984 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota South (ftUS) (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHE");
    add_srs_wkt (p, 2,
		 "ROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"standard_parallel_1\",45.21666666666667]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",43.78333333333333],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",43],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-94],PARAMETER[\"false_easting\",800000.000010");
    add_srs_wkt (p, 12,
		 "1601],PARAMETER[\"false_northing\",99999.99998984],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"26821\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 26822, "epsg", 26822,
		      "NAD83 / Nebraska (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000.0000101601 +y_0=0 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nebraska (ftUS) (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"standard_parallel_1\",43],PARAMETER[\"standard_p");
    add_srs_wkt (p, 9,
		 "arallel_2\",40],PARAMETER[\"latitude_of_origin\",39.8333");
    add_srs_wkt (p, 10,
		 "3333333334],PARAMETER[\"central_meridian\",-100],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",500000.0000101601],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"26822\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26823, "epsg", 26823,
		      "NAD83 / West Virginia North (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=1968500 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / West Virginia North (ftUS) (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",40.25],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_2\",39],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 10,
		 "gin\",38.5],PARAMETER[\"central_meridian\",-79.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",1968500],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"26823\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26824, "epsg", 26824,
		      "NAD83 / West Virginia South (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=1968500 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / West Virginia South (ftUS) (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"NAD83\",DATUM[\"North_American_Datum_1983\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"standard_parallel_1\",38.8833333333333");
    add_srs_wkt (p, 9,
		 "3],PARAMETER[\"standard_parallel_2\",37.48333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",37],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-81],PARAMETER[\"false_easting\",1968500],PA");
    add_srs_wkt (p, 12,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26824");
    add_srs_wkt (p, 13, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26825, "epsg", 26825,
		      "NAD83(HARN) / Maine East (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine East (ftUS) (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regio");
    add_srs_wkt (p, 2,
		 "nal_Network\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"615");
    add_srs_wkt (p, 4,
		 "2\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",43.6666666");
    add_srs_wkt (p, 9,
		 "6666666],PARAMETER[\"central_meridian\",-68.5],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",30");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"26825\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26826, "epsg", 26826,
		      "NAD83(HARN) / Maine West (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine West (ftUS) (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regio");
    add_srs_wkt (p, 2,
		 "nal_Network\",SPHEROID[\"GRS 1980\",6378137,298.25722210");
    add_srs_wkt (p, 3,
		 "1,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"615");
    add_srs_wkt (p, 4,
		 "2\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 5,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",42.8333333");
    add_srs_wkt (p, 9,
		 "3333334],PARAMETER[\"central_meridian\",-70.166666666666");
    add_srs_wkt (p, 10,
		 "67],PARAMETER[\"scale_factor\",0.999966667],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",900000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"26826\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 26830, "epsg", 26830,
		      "NAD83(HARN) / Minnesota North (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000.00");
    add_proj4text (p, 2,
		   "00101601 +y_0=99999.99998984 +ellps=GRS80 +units=m +no_d");
    add_proj4text (p, 3, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota North (ftUS) (deprecate");
    add_srs_wkt (p, 1,
		 "d)\",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_");
    add_srs_wkt (p, 2,
		 "Regional_Network\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6152\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 8,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_1\",48.63333333333333],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 10,
		 ",47.03333333333333],PARAMETER[\"latitude_of_origin\",46.");
    add_srs_wkt (p, 11,
		 "5],PARAMETER[\"central_meridian\",-93.1],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_easting\",800000.0000101601],PARAMETER[\"false_northi");
    add_srs_wkt (p, 13,
		 "ng\",99999.99998984],AUTHORITY[\"EPSG\",\"26830\"],AXIS[");
    add_srs_wkt (p, 14, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26831, "epsg", 26831,
		      "NAD83(HARN) / Minnesota Central (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000.0000101601 +y_0=99999.999989");
    add_proj4text (p, 2, "84 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota Central (ftUS) (depreca");
    add_srs_wkt (p, 1,
		 "ted)\",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accurac");
    add_srs_wkt (p, 2,
		 "y_Regional_Network\",SPHEROID[\"GRS 1980\",6378137,298.2");
    add_srs_wkt (p, 3,
		 "57222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"6152\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 8,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_1\",47.05],PARAMETER[\"standard_parallel_2\",45.6166666");
    add_srs_wkt (p, 10,
		 "6666667],PARAMETER[\"latitude_of_origin\",45],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"central_meridian\",-94.25],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",800000.0000101601],PARAMETER[\"false_northing\",99999.9");
    add_srs_wkt (p, 13,
		 "9998984],AUTHORITY[\"EPSG\",\"26831\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 14, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26832, "epsg", 26832,
		      "NAD83(HARN) / Minnesota South (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000.0000101601 +y_0=999");
    add_proj4text (p, 2, "99.99998984 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota South (ftUS) (deprecate");
    add_srs_wkt (p, 1,
		 "d)\",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_");
    add_srs_wkt (p, 2,
		 "Regional_Network\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6152\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 8,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_1\",45.21666666666667],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 10,
		 ",43.78333333333333],PARAMETER[\"latitude_of_origin\",43]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"central_meridian\",-94],PARAMETER[\"false_e");
    add_srs_wkt (p, 12,
		 "asting\",800000.0000101601],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",99999.99998984],AUTHORITY[\"EPSG\",\"26832\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26833, "epsg", 26833,
		      "NAD83(HARN) / Nebraska (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000.0000101601 +y_0=0 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nebraska (ftUS) (deprecated)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regiona");
    add_srs_wkt (p, 2,
		 "l_Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 5,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Confo");
    add_srs_wkt (p, 8,
		 "rmal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"standard_parallel_2\",40],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",39.83333333333334],PARAMETER[\"central_me");
    add_srs_wkt (p, 11,
		 "ridian\",-100],PARAMETER[\"false_easting\",500000.000010");
    add_srs_wkt (p, 12,
		 "1601],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"26833\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26834, "epsg", 26834,
		      "NAD83(HARN) / West Virginia North (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=1968500 +y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / West Virginia North (ftUS) (depre");
    add_srs_wkt (p, 1,
		 "cated)\",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accur");
    add_srs_wkt (p, 2,
		 "acy_Regional_Network\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6152\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_1\",40.25],PARAMETER[\"standard_parallel_2\",39],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",38.5],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-79.5],PARAMETER[\"false_easting\",1968500],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"268");
    add_srs_wkt (p, 13, "34\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26835, "epsg", 26835,
		      "NAD83(HARN) / West Virginia South (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=1968500 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / West Virginia South (ftUS) (depre");
    add_srs_wkt (p, 1,
		 "cated)\",GEOGCS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accur");
    add_srs_wkt (p, 2,
		 "acy_Regional_Network\",SPHEROID[\"GRS 1980\",6378137,298");
    add_srs_wkt (p, 3,
		 ".257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"6152\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_1\",38.88333333333333],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 10,
		 "l_2\",37.48333333333333],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",37],PARAMETER[\"central_meridian\",-81],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_easting\",1968500],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 13,
		 "THORITY[\"EPSG\",\"26835\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 14, "NORTH]]");
    p = add_epsg_def (first, last, 26836, "epsg", 26836,
		      "NAD83(NSRS2007) / Maine East (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0");
    add_proj4text (p, 2, ",0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine East (ftUS) (deprecated");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Sp");
    add_srs_wkt (p, 2,
		 "atial_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 3,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 5,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 6,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 8,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"latitude_of_origin\",43.66666666666666],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"central_meridian\",-68.5],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",0.9999],PARAMETER[\"false_easting\",300000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"26836\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26837, "epsg", 26837,
		      "NAD83(NSRS2007) / Maine West (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine West (ftUS) (deprecated");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Sp");
    add_srs_wkt (p, 2,
		 "atial_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 3,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 5,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 6,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 8,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"latitude_of_origin\",42.83333333333334],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"central_meridian\",-70.16666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 11,
		 "cale_factor\",0.999966667],PARAMETER[\"false_easting\",9");
    add_srs_wkt (p, 12,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"26837\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26841, "epsg", 26841,
		      "NAD83(NSRS2007) / Minnesota North (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000.00");
    add_proj4text (p, 2,
		   "00101601 +y_0=99999.99998984 +ellps=GRS80 +towgs84=0,0,0");
    add_proj4text (p, 3, ",0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota North (ftUS) (depre");
    add_srs_wkt (p, 1,
		 "cated)\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_Nation");
    add_srs_wkt (p, 2,
		 "al_Spatial_Reference_System_2007\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 3,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 4,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM");
    add_srs_wkt (p, 5,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 6,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 9,
		 "ic_2SP\"],PARAMETER[\"standard_parallel_1\",48.633333333");
    add_srs_wkt (p, 10,
		 "33333],PARAMETER[\"standard_parallel_2\",47.033333333333");
    add_srs_wkt (p, 11,
		 "33],PARAMETER[\"latitude_of_origin\",46.5],PARAMETER[\"c");
    add_srs_wkt (p, 12,
		 "entral_meridian\",-93.1],PARAMETER[\"false_easting\",800");
    add_srs_wkt (p, 13,
		 "000.0000101601],PARAMETER[\"false_northing\",99999.99998");
    add_srs_wkt (p, 14,
		 "984],AUTHORITY[\"EPSG\",\"26841\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 15, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26842, "epsg", 26842,
		      "NAD83(NSRS2007) / Minnesota Central (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000.0000101601 +y_0=99999.999989");
    add_proj4text (p, 2,
		   "84 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota Central (ftUS) (dep");
    add_srs_wkt (p, 1,
		 "recated)\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_Nati");
    add_srs_wkt (p, 2,
		 "onal_Spatial_Reference_System_2007\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 3,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 4,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIM");
    add_srs_wkt (p, 5,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 6,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 7,
		 "]],AUTHORITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 8,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 9,
		 "onic_2SP\"],PARAMETER[\"standard_parallel_1\",47.05],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"standard_parallel_2\",45.61666666666667],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"latitude_of_origin\",45],PARAMETER[\"central_merid");
    add_srs_wkt (p, 12,
		 "ian\",-94.25],PARAMETER[\"false_easting\",800000.0000101");
    add_srs_wkt (p, 13,
		 "601],PARAMETER[\"false_northing\",99999.99998984],AUTHOR");
    add_srs_wkt (p, 14,
		 "ITY[\"EPSG\",\"26842\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 15, "H]]");
    p = add_epsg_def (first, last, 26843, "epsg", 26843,
		      "NAD83(NSRS2007) / Minnesota South (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000.0000101601 +y_0=999");
    add_proj4text (p, 2,
		   "99.99998984 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota South (ftUS) (depre");
    add_srs_wkt (p, 1,
		 "cated)\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_Nation");
    add_srs_wkt (p, 2,
		 "al_Spatial_Reference_System_2007\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 3,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOW");
    add_srs_wkt (p, 4,
		 "GS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM");
    add_srs_wkt (p, 5,
		 "[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"de");
    add_srs_wkt (p, 6,
		 "gree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 9,
		 "ic_2SP\"],PARAMETER[\"standard_parallel_1\",45.216666666");
    add_srs_wkt (p, 10,
		 "66667],PARAMETER[\"standard_parallel_2\",43.783333333333");
    add_srs_wkt (p, 11,
		 "33],PARAMETER[\"latitude_of_origin\",43],PARAMETER[\"cen");
    add_srs_wkt (p, 12,
		 "tral_meridian\",-94],PARAMETER[\"false_easting\",800000.");
    add_srs_wkt (p, 13,
		 "0000101601],PARAMETER[\"false_northing\",99999.99998984]");
    add_srs_wkt (p, 14,
		 ",AUTHORITY[\"EPSG\",\"26843\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 15, "\",NORTH]]");
    p = add_epsg_def (first, last, 26844, "epsg", 26844,
		      "NAD83(NSRS2007) / Nebraska (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000.0000101601 +y_0=0 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nebraska (ftUS) (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spati");
    add_srs_wkt (p, 2,
		 "al_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 3,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0");
    add_srs_wkt (p, 4,
		 ",0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 5,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 6,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"standard_parallel_1\",43],PARAMETER[\"stan");
    add_srs_wkt (p, 10,
		 "dard_parallel_2\",40],PARAMETER[\"latitude_of_origin\",3");
    add_srs_wkt (p, 11,
		 "9.83333333333334],PARAMETER[\"central_meridian\",-100],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_easting\",500000.0000101601],PARAMETER[");
    add_srs_wkt (p, 13,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"26844\"],AXIS");
    add_srs_wkt (p, 14, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26845, "epsg", 26845,
		      "NAD83(NSRS2007) / West Virginia North (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=1968500 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,");
    add_proj4text (p, 2, "0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / West Virginia North (ftUS) (d");
    add_srs_wkt (p, 1,
		 "eprecated)\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_Na");
    add_srs_wkt (p, 2,
		 "tional_Spatial_Reference_System_2007\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 3,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 4,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PR");
    add_srs_wkt (p, 5,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 7,
		 "\"]],AUTHORITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 8,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 9,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.25],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"standard_parallel_2\",39],PARAMETER[\"latitud");
    add_srs_wkt (p, 11,
		 "e_of_origin\",38.5],PARAMETER[\"central_meridian\",-79.5");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",1968500],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"26845\"],AXIS[\"X\",");
    add_srs_wkt (p, 14, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26846, "epsg", 26846,
		      "NAD83(NSRS2007) / West Virginia South (ftUS) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=1968500 +y_0=0 +ellps=GRS8");
    add_proj4text (p, 2, "0 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / West Virginia South (ftUS) (d");
    add_srs_wkt (p, 1,
		 "eprecated)\",GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_Na");
    add_srs_wkt (p, 2,
		 "tional_Spatial_Reference_System_2007\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 3,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 4,
		 ",TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PR");
    add_srs_wkt (p, 5,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 7,
		 "\"]],AUTHORITY[\"EPSG\",\"4759\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 8,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 9,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.88333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"standard_parallel_2\",37.48333333");
    add_srs_wkt (p, 11,
		 "333333],PARAMETER[\"latitude_of_origin\",37],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "central_meridian\",-81],PARAMETER[\"false_easting\",1968");
    add_srs_wkt (p, 13,
		 "500],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 14, "\"26846\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26847, "epsg", 26847,
		      "NAD83 / Maine East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine East (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",43.66666666666666],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",-68.5],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9999],PARAMETER[\"false_easting\",984250.0000");
    add_srs_wkt (p, 11,
		 "000002],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"26847\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26848, "epsg", 26848,
		      "NAD83 / Maine West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine West (ftUS)\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",42.83333333333334],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"central_meridian\",-70.16666666666667],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.999966667],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",2952750],PARAMETER[\"false_northing\",0],AUTHORIT");
    add_srs_wkt (p, 12,
		 "Y[\"EPSG\",\"26848\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 13, "]");
    p = add_epsg_def (first, last, 26849, "epsg", 26849,
		      "NAD83 / Minnesota North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000.00");
    add_proj4text (p, 2,
		   "00101599 +y_0=99999.99998983997 +ellps=GRS80 +datum=NAD8");
    add_proj4text (p, 3, "3 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota North (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",48.63");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",47.03333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",46.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-93.1],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",2624666.6667],PARAMETER[\"false_northing\",328083.33");
    add_srs_wkt (p, 13,
		 "33],AUTHORITY[\"EPSG\",\"26849\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 14, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26850, "epsg", 26850,
		      "NAD83 / Minnesota Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000.0000101599 +y_0=99999.999989");
    add_proj4text (p, 2,
		   "83997 +ellps=GRS80 +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota Central (ftUS)\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",47.");
    add_srs_wkt (p, 9,
		 "05],PARAMETER[\"standard_parallel_2\",45.61666666666667]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"latitude_of_origin\",45],PARAMETER[\"centra");
    add_srs_wkt (p, 11,
		 "l_meridian\",-94.25],PARAMETER[\"false_easting\",2624666");
    add_srs_wkt (p, 12,
		 ".6667],PARAMETER[\"false_northing\",328083.3333],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"26850\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 26851, "epsg", 26851,
		      "NAD83 / Minnesota South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000.0000101599 +y_0=999");
    add_proj4text (p, 2,
		   "99.99998983997 +ellps=GRS80 +datum=NAD83 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota South (ftUS)\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",45.21");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",43.78333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",43],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-94],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "2624666.6667],PARAMETER[\"false_northing\",328083.3333],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"26851\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 26852, "epsg", 26852,
		      "NAD83 / Nebraska (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000.00001016 +y_0=0 +ellps=GRS80 +dat");
    add_proj4text (p, 2, "um=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nebraska (ftUS)\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 8,
		 "ic_2SP\"],PARAMETER[\"standard_parallel_1\",43],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"standard_parallel_2\",40],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 10,
		 "igin\",39.83333333333334],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-100],PARAMETER[\"false_easting\",1640416.6667],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26852\"],A");
    add_srs_wkt (p, 13, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26853, "epsg", 26853,
		      "NAD83 / West Virginia North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=600000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us");
    add_proj4text (p, 2, "-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / West Virginia North (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4269\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",4");
    add_srs_wkt (p, 9,
		 "0.25],PARAMETER[\"standard_parallel_2\",39],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",38.5],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-79.5],PARAMETER[\"false_easting\",1968500],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"26853\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26854, "epsg", 26854,
		      "NAD83 / West Virginia South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / West Virginia South (ftUS)\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "AD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GR");
    add_srs_wkt (p, 2,
		 "S 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4269\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",3");
    add_srs_wkt (p, 9,
		 "8.88333333333333],PARAMETER[\"standard_parallel_2\",37.4");
    add_srs_wkt (p, 10,
		 "8333333333333],PARAMETER[\"latitude_of_origin\",37],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"central_meridian\",-81],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",1968500],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"26854\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26855, "epsg", 26855,
		      "NAD83(HARN) / Maine East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +units=us");
    add_proj4text (p, 2, "-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine East (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",43.66666666666666],PARAMETER[\"central_meridian\",-68.5");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",984250.0000000002],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"26855\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26856, "epsg", 26856,
		      "NAD83(HARN) / Maine West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +u");
    add_proj4text (p, 2, "nits=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Maine West (ftUS)\",GEOGCS[\"NAD8");
    add_srs_wkt (p, 1,
		 "3(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 9,
		 ",42.83333333333334],PARAMETER[\"central_meridian\",-70.1");
    add_srs_wkt (p, 10,
		 "6666666666667],PARAMETER[\"scale_factor\",0.999966667],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",2952750],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"26856\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26857, "epsg", 26857,
		      "NAD83(HARN) / Minnesota North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000.00");
    add_proj4text (p, 2,
		   "00101599 +y_0=99999.99998983997 +ellps=GRS80 +units=us-f");
    add_proj4text (p, 3, "t +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota North (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",48.63333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",47.03333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",46.5],PARAMETER[\"central_meridian\",-93.1],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_easting\",2624666.6667],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",328083.3333],AUTHORITY[\"EPSG\",\"26857\"],A");
    add_srs_wkt (p, 14, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26858, "epsg", 26858,
		      "NAD83(HARN) / Minnesota Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000.0000101599 +y_0=99999.999989");
    add_proj4text (p, 2, "83997 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota Central (ftUS)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Ne");
    add_srs_wkt (p, 2,
		 "twork\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],");
    add_srs_wkt (p, 4,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 5,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 6,
		 "22\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey fo");
    add_srs_wkt (p, 7,
		 "ot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 9,
		 "andard_parallel_1\",47.05],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 10,
		 "_2\",45.61666666666667],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 11,
		 ",45],PARAMETER[\"central_meridian\",-94.25],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",2624666.6667],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",328083.3333],AUTHORITY[\"EPSG\",\"26858\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26859, "epsg", 26859,
		      "NAD83(HARN) / Minnesota South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000.0000101599 +y_0=999");
    add_proj4text (p, 2, "99.99998983997 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Minnesota South (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Netwo");
    add_srs_wkt (p, 2,
		 "rk\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRI");
    add_srs_wkt (p, 4,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\"");
    add_srs_wkt (p, 7,
		 ",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJEC");
    add_srs_wkt (p, 8,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_1\",45.21666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 10,
		 "parallel_2\",43.78333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 11,
		 "origin\",43],PARAMETER[\"central_meridian\",-94],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",2624666.6667],PARAMETER[\"false_nor");
    add_srs_wkt (p, 13,
		 "thing\",328083.3333],AUTHORITY[\"EPSG\",\"26859\"],AXIS[");
    add_srs_wkt (p, 14, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26860, "epsg", 26860,
		      "NAD83(HARN) / Nebraska (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000.00001016 +y_0=0 +ellps=GRS80 +uni");
    add_proj4text (p, 2, "ts=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / Nebraska (ftUS)\",GEOGCS[\"NAD83(");
    add_srs_wkt (p, 1,
		 "HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_Network\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey foot\",0.30");
    add_srs_wkt (p, 7,
		 "48006096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[");
    add_srs_wkt (p, 8,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_1\",43],PARAMETER[\"standard_parallel_2\",40],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"latitude_of_origin\",39.83333333333334],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"central_meridian\",-100],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",1640416.6667],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"26860\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 14, "");
    p = add_epsg_def (first, last, 26861, "epsg", 26861,
		      "NAD83(HARN) / West Virginia North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=600000 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / West Virginia North (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_");
    add_srs_wkt (p, 2,
		 "Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",40.25],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 10,
		 "el_2\",39],PARAMETER[\"latitude_of_origin\",38.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-79.5],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",1968500],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"26861\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26862, "epsg", 26862,
		      "NAD83(HARN) / West Virginia South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(HARN) / West Virginia South (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(HARN)\",DATUM[\"NAD83_High_Accuracy_Regional_");
    add_srs_wkt (p, 2,
		 "Network\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6152\"]");
    add_srs_wkt (p, 4,
		 "],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],U");
    add_srs_wkt (p, 5,
		 "NIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4152\"]],UNIT[\"US survey ");
    add_srs_wkt (p, 7,
		 "foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_1\",38.88333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 10,
		 "ndard_parallel_2\",37.48333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 11,
		 "de_of_origin\",37],PARAMETER[\"central_meridian\",-81],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_easting\",1968500],PARAMETER[\"false_no");
    add_srs_wkt (p, 13,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"26862\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 14, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26863, "epsg", 26863,
		      "NAD83(NSRS2007) / Maine East (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000.0000000001 +y_0=0 +ellps=GRS80 +towgs84=");
    add_proj4text (p, 2, "0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine East (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 9,
		 "cator\"],PARAMETER[\"latitude_of_origin\",43.66666666666");
    add_srs_wkt (p, 10,
		 "666],PARAMETER[\"central_meridian\",-68.5],PARAMETER[\"s");
    add_srs_wkt (p, 11,
		 "cale_factor\",0.9999],PARAMETER[\"false_easting\",984250");
    add_srs_wkt (p, 12,
		 ".0000000002],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"26863\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26864, "epsg", 26864,
		      "NAD83(NSRS2007) / Maine West (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +t");
    add_proj4text (p, 2, "owgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Maine West (ftUS)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Referen");
    add_srs_wkt (p, 2,
		 "ce_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.25722");
    add_srs_wkt (p, 3,
		 "2101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 8,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 9,
		 "cator\"],PARAMETER[\"latitude_of_origin\",42.83333333333");
    add_srs_wkt (p, 10,
		 "334],PARAMETER[\"central_meridian\",-70.16666666666667],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"scale_factor\",0.999966667],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",2952750],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 13,
		 "HORITY[\"EPSG\",\"26864\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 14, "ORTH]]");
    p = add_epsg_def (first, last, 26865, "epsg", 26865,
		      "NAD83(NSRS2007) / Minnesota North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000.00");
    add_proj4text (p, 2,
		   "00101599 +y_0=99999.99998983997 +ellps=GRS80 +towgs84=0,");
    add_proj4text (p, 3, "0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota North (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "48.63333333333333],PARAMETER[\"standard_parallel_2\",47.");
    add_srs_wkt (p, 11,
		 "03333333333333],PARAMETER[\"latitude_of_origin\",46.5],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"central_meridian\",-93.1],PARAMETER[\"false_e");
    add_srs_wkt (p, 13,
		 "asting\",2624666.6667],PARAMETER[\"false_northing\",3280");
    add_srs_wkt (p, 14,
		 "83.3333],AUTHORITY[\"EPSG\",\"26865\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 15, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26866, "epsg", 26866,
		      "NAD83(NSRS2007) / Minnesota Central (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000.0000101599 +y_0=99999.999989");
    add_proj4text (p, 2,
		   "83997 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota Central (ftUS)\",GE");
    add_srs_wkt (p, 1,
		 "OGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_");
    add_srs_wkt (p, 2,
		 "Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 3,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 5,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 8,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 9,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 10,
		 "\",47.05],PARAMETER[\"standard_parallel_2\",45.616666666");
    add_srs_wkt (p, 11,
		 "66667],PARAMETER[\"latitude_of_origin\",45],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "central_meridian\",-94.25],PARAMETER[\"false_easting\",2");
    add_srs_wkt (p, 13,
		 "624666.6667],PARAMETER[\"false_northing\",328083.3333],A");
    add_srs_wkt (p, 14,
		 "UTHORITY[\"EPSG\",\"26866\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 15, ",NORTH]]");
    p = add_epsg_def (first, last, 26867, "epsg", 26867,
		      "NAD83(NSRS2007) / Minnesota South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000.0000101599 +y_0=999");
    add_proj4text (p, 2,
		   "99.99998983997 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +unit");
    add_proj4text (p, 3, "s=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Minnesota South (ftUS)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Re");
    add_srs_wkt (p, 2,
		 "ference_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.");
    add_srs_wkt (p, 3,
		 "257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 5,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 6,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 7,
		 "PSG\",\"4759\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 8,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 9,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 10,
		 "45.21666666666667],PARAMETER[\"standard_parallel_2\",43.");
    add_srs_wkt (p, 11,
		 "78333333333333],PARAMETER[\"latitude_of_origin\",43],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"central_meridian\",-94],PARAMETER[\"false_easti");
    add_srs_wkt (p, 13,
		 "ng\",2624666.6667],PARAMETER[\"false_northing\",328083.3");
    add_srs_wkt (p, 14,
		 "333],AUTHORITY[\"EPSG\",\"26867\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 15, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26868, "epsg", 26868,
		      "NAD83(NSRS2007) / Nebraska (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000.00001016 +y_0=0 +ellps=GRS80 +tow");
    add_proj4text (p, 2, "gs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / Nebraska (ftUS)\",GEOGCS[\"NA");
    add_srs_wkt (p, 1,
		 "D83(NSRS2007)\",DATUM[\"NAD83_National_Spatial_Reference");
    add_srs_wkt (p, 2,
		 "_System_2007\",SPHEROID[\"GRS 1980\",6378137,298.2572221");
    add_srs_wkt (p, 3,
		 "01,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 7,
		 "\"4759\"]],UNIT[\"US survey foot\",0.3048006096012192,AU");
    add_srs_wkt (p, 8,
		 "THORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conform");
    add_srs_wkt (p, 9,
		 "al_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",43],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"standard_parallel_2\",40],PARAMETER[\"latitude");
    add_srs_wkt (p, 11,
		 "_of_origin\",39.83333333333334],PARAMETER[\"central_meri");
    add_srs_wkt (p, 12,
		 "dian\",-100],PARAMETER[\"false_easting\",1640416.6667],P");
    add_srs_wkt (p, 13,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2686");
    add_srs_wkt (p, 14, "8\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26869, "epsg", 26869,
		      "NAD83(NSRS2007) / West Virginia North (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=600000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0");
    add_proj4text (p, 2, " +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / West Virginia North (ftUS)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatia");
    add_srs_wkt (p, 2,
		 "l_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 3,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060");
    add_srs_wkt (p, 8,
		 "96012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 9,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 10,
		 "_1\",40.25],PARAMETER[\"standard_parallel_2\",39],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"latitude_of_origin\",38.5],PARAMETER[\"central_mer");
    add_srs_wkt (p, 12,
		 "idian\",-79.5],PARAMETER[\"false_easting\",1968500],PARA");
    add_srs_wkt (p, 13,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26869\"");
    add_srs_wkt (p, 14, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26870, "epsg", 26870,
		      "NAD83(NSRS2007) / West Virginia South (ftUS)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +towgs84=0,0,0,0,0,0,0 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(NSRS2007) / West Virginia South (ftUS)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"NAD83(NSRS2007)\",DATUM[\"NAD83_National_Spatia");
    add_srs_wkt (p, 2,
		 "l_Reference_System_2007\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 3,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6759\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 5,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 6,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4759\"]],UNIT[\"US survey foot\",0.30480060");
    add_srs_wkt (p, 8,
		 "96012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamb");
    add_srs_wkt (p, 9,
		 "ert_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 10,
		 "_1\",38.88333333333333],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 11,
		 ",37.48333333333333],PARAMETER[\"latitude_of_origin\",37]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"central_meridian\",-81],PARAMETER[\"false_e");
    add_srs_wkt (p, 13,
		 "asting\",1968500],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 14,
		 "ITY[\"EPSG\",\"26870\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 15, "H]]");
    p = add_epsg_def (first, last, 26891, "epsg", 26891,
		      "NAD83(CSRS) / MTM zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-82.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 11\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-82.5],PARAMETER[\"scale_factor\",0.9999],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",304800],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"26891\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26892, "epsg", 26892,
		      "NAD83(CSRS) / MTM zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-81 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 12\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-81],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"26892\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26893, "epsg", 26893,
		      "NAD83(CSRS) / MTM zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-84 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 13\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-84],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"26893\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26894, "epsg", 26894,
		      "NAD83(CSRS) / MTM zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-87 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 14\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-87],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"26894\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26895, "epsg", 26895,
		      "NAD83(CSRS) / MTM zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-90 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 15\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-90],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"26895\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26896, "epsg", 26896,
		      "NAD83(CSRS) / MTM zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-93 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 16\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-93],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"26896\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26897, "epsg", 26897,
		      "NAD83(CSRS) / MTM zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-96 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 17\",GEOGCS[\"NAD83(CSRS");
    add_srs_wkt (p, 1,
		 ")\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SP");
    add_srs_wkt (p, 2,
		 "HEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",-96],PARAMETER[\"scale_factor\",0.9999],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",304800],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"26897\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 26898, "epsg", 26898,
		      "NAD83(CSRS) / MTM zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-53 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 1\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-53],PARAMETER[\"scale_factor\",0.9999],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",304800],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"26898\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 26899, "epsg", 26899,
		      "NAD83(CSRS) / MTM zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-56 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83(CSRS) / MTM zone 2\",GEOGCS[\"NAD83(CSRS)");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"NAD83_Canadian_Spatial_Reference_System\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7019\"]],AUTHORITY[\"EPSG\",\"6140\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4617\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",0],PARAMETER[\"central_meri");
    add_srs_wkt (p, 9,
		 "dian\",-56],PARAMETER[\"scale_factor\",0.9999],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",304800],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"26899\"],AXIS[\"E(X)\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 26901, "epsg", 26901, "NAD83 / UTM zone 1N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 1N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-177],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26901\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26902, "epsg", 26902, "NAD83 / UTM zone 2N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 2N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-171],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26902\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26903, "epsg", 26903, "NAD83 / UTM zone 3N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 3N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-165],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26903\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26904, "epsg", 26904, "NAD83 / UTM zone 4N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 4N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-159],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26904\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26905, "epsg", 26905, "NAD83 / UTM zone 5N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 5N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-153],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26905\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26906, "epsg", 26906, "NAD83 / UTM zone 6N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 6N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-147],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26906\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26907, "epsg", 26907, "NAD83 / UTM zone 7N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 7N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-141],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26907\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26908, "epsg", 26908, "NAD83 / UTM zone 8N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 8N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-135],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26908\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26909, "epsg", 26909, "NAD83 / UTM zone 9N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +ellps=GRS80 +datum=NAD83 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 9N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-129],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26909\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26910, "epsg", 26910,
		      "NAD83 / UTM zone 10N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 10N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-123],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26910\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26911, "epsg", 26911,
		      "NAD83 / UTM zone 11N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 11N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-117],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26911\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26912, "epsg", 26912,
		      "NAD83 / UTM zone 12N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 12N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-111],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26912\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26913, "epsg", 26913,
		      "NAD83 / UTM zone 13N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 13N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-105],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26913\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26914, "epsg", 26914,
		      "NAD83 / UTM zone 14N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 14N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-99],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26914\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26915, "epsg", 26915,
		      "NAD83 / UTM zone 15N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 15N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-93],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26915\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26916, "epsg", 26916,
		      "NAD83 / UTM zone 16N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 16N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-87],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26916\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26917, "epsg", 26917,
		      "NAD83 / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 17N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-81],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26917\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26918, "epsg", 26918,
		      "NAD83 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 18N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-75],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26918\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26919, "epsg", 26919,
		      "NAD83 / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 19N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-69],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26919\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26920, "epsg", 26920,
		      "NAD83 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 20N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-63],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26920\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26921, "epsg", 26921,
		      "NAD83 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 21N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-57],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26921\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26922, "epsg", 26922,
		      "NAD83 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 22N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-51],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26922\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26923, "epsg", 26923,
		      "NAD83 / UTM zone 23N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +ellps=GRS80 +datum=NAD83 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / UTM zone 23N\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-45],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "26923\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 26929, "epsg", 26929,
		      "NAD83 / Alabama East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30.5 +lon_0=-85.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9996 +x_0=200000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alabama East\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",30.5],PARAMETER[\"central_meridian\",-85.8333333333");
    add_srs_wkt (p, 9,
		 "3333],PARAMETER[\"scale_factor\",0.99996],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",200000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 11,
		 "THORITY[\"EPSG\",\"26929\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 26930, "epsg", 26930,
		      "NAD83 / Alabama West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-87.5 +k=0.999933333 +x_0=6");
    add_proj4text (p, 1,
		   "00000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_proj4text (p, 2, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alabama West\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",30],PARAMETER[\"central_meridian\",-87.5],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.999933333],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",600000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11, "PSG\",\"26930\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26931, "epsg", 26931,
		      "NAD83 / Alaska zone 1");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=57 +lonc=-133.6666666666667 +alpha=32");
    add_proj4text (p, 1,
		   "3.1301023611111 +k=0.9999 +x_0=5000000 +y_0=-5000000 +el");
    add_proj4text (p, 2, "lps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 1\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Hotine_Oblique_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_center\",57],PARAMETER[\"longitude_of_center\",-133.666");
    add_srs_wkt (p, 9,
		 "6666666667],PARAMETER[\"azimuth\",323.1301023611111],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"rectified_grid_angle\",323.1301023611111],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",5000000],PARAMETER[\"false_northing\",-5000000],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"26931\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 26932, "epsg", 26932,
		      "NAD83 / Alaska zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-142 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 2\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-142],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26932\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26933, "epsg", 26933,
		      "NAD83 / Alaska zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-146 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 3\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-146],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26933\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26934, "epsg", 26934,
		      "NAD83 / Alaska zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-150 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 4\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-150],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26934\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26935, "epsg", 26935,
		      "NAD83 / Alaska zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-154 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 5\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-154],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26935\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26936, "epsg", 26936,
		      "NAD83 / Alaska zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-158 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 6\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-158],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26936\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26937, "epsg", 26937,
		      "NAD83 / Alaska zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-162 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 7\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-162],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26937\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_22 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 26938, "epsg", 26938,
		      "NAD83 / Alaska zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-166 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 8\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-166],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26938\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26939, "epsg", 26939,
		      "NAD83 / Alaska zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=54 +lon_0=-170 +k=0.9999 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 9\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",54],PARAMETER[\"central_meridian\",-170],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 10,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"26939\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26940, "epsg", 26940,
		      "NAD83 / Alaska zone 10");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=53.83333333333334 +lat_2=51.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=51 +lon_0=-176 +x_0=1000000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Alaska zone 10\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",53.83333333333334],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",51.83333333333334],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",51],PARAMETER[\"central_meridian\",-176],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",1000000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"26940\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26941, "epsg", 26941,
		      "NAD83 / California zone 1");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.66666666666666 +lat_2=40 +lat_0=39.3");
    add_proj4text (p, 1,
		   "3333333333334 +lon_0=-122 +x_0=2000000 +y_0=500000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 1\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",41.66666666666666],PARAMETER[\"stand");
    add_srs_wkt (p, 9,
		 "ard_parallel_2\",40],PARAMETER[\"latitude_of_origin\",39");
    add_srs_wkt (p, 10,
		 ".33333333333334],PARAMETER[\"central_meridian\",-122],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",2000000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",500000],AUTHORITY[\"EPSG\",\"26941\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26942, "epsg", 26942,
		      "NAD83 / California zone 2");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.83333333333334 +lat_2=38.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=37.66666666666666 +lon_0=-122 +x_0=2000000 +y");
    add_proj4text (p, 2,
		   "_0=500000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 2\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",39.83333333333334],PARAMETER[\"stand");
    add_srs_wkt (p, 9,
		 "ard_parallel_2\",38.33333333333334],PARAMETER[\"latitude");
    add_srs_wkt (p, 10,
		 "_of_origin\",37.66666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 11,
		 "dian\",-122],PARAMETER[\"false_easting\",2000000],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"2694");
    add_srs_wkt (p, 13, "2\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26943, "epsg", 26943,
		      "NAD83 / California zone 3");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.5 +lon_0=-120.5 +x_0=2000000 +y_0=500000 +");
    add_proj4text (p, 2, "ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 3\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",38.43333333333333],PARAMETER[\"stand");
    add_srs_wkt (p, 9,
		 "ard_parallel_2\",37.06666666666667],PARAMETER[\"latitude");
    add_srs_wkt (p, 10,
		 "_of_origin\",36.5],PARAMETER[\"central_meridian\",-120.5");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_easting\",2000000],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_northing\",500000],AUTHORITY[\"EPSG\",\"26943\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26944, "epsg", 26944,
		      "NAD83 / California zone 4");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.25 +lat_2=36 +lat_0=35.3333333333333");
    add_proj4text (p, 1,
		   "4 +lon_0=-119 +x_0=2000000 +y_0=500000 +ellps=GRS80 +dat");
    add_proj4text (p, 2, "um=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 4\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",37.25],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_2\",36],PARAMETER[\"latitude_of_origin\",35.33333333333");
    add_srs_wkt (p, 10,
		 "334],PARAMETER[\"central_meridian\",-119],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_easting\",2000000],PARAMETER[\"false_northing\",5000");
    add_srs_wkt (p, 12,
		 "00],AUTHORITY[\"EPSG\",\"26944\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26945, "epsg", 26945,
		      "NAD83 / California zone 5");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.46666666666667 +lat_2=34.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.5 +lon_0=-118 +x_0=2000000 +y_0=500000 +el");
    add_proj4text (p, 2, "lps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 5\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",35.46666666666667],PARAMETER[\"stand");
    add_srs_wkt (p, 9,
		 "ard_parallel_2\",34.03333333333333],PARAMETER[\"latitude");
    add_srs_wkt (p, 10,
		 "_of_origin\",33.5],PARAMETER[\"central_meridian\",-118],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",2000000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",500000],AUTHORITY[\"EPSG\",\"26945\"],AXIS[\"X");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26946, "epsg", 26946,
		      "NAD83 / California zone 6");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.88333333333333 +lat_2=32.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=32.16666666666666 +lon_0=-116.25 +x_0=2000000");
    add_proj4text (p, 2,
		   " +y_0=500000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_proj4text (p, 3, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / California zone 6\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",33.88333333333333],PARAMETER[\"stand");
    add_srs_wkt (p, 9,
		 "ard_parallel_2\",32.78333333333333],PARAMETER[\"latitude");
    add_srs_wkt (p, 10,
		 "_of_origin\",32.16666666666666],PARAMETER[\"central_meri");
    add_srs_wkt (p, 11,
		 "dian\",-116.25],PARAMETER[\"false_easting\",2000000],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 13, "6946\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26948, "epsg", 26948,
		      "NAD83 / Arizona East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-110.1666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arizona East\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",31],PARAMETER[\"central_meridian\",-110.16666666666");
    add_srs_wkt (p, 9,
		 "67],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",213360],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"26948\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 26949, "epsg", 26949,
		      "NAD83 / Arizona Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-111.9166666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=213360 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arizona Central\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",31],PARAMETER[\"central_meridian\",-111.9166666");
    add_srs_wkt (p, 9,
		 "666667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",213360],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"26949\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26950, "epsg", 26950,
		      "NAD83 / Arizona West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-113.75 +k=0.999933333 +x_0");
    add_proj4text (p, 1,
		   "=213360 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arizona West\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",31],PARAMETER[\"central_meridian\",-113.75],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.999933333],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",213360],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"26950\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26951, "epsg", 26951,
		      "NAD83 / Arkansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.23333333333333 +lat_2=34.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=34.33333333333334 +lon_0=-92 +x_0=400000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arkansas North\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",36.23333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",34.93333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",34.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-92],PARAMETER[\"false_easting\",400000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"26951\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26952, "epsg", 26952,
		      "NAD83 / Arkansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.76666666666667 +lat_2=33.3 +lat_0=32");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-92 +x_0=400000 +y_0=400000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Arkansas South\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",34.76666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",33.3],PARAMETER[\"latitude_of_origin\",32.6");
    add_srs_wkt (p, 10,
		 "6666666666666],PARAMETER[\"central_meridian\",-92],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",400000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",400000],AUTHORITY[\"EPSG\",\"26952\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26953, "epsg", 26953,
		      "NAD83 / Colorado North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.78333333333333 +lat_2=39.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=39.33333333333334 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "289 +y_0=304800.6096 +ellps=GRS80 +datum=NAD83 +units=m ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Colorado North\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",40.78333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",39.71666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",39.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-105.5],PARAMETER[\"false_easting\",914401.8289],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",304800.6096],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"26953\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26954, "epsg", 26954,
		      "NAD83 / Colorado Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.75 +lat_2=38.45 +lat_0=37.8333333333");
    add_proj4text (p, 1,
		   "3334 +lon_0=-105.5 +x_0=914401.8289 +y_0=304800.6096 +el");
    add_proj4text (p, 2, "lps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Colorado Central\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 8,
		 "ndard_parallel_1\",39.75],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 9,
		 "2\",38.45],PARAMETER[\"latitude_of_origin\",37.833333333");
    add_srs_wkt (p, 10,
		 "33334],PARAMETER[\"central_meridian\",-105.5],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",914401.8289],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",304800.6096],AUTHORITY[\"EPSG\",\"26954\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26955, "epsg", 26955,
		      "NAD83 / Colorado South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.43333333333333 +lat_2=37.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.66666666666666 +lon_0=-105.5 +x_0=914401.8");
    add_proj4text (p, 2,
		   "289 +y_0=304800.6096 +ellps=GRS80 +datum=NAD83 +units=m ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Colorado South\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",38.43333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",37.23333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",36.66666666666666],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-105.5],PARAMETER[\"false_easting\",914401.8289],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",304800.6096],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"26955\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26956, "epsg", 26956, "NAD83 / Connecticut");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.86666666666667 +lat_2=41.2 +lat_0=40");
    add_proj4text (p, 1,
		   ".83333333333334 +lon_0=-72.75 +x_0=304800.6096 +y_0=1524");
    add_proj4text (p, 2, "00.3048 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Connecticut\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",41.86666666666667],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_2\",41.2],PARAMETER[\"latitude_of_origin\",40.833");
    add_srs_wkt (p, 10,
		 "33333333334],PARAMETER[\"central_meridian\",-72.75],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",304800.6096],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",152400.3048],AUTHORITY[\"EPSG\",\"26956\"],AXI");
    add_srs_wkt (p, 13, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26957, "epsg", 26957, "NAD83 / Delaware");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38 +lon_0=-75.41666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "995 +x_0=200000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Delaware\",GEOGCS[\"NAD83\",DATUM[\"Nor");
    add_srs_wkt (p, 1,
		 "th_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 2,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",38],PARAMETER[\"central_meridian\",-75.41666666666667],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",0.999995],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",200000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"26957\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 26958, "epsg", 26958,
		      "NAD83 / Florida East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-81 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Florida East\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",24.33333333333333],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "81],PARAMETER[\"scale_factor\",0.999941177],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",200000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"26958\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26959, "epsg", 26959,
		      "NAD83 / Florida West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.33333333333333 +lon_0=-82 +k=0.999");
    add_proj4text (p, 1,
		   "941177 +x_0=200000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Florida West\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",24.33333333333333],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "82],PARAMETER[\"scale_factor\",0.999941177],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",200000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"26959\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26960, "epsg", 26960,
		      "NAD83 / Florida North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.75 +lat_2=29.58333333333333 +lat_0=2");
    add_proj4text (p, 1,
		   "9 +lon_0=-84.5 +x_0=600000 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Florida North\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standar");
    add_srs_wkt (p, 8,
		 "d_parallel_1\",30.75],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 9,
		 "29.58333333333333],PARAMETER[\"latitude_of_origin\",29],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"central_meridian\",-84.5],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",600000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"26960\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 13, "H]]");
    p = add_epsg_def (first, last, 26961, "epsg", 26961,
		      "NAD83 / Hawaii zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=18.83333333333333 +lon_0=-155.5 +k=0.");
    add_proj4text (p, 1,
		   "999966667 +x_0=500000 +y_0=0 +ellps=GRS80 +datum=NAD83 +");
    add_proj4text (p, 2, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Hawaii zone 1\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",18.83333333333333],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-155.5],PARAMETER[\"scale_factor\",0.999966667],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"26961\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26962, "epsg", 26962,
		      "NAD83 / Hawaii zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=20.33333333333333 +lon_0=-156.6666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=500000 +y_0=0 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Hawaii zone 2\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",20.33333333333333],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-156.6666666666667],PARAMETER[\"scale_factor\",0.9999666");
    add_srs_wkt (p, 10,
		 "67],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"26962\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26963, "epsg", 26963,
		      "NAD83 / Hawaii zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.16666666666667 +lon_0=-158 +k=0.99");
    add_proj4text (p, 1,
		   "999 +x_0=500000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Hawaii zone 3\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",21.16666666666667],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-158],PARAMETER[\"scale_factor\",0.99999],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",500000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 11,
		 "THORITY[\"EPSG\",\"26963\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 26964, "epsg", 26964,
		      "NAD83 / Hawaii zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.83333333333333 +lon_0=-159.5 +k=0.");
    add_proj4text (p, 1,
		   "99999 +x_0=500000 +y_0=0 +ellps=GRS80 +datum=NAD83 +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Hawaii zone 4\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",21.83333333333333],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-159.5],PARAMETER[\"scale_factor\",0.99999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"26964\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 26965, "epsg", 26965,
		      "NAD83 / Hawaii zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=21.66666666666667 +lon_0=-160.1666666");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=500000 +y_0=0 +ellps=GRS80 +datum=NAD83");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Hawaii zone 5\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",21.66666666666667],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-160.1666666666667],PARAMETER[\"scale_factor\",1],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 11,
		 "\",0],AUTHORITY[\"EPSG\",\"26965\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26966, "epsg", 26966,
		      "NAD83 / Georgia East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-82.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=200000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Georgia East\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",30],PARAMETER[\"central_meridian\",-82.166666666666");
    add_srs_wkt (p, 9,
		 "67],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",200000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"26966\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 26967, "epsg", 26967,
		      "NAD83 / Georgia West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=30 +lon_0=-84.16666666666667 +k=0.999");
    add_proj4text (p, 1,
		   "9 +x_0=700000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Georgia West\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",30],PARAMETER[\"central_meridian\",-84.166666666666");
    add_srs_wkt (p, 9,
		 "67],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",700000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"26967\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 26968, "epsg", 26968, "NAD83 / Idaho East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-112.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999473679999999 +x_0=200000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Idaho East\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",41.66666666666666],PARAMETER[\"central_meridian\",-11");
    add_srs_wkt (p, 9,
		 "2.1666666666667],PARAMETER[\"scale_factor\",0.999947368]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",200000],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"26968\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26969, "epsg", 26969,
		      "NAD83 / Idaho Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-114 +k=0.99");
    add_proj4text (p, 1,
		   "99473679999999 +x_0=500000 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Idaho Central\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",41.66666666666666],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-114],PARAMETER[\"scale_factor\",0.999947368],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"26969\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26970, "epsg", 26970, "NAD83 / Idaho West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.66666666666666 +lon_0=-115.75 +k=0");
    add_proj4text (p, 1,
		   ".999933333 +x_0=800000 +y_0=0 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Idaho West\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",41.66666666666666],PARAMETER[\"central_meridian\",-11");
    add_srs_wkt (p, 9,
		 "5.75],PARAMETER[\"scale_factor\",0.999933333],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",800000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"26970\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26971, "epsg", 26971,
		      "NAD83 / Illinois East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-88.33333333");
    add_proj4text (p, 1,
		   "333333 +k=0.9999749999999999 +x_0=300000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Illinois East\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",36.66666666666666],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-88.33333333333333],PARAMETER[\"scale_factor\",0.999975]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",300000],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"26971\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26972, "epsg", 26972,
		      "NAD83 / Illinois West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.66666666666666 +lon_0=-90.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=700000 +y_0=0 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Illinois West\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",36.66666666666666],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-90.16666666666667],PARAMETER[\"scale_factor\",0.9999411");
    add_srs_wkt (p, 10,
		 "77],PARAMETER[\"false_easting\",700000],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"26972\"],AXIS[\"X\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26973, "epsg", 26973,
		      "NAD83 / Indiana East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-85.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=100000 +y_0=250000 +ellps=GRS80 +datum=NAD");
    add_proj4text (p, 2, "83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Indiana East\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",37.5],PARAMETER[\"central_meridian\",-85.6666666666");
    add_srs_wkt (p, 9,
		 "6667],PARAMETER[\"scale_factor\",0.999966667],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",100000],PARAMETER[\"false_northing\",2");
    add_srs_wkt (p, 11,
		 "50000],AUTHORITY[\"EPSG\",\"26973\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26974, "epsg", 26974,
		      "NAD83 / Indiana West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=37.5 +lon_0=-87.08333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=900000 +y_0=250000 +ellps=GRS80 +datum=NAD");
    add_proj4text (p, 2, "83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Indiana West\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",37.5],PARAMETER[\"central_meridian\",-87.0833333333");
    add_srs_wkt (p, 9,
		 "3333],PARAMETER[\"scale_factor\",0.999966667],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"false_easting\",900000],PARAMETER[\"false_northing\",2");
    add_srs_wkt (p, 11,
		 "50000],AUTHORITY[\"EPSG\",\"26974\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26975, "epsg", 26975, "NAD83 / Iowa North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.26666666666667 +lat_2=42.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41.5 +lon_0=-93.5 +x_0=1500000 +y_0=1000000 +");
    add_proj4text (p, 2, "ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Iowa North\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 8,
		 "arallel_1\",43.26666666666667],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",42.06666666666667],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",41.5],PARAMETER[\"central_meridian\",-93.5],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",1500000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",1000000],AUTHORITY[\"EPSG\",\"26975\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26976, "epsg", 26976, "NAD83 / Iowa South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.61666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40 +lon_0=-93.5 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Iowa South\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 8,
		 "arallel_1\",41.78333333333333],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",40.61666666666667],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",40],PARAMETER[\"central_meridian\",-93.5],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"26976\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26977, "epsg", 26977,
		      "NAD83 / Kansas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.78333333333333 +lat_2=38.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=38.33333333333334 +lon_0=-98 +x_0=400000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kansas North\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",39.78333333333333],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_2\",38.71666666666667],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 10,
		 "igin\",38.33333333333334],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-98],PARAMETER[\"false_easting\",400000],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"26977\"],AXIS[\"X");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26978, "epsg", 26978,
		      "NAD83 / Kansas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.56666666666667 +lat_2=37.26666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.66666666666666 +lon_0=-98.5 +x_0=400000 +y");
    add_proj4text (p, 2,
		   "_0=400000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kansas South\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",38.56666666666667],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_2\",37.26666666666667],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 10,
		 "igin\",36.66666666666666],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-98.5],PARAMETER[\"false_easting\",400000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",400000],AUTHORITY[\"EPSG\",\"26978\"],A");
    add_srs_wkt (p, 13, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26979, "epsg", 26979,
		      "NAD83 / Kentucky North (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=37.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=37.5 +lon_0=-84.25 +x_0=500000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kentucky North (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"G");
    add_srs_wkt (p, 2,
		 "RS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"701");
    add_srs_wkt (p, 3,
		 "9\"]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"standard_parallel_1\",37.96666666666667],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"standard_parallel_2\",37.96666666666667],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"latitude_of_origin\",37.5],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 11,
		 "n\",-84.25],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26979\"],AX");
    add_srs_wkt (p, 13, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26980, "epsg", 26980,
		      "NAD83 / Kentucky South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.93333333333333 +lat_2=36.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=36.33333333333334 +lon_0=-85.75 +x_0=500000 +");
    add_proj4text (p, 2,
		   "y_0=500000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Kentucky South\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",37.93333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",36.73333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",36.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-85.75],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"26980\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26981, "epsg", 26981,
		      "NAD83 / Louisiana North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.66666666666666 +lat_2=31.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=30.5 +lon_0=-92.5 +x_0=1000000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Louisiana North\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",32.66666666666666],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_2\",31.16666666666667],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 10,
		 "f_origin\",30.5],PARAMETER[\"central_meridian\",-92.5],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",1000000],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"26981\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26982, "epsg", 26982,
		      "NAD83 / Louisiana South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.7 +lat_2=29.3 +lat_0=28.5 +lon_0=-91");
    add_proj4text (p, 1,
		   ".33333333333333 +x_0=1000000 +y_0=0 +ellps=GRS80 +datum=");
    add_proj4text (p, 2, "NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Louisiana South\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",30.7],PARAMETER[\"standard_parallel_2\"");
    add_srs_wkt (p, 9,
		 ",29.3],PARAMETER[\"latitude_of_origin\",28.5],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"central_meridian\",-91.33333333333333],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_easting\",1000000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 12,
		 "THORITY[\"EPSG\",\"26982\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 13, "NORTH]]");
    p = add_epsg_def (first, last, 26983, "epsg", 26983, "NAD83 / Maine East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=43.66666666666666 +lon_0=-68.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=300000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine East\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",43.66666666666666],PARAMETER[\"central_meridian\",-68");
    add_srs_wkt (p, 9,
		 ".5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",300000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"26983\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 26984, "epsg", 26984, "NAD83 / Maine West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.83333333333334 +lon_0=-70.16666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999966667 +x_0=900000 +y_0=0 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maine West\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",42.83333333333334],PARAMETER[\"central_meridian\",-70");
    add_srs_wkt (p, 9,
		 ".16666666666667],PARAMETER[\"scale_factor\",0.999966667]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_easting\",900000],PARAMETER[\"false_n");
    add_srs_wkt (p, 11,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"26984\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26985, "epsg", 26985, "NAD83 / Maryland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.45 +lat_2=38.3 +lat_0=37.66666666666");
    add_proj4text (p, 1,
		   "666 +lon_0=-77 +x_0=400000 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Maryland\",GEOGCS[\"NAD83\",DATUM[\"Nor");
    add_srs_wkt (p, 1,
		 "th_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 2,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 8,
		 "rallel_1\",39.45],PARAMETER[\"standard_parallel_2\",38.3");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",37.66666666666666],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"central_meridian\",-77],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",400000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 12,
		 "\"EPSG\",\"26985\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26986, "epsg", 26986,
		      "NAD83 / Massachusetts Mainland");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.68333333333333 +lat_2=41.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=41 +lon_0=-71.5 +x_0=200000 +y_0=750000 +ellp");
    add_proj4text (p, 2, "s=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Massachusetts Mainland\",GEOGCS[\"NAD83");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 19");
    add_srs_wkt (p, 2,
		 "80\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",42.68333333333333],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "standard_parallel_2\",41.71666666666667],PARAMETER[\"lat");
    add_srs_wkt (p, 10,
		 "itude_of_origin\",41],PARAMETER[\"central_meridian\",-71");
    add_srs_wkt (p, 11,
		 ".5],PARAMETER[\"false_easting\",200000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",750000],AUTHORITY[\"EPSG\",\"26986\"],AXIS[");
    add_srs_wkt (p, 13, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26987, "epsg", 26987,
		      "NAD83 / Massachusetts Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.48333333333333 +lat_2=41.28333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=41 +lon_0=-70.5 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Massachusetts Island\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",41.48333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_2\",41.28333333333333],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",41],PARAMETER[\"central_meridian\",-70.5]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",500000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"26987\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26988, "epsg", 26988,
		      "NAD83 / Michigan North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.08333333333334 +lat_2=45.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=44.78333333333333 +lon_0=-87 +x_0=8000000 +y_");
    add_proj4text (p, 2, "0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Michigan North\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",47.08333333333334],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",45.48333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",44.78333333333333],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-87],PARAMETER[\"false_easting\",8000000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"26988\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26989, "epsg", 26989,
		      "NAD83 / Michigan Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.7 +lat_2=44.18333333333333 +lat_0=43");
    add_proj4text (p, 1,
		   ".31666666666667 +lon_0=-84.36666666666666 +x_0=6000000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Michigan Central\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 8,
		 "ndard_parallel_1\",45.7],PARAMETER[\"standard_parallel_2");
    add_srs_wkt (p, 9,
		 "\",44.18333333333333],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 10,
		 "3.31666666666667],PARAMETER[\"central_meridian\",-84.366");
    add_srs_wkt (p, 11,
		 "66666666666],PARAMETER[\"false_easting\",6000000],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"26989\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26990, "epsg", 26990,
		      "NAD83 / Michigan South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43.66666666666666 +lat_2=42.1 +lat_0=41");
    add_proj4text (p, 1,
		   ".5 +lon_0=-84.36666666666666 +x_0=4000000 +y_0=0 +ellps=");
    add_proj4text (p, 2, "GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Michigan South\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",43.66666666666666],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",42.1],PARAMETER[\"latitude_of_origin\",41.5");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"central_meridian\",-84.36666666666666],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",4000000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"26990\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 13, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26991, "epsg", 26991,
		      "NAD83 / Minnesota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.63333333333333 +lat_2=47.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=46.5 +lon_0=-93.09999999999999 +x_0=800000 +y");
    add_proj4text (p, 2,
		   "_0=100000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota North\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",48.63333333333333],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_2\",47.03333333333333],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 10,
		 "f_origin\",46.5],PARAMETER[\"central_meridian\",-93.1],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",800000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",100000],AUTHORITY[\"EPSG\",\"26991\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26992, "epsg", 26992,
		      "NAD83 / Minnesota Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.05 +lat_2=45.61666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "5 +lon_0=-94.25 +x_0=800000 +y_0=100000 +ellps=GRS80 +da");
    add_proj4text (p, 2, "tum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota Central\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",47.05],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_2\",45.61666666666667],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 10,
		 ",45],PARAMETER[\"central_meridian\",-94.25],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",800000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 12,
		 "000],AUTHORITY[\"EPSG\",\"26992\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 13, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26993, "epsg", 26993,
		      "NAD83 / Minnesota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.21666666666667 +lat_2=43.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43 +lon_0=-94 +x_0=800000 +y_0=100000 +ellps=");
    add_proj4text (p, 2, "GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Minnesota South\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",45.21666666666667],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_2\",43.78333333333333],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 10,
		 "f_origin\",43],PARAMETER[\"central_meridian\",-94],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",800000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",100000],AUTHORITY[\"EPSG\",\"26993\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 13, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26994, "epsg", 26994,
		      "NAD83 / Mississippi East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-88.83333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=300000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Mississippi East\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",29.5],PARAMETER[\"central_meridian\",-88.83333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"scale_factor\",0.99995],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",300000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"26994\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26995, "epsg", 26995,
		      "NAD83 / Mississippi West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=29.5 +lon_0=-90.33333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "9995 +x_0=700000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Mississippi West\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",29.5],PARAMETER[\"central_meridian\",-90.33333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"scale_factor\",0.99995],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",700000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"26995\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26996, "epsg", 26996,
		      "NAD83 / Missouri East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-90.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=250000 +y_0=0 +ellps=GRS80 +datum=NAD83 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Missouri East\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",35.83333333333334],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-90.5],PARAMETER[\"scale_factor\",0.999933333],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",250000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"26996\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 26997, "epsg", 26997,
		      "NAD83 / Missouri Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=35.83333333333334 +lon_0=-92.5 +k=0.9");
    add_proj4text (p, 1,
		   "99933333 +x_0=500000 +y_0=0 +ellps=GRS80 +datum=NAD83 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Missouri Central\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",35.83333333333334],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 9,
		 "n\",-92.5],PARAMETER[\"scale_factor\",0.999933333],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",0],AUTHORITY[\"EPSG\",\"26997\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 12, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 26998, "epsg", 26998,
		      "NAD83 / Missouri West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36.16666666666666 +lon_0=-94.5 +k=0.9");
    add_proj4text (p, 1,
		   "99941177 +x_0=850000 +y_0=0 +ellps=GRS80 +datum=NAD83 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Missouri West\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",36.16666666666666],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-94.5],PARAMETER[\"scale_factor\",0.999941177],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",850000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"26998\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 27037, "epsg", 27037,
		      "Nahrwan 1967 / UTM zone 37N");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nahrwan 1967 / UTM zone 37N\",GEOGCS[\"Nahrwan ");
    add_srs_wkt (p, 1,
		 "1967\",DATUM[\"Nahrwan_1967\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6270\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4270\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",39],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"27037\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 27038, "epsg", 27038,
		      "Nahrwan 1967 / UTM zone 38N");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nahrwan 1967 / UTM zone 38N\",GEOGCS[\"Nahrwan ");
    add_srs_wkt (p, 1,
		 "1967\",DATUM[\"Nahrwan_1967\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6270\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4270\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",45],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"27038\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 27039, "epsg", 27039,
		      "Nahrwan 1967 / UTM zone 39N");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nahrwan 1967 / UTM zone 39N\",GEOGCS[\"Nahrwan ");
    add_srs_wkt (p, 1,
		 "1967\",DATUM[\"Nahrwan_1967\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6270\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4270\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",51],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"27039\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 27040, "epsg", 27040,
		      "Nahrwan 1967 / UTM zone 40N");
    add_proj4text (p, 0, "+proj=utm +zone=40 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nahrwan 1967 / UTM zone 40N\",GEOGCS[\"Nahrwan ");
    add_srs_wkt (p, 1,
		 "1967\",DATUM[\"Nahrwan_1967\",SPHEROID[\"Clarke 1880 (RG");
    add_srs_wkt (p, 2,
		 "S)\",6378249.145,293.465,AUTHORITY[\"EPSG\",\"7012\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6270\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4270\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",57],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"27040\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 27120, "epsg", 27120,
		      "Naparima 1972 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Naparima 1972 / UTM zone 20N\",GEOGCS[\"Naparim");
    add_srs_wkt (p, 1,
		 "a 1972\",DATUM[\"Naparima_1972\",SPHEROID[\"Internationa");
    add_srs_wkt (p, 2,
		 "l 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6271\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"427");
    add_srs_wkt (p, 6,
		 "1\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",-63],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11,
		 "\",\"27120\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",N");
    add_srs_wkt (p, 12, "ORTH]]");
    p = add_epsg_def (first, last, 27200, "epsg", 27200,
		      "NZGD49 / New Zealand Map Grid");
    add_proj4text (p, 0,
		   "+proj=nzmg +lat_0=-41 +lon_0=173 +x_0=2510000 +y_0=60231");
    add_proj4text (p, 1, "50 +ellps=intl +datum=nzgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / New Zealand Map Grid\",GEOGCS[\"NZGD49");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.599");
    add_srs_wkt (p, 4,
		 "3],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 6,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 8,
		 "\"]],PROJECTION[\"New_Zealand_Map_Grid\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",-41],PARAMETER[\"central_meridian\",1");
    add_srs_wkt (p, 10,
		 "73],PARAMETER[\"false_easting\",2510000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",6023150],AUTHORITY[\"EPSG\",\"27200\"],AXI");
    add_srs_wkt (p, 12, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 27205, "epsg", 27205,
		      "NZGD49 / Mount Eden Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-36.87986527777778 +lon_0=174.7643393");
    add_proj4text (p, 1,
		   "611111 +k=0.9999 +x_0=300000 +y_0=700000 +ellps=intl +da");
    add_proj4text (p, 2, "tum=nzgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Mount Eden Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-36.87986527777778],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",174.7643393611111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",0.9999],PARAMETER[\"false_easting\",300000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",700000],AUTHORITY[\"EPSG\",\"27205\"],A");
    add_srs_wkt (p, 13, "XIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27206, "epsg", 27206,
		      "NZGD49 / Bay of Plenty Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-37.76124980555556 +lon_0=176.4661972");
    add_proj4text (p, 1,
		   "5 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=nzgd49");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Bay of Plenty Circuit\",GEOGCS[\"NZGD4");
    add_srs_wkt (p, 1,
		 "9\",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.599");
    add_srs_wkt (p, 4,
		 "3],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 6,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 8,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",-37.76124980555556],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",176.46619725],PARAMETER[\"scale_factor\",1]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",300000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",700000],AUTHORITY[\"EPSG\",\"27206\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27207, "epsg", 27207,
		      "NZGD49 / Poverty Bay Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-38.62470277777778 +lon_0=177.8856362");
    add_proj4text (p, 1,
		   "777778 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Poverty Bay Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-38.62470277777778],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",177.8856362777778],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",700000],AUTHORITY[\"EPSG\",\"27207\"],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27208, "epsg", 27208,
		      "NZGD49 / Hawkes Bay Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-39.65092930555556 +lon_0=176.6736805");
    add_proj4text (p, 1,
		   "277778 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Hawkes Bay Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-39.65092930555556],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",176.6736805277778],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",700000],AUTHORITY[\"EPSG\",\"27208\"],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27209, "epsg", 27209,
		      "NZGD49 / Taranaki Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-39.13575830555556 +lon_0=174.2280117");
    add_proj4text (p, 1,
		   "5 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=nzgd49");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Taranaki Circuit\",GEOGCS[\"NZGD49\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Inte");
    add_srs_wkt (p, 2,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",-39.13575830555556],PARAMETER[\"central_mer");
    add_srs_wkt (p, 10,
		 "idian\",174.22801175],PARAMETER[\"scale_factor\",1],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",300000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 12,
		 "ng\",700000],AUTHORITY[\"EPSG\",\"27209\"],AXIS[\"Northi");
    add_srs_wkt (p, 13, "ng\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27210, "epsg", 27210,
		      "NZGD49 / Tuhirangi Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-39.51247038888889 +lon_0=175.6400368");
    add_proj4text (p, 1,
		   "055556 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Tuhirangi Circuit\",GEOGCS[\"NZGD49\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Int");
    add_srs_wkt (p, 2,
		 "ernational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",-39.51247038888889],PARAMETER[\"central_mer");
    add_srs_wkt (p, 10,
		 "idian\",175.6400368055556],PARAMETER[\"scale_factor\",1]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",300000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",700000],AUTHORITY[\"EPSG\",\"27210\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27211, "epsg", 27211,
		      "NZGD49 / Wanganui Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-40.24194713888889 +lon_0=175.4880996");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Wanganui Circuit\",GEOGCS[\"NZGD49\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Inte");
    add_srs_wkt (p, 2,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",-40.24194713888889],PARAMETER[\"central_mer");
    add_srs_wkt (p, 10,
		 "idian\",175.4880996111111],PARAMETER[\"scale_factor\",1]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",300000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",700000],AUTHORITY[\"EPSG\",\"27211\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27212, "epsg", 27212,
		      "NZGD49 / Wairarapa Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-40.92553263888889 +lon_0=175.6473496");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Wairarapa Circuit\",GEOGCS[\"NZGD49\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Int");
    add_srs_wkt (p, 2,
		 "ernational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",-40.92553263888889],PARAMETER[\"central_mer");
    add_srs_wkt (p, 10,
		 "idian\",175.6473496666667],PARAMETER[\"scale_factor\",1]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",300000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",700000],AUTHORITY[\"EPSG\",\"27212\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27213, "epsg", 27213,
		      "NZGD49 / Wellington Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.30131963888888 +lon_0=174.7766231");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Wellington Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-41.30131963888888],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",174.7766231111111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",700000],AUTHORITY[\"EPSG\",\"27213\"],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27214, "epsg", 27214,
		      "NZGD49 / Collingwood Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-40.71475905555556 +lon_0=172.6720465");
    add_proj4text (p, 1,
		   " +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=nzgd49 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Collingwood Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-40.71475905555556],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",172.6720465],PARAMETER[\"scale_factor\",1],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",300000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",700000],AUTHORITY[\"EPSG\",\"27214\"],AXIS[\"Nort");
    add_srs_wkt (p, 13, "hing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27215, "epsg", 27215,
		      "NZGD49 / Nelson Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.27454472222222 +lon_0=173.2993168");
    add_proj4text (p, 1,
		   "055555 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Nelson Circuit\",GEOGCS[\"NZGD49\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",-41.27454472222222],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",173.2993168055555],PARAMETER[\"scale_factor\",1],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",300000],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",700000],AUTHORITY[\"EPSG\",\"27215\"],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27216, "epsg", 27216,
		      "NZGD49 / Karamea Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.28991152777778 +lon_0=172.1090281");
    add_proj4text (p, 1,
		   "944444 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Karamea Circuit\",GEOGCS[\"NZGD49\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Inter");
    add_srs_wkt (p, 2,
		 "national 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",-41.28991152777778],PARAMETER[\"central_mer");
    add_srs_wkt (p, 10,
		 "idian\",172.1090281944444],PARAMETER[\"scale_factor\",1]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",300000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",700000],AUTHORITY[\"EPSG\",\"27216\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27217, "epsg", 27217,
		      "NZGD49 / Buller Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.81080286111111 +lon_0=171.5812600");
    add_proj4text (p, 1,
		   "555556 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Buller Circuit\",GEOGCS[\"NZGD49\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",-41.81080286111111],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",171.5812600555556],PARAMETER[\"scale_factor\",1],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",300000],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",700000],AUTHORITY[\"EPSG\",\"27217\"],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27218, "epsg", 27218,
		      "NZGD49 / Grey Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-42.33369427777778 +lon_0=171.5497713");
    add_proj4text (p, 1,
		   "055556 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Grey Circuit\",GEOGCS[\"NZGD49\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"427");
    add_srs_wkt (p, 7,
		 "2\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",-42.33369427777778],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",171.5497713055556],PARAMETER[\"scale_factor\",1],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",300000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",700000],AUTHORITY[\"EPSG\",\"27218\"],AXIS[\"Nort");
    add_srs_wkt (p, 13, "hing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27219, "epsg", 27219,
		      "NZGD49 / Amuri Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-42.68911658333333 +lon_0=173.0101333");
    add_proj4text (p, 1,
		   "888889 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Amuri Circuit\",GEOGCS[\"NZGD49\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "72\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",-42.68911658333333],PARAMETER[\"central_merid");
    add_srs_wkt (p, 10,
		 "ian\",173.0101333888889],PARAMETER[\"scale_factor\",1],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",300000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",700000],AUTHORITY[\"EPSG\",\"27219\"],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27220, "epsg", 27220,
		      "NZGD49 / Marlborough Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-41.54448666666666 +lon_0=173.8020741");
    add_proj4text (p, 1,
		   "111111 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Marlborough Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-41.54448666666666],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",173.8020741111111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",700000],AUTHORITY[\"EPSG\",\"27220\"],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27221, "epsg", 27221,
		      "NZGD49 / Hokitika Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-42.88632236111111 +lon_0=170.9799935");
    add_proj4text (p, 1,
		   " +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=nzgd49 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Hokitika Circuit\",GEOGCS[\"NZGD49\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Inte");
    add_srs_wkt (p, 2,
		 "rnational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",-42.88632236111111],PARAMETER[\"central_mer");
    add_srs_wkt (p, 10,
		 "idian\",170.9799935],PARAMETER[\"scale_factor\",1],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",300000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",700000],AUTHORITY[\"EPSG\",\"27221\"],AXIS[\"Northin");
    add_srs_wkt (p, 13, "g\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27222, "epsg", 27222,
		      "NZGD49 / Okarito Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.11012813888889 +lon_0=170.2609258");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Okarito Circuit\",GEOGCS[\"NZGD49\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Inter");
    add_srs_wkt (p, 2,
		 "national 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]");
    add_srs_wkt (p, 3,
		 "],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",-43.11012813888889],PARAMETER[\"central_mer");
    add_srs_wkt (p, 10,
		 "idian\",170.2609258333333],PARAMETER[\"scale_factor\",1]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",300000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",700000],AUTHORITY[\"EPSG\",\"27222\"],AXIS[\"N");
    add_srs_wkt (p, 13, "orthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27223, "epsg", 27223,
		      "NZGD49 / Jacksons Bay Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.97780288888889 +lon_0=168.606267 ");
    add_proj4text (p, 1,
		   "+k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=nzgd49 +");
    add_proj4text (p, 2, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Jacksons Bay Circuit\",GEOGCS[\"NZGD49");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.599");
    add_srs_wkt (p, 4,
		 "3],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 6,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 8,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",-43.97780288888889],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",168.606267],PARAMETER[\"scale_factor\",1],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",300000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",700000],AUTHORITY[\"EPSG\",\"27223\"],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27224, "epsg", 27224,
		      "NZGD49 / Mount Pleasant Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.59063758333333 +lon_0=172.7271935");
    add_proj4text (p, 1,
		   "833333 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Mount Pleasant Circuit\",GEOGCS[\"NZGD");
    add_srs_wkt (p, 1,
		 "49\",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7022\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5");
    add_srs_wkt (p, 4,
		 "993],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",-43.59063758333333],PARAMETER[\"cent");
    add_srs_wkt (p, 10,
		 "ral_meridian\",172.7271935833333],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 11,
		 "or\",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",700000],AUTHORITY[\"EPSG\",\"27224\"],A");
    add_srs_wkt (p, 13, "XIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27225, "epsg", 27225,
		      "NZGD49 / Gawler Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-43.74871155555556 +lon_0=171.3607484");
    add_proj4text (p, 1,
		   "722222 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Gawler Circuit\",GEOGCS[\"NZGD49\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",-43.74871155555556],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",171.3607484722222],PARAMETER[\"scale_factor\",1],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",300000],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",700000],AUTHORITY[\"EPSG\",\"27225\"],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27226, "epsg", 27226,
		      "NZGD49 / Timaru Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-44.40222036111111 +lon_0=171.0572508");
    add_proj4text (p, 1,
		   "333333 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Timaru Circuit\",GEOGCS[\"NZGD49\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Intern");
    add_srs_wkt (p, 2,
		 "ational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",-44.40222036111111],PARAMETER[\"central_meri");
    add_srs_wkt (p, 10,
		 "dian\",171.0572508333333],PARAMETER[\"scale_factor\",1],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",300000],PARAMETER[\"false_no");
    add_srs_wkt (p, 12,
		 "rthing\",700000],AUTHORITY[\"EPSG\",\"27226\"],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27227, "epsg", 27227,
		      "NZGD49 / Lindis Peak Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-44.73526797222222 +lon_0=169.4677550");
    add_proj4text (p, 1,
		   "833333 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Lindis Peak Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-44.73526797222222],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",169.4677550833333],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",700000],AUTHORITY[\"EPSG\",\"27227\"],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27228, "epsg", 27228,
		      "NZGD49 / Mount Nicholas Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.13290258333333 +lon_0=168.3986411");
    add_proj4text (p, 1,
		   "944444 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Mount Nicholas Circuit\",GEOGCS[\"NZGD");
    add_srs_wkt (p, 1,
		 "49\",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7022\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5");
    add_srs_wkt (p, 4,
		 "993],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 5,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 6,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 9,
		 "atitude_of_origin\",-45.13290258333333],PARAMETER[\"cent");
    add_srs_wkt (p, 10,
		 "ral_meridian\",168.3986411944444],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 11,
		 "or\",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",700000],AUTHORITY[\"EPSG\",\"27228\"],A");
    add_srs_wkt (p, 13, "XIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27229, "epsg", 27229,
		      "NZGD49 / Mount York Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.56372616666666 +lon_0=167.7388617");
    add_proj4text (p, 1,
		   "777778 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Mount York Circuit\",GEOGCS[\"NZGD49\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",-45.56372616666666],PARAMETER[\"central_");
    add_srs_wkt (p, 10,
		 "meridian\",167.7388617777778],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 11,
		 ",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",700000],AUTHORITY[\"EPSG\",\"27229\"],AXIS[");
    add_srs_wkt (p, 13, "\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27230, "epsg", 27230,
		      "NZGD49 / Observation Point Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.81619661111111 +lon_0=170.6285951");
    add_proj4text (p, 1,
		   "666667 +k=1 +x_0=300000 +y_0=700000 +ellps=intl +datum=n");
    add_proj4text (p, 2, "zgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Observation Point Circuit\",GEOGCS[\"N");
    add_srs_wkt (p, 1,
		 "ZGD49\",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7022\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-");
    add_srs_wkt (p, 4,
		 "4.5993],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 5,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",-45.81619661111111],PARAMETER[\"cen");
    add_srs_wkt (p, 10,
		 "tral_meridian\",170.6285951666667],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 11,
		 "tor\",1],PARAMETER[\"false_easting\",300000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",700000],AUTHORITY[\"EPSG\",\"27230\"],A");
    add_srs_wkt (p, 13, "XIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27231, "epsg", 27231,
		      "NZGD49 / North Taieri Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-45.86151336111111 +lon_0=170.2825891");
    add_proj4text (p, 1,
		   "111111 +k=0.99996 +x_0=300000 +y_0=700000 +ellps=intl +d");
    add_proj4text (p, 2, "atum=nzgd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / North Taieri Circuit\",GEOGCS[\"NZGD49");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.599");
    add_srs_wkt (p, 4,
		 "3],AUTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 6,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"4272\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 8,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",-45.86151336111111],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",170.2825891111111],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 11,
		 "\",0.99996],PARAMETER[\"false_easting\",300000],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_northing\",700000],AUTHORITY[\"EPSG\",\"27231\"");
    add_srs_wkt (p, 13, "],AXIS[\"Northing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27232, "epsg", 27232,
		      "NZGD49 / Bluff Circuit");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-46.60000961111111 +lon_0=168.342872 ");
    add_proj4text (p, 1,
		   "+k=1 +x_0=300002.66 +y_0=699999.58 +ellps=intl +datum=nz");
    add_proj4text (p, 2, "gd49 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / Bluff Circuit\",GEOGCS[\"NZGD49\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Interna");
    add_srs_wkt (p, 2,
		 "tional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],");
    add_srs_wkt (p, 3,
		 "TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "72\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",-46.60000961111111],PARAMETER[\"central_merid");
    add_srs_wkt (p, 10,
		 "ian\",168.342872],PARAMETER[\"scale_factor\",1],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_easting\",300002.66],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",699999.58],AUTHORITY[\"EPSG\",\"27232\"],AXIS[\"Nort");
    add_srs_wkt (p, 13, "hing\",NORTH],AXIS[\"Easting\",EAST]]");
    p = add_epsg_def (first, last, 27258, "epsg", 27258,
		      "NZGD49 / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=intl +datum=nzgd49 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / UTM zone 58S\",GEOGCS[\"NZGD49\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"427");
    add_srs_wkt (p, 7,
		 "2\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",165],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",10000000],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"27258\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 13, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 27259, "epsg", 27259,
		      "NZGD49 / UTM zone 59S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=intl +datum=nzgd49 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / UTM zone 59S\",GEOGCS[\"NZGD49\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"427");
    add_srs_wkt (p, 7,
		 "2\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",171],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",10000000],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"27259\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 13, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 27260, "epsg", 27260,
		      "NZGD49 / UTM zone 60S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +south +ellps=intl +datum=nzgd49 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / UTM zone 60S\",GEOGCS[\"NZGD49\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Internat");
    add_srs_wkt (p, 2,
		 "ional 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 6,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"427");
    add_srs_wkt (p, 7,
		 "2\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 8,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 9,
		 "_origin\",0],PARAMETER[\"central_meridian\",177],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",10000000],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"27260\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 13, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 27291, "epsg", 27291,
		      "NZGD49 / North Island Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-39 +lon_0=175.5 +k=1 +x_0=274319.524");
    add_proj4text (p, 1,
		   "3848086 +y_0=365759.3658464114 +ellps=intl +datum=nzgd49");
    add_proj4text (p, 2, " +to_meter=0.9143984146160287 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / North Island Grid\",GEOGCS[\"NZGD49\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Int");
    add_srs_wkt (p, 2,
		 "ernational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"British yard (Sears 1922)\",0.9143984146");
    add_srs_wkt (p, 8,
		 "160287,AUTHORITY[\"EPSG\",\"9040\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 9,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",-39],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"central_meridian\",175.5],PARAMETER[\"scale_f");
    add_srs_wkt (p, 11,
		 "actor\",1],PARAMETER[\"false_easting\",300000],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_northing\",400000],AUTHORITY[\"EPSG\",\"27291\"");
    add_srs_wkt (p, 13, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 27292, "epsg", 27292,
		      "NZGD49 / South Island Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=-44 +lon_0=171.5 +k=1 +x_0=457199.207");
    add_proj4text (p, 1,
		   "3080143 +y_0=457199.2073080143 +ellps=intl +datum=nzgd49");
    add_proj4text (p, 2, " +to_meter=0.9143984146160287 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NZGD49 / South Island Grid\",GEOGCS[\"NZGD49\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"New_Zealand_Geodetic_Datum_1949\",SPHEROID[\"Int");
    add_srs_wkt (p, 2,
		 "ernational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6272\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 5,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 6,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4272\"]],UNIT[\"British yard (Sears 1922)\",0.9143984146");
    add_srs_wkt (p, 8,
		 "160287,AUTHORITY[\"EPSG\",\"9040\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 9,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",-44],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"central_meridian\",171.5],PARAMETER[\"scale_f");
    add_srs_wkt (p, 11,
		 "actor\",1],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"27292\"");
    add_srs_wkt (p, 13, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 27391, "epsg", 27391,
		      "NGO 1948 (Oslo) / NGO zone I");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=-4.666666666666667 +k=1 +x_");
    add_proj4text (p, 1,
		   "0=0 +y_0=0 +a=6377492.018 +b=6356173.508712696 +towgs84=");
    add_proj4text (p, 2,
		   "278.3,93,474.5,7.889,0.05,-6.61,6.21 +pm=oslo +units=m +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone I\",GEOGCS[\"NGO 194");
    add_srs_wkt (p, 1,
		 "8 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel Mod");
    add_srs_wkt (p, 2,
		 "ified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "5\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.72291666666");
    add_srs_wkt (p, 5,
		 "667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.0174");
    add_srs_wkt (p, 6,
		 "5329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",58],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "-4.666666666666667],PARAMETER[\"scale_factor\",1],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",0],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"27391\"],AXIS[\"x\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "y\",EAST]]");
    p = add_epsg_def (first, last, 27392, "epsg", 27392,
		      "NGO 1948 (Oslo) / NGO zone II");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=-2.333333333333333 +k=1 +x_");
    add_proj4text (p, 1,
		   "0=0 +y_0=0 +a=6377492.018 +b=6356173.508712696 +towgs84=");
    add_proj4text (p, 2,
		   "278.3,93,474.5,7.889,0.05,-6.61,6.21 +pm=oslo +units=m +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone II\",GEOGCS[\"NGO 19");
    add_srs_wkt (p, 1,
		 "48 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel Mo");
    add_srs_wkt (p, 2,
		 "dified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "05\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.7229166666");
    add_srs_wkt (p, 5,
		 "6667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",58],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "-2.333333333333333],PARAMETER[\"scale_factor\",1],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",0],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"27392\"],AXIS[\"x\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "y\",EAST]]");
    p = add_epsg_def (first, last, 27393, "epsg", 27393,
		      "NGO 1948 (Oslo) / NGO zone III");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=0 +k=1 +x_0=0 +y_0=0 +a=637");
    add_proj4text (p, 1,
		   "7492.018 +b=6356173.508712696 +towgs84=278.3,93,474.5,7.");
    add_proj4text (p, 2, "889,0.05,-6.61,6.21 +pm=oslo +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone III\",GEOGCS[\"NGO 1");
    add_srs_wkt (p, 1,
		 "948 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel M");
    add_srs_wkt (p, 2,
		 "odified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "005\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.722916666");
    add_srs_wkt (p, 5,
		 "66667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 8,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"latitude_of_origin\",58],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 10,
		 "\",0],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_ea");
    add_srs_wkt (p, 11,
		 "sting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12, "PSG\",\"27393\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 27394, "epsg", 27394,
		      "NGO 1948 (Oslo) / NGO zone IV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=2.5 +k=1 +x_0=0 +y_0=0 +a=6");
    add_proj4text (p, 1,
		   "377492.018 +b=6356173.508712696 +towgs84=278.3,93,474.5,");
    add_proj4text (p, 2, "7.889,0.05,-6.61,6.21 +pm=oslo +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone IV\",GEOGCS[\"NGO 19");
    add_srs_wkt (p, 1,
		 "48 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel Mo");
    add_srs_wkt (p, 2,
		 "dified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "05\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.7229166666");
    add_srs_wkt (p, 5,
		 "6667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",58],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "2.5],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 12, "SG\",\"27394\"],AXIS[\"x\",NORTH],AXIS[\"y\",EAST]]");
    p = add_epsg_def (first, last, 27395, "epsg", 27395,
		      "NGO 1948 (Oslo) / NGO zone V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=6.166666666666667 +k=1 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +a=6377492.018 +b=6356173.508712696 +towgs84=2");
    add_proj4text (p, 2,
		   "78.3,93,474.5,7.889,0.05,-6.61,6.21 +pm=oslo +units=m +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone V\",GEOGCS[\"NGO 194");
    add_srs_wkt (p, 1,
		 "8 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel Mod");
    add_srs_wkt (p, 2,
		 "ified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "5\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.72291666666");
    add_srs_wkt (p, 5,
		 "667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.0174");
    add_srs_wkt (p, 6,
		 "5329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",58],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "6.166666666666667],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",0],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"27395\"],AXIS[\"x\",NORTH],AXIS[\"y");
    add_srs_wkt (p, 13, "\",EAST]]");
    p = add_epsg_def (first, last, 27396, "epsg", 27396,
		      "NGO 1948 (Oslo) / NGO zone VI");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=10.16666666666667 +k=1 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +a=6377492.018 +b=6356173.508712696 +towgs84=2");
    add_proj4text (p, 2,
		   "78.3,93,474.5,7.889,0.05,-6.61,6.21 +pm=oslo +units=m +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone VI\",GEOGCS[\"NGO 19");
    add_srs_wkt (p, 1,
		 "48 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel Mo");
    add_srs_wkt (p, 2,
		 "dified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "05\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],AUT");
    add_srs_wkt (p, 4,
		 "HORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.7229166666");
    add_srs_wkt (p, 5,
		 "6667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 6,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 8,
		 "9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_origin\",58],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 10,
		 "10.16666666666667],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",0],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"27396\"],AXIS[\"x\",NORTH],AXIS[\"y");
    add_srs_wkt (p, 13, "\",EAST]]");
    p = add_epsg_def (first, last, 27397, "epsg", 27397,
		      "NGO 1948 (Oslo) / NGO zone VII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=14.16666666666667 +k=1 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +a=6377492.018 +b=6356173.508712696 +towgs84=2");
    add_proj4text (p, 2,
		   "78.3,93,474.5,7.889,0.05,-6.61,6.21 +pm=oslo +units=m +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone VII\",GEOGCS[\"NGO 1");
    add_srs_wkt (p, 1,
		 "948 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel M");
    add_srs_wkt (p, 2,
		 "odified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "005\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.722916666");
    add_srs_wkt (p, 5,
		 "66667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 6,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 8,
		 "\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"latitude_of_origin\",58],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 10,
		 "\",14.16666666666667],PARAMETER[\"scale_factor\",1],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",0],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"27397\"],AXIS[\"x\",NORTH],AXIS[");
    add_srs_wkt (p, 13, "\"y\",EAST]]");
    p = add_epsg_def (first, last, 27398, "epsg", 27398,
		      "NGO 1948 (Oslo) / NGO zone VIII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=58 +lon_0=18.33333333333333 +k=1 +x_0");
    add_proj4text (p, 1,
		   "=0 +y_0=0 +a=6377492.018 +b=6356173.508712696 +towgs84=2");
    add_proj4text (p, 2,
		   "78.3,93,474.5,7.889,0.05,-6.61,6.21 +pm=oslo +units=m +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGO 1948 (Oslo) / NGO zone VIII\",GEOGCS[\"NGO ");
    add_srs_wkt (p, 1,
		 "1948 (Oslo)\",DATUM[\"NGO_1948_Oslo\",SPHEROID[\"Bessel ");
    add_srs_wkt (p, 2,
		 "Modified\",6377492.018,299.1528128,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7005\"]],TOWGS84[278.3,93,474.5,7.889,0.05,-6.61,6.21],A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"6817\"]],PRIMEM[\"Oslo\",10.72291666");
    add_srs_wkt (p, 5,
		 "666667,AUTHORITY[\"EPSG\",\"8913\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 6,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4817\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"latitude_of_origin\",58],PARAMETER[\"central_meridia");
    add_srs_wkt (p, 10,
		 "n\",18.33333333333333],PARAMETER[\"scale_factor\",1],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",0],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"27398\"],AXIS[\"x\",NORTH],AXIS");
    add_srs_wkt (p, 13, "[\"y\",EAST]]");
    p = add_epsg_def (first, last, 27429, "epsg", 27429,
		      "Datum 73 / UTM zone 29N");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Datum 73 / UTM zone 29N\",GEOGCS[\"Datum 73\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Datum_73\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 2,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "6274\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 4,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 5,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4274\"]],UNIT[\"met");
    add_srs_wkt (p, 6,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 7,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 8,
		 "AMETER[\"central_meridian\",-9],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 9,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"27429\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 27492, "epsg", 27492,
		      "Datum 73 / Modified Portuguese Grid (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=39.66666666666666 +lon_0=-8.131906111");
    add_proj4text (p, 1,
		   "111112 +k=1 +x_0=180.598 +y_0=-86.98999999999999 +ellps=");
    add_proj4text (p, 2, "intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Datum 73 / Modified Portuguese Grid (deprecated");
    add_srs_wkt (p, 1,
		 ")\",GEOGCS[\"Datum 73\",DATUM[\"Datum_73\",SPHEROID[\"In");
    add_srs_wkt (p, 2,
		 "ternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6274\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4274\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",39.66666666666666],PARAMETER[\"centr");
    add_srs_wkt (p, 9,
		 "al_meridian\",-8.131906111111112],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",1],PARAMETER[\"false_easting\",180.598],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",-86.99],AUTHORITY[\"EPSG\",\"27492\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 27493, "epsg", 27493,
		      "Datum 73 / Modified Portuguese Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=39.66666666666666 +lon_0=-8.131906111");
    add_proj4text (p, 1,
		   "111112 +k=1 +x_0=180.598 +y_0=-86.98999999999999 +ellps=");
    add_proj4text (p, 2, "intl +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Datum 73 / Modified Portuguese Grid\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Datum 73\",DATUM[\"Datum_73\",SPHEROID[\"International 1");
    add_srs_wkt (p, 2,
		 "924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6274\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4274\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",39.66666666666666],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-8.131906111111112],PARAMETER[\"scale_factor\",1],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_easting\",180.598],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",-86.99],AUTHORITY[\"EPSG\",\"27493\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 12, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27500, "epsg", 27500, "unnamed");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.5 +lat_0=49.5 +lon_0=5.4 +k_0=0.9995");
    add_proj4text (p, 1,
		   "0908 +x_0=500000 +y_0=300000 +a=6376523 +b=6355862.93325");
    add_proj4text (p, 2, "5573 +pm=2.3372291666985 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"unnamed\",GEOGCS[\"unnamed ellipse\",DATUM[\"un");
    add_srs_wkt (p, 1,
		 "known\",SPHEROID[\"unnamed\",6376523,308.6399999999991]]");
    add_srs_wkt (p, 2,
		 ",PRIMEM[\"unnamed\",2.3372291666985],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 3,
		 "174532925199433]],PROJECTION[\"Lambert_Conformal_Conic_1");
    add_srs_wkt (p, 4,
		 "SP\"],PARAMETER[\"latitude_of_origin\",49.5],PARAMETER[\"");
    add_srs_wkt (p, 5,
		 "central_meridian\",5.4],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 6,
		 "50908],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 7,
		 "alse_northing\",300000],UNIT[\"Meter\",1],AUTHORITY[\"EP");
    add_srs_wkt (p, 8, "SG\",\"27500\"]]");
    p = add_epsg_def (first, last, 27561, "epsg", 27561,
		      "NTF (Paris) / Lambert Nord France");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.50000000000001 +lat_0=49.50000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877341 +x_0=600000 +y_0=200000 +a");
    add_proj4text (p, 2,
		   "=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm=");
    add_proj4text (p, 3, "paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert Nord France\",GEOGCS[\"NT");
    add_srs_wkt (p, 1,
		 "F (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_Par");
    add_srs_wkt (p, 2,
		 "is\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.466021");
    add_srs_wkt (p, 3,
		 "2936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,3");
    add_srs_wkt (p, 4,
		 "20,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris");
    add_srs_wkt (p, 5,
		 "\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\"");
    add_srs_wkt (p, 6,
		 ",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 8,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP");
    add_srs_wkt (p, 9,
		 "\"],PARAMETER[\"latitude_of_origin\",55],PARAMETER[\"cen");
    add_srs_wkt (p, 10,
		 "tral_meridian\",0],PARAMETER[\"scale_factor\",0.99987734");
    add_srs_wkt (p, 11,
		 "1],PARAMETER[\"false_easting\",600000],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_northing\",200000],AUTHORITY[\"EPSG\",\"27561\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27562, "epsg", 27562,
		      "NTF (Paris) / Lambert Centre France");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.8 +lat_0=46.8 +lon_0=0 +k_0=0.999877");
    add_proj4text (p, 1,
		   "42 +x_0=600000 +y_0=200000 +a=6378249.2 +b=6356515 +towg");
    add_proj4text (p, 2,
		   "s84=-168,-60,320,0,0,0,0 +pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert Centre France\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_P");
    add_srs_wkt (p, 2,
		 "aris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660");
    add_srs_wkt (p, 3,
		 "212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60");
    add_srs_wkt (p, 4,
		 ",320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Par");
    add_srs_wkt (p, 5,
		 "is\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"gra");
    add_srs_wkt (p, 6,
		 "d\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 8,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1");
    add_srs_wkt (p, 9,
		 "SP\"],PARAMETER[\"latitude_of_origin\",52],PARAMETER[\"c");
    add_srs_wkt (p, 10,
		 "entral_meridian\",0],PARAMETER[\"scale_factor\",0.999877");
    add_srs_wkt (p, 11,
		 "42],PARAMETER[\"false_easting\",600000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",200000],AUTHORITY[\"EPSG\",\"27562\"],AXIS[");
    add_srs_wkt (p, 13, "\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27563, "epsg", 27563,
		      "NTF (Paris) / Lambert Sud France");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.10000000000001 +lat_0=44.10000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877499 +x_0=600000 +y_0=200000 +a");
    add_proj4text (p, 2,
		   "=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm=");
    add_proj4text (p, 3, "paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert Sud France\",GEOGCS[\"NTF");
    add_srs_wkt (p, 1,
		 " (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_Pari");
    add_srs_wkt (p, 2,
		 "s\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212");
    add_srs_wkt (p, 3,
		 "936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,32");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris\"");
    add_srs_wkt (p, 5,
		 ",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",");
    add_srs_wkt (p, 6,
		 "0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 8,
		 "G\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",49],PARAMETER[\"centr");
    add_srs_wkt (p, 10,
		 "al_meridian\",0],PARAMETER[\"scale_factor\",0.999877499]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",600000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",200000],AUTHORITY[\"EPSG\",\"27563\"],AXIS[\"X");
    add_srs_wkt (p, 13, "\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27564, "epsg", 27564,
		      "NTF (Paris) / Lambert Corse");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.16500000000001 +lat_0=42.16500000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.99994471 +x_0=234.358 +y_0=185861.36");
    add_proj4text (p, 2,
		   "9 +a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 ");
    add_proj4text (p, 3, "+pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert Corse\",GEOGCS[\"NTF (Par");
    add_srs_wkt (p, 1,
		 "is)\",DATUM[\"Nouvelle_Triangulation_Francaise_Paris\",S");
    add_srs_wkt (p, 2,
		 "PHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.466021293626");
    add_srs_wkt (p, 3,
		 "5,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,320,0,0");
    add_srs_wkt (p, 4,
		 ",0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris\",2.3");
    add_srs_wkt (p, 5,
		 "3722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.01");
    add_srs_wkt (p, 6,
		 "570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 8,
		 "\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"latitude_of_origin\",46.85],PARAMETER[\"centr");
    add_srs_wkt (p, 10,
		 "al_meridian\",0],PARAMETER[\"scale_factor\",0.99994471],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_easting\",234.358],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",185861.369],AUTHORITY[\"EPSG\",\"27564\"],AXIS");
    add_srs_wkt (p, 13, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27571, "epsg", 27571,
		      "NTF (Paris) / Lambert zone I");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.50000000000001 +lat_0=49.50000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877341 +x_0=600000 +y_0=1200000 +");
    add_proj4text (p, 2,
		   "a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm");
    add_proj4text (p, 3, "=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert zone I\",GEOGCS[\"NTF (Pa");
    add_srs_wkt (p, 1,
		 "ris)\",DATUM[\"Nouvelle_Triangulation_Francaise_Paris\",");
    add_srs_wkt (p, 2,
		 "SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.46602129362");
    add_srs_wkt (p, 3,
		 "65,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,320,0,");
    add_srs_wkt (p, 4,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris\",2.");
    add_srs_wkt (p, 5,
		 "33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.0");
    add_srs_wkt (p, 6,
		 "1570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORITY");
    add_srs_wkt (p, 7,
		 "[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"latitude_of_origin\",55],PARAMETER[\"central");
    add_srs_wkt (p, 10,
		 "_meridian\",0],PARAMETER[\"scale_factor\",0.999877341],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",600000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",1200000],AUTHORITY[\"EPSG\",\"27571\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27572, "epsg", 27572,
		      "NTF (Paris) / Lambert zone II");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.8 +lat_0=46.8 +lon_0=0 +k_0=0.999877");
    add_proj4text (p, 1,
		   "42 +x_0=600000 +y_0=2200000 +a=6378249.2 +b=6356515 +tow");
    add_proj4text (p, 2,
		   "gs84=-168,-60,320,0,0,0,0 +pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert zone II\",GEOGCS[\"NTF (P");
    add_srs_wkt (p, 1,
		 "aris)\",DATUM[\"Nouvelle_Triangulation_Francaise_Paris\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212936");
    add_srs_wkt (p, 3,
		 "265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,320,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris\",2");
    add_srs_wkt (p, 5,
		 ".33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.");
    add_srs_wkt (p, 6,
		 "01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"latitude_of_origin\",52],PARAMETER[\"central");
    add_srs_wkt (p, 10,
		 "_meridian\",0],PARAMETER[\"scale_factor\",0.99987742],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",600000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",2200000],AUTHORITY[\"EPSG\",\"27572\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27573, "epsg", 27573,
		      "NTF (Paris) / Lambert zone III");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.10000000000001 +lat_0=44.10000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877499 +x_0=600000 +y_0=3200000 +");
    add_proj4text (p, 2,
		   "a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm");
    add_proj4text (p, 3, "=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert zone III\",GEOGCS[\"NTF (");
    add_srs_wkt (p, 1,
		 "Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_Paris\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212936");
    add_srs_wkt (p, 3,
		 "265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,320,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris\",2");
    add_srs_wkt (p, 5,
		 ".33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.");
    add_srs_wkt (p, 6,
		 "01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"latitude_of_origin\",49],PARAMETER[\"central");
    add_srs_wkt (p, 10,
		 "_meridian\",0],PARAMETER[\"scale_factor\",0.999877499],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_easting\",600000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",3200000],AUTHORITY[\"EPSG\",\"27573\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27574, "epsg", 27574,
		      "NTF (Paris) / Lambert zone IV");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.16500000000001 +lat_0=42.16500000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.99994471 +x_0=234.358 +y_0=4185861.3");
    add_proj4text (p, 2,
		   "69 +a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0");
    add_proj4text (p, 3, " +pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Lambert zone IV\",GEOGCS[\"NTF (P");
    add_srs_wkt (p, 1,
		 "aris)\",DATUM[\"Nouvelle_Triangulation_Francaise_Paris\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212936");
    add_srs_wkt (p, 3,
		 "265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,320,0");
    add_srs_wkt (p, 4,
		 ",0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris\",2");
    add_srs_wkt (p, 5,
		 ".33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.");
    add_srs_wkt (p, 6,
		 "01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORIT");
    add_srs_wkt (p, 7,
		 "Y[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 8,
		 ",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"latitude_of_origin\",46.85],PARAMETER[\"cent");
    add_srs_wkt (p, 10,
		 "ral_meridian\",0],PARAMETER[\"scale_factor\",0.99994471]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",234.358],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "northing\",4185861.369],AUTHORITY[\"EPSG\",\"27574\"],AX");
    add_srs_wkt (p, 13, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27581, "epsg", 27581,
		      "NTF (Paris) / France I (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.50000000000001 +lat_0=49.50000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877341 +x_0=600000 +y_0=1200000 +");
    add_proj4text (p, 2,
		   "a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm");
    add_proj4text (p, 3, "=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / France I (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_P");
    add_srs_wkt (p, 2,
		 "aris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660");
    add_srs_wkt (p, 3,
		 "212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60");
    add_srs_wkt (p, 4,
		 ",320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Par");
    add_srs_wkt (p, 5,
		 "is\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"gra");
    add_srs_wkt (p, 6,
		 "d\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 8,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1");
    add_srs_wkt (p, 9,
		 "SP\"],PARAMETER[\"latitude_of_origin\",55],PARAMETER[\"c");
    add_srs_wkt (p, 10,
		 "entral_meridian\",0],PARAMETER[\"scale_factor\",0.999877");
    add_srs_wkt (p, 11,
		 "341],PARAMETER[\"false_easting\",600000],PARAMETER[\"fal");
    add_srs_wkt (p, 12,
		 "se_northing\",1200000],AUTHORITY[\"EPSG\",\"27581\"],AXI");
    add_srs_wkt (p, 13, "S[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27582, "epsg", 27582,
		      "NTF (Paris) / France II (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.8 +lat_0=46.8 +lon_0=0 +k_0=0.999877");
    add_proj4text (p, 1,
		   "42 +x_0=600000 +y_0=2200000 +a=6378249.2 +b=6356515 +tow");
    add_proj4text (p, 2,
		   "gs84=-168,-60,320,0,0,0,0 +pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / France II (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_P");
    add_srs_wkt (p, 2,
		 "aris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660");
    add_srs_wkt (p, 3,
		 "212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60");
    add_srs_wkt (p, 4,
		 ",320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Par");
    add_srs_wkt (p, 5,
		 "is\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"gra");
    add_srs_wkt (p, 6,
		 "d\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 8,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1");
    add_srs_wkt (p, 9,
		 "SP\"],PARAMETER[\"latitude_of_origin\",52],PARAMETER[\"c");
    add_srs_wkt (p, 10,
		 "entral_meridian\",0],PARAMETER[\"scale_factor\",0.999877");
    add_srs_wkt (p, 11,
		 "42],PARAMETER[\"false_easting\",600000],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",2200000],AUTHORITY[\"EPSG\",\"27582\"],AXIS");
    add_srs_wkt (p, 13, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27583, "epsg", 27583,
		      "NTF (Paris) / France III (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.10000000000001 +lat_0=44.10000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877499 +x_0=600000 +y_0=3200000 +");
    add_proj4text (p, 2,
		   "a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm");
    add_proj4text (p, 3, "=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / France III (deprecated)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise");
    add_srs_wkt (p, 2,
		 "_Paris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.46");
    add_srs_wkt (p, 3,
		 "60212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-");
    add_srs_wkt (p, 4,
		 "60,320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"P");
    add_srs_wkt (p, 5,
		 "aris\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"g");
    add_srs_wkt (p, 6,
		 "rad\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY");
    add_srs_wkt (p, 8,
		 "[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Coni");
    add_srs_wkt (p, 9,
		 "c_1SP\"],PARAMETER[\"latitude_of_origin\",49],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"central_meridian\",0],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 11,
		 "877499],PARAMETER[\"false_easting\",600000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",3200000],AUTHORITY[\"EPSG\",\"27583\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27584, "epsg", 27584,
		      "NTF (Paris) / France IV (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.16500000000001 +lat_0=42.16500000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.99994471 +x_0=234.358 +y_0=4185861.3");
    add_proj4text (p, 2,
		   "69 +a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0");
    add_proj4text (p, 3, " +pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / France IV (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_P");
    add_srs_wkt (p, 2,
		 "aris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660");
    add_srs_wkt (p, 3,
		 "212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60");
    add_srs_wkt (p, 4,
		 ",320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Par");
    add_srs_wkt (p, 5,
		 "is\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"gra");
    add_srs_wkt (p, 6,
		 "d\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AU");
    add_srs_wkt (p, 7,
		 "THORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 8,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1");
    add_srs_wkt (p, 9,
		 "SP\"],PARAMETER[\"latitude_of_origin\",46.85],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"central_meridian\",0],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 11,
		 "94471],PARAMETER[\"false_easting\",234.358],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",4185861.369],AUTHORITY[\"EPSG\",\"27584");
    add_srs_wkt (p, 13, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27591, "epsg", 27591,
		      "NTF (Paris) / Nord France (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.50000000000001 +lat_0=49.50000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877341 +x_0=600000 +y_0=200000 +a");
    add_proj4text (p, 2,
		   "=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm=");
    add_proj4text (p, 3, "paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Nord France (deprecated)\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Francais");
    add_srs_wkt (p, 2,
		 "e_Paris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4");
    add_srs_wkt (p, 3,
		 "660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,");
    add_srs_wkt (p, 4,
		 "-60,320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"");
    add_srs_wkt (p, 5,
		 "Paris\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"");
    add_srs_wkt (p, 6,
		 "grad\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Con");
    add_srs_wkt (p, 9,
		 "ic_1SP\"],PARAMETER[\"latitude_of_origin\",55],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"central_meridian\",0],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 11,
		 "9877341],PARAMETER[\"false_easting\",600000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",200000],AUTHORITY[\"EPSG\",\"27591\"],A");
    add_srs_wkt (p, 13, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27592, "epsg", 27592,
		      "NTF (Paris) / Centre France (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.8 +lat_0=46.8 +lon_0=0 +k_0=0.999877");
    add_proj4text (p, 1,
		   "42 +x_0=600000 +y_0=200000 +a=6378249.2 +b=6356515 +towg");
    add_proj4text (p, 2,
		   "s84=-168,-60,320,0,0,0,0 +pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Centre France (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Franca");
    add_srs_wkt (p, 2,
		 "ise_Paris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293");
    add_srs_wkt (p, 3,
		 ".4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-16");
    add_srs_wkt (p, 4,
		 "8,-60,320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[");
    add_srs_wkt (p, 5,
		 "\"Paris\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[");
    add_srs_wkt (p, 6,
		 "\"grad\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"");
    add_srs_wkt (p, 7,
		 "]],AUTHORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 8,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_C");
    add_srs_wkt (p, 9,
		 "onic_1SP\"],PARAMETER[\"latitude_of_origin\",52],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"central_meridian\",0],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 11,
		 "99987742],PARAMETER[\"false_easting\",600000],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_northing\",200000],AUTHORITY[\"EPSG\",\"27592\"]");
    add_srs_wkt (p, 13, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27593, "epsg", 27593,
		      "NTF (Paris) / Sud France (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.10000000000001 +lat_0=44.10000000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.999877499 +x_0=600000 +y_0=200000 +a");
    add_proj4text (p, 2,
		   "=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 +pm=");
    add_proj4text (p, 3, "paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Sud France (deprecated)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"NTF (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise");
    add_srs_wkt (p, 2,
		 "_Paris\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.46");
    add_srs_wkt (p, 3,
		 "60212936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-");
    add_srs_wkt (p, 4,
		 "60,320,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"P");
    add_srs_wkt (p, 5,
		 "aris\",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"g");
    add_srs_wkt (p, 6,
		 "rad\",0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],");
    add_srs_wkt (p, 7,
		 "AUTHORITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY");
    add_srs_wkt (p, 8,
		 "[\"EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Coni");
    add_srs_wkt (p, 9,
		 "c_1SP\"],PARAMETER[\"latitude_of_origin\",49],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"central_meridian\",0],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 11,
		 "877499],PARAMETER[\"false_easting\",600000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",200000],AUTHORITY[\"EPSG\",\"27593\"],A");
    add_srs_wkt (p, 13, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27594, "epsg", 27594,
		      "NTF (Paris) / Corse (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.16500000000001 +lat_0=42.16500000000");
    add_proj4text (p, 1,
		   "001 +lon_0=0 +k_0=0.99994471 +x_0=234.358 +y_0=185861.36");
    add_proj4text (p, 2,
		   "9 +a=6378249.2 +b=6356515 +towgs84=-168,-60,320,0,0,0,0 ");
    add_proj4text (p, 3, "+pm=paris +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NTF (Paris) / Corse (deprecated)\",GEOGCS[\"NTF");
    add_srs_wkt (p, 1,
		 " (Paris)\",DATUM[\"Nouvelle_Triangulation_Francaise_Pari");
    add_srs_wkt (p, 2,
		 "s\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.4660212");
    add_srs_wkt (p, 3,
		 "936265,AUTHORITY[\"EPSG\",\"7011\"]],TOWGS84[-168,-60,32");
    add_srs_wkt (p, 4,
		 "0,0,0,0,0],AUTHORITY[\"EPSG\",\"6807\"]],PRIMEM[\"Paris\"");
    add_srs_wkt (p, 5,
		 ",2.33722917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",");
    add_srs_wkt (p, 6,
		 "0.01570796326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"4807\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 8,
		 "G\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",46.85],PARAMETER[\"ce");
    add_srs_wkt (p, 10,
		 "ntral_meridian\",0],PARAMETER[\"scale_factor\",0.9999447");
    add_srs_wkt (p, 11,
		 "1],PARAMETER[\"false_easting\",234.358],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_northing\",185861.369],AUTHORITY[\"EPSG\",\"27594\"],A");
    add_srs_wkt (p, 13, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 27700, "epsg", 27700,
		      "OSGB 1936 / British National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=49 +lon_0=-2 +k=0.9996012717 +x_0=400");
    add_proj4text (p, 1,
		   "000 +y_0=-100000 +ellps=airy +datum=OSGB36 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"OSGB 1936 / British National Grid\",GEOGCS[\"OS");
    add_srs_wkt (p, 1,
		 "GB 1936\",DATUM[\"OSGB_1936\",SPHEROID[\"Airy 1830\",637");
    add_srs_wkt (p, 2,
		 "7563.396,299.3249646,AUTHORITY[\"EPSG\",\"7001\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6277\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"427");
    add_srs_wkt (p, 6,
		 "7\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",49],PARAMETER[\"central_meridian\",-2],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9996012717],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",400000],PARAMETER[\"false_northing\",-100000],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"27700\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_23 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 28191, "epsg", 28191,
		      "Palestine 1923 / Palestine Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=31.73409694444445 +lon_0=35.2120805555");
    add_proj4text (p, 1,
		   "5556 +x_0=170251.555 +y_0=126867.909 +a=6378300.789 +b=6");
    add_proj4text (p, 2,
		   "356566.435 +towgs84=-275.722,94.7824,340.894,-8.001,-4.4");
    add_proj4text (p, 3, "2,-11.821,1 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Palestine 1923 / Palestine Grid\",GEOGCS[\"Pale");
    add_srs_wkt (p, 1,
		 "stine 1923\",DATUM[\"Palestine_1923\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1880 (Benoit)\",6378300.789,293.4663155389802,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7010\"]],TOWGS84[-275.722,94.7824,340.894,-8.");
    add_srs_wkt (p, 4,
		 "001,-4.42,-11.821,1],AUTHORITY[\"EPSG\",\"6281\"]],PRIME");
    add_srs_wkt (p, 5,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 6,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 7,
		 "],AUTHORITY[\"EPSG\",\"4281\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 8,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Cassini_Soldner\"],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"latitude_of_origin\",31.73409694444445],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"central_meridian\",35.21208055555556],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",170251.555],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",126867.909],AUTHORITY[\"EPSG\",\"28191\"],AXIS[\"Eastin");
    add_srs_wkt (p, 13, "g\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28192, "epsg", 28192,
		      "Palestine 1923 / Palestine Belt");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31.73409694444445 +lon_0=35.212080555");
    add_proj4text (p, 1,
		   "55556 +k=1 +x_0=170251.555 +y_0=1126867.909 +a=6378300.7");
    add_proj4text (p, 2,
		   "89 +b=6356566.435 +towgs84=-275.722,94.7824,340.894,-8.0");
    add_proj4text (p, 3, "01,-4.42,-11.821,1 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Palestine 1923 / Palestine Belt\",GEOGCS[\"Pale");
    add_srs_wkt (p, 1,
		 "stine 1923\",DATUM[\"Palestine_1923\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1880 (Benoit)\",6378300.789,293.4663155389802,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7010\"]],TOWGS84[-275.722,94.7824,340.894,-8.");
    add_srs_wkt (p, 4,
		 "001,-4.42,-11.821,1],AUTHORITY[\"EPSG\",\"6281\"]],PRIME");
    add_srs_wkt (p, 5,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 6,
		 "egree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]");
    add_srs_wkt (p, 7,
		 "],AUTHORITY[\"EPSG\",\"4281\"]],UNIT[\"metre\",1,AUTHORI");
    add_srs_wkt (p, 8,
		 "TY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",31.73409694444445],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"central_meridian\",35.21208055555556],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",170251");
    add_srs_wkt (p, 12,
		 ".555],PARAMETER[\"false_northing\",1126867.909],AUTHORIT");
    add_srs_wkt (p, 13,
		 "Y[\"EPSG\",\"28192\"],AXIS[\"Easting\",EAST],AXIS[\"Nort");
    add_srs_wkt (p, 14, "hing\",NORTH]]");
    p = add_epsg_def (first, last, 28193, "epsg", 28193,
		      "Palestine 1923 / Israeli CS Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=31.73409694444445 +lon_0=35.2120805555");
    add_proj4text (p, 1,
		   "5556 +x_0=170251.555 +y_0=1126867.909 +a=6378300.789 +b=");
    add_proj4text (p, 2,
		   "6356566.435 +towgs84=-275.722,94.7824,340.894,-8.001,-4.");
    add_proj4text (p, 3, "42,-11.821,1 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Palestine 1923 / Israeli CS Grid\",GEOGCS[\"Pal");
    add_srs_wkt (p, 1,
		 "estine 1923\",DATUM[\"Palestine_1923\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 2,
		 " 1880 (Benoit)\",6378300.789,293.4663155389802,AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"7010\"]],TOWGS84[-275.722,94.7824,340.894,-8");
    add_srs_wkt (p, 4,
		 ".001,-4.42,-11.821,1],AUTHORITY[\"EPSG\",\"6281\"]],PRIM");
    add_srs_wkt (p, 5,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 6,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 7,
		 "]],AUTHORITY[\"EPSG\",\"4281\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 8,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Cassini_Soldner\"],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"latitude_of_origin\",31.73409694444445],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"central_meridian\",35.21208055555556],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",170251.555],PARAMETER[\"false_northing");
    add_srs_wkt (p, 12,
		 "\",1126867.909],AUTHORITY[\"EPSG\",\"28193\"],AXIS[\"Eas");
    add_srs_wkt (p, 13, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28232, "epsg", 28232,
		      "Pointe Noire / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +a=6378249.2 +b=6356515 +units");
    add_proj4text (p, 1, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pointe Noire / UTM zone 32S\",GEOGCS[\"Pointe N");
    add_srs_wkt (p, 1,
		 "oire\",DATUM[\"Congo_1960_Pointe_Noire\",SPHEROID[\"Clar");
    add_srs_wkt (p, 2,
		 "ke 1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7011\"]],AUTHORITY[\"EPSG\",\"6282\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4282\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",9],PARAMETER[\"scale_factor\",0.9996],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",500000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "10000000],AUTHORITY[\"EPSG\",\"28232\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 12, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28348, "epsg", 28348, "GDA94 / MGA zone 48");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 48\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",105],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28348\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28349, "epsg", 28349, "GDA94 / MGA zone 49");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 49\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",111],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28349\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28350, "epsg", 28350, "GDA94 / MGA zone 50");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 50\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",117],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28350\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28351, "epsg", 28351, "GDA94 / MGA zone 51");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 51\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",123],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28351\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28352, "epsg", 28352, "GDA94 / MGA zone 52");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 52\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",129],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28352\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28353, "epsg", 28353, "GDA94 / MGA zone 53");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 53\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",135],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28353\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28354, "epsg", 28354, "GDA94 / MGA zone 54");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 54\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",141],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28354\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28355, "epsg", 28355, "GDA94 / MGA zone 55");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 55\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",147],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28355\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28356, "epsg", 28356, "GDA94 / MGA zone 56");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 56\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",153],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28356\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28357, "epsg", 28357, "GDA94 / MGA zone 57");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 57\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",159],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28357\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28358, "epsg", 28358, "GDA94 / MGA zone 58");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"GDA94 / MGA zone 58\",GEOGCS[\"GDA94\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Geocentric_Datum_of_Australia_1994\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6283\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4283\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",165],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"28358\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28402, "epsg", 28402,
		      "Pulkovo 1942 / Gauss-Kruger zone 2 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=2500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 2 (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",9],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",2");
    add_srs_wkt (p, 11,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 12, "\",\"28402\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28403, "epsg", 28403,
		      "Pulkovo 1942 / Gauss-Kruger zone 3 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=3500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 3 (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHERO");
    add_srs_wkt (p, 2,
		 "ID[\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12]");
    add_srs_wkt (p, 4,
		 ",AUTHORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 6,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",15],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 11,
		 "3500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"28403\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28404, "epsg", 28404,
		      "Pulkovo 1942 / Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 4\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",21],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",4500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28404\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28405, "epsg", 28405,
		      "Pulkovo 1942 / Gauss-Kruger zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 5\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",27],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",5500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28405\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28406, "epsg", 28406,
		      "Pulkovo 1942 / Gauss-Kruger zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 6\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",33],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",6500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28406\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28407, "epsg", 28407,
		      "Pulkovo 1942 / Gauss-Kruger zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=7500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 7\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",39],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",7500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28407\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28408, "epsg", 28408,
		      "Pulkovo 1942 / Gauss-Kruger zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 8\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",45],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",8500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28408\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28409, "epsg", 28409,
		      "Pulkovo 1942 / Gauss-Kruger zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=9500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 9\",GEOGCS[\"P");
    add_srs_wkt (p, 1,
		 "ulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassows");
    add_srs_wkt (p, 2,
		 "ky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",51],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",9500000],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28409\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28410, "epsg", 28410,
		      "Pulkovo 1942 / Gauss-Kruger zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=10500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 10\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",57],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",10500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28410\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28411, "epsg", 28411,
		      "Pulkovo 1942 / Gauss-Kruger zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=11500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 11\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",63],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",11500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28411\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28412, "epsg", 28412,
		      "Pulkovo 1942 / Gauss-Kruger zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=12500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 12\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",69],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",12500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28412\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28413, "epsg", 28413,
		      "Pulkovo 1942 / Gauss-Kruger zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=13500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 13\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",75],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",13500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28413\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28414, "epsg", 28414,
		      "Pulkovo 1942 / Gauss-Kruger zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=14500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 14\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",81],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",14500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28414\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28415, "epsg", 28415,
		      "Pulkovo 1942 / Gauss-Kruger zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=15500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 15\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",87],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",15500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28415\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28416, "epsg", 28416,
		      "Pulkovo 1942 / Gauss-Kruger zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=16500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 16\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",93],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",16500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28416\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28417, "epsg", 28417,
		      "Pulkovo 1942 / Gauss-Kruger zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=17500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 17\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",99],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",1],PARAMETER[\"false_easting\",17500000],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28417\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28418, "epsg", 28418,
		      "Pulkovo 1942 / Gauss-Kruger zone 18");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=18500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 18\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",105],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",18500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28418");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28419, "epsg", 28419,
		      "Pulkovo 1942 / Gauss-Kruger zone 19");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=19500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 19\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",111],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",19500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28419");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28420, "epsg", 28420,
		      "Pulkovo 1942 / Gauss-Kruger zone 20");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=20500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 20\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",117],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",20500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28420");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28421, "epsg", 28421,
		      "Pulkovo 1942 / Gauss-Kruger zone 21");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=21500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 21\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",123],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",21500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28421");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28422, "epsg", 28422,
		      "Pulkovo 1942 / Gauss-Kruger zone 22");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=22500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 22\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",129],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",22500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28422");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28423, "epsg", 28423,
		      "Pulkovo 1942 / Gauss-Kruger zone 23");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=23500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 23\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",135],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",23500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28423");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28424, "epsg", 28424,
		      "Pulkovo 1942 / Gauss-Kruger zone 24");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=24500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 24\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",24500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28424");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28425, "epsg", 28425,
		      "Pulkovo 1942 / Gauss-Kruger zone 25");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=25500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 25\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",147],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",25500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28425");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28426, "epsg", 28426,
		      "Pulkovo 1942 / Gauss-Kruger zone 26");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=26500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 26\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",153],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",26500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28426");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28427, "epsg", 28427,
		      "Pulkovo 1942 / Gauss-Kruger zone 27");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=27500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 27\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",159],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",27500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28427");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28428, "epsg", 28428,
		      "Pulkovo 1942 / Gauss-Kruger zone 28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=28500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 28\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",165],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",28500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28428");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28429, "epsg", 28429,
		      "Pulkovo 1942 / Gauss-Kruger zone 29");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=29500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 29\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",171],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",29500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28429");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28430, "epsg", 28430,
		      "Pulkovo 1942 / Gauss-Kruger zone 30");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=30500000 +y_0=");
    add_proj4text (p, 1, "0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 30\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",177],PARAMETER[\"sc");
    add_srs_wkt (p, 10,
		 "ale_factor\",1],PARAMETER[\"false_easting\",30500000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"28430");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28431, "epsg", 28431,
		      "Pulkovo 1942 / Gauss-Kruger zone 31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=31500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 31\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",-177],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",31500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2843");
    add_srs_wkt (p, 12, "1\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28432, "epsg", 28432,
		      "Pulkovo 1942 / Gauss-Kruger zone 32");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=32500000 +y_0");
    add_proj4text (p, 1, "=0 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger zone 32\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"Krassow");
    add_srs_wkt (p, 2,
		 "sky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7024\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4284\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 9,
		 "\",0],PARAMETER[\"central_meridian\",-171],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",1],PARAMETER[\"false_easting\",32500000],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2843");
    add_srs_wkt (p, 12, "2\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28462, "epsg", 28462,
		      "Pulkovo 1942 / Gauss-Kruger 2N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=500000 +y_0=0 +e");
    add_proj4text (p, 1, "llps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 2N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",9],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"scale_factor\",1],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"2");
    add_srs_wkt (p, 12, "8462\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28463, "epsg", 28463,
		      "Pulkovo 1942 / Gauss-Kruger 3N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 3N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",15],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "28463\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28464, "epsg", 28464,
		      "Pulkovo 1942 / Gauss-Kruger 4N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 4N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",21],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "28464\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28465, "epsg", 28465,
		      "Pulkovo 1942 / Gauss-Kruger 5N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=27 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 5N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",27],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "28465\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28466, "epsg", 28466,
		      "Pulkovo 1942 / Gauss-Kruger 6N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=33 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 6N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",33],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "28466\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28467, "epsg", 28467,
		      "Pulkovo 1942 / Gauss-Kruger 7N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=39 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 7N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",39],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "28467\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28468, "epsg", 28468,
		      "Pulkovo 1942 / Gauss-Kruger 8N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=45 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 8N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",45],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "28468\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28469, "epsg", 28469,
		      "Pulkovo 1942 / Gauss-Kruger 9N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=51 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 9N (deprecated)\",G");
    add_srs_wkt (p, 1,
		 "EOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "4\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 5,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 6,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 7,
		 "84\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 8,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",0],PARAMETER[\"central_meridian\",51],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"scale_factor\",1],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "28469\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28470, "epsg", 28470,
		      "Pulkovo 1942 / Gauss-Kruger 10N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=57 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 10N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",57],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28470\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28471, "epsg", 28471,
		      "Pulkovo 1942 / Gauss-Kruger 11N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=63 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 11N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",63],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28471\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28472, "epsg", 28472,
		      "Pulkovo 1942 / Gauss-Kruger 12N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=69 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 12N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",69],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28472\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28473, "epsg", 28473,
		      "Pulkovo 1942 / Gauss-Kruger 13N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=75 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 13N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",75],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28473\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28474, "epsg", 28474,
		      "Pulkovo 1942 / Gauss-Kruger 14N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=81 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 14N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",81],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28474\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28475, "epsg", 28475,
		      "Pulkovo 1942 / Gauss-Kruger 15N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=87 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 15N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",87],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28475\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28476, "epsg", 28476,
		      "Pulkovo 1942 / Gauss-Kruger 16N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=93 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 16N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",93],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28476\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28477, "epsg", 28477,
		      "Pulkovo 1942 / Gauss-Kruger 17N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=99 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 17N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",99],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",1],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"28477\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28478, "epsg", 28478,
		      "Pulkovo 1942 / Gauss-Kruger 18N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=105 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 18N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",105],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28478\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28479, "epsg", 28479,
		      "Pulkovo 1942 / Gauss-Kruger 19N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=111 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 19N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",111],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28479\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28480, "epsg", 28480,
		      "Pulkovo 1942 / Gauss-Kruger 20N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=117 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 20N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",117],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28480\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28481, "epsg", 28481,
		      "Pulkovo 1942 / Gauss-Kruger 21N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=123 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 21N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",123],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28481\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28482, "epsg", 28482,
		      "Pulkovo 1942 / Gauss-Kruger 22N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=129 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 22N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",129],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28482\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28483, "epsg", 28483,
		      "Pulkovo 1942 / Gauss-Kruger 23N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=135 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 23N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",135],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28483\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28484, "epsg", 28484,
		      "Pulkovo 1942 / Gauss-Kruger 24N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=141 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 24N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",141],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28484\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28485, "epsg", 28485,
		      "Pulkovo 1942 / Gauss-Kruger 25N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=147 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 25N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",147],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28485\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28486, "epsg", 28486,
		      "Pulkovo 1942 / Gauss-Kruger 26N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=153 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 26N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",153],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28486\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28487, "epsg", 28487,
		      "Pulkovo 1942 / Gauss-Kruger 27N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=159 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 27N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",159],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28487\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28488, "epsg", 28488,
		      "Pulkovo 1942 / Gauss-Kruger 28N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=165 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 28N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",165],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28488\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28489, "epsg", 28489,
		      "Pulkovo 1942 / Gauss-Kruger 29N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=171 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 29N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",171],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28489\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28490, "epsg", 28490,
		      "Pulkovo 1942 / Gauss-Kruger 30N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=177 +k=1 +x_0=500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 30N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",177],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"scale_factor\",1],PARAMETER[\"false_easting\",50");
    add_srs_wkt (p, 11,
		 "0000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28490\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28491, "epsg", 28491,
		      "Pulkovo 1942 / Gauss-Kruger 31N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-177 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 31N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-177],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28491\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28492, "epsg", 28492,
		      "Pulkovo 1942 / Gauss-Kruger 32N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-171 +k=1 +x_0=500000 +y_0=0");
    add_proj4text (p, 1, " +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Pulkovo 1942 / Gauss-Kruger 32N (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Pulkovo 1942\",DATUM[\"Pulkovo_1942\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Krassowsky 1940\",6378245,298.3,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "024\"]],TOWGS84[23.9,-141.3,-80.9,-0,0.37,0.85,-0.12],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6284\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4284\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-171],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"28492\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 28600, "epsg", 28600,
		      "Qatar 1974 / Qatar National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=24.45 +lon_0=51.21666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "9999 +x_0=200000 +y_0=300000 +ellps=intl +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Qatar 1974 / Qatar National Grid\",GEOGCS[\"Qat");
    add_srs_wkt (p, 1,
		 "ar 1974\",DATUM[\"Qatar_1974\",SPHEROID[\"International ");
    add_srs_wkt (p, 2,
		 "1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6285\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4285\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",24.45],PARAMETER[\"central_meridian\",51.21666666");
    add_srs_wkt (p, 9,
		 "666667],PARAMETER[\"scale_factor\",0.99999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",200000],PARAMETER[\"false_northing\",300");
    add_srs_wkt (p, 11,
		 "000],AUTHORITY[\"EPSG\",\"28600\"],AXIS[\"Easting\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 28991, "epsg", 28991, "Amersfoort / RD Old");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=52.15616055555555 +lon_0=5.387638888");
    add_proj4text (p, 1,
		   "88889 +k=0.9999079 +x_0=0 +y_0=0 +ellps=bessel +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Amersfoort / RD Old\",GEOGCS[\"Amersfoort\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Amersfoort\",SPHEROID[\"Bessel 1841\",6377397.155,2");
    add_srs_wkt (p, 2,
		 "99.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6289\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4289\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Oblique_Stereographic\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",52.15616055555555],PARAMETER[\"central_meridian\",5.387");
    add_srs_wkt (p, 9,
		 "63888888889],PARAMETER[\"scale_factor\",0.9999079],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",0],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 11,
		 "],AUTHORITY[\"EPSG\",\"28991\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 28992, "epsg", 28992, "Amersfoort / RD New");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=52.15616055555555 +lon_0=5.387638888");
    add_proj4text (p, 1,
		   "88889 +k=0.9999079 +x_0=155000 +y_0=463000 +ellps=bessel");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Amersfoort / RD New\",GEOGCS[\"Amersfoort\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"Amersfoort\",SPHEROID[\"Bessel 1841\",6377397.155,2");
    add_srs_wkt (p, 2,
		 "99.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"6289\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4289\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Oblique_Stereographic\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",52.15616055555555],PARAMETER[\"central_meridian\",5.387");
    add_srs_wkt (p, 9,
		 "63888888889],PARAMETER[\"scale_factor\",0.9999079],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",155000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 11,
		 "g\",463000],AUTHORITY[\"EPSG\",\"28992\"],AXIS[\"X\",EAS");
    add_srs_wkt (p, 12, "T],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 29100, "epsg", 29100,
		      "SAD69 / Brazil Polyconic (deprecated)");
    add_proj4text (p, 0,
		   "+proj=poly +lat_0=0 +lon_0=-54 +x_0=5000000 +y_0=1000000");
    add_proj4text (p, 1, "0 +ellps=GRS67 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / Brazil Polyconic (deprecated)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"SAD69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "GRS 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "36\"]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "32925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Polyconic\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",0],PARAMETER[\"central_meridian\",-54],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"false_easting\",5000000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 10,
		 "\",10000000],AUTHORITY[\"EPSG\",\"29100\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 11, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 29101, "epsg", 29101,
		      "SAD69 / Brazil Polyconic");
    add_proj4text (p, 0,
		   "+proj=poly +lat_0=0 +lon_0=-54 +x_0=5000000 +y_0=1000000");
    add_proj4text (p, 1, "0 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / Brazil Polyconic\",GEOGCS[\"SAD69\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"South_American_Datum_1969\",SPHEROID[\"GRS 1967 Mod");
    add_srs_wkt (p, 2,
		 "ified\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "618\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Polyconic\"],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 8,
		 ",0],PARAMETER[\"central_meridian\",-54],PARAMETER[\"fals");
    add_srs_wkt (p, 9,
		 "e_easting\",5000000],PARAMETER[\"false_northing\",100000");
    add_srs_wkt (p, 10,
		 "00],AUTHORITY[\"EPSG\",\"29101\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 11, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 29118, "epsg", 29118,
		      "SAD69 / UTM zone 18N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=GRS67 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 18N (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-75],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"29118\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 29119, "epsg", 29119,
		      "SAD69 / UTM zone 19N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=19 +ellps=GRS67 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 19N (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-69],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"29119\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 29120, "epsg", 29120,
		      "SAD69 / UTM zone 20N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=GRS67 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 20N (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-63],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"29120\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 29121, "epsg", 29121,
		      "SAD69 / UTM zone 21N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=21 +ellps=GRS67 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 21N (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-57],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"29121\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 29122, "epsg", 29122,
		      "SAD69 / UTM zone 22N (deprecated)");
    add_proj4text (p, 0, "+proj=utm +zone=22 +ellps=GRS67 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 22N (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-51],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"29122\"],AXIS[\"Easting\",EAST],AXIS[\"Northi");
    add_srs_wkt (p, 12, "ng\",NORTH]]");
    p = add_epsg_def (first, last, 29168, "epsg", 29168,
		      "SAD69 / UTM zone 18N");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 18N\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-75],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "29168\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 29169, "epsg", 29169,
		      "SAD69 / UTM zone 19N");
    add_proj4text (p, 0, "+proj=utm +zone=19 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 19N\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-69],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "29169\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 29170, "epsg", 29170,
		      "SAD69 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 20N\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-63],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "29170\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 29171, "epsg", 29171,
		      "SAD69 / UTM zone 21N");
    add_proj4text (p, 0, "+proj=utm +zone=21 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 21N\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-57],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "29171\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 29172, "epsg", 29172,
		      "SAD69 / UTM zone 22N");
    add_proj4text (p, 0, "+proj=utm +zone=22 +ellps=aust_SA +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 22N\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-51],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "29172\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 29177, "epsg", 29177,
		      "SAD69 / UTM zone 17S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 17S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-81],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29177\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29178, "epsg", 29178,
		      "SAD69 / UTM zone 18S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 18S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-75],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29178\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29179, "epsg", 29179,
		      "SAD69 / UTM zone 19S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 19S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-69],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29179\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29180, "epsg", 29180,
		      "SAD69 / UTM zone 20S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 20S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-63],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29180\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29181, "epsg", 29181,
		      "SAD69 / UTM zone 21S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 21S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-57],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29181\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29182, "epsg", 29182,
		      "SAD69 / UTM zone 22S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 22S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-51],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29182\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29183, "epsg", 29183,
		      "SAD69 / UTM zone 23S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 23S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-45],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29183\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29184, "epsg", 29184,
		      "SAD69 / UTM zone 24S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 24S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-39],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29184\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29185, "epsg", 29185,
		      "SAD69 / UTM zone 25S (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=GRS67 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 25S (deprecated)\",GEOGCS[\"SA");
    add_srs_wkt (p, 1,
		 "D69\",DATUM[\"South_American_Datum_1969\",SPHEROID[\"GRS");
    add_srs_wkt (p, 2,
		 " 1967\",6378160,298.247167427,AUTHORITY[\"EPSG\",\"7036\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6291\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "25199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4291\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-33],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",500000],PARAMETER[\"false_northing\",10000000],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"29185\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29187, "epsg", 29187,
		      "SAD69 / UTM zone 17S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 17S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-81],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29187\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29188, "epsg", 29188,
		      "SAD69 / UTM zone 18S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 18S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-75],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29188\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29189, "epsg", 29189,
		      "SAD69 / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 19S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-69],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29189\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29190, "epsg", 29190,
		      "SAD69 / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 20S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-63],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29190\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29191, "epsg", 29191,
		      "SAD69 / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 21S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-57],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29191\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29192, "epsg", 29192,
		      "SAD69 / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 22S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-51],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29192\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29193, "epsg", 29193,
		      "SAD69 / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 23S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-45],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29193\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29194, "epsg", 29194,
		      "SAD69 / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 24S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-39],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29194\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29195, "epsg", 29195,
		      "SAD69 / UTM zone 25S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=aust_SA +units=m +no_de");
    add_proj4text (p, 1, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SAD69 / UTM zone 25S\",GEOGCS[\"SAD69\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "South_American_Datum_1969\",SPHEROID[\"GRS 1967 Modified");
    add_srs_wkt (p, 2,
		 "\",6378160,298.25,AUTHORITY[\"EPSG\",\"7050\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6618\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4618\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",-33],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9996],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",10000000],AUTHORITY[\"EP");
    add_srs_wkt (p, 11,
		 "SG\",\"29195\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 29220, "epsg", 29220,
		      "Sapper Hill 1943 / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=intl +towgs84=-355,21,7");
    add_proj4text (p, 1, "2,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sapper Hill 1943 / UTM zone 20S\",GEOGCS[\"Sapp");
    add_srs_wkt (p, 1,
		 "er Hill 1943\",DATUM[\"Sapper_Hill_1943\",SPHEROID[\"Int");
    add_srs_wkt (p, 2,
		 "ernational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-355,21,72,0,0,0,0],AUTHORITY[\"EPSG\",\"6292");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4292\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-63],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"29220\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29221, "epsg", 29221,
		      "Sapper Hill 1943 / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=intl +towgs84=-355,21,7");
    add_proj4text (p, 1, "2,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sapper Hill 1943 / UTM zone 21S\",GEOGCS[\"Sapp");
    add_srs_wkt (p, 1,
		 "er Hill 1943\",DATUM[\"Sapper_Hill_1943\",SPHEROID[\"Int");
    add_srs_wkt (p, 2,
		 "ernational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[-355,21,72,0,0,0,0],AUTHORITY[\"EPSG\",\"6292");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4292\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-57],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"29221\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29333, "epsg", 29333,
		      "Schwarzeck / UTM zone 33S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +south +ellps=bess_nam +units=m +no_d");
    add_proj4text (p, 1, "efs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Schwarzeck / UTM zone 33S\",GEOGCS[\"Schwarzeck");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Schwarzeck\",SPHEROID[\"Bessel Namibia (GLM)\"");
    add_srs_wkt (p, 2,
		 ",6377483.865280419,299.1528128,AUTHORITY[\"EPSG\",\"7046");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6293\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4293\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 7,
		 "01\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"l");
    add_srs_wkt (p, 8,
		 "atitude_of_origin\",0],PARAMETER[\"central_meridian\",15");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 10,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"29333\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29635, "epsg", 29635,
		      "Sudan / UTM zone 35N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +a=6378249.2 +b=6356515 +units=m +no_");
    add_proj4text (p, 1, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sudan / UTM zone 35N (deprecated)\",GEOGCS[\"Su");
    add_srs_wkt (p, 1,
		 "dan\",DATUM[\"Sudan\",SPHEROID[\"Clarke 1880 (IGN)\",637");
    add_srs_wkt (p, 2,
		 "8249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6296\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4296\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",27],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"29635\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 29636, "epsg", 29636,
		      "Sudan / UTM zone 36N (deprecated)");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +a=6378249.2 +b=6356515 +units=m +no_");
    add_proj4text (p, 1, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Sudan / UTM zone 36N (deprecated)\",GEOGCS[\"Su");
    add_srs_wkt (p, 1,
		 "dan\",DATUM[\"Sudan\",SPHEROID[\"Clarke 1880 (IGN)\",637");
    add_srs_wkt (p, 2,
		 "8249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"7011\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6296\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "9433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4296\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",33],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 11,
		 "G\",\"29636\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 29700, "epsg", 29700,
		      "Tananarive (Paris) / Laborde Grid (deprecated)");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=-18.9 +lonc=44.10000000000001 +alpha=");
    add_proj4text (p, 1,
		   "18.9 +k=0.9995000000000001 +x_0=400000 +y_0=800000 +ellp");
    add_proj4text (p, 2,
		   "s=intl +towgs84=-189,-242,-91,0,0,0,0 +pm=paris +units=m");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tananarive (Paris) / Laborde Grid (deprecated)\"");
    add_srs_wkt (p, 1,
		 ",GEOGCS[\"Tananarive (Paris)\",DATUM[\"Tananarive_1925_P");
    add_srs_wkt (p, 2,
		 "aris\",SPHEROID[\"International 1924\",6378388,297,AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"7022\"]],TOWGS84[-189,-242,-91,0,0,0,0],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6810\"]],PRIMEM[\"Paris\",2.3372291");
    add_srs_wkt (p, 5,
		 "7,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.01570796");
    add_srs_wkt (p, 6,
		 "326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"4810\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 8,
		 "\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "latitude_of_center\",-21],PARAMETER[\"longitude_of_cente");
    add_srs_wkt (p, 10,
		 "r\",49],PARAMETER[\"azimuth\",21],PARAMETER[\"rectified_");
    add_srs_wkt (p, 11,
		 "grid_angle\",21],PARAMETER[\"scale_factor\",0.9995],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_easting\",400000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 13,
		 "ng\",800000],AUTHORITY[\"EPSG\",\"29700\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 14, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 29702, "epsg", 29702,
		      "Tananarive (Paris) / Laborde Grid approximation");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=-18.9 +lonc=44.10000000000001 +alpha=");
    add_proj4text (p, 1,
		   "18.9 +k=0.9995000000000001 +x_0=400000 +y_0=800000 +ellp");
    add_proj4text (p, 2,
		   "s=intl +towgs84=-189,-242,-91,0,0,0,0 +pm=paris +units=m");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tananarive (Paris) / Laborde Grid approximation");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"Tananarive (Paris)\",DATUM[\"Tananarive_1925");
    add_srs_wkt (p, 2,
		 "_Paris\",SPHEROID[\"International 1924\",6378388,297,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7022\"]],TOWGS84[-189,-242,-91,0,0,0,0");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6810\"]],PRIMEM[\"Paris\",2.33722");
    add_srs_wkt (p, 5,
		 "917,AUTHORITY[\"EPSG\",\"8903\"]],UNIT[\"grad\",0.015707");
    add_srs_wkt (p, 6,
		 "96326794897,AUTHORITY[\"EPSG\",\"9105\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"4810\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"90");
    add_srs_wkt (p, 8,
		 "01\"]],PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"latitude_of_center\",-21],PARAMETER[\"longitude_of_ce");
    add_srs_wkt (p, 10,
		 "nter\",49],PARAMETER[\"azimuth\",21],PARAMETER[\"rectifi");
    add_srs_wkt (p, 11,
		 "ed_grid_angle\",21],PARAMETER[\"scale_factor\",0.9995],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_easting\",400000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 13,
		 "thing\",800000],AUTHORITY[\"EPSG\",\"29702\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 29738, "epsg", 29738,
		      "Tananarive / UTM zone 38S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +south +ellps=intl +towgs84=-189,-242");
    add_proj4text (p, 1, ",-91,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tananarive / UTM zone 38S\",GEOGCS[\"Tananarive");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Tananarive_1925\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-");
    add_srs_wkt (p, 3,
		 "189,-242,-91,0,0,0,0],AUTHORITY[\"EPSG\",\"6297\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4297\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",45],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"29738\"],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29739, "epsg", 29739,
		      "Tananarive / UTM zone 39S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +south +ellps=intl +towgs84=-189,-242");
    add_proj4text (p, 1, ",-91,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tananarive / UTM zone 39S\",GEOGCS[\"Tananarive");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"Tananarive_1925\",SPHEROID[\"International 19");
    add_srs_wkt (p, 2,
		 "24\",6378388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-");
    add_srs_wkt (p, 3,
		 "189,-242,-91,0,0,0,0],AUTHORITY[\"EPSG\",\"6297\"]],PRIM");
    add_srs_wkt (p, 4,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4297\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",51],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 11,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"29739\"],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29849, "epsg", 29849,
		      "Timbalai 1948 / UTM zone 49N");
    add_proj4text (p, 0, "+proj=utm +zone=49 +ellps=evrstSS +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Timbalai 1948 / UTM zone 49N\",GEOGCS[\"Timbala");
    add_srs_wkt (p, 1,
		 "i 1948\",DATUM[\"Timbalai_1948\",SPHEROID[\"Everest 1830");
    add_srs_wkt (p, 2,
		 " (1967 Definition)\",6377298.556,300.8017,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7016\"]],AUTHORITY[\"EPSG\",\"6298\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4298\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",111],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"29849\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29850, "epsg", 29850,
		      "Timbalai 1948 / UTM zone 50N");
    add_proj4text (p, 0, "+proj=utm +zone=50 +ellps=evrstSS +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Timbalai 1948 / UTM zone 50N\",GEOGCS[\"Timbala");
    add_srs_wkt (p, 1,
		 "i 1948\",DATUM[\"Timbalai_1948\",SPHEROID[\"Everest 1830");
    add_srs_wkt (p, 2,
		 " (1967 Definition)\",6377298.556,300.8017,AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"7016\"]],AUTHORITY[\"EPSG\",\"6298\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4298\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",117],PARAMETER[\"scale_factor\",0.9996],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"29850\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29871, "epsg", 29871,
		      "Timbalai 1948 / RSO Borneo (ch)");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=115 +alpha=53.31582047222222 ");
    add_proj4text (p, 1,
		   "+k=0.99984 +x_0=590476.8714630401 +y_0=442857.653094361 ");
    add_proj4text (p, 2, "+ellps=evrstSS +to_meter=20.11676512155263 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Timbalai 1948 / RSO Borneo (ch)\",GEOGCS[\"Timb");
    add_srs_wkt (p, 1,
		 "alai 1948\",DATUM[\"Timbalai_1948\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1967 Definition)\",6377298.556,300.8017,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7016\"]],AUTHORITY[\"EPSG\",\"6298\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4298\"]],UNIT[\"British chain (Sears ");
    add_srs_wkt (p, 7,
		 "1922)\",20.11676512155263,AUTHORITY[\"EPSG\",\"9042\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETER[\"lati");
    add_srs_wkt (p, 9,
		 "tude_of_center\",4],PARAMETER[\"longitude_of_center\",11");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"azimuth\",53.31582047222222],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "rectified_grid_angle\",53.13010236111111],PARAMETER[\"sc");
    add_srs_wkt (p, 12,
		 "ale_factor\",0.99984],PARAMETER[\"false_easting\",29352.");
    add_srs_wkt (p, 13,
		 "4763],PARAMETER[\"false_northing\",22014.3572],AUTHORITY");
    add_srs_wkt (p, 14,
		 "[\"EPSG\",\"29871\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 15, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 29872, "epsg", 29872,
		      "Timbalai 1948 / RSO Borneo (ft)");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=115 +alpha=53.31582047222222 ");
    add_proj4text (p, 1,
		   "+k=0.99984 +x_0=590476.8727431979 +y_0=442857.6545573985");
    add_proj4text (p, 2,
		   " +ellps=evrstSS +to_meter=0.3047994715386762 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Timbalai 1948 / RSO Borneo (ft)\",GEOGCS[\"Timb");
    add_srs_wkt (p, 1,
		 "alai 1948\",DATUM[\"Timbalai_1948\",SPHEROID[\"Everest 1");
    add_srs_wkt (p, 2,
		 "830 (1967 Definition)\",6377298.556,300.8017,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7016\"]],AUTHORITY[\"EPSG\",\"6298\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4298\"]],UNIT[\"British foot (Sears 1");
    add_srs_wkt (p, 7,
		 "922)\",0.3047994715386762,AUTHORITY[\"EPSG\",\"9041\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Hotine_Oblique_Mercator\"],PARAMETER[\"lati");
    add_srs_wkt (p, 9,
		 "tude_of_center\",4],PARAMETER[\"longitude_of_center\",11");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"azimuth\",53.31582047222222],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "rectified_grid_angle\",53.13010236111111],PARAMETER[\"sc");
    add_srs_wkt (p, 12,
		 "ale_factor\",0.99984],PARAMETER[\"false_easting\",193726");
    add_srs_wkt (p, 13,
		 "3.44],PARAMETER[\"false_northing\",1452947.58],AUTHORITY");
    add_srs_wkt (p, 14,
		 "[\"EPSG\",\"29872\"],AXIS[\"Easting\",EAST],AXIS[\"North");
    add_srs_wkt (p, 15, "ing\",NORTH]]");
    p = add_epsg_def (first, last, 29873, "epsg", 29873,
		      "Timbalai 1948 / RSO Borneo (m)");
    add_proj4text (p, 0,
		   "+proj=omerc +lat_0=4 +lonc=115 +alpha=53.31582047222222 ");
    add_proj4text (p, 1,
		   "+k=0.99984 +x_0=590476.87 +y_0=442857.65 +ellps=evrstSS ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Timbalai 1948 / RSO Borneo (m)\",GEOGCS[\"Timba");
    add_srs_wkt (p, 1,
		 "lai 1948\",DATUM[\"Timbalai_1948\",SPHEROID[\"Everest 18");
    add_srs_wkt (p, 2,
		 "30 (1967 Definition)\",6377298.556,300.8017,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7016\"]],AUTHORITY[\"EPSG\",\"6298\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4298\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Hotine_Oblique_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_center\",4],PARAMETER[\"longit");
    add_srs_wkt (p, 9,
		 "ude_of_center\",115],PARAMETER[\"azimuth\",53.3158204722");
    add_srs_wkt (p, 10,
		 "2222],PARAMETER[\"rectified_grid_angle\",53.130102361111");
    add_srs_wkt (p, 11,
		 "11],PARAMETER[\"scale_factor\",0.99984],PARAMETER[\"fals");
    add_srs_wkt (p, 12,
		 "e_easting\",590476.87],PARAMETER[\"false_northing\",4428");
    add_srs_wkt (p, 13,
		 "57.65],AUTHORITY[\"EPSG\",\"29873\"],AXIS[\"Easting\",EA");
    add_srs_wkt (p, 14, "ST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29900, "epsg", 29900,
		      "TM65 / Irish National Grid (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=53.5 +lon_0=-8 +k=1.000035 +x_0=20000");
    add_proj4text (p, 1,
		   "0 +y_0=250000 +ellps=mod_airy +datum=ire65 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TM65 / Irish National Grid (deprecated)\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"TM65\",DATUM[\"TM65\",SPHEROID[\"Airy Modified 1849\"");
    add_srs_wkt (p, 2,
		 ",6377340.189,299.3249646,AUTHORITY[\"EPSG\",\"7002\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6299\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4299\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",53.5],PARAMETER[\"central_meridian\",-8],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",1.000035],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",200000],PARAMETER[\"false_northing\",250000],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"29900\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29901, "epsg", 29901,
		      "OSNI 1952 / Irish National Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=53.5 +lon_0=-8 +k=1 +x_0=200000 +y_0=");
    add_proj4text (p, 1,
		   "250000 +ellps=airy +towgs84=482.5,-130.6,564.6,-1.042,-0");
    add_proj4text (p, 2, ".214,-0.631,8.15 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"OSNI 1952 / Irish National Grid\",GEOGCS[\"OSNI");
    add_srs_wkt (p, 1,
		 " 1952\",DATUM[\"OSNI_1952\",SPHEROID[\"Airy 1830\",63775");
    add_srs_wkt (p, 2,
		 "63.396,299.3249646,AUTHORITY[\"EPSG\",\"7001\"]],TOWGS84");
    add_srs_wkt (p, 3,
		 "[482.5,-130.6,564.6,-1.042,-0.214,-0.631,8.15],AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"6188\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4188\"]");
    add_srs_wkt (p, 7,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 8,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 9,
		 "gin\",53.5],PARAMETER[\"central_meridian\",-8],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",200000]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",250000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12,
		 ",\"29901\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 29902, "epsg", 29902, "TM65 / Irish Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=53.5 +lon_0=-8 +k=1.000035 +x_0=20000");
    add_proj4text (p, 1,
		   "0 +y_0=250000 +ellps=mod_airy +datum=ire65 +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TM65 / Irish Grid\",GEOGCS[\"TM65\",DATUM[\"TM6");
    add_srs_wkt (p, 1,
		 "5\",SPHEROID[\"Airy Modified 1849\",6377340.189,299.3249");
    add_srs_wkt (p, 2,
		 "646,AUTHORITY[\"EPSG\",\"7002\"]],AUTHORITY[\"EPSG\",\"6");
    add_srs_wkt (p, 3,
		 "299\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901");
    add_srs_wkt (p, 4,
		 "\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"9122\"]],AUTHORITY[\"EPSG\",\"4299\"]],UNIT[\"metr");
    add_srs_wkt (p, 6,
		 "e\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transve");
    add_srs_wkt (p, 7,
		 "rse_Mercator\"],PARAMETER[\"latitude_of_origin\",53.5],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",-8],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 9,
		 "or\",1.000035],PARAMETER[\"false_easting\",200000],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_northing\",250000],AUTHORITY[\"EPSG\",\"299");
    add_srs_wkt (p, 11,
		 "02\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 29903, "epsg", 29903, "TM75 / Irish Grid");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=53.5 +lon_0=-8 +k=1.000035 +x_0=20000");
    add_proj4text (p, 1, "0 +y_0=250000 +ellps=mod_airy +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TM75 / Irish Grid\",GEOGCS[\"TM75\",DATUM[\"Geo");
    add_srs_wkt (p, 1,
		 "detic_Datum_of_1965\",SPHEROID[\"Airy Modified 1849\",63");
    add_srs_wkt (p, 2,
		 "77340.189,299.3249646,AUTHORITY[\"EPSG\",\"7002\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6300\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"43");
    add_srs_wkt (p, 6,
		 "00\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 8,
		 "f_origin\",53.5],PARAMETER[\"central_meridian\",-8],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",1.000035],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",200000],PARAMETER[\"false_northing\",250000],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"29903\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 12, "rthing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_24 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 30161, "epsg", 30161,
		      "Tokyo / Japan Plane Rectangular CS I");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=33 +lon_0=129.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1, "=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS I\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377397");
    add_srs_wkt (p, 2,
		 ".155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",33],PARAMETER[\"central_meridian\",129.5],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",0");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 11, "0161\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30162, "epsg", 30162,
		      "Tokyo / Japan Plane Rectangular CS II");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=33 +lon_0=131 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS II\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",63773");
    add_srs_wkt (p, 2,
		 "97.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",33],PARAMETER[\"central_meridian\",131],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",0]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "162\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30163, "epsg", 30163,
		      "Tokyo / Japan Plane Rectangular CS III");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=132.1666666666667 +k=0.9999");
    add_proj4text (p, 1, " +x_0=0 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS III\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377");
    add_srs_wkt (p, 2,
		 "397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",36],PARAMETER[\"central_meridian\",132.16666666");
    add_srs_wkt (p, 9,
		 "66667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",0],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"30163\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 30164, "epsg", 30164,
		      "Tokyo / Japan Plane Rectangular CS IV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=33 +lon_0=133.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1, "=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS IV\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",63773");
    add_srs_wkt (p, 2,
		 "97.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",33],PARAMETER[\"central_meridian\",133.5],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30164\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30165, "epsg", 30165,
		      "Tokyo / Japan Plane Rectangular CS V");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=134.3333333333333 +k=0.9999");
    add_proj4text (p, 1, " +x_0=0 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS V\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377397");
    add_srs_wkt (p, 2,
		 ".155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",36],PARAMETER[\"central_meridian\",134.33333333333");
    add_srs_wkt (p, 9,
		 "33],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"30165\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30166, "epsg", 30166,
		      "Tokyo / Japan Plane Rectangular CS VI");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=136 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS VI\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",63773");
    add_srs_wkt (p, 2,
		 "97.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",36],PARAMETER[\"central_meridian\",136],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",0]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30");
    add_srs_wkt (p, 11, "166\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30167, "epsg", 30167,
		      "Tokyo / Japan Plane Rectangular CS VII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=137.1666666666667 +k=0.9999");
    add_proj4text (p, 1, " +x_0=0 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS VII\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377");
    add_srs_wkt (p, 2,
		 "397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",36],PARAMETER[\"central_meridian\",137.16666666");
    add_srs_wkt (p, 9,
		 "66667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",0],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 11,
		 "TY[\"EPSG\",\"30167\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST");
    add_srs_wkt (p, 12, "]]");
    p = add_epsg_def (first, last, 30168, "epsg", 30168,
		      "Tokyo / Japan Plane Rectangular CS VIII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=138.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1, "=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS VIII\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",637");
    add_srs_wkt (p, 2,
		 "7397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"430");
    add_srs_wkt (p, 6,
		 "1\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",36],PARAMETER[\"central_meridian\",138.5],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9999],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"30168\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30169, "epsg", 30169,
		      "Tokyo / Japan Plane Rectangular CS IX");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=36 +lon_0=139.8333333333333 +k=0.9999");
    add_proj4text (p, 1, " +x_0=0 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS IX\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",63773");
    add_srs_wkt (p, 2,
		 "97.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",36],PARAMETER[\"central_meridian\",139.8333333333");
    add_srs_wkt (p, 9,
		 "333],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"30169\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 30170, "epsg", 30170,
		      "Tokyo / Japan Plane Rectangular CS X");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=140.8333333333333 +k=0.9999");
    add_proj4text (p, 1, " +x_0=0 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS X\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377397");
    add_srs_wkt (p, 2,
		 ".155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",40],PARAMETER[\"central_meridian\",140.83333333333");
    add_srs_wkt (p, 9,
		 "33],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"30170\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30171, "epsg", 30171,
		      "Tokyo / Japan Plane Rectangular CS XI");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=44 +lon_0=140.25 +k=0.9999 +x_0=0 +y_");
    add_proj4text (p, 1, "0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XI\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",63773");
    add_srs_wkt (p, 2,
		 "97.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",44],PARAMETER[\"central_meridian\",140.25],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30171\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30172, "epsg", 30172,
		      "Tokyo / Japan Plane Rectangular CS XII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=44 +lon_0=142.25 +k=0.9999 +x_0=0 +y_");
    add_proj4text (p, 1, "0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XII\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377");
    add_srs_wkt (p, 2,
		 "397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",44],PARAMETER[\"central_meridian\",142.25],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9999],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"30172\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30173, "epsg", 30173,
		      "Tokyo / Japan Plane Rectangular CS XIII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=44 +lon_0=144.25 +k=0.9999 +x_0=0 +y_");
    add_proj4text (p, 1, "0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XIII\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",637");
    add_srs_wkt (p, 2,
		 "7397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"430");
    add_srs_wkt (p, 6,
		 "1\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",44],PARAMETER[\"central_meridian\",144.25],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"scale_factor\",0.9999],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 10,
		 "g\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 11, ",\"30173\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30174, "epsg", 30174,
		      "Tokyo / Japan Plane Rectangular CS XIV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=142 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XIV\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377");
    add_srs_wkt (p, 2,
		 "397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",26],PARAMETER[\"central_meridian\",142],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30174\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30175, "epsg", 30175,
		      "Tokyo / Japan Plane Rectangular CS XV");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=127.5 +k=0.9999 +x_0=0 +y_0");
    add_proj4text (p, 1, "=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XV\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",63773");
    add_srs_wkt (p, 2,
		 "97.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",26],PARAMETER[\"central_meridian\",127.5],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30175\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30176, "epsg", 30176,
		      "Tokyo / Japan Plane Rectangular CS XVI");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=124 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XVI\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377");
    add_srs_wkt (p, 2,
		 "397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",26],PARAMETER[\"central_meridian\",124],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30176\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30177, "epsg", 30177,
		      "Tokyo / Japan Plane Rectangular CS XVII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=131 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XVII\",GEOGC");
    add_srs_wkt (p, 1,
		 "S[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",637");
    add_srs_wkt (p, 2,
		 "7397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"430");
    add_srs_wkt (p, 6,
		 "1\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",26],PARAMETER[\"central_meridian\",131],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30177\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30178, "epsg", 30178,
		      "Tokyo / Japan Plane Rectangular CS XVIII");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=20 +lon_0=136 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XVIII\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",63");
    add_srs_wkt (p, 2,
		 "77397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"43");
    add_srs_wkt (p, 6,
		 "01\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 8,
		 "f_origin\",20],PARAMETER[\"central_meridian\",136],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 10,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30178\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30179, "epsg", 30179,
		      "Tokyo / Japan Plane Rectangular CS XIX");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=26 +lon_0=154 +k=0.9999 +x_0=0 +y_0=0");
    add_proj4text (p, 1, " +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Tokyo / Japan Plane Rectangular CS XIX\",GEOGCS");
    add_srs_wkt (p, 1,
		 "[\"Tokyo\",DATUM[\"Tokyo\",SPHEROID[\"Bessel 1841\",6377");
    add_srs_wkt (p, 2,
		 "397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6301\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4301");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",26],PARAMETER[\"central_meridian\",154],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"scale_factor\",0.9999],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "30179\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 30200, "epsg", 30200,
		      "Trinidad 1903 / Trinidad Grid");
    add_proj4text (p, 0,
		   "+proj=cass +lat_0=10.44166666666667 +lon_0=-61.333333333");
    add_proj4text (p, 1,
		   "33334 +x_0=86501.46392051999 +y_0=65379.0134283 +a=63782");
    add_proj4text (p, 2,
		   "93.645208759 +b=6356617.987679838 +to_meter=0.2011661951");
    add_proj4text (p, 3, "64 +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Trinidad 1903 / Trinidad Grid\",GEOGCS[\"Trinid");
    add_srs_wkt (p, 1,
		 "ad 1903\",DATUM[\"Trinidad_1903\",SPHEROID[\"Clarke 1858");
    add_srs_wkt (p, 2,
		 "\",6378293.645208759,294.2606763692569,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7007\"]],AUTHORITY[\"EPSG\",\"6302\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4302\"]],UNIT[\"Clarke's link\",0.20116619");
    add_srs_wkt (p, 7,
		 "5164,AUTHORITY[\"EPSG\",\"9039\"]],PROJECTION[\"Cassini_");
    add_srs_wkt (p, 8,
		 "Soldner\"],PARAMETER[\"latitude_of_origin\",10.441666666");
    add_srs_wkt (p, 9,
		 "66667],PARAMETER[\"central_meridian\",-61.33333333333334");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",430000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",325000],AUTHORITY[\"EPSG\",\"30200\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 30339, "epsg", 30339,
		      "TC(1948) / UTM zone 39N");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=helmert +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TC(1948) / UTM zone 39N\",GEOGCS[\"TC(1948)\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Trucial_Coast_1948\",SPHEROID[\"Helmert 1906\",63");
    add_srs_wkt (p, 2,
		 "78200,298.3,AUTHORITY[\"EPSG\",\"7020\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6303\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4303\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",51],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",0.9996],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30339\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 30340, "epsg", 30340,
		      "TC(1948) / UTM zone 40N");
    add_proj4text (p, 0, "+proj=utm +zone=40 +ellps=helmert +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"TC(1948) / UTM zone 40N\",GEOGCS[\"TC(1948)\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Trucial_Coast_1948\",SPHEROID[\"Helmert 1906\",63");
    add_srs_wkt (p, 2,
		 "78200,298.3,AUTHORITY[\"EPSG\",\"7020\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6303\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4303\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",57],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",0.9996],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"30340\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 30491, "epsg", 30491,
		      "Voirol 1875 / Nord Algerie (ancienne)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36 +lat_0=36 +lon_0=2.7 +k_0=0.99962554");
    add_proj4text (p, 1,
		   "4 +x_0=500000 +y_0=300000 +a=6378249.2 +b=6356515 +towgs");
    add_proj4text (p, 2, "84=-73,-247,227,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Voirol 1875 / Nord Algerie (ancienne)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Voirol 1875\",DATUM[\"Voirol_1875\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7011\"]],TOWGS84[-73,-247,227,0,0,0,0],AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"6304\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 5,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4304\"]],U");
    add_srs_wkt (p, 7,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 8,
		 "[\"Lambert_Conformal_Conic_1SP\"],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 9,
		 "f_origin\",36],PARAMETER[\"central_meridian\",2.7],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.999625544],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",500000],PARAMETER[\"false_northing\",300000],AUTH");
    add_srs_wkt (p, 12,
		 "ORITY[\"EPSG\",\"30491\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 13, "RTH]]");
    p = add_epsg_def (first, last, 30492, "epsg", 30492,
		      "Voirol 1875 / Sud Algerie (ancienne)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.3 +lat_0=33.3 +lon_0=2.7 +k_0=0.9996");
    add_proj4text (p, 1,
		   "25769 +x_0=500000 +y_0=300000 +a=6378249.2 +b=6356515 +t");
    add_proj4text (p, 2, "owgs84=-73,-247,227,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Voirol 1875 / Sud Algerie (ancienne)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Voirol 1875\",DATUM[\"Voirol_1875\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "80 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7011\"]],TOWGS84[-73,-247,227,0,0,0,0],AUTHORITY[\"EP");
    add_srs_wkt (p, 4,
		 "SG\",\"6304\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4304\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Lambert_Conformal_Conic_1SP\"],PARAMETER[\"latitude_of_o");
    add_srs_wkt (p, 9,
		 "rigin\",33.3],PARAMETER[\"central_meridian\",2.7],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",0.999625769],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",500000],PARAMETER[\"false_northing\",300000],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"30492\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 13, "TH]]");
    p = add_epsg_def (first, last, 30493, "epsg", 30493,
		      "Voirol 1879 / Nord Algerie (ancienne)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36 +lat_0=36 +lon_0=2.7 +k_0=0.99962554");
    add_proj4text (p, 1,
		   "4 +x_0=500000 +y_0=300000 +a=6378249.2 +b=6356515 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Voirol 1879 / Nord Algerie (ancienne)\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Voirol 1879\",DATUM[\"Voirol_1879\",SPHEROID[\"Clarke ");
    add_srs_wkt (p, 2,
		 "1880 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7011\"]],AUTHORITY[\"EPSG\",\"6671\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4671\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",36],PARAMETER[\"cen");
    add_srs_wkt (p, 9,
		 "tral_meridian\",2.7],PARAMETER[\"scale_factor\",0.999625");
    add_srs_wkt (p, 10,
		 "544],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",300000],AUTHORITY[\"EPSG\",\"30493\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 30494, "epsg", 30494,
		      "Voirol 1879 / Sud Algerie (ancienne)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.3 +lat_0=33.3 +lon_0=2.7 +k_0=0.9996");
    add_proj4text (p, 1,
		   "25769 +x_0=500000 +y_0=300000 +a=6378249.2 +b=6356515 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Voirol 1879 / Sud Algerie (ancienne)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Voirol 1879\",DATUM[\"Voirol_1879\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "80 (IGN)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7011\"]],AUTHORITY[\"EPSG\",\"6671\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4671\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",33.3],PARAMETER[\"cen");
    add_srs_wkt (p, 9,
		 "tral_meridian\",2.7],PARAMETER[\"scale_factor\",0.999625");
    add_srs_wkt (p, 10,
		 "769],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",300000],AUTHORITY[\"EPSG\",\"30494\"],AXIS");
    add_srs_wkt (p, 12, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 30729, "epsg", 30729,
		      "Nord Sahara 1959 / UTM zone 29N");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nord Sahara 1959 / UTM zone 29N\",GEOGCS[\"Nord");
    add_srs_wkt (p, 1,
		 " Sahara 1959\",DATUM[\"Nord_Sahara_1959\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7012\"]],AUTHORITY[\"EPSG\",\"6307\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4307\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",500000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 11,
		 "THORITY[\"EPSG\",\"30729\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 30730, "epsg", 30730,
		      "Nord Sahara 1959 / UTM zone 30N");
    add_proj4text (p, 0, "+proj=utm +zone=30 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nord Sahara 1959 / UTM zone 30N\",GEOGCS[\"Nord");
    add_srs_wkt (p, 1,
		 " Sahara 1959\",DATUM[\"Nord_Sahara_1959\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7012\"]],AUTHORITY[\"EPSG\",\"6307\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4307\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",500000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 11,
		 "THORITY[\"EPSG\",\"30730\"],AXIS[\"Easting\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 30731, "epsg", 30731,
		      "Nord Sahara 1959 / UTM zone 31N");
    add_proj4text (p, 0, "+proj=utm +zone=31 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nord Sahara 1959 / UTM zone 31N\",GEOGCS[\"Nord");
    add_srs_wkt (p, 1,
		 " Sahara 1959\",DATUM[\"Nord_Sahara_1959\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7012\"]],AUTHORITY[\"EPSG\",\"6307\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4307\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",3],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",500000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"30731\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 30732, "epsg", 30732,
		      "Nord Sahara 1959 / UTM zone 32N");
    add_proj4text (p, 0, "+proj=utm +zone=32 +ellps=clrk80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nord Sahara 1959 / UTM zone 32N\",GEOGCS[\"Nord");
    add_srs_wkt (p, 1,
		 " Sahara 1959\",DATUM[\"Nord_Sahara_1959\",SPHEROID[\"Cla");
    add_srs_wkt (p, 2,
		 "rke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 3,
		 "\"7012\"]],AUTHORITY[\"EPSG\",\"6307\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4307\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",9],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",500000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"30732\"],AXIS[\"Easting\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Northing\",NORTH]]");
    p = add_epsg_def (first, last, 30791, "epsg", 30791,
		      "Nord Sahara 1959 / Voirol Unifie Nord");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36 +lat_0=36 +lon_0=2.7 +k_0=0.99962554");
    add_proj4text (p, 1,
		   "4 +x_0=500135 +y_0=300090 +ellps=clrk80 +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nord Sahara 1959 / Voirol Unifie Nord\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"Nord Sahara 1959\",DATUM[\"Nord_Sahara_1959\",SPHEROID");
    add_srs_wkt (p, 2,
		 "[\"Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"7012\"]],AUTHORITY[\"EPSG\",\"6307\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4307\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1");
    add_srs_wkt (p, 8,
		 "SP\"],PARAMETER[\"latitude_of_origin\",36],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",2.7],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "25544],PARAMETER[\"false_easting\",500135],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",300090],AUTHORITY[\"EPSG\",\"30791\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 30792, "epsg", 30792,
		      "Nord Sahara 1959 / Voirol Unifie Sud");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.3 +lat_0=33.3 +lon_0=2.7 +k_0=0.9996");
    add_proj4text (p, 1,
		   "25769 +x_0=500135 +y_0=300090 +ellps=clrk80 +units=m +no");
    add_proj4text (p, 2, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Nord Sahara 1959 / Voirol Unifie Sud\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "Nord Sahara 1959\",DATUM[\"Nord_Sahara_1959\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Clarke 1880 (RGS)\",6378249.145,293.465,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7012\"]],AUTHORITY[\"EPSG\",\"6307\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 4,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4307\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 7,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",33.3],PARAMETER[\"c");
    add_srs_wkt (p, 9,
		 "entral_meridian\",2.7],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 10,
		 "25769],PARAMETER[\"false_easting\",500135],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",300090],AUTHORITY[\"EPSG\",\"30792\"],AX");
    add_srs_wkt (p, 12, "IS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 30800, "epsg", 30800,
		      "RT38 2.5 gon W (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15.80827777777778 +k=1 +x_0=");
    add_proj4text (p, 1, "1500000 +y_0=0 +ellps=bessel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"RT38 2.5 gon W (deprecated)\",GEOGCS[\"RT38\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Stockholm_1938\",SPHEROID[\"Bessel 1841\",6377397");
    add_srs_wkt (p, 2,
		 ".155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY");
    add_srs_wkt (p, 3,
		 "[\"EPSG\",\"6308\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4308\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",0],PARAMETER[\"central_meridian\",15.8082777777777");
    add_srs_wkt (p, 9,
		 "8],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",1500000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 11,
		 "\"EPSG\",\"30800\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31028, "epsg", 31028, "Yoff / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +a=6378249.2 +b=6356515 +units=m +no_");
    add_proj4text (p, 1, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Yoff / UTM zone 28N\",GEOGCS[\"Yoff\",DATUM[\"Y");
    add_srs_wkt (p, 1,
		 "off\",SPHEROID[\"Clarke 1880 (IGN)\",6378249.2,293.46602");
    add_srs_wkt (p, 2,
		 "12936265,AUTHORITY[\"EPSG\",\"7011\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"6310\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 5,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4310\"]],UNIT[\"m");
    add_srs_wkt (p, 6,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 7,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 8,
		 "ARAMETER[\"central_meridian\",-15],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 9,
		 "tor\",0.9996],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"31028\"],");
    add_srs_wkt (p, 11, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31121, "epsg", 31121,
		      "Zanderij / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=intl +towgs84=-265,120,-358,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Zanderij / UTM zone 21N\",GEOGCS[\"Zanderij\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Zanderij\",SPHEROID[\"International 1924\",637838");
    add_srs_wkt (p, 2,
		 "8,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-265,120,-35");
    add_srs_wkt (p, 3,
		 "8,0,0,0,0],AUTHORITY[\"EPSG\",\"6311\"]],PRIMEM[\"Greenw");
    add_srs_wkt (p, 4,
		 "ich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.");
    add_srs_wkt (p, 5,
		 "01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"4311\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-57],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",500000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"31121\"],AXIS[\"Easting\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31154, "epsg", 31154, "Zanderij / TM 54 NW");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-54 +k=0.9996 +x_0=500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +towgs84=-265,120,-358,0,0,0,0 +units=m");
    add_proj4text (p, 2, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Zanderij / TM 54 NW\",GEOGCS[\"Zanderij\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Zanderij\",SPHEROID[\"International 1924\",6378388,29");
    add_srs_wkt (p, 2,
		 "7,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-265,120,-358,0,");
    add_srs_wkt (p, 3,
		 "0,0,0],AUTHORITY[\"EPSG\",\"6311\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4311\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 7,
		 "001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "latitude_of_origin\",0],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "54],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"31154\"],AXIS[\"Easting\",EAST],AXIS[\"N");
    add_srs_wkt (p, 12, "orthing\",NORTH]]");
    p = add_epsg_def (first, last, 31170, "epsg", 31170,
		      "Zanderij / Suriname Old TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-55.68333333333333 +k=0.9996");
    add_proj4text (p, 1,
		   " +x_0=500000 +y_0=0 +ellps=intl +towgs84=-265,120,-358,0");
    add_proj4text (p, 2, ",0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Zanderij / Suriname Old TM\",GEOGCS[\"Zanderij\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"Zanderij\",SPHEROID[\"International 1924\",6378");
    add_srs_wkt (p, 2,
		 "388,297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-265,120,-");
    add_srs_wkt (p, 3,
		 "358,0,0,0,0],AUTHORITY[\"EPSG\",\"6311\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 4,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 5,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"4311\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAM");
    add_srs_wkt (p, 8,
		 "ETER[\"latitude_of_origin\",0],PARAMETER[\"central_merid");
    add_srs_wkt (p, 9,
		 "ian\",-55.68333333333333],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 10,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 11,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"31170\"],AXIS[\"Ea");
    add_srs_wkt (p, 12, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31171, "epsg", 31171,
		      "Zanderij / Suriname TM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-55.68333333333333 +k=0.9999");
    add_proj4text (p, 1,
		   " +x_0=500000 +y_0=0 +ellps=intl +towgs84=-265,120,-358,0");
    add_proj4text (p, 2, ",0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Zanderij / Suriname TM\",GEOGCS[\"Zanderij\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"Zanderij\",SPHEROID[\"International 1924\",6378388");
    add_srs_wkt (p, 2,
		 ",297,AUTHORITY[\"EPSG\",\"7022\"]],TOWGS84[-265,120,-358");
    add_srs_wkt (p, 3,
		 ",0,0,0,0],AUTHORITY[\"EPSG\",\"6311\"]],PRIMEM[\"Greenwi");
    add_srs_wkt (p, 4,
		 "ch\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0");
    add_srs_wkt (p, 5,
		 "1745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"4311\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETE");
    add_srs_wkt (p, 8,
		 "R[\"latitude_of_origin\",0],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 9,
		 "\",-55.68333333333333],PARAMETER[\"scale_factor\",0.9999");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "northing\",0],AUTHORITY[\"EPSG\",\"31171\"],AXIS[\"Easti");
    add_srs_wkt (p, 12, "ng\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31251, "epsg", 31251,
		      "MGI (Ferro) / Austria GK West Zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28 +k=1 +x_0=0 +y_0=-5000000");
    add_proj4text (p, 1, " +ellps=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria GK West Zone\",GEOGCS[\"M");
    add_srs_wkt (p, 1,
		 "GI (Ferro)\",DATUM[\"Militar_Geographische_Institut_Ferr");
    add_srs_wkt (p, 2,
		 "o\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6805\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Ferro\",-17.66666666666667,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8909\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",28],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",-5000000],AUTHORITY[\"EPSG\",\"31251\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31252, "epsg", 31252,
		      "MGI (Ferro) / Austria GK Central Zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=31 +k=1 +x_0=0 +y_0=-5000000");
    add_proj4text (p, 1, " +ellps=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria GK Central Zone\",GEOGCS[");
    add_srs_wkt (p, 1,
		 "\"MGI (Ferro)\",DATUM[\"Militar_Geographische_Institut_F");
    add_srs_wkt (p, 2,
		 "erro\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,");
    add_srs_wkt (p, 3,
		 "AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6805\"");
    add_srs_wkt (p, 4,
		 "]],PRIMEM[\"Ferro\",-17.66666666666667,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"8909\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 6,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT");
    add_srs_wkt (p, 7,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 8,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 9,
		 "0],PARAMETER[\"central_meridian\",31],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",-5000000],AUTHORITY[\"EPSG\",\"31252\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31253, "epsg", 31253,
		      "MGI (Ferro) / Austria GK East Zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=34 +k=1 +x_0=0 +y_0=-5000000");
    add_proj4text (p, 1, " +ellps=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria GK East Zone\",GEOGCS[\"M");
    add_srs_wkt (p, 1,
		 "GI (Ferro)\",DATUM[\"Militar_Geographische_Institut_Ferr");
    add_srs_wkt (p, 2,
		 "o\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6805\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Ferro\",-17.66666666666667,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8909\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",34],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",-5000000],AUTHORITY[\"EPSG\",\"31253\"],A");
    add_srs_wkt (p, 12, "XIS[\"X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31254, "epsg", 31254,
		      "MGI / Austria GK West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=10.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "0 +y_0=-5000000 +ellps=bessel +datum=hermannskogel +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria GK West\",GEOGCS[\"MGI\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Militar_Geographische_Institute\",SPHEROID[\"Bessel 1841");
    add_srs_wkt (p, 2,
		 "\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",10.333");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",0],PARAMETER[\"false_northing\",-5000000]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"31254\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31255, "epsg", 31255,
		      "MGI / Austria GK Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "0 +y_0=-5000000 +ellps=bessel +datum=hermannskogel +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria GK Central\",GEOGCS[\"MGI\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"Militar_Geographische_Institute\",SPHEROID[\"Bessel 1");
    add_srs_wkt (p, 2,
		 "841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"");
    add_srs_wkt (p, 3,
		 "]],TOWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.42");
    add_srs_wkt (p, 4,
		 "32],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 8,
		 "\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"lat");
    add_srs_wkt (p, 9,
		 "itude_of_origin\",0],PARAMETER[\"central_meridian\",13.3");
    add_srs_wkt (p, 10,
		 "3333333333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",0],PARAMETER[\"false_northing\",-5000000");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"31255\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31256, "epsg", 31256,
		      "MGI / Austria GK East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "0 +y_0=-5000000 +ellps=bessel +datum=hermannskogel +unit");
    add_proj4text (p, 2, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria GK East\",GEOGCS[\"MGI\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Militar_Geographische_Institute\",SPHEROID[\"Bessel 1841");
    add_srs_wkt (p, 2,
		 "\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",16.333");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",0],PARAMETER[\"false_northing\",-5000000]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"31256\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31257, "epsg", 31257,
		      "MGI / Austria GK M28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=10.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "150000 +y_0=-5000000 +ellps=bessel +datum=hermannskogel ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria GK M28\",GEOGCS[\"MGI\",DATUM[\"M");
    add_srs_wkt (p, 1,
		 "ilitar_Geographische_Institute\",SPHEROID[\"Bessel 1841\"");
    add_srs_wkt (p, 2,
		 ",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "99433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",10.333333");
    add_srs_wkt (p, 10,
		 "33333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",150000],PARAMETER[\"false_northing\",-500000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"31257\"],AXIS[\"X\",NORTH],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31258, "epsg", 31258,
		      "MGI / Austria GK M31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "450000 +y_0=-5000000 +ellps=bessel +datum=hermannskogel ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria GK M31\",GEOGCS[\"MGI\",DATUM[\"M");
    add_srs_wkt (p, 1,
		 "ilitar_Geographische_Institute\",SPHEROID[\"Bessel 1841\"");
    add_srs_wkt (p, 2,
		 ",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "99433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",13.333333");
    add_srs_wkt (p, 10,
		 "33333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",450000],PARAMETER[\"false_northing\",-500000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"31258\"],AXIS[\"X\",NORTH],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31259, "epsg", 31259,
		      "MGI / Austria GK M34");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "750000 +y_0=-5000000 +ellps=bessel +datum=hermannskogel ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria GK M34\",GEOGCS[\"MGI\",DATUM[\"M");
    add_srs_wkt (p, 1,
		 "ilitar_Geographische_Institute\",SPHEROID[\"Bessel 1841\"");
    add_srs_wkt (p, 2,
		 ",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],T");
    add_srs_wkt (p, 3,
		 "OWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232],");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251");
    add_srs_wkt (p, 6,
		 "99433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 7,
		 "4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 8,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 9,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",16.333333");
    add_srs_wkt (p, 10,
		 "33333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"fals");
    add_srs_wkt (p, 11,
		 "e_easting\",750000],PARAMETER[\"false_northing\",-500000");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"31259\"],AXIS[\"X\",NORTH],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31265, "epsg", 31265,
		      "MGI / 3-degree Gauss zone 5 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=bessel +datum=hermannskogel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / 3-degree Gauss zone 5 (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"MGI\",DATUM[\"Militar_Geographische_Institute\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137");
    add_srs_wkt (p, 4,
		 ",1.474,5.297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIME");
    add_srs_wkt (p, 5,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 6,
		 "egree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",15],PARAMETER[\"scale_factor\",1],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",5500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"31265\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31266, "epsg", 31266,
		      "MGI / 3-degree Gauss zone 6 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=1 +x_0=6500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=bessel +datum=hermannskogel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / 3-degree Gauss zone 6 (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"MGI\",DATUM[\"Militar_Geographische_Institute\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137");
    add_srs_wkt (p, 4,
		 ",1.474,5.297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIME");
    add_srs_wkt (p, 5,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 6,
		 "egree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",18],PARAMETER[\"scale_factor\",1],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",6500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"31266\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31267, "epsg", 31267,
		      "MGI / 3-degree Gauss zone 7 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=1 +x_0=7500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=bessel +datum=hermannskogel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / 3-degree Gauss zone 7 (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"MGI\",DATUM[\"Militar_Geographische_Institute\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137");
    add_srs_wkt (p, 4,
		 ",1.474,5.297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIME");
    add_srs_wkt (p, 5,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 6,
		 "egree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",21],PARAMETER[\"scale_factor\",1],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",7500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"31267\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31268, "epsg", 31268,
		      "MGI / 3-degree Gauss zone 8 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=1 +x_0=8500000 +y_0=0 ");
    add_proj4text (p, 1,
		   "+ellps=bessel +datum=hermannskogel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / 3-degree Gauss zone 8 (deprecated)\",GEOG");
    add_srs_wkt (p, 1,
		 "CS[\"MGI\",DATUM[\"Militar_Geographische_Institute\",SPH");
    add_srs_wkt (p, 2,
		 "EROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137");
    add_srs_wkt (p, 4,
		 ",1.474,5.297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIME");
    add_srs_wkt (p, 5,
		 "M[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"d");
    add_srs_wkt (p, 6,
		 "egree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]]");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORIT");
    add_srs_wkt (p, 8,
		 "Y[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"centra");
    add_srs_wkt (p, 10,
		 "l_meridian\",24],PARAMETER[\"scale_factor\",1],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",8500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"31268\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31275, "epsg", 31275,
		      "MGI / Balkans zone 5 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9999 +x_0=5500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=bessel +datum=hermannskogel +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Balkans zone 5 (deprecated)\",GEOGCS[\"MG");
    add_srs_wkt (p, 1,
		 "I\",DATUM[\"Militar_Geographische_Institute\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137,1.474,5");
    add_srs_wkt (p, 4,
		 ".297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 5,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 6,
		 "0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",15],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",5500000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"31275\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31276, "epsg", 31276,
		      "MGI / Balkans zone 6 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=18 +k=0.9999 +x_0=6500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=bessel +datum=hermannskogel +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Balkans zone 6 (deprecated)\",GEOGCS[\"MG");
    add_srs_wkt (p, 1,
		 "I\",DATUM[\"Militar_Geographische_Institute\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137,1.474,5");
    add_srs_wkt (p, 4,
		 ".297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 5,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 6,
		 "0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",18],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",6500000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"31276\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31277, "epsg", 31277,
		      "MGI / Balkans zone 7 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=0.9999 +x_0=7500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=bessel +datum=hermannskogel +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Balkans zone 7 (deprecated)\",GEOGCS[\"MG");
    add_srs_wkt (p, 1,
		 "I\",DATUM[\"Militar_Geographische_Institute\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137,1.474,5");
    add_srs_wkt (p, 4,
		 ".297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 5,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 6,
		 "0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",21],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",7500000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"31277\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31278, "epsg", 31278,
		      "MGI / Balkans zone 8 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=21 +k=0.9999 +x_0=7500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=bessel +datum=hermannskogel +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Balkans zone 8 (deprecated)\",GEOGCS[\"MG");
    add_srs_wkt (p, 1,
		 "I\",DATUM[\"Militar_Geographische_Institute\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137,1.474,5");
    add_srs_wkt (p, 4,
		 ".297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 5,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 6,
		 "0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",21],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",7500000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"31278\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31279, "epsg", 31279,
		      "MGI / Balkans zone 8 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=24 +k=0.9999 +x_0=8500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=bessel +datum=hermannskogel +units=m +no_def");
    add_proj4text (p, 2, "s");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Balkans zone 8 (deprecated)\",GEOGCS[\"MG");
    add_srs_wkt (p, 1,
		 "I\",DATUM[\"Militar_Geographische_Institute\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137,1.474,5");
    add_srs_wkt (p, 4,
		 ".297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 5,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 6,
		 "0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 8,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 9,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 10,
		 "an\",24],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_easting\",8500000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 12,
		 ",AUTHORITY[\"EPSG\",\"31279\"],AXIS[\"X\",NORTH],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",EAST]]");
    p = add_epsg_def (first, last, 31281, "epsg", 31281,
		      "MGI (Ferro) / Austria West Zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28 +k=1 +x_0=0 +y_0=0 +ellps");
    add_proj4text (p, 1, "=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria West Zone\",GEOGCS[\"MGI ");
    add_srs_wkt (p, 1,
		 "(Ferro)\",DATUM[\"Militar_Geographische_Institut_Ferro\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6805\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Ferro\",-17.66666666666667,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "909\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",28],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"31281\"],AXIS[\"X\",");
    add_srs_wkt (p, 12, "NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31282, "epsg", 31282,
		      "MGI (Ferro) / Austria Central Zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=31 +k=1 +x_0=0 +y_0=0 +ellps");
    add_proj4text (p, 1, "=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria Central Zone\",GEOGCS[\"M");
    add_srs_wkt (p, 1,
		 "GI (Ferro)\",DATUM[\"Militar_Geographische_Institut_Ferr");
    add_srs_wkt (p, 2,
		 "o\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6805\"]]");
    add_srs_wkt (p, 4,
		 ",PRIMEM[\"Ferro\",-17.66666666666667,AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 5,
		 "\"8909\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY");
    add_srs_wkt (p, 6,
		 "[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[");
    add_srs_wkt (p, 7,
		 "\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",31],PARAMETER[\"scale_f");
    add_srs_wkt (p, 10,
		 "actor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"31282\"],AXIS[\"X");
    add_srs_wkt (p, 12, "\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31283, "epsg", 31283,
		      "MGI (Ferro) / Austria East Zone");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=34 +k=1 +x_0=0 +y_0=0 +ellps");
    add_proj4text (p, 1, "=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria East Zone\",GEOGCS[\"MGI ");
    add_srs_wkt (p, 1,
		 "(Ferro)\",DATUM[\"Militar_Geographische_Institut_Ferro\"");
    add_srs_wkt (p, 2,
		 ",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6805\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Ferro\",-17.66666666666667,AUTHORITY[\"EPSG\",\"8");
    add_srs_wkt (p, 5,
		 "909\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[\"m");
    add_srs_wkt (p, 7,
		 "etre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Tran");
    add_srs_wkt (p, 8,
		 "sverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"central_meridian\",34],PARAMETER[\"scale_fact");
    add_srs_wkt (p, 10,
		 "or\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"31283\"],AXIS[\"X\",");
    add_srs_wkt (p, 12, "NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31284, "epsg", 31284, "MGI / Austria M28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=10.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "150000 +y_0=0 +ellps=bessel +datum=hermannskogel +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria M28\",GEOGCS[\"MGI\",DATUM[\"Mili");
    add_srs_wkt (p, 1,
		 "tar_Geographische_Institute\",SPHEROID[\"Bessel 1841\",6");
    add_srs_wkt (p, 2,
		 "377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",10.3333333");
    add_srs_wkt (p, 10,
		 "3333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_easting\",150000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"31284\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 13, "ST]]");
    p = add_epsg_def (first, last, 31285, "epsg", 31285, "MGI / Austria M31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "450000 +y_0=0 +ellps=bessel +datum=hermannskogel +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria M31\",GEOGCS[\"MGI\",DATUM[\"Mili");
    add_srs_wkt (p, 1,
		 "tar_Geographische_Institute\",SPHEROID[\"Bessel 1841\",6");
    add_srs_wkt (p, 2,
		 "377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",13.3333333");
    add_srs_wkt (p, 10,
		 "3333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_easting\",450000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"31285\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 13, "ST]]");
    p = add_epsg_def (first, last, 31286, "epsg", 31286, "MGI / Austria M34");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "750000 +y_0=0 +ellps=bessel +datum=hermannskogel +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria M34\",GEOGCS[\"MGI\",DATUM[\"Mili");
    add_srs_wkt (p, 1,
		 "tar_Geographische_Institute\",SPHEROID[\"Bessel 1841\",6");
    add_srs_wkt (p, 2,
		 "377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],TOW");
    add_srs_wkt (p, 3,
		 "GS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232],AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 6,
		 "433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",16.3333333");
    add_srs_wkt (p, 10,
		 "3333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_easting\",750000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 12,
		 "RITY[\"EPSG\",\"31286\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 13, "ST]]");
    p = add_epsg_def (first, last, 31287, "epsg", 31287,
		      "MGI / Austria Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=46 +lat_0=47.5 +lon_0=13.3333");
    add_proj4text (p, 1,
		   "3333333333 +x_0=400000 +y_0=400000 +ellps=bessel +datum=");
    add_proj4text (p, 2, "hermannskogel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria Lambert\",GEOGCS[\"MGI\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Militar_Geographische_Institute\",SPHEROID[\"Bessel 1841");
    add_srs_wkt (p, 2,
		 "\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 9,
		 "[\"standard_parallel_1\",49],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 10,
		 "el_2\",46],PARAMETER[\"latitude_of_origin\",47.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",13.33333333333333],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",400000],PARAMETER[\"false_northing\",400");
    add_srs_wkt (p, 13,
		 "000],AUTHORITY[\"EPSG\",\"31287\"],AXIS[\"X\",NORTH],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31288, "epsg", 31288, "MGI (Ferro) / M28");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28 +k=1 +x_0=150000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / M28\",GEOGCS[\"MGI (Ferro)\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"Militar_Geographische_Institut_Ferro\",SPHEROID[\"Be");
    add_srs_wkt (p, 2,
		 "ssel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7004\"]],AUTHORITY[\"EPSG\",\"6805\"]],PRIMEM[\"Ferro\",");
    add_srs_wkt (p, 4,
		 "-17.66666666666667,AUTHORITY[\"EPSG\",\"8909\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",28],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",150000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"31288\"],AXIS[\"X\",NORTH],AXIS");
    add_srs_wkt (p, 12, "[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31289, "epsg", 31289, "MGI (Ferro) / M31");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=31 +k=1 +x_0=450000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / M31\",GEOGCS[\"MGI (Ferro)\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"Militar_Geographische_Institut_Ferro\",SPHEROID[\"Be");
    add_srs_wkt (p, 2,
		 "ssel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7004\"]],AUTHORITY[\"EPSG\",\"6805\"]],PRIMEM[\"Ferro\",");
    add_srs_wkt (p, 4,
		 "-17.66666666666667,AUTHORITY[\"EPSG\",\"8909\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",31],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",450000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"31289\"],AXIS[\"X\",NORTH],AXIS");
    add_srs_wkt (p, 12, "[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31290, "epsg", 31290, "MGI (Ferro) / M34");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=34 +k=1 +x_0=750000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / M34\",GEOGCS[\"MGI (Ferro)\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"Militar_Geographische_Institut_Ferro\",SPHEROID[\"Be");
    add_srs_wkt (p, 2,
		 "ssel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7004\"]],AUTHORITY[\"EPSG\",\"6805\"]],PRIMEM[\"Ferro\",");
    add_srs_wkt (p, 4,
		 "-17.66666666666667,AUTHORITY[\"EPSG\",\"8909\"]],UNIT[\"");
    add_srs_wkt (p, 5,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 6,
		 "]],AUTHORITY[\"EPSG\",\"4805\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 8,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 9,
		 "ral_meridian\",34],PARAMETER[\"scale_factor\",1],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"false_easting\",750000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"31290\"],AXIS[\"X\",NORTH],AXIS");
    add_srs_wkt (p, 12, "[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31291, "epsg", 31291,
		      "MGI (Ferro) / Austria West Zone (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=28 +k=1 +x_0=0 +y_0=0 +ellps");
    add_proj4text (p, 1, "=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria West Zone (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"MGI (Ferro)\",DATUM[\"Militar_Geographische_Ins");
    add_srs_wkt (p, 2,
		 "titut_Ferro\",SPHEROID[\"Bessel 1841\",6377397.155,299.1");
    add_srs_wkt (p, 3,
		 "528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6805\"]],PRIMEM[\"Ferro\",-17.66666666666667,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8909\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",0],PARAMETER[\"central_meridian\",28],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"31291\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31292, "epsg", 31292,
		      "MGI (Ferro) / Austria Central Zone (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=31 +k=1 +x_0=0 +y_0=0 +ellps");
    add_proj4text (p, 1, "=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria Central Zone (deprecated)");
    add_srs_wkt (p, 1,
		 "\",GEOGCS[\"MGI (Ferro)\",DATUM[\"Militar_Geographische_");
    add_srs_wkt (p, 2,
		 "Institut_Ferro\",SPHEROID[\"Bessel 1841\",6377397.155,29");
    add_srs_wkt (p, 3,
		 "9.1528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"6805\"]],PRIMEM[\"Ferro\",-17.66666666666667,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"8909\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 6,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 7,
		 "805\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 8,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 9,
		 "of_origin\",0],PARAMETER[\"central_meridian\",31],PARAME");
    add_srs_wkt (p, 10,
		 "TER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3129");
    add_srs_wkt (p, 12, "2\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31293, "epsg", 31293,
		      "MGI (Ferro) / Austria East Zone (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=34 +k=1 +x_0=0 +y_0=0 +ellps");
    add_proj4text (p, 1, "=bessel +pm=ferro +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI (Ferro) / Austria East Zone (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"MGI (Ferro)\",DATUM[\"Militar_Geographische_Ins");
    add_srs_wkt (p, 2,
		 "titut_Ferro\",SPHEROID[\"Bessel 1841\",6377397.155,299.1");
    add_srs_wkt (p, 3,
		 "528128,AUTHORITY[\"EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 4,
		 "\"6805\"]],PRIMEM[\"Ferro\",-17.66666666666667,AUTHORITY");
    add_srs_wkt (p, 5,
		 "[\"EPSG\",\"8909\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 6,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4805");
    add_srs_wkt (p, 7,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 8,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 9,
		 "origin\",0],PARAMETER[\"central_meridian\",34],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"31293\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31294, "epsg", 31294,
		      "MGI / M28 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=10.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "150000 +y_0=0 +ellps=bessel +datum=hermannskogel +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / M28 (deprecated)\",GEOGCS[\"MGI\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Militar_Geographische_Institute\",SPHEROID[\"Bessel 1841");
    add_srs_wkt (p, 2,
		 "\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",10.333");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",150000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"31294\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 31295, "epsg", 31295,
		      "MGI / M31 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=13.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "450000 +y_0=0 +ellps=bessel +datum=hermannskogel +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / M31 (deprecated)\",GEOGCS[\"MGI\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Militar_Geographische_Institute\",SPHEROID[\"Bessel 1841");
    add_srs_wkt (p, 2,
		 "\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",13.333");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",450000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"31295\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 31296, "epsg", 31296,
		      "MGI / M34 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=16.33333333333333 +k=1 +x_0=");
    add_proj4text (p, 1,
		   "750000 +y_0=0 +ellps=bessel +datum=hermannskogel +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / M34 (deprecated)\",GEOGCS[\"MGI\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "Militar_Geographische_Institute\",SPHEROID[\"Bessel 1841");
    add_srs_wkt (p, 2,
		 "\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]]");
    add_srs_wkt (p, 3,
		 ",TOWGS84[577.326,90.129,463.919,5.137,1.474,5.297,2.4232");
    add_srs_wkt (p, 4,
		 "],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 5,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 6,
		 "5199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 7,
		 ",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 8,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 9,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",16.333");
    add_srs_wkt (p, 10,
		 "33333333333],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",750000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"31296\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 31297, "epsg", 31297,
		      "MGI / Austria Lambert (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=46 +lat_0=47.5 +lon_0=13.3333");
    add_proj4text (p, 1,
		   "3333333333 +x_0=400000 +y_0=400000 +ellps=bessel +datum=");
    add_proj4text (p, 2, "hermannskogel +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"MGI / Austria Lambert (deprecated)\",GEOGCS[\"M");
    add_srs_wkt (p, 1,
		 "GI\",DATUM[\"Militar_Geographische_Institute\",SPHEROID[");
    add_srs_wkt (p, 2,
		 "\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 3,
		 "\",\"7004\"]],TOWGS84[577.326,90.129,463.919,5.137,1.474");
    add_srs_wkt (p, 4,
		 ",5.297,2.4232],AUTHORITY[\"EPSG\",\"6312\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 5,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 6,
		 ",0.0174532925199433,AUTHORITY[\"EPSG\",\"9108\"]],AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"4312\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 8,
		 "G\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"standard_parallel_1\",49],PARAMETER[\"stan");
    add_srs_wkt (p, 10,
		 "dard_parallel_2\",46],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 11,
		 "7.5],PARAMETER[\"central_meridian\",13.33333333333333],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_easting\",400000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 13,
		 "thing\",400000],AUTHORITY[\"EPSG\",\"31297\"],AXIS[\"X\"");
    add_srs_wkt (p, 14, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31300, "epsg", 31300,
		      "Belge 1972 / Belge Lambert 72");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49.83333333333334 +lat_2=51.16666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=90 +lon_0=4.356939722222222 +x_0=150000.01256");
    add_proj4text (p, 2,
		   " +y_0=5400088.4378 +ellps=intl +towgs84=106.869,-52.2978");
    add_proj4text (p, 3,
		   ",103.724,-0.33657,0.456955,-1.84218,1 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Belge 1972 / Belge Lambert 72\",GEOGCS[\"Belge ");
    add_srs_wkt (p, 1,
		 "1972\",DATUM[\"Reseau_National_Belge_1972\",SPHEROID[\"I");
    add_srs_wkt (p, 2,
		 "nternational 1924\",6378388,297,AUTHORITY[\"EPSG\",\"702");
    add_srs_wkt (p, 3,
		 "2\"]],TOWGS84[106.869,-52.2978,103.724,-0.33657,0.456955");
    add_srs_wkt (p, 4,
		 ",-1.84218,1],AUTHORITY[\"EPSG\",\"6313\"]],PRIMEM[\"Gree");
    add_srs_wkt (p, 5,
		 "nwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",");
    add_srs_wkt (p, 6,
		 "0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHOR");
    add_srs_wkt (p, 7,
		 "ITY[\"EPSG\",\"4313\"]],UNIT[\"metre\",1,AUTHORITY[\"EPS");
    add_srs_wkt (p, 8,
		 "G\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP_");
    add_srs_wkt (p, 9,
		 "Belgium\"],PARAMETER[\"standard_parallel_1\",49.83333333");
    add_srs_wkt (p, 10,
		 "333334],PARAMETER[\"standard_parallel_2\",51.16666666666");
    add_srs_wkt (p, 11,
		 "666],PARAMETER[\"latitude_of_origin\",90],PARAMETER[\"ce");
    add_srs_wkt (p, 12,
		 "ntral_meridian\",4.356939722222222],PARAMETER[\"false_ea");
    add_srs_wkt (p, 13,
		 "sting\",150000.01256],PARAMETER[\"false_northing\",54000");
    add_srs_wkt (p, 14,
		 "88.4378],AUTHORITY[\"EPSG\",\"31300\"],AXIS[\"X\",EAST],");
    add_srs_wkt (p, 15, "AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31370, "epsg", 31370,
		      "Belge 1972 / Belgian Lambert 72");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=51.16666723333333 +lat_2=49.8333339 +la");
    add_proj4text (p, 1,
		   "t_0=90 +lon_0=4.367486666666666 +x_0=150000.013 +y_0=540");
    add_proj4text (p, 2,
		   "0088.438 +ellps=intl +towgs84=106.869,-52.2978,103.724,-");
    add_proj4text (p, 3, "0.33657,0.456955,-1.84218,1 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Belge 1972 / Belgian Lambert 72\",GEOGCS[\"Belg");
    add_srs_wkt (p, 1,
		 "e 1972\",DATUM[\"Reseau_National_Belge_1972\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "International 1924\",6378388,297,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "22\"]],TOWGS84[106.869,-52.2978,103.724,-0.33657,0.45695");
    add_srs_wkt (p, 4,
		 "5,-1.84218,1],AUTHORITY[\"EPSG\",\"6313\"]],PRIMEM[\"Gre");
    add_srs_wkt (p, 5,
		 "enwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 6,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 7,
		 "RITY[\"EPSG\",\"4313\"]],UNIT[\"metre\",1,AUTHORITY[\"EP");
    add_srs_wkt (p, 8,
		 "SG\",\"9001\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP");
    add_srs_wkt (p, 9,
		 "\"],PARAMETER[\"standard_parallel_1\",51.16666723333333]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"standard_parallel_2\",49.8333339],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"latitude_of_origin\",90],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 12,
		 "\",4.367486666666666],PARAMETER[\"false_easting\",150000");
    add_srs_wkt (p, 13,
		 ".013],PARAMETER[\"false_northing\",5400088.438],AUTHORIT");
    add_srs_wkt (p, 14,
		 "Y[\"EPSG\",\"31370\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]");
    add_srs_wkt (p, 15, "]");
    p = add_epsg_def (first, last, 31461, "epsg", 31461,
		      "DHDN / 3-degree Gauss zone 1 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=3 +k=1 +x_0=1500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss zone 1 (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",3],PARAMETER[\"scale_factor\",1],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",1500000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"31461\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 31462, "epsg", 31462,
		      "DHDN / 3-degree Gauss zone 2 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=6 +k=1 +x_0=2500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss zone 2 (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",6],PARAMETER[\"scale_factor\",1],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",2500000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"31462\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 31463, "epsg", 31463,
		      "DHDN / 3-degree Gauss zone 3 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=3500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss zone 3 (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",9],PARAMETER[\"scale_factor\",1],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",3500000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"31463\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 31464, "epsg", 31464,
		      "DHDN / 3-degree Gauss zone 4 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss zone 4 (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",12],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",4500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"31464\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 31465, "epsg", 31465,
		      "DHDN / 3-degree Gauss zone 5 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss zone 5 (deprecated)\",GEO");
    add_srs_wkt (p, 1,
		 "GCS[\"DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"");
    add_srs_wkt (p, 4,
		 "Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degre");
    add_srs_wkt (p, 5,
		 "e\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",15],PARAMETER[\"scale_factor\",1],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",5500000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"31465\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 31466, "epsg", 31466,
		      "DHDN / 3-degree Gauss-Kruger zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=6 +k=1 +x_0=2500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss-Kruger zone 2\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",6],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",2500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"31466\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 12, "ST]]");
    p = add_epsg_def (first, last, 31467, "epsg", 31467,
		      "DHDN / 3-degree Gauss-Kruger zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9 +k=1 +x_0=3500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss-Kruger zone 3\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",9],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "easting\",3500000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"31467\"],AXIS[\"X\",NORTH],AXIS[\"Y\",EA");
    add_srs_wkt (p, 12, "ST]]");
    p = add_epsg_def (first, last, 31468, "epsg", 31468,
		      "DHDN / 3-degree Gauss-Kruger zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=12 +k=1 +x_0=4500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss-Kruger zone 4\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",12],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",4500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"31468\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 31469, "epsg", 31469,
		      "DHDN / 3-degree Gauss-Kruger zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=1 +x_0=5500000 +y_0=0 ");
    add_proj4text (p, 1, "+ellps=bessel +datum=potsdam +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"DHDN / 3-degree Gauss-Kruger zone 5\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "DHDN\",DATUM[\"Deutsches_Hauptdreiecksnetz\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7004\"]],AUTHORITY[\"EPSG\",\"6314\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4314\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PARAME");
    add_srs_wkt (p, 8,
		 "TER[\"latitude_of_origin\",0],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",15],PARAMETER[\"scale_factor\",1],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",5500000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 11,
		 "ORITY[\"EPSG\",\"31469\"],AXIS[\"X\",NORTH],AXIS[\"Y\",E");
    add_srs_wkt (p, 12, "AST]]");
    p = add_epsg_def (first, last, 31528, "epsg", 31528,
		      "Conakry 1905 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +a=6378249.2 +b=6356515 +towgs84=-23,");
    add_proj4text (p, 1, "259,-9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Conakry 1905 / UTM zone 28N\",GEOGCS[\"Conakry ");
    add_srs_wkt (p, 1,
		 "1905\",DATUM[\"Conakry_1905\",SPHEROID[\"Clarke 1880 (IG");
    add_srs_wkt (p, 2,
		 "N)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "11\"]],TOWGS84[-23,259,-9,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6315\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4315\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",-15],PARAMETER[\"scale_facto");
    add_srs_wkt (p, 10,
		 "r\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_northing\",0],AUTHORITY[\"EPSG\",\"31528\"],AX");
    add_srs_wkt (p, 12, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31529, "epsg", 31529,
		      "Conakry 1905 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +a=6378249.2 +b=6356515 +towgs84=-23,");
    add_proj4text (p, 1, "259,-9,0,0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Conakry 1905 / UTM zone 29N\",GEOGCS[\"Conakry ");
    add_srs_wkt (p, 1,
		 "1905\",DATUM[\"Conakry_1905\",SPHEROID[\"Clarke 1880 (IG");
    add_srs_wkt (p, 2,
		 "N)\",6378249.2,293.4660212936265,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "11\"]],TOWGS84[-23,259,-9,0,0,0,0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 4,
		 "6315\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"890");
    add_srs_wkt (p, 5,
		 "1\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4315\"]],UNIT[\"met");
    add_srs_wkt (p, 7,
		 "re\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transv");
    add_srs_wkt (p, 8,
		 "erse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"central_meridian\",-9],PARAMETER[\"scale_factor");
    add_srs_wkt (p, 10,
		 "\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",0],AUTHORITY[\"EPSG\",\"31529\"],AXI");
    add_srs_wkt (p, 12, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31600, "epsg", 31600,
		      "Dealul Piscului 1930 / Stereo 33");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=45.9 +lon_0=25.39246588888889 +k=0.9");
    add_proj4text (p, 1,
		   "996667 +x_0=500000 +y_0=500000 +ellps=intl +units=m +no_");
    add_proj4text (p, 2, "defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Dealul Piscului 1930 / Stereo 33\",GEOGCS[\"Dea");
    add_srs_wkt (p, 1,
		 "lul Piscului 1930\",DATUM[\"Dealul_Piscului_1930\",SPHER");
    add_srs_wkt (p, 2,
		 "OID[\"International 1924\",6378388,297,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 3,
		 ",\"7022\"]],AUTHORITY[\"EPSG\",\"6316\"]],PRIMEM[\"Green");
    add_srs_wkt (p, 4,
		 "wich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0");
    add_srs_wkt (p, 5,
		 ".01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORI");
    add_srs_wkt (p, 6,
		 "TY[\"EPSG\",\"4316\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 7,
		 "\",\"9001\"]],PROJECTION[\"Oblique_Stereographic\"],PARA");
    add_srs_wkt (p, 8,
		 "METER[\"latitude_of_origin\",45.9],PARAMETER[\"central_m");
    add_srs_wkt (p, 9,
		 "eridian\",25.39246588888889],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996667],PARAMETER[\"false_easting\",500000],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_northing\",500000],AUTHORITY[\"EPSG\",\"31600\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 31700, "epsg", 31700,
		      "Dealul Piscului 1970/ Stereo 70 (deprecated)");
    add_proj4text (p, 0,
		   "+proj=sterea +lat_0=46 +lon_0=25 +k=0.99975 +x_0=500000 ");
    add_proj4text (p, 1, "+y_0=500000 +ellps=krass +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"Dealul Piscului 1970/ Stereo 70 (deprecated)\",");
    add_srs_wkt (p, 1,
		 "GEOGCS[\"Dealul Piscului 1970\",DATUM[\"Dealul_Piscului_");
    add_srs_wkt (p, 2,
		 "1970\",SPHEROID[\"Krassowsky 1940\",6378245,298.3,AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"7024\"]],AUTHORITY[\"EPSG\",\"6317\"]],PR");
    add_srs_wkt (p, 4,
		 "IMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[");
    add_srs_wkt (p, 5,
		 "\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122");
    add_srs_wkt (p, 6,
		 "\"]],AUTHORITY[\"EPSG\",\"4317\"]],UNIT[\"metre\",1,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Oblique_Stereogra");
    add_srs_wkt (p, 8,
		 "phic\"],PARAMETER[\"latitude_of_origin\",46],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",25],PARAMETER[\"scale_factor\",0.9997");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 11,
		 "_northing\",500000],AUTHORITY[\"EPSG\",\"31700\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",NORTH],AXIS[\"Y\",EAST]]");
    p = add_epsg_def (first, last, 31838, "epsg", 31838, "NGN / UTM zone 38N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +ellps=WGS84 +towgs84=-3.2,-5.7,2.8,0");
    add_proj4text (p, 1, ",0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGN / UTM zone 38N\",GEOGCS[\"NGN\",DATUM[\"Nat");
    add_srs_wkt (p, 1,
		 "ional_Geodetic_Network\",SPHEROID[\"WGS 84\",6378137,298");
    add_srs_wkt (p, 2,
		 ".257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[-3.2,-5");
    add_srs_wkt (p, 3,
		 ".7,2.8,0,0,0,0],AUTHORITY[\"EPSG\",\"6318\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4318\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",45],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"31838\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31839, "epsg", 31839, "NGN / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=WGS84 +towgs84=-3.2,-5.7,2.8,0");
    add_proj4text (p, 1, ",0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NGN / UTM zone 39N\",GEOGCS[\"NGN\",DATUM[\"Nat");
    add_srs_wkt (p, 1,
		 "ional_Geodetic_Network\",SPHEROID[\"WGS 84\",6378137,298");
    add_srs_wkt (p, 2,
		 ".257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[-3.2,-5");
    add_srs_wkt (p, 3,
		 ".7,2.8,0,0,0,0],AUTHORITY[\"EPSG\",\"6318\"]],PRIMEM[\"G");
    add_srs_wkt (p, 4,
		 "reenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree");
    add_srs_wkt (p, 5,
		 "\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUT");
    add_srs_wkt (p, 6,
		 "HORITY[\"EPSG\",\"4318\"]],UNIT[\"metre\",1,AUTHORITY[\"");
    add_srs_wkt (p, 7,
		 "EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator\"],PA");
    add_srs_wkt (p, 8,
		 "RAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_me");
    add_srs_wkt (p, 9,
		 "ridian\",51],PARAMETER[\"scale_factor\",0.9996],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",500000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"31839\"],AXIS[\"Easting\",EAST]");
    add_srs_wkt (p, 12, ",AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31900, "epsg", 31900,
		      "KUDAMS / KTM (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=48 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1, "0=0 +ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KUDAMS / KTM (deprecated)\",GEOGCS[\"KUDAMS\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"Kuwait_Utility\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 2,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6319\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4319\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "0],PARAMETER[\"central_meridian\",48],PARAMETER[\"scale_");
    add_srs_wkt (p, 9,
		 "factor\",0.9996],PARAMETER[\"false_easting\",500000],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"31900\"");
    add_srs_wkt (p, 11, "],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31901, "epsg", 31901, "KUDAMS / KTM");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=48 +k=1 +x_0=500000 +y_0=0 +");
    add_proj4text (p, 1, "ellps=GRS80 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"KUDAMS / KTM\",GEOGCS[\"KUDAMS\",DATUM[\"Kuwait");
    add_srs_wkt (p, 1,
		 "_Utility\",SPHEROID[\"GRS 1980\",6378137,298.257222101,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EPSG\",\"6319\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4319\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",48],PARAMETER[\"scale_factor\",1],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"31901\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31965, "epsg", 31965,
		      "SIRGAS 2000 / UTM zone 11N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 11N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-11");
    add_srs_wkt (p, 10,
		 "7],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"31965\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 31966, "epsg", 31966,
		      "SIRGAS 2000 / UTM zone 12N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 12N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-11");
    add_srs_wkt (p, 10,
		 "1],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"31966\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 31967, "epsg", 31967,
		      "SIRGAS 2000 / UTM zone 13N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 13N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-10");
    add_srs_wkt (p, 10,
		 "5],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",500000],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 12,
		 "ITY[\"EPSG\",\"31967\"],AXIS[\"Easting\",EAST],AXIS[\"No");
    add_srs_wkt (p, 13, "rthing\",NORTH]]");
    p = add_epsg_def (first, last, 31968, "epsg", 31968,
		      "SIRGAS 2000 / UTM zone 14N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 14N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-99");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31968\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31969, "epsg", 31969,
		      "SIRGAS 2000 / UTM zone 15N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 15N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-93");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31969\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31970, "epsg", 31970,
		      "SIRGAS 2000 / UTM zone 16N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 16N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-87");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31970\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31971, "epsg", 31971,
		      "SIRGAS 2000 / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 17N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-81");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31971\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31972, "epsg", 31972,
		      "SIRGAS 2000 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 18N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-75");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31972\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31973, "epsg", 31973,
		      "SIRGAS 2000 / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 19N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-69");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31973\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31974, "epsg", 31974,
		      "SIRGAS 2000 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 20N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-63");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31974\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31975, "epsg", 31975,
		      "SIRGAS 2000 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 21N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-57");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31975\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31976, "epsg", 31976,
		      "SIRGAS 2000 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 22N\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-51");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31976\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31977, "epsg", 31977,
		      "SIRGAS 2000 / UTM zone 17S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 17S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-81");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31977\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31978, "epsg", 31978,
		      "SIRGAS 2000 / UTM zone 18S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 18S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-75");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31978\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31979, "epsg", 31979,
		      "SIRGAS 2000 / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 19S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-69");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31979\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31980, "epsg", 31980,
		      "SIRGAS 2000 / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 20S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-63");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31980\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31981, "epsg", 31981,
		      "SIRGAS 2000 / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 21S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-57");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31981\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31982, "epsg", 31982,
		      "SIRGAS 2000 / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 22S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-51");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31982\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31983, "epsg", 31983,
		      "SIRGAS 2000 / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 23S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-45");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31983\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31984, "epsg", 31984,
		      "SIRGAS 2000 / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 24S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-39");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31984\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31985, "epsg", 31985,
		      "SIRGAS 2000 / UTM zone 25S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 2000 / UTM zone 25S\",GEOGCS[\"SIRGAS 20");
    add_srs_wkt (p, 1,
		 "00\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_2000\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6674\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4674\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-33");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31985\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31986, "epsg", 31986,
		      "SIRGAS 1995 / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 17N\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-81");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31986\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31987, "epsg", 31987,
		      "SIRGAS 1995 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 18N\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-75");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31987\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31988, "epsg", 31988,
		      "SIRGAS 1995 / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 19N\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-69");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31988\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31989, "epsg", 31989,
		      "SIRGAS 1995 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 20N\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-63");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31989\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31990, "epsg", 31990,
		      "SIRGAS 1995 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 21N\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-57");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31990\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31991, "epsg", 31991,
		      "SIRGAS 1995 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +");
    add_proj4text (p, 1, "units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 22N\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-51");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"31991\"],AXIS[\"Easting\",EAST],AXIS[\"Nor");
    add_srs_wkt (p, 13, "thing\",NORTH]]");
    p = add_epsg_def (first, last, 31992, "epsg", 31992,
		      "SIRGAS 1995 / UTM zone 17S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 17S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-81");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31992\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31993, "epsg", 31993,
		      "SIRGAS 1995 / UTM zone 18S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 18S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-75");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31993\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31994, "epsg", 31994,
		      "SIRGAS 1995 / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 19S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-69");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31994\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31995, "epsg", 31995,
		      "SIRGAS 1995 / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 20S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-63");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31995\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31996, "epsg", 31996,
		      "SIRGAS 1995 / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 21S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-57");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31996\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31997, "epsg", 31997,
		      "SIRGAS 1995 / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 22S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-51");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31997\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31998, "epsg", 31998,
		      "SIRGAS 1995 / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 23S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-45");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31998\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 31999, "epsg", 31999,
		      "SIRGAS 1995 / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 24S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-39");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"31999\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32000, "epsg", 32000,
		      "SIRGAS 1995 / UTM zone 25S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=GRS80 +towgs84=0,0,0,0,");
    add_proj4text (p, 1, "0,0,0 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"SIRGAS 1995 / UTM zone 25S\",GEOGCS[\"SIRGAS 19");
    add_srs_wkt (p, 1,
		 "95\",DATUM[\"Sistema_de_Referencia_Geocentrico_para_Amer");
    add_srs_wkt (p, 2,
		 "ica_del_Sur_1995\",SPHEROID[\"GRS 1980\",6378137,298.257");
    add_srs_wkt (p, 3,
		 "222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0");
    add_srs_wkt (p, 4,
		 ",0],AUTHORITY[\"EPSG\",\"6170\"]],PRIMEM[\"Greenwich\",0");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532");
    add_srs_wkt (p, 6,
		 "9251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPS");
    add_srs_wkt (p, 7,
		 "G\",\"4170\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"900");
    add_srs_wkt (p, 8,
		 "1\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"la");
    add_srs_wkt (p, 9,
		 "titude_of_origin\",0],PARAMETER[\"central_meridian\",-33");
    add_srs_wkt (p, 10,
		 "],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",10000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"32000\"],AXIS[\"Easting\",EAST],AXI");
    add_srs_wkt (p, 13, "S[\"Northing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_25 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 32001, "epsg", 32001,
		      "NAD27 / Montana North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.71666666666667 +lat_2=47.85 +lat_0=4");
    add_proj4text (p, 1,
		   "7 +lon_0=-109.5 +x_0=609601.2192024384 +y_0=0 +ellps=clr");
    add_proj4text (p, 2, "k66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Montana North\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",48.71");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",47.85],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"latitude_of_origin\",47],PARAMETER[\"central_");
    add_srs_wkt (p, 11,
		 "meridian\",-109.5],PARAMETER[\"false_easting\",2000000],");
    add_srs_wkt (p, 12,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"320");
    add_srs_wkt (p, 13, "01\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32002, "epsg", 32002,
		      "NAD27 / Montana Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.88333333333333 +lat_2=46.45 +lat_0=4");
    add_proj4text (p, 1,
		   "5.83333333333334 +lon_0=-109.5 +x_0=609601.2192024384 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Montana Central\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",47.");
    add_srs_wkt (p, 9,
		 "88333333333333],PARAMETER[\"standard_parallel_2\",46.45]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"latitude_of_origin\",45.83333333333334],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"central_meridian\",-109.5],PARAMETER[\"false_ea");
    add_srs_wkt (p, 12,
		 "sting\",2000000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"32002\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 32003, "epsg", 32003,
		      "NAD27 / Montana South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.4 +lat_2=44.86666666666667 +lat_0=44");
    add_proj4text (p, 1,
		   " +lon_0=-109.5 +x_0=609601.2192024384 +y_0=0 +ellps=clrk");
    add_proj4text (p, 2, "66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Montana South\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",46.4]");
    add_srs_wkt (p, 9,
		 ",PARAMETER[\"standard_parallel_2\",44.86666666666667],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",44],PARAMETER[\"central_m");
    add_srs_wkt (p, 11,
		 "eridian\",-109.5],PARAMETER[\"false_easting\",2000000],P");
    add_srs_wkt (p, 12,
		 "ARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3200");
    add_srs_wkt (p, 13, "3\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32005, "epsg", 32005,
		      "NAD27 / Nebraska North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.85 +lat_2=42.81666666666667 +lat_0=4");
    add_proj4text (p, 1,
		   "1.33333333333334 +lon_0=-100 +x_0=609601.2192024384 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Nebraska North\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",41.85");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"standard_parallel_2\",42.81666666666667],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"latitude_of_origin\",41.33333333333334],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"central_meridian\",-100],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"32005\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32006, "epsg", 32006,
		      "NAD27 / Nebraska South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.28333333333333 +lat_2=41.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=39.66666666666666 +lon_0=-99.5 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Nebraska South\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.28");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",41.71666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"latitude_of_origin\",39.666666666");
    add_srs_wkt (p, 11,
		 "66666],PARAMETER[\"central_meridian\",-99.5],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",2000000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 13,
		 ",AUTHORITY[\"EPSG\",\"32006\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 32007, "epsg", 32007, "NAD27 / Nevada East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-115.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=");
    add_proj4text (p, 2, "NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Nevada East\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",34.75],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-115.5833333333333],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.9999],PARAMETER[\"false_easting\",500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"320");
    add_srs_wkt (p, 12, "07\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32008, "epsg", 32008,
		      "NAD27 / Nevada Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-116.6666666666667 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=");
    add_proj4text (p, 2, "NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Nevada Central\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",34.75],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-116.6666666666667],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "32008\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32009, "epsg", 32009, "NAD27 / Nevada West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-118.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=");
    add_proj4text (p, 2, "NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Nevada West\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",34.75],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-118.5833333333333],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.9999],PARAMETER[\"false_easting\",500000],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"320");
    add_srs_wkt (p, 12, "09\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32010, "epsg", 32010,
		      "NAD27 / New Hampshire");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-71.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +da");
    add_proj4text (p, 2, "tum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New Hampshire\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",42.5],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-71.66666666666667],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999966667],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"32010\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32011, "epsg", 32011, "NAD27 / New Jersey");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.66666666");
    add_proj4text (p, 1,
		   "666667 +k=0.9999749999999999 +x_0=609601.2192024384 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New Jersey\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",38.83333333333334");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-74.66666666666667],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999975],PARAMETER[\"false_east");
    add_srs_wkt (p, 11,
		 "ing\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"32011\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 32012, "epsg", 32012,
		      "NAD27 / New Mexico East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-104.3333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "909091 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datu");
    add_proj4text (p, 2, "m=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New Mexico East\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",31],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-104.3333333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999909091],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"32012\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32013, "epsg", 32013,
		      "NAD27 / New Mexico Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-106.25 +k=0.9999 +x_0=1524");
    add_proj4text (p, 1,
		   "00.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=u");
    add_proj4text (p, 2, "s-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New Mexico Central\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transvers");
    add_srs_wkt (p, 8,
		 "e_Mercator\"],PARAMETER[\"latitude_of_origin\",31],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"central_meridian\",-106.25],PARAMETER[\"scale_fac");
    add_srs_wkt (p, 10,
		 "tor\",0.9999],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32013\"],");
    add_srs_wkt (p, 12, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32014, "epsg", 32014,
		      "NAD27 / New Mexico West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-107.8333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "916667 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datu");
    add_proj4text (p, 2, "m=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New Mexico West\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",31],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-107.8333333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.999916667],PARAMETER[\"false_easting\",5");
    add_srs_wkt (p, 11,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 12, ",\"32014\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32015, "epsg", 32015,
		      "NAD27 / New York East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-74.33333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "966667 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datu");
    add_proj4text (p, 2, "m=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New York East\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",40],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-74.33333333333333],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.999966667],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"32015\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32016, "epsg", 32016,
		      "NAD27 / New York Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-76.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=");
    add_proj4text (p, 2, "NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New York Central\",GEOGCS[\"NAD27\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_M");
    add_srs_wkt (p, 8,
		 "ercator\"],PARAMETER[\"latitude_of_origin\",40],PARAMETE");
    add_srs_wkt (p, 9,
		 "R[\"central_meridian\",-76.58333333333333],PARAMETER[\"s");
    add_srs_wkt (p, 10,
		 "cale_factor\",0.9999375],PARAMETER[\"false_easting\",500");
    add_srs_wkt (p, 11,
		 "000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 12, "\"32016\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32017, "epsg", 32017,
		      "NAD27 / New York West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-78.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=");
    add_proj4text (p, 2, "NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New York West\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",40],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"central_meridian\",-78.58333333333333],PARAMETER[\"sca");
    add_srs_wkt (p, 10,
		 "le_factor\",0.9999375],PARAMETER[\"false_easting\",50000");
    add_srs_wkt (p, 11,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "32017\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32018, "epsg", 32018,
		      "NAD27 / New York Long Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.03333333333333 +lat_2=40.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=40.5 +lon_0=-74 +x_0=304800.6096012192 +y_0=0");
    add_proj4text (p, 2, " +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / New York Long Island\",GEOGCS[\"NAD27\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 2,
		 "866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 7,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 8,
		 "Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 9,
		 ",41.03333333333333],PARAMETER[\"standard_parallel_2\",40");
    add_srs_wkt (p, 10,
		 ".66666666666666],PARAMETER[\"latitude_of_origin\",40.5],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"central_meridian\",-74],PARAMETER[\"false_ea");
    add_srs_wkt (p, 12,
		 "sting\",1000000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"32018\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 32019, "epsg", 32019,
		      "NAD27 / North Carolina");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.33333333333334 +lat_2=36.16666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=33.75 +lon_0=-79 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2, "0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / North Carolina\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",34.33");
    add_srs_wkt (p, 9,
		 "333333333334],PARAMETER[\"standard_parallel_2\",36.16666");
    add_srs_wkt (p, 10,
		 "666666666],PARAMETER[\"latitude_of_origin\",33.75],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"central_meridian\",-79],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"32019\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32020, "epsg", 32020,
		      "NAD27 / North Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.43333333333333 +lat_2=48.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=47 +lon_0=-100.5 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2, "0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / North Dakota North\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "47.43333333333333],PARAMETER[\"standard_parallel_2\",48.");
    add_srs_wkt (p, 10,
		 "73333333333333],PARAMETER[\"latitude_of_origin\",47],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"central_meridian\",-100.5],PARAMETER[\"false_ea");
    add_srs_wkt (p, 12,
		 "sting\",2000000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 13,
		 "TY[\"EPSG\",\"32020\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 14, "]]");
    p = add_epsg_def (first, last, 32021, "epsg", 32021,
		      "NAD27 / North Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.18333333333333 +lat_2=47.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=45.66666666666666 +lon_0=-100.5 +x_0=609601.2");
    add_proj4text (p, 2,
		   "192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / North Dakota South\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "46.18333333333333],PARAMETER[\"standard_parallel_2\",47.");
    add_srs_wkt (p, 10,
		 "48333333333333],PARAMETER[\"latitude_of_origin\",45.6666");
    add_srs_wkt (p, 11,
		 "6666666666],PARAMETER[\"central_meridian\",-100.5],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_easting\",2000000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 13,
		 "ng\",0],AUTHORITY[\"EPSG\",\"32021\"],AXIS[\"X\",EAST],A");
    add_srs_wkt (p, 14, "XIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32022, "epsg", 32022, "NAD27 / Ohio North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.43333333333333 +lat_2=41.7 +lat_0=39");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-82.5 +x_0=609601.2192024384 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Ohio North\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.43333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",41.7],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"latitude_of_origin\",39.66666666666666],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"central_meridian\",-82.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"32022\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32023, "epsg", 32023, "NAD27 / Ohio South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.73333333333333 +lat_2=40.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=38 +lon_0=-82.5 +x_0=609601.2192024384 +y_0=0");
    add_proj4text (p, 2, " +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Ohio South\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.73333");
    add_srs_wkt (p, 9,
		 "333333333],PARAMETER[\"standard_parallel_2\",40.03333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"latitude_of_origin\",38],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-82.5],PARAMETER[\"false_easting\",20");
    add_srs_wkt (p, 12,
		 "00000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"32023\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32024, "epsg", 32024,
		      "NAD27 / Oklahoma North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.56666666666667 +lat_2=36.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=35 +lon_0=-98 +x_0=609601.2192024384 +y_0=0 +");
    add_proj4text (p, 2, "ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Oklahoma North\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",35.56");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",36.76666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"latitude_of_origin\",35],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-98],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 12,
		 "2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 13, "G\",\"32024\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32025, "epsg", 32025,
		      "NAD27 / Oklahoma South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.93333333333333 +lat_2=35.23333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.33333333333334 +lon_0=-98 +x_0=609601.2192");
    add_proj4text (p, 2,
		   "024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Oklahoma South\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",33.93");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",35.23333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",33.333333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-98],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",2000000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"32025\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 32026, "epsg", 32026,
		      "NAD27 / Oregon North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.33333333333334 +lat_2=46 +lat_0=43.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2, "0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Oregon North\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",44.33333");
    add_srs_wkt (p, 9,
		 "333333334],PARAMETER[\"standard_parallel_2\",46],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"latitude_of_origin\",43.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-120.5],PARAMETER[\"false_easting\",2");
    add_srs_wkt (p, 12,
		 "000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13, "\",\"32026\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32027, "epsg", 32027,
		      "NAD27 / Oregon South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.33333333333334 +lat_2=44 +lat_0=41.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=609601.2192024384 +y_0=");
    add_proj4text (p, 2, "0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Oregon South\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",42.33333");
    add_srs_wkt (p, 9,
		 "333333334],PARAMETER[\"standard_parallel_2\",44],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"latitude_of_origin\",41.66666666666666],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "central_meridian\",-120.5],PARAMETER[\"false_easting\",2");
    add_srs_wkt (p, 12,
		 "000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 13, "\",\"32027\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32028, "epsg", 32028,
		      "NAD27 / Pennsylvania North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.88333333333333 +lat_2=41.95 +lat_0=4");
    add_proj4text (p, 1,
		   "0.16666666666666 +lon_0=-77.75 +x_0=609601.2192024384 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Pennsylvania North\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "40.88333333333333],PARAMETER[\"standard_parallel_2\",41.");
    add_srs_wkt (p, 10,
		 "95],PARAMETER[\"latitude_of_origin\",40.16666666666666],");
    add_srs_wkt (p, 11,
		 "PARAMETER[\"central_meridian\",-77.75],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_easting\",2000000],PARAMETER[\"false_northing\",0],AUTH");
    add_srs_wkt (p, 13,
		 "ORITY[\"EPSG\",\"32028\"],AXIS[\"X\",EAST],AXIS[\"Y\",NO");
    add_srs_wkt (p, 14, "RTH]]");
    p = add_epsg_def (first, last, 32029, "epsg", 32029,
		      "NAD27 / Pennsylvania South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.93333333333333 +lat_2=40.8 +lat_0=39");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-77.75 +x_0=609601.2192024384 +y_");
    add_proj4text (p, 2,
		   "0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Pennsylvania South\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "39.93333333333333],PARAMETER[\"standard_parallel_2\",40.");
    add_srs_wkt (p, 10,
		 "8],PARAMETER[\"latitude_of_origin\",39.33333333333334],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"central_meridian\",-77.75],PARAMETER[\"false_");
    add_srs_wkt (p, 12,
		 "easting\",2000000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"32029\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 32030, "epsg", 32030,
		      "NAD27 / Rhode Island");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.08333333333334 +lon_0=-71.5 +k=0.9");
    add_proj4text (p, 1,
		   "999938 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +datu");
    add_proj4text (p, 2, "m=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Rhode Island\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",41.08333333333334");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-71.5],PARAMETER[\"scal");
    add_srs_wkt (p, 10,
		 "e_factor\",0.9999938],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"3");
    add_srs_wkt (p, 12, "2030\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32031, "epsg", 32031,
		      "NAD27 / South Carolina North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.76666666666667 +lat_2=34.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=33 +lon_0=-81 +x_0=609601.2192024384 +y_0=0 +");
    add_proj4text (p, 2, "ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / South Carolina North\",GEOGCS[\"NAD27\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 2,
		 "866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 7,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 8,
		 "Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 9,
		 ",33.76666666666667],PARAMETER[\"standard_parallel_2\",34");
    add_srs_wkt (p, 10,
		 ".96666666666667],PARAMETER[\"latitude_of_origin\",33],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"central_meridian\",-81],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 13,
		 "[\"EPSG\",\"32031\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 14, "");
    p = add_epsg_def (first, last, 32033, "epsg", 32033,
		      "NAD27 / South Carolina South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.33333333333334 +lat_2=33.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=31.83333333333333 +lon_0=-81 +x_0=609601.2192");
    add_proj4text (p, 2,
		   "024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / South Carolina South\",GEOGCS[\"NAD27\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 2,
		 "866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 7,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_");
    add_srs_wkt (p, 8,
		 "Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\"");
    add_srs_wkt (p, 9,
		 ",32.33333333333334],PARAMETER[\"standard_parallel_2\",33");
    add_srs_wkt (p, 10,
		 ".66666666666666],PARAMETER[\"latitude_of_origin\",31.833");
    add_srs_wkt (p, 11,
		 "33333333333],PARAMETER[\"central_meridian\",-81],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",2000000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",0],AUTHORITY[\"EPSG\",\"32033\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32034, "epsg", 32034,
		      "NAD27 / South Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.41666666666666 +lat_2=45.68333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=43.83333333333334 +lon_0=-100 +x_0=609601.219");
    add_proj4text (p, 2,
		   "2024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 3, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / South Dakota North\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "44.41666666666666],PARAMETER[\"standard_parallel_2\",45.");
    add_srs_wkt (p, 10,
		 "68333333333333],PARAMETER[\"latitude_of_origin\",43.8333");
    add_srs_wkt (p, 11,
		 "3333333334],PARAMETER[\"central_meridian\",-100],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_easting\",2000000],PARAMETER[\"false_northing");
    add_srs_wkt (p, 13,
		 "\",0],AUTHORITY[\"EPSG\",\"32034\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 14, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32035, "epsg", 32035,
		      "NAD27 / South Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.83333333333334 +lat_2=44.4 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / South Dakota South\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "42.83333333333334],PARAMETER[\"standard_parallel_2\",44.");
    add_srs_wkt (p, 10,
		 "4],PARAMETER[\"latitude_of_origin\",42.33333333333334],P");
    add_srs_wkt (p, 11,
		 "ARAMETER[\"central_meridian\",-100.3333333333333],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_easting\",2000000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 13,
		 "g\",0],AUTHORITY[\"EPSG\",\"32035\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 14, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32036, "epsg", 32036,
		      "NAD27 / Tennessee (deprecated)");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.25 +lat_2=36.41666666666666 +lat_0=3");
    add_proj4text (p, 1,
		   "4.66666666666666 +lon_0=-86 +x_0=30480.06096012192 +y_0=");
    add_proj4text (p, 2, "0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Tennessee (deprecated)\",GEOGCS[\"NAD27");
    add_srs_wkt (p, 1,
		 "\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke");
    add_srs_wkt (p, 2,
		 " 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 3,
		 "7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwic");
    add_srs_wkt (p, 4,
		 "h\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01");
    add_srs_wkt (p, 5,
		 "745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[");
    add_srs_wkt (p, 6,
		 "\"EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096");
    add_srs_wkt (p, 7,
		 "012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lamber");
    add_srs_wkt (p, 8,
		 "t_Conformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1");
    add_srs_wkt (p, 9,
		 "\",35.25],PARAMETER[\"standard_parallel_2\",36.416666666");
    add_srs_wkt (p, 10,
		 "66666],PARAMETER[\"latitude_of_origin\",34.6666666666666");
    add_srs_wkt (p, 11,
		 "6],PARAMETER[\"central_meridian\",-86],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_easting\",100000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"32036\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 32037, "epsg", 32037, "NAD27 / Texas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.65 +lat_2=36.18333333333333 +lat_0=3");
    add_proj4text (p, 1,
		   "4 +lon_0=-101.5 +x_0=609601.2192024384 +y_0=0 +ellps=clr");
    add_proj4text (p, 2, "k66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Texas North\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",34.65],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"standard_parallel_2\",36.18333333333333],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",34],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-101.5],PARAMETER[\"false_easting\",2000000],PAR");
    add_srs_wkt (p, 12,
		 "AMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32037\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32038, "epsg", 32038,
		      "NAD27 / Texas North Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=32.13333333333333 +lat_2=33.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=31.66666666666667 +lon_0=-97.5 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Texas North Central\",GEOGCS[\"NAD27\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "66\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "08\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "32.13333333333333],PARAMETER[\"standard_parallel_2\",33.");
    add_srs_wkt (p, 10,
		 "96666666666667],PARAMETER[\"latitude_of_origin\",31.6666");
    add_srs_wkt (p, 11,
		 "6666666667],PARAMETER[\"central_meridian\",-97.5],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_easting\",2000000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 13,
		 "g\",0],AUTHORITY[\"EPSG\",\"32038\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 14, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32039, "epsg", 32039,
		      "NAD27 / Texas Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.11666666666667 +lat_2=31.88333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=29.66666666666667 +lon_0=-100.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=609601.2192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 ");
    add_proj4text (p, 3, "+units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Texas Central\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",30.11");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",31.88333");
    add_srs_wkt (p, 10,
		 "333333333],PARAMETER[\"latitude_of_origin\",29.666666666");
    add_srs_wkt (p, 11,
		 "66667],PARAMETER[\"central_meridian\",-100.3333333333333");
    add_srs_wkt (p, 12,
		 "],PARAMETER[\"false_easting\",2000000],PARAMETER[\"false");
    add_srs_wkt (p, 13,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"32039\"],AXIS[\"X\",");
    add_srs_wkt (p, 14, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32040, "epsg", 32040,
		      "NAD27 / Texas South Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=28.38333333333333 +lat_2=30.28333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=27.83333333333333 +lon_0=-99 +x_0=609601.2192");
    add_proj4text (p, 2,
		   "024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Texas South Central\",GEOGCS[\"NAD27\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "66\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "08\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "28.38333333333333],PARAMETER[\"standard_parallel_2\",30.");
    add_srs_wkt (p, 10,
		 "28333333333333],PARAMETER[\"latitude_of_origin\",27.8333");
    add_srs_wkt (p, 11,
		 "3333333333],PARAMETER[\"central_meridian\",-99],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",2000000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"32040\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 14, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32041, "epsg", 32041, "NAD27 / Texas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=26.16666666666667 +lat_2=27.83333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=25.66666666666667 +lon_0=-98.5 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Texas South\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",26.16666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",27.83333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"latitude_of_origin\",25.666666666666");
    add_srs_wkt (p, 11,
		 "67],PARAMETER[\"central_meridian\",-98.5],PARAMETER[\"fa");
    add_srs_wkt (p, 12,
		 "lse_easting\",2000000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 13,
		 "UTHORITY[\"EPSG\",\"32041\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 32042, "epsg", 32042, "NAD27 / Utah North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.71666666666667 +lat_2=41.78333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=609601.2");
    add_proj4text (p, 2,
		   "192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Utah North\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",40.71666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",41.78333333");
    add_srs_wkt (p, 10,
		 "333333],PARAMETER[\"latitude_of_origin\",40.333333333333");
    add_srs_wkt (p, 11,
		 "34],PARAMETER[\"central_meridian\",-111.5],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_easting\",2000000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 13,
		 "AUTHORITY[\"EPSG\",\"32042\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 14, ",NORTH]]");
    p = add_epsg_def (first, last, 32043, "epsg", 32043,
		      "NAD27 / Utah Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.01666666666667 +lat_2=40.65 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=609601.2192024384 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Utah Central\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",39.01666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",40.65],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",38.33333333333334],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-111.5],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"32043\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32044, "epsg", 32044, "NAD27 / Utah South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.21666666666667 +lat_2=38.35 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=609601.2192024384 +y");
    add_proj4text (p, 2,
		   "_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Utah South\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conformal");
    add_srs_wkt (p, 8,
		 "_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",37.21666");
    add_srs_wkt (p, 9,
		 "666666667],PARAMETER[\"standard_parallel_2\",38.35],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"latitude_of_origin\",36.66666666666666],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"central_meridian\",-111.5],PARAMETER[\"false_easting");
    add_srs_wkt (p, 12,
		 "\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"32044\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32045, "epsg", 32045, "NAD27 / Vermont");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-72.5 +k=0.999964286 +x_0");
    add_proj4text (p, 1,
		   "=152400.3048006096 +y_0=0 +ellps=clrk66 +datum=NAD27 +un");
    add_proj4text (p, 2, "its=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Vermont\",GEOGCS[\"NAD27\",DATUM[\"Nort");
    add_srs_wkt (p, 1,
		 "h_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378206");
    add_srs_wkt (p, 2,
		 ".4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "7\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORI");
    add_srs_wkt (p, 7,
		 "TY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator\"");
    add_srs_wkt (p, 8,
		 "],PARAMETER[\"latitude_of_origin\",42.5],PARAMETER[\"cen");
    add_srs_wkt (p, 9,
		 "tral_meridian\",-72.5],PARAMETER[\"scale_factor\",0.9999");
    add_srs_wkt (p, 10,
		 "64286],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32045\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32046, "epsg", 32046,
		      "NAD27 / Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.03333333333333 +lat_2=39.2 +lat_0=37");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-78.5 +x_0=609601.2192024384 +y_0");
    add_proj4text (p, 2, "=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Virginia North\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",38.03");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"standard_parallel_2\",39.2],PA");
    add_srs_wkt (p, 10,
		 "RAMETER[\"latitude_of_origin\",37.66666666666666],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-78.5],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"32046\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32047, "epsg", 32047,
		      "NAD27 / Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.76666666666667 +lat_2=37.96666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.33333333333334 +lon_0=-78.5 +x_0=609601.21");
    add_proj4text (p, 2,
		   "92024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft ");
    add_proj4text (p, 3, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Virginia South\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Confor");
    add_srs_wkt (p, 8,
		 "mal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",36.76");
    add_srs_wkt (p, 9,
		 "666666666667],PARAMETER[\"standard_parallel_2\",37.96666");
    add_srs_wkt (p, 10,
		 "666666667],PARAMETER[\"latitude_of_origin\",36.333333333");
    add_srs_wkt (p, 11,
		 "33334],PARAMETER[\"central_meridian\",-78.5],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",2000000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 13,
		 ",AUTHORITY[\"EPSG\",\"32047\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 32048, "epsg", 32048,
		      "NAD27 / Washington North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.5 +lat_2=48.73333333333333 +lat_0=47");
    add_proj4text (p, 1,
		   " +lon_0=-120.8333333333333 +x_0=609601.2192024384 +y_0=0");
    add_proj4text (p, 2, " +ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Washington North\",GEOGCS[\"NAD27\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",47.");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"standard_parallel_2\",48.73333333333333],");
    add_srs_wkt (p, 10,
		 "PARAMETER[\"latitude_of_origin\",47],PARAMETER[\"central");
    add_srs_wkt (p, 11,
		 "_meridian\",-120.8333333333333],PARAMETER[\"false_eastin");
    add_srs_wkt (p, 12,
		 "g\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 13,
		 "EPSG\",\"32048\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32049, "epsg", 32049,
		      "NAD27 / Washington South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.83333333333334 +lat_2=47.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=45.33333333333334 +lon_0=-120.5 +x_0=609601.2");
    add_proj4text (p, 2,
		   "192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft");
    add_proj4text (p, 3, " +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Washington South\",GEOGCS[\"NAD27\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",45.");
    add_srs_wkt (p, 9,
		 "83333333333334],PARAMETER[\"standard_parallel_2\",47.333");
    add_srs_wkt (p, 10,
		 "33333333334],PARAMETER[\"latitude_of_origin\",45.3333333");
    add_srs_wkt (p, 11,
		 "3333334],PARAMETER[\"central_meridian\",-120.5],PARAMETE");
    add_srs_wkt (p, 12,
		 "R[\"false_easting\",2000000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 13,
		 ",0],AUTHORITY[\"EPSG\",\"32049\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 14, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32050, "epsg", 32050,
		      "NAD27 / West Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39 +lat_2=40.25 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=609601.2192024384 +y_0=0 +ellps=clrk66 +datum=NAD");
    add_proj4text (p, 2, "27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / West Virginia North\",GEOGCS[\"NAD27\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "66\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "08\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "39],PARAMETER[\"standard_parallel_2\",40.25],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "latitude_of_origin\",38.5],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-79.5],PARAMETER[\"false_easting\",2000000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32050\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32051, "epsg", 32051,
		      "NAD27 / West Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.48333333333333 +lat_2=38.88333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=609601.2192024384 +y_0=0 +");
    add_proj4text (p, 2, "ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / West Virginia South\",GEOGCS[\"NAD27\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 18");
    add_srs_wkt (p, 2,
		 "66\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"70");
    add_srs_wkt (p, 3,
		 "08\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "37.48333333333333],PARAMETER[\"standard_parallel_2\",38.");
    add_srs_wkt (p, 10,
		 "88333333333333],PARAMETER[\"latitude_of_origin\",37],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"central_meridian\",-81],PARAMETER[\"false_easti");
    add_srs_wkt (p, 12,
		 "ng\",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[");
    add_srs_wkt (p, 13,
		 "\"EPSG\",\"32051\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32052, "epsg", 32052,
		      "NAD27 / Wisconsin North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.56666666666667 +lat_2=46.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=45.16666666666666 +lon_0=-90 +x_0=609601.2192");
    add_proj4text (p, 2,
		   "024384 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wisconsin North\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",45.");
    add_srs_wkt (p, 9,
		 "56666666666667],PARAMETER[\"standard_parallel_2\",46.766");
    add_srs_wkt (p, 10,
		 "66666666667],PARAMETER[\"latitude_of_origin\",45.1666666");
    add_srs_wkt (p, 11,
		 "6666666],PARAMETER[\"central_meridian\",-90],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_easting\",2000000],PARAMETER[\"false_northing\",0]");
    add_srs_wkt (p, 13,
		 ",AUTHORITY[\"EPSG\",\"32052\"],AXIS[\"X\",EAST],AXIS[\"Y");
    add_srs_wkt (p, 14, "\",NORTH]]");
    p = add_epsg_def (first, last, 32053, "epsg", 32053,
		      "NAD27 / Wisconsin Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.25 +lat_2=45.5 +lat_0=43.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-90 +x_0=609601.2192024384 +y_0=0 +ellps=clrk");
    add_proj4text (p, 2, "66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wisconsin Central\",GEOGCS[\"NAD27\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866");
    add_srs_wkt (p, 2,
		 "\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008");
    add_srs_wkt (p, 3,
		 "\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",");
    add_srs_wkt (p, 4,
		 "0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453");
    add_srs_wkt (p, 5,
		 "29251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 6,
		 "SG\",\"4267\"]],UNIT[\"US survey foot\",0.30480060960121");
    add_srs_wkt (p, 7,
		 "92,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Co");
    add_srs_wkt (p, 8,
		 "nformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",4");
    add_srs_wkt (p, 9,
		 "4.25],PARAMETER[\"standard_parallel_2\",45.5],PARAMETER[");
    add_srs_wkt (p, 10,
		 "\"latitude_of_origin\",43.83333333333334],PARAMETER[\"ce");
    add_srs_wkt (p, 11,
		 "ntral_meridian\",-90],PARAMETER[\"false_easting\",200000");
    add_srs_wkt (p, 12,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 13, "32053\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32054, "epsg", 32054,
		      "NAD27 / Wisconsin South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=42.73333333333333 +lat_2=44.06666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=42 +lon_0=-90 +x_0=609601.2192024384 +y_0=0 +");
    add_proj4text (p, 2, "ellps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wisconsin South\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192");
    add_srs_wkt (p, 7,
		 ",AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_Conf");
    add_srs_wkt (p, 8,
		 "ormal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",42.");
    add_srs_wkt (p, 9,
		 "73333333333333],PARAMETER[\"standard_parallel_2\",44.066");
    add_srs_wkt (p, 10,
		 "66666666667],PARAMETER[\"latitude_of_origin\",42],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"central_meridian\",-90],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 12,
		 ",2000000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 13, "SG\",\"32054\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32055, "epsg", 32055,
		      "NAD27 / Wyoming East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.66666666666666 +lon_0=-105.1666666");
    add_proj4text (p, 1,
		   "666667 +k=0.999941177 +x_0=152400.3048006096 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wyoming East\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",40.66666666666666");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-105.1666666666667],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999941177],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"32055\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 32056, "epsg", 32056,
		      "NAD27 / Wyoming East Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.66666666666666 +lon_0=-107.3333333");
    add_proj4text (p, 1,
		   "333333 +k=0.999941177 +x_0=152400.3048006096 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wyoming East Central\",GEOGCS[\"NAD27\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 2,
		 "866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 7,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transver");
    add_srs_wkt (p, 8,
		 "se_Mercator\"],PARAMETER[\"latitude_of_origin\",40.66666");
    add_srs_wkt (p, 9,
		 "666666666],PARAMETER[\"central_meridian\",-107.333333333");
    add_srs_wkt (p, 10,
		 "3333],PARAMETER[\"scale_factor\",0.999941177],PARAMETER[");
    add_srs_wkt (p, 11,
		 "\"false_easting\",500000],PARAMETER[\"false_northing\",0");
    add_srs_wkt (p, 12,
		 "],AUTHORITY[\"EPSG\",\"32056\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 32057, "epsg", 32057,
		      "NAD27 / Wyoming West Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.66666666666666 +lon_0=-108.75 +k=0");
    add_proj4text (p, 1,
		   ".999941177 +x_0=152400.3048006096 +y_0=0 +ellps=clrk66 +");
    add_proj4text (p, 2, "datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wyoming West Central\",GEOGCS[\"NAD27\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1");
    add_srs_wkt (p, 2,
		 "866\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7");
    add_srs_wkt (p, 3,
		 "008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich");
    add_srs_wkt (p, 4,
		 "\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017");
    add_srs_wkt (p, 5,
		 "45329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"");
    add_srs_wkt (p, 6,
		 "EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.304800609601");
    add_srs_wkt (p, 7,
		 "2192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transver");
    add_srs_wkt (p, 8,
		 "se_Mercator\"],PARAMETER[\"latitude_of_origin\",40.66666");
    add_srs_wkt (p, 9,
		 "666666666],PARAMETER[\"central_meridian\",-108.75],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"scale_factor\",0.999941177],PARAMETER[\"false_eas");
    add_srs_wkt (p, 11,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 12,
		 "[\"EPSG\",\"32057\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 13, "");
    p = add_epsg_def (first, last, 32058, "epsg", 32058,
		      "NAD27 / Wyoming West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.66666666666666 +lon_0=-110.0833333");
    add_proj4text (p, 1,
		   "333333 +k=0.999941177 +x_0=152400.3048006096 +y_0=0 +ell");
    add_proj4text (p, 2, "ps=clrk66 +datum=NAD27 +units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Wyoming West\",GEOGCS[\"NAD27\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",637");
    add_srs_wkt (p, 2,
		 "8206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"US survey foot\",0.3048006096012192,AUTH");
    add_srs_wkt (p, 7,
		 "ORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercat");
    add_srs_wkt (p, 8,
		 "or\"],PARAMETER[\"latitude_of_origin\",40.66666666666666");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-110.0833333333333],PAR");
    add_srs_wkt (p, 10,
		 "AMETER[\"scale_factor\",0.999941177],PARAMETER[\"false_e");
    add_srs_wkt (p, 11,
		 "asting\",500000],PARAMETER[\"false_northing\",0],AUTHORI");
    add_srs_wkt (p, 12,
		 "TY[\"EPSG\",\"32058\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH");
    add_srs_wkt (p, 13, "]]");
    p = add_epsg_def (first, last, 32061, "epsg", 32061,
		      "NAD27 / Guatemala Norte");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=16.81666666666667 +lat_0=16.81666666666");
    add_proj4text (p, 1,
		   "667 +lon_0=-90.33333333333333 +k_0=0.99992226 +x_0=50000");
    add_proj4text (p, 2,
		   "0 +y_0=292209.579 +ellps=clrk66 +datum=NAD27 +units=m +n");
    add_proj4text (p, 3, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Guatemala Norte\",GEOGCS[\"NAD27\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\"");
    add_srs_wkt (p, 2,
		 ",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"latitude_of_origin\",16.81666666666667],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-90.33333333333333],PARAMETER[\"scale");
    add_srs_wkt (p, 10,
		 "_factor\",0.99992226],PARAMETER[\"false_easting\",500000");
    add_srs_wkt (p, 11,
		 "],PARAMETER[\"false_northing\",292209.579],AUTHORITY[\"E");
    add_srs_wkt (p, 12, "PSG\",\"32061\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32062, "epsg", 32062,
		      "NAD27 / Guatemala Sur");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=14.9 +lat_0=14.9 +lon_0=-90.33333333333");
    add_proj4text (p, 1,
		   "333 +k_0=0.99989906 +x_0=500000 +y_0=325992.681 +ellps=c");
    add_proj4text (p, 2, "lrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Guatemala Sur\",GEOGCS[\"NAD27\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6");
    add_srs_wkt (p, 2,
		 "378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]]");
    add_srs_wkt (p, 3,
		 ",AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AU");
    add_srs_wkt (p, 4,
		 "THORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925");
    add_srs_wkt (p, 5,
		 "1994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_1SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"latitude_of_origin\",14.9],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 9,
		 "an\",-90.33333333333333],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 10,
		 "989906],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",325992.681],AUTHORITY[\"EPSG\",\"32062\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32064, "epsg", 32064,
		      "NAD27 / BLM 14N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-99 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 14N (ftUS)\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-99],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",1640416.67],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32064\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32065, "epsg", 32065,
		      "NAD27 / BLM 15N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-93 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 15N (ftUS)\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-93],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",1640416.67],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32065\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32066, "epsg", 32066,
		      "NAD27 / BLM 16N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-87 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 16N (ftUS)\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-87],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",1640416.67],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32066\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32067, "epsg", 32067,
		      "NAD27 / BLM 17N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-81 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 17N (ftUS)\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012192,A");
    add_srs_wkt (p, 7,
		 "UTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mer");
    add_srs_wkt (p, 8,
		 "cator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "central_meridian\",-81],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 10,
		 "6],PARAMETER[\"false_easting\",1640416.67],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32067\"],AXIS[\"");
    add_srs_wkt (p, 12, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32074, "epsg", 32074,
		      "NAD27 / BLM 14N (feet) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-99 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 14N (feet) (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.30480");
    add_srs_wkt (p, 7,
		 "06096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-99],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9996],PARAMETER[\"false_easting\",1640416.67]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32");
    add_srs_wkt (p, 12, "074\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32075, "epsg", 32075,
		      "NAD27 / BLM 15N (feet) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-93 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 15N (feet) (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.30480");
    add_srs_wkt (p, 7,
		 "06096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-93],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9996],PARAMETER[\"false_easting\",1640416.67]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32");
    add_srs_wkt (p, 12, "075\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32076, "epsg", 32076,
		      "NAD27 / BLM 16N (feet) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-87 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 16N (feet) (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.30480");
    add_srs_wkt (p, 7,
		 "06096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-87],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9996],PARAMETER[\"false_easting\",1640416.67]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32");
    add_srs_wkt (p, 12, "076\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32077, "epsg", 32077,
		      "NAD27 / BLM 17N (feet) (deprecated)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-81 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=us-ft +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / BLM 17N (feet) (deprecated)\",GEOGCS[\"");
    add_srs_wkt (p, 1,
		 "NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"C");
    add_srs_wkt (p, 2,
		 "larke 1866\",6378206.4,294.9786982139006,AUTHORITY[\"EPS");
    add_srs_wkt (p, 3,
		 "G\",\"7008\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Gr");
    add_srs_wkt (p, 4,
		 "eenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\"");
    add_srs_wkt (p, 5,
		 ",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHO");
    add_srs_wkt (p, 6,
		 "RITY[\"EPSG\",\"4267\"]],UNIT[\"US survey foot\",0.30480");
    add_srs_wkt (p, 7,
		 "06096012192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"T");
    add_srs_wkt (p, 8,
		 "ransverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"central_meridian\",-81],PARAMETER[\"scale_");
    add_srs_wkt (p, 10,
		 "factor\",0.9996],PARAMETER[\"false_easting\",1640416.67]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32");
    add_srs_wkt (p, 12, "077\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32081, "epsg", 32081, "NAD27 / MTM zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-53 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / MTM zone 1\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-53],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9999],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"32081\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 32082, "epsg", 32082, "NAD27 / MTM zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-56 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / MTM zone 2\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-56],PARA");
    add_srs_wkt (p, 9,
		 "METER[\"scale_factor\",0.9999],PARAMETER[\"false_easting");
    add_srs_wkt (p, 10,
		 "\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 11,
		 "PSG\",\"32082\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 32083, "epsg", 32083, "NAD27 / MTM zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-58.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / MTM zone 3\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-58.5],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"32083\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 32084, "epsg", 32084, "NAD27 / MTM zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-61.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / MTM zone 4\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-61.5],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"32084\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 32085, "epsg", 32085, "NAD27 / MTM zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-64.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / MTM zone 5\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-64.5],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"32085\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 32086, "epsg", 32086, "NAD27 / MTM zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-67.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / MTM zone 6\",GEOGCS[\"NAD27\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378");
    add_srs_wkt (p, 2,
		 "206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]],AU");
    add_srs_wkt (p, 3,
		 "THORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,AUTHO");
    add_srs_wkt (p, 4,
		 "RITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199");
    add_srs_wkt (p, 5,
		 "4328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",0],PARAMETER[\"central_meridian\",-67.5],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_easti");
    add_srs_wkt (p, 10,
		 "ng\",304800],PARAMETER[\"false_northing\",0],AUTHORITY[\"");
    add_srs_wkt (p, 11,
		 "EPSG\",\"32086\"],AXIS[\"Easting\",EAST],AXIS[\"Northing");
    add_srs_wkt (p, 12, "\",NORTH]]");
    p = add_epsg_def (first, last, 32098, "epsg", 32098,
		      "NAD27 / Quebec Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=60 +lat_2=46 +lat_0=44 +lon_0=-68.5 +x_");
    add_proj4text (p, 1,
		   "0=0 +y_0=0 +ellps=clrk66 +datum=NAD27 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Quebec Lambert\",GEOGCS[\"NAD27\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",");
    add_srs_wkt (p, 2,
		 "6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"7008\"]");
    add_srs_wkt (p, 3,
		 "],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\",0,A");
    add_srs_wkt (p, 4,
		 "UTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292");
    add_srs_wkt (p, 5,
		 "51994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"4267\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"standard_parallel_1\",60],PARAMETER[\"standard_parall");
    add_srs_wkt (p, 9,
		 "el_2\",46],PARAMETER[\"latitude_of_origin\",44],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"central_meridian\",-68.5],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 12, "32098\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32099, "epsg", 32099,
		      "NAD27 / Louisiana Offshore");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.66666666666667 +lon_0=-91.33333333333333 +");
    add_proj4text (p, 2,
		   "x_0=609601.2192024384 +y_0=0 +ellps=clrk66 +datum=NAD27 ");
    add_proj4text (p, 3, "+units=us-ft +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD27 / Louisiana Offshore\",GEOGCS[\"NAD27\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 186");
    add_srs_wkt (p, 2,
		 "6\",6378206.4,294.9786982139006,AUTHORITY[\"EPSG\",\"700");
    add_srs_wkt (p, 3,
		 "8\"]],AUTHORITY[\"EPSG\",\"6267\"]],PRIMEM[\"Greenwich\"");
    add_srs_wkt (p, 4,
		 ",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745");
    add_srs_wkt (p, 5,
		 "329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 6,
		 "PSG\",\"4267\"]],UNIT[\"US survey foot\",0.3048006096012");
    add_srs_wkt (p, 7,
		 "192,AUTHORITY[\"EPSG\",\"9003\"]],PROJECTION[\"Lambert_C");
    add_srs_wkt (p, 8,
		 "onformal_Conic_2SP\"],PARAMETER[\"standard_parallel_1\",");
    add_srs_wkt (p, 9,
		 "27.83333333333333],PARAMETER[\"standard_parallel_2\",26.");
    add_srs_wkt (p, 10,
		 "16666666666667],PARAMETER[\"latitude_of_origin\",25.6666");
    add_srs_wkt (p, 11,
		 "6666666667],PARAMETER[\"central_meridian\",-91.333333333");
    add_srs_wkt (p, 12,
		 "33333],PARAMETER[\"false_easting\",2000000],PARAMETER[\"");
    add_srs_wkt (p, 13,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32099\"],AXIS[\"");
    add_srs_wkt (p, 14, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32100, "epsg", 32100, "NAD83 / Montana");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=49 +lat_2=45 +lat_0=44.25 +lon_0=-109.5");
    add_proj4text (p, 1,
		   " +x_0=600000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +");
    add_proj4text (p, 2, "no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Montana\",GEOGCS[\"NAD83\",DATUM[\"Nort");
    add_srs_wkt (p, 1,
		 "h_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 2,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_para");
    add_srs_wkt (p, 8,
		 "llel_1\",49],PARAMETER[\"standard_parallel_2\",45],PARAM");
    add_srs_wkt (p, 9,
		 "ETER[\"latitude_of_origin\",44.25],PARAMETER[\"central_m");
    add_srs_wkt (p, 10,
		 "eridian\",-109.5],PARAMETER[\"false_easting\",600000],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32100");
    add_srs_wkt (p, 12, "\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32104, "epsg", 32104, "NAD83 / Nebraska");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=43 +lat_2=40 +lat_0=39.83333333333334 +");
    add_proj4text (p, 1,
		   "lon_0=-100 +x_0=500000 +y_0=0 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nebraska\",GEOGCS[\"NAD83\",DATUM[\"Nor");
    add_srs_wkt (p, 1,
		 "th_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,2");
    add_srs_wkt (p, 2,
		 "98.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 8,
		 "rallel_1\",43],PARAMETER[\"standard_parallel_2\",40],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"latitude_of_origin\",39.83333333333334],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"central_meridian\",-100],PARAMETER[\"false_easting\"");
    add_srs_wkt (p, 11,
		 ",500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPS");
    add_srs_wkt (p, 12, "G\",\"32104\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32107, "epsg", 32107, "NAD83 / Nevada East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-115.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=200000 +y_0=8000000 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nevada East\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",34.75],PARAMETER[\"central_meridian\",-115.58333333");
    add_srs_wkt (p, 9,
		 "33333],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",200000],PARAMETER[\"false_northing\",80000");
    add_srs_wkt (p, 11,
		 "00],AUTHORITY[\"EPSG\",\"32107\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32108, "epsg", 32108,
		      "NAD83 / Nevada Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-116.6666666666667 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=500000 +y_0=6000000 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nevada Central\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 8,
		 "igin\",34.75],PARAMETER[\"central_meridian\",-116.666666");
    add_srs_wkt (p, 9,
		 "6666667],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",500000],PARAMETER[\"false_northing\",600");
    add_srs_wkt (p, 11,
		 "0000],AUTHORITY[\"EPSG\",\"32108\"],AXIS[\"X\",EAST],AXI");
    add_srs_wkt (p, 12, "S[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32109, "epsg", 32109, "NAD83 / Nevada West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=34.75 +lon_0=-118.5833333333333 +k=0.");
    add_proj4text (p, 1,
		   "9999 +x_0=800000 +y_0=4000000 +ellps=GRS80 +datum=NAD83 ");
    add_proj4text (p, 2, "+units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Nevada West\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",34.75],PARAMETER[\"central_meridian\",-118.58333333");
    add_srs_wkt (p, 9,
		 "33333],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",800000],PARAMETER[\"false_northing\",40000");
    add_srs_wkt (p, 11,
		 "00],AUTHORITY[\"EPSG\",\"32109\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32110, "epsg", 32110,
		      "NAD83 / New Hampshire");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-71.66666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "99966667 +x_0=300000 +y_0=0 +ellps=GRS80 +datum=NAD83 +u");
    add_proj4text (p, 2, "nits=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Hampshire\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",42.5],PARAMETER[\"central_meridian\",-71.666666666");
    add_srs_wkt (p, 9,
		 "66667],PARAMETER[\"scale_factor\",0.999966667],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",300000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"32110\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 32111, "epsg", 32111, "NAD83 / New Jersey");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Jersey\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",38.83333333333334],PARAMETER[\"central_meridian\",-74");
    add_srs_wkt (p, 9,
		 ".5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_easting\",150000],PARAMETER[\"false_northing\",0],AUTHO");
    add_srs_wkt (p, 11,
		 "RITY[\"EPSG\",\"32111\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 12, "TH]]");
    p = add_epsg_def (first, last, 32112, "epsg", 32112,
		      "NAD83 / New Mexico East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-104.3333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "909091 +x_0=165000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Mexico East\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",31],PARAMETER[\"central_meridian\",-104.3333333");
    add_srs_wkt (p, 9,
		 "333333],PARAMETER[\"scale_factor\",0.999909091],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",165000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"32112\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32113, "epsg", 32113,
		      "NAD83 / New Mexico Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-106.25 +k=0.9999 +x_0=5000");
    add_proj4text (p, 1,
		   "00 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Mexico Central\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_");
    add_srs_wkt (p, 8,
		 "of_origin\",31],PARAMETER[\"central_meridian\",-106.25],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"false_eas");
    add_srs_wkt (p, 10,
		 "ting\",500000],PARAMETER[\"false_northing\",0],AUTHORITY");
    add_srs_wkt (p, 11,
		 "[\"EPSG\",\"32113\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    add_srs_wkt (p, 12, "");
    p = add_epsg_def (first, last, 32114, "epsg", 32114,
		      "NAD83 / New Mexico West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=31 +lon_0=-107.8333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "916667 +x_0=830000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New Mexico West\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 8,
		 "origin\",31],PARAMETER[\"central_meridian\",-107.8333333");
    add_srs_wkt (p, 9,
		 "333333],PARAMETER[\"scale_factor\",0.999916667],PARAMETE");
    add_srs_wkt (p, 10,
		 "R[\"false_easting\",830000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 11,
		 ",0],AUTHORITY[\"EPSG\",\"32114\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 12, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32115, "epsg", 32115,
		      "NAD83 / New York East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=38.83333333333334 +lon_0=-74.5 +k=0.9");
    add_proj4text (p, 1,
		   "999 +x_0=150000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York East\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",38.83333333333334],PARAMETER[\"central_meridian\",");
    add_srs_wkt (p, 9,
		 "-74.5],PARAMETER[\"scale_factor\",0.9999],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_easting\",150000],PARAMETER[\"false_northing\",0],AU");
    add_srs_wkt (p, 11,
		 "THORITY[\"EPSG\",\"32115\"],AXIS[\"X\",EAST],AXIS[\"Y\",");
    add_srs_wkt (p, 12, "NORTH]]");
    p = add_epsg_def (first, last, 32116, "epsg", 32116,
		      "NAD83 / New York Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-76.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=250000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York Central\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of");
    add_srs_wkt (p, 8,
		 "_origin\",40],PARAMETER[\"central_meridian\",-76.5833333");
    add_srs_wkt (p, 9,
		 "3333333],PARAMETER[\"scale_factor\",0.9999375],PARAMETER");
    add_srs_wkt (p, 10,
		 "[\"false_easting\",250000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 11,
		 "0],AUTHORITY[\"EPSG\",\"32116\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 12, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 32117, "epsg", 32117,
		      "NAD83 / New York West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40 +lon_0=-78.58333333333333 +k=0.999");
    add_proj4text (p, 1,
		   "9375 +x_0=350000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 2, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York West\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_ori");
    add_srs_wkt (p, 8,
		 "gin\",40],PARAMETER[\"central_meridian\",-78.58333333333");
    add_srs_wkt (p, 9,
		 "333],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"f");
    add_srs_wkt (p, 10,
		 "alse_easting\",350000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 11,
		 "UTHORITY[\"EPSG\",\"32117\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 32118, "epsg", 32118,
		      "NAD83 / New York Long Island");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.03333333333333 +lat_2=40.66666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=40.16666666666666 +lon_0=-74 +x_0=300000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / New York Long Island\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "standard_parallel_1\",41.03333333333333],PARAMETER[\"sta");
    add_srs_wkt (p, 9,
		 "ndard_parallel_2\",40.66666666666666],PARAMETER[\"latitu");
    add_srs_wkt (p, 10,
		 "de_of_origin\",40.16666666666666],PARAMETER[\"central_me");
    add_srs_wkt (p, 11,
		 "ridian\",-74],PARAMETER[\"false_easting\",300000],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32118\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32119, "epsg", 32119,
		      "NAD83 / North Carolina");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=33.75 +lon_0=-79 +x_0=609601.22 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / North Carolina\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",36.16666666666666],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",34.33333333333334],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",33.75],PARAMETER[\"central_meridian\",-79],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_easting\",609601.22],PARAMETER[\"false_nor");
    add_srs_wkt (p, 12,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32119\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 13, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32120, "epsg", 32120,
		      "NAD83 / North Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.43333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=47 +lon_0=-100.5 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / North Dakota North\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",48.73333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",47.43333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",47],PARAMETER[\"central_meridian\",-100.5]");
    add_srs_wkt (p, 11,
		 ",PARAMETER[\"false_easting\",600000],PARAMETER[\"false_n");
    add_srs_wkt (p, 12,
		 "orthing\",0],AUTHORITY[\"EPSG\",\"32120\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32121, "epsg", 32121,
		      "NAD83 / North Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.48333333333333 +lat_2=46.18333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=45.66666666666666 +lon_0=-100.5 +x_0=600000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / North Dakota South\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",47.48333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",46.18333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",45.66666666666666],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-100.5],PARAMETER[\"false_easting\",600000],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32121\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32122, "epsg", 32122, "NAD83 / Ohio North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.7 +lat_2=40.43333333333333 +lat_0=39");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Ohio North\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 8,
		 "arallel_1\",41.7],PARAMETER[\"standard_parallel_2\",40.4");
    add_srs_wkt (p, 9,
		 "3333333333333],PARAMETER[\"latitude_of_origin\",39.66666");
    add_srs_wkt (p, 10,
		 "666666666],PARAMETER[\"central_meridian\",-82.5],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",600000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"32122\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32123, "epsg", 32123, "NAD83 / Ohio South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.03333333333333 +lat_2=38.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=38 +lon_0=-82.5 +x_0=600000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Ohio South\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 8,
		 "arallel_1\",40.03333333333333],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",38.73333333333333],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",38],PARAMETER[\"central_meridian\",-82.5],PARAMETER");
    add_srs_wkt (p, 11,
		 "[\"false_easting\",600000],PARAMETER[\"false_northing\",");
    add_srs_wkt (p, 12,
		 "0],AUTHORITY[\"EPSG\",\"32123\"],AXIS[\"X\",EAST],AXIS[\"");
    add_srs_wkt (p, 13, "Y\",NORTH]]");
    p = add_epsg_def (first, last, 32124, "epsg", 32124,
		      "NAD83 / Oklahoma North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.76666666666667 +lat_2=35.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=35 +lon_0=-98 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oklahoma North\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",36.76666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",35.56666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",35],PARAMETER[\"central_meridian\",-98],PARAMET");
    add_srs_wkt (p, 11,
		 "ER[\"false_easting\",600000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"32124\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32125, "epsg", 32125,
		      "NAD83 / Oklahoma South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=35.23333333333333 +lat_2=33.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=33.33333333333334 +lon_0=-98 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oklahoma South\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",35.23333333333333],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",33.93333333333333],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",33.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-98],PARAMETER[\"false_easting\",600000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32125\"],AXIS[\"");
    add_srs_wkt (p, 13, "X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32126, "epsg", 32126,
		      "NAD83 / Oregon North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46 +lat_2=44.33333333333334 +lat_0=43.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=2500000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oregon North\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",46],PARAMETER[\"standard_parallel_2\",44.3");
    add_srs_wkt (p, 9,
		 "3333333333334],PARAMETER[\"latitude_of_origin\",43.66666");
    add_srs_wkt (p, 10,
		 "666666666],PARAMETER[\"central_meridian\",-120.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",2500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"32126\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32127, "epsg", 32127,
		      "NAD83 / Oregon South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44 +lat_2=42.33333333333334 +lat_0=41.6");
    add_proj4text (p, 1,
		   "6666666666666 +lon_0=-120.5 +x_0=1500000 +y_0=0 +ellps=G");
    add_proj4text (p, 2, "RS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Oregon South\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",44],PARAMETER[\"standard_parallel_2\",42.3");
    add_srs_wkt (p, 9,
		 "3333333333334],PARAMETER[\"latitude_of_origin\",41.66666");
    add_srs_wkt (p, 10,
		 "666666666],PARAMETER[\"central_meridian\",-120.5],PARAME");
    add_srs_wkt (p, 11,
		 "TER[\"false_easting\",1500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"32127\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32128, "epsg", 32128,
		      "NAD83 / Pennsylvania North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.95 +lat_2=40.88333333333333 +lat_0=4");
    add_proj4text (p, 1,
		   "0.16666666666666 +lon_0=-77.75 +x_0=600000 +y_0=0 +ellps");
    add_proj4text (p, 2, "=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Pennsylvania North\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",41.95],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 9,
		 "l_2\",40.88333333333333],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 10,
		 ",40.16666666666666],PARAMETER[\"central_meridian\",-77.7");
    add_srs_wkt (p, 11,
		 "5],PARAMETER[\"false_easting\",600000],PARAMETER[\"false");
    add_srs_wkt (p, 12,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"32128\"],AXIS[\"X\",");
    add_srs_wkt (p, 13, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32129, "epsg", 32129,
		      "NAD83 / Pennsylvania South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.96666666666667 +lat_2=39.93333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=39.33333333333334 +lon_0=-77.75 +x_0=600000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Pennsylvania South\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",40.96666666666667],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",39.93333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",39.33333333333334],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-77.75],PARAMETER[\"false_easting\",600000],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32129\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32130, "epsg", 32130,
		      "NAD83 / Rhode Island");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=41.08333333333334 +lon_0=-71.5 +k=0.9");
    add_proj4text (p, 1,
		   "9999375 +x_0=100000 +y_0=0 +ellps=GRS80 +datum=NAD83 +un");
    add_proj4text (p, 2, "its=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Rhode Island\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",41.08333333333334],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 9,
		 "71.5],PARAMETER[\"scale_factor\",0.99999375],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",100000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"32130\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 32133, "epsg", 32133,
		      "NAD83 / South Carolina");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=34.83333333333334 +lat_2=32.5 +lat_0=31");
    add_proj4text (p, 1,
		   ".83333333333333 +lon_0=-81 +x_0=609600 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / South Carolina\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",34.83333333333334],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",32.5],PARAMETER[\"latitude_of_origin\",31.8");
    add_srs_wkt (p, 10,
		 "3333333333333],PARAMETER[\"central_meridian\",-81],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",609600],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"32133\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32134, "epsg", 32134,
		      "NAD83 / South Dakota North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.68333333333333 +lat_2=44.41666666666");
    add_proj4text (p, 1,
		   "666 +lat_0=43.83333333333334 +lon_0=-100 +x_0=600000 +y_");
    add_proj4text (p, 2, "0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / South Dakota North\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",45.68333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",44.41666666666666],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",43.83333333333334],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-100],PARAMETER[\"false_easting\",600000],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32134\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32135, "epsg", 32135,
		      "NAD83 / South Dakota South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.4 +lat_2=42.83333333333334 +lat_0=42");
    add_proj4text (p, 1,
		   ".33333333333334 +lon_0=-100.3333333333333 +x_0=600000 +y");
    add_proj4text (p, 2, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / South Dakota South\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",44.4],PARAMETER[\"standard_parallel");
    add_srs_wkt (p, 9,
		 "_2\",42.83333333333334],PARAMETER[\"latitude_of_origin\"");
    add_srs_wkt (p, 10,
		 ",42.33333333333334],PARAMETER[\"central_meridian\",-100.");
    add_srs_wkt (p, 11,
		 "3333333333333],PARAMETER[\"false_easting\",600000],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32135\"]");
    add_srs_wkt (p, 13, ",AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32136, "epsg", 32136, "NAD83 / Tennessee");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.41666666666666 +lat_2=35.25 +lat_0=3");
    add_proj4text (p, 1,
		   "4.33333333333334 +lon_0=-86 +x_0=600000 +y_0=0 +ellps=GR");
    add_proj4text (p, 2, "S80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Tennessee\",GEOGCS[\"NAD83\",DATUM[\"No");
    add_srs_wkt (p, 1,
		 "rth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,");
    add_srs_wkt (p, 2,
		 "298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 8,
		 "arallel_1\",36.41666666666666],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",35.25],PARAMETER[\"latitude_of_origin\",34.3333");
    add_srs_wkt (p, 10,
		 "3333333334],PARAMETER[\"central_meridian\",-86],PARAMETE");
    add_srs_wkt (p, 11,
		 "R[\"false_easting\",600000],PARAMETER[\"false_northing\"");
    add_srs_wkt (p, 12,
		 ",0],AUTHORITY[\"EPSG\",\"32136\"],AXIS[\"X\",EAST],AXIS[");
    add_srs_wkt (p, 13, "\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32137, "epsg", 32137, "NAD83 / Texas North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=36.18333333333333 +lat_2=34.65 +lat_0=3");
    add_proj4text (p, 1,
		   "4 +lon_0=-101.5 +x_0=200000 +y_0=1000000 +ellps=GRS80 +d");
    add_proj4text (p, 2, "atum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas North\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",36.18333333333333],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_2\",34.65],PARAMETER[\"latitude_of_origin\",34],P");
    add_srs_wkt (p, 10,
		 "ARAMETER[\"central_meridian\",-101.5],PARAMETER[\"false_");
    add_srs_wkt (p, 11,
		 "easting\",200000],PARAMETER[\"false_northing\",1000000],");
    add_srs_wkt (p, 12,
		 "AUTHORITY[\"EPSG\",\"32137\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 32138, "epsg", 32138,
		      "NAD83 / Texas North Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=33.96666666666667 +lat_2=32.13333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=31.66666666666667 +lon_0=-98.5 +x_0=600000 +y");
    add_proj4text (p, 2,
		   "_0=2000000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas North Central\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",33.96666666666667],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",32.13333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",31.66666666666667],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-98.5],PARAMETER[\"false_easting\",600000],PARAM");
    add_srs_wkt (p, 12,
		 "ETER[\"false_northing\",2000000],AUTHORITY[\"EPSG\",\"32");
    add_srs_wkt (p, 13, "138\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32139, "epsg", 32139,
		      "NAD83 / Texas Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=31.88333333333333 +lat_2=30.11666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=29.66666666666667 +lon_0=-100.3333333333333 +");
    add_proj4text (p, 2,
		   "x_0=700000 +y_0=3000000 +ellps=GRS80 +datum=NAD83 +units");
    add_proj4text (p, 3, "=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas Central\",GEOGCS[\"NAD83\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378");
    add_srs_wkt (p, 2,
		 "137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORIT");
    add_srs_wkt (p, 3,
		 "Y[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"");
    add_srs_wkt (p, 4,
		 "EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AU");
    add_srs_wkt (p, 5,
		 "THORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]");
    add_srs_wkt (p, 6,
		 "],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECT");
    add_srs_wkt (p, 7,
		 "ION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standar");
    add_srs_wkt (p, 8,
		 "d_parallel_1\",31.88333333333333],PARAMETER[\"standard_p");
    add_srs_wkt (p, 9,
		 "arallel_2\",30.11666666666667],PARAMETER[\"latitude_of_o");
    add_srs_wkt (p, 10,
		 "rigin\",29.66666666666667],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-100.3333333333333],PARAMETER[\"false_easting\",700000]");
    add_srs_wkt (p, 12,
		 ",PARAMETER[\"false_northing\",3000000],AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 13, ",\"32139\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32140, "epsg", 32140,
		      "NAD83 / Texas South Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=30.28333333333333 +lat_2=28.38333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=27.83333333333333 +lon_0=-99 +x_0=600000 +y_0");
    add_proj4text (p, 2,
		   "=4000000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas South Central\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",30.28333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",28.38333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",27.83333333333333],PARAMETER[\"central_mer");
    add_srs_wkt (p, 11,
		 "idian\",-99],PARAMETER[\"false_easting\",600000],PARAMET");
    add_srs_wkt (p, 12,
		 "ER[\"false_northing\",4000000],AUTHORITY[\"EPSG\",\"3214");
    add_srs_wkt (p, 13, "0\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32141, "epsg", 32141, "NAD83 / Texas South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.66666666666667 +lon_0=-98.5 +x_0=300000 +y");
    add_proj4text (p, 2,
		   "_0=5000000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Texas South\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",27.83333333333333],PARAMETER[\"standard_pa");
    add_srs_wkt (p, 9,
		 "rallel_2\",26.16666666666667],PARAMETER[\"latitude_of_or");
    add_srs_wkt (p, 10,
		 "igin\",25.66666666666667],PARAMETER[\"central_meridian\"");
    add_srs_wkt (p, 11,
		 ",-98.5],PARAMETER[\"false_easting\",300000],PARAMETER[\"");
    add_srs_wkt (p, 12,
		 "false_northing\",5000000],AUTHORITY[\"EPSG\",\"32141\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32142, "epsg", 32142, "NAD83 / Utah North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=41.78333333333333 +lat_2=40.71666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=40.33333333333334 +lon_0=-111.5 +x_0=500000 +");
    add_proj4text (p, 2,
		   "y_0=1000000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah North\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 8,
		 "arallel_1\",41.78333333333333],PARAMETER[\"standard_para");
    add_srs_wkt (p, 9,
		 "llel_2\",40.71666666666667],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 10,
		 "in\",40.33333333333334],PARAMETER[\"central_meridian\",-");
    add_srs_wkt (p, 11,
		 "111.5],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 12,
		 "alse_northing\",1000000],AUTHORITY[\"EPSG\",\"32142\"],A");
    add_srs_wkt (p, 13, "XIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32143, "epsg", 32143,
		      "NAD83 / Utah Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.65 +lat_2=39.01666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "8.33333333333334 +lon_0=-111.5 +x_0=500000 +y_0=2000000 ");
    add_proj4text (p, 2, "+ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah Central\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard");
    add_srs_wkt (p, 8,
		 "_parallel_1\",40.65],PARAMETER[\"standard_parallel_2\",3");
    add_srs_wkt (p, 9,
		 "9.01666666666667],PARAMETER[\"latitude_of_origin\",38.33");
    add_srs_wkt (p, 10,
		 "333333333334],PARAMETER[\"central_meridian\",-111.5],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 12,
		 "ing\",2000000],AUTHORITY[\"EPSG\",\"32143\"],AXIS[\"X\",");
    add_srs_wkt (p, 13, "EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32144, "epsg", 32144, "NAD83 / Utah South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.35 +lat_2=37.21666666666667 +lat_0=3");
    add_proj4text (p, 1,
		   "6.66666666666666 +lon_0=-111.5 +x_0=500000 +y_0=3000000 ");
    add_proj4text (p, 2, "+ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Utah South\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standard_p");
    add_srs_wkt (p, 8,
		 "arallel_1\",38.35],PARAMETER[\"standard_parallel_2\",37.");
    add_srs_wkt (p, 9,
		 "21666666666667],PARAMETER[\"latitude_of_origin\",36.6666");
    add_srs_wkt (p, 10,
		 "6666666666],PARAMETER[\"central_meridian\",-111.5],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",500000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",3000000],AUTHORITY[\"EPSG\",\"32144\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 13, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32145, "epsg", 32145, "NAD83 / Vermont");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=42.5 +lon_0=-72.5 +k=0.999964286 +x_0");
    add_proj4text (p, 1,
		   "=500000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Vermont\",GEOGCS[\"NAD83\",DATUM[\"Nort");
    add_srs_wkt (p, 1,
		 "h_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,29");
    add_srs_wkt (p, 2,
		 "8.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"EP");
    add_srs_wkt (p, 3,
		 "SG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 4,
		 ",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORIT");
    add_srs_wkt (p, 5,
		 "Y[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],UNIT");
    add_srs_wkt (p, 6,
		 "[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"");
    add_srs_wkt (p, 7,
		 "Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",");
    add_srs_wkt (p, 8,
		 "42.5],PARAMETER[\"central_meridian\",-72.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.999964286],PARAMETER[\"false_easting\",");
    add_srs_wkt (p, 10,
		 "500000],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 11, "\",\"32145\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32146, "epsg", 32146,
		      "NAD83 / Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=39.2 +lat_2=38.03333333333333 +lat_0=37");
    add_proj4text (p, 1,
		   ".66666666666666 +lon_0=-78.5 +x_0=3500000 +y_0=2000000 +");
    add_proj4text (p, 2, "ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Virginia North\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",39.2],PARAMETER[\"standard_parallel_2\",");
    add_srs_wkt (p, 9,
		 "38.03333333333333],PARAMETER[\"latitude_of_origin\",37.6");
    add_srs_wkt (p, 10,
		 "6666666666666],PARAMETER[\"central_meridian\",-78.5],PAR");
    add_srs_wkt (p, 11,
		 "AMETER[\"false_easting\",3500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",2000000],AUTHORITY[\"EPSG\",\"32146\"],AXIS[\"X\"");
    add_srs_wkt (p, 13, ",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32147, "epsg", 32147,
		      "NAD83 / Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=37.96666666666667 +lat_2=36.76666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=36.33333333333334 +lon_0=-78.5 +x_0=3500000 +");
    add_proj4text (p, 2,
		   "y_0=1000000 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Virginia South\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",37.96666666666667],PARAMETER[\"standard_");
    add_srs_wkt (p, 9,
		 "parallel_2\",36.76666666666667],PARAMETER[\"latitude_of_");
    add_srs_wkt (p, 10,
		 "origin\",36.33333333333334],PARAMETER[\"central_meridian");
    add_srs_wkt (p, 11,
		 "\",-78.5],PARAMETER[\"false_easting\",3500000],PARAMETER");
    add_srs_wkt (p, 12,
		 "[\"false_northing\",1000000],AUTHORITY[\"EPSG\",\"32147\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32148, "epsg", 32148,
		      "NAD83 / Washington North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=48.73333333333333 +lat_2=47.5 +lat_0=47");
    add_proj4text (p, 1,
		   " +lon_0=-120.8333333333333 +x_0=500000 +y_0=0 +ellps=GRS");
    add_proj4text (p, 2, "80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Washington North\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 8,
		 "ndard_parallel_1\",48.73333333333333],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_2\",47.5],PARAMETER[\"latitude_of_origin\",4");
    add_srs_wkt (p, 10,
		 "7],PARAMETER[\"central_meridian\",-120.8333333333333],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32148\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 13, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32149, "epsg", 32149,
		      "NAD83 / Washington South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=47.33333333333334 +lat_2=45.83333333333");
    add_proj4text (p, 1,
		   "334 +lat_0=45.33333333333334 +lon_0=-120.5 +x_0=500000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Washington South\",GEOGCS[\"NAD83\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6");
    add_srs_wkt (p, 2,
		 "378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHO");
    add_srs_wkt (p, 3,
		 "RITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORIT");
    add_srs_wkt (p, 4,
		 "Y[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199432");
    add_srs_wkt (p, 5,
		 "8,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"426");
    add_srs_wkt (p, 6,
		 "9\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PRO");
    add_srs_wkt (p, 7,
		 "JECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"sta");
    add_srs_wkt (p, 8,
		 "ndard_parallel_1\",47.33333333333334],PARAMETER[\"standa");
    add_srs_wkt (p, 9,
		 "rd_parallel_2\",45.83333333333334],PARAMETER[\"latitude_");
    add_srs_wkt (p, 10,
		 "of_origin\",45.33333333333334],PARAMETER[\"central_merid");
    add_srs_wkt (p, 11,
		 "ian\",-120.5],PARAMETER[\"false_easting\",500000],PARAME");
    add_srs_wkt (p, 12,
		 "TER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32149\"],");
    add_srs_wkt (p, 13, "AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32150, "epsg", 32150,
		      "NAD83 / West Virginia North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=40.25 +lat_2=39 +lat_0=38.5 +lon_0=-79.");
    add_proj4text (p, 1,
		   "5 +x_0=600000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / West Virginia North\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",40.25],PARAMETER[\"standard_paralle");
    add_srs_wkt (p, 9,
		 "l_2\",39],PARAMETER[\"latitude_of_origin\",38.5],PARAMET");
    add_srs_wkt (p, 10,
		 "ER[\"central_meridian\",-79.5],PARAMETER[\"false_easting");
    add_srs_wkt (p, 11,
		 "\",600000],PARAMETER[\"false_northing\",0],AUTHORITY[\"E");
    add_srs_wkt (p, 12, "PSG\",\"32150\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32151, "epsg", 32151,
		      "NAD83 / West Virginia South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=38.88333333333333 +lat_2=37.48333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=37 +lon_0=-81 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / West Virginia South\",GEOGCS[\"NAD83\",");
    add_srs_wkt (p, 1,
		 "DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",38.88333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",37.48333333333333],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",37],PARAMETER[\"central_meridian\",-81],PA");
    add_srs_wkt (p, 11,
		 "RAMETER[\"false_easting\",600000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 12,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32151\"],AXIS[\"X\",EAST]");
    add_srs_wkt (p, 13, ",AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32152, "epsg", 32152,
		      "NAD83 / Wisconsin North");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=46.76666666666667 +lat_2=45.56666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=45.16666666666666 +lon_0=-90 +x_0=600000 +y_0");
    add_proj4text (p, 2, "=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wisconsin North\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",46.76666666666667],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_2\",45.56666666666667],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 10,
		 "f_origin\",45.16666666666666],PARAMETER[\"central_meridi");
    add_srs_wkt (p, 11,
		 "an\",-90],PARAMETER[\"false_easting\",600000],PARAMETER[");
    add_srs_wkt (p, 12,
		 "\"false_northing\",0],AUTHORITY[\"EPSG\",\"32152\"],AXIS");
    add_srs_wkt (p, 13, "[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32153, "epsg", 32153,
		      "NAD83 / Wisconsin Central");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=45.5 +lat_2=44.25 +lat_0=43.83333333333");
    add_proj4text (p, 1,
		   "334 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80 +datum=NA");
    add_proj4text (p, 2, "D83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wisconsin Central\",GEOGCS[\"NAD83\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",");
    add_srs_wkt (p, 2,
		 "6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTH");
    add_srs_wkt (p, 3,
		 "ORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORI");
    add_srs_wkt (p, 4,
		 "TY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519943");
    add_srs_wkt (p, 5,
		 "28,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"42");
    add_srs_wkt (p, 6,
		 "69\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PR");
    add_srs_wkt (p, 7,
		 "OJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"st");
    add_srs_wkt (p, 8,
		 "andard_parallel_1\",45.5],PARAMETER[\"standard_parallel_");
    add_srs_wkt (p, 9,
		 "2\",44.25],PARAMETER[\"latitude_of_origin\",43.833333333");
    add_srs_wkt (p, 10,
		 "33334],PARAMETER[\"central_meridian\",-90],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_easting\",600000],PARAMETER[\"false_northing\",0],A");
    add_srs_wkt (p, 12,
		 "UTHORITY[\"EPSG\",\"32153\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 13, ",NORTH]]");
    p = add_epsg_def (first, last, 32154, "epsg", 32154,
		      "NAD83 / Wisconsin South");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=44.06666666666667 +lat_2=42.73333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=42 +lon_0=-90 +x_0=600000 +y_0=0 +ellps=GRS80");
    add_proj4text (p, 2, " +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wisconsin South\",GEOGCS[\"NAD83\",DATU");
    add_srs_wkt (p, 1,
		 "M[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",63");
    add_srs_wkt (p, 2,
		 "78137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHOR");
    add_srs_wkt (p, 3,
		 "ITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY");
    add_srs_wkt (p, 4,
		 "[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328");
    add_srs_wkt (p, 5,
		 ",AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269");
    add_srs_wkt (p, 6,
		 "\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJ");
    add_srs_wkt (p, 7,
		 "ECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"stan");
    add_srs_wkt (p, 8,
		 "dard_parallel_1\",44.06666666666667],PARAMETER[\"standar");
    add_srs_wkt (p, 9,
		 "d_parallel_2\",42.73333333333333],PARAMETER[\"latitude_o");
    add_srs_wkt (p, 10,
		 "f_origin\",42],PARAMETER[\"central_meridian\",-90],PARAM");
    add_srs_wkt (p, 11,
		 "ETER[\"false_easting\",600000],PARAMETER[\"false_northin");
    add_srs_wkt (p, 12,
		 "g\",0],AUTHORITY[\"EPSG\",\"32154\"],AXIS[\"X\",EAST],AX");
    add_srs_wkt (p, 13, "IS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32155, "epsg", 32155,
		      "NAD83 / Wyoming East");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-105.1666666666667 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=200000 +y_0=0 +ellps=GRS80 +datum=NAD83 +uni");
    add_proj4text (p, 2, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming East\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",40.5],PARAMETER[\"central_meridian\",-105.166666666");
    add_srs_wkt (p, 9,
		 "6667],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",200000],PARAMETER[\"false_northing\",0],");
    add_srs_wkt (p, 11,
		 "AUTHORITY[\"EPSG\",\"32155\"],AXIS[\"X\",EAST],AXIS[\"Y\"");
    add_srs_wkt (p, 12, ",NORTH]]");
    p = add_epsg_def (first, last, 32156, "epsg", 32156,
		      "NAD83 / Wyoming East Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-107.3333333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=400000 +y_0=100000 +ellps=GRS80 +datum=NAD83");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming East Central\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",40.5],PARAMETER[\"central_meridian\",-107.3");
    add_srs_wkt (p, 9,
		 "333333333333],PARAMETER[\"scale_factor\",0.9999375],PARA");
    add_srs_wkt (p, 10,
		 "METER[\"false_easting\",400000],PARAMETER[\"false_northi");
    add_srs_wkt (p, 11,
		 "ng\",100000],AUTHORITY[\"EPSG\",\"32156\"],AXIS[\"X\",EA");
    add_srs_wkt (p, 12, "ST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32157, "epsg", 32157,
		      "NAD83 / Wyoming West Central");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-108.75 +k=0.9999375 +x_0");
    add_proj4text (p, 1,
		   "=600000 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_de");
    add_proj4text (p, 2, "fs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming West Central\",GEOGCS[\"NAD83\"");
    add_srs_wkt (p, 1,
		 ",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980");
    add_srs_wkt (p, 2,
		 "\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],A");
    add_srs_wkt (p, 3,
		 "UTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTH");
    add_srs_wkt (p, 4,
		 "ORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.017453292519");
    add_srs_wkt (p, 5,
		 "94328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 6,
		 "4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],");
    add_srs_wkt (p, 7,
		 "PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude");
    add_srs_wkt (p, 8,
		 "_of_origin\",40.5],PARAMETER[\"central_meridian\",-108.7");
    add_srs_wkt (p, 9,
		 "5],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_easting\",600000],PARAMETER[\"false_northing\",0],AUT");
    add_srs_wkt (p, 11,
		 "HORITY[\"EPSG\",\"32157\"],AXIS[\"X\",EAST],AXIS[\"Y\",N");
    add_srs_wkt (p, 12, "ORTH]]");
    p = add_epsg_def (first, last, 32158, "epsg", 32158,
		      "NAD83 / Wyoming West");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=40.5 +lon_0=-110.0833333333333 +k=0.9");
    add_proj4text (p, 1,
		   "999375 +x_0=800000 +y_0=100000 +ellps=GRS80 +datum=NAD83");
    add_proj4text (p, 2, " +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Wyoming West\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",40.5],PARAMETER[\"central_meridian\",-110.083333333");
    add_srs_wkt (p, 9,
		 "3333],PARAMETER[\"scale_factor\",0.9999375],PARAMETER[\"");
    add_srs_wkt (p, 10,
		 "false_easting\",800000],PARAMETER[\"false_northing\",100");
    add_srs_wkt (p, 11,
		 "000],AUTHORITY[\"EPSG\",\"32158\"],AXIS[\"X\",EAST],AXIS");
    add_srs_wkt (p, 12, "[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32161, "epsg", 32161,
		      "NAD83 / Puerto Rico & Virgin Is.");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=18.43333333333333 +lat_2=18.03333333333");
    add_proj4text (p, 1,
		   "333 +lat_0=17.83333333333333 +lon_0=-66.43333333333334 +");
    add_proj4text (p, 2,
		   "x_0=200000 +y_0=200000 +ellps=GRS80 +datum=NAD83 +units=");
    add_proj4text (p, 3, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Puerto Rico & Virgin Is.\",GEOGCS[\"NAD");
    add_srs_wkt (p, 1,
		 "83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS ");
    add_srs_wkt (p, 2,
		 "1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"");
    add_srs_wkt (p, 3,
		 "]],AUTHORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,");
    add_srs_wkt (p, 4,
		 "AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329");
    add_srs_wkt (p, 5,
		 "251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG");
    add_srs_wkt (p, 6,
		 "\",\"4269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001");
    add_srs_wkt (p, 7,
		 "\"]],PROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMET");
    add_srs_wkt (p, 8,
		 "ER[\"standard_parallel_1\",18.43333333333333],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"standard_parallel_2\",18.03333333333333],PARAMETER[\"l");
    add_srs_wkt (p, 10,
		 "atitude_of_origin\",17.83333333333333],PARAMETER[\"centr");
    add_srs_wkt (p, 11,
		 "al_meridian\",-66.43333333333334],PARAMETER[\"false_east");
    add_srs_wkt (p, 12,
		 "ing\",200000],PARAMETER[\"false_northing\",200000],AUTHO");
    add_srs_wkt (p, 13,
		 "RITY[\"EPSG\",\"32161\"],AXIS[\"X\",EAST],AXIS[\"Y\",NOR");
    add_srs_wkt (p, 14, "TH]]");
    p = add_epsg_def (first, last, 32164, "epsg", 32164,
		      "NAD83 / BLM 14N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-99 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / BLM 14N (ftUS)\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_");
    add_srs_wkt (p, 9,
		 "meridian\",-99],PARAMETER[\"scale_factor\",0.9996],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",1640416.67],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32164\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Y\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_26 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 32165, "epsg", 32165,
		      "NAD83 / BLM 15N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-93 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / BLM 15N (ftUS)\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_");
    add_srs_wkt (p, 9,
		 "meridian\",-93],PARAMETER[\"scale_factor\",0.9996],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",1640416.67],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32165\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32166, "epsg", 32166,
		      "NAD83 / BLM 16N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-87 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / BLM 16N (ftUS)\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_");
    add_srs_wkt (p, 9,
		 "meridian\",-87],PARAMETER[\"scale_factor\",0.9996],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",1640416.67],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32166\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32167, "epsg", 32167,
		      "NAD83 / BLM 17N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-81 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / BLM 17N (ftUS)\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"US survey foot\",0.3048006096012192,AUTHORITY[");
    add_srs_wkt (p, 7,
		 "\"EPSG\",\"9003\"]],PROJECTION[\"Transverse_Mercator\"],");
    add_srs_wkt (p, 8,
		 "PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_");
    add_srs_wkt (p, 9,
		 "meridian\",-81],PARAMETER[\"scale_factor\",0.9996],PARAM");
    add_srs_wkt (p, 10,
		 "ETER[\"false_easting\",1640416.67],PARAMETER[\"false_nor");
    add_srs_wkt (p, 11,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32167\"],AXIS[\"X\",EAST");
    add_srs_wkt (p, 12, "],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32180, "epsg", 32180,
		      "NAD83 / SCoPQ zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-55.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / SCoPQ zone 2\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-55.5],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"scale_factor\",0.9999],PARAMETER[\"false_easting\",304");
    add_srs_wkt (p, 10,
		 "800],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"32180\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32181, "epsg", 32181, "NAD83 / MTM zone 1");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-53 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 1\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-53],PARAMETER[\"sc");
    add_srs_wkt (p, 9,
		 "ale_factor\",0.9999],PARAMETER[\"false_easting\",304800]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32");
    add_srs_wkt (p, 11, "181\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32182, "epsg", 32182, "NAD83 / MTM zone 2");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-56 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 2\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-56],PARAMETER[\"sc");
    add_srs_wkt (p, 9,
		 "ale_factor\",0.9999],PARAMETER[\"false_easting\",304800]");
    add_srs_wkt (p, 10,
		 ",PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32");
    add_srs_wkt (p, 11, "182\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32183, "epsg", 32183, "NAD83 / MTM zone 3");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-58.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 3\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-58.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "32183\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32184, "epsg", 32184, "NAD83 / MTM zone 4");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-61.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 4\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-61.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "32184\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32185, "epsg", 32185, "NAD83 / MTM zone 5");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-64.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 5\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-64.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "32185\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32186, "epsg", 32186, "NAD83 / MTM zone 6");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-67.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 6\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-67.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "32186\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32187, "epsg", 32187, "NAD83 / MTM zone 7");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-70.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 7\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-70.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "32187\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32188, "epsg", 32188, "NAD83 / MTM zone 8");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-73.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 8\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-73.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "32188\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32189, "epsg", 32189, "NAD83 / MTM zone 9");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-76.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 9\",GEOGCS[\"NAD83\",DATUM[\"N");
    add_srs_wkt (p, 1,
		 "orth_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137");
    add_srs_wkt (p, 2,
		 ",298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[\"");
    add_srs_wkt (p, 3,
		 "EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPS");
    add_srs_wkt (p, 4,
		 "G\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHO");
    add_srs_wkt (p, 5,
		 "RITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]],U");
    add_srs_wkt (p, 6,
		 "NIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION");
    add_srs_wkt (p, 7,
		 "[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin");
    add_srs_wkt (p, 8,
		 "\",0],PARAMETER[\"central_meridian\",-76.5],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11, "32189\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32190, "epsg", 32190, "NAD83 / MTM zone 10");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-79.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 10\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-79.5],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"scale_factor\",0.9999],PARAMETER[\"false_easting\",304");
    add_srs_wkt (p, 10,
		 "800],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11, "\"32190\"],AXIS[\"E(X)\",EAST],AXIS[\"N(Y)\",NORTH]]");
    p = add_epsg_def (first, last, 32191, "epsg", 32191, "NAD83 / MTM zone 11");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-82.5 +k=0.9999 +x_0=304800 ");
    add_proj4text (p, 1, "+y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 11\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-82.5],PARAMETER[");
    add_srs_wkt (p, 9,
		 "\"scale_factor\",0.9999],PARAMETER[\"false_easting\",304");
    add_srs_wkt (p, 10,
		 "800],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",");
    add_srs_wkt (p, 11,
		 "\"32191\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 32192, "epsg", 32192, "NAD83 / MTM zone 12");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-81 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 12\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-81],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "32192\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 32193, "epsg", 32193, "NAD83 / MTM zone 13");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-84 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 13\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-84],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "32193\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 32194, "epsg", 32194, "NAD83 / MTM zone 14");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-87 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 14\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-87],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "32194\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 32195, "epsg", 32195, "NAD83 / MTM zone 15");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-90 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 15\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-90],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "32195\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 32196, "epsg", 32196, "NAD83 / MTM zone 16");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-93 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 16\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-93],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "32196\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 32197, "epsg", 32197, "NAD83 / MTM zone 17");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-96 +k=0.9999 +x_0=304800 +y");
    add_proj4text (p, 1, "_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / MTM zone 17\",GEOGCS[\"NAD83\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637813");
    add_srs_wkt (p, 2,
		 "7,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORITY[");
    add_srs_wkt (p, 3,
		 "\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"E");
    add_srs_wkt (p, 4,
		 "PSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUT");
    add_srs_wkt (p, 5,
		 "HORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"]]");
    add_srs_wkt (p, 6,
		 ",UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTI");
    add_srs_wkt (p, 7,
		 "ON[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_orig");
    add_srs_wkt (p, 8,
		 "in\",0],PARAMETER[\"central_meridian\",-96],PARAMETER[\"");
    add_srs_wkt (p, 9,
		 "scale_factor\",0.9999],PARAMETER[\"false_easting\",30480");
    add_srs_wkt (p, 10,
		 "0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 11,
		 "32197\"],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]");
    add_srs_wkt (p, 12, "]");
    p = add_epsg_def (first, last, 32198, "epsg", 32198,
		      "NAD83 / Quebec Lambert");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=60 +lat_2=46 +lat_0=44 +lon_0=-68.5 +x_");
    add_proj4text (p, 1,
		   "0=0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Quebec Lambert\",GEOGCS[\"NAD83\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",637");
    add_srs_wkt (p, 2,
		 "8137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUTHORI");
    add_srs_wkt (p, 3,
		 "TY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[");
    add_srs_wkt (p, 4,
		 "\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,");
    add_srs_wkt (p, 5,
		 "AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4269\"");
    add_srs_wkt (p, 6,
		 "]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJEC");
    add_srs_wkt (p, 7,
		 "TION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"standa");
    add_srs_wkt (p, 8,
		 "rd_parallel_1\",60],PARAMETER[\"standard_parallel_2\",46");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"latitude_of_origin\",44],PARAMETER[\"centr");
    add_srs_wkt (p, 10,
		 "al_meridian\",-68.5],PARAMETER[\"false_easting\",0],PARA");
    add_srs_wkt (p, 11,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32198\"");
    add_srs_wkt (p, 12, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32199, "epsg", 32199,
		      "NAD83 / Louisiana Offshore");
    add_proj4text (p, 0,
		   "+proj=lcc +lat_1=27.83333333333333 +lat_2=26.16666666666");
    add_proj4text (p, 1,
		   "667 +lat_0=25.5 +lon_0=-91.33333333333333 +x_0=1000000 +");
    add_proj4text (p, 2, "y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"NAD83 / Louisiana Offshore\",GEOGCS[\"NAD83\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\"");
    add_srs_wkt (p, 2,
		 ",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],AUT");
    add_srs_wkt (p, 3,
		 "HORITY[\"EPSG\",\"6269\"]],PRIMEM[\"Greenwich\",0,AUTHOR");
    add_srs_wkt (p, 4,
		 "ITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994");
    add_srs_wkt (p, 5,
		 "328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4");
    add_srs_wkt (p, 6,
		 "269\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],P");
    add_srs_wkt (p, 7,
		 "ROJECTION[\"Lambert_Conformal_Conic_2SP\"],PARAMETER[\"s");
    add_srs_wkt (p, 8,
		 "tandard_parallel_1\",27.83333333333333],PARAMETER[\"stan");
    add_srs_wkt (p, 9,
		 "dard_parallel_2\",26.16666666666667],PARAMETER[\"latitud");
    add_srs_wkt (p, 10,
		 "e_of_origin\",25.5],PARAMETER[\"central_meridian\",-91.3");
    add_srs_wkt (p, 11,
		 "3333333333333],PARAMETER[\"false_easting\",1000000],PARA");
    add_srs_wkt (p, 12,
		 "METER[\"false_northing\",0],AUTHORITY[\"EPSG\",\"32199\"");
    add_srs_wkt (p, 13, "],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32201, "epsg", 32201,
		      "WGS 72 / UTM zone 1N");
    add_proj4text (p, 0, "+proj=utm +zone=1 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 1N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-177],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32201\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32202, "epsg", 32202,
		      "WGS 72 / UTM zone 2N");
    add_proj4text (p, 0, "+proj=utm +zone=2 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 2N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-171],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32202\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32203, "epsg", 32203,
		      "WGS 72 / UTM zone 3N");
    add_proj4text (p, 0, "+proj=utm +zone=3 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 3N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-165],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32203\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32204, "epsg", 32204,
		      "WGS 72 / UTM zone 4N");
    add_proj4text (p, 0, "+proj=utm +zone=4 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 4N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-159],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32204\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32205, "epsg", 32205,
		      "WGS 72 / UTM zone 5N");
    add_proj4text (p, 0, "+proj=utm +zone=5 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 5N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-153],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32205\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32206, "epsg", 32206,
		      "WGS 72 / UTM zone 6N");
    add_proj4text (p, 0, "+proj=utm +zone=6 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 6N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-147],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32206\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32207, "epsg", 32207,
		      "WGS 72 / UTM zone 7N");
    add_proj4text (p, 0, "+proj=utm +zone=7 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 7N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-141],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32207\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32208, "epsg", 32208,
		      "WGS 72 / UTM zone 8N");
    add_proj4text (p, 0, "+proj=utm +zone=8 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 8N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-135],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32208\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32209, "epsg", 32209,
		      "WGS 72 / UTM zone 9N");
    add_proj4text (p, 0, "+proj=utm +zone=9 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 9N\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-129],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32209\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32210, "epsg", 32210,
		      "WGS 72 / UTM zone 10N");
    add_proj4text (p, 0, "+proj=utm +zone=10 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 10N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-123],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32210\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32211, "epsg", 32211,
		      "WGS 72 / UTM zone 11N");
    add_proj4text (p, 0, "+proj=utm +zone=11 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 11N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-117],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32211\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32212, "epsg", 32212,
		      "WGS 72 / UTM zone 12N");
    add_proj4text (p, 0, "+proj=utm +zone=12 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 12N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-111],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32212\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32213, "epsg", 32213,
		      "WGS 72 / UTM zone 13N");
    add_proj4text (p, 0, "+proj=utm +zone=13 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 13N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-105],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",0],AUTHORITY[\"EPSG\",\"32213\"],AXIS[\"Easting");
    add_srs_wkt (p, 11, "\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32214, "epsg", 32214,
		      "WGS 72 / UTM zone 14N");
    add_proj4text (p, 0, "+proj=utm +zone=14 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 14N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-99],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32214\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32215, "epsg", 32215,
		      "WGS 72 / UTM zone 15N");
    add_proj4text (p, 0, "+proj=utm +zone=15 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 15N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-93],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32215\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32216, "epsg", 32216,
		      "WGS 72 / UTM zone 16N");
    add_proj4text (p, 0, "+proj=utm +zone=16 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 16N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-87],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32216\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32217, "epsg", 32217,
		      "WGS 72 / UTM zone 17N");
    add_proj4text (p, 0, "+proj=utm +zone=17 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 17N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-81],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32217\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32218, "epsg", 32218,
		      "WGS 72 / UTM zone 18N");
    add_proj4text (p, 0, "+proj=utm +zone=18 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 18N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-75],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32218\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32219, "epsg", 32219,
		      "WGS 72 / UTM zone 19N");
    add_proj4text (p, 0, "+proj=utm +zone=19 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 19N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-69],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32219\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32220, "epsg", 32220,
		      "WGS 72 / UTM zone 20N");
    add_proj4text (p, 0, "+proj=utm +zone=20 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 20N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-63],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32220\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32221, "epsg", 32221,
		      "WGS 72 / UTM zone 21N");
    add_proj4text (p, 0, "+proj=utm +zone=21 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 21N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-57],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32221\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32222, "epsg", 32222,
		      "WGS 72 / UTM zone 22N");
    add_proj4text (p, 0, "+proj=utm +zone=22 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 22N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-51],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32222\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32223, "epsg", 32223,
		      "WGS 72 / UTM zone 23N");
    add_proj4text (p, 0, "+proj=utm +zone=23 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 23N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-45],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32223\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32224, "epsg", 32224,
		      "WGS 72 / UTM zone 24N");
    add_proj4text (p, 0, "+proj=utm +zone=24 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 24N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-39],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32224\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32225, "epsg", 32225,
		      "WGS 72 / UTM zone 25N");
    add_proj4text (p, 0, "+proj=utm +zone=25 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 25N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-33],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32225\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32226, "epsg", 32226,
		      "WGS 72 / UTM zone 26N");
    add_proj4text (p, 0, "+proj=utm +zone=26 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 26N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-27],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32226\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32227, "epsg", 32227,
		      "WGS 72 / UTM zone 27N");
    add_proj4text (p, 0, "+proj=utm +zone=27 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 27N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-21],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32227\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32228, "epsg", 32228,
		      "WGS 72 / UTM zone 28N");
    add_proj4text (p, 0, "+proj=utm +zone=28 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 28N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-15],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32228\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32229, "epsg", 32229,
		      "WGS 72 / UTM zone 29N");
    add_proj4text (p, 0, "+proj=utm +zone=29 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 29N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-9],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32229\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32230, "epsg", 32230,
		      "WGS 72 / UTM zone 30N");
    add_proj4text (p, 0, "+proj=utm +zone=30 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 30N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-3],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32230\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32231, "epsg", 32231,
		      "WGS 72 / UTM zone 31N");
    add_proj4text (p, 0, "+proj=utm +zone=31 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 31N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",3],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 10,
		 "ing\",0],AUTHORITY[\"EPSG\",\"32231\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 11, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32232, "epsg", 32232,
		      "WGS 72 / UTM zone 32N");
    add_proj4text (p, 0, "+proj=utm +zone=32 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 32N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",9],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 10,
		 "ing\",0],AUTHORITY[\"EPSG\",\"32232\"],AXIS[\"Easting\",");
    add_srs_wkt (p, 11, "EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32233, "epsg", 32233,
		      "WGS 72 / UTM zone 33N");
    add_proj4text (p, 0, "+proj=utm +zone=33 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 33N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",15],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32233\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32234, "epsg", 32234,
		      "WGS 72 / UTM zone 34N");
    add_proj4text (p, 0, "+proj=utm +zone=34 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 34N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",21],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32234\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32235, "epsg", 32235,
		      "WGS 72 / UTM zone 35N");
    add_proj4text (p, 0, "+proj=utm +zone=35 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 35N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",27],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32235\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32236, "epsg", 32236,
		      "WGS 72 / UTM zone 36N");
    add_proj4text (p, 0, "+proj=utm +zone=36 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 36N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",33],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32236\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32237, "epsg", 32237,
		      "WGS 72 / UTM zone 37N");
    add_proj4text (p, 0, "+proj=utm +zone=37 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 37N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",39],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32237\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32238, "epsg", 32238,
		      "WGS 72 / UTM zone 38N");
    add_proj4text (p, 0, "+proj=utm +zone=38 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 38N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",45],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32238\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32239, "epsg", 32239,
		      "WGS 72 / UTM zone 39N");
    add_proj4text (p, 0, "+proj=utm +zone=39 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 39N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",51],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32239\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32240, "epsg", 32240,
		      "WGS 72 / UTM zone 40N");
    add_proj4text (p, 0, "+proj=utm +zone=40 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 40N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",57],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32240\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32241, "epsg", 32241,
		      "WGS 72 / UTM zone 41N");
    add_proj4text (p, 0, "+proj=utm +zone=41 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 41N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",63],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32241\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32242, "epsg", 32242,
		      "WGS 72 / UTM zone 42N");
    add_proj4text (p, 0, "+proj=utm +zone=42 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 42N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",69],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32242\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32243, "epsg", 32243,
		      "WGS 72 / UTM zone 43N");
    add_proj4text (p, 0, "+proj=utm +zone=43 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 43N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",75],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32243\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32244, "epsg", 32244,
		      "WGS 72 / UTM zone 44N");
    add_proj4text (p, 0, "+proj=utm +zone=44 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 44N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",81],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32244\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32245, "epsg", 32245,
		      "WGS 72 / UTM zone 45N");
    add_proj4text (p, 0, "+proj=utm +zone=45 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 45N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",87],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32245\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32246, "epsg", 32246,
		      "WGS 72 / UTM zone 46N");
    add_proj4text (p, 0, "+proj=utm +zone=46 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 46N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",93],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32246\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32247, "epsg", 32247,
		      "WGS 72 / UTM zone 47N");
    add_proj4text (p, 0, "+proj=utm +zone=47 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 47N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",99],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",0],AUTHORITY[\"EPSG\",\"32247\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32248, "epsg", 32248,
		      "WGS 72 / UTM zone 48N");
    add_proj4text (p, 0, "+proj=utm +zone=48 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 48N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",105],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32248\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32249, "epsg", 32249,
		      "WGS 72 / UTM zone 49N");
    add_proj4text (p, 0, "+proj=utm +zone=49 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 49N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",111],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32249\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32250, "epsg", 32250,
		      "WGS 72 / UTM zone 50N");
    add_proj4text (p, 0, "+proj=utm +zone=50 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 50N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",117],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32250\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32251, "epsg", 32251,
		      "WGS 72 / UTM zone 51N");
    add_proj4text (p, 0, "+proj=utm +zone=51 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 51N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",123],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32251\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32252, "epsg", 32252,
		      "WGS 72 / UTM zone 52N");
    add_proj4text (p, 0, "+proj=utm +zone=52 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 52N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",129],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32252\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32253, "epsg", 32253,
		      "WGS 72 / UTM zone 53N");
    add_proj4text (p, 0, "+proj=utm +zone=53 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 53N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",135],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32253\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32254, "epsg", 32254,
		      "WGS 72 / UTM zone 54N");
    add_proj4text (p, 0, "+proj=utm +zone=54 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 54N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",141],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32254\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32255, "epsg", 32255,
		      "WGS 72 / UTM zone 55N");
    add_proj4text (p, 0, "+proj=utm +zone=55 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 55N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",147],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32255\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32256, "epsg", 32256,
		      "WGS 72 / UTM zone 56N");
    add_proj4text (p, 0, "+proj=utm +zone=56 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 56N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",153],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32256\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32257, "epsg", 32257,
		      "WGS 72 / UTM zone 57N");
    add_proj4text (p, 0, "+proj=utm +zone=57 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 57N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",159],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32257\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32258, "epsg", 32258,
		      "WGS 72 / UTM zone 58N");
    add_proj4text (p, 0, "+proj=utm +zone=58 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 58N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",165],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32258\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32259, "epsg", 32259,
		      "WGS 72 / UTM zone 59N");
    add_proj4text (p, 0, "+proj=utm +zone=59 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 59N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",171],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32259\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32260, "epsg", 32260,
		      "WGS 72 / UTM zone 60N");
    add_proj4text (p, 0, "+proj=utm +zone=60 +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 60N\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",177],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",0],AUTHORITY[\"EPSG\",\"32260\"],AXIS[\"Easting\"");
    add_srs_wkt (p, 11, ",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32301, "epsg", 32301,
		      "WGS 72 / UTM zone 1S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 1S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-177],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32301\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32302, "epsg", 32302,
		      "WGS 72 / UTM zone 2S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 2S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-171],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32302\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32303, "epsg", 32303,
		      "WGS 72 / UTM zone 3S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 3S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-165],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32303\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32304, "epsg", 32304,
		      "WGS 72 / UTM zone 4S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 4S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-159],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32304\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32305, "epsg", 32305,
		      "WGS 72 / UTM zone 5S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 5S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-153],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32305\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32306, "epsg", 32306,
		      "WGS 72 / UTM zone 6S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 6S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-147],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32306\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32307, "epsg", 32307,
		      "WGS 72 / UTM zone 7S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 7S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-141],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32307\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32308, "epsg", 32308,
		      "WGS 72 / UTM zone 8S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 8S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-135],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32308\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32309, "epsg", 32309,
		      "WGS 72 / UTM zone 9S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +south +ellps=WGS72 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 9S\",GEOGCS[\"WGS 72\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORIT");
    add_srs_wkt (p, 2,
		 "Y[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRIM");
    add_srs_wkt (p, 3,
		 "EM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-129],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32309\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32310, "epsg", 32310,
		      "WGS 72 / UTM zone 10S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 10S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-123],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32310\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32311, "epsg", 32311,
		      "WGS 72 / UTM zone 11S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 11S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-117],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32311\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32312, "epsg", 32312,
		      "WGS 72 / UTM zone 12S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 12S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-111],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32312\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32313, "epsg", 32313,
		      "WGS 72 / UTM zone 13S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 13S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-105],PARAMETER[\"scale_factor\",0.9996],");
    add_srs_wkt (p, 9,
		 "PARAMETER[\"false_easting\",500000],PARAMETER[\"false_no");
    add_srs_wkt (p, 10,
		 "rthing\",10000000],AUTHORITY[\"EPSG\",\"32313\"],AXIS[\"");
    add_srs_wkt (p, 11, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32314, "epsg", 32314,
		      "WGS 72 / UTM zone 14S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 14S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-99],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32314\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32315, "epsg", 32315,
		      "WGS 72 / UTM zone 15S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 15S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-93],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32315\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32316, "epsg", 32316,
		      "WGS 72 / UTM zone 16S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 16S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-87],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32316\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32317, "epsg", 32317,
		      "WGS 72 / UTM zone 17S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 17S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-81],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32317\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32318, "epsg", 32318,
		      "WGS 72 / UTM zone 18S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 18S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-75],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32318\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32319, "epsg", 32319,
		      "WGS 72 / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 19S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-69],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32319\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32320, "epsg", 32320,
		      "WGS 72 / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 20S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-63],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32320\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32321, "epsg", 32321,
		      "WGS 72 / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 21S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-57],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32321\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32322, "epsg", 32322,
		      "WGS 72 / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 22S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-51],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32322\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32323, "epsg", 32323,
		      "WGS 72 / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 23S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-45],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32323\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32324, "epsg", 32324,
		      "WGS 72 / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 24S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-39],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32324\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32325, "epsg", 32325,
		      "WGS 72 / UTM zone 25S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 25S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-33],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32325\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32326, "epsg", 32326,
		      "WGS 72 / UTM zone 26S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 26S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-27],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32326\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32327, "epsg", 32327,
		      "WGS 72 / UTM zone 27S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 27S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-21],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32327\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32328, "epsg", 32328,
		      "WGS 72 / UTM zone 28S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 28S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-15],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32328\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32329, "epsg", 32329,
		      "WGS 72 / UTM zone 29S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 29S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-9],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32329\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32330, "epsg", 32330,
		      "WGS 72 / UTM zone 30S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 30S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",-3],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32330\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32331, "epsg", 32331,
		      "WGS 72 / UTM zone 31S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=31 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 31S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",3],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 10,
		 "ing\",10000000],AUTHORITY[\"EPSG\",\"32331\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32332, "epsg", 32332,
		      "WGS 72 / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 32S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",9],PARAMETER[\"scale_factor\",0.9996],PAR");
    add_srs_wkt (p, 9,
		 "AMETER[\"false_easting\",500000],PARAMETER[\"false_north");
    add_srs_wkt (p, 10,
		 "ing\",10000000],AUTHORITY[\"EPSG\",\"32332\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32333, "epsg", 32333,
		      "WGS 72 / UTM zone 33S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 33S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",15],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32333\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32334, "epsg", 32334,
		      "WGS 72 / UTM zone 34S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 34S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",21],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32334\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32335, "epsg", 32335,
		      "WGS 72 / UTM zone 35S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 35S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",27],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32335\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32336, "epsg", 32336,
		      "WGS 72 / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 36S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",33],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32336\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32337, "epsg", 32337,
		      "WGS 72 / UTM zone 37S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 37S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",39],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32337\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32338, "epsg", 32338,
		      "WGS 72 / UTM zone 38S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 38S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",45],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32338\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32339, "epsg", 32339,
		      "WGS 72 / UTM zone 39S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 39S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",51],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32339\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32340, "epsg", 32340,
		      "WGS 72 / UTM zone 40S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=40 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 40S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",57],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32340\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32341, "epsg", 32341,
		      "WGS 72 / UTM zone 41S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=41 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 41S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",63],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32341\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32342, "epsg", 32342,
		      "WGS 72 / UTM zone 42S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 42S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",69],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32342\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32343, "epsg", 32343,
		      "WGS 72 / UTM zone 43S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=43 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 43S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",75],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32343\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32344, "epsg", 32344,
		      "WGS 72 / UTM zone 44S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=44 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 44S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",81],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32344\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32345, "epsg", 32345,
		      "WGS 72 / UTM zone 45S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=45 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 45S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",87],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32345\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_27 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 32346, "epsg", 32346,
		      "WGS 72 / UTM zone 46S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 46S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",93],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32346\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32347, "epsg", 32347,
		      "WGS 72 / UTM zone 47S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 47S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",99],PARAMETER[\"scale_factor\",0.9996],PA");
    add_srs_wkt (p, 9,
		 "RAMETER[\"false_easting\",500000],PARAMETER[\"false_nort");
    add_srs_wkt (p, 10,
		 "hing\",10000000],AUTHORITY[\"EPSG\",\"32347\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32348, "epsg", 32348,
		      "WGS 72 / UTM zone 48S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 48S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",105],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32348\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32349, "epsg", 32349,
		      "WGS 72 / UTM zone 49S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 49S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",111],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32349\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32350, "epsg", 32350,
		      "WGS 72 / UTM zone 50S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 50S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",117],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32350\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32351, "epsg", 32351,
		      "WGS 72 / UTM zone 51S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 51S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",123],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32351\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32352, "epsg", 32352,
		      "WGS 72 / UTM zone 52S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 52S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",129],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32352\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32353, "epsg", 32353,
		      "WGS 72 / UTM zone 53S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 53S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",135],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32353\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32354, "epsg", 32354,
		      "WGS 72 / UTM zone 54S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 54S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",141],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32354\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32355, "epsg", 32355,
		      "WGS 72 / UTM zone 55S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 55S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",147],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32355\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32356, "epsg", 32356,
		      "WGS 72 / UTM zone 56S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 56S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",153],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32356\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32357, "epsg", 32357,
		      "WGS 72 / UTM zone 57S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 57S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",159],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32357\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32358, "epsg", 32358,
		      "WGS 72 / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 58S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",165],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32358\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32359, "epsg", 32359,
		      "WGS 72 / UTM zone 59S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 59S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",171],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32359\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32360, "epsg", 32360,
		      "WGS 72 / UTM zone 60S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +south +ellps=WGS72 +units=m +no_defs");
    add_proj4text (p, 1, "");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72 / UTM zone 60S\",GEOGCS[\"WGS 72\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1972\",SPHEROID[\"WGS 72\",6378135,298.26,AUTHORI");
    add_srs_wkt (p, 2,
		 "TY[\"EPSG\",\"7043\"]],AUTHORITY[\"EPSG\",\"6322\"]],PRI");
    add_srs_wkt (p, 3,
		 "MEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"");
    add_srs_wkt (p, 4,
		 "degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"");
    add_srs_wkt (p, 5,
		 "]],AUTHORITY[\"EPSG\",\"4322\"]],UNIT[\"metre\",1,AUTHOR");
    add_srs_wkt (p, 6,
		 "ITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Mercator");
    add_srs_wkt (p, 7,
		 "\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"cent");
    add_srs_wkt (p, 8,
		 "ral_meridian\",177],PARAMETER[\"scale_factor\",0.9996],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"false_easting\",500000],PARAMETER[\"false_nor");
    add_srs_wkt (p, 10,
		 "thing\",10000000],AUTHORITY[\"EPSG\",\"32360\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32401, "epsg", 32401,
		      "WGS 72BE / UTM zone 1N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 1N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-177],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32401\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32402, "epsg", 32402,
		      "WGS 72BE / UTM zone 2N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 2N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-171],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32402\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32403, "epsg", 32403,
		      "WGS 72BE / UTM zone 3N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 3N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-165],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32403\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32404, "epsg", 32404,
		      "WGS 72BE / UTM zone 4N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 4N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-159],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32404\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32405, "epsg", 32405,
		      "WGS 72BE / UTM zone 5N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 5N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-153],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32405\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32406, "epsg", 32406,
		      "WGS 72BE / UTM zone 6N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 6N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-147],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32406\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32407, "epsg", 32407,
		      "WGS 72BE / UTM zone 7N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 7N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-141],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32407\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32408, "epsg", 32408,
		      "WGS 72BE / UTM zone 8N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 8N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-135],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32408\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32409, "epsg", 32409,
		      "WGS 72BE / UTM zone 9N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.81");
    add_proj4text (p, 1, "4,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 9N\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-129],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32409\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32410, "epsg", 32410,
		      "WGS 72BE / UTM zone 10N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 10N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-123],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32410\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32411, "epsg", 32411,
		      "WGS 72BE / UTM zone 11N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 11N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-117],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32411\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32412, "epsg", 32412,
		      "WGS 72BE / UTM zone 12N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 12N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32412\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32413, "epsg", 32413,
		      "WGS 72BE / UTM zone 13N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 13N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-105],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32413\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32414, "epsg", 32414,
		      "WGS 72BE / UTM zone 14N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 14N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-99],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32414\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32415, "epsg", 32415,
		      "WGS 72BE / UTM zone 15N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 15N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-93],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32415\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32416, "epsg", 32416,
		      "WGS 72BE / UTM zone 16N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 16N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-87],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32416\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32417, "epsg", 32417,
		      "WGS 72BE / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 17N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-81],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32417\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32418, "epsg", 32418,
		      "WGS 72BE / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 18N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-75],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32418\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32419, "epsg", 32419,
		      "WGS 72BE / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 19N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-69],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32419\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32420, "epsg", 32420,
		      "WGS 72BE / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 20N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-63],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32420\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32421, "epsg", 32421,
		      "WGS 72BE / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 21N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-57],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32421\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32422, "epsg", 32422,
		      "WGS 72BE / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 22N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-51],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32422\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32423, "epsg", 32423,
		      "WGS 72BE / UTM zone 23N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 23N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-45],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32423\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32424, "epsg", 32424,
		      "WGS 72BE / UTM zone 24N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 24N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-39],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32424\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32425, "epsg", 32425,
		      "WGS 72BE / UTM zone 25N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 25N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-33],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32425\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32426, "epsg", 32426,
		      "WGS 72BE / UTM zone 26N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 26N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-27],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32426\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32427, "epsg", 32427,
		      "WGS 72BE / UTM zone 27N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 27N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-21],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32427\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32428, "epsg", 32428,
		      "WGS 72BE / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 28N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-15],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32428\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32429, "epsg", 32429,
		      "WGS 72BE / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 29N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-9],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32429\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32430, "epsg", 32430,
		      "WGS 72BE / UTM zone 30N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 30N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-3],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32430\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32431, "epsg", 32431,
		      "WGS 72BE / UTM zone 31N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=31 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 31N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",3],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32431\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32432, "epsg", 32432,
		      "WGS 72BE / UTM zone 32N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 32N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",9],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32432\"],AXIS[\"E");
    add_srs_wkt (p, 12, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32433, "epsg", 32433,
		      "WGS 72BE / UTM zone 33N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 33N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",15],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32433\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32434, "epsg", 32434,
		      "WGS 72BE / UTM zone 34N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 34N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",21],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32434\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32435, "epsg", 32435,
		      "WGS 72BE / UTM zone 35N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 35N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",27],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32435\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32436, "epsg", 32436,
		      "WGS 72BE / UTM zone 36N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 36N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",33],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32436\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32437, "epsg", 32437,
		      "WGS 72BE / UTM zone 37N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 37N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",39],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32437\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32438, "epsg", 32438,
		      "WGS 72BE / UTM zone 38N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 38N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",45],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32438\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32439, "epsg", 32439,
		      "WGS 72BE / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 39N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",51],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32439\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32440, "epsg", 32440,
		      "WGS 72BE / UTM zone 40N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=40 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 40N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",57],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32440\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32441, "epsg", 32441,
		      "WGS 72BE / UTM zone 41N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=41 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 41N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",63],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32441\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32442, "epsg", 32442,
		      "WGS 72BE / UTM zone 42N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 42N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",69],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32442\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32443, "epsg", 32443,
		      "WGS 72BE / UTM zone 43N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=43 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 43N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",75],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32443\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32444, "epsg", 32444,
		      "WGS 72BE / UTM zone 44N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=44 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 44N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",81],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32444\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32445, "epsg", 32445,
		      "WGS 72BE / UTM zone 45N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=45 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 45N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",87],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32445\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32446, "epsg", 32446,
		      "WGS 72BE / UTM zone 46N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 46N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",93],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32446\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32447, "epsg", 32447,
		      "WGS 72BE / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 47N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",99],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",0],AUTHORITY[\"EPSG\",\"32447\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32448, "epsg", 32448,
		      "WGS 72BE / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 48N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",105],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32448\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32449, "epsg", 32449,
		      "WGS 72BE / UTM zone 49N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 49N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",111],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32449\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32450, "epsg", 32450,
		      "WGS 72BE / UTM zone 50N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 50N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",117],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32450\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32451, "epsg", 32451,
		      "WGS 72BE / UTM zone 51N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 51N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",123],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32451\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32452, "epsg", 32452,
		      "WGS 72BE / UTM zone 52N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 52N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",129],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32452\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32453, "epsg", 32453,
		      "WGS 72BE / UTM zone 53N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 53N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",135],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32453\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32454, "epsg", 32454,
		      "WGS 72BE / UTM zone 54N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 54N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",141],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32454\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32455, "epsg", 32455,
		      "WGS 72BE / UTM zone 55N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 55N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",147],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32455\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32456, "epsg", 32456,
		      "WGS 72BE / UTM zone 56N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 56N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",153],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32456\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32457, "epsg", 32457,
		      "WGS 72BE / UTM zone 57N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 57N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",159],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32457\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32458, "epsg", 32458,
		      "WGS 72BE / UTM zone 58N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 58N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",165],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32458\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32459, "epsg", 32459,
		      "WGS 72BE / UTM zone 59N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 59N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",171],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32459\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32460, "epsg", 32460,
		      "WGS 72BE / UTM zone 60N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +ellps=WGS72 +towgs84=0,0,1.9,0,0,0.8");
    add_proj4text (p, 1, "14,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 60N\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",177],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",0],AUTHORITY[\"EPSG\",\"32460\"],AXIS[\"");
    add_srs_wkt (p, 12, "Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32501, "epsg", 32501,
		      "WGS 72BE / UTM zone 1S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 1S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-177],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32501\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32502, "epsg", 32502,
		      "WGS 72BE / UTM zone 2S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 2S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-171],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32502\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32503, "epsg", 32503,
		      "WGS 72BE / UTM zone 3S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 3S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-165],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32503\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32504, "epsg", 32504,
		      "WGS 72BE / UTM zone 4S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 4S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-159],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32504\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32505, "epsg", 32505,
		      "WGS 72BE / UTM zone 5S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 5S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-153],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32505\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32506, "epsg", 32506,
		      "WGS 72BE / UTM zone 6S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 6S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-147],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32506\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32507, "epsg", 32507,
		      "WGS 72BE / UTM zone 7S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 7S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-141],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32507\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32508, "epsg", 32508,
		      "WGS 72BE / UTM zone 8S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 8S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-135],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32508\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32509, "epsg", 32509,
		      "WGS 72BE / UTM zone 9S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +south +ellps=WGS72 +towgs84=0,0,1.9,0");
    add_proj4text (p, 1, ",0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 9S\",GEOGCS[\"WGS 72BE\",DA");
    add_srs_wkt (p, 1,
		 "TUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-129],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32509\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32510, "epsg", 32510,
		      "WGS 72BE / UTM zone 10S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 10S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-123],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32510\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32511, "epsg", 32511,
		      "WGS 72BE / UTM zone 11S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 11S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-117],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32511\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32512, "epsg", 32512,
		      "WGS 72BE / UTM zone 12S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 12S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-111],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32512\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32513, "epsg", 32513,
		      "WGS 72BE / UTM zone 13S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 13S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-105],PARAMETER[\"scale_factor\"");
    add_srs_wkt (p, 10,
		 ",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32513\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32514, "epsg", 32514,
		      "WGS 72BE / UTM zone 14S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 14S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-99],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32514\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32515, "epsg", 32515,
		      "WGS 72BE / UTM zone 15S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 15S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-93],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32515\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32516, "epsg", 32516,
		      "WGS 72BE / UTM zone 16S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 16S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-87],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32516\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32517, "epsg", 32517,
		      "WGS 72BE / UTM zone 17S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 17S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-81],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32517\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32518, "epsg", 32518,
		      "WGS 72BE / UTM zone 18S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 18S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-75],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32518\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32519, "epsg", 32519,
		      "WGS 72BE / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 19S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-69],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32519\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32520, "epsg", 32520,
		      "WGS 72BE / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 20S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-63],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32520\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32521, "epsg", 32521,
		      "WGS 72BE / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 21S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-57],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32521\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32522, "epsg", 32522,
		      "WGS 72BE / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 22S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-51],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32522\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32523, "epsg", 32523,
		      "WGS 72BE / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 23S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-45],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32523\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32524, "epsg", 32524,
		      "WGS 72BE / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 24S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-39],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32524\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32525, "epsg", 32525,
		      "WGS 72BE / UTM zone 25S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 25S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-33],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32525\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32526, "epsg", 32526,
		      "WGS 72BE / UTM zone 26S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 26S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-27],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32526\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32527, "epsg", 32527,
		      "WGS 72BE / UTM zone 27S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 27S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-21],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32527\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32528, "epsg", 32528,
		      "WGS 72BE / UTM zone 28S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 28S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-15],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32528\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32529, "epsg", 32529,
		      "WGS 72BE / UTM zone 29S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 29S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-9],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32529\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32530, "epsg", 32530,
		      "WGS 72BE / UTM zone 30S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 30S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",-3],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32530\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32531, "epsg", 32531,
		      "WGS 72BE / UTM zone 31S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=31 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 31S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",3],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32531\"],A");
    add_srs_wkt (p, 12, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32532, "epsg", 32532,
		      "WGS 72BE / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 32S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",9],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 10,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 11,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32532\"],A");
    add_srs_wkt (p, 12, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32533, "epsg", 32533,
		      "WGS 72BE / UTM zone 33S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 33S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",15],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32533\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32534, "epsg", 32534,
		      "WGS 72BE / UTM zone 34S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 34S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",21],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32534\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32535, "epsg", 32535,
		      "WGS 72BE / UTM zone 35S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 35S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",27],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32535\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32536, "epsg", 32536,
		      "WGS 72BE / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 36S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",33],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32536\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32537, "epsg", 32537,
		      "WGS 72BE / UTM zone 37S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 37S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",39],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32537\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32538, "epsg", 32538,
		      "WGS 72BE / UTM zone 38S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 38S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",45],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32538\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32539, "epsg", 32539,
		      "WGS 72BE / UTM zone 39S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 39S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",51],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32539\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32540, "epsg", 32540,
		      "WGS 72BE / UTM zone 40S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=40 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 40S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",57],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32540\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32541, "epsg", 32541,
		      "WGS 72BE / UTM zone 41S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=41 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 41S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",63],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32541\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32542, "epsg", 32542,
		      "WGS 72BE / UTM zone 42S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 42S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",69],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32542\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32543, "epsg", 32543,
		      "WGS 72BE / UTM zone 43S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=43 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 43S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",75],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32543\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32544, "epsg", 32544,
		      "WGS 72BE / UTM zone 44S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=44 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 44S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",81],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32544\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32545, "epsg", 32545,
		      "WGS 72BE / UTM zone 45S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=45 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 45S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",87],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32545\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32546, "epsg", 32546,
		      "WGS 72BE / UTM zone 46S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 46S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",93],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32546\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32547, "epsg", 32547,
		      "WGS 72BE / UTM zone 47S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 47S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",99],PARAMETER[\"scale_factor\",0");
    add_srs_wkt (p, 10,
		 ".9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"f");
    add_srs_wkt (p, 11,
		 "alse_northing\",10000000],AUTHORITY[\"EPSG\",\"32547\"],");
    add_srs_wkt (p, 12, "AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32548, "epsg", 32548,
		      "WGS 72BE / UTM zone 48S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 48S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",105],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32548\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32549, "epsg", 32549,
		      "WGS 72BE / UTM zone 49S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 49S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",111],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32549\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32550, "epsg", 32550,
		      "WGS 72BE / UTM zone 50S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 50S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",117],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32550\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32551, "epsg", 32551,
		      "WGS 72BE / UTM zone 51S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 51S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",123],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32551\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32552, "epsg", 32552,
		      "WGS 72BE / UTM zone 52S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 52S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",129],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32552\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32553, "epsg", 32553,
		      "WGS 72BE / UTM zone 53S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 53S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",135],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32553\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
#endif /* full EPSG initialization enabled/disabled */
    if (first == last)
	last = first;		/* suppressing stupid compiler warnings */
}

static void
initialize_epsg_28 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
    struct epsg_defs *p;
#ifndef OMIT_EPSG		/* full EPSG initialization enabled */
    p = add_epsg_def (first, last, 32554, "epsg", 32554,
		      "WGS 72BE / UTM zone 54S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 54S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",141],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32554\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32555, "epsg", 32555,
		      "WGS 72BE / UTM zone 55S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 55S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",147],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32555\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32556, "epsg", 32556,
		      "WGS 72BE / UTM zone 56S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 56S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",153],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32556\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32557, "epsg", 32557,
		      "WGS 72BE / UTM zone 57S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 57S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",159],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32557\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32558, "epsg", 32558,
		      "WGS 72BE / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 58S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",165],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32558\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32559, "epsg", 32559,
		      "WGS 72BE / UTM zone 59S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 59S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",171],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32559\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32560, "epsg", 32560,
		      "WGS 72BE / UTM zone 60S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +south +ellps=WGS72 +towgs84=0,0,1.9,");
    add_proj4text (p, 1, "0,0,0.814,-0.38 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 72BE / UTM zone 60S\",GEOGCS[\"WGS 72BE\",D");
    add_srs_wkt (p, 1,
		 "ATUM[\"WGS_1972_Transit_Broadcast_Ephemeris\",SPHEROID[\"");
    add_srs_wkt (p, 2,
		 "WGS 72\",6378135,298.26,AUTHORITY[\"EPSG\",\"7043\"]],TO");
    add_srs_wkt (p, 3,
		 "WGS84[0,0,1.9,0,0,0.814,-0.38],AUTHORITY[\"EPSG\",\"6324");
    add_srs_wkt (p, 4,
		 "\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]");
    add_srs_wkt (p, 5,
		 "],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 6,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4324\"]],UNIT[\"metre\"");
    add_srs_wkt (p, 7,
		 ",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse");
    add_srs_wkt (p, 8,
		 "_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMET");
    add_srs_wkt (p, 9,
		 "ER[\"central_meridian\",177],PARAMETER[\"scale_factor\",");
    add_srs_wkt (p, 10,
		 "0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"");
    add_srs_wkt (p, 11,
		 "false_northing\",10000000],AUTHORITY[\"EPSG\",\"32560\"]");
    add_srs_wkt (p, 12, ",AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");

#endif /* unconditionally loading the WGS82/UTM zones */
    p = add_epsg_def (first, last, 32601, "epsg", 32601,
		      "WGS 84 / UTM zone 1N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 1N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-177],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32601\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32602, "epsg", 32602,
		      "WGS 84 / UTM zone 2N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 2N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-171],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32602\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32603, "epsg", 32603,
		      "WGS 84 / UTM zone 3N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 3N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-165],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32603\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32604, "epsg", 32604,
		      "WGS 84 / UTM zone 4N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 4N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-159],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32604\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32605, "epsg", 32605,
		      "WGS 84 / UTM zone 5N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 5N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-153],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32605\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32606, "epsg", 32606,
		      "WGS 84 / UTM zone 6N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 6N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-147],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32606\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32607, "epsg", 32607,
		      "WGS 84 / UTM zone 7N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 7N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-141],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32607\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32608, "epsg", 32608,
		      "WGS 84 / UTM zone 8N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 8N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-135],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32608\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32609, "epsg", 32609,
		      "WGS 84 / UTM zone 9N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +ellps=WGS84 +datum=WGS84 +units=m +no");
    add_proj4text (p, 1, "_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 9N\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-129],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32609\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32610, "epsg", 32610,
		      "WGS 84 / UTM zone 10N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 10N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-123],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32610\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32611, "epsg", 32611,
		      "WGS 84 / UTM zone 11N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 11N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-117],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32611\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32612, "epsg", 32612,
		      "WGS 84 / UTM zone 12N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 12N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-111],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32612\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32613, "epsg", 32613,
		      "WGS 84 / UTM zone 13N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 13N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-105],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",0],AUTHORITY[\"EPSG\",\"32613\"],AXIS[\"E");
    add_srs_wkt (p, 11, "asting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32614, "epsg", 32614,
		      "WGS 84 / UTM zone 14N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 14N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-99],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32614\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32615, "epsg", 32615,
		      "WGS 84 / UTM zone 15N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 15N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-93],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32615\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32616, "epsg", 32616,
		      "WGS 84 / UTM zone 16N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 16N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-87],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32616\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32617, "epsg", 32617,
		      "WGS 84 / UTM zone 17N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 17N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-81],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32617\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32618, "epsg", 32618,
		      "WGS 84 / UTM zone 18N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 18N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-75],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32618\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32619, "epsg", 32619,
		      "WGS 84 / UTM zone 19N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 19N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-69],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32619\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32620, "epsg", 32620,
		      "WGS 84 / UTM zone 20N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 20N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-63],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32620\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32621, "epsg", 32621,
		      "WGS 84 / UTM zone 21N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 21N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-57],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32621\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32622, "epsg", 32622,
		      "WGS 84 / UTM zone 22N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 22N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-51],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32622\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32623, "epsg", 32623,
		      "WGS 84 / UTM zone 23N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 23N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-45],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32623\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32624, "epsg", 32624,
		      "WGS 84 / UTM zone 24N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 24N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-39],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32624\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32625, "epsg", 32625,
		      "WGS 84 / UTM zone 25N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 25N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-33],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32625\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32626, "epsg", 32626,
		      "WGS 84 / UTM zone 26N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 26N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-27],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32626\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32627, "epsg", 32627,
		      "WGS 84 / UTM zone 27N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 27N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-21],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32627\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32628, "epsg", 32628,
		      "WGS 84 / UTM zone 28N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 28N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-15],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32628\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32629, "epsg", 32629,
		      "WGS 84 / UTM zone 29N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 29N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-9],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32629\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32630, "epsg", 32630,
		      "WGS 84 / UTM zone 30N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 30N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-3],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32630\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32631, "epsg", 32631,
		      "WGS 84 / UTM zone 31N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=31 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 31N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",3],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"32631\"],AXIS[\"East");
    add_srs_wkt (p, 11, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32632, "epsg", 32632,
		      "WGS 84 / UTM zone 32N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 32N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",9],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",0],AUTHORITY[\"EPSG\",\"32632\"],AXIS[\"East");
    add_srs_wkt (p, 11, "ing\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32633, "epsg", 32633,
		      "WGS 84 / UTM zone 33N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 33N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",15],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32633\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32634, "epsg", 32634,
		      "WGS 84 / UTM zone 34N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 34N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",21],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32634\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32635, "epsg", 32635,
		      "WGS 84 / UTM zone 35N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 35N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",27],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32635\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32636, "epsg", 32636,
		      "WGS 84 / UTM zone 36N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 36N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",33],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32636\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32637, "epsg", 32637,
		      "WGS 84 / UTM zone 37N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 37N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",39],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32637\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32638, "epsg", 32638,
		      "WGS 84 / UTM zone 38N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 38N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",45],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32638\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32639, "epsg", 32639,
		      "WGS 84 / UTM zone 39N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 39N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",51],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32639\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32640, "epsg", 32640,
		      "WGS 84 / UTM zone 40N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=40 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 40N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",57],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32640\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32641, "epsg", 32641,
		      "WGS 84 / UTM zone 41N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=41 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 41N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",63],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32641\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32642, "epsg", 32642,
		      "WGS 84 / UTM zone 42N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 42N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",69],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32642\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32643, "epsg", 32643,
		      "WGS 84 / UTM zone 43N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=43 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 43N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",75],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32643\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32644, "epsg", 32644,
		      "WGS 84 / UTM zone 44N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=44 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 44N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",81],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32644\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32645, "epsg", 32645,
		      "WGS 84 / UTM zone 45N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=45 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 45N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",87],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32645\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32646, "epsg", 32646,
		      "WGS 84 / UTM zone 46N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 46N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",93],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32646\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32647, "epsg", 32647,
		      "WGS 84 / UTM zone 47N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 47N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",99],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",0],AUTHORITY[\"EPSG\",\"32647\"],AXIS[\"Eas");
    add_srs_wkt (p, 11, "ting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32648, "epsg", 32648,
		      "WGS 84 / UTM zone 48N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 48N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",105],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32648\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32649, "epsg", 32649,
		      "WGS 84 / UTM zone 49N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 49N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",111],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32649\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32650, "epsg", 32650,
		      "WGS 84 / UTM zone 50N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 50N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",117],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32650\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32651, "epsg", 32651,
		      "WGS 84 / UTM zone 51N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 51N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",123],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32651\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32652, "epsg", 32652,
		      "WGS 84 / UTM zone 52N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 52N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",129],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32652\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32653, "epsg", 32653,
		      "WGS 84 / UTM zone 53N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 53N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",135],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32653\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32654, "epsg", 32654,
		      "WGS 84 / UTM zone 54N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32654\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32655, "epsg", 32655,
		      "WGS 84 / UTM zone 55N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 55N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",147],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32655\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32656, "epsg", 32656,
		      "WGS 84 / UTM zone 56N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 56N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",153],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32656\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32657, "epsg", 32657,
		      "WGS 84 / UTM zone 57N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 57N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",159],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32657\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32658, "epsg", 32658,
		      "WGS 84 / UTM zone 58N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 58N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",165],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32658\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32659, "epsg", 32659,
		      "WGS 84 / UTM zone 59N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 59N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",171],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32659\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32660, "epsg", 32660,
		      "WGS 84 / UTM zone 60N");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +ellps=WGS84 +datum=WGS84 +units=m +n");
    add_proj4text (p, 1, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 60N\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",177],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",0],AUTHORITY[\"EPSG\",\"32660\"],AXIS[\"Ea");
    add_srs_wkt (p, 11, "sting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32661, "epsg", 32661, "WGS 84 / UPS North");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=90 +lat_ts=90 +lon_0=0 +k=0.994 +x_0=");
    add_proj4text (p, 1,
		   "2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 +units=m ");
    add_proj4text (p, 2, "+no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UPS North\",GEOGCS[\"WGS 84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 4,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 5,
		 "122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 6,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_Stereogr");
    add_srs_wkt (p, 7,
		 "aphic\"],PARAMETER[\"latitude_of_origin\",90],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",0],PARAMETER[\"scale_factor\",0.994");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",2000000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",2000000],AUTHORITY[\"EPSG\",\"32661\"],AXIS[");
    add_srs_wkt (p, 11, "\"Northing\",UNKNOWN],AXIS[\"Easting\",UNKNOWN]]");
    p = add_epsg_def (first, last, 32662, "epsg", 32662,
		      "WGS 84 / Plate Carree (deprecated)");
    add_proj4text (p, 0,
		   "+proj=eqc +lat_ts=0 +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +ell");
    add_proj4text (p, 1, "ps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / Plate Carree (deprecated)\",GEOGCS[\"W");
    add_srs_wkt (p, 1,
		 "GS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,2");
    add_srs_wkt (p, 2,
		 "98.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"E");
    add_srs_wkt (p, 3,
		 "PSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG");
    add_srs_wkt (p, 4,
		 "\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHOR");
    add_srs_wkt (p, 5,
		 "ITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UN");
    add_srs_wkt (p, 6,
		 "IT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[");
    add_srs_wkt (p, 7,
		 "\"Equirectangular\"],PARAMETER[\"latitude_of_origin\",0]");
    add_srs_wkt (p, 8,
		 ",PARAMETER[\"central_meridian\",0],PARAMETER[\"false_eas");
    add_srs_wkt (p, 9,
		 "ting\",0],PARAMETER[\"false_northing\",0],AUTHORITY[\"EP");
    add_srs_wkt (p, 10, "SG\",\"32662\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]");
    p = add_epsg_def (first, last, 32664, "epsg", 32664,
		      "WGS 84 / BLM 14N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-99 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / BLM 14N (ftUS)\",GEOGCS[\"WGS 84\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.25722356");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"632");
    add_srs_wkt (p, 3,
		 "6\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"US surv");
    add_srs_wkt (p, 6,
		 "ey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-99],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",1640416.67],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"32664\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 32665, "epsg", 32665,
		      "WGS 84 / BLM 15N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-93 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / BLM 15N (ftUS)\",GEOGCS[\"WGS 84\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.25722356");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"632");
    add_srs_wkt (p, 3,
		 "6\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"US surv");
    add_srs_wkt (p, 6,
		 "ey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-93],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",1640416.67],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"32665\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 32666, "epsg", 32666,
		      "WGS 84 / BLM 16N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-87 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / BLM 16N (ftUS)\",GEOGCS[\"WGS 84\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.25722356");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"632");
    add_srs_wkt (p, 3,
		 "6\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"US surv");
    add_srs_wkt (p, 6,
		 "ey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-87],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",1640416.67],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"32666\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 32667, "epsg", 32667,
		      "WGS 84 / BLM 17N (ftUS)");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=-81 +k=0.9996 +x_0=500000.00");
    add_proj4text (p, 1,
		   "1016002 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=us-ft +n");
    add_proj4text (p, 2, "o_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / BLM 17N (ftUS)\",GEOGCS[\"WGS 84\",DAT");
    add_srs_wkt (p, 1,
		 "UM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.25722356");
    add_srs_wkt (p, 2,
		 "3,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"632");
    add_srs_wkt (p, 3,
		 "6\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"");
    add_srs_wkt (p, 4,
		 "]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\"");
    add_srs_wkt (p, 5,
		 ",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"US surv");
    add_srs_wkt (p, 6,
		 "ey foot\",0.3048006096012192,AUTHORITY[\"EPSG\",\"9003\"");
    add_srs_wkt (p, 7,
		 "]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latit");
    add_srs_wkt (p, 8,
		 "ude_of_origin\",0],PARAMETER[\"central_meridian\",-81],P");
    add_srs_wkt (p, 9,
		 "ARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_east");
    add_srs_wkt (p, 10,
		 "ing\",1640416.67],PARAMETER[\"false_northing\",0],AUTHOR");
    add_srs_wkt (p, 11,
		 "ITY[\"EPSG\",\"32667\"],AXIS[\"X\",EAST],AXIS[\"Y\",NORT");
    add_srs_wkt (p, 12, "H]]");
    p = add_epsg_def (first, last, 32701, "epsg", 32701,
		      "WGS 84 / UTM zone 1S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=1 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 1S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-177],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32701\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32702, "epsg", 32702,
		      "WGS 84 / UTM zone 2S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=2 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 2S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-171],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32702\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32703, "epsg", 32703,
		      "WGS 84 / UTM zone 3S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=3 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 3S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-165],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32703\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32704, "epsg", 32704,
		      "WGS 84 / UTM zone 4S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=4 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 4S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-159],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32704\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32705, "epsg", 32705,
		      "WGS 84 / UTM zone 5S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=5 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 5S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-153],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32705\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32706, "epsg", 32706,
		      "WGS 84 / UTM zone 6S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=6 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 6S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-147],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32706\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32707, "epsg", 32707,
		      "WGS 84 / UTM zone 7S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=7 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 7S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-141],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32707\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32708, "epsg", 32708,
		      "WGS 84 / UTM zone 8S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=8 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 8S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-135],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32708\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32709, "epsg", 32709,
		      "WGS 84 / UTM zone 9S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=9 +south +ellps=WGS84 +datum=WGS84 +unit");
    add_proj4text (p, 1, "s=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 9S\",GEOGCS[\"WGS 84\",DATUM[");
    add_srs_wkt (p, 1,
		 "\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,A");
    add_srs_wkt (p, 2,
		 "UTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-129],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32709\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32710, "epsg", 32710,
		      "WGS 84 / UTM zone 10S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=10 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 10S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-123],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32710\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32711, "epsg", 32711,
		      "WGS 84 / UTM zone 11S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=11 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 11S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-117],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32711\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32712, "epsg", 32712,
		      "WGS 84 / UTM zone 12S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=12 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 12S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-111],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32712\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32713, "epsg", 32713,
		      "WGS 84 / UTM zone 13S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=13 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 13S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-105],PARAMETER[\"scale_factor\",0.");
    add_srs_wkt (p, 9,
		 "9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fa");
    add_srs_wkt (p, 10,
		 "lse_northing\",10000000],AUTHORITY[\"EPSG\",\"32713\"],A");
    add_srs_wkt (p, 11, "XIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32714, "epsg", 32714,
		      "WGS 84 / UTM zone 14S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=14 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 14S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-99],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32714\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32715, "epsg", 32715,
		      "WGS 84 / UTM zone 15S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=15 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 15S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-93],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32715\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32716, "epsg", 32716,
		      "WGS 84 / UTM zone 16S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=16 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 16S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-87],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32716\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32717, "epsg", 32717,
		      "WGS 84 / UTM zone 17S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=17 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 17S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-81],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32717\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32718, "epsg", 32718,
		      "WGS 84 / UTM zone 18S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=18 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 18S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-75],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32718\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32719, "epsg", 32719,
		      "WGS 84 / UTM zone 19S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=19 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 19S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-69],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32719\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32720, "epsg", 32720,
		      "WGS 84 / UTM zone 20S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=20 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 20S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-63],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32720\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32721, "epsg", 32721,
		      "WGS 84 / UTM zone 21S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=21 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 21S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-57],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32721\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32722, "epsg", 32722,
		      "WGS 84 / UTM zone 22S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=22 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 22S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-51],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32722\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32723, "epsg", 32723,
		      "WGS 84 / UTM zone 23S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=23 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 23S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-45],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32723\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32724, "epsg", 32724,
		      "WGS 84 / UTM zone 24S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=24 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 24S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-39],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32724\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32725, "epsg", 32725,
		      "WGS 84 / UTM zone 25S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=25 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 25S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-33],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32725\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32726, "epsg", 32726,
		      "WGS 84 / UTM zone 26S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=26 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 26S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-27],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32726\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32727, "epsg", 32727,
		      "WGS 84 / UTM zone 27S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=27 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 27S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-21],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32727\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32728, "epsg", 32728,
		      "WGS 84 / UTM zone 28S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=28 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 28S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-15],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32728\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32729, "epsg", 32729,
		      "WGS 84 / UTM zone 29S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=29 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 29S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-9],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32729\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32730, "epsg", 32730,
		      "WGS 84 / UTM zone 30S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=30 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 30S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",-3],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32730\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32731, "epsg", 32731,
		      "WGS 84 / UTM zone 31S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=31 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 31S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",3],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"32731\"],AXIS");
    add_srs_wkt (p, 11, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32732, "epsg", 32732,
		      "WGS 84 / UTM zone 32S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=32 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 32S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",9],PARAMETER[\"scale_factor\",0.999");
    add_srs_wkt (p, 9,
		 "6],PARAMETER[\"false_easting\",500000],PARAMETER[\"false");
    add_srs_wkt (p, 10,
		 "_northing\",10000000],AUTHORITY[\"EPSG\",\"32732\"],AXIS");
    add_srs_wkt (p, 11, "[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32733, "epsg", 32733,
		      "WGS 84 / UTM zone 33S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=33 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 33S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",15],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32733\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32734, "epsg", 32734,
		      "WGS 84 / UTM zone 34S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=34 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 34S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",21],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32734\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32735, "epsg", 32735,
		      "WGS 84 / UTM zone 35S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=35 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 35S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",27],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32735\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32736, "epsg", 32736,
		      "WGS 84 / UTM zone 36S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=36 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 36S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",33],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32736\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32737, "epsg", 32737,
		      "WGS 84 / UTM zone 37S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=37 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 37S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",39],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32737\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32738, "epsg", 32738,
		      "WGS 84 / UTM zone 38S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=38 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 38S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",45],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32738\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32739, "epsg", 32739,
		      "WGS 84 / UTM zone 39S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=39 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 39S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",51],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32739\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32740, "epsg", 32740,
		      "WGS 84 / UTM zone 40S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=40 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 40S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",57],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32740\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32741, "epsg", 32741,
		      "WGS 84 / UTM zone 41S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=41 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 41S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",63],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32741\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32742, "epsg", 32742,
		      "WGS 84 / UTM zone 42S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=42 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 42S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",69],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32742\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32743, "epsg", 32743,
		      "WGS 84 / UTM zone 43S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=43 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 43S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",75],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32743\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32744, "epsg", 32744,
		      "WGS 84 / UTM zone 44S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=44 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 44S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",81],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32744\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32745, "epsg", 32745,
		      "WGS 84 / UTM zone 45S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=45 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 45S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",87],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32745\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32746, "epsg", 32746,
		      "WGS 84 / UTM zone 46S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=46 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 46S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",93],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32746\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32747, "epsg", 32747,
		      "WGS 84 / UTM zone 47S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=47 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 47S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",99],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "96],PARAMETER[\"false_easting\",500000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",10000000],AUTHORITY[\"EPSG\",\"32747\"],AXI");
    add_srs_wkt (p, 11, "S[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32748, "epsg", 32748,
		      "WGS 84 / UTM zone 48S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=48 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 48S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",105],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32748\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32749, "epsg", 32749,
		      "WGS 84 / UTM zone 49S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=49 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 49S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",111],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32749\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32750, "epsg", 32750,
		      "WGS 84 / UTM zone 50S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=50 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 50S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",117],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32750\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32751, "epsg", 32751,
		      "WGS 84 / UTM zone 51S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=51 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 51S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",123],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32751\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32752, "epsg", 32752,
		      "WGS 84 / UTM zone 52S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=52 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 52S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",129],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32752\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32753, "epsg", 32753,
		      "WGS 84 / UTM zone 53S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=53 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 53S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",135],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32753\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32754, "epsg", 32754,
		      "WGS 84 / UTM zone 54S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=54 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 54S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32754\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32755, "epsg", 32755,
		      "WGS 84 / UTM zone 55S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=55 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 55S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",147],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32755\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
}

static void
initialize_epsg_29 (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
    struct epsg_defs *p;
    p = add_epsg_def (first, last, 32756, "epsg", 32756,
		      "WGS 84 / UTM zone 56S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=56 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 56S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",153],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32756\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32757, "epsg", 32757,
		      "WGS 84 / UTM zone 57S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=57 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 57S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",159],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32757\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32758, "epsg", 32758,
		      "WGS 84 / UTM zone 58S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=58 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 58S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",165],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32758\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32759, "epsg", 32759,
		      "WGS 84 / UTM zone 59S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=59 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 59S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",171],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32759\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32760, "epsg", 32760,
		      "WGS 84 / UTM zone 60S");
    add_proj4text (p, 0,
		   "+proj=utm +zone=60 +south +ellps=WGS84 +datum=WGS84 +uni");
    add_proj4text (p, 1, "ts=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UTM zone 60S\",GEOGCS[\"WGS 84\",DATUM");
    add_srs_wkt (p, 1,
		 "[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,");
    add_srs_wkt (p, 2,
		 "AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"");
    add_srs_wkt (p, 3,
		 "]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],");
    add_srs_wkt (p, 4,
		 "UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"");
    add_srs_wkt (p, 5,
		 "9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,");
    add_srs_wkt (p, 6,
		 "AUTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Me");
    add_srs_wkt (p, 7,
		 "rcator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[");
    add_srs_wkt (p, 8,
		 "\"central_meridian\",177],PARAMETER[\"scale_factor\",0.9");
    add_srs_wkt (p, 9,
		 "996],PARAMETER[\"false_easting\",500000],PARAMETER[\"fal");
    add_srs_wkt (p, 10,
		 "se_northing\",10000000],AUTHORITY[\"EPSG\",\"32760\"],AX");
    add_srs_wkt (p, 11, "IS[\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
    p = add_epsg_def (first, last, 32761, "epsg", 32761, "WGS 84 / UPS South");
    add_proj4text (p, 0,
		   "+proj=stere +lat_0=-90 +lat_ts=-90 +lon_0=0 +k=0.994 +x_");
    add_proj4text (p, 1,
		   "0=2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 +units=");
    add_proj4text (p, 2, "m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / UPS South\",GEOGCS[\"WGS 84\",DATUM[\"");
    add_srs_wkt (p, 1,
		 "WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUT");
    add_srs_wkt (p, 2,
		 "HORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]]");
    add_srs_wkt (p, 3,
		 ",PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UN");
    add_srs_wkt (p, 4,
		 "IT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9");
    add_srs_wkt (p, 5,
		 "122\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,A");
    add_srs_wkt (p, 6,
		 "UTHORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Polar_Stereogr");
    add_srs_wkt (p, 7,
		 "aphic\"],PARAMETER[\"latitude_of_origin\",-90],PARAMETER");
    add_srs_wkt (p, 8,
		 "[\"central_meridian\",0],PARAMETER[\"scale_factor\",0.99");
    add_srs_wkt (p, 9,
		 "4],PARAMETER[\"false_easting\",2000000],PARAMETER[\"fals");
    add_srs_wkt (p, 10,
		 "e_northing\",2000000],AUTHORITY[\"EPSG\",\"32761\"],AXIS");
    add_srs_wkt (p, 11, "[\"Northing\",UNKNOWN],AXIS[\"Easting\",UNKNOWN]]");
    p = add_epsg_def (first, last, 32766, "epsg", 32766, "WGS 84 / TM 36 SE");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=36 +k=0.9996 +x_0=500000 +y_");
    add_proj4text (p, 1,
		   "0=10000000 +ellps=WGS84 +datum=WGS84 +units=m +no_defs");
    add_srs_wkt (p, 0,
		 "PROJCS[\"WGS 84 / TM 36 SE\",GEOGCS[\"WGS 84\",DATUM[\"W");
    add_srs_wkt (p, 1,
		 "GS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTH");
    add_srs_wkt (p, 2,
		 "ORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],");
    add_srs_wkt (p, 3,
		 "PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNI");
    add_srs_wkt (p, 4,
		 "T[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"91");
    add_srs_wkt (p, 5,
		 "22\"]],AUTHORITY[\"EPSG\",\"4326\"]],UNIT[\"metre\",1,AU");
    add_srs_wkt (p, 6,
		 "THORITY[\"EPSG\",\"9001\"]],PROJECTION[\"Transverse_Merc");
    add_srs_wkt (p, 7,
		 "ator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"");
    add_srs_wkt (p, 8,
		 "central_meridian\",36],PARAMETER[\"scale_factor\",0.9996");
    add_srs_wkt (p, 9,
		 "],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_");
    add_srs_wkt (p, 10,
		 "northing\",10000000],AUTHORITY[\"EPSG\",\"32766\"],AXIS[");
    add_srs_wkt (p, 11, "\"Easting\",EAST],AXIS[\"Northing\",NORTH]]");
#ifndef OMIT_EPSG		/* resuming conditional EPSG initialization */

    p = add_epsg_def (first, last, 40000, "gfoss.it", 1,
		      "Italy mainland zone 1 GB Roma40");
    add_proj4text (p, 0,
		   "+proj=tmerc+lat_0=0 +lon_0=9  +k=0.9996 +x_0=1500000 +y_");
    add_proj4text (p, 1,
		   "0=0 +ellps=intl +units=m +towgs84=-104.1,-49.1,-9.9,0.97");
    add_proj4text (p, 2, "1,-2.917,0.714,-11.68 +no_defs");
    add_srs_wkt (p, 0, "");
    p = add_epsg_def (first, last, 40001, "gfoss.it", 2,
		      "Italy mainland zone 2 GB Roma40");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9996 +x_0=2520000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +units=m +towgs84=-104.1,-49.1,-9.9,0.9");
    add_proj4text (p, 2, "71,-2.917,0.714,-11.68 +no_defs");
    add_srs_wkt (p, 0, "");
    p = add_epsg_def (first, last, 40002, "gfoss.it", 3,
		      "Italy Sardinia GB Roma40");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9  +k=0.9996 +x_0=1500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +units=m +towgs84=-168.6,-34.0,38.6,-0.");
    add_proj4text (p, 2, "374,-0.679,-1.379,-9.48 +no_defs");
    add_srs_wkt (p, 0, "");
    p = add_epsg_def (first, last, 40003, "gfoss.it", 4,
		      "Italy Sicily GB Roma40");
    add_proj4text (p, 0,
		   "+proj=tmerc +lat_0=0 +lon_0=9  +k=0.9996 +x_0=1500000 +y");
    add_proj4text (p, 1,
		   "_0=0 +ellps=intl +units=m +towgs84=-50.2,-50.4,84.8,-0.6");
    add_proj4text (p, 2, "90,-2.012,0.459,-28.08  +no_defs");
    add_srs_wkt (p, 0, "");
#endif /* full EPSG initialization enabled/disabled */
}

static void
initialize_epsg (struct epsg_defs **first, struct epsg_defs **last)
{
/* initializing the EPSG defs list */
    initialize_epsg_00 (first, last);
    initialize_epsg_01 (first, last);
    initialize_epsg_02 (first, last);
    initialize_epsg_03 (first, last);
    initialize_epsg_04 (first, last);
    initialize_epsg_05 (first, last);
    initialize_epsg_06 (first, last);
    initialize_epsg_07 (first, last);
    initialize_epsg_08 (first, last);
    initialize_epsg_09 (first, last);
    initialize_epsg_10 (first, last);
    initialize_epsg_11 (first, last);
    initialize_epsg_12 (first, last);
    initialize_epsg_13 (first, last);
    initialize_epsg_14 (first, last);
    initialize_epsg_15 (first, last);
    initialize_epsg_16 (first, last);
    initialize_epsg_17 (first, last);
    initialize_epsg_18 (first, last);
    initialize_epsg_19 (first, last);
    initialize_epsg_20 (first, last);
    initialize_epsg_21 (first, last);
    initialize_epsg_22 (first, last);
    initialize_epsg_23 (first, last);
    initialize_epsg_24 (first, last);
    initialize_epsg_25 (first, last);
    initialize_epsg_26 (first, last);
    initialize_epsg_27 (first, last);
    initialize_epsg_28 (first, last);
    initialize_epsg_29 (first, last);
}

static void
free_epsg (struct epsg_defs *first)
{
/* memory cleanup - destroying the EPSG list */
    struct epsg_defs *p = first;
    struct epsg_defs *pn;
    while (p)
      {
	  pn = p->next;
	  free_epsg_def (p);
	  p = pn;
      }
}

static int
populate_spatial_ref_sys (sqlite3 * handle)
{
/* populating the EPSG dataset into the SPATIAL_REF_SYS table */
    struct epsg_defs *first = NULL;
    struct epsg_defs *last = NULL;
    struct epsg_defs *p;
    char sql[1024];
    char *errMsg = NULL;
    int ret;
    sqlite3_stmt *stmt;

/* initializing the EPSG defs list */
    initialize_epsg (&first, &last);

/* starting a transaction */
    ret = sqlite3_exec (handle, "BEGIN", NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "%s\n", errMsg);
	  sqlite3_free (errMsg);
	  goto error;
      }

/* preparing the SQL parameterized statement */
    strcpy (sql, "INSERT INTO spatial_ref_sys ");
    strcat (sql,
	    "(srid, auth_name, auth_srid, ref_sys_name, proj4text, srs_wkt) ");
    strcat (sql, "VALUES (?, ?, ?, ?, ?, ?)");
    ret = sqlite3_prepare_v2 (handle, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "%s\n", sqlite3_errmsg (handle));
	  goto error;
      }
    p = first;
    while (p)
      {
	  if (p->srid < 0 || p->auth_name == NULL)
	      break;
	  sqlite3_reset (stmt);
	  sqlite3_clear_bindings (stmt);
	  sqlite3_bind_int (stmt, 1, p->srid);
	  sqlite3_bind_text (stmt, 2, p->auth_name, strlen (p->auth_name),
			     SQLITE_STATIC);
	  sqlite3_bind_int (stmt, 3, p->auth_srid);
	  sqlite3_bind_text (stmt, 4, p->ref_sys_name, strlen (p->ref_sys_name),
			     SQLITE_STATIC);
	  sqlite3_bind_text (stmt, 5, p->proj4text, strlen (p->proj4text),
			     SQLITE_STATIC);
	  if (strlen (p->srs_wkt) == 0)
	      sqlite3_bind_null (stmt, 6);
	  else
	      sqlite3_bind_text (stmt, 6, p->srs_wkt, strlen (p->srs_wkt),
				 SQLITE_STATIC);
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	      ;
	  else
	    {
		fprintf (stderr, "%s\n", sqlite3_errmsg (handle));
		sqlite3_finalize (stmt);
		goto error;
	    }
	  p = p->next;
      }
    sqlite3_finalize (stmt);

/* confirming the transaction */
    ret = sqlite3_exec (handle, "COMMIT", NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "%s\n", errMsg);
	  sqlite3_free (errMsg);
	  goto error;
      }

/* freeing the EPSG defs list */
    free_epsg (first);

    return 1;
  error:
/* trying to perform a ROLLBACK anyway */
    ret = sqlite3_exec (handle, "ROLLBACK", NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "%s\n", errMsg);
	  sqlite3_free (errMsg);
      }

/* freeing the EPSG defs list */
    free_epsg (first);

    return 0;
}

static int
exists_spatial_ref_sys (sqlite3 * handle)
{
/* checking if the SPATIAL_REF_SYS table exists */
    int ret;
    int ok = 0;
    char sql[1024];
    char **results;
    int n_rows;
    int n_columns;
    char *err_msg = NULL;

    strcpy (sql,
	    "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE 'spatial_ref_sys'");
    ret =
	sqlite3_get_table (handle, sql, &results, &n_rows, &n_columns,
			   &err_msg);
    if (ret != SQLITE_OK)
      {
/* some error occurred */
	  fprintf (stderr, "XX %s\n", err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    if (n_rows > 0)
	ok = 1;
    sqlite3_free_table (results);
    return ok;
}

static int
check_spatial_ref_sys (sqlite3 * handle)
{
/* checking if the SPATIAL_REF_SYS table has an appropriate layout */
    int ret;
    int i;
    const char *name;
    char sql[1024];
    char **results;
    int n_rows;
    int n_columns;
    char *err_msg = NULL;
    int rs_srid = 0;
    int auth_name = 0;
    int auth_srid = 0;
    int ref_sys_name = 0;
    int proj4text = 0;
    int srs_wkt = 0;

    strcpy (sql, "PRAGMA table_info(spatial_ref_sys)");
    ret =
	sqlite3_get_table (handle, sql, &results, &n_rows, &n_columns,
			   &err_msg);
    if (ret != SQLITE_OK)
      {
/* some error occurred */
	  fprintf (stderr, "%s\n", err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    if (n_rows > 0)
      {
	  for (i = 1; i <= n_rows; i++)
	    {
		name = results[(i * n_columns) + 1];
		if (strcasecmp (name, "srid") == 0)
		    rs_srid = 1;
		if (strcasecmp (name, "auth_name") == 0)
		    auth_name = 1;
		if (strcasecmp (name, "auth_srid") == 0)
		    auth_srid = 1;
		if (strcasecmp (name, "ref_sys_name") == 0)
		    ref_sys_name = 1;
		if (strcasecmp (name, "proj4text") == 0)
		    proj4text = 1;
		if (strcasecmp (name, "srs_wkt") == 0)
		    srs_wkt = 1;
	    }
      }
    sqlite3_free_table (results);
    if (rs_srid && auth_name && auth_srid && ref_sys_name && proj4text
	&& srs_wkt)
	ret = 1;
    else
	ret = 0;
    return ret;
}

static int
spatial_ref_sys_count (sqlite3 * handle)
{
/* checking if the SPATIAL_REF_SYS table is empty */
    int ret;
    int i;
    int count = 0;
    char sql[1024];
    char **results;
    int n_rows;
    int n_columns;
    char *err_msg = NULL;

    strcpy (sql, "SELECT Count(*) FROM spatial_ref_sys");
    ret =
	sqlite3_get_table (handle, sql, &results, &n_rows, &n_columns,
			   &err_msg);
    if (ret != SQLITE_OK)
      {
/* some error occurred */
	  fprintf (stderr, "%s\n", err_msg);
	  sqlite3_free (err_msg);
	  return 0;
      }
    if (n_rows > 0)
      {
	  for (i = 1; i <= n_rows; i++)
	    {
		count = atoi (results[(i * n_columns) + 0]);
	    }
      }
    sqlite3_free_table (results);
    return count;
}

SPATIALITE_DECLARE int
spatial_ref_sys_init (sqlite3 * handle, int verbose)
{
/* populating the EPSG dataset into the SPATIAL_REF_SYS table */
    if (!exists_spatial_ref_sys (handle))
      {
	  if (verbose)
	      fprintf (stderr, "the SPATIAL_REF_SYS table doesn't exists\n");
	  return 0;
      }
    if (!check_spatial_ref_sys (handle))
      {
	  if (verbose)
	      fprintf (stderr,
		       "the SPATIAL_REF_SYS table has an unsupported layout\n");
	  return 0;
      }
    if (spatial_ref_sys_count (handle))
      {
	  if (verbose)
	      fprintf (stderr,
		       "the SPATIAL_REF_SYS table already contains some row(s)\n");
	  return 0;
      }
    if (populate_spatial_ref_sys (handle))
      {
	  if (verbose)
	      fprintf (stderr,
		       "OK: the SPATIAL_REF_SYS table was successfully populated\n");
	  return 1;
      }
    return 0;
}
/**************** End file: srs_init.c **********/


/**************** Begin file: shapefiles.c **********/

#if defined(_WIN32) && !defined(__MINGW32__)
/* MSVC strictly requires this include [off_t] */
/* #include <sys/types.h> */
#endif

/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

#ifdef SPATIALITE_AMALGAMATION
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/gaiaaux.h> */
/* #include <spatialite/gaiageo.h> */
/* #include <spatialite.h> */

/* #include <freexl.h> */

#if defined(_WIN32) && !defined(__MINGW32__)
#define strcasecmp	_stricmp
#endif

struct dupl_column
{
/* a column value in a duplicated row */
    int pos;
    char *name;
    int type;
    sqlite3_int64 int_value;
    double dbl_value;
    const char *txt_value;
    const void *blob;
    int size;
    int query_pos;
    struct dupl_column *next;
};

struct dupl_row
{
/* a duplicated row with column values */
    int count;
    struct dupl_column *first;
    struct dupl_column *last;
    const char *table;
};

static void
clean_dupl_row (struct dupl_row *str)
{
/* destroying a duplicated row struct */
    struct dupl_column *p;
    struct dupl_column *pn;
    p = str->first;
    while (p)
      {
	  pn = p->next;
	  free (p->name);
	  free (p);
	  p = pn;
      }
}

static void
add_to_dupl_row (struct dupl_row *str, const char *name)
{
/* adding a column to the duplicated row struct */
    int len;
    struct dupl_column *p = malloc (sizeof (struct dupl_column));
    p->pos = str->count;
    len = strlen (name);
    p->name = malloc (len + 1);
    strcpy (p->name, name);
    str->count++;
    p->type = SQLITE_NULL;
    p->next = NULL;
    if (str->first == NULL)
	str->first = p;
    if (str->last)
	str->last->next = p;
    str->last = p;
}

static void
set_int_value (struct dupl_row *str, int pos, sqlite3_int64 value)
{
/* setting up an integer value */
    struct dupl_column *p = str->first;
    while (p)
      {
	  if (p->pos == pos)
	    {
		p->type = SQLITE_INTEGER;
		p->int_value = value;
		return;
	    }
	  p = p->next;
      }
}

static void
set_double_value (struct dupl_row *str, int pos, double value)
{
/* setting up a double value */
    struct dupl_column *p = str->first;
    while (p)
      {
	  if (p->pos == pos)
	    {
		p->type = SQLITE_FLOAT;
		p->dbl_value = value;
		return;
	    }
	  p = p->next;
      }
}

static void
set_text_value (struct dupl_row *str, int pos, const char *value)
{
/* setting up a text value */
    struct dupl_column *p = str->first;
    while (p)
      {
	  if (p->pos == pos)
	    {
		p->type = SQLITE_TEXT;
		p->txt_value = value;
		return;
	    }
	  p = p->next;
      }
}

static void
set_blob_value (struct dupl_row *str, int pos, const void *blob, int size)
{
/* setting up a blob value */
    struct dupl_column *p = str->first;
    while (p)
      {
	  if (p->pos == pos)
	    {
		p->type = SQLITE_BLOB;
		p->blob = blob;
		p->size = size;
		return;
	    }
	  p = p->next;
      }
}

static void
set_null_value (struct dupl_row *str, int pos)
{
/* setting up a NULL value */
    struct dupl_column *p = str->first;
    while (p)
      {
	  if (p->pos == pos)
	    {
		p->type = SQLITE_NULL;
		return;
	    }
	  p = p->next;
      }
}

static void
reset_query_pos (struct dupl_row *str)
{
/* resetting QueryPos for BLOBs */
    struct dupl_column *p = str->first;
    while (p)
      {
	  p->query_pos = -1;
	  p = p->next;
      }
}

static int
check_dupl_blob2 (struct dupl_column *ptr, const void *blob, int size)
{
/* checking a BLOB value */
    if (ptr->type != SQLITE_BLOB)
	return 0;
    if (ptr->size != size)
	return 0;
    if (memcmp (ptr->blob, blob, size) != 0)
	return 0;
    return 1;
}

static int
check_dupl_blob (struct dupl_row *str, int pos, const void *blob, int size)
{
/* checking a BLOB value */
    struct dupl_column *p = str->first;
    while (p)
      {
	  if (p->query_pos == pos)
	    {
		return check_dupl_blob2 (p, blob, size);
	    }
	  p = p->next;
      }
    return 0;
}

static gaiaDbfFieldPtr
getDbfField (gaiaDbfListPtr list, char *name)
{
/* find a DBF attribute by name */
    gaiaDbfFieldPtr fld = list->First;
    while (fld)
      {
	  if (strcasecmp (fld->Name, name) == 0)
	      return fld;
	  fld = fld->Next;
      }
    return NULL;
}

#ifndef OMIT_ICONV		/* ICONV enabled: supporting SHP */

SPATIALITE_DECLARE int
load_shapefile (sqlite3 * sqlite, char *shp_path, char *table, char *charset,
		int srid, char *column, int coerce2d, int compressed,
		int verbose, int spatial_index, int *rows, char *err_msg)
{
    sqlite3_stmt *stmt = NULL;
    int ret;
    char *errMsg = NULL;
    char sql[65536];
    char dummyName[4096];
    int already_exists = 0;
    int metadata = 0;
    int sqlError = 0;
    gaiaShapefilePtr shp = NULL;
    gaiaDbfFieldPtr dbf_field;
    int cnt;
    int col_cnt;
    int seed;
    int len;
    int dup;
    int idup;
    int current_row;
    char **col_name = NULL;
    unsigned char *blob;
    int blob_size;
    char *geom_type;
    char *txt_dims;
    char *geo_column = column;
    if (!geo_column)
	geo_column = "Geometry";
/* checking if TABLE already exists */
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE '%s'",
	     table);
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load shapefile error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  else
	      sprintf (err_msg, "load shapefile error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  return 0;
      }
    while (1)
      {
	  /* scrolling the result set */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	      already_exists = 1;
	  else
	    {
		fprintf (stderr, "load shapefile error: <%s>\n",
			 sqlite3_errmsg (sqlite));
		break;
	    }
      }
    sqlite3_finalize (stmt);
    if (already_exists)
      {
	  if (!err_msg)
	      fprintf (stderr,
		       "load shapefile error: table '%s' already exists\n",
		       table);
	  else
	      sprintf (err_msg,
		       "load shapefile error: table '%s' already exists\n",
		       table);
	  return 0;
      }
/* checking if MetaData GEOMETRY_COLUMNS exists */
    strcpy (sql,
	    "SELECT name FROM sqlite_master WHERE type = 'table' AND name = 'geometry_columns'");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load shapefile error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  else
	      sprintf (err_msg, "load shapefile error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  return 0;
      }
    while (1)
      {
	  /* scrolling the result set */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	      metadata = 1;
	  else
	    {
		fprintf (stderr, "load shapefile error: <%s>\n",
			 sqlite3_errmsg (sqlite));
		break;
	    }
      }
    sqlite3_finalize (stmt);
    shp = gaiaAllocShapefile ();
    gaiaOpenShpRead (shp, shp_path, charset, "UTF-8");
    if (!(shp->Valid))
      {
	  if (!err_msg)
	    {
		fprintf (stderr,
			 "load shapefile error: cannot open shapefile '%s'\n",
			 shp_path);
		if (shp->LastError)
		    fprintf (stderr, "\tcause: %s\n", shp->LastError);
	    }
	  else
	    {
		char extra[512];
		*extra = '\0';
		if (shp->LastError)
		    sprintf (extra, "\n\tcause: %s\n", shp->LastError);
		sprintf (err_msg,
			 "load shapefile error: cannot open shapefile '%s'%s",
			 shp_path, extra);
	    }
	  gaiaFreeShapefile (shp);
	  return 0;
      }
/* checking for duplicate / illegal column names and antialising them */
    col_cnt = 0;
    dbf_field = shp->Dbf->First;
    while (dbf_field)
      {
	  /* counting DBF fields */
	  col_cnt++;
	  dbf_field = dbf_field->Next;
      }
    col_name = malloc (sizeof (char *) * col_cnt);
    cnt = 0;
    seed = 0;
    dbf_field = shp->Dbf->First;
    while (dbf_field)
      {
	  /* preparing column names */
	  strcpy (dummyName, dbf_field->Name);
	  dup = 0;
	  for (idup = 0; idup < cnt; idup++)
	    {
		if (strcasecmp (dummyName, *(col_name + idup)) == 0)
		    dup = 1;
	    }
	  if (strcasecmp (dummyName, "PK_UID") == 0)
	      dup = 1;
	  if (strcasecmp (dummyName, geo_column) == 0)
	      dup = 1;
	  if (dup)
	      sprintf (dummyName, "COL_%d", seed++);
	  len = strlen (dummyName);
	  *(col_name + cnt) = malloc (len + 1);
	  strcpy (*(col_name + cnt), dummyName);
	  cnt++;
	  dbf_field = dbf_field->Next;
      }
    if (verbose)
	fprintf (stderr,
		 "========\nLoading shapefile at '%s' into SQLite table '%s'\n",
		 shp_path, table);
/* starting a transaction */
    if (verbose)
	fprintf (stderr, "\nBEGIN;\n");
    ret = sqlite3_exec (sqlite, "BEGIN", NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load shapefile error: <%s>\n", errMsg);
	  else
	      sprintf (err_msg, "load shapefile error: <%s>\n", errMsg);
	  sqlite3_free (errMsg);
	  sqlError = 1;
	  goto clean_up;
      }
/* creating the Table */
    sprintf (sql, "CREATE TABLE %s", table);
    strcat (sql, " (\nPK_UID INTEGER PRIMARY KEY AUTOINCREMENT");
    cnt = 0;
    dbf_field = shp->Dbf->First;
    while (dbf_field)
      {
	  strcat (sql, ",\n\"");
	  strcat (sql, *(col_name + cnt));
	  cnt++;
	  switch (dbf_field->Type)
	    {
	    case 'C':
		strcat (sql, "\" TEXT");
		break;
	    case 'N':
		fflush (stderr);
		if (dbf_field->Decimals)
		    strcat (sql, "\" DOUBLE");
		else
		  {
		      if (dbf_field->Length <= 18)
			  strcat (sql, "\" INTEGER");
		      else
			  strcat (sql, "\" DOUBLE");
		  }
		break;
	    case 'D':
		strcat (sql, "\" DOUBLE");
		break;
	    case 'F':
		strcat (sql, "\" DOUBLE");
		break;
	    case 'L':
		strcat (sql, "\" INTEGER");
		break;
	    };
	  dbf_field = dbf_field->Next;
      }
    if (metadata)
	strcat (sql, ")");
    else
      {
	  strcat (sql, ",\n");
	  strcat (sql, geo_column);
	  strcat (sql, " BLOB)");
      }
    if (verbose)
	fprintf (stderr, "%s;\n", sql);
    ret = sqlite3_exec (sqlite, sql, NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load shapefile error: <%s>\n", errMsg);
	  else
	      sprintf (err_msg, "load shapefile error: <%s>\n", errMsg);
	  sqlite3_free (errMsg);
	  sqlError = 1;
	  goto clean_up;
      }
    if (metadata)
      {
	  /* creating Geometry column */
	  switch (shp->Shape)
	    {
	    case GAIA_SHP_POINT:
	    case GAIA_SHP_POINTM:
	    case GAIA_SHP_POINTZ:
		geom_type = "POINT";
		break;
	    case GAIA_SHP_MULTIPOINT:
	    case GAIA_SHP_MULTIPOINTM:
	    case GAIA_SHP_MULTIPOINTZ:
		geom_type = "MULTIPOINT";
		break;
	    case GAIA_SHP_POLYLINE:
	    case GAIA_SHP_POLYLINEM:
	    case GAIA_SHP_POLYLINEZ:
		gaiaShpAnalyze (shp);
		if (shp->EffectiveType == GAIA_LINESTRING)
		    geom_type = "LINESTRING";
		else
		    geom_type = "MULTILINESTRING";
		break;
	    case GAIA_SHP_POLYGON:
	    case GAIA_SHP_POLYGONM:
	    case GAIA_SHP_POLYGONZ:
		gaiaShpAnalyze (shp);
		if (shp->EffectiveType == GAIA_POLYGON)
		    geom_type = "POLYGON";
		else
		    geom_type = "MULTIPOLYGON";
		break;
	    };
	  if (coerce2d)
	      shp->EffectiveDims = GAIA_XY;
	  switch (shp->EffectiveDims)
	    {
	    case GAIA_XY_Z:
		txt_dims = "XYZ";
		break;
	    case GAIA_XY_M:
		txt_dims = "XYM";
		break;
	    case GAIA_XY_Z_M:
		txt_dims = "XYZM";
		break;
	    default:
		txt_dims = "XY";
		break;
	    };
	  sprintf (sql, "SELECT AddGeometryColumn('%s', '%s', %d, '%s', '%s')",
		   table, geo_column, srid, geom_type, txt_dims);
	  if (verbose)
	      fprintf (stderr, "%s;\n", sql);
	  ret = sqlite3_exec (sqlite, sql, NULL, 0, &errMsg);
	  if (ret != SQLITE_OK)
	    {
		if (!err_msg)
		    fprintf (stderr, "load shapefile error: <%s>\n", errMsg);
		else
		    sprintf (err_msg, "load shapefile error: <%s>\n", errMsg);
		sqlite3_free (errMsg);
		sqlError = 1;
		goto clean_up;
	    }
	  if (spatial_index)
	    {
		/* creating the Spatial Index */
		sprintf (sql, "SELECT CreateSpatialIndex('%s', '%s')",
			 table, geo_column);
		ret = sqlite3_exec (sqlite, sql, NULL, 0, &errMsg);
		if (ret != SQLITE_OK)
		  {
		      if (!err_msg)
			  fprintf (stderr, "load shapefile error: <%s>\n",
				   errMsg);
		      else
			  sprintf (err_msg, "load shapefile error: <%s>\n",
				   errMsg);
		      sqlite3_free (errMsg);
		      sqlError = 1;
		      goto clean_up;
		  }
	    }
      }
    else
      {
	  /* no Metadata */
	  if (shp->Shape == GAIA_SHP_POLYLINE
	      || shp->Shape == GAIA_SHP_POLYLINEM
	      || shp->Shape == GAIA_SHP_POLYLINEZ
	      || shp->Shape == GAIA_SHP_POLYGON
	      || shp->Shape == GAIA_SHP_POLYGONM
	      || shp->Shape == GAIA_SHP_POLYGONZ)
	    {
		/* 
		   / fixing anyway the Geometry type for 
		   / LINESTRING/MULTILINESTRING or 
		   / POLYGON/MULTIPOLYGON
		 */
		gaiaShpAnalyze (shp);
	    }
      }
    /* preparing the INSERT INTO parametrerized statement */
    sprintf (sql, "INSERT INTO %s (PK_UID,", table);
    cnt = 0;
    dbf_field = shp->Dbf->First;
    while (dbf_field)
      {
	  /* columns corresponding to some DBF attribute */
	  strcat (sql, "\"");
	  strcat (sql, *(col_name + cnt++));
	  strcat (sql, "\" ,");
	  dbf_field = dbf_field->Next;
      }
    strcat (sql, geo_column);	/* the GEOMETRY column */
    strcat (sql, ")\nVALUES (?");
    dbf_field = shp->Dbf->First;
    while (dbf_field)
      {
	  /* column values */
	  strcat (sql, ", ?");
	  dbf_field = dbf_field->Next;
      }
    strcat (sql, ", ?)");	/* the GEOMETRY column */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load shapefile error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  else
	      sprintf (err_msg, "load shapefile error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  sqlError = 1;
	  goto clean_up;
      }
    current_row = 0;
    while (1)
      {
	  /* inserting rows from shapefile */
	  ret = gaiaReadShpEntity (shp, current_row, srid);
	  if (!ret)
	    {
		if (!(shp->LastError))	/* normal SHP EOF */
		    break;
		if (!err_msg)
		    fprintf (stderr, "%s\n", shp->LastError);
		else
		    sprintf (err_msg, "%s\n", shp->LastError);
		sqlError = 1;
		sqlite3_finalize (stmt);
		goto clean_up;
	    }
	  current_row++;
	  /* binding query params */
	  sqlite3_reset (stmt);
	  sqlite3_clear_bindings (stmt);
	  sqlite3_bind_int (stmt, 1, current_row);
	  cnt = 0;
	  dbf_field = shp->Dbf->First;
	  while (dbf_field)
	    {
		/* column values */
		if (!(dbf_field->Value))
		    sqlite3_bind_null (stmt, cnt + 2);
		else
		  {
		      switch (dbf_field->Value->Type)
			{
			case GAIA_INT_VALUE:
			    sqlite3_bind_int64 (stmt, cnt + 2,
						dbf_field->Value->IntValue);
			    break;
			case GAIA_DOUBLE_VALUE:
			    sqlite3_bind_double (stmt, cnt + 2,
						 dbf_field->Value->DblValue);
			    break;
			case GAIA_TEXT_VALUE:
			    sqlite3_bind_text (stmt, cnt + 2,
					       dbf_field->Value->TxtValue,
					       strlen (dbf_field->
						       Value->TxtValue),
					       SQLITE_STATIC);
			    break;
			default:
			    sqlite3_bind_null (stmt, cnt + 2);
			    break;
			}
		  }
		cnt++;
		dbf_field = dbf_field->Next;
	    }
	  if (shp->Dbf->Geometry)
	    {
		if (compressed)
		    gaiaToCompressedBlobWkb (shp->Dbf->Geometry, &blob,
					     &blob_size);
		else
		    gaiaToSpatiaLiteBlobWkb (shp->Dbf->Geometry, &blob,
					     &blob_size);
		sqlite3_bind_blob (stmt, cnt + 2, blob, blob_size, free);
	    }
	  else
	    {
		/* handling a NULL-Geometry */
		sqlite3_bind_null (stmt, cnt + 2);
	    }
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	      ;
	  else
	    {
		if (!err_msg)
		    fprintf (stderr, "load shapefile error: <%s>\n",
			     sqlite3_errmsg (sqlite));
		else
		    sprintf (err_msg, "load shapefile error: <%s>\n",
			     sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		sqlError = 1;
		goto clean_up;
	    }
      }
    sqlite3_finalize (stmt);
  clean_up:
    gaiaFreeShapefile (shp);
    if (col_name)
      {
	  /* releasing memory allocation for column names */
	  for (cnt = 0; cnt < col_cnt; cnt++)
	      free (*(col_name + cnt));
	  free (col_name);
      }
    if (sqlError)
      {
	  /* some error occurred - ROLLBACK */
	  if (verbose)
	      fprintf (stderr, "ROLLBACK;\n");
	  ret = sqlite3_exec (sqlite, "ROLLBACK", NULL, 0, &errMsg);
	  if (ret != SQLITE_OK)
	    {
		fprintf (stderr, "load shapefile error: <%s>\n", errMsg);
		sqlite3_free (errMsg);
	    }
	  if (rows)
	      *rows = current_row;
	  return 0;
      }
    else
      {
	  /* ok - confirming pending transaction - COMMIT */
	  if (verbose)
	      fprintf (stderr, "COMMIT;\n");
	  ret = sqlite3_exec (sqlite, "COMMIT", NULL, 0, &errMsg);
	  if (ret != SQLITE_OK)
	    {
		if (!err_msg)
		    fprintf (stderr, "load shapefile error: <%s>\n", errMsg);
		else
		    sprintf (err_msg, "load shapefile error: <%s>\n", errMsg);
		sqlite3_free (errMsg);
		return 0;
	    }
	  if (rows)
	      *rows = current_row;
	  if (verbose)
	      fprintf (stderr,
		       "\nInserted %d rows into '%s' from SHAPEFILE\n========\n",
		       current_row, table);
	  if (err_msg)
	      sprintf (err_msg, "Inserted %d rows into '%s' from SHAPEFILE",
		       current_row, table);
	  return 1;
      }
}

#endif /* end ICONV (SHP) */

static void
output_prj_file (sqlite3 * sqlite, char *path, char *table, char *column)
{
/* exporting [if possible] a .PRJ file */
    char **results;
    int rows;
    int columns;
    int i;
    char *errMsg = NULL;
    int srid = -1;
    char sql[1024];
    char sql2[1024];
    int ret;
    int rs_srid = 0;
    int rs_srs_wkt = 0;
    const char *name;
    char srsWkt[8192];
    char dummy[8192];
    FILE *out;

/* step I: retrieving the SRID */
    sprintf (sql,
	     "SELECT srid FROM geometry_columns WHERE f_table_name = '%s' AND f_geometry_column = '%s'",
	     table, column);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "dump shapefile MetaData error: <%s>\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    for (i = 1; i <= rows; i++)
      {
	  srid = atoi (results[(i * columns) + 0]);
      }
    sqlite3_free_table (results);
    if (srid < 0)
      {
	  /* srid still undefined, so we'll read VIEWS_GEOMETRY_COLUMNS */
	  strcpy (sql, "SELECT srid FROM views_geometry_columns ");
	  strcat (sql,
		  "JOIN geometry_columns USING (f_table_name, f_geometry_column) ");
	  sprintf (sql2, "WHERE view_name = '%s' AND view_geometry = '%s'",
		   table, column);
	  strcat (sql, sql2);
	  ret =
	      sqlite3_get_table (sqlite, sql, &results, &rows, &columns,
				 &errMsg);
	  if (ret != SQLITE_OK)
	    {
		fprintf (stderr, "dump shapefile MetaData error: <%s>\n",
			 errMsg);
		sqlite3_free (errMsg);
		return;
	    }
	  for (i = 1; i <= rows; i++)
	    {
		srid = atoi (results[(i * columns) + 0]);
	    }
	  sqlite3_free_table (results);
      }
    if (srid < 0)
	return;

/* step II: checking if the SRS_WKT column actually exists */
    ret =
	sqlite3_get_table (sqlite, "PRAGMA table_info(spatial_ref_sys)",
			   &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "dump shapefile MetaData error: <%s>\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, "srid") == 0)
		    rs_srid = 1;
		if (strcasecmp (name, "auth_name") == 0)
		    rs_srs_wkt = 1;
	    }
      }
    sqlite3_free_table (results);
    if (rs_srid == 0 || rs_srs_wkt == 0)
	return;

/* step III: fetching WKT SRS */
    *srsWkt = '\0';
    sprintf (sql,
	     "SELECT srs_wkt FROM spatial_ref_sys WHERE srid = %d AND srs_wkt IS NOT NULL",
	     srid);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "dump shapefile MetaData error: <%s>\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		strcpy (srsWkt, results[(i * columns) + 0]);
	    }
      }
    sqlite3_free_table (results);
    if (strlen (srsWkt) == 0)
	return;

/* step IV: generating the .PRJ file */
    sprintf (dummy, "%s.prj", path);
    out = fopen (dummy, "wb");
    if (!out)
	goto no_file;
    fprintf (out, "%s\r\n", srsWkt);
    fclose (out);
  no_file:
    return;
}

static void
shp_double_quoted_sql (char *buf)
{
/* well-formatting a string to be used as an SQL name */
    char tmp[1024];
    char *in = tmp;
    char *out = buf;
    strcpy (tmp, buf);
    *out++ = '"';
    while (*in != '\0')
      {
	  if (*in == '"')
	      *out++ = '"';
	  *out++ = *in++;
      }
    *out++ = '"';
    *out = '\0';
}

#ifndef OMIT_ICONV		/* ICONV enabled: supporting SHAPEFILE and DBF */

SPATIALITE_DECLARE int
dump_shapefile (sqlite3 * sqlite, char *table, char *column, char *shp_path,
		char *charset, char *geom_type, int verbose, int *xrows,
		char *err_msg)
{
/* SHAPEFILE dump */
    char sql[1024];
    char dummy[1024];
    int shape = -1;
    int len;
    int ret;
    char *errMsg = NULL;
    sqlite3_stmt *stmt;
    int row1 = 0;
    int n_cols = 0;
    int offset = 0;
    int i;
    int rows = 0;
    int type;
    const void *blob_value;
    gaiaShapefilePtr shp = NULL;
    gaiaDbfListPtr dbf_export_list = NULL;
    gaiaDbfListPtr dbf_list = NULL;
    gaiaDbfListPtr dbf_write;
    gaiaDbfFieldPtr dbf_field;
    int *max_length = NULL;
    int *sql_type = NULL;
    if (geom_type)
      {
	  /* normalizing required geometry type */
	  if (strcasecmp ((char *) geom_type, "POINT") == 0)
	      shape = GAIA_POINT;
	  if (strcasecmp ((char *) geom_type, "LINESTRING") == 0)
	      shape = GAIA_LINESTRING;
	  if (strcasecmp ((char *) geom_type, "POLYGON") == 0)
	      shape = GAIA_POLYGON;
	  if (strcasecmp ((char *) geom_type, "MULTIPOINT") == 0)
	      shape = GAIA_POINT;
      }
    if (shape < 0)
      {
	  /* preparing SQL statement [no type was explicitly required, so we'll read GEOMETRY_COLUMNS */
	  char **results;
	  int rows;
	  int columns;
	  int i;
	  char metatype[256];
	  char metadims[256];
	  sprintf (sql,
		   "SELECT type, coord_dimension FROM geometry_columns WHERE f_table_name = '%s' AND f_geometry_column = '%s'",
		   table, column);
	  ret =
	      sqlite3_get_table (sqlite, sql, &results, &rows, &columns,
				 &errMsg);
	  if (ret != SQLITE_OK)
	    {
		if (!err_msg)
		    fprintf (stderr, "dump shapefile MetaData error: <%s>\n",
			     errMsg);
		else
		    sprintf (err_msg, "dump shapefile MetaData error: <%s>\n",
			     errMsg);
		sqlite3_free (errMsg);
		return 0;
	    }
	  *metatype = '\0';
	  *metadims = '\0';
	  for (i = 1; i <= rows; i++)
	    {
		strcpy (metatype, results[(i * columns) + 0]);
		strcpy (metadims, results[(i * columns) + 1]);
	    }
	  sqlite3_free_table (results);
	  if (strcasecmp (metatype, "POINT") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_POINTZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_POINTM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_POINTZM;
		else
		    shape = GAIA_POINT;
	    }
	  if (strcasecmp (metatype, "MULTIPOINT") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_MULTIPOINTZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_MULTIPOINTM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_MULTIPOINTZM;
		else
		    shape = GAIA_MULTIPOINT;
	    }
	  if (strcasecmp (metatype, "LINESTRING") == 0
	      || strcasecmp (metatype, "MULTILINESTRING") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_LINESTRINGZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_LINESTRINGM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_LINESTRINGZM;
		else
		    shape = GAIA_LINESTRING;
	    }
	  if (strcasecmp (metatype, "POLYGON") == 0
	      || strcasecmp (metatype, "MULTIPOLYGON") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_POLYGONZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_POLYGONM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_POLYGONZM;
		else
		    shape = GAIA_POLYGON;
	    }
      }
    if (shape < 0)
      {
	  /* preparing SQL statement [type still undefined, so we'll read VIEWS_GEOMETRY_COLUMNS */
	  char **results;
	  int rows;
	  int columns;
	  int i;
	  char metatype[256];
	  char metadims[256];
	  char sql2[1024];
	  strcpy (sql,
		  "SELECT type, coord_dimension FROM views_geometry_columns ");
	  strcat (sql,
		  "JOIN geometry_columns USING (f_table_name, f_geometry_column) ");
	  sprintf (sql2, "WHERE view_name = '%s' AND view_geometry = '%s'",
		   table, column);
	  strcat (sql, sql2);
	  ret =
	      sqlite3_get_table (sqlite, sql, &results, &rows, &columns,
				 &errMsg);
	  if (ret != SQLITE_OK)
	    {
		if (!err_msg)
		    fprintf (stderr, "dump shapefile MetaData error: <%s>\n",
			     errMsg);
		else
		    sprintf (err_msg, "dump shapefile MetaData error: <%s>\n",
			     errMsg);
		sqlite3_free (errMsg);
		return 0;
	    }
	  *metatype = '\0';
	  *metadims = '\0';
	  for (i = 1; i <= rows; i++)
	    {
		strcpy (metatype, results[(i * columns) + 0]);
		strcpy (metadims, results[(i * columns) + 1]);
	    }
	  sqlite3_free_table (results);
	  if (strcasecmp (metatype, "POINT") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_POINTZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_POINTM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_POINTZM;
		else
		    shape = GAIA_POINT;
	    }
	  if (strcasecmp (metatype, "MULTIPOINT") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_MULTIPOINTZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_MULTIPOINTM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_MULTIPOINTZM;
		else
		    shape = GAIA_MULTIPOINT;
	    }
	  if (strcasecmp (metatype, "LINESTRING") == 0
	      || strcasecmp (metatype, "MULTILINESTRING") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_LINESTRINGZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_LINESTRINGM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_LINESTRINGZM;
		else
		    shape = GAIA_LINESTRING;
	    }
	  if (strcasecmp (metatype, "POLYGON") == 0
	      || strcasecmp (metatype, "MULTIPOLYGON") == 0)
	    {
		if (strcasecmp (metadims, "XYZ") == 0
		    || strcmp (metadims, "3") == 0)
		    shape = GAIA_POLYGONZ;
		else if (strcasecmp (metadims, "XYM") == 0)
		    shape = GAIA_POLYGONM;
		else if (strcasecmp (metadims, "XYZM") == 0)
		    shape = GAIA_POLYGONZM;
		else
		    shape = GAIA_POLYGON;
	    }
      }
    if (shape < 0)
      {
	  if (!err_msg)
	      fprintf (stderr,
		       "Unable to detect GeometryType for \"%s\".\"%s\" ... sorry\n",
		       table, column);
	  else
	      sprintf (err_msg,
		       "Unable to detect GeometryType for \"%s\".\"%s\" ... sorry\n",
		       table, column);
	  return 0;
      }
    if (verbose)
	fprintf (stderr,
		 "========\nDumping SQLite table '%s' into shapefile at '%s'\n",
		 table, shp_path);
    /* preparing SQL statement */
    sprintf (sql, "SELECT * FROM \"%s\" WHERE GeometryAliasType(\"%s\") = ",
	     table, column);
    if (shape == GAIA_LINESTRING || shape == GAIA_LINESTRINGZ
	|| shape == GAIA_LINESTRINGM || shape == GAIA_LINESTRINGZM)
      {
	  strcat (sql, "'LINESTRING' OR GeometryAliasType(\"");
	  strcat (sql, (char *) column);
	  strcat (sql, "\") = 'MULTILINESTRING'");
      }
    else if (shape == GAIA_POLYGON || shape == GAIA_POLYGONZ
	     || shape == GAIA_POLYGONM || shape == GAIA_POLYGONZM)
      {
	  strcat (sql, "'POLYGON' OR GeometryAliasType(\"");
	  strcat (sql, (char *) column);
	  strcat (sql, "\") = 'MULTIPOLYGON'");
      }
    else if (shape == GAIA_MULTIPOINT || shape == GAIA_MULTIPOINTZ
	     || shape == GAIA_MULTIPOINTM || shape == GAIA_MULTIPOINTZM)
      {
	  strcat (sql, "'POINT' OR GeometryAliasType(\"");
	  strcat (sql, (char *) column);
	  strcat (sql, "\") = 'MULTIPOINT'");
      }
    else
	strcat (sql, "'POINT'");
/* fetching anyway NULL Geometries */
    strcat (sql, " OR \"");
    strcat (sql, (char *) column);
    strcat (sql, "\" IS NULL");
/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	goto sql_error;
    while (1)
      {
	  /* Pass I - scrolling the result set to compute real DBF attributes' sizes and types */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* processing a result set row */
		row1++;
		if (n_cols == 0)
		  {
		      /* this one is the first row, so we are going to prepare the DBF Fields list */
		      n_cols = sqlite3_column_count (stmt);
		      dbf_export_list = gaiaAllocDbfList ();
		      max_length = malloc (sizeof (int) * n_cols);
		      sql_type = malloc (sizeof (int) * n_cols);
		      for (i = 0; i < n_cols; i++)
			{
			    /* initializes the DBF export fields */
			    strcpy (dummy, sqlite3_column_name (stmt, i));
			    gaiaAddDbfField (dbf_export_list, dummy, '\0', 0, 0,
					     0);
			    max_length[i] = 0;
			    sql_type[i] = SQLITE_NULL;
			}
		  }
		for (i = 0; i < n_cols; i++)
		  {
		      /* update the DBF export fields analyzing fetched data */
		      type = sqlite3_column_type (stmt, i);
		      if (type == SQLITE_NULL || type == SQLITE_BLOB)
			  continue;
		      if (type == SQLITE_TEXT)
			{
			    len = sqlite3_column_bytes (stmt, i);
			    sql_type[i] = SQLITE_TEXT;
			    if (len > max_length[i])
				max_length[i] = len;
			}
		      else if (type == SQLITE_FLOAT
			       && sql_type[i] != SQLITE_TEXT)
			  sql_type[i] = SQLITE_FLOAT;	/* promoting a numeric column to be DOUBLE */
		      else if (type == SQLITE_INTEGER &&
			       (sql_type[i] == SQLITE_NULL
				|| sql_type[i] == SQLITE_INTEGER))
			  sql_type[i] = SQLITE_INTEGER;	/* promoting a null column to be INTEGER */
		      if (type == SQLITE_INTEGER && max_length[i] < 18)
			  max_length[i] = 18;
		      if (type == SQLITE_FLOAT && max_length[i] < 24)
			  max_length[i] = 24;
		  }
	    }
	  else
	      goto sql_error;
      }
    if (!row1)
	goto empty_result_set;
    i = 0;
    offset = 0;
    dbf_list = gaiaAllocDbfList ();
    dbf_field = dbf_export_list->First;
    while (dbf_field)
      {
	  /* preparing the final DBF attribute list */
	  if (sql_type[i] == SQLITE_NULL)
	    {
		i++;
		dbf_field = dbf_field->Next;
		continue;
	    }
	  if (sql_type[i] == SQLITE_TEXT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'C', offset,
				 max_length[i], 0);
		offset += max_length[i];
	    }
	  if (sql_type[i] == SQLITE_FLOAT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 19, 6);
		offset += 19;
	    }
	  if (sql_type[i] == SQLITE_INTEGER)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 18, 0);
		offset += 18;
	    }
	  i++;
	  dbf_field = dbf_field->Next;
      }
    free (max_length);
    free (sql_type);
    gaiaFreeDbfList (dbf_export_list);
/* resetting SQLite query */
    if (verbose)
	fprintf (stderr, "\n%s;\n", sql);
    ret = sqlite3_reset (stmt);
    if (ret != SQLITE_OK)
	goto sql_error;
/* trying to open shapefile files */
    shp = gaiaAllocShapefile ();
    gaiaOpenShpWrite (shp, shp_path, shape, dbf_list, "UTF-8", charset);
    if (!(shp->Valid))
	goto no_file;
/* trying to export the .PRJ file */
    output_prj_file (sqlite, shp_path, table, column);
    while (1)
      {
	  /* Pass II - scrolling the result set to dump data into shapefile */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		rows++;
		dbf_write = gaiaCloneDbfEntity (dbf_list);
		for (i = 0; i < n_cols; i++)
		  {
		      if (strcasecmp
			  ((char *) column,
			   (char *) sqlite3_column_name (stmt, i)) == 0)
			{
			    /* this one is the internal BLOB encoded GEOMETRY to be exported */
			    if (sqlite3_column_type (stmt, i) != SQLITE_BLOB)
			      {
				  /* this one is a NULL Geometry */
				  dbf_write->Geometry = NULL;
			      }
			    else
			      {
				  blob_value = sqlite3_column_blob (stmt, i);
				  len = sqlite3_column_bytes (stmt, i);
				  dbf_write->Geometry =
				      gaiaFromSpatiaLiteBlobWkb (blob_value,
								 len);
			      }
			}
		      strcpy (dummy, sqlite3_column_name (stmt, i));
		      dbf_field = getDbfField (dbf_write, dummy);
		      if (!dbf_field)
			  continue;
		      if (sqlite3_column_type (stmt, i) == SQLITE_NULL)
			{
			    /* handling NULL values */
			    gaiaSetNullValue (dbf_field);
			}
		      else
			{
			    switch (dbf_field->Type)
			      {
			      case 'N':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_INTEGER)
				      gaiaSetIntValue (dbf_field,
						       sqlite3_column_int64
						       (stmt, i));
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				      gaiaSetDoubleValue (dbf_field,
							  sqlite3_column_double
							  (stmt, i));
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      case 'C':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_TEXT)
				    {
					strcpy (dummy,
						(char *)
						sqlite3_column_text (stmt, i));
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_INTEGER)
				    {
#if defined(_WIN32) || defined(__MINGW32__)
					/* CAVEAT - M$ runtime doesn't supports %lld for 64 bits */
					sprintf (dummy, "%I64d",
						 sqlite3_column_int64 (stmt,
								       i));
#else
					sprintf (dummy, "%lld",
						 sqlite3_column_int64 (stmt,
								       i));
#endif
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				    {
					sprintf (dummy, "%1.6f",
						 sqlite3_column_double (stmt,
									i));
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      };
			}
		  }
		if (!gaiaWriteShpEntity (shp, dbf_write))
		    fprintf (stderr, "shapefile write error\n");
		gaiaFreeDbfList (dbf_write);
	    }
	  else
	      goto sql_error;
      }
    sqlite3_finalize (stmt);
    gaiaFlushShpHeaders (shp);
    gaiaFreeShapefile (shp);
    if (verbose)
	fprintf (stderr, "\nExported %d rows into SHAPEFILE\n========\n", rows);
    if (xrows)
	*xrows = rows;
    if (err_msg)
	sprintf (err_msg, "Exported %d rows into SHAPEFILE", rows);
    return 1;
  sql_error:
/* some SQL error occurred */
    sqlite3_finalize (stmt);
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (shp)
	gaiaFreeShapefile (shp);
    if (!err_msg)
	fprintf (stderr, "SELECT failed: %s", sqlite3_errmsg (sqlite));
    else
	sprintf (err_msg, "SELECT failed: %s", sqlite3_errmsg (sqlite));
    return 0;
  no_file:
/* shapefile can't be created/opened */
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (shp)
	gaiaFreeShapefile (shp);
    if (!err_msg)
	fprintf (stderr, "ERROR: unable to open '%s' for writing", shp_path);
    else
	sprintf (err_msg, "ERROR: unable to open '%s' for writing", shp_path);
    return 0;
  empty_result_set:
/* the result set is empty - nothing to do */
    sqlite3_finalize (stmt);
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (shp)
	gaiaFreeShapefile (shp);
    if (!err_msg)
	fprintf (stderr,
		 "The SQL SELECT returned an empty result set ... there is nothing to export ...");
    else
	sprintf (err_msg,
		 "The SQL SELECT returned an empty result set ... there is nothing to export ...");
    return 0;
}

SPATIALITE_DECLARE int
load_dbf (sqlite3 * sqlite, char *dbf_path, char *table, char *charset,
	  int verbose, int *rows, char *err_msg)
{
    sqlite3_stmt *stmt;
    int ret;
    char *errMsg = NULL;
    char sql[65536];
    char dummyName[4096];
    int already_exists = 0;
    int sqlError = 0;
    gaiaDbfPtr dbf = NULL;
    gaiaDbfFieldPtr dbf_field;
    int cnt;
    int col_cnt;
    int seed;
    int len;
    int dup;
    int idup;
    int current_row;
    char **col_name = NULL;
    int deleted;
/* checking if TABLE already exists */
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE '%s'",
	     table);
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load DBF error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  else
	      sprintf (err_msg, "load DBF error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  return 0;
      }
    while (1)
      {
	  /* scrolling the result set */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	      already_exists = 1;
	  else
	    {
		fprintf (stderr, "load DBF error: <%s>\n",
			 sqlite3_errmsg (sqlite));
		break;
	    }
      }
    sqlite3_finalize (stmt);
    if (already_exists)
      {
	  if (!err_msg)
	      fprintf (stderr, "load DBF error: table '%s' already exists\n",
		       table);
	  else
	      sprintf (err_msg, "load DBF error: table '%s' already exists\n",
		       table);
	  return 0;
      }
    dbf = gaiaAllocDbf ();
    gaiaOpenDbfRead (dbf, dbf_path, charset, "UTF-8");
    if (!(dbf->Valid))
      {
	  if (!err_msg)
	    {
		fprintf (stderr, "load DBF error: cannot open '%s'\n",
			 dbf_path);
		if (dbf->LastError)
		    fprintf (stderr, "\tcause: %s\n", dbf->LastError);
	    }
	  else
	    {
		char extra[512];
		*extra = '\0';
		if (dbf->LastError)
		    sprintf (extra, "\n\tcause: %s", dbf->LastError);
		sprintf (err_msg, "load DBF error: cannot open '%s'%s",
			 dbf_path, extra);
	    }
	  gaiaFreeDbf (dbf);
	  return 0;
      }
/* checking for duplicate / illegal column names and antialising them */
    col_cnt = 0;
    dbf_field = dbf->Dbf->First;
    while (dbf_field)
      {
	  /* counting DBF fields */
	  col_cnt++;
	  dbf_field = dbf_field->Next;
      }
    col_name = malloc (sizeof (char *) * col_cnt);
    cnt = 0;
    seed = 0;
    dbf_field = dbf->Dbf->First;
    while (dbf_field)
      {
	  /* preparing column names */
	  strcpy (dummyName, dbf_field->Name);
	  dup = 0;
	  for (idup = 0; idup < cnt; idup++)
	    {
		if (strcasecmp (dummyName, *(col_name + idup)) == 0)
		    dup = 1;
	    }
	  if (strcasecmp (dummyName, "PK_UID") == 0)
	      dup = 1;
	  if (dup)
	      sprintf (dummyName, "COL_%d", seed++);
	  len = strlen (dummyName);
	  *(col_name + cnt) = malloc (len + 1);
	  strcpy (*(col_name + cnt), dummyName);
	  cnt++;
	  dbf_field = dbf_field->Next;
      }
    if (verbose)
	fprintf (stderr,
		 "========\nLoading DBF at '%s' into SQLite table '%s'\n",
		 dbf_path, table);
/* starting a transaction */
    if (verbose)
	fprintf (stderr, "\nBEGIN;\n");
    ret = sqlite3_exec (sqlite, "BEGIN", NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load DBF error: <%s>\n", errMsg);
	  else
	      sprintf (err_msg, "load DBF error: <%s>\n", errMsg);
	  sqlite3_free (errMsg);
	  sqlError = 1;
	  goto clean_up;
      }
/* creating the Table */
    sprintf (sql, "CREATE TABLE %s", table);
    strcat (sql, " (\nPK_UID INTEGER PRIMARY KEY AUTOINCREMENT");
    cnt = 0;
    dbf_field = dbf->Dbf->First;
    while (dbf_field)
      {
	  strcat (sql, ",\n\"");
	  strcat (sql, *(col_name + cnt));
	  cnt++;
	  switch (dbf_field->Type)
	    {
	    case 'C':
		strcat (sql, "\" TEXT");
		break;
	    case 'N':
		fflush (stderr);
		if (dbf_field->Decimals)
		    strcat (sql, "\" DOUBLE");
		else
		  {
		      if (dbf_field->Length <= 18)
			  strcat (sql, "\" INTEGER");
		      else
			  strcat (sql, "\" DOUBLE");
		  }
		break;
	    case 'D':
		strcat (sql, "\" DOUBLE");
		break;
	    case 'F':
		strcat (sql, "\" DOUBLE");
		break;
	    case 'L':
		strcat (sql, "\" INTEGER");
		break;
	    };
	  dbf_field = dbf_field->Next;
      }
    strcat (sql, ")");
    if (verbose)
	fprintf (stderr, "%s;\n", sql);
    ret = sqlite3_exec (sqlite, sql, NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load DBF error: <%s>\n", errMsg);
	  else
	      sprintf (err_msg, "load DBF error: <%s>\n", errMsg);
	  sqlite3_free (errMsg);
	  sqlError = 1;
	  goto clean_up;
      }
    /* preparing the INSERT INTO parametrerized statement */
    sprintf (sql, "INSERT INTO %s (PK_UID", table);
    cnt = 0;
    dbf_field = dbf->Dbf->First;
    while (dbf_field)
      {
	  /* columns corresponding to some DBF attribute */
	  strcat (sql, ",\"");
	  strcat (sql, *(col_name + cnt++));
	  strcat (sql, "\"");
	  dbf_field = dbf_field->Next;
      }
    strcat (sql, ")\nVALUES (?");
    dbf_field = dbf->Dbf->First;
    while (dbf_field)
      {
	  /* column values */
	  strcat (sql, ", ?");
	  dbf_field = dbf_field->Next;
      }
    strcat (sql, ")");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load DBF error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  else
	      sprintf (err_msg, "load DBF error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  sqlError = 1;
	  goto clean_up;
      }
    current_row = 0;
    while (1)
      {
	  /* inserting rows from DBF */
	  ret = gaiaReadDbfEntity (dbf, current_row, &deleted);
	  if (!ret)
	    {
		if (!(dbf->LastError))	/* normal DBF EOF */
		    break;
		if (!err_msg)
		    fprintf (stderr, "%s\n", dbf->LastError);
		else
		    sprintf (err_msg, "%s\n", dbf->LastError);
		sqlError = 1;
		goto clean_up;
	    }
	  current_row++;
	  if (deleted)
	    {
		/* skipping DBF deleted row */
		continue;
	    }
	  /* binding query params */
	  sqlite3_reset (stmt);
	  sqlite3_clear_bindings (stmt);
	  sqlite3_bind_int (stmt, 1, current_row);
	  cnt = 0;
	  dbf_field = dbf->Dbf->First;
	  while (dbf_field)
	    {
		/* column values */
		if (!(dbf_field->Value))
		    sqlite3_bind_null (stmt, cnt + 2);
		else
		  {
		      switch (dbf_field->Value->Type)
			{
			case GAIA_INT_VALUE:
			    sqlite3_bind_int64 (stmt, cnt + 2,
						dbf_field->Value->IntValue);
			    break;
			case GAIA_DOUBLE_VALUE:
			    sqlite3_bind_double (stmt, cnt + 2,
						 dbf_field->Value->DblValue);
			    break;
			case GAIA_TEXT_VALUE:
			    sqlite3_bind_text (stmt, cnt + 2,
					       dbf_field->Value->TxtValue,
					       strlen (dbf_field->
						       Value->TxtValue),
					       SQLITE_STATIC);
			    break;
			default:
			    sqlite3_bind_null (stmt, cnt + 2);
			    break;
			}
		  }
		cnt++;
		dbf_field = dbf_field->Next;
	    }
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	      ;
	  else
	    {
		if (!err_msg)
		    fprintf (stderr, "load DBF error: <%s>\n",
			     sqlite3_errmsg (sqlite));
		else
		    sprintf (err_msg, "load DBF error: <%s>\n",
			     sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		sqlError = 1;
		goto clean_up;
	    }
      }
    sqlite3_finalize (stmt);
  clean_up:
    gaiaFreeDbf (dbf);
    if (col_name)
      {
	  /* releasing memory allocation for column names */
	  for (cnt = 0; cnt < col_cnt; cnt++)
	      free (*(col_name + cnt));
	  free (col_name);
      }
    if (sqlError)
      {
	  /* some error occurred - ROLLBACK */
	  if (verbose)
	      fprintf (stderr, "ROLLBACK;\n");
	  ret = sqlite3_exec (sqlite, "ROLLBACK", NULL, 0, &errMsg);
	  if (ret != SQLITE_OK)
	    {
		fprintf (stderr, "load DBF error: <%s>\n", errMsg);
		sqlite3_free (errMsg);
	    };
	  if (rows)
	      *rows = current_row;
	  return 0;
      }
    else
      {
	  /* ok - confirming pending transaction - COMMIT */
	  if (verbose)
	      fprintf (stderr, "COMMIT;\n");
	  ret = sqlite3_exec (sqlite, "COMMIT", NULL, 0, &errMsg);
	  if (ret != SQLITE_OK)
	    {
		fprintf (stderr, "load DBF error: <%s>\n", errMsg);
		sqlite3_free (errMsg);
		return 0;
	    }
	  if (rows)
	      *rows = current_row;
	  if (verbose)
	      fprintf (stderr,
		       "\nInserted %d rows into '%s' from DBF\n========\n",
		       current_row, table);
	  if (err_msg)
	      sprintf (err_msg, "Inserted %d rows into '%s' from DBF",
		       current_row, table);
	  return 1;
      }
}

SPATIALITE_DECLARE int
dump_dbf (sqlite3 * sqlite, char *table, char *dbf_path, char *charset,
	  char *err_msg)
{
/* DBF dump */
    int rows;
    int i;
    char sql[4096];
    char xtable[4096];
    sqlite3_stmt *stmt;
    int row1 = 0;
    int n_cols = 0;
    int offset = 0;
    int type;
    gaiaDbfPtr dbf = NULL;
    gaiaDbfListPtr dbf_export_list = NULL;
    gaiaDbfListPtr dbf_list = NULL;
    gaiaDbfListPtr dbf_write;
    gaiaDbfFieldPtr dbf_field;
    int *max_length = NULL;
    int *sql_type = NULL;
    char dummy[1024];
    int len;
    int ret;
/*
/ preparing SQL statement 
*/
    strcpy (xtable, table);
    shp_double_quoted_sql (xtable);
    sprintf (sql, "SELECT * FROM %s", xtable);
/*
/ compiling SQL prepared statement 
*/
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	goto sql_error;
    rows = 0;
    while (1)
      {
	  /*
	     / Pass I - scrolling the result set to compute real DBF attributes' sizes and types 
	   */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* processing a result set row */
		row1++;
		if (n_cols == 0)
		  {
		      /* this one is the first row, so we are going to prepare the DBF Fields list */
		      n_cols = sqlite3_column_count (stmt);
		      dbf_export_list = gaiaAllocDbfList ();
		      max_length = (int *) malloc (sizeof (int) * n_cols);
		      sql_type = (int *) malloc (sizeof (int) * n_cols);
		      for (i = 0; i < n_cols; i++)
			{
			    /* initializes the DBF export fields */
			    strcpy (dummy, sqlite3_column_name (stmt, i));
			    gaiaAddDbfField (dbf_export_list, dummy, '\0', 0, 0,
					     0);
			    max_length[i] = 0;
			    sql_type[i] = SQLITE_NULL;
			}
		  }
		for (i = 0; i < n_cols; i++)
		  {
		      /* update the DBF export fields analyzing fetched data */
		      type = sqlite3_column_type (stmt, i);
		      if (type == SQLITE_NULL || type == SQLITE_BLOB)
			  continue;
		      if (type == SQLITE_TEXT)
			{
			    len = sqlite3_column_bytes (stmt, i);
			    sql_type[i] = SQLITE_TEXT;
			    if (len > max_length[i])
				max_length[i] = len;
			}
		      else if (type == SQLITE_FLOAT
			       && sql_type[i] != SQLITE_TEXT)
			  sql_type[i] = SQLITE_FLOAT;	/* promoting a numeric column to be DOUBLE */
		      else if (type == SQLITE_INTEGER
			       && (sql_type[i] == SQLITE_NULL
				   || sql_type[i] == SQLITE_INTEGER))
			  sql_type[i] = SQLITE_INTEGER;	/* promoting a null column to be INTEGER */
		      if (type == SQLITE_INTEGER && max_length[i] < 18)
			  max_length[i] = 18;
		      if (type == SQLITE_FLOAT && max_length[i] < 24)
			  max_length[i] = 24;
		  }
	    }
	  else
	      goto sql_error;
      }
    if (!row1)
	goto empty_result_set;
    i = 0;
    offset = 0;
    dbf_list = gaiaAllocDbfList ();
    dbf_field = dbf_export_list->First;
    while (dbf_field)
      {
	  /* preparing the final DBF attribute list */
	  if (sql_type[i] == SQLITE_NULL || sql_type[i] == SQLITE_BLOB)
	    {
		i++;
		dbf_field = dbf_field->Next;
		continue;
	    }
	  if (sql_type[i] == SQLITE_TEXT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'C', offset,
				 max_length[i], 0);
		offset += max_length[i];
	    }
	  if (sql_type[i] == SQLITE_FLOAT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 19, 6);
		offset += 19;
	    }
	  if (sql_type[i] == SQLITE_INTEGER)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 18, 0);
		offset += 18;
	    }
	  i++;
	  dbf_field = dbf_field->Next;
      }
    free (max_length);
    free (sql_type);
    gaiaFreeDbfList (dbf_export_list);
    dbf_export_list = NULL;
/* resetting SQLite query */
    ret = sqlite3_reset (stmt);
    if (ret != SQLITE_OK)
	goto sql_error;
/* trying to open the DBF file */
    dbf = gaiaAllocDbf ();
/* xfering export-list ownership */
    dbf->Dbf = dbf_list;
    dbf_list = NULL;
    gaiaOpenDbfWrite (dbf, dbf_path, "UTF-8", charset);
    if (!(dbf->Valid))
	goto no_file;
    while (1)
      {
	  /* Pass II - scrolling the result set to dump data into DBF */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		rows++;
		dbf_write = gaiaCloneDbfEntity (dbf->Dbf);
		for (i = 0; i < n_cols; i++)
		  {
		      strcpy (dummy, sqlite3_column_name (stmt, i));
		      dbf_field = getDbfField (dbf_write, dummy);
		      if (!dbf_field)
			  continue;
		      if (sqlite3_column_type (stmt, i) == SQLITE_NULL
			  || sqlite3_column_type (stmt, i) == SQLITE_BLOB)
			{
			    /* handling NULL values */
			    gaiaSetNullValue (dbf_field);
			}
		      else
			{
			    switch (dbf_field->Type)
			      {
			      case 'N':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_INTEGER)
				      gaiaSetIntValue (dbf_field,
						       sqlite3_column_int64
						       (stmt, i));
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				      gaiaSetDoubleValue (dbf_field,
							  sqlite3_column_double
							  (stmt, i));
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      case 'C':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_TEXT)
				    {
					strcpy (dummy,
						(char *)
						sqlite3_column_text (stmt, i));
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_INTEGER)
				    {
#if defined(_WIN32) || defined(__MINGW32__)
					/* CAVEAT - M$ runtime doesn't supports %lld for 64 bits */
					sprintf (dummy, "%I64d",
						 sqlite3_column_int64 (stmt,
								       i));
#else
					sprintf (dummy, "%lld",
						 sqlite3_column_int64 (stmt,
								       i));
#endif
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				    {
					sprintf (dummy, "%1.6f",
						 sqlite3_column_double (stmt,
									i));
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      };
			}
		  }
		if (!gaiaWriteDbfEntity (dbf, dbf_write))
		    fprintf (stderr, "DBF write error\n");
		gaiaFreeDbfList (dbf_write);
	    }
	  else
	      goto sql_error;
      }
    sqlite3_finalize (stmt);
    gaiaFlushDbfHeader (dbf);
    gaiaFreeDbf (dbf);
    if (!err_msg)
	fprintf (stderr, "Exported %d rows into the DBF file\n", rows);
    else
	sprintf (err_msg, "Exported %d rows into the DBF file\n", rows);
    return 1;
  sql_error:
/* some SQL error occurred */
    sqlite3_finalize (stmt);
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (dbf)
	gaiaFreeDbf (dbf);
    if (!err_msg)
	fprintf (stderr, "dump DBF file error: %s\n", sqlite3_errmsg (sqlite));
    else
	sprintf (err_msg, "dump DBF file error: %s\n", sqlite3_errmsg (sqlite));
    return 0;
  no_file:
/* DBF can't be created/opened */
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (dbf)
	gaiaFreeDbf (dbf);
    if (!err_msg)
	fprintf (stderr, "ERROR: unable to open '%s' for writing\n", dbf_path);
    else
	sprintf (err_msg, "ERROR: unable to open '%s' for writing\n", dbf_path);
    return 0;
  empty_result_set:
/* the result set is empty - nothing to do */
    sqlite3_finalize (stmt);
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (dbf)
	gaiaFreeDbf (dbf);
    if (!err_msg)
	fprintf (stderr,
		 "The SQL SELECT returned an empty result set ... there is nothing to export ...\n");
    else
	sprintf (err_msg,
		 "The SQL SELECT returned an empty result set ... there is nothing to export ...\n");
    return 0;
}

#endif /* end ICONV (SHP and DBF) */

SPATIALITE_DECLARE int
is_kml_constant (sqlite3 * sqlite, char *table, char *column)
{
/* checking a possible column name for KML dump */
    char sql[1024];
    int ret;
    int k = 1;
    const char *name;
    char **results;
    int rows;
    int columns;
    int i;
    char *errMsg = NULL;

    sprintf (sql, "PRAGMA table_info(%s)", table);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
	return 1;
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		if (strcasecmp (name, column) == 0)
		    k = 0;
	    }
      }
    sqlite3_free_table (results);
    return k;
}

SPATIALITE_DECLARE int
dump_kml (sqlite3 * sqlite, char *table, char *geom_col, char *kml_path,
	  char *name_col, char *desc_col, int precision)
{
/* dumping a  geometry table as KML */
    char sql[4096];
    char xname[1024];
    char xdesc[1024];
    sqlite3_stmt *stmt = NULL;
    FILE *out = NULL;
    int ret;
    int rows = 0;
    int is_const = 1;

/* opening/creating the KML file */
    out = fopen (kml_path, "wb");
    if (!out)
	goto no_file;

/* preparing SQL statement */
    if (name_col == NULL)
	strcpy (xname, "'name'");
    else
      {
	  is_const = is_kml_constant (sqlite, table, name_col);
	  if (is_const)
	      sprintf (xname, "'%s'", name_col);
	  else
	      strcpy (xname, name_col);
      }
    if (desc_col == NULL)
	strcpy (xdesc, "'description'");
    else
      {
	  is_const = is_kml_constant (sqlite, table, desc_col);
	  if (is_const)
	      sprintf (xdesc, "'%s'", desc_col);
	  else
	      strcpy (xdesc, desc_col);
      }
    sprintf (sql, "SELECT AsKML(%s, %s, %s, %d) FROM %s ", xname, xdesc,
	     geom_col, precision, table);
/* excluding NULL Geometries */
    strcat (sql, "WHERE ");
    strcat (sql, geom_col);
    strcat (sql, " IS NOT NULL");
/* compiling SQL prepared statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	goto sql_error;

    while (1)
      {
	  /* scrolling the result set */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* processing a result set row */
		if (rows == 0)
		  {
		      fprintf (out,
			       "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n");
		      fprintf (out,
			       "<kml xmlns=\"http://www.opengis.net/kml/2.2\">\r\n");
		      fprintf (out, "<Document>\r\n");
		  }
		rows++;
		fprintf (out, "\t%s\r\n", sqlite3_column_text (stmt, 0));
	    }
	  else
	      goto sql_error;
      }
    if (!rows)
	goto empty_result_set;


    fprintf (out, "</Document>\r\n");
    fprintf (out, "</kml>\r\n");
    sqlite3_finalize (stmt);
    fclose (out);
    return 1;

  sql_error:
/* some SQL error occurred */
    if (stmt)
	sqlite3_finalize (stmt);
    if (out)
	fclose (out);
    fprintf (stderr, "Dump KML error: %s\n", sqlite3_errmsg (sqlite));
    return 0;
  no_file:
/* KML file can't be created/opened */
    if (stmt)
	sqlite3_finalize (stmt);
    if (out)
	fclose (out);
    fprintf (stderr, "ERROR: unable to open '%s' for writing\n", kml_path);
    return 0;
  empty_result_set:
/* the result set is empty - nothing to do */
    if (stmt)
	sqlite3_finalize (stmt);
    if (out)
	fclose (out);
    fprintf (stderr,
	     "The SQL SELECT returned an empty result set\n... there is nothing to export ...\n");
    return 0;
}

static int
is_table (sqlite3 * sqlite, const char *table)
{
/* check if this one really is a TABLE */
    char sql[8192];
    int ret;
    char **results;
    int rows;
    int columns;
    char *errMsg = NULL;
    int ok = 0;

    strcpy (sql, "SELECT tbl_name FROM sqlite_master ");
    strcat (sql, "WHERE type = 'table' AND tbl_name LIKE '");
    strcat (sql, table);
    strcat (sql, "'");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQLite SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  return ok;
      }
    if (rows < 1)
	;
    else
	ok = 1;
    sqlite3_free_table (results);
    return ok;
}

SPATIALITE_DECLARE void
check_duplicated_rows (sqlite3 * sqlite, char *table, int *dupl_count)
{
/* Checking a Table for Duplicate rows */
    char sql[8192];
    char col_list[4196];
    int first = 1;
    char xname[1024];
    int pk;
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    char *errMsg = NULL;
    sqlite3_stmt *stmt = NULL;

    *dupl_count = 0;

    if (is_table (sqlite, table) == 0)
      {
	  fprintf (stderr, ".chkdupl %s: no such table\n", table);
	  return;
      }
/* extracting the column names (excluding any Primary Key) */
    sprintf (sql, "PRAGMA table_info(%s)", table);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQLite SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    if (rows < 1)
	;
    else
      {
	  *col_list = '\0';
	  for (i = 1; i <= rows; i++)
	    {
		strcpy (xname, results[(i * columns) + 1]);
		pk = atoi (results[(i * columns) + 5]);
		if (!pk)
		  {
		      if (first)
			  first = 0;
		      else
			  strcat (col_list, ", ");
		      shp_double_quoted_sql (xname);
		      strcat (col_list, xname);
		  }
	    }
      }
    sqlite3_free_table (results);
    /* preparing the SQL statement */
    strcpy (sql, "SELECT Count(*) AS \"[dupl-count]\", ");
    strcat (sql, col_list);
    strcat (sql, "\nFROM ");
    strcat (sql, table);
    strcat (sql, "\nGROUP BY ");
    strcat (sql, col_list);
    strcat (sql, "\nHAVING \"[dupl-count]\" > 1");
    strcat (sql, "\nORDER BY \"[dupl-count]\" DESC");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
	  return;
      }
    while (1)
      {
	  /* fetching the result set rows */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* fetching a row */
		*dupl_count += sqlite3_column_int (stmt, 0) - 1;
	    }
	  else
	    {
		fprintf (stderr, "SQL error: %s", sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		return;
	    }
      }
    sqlite3_finalize (stmt);
    if (*dupl_count)
	fprintf (stderr, "%d duplicated rows found !!!\n", *dupl_count);
    else
	fprintf (stderr, "No duplicated rows have been identified\n");
}

static int
do_delete_duplicates2 (sqlite3 * sqlite, sqlite3_stmt * stmt1,
		       struct dupl_row *value_list, int *count)
{
/* deleting duplicate rows [actual delete] */
    int cnt = 0;
    int row_no = 0;
    char sql[8192];
    char where[4196];
    char condition[1024];
    int ret;
    sqlite3_stmt *stmt2 = NULL;
    struct dupl_column *col;
    int first = 1;
    int qcnt = 0;
    int param = 1;
    int match;
    int n_cols;
    int col_no;

    *count = 0;
    reset_query_pos (value_list);

/* preparing the query statement */
    strcpy (sql, "SELECT ROWID");
    strcpy (where, "\nWHERE ");
    col = value_list->first;
    while (col)
      {
	  if (col->type == SQLITE_BLOB)
	    {
		strcat (sql, ", ");
		strcat (sql, col->name);
		col->query_pos = qcnt++;
	    }
	  else if (col->type == SQLITE_NULL)
	    {
		if (first)
		  {
		      first = 0;
		      strcpy (condition, col->name);
		  }
		else
		  {
		      strcpy (condition, " AND ");
		      strcat (condition, col->name);
		  }
		strcat (condition, " IS NULL");
		strcat (where, condition);
	    }
	  else
	    {
		if (first)
		  {
		      first = 0;
		      strcpy (condition, col->name);
		  }
		else
		  {
		      strcpy (condition, " AND ");
		      strcat (condition, col->name);
		  }
		strcat (condition, " = ?");
		strcat (where, condition);
		col->query_pos = param++;
	    }
	  col = col->next;
      }
    strcat (sql, "\nFROM ");
    strcat (sql, value_list->table);
    strcat (sql, where);

    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt2, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
	  goto error;
      }

    sqlite3_reset (stmt2);
    sqlite3_clear_bindings (stmt2);
    col = value_list->first;
    while (col)
      {
	  /* binding query params */
	  if (col->type == SQLITE_INTEGER)
	      sqlite3_bind_int64 (stmt2, col->query_pos, col->int_value);
	  if (col->type == SQLITE_FLOAT)
	      sqlite3_bind_double (stmt2, col->query_pos, col->dbl_value);
	  if (col->type == SQLITE_TEXT)
	      sqlite3_bind_text (stmt2, col->query_pos, col->txt_value,
				 strlen (col->txt_value), SQLITE_STATIC);
	  col = col->next;
      }

    while (1)
      {
	  /* fetching the result set rows */
	  ret = sqlite3_step (stmt2);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* fetching a row */
		match = 1;
		n_cols = sqlite3_column_count (stmt2);
		for (col_no = 1; col_no < n_cols; col_no++)
		  {
		      /* checking blob columns */
		      if (sqlite3_column_type (stmt2, col_no) == SQLITE_BLOB)
			{
			    const void *blob =
				sqlite3_column_blob (stmt2, col_no);
			    int blob_size =
				sqlite3_column_bytes (stmt2, col_no);
			    if (check_dupl_blob
				(value_list, col_no - 1, blob, blob_size) == 0)
				match = 0;
			}
		      else
			  match = 0;
		      if (match == 0)
			  break;
		  }
		if (match == 0)
		    continue;
		row_no++;
		if (row_no > 1)
		  {
		      /* deleting any duplicated row except the first one */
		      sqlite3_reset (stmt1);
		      sqlite3_clear_bindings (stmt1);
		      sqlite3_bind_int64 (stmt1, 1,
					  sqlite3_column_int64 (stmt2, 0));
		      ret = sqlite3_step (stmt1);
		      if (ret == SQLITE_DONE || ret == SQLITE_ROW)
			  cnt++;
		      else
			{
			    fprintf (stderr, "SQL error: %s\n",
				     sqlite3_errmsg (sqlite));
			    goto error;
			}
		  }
	    }
	  else
	    {
		fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
		goto error;
	    }
      }
    if (stmt2)
	sqlite3_finalize (stmt2);
    *count = cnt;
    return 1;

  error:
    if (stmt2)
	sqlite3_finalize (stmt2);
    *count = 0;

    return 0;
}

static int
do_delete_duplicates (sqlite3 * sqlite, const char *sql1, const char *sql2,
		      struct dupl_row *value_list, int *count)
{
/* deleting duplicate rows */
    sqlite3_stmt *stmt1 = NULL;
    sqlite3_stmt *stmt2 = NULL;
    int ret;
    int xcnt;
    int cnt = 0;
    int n_cols;
    int col_no;
    char *sql_err = NULL;

    *count = 0;

/* the complete operation is handled as an unique SQL Transaction */
    ret = sqlite3_exec (sqlite, "BEGIN", NULL, NULL, &sql_err);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "BEGIN TRANSACTION error: %s\n", sql_err);
	  sqlite3_free (sql_err);
	  return 0;
      }
/* preparing the main SELECT statement */
    ret = sqlite3_prepare_v2 (sqlite, sql1, strlen (sql1), &stmt1, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
	  return 0;
      }
/* preparing the DELETE statement */
    ret = sqlite3_prepare_v2 (sqlite, sql2, strlen (sql2), &stmt2, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
	  goto error;
      }

    while (1)
      {
	  /* fetching the result set rows */
	  ret = sqlite3_step (stmt1);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* fetching a row */
		sqlite3_reset (stmt2);
		sqlite3_clear_bindings (stmt2);
		n_cols = sqlite3_column_count (stmt1);
		for (col_no = 1; col_no < n_cols; col_no++)
		  {
		      /* saving column values */
		      if (sqlite3_column_type (stmt1, col_no) == SQLITE_INTEGER)
			  set_int_value (value_list, col_no - 1,
					 sqlite3_column_int64 (stmt1, col_no));
		      if (sqlite3_column_type (stmt1, col_no) == SQLITE_FLOAT)
			  set_double_value (value_list, col_no - 1,
					    sqlite3_column_double (stmt1,
								   col_no));
		      if (sqlite3_column_type (stmt1, col_no) == SQLITE_TEXT)
			{
			    const char *xtext =
				(const char *) sqlite3_column_text (stmt1,
								    col_no);
			    set_text_value (value_list, col_no - 1, xtext);
			}
		      if (sqlite3_column_type (stmt1, col_no) == SQLITE_BLOB)
			{
			    const void *blob =
				sqlite3_column_blob (stmt1, col_no);
			    int blob_size =
				sqlite3_column_bytes (stmt1, col_no);
			    set_blob_value (value_list, col_no - 1, blob,
					    blob_size);
			}
		      if (sqlite3_column_type (stmt1, col_no) == SQLITE_NULL)
			  set_null_value (value_list, col_no - 1);
		  }
		if (do_delete_duplicates2 (sqlite, stmt2, value_list, &xcnt))
		    cnt += xcnt;
		else
		    goto error;
	    }
	  else
	    {
		fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
		goto error;
	    }
      }

    sqlite3_finalize (stmt1);
    sqlite3_finalize (stmt2);

/* confirm the still pending Transaction */
    ret = sqlite3_exec (sqlite, "COMMIT", NULL, NULL, &sql_err);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "COMMIT TRANSACTION error: %s\n", sql_err);
	  sqlite3_free (sql_err);
	  return 0;
      }

    *count = cnt;
    return 1;

  error:
    *count = 0;
    if (stmt1)
	sqlite3_finalize (stmt1);
    if (stmt2)
	sqlite3_finalize (stmt2);

/* performing a ROLLBACK anyway */
    ret = sqlite3_exec (sqlite, "ROLLBACK", NULL, NULL, &sql_err);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "ROLLBACK TRANSACTION error: %s\n", sql_err);
	  sqlite3_free (sql_err);
	  return 0;
      }

    return 0;
}

SPATIALITE_DECLARE void
remove_duplicated_rows (sqlite3 * sqlite, char *table)
{
/* attempting to delete Duplicate rows from a table */
    struct dupl_row value_list;
    char sql[8192];
    char sql2[1024];
    char col_list[4196];
    int first = 1;
    char xname[1024];
    int pk;
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    char *errMsg = NULL;
    int count;

    value_list.count = 0;
    value_list.first = NULL;
    value_list.last = NULL;
    value_list.table = table;

    if (is_table (sqlite, table) == 0)
      {
	  fprintf (stderr, ".remdupl %s: no such table\n", table);
	  return;
      }
/* extracting the column names (excluding any Primary Key) */
    sprintf (sql, "PRAGMA table_info(%s)", table);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQLite SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    if (rows < 1)
	;
    else
      {
	  *col_list = '\0';
	  for (i = 1; i <= rows; i++)
	    {
		strcpy (xname, results[(i * columns) + 1]);
		pk = atoi (results[(i * columns) + 5]);
		if (!pk)
		  {
		      if (first)
			  first = 0;
		      else
			  strcat (col_list, ", ");
		      shp_double_quoted_sql (xname);
		      strcat (col_list, xname);
		      add_to_dupl_row (&value_list, xname);
		  }
	    }
      }
    sqlite3_free_table (results);
/* preparing the SQL statement (identifying duplicated rows) */
    strcpy (sql, "SELECT Count(*) AS \"[dupl-count]\", ");
    strcat (sql, col_list);
    strcat (sql, "\nFROM ");
    strcat (sql, table);
    strcat (sql, "\nGROUP BY ");
    strcat (sql, col_list);
    strcat (sql, "\nHAVING \"[dupl-count]\" > 1");
/* preparing the SQL statement [delete] */
    strcpy (sql2, "DELETE FROM ");
    strcat (sql2, table);
    strcat (sql2, " WHERE ROWID = ?");

    if (do_delete_duplicates (sqlite, sql, sql2, &value_list, &count))
      {
	  if (!count)
	      fprintf (stderr, "No duplicated rows have been identified\n");
	  else
	      fprintf (stderr, "%d duplicated rows deleted from: %s\n", count,
		       table);
      }
    clean_dupl_row (&value_list);
}

static int
check_elementary (sqlite3 * sqlite, const char *inTable, const char *geom,
		  const char *outTable, const char *pKey, const char *multiID,
		  char *type, int *srid, char *coordDims)
{
/* preliminary check for ELEMENTARY GEOMETRIES */
    char sql[8192];
    int ret;
    char **results;
    int rows;
    int columns;
    char *errMsg = NULL;
    int ok = 0;
    int i;
    char *gtp;
    char *dims;
    char *quoted;

/* fetching metadata */
    strcpy (sql, "SELECT type, coord_dimension, srid ");
    strcat (sql, "FROM geometry_columns WHERE f_table_name LIKE '");
    quoted = gaiaSingleQuotedSql (inTable);
    if (quoted)
      {
	  strcat (sql, quoted);
	  free (quoted);
      }
    strcat (sql, "' AND f_geometry_column LIKE '");
    quoted = gaiaSingleQuotedSql (geom);
    if (quoted)
      {
	  strcat (sql, quoted);
	  free (quoted);
      }
    strcat (sql, "'");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  return 0;
      }
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		gtp = results[(i * columns) + 0];
		dims = results[(i * columns) + 1];
		*srid = atoi (results[(i * columns) + 2]);
		if (strcasecmp (gtp, "POINT") == 0
		    || strcasecmp (gtp, "MULTIPOINT") == 0)
		    strcpy (type, "POINT");
		else if (strcasecmp (gtp, "LINESTRING") == 0
			 || strcasecmp (gtp, "MULTILINESTRING") == 0)
		    strcpy (type, "LINESTRING");
		else if (strcasecmp (gtp, "POLYGON") == 0
			 || strcasecmp (gtp, "MULTIPOLYGON") == 0)
		    strcpy (type, "POLYGON");
		else
		    strcpy (type, "GEOMETRY");
		strcpy (coordDims, dims);
		ok = 1;
	    }
      }
    sqlite3_free_table (results);
    if (!ok)
	return 0;

/* checking if PrimaryKey already exists */
    strcpy (sql, "PRAGMA table_info(\"");
    quoted = gaiaDoubleQuotedSql (inTable);
    if (quoted)
      {
	  strcat (sql, quoted);
	  free (quoted);
      }
    strcat (sql, "\")");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  return 0;
      }
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		if (strcasecmp (pKey, results[(i * columns) + 1]) == 0)
		    ok = 0;
	    }
      }
    sqlite3_free_table (results);
    if (!ok)
	return 0;

/* checking if MultiID already exists */
    strcpy (sql, "PRAGMA table_info(\"");
    quoted = gaiaDoubleQuotedSql (inTable);
    if (quoted)
      {
	  strcat (sql, quoted);
	  free (quoted);
      }
    strcat (sql, "\")");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  return 0;
      }
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		if (strcasecmp (multiID, results[(i * columns) + 1]) == 0)
		    ok = 0;
	    }
      }
    sqlite3_free_table (results);
    if (!ok)
	return 0;

/* cheching if Output Table already exists */
    strcpy (sql, "SELECT Count(*) FROM sqlite_master WHERE type ");
    strcat (sql, "LIKE 'table' AND tbl_name LIKE '");
    quoted = gaiaSingleQuotedSql (outTable);
    if (quoted)
      {
	  strcat (sql, quoted);
	  free (quoted);
      }
    strcat (sql, "'");
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  return 0;
      }
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		if (atoi (results[(i * columns) + 0]) != 0)
		    ok = 0;
	    }
      }
    sqlite3_free_table (results);

    return ok;
}

static gaiaGeomCollPtr
elemGeomFromPoint (gaiaPointPtr pt, int srid)
{
/* creating a Geometry containing a single Point */
    gaiaGeomCollPtr g = NULL;
    switch (pt->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  g = gaiaAllocGeomCollXYZM ();
	  break;
      case GAIA_XY_Z:
	  g = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  g = gaiaAllocGeomCollXYM ();
	  break;
      default:
	  g = gaiaAllocGeomColl ();
	  break;
      };
    if (!g)
	return NULL;
    g->Srid = srid;
    g->DeclaredType = GAIA_POINT;
    switch (pt->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  gaiaAddPointToGeomCollXYZM (g, pt->X, pt->Y, pt->Z, pt->M);
	  break;
      case GAIA_XY_Z:
	  gaiaAddPointToGeomCollXYZ (g, pt->X, pt->Y, pt->Z);
	  break;
      case GAIA_XY_M:
	  gaiaAddPointToGeomCollXYM (g, pt->X, pt->Y, pt->M);
	  break;
      default:
	  gaiaAddPointToGeomColl (g, pt->X, pt->Y);
	  break;
      };
    return g;
}

static gaiaGeomCollPtr
elemGeomFromLinestring (gaiaLinestringPtr ln, int srid)
{
/* creating a Geometry containing a single Linestring */
    gaiaGeomCollPtr g = NULL;
    gaiaLinestringPtr ln2;
    int iv;
    double x;
    double y;
    double z;
    double m;
    switch (ln->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  g = gaiaAllocGeomCollXYZM ();
	  break;
      case GAIA_XY_Z:
	  g = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  g = gaiaAllocGeomCollXYM ();
	  break;
      default:
	  g = gaiaAllocGeomColl ();
	  break;
      };
    if (!g)
	return NULL;
    g->Srid = srid;
    g->DeclaredType = GAIA_LINESTRING;
    ln2 = gaiaAddLinestringToGeomColl (g, ln->Points);
    switch (ln->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		gaiaGetPointXYZM (ln->Coords, iv, &x, &y, &z, &m);
		gaiaSetPointXYZM (ln2->Coords, iv, x, y, z, m);
	    }
	  break;
      case GAIA_XY_Z:
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		gaiaGetPointXYZ (ln->Coords, iv, &x, &y, &z);
		gaiaSetPointXYZ (ln2->Coords, iv, x, y, z);
	    }
	  break;
      case GAIA_XY_M:
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		gaiaGetPointXYM (ln->Coords, iv, &x, &y, &m);
		gaiaSetPointXYM (ln2->Coords, iv, x, y, m);
	    }
	  break;
      default:
	  for (iv = 0; iv < ln->Points; iv++)
	    {
		gaiaGetPoint (ln->Coords, iv, &x, &y);
		gaiaSetPoint (ln2->Coords, iv, x, y);
	    }
	  break;
      };
    return g;
}

static gaiaGeomCollPtr
elemGeomFromPolygon (gaiaPolygonPtr pg, int srid)
{
/* creating a Geometry containing a single Polygon */
    gaiaGeomCollPtr g = NULL;
    gaiaPolygonPtr pg2;
    gaiaRingPtr rng;
    gaiaRingPtr rng2;
    int ib;
    int iv;
    double x;
    double y;
    double z;
    double m;
    switch (pg->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  g = gaiaAllocGeomCollXYZM ();
	  break;
      case GAIA_XY_Z:
	  g = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  g = gaiaAllocGeomCollXYM ();
	  break;
      default:
	  g = gaiaAllocGeomColl ();
	  break;
      };
    if (!g)
	return NULL;
    g->Srid = srid;
    g->DeclaredType = GAIA_POLYGON;
    rng = pg->Exterior;
    pg2 = gaiaAddPolygonToGeomColl (g, rng->Points, pg->NumInteriors);
    rng2 = pg2->Exterior;
    switch (pg->DimensionModel)
      {
      case GAIA_XY_Z_M:
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		gaiaSetPointXYZM (rng2->Coords, iv, x, y, z, m);
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		rng2 = gaiaAddInteriorRing (pg2, ib, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYZM (rng->Coords, iv, &x, &y, &z, &m);
		      gaiaSetPointXYZM (rng2->Coords, iv, x, y, z, m);
		  }
	    }
	  break;
      case GAIA_XY_Z:
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		gaiaSetPointXYZ (rng2->Coords, iv, x, y, z);
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		rng2 = gaiaAddInteriorRing (pg2, ib, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYZ (rng->Coords, iv, &x, &y, &z);
		      gaiaSetPointXYZ (rng2->Coords, iv, x, y, z);
		  }
	    }
	  break;
      case GAIA_XY_M:
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		gaiaSetPointXYM (rng2->Coords, iv, x, y, m);
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		rng2 = gaiaAddInteriorRing (pg2, ib, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPointXYM (rng->Coords, iv, &x, &y, &m);
		      gaiaSetPointXYM (rng2->Coords, iv, x, y, m);
		  }
	    }
	  break;
      default:
	  for (iv = 0; iv < rng->Points; iv++)
	    {
		gaiaGetPoint (rng->Coords, iv, &x, &y);
		gaiaSetPoint (rng2->Coords, iv, x, y);
	    }
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		rng2 = gaiaAddInteriorRing (pg2, ib, rng->Points);
		for (iv = 0; iv < rng->Points; iv++)
		  {
		      gaiaGetPoint (rng->Coords, iv, &x, &y);
		      gaiaSetPoint (rng2->Coords, iv, x, y);
		  }
	    }
	  break;
      };
    return g;
}

SPATIALITE_DECLARE void
elementary_geometries (sqlite3 * sqlite,
		       char *inTable, char *geometry, char *outTable,
		       char *pKey, char *multiId)
{
/* attempting to create a derived table surely containing elemetary Geoms */
    char type[128];
    int srid;
    char dims[64];
    char sql[8192];
    char sql2[8192];
    char sql3[8192];
    char sql4[8192];
    char sqlx[1024];
    char sql_geom[1024];
    char dummy[1024];
    char *quoted;
    int ret;
    int comma = 0;
    char *errMsg = NULL;
    int i;
    char **results;
    int rows;
    int columns;
    int geom_idx = -1;
    sqlite3_stmt *stmt_in = NULL;
    sqlite3_stmt *stmt_out = NULL;
    int n_columns;
    sqlite3_int64 id = 0;

    if (check_elementary
	(sqlite, inTable, geometry, outTable, pKey, multiId, type, &srid,
	 dims) == 0)
      {
	  fprintf (stderr, ".elemgeo: invalid args\n");
	  return;
      }

/* starts a transaction */
    ret = sqlite3_exec (sqlite, "BEGIN", NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  goto abort;
      }

    strcpy (sql, "SELECT ");
    strcpy (sql2, "INSERT INTO \"");
    strcpy (sql3, ") VALUES (NULL, ?");
    strcpy (sql4, "CREATE TABLE \"");
    quoted = gaiaDoubleQuotedSql (outTable);
    if (quoted)
      {
	  strcat (sql2, quoted);
	  strcat (sql4, quoted);
	  free (quoted);
      }
    strcat (sql2, "\" (\"");
    quoted = gaiaDoubleQuotedSql (pKey);
    if (quoted)
      {
	  strcat (sql2, quoted);
	  free (quoted);
      }
    strcat (sql2, "\", \"");
    quoted = gaiaDoubleQuotedSql (multiId);
    if (quoted)
      {
	  strcat (sql2, quoted);
	  free (quoted);
      }
    strcat (sql2, "\"");
    strcat (sql4, "\" (\n\t\"");
    quoted = gaiaDoubleQuotedSql (pKey);
    if (quoted)
      {
	  strcat (sql4, quoted);
	  free (quoted);
      }
    strcat (sql4, "\" INTEGER PRIMARY KEY AUTOINCREMENT");
    strcat (sql4, ",\n\t\"");
    quoted = gaiaDoubleQuotedSql (multiId);
    if (quoted)
      {
	  strcat (sql4, quoted);
	  free (quoted);
      }
    strcat (sql4, "\" INTEGER NOT NULL");

    strcpy (sqlx, "PRAGMA table_info(\"");
    quoted = gaiaDoubleQuotedSql (inTable);
    if (quoted)
      {
	  strcat (sqlx, quoted);
	  free (quoted);
      }
    strcat (sqlx, "\")");
    ret = sqlite3_get_table (sqlite, sqlx, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  goto abort;
      }
    if (rows < 1)
	;
    else
      {
	  for (i = 1; i <= rows; i++)
	    {
		if (comma)
		    strcat (sql, ", \"");
		else
		  {
		      comma = 1;
		      strcat (sql, "\"");
		  }
		strcat (sql2, ", \"");
		quoted = gaiaDoubleQuotedSql (results[(i * columns) + 1]);
		if (quoted)
		  {
		      strcat (sql, quoted);
		      strcat (sql2, quoted);
		      free (quoted);
		  }
		strcat (sql2, "\"");
		strcat (sql3, ", ?");

		if (strcasecmp (geometry, results[(i * columns) + 1]) == 0)
		    geom_idx = i - 1;
		else
		  {
		      strcat (sql4, ",\n\t\"");
		      quoted = gaiaDoubleQuotedSql (results[(i * columns) + 1]);
		      if (quoted)
			{
			    strcat (sql4, quoted);
			    free (quoted);
			}
		      strcat (sql4, "\" ");
		      strcat (sql4, results[(i * columns) + 2]);
		      if (atoi (results[(i * columns) + 3]) != 0)
			  strcat (sql4, " NOT NULL");
		  }
	    }
      }
    sqlite3_free_table (results);
    if (geom_idx < 0)
	goto abort;

    strcat (sql, " FROM \"");
    quoted = gaiaDoubleQuotedSql (inTable);
    if (quoted)
      {
	  strcat (sql, quoted);
	  free (quoted);
      }
    strcat (sql, "\"");
    strcat (sql2, sql3);
    strcat (sql2, ")");
    strcat (sql4, ")");

    strcpy (sql_geom, "SELECT AddGeometryColumn('");
    quoted = gaiaSingleQuotedSql (outTable);
    if (quoted)
      {
	  strcat (sql_geom, quoted);
	  free (quoted);
      }
    strcat (sql_geom, "', '");
    quoted = gaiaSingleQuotedSql (geometry);
    if (quoted)
      {
	  strcat (sql_geom, quoted);
	  free (quoted);
      }
    strcat (sql_geom, "', ");
    sprintf (dummy, "%d, '", srid);
    strcat (sql_geom, dummy);
    quoted = gaiaSingleQuotedSql (type);
    if (quoted)
      {
	  strcat (sql_geom, quoted);
	  free (quoted);
      }
    strcat (sql_geom, "', '");
    quoted = gaiaSingleQuotedSql (dims);
    if (quoted)
      {
	  strcat (sql_geom, quoted);
	  free (quoted);
      }
    strcat (sql_geom, "')");

/* creating the output table */
    ret = sqlite3_exec (sqlite, sql4, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  goto abort;
      }
/* creating the output Geometry */
    ret = sqlite3_exec (sqlite, sql_geom, NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  goto abort;
      }

/* preparing the INPUT statement */
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt_in, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
	  goto abort;
      }

/* preparing the OUTPUT statement */
    ret = sqlite3_prepare_v2 (sqlite, sql2, strlen (sql2), &stmt_out, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", sqlite3_errmsg (sqlite));
	  goto abort;
      }

/* data transfer */
    n_columns = sqlite3_column_count (stmt_in);
    while (1)
      {
	  ret = sqlite3_step (stmt_in);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		gaiaGeomCollPtr g =
		    gaiaFromSpatiaLiteBlobWkb ((const unsigned char *)
					       sqlite3_column_blob (stmt_in,
								    geom_idx),
					       sqlite3_column_bytes (stmt_in,
								     geom_idx));
		if (!g)
		  {
		      /* NULL input geometry */
		      sqlite3_reset (stmt_out);
		      sqlite3_clear_bindings (stmt_out);
		      sqlite3_bind_int64 (stmt_out, 1, id);
		      sqlite3_bind_null (stmt_out, geom_idx + 2);

		      for (i = 0; i < n_columns; i++)
			{
			    int type = sqlite3_column_type (stmt_in, i);
			    if (i == geom_idx)
				continue;
			    switch (type)
			      {
			      case SQLITE_INTEGER:
				  sqlite3_bind_int64 (stmt_out, i + 2,
						      sqlite3_column_int
						      (stmt_in, i));
				  break;
			      case SQLITE_FLOAT:
				  sqlite3_bind_double (stmt_out, i + 2,
						       sqlite3_column_double
						       (stmt_in, i));
				  break;
			      case SQLITE_TEXT:
				  sqlite3_bind_text (stmt_out, i + 2,
						     (const char *)
						     sqlite3_column_text
						     (stmt_in, i),
						     sqlite3_column_bytes
						     (stmt_in, i),
						     SQLITE_STATIC);
				  break;
			      case SQLITE_BLOB:
				  sqlite3_bind_blob (stmt_out, i + 2,
						     sqlite3_column_blob
						     (stmt_in, i),
						     sqlite3_column_bytes
						     (stmt_in, i),
						     SQLITE_STATIC);
				  break;
			      case SQLITE_NULL:
			      default:
				  sqlite3_bind_null (stmt_out, i + 2);
				  break;
			      };
			}

		      ret = sqlite3_step (stmt_out);
		      if (ret == SQLITE_DONE || ret == SQLITE_ROW)
			  ;
		      else
			{
			    fprintf (stderr, "[OUT]step error: %s\n",
				     sqlite3_errmsg (sqlite));
			    goto abort;
			}
		  }
		else
		  {
		      /* separating Elementary Geoms */
		      gaiaPointPtr pt;
		      gaiaLinestringPtr ln;
		      gaiaPolygonPtr pg;
		      gaiaGeomCollPtr outGeom;
		      pt = g->FirstPoint;
		      while (pt)
			{
			    /* separating Points */
			    outGeom = elemGeomFromPoint (pt, g->Srid);
			    sqlite3_reset (stmt_out);
			    sqlite3_clear_bindings (stmt_out);
			    sqlite3_bind_int64 (stmt_out, 1, id);
			    if (!outGeom)
				sqlite3_bind_null (stmt_out, geom_idx + 2);
			    else
			      {
				  unsigned char *blob;
				  int size;
				  gaiaToSpatiaLiteBlobWkb (outGeom, &blob,
							   &size);
				  sqlite3_bind_blob (stmt_out, geom_idx + 2,
						     blob, size, free);
				  gaiaFreeGeomColl (outGeom);
			      }

			    for (i = 0; i < n_columns; i++)
			      {
				  int type = sqlite3_column_type (stmt_in, i);
				  if (i == geom_idx)
				      continue;
				  switch (type)
				    {
				    case SQLITE_INTEGER:
					sqlite3_bind_int64 (stmt_out, i + 2,
							    sqlite3_column_int
							    (stmt_in, i));
					break;
				    case SQLITE_FLOAT:
					sqlite3_bind_double (stmt_out, i + 2,
							     sqlite3_column_double
							     (stmt_in, i));
					break;
				    case SQLITE_TEXT:
					sqlite3_bind_text (stmt_out, i + 2,
							   (const char *)
							   sqlite3_column_text
							   (stmt_in, i),
							   sqlite3_column_bytes
							   (stmt_in, i),
							   SQLITE_STATIC);
					break;
				    case SQLITE_BLOB:
					sqlite3_bind_blob (stmt_out, i + 2,
							   sqlite3_column_blob
							   (stmt_in, i),
							   sqlite3_column_bytes
							   (stmt_in, i),
							   SQLITE_STATIC);
					break;
				    case SQLITE_NULL:
				    default:
					sqlite3_bind_null (stmt_out, i + 2);
					break;
				    };
			      }

			    ret = sqlite3_step (stmt_out);
			    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
				;
			    else
			      {
				  fprintf (stderr, "[OUT]step error: %s\n",
					   sqlite3_errmsg (sqlite));
				  goto abort;
			      }
			    pt = pt->Next;
			}
		      ln = g->FirstLinestring;
		      while (ln)
			{
			    /* separating Linestrings */
			    outGeom = elemGeomFromLinestring (ln, g->Srid);
			    sqlite3_reset (stmt_out);
			    sqlite3_clear_bindings (stmt_out);
			    sqlite3_bind_int64 (stmt_out, 1, id);
			    if (!outGeom)
				sqlite3_bind_null (stmt_out, geom_idx + 2);
			    else
			      {
				  unsigned char *blob;
				  int size;
				  gaiaToSpatiaLiteBlobWkb (outGeom, &blob,
							   &size);
				  sqlite3_bind_blob (stmt_out, geom_idx + 2,
						     blob, size, free);
				  gaiaFreeGeomColl (outGeom);
			      }

			    for (i = 0; i < n_columns; i++)
			      {
				  int type = sqlite3_column_type (stmt_in, i);
				  if (i == geom_idx)
				      continue;
				  switch (type)
				    {
				    case SQLITE_INTEGER:
					sqlite3_bind_int64 (stmt_out, i + 2,
							    sqlite3_column_int
							    (stmt_in, i));
					break;
				    case SQLITE_FLOAT:
					sqlite3_bind_double (stmt_out, i + 2,
							     sqlite3_column_double
							     (stmt_in, i));
					break;
				    case SQLITE_TEXT:
					sqlite3_bind_text (stmt_out, i + 2,
							   (const char *)
							   sqlite3_column_text
							   (stmt_in, i),
							   sqlite3_column_bytes
							   (stmt_in, i),
							   SQLITE_STATIC);
					break;
				    case SQLITE_BLOB:
					sqlite3_bind_blob (stmt_out, i + 2,
							   sqlite3_column_blob
							   (stmt_in, i),
							   sqlite3_column_bytes
							   (stmt_in, i),
							   SQLITE_STATIC);
					break;
				    case SQLITE_NULL:
				    default:
					sqlite3_bind_null (stmt_out, i + 2);
					break;
				    };
			      }

			    ret = sqlite3_step (stmt_out);
			    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
				;
			    else
			      {
				  fprintf (stderr, "[OUT]step error: %s\n",
					   sqlite3_errmsg (sqlite));
				  goto abort;
			      }
			    ln = ln->Next;
			}
		      pg = g->FirstPolygon;
		      while (pg)
			{
			    /* separating Polygons */
			    outGeom = elemGeomFromPolygon (pg, g->Srid);
			    sqlite3_reset (stmt_out);
			    sqlite3_clear_bindings (stmt_out);
			    sqlite3_bind_int64 (stmt_out, 1, id);
			    if (!outGeom)
				sqlite3_bind_null (stmt_out, geom_idx + 2);
			    else
			      {
				  unsigned char *blob;
				  int size;
				  gaiaToSpatiaLiteBlobWkb (outGeom, &blob,
							   &size);
				  sqlite3_bind_blob (stmt_out, geom_idx + 2,
						     blob, size, free);
				  gaiaFreeGeomColl (outGeom);
			      }

			    for (i = 0; i < n_columns; i++)
			      {
				  int type = sqlite3_column_type (stmt_in, i);
				  if (i == geom_idx)
				      continue;
				  switch (type)
				    {
				    case SQLITE_INTEGER:
					sqlite3_bind_int64 (stmt_out, i + 2,
							    sqlite3_column_int
							    (stmt_in, i));
					break;
				    case SQLITE_FLOAT:
					sqlite3_bind_double (stmt_out, i + 2,
							     sqlite3_column_double
							     (stmt_in, i));
					break;
				    case SQLITE_TEXT:
					sqlite3_bind_text (stmt_out, i + 2,
							   (const char *)
							   sqlite3_column_text
							   (stmt_in, i),
							   sqlite3_column_bytes
							   (stmt_in, i),
							   SQLITE_STATIC);
					break;
				    case SQLITE_BLOB:
					sqlite3_bind_blob (stmt_out, i + 2,
							   sqlite3_column_blob
							   (stmt_in, i),
							   sqlite3_column_bytes
							   (stmt_in, i),
							   SQLITE_STATIC);
					break;
				    case SQLITE_NULL:
				    default:
					sqlite3_bind_null (stmt_out, i + 2);
					break;
				    };
			      }

			    ret = sqlite3_step (stmt_out);
			    if (ret == SQLITE_DONE || ret == SQLITE_ROW)
				;
			    else
			      {
				  fprintf (stderr, "[OUT]step error: %s\n",
					   sqlite3_errmsg (sqlite));
				  goto abort;
			      }
			    pg = pg->Next;
			}
		      gaiaFreeGeomColl (g);
		  }
		id++;
	    }
	  else
	    {
		fprintf (stderr, "[IN]step error: %s\n",
			 sqlite3_errmsg (sqlite));
		goto abort;
	    }
      }
    sqlite3_finalize (stmt_in);
    sqlite3_finalize (stmt_out);

/* commits the transaction */
    ret = sqlite3_exec (sqlite, "COMMIT", NULL, NULL, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "SQL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  goto abort;
      }
    return;

  abort:
    if (stmt_in)
	sqlite3_finalize (stmt_in);
    if (stmt_out)
	sqlite3_finalize (stmt_out);
}

#ifndef OMIT_FREEXL		/* including FreeXL */

SPATIALITE_DECLARE int
load_XL (sqlite3 * sqlite, const char *path, const char *table,
	 unsigned int worksheetIndex, int first_titles, unsigned int *rows,
	 char *err_msg)
{
/* loading an XL spreadsheet as a new DB table */
    sqlite3_stmt *stmt;
    unsigned int current_row;
    int ret;
    char *errMsg = NULL;
    char xname[1024];
    char dummyName[4096];
    char sql[65536];
    int sqlError = 0;
    const void *xl_handle;
    unsigned int info;
    unsigned short columns;
    unsigned short col;
    FreeXL_CellValue cell;
    int already_exists = 0;
/* checking if TABLE already exists */
    sprintf (sql,
	     "SELECT name FROM sqlite_master WHERE type = 'table' AND name LIKE '%s'",
	     table);
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  if (!err_msg)
	      fprintf (stderr, "load XL error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  else
	      sprintf (err_msg, "load XL error: <%s>\n",
		       sqlite3_errmsg (sqlite));
	  return 0;
      }
    while (1)
      {
	  /* scrolling the result set */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	      already_exists = 1;
	  else
	    {
		fprintf (stderr, "load XL error: <%s>\n",
			 sqlite3_errmsg (sqlite));
		break;
	    }
      }
    sqlite3_finalize (stmt);
    if (already_exists)
      {
	  if (!err_msg)
	      fprintf (stderr, "load XL error: table '%s' already exists\n",
		       table);
	  else
	      sprintf (err_msg, "load XL error: table '%s' already exists\n",
		       table);
	  return 0;
      }
/* opening the .XLS file [Workbook] */
    ret = freexl_open (path, &xl_handle);
    if (ret != FREEXL_OK)
	goto error;
/* checking if Password protected */
    ret = freexl_get_info (xl_handle, FREEXL_BIFF_PASSWORD, &info);
    if (ret != FREEXL_OK)
	goto error;
    if (info != FREEXL_BIFF_PLAIN)
	goto error;
/* Worksheet entries */
    ret = freexl_get_info (xl_handle, FREEXL_BIFF_SHEET_COUNT, &info);
    if (ret != FREEXL_OK)
	goto error;
    if (info == 0)
	goto error;
    if (worksheetIndex < info)
	;
    else
	goto error;
    ret = freexl_select_active_worksheet (xl_handle, worksheetIndex);
    if (ret != FREEXL_OK)
	goto error;
    ret = freexl_worksheet_dimensions (xl_handle, rows, &columns);
    if (ret != FREEXL_OK)
	goto error;
/* starting a transaction */
    ret = sqlite3_exec (sqlite, "BEGIN", NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "load XL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  sqlError = 1;
	  goto clean_up;
      }
/* creating the Table */
    strcpy (xname, table);
    shp_double_quoted_sql (xname);
    sprintf (sql, "CREATE TABLE %s", xname);
    strcat (sql, " (\nPK_UID INTEGER PRIMARY KEY AUTOINCREMENT");
    for (col = 0; col < columns; col++)
      {
	  if (first_titles)
	    {
		/* fetching column names */
		for (col = 0; col < columns; col++)
		  {
		      ret = freexl_get_cell_value (xl_handle, 0, col, &cell);
		      if (ret != FREEXL_OK)
			  sprintf (dummyName, "col_%d", col);
		      else
			{
			    if (cell.type == FREEXL_CELL_INT)
				sprintf (dummyName, "%d", cell.value.int_value);
			    else if (cell.type == FREEXL_CELL_DOUBLE)
				sprintf (dummyName, "%1.2f",
					 cell.value.double_value);
			    else if (cell.type == FREEXL_CELL_TEXT
				     || cell.type == FREEXL_CELL_SST_TEXT
				     || cell.type == FREEXL_CELL_DATE
				     || cell.type == FREEXL_CELL_DATETIME
				     || cell.type == FREEXL_CELL_TIME)
			      {
				  int len = strlen (cell.value.text_value);
				  if (len < 256)
				      strcpy (dummyName, cell.value.text_value);
				  else
				      sprintf (dummyName, "col_%d", col);
			      }
			    else
				sprintf (dummyName, "col_%d", col);
			}
		      shp_double_quoted_sql (dummyName);
		      strcat (sql, ", ");
		      strcat (sql, dummyName);
		  }
	    }
	  else
	    {
		/* setting default column names */
		for (col = 0; col < columns; col++)
		  {
		      sprintf (dummyName, "col_%d", col);
		      shp_double_quoted_sql (dummyName);
		      strcat (sql, ", ");
		      strcat (sql, dummyName);
		  }
	    }
      }
    strcat (sql, ")");
    ret = sqlite3_exec (sqlite, sql, NULL, 0, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "load XL error: %s\n", errMsg);
	  sqlite3_free (errMsg);
	  sqlError = 1;
	  goto clean_up;
      }
/* preparing the INSERT INTO parameterized statement */
    strcpy (xname, table);
    shp_double_quoted_sql (xname);
    sprintf (sql, "INSERT INTO %s (PK_UID", xname);
    for (col = 0; col < columns; col++)
      {
	  if (first_titles)
	    {
		ret = freexl_get_cell_value (xl_handle, 0, col, &cell);
		if (ret != FREEXL_OK)
		    sprintf (dummyName, "col_%d", col);
		else
		  {
		      if (cell.type == FREEXL_CELL_INT)
			  sprintf (dummyName, "%d", cell.value.int_value);
		      else if (cell.type == FREEXL_CELL_DOUBLE)
			  sprintf (dummyName, "%1.2f", cell.value.double_value);
		      else if (cell.type == FREEXL_CELL_TEXT
			       || cell.type == FREEXL_CELL_SST_TEXT
			       || cell.type == FREEXL_CELL_DATE
			       || cell.type == FREEXL_CELL_DATETIME
			       || cell.type == FREEXL_CELL_TIME)
			{
			    int len = strlen (cell.value.text_value);
			    if (len < 256)
				strcpy (dummyName, cell.value.text_value);
			    else
				sprintf (dummyName, "col_%d", col);
			}
		      else
			  sprintf (dummyName, "col_%d", col);
		  }
		shp_double_quoted_sql (dummyName);
		strcat (sql, ", ");
		strcat (sql, dummyName);
	    }
	  else
	    {
		/* setting default column names  */
		sprintf (dummyName, "col_%d", col);
		shp_double_quoted_sql (dummyName);
		strcat (sql, ", ");
		strcat (sql, dummyName);
	    }
      }
    strcat (sql, ")\nVALUES (NULL");
    for (col = 0; col < columns; col++)
      {
	  /* column values */
	  strcat (sql, ", ?");
      }
    strcat (sql, ")");
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "load XL error: %s\n", sqlite3_errmsg (sqlite));
	  sqlError = 1;
	  goto clean_up;
      }
    if (first_titles)
	current_row = 1;
    else
	current_row = 0;
    while (current_row < *rows)
      {
	  /* binding query params */
	  sqlite3_reset (stmt);
	  sqlite3_clear_bindings (stmt);
	  for (col = 0; col < columns; col++)
	    {
		/* column values */
		ret =
		    freexl_get_cell_value (xl_handle, current_row, col, &cell);
		if (ret != FREEXL_OK)
		    sqlite3_bind_null (stmt, col + 1);
		else
		  {
		      switch (cell.type)
			{
			case FREEXL_CELL_INT:
			    sqlite3_bind_int (stmt, col + 1,
					      cell.value.int_value);
			    break;
			case FREEXL_CELL_DOUBLE:
			    sqlite3_bind_double (stmt, col + 1,
						 cell.value.double_value);
			    break;
			case FREEXL_CELL_TEXT:
			case FREEXL_CELL_SST_TEXT:
			case FREEXL_CELL_DATE:
			case FREEXL_CELL_DATETIME:
			case FREEXL_CELL_TIME:
			    sqlite3_bind_text (stmt, col + 1,
					       cell.value.text_value,
					       strlen (cell.value.text_value),
					       SQLITE_STATIC);
			    break;
			default:
			    sqlite3_bind_null (stmt, col + 1);
			    break;
			};
		  }
	    }
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE || ret == SQLITE_ROW)
	      ;
	  else
	    {
		fprintf (stderr, "load XL error: %s\n",
			 sqlite3_errmsg (sqlite));
		sqlite3_finalize (stmt);
		sqlError = 1;
		goto clean_up;
	    }
	  current_row++;
      }
    sqlite3_finalize (stmt);
  clean_up:
    if (sqlError)
      {
	  /* some error occurred - ROLLBACK */
	  ret = sqlite3_exec (sqlite, "ROLLBACK", NULL, 0, &errMsg);
	  if (ret != SQLITE_OK)
	    {
		fprintf (stderr, "load XL error: %s\n", errMsg);
		sqlite3_free (errMsg);
	    }
	  fprintf (stderr,
		   "XL not loaded\n\n\na ROLLBACK was automatically performed\n");
      }
    else
      {
	  /* ok - confirming pending transaction - COMMIT */
	  ret = sqlite3_exec (sqlite, "COMMIT", NULL, 0, &errMsg);
	  if (ret != SQLITE_OK)
	    {
		if (!err_msg)
		    fprintf (stderr, "load XL error: %s\n", errMsg);
		else
		    sprintf (err_msg, "load XL error: %s\n", errMsg);
		sqlite3_free (errMsg);
		return 0;
	    }
	  if (first_titles)
	      *rows = *rows - 1;	/* allow for header row */
	  fprintf (stderr, "XL loaded\n\n%d inserted rows\n", *rows);
      }
    freexl_close (xl_handle);
    return 1;

  error:
    freexl_close (xl_handle);
    if (!err_msg)
	fprintf (stderr, "XL datasource '%s' is not valid\n", path);
    else
	sprintf (err_msg, "XL datasource '%s' is not valid\n", path);
    return 0;
}

#endif /* FreeXL enabled/disabled */

SPATIALITE_DECLARE int
dump_geojson (sqlite3 * sqlite, char *table, char *geom_col, char *outfile_path,
	      int precision, int option)
{
/* dumping a  geometry table as GeoJSON - Brad Hards 2011-11-09 */
    char sql[4096];
    sqlite3_stmt *stmt = NULL;
    FILE *out = NULL;
    int ret;
    int rows = 0;
    int is_const = 1;

/* opening/creating the GeoJSON output file */
    out = fopen (outfile_path, "wb");
    if (!out)
	goto no_file;

/* preparing SQL statement */
    sprintf (sql, "SELECT AsGeoJSON(%s, %d, %d) FROM %s WHERE %s IS NOT NULL",
	     geom_col, precision, option, table, geom_col);
    ret = sqlite3_prepare_v2 (sqlite, sql, strlen (sql), &stmt, NULL);
    if (ret != SQLITE_OK)
	goto sql_error;

    while (1)
      {
	  /* scrolling the result set */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	    {
		break;		/* end of result set */
	    }
	  if (ret == SQLITE_ROW)
	    {
		rows++;
		fprintf (out, "%s\r\n", sqlite3_column_text (stmt, 0));
	    }
	  else
	    {
		goto sql_error;
	    }
      }
    if (rows == 0)
      {
	  goto empty_result_set;
      }

    sqlite3_finalize (stmt);
    fclose (out);
    return 1;

  sql_error:
/* an SQL error occurred */
    if (stmt)
      {
	  sqlite3_finalize (stmt);
      }
    if (out)
      {
	  fclose (out);
      }
    fprintf (stderr, "Dump GeoJSON error: %s\n", sqlite3_errmsg (sqlite));
    return 0;

  no_file:
/* Output file could not be created / opened */
    if (stmt)
      {
	  sqlite3_finalize (stmt);
      }
    if (out)
      {
	  fclose (out);
      }
    fprintf (stderr, "ERROR: unable to open '%s' for writing\n", outfile_path);
    return 0;

  empty_result_set:
/* the result set is empty - nothing to do */
    if (stmt)
      {
	  sqlite3_finalize (stmt);
      }
    if (out)
      {
	  fclose (out);
      }
    fprintf (stderr, "The SQL SELECT returned no data to export...\n");
    return 0;
}
/**************** End file: shapefiles.c **********/


/**************** Begin file: gg_vanuatu.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

/* #include <assert.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

#if defined(_WIN32) || defined(WIN32)
# include <io.h>
#define isatty  _isatty
#define fileno  _fileno
#endif

int vanuatu_parse_error;

static int
vanuatuCheckValidity (gaiaGeomCollPtr geom)
{
/* checks if this one is a degenerated geometry */
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int ib;
    int entities = 0;
    pt = geom->FirstPoint;
    while (pt)
      {
	  /* checking points */
	  entities++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* checking linestrings */
	  if (ln->Points < 2)
	      return 0;
	  entities++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  /* checking polygons */
	  rng = pg->Exterior;
	  if (rng->Points < 4)
	      return 0;
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		if (rng->Points < 4)
		    return 0;
	    }
	  entities++;
	  pg = pg->Next;
      }
    if (!entities)
	return 0;
    return 1;
}

static gaiaGeomCollPtr
gaiaGeometryFromPoint (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINT */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomColl ();
    geom->DeclaredType = GAIA_POINT;
    gaiaAddPointToGeomColl (geom, point->X, point->Y);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeometryFromPointZ (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINTZ */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomCollXYZ ();
    geom->DeclaredType = GAIA_POINTZ;
    gaiaAddPointToGeomCollXYZ (geom, point->X, point->Y, point->Z);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeometryFromPointM (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINTM */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomCollXYM ();
    geom->DeclaredType = GAIA_POINTM;
    gaiaAddPointToGeomCollXYM (geom, point->X, point->Y, point->M);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeometryFromPointZM (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINTZM */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomCollXYZM ();
    geom->DeclaredType = GAIA_POINTZM;
    gaiaAddPointToGeomCollXYZM (geom, point->X, point->Y, point->Z, point->M);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeometryFromLinestring (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRING */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    geom = gaiaAllocGeomColl ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPoint (line->Coords, iv, &x, &y);
	  gaiaSetPoint (line2->Coords, iv, x, y);
      }
    gaiaFreeLinestring (line);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeometryFromLinestringZ (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRINGZ */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    double z;
    geom = gaiaAllocGeomCollXYZ ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	  gaiaSetPointXYZ (line2->Coords, iv, x, y, z);
      }
    gaiaFreeLinestring (line);
    return geom;
}


static gaiaGeomCollPtr
gaiaGeometryFromLinestringM (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRINGM */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    double m;
    geom = gaiaAllocGeomCollXYM ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	  gaiaSetPointXYM (line2->Coords, iv, x, y, m);
      }
    gaiaFreeLinestring (line);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeometryFromLinestringZM (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRINGZM */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    double z;
    double m;
    geom = gaiaAllocGeomCollXYZM ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	  gaiaSetPointXYZM (line2->Coords, iv, x, y, z, m);
      }
    gaiaFreeLinestring (line);
    return geom;
}

/******************************************************************************
** The following code was created by Team Vanuatu of The University of Toronto.
** It is responsible for handling the parsing of wkt expressions.  The parser
** is built using LEMON and the cooresponding methods were written by the 
** students.

Authors:
Ruppi Rana			ruppi.rana@gmail.com
Dev Tanna			dev.tanna@gmail.com
Elias Adum			elias.adum@gmail.com
Benton Hui			benton.hui@gmail.com
Abhayan Sundararajan		abhayan@gmail.com
Chee-Lun Michael Stephen Cho	cheelun.cho@gmail.com
Nikola Banovic			nikola.banovic@gmail.com
Yong Jian			yong.jian@utoronto.ca

Supervisor:
Greg Wilson			gvwilson@cs.toronto.ca

-------------------------------------------------------------------------------
*/

/* 
 * Creates a 2D (xy) point in SpatiaLite
 * x and y are pointers to doubles which represent the x and y coordinates of the point to be created.
 * Returns a gaiaPointPtr representing the created point.
 *
 * Creates a 2D (xy) point. This is a parser helper function which is called when 2D coordinates are encountered.
 * Parameters x and y are pointers to doubles which represent the x and y coordinates of the point to be created.
 * Returns a gaiaPointPtr pointing to the created 2D point.
 */
static gaiaPointPtr
vanuatu_point_xy (double *x, double *y)
{
    return gaiaAllocPoint (*x, *y);
}

/* 
 * Creates a 3D (xyz) point in SpatiaLite
 * x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created.
 * Returns a gaiaPointPtr representing the created point.
 *
 * Creates a 3D (xyz) point. This is a parser helper function which is called when 3D coordinates are encountered.
 * Parameters x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created.
 * Returns a gaiaPointPtr pointing to the 3D created point.
 */
static gaiaPointPtr
vanuatu_point_xyz (double *x, double *y, double *z)
{
    return gaiaAllocPointXYZ (*x, *y, *z);
}

/* 
 * Creates a 2D (xy) point with an m value which is a part of the linear reference system. This is a parser helper
 * function which is called when 2D   *coordinates with an m value are encountered.
 * Parameters x and y are pointers to doubles which represent the x and y coordinates of the point to be created.
 * Parameter m is a pointer to a double which represents the part of the linear reference system.
 * Returns a gaiaPointPtr pointing to the created 2D point with an m value.
 */
static gaiaPointPtr
vanuatu_point_xym (double *x, double *y, double *m)
{
    return gaiaAllocPointXYM (*x, *y, *m);
}

/* 
 * Creates a 4D (xyz) point with an m value which is a part of the linear reference system. This is a parser helper
 * function which is called when  *4Dcoordinates with an m value are encountered
 * Parameters x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created. 
 * Parameter m is a pointer to a double which represents the part of the linear reference system.
 * Returns a gaiaPointPtr pointing the created 4D point with an m value.
 */
gaiaPointPtr
vanuatu_point_xyzm (double *x, double *y, double *z, double *m)
{
    return gaiaAllocPointXYZM (*x, *y, *z, *m);
}

/*
 * Builds a geometry collection from a point. The geometry collection should contain only one element ? the point. 
 * The correct geometry type must be *decided based on the point type. The parser should call this function when the 
 * ?POINT? WKT expression is encountered.
 * Parameter point is a pointer to a 2D, 3D, 2D with an m value, or 4D with an m value point.
 * Returns a geometry collection containing the point. The geometry must have FirstPoint and LastPoint  pointing to the
 * same place as point.  *DimensionModel must be the same as the model of the point and DimensionType must be GAIA_TYPE_POINT.
 */
static gaiaGeomCollPtr
vanuatu_buildGeomFromPoint (gaiaPointPtr point)
{
    switch (point->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaGeometryFromPoint (point);
	  break;
      case GAIA_XY_Z:
	  return gaiaGeometryFromPointZ (point);
	  break;
      case GAIA_XY_M:
	  return gaiaGeometryFromPointM (point);
	  break;
      case GAIA_XY_Z_M:
	  return gaiaGeometryFromPointZM (point);
	  break;
      }
    return NULL;
}

/* 
 * Creates a 2D (xy) linestring from a list of 2D points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 2D (xy) points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
vanuatu_linestring_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestring (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPoint (linestring->Coords, i, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/* 
 * Creates a 3D (xyz) linestring from a list of 3D points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 3D (xyz) points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
vanuatu_linestring_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestringXYZ (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPointXYZ (linestring->Coords, i, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/* 
 * Creates a 2D (xy) with m value linestring from a list of 2D with m value points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 2D (xy) with m value points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
vanuatu_linestring_xym (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestringXYM (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPointXYM (linestring->Coords, i, p->X, p->Y, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/* 
 * Creates a 4D (xyz) with m value linestring from a list of 4D with m value points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 4D (xyz) with m value points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
vanuatu_linestring_xyzm (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestringXYZM (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPointXYZM (linestring->Coords, i, p->X, p->Y, p->Z, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/*
 * Builds a geometry collection from a linestring.
 */
static gaiaGeomCollPtr
vanuatu_buildGeomFromLinestring (gaiaLinestringPtr line)
{
    switch (line->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaGeometryFromLinestring (line);
	  break;
      case GAIA_XY_Z:
	  return gaiaGeometryFromLinestringZ (line);
	  break;
      case GAIA_XY_M:
	  return gaiaGeometryFromLinestringM (line);
	  break;
      case GAIA_XY_Z_M:
	  return gaiaGeometryFromLinestringZM (line);
	  break;
      }
    return NULL;
}

/*
 * Helper function that determines the number of points in the linked list.
 */
static int
vanuatu_count_points (gaiaPointPtr first)
{
    /* Counts the number of points in the ring. */
    gaiaPointPtr p = first;
    int numpoints = 0;
    while (p != NULL)
      {
	  numpoints++;
	  p = p->Next;
      }
    return numpoints;
}

/*
 * Creates a 2D (xy) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 2D (xy) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
vanuatu_ring_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = vanuatu_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRing (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPoint (ring->Coords, index, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Creates a 3D (xyz) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 3D (xyz) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
vanuatu_ring_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = vanuatu_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRingXYZ (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPointXYZ (ring->Coords, index, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Creates a 2D (xym) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 2D (xym) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
vanuatu_ring_xym (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = vanuatu_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRingXYM (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPointXYM (ring->Coords, index, p->X, p->Y, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Creates a 3D (xyzm) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 3D (xyzm) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
vanuatu_ring_xyzm (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = vanuatu_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRingXYZM (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPointXYZM (ring->Coords, index, p->X, p->Y, p->Z, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Helper function that will create any type of polygon (xy, xym, xyz, xyzm) in SpatiaLite.
 * 
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are of the same type. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
vanuatu_polygon_any_type (gaiaRingPtr first)
{
    gaiaRingPtr p;
    gaiaRingPtr p_n;
    gaiaPolygonPtr polygon;
    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a polygon structure with the exterior ring. */
    polygon = gaiaCreatePolygon (first);
    if (polygon == NULL)
	return NULL;

    /* Adds all interior rings into the polygon structure. */
    p = first;
    while (p != NULL)
      {
	  p_n = p->Next;
	  if (p == first)
	      gaiaFreeRing (p);
	  else
	      gaiaAddRingToPolyg (polygon, p);
	  p = p_n;
      }

    return polygon;
}

/* 
 * Creates a 2D (xy) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 2D (xy) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
vanuatu_polygon_xy (gaiaRingPtr first)
{
    return vanuatu_polygon_any_type (first);
}

/* 
 * Creates a 3D (xyz) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 3D (xyz) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
vanuatu_polygon_xyz (gaiaRingPtr first)
{
    return vanuatu_polygon_any_type (first);
}

/* 
 * Creates a 2D (xym) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 2D (xym) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
vanuatu_polygon_xym (gaiaRingPtr first)
{
    return vanuatu_polygon_any_type (first);
}

/* 
 * Creates a 3D (xyzm) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 3D (xyzm) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
vanuatu_polygon_xyzm (gaiaRingPtr first)
{
    return vanuatu_polygon_any_type (first);
}

/*
 * Builds a geometry collection from a polygon.
 * NOTE: This function may already be implemented in the SpatiaLite code base. If it is, make sure that we
 *              can use it (ie. it doesn't use any other variables or anything else set by Sandro's parser). If you find
 *              that we can use an existing function then ignore this one.
 */
static gaiaGeomCollPtr
vanuatu_buildGeomFromPolygon (gaiaPolygonPtr polygon)
{
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (polygon == NULL)
      {
	  return NULL;
      }

    /* Creates and allocates a geometry collection containing a multipoint. */
    switch (polygon->DimensionModel)
      {
      case GAIA_XY:
	  geom = gaiaAllocGeomColl ();
	  break;
      case GAIA_XY_Z:
	  geom = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  geom = gaiaAllocGeomCollXYM ();
	  break;
      case GAIA_XY_Z_M:
	  geom = gaiaAllocGeomCollXYZM ();
	  break;
      }
    if (geom == NULL)
      {
	  return NULL;
      }
    geom->DeclaredType = GAIA_POLYGON;

    /* Stores the location of the first and last polygons in the linked list. */
    geom->FirstPolygon = polygon;
    while (polygon != NULL)
      {
	  geom->LastPolygon = polygon;
	  polygon = polygon->Next;
      }
    return geom;
}

/* 
 * Creates a 2D (xy) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 2D (xy) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
vanuatu_multipoint_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 2D (xy) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomColl (geom, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/* 
 * Creates a 3D (xyz) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 3D (xyz) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
vanuatu_multipoint_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomCollXYZ ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 3D (xyz) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomCollXYZ (geom, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/* 
 * Creates a 2D (xym) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 2D (xym) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
vanuatu_multipoint_xym (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomCollXYM ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 2D (xym) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomCollXYM (geom, p->X, p->Y, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/* 
 * Creates a 3D (xyzm) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points given to the function are 3D (xyzm) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
vanuatu_multipoint_xyzm (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomCollXYZM ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 3D (xyzm) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomCollXYZM (geom, p->X, p->Y, p->Z, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/*
 * Creates a geometry collection containing 2D (xy) linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 2D (xy) linestrings. There must be at least 1 linestring in the list.
 * Returns a pointer to the created geometry collection of 2D linestrings. The geometry must have FirstLinestring pointing to the
 * first linestring in the list pointed by first and LastLinestring pointing to the last element of the same list. DimensionModel
 * must be GAIA_XY and DimensionType must be *GAIA_TYPE_LINESTRING.
 */

static gaiaGeomCollPtr
vanuatu_multilinestring_xy (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomColl ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }

    return a;
}

/* 
 * Returns a geometry collection containing the created multilinestring object (?).
 * Creates a geometry collection containing 3D (xyz) linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 3D (xyz) linestrings. There must be at least 1 linestring in the list.
 * Returns a pointer to the created geometry collection of 3D linestrings. The geometry must have FirstLinestring pointing to the
 * first linestring in the *list pointed by first and LastLinestring pointing to the last element of the same list. DimensionModel
 * must be GAIA_XYZ and DimensionType must be *GAIA_TYPE_LINESTRING.
 */
static gaiaGeomCollPtr
vanuatu_multilinestring_xyz (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomCollXYZ ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY_Z;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }
    return a;
}

/* 
 * Creates a geometry collection containing 2D (xy) with m value linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the  *linestrings in the list must be 2D (xy) with m value linestrings. There must be at least 1 linestring 
 * in the list.
 * Returns a pointer to the created geometry collection of 2D with m value linestrings. The geometry must have FirstLinestring
 * pointing to the first *linestring in the list pointed by first and LastLinestring pointing to the last element of the same list. 
 * DimensionModel must be GAIA_XYM and *DimensionType must be GAIA_TYPE_LINESTRING.
 */
static gaiaGeomCollPtr
vanuatu_multilinestring_xym (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomCollXYM ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY_M;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }

    return a;
}

/* 
 * Creates a geometry collection containing 4D (xyz) with m value linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 4D (xyz) with m value linestrings. There must be at least 1 linestring
 * in the list.
 * Returns a pointer to the created geometry collection of 4D with m value linestrings. The geometry must have FirstLinestring
 * pointing to the first *linestring in the list pointed by first and LastLinestring pointing to the last element of the same list. 
 * DimensionModel must be GAIA_XYZM and *DimensionType must be GAIA_TYPE_LINESTRING.
 */
static gaiaGeomCollPtr
vanuatu_multilinestring_xyzm (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomCollXYZM ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY_Z_M;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }
    return a;
}

/* 
 * Creates a geometry collection containing 2D (xy) polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should 
 * be added to the collection. All of the polygons in the list must be 2D (xy) polygons. There must be 
 * at least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 2D polygons. The geometry must have 
 * FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing 
 * to the last element of the same list. DimensionModel must be GAIA_XY and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
vanuatu_multipolygon_xy (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

/* 
 * Creates a geometry collection containing 3D (xyz) polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should be 
 * added to the collection. All of the polygons in the list must be 3D (xyz) polygons. There must be at
 * least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 3D polygons. The geometry must have 
 * FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing to 
 * the last element of the same list. DimensionModel must be GAIA_XYZ and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
vanuatu_multipolygon_xyz (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomCollXYZ ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

/* 
 * Creates a geometry collection containing 2D (xy) with m value polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should
 * be added to the collection. All of the polygons in the list must be 2D (xy) with m value polygons.
 * There must be at least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 2D with m value polygons. The geometry 
 * must have FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon 
 * pointing to the last element of the same list. DimensionModel must be GAIA_XYM and DimensionType 
 * must be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
vanuatu_multipolygon_xym (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomCollXYM ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

/*
 * Creates a geometry collection containing 4D (xyz) with m value polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should be 
 * added to the collection. All of the polygons in the list must be 4D (xyz) with m value polygons. 
 * There must be at least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 4D with m value polygons. The geometry must 
 * have FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing 
//  * to the last element of the same list. DimensionModel must be GAIA_XYZM and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
vanuatu_multipolygon_xyzm (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomCollXYZM ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

static void
vanuatu_geomColl_common (gaiaGeomCollPtr org, gaiaGeomCollPtr dst)
{
/* 
/ helper function: xfers entities between the Origin and Destination 
/ Sandro Furieri: 2010 October 12
*/
    gaiaGeomCollPtr p = org;
    gaiaGeomCollPtr p_n;
    gaiaPointPtr pt;
    gaiaPointPtr pt_n;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr ln_n;
    gaiaPolygonPtr pg;
    gaiaPolygonPtr pg_n;
    while (p)
      {
	  pt = p->FirstPoint;
	  while (pt)
	    {
		pt_n = pt->Next;
		pt->Next = NULL;
		if (dst->FirstPoint == NULL)
		    dst->FirstPoint = pt;
		if (dst->LastPoint != NULL)
		    dst->LastPoint->Next = pt;
		dst->LastPoint = pt;
		pt = pt_n;
	    }
	  ln = p->FirstLinestring;
	  while (ln)
	    {
		ln_n = ln->Next;
		ln->Next = NULL;
		if (dst->FirstLinestring == NULL)
		    dst->FirstLinestring = ln;
		if (dst->LastLinestring != NULL)
		    dst->LastLinestring->Next = ln;
		dst->LastLinestring = ln;
		ln = ln_n;
	    }
	  pg = p->FirstPolygon;
	  while (pg)
	    {
		pg_n = pg->Next;
		pg->Next = NULL;
		if (dst->FirstPolygon == NULL)
		    dst->FirstPolygon = pg;
		if (dst->LastPolygon != NULL)
		    dst->LastPolygon->Next = pg;
		dst->LastPolygon = pg;
		pg = pg_n;
	    }
	  p_n = p->Next;
	  p->FirstPoint = NULL;
	  p->LastPoint = NULL;
	  p->FirstLinestring = NULL;
	  p->LastLinestring = NULL;
	  p->FirstPolygon = NULL;
	  p->LastPolygon = NULL;
	  gaiaFreeGeomColl (p);
	  p = p_n;
      }
}

/* Creates a 2D (xy) geometry collection in SpatiaLite
 *
 * first is the first geometry collection in a linked list of geometry collections.
 * Each geometry collection represents a single type of object (eg. one could be a POINT, 
 * another could be a LINESTRING, another could be a MULTILINESTRING, etc.).
 *
 * The type of object represented by any geometry collection is stored in the declaredType 
 * field of its struct. For example, if first->declaredType = GAIA_POINT, then first represents a point.
 * If first->declaredType = GAIA_MULTIPOINT, then first represents a multipoint.
 *
 * NOTE: geometry collections cannot contain other geometry collections (have to confirm this 
 * with Sandro).
 *
 * The goal of this function is to take the information from all of the structs in the linked list and 
 * return one geomColl struct containing all of that information.
 *
 * The integers used for 'declaredType' are defined in gaiageo.h. In this function, the only values 
 * contained in 'declaredType' that will be encountered will be:
 *
 *	GAIA_POINT, GAIA_LINESTRING, GAIA_POLYGON, 
 *	GAIA_MULTIPOINT, GAIA_MULTILINESTRING, GAIA_MULTIPOLYGON
 */
static gaiaGeomCollPtr
vanuatu_geomColl_xy (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY;
    vanuatu_geomColl_common (first, geom);
    return geom;
}

/*
 * See geomColl_xy for description.
 *
 * The only difference between this function and geomColl_xy is that the 'declaredType' field of the structs
 * in the linked list for this function will only contain the following types:
 *
 *	GAIA_POINTZ, GAIA_LINESTRINGZ, GAIA_POLYGONZ,
 * 	GAIA_MULTIPOINTZ, GAIA_MULTILINESTRINGZ, GAIA_MULTIPOLYGONZ
 */
static gaiaGeomCollPtr
vanuatu_geomColl_xyz (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY_Z;
    vanuatu_geomColl_common (first, geom);
    return geom;
}

/*
 * See geomColl_xy for description.
 *
 * The only difference between this function and geomColl_xy is that the 'declaredType' field of the structs
 * in the linked list for this function will only contain the following types:
 *
 *	GAIA_POINTM, GAIA_LINESTRINGM, GAIA_POLYGONM,
 * 	GAIA_MULTIPOINTM, GAIA_MULTILINESTRINGM, GAIA_MULTIPOLYGONM
 */
static gaiaGeomCollPtr
vanuatu_geomColl_xym (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY_M;
    vanuatu_geomColl_common (first, geom);
    return geom;
}

/*
 * See geomColl_xy for description.
 *
 * The only difference between this function and geomColl_xy is that the 'declaredType' field of the structs
 * in the linked list for this function will only contain the following types:
 *
 *	GAIA_POINTZM, GAIA_LINESTRINGZM, GAIA_POLYGONZM,
 * 	GAIA_MULTIPOINTZM, GAIA_MULTILINESTRINGZM, GAIA_MULTIPOLYGONZM
 */
static gaiaGeomCollPtr
vanuatu_geomColl_xyzm (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY_Z_M;
    vanuatu_geomColl_common (first, geom);
    return geom;
}



/*
** CAVEAT: we must redefine any Lemon/Flex own macro
*/
#define YYMINORTYPE		VANUATU_MINORTYPE
#define YY_CHAR			VANUATU_YY_CHAR
#define	input			vanuatu_input
#define ParseAlloc		vanuatuParseAlloc
#define ParseFree		vanuatuParseFree
#define ParseStackPeak		vanuatuParseStackPeak
#define Parse			vanuatuParse
#define yyStackEntry		vanuatu_yyStackEntry
#define yyzerominor		vanuatu_yyzerominor
#define yy_accept		vanuatu_yy_accept
#define yy_action		vanuatu_yy_action
#define yy_base			vanuatu_yy_base
#define yy_buffer_stack		vanuatu_yy_buffer_stack
#define yy_buffer_stack_max	vanuatu_yy_buffer_stack_max
#define yy_buffer_stack_top	vanuatu_yy_buffer_stack_top
#define yy_c_buf_p		vanuatu_yy_c_buf_p
#define yy_chk			vanuatu_yy_chk
#define yy_def			vanuatu_yy_def
#define yy_default		vanuatu_yy_default
#define yy_destructor		vanuatu_yy_destructor
#define yy_ec			vanuatu_yy_ec
#define yy_fatal_error		vanuatu_yy_fatal_error
#define yy_find_reduce_action	vanuatu_yy_find_reduce_action
#define yy_find_shift_action	vanuatu_yy_find_shift_action
#define yy_get_next_buffer	vanuatu_yy_get_next_buffer
#define yy_get_previous_state	vanuatu_yy_get_previous_state
#define yy_init			vanuatu_yy_init
#define yy_init_globals		vanuatu_yy_init_globals
#define yy_lookahead		vanuatu_yy_lookahead
#define yy_meta			vanuatu_yy_meta
#define yy_nxt			vanuatu_yy_nxt
#define yy_parse_failed		vanuatu_yy_parse_failed
#define yy_pop_parser_stack	vanuatu_yy_pop_parser_stack
#define yy_reduce		vanuatu_yy_reduce
#define yy_reduce_ofst		vanuatu_yy_reduce_ofst
#define yy_shift		vanuatu_yy_shift
#define yy_shift_ofst		vanuatu_yy_shift_ofst
#define yy_start		vanuatu_yy_start
#define yy_state_type		vanuatu_yy_state_type
#define yy_syntax_error		vanuatu_yy_syntax_error
#define yy_trans_info		vanuatu_yy_trans_info
#define yy_try_NUL_trans	vanuatu_yy_try_NUL_trans
#define yyParser		vanuatu_yyParser
#define yyStackEntry		vanuatu_yyStackEntry
#define yyStackOverflow		vanuatu_yyStackOverflow
#define yyRuleInfo		vanuatu_yyRuleInfo
#define yyunput			vanuatu_yyunput
#define yyzerominor		vanuatu_yyzerominor
#define yyTraceFILE		vanuatu_yyTraceFILE
#define yyTracePrompt		vanuatu_yyTracePrompt
#define yyTokenName		vanuatu_yyTokenName
#define yyRuleName		vanuatu_yyRuleName
#define ParseTrace		vanuatu_ParseTrace


/*
 VANUATU_LEMON_H_START - LEMON generated header starts here 
*/
#define VANUATU_NEWLINE                 1
#define VANUATU_POINT                   2
#define VANUATU_OPEN_BRACKET            3
#define VANUATU_CLOSE_BRACKET           4
#define VANUATU_POINT_M                 5
#define VANUATU_POINT_Z                 6
#define VANUATU_POINT_ZM                7
#define VANUATU_NUM                     8
#define VANUATU_COMMA                   9
#define VANUATU_LINESTRING             10
#define VANUATU_LINESTRING_M           11
#define VANUATU_LINESTRING_Z           12
#define VANUATU_LINESTRING_ZM          13
#define VANUATU_POLYGON                14
#define VANUATU_POLYGON_M              15
#define VANUATU_POLYGON_Z              16
#define VANUATU_POLYGON_ZM             17
#define VANUATU_MULTIPOINT             18
#define VANUATU_MULTIPOINT_M           19
#define VANUATU_MULTIPOINT_Z           20
#define VANUATU_MULTIPOINT_ZM          21
#define VANUATU_MULTILINESTRING        22
#define VANUATU_MULTILINESTRING_M      23
#define VANUATU_MULTILINESTRING_Z      24
#define VANUATU_MULTILINESTRING_ZM     25
#define VANUATU_MULTIPOLYGON           26
#define VANUATU_MULTIPOLYGON_M         27
#define VANUATU_MULTIPOLYGON_Z         28
#define VANUATU_MULTIPOLYGON_ZM        29
#define VANUATU_GEOMETRYCOLLECTION     30
#define VANUATU_GEOMETRYCOLLECTION_M   31
#define VANUATU_GEOMETRYCOLLECTION_Z   32
#define VANUATU_GEOMETRYCOLLECTION_ZM  33
/*
 VANUATU_LEMON_H_END - LEMON generated header ends here 
*/



typedef union
{
    double dval;
    struct symtab *symp;
} vanuatu_yystype;
#define YYSTYPE vanuatu_yystype

/* extern YYSTYPE yylval; */
YYSTYPE VanuatuWktlval;



/*
 VANUATU_LEMON_START - LEMON generated code starts here 
*/

/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is included that follows the "include" declaration
** in the input grammar file. */
/* #include <stdio.h> */
#line 77 "vanuatuWkt.y"

#line 10 "vanuatuWkt.c"
/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash 
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    ParseTOKENTYPE     is the data type used for minor tokens given 
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is ParseTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    ParseARG_SDECL     A static variable declaration for the %extra_argument
**    ParseARG_PDECL     A parameter declaration for the %extra_argument
**    ParseARG_STORE     Code to store %extra_argument into yypParser
**    ParseARG_FETCH     Code to extract %extra_argument from yypParser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 125
#define YYACTIONTYPE unsigned short int
#define ParseTOKENTYPE void *
typedef union {
  int yyinit;
  ParseTOKENTYPE yy0;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 1000000
#endif
#define ParseARG_SDECL  gaiaGeomCollPtr *result ;
#define ParseARG_PDECL , gaiaGeomCollPtr *result 
#define ParseARG_FETCH  gaiaGeomCollPtr *result  = yypParser->result 
#define ParseARG_STORE yypParser->result  = result 
#define YYNSTATE 358
#define YYNRULE 153
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* The yyzerominor constant is used to initialize instances of
** YYMINORTYPE objects to zero. */
static const YYMINORTYPE yyzerominor = { 0 };

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
static const YYACTIONTYPE yy_action[] = {
 /*     0 */   166,  228,  229,  230,  231,  232,  233,  234,  235,  236,
 /*    10 */   237,  238,  239,  240,  241,  242,  243,  244,  245,  246,
 /*    20 */   247,  248,  249,  250,  251,  252,  253,  254,  255,  256,
 /*    30 */   257,  258,  259,  260,  358,  263,  167,  512,    1,  169,
 /*    40 */   171,  173,  174,   51,   54,   57,   60,   63,   66,   72,
 /*    50 */    78,   84,   90,   92,   94,   96,   98,  103,  108,  113,
 /*    60 */   118,  123,  128,  133,  138,  145,  152,  159,  167,  139,
 /*    70 */   143,  144,  140,  141,  142,  169,   54,  146,  150,  151,
 /*    80 */    66,   57,  147,  148,  149,   72,  171,  153,  157,  158,
 /*    90 */   170,  262,   60,   47,  173,  270,   78,  154,  155,  156,
 /*   100 */    63,  172,  273,   49,   84,  160,  164,  165,   48,  161,
 /*   110 */   162,  163,   14,  168,  175,   55,  176,   46,   46,   46,
 /*   120 */    56,  265,  177,   58,   46,   47,   47,   59,   50,  179,
 /*   130 */    47,   49,   61,   62,   49,   49,  181,   51,   64,   51,
 /*   140 */   184,   65,   51,  185,   46,  186,   70,   46,  189,   46,
 /*   150 */    46,   47,  267,  190,  191,   76,   47,   47,   47,  194,
 /*   160 */    52,   49,  195,  196,   49,   49,   82,   53,   49,  199,
 /*   170 */    51,  200,   51,  201,   51,   88,   51,   91,   95,   93,
 /*   180 */    49,   46,   47,  269,   97,   51,   16,  271,   17,   19,
 /*   190 */   178,  274,   20,   22,  276,  180,  277,   23,  279,   25,
 /*   200 */   182,  280,   67,  282,   26,   69,  187,   73,   68,  286,
 /*   210 */    30,   75,  183,   71,  192,  285,   79,   74,  290,  188,
 /*   220 */   289,   77,   34,   81,  197,   80,   85,  193,   83,  294,
 /*   230 */    38,  293,  202,   86,   87,  198,  297,   89,   42,  203,
 /*   240 */   298,   43,  300,  204,   44,  302,  205,   45,  304,  206,
 /*   250 */    99,  306,  100,  102,  101,  104,  207,  309,  308,  105,
 /*   260 */   106,  109,  208,  107,  311,  110,  312,  209,  111,  112,
 /*   270 */   114,  314,  315,  116,  115,  117,  119,  120,  210,  121,
 /*   280 */   124,  317,  122,  318,  126,  125,  129,  131,  211,  130,
 /*   290 */   321,  320,  132,  127,  134,  136,    2,  135,    3,  212,
 /*   300 */     4,  323,  137,    5,    6,  324,    8,    7,    9,  227,
 /*   310 */    10,  213,  513,  261,   11,   15,  264,   12,   13,  326,
 /*   320 */   327,  266,  268,  272,  275,   18,  214,  329,  278,  281,
 /*   330 */    21,   24,  330,  283,  513,   27,   28,  215,  332,   29,
 /*   340 */   333,  334,  216,  337,  217,  284,  287,   31,   32,  218,
 /*   350 */   339,  340,  341,  219,  220,  344,  221,   33,  346,  288,
 /*   360 */   291,  347,  348,   35,  222,  223,   36,  351,   37,  224,
 /*   370 */   292,  295,  353,  354,   39,  513,  355,  225,  226,   40,
 /*   380 */    41,  296,  299,  301,  303,  305,  307,  310,  313,  316,
 /*   390 */   319,  322,  325,  328,  331,  335,  336,  338,  342,  343,
 /*   400 */   345,  349,  350,  352,  356,  357,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */    37,   38,   39,   40,   41,   42,   43,   44,   45,   46,
 /*    10 */    47,   48,   49,   50,   51,   52,   53,   54,   55,   56,
 /*    20 */    57,   58,   59,   60,   61,   62,   63,   64,   65,   66,
 /*    30 */    67,   68,   69,   70,    0,    8,    2,   35,   36,    5,
 /*    40 */     6,    7,   74,   75,   10,   11,   12,   13,   14,   15,
 /*    50 */    16,   17,   18,   19,   20,   21,   22,   23,   24,   25,
 /*    60 */    26,   27,   28,   29,   30,   31,   32,   33,    2,   43,
 /*    70 */    44,   45,   43,   44,   45,    5,   10,   57,   58,   59,
 /*    80 */    14,   11,   57,   58,   59,   15,    6,   50,   51,   52,
 /*    90 */    72,   75,   12,   75,    7,   80,   16,   50,   51,   52,
 /*   100 */    13,   73,   81,   75,   17,   64,   65,   66,   75,   64,
 /*   110 */    65,   66,    3,   71,   71,   71,   76,   75,   75,   75,
 /*   120 */    71,   75,   72,   72,   75,   75,   75,   72,   75,   73,
 /*   130 */    75,   75,   73,   73,   75,   75,   74,   75,   74,   75,
 /*   140 */    71,   74,   75,   71,   75,   71,   71,   75,   72,   75,
 /*   150 */    75,   75,   75,   72,   72,   72,   75,   75,   75,   73,
 /*   160 */    75,   75,   73,   73,   75,   75,   73,   75,   75,   74,
 /*   170 */    75,   74,   75,   74,   75,   74,   75,   71,   73,   72,
 /*   180 */    75,   75,   75,   75,   74,   75,    9,   76,    3,    9,
 /*   190 */    77,   77,    3,    9,   82,   78,   78,    3,   83,    9,
 /*   200 */    79,   79,    3,   84,    3,    9,   76,    3,   88,   85,
 /*   210 */     3,    9,   89,   88,   77,   89,    3,   90,   86,   91,
 /*   220 */    91,   90,    3,    9,   78,   92,    3,   93,   92,   87,
 /*   230 */     3,   93,   79,   94,    9,   95,   95,   94,    3,   76,
 /*   240 */    96,    3,   97,   77,    3,   98,   78,    3,   99,   79,
 /*   250 */     3,  100,   80,   80,    9,    3,  104,  101,  104,   81,
 /*   260 */     9,    3,  105,   81,  105,   82,  102,  106,    9,   82,
 /*   270 */     3,  106,  103,    9,   83,   83,    3,   84,  107,    9,
 /*   280 */     3,  107,   84,  108,    9,   85,    3,    9,  112,   86,
 /*   290 */   109,  112,   86,   85,    3,    9,    3,   87,    9,  113,
 /*   300 */     3,  113,   87,    9,    3,  110,    3,    9,    9,    1,
 /*   310 */     3,  114,  124,    4,    3,    9,    4,    3,    3,  114,
 /*   320 */   111,    4,    4,    4,    4,    9,  115,  115,    4,    4,
 /*   330 */     9,    9,  116,    4,  124,    9,    9,  120,  120,    9,
 /*   340 */   120,  120,  120,  117,  120,    4,    4,    9,    9,  121,
 /*   350 */   121,  121,  121,  121,  121,  118,  122,    9,  122,    4,
 /*   360 */     4,  122,  122,    9,  122,  122,    9,  119,    9,  123,
 /*   370 */     4,    4,  123,  123,    9,  124,  123,  123,  123,    9,
 /*   380 */     9,    4,    4,    4,    4,    4,    4,    4,    4,    4,
 /*   390 */     4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
 /*   400 */     4,    4,    4,    4,    4,    4,
};
#define YY_SHIFT_USE_DFLT (-1)
#define YY_SHIFT_MAX 226
static const short yy_shift_ofst[] = {
 /*     0 */    -1,   34,   66,   66,   70,   70,   80,   80,   87,   87,
 /*    10 */    27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    20 */    27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    30 */    27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    40 */    27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    50 */    27,   27,   27,   27,  109,  177,  177,  185,  180,  180,
 /*    60 */   189,  184,  184,  194,  190,  190,  199,  201,  196,  201,
 /*    70 */   177,  196,  204,  207,  202,  207,  180,  202,  213,  219,
 /*    80 */   214,  219,  184,  214,  223,  227,  225,  227,  190,  225,
 /*    90 */   235,  177,  238,  180,  241,  184,  244,  190,  247,  109,
 /*   100 */   245,  109,  245,  252,  185,  251,  185,  251,  258,  189,
 /*   110 */   259,  189,  259,  267,  194,  264,  194,  264,  273,  199,
 /*   120 */   270,  199,  270,  277,  204,  275,  204,  275,  283,  213,
 /*   130 */   278,  213,  278,  291,  223,  286,  223,  286,  293,  289,
 /*   140 */   289,  289,  289,  289,  289,  297,  294,  294,  294,  294,
 /*   150 */   294,  294,  301,  298,  298,  298,  298,  298,  298,  303,
 /*   160 */   299,  299,  299,  299,  299,  299,  308,  307,  309,  311,
 /*   170 */   312,  314,  317,  315,  318,  306,  319,  316,  320,  321,
 /*   180 */   324,  322,  325,  329,  326,  327,  330,  341,  342,  338,
 /*   190 */   339,  348,  355,  356,  354,  357,  359,  366,  367,  365,
 /*   200 */   370,  371,  377,  378,  379,  380,  381,  382,  383,  384,
 /*   210 */   385,  386,  387,  388,  389,  390,  391,  392,  393,  394,
 /*   220 */   395,  396,  397,  398,  399,  400,  401,
};
#define YY_REDUCE_USE_DFLT (-38)
#define YY_REDUCE_MAX 165
static const short yy_reduce_ofst[] = {
 /*     0 */     2,  -37,   26,   29,   20,   25,   37,   47,   41,   45,
 /*    10 */    42,   18,   28,  -32,   43,   44,   49,   50,   51,   55,
 /*    20 */    56,   59,   60,   62,   64,   67,   69,   72,   74,   75,
 /*    30 */    76,   81,   82,   83,   86,   89,   90,   93,   95,   97,
 /*    40 */    99,  101,  106,  107,  105,  110,   16,   33,   46,   53,
 /*    50 */    77,   85,   92,  108,   15,   40,  111,   21,  113,  114,
 /*    60 */   112,  117,  118,  115,  121,  122,  119,  120,  123,  125,
 /*    70 */   130,  126,  124,  127,  128,  131,  137,  129,  132,  133,
 /*    80 */   134,  136,  146,  138,  142,  139,  140,  143,  153,  141,
 /*    90 */   144,  163,  145,  166,  147,  168,  149,  170,  151,  172,
 /*   100 */   152,  173,  154,  156,  178,  157,  182,  159,  164,  183,
 /*   110 */   161,  187,  165,  169,  191,  171,  192,  174,  175,  193,
 /*   120 */   176,  198,  179,  181,  200,  186,  208,  188,  195,  203,
 /*   130 */   197,  206,  205,  209,  210,  211,  215,  212,  216,  217,
 /*   140 */   218,  220,  221,  222,  224,  226,  228,  229,  230,  231,
 /*   150 */   232,  233,  237,  234,  236,  239,  240,  242,  243,  248,
 /*   160 */   246,  249,  250,  253,  254,  255,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */   359,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*    10 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*    20 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*    30 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*    40 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*    50 */   511,  511,  511,  511,  511,  403,  403,  511,  405,  405,
 /*    60 */   511,  407,  407,  511,  409,  409,  511,  511,  428,  511,
 /*    70 */   403,  428,  511,  511,  431,  511,  405,  431,  511,  511,
 /*    80 */   434,  511,  407,  434,  511,  511,  437,  511,  409,  437,
 /*    90 */   511,  403,  511,  405,  511,  407,  511,  409,  511,  511,
 /*   100 */   452,  511,  452,  511,  511,  455,  511,  455,  511,  511,
 /*   110 */   458,  511,  458,  511,  511,  461,  511,  461,  511,  511,
 /*   120 */   468,  511,  468,  511,  511,  471,  511,  471,  511,  511,
 /*   130 */   474,  511,  474,  511,  511,  477,  511,  477,  511,  486,
 /*   140 */   486,  486,  486,  486,  486,  511,  493,  493,  493,  493,
 /*   150 */   493,  493,  511,  500,  500,  500,  500,  500,  500,  511,
 /*   160 */   507,  507,  507,  507,  507,  507,  511,  511,  511,  511,
 /*   170 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*   180 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*   190 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*   200 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*   210 */   511,  511,  511,  511,  511,  511,  511,  511,  511,  511,
 /*   220 */   511,  511,  511,  511,  511,  511,  511,  360,  361,  362,
 /*   230 */   363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
 /*   240 */   373,  374,  375,  376,  377,  378,  379,  380,  381,  382,
 /*   250 */   383,  384,  385,  386,  387,  388,  389,  390,  391,  392,
 /*   260 */   393,  394,  398,  402,  395,  399,  396,  400,  397,  401,
 /*   270 */   411,  404,  415,  412,  406,  416,  413,  408,  417,  414,
 /*   280 */   410,  418,  419,  423,  427,  429,  420,  424,  430,  432,
 /*   290 */   421,  425,  433,  435,  422,  426,  436,  438,  439,  443,
 /*   300 */   440,  444,  441,  445,  442,  446,  447,  451,  453,  448,
 /*   310 */   454,  456,  449,  457,  459,  450,  460,  462,  463,  467,
 /*   320 */   469,  464,  470,  472,  465,  473,  475,  466,  476,  478,
 /*   330 */   479,  483,  487,  488,  489,  484,  485,  480,  490,  494,
 /*   340 */   495,  496,  491,  492,  481,  497,  501,  502,  503,  498,
 /*   350 */   499,  482,  504,  508,  509,  510,  505,  506,
};
#define YY_SZ_ACTTAB (int)(sizeof(yy_action)/sizeof(yy_action[0]))

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  int yyidx;                    /* Index of top element in stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyidxMax;                 /* Maximum value of yyidx */
#endif
  int yyerrcnt;                 /* Shifts left before out of the error */
  ParseARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
/* #include <stdio.h> */
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void ParseTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "VANUATU_NEWLINE",  "VANUATU_POINT",  "VANUATU_OPEN_BRACKET",
  "VANUATU_CLOSE_BRACKET",  "VANUATU_POINT_M",  "VANUATU_POINT_Z",  "VANUATU_POINT_ZM",
  "VANUATU_NUM",   "VANUATU_COMMA",  "VANUATU_LINESTRING",  "VANUATU_LINESTRING_M",
  "VANUATU_LINESTRING_Z",  "VANUATU_LINESTRING_ZM",  "VANUATU_POLYGON",  "VANUATU_POLYGON_M",
  "VANUATU_POLYGON_Z",  "VANUATU_POLYGON_ZM",  "VANUATU_MULTIPOINT",  "VANUATU_MULTIPOINT_M",
  "VANUATU_MULTIPOINT_Z",  "VANUATU_MULTIPOINT_ZM",  "VANUATU_MULTILINESTRING",  "VANUATU_MULTILINESTRING_M",
  "VANUATU_MULTILINESTRING_Z",  "VANUATU_MULTILINESTRING_ZM",  "VANUATU_MULTIPOLYGON",  "VANUATU_MULTIPOLYGON_M",
  "VANUATU_MULTIPOLYGON_Z",  "VANUATU_MULTIPOLYGON_ZM",  "VANUATU_GEOMETRYCOLLECTION",  "VANUATU_GEOMETRYCOLLECTION_M",
  "VANUATU_GEOMETRYCOLLECTION_Z",  "VANUATU_GEOMETRYCOLLECTION_ZM",  "error",         "main",        
  "in",            "state",         "program",       "geo_text",    
  "geo_textz",     "geo_textm",     "geo_textzm",    "point",       
  "linestring",    "polygon",       "multipoint",    "multilinestring",
  "multipolygon",  "geocoll",       "pointz",        "linestringz", 
  "polygonz",      "multipointz",   "multilinestringz",  "multipolygonz",
  "geocollz",      "pointm",        "linestringm",   "polygonm",    
  "multipointm",   "multilinestringm",  "multipolygonm",  "geocollm",    
  "pointzm",       "linestringzm",  "polygonzm",     "multipointzm",
  "multilinestringzm",  "multipolygonzm",  "geocollzm",     "point_coordxy",
  "point_coordxym",  "point_coordxyz",  "point_coordxyzm",  "coord",       
  "extra_pointsxy",  "extra_pointsxym",  "extra_pointsxyz",  "extra_pointsxyzm",
  "linestring_text",  "linestring_textm",  "linestring_textz",  "linestring_textzm",
  "polygon_text",  "polygon_textm",  "polygon_textz",  "polygon_textzm",
  "ring",          "extra_rings",   "ringm",         "extra_ringsm",
  "ringz",         "extra_ringsz",  "ringzm",        "extra_ringszm",
  "multipoint_text",  "multipoint_textm",  "multipoint_textz",  "multipoint_textzm",
  "multilinestring_text",  "multilinestring_textm",  "multilinestring_textz",  "multilinestring_textzm",
  "multilinestring_text2",  "multilinestring_textm2",  "multilinestring_textz2",  "multilinestring_textzm2",
  "multipolygon_text",  "multipolygon_textm",  "multipolygon_textz",  "multipolygon_textzm",
  "multipolygon_text2",  "multipolygon_textm2",  "multipolygon_textz2",  "multipolygon_textzm2",
  "geocoll_text",  "geocoll_textm",  "geocoll_textz",  "geocoll_textzm",
  "geocoll_text2",  "geocoll_textm2",  "geocoll_textz2",  "geocoll_textzm2",
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "main ::= in",
 /*   1 */ "in ::=",
 /*   2 */ "in ::= in state VANUATU_NEWLINE",
 /*   3 */ "state ::= program",
 /*   4 */ "program ::= geo_text",
 /*   5 */ "program ::= geo_textz",
 /*   6 */ "program ::= geo_textm",
 /*   7 */ "program ::= geo_textzm",
 /*   8 */ "geo_text ::= point",
 /*   9 */ "geo_text ::= linestring",
 /*  10 */ "geo_text ::= polygon",
 /*  11 */ "geo_text ::= multipoint",
 /*  12 */ "geo_text ::= multilinestring",
 /*  13 */ "geo_text ::= multipolygon",
 /*  14 */ "geo_text ::= geocoll",
 /*  15 */ "geo_textz ::= pointz",
 /*  16 */ "geo_textz ::= linestringz",
 /*  17 */ "geo_textz ::= polygonz",
 /*  18 */ "geo_textz ::= multipointz",
 /*  19 */ "geo_textz ::= multilinestringz",
 /*  20 */ "geo_textz ::= multipolygonz",
 /*  21 */ "geo_textz ::= geocollz",
 /*  22 */ "geo_textm ::= pointm",
 /*  23 */ "geo_textm ::= linestringm",
 /*  24 */ "geo_textm ::= polygonm",
 /*  25 */ "geo_textm ::= multipointm",
 /*  26 */ "geo_textm ::= multilinestringm",
 /*  27 */ "geo_textm ::= multipolygonm",
 /*  28 */ "geo_textm ::= geocollm",
 /*  29 */ "geo_textzm ::= pointzm",
 /*  30 */ "geo_textzm ::= linestringzm",
 /*  31 */ "geo_textzm ::= polygonzm",
 /*  32 */ "geo_textzm ::= multipointzm",
 /*  33 */ "geo_textzm ::= multilinestringzm",
 /*  34 */ "geo_textzm ::= multipolygonzm",
 /*  35 */ "geo_textzm ::= geocollzm",
 /*  36 */ "point ::= VANUATU_POINT VANUATU_OPEN_BRACKET point_coordxy VANUATU_CLOSE_BRACKET",
 /*  37 */ "pointm ::= VANUATU_POINT_M VANUATU_OPEN_BRACKET point_coordxym VANUATU_CLOSE_BRACKET",
 /*  38 */ "pointz ::= VANUATU_POINT_Z VANUATU_OPEN_BRACKET point_coordxyz VANUATU_CLOSE_BRACKET",
 /*  39 */ "pointzm ::= VANUATU_POINT_ZM VANUATU_OPEN_BRACKET point_coordxyzm VANUATU_CLOSE_BRACKET",
 /*  40 */ "point_coordxy ::= coord coord",
 /*  41 */ "point_coordxym ::= coord coord coord",
 /*  42 */ "point_coordxyz ::= coord coord coord",
 /*  43 */ "point_coordxyzm ::= coord coord coord coord",
 /*  44 */ "coord ::= VANUATU_NUM",
 /*  45 */ "extra_pointsxy ::=",
 /*  46 */ "extra_pointsxy ::= VANUATU_COMMA point_coordxy extra_pointsxy",
 /*  47 */ "extra_pointsxym ::=",
 /*  48 */ "extra_pointsxym ::= VANUATU_COMMA point_coordxym extra_pointsxym",
 /*  49 */ "extra_pointsxyz ::=",
 /*  50 */ "extra_pointsxyz ::= VANUATU_COMMA point_coordxyz extra_pointsxyz",
 /*  51 */ "extra_pointsxyzm ::=",
 /*  52 */ "extra_pointsxyzm ::= VANUATU_COMMA point_coordxyzm extra_pointsxyzm",
 /*  53 */ "linestring ::= VANUATU_LINESTRING linestring_text",
 /*  54 */ "linestringm ::= VANUATU_LINESTRING_M linestring_textm",
 /*  55 */ "linestringz ::= VANUATU_LINESTRING_Z linestring_textz",
 /*  56 */ "linestringzm ::= VANUATU_LINESTRING_ZM linestring_textzm",
 /*  57 */ "linestring_text ::= VANUATU_OPEN_BRACKET point_coordxy VANUATU_COMMA point_coordxy extra_pointsxy VANUATU_CLOSE_BRACKET",
 /*  58 */ "linestring_textm ::= VANUATU_OPEN_BRACKET point_coordxym VANUATU_COMMA point_coordxym extra_pointsxym VANUATU_CLOSE_BRACKET",
 /*  59 */ "linestring_textz ::= VANUATU_OPEN_BRACKET point_coordxyz VANUATU_COMMA point_coordxyz extra_pointsxyz VANUATU_CLOSE_BRACKET",
 /*  60 */ "linestring_textzm ::= VANUATU_OPEN_BRACKET point_coordxyzm VANUATU_COMMA point_coordxyzm extra_pointsxyzm VANUATU_CLOSE_BRACKET",
 /*  61 */ "polygon ::= VANUATU_POLYGON polygon_text",
 /*  62 */ "polygonm ::= VANUATU_POLYGON_M polygon_textm",
 /*  63 */ "polygonz ::= VANUATU_POLYGON_Z polygon_textz",
 /*  64 */ "polygonzm ::= VANUATU_POLYGON_ZM polygon_textzm",
 /*  65 */ "polygon_text ::= VANUATU_OPEN_BRACKET ring extra_rings VANUATU_CLOSE_BRACKET",
 /*  66 */ "polygon_textm ::= VANUATU_OPEN_BRACKET ringm extra_ringsm VANUATU_CLOSE_BRACKET",
 /*  67 */ "polygon_textz ::= VANUATU_OPEN_BRACKET ringz extra_ringsz VANUATU_CLOSE_BRACKET",
 /*  68 */ "polygon_textzm ::= VANUATU_OPEN_BRACKET ringzm extra_ringszm VANUATU_CLOSE_BRACKET",
 /*  69 */ "ring ::= VANUATU_OPEN_BRACKET point_coordxy VANUATU_COMMA point_coordxy VANUATU_COMMA point_coordxy VANUATU_COMMA point_coordxy extra_pointsxy VANUATU_CLOSE_BRACKET",
 /*  70 */ "extra_rings ::=",
 /*  71 */ "extra_rings ::= VANUATU_COMMA ring extra_rings",
 /*  72 */ "ringm ::= VANUATU_OPEN_BRACKET point_coordxym VANUATU_COMMA point_coordxym VANUATU_COMMA point_coordxym VANUATU_COMMA point_coordxym extra_pointsxym VANUATU_CLOSE_BRACKET",
 /*  73 */ "extra_ringsm ::=",
 /*  74 */ "extra_ringsm ::= VANUATU_COMMA ringm extra_ringsm",
 /*  75 */ "ringz ::= VANUATU_OPEN_BRACKET point_coordxyz VANUATU_COMMA point_coordxyz VANUATU_COMMA point_coordxyz VANUATU_COMMA point_coordxyz extra_pointsxyz VANUATU_CLOSE_BRACKET",
 /*  76 */ "extra_ringsz ::=",
 /*  77 */ "extra_ringsz ::= VANUATU_COMMA ringz extra_ringsz",
 /*  78 */ "ringzm ::= VANUATU_OPEN_BRACKET point_coordxyzm VANUATU_COMMA point_coordxyzm VANUATU_COMMA point_coordxyzm VANUATU_COMMA point_coordxyzm extra_pointsxyzm VANUATU_CLOSE_BRACKET",
 /*  79 */ "extra_ringszm ::=",
 /*  80 */ "extra_ringszm ::= VANUATU_COMMA ringzm extra_ringszm",
 /*  81 */ "multipoint ::= VANUATU_MULTIPOINT multipoint_text",
 /*  82 */ "multipointm ::= VANUATU_MULTIPOINT_M multipoint_textm",
 /*  83 */ "multipointz ::= VANUATU_MULTIPOINT_Z multipoint_textz",
 /*  84 */ "multipointzm ::= VANUATU_MULTIPOINT_ZM multipoint_textzm",
 /*  85 */ "multipoint_text ::= VANUATU_OPEN_BRACKET point_coordxy extra_pointsxy VANUATU_CLOSE_BRACKET",
 /*  86 */ "multipoint_textm ::= VANUATU_OPEN_BRACKET point_coordxym extra_pointsxym VANUATU_CLOSE_BRACKET",
 /*  87 */ "multipoint_textz ::= VANUATU_OPEN_BRACKET point_coordxyz extra_pointsxyz VANUATU_CLOSE_BRACKET",
 /*  88 */ "multipoint_textzm ::= VANUATU_OPEN_BRACKET point_coordxyzm extra_pointsxyzm VANUATU_CLOSE_BRACKET",
 /*  89 */ "multilinestring ::= VANUATU_MULTILINESTRING multilinestring_text",
 /*  90 */ "multilinestringm ::= VANUATU_MULTILINESTRING_M multilinestring_textm",
 /*  91 */ "multilinestringz ::= VANUATU_MULTILINESTRING_Z multilinestring_textz",
 /*  92 */ "multilinestringzm ::= VANUATU_MULTILINESTRING_ZM multilinestring_textzm",
 /*  93 */ "multilinestring_text ::= VANUATU_OPEN_BRACKET linestring_text multilinestring_text2 VANUATU_CLOSE_BRACKET",
 /*  94 */ "multilinestring_text2 ::=",
 /*  95 */ "multilinestring_text2 ::= VANUATU_COMMA linestring_text multilinestring_text2",
 /*  96 */ "multilinestring_textm ::= VANUATU_OPEN_BRACKET linestring_textm multilinestring_textm2 VANUATU_CLOSE_BRACKET",
 /*  97 */ "multilinestring_textm2 ::=",
 /*  98 */ "multilinestring_textm2 ::= VANUATU_COMMA linestring_textm multilinestring_textm2",
 /*  99 */ "multilinestring_textz ::= VANUATU_OPEN_BRACKET linestring_textz multilinestring_textz2 VANUATU_CLOSE_BRACKET",
 /* 100 */ "multilinestring_textz2 ::=",
 /* 101 */ "multilinestring_textz2 ::= VANUATU_COMMA linestring_textz multilinestring_textz2",
 /* 102 */ "multilinestring_textzm ::= VANUATU_OPEN_BRACKET linestring_textzm multilinestring_textzm2 VANUATU_CLOSE_BRACKET",
 /* 103 */ "multilinestring_textzm2 ::=",
 /* 104 */ "multilinestring_textzm2 ::= VANUATU_COMMA linestring_textzm multilinestring_textzm2",
 /* 105 */ "multipolygon ::= VANUATU_MULTIPOLYGON multipolygon_text",
 /* 106 */ "multipolygonm ::= VANUATU_MULTIPOLYGON_M multipolygon_textm",
 /* 107 */ "multipolygonz ::= VANUATU_MULTIPOLYGON_Z multipolygon_textz",
 /* 108 */ "multipolygonzm ::= VANUATU_MULTIPOLYGON_ZM multipolygon_textzm",
 /* 109 */ "multipolygon_text ::= VANUATU_OPEN_BRACKET polygon_text multipolygon_text2 VANUATU_CLOSE_BRACKET",
 /* 110 */ "multipolygon_text2 ::=",
 /* 111 */ "multipolygon_text2 ::= VANUATU_COMMA polygon_text multipolygon_text2",
 /* 112 */ "multipolygon_textm ::= VANUATU_OPEN_BRACKET polygon_textm multipolygon_textm2 VANUATU_CLOSE_BRACKET",
 /* 113 */ "multipolygon_textm2 ::=",
 /* 114 */ "multipolygon_textm2 ::= VANUATU_COMMA polygon_textm multipolygon_textm2",
 /* 115 */ "multipolygon_textz ::= VANUATU_OPEN_BRACKET polygon_textz multipolygon_textz2 VANUATU_CLOSE_BRACKET",
 /* 116 */ "multipolygon_textz2 ::=",
 /* 117 */ "multipolygon_textz2 ::= VANUATU_COMMA polygon_textz multipolygon_textz2",
 /* 118 */ "multipolygon_textzm ::= VANUATU_OPEN_BRACKET polygon_textzm multipolygon_textzm2 VANUATU_CLOSE_BRACKET",
 /* 119 */ "multipolygon_textzm2 ::=",
 /* 120 */ "multipolygon_textzm2 ::= VANUATU_COMMA polygon_textzm multipolygon_textzm2",
 /* 121 */ "geocoll ::= VANUATU_GEOMETRYCOLLECTION geocoll_text",
 /* 122 */ "geocollm ::= VANUATU_GEOMETRYCOLLECTION_M geocoll_textm",
 /* 123 */ "geocollz ::= VANUATU_GEOMETRYCOLLECTION_Z geocoll_textz",
 /* 124 */ "geocollzm ::= VANUATU_GEOMETRYCOLLECTION_ZM geocoll_textzm",
 /* 125 */ "geocoll_text ::= VANUATU_OPEN_BRACKET point geocoll_text2 VANUATU_CLOSE_BRACKET",
 /* 126 */ "geocoll_text ::= VANUATU_OPEN_BRACKET linestring geocoll_text2 VANUATU_CLOSE_BRACKET",
 /* 127 */ "geocoll_text ::= VANUATU_OPEN_BRACKET polygon geocoll_text2 VANUATU_CLOSE_BRACKET",
 /* 128 */ "geocoll_text2 ::=",
 /* 129 */ "geocoll_text2 ::= VANUATU_COMMA point geocoll_text2",
 /* 130 */ "geocoll_text2 ::= VANUATU_COMMA linestring geocoll_text2",
 /* 131 */ "geocoll_text2 ::= VANUATU_COMMA polygon geocoll_text2",
 /* 132 */ "geocoll_textm ::= VANUATU_OPEN_BRACKET pointm geocoll_textm2 VANUATU_CLOSE_BRACKET",
 /* 133 */ "geocoll_textm ::= VANUATU_OPEN_BRACKET linestringm geocoll_textm2 VANUATU_CLOSE_BRACKET",
 /* 134 */ "geocoll_textm ::= VANUATU_OPEN_BRACKET polygonm geocoll_textm2 VANUATU_CLOSE_BRACKET",
 /* 135 */ "geocoll_textm2 ::=",
 /* 136 */ "geocoll_textm2 ::= VANUATU_COMMA pointm geocoll_textm2",
 /* 137 */ "geocoll_textm2 ::= VANUATU_COMMA linestringm geocoll_textm2",
 /* 138 */ "geocoll_textm2 ::= VANUATU_COMMA polygonm geocoll_textm2",
 /* 139 */ "geocoll_textz ::= VANUATU_OPEN_BRACKET pointz geocoll_textz2 VANUATU_CLOSE_BRACKET",
 /* 140 */ "geocoll_textz ::= VANUATU_OPEN_BRACKET linestringz geocoll_textz2 VANUATU_CLOSE_BRACKET",
 /* 141 */ "geocoll_textz ::= VANUATU_OPEN_BRACKET polygonz geocoll_textz2 VANUATU_CLOSE_BRACKET",
 /* 142 */ "geocoll_textz2 ::=",
 /* 143 */ "geocoll_textz2 ::= VANUATU_COMMA pointz geocoll_textz2",
 /* 144 */ "geocoll_textz2 ::= VANUATU_COMMA linestringz geocoll_textz2",
 /* 145 */ "geocoll_textz2 ::= VANUATU_COMMA polygonz geocoll_textz2",
 /* 146 */ "geocoll_textzm ::= VANUATU_OPEN_BRACKET pointzm geocoll_textzm2 VANUATU_CLOSE_BRACKET",
 /* 147 */ "geocoll_textzm ::= VANUATU_OPEN_BRACKET linestringzm geocoll_textzm2 VANUATU_CLOSE_BRACKET",
 /* 148 */ "geocoll_textzm ::= VANUATU_OPEN_BRACKET polygonzm geocoll_textzm2 VANUATU_CLOSE_BRACKET",
 /* 149 */ "geocoll_textzm2 ::=",
 /* 150 */ "geocoll_textzm2 ::= VANUATU_COMMA pointzm geocoll_textzm2",
 /* 151 */ "geocoll_textzm2 ::= VANUATU_COMMA linestringzm geocoll_textzm2",
 /* 152 */ "geocoll_textzm2 ::= VANUATU_COMMA polygonzm geocoll_textzm2",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.
*/
static void yyGrowStack(yyParser *p){
  int newSize;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  if( pNew ){
    p->yystack = pNew;
    p->yystksz = newSize;
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows to %d entries!\n",
              yyTracePrompt, p->yystksz);
    }
#endif
  }
}
#endif

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Parse and ParseFree.
*/
void *ParseAlloc(void *(*mallocProc)(size_t)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (size_t)sizeof(yyParser) );
  if( pParser ){
    pParser->yyidx = -1;
#ifdef YYTRACKMAXSTACKDEPTH
    pParser->yyidxMax = 0;
#endif
#if YYSTACKDEPTH<=0
    pParser->yystack = NULL;
    pParser->yystksz = 0;
    yyGrowStack(pParser);
#endif
  }
  return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  ParseARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are not used
    ** inside the C code.
    */
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(yyParser *pParser){
  YYCODETYPE yymajor;
  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

  if( pParser->yyidx<0 ) return 0;
#ifndef NDEBUG
  if( yyTraceFILE && pParser->yyidx>=0 ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yymajor = yytos->major;
  yy_destructor(pParser, yymajor, &yytos->minor);
  pParser->yyidx--;
  return yymajor;
}

/* 
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from ParseAlloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void ParseFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
  if( pParser==0 ) return;
  while( pParser->yyidx>=0 ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  free(pParser->yystack);
#endif
  (*freeProc)((void*)pParser);
}

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int ParseStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyidxMax;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yystack[pParser->yyidx].stateno;
 
  if( stateno>YY_SHIFT_MAX || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
    return yy_default[stateno];
  }
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    if( iLookAhead>0 ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        return yy_find_shift_action(pParser, iFallback);
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( j>=0 && j<YY_SZ_ACTTAB && yy_lookahead[j]==YYWILDCARD ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
    }
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_MAX ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_MAX );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_SZ_ACTTAB );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser, YYMINORTYPE *yypMinor){
   ParseARG_FETCH;
   yypParser->yyidx--;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */
#line 69 "vanuatuWkt.y"

     fprintf(stderr,"Giving up.  Parser stack overflow\n");
#line 851 "vanuatuWkt.c"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor         /* Pointer to the minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yyidx++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( yypParser->yyidx>yypParser->yyidxMax ){
    yypParser->yyidxMax = yypParser->yyidx;
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yyidx>=YYSTACKDEPTH ){
    yyStackOverflow(yypParser, yypMinor);
    return;
  }
#else
  if( yypParser->yyidx>=yypParser->yystksz ){
    yyGrowStack(yypParser);
    if( yypParser->yyidx>=yypParser->yystksz ){
      yyStackOverflow(yypParser, yypMinor);
      return;
    }
  }
#endif
  yytos = &yypParser->yystack[yypParser->yyidx];
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor = *yypMinor;
#ifndef NDEBUG
  if( yyTraceFILE && yypParser->yyidx>0 ){
    int i;
    fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
    fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
    for(i=1; i<=yypParser->yyidx; i++)
      fprintf(yyTraceFILE," %s",yyTokenName[yypParser->yystack[i].major]);
    fprintf(yyTraceFILE,"\n");
  }
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 35, 1 },
  { 36, 0 },
  { 36, 3 },
  { 37, 1 },
  { 38, 1 },
  { 38, 1 },
  { 38, 1 },
  { 38, 1 },
  { 39, 1 },
  { 39, 1 },
  { 39, 1 },
  { 39, 1 },
  { 39, 1 },
  { 39, 1 },
  { 39, 1 },
  { 40, 1 },
  { 40, 1 },
  { 40, 1 },
  { 40, 1 },
  { 40, 1 },
  { 40, 1 },
  { 40, 1 },
  { 41, 1 },
  { 41, 1 },
  { 41, 1 },
  { 41, 1 },
  { 41, 1 },
  { 41, 1 },
  { 41, 1 },
  { 42, 1 },
  { 42, 1 },
  { 42, 1 },
  { 42, 1 },
  { 42, 1 },
  { 42, 1 },
  { 42, 1 },
  { 43, 4 },
  { 57, 4 },
  { 50, 4 },
  { 64, 4 },
  { 71, 2 },
  { 72, 3 },
  { 73, 3 },
  { 74, 4 },
  { 75, 1 },
  { 76, 0 },
  { 76, 3 },
  { 77, 0 },
  { 77, 3 },
  { 78, 0 },
  { 78, 3 },
  { 79, 0 },
  { 79, 3 },
  { 44, 2 },
  { 58, 2 },
  { 51, 2 },
  { 65, 2 },
  { 80, 6 },
  { 81, 6 },
  { 82, 6 },
  { 83, 6 },
  { 45, 2 },
  { 59, 2 },
  { 52, 2 },
  { 66, 2 },
  { 84, 4 },
  { 85, 4 },
  { 86, 4 },
  { 87, 4 },
  { 88, 10 },
  { 89, 0 },
  { 89, 3 },
  { 90, 10 },
  { 91, 0 },
  { 91, 3 },
  { 92, 10 },
  { 93, 0 },
  { 93, 3 },
  { 94, 10 },
  { 95, 0 },
  { 95, 3 },
  { 46, 2 },
  { 60, 2 },
  { 53, 2 },
  { 67, 2 },
  { 96, 4 },
  { 97, 4 },
  { 98, 4 },
  { 99, 4 },
  { 47, 2 },
  { 61, 2 },
  { 54, 2 },
  { 68, 2 },
  { 100, 4 },
  { 104, 0 },
  { 104, 3 },
  { 101, 4 },
  { 105, 0 },
  { 105, 3 },
  { 102, 4 },
  { 106, 0 },
  { 106, 3 },
  { 103, 4 },
  { 107, 0 },
  { 107, 3 },
  { 48, 2 },
  { 62, 2 },
  { 55, 2 },
  { 69, 2 },
  { 108, 4 },
  { 112, 0 },
  { 112, 3 },
  { 109, 4 },
  { 113, 0 },
  { 113, 3 },
  { 110, 4 },
  { 114, 0 },
  { 114, 3 },
  { 111, 4 },
  { 115, 0 },
  { 115, 3 },
  { 49, 2 },
  { 63, 2 },
  { 56, 2 },
  { 70, 2 },
  { 116, 4 },
  { 116, 4 },
  { 116, 4 },
  { 120, 0 },
  { 120, 3 },
  { 120, 3 },
  { 120, 3 },
  { 117, 4 },
  { 117, 4 },
  { 117, 4 },
  { 121, 0 },
  { 121, 3 },
  { 121, 3 },
  { 121, 3 },
  { 118, 4 },
  { 118, 4 },
  { 118, 4 },
  { 122, 0 },
  { 122, 3 },
  { 122, 3 },
  { 122, 3 },
  { 119, 4 },
  { 119, 4 },
  { 119, 4 },
  { 123, 0 },
  { 123, 3 },
  { 123, 3 },
  { 123, 3 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  int yyruleno                 /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  ParseARG_FETCH;
  yymsp = &yypParser->yystack[yypParser->yyidx];
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno>=0 
        && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
      yyRuleName[yyruleno]);
  }
#endif /* NDEBUG */

  /* Silence complaints from purify about yygotominor being uninitialized
  ** in some cases when it is copied into the stack after the following
  ** switch.  yygotominor is uninitialized when a rule reduces that does
  ** not set the value of its left-hand side nonterminal.  Leaving the
  ** value of the nonterminal uninitialized is utterly harmless as long
  ** as the value is never used.  So really the only thing this code
  ** accomplishes is to quieten purify.  
  **
  ** 2007-01-16:  The wireshark project (www.wireshark.org) reports that
  ** without this code, their parser segfaults.  I'm not sure what there
  ** parser is doing to make this happen.  This is the second bug report
  ** from wireshark this week.  Clearly they are stressing Lemon in ways
  ** that it has not been previously stressed...  (SQLite ticket #2172)
  */
  /*memset(&yygotominor, 0, sizeof(yygotominor));*/
  yygotominor = yyzerominor;


  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
      case 8: /* geo_text ::= point */
      case 9: /* geo_text ::= linestring */ yytestcase(yyruleno==9);
      case 10: /* geo_text ::= polygon */ yytestcase(yyruleno==10);
      case 11: /* geo_text ::= multipoint */ yytestcase(yyruleno==11);
      case 12: /* geo_text ::= multilinestring */ yytestcase(yyruleno==12);
      case 13: /* geo_text ::= multipolygon */ yytestcase(yyruleno==13);
      case 14: /* geo_text ::= geocoll */ yytestcase(yyruleno==14);
      case 15: /* geo_textz ::= pointz */ yytestcase(yyruleno==15);
      case 16: /* geo_textz ::= linestringz */ yytestcase(yyruleno==16);
      case 17: /* geo_textz ::= polygonz */ yytestcase(yyruleno==17);
      case 18: /* geo_textz ::= multipointz */ yytestcase(yyruleno==18);
      case 19: /* geo_textz ::= multilinestringz */ yytestcase(yyruleno==19);
      case 20: /* geo_textz ::= multipolygonz */ yytestcase(yyruleno==20);
      case 21: /* geo_textz ::= geocollz */ yytestcase(yyruleno==21);
      case 22: /* geo_textm ::= pointm */ yytestcase(yyruleno==22);
      case 23: /* geo_textm ::= linestringm */ yytestcase(yyruleno==23);
      case 24: /* geo_textm ::= polygonm */ yytestcase(yyruleno==24);
      case 25: /* geo_textm ::= multipointm */ yytestcase(yyruleno==25);
      case 26: /* geo_textm ::= multilinestringm */ yytestcase(yyruleno==26);
      case 27: /* geo_textm ::= multipolygonm */ yytestcase(yyruleno==27);
      case 28: /* geo_textm ::= geocollm */ yytestcase(yyruleno==28);
      case 29: /* geo_textzm ::= pointzm */ yytestcase(yyruleno==29);
      case 30: /* geo_textzm ::= linestringzm */ yytestcase(yyruleno==30);
      case 31: /* geo_textzm ::= polygonzm */ yytestcase(yyruleno==31);
      case 32: /* geo_textzm ::= multipointzm */ yytestcase(yyruleno==32);
      case 33: /* geo_textzm ::= multilinestringzm */ yytestcase(yyruleno==33);
      case 34: /* geo_textzm ::= multipolygonzm */ yytestcase(yyruleno==34);
      case 35: /* geo_textzm ::= geocollzm */ yytestcase(yyruleno==35);
#line 115 "vanuatuWkt.y"
{ *result = yymsp[0].minor.yy0; }
#line 1145 "vanuatuWkt.c"
        break;
      case 36: /* point ::= VANUATU_POINT VANUATU_OPEN_BRACKET point_coordxy VANUATU_CLOSE_BRACKET */
#line 154 "vanuatuWkt.y"
{ yygotominor.yy0 = vanuatu_buildGeomFromPoint((gaiaPointPtr)yymsp[-1].minor.yy0); }
#line 1150 "vanuatuWkt.c"
        break;
      case 37: /* pointm ::= VANUATU_POINT_M VANUATU_OPEN_BRACKET point_coordxym VANUATU_CLOSE_BRACKET */
      case 38: /* pointz ::= VANUATU_POINT_Z VANUATU_OPEN_BRACKET point_coordxyz VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==38);
      case 39: /* pointzm ::= VANUATU_POINT_ZM VANUATU_OPEN_BRACKET point_coordxyzm VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==39);
#line 156 "vanuatuWkt.y"
{ yygotominor.yy0 = vanuatu_buildGeomFromPoint((gaiaPointPtr)yymsp[-1].minor.yy0);  }
#line 1157 "vanuatuWkt.c"
        break;
      case 40: /* point_coordxy ::= coord coord */
#line 165 "vanuatuWkt.y"
{ yygotominor.yy0 = (void *) vanuatu_point_xy((double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
#line 1162 "vanuatuWkt.c"
        break;
      case 41: /* point_coordxym ::= coord coord coord */
#line 167 "vanuatuWkt.y"
{ yygotominor.yy0 = (void *) vanuatu_point_xym((double *)yymsp[-2].minor.yy0, (double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
#line 1167 "vanuatuWkt.c"
        break;
      case 42: /* point_coordxyz ::= coord coord coord */
#line 169 "vanuatuWkt.y"
{ yygotominor.yy0 = (void *) vanuatu_point_xyz((double *)yymsp[-2].minor.yy0, (double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
#line 1172 "vanuatuWkt.c"
        break;
      case 43: /* point_coordxyzm ::= coord coord coord coord */
#line 171 "vanuatuWkt.y"
{ yygotominor.yy0 = (void *) vanuatu_point_xyzm((double *)yymsp[-3].minor.yy0, (double *)yymsp[-2].minor.yy0, (double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
#line 1177 "vanuatuWkt.c"
        break;
      case 44: /* coord ::= VANUATU_NUM */
      case 81: /* multipoint ::= VANUATU_MULTIPOINT multipoint_text */ yytestcase(yyruleno==81);
      case 82: /* multipointm ::= VANUATU_MULTIPOINT_M multipoint_textm */ yytestcase(yyruleno==82);
      case 83: /* multipointz ::= VANUATU_MULTIPOINT_Z multipoint_textz */ yytestcase(yyruleno==83);
      case 84: /* multipointzm ::= VANUATU_MULTIPOINT_ZM multipoint_textzm */ yytestcase(yyruleno==84);
      case 89: /* multilinestring ::= VANUATU_MULTILINESTRING multilinestring_text */ yytestcase(yyruleno==89);
      case 90: /* multilinestringm ::= VANUATU_MULTILINESTRING_M multilinestring_textm */ yytestcase(yyruleno==90);
      case 91: /* multilinestringz ::= VANUATU_MULTILINESTRING_Z multilinestring_textz */ yytestcase(yyruleno==91);
      case 92: /* multilinestringzm ::= VANUATU_MULTILINESTRING_ZM multilinestring_textzm */ yytestcase(yyruleno==92);
      case 105: /* multipolygon ::= VANUATU_MULTIPOLYGON multipolygon_text */ yytestcase(yyruleno==105);
      case 106: /* multipolygonm ::= VANUATU_MULTIPOLYGON_M multipolygon_textm */ yytestcase(yyruleno==106);
      case 107: /* multipolygonz ::= VANUATU_MULTIPOLYGON_Z multipolygon_textz */ yytestcase(yyruleno==107);
      case 108: /* multipolygonzm ::= VANUATU_MULTIPOLYGON_ZM multipolygon_textzm */ yytestcase(yyruleno==108);
      case 121: /* geocoll ::= VANUATU_GEOMETRYCOLLECTION geocoll_text */ yytestcase(yyruleno==121);
      case 122: /* geocollm ::= VANUATU_GEOMETRYCOLLECTION_M geocoll_textm */ yytestcase(yyruleno==122);
      case 123: /* geocollz ::= VANUATU_GEOMETRYCOLLECTION_Z geocoll_textz */ yytestcase(yyruleno==123);
      case 124: /* geocollzm ::= VANUATU_GEOMETRYCOLLECTION_ZM geocoll_textzm */ yytestcase(yyruleno==124);
#line 174 "vanuatuWkt.y"
{ yygotominor.yy0 = yymsp[0].minor.yy0; }
#line 1198 "vanuatuWkt.c"
        break;
      case 45: /* extra_pointsxy ::= */
      case 47: /* extra_pointsxym ::= */ yytestcase(yyruleno==47);
      case 49: /* extra_pointsxyz ::= */ yytestcase(yyruleno==49);
      case 51: /* extra_pointsxyzm ::= */ yytestcase(yyruleno==51);
      case 70: /* extra_rings ::= */ yytestcase(yyruleno==70);
      case 73: /* extra_ringsm ::= */ yytestcase(yyruleno==73);
      case 76: /* extra_ringsz ::= */ yytestcase(yyruleno==76);
      case 79: /* extra_ringszm ::= */ yytestcase(yyruleno==79);
      case 94: /* multilinestring_text2 ::= */ yytestcase(yyruleno==94);
      case 97: /* multilinestring_textm2 ::= */ yytestcase(yyruleno==97);
      case 100: /* multilinestring_textz2 ::= */ yytestcase(yyruleno==100);
      case 103: /* multilinestring_textzm2 ::= */ yytestcase(yyruleno==103);
      case 110: /* multipolygon_text2 ::= */ yytestcase(yyruleno==110);
      case 113: /* multipolygon_textm2 ::= */ yytestcase(yyruleno==113);
      case 116: /* multipolygon_textz2 ::= */ yytestcase(yyruleno==116);
      case 119: /* multipolygon_textzm2 ::= */ yytestcase(yyruleno==119);
      case 128: /* geocoll_text2 ::= */ yytestcase(yyruleno==128);
      case 135: /* geocoll_textm2 ::= */ yytestcase(yyruleno==135);
      case 142: /* geocoll_textz2 ::= */ yytestcase(yyruleno==142);
      case 149: /* geocoll_textzm2 ::= */ yytestcase(yyruleno==149);
#line 179 "vanuatuWkt.y"
{ yygotominor.yy0 = NULL; }
#line 1222 "vanuatuWkt.c"
        break;
      case 46: /* extra_pointsxy ::= VANUATU_COMMA point_coordxy extra_pointsxy */
      case 48: /* extra_pointsxym ::= VANUATU_COMMA point_coordxym extra_pointsxym */ yytestcase(yyruleno==48);
      case 50: /* extra_pointsxyz ::= VANUATU_COMMA point_coordxyz extra_pointsxyz */ yytestcase(yyruleno==50);
      case 52: /* extra_pointsxyzm ::= VANUATU_COMMA point_coordxyzm extra_pointsxyzm */ yytestcase(yyruleno==52);
#line 181 "vanuatuWkt.y"
{ ((gaiaPointPtr)yymsp[-1].minor.yy0)->Next = (gaiaPointPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
#line 1230 "vanuatuWkt.c"
        break;
      case 53: /* linestring ::= VANUATU_LINESTRING linestring_text */
      case 54: /* linestringm ::= VANUATU_LINESTRING_M linestring_textm */ yytestcase(yyruleno==54);
      case 55: /* linestringz ::= VANUATU_LINESTRING_Z linestring_textz */ yytestcase(yyruleno==55);
      case 56: /* linestringzm ::= VANUATU_LINESTRING_ZM linestring_textzm */ yytestcase(yyruleno==56);
#line 199 "vanuatuWkt.y"
{ yygotominor.yy0 = vanuatu_buildGeomFromLinestring((gaiaLinestringPtr)yymsp[0].minor.yy0); }
#line 1238 "vanuatuWkt.c"
        break;
      case 57: /* linestring_text ::= VANUATU_OPEN_BRACKET point_coordxy VANUATU_COMMA point_coordxy extra_pointsxy VANUATU_CLOSE_BRACKET */
#line 210 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) vanuatu_linestring_xy((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
#line 1247 "vanuatuWkt.c"
        break;
      case 58: /* linestring_textm ::= VANUATU_OPEN_BRACKET point_coordxym VANUATU_COMMA point_coordxym extra_pointsxym VANUATU_CLOSE_BRACKET */
#line 217 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) vanuatu_linestring_xym((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
#line 1256 "vanuatuWkt.c"
        break;
      case 59: /* linestring_textz ::= VANUATU_OPEN_BRACKET point_coordxyz VANUATU_COMMA point_coordxyz extra_pointsxyz VANUATU_CLOSE_BRACKET */
#line 224 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) vanuatu_linestring_xyz((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
#line 1265 "vanuatuWkt.c"
        break;
      case 60: /* linestring_textzm ::= VANUATU_OPEN_BRACKET point_coordxyzm VANUATU_COMMA point_coordxyzm extra_pointsxyzm VANUATU_CLOSE_BRACKET */
#line 231 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) vanuatu_linestring_xyzm((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
#line 1274 "vanuatuWkt.c"
        break;
      case 61: /* polygon ::= VANUATU_POLYGON polygon_text */
      case 62: /* polygonm ::= VANUATU_POLYGON_M polygon_textm */ yytestcase(yyruleno==62);
      case 63: /* polygonz ::= VANUATU_POLYGON_Z polygon_textz */ yytestcase(yyruleno==63);
      case 64: /* polygonzm ::= VANUATU_POLYGON_ZM polygon_textzm */ yytestcase(yyruleno==64);
#line 241 "vanuatuWkt.y"
{ yygotominor.yy0 = vanuatu_buildGeomFromPolygon((gaiaPolygonPtr)yymsp[0].minor.yy0); }
#line 1282 "vanuatuWkt.c"
        break;
      case 65: /* polygon_text ::= VANUATU_OPEN_BRACKET ring extra_rings VANUATU_CLOSE_BRACKET */
#line 252 "vanuatuWkt.y"
{ 
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_polygon_xy((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
#line 1290 "vanuatuWkt.c"
        break;
      case 66: /* polygon_textm ::= VANUATU_OPEN_BRACKET ringm extra_ringsm VANUATU_CLOSE_BRACKET */
#line 258 "vanuatuWkt.y"
{ 
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_polygon_xym((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
#line 1298 "vanuatuWkt.c"
        break;
      case 67: /* polygon_textz ::= VANUATU_OPEN_BRACKET ringz extra_ringsz VANUATU_CLOSE_BRACKET */
#line 264 "vanuatuWkt.y"
{  
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_polygon_xyz((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
#line 1306 "vanuatuWkt.c"
        break;
      case 68: /* polygon_textzm ::= VANUATU_OPEN_BRACKET ringzm extra_ringszm VANUATU_CLOSE_BRACKET */
#line 270 "vanuatuWkt.y"
{ 
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_polygon_xyzm((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
#line 1314 "vanuatuWkt.c"
        break;
      case 69: /* ring ::= VANUATU_OPEN_BRACKET point_coordxy VANUATU_COMMA point_coordxy VANUATU_COMMA point_coordxy VANUATU_COMMA point_coordxy extra_pointsxy VANUATU_CLOSE_BRACKET */
#line 278 "vanuatuWkt.y"
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_ring_xy((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
#line 1325 "vanuatuWkt.c"
        break;
      case 71: /* extra_rings ::= VANUATU_COMMA ring extra_rings */
      case 74: /* extra_ringsm ::= VANUATU_COMMA ringm extra_ringsm */ yytestcase(yyruleno==74);
      case 77: /* extra_ringsz ::= VANUATU_COMMA ringz extra_ringsz */ yytestcase(yyruleno==77);
      case 80: /* extra_ringszm ::= VANUATU_COMMA ringzm extra_ringszm */ yytestcase(yyruleno==80);
#line 289 "vanuatuWkt.y"
{
		((gaiaRingPtr)yymsp[-1].minor.yy0)->Next = (gaiaRingPtr)yymsp[0].minor.yy0;
		yygotominor.yy0 = yymsp[-1].minor.yy0;
	}
#line 1336 "vanuatuWkt.c"
        break;
      case 72: /* ringm ::= VANUATU_OPEN_BRACKET point_coordxym VANUATU_COMMA point_coordxym VANUATU_COMMA point_coordxym VANUATU_COMMA point_coordxym extra_pointsxym VANUATU_CLOSE_BRACKET */
#line 295 "vanuatuWkt.y"
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_ring_xym((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
#line 1347 "vanuatuWkt.c"
        break;
      case 75: /* ringz ::= VANUATU_OPEN_BRACKET point_coordxyz VANUATU_COMMA point_coordxyz VANUATU_COMMA point_coordxyz VANUATU_COMMA point_coordxyz extra_pointsxyz VANUATU_CLOSE_BRACKET */
#line 312 "vanuatuWkt.y"
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_ring_xyz((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
#line 1358 "vanuatuWkt.c"
        break;
      case 78: /* ringzm ::= VANUATU_OPEN_BRACKET point_coordxyzm VANUATU_COMMA point_coordxyzm VANUATU_COMMA point_coordxyzm VANUATU_COMMA point_coordxyzm extra_pointsxyzm VANUATU_CLOSE_BRACKET */
#line 329 "vanuatuWkt.y"
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_ring_xyzm((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
#line 1369 "vanuatuWkt.c"
        break;
      case 85: /* multipoint_text ::= VANUATU_OPEN_BRACKET point_coordxy extra_pointsxy VANUATU_CLOSE_BRACKET */
#line 355 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipoint_xy((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
#line 1377 "vanuatuWkt.c"
        break;
      case 86: /* multipoint_textm ::= VANUATU_OPEN_BRACKET point_coordxym extra_pointsxym VANUATU_CLOSE_BRACKET */
#line 360 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipoint_xym((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
#line 1385 "vanuatuWkt.c"
        break;
      case 87: /* multipoint_textz ::= VANUATU_OPEN_BRACKET point_coordxyz extra_pointsxyz VANUATU_CLOSE_BRACKET */
#line 365 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipoint_xyz((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
#line 1393 "vanuatuWkt.c"
        break;
      case 88: /* multipoint_textzm ::= VANUATU_OPEN_BRACKET point_coordxyzm extra_pointsxyzm VANUATU_CLOSE_BRACKET */
#line 370 "vanuatuWkt.y"
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipoint_xyzm((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
#line 1401 "vanuatuWkt.c"
        break;
      case 93: /* multilinestring_text ::= VANUATU_OPEN_BRACKET linestring_text multilinestring_text2 VANUATU_CLOSE_BRACKET */
#line 386 "vanuatuWkt.y"
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multilinestring_xy((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
#line 1409 "vanuatuWkt.c"
        break;
      case 95: /* multilinestring_text2 ::= VANUATU_COMMA linestring_text multilinestring_text2 */
      case 98: /* multilinestring_textm2 ::= VANUATU_COMMA linestring_textm multilinestring_textm2 */ yytestcase(yyruleno==98);
      case 101: /* multilinestring_textz2 ::= VANUATU_COMMA linestring_textz multilinestring_textz2 */ yytestcase(yyruleno==101);
      case 104: /* multilinestring_textzm2 ::= VANUATU_COMMA linestring_textzm multilinestring_textzm2 */ yytestcase(yyruleno==104);
#line 394 "vanuatuWkt.y"
{ ((gaiaLinestringPtr)yymsp[-1].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
#line 1417 "vanuatuWkt.c"
        break;
      case 96: /* multilinestring_textm ::= VANUATU_OPEN_BRACKET linestring_textm multilinestring_textm2 VANUATU_CLOSE_BRACKET */
#line 397 "vanuatuWkt.y"
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multilinestring_xym((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
#line 1425 "vanuatuWkt.c"
        break;
      case 99: /* multilinestring_textz ::= VANUATU_OPEN_BRACKET linestring_textz multilinestring_textz2 VANUATU_CLOSE_BRACKET */
#line 407 "vanuatuWkt.y"
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multilinestring_xyz((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
#line 1433 "vanuatuWkt.c"
        break;
      case 102: /* multilinestring_textzm ::= VANUATU_OPEN_BRACKET linestring_textzm multilinestring_textzm2 VANUATU_CLOSE_BRACKET */
#line 417 "vanuatuWkt.y"
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multilinestring_xyzm((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
#line 1441 "vanuatuWkt.c"
        break;
      case 109: /* multipolygon_text ::= VANUATU_OPEN_BRACKET polygon_text multipolygon_text2 VANUATU_CLOSE_BRACKET */
#line 437 "vanuatuWkt.y"
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipolygon_xy((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
#line 1449 "vanuatuWkt.c"
        break;
      case 111: /* multipolygon_text2 ::= VANUATU_COMMA polygon_text multipolygon_text2 */
      case 114: /* multipolygon_textm2 ::= VANUATU_COMMA polygon_textm multipolygon_textm2 */ yytestcase(yyruleno==114);
      case 117: /* multipolygon_textz2 ::= VANUATU_COMMA polygon_textz multipolygon_textz2 */ yytestcase(yyruleno==117);
      case 120: /* multipolygon_textzm2 ::= VANUATU_COMMA polygon_textzm multipolygon_textzm2 */ yytestcase(yyruleno==120);
#line 445 "vanuatuWkt.y"
{ ((gaiaPolygonPtr)yymsp[-1].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
#line 1457 "vanuatuWkt.c"
        break;
      case 112: /* multipolygon_textm ::= VANUATU_OPEN_BRACKET polygon_textm multipolygon_textm2 VANUATU_CLOSE_BRACKET */
#line 448 "vanuatuWkt.y"
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipolygon_xym((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
#line 1465 "vanuatuWkt.c"
        break;
      case 115: /* multipolygon_textz ::= VANUATU_OPEN_BRACKET polygon_textz multipolygon_textz2 VANUATU_CLOSE_BRACKET */
#line 458 "vanuatuWkt.y"
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipolygon_xyz((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
#line 1473 "vanuatuWkt.c"
        break;
      case 118: /* multipolygon_textzm ::= VANUATU_OPEN_BRACKET polygon_textzm multipolygon_textzm2 VANUATU_CLOSE_BRACKET */
#line 468 "vanuatuWkt.y"
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) vanuatu_multipolygon_xyzm((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
#line 1481 "vanuatuWkt.c"
        break;
      case 125: /* geocoll_text ::= VANUATU_OPEN_BRACKET point geocoll_text2 VANUATU_CLOSE_BRACKET */
      case 126: /* geocoll_text ::= VANUATU_OPEN_BRACKET linestring geocoll_text2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==126);
      case 127: /* geocoll_text ::= VANUATU_OPEN_BRACKET polygon geocoll_text2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==127);
#line 487 "vanuatuWkt.y"
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_geomColl_xy((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
#line 1491 "vanuatuWkt.c"
        break;
      case 129: /* geocoll_text2 ::= VANUATU_COMMA point geocoll_text2 */
      case 130: /* geocoll_text2 ::= VANUATU_COMMA linestring geocoll_text2 */ yytestcase(yyruleno==130);
      case 131: /* geocoll_text2 ::= VANUATU_COMMA polygon geocoll_text2 */ yytestcase(yyruleno==131);
      case 136: /* geocoll_textm2 ::= VANUATU_COMMA pointm geocoll_textm2 */ yytestcase(yyruleno==136);
      case 137: /* geocoll_textm2 ::= VANUATU_COMMA linestringm geocoll_textm2 */ yytestcase(yyruleno==137);
      case 138: /* geocoll_textm2 ::= VANUATU_COMMA polygonm geocoll_textm2 */ yytestcase(yyruleno==138);
      case 143: /* geocoll_textz2 ::= VANUATU_COMMA pointz geocoll_textz2 */ yytestcase(yyruleno==143);
      case 144: /* geocoll_textz2 ::= VANUATU_COMMA linestringz geocoll_textz2 */ yytestcase(yyruleno==144);
      case 145: /* geocoll_textz2 ::= VANUATU_COMMA polygonz geocoll_textz2 */ yytestcase(yyruleno==145);
      case 150: /* geocoll_textzm2 ::= VANUATU_COMMA pointzm geocoll_textzm2 */ yytestcase(yyruleno==150);
      case 151: /* geocoll_textzm2 ::= VANUATU_COMMA linestringzm geocoll_textzm2 */ yytestcase(yyruleno==151);
      case 152: /* geocoll_textzm2 ::= VANUATU_COMMA polygonzm geocoll_textzm2 */ yytestcase(yyruleno==152);
#line 507 "vanuatuWkt.y"
{
		((gaiaGeomCollPtr)yymsp[-1].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[0].minor.yy0;
		yygotominor.yy0 = yymsp[-1].minor.yy0;
	}
#line 1510 "vanuatuWkt.c"
        break;
      case 132: /* geocoll_textm ::= VANUATU_OPEN_BRACKET pointm geocoll_textm2 VANUATU_CLOSE_BRACKET */
      case 133: /* geocoll_textm ::= VANUATU_OPEN_BRACKET linestringm geocoll_textm2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==133);
      case 134: /* geocoll_textm ::= VANUATU_OPEN_BRACKET polygonm geocoll_textm2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==134);
#line 526 "vanuatuWkt.y"
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_geomColl_xym((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
#line 1520 "vanuatuWkt.c"
        break;
      case 139: /* geocoll_textz ::= VANUATU_OPEN_BRACKET pointz geocoll_textz2 VANUATU_CLOSE_BRACKET */
      case 140: /* geocoll_textz ::= VANUATU_OPEN_BRACKET linestringz geocoll_textz2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==140);
      case 141: /* geocoll_textz ::= VANUATU_OPEN_BRACKET polygonz geocoll_textz2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==141);
#line 564 "vanuatuWkt.y"
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_geomColl_xyz((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
#line 1530 "vanuatuWkt.c"
        break;
      case 146: /* geocoll_textzm ::= VANUATU_OPEN_BRACKET pointzm geocoll_textzm2 VANUATU_CLOSE_BRACKET */
      case 147: /* geocoll_textzm ::= VANUATU_OPEN_BRACKET linestringzm geocoll_textzm2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==147);
      case 148: /* geocoll_textzm ::= VANUATU_OPEN_BRACKET polygonzm geocoll_textzm2 VANUATU_CLOSE_BRACKET */ yytestcase(yyruleno==148);
#line 602 "vanuatuWkt.y"
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) vanuatu_geomColl_xyzm((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
#line 1540 "vanuatuWkt.c"
        break;
      default:
      /* (0) main ::= in */ yytestcase(yyruleno==0);
      /* (1) in ::= */ yytestcase(yyruleno==1);
      /* (2) in ::= in state VANUATU_NEWLINE */ yytestcase(yyruleno==2);
      /* (3) state ::= program */ yytestcase(yyruleno==3);
      /* (4) program ::= geo_text */ yytestcase(yyruleno==4);
      /* (5) program ::= geo_textz */ yytestcase(yyruleno==5);
      /* (6) program ::= geo_textm */ yytestcase(yyruleno==6);
      /* (7) program ::= geo_textzm */ yytestcase(yyruleno==7);
        break;
  };
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yypParser->yyidx -= yysize;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact < YYNSTATE ){
#ifdef NDEBUG
    /* If we are not debugging and the reduce action popped at least
    ** one element off the stack, then we can push the new element back
    ** onto the stack here, and skip the stack overflow test in yy_shift().
    ** That gives a significant speed improvement. */
    if( yysize ){
      yypParser->yyidx++;
      yymsp -= yysize-1;
      yymsp->stateno = (YYACTIONTYPE)yyact;
      yymsp->major = (YYCODETYPE)yygoto;
      yymsp->minor = yygotominor;
    }else
#endif
    {
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
    }
  }else{
    assert( yyact == YYNSTATE + YYNRULE + 1 );
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  YYMINORTYPE yyminor            /* The minor type of the error token */
){
  ParseARG_FETCH;
#define TOKEN (yyminor.yy0)
#line 84 "vanuatuWkt.y"

/* 
** Sandro Furieri 2010 Apr 4
** when the LEMON parser encounters an error
** then this global variable is set 
*/
	vanuatu_parse_error = 1;
	*result = NULL;
#line 1619 "vanuatuWkt.c"
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "ParseAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void Parse(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  ParseTOKENTYPE yyminor       /* The value for the token */
  ParseARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  int yyact;            /* The parser action. */
  int yyendofinput;     /* True if we are at the end of input */
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  /* (re)initialize the parser, if necessary */
  yypParser = (yyParser*)yyp;
  if( yypParser->yyidx<0 ){
#if YYSTACKDEPTH<=0
    if( yypParser->yystksz <=0 ){
      /*memset(&yyminorunion, 0, sizeof(yyminorunion));*/
      yyminorunion = yyzerominor;
      yyStackOverflow(yypParser, &yyminorunion);
      return;
    }
#endif
    yypParser->yyidx = 0;
    yypParser->yyerrcnt = -1;
    yypParser->yystack[0].stateno = 0;
    yypParser->yystack[0].major = 0;
  }
  yyminorunion.yy0 = yyminor;
  yyendofinput = (yymajor==0);
  ParseARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact<YYNSTATE ){
      assert( !yyendofinput );  /* Impossible to shift the $ token */
      yy_shift(yypParser,yyact,yymajor,&yyminorunion);
      yypParser->yyerrcnt--;
      yymajor = YYNOCODE;
    }else if( yyact < YYNSTATE + YYNRULE ){
      yy_reduce(yypParser,yyact-YYNSTATE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yymx = yypParser->yystack[yypParser->yyidx].major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor,&yyminorunion);
        yymajor = YYNOCODE;
      }else{
         while(
          yypParser->yyidx >= 0 &&
          yymx != YYERRORSYMBOL &&
          (yyact = yy_find_reduce_action(
                        yypParser->yystack[yypParser->yyidx].stateno,
                        YYERRORSYMBOL)) >= YYNSTATE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yyidx < 0 || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          YYMINORTYPE u2;
          u2.YYERRSYMDT = 0;
          yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor,yyminorunion);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;
      
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
  return;
}


/*
 VANUATU_LEMON_END - LEMON generated code ends here 
*/

















/*
** CAVEAT: there is an incompatibility between LEMON and FLEX
** this macro resolves the issue
*/
#undef yy_accept
#define yy_accept	yy_vanuatu_flex_accept



/*
 VANUATU_FLEX_START - FLEX generated code starts here 
*/

#line 3 "lex.VanuatuWkt.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer VanuatuWkt_create_buffer
#define yy_delete_buffer VanuatuWkt_delete_buffer
#define yy_flex_debug VanuatuWkt_flex_debug
#define yy_init_buffer VanuatuWkt_init_buffer
#define yy_flush_buffer VanuatuWkt_flush_buffer
#define yy_load_buffer_state VanuatuWkt_load_buffer_state
#define yy_switch_to_buffer VanuatuWkt_switch_to_buffer
#define yyin VanuatuWktin
#define yyleng VanuatuWktleng
#define yylex VanuatuWktlex
#define yylineno VanuatuWktlineno
#define yyout VanuatuWktout
#define yyrestart VanuatuWktrestart
#define yytext VanuatuWkttext
#define yywrap VanuatuWktwrap
#define yyalloc VanuatuWktalloc
#define yyrealloc VanuatuWktrealloc
#define yyfree VanuatuWktfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <errno.h> */
/* #include <stdlib.h> */

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE VanuatuWktrestart(VanuatuWktin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int VanuatuWktleng;

extern FILE *VanuatuWktin, *VanuatuWktout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up VanuatuWkttext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up VanuatuWkttext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via VanuatuWktrestart()), so that the user can continue scanning by
	 * just pointing VanuatuWktin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when VanuatuWkttext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int VanuatuWktleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow VanuatuWktwrap()'s to do buffer switches
 * instead of setting up a fresh VanuatuWktin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void VanuatuWktrestart (FILE *input_file  );
void VanuatuWkt_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE VanuatuWkt_create_buffer (FILE *file,int size  );
void VanuatuWkt_delete_buffer (YY_BUFFER_STATE b  );
void VanuatuWkt_flush_buffer (YY_BUFFER_STATE b  );
void VanuatuWktpush_buffer_state (YY_BUFFER_STATE new_buffer  );
void VanuatuWktpop_buffer_state (void );

static void VanuatuWktensure_buffer_stack (void );
static void VanuatuWkt_load_buffer_state (void );
static void VanuatuWkt_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER VanuatuWkt_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE VanuatuWkt_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE VanuatuWkt_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE VanuatuWkt_scan_bytes (yyconst char *bytes,int len  );

void *VanuatuWktalloc (yy_size_t  );
void *VanuatuWktrealloc (void *,yy_size_t  );
void VanuatuWktfree (void *  );

#define yy_new_buffer VanuatuWkt_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        VanuatuWktensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            VanuatuWkt_create_buffer(VanuatuWktin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        VanuatuWktensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            VanuatuWkt_create_buffer(VanuatuWktin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *VanuatuWktin = (FILE *) 0, *VanuatuWktout = (FILE *) 0;

typedef int yy_state_type;

extern int VanuatuWktlineno;

int VanuatuWktlineno = 1;

extern char *VanuatuWkttext;
#define yytext_ptr VanuatuWkttext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up VanuatuWkttext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	VanuatuWktleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 36
#define YY_END_OF_BUFFER 37
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[114] =
    {   0,
        0,    0,   37,   35,   33,   34,    3,    4,   35,    2,
       35,    1,   35,   35,   35,   35,    1,    1,    1,    1,
        0,    0,    0,    0,    1,    1,    1,    0,    0,    0,
        0,    0,    1,    1,    0,    0,    0,    0,    0,    0,
        0,    0,    5,    0,    0,    0,    0,    0,    0,    7,
        6,    0,    0,    0,    0,    0,    8,   13,    0,    0,
        0,    0,    0,    0,   15,   14,    0,    0,    0,    0,
        0,   16,    0,    9,    0,   17,    0,    0,    0,   11,
       10,    0,    0,   19,   18,    0,    0,   12,    0,   20,
       25,    0,    0,    0,   27,   26,    0,    0,   28,    0,

       21,    0,    0,   23,   22,    0,   24,   29,    0,   31,
       30,   32,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    1,    1,    1,    1,    1,    1,    4,
        5,    1,    6,    7,    8,    9,    1,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,    1,    1,    1,
        1,    1,    1,    1,    1,    1,   11,    1,   12,    1,
       13,    1,   14,    1,    1,   15,   16,   17,   18,   19,
        1,   20,   21,   22,   23,    1,    1,    1,   24,   25,
        1,    1,    1,    1,    1,    1,    1,    1,   26,    1,

       27,    1,   28,    1,   29,    1,    1,   30,   31,   32,
       33,   34,    1,   35,   36,   37,   38,    1,    1,    1,
       39,   40,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[41] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[115] =
    {   0,
        0,   33,  211,  255,  255,  255,  255,  255,  192,  255,
      191,    2,    2,    3,    4,    0,   11,   13,  185,   15,
       26,   28,   32,   36,  162,  161,  154,   37,   42,   33,
       39,   33,   71,   16,   46,   53,   61,   54,   64,   56,
       57,   65,   80,   67,   66,   67,   74,   79,    0,  255,
       82,   87,   82,   93,   91,   95,  255,  112,  105,  100,
      106,  109,  103,    0,  255,  113,  112,  121,  114,  114,
      125,  255,  124,  144,  118,  154,  130,  143,    0,  255,
      143,  141,    0,  255,  146,  148,  154,  255,  153,  255,
      184,  157,  161,    0,  255,  172,  167,  177,  255,  177,

      194,  174,    0,  255,  182,  182,  255,  214,    0,  255,
      192,  255,  255,    0
    } ;

static yyconst flex_int16_t yy_def[115] =
    {   0,
      114,  114,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,   43,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,   58,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,   74,  113,
      113,  113,   76,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,   91,  113,  113,  113,  113,  113,  113,

      113,  113,  101,  113,  113,  113,  113,  113,  108,  113,
      113,  113,    0,  113
    } ;

static yyconst flex_int16_t yy_nxt[296] =
    {   0,
        4,    5,    6,    7,    8,    9,   10,   11,  113,   12,
       19,   20,   13,   21,   14,   15,   22,   24,   16,   25,
       17,   26,   18,   19,   20,   34,   23,   13,   21,   14,
       15,   22,   24,   16,    5,    6,    7,    8,    9,   10,
       11,   23,   12,   28,   29,   13,   30,   14,   15,   31,
       32,   16,   35,   36,   37,   38,   39,   40,   28,   29,
       13,   30,   14,   15,   31,   32,   16,   35,   36,   37,
       38,   39,   40,   41,   42,   43,   44,   45,   46,   47,
       33,   49,   49,   48,   52,   53,   54,   55,   41,   42,
       43,   44,   45,   46,   47,   50,   56,   57,   48,   52,

       53,   54,   55,   58,   51,   59,   60,   61,   62,   63,
       50,   56,   57,   64,   64,   67,   68,   69,   58,   51,
       59,   60,   61,   62,   63,   70,   71,   65,   72,   73,
       67,   68,   69,   74,   75,   76,   66,   77,   78,   82,
       70,   71,   65,   72,   73,   79,   79,   86,   74,   75,
       76,   66,   77,   78,   82,   83,   83,   87,   88,   80,
       89,   90,   86,   27,   91,   92,   93,   97,   81,   84,
       34,   33,   87,   88,   80,   89,   90,   98,   85,   91,
       92,   93,   97,   81,   84,   94,   94,   99,  100,  101,
      102,  106,   98,   85,   27,  103,  103,  107,  108,   95,

       18,   17,   99,  100,  101,  102,  106,  112,   96,  104,
      113,  113,  107,  108,   95,  109,  109,  113,  105,  113,
      113,  113,  112,   96,  104,  113,  113,  113,  113,  110,
      113,  113,  113,  105,  113,  113,  113,  113,  111,  113,
      113,  113,  113,  113,  110,  113,  113,  113,  113,  113,
      113,  113,  113,  111,    3,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113

    } ;

static yyconst flex_int16_t yy_chk[296] =
    {   0,
      114,    1,    1,    1,    1,    1,    1,    1,    0,    1,
       12,   12,    1,   13,    1,    1,   14,   16,    1,   17,
       17,   18,   18,   20,   20,   34,   15,    1,   13,    1,
        1,   14,   16,    1,    2,    2,    2,    2,    2,    2,
        2,   15,    2,   21,   22,    2,   23,    2,    2,   24,
       24,    2,   28,   29,   30,   31,   32,   35,   21,   22,
        2,   23,    2,    2,   24,   24,    2,   28,   29,   30,
       31,   32,   35,   36,   37,   38,   39,   40,   41,   42,
       33,   43,   43,   42,   44,   45,   46,   47,   36,   37,
       38,   39,   40,   41,   42,   43,   48,   51,   42,   44,

       45,   46,   47,   52,   43,   53,   54,   55,   56,   56,
       43,   48,   51,   58,   58,   59,   60,   61,   52,   43,
       53,   54,   55,   56,   56,   62,   63,   58,   66,   67,
       59,   60,   61,   68,   69,   70,   58,   71,   73,   75,
       62,   63,   58,   66,   67,   74,   74,   77,   68,   69,
       70,   58,   71,   73,   75,   76,   76,   78,   81,   74,
       82,   85,   77,   27,   86,   87,   89,   92,   74,   76,
       26,   25,   78,   81,   74,   82,   85,   93,   76,   86,
       87,   89,   92,   74,   76,   91,   91,   96,   97,   98,
      100,  102,   93,   76,   19,  101,  101,  105,  106,   91,

       11,    9,   96,   97,   98,  100,  102,  111,   91,  101,
        3,    0,  105,  106,   91,  108,  108,    0,  101,    0,
        0,    0,  111,   91,  101,    0,    0,    0,    0,  108,
        0,    0,    0,  101,    0,    0,    0,    0,  108,    0,
        0,    0,    0,    0,  108,    0,    0,    0,    0,    0,
        0,    0,    0,  108,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
      113,  113,  113,  113,  113

    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int VanuatuWkt_flex_debug;
int VanuatuWkt_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *VanuatuWkttext;
/* 
 vanuatuLexer.l -- Vanuatu WKT parser - FLEX config
  
 version 2.4, 2010 April 2

 Author: Sandro Furieri a.furieri@lqt.it

 ------------------------------------------------------------------------------
 
 Version: MPL 1.1/GPL 2.0/LGPL 2.1
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the SpatiaLite library

The Initial Developer of the Original Code is Alessandro Furieri
 
Portions created by the Initial Developer are Copyright (C) 2008
the Initial Developer. All Rights Reserved.

Contributor(s):
The Vanuatu Team - University of Toronto

Alternatively, the contents of this file may be used under the terms of
either the GNU General Public License Version 2 or later (the "GPL"), or
the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
in which case the provisions of the GPL or the LGPL are applicable instead
of those above. If you wish to allow use of your version of this file only
under the terms of either the GPL or the LGPL, and not to allow others to
use your version of this file under the terms of the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the GPL or the LGPL. If you do not delete
the provisions above, a recipient may use your version of this file under
the terms of any one of the MPL, the GPL or the LGPL.
 
*/
/******************************************************************************
** The following code was created by Team Vanuatu of The University of Toronto.

Authors:
Ruppi Rana			ruppi.rana@gmail.com
Dev Tanna			dev.tanna@gmail.com
Elias Adum			elias.adum@gmail.com
Benton Hui			benton.hui@gmail.com
Abhayan Sundararajan		abhayan@gmail.com
Chee-Lun Michael Stephen Cho	cheelun.cho@gmail.com
Nikola Banovic			nikola.banovic@gmail.com
Yong Jian			yong.jian@utoronto.ca

Supervisor:
Greg Wilson			gvwilson@cs.toronto.ca

-------------------------------------------------------------------------------
*/

/* For debugging purposes */
int vanuatu_line = 1, vanuatu_col = 1;

/**
*  The main string-token matcher.
*  The lower case part is probably not needed.  We should really be converting 
*  The string to all uppercase/lowercase to make it case iNsEnSiTiVe.
*  What Flex will do is, For the input string, beginning from the front, Flex
*  will try to match with any of the defined tokens from below.  Flex will 
*  then match the string of longest length.  Suppose the string is: POINT ZM,
*  Flex would match both POINT Z and POINT ZM, but since POINT ZM is the longer
*  of the two tokens, FLEX will match POINT ZM.
*/

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* #include <unistd.h> */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int VanuatuWktlex_destroy (void );

int VanuatuWktget_debug (void );

void VanuatuWktset_debug (int debug_flag  );

YY_EXTRA_TYPE VanuatuWktget_extra (void );

void VanuatuWktset_extra (YY_EXTRA_TYPE user_defined  );

FILE *VanuatuWktget_in (void );

void VanuatuWktset_in  (FILE * in_str  );

FILE *VanuatuWktget_out (void );

void VanuatuWktset_out  (FILE * out_str  );

int VanuatuWktget_leng (void );

char *VanuatuWktget_text (void );

int VanuatuWktget_lineno (void );

void VanuatuWktset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int VanuatuWktwrap (void );
#else
extern int VanuatuWktwrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( VanuatuWkttext, VanuatuWktleng, 1, VanuatuWktout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( VanuatuWktin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( VanuatuWktin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, VanuatuWktin))==0 && ferror(VanuatuWktin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(VanuatuWktin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int VanuatuWktlex (void);

#define YY_DECL int VanuatuWktlex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after VanuatuWkttext and VanuatuWktleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! VanuatuWktin )
			VanuatuWktin = stdin;

		if ( ! VanuatuWktout )
			VanuatuWktout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			VanuatuWktensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				VanuatuWkt_create_buffer(VanuatuWktin,YY_BUF_SIZE );
		}

		VanuatuWkt_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of VanuatuWkttext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 114 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 255 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
{ vanuatu_col += (int) strlen(VanuatuWkttext);  VanuatuWktlval.dval = atof(VanuatuWkttext); return VANUATU_NUM; }
	YY_BREAK
case 2:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_COMMA; }
	YY_BREAK
case 3:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_OPEN_BRACKET; }
	YY_BREAK
case 4:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_CLOSE_BRACKET; }
	YY_BREAK
case 5:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POINT; }
	YY_BREAK
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POINT_Z; }
	YY_BREAK
case 7:
/* rule 7 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POINT_M; }
	YY_BREAK
case 8:
/* rule 8 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POINT_ZM; }
	YY_BREAK
case 9:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_LINESTRING; }
	YY_BREAK
case 10:
/* rule 10 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_LINESTRING_Z; }
	YY_BREAK
case 11:
/* rule 11 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_LINESTRING_M; }
	YY_BREAK
case 12:
/* rule 12 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_LINESTRING_ZM; }
	YY_BREAK
case 13:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POLYGON; }
	YY_BREAK
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POLYGON_Z; }
	YY_BREAK
case 15:
/* rule 15 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POLYGON_M; }
	YY_BREAK
case 16:
/* rule 16 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_POLYGON_ZM; }
	YY_BREAK
case 17:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOINT; }
	YY_BREAK
case 18:
/* rule 18 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOINT_Z; }
	YY_BREAK
case 19:
/* rule 19 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOINT_M; }
	YY_BREAK
case 20:
/* rule 20 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOINT_ZM; }
	YY_BREAK
case 21:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTILINESTRING; }
	YY_BREAK
case 22:
/* rule 22 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTILINESTRING_Z; }
	YY_BREAK
case 23:
/* rule 23 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTILINESTRING_M; }
	YY_BREAK
case 24:
/* rule 24 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTILINESTRING_ZM; }	
	YY_BREAK
case 25:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOLYGON; }
	YY_BREAK
case 26:
/* rule 26 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOLYGON_Z; }
	YY_BREAK
case 27:
/* rule 27 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOLYGON_M; }
	YY_BREAK
case 28:
/* rule 28 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_MULTIPOLYGON_ZM; }
	YY_BREAK
case 29:
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_GEOMETRYCOLLECTION; }
	YY_BREAK
case 30:
/* rule 30 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_GEOMETRYCOLLECTION_Z; }
	YY_BREAK
case 31:
/* rule 31 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_GEOMETRYCOLLECTION_M; }
	YY_BREAK
case 32:
/* rule 32 can match eol */
YY_RULE_SETUP
{ VanuatuWktlval.dval = 0; return VANUATU_GEOMETRYCOLLECTION_ZM; }
	YY_BREAK
case 33:
YY_RULE_SETUP
{ vanuatu_col += (int) strlen(VanuatuWkttext); }               /* ignore but count white space */
	YY_BREAK
case 34:
/* rule 34 can match eol */
YY_RULE_SETUP
{ vanuatu_col = 0; ++vanuatu_line; }
	YY_BREAK
case 35:
YY_RULE_SETUP
{ vanuatu_col += (int) strlen(VanuatuWkttext); return -1; }
	YY_BREAK
case 36:
YY_RULE_SETUP
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed VanuatuWktin at a new source and called
			 * VanuatuWktlex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = VanuatuWktin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( VanuatuWktwrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * VanuatuWkttext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of VanuatuWktlex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					VanuatuWktrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			VanuatuWktrestart(VanuatuWktin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) VanuatuWktrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 114 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 114 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 113);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up VanuatuWkttext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					VanuatuWktrestart(VanuatuWktin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( VanuatuWktwrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve VanuatuWkttext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void VanuatuWktrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        VanuatuWktensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            VanuatuWkt_create_buffer(VanuatuWktin,YY_BUF_SIZE );
	}

	VanuatuWkt_init_buffer(YY_CURRENT_BUFFER,input_file );
	VanuatuWkt_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void VanuatuWkt_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		VanuatuWktpop_buffer_state();
	 *		VanuatuWktpush_buffer_state(new_buffer);
     */
	VanuatuWktensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	VanuatuWkt_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (VanuatuWktwrap()) processing, but the only time this flag
	 * is looked at is after VanuatuWktwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void VanuatuWkt_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	VanuatuWktin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE VanuatuWkt_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) VanuatuWktalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in VanuatuWkt_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) VanuatuWktalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in VanuatuWkt_create_buffer()" );

	b->yy_is_our_buffer = 1;

	VanuatuWkt_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with VanuatuWkt_create_buffer()
 * 
 */
    void VanuatuWkt_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		VanuatuWktfree((void *) b->yy_ch_buf  );

	VanuatuWktfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a VanuatuWktrestart() or at EOF.
 */
    static void VanuatuWkt_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	VanuatuWkt_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then VanuatuWkt_init_buffer was _probably_
     * called from VanuatuWktrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void VanuatuWkt_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		VanuatuWkt_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void VanuatuWktpush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	VanuatuWktensure_buffer_stack();

	/* This block is copied from VanuatuWkt_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from VanuatuWkt_switch_to_buffer. */
	VanuatuWkt_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void VanuatuWktpop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	VanuatuWkt_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		VanuatuWkt_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void VanuatuWktensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)VanuatuWktalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in VanuatuWktensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)VanuatuWktrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in VanuatuWktensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE VanuatuWkt_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) VanuatuWktalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in VanuatuWkt_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	VanuatuWkt_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to VanuatuWktlex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       VanuatuWkt_scan_bytes() instead.
 */
YY_BUFFER_STATE VanuatuWkt_scan_string (yyconst char * yystr )
{
    
	return VanuatuWkt_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to VanuatuWktlex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE VanuatuWkt_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) VanuatuWktalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in VanuatuWkt_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = VanuatuWkt_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in VanuatuWkt_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up VanuatuWkttext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		VanuatuWkttext[VanuatuWktleng] = (yy_hold_char); \
		(yy_c_buf_p) = VanuatuWkttext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		VanuatuWktleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int VanuatuWktget_lineno  (void)
{
        
    return VanuatuWktlineno;
}

/** Get the input stream.
 * 
 */
FILE *VanuatuWktget_in  (void)
{
        return VanuatuWktin;
}

/** Get the output stream.
 * 
 */
FILE *VanuatuWktget_out  (void)
{
        return VanuatuWktout;
}

/** Get the length of the current token.
 * 
 */
int VanuatuWktget_leng  (void)
{
        return VanuatuWktleng;
}

/** Get the current token.
 * 
 */

char *VanuatuWktget_text  (void)
{
        return VanuatuWkttext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void VanuatuWktset_lineno (int  line_number )
{
    
    VanuatuWktlineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see VanuatuWkt_switch_to_buffer
 */
void VanuatuWktset_in (FILE *  in_str )
{
        VanuatuWktin = in_str ;
}

void VanuatuWktset_out (FILE *  out_str )
{
        VanuatuWktout = out_str ;
}

int VanuatuWktget_debug  (void)
{
        return VanuatuWkt_flex_debug;
}

void VanuatuWktset_debug (int  bdebug )
{
        VanuatuWkt_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from VanuatuWktlex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    VanuatuWktin = stdin;
    VanuatuWktout = stdout;
#else
    VanuatuWktin = (FILE *) 0;
    VanuatuWktout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * VanuatuWktlex_init()
     */
    return 0;
}

/* VanuatuWktlex_destroy is for both reentrant and non-reentrant scanners. */
int VanuatuWktlex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		VanuatuWkt_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		VanuatuWktpop_buffer_state();
	}

	/* Destroy the stack itself. */
	VanuatuWktfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * VanuatuWktlex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *VanuatuWktalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *VanuatuWktrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void VanuatuWktfree (void * ptr )
{
	free( (char *) ptr );	/* see VanuatuWktrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

/**
 * reset the line and column count
 *
 *
 */
void vanuatu_reset_lexer(void)
{

  vanuatu_line = 1;
  vanuatu_col  = 1;

}

/**
 * VanuatuWkterror() is invoked when the lexer or the parser encounter
 * an error. The error message is passed via *s
 *
 *
 */
void VanuatuWkterror(char *s)
{
  printf("error: %s at line: %d col: %d\n",s,vanuatu_line,vanuatu_col);

}

int VanuatuWktwrap(void)
{
  return 1;
}

/******************************************************************************
** This is the end of the code that was created by Team Vanuatu
** of The University of Toronto.

Authors:
Ruppi Rana			ruppi.rana@gmail.com
Dev Tanna			dev.tanna@gmail.com
Elias Adum			elias.adum@gmail.com
Benton Hui			benton.hui@gmail.com
Abhayan Sundararajan		abhayan@gmail.com
Chee-Lun Michael Stephen Cho	cheelun.cho@gmail.com
Nikola Banovic			nikola.banovic@gmail.com
Yong Jian			yong.jian@utoronto.ca

Supervisor:
Greg Wilson			gvwilson@cs.toronto.ca

-------------------------------------------------------------------------------
*/


/* 
 VANUATU_FLEX_END - FLEX generated code ends here 
*/



/*
** This is a linked-list struct to store all the values for each token.
** All tokens will have a value of 0, except tokens denoted as NUM.
** NUM tokens are geometry coordinates and will contain the floating
** point number.
*/
typedef struct vanuatuFlexTokenStruct
{
    double value;
    struct vanuatuFlexTokenStruct *Next;
} vanuatuFlexToken;

/*
** Function to clean up the linked-list of token values.
*/
static int
vanuatu_cleanup (vanuatuFlexToken * token)
{
    vanuatuFlexToken *ptok;
    vanuatuFlexToken *ptok_n;
    if (token == NULL)
	return 0;
    ptok = token;
    while (ptok)
      {
	  ptok_n = ptok->Next;
	  free (ptok);
	  ptok = ptok_n;
      }
    return 0;
}

gaiaGeomCollPtr
gaiaParseWkt (const unsigned char *dirty_buffer, short type)
{
    void *pParser = ParseAlloc (malloc);
    /* Linked-list of token values */
    vanuatuFlexToken *tokens = malloc (sizeof (vanuatuFlexToken));
    /* Pointer to the head of the list */
    vanuatuFlexToken *head = tokens;
    int yv;
    gaiaGeomCollPtr result = NULL;

    tokens->Next = NULL;
    /*
     ** Sandro Furieri 2010 Apr 4
     ** unsetting the parser error flag
     */
    vanuatu_parse_error = 0;

    VanuatuWkt_scan_string ((char *) dirty_buffer);

    /*
       / Keep tokenizing until we reach the end
       / yylex() will return the next matching Token for us.
     */
    while ((yv = yylex ()) != 0)
      {
	  if (yv == -1)
	    {
		vanuatu_parse_error = 1;
		break;
	    }
	  tokens->Next = malloc (sizeof (vanuatuFlexToken));
	  tokens->Next->Next = NULL;
	  /*
	     /VanuatuWktlval is a global variable from FLEX.
	     /VanuatuWktlval is defined in vanuatuLexglobal.h
	   */
	  tokens->Next->value = VanuatuWktlval.dval;
	  /* Pass the token to the wkt parser created from lemon */
	  Parse (pParser, yv, &(tokens->Next->value), &result);
	  tokens = tokens->Next;
      }
    /* This denotes the end of a line as well as the end of the parser */
    Parse (pParser, VANUATU_NEWLINE, 0, &result);
    ParseFree (pParser, free);
    VanuatuWktlex_destroy ();

    /* Assigning the token as the end to avoid seg faults while cleaning */
    tokens->Next = NULL;
    vanuatu_cleanup (head);

    /*
     ** Sandro Furieri 2010 Apr 4
     ** checking if any parsing error was encountered 
     */
    if (vanuatu_parse_error)
      {
	  if (result)
	      gaiaFreeGeomColl (result);
	  return NULL;
      }

    /*
     ** Sandro Furieri 2010 Apr 4
     ** final checkup for validity
     */
    if (!result)
	return NULL;
    if (!vanuatuCheckValidity (result))
      {
	  gaiaFreeGeomColl (result);
	  return NULL;
      }
    if (type < 0)
	;			/* no restrinction about GEOMETRY CLASS TYPE */
    else
      {
	  if (result->DeclaredType != type)
	    {
		/* invalid CLASS TYPE for request */
		gaiaFreeGeomColl (result);
		return NULL;
	    }
      }

    gaiaMbrGeometry (result);

    return result;
}

/******************************************************************************
** This is the end of the code that was created by Team Vanuatu 
** of The University of Toronto.

Authors:
Ruppi Rana			ruppi.rana@gmail.com
Dev Tanna			dev.tanna@gmail.com
Elias Adum			elias.adum@gmail.com
Benton Hui			benton.hui@gmail.com
Abhayan Sundararajan		abhayan@gmail.com
Chee-Lun Michael Stephen Cho	cheelun.cho@gmail.com
Nikola Banovic			nikola.banovic@gmail.com
Yong Jian			yong.jian@utoronto.ca

Supervisor:
Greg Wilson			gvwilson@cs.toronto.ca

-------------------------------------------------------------------------------

*/

/*
** CAVEAT: we must now undefine any Lemon/Flex own macro
*/
#undef YYNOCODE
#undef YYNSTATE
#undef YYNRULE
#undef YY_SHIFT_MAX
#undef YY_SHIFT_USE_DFLT
#undef YY_REDUCE_USE_DFLT
#undef YY_REDUCE_MAX
#undef YY_FLUSH_BUFFER
#undef YY_DO_BEFORE_ACTION
#undef YY_NUM_RULES
#undef YY_END_OF_BUFFER
#undef YY_END_FILE
#undef YYACTIONTYPE
#undef YY_SZ_ACTTAB
#undef YY_NEW_FILE
#undef BEGIN
#undef YY_START
#undef YY_CURRENT_BUFFER
#undef YY_CURRENT_BUFFER_LVALUE
#undef YY_STATE_BUF_SIZE
#undef YY_DECL
#undef YY_FATAL_ERROR
#undef YYMINORTYPE
#undef YY_CHAR
#undef YYSTYPE
#undef input
#undef ParseAlloc
#undef ParseFree
#undef ParseStackPeak
#undef Parse
#undef yyalloc
#undef yyfree
#undef yyin
#undef yyleng
#undef yyless
#undef yylex
#undef yylineno
#undef yyout
#undef yyrealloc
#undef yyrestart
#undef yyStackEntry
#undef yytext
#undef yywrap
#undef yyzerominor
#undef yy_accept
#undef yy_action
#undef yy_base
#undef yy_buffer_stack
#undef yy_buffer_stack_max
#undef yy_buffer_stack_top
#undef yy_c_buf_p
#undef yy_chk
#undef yy_create_buffer
#undef yy_def
#undef yy_default
#undef yy_delete_buffer
#undef yy_destructor
#undef yy_ec
#undef yy_fatal_error
#undef yy_find_reduce_action
#undef yy_find_shift_action
#undef yy_flex_debug
#undef yy_flush_buffer
#undef yy_get_next_buffer
#undef yy_get_previous_state
#undef yy_init
#undef yy_init_buffer
#undef yy_init_globals
#undef yy_load_buffer
#undef yy_load_buffer_state
#undef yy_lookahead
#undef yy_meta
#undef yy_new_buffer
#undef yy_nxt
#undef yy_parse_failed
#undef yy_pop_parser_stack
#undef yy_reduce
#undef yy_reduce_ofst
#undef yy_set_bol
#undef yy_set_interactive
#undef yy_shift
#undef yy_shift_ofst
#undef yy_start
#undef yy_state_type
#undef yy_switch_to_buffer
#undef yy_syntax_error
#undef yy_trans_info
#undef yy_try_NUL_trans
#undef yyParser
#undef yyStackEntry
#undef yyStackOverflow
#undef yyRuleInfo
#undef yytext_ptr
#undef yyunput
#undef yyzerominor
#undef ParseARG_SDECL
#undef ParseARG_PDECL
#undef ParseARG_FETCH
#undef ParseARG_STORE
#undef REJECT
#undef yymore
#undef YY_MORE_ADJ
#undef YY_RESTORE_YY_MORE_OFFSET
#undef YY_LESS_LINENO
#undef yyTracePrompt
#undef yyTraceFILE
#undef yyTokenName
#undef yyRuleName
#undef ParseTrace
/**************** End file: gg_vanuatu.c **********/


/**************** Begin file: gg_ewkt.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

/* #include <assert.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

#ifdef _WIN32
#define strcasecmp	_stricmp
#define strncasecmp	_strnicmp
#define atoll	_atoi64
#endif /* not WIN32 */

#if defined(_WIN32) || defined(WIN32)
# include <io.h>
#define isatty	_isatty
#define fileno	_fileno
#endif

int ewkt_parse_error;

static int
ewktCheckValidity (gaiaGeomCollPtr geom)
{
/* checks if this one is a degenerated geometry */
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int ib;
    int entities = 0;
    pt = geom->FirstPoint;
    while (pt)
      {
	  /* checking points */
	  entities++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* checking linestrings */
	  if (ln->Points < 2)
	      return 0;
	  entities++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  /* checking polygons */
	  rng = pg->Exterior;
	  if (rng->Points < 4)
	      return 0;
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		if (rng->Points < 4)
		    return 0;
	    }
	  entities++;
	  pg = pg->Next;
      }
    if (!entities)
	return 0;
    return 1;
}

static gaiaGeomCollPtr
gaiaEwktGeometryFromPoint (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINT */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomColl ();
    geom->DeclaredType = GAIA_POINT;
    gaiaAddPointToGeomColl (geom, point->X, point->Y);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaEwktGeometryFromPointZ (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINTZ */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomCollXYZ ();
    geom->DeclaredType = GAIA_POINTZ;
    gaiaAddPointToGeomCollXYZ (geom, point->X, point->Y, point->Z);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaEwktGeometryFromPointM (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINTM */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomCollXYM ();
    geom->DeclaredType = GAIA_POINTM;
    gaiaAddPointToGeomCollXYM (geom, point->X, point->Y, point->M);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaEwktGeometryFromPointZM (gaiaPointPtr point)
{
/* builds a GEOMETRY containing a POINTZM */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomCollXYZM ();
    geom->DeclaredType = GAIA_POINTZM;
    gaiaAddPointToGeomCollXYZM (geom, point->X, point->Y, point->Z, point->M);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaEwktGeometryFromLinestring (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRING */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    geom = gaiaAllocGeomColl ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPoint (line->Coords, iv, &x, &y);
	  gaiaSetPoint (line2->Coords, iv, x, y);
      }
    gaiaFreeLinestring (line);
    return geom;
}

static gaiaGeomCollPtr
gaiaEwktGeometryFromLinestringZ (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRINGZ */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    double z;
    geom = gaiaAllocGeomCollXYZ ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	  gaiaSetPointXYZ (line2->Coords, iv, x, y, z);
      }
    gaiaFreeLinestring (line);
    return geom;
}


static gaiaGeomCollPtr
gaiaEwktGeometryFromLinestringM (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRINGM */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    double m;
    geom = gaiaAllocGeomCollXYM ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPointXYM (line->Coords, iv, &x, &y, &m);
	  gaiaSetPointXYM (line2->Coords, iv, x, y, m);
      }
    gaiaFreeLinestring (line);
    return geom;
}

static gaiaGeomCollPtr
gaiaEwktGeometryFromLinestringZM (gaiaLinestringPtr line)
{
/* builds a GEOMETRY containing a LINESTRINGZM */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    double z;
    double m;
    geom = gaiaAllocGeomCollXYZM ();
    geom->DeclaredType = GAIA_LINESTRING;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPointXYZM (line->Coords, iv, &x, &y, &z, &m);
	  gaiaSetPointXYZM (line2->Coords, iv, x, y, z, m);
      }
    gaiaFreeLinestring (line);
    return geom;
}

static gaiaPointPtr
ewkt_point_xy (double *x, double *y)
{
    return gaiaAllocPoint (*x, *y);
}

/* 
 * Creates a 3D (xyz) point in SpatiaLite
 * x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created.
 * Returns a gaiaPointPtr representing the created point.
 *
 * Creates a 3D (xyz) point. This is a parser helper function which is called when 3D coordinates are encountered.
 * Parameters x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created.
 * Returns a gaiaPointPtr pointing to the 3D created point.
 */
static gaiaPointPtr
ewkt_point_xyz (double *x, double *y, double *z)
{
    return gaiaAllocPointXYZ (*x, *y, *z);
}

/* 
 * Creates a 2D (xy) point with an m value which is a part of the linear reference system. This is a parser helper
 * function which is called when 2D   *coordinates with an m value are encountered.
 * Parameters x and y are pointers to doubles which represent the x and y coordinates of the point to be created.
 * Parameter m is a pointer to a double which represents the part of the linear reference system.
 * Returns a gaiaPointPtr pointing to the created 2D point with an m value.
 */
static gaiaPointPtr
ewkt_point_xym (double *x, double *y, double *m)
{
    return gaiaAllocPointXYM (*x, *y, *m);
}

/* 
 * Creates a 4D (xyz) point with an m value which is a part of the linear reference system. This is a parser helper
 * function which is called when  *4Dcoordinates with an m value are encountered
 * Parameters x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created. 
 * Parameter m is a pointer to a double which represents the part of the linear reference system.
 * Returns a gaiaPointPtr pointing the created 4D point with an m value.
 */
gaiaPointPtr
ewkt_point_xyzm (double *x, double *y, double *z, double *m)
{
    return gaiaAllocPointXYZM (*x, *y, *z, *m);
}

/*
 * Builds a geometry collection from a point. The geometry collection should contain only one element ? the point. 
 * The correct geometry type must be *decided based on the point type. The parser should call this function when the 
 * ?POINT? WKT expression is encountered.
 * Parameter point is a pointer to a 2D, 3D, 2D with an m value, or 4D with an m value point.
 * Returns a geometry collection containing the point. The geometry must have FirstPoint and LastPoint  pointing to the
 * same place as point.  *DimensionModel must be the same as the model of the point and DimensionType must be GAIA_TYPE_POINT.
 */
static gaiaGeomCollPtr
ewkt_buildGeomFromPoint (gaiaPointPtr point)
{
    switch (point->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaEwktGeometryFromPoint (point);
	  break;
      case GAIA_XY_Z:
	  return gaiaEwktGeometryFromPointZ (point);
	  break;
      case GAIA_XY_M:
	  return gaiaEwktGeometryFromPointM (point);
	  break;
      case GAIA_XY_Z_M:
	  return gaiaEwktGeometryFromPointZM (point);
	  break;
      }
    return NULL;
}

/* 
 * Creates a 2D (xy) linestring from a list of 2D points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 2D (xy) points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
ewkt_linestring_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestring (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPoint (linestring->Coords, i, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/* 
 * Creates a 3D (xyz) linestring from a list of 3D points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 3D (xyz) points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
ewkt_linestring_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestringXYZ (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPointXYZ (linestring->Coords, i, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/* 
 * Creates a 2D (xy) with m value linestring from a list of 2D with m value points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 2D (xy) with m value points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
ewkt_linestring_xym (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestringXYM (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPointXYM (linestring->Coords, i, p->X, p->Y, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/* 
 * Creates a 4D (xyz) with m value linestring from a list of 4D with m value points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 4D (xyz) with m value points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
ewkt_linestring_xyzm (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestringXYZM (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPointXYZM (linestring->Coords, i, p->X, p->Y, p->Z, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/*
 * Builds a geometry collection from a linestring.
 */
static gaiaGeomCollPtr
ewkt_buildGeomFromLinestring (gaiaLinestringPtr line)
{
    switch (line->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaEwktGeometryFromLinestring (line);
	  break;
      case GAIA_XY_Z:
	  return gaiaEwktGeometryFromLinestringZ (line);
	  break;
      case GAIA_XY_M:
	  return gaiaEwktGeometryFromLinestringM (line);
	  break;
      case GAIA_XY_Z_M:
	  return gaiaEwktGeometryFromLinestringZM (line);
	  break;
      }
    return NULL;
}

/*
 * Helper function that determines the number of points in the linked list.
 */
static int
ewkt_count_points (gaiaPointPtr first)
{
    /* Counts the number of points in the ring. */
    gaiaPointPtr p = first;
    int numpoints = 0;
    while (p != NULL)
      {
	  numpoints++;
	  p = p->Next;
      }
    return numpoints;
}

/*
 * Creates a 2D (xy) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 2D (xy) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
ewkt_ring_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = ewkt_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRing (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPoint (ring->Coords, index, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Creates a 3D (xyz) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 3D (xyz) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
ewkt_ring_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = ewkt_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRingXYZ (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPointXYZ (ring->Coords, index, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Creates a 2D (xym) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 2D (xym) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
ewkt_ring_xym (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = ewkt_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRingXYM (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPointXYM (ring->Coords, index, p->X, p->Y, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Creates a 3D (xyzm) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 3D (xyzm) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
ewkt_ring_xyzm (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = ewkt_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRingXYZM (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPointXYZM (ring->Coords, index, p->X, p->Y, p->Z, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Helper function that will create any type of polygon (xy, xym, xyz, xyzm) in SpatiaLite.
 * 
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are of the same type. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
ewkt_polygon_any_type (gaiaRingPtr first)
{
    gaiaRingPtr p;
    gaiaRingPtr p_n;
    gaiaPolygonPtr polygon;
    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a polygon structure with the exterior ring. */
    polygon = gaiaCreatePolygon (first);
    if (polygon == NULL)
	return NULL;

    /* Adds all interior rings into the polygon structure. */
    p = first;
    while (p != NULL)
      {
	  p_n = p->Next;
	  if (p == first)
	      gaiaFreeRing (p);
	  else
	      gaiaAddRingToPolyg (polygon, p);
	  p = p_n;
      }

    return polygon;
}

/* 
 * Creates a 2D (xy) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 2D (xy) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
ewkt_polygon_xy (gaiaRingPtr first)
{
    return ewkt_polygon_any_type (first);
}

/* 
 * Creates a 3D (xyz) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 3D (xyz) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
ewkt_polygon_xyz (gaiaRingPtr first)
{
    return ewkt_polygon_any_type (first);
}

/* 
 * Creates a 2D (xym) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 2D (xym) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
ewkt_polygon_xym (gaiaRingPtr first)
{
    return ewkt_polygon_any_type (first);
}

/* 
 * Creates a 3D (xyzm) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 3D (xyzm) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
ewkt_polygon_xyzm (gaiaRingPtr first)
{
    return ewkt_polygon_any_type (first);
}

/*
 * Builds a geometry collection from a polygon.
 * NOTE: This function may already be implemented in the SpatiaLite code base. If it is, make sure that we
 *              can use it (ie. it doesn't use any other variables or anything else set by Sandro's parser). If you find
 *              that we can use an existing function then ignore this one.
 */
static gaiaGeomCollPtr
ewkt_buildGeomFromPolygon (gaiaPolygonPtr polygon)
{
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (polygon == NULL)
      {
	  return NULL;
      }

    /* Creates and allocates a geometry collection containing a multipoint. */
    switch (polygon->DimensionModel)
      {
      case GAIA_XY:
	  geom = gaiaAllocGeomColl ();
	  break;
      case GAIA_XY_Z:
	  geom = gaiaAllocGeomCollXYZ ();
	  break;
      case GAIA_XY_M:
	  geom = gaiaAllocGeomCollXYM ();
	  break;
      case GAIA_XY_Z_M:
	  geom = gaiaAllocGeomCollXYZM ();
	  break;
      }
    if (geom == NULL)
      {
	  return NULL;
      }
    geom->DeclaredType = GAIA_POLYGON;

    /* Stores the location of the first and last polygons in the linked list. */
    geom->FirstPolygon = polygon;
    while (polygon != NULL)
      {
	  geom->LastPolygon = polygon;
	  polygon = polygon->Next;
      }
    return geom;
}

/* 
 * Creates a 2D (xy) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 2D (xy) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
ewkt_multipoint_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 2D (xy) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomColl (geom, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/* 
 * Creates a 3D (xyz) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 3D (xyz) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
ewkt_multipoint_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomCollXYZ ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 3D (xyz) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomCollXYZ (geom, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/* 
 * Creates a 2D (xym) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 2D (xym) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
ewkt_multipoint_xym (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomCollXYM ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 2D (xym) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomCollXYM (geom, p->X, p->Y, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/* 
 * Creates a 3D (xyzm) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points given to the function are 3D (xyzm) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
ewkt_multipoint_xyzm (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomCollXYZM ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 3D (xyzm) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomCollXYZM (geom, p->X, p->Y, p->Z, p->M);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/*
 * Creates a geometry collection containing 2D (xy) linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 2D (xy) linestrings. There must be at least 1 linestring in the list.
 * Returns a pointer to the created geometry collection of 2D linestrings. The geometry must have FirstLinestring pointing to the
 * first linestring in the list pointed by first and LastLinestring pointing to the last element of the same list. DimensionModel
 * must be GAIA_XY and DimensionType must be *GAIA_TYPE_LINESTRING.
 */

static gaiaGeomCollPtr
ewkt_multilinestring_xy (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomColl ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }

    return a;
}

/* 
 * Returns a geometry collection containing the created multilinestring object (?).
 * Creates a geometry collection containing 3D (xyz) linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 3D (xyz) linestrings. There must be at least 1 linestring in the list.
 * Returns a pointer to the created geometry collection of 3D linestrings. The geometry must have FirstLinestring pointing to the
 * first linestring in the *list pointed by first and LastLinestring pointing to the last element of the same list. DimensionModel
 * must be GAIA_XYZ and DimensionType must be *GAIA_TYPE_LINESTRING.
 */
static gaiaGeomCollPtr
ewkt_multilinestring_xyz (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomCollXYZ ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY_Z;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }
    return a;
}

/* 
 * Creates a geometry collection containing 2D (xy) with m value linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the  *linestrings in the list must be 2D (xy) with m value linestrings. There must be at least 1 linestring 
 * in the list.
 * Returns a pointer to the created geometry collection of 2D with m value linestrings. The geometry must have FirstLinestring
 * pointing to the first *linestring in the list pointed by first and LastLinestring pointing to the last element of the same list. 
 * DimensionModel must be GAIA_XYM and *DimensionType must be GAIA_TYPE_LINESTRING.
 */
static gaiaGeomCollPtr
ewkt_multilinestring_xym (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomCollXYM ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY_M;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }

    return a;
}

/* 
 * Creates a geometry collection containing 4D (xyz) with m value linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 4D (xyz) with m value linestrings. There must be at least 1 linestring
 * in the list.
 * Returns a pointer to the created geometry collection of 4D with m value linestrings. The geometry must have FirstLinestring
 * pointing to the first *linestring in the list pointed by first and LastLinestring pointing to the last element of the same list. 
 * DimensionModel must be GAIA_XYZM and *DimensionType must be GAIA_TYPE_LINESTRING.
 */
static gaiaGeomCollPtr
ewkt_multilinestring_xyzm (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomCollXYZM ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY_Z_M;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }
    return a;
}

/* 
 * Creates a geometry collection containing 2D (xy) polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should 
 * be added to the collection. All of the polygons in the list must be 2D (xy) polygons. There must be 
 * at least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 2D polygons. The geometry must have 
 * FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing 
 * to the last element of the same list. DimensionModel must be GAIA_XY and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
ewkt_multipolygon_xy (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

/* 
 * Creates a geometry collection containing 3D (xyz) polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should be 
 * added to the collection. All of the polygons in the list must be 3D (xyz) polygons. There must be at
 * least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 3D polygons. The geometry must have 
 * FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing to 
 * the last element of the same list. DimensionModel must be GAIA_XYZ and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
ewkt_multipolygon_xyz (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomCollXYZ ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

/* 
 * Creates a geometry collection containing 2D (xy) with m value polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should
 * be added to the collection. All of the polygons in the list must be 2D (xy) with m value polygons.
 * There must be at least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 2D with m value polygons. The geometry 
 * must have FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon 
 * pointing to the last element of the same list. DimensionModel must be GAIA_XYM and DimensionType 
 * must be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
ewkt_multipolygon_xym (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomCollXYM ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

/*
 * Creates a geometry collection containing 4D (xyz) with m value polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should be 
 * added to the collection. All of the polygons in the list must be 4D (xyz) with m value polygons. 
 * There must be at least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 4D with m value polygons. The geometry must 
 * have FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing 
//  * to the last element of the same list. DimensionModel must be GAIA_XYZM and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
ewkt_multipolygon_xyzm (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomCollXYZM ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

static void
ewkt_geomColl_common (gaiaGeomCollPtr org, gaiaGeomCollPtr dst)
{
/* 
/ helper function: xfers entities between the Origin and Destination 
/ Sandro Furieri: 2010 October 12
*/
    gaiaGeomCollPtr p = org;
    gaiaGeomCollPtr p_n;
    gaiaPointPtr pt;
    gaiaPointPtr pt_n;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr ln_n;
    gaiaPolygonPtr pg;
    gaiaPolygonPtr pg_n;
    while (p)
      {
	  pt = p->FirstPoint;
	  while (pt)
	    {
		pt_n = pt->Next;
		pt->Next = NULL;
		if (dst->FirstPoint == NULL)
		    dst->FirstPoint = pt;
		if (dst->LastPoint != NULL)
		    dst->LastPoint->Next = pt;
		dst->LastPoint = pt;
		pt = pt_n;
	    }
	  ln = p->FirstLinestring;
	  while (ln)
	    {
		ln_n = ln->Next;
		ln->Next = NULL;
		if (dst->FirstLinestring == NULL)
		    dst->FirstLinestring = ln;
		if (dst->LastLinestring != NULL)
		    dst->LastLinestring->Next = ln;
		dst->LastLinestring = ln;
		ln = ln_n;
	    }
	  pg = p->FirstPolygon;
	  while (pg)
	    {
		pg_n = pg->Next;
		pg->Next = NULL;
		if (dst->FirstPolygon == NULL)
		    dst->FirstPolygon = pg;
		if (dst->LastPolygon != NULL)
		    dst->LastPolygon->Next = pg;
		dst->LastPolygon = pg;
		pg = pg_n;
	    }
	  p_n = p->Next;
	  p->FirstPoint = NULL;
	  p->LastPoint = NULL;
	  p->FirstLinestring = NULL;
	  p->LastLinestring = NULL;
	  p->FirstPolygon = NULL;
	  p->LastPolygon = NULL;
	  gaiaFreeGeomColl (p);
	  p = p_n;
      }
}

/* Creates a 2D (xy) geometry collection in SpatiaLite
 *
 * first is the first geometry collection in a linked list of geometry collections.
 * Each geometry collection represents a single type of object (eg. one could be a POINT, 
 * another could be a LINESTRING, another could be a MULTILINESTRING, etc.).
 *
 * The type of object represented by any geometry collection is stored in the declaredType 
 * field of its struct. For example, if first->declaredType = GAIA_POINT, then first represents a point.
 * If first->declaredType = GAIA_MULTIPOINT, then first represents a multipoint.
 *
 * NOTE: geometry collections cannot contain other geometry collections (have to confirm this 
 * with Sandro).
 *
 * The goal of this function is to take the information from all of the structs in the linked list and 
 * return one geomColl struct containing all of that information.
 *
 * The integers used for 'declaredType' are defined in gaiageo.h. In this function, the only values 
 * contained in 'declaredType' that will be encountered will be:
 *
 *	GAIA_POINT, GAIA_LINESTRING, GAIA_POLYGON, 
 *	GAIA_MULTIPOINT, GAIA_MULTILINESTRING, GAIA_MULTIPOLYGON
 */
static gaiaGeomCollPtr
ewkt_geomColl_xy (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY;
    ewkt_geomColl_common (first, geom);
    return geom;
}

/*
 * See geomColl_xy for description.
 *
 * The only difference between this function and geomColl_xy is that the 'declaredType' field of the structs
 * in the linked list for this function will only contain the following types:
 *
 *	GAIA_POINTZ, GAIA_LINESTRINGZ, GAIA_POLYGONZ,
 * 	GAIA_MULTIPOINTZ, GAIA_MULTILINESTRINGZ, GAIA_MULTIPOLYGONZ
 */
static gaiaGeomCollPtr
ewkt_geomColl_xyz (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY_Z;
    ewkt_geomColl_common (first, geom);
    return geom;
}

/*
 * See geomColl_xy for description.
 *
 * The only difference between this function and geomColl_xy is that the 'declaredType' field of the structs
 * in the linked list for this function will only contain the following types:
 *
 *	GAIA_POINTM, GAIA_LINESTRINGM, GAIA_POLYGONM,
 * 	GAIA_MULTIPOINTM, GAIA_MULTILINESTRINGM, GAIA_MULTIPOLYGONM
 */
static gaiaGeomCollPtr
ewkt_geomColl_xym (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY_M;
    ewkt_geomColl_common (first, geom);
    return geom;
}

/*
 * See geomColl_xy for description.
 *
 * The only difference between this function and geomColl_xy is that the 'declaredType' field of the structs
 * in the linked list for this function will only contain the following types:
 *
 *	GAIA_POINTZM, GAIA_LINESTRINGZM, GAIA_POLYGONZM,
 * 	GAIA_MULTIPOINTZM, GAIA_MULTILINESTRINGZM, GAIA_MULTIPOLYGONZM
 */
static gaiaGeomCollPtr
ewkt_geomColl_xyzm (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY_Z_M;
    ewkt_geomColl_common (first, geom);
    return geom;
}



/*
** CAVEAT: we must redefine any Lemon/Flex own macro
*/
#define YYMINORTYPE		EWKT_MINORTYPE
#define YY_CHAR			EWKT_YY_CHAR
#define	input			ewkt_input
#define ParseAlloc		ewktParseAlloc
#define ParseFree		ewktParseFree
#define ParseStackPeak		ewktParseStackPeak
#define Parse			ewktParse
#define yyStackEntry		ewkt_yyStackEntry
#define yyzerominor		ewkt_yyzerominor
#define yy_accept		ewkt_yy_accept
#define yy_action		ewkt_yy_action
#define yy_base			ewkt_yy_base
#define yy_buffer_stack		ewkt_yy_buffer_stack
#define yy_buffer_stack_max	ewkt_yy_buffer_stack_max
#define yy_buffer_stack_top	ewkt_yy_buffer_stack_top
#define yy_c_buf_p		ewkt_yy_c_buf_p
#define yy_chk			ewkt_yy_chk
#define yy_def			ewkt_yy_def
#define yy_default		ewkt_yy_default
#define yy_destructor		ewkt_yy_destructor
#define yy_ec			ewkt_yy_ec
#define yy_fatal_error		ewkt_yy_fatal_error
#define yy_find_reduce_action	ewkt_yy_find_reduce_action
#define yy_find_shift_action	ewkt_yy_find_shift_action
#define yy_get_next_buffer	ewkt_yy_get_next_buffer
#define yy_get_previous_state	ewkt_yy_get_previous_state
#define yy_init			ewkt_yy_init
#define yy_init_globals		ewkt_yy_init_globals
#define yy_lookahead		ewkt_yy_lookahead
#define yy_meta			ewkt_yy_meta
#define yy_nxt			ewkt_yy_nxt
#define yy_parse_failed		ewkt_yy_parse_failed
#define yy_pop_parser_stack	ewkt_yy_pop_parser_stack
#define yy_reduce		ewkt_yy_reduce
#define yy_reduce_ofst		ewkt_yy_reduce_ofst
#define yy_shift		ewkt_yy_shift
#define yy_shift_ofst		ewkt_yy_shift_ofst
#define yy_start		ewkt_yy_start
#define yy_state_type		ewkt_yy_state_type
#define yy_syntax_error		ewkt_yy_syntax_error
#define yy_trans_info		ewkt_yy_trans_info
#define yy_try_NUL_trans	ewkt_yy_try_NUL_trans
#define yyParser		ewkt_yyParser
#define yyStackEntry		ewkt_yyStackEntry
#define yyStackOverflow		ewkt_yyStackOverflow
#define yyRuleInfo		ewkt_yyRuleInfo
#define yyunput			ewkt_yyunput
#define yyzerominor		ewkt_yyzerominor
#define yyTraceFILE		ewkt_yyTraceFILE
#define yyTracePrompt		ewkt_yyTracePrompt
#define yyTokenName		ewkt_yyTokenName
#define yyRuleName		ewkt_yyRuleName
#define ParseTrace		ewkt_ParseTrace


/*
 EWKT_LEMON_H_START - LEMON generated header starts here 
*/
#define EWKT_NEWLINE                    1
#define EWKT_POINT                      2
#define EWKT_OPEN_BRACKET               3
#define EWKT_CLOSE_BRACKET              4
#define EWKT_POINT_M                    5
#define EWKT_NUM                        6
#define EWKT_COMMA                      7
#define EWKT_LINESTRING                 8
#define EWKT_LINESTRING_M               9
#define EWKT_POLYGON                   10
#define EWKT_POLYGON_M                 11
#define EWKT_MULTIPOINT                12
#define EWKT_MULTIPOINT_M              13
#define EWKT_MULTILINESTRING           14
#define EWKT_MULTILINESTRING_M         15
#define EWKT_MULTIPOLYGON              16
#define EWKT_MULTIPOLYGON_M            17
#define EWKT_GEOMETRYCOLLECTION        18
#define EWKT_GEOMETRYCOLLECTION_M      19
/*
 EWKT_LEMON_H_END - LEMON generated header ends here 
*/



typedef union
{
    double dval;
    struct symtab *symp;
} ewkt_yystype;
#define YYSTYPE ewkt_yystype

/* extern YYSTYPE yylval; */
YYSTYPE EwktLval;



/*
 EWKT_LEMON_START - LEMON generated code starts here 
*/

/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is included that follows the "include" declaration
** in the input grammar file. */
/* #include <stdio.h> */

/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash 
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    ParseTOKENTYPE     is the data type used for minor tokens given 
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is ParseTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    ParseARG_SDECL     A static variable declaration for the %extra_argument
**    ParseARG_PDECL     A parameter declaration for the %extra_argument
**    ParseARG_STORE     Code to store %extra_argument into yypParser
**    ParseARG_FETCH     Code to extract %extra_argument from yypParser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 109
#define YYACTIONTYPE unsigned short int
#define ParseTOKENTYPE void *
typedef union {
  int yyinit;
  ParseTOKENTYPE yy0;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 1000000
#endif
#define ParseARG_SDECL  gaiaGeomCollPtr *result ;
#define ParseARG_PDECL , gaiaGeomCollPtr *result 
#define ParseARG_FETCH  gaiaGeomCollPtr *result  = yypParser->result 
#define ParseARG_STORE yypParser->result  = result 
#define YYNSTATE 350
#define YYNRULE 151
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* The yyzerominor constant is used to initialize instances of
** YYMINORTYPE objects to zero. */
static const YYMINORTYPE yyzerominor = { 0 };

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
static const YYACTIONTYPE yy_action[] = {
 /*     0 */   159,  222,  223,  224,  225,  226,  227,  228,  229,  230,
 /*    10 */   231,  232,  233,  234,  235,  236,  237,  238,  239,  240,
 /*    20 */   241,  242,  243,  244,  245,  246,  247,  248,  249,  250,
 /*    30 */   251,  252,  350,  257,  160,  161,  162,  164,  163,   54,
 /*    40 */    12,   71,   13,   86,   15,   95,   16,  106,   18,  123,
 /*    50 */    20,  152,  128,  136,  144,  134,  142,  150,  135,  143,
 /*    60 */   151,  166,  168,    4,  170,   54,  175,  180,   55,  185,
 /*    70 */    54,   92,   93,  160,   94,   54,  207,  131,   14,   12,
 /*    80 */   132,   13,  129,  133,  130,  211,  139,   64,  215,  140,
 /*    90 */    62,  137,  141,  138,  145,  147,  146,   56,  148,  164,
 /*   100 */    74,  149,    5,   71,   78,   86,   82,  153,  157,  158,
 /*   110 */   154,  155,  156,  260,    6,  261,  262,  502,    1,  274,
 /*   120 */    60,  275,  276,  290,   59,  291,  292,  165,  298,   57,
 /*   130 */   299,  300,   97,   65,  100,  103,   62,  310,  256,  311,
 /*   140 */   312,  111,  167,  115,  119,  264,  322,   61,  323,  324,
 /*   150 */   172,   59,   57,   69,   66,   70,   66,   72,   73,   57,
 /*   160 */    57,  175,  176,  177,   76,   59,   59,   59,   59,  180,
 /*   170 */   181,   17,   62,   62,  182,   80,   19,   62,   62,  185,
 /*   180 */    66,  186,   66,  187,   66,   84,   66,  190,  191,   57,
 /*   190 */    57,  192,   90,   57,   57,   96,  166,   57,  168,    2,
 /*   200 */    59,   62,  170,   66,  161,   23,  162,   58,   59,   62,
 /*   210 */   163,   66,  259,  263,   63,  266,   25,   27,   67,  169,
 /*   220 */   267,   68,   28,  171,  269,   30,  173,  272,  271,   75,
 /*   230 */    31,  178,   79,   35,   39,   83,  183,  174,   87,   77,
 /*   240 */   188,   43,  279,   89,  195,   47,  193,  179,   81,  286,
 /*   250 */   194,  196,  282,  197,   98,  184,   85,   88,  285,  189,
 /*   260 */    91,   48,  289,   99,  101,   49,  102,  104,   50,  198,
 /*   270 */   296,  105,  107,  108,  109,  112,  110,  113,   74,  302,
 /*   280 */   114,  117,  116,  199,  304,  118,  200,  307,  306,  120,
 /*   290 */   201,  121,  309,   78,  122,  124,  126,    7,   82,   10,
 /*   300 */     8,  202,  125,  314,  260,    9,  127,  274,   11,  221,
 /*   310 */     3,  203,  316,  253,  261,  204,  254,  319,  255,  275,
 /*   320 */   318,   21,   22,  205,  258,  265,  321,  276,  262,   24,
 /*   330 */   268,   26,  270,  273,   29,  277,  206,   32,  326,   33,
 /*   340 */    34,  278,  280,   36,  281,  327,   37,  328,  208,  209,
 /*   350 */    38,  283,  210,   40,  284,   41,  332,  333,  334,  212,
 /*   360 */   213,   42,  214,  287,   44,   45,  338,   46,  288,  293,
 /*   370 */   339,  340,  343,  216,  294,  295,  297,  217,  218,  301,
 /*   380 */   303,  345,  346,  305,  347,  219,  220,  308,  313,  315,
 /*   390 */   317,  320,  325,   51,  503,  329,  330,  331,  335,   52,
 /*   400 */   503,  336,  337,   53,  503,  503,  341,  342,  344,  503,
 /*   410 */   503,  348,  349,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */    23,   24,   25,   26,   27,   28,   29,   30,   31,   32,
 /*    10 */    33,   34,   35,   36,   37,   38,   39,   40,   41,   42,
 /*    20 */    43,   44,   45,   46,   47,   48,   49,   50,   51,   52,
 /*    30 */    53,   54,    0,    6,    2,   55,   56,    5,   58,   59,
 /*    40 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*    50 */    18,   19,   27,   28,   29,   30,   31,   32,   33,   34,
 /*    60 */    35,   55,   56,    3,   58,   59,   55,   56,   59,   58,
 /*    70 */    59,   55,   56,    2,   58,   59,    2,   27,    3,    8,
 /*    80 */    30,   10,    8,   33,   10,    2,   28,   56,    2,   31,
 /*    90 */    59,    8,   34,   10,    8,   29,   10,   59,   32,    5,
 /*   100 */    72,   35,    3,    9,   76,   11,   78,   48,   49,   50,
 /*   110 */    48,   49,   50,   64,    3,   66,   67,   21,   22,   68,
 /*   120 */    55,   70,   71,   80,   59,   82,   83,   57,   84,   59,
 /*   130 */    86,   87,   64,   56,   66,   67,   59,   92,   59,   94,
 /*   140 */    95,   68,   60,   70,   71,   60,  100,   55,  102,  103,
 /*   150 */    57,   59,   59,   58,   59,   58,   59,   57,   57,   59,
 /*   160 */    59,   55,   55,   55,   55,   59,   59,   59,   59,   56,
 /*   170 */    56,    3,   59,   59,   56,   56,    3,   59,   59,   58,
 /*   180 */    59,   58,   59,   58,   59,   58,   59,   57,   57,   59,
 /*   190 */    59,   57,   57,   59,   59,   57,   55,   59,   56,    3,
 /*   200 */    59,   59,   58,   59,   55,    7,   56,   59,   59,   59,
 /*   210 */    58,   59,   59,   59,   59,   59,    7,    7,   59,   62,
 /*   220 */    62,   59,    3,   63,   63,    7,   61,   61,   65,    7,
 /*   230 */     3,   60,    7,    3,    3,    7,   62,   73,    3,   72,
 /*   240 */    63,    3,   73,    7,   62,    3,   61,   77,   76,   69,
 /*   250 */    60,   63,   77,   61,    7,   79,   78,   74,   79,   75,
 /*   260 */    74,    3,   75,   64,    7,    3,   66,    7,    3,   88,
 /*   270 */    81,   67,    3,   65,    7,    7,   65,    3,   72,   88,
 /*   280 */    68,    3,    7,   90,   90,   70,   91,   85,   91,    7,
 /*   290 */    89,    3,   89,   76,   71,    3,    7,    7,   78,    3,
 /*   300 */     7,   96,   69,   96,   64,    7,   69,   68,    7,    1,
 /*   310 */     3,   98,   98,    4,   66,   99,    4,   93,    4,   70,
 /*   320 */    99,    3,    7,   97,    4,    4,   97,   71,   67,    7,
 /*   330 */     4,    7,    4,    4,    7,    4,  104,    7,  104,    7,
 /*   340 */     7,    4,    4,    7,    4,  104,    7,  104,  104,  104,
 /*   350 */     7,    4,  106,    7,    4,    7,  106,  106,  106,  106,
 /*   360 */   106,    7,  107,    4,    7,    7,  107,    7,    4,    4,
 /*   370 */   107,  107,  101,  107,    4,    4,    4,  107,  105,    4,
 /*   380 */     4,  105,  105,    4,  105,  105,  105,    4,    4,    4,
 /*   390 */     4,    4,    4,    3,  108,    4,    4,    4,    4,    3,
 /*   400 */   108,    4,    4,    3,  108,  108,    4,    4,    4,  108,
 /*   410 */   108,    4,    4,
};
#define YY_SHIFT_USE_DFLT (-1)
#define YY_SHIFT_MAX 220
static const short yy_shift_ofst[] = {
 /*     0 */    -1,   32,   71,   27,   27,   27,   27,   74,   83,   86,
 /*    10 */    94,   94,   60,   75,   99,  111,  168,   60,  173,   75,
 /*    20 */   196,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    30 */    27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    40 */    27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    50 */    27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
 /*    60 */   198,  198,   27,   27,  209,  209,   27,   27,   27,  210,
 /*    70 */   210,  219,  218,  218,  222,  227,  198,  222,  225,  230,
 /*    80 */   209,  225,  228,  231,  210,  228,  235,  238,  236,  238,
 /*    90 */   218,  236,  198,  209,  210,  242,  218,  247,  258,  247,
 /*   100 */   257,  262,  257,  260,  265,  260,  269,  219,  267,  219,
 /*   110 */   267,  268,  274,  227,  268,  275,  278,  230,  275,  282,
 /*   120 */   288,  231,  282,  292,  235,  289,  235,  289,  290,  258,
 /*   130 */   274,  290,  290,  290,  290,  290,  293,  262,  278,  293,
 /*   140 */   293,  293,  293,  293,  298,  265,  288,  298,  298,  298,
 /*   150 */   298,  298,  296,  301,  301,  301,  301,  301,  301,  308,
 /*   160 */   307,  309,  312,  314,  318,  320,  315,  321,  322,  326,
 /*   170 */   324,  328,  327,  329,  331,  330,  332,  333,  337,  338,
 /*   180 */   336,  339,  343,  340,  347,  346,  348,  354,  350,  359,
 /*   190 */   357,  358,  360,  364,  365,  370,  371,  372,  375,  376,
 /*   200 */   379,  383,  384,  385,  386,  387,  388,  390,  391,  392,
 /*   210 */   393,  396,  394,  397,  398,  400,  402,  403,  404,  407,
 /*   220 */   408,
};
#define YY_REDUCE_USE_DFLT (-24)
#define YY_REDUCE_MAX 158
static const short yy_reduce_ofst[] = {
 /*     0 */    96,  -23,   25,  -20,    6,   11,   16,   50,   58,   66,
 /*    10 */    59,   62,   49,   51,   28,   43,   44,   68,   45,   73,
 /*    20 */    46,   70,   65,   92,   31,   77,   95,   97,   93,  100,
 /*    30 */   101,  106,  107,  108,  109,  113,  114,  118,  119,  121,
 /*    40 */   123,  125,  127,  130,  131,  134,  135,  138,  141,  142,
 /*    50 */   144,  149,  150,  152,    9,   38,   79,  148,  153,  154,
 /*    60 */    82,   85,  155,  156,  157,  158,  159,  162,   79,  160,
 /*    70 */   161,  163,  165,  166,  164,  167,  171,  169,  170,  172,
 /*    80 */   174,  175,  176,  178,  177,  179,  180,  183,  184,  186,
 /*    90 */   185,  187,  190,  182,  188,  189,  192,  181,  199,  191,
 /*   100 */   193,  200,  194,  195,  204,  197,  202,  208,  201,  211,
 /*   110 */   203,  205,  212,  206,  207,  213,  215,  217,  214,  216,
 /*   120 */   223,  220,  221,  224,  233,  226,  237,  229,  232,  240,
 /*   130 */   239,  234,  241,  243,  244,  245,  246,  248,  249,  250,
 /*   140 */   251,  252,  253,  254,  255,  261,  256,  259,  263,  264,
 /*   150 */   266,  270,  271,  273,  276,  277,  279,  280,  281,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */   351,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*    10 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*    20 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*    30 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*    40 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*    50 */   501,  501,  501,  501,  501,  388,  390,  501,  501,  501,
 /*    60 */   393,  393,  501,  501,  397,  397,  501,  501,  501,  399,
 /*    70 */   399,  501,  395,  395,  418,  501,  393,  418,  424,  501,
 /*    80 */   397,  424,  427,  501,  399,  427,  501,  501,  421,  501,
 /*    90 */   395,  421,  393,  397,  399,  501,  395,  442,  501,  442,
 /*   100 */   448,  501,  448,  451,  501,  451,  501,  501,  445,  501,
 /*   110 */   445,  458,  501,  501,  458,  464,  501,  501,  464,  467,
 /*   120 */   501,  501,  467,  501,  501,  461,  501,  461,  476,  501,
 /*   130 */   501,  476,  476,  476,  476,  476,  490,  501,  501,  490,
 /*   140 */   490,  490,  490,  490,  497,  501,  501,  497,  497,  497,
 /*   150 */   497,  497,  501,  483,  483,  483,  483,  483,  483,  501,
 /*   160 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*   170 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*   180 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*   190 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*   200 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*   210 */   501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
 /*   220 */   501,  352,  353,  354,  355,  356,  357,  358,  359,  360,
 /*   230 */   361,  362,  363,  364,  365,  366,  367,  368,  369,  370,
 /*   240 */   371,  372,  373,  374,  375,  376,  377,  378,  379,  380,
 /*   250 */   381,  382,  383,  384,  385,  387,  391,  392,  386,  389,
 /*   260 */   401,  403,  404,  388,  394,  405,  390,  398,  407,  400,
 /*   270 */   408,  402,  396,  406,  409,  411,  412,  413,  417,  419,
 /*   280 */   415,  423,  425,  416,  426,  428,  410,  414,  420,  422,
 /*   290 */   429,  431,  432,  433,  435,  436,  430,  434,  437,  439,
 /*   300 */   440,  441,  443,  447,  449,  450,  452,  438,  444,  446,
 /*   310 */   453,  455,  456,  457,  459,  463,  465,  466,  468,  454,
 /*   320 */   460,  462,  469,  471,  472,  473,  477,  478,  479,  474,
 /*   330 */   475,  487,  491,  492,  493,  488,  489,  494,  498,  499,
 /*   340 */   500,  495,  496,  470,  480,  484,  485,  486,  481,  482,
};
#define YY_SZ_ACTTAB (int)(sizeof(yy_action)/sizeof(yy_action[0]))

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  int yyidx;                    /* Index of top element in stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyidxMax;                 /* Maximum value of yyidx */
#endif
  int yyerrcnt;                 /* Shifts left before out of the error */
  ParseARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
/* #include <stdio.h> */
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void ParseTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "EWKT_NEWLINE",  "EWKT_POINT",    "EWKT_OPEN_BRACKET",
  "EWKT_CLOSE_BRACKET",  "EWKT_POINT_M",  "EWKT_NUM",      "EWKT_COMMA",  
  "EWKT_LINESTRING",  "EWKT_LINESTRING_M",  "EWKT_POLYGON",  "EWKT_POLYGON_M",
  "EWKT_MULTIPOINT",  "EWKT_MULTIPOINT_M",  "EWKT_MULTILINESTRING",  "EWKT_MULTILINESTRING_M",
  "EWKT_MULTIPOLYGON",  "EWKT_MULTIPOLYGON_M",  "EWKT_GEOMETRYCOLLECTION",  "EWKT_GEOMETRYCOLLECTION_M",
  "error",         "main",          "in",            "state",       
  "program",       "geo_text",      "geo_textm",     "point",       
  "pointz",        "pointzm",       "linestring",    "linestringz", 
  "linestringzm",  "polygon",       "polygonz",      "polygonzm",   
  "multipoint",    "multipointz",   "multipointzm",  "multilinestring",
  "multilinestringz",  "multilinestringzm",  "multipolygon",  "multipolygonz",
  "multipolygonzm",  "geocoll",       "geocollz",      "geocollzm",   
  "pointm",        "linestringm",   "polygonm",      "multipointm", 
  "multilinestringm",  "multipolygonm",  "geocollm",      "point_coordxy",
  "point_coordxyz",  "point_coordxym",  "point_coordxyzm",  "coord",       
  "extra_pointsxy",  "extra_pointsxym",  "extra_pointsxyz",  "extra_pointsxyzm",
  "linestring_text",  "linestring_textm",  "linestring_textz",  "linestring_textzm",
  "polygon_text",  "polygon_textm",  "polygon_textz",  "polygon_textzm",
  "ring",          "extra_rings",   "ringm",         "extra_ringsm",
  "ringz",         "extra_ringsz",  "ringzm",        "extra_ringszm",
  "multipoint_text",  "multipoint_textm",  "multipoint_textz",  "multipoint_textzm",
  "multilinestring_text",  "multilinestring_textm",  "multilinestring_textz",  "multilinestring_textzm",
  "multilinestring_text2",  "multilinestring_textm2",  "multilinestring_textz2",  "multilinestring_textzm2",
  "multipolygon_text",  "multipolygon_textm",  "multipolygon_textz",  "multipolygon_textzm",
  "multipolygon_text2",  "multipolygon_textm2",  "multipolygon_textz2",  "multipolygon_textzm2",
  "geocoll_text",  "geocoll_textm",  "geocoll_textz",  "geocoll_textzm",
  "geocoll_text2",  "geocoll_textm2",  "geocoll_textz2",  "geocoll_textzm2",
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "main ::= in",
 /*   1 */ "in ::=",
 /*   2 */ "in ::= in state EWKT_NEWLINE",
 /*   3 */ "state ::= program",
 /*   4 */ "program ::= geo_text",
 /*   5 */ "program ::= geo_textm",
 /*   6 */ "geo_text ::= point",
 /*   7 */ "geo_text ::= pointz",
 /*   8 */ "geo_text ::= pointzm",
 /*   9 */ "geo_text ::= linestring",
 /*  10 */ "geo_text ::= linestringz",
 /*  11 */ "geo_text ::= linestringzm",
 /*  12 */ "geo_text ::= polygon",
 /*  13 */ "geo_text ::= polygonz",
 /*  14 */ "geo_text ::= polygonzm",
 /*  15 */ "geo_text ::= multipoint",
 /*  16 */ "geo_text ::= multipointz",
 /*  17 */ "geo_text ::= multipointzm",
 /*  18 */ "geo_text ::= multilinestring",
 /*  19 */ "geo_text ::= multilinestringz",
 /*  20 */ "geo_text ::= multilinestringzm",
 /*  21 */ "geo_text ::= multipolygon",
 /*  22 */ "geo_text ::= multipolygonz",
 /*  23 */ "geo_text ::= multipolygonzm",
 /*  24 */ "geo_text ::= geocoll",
 /*  25 */ "geo_text ::= geocollz",
 /*  26 */ "geo_text ::= geocollzm",
 /*  27 */ "geo_textm ::= pointm",
 /*  28 */ "geo_textm ::= linestringm",
 /*  29 */ "geo_textm ::= polygonm",
 /*  30 */ "geo_textm ::= multipointm",
 /*  31 */ "geo_textm ::= multilinestringm",
 /*  32 */ "geo_textm ::= multipolygonm",
 /*  33 */ "geo_textm ::= geocollm",
 /*  34 */ "point ::= EWKT_POINT EWKT_OPEN_BRACKET point_coordxy EWKT_CLOSE_BRACKET",
 /*  35 */ "pointz ::= EWKT_POINT EWKT_OPEN_BRACKET point_coordxyz EWKT_CLOSE_BRACKET",
 /*  36 */ "pointm ::= EWKT_POINT_M EWKT_OPEN_BRACKET point_coordxym EWKT_CLOSE_BRACKET",
 /*  37 */ "pointzm ::= EWKT_POINT EWKT_OPEN_BRACKET point_coordxyzm EWKT_CLOSE_BRACKET",
 /*  38 */ "point_coordxy ::= coord coord",
 /*  39 */ "point_coordxym ::= coord coord coord",
 /*  40 */ "point_coordxyz ::= coord coord coord",
 /*  41 */ "point_coordxyzm ::= coord coord coord coord",
 /*  42 */ "coord ::= EWKT_NUM",
 /*  43 */ "extra_pointsxy ::=",
 /*  44 */ "extra_pointsxy ::= EWKT_COMMA point_coordxy extra_pointsxy",
 /*  45 */ "extra_pointsxym ::=",
 /*  46 */ "extra_pointsxym ::= EWKT_COMMA point_coordxym extra_pointsxym",
 /*  47 */ "extra_pointsxyz ::=",
 /*  48 */ "extra_pointsxyz ::= EWKT_COMMA point_coordxyz extra_pointsxyz",
 /*  49 */ "extra_pointsxyzm ::=",
 /*  50 */ "extra_pointsxyzm ::= EWKT_COMMA point_coordxyzm extra_pointsxyzm",
 /*  51 */ "linestring ::= EWKT_LINESTRING linestring_text",
 /*  52 */ "linestringm ::= EWKT_LINESTRING_M linestring_textm",
 /*  53 */ "linestringz ::= EWKT_LINESTRING linestring_textz",
 /*  54 */ "linestringzm ::= EWKT_LINESTRING linestring_textzm",
 /*  55 */ "linestring_text ::= EWKT_OPEN_BRACKET point_coordxy EWKT_COMMA point_coordxy extra_pointsxy EWKT_CLOSE_BRACKET",
 /*  56 */ "linestring_textm ::= EWKT_OPEN_BRACKET point_coordxym EWKT_COMMA point_coordxym extra_pointsxym EWKT_CLOSE_BRACKET",
 /*  57 */ "linestring_textz ::= EWKT_OPEN_BRACKET point_coordxyz EWKT_COMMA point_coordxyz extra_pointsxyz EWKT_CLOSE_BRACKET",
 /*  58 */ "linestring_textzm ::= EWKT_OPEN_BRACKET point_coordxyzm EWKT_COMMA point_coordxyzm extra_pointsxyzm EWKT_CLOSE_BRACKET",
 /*  59 */ "polygon ::= EWKT_POLYGON polygon_text",
 /*  60 */ "polygonm ::= EWKT_POLYGON_M polygon_textm",
 /*  61 */ "polygonz ::= EWKT_POLYGON polygon_textz",
 /*  62 */ "polygonzm ::= EWKT_POLYGON polygon_textzm",
 /*  63 */ "polygon_text ::= EWKT_OPEN_BRACKET ring extra_rings EWKT_CLOSE_BRACKET",
 /*  64 */ "polygon_textm ::= EWKT_OPEN_BRACKET ringm extra_ringsm EWKT_CLOSE_BRACKET",
 /*  65 */ "polygon_textz ::= EWKT_OPEN_BRACKET ringz extra_ringsz EWKT_CLOSE_BRACKET",
 /*  66 */ "polygon_textzm ::= EWKT_OPEN_BRACKET ringzm extra_ringszm EWKT_CLOSE_BRACKET",
 /*  67 */ "ring ::= EWKT_OPEN_BRACKET point_coordxy EWKT_COMMA point_coordxy EWKT_COMMA point_coordxy EWKT_COMMA point_coordxy extra_pointsxy EWKT_CLOSE_BRACKET",
 /*  68 */ "extra_rings ::=",
 /*  69 */ "extra_rings ::= EWKT_COMMA ring extra_rings",
 /*  70 */ "ringm ::= EWKT_OPEN_BRACKET point_coordxym EWKT_COMMA point_coordxym EWKT_COMMA point_coordxym EWKT_COMMA point_coordxym extra_pointsxym EWKT_CLOSE_BRACKET",
 /*  71 */ "extra_ringsm ::=",
 /*  72 */ "extra_ringsm ::= EWKT_COMMA ringm extra_ringsm",
 /*  73 */ "ringz ::= EWKT_OPEN_BRACKET point_coordxyz EWKT_COMMA point_coordxyz EWKT_COMMA point_coordxyz EWKT_COMMA point_coordxyz extra_pointsxyz EWKT_CLOSE_BRACKET",
 /*  74 */ "extra_ringsz ::=",
 /*  75 */ "extra_ringsz ::= EWKT_COMMA ringz extra_ringsz",
 /*  76 */ "ringzm ::= EWKT_OPEN_BRACKET point_coordxyzm EWKT_COMMA point_coordxyzm EWKT_COMMA point_coordxyzm EWKT_COMMA point_coordxyzm extra_pointsxyzm EWKT_CLOSE_BRACKET",
 /*  77 */ "extra_ringszm ::=",
 /*  78 */ "extra_ringszm ::= EWKT_COMMA ringzm extra_ringszm",
 /*  79 */ "multipoint ::= EWKT_MULTIPOINT multipoint_text",
 /*  80 */ "multipointm ::= EWKT_MULTIPOINT_M multipoint_textm",
 /*  81 */ "multipointz ::= EWKT_MULTIPOINT multipoint_textz",
 /*  82 */ "multipointzm ::= EWKT_MULTIPOINT multipoint_textzm",
 /*  83 */ "multipoint_text ::= EWKT_OPEN_BRACKET point_coordxy extra_pointsxy EWKT_CLOSE_BRACKET",
 /*  84 */ "multipoint_textm ::= EWKT_OPEN_BRACKET point_coordxym extra_pointsxym EWKT_CLOSE_BRACKET",
 /*  85 */ "multipoint_textz ::= EWKT_OPEN_BRACKET point_coordxyz extra_pointsxyz EWKT_CLOSE_BRACKET",
 /*  86 */ "multipoint_textzm ::= EWKT_OPEN_BRACKET point_coordxyzm extra_pointsxyzm EWKT_CLOSE_BRACKET",
 /*  87 */ "multilinestring ::= EWKT_MULTILINESTRING multilinestring_text",
 /*  88 */ "multilinestringm ::= EWKT_MULTILINESTRING_M multilinestring_textm",
 /*  89 */ "multilinestringz ::= EWKT_MULTILINESTRING multilinestring_textz",
 /*  90 */ "multilinestringzm ::= EWKT_MULTILINESTRING multilinestring_textzm",
 /*  91 */ "multilinestring_text ::= EWKT_OPEN_BRACKET linestring_text multilinestring_text2 EWKT_CLOSE_BRACKET",
 /*  92 */ "multilinestring_text2 ::=",
 /*  93 */ "multilinestring_text2 ::= EWKT_COMMA linestring_text multilinestring_text2",
 /*  94 */ "multilinestring_textm ::= EWKT_OPEN_BRACKET linestring_textm multilinestring_textm2 EWKT_CLOSE_BRACKET",
 /*  95 */ "multilinestring_textm2 ::=",
 /*  96 */ "multilinestring_textm2 ::= EWKT_COMMA linestring_textm multilinestring_textm2",
 /*  97 */ "multilinestring_textz ::= EWKT_OPEN_BRACKET linestring_textz multilinestring_textz2 EWKT_CLOSE_BRACKET",
 /*  98 */ "multilinestring_textz2 ::=",
 /*  99 */ "multilinestring_textz2 ::= EWKT_COMMA linestring_textz multilinestring_textz2",
 /* 100 */ "multilinestring_textzm ::= EWKT_OPEN_BRACKET linestring_textzm multilinestring_textzm2 EWKT_CLOSE_BRACKET",
 /* 101 */ "multilinestring_textzm2 ::=",
 /* 102 */ "multilinestring_textzm2 ::= EWKT_COMMA linestring_textzm multilinestring_textzm2",
 /* 103 */ "multipolygon ::= EWKT_MULTIPOLYGON multipolygon_text",
 /* 104 */ "multipolygonm ::= EWKT_MULTIPOLYGON_M multipolygon_textm",
 /* 105 */ "multipolygonz ::= EWKT_MULTIPOLYGON multipolygon_textz",
 /* 106 */ "multipolygonzm ::= EWKT_MULTIPOLYGON multipolygon_textzm",
 /* 107 */ "multipolygon_text ::= EWKT_OPEN_BRACKET polygon_text multipolygon_text2 EWKT_CLOSE_BRACKET",
 /* 108 */ "multipolygon_text2 ::=",
 /* 109 */ "multipolygon_text2 ::= EWKT_COMMA polygon_text multipolygon_text2",
 /* 110 */ "multipolygon_textm ::= EWKT_OPEN_BRACKET polygon_textm multipolygon_textm2 EWKT_CLOSE_BRACKET",
 /* 111 */ "multipolygon_textm2 ::=",
 /* 112 */ "multipolygon_textm2 ::= EWKT_COMMA polygon_textm multipolygon_textm2",
 /* 113 */ "multipolygon_textz ::= EWKT_OPEN_BRACKET polygon_textz multipolygon_textz2 EWKT_CLOSE_BRACKET",
 /* 114 */ "multipolygon_textz2 ::=",
 /* 115 */ "multipolygon_textz2 ::= EWKT_COMMA polygon_textz multipolygon_textz2",
 /* 116 */ "multipolygon_textzm ::= EWKT_OPEN_BRACKET polygon_textzm multipolygon_textzm2 EWKT_CLOSE_BRACKET",
 /* 117 */ "multipolygon_textzm2 ::=",
 /* 118 */ "multipolygon_textzm2 ::= EWKT_COMMA polygon_textzm multipolygon_textzm2",
 /* 119 */ "geocoll ::= EWKT_GEOMETRYCOLLECTION geocoll_text",
 /* 120 */ "geocollm ::= EWKT_GEOMETRYCOLLECTION_M geocoll_textm",
 /* 121 */ "geocollz ::= EWKT_GEOMETRYCOLLECTION geocoll_textz",
 /* 122 */ "geocollzm ::= EWKT_GEOMETRYCOLLECTION geocoll_textzm",
 /* 123 */ "geocoll_text ::= EWKT_OPEN_BRACKET point geocoll_text2 EWKT_CLOSE_BRACKET",
 /* 124 */ "geocoll_text ::= EWKT_OPEN_BRACKET linestring geocoll_text2 EWKT_CLOSE_BRACKET",
 /* 125 */ "geocoll_text ::= EWKT_OPEN_BRACKET polygon geocoll_text2 EWKT_CLOSE_BRACKET",
 /* 126 */ "geocoll_text2 ::=",
 /* 127 */ "geocoll_text2 ::= EWKT_COMMA point geocoll_text2",
 /* 128 */ "geocoll_text2 ::= EWKT_COMMA linestring geocoll_text2",
 /* 129 */ "geocoll_text2 ::= EWKT_COMMA polygon geocoll_text2",
 /* 130 */ "geocoll_textm ::= EWKT_OPEN_BRACKET pointm geocoll_textm2 EWKT_CLOSE_BRACKET",
 /* 131 */ "geocoll_textm ::= EWKT_OPEN_BRACKET linestringm geocoll_textm2 EWKT_CLOSE_BRACKET",
 /* 132 */ "geocoll_textm ::= EWKT_OPEN_BRACKET polygonm geocoll_textm2 EWKT_CLOSE_BRACKET",
 /* 133 */ "geocoll_textm2 ::=",
 /* 134 */ "geocoll_textm2 ::= EWKT_COMMA pointm geocoll_textm2",
 /* 135 */ "geocoll_textm2 ::= EWKT_COMMA linestringm geocoll_textm2",
 /* 136 */ "geocoll_textm2 ::= EWKT_COMMA polygonm geocoll_textm2",
 /* 137 */ "geocoll_textz ::= EWKT_OPEN_BRACKET pointz geocoll_textz2 EWKT_CLOSE_BRACKET",
 /* 138 */ "geocoll_textz ::= EWKT_OPEN_BRACKET linestringz geocoll_textz2 EWKT_CLOSE_BRACKET",
 /* 139 */ "geocoll_textz ::= EWKT_OPEN_BRACKET polygonz geocoll_textz2 EWKT_CLOSE_BRACKET",
 /* 140 */ "geocoll_textz2 ::=",
 /* 141 */ "geocoll_textz2 ::= EWKT_COMMA pointz geocoll_textz2",
 /* 142 */ "geocoll_textz2 ::= EWKT_COMMA linestringz geocoll_textz2",
 /* 143 */ "geocoll_textz2 ::= EWKT_COMMA polygonz geocoll_textz2",
 /* 144 */ "geocoll_textzm ::= EWKT_OPEN_BRACKET pointzm geocoll_textzm2 EWKT_CLOSE_BRACKET",
 /* 145 */ "geocoll_textzm ::= EWKT_OPEN_BRACKET linestringzm geocoll_textzm2 EWKT_CLOSE_BRACKET",
 /* 146 */ "geocoll_textzm ::= EWKT_OPEN_BRACKET polygonzm geocoll_textzm2 EWKT_CLOSE_BRACKET",
 /* 147 */ "geocoll_textzm2 ::=",
 /* 148 */ "geocoll_textzm2 ::= EWKT_COMMA pointzm geocoll_textzm2",
 /* 149 */ "geocoll_textzm2 ::= EWKT_COMMA linestringzm geocoll_textzm2",
 /* 150 */ "geocoll_textzm2 ::= EWKT_COMMA polygonzm geocoll_textzm2",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.
*/
static void yyGrowStack(yyParser *p){
  int newSize;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  if( pNew ){
    p->yystack = pNew;
    p->yystksz = newSize;
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows to %d entries!\n",
              yyTracePrompt, p->yystksz);
    }
#endif
  }
}
#endif

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Parse and ParseFree.
*/
void *ParseAlloc(void *(*mallocProc)(size_t)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (size_t)sizeof(yyParser) );
  if( pParser ){
    pParser->yyidx = -1;
#ifdef YYTRACKMAXSTACKDEPTH
    pParser->yyidxMax = 0;
#endif
#if YYSTACKDEPTH<=0
    pParser->yystack = NULL;
    pParser->yystksz = 0;
    yyGrowStack(pParser);
#endif
  }
  return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  ParseARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are not used
    ** inside the C code.
    */
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(yyParser *pParser){
  YYCODETYPE yymajor;
  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

  if( pParser->yyidx<0 ) return 0;
#ifndef NDEBUG
  if( yyTraceFILE && pParser->yyidx>=0 ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yymajor = yytos->major;
  yy_destructor(pParser, yymajor, &yytos->minor);
  pParser->yyidx--;
  return yymajor;
}

/* 
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from ParseAlloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void ParseFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
  if( pParser==0 ) return;
  while( pParser->yyidx>=0 ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  free(pParser->yystack);
#endif
  (*freeProc)((void*)pParser);
}

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int ParseStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyidxMax;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yystack[pParser->yyidx].stateno;
 
  if( stateno>YY_SHIFT_MAX || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
    return yy_default[stateno];
  }
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    if( iLookAhead>0 ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        return yy_find_shift_action(pParser, iFallback);
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( j>=0 && j<YY_SZ_ACTTAB && yy_lookahead[j]==YYWILDCARD ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
    }
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_MAX ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_MAX );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_SZ_ACTTAB );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser, YYMINORTYPE *yypMinor){
   ParseARG_FETCH;
   yypParser->yyidx--;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */

     fprintf(stderr,"Giving up.  Parser stack overflow\n");
   ParseARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor         /* Pointer to the minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yyidx++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( yypParser->yyidx>yypParser->yyidxMax ){
    yypParser->yyidxMax = yypParser->yyidx;
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yyidx>=YYSTACKDEPTH ){
    yyStackOverflow(yypParser, yypMinor);
    return;
  }
#else
  if( yypParser->yyidx>=yypParser->yystksz ){
    yyGrowStack(yypParser);
    if( yypParser->yyidx>=yypParser->yystksz ){
      yyStackOverflow(yypParser, yypMinor);
      return;
    }
  }
#endif
  yytos = &yypParser->yystack[yypParser->yyidx];
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor = *yypMinor;
#ifndef NDEBUG
  if( yyTraceFILE && yypParser->yyidx>0 ){
    int i;
    fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
    fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
    for(i=1; i<=yypParser->yyidx; i++)
      fprintf(yyTraceFILE," %s",yyTokenName[yypParser->yystack[i].major]);
    fprintf(yyTraceFILE,"\n");
  }
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 21, 1 },
  { 22, 0 },
  { 22, 3 },
  { 23, 1 },
  { 24, 1 },
  { 24, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 25, 1 },
  { 26, 1 },
  { 26, 1 },
  { 26, 1 },
  { 26, 1 },
  { 26, 1 },
  { 26, 1 },
  { 26, 1 },
  { 27, 4 },
  { 28, 4 },
  { 48, 4 },
  { 29, 4 },
  { 55, 2 },
  { 57, 3 },
  { 56, 3 },
  { 58, 4 },
  { 59, 1 },
  { 60, 0 },
  { 60, 3 },
  { 61, 0 },
  { 61, 3 },
  { 62, 0 },
  { 62, 3 },
  { 63, 0 },
  { 63, 3 },
  { 30, 2 },
  { 49, 2 },
  { 31, 2 },
  { 32, 2 },
  { 64, 6 },
  { 65, 6 },
  { 66, 6 },
  { 67, 6 },
  { 33, 2 },
  { 50, 2 },
  { 34, 2 },
  { 35, 2 },
  { 68, 4 },
  { 69, 4 },
  { 70, 4 },
  { 71, 4 },
  { 72, 10 },
  { 73, 0 },
  { 73, 3 },
  { 74, 10 },
  { 75, 0 },
  { 75, 3 },
  { 76, 10 },
  { 77, 0 },
  { 77, 3 },
  { 78, 10 },
  { 79, 0 },
  { 79, 3 },
  { 36, 2 },
  { 51, 2 },
  { 37, 2 },
  { 38, 2 },
  { 80, 4 },
  { 81, 4 },
  { 82, 4 },
  { 83, 4 },
  { 39, 2 },
  { 52, 2 },
  { 40, 2 },
  { 41, 2 },
  { 84, 4 },
  { 88, 0 },
  { 88, 3 },
  { 85, 4 },
  { 89, 0 },
  { 89, 3 },
  { 86, 4 },
  { 90, 0 },
  { 90, 3 },
  { 87, 4 },
  { 91, 0 },
  { 91, 3 },
  { 42, 2 },
  { 53, 2 },
  { 43, 2 },
  { 44, 2 },
  { 92, 4 },
  { 96, 0 },
  { 96, 3 },
  { 93, 4 },
  { 97, 0 },
  { 97, 3 },
  { 94, 4 },
  { 98, 0 },
  { 98, 3 },
  { 95, 4 },
  { 99, 0 },
  { 99, 3 },
  { 45, 2 },
  { 54, 2 },
  { 46, 2 },
  { 47, 2 },
  { 100, 4 },
  { 100, 4 },
  { 100, 4 },
  { 104, 0 },
  { 104, 3 },
  { 104, 3 },
  { 104, 3 },
  { 101, 4 },
  { 101, 4 },
  { 101, 4 },
  { 105, 0 },
  { 105, 3 },
  { 105, 3 },
  { 105, 3 },
  { 102, 4 },
  { 102, 4 },
  { 102, 4 },
  { 106, 0 },
  { 106, 3 },
  { 106, 3 },
  { 106, 3 },
  { 103, 4 },
  { 103, 4 },
  { 103, 4 },
  { 107, 0 },
  { 107, 3 },
  { 107, 3 },
  { 107, 3 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  int yyruleno                 /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  ParseARG_FETCH;
  yymsp = &yypParser->yystack[yypParser->yyidx];
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno>=0 
        && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
      yyRuleName[yyruleno]);
  }
#endif /* NDEBUG */

  /* Silence complaints from purify about yygotominor being uninitialized
  ** in some cases when it is copied into the stack after the following
  ** switch.  yygotominor is uninitialized when a rule reduces that does
  ** not set the value of its left-hand side nonterminal.  Leaving the
  ** value of the nonterminal uninitialized is utterly harmless as long
  ** as the value is never used.  So really the only thing this code
  ** accomplishes is to quieten purify.  
  **
  ** 2007-01-16:  The wireshark project (www.wireshark.org) reports that
  ** without this code, their parser segfaults.  I'm not sure what there
  ** parser is doing to make this happen.  This is the second bug report
  ** from wireshark this week.  Clearly they are stressing Lemon in ways
  ** that it has not been previously stressed...  (SQLite ticket #2172)
  */
  /*memset(&yygotominor, 0, sizeof(yygotominor));*/
  yygotominor = yyzerominor;


  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
      case 6: /* geo_text ::= point */
      case 7: /* geo_text ::= pointz */ yytestcase(yyruleno==7);
      case 8: /* geo_text ::= pointzm */ yytestcase(yyruleno==8);
      case 9: /* geo_text ::= linestring */ yytestcase(yyruleno==9);
      case 10: /* geo_text ::= linestringz */ yytestcase(yyruleno==10);
      case 11: /* geo_text ::= linestringzm */ yytestcase(yyruleno==11);
      case 12: /* geo_text ::= polygon */ yytestcase(yyruleno==12);
      case 13: /* geo_text ::= polygonz */ yytestcase(yyruleno==13);
      case 14: /* geo_text ::= polygonzm */ yytestcase(yyruleno==14);
      case 15: /* geo_text ::= multipoint */ yytestcase(yyruleno==15);
      case 16: /* geo_text ::= multipointz */ yytestcase(yyruleno==16);
      case 17: /* geo_text ::= multipointzm */ yytestcase(yyruleno==17);
      case 18: /* geo_text ::= multilinestring */ yytestcase(yyruleno==18);
      case 19: /* geo_text ::= multilinestringz */ yytestcase(yyruleno==19);
      case 20: /* geo_text ::= multilinestringzm */ yytestcase(yyruleno==20);
      case 21: /* geo_text ::= multipolygon */ yytestcase(yyruleno==21);
      case 22: /* geo_text ::= multipolygonz */ yytestcase(yyruleno==22);
      case 23: /* geo_text ::= multipolygonzm */ yytestcase(yyruleno==23);
      case 24: /* geo_text ::= geocoll */ yytestcase(yyruleno==24);
      case 25: /* geo_text ::= geocollz */ yytestcase(yyruleno==25);
      case 26: /* geo_text ::= geocollzm */ yytestcase(yyruleno==26);
      case 27: /* geo_textm ::= pointm */ yytestcase(yyruleno==27);
      case 28: /* geo_textm ::= linestringm */ yytestcase(yyruleno==28);
      case 29: /* geo_textm ::= polygonm */ yytestcase(yyruleno==29);
      case 30: /* geo_textm ::= multipointm */ yytestcase(yyruleno==30);
      case 31: /* geo_textm ::= multilinestringm */ yytestcase(yyruleno==31);
      case 32: /* geo_textm ::= multipolygonm */ yytestcase(yyruleno==32);
      case 33: /* geo_textm ::= geocollm */ yytestcase(yyruleno==33);
{ *result = yymsp[0].minor.yy0; }
        break;
      case 34: /* point ::= EWKT_POINT EWKT_OPEN_BRACKET point_coordxy EWKT_CLOSE_BRACKET */
      case 35: /* pointz ::= EWKT_POINT EWKT_OPEN_BRACKET point_coordxyz EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==35);
      case 37: /* pointzm ::= EWKT_POINT EWKT_OPEN_BRACKET point_coordxyzm EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==37);
{ yygotominor.yy0 = ewkt_buildGeomFromPoint((gaiaPointPtr)yymsp[-1].minor.yy0); }
        break;
      case 36: /* pointm ::= EWKT_POINT_M EWKT_OPEN_BRACKET point_coordxym EWKT_CLOSE_BRACKET */
{ yygotominor.yy0 = ewkt_buildGeomFromPoint((gaiaPointPtr)yymsp[-1].minor.yy0);  }
        break;
      case 38: /* point_coordxy ::= coord coord */
{ yygotominor.yy0 = (void *) ewkt_point_xy((double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
        break;
      case 39: /* point_coordxym ::= coord coord coord */
{ yygotominor.yy0 = (void *) ewkt_point_xym((double *)yymsp[-2].minor.yy0, (double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
        break;
      case 40: /* point_coordxyz ::= coord coord coord */
{ yygotominor.yy0 = (void *) ewkt_point_xyz((double *)yymsp[-2].minor.yy0, (double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
        break;
      case 41: /* point_coordxyzm ::= coord coord coord coord */
{ yygotominor.yy0 = (void *) ewkt_point_xyzm((double *)yymsp[-3].minor.yy0, (double *)yymsp[-2].minor.yy0, (double *)yymsp[-1].minor.yy0, (double *)yymsp[0].minor.yy0); }
        break;
      case 42: /* coord ::= EWKT_NUM */
      case 79: /* multipoint ::= EWKT_MULTIPOINT multipoint_text */ yytestcase(yyruleno==79);
      case 80: /* multipointm ::= EWKT_MULTIPOINT_M multipoint_textm */ yytestcase(yyruleno==80);
      case 81: /* multipointz ::= EWKT_MULTIPOINT multipoint_textz */ yytestcase(yyruleno==81);
      case 82: /* multipointzm ::= EWKT_MULTIPOINT multipoint_textzm */ yytestcase(yyruleno==82);
      case 87: /* multilinestring ::= EWKT_MULTILINESTRING multilinestring_text */ yytestcase(yyruleno==87);
      case 88: /* multilinestringm ::= EWKT_MULTILINESTRING_M multilinestring_textm */ yytestcase(yyruleno==88);
      case 89: /* multilinestringz ::= EWKT_MULTILINESTRING multilinestring_textz */ yytestcase(yyruleno==89);
      case 90: /* multilinestringzm ::= EWKT_MULTILINESTRING multilinestring_textzm */ yytestcase(yyruleno==90);
      case 103: /* multipolygon ::= EWKT_MULTIPOLYGON multipolygon_text */ yytestcase(yyruleno==103);
      case 104: /* multipolygonm ::= EWKT_MULTIPOLYGON_M multipolygon_textm */ yytestcase(yyruleno==104);
      case 105: /* multipolygonz ::= EWKT_MULTIPOLYGON multipolygon_textz */ yytestcase(yyruleno==105);
      case 106: /* multipolygonzm ::= EWKT_MULTIPOLYGON multipolygon_textzm */ yytestcase(yyruleno==106);
      case 119: /* geocoll ::= EWKT_GEOMETRYCOLLECTION geocoll_text */ yytestcase(yyruleno==119);
      case 120: /* geocollm ::= EWKT_GEOMETRYCOLLECTION_M geocoll_textm */ yytestcase(yyruleno==120);
      case 121: /* geocollz ::= EWKT_GEOMETRYCOLLECTION geocoll_textz */ yytestcase(yyruleno==121);
      case 122: /* geocollzm ::= EWKT_GEOMETRYCOLLECTION geocoll_textzm */ yytestcase(yyruleno==122);
{ yygotominor.yy0 = yymsp[0].minor.yy0; }
        break;
      case 43: /* extra_pointsxy ::= */
      case 45: /* extra_pointsxym ::= */ yytestcase(yyruleno==45);
      case 47: /* extra_pointsxyz ::= */ yytestcase(yyruleno==47);
      case 49: /* extra_pointsxyzm ::= */ yytestcase(yyruleno==49);
      case 68: /* extra_rings ::= */ yytestcase(yyruleno==68);
      case 71: /* extra_ringsm ::= */ yytestcase(yyruleno==71);
      case 74: /* extra_ringsz ::= */ yytestcase(yyruleno==74);
      case 77: /* extra_ringszm ::= */ yytestcase(yyruleno==77);
      case 92: /* multilinestring_text2 ::= */ yytestcase(yyruleno==92);
      case 95: /* multilinestring_textm2 ::= */ yytestcase(yyruleno==95);
      case 98: /* multilinestring_textz2 ::= */ yytestcase(yyruleno==98);
      case 101: /* multilinestring_textzm2 ::= */ yytestcase(yyruleno==101);
      case 108: /* multipolygon_text2 ::= */ yytestcase(yyruleno==108);
      case 111: /* multipolygon_textm2 ::= */ yytestcase(yyruleno==111);
      case 114: /* multipolygon_textz2 ::= */ yytestcase(yyruleno==114);
      case 117: /* multipolygon_textzm2 ::= */ yytestcase(yyruleno==117);
      case 126: /* geocoll_text2 ::= */ yytestcase(yyruleno==126);
      case 133: /* geocoll_textm2 ::= */ yytestcase(yyruleno==133);
      case 140: /* geocoll_textz2 ::= */ yytestcase(yyruleno==140);
      case 147: /* geocoll_textzm2 ::= */ yytestcase(yyruleno==147);
{ yygotominor.yy0 = NULL; }
        break;
      case 44: /* extra_pointsxy ::= EWKT_COMMA point_coordxy extra_pointsxy */
      case 46: /* extra_pointsxym ::= EWKT_COMMA point_coordxym extra_pointsxym */ yytestcase(yyruleno==46);
      case 48: /* extra_pointsxyz ::= EWKT_COMMA point_coordxyz extra_pointsxyz */ yytestcase(yyruleno==48);
      case 50: /* extra_pointsxyzm ::= EWKT_COMMA point_coordxyzm extra_pointsxyzm */ yytestcase(yyruleno==50);
{ ((gaiaPointPtr)yymsp[-1].minor.yy0)->Next = (gaiaPointPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 51: /* linestring ::= EWKT_LINESTRING linestring_text */
      case 52: /* linestringm ::= EWKT_LINESTRING_M linestring_textm */ yytestcase(yyruleno==52);
      case 53: /* linestringz ::= EWKT_LINESTRING linestring_textz */ yytestcase(yyruleno==53);
      case 54: /* linestringzm ::= EWKT_LINESTRING linestring_textzm */ yytestcase(yyruleno==54);
{ yygotominor.yy0 = ewkt_buildGeomFromLinestring((gaiaLinestringPtr)yymsp[0].minor.yy0); }
        break;
      case 55: /* linestring_text ::= EWKT_OPEN_BRACKET point_coordxy EWKT_COMMA point_coordxy extra_pointsxy EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) ewkt_linestring_xy((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
        break;
      case 56: /* linestring_textm ::= EWKT_OPEN_BRACKET point_coordxym EWKT_COMMA point_coordxym extra_pointsxym EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) ewkt_linestring_xym((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
        break;
      case 57: /* linestring_textz ::= EWKT_OPEN_BRACKET point_coordxyz EWKT_COMMA point_coordxyz extra_pointsxyz EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) ewkt_linestring_xyz((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
        break;
      case 58: /* linestring_textzm ::= EWKT_OPEN_BRACKET point_coordxyzm EWKT_COMMA point_coordxyzm extra_pointsxyzm EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) ewkt_linestring_xyzm((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
        break;
      case 59: /* polygon ::= EWKT_POLYGON polygon_text */
      case 60: /* polygonm ::= EWKT_POLYGON_M polygon_textm */ yytestcase(yyruleno==60);
      case 61: /* polygonz ::= EWKT_POLYGON polygon_textz */ yytestcase(yyruleno==61);
      case 62: /* polygonzm ::= EWKT_POLYGON polygon_textzm */ yytestcase(yyruleno==62);
{ yygotominor.yy0 = ewkt_buildGeomFromPolygon((gaiaPolygonPtr)yymsp[0].minor.yy0); }
        break;
      case 63: /* polygon_text ::= EWKT_OPEN_BRACKET ring extra_rings EWKT_CLOSE_BRACKET */
{ 
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_polygon_xy((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 64: /* polygon_textm ::= EWKT_OPEN_BRACKET ringm extra_ringsm EWKT_CLOSE_BRACKET */
{ 
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_polygon_xym((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 65: /* polygon_textz ::= EWKT_OPEN_BRACKET ringz extra_ringsz EWKT_CLOSE_BRACKET */
{  
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_polygon_xyz((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 66: /* polygon_textzm ::= EWKT_OPEN_BRACKET ringzm extra_ringszm EWKT_CLOSE_BRACKET */
{ 
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_polygon_xyzm((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 67: /* ring ::= EWKT_OPEN_BRACKET point_coordxy EWKT_COMMA point_coordxy EWKT_COMMA point_coordxy EWKT_COMMA point_coordxy extra_pointsxy EWKT_CLOSE_BRACKET */
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_ring_xy((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
        break;
      case 69: /* extra_rings ::= EWKT_COMMA ring extra_rings */
      case 72: /* extra_ringsm ::= EWKT_COMMA ringm extra_ringsm */ yytestcase(yyruleno==72);
      case 75: /* extra_ringsz ::= EWKT_COMMA ringz extra_ringsz */ yytestcase(yyruleno==75);
      case 78: /* extra_ringszm ::= EWKT_COMMA ringzm extra_ringszm */ yytestcase(yyruleno==78);
{
		((gaiaRingPtr)yymsp[-1].minor.yy0)->Next = (gaiaRingPtr)yymsp[0].minor.yy0;
		yygotominor.yy0 = yymsp[-1].minor.yy0;
	}
        break;
      case 70: /* ringm ::= EWKT_OPEN_BRACKET point_coordxym EWKT_COMMA point_coordxym EWKT_COMMA point_coordxym EWKT_COMMA point_coordxym extra_pointsxym EWKT_CLOSE_BRACKET */
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_ring_xym((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
        break;
      case 73: /* ringz ::= EWKT_OPEN_BRACKET point_coordxyz EWKT_COMMA point_coordxyz EWKT_COMMA point_coordxyz EWKT_COMMA point_coordxyz extra_pointsxyz EWKT_CLOSE_BRACKET */
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_ring_xyz((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
        break;
      case 76: /* ringzm ::= EWKT_OPEN_BRACKET point_coordxyzm EWKT_COMMA point_coordxyzm EWKT_COMMA point_coordxyzm EWKT_COMMA point_coordxyzm extra_pointsxyzm EWKT_CLOSE_BRACKET */
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_ring_xyzm((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
        break;
      case 83: /* multipoint_text ::= EWKT_OPEN_BRACKET point_coordxy extra_pointsxy EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipoint_xy((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 84: /* multipoint_textm ::= EWKT_OPEN_BRACKET point_coordxym extra_pointsxym EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipoint_xym((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 85: /* multipoint_textz ::= EWKT_OPEN_BRACKET point_coordxyz extra_pointsxyz EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipoint_xyz((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 86: /* multipoint_textzm ::= EWKT_OPEN_BRACKET point_coordxyzm extra_pointsxyzm EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipoint_xyzm((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 91: /* multilinestring_text ::= EWKT_OPEN_BRACKET linestring_text multilinestring_text2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multilinestring_xy((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 93: /* multilinestring_text2 ::= EWKT_COMMA linestring_text multilinestring_text2 */
      case 96: /* multilinestring_textm2 ::= EWKT_COMMA linestring_textm multilinestring_textm2 */ yytestcase(yyruleno==96);
      case 99: /* multilinestring_textz2 ::= EWKT_COMMA linestring_textz multilinestring_textz2 */ yytestcase(yyruleno==99);
      case 102: /* multilinestring_textzm2 ::= EWKT_COMMA linestring_textzm multilinestring_textzm2 */ yytestcase(yyruleno==102);
{ ((gaiaLinestringPtr)yymsp[-1].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 94: /* multilinestring_textm ::= EWKT_OPEN_BRACKET linestring_textm multilinestring_textm2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multilinestring_xym((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 97: /* multilinestring_textz ::= EWKT_OPEN_BRACKET linestring_textz multilinestring_textz2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multilinestring_xyz((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 100: /* multilinestring_textzm ::= EWKT_OPEN_BRACKET linestring_textzm multilinestring_textzm2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multilinestring_xyzm((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 107: /* multipolygon_text ::= EWKT_OPEN_BRACKET polygon_text multipolygon_text2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipolygon_xy((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 109: /* multipolygon_text2 ::= EWKT_COMMA polygon_text multipolygon_text2 */
      case 112: /* multipolygon_textm2 ::= EWKT_COMMA polygon_textm multipolygon_textm2 */ yytestcase(yyruleno==112);
      case 115: /* multipolygon_textz2 ::= EWKT_COMMA polygon_textz multipolygon_textz2 */ yytestcase(yyruleno==115);
      case 118: /* multipolygon_textzm2 ::= EWKT_COMMA polygon_textzm multipolygon_textzm2 */ yytestcase(yyruleno==118);
{ ((gaiaPolygonPtr)yymsp[-1].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 110: /* multipolygon_textm ::= EWKT_OPEN_BRACKET polygon_textm multipolygon_textm2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipolygon_xym((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 113: /* multipolygon_textz ::= EWKT_OPEN_BRACKET polygon_textz multipolygon_textz2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipolygon_xyz((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 116: /* multipolygon_textzm ::= EWKT_OPEN_BRACKET polygon_textzm multipolygon_textzm2 EWKT_CLOSE_BRACKET */
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) ewkt_multipolygon_xyzm((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 123: /* geocoll_text ::= EWKT_OPEN_BRACKET point geocoll_text2 EWKT_CLOSE_BRACKET */
      case 124: /* geocoll_text ::= EWKT_OPEN_BRACKET linestring geocoll_text2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==124);
      case 125: /* geocoll_text ::= EWKT_OPEN_BRACKET polygon geocoll_text2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==125);
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_geomColl_xy((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 127: /* geocoll_text2 ::= EWKT_COMMA point geocoll_text2 */
      case 128: /* geocoll_text2 ::= EWKT_COMMA linestring geocoll_text2 */ yytestcase(yyruleno==128);
      case 129: /* geocoll_text2 ::= EWKT_COMMA polygon geocoll_text2 */ yytestcase(yyruleno==129);
      case 134: /* geocoll_textm2 ::= EWKT_COMMA pointm geocoll_textm2 */ yytestcase(yyruleno==134);
      case 135: /* geocoll_textm2 ::= EWKT_COMMA linestringm geocoll_textm2 */ yytestcase(yyruleno==135);
      case 136: /* geocoll_textm2 ::= EWKT_COMMA polygonm geocoll_textm2 */ yytestcase(yyruleno==136);
      case 141: /* geocoll_textz2 ::= EWKT_COMMA pointz geocoll_textz2 */ yytestcase(yyruleno==141);
      case 142: /* geocoll_textz2 ::= EWKT_COMMA linestringz geocoll_textz2 */ yytestcase(yyruleno==142);
      case 143: /* geocoll_textz2 ::= EWKT_COMMA polygonz geocoll_textz2 */ yytestcase(yyruleno==143);
      case 148: /* geocoll_textzm2 ::= EWKT_COMMA pointzm geocoll_textzm2 */ yytestcase(yyruleno==148);
      case 149: /* geocoll_textzm2 ::= EWKT_COMMA linestringzm geocoll_textzm2 */ yytestcase(yyruleno==149);
      case 150: /* geocoll_textzm2 ::= EWKT_COMMA polygonzm geocoll_textzm2 */ yytestcase(yyruleno==150);
{
		((gaiaGeomCollPtr)yymsp[-1].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[0].minor.yy0;
		yygotominor.yy0 = yymsp[-1].minor.yy0;
	}
        break;
      case 130: /* geocoll_textm ::= EWKT_OPEN_BRACKET pointm geocoll_textm2 EWKT_CLOSE_BRACKET */
      case 131: /* geocoll_textm ::= EWKT_OPEN_BRACKET linestringm geocoll_textm2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==131);
      case 132: /* geocoll_textm ::= EWKT_OPEN_BRACKET polygonm geocoll_textm2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==132);
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_geomColl_xym((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 137: /* geocoll_textz ::= EWKT_OPEN_BRACKET pointz geocoll_textz2 EWKT_CLOSE_BRACKET */
      case 138: /* geocoll_textz ::= EWKT_OPEN_BRACKET linestringz geocoll_textz2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==138);
      case 139: /* geocoll_textz ::= EWKT_OPEN_BRACKET polygonz geocoll_textz2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==139);
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_geomColl_xyz((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 144: /* geocoll_textzm ::= EWKT_OPEN_BRACKET pointzm geocoll_textzm2 EWKT_CLOSE_BRACKET */
      case 145: /* geocoll_textzm ::= EWKT_OPEN_BRACKET linestringzm geocoll_textzm2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==145);
      case 146: /* geocoll_textzm ::= EWKT_OPEN_BRACKET polygonzm geocoll_textzm2 EWKT_CLOSE_BRACKET */ yytestcase(yyruleno==146);
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) ewkt_geomColl_xyzm((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
        break;
      default:
      /* (0) main ::= in */ yytestcase(yyruleno==0);
      /* (1) in ::= */ yytestcase(yyruleno==1);
      /* (2) in ::= in state EWKT_NEWLINE */ yytestcase(yyruleno==2);
      /* (3) state ::= program */ yytestcase(yyruleno==3);
      /* (4) program ::= geo_text */ yytestcase(yyruleno==4);
      /* (5) program ::= geo_textm */ yytestcase(yyruleno==5);
        break;
  };
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yypParser->yyidx -= yysize;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact < YYNSTATE ){
#ifdef NDEBUG
    /* If we are not debugging and the reduce action popped at least
    ** one element off the stack, then we can push the new element back
    ** onto the stack here, and skip the stack overflow test in yy_shift().
    ** That gives a significant speed improvement. */
    if( yysize ){
      yypParser->yyidx++;
      yymsp -= yysize-1;
      yymsp->stateno = (YYACTIONTYPE)yyact;
      yymsp->major = (YYCODETYPE)yygoto;
      yymsp->minor = yygotominor;
    }else
#endif
    {
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
    }
  }else{
    assert( yyact == YYNSTATE + YYNRULE + 1 );
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  YYMINORTYPE yyminor            /* The minor type of the error token */
){
  ParseARG_FETCH;
#define TOKEN (yyminor.yy0)

/* 
** when the LEMON parser encounters an error
** then this global variable is set 
*/
	ewkt_parse_error = 1;
	*result = NULL;
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "ParseAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void Parse(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  ParseTOKENTYPE yyminor       /* The value for the token */
  ParseARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  int yyact;            /* The parser action. */
  int yyendofinput;     /* True if we are at the end of input */
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  /* (re)initialize the parser, if necessary */
  yypParser = (yyParser*)yyp;
  if( yypParser->yyidx<0 ){
#if YYSTACKDEPTH<=0
    if( yypParser->yystksz <=0 ){
      /*memset(&yyminorunion, 0, sizeof(yyminorunion));*/
      yyminorunion = yyzerominor;
      yyStackOverflow(yypParser, &yyminorunion);
      return;
    }
#endif
    yypParser->yyidx = 0;
    yypParser->yyerrcnt = -1;
    yypParser->yystack[0].stateno = 0;
    yypParser->yystack[0].major = 0;
  }
  yyminorunion.yy0 = yyminor;
  yyendofinput = (yymajor==0);
  ParseARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact<YYNSTATE ){
      assert( !yyendofinput );  /* Impossible to shift the $ token */
      yy_shift(yypParser,yyact,yymajor,&yyminorunion);
      yypParser->yyerrcnt--;
      yymajor = YYNOCODE;
    }else if( yyact < YYNSTATE + YYNRULE ){
      yy_reduce(yypParser,yyact-YYNSTATE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yymx = yypParser->yystack[yypParser->yyidx].major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor,&yyminorunion);
        yymajor = YYNOCODE;
      }else{
         while(
          yypParser->yyidx >= 0 &&
          yymx != YYERRORSYMBOL &&
          (yyact = yy_find_reduce_action(
                        yypParser->yystack[yypParser->yyidx].stateno,
                        YYERRORSYMBOL)) >= YYNSTATE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yyidx < 0 || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          YYMINORTYPE u2;
          u2.YYERRSYMDT = 0;
          yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor,yyminorunion);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;
      
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
  return;
}


/*
 EWKT_LEMON_END - LEMON generated code ends here 
*/

















/*
** CAVEAT: there is an incompatibility between LEMON and FLEX
** this macro resolves the issue
*/
#undef yy_accept
#define yy_accept	yy_ewkt_flex_accept




/*
 EWKT_FLEX_START - FLEX generated code starts here 
*/

#line 3 "lex.Ewkt.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer Ewkt_create_buffer
#define yy_delete_buffer Ewkt_delete_buffer
#define yy_flex_debug Ewkt_flex_debug
#define yy_init_buffer Ewkt_init_buffer
#define yy_flush_buffer Ewkt_flush_buffer
#define yy_load_buffer_state Ewkt_load_buffer_state
#define yy_switch_to_buffer Ewkt_switch_to_buffer
#define yyin Ewktin
#define yyleng Ewktleng
#define yylex Ewktlex
#define yylineno Ewktlineno
#define yyout Ewktout
#define yyrestart Ewktrestart
#define yytext Ewkttext
#define yywrap Ewktwrap
#define yyalloc Ewktalloc
#define yyrealloc Ewktrealloc
#define yyfree Ewktfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <errno.h> */
/* #include <stdlib.h> */

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE Ewktrestart(Ewktin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int Ewktleng;

extern FILE *Ewktin, *Ewktout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up Ewkttext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up Ewkttext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via Ewktrestart()), so that the user can continue scanning by
	 * just pointing Ewktin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when Ewkttext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int Ewktleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow Ewktwrap()'s to do buffer switches
 * instead of setting up a fresh Ewktin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void Ewktrestart (FILE *input_file  );
void Ewkt_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE Ewkt_create_buffer (FILE *file,int size  );
void Ewkt_delete_buffer (YY_BUFFER_STATE b  );
void Ewkt_flush_buffer (YY_BUFFER_STATE b  );
void Ewktpush_buffer_state (YY_BUFFER_STATE new_buffer  );
void Ewktpop_buffer_state (void );

static void Ewktensure_buffer_stack (void );
static void Ewkt_load_buffer_state (void );
static void Ewkt_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER Ewkt_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE Ewkt_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE Ewkt_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE Ewkt_scan_bytes (yyconst char *bytes,int len  );

void *Ewktalloc (yy_size_t  );
void *Ewktrealloc (void *,yy_size_t  );
void Ewktfree (void *  );

#define yy_new_buffer Ewkt_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        Ewktensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            Ewkt_create_buffer(Ewktin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        Ewktensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            Ewkt_create_buffer(Ewktin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *Ewktin = (FILE *) 0, *Ewktout = (FILE *) 0;

typedef int yy_state_type;

extern int Ewktlineno;

int Ewktlineno = 1;

extern char *Ewkttext;
#define yytext_ptr Ewkttext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up Ewkttext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	Ewktleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 22
#define YY_END_OF_BUFFER 23
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[93] =
    {   0,
        0,    0,   23,   21,   19,   20,    3,    4,   21,    2,
       21,    1,   21,   21,   21,   21,    1,    1,    1,    1,
        0,    0,    0,    0,    1,    1,    1,    0,    0,    0,
        0,    0,    1,    1,    0,    0,    0,    0,    0,    0,
        0,    0,    5,    0,    0,    0,    0,    0,    6,    0,
        0,    0,    0,    0,    9,    0,    0,    0,    0,    0,
       10,    0,    0,    0,    0,    0,    0,    7,    0,   11,
        0,    0,    8,    0,   12,    0,    0,    0,   15,    0,
        0,   16,    0,    0,    0,   13,    0,   14,    0,   17,
       18,    0

    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    1,    1,    1,    1,    1,    1,    4,
        5,    1,    6,    7,    8,    9,    1,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,    1,    1,    1,
        1,    1,    1,    1,    1,    1,   11,    1,   12,    1,
       13,    1,   14,    1,    1,   15,   16,   17,   18,   19,
        1,   20,   21,   22,   23,    1,    1,    1,   24,    1,
        1,    1,    1,    1,    1,    1,    1,    1,   25,    1,

       26,    1,   27,    1,   28,    1,    1,   29,   30,   31,
       32,   33,    1,   34,   35,   36,   37,    1,    1,    1,
       38,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[39] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[93] =
    {   0,
        0,    0,  187,  188,  188,  188,  188,  188,  174,  188,
      173,   30,   29,   28,   20,   26,   36,   38,  172,   40,
       33,   35,   38,   45,  171,  166,  165,   38,   49,   40,
       46,   40,  164,  163,   57,   49,   57,   50,   66,   58,
       59,   72,   66,   70,   69,   70,   78,   79,  188,   81,
       75,   86,   90,   94,   94,  104,   99,  105,  101,   95,
      188,  102,  112,  105,  105,  115,  120,  120,  115,  122,
      125,  129,  188,  125,  188,  129,  135,  134,  137,  143,
      138,  188,  134,  150,  150,  149,  148,  188,  154,  156,
      188,  188

    } ;

static yyconst flex_int16_t yy_def[93] =
    {   0,
       92,    1,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,    0

    } ;

static yyconst flex_int16_t yy_nxt[227] =
    {   0,
        4,    5,    6,    7,    8,    9,   10,   11,    4,   12,
        4,    4,   13,    4,   14,   15,    4,    4,   16,    4,
        4,    4,    4,    4,    4,    4,   13,    4,   14,   15,
        4,    4,   16,    4,    4,    4,    4,    4,   19,   20,
       21,   22,   23,   24,   25,   17,   26,   18,   19,   20,
       28,   29,   30,   35,   21,   22,   23,   24,   31,   32,
       36,   37,   38,   39,   28,   29,   30,   35,   40,   41,
       42,   43,   31,   32,   36,   37,   38,   39,   44,   45,
       46,   49,   40,   41,   42,   43,   47,   50,   51,   52,
       48,   53,   44,   45,   46,   49,   54,   55,   56,   57,

       47,   50,   51,   52,   48,   53,   58,   59,   60,   61,
       54,   55,   56,   57,   62,   63,   64,   65,   66,   67,
       58,   59,   60,   61,   68,   69,   70,   71,   62,   63,
       64,   65,   66,   67,   72,   73,   74,   75,   68,   69,
       70,   71,   76,   77,   78,   79,   80,   81,   72,   73,
       74,   75,   82,   83,   84,   85,   76,   77,   78,   79,
       80,   81,   86,   87,   88,   89,   82,   83,   84,   85,
       90,   91,   34,   33,   27,   34,   86,   87,   88,   89,
       33,   27,   18,   17,   90,   91,   92,    3,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,

       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92
    } ;

static yyconst flex_int16_t yy_chk[227] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,   12,   12,
       13,   14,   15,   16,   17,   17,   18,   18,   20,   20,
       21,   22,   23,   28,   13,   14,   15,   16,   24,   24,
       29,   30,   31,   32,   21,   22,   23,   28,   35,   36,
       37,   38,   24,   24,   29,   30,   31,   32,   39,   40,
       41,   43,   35,   36,   37,   38,   42,   44,   45,   46,
       42,   47,   39,   40,   41,   43,   48,   50,   51,   52,

       42,   44,   45,   46,   42,   47,   53,   54,   54,   55,
       48,   50,   51,   52,   56,   57,   58,   59,   60,   62,
       53,   54,   54,   55,   63,   64,   65,   66,   56,   57,
       58,   59,   60,   62,   67,   68,   69,   70,   63,   64,
       65,   66,   71,   72,   74,   76,   77,   78,   67,   68,
       69,   70,   79,   80,   81,   83,   71,   72,   74,   76,
       77,   78,   84,   85,   86,   87,   79,   80,   81,   83,
       89,   90,   34,   33,   27,   26,   84,   85,   86,   87,
       25,   19,   11,    9,   89,   90,    3,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,

       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
       92,   92,   92,   92,   92,   92
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int Ewkt_flex_debug;
int Ewkt_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *Ewkttext;
/* 
 EwktLexer.l -- EWKT parser - FLEX config
  
 version 2.4, 2011 May 14

 Author: Sandro Furieri a.furieri@lqt.it

 ------------------------------------------------------------------------------
 
 Version: MPL 1.1/GPL 2.0/LGPL 2.1
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the SpatiaLite library

The Initial Developer of the Original Code is Alessandro Furieri
 
Portions created by the Initial Developer are Copyright (C) 2011
the Initial Developer. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of
either the GNU General Public License Version 2 or later (the "GPL"), or
the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
in which case the provisions of the GPL or the LGPL are applicable instead
of those above. If you wish to allow use of your version of this file only
under the terms of either the GPL or the LGPL, and not to allow others to
use your version of this file under the terms of the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the GPL or the LGPL. If you do not delete
the provisions above, a recipient may use your version of this file under
the terms of any one of the MPL, the GPL or the LGPL.
 
*/

/* For debugging purposes */
int ewkt_line = 1, ewkt_col = 1;

/**
*  The main string-token matcher.
*  The lower case part is probably not needed.  We should really be converting 
*  The string to all uppercase/lowercase to make it case iNsEnSiTiVe.
*  What Flex will do is, For the input string, beginning from the front, Flex
*  will try to match with any of the defined tokens from below.  Flex will 
*  then match the string of longest length.  Suppose the string is: POINTM,
*  Flex would match both POINT and POINTM, but since POINTM is the longer
*  of the two tokens, FLEX will match POINTM.
*/

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* #include <unistd.h> */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int Ewktlex_destroy (void );

int Ewktget_debug (void );

void Ewktset_debug (int debug_flag  );

YY_EXTRA_TYPE Ewktget_extra (void );

void Ewktset_extra (YY_EXTRA_TYPE user_defined  );

FILE *Ewktget_in (void );

void Ewktset_in  (FILE * in_str  );

FILE *Ewktget_out (void );

void Ewktset_out  (FILE * out_str  );

int Ewktget_leng (void );

char *Ewktget_text (void );

int Ewktget_lineno (void );

void Ewktset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int Ewktwrap (void );
#else
extern int Ewktwrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( Ewkttext, Ewktleng, 1, Ewktout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( Ewktin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( Ewktin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, Ewktin))==0 && ferror(Ewktin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(Ewktin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int Ewktlex (void);

#define YY_DECL int Ewktlex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after Ewkttext and Ewktleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! Ewktin )
			Ewktin = stdin;

		if ( ! Ewktout )
			Ewktout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			Ewktensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				Ewkt_create_buffer(Ewktin,YY_BUF_SIZE );
		}

		Ewkt_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of Ewkttext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 93 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 188 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
{ ewkt_col += (int) strlen(Ewkttext);  EwktLval.dval = atof(Ewkttext); return EWKT_NUM; }
	YY_BREAK
case 2:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_COMMA; }
	YY_BREAK
case 3:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_OPEN_BRACKET; }
	YY_BREAK
case 4:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_CLOSE_BRACKET; }
	YY_BREAK
case 5:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_POINT; }
	YY_BREAK
case 6:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_POINT_M; }
	YY_BREAK
case 7:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_LINESTRING; }
	YY_BREAK
case 8:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_LINESTRING_M; }
	YY_BREAK
case 9:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_POLYGON; }
	YY_BREAK
case 10:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_POLYGON_M; }
	YY_BREAK
case 11:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_MULTIPOINT; }
	YY_BREAK
case 12:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_MULTIPOINT_M; }
	YY_BREAK
case 13:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_MULTILINESTRING; }
	YY_BREAK
case 14:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_MULTILINESTRING_M; }
	YY_BREAK
case 15:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_MULTIPOLYGON; }
	YY_BREAK
case 16:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_MULTIPOLYGON_M; }
	YY_BREAK
case 17:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_GEOMETRYCOLLECTION; }
	YY_BREAK
case 18:
YY_RULE_SETUP
{ EwktLval.dval = 0; return EWKT_GEOMETRYCOLLECTION_M; }
	YY_BREAK
case 19:
YY_RULE_SETUP
{ ewkt_col += (int) strlen(Ewkttext); }               /* ignore but count white space */
	YY_BREAK
case 20:
/* rule 20 can match eol */
YY_RULE_SETUP
{ ewkt_col = 0; ++ewkt_line; }
	YY_BREAK
case 21:
YY_RULE_SETUP
{ ewkt_col += (int) strlen(Ewkttext); return -1; }
	YY_BREAK
case 22:
YY_RULE_SETUP
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed Ewktin at a new source and called
			 * Ewktlex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = Ewktin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( Ewktwrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * Ewkttext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of Ewktlex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					Ewktrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			Ewktrestart(Ewktin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) Ewktrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 93 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 93 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 92);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up Ewkttext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					Ewktrestart(Ewktin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( Ewktwrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve Ewkttext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void Ewktrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        Ewktensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            Ewkt_create_buffer(Ewktin,YY_BUF_SIZE );
	}

	Ewkt_init_buffer(YY_CURRENT_BUFFER,input_file );
	Ewkt_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void Ewkt_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		Ewktpop_buffer_state();
	 *		Ewktpush_buffer_state(new_buffer);
     */
	Ewktensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	Ewkt_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (Ewktwrap()) processing, but the only time this flag
	 * is looked at is after Ewktwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void Ewkt_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	Ewktin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE Ewkt_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) Ewktalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in Ewkt_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) Ewktalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in Ewkt_create_buffer()" );

	b->yy_is_our_buffer = 1;

	Ewkt_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with Ewkt_create_buffer()
 * 
 */
    void Ewkt_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		Ewktfree((void *) b->yy_ch_buf  );

	Ewktfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a Ewktrestart() or at EOF.
 */
    static void Ewkt_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	Ewkt_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then Ewkt_init_buffer was _probably_
     * called from Ewktrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void Ewkt_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		Ewkt_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void Ewktpush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	Ewktensure_buffer_stack();

	/* This block is copied from Ewkt_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from Ewkt_switch_to_buffer. */
	Ewkt_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void Ewktpop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	Ewkt_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		Ewkt_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void Ewktensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)Ewktalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in Ewktensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)Ewktrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in Ewktensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE Ewkt_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) Ewktalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in Ewkt_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	Ewkt_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to Ewktlex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       Ewkt_scan_bytes() instead.
 */
YY_BUFFER_STATE Ewkt_scan_string (yyconst char * yystr )
{
    
	return Ewkt_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to Ewktlex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE Ewkt_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) Ewktalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in Ewkt_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = Ewkt_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in Ewkt_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up Ewkttext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		Ewkttext[Ewktleng] = (yy_hold_char); \
		(yy_c_buf_p) = Ewkttext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		Ewktleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int Ewktget_lineno  (void)
{
        
    return Ewktlineno;
}

/** Get the input stream.
 * 
 */
FILE *Ewktget_in  (void)
{
        return Ewktin;
}

/** Get the output stream.
 * 
 */
FILE *Ewktget_out  (void)
{
        return Ewktout;
}

/** Get the length of the current token.
 * 
 */
int Ewktget_leng  (void)
{
        return Ewktleng;
}

/** Get the current token.
 * 
 */

char *Ewktget_text  (void)
{
        return Ewkttext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void Ewktset_lineno (int  line_number )
{
    
    Ewktlineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see Ewkt_switch_to_buffer
 */
void Ewktset_in (FILE *  in_str )
{
        Ewktin = in_str ;
}

void Ewktset_out (FILE *  out_str )
{
        Ewktout = out_str ;
}

int Ewktget_debug  (void)
{
        return Ewkt_flex_debug;
}

void Ewktset_debug (int  bdebug )
{
        Ewkt_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from Ewktlex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    Ewktin = stdin;
    Ewktout = stdout;
#else
    Ewktin = (FILE *) 0;
    Ewktout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * Ewktlex_init()
     */
    return 0;
}

/* Ewktlex_destroy is for both reentrant and non-reentrant scanners. */
int Ewktlex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		Ewkt_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		Ewktpop_buffer_state();
	}

	/* Destroy the stack itself. */
	Ewktfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * Ewktlex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *Ewktalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *Ewktrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void Ewktfree (void * ptr )
{
	free( (char *) ptr );	/* see Ewktrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

/**
 * reset the line and column count
 *
 *
 */
void ewkt_reset_lexer(void)
{

  ewkt_line = 1;
  ewkt_col  = 1;

}

/**
 * EwktError() is invoked when the lexer or the parser encounter
 * an error. The error message is passed via *s
 *
 *
 */
void EwktError(char *s)
{
  printf("error: %s at line: %d col: %d\n",s,ewkt_line,ewkt_col);

}

int Ewktwrap(void)
{
  return 1;
}


/* 
 EWKT_FLEX_END - FLEX generated code ends here 
*/



/*
** This is a linked-list struct to store all the values for each token.
** All tokens will have a value of 0, except tokens denoted as NUM.
** NUM tokens are geometry coordinates and will contain the floating
** point number.
*/
typedef struct ewktFlexTokenStruct
{
    double value;
    struct ewktFlexTokenStruct *Next;
} ewktFlexToken;

/*
** Function to clean up the linked-list of token values.
*/
static int
ewkt_cleanup (ewktFlexToken * token)
{
    ewktFlexToken *ptok;
    ewktFlexToken *ptok_n;
    if (token == NULL)
	return 0;
    ptok = token;
    while (ptok)
      {
	  ptok_n = ptok->Next;
	  free (ptok);
	  ptok = ptok_n;
      }
    return 0;
}

static int
findEwktSrid (const char *buffer, int *base_offset)
{
/* attempting to identify the EWKT SRID */
    char dummy[1024];
    char *out;
    const char *in = buffer;
    int end = -1;
    int i;
    *base_offset = 0;

    while (*in != '\0')
      {
	  /* searching the first semi-colon [srid delimiter] */
	  if (*in == ';')
	    {
		end = in - buffer;
		break;
	    }
	  in++;
      }
    if (end < 0)
	return -1;
    in = buffer;
    out = dummy;
    for (i = 0; i < end; i++)
      {
	  /* normalizing whitespaces */
	  char c = *in;
	  if (c == ' ' || c == '\t' || c == '\n')
	    {
		in++;
		continue;
	    }
	  *out++ = *in++;
      }
    *out = '\0';
    if (strncasecmp (dummy, "SRID=", 5) != 0)
	return -1;
    for (i = 5; i < (int) strlen (dummy); i++)
      {
	  if (i == 5)
	    {
		if (dummy[i] == '-' || dummy[i] == '+')
		    continue;
	    }
	  if (dummy[i] >= '0' && dummy[i] <= '9')
	      continue;
	  return -1;
      }
    *base_offset = end + 1;
    return atoi (dummy + 5);
}

gaiaGeomCollPtr
gaiaParseEWKT (const unsigned char *dirty_buffer)
{
    void *pParser = ParseAlloc (malloc);
    /* Linked-list of token values */
    ewktFlexToken *tokens = malloc (sizeof (ewktFlexToken));
    /* Pointer to the head of the list */
    ewktFlexToken *head = tokens;
    int yv;
    int srid;
    int base_offset;
    gaiaGeomCollPtr result = NULL;

    tokens->Next = NULL;
    ewkt_parse_error = 0;

    srid = findEwktSrid ((char *) dirty_buffer, &base_offset);
    Ewkt_scan_string ((char *) dirty_buffer + base_offset);

    /*
       / Keep tokenizing until we reach the end
       / yylex() will return the next matching Token for us.
     */
    while ((yv = yylex ()) != 0)
      {
	  if (yv == -1)
	    {
		ewkt_parse_error = 1;
		break;
	    }
	  tokens->Next = malloc (sizeof (ewktFlexToken));
	  tokens->Next->Next = NULL;
	  /*
	     /EwktLval is a global variable from FLEX.
	     /EwktLval is defined in ewktLexglobal.h
	   */
	  tokens->Next->value = EwktLval.dval;
	  /* Pass the token to the wkt parser created from lemon */
	  Parse (pParser, yv, &(tokens->Next->value), &result);
	  tokens = tokens->Next;
      }
    /* This denotes the end of a line as well as the end of the parser */
    Parse (pParser, EWKT_NEWLINE, 0, &result);
    ParseFree (pParser, free);
    Ewktlex_destroy ();

    /* Assigning the token as the end to avoid seg faults while cleaning */
    tokens->Next = NULL;
    ewkt_cleanup (head);

    if (ewkt_parse_error)
      {
	  if (result)
	      gaiaFreeGeomColl (result);
	  return NULL;
      }

    if (!result)
	return NULL;
    if (!ewktCheckValidity (result))
      {
	  gaiaFreeGeomColl (result);
	  return NULL;
      }

    gaiaMbrGeometry (result);
    result->Srid = srid;

    return result;
}


/*
** CAVEAT: we must now undefine any Lemon/Flex own macro
*/
#undef YYNOCODE
#undef YYNSTATE
#undef YYNRULE
#undef YY_SHIFT_MAX
#undef YY_SHIFT_USE_DFLT
#undef YY_REDUCE_USE_DFLT
#undef YY_REDUCE_MAX
#undef YY_FLUSH_BUFFER
#undef YY_DO_BEFORE_ACTION
#undef YY_NUM_RULES
#undef YY_END_OF_BUFFER
#undef YY_END_FILE
#undef YYACTIONTYPE
#undef YY_SZ_ACTTAB
#undef YY_NEW_FILE
#undef BEGIN
#undef YY_START
#undef YY_CURRENT_BUFFER
#undef YY_CURRENT_BUFFER_LVALUE
#undef YY_STATE_BUF_SIZE
#undef YY_DECL
#undef YY_FATAL_ERROR
#undef YYMINORTYPE
#undef YY_CHAR
#undef YYSTYPE
#undef input
#undef ParseAlloc
#undef ParseFree
#undef ParseStackPeak
#undef Parse
#undef yyalloc
#undef yyfree
#undef yyin
#undef yyleng
#undef yyless
#undef yylex
#undef yylineno
#undef yyout
#undef yyrealloc
#undef yyrestart
#undef yyStackEntry
#undef yytext
#undef yywrap
#undef yyzerominor
#undef yy_accept
#undef yy_action
#undef yy_base
#undef yy_buffer_stack
#undef yy_buffer_stack_max
#undef yy_buffer_stack_top
#undef yy_c_buf_p
#undef yy_chk
#undef yy_create_buffer
#undef yy_def
#undef yy_default
#undef yy_delete_buffer
#undef yy_destructor
#undef yy_ec
#undef yy_fatal_error
#undef yy_find_reduce_action
#undef yy_find_shift_action
#undef yy_flex_debug
#undef yy_flush_buffer
#undef yy_get_next_buffer
#undef yy_get_previous_state
#undef yy_init
#undef yy_init_buffer
#undef yy_init_globals
#undef yy_load_buffer
#undef yy_load_buffer_state
#undef yy_lookahead
#undef yy_meta
#undef yy_new_buffer
#undef yy_nxt
#undef yy_parse_failed
#undef yy_pop_parser_stack
#undef yy_reduce
#undef yy_reduce_ofst
#undef yy_set_bol
#undef yy_set_interactive
#undef yy_shift
#undef yy_shift_ofst
#undef yy_start
#undef yy_state_type
#undef yy_switch_to_buffer
#undef yy_syntax_error
#undef yy_trans_info
#undef yy_try_NUL_trans
#undef yyParser
#undef yyStackEntry
#undef yyStackOverflow
#undef yyRuleInfo
#undef yytext_ptr
#undef yyunput
#undef yyzerominor
#undef ParseARG_SDECL
#undef ParseARG_PDECL
#undef ParseARG_FETCH
#undef ParseARG_STORE
#undef REJECT
#undef yymore
#undef YY_MORE_ADJ
#undef YY_RESTORE_YY_MORE_OFFSET
#undef YY_LESS_LINENO
#undef yyTracePrompt
#undef yyTraceFILE
#undef yyTokenName
#undef yyRuleName
#undef ParseTrace
/**************** End file: gg_ewkt.c **********/


/**************** Begin file: gg_geoJSON.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

/* #include <assert.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

int geoJSON_parse_error;

static int
geoJsonCheckValidity (gaiaGeomCollPtr geom)
{
/* checks if this one is a degenerated geometry */
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaRingPtr rng;
    int ib;
    int entities = 0;
    pt = geom->FirstPoint;
    while (pt)
      {
	  /* checking points */
	  entities++;
	  pt = pt->Next;
      }
    ln = geom->FirstLinestring;
    while (ln)
      {
	  /* checking linestrings */
	  if (ln->Points < 2)
	      return 0;
	  entities++;
	  ln = ln->Next;
      }
    pg = geom->FirstPolygon;
    while (pg)
      {
	  /* checking polygons */
	  rng = pg->Exterior;
	  if (rng->Points < 4)
	      return 0;
	  for (ib = 0; ib < pg->NumInteriors; ib++)
	    {
		rng = pg->Interiors + ib;
		if (rng->Points < 4)
		    return 0;
	    }
	  entities++;
	  pg = pg->Next;
      }
    if (!entities)
	return 0;
    return 1;
}

static gaiaGeomCollPtr
geoJSON_setSrid (gaiaGeomCollPtr geom, int *srid)
{
/* setting up the SRID */
    if (!geom)
	return NULL;
    geom->Srid = *srid;
    return geom;
}

static gaiaGeomCollPtr
gaiaGeoJsonGeometryFromPoint (gaiaPointPtr point, int srid)
{
/* builds a GEOMETRY containing a POINT */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomColl ();
    geom->DeclaredType = GAIA_POINT;
    geom->Srid = srid;
    gaiaAddPointToGeomColl (geom, point->X, point->Y);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeoJsonGeometryFromPointZ (gaiaPointPtr point, int srid)
{
/* builds a GEOMETRY containing a POINTZ */
    gaiaGeomCollPtr geom = NULL;
    geom = gaiaAllocGeomCollXYZ ();
    geom->DeclaredType = GAIA_POINTZ;
    geom->Srid = srid;
    gaiaAddPointToGeomCollXYZ (geom, point->X, point->Y, point->Z);
    gaiaFreePoint (point);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeoJsonGeometryFromLinestring (gaiaLinestringPtr line, int srid)
{
/* builds a GEOMETRY containing a LINESTRING */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    geom = gaiaAllocGeomColl ();
    geom->DeclaredType = GAIA_LINESTRING;
    geom->Srid = srid;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPoint (line->Coords, iv, &x, &y);
	  gaiaSetPoint (line2->Coords, iv, x, y);
      }
    gaiaFreeLinestring (line);
    return geom;
}

static gaiaGeomCollPtr
gaiaGeoJsonGeometryFromLinestringZ (gaiaLinestringPtr line, int srid)
{
/* builds a GEOMETRY containing a LINESTRINGZ */
    gaiaGeomCollPtr geom = NULL;
    gaiaLinestringPtr line2;
    int iv;
    double x;
    double y;
    double z;
    geom = gaiaAllocGeomCollXYZ ();
    geom->DeclaredType = GAIA_LINESTRING;
    geom->Srid = srid;
    line2 = gaiaAddLinestringToGeomColl (geom, line->Points);
    for (iv = 0; iv < line2->Points; iv++)
      {
	  /* sets the POINTS for the exterior ring */
	  gaiaGetPointXYZ (line->Coords, iv, &x, &y, &z);
	  gaiaSetPointXYZ (line2->Coords, iv, x, y, z);
      }
    gaiaFreeLinestring (line);
    return geom;
}

static gaiaPointPtr
geoJSON_point_xy (double *x, double *y)
{
    return gaiaAllocPoint (*x, *y);
}

/* 
 * Creates a 3D (xyz) point in SpatiaLite
 * x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created.
 * Returns a gaiaPointPtr representing the created point.
 *
 * Creates a 3D (xyz) point. This is a parser helper function which is called when 3D coordinates are encountered.
 * Parameters x, y, and z are pointers to doubles which represent the x, y, and z coordinates of the point to be created.
 * Returns a gaiaPointPtr pointing to the 3D created point.
 */
static gaiaPointPtr
geoJSON_point_xyz (double *x, double *y, double *z)
{
    return gaiaAllocPointXYZ (*x, *y, *z);
}

/*
 * Builds a geometry collection from a point. The geometry collection should contain only one element ? the point. 
 * The correct geometry type must be *decided based on the point type. The parser should call this function when the 
 * ?POINT? WKT expression is encountered.
 * Parameter point is a pointer to a 2D, 3D, 2D with an m value, or 4D with an m value point.
 * Returns a geometry collection containing the point. The geometry must have FirstPoint and LastPoint  pointing to the
 * same place as point.  *DimensionModel must be the same as the model of the point and DimensionType must be GAIA_TYPE_POINT.
 */
static gaiaGeomCollPtr
geoJSON_buildGeomFromPoint (gaiaPointPtr point)
{
    switch (point->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaGeoJsonGeometryFromPoint (point, -1);
	  break;
      case GAIA_XY_Z:
	  return gaiaGeoJsonGeometryFromPointZ (point, -1);
	  break;
      }
    return NULL;
}

static gaiaGeomCollPtr
geoJSON_buildGeomFromPointSrid (gaiaPointPtr point, int *srid)
{
    switch (point->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaGeoJsonGeometryFromPoint (point, *srid);
	  break;
      case GAIA_XY_Z:
	  return gaiaGeoJsonGeometryFromPointZ (point, *srid);
	  break;
      }
    return NULL;
}


/* 
 * Creates a 2D (xy) linestring from a list of 2D points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 2D (xy) points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
geoJSON_linestring_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestring (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPoint (linestring->Coords, i, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/* 
 * Creates a 3D (xyz) linestring from a list of 3D points.
 *
 * Parameter first is a gaiaPointPtr to the first point in a linked list of points which define the linestring.
 * All of the points in the list must be 3D (xyz) points. There must be at least 2 points in the list.
 *
 * Returns a pointer to linestring containing all of the points in the list.
 */
static gaiaLinestringPtr
geoJSON_linestring_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    int points = 0;
    int i = 0;
    gaiaLinestringPtr linestring;

    while (p != NULL)
      {
	  p = p->Next;
	  points++;
      }

    linestring = gaiaAllocLinestringXYZ (points);

    p = first;
    while (p != NULL)
      {
	  gaiaSetPointXYZ (linestring->Coords, i, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
	  i++;
      }

    return linestring;
}

/*
 * Builds a geometry collection from a linestring.
 */
static gaiaGeomCollPtr
geoJSON_buildGeomFromLinestring (gaiaLinestringPtr line)
{
    switch (line->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaGeoJsonGeometryFromLinestring (line, -1);
	  break;
      case GAIA_XY_Z:
	  return gaiaGeoJsonGeometryFromLinestringZ (line, -1);
	  break;
      }
    return NULL;
}

static gaiaGeomCollPtr
geoJSON_buildGeomFromLinestringSrid (gaiaLinestringPtr line, int *srid)
{
    switch (line->DimensionModel)
      {
      case GAIA_XY:
	  return gaiaGeoJsonGeometryFromLinestring (line, *srid);
	  break;
      case GAIA_XY_Z:
	  return gaiaGeoJsonGeometryFromLinestringZ (line, *srid);
	  break;
      }
    return NULL;
}

/*
 * Helper function that determines the number of points in the linked list.
 */
static int
geoJSON_count_points (gaiaPointPtr first)
{
    /* Counts the number of points in the ring. */
    gaiaPointPtr p = first;
    int numpoints = 0;
    while (p != NULL)
      {
	  numpoints++;
	  p = p->Next;
      }
    return numpoints;
}

/*
 * Creates a 2D (xy) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 2D (xy) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
geoJSON_ring_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = geoJSON_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRing (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPoint (ring->Coords, index, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Creates a 3D (xyz) ring in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points which define the polygon.
 * All of the points given to the function are 3D (xyz) points. There will be at least 4 points in the list.
 *
 * Returns the ring defined by the points given to the function.
 */
static gaiaRingPtr
geoJSON_ring_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaRingPtr ring = NULL;
    int numpoints;
    int index;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Counts the number of points in the ring. */
    numpoints = geoJSON_count_points (first);
    if (numpoints < 4)
	return NULL;

    /* Creates and allocates a ring structure. */
    ring = gaiaAllocRingXYZ (numpoints);
    if (ring == NULL)
	return NULL;

    /* Adds every point into the ring structure. */
    p = first;
    for (index = 0; index < numpoints; index++)
      {
	  gaiaSetPointXYZ (ring->Coords, index, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }

    return ring;
}

/*
 * Helper function that will create any type of polygon (xy, xyz) in SpatiaLite.
 * 
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are of the same type. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
geoJSON_polygon_any_type (gaiaRingPtr first)
{
    gaiaRingPtr p;
    gaiaRingPtr p_n;
    gaiaPolygonPtr polygon;
    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a polygon structure with the exterior ring. */
    polygon = gaiaCreatePolygon (first);
    if (polygon == NULL)
	return NULL;

    /* Adds all interior rings into the polygon structure. */
    p = first;
    while (p != NULL)
      {
	  p_n = p->Next;
	  if (p == first)
	      gaiaFreeRing (p);
	  else
	      gaiaAddRingToPolyg (polygon, p);
	  p = p_n;
      }

    return polygon;
}

/* 
 * Creates a 2D (xy) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 2D (xy) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
geoJSON_polygon_xy (gaiaRingPtr first)
{
    return geoJSON_polygon_any_type (first);
}

/* 
 * Creates a 3D (xyz) polygon in SpatiaLite
 *
 * first is a gaiaRingPtr to the first ring in a linked list of rings which define the polygon.
 * The first ring in the linked list is the external ring while the rest (if any) are internal rings.
 * All of the rings given to the function are 3D (xyz) rings. There will be at least 1 ring in the list.
 *
 * Returns the polygon defined by the rings given to the function.
 */
static gaiaPolygonPtr
geoJSON_polygon_xyz (gaiaRingPtr first)
{
    return geoJSON_polygon_any_type (first);
}

/*
 * Builds a geometry collection from a polygon.
 * NOTE: This function may already be implemented in the SpatiaLite code base. If it is, make sure that we
 *              can use it (ie. it doesn't use any other variables or anything else set by Sandro's parser). If you find
 *              that we can use an existing function then ignore this one.
 */
static gaiaGeomCollPtr
geoJSON_buildGeomFromPolygon (gaiaPolygonPtr polygon)
{
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (polygon == NULL)
      {
	  return NULL;
      }

    /* Creates and allocates a geometry collection containing a multipoint. */
    switch (polygon->DimensionModel)
      {
      case GAIA_XY:
	  geom = gaiaAllocGeomColl ();
	  break;
      case GAIA_XY_Z:
	  geom = gaiaAllocGeomCollXYZ ();
	  break;
      }
    if (geom == NULL)
      {
	  return NULL;
      }
    geom->DeclaredType = GAIA_POLYGON;

    /* Stores the location of the first and last polygons in the linked list. */
    geom->FirstPolygon = polygon;
    while (polygon != NULL)
      {
	  geom->LastPolygon = polygon;
	  polygon = polygon->Next;
      }
    return geom;
}

static gaiaGeomCollPtr
geoJSON_buildGeomFromPolygonSrid (gaiaPolygonPtr polygon, int *srid)
{
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (polygon == NULL)
      {
	  return NULL;
      }

    /* Creates and allocates a geometry collection containing a multipoint. */
    switch (polygon->DimensionModel)
      {
      case GAIA_XY:
	  geom = gaiaAllocGeomColl ();
	  break;
      case GAIA_XY_Z:
	  geom = gaiaAllocGeomCollXYZ ();
	  break;
      }
    if (geom == NULL)
      {
	  return NULL;
      }
    geom->DeclaredType = GAIA_POLYGON;
    geom->Srid = *srid;

    /* Stores the location of the first and last polygons in the linked list. */
    geom->FirstPolygon = polygon;
    while (polygon != NULL)
      {
	  geom->LastPolygon = polygon;
	  polygon = polygon->Next;
      }
    return geom;
}

/* 
 * Creates a 2D (xy) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 2D (xy) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
geoJSON_multipoint_xy (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 2D (xy) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomColl (geom, p->X, p->Y);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/* 
 * Creates a 3D (xyz) multipoint object in SpatiaLite
 *
 * first is a gaiaPointPtr to the first point in a linked list of points.
 * All of the points given to the function are 3D (xyz) points. There will be at least 1 point in the list.
 *
 * Returns a geometry collection containing the created multipoint object.
 */
static gaiaGeomCollPtr
geoJSON_multipoint_xyz (gaiaPointPtr first)
{
    gaiaPointPtr p = first;
    gaiaPointPtr p_n;
    gaiaGeomCollPtr geom = NULL;

    /* If no pointers are given, return. */
    if (first == NULL)
	return NULL;

    /* Creates and allocates a geometry collection containing a multipoint. */
    geom = gaiaAllocGeomCollXYZ ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_MULTIPOINT;

    /* For every 3D (xyz) point, add it to the geometry collection. */
    while (p != NULL)
      {
	  gaiaAddPointToGeomCollXYZ (geom, p->X, p->Y, p->Z);
	  p_n = p->Next;
	  gaiaFreePoint (p);
	  p = p_n;
      }
    return geom;
}

/*
 * Creates a geometry collection containing 2D (xy) linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 2D (xy) linestrings. There must be at least 1 linestring in the list.
 * Returns a pointer to the created geometry collection of 2D linestrings. The geometry must have FirstLinestring pointing to the
 * first linestring in the list pointed by first and LastLinestring pointing to the last element of the same list. DimensionModel
 * must be GAIA_XY and DimensionType must be *GAIA_TYPE_LINESTRING.
 */

static gaiaGeomCollPtr
geoJSON_multilinestring_xy (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomColl ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }

    return a;
}

/* 
 * Returns a geometry collection containing the created multilinestring object (?).
 * Creates a geometry collection containing 3D (xyz) linestrings.
 * Parameter first is a gaiaLinestringPtr to the first linestring in a linked list of linestrings which should be added to the
 * collection. All of the *linestrings in the list must be 3D (xyz) linestrings. There must be at least 1 linestring in the list.
 * Returns a pointer to the created geometry collection of 3D linestrings. The geometry must have FirstLinestring pointing to the
 * first linestring in the *list pointed by first and LastLinestring pointing to the last element of the same list. DimensionModel
 * must be GAIA_XYZ and DimensionType must be *GAIA_TYPE_LINESTRING.
 */
static gaiaGeomCollPtr
geoJSON_multilinestring_xyz (gaiaLinestringPtr first)
{
    gaiaLinestringPtr p = first;
    gaiaLinestringPtr p_n;
    gaiaLinestringPtr new_line;
    gaiaGeomCollPtr a = gaiaAllocGeomCollXYZ ();
    a->DeclaredType = GAIA_MULTILINESTRING;
    a->DimensionModel = GAIA_XY_Z;

    while (p)
      {
	  new_line = gaiaAddLinestringToGeomColl (a, p->Points);
	  gaiaCopyLinestringCoords (new_line, p);
	  p_n = p->Next;
	  gaiaFreeLinestring (p);
	  p = p_n;
      }
    return a;
}

/* 
 * Creates a geometry collection containing 2D (xy) polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should 
 * be added to the collection. All of the polygons in the list must be 2D (xy) polygons. There must be 
 * at least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 2D polygons. The geometry must have 
 * FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing 
 * to the last element of the same list. DimensionModel must be GAIA_XY and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
geoJSON_multipolygon_xy (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

/* 
 * Creates a geometry collection containing 3D (xyz) polygons.
 *
 * Parameter first is a gaiaPolygonPtr to the first polygon in a linked list of polygons which should be 
 * added to the collection. All of the polygons in the list must be 3D (xyz) polygons. There must be at
 * least 1 polygon in the list.
 *
 * Returns a pointer to the created geometry collection of 3D polygons. The geometry must have 
 * FirstPolygon pointing to the first polygon in the list pointed by first and LastPolygon pointing to 
 * the last element of the same list. DimensionModel must be GAIA_XYZ and DimensionType must 
 * be GAIA_TYPE_POLYGON.
 *
 */
static gaiaGeomCollPtr
geoJSON_multipolygon_xyz (gaiaPolygonPtr first)
{
    gaiaPolygonPtr p = first;
    gaiaPolygonPtr p_n;
    int i = 0;
    gaiaPolygonPtr new_polyg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    gaiaGeomCollPtr geom = gaiaAllocGeomCollXYZ ();

    geom->DeclaredType = GAIA_MULTIPOLYGON;

    while (p)
      {
	  i_ring = p->Exterior;
	  new_polyg =
	      gaiaAddPolygonToGeomColl (geom, i_ring->Points, p->NumInteriors);
	  o_ring = new_polyg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);

	  for (i = 0; i < new_polyg->NumInteriors; i++)
	    {
		i_ring = p->Interiors + i;
		o_ring = gaiaAddInteriorRing (new_polyg, i, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }

	  p_n = p->Next;
	  gaiaFreePolygon (p);
	  p = p_n;
      }

    return geom;
}

static void
geoJSON_geomColl_common (gaiaGeomCollPtr org, gaiaGeomCollPtr dst)
{
/* 
/ helper function: xfers entities between the Origin and Destination 
/ Sandro Furieri: 2010 October 12
*/
    gaiaGeomCollPtr p = org;
    gaiaGeomCollPtr p_n;
    gaiaPointPtr pt;
    gaiaPointPtr pt_n;
    gaiaLinestringPtr ln;
    gaiaLinestringPtr ln_n;
    gaiaPolygonPtr pg;
    gaiaPolygonPtr pg_n;
    while (p)
      {
	  pt = p->FirstPoint;
	  while (pt)
	    {
		pt_n = pt->Next;
		pt->Next = NULL;
		if (dst->FirstPoint == NULL)
		    dst->FirstPoint = pt;
		if (dst->LastPoint != NULL)
		    dst->LastPoint->Next = pt;
		dst->LastPoint = pt;
		pt = pt_n;
	    }
	  ln = p->FirstLinestring;
	  while (ln)
	    {
		ln_n = ln->Next;
		ln->Next = NULL;
		if (dst->FirstLinestring == NULL)
		    dst->FirstLinestring = ln;
		if (dst->LastLinestring != NULL)
		    dst->LastLinestring->Next = ln;
		dst->LastLinestring = ln;
		ln = ln_n;
	    }
	  pg = p->FirstPolygon;
	  while (pg)
	    {
		pg_n = pg->Next;
		pg->Next = NULL;
		if (dst->FirstPolygon == NULL)
		    dst->FirstPolygon = pg;
		if (dst->LastPolygon != NULL)
		    dst->LastPolygon->Next = pg;
		dst->LastPolygon = pg;
		pg = pg_n;
	    }
	  p_n = p->Next;
	  p->FirstPoint = NULL;
	  p->LastPoint = NULL;
	  p->FirstLinestring = NULL;
	  p->LastLinestring = NULL;
	  p->FirstPolygon = NULL;
	  p->LastPolygon = NULL;
	  gaiaFreeGeomColl (p);
	  p = p_n;
      }
}

/* Creates a 2D (xy) geometry collection in SpatiaLite
 *
 * first is the first geometry collection in a linked list of geometry collections.
 * Each geometry collection represents a single type of object (eg. one could be a POINT, 
 * another could be a LINESTRING, another could be a MULTILINESTRING, etc.).
 *
 * The type of object represented by any geometry collection is stored in the declaredType 
 * field of its struct. For example, if first->declaredType = GAIA_POINT, then first represents a point.
 * If first->declaredType = GAIA_MULTIPOINT, then first represents a multipoint.
 *
 * NOTE: geometry collections cannot contain other geometry collections (have to confirm this 
 * with Sandro).
 *
 * The goal of this function is to take the information from all of the structs in the linked list and 
 * return one geomColl struct containing all of that information.
 *
 * The integers used for 'declaredType' are defined in gaiageo.h. In this function, the only values 
 * contained in 'declaredType' that will be encountered will be:
 *
 *	GAIA_POINT, GAIA_LINESTRING, GAIA_POLYGON, 
 *	GAIA_MULTIPOINT, GAIA_MULTILINESTRING, GAIA_MULTIPOLYGON
 */
static gaiaGeomCollPtr
geoJSON_geomColl_xy (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY;
    geoJSON_geomColl_common (first, geom);
    return geom;
}

/*
 * See geomColl_xy for description.
 *
 * The only difference between this function and geomColl_xy is that the 'declaredType' field of the structs
 * in the linked list for this function will only contain the following types:
 *
 *	GAIA_POINTZ, GAIA_LINESTRINGZ, GAIA_POLYGONZ,
 * 	GAIA_MULTIPOINTZ, GAIA_MULTILINESTRINGZ, GAIA_MULTIPOLYGONZ
 */
static gaiaGeomCollPtr
geoJSON_geomColl_xyz (gaiaGeomCollPtr first)
{
    gaiaGeomCollPtr geom = gaiaAllocGeomColl ();
    if (geom == NULL)
	return NULL;
    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
    geom->DimensionModel = GAIA_XY_Z;
    geoJSON_geomColl_common (first, geom);
    return geom;
}



/*
** CAVEAT: we must redefine any Lemon/Flex own macro
*/
#define YYMINORTYPE		GEO_JSON_MINORTYPE
#define YY_CHAR			GEO_JSON_YY_CHAR
#define	input			geoJSON_input
#define ParseAlloc		geoJSONParseAlloc
#define ParseFree		geoJSONParseFree
#define ParseStackPeak		geoJSONParseStackPeak
#define Parse			geoJSONParse
#define yyStackEntry		geoJSON_yyStackEntry
#define yyzerominor		geoJSON_yyzerominor
#define yy_accept		geoJSON_yy_accept
#define yy_action		geoJSON_yy_action
#define yy_base			geoJSON_yy_base
#define yy_buffer_stack		geoJSON_yy_buffer_stack
#define yy_buffer_stack_max	geoJSON_yy_buffer_stack_max
#define yy_buffer_stack_top	geoJSON_yy_buffer_stack_top
#define yy_c_buf_p		geoJSON_yy_c_buf_p
#define yy_chk			geoJSON_yy_chk
#define yy_def			geoJSON_yy_def
#define yy_default		geoJSON_yy_default
#define yy_destructor		geoJSON_yy_destructor
#define yy_ec			geoJSON_yy_ec
#define yy_fatal_error		geoJSON_yy_fatal_error
#define yy_find_reduce_action	geoJSON_yy_find_reduce_action
#define yy_find_shift_action	geoJSON_yy_find_shift_action
#define yy_get_next_buffer	geoJSON_yy_get_next_buffer
#define yy_get_previous_state	geoJSON_yy_get_previous_state
#define yy_init			geoJSON_yy_init
#define yy_init_globals		geoJSON_yy_init_globals
#define yy_lookahead		geoJSON_yy_lookahead
#define yy_meta			geoJSON_yy_meta
#define yy_nxt			geoJSON_yy_nxt
#define yy_parse_failed		geoJSON_yy_parse_failed
#define yy_pop_parser_stack	geoJSON_yy_pop_parser_stack
#define yy_reduce		geoJSON_yy_reduce
#define yy_reduce_ofst		geoJSON_yy_reduce_ofst
#define yy_shift		geoJSON_yy_shift
#define yy_shift_ofst		geoJSON_yy_shift_ofst
#define yy_start		geoJSON_yy_start
#define yy_state_type		geoJSON_yy_state_type
#define yy_syntax_error		geoJSON_yy_syntax_error
#define yy_trans_info		geoJSON_yy_trans_info
#define yy_try_NUL_trans	geoJSON_yy_try_NUL_trans
#define yyParser		geoJSON_yyParser
#define yyStackEntry		geoJSON_yyStackEntry
#define yyStackOverflow		geoJSON_yyStackOverflow
#define yyRuleInfo		geoJSON_yyRuleInfo
#define yyunput			geoJSON_yyunput
#define yyzerominor		geoJSON_yyzerominor
#define yyTraceFILE		geoJSON_yyTraceFILE
#define yyTracePrompt		geoJSON_yyTracePrompt
#define yyTokenName		geoJSON_yyTokenName
#define yyRuleName		geoJSON_yyRuleName
#define ParseTrace		geoJSON_ParseTrace


/*
 GEOJSON_LEMON_H_START - LEMON generated header starts here 
*/
#define GEOJSON_NEWLINE                 1
#define GEOJSON_OPEN_BRACE              2
#define GEOJSON_TYPE                    3
#define GEOJSON_COLON                   4
#define GEOJSON_POINT                   5
#define GEOJSON_COMMA                   6
#define GEOJSON_COORDS                  7
#define GEOJSON_CLOSE_BRACE             8
#define GEOJSON_BBOX                    9
#define GEOJSON_OPEN_BRACKET           10
#define GEOJSON_CLOSE_BRACKET          11
#define GEOJSON_CRS                    12
#define GEOJSON_NAME                   13
#define GEOJSON_PROPS                  14
#define GEOJSON_NUM                    15
#define GEOJSON_SHORT_SRID             16
#define GEOJSON_LONG_SRID              17
#define GEOJSON_LINESTRING             18
#define GEOJSON_POLYGON                19
#define GEOJSON_MULTIPOINT             20
#define GEOJSON_MULTILINESTRING        21
#define GEOJSON_MULTIPOLYGON           22
#define GEOJSON_GEOMETRYCOLLECTION     23
#define GEOJSON_GEOMS                  24
/*
 GEOJSON_LEMON_H_END - LEMON generated header ends here 
*/


typedef union
{
    double dval;
    int ival;
    struct symtab *symp;
} geoJSON_yystype;
#define YYSTYPE geoJSON_yystype


/* extern YYSTYPE yylval; */
YYSTYPE GeoJsonLval;



/*
 GEOJSON_LEMON_START - LEMON generated code starts here 
*/

/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is included that follows the "include" declaration
** in the input grammar file. */
/* #include <stdio.h> */

/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash 
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    ParseTOKENTYPE     is the data type used for minor tokens given 
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is ParseTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    ParseARG_SDECL     A static variable declaration for the %extra_argument
**    ParseARG_PDECL     A parameter declaration for the %extra_argument
**    ParseARG_STORE     Code to store %extra_argument into yypParser
**    ParseARG_FETCH     Code to extract %extra_argument from yypParser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 84
#define YYACTIONTYPE unsigned short int
#define ParseTOKENTYPE void *
typedef union {
  int yyinit;
  ParseTOKENTYPE yy0;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 1000000
#endif
#define ParseARG_SDECL  gaiaGeomCollPtr *result ;
#define ParseARG_PDECL , gaiaGeomCollPtr *result 
#define ParseARG_FETCH  gaiaGeomCollPtr *result  = yypParser->result 
#define ParseARG_STORE yypParser->result  = result 
#define YYNSTATE 679
#define YYNRULE 159
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* The yyzerominor constant is used to initialize instances of
** YYMINORTYPE objects to zero. */
static const YYMINORTYPE yyzerominor = { 0 };

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
static const YYACTIONTYPE yy_action[] = {
 /*     0 */   184,  533,  534,  535,  536,  537,  538,  539,  540,  541,
 /*    10 */   542,  543,  544,  545,  546,  547,  548,  187,  139,  189,
 /*    20 */   146,  147,  148,  190,  155,  156,  679,  104,  185,  112,
 /*    30 */   242,  287,  334,  373,  412,  451,  139,  195,  146,  147,
 /*    40 */   140,  201,  141,  142,  512,  149,   88,  150,  151,  157,
 /*    50 */   148,  551,  155,  156,  496,  188,  243,  193,  254,  509,
 /*    60 */   204,  262,  455,  288,  335,  301,  340,  471,  309,  348,
 /*    70 */   507,  374,  413,  379,  418,  457,  387,  426,  482,  191,
 /*    80 */   498,  480,  452,  567,  568,  839,    1,  169,  461,  465,
 /*    90 */    93,  486,  490,  501,  504,  238,  240,  211,  199,  207,
 /*   100 */    86,  201,  200,  208,  101,  244,  245,  215,  205,  217,
 /*   110 */   553,  216,  223,  219,  227,   99,  201,  220,  228,  246,
 /*   120 */   229,   19,  256,  250,  263,  275,  201,  260,  261,   95,
 /*   130 */    30,  269,   31,  265,  266,  201,  273,  274,  277,  278,
 /*   140 */   281,  285,  286,   42,  201,  292,  289,  290,  303,  297,
 /*   150 */   307,  308,  201,  310,  322,  312,  313,   96,  316,  328,
 /*   160 */   320,  321,  201,  201,  324,  325,  332,  333,  206,  120,
 /*   170 */   209,  336,  337,  121,  342,  192,  349,  361,  201,   53,
 /*   180 */   346,  347,  351,  352,  355,   64,  359,  360,  201,  363,
 /*   190 */   364,  367,  375,  376,    2,  201,  381,  371,  372,  394,
 /*   200 */   201,  122,  126,  201,  385,  386,  388,  400,  390,  391,
 /*   210 */   414,  415,  398,  399,  402,  403,  406,  410,  411,  420,
 /*   220 */   201,  130,  134,  201,  424,  425,  427,  439,  433,  429,
 /*   230 */   430,  218,  201,  221,  437,  438,  441,  442,  445,  477,
 /*   240 */   478,  202,  201,  449,  450,  473,  460,  468,  493,  201,
 /*   250 */   485,  464,  489,  508,  520,  514,  510,  511,  524,  201,
 /*   260 */   518,  519,  522,  523,  526,  203,  530,  531,  201,  264,
 /*   270 */   276,  267,  279,  521,  311,  323,  314,  326,  350,  362,
 /*   280 */   353,  365,  389,  401,  392,  404,  428,  440,  431,  443,
 /*   290 */   556,  247,  248,   97,   99,  251,  249,  571,  102,  105,
 /*   300 */   252,  253,  103,  292,  106,  573,  291,  295,  113,  293,
 /*   310 */   294,  111,  123,  114,  110,  589,  125,  124,  296,  297,
 /*   320 */   119,  127,  298,  299,  377,  840,  118,  620,  592,  128,
 /*   330 */   246,  131,  300,  132,  338,  135,  339,  250,  136,    4,
 /*   340 */     3,    5,  129,  532,    6,  158,  186,  133,    8,  552,
 /*   350 */   104,  378,  159,  549,  460,  622,  137,  550,  112,   87,
 /*   360 */   416,  194,    9,   10,  197,  464,  636,  196,  417,  468,
 /*   370 */   485,  638,  198,  453,  554,  555,  489,  454,   89,   90,
 /*   380 */   651,  652,  493,   91,  653,   11,  170,  469,   12,  470,
 /*   390 */   557,  558,  210,  213,  479,   13,  662,  663,  664,  214,
 /*   400 */   212,   14,  494,  559,  560,  678,  171,  495,   15,  561,
 /*   410 */   562,  222,  225,   16,   17,  226,  224,  563,  230,  564,
 /*   420 */   231,  236,  235,  233,   20,  232,    7,  234,  456,  239,
 /*   430 */   840,  565,  241,  237,  566,  569,  160,   18,  840,  551,
 /*   440 */   570,   92,   94,   98,  100,  572,   88,  255,  840,   21,
 /*   450 */   258,  840,  574,  257,  259,   22,  575,  270,  576,  172,
 /*   460 */    23,  268,  577,  578,  271,   25,  840,  272,   26,   24,
 /*   470 */   579,  580,  173,  581,  280,  840,  582,  840,  283,   27,
 /*   480 */   284,   28,  282,  583,   32,   29,  161,  584,  840,  840,
 /*   490 */   585,  586,  107,  108,  109,  587,  304,  840,  840,  115,
 /*   500 */   116,  117,  588,  590,  302,   33,  840,  591,  305,  840,
 /*   510 */   306,  840,  593,  594,  174,   34,   36,   35,  840,   39,
 /*   520 */   315,  476,  595,  596,  318,   37,   75,  840,  317,  840,
 /*   530 */   319,  329,  597,  598,  599,  175,   40,   38,  327,  600,
 /*   540 */   840,  330,  840,  331,   41,  601,  602,  603,  162,  840,
 /*   550 */   840,  341,  604,  840,  840,  840,  605,  606,   43,  344,
 /*   560 */   343,  840,   44,  356,  345,   45,  840,  607,  608,  176,
 /*   570 */    46,  358,  840,  609,  354,  840,  610,  357,   48,   47,
 /*   580 */   611,   49,  840,  612,  177,  366,  840,  613,  614,  840,
 /*   590 */   840,  369,   50,  370,  163,  368,   51,  615,   52,  840,
 /*   600 */   840,  616,  840,  617,  618,  840,  380,  840,  619,  621,
 /*   610 */   382,  383,  840,  178,   54,   55,  384,  840,  623,   56,
 /*   620 */   624,   57,  625,  393,  626,  840,  840,   59,  396,  840,
 /*   630 */    58,  397,  395,  627,  840,  179,  628,   60,  405,  629,
 /*   640 */   630,  840,  840,   62,  408,  840,   61,  409,  407,  631,
 /*   650 */   840,  164,  632,   63,  840,  633,  634,  419,  840,  840,
 /*   660 */   481,  635,  840,  637,   65,  423,  840,  421,  422,   66,
 /*   670 */   840,   81,  639,   67,  640,  840,  180,   68,   69,  840,
 /*   680 */   840,  432,  641,  642,  840,  435,   70,  840,  840,  434,
 /*   690 */   436,  840,  643,  181,  644,   71,  517,  840,  645,  444,
 /*   700 */   840,  646,  840,  183,   72,  447,  840,  446,  647,  448,
 /*   710 */    73,  840,  674,  648,  165,  138,  840,  649,  166,  143,
 /*   720 */   840,  650,  458,   84,  840,  459,  462,  654,  840,  463,
 /*   730 */   144,  655,  840,  466,  145,  467,  840,  840,  840,  656,
 /*   740 */   472,  840,  840,  657,  840,  658,  474,  475,  529,  840,
 /*   750 */    74,  840,  659,   85,  660,  167,  665,  840,  661,  483,
 /*   760 */   152,  840,  484,  487,  153,  840,  488,  154,  840,  666,
 /*   770 */   491,  840,  492,  840,  497,  667,  840,  168,  840,  668,
 /*   780 */   840,  669,  499,  670,   76,  500,  840,  840,  502,   77,
 /*   790 */   503,  840,  505,  840,   78,   79,  506,  840,  840,  182,
 /*   800 */    80,  513,  840,  671,  840,  840,   82,  516,  840,  840,
 /*   810 */   840,  515,  672,  840,  840,  673,   83,  840,  840,  527,
 /*   820 */   525,  675,  840,  528,  840,  840,  840,  840,  676,  677,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */    28,   29,   30,   31,   32,   33,   34,   35,   36,   37,
 /*    10 */    38,   39,   40,   41,   42,   43,   44,    5,   75,   45,
 /*    20 */    77,   78,   79,   49,   81,   82,    0,   59,    2,   61,
 /*    30 */    18,   19,   20,   21,   22,   23,   75,   46,   77,   78,
 /*    40 */    75,   50,   77,   78,    9,   79,    6,   81,   82,   23,
 /*    50 */    79,   11,   81,   82,    2,    7,    7,    9,    9,   24,
 /*    60 */    12,   12,    2,    7,    7,    9,    9,    9,   12,   12,
 /*    70 */    12,    7,    7,    9,    9,    5,   12,   12,    5,   50,
 /*    80 */     5,    2,   24,   16,   17,   26,   27,   50,   18,   19,
 /*    90 */    10,   18,   19,   18,   19,   51,   52,   46,   45,   45,
 /*   100 */    10,   50,   49,   49,   49,   55,   56,   45,   47,   48,
 /*   110 */    15,   49,   46,   45,   45,   10,   50,   49,   49,   45,
 /*   120 */     2,   10,   46,   49,   47,   48,   50,   55,   56,   45,
 /*   130 */    10,   46,   10,   55,   56,   50,   55,   56,   55,   56,
 /*   140 */    46,   55,   56,   10,   50,   45,   57,   58,   46,   49,
 /*   150 */    57,   58,   50,   47,   48,   57,   58,    6,   46,   46,
 /*   160 */    57,   58,   50,   50,   57,   58,   57,   58,    7,   45,
 /*   170 */     9,   63,   64,   49,   46,   50,   47,   48,   50,   10,
 /*   180 */    63,   64,   63,   64,   46,   10,   63,   64,   50,   63,
 /*   190 */    64,   46,   65,   66,   10,   50,   46,   63,   64,   46,
 /*   200 */    50,   55,   56,   50,   65,   66,   47,   48,   65,   66,
 /*   210 */    69,   70,   65,   66,   65,   66,   46,   65,   66,   46,
 /*   220 */    50,   57,   58,   50,   69,   70,   47,   48,   46,   69,
 /*   230 */    70,    7,   50,    9,   69,   70,   69,   70,   46,   73,
 /*   240 */    74,   50,   50,   69,   70,   46,   45,   57,   58,   50,
 /*   250 */    49,   55,   56,   47,   48,   46,   73,   74,    9,   50,
 /*   260 */    73,   74,   73,   74,   46,   50,   73,   74,   50,    7,
 /*   270 */     7,    9,    9,   24,    7,    7,    9,    9,    7,    7,
 /*   280 */     9,    9,    7,    7,    9,    9,    7,    7,    9,    9,
 /*   290 */    50,   50,   50,   45,   10,   50,   53,   53,    6,    6,
 /*   300 */    50,   54,   49,   45,   10,   54,   60,   53,    6,   45,
 /*   310 */    45,   59,    6,   10,   45,   60,   55,   10,   62,   49,
 /*   320 */    61,    6,   49,   49,   67,   83,   49,   67,   62,   10,
 /*   330 */    45,    6,   54,   10,   53,    6,   54,   49,   10,    6,
 /*   340 */    10,    6,   56,    1,   10,    4,    3,   57,    4,   11,
 /*   350 */    59,   68,    6,    8,   45,   68,   58,    8,   61,    6,
 /*   360 */    71,    4,   10,    4,    6,   55,   71,   11,   72,   57,
 /*   370 */    49,   72,    7,   73,    8,    8,   56,   76,    6,    6,
 /*   380 */    76,   76,   58,    6,   76,    4,    6,   76,    4,   76,
 /*   390 */     8,    8,    4,    6,   80,   10,   80,   80,   80,    7,
 /*   400 */    11,    4,   80,    8,    8,   74,    6,   80,    4,    8,
 /*   410 */     8,    4,    6,   10,    4,    7,   11,    8,    3,    8,
 /*   420 */     4,    2,    4,    6,   10,   13,    4,   14,    3,    8,
 /*   430 */    83,    8,    8,   13,    8,    8,    6,    4,   83,   11,
 /*   440 */     8,    6,    6,    6,    6,   11,    6,    4,   83,    4,
 /*   450 */     6,   83,   11,   11,    7,    4,    8,   11,    8,    6,
 /*   460 */     4,    4,    8,    8,    6,    4,   83,    7,    4,   10,
 /*   470 */     8,    8,    6,    8,    4,   83,    8,   83,    6,   10,
 /*   480 */     7,    4,   11,    8,   10,    4,    6,    8,   83,   83,
 /*   490 */     8,    8,    6,    6,    6,   11,   11,   83,   83,    6,
 /*   500 */     6,    6,   11,   11,    4,    4,   83,   11,    6,   83,
 /*   510 */     7,   83,    8,    8,    6,    4,   10,    4,   83,   10,
 /*   520 */     4,   24,    8,    8,    6,    4,    4,   83,   11,   83,
 /*   530 */     7,   11,    8,    8,    8,    6,    4,    4,    4,    8,
 /*   540 */    83,    6,   83,    7,    4,    8,    8,    8,    6,   83,
 /*   550 */    83,    4,    8,   83,   83,   83,   11,   11,   10,    6,
 /*   560 */    11,   83,    4,   11,    7,    4,   83,    8,    8,    6,
 /*   570 */     4,    7,   83,    8,    4,   83,    8,    6,    4,   10,
 /*   580 */     8,    4,   83,    8,    6,    4,   83,    8,    8,   83,
 /*   590 */    83,    6,   10,    7,    6,   11,    4,    8,    4,   83,
 /*   600 */    83,    8,   83,    8,    8,   83,    4,   83,   11,   11,
 /*   610 */    11,    6,   83,    6,   10,    4,    7,   83,    8,    4,
 /*   620 */     8,    4,    8,    4,    8,   83,   83,    4,    6,   83,
 /*   630 */    10,    7,   11,    8,   83,    6,    8,    4,    4,    8,
 /*   640 */     8,   83,   83,    4,    6,   83,   10,    7,   11,    8,
 /*   650 */    83,    6,    8,    4,   83,    8,    8,    4,   83,   83,
 /*   660 */     3,   11,   83,   11,   10,    7,   83,   11,    6,    4,
 /*   670 */    83,   10,    8,    4,    8,   83,    6,    4,   10,   83,
 /*   680 */    83,    4,    8,    8,   83,    6,    4,   83,   83,   11,
 /*   690 */     7,   83,    8,    6,    8,    4,   24,   83,    8,    4,
 /*   700 */    83,    8,   83,    6,   10,    6,   83,   11,    8,    7,
 /*   710 */     4,   83,    8,    8,    6,    4,   83,    8,    4,    4,
 /*   720 */    83,   11,    6,   10,   83,    7,    6,    8,   83,    7,
 /*   730 */     4,    8,   83,    6,    4,    7,   83,   83,   83,    8,
 /*   740 */     4,   83,   83,   11,   83,   11,   11,    6,   24,   83,
 /*   750 */    10,   83,    8,    4,    8,    4,    8,   83,   11,    6,
 /*   760 */     4,   83,    7,    6,    4,   83,    7,    4,   83,    8,
 /*   770 */     6,   83,    7,   83,    3,    8,   83,    4,   83,   11,
 /*   780 */    83,   11,    6,    8,    4,    7,   83,   83,    6,    4,
 /*   790 */     7,   83,    6,   83,    4,    4,    7,   83,   83,    6,
 /*   800 */     4,    4,   83,    8,   83,   83,    4,    6,   83,   83,
 /*   810 */    83,   11,    8,   83,   83,    8,    4,   83,   83,   11,
 /*   820 */     4,    8,   83,    6,   83,   83,   83,   83,    8,    8,
};
#define YY_SHIFT_USE_DFLT (-1)
#define YY_SHIFT_MAX 531
static const short yy_shift_ofst[] = {
 /*     0 */    -1,   26,   52,   60,   60,   79,   79,   67,   90,   95,
 /*    10 */    90,  118,   90,   95,   90,   90,   95,   90,  111,   90,
 /*    20 */    95,  111,  118,  111,   95,  111,  111,   95,  111,  120,
 /*    30 */   122,   90,   95,  120,  118,  120,   95,  120,  120,   95,
 /*    40 */   120,  133,   90,   95,  133,  118,  133,   95,  133,  133,
 /*    50 */    95,  133,  169,  111,   95,  169,  118,  169,   95,  169,
 /*    60 */   169,   95,  169,  175,  120,   95,  175,  118,  175,   95,
 /*    70 */   175,  175,   95,  175,   95,  184,   90,  111,  120,  118,
 /*    80 */   184,   95,  184,  184,   95,  184,   95,   95,   95,   95,
 /*    90 */    95,   95,   80,   95,   95,  151,   80,  151,  284,   95,
 /*   100 */    95,  292,  105,  292,  293,  294,   80,   80,   80,   80,
 /*   110 */   151,  293,  302,  303,  105,  105,  105,  105,  292,  302,
 /*   120 */   151,  292,  306,  307,   80,  306,  315,  319,  105,  315,
 /*   130 */   325,  323,  294,  325,  329,  328,  303,  329,  330,  333,
 /*   140 */   333,  333,  333,   80,  307,  323,  333,  333,  335,  335,
 /*   150 */   335,  335,  105,  319,  328,  335,  335,  334,   12,   48,
 /*   160 */    49,   56,   57,   64,   65,   58,   70,   73,   75,   40,
 /*   170 */   161,  224,  262,  263,  267,  268,  271,  272,  275,  276,
 /*   180 */   279,  280,   35,  249,  342,  343,  341,  346,  344,  345,
 /*   190 */   349,  353,  338,  357,  352,  356,  358,  365,  359,  366,
 /*   200 */   367,  372,  373,  377,  381,  380,  384,  382,  383,  388,
 /*   210 */   385,  389,  387,  392,  397,  395,  396,  400,  404,  401,
 /*   220 */   402,  407,  403,  405,  406,  408,  410,  409,  411,  415,
 /*   230 */   416,  412,  417,  413,  418,  419,  420,  422,  421,  423,
 /*   240 */   424,  426,  430,  433,  427,  432,  435,  436,  428,  434,
 /*   250 */   437,  438,  440,  441,  443,  414,  442,  444,  447,  445,
 /*   260 */   448,  450,  451,  453,  456,  454,  455,  457,  459,  446,
 /*   270 */   458,  460,  461,  462,  463,  466,  464,  465,  468,  470,
 /*   280 */   469,  471,  472,  473,  477,  475,  479,  480,  481,  482,
 /*   290 */   483,  484,  486,  487,  488,  491,  492,  493,  494,  495,
 /*   300 */   496,  500,  474,  485,  502,  503,  501,  504,  505,  511,
 /*   310 */   508,  513,  514,  515,  516,  506,  517,  518,  523,  521,
 /*   320 */   524,  525,  529,  533,  526,  531,  534,  509,  520,  535,
 /*   330 */   536,  532,  537,  538,  542,  540,  539,  544,  545,  546,
 /*   340 */   547,  548,  549,  553,  557,  558,  559,  560,  561,  563,
 /*   350 */   566,  565,  568,  570,  569,  552,  571,  564,  574,  572,
 /*   360 */   575,  578,  577,  579,  580,  581,  582,  584,  585,  586,
 /*   370 */   592,  589,  593,  588,  594,  595,  596,  597,  598,  602,
 /*   380 */   604,  599,  605,  609,  611,  610,  612,  615,  607,  617,
 /*   390 */   614,  616,  619,  620,  621,  622,  624,  623,  625,  628,
 /*   400 */   629,  633,  631,  632,  634,  636,  637,  638,  640,  639,
 /*   410 */   641,  644,  645,  649,  647,  648,  650,  652,  653,  654,
 /*   420 */   656,  662,  658,  665,  664,  666,  669,  670,  673,  674,
 /*   430 */   675,  677,  668,  678,  679,  683,  682,  684,  686,  687,
 /*   440 */   691,  690,  693,  695,  694,  696,  699,  702,  706,  700,
 /*   450 */   705,  708,  711,  709,  710,  425,  714,  716,  718,  715,
 /*   460 */   719,  720,  722,  726,  723,  727,  728,  730,  731,  732,
 /*   470 */   734,  736,  740,  735,  741,  497,  522,  744,  746,  747,
 /*   480 */   657,  751,  753,  755,  756,  748,  757,  759,  760,  761,
 /*   490 */   764,  765,  763,  767,  768,  770,  771,  773,  776,  778,
 /*   500 */   780,  782,  783,  785,  786,  789,  790,  791,  793,  796,
 /*   510 */   775,  795,  797,  661,  800,  801,  672,  802,  804,  807,
 /*   520 */   697,  812,  704,  813,  816,  713,  808,  817,  724,  749,
 /*   530 */   820,  821,
};
#define YY_REDUCE_USE_DFLT (-58)
#define YY_REDUCE_MAX 157
static const short yy_reduce_ofst[] = {
 /*     0 */    59,  -28,  -57,  -39,  -35,  -34,  -29,   44,  -26,   -9,
 /*    10 */    53,   61,   54,   51,   62,   68,   66,   69,   50,   74,
 /*    20 */    76,   72,   77,   78,   85,   81,   83,   94,   86,   89,
 /*    30 */   -32,  100,  102,   93,  106,   98,  112,  103,  107,  113,
 /*    40 */   109,  108,  124,  128,  117,  129,  119,  138,  123,  126,
 /*    50 */   145,  134,  127,  146,  150,  139,  159,  143,  153,  147,
 /*    60 */   149,  170,  152,  141,  164,  173,  155,  179,  160,  182,
 /*    70 */   165,  167,  192,  174,  199,  166,  201,  196,  190,  206,
 /*    80 */   183,  209,  187,  189,  218,  193,   29,   37,  125,  191,
 /*    90 */   215,  240,   84,  241,  242,  243,  248,  244,   55,  245,
 /*   100 */   250,  247,  253,  251,  246,  252,  258,  264,  265,  269,
 /*   110 */   254,  255,  256,  259,  270,  273,  274,  277,  278,  266,
 /*   120 */   281,  282,  257,  261,  285,  260,  283,  286,  288,  287,
 /*   130 */   289,  290,  291,  295,  296,  298,  297,  299,  300,  301,
 /*   140 */   304,  305,  308,  309,  310,  312,  311,  313,  314,  316,
 /*   150 */   317,  318,  321,  320,  324,  322,  327,  331,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */   680,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    10 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    20 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    30 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    40 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    50 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    60 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    70 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    80 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*    90 */   838,  838,  838,  838,  838,  718,  838,  718,  838,  838,
 /*   100 */   838,  720,  838,  720,  751,  838,  838,  838,  838,  838,
 /*   110 */   718,  751,  754,  838,  838,  838,  838,  838,  720,  754,
 /*   120 */   718,  720,  783,  838,  838,  783,  786,  838,  838,  786,
 /*   130 */   801,  838,  838,  801,  804,  838,  838,  804,  838,  821,
 /*   140 */   821,  821,  821,  838,  838,  838,  821,  821,  828,  828,
 /*   150 */   828,  828,  838,  838,  838,  828,  828,  838,  838,  838,
 /*   160 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   170 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   180 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   190 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   200 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   210 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   220 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   230 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   240 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   250 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   260 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   270 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   280 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   290 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   300 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   310 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   320 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   330 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   340 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   350 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   360 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   370 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   380 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   390 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   400 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   410 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   420 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   430 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   440 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   450 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   460 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   470 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   480 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   490 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   500 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   510 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   520 */   838,  838,  838,  838,  838,  838,  838,  838,  838,  838,
 /*   530 */   838,  838,  681,  682,  683,  684,  685,  686,  687,  688,
 /*   540 */   689,  690,  691,  692,  693,  694,  695,  696,  697,  698,
 /*   550 */   704,  713,  714,  715,  699,  705,  710,  700,  706,  702,
 /*   560 */   708,  701,  707,  703,  709,  711,  712,  716,  717,  722,
 /*   570 */   728,  719,  734,  721,  735,  723,  729,  724,  730,  726,
 /*   580 */   732,  725,  731,  727,  733,  736,  742,  748,  750,  752,
 /*   590 */   749,  753,  755,  737,  743,  738,  744,  740,  746,  739,
 /*   600 */   745,  741,  747,  756,  762,  768,  769,  757,  763,  758,
 /*   610 */   764,  760,  766,  759,  765,  761,  767,  770,  776,  782,
 /*   620 */   784,  785,  787,  771,  777,  772,  778,  774,  780,  773,
 /*   630 */   779,  775,  781,  788,  794,  800,  802,  803,  805,  789,
 /*   640 */   795,  790,  796,  792,  798,  791,  797,  793,  799,  806,
 /*   650 */   818,  822,  823,  824,  832,  834,  836,  819,  820,  807,
 /*   660 */   813,  825,  829,  830,  831,  833,  835,  837,  826,  827,
 /*   670 */   808,  814,  810,  816,  809,  815,  811,  817,  812,
};
#define YY_SZ_ACTTAB (int)(sizeof(yy_action)/sizeof(yy_action[0]))

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  int yyidx;                    /* Index of top element in stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyidxMax;                 /* Maximum value of yyidx */
#endif
  int yyerrcnt;                 /* Shifts left before out of the error */
  ParseARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
/* #include <stdio.h> */
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void ParseTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "GEOJSON_NEWLINE",  "GEOJSON_OPEN_BRACE",  "GEOJSON_TYPE",
  "GEOJSON_COLON",  "GEOJSON_POINT",  "GEOJSON_COMMA",  "GEOJSON_COORDS",
  "GEOJSON_CLOSE_BRACE",  "GEOJSON_BBOX",  "GEOJSON_OPEN_BRACKET",  "GEOJSON_CLOSE_BRACKET",
  "GEOJSON_CRS",   "GEOJSON_NAME",  "GEOJSON_PROPS",  "GEOJSON_NUM", 
  "GEOJSON_SHORT_SRID",  "GEOJSON_LONG_SRID",  "GEOJSON_LINESTRING",  "GEOJSON_POLYGON",
  "GEOJSON_MULTIPOINT",  "GEOJSON_MULTILINESTRING",  "GEOJSON_MULTIPOLYGON",  "GEOJSON_GEOMETRYCOLLECTION",
  "GEOJSON_GEOMS",  "error",         "main",          "in",          
  "state",         "program",       "geo_text",      "point",       
  "pointz",        "linestring",    "linestringz",   "polygon",     
  "polygonz",      "multipoint",    "multipointz",   "multilinestring",
  "multilinestringz",  "multipolygon",  "multipolygonz",  "geocoll",     
  "geocollz",      "point_coordxy",  "bbox",          "short_crs",   
  "long_crs",      "point_coordxyz",  "coord",         "short_srid",  
  "long_srid",     "extra_pointsxy",  "extra_pointsxyz",  "linestring_text",
  "linestring_textz",  "polygon_text",  "polygon_textz",  "ring",        
  "extra_rings",   "ringz",         "extra_ringsz",  "multipoint_text",
  "multipoint_textz",  "multilinestring_text",  "multilinestring_textz",  "multilinestring_text2",
  "multilinestring_textz2",  "multipolygon_text",  "multipolygon_textz",  "multipolygon_text2",
  "multipolygon_textz2",  "geocoll_text",  "geocoll_textz",  "coll_point",  
  "geocoll_text2",  "coll_linestring",  "coll_polygon",  "coll_pointz", 
  "geocoll_textz2",  "coll_linestringz",  "coll_polygonz",
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "main ::= in",
 /*   1 */ "in ::=",
 /*   2 */ "in ::= in state GEOJSON_NEWLINE",
 /*   3 */ "state ::= program",
 /*   4 */ "program ::= geo_text",
 /*   5 */ "geo_text ::= point",
 /*   6 */ "geo_text ::= pointz",
 /*   7 */ "geo_text ::= linestring",
 /*   8 */ "geo_text ::= linestringz",
 /*   9 */ "geo_text ::= polygon",
 /*  10 */ "geo_text ::= polygonz",
 /*  11 */ "geo_text ::= multipoint",
 /*  12 */ "geo_text ::= multipointz",
 /*  13 */ "geo_text ::= multilinestring",
 /*  14 */ "geo_text ::= multilinestringz",
 /*  15 */ "geo_text ::= multipolygon",
 /*  16 */ "geo_text ::= multipolygonz",
 /*  17 */ "geo_text ::= geocoll",
 /*  18 */ "geo_text ::= geocollz",
 /*  19 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE",
 /*  20 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE",
 /*  21 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE",
 /*  22 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE",
 /*  23 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE",
 /*  24 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE",
 /*  25 */ "pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE",
 /*  26 */ "pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE",
 /*  27 */ "pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE",
 /*  28 */ "pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE",
 /*  29 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE",
 /*  30 */ "point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE",
 /*  31 */ "bbox ::= coord GEOJSON_COMMA coord GEOJSON_COMMA coord GEOJSON_COMMA coord",
 /*  32 */ "short_crs ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_NAME GEOJSON_COMMA GEOJSON_PROPS GEOJSON_COLON GEOJSON_OPEN_BRACE GEOJSON_NAME GEOJSON_COLON short_srid GEOJSON_CLOSE_BRACE GEOJSON_CLOSE_BRACE",
 /*  33 */ "long_crs ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_NAME GEOJSON_COMMA GEOJSON_PROPS GEOJSON_COLON GEOJSON_OPEN_BRACE GEOJSON_NAME GEOJSON_COLON long_srid GEOJSON_CLOSE_BRACE GEOJSON_CLOSE_BRACE",
 /*  34 */ "point_coordxy ::= GEOJSON_OPEN_BRACKET coord GEOJSON_COMMA coord GEOJSON_CLOSE_BRACKET",
 /*  35 */ "point_coordxyz ::= GEOJSON_OPEN_BRACKET coord GEOJSON_COMMA coord GEOJSON_COMMA coord GEOJSON_CLOSE_BRACKET",
 /*  36 */ "coord ::= GEOJSON_NUM",
 /*  37 */ "short_srid ::= GEOJSON_SHORT_SRID",
 /*  38 */ "long_srid ::= GEOJSON_LONG_SRID",
 /*  39 */ "extra_pointsxy ::=",
 /*  40 */ "extra_pointsxy ::= GEOJSON_COMMA point_coordxy extra_pointsxy",
 /*  41 */ "extra_pointsxyz ::=",
 /*  42 */ "extra_pointsxyz ::= GEOJSON_COMMA point_coordxyz extra_pointsxyz",
 /*  43 */ "linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE",
 /*  44 */ "linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE",
 /*  45 */ "linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE",
 /*  46 */ "linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE",
 /*  47 */ "linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE",
 /*  48 */ "linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE",
 /*  49 */ "linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE",
 /*  50 */ "linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE",
 /*  51 */ "linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE",
 /*  52 */ "linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE",
 /*  53 */ "linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE",
 /*  54 */ "linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE",
 /*  55 */ "linestring_text ::= GEOJSON_OPEN_BRACKET point_coordxy GEOJSON_COMMA point_coordxy extra_pointsxy GEOJSON_CLOSE_BRACKET",
 /*  56 */ "linestring_textz ::= GEOJSON_OPEN_BRACKET point_coordxyz GEOJSON_COMMA point_coordxyz extra_pointsxyz GEOJSON_CLOSE_BRACKET",
 /*  57 */ "polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE",
 /*  58 */ "polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE",
 /*  59 */ "polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE",
 /*  60 */ "polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE",
 /*  61 */ "polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE",
 /*  62 */ "polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE",
 /*  63 */ "polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE",
 /*  64 */ "polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE",
 /*  65 */ "polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE",
 /*  66 */ "polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE",
 /*  67 */ "polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE",
 /*  68 */ "polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE",
 /*  69 */ "polygon_text ::= GEOJSON_OPEN_BRACKET ring extra_rings GEOJSON_CLOSE_BRACKET",
 /*  70 */ "polygon_textz ::= GEOJSON_OPEN_BRACKET ringz extra_ringsz GEOJSON_CLOSE_BRACKET",
 /*  71 */ "ring ::= GEOJSON_OPEN_BRACKET point_coordxy GEOJSON_COMMA point_coordxy GEOJSON_COMMA point_coordxy GEOJSON_COMMA point_coordxy extra_pointsxy GEOJSON_CLOSE_BRACKET",
 /*  72 */ "extra_rings ::=",
 /*  73 */ "extra_rings ::= GEOJSON_COMMA ring extra_rings",
 /*  74 */ "ringz ::= GEOJSON_OPEN_BRACKET point_coordxyz GEOJSON_COMMA point_coordxyz GEOJSON_COMMA point_coordxyz GEOJSON_COMMA point_coordxyz extra_pointsxyz GEOJSON_CLOSE_BRACKET",
 /*  75 */ "extra_ringsz ::=",
 /*  76 */ "extra_ringsz ::= GEOJSON_COMMA ringz extra_ringsz",
 /*  77 */ "multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE",
 /*  78 */ "multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE",
 /*  79 */ "multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE",
 /*  80 */ "multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE",
 /*  81 */ "multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE",
 /*  82 */ "multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE",
 /*  83 */ "multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE",
 /*  84 */ "multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE",
 /*  85 */ "multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE",
 /*  86 */ "multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE",
 /*  87 */ "multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE",
 /*  88 */ "multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE",
 /*  89 */ "multipoint_text ::= GEOJSON_OPEN_BRACKET point_coordxy extra_pointsxy GEOJSON_CLOSE_BRACKET",
 /*  90 */ "multipoint_textz ::= GEOJSON_OPEN_BRACKET point_coordxyz extra_pointsxyz GEOJSON_CLOSE_BRACKET",
 /*  91 */ "multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE",
 /*  92 */ "multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE",
 /*  93 */ "multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE",
 /*  94 */ "multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE",
 /*  95 */ "multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE",
 /*  96 */ "multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE",
 /*  97 */ "multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE",
 /*  98 */ "multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE",
 /*  99 */ "multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE",
 /* 100 */ "multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE",
 /* 101 */ "multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE",
 /* 102 */ "multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE",
 /* 103 */ "multilinestring_text ::= GEOJSON_OPEN_BRACKET linestring_text multilinestring_text2 GEOJSON_CLOSE_BRACKET",
 /* 104 */ "multilinestring_text2 ::=",
 /* 105 */ "multilinestring_text2 ::= GEOJSON_COMMA linestring_text multilinestring_text2",
 /* 106 */ "multilinestring_textz ::= GEOJSON_OPEN_BRACKET linestring_textz multilinestring_textz2 GEOJSON_CLOSE_BRACKET",
 /* 107 */ "multilinestring_textz2 ::=",
 /* 108 */ "multilinestring_textz2 ::= GEOJSON_COMMA linestring_textz multilinestring_textz2",
 /* 109 */ "multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE",
 /* 110 */ "multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE",
 /* 111 */ "multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE",
 /* 112 */ "multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE",
 /* 113 */ "multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE",
 /* 114 */ "multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE",
 /* 115 */ "multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE",
 /* 116 */ "multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE",
 /* 117 */ "multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE",
 /* 118 */ "multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE",
 /* 119 */ "multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE",
 /* 120 */ "multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE",
 /* 121 */ "multipolygon_text ::= GEOJSON_OPEN_BRACKET polygon_text multipolygon_text2 GEOJSON_CLOSE_BRACKET",
 /* 122 */ "multipolygon_text2 ::=",
 /* 123 */ "multipolygon_text2 ::= GEOJSON_COMMA polygon_text multipolygon_text2",
 /* 124 */ "multipolygon_textz ::= GEOJSON_OPEN_BRACKET polygon_textz multipolygon_textz2 GEOJSON_CLOSE_BRACKET",
 /* 125 */ "multipolygon_textz2 ::=",
 /* 126 */ "multipolygon_textz2 ::= GEOJSON_COMMA polygon_textz multipolygon_textz2",
 /* 127 */ "geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE",
 /* 128 */ "geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE",
 /* 129 */ "geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE",
 /* 130 */ "geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE",
 /* 131 */ "geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE",
 /* 132 */ "geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE",
 /* 133 */ "geocollz ::= GEOJSON_GEOMETRYCOLLECTION geocoll_textz",
 /* 134 */ "geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE",
 /* 135 */ "geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE",
 /* 136 */ "geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE",
 /* 137 */ "geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE",
 /* 138 */ "geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE",
 /* 139 */ "geocoll_text ::= GEOJSON_OPEN_BRACKET coll_point geocoll_text2 GEOJSON_CLOSE_BRACKET",
 /* 140 */ "geocoll_text ::= GEOJSON_OPEN_BRACKET coll_linestring geocoll_text2 GEOJSON_CLOSE_BRACKET",
 /* 141 */ "geocoll_text ::= GEOJSON_OPEN_BRACKET coll_polygon geocoll_text2 GEOJSON_CLOSE_BRACKET",
 /* 142 */ "geocoll_text2 ::=",
 /* 143 */ "geocoll_text2 ::= GEOJSON_COMMA coll_point geocoll_text2",
 /* 144 */ "geocoll_text2 ::= GEOJSON_COMMA coll_linestring geocoll_text2",
 /* 145 */ "geocoll_text2 ::= GEOJSON_COMMA coll_polygon geocoll_text2",
 /* 146 */ "geocoll_textz ::= GEOJSON_OPEN_BRACKET coll_pointz geocoll_textz2 GEOJSON_CLOSE_BRACKET",
 /* 147 */ "geocoll_textz ::= GEOJSON_OPEN_BRACKET coll_linestringz geocoll_textz2 GEOJSON_CLOSE_BRACKET",
 /* 148 */ "geocoll_textz ::= GEOJSON_OPEN_BRACKET coll_polygonz geocoll_textz2 GEOJSON_CLOSE_BRACKET",
 /* 149 */ "geocoll_textz2 ::=",
 /* 150 */ "geocoll_textz2 ::= GEOJSON_COMMA coll_pointz geocoll_textz2",
 /* 151 */ "geocoll_textz2 ::= GEOJSON_COMMA coll_linestringz geocoll_textz2",
 /* 152 */ "geocoll_textz2 ::= GEOJSON_COMMA coll_polygonz geocoll_textz2",
 /* 153 */ "coll_point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE",
 /* 154 */ "coll_pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE",
 /* 155 */ "coll_linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE",
 /* 156 */ "coll_linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE",
 /* 157 */ "coll_polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE",
 /* 158 */ "coll_polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.
*/
static void yyGrowStack(yyParser *p){
  int newSize;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  if( pNew ){
    p->yystack = pNew;
    p->yystksz = newSize;
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows to %d entries!\n",
              yyTracePrompt, p->yystksz);
    }
#endif
  }
}
#endif

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Parse and ParseFree.
*/
void *ParseAlloc(void *(*mallocProc)(size_t)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (size_t)sizeof(yyParser) );
  if( pParser ){
    pParser->yyidx = -1;
#ifdef YYTRACKMAXSTACKDEPTH
    pParser->yyidxMax = 0;
#endif
#if YYSTACKDEPTH<=0
    pParser->yystack = NULL;
    pParser->yystksz = 0;
    yyGrowStack(pParser);
#endif
  }
  return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  ParseARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are not used
    ** inside the C code.
    */
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(yyParser *pParser){
  YYCODETYPE yymajor;
  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

  if( pParser->yyidx<0 ) return 0;
#ifndef NDEBUG
  if( yyTraceFILE && pParser->yyidx>=0 ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yymajor = yytos->major;
  yy_destructor(pParser, yymajor, &yytos->minor);
  pParser->yyidx--;
  return yymajor;
}

/* 
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from ParseAlloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void ParseFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
  if( pParser==0 ) return;
  while( pParser->yyidx>=0 ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  free(pParser->yystack);
#endif
  (*freeProc)((void*)pParser);
}

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int ParseStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyidxMax;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yystack[pParser->yyidx].stateno;
 
  if( stateno>YY_SHIFT_MAX || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
    return yy_default[stateno];
  }
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    if( iLookAhead>0 ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        return yy_find_shift_action(pParser, iFallback);
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( j>=0 && j<YY_SZ_ACTTAB && yy_lookahead[j]==YYWILDCARD ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
    }
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_MAX ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_MAX );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_SZ_ACTTAB );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser, YYMINORTYPE *yypMinor){
   ParseARG_FETCH;
   yypParser->yyidx--;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */

     fprintf(stderr,"Giving up.  Parser stack overflow\n");
   ParseARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor         /* Pointer to the minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yyidx++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( yypParser->yyidx>yypParser->yyidxMax ){
    yypParser->yyidxMax = yypParser->yyidx;
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yyidx>=YYSTACKDEPTH ){
    yyStackOverflow(yypParser, yypMinor);
    return;
  }
#else
  if( yypParser->yyidx>=yypParser->yystksz ){
    yyGrowStack(yypParser);
    if( yypParser->yyidx>=yypParser->yystksz ){
      yyStackOverflow(yypParser, yypMinor);
      return;
    }
  }
#endif
  yytos = &yypParser->yystack[yypParser->yyidx];
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor = *yypMinor;
#ifndef NDEBUG
  if( yyTraceFILE && yypParser->yyidx>0 ){
    int i;
    fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
    fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
    for(i=1; i<=yypParser->yyidx; i++)
      fprintf(yyTraceFILE," %s",yyTokenName[yypParser->yystack[i].major]);
    fprintf(yyTraceFILE,"\n");
  }
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 26, 1 },
  { 27, 0 },
  { 27, 3 },
  { 28, 1 },
  { 29, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 30, 1 },
  { 31, 9 },
  { 31, 15 },
  { 31, 13 },
  { 31, 13 },
  { 31, 19 },
  { 31, 19 },
  { 32, 9 },
  { 32, 15 },
  { 32, 13 },
  { 32, 13 },
  { 31, 19 },
  { 31, 19 },
  { 46, 7 },
  { 47, 13 },
  { 48, 13 },
  { 45, 5 },
  { 49, 7 },
  { 50, 1 },
  { 51, 1 },
  { 52, 1 },
  { 53, 0 },
  { 53, 3 },
  { 54, 0 },
  { 54, 3 },
  { 33, 9 },
  { 33, 15 },
  { 33, 13 },
  { 33, 13 },
  { 33, 19 },
  { 33, 19 },
  { 34, 9 },
  { 34, 15 },
  { 34, 13 },
  { 34, 13 },
  { 34, 19 },
  { 34, 19 },
  { 55, 6 },
  { 56, 6 },
  { 35, 9 },
  { 35, 15 },
  { 35, 13 },
  { 35, 13 },
  { 35, 19 },
  { 35, 19 },
  { 36, 9 },
  { 36, 15 },
  { 36, 13 },
  { 36, 13 },
  { 36, 19 },
  { 36, 19 },
  { 57, 4 },
  { 58, 4 },
  { 59, 10 },
  { 60, 0 },
  { 60, 3 },
  { 61, 10 },
  { 62, 0 },
  { 62, 3 },
  { 37, 9 },
  { 37, 15 },
  { 37, 13 },
  { 37, 13 },
  { 37, 19 },
  { 37, 19 },
  { 38, 9 },
  { 38, 15 },
  { 38, 13 },
  { 38, 13 },
  { 38, 19 },
  { 38, 19 },
  { 63, 4 },
  { 64, 4 },
  { 39, 9 },
  { 39, 15 },
  { 39, 13 },
  { 39, 13 },
  { 39, 19 },
  { 39, 19 },
  { 40, 9 },
  { 40, 15 },
  { 40, 13 },
  { 40, 13 },
  { 40, 19 },
  { 40, 19 },
  { 65, 4 },
  { 67, 0 },
  { 67, 3 },
  { 66, 4 },
  { 68, 0 },
  { 68, 3 },
  { 41, 9 },
  { 41, 15 },
  { 41, 13 },
  { 41, 13 },
  { 41, 19 },
  { 41, 19 },
  { 42, 9 },
  { 42, 15 },
  { 42, 13 },
  { 42, 13 },
  { 42, 19 },
  { 42, 19 },
  { 69, 4 },
  { 71, 0 },
  { 71, 3 },
  { 70, 4 },
  { 72, 0 },
  { 72, 3 },
  { 43, 9 },
  { 43, 15 },
  { 43, 13 },
  { 43, 13 },
  { 43, 19 },
  { 43, 19 },
  { 44, 2 },
  { 44, 15 },
  { 44, 13 },
  { 44, 13 },
  { 44, 19 },
  { 44, 19 },
  { 73, 4 },
  { 73, 4 },
  { 73, 4 },
  { 76, 0 },
  { 76, 3 },
  { 76, 3 },
  { 76, 3 },
  { 74, 4 },
  { 74, 4 },
  { 74, 4 },
  { 80, 0 },
  { 80, 3 },
  { 80, 3 },
  { 80, 3 },
  { 75, 9 },
  { 79, 9 },
  { 77, 9 },
  { 81, 9 },
  { 78, 9 },
  { 82, 9 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  int yyruleno                 /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  ParseARG_FETCH;
  yymsp = &yypParser->yystack[yypParser->yyidx];
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno>=0 
        && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
      yyRuleName[yyruleno]);
  }
#endif /* NDEBUG */

  /* Silence complaints from purify about yygotominor being uninitialized
  ** in some cases when it is copied into the stack after the following
  ** switch.  yygotominor is uninitialized when a rule reduces that does
  ** not set the value of its left-hand side nonterminal.  Leaving the
  ** value of the nonterminal uninitialized is utterly harmless as long
  ** as the value is never used.  So really the only thing this code
  ** accomplishes is to quieten purify.  
  **
  ** 2007-01-16:  The wireshark project (www.wireshark.org) reports that
  ** without this code, their parser segfaults.  I'm not sure what there
  ** parser is doing to make this happen.  This is the second bug report
  ** from wireshark this week.  Clearly they are stressing Lemon in ways
  ** that it has not been previously stressed...  (SQLite ticket #2172)
  */
  /*memset(&yygotominor, 0, sizeof(yygotominor));*/
  yygotominor = yyzerominor;


  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
      case 5: /* geo_text ::= point */
      case 6: /* geo_text ::= pointz */ yytestcase(yyruleno==6);
      case 7: /* geo_text ::= linestring */ yytestcase(yyruleno==7);
      case 8: /* geo_text ::= linestringz */ yytestcase(yyruleno==8);
      case 9: /* geo_text ::= polygon */ yytestcase(yyruleno==9);
      case 10: /* geo_text ::= polygonz */ yytestcase(yyruleno==10);
      case 11: /* geo_text ::= multipoint */ yytestcase(yyruleno==11);
      case 12: /* geo_text ::= multipointz */ yytestcase(yyruleno==12);
      case 13: /* geo_text ::= multilinestring */ yytestcase(yyruleno==13);
      case 14: /* geo_text ::= multilinestringz */ yytestcase(yyruleno==14);
      case 15: /* geo_text ::= multipolygon */ yytestcase(yyruleno==15);
      case 16: /* geo_text ::= multipolygonz */ yytestcase(yyruleno==16);
      case 17: /* geo_text ::= geocoll */ yytestcase(yyruleno==17);
      case 18: /* geo_text ::= geocollz */ yytestcase(yyruleno==18);
{ *result = yymsp[0].minor.yy0; }
        break;
      case 19: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE */
      case 20: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==20);
      case 25: /* pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==25);
      case 26: /* pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==26);
      case 153: /* coll_point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==153);
      case 154: /* coll_pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==154);
{ yygotominor.yy0 = geoJSON_buildGeomFromPoint((gaiaPointPtr)yymsp[-1].minor.yy0); }
        break;
      case 21: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE */
      case 22: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==22);
      case 27: /* pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==27);
      case 28: /* pointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==28);
{ yygotominor.yy0 = geoJSON_buildGeomFromPointSrid((gaiaPointPtr)yymsp[-1].minor.yy0, (int *)yymsp[-5].minor.yy0); }
        break;
      case 23: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE */
      case 24: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxy GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==24);
      case 29: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==29);
      case 30: /* point ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON point_coordxyz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==30);
{ yygotominor.yy0 = geoJSON_buildGeomFromPointSrid((gaiaPointPtr)yymsp[-1].minor.yy0, (int *)yymsp[-11].minor.yy0); }
        break;
      case 32: /* short_crs ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_NAME GEOJSON_COMMA GEOJSON_PROPS GEOJSON_COLON GEOJSON_OPEN_BRACE GEOJSON_NAME GEOJSON_COLON short_srid GEOJSON_CLOSE_BRACE GEOJSON_CLOSE_BRACE */
      case 33: /* long_crs ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_NAME GEOJSON_COMMA GEOJSON_PROPS GEOJSON_COLON GEOJSON_OPEN_BRACE GEOJSON_NAME GEOJSON_COLON long_srid GEOJSON_CLOSE_BRACE GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==33);
{ yygotominor.yy0 = yymsp[-2].minor.yy0; }
        break;
      case 34: /* point_coordxy ::= GEOJSON_OPEN_BRACKET coord GEOJSON_COMMA coord GEOJSON_CLOSE_BRACKET */
{ yygotominor.yy0 = (void *) geoJSON_point_xy((double *)yymsp[-3].minor.yy0, (double *)yymsp[-1].minor.yy0); }
        break;
      case 35: /* point_coordxyz ::= GEOJSON_OPEN_BRACKET coord GEOJSON_COMMA coord GEOJSON_COMMA coord GEOJSON_CLOSE_BRACKET */
{ yygotominor.yy0 = (void *) geoJSON_point_xyz((double *)yymsp[-5].minor.yy0, (double *)yymsp[-3].minor.yy0, (double *)yymsp[-1].minor.yy0); }
        break;
      case 36: /* coord ::= GEOJSON_NUM */
      case 37: /* short_srid ::= GEOJSON_SHORT_SRID */ yytestcase(yyruleno==37);
      case 38: /* long_srid ::= GEOJSON_LONG_SRID */ yytestcase(yyruleno==38);
      case 133: /* geocollz ::= GEOJSON_GEOMETRYCOLLECTION geocoll_textz */ yytestcase(yyruleno==133);
{ yygotominor.yy0 = yymsp[0].minor.yy0; }
        break;
      case 39: /* extra_pointsxy ::= */
      case 41: /* extra_pointsxyz ::= */ yytestcase(yyruleno==41);
      case 72: /* extra_rings ::= */ yytestcase(yyruleno==72);
      case 75: /* extra_ringsz ::= */ yytestcase(yyruleno==75);
      case 104: /* multilinestring_text2 ::= */ yytestcase(yyruleno==104);
      case 107: /* multilinestring_textz2 ::= */ yytestcase(yyruleno==107);
      case 122: /* multipolygon_text2 ::= */ yytestcase(yyruleno==122);
      case 125: /* multipolygon_textz2 ::= */ yytestcase(yyruleno==125);
      case 142: /* geocoll_text2 ::= */ yytestcase(yyruleno==142);
      case 149: /* geocoll_textz2 ::= */ yytestcase(yyruleno==149);
{ yygotominor.yy0 = NULL; }
        break;
      case 40: /* extra_pointsxy ::= GEOJSON_COMMA point_coordxy extra_pointsxy */
      case 42: /* extra_pointsxyz ::= GEOJSON_COMMA point_coordxyz extra_pointsxyz */ yytestcase(yyruleno==42);
{ ((gaiaPointPtr)yymsp[-1].minor.yy0)->Next = (gaiaPointPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 43: /* linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE */
      case 44: /* linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==44);
      case 49: /* linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==49);
      case 50: /* linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==50);
      case 155: /* coll_linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==155);
      case 156: /* coll_linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==156);
{ yygotominor.yy0 = geoJSON_buildGeomFromLinestring((gaiaLinestringPtr)yymsp[-1].minor.yy0); }
        break;
      case 45: /* linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE */
      case 46: /* linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==46);
      case 51: /* linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==51);
      case 52: /* linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==52);
{ yygotominor.yy0 = geoJSON_buildGeomFromLinestringSrid((gaiaLinestringPtr)yymsp[-1].minor.yy0, (int *)yymsp[-5].minor.yy0); }
        break;
      case 47: /* linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE */
      case 48: /* linestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==48);
      case 53: /* linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==53);
      case 54: /* linestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_LINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON linestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==54);
{ yygotominor.yy0 = geoJSON_buildGeomFromLinestringSrid((gaiaLinestringPtr)yymsp[-1].minor.yy0, (int *)yymsp[-11].minor.yy0); }
        break;
      case 55: /* linestring_text ::= GEOJSON_OPEN_BRACKET point_coordxy GEOJSON_COMMA point_coordxy extra_pointsxy GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) geoJSON_linestring_xy((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
        break;
      case 56: /* linestring_textz ::= GEOJSON_OPEN_BRACKET point_coordxyz GEOJSON_COMMA point_coordxyz extra_pointsxyz GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   ((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0;
	   yygotominor.yy0 = (void *) geoJSON_linestring_xyz((gaiaPointPtr)yymsp[-4].minor.yy0);
	}
        break;
      case 57: /* polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE */
      case 58: /* polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==58);
      case 63: /* polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==63);
      case 64: /* polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==64);
      case 157: /* coll_polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==157);
      case 158: /* coll_polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==158);
{ yygotominor.yy0 = geoJSON_buildGeomFromPolygon((gaiaPolygonPtr)yymsp[-1].minor.yy0); }
        break;
      case 59: /* polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE */
      case 60: /* polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==60);
      case 65: /* polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==65);
      case 66: /* polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==66);
{ yygotominor.yy0 = geoJSON_buildGeomFromPolygonSrid((gaiaPolygonPtr)yymsp[-1].minor.yy0, (int *)yymsp[-5].minor.yy0); }
        break;
      case 61: /* polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE */
      case 62: /* polygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==62);
      case 67: /* polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==67);
      case 68: /* polygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_POLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON polygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==68);
{ yygotominor.yy0 = geoJSON_buildGeomFromPolygonSrid((gaiaPolygonPtr)yymsp[-1].minor.yy0, (int *)yymsp[-11].minor.yy0); }
        break;
      case 69: /* polygon_text ::= GEOJSON_OPEN_BRACKET ring extra_rings GEOJSON_CLOSE_BRACKET */
{ 
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) geoJSON_polygon_xy((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 70: /* polygon_textz ::= GEOJSON_OPEN_BRACKET ringz extra_ringsz GEOJSON_CLOSE_BRACKET */
{  
		((gaiaRingPtr)yymsp[-2].minor.yy0)->Next = (gaiaRingPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) geoJSON_polygon_xyz((gaiaRingPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 71: /* ring ::= GEOJSON_OPEN_BRACKET point_coordxy GEOJSON_COMMA point_coordxy GEOJSON_COMMA point_coordxy GEOJSON_COMMA point_coordxy extra_pointsxy GEOJSON_CLOSE_BRACKET */
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) geoJSON_ring_xy((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
        break;
      case 73: /* extra_rings ::= GEOJSON_COMMA ring extra_rings */
      case 76: /* extra_ringsz ::= GEOJSON_COMMA ringz extra_ringsz */ yytestcase(yyruleno==76);
{
		((gaiaRingPtr)yymsp[-1].minor.yy0)->Next = (gaiaRingPtr)yymsp[0].minor.yy0;
		yygotominor.yy0 = yymsp[-1].minor.yy0;
	}
        break;
      case 74: /* ringz ::= GEOJSON_OPEN_BRACKET point_coordxyz GEOJSON_COMMA point_coordxyz GEOJSON_COMMA point_coordxyz GEOJSON_COMMA point_coordxyz extra_pointsxyz GEOJSON_CLOSE_BRACKET */
{
		((gaiaPointPtr)yymsp[-8].minor.yy0)->Next = (gaiaPointPtr)yymsp[-6].minor.yy0; 
		((gaiaPointPtr)yymsp[-6].minor.yy0)->Next = (gaiaPointPtr)yymsp[-4].minor.yy0;
		((gaiaPointPtr)yymsp[-4].minor.yy0)->Next = (gaiaPointPtr)yymsp[-2].minor.yy0; 
		((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) geoJSON_ring_xyz((gaiaPointPtr)yymsp[-8].minor.yy0);
	}
        break;
      case 77: /* multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE */
      case 78: /* multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==78);
      case 83: /* multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==83);
      case 84: /* multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==84);
      case 91: /* multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==91);
      case 92: /* multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==92);
      case 97: /* multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==97);
      case 98: /* multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==98);
      case 109: /* multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==109);
      case 110: /* multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==110);
      case 115: /* multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==115);
      case 116: /* multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==116);
      case 127: /* geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==127);
      case 128: /* geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==128);
      case 134: /* geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==134);
{ yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 79: /* multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE */
      case 80: /* multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==80);
      case 85: /* multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==85);
      case 86: /* multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==86);
      case 93: /* multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==93);
      case 94: /* multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==94);
      case 99: /* multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==99);
      case 100: /* multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==100);
      case 111: /* multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==111);
      case 112: /* multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==112);
      case 117: /* multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==117);
      case 118: /* multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==118);
      case 129: /* geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==129);
      case 130: /* geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==130);
      case 135: /* geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==135);
      case 136: /* geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==136);
{ yygotominor.yy0 = (void *) geoJSON_setSrid((gaiaGeomCollPtr)yymsp[-1].minor.yy0, (int *)yymsp[-5].minor.yy0); }
        break;
      case 81: /* multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE */
      case 82: /* multipoint ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==82);
      case 87: /* multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==87);
      case 88: /* multipointz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOINT GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipoint_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==88);
      case 95: /* multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==95);
      case 96: /* multilinestring ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==96);
      case 101: /* multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==101);
      case 102: /* multilinestringz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTILINESTRING GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multilinestring_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==102);
      case 113: /* multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==113);
      case 114: /* multipolygon ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==114);
      case 119: /* multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==119);
      case 120: /* multipolygonz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_MULTIPOLYGON GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_COORDS GEOJSON_COLON multipolygon_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==120);
      case 131: /* geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==131);
      case 132: /* geocoll ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_text GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==132);
      case 137: /* geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON short_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==137);
      case 138: /* geocollz ::= GEOJSON_OPEN_BRACE GEOJSON_TYPE GEOJSON_COLON GEOJSON_GEOMETRYCOLLECTION GEOJSON_COMMA GEOJSON_CRS GEOJSON_COLON long_crs GEOJSON_COMMA GEOJSON_BBOX GEOJSON_COLON GEOJSON_OPEN_BRACKET bbox GEOJSON_CLOSE_BRACKET GEOJSON_COMMA GEOJSON_GEOMS GEOJSON_COLON geocoll_textz GEOJSON_CLOSE_BRACE */ yytestcase(yyruleno==138);
{ yygotominor.yy0 = (void *) geoJSON_setSrid((gaiaGeomCollPtr)yymsp[-1].minor.yy0, (int *)yymsp[-11].minor.yy0); }
        break;
      case 89: /* multipoint_text ::= GEOJSON_OPEN_BRACKET point_coordxy extra_pointsxy GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) geoJSON_multipoint_xy((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 90: /* multipoint_textz ::= GEOJSON_OPEN_BRACKET point_coordxyz extra_pointsxyz GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaPointPtr)yymsp[-2].minor.yy0)->Next = (gaiaPointPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) geoJSON_multipoint_xyz((gaiaPointPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 103: /* multilinestring_text ::= GEOJSON_OPEN_BRACKET linestring_text multilinestring_text2 GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) geoJSON_multilinestring_xy((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 105: /* multilinestring_text2 ::= GEOJSON_COMMA linestring_text multilinestring_text2 */
      case 108: /* multilinestring_textz2 ::= GEOJSON_COMMA linestring_textz multilinestring_textz2 */ yytestcase(yyruleno==108);
{ ((gaiaLinestringPtr)yymsp[-1].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 106: /* multilinestring_textz ::= GEOJSON_OPEN_BRACKET linestring_textz multilinestring_textz2 GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaLinestringPtr)yymsp[-2].minor.yy0)->Next = (gaiaLinestringPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) geoJSON_multilinestring_xyz((gaiaLinestringPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 121: /* multipolygon_text ::= GEOJSON_OPEN_BRACKET polygon_text multipolygon_text2 GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) geoJSON_multipolygon_xy((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 123: /* multipolygon_text2 ::= GEOJSON_COMMA polygon_text multipolygon_text2 */
      case 126: /* multipolygon_textz2 ::= GEOJSON_COMMA polygon_textz multipolygon_textz2 */ yytestcase(yyruleno==126);
{ ((gaiaPolygonPtr)yymsp[-1].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 124: /* multipolygon_textz ::= GEOJSON_OPEN_BRACKET polygon_textz multipolygon_textz2 GEOJSON_CLOSE_BRACKET */
{ 
	   ((gaiaPolygonPtr)yymsp[-2].minor.yy0)->Next = (gaiaPolygonPtr)yymsp[-1].minor.yy0; 
	   yygotominor.yy0 = (void *) geoJSON_multipolygon_xyz((gaiaPolygonPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 139: /* geocoll_text ::= GEOJSON_OPEN_BRACKET coll_point geocoll_text2 GEOJSON_CLOSE_BRACKET */
      case 140: /* geocoll_text ::= GEOJSON_OPEN_BRACKET coll_linestring geocoll_text2 GEOJSON_CLOSE_BRACKET */ yytestcase(yyruleno==140);
      case 141: /* geocoll_text ::= GEOJSON_OPEN_BRACKET coll_polygon geocoll_text2 GEOJSON_CLOSE_BRACKET */ yytestcase(yyruleno==141);
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) geoJSON_geomColl_xy((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
        break;
      case 143: /* geocoll_text2 ::= GEOJSON_COMMA coll_point geocoll_text2 */
      case 144: /* geocoll_text2 ::= GEOJSON_COMMA coll_linestring geocoll_text2 */ yytestcase(yyruleno==144);
      case 145: /* geocoll_text2 ::= GEOJSON_COMMA coll_polygon geocoll_text2 */ yytestcase(yyruleno==145);
      case 150: /* geocoll_textz2 ::= GEOJSON_COMMA coll_pointz geocoll_textz2 */ yytestcase(yyruleno==150);
      case 151: /* geocoll_textz2 ::= GEOJSON_COMMA coll_linestringz geocoll_textz2 */ yytestcase(yyruleno==151);
      case 152: /* geocoll_textz2 ::= GEOJSON_COMMA coll_polygonz geocoll_textz2 */ yytestcase(yyruleno==152);
{
		((gaiaGeomCollPtr)yymsp[-1].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[0].minor.yy0;
		yygotominor.yy0 = yymsp[-1].minor.yy0;
	}
        break;
      case 146: /* geocoll_textz ::= GEOJSON_OPEN_BRACKET coll_pointz geocoll_textz2 GEOJSON_CLOSE_BRACKET */
      case 147: /* geocoll_textz ::= GEOJSON_OPEN_BRACKET coll_linestringz geocoll_textz2 GEOJSON_CLOSE_BRACKET */ yytestcase(yyruleno==147);
      case 148: /* geocoll_textz ::= GEOJSON_OPEN_BRACKET coll_polygonz geocoll_textz2 GEOJSON_CLOSE_BRACKET */ yytestcase(yyruleno==148);
{ 
		((gaiaGeomCollPtr)yymsp[-2].minor.yy0)->Next = (gaiaGeomCollPtr)yymsp[-1].minor.yy0;
		yygotominor.yy0 = (void *) geoJSON_geomColl_xyz((gaiaGeomCollPtr)yymsp[-2].minor.yy0);
	}
        break;
      default:
      /* (0) main ::= in */ yytestcase(yyruleno==0);
      /* (1) in ::= */ yytestcase(yyruleno==1);
      /* (2) in ::= in state GEOJSON_NEWLINE */ yytestcase(yyruleno==2);
      /* (3) state ::= program */ yytestcase(yyruleno==3);
      /* (4) program ::= geo_text */ yytestcase(yyruleno==4);
      /* (31) bbox ::= coord GEOJSON_COMMA coord GEOJSON_COMMA coord GEOJSON_COMMA coord */ yytestcase(yyruleno==31);
        break;
  };
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yypParser->yyidx -= yysize;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact < YYNSTATE ){
#ifdef NDEBUG
    /* If we are not debugging and the reduce action popped at least
    ** one element off the stack, then we can push the new element back
    ** onto the stack here, and skip the stack overflow test in yy_shift().
    ** That gives a significant speed improvement. */
    if( yysize ){
      yypParser->yyidx++;
      yymsp -= yysize-1;
      yymsp->stateno = (YYACTIONTYPE)yyact;
      yymsp->major = (YYCODETYPE)yygoto;
      yymsp->minor = yygotominor;
    }else
#endif
    {
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
    }
  }else{
    assert( yyact == YYNSTATE + YYNRULE + 1 );
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  YYMINORTYPE yyminor            /* The minor type of the error token */
){
  ParseARG_FETCH;
#define TOKEN (yyminor.yy0)

/* 
** when the LEMON parser encounters an error
** then this global variable is set 
*/
	geoJSON_parse_error = 1;
	*result = NULL;
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "ParseAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void Parse(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  ParseTOKENTYPE yyminor       /* The value for the token */
  ParseARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  int yyact;            /* The parser action. */
  int yyendofinput;     /* True if we are at the end of input */
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  /* (re)initialize the parser, if necessary */
  yypParser = (yyParser*)yyp;
  if( yypParser->yyidx<0 ){
#if YYSTACKDEPTH<=0
    if( yypParser->yystksz <=0 ){
      /*memset(&yyminorunion, 0, sizeof(yyminorunion));*/
      yyminorunion = yyzerominor;
      yyStackOverflow(yypParser, &yyminorunion);
      return;
    }
#endif
    yypParser->yyidx = 0;
    yypParser->yyerrcnt = -1;
    yypParser->yystack[0].stateno = 0;
    yypParser->yystack[0].major = 0;
  }
  yyminorunion.yy0 = yyminor;
  yyendofinput = (yymajor==0);
  ParseARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact<YYNSTATE ){
      assert( !yyendofinput );  /* Impossible to shift the $ token */
      yy_shift(yypParser,yyact,yymajor,&yyminorunion);
      yypParser->yyerrcnt--;
      yymajor = YYNOCODE;
    }else if( yyact < YYNSTATE + YYNRULE ){
      yy_reduce(yypParser,yyact-YYNSTATE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yymx = yypParser->yystack[yypParser->yyidx].major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor,&yyminorunion);
        yymajor = YYNOCODE;
      }else{
         while(
          yypParser->yyidx >= 0 &&
          yymx != YYERRORSYMBOL &&
          (yyact = yy_find_reduce_action(
                        yypParser->yystack[yypParser->yyidx].stateno,
                        YYERRORSYMBOL)) >= YYNSTATE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yyidx < 0 || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          YYMINORTYPE u2;
          u2.YYERRSYMDT = 0;
          yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor,yyminorunion);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;
      
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
  return;
}


/*
 GEOJSON_LEMON_END - LEMON generated code ends here 
*/

















/*
** CAVEAT: there is an incompatibility between LEMON and FLEX
** this macro resolves the issue
*/
#undef yy_accept
#define yy_accept	yy_geo_json_flex_accept



/*
 GEOJSON_FLEX_START - FLEX generated code starts here 
*/

#line 3 "lex.GeoJson.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer GeoJson_create_buffer
#define yy_delete_buffer GeoJson_delete_buffer
#define yy_flex_debug GeoJson_flex_debug
#define yy_init_buffer GeoJson_init_buffer
#define yy_flush_buffer GeoJson_flush_buffer
#define yy_load_buffer_state GeoJson_load_buffer_state
#define yy_switch_to_buffer GeoJson_switch_to_buffer
#define yyin GeoJsonin
#define yyleng GeoJsonleng
#define yylex GeoJsonlex
#define yylineno GeoJsonlineno
#define yyout GeoJsonout
#define yyrestart GeoJsonrestart
#define yytext GeoJsontext
#define yywrap GeoJsonwrap
#define yyalloc GeoJsonalloc
#define yyrealloc GeoJsonrealloc
#define yyfree GeoJsonfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <errno.h> */
/* #include <stdlib.h> */

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE GeoJsonrestart(GeoJsonin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int GeoJsonleng;

extern FILE *GeoJsonin, *GeoJsonout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up GeoJsontext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up GeoJsontext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via GeoJsonrestart()), so that the user can continue scanning by
	 * just pointing GeoJsonin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when GeoJsontext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int GeoJsonleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow GeoJsonwrap()'s to do buffer switches
 * instead of setting up a fresh GeoJsonin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void GeoJsonrestart (FILE *input_file  );
void GeoJson_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE GeoJson_create_buffer (FILE *file,int size  );
void GeoJson_delete_buffer (YY_BUFFER_STATE b  );
void GeoJson_flush_buffer (YY_BUFFER_STATE b  );
void GeoJsonpush_buffer_state (YY_BUFFER_STATE new_buffer  );
void GeoJsonpop_buffer_state (void );

static void GeoJsonensure_buffer_stack (void );
static void GeoJson_load_buffer_state (void );
static void GeoJson_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER GeoJson_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE GeoJson_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE GeoJson_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE GeoJson_scan_bytes (yyconst char *bytes,int len  );

void *GeoJsonalloc (yy_size_t  );
void *GeoJsonrealloc (void *,yy_size_t  );
void GeoJsonfree (void *  );

#define yy_new_buffer GeoJson_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        GeoJsonensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            GeoJson_create_buffer(GeoJsonin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        GeoJsonensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            GeoJson_create_buffer(GeoJsonin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *GeoJsonin = (FILE *) 0, *GeoJsonout = (FILE *) 0;

typedef int yy_state_type;

extern int GeoJsonlineno;

int GeoJsonlineno = 1;

extern char *GeoJsontext;
#define yytext_ptr GeoJsontext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up GeoJsontext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	GeoJsonleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 27
#define YY_END_OF_BUFFER 28
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[182] =
    {   0,
        0,    0,   28,   26,   24,   25,   26,   26,    4,   26,
        1,    5,    8,    9,    6,    7,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    1,    0,
        1,    1,    1,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    1,    1,    1,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    1,    1,    0,    0,    0,    0,    0,
        0,    0,    0,   16,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,   13,    0,    0,   14,    0,
       10,    0,    0,    0,    0,    0,    0,    0,   17,    0,

        0,    0,    0,    0,    2,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,   19,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,   18,    0,   20,    0,    0,   12,   15,
        0,    0,    0,    0,   11,    0,    0,    0,   22,    0,
        0,    0,    0,    0,    0,    0,    0,   21,    0,    0,
        0,    0,    0,   23,    0,    0,    0,    0,    0,    3,
        0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    4,    1,    1,    1,    1,    1,    1,
        1,    1,    5,    6,    7,    8,    1,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,   10,    1,    1,
        1,    1,    1,    1,    1,    1,   11,    1,   12,    1,
       13,    1,    1,    1,    1,   14,   15,    1,    1,   16,
        1,    1,   17,    1,    1,    1,    1,    1,    1,    1,
       18,    1,   19,    1,    1,    1,   20,   21,   22,   23,

       24,   25,   26,    1,   27,    1,    1,   28,   29,   30,
       31,   32,    1,   33,   34,   35,   36,    1,    1,   37,
       38,    1,   39,    1,   40,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[41] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[183] =
    {   0,
        0,   39,  203,  204,  204,  204,   68,  193,  204,    4,
        3,  204,  204,  204,  204,  204,  185,  176,  172,  162,
      166,  175,    0,  171,  174,  160,  154,  158,    6,   11,
        8,  181,   12,  172,  157,  157,  158,    0,  154,  153,
      149,  151,  152,  149,  147,  148,  168,  167,  166,  161,
      144,  148,  136,  140,  131,  131,  134,  162,  136,  140,
      131,  138,  151,  151,  150,  148,  133,  139,  128,  119,
      127,  148,  128,  204,  126,  145,  124,  143,  115,   15,
      110,  109,   16,  139,  111,  204,  114,  105,  204,  106,
      204,  112,  128,   25,  103,  102,  107,  102,  204,  102,

      101,   97,   94,  106,  204,   89,   99,   95,    8,  120,
      103,   95,   94,  110,  108,   88,   93,   86,   77,  204,
       79,   89,   88,   88,   79,   83,   91,   72,   80,   81,
       68,   67,   75,   69,   92,   60,   89,   61,   57,   84,
       83,   61,   57,  204,   44,  204,   46,   71,  204,  204,
       64,   49,   45,   67,  204,   48,   47,   38,  204,   34,
       31,   39,   30,   36,   58,   51,   29,  204,   47,   25,
       38,   49,   21,  204,   24,   16,   43,   16,   47,  204,
      204,    0
    } ;

static yyconst flex_int16_t yy_def[183] =
    {   0,
      182,  182,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,

      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
        0,  181
    } ;

static yyconst flex_int16_t yy_nxt[245] =
    {   0,
        4,    5,    6,    7,    8,    9,   10,   30,   11,   12,
       32,   33,   31,   47,   29,   48,   31,   13,   14,   32,
       33,   93,   17,   94,  179,  177,   54,   55,  105,   97,
       40,   98,   41,   94,  118,  119,  176,  175,   15,   16,
        5,    6,    7,    8,    9,   10,   28,   11,   12,  178,
      180,  179,  174,  173,  172,  179,   13,   14,  171,  170,
      169,  168,  167,  166,  165,  164,  163,  162,  161,  160,
      159,  158,  157,  156,  155,  154,  153,   15,   16,   17,
       18,   19,   20,   21,  152,  151,  150,  149,   22,   23,
      148,  147,  146,   24,  145,  144,  143,   25,  142,   26,

      141,  140,   27,   28,  139,  138,  137,  136,  135,  134,
      133,  132,  131,  130,  129,  128,  127,  126,  125,  124,
      123,  122,  121,  120,  117,  116,  115,  114,  113,  112,
      111,  110,  109,  108,  107,  106,   94,  104,  103,  102,
      101,  100,   99,   96,   95,   92,   91,   90,   89,   88,
       87,   86,   85,   84,   83,   82,   81,   80,   65,   64,
       79,   78,   77,   76,   75,   74,   73,   72,   71,   70,
       69,   68,   67,   66,   49,   65,   64,   63,   62,   61,
       60,   59,   58,   57,   56,   53,   52,   51,   50,   49,
       46,   45,   44,   43,   42,   39,   38,   37,   36,   35,

       34,   29,  181,    3,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181
    } ;

static yyconst flex_int16_t yy_chk[245] =
    {   0,
      182,    1,    1,    1,    1,    1,    1,   10,    1,    1,
       11,   11,   10,   29,   29,   31,   31,    1,    1,   33,
       33,   80,   30,   80,  178,  176,   38,   38,   94,   83,
       23,   83,   23,   94,  109,  109,  175,  173,    1,    1,
        2,    2,    2,    2,    2,    2,   30,    2,    2,  177,
      179,  177,  172,  171,  170,  179,    2,    2,  169,  167,
      166,  165,  164,  163,  162,  161,  160,  158,  157,  156,
      154,  153,  152,  151,  148,  147,  145,    2,    2,    7,
        7,    7,    7,    7,  143,  142,  141,  140,    7,    7,
      139,  138,  137,    7,  136,  135,  134,    7,  133,    7,

      132,  131,    7,    7,  130,  129,  128,  127,  126,  125,
      124,  123,  122,  121,  119,  118,  117,  116,  115,  114,
      113,  112,  111,  110,  108,  107,  106,  104,  103,  102,
      101,  100,   98,   97,   96,   95,   93,   92,   90,   88,
       87,   85,   84,   82,   81,   79,   78,   77,   76,   75,
       73,   72,   71,   70,   69,   68,   67,   66,   65,   64,
       63,   62,   61,   60,   59,   58,   57,   56,   55,   54,
       53,   52,   51,   50,   49,   48,   47,   46,   45,   44,
       43,   42,   41,   40,   39,   37,   36,   35,   34,   32,
       28,   27,   26,   25,   24,   22,   21,   20,   19,   18,

       17,    8,    3,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  181,  181,
      181,  181,  181,  181
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int GeoJson_flex_debug;
int GeoJson_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *GeoJsontext;
/* 
 geoJsonLexer.l -- GeoJSON parser - FLEX config
  
 version 2.4, 2011 May 16

 Author: Sandro Furieri a.furieri@lqt.it

 ------------------------------------------------------------------------------
 
 Version: MPL 1.1/GPL 2.0/LGPL 2.1
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the SpatiaLite library

The Initial Developer of the Original Code is Alessandro Furieri
 
Portions created by the Initial Developer are Copyright (C) 2011
the Initial Developer. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of
either the GNU General Public License Version 2 or later (the "GPL"), or
the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
in which case the provisions of the GPL or the LGPL are applicable instead
of those above. If you wish to allow use of your version of this file only
under the terms of either the GPL or the LGPL, and not to allow others to
use your version of this file under the terms of the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the GPL or the LGPL. If you do not delete
the provisions above, a recipient may use your version of this file under
the terms of any one of the MPL, the GPL or the LGPL.
 
*/

/* For debugging purposes */
int geoJSON_line = 1, geoJSON_col = 1;

/**
*  The main string-token matcher.
*  The lower case part is probably not needed.  We should really be converting 
*  The string to all uppercase/lowercase to make it case iNsEnSiTiVe.
*  What Flex will do is, For the input string, beginning from the front, Flex
*  will try to match with any of the defined tokens from below.  Flex will 
*  then match the string of longest length.  Suppose the string is: POINTM,
*  Flex would match both POINT and POINTM, but since POINTM is the longer
*  of the two tokens, FLEX will match POINTM.
*/

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* #include <unistd.h> */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int GeoJsonlex_destroy (void );

int GeoJsonget_debug (void );

void GeoJsonset_debug (int debug_flag  );

YY_EXTRA_TYPE GeoJsonget_extra (void );

void GeoJsonset_extra (YY_EXTRA_TYPE user_defined  );

FILE *GeoJsonget_in (void );

void GeoJsonset_in  (FILE * in_str  );

FILE *GeoJsonget_out (void );

void GeoJsonset_out  (FILE * out_str  );

int GeoJsonget_leng (void );

char *GeoJsonget_text (void );

int GeoJsonget_lineno (void );

void GeoJsonset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int GeoJsonwrap (void );
#else
extern int GeoJsonwrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( GeoJsontext, GeoJsonleng, 1, GeoJsonout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( GeoJsonin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( GeoJsonin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, GeoJsonin))==0 && ferror(GeoJsonin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(GeoJsonin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int GeoJsonlex (void);

#define YY_DECL int GeoJsonlex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after GeoJsontext and GeoJsonleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! GeoJsonin )
			GeoJsonin = stdin;

		if ( ! GeoJsonout )
			GeoJsonout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			GeoJsonensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				GeoJson_create_buffer(GeoJsonin,YY_BUF_SIZE );
		}

		GeoJson_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of GeoJsontext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 182 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 204 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
{ geoJSON_col += (int) strlen(GeoJsontext);  GeoJsonLval.dval = atof(GeoJsontext); return GEOJSON_NUM; }
	YY_BREAK
case 2:
YY_RULE_SETUP
{ geoJSON_col += (int) strlen(GeoJsontext);  GeoJsonLval.ival = atoi(GeoJsontext+6); return GEOJSON_SHORT_SRID; }
	YY_BREAK
case 3:
YY_RULE_SETUP
{ geoJSON_col += (int) strlen(GeoJsontext);  GeoJsonLval.ival = atoi(GeoJsontext+22); return GEOJSON_LONG_SRID; }
	YY_BREAK
case 4:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_COMMA; }
	YY_BREAK
case 5:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_COLON; }
	YY_BREAK
case 6:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_OPEN_BRACE; }
	YY_BREAK
case 7:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_CLOSE_BRACE; }
	YY_BREAK
case 8:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_OPEN_BRACKET; }
	YY_BREAK
case 9:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_CLOSE_BRACKET; }
	YY_BREAK
case 10:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_TYPE; }
	YY_BREAK
case 11:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_COORDS; }
	YY_BREAK
case 12:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_GEOMS; }
	YY_BREAK
case 13:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_BBOX; }
	YY_BREAK
case 14:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_NAME; }
	YY_BREAK
case 15:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_PROPS; }
	YY_BREAK
case 16:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_CRS; }
	YY_BREAK
case 17:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_POINT; }
	YY_BREAK
case 18:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_LINESTRING; }
	YY_BREAK
case 19:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_POLYGON; }
	YY_BREAK
case 20:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_MULTIPOINT; }
	YY_BREAK
case 21:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_MULTILINESTRING; }
	YY_BREAK
case 22:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_MULTIPOLYGON; }
	YY_BREAK
case 23:
YY_RULE_SETUP
{ GeoJsonLval.dval = 0; return GEOJSON_GEOMETRYCOLLECTION; }
	YY_BREAK
case 24:
YY_RULE_SETUP
{ geoJSON_col += (int) strlen(GeoJsontext); }               /* ignore but count white space */
	YY_BREAK
case 25:
/* rule 25 can match eol */
YY_RULE_SETUP
{ geoJSON_col = 0; ++geoJSON_line; }
	YY_BREAK
case 26:
YY_RULE_SETUP
{ geoJSON_col += (int) strlen(GeoJsontext); return -1; }
	YY_BREAK
case 27:
YY_RULE_SETUP
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed GeoJsonin at a new source and called
			 * GeoJsonlex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = GeoJsonin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( GeoJsonwrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * GeoJsontext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of GeoJsonlex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					GeoJsonrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			GeoJsonrestart(GeoJsonin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) GeoJsonrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 182 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 182 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 181);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up GeoJsontext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					GeoJsonrestart(GeoJsonin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( GeoJsonwrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve GeoJsontext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void GeoJsonrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        GeoJsonensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            GeoJson_create_buffer(GeoJsonin,YY_BUF_SIZE );
	}

	GeoJson_init_buffer(YY_CURRENT_BUFFER,input_file );
	GeoJson_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void GeoJson_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		GeoJsonpop_buffer_state();
	 *		GeoJsonpush_buffer_state(new_buffer);
     */
	GeoJsonensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	GeoJson_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (GeoJsonwrap()) processing, but the only time this flag
	 * is looked at is after GeoJsonwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void GeoJson_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	GeoJsonin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE GeoJson_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) GeoJsonalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in GeoJson_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) GeoJsonalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in GeoJson_create_buffer()" );

	b->yy_is_our_buffer = 1;

	GeoJson_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with GeoJson_create_buffer()
 * 
 */
    void GeoJson_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		GeoJsonfree((void *) b->yy_ch_buf  );

	GeoJsonfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a GeoJsonrestart() or at EOF.
 */
    static void GeoJson_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	GeoJson_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then GeoJson_init_buffer was _probably_
     * called from GeoJsonrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void GeoJson_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		GeoJson_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void GeoJsonpush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	GeoJsonensure_buffer_stack();

	/* This block is copied from GeoJson_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from GeoJson_switch_to_buffer. */
	GeoJson_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void GeoJsonpop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	GeoJson_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		GeoJson_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void GeoJsonensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)GeoJsonalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in GeoJsonensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)GeoJsonrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in GeoJsonensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE GeoJson_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) GeoJsonalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in GeoJson_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	GeoJson_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to GeoJsonlex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       GeoJson_scan_bytes() instead.
 */
YY_BUFFER_STATE GeoJson_scan_string (yyconst char * yystr )
{
    
	return GeoJson_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to GeoJsonlex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE GeoJson_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) GeoJsonalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in GeoJson_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = GeoJson_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in GeoJson_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up GeoJsontext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		GeoJsontext[GeoJsonleng] = (yy_hold_char); \
		(yy_c_buf_p) = GeoJsontext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		GeoJsonleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int GeoJsonget_lineno  (void)
{
        
    return GeoJsonlineno;
}

/** Get the input stream.
 * 
 */
FILE *GeoJsonget_in  (void)
{
        return GeoJsonin;
}

/** Get the output stream.
 * 
 */
FILE *GeoJsonget_out  (void)
{
        return GeoJsonout;
}

/** Get the length of the current token.
 * 
 */
int GeoJsonget_leng  (void)
{
        return GeoJsonleng;
}

/** Get the current token.
 * 
 */

char *GeoJsonget_text  (void)
{
        return GeoJsontext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void GeoJsonset_lineno (int  line_number )
{
    
    GeoJsonlineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see GeoJson_switch_to_buffer
 */
void GeoJsonset_in (FILE *  in_str )
{
        GeoJsonin = in_str ;
}

void GeoJsonset_out (FILE *  out_str )
{
        GeoJsonout = out_str ;
}

int GeoJsonget_debug  (void)
{
        return GeoJson_flex_debug;
}

void GeoJsonset_debug (int  bdebug )
{
        GeoJson_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from GeoJsonlex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    GeoJsonin = stdin;
    GeoJsonout = stdout;
#else
    GeoJsonin = (FILE *) 0;
    GeoJsonout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * GeoJsonlex_init()
     */
    return 0;
}

/* GeoJsonlex_destroy is for both reentrant and non-reentrant scanners. */
int GeoJsonlex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		GeoJson_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		GeoJsonpop_buffer_state();
	}

	/* Destroy the stack itself. */
	GeoJsonfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * GeoJsonlex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *GeoJsonalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *GeoJsonrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void GeoJsonfree (void * ptr )
{
	free( (char *) ptr );	/* see GeoJsonrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

/**
 * reset the line and column count
 *
 *
 */
void geoJSON_reset_lexer(void)
{

  geoJSON_line = 1;
  geoJSON_col  = 1;

}

/**
 * GeoJsonError() is invoked when the lexer or the parser encounter
 * an error. The error message is passed via *s
 *
 *
 */
void GeoJsonError(char *s)
{
  printf("error: %s at line: %d col: %d\n",s,geoJSON_line,geoJSON_col);

}

int GeoJsonwrap(void)
{
  return 1;
}


/* 
 GEOJSON_FLEX_END - FLEX generated code ends here 
*/



/*
** This is a linked-list struct to store all the values for each token.
** All tokens will have a value of 0, except tokens denoted as NUM.
** NUM tokens are geometry coordinates and will contain the floating
** point number.
*/
typedef struct geoJsonFlexTokenStruct
{
    double value;
    struct geoJsonFlexTokenStruct *Next;
} geoJsonFlexToken;

/*
** Function to clean up the linked-list of token values.
*/
static int
geoJSON_cleanup (geoJsonFlexToken * token)
{
    geoJsonFlexToken *ptok;
    geoJsonFlexToken *ptok_n;
    if (token == NULL)
	return 0;
    ptok = token;
    while (ptok)
      {
	  ptok_n = ptok->Next;
	  free (ptok);
	  ptok = ptok_n;
      }
    return 0;
}

gaiaGeomCollPtr
gaiaParseGeoJSON (const unsigned char *dirty_buffer)
{
    void *pParser = ParseAlloc (malloc);
    /* Linked-list of token values */
    geoJsonFlexToken *tokens = malloc (sizeof (geoJsonFlexToken));
    /* Pointer to the head of the list */
    geoJsonFlexToken *head = tokens;
    int yv;
    gaiaGeomCollPtr result = NULL;

    tokens->Next = NULL;
    geoJSON_parse_error = 0;

    GeoJson_scan_string ((char *) dirty_buffer);

    /*
       / Keep tokenizing until we reach the end
       / yylex() will return the next matching Token for us.
     */
    while ((yv = yylex ()) != 0)
      {
	  if (yv == -1)
	    {
		geoJSON_parse_error = 1;
		break;
	    }
	  tokens->Next = malloc (sizeof (geoJsonFlexToken));
	  tokens->Next->Next = NULL;
	  /*
	     /GeoJsonLval is a global variable from FLEX.
	     /GeoJsonLval is defined in geoJsonLexglobal.h
	   */
	  tokens->Next->value = GeoJsonLval.dval;
	  /* Pass the token to the wkt parser created from lemon */
	  Parse (pParser, yv, &(tokens->Next->value), &result);
	  tokens = tokens->Next;
      }
    /* This denotes the end of a line as well as the end of the parser */
    Parse (pParser, GEOJSON_NEWLINE, 0, &result);
    ParseFree (pParser, free);
    GeoJsonlex_destroy ();

    /* Assigning the token as the end to avoid seg faults while cleaning */
    tokens->Next = NULL;
    geoJSON_cleanup (head);

    if (geoJSON_parse_error)
      {
	  if (result)
	      gaiaFreeGeomColl (result);
	  return NULL;
      }

    if (!result)
	return NULL;
    if (!geoJsonCheckValidity (result))
      {
	  gaiaFreeGeomColl (result);
	  return NULL;
      }

    gaiaMbrGeometry (result);

    return result;
}


/*
** CAVEAT: we must now undefine any Lemon/Flex own macro
*/
#undef YYNOCODE
#undef YYNSTATE
#undef YYNRULE
#undef YY_SHIFT_MAX
#undef YY_SHIFT_USE_DFLT
#undef YY_REDUCE_USE_DFLT
#undef YY_REDUCE_MAX
#undef YY_FLUSH_BUFFER
#undef YY_DO_BEFORE_ACTION
#undef YY_NUM_RULES
#undef YY_END_OF_BUFFER
#undef YY_END_FILE
#undef YYACTIONTYPE
#undef YY_SZ_ACTTAB
#undef YY_NEW_FILE
#undef BEGIN
#undef YY_START
#undef YY_CURRENT_BUFFER
#undef YY_CURRENT_BUFFER_LVALUE
#undef YY_STATE_BUF_SIZE
#undef YY_DECL
#undef YY_FATAL_ERROR
#undef YYMINORTYPE
#undef YY_CHAR
#undef YYSTYPE
#undef input
#undef ParseAlloc
#undef ParseFree
#undef ParseStackPeak
#undef Parse
#undef yyalloc
#undef yyfree
#undef yyin
#undef yyleng
#undef yyless
#undef yylex
#undef yylineno
#undef yyout
#undef yyrealloc
#undef yyrestart
#undef yyStackEntry
#undef yytext
#undef yywrap
#undef yyzerominor
#undef yy_accept
#undef yy_action
#undef yy_base
#undef yy_buffer_stack
#undef yy_buffer_stack_max
#undef yy_buffer_stack_top
#undef yy_c_buf_p
#undef yy_chk
#undef yy_create_buffer
#undef yy_def
#undef yy_default
#undef yy_delete_buffer
#undef yy_destructor
#undef yy_ec
#undef yy_fatal_error
#undef yy_find_reduce_action
#undef yy_find_shift_action
#undef yy_flex_debug
#undef yy_flush_buffer
#undef yy_get_next_buffer
#undef yy_get_previous_state
#undef yy_init
#undef yy_init_buffer
#undef yy_init_globals
#undef yy_load_buffer
#undef yy_load_buffer_state
#undef yy_lookahead
#undef yy_meta
#undef yy_new_buffer
#undef yy_nxt
#undef yy_parse_failed
#undef yy_pop_parser_stack
#undef yy_reduce
#undef yy_reduce_ofst
#undef yy_set_bol
#undef yy_set_interactive
#undef yy_shift
#undef yy_shift_ofst
#undef yy_start
#undef yy_state_type
#undef yy_switch_to_buffer
#undef yy_syntax_error
#undef yy_trans_info
#undef yy_try_NUL_trans
#undef yyParser
#undef yyStackEntry
#undef yyStackOverflow
#undef yyRuleInfo
#undef yytext_ptr
#undef yyunput
#undef yyzerominor
#undef ParseARG_SDECL
#undef ParseARG_PDECL
#undef ParseARG_FETCH
#undef ParseARG_STORE
#undef REJECT
#undef yymore
#undef YY_MORE_ADJ
#undef YY_RESTORE_YY_MORE_OFFSET
#undef YY_LESS_LINENO
#undef yyTracePrompt
#undef yyTraceFILE
#undef yyTokenName
#undef yyRuleName
#undef ParseTrace
/**************** End file: gg_geoJSON.c **********/


/**************** Begin file: gg_kml.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

/* #include <assert.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3ext.h> */
#else
/* #include <sqlite3ext.h> */
#endif

/* #include <spatialite/gaiageo.h> */

#if defined(_WIN32) || defined(WIN32)
# include <io.h>
#define isatty  _isatty
#define fileno  _fileno
#endif

int kml_parse_error;

#define KML_PARSER_OPEN_NODE		1
#define KML_PARSER_SELF_CLOSED_NODE	2
#define KML_PARSER_CLOSED_NODE		3

#define GAIA_KML_UNKNOWN		0
#define GAIA_KML_POINT			1
#define GAIA_KML_LINESTRING		2
#define GAIA_KML_POLYGON		3
#define GAIA_KML_MULTIGEOMETRY		4

/*
** This is a linked-list struct to store all the values for each token.
*/
typedef struct kmlFlexTokenStruct
{
    char *value;
    struct kmlFlexTokenStruct *Next;
} kmlFlexToken;

typedef struct kml_coord
{
    char *Value;
    struct kml_coord *Next;
} kmlCoord;
typedef kmlCoord *kmlCoordPtr;

typedef struct kml_attr
{
    char *Key;
    char *Value;
    struct kml_attr *Next;
} kmlAttr;
typedef kmlAttr *kmlAttrPtr;

typedef struct kml_node
{
    char *Tag;
    int Type;
    int Error;
    struct kml_attr *Attributes;
    struct kml_coord *Coordinates;
    struct kml_node *Next;
} kmlNode;
typedef kmlNode *kmlNodePtr;

typedef struct kml_dynamic_ring
{
    gaiaDynamicLinePtr ring;
    int interior;
    int has_z;
    struct kml_dynamic_ring *next;
} kmlDynamicRing;
typedef kmlDynamicRing *kmlDynamicRingPtr;

typedef struct kml_dynamic_polygon
{
    struct kml_dynamic_ring *first;
    struct kml_dynamic_ring *last;
} kmlDynamicPolygon;
typedef kmlDynamicPolygon *kmlDynamicPolygonPtr;

static kmlDynamicPolygonPtr
kml_alloc_dyn_polygon (void)
{
/* creating a dynamic polygon (ring collection) */
    kmlDynamicPolygonPtr p = malloc (sizeof (kmlDynamicPolygon));
    p->first = NULL;
    p->last = NULL;
    return p;
}

static void
kml_free_dyn_polygon (kmlDynamicPolygonPtr dyn)
{
/* deleting a dynamic polygon (ring collection) */
    kmlDynamicRingPtr r;
    kmlDynamicRingPtr rn;
    if (!dyn)
	return;
    r = dyn->first;
    while (r)
      {
	  rn = r->next;
	  if (r->ring)
	      gaiaFreeDynamicLine (r->ring);
	  free (r);
	  r = rn;
      }
    free (dyn);
}

static void
kml_add_polygon_ring (kmlDynamicPolygonPtr dyn_pg, gaiaDynamicLinePtr dyn,
		      int interior, int has_z)
{
/* inserting a further ring into the collection (dynamic polygon) */
    kmlDynamicRingPtr p = malloc (sizeof (kmlDynamicRing));
    p->ring = dyn;
    p->interior = interior;
    p->has_z = has_z;
    p->next = NULL;
    if (dyn_pg->first == NULL)
	dyn_pg->first = p;
    if (dyn_pg->last != NULL)
	dyn_pg->last->next = p;
    dyn_pg->last = p;
}

static void
kml_freeString (char **ptr)
{
/* releasing a string from the lexer */
    if (*ptr != NULL)
	free (*ptr);
    *ptr = NULL;
}

static void
kml_saveString (char **ptr, const char *str)
{
/* saving a string from the lexer */
    int len = strlen (str);
    kml_freeString (ptr);
    *ptr = malloc (len + 1);
    strcpy (*ptr, str);
}

static kmlCoordPtr
kml_coord (void *value)
{
/* creating a coord Item */
    int len;
    kmlFlexToken *tok = (kmlFlexToken *) value;
    kmlCoordPtr c = malloc (sizeof (kmlCoord));
    len = strlen (tok->value);
    c->Value = malloc (len + 1);
    strcpy (c->Value, tok->value);
    c->Next = NULL;
    return c;
}

static void
kml_freeCoordinate (kmlCoordPtr c)
{
/* deleting a KML coordinate */
    if (c == NULL)
	return;
    if (c->Value)
	free (c->Value);
    free (c);
}

static kmlAttrPtr
kml_attribute (void *key, void *value)
{
/* creating an attribute */
    int len;
    kmlFlexToken *k_tok = (kmlFlexToken *) key;
    kmlFlexToken *v_tok = (kmlFlexToken *) value;
    kmlAttrPtr a = malloc (sizeof (kmlAttr));
    len = strlen (k_tok->value);
    a->Key = malloc (len + 1);
    strcpy (a->Key, k_tok->value);
    len = strlen (v_tok->value);
/* we need to de-quote the string, removing first and last ".." */
    if (*(v_tok->value + 0) == '"' && *(v_tok->value + len - 1) == '"')
      {
	  a->Value = malloc (len - 1);
	  memcpy (a->Value, v_tok->value + 1, len - 1);
	  *(a->Value + len - 1) = '\0';
      }
    else
      {
	  a->Value = malloc (len + 1);
	  strcpy (a->Value, v_tok->value);
      }
    a->Next = NULL;
    return a;
}

static void
kml_freeAttribute (kmlAttrPtr a)
{
/* deleting a KML attribute */
    if (a == NULL)
	return;
    if (a->Key)
	free (a->Key);
    if (a->Value)
	free (a->Value);
    free (a);
}

static void
kml_freeNode (kmlNodePtr n)
{
/* deleting a KML node */
    kmlAttrPtr a;
    kmlAttrPtr an;
    kmlCoordPtr c;
    kmlCoordPtr cn;
    if (n == NULL)
	return;
    a = n->Attributes;
    while (a)
      {
	  an = a->Next;
	  kml_freeAttribute (a);
	  a = an;
      }
    c = n->Coordinates;
    while (c)
      {
	  cn = c->Next;
	  kml_freeCoordinate (c);
	  c = cn;
      }
    if (n->Tag)
	free (n->Tag);
    free (n);
}

static void
kml_freeTree (kmlNodePtr t)
{
/* deleting a KML tree */
    kmlNodePtr n;
    kmlNodePtr nn;
    n = t;
    while (n)
      {
	  nn = n->Next;
	  kml_freeNode (n);
	  n = nn;
      }
}

static kmlNodePtr
kml_createNode (void *tag, void *attributes, void *coords)
{
/* creating a node */
    int len;
    kmlFlexToken *tok = (kmlFlexToken *) tag;
    kmlNodePtr n = malloc (sizeof (kmlNode));
    len = strlen (tok->value);
    n->Tag = malloc (len + 1);
    strcpy (n->Tag, tok->value);
    n->Type = KML_PARSER_OPEN_NODE;
    n->Error = 0;
    n->Attributes = attributes;
    n->Coordinates = coords;
    n->Next = NULL;
    return n;
}

static kmlNodePtr
kml_createSelfClosedNode (void *tag, void *attributes)
{
/* creating a self-closed node */
    int len;
    kmlFlexToken *tok = (kmlFlexToken *) tag;
    kmlNodePtr n = malloc (sizeof (kmlNode));
    len = strlen (tok->value);
    n->Tag = malloc (len + 1);
    strcpy (n->Tag, tok->value);
    n->Type = KML_PARSER_SELF_CLOSED_NODE;
    n->Error = 0;
    n->Attributes = attributes;
    n->Coordinates = NULL;
    n->Next = NULL;
    return n;
}

static kmlNodePtr
kml_closingNode (void *tag)
{
/* creating a closing node */
    int len;
    kmlFlexToken *tok = (kmlFlexToken *) tag;
    kmlNodePtr n = malloc (sizeof (kmlNode));
    len = strlen (tok->value);
    n->Tag = malloc (len + 1);
    strcpy (n->Tag, tok->value);
    n->Type = KML_PARSER_CLOSED_NODE;
    n->Error = 0;
    n->Attributes = NULL;
    n->Coordinates = NULL;
    n->Next = NULL;
    return n;
}

static int
kml_cleanup (kmlFlexToken * token)
{
    kmlFlexToken *ptok;
    kmlFlexToken *ptok_n;
    if (token == NULL)
	return 0;
    ptok = token;
    while (ptok)
      {
	  ptok_n = ptok->Next;
	  if (ptok->value != NULL)
	      free (ptok->value);
	  free (ptok);
	  ptok = ptok_n;
      }
    return 0;
}

static void
kml_xferString (char **p, const char *str)
{
/* saving some token */
    int len;
    if (str == NULL)
      {
	  *p = NULL;
	  return;
      }
    len = strlen (str);
    *p = malloc (len + 1);
    strcpy (*p, str);
}

static int
guessKmlGeometryType (kmlNodePtr node)
{
/* attempting to guess the Geometry Type for a KML node */
    int type = GAIA_KML_UNKNOWN;
    if (strcmp (node->Tag, "Point") == 0)
	type = GAIA_KML_POINT;
    if (strcmp (node->Tag, "LineString") == 0)
	type = GAIA_KML_LINESTRING;
    if (strcmp (node->Tag, "Polygon") == 0)
	type = GAIA_KML_POLYGON;
    if (strcmp (node->Tag, "MultiGeometry") == 0)
	type = GAIA_KML_MULTIGEOMETRY;
    return type;
}

static int
kml_check_coord (const char *value)
{
/* checking a KML coordinate */
    int decimal = 0;
    const char *p = value;
    if (*p == '+' || *p == '-')
	p++;
    while (*p != '\0')
      {
	  if (*p == '.')
	    {
		if (!decimal)
		    decimal = 1;
		else
		    return 0;
	    }
	  else if (*p >= '0' && *p <= '9')
	      ;
	  else
	      return 0;
	  p++;
      }
    return 1;
}

static int
kml_extract_coords (const char *value, double *x, double *y, double *z,
		    int *count)
{
/* extracting KML coords from a comma-separated string */
    const char *in = value;
    char buf[1024];
    char *out = buf;
    *out = '\0';

    while (*in != '\0')
      {
	  if (*in == ',')
	    {
		*out = '\0';
		if (*buf != '\0')
		  {
		      if (!kml_check_coord (buf))
			  return 0;
		      switch (*count)
			{
			case 0:
			    *x = atof (buf);
			    *count += 1;
			    break;
			case 1:
			    *y = atof (buf);
			    *count += 1;
			    break;
			case 2:
			    *z = atof (buf);
			    *count += 1;
			    break;
			default:
			    *count += 1;
			    break;
			};
		  }
		in++;
		out = buf;
		*out = '\0';
		continue;
	    }
	  *out++ = *in++;
      }
    *out = '\0';
/* parsing the last item */
    if (*buf != '\0')
      {
	  if (!kml_check_coord (buf))
	      return 0;
	  switch (*count)
	    {
	    case 0:
		*x = atof (buf);
		*count += 1;
		break;
	    case 1:
		*y = atof (buf);
		*count += 1;
		break;
	    case 2:
		*z = atof (buf);
		*count += 1;
		break;
	    default:
		*count += 1;
		break;
	    };
      }
    return 1;
}

static int
kml_parse_point_v2 (kmlCoordPtr coord, double *x, double *y, double *z,
		    int *has_z)
{
/* parsing KML <coordinates> [Point] */
    int count = 0;
    kmlCoordPtr c = coord;
    while (c)
      {
	  if (!kml_extract_coords (c->Value, x, y, z, &count))
	      return 0;
	  c = c->Next;
      }
    if (count == 2)
      {
	  *has_z = 0;
	  return 1;
      }
    if (count == 3)
      {
	  *has_z = 1;
	  return 1;
      }
    return 0;
}

static int
kml_parse_point (gaiaGeomCollPtr geom, kmlNodePtr node, kmlNodePtr * next)
{
/* parsing a <Point> */
    double x;
    double y;
    double z;
    int has_z;
    gaiaGeomCollPtr pt;
    gaiaGeomCollPtr last;

    if (strcmp (node->Tag, "coordinates") == 0)
      {
	  /* parsing a KML <Point> */
	  if (!kml_parse_point_v2 (node->Coordinates, &x, &y, &z, &has_z))
	      return 0;
	  node = node->Next;
	  if (node == NULL)
	      return 0;
	  if (strcmp (node->Tag, "coordinates") == 0)
	      ;
	  else
	      return 0;
	  node = node->Next;
	  if (node == NULL)
	      return 0;
	  if (strcmp (node->Tag, "Point") == 0)
	      ;
	  else
	      return 0;
	  *next = node->Next;
	  goto ok;
      }
    return 0;

  ok:
/* ok, KML nodes match as expected */
    if (has_z)
      {
	  pt = gaiaAllocGeomCollXYZ ();
	  gaiaAddPointToGeomCollXYZ (pt, x, y, z);
      }
    else
      {
	  pt = gaiaAllocGeomColl ();
	  gaiaAddPointToGeomColl (pt, x, y);
      }
    last = geom;
    while (1)
      {
	  /* searching the last Geometry within chain */
	  if (last->Next == NULL)
	      break;
	  last = last->Next;
      }
    last->Next = pt;
    return 1;
}

static int
kml_extract_multi_coord (const char *value, double *x, double *y, double *z,
			 int *count, int *follow)
{
/* extracting KML coords from a comma-separated string */
    const char *in = value;
    char buf[1024];
    char *out = buf;
    int last;
    *out = '\0';
    while (*in != '\0')
      {
	  last = *in;
	  if (*in == ',')
	    {
		*out = '\0';
		if (*buf != '\0')
		  {
		      if (!kml_check_coord (buf))
			  return 0;
		      switch (*count)
			{
			case 0:
			    *x = atof (buf);
			    *count += 1;
			    break;
			case 1:
			    *y = atof (buf);
			    *count += 1;
			    break;
			case 2:
			    *z = atof (buf);
			    *count += 1;
			    break;
			default:
			    *count += 1;
			    break;
			};
		  }
		in++;
		out = buf;
		*out = '\0';
		continue;
	    }
	  *out++ = *in++;
      }
    *out = '\0';
/* parsing the last item */
    if (*buf != '\0')
      {
	  if (!kml_check_coord (buf))
	      return 0;
	  switch (*count)
	    {
	    case 0:
		*x = atof (buf);
		*count += 1;
		break;
	    case 1:
		*y = atof (buf);
		*count += 1;
		break;
	    case 2:
		*z = atof (buf);
		*count += 1;
		break;
	    default:
		*count += 1;
		break;
	    };
      }
    if (last == ',')
	*follow = 1;
    else
	*follow = 0;
    return 1;
}

static int
kml_extract_multi_coords (kmlCoordPtr coord, double *x, double *y, double *z,
			  int *count, kmlCoordPtr * next)
{
/* extracting KML coords from a comma-separated string */
    int follow;
    kmlCoordPtr c = coord;
    while (c)
      {
	  if (!kml_extract_multi_coord (c->Value, x, y, z, count, &follow))
	      return 0;
	  if (!follow && c->Next != NULL)
	    {
		if (*(c->Next->Value) == ',')
		    follow = 1;
	    }
	  if (follow)
	      c = c->Next;
	  else
	    {
		*next = c->Next;
		break;
	    }
      }
    return 1;
}

static void
kml_add_point_to_line (gaiaDynamicLinePtr dyn, double x, double y)
{
/* appending a point */
    gaiaAppendPointToDynamicLine (dyn, x, y);
}

static void
kml_add_point_to_lineZ (gaiaDynamicLinePtr dyn, double x, double y, double z)
{
/* appending a point */
    gaiaAppendPointZToDynamicLine (dyn, x, y, z);
}

static int
kml_parse_coordinates (kmlCoordPtr coord, gaiaDynamicLinePtr dyn, int *has_z)
{
/* parsing KML <coordinates> [Linestring or Ring] */
    int count = 0;
    double x;
    double y;
    double z;
    kmlCoordPtr next;
    kmlCoordPtr c = coord;
    while (c)
      {
	  if (!kml_extract_multi_coords (c, &x, &y, &z, &count, &next))
	      return 0;
	  if (count == 2)
	    {
		*has_z = 0;
		kml_add_point_to_line (dyn, x, y);
		count = 0;
	    }
	  else if (count == 3)
	    {
		kml_add_point_to_lineZ (dyn, x, y, z);
		count = 0;
	    }
	  else
	      return 0;
	  c = next;
      }
    return 1;
}

static int
kml_count_dyn_points (gaiaDynamicLinePtr dyn)
{
/* count how many vertices are into sone linestring/ring */
    int iv = 0;
    gaiaPointPtr pt = dyn->First;
    while (pt)
      {
	  iv++;
	  pt = pt->Next;
      }
    return iv;
}

static int
kml_parse_linestring (gaiaGeomCollPtr geom, kmlNodePtr node, kmlNodePtr * next)
{
/* parsing a <LineString> */
    gaiaGeomCollPtr ln;
    gaiaGeomCollPtr last;
    gaiaLinestringPtr new_ln;
    gaiaPointPtr pt;
    gaiaDynamicLinePtr dyn = gaiaAllocDynamicLine ();
    int iv;
    int has_z = 1;
    int points = 0;

    if (strcmp (node->Tag, "coordinates") == 0)
      {
	  /* parsing a KML <LineString> */
	  if (!kml_parse_coordinates (node->Coordinates, dyn, &has_z))
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "coordinates") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "LineString") == 0)
	      ;
	  else
	      goto error;
	  *next = node->Next;
      }

/* ok, KML nodes match as expected */
    points = kml_count_dyn_points (dyn);
    if (points < 2)
	goto error;
    if (has_z)
      {
	  ln = gaiaAllocGeomCollXYZ ();
	  new_ln = gaiaAddLinestringToGeomColl (ln, points);
	  pt = dyn->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPointXYZ (new_ln->Coords, iv, pt->X, pt->Y, pt->Z);
		iv++;
		pt = pt->Next;
	    }
      }
    else
      {
	  ln = gaiaAllocGeomColl ();
	  new_ln = gaiaAddLinestringToGeomColl (ln, points);
	  pt = dyn->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPoint (new_ln->Coords, iv, pt->X, pt->Y);
		iv++;
		pt = pt->Next;
	    }
      }
    last = geom;
    while (1)
      {
	  /* searching the last Geometry within chain */
	  if (last->Next == NULL)
	      break;
	  last = last->Next;
      }
    last->Next = ln;
    gaiaFreeDynamicLine (dyn);
    return 1;

  error:
    gaiaFreeDynamicLine (dyn);
    return 0;
}

static gaiaDynamicLinePtr
kml_parse_ring (kmlNodePtr node, int *interior, int *has_z, kmlNodePtr * next)
{
/* parsing a generic KML ring */
    gaiaDynamicLinePtr dyn = gaiaAllocDynamicLine ();
    *has_z = 1;

    if (strcmp (node->Tag, "outerBoundaryIs") == 0)
      {
	  /* parsing a KML <outerBoundaryIs> */
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "coordinates") == 0)
	    {
		/* parsing a KML <kml:coordinates> */
		if (!kml_parse_coordinates (node->Coordinates, dyn, has_z))
		    goto error;
		node = node->Next;
		if (node == NULL)
		    goto error;
		if (strcmp (node->Tag, "coordinates") == 0)
		    ;
		else
		    goto error;
	    }
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "outerBoundaryIs") == 0)
	      ;
	  else
	      goto error;
	  *interior = 0;
	  *next = node->Next;
	  return dyn;
      }
    if (strcmp (node->Tag, "innerBoundaryIs") == 0)
      {
	  /* parsing a KML <innerBoundaryIs> */
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "coordinates") == 0)
	    {
		/* parsing a KML <coordinates> */
		if (!kml_parse_coordinates (node->Coordinates, dyn, has_z))
		    goto error;
		node = node->Next;
		if (node == NULL)
		    goto error;
		if (strcmp (node->Tag, "coordinates") == 0)
		    ;
		else
		    goto error;
	    }
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "innerBoundaryIs") == 0)
	      ;
	  else
	      goto error;
	  *interior = 1;
	  *next = node->Next;
	  return dyn;
      }

  error:
    gaiaFreeDynamicLine (dyn);
    return 0;
}

static int
kml_parse_polygon (gaiaGeomCollPtr geom, kmlNodePtr node, kmlNodePtr * next_n)
{
/* parsing a <Polygon> */
    int interior;
    int has_z;
    int inners;
    int outers;
    int points;
    int iv;
    int ib = 0;
    gaiaGeomCollPtr pg;
    gaiaGeomCollPtr last_g;
    gaiaPolygonPtr new_pg;
    gaiaRingPtr ring;
    gaiaDynamicLinePtr dyn;
    gaiaPointPtr pt;
    gaiaDynamicLinePtr exterior_ring;
    kmlNodePtr next;
    kmlDynamicRingPtr dyn_rng;
    kmlDynamicPolygonPtr dyn_pg = kml_alloc_dyn_polygon ();
    kmlNodePtr n = node;
    while (n)
      {
	  /* looping on rings */
	  if (strcmp (n->Tag, "Polygon") == 0)
	    {
		*next_n = n->Next;
		break;
	    }
	  dyn = kml_parse_ring (n, &interior, &has_z, &next);
	  if (dyn == NULL)
	      goto error;
	  if (kml_count_dyn_points (dyn) < 4)
	    {
		/* cannot be a valid ring */
		goto error;
	    }
	  /* checking if the ring is closed */
	  if (has_z)
	    {
		if (dyn->First->X == dyn->Last->X
		    && dyn->First->Y == dyn->Last->Y
		    && dyn->First->Z == dyn->Last->Z)
		    ;
		else
		    goto error;
	    }
	  else
	    {
		if (dyn->First->X == dyn->Last->X
		    && dyn->First->Y == dyn->Last->Y)
		    ;
		else
		    goto error;
	    }
	  kml_add_polygon_ring (dyn_pg, dyn, interior, has_z);
	  n = next;
      }
/* ok, KML nodes match as expected */
    inners = 0;
    outers = 0;
    has_z = 1;
    dyn_rng = dyn_pg->first;
    while (dyn_rng)
      {
	  /* verifying the rings collection */
	  if (dyn_rng->has_z == 0)
	      has_z = 0;
	  if (dyn_rng->interior)
	      inners++;
	  else
	    {
		outers++;
		points = kml_count_dyn_points (dyn_rng->ring);
		exterior_ring = dyn_rng->ring;
	    }
	  dyn_rng = dyn_rng->next;
      }
    if (outers != 1)		/* no exterior ring declared */
	goto error;

    if (has_z)
      {
	  pg = gaiaAllocGeomCollXYZ ();
	  new_pg = gaiaAddPolygonToGeomColl (pg, points, inners);
	  /* initializing the EXTERIOR RING */
	  ring = new_pg->Exterior;
	  pt = exterior_ring->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPointXYZ (ring->Coords, iv, pt->X, pt->Y, pt->Z);
		iv++;
		pt = pt->Next;
	    }
	  dyn_rng = dyn_pg->first;
	  while (dyn_rng)
	    {
		/* initializing any INTERIOR RING */
		if (dyn_rng->interior == 0)
		  {
		      dyn_rng = dyn_rng->next;
		      continue;
		  }
		points = kml_count_dyn_points (dyn_rng->ring);
		ring = gaiaAddInteriorRing (new_pg, ib, points);
		ib++;
		pt = dyn_rng->ring->First;
		iv = 0;
		while (pt)
		  {
		      gaiaSetPointXYZ (ring->Coords, iv, pt->X, pt->Y, pt->Z);
		      iv++;
		      pt = pt->Next;
		  }
		dyn_rng = dyn_rng->next;
	    }
      }
    else
      {
	  pg = gaiaAllocGeomColl ();
	  new_pg = gaiaAddPolygonToGeomColl (pg, points, inners);
	  /* initializing the EXTERIOR RING */
	  ring = new_pg->Exterior;
	  pt = exterior_ring->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPoint (ring->Coords, iv, pt->X, pt->Y);
		iv++;
		pt = pt->Next;
	    }
	  dyn_rng = dyn_pg->first;
	  while (dyn_rng)
	    {
		/* initializing any INTERIOR RING */
		if (dyn_rng->interior == 0)
		  {
		      dyn_rng = dyn_rng->next;
		      continue;
		  }
		points = kml_count_dyn_points (dyn_rng->ring);
		ring = gaiaAddInteriorRing (new_pg, ib, points);
		ib++;
		pt = dyn_rng->ring->First;
		iv = 0;
		while (pt)
		  {
		      gaiaSetPoint (ring->Coords, iv, pt->X, pt->Y);
		      iv++;
		      pt = pt->Next;
		  }
		dyn_rng = dyn_rng->next;
	    }
      }

    last_g = geom;
    while (1)
      {
	  /* searching the last Geometry within chain */
	  if (last_g->Next == NULL)
	      break;
	  last_g = last_g->Next;
      }
    last_g->Next = pg;
    kml_free_dyn_polygon (dyn_pg);
    return 1;

  error:
    kml_free_dyn_polygon (dyn_pg);
    return 0;
}

static int
kml_parse_multi_geometry (gaiaGeomCollPtr geom, kmlNodePtr node)
{
/* parsing a <MultiGeometry> */
    kmlNodePtr next;
    kmlNodePtr n = node;
    while (n)
      {
	  /* looping on Geometry Members */
	  if (n->Next == NULL)
	    {
		/* verifying the last KML node */
		if (strcmp (n->Tag, "MultiGeometry") == 0)
		    break;
		else
		    return 0;
	    }
	  if (strcmp (n->Tag, "Point") == 0)
	    {
		n = n->Next;
		if (n == NULL)
		    return 0;
		if (!kml_parse_point (geom, n, &next))
		    return 0;
		n = next;
		continue;
	    }
	  else if (strcmp (n->Tag, "LineString") == 0)
	    {
		n = n->Next;
		if (n == NULL)
		    return 0;
		if (!kml_parse_linestring (geom, n, &next))
		    return 0;
		n = next;
		continue;
	    }
	  else if (strcmp (n->Tag, "Polygon") == 0)
	    {
		n = n->Next;
		if (n == NULL)
		    return 0;
		if (!kml_parse_polygon (geom, n, &next))
		    return 0;
		n = next;
		continue;
	    }
	  else
	      return 0;
      }
    return 1;
}

static gaiaGeomCollPtr
kml_validate_geometry (gaiaGeomCollPtr chain)
{
    int xy = 0;
    int xyz = 0;
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaPointPtr save_pt;
    gaiaLinestringPtr save_ln;
    gaiaPolygonPtr save_pg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    int ib;
    gaiaGeomCollPtr g;
    gaiaGeomCollPtr geom;

    g = chain;
    while (g)
      {
	  if (g != chain)
	    {
		if (g->DimensionModel == GAIA_XY)
		    xy++;
		if (g->DimensionModel == GAIA_XY_Z)
		    xyz++;
	    }
	  pt = g->FirstPoint;
	  while (pt)
	    {
		pts++;
		save_pt = pt;
		pt = pt->Next;
	    }
	  ln = g->FirstLinestring;
	  while (ln)
	    {
		lns++;
		save_ln = ln;
		ln = ln->Next;
	    }
	  pg = g->FirstPolygon;
	  while (pg)
	    {
		pgs++;
		save_pg = pg;
		pg = pg->Next;
	    }
	  g = g->Next;
      }
    if (pts == 1 && lns == 0 && pgs == 0)
      {
	  /* POINT */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_MULTIPOINT;
		else
		    geom->DeclaredType = GAIA_POINT;
		gaiaAddPointToGeomColl (geom, save_pt->X, save_pt->Y);
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_MULTIPOINT;
		else
		    geom->DeclaredType = GAIA_POINT;
		gaiaAddPointToGeomCollXYZ (geom, save_pt->X, save_pt->Y,
					   save_pt->Z);
		return geom;
	    }
      }
    if (pts == 0 && lns == 1 && pgs == 0)
      {
	  /* LINESTRING */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
	    }
	  if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      geom->DeclaredType = GAIA_MULTILINESTRING;
	  else
	      geom->DeclaredType = GAIA_LINESTRING;
	  ln = gaiaAddLinestringToGeomColl (geom, save_ln->Points);
	  gaiaCopyLinestringCoords (ln, save_ln);
	  return geom;
      }
    if (pts == 0 && lns == 0 && pgs == 1)
      {
	  /* POLYGON */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
	    }
	  if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      geom->DeclaredType = GAIA_MULTIPOLYGON;
	  else
	      geom->DeclaredType = GAIA_POLYGON;
	  i_ring = save_pg->Exterior;
	  pg = gaiaAddPolygonToGeomColl (geom, i_ring->Points,
					 save_pg->NumInteriors);
	  o_ring = pg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < save_pg->NumInteriors; ib++)
	    {
		i_ring = save_pg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (pg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  return geom;
      }
    if (pts >= 1 && lns == 0 && pgs == 0)
      {
	  /* MULTIPOINT */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->DeclaredType = GAIA_MULTIPOINT;
		g = chain;
		while (g)
		  {
		      pt = g->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomColl (geom, pt->X, pt->Y);
			    pt = pt->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->DeclaredType = GAIA_MULTIPOINT;
		g = chain;
		while (g)
		  {
		      pt = g->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomCollXYZ (geom, pt->X, pt->Y,
						       pt->Z);
			    pt = pt->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
      }
    if (pts == 0 && lns >= 1 && pgs == 0)
      {
	  /* MULTILINESTRING */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->DeclaredType = GAIA_MULTILINESTRING;
		g = chain;
		while (g)
		  {
		      ln = g->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->DeclaredType = GAIA_MULTILINESTRING;
		g = chain;
		while (g)
		  {
		      ln = g->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
      }
    if (pts == 0 && lns == 0 && pgs >= 1)
      {
	  /* MULTIPOLYGON */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->DeclaredType = GAIA_MULTIPOLYGON;
		g = chain;
		while (g)
		  {
		      pg = g->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->DeclaredType = GAIA_MULTIPOLYGON;
		g = chain;
		while (g)
		  {
		      pg = g->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
      }
    if ((pts + lns + pgs) > 0)
      {
	  /* GEOMETRYCOLLECTION */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		g = chain;
		while (g)
		  {
		      pt = g->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomColl (geom, pt->X, pt->Y);
			    pt = pt->Next;
			}
		      ln = g->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      pg = g->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		g = chain;
		while (g)
		  {
		      pt = g->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomCollXYZ (geom, pt->X, pt->Y,
						       pt->Z);
			    pt = pt->Next;
			}
		      ln = g->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      pg = g->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      g = g->Next;
		  }
		return geom;
	    }
      }
    return NULL;
}

static void
kml_free_geom_chain (gaiaGeomCollPtr geom)
{
/* deleting a chain of preliminary geometries */
    gaiaGeomCollPtr gn;
    while (geom)
      {
	  gn = geom->Next;
	  gaiaFreeGeomColl (geom);
	  geom = gn;
      }
}

static gaiaGeomCollPtr
kml_build_geometry (kmlNodePtr tree)
{
/* attempting to build a geometry from KML nodes */
    gaiaGeomCollPtr geom;
    gaiaGeomCollPtr result;
    int geom_type;
    kmlNodePtr next;

    if (tree == NULL)
	return NULL;
    geom_type = guessKmlGeometryType (tree);
    if (geom_type == GAIA_KML_UNKNOWN)
      {
	  /* unsupported main geometry type */
	  return NULL;
      }
/* creating the main geometry */
    geom = gaiaAllocGeomColl ();

    switch (geom_type)
      {
	  /* parsing KML nodes accordingly with declared KML type */
      case GAIA_KML_POINT:
	  geom->DeclaredType = GAIA_POINT;
	  if (!kml_parse_point (geom, tree->Next, &next))
	      goto error;
	  break;
      case GAIA_KML_LINESTRING:
	  geom->DeclaredType = GAIA_LINESTRING;
	  if (!kml_parse_linestring (geom, tree->Next, &next))
	      goto error;
	  break;
      case GAIA_KML_POLYGON:
	  geom->DeclaredType = GAIA_POLYGON;
	  if (!kml_parse_polygon (geom, tree->Next, &next))
	      goto error;
	  if (next != NULL)
	      goto error;
	  break;
      case GAIA_KML_MULTIGEOMETRY:
	  geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
	  if (!kml_parse_multi_geometry (geom, tree->Next))
	      goto error;
	  break;
      };

/* attempting to build the final geometry */
    result = kml_validate_geometry (geom);
    if (result == NULL)
	goto error;
    kml_free_geom_chain (geom);
    return result;

  error:
    kml_free_geom_chain (geom);
    return NULL;
}



/*
** CAVEAT: we must redefine any Lemon/Flex own macro
*/
#define YYMINORTYPE		KML_MINORTYPE
#define YY_CHAR			KML_YY_CHAR
#define	input			kml_input
#define ParseAlloc		kmlParseAlloc
#define ParseFree		kmlParseFree
#define ParseStackPeak		kmlParseStackPeak
#define Parse			kmlParse
#define yyStackEntry		kml_yyStackEntry
#define yyzerominor		kml_yyzerominor
#define yy_accept		kml_yy_accept
#define yy_action		kml_yy_action
#define yy_base			kml_yy_base
#define yy_buffer_stack		kml_yy_buffer_stack
#define yy_buffer_stack_max	kml_yy_buffer_stack_max
#define yy_buffer_stack_top	kml_yy_buffer_stack_top
#define yy_c_buf_p		kml_yy_c_buf_p
#define yy_chk			kml_yy_chk
#define yy_def			kml_yy_def
#define yy_default		kml_yy_default
#define yy_destructor		kml_yy_destructor
#define yy_ec			kml_yy_ec
#define yy_fatal_error		kml_yy_fatal_error
#define yy_find_reduce_action	kml_yy_find_reduce_action
#define yy_find_shift_action	kml_yy_find_shift_action
#define yy_get_next_buffer	kml_yy_get_next_buffer
#define yy_get_previous_state	kml_yy_get_previous_state
#define yy_init			kml_yy_init
#define yy_init_globals		kml_yy_init_globals
#define yy_lookahead		kml_yy_lookahead
#define yy_meta			kml_yy_meta
#define yy_nxt			kml_yy_nxt
#define yy_parse_failed		kml_yy_parse_failed
#define yy_pop_parser_stack	kml_yy_pop_parser_stack
#define yy_reduce		kml_yy_reduce
#define yy_reduce_ofst		kml_yy_reduce_ofst
#define yy_shift		kml_yy_shift
#define yy_shift_ofst		kml_yy_shift_ofst
#define yy_start		kml_yy_start
#define yy_state_type		kml_yy_state_type
#define yy_syntax_error		kml_yy_syntax_error
#define yy_trans_info		kml_yy_trans_info
#define yy_try_NUL_trans	kml_yy_try_NUL_trans
#define yyParser		kml_yyParser
#define yyStackEntry		kml_yyStackEntry
#define yyStackOverflow		kml_yyStackOverflow
#define yyRuleInfo		kml_yyRuleInfo
#define yyunput			kml_yyunput
#define yyzerominor		kml_yyzerominor
#define yyTraceFILE		kml_yyTraceFILE
#define yyTracePrompt		kml_yyTracePrompt
#define yyTokenName		kml_yyTokenName
#define yyRuleName		kml_yyRuleName
#define ParseTrace		kml_ParseTrace


/*
 KML_LEMON_H_START - LEMON generated header starts here 
*/
#define KML_NEWLINE                     1
#define KML_END                         2
#define KML_CLOSE                       3
#define KML_OPEN                        4
#define KML_KEYWORD                     5
#define KML_EQ                          6
#define KML_VALUE                       7
#define KML_COORD                       8
/*
 KML_LEMON_H_END - LEMON generated header ends here 
*/


typedef union
{
    char *pval;
    struct symtab *symp;
} kml_yystype;
#define YYSTYPE kml_yystype


/* extern YYSTYPE yylval; */
YYSTYPE KmlLval;



/*
 KML_LEMON_START - LEMON generated code starts here 
*/

/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is included that follows the "include" declaration
** in the input grammar file. */
/* #include <stdio.h> */

/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash 
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    ParseTOKENTYPE     is the data type used for minor tokens given 
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is ParseTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    ParseARG_SDECL     A static variable declaration for the %extra_argument
**    ParseARG_PDECL     A parameter declaration for the %extra_argument
**    ParseARG_STORE     Code to store %extra_argument into yypParser
**    ParseARG_FETCH     Code to extract %extra_argument from yypParser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 28
#define YYACTIONTYPE unsigned char
#define ParseTOKENTYPE void *
typedef union {
  int yyinit;
  ParseTOKENTYPE yy0;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 1000000
#endif
#define ParseARG_SDECL  kmlNodePtr *result ;
#define ParseARG_PDECL , kmlNodePtr *result 
#define ParseARG_FETCH  kmlNodePtr *result  = yypParser->result 
#define ParseARG_STORE yypParser->result  = result 
#define YYNSTATE 49
#define YYNRULE 34
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* The yyzerominor constant is used to initialize instances of
** YYMINORTYPE objects to zero. */
static const YYMINORTYPE yyzerominor = { 0 };

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
static const YYACTIONTYPE yy_action[] = {
 /*     0 */    20,   28,   29,    4,   48,    5,    3,    3,    5,    5,
 /*    10 */    42,   84,    1,   42,   42,   47,   46,    2,   10,    5,
 /*    20 */    21,   12,   32,   23,   42,   38,   22,    6,   49,   23,
 /*    30 */    13,   19,   14,   15,   35,    8,    8,   10,   25,   11,
 /*    40 */    18,   34,   33,   45,   37,   16,   40,   17,   41,   14,
 /*    50 */     9,   23,   43,    7,   45,   27,   30,   26,   31,   36,
 /*    60 */    39,   44,   24,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */    12,   13,   14,   15,   16,   17,   15,   15,   17,   17,
 /*    10 */    22,   10,   11,   22,   22,   24,   24,   15,   18,   17,
 /*    20 */     2,    3,    8,    5,   22,   25,    2,    3,    0,    5,
 /*    30 */    18,   19,    4,   20,   21,   20,   20,   18,    2,    3,
 /*    40 */     2,   26,   26,    5,   25,   20,   21,   20,   21,    4,
 /*    50 */    18,    5,   23,   20,    5,    1,    3,   23,    3,    7,
 /*    60 */     3,    3,    6,
};
#define YY_SHIFT_USE_DFLT (-1)
#define YY_SHIFT_MAX 26
static const signed char yy_shift_ofst[] = {
 /*     0 */    -1,   28,   45,   45,   45,   18,   14,   14,   14,   46,
 /*    10 */    46,   14,   14,   24,   38,   14,   14,   14,   49,   36,
 /*    20 */    54,   53,   55,   56,   52,   57,   58,
};
#define YY_REDUCE_USE_DFLT (-13)
#define YY_REDUCE_MAX 18
static const signed char yy_reduce_ofst[] = {
 /*     0 */     1,  -12,   -9,   -8,    2,   12,   13,   15,   16,    0,
 /*    10 */    19,   25,   27,   32,   29,   33,   33,   33,   34,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */    50,   83,   72,   72,   54,   83,   60,   80,   80,   76,
 /*    10 */    76,   61,   59,   83,   83,   64,   66,   62,   83,   83,
 /*    20 */    83,   83,   83,   83,   83,   83,   83,   51,   52,   53,
 /*    30 */    56,   57,   79,   81,   82,   65,   75,   77,   78,   58,
 /*    40 */    67,   63,   68,   69,   70,   71,   73,   74,   55,
};
#define YY_SZ_ACTTAB (int)(sizeof(yy_action)/sizeof(yy_action[0]))

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  int yyidx;                    /* Index of top element in stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyidxMax;                 /* Maximum value of yyidx */
#endif
  int yyerrcnt;                 /* Shifts left before out of the error */
  ParseARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
/* #include <stdio.h> */
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void ParseTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "KML_NEWLINE",   "KML_END",       "KML_CLOSE",   
  "KML_OPEN",      "KML_KEYWORD",   "KML_EQ",        "KML_VALUE",   
  "KML_COORD",     "error",         "main",          "in",          
  "state",         "program",       "kml_tree",      "node",        
  "node_chain",    "open_tag",      "attr",          "attributes",  
  "coord",         "coord_chain",   "close_tag",     "keyword",     
  "extra_nodes",   "extra_attr",    "extra_coord", 
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "main ::= in",
 /*   1 */ "in ::=",
 /*   2 */ "in ::= in state KML_NEWLINE",
 /*   3 */ "state ::= program",
 /*   4 */ "program ::= kml_tree",
 /*   5 */ "kml_tree ::= node",
 /*   6 */ "kml_tree ::= node_chain",
 /*   7 */ "node ::= open_tag KML_END KML_CLOSE",
 /*   8 */ "node ::= open_tag attr KML_END KML_CLOSE",
 /*   9 */ "node ::= open_tag attributes KML_END KML_CLOSE",
 /*  10 */ "node ::= open_tag KML_CLOSE",
 /*  11 */ "node ::= open_tag attr KML_CLOSE",
 /*  12 */ "node ::= open_tag attributes KML_CLOSE",
 /*  13 */ "node ::= open_tag KML_CLOSE coord",
 /*  14 */ "node ::= open_tag KML_CLOSE coord_chain",
 /*  15 */ "node ::= open_tag attr KML_CLOSE coord",
 /*  16 */ "node ::= open_tag attr KML_CLOSE coord_chain",
 /*  17 */ "node ::= open_tag attributes KML_CLOSE coord",
 /*  18 */ "node ::= open_tag attributes KML_CLOSE coord_chain",
 /*  19 */ "node ::= close_tag",
 /*  20 */ "open_tag ::= KML_OPEN keyword",
 /*  21 */ "close_tag ::= KML_OPEN KML_END keyword KML_CLOSE",
 /*  22 */ "keyword ::= KML_KEYWORD",
 /*  23 */ "extra_nodes ::=",
 /*  24 */ "extra_nodes ::= node extra_nodes",
 /*  25 */ "node_chain ::= node node extra_nodes",
 /*  26 */ "attr ::= KML_KEYWORD KML_EQ KML_VALUE",
 /*  27 */ "extra_attr ::=",
 /*  28 */ "extra_attr ::= attr extra_attr",
 /*  29 */ "attributes ::= attr attr extra_attr",
 /*  30 */ "coord ::= KML_COORD",
 /*  31 */ "extra_coord ::=",
 /*  32 */ "extra_coord ::= coord extra_coord",
 /*  33 */ "coord_chain ::= coord coord extra_coord",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.
*/
static void yyGrowStack(yyParser *p){
  int newSize;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  if( pNew ){
    p->yystack = pNew;
    p->yystksz = newSize;
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows to %d entries!\n",
              yyTracePrompt, p->yystksz);
    }
#endif
  }
}
#endif

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Parse and ParseFree.
*/
void *ParseAlloc(void *(*mallocProc)(size_t)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (size_t)sizeof(yyParser) );
  if( pParser ){
    pParser->yyidx = -1;
#ifdef YYTRACKMAXSTACKDEPTH
    pParser->yyidxMax = 0;
#endif
#if YYSTACKDEPTH<=0
    pParser->yystack = NULL;
    pParser->yystksz = 0;
    yyGrowStack(pParser);
#endif
  }
  return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  ParseARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are not used
    ** inside the C code.
    */
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(yyParser *pParser){
  YYCODETYPE yymajor;
  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

  if( pParser->yyidx<0 ) return 0;
#ifndef NDEBUG
  if( yyTraceFILE && pParser->yyidx>=0 ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yymajor = yytos->major;
  yy_destructor(pParser, yymajor, &yytos->minor);
  pParser->yyidx--;
  return yymajor;
}

/* 
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from ParseAlloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void ParseFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
  if( pParser==0 ) return;
  while( pParser->yyidx>=0 ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  free(pParser->yystack);
#endif
  (*freeProc)((void*)pParser);
}

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int ParseStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyidxMax;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yystack[pParser->yyidx].stateno;
 
  if( stateno>YY_SHIFT_MAX || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
    return yy_default[stateno];
  }
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    if( iLookAhead>0 ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        return yy_find_shift_action(pParser, iFallback);
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( j>=0 && j<YY_SZ_ACTTAB && yy_lookahead[j]==YYWILDCARD ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
    }
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_MAX ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_MAX );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_SZ_ACTTAB );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser, YYMINORTYPE *yypMinor){
   ParseARG_FETCH;
   yypParser->yyidx--;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */

     fprintf(stderr,"Giving up.  Parser stack overflow\n");
   ParseARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor         /* Pointer to the minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yyidx++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( yypParser->yyidx>yypParser->yyidxMax ){
    yypParser->yyidxMax = yypParser->yyidx;
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yyidx>=YYSTACKDEPTH ){
    yyStackOverflow(yypParser, yypMinor);
    return;
  }
#else
  if( yypParser->yyidx>=yypParser->yystksz ){
    yyGrowStack(yypParser);
    if( yypParser->yyidx>=yypParser->yystksz ){
      yyStackOverflow(yypParser, yypMinor);
      return;
    }
  }
#endif
  yytos = &yypParser->yystack[yypParser->yyidx];
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor = *yypMinor;
#ifndef NDEBUG
  if( yyTraceFILE && yypParser->yyidx>0 ){
    int i;
    fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
    fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
    for(i=1; i<=yypParser->yyidx; i++)
      fprintf(yyTraceFILE," %s",yyTokenName[yypParser->yystack[i].major]);
    fprintf(yyTraceFILE,"\n");
  }
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 10, 1 },
  { 11, 0 },
  { 11, 3 },
  { 12, 1 },
  { 13, 1 },
  { 14, 1 },
  { 14, 1 },
  { 15, 3 },
  { 15, 4 },
  { 15, 4 },
  { 15, 2 },
  { 15, 3 },
  { 15, 3 },
  { 15, 3 },
  { 15, 3 },
  { 15, 4 },
  { 15, 4 },
  { 15, 4 },
  { 15, 4 },
  { 15, 1 },
  { 17, 2 },
  { 22, 4 },
  { 23, 1 },
  { 24, 0 },
  { 24, 2 },
  { 16, 3 },
  { 18, 3 },
  { 25, 0 },
  { 25, 2 },
  { 19, 3 },
  { 20, 1 },
  { 26, 0 },
  { 26, 2 },
  { 21, 3 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  int yyruleno                 /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  ParseARG_FETCH;
  yymsp = &yypParser->yystack[yypParser->yyidx];
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno>=0 
        && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
      yyRuleName[yyruleno]);
  }
#endif /* NDEBUG */

  /* Silence complaints from purify about yygotominor being uninitialized
  ** in some cases when it is copied into the stack after the following
  ** switch.  yygotominor is uninitialized when a rule reduces that does
  ** not set the value of its left-hand side nonterminal.  Leaving the
  ** value of the nonterminal uninitialized is utterly harmless as long
  ** as the value is never used.  So really the only thing this code
  ** accomplishes is to quieten purify.  
  **
  ** 2007-01-16:  The wireshark project (www.wireshark.org) reports that
  ** without this code, their parser segfaults.  I'm not sure what there
  ** parser is doing to make this happen.  This is the second bug report
  ** from wireshark this week.  Clearly they are stressing Lemon in ways
  ** that it has not been previously stressed...  (SQLite ticket #2172)
  */
  /*memset(&yygotominor, 0, sizeof(yygotominor));*/
  yygotominor = yyzerominor;


  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
      case 5: /* kml_tree ::= node */
      case 6: /* kml_tree ::= node_chain */ yytestcase(yyruleno==6);
{ *result = yymsp[0].minor.yy0; }
        break;
      case 7: /* node ::= open_tag KML_END KML_CLOSE */
{ yygotominor.yy0 = kml_createSelfClosedNode((void *)yymsp[-2].minor.yy0, NULL); }
        break;
      case 8: /* node ::= open_tag attr KML_END KML_CLOSE */
      case 9: /* node ::= open_tag attributes KML_END KML_CLOSE */ yytestcase(yyruleno==9);
{ yygotominor.yy0 = kml_createSelfClosedNode((void *)yymsp[-3].minor.yy0, (void *)yymsp[-2].minor.yy0); }
        break;
      case 10: /* node ::= open_tag KML_CLOSE */
{ yygotominor.yy0 = kml_createNode((void *)yymsp[-1].minor.yy0, NULL, NULL); }
        break;
      case 11: /* node ::= open_tag attr KML_CLOSE */
      case 12: /* node ::= open_tag attributes KML_CLOSE */ yytestcase(yyruleno==12);
{ yygotominor.yy0 = kml_createNode((void *)yymsp[-2].minor.yy0, (void *)yymsp[-1].minor.yy0, NULL); }
        break;
      case 13: /* node ::= open_tag KML_CLOSE coord */
      case 14: /* node ::= open_tag KML_CLOSE coord_chain */ yytestcase(yyruleno==14);
{ yygotominor.yy0 = kml_createNode((void *)yymsp[-2].minor.yy0, NULL, (void *)yymsp[0].minor.yy0); }
        break;
      case 15: /* node ::= open_tag attr KML_CLOSE coord */
      case 16: /* node ::= open_tag attr KML_CLOSE coord_chain */ yytestcase(yyruleno==16);
      case 17: /* node ::= open_tag attributes KML_CLOSE coord */ yytestcase(yyruleno==17);
      case 18: /* node ::= open_tag attributes KML_CLOSE coord_chain */ yytestcase(yyruleno==18);
{ yygotominor.yy0 = kml_createNode((void *)yymsp[-3].minor.yy0, (void *)yymsp[-2].minor.yy0, (void *)yymsp[0].minor.yy0); }
        break;
      case 19: /* node ::= close_tag */
{ yygotominor.yy0 = kml_closingNode((void *)yymsp[0].minor.yy0); }
        break;
      case 20: /* open_tag ::= KML_OPEN keyword */
      case 22: /* keyword ::= KML_KEYWORD */ yytestcase(yyruleno==22);
{ yygotominor.yy0 = yymsp[0].minor.yy0; }
        break;
      case 21: /* close_tag ::= KML_OPEN KML_END keyword KML_CLOSE */
{ yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 23: /* extra_nodes ::= */
      case 27: /* extra_attr ::= */ yytestcase(yyruleno==27);
      case 31: /* extra_coord ::= */ yytestcase(yyruleno==31);
{ yygotominor.yy0 = NULL; }
        break;
      case 24: /* extra_nodes ::= node extra_nodes */
{ ((kmlNodePtr)yymsp[-1].minor.yy0)->Next = (kmlNodePtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 25: /* node_chain ::= node node extra_nodes */
{ 
	   ((kmlNodePtr)yymsp[-1].minor.yy0)->Next = (kmlNodePtr)yymsp[0].minor.yy0; 
	   ((kmlNodePtr)yymsp[-2].minor.yy0)->Next = (kmlNodePtr)yymsp[-1].minor.yy0;
	   yygotominor.yy0 = yymsp[-2].minor.yy0;
	}
        break;
      case 26: /* attr ::= KML_KEYWORD KML_EQ KML_VALUE */
{ yygotominor.yy0 = kml_attribute((void *)yymsp[-2].minor.yy0, (void *)yymsp[0].minor.yy0); }
        break;
      case 28: /* extra_attr ::= attr extra_attr */
{ ((kmlAttrPtr)yymsp[-1].minor.yy0)->Next = (kmlAttrPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 29: /* attributes ::= attr attr extra_attr */
{ 
	   ((kmlAttrPtr)yymsp[-1].minor.yy0)->Next = (kmlAttrPtr)yymsp[0].minor.yy0; 
	   ((kmlAttrPtr)yymsp[-2].minor.yy0)->Next = (kmlAttrPtr)yymsp[-1].minor.yy0;
	   yygotominor.yy0 = yymsp[-2].minor.yy0;
	}
        break;
      case 30: /* coord ::= KML_COORD */
{ yygotominor.yy0 = kml_coord((void *)yymsp[0].minor.yy0); }
        break;
      case 32: /* extra_coord ::= coord extra_coord */
{ ((kmlCoordPtr)yymsp[-1].minor.yy0)->Next = (kmlCoordPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 33: /* coord_chain ::= coord coord extra_coord */
{ 
	   ((kmlCoordPtr)yymsp[-1].minor.yy0)->Next = (kmlCoordPtr)yymsp[0].minor.yy0; 
	   ((kmlCoordPtr)yymsp[-2].minor.yy0)->Next = (kmlCoordPtr)yymsp[-1].minor.yy0;
	   yygotominor.yy0 = yymsp[-2].minor.yy0;
	}
        break;
      default:
      /* (0) main ::= in */ yytestcase(yyruleno==0);
      /* (1) in ::= */ yytestcase(yyruleno==1);
      /* (2) in ::= in state KML_NEWLINE */ yytestcase(yyruleno==2);
      /* (3) state ::= program */ yytestcase(yyruleno==3);
      /* (4) program ::= kml_tree */ yytestcase(yyruleno==4);
        break;
  };
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yypParser->yyidx -= yysize;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact < YYNSTATE ){
#ifdef NDEBUG
    /* If we are not debugging and the reduce action popped at least
    ** one element off the stack, then we can push the new element back
    ** onto the stack here, and skip the stack overflow test in yy_shift().
    ** That gives a significant speed improvement. */
    if( yysize ){
      yypParser->yyidx++;
      yymsp -= yysize-1;
      yymsp->stateno = (YYACTIONTYPE)yyact;
      yymsp->major = (YYCODETYPE)yygoto;
      yymsp->minor = yygotominor;
    }else
#endif
    {
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
    }
  }else{
    assert( yyact == YYNSTATE + YYNRULE + 1 );
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  YYMINORTYPE yyminor            /* The minor type of the error token */
){
  ParseARG_FETCH;
#define TOKEN (yyminor.yy0)

/* 
** when the LEMON parser encounters an error
** then this global variable is set 
*/
	kml_parse_error = 1;
	*result = NULL;
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "ParseAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void Parse(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  ParseTOKENTYPE yyminor       /* The value for the token */
  ParseARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  int yyact;            /* The parser action. */
  int yyendofinput;     /* True if we are at the end of input */
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  /* (re)initialize the parser, if necessary */
  yypParser = (yyParser*)yyp;
  if( yypParser->yyidx<0 ){
#if YYSTACKDEPTH<=0
    if( yypParser->yystksz <=0 ){
      /*memset(&yyminorunion, 0, sizeof(yyminorunion));*/
      yyminorunion = yyzerominor;
      yyStackOverflow(yypParser, &yyminorunion);
      return;
    }
#endif
    yypParser->yyidx = 0;
    yypParser->yyerrcnt = -1;
    yypParser->yystack[0].stateno = 0;
    yypParser->yystack[0].major = 0;
  }
  yyminorunion.yy0 = yyminor;
  yyendofinput = (yymajor==0);
  ParseARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact<YYNSTATE ){
      assert( !yyendofinput );  /* Impossible to shift the $ token */
      yy_shift(yypParser,yyact,yymajor,&yyminorunion);
      yypParser->yyerrcnt--;
      yymajor = YYNOCODE;
    }else if( yyact < YYNSTATE + YYNRULE ){
      yy_reduce(yypParser,yyact-YYNSTATE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yymx = yypParser->yystack[yypParser->yyidx].major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor,&yyminorunion);
        yymajor = YYNOCODE;
      }else{
         while(
          yypParser->yyidx >= 0 &&
          yymx != YYERRORSYMBOL &&
          (yyact = yy_find_reduce_action(
                        yypParser->yystack[yypParser->yyidx].stateno,
                        YYERRORSYMBOL)) >= YYNSTATE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yyidx < 0 || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          YYMINORTYPE u2;
          u2.YYERRSYMDT = 0;
          yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor,yyminorunion);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;
      
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
  return;
}


/*
 KML_LEMON_END - LEMON generated code ends here 
*/

















/*
** CAVEAT: there is an incompatibility between LEMON and FLEX
** this macro resolves the issue
*/
#undef yy_accept
#define yy_accept	yy_kml_flex_accept



/*
 KML_FLEX_START - FLEX generated code starts here 
*/

#line 3 "lex.Kml.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer Kml_create_buffer
#define yy_delete_buffer Kml_delete_buffer
#define yy_flex_debug Kml_flex_debug
#define yy_init_buffer Kml_init_buffer
#define yy_flush_buffer Kml_flush_buffer
#define yy_load_buffer_state Kml_load_buffer_state
#define yy_switch_to_buffer Kml_switch_to_buffer
#define yyin Kmlin
#define yyleng Kmlleng
#define yylex Kmllex
#define yylineno Kmllineno
#define yyout Kmlout
#define yyrestart Kmlrestart
#define yytext Kmltext
#define yywrap Kmlwrap
#define yyalloc Kmlalloc
#define yyrealloc Kmlrealloc
#define yyfree Kmlfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <errno.h> */
/* #include <stdlib.h> */

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE Kmlrestart(Kmlin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int Kmlleng;

extern FILE *Kmlin, *Kmlout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up Kmltext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up Kmltext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via Kmlrestart()), so that the user can continue scanning by
	 * just pointing Kmlin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when Kmltext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int Kmlleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow Kmlwrap()'s to do buffer switches
 * instead of setting up a fresh Kmlin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void Kmlrestart (FILE *input_file  );
void Kml_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE Kml_create_buffer (FILE *file,int size  );
void Kml_delete_buffer (YY_BUFFER_STATE b  );
void Kml_flush_buffer (YY_BUFFER_STATE b  );
void Kmlpush_buffer_state (YY_BUFFER_STATE new_buffer  );
void Kmlpop_buffer_state (void );

static void Kmlensure_buffer_stack (void );
static void Kml_load_buffer_state (void );
static void Kml_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER Kml_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE Kml_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE Kml_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE Kml_scan_bytes (yyconst char *bytes,int len  );

void *Kmlalloc (yy_size_t  );
void *Kmlrealloc (void *,yy_size_t  );
void Kmlfree (void *  );

#define yy_new_buffer Kml_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        Kmlensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            Kml_create_buffer(Kmlin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        Kmlensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            Kml_create_buffer(Kmlin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *Kmlin = (FILE *) 0, *Kmlout = (FILE *) 0;

typedef int yy_state_type;

extern int Kmllineno;

int Kmllineno = 1;

extern char *Kmltext;
#define yytext_ptr Kmltext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up Kmltext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	Kmlleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 11
#define YY_END_OF_BUFFER 12
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[19] =
    {   0,
        5,    5,   12,   10,    8,    9,   10,    5,    1,    3,
        2,    4,    7,    0,    6,    5,    7,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    4,    1,    1,    1,    1,    1,    1,
        1,    1,    5,    5,    5,    5,    6,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    8,    1,    9,
       10,   11,    1,    1,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
        1,    1,    1,    1,   12,    1,   12,   12,   12,   12,

       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[13] =
    {   0,
        1,    1,    1,    1,    2,    1,    3,    4,    5,    1,
        5,    4
    } ;

static yyconst flex_int16_t yy_base[22] =
    {   0,
        0,    0,   23,   24,   24,   24,   18,    0,   24,   24,
       24,   24,    0,   17,   24,    0,    0,   24,   12,   15,
       16
    } ;

static yyconst flex_int16_t yy_def[22] =
    {   0,
       18,    1,   18,   18,   18,   18,   19,   20,   18,   18,
       18,   18,   21,   19,   18,   20,   21,    0,   18,   18,
       18
    } ;

static yyconst flex_int16_t yy_nxt[37] =
    {   0,
        4,    5,    6,    7,    8,    9,    8,    4,   10,   11,
       12,   13,   14,   14,   14,   14,   16,   16,   17,   17,
       15,   15,   18,    3,   18,   18,   18,   18,   18,   18,
       18,   18,   18,   18,   18,   18
    } ;

static yyconst flex_int16_t yy_chk[37] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,   19,   19,   19,   19,   20,   20,   21,   21,
       14,    7,    3,   18,   18,   18,   18,   18,   18,   18,
       18,   18,   18,   18,   18,   18
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int Kml_flex_debug;
int Kml_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *Kmltext;
/* 
 kmlLexer.l -- KML parser - FLEX config
  
 version 2.4, 2011 June 14

 Author: Sandro Furieri a.furieri@lqt.it

 ------------------------------------------------------------------------------
 
 Version: MPL 1.1/GPL 2.0/LGPL 2.1
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the SpatiaLite library

The Initial Developer of the Original Code is Alessandro Furieri
 
Portions created by the Initial Developer are Copyright (C) 2011
the Initial Developer. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of
either the GNU General Public License Version 2 or later (the "GPL"), or
the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
in which case the provisions of the GPL or the LGPL are applicable instead
of those above. If you wish to allow use of your version of this file only
under the terms of either the GPL or the LGPL, and not to allow others to
use your version of this file under the terms of the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the GPL or the LGPL. If you do not delete
the provisions above, a recipient may use your version of this file under
the terms of any one of the MPL, the GPL or the LGPL.
 
*/

/* For debugging purposes */
int kml_line = 1, kml_col = 1;

/**
*  The main string-token matcher.
*  The lower case part is probably not needed.  We should really be converting 
*  The string to all uppercase/lowercase to make it case iNsEnSiTiVe.
*  What Flex will do is, For the input string, beginning from the front, Flex
*  will try to match with any of the defined tokens from below.  Flex will 
*  then match the string of longest length.  Suppose the string is: POINTM,
*  Flex would match both POINT and POINTM, but since POINTM is the longer
*  of the two tokens, FLEX will match POINTM.
*/

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* #include <unistd.h> */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int Kmllex_destroy (void );

int Kmlget_debug (void );

void Kmlset_debug (int debug_flag  );

YY_EXTRA_TYPE Kmlget_extra (void );

void Kmlset_extra (YY_EXTRA_TYPE user_defined  );

FILE *Kmlget_in (void );

void Kmlset_in  (FILE * in_str  );

FILE *Kmlget_out (void );

void Kmlset_out  (FILE * out_str  );

int Kmlget_leng (void );

char *Kmlget_text (void );

int Kmlget_lineno (void );

void Kmlset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int Kmlwrap (void );
#else
extern int Kmlwrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( Kmltext, Kmlleng, 1, Kmlout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( Kmlin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( Kmlin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, Kmlin))==0 && ferror(Kmlin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(Kmlin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int Kmllex (void);

#define YY_DECL int Kmllex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after Kmltext and Kmlleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! Kmlin )
			Kmlin = stdin;

		if ( ! Kmlout )
			Kmlout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			Kmlensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				Kml_create_buffer(Kmlin,YY_BUF_SIZE );
		}

		Kml_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of Kmltext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 19 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 24 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
{ kml_freeString(&(KmlLval.pval)); return KML_END; }
	YY_BREAK
case 2:
YY_RULE_SETUP
{ kml_freeString(&(KmlLval.pval)); return KML_EQ; }
	YY_BREAK
case 3:
YY_RULE_SETUP
{ kml_freeString(&(KmlLval.pval)); return KML_OPEN; }
	YY_BREAK
case 4:
YY_RULE_SETUP
{ kml_freeString(&(KmlLval.pval)); return KML_CLOSE; }
	YY_BREAK
case 5:
YY_RULE_SETUP
{ kml_saveString(&(KmlLval.pval), Kmltext); return KML_COORD; }
	YY_BREAK
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
{ kml_saveString(&(KmlLval.pval), Kmltext); return KML_VALUE; }
	YY_BREAK
case 7:
YY_RULE_SETUP
{ kml_saveString(&(KmlLval.pval), Kmltext); return KML_KEYWORD; }
	YY_BREAK
case 8:
YY_RULE_SETUP
{ kml_freeString(&(KmlLval.pval)); kml_col += (int) strlen(Kmltext); }               /* ignore but count white space */
	YY_BREAK
case 9:
/* rule 9 can match eol */
YY_RULE_SETUP
{ kml_freeString(&(KmlLval.pval)); kml_col = 0; ++kml_line; }
	YY_BREAK
case 10:
YY_RULE_SETUP
{ kml_freeString(&(KmlLval.pval)); kml_col += (int) strlen(Kmltext); return -1; }
	YY_BREAK
case 11:
YY_RULE_SETUP
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed Kmlin at a new source and called
			 * Kmllex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = Kmlin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( Kmlwrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * Kmltext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of Kmllex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					Kmlrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			Kmlrestart(Kmlin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) Kmlrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 19 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 19 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 18);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up Kmltext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					Kmlrestart(Kmlin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( Kmlwrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve Kmltext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void Kmlrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        Kmlensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            Kml_create_buffer(Kmlin,YY_BUF_SIZE );
	}

	Kml_init_buffer(YY_CURRENT_BUFFER,input_file );
	Kml_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void Kml_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		Kmlpop_buffer_state();
	 *		Kmlpush_buffer_state(new_buffer);
     */
	Kmlensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	Kml_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (Kmlwrap()) processing, but the only time this flag
	 * is looked at is after Kmlwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void Kml_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	Kmlin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE Kml_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) Kmlalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in Kml_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) Kmlalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in Kml_create_buffer()" );

	b->yy_is_our_buffer = 1;

	Kml_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with Kml_create_buffer()
 * 
 */
    void Kml_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		Kmlfree((void *) b->yy_ch_buf  );

	Kmlfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a Kmlrestart() or at EOF.
 */
    static void Kml_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	Kml_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then Kml_init_buffer was _probably_
     * called from Kmlrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void Kml_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		Kml_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void Kmlpush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	Kmlensure_buffer_stack();

	/* This block is copied from Kml_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from Kml_switch_to_buffer. */
	Kml_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void Kmlpop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	Kml_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		Kml_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void Kmlensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)Kmlalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in Kmlensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)Kmlrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in Kmlensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE Kml_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) Kmlalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in Kml_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	Kml_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to Kmllex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       Kml_scan_bytes() instead.
 */
YY_BUFFER_STATE Kml_scan_string (yyconst char * yystr )
{
    
	return Kml_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to Kmllex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE Kml_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) Kmlalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in Kml_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = Kml_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in Kml_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up Kmltext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		Kmltext[Kmlleng] = (yy_hold_char); \
		(yy_c_buf_p) = Kmltext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		Kmlleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int Kmlget_lineno  (void)
{
        
    return Kmllineno;
}

/** Get the input stream.
 * 
 */
FILE *Kmlget_in  (void)
{
        return Kmlin;
}

/** Get the output stream.
 * 
 */
FILE *Kmlget_out  (void)
{
        return Kmlout;
}

/** Get the length of the current token.
 * 
 */
int Kmlget_leng  (void)
{
        return Kmlleng;
}

/** Get the current token.
 * 
 */

char *Kmlget_text  (void)
{
        return Kmltext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void Kmlset_lineno (int  line_number )
{
    
    Kmllineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see Kml_switch_to_buffer
 */
void Kmlset_in (FILE *  in_str )
{
        Kmlin = in_str ;
}

void Kmlset_out (FILE *  out_str )
{
        Kmlout = out_str ;
}

int Kmlget_debug  (void)
{
        return Kml_flex_debug;
}

void Kmlset_debug (int  bdebug )
{
        Kml_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from Kmllex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    Kmlin = stdin;
    Kmlout = stdout;
#else
    Kmlin = (FILE *) 0;
    Kmlout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * Kmllex_init()
     */
    return 0;
}

/* Kmllex_destroy is for both reentrant and non-reentrant scanners. */
int Kmllex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		Kml_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		Kmlpop_buffer_state();
	}

	/* Destroy the stack itself. */
	Kmlfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * Kmllex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *Kmlalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *Kmlrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void Kmlfree (void * ptr )
{
	free( (char *) ptr );	/* see Kmlrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

/**
 * reset the line and column count
 *
 *
 */
void kml_reset_lexer(void)
{

  kml_line = 1;
  kml_col  = 1;

}

/**
 * kmlError() is invoked when the lexer or the parser encounter
 * an error. The error message is passed via *s
 *
 *
 */
void KmlError(char *s)
{
  printf("error: %s at line: %d col: %d\n",s,kml_line,kml_col);

}

int Kmlwrap(void)
{
  return 1;
}


/* 
 KML_FLEX_END - FLEX generated code ends here 
*/



gaiaGeomCollPtr
gaiaParseKml (const unsigned char *dirty_buffer)
{
    void *pParser = ParseAlloc (malloc);
    /* Linked-list of token values */
    kmlFlexToken *tokens = malloc (sizeof (kmlFlexToken));
    /* Pointer to the head of the list */
    kmlFlexToken *head = tokens;
    int yv;
    kmlNodePtr result = NULL;
    gaiaGeomCollPtr geom = NULL;

    KmlLval.pval = NULL;
    tokens->value = NULL;
    tokens->Next = NULL;
    kml_parse_error = 0;
    Kml_scan_string ((char *) dirty_buffer);

    /*
       / Keep tokenizing until we reach the end
       / yylex() will return the next matching Token for us.
     */
    while ((yv = yylex ()) != 0)
      {
	  if (yv == -1)
	    {
		kml_parse_error = 1;
		break;
	    }
	  tokens->Next = malloc (sizeof (kmlFlexToken));
	  tokens->Next->Next = NULL;
	  /*
	     /KmlLval is a global variable from FLEX.
	     /KmlLval is defined in kmlLexglobal.h
	   */
	  kml_xferString (&(tokens->Next->value), KmlLval.pval);
	  /* Pass the token to the wkt parser created from lemon */
	  Parse (pParser, yv, &(tokens->Next->value), &result);
	  tokens = tokens->Next;
      }
    /* This denotes the end of a line as well as the end of the parser */
    Parse (pParser, KML_NEWLINE, 0, &result);
    ParseFree (pParser, free);
    Kmllex_destroy ();

    /* Assigning the token as the end to avoid seg faults while cleaning */
    tokens->Next = NULL;
    kml_cleanup (head);
    kml_freeString (&(KmlLval.pval));

    if (kml_parse_error)
      {
	  if (result)
	      kml_freeTree (result);
	  return NULL;
      }

    /* attempting to build a geometry from KML */
    geom = kml_build_geometry (result);
    geom->Srid = 4326;
    kml_freeTree (result);
    return geom;
}


/*
** CAVEAT: we must now undefine any Lemon/Flex own macro
*/
#undef YYNOCODE
#undef YYNSTATE
#undef YYNRULE
#undef YY_SHIFT_MAX
#undef YY_REDUCE_USE_DFLT
#undef YY_REDUCE_MAX
#undef YY_FLUSH_BUFFER
#undef YY_DO_BEFORE_ACTION
#undef YY_NUM_RULES
#undef YY_END_OF_BUFFER
#undef YY_END_FILE
#undef YYACTIONTYPE
#undef YY_SZ_ACTTAB
#undef YY_NEW_FILE
#undef BEGIN
#undef YY_START
#undef YY_CURRENT_BUFFER
#undef YY_CURRENT_BUFFER_LVALUE
#undef YY_STATE_BUF_SIZE
#undef YY_DECL
#undef YY_FATAL_ERROR
#undef YYMINORTYPE
#undef YY_CHAR
#undef YYSTYPE
#undef input
#undef ParseAlloc
#undef ParseFree
#undef ParseStackPeak
#undef Parse
#undef yyalloc
#undef yyfree
#undef yyin
#undef yyleng
#undef yyless
#undef yylex
#undef yylineno
#undef yyout
#undef yyrealloc
#undef yyrestart
#undef yyStackEntry
#undef yytext
#undef yywrap
#undef yyzerominor
#undef yy_accept
#undef yy_action
#undef yy_base
#undef yy_buffer_stack
#undef yy_buffer_stack_max
#undef yy_buffer_stack_top
#undef yy_c_buf_p
#undef yy_chk
#undef yy_create_buffer
#undef yy_def
#undef yy_default
#undef yy_delete_buffer
#undef yy_destructor
#undef yy_ec
#undef yy_fatal_error
#undef yy_find_reduce_action
#undef yy_find_shift_action
#undef yy_flex_debug
#undef yy_flush_buffer
#undef yy_get_next_buffer
#undef yy_get_previous_state
#undef yy_init
#undef yy_init_buffer
#undef yy_init_globals
#undef yy_load_buffer
#undef yy_load_buffer_state
#undef yy_lookahead
#undef yy_meta
#undef yy_new_buffer
#undef yy_nxt
#undef yy_parse_failed
#undef yy_pop_parser_stack
#undef yy_reduce
#undef yy_reduce_ofst
#undef yy_set_bol
#undef yy_set_interactive
#undef yy_shift
#undef yy_shift_ofst
#undef yy_start
#undef yy_state_type
#undef yy_switch_to_buffer
#undef yy_syntax_error
#undef yy_trans_info
#undef yy_try_NUL_trans
#undef yyParser
#undef yyStackEntry
#undef yyStackOverflow
#undef yyRuleInfo
#undef yytext_ptr
#undef yyunput
#undef yyzerominor
#undef ParseARG_SDECL
#undef ParseARG_PDECL
#undef ParseARG_FETCH
#undef ParseARG_STORE
#undef REJECT
#undef yymore
#undef YY_MORE_ADJ
#undef YY_RESTORE_YY_MORE_OFFSET
#undef YY_LESS_LINENO
#undef yyTracePrompt
#undef yyTraceFILE
#undef yyTokenName
#undef yyRuleName
#undef ParseTrace
/**************** End file: gg_kml.c **********/


/**************** Begin file: gg_gml.c **********/

/* #include <sys/types.h> */
/* #include <stdlib.h> */
/* #include <stdio.h> */
/* #include <string.h> */

/* #include <assert.h> */

#ifdef SPL_AMALGAMATION		/* spatialite-amalgamation */
/* #include <spatialite/sqlite3.h> */
#else
/* #include <sqlite3.h> */
#endif

/* #include <spatialite/gaiageo.h> */

#if defined(_WIN32) || defined(WIN32)
# include <io.h>
#define isatty  _isatty
#define fileno  _fileno
#endif

int gml_parse_error;

#define GML_PARSER_OPEN_NODE		1
#define GML_PARSER_SELF_CLOSED_NODE	2
#define GML_PARSER_CLOSED_NODE		3

#define GAIA_GML_UNKNOWN		0
#define GAIA_GML_POINT			1
#define GAIA_GML_LINESTRING		2
#define GAIA_GML_CURVE			3
#define GAIA_GML_POLYGON		4
#define GAIA_GML_MULTIPOINT		5
#define GAIA_GML_MULTILINESTRING	6
#define GAIA_GML_MULTICURVE		7
#define GAIA_GML_MULTIPOLYGON		8
#define GAIA_GML_MULTISURFACE		9
#define GAIA_GML_MULTIGEOMETRY		10

/*
** This is a linked-list struct to store all the values for each token.
*/
typedef struct gmlFlexTokenStruct
{
    char *value;
    struct gmlFlexTokenStruct *Next;
} gmlFlexToken;

typedef struct gml_coord
{
    char *Value;
    struct gml_coord *Next;
} gmlCoord;
typedef gmlCoord *gmlCoordPtr;

typedef struct gml_attr
{
    char *Key;
    char *Value;
    struct gml_attr *Next;
} gmlAttr;
typedef gmlAttr *gmlAttrPtr;

typedef struct gml_node
{
    char *Tag;
    int Type;
    int Error;
    struct gml_attr *Attributes;
    struct gml_coord *Coordinates;
    struct gml_node *Next;
} gmlNode;
typedef gmlNode *gmlNodePtr;

typedef struct gml_dynamic_ring
{
    gaiaDynamicLinePtr ring;
    int interior;
    int has_z;
    struct gml_dynamic_ring *next;
} gmlDynamicRing;
typedef gmlDynamicRing *gmlDynamicRingPtr;

typedef struct gml_dynamic_polygon
{
    struct gml_dynamic_ring *first;
    struct gml_dynamic_ring *last;
} gmlDynamicPolygon;
typedef gmlDynamicPolygon *gmlDynamicPolygonPtr;

static void
gml_proj_params (sqlite3 * sqlite, int srid, char *proj_params)
{
/* retrives the PROJ params from SPATIAL_SYS_REF table, if possible */
    char sql[256];
    char **results;
    int rows;
    int columns;
    int i;
    int ret;
    char *errMsg = NULL;
    *proj_params = '\0';
    sprintf (sql,
	     "SELECT proj4text FROM spatial_ref_sys WHERE srid = %d", srid);
    ret = sqlite3_get_table (sqlite, sql, &results, &rows, &columns, &errMsg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "unknown SRID: %d\t<%s>\n", srid, errMsg);
	  sqlite3_free (errMsg);
	  return;
      }
    for (i = 1; i <= rows; i++)
	strcpy (proj_params, results[(i * columns)]);
    if (*proj_params == '\0')
	fprintf (stderr, "unknown SRID: %d\n", srid);
    sqlite3_free_table (results);
}

static gmlDynamicPolygonPtr
gml_alloc_dyn_polygon (void)
{
/* creating a dynamic polygon (ring collection) */
    gmlDynamicPolygonPtr p = malloc (sizeof (gmlDynamicPolygon));
    p->first = NULL;
    p->last = NULL;
    return p;
}

static void
gml_free_dyn_polygon (gmlDynamicPolygonPtr dyn)
{
/* deleting a dynamic polygon (ring collection) */
    gmlDynamicRingPtr r;
    gmlDynamicRingPtr rn;
    if (!dyn)
	return;
    r = dyn->first;
    while (r)
      {
	  rn = r->next;
	  if (r->ring)
	      gaiaFreeDynamicLine (r->ring);
	  free (r);
	  r = rn;
      }
    free (dyn);
}

static void
gml_add_polygon_ring (gmlDynamicPolygonPtr dyn_pg, gaiaDynamicLinePtr dyn,
		      int interior, int has_z)
{
/* inserting a further ring into the collection (dynamic polygon) */
    gmlDynamicRingPtr p = malloc (sizeof (gmlDynamicRing));
    p->ring = dyn;
    p->interior = interior;
    p->has_z = has_z;
    p->next = NULL;
    if (dyn_pg->first == NULL)
	dyn_pg->first = p;
    if (dyn_pg->last != NULL)
	dyn_pg->last->next = p;
    dyn_pg->last = p;
}

static void
gml_freeString (char **ptr)
{
/* releasing a string from the lexer */
    if (*ptr != NULL)
	free (*ptr);
    *ptr = NULL;
}

static void
gml_saveString (char **ptr, const char *str)
{
/* saving a string from the lexer */
    int len = strlen (str);
    gml_freeString (ptr);
    *ptr = malloc (len + 1);
    strcpy (*ptr, str);
}

static gmlCoordPtr
gml_coord (void *value)
{
/* creating a coord Item */
    int len;
    gmlFlexToken *tok = (gmlFlexToken *) value;
    gmlCoordPtr c = malloc (sizeof (gmlCoord));
    len = strlen (tok->value);
    c->Value = malloc (len + 1);
    strcpy (c->Value, tok->value);
    c->Next = NULL;
    return c;
}

static void
gml_freeCoordinate (gmlCoordPtr c)
{
/* deleting a GML coordinate */
    if (c == NULL)
	return;
    if (c->Value)
	free (c->Value);
    free (c);
}

static gmlAttrPtr
gml_attribute (void *key, void *value)
{
/* creating an attribute */
    int len;
    gmlFlexToken *k_tok = (gmlFlexToken *) key;
    gmlFlexToken *v_tok = (gmlFlexToken *) value;
    gmlAttrPtr a = malloc (sizeof (gmlAttr));
    len = strlen (k_tok->value);
    a->Key = malloc (len + 1);
    strcpy (a->Key, k_tok->value);
    len = strlen (v_tok->value);
/* we need to de-quote the string, removing first and last ".." */
    if (*(v_tok->value + 0) == '"' && *(v_tok->value + len - 1) == '"')
      {
	  a->Value = malloc (len - 1);
	  memcpy (a->Value, v_tok->value + 1, len - 1);
	  *(a->Value + len - 1) = '\0';
      }
    else
      {
	  a->Value = malloc (len + 1);
	  strcpy (a->Value, v_tok->value);
      }
    a->Next = NULL;
    return a;
}

static void
gml_freeAttribute (gmlAttrPtr a)
{
/* deleting a GML attribute */
    if (a == NULL)
	return;
    if (a->Key)
	free (a->Key);
    if (a->Value)
	free (a->Value);
    free (a);
}

static void
gml_freeNode (gmlNodePtr n)
{
/* deleting a GML node */
    gmlAttrPtr a;
    gmlAttrPtr an;
    gmlCoordPtr c;
    gmlCoordPtr cn;
    if (n == NULL)
	return;
    a = n->Attributes;
    while (a)
      {
	  an = a->Next;
	  gml_freeAttribute (a);
	  a = an;
      }
    c = n->Coordinates;
    while (c)
      {
	  cn = c->Next;
	  gml_freeCoordinate (c);
	  c = cn;
      }
    if (n->Tag)
	free (n->Tag);
    free (n);
}

static void
gml_freeTree (gmlNodePtr t)
{
/* deleting a GML tree */
    gmlNodePtr n;
    gmlNodePtr nn;
    n = t;
    while (n)
      {
	  nn = n->Next;
	  gml_freeNode (n);
	  n = nn;
      }
}

static gmlNodePtr
gml_createNode (void *tag, void *attributes, void *coords)
{
/* creating a node */
    int len;
    gmlFlexToken *tok = (gmlFlexToken *) tag;
    gmlNodePtr n = malloc (sizeof (gmlNode));
    len = strlen (tok->value);
    n->Tag = malloc (len + 1);
    strcpy (n->Tag, tok->value);
    n->Type = GML_PARSER_OPEN_NODE;
    n->Error = 0;
    n->Attributes = attributes;
    n->Coordinates = coords;
    n->Next = NULL;
    return n;
}

static gmlNodePtr
gml_createSelfClosedNode (void *tag, void *attributes)
{
/* creating a self-closed node */
    int len;
    gmlFlexToken *tok = (gmlFlexToken *) tag;
    gmlNodePtr n = malloc (sizeof (gmlNode));
    len = strlen (tok->value);
    n->Tag = malloc (len + 1);
    strcpy (n->Tag, tok->value);
    n->Type = GML_PARSER_SELF_CLOSED_NODE;
    n->Error = 0;
    n->Attributes = attributes;
    n->Coordinates = NULL;
    n->Next = NULL;
    return n;
}

static gmlNodePtr
gml_closingNode (void *tag)
{
/* creating a closing node */
    int len;
    gmlFlexToken *tok = (gmlFlexToken *) tag;
    gmlNodePtr n = malloc (sizeof (gmlNode));
    len = strlen (tok->value);
    n->Tag = malloc (len + 1);
    strcpy (n->Tag, tok->value);
    n->Type = GML_PARSER_CLOSED_NODE;
    n->Error = 0;
    n->Attributes = NULL;
    n->Coordinates = NULL;
    n->Next = NULL;
    return n;
}

static int
gml_cleanup (gmlFlexToken * token)
{
    gmlFlexToken *ptok;
    gmlFlexToken *ptok_n;
    if (token == NULL)
	return 0;
    ptok = token;
    while (ptok)
      {
	  ptok_n = ptok->Next;
	  if (ptok->value != NULL)
	      free (ptok->value);
	  free (ptok);
	  ptok = ptok_n;
      }
    return 0;
}

static void
gml_xferString (char **p, const char *str)
{
/* saving some token */
    int len;
    if (str == NULL)
      {
	  *p = NULL;
	  return;
      }
    len = strlen (str);
    *p = malloc (len + 1);
    strcpy (*p, str);
}

static int
guessGmlSrid (gmlNodePtr node)
{
/* attempting to guess the SRID */
    int len;
    gmlAttrPtr attr = node->Attributes;
    while (attr)
      {
	  if (strcmp (attr->Key, "srsName") == 0)
	    {
		len = strlen (attr->Value);
		if (len > 5)
		  {
		      if (strncmp (attr->Value, "EPSG:", 5) == 0)
			  return atoi (attr->Value + 5);
		  }
		if (len > 21)
		  {
		      if (strncmp (attr->Value, "urn:ogc:def:crs:EPSG:", 21) ==
			  0)
			{
			    int i = strlen (attr->Value) - 1;
			    for (; i >= 0; i--)
			      {
				  if (*(attr->Value + i) == ':')
				      return atoi (attr->Value + i + 1);
			      }
			}
		  }
	    }
	  attr = attr->Next;
      }
    return -1;
}

static int
gml_get_srsDimension (gmlNodePtr node)
{
/* attempting to establis if there is a Z coordinate */
    gmlAttrPtr attr = node->Attributes;
    while (attr)
      {
	  if (strcmp (attr->Key, "srsDimension") == 0)
	    {
		if (atoi (attr->Value) == 3)
		    return 1;
		else
		    return 0;
	    }
	  attr = attr->Next;
      }
    return 0;
}

static int
guessGmlGeometryType (gmlNodePtr node)
{
/* attempting to guess the Geometry Type for a GML node */
    int type = GAIA_GML_UNKNOWN;
    if (strcmp (node->Tag, "gml:Point") == 0
	|| strcmp (node->Tag, "Point") == 0)
	type = GAIA_GML_POINT;
    if (strcmp (node->Tag, "gml:LineString") == 0
	|| strcmp (node->Tag, "LineString") == 0)
	type = GAIA_GML_LINESTRING;
    if (strcmp (node->Tag, "gml:Curve") == 0
	|| strcmp (node->Tag, "Curve") == 0)
	type = GAIA_GML_CURVE;
    if (strcmp (node->Tag, "gml:Polygon") == 0
	|| strcmp (node->Tag, "Polygon") == 0)
	type = GAIA_GML_POLYGON;
    if (strcmp (node->Tag, "gml:MultiPoint") == 0
	|| strcmp (node->Tag, "MultiPoint") == 0)
	type = GAIA_GML_MULTIPOINT;
    if (strcmp (node->Tag, "gml:MultiLineString") == 0
	|| strcmp (node->Tag, "MultiLineString") == 0)
	type = GAIA_GML_MULTILINESTRING;
    if (strcmp (node->Tag, "gml:MultiCurve") == 0
	|| strcmp (node->Tag, "MultiCurve") == 0)
	type = GAIA_GML_MULTICURVE;
    if (strcmp (node->Tag, "gml:MultiPolygon") == 0
	|| strcmp (node->Tag, "MultiPolygon") == 0)
	type = GAIA_GML_MULTIPOLYGON;
    if (strcmp (node->Tag, "gml:MultiSurface") == 0
	|| strcmp (node->Tag, "MultiSurface") == 0)
	type = GAIA_GML_MULTISURFACE;
    if (strcmp (node->Tag, "gml:MultiGeometry") == 0
	|| strcmp (node->Tag, "MultiGeometry") == 0)
	type = GAIA_GML_MULTIGEOMETRY;
    return type;
}

static int
gml_check_coord (const char *value)
{
/* checking a GML coordinate */
    int decimal = 0;
    const char *p = value;
    if (*p == '+' || *p == '-')
	p++;
    while (*p != '\0')
      {
	  if (*p == '.')
	    {
		if (!decimal)
		    decimal = 1;
		else
		    return 0;
	    }
	  else if (*p >= '0' && *p <= '9')
	      ;
	  else
	      return 0;
	  p++;
      }
    return 1;
}

static int
gml_extract_coords (const char *value, double *x, double *y, double *z,
		    int *count)
{
/* extracting GML v2.x coords from a comma-separated string */
    const char *in = value;
    char buf[1024];
    char *out = buf;
    *out = '\0';

    while (*in != '\0')
      {
	  if (*in == ',')
	    {
		*out = '\0';
		if (*buf != '\0')
		  {
		      if (!gml_check_coord (buf))
			  return 0;
		      switch (*count)
			{
			case 0:
			    *x = atof (buf);
			    *count += 1;
			    break;
			case 1:
			    *y = atof (buf);
			    *count += 1;
			    break;
			case 2:
			    *z = atof (buf);
			    *count += 1;
			    break;
			default:
			    *count += 1;
			    break;
			};
		  }
		in++;
		out = buf;
		*out = '\0';
		continue;
	    }
	  *out++ = *in++;
      }
    *out = '\0';
/* parsing the last item */
    if (*buf != '\0')
      {
	  if (!gml_check_coord (buf))
	      return 0;
	  switch (*count)
	    {
	    case 0:
		*x = atof (buf);
		*count += 1;
		break;
	    case 1:
		*y = atof (buf);
		*count += 1;
		break;
	    case 2:
		*z = atof (buf);
		*count += 1;
		break;
	    default:
		*count += 1;
		break;
	    };
      }
    return 1;
}

static int
gml_parse_point_v2 (gmlCoordPtr coord, double *x, double *y, double *z,
		    int *has_z)
{
/* parsing GML v2.x <gml:coordinates> [Point] */
    int count = 0;
    gmlCoordPtr c = coord;
    while (c)
      {
	  if (!gml_extract_coords (c->Value, x, y, z, &count))
	      return 0;
	  c = c->Next;
      }
    if (count == 2)
      {
	  *has_z = 0;
	  return 1;
      }
    if (count == 3)
      {
	  *has_z = 1;
	  return 1;
      }
    return 0;
}

static int
gml_parse_point_v3 (gmlCoordPtr coord, double *x, double *y, double *z,
		    int *has_z)
{
/* parsing GML v2.x <gml:pos> [Point] */
    int count = 0;
    gmlCoordPtr c = coord;
    while (c)
      {
	  if (!gml_check_coord (c->Value))
	      return 0;
	  switch (count)
	    {
	    case 0:
		*x = atof (c->Value);
		count++;
		break;
	    case 1:
		*y = atof (c->Value);
		count++;
		break;
	    case 2:
		*z = atof (c->Value);
		count++;
		break;
	    default:
		count++;
		break;
	    };
	  c = c->Next;
      }
    if (count == 2)
      {
	  *has_z = 0;
	  return 1;
      }
    if (count == 3)
      {
	  *has_z = 1;
	  return 1;
      }
    return 0;
}

static int
gml_parse_point (gaiaGeomCollPtr geom, gmlNodePtr node, int srid,
		 gmlNodePtr * next)
{
/* parsing a <gml:Point> */
    double x;
    double y;
    double z;
    int has_z;
    gaiaGeomCollPtr pt;
    gaiaGeomCollPtr last;

    if (strcmp (node->Tag, "gml:coordinates") == 0
	|| strcmp (node->Tag, "coordinates") == 0)
      {
	  /* parsing a GML v.2.x <gml:Point> */
	  if (!gml_parse_point_v2 (node->Coordinates, &x, &y, &z, &has_z))
	      return 0;
	  node = node->Next;
	  if (node == NULL)
	      return 0;
	  if (strcmp (node->Tag, "gml:coordinates") == 0
	      || strcmp (node->Tag, "coordinates") == 0)
	      ;
	  else
	      return 0;
	  node = node->Next;
	  if (node == NULL)
	      return 0;
	  if (strcmp (node->Tag, "gml:Point") == 0
	      || strcmp (node->Tag, "Point") == 0)
	      ;
	  else
	      return 0;
	  *next = node->Next;
	  goto ok;
      }
    if (strcmp (node->Tag, "gml:pos") == 0 || strcmp (node->Tag, "pos") == 0)
      {
	  /* parsing a GML v.3.x <gml:Point> */
	  if (!gml_parse_point_v3 (node->Coordinates, &x, &y, &z, &has_z))
	      return 0;
	  node = node->Next;
	  if (node == NULL)
	      return 0;
	  if (strcmp (node->Tag, "gml:pos") == 0
	      || strcmp (node->Tag, "pos") == 0)
	      ;
	  else
	      return 0;
	  node = node->Next;
	  if (node == NULL)
	      return 0;
	  if (strcmp (node->Tag, "gml:Point") == 0
	      || strcmp (node->Tag, "Point") == 0)
	      ;
	  else
	      return 0;
	  *next = node->Next;
	  goto ok;
      }
    return 0;

  ok:
/* ok, GML nodes match as expected */
    if (has_z)
      {
	  pt = gaiaAllocGeomCollXYZ ();
	  pt->Srid = srid;
	  gaiaAddPointToGeomCollXYZ (pt, x, y, z);
      }
    else
      {
	  pt = gaiaAllocGeomColl ();
	  pt->Srid = srid;
	  gaiaAddPointToGeomColl (pt, x, y);
      }
    last = geom;
    while (1)
      {
	  /* searching the last Geometry within chain */
	  if (last->Next == NULL)
	      break;
	  last = last->Next;
      }
    last->Next = pt;
    return 1;
}

static int
gml_extract_multi_coord (const char *value, double *x, double *y, double *z,
			 int *count, int *follow)
{
/* extracting GML v2.x coords from a comma-separated string */
    const char *in = value;
    char buf[1024];
    char *out = buf;
    int last;
    *out = '\0';
    while (*in != '\0')
      {
	  last = *in;
	  if (*in == ',')
	    {
		*out = '\0';
		if (*buf != '\0')
		  {
		      if (!gml_check_coord (buf))
			  return 0;
		      switch (*count)
			{
			case 0:
			    *x = atof (buf);
			    *count += 1;
			    break;
			case 1:
			    *y = atof (buf);
			    *count += 1;
			    break;
			case 2:
			    *z = atof (buf);
			    *count += 1;
			    break;
			default:
			    *count += 1;
			    break;
			};
		  }
		in++;
		out = buf;
		*out = '\0';
		continue;
	    }
	  *out++ = *in++;
      }
    *out = '\0';
/* parsing the last item */
    if (*buf != '\0')
      {
	  if (!gml_check_coord (buf))
	      return 0;
	  switch (*count)
	    {
	    case 0:
		*x = atof (buf);
		*count += 1;
		break;
	    case 1:
		*y = atof (buf);
		*count += 1;
		break;
	    case 2:
		*z = atof (buf);
		*count += 1;
		break;
	    default:
		*count += 1;
		break;
	    };
      }
    if (last == ',')
	*follow = 1;
    else
	*follow = 0;
    return 1;
}

static int
gml_extract_multi_coords (gmlCoordPtr coord, double *x, double *y, double *z,
			  int *count, gmlCoordPtr * next)
{
/* extracting GML v2.x coords from a comma-separated string */
    int follow;
    gmlCoordPtr c = coord;
    while (c)
      {
	  if (!gml_extract_multi_coord (c->Value, x, y, z, count, &follow))
	      return 0;
	  if (!follow && c->Next != NULL)
	    {
		if (*(c->Next->Value) == ',')
		    follow = 1;
	    }
	  if (follow)
	      c = c->Next;
	  else
	    {
		*next = c->Next;
		break;
	    }
      }
    return 1;
}

static void
gml_add_point_to_line (gaiaDynamicLinePtr dyn, double x, double y)
{
/* appending a point */
    gaiaAppendPointToDynamicLine (dyn, x, y);
}

static void
gml_add_point_to_lineZ (gaiaDynamicLinePtr dyn, double x, double y, double z)
{
/* appending a point */
    gaiaAppendPointZToDynamicLine (dyn, x, y, z);
}

static int
gml_parse_coordinates (gmlCoordPtr coord, gaiaDynamicLinePtr dyn, int *has_z)
{
/* parsing GML v2.x <gml:coordinates> [Linestring or Ring] */
    int count = 0;
    double x;
    double y;
    double z;
    gmlCoordPtr next;
    gmlCoordPtr c = coord;
    while (c)
      {
	  if (!gml_extract_multi_coords (c, &x, &y, &z, &count, &next))
	      return 0;
	  if (count == 2)
	    {
		*has_z = 0;
		gml_add_point_to_line (dyn, x, y);
		count = 0;
	    }
	  else if (count == 3)
	    {
		gml_add_point_to_lineZ (dyn, x, y, z);
		count = 0;
	    }
	  else
	      return 0;
	  c = next;
      }
    return 1;
}

static int
gml_parse_posList (gmlCoordPtr coord, gaiaDynamicLinePtr dyn, int has_z)
{
/* parsing GML v3.x <gml:posList> [Linestring or Ring] */
    int count = 0;
    double x;
    double y;
    double z;
    gmlCoordPtr c = coord;
    while (c)
      {
	  if (!gml_check_coord (c->Value))
	      return 0;
	  if (!has_z)
	    {
		switch (count)
		  {
		  case 0:
		      x = atof (c->Value);
		      count++;
		      break;
		  case 1:
		      y = atof (c->Value);
		      gml_add_point_to_line (dyn, x, y);
		      count = 0;
		      break;
		  };
	    }
	  else
	    {
		switch (count)
		  {
		  case 0:
		      x = atof (c->Value);
		      count++;
		      break;
		  case 1:
		      y = atof (c->Value);
		      count++;
		      break;
		  case 2:
		      z = atof (c->Value);
		      gml_add_point_to_lineZ (dyn, x, y, z);
		      count = 0;
		      break;
		  };
	    }
	  c = c->Next;
      }
    if (count != 0)
	return 0;
    return 1;
}

static int
gml_count_dyn_points (gaiaDynamicLinePtr dyn)
{
/* count how many vertices are into sone linestring/ring */
    int iv = 0;
    gaiaPointPtr pt = dyn->First;
    while (pt)
      {
	  iv++;
	  pt = pt->Next;
      }
    return iv;
}

static int
gml_parse_linestring (gaiaGeomCollPtr geom, gmlNodePtr node, int srid,
		      gmlNodePtr * next)
{
/* parsing a <gml:LineString> */
    gaiaGeomCollPtr ln;
    gaiaGeomCollPtr last;
    gaiaLinestringPtr new_ln;
    gaiaPointPtr pt;
    gaiaDynamicLinePtr dyn = gaiaAllocDynamicLine ();
    int iv;
    int has_z = 1;
    int points = 0;

    if (strcmp (node->Tag, "gml:coordinates") == 0
	|| strcmp (node->Tag, "coordinates") == 0)
      {
	  /* parsing a GML v.2.x <gml:LineString> */
	  if (!gml_parse_coordinates (node->Coordinates, dyn, &has_z))
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:coordinates") == 0
	      || strcmp (node->Tag, "coordinates") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LineString") == 0
	      || strcmp (node->Tag, "LineString") == 0)
	      ;
	  else
	      goto error;
	  *next = node->Next;
	  goto ok;
      }
    if (strcmp (node->Tag, "gml:posList") == 0
	|| strcmp (node->Tag, "posList") == 0)
      {
	  /* parsing a GML v.3.x <gml:LineString> */
	  has_z = gml_get_srsDimension (node);
	  if (!gml_parse_posList (node->Coordinates, dyn, has_z))
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:posList") == 0
	      || strcmp (node->Tag, "posList") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LineString") == 0
	      || strcmp (node->Tag, "LineString") == 0)
	      ;
	  else
	      goto error;
	  *next = node->Next;
	  goto ok;
      }
    goto error;

  ok:
/* ok, GML nodes match as expected */
    points = gml_count_dyn_points (dyn);
    if (points < 2)
	goto error;
    if (has_z)
      {
	  ln = gaiaAllocGeomCollXYZ ();
	  ln->Srid = srid;
	  new_ln = gaiaAddLinestringToGeomColl (ln, points);
	  pt = dyn->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPointXYZ (new_ln->Coords, iv, pt->X, pt->Y, pt->Z);
		iv++;
		pt = pt->Next;
	    }
      }
    else
      {
	  ln = gaiaAllocGeomColl ();
	  ln->Srid = srid;
	  new_ln = gaiaAddLinestringToGeomColl (ln, points);
	  pt = dyn->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPoint (new_ln->Coords, iv, pt->X, pt->Y);
		iv++;
		pt = pt->Next;
	    }
      }
    last = geom;
    while (1)
      {
	  /* searching the last Geometry within chain */
	  if (last->Next == NULL)
	      break;
	  last = last->Next;
      }
    last->Next = ln;
    gaiaFreeDynamicLine (dyn);
    return 1;

  error:
    gaiaFreeDynamicLine (dyn);
    return 0;
}

static int
gml_parse_curve (gaiaGeomCollPtr geom, gmlNodePtr node, int srid,
		 gmlNodePtr * next)
{
/* parsing a <gml:Curve> */
    gaiaGeomCollPtr ln;
    gaiaGeomCollPtr last;
    gaiaLinestringPtr new_ln;
    gaiaPointPtr pt;
    gaiaDynamicLinePtr dyn = gaiaAllocDynamicLine ();
    int iv;
    int has_z = 1;
    int points = 0;

    if (strcmp (node->Tag, "gml:segments") == 0
	|| strcmp (node->Tag, "segments") == 0)
      {
	  /* parsing a GML v.3.x <gml:Curve> */
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LineStringSegment") == 0
	      || strcmp (node->Tag, "LineStringSegment") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:posList") == 0
	      || strcmp (node->Tag, "posList") == 0)
	      ;
	  else
	      goto error;
	  has_z = gml_get_srsDimension (node);
	  if (!gml_parse_posList (node->Coordinates, dyn, has_z))
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:posList") == 0
	      || strcmp (node->Tag, "posList") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LineStringSegment") == 0
	      || strcmp (node->Tag, "LineStringSegment") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:segments") == 0
	      || strcmp (node->Tag, "segments") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:Curve") == 0
	      || strcmp (node->Tag, "Curve") == 0)
	      ;
	  else
	      goto error;
	  *next = node->Next;
	  goto ok;
      }
    goto error;

  ok:
/* ok, GML nodes match as expected */
    points = gml_count_dyn_points (dyn);
    if (points < 2)
	goto error;
    if (has_z)
      {
	  ln = gaiaAllocGeomCollXYZ ();
	  ln->Srid = srid;
	  new_ln = gaiaAddLinestringToGeomColl (ln, points);
	  pt = dyn->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPointXYZ (new_ln->Coords, iv, pt->X, pt->Y, pt->Z);
		iv++;
		pt = pt->Next;
	    }
      }
    else
      {
	  ln = gaiaAllocGeomColl ();
	  ln->Srid = srid;
	  new_ln = gaiaAddLinestringToGeomColl (ln, points);
	  pt = dyn->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPoint (new_ln->Coords, iv, pt->X, pt->Y);
		iv++;
		pt = pt->Next;
	    }
      }
    last = geom;
    while (1)
      {
	  /* searching the last Geometry within chain */
	  if (last->Next == NULL)
	      break;
	  last = last->Next;
      }
    last->Next = ln;
    gaiaFreeDynamicLine (dyn);
    return 1;

  error:
    gaiaFreeDynamicLine (dyn);
    return 0;
}

static gaiaDynamicLinePtr
gml_parse_ring (gmlNodePtr node, int *interior, int *has_z, gmlNodePtr * next)
{
/* parsing a generic GML ring */
    gaiaDynamicLinePtr dyn = gaiaAllocDynamicLine ();
    *has_z = 1;

    if (strcmp (node->Tag, "gml:outerBoundaryIs") == 0
	|| strcmp (node->Tag, "outerBoundaryIs") == 0)
      {
	  /* parsing a GML v.2.x <gml:outerBoundaryIs> */
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:coordinates") == 0
	      || strcmp (node->Tag, "coordinates") == 0)
	    {
		/* parsing a GML v.2.x <gml:coordinates> */
		if (!gml_parse_coordinates (node->Coordinates, dyn, has_z))
		    goto error;
		node = node->Next;
		if (node == NULL)
		    goto error;
		if (strcmp (node->Tag, "gml:coordinates") == 0
		    || strcmp (node->Tag, "coordinates") == 0)
		    ;
		else
		    goto error;
	    }
	  else if (strcmp (node->Tag, "gml:posList") == 0
		   || strcmp (node->Tag, "posList") == 0)
	    {
		/* parsing a GML v.3.x <gml:posList> */
		*has_z = gml_get_srsDimension (node);
		if (!gml_parse_posList (node->Coordinates, dyn, *has_z))
		    goto error;
		node = node->Next;
		if (node == NULL)
		    goto error;
		if (strcmp (node->Tag, "gml:posList") == 0
		    || strcmp (node->Tag, "posList") == 0)
		    ;
		else
		    goto error;
	    }
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:outerBoundaryIs") == 0
	      || strcmp (node->Tag, "outerBoundaryIs") == 0)
	      ;
	  else
	      goto error;
	  *interior = 0;
	  *next = node->Next;
	  return dyn;
      }
    if (strcmp (node->Tag, "gml:innerBoundaryIs") == 0
	|| strcmp (node->Tag, "innerBoundaryIs") == 0)
      {
	  /* parsing a GML v.2.x <gml:innerBoundaryIs> */
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:coordinates") == 0
	      || strcmp (node->Tag, "coordinates") == 0)
	    {
		/* parsing a GML v.2.x <gml:coordinates> */
		if (!gml_parse_coordinates (node->Coordinates, dyn, has_z))
		    goto error;
		node = node->Next;
		if (node == NULL)
		    goto error;
		if (strcmp (node->Tag, "gml:coordinates") == 0
		    || strcmp (node->Tag, "coordinates") == 0)
		    ;
		else
		    goto error;
	    }
	  else if (strcmp (node->Tag, "gml:posList") == 0
		   || strcmp (node->Tag, "posList") == 0)
	    {
		/* parsing a GML v.3.x <gml:posList> */
		*has_z = gml_get_srsDimension (node);
		if (!gml_parse_posList (node->Coordinates, dyn, *has_z))
		    goto error;
		node = node->Next;
		if (node == NULL)
		    goto error;
		if (strcmp (node->Tag, "gml:posList") == 0
		    || strcmp (node->Tag, "posList") == 0)
		    ;
		else
		    goto error;
	    }
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:innerBoundaryIs") == 0
	      || strcmp (node->Tag, "innerBoundaryIs") == 0)
	      ;
	  else
	      goto error;
	  *interior = 1;
	  *next = node->Next;
	  return dyn;
      }
    if (strcmp (node->Tag, "gml:exterior") == 0
	|| strcmp (node->Tag, "exterior") == 0)
      {
	  /* parsing a GML v.3.x <gml:exterior> */
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:posList") == 0
	      || strcmp (node->Tag, "posList") == 0)
	      ;
	  else
	      goto error;
	  *has_z = gml_get_srsDimension (node);
	  if (!gml_parse_posList (node->Coordinates, dyn, *has_z))
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:posList") == 0
	      || strcmp (node->Tag, "posList") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:exterior") == 0
	      || strcmp (node->Tag, "exterior") == 0)
	      ;
	  else
	      goto error;
	  *interior = 0;
	  *next = node->Next;
	  return dyn;
      }
    if (strcmp (node->Tag, "gml:interior") == 0
	|| strcmp (node->Tag, "interior") == 0)
      {
	  /* parsing a GML v.3.x <gml:interior> */
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:posList") == 0
	      || strcmp (node->Tag, "posList") == 0)
	      ;
	  else
	      goto error;
	  *has_z = gml_get_srsDimension (node);
	  if (!gml_parse_posList (node->Coordinates, dyn, *has_z))
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:posList") == 0
	      || strcmp (node->Tag, "posList") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:LinearRing") == 0
	      || strcmp (node->Tag, "LinearRing") == 0)
	      ;
	  else
	      goto error;
	  node = node->Next;
	  if (node == NULL)
	      goto error;
	  if (strcmp (node->Tag, "gml:interior") == 0
	      || strcmp (node->Tag, "interior") == 0)
	      ;
	  else
	      goto error;
	  *interior = 1;
	  *next = node->Next;
	  return dyn;
      }

  error:
    gaiaFreeDynamicLine (dyn);
    return 0;
}

static int
gml_parse_polygon (gaiaGeomCollPtr geom, gmlNodePtr node, int srid,
		   gmlNodePtr * next_n)
{
/* parsing a <gml:Polygon> */
    int interior;
    int has_z;
    int inners;
    int outers;
    int points;
    int iv;
    int ib = 0;
    gaiaGeomCollPtr pg;
    gaiaGeomCollPtr last_g;
    gaiaPolygonPtr new_pg;
    gaiaRingPtr ring;
    gaiaDynamicLinePtr dyn;
    gaiaPointPtr pt;
    gaiaDynamicLinePtr exterior_ring;
    gmlNodePtr next;
    gmlDynamicRingPtr dyn_rng;
    gmlDynamicPolygonPtr dyn_pg = gml_alloc_dyn_polygon ();
    gmlNodePtr n = node;
    while (n)
      {
	  /* looping on rings */
	  if (strcmp (n->Tag, "gml:Polygon") == 0
	      || strcmp (n->Tag, "Polygon") == 0)
	    {
		*next_n = n->Next;
		break;
	    }
	  dyn = gml_parse_ring (n, &interior, &has_z, &next);
	  if (dyn == NULL)
	      goto error;
	  if (gml_count_dyn_points (dyn) < 4)
	    {
		/* cannot be a valid ring */
		goto error;
	    }
	  /* checking if the ring is closed */
	  if (has_z)
	    {
		if (dyn->First->X == dyn->Last->X
		    && dyn->First->Y == dyn->Last->Y
		    && dyn->First->Z == dyn->Last->Z)
		    ;
		else
		    goto error;
	    }
	  else
	    {
		if (dyn->First->X == dyn->Last->X
		    && dyn->First->Y == dyn->Last->Y)
		    ;
		else
		    goto error;
	    }
	  gml_add_polygon_ring (dyn_pg, dyn, interior, has_z);
	  n = next;
      }
/* ok, GML nodes match as expected */
    inners = 0;
    outers = 0;
    has_z = 1;
    dyn_rng = dyn_pg->first;
    while (dyn_rng)
      {
	  /* verifying the rings collection */
	  if (dyn_rng->has_z == 0)
	      has_z = 0;
	  if (dyn_rng->interior)
	      inners++;
	  else
	    {
		outers++;
		points = gml_count_dyn_points (dyn_rng->ring);
		exterior_ring = dyn_rng->ring;
	    }
	  dyn_rng = dyn_rng->next;
      }
    if (outers != 1)		/* no exterior ring declared */
	goto error;

    if (has_z)
      {
	  pg = gaiaAllocGeomCollXYZ ();
	  pg->Srid = srid;
	  new_pg = gaiaAddPolygonToGeomColl (pg, points, inners);
	  /* initializing the EXTERIOR RING */
	  ring = new_pg->Exterior;
	  pt = exterior_ring->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPointXYZ (ring->Coords, iv, pt->X, pt->Y, pt->Z);
		iv++;
		pt = pt->Next;
	    }
	  dyn_rng = dyn_pg->first;
	  while (dyn_rng)
	    {
		/* initializing any INTERIOR RING */
		if (dyn_rng->interior == 0)
		  {
		      dyn_rng = dyn_rng->next;
		      continue;
		  }
		points = gml_count_dyn_points (dyn_rng->ring);
		ring = gaiaAddInteriorRing (new_pg, ib, points);
		ib++;
		pt = dyn_rng->ring->First;
		iv = 0;
		while (pt)
		  {
		      gaiaSetPointXYZ (ring->Coords, iv, pt->X, pt->Y, pt->Z);
		      iv++;
		      pt = pt->Next;
		  }
		dyn_rng = dyn_rng->next;
	    }
      }
    else
      {
	  pg = gaiaAllocGeomColl ();
	  pg->Srid = srid;
	  new_pg = gaiaAddPolygonToGeomColl (pg, points, inners);
	  /* initializing the EXTERIOR RING */
	  ring = new_pg->Exterior;
	  pt = exterior_ring->First;
	  iv = 0;
	  while (pt)
	    {
		gaiaSetPoint (ring->Coords, iv, pt->X, pt->Y);
		iv++;
		pt = pt->Next;
	    }
	  dyn_rng = dyn_pg->first;
	  while (dyn_rng)
	    {
		/* initializing any INTERIOR RING */
		if (dyn_rng->interior == 0)
		  {
		      dyn_rng = dyn_rng->next;
		      continue;
		  }
		points = gml_count_dyn_points (dyn_rng->ring);
		ring = gaiaAddInteriorRing (new_pg, ib, points);
		ib++;
		pt = dyn_rng->ring->First;
		iv = 0;
		while (pt)
		  {
		      gaiaSetPoint (ring->Coords, iv, pt->X, pt->Y);
		      iv++;
		      pt = pt->Next;
		  }
		dyn_rng = dyn_rng->next;
	    }
      }

    last_g = geom;
    while (1)
      {
	  /* searching the last Geometry within chain */
	  if (last_g->Next == NULL)
	      break;
	  last_g = last_g->Next;
      }
    last_g->Next = pg;
    gml_free_dyn_polygon (dyn_pg);
    return 1;

  error:
    gml_free_dyn_polygon (dyn_pg);
    return 0;
}

static int
gml_parse_multi_point (gaiaGeomCollPtr geom, gmlNodePtr node)
{
/* parsing a <gml:MultiPoint> */
    int srid;
    gmlNodePtr next;
    gmlNodePtr n = node;
    while (n)
      {
	  /* looping on Point Members */
	  if (n->Next == NULL)
	    {
		/* verifying the last GML node */
		if (strcmp (n->Tag, "gml:MultiPoint") == 0
		    || strcmp (n->Tag, "MultiPoint") == 0)
		    break;
		else
		    return 0;
	    }
	  if (strcmp (n->Tag, "gml:pointMember") == 0
	      || strcmp (n->Tag, "pointMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:Point") == 0
	      || strcmp (n->Tag, "Point") == 0)
	      ;
	  else
	      return 0;
	  srid = guessGmlSrid (n);
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (!gml_parse_point (geom, n, srid, &next))
	      return 0;
	  n = next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:pointMember") == 0
	      || strcmp (n->Tag, "pointMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
      }
    return 1;
}

static int
gml_parse_multi_linestring (gaiaGeomCollPtr geom, gmlNodePtr node)
{
/* parsing a <gml:MultiLineString> */
    int srid;
    gmlNodePtr next;
    gmlNodePtr n = node;
    while (n)
      {
	  /* looping on LineString Members */
	  if (n->Next == NULL)
	    {
		/* verifying the last GML node */
		if (strcmp (n->Tag, "gml:MultiLineString") == 0
		    || strcmp (n->Tag, "MultiLineString") == 0)
		    break;
		else
		    return 0;
	    }
	  if (strcmp (n->Tag, "gml:lineStringMember") == 0
	      || strcmp (n->Tag, "lineStringMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:LineString") == 0
	      || strcmp (n->Tag, "LineString") == 0)
	      ;
	  else
	      return 0;
	  srid = guessGmlSrid (n);
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (!gml_parse_linestring (geom, n, srid, &next))
	      return 0;
	  n = next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:lineStringMember") == 0
	      || strcmp (n->Tag, "lineStringMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
      }
    return 1;
}

static int
gml_parse_multi_curve (gaiaGeomCollPtr geom, gmlNodePtr node)
{
/* parsing a <gml:MultiCurve> */
    int srid;
    gmlNodePtr next;
    gmlNodePtr n = node;
    while (n)
      {
	  /* looping on Curve Members */
	  if (n->Next == NULL)
	    {
		/* verifying the last GML node */
		if (strcmp (n->Tag, "gml:MultiCurve") == 0
		    || strcmp (n->Tag, "MultiCurve") == 0)
		    break;
		else
		    return 0;
	    }
	  if (strcmp (n->Tag, "gml:curveMember") == 0
	      || strcmp (n->Tag, "curveMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:Curve") == 0
	      || strcmp (n->Tag, "Curve") == 0)
	      ;
	  else
	      return 0;
	  srid = guessGmlSrid (n);
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (!gml_parse_curve (geom, n, srid, &next))
	      return 0;
	  n = next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:curveMember") == 0
	      || strcmp (n->Tag, "curveMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
      }
    return 1;
}

static int
gml_parse_multi_polygon (gaiaGeomCollPtr geom, gmlNodePtr node)
{
/* parsing a <gml:MultiPolygon> */
    int srid;
    gmlNodePtr next;
    gmlNodePtr n = node;
    while (n)
      {
	  /* looping on Polygon Members */
	  if (n->Next == NULL)
	    {
		/* verifying the last GML node */
		if (strcmp (n->Tag, "gml:MultiPolygon") == 0
		    || strcmp (n->Tag, "MultiPolygon") == 0)
		    break;
		else
		    return 0;
	    }
	  if (strcmp (n->Tag, "gml:polygonMember") == 0
	      || strcmp (n->Tag, "polygonMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:Polygon") == 0
	      || strcmp (n->Tag, "Polygon") == 0)
	      ;
	  else
	      return 0;
	  srid = guessGmlSrid (n);
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (!gml_parse_polygon (geom, n, srid, &next))
	      return 0;
	  n = next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:polygonMember") == 0
	      || strcmp (n->Tag, "polygonMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
      }
    return 1;
}

static int
gml_parse_multi_surface (gaiaGeomCollPtr geom, gmlNodePtr node)
{
/* parsing a <gml:MultiSurface> */
    int srid;
    gmlNodePtr next;
    gmlNodePtr n = node;
    while (n)
      {
	  /* looping on Surface Members */
	  if (n->Next == NULL)
	    {
		/* verifying the last GML node */
		if (strcmp (n->Tag, "gml:MultiSurface") == 0
		    || strcmp (n->Tag, "MultiSurface") == 0)
		    break;
		else
		    return 0;
	    }
	  if (strcmp (n->Tag, "gml:surfaceMember") == 0
	      || strcmp (n->Tag, "surfaceMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:Polygon") == 0
	      || strcmp (n->Tag, "Polygon") == 0)
	      ;
	  else
	      return 0;
	  srid = guessGmlSrid (n);
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (!gml_parse_polygon (geom, n, srid, &next))
	      return 0;
	  n = next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:surfaceMember") == 0
	      || strcmp (n->Tag, "surfaceMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
      }
    return 1;
}

static int
gml_parse_multi_geometry (gaiaGeomCollPtr geom, gmlNodePtr node)
{
/* parsing a <gml:MultiGeometry> */
    int srid;
    gmlNodePtr next;
    gmlNodePtr n = node;
    while (n)
      {
	  /* looping on Geometry Members */
	  if (n->Next == NULL)
	    {
		/* verifying the last GML node */
		if (strcmp (n->Tag, "gml:MultiGeometry") == 0
		    || strcmp (n->Tag, "MultiGeometry") == 0)
		    break;
		else
		    return 0;
	    }
	  if (strcmp (n->Tag, "gml:geometryMember") == 0
	      || strcmp (n->Tag, "geometryMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:Point") == 0
	      || strcmp (n->Tag, "Point") == 0)
	    {
		srid = guessGmlSrid (n);
		n = n->Next;
		if (n == NULL)
		    return 0;
		if (!gml_parse_point (geom, n, srid, &next))
		    return 0;
		n = next;
	    }
	  else if (strcmp (n->Tag, "gml:LineString") == 0
		   || strcmp (n->Tag, "LineString") == 0)
	    {
		srid = guessGmlSrid (n);
		n = n->Next;
		if (n == NULL)
		    return 0;
		if (!gml_parse_linestring (geom, n, srid, &next))
		    return 0;
		n = next;
	    }
	  else if (strcmp (n->Tag, "gml:Curve") == 0
		   || strcmp (n->Tag, "Curve") == 0)
	    {
		srid = guessGmlSrid (n);
		n = n->Next;
		if (n == NULL)
		    return 0;
		if (!gml_parse_curve (geom, n, srid, &next))
		    return 0;
		n = next;
	    }
	  else if (strcmp (n->Tag, "gml:Polygon") == 0
		   || strcmp (n->Tag, "Polygon") == 0)
	    {
		srid = guessGmlSrid (n);
		n = n->Next;
		if (n == NULL)
		    return 0;
		if (!gml_parse_polygon (geom, n, srid, &next))
		    return 0;
		n = next;
	    }
	  else
	      return 0;
	  if (n == NULL)
	      return 0;
	  if (strcmp (n->Tag, "gml:geometryMember") == 0
	      || strcmp (n->Tag, "geometryMember") == 0)
	      ;
	  else
	      return 0;
	  n = n->Next;
      }
    return 1;
}

static gaiaGeomCollPtr
gml_validate_geometry (gaiaGeomCollPtr chain, sqlite3 * sqlite_handle)
{
    int xy = 0;
    int xyz = 0;
    int pts = 0;
    int lns = 0;
    int pgs = 0;
    gaiaPointPtr pt;
    gaiaLinestringPtr ln;
    gaiaPolygonPtr pg;
    gaiaPointPtr save_pt;
    gaiaLinestringPtr save_ln;
    gaiaPolygonPtr save_pg;
    gaiaRingPtr i_ring;
    gaiaRingPtr o_ring;
    int ib;
    int delete_g2;
    gaiaGeomCollPtr g;
    gaiaGeomCollPtr g2;
    gaiaGeomCollPtr geom;
    char proj_from[2048];
    char proj_to[2048];

    g = chain;
    while (g)
      {
	  if (g != chain)
	    {
		if (g->DimensionModel == GAIA_XY)
		    xy++;
		if (g->DimensionModel == GAIA_XY_Z)
		    xyz++;
	    }
	  pt = g->FirstPoint;
	  while (pt)
	    {
		pts++;
		save_pt = pt;
		pt = pt->Next;
	    }
	  ln = g->FirstLinestring;
	  while (ln)
	    {
		lns++;
		save_ln = ln;
		ln = ln->Next;
	    }
	  pg = g->FirstPolygon;
	  while (pg)
	    {
		pgs++;
		save_pg = pg;
		pg = pg->Next;
	    }
	  g = g->Next;
      }
    if (pts == 1 && lns == 0 && pgs == 0)
      {
	  /* POINT */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_MULTIPOINT)
		    geom->DeclaredType = GAIA_MULTIPOINT;
		else if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_POINT;
		gaiaAddPointToGeomColl (geom, save_pt->X, save_pt->Y);
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_MULTIPOINT)
		    geom->DeclaredType = GAIA_MULTIPOINT;
		else if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_POINT;
		gaiaAddPointToGeomCollXYZ (geom, save_pt->X, save_pt->Y,
					   save_pt->Z);
		return geom;
	    }
      }
    if (pts == 0 && lns == 1 && pgs == 0)
      {
	  /* LINESTRING */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
	    }
	  geom->Srid = chain->Srid;
	  if (chain->DeclaredType == GAIA_MULTILINESTRING)
	      geom->DeclaredType = GAIA_MULTILINESTRING;
	  else if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
	  else
	      geom->DeclaredType = GAIA_LINESTRING;
	  ln = gaiaAddLinestringToGeomColl (geom, save_ln->Points);
	  gaiaCopyLinestringCoords (ln, save_ln);
	  return geom;
      }
    if (pts == 0 && lns == 0 && pgs == 1)
      {
	  /* POLYGON */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
	    }
	  geom->Srid = chain->Srid;
	  if (chain->DeclaredType == GAIA_MULTIPOLYGON)
	      geom->DeclaredType = GAIA_MULTIPOLYGON;
	  else if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
	      geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
	  else
	      geom->DeclaredType = GAIA_POLYGON;
	  i_ring = save_pg->Exterior;
	  pg = gaiaAddPolygonToGeomColl (geom, i_ring->Points,
					 save_pg->NumInteriors);
	  o_ring = pg->Exterior;
	  gaiaCopyRingCoords (o_ring, i_ring);
	  for (ib = 0; ib < save_pg->NumInteriors; ib++)
	    {
		i_ring = save_pg->Interiors + ib;
		o_ring = gaiaAddInteriorRing (pg, ib, i_ring->Points);
		gaiaCopyRingCoords (o_ring, i_ring);
	    }
	  return geom;
      }
    if (pts >= 1 && lns == 0 && pgs == 0)
      {
	  /* MULTIPOINT */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_MULTIPOINT;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet set any SRID */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      pt = g2->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomColl (geom, pt->X, pt->Y);
			    pt = pt->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_MULTIPOINT;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet a SRID set */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      pt = g2->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomCollXYZ (geom, pt->X, pt->Y,
						       pt->Z);
			    pt = pt->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
      }
    if (pts == 0 && lns >= 1 && pgs == 0)
      {
	  /* MULTILINESTRING */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_MULTILINESTRING;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet set any SRID */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      ln = g2->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_MULTILINESTRING;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet a SRID set */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      ln = g2->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
      }
    if (pts == 0 && lns == 0 && pgs >= 1)
      {
	  /* MULTIPOLYGON */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_MULTIPOLYGON;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet set any SRID */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      pg = g2->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->Srid = chain->Srid;
		if (chain->DeclaredType == GAIA_GEOMETRYCOLLECTION)
		    geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		else
		    geom->DeclaredType = GAIA_MULTIPOLYGON;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet a SRID set */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      pg = g2->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
      }
    if ((pts + lns + pgs) > 0)
      {
	  /* GEOMETRYCOLLECTION */
	  if (xy > 0)
	    {
		/* 2D [XY] */
		geom = gaiaAllocGeomColl ();
		geom->Srid = chain->Srid;
		geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet set any SRID */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      pt = g2->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomColl (geom, pt->X, pt->Y);
			    pt = pt->Next;
			}
		      ln = g2->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      pg = g2->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
	  else
	    {
		/* 3D [XYZ] */
		geom = gaiaAllocGeomCollXYZ ();
		geom->Srid = chain->Srid;
		geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
		g = chain;
		while (g)
		  {
		      if (geom->Srid == -1)
			{
			    /* we haven't yet a SRID set */
			    geom->Srid = g->Srid;
			}
		      g2 = g;
		      delete_g2 = 0;
		      if (g->Srid != geom->Srid && g->Srid != -1
			  && sqlite_handle != NULL)
			{
			    /* we'll try to apply a reprojection */
#ifndef OMIT_PROJ		/* but only if PROJ.4 is actually available */
			    gml_proj_params (sqlite_handle, g->Srid, proj_from);
			    gml_proj_params (sqlite_handle, geom->Srid,
					     proj_to);
			    if (*proj_to == '\0' || *proj_from == '\0')
				;
			    else
			      {
				  g2 = gaiaTransform (g, proj_from, proj_to);
				  if (!g2)
				      g2 = g;
				  else
				      delete_g2 = 1;
			      }
#endif
			}
		      pt = g2->FirstPoint;
		      while (pt)
			{
			    gaiaAddPointToGeomCollXYZ (geom, pt->X, pt->Y,
						       pt->Z);
			    pt = pt->Next;
			}
		      ln = g2->FirstLinestring;
		      while (ln)
			{
			    save_ln =
				gaiaAddLinestringToGeomColl (geom, ln->Points);
			    gaiaCopyLinestringCoords (save_ln, ln);
			    ln = ln->Next;
			}
		      pg = g2->FirstPolygon;
		      while (pg)
			{
			    i_ring = pg->Exterior;
			    save_pg =
				gaiaAddPolygonToGeomColl (geom, i_ring->Points,
							  pg->NumInteriors);
			    o_ring = save_pg->Exterior;
			    gaiaCopyRingCoords (o_ring, i_ring);
			    for (ib = 0; ib < pg->NumInteriors; ib++)
			      {
				  i_ring = pg->Interiors + ib;
				  o_ring =
				      gaiaAddInteriorRing (save_pg, ib,
							   i_ring->Points);
				  gaiaCopyRingCoords (o_ring, i_ring);
			      }
			    pg = pg->Next;
			}
		      if (delete_g2)
			  gaiaFreeGeomColl (g2);
		      g = g->Next;
		  }
		return geom;
	    }
      }
    return NULL;
}

static void
gml_free_geom_chain (gaiaGeomCollPtr geom)
{
/* deleting a chain of preliminary geometries */
    gaiaGeomCollPtr gn;
    while (geom)
      {
	  gn = geom->Next;
	  gaiaFreeGeomColl (geom);
	  geom = gn;
      }
}

static gaiaGeomCollPtr
gml_build_geometry (gmlNodePtr tree, sqlite3 * sqlite_handle)
{
/* attempting to build a geometry from GML nodes */
    gaiaGeomCollPtr geom;
    gaiaGeomCollPtr result;
    int geom_type;
    gmlNodePtr next;

    if (tree == NULL)
	return NULL;
    geom_type = guessGmlGeometryType (tree);
    if (geom_type == GAIA_GML_UNKNOWN)
      {
	  /* unsupported main geometry type */
	  return NULL;
      }
/* creating the main geometry */
    geom = gaiaAllocGeomColl ();
    geom->Srid = guessGmlSrid (tree);

    switch (geom_type)
      {
	  /* parsing GML nodes accordingly with declared GML type */
      case GAIA_GML_POINT:
	  geom->DeclaredType = GAIA_POINT;
	  if (!gml_parse_point (geom, tree->Next, geom->Srid, &next))
	      goto error;
	  break;
      case GAIA_GML_LINESTRING:
	  geom->DeclaredType = GAIA_LINESTRING;
	  if (!gml_parse_linestring (geom, tree->Next, geom->Srid, &next))
	      goto error;
	  break;
      case GAIA_GML_CURVE:
	  geom->DeclaredType = GAIA_LINESTRING;
	  if (!gml_parse_curve (geom, tree->Next, geom->Srid, &next))
	      goto error;
	  break;
      case GAIA_GML_POLYGON:
	  geom->DeclaredType = GAIA_POLYGON;
	  if (!gml_parse_polygon (geom, tree->Next, geom->Srid, &next))
	      goto error;
	  if (next != NULL)
	      goto error;
	  break;
      case GAIA_GML_MULTIPOINT:
	  geom->DeclaredType = GAIA_MULTIPOINT;
	  if (!gml_parse_multi_point (geom, tree->Next))
	      goto error;
	  break;
      case GAIA_GML_MULTILINESTRING:
	  geom->DeclaredType = GAIA_MULTILINESTRING;
	  if (!gml_parse_multi_linestring (geom, tree->Next))
	      goto error;
	  break;
      case GAIA_GML_MULTICURVE:
	  geom->DeclaredType = GAIA_MULTILINESTRING;
	  if (!gml_parse_multi_curve (geom, tree->Next))
	      goto error;
	  break;
      case GAIA_GML_MULTIPOLYGON:
	  geom->DeclaredType = GAIA_MULTIPOLYGON;
	  if (!gml_parse_multi_polygon (geom, tree->Next))
	      goto error;
	  break;
      case GAIA_GML_MULTISURFACE:
	  geom->DeclaredType = GAIA_MULTIPOLYGON;
	  if (!gml_parse_multi_surface (geom, tree->Next))
	      goto error;
	  break;
      case GAIA_GML_MULTIGEOMETRY:
	  geom->DeclaredType = GAIA_GEOMETRYCOLLECTION;
	  if (!gml_parse_multi_geometry (geom, tree->Next))
	      goto error;
	  break;
      };

/* attempting to build the final geometry */
    result = gml_validate_geometry (geom, sqlite_handle);
    if (result == NULL)
	goto error;
    gml_free_geom_chain (geom);
    return result;

  error:
    gml_free_geom_chain (geom);
    return NULL;
}



/*
** CAVEAT: we must redefine any Lemon/Flex own macro
*/
#define YYMINORTYPE		GML_MINORTYPE
#define YY_CHAR			GML_YY_CHAR
#define	input			gml_input
#define ParseAlloc		gmlParseAlloc
#define ParseFree		gmlParseFree
#define ParseStackPeak		gmlParseStackPeak
#define Parse			gmlParse
#define yyStackEntry		gml_yyStackEntry
#define yyzerominor		gml_yyzerominor
#define yy_accept		gml_yy_accept
#define yy_action		gml_yy_action
#define yy_base			gml_yy_base
#define yy_buffer_stack		gml_yy_buffer_stack
#define yy_buffer_stack_max	gml_yy_buffer_stack_max
#define yy_buffer_stack_top	gml_yy_buffer_stack_top
#define yy_c_buf_p		gml_yy_c_buf_p
#define yy_chk			gml_yy_chk
#define yy_def			gml_yy_def
#define yy_default		gml_yy_default
#define yy_destructor		gml_yy_destructor
#define yy_ec			gml_yy_ec
#define yy_fatal_error		gml_yy_fatal_error
#define yy_find_reduce_action	gml_yy_find_reduce_action
#define yy_find_shift_action	gml_yy_find_shift_action
#define yy_get_next_buffer	gml_yy_get_next_buffer
#define yy_get_previous_state	gml_yy_get_previous_state
#define yy_init			gml_yy_init
#define yy_init_globals		gml_yy_init_globals
#define yy_lookahead		gml_yy_lookahead
#define yy_meta			gml_yy_meta
#define yy_nxt			gml_yy_nxt
#define yy_parse_failed		gml_yy_parse_failed
#define yy_pop_parser_stack	gml_yy_pop_parser_stack
#define yy_reduce		gml_yy_reduce
#define yy_reduce_ofst		gml_yy_reduce_ofst
#define yy_shift		gml_yy_shift
#define yy_shift_ofst		gml_yy_shift_ofst
#define yy_start		gml_yy_start
#define yy_state_type		gml_yy_state_type
#define yy_syntax_error		gml_yy_syntax_error
#define yy_trans_info		gml_yy_trans_info
#define yy_try_NUL_trans	gml_yy_try_NUL_trans
#define yyParser		gml_yyParser
#define yyStackEntry		gml_yyStackEntry
#define yyStackOverflow		gml_yyStackOverflow
#define yyRuleInfo		gml_yyRuleInfo
#define yyunput			gml_yyunput
#define yyzerominor		gml_yyzerominor
#define yyTraceFILE		gml_yyTraceFILE
#define yyTracePrompt		gml_yyTracePrompt
#define yyTokenName		gml_yyTokenName
#define yyRuleName		gml_yyRuleName
#define ParseTrace		gml_ParseTrace


/*
 GML_LEMON_H_START - LEMON generated header starts here 
*/
#define GML_NEWLINE                     1
#define GML_END                         2
#define GML_CLOSE                       3
#define GML_OPEN                        4
#define GML_KEYWORD                     5
#define GML_EQ                          6
#define GML_VALUE                       7
#define GML_COORD                       8
/*
 GML_LEMON_H_END - LEMON generated header ends here 
*/


typedef union
{
    char *pval;
    struct symtab *symp;
} gml_yystype;
#define YYSTYPE gml_yystype


/* extern YYSTYPE yylval; */
YYSTYPE GmlLval;



/*
 GML_LEMON_START - LEMON generated code starts here 
*/

/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is included that follows the "include" declaration
** in the input grammar file. */
/* #include <stdio.h> */

/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash 
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    ParseTOKENTYPE     is the data type used for minor tokens given 
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is ParseTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    ParseARG_SDECL     A static variable declaration for the %extra_argument
**    ParseARG_PDECL     A parameter declaration for the %extra_argument
**    ParseARG_STORE     Code to store %extra_argument into yypParser
**    ParseARG_FETCH     Code to extract %extra_argument from yypParser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 28
#define YYACTIONTYPE unsigned char
#define ParseTOKENTYPE void *
typedef union {
  int yyinit;
  ParseTOKENTYPE yy0;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 1000000
#endif
#define ParseARG_SDECL  gmlNodePtr *result ;
#define ParseARG_PDECL , gmlNodePtr *result 
#define ParseARG_FETCH  gmlNodePtr *result  = yypParser->result 
#define ParseARG_STORE yypParser->result  = result 
#define YYNSTATE 49
#define YYNRULE 34
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* The yyzerominor constant is used to initialize instances of
** YYMINORTYPE objects to zero. */
static const YYMINORTYPE yyzerominor = { 0 };

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
static const YYACTIONTYPE yy_action[] = {
 /*     0 */    20,   28,   29,    4,   48,    5,    3,    3,    5,    5,
 /*    10 */    42,   84,    1,   42,   42,   47,   46,    2,   10,    5,
 /*    20 */    21,   12,   32,   23,   42,   38,   22,    6,   49,   23,
 /*    30 */    13,   19,   14,   15,   35,    8,    8,   10,   25,   11,
 /*    40 */    18,   34,   33,   45,   37,   16,   40,   17,   41,   14,
 /*    50 */     9,   23,   43,    7,   45,   27,   30,   26,   31,   36,
 /*    60 */    39,   44,   24,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */    12,   13,   14,   15,   16,   17,   15,   15,   17,   17,
 /*    10 */    22,   10,   11,   22,   22,   24,   24,   15,   18,   17,
 /*    20 */     2,    3,    8,    5,   22,   25,    2,    3,    0,    5,
 /*    30 */    18,   19,    4,   20,   21,   20,   20,   18,    2,    3,
 /*    40 */     2,   26,   26,    5,   25,   20,   21,   20,   21,    4,
 /*    50 */    18,    5,   23,   20,    5,    1,    3,   23,    3,    7,
 /*    60 */     3,    3,    6,
};
#define YY_SHIFT_USE_DFLT (-1)
#define YY_SHIFT_MAX 26
static const signed char yy_shift_ofst[] = {
 /*     0 */    -1,   28,   45,   45,   45,   18,   14,   14,   14,   46,
 /*    10 */    46,   14,   14,   24,   38,   14,   14,   14,   49,   36,
 /*    20 */    54,   53,   55,   56,   52,   57,   58,
};
#define YY_REDUCE_USE_DFLT (-13)
#define YY_REDUCE_MAX 18
static const signed char yy_reduce_ofst[] = {
 /*     0 */     1,  -12,   -9,   -8,    2,   12,   13,   15,   16,    0,
 /*    10 */    19,   25,   27,   32,   29,   33,   33,   33,   34,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */    50,   83,   72,   72,   54,   83,   60,   80,   80,   76,
 /*    10 */    76,   61,   59,   83,   83,   64,   66,   62,   83,   83,
 /*    20 */    83,   83,   83,   83,   83,   83,   83,   51,   52,   53,
 /*    30 */    56,   57,   79,   81,   82,   65,   75,   77,   78,   58,
 /*    40 */    67,   63,   68,   69,   70,   71,   73,   74,   55,
};
#define YY_SZ_ACTTAB (int)(sizeof(yy_action)/sizeof(yy_action[0]))

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  int yyidx;                    /* Index of top element in stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyidxMax;                 /* Maximum value of yyidx */
#endif
  int yyerrcnt;                 /* Shifts left before out of the error */
  ParseARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
/* #include <stdio.h> */
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void ParseTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "GML_NEWLINE",   "GML_END",       "GML_CLOSE",   
  "GML_OPEN",      "GML_KEYWORD",   "GML_EQ",        "GML_VALUE",   
  "GML_COORD",     "error",         "main",          "in",          
  "state",         "program",       "gml_tree",      "node",        
  "node_chain",    "open_tag",      "attr",          "attributes",  
  "coord",         "coord_chain",   "close_tag",     "keyword",     
  "extra_nodes",   "extra_attr",    "extra_coord", 
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "main ::= in",
 /*   1 */ "in ::=",
 /*   2 */ "in ::= in state GML_NEWLINE",
 /*   3 */ "state ::= program",
 /*   4 */ "program ::= gml_tree",
 /*   5 */ "gml_tree ::= node",
 /*   6 */ "gml_tree ::= node_chain",
 /*   7 */ "node ::= open_tag GML_END GML_CLOSE",
 /*   8 */ "node ::= open_tag attr GML_END GML_CLOSE",
 /*   9 */ "node ::= open_tag attributes GML_END GML_CLOSE",
 /*  10 */ "node ::= open_tag GML_CLOSE",
 /*  11 */ "node ::= open_tag attr GML_CLOSE",
 /*  12 */ "node ::= open_tag attributes GML_CLOSE",
 /*  13 */ "node ::= open_tag GML_CLOSE coord",
 /*  14 */ "node ::= open_tag GML_CLOSE coord_chain",
 /*  15 */ "node ::= open_tag attr GML_CLOSE coord",
 /*  16 */ "node ::= open_tag attr GML_CLOSE coord_chain",
 /*  17 */ "node ::= open_tag attributes GML_CLOSE coord",
 /*  18 */ "node ::= open_tag attributes GML_CLOSE coord_chain",
 /*  19 */ "node ::= close_tag",
 /*  20 */ "open_tag ::= GML_OPEN keyword",
 /*  21 */ "close_tag ::= GML_OPEN GML_END keyword GML_CLOSE",
 /*  22 */ "keyword ::= GML_KEYWORD",
 /*  23 */ "extra_nodes ::=",
 /*  24 */ "extra_nodes ::= node extra_nodes",
 /*  25 */ "node_chain ::= node node extra_nodes",
 /*  26 */ "attr ::= GML_KEYWORD GML_EQ GML_VALUE",
 /*  27 */ "extra_attr ::=",
 /*  28 */ "extra_attr ::= attr extra_attr",
 /*  29 */ "attributes ::= attr attr extra_attr",
 /*  30 */ "coord ::= GML_COORD",
 /*  31 */ "extra_coord ::=",
 /*  32 */ "extra_coord ::= coord extra_coord",
 /*  33 */ "coord_chain ::= coord coord extra_coord",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.
*/
static void yyGrowStack(yyParser *p){
  int newSize;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  if( pNew ){
    p->yystack = pNew;
    p->yystksz = newSize;
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows to %d entries!\n",
              yyTracePrompt, p->yystksz);
    }
#endif
  }
}
#endif

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Parse and ParseFree.
*/
void *ParseAlloc(void *(*mallocProc)(size_t)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (size_t)sizeof(yyParser) );
  if( pParser ){
    pParser->yyidx = -1;
#ifdef YYTRACKMAXSTACKDEPTH
    pParser->yyidxMax = 0;
#endif
#if YYSTACKDEPTH<=0
    pParser->yystack = NULL;
    pParser->yystksz = 0;
    yyGrowStack(pParser);
#endif
  }
  return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  ParseARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are not used
    ** inside the C code.
    */
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(yyParser *pParser){
  YYCODETYPE yymajor;
  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

  if( pParser->yyidx<0 ) return 0;
#ifndef NDEBUG
  if( yyTraceFILE && pParser->yyidx>=0 ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yymajor = yytos->major;
  yy_destructor(pParser, yymajor, &yytos->minor);
  pParser->yyidx--;
  return yymajor;
}

/* 
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from ParseAlloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void ParseFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
  if( pParser==0 ) return;
  while( pParser->yyidx>=0 ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  free(pParser->yystack);
#endif
  (*freeProc)((void*)pParser);
}

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int ParseStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyidxMax;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yystack[pParser->yyidx].stateno;
 
  if( stateno>YY_SHIFT_MAX || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
    return yy_default[stateno];
  }
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    if( iLookAhead>0 ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        return yy_find_shift_action(pParser, iFallback);
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( j>=0 && j<YY_SZ_ACTTAB && yy_lookahead[j]==YYWILDCARD ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
    }
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_MAX ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_MAX );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_SZ_ACTTAB );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser, YYMINORTYPE *yypMinor){
   ParseARG_FETCH;
   yypParser->yyidx--;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */

     fprintf(stderr,"Giving up.  Parser stack overflow\n");
   ParseARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor         /* Pointer to the minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yyidx++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( yypParser->yyidx>yypParser->yyidxMax ){
    yypParser->yyidxMax = yypParser->yyidx;
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yyidx>=YYSTACKDEPTH ){
    yyStackOverflow(yypParser, yypMinor);
    return;
  }
#else
  if( yypParser->yyidx>=yypParser->yystksz ){
    yyGrowStack(yypParser);
    if( yypParser->yyidx>=yypParser->yystksz ){
      yyStackOverflow(yypParser, yypMinor);
      return;
    }
  }
#endif
  yytos = &yypParser->yystack[yypParser->yyidx];
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor = *yypMinor;
#ifndef NDEBUG
  if( yyTraceFILE && yypParser->yyidx>0 ){
    int i;
    fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
    fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
    for(i=1; i<=yypParser->yyidx; i++)
      fprintf(yyTraceFILE," %s",yyTokenName[yypParser->yystack[i].major]);
    fprintf(yyTraceFILE,"\n");
  }
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 10, 1 },
  { 11, 0 },
  { 11, 3 },
  { 12, 1 },
  { 13, 1 },
  { 14, 1 },
  { 14, 1 },
  { 15, 3 },
  { 15, 4 },
  { 15, 4 },
  { 15, 2 },
  { 15, 3 },
  { 15, 3 },
  { 15, 3 },
  { 15, 3 },
  { 15, 4 },
  { 15, 4 },
  { 15, 4 },
  { 15, 4 },
  { 15, 1 },
  { 17, 2 },
  { 22, 4 },
  { 23, 1 },
  { 24, 0 },
  { 24, 2 },
  { 16, 3 },
  { 18, 3 },
  { 25, 0 },
  { 25, 2 },
  { 19, 3 },
  { 20, 1 },
  { 26, 0 },
  { 26, 2 },
  { 21, 3 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  int yyruleno                 /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  ParseARG_FETCH;
  yymsp = &yypParser->yystack[yypParser->yyidx];
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno>=0 
        && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
      yyRuleName[yyruleno]);
  }
#endif /* NDEBUG */

  /* Silence complaints from purify about yygotominor being uninitialized
  ** in some cases when it is copied into the stack after the following
  ** switch.  yygotominor is uninitialized when a rule reduces that does
  ** not set the value of its left-hand side nonterminal.  Leaving the
  ** value of the nonterminal uninitialized is utterly harmless as long
  ** as the value is never used.  So really the only thing this code
  ** accomplishes is to quieten purify.  
  **
  ** 2007-01-16:  The wireshark project (www.wireshark.org) reports that
  ** without this code, their parser segfaults.  I'm not sure what there
  ** parser is doing to make this happen.  This is the second bug report
  ** from wireshark this week.  Clearly they are stressing Lemon in ways
  ** that it has not been previously stressed...  (SQLite ticket #2172)
  */
  /*memset(&yygotominor, 0, sizeof(yygotominor));*/
  yygotominor = yyzerominor;


  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
      case 5: /* gml_tree ::= node */
      case 6: /* gml_tree ::= node_chain */ yytestcase(yyruleno==6);
{ *result = yymsp[0].minor.yy0; }
        break;
      case 7: /* node ::= open_tag GML_END GML_CLOSE */
{ yygotominor.yy0 = gml_createSelfClosedNode((void *)yymsp[-2].minor.yy0, NULL); }
        break;
      case 8: /* node ::= open_tag attr GML_END GML_CLOSE */
      case 9: /* node ::= open_tag attributes GML_END GML_CLOSE */ yytestcase(yyruleno==9);
{ yygotominor.yy0 = gml_createSelfClosedNode((void *)yymsp[-3].minor.yy0, (void *)yymsp[-2].minor.yy0); }
        break;
      case 10: /* node ::= open_tag GML_CLOSE */
{ yygotominor.yy0 = gml_createNode((void *)yymsp[-1].minor.yy0, NULL, NULL); }
        break;
      case 11: /* node ::= open_tag attr GML_CLOSE */
      case 12: /* node ::= open_tag attributes GML_CLOSE */ yytestcase(yyruleno==12);
{ yygotominor.yy0 = gml_createNode((void *)yymsp[-2].minor.yy0, (void *)yymsp[-1].minor.yy0, NULL); }
        break;
      case 13: /* node ::= open_tag GML_CLOSE coord */
      case 14: /* node ::= open_tag GML_CLOSE coord_chain */ yytestcase(yyruleno==14);
{ yygotominor.yy0 = gml_createNode((void *)yymsp[-2].minor.yy0, NULL, (void *)yymsp[0].minor.yy0); }
        break;
      case 15: /* node ::= open_tag attr GML_CLOSE coord */
      case 16: /* node ::= open_tag attr GML_CLOSE coord_chain */ yytestcase(yyruleno==16);
      case 17: /* node ::= open_tag attributes GML_CLOSE coord */ yytestcase(yyruleno==17);
      case 18: /* node ::= open_tag attributes GML_CLOSE coord_chain */ yytestcase(yyruleno==18);
{ yygotominor.yy0 = gml_createNode((void *)yymsp[-3].minor.yy0, (void *)yymsp[-2].minor.yy0, (void *)yymsp[0].minor.yy0); }
        break;
      case 19: /* node ::= close_tag */
{ yygotominor.yy0 = gml_closingNode((void *)yymsp[0].minor.yy0); }
        break;
      case 20: /* open_tag ::= GML_OPEN keyword */
      case 22: /* keyword ::= GML_KEYWORD */ yytestcase(yyruleno==22);
{ yygotominor.yy0 = yymsp[0].minor.yy0; }
        break;
      case 21: /* close_tag ::= GML_OPEN GML_END keyword GML_CLOSE */
{ yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 23: /* extra_nodes ::= */
      case 27: /* extra_attr ::= */ yytestcase(yyruleno==27);
      case 31: /* extra_coord ::= */ yytestcase(yyruleno==31);
{ yygotominor.yy0 = NULL; }
        break;
      case 24: /* extra_nodes ::= node extra_nodes */
{ ((gmlNodePtr)yymsp[-1].minor.yy0)->Next = (gmlNodePtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 25: /* node_chain ::= node node extra_nodes */
{ 
	   ((gmlNodePtr)yymsp[-1].minor.yy0)->Next = (gmlNodePtr)yymsp[0].minor.yy0; 
	   ((gmlNodePtr)yymsp[-2].minor.yy0)->Next = (gmlNodePtr)yymsp[-1].minor.yy0;
	   yygotominor.yy0 = yymsp[-2].minor.yy0;
	}
        break;
      case 26: /* attr ::= GML_KEYWORD GML_EQ GML_VALUE */
{ yygotominor.yy0 = gml_attribute((void *)yymsp[-2].minor.yy0, (void *)yymsp[0].minor.yy0); }
        break;
      case 28: /* extra_attr ::= attr extra_attr */
{ ((gmlAttrPtr)yymsp[-1].minor.yy0)->Next = (gmlAttrPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 29: /* attributes ::= attr attr extra_attr */
{ 
	   ((gmlAttrPtr)yymsp[-1].minor.yy0)->Next = (gmlAttrPtr)yymsp[0].minor.yy0; 
	   ((gmlAttrPtr)yymsp[-2].minor.yy0)->Next = (gmlAttrPtr)yymsp[-1].minor.yy0;
	   yygotominor.yy0 = yymsp[-2].minor.yy0;
	}
        break;
      case 30: /* coord ::= GML_COORD */
{ yygotominor.yy0 = gml_coord((void *)yymsp[0].minor.yy0); }
        break;
      case 32: /* extra_coord ::= coord extra_coord */
{ ((gmlCoordPtr)yymsp[-1].minor.yy0)->Next = (gmlCoordPtr)yymsp[0].minor.yy0;  yygotominor.yy0 = yymsp[-1].minor.yy0; }
        break;
      case 33: /* coord_chain ::= coord coord extra_coord */
{ 
	   ((gmlCoordPtr)yymsp[-1].minor.yy0)->Next = (gmlCoordPtr)yymsp[0].minor.yy0; 
	   ((gmlCoordPtr)yymsp[-2].minor.yy0)->Next = (gmlCoordPtr)yymsp[-1].minor.yy0;
	   yygotominor.yy0 = yymsp[-2].minor.yy0;
	}
        break;
      default:
      /* (0) main ::= in */ yytestcase(yyruleno==0);
      /* (1) in ::= */ yytestcase(yyruleno==1);
      /* (2) in ::= in state GML_NEWLINE */ yytestcase(yyruleno==2);
      /* (3) state ::= program */ yytestcase(yyruleno==3);
      /* (4) program ::= gml_tree */ yytestcase(yyruleno==4);
        break;
  };
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yypParser->yyidx -= yysize;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact < YYNSTATE ){
#ifdef NDEBUG
    /* If we are not debugging and the reduce action popped at least
    ** one element off the stack, then we can push the new element back
    ** onto the stack here, and skip the stack overflow test in yy_shift().
    ** That gives a significant speed improvement. */
    if( yysize ){
      yypParser->yyidx++;
      yymsp -= yysize-1;
      yymsp->stateno = (YYACTIONTYPE)yyact;
      yymsp->major = (YYCODETYPE)yygoto;
      yymsp->minor = yygotominor;
    }else
#endif
    {
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
    }
  }else{
    assert( yyact == YYNSTATE + YYNRULE + 1 );
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  YYMINORTYPE yyminor            /* The minor type of the error token */
){
  ParseARG_FETCH;
#define TOKEN (yyminor.yy0)

/* 
** when the LEMON parser encounters an error
** then this global variable is set 
*/
	gml_parse_error = 1;
	*result = NULL;
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "ParseAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void Parse(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  ParseTOKENTYPE yyminor       /* The value for the token */
  ParseARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  int yyact;            /* The parser action. */
  int yyendofinput;     /* True if we are at the end of input */
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  /* (re)initialize the parser, if necessary */
  yypParser = (yyParser*)yyp;
  if( yypParser->yyidx<0 ){
#if YYSTACKDEPTH<=0
    if( yypParser->yystksz <=0 ){
      /*memset(&yyminorunion, 0, sizeof(yyminorunion));*/
      yyminorunion = yyzerominor;
      yyStackOverflow(yypParser, &yyminorunion);
      return;
    }
#endif
    yypParser->yyidx = 0;
    yypParser->yyerrcnt = -1;
    yypParser->yystack[0].stateno = 0;
    yypParser->yystack[0].major = 0;
  }
  yyminorunion.yy0 = yyminor;
  yyendofinput = (yymajor==0);
  ParseARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact<YYNSTATE ){
      assert( !yyendofinput );  /* Impossible to shift the $ token */
      yy_shift(yypParser,yyact,yymajor,&yyminorunion);
      yypParser->yyerrcnt--;
      yymajor = YYNOCODE;
    }else if( yyact < YYNSTATE + YYNRULE ){
      yy_reduce(yypParser,yyact-YYNSTATE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yymx = yypParser->yystack[yypParser->yyidx].major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor,&yyminorunion);
        yymajor = YYNOCODE;
      }else{
         while(
          yypParser->yyidx >= 0 &&
          yymx != YYERRORSYMBOL &&
          (yyact = yy_find_reduce_action(
                        yypParser->yystack[yypParser->yyidx].stateno,
                        YYERRORSYMBOL)) >= YYNSTATE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yyidx < 0 || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          YYMINORTYPE u2;
          u2.YYERRSYMDT = 0;
          yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor,yyminorunion);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;
      
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
  return;
}


/*
 GML_LEMON_END - LEMON generated code ends here 
*/

















/*
** CAVEAT: there is an incompatibility between LEMON and FLEX
** this macro resolves the issue
*/
#undef yy_accept
#define yy_accept	yy_gml_flex_accept



/*
 GML_FLEX_START - FLEX generated code starts here 
*/

#line 3 "lex.Gml.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer Gml_create_buffer
#define yy_delete_buffer Gml_delete_buffer
#define yy_flex_debug Gml_flex_debug
#define yy_init_buffer Gml_init_buffer
#define yy_flush_buffer Gml_flush_buffer
#define yy_load_buffer_state Gml_load_buffer_state
#define yy_switch_to_buffer Gml_switch_to_buffer
#define yyin Gmlin
#define yyleng Gmlleng
#define yylex Gmllex
#define yylineno Gmllineno
#define yyout Gmlout
#define yyrestart Gmlrestart
#define yytext Gmltext
#define yywrap Gmlwrap
#define yyalloc Gmlalloc
#define yyrealloc Gmlrealloc
#define yyfree Gmlfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* #include <stdio.h> */
/* #include <string.h> */
/* #include <errno.h> */
/* #include <stdlib.h> */

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE Gmlrestart(Gmlin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int Gmlleng;

extern FILE *Gmlin, *Gmlout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    /* Note: We specifically omit the test for yy_rule_can_match_eol because it requires
     *       access to the local variable yy_act. Since yyless() is a macro, it would break
     *       existing scanners that call yyless() from OUTSIDE Gmllex. 
     *       One obvious solution it to make yy_act a global. I tried that, and saw
     *       a 5% performance hit in a non-Gmllineno scanner, because yy_act is
     *       normally declared as a register variable-- so it is not worth it.
     */
    #define  YY_LESS_LINENO(n) \
            do { \
                int yyl;\
                for ( yyl = n; yyl < Gmlleng; ++yyl )\
                    if ( Gmltext[yyl] == '\n' )\
                        --Gmllineno;\
            }while(0)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up Gmltext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up Gmltext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via Gmlrestart()), so that the user can continue scanning by
	 * just pointing Gmlin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when Gmltext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int Gmlleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow Gmlwrap()'s to do buffer switches
 * instead of setting up a fresh Gmlin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void Gmlrestart (FILE *input_file  );
void Gml_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE Gml_create_buffer (FILE *file,int size  );
void Gml_delete_buffer (YY_BUFFER_STATE b  );
void Gml_flush_buffer (YY_BUFFER_STATE b  );
void Gmlpush_buffer_state (YY_BUFFER_STATE new_buffer  );
void Gmlpop_buffer_state (void );

static void Gmlensure_buffer_stack (void );
static void Gml_load_buffer_state (void );
static void Gml_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER Gml_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE Gml_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE Gml_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE Gml_scan_bytes (yyconst char *bytes,int len  );

void *Gmlalloc (yy_size_t  );
void *Gmlrealloc (void *,yy_size_t  );
void Gmlfree (void *  );

#define yy_new_buffer Gml_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        Gmlensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            Gml_create_buffer(Gmlin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        Gmlensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            Gml_create_buffer(Gmlin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *Gmlin = (FILE *) 0, *Gmlout = (FILE *) 0;

typedef int yy_state_type;

#define YY_FLEX_LEX_COMPAT
extern int Gmllineno;

int Gmllineno = 1;

extern char Gmltext[];

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up Gmltext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	Gmlleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	if ( Gmlleng + (yy_more_offset) >= YYLMAX ) \
		YY_FATAL_ERROR( "token too large, exceeds YYLMAX" ); \
	yy_flex_strncpy( &Gmltext[(yy_more_offset)], (yytext_ptr), Gmlleng + 1 ); \
	Gmlleng += (yy_more_offset); \
	(yy_prev_more_offset) = (yy_more_offset); \
	(yy_more_offset) = 0; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 11
#define YY_END_OF_BUFFER 12
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_acclist[34] =
    {   0,
        5,    5,   12,   10,   11,    8,   10,   11,    9,   11,
       10,   11,    5,   10,   11,    1,   10,   11,    3,   10,
       11,    2,   10,   11,    4,   10,   11,    7,   10,   11,
        6,    5,    7
    } ;

static yyconst flex_int16_t yy_accept[20] =
    {   0,
        1,    2,    3,    4,    6,    9,   11,   13,   16,   19,
       22,   25,   28,   31,   31,   32,   33,   34,   34
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    4,    1,    1,    1,    1,    1,    1,
        1,    1,    5,    5,    5,    5,    6,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    8,    1,    9,
       10,   11,    1,    1,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
        1,    1,    1,    1,   12,    1,   12,   12,   12,   12,

       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[13] =
    {   0,
        1,    1,    1,    1,    2,    1,    3,    4,    5,    1,
        5,    4
    } ;

static yyconst flex_int16_t yy_base[22] =
    {   0,
        0,    0,   23,   24,   24,   24,   18,    0,   24,   24,
       24,   24,    0,   17,   24,    0,    0,   24,   12,   15,
       16
    } ;

static yyconst flex_int16_t yy_def[22] =
    {   0,
       18,    1,   18,   18,   18,   18,   19,   20,   18,   18,
       18,   18,   21,   19,   18,   20,   21,    0,   18,   18,
       18
    } ;

static yyconst flex_int16_t yy_nxt[37] =
    {   0,
        4,    5,    6,    7,    8,    9,    8,    4,   10,   11,
       12,   13,   14,   14,   14,   14,   16,   16,   17,   17,
       15,   15,   18,    3,   18,   18,   18,   18,   18,   18,
       18,   18,   18,   18,   18,   18
    } ;

static yyconst flex_int16_t yy_chk[37] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,   19,   19,   19,   19,   20,   20,   21,   21,
       14,    7,    3,   18,   18,   18,   18,   18,   18,   18,
       18,   18,   18,   18,   18,   18
    } ;

/* Table of booleans, true if rule could match eol. */
static yyconst flex_int32_t yy_rule_can_match_eol[12] =
    {   0,
0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0,     };

extern int Gml_flex_debug;
int Gml_flex_debug = 0;

static yy_state_type *yy_state_buf=0, *yy_state_ptr=0;
static char *yy_full_match;
static int yy_lp;
#define REJECT \
{ \
*yy_cp = (yy_hold_char); /* undo effects of setting up Gmltext */ \
yy_cp = (yy_full_match); /* restore poss. backed-over text */ \
++(yy_lp); \
goto find_rule; \
}

static int yy_more_offset = 0;
static int yy_prev_more_offset = 0;
#define yymore() ((yy_more_offset) = yy_flex_strlen( Gmltext ))
#define YY_NEED_STRLEN
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET \
	{ \
	(yy_more_offset) = (yy_prev_more_offset); \
	Gmlleng -= (yy_more_offset); \
	}
#ifndef YYLMAX
#define YYLMAX 8192
#endif

char Gmltext[YYLMAX];
char *yytext_ptr;
#line 1 "gmlLexer.l"
/* 
 gmlLexer.l -- GML parser - FLEX config
  
 version 2.4, 2011 June 3

 Author: Sandro Furieri a.furieri@lqt.it

 ------------------------------------------------------------------------------
 
 Version: MPL 1.1/GPL 2.0/LGPL 2.1
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the SpatiaLite library

The Initial Developer of the Original Code is Alessandro Furieri
 
Portions created by the Initial Developer are Copyright (C) 2011
the Initial Developer. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of
either the GNU General Public License Version 2 or later (the "GPL"), or
the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
in which case the provisions of the GPL or the LGPL are applicable instead
of those above. If you wish to allow use of your version of this file only
under the terms of either the GPL or the LGPL, and not to allow others to
use your version of this file under the terms of the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the GPL or the LGPL. If you do not delete
the provisions above, a recipient may use your version of this file under
the terms of any one of the MPL, the GPL or the LGPL.
 
*/
#line 46 "gmlLexer.l"

/* For debugging purposes */
int gml_line = 1, gml_col = 1;

/**
*  The main string-token matcher.
*  The lower case part is probably not needed.  We should really be converting 
*  The string to all uppercase/lowercase to make it case iNsEnSiTiVe.
*  What Flex will do is, For the input string, beginning from the front, Flex
*  will try to match with any of the defined tokens from below.  Flex will 
*  then match the string of longest length.  Suppose the string is: POINTM,
*  Flex would match both POINT and POINTM, but since POINTM is the longer
*  of the two tokens, FLEX will match POINTM.
*/
#line 593 "lex.Gml.c"

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* #include <unistd.h> */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int Gmllex_destroy (void );

int Gmlget_debug (void );

void Gmlset_debug (int debug_flag  );

YY_EXTRA_TYPE Gmlget_extra (void );

void Gmlset_extra (YY_EXTRA_TYPE user_defined  );

FILE *Gmlget_in (void );

void Gmlset_in  (FILE * in_str  );

FILE *Gmlget_out (void );

void Gmlset_out  (FILE * out_str  );

int Gmlget_leng (void );

char *Gmlget_text (void );

int Gmlget_lineno (void );

void Gmlset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int Gmlwrap (void );
#else
extern int Gmlwrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( Gmltext, Gmlleng, 1, Gmlout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( Gmlin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( Gmlin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, Gmlin))==0 && ferror(Gmlin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(Gmlin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int Gmllex (void);

#define YY_DECL int Gmllex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after Gmltext and Gmlleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
#line 62 "gmlLexer.l"

#line 782 "lex.Gml.c"

	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

        /* Create the reject buffer large enough to save one state per allowed character. */
        if ( ! (yy_state_buf) )
            (yy_state_buf) = (yy_state_type *)Gmlalloc(YY_STATE_BUF_SIZE  );
            if ( ! (yy_state_buf) )
                YY_FATAL_ERROR( "out of dynamic memory in Gmllex()" );

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! Gmlin )
			Gmlin = stdin;

		if ( ! Gmlout )
			Gmlout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			Gmlensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				Gml_create_buffer(Gmlin,YY_BUF_SIZE );
		}

		Gml_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of Gmltext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);

		(yy_state_ptr) = (yy_state_buf);
		*(yy_state_ptr)++ = yy_current_state;

yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 19 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			*(yy_state_ptr)++ = yy_current_state;
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 24 );

yy_find_action:
		yy_current_state = *--(yy_state_ptr);
		(yy_lp) = yy_accept[yy_current_state];
find_rule: /* we branch to this label when backing up */
		for ( ; ; ) /* until we find what rule we matched */
			{
			if ( (yy_lp) && (yy_lp) < yy_accept[yy_current_state + 1] )
				{
				yy_act = yy_acclist[(yy_lp)];
					{
					(yy_full_match) = yy_cp;
					break;
					}
				}
			--yy_cp;
			yy_current_state = *--(yy_state_ptr);
			(yy_lp) = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

		if ( yy_act != YY_END_OF_BUFFER && yy_rule_can_match_eol[yy_act] )
			{
			int yyl;
			for ( yyl = (yy_prev_more_offset); yyl < Gmlleng; ++yyl )
				if ( Gmltext[yyl] == '\n' )
					   
    Gmllineno++;
;
			}

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
case 1:
YY_RULE_SETUP
#line 63 "gmlLexer.l"
{ gml_freeString(&(GmlLval.pval)); return GML_END; }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 64 "gmlLexer.l"
{ gml_freeString(&(GmlLval.pval)); return GML_EQ; }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 65 "gmlLexer.l"
{ gml_freeString(&(GmlLval.pval)); return GML_OPEN; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 66 "gmlLexer.l"
{ gml_freeString(&(GmlLval.pval)); return GML_CLOSE; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 67 "gmlLexer.l"
{ gml_saveString(&(GmlLval.pval), Gmltext); return GML_COORD; }
	YY_BREAK
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
#line 68 "gmlLexer.l"
{ gml_saveString(&(GmlLval.pval), Gmltext); return GML_VALUE; }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 69 "gmlLexer.l"
{ gml_saveString(&(GmlLval.pval), Gmltext); return GML_KEYWORD; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 71 "gmlLexer.l"
{ gml_freeString(&(GmlLval.pval)); gml_col += (int) strlen(Gmltext); }               /* ignore but count white space */
	YY_BREAK
case 9:
/* rule 9 can match eol */
YY_RULE_SETUP
#line 73 "gmlLexer.l"
{ gml_freeString(&(GmlLval.pval)); gml_col = 0; ++gml_line; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 75 "gmlLexer.l"
{ gml_freeString(&(GmlLval.pval)); gml_col += (int) strlen(Gmltext); return -1; }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 76 "gmlLexer.l"
ECHO;
	YY_BREAK
#line 941 "lex.Gml.c"
			case YY_STATE_EOF(INITIAL):
				yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed Gmlin at a new source and called
			 * Gmllex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = Gmlin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( Gmlwrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * Gmltext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of Gmllex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			Gmlrestart(Gmlin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) Gmlrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	(yy_state_ptr) = (yy_state_buf);
	*(yy_state_ptr)++ = yy_current_state;

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 19 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		*(yy_state_ptr)++ = yy_current_state;
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    
	register YY_CHAR yy_c = 1;
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 19 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 18);
	if ( ! yy_is_jam )
		*(yy_state_ptr)++ = yy_current_state;

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up Gmltext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

    if ( c == '\n' ){
        --Gmllineno;
    }

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					Gmlrestart(Gmlin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( Gmlwrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve Gmltext */
	(yy_hold_char) = *++(yy_c_buf_p);

	if ( c == '\n' )
		   
    Gmllineno++;
;

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void Gmlrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        Gmlensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            Gml_create_buffer(Gmlin,YY_BUF_SIZE );
	}

	Gml_init_buffer(YY_CURRENT_BUFFER,input_file );
	Gml_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void Gml_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		Gmlpop_buffer_state();
	 *		Gmlpush_buffer_state(new_buffer);
     */
	Gmlensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	Gml_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (Gmlwrap()) processing, but the only time this flag
	 * is looked at is after Gmlwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void Gml_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	Gmlin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE Gml_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) Gmlalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in Gml_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) Gmlalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in Gml_create_buffer()" );

	b->yy_is_our_buffer = 1;

	Gml_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with Gml_create_buffer()
 * 
 */
    void Gml_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		Gmlfree((void *) b->yy_ch_buf  );

	Gmlfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a Gmlrestart() or at EOF.
 */
    static void Gml_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	Gml_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then Gml_init_buffer was _probably_
     * called from Gmlrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void Gml_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		Gml_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void Gmlpush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	Gmlensure_buffer_stack();

	/* This block is copied from Gml_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from Gml_switch_to_buffer. */
	Gml_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void Gmlpop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	Gml_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		Gml_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void Gmlensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)Gmlalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in Gmlensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)Gmlrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in Gmlensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE Gml_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) Gmlalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in Gml_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	Gml_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to Gmllex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       Gml_scan_bytes() instead.
 */
YY_BUFFER_STATE Gml_scan_string (yyconst char * yystr )
{
    
	return Gml_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to Gmllex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE Gml_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) Gmlalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in Gml_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = Gml_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in Gml_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up Gmltext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		Gmltext[Gmlleng] = (yy_hold_char); \
		(yy_c_buf_p) = Gmltext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		Gmlleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int Gmlget_lineno  (void)
{
        
    return Gmllineno;
}

/** Get the input stream.
 * 
 */
FILE *Gmlget_in  (void)
{
        return Gmlin;
}

/** Get the output stream.
 * 
 */
FILE *Gmlget_out  (void)
{
        return Gmlout;
}

/** Get the length of the current token.
 * 
 */
int Gmlget_leng  (void)
{
        return Gmlleng;
}

/** Get the current token.
 * 
 */

char *Gmlget_text  (void)
{
        return Gmltext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void Gmlset_lineno (int  line_number )
{
    
    Gmllineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see Gml_switch_to_buffer
 */
void Gmlset_in (FILE *  in_str )
{
        Gmlin = in_str ;
}

void Gmlset_out (FILE *  out_str )
{
        Gmlout = out_str ;
}

int Gmlget_debug  (void)
{
        return Gml_flex_debug;
}

void Gmlset_debug (int  bdebug )
{
        Gml_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from Gmllex_destroy(), so don't allocate here.
     */

    /* We do not touch Gmllineno unless the option is enabled. */
    Gmllineno =  1;
    
    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

    (yy_state_buf) = 0;
    (yy_state_ptr) = 0;
    (yy_full_match) = 0;
    (yy_lp) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    Gmlin = stdin;
    Gmlout = stdout;
#else
    Gmlin = (FILE *) 0;
    Gmlout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * Gmllex_init()
     */
    return 0;
}

/* Gmllex_destroy is for both reentrant and non-reentrant scanners. */
int Gmllex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		Gml_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		Gmlpop_buffer_state();
	}

	/* Destroy the stack itself. */
	Gmlfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    Gmlfree ( (yy_state_buf) );
    (yy_state_buf)  = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * Gmllex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *Gmlalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *Gmlrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void Gmlfree (void * ptr )
{
	free( (char *) ptr );	/* see Gmlrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 76 "gmlLexer.l"


/**
 * reset the line and column count
 *
 *
 */
void gml_reset_lexer(void)
{

  gml_line = 1;
  gml_col  = 1;

}

/**
 * gmlError() is invoked when the lexer or the parser encounter
 * an error. The error message is passed via *s
 *
 *
 */
void GmlError(char *s)
{
  printf("error: %s at line: %d col: %d\n",s,gml_line,gml_col);

}

int Gmlwrap(void)
{
  return 1;
}


/*
 GML_FLEX_END - FLEX generated code ends here 
*/



gaiaGeomCollPtr
gaiaParseGml (const unsigned char *dirty_buffer, sqlite3 * sqlite_handle)
{
    void *pParser = ParseAlloc (malloc);
    /* Linked-list of token values */
    gmlFlexToken *tokens = malloc (sizeof (gmlFlexToken));
    /* Pointer to the head of the list */
    gmlFlexToken *head = tokens;
    int yv;
    gmlNodePtr result = NULL;
    gaiaGeomCollPtr geom = NULL;

    GmlLval.pval = NULL;
    tokens->value = NULL;
    tokens->Next = NULL;
    gml_parse_error = 0;
    Gml_scan_string ((char *) dirty_buffer);

    /*
       / Keep tokenizing until we reach the end
       / yylex() will return the next matching Token for us.
     */
    while ((yv = yylex ()) != 0)
      {
	  if (yv == -1)
	    {
		gml_parse_error = 1;
		break;
	    }
	  tokens->Next = malloc (sizeof (gmlFlexToken));
	  tokens->Next->Next = NULL;
	  /*
	     /GmlLval is a global variable from FLEX.
	     /GmlLval is defined in gmlLexglobal.h
	   */
	  gml_xferString (&(tokens->Next->value), GmlLval.pval);
	  /* Pass the token to the wkt parser created from lemon */
	  Parse (pParser, yv, &(tokens->Next->value), &result);
	  tokens = tokens->Next;
      }
    /* This denotes the end of a line as well as the end of the parser */
    Parse (pParser, GML_NEWLINE, 0, &result);
    ParseFree (pParser, free);
    Gmllex_destroy ();

    /* Assigning the token as the end to avoid seg faults while cleaning */
    tokens->Next = NULL;
    gml_cleanup (head);
    gml_freeString (&(GmlLval.pval));

    if (gml_parse_error)
      {
	  if (result)
	      gml_freeTree (result);
	  return NULL;
      }

    /* attempting to build a geometry from GML */
    geom = gml_build_geometry (result, sqlite_handle);
    gml_freeTree (result);
    return geom;
}


/*
** CAVEAT: we must now undefine any Lemon/Flex own macro
*/
#undef YYNOCODE
#undef YYNSTATE
#undef YYNRULE
#undef YY_SHIFT_MAX
#undef YY_SHIFT_USE_DFLT
#undef YY_REDUCE_USE_DFLT
#undef YY_REDUCE_MAX
#undef YY_FLUSH_BUFFER
#undef YY_DO_BEFORE_ACTION
#undef YY_NUM_RULES
#undef YY_END_OF_BUFFER
#undef YY_END_FILE
#undef YYACTIONTYPE
#undef YY_SZ_ACTTAB
#undef YY_NEW_FILE
#undef BEGIN
#undef YY_START
#undef YY_CURRENT_BUFFER
#undef YY_CURRENT_BUFFER_LVALUE
#undef YY_STATE_BUF_SIZE
#undef YY_DECL
#undef YY_FATAL_ERROR
#undef YYMINORTYPE
#undef YY_CHAR
#undef YYSTYPE
#undef input
#undef ParseAlloc
#undef ParseFree
#undef ParseStackPeak
#undef Parse
#undef yyalloc
#undef yyfree
#undef yyin
#undef yyleng
#undef yyless
#undef yylex
#undef yylineno
#undef yyout
#undef yyrealloc
#undef yyrestart
#undef yyStackEntry
#undef yytext
#undef yywrap
#undef yyzerominor
#undef yy_accept
#undef yy_action
#undef yy_base
#undef yy_buffer_stack
#undef yy_buffer_stack_max
#undef yy_buffer_stack_top
#undef yy_c_buf_p
#undef yy_chk
#undef yy_create_buffer
#undef yy_def
#undef yy_default
#undef yy_delete_buffer
#undef yy_destructor
#undef yy_ec
#undef yy_fatal_error
#undef yy_find_reduce_action
#undef yy_find_shift_action
#undef yy_flex_debug
#undef yy_flush_buffer
#undef yy_get_next_buffer
#undef yy_get_previous_state
#undef yy_init
#undef yy_init_buffer
#undef yy_init_globals
#undef yy_load_buffer
#undef yy_load_buffer_state
#undef yy_lookahead
#undef yy_meta
#undef yy_new_buffer
#undef yy_nxt
#undef yy_parse_failed
#undef yy_pop_parser_stack
#undef yy_reduce
#undef yy_reduce_ofst
#undef yy_set_bol
#undef yy_set_interactive
#undef yy_shift
#undef yy_shift_ofst
#undef yy_start
#undef yy_state_type
#undef yy_switch_to_buffer
#undef yy_syntax_error
#undef yy_trans_info
#undef yy_try_NUL_trans
#undef yyParser
#undef yyStackEntry
#undef yyStackOverflow
#undef yyRuleInfo
#undef yytext_ptr
#undef yyunput
#undef yyzerominor
#undef ParseARG_SDECL
#undef ParseARG_PDECL
#undef ParseARG_FETCH
#undef ParseARG_STORE
#undef REJECT
#undef yymore
#undef YY_MORE_ADJ
#undef YY_RESTORE_YY_MORE_OFFSET
#undef YY_LESS_LINENO
#undef yyTracePrompt
#undef yyTraceFILE
#undef yyTokenName
#undef yyRuleName
#undef ParseTrace
/**************** End file: gg_gml.c **********/

